/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file   psp_af.h
 *
 *  \brief  AF interface definition
 *
 *  This file contains the interfaces, data types and symbolic definitions
 *  that are needed by the application to utilize the serivces of the AF module
 *  of H3A device driver.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author     EI4
 *
 *  \version    1.0     Created
 */

#ifndef _PSP_AF_H_
#define _PSP_AF_H_

/******************************************************************************
                            Header File Inclusion
******************************************************************************/
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/tistdtypes.h>
#include <ti/sdo/pspdrivers/common/psp_common.h>

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
                                  Macros
******************************************************************************/
#define PSP_AF_NUMBER_OF_COEF       (11u)
/**< Number of Coefficients */
#define PSP_AF_PAXEL_SIZE           (48u)

/******************************************************************************
                                Enumerations
******************************************************************************/
/**
 *  \brief  _PSP_AF_Alaw
 *
 *  Enumeration defintion for status of A law
 */
typedef enum _PSP_AF_Alaw
{
    PSP_AF_ALAW_DISABLE     = 0,
    /**< Disable A Law      */
    PSP_AF_ALAW_ENABLE      = 1
    /**< Enable A Law       */
} PSP_AF_Alaw;

/**
 *  \brief  _PSP_AF_HMF_law
 *
 *  Enumeration defintion for status of Median Filter Law
 */
typedef enum _PSP_AF_HMF_law
{
    PSP_AF_HMF_DISABLE  = 0,
    /**< Disable HMF Law    */
    PSP_AF_HMF_ENABLE   = 1
    /**< Disable HMF Law    */
} PSP_AF_HMF_law;

/**
 *  \brief  _PSP_AF_mode
 *
 *  Enumeration defintion for status of Accumulator
 */
typedef enum _PSP_AF_mode
{
    PSP_AF_ACCUMULATOR_SUMMED  = 0,
    /**< Summed mode of Accumulator     */
    PSP_AF_ACCUMULATOR_PEAK    = 1
    /**< Peak Mode of the accumulator   */
} PSP_AF_mode;

/**
 *  \brief  _PSP_AF_rgbpos
 *
 *  Enumeration defintion for RGB Position
 */
typedef enum _PSP_AF_rgbpos
{
    PSP_AF_GR_GB_BAYER      =   0,
    /**< GR and GB as Bayer pattern     */
    PSP_AF_RG_GB_BAYER      =   1,
    /**< RG and GB as Bayer pattern     */
    PSP_AF_GR_BG_BAYER      =   2,
    /**< GR and BG as Bayer pattern     */
    PSP_AF_RG_BG_BAYER      =   3,
    /**< RG and BG as Bayer pattern     */
    PSP_AF_GG_RB_CUSTOM    =   4,
    /**< GG and GB as Custom pattern    */
    PSP_AF_RB_GG_CUSTOM    =   5
    /**< RB and GG as Custom pattern    */
} PSP_AF_rgbpos;

/******************************************************************************
                             Data-Structures
******************************************************************************/
/**
 * \brief _PSP_AFHmf
 * Structure defintion for Horizontal Median Filter
 */
typedef struct _PSP_AFHmf
{
    PSP_AF_HMF_law  enable;
    /**< Threshhold Value for Horizontal Median Filter      */
    Uint32            threshold;
    /**< Threshhold Value for Horizontal Median Filter      */
} PSP_AFHmf;

/**
 *  \brief _PSP_AFIir
 *
 *  Structure defintion for IIR Filter
 */
typedef struct _PSP_AFIir
{
    Uint32    hzStarPos;
    /**<IIR Start Register Value                */
    Int     coeffSet0[PSP_AF_NUMBER_OF_COEF];
    /**<IIR Filter Coefficient for Set 0        */
    Int     coeffSet1[PSP_AF_NUMBER_OF_COEF];
    /**< IIR Filter Coefficient for Set 1       */
} PSP_AFIir;

/**
 *  \brief _PSP_AFPaxel
 *
 *  Structure defintion contains inforamtion regrading Paxels
 */
typedef struct _PSP_AFPaxel
{
    Uint32    width;
    /**< Width of the Paxel         */
    Uint32    height;
    /**< Height of the Paxel        */
    Uint32    hzStart;
    /**< Horizontal Start Position  */
    Uint32    vtStart;
    /**< Vertical Start Position    */
    Uint32    hzCnt;
    /**< Horizontal Count           */
    Uint32    vtCnt;
     /**< vertical Count            */
    Uint32    lineIncr;
    /**< Line Increment             */
} PSP_AFPaxel;

/**
 *  \brief _PSP_AFParams
 *
 *  AF Parameter Structure
 */
typedef struct _PSP_AFParams
{
    PSP_AF_Alaw     enable;
    /**<ALWAW status                */
    PSP_AFHmf       hmfConfig;
    /**<HMF configurations          */
    PSP_AF_rgbpos   rgbPos;
    /**<RGB Positions               */
    PSP_AFIir       iirConfig;
    /**<IIR filter configurations   */
    PSP_AFPaxel     paxelConfig;
    /**<Paxel parameters            */
    PSP_AF_mode     mode;
    /**<Accumulator mode            */
} PSP_AFParams;

#ifdef __cplusplus
}
#endif

#endif  /* _PSP_AF_H_ */
