/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file   psp_h3a.h
 *
 *  \brief  H3A interface definition
 *
 *  This file contains the interfaces, data types and symbolic definitions
 *  that are needed by the application to utilize the serivces of the H3A
 *  device driver.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author     PSP Team
 *
 *  \version    1.0     Created
 *
 *  <b>Compiler Switch Used</b><br>
 *
 *  <b>_DEBUG</b><br>
 *   This is used as a flag to compiler whether to include the debug statements 
 *   inserted in the code into the final image. This flag helps to build DEBUG 
 *   image of the program. For RELEASE images this is not passed to the compiler.
 *
 *  <b>CHIP_XXXX</b><br>
 *   The CSL layer is written in a common file for all the variants of a SOC.
 *   This flag differentiates the variant we are compiling for, for e.g.
 *   CHIP_DM648, and the CSL definitions for that variant appropriately
 *   gets defined for register base addresses, num of ports of a peripheral etc.
 *
 *  <b>H3A_INSTRUMENTATION_ENABLED</b><br>
 *   This flag is passed to the compiler to include the instrumentation code
 *   parts into the final image/lib of the program. This helps build the
 *   iRelease/iDebug versions of the image/lib with a common code base
 *
 */

#ifndef _PSP_H3A_H_
#define _PSP_H3A_H_

/******************************************************************************
                            Header File Inclusion
******************************************************************************/
#include <ti/sdo/pspdrivers/pal_os/pal_osSem.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osList.h>
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/tistdtypes.h>
#include <ti/sdo/pspdrivers/common/psp_common.h>
#include <ti/sdo/pspdrivers/system/dm6437/bios/evmDM6437/video/psp_vpss.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * PSP Common header provides standard driver return codes, operation modes
 * and other generic defines required by all PSP drivers
 */

/******************************************************************************
                                  Macros
******************************************************************************/
#define PSP_H3A_NUM_INSTANCES (1u)
/**< Maximum number of device instances */

/******************************************************************************
                                Enumerations
******************************************************************************/

/**
 *  \brief  PSP H3A Ioctl commands
 *
 *  H3A Ioctl commands
 */
typedef enum _PSP_H3AIoctlCmd
{
    PSP_H3A_IOCTL_SET_PARAM = 128,
    /**< Set H3A config params, cmdArg = parameter structure    */
    PSP_H3A_IOCTL_GET_PARAM,
    /**< Get H3A config params, cmdArg = parameter structure    */
    PSP_H3A_IOCTL_SET_SEM_TIMEOUT
    /**< Change the timeout value used in semaphore pending operation.
         Command Argument:  Timeout values<Int32>                           */
} PSP_H3AIoctlCmd;

/**
  * \brief Enum for the Channel type Commands
  *
  */
typedef enum _PSP_ChannelType
{
    PSP_H3A_AF = 0,
    /**< AF channel     */
    PSP_H3A_AEW
    /**< AEW channel    */
} PSP_H3AChannelType;

/**
 *  \brief  Enum for checking data validity of buffer
 *
 */
typedef enum _PSP_H3ABufferStatus
{
    PSP_H3A_BUFFER_DATA_VALID = 0,
    /**< Get H3A config params, cmdArg = parameter structure    */
    PSP_H3A_BUFFER_DATA_CORRUPTED
    /**< Set H3A config params, cmdArg = parameter structure    */

} PSP_H3ABufferStatus;

/**
 * This enum describes the various states of H3A driver goes through
 * the entire cycle.
 *
 */
typedef enum _PSP_H3A_DEVICE_STATE
{
    PSP_H3A_DELETED = 0,
    /**< H3A in Deleted state   */
    PSP_H3A_CREATED,
    /**< H3A in Created state   */
    PSP_H3A_OPENED,
    /**< H3A in Opened state    */
    PSP_H3A_CLOSED
    /**< H3A in Closed state    */
} PSP_H3AState;


/******************************************************************************
                             Data-Structures
******************************************************************************/
/**
 *  \brief _PSP_H3ABuffer
 *
 *  H3A Buffer Structure
 */
typedef struct _PSP_H3ABuffer
{
    PAL_OsListNodeHeader    nodeEntry;
    /**< Node for queue entry               */
    Ptr                     ramIpAddr;
    /**< Buffer address to store data       */
    Uint32                  timeStamp;
    /** <Time Stamp                         */
    Uint32                  size;
    /** Size of the enqueued buffer         */
    PSP_H3ABufferStatus     buffStatus;
    /** Status of the enqueued buffer       */
} PSP_H3ABuffer;

#ifdef __cplusplus
}
#endif

#endif  /* _PSP_H3A_H_ */
