/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \file   dda_h3aIOM.c
 *
 *  \brief  H3A IOM layer Functions
 *
 *  This file contains the Implementation details of IOM layer functions
 *
 *  (C) Copyright 2010, Texas Instruments, Inc
 *
 *  \author     EI4
 *
 *  \version    0.1             Created
 *              1.0             First Release
 */
/******************************************************************************
  Header File Inclusion
 ******************************************************************************/
#include <std.h>
#include <ti/sdo/pspdrivers/common/psp_common.h>
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/tistdtypes.h>
#include <ti/sdo/pspdrivers/drivers/h3a/psp_h3a.h>
#include <ti/sdo/pspdrivers/drivers/h3a/src/psp_h3aApi.h>
#include <ti/sdo/pspdrivers/drivers/h3a/src/dda_h3aIOM_BIOS.h>
#include <ti/sdo/pspdrivers/drivers/h3a/src/dda_h3a.h>
#include <ti/sdo/pspdrivers/drivers/h3a/src/ddc_h3a.h>
#ifdef H3A_INSTRUMENTATION_ENABLED
#include <ti/sdo/pspdrivers/pal_os/bios/psp_log.h>
#endif /* H3A_INSTRUMENTATION_ENABLED */

/******************************************************************************
  Internal Functions
 *****************************************************************************/
/**
 *  \brief getIomErrorCode
 *
 * This function converts PSP error codes to appropriate IOM error code.
 *
 * \param  pspCode          [IN]    PSP Error code.
 * \return IOM ERROR CODE   [OUT]   Equivalent IOM error code. For unrecoganized
 *                                  error code IOM_EBADIO is returned.
 */
static Int getIomErrorCode(PSP_Result pspCode);

/*******************************************************************************
  Mini Driver Function interface table
  *****************************************************************************/
/**
 * \brief Mini Driver Function interface table called by BIOS Itself
 *
 */
IOM_Fxns H3AMD_FXNS =
{
    &H3A_mdBindDev,
    &H3A_mdUnBindDev,
    &H3A_mdControlChan,
    &H3A_mdCreateChan,
    &H3A_mdDeleteChan,
    &H3A_mdSubmitChan,
};

/*******************************************************************************
  IOM Error code look up table
  *****************************************************************************/
/**
 * The following mapping shows the mapping done from PSP_Result to IOM error
 * code.
 *
 * PSP_SOK                  (0)   IOM_COMPLETED            0
 * PSP_SINPROGRESS          (1)   IOM_PENDING              1
 * PSP_E_DRIVER_INIT        (-1)  IOM_ETIMEOUTUNREC      -11
 * PSP_E_NO_MEMORY          (-2)  IOM_EALLOC              -5
 * PSP_E_RESOURCES          (-3)  IOM_EALLOC              -5
 * PSP_E_INVAL_STATE        (-4)  IOM_EBADMODE            -7
 * PSP_E_INVAL_PARAM        (-5)  IOM_EBADARGS           -10
 * PSP_E_NOT_SUPPORTED      (-6)  IOM_ENOTIMPL            -9
 * PSP_E_IO_CANCEL_FAIL     (-7)  IOM_ETIMEOUT            -2
 * PSP_E_FIFO_NOT_ENABLED   (-8)  IOM_EBADMODE            -7
 * PSP_E_INVALID_MODE       (-9)  IOM_EBADMODE            -7
 * PSP_E_INVALID_CMD        (-10) IOM_EBADARGS           -10
 *
 */

/**
 * \brief IOM Port Object
 *
 */
/* Setting initial global values */
H3APortObject h3a_ports =
    { DDC_H3A_INIT_PORT_VAL, NULL, PSP_H3A_DELETED, {NULL, NULL}, {NULL, NULL}};

/******************************************************************************
  Functions
 ******************************************************************************/
/**
 * \brief getIomErrorCode
 * This function converts PSP error codes to appropriate IOM error code.
 */
static Int getIomErrorCode(PSP_Result pspCode)
{
    Int iomErrorCodeList[12] =
    {
        IOM_EBADARGS,
        IOM_EBADMODE,
        IOM_EBADMODE,
        IOM_ETIMEOUT,
        IOM_ENOTIMPL,
        IOM_EBADARGS,
        IOM_EBADMODE,
        IOM_EALLOC,
        IOM_EALLOC,
        IOM_ETIMEOUTUNREC,
        IOM_COMPLETED,
        IOM_PENDING
    };
    /* Mapping of psp level error code with the iom layer code. The maximum
       error code return from the below layer is 10 so taking the error code*/

    Int iomCode = pspCode+10;
    /* Mapping the error code with its equivalent IOM layer error code */
    if ((12 > iomCode)  && (iomCode >= 0))
    {
        iomCode = iomErrorCodeList[iomCode];
    }
    else
    {
        iomCode = IOM_EBADIO;
    }
    return(iomCode);
}

/**
 * \brief H3A_mdBindDev
 * H3A_mdBindDev() is called by DEV_init() to bind and initialize the instance
 */
/* ARGSUSED */
Int H3A_mdBindDev(PSP_Handle *devp, Int devid, Ptr devParams)
{
    Int     statusBnd   = IOM_COMPLETED;
    DDC_H3ADeviceObject *ddcH3aDevObj;
    Uint32  ldevid      = (Uint32)devid;

    /* Check the Parameters */
    if ((NULL == devp) || (PSP_H3A_NUM_INSTANCES <= (Uint32)ldevid))
    {
        DDAIOM_H3A_DEBUG_PRINT(
            "H3A_mdBindDev: Error: Device creation failed\n");
        statusBnd =  (IOM_EBADARGS);
    }
    else
    {
        /* This function will initialize and create an instance of H3A */
        ddcH3aDevObj =(DDC_H3ADeviceObject *) PSP_H3ACreate((Uint32)devid);

        if (NULL == ddcH3aDevObj)
        {
            DDAIOM_H3A_DEBUG_PRINT(
                "H3A_mdBindDev: Error: Device creation failed\n");
            statusBnd = IOM_EBADARGS ;
        }
        else
        {
            /* Update H3A port object */
            h3a_ports.deviceObj = ddcH3aDevObj;
            h3a_ports.portNumber  = ldevid;
            h3a_ports.state      = PSP_H3A_CREATED;
            *devp = &h3a_ports;
        }
    }
    return(statusBnd);
}

/**
 * \brief H3A_mdUnBindDev
 * This function is called by DEV_init() to unbind and de-init the instance
 */
Int H3A_mdUnBindDev(PSP_Handle devp)
{
    Int statusUnBnd = IOM_COMPLETED;
    H3APortObject *portUnBnd = NULL;
    if (NULL == devp)
    {
        DDAIOM_H3A_DEBUG_PRINT("H3A_mdUnBindDev: Error: UnBind failed\n");
        statusUnBnd = IOM_EBADARGS;
    }
    else
    {
        portUnBnd = (H3APortObject *)devp;
        /* Check the State of Port */
        if ((PSP_H3A_CREATED != portUnBnd->state)
            && (PSP_H3A_CLOSED  != portUnBnd->state))
        {
            DDAIOM_H3A_DEBUG_PRINT("H3A_mdUnBindDev: Error: UnBind failed\n");
            statusUnBnd = IOM_EBADMODE;
        }
        /* Compare the instance number with maximum limits */
        else if (portUnBnd->portNumber >= PSP_H3A_NUM_INSTANCES )
        {
            statusUnBnd = IOM_EBADARGS;
        }
        else
        {
            /* This function de-initialize and closes handle of a channel */
            statusUnBnd = PSP_H3ADelete(portUnBnd->portNumber);
            if (PAL_SOK != statusUnBnd)
            {
                /* Generate the IOM Error code based on PSP error */
                statusUnBnd = getIomErrorCode(statusUnBnd);
            }
            else
            {
                portUnBnd->state = PSP_H3A_DELETED;
            }
        }
    }
    return(statusUnBnd);
}

/**
 * \brief H3A_mdCreateChan
 * Creates the channel for the data transfer by setting up some params
 */
/* ARGSUSED */
Int H3A_mdCreateChan(PSP_Handle *chanp, PSP_Handle devp, String name, Int mode,
    Ptr chanParams, IOM_TiomCallback cbFxn, Ptr cbArg)
{
    Int statusCR = IOM_COMPLETED;
    H3APortObject* portCR = NULL;
    H3AChannelObject* channel = NULL;
    PSP_H3AChannelType type;
    if ((((NULL == chanp) || (NULL == devp)) || (NULL == chanParams))
        || ((NULL == cbFxn) || (NULL == cbArg)))
    {
        DDAIOM_H3A_DEBUG_PRINT(
            "H3A_mdCreateChan: Error: Channel creation failed\n");
        statusCR = IOM_EBADARGS;
    }
    else
    {
        portCR = (H3APortObject *)devp;
        if ( ((IOM_INPUT != mode) && (IOM_OUTPUT != mode) )
            && (IOM_INOUT != mode) )
        {
            DDAIOM_H3A_DEBUG_PRINT(
                "H3A_mdCreateChan: Error: Channel creation failed\n");
            statusCR = IOM_EBADMODE;
        }
        /* Check the State of the Port */
        if (((PSP_H3A_CREATED  != portCR->state)
                && (PSP_H3A_CLOSED != portCR->state))
            && (PSP_H3A_OPENED   != portCR->state))
        {
            DDAIOM_H3A_DEBUG_PRINT(
                "H3A_mdCreateChan: Error: Channel creation failed\n");
            statusCR = (IOM_EBADMODE);
        }
        else
        {
            /* Get the Channel Type */
            type = (PSP_H3AChannelType)(*(Int *)chanParams);
            /* Validate Channel Type */
            if ((PSP_H3A_AF != type) && (PSP_H3A_AEW != type))
            {
                statusCR = IOM_EBADARGS;
            }
            /* Verify that channel is already assigned */
            if ((PSP_H3A_AF == type) && (NULL == portCR->afChan.ddc_Handle))
            {
                /* If Channel is not in use then assign the channel */
                channel = &(portCR->afChan);
            }
            /* Verify that channel is already assigned */
            else if ((PSP_H3A_AEW == type)
                && (NULL == portCR->aewChan.ddc_Handle))
            {
                /* If Channel is not in use then assign the channel */
                channel = &(portCR->aewChan);
            }
            else
            {
                DDAIOM_H3A_DEBUG_PRINT(
                    "H3A_mdCreateChan: Error: Channel creation failed\n");
                statusCR = IOM_EINUSE;
            }
        }
    }
    if (IOM_COMPLETED == statusCR)
    {
        if ( NULL != channel )
        {
            /* Initialize and Creates the Channel Handle for AF or AEW */
            channel->ddc_Handle = (PSP_Handle)PSP_H3AOpen(type);
            if (NULL == channel->ddc_Handle)
            {
                DDAIOM_H3A_DEBUG_PRINT(
                    "H3A_mdCreateChan: Error: Channel creation failed\n");
                statusCR = IOM_EBADARGS;
            }
            else
            {
                /* Update the State as opened if Channel Handle not NULL */
                portCR->state = PSP_H3A_OPENED;
                /* sets for back referencing */
                channel->port = (H3APortObject *)portCR;
                *chanp = channel;
            }
        }
    }
    return(statusCR);
}

/**
 * \brief H3A_mdDeleteChan
 * Deletes an instance of the H3A channel.
 */
Int H3A_mdDeleteChan(PSP_Handle chanp)
{
    Int                 statusDL    = IOM_COMPLETED;
    H3AChannelObject*   chanDL      = NULL;
    H3APortObject*      portDL      = NULL;

    if (NULL == chanp)
    {
        DDAIOM_H3A_DEBUG_PRINT(
            "H3A_mdDeleteChan: Error: Channel deletion failed\n");
        statusDL = IOM_EBADARGS;
    }
    else
    {
        chanDL = (H3AChannelObject *)chanp;
        portDL = (H3APortObject *)chanDL->port;
        /* Check if Channel in use */
        if (NULL == portDL)
        {
            DDAIOM_H3A_DEBUG_PRINT(
                "H3A_mdDeleteChan: Error: Channel deletion failed\n");
            statusDL = IOM_EBADARGS;
        }
        else if (PSP_H3A_OPENED != portDL->state)
        {
            DDAIOM_H3A_DEBUG_PRINT(
                "H3A_mdDeleteChan: Error: Channel deletion failed\n");
            statusDL = IOM_EBADARGS;
        }
        else
        {
            statusDL = PSP_H3AClose(chanDL->ddc_Handle);
            if (PAL_SOK != statusDL)
            {
                DDAIOM_H3A_DEBUG_PRINT(
                    "H3A_mdDeleteChan: Error: Channel deletion failed\n");
                /* Generate the IOM error code equivalent to PSP error */
                statusDL = getIomErrorCode(statusDL);
            }
            else
            {
                /* Updates the state and makes the channel port void */
                chanDL->port = NULL;
                /* If all channels (both AF and AEW ) are closed
                   then update the state to closed */
                if (PSP_H3A_CLOSED == portDL->deviceObj->state)
                {
                    portDL->state = PSP_H3A_CLOSED;
                }
                chanDL->ddc_Handle = NULL;
                chanp = &chanDL;
            }
        }
    }
    return(statusDL);
}

/**
 *\ brief H3A_mdControlChan
 *  This function is used to config the ioctl operation
 */
Int H3A_mdControlChan(PSP_Handle chanp, Uint32 cmd, Ptr arg)
{
    Int statusCTRL = IOM_COMPLETED;
    H3AChannelObject *chanCTRL = NULL;
    H3APortObject *portCTRL = NULL;
    if (NULL == chanp)
    {
        DDAIOM_H3A_DEBUG_PRINT("H3A_mdControlChan: Error: IOCTL failed\n");
        statusCTRL = IOM_EBADARGS;
    }
    else
    {
        chanCTRL = (H3AChannelObject *)chanp;
        portCTRL = (H3APortObject* )chanCTRL->port;
        /* Check the port handle and port State */
        if ((NULL == portCTRL))
        {
            DDAIOM_H3A_DEBUG_PRINT("H3A_mdControlChan: Error: IOCTL failed\n");
            statusCTRL = IOM_EBADARGS;
        }
        else if (PSP_H3A_OPENED != portCTRL->state)
        {
            DDAIOM_H3A_DEBUG_PRINT("H3A_mdControlChan: Error: IOCTL failed\n");
            statusCTRL = IOM_EBADMODE;
        }
        else
        {
            /* DDA Layer function is called to change Dynamic Configuration
               of the Channels */
            statusCTRL = PSP_H3AIoctl(chanCTRL->ddc_Handle,
                (PSP_H3AIoctlCmd)cmd, arg, NULL);
            if (PSP_SOK != statusCTRL)
            {
                DDAIOM_H3A_DEBUG_PRINT(
                    "H3A_mdControlChan: Error: IOCTL failed\n");
                /*Generate the IOM Errorcode equivalent to PSP Error Code */
                statusCTRL = getIomErrorCode(statusCTRL);
            }
        }
    }
    return(statusCTRL);
}

/**
 * \brief H3A_mdSubmitChan
 * H3A_mdSubmitChan is used to submit the IO request.
 */
Int H3A_mdSubmitChan(PSP_Handle chanp, IOM_Packet *packet)
{
    Int32                 statusSC = IOM_COMPLETED;
    H3AChannelObject    * chanSC     = NULL;
    H3APortObject       * portSC     = NULL;
#ifdef H3A_INSTRUMENTATION_ENABLED
    Uint32 idSBT;
#endif /*H3A_INSTRUMENTATION_ENABLED*/
    /* Check the formal arguments whether NULL */
    if ( (NULL == chanp) || (NULL == packet))
    {
        statusSC = IOM_EBADARGS;
    }
    else if (sizeof(PSP_H3ABuffer) != packet->size)
    {
        statusSC = IOM_EBADARGS;
    }
    else
    {
        chanSC = (H3AChannelObject *)chanp;

#ifdef H3A_INSTRUMENTATION_ENABLED
        idSBT = *(Uint32 *)chanSC->ddc_Handle;
        /* PSP_DVT_dCHANNEL_ID  =  chanSC->ddc_handle
           PSP_DVT_dPACKET_ID = packet      */
        if(1 == idSBT){
            PSP_LOG_EVENT(  &DVTEvent_Log,
                "H3A",
                PSP_DVT_DESC(
                PSP_DVT_eFUNC_START,
                PSP_DVT_dCHANNEL_ID_INST,
                PSP_DVT_dPACKET_ID,
                PSP_DVT_dSIZE_BYTES),
                CHANNELID_INSTANCEID(1,0),                    /* channel 1 for AEW */
                packet->addr,
                packet->size
                );
        }
        else{
            PSP_LOG_EVENT(  &DVTEvent_Log,
                "H3A",
                PSP_DVT_DESC(
                PSP_DVT_eFUNC_START,
                PSP_DVT_dCHANNEL_ID_INST,
                PSP_DVT_dPACKET_ID,
                PSP_DVT_dSIZE_BYTES),
                CHANNELID_INSTANCEID(0,0),                    /* channel 0 for AF */
                packet->addr,
                packet->size
                );
        }
#endif /* H3A_INSTRUMENTATION_ENABLED */

        if (NULL == chanSC->port)
        {
            DDAIOM_H3A_DEBUG_PRINT("H3A_mdSubmitChan: Error: Submit failed\n");
            statusSC = IOM_EBADARGS;
        }
        else
        {
            portSC = (H3APortObject *)chanSC->port;
            if (NULL == portSC)
            {
                DDAIOM_H3A_DEBUG_PRINT(
                    "H3A_mdSubmitChan: Error: Submit failed\n");
                statusSC = IOM_EBADMODE;
            }
            else
            {
                /* Check the State of port */
                if (portSC->state != PSP_H3A_OPENED)
                {
                    statusSC = IOM_EBADMODE;
                }
                else
                {
                    /* DDA Layer function is called
                       to submit the data request */
                    statusSC = PSP_H3ASubmit(chanSC->ddc_Handle,
                        (PSP_VPSSSubmitCommand)(packet->cmd),
                        packet->addr);
                    if (statusSC != PSP_SOK)
                    {
                        DDAIOM_H3A_DEBUG_PRINT(
                            "H3A_mdSubmitChan: Error: Submit failed\n");
                        /* Generate the IOM error code
                           equivalent to PSP error code */
                        statusSC = getIomErrorCode(statusSC);
                    }
                }
            }
        }
    }
#ifdef H3A_INSTRUMENTATION_ENABLED
    /*PSP_DVT_dCHANNEL_ID  = chanSC->ddc_handle
      PSP_DVT_dPACKET_ID = packet->cmd      */
        if(1 == idSBT){
            PSP_LOG_EVENT(  &DVTEvent_Log,
                "H3A",
                PSP_DVT_DESC(
                PSP_DVT_eFUNC_END,
                PSP_DVT_dCHANNEL_ID_INST,
                PSP_DVT_dPACKET_ID,
                PSP_DVT_dSIZE_BYTES),
                CHANNELID_INSTANCEID(1,0),                    /* channel 1 for AEW */
                packet->addr,
                0
                );
        }
        else{
            PSP_LOG_EVENT(  &DVTEvent_Log,
                "H3A",
                PSP_DVT_DESC(
                PSP_DVT_eFUNC_END,
                PSP_DVT_dCHANNEL_ID_INST,
                PSP_DVT_dPACKET_ID,
                PSP_DVT_dSIZE_BYTES),
                CHANNELID_INSTANCEID(1,0),                    /* channel 0 for AF */
                packet->addr,
                0
                );
        }
#endif /*H3A_INSTRUMENTATION_ENABLED */
    return statusSC;
}

