/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \file   dda_h3aIOM_BIOS.h
 *
 *  \brief  H3A IOM and BIOS layer definition
 *
 *  This file contains the interfaces, data types and symbolic definitions
 *  that are needed by the application to utilize the serivces of the H3A
 *  device driver.
 *
 *  (C) Copyright 2010, Texas Instruments, Inc
 *
 *  \author     EI4
 *
 *  \version    0.1             Created
 *              1.0             First Release
 */

#ifndef _DDA_H3AIOM_BIOS_H
#define _DDA_H3AIOM_BIOS_H

/******************************************************************************
  Header File Inclusion
 ******************************************************************************/
#include <gio.h>
#include <ti/sdo/pspdrivers/drivers/h3a/psp_h3a.h>
#include <ti/sdo/pspdrivers/drivers/h3a/src/ddc_h3a.h>

/******************************************************************************
  Data-Structures
 ******************************************************************************/
/*******************************************************************************
  IOM Function Table
  Driver function table to be used by applications.
  *****************************************************************************/
#ifdef __cplusplus
extern "C" {
#endif
    extern IOM_Fxns H3AMD_FXNS;

/******************************************************************************
  Data-Structures
 ******************************************************************************/
/**
 * \brief H3A_IOMCHANNELOBJ
 * This structure describes the channel Object
 */
typedef struct H3A_IOMCHANNELOBJ
{
    struct _H3APORTObj          *port;
    /**< argument for cbFxn         */
    PSP_Handle                   ddc_Handle;
    /**< Handle for the channels    */
} H3AChannelObject;

/**
 * \brief _H3APORTObj
 * This structure describes the H3A Port Object
 */
typedef struct _H3APORTObj
{
    Uint32              portNumber;
    /**< port number                */
    DDC_H3ADeviceObject *deviceObj;
    /**< H3A device object          */
    PSP_H3AState        state;
    /**< state of the device        */
    H3AChannelObject    afChan;
    /**< H3A channel object for AF  */
    H3AChannelObject    aewChan;
    /**< H3A channel object for AEW */
} H3APortObject;

/******************************************************************************
  Functions
 ******************************************************************************/
/**
 * \brief H3A_mdBindDev
 * Initialize a given H3A driver (instance)
 * It performs register map overlaying and software book-keeping. It register
 * the interrupt Handler.
 *
 * \param  devp        [IN]    Void pointer passed by the GIO Layer to
 *                             store the address of the port structure
 *
 * \param  devid       [IN]    device instance number
 *
 * \param  devParams   [IN]    Initialization configuration structure
 *
 * \return success             IOM_COMPLETED
 *         failure             IOM Error
 */
Int32 H3A_mdBindDev(Ptr *devp, Int32 devid, Ptr devParams);

/**
 * \brief H3A_mdUnBindDev
 * Delete a given H3A driver (instance)
 * This function is called last and is used for driver deletion.This function
 * does the deletion of the entire H3A hardware.
 *
 * \param  devp        [IN]    pointer in the GIO layer giving the address of
 *                             port object of that particular instance
 *
 * \Return                     On success returns IOM_COMPLETED
 *                             On failure returns IOM Error code
 */
Int32 H3A_mdUnBindDev(Ptr devp);

/**
 * \brief H3A_mdCreateChan
 * Open Channel of the H3A device
 * The function prepares channel of the driver instance for data transfers
 * and returns an handle to the driver instance.
 *
 * \param chanp        [IN]     pointer to the channel structure
 *
 * \param devp         [IN/OUT] pointer to the port structure
 *
 * \param name         [IN]     name of the driver
 *
 * \param mode         [IN]     mode of the channel i.e. INPUT or OUTPUT
 *
 * \param chanParams   [IN/OUT] Extra parameter (Not used in this driver)
 *
 * \param cbFxn        [IN]     callback to the GIO Layer
 *
 * \param cbArg        [IN]     arguments for the callback
 *
 * \Returns                     On success returns IOM_COMPLETED
 *                              On failure returns IOM Error code
 */
Int32 H3A_mdCreateChan(Ptr *chanp, Ptr devp, String name, Int32 mode,
    Ptr chanParams, IOM_TiomCallback cbFxn, Ptr cbArg);

/**
 * \brief H3A_mdDeleteChan
 * Close Channel of the H3A device
 * This function is used to close Channel handle for individual H3A
 * Modules.
 *
 * \param  chanp       [IN]    H3A Driver Channel Handle
 *
 * \return                     On success returns IOM_COMPLETED
 *                             On failure returns IOM Error code
 */
Int32 H3A_mdDeleteChan(Ptr chanp);

/**
 * \brief H3A_mdControlChan
 *
 * This function provides ioctl functionality for H3A
 *
 * \param  chanp       [IN]     H3A Driver Channel Handle
 *
 * \param  cmd         [IN]     Operation to be performed, typically an enum
 *                              gets passed
 *
 * \param  arg         [IN/OUT] Provides additonal information related to the
 *                              operation
 *
 * \return success              IOM_COMPLETED
 *         failure              IOM Error
 */
Int32 H3A_mdControlChan(Ptr chanp, Uint32 cmd, Ptr arg);

/**
 * \brief H3A_mdSubmitChan
 *
 *  This function is called to Queue/Dequeue Frame buffer to the  H3A
 *  device.
 *
 * \param  chanp       [IN]    H3A Driver Channel Handle
 *
 * \param  packet      [IN]    IOM packet passed by the upper layer containing
 *                             information about the size, address, timeout and
 *                             other necessary parameters
 *
 * \return                     On success, returns positive number - depending
 *                             upon the operation requested
 *
 *                             If failed, returns negative number - IOM Error
 *                             code
 */
Int32 H3A_mdSubmitChan(Ptr chanp, IOM_Packet *packet);

/******************************************************************************
  BIOS SPECIFIC FUNCTIONS
 ******************************************************************************/
/**
 * \brief DDA_H3ARegisterIntrHandler
 * Registers interrupt handler for H3A
 *
 * \param  IntNum      [IN]    interrupt number
 * \return void
 */
void DDA_H3ARegisterIntrHandler( Uint32 IntNum);

/**
 * \brief DDA_H3AUnregisterIntrHandler
 * Unregister Interrupt handler for H3A
 *
 * \param  intrNum     [IN]    interrupt number
 * \return void
 */
void DDA_H3AUnregisterIntrHandler(Uint32 intrNum);

#ifdef __cplusplus
}
#endif

#endif /*_DDA_H3AIOM_BIOS_H */

