/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \file   ddc_aew.h
 *
 *  \brief  AEW interface definition
 *
 *  This file contains the interfaces, data types and symbolic definitions
 *  that are needed by the application to utilize the serivces of the AEW module
 *  of H3A device driver.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author     EI4
 *
 *  \version    0.1             Created
 *              1.0             First Release
 */
#ifndef _DDC_AEW_H_
#define _DDC_AEW_H_

/******************************************************************************
  Header File Inclusion
 ******************************************************************************/
#include <ti/sdo/pspdrivers/pal_os/pal_osSem.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osList.h>
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/tistdtypes.h>
#include <ti/sdo/pspdrivers/drivers/h3a/psp_h3a.h>

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
  Macros
 ******************************************************************************/
#define DDC_AEW_MIN_HZCOUNT             (2u)
#define DDC_AEW_MAX_HZCOUNT             (36u)

#define DDC_AEW_MIN_VTCOUNT             (1u)
#define DDC_AEW_MAX_VTCOUNT             (128u)

#define DDC_AEW_MIN_WIDTH               (6u)
#define DDC_AEW_MAX_WIDTH               (256u)

#define DDC_AEW_MIN_HEIGHT              (2u)
#define DDC_AEW_MAX_HEIGHT              (256u)

#define DDC_AEW_MAX_HZSTART             (4095u)

#define DDC_AEW_MAX_VTSTART             (4095u)

#define DDC_AEW_MAX_BLKWIN_VTSTART       (4095u)

#define DDC_AEW_MIN_BLKWIN_HEIGHT       (2u)
#define DDC_AEW_MAX_BLKWIN_HEIGHT       (256u)

#define DDC_AEW_MIN_HZLINEINCR          (2u)
#define DDC_AEW_MAX_HZLINEINCR          (32u)

#define DDC_AEW_MIN_VTLINEINCR          (2u)
#define DDC_AEW_MAX_VTLINEINCR          (32u)

#define DDC_AEW_AVE2LMT_MAX             (1023u)
#define DDC_AEW_AVE2LMT_MIN             (0)
/******************************************************************************
  Data-Structures
 ******************************************************************************/
/**
 * \brief _DDC_AEWChannelObject
 * This structure describes the AEW Device Object
 */
typedef struct _DDC_AEWChannelObject
{
    Uint32                      channelId;
    /**< Channel id for AEW object                              */
    DDC_H3AChanState            state;
    /**< state for the channel                                  */
    PSP_AEWParams               params;
    /**< Handle to access the AEW config parameters             */
    PAL_OsSemHandle             sem;
    /**< Synchronizing Object                                   */
    PAL_OsSemHandle             semISR;
    /**< Synchronizing Object for ISR                           */
    DDC_H3ABufferObject         buffObj;
    /**< Handle for accessing the buffer object                 */
    Uint32                      buffSize;
    /**< Size of the buffer                                     */
    DDC_ParamsConfigDone        configFlag;
    /**< configuration Flag to indicate whether set up is done  */
    Int32                       semTimeOut;
    /**< Pending on semaphore timeout */
} DDC_AEWChannelObject;

/******************************************************************************
  Functions
 ******************************************************************************/
/**
 * \brief DDC_AEWSubmit
 * execute the Submit Command for AEW
 *
 * \param handle            [IN]    PSP handle
 *
 * \param cmd               [IN]    IOCTL command
 *
 * \param cmdArg            [IN]    Command Arguments to be passed.
 *                                  ConfigParams to be passed as cmdArg
 *                                  depending upon the channel
 *
 * \return Returns          [OUT]   DDC status
 */
DDC_H3AStatus DDC_AEWSubmit(PSP_Handle handle, PSP_VPSSSubmitCommand cmd,
    Ptr cmdArg);

/**
 * \brief DDC_AEWOpenHandle
 * Returs the Channel handle for AEW
 *
 * \param  void
 *
 * \return Returns          [OUT]   PSP Handle
 */
PSP_Handle DDC_AEWOpenHandle(void);

/**
 * \brief DDC_AEWIsChannelHandleClosed
 * Returns status of the handle for AEW
 *
 * \param  handle           [IN]    Channel handle
 *
 * \return Returns          [OUT]   DDC status
 */
DDC_H3AStatus DDC_AEWIsChannelHandleClosed (PSP_Handle handle);

/**
 * \brief DDC_AEWValidateParameters
 * Validates parameter passed by application
 *
 * \param  paramsaew        [IN]    AEW channel parameter
 *
 * \return Returns          [OUT]   DDC status
 */
DDC_H3AStatus DDC_AEWValidateParameters(PSP_AEWParams *);

/**
 * \brief DDC_AEWGetParameters
 * Get channel specific parameters for AEW
 *
 * \param  handleAEW        [IN]    AEW channel handle
 *
 * \param  params           [IN]    AEW channel parameter
 *
 * \return Returns          [OUT]   DDC status
 */
DDC_H3AStatus DDC_AEWGetParameters(DDC_AEWChannelObject *,
    PSP_AEWParams *const);

/**
 * \brief DDC_AEWCloseHandle
 * Closes the channel associated with handle of AEW
 *
 * \param  handle           [IN]    AEW channel handle
 *
 * \return Returns          [OUT]   DDC status
 */
DDC_H3AStatus DDC_AEWCloseHandle(PSP_Handle handle);

/**
 * \brief DDC_AEWisr
 * This function is Interrupt service routine for AEW
 *
 * \param  void
 *
 * \return void
 */
void DDC_AEWisr(void);

/**
 * \brief DDC_AEWIoctl
 * execute the control command
 *
 * \param handle            [IN]    PSP handle
 *
 * \param cmd               [IN]    IOCTL command
 *
 * \param cmdArg            [IN]    Command Arguments to be passed.
 *                                  ConfigParams to be passed as cmdArg
 *                                  depending upon the channel
 *
 * \param Params            [IN]    pointer to params structure of AEW
 *
 * \return Returns          [OUT]   DDC status
 */
DDC_H3AStatus DDC_AEWIoctl(PSP_Handle handle, PSP_H3AIoctlCmd cmd, Ptr cmdArg,
    Ptr params);

#ifdef __cplusplus
}
#endif

#endif  /* _DDC_AEW_H_ */

