/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \file   ddc_af.h
 *
 *  \brief  AF interface definition
 *
 *  This file contains the interfaces, data types and symbolic definitions
 *  that are needed by the application to utilize the serivces of the AF module
 *  of H3A device driver.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author     EI4
 *
 *  \version    0.1             Created
 *              1.0             First Release
 */

#ifndef _DDC_AF_H_
#define _DDC_AF_H_

/******************************************************************************
  Header File Inclusion
 ******************************************************************************/
#include <ti/sdo/pspdrivers/pal_os/pal_osSem.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osList.h>
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/tistdtypes.h>
#include <ti/sdo/pspdrivers/drivers/h3a/psp_h3a.h>

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
  Macros
 ******************************************************************************/
#define DDC_AF_MIN_HZCOUNT      (2u)
#define DDC_AF_MAX_HZCOUNT      (36u)
#define DDC_AF_MIN_VTCOUNT      (1u)
#define DDC_AF_MAX_VTCOUNT      (128u)

#define DDC_AF_MIN_HZSTART      (2u)
#define DDC_AF_MAX_HZSTART      (4094u)

#define DDC_AF_MIN_WIDTH        (6u)
#define DDC_AF_MAX_WIDTH        (256u)

/* minimum value is -2^11 sign extended value */
#define DDC_AF_MIN_COEF         ((Int32)0xFFFFF800)
/* maximum value is +2^11- 1 sign extended value */
#define DDC_AF_MAX_COEF         ((Int32)0x000007FF)

#define DDC_AF_MIN_HEIGHT       (2u)
#define DDC_AF_MAX_HEIGHT       (256u)

#define DDC_AF_MIN_LINE_INCR    (2u)
#define DDC_AF_MAX_LINE_INCR    (32u)

#define DDC_AF_MAX_VTSTART      (4095u)

#define DDC_AF_MAX_IIRSH        (4094u)

#define DDC_AF_HMF_MEDTH_MAX    (255u)


/******************************************************************************
  Data-Structures
 ******************************************************************************/
/**
 * \brief _DDC_AFChannelObject
 * This structure describes the AF Channel Object
 */
typedef struct _DDC_AFChannelObject
{
    Uint32                  channelId;
    /**< Channel id for AF object                               */
    DDC_H3AChanState        state;
    /**< state for the channel                                  */
    PSP_AFParams            params;
    /**< Handle to access the AF config parameters              */
    PAL_OsSemHandle         sem;
    /**< Synchronizing Object                                   */
    PAL_OsSemHandle         semISR;
    /**< Synchronizing Object   for ISR                         */
    DDC_H3ABufferObject     buffObj;
    /**< Handle for accessing the buffer object                 */
    Uint32                  buffSize;
    /**< Size of the buffer                                     */
    DDC_ParamsConfigDone    configFlag;
    /**< configuration Flag to indicate whether set up is done  */
    Int32                   semTimeOut;
    /**< Pending on semaphore timeout */
} DDC_AFChannelObject;

/******************************************************************************
  Functions
 ******************************************************************************/
/**
 * \brief DDC_AFSubmit
 * execute the Submit Command for AF
 *
 * \param  handle           [IN]    PSP handle
 *
 * \param cmd               [IN]    IOCTL command
 *
 * \param cmdArg            [IN]    Command Arguments to be passed.
 *                                  ConfigParams to be passed as cmdArg
 *                                  depending upon the channel
 *
 * \return Returns          [OUT]   DDC status
 */
DDC_H3AStatus DDC_AFSubmit(PSP_Handle handle, PSP_VPSSSubmitCommand cmd,
    Ptr cmdArg);

/**
 * \brief DDC_AFOpenHandle
 * Returs the Channel handle for AF
 *
 * \param  void
 *
 * \return Returns          [OUT]   PSP Handle
 */
PSP_Handle DDC_AFOpenHandle(void);

/**
 * \brief DDC_AFIsChannelHandleClosed
 * Returns status of the handle for AF
 *
 * \param  handle           [IN]    Channel handle
 *
 * \return Returns          [OUT]   DDC status
 */
DDC_H3AStatus DDC_AFIsChannelHandleClosed (PSP_Handle handle);

/**
 * \brief DDC_AFValidateParameters
 * Validates parameter passed by application
 *
 * \param  paramsaf         [IN]    AF channel parameter
 *
 * \return Returns          [OUT]   DDC status
 */
DDC_H3AStatus DDC_AFValidateParameters(PSP_AFParams *);

/**
 * \brief DDC_AFGetParameters
 * Get channel specific parameters for AF
 *
 * \param  handleAF         [IN]    AF channel handle
 *
 * \param  params           [IN]    AF channel parameter
 *
 * \return Returns          [OUT]   DDC status
 */
DDC_H3AStatus DDC_AFGetParameters(DDC_AFChannelObject *, PSP_AFParams *const);

/**
 * \brief DDC_AFCloseHandle
 * Closes the channel associated with handle of AF
 *
 * \param  handle           [IN]    AF channel handle
 *
 * \return Returns          [OUT]   DDC status
 */
DDC_H3AStatus DDC_AFCloseHandle(PSP_Handle handle);

/**
 * \brief DDC_AFisr
 * This function is Interrupt service routine for AF
 *
 * \param  void
 *
 * \return void
 */
void DDC_AFisr(void);

/**
 *  \brief  execute the control command
 *
 *  \param handle           [IN]    PSP handle
 *
 *  \param cmd              [IN]    IOCTL command
 *
 *  \param cmdArg           [IN]    Command Arguments to be passed.
 *                                  ConfigParams to be passed as cmdArg
 *                                  depending upon the channel
 *
 *  \param Params           [IN]    pointer to params structure of AF
 *
 * \return Returns          [OUT]   DDC status
 */
DDC_H3AStatus DDC_AFIoctl(PSP_Handle handle, PSP_H3AIoctlCmd cmd, Ptr cmdArg,
    Ptr params);

#ifdef __cplusplus
}
#endif

#endif  /* _DDC_AF_H_ */

