/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \file   llc_h3a.h
 *
 *  \brief  H3A LLC layer Function dependencies
 *
 *  This file contains the decleration of LLC layer functions and macros.
 *
 *  (C) Copyright 2010, Texas Instruments, Inc
 *
 *  \author     EI4
 *
 *  \version    0.1             Created
 *              1.0             First Release
 */

#ifndef _LLC_H3A_H
#define _LLC_H3A_H

/******************************************************************************
  Header File Inclusion
 ******************************************************************************/
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/csl_types.h>
#ifdef H3A_DEBUG
#include <ti/sdo/pspdrivers/pal_os/bios/psputils.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
  Macros
 ******************************************************************************/
#ifdef H3A_DEBUG
#define LLC_H3A_DEBUG_PRINT(x)    PSP_DEBUG(x)
#else
#define LLC_H3A_DEBUG_PRINT(x)
#endif

/* Converting value passed by application to the range required for hardware
    configuration */
#define LLC_H3A_SETREG(number)      ((number/(2u))-(1u))

/******************************************************************************
  Functions
 ******************************************************************************/
/**
 * \brief LLC_AFHardwareSetup
 * Writes configuration parameters to the hardware for AF
 *
 * \param  params           [IN]    AF parameter structure pointer
 * \param  regs             [IN]    Register overlaying pointer
 *
 * \return success                  CSL_SOK
 *         failure                  CSL_ESYS_INVPARAMS
 */
CSL_Status LLC_AFHardwareSetup(PSP_AFParams * params, Ptr regs);

/**
 * \brief LLC_AEWHardwareSetup
 * Writes configuration parameters to the hardware for AEW
 *
 * \param  params           [IN]    AEW parameter structure pointer
 * \param  regs             [IN]    Register overlaying pointer
 *
 * \return success                  CSL_SOK
 *         failure                  CSL_ESYS_INVPARAMS
 */
CSL_Status LLC_AEWHardwareSetup(PSP_AEWParams * params, Ptr regs);

/**
 * \brief LLC_SetAFEngine
 * Start or stop AF engine according to the value passed
 *
 * \param  value            [IN]    value to be written to start or stop
 * \param  regs             [IN]    Register overlaying pointer
 *
 * \return success                  CSL_SOK
 *         failure                  CSL_ESYS_INVPARAMS
 */
CSL_Status LLC_SetAFEngine(Uint32 value, Ptr regs);

/**
 * \brief LLC_SetAEWEngine
 * Start or stop AEW engine according to the value passed
 *
 * \param  value            [IN]    value to be written to start or stop
 * \param  regs             [IN]    Register overlaying pointer
 *
 * \return success                  CSL_SOK
 *         failure                  CSL_ESYS_INVPARAMS
 */
CSL_Status LLC_SetAEWEngine(Uint32 value, Ptr regs);

/**
 * \brief LLC_SetAFBUFST
 * Sets Buffer starting address register for AF
 *
 * \param  regs             [IN]    Register overlaying pointer
 * \param  address          [IN]    holds memory address
 *
 * \return success                  CSL_SOK
 *         failure                  CSL_ESYS_INVPARAMS
 */
CSL_Status LLC_SetAFBUFST (Ptr regs, Uint32 address);

/**
 * \brief LLC_SetAEWBUFST
 * Sets Buffer starting address register for AEW
 *
 * \param  regs             [IN]    Register overlaying pointer
 * \param  address          [IN]    holds memory address
 *
 * \return success                  CSL_SOK
 *         failure                  CSL_ESYS_INVPARAMS
 */
CSL_Status LLC_SetAEWBUFST (Ptr regs, Uint32 address);

/**
 * \brief LLC_GetBusyAF
 * Get busy bit status of AF
 *
 * \param  regs             [IN]    Register overlaying pointer
 *
 * \return                          Value of busy bit for AF
 */
Int32 LLC_GetBusyAF (Ptr regs);

/**
 * \brief LLC_GetBusyAEW
 * Get busy bit status of AEW
 *
 * \param  regs             [IN]    Register overlaying pointer
 *
 * \return                          Value of busy bit for AEW
 */
Int32 LLC_GetBusyAEW (Ptr regs);

/**
 * \brief LLC_GetAFHWStatus
 * Get status of AF engine
 *
 * \param  regs             [IN]    Register overlaying pointer
 *
 * \return                          Value according to engine is running or busy
 */
Int32 LLC_GetAFHWStatus (Ptr regs);

/**
 * \brief LLC_GetAEWHWStatus
 * Get status of AEW engine
 *
 * \param  regs             [IN]    Register overlaying pointer
 *
 * \return                          Value according to engine is running or busy
 */
Int32  LLC_GetAEWHWStatus (Ptr regs);

/**
 * \brief LLC_AFGetWriteBufferStatus
 * Get Buffer overflow bit status of AF
 *
 * \param  regs             [IN]    Register overlaying pointer
 *
 * \return                          Value of overflow bit for AF
 */
Int32 LLC_AFGetWriteBufferStatus (Ptr regs);

/**
 * \brief LLC_AEWGetWriteBufferStatus
 * Get Buffer overflow bit status of AEW
 *
 * \param  regs             [IN]    Register overlaying pointer
 *
 * \return                          Value of overflow bit for AEW
 */
Int32 LLC_AEWGetWriteBufferStatus (Ptr regs);

/**
 * \brief LLC_AFInitRegister
 * Initializes AF hardware register with reset values
 *
 * \param  regs             [IN]    Register overlaying pointer
 *
 * \return success                  CSL_SOK
 *         failure                  CSL_ESYS_INVPARAMS
 */
CSL_Status LLC_AFInitRegister(Ptr regs);

/**
 * \brief LLC_AEWInitRegister
 * Initializes AEW hardware register with reset values
 *
 * \param  regs             [IN]    Register overlaying pointer
 *
 * \return success                  CSL_SOK
 *         failure                  CSL_ESYS_INVPARAMS
 */
CSL_Status LLC_AEWInitRegister(Ptr regs);

/**
 * \brief LLC_H3AResetPCRRegister
 * Initializes PCR register with reset value
 *
 * \param  regs             [IN]    Register overlaying pointer
 *
 * \return success                  CSL_SOK
 *         failure                  CSL_ESYS_INVPARAMS
 */
CSL_Status LLC_H3AResetPCRRegister(Ptr const regs);

/**
 * \brief LLC_AFClearWriteBuffer
 * Clears buffer overflow bit for AF
 *
 * \param  regs             [IN]    Register overlaying pointer
 *
 * \return success                  CSL_SOK
 *         failure                  CSL_ESYS_INVPARAMS
 */
CSL_Status LLC_AFClearWriteBuffer (Ptr const regs);

/**
 * \brief LLC_AEWClearWriteBuffer
 * Clears buffer overflow bit for AEW
 *
 * \param  regs             [IN]    Register overlaying pointer
 *
 * \return success                  CSL_SOK
 *         failure                  CSL_ESYS_INVPARAMS
 */
CSL_Status LLC_AEWClearWriteBuffer (Ptr const regs);

#ifdef __cplusplus
}
#endif
#endif  /* _LLC_H3A_H */


