/******************************************************************************
 **+-------------------------------------------------------------------------+**
 **|                            ****                                         |**
 **|                            ****                                         |**
 **|                            ******o***                                   |**
 **|                      ********_///_****                                  |**
 **|                      ***** /_//_/ ****                                  |**
 **|                       ** ** (__/ ****                                   |**
 **|                           *********                                     |**
 **|                            ****                                         |**
 **|                            ***                                          |**
 **|                                                                         |**
 **|         Copyright (c) 2006-2010 Texas Instruments Incorporated          |**
 **|                        ALL RIGHTS RESERVED                              |**
 **|                                                                         |**
 **| Permission is hereby granted to licensees of Texas Instruments          |**
 **| Incorporated (TI) products to use this computer program for the sole    |**
 **| purpose of implementing a licensee product based on TI products.        |**
 **| No other rights to reproduce, use, or disseminate this computer         |**
 **| program, whether in part or in whole, are granted.                      |**
 **|                                                                         |**
 **| TI makes no representation or warranties with respect to the            |**
 **| performance of this computer program, and specifically disclaims        |**
 **| any responsibility for any damages, special or consequential,           |**
 **| connected with the use of this program.                                 |**
 **|                                                                         |**
 **+-------------------------------------------------------------------------+**
 ******************************************************************************/
/**
 * \file   psp_histogram.h
 *
 * \brief  Histogram interface definition
 *
 * This file contains the interfaces, data types and symbolic definitions
 * that are needed by the application to utilize the serivces of the Histogram
 * device driver.
 *
 * (C) Copyright 2010, Texas Instruments, Inc
 *
 * \author     PSP Team
 *
 * \version    0.1       created
 *             1.0       First Release
 *
 * <b>Compiler Switch Used</b>
 *
 * <b>_DEBUG</b><br>
 *   This is used as a flag to compiler whether to include the debug statements
 *   inserted in the code into the final image. This flag helps to build DEBUG
 *   image of the program. For RELEASE images this is not passed to the compiler.
 *
 * <b>CHIP_XXXX</b><br>
 *   The CSL layer is written in a common file for all the variants of a SOC.
 *   This flag differentiates the variant we are compiling for, for e.g.
 *   CHIP_DM648, and the CSL definitions for that variant appropriately gets
 *   defined for register base addresses, num of ports of a peripheral etc.
 *
 * <b>HIST_INSTRUMENTATION_ENABLED</b><br>
 *   This flag is passed to the compiler to include the instrumentation code
 *   parts into the final image/lib of the program. This helps build the
 *   iRelease/iDebug versions of the image/lib with a common code base
 *
 */

#ifndef _PSP_HISTOGRAM_H
#define _PSP_HISTOGRAM_H

/******************************************************************************
  Header File Inclusion
 ******************************************************************************/
/**
 * PSP Common header provides standard driver return codes, operation modes
 * and other generic defines required by all PSP drivers
 */
#include <std.h>
#include <gio.h>
#include <ti/sdo/pspdrivers/pal_os/pal_oslist.h>
#include <ti/sdo/pspdrivers/system/dm6437/bios/evmDM6437/video/psp_vpss.h>

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
  Defines
 ******************************************************************************/
#define PSP_HIST_REGION_MAX_HPOS (16384u)
/**< Region maximum horizontal position : 14bits */
#define PSP_HIST_REGION_MAX_VPOS (16384u)
/**< Region maximum vertical position : 14bits   */
#define PSP_HIST_NUM_INSTANCES   (1u)
/**< Histogram device maximum instance           */

/******************************************************************************
  Enumerations
 ******************************************************************************/
/**
 * \brief Enum for clearing of Histogram output data after read.
 *
 */
typedef enum _PSP_HISTclearopdata
{
    PSP_HIST_NO_CLEAR_AFTER_READ = 0,
    /**< Do not clear Histogram output data after read */
    PSP_HIST_CLEAR_AFTER_READ
    /**< Clear Histogram output data after read        */
} PSP_HISTclearopdata;

/**
 * \brief Enum for bins per Histogram.
 *
 */
typedef enum _PSP_HISTbins
{
    PSP_HIST_32BINS  = 0,
    /**< 32 bins per Histogram  */
    PSP_HIST_64BINS,
    /**< 64 bins per Histogram  */
    PSP_HIST_128BINS,
    /**< 128 bins per Histogram */
    PSP_HIST_256BINS
    /**< 256 bins per Histogram */
} PSP_HISTbins;

/**
 * \brief Enum for right shift of data for Histogram bining
 *
 */
typedef enum _PSP_HISTdatarightshift
{
    PSP_HIST_DATA_RSHIFT_ZERO  = 0,
    /**< Right shift data by 0 for Histogram bining */
    PSP_HIST_DATA_RSHIFT_ONE,
    /**< Right shift data by 1 for Histogram bining */
    PSP_HIST_DATA_RSHIFT_TWO,
    /**< Right shift data by 2 for Histogram bining */
    PSP_HIST_DATA_RSHIFT_THREE,
    /**< Right shift data by 3 for Histogram bining */
    PSP_HIST_DATA_RSHIFT_FOUR,
    /**< Right shift data by 4 for Histogram bining */
    PSP_HIST_DATA_RSHIFT_FIVE,
    /**< Right shift data by 5 for Histogram bining */
    PSP_HIST_DATA_RSHIFT_SIX,
    /**< Right shift data by 6 for Histogram bining */
    PSP_HIST_DATA_RSHIFT_SEVEN
    /**< Right shift data by 7 for Histogram bining */
} PSP_HISTdatarightshift;

/**
 * \brief Enum for Ioctl for various types of control
 *
 */
typedef enum _PSP_HISTIoctlCmd
{
    PSP_HIST_IOCTL_SET_PARAMS = IOM_USER,
    /**< Set histogram configuratoin parameter */
    PSP_HIST_IOCTL_GET_PARAMS,
    /**< Get histogram configuratoin parameter */
    PSP_HIST_IOCTL_SET_SEM_TIMEOUT
    /**< Change the timeout value used in semaphore pending operation.
         Command Argument:  Timeout values<Int32>                           */
} PSP_HISTIoctlCmd;

/******************************************************************************
  Data-Structures
 ******************************************************************************/
/**
 * \brief  HISTMD_FXNS
 *
 *  This is used by application as function table pointer.
 */
    extern far IOM_Fxns HISTMD_FXNS;

/**
 * \brief  PSP_HISTregionboundries
 *         structure for histogram region boundries
 *
 */
typedef struct _PSP_HISTregionboundries
{
    Uint16     hStart;
    /**< Horizontal start position */
    Uint16     hEnd;
    /**< Horizontal end position   */
    Uint16     vStart;
    /**< Vertical start position   */
    Uint16     vEnd;
    /**< Vertical end position     */
} PSP_HISTregionboundries;

/**
 * \brief  PSP_HISTparam
 *         structure for Histogram configuration parameter
 *
 */
typedef struct _PSP_HISTparam
{
    PSP_HISTclearopdata     clearOpDataAfterRead;
    /**< Clearing of Histogram output data after read */
    PSP_HISTbins            bins;
    /**< Bins per Histogram                           */
    PSP_HISTdatarightshift  shift;
    /**< Right shift of data for Histogram bining     */
    Uint8                   wbGain[4];
    /**< White balance gain for all colors            */
    PSP_HISTregionboundries region[4];
    /**< Region boundries for all regions             */
} PSP_HISTparam;

/**
 * \brief  PSP_HISTbuffer
 *         structure to read Histogram output data
 *
 */
typedef struct _PSP_HISTbuffer
{
    PAL_OsListNodeHeader  hNode;
    /**< Head node to queue list    */
    Ptr                   ramIpAddr;
    /**< Buffer address             */
    Uint32                bufferSize;
    /**< Buffer size                */
    Uint32                timeStmp;
    /**< Timestamp to captured data */
} PSP_HISTbuffer;

#ifdef __cplusplus
}
#endif

#endif /* _PSP_HISTOGRAM_H */
