/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/
/**
 * \file   dda_histIOM.c
 *
 * \brief  Histogram IOM layer Functions
 *
 * This file contains the Implementation details of IOM layer functions
 *
 * (C) Copyright 2010, Texas Instruments, Inc
 *
 * \author     EI5
 *
 * \version    0.1             Created
 *             1.0             First Release
 */

/******************************************************************************
  Header File Inclusion
 ******************************************************************************/
#include <ti/sdo/pspdrivers/drivers/histogram/psp_histogram.h>
#include <ti/sdo/pspdrivers/drivers/histogram/src/psp_histApi.h>
#include <ti/sdo/pspdrivers/drivers/histogram/src/dda_histIOM.h>
#include <iom.h>

#ifdef HIST_INSTRUMENTATION_ENABLED
#include <ti/sdo/pspdrivers/pal_os/bios/psp_log.h>
#endif /* HIST_INSTRUMENTATION_ENABLED */

/******************************************************************************
  Globals
 ******************************************************************************/
/**
 * \brief   HISTMD_FXNS
 *          Mini Driver Function interface table
 *
 */
IOM_Fxns HISTMD_FXNS =
{
    &HIST_mdBindDev,
    &HIST_mdUnBindDev,
    &HIST_mdControlChan,
    &HIST_mdCreateChan,
    &HIST_mdDeleteChan,
    &HIST_mdSubmitChan
};

/**
 * The following mapping shows the mapping done from PSP_Result to IOM error
 * code.
 *
 * PSP_SOK                  (0)   IOM_COMPLETED            0
 * PSP_SINPROGRESS          (1)   IOM_PENDING              1
 * PSP_E_DRIVER_INIT        (-1)  IOM_ETIMEOUTUNREC      -11
 * PSP_E_NO_MEMORY          (-2)  IOM_EALLOC              -5
 * PSP_E_RESOURCES          (-3)  IOM_EALLOC              -5
 * PSP_E_INVAL_STATE        (-4)  IOM_EBADMODE            -7
 * PSP_E_INVAL_PARAM        (-5)  IOM_EBADARGS           -10
 * PSP_E_NOT_SUPPORTED      (-6)  IOM_ENOTIMPL            -9
 * PSP_E_IO_CANCEL_FAIL     (-7)  IOM_ETIMEOUT            -2
 * PSP_E_FIFO_NOT_ENABLED   (-8)  IOM_EBADMODE            -7
 * PSP_E_INVALID_MODE       (-9)  IOM_EBADMODE            -7
 * PSP_E_INVALID_CMD        (-10) IOM_EBADARGS           -10
 *
 */

/**
 * \brief IOM Port Object
 */
HISTPortObject hist_port =
    {DDAIOM_HIST_PORT_INITVAL, {NULL, NULL}, NULL, DDC_HIST_DELETED};

/**
 * \brief  getIomErrorCode
 *         This function converts PSP error codes to appropriate IOM error
 *         code.
 *
 * \param  pspCode          [IN]   PSP Error code.
 *
 * \return IOM ERROR CODE   [OUT]  Equivalent IOM error code. For unrecoganized
 *                                 error code IOM_EBADIO is returned.
 *
 */
static Int getIomErrorCode(PSP_Result pspCode);

/******************************************************************************
  Functions
 ******************************************************************************/
/**
 * \brief  getIomErrorCode
 *         This function converts PSP error codes to appropriate IOM error
 *         code.
 */
static Int getIomErrorCode(PSP_Result pspCode)
{
    Int iomErrorCodeList[12] = {
        IOM_EBADARGS,
        IOM_EBADMODE,
        IOM_EBADMODE,
        IOM_ETIMEOUT,
        IOM_ENOTIMPL,
        IOM_EBADARGS,
        IOM_EBADMODE,
        IOM_EALLOC,
        IOM_EALLOC,
        IOM_ETIMEOUTUNREC,
        IOM_COMPLETED,
        IOM_PENDING
    };

    /* Mapping of psp level error code with the iom layer code. The maximum
       error code return from the below layer is 10 so taking the error code*/
    Int iomCode = pspCode + 10;
    /* Mapping the error code with its equivalent IOM layer error code */
    if ((12 > iomCode) && (iomCode >= 0))
    {
        iomCode = iomErrorCodeList[iomCode];
    }
    else
    {
        iomCode = IOM_EBADIO;
    }

    return(iomCode);
}

/**
 * \brief   HIST_mdBindDev
 *          This function is called at the time of device creation to bind
 *          and initialize the instance
 */
/* ARGSUSED */
Int HIST_mdBindDev(Ptr *devp, Int devid, Ptr devParams)
{
    DDC_HISTdeviceobject *histDeviceObject;
    Int     statusBnd = IOM_COMPLETED;
    Uint32  ldevid    = (Uint32)devid;

    /*Check the Parameters are not NULL and validate the instance */
    if ((NULL == devp) || ((Uns)ldevid >= PSP_HIST_NUM_INSTANCES))
    {
        statusBnd = (IOM_EBADARGS);
        DDAIOM_DEBUG_PRINTF("HIST_mdBindDev : Invalid parameter");
    }
    else
    {
        /*initialize histogram driver */
        histDeviceObject = (DDC_HISTdeviceobject*)PSP_HISTCreate((Uint32)devid);
        if (NULL == histDeviceObject)
        {
            statusBnd = (IOM_EBADARGS);
            DDAIOM_DEBUG_PRINTF(
                "HIST_mdBindDev : device object creation failed");
        }
        else
        {
            /*Update the Port object */
            hist_port.deviceObj         = histDeviceObject;
            hist_port.histPortNumber    = ldevid;
            hist_port.state             = DDC_HIST_CREATED;
            *devp                       = &hist_port;
        }
    }
    return(statusBnd);
}

/**
 * \brief   HIST_mdUnBindDev
 *          This function is called by DEV_init(). This function is called
 *          last and is used for driver deletion.
 */
Int HIST_mdUnBindDev(Ptr devp)
{
    Int status = IOM_COMPLETED;
    HISTPortObject *portUBD = NULL;

    /*Check the parameter whether its NULL */
    if (NULL == devp)
    {
        status = IOM_EBADARGS;
        DDAIOM_DEBUG_PRINTF("HIST_mdUnBindDev : Invalid parameter");
    }
    else
    {
        portUBD = (HISTPortObject *)devp;

        /* Check the State of Histogram device Port */
        if ((DDC_HIST_CREATED != portUBD->state)
            && (DDC_HIST_CLOSED  != portUBD->state))
        {
            status = IOM_EBADMODE;
            DDAIOM_DEBUG_PRINTF("HIST_mdUnBindDev : Invalid state");
        }
        /* Check the instance number with maximum limits */
        else if (portUBD->histPortNumber >= PSP_HIST_NUM_INSTANCES )
        {
            status = IOM_EBADARGS;
            DDAIOM_DEBUG_PRINTF("HIST_mdUnBindDev : Invalid device instance");
        }
        else
        {
            /*This function de-initialize and closes handle of a channel */
            status = PSP_HISTDelete(portUBD->histPortNumber);
            /* Check status for success */
            if (PSP_SOK != status)
            {
                /*Generate the IOM Error code based on PSP error */
                status = getIomErrorCode(status);
                DDAIOM_DEBUG_PRINTF(
                    "HIST_mdUnBindDev : Device deletion failed");
            }
            else
            {
                portUBD->state = DDC_HIST_DELETED;
            }
        }
    }
    return(status);
}

/**
 * \brief   HIST_mdCreateChan
 *          This function creates the channel of the driver instance for data
 *          transfers.
 */
/* ARGSUSED */
Int HIST_mdCreateChan(Ptr *chanp, Ptr devp, String name, Int mode,
    Ptr chanParams, IOM_TiomCallback cbFxn, Ptr cbArg)
{
    Int                         statusCR    = IOM_COMPLETED;
    HISTPortObject*             portCR      = NULL;
    HIST_iomchannelobj*         channelCR   = NULL;

    /* Check the Parameters whether they are NULL */
    if (((NULL == chanp) || (NULL == devp))
        || ( ((NULL == cbFxn) || (NULL == cbArg)) || (NULL == chanParams) ) )
    {
        statusCR = IOM_EBADARGS;
        DDAIOM_DEBUG_PRINTF("HIST_mdCreateChan : Invalid parameter");
    }
    else
    {
        portCR = (HISTPortObject *)devp;

        /* Check the State of the Port */
        if ((DDC_HIST_CREATED != portCR->state)
            && (DDC_HIST_CLOSED  != portCR->state))
        {
            statusCR =  (IOM_EBADMODE);
            DDAIOM_DEBUG_PRINTF("HIST_mdCreateChan : Invalid state");
        }
        else if (IOM_INPUT != mode)
        {
            statusCR = (IOM_EBADMODE);
            DDAIOM_DEBUG_PRINTF("HIST_mdCreateChan : Invalid mode");
        }
        else
        {
            channelCR = &(portCR->iomChannelObj);
        }
    }
    if (IOM_COMPLETED == statusCR)
    {
        if (NULL == channelCR)
        {
            statusCR = IOM_EBADARGS;
            DDAIOM_DEBUG_PRINTF("HIST_mdCreateChan : Invalid parameter");
        }
        else
        {
            /*Initialize and Creates the Channel Handle */
            channelCR->ddcHandle = (PSP_Handle)PSP_HISTOpen(chanParams);
            /* Check whether the Channel Handle is NULL */
            if (NULL == channelCR->ddcHandle)
            {
                statusCR = IOM_EBADARGS;
                DDAIOM_DEBUG_PRINTF("HIST_mdCreateChan : Invalid parameter");
            }
            else
            {
                /*Update the State as opened if Channel Handle not NULL */
                portCR->state = DDC_HIST_OPENED;
                channelCR->port = portCR;
                *chanp = channelCR;
            }
        }
    }
    return(statusCR);
}

/**
 * \brief   HIST_mdDeleteChan
 *          This function is used to close Channel handle for Histogram Module.
 */
Int HIST_mdDeleteChan(Ptr chanp)
{
    Int statusDC = IOM_COMPLETED;
    HIST_iomchannelobj* chanDC = NULL;
    HISTPortObject* portDC = NULL;

    /*Check Channel pointer whether it's NULL */
    if (NULL == chanp)
    {
        statusDC = IOM_EBADARGS;
        DDAIOM_DEBUG_PRINTF("HIST_mdDeleteChan : Invalid parameter");
    }
    else
    {
        chanDC = (HIST_iomchannelobj *)chanp;
        portDC = (HISTPortObject *)chanDC->port;

        /*Check the Channel State, port handle and Channel inuse */
        if ( (NULL == portDC) || (DDC_HIST_OPENED != portDC->state) )
        {
            statusDC = IOM_EBADARGS;
            DDAIOM_DEBUG_PRINTF("HIST_mdDeleteChan : Invalid parameter");
        }
    }/*else*/

    if (IOM_COMPLETED == statusDC)
    {
        /*Function closes the front end module Channel */
        statusDC = PSP_HISTClose(chanDC->ddcHandle);
        /*Check the Status if success */
        if (PSP_SOK != statusDC)
        {
            /*Generate the IOM error code equivalent to PSP error */
            statusDC = getIomErrorCode(statusDC);
            DDAIOM_DEBUG_PRINTF("HIST_mdDeleteChan : Channel deletion failed");
        }
        else
        {
            /* Updates the state and makes the channel port NULL */
            chanDC->port = NULL;
            portDC->state = DDC_HIST_CLOSED;
            chanp = &chanDC;
        }
    }/* IOM_COMPLETED == statusDC */

    return(statusDC);
}

/**
 * \brief   HIST_mdControlChan
 *          This function performs the ioctl operation.
 */
Int HIST_mdControlChan(Ptr chanp, Uns cmd, Ptr arg)
{
    Int statusCC = IOM_COMPLETED;
    HIST_iomchannelobj  *chanCC   = NULL;
    HISTPortObject     *portCC    = NULL;

    /*Check Channel Handle is NULL */
    if ((NULL == chanp) || (NULL == arg))
    {
        statusCC = IOM_EBADARGS;
        DDAIOM_DEBUG_PRINTF("HIST_mdControlChan : Invalid parameter");
    }
    else
    {
        chanCC = (HIST_iomchannelobj*)chanp;
        portCC = (HISTPortObject* )chanCC->port;

        /*Check the port and port State */
        if ((NULL == portCC) || (DDC_HIST_OPENED != portCC->state))
        {
            statusCC = IOM_EBADMODE;
            DDAIOM_DEBUG_PRINTF("HIST_mdControlChan : Invalid parameter");
        }
        else
        {
            /*DDA Layer function is called to change Dynamic Configuration
              of the Channels */
            statusCC = (Int)PSP_HISTIoctl(chanCC->ddcHandle,
                (PSP_HISTIoctlCmd)cmd, arg);
            /*Check the status if success */
            if (PSP_SOK != statusCC)
            {
                /*Generate the IOM Errorcode equivalent to PSP Error Code */
                statusCC = getIomErrorCode(statusCC);
                DDAIOM_DEBUG_PRINTF(
                    "HIST_mdControlChan : Control channel failed");
            }
        }/* else part of port Check */
    }/*else part of channel Handle check */
    return(statusCC);
}

/**
 * \brief   HIST_mdSubmitChan
 *          This function submits the IO request.
 *
 */
Int HIST_mdSubmitChan(Ptr chanp, IOM_Packet * const packet)
{
    Int statusSC = IOM_COMPLETED;
    HIST_iomchannelobj *chanSC = NULL;
    HISTPortObject *portSC = NULL;

    /*Check the formal arguments whether NULL */
    if ((NULL == chanp) || (NULL == packet))
    {
        statusSC = IOM_EBADARGS;
        DDAIOM_DEBUG_PRINTF("HIST_mdSubmitChan : Invalid parameter");
    }
    else
    {
        chanSC = (HIST_iomchannelobj*)chanp;

        /*Check the port and address if NULL */
        if ((NULL == packet->addr) || (NULL == chanSC->port))
        {
            statusSC = IOM_EBADARGS;
        }
        else
        {
            portSC = (HISTPortObject *)chanSC->port;
            if (NULL == portSC)
            {
                statusSC = IOM_EBADARGS;
                DDAIOM_DEBUG_PRINTF("HIST_mdSubmitChan : Invalid parameter");
            }
            else
            {
                /*Check the State of port */
                if (DDC_HIST_OPENED != portSC->state)
                {
                    statusSC = IOM_EBADMODE;
                    DDAIOM_DEBUG_PRINTF("HIST_mdSubmitChan : Invalid state");
                }
                else
                {
#ifdef HIST_INSTRUMENTATION_ENABLED
                    PSP_LOG_EVENT(  &DVTEvent_Log, "HISTOGRAM",
                        PSP_DVT_DESC(
                                PSP_DVT_eFUNC_START,
                                PSP_DVT_dCHANNEL_ID_INST,
                                PSP_DVT_dPACKET_ID,
                                PSP_DVT_dSIZE_BYTES),
                        CHANNELID_INSTANCEID(0,0),
                        packet->addr,
                        packet->size
                        );
#endif /* HIST_INSTRUMENTATION_ENABLED */

                    /*DDA Layer function is called to submit the data request */
                    statusSC = PSP_HISTSubmit(chanSC->ddcHandle,
                        (PSP_VPSSSubmitCommand)packet->cmd, packet->addr);
                    /*Check the status if succes */
                    if (PSP_SOK != statusSC)
                    {
                        /*Generate the IOM error code equivalent to PSP
                         *error code */
                        statusSC = getIomErrorCode(statusSC);
                        DDAIOM_DEBUG_PRINTF(
                            "HIST_mdSubmitChan : Submit channel failed");
                    }

#ifdef HIST_INSTRUMENTATION_ENABLED
                    PSP_LOG_EVENT(  &DVTEvent_Log, "HISTOGRAM",
                        PSP_DVT_DESC(
                                PSP_DVT_eFUNC_END,
                                PSP_DVT_dCHANNEL_ID_INST,
                                PSP_DVT_dPACKET_ID,
                                PSP_DVT_dSIZE_BYTES),
                            CHANNELID_INSTANCEID(0,0),
                            packet->addr,
                            0
                        );
#endif /* HIST_INSTRUMENTATION_ENABLED */

                }/*Check state of the port */
            }
        }
    }/*else */

    return statusSC;
}

