/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/
/**
 * \file   dda_histIOM.h
 *
 * \brief  Histogram IOM layer header file
 *
 * This file contains the interfaces, data types and symbolic definitions
 * that are needed by the application to utilize the serivces of the Histogram
 * device driver.
 *
 * (C) Copyright 2010, Texas Instruments, Inc
 *
 * \author     EI5
 *
 * \version    0.1             Created
 *             1.0             First Release
 */

#ifndef _DDA_HISTIOM_H
#define _DDA_HISTIOM_H

/******************************************************************************
  Header File Inclusion
 ******************************************************************************/
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/tistdtypes.h>
#include <ti/sdo/pspdrivers/drivers/histogram/src/ddc_hist.h>

#ifdef DEBUG_HIST
#include <ti/sdo/pspdrivers/pal_os/bios/psputils.h>
#endif /* DEBUG_HIST */

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
  Defines
 ******************************************************************************/
#define DDAIOM_HIST_PORT_INITVAL     0

#ifdef DEBUG_HIST
#define DDAIOM_DEBUG_PRINTF(STR) PSP_DEBUG1("DDAIOM : %s\n", STR)
#else
#define DDAIOM_DEBUG_PRINTF(STR)
#endif /* DEBUG_HIST */

/******************************************************************************
  Data-Structures
 ******************************************************************************/
/**
 * \brief  HIST_iomchannelobj
 *         This structure defines IOM Channel structure used by the IOM layer
 */
typedef struct HISTIOMCHANNELOBJ{
    /* handle to port */
    struct HISTPORTOBJ      *port;
    /* handle to channel object */
    PSP_Handle              ddcHandle;
} HIST_iomchannelobj;

/**
 * \brief   HISTPortObject
 *          This structure defines the IOM Port structure used by the IOM layer
 */
typedef struct HISTPORTOBJ{
    /* Device port number */
    Uns                         histPortNumber;
    /* channel for the Histogram */
    HIST_iomchannelobj          iomChannelObj;
    /* handle to device */
    DDC_HISTdeviceobject        *deviceObj;
    /* state of Histogram driver */
    DDC_HISTOGRAM_STATE         state;
} HISTPortObject;

/******************************************************************************
  Functions
 ******************************************************************************/
/**
 * \brief   HIST_mdBindDev
 *          This function is called at the time of device creation to bind
 *          and initialize the instance
 *
 * \param   devp        [IN]    Void pointer passed by the GIO Layer to
 *                              store the address of the port structure
 *
 * \param   devid       [IN]    device instance number
 *
 * \param   devParams   [IN]    Initialization configuration structure
 *
 * \return  On success returns IOM_COMPLETED
 *          On failure returns IOM Error code
 */
Int HIST_mdBindDev(Ptr *devp, Int devid, Ptr devParams);

/**
 * \brief   HIST_mdUnBindDev
 *          This function is called by DEV_init(). This function is called
 *          last and is used for driver deletion.
 *
 * \param   devp        [IN]    pointer in the GIO layer giving the address of
 *                              port object of that particular instance
 *
 * \Return  On success returns IOM_COMPLETED
 *          On failure returns IOM Error code
 */
Int HIST_mdUnBindDev(Ptr devp);

/**
 * \brief   HIST_mdCreateChan
 *          This function creates the channel of the driver instance for data
 *          transfers.
 *
 * \param   chanp        [IN/OUT]   pointer to the IOM layer channel structure
 *
 * \param   devp         [IN]       pointer to the port structure
 *
 * \param   name         [IN]       name of the driver
 *
 * \param   mode         [IN]       mode of the channel i.e. INPUT or OUTPUT
 *
 * \param   chanParams   [IN/OUT]   Extra parameter (Optional). In this module
 *                                  it points to structure containing EDMA
 *                                  related parameters.
 *
 * \param   cbFxn        [IN]       callback to the GIO Layer (NULL)
 *
 * \param   cbArg        [IN]       arguments for the callback (NULL)
 *
 * \Returns On success returns IOM_COMPLETED
 *          On failure returns IOM Error code
 */
Int HIST_mdCreateChan(Ptr *chanp, Ptr devp, String name, Int mode,
    Ptr chanParams, IOM_TiomCallback cbFxn, Ptr cbArg);

/**
 * \brief   HIST_mdDeleteChan
 *          This function is used to close Channel handle for Histogram Module.
 *
 * \param   chanp       [IN]    IOM layer Channel Handle
 *
 * \return  On success returns IOM_COMPLETED
 *          On failure returns IOM Error code
 */
Int HIST_mdDeleteChan(Ptr chanp);

/**
 * \brief   HIST_mdControlChan
 *          This function performs the ioctl operation.
 *
 * \param   chanp       [IN]        IOM layer Channel Handle
 *
 * \param   cmd         [IN]        IOCTL command
 *
 * \param   arg         [IN/OUT]    Command argument
 *
 * \return  On success returns IOM_COMPLETED
 *          On failure returns IOM Error code
 */
Int HIST_mdControlChan(Ptr chanp, Uns cmd, Ptr arg);

/**
 * \brief  HIST_mdSubmitChan
 *         This function submits the IO request.
 *
 * \param  chanp       [IN]    IOM layer Channel Handle
 *
 * \param  packet      [IN]    IOM packet passed by the upper layer containing
 *                             information about the size, address, timeout and
 *                             other necessary parameters
 *
 * \return On success returns IOM_COMPLETED/IOM_PENDING
 *         On failure returns IOM Error code
 */
Int  HIST_mdSubmitChan(Ptr chanp, IOM_Packet *packet);

#ifdef __cplusplus
}
#endif

#endif /*_DDA_HISTIOM_H */

