/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/
/**
 * \file   ddc_hist.h
 *
 * \brief  DDC layer header file for Histogram
 *
 * This file contains the interfaces, data types and symbolic definitions
 * that are needed by the application to utilize the serivces of the Histogram
 * device driver. In addition this file provides an interface to DSP-BIOS APIs.
 *
 * (C) Copyright 2010, Texas Instruments, Inc
 *
 * \author     EI5
 *
 * \version    0.1             Created
 *             1.0             First Release
 */

#ifndef _DDC_HIST_H
#define _DDC_HIST_H

/******************************************************************************
  Header File Inclusion
 ******************************************************************************/
#include <ti/sdo/pspdrivers/drivers/histogram/psp_histogram.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osSem.h>
#include <ti/sdo/edma3/rm/edma3_rm.h>

#ifdef DEBUG_HIST
#include <ti/sdo/pspdrivers/pal_os/bios/psputils.h>
#endif /* DEBUG_HIST */

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
  Defines
 ******************************************************************************/
/*Dequeue bloking sem timeout*/
#define DDC_HIST_ISRSEM_TMOUT           2000
/*Maximum output buffer size */
#define DDC_HIST_MAX_OP_BUFF_SIZE       1024u
/*Minimum Enqueue element at run time.*/
#define DDC_HIST_MIN_QUEUE_ELEMENT      2u
/*Minimum Enqueue element at the time of Enqueue Node creation.*/
#define DDC_HIST_MIN_ENQUEUE_ELEMENT    1u
/*Minimum Dequeue element at the time of Dequeue Node creation.*/
#define DDC_HIST_MIN_DEQUEUE_ELEMENT    1u
#define DDC_HIST_QUEUE_IDX_INITVAL      0
#define DDC_HIST_HWISTATUS_INITVAL      0
#define DDC_HIST_SIZEOF_HIST_DATA       4u
#define DDC_HIST_BUSY                   1
#define DDC_HIST_INSTANCE_ID            0
#define DDC_HIST_WBGAIN_INITVAL         0
#define DDC_HIST_REG_BNDRY_INITVAL      0
#define DDC_HIST_256BINS_MAXREGION      1
#define DDC_HIST_128BINS_MAXREGION      2
#define DDC_HIST_64BINS_MAXREGION       4
#define DDC_HIST_EDMA_CHNL_INITVAL      0
#define DDC_HIST_EDMA_TCC_INITVAL       0
#define DDC_HIST_EDMA_A_CNT             4u
#define DDC_HIST_EDMA_C_CNT             1u
#define DDC_HIST_EDMA_REGIONID          1u
#define DDC_HIST_EDMA_DEST_BINDEX       4
#define DDC_HIST_EDMA_DEST_CINDEX       4
#define DDC_HIST_EDMA_SRC_BINDEX        0u
#define DDC_HIST_EDMA_SRC_CINDEX        0u
#define DDC_HIST_EDMA_INSTANCEID        0u
#define DDC_HIST_EDMA_B_CNT_RLD         0u
#define DDC_HIST_EDMA_EVNT_QUEUE        0u
#define DDC_HIST_REGPARAM_INITVAL       0
#define DDC_HIST_CHNLSEM_COUNT          1
#define DDC_HIST_ISRSEM_COUNT           0

#ifdef DEBUG_HIST
#define DDC_DEBUG_PRINTF(STR) PSP_DEBUG1("DDC : %s\n", STR)
#else
#define DDC_DEBUG_PRINTF(STR)
#endif /* DEBUG_HIST */

/******************************************************************************
  Enumerations
 ******************************************************************************/
/**
 * \brief   This enum indicates the states of the channel parameter
 *          configuration.
 */
typedef enum _DDC_HISTParams_config_status
{
    DDC_STATE_CONFIGURED = 0,
    DDC_STATE_NOT_CONFIGURED
} DDC_HISTParams_config_status;

/**
 * \brief   This enum contains DDC related error codes that will be passed
 *          to the DDA layer.
 */
typedef enum _DDC_HISTstatus
{
    DDC_HIST_SOK = 0,
    DDC_HIST_E_FAIL,
    DDC_HIST_INVALID_STATE,
    DDC_HIST_INVALID_PARAM,
    DDC_HIST_NOT_SUPPORTED,
    DDC_HIST_NO_DATA
} DDC_HISTstatus;

/**
 * \brief   This enum indicates the states of the channel.
 *
 */
typedef enum _DDC_HISTchannelstate
{
    DDC_HIST_CHAN_CLOSED = 0,
    DDC_HIST_CHAN_OPENED
} DDC_HISTchannelstate;

/**
 * \brief   This enum describes the various states of Histogram driver goes
 *          through the entire cycle.
 *
 */
typedef enum _DDC_HISTOGRAM_STATE{
    DDC_HIST_DELETED = 0,
    DDC_HIST_CREATED,
    DDC_HIST_OPENED,
    DDC_HIST_CLOSED
} DDC_HISTOGRAM_STATE;

/******************************************************************************
  Data-Structures
 ******************************************************************************/
/**
 * \brief   DDC_HISTbufferobject
 *          Stucture for the Histogram enqueue/dequeue buffer management
 *
 */
typedef struct _DDC_HISTbufferobject
{
    /* Index indicating the number of buffer pushed on the queue */
    Uint32     indexEnQueue;
    /* Index indicating the number of buffers containing statistics */
    Uint32     indexDeQueue;
    /* enqueue head pointer */
    PAL_OsListNodeHeader    enQueue;
    /* dequeue head pointer */
    PAL_OsListNodeHeader    deQueue;
} DDC_HISTbufferobject;

/**
 * \brief   DDC_HISTchannelobject
 *          Stucture for the Histogram channel object
 *
 */
typedef struct _DDC_HISTchannelobject
{
    /* indicates the status of channel */
    DDC_HISTchannelstate             stateCH;
    /* instanse of parameter structure */
    PSP_HISTparam                    regParam;
    /* indicates the status of parameter configuration */
    DDC_HISTParams_config_status     paramConfigState;
    /* bloking semaphore for channel */
    DDC_HISTbufferobject             bufferObj;
    /* bloking semaphore for channel */
    PAL_OsSemHandle                  channelSem;
    /* EDMA  Handler */
    PSP_Handle                       edmaHandle;
    /* EDMA  tcc */
    Uint32                           tcc ;
    /* bloking semaphore for dequeue call */
    PAL_OsSemHandle                  semIsr;
    /* Pending on semaphore timeout */
    Int32                            semTimeOut;
} DDC_HISTchannelobject;

/**
 * \brief   DDC_HISTdeviceobject
 *          Stucture for the Histogram device object
 *
 */
typedef struct _DDC_HISTdeviceobject
{
    /* histogram device instance id */
    Uint32                  histInstanceId;
    /* Histogram device state */
    DDC_HISTOGRAM_STATE     stateDVC;
    /*  Histogram EDMA event number */
    Uint32                  edmaChannelNo;
    /*  Handle for Histogram overlaying register */
    Ptr                     regs;
    /*  Handle for Histogram channel object */
    DDC_HISTchannelobject   histChannelObj;
} DDC_HISTdeviceobject;

/******************************************************************************
  FUNCTIONS
 ******************************************************************************/
/**
 * \brief   DDC_HISTOpenHandle
 *          This function opens the channel handle, initializes the Enqueue and
 *          Dequeue & creates the semaphore. It also creates & opens the edma
 *          handle if it is not provided by the application and request the
 *          edma channel.
 *
 * \param   edmaHandleDDC   [IN]   EDMA Handle at DDC layer
 *
 * \returns Handle to the channel
 */
PSP_Handle DDC_HISTOpenHandle(PSP_Handle edmaHandleDDC);

/**
 * \brief   DDC_HISTCloseHandle
 *          This function closes the channel handle & deletes the semaphore.
 *          It also delete & close the edma handle if it is created & opened
 *          by the driver and make the edma channel free.
 *
 * \param   handle      [IN]    Handle to the channel
 *
 * \returns DDC_HIST_SOK if success, else error code
 */
DDC_HISTstatus DDC_HISTCloseHandle(PSP_Handle handle);

/**
 * \brief   DDC_HISTIsChannelHandleClosed
 *          This function checks weather the channel handle is closed or not.
 *
 * \param   handle      [IN]    handle to channel
 *
 * \returns DDC_HIST_SOK if success, else error code
 */
DDC_HISTstatus DDC_HISTIsChannelHandleClosed(PSP_Handle handle);

/**
 * \brief   DDC_HISTGetDeviceHandle
 *          Get the histogram device handle.
 *
 * \returns Histogram device handle
 */
PSP_Handle DDC_HISTGetDeviceHandle();

/**
 * \brief   DDC_HISTGetChannelHandle
 *          Get the histogram channel handle.
 *
 * \returns Histogram channel handle
 */
PSP_Handle DDC_HISTGetChannelHandle();

/**
 * \brief   DDC_HISTEdmaisr
 *          EDMA Interrupt service routine, it extracts the buffer containing
 *          statistics from the Enqueue queue & put that Into the Dequeuue
 *          queue. It also configures the EDMA parameters based on the next
 *          empty buffer. If no any empty buffer is there then it disables the
 *          Histogram module.
 *
 * \param   tcc       [IN]  EDMA Channel NO which has generated the interrupt
 *
 * \param   status    [IN]  Status of data Transfer
 *
 * \param   data      [IN]  Handle to device channel
 */
void DDC_HISTEdmaisr(Uint32 tcc, EDMA3_RM_TccStatus status, PSP_Handle data);

/**
 * \brief   DDC_HISTPerformRegOverlaying
 *          Gets the register base address.
 *
 * \returns DDC_HIST_SOK if success, else error code
 */
DDC_HISTstatus DDC_HISTPerformRegOverlaying();

/**
 * \brief   DDC_HISTSetDeviceEventNumber
 *          Get the EDMA channel number.
 *
 * \returns DDC_HIST_SOK if success, else error code
 */
DDC_HISTstatus DDC_HISTSetDeviceEventNumber();

/**
 * \brief   DDC_HISTValidateParameters
 *          This function Validates the configuration parameters provided by
 *          application. If all parameters are valid & if Histogram module is
 *          not busy then it calls LLC_HISTWritereg() function to write
 *          registers of Histogram module.
 *
 * \param   histConfChan    [IN]    Pointer to channel object
 *
 * \param   params          [IN]    Pointer to parameter structure
 *
 * \returns DDC_HIST_SOK if success, else error code
 */
DDC_HISTstatus DDC_HISTValidateParameters(DDC_HISTchannelobject *histConfChan,
    PSP_HISTparam  *params);

/**
 * \brief   DDC_HISTGetParameters
 *          This function provides the configuration parameters to application.
 *
 * \param   histConfChan    [IN]    Pointer to channel object
 *
 * \param   params          [IN]    Pointer to params structure passed by
 *                                  application
 *
 * \returns DDC_HIST_SOK if success, else error code
 */
DDC_HISTstatus DDC_HISTGetParameters(DDC_HISTchannelobject *histConfChan,
    PSP_HISTparam  *params);

/**
 * \brief  DDC_HISTIoctl
 *         This function performs the control operation depending on the ioctl.
 *
 * \param  handle          [IN]    handle to channel object
 *
 * \param  cmd             [IN]    ioctl
 *
 * \param  cmdArg          [IN]    ioctl argument
 *
 * \return error code for success/error.
 */
DDC_HISTstatus  DDC_HISTIoctl(PSP_Handle handle, PSP_HISTIoctlCmd cmd,
    Ptr cmdArg);

/**
 * \brief   DDC_HISTRemoveModule
 *          Removes the device.
 *
 */
PSP_Handle DDC_HISTRemoveModule();

/**
 * \brief   DDC_HISTInitializeModule
 *          Creates the device handle.
 *
 */
PSP_Handle DDC_HISTInitializeModule(Uint32);

/**
 * \brief   DDC_HISTEnquebuffer
 *          This function queues the buffer provided by application Into the
 *          Enqueue and if it is the first buffer it will configure the EDMA
 *          parameters & Enables the Histogram module.
 *
 * \param   chnlHandle          [IN]    handle to channel
 *
 * \param   cmdArg              [IN]    pointer to buffer structure
 *
 * \returns DDC_HIST_SOK if success, else error code
 */
DDC_HISTstatus  DDC_HISTEnquebuffer(PSP_Handle chnlHandle, Ptr cmdArg);

/**
 * \brief   DDC_HISTDequebuffer
 *          This function extract buffer containing statistics from the Dequeue
 *          and provide it to the application.
 *
 * \param   chnlHandle          [IN]    handle to channel
 *
 * \param   cmdArg              [OUT]   pointer to buffer structure
 *
 * \returns DDC_HIST_SOK if success, else error code
 */
DDC_HISTstatus DDC_HISTDequebuffer(PSP_Handle chnlHandle, Ptr cmdArg);

/**
 * \brief   DDC_HISTSubmit
 *          This function calls the function for Enqueue/Dequeue operation
 *          based on the command.
 *
 * \param   handle          [IN]    handle to channel object
 *
 * \param   cmd             [IN]    command to enqueue/dequeue
 *
 * \param   cmdArg          [IN]    command argument
 *
 * \return  DDC_HIST_SOK if success, else error code
 */
DDC_HISTstatus  DDC_HISTSubmit(PSP_Handle chnlHandle, PSP_VPSSSubmitCommand cmd,
    Ptr cmdArg);

#ifdef __cplusplus
}
#endif

#endif /*_DDC_HIST_H */
