/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/
/*!
    @{

*/

/*!
 *  \file   psp_i2c.h
 *
 *  \brief  I2C interface definition
 *
 *  This file contains the interfaces, data types and symbolic definitions
 *  that are needed by the application to utilize the services of the I2C
 *  device driver.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author     PSG
 *
 *  \version    0.2
 *
 *  <b>Compiler Switch Used</b>
 *
 *  <b>_DEBUG</b><br>
 *   This is used as a flag to compiler whether to include the debug statement
 *   inserted in the code into the final image. This flag helps to build DEBUG
 *   image of the program. For RELEASE images this is not pased to the compiler.
 *
 *  <b>CHIP_XXXX</b><br>
 *   The CSL layer is written in a common file for all the variants of a SOC.
 *   This flag differentiates the variant we are compiling for, for e.g.
 *   CHIP_DM648, and the CSL definitions for that variant appropriately gets
 *   defined for register base addresses, num of ports of a peripheral etc.
 *
 *  <b>I2C_INSTRUMENTATION_ENABLED</b><br>
 *   This flag is passed to the compiler to include the instrumentation code parts
 *   into the final image/lib of the program. This helps build the
 *   iRelease/iDebug
 *   versions of the image/lib with a common code base.
 *
 *  <b>I2C_DEBUG_PRINTF</b><br>
 *   They are the macros used to print debug messages. These expand a
 *   valid print statement.
 */

#ifndef _PSP_I2C_H_
#define _PSP_I2C_H_

/* Header file included */
#include <ti/sdo/pspdrivers/common/psp_common.h>

#ifdef CHIP_DM6437
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/tistdtypes.h>
#endif

#ifdef CHIP_C6424
#include <ti/sdo/pspdrivers/soc/c6424/dsp/tistdtypes.h>
#endif

#ifdef CHIP_DM648
#include <ti/sdo/pspdrivers/soc/dm648/dsp/tistdtypes.h>
#endif

#ifdef CHIP_C6452
#include <ti/sdo/pspdrivers/soc/c6452/dsp/tistdtypes.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

/*!
 * \brief  I2C Driver Error codes
 */
/** Error Code base                                             */
#define PSP_I2C_ERROR_BASE              (-11)
/** Returned when the I2C bus find that the bus is buzy         */
#define PSP_I2C_BUS_BUSY_ERR            (PSP_I2C_ERROR_BASE - 1)
/** Returned when the I2C driver lost the bus arbitration       */
#define PSP_I2C_ARBITRATION_LOSS_ERR    (PSP_I2C_ERROR_BASE - 2)
/** Returned when the I2C slave did not generate an acknowledge */
#define PSP_I2C_NACK_ERR                (PSP_I2C_ERROR_BASE - 3)
/** Returned in case of an transmit underflow error             */
#define PSP_I2C_TRANSMIT_UNDERFLOW_ERR  (PSP_I2C_ERROR_BASE - 4)
/** Returned in case of an receive overflow error               */
#define PSP_I2C_RECEIVE_OVERFLOW_ERR    (PSP_I2C_ERROR_BASE - 5)
/** Returned in case of an Cancel io condition                  */
#define PSP_I2C_CANCEL_IO_ERROR         (PSP_I2C_ERROR_BASE - 6)

/** Max I2C driver instances                                    */
#define PSP_I2C_NUM_INSTANCES               (1u)

/** Max I2C driver opens allowed                                */
#define PSP_I2C_NUM_OPENS                   (5u)


#define PSP_I2C_MAX_REQ       		        (30)


/* Note: The following flags offer the user maximum flexibility in terms
 * of making the right I2C transaction. In case the user does not want
 * to set the flags on his own, the default read/write flag can be specified
 *
 * NOTE: If no flag is specified by default in the flags, read is assumed
 */
/** Read from I2C bus (device)                          */
#define PSP_I2C_READ                (0x1u)
/** Write to I2C bus (device)                           */
#define PSP_I2C_WRITE               (0x2u)
/** If this flag is not set, default is slave mode      */
#define PSP_I2C_MASTER              (0x20u)
/** Generate Start - valid in master mode only          */
#define PSP_I2C_START               (0x100u)
/** Generate Stop - valid in master mode only           */
#define PSP_I2C_STOP                (0x200u)
/** Re-Start is generated by Master                     */
#define PSP_I2C_RESTART             (0x400u)
/** Repeat mode as per TI I2C specs                     */
#define PSP_I2C_REPEAT              (0x2000u)
/** Ignore NAK from device                              */
#define PSP_I2C_IGNORE_NAK          (0x10000u)
/** Ignore Bus Busy condition                           */
#define PSP_I2C_IGNORE_BUS_BUSY     (0x20000u)

/** Default read flag                                   */
#define PSP_I2C_DEFAULT_READ \
                (PSP_I2C_READ | PSP_I2C_MASTER | PSP_I2C_START | PSP_I2C_STOP)
/** Default write flag                                  */
#define PSP_I2C_DEFAULT_WRITE \
                (PSP_I2C_WRITE | PSP_I2C_MASTER | PSP_I2C_START | PSP_I2C_STOP)

/*!
 *  \brief  PSP I2C Ioctl commands
 *
 *  I2C Ioctl commands
 */
typedef enum
{
    PSP_I2C_IOCTL_SET_BIT_RATE,           /**< Set the I2C clock              */
    PSP_I2C_IOCTL_GET_BIT_RATE,           /**< Get the I2C clock              */
    PSP_I2C_IOCTL_CANCEL_PENDING_IO,      /**< To cancel pending IO           */
    PSP_I2C_IOCTL_BIT_COUNT,              /**< To set bit Count value         */
    PSP_I2C_IOCTL_NACK,                   /**< To enable or disable NACK
                                               dynamically                    */
    PSP_I2C_IOCTL_SET_OWN_ADDR,           /**< To change own address
                                               dynamically                    */
    PSP_I2C_IOCTL_GET_OWN_ADDR            /**< To Get own address
                                               dynamically                    */
} PSP_I2cIoctlCmd;

/*!
 *  \brief I2C PSP Configuration Object
 *
 *  This structure basically holds the mode of operation and passes it on
 *  to the driver.
 *
 */
typedef struct PSP_I2cConfig_t
{
    PSP_OpMode  mode;               /**< Driver operating mode - polled or
                                         interrupt                            */
    Uint32      i2cOwnAddr;         /**< Own address (7 or 10 bit)            */
    Uint32      numBits;            /**< Number of bits to be sent/received   */
    Uint32      i2cBusFreq;         /**< I2C Bus Frequency                    */
    Uint32      moduleInputClkFreq; /**< Module input clock freq              */
    Bool        addressing;         /**< 7bit/10bit Addressing mode           */
    Bool        dlb;                /**< Digital Loop Back (DLB) mode         */
} PSP_I2cConfig;

/*!
 *  \brief  I2C Transaction structure
 *
 *  This structure holds the information needed to carry out the transaction
 *  on an I2C bus.
 *
 */
typedef struct PSP_I2cTransaction_t
{
    Uint32              slaveAddr;
    /**< Address of the slave to talk to, not valid in Slave mode           */
    Uint8               *buffer;
    /**< Data Buffer                                                        */
    Uint32              bufLen;
    /**< Length of buffer                                                   */
    Uint32              flags;
    /**< Flags to indicate the various modes of operation                   */
    Ptr                 param;
    /**< Extra parameter for future use                                     */
} PSP_I2cTransaction;

/*!
 *  \brief   data packet element structure
 *
 *  Structure for the i2c specific buffer address to be passed to the GIO.
 */
typedef struct
{
    PSP_I2cTransaction  i2cTrans;   /**< i2c transaction structure       */
    Int                 timeout;    /**< Timeout value                   */

}PSP_i2cDataParam;



#ifdef __cplusplus
}
#endif

#endif  /* _PSP_I2C_H_ */



/*!
 * Revision History
 * ================
 *  \version    0.1 - previous versions of interfaces
 *  \version    0.2 - cleanup effort and review comments
 */

