/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file   dda_i2cBios.c
 *
 *  \brief  i2c Driver Adaptation for BIOS
 *
 *  This file contains adaptation of i2c driver for BIOS
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author   PSG
 *
 *  \version    0.6
 */

/* Header files inclusion */
#include <std.h>
#include <iom.h>
#include <string.h>
#include <ecm.h>
#include <C64.h>
#include <ti/sdo/pspdrivers/pal_os/pal_oswait.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osprotect.h>
#include <ti/sdo/pspdrivers/drivers/i2c/psp_i2c.h>
#include <ti/sdo/pspdrivers/drivers/i2c/src/dda_i2cBios.h>

#ifdef CHIP_DM6437
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/soc.h>
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/tistdtypes.h>
#endif

#ifdef CHIP_C6424
#include <ti/sdo/pspdrivers/soc/c6424/dsp/soc.h>
#include <ti/sdo/pspdrivers/soc/c6424/dsp/tistdtypes.h>
#endif

#ifdef CHIP_DM648
#include <ti/sdo/pspdrivers/soc/dm648/dsp/soc.h>
#include <ti/sdo/pspdrivers/soc/dm648/dsp/tistdtypes.h>
#endif

#ifdef CHIP_C6452
#include <ti/sdo/pspdrivers/soc/c6452/dsp/soc.h>
#include <ti/sdo/pspdrivers/soc/c6452/dsp/tistdtypes.h>
#endif

#ifdef I2C_INSTRUMENTATION_ENABLED
    #include <ti/sdo/pspdrivers/pal_os/bios/psp_log.h>
#endif /*I2C_INSTRUMENTATION_ENABLED*/

/** Hardware interrupt number */
#if defined(CHIP_DM648) || defined (CHIP_C6452)
#define HW_INTR_NUM     HWI_EVT01_CMB_IRQ
#endif

#if defined(CHIP_DM6437) || defined (CHIP_C6424)
#define HW_INTR_NUM     HWI_EVT03_CMB_IRQ
#endif
/*
 * This is the globally declared port object which contains all the information
 * for the parameters of the driver.
 */
I2cPortObj ports[PSP_I2C_NUM_INSTANCES] = {0,
                                            {  {0,NULL,NULL,NULL,NULL},
                                                {0,NULL,NULL,NULL,NULL},
                                                {0,NULL,NULL,NULL,NULL},
                                                {0,NULL,NULL,NULL,NULL},
                                                {0,NULL,NULL,NULL,NULL}
                                            },
                                           I2C_DELETED};


/*----------------------------Static Fucntions------------------------------*/
static Int i2c_mdBindDev(Ptr *devp, Int devid, Ptr devParams);

static Int i2c_mdControlChan(Ptr chanp, Uns cmd, Ptr arg);

static Int i2c_mdCreateChan(
                            Ptr *chanp,
                            Ptr devp,
                            String name,
                            Int mode,
                            Ptr chanParams,
                            IOM_TiomCallback cbFxn,
                            Ptr cbArg
                            );

static Int i2c_mdDeleteChan(Ptr chanp);

static Int i2c_mdSubmitChan(Ptr chanp, IOM_Packet *packet);

static Int i2c_mdUnBindDev(Ptr devp);

static Int32 i2c_getIomErrorCode(PSP_Result pspErrCode);

static void mdLayerCallback(Ptr Port,Ptr arg, const PSP_I2cRequest *ioRequest);


/**
 * \brief Public Mini Driver interface table.
 */
IOM_Fxns I2CMD_FXNS =
{
    &i2c_mdBindDev,
    &i2c_mdUnBindDev,
    &i2c_mdControlChan,
    &i2c_mdCreateChan,
    &i2c_mdDeleteChan,
    &i2c_mdSubmitChan,
};


/*!
 *  ======== _DDA_i2cRegisterIntrHandler ========
 *  \brief  Register interrrupt with BIOS and enable it
 *
 *  \param  instNum     Instanse number
 *  \param  intNum      Interrupt number
 *  \param  isrFunc     Function Pointer to ISR function
 *  \param  i2cObj      Handle to be passed to ISR function
 *
 *  \return void
 */
/* ARGSUSED */
void DDA_i2cRegisterIntrHandler(
                                Uint32      instNum,
                                Uint32      intNum,
                                pspI2cIsr   isrFunc,
                                PSP_Handle  i2cObj
                               )
{

    ECM_Attrs ecmattrs = ECM_ATTRS;

    if ((i2cObj == NULL) && (isrFunc == NULL))
    {
        return;
    }
    else
    {
        /* Depending upon the device id of the i2c */
        switch(instNum)
        {
            case 0:
                /* Mapping the eventid to the ECM Dispatch of the DSP/BIOS */
                ecmattrs.unmask = 1u;
                /*setup arg such that isr will get 'chan' as arg*/
                ecmattrs.arg    = (Arg)i2cObj;

                ECM_dispatchPlug(
                                 intNum,
                                 (ECM_Fxn)isrFunc,
                                 &ecmattrs
                                 );
                /* Enabling the event */
                ECM_enableEvent(intNum);
                break;

            default:
                break;
        }

        /* Enabling the HWI_ID */
        C64_enableIER(HW_INTR_NUM);
    }
}


/*!
 *  ======== _DDA_i2cUnregisterIntrHandler ========
 *  \brief  Un-Register interrrupt with BIOS and disable it
 *
 *  \param  instNum      Instanse number
 *  \param  intNum       Interrupt number
 *
 *  \return void
 */
/* ARGSUSED */
void DDA_i2cUnregisterIntrHandler(Uint32 instNum, Uint32 intNum)
{
    ECM_disableEvent(intNum);
}


/*!
 *  \brief Initialize a given I2C driver (instance)
 *
 *  Initialialization parameters of driver instance is passed in the call.
 *  This function prepares the device hardware for data transfer and used
 *  by the upper layer driver software.
 *
 *  \param  devp      [IN]      Void pointer passed by the GIO Layer to
 *                              store the address of the port structure
 *  \param  devid     [IN]      I2C instance number
 *  \param  devParams [IN]      Initialization configuration structure
 *  \return IOM_COMPLETED or IOM Error code
 */
static Int i2c_mdBindDev(Ptr *devp, Int devid, Ptr devParams)
{
    Int             status      = IOM_COMPLETED;
    PSP_I2cConfig   *params     = NULL;
    Uint32          numChans    = 0;
    Uint32          numInstance = 0;

    /* Check the Parameters */
    if( ( (NULL == devp) || (NULL == devParams) ) ||
                    (devid >= PSP_I2C_NUM_INSTANCES))
    {
        status =  (IOM_EBADARGS);
    }
    else
    {
        params = (PSP_I2cConfig *)devParams;
        /* Create an instance of i2c */
        status = PSP_i2cCreate((Uint32)devid,params);
        /* Checks for the success of operation  */
        if(status != PAL_SOK)
        {
            status = i2c_getIomErrorCode(status);
        }
        else
        {
            /** mAKE I2C DDC port handle NULL */
            for (numInstance=0;numInstance<PSP_I2C_NUM_INSTANCES;numInstance++)
            {
                for (numChans=0;numChans<PSP_I2C_NUM_OPENS;numChans++)
                {
                    ports[numInstance].chans[numChans].port = NULL;
                }
            }

            /* Update port object */
            ports[devid].i2cPortNumber  = devid;
            /* Update State       */
            ports[devid].i2cState       = I2C_CREATED;
            /* Assigning the port object to the global data structure   */
            *devp = &ports[devid];
        }/* if Create fails */
    }
    return status;
}


/*!
 *  \brief Open Channel of the I2C device
 *
 *  The function prepares channel of the driver instance for data transfers
 *  and returns an handle to the driver instance.
 *
 *  \param chanp     [IN]       pointer to the channel structure after
 *                              populating
 *  \param devp      [IN/OUT]   pointer to the port structure
 *  \param name      [IN]       name of the driver
 *  \param mode      [IN]       mode of the channel i.e. INPUT or OUTPUT
 *  \param chanParams [IN/OUT]  Extra parameter (not used in this driver)
 *  \param cbFxn     [IN]       callback to the GIO Layer
 *  \param cbArg     [IN]       arguments for the callback
 *  \return IOM_COMPLETED or IOM Error Code
 */
/* ARGSUSED */
static Int i2c_mdCreateChan(
                            Ptr *chanp,
                            Ptr devp,
                            String name,
                            Int mode,
                            Ptr chanParams,
                            IOM_TiomCallback cbFxn,
                            Ptr cbArg
                           )

{
    Int             status     = IOM_COMPLETED;
    I2cPortObj     *port       = NULL;
    I2cChanObj     *chan       = NULL;
    Int             chanCount  = 0;
    /* Check if Input Parameters is NULL */

    if(((NULL == devp) || (NULL == chanp))
                 || ((NULL == cbFxn) || (NULL == cbArg)))
    {
        status = IOM_EBADARGS;
    }
    else
    {
        port = (I2cPortHandle)devp;
        /*Check the State of I2C Instance */
        if (((port->i2cState != I2C_CREATED) && (port->i2cState != I2C_CLOSED))
                         && (port->i2cState != I2C_OPENED))
        {
            status =  (IOM_EBADMODE);
        }
        else
        {
            for(chanCount = 0 ; chanCount < IOM_I2C_NUMCHANS ; chanCount++)
            {
                if(port->chans[chanCount].port == NULL)
                {
                    /* Assignment of channel*/
                    chan = &port->chans[chanCount];
                    break;
                }
            }
            /* Check if channel is NULL */
            if (NULL == chan)
            {
                status = IOM_EBADARGS;
            }
            else
            {
                /* Check the Mode */
                if ( ((mode != IOM_INPUT) && (mode != IOM_OUTPUT) )
                             && (mode != IOM_INOUT) )
                {
                    status = IOM_EBADMODE;
                }
                else
                {
                    /* Assign the Mode */
                    chan->mode = mode;
                }
            }/* check if chan is NULL */
        }/* Check the State*/
    }/*Check if Input Parameter is NULL */

    if(IOM_COMPLETED == status)
    {
         /* Function will create a channel for an instance of i2c   */
        chan->ddc_Handle = PSP_i2cOpen(port->i2cPortNumber,
                                    (PSP_I2cAppCallback)&mdLayerCallback,port);
        /* Check if Channel Handle is NULL */
        if(NULL == chan->ddc_Handle )
        {
            status = IOM_EBADARGS;
        }
        else
        {
            /* Updating port and chan object */
            port->i2cState = I2C_OPENED;
            chan->cbFxn = cbFxn;
            chan->cbArg = cbArg;
            /* sets for back referencing */
            chan->port = (struct I2cPortObj_t *)port;
            *chanp = chan;

        }/* Check if Chan Handle is NULL */

    }/* status */
    return status;
}


/*!
 *  \fn static void mdLayerCallback(Ptr Port, Ptr arg,
                                const PSP_I2cRequest *ioRequest)
 *
 *  \brief DDC layer callback
 *
 * This callback is made from the DDC layer. This function restores appropriate
 *  data into the IOP and  calls GIO callback
 *
 *  \param  [IN]  Port          Port handle
 *  \param  [IN]  arg           optional device-defined data structure
 *  \param  [IN]  ioRequest     pointer to io request
 *  \param  [IN]  mode          channel  mode -> input or output
 *
 *  \return  Void
 */
static void mdLayerCallback(Ptr Port,Ptr arg, const PSP_I2cRequest *ioRequest)
{
    I2cChanObj      *chan   = NULL;
    IOM_Packet  *packet     = NULL;
    I2cPortObj  *port       = (I2cPortObj *)Port;


    if (( NULL != port ) && ( NULL != ioRequest ) )
    {
        arg = arg;  /* To remove warning */
        packet = (IOM_Packet *)ioRequest->appPtr;
        /* Channel ptr of the packet */
        chan = ioRequest->chanPtr;
        if( NULL != packet)
        {
            packet->size    = ioRequest->actualSize;
            packet->status  = ioRequest->status;
            packet->status = i2c_getIomErrorCode(packet->status);

            /* Check if channel is NULL */
            if (NULL != chan)
            {
                if(NULL != chan->cbFxn)
                {
                    (*chan->cbFxn)(chan->cbArg,packet);

                }
            }
        }
    }
}

/*!
 *  \brief submit IO Request
 *
 *  This function performs transfer operations on the I2C Instance.
 *
 *  \param  chanp  [IN]     I2C Driver Channel Handle
 *  \param  packet [IN]     IOM packet passed by the upper layer containing
 *                          information about the size,address,timeout and
 *                          other necessary parameters
 *  \return If successful, returns Success
 *          If failed, returns  IOM Error code or I2C error code
 */


static Int i2c_mdSubmitChan(Ptr chanp, IOM_Packet *packet)
{
    Int                 status    = IOM_COMPLETED;
    I2cChanObj         *chan      = NULL;
    I2cPortObj         *port      = NULL;
    PSP_I2cRequest      ioRequest;

    /* Check if the Input Parameters is NULL */
    if( (NULL == chanp )||(NULL == packet ) )
    {
        status = IOM_EBADARGS;
    }
    else
    {
        chan = (I2cChanHandle)chanp;
        /* Validate the packet send by the user */
        if ((NULL == packet->addr ) || (NULL == chan->port))
        {
            status = IOM_EBADARGS;
        }
        else
        {
            port = (I2cPortObj* )chan->port;
            /* validate channel and mode */
            if(chan->port == NULL)
            {
                status = IOM_EBADMODE;
            }
            else
            {    /* Check the State of I2C Instance */
                if(port->i2cState != I2C_OPENED)
                {
                    status = IOM_EBADMODE;
                }
                else
                {
                    /* Update dataparams */
                   ioRequest.dataparam  = (PSP_i2cDataParam *)packet->addr;
                   ioRequest.actualSize = 0;
                   ioRequest.appCbk     = NULL;
                   ioRequest.appPtr     = (Ptr)packet;
                   ioRequest.status     = PSP_SOK;
                   ioRequest.chanPtr    = chanp;


#ifdef I2C_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(  &DVTEvent_Log,
                        "I2C",
                        PSP_DVT_DESC(   PSP_DVT_eFUNC_START,
                                        PSP_DVT_dCHANNEL_ID_INST,
                                        PSP_DVT_dPACKET_ID,
                                        PSP_DVT_dSIZE_BYTES),
                  CHANNELID_INSTANCEID(
                   ((packet->cmd == IOM_WRITE) ? PSP_I2C_WRITE : PSP_I2C_READ),
                           ((I2cPortObj* )port)->i2cPortNumber),
                       ioRequest.dataparam->i2cTrans.buffer,
                       ioRequest.dataparam->i2cTrans.bufLen);
#endif /*I2C_INSTRUMENTATION_ENABLED */
                    switch (packet->cmd)
                    {
                        /* validating mode of creating the channel and type of
                         * operation to transfer the data
                         */
                        case IOM_WRITE:
                            /* If the Write call is there */
                            if ((chan->mode & IOM_OUTPUT) == NULL )
                            {
                                status = IOM_EBADARGS;
                            }
                            else
                            {
                                /* Update the flag and give Write Command to
                                   DDC layer */
                                ioRequest.dataparam->i2cTrans.flags |=
                                                                 PSP_I2C_WRITE;
                            }
                            break;
                        case IOM_READ:
                            /* If the Read call is there */
                            if ( (chan->mode & IOM_INPUT) == NULL )
                            {
                                status = IOM_EBADARGS;
                            }
                            else
                            {
                                /* Update the flag and give Read Command to
                                   DDC layer */
                                ioRequest.dataparam->i2cTrans.flags |=
                                                                PSP_I2C_READ;
                            }
                            break;
                            /* Not supported by the driver */
                        case IOM_ABORT:
                            status = IOM_ENOTIMPL;
                            break;
                        case IOM_FLUSH:
                            /* Call IOCTL functions and pass Cancel Pending
                               IO command */
                            status = i2c_mdControlChan(chanp,
                                         PSP_I2C_IOCTL_CANCEL_PENDING_IO,NULL);
                            /* Check for the Sucess of Operation */
                            if (status == PSP_SOK)
                            {
                                status = IOM_FLUSHED;
                            }
                            else
                            {

                                status = i2c_getIomErrorCode(status);
                            }
                            break;
                        default:
                            status  = IOM_EBADARGS;
                            break;
                    }
                    /* start transmission if command is read or write */
                    if (status == IOM_COMPLETED)
                    {

                        /* Call DDC Layer transfer function */
                        status = PSP_i2cTransferIOP(chan->ddc_Handle,
                                      &ioRequest,(Ptr)NULL);

                        /* Check status for any errors during Data transfer */
                        if (status != PSP_SOK)
                        {
                            status = i2c_getIomErrorCode(status);
                        }
                        else
                        {

                           /* if mode == polling...request is processed already*/
                            if((((DDC_I2cDriverObject *)
                                 (chan->ddc_Handle))->pi2cInstHandle->mode)
                                 != PSP_OPMODE_POLLED)
                            {
                                status = PSP_SINPROGRESS;
                            }
                        }

                    }
#ifdef I2C_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(  &DVTEvent_Log,
                        "I2C",
                        PSP_DVT_DESC(   PSP_DVT_eFUNC_END,
                                        PSP_DVT_dCHANNEL_ID_INST,
                                        PSP_DVT_dPACKET_ID,
                                        PSP_DVT_dSIZE_BYTES),
                 CHANNELID_INSTANCEID(
                   ((packet->cmd == IOM_WRITE) ? PSP_I2C_WRITE : PSP_I2C_READ),
                         ((I2cPortObj* )port)->i2cPortNumber),
                    ioRequest.dataparam->i2cTrans.buffer,
                     0);
#endif /*I2C_INSTRUMENTATION_ENABLED */

                }/* else of opened */
            }/* else of validation of chan and mode */
        } /* else of packet validation */
    }/* else of chan validation */
    return status;
}


/*!
 *  \brief I2C Ioctl
 *
 *  This function provides ioctl functionality for I2C
 *
 *  \param chanp    [IN]        I2C Driver Channel Handle
 *  \param cmd      [IN]        Operation to be performed, typically an enum
 *                                gets passed
 *  \param arg      [IN/OUT]    Provides additonal information related to the
 *                                operation
 *  \return IOM_COMPLETED or IOM Error code
 */
static Int i2c_mdControlChan(Ptr chanp, Uns cmd, Ptr arg)
{
    Int         status   = IOM_COMPLETED;
    I2cChanObj *chan     = NULL;

    /* Check if Channel Object is NULL */
    if(chanp == NULL)
    {
        status = IOM_EBADARGS;
    }
    else
    {
        chan = (I2cChanHandle)chanp;
        /* check for channel, port, state */
        if((NULL == chan) ||
               ((NULL == chan->port) || (chan->port->i2cState != I2C_OPENED)))
        {
            status = IOM_EBADMODE;
        }
        else
        {
            /* Function will perform run time configurations for i2c */
            status = PSP_i2cIoctl(chan->ddc_Handle,(PSP_I2cIoctlCmd)cmd,arg,
                                                                        NULL);
            /* Checks for the success of operation  */
            if ( status != PSP_SOK )
            {
                status = i2c_getIomErrorCode(status);
            }
        }
    }
    return status;
}


/*!
 *  \brief Delete a given I2C driver (instance)
 *
 *  The instance of I2C is deleted in softare.It closes the device for data
 *  transfers and usage by the upper layer driver software. The hardware is
 *  programmed to stop/abort data transfer (depending upon the type of device
 *  and its specifics) and the device ISR is "disabled" by the upper layer
 *  driver software after the completion of this call.
 *
 *  \param  devp [IN]        pointer in the GIO layer giving the address of the
 *                           port object of that particular instance
 *  \return IOM_COMPLETED or IOM Error code
 */
static Int i2c_mdUnBindDev(Ptr devp)
{
    Int         status = IOM_COMPLETED;
    I2cPortObj  *port  = NULL;

    if(NULL == devp )
    {
        status = IOM_EBADARGS;
    }
    else
    {
        port = (I2cPortObj *)devp;
        /* Check for port state */
        if ((port->i2cState != I2C_CREATED) && (port->i2cState != I2C_CLOSED))
        {
            status =  IOM_EBADMODE;
        }
        /* Check for the number of the I2c's Possible */
        else if (port->i2cPortNumber >= PSP_I2C_NUM_INSTANCES )
        {
            status = IOM_EBADARGS;
        }
        else
        {
            /* Delete the I2C Instance */
            status = PSP_i2cDelete(port->i2cPortNumber);
            if(status != PAL_SOK)
            {
                status = i2c_getIomErrorCode(status);
            }
            else
            {
                status          = IOM_COMPLETED;
                port->i2cState  = I2C_DELETED;
            }
        }
    }
    return status;
}


/*!
 *  \brief Close Channel of the I2C device
 *
 *  After the successful completion of this call, thedevice cannot perform any
 *    data IO on that channel.
 *
 *  \param  chanp [IN]          I2C Driver Channel Handle
 *  \return IOM_COMPLETED or IOM Error code
 */
static Int i2c_mdDeleteChan(Ptr chanp)
{
    Int             status      = IOM_COMPLETED;
    I2cChanObj     *chan        = NULL;
    I2cPortObj     *port        = NULL;
    Int             chanCount   = 0;

    /* Check if Channel Handle is NULL */
    if(NULL == chanp )
    {
        status = IOM_EBADARGS;
    }
    else
    {
        /* setting up channel and port handle */
        chan = (I2cChanHandle)chanp;
        /* validate channel, port,state */
        if((NULL == chan->port) || (chan->port->i2cState != I2C_OPENED))
        {
            status = IOM_EBADARGS;
        }
        else
        {
            port = (I2cPortObj*)chan->port;
            /* Function will deletes a channel for an instance of i2c   */
            status = PSP_i2cClose(chan->ddc_Handle);
            if(status != PAL_SOK)
            {
                status = i2c_getIomErrorCode(status);
            }
            else
            {
                chan->port = NULL;
                /* Check if all the channels are closed */
                for(chanCount = 0; chanCount < IOM_I2C_NUMCHANS ; chanCount++)
                {   /* If any Channel is in Use then come out of for loop */
                    if (port->chans[chanCount].port != NULL)
                    {
                        break;
                    }
                }
               /* If all channels are closed then update the state to closed */
                if (chanCount == IOM_I2C_NUMCHANS)
                {
                    port->i2cState = I2C_CLOSED;
                }
            }/* status */
        }/* Validate the Channel  */
    }/* If channel handle is NULL */
    return status;
}


/**
 *   \brief    PSP to IOM error code mapping
 *
 *   This function is used to map PSP error codes into appropriate IOM
 *   error codes.
 *
 *   \param    pspErrCode    [IN]    DDC layer error code
 *
 *   \return   IOM Error code
 */
static Int32 i2c_getIomErrorCode(PSP_Result pspErrCode)
{
    /*------------------------------------------------------------------
                            IOM Error code look up table
     -------------------------------------------------------------------
     * The following mapping shows the mapping done from PSP_Result to
     * IOM error code. This will be used in scenario where IOMini driver
     * interfaces with the DDC (PSP component)
     *
     * PSP_SOK                     (0)   IOM_COMPLETED            0
     * PSP_SINPROGRESS             (1)   IOM_PENDING              1
     * PSP_E_DRIVER_INIT           (-1)  IOM_ETIMEOUTUNREC      -11
     * PSP_E_NO_MEMORY             (-2)  IOM_EALLOC              -5
     * PSP_E_RESOURCES             (-3)  IOM_EALLOC              -5
     * PSP_E_INVAL_STATE           (-4)  IOM_EBADMODE            -7
     * PSP_E_INVAL_PARAM           (-5)  IOM_EBADARGS           -10
     * PSP_E_NOT_SUPPORTED         (-6)  IOM_ENOTIMPL            -9
     * PSP_E_IO_CANCEL_FAIL        (-7)  IOM_ETIMEOUT            -2
     * PSP_E_FIFO_NOT_ENABLED      (-8)  IOM_EBADMODE            -7
     * PSP_E_INVALID_MODE          (-9)  IOM_EBADMODE            -7
     * PSP_E_INVALID_CMD           (-10) IOM_EBADARGS           -10
     *
     */
    static Int32 iomErrorCodeList[12] =
    {
        IOM_EBADARGS, /* 0 */
        IOM_EBADMODE, /* 1 */
        IOM_EBADMODE, /* 2 */
        IOM_ETIMEOUT, /* 3 */
        IOM_ENOTIMPL, /* 4 */
        IOM_EBADARGS, /* 5 */
        IOM_EBADMODE, /* 6 */
        IOM_EALLOC,   /* 7 */
        IOM_EALLOC,   /* 8 */
        IOM_ETIMEOUTUNREC,/* 9 */
        IOM_COMPLETED,/* 10 */
        IOM_PENDING,  /* 11 */
    };

    Int32 iomCode = pspErrCode + 10;

    if( (iomCode >=0) && (iomCode < 12 ) )
    {
        iomCode = iomErrorCodeList[iomCode];
    }
    else
    {
        iomCode = pspErrCode;
    }

    return(iomCode);
}
/*!
 * Revision History
 * ================
 *  \version    0.1       Created                             Nagarjuna Kristam
 *  \version    0.2       Modified interrupt unregistration   Maulik Desai
 *  \version    0.3       MISRAC and formatting               Nagarjuna Kristam
 *  \version    0.4       Implementing review commnets 1      Nagarjuna Kristam
 *  \version    0.5       Implementing review commnets 2      Nagarjuna Kristam
 *  \version    0.6       Merging with dda_i2cIom.c file      Nagarjuna Kristam
 *  \version    0.7       Merging with LC code and updated    Nagarjuna Kristam
 *  \version    0.8       Initialized port object while
                          initializing                        Nagarjuna Kristam
 */

