/*****************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims       |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file   ddc_i2c.c
 *
 *  \brief  I2C implementation
 *
 *  This file contains the implementation of the I2C driver to utilize the
 *  serivces of the I2C device driver.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author     PSG
 *
 *  \version    0.6
 *
 */
#include <ti/sdo/pspdrivers/pal_os/pal_osProtect.h>
#include <ti/sdo/pspdrivers/pal_os/pal_ostime.h>
#include <ti/sdo/pspdrivers/pal_os/pal_oswait.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osmem.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osTask.h>

#include <ti/sdo/pspdrivers/drivers/i2c/src/dda_i2cBios.h>
#include <ti/sdo/pspdrivers/drivers/i2c/psp_i2c.h>
#include <ti/sdo/pspdrivers/drivers/i2c/src/psp_i2cApi.h>
#include <ti/sdo/pspdrivers/drivers/i2c/src/ddc_i2c.h>


#ifdef I2C_INSTRUMENTATION_ENABLED
    #include <ti/sdo/pspdrivers/pal_os/bios/psp_log.h>
#endif /*I2C_INSTRUMENTATION_ENABLED*/

#ifdef I2C_DEBUG_PRINTF
#include <stdio.h>
#define     I2C_DEBUG   printf
#endif /* I2C_DEBUG_PRINTF */

/**< timeout value to break polled mode operation because
  of any hard ware limitations */
#define I2C_POLLED_TIMEOUT          (0x000FFFFFu)

/**  I2C DDC Object.One per instance                */
DDC_I2cObject ddcI2CObj[PSP_I2C_NUM_INSTANCES] = DDC_I2C_OBJECT_DEFAULT;
/**  I2C Driver object.One per instance per open    */
DDC_I2cDriverObject ddcI2CDriver[PSP_I2C_NUM_INSTANCES][PSP_I2C_NUM_OPENS];


/*------------------------Global Extern Functions---------------------------*/
extern void DDA_i2cRegisterIntrHandler(
                                       Uint32      instNum,
                                       Uint32      intNum,
                                       pspI2cIsr   isrFunc,
                                       PSP_Handle  i2cObj
                                      );

extern void DDA_i2cUnregisterIntrHandler(Uint32 instNum, Uint32 intNum);


/*------------------------Static Functions Declerations---------------------*/

/*
 *  This is the function in which a dual data transfer happens.Function
 *  validates input parameters,set the parameter values in to the registers
 *  and starts transmission.
 */
static Int DDC_i2cTransferIOP (PSP_Handle handle,DDC_I2cIOP *ioPacket,Ptr param);
/*
 * To calculate clock settings of I2C
 */
static Int i2cCalculateClockSettings(
                                     const DDC_I2cObject *obj,
                                     Uint32 *psc,
                                     Uint32 *clock
                                    );
/*
 * To cancel IO
 */
static void i2c_cancelIO(DDC_I2cObject *i2cObj,Int retCode);

static void DDC_copyRequestToIOP(
                         PSP_I2cRequest  const *pspRequest,
                         DDC_I2cIOP       *ioPacket
                         );
static void DDC_copyIOPToRequest(
                         DDC_I2cIOP const *ioPacket,
                         PSP_I2cRequest        *pspRequest
                         );
static Void i2cCompleteIOInIsr (DDC_I2cObject *i2cObj);
static Void I2c_localEnsureIsReadyforNext (DDC_I2cObject *i2cObj);
static Void I2c_localPrepareForTransfer(DDC_I2cObject *i2cObj);
static Void I2c_localGetNextChannel(Ptr *appHandle,
                                     DDC_I2cDriverObject **pChanHandle);
/*!
 *  \brief Initialize I2C driver (instance)
 *
 *  Typically,software bookkeeping functions are performed in this call.
 *    Configuration information is passed in the call and initialization is
 *    done based upon the information.
 *
 *  \param  instanceId [IN]     I2C instance number
 *  \param  initConfig [IN]     Initialization configuration
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result  PSP_i2cCreate(Uint32 instanceId, const PSP_I2cConfig *initConfig)
{
    DDC_I2cObject   *i2cObj     = NULL;
    PSP_Result      retVal      = PSP_SOK;
    Uint32          chanCount   = 0;

    /* Validate instance ID */
    if ((instanceId >= PSP_I2C_NUM_INSTANCES)
                        || ((PSP_I2cConfig *)NULL == initConfig))
    {
        retVal = PSP_E_INVAL_PARAM;
    }
    else
    {
        i2cObj = &ddcI2CObj[instanceId];
        if ((DDC_I2cObject *)NULL == i2cObj)
        {
            retVal = PSP_E_INVAL_PARAM;
        }
        /* Check state of driver */
        else if (i2cObj->state != I2C_DELETED)
        {
            retVal = PSP_E_INVAL_STATE;
        }
        /* Check input freq param limits */
        else if ((initConfig->moduleInputClkFreq < DDC_I2C_MIN_INPUT_FREQ)
                || (initConfig->moduleInputClkFreq > DDC_I2C_MAX_INPUT_FREQ))
        {
                retVal = PSP_E_INVAL_PARAM;
        }
        /* Check the I2C Bus Frequency */
        else if ((initConfig->i2cBusFreq < DDC_I2C_MIN_BUS_FREQ) ||
            (initConfig->i2cBusFreq > DDC_I2C_MAX_BUS_FREQ))
        {
                retVal = PSP_E_INVAL_PARAM;
        }
        /* Check the Number of Bits */
        else if ((initConfig->numBits < DDC_I2C_MIN_NUM_BITS)
                            || (initConfig->numBits > DDC_I2C_MAX_NUM_BITS))
        {
                retVal = PSP_E_INVAL_PARAM;
        }
        /* Check if addressing is to be enabled/disabled */
        else if ((initConfig->addressing != (Bool)TRUE)
                                && (initConfig->addressing != FALSE))
        {
            retVal = PSP_E_INVAL_PARAM;
        }
        /* Check if dlb is to be enabled/disabled */
        else
        {
            if ((initConfig->dlb != (Bool)TRUE) && (initConfig->dlb != FALSE))
            {
                retVal = PSP_E_INVAL_PARAM;
            }
        }
    } /*Instance Id Validation */
    /* Copy init parameters    */
    if (PSP_SOK == retVal)
    {
        /** mAKE I2C DDC object handle NULL */
        for (chanCount=0;chanCount<PSP_I2C_NUM_OPENS;chanCount++)
        {
            /* setting Global driver object initial value to zero */
            PAL_osMemSet(&ddcI2CDriver[instanceId][chanCount],
                         0,
                         sizeof(DDC_I2cDriverObject));
        }
        /* Check the operation mode */
        if (PSP_OPMODE_INTERRUPT == initConfig->mode)
        {
            i2cObj->mode = PSP_OPMODE_INTERRUPT;
        }
        else
        {
            i2cObj->mode = PSP_OPMODE_POLLED;
        }
        /* Update the Configuration Parameters */
        i2cObj->i2cOwnAddr          = initConfig->i2cOwnAddr;
        i2cObj->moduleInputClkFreq  = initConfig->moduleInputClkFreq;
        i2cObj->i2cBusFreq          = initConfig->i2cBusFreq;
        i2cObj->numBits             = initConfig->numBits;
        i2cObj->addressing          = initConfig->addressing;
        i2cObj->dlb                 = initConfig->dlb;
    } /* retVal check condition */

    /* Set state as created */
    if(PSP_SOK == retVal)
    {
        i2cObj->numOpens = 0;
        /* Update the State */
        i2cObj->state = I2C_CREATED;
        i2cObj->currentActiveChannel = NULL;
        retVal = PSP_SOK;
    }
    return retVal;
}
/*!
 *  \brief Delete I2C driver (instance)
 *
 *  The I2C instance is deleted in software. If there are open handles then
 *  they are invalidated
 *
 *  \param  instanceId [IN]     I2C instance number
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_i2cDelete(Uint32 instanceId)
{
    DDC_I2cObject   *i2cObj = NULL;
    Int             retVal  = PSP_SOK;

    /* Validate instance ID */
    if (instanceId >= PSP_I2C_NUM_INSTANCES)
    {
            retVal = (PSP_E_INVAL_PARAM);
    }
    else
    {
        i2cObj = &ddcI2CObj[instanceId];
        /*Check if DDC Object is NULL */
        if ((DDC_I2cObject *)NULL == i2cObj)
        {
            retVal = (PSP_E_INVAL_PARAM);
        }
        else
        { /* Check state of driver */
            if ((i2cObj->state != I2C_CREATED)
                        && ((i2cObj->state != I2C_CLOSED)))
            {
                retVal = (PSP_E_INVAL_STATE);
            }
        }
    }
    if (PSP_SOK == retVal)
    {
       /* set state to deleted */
        i2cObj->numOpens = 0u;
        i2cObj->state = I2C_DELETED;
    }
    return retVal;
}
/*!
 *  \brief Open Instance of the I2C device
 *
 *   This function prepares the device hardware for data transfers and usage
 *   by the upper layer driver software.It returns an handle to the driver
 *     instance.
 *
 *
 *  \param  instanceId  [IN]    I2C instance number
 *  \param  callback    [IN]    Callback function for events notification
 *  \param  appHandle   [IN]    This appHandle will be provided by user and will
 *                              be returned along with call back
 *  \return PSP_Handle  [OUT]   If successful returns driver instance Handle
 *                              else negative error code
 *
 *  \Note:- The driver can be opened multiple times
 */
/* ARGSUSED */
PSP_Handle  PSP_i2cOpen(
                        Uint32 instanceId,
                        PSP_I2cAppCallback callback,
                        Ptr appHandle
                        )
{
    DDC_I2cObject        *i2cObj    = NULL;
    DDC_I2cDriverObject  *i2cHandle = NULL;
    Uint32                psc       = 0;
    Uint32                clk       = 0;
    Uint32                cnt       = 0;
    Uint32                cookie    = 0;
    Uint8                 i         = 0;

    PAL_OsSemAttrs        semAttrs        = PAL_OSSEM_ATTRS;
    PAL_OsTaskHandle      thisTask        = NULL;


    /* Validate instance ID */
    if (instanceId < PSP_I2C_NUM_INSTANCES)
    {
        /* Check state of driver */
        i2cObj = &ddcI2CObj[instanceId];
        if ( ((NULL != i2cObj) && (i2cObj->state != I2C_DELETED)) &&
                                    (i2cObj->numOpens < PSP_I2C_NUM_OPENS))
        {
            /* Create i2c driver object */
            for (cnt=0; cnt < PSP_I2C_NUM_OPENS; cnt++)
            {
                if(NULL== ddcI2CDriver[i2cObj->instanceId][cnt].pi2cInstHandle)
                {
                    ddcI2CDriver[i2cObj->instanceId][cnt].pi2cInstHandle=i2cObj;
                    i2cHandle =&ddcI2CDriver[i2cObj->instanceId][cnt];
                    break;
                }
            }
        }
    }
    if(NULL != i2cHandle)
    {
        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);
        /* Init hardware if first open */
        if (NULL == i2cObj->numOpens)
        {
            i2cObj->instanceId   = (int)instanceId;
            i2cObj->i2cRegs      = (CSL_I2cRegs *) CSL_I2C_0_REGS;
            i2cObj->intNum       = CSL_INTC_EVENTID_I2CINT0;
            /* Put the module in reset */
            i2cObj->i2cRegs->ICMDR = 0;
            /* Calculate the Clock according to Prescalar */
            if(0 != i2cCalculateClockSettings(i2cObj , &psc, &clk))
            {
                /* Exit of Critical Section */
                PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
                i2cHandle = NULL;
            }
            else
            {
                /* Load Prescalar value into Register */
                i2cObj->i2cRegs->ICPSC = psc;

                /* Load Clock Register into Register  */
                i2cObj->i2cRegs->ICCLKH = clk/2u;
                i2cObj->i2cRegs->ICCLKL = clk - i2cObj->i2cRegs->ICCLKH;

                /* Set own address and bits/byte      */
                i2cObj->i2cRegs->ICOAR = i2cObj->i2cOwnAddr;
                i2cObj->i2cRegs->ICMDR = (i2cObj->numBits
                                            & DDC_I2C_NUMBITS_MASK)
                                                | CSL_I2C_ICMDR_MST_MASK;
                /* Enable/Disable DLB */
                if (TRUE == i2cObj->dlb)
                {
                    i2cObj->i2cRegs->ICMDR |= CSL_I2C_ICMDR_DLB_MASK;
                }
                else
                {
                    i2cObj->i2cRegs->ICMDR &= ~(CSL_I2C_ICMDR_DLB_MASK);
                }
                /* Enable/Disable 7/10 bit Addressing mode */
                if (TRUE == i2cObj->addressing)
                {
                    i2cObj->i2cRegs->ICMDR |= CSL_I2C_ICMDR_XA_MASK;
                }
                else
                {
                    i2cObj->i2cRegs->ICMDR &= (~(CSL_I2C_ICMDR_XA_MASK));
                }
                /* If interrupt mode, register ISR */
                if (i2cObj->mode != PSP_OPMODE_POLLED)
                {
                    /* Call DDA layer to Register the Handler */
                    DDA_i2cRegisterIntrHandler(instanceId,
                                               i2cObj->intNum,
                                               (pspI2cIsr)&DDC_i2cIntrHandler,
                                               i2cObj);
                }
                /* Bring module out of reset */
                i2cObj->i2cRegs->ICMDR |= CSL_I2C_ICMDR_IRS_MASK;

                /* Set the backward compatibility mode off - this is
                 * critical to get interrupts correctly */
                i2cObj->i2cRegs->ICEMDR &= (~CSL_I2C_ICEMDR_BCM_MASK);
            } /* else retCode */
        } /* First open initialization */


    } /* retVal = NULL before first open */

    if(NULL != i2cHandle)
    {
        /* Increment open count and return driver handle */
        ++i2cObj->numOpens;
        /* Updated the driver state */
        i2cObj->state = I2C_OPENED;

        i2cHandle->channelState = I2C_OPENED;
        i2cHandle->activeIOP    = NULL;
        i2cHandle->flush        = FALSE;

        if (TRUE == PAL_osTskIsTsk())
        {
            /* Returns a handle to the currently executing Task object( );   */
            thisTask =(PAL_OsTaskHandle) PAL_osTskSelf();

            /* Get task priority( Task_Handle handle );                      */
            i2cHandle->taskPriority = PAL_osTskGetPriority(thisTask);
        }


        i2cHandle->callBack  = callback;
        i2cHandle->appHandle = appHandle;


        /* make node */
        PAL_OSLIST_MKNODE(&i2cHandle->queueFreeList);
        PAL_OSLIST_MKNODE(&i2cHandle->queueReqList);


        i = 0;

        /*
        link in all the allocatated IOPs into the FreeQueue list
        */
        while (i < PSP_I2C_MAX_REQ)
        {
            PAL_osListEnqueue(
                            (Ptr)&i2cHandle->queueFreeList,
                            (Ptr)&i2cHandle->ioRequests[i]
                            );
            i++;
        }

        /*
         * This semaphore will be used to pend to all TX requests are
         * done - during a flush request
         */
        if ( PAL_SOK != PAL_osSemCreate(
                                    NULL,
                                    0,
                                    &semAttrs,
                                    (PAL_OsSemHandle)&i2cHandle->syncSem)
                                    )
        {
            PSP_i2cClose((PSP_Handle)i2cHandle);
            i2cHandle = NULL;
            return (i2cHandle);
        }

        /* End of critical section  */
        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
    }
    return (PSP_Handle)i2cHandle;
}
/*!
 *  \brief Close Instance of the I2C device
 *
 *  This function closes the device for data transfers and used by the
 *  upper layer driver software. The hardware is programmed to stop/abort data
 *  transfer (depending upon the type of device and its specifics) and the
 *  device ISR is "disabled" by the upper layer driver software after the
 *  completion of this call. After the successful completion of this call, the
 *  device cannot perform any data IO.
 *
 *  \param  handle [IN]         I2C Driver Instance Handle
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_i2cClose(PSP_Handle handle)
{
    DDC_I2cDriverObject  *pi2cHandle = NULL;
    DDC_I2cObject        *i2cObj     = NULL;
    Uint32               cookie      = 0;
    Int                  retVal      = PSP_SOK;
    Int                  retCode     = 0;
    /* Check state of handle */
    if (NULL == handle)
    {
        retVal = PSP_E_INVAL_PARAM;
    }
    else
    {
        pi2cHandle = (DDC_I2cDriverObject *)handle;
        if ( ((DDC_I2cDriverObject *) NULL == pi2cHandle) ||
                         (pi2cHandle->channelState != I2C_OPENED) )
        {
            retVal = PSP_E_INVAL_PARAM;
        }
        else
        {
            i2cObj =  pi2cHandle->pi2cInstHandle;
            if ((DDC_I2cObject *) NULL == i2cObj)
            {
                retVal = (PSP_E_INVAL_PARAM);
            }
            else
            {
                /* Check for the validity of the I2C STATE */
                if ( i2cObj->state != I2C_OPENED )
                {
                    retVal = (PSP_E_INVAL_STATE);
                }
            }
        }
    }/*if Handle is NULL */

    if (PSP_SOK == retVal)
    {
        /*  Check for any IO is pending, if yes then cancel io */
        if ( i2cObj->pendingState == TRUE )
        {
            retCode = PSP_i2cIoctl(
                                   handle,
                                   PSP_I2C_IOCTL_CANCEL_PENDING_IO,
                                   NULL,
                                   NULL
                                   );

            if(retCode != PSP_SOK)
            {
                retVal = retCode;
            }
        }

        /* delete the semaphore created for the syncronisation */
        if (NULL != pi2cHandle->syncSem)
        {
            /* delate only if the semaphore exists             */
            PAL_osSemDelete(pi2cHandle->syncSem);
        }

        /* Start of Critical Section */
        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);
        /* Decrease the Number of Opens */
        --i2cObj->numOpens;
        /* Exit of Critical Section */
        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
        pi2cHandle->pi2cInstHandle = NULL;
        pi2cHandle->channelState   = I2C_CLOSED;
        pi2cHandle = NULL;
        if (0 == i2cObj->numOpens)
        {
            /* If interrupt mode, unregister ISR */
            if (i2cObj->mode != PSP_OPMODE_POLLED)
            {
                /* Call DDA Layer function to unregister the ISR */
                DDA_i2cUnregisterIntrHandler(
                                             (Uint32)i2cObj->instanceId,
                                             i2cObj->intNum
                                             );
            }
            i2cObj->state = I2C_CLOSED;
        }
    }/* retVal */
    return retVal;
}
/*!
 *  ======== i2cWaitForBB ========
 *  \brief  The local routine Check for Bus Busy
 *
 *  \param  i2cObj          DDC i2c structure object
 *  \param  waitCount       Count to wait when bus is busy
 *  \param  sleepAllowed    Verify whether sleep allowed while waiting
 *
 */
static Int i2cWaitForBB(
                        const  DDC_I2cObject *i2cObj,
                        Uint32 waitCount,
                        Uint32 sleepAllowed
                       )
{
    volatile Uint32 countdown = waitCount;
    Int             retVal    = 0;
    /* Check the Handle */
    if (i2cObj == ((DDC_I2cObject *) NULL))
    {
        retVal = PSP_E_INVAL_PARAM;
    }
    else
    {
        while (i2cObj->i2cRegs->ICSTR & CSL_I2C_ICSTR_BB_MASK)
        {
            if (countdown != 0)
            {
                if (sleepAllowed != 0)
                {
                    PAL_osWaitMsecs(1u); /* Wait for 1 milliSeconds */
                }
            }
            else
            {
#ifdef I2C_DEBUG_PRINTF
                I2C_DEBUG("\ni2cWaitForBB: Bus is busy\n");
#endif
                retVal = -1;
                break;
            }
            --countdown;/* Decrement the Countdown */
        }
    } /* retVal */
    return retVal;
}
/*!
 *  \brief Read/Write (single transaction) using I2C
 *
 *  This function read/writes single transaction data using the I2C peripheral.
 *
 *  \param  handle      [IN]    I2C Driver Instance Handle
 *  \param  ioPacket   [IN]    Packet containg trns data
 *  \param  param     [IN]    Extra paramter for future usage
 *  \return returns either number of bytes sucessfully transmitted on success
 *          or negative error code on failure
 */

Int PSP_i2cTransferIOP (
                     PSP_Handle handle,
                     PSP_I2cRequest *ioRequest,
                     Ptr        param
                     )

{
    DDC_I2cDriverObject *pi2cHandle = NULL;
    DDC_I2cObject       *i2cObj     = NULL;
    Int                  retVal     = 0;
    volatile Uint32     delay       = I2C_POLLED_TIMEOUT;
    DDC_I2cIOP          *ioPacket   = NULL;

    /* Check if Handle is NULL */
    if ((handle == NULL) || (NULL == ioRequest))
    {
        retVal = (PSP_E_INVAL_PARAM);
    }
    else
    {
        param       = param;  /* To remove warning*/
        pi2cHandle  = (DDC_I2cDriverObject *)handle;
        i2cObj      =  pi2cHandle->pi2cInstHandle;
        if (i2cObj == ((DDC_I2cObject *) NULL) )
        {
            retVal = (PSP_E_INVAL_PARAM);
        }

        if( pi2cHandle->channelState != I2C_OPENED )
        {
            retVal = PSP_E_INVAL_STATE;
        }

    }

    if(NULL == retVal)
    {
        if ((ioRequest->dataparam->i2cTrans.flags & PSP_I2C_REPEAT) == 0)
        {
            /* Check input parameters */
          if ( ((ioRequest->dataparam->i2cTrans.buffer == NULL) ||
           (ioRequest->dataparam->i2cTrans.bufLen > DDC_I2C_BUFFER_MAXLENGTH))
                       || (ioRequest->dataparam->i2cTrans.bufLen == NULL) )
         {
             retVal = (PSP_E_INVAL_PARAM);
         }
         else
         {
          if(( (ioRequest->dataparam->i2cTrans.flags & PSP_I2C_READ) != 0 )
             && ( (ioRequest->dataparam->i2cTrans.flags & PSP_I2C_WRITE) != 0 ))
             {
                 retVal = (PSP_E_INVAL_PARAM);
             }
          }
        }
        /* Return error if aplication sets stop bit during
           repeat mode and bufLen > 0 */
        if ((retVal == PAL_SOK) &&
           ((ioRequest->dataparam->i2cTrans.flags & PSP_I2C_REPEAT) != 0))
        {
            if ( (ioRequest->dataparam->i2cTrans.bufLen > 0)
               && ((ioRequest->dataparam->i2cTrans.flags & PSP_I2C_STOP) != 0))
            {
                retVal = (PSP_E_INVAL_PARAM);
            }
        }
        if (retVal == PAL_SOK)
        {
            if (ioRequest->dataparam->timeout == 0)
            {
                retVal = PSP_E_TIMEOUT;
            }
        }
    } /* retVal = NULL */
    if (NULL == retVal)
    {


         /* no transfer is happening in any of the channels in this
         * instance of the driver.So for sure we can initiate the transfer here
         * itself */

            if(TRUE == PAL_osListIsEmpty (&pi2cHandle->queueFreeList) )
            {

                /*
                all packets already exhausted
                */
                retVal = PSP_E_INVAL_PARAM;

            }
            else
            {
                ioPacket = (DDC_I2cIOP *)PAL_osListDequeue(
                                        &pi2cHandle->queueFreeList);

                if(NULL != ioPacket)
                {

                    DDC_copyRequestToIOP(ioRequest,ioPacket);
                    if(NULL == i2cObj->currentActiveChannel)
                    {
                        /* no transfer is happening in any of the channels in
                         * this instance of the driver.So for sure we can
                         * initiate the transfer here itself */
                         i2cObj->currentActiveChannel = pi2cHandle;
                         pi2cHandle->activeIOP        = ioPacket;

                             /* call i2c transfer function                  */
                        retVal =  DDC_i2cTransferIOP(
                                                i2cObj,
                                                ioPacket,
                                                NULL);
                        if ( i2cObj->mode == PSP_OPMODE_POLLED )
                        {

                            PAL_osListEnqueue(&(pi2cHandle->queueFreeList),
                                                     (Ptr)ioPacket);
                        }
                    }
                    else
                    {
                          PAL_osListEnqueue (&pi2cHandle->queueReqList,
                                                (Ptr)ioPacket);
                    }

                }


            }


      }
    return retVal;
}

/**
 *  \brief DDC_copyRequestToIOP
 *
 *  Copies data from request structure provided by DDA layer to
 *  meaningful IO structure at  DDC layer.
 *
 *  \param    pspRequest       [IN]    Porinter to DDA layer reuqest structuer
 *  \param    ioPacket         [IN]    Porinter to DDC layer IO structuer
 *  \return   void
 */
static void DDC_copyRequestToIOP(
                         PSP_I2cRequest  const *pspRequest,
                         DDC_I2cIOP       *ioPacket
                         )
{

    if ((NULL != pspRequest ) && (NULL != ioPacket))
    {

        ioPacket->actualSize    = pspRequest->actualSize;
        ioPacket->appCbk        = pspRequest->appCbk;
        ioPacket->appPtr        = pspRequest->appPtr;
        ioPacket->chanPtr       = pspRequest->chanPtr;
        ioPacket->dataparam     = pspRequest->dataparam;
        ioPacket->status        = pspRequest->status;


    }
}
/**
 *  \brief DDC_copyIOPToRequest
 *
 *  Restores the data from DDC layer request structure to
 *  DDA understandable psp request structure.
 *
 *  \param    ioPacket         [IN]    Porinter to DDC layer IO structuer
 *  \param    pspRequest       [IN]    Porinter to DDA layer reuqest structuer
 *
 *  \return   void
 */
static void DDC_copyIOPToRequest(
                         DDC_I2cIOP const *ioPacket,
                         PSP_I2cRequest        *pspRequest
                         )
{
    if ((NULL != pspRequest ) && (NULL != ioPacket))
    {

        pspRequest->actualSize  = ioPacket->actualSize;
        pspRequest->appCbk      = ioPacket->appCbk;
        pspRequest->appPtr      = ioPacket->appPtr;
        pspRequest->chanPtr     = ioPacket->chanPtr;
        pspRequest->dataparam   = ioPacket->dataparam;
        pspRequest->status      = ioPacket->status;

    }

}



/*!
 * \brief Function implemented by DDC
 *
 * These function are implemented in DDC and are called by DDA.
 * \Note:   count has been added to this function as an exit condition for
 *          continuous interrupt conditions which may arise because of any
 *          hardware issue.
 *  \param      *i2cObj     Pointer to the I2c Handle
 */
void DDC_i2cIntrHandler(DDC_I2cObject *i2cObj)
{
    volatile Uint32 intStatus;
    Uint32          count = 0;  /*  Varaiable used to check if wrong interrupts
                                    comes because of any hardware problems */
    if (i2cObj == ((DDC_I2cObject *) NULL))
    {
        return ;
    }
    /* Read Status from ICIVR register */
    intStatus = i2cObj->i2cRegs->ICIVR;
    while ((0 != intStatus) && (count < 10u))
    {

#ifdef I2C_INSTRUMENTATION_ENABLED
        if(intStatus == CSL_I2C_ICIVR_INTCODE_ICXRDY)
        {
                    PSP_LOG_EVENT(
                        &DVTEvent_Log,
                        "I2C",
                        PSP_DVT_DESC(
                                    PSP_DVT_eINT_START,
                                    PSP_DVT_dINST,
                                    PSP_DVT_dCHANNEL_ID,
                                    PSP_DVT_dNONE
                                    ),
                        i2cObj->instanceId,
                        PSP_I2C_WRITE
                        );
        }
        else if(intStatus == CSL_I2C_ICIVR_INTCODE_ICRRDY)
        {
                    PSP_LOG_EVENT(
                        &DVTEvent_Log,
                        "I2C",
                        PSP_DVT_DESC(
                                    PSP_DVT_eINT_START,
                                    PSP_DVT_dINST,
                                    PSP_DVT_dCHANNEL_ID,
                                    PSP_DVT_dNONE
                                    ),
                        i2cObj->instanceId,
                        PSP_I2C_READ
                        );
        }
#endif /*I2C_INSTRUMENTATION_ENABLED */

        switch (intStatus) {
            case CSL_I2C_ICIVR_INTCODE_NONE:
                /* No interrupt - Ideally, control should not come here. */
                break;
                /* Arbitration loss */
            case CSL_I2C_ICIVR_INTCODE_AL:
                i2cObj->currError = (Int)PSP_I2C_ARBITRATION_LOSS_ERR;
                i2c_cancelIO(i2cObj,PSP_I2C_ARBITRATION_LOSS_ERR);
                i2cCompleteIOInIsr(i2cObj);
                /* disable interrupts once releasing the semaphore to avaid any
                   interrupts in getting raised */
                i2cObj->i2cRegs->ICIMR = CSL_I2C_ICIMR_RESETVAL;
                break;
                /* NACK interrupt */
            case CSL_I2C_ICIVR_INTCODE_NACK:
                if ((i2cObj->currFlags & PSP_I2C_IGNORE_NAK) != 0){
                    i2cObj->currError = PSP_SOK;
                } else {
                    i2cObj->currError = (Int)PSP_I2C_NACK_ERR;
                    i2c_cancelIO(i2cObj,PSP_I2C_NACK_ERR);
                    i2cCompleteIOInIsr(i2cObj);
                }
                /* disable interrupts once releasing the semaphore to avaid any
                   interrupts in getting raised */
                i2cObj->i2cRegs->ICIMR = CSL_I2C_ICIMR_RESETVAL;
                break;
            /* Register Access ready */
            case CSL_I2C_ICIVR_INTCODE_ARDY:
                /* We do not need to handle this interrupt in this driver */
                /* Disable this interrupt enable bit */
                i2cObj->i2cRegs->ICIMR &= ~DDC_ICIVR_INTCODE_ARDY;
                break;
            /* Receive Data ready */
            case CSL_I2C_ICIVR_INTCODE_ICRRDY:
                if (i2cObj->currBufferLen != NULL)
                {
                    *i2cObj->currBuffer++ = (Uint8) (i2cObj->i2cRegs->ICDRR);
                    /* Decrement the BufferLen */
                    --i2cObj->currBufferLen;
                    /* Update the actual data transfer size */
                    i2cObj->currentActiveChannel->activeIOP->actualSize++;

                    if ((i2cObj->currBufferLen == NULL)||
                         (TRUE == i2cObj->Cancel_Pending_IO))
                    {
                        i2cCompleteIOInIsr(i2cObj);
                    }
                }
                break;
            /* Transmit Data ready */
            case CSL_I2C_ICIVR_INTCODE_ICXRDY:
                if (i2cObj->currBufferLen != NULL)
                {
                    i2cObj->i2cRegs->ICDXR = (Uint8)*(i2cObj->currBuffer++);
                    --i2cObj->currBufferLen;
                    /* Update the actual data transfer size */
                    i2cObj->currentActiveChannel->activeIOP->actualSize++;

                    if ((i2cObj->currBufferLen == NULL)||
                         (TRUE == i2cObj->Cancel_Pending_IO))
                    {
                        i2cCompleteIOInIsr(i2cObj);
                    }

                }
                break;
            /* Stop condition detected */
            case CSL_I2C_ICIVR_INTCODE_SCD:
                i2cCompleteIOInIsr(i2cObj);
                break;
            /* Addressed as slave */
            case CSL_I2C_ICIVR_INTCODE_AAS:
                /* Slave mode is not supported currently*/
                /* Disable Addressed as slave interrupt */
                i2cObj->i2cRegs->ICIMR &= ~(CSL_I2C_ICIMR_AAS_MASK);
                break;

            default:
                /* Wrong interrupt - Ideally, control should not come here. */
                count++;
                break;
        }

#ifdef I2C_INSTRUMENTATION_ENABLED
        if(intStatus == CSL_I2C_ICIVR_INTCODE_ICXRDY)
        {
            PSP_LOG_EVENT(  &DVTEvent_Log,
                            "I2C",
                            PSP_DVT_DESC(   PSP_DVT_eINT_END,
                                            PSP_DVT_dINST,
                                            PSP_DVT_dCHANNEL_ID,
                                            PSP_DVT_dNONE
                                        ),
                                i2cObj->instanceId,
                                PSP_I2C_WRITE);
        }

        else if(intStatus == CSL_I2C_ICIVR_INTCODE_ICRRDY)
        {
            PSP_LOG_EVENT(  &DVTEvent_Log,
                            "I2C",
                            PSP_DVT_DESC(   PSP_DVT_eINT_END,
                                            PSP_DVT_dINST,
                                            PSP_DVT_dCHANNEL_ID,
                                            PSP_DVT_dNONE
                                        ),
                            i2cObj->instanceId,
                            PSP_I2C_READ);
        }
#endif /*I2C_INSTRUMENTATION_ENABLED */

        /* Read Status from ICIVR register */
        intStatus = i2cObj->i2cRegs->ICIVR;

    }

}


/*!
 *  \brief Gets the next channel for IO
 *
 *  This function Gets the next channel  which is having the IO to perform
 *  depending the Task Priority.
 *
 *  \param  appHandle      [IN]    I2C Port Object
 *  \param  pChanHandle   [OUT]    Address of the channel

 *  \return returns the channel which is gaving the IO packets to perform.
 */




static Void I2c_localGetNextChannel(Ptr *appHandle,
                             DDC_I2cDriverObject **pChanHandle)
{
    Uint32 counter,tskPriority = 0;
    Uint32 chanIndexWithMaxPri = IOM_I2C_NUMCHANS;
    Uint32 lastFoundMaxPri     = 0;
    I2cPortObj  *instHandle = NULL;
    DDC_I2CState chnState;

    instHandle = (I2cPortObj *)appHandle;

    for(counter=0; counter<IOM_I2C_NUMCHANS; counter++)
    {

        chnState    = ((DDC_I2cDriverObject *)
                        (instHandle->chans[counter].ddc_Handle))->channelState;
        tskPriority = ((DDC_I2cDriverObject *)
                        (instHandle->chans[counter].ddc_Handle))->taskPriority;
        if((I2C_OPENED == chnState)&&
            (lastFoundMaxPri < tskPriority))
        {

            if(FALSE == PAL_osListIsEmpty (&(((DDC_I2cDriverObject *)
                (instHandle->chans[counter].ddc_Handle))->queueReqList )) )
            {
                lastFoundMaxPri = tskPriority;
                chanIndexWithMaxPri = counter;
            }
        }
    }

    if (IOM_I2C_NUMCHANS != chanIndexWithMaxPri)
    {
        *pChanHandle = (DDC_I2cDriverObject *)(&
                        ( instHandle->chans[chanIndexWithMaxPri].ddc_Handle ));
    }
    else
    {
        *pChanHandle = NULL;
    }
}


/*!
 *  \brief This function is called from the Interrupt Handler after the packet
 *   is processed(Send/Receive).
 *  This function does the following --
 *     1.Update the Size and status of the processed pkt
 *     2.Calls the DDC layer call back.
       3.Gets the Next channel which is having IOP to send/receive.
 *  \param  i2cObj      [IN]    I2C Port Object
 *  \return returns the channel which is gaving the IO packets to perform.
 */




static Void i2cCompleteIOInIsr (DDC_I2cObject *i2cObj)
{

    DDC_I2cDriverObject         *chanHandle = NULL;
    PSP_i2cDataParam            *dataParam  = NULL;
    DDC_I2cIOP                  *ioPacket   = NULL;
    PSP_I2cRequest              pspRequest  ={0};


    if (i2cObj == ((DDC_I2cObject *) NULL))
    {
        return ;
    }
    i2cObj->i2cRegs->ICIMR = CSL_I2C_ICIMR_RESETVAL;

    /* Check the Pending State   TBD to test                                  */
    if (TRUE == i2cObj->Cancel_Pending_IO)
    {
        /* Error of Cancel IO                                                 */
      i2cObj->currentActiveChannel->activeIOP->status= PSP_I2C_CANCEL_IO_ERROR;
      i2cObj->Cancel_Pending_IO                           = FALSE;
      i2cObj->currentActiveChannel->activeIOP->actualSize=i2cObj->currBufferLen;
      i2cObj->pendingState = FALSE;
        /*set IOP error with proper error raise value TBD                     */
    }
    else
    {
      I2c_localEnsureIsReadyforNext(i2cObj);

       if( i2cObj->currError == PSP_SOK )
        i2cObj->currentActiveChannel->activeIOP->actualSize =
           i2cObj->currentActiveChannel->activeIOP->dataparam->i2cTrans.bufLen;
       else
       {
        i2cObj->currentActiveChannel->activeIOP->actualSize = 0;
        i2cObj->currentActiveChannel->activeIOP->status     = i2cObj->currError;
       }
        i2cObj->pendingState=FALSE;
    }

    /* call the application completion callback function registered           *
     * with us during opening of the channel                                  */
    if (NULL != i2cObj->currentActiveChannel->callBack)
    {

         DDC_copyIOPToRequest(i2cObj->currentActiveChannel->activeIOP,
                                (PSP_I2cRequest *)&pspRequest);

        /* Invoke Application callback for this channel                       */
         (*i2cObj->currentActiveChannel->callBack)((Ptr)
                                 i2cObj->currentActiveChannel->appHandle,NULL,
                                 (&pspRequest));
    }


    PAL_osListEnqueue(&(i2cObj->currentActiveChannel->queueFreeList),
                             (i2cObj->currentActiveChannel->activeIOP) );

    i2cObj->currentActiveChannel->activeIOP = NULL;

    /*get the channel with highest priority                                  */

    I2c_localGetNextChannel(i2cObj->currentActiveChannel->appHandle,
                                 &(i2cObj->currentActiveChannel));

    chanHandle = i2cObj->currentActiveChannel;

    /* check and load next pending packet                                    */
    if (NULL != chanHandle)
    {
        /* we have atleast one packet                                        */
        ioPacket = (DDC_I2cIOP *)PAL_osListDequeue(&chanHandle->queueReqList);

        /* validate and update the iop                                       */
        if (NULL  != ioPacket)
        {
            chanHandle->activeIOP = ioPacket;
            dataParam = ioPacket->dataparam;
            chanHandle->tempICMDRValue =0;
            I2c_localPrepareForTransfer(i2cObj);

            /* Set the current buffer params correctly                       */
            i2cObj->currError     = 0;
            i2cObj->currFlags     = dataParam->i2cTrans.flags;
            i2cObj->currBuffer    = (dataParam->i2cTrans.buffer);
            i2cObj->currBufferLen = (dataParam->i2cTrans.bufLen);

            /* Enable interrupts                                             */
            i2cObj->i2cRegs->ICIMR =
                (CSL_I2C_ICIMR_AL_MASK |    /* Arbitration lost              */
                CSL_I2C_ICIMR_NACK_MASK |   /* NACK received/not received    */
                CSL_I2C_ICIMR_AAS_MASK);    /* Addressed as Slave            */

            if ((dataParam->i2cTrans.flags & PSP_I2C_WRITE) != NULL)
            {
                /* Transmit ready                                            */
                i2cObj->i2cRegs->ICIMR |=
                    CSL_I2C_ICIMR_ICXRDY_MASK;

            }
            else if((dataParam->i2cTrans.flags & PSP_I2C_READ) != NULL)
            {
                /* Receive ready                                             */
                i2cObj->i2cRegs->ICIMR |=
                    CSL_I2C_ICIMR_ICRRDY_MASK;
            }
            /* Start I2C transaction                                         */
            /* Clear IVR before doing a start                                */
            i2cObj->i2cRegs->ICIVR =
                CSL_I2C_ICIVR_INTCODE_RESETVAL;

            /* Start bit set here to start transaction                       */
            i2cObj->i2cRegs->ICMDR |=
             chanHandle->tempICMDRValue;

        }


    }

}


/*!
 *  \brief This function is called from the Interrupt Handler after the packet
 *   is processed(Send/Receive).
 *  This function does the following --
 *     1.Update the Size and status of the processed pkt
 *     2.Calls the DDC layer call back.
       3.Gets the Next channel which is having IOP to send/receive.
 *  \param  i2cObj      [IN]    I2C Port Object
 *  \return returns the channel which is gaving the IO packets to perform.
 */




static void I2c_localPrepareForTransfer(DDC_I2cObject *i2cObj)
{

    DDC_I2cDriverObject         *chanHandle = i2cObj->currentActiveChannel;
    PSP_i2cDataParam            *dataParam  =
                            i2cObj->currentActiveChannel->activeIOP->dataparam;


    /* Set count for data transfer and init mode control register            */
    i2cObj->i2cRegs->ICCNT =
        (dataParam->i2cTrans.bufLen & DDC_I2C_BUFFER_MAXLENGTH);

    /* Check flags and set accordingly                                       */
    /* Except for start and stop bit others can be
     * written directly on to the register                                   */
    if ((dataParam->i2cTrans.flags & PSP_I2C_MASTER)  != 0)
    {
        i2cObj->i2cRegs->ICMDR |=
            CSL_I2C_ICMDR_MST_MASK;
    }
    else
    {
        i2cObj->i2cRegs->ICMDR &=
            ~CSL_I2C_ICMDR_MST_MASK;
    }
    if ((dataParam->i2cTrans.flags & PSP_I2C_START)  != 0)
    {
        chanHandle->tempICMDRValue |= CSL_I2C_ICMDR_STT_MASK;
    }
    if ((dataParam->i2cTrans.flags & PSP_I2C_STOP) != 0)
    {
        chanHandle->tempICMDRValue |= CSL_I2C_ICMDR_STP_MASK;
    }
    if ((dataParam->i2cTrans.flags & PSP_I2C_WRITE) != 0)
    {
        i2cObj->i2cRegs->ICMDR |=
            CSL_I2C_ICMDR_TRX_MASK;
    }
    if ((dataParam->i2cTrans.flags & PSP_I2C_READ) != 0)
    {
        i2cObj->i2cRegs->ICMDR &=
            ~CSL_I2C_ICMDR_TRX_MASK;
    }

    if ((dataParam->i2cTrans.flags & PSP_I2C_REPEAT) != 0)
    {
        chanHandle->tempICMDRValue |= CSL_I2C_ICMDR_RM_MASK;
    }
    else
    {
        i2cObj->i2cRegs->ICMDR &=
            ~CSL_I2C_ICMDR_RM_MASK;
    }

    /* Write slave address - restrict to 10 bits                             */
    i2cObj->i2cRegs->ICSAR =
        (((Uint)dataParam->i2cTrans.slaveAddr) & DDC_I2C_SLAVEADRR_10BIT);

    /* Check if restart required to set the bus busy                         */
    if ((dataParam->i2cTrans.flags & PSP_I2C_RESTART) != 0)
    {
        i2cObj->i2cRegs->ICSTR |=
            CSL_I2C_ICSTR_BB_MASK;
        chanHandle->tempICMDRValue |= CSL_I2C_ICMDR_STT_MASK;
    }

    /* To make transaction is in progress                                    */
    chanHandle->pi2cInstHandle->pendingState = TRUE;
    chanHandle->pi2cInstHandle->currBuffer   = NULL;
}


static Void I2c_localEnsureIsReadyforNext (DDC_I2cObject *i2cObj)
{
    Uint32           counter   = 0;
    Uint32           delay     = 0;
    PSP_i2cDataParam   *dataParam = NULL;

    dataParam = i2cObj->currentActiveChannel->activeIOP->dataparam;


    /* If Bus is Busy after successful transmission                           *
     * then check for nack errors                                            */
  if((i2cObj->i2cRegs->ICSTR & CSL_I2C_ICSTR_BB_MASK) == CSL_I2C_ICSTR_BB_MASK)
  {
        /* Running this check two times for making sure that I2C              *
         * bus free before exiting to app                                    */
        for (counter = 0; counter <= 1u; counter ++)
        {
            delay = I2C_POLLED_TIMEOUT;

            /* wait till stop bit is generated                               */
             while ( (((i2cObj->i2cRegs->ICSTR &
                             CSL_I2C_ICSTR_BB_MASK) != 0) &&
                     ((i2cObj->i2cRegs->ICSTR &
                             CSL_I2C_ICSTR_ARDY_MASK) == 0)) &&
                     (delay != 0) )
                    {
                        delay--;
                    }

            if (FALSE != (dataParam->i2cTrans.flags & PSP_I2C_STOP))
            {
                delay = I2C_POLLED_TIMEOUT;
                /* wait till Master bit goes low                            */
                while (((i2cObj->i2cRegs->ICMDR
                    & CSL_I2C_ICMDR_MST_MASK) != FALSE)
                     && (delay != 0))
                {
                    delay--;
                }
            }
            else
            {
               /* wait till ARDY bit is set                                  */
                while (((i2cObj->i2cRegs->ICSTR
                    & CSL_I2C_ICSTR_ARDY_MASK) == 0)
                    && (delay != 0))
                {
                    delay--;
                }
            }
        }
  }

   /* Mask off interrupts                                                    */
    i2cObj->i2cRegs->ICIMR = 0;
}

/*!
 *  \brief IO Control for I2C controller.
 *
 *  This function supports various IOCTLs for the I2C controller.
 *
 *  \param  handle  [IN]    I2C Driver Instance Handle
 *  \param  cmd     [IN]    IOCTL command
 *  \param  cmdArg  [IN]    Arguments, if any, for the command
 *  \param  param   [IN]    IOCTL specific parameter (driver specific).
 *  \return PSP_SOK or PSP Error code.
 *
 */
/* ARGSUSED */
PSP_Result  PSP_i2cIoctl(
                         PSP_Handle      handle,
                         PSP_I2cIoctlCmd cmd,
                         Ptr             cmdArg,
                         Ptr             param
                        )
{
    PSP_Result              retVal      = PSP_SOK;
    PSP_Result              retCode     = PSP_SOK;
    DDC_I2cDriverObject     *pi2cHandle = NULL;
    DDC_I2cObject           *i2cObj     = NULL;
    Bool                    nack        = FALSE;
    Uint32                  cookie      = 0;
    Uint32                  bitrate     = 0;
    Uint32                  psc         = 0;
    Uint32                  clk         = 0;


    /* Check the state of handle */
    if (handle == NULL)
    {
        retVal = (PSP_E_INVAL_PARAM);
    }
    else
    {
        pi2cHandle = (DDC_I2cDriverObject *)handle;
        i2cObj =  pi2cHandle->pi2cInstHandle;
        if (i2cObj == ((DDC_I2cObject *) NULL))
        {
            retVal = (PSP_E_INVAL_PARAM);
        }
    }

    if(retVal == PSP_SOK)
    {
        switch (cmd)
        {
            case PSP_I2C_IOCTL_SET_BIT_RATE:
                if (cmdArg == NULL)
                {
                    retVal = PSP_E_INVAL_PARAM;
                }
                else if(i2cObj->pendingState == TRUE)
                {
                    retVal = PSP_E_INVAL_STATE;
                }
                else
                {
                    PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);
                    bitrate = (*((Uint32*)cmdArg)) ;
                    i2cObj->i2cBusFreq  = bitrate;
                    /* Calculate the Bitrate and load in Register */
                    retCode = i2cCalculateClockSettings(i2cObj , &psc, &clk) ;
                    if(retCode != 0)
                    {
                        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
                        retVal = (PSP_Result) retCode ;
                    }
                    else
                    {
                        /* Put the module in reset */
                        i2cObj->i2cRegs->ICMDR = CSL_I2C_ICMDR_IRS_RESETVAL;

                        /* Load the Prescalar value in Register */
                        i2cObj->i2cRegs->ICPSC = psc;
                        i2cObj->i2cRegs->ICCLKH = clk/2;
                        i2cObj->i2cRegs->ICCLKL = clk - i2cObj->i2cRegs->ICCLKH;

                        /* Set own address and bits/byte */
                        i2cObj->i2cRegs->ICOAR = i2cObj->i2cOwnAddr;
                        i2cObj->i2cRegs->ICMDR = (i2cObj->numBits &
                                                      DDC_I2C_NUMBITS_MASK);

                        /* Bring module out of reset */
                        i2cObj->i2cRegs->ICMDR |= CSL_I2C_ICMDR_IRS_MASK;
                        /* Set the backward compatibility mode off -
                         * this is critical to get interrupts correctly */
                        i2cObj->i2cRegs->ICEMDR &= (~CSL_I2C_ICEMDR_BCM_MASK);
                        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
                    }
                } /* retVAL */
                break;

            case PSP_I2C_IOCTL_GET_BIT_RATE:
                if (cmdArg == NULL)
                {
                    retVal = PSP_E_INVAL_PARAM;
                }
                else
                {
                    *((Uint32 *)cmdArg) = i2cObj->i2cBusFreq ;
                }
                break;

            case PSP_I2C_IOCTL_CANCEL_PENDING_IO:
                /* Start of Critical Section */
                PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);
                /* Calling this when no io is in transmission */
                if(i2cObj->pendingState == TRUE)
                {
                    /* to cancel io in ddc*/
                    i2cObj->Cancel_Pending_IO = TRUE;
                }
                /* End of Critical Section */
                PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
                break;

            case PSP_I2C_IOCTL_BIT_COUNT:
                if (cmdArg == ((Uint32 *) NULL))
                {
                    retVal = (PSP_E_INVAL_PARAM);
                }
                else
                {
                    /* reset bitcount before updating */
                    i2cObj->i2cRegs->ICMDR &= ~DDC_I2C_NUMBITS_MASK;
                    /* Set the bit count value */
                    i2cObj->i2cRegs->ICMDR |= ( *((Uint32 *)cmdArg)
                                                    & DDC_I2C_NUMBITS_MASK);
                }
                break;

            case PSP_I2C_IOCTL_NACK:
                if (cmdArg == ((Uint32 *) NULL))
                {
                    retVal = (PSP_E_INVAL_PARAM);
                }
                else
                {
                    nack = (*((Bool*)cmdArg)) ;
                    if(nack == 1)
                    {
                        /* set the nack mode, MSB of ICMDR*/
                        i2cObj->i2cRegs->ICMDR = i2cObj->i2cRegs->ICMDR
                                                 | CSL_I2C_ICMDR_NACKMOD_MASK;
                    }
                    else
                    {
                        /* disable the nack mode, */
                        i2cObj->i2cRegs->ICMDR = i2cObj->i2cRegs->ICMDR
                                               & ~(CSL_I2C_ICMDR_NACKMOD_MASK);
                    }
                }
                break;

            case PSP_I2C_IOCTL_SET_OWN_ADDR:
                if (cmdArg == ((Uint32 *) NULL))
                {
                    retVal = PSP_E_INVAL_PARAM;
                }
                else if(i2cObj->pendingState == TRUE)
                {
                    retVal = PSP_E_INVAL_STATE;
                }
                else
                {
                    i2cObj->i2cOwnAddr      = *((Uint32 *)cmdArg);
                    i2cObj->i2cRegs->ICOAR  = i2cObj->i2cOwnAddr;
                }
                break;
            case PSP_I2C_IOCTL_GET_OWN_ADDR:
                if (cmdArg == ((Uint32 *) NULL))
                {
                    retVal = PSP_E_INVAL_PARAM;
                }
                else
                {
                    *((Uint32 *)cmdArg) = i2cObj->i2cOwnAddr;
                }
                break;

            default:
                retVal = PSP_E_INVAL_PARAM;
                break;
        }
    } /* retVal */
    return retVal ;
}
/*!
 *  \brief  DDC Layer function used to transfer the data
 *
 *  This is the function in which a dual data transfer happens.Function
 *  validates input parameters,set the parameter values in to the registers
 *  and starts transmission.
 *
 *  \param  handle      [IN]    I2C Driver Instance Handle
 *  \param  slaveAddr   [IN]    Slave address (only valid in Master mode)
 *  \param  buffer      [IN]    Buffer (to read into or write from)
 *  \param  bufLen      [IN]    Buffer length - number of bytes to transfer
 *  \param  flags       [IN]    Flags to specify mode of operation
 *  \param  timeout     [IN]    Timeout for i2c transaction
 *  \param  param       [IN]    Extra paramter for future usage
 *  \param  xferActual  [IN]    Number of bytes actually transferred
 *
 * \return  Success  Number of bytes transferred.
 *              Other    Errors
 *
 */
/* ARGSUSED */

static Int DDC_i2cTransferIOP (PSP_Handle handle,DDC_I2cIOP *ioPacket,Ptr param)

{
    DDC_I2cObject     *i2cObj  = NULL;
    Uint32             cnt     = 0;
    Uint32             icmdr   = 0;
    Uint32             status  = 0;
    PSP_Result         retCode = PAL_SOK;
    volatile Uint32    delay   = 0;
    Int                retVal  = PAL_SOK;
    Uint32             timeCnt = 0;
    Uint32     slaveAddr = ioPacket->dataparam->i2cTrans.slaveAddr;
    Uint8      *buffer   = ioPacket->dataparam->i2cTrans.buffer;
    Uint32     bufLen    = ioPacket->dataparam->i2cTrans.bufLen;
    Uint32     flags     = ioPacket->dataparam->i2cTrans.flags;
    Int        timeout   = ioPacket->dataparam->timeout;
    Ptr     xferActual = &(ioPacket->actualSize);



    /* Check the state of handle */
    if (((handle == NULL) || (buffer == NULL)) )
    {
        retVal = (PSP_E_INVAL_PARAM);
    }
    else
    {
        i2cObj =  (DDC_I2cObject *) handle;
        if (i2cObj == ((DDC_I2cObject *) NULL))
        {
            retVal = (PSP_E_INVAL_PARAM);
        }
        /* Check user has not set both read and write flag at same time*/
        else if (((flags & PSP_I2C_READ) != 0)
                                && ((flags & PSP_I2C_WRITE) != 0))
        {
            retVal = (PSP_E_INVAL_PARAM);
            i2cObj->currentActiveChannel = NULL;
        }
        else
        {
            /* Check for Bus Busy if not restarting */
            if (((flags & PSP_I2C_RESTART) == 0)
                                && ((flags & PSP_I2C_IGNORE_BUS_BUSY) == 0))
            {
                /* Loop on busy condition */
                if (i2cObj->mode == PSP_OPMODE_POLLED)
                {
                    /*Check for Bus Busy */
                    retCode = i2cWaitForBB(i2cObj,
                                           DDC_I2C_BUS_BUSY_TIMEOUT_CNT,
                                           0);
                    if ( retCode != PAL_SOK )
                    {
                        retVal = (PSP_I2C_BUS_BUSY_ERR);
                        i2cObj->currentActiveChannel = NULL;
                    }
                }
                else
                {
                    /*Check for Bus Busy */
                    retCode = i2cWaitForBB(
                                           i2cObj,
                                           DDC_I2C_BUS_BUSY_TIMEOUT_MSECS,
                                           1u
                                           );

                    if ( retCode != PAL_SOK )
                    {
                        retVal = (PSP_I2C_BUS_BUSY_ERR);
                        i2cObj->currentActiveChannel = NULL;
                    }
                }
            }
        } /* else */
    } /* retVal   */
    if (PAL_SOK == retVal)
    {
#ifdef I2C_INSTRUMENTATION_ENABLED
    if(0 == (flags & PSP_I2C_WRITE))
    {
        PSP_LOG_EVENT(  &DVTEvent_Log,
                        "I2C",
                        PSP_DVT_DESC(   PSP_DVT_eRCV_START,
                                        PSP_DVT_dCHANNEL_ID_INST,
                                        PSP_DVT_dPACKET_ID,
                                        PSP_DVT_dNONE),
                         CHANNELID_INSTANCEID(
                            PSP_I2C_READ,
                            i2cObj->instanceId),
                         buffer);
    }
    else
    {
        PSP_LOG_EVENT(  &DVTEvent_Log,
                        "I2C",
                        PSP_DVT_DESC(   PSP_DVT_eDATA_SND_START,
                                        PSP_DVT_dCHANNEL_ID_INST,
                                        PSP_DVT_dPACKET_ID,
                                        PSP_DVT_dNONE),
                         CHANNELID_INSTANCEID(
                            PSP_I2C_WRITE,
                            i2cObj->instanceId),
                         buffer);
    }
#endif/*#ifdef I2C_INSTRUMENTATION_ENABLED*/
        /* Set count for data transfer and init mode control register */
        i2cObj->i2cRegs->ICCNT = (bufLen & DDC_I2C_BUFFER_MAXLENGTH);

        /* Check flags and set accordingly */
        /* Except for start and stop bit others can be
         * written directly on to the register */
        if ((flags & PSP_I2C_MASTER)  != 0)
        {
            i2cObj->i2cRegs->ICMDR |= CSL_I2C_ICMDR_MST_MASK;
        }
        else
        {
            i2cObj->i2cRegs->ICMDR &= ~CSL_I2C_ICMDR_MST_MASK;
        }
        if ((flags & PSP_I2C_START)  != 0)
        {
            icmdr |= CSL_I2C_ICMDR_STT_MASK;
        }
        if ((flags & PSP_I2C_STOP) != 0)
        {
            icmdr |= CSL_I2C_ICMDR_STP_MASK;
        }
        if ((flags & PSP_I2C_WRITE) != 0)
        {
            i2cObj->i2cRegs->ICMDR |= CSL_I2C_ICMDR_TRX_MASK;
        }
        if ((flags & PSP_I2C_READ) != 0)
        {
            i2cObj->i2cRegs->ICMDR &= ~CSL_I2C_ICMDR_TRX_MASK;
        }

        if ((flags & PSP_I2C_REPEAT) != 0)
        {
            icmdr |= CSL_I2C_ICMDR_RM_MASK;
        }
        else
        {
            i2cObj->i2cRegs->ICMDR &= ~CSL_I2C_ICMDR_RM_MASK;
        }

        /* Write slave address - restrict to 10 bits */
        i2cObj->i2cRegs->ICSAR = (((Uint)slaveAddr) & DDC_I2C_SLAVEADRR_10BIT);
        /* Check if restart required to set the bus busy */
        if ((flags & PSP_I2C_RESTART) != 0)
        {
            i2cObj->i2cRegs->ICSTR |= CSL_I2C_ICSTR_BB_MASK;
            icmdr |= CSL_I2C_ICMDR_STT_MASK;
        }
        /* To make transaction is in progress */
        i2cObj->pendingState = TRUE;
        i2cObj->currBuffer   = 0;

/*================ POLLED MODE CODE STARTS ==================================*/

        /* If polled mode, wait for data transfer and completion,
         * else wait (on a semaphore) for transaction to complete */
        if (i2cObj->mode == PSP_OPMODE_POLLED)
        {
            i2cObj->i2cRegs->ICIMR = 0;        /* Mask all interrupts - :)
                                                * just to make sure */
            i2cObj->i2cRegs->ICMDR |= icmdr;    /* Start bit set here to
                                                       * start transaction */
            delay = I2C_POLLED_TIMEOUT;
            /* wait for start bit to get generated only when start bit set*/
            if ((flags & PSP_I2C_START)  != 0)
            {
                while ( ((i2cObj->i2cRegs->ICSTR &
                                CSL_I2C_ICSTR_BB_MASK) == 0) &&
                        (delay != 0) )
                {
                    delay--;
                }
            }

            /** set this value to exit from I2C when the delay expires as
              * stops responding hardware
              */
            delay = I2C_POLLED_TIMEOUT;

            timeCnt = PAL_osGetCurrentTick();
            while (cnt < bufLen)
            {
                if ( (FALSE == PAL_osCheckTimeOut(timeCnt,timeout))
                                                 && (PAL_SOK == retCode) )
                {
                    delay --;
                    if(0 == delay)
                    {
                        retCode = PSP_E_TIMEOUT;
                    }

                    if ((flags & PSP_I2C_WRITE) != NULL)
                    {
                        /* Transmit */
                        if ((i2cObj->i2cRegs->ICSTR
                                            & CSL_I2C_ICSTR_ICXRDY_MASK) != 0)
                        {
                            i2cObj->i2cRegs->ICDXR = (Uint32)*buffer;
                            ++buffer;
                            ++cnt;
                            delay = I2C_POLLED_TIMEOUT;
                        }
                    }
                    else
                    {
                        /* Receive */
                        if ((i2cObj->i2cRegs->ICSTR
                                            & CSL_I2C_ICIMR_ICRRDY_MASK) != 0)
                        {
                            *buffer = (Uint8)i2cObj->i2cRegs->ICDRR;
                            ++buffer;
                            ++cnt;
                            delay = I2C_POLLED_TIMEOUT;
                        }
                    }
                    /* Check for Arbitration loss, transmit underflow and
                     * receive overflow errors */
                    status = i2cObj->i2cRegs->ICSTR;
                    if ((status & CSL_I2C_ICSTR_AL_MASK) != 0)
                    {
                        retCode = PSP_I2C_ARBITRATION_LOSS_ERR;
                    }
                    else if ((status & CSL_I2C_ICSTR_RSFULL_MASK) != 0)
                    {
                        retCode = PSP_I2C_RECEIVE_OVERFLOW_ERR;
                    }
                    /* Check for NACK error -
                     */
                    else
                    {
                        if ((status & CSL_I2C_ICSTR_NACK_MASK) != 0)
                        {
                            if ((flags & PSP_I2C_IGNORE_NAK) != 0){
                                retCode = PSP_SOK;
                            }else{
                                retCode = PSP_I2C_NACK_ERR;
                            }
                        }
                    }
                    /* check for cancelling IO */
                    if (TRUE == i2cObj->Cancel_Pending_IO)
                    {
                        i2cObj->Cancel_Pending_IO = FALSE;
                        retCode = PSP_I2C_CANCEL_IO_ERROR;
                    }/* Cancel IO */
                }
                else
                {
                    if (retCode == PSP_SOK)
                    {
                        retCode = PSP_E_TIMEOUT;
                    }
                    break;
                }
            } /* While number of byte to be transmitted/received loop */

            if (0 != retCode)
            {
                i2c_cancelIO(i2cObj,retCode);
                retVal = retCode;
            }
            *(Uint32 *)xferActual = cnt;
            i2cObj->pendingState=FALSE;
            i2cObj->currentActiveChannel = NULL;
        } /* if polled mode */

   /*================ POLLED MODE CODE ENDS==================================*/

        else
        {
    /*========== INTERUPPT MODE CODE STARTS==================================*/
            /* Set the current buffer params correctly */
            i2cObj->currError = 0;
            i2cObj->currFlags = flags;
            i2cObj->currBuffer = (buffer+cnt);
            i2cObj->currBufferLen = (bufLen-cnt);

           if ( ((flags & PSP_I2C_STOP) != 0)  &&
                        ((flags & PSP_I2C_REPEAT) != 0) )
            {
                /* Enable interrupts */
                i2cObj->i2cRegs->ICIMR = (  CSL_I2C_ICIMR_AL_MASK |
                                            /* Arbitration lost              */
                                            CSL_I2C_ICIMR_NACK_MASK |
                                            /* NACK received/not received    */
                                            CSL_I2C_ICIMR_SCD_MASK |
                                            CSL_I2C_ICIMR_AAS_MASK);
                                            /* Addressed as Slave            */


            }
            else
            {

                /* Enable interrupts */
                i2cObj->i2cRegs->ICIMR = (  CSL_I2C_ICIMR_AL_MASK |
                                            /* Arbitration lost              */
                                            CSL_I2C_ICIMR_NACK_MASK |
                                            /* NACK received/not received    */
                                            CSL_I2C_ICIMR_AAS_MASK);
                                           /* Addressed as Slave            */
             }

            if ((flags & PSP_I2C_WRITE) != NULL)
            {   /* Transmit ready        */
                    i2cObj->i2cRegs->ICIMR |= CSL_I2C_ICIMR_ICXRDY_MASK ;

            }
            else if((flags & PSP_I2C_READ) != NULL)
            {   /* Receive ready */
                    i2cObj->i2cRegs->ICIMR |= CSL_I2C_ICIMR_ICRRDY_MASK ;
            }
            /* Start I2C transaction */
            /* Clear IVR before doing a start           */
            i2cObj->i2cRegs->ICIVR = CSL_I2C_ICIVR_INTCODE_RESETVAL;
            /* Start bit set here to start transaction  */
            i2cObj->i2cRegs->ICMDR |= icmdr;

            retVal = PSP_SINPROGRESS;

        }


#ifdef I2C_INSTRUMENTATION_ENABLED
    if((flags & PSP_I2C_WRITE) == 0)
    {
        PSP_LOG_EVENT(  &DVTEvent_Log,
                        "I2C",
                        PSP_DVT_DESC(   PSP_DVT_eRCV_END,
                                        PSP_DVT_dCHANNEL_ID_INST,
                                        PSP_DVT_dPACKET_ID,
                                        PSP_DVT_dNONE),
                         CHANNELID_INSTANCEID(
                            PSP_I2C_READ,
                            i2cObj->instanceId),
                         buffer);
    }
    else
    {
        PSP_LOG_EVENT(  &DVTEvent_Log,
                        "I2C",
                        PSP_DVT_DESC(   PSP_DVT_eDATA_SND_END,
                                        PSP_DVT_dCHANNEL_ID_INST,
                                        PSP_DVT_dPACKET_ID,
                                        PSP_DVT_dNONE),
                         CHANNELID_INSTANCEID(
                            PSP_I2C_WRITE,
                            i2cObj->instanceId),
                         buffer);
    }
#endif/*#ifdef I2C_INSTRUMENTATION_ENABLED*/

    } /* retVal */

    return retVal;
}

/*!
 *\brief  As per I2C specs the following formulas provide presalar and low/high
 *          divider values
 *
 * input clk --> PSC Div -----------> ICCL/H Div --> output clock
 *                       module clk
 *
 * output clk = module clk / (PSC + 1) [ (ICCL + d) + (ICCH + d) ]
 *
 * Thus,
 * ICCL = ICCH = clk = input clk / ( (psc +1) * output clk * 2 ) ;
 *
 * where if PSC == 0, d = 7,
 *       if PSC == 1, d = 6
 *       if PSC > 1 , d = 5
 *
 *  \param   *obj   [IN]   Pointer to I2c handle
 *  \param   *psc   [IN]   Pointer that points to prescalar value
 *  \param   *clock [IN]   Pointerst that points to the calculated clock value
 *
 *  \return  success or negative error code
 */

static Int i2cCalculateClockSettings(
                              const DDC_I2cObject *obj,
                              Uint32 *psc,
                              Uint32 *clock
                             )
{
    Uint32 prescalar   = 0;
    Uint32 D           = 0;
    Int    clk         = 0;
    Int    retVal      = 0;

    if ( ( (obj == NULL) || (psc == (Uint32 *)NULL) ) ||
                    ( (clock == (Uint32 *)NULL) || (obj == NULL) ) )
    {
        retVal = (PSP_E_INVAL_PARAM);
    }
    else
    {
        /* Divisor 12MHz is I2C perpheral module clock frewuency */
        prescalar = (obj->moduleInputClkFreq/12000000u) - 1u;

        /* Select the D Value as per Prescalar Value */
        if(prescalar > 1u)
        {
            D= 5u;
        }
        if(prescalar == 1u)
        {
            D= 6u;
        }
        if(prescalar == 0)
        {
            D= 7u;
        }
        clk = (int)(obj->moduleInputClkFreq /
                            ( (prescalar +1u) * obj->i2cBusFreq )) - (2u*D);
        if( clk <= 0)
        {
            /* CLKL/CLKH are gettting 0 or negative values - flag error */
            /* Cannot go so high in input freq */
            retVal = ( PSP_E_INVAL_PARAM);
        }
        /* Pass the prescalar value */
        *psc = prescalar ;
        *clock = (Uint32)clk ;
    }
    return retVal;
}
/*!
 * \brief Cancel pending IO operations
 *
 *  This function is used to cancel the pending IO operations.
 *
 *  \param  *i2cObj    Pointer to I2c Handle
 *  \param  retCode    error during transfer operation
 */
static void i2c_cancelIO(DDC_I2cObject *i2cObj,Int retCode)
{
    /* local variables to store register contents */
    Uint32 psc      = 0;
    Uint32 clkh     = 0;
    Uint32 clkl     = 0;
    Uint32 oar      = 0;
    volatile Uint32 icmdr    = 0;
    volatile Uint32 delay    = I2C_POLLED_TIMEOUT;
    volatile Uint8  buffer   = 0x00;
    Uint32 counter  = 0;

    if (i2cObj == NULL)
    {
        return;
    }
    else
    {
        /* Store the register values into temporary variables */
        icmdr = i2cObj->i2cRegs->ICMDR & DDC_I2C_VALUE_RESTORE;
        psc   = i2cObj->i2cRegs->ICPSC;
        clkh  = i2cObj->i2cRegs->ICCLKH;
        clkl  = i2cObj->i2cRegs->ICCLKL;
        oar   = i2cObj->i2cRegs->ICOAR;
    /* Wair for read / write to complete only when there is no I2C bus error */
        if ( (retCode == PSP_E_TIMEOUT) &&
                ((i2cObj->i2cRegs->ICSTR & CSL_I2C_ICSTR_BB_MASK) != 0) )
        {
            for (counter=0;counter<1u;counter++)
            {
                if ( (i2cObj->i2cRegs->ICMDR & CSL_I2C_ICMDR_TRX_MASK) == 0 )
                {
                  while ( ((i2cObj->i2cRegs->ICSTR & CSL_I2C_ICIMR_ICRRDY_MASK)
                              == 0) ||    (delay == 0) )
                    {
                        delay--;
                    }
                }
            }
        }

        /* Put the module in reset  */
        i2cObj->i2cRegs->ICMDR = CSL_I2C_ICMDR_IRS_RESETVAL;

        /* Wait till status register reads its default value */
        delay = I2C_POLLED_TIMEOUT;
        /** wait till stop bit is generated */
        while ( (i2cObj->i2cRegs->ICSTR != CSL_I2C_ICSTR_RESETVAL) &&
                        (delay != 0) )
        {
            delay--;
        }

        /* Load the Counter register with zero value */
        i2cObj->i2cRegs->ICCNT = CSL_I2C_ICCNT_RESETVAL;
        /* Restore the value from temporary variables to register */
        i2cObj->i2cRegs->ICPSC   = psc;
        i2cObj->i2cRegs->ICCLKH  = clkh;
        i2cObj->i2cRegs->ICCLKL  = clkl;
        i2cObj->i2cRegs->ICOAR   = oar;
        /* Bring module out of reset */
        i2cObj->i2cRegs->ICMDR   = icmdr;
        /*  Set the backward compatibility mode off -this is critical to get
            interrupts correctly */
        i2cObj->i2cRegs->ICEMDR &= (~CSL_I2C_ICEMDR_BCM_MASK);
    }
}

/*!
 * Revision History
 * ================
 *  \version    0.1     Implementation on Davinci      Anat Gole
 *  \version    0.2     Modified for DRA44X            Nagarjuna Kristam
 *  \version    0.3     Added instrumentation          Maulik Desai
 *  \version    0.4     Modified for DM648             Nagarjuna Kristam
 *  \version    0.5     MISRAC and Formatting          Nagarjuna Kristam
 *  \version    0.6     Implementing review commnets   Nagarjuna Kristam
 *  \version    0.7     Merged with DM6437 added
                        instrumentation details        Nagarjuna Kristam
 */

