/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/
/*!
    @{

*/

/** \mainpage    PSP I2C Driver on BIOS
 *
 *  \section    Introduction
 *
 *  \subsection    xxx    Purpose and Scope
 *  The purpose of this document is to explain the I2C driver on the PSP
 *  Framework architecture written in DSP/BIOS
 *
 *  \subsection    aaa    PSP Architecture
 *  A PSP framework is a software entity providing platform support
 *  for devices that are based on Texas Instruments SoC's. The specific
 *  (Hardware and software) implementation of the PSP framework (the PAL
 *  implementation and the group of device drivers) is referred to as
 *  PSP System. The PSP framework defines a software environment for device
 *  drivers to plug-in by defining standard usage interfaces and
 *  abstractions (OS, CPU/Peripherals). For every platform, the PSP
 *  implements platform specific functionality such as OS adaptation, CPU and
 *  peripherals specific mechanisms, and generic system services such as
 *  debug control,traces, persistent storage, etc.
 *
 *  \subsection    bbb    I2C Controller
 *  The I2C bus is a multi master bus which allows more than one device capable
 *  of controlling the bus to be connected to it. Each I2C device is recognized
 *  by a unique address and can operate as either transmitter or receiver
 *  depending on the function of the device. In addition to being a transmitter
 *  or receiver, device connected to the I2C bus can also be considered as
 *  master or slave when performing data transfers. Note that a master device
 *  is the device which initiates a data transfer on the bus and generates
 *  the clock signals to permit that transfer. During this transfer, any device
 *  addressed by this master is considered a slave.
 *
 *  \subsection    ccc    DSP/BIOS System
 *  DSP/BIOS is a real time kernel provided by TI to use with its
 *  DSP processors.
 *
 *
 *  \subsection    References
 *    -# I2C functional specification, <b>spruek8_I2C.pdf</b>
 *    -# Design Document for I2C driver,
 *                         <b>BIOS_I2C_Driver_Arch.doc</b>
 *    -# PSP Framework Overview, <b>PSPFramework_Overview.pdf</b>
 *    -# DSP/BIOS driver development <b>SPRU616.pdf</b>
 *
 */

/*!
 *  \file   ddc_i2c.h
 *
 *  \brief  I2C Core header file
 *
 *  This file contains the interfaces, data types and symbolic definitions
 *  that are needed by the Device Driver Core to utilize the serivces of the
 *  I2C device driver.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author     PSG
 *
 *  \version    0.4
 */

#ifndef _DDC_I2C_H_
#define _DDC_I2C_H_

/*Header file included */
#include <ti/sdo/pspdrivers/common/psp_common.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osSem.h>
#include <ti/sdo/pspdrivers/drivers/i2c/src/psp_i2cApi.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osList.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osSem.h>


#ifdef CHIP_DM6437
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/soc.h>
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/cslr_i2c.h>
#endif

#ifdef CHIP_C6424
#include <ti/sdo/pspdrivers/soc/c6424/dsp/soc.h>
#include <ti/sdo/pspdrivers/soc/c6424/dsp/cslr_i2c.h>
#endif

#ifdef CHIP_DM648
#include <ti/sdo/pspdrivers/soc/dm648/dsp/soc.h>
#include <ti/sdo/pspdrivers/soc/dm648/dsp/cslr_i2c.h>
#endif

#ifdef CHIP_C6452
#include <ti/sdo/pspdrivers/soc/c6452/dsp/soc.h>
#include <ti/sdo/pspdrivers/soc/c6452/dsp/cslr_i2c.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

/** Min input module clk frequencies(1MHz)                          */
#define DDC_I2C_MIN_INPUT_FREQ          (1000000u)
/** Max input module clk frequencies(100MHz)                        */
#define DDC_I2C_MAX_INPUT_FREQ          (150000000u)

/** Min output i2c bus frequencies(10KHz)                           */
#define DDC_I2C_MIN_BUS_FREQ            (10000u)
/** Max output i2c bus frequencies(400KHz)                          */
#define DDC_I2C_MAX_BUS_FREQ            (400000u)
/** Bus Busy Timeout - if polled mode, this shall be a counter
    Should be based upon CPU speed                                  */
#define DDC_I2C_BUS_BUSY_TIMEOUT_CNT    (10000000u)
/** Bus Busy Timeout if interrupt this will be in mSecs(10mSec)     */
#define DDC_I2C_BUS_BUSY_TIMEOUT_MSECS  (10u)
/** Min Number of Bits                                              */
#define DDC_I2C_MIN_NUM_BITS            (2u)
/** Max Number of Bits                                              */
#define DDC_I2C_MAX_NUM_BITS            (8u)
/** Bit Mask for Number of Bits to be sent/received                 */
#define DDC_I2C_NUMBITS_MASK            (0x7u)
/** Maximum Buffer Length to be transfered                          */
#define DDC_I2C_BUFFER_MAXLENGTH        (0xFFFFu)
/** Slave Address Bit Mask to 10 Bit                                */
#define DDC_I2C_SLAVEADRR_10BIT         (0x3FFu)
/** Reset I2C Device                                                */
#define DDC_I2C_RESET                   (0x0020u)
/** Restore the Value                                               */
#define DDC_I2C_VALUE_RESTORE           (0x0000D1FFu)
/** Register Access Ready Interrupt Enable Bit                      */
#define DDC_ICIVR_INTCODE_ARDY          (0x00000004u)
/**  prescalar value                                                */

/*!
 * \brief State of I2C Driver
 *
 *  These enums are used to maintain the current state of the driver
 */
typedef enum DDC_I2CState_t
{
    I2C_DELETED = 0u,      /**< Instance Deleted   */
    I2C_CREATED,           /**< Instance Created   */
    I2C_OPENED,            /**< Instance Open      */
    I2C_CLOSED             /**< Instance Closed    */
} DDC_I2CState;

/*!
 *  \brief DDC I2C Object
 *
 *  This structure basically holds the hardware configuration for each instance.
 *  One such storage exists for each instance of the I2C peripheral.
 */
typedef struct DDC_I2cObject_t
{
    /* SOC Related Details      */
    CSL_I2cRegs                      *i2cRegs;            /**< I2C peripheral register map  */
    Uint32                           intNum;              /**< Interrupt number             */

    /* I2C Peripheral details  */
    PSP_OpMode                       mode;                /**< Driver operating mode - polled,
                                                               interrupt      */
    Uint32                           i2cOwnAddr;          /**< Own address (7 or 10 bit)    */
    Uint32                           moduleInputClkFreq;  /**< Module input clock freq      */
    Uint32                           i2cBusFreq;          /**< I2C Bus Frequency            */
    Uint32                           numBits;             /**< Number of bits/byte to be
                                                         sent/received        */
    Bool                             addressing;          /**< 7bit/10bit Addressing mode   */
    Bool                             dlb;                 /**< Digital Loob Back (DLB) mode */

    /* Driver internal members */
    Int                              instanceId;          /**< Instance number              */
    DDC_I2CState                     state;               /**< State of driver              */
    Uint32                           numOpens;            /**< Number of active opens       */
    Bool                             pendingState;        /**< Shows whether io is in pending
                                                              state or not    */
    Bool                             Cancel_Pending_IO;   /**< Shows whether IO has to
                                                      cancel or not          */
    /* Driver internal Transmission details */
    Int                              currError;           /**< Error code                   */
    Uint32                           currFlags;           /**< Current Flags for read/write */
    Uint8                            *currBuffer;         /**< User buffer for read/write   */
    Uint32                           currBufferLen;       /**< User buffer length           */

    /* NOTE: Though slave mode is not supported by this driver,incase of any
             addressed-as-slave condition, the driver can call an application
             registered callback.This is provided to indicate an unexpected
             I2C activity.
     */
    /* Driver hardware event callback details */
    PSP_I2cHwEventCallback           hwEventCallback; /**< callback when accessed as
                                                             slave            */
    Ptr                              appSlvHandle;    /**<App handle which will be
                                                 provided while regsitering
                                                 callback and will be returned
                                                 with slave callback          */


    struct DDC_I2cDriverObject_t     *currentActiveChannel;

} DDC_I2cObject;

/**
 * \brief    Default DDC I2C OBJECT values
 */
#define DDC_I2C_OBJECT_DEFAULT                  \
        {                                       \
         /*casl regs*/          NULL,           \
        /* interrupt number   */0,              \
        /* mode       */        PSP_OPMODE_POLLED,  \
        /* own address*/        0,              \
        /*module clk*/          0,              \
        /*bus freq */           0,              \
        /* num bits */          0,              \
        /*addressing*/          0,              \
        /* dlb */               0,              \
        /*instanceId */         0,              \
        /*state*/               I2C_DELETED,    \
        /*numopens*/            0,              \
        /*pending*/             0,              \
        /*cancel io*/           0,              \
        /* error*/              0,              \
        /*flags*/               0,              \
        /*buffer*/              NULL,           \
        /*buf len*/             0,              \
        /* hw even */           0,              \
        /* slv handle*/         NULL            \
        }




/*
 *  \brief  I2c DDC layer request structure
 *  DDC layer request strcture containing request information
 *
 */
typedef struct DDC_I2cIOP_t
{

    PAL_OsListNodeHeader             iopLink;
    /**< Link to the next & previous in the list                        */

    PSP_i2cDataParam                 *dataparam ;

    Uint32                           actualSize;
    /**< Actual Data transferred                                        */

    PSP_Result                       status;
    /**< Status of the Request. Updated by the Driver                   */

    PSP_I2cAppCallback               appCbk;
    /**< McBSP Application call back registered during Open             */

    Ptr                              appPtr;
    /**< Application provided Pointer                                   */

    Ptr                              misc;
    /**< misc parameter                                            */
    Ptr                              chanPtr;
    /** Channel Pointer for the IOP */

} DDC_I2cIOP;






/*!
 *  \brief I2C DDC Driver Object
 *
 */
typedef struct DDC_I2cDriverObject_t
{
    PSP_I2cAppCallback            callBack;       /**< Callback from applications       */
    Ptr                           appHandle;      /**< application handle which will be
                                             returned with callback           */
    DDC_I2cObject*                pi2cInstHandle; /**< i2c Insatnce object handle       */

    PAL_OsListNodeHeader          queueFreeList;
    /**< List containing the Free IO Packets                            */

    PAL_OsListNodeHeader          queueReqList;
    /**< List containing the Requests received from application. Driver
         checks this list to determine wether TX or RX is active        */

    PAL_OsSemHandle               syncSem;
    /**< Semaphore used during Flush Request                            */

    DDC_I2cIOP                    ioRequests[PSP_I2C_MAX_REQ];
    /**< Channel transfer complete code                                 */

    DDC_I2cIOP                    *activeIOP;
    /**< Current data packet being serviced.                */

    volatile Bool                 flush;
    /**< TRUE, if Flush Request is called             */

    Uint32                        tempICMDRValue;
  /* this stores temp ICMDR value from input flags                      */

    Uint32                        taskPriority;
  /*this will hold the priority of the task that created this channel   */

    DDC_I2CState                  channelState;
       /*  state of the I2C channel                                          */



}DDC_I2cDriverObject;

/**
 * \brief    Default DDC I2C DRIVER OBJECT values
 */
#define DDC_I2C_DRIVER_OBJECT_DEFAULT                  \
        {                                       \
         /*call back*/          NULL,           \
        /* handle*/             NULL,           \
        /* driver obj ptr*/     NULL            \
        }
/*!
 * \brief Function implemented by DDC
 *
 * These function are implemented in DDC and performs ISR operations.
 *  \param      *i2cObj     Pointer to the I2c Handle
 */
void DDC_i2cIntrHandler(DDC_I2cObject *i2cObj);

#ifdef __cplusplus
}
#endif

#endif  /*_DDC_I2C_H_ */

/*!
 * Revision History
 * ================
 *  \version    0.1    - previous versions of interfaces   Nagarjuna Kristam
 *  \version    0.2    - cleanup effort                    Maulik Desai
 *  \version    0.3    - MISRAC Formatting and added
 *                       DDC MACROs                        Naagrjuna Kristam
 *  \version    0.4    - Implementing review commnets      Nagarjuna Kristam
 *  \version    0.5    - Clean up of code and added init   Nagarjuna Kristam
                         time macros to intialize default values
 */

