/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2007 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/** \file     psp_mcasp.h
 *
 *  \brief    Header file for McASP mini-driver
 *
 *  This file has the globally visible data structures and definitions
 *  for the McASP mini-driver.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \authors    PSP Team (please mail your comments/ queries to
 *              psparch-feedback@list.ti.com)
 *
 *  \note       Set tabstop to 4 (:se ts=4) while viewing this file in an
 *              editor
 *  \author Platform Support Group
 *
 *  \Revision
 *          0.1  Created
 *          0.2  Created  : Dynamic mode start/stop implemented
 *          0.3  modified : modified to PSP architecture and
 *                          implemented EDMA linking  of next pending packet
 *          0.4  modified : DM64LC - post silicon changes and cleanup
 *
 *  <b>Compiler Switch Used</b>
 *
 *  <b>_DEBUG</b><br>
 *   This is used as a flag to compiler whether to include the debug statements
 *   inserted in the code into the final image. This flag helps to build DEBUG
 *   image of the program. For RELEASE images this is not passed to the compiler.
 *
 *  <b>CHIP_XXXX</b><br>
 *   The CSL layer is written in a common file for all the variants of a SOC. This
 *   flag differentiates the variant we are compiling for, for e.g. CHIP_DM648,
 *   and the CSL definitions for that variant appropriately gets defined for
 *   register base addresses, num of ports of a peripheral etc.
 *
 *  <b>MCASP_INSTRUMENTATION_ENABLED</b><br>
 *   This flag is passed to the compiler to include the instrumentation code parts
 *   into the final image/lib of the program. This helps build the iRelease/iDebug
 *   versions of the image/lib with a common code base.
 *
 */

#ifndef _PSP_MCASP_H
#define _PSP_MCASP_H



#include <ti/sdo/pspdrivers/common/psp_common.h>

#ifdef CHIP_DM648
#include <ti/sdo/pspdrivers/soc/dm648/dsp/tistdtypes.h>
#endif

#ifdef CHIP_C6424
#include <ti/sdo/pspdrivers/soc/c6424/dsp/tistdtypes.h>
#endif

#ifdef CHIP_DM6437
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/tistdtypes.h>
#endif

#ifdef CHIP_C6452
#include <ti/sdo/pspdrivers/soc/c6452/dsp/tistdtypes.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif


/*
Macros that need configuration when no. of mcasp instances
and no. of serialisers in each mcasp instance changes
*/

/** The max number of serializers on McASP port */
#define PSP_MCASP_MAX_SERS       (8u)

/** Support One McASP instance
This macro needs to be changed when porting to new SOC and if the new
SOC contains different no. of  mcasp instances */
#define PSP_MCASP_NUM_PORTS      (1u)
/** I/O flushed. Non-completed read or write requests return w/ IOM_FLUSHED.*/

/** Port can have two channels, one each for IOM_INPUT and IOM_OUTPUT */
#define PSP_MCASP_NUM_CHANS      (2u)

/** Mcasp Word length */
#define PSP_MCASP_WORDLEN_8      (8u)
/** Mcasp Word length */
#define PSP_MCASP_WORDLEN_12     (12u)
/** Mcasp Word length */
#define PSP_MCASP_WORDLEN_16     (16u)
/** Mcasp Word length */
#define PSP_MCASP_WORDLEN_20     (20u)
/** Mcasp Word length */
#define PSP_MCASP_WORDLEN_24     (24u)
/** Mcasp Word length */
#define PSP_MCASP_WORDLEN_32     (32u)



/** Enumeration for operation mode */
typedef enum {

    /** TDM Mode for channel operation */
    PSP_MCASP_TDM_MODE = 0,

    /** DIT Mode for channel operation */
    PSP_MCASP_DIT_MODE = 1

} PSP_McaspOpMode;


/** Forward Declaration */
struct _PSP_mcaspReq_t;


/**
 * \brief Application callback to be called by the driver
 *        when the transmit or recieve is complete
 *
 * \param devId         device id for which the callback is sent
 * \param cbkContext    call back context for application
 * \param buf           pointer to buffer sent for loop job
 * \param xferSize      size of the buffer sent
 * callback function that shall be called from the McASP IOM layer context
 * when the loop job is running
 */
typedef void (*appCallback)(Int devId,Ptr cbkContext,Ptr buf,Uint16 xferSize);

/**
 *  \brief Global error callback function protype
 *
 *  This is the global error callback function for the McASP driver.
 *  This function is called directly called from ISR context in
 *  case of error.
 */
typedef void (*PSP_mcaspGblCallback)( Int status );

/**
 *  \brief Application callback function protype
 *
 *  This is the application-level callback function for the McASP driver .
 */
typedef void (*PSP_mcaspAppCallback)(   Ptr chan,
                                        Ptr arg,
                                        const struct _PSP_mcaspReq_t *Pkt,
                                        Int32 mode);


/**
 *  \brief Mcasp Interrupt service routine
 *
 *  This is McASP ISR. Interrupt registration takes place by a call
 *  to DDA layer function and ISR is hooked to the interrupt no
 */
typedef void (* pspMcaspIsr)(Ptr iohandlep);


/** Hardware setup global structure */
typedef struct _PSP_McaspHwSetupGbl_t {
    /** Pin function register */
    Uint32  pfunc;
    /** Pin direction register */
    Uint32  pdir;
    /** Global control register - GBLCTL*/
    Uint32  ctl;
    /** Decides whether McASP operates in DIT mode - DITCTL */
    Uint32  ditCtl;
    /** Digital loopback mode setup - DLBEN */
    Uint32  dlbMode;
    /** Mute control register - AMUTE */
    Uint32  amute;
    /** Setup serializer control register (SRCTL0-15) */
    Uint32  serSetup[PSP_MCASP_MAX_SERS];
} PSP_McaspHwSetupGbl;

/** Hardware setup data clock structure */
typedef struct _PSP_McaspHwSetupDataClk_t {
    /** Clock details ACLK(R/X)CTL */
    Uint32  clkSetupClk;
    /** High clock details AHCLK(R/X)CTL */
    Uint32  clkSetupHiClk;
    /** Configures receive/transmit clock failure detection R/XCLKCHK */
    Uint32  clkChk;
} PSP_McaspHwSetupDataClk;

/** Hardware setup data structure */
typedef struct _PSP_McaspHwSetupData_t {
    /** To mask or not to mask - R/XMASK */
    Uint32                      mask;
    /** Format details as per  - R/XFMT */
    Uint32                      fmt;
    /** Configure the rcv/xmt frame sync - AFSR/XCTL */
    Uint32                      frSyncCtl;
    /** Specifies which TDM slots are active - R/XTDM */
    Uint32                      tdm;
    /** Controls generation of McASP interrupts - R/XINTCTL */
    Uint32                      intCtl;
    /** Status register (controls writable fields of STAT register)-R/XSTAT */
    Uint32                      stat;
    /** Event control register - R/XEVTCTL */
    Uint32                      evtCtl;
    /** Clock settings for rcv/xmt */
    PSP_McaspHwSetupDataClk     clk;
} PSP_McaspHwSetupData;

/** Hardware setup structure */
typedef struct _PSP_McaspHwSetup_t {
    /** Value to be loaded in global control register (GLBCTL) */
    PSP_McaspHwSetupGbl     glb;
    /** Receiver settings */
    PSP_McaspHwSetupData    rx;
    /** Transmitter settings */
    PSP_McaspHwSetupData    tx;
    /** Power down emulation mode params - PWRDEMU */
    Uint32                  emu;
} PSP_McaspHwSetup;

/**
 * \brief PSP_Mcasp_DevParams
 * Device Parameters to initialize Device Object
 *
 * This structure passes a handle to setup the McASP
 * device registers. The application provides the
 * McASPHwSetup structure to initialize the device.
 */


/** DIT channel status register structure */
typedef struct _PSP_McaspChStatusRam_t {
    /** Left channel status registers (DITCSRA0-5) */
    Uint32 chStatusLeft[6];
    /** Right channel status register (DITCSRB0-5) */
    Uint32 chStatusRight[6];
} PSP_McaspChStatusRam;

/** DIT channel user data register structure */
typedef struct _PSP_McaspUserDataRam_t {
    /** Left channel user data registers (DITUDRA0-5) */
    Uint32 userDataLeft[6];
    /** Right channel user data registers (DITUDRB0-5) */
    Uint32 userDataRight[6];
} PSP_McaspUserDataRam;



typedef struct PSP_Mcasp_DevParams_t {

    Bool                enablecache;
    /**< Submitted buffers are in cacheable memory                      */

    Bool                isDataBufferPayloadStructure;
    /**< Submitted buffers have payload information present or absent   */

    PSP_McaspHwSetup *mcaspHwSetup;

    /**<  Initial setup for the McASP */

} PSP_mcaspDevParams;

/**
 * \brief channel Parameters to initialize channel Object
 *
 * This structure stores user requested parameters
 */

typedef struct PSP_mcaspChanParams_t
{

   Uint16               noOfSerRequested;
   /**< Serializer requested by channel. Channel can ask for both.      */

   Uint32               indexOfSersRequested[PSP_MCASP_MAX_SERS];
   /**< Multi Serializer numbers requested by channel                  */

   PSP_McaspHwSetupData *mcaspSetup;
   /**< Setup information for xmt/rcv sections of the McASP             */

   Bool                 isDmaDriven;
   /**< This parameters determines whether channel operates in DMA mode
       All DMA parameters would be read only if this is TRUE           */

   PSP_McaspOpMode      channelMode;

   Uint32               wordWidth;
   /**< The parameter informs the driver what is the width word (not slot)
    and this help driver indirectly to decided no. of bytes to be transfered
    into each serialer for each slot- This is very important parameter - in case of
    invalid value default value driver will assume is 32*/


   Ptr                  userLoopJobBuffer;
   /**< Buffer to be transferred when the loop job is running
      it should be noted that this buffer size should be n*userLoopjobLength
      where n is the no of serialisers configured in the direction of the
      channel we are creating*/

   Uint16               userLoopJobLength;
   /**< Number of bytes of the userloopjob buffer for each serialiser
    Please note that this is no. of bytes and this should be pre-calcuated properly
    for word width of slot - Please refer the wordWidth of this structure*/

    PSP_Handle          edmaHandle;
    /**< Handle to the EDMA Driver                                      */

    PSP_mcaspGblCallback    gblCbk;
    /**<
     * callback required when global error occurs -
     * must be callable  directly from the ISR context
     */

    Uint32                  noOfChannels;
    /**< No of channels of data to be transmitted after the frame sync
    This input is valid only for TDM in DSP mode mode of communication
    E.g.--For Stereo data the value is 2 and for 6 channel dac taking
    all channel data through one serialiser the value of this member will be 6
    Note: But for same 6ch dac taking stereo data though 3 seperate serialiser
    value should be 2*/

} PSP_mcaspChanParams;


/**
 * \brief McASP Device ID.
 *
 * The ID of the device that the application wants to send/receive data on.
 * This would be of less significance, if the system contains only one
 * device of this type.
 */
typedef int PSP_McaspInstanceId;

/**<
 *  \brief Structure containing the application callback function and argument
 *
 *  Application shall pass a pointer to PSP_appCallback structure by
 *  specifying the callback function to be called and the argument to be
 *  passed back for the PSP_mcaspChanRead OR PSP_mcaspChanWrite APIs.
 *  The callback function 'fxn' is called with 'arg' and the other params as
 *  defined by the PSP_mcaspAppCallback typedef.
 */
typedef struct PSP_appCallback_t
{
    PSP_mcaspAppCallback     fxn;
    /**< Application callabck                                           */
    Ptr                      arg;
    /**< Application callabck argument                                  */
} PSP_appCallback;



/**
 *  \brief  PSP McASP Ioctl commands
 *
 *  McASP Ioctl commands
 */
typedef enum PSP_McaspIoctlCmd
{

    PSP_MCASP_DEVICE_RESET =           136,

    /** Control command: Enable/Disable AMUTE*/
    PSP_MCASP_CNTRL_AMUTE,

    /** GPIO enable/diable  */
    PSP_MCASP_GPIO_CONFIG,

    /** GPIO Read */
    PSP_MCASP_GPIO_READ ,

    /** GPIO Write     */
    PSP_MCASP_GPIO_WRITE,

    /** Start McASP serial port */
    PSP_MCASP_START_PORT,

    /** Stop McASP serial port */
    PSP_MCASP_STOP_PORT ,

    /** Query McASP Amute register */
    PSP_MCASP_QUERY_AMUTE ,

    /** Loop job modification McASP */
    PSP_CTRL_McASP_MODIFY_LOOPJOB,

    /** Mute ON */
    PSP_CTRL_McASP_MUTE_ON,

    /** Mute OFF and resume */
    PSP_CTRL_McASP_MUTE_OFF,

    /** Pause data transfer */
    PSP_MCASP_PAUSE,

    /** Resume the data transfer */
    PSP_MCASP_RESUME,

    /** Configure receive section in GPIO input mode */
    PSP_CTRL_RCV_GPIO_INPUT,

    /** Control command: Set DIT mode */
    PSP_MCASP_SET_DIT_MODE,

    /** Control command: Send channel timeout cmd */
    PSP_CHAN_TIMEDOUT,

    /** Control command: Send channel reset cmd */
    PSP_CHAN_RESET,

    /** Control command: Set channel format */
    PSP_MCASP_CNTRL_SET_FORMAT_CHAN,

    /** Control command: Get channel format */
    PSP_MCASP_CNTRL_GET_FORMAT_CHAN,

    /** Control command: Set registers affecting McASP device. */
    PSP_MCASP_CNTRL_SET_GBL_REGS,

    /** Control command: Set digital loopback mode. */
    PSP_MCASP_SET_DLB_MODE

} PSP_McaspIoctlCmd;


/**
  * \brief McASP Mode Enumeration
  *
  *  This enum defines the mode of operation of the McASP Driver
  */
typedef enum
{
    /**< Use McASP in Transmit Only */
    PSP_MCASP_TX = 0,
    /**< Use McASP in Receive Only */
    PSP_MCASP_RX

} PSP_mcaspMode;


/**
 * \brief Structure passed through IOM_Packet addr field
 *
 * In order to pass data for the DIT Channel Status and
 * User Data RAM registers, the addr field of the
 * IOM_Packet is cast to PSP_Mcasp_PktAddrPayload. The
 * addr field holds pointers to PSP_McaspChStatusRam and
 * PSP_McaspUserDataRam in addition to the DMA address to
 * be programmed.
 *
 * NOTE: The PSP_McaspChStatusRam and PSP_McaspChStatusRam
 * fields will be read only for a transmit channel
 * operating in DIT mode.
 */

typedef struct _PSP_Mcasp_PktAddrPayload_t {
    PSP_McaspChStatusRam *chStat;
    /**< Channel Status RAM info                                   */
    PSP_McaspUserDataRam *userData;
    /**< User Data RAM info                                        */
    Bool                 writeDitParams;
    /**< Determines whether Channel Status and User Data get
     *  written in case of interrupt mode                         */
    Uint32               *addr;
    /**< Actual address to program DMA with
        Address of data word if transactions are interrupt driven */
} PSP_Mcasp_PktAddrPayload;


/**
  * \brief McASP Request Structure
  *
  *  This structure is required to be filled by application while making
  *  a request to McASP Driver.
  */
typedef struct  _PSP_mcaspReq_t
{
    Uint8               *addr;
    /**< Pointer to the application Buffer */

    Uint32              size;
    /**< Size of the application Buffer */

    Uint32              actualSize;
    /**< Acutal bytes transmitted by the driver */

    PSP_mcaspAppCallback    cbFxn;
    /**< McASP Application call back per Request */

    Ptr                 appPtr;
    /**< Application Data. This data can be filled by application for
         request identification after completion */

    Ptr                 misc;
    /**< misc parameter                                             */

    PSP_Result          status;
    /**< Status of the transfer. This data is filled by the driver. */

    PSP_McaspChStatusRam *chStat;
    /**< Channel Status RAM info                                   */

    PSP_McaspUserDataRam *userData;
    /**< User Data RAM info                                        */

    Bool                 writeDitParams;
    /**< Determines whether Channel Status and User Data get
     *  written in case of interrupt mode                         */
} PSP_mcaspReq;


/** Enumeration for channel mode */
typedef enum {
   PSP_MCASP_CHAN_FREE    = 0,
   /**< Channel not allocated */
   PSP_MCASP_CHAN_XMT_DIT = 1u,
   /**< Transmit channel: DIT mode */
   PSP_MCASP_CHAN_XMT_TDM = 2u,
   /**< Transmit channel: TDM mode */
   PSP_MCASP_CHAN_RCV     = 3u
   /**< Receive channel */
} PSP_McaspChanMode;

/**
 * \brief  Enumeration for channel allocation status
 */
typedef enum {
   PSP_MCASP_UNALLOCATED = 0,
   /**< Channel not allocated  */
   PSP_MCASP_ALLOCATED = 1
   /**< Channel already in use */
} PSP_McaspChannelStatus;


/** Enumeration for the serializer numbers */
typedef enum {
    /** SRCTL0 */
    SERIALIZER_0 = 0,
    /** SRCTL1 */
    SERIALIZER_1 = 1,
    /** SRCTL2 */
    SERIALIZER_2 = 2,
    /** SRCTL3 */
    SERIALIZER_3 = 3,
    /** SRCTL4 */
    SERIALIZER_4 = 4,
    /** SRCTL5 */
    SERIALIZER_5 = 5,
    /** SRCTL6 */
    SERIALIZER_6 = 6,
    /** SRCTL7 */
    SERIALIZER_7 = 7,
    /** SRCTL8 */
    SERIALIZER_8 = 8,
    /** SRCTL9 */
    SERIALIZER_9 = 9,
    /** SRCTL10 */
    SERIALIZER_10 = 10,
    /** SRCTL11 */
    SERIALIZER_11 = 11,
    /** SRCTL12 */
    SERIALIZER_12 = 12,
    /** SRCTL13 */
    SERIALIZER_13 = 13,
    /** SRCTL14 */
    SERIALIZER_14 = 14,
    /** SRCTL15 */
    SERIALIZER_15 = 15
} PSP_McaspSerializerNum;

#ifdef __cplusplus
}
#endif /* extern "C" */

#endif
