/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**     \file      dda_mcasp_bios.c
 *
 *      \brief      Mini-driver for the McASP.
 *
 *      This file implements the McASP mini-driver to be used with DSP-BIOS.
 *
 *      (C)      Copyright 2006,      Texas Instruments, Inc
 *
 *  \author Platform Support Group
 *
 *  \Revision
 *          0.1  Created
 *          0.2  Created  : Dynamic mode start/stop implemented
            0.3  Created  : psp architecture implementation
 */


/*
 * DSP/BIOS headers
 */
#include <std.h>
#include <log.h>
#include <ecm.h>
#include <c64.h>

#include <ti/sdo/pspdrivers/pal_os/pal_osProtect.h>
#include <ti/sdo/pspdrivers/pal_os/pal_oscache.h>
#include <ti/sdo/pspdrivers/drivers/mcasp/src/dda_mcaspBios.h>
#include <ti/sdo/pspdrivers/drivers/mcasp/psp_mcasp.h>
#include <ti/sdo/pspdrivers/drivers/mcasp/src/psp_mcaspApi.h>

#ifdef CHIP_DM648
#include <ti/sdo/pspdrivers/soc/dm648/dsp/soc.h>
#include <ti/sdo/pspdrivers/soc/dm648/dsp/tistdtypes.h>
#endif

#ifdef CHIP_DM6437
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/soc.h>
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/tistdtypes.h>
#endif

#ifdef CHIP_C6424
#include <ti/sdo/pspdrivers/soc/c6424/dsp/soc.h>
#include <ti/sdo/pspdrivers/soc/c6424/dsp/tistdtypes.h>
#endif

#ifdef CHIP_C6452
#include <ti/sdo/pspdrivers/soc/c6452/dsp/soc.h>
#include <ti/sdo/pspdrivers/soc/c6452/dsp/tistdtypes.h>
#endif

#ifdef MCASP_INSTRUMENTATION_ENABLED
#include    <ti/sdo/pspdrivers/pal_os/bios/psp_log.h>
#endif /* MCASP_INSTRUMENTATION_ENABLED */






/*
 * Forward declaration of IOM interface functions
 */
static Int mcasp_mdBindDev(Ptr *devp, Int devid, Ptr devParams);
static Int mcasp_mdControlChan(Ptr chanp, Uint32 cmd, Ptr arg);
static Int mcasp_mdDeleteChan(Ptr chanp);
static Int mcasp_mdSubmitChan(Ptr chanp, IOM_Packet *packet);
static Int mcasp_mdUnBindDev(Ptr devp);

static Int getIomErrorCode(PSP_Result pspCode);
static Int mcasp_mdCreateChan(
                    Ptr                 *chanp,
                    Ptr                 devp,
                    String              name,
                    Int                 mode,
                    Ptr                 chanParams,
                    IOM_TiomCallback    cbFxn,
                    Ptr                 cbArg
                    );


static void mdLayerCallback(Ptr Port, Ptr arg, const PSP_mcaspReq *ioRequest, Int mode);


/**
 *  Mcasp port object which will hold the instance specific information of all
 *  the instances of the Mcasp driver.
 */
static McASPPortObj mcaspPort[PSP_MCASP_NUM_PORTS]={PORTOBJINIT};

/**
 * \brief Public IOM interface table
 *
 * This table holds the interface functions exposed by the mini-driver to
 * the GIO layer.
 */

IOM_Fxns McASP_IOM_FXNS =
{
    &mcasp_mdBindDev,
    &mcasp_mdUnBindDev,
    &mcasp_mdControlChan,
    &mcasp_mdCreateChan,
    &mcasp_mdDeleteChan,
    &mcasp_mdSubmitChan,
};



/**
 *
 *  \fn static Int mcasp_mdBindDev(Ptr *devp, Int devId , Ptr devParams )
 *  \brief Allocates and configures the McASP port specified by devid by
           calling PSP_mcaspCreate() function.
 *
 *  This function calls PSP_mcaspCreate () function which initialises the
 *  driver core layer for neccessary things
 *
 * \param     devp         [OUT]    Pointer to hold allocated port object
 * \param     devid        [IN]     Device number (in this case, there are
 *                                  Four McASP present from 0 to 3)
 * \param     devParams    [IN]     Device parameter structure that supplies
 *                                  data to initialize the McASP
 *
 * \return
 *      if successful
 *                 IOM_COMPLETED
 *  if not successful
 *            IOM_EBADARGS
 *            IOM_EALLOC
 */
static Int mcasp_mdBindDev(Ptr *devp, Int devid, Ptr devParams)
{

    McASPPortHandle     mcaspport   = NULL;
    PSP_mcaspDevParams *devparams   = NULL;
    PSP_Result          result;
    Int32               IOMstatus   = IOM_COMPLETED;

    /* The driver must recieve a valid device parameters  */
    if(   (( NULL != devParams ) && ( PSP_MCASP_NUM_PORTS > devid ) ) &&
            (NULL !=devp))
    {

        devparams  = (PSP_mcaspDevParams *)devParams;

        /* Get the device parameters of the specified port */
        mcaspport = &mcaspPort[devid];
        mcaspport->instNum = (Uint16)devid;

		/* check for USE/UNUSE mcasp instance */
		if(TRUE == mcaspport->inUse)
		{
            IOMstatus = IOM_ENOTIMPL;
		}
        else
		{
            /* Mark the port as in use */
            mcaspport->inUse = TRUE;

            mcaspport->isDataBufferPayloadStructure= \
                    devparams->isDataBufferPayloadStructure;
            /*Inturn call the PSP function to do all init work*/
            result = (PSP_Result)PSP_mcaspCreate(
                                    devid,
                                    (PSP_mcaspDevParams *)devparams,
                                    (Ptr)mcaspport
                                    );

            if( result != PSP_SOK )
            {
                IOMstatus = IOM_EALLOC;
                *devp = NULL;
            }
            else
            {
                *devp = mcaspport;
            }
        }
    }
    else
    {
        IOMstatus = IOM_EBADARGS;
    }

    return IOMstatus;
}


/**
 *  \fn static Int mcasp_mdCreateChan(Ptr *chanp, Ptr devp, String name, Int mode,
 *                    Ptr chanParams, IOM_TiomCallback cbFxn, Ptr cbArg)
 *
 *
 *
 *  \brief  Creates a communication channel in specified mode to communicate
 *          data between the application and the McASP device instance. This function
 *          inturn calls DDC layer function PSP_mcaspOpen ().
 *       -  Returns configured channel handle to application.

 *  This function is called in response to a GIO_create call.
 *
 *      Pre-requisites:
 *       -  Valid chanParams structure
                This takes much information pertaining to
                mcasp channel configuration such as
                * how many serialisers are used for this channel
                * what are they
                *communication parameters , clock setting etc

 *       -  Valid device pointer
 *
 * \note      Please refer to Appendix A : IOM interface of spru616
 *            DSP/BIOS Driver Developer's Guide for details on
 *            mcasp_mdCreateChan parameters.
 *
 * \param     chanp        [IN]     Channel Handler
 * \param     devp         [IN]     Device pointer
 * \param     name         [IN]     McAsp Instance name like MCASP0
 * \param     mode         [IN]     channel  mode -> input or output
 * \param     chanParams   [IN]     channel parameters from user
 * \param     cbFxn        [IN]     callback function pointer
 * \param     cbArg        [IN]     callback function Arguments
 *
 * \return    IOM_COMPLETED     if successful
 *            IOM_EBADIO        if not successful
 *            IOM_EALLOC            "
 *            IOM_EBADARGS      if passed invalid chanParams structure
 *
 */
/* ARGSUSED */
static Int mcasp_mdCreateChan (
                        Ptr              *chanp,
                        Ptr              devp,
                        String           name,
                        Int              mode,
                        Ptr              chanParams,
                        IOM_TiomCallback cbFxn,
                        Ptr              cbArg
                        )
{

    McASPPortHandle port            = NULL;
    Uint32          cookie          = 0;
    Int32           IOMstatus       = IOM_EBADARGS;
    PAL_Result      result          = PSP_E_INVAL_PARAM;
    McASPChannelHandle  chan        = NULL;
    PSP_mcaspChanParams *chanparam  = NULL;

    /*validate parameters*/
    if ( (((NULL == chanParams) || (NULL == cbFxn)) || (NULL == cbArg)) ||
         (((IOM_INPUT != mode) && (IOM_OUTPUT != mode)) || (NULL == devp)))
    {
         IOMstatus = IOM_EBADARGS;
    }
    else
    {

        port = (McASPPortHandle)devp;
        chanparam = (PSP_mcaspChanParams *) chanParams;

        if((NULL == port) || (NULL == chanparam))
        {
            IOMstatus = IOM_EBADARGS;
        }
        else
        {
            if((NULL == chanparam->edmaHandle) && (TRUE == chanparam->isDmaDriven))
            {
               IOMstatus = IOM_EBADARGS;
            }
            else
            {
            /*set channel mode and assign the respective channel to the port*/
            if (IOM_INPUT == mode)
            {
                chan = (McASPChannelHandle)&port->RcvObj;
            }
            else
            {
                chan = (McASPChannelHandle)&port->XmtObj;
            }

            if(NULL == chan)
            {
                IOMstatus = IOM_EBADARGS;
            }
            else
            {
                if (IOM_INPUT == mode)
                {
                    chan->mode = IOM_INPUT;
                }
                else
                {
                    chan->mode = IOM_OUTPUT;
                }

                chan->portHandle = port;

                /* Make sure the channel wasn't already created */
                if (TRUE == chan->inUse)
                {
                     IOMstatus = IOM_EALLOC;
                }
                else

                {
                    /* enter critical section */
                    PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

                    chan->inUse = TRUE;
                    /* Mark channel status as allocated.*/
                    chan->status = PSP_MCASP_ALLOCATED;


                    /* Leave critical section */
                    PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

                    /*Assign the respective callback function*/
                    chan->cbFxn = cbFxn;
                    chan->cbArg = cbArg;


                    chan->ioHandle = PSP_mcaspOpen(
                                                port->instNum,
                                                (PSP_mcaspMode)chan->mode,
                                                chanParams,
                                                (PSP_mcaspAppCallback)&mdLayerCallback,
                                                chanparam->edmaHandle,
                                                NULL
                                                );



                    if (NULL == chan->ioHandle)
                    {
                        result =  PSP_E_DRIVER_INIT;
                    }
                    else
                    {
                        result = PSP_SOK;
                    }

                    /*Retrive the error code returned*/
                    IOMstatus = getIomErrorCode (result);

                    if( IOM_COMPLETED == IOMstatus )
                    {
                        *chanp = (Ptr) chan;
                        }
                    }
                }
            }
        }
    }

    return IOMstatus;
}

/**
 *  \fn static Int mcasp_mdSubmitChan(Ptr chanp, IOM_Packet *packet)
 *
 *  \brief Submit a I/O packet to a channel for processing
 *
 *  The GIO layer calls this function to cause the mini-driver
 *  to process the IOM_Packet for read/write/flush/abort operations.
 *  This function calls DDC layer funtion PSP_mcaspRead / PSP_mcaspwrite /
 *  PSP_mcaspFlushRequests / PSP_mcaspAbortRequests calls  to execute the request.
 *  Submits and I/O packet to channel specified
 *
 *  \param    chanp         [IN]    Pointer to channel
 *  \param   packet         [IN]    Pointer to packet to be submitted
 *
 *  \return  IOM_COMPLETED, if packet is fully processed
 *           IOM_ABORTED,   if packet is aborted
 *           IOM_PENDING,   if packet is not fully processed
 */

static Int mcasp_mdSubmitChan(Ptr chanp, IOM_Packet *packet)
{
    McASPChannelHandle        chan          = NULL;
    PAL_Result                result        = PSP_E_INVAL_PARAM;
    Int32                     IOMstatus     = IOM_EBADARGS;
    PSP_Mcasp_PktAddrPayload  *payload      = NULL;
    PSP_mcaspReq              ioRequest     =   {
                                                    NULL,   /* addr */
                                                    0,      /* size */
                                                    0,      /* actualSize */
                                                    NULL,   /* cbFxn */
                                                    NULL,   /* appPtr */
                                                    NULL,   /* misc */
                                                    0,      /* status */
                                                    NULL,   /* chStat */
                                                    NULL,   /* userData */
                                                    0       /* writeDitParams */
                                                };



        /* Abort if channel is not created /allocated*/

    if((NULL == chanp) || (NULL == packet))
    {
        IOMstatus = IOM_EBADARGS;
    }
    else
    {
        /* Channel Handle */
        chan = chanp;
        payload = (PSP_Mcasp_PktAddrPayload *)packet->addr;

#ifdef MCASP_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT(
                &DVTEvent_Log,
                "McASP",
                PSP_DVT_DESC(
                            PSP_DVT_eFUNC_START,
                            PSP_DVT_dCHANNEL_ID_INST,
                            PSP_DVT_dPACKET_ID,
                            PSP_DVT_dSIZE_BYTES
                            ),
                CHANNELID_INSTANCEID(
                    chan->mode,
                    chan->portHandle->instNum),
                packet->addr,
                packet->size);
#endif /* MCASP_INSTRUMENTATION_ENABLED */

        if( FALSE == chan->inUse )
        {
            IOMstatus = IOM_EBADARGS;
        }
        else
        {
            if( NULL == payload)
            {
                if((IOM_READ == packet->cmd)  && (IOM_WRITE == packet->cmd))
                 {
                    IOMstatus = IOM_EBADARGS;
                 }
            }
            else
            {
                /*
                 * This ioRequest is the intermediate structure used to convey
                 * the driver core about the request from GIO, to the underlying
                 * PSP_mcaspxxxx calls.The GIO IOP pointer is stored as a member,
                 * because we will need that during IO completion
                 */
                if (FALSE == chan->portHandle->isDataBufferPayloadStructure)
                {
                    ioRequest.addr          = (Uint8 *)packet->addr;
                    ioRequest.chStat        = NULL;
                    ioRequest.writeDitParams= FALSE;
                    ioRequest.userData      = NULL;
                }
                else
                {
                    ioRequest.addr          = (Uint8 *)payload->addr;
                    ioRequest.chStat        = payload->chStat;
                    ioRequest.writeDitParams= payload->writeDitParams;
                    ioRequest.userData      = payload->userData;
                }
                ioRequest.size          = packet->size;
                ioRequest.cbFxn         = NULL;
                ioRequest.appPtr        = (Ptr)packet;
                ioRequest.actualSize    = 0;
                ioRequest.status        = PSP_SOK;

            }

            switch (packet->cmd)
            {
                case IOM_READ:
                        if ( (NULL != ioRequest.addr) && (NULL != payload))
                        {
                            result =   PSP_mcaspRead ((PSP_Handle)chan->ioHandle, &ioRequest, NULL);
                        }
                        break;

                case IOM_WRITE:
                        if ( (NULL != ioRequest.addr)  && (NULL != payload))
                        {
                            result =   PSP_mcaspWrite ((PSP_Handle)chan->ioHandle,&ioRequest, NULL);
                        }
                        break;

                case IOM_ABORT:
                        result =   PSP_mcaspAbortRequests ((PSP_Handle)chan->ioHandle, NULL);
                        break;

                case IOM_FLUSH:
                        result =   PSP_mcaspFlushRequests ((PSP_Handle)chan->ioHandle, NULL);
                        break;

                default:
                        result = PSP_E_INVAL_PARAM;
                        break;

            }

            IOMstatus = getIomErrorCode (result);
        }
    }

#ifdef MCASP_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT(
                &DVTEvent_Log,
                "McASP",
                PSP_DVT_DESC(
                            PSP_DVT_eFUNC_END,
                            PSP_DVT_dCHANNEL_ID_INST,
                            PSP_DVT_dPACKET_ID,
                            PSP_DVT_dSIZE_BYTES
                            ),
                CHANNELID_INSTANCEID(
                    chan->mode,
                    chan->portHandle->instNum),
                packet->addr,
                0);
#endif /* MCASP_INSTRUMENTATION_ENABLED */

    return IOMstatus;
}

/**
 *  \fn static Int mcasp_mdUnBindDev(Ptr devp)
 *
 *  \brief Closes the McASP device from use.
 *
 *  mdUnBindDev() unwinds the actions performed in mdbinddev.
 *  This is accomplished by calling PSP_mcaspDelete
 *
 * \param     devp         [IN]    Pointer to port object to be freed
 *
 * \return    IOM_COMPLETED     if successful
 *            IOM_ABORTED       if not successful
 */

static Int mcasp_mdUnBindDev(Ptr devp)
{

    McASPPortHandle     mcaspport   = NULL;
    PAL_Result          result      = PSP_E_INVAL_PARAM;
    Int32               IOMstatus   = IOM_EBADARGS;

    /*
    * Check if this is a valid handle
    */
    if (NULL == devp)
    {
        IOMstatus = IOM_EBADARGS;
    }
    else
    {
        mcaspport = (McASPPortHandle) devp;

        if ((NULL == mcaspport)|| (TRUE != mcaspport->inUse))
        {
            IOMstatus = IOM_EBADARGS;
        }
        else
        {
            /*
            * Free allocated port
            */
            mcaspport->inUse = FALSE;

            /*
            * Close and free mcasp port handle
            */
            result = PSP_mcaspDelete(mcaspport->instNum,NULL);

            IOMstatus = getIomErrorCode(result);
        }
    }

    return IOMstatus;

}

/**
 *  \fn static Int mcasp_mdDeleteChan(Ptr chanp)
 *
 *  \brief frees a channel and all it's associated resources.
 *
 *  mdDeleteChan() acts as a wrapper function to DDC layer funtion
 *
 *  \param    chanp  [IN]    Pointer to channel
 *
 * \return    IOM_COMPLETED     if successful
 *            IOM_ABORTED       if not successful
 */

static Int mcasp_mdDeleteChan(Ptr chanp)
{
    McASPChannelHandle  chan        = NULL;
    PAL_Result          result      = PSP_E_INVAL_PARAM;
    Int32               IOMstatus   = IOM_EBADARGS;

    if(NULL == chanp)
    {
        IOMstatus = IOM_EBADARGS;
    }
    else
    {
        chan        = (McASPChannelHandle) chanp;

        if ((NULL == chan ) || (TRUE != chan->inUse) )
        {
            IOMstatus = IOM_EBADARGS;
        }
        else
        {
            result = PSP_mcaspClose((PSP_Handle)chan->ioHandle , NULL );

            IOMstatus = getIomErrorCode(result);

            if (IOM_COMPLETED == IOMstatus)
            {
                chan->inUse = FALSE;
            }
        }
    }
    return IOMstatus;

}

/**
 *  \fn static Int mdControlChan(Ptr chanp, Uns cmd, Ptr arg)
 *
 *  \brief Implements the IOCTLS for McASP IOM mini driver by calling
 *         DDC layer function ddc_mcaspControlChan.
 *
 *  mdControlChan() implements recieves IOCTL commands from the
 *  GIO layer and then forwards the commands to the DDC by calling
 *  PSP_mcaspIoctl.
 *
 *  \return  IOM_COMPLETED,  if command is executed correctly
 *            IOM_ABORTED,   if command returns error during execution
 *            IOM_ENOTIMPL,  if command is not supported
 *            IOM_EBADARGS   if args are not correct
 */

static Int mcasp_mdControlChan(Ptr chanp, Uint32 cmd, Ptr arg)
{

    McASPChannelHandle  chan        = NULL;
    PAL_Result          result      = PSP_E_INVAL_PARAM;
    Int32               IOMstatus   = IOM_EBADARGS;

    if(NULL == chanp)
    {
        IOMstatus = IOM_EBADARGS;
    }
    else
    {
        chan = (McASPChannelHandle ) chanp;
        if (TRUE != chan->inUse)
        {
            IOMstatus = IOM_EBADARGS;
        }
        else
        {
            result = PSP_mcaspIoctl ( (PSP_Handle)chan->ioHandle,(PSP_McaspIoctlCmd)cmd,arg,NULL);

            IOMstatus = getIomErrorCode(result);
        }
    }
    return (IOMstatus);
}
/*!
 *  \fn void  MCASP_IOM_init()
 *
 *  \brief Initializes McASP IOM driver's data structures
 *
 *  This function initializes the McASP IOM driver's data structures including
 *  PortObject instance and channel object instance. Because the BIOS is not
 *  initialsing the globals and these leads the problematic situations.
 *
 *  \param  [IN]  none
 *
 *  \return Pointer to the McASP IOM driver function table
 */
void  MCASP_IOM_init()
{
    McASPPortHandle     port        = NULL;
    McASPChannelHandle  chan        = NULL;
    Uint32              numPorts    = 0;

    for (numPorts = 0; numPorts < PSP_MCASP_NUM_PORTS; numPorts++)
    {
        port = (McASPPortHandle) &mcaspPort[numPorts];
        port->inUse = FALSE;

        chan = (McASPChannelHandle) &port->RcvObj;
        if(NULL != chan)
        {
            chan->inUse = FALSE;
        }

        chan = (McASPChannelHandle) &port->XmtObj;
        if(NULL != chan)
        {
            chan->inUse = FALSE;
        }

        PSP_mcaspInit(numPorts);
    }
    return ;
}

/*!
 *  \fn Void mdLayerCallback(Ptr Port, Ptr Arg, const PSP_mcaspReq pkt,Int mode)
 *
 *  \brief DDC layer callback
 *
 *  This callback is made from the DDC layer. This function restores appropriate
 *  data into the IOP and  calls GIO callback
 *
 *  \param  [IN]  Port      Port handle
 *  \param  [IN]  arg       optional device-defined data structure
 *  \param  [IN]  ioRequest pointer to io request
 *  \param  [IN]  mode      channel  mode -> input or output
 *
 *  \return  Void
 */
static void mdLayerCallback(Ptr Port, Ptr arg, const PSP_mcaspReq *ioRequest, Int mode)
{
    IOM_Packet          *packet     = NULL;
    McASPPortHandle     port        = NULL;
    McASPChannelHandle  chan        = NULL;

    if (( NULL != Port ) && ( NULL != ioRequest ) )
    {
        /* To remove compiler warning */
        if(NULL != arg)
        {
            arg = arg;
        }
        port = Port;
        packet    = (IOM_Packet *)ioRequest->appPtr;

        if ( NULL != packet)
        {
            packet->size    = ioRequest->size;
            packet->status  = ioRequest->status;

            if (IOM_INPUT == mode)
            {
                chan = (McASPChannelHandle)&port->RcvObj;
            }
            else
            {
                chan = (McASPChannelHandle)&port->XmtObj;
            }

            packet->status = getIomErrorCode(packet->status);

            if (NULL != chan )
            {
                if (NULL != chan->cbFxn)
                {
                    /* Application callback */
                    (*chan->cbFxn)(chan->cbArg,packet);
                }
            }
        }
    }

}

/**
  *\ fuction will get the IOM Error Code
  *
  */
static Int getIomErrorCode(PSP_Result pspCode)
{
    /*-----------------------------------------------------------------------------
                            IOM Error code look up table
     -----------------------------------------------------------------------------*/
    /**
     * The following mapping shows the mapping done from PSP_Result to IOM error
     * code. This will be used in scenario where IOMini driver interfaces with
     * the DDC- PSP component.
     *
     * PSP_SOK                  (0)   IOM_COMPLETED            0
     * PSP_SINPROGRESS          (1)   IOM_PENDING              1
     * PSP_E_DRIVER_INIT        (-1)  IOM_ETIMEOUTUNREC      -11
     * PSP_E_NO_MEMORY          (-2)  IOM_EALLOC              -5
     * PSP_E_RESOURCES          (-3)  IOM_EALLOC              -5
     * PSP_E_INVAL_STATE        (-4)  IOM_EBADMODE            -7
     * PSP_E_INVAL_PARAM        (-5)  IOM_EBADARGS           -10
     * PSP_E_NOT_SUPPORTED      (-6)  IOM_ENOTIMPL            -9
     * PSP_E_IO_CANCEL_FAIL     (-7)  IOM_ETIMEOUT            -2
     * PSP_E_FIFO_NOT_ENABLED   (-8)  IOM_EBADMODE            -7
     * PSP_E_INVALID_MODE       (-9)  IOM_EBADMODE            -7
     * PSP_E_INVALID_CMD        (-10) IOM_EBADARGS           -10
     *
     */
    static Int iomErrorCodeList[14] ={
                                        IOM_EBADARGS,
                                        IOM_EBADMODE,
                                        IOM_EBADMODE,
                                        IOM_ETIMEOUT,
                                        IOM_ENOTIMPL,
                                        IOM_EBADARGS,
                                        IOM_EBADMODE,
                                        IOM_EALLOC,
                                        IOM_EALLOC,
                                        IOM_ETIMEOUTUNREC,
                                        IOM_COMPLETED,
                                        IOM_PENDING,
                                        IOM_FLUSHED,
                                        IOM_ABORTED
                                     };
    Int iomCode = pspCode+10;

    if((iomCode < 14) && (iomCode >= 0))
    {
        iomCode = iomErrorCodeList[iomCode];
    }
    else
    {
        iomCode = IOM_EBADIO;
    }

    return(iomCode);
}




/**
 *  \brief _DDA_McASP_RegisterInt
 *
 *
 *  \brief   register interrupt handler for channel mode
 *
 *  \param    chanp        [IN]    Pointer to channel
 *  \param    instNum      [IN]    instance number
 *  \param    evt          [IN]    event number
 *  \param    intIsr       [IN]    function pointer
 *
 *  \return   If success return PSP_SOK
 */
PSP_Result _DDA_McASP_RegisterInt(Ptr chanp, Uint32 instNum, Uint16 evt, pspMcaspIsr intIsr)
{

    McASPChannelHandle  chan        = NULL;
    ECM_Attrs           ecmattrs    = ECM_ATTRS;
    static Uns          cookie      = 0;
    PSP_Result          result      = PSP_SOK;
    Uint32              mask        = 0;

    if(( NULL == chanp) || ( NULL == intIsr) )
    {
        result = PSP_E_INVAL_PARAM;
    }
    else
    {
        switch(instNum)
        {
            case 0:
                mask = HWI_EVT02_CMB_IRQ;
                result = PSP_SOK;
                break;
            default :
                result = PSP_E_INVAL_PARAM;
                break;
        }
        if(PSP_SOK == result)
        {
        chan = (McASPChannelHandle)chanp;

        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);


        /*Enable the interrupt later*/
        ecmattrs.unmask = 0u;

        /*setup arg such that isr will get 'chan' as arg*/
        ecmattrs.arg    = (Arg)chan;

        /* Plug the ISR with interrupt (event) number */
        ECM_dispatchPlug(evt, (ECM_Fxn)intIsr, &ecmattrs);

        /*
        * Enabling the HWI_ID where corresponding ECM group is configured
        */
        C64_enableIER(mask);


        /* Enabling the interrupts */
        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
        }
    }

    return result;
}

/**
 *  \brief _DDA_McASP_UnRegisterInt
 *
 *
 *  \brief   unregister interrupt handler for channel mode
 *
 *  \param    devid        [IN]    instance number
 *  \param    evt          [IN]    interrupt number
 *  \param    mode         [IN]    input or output
 *
 *  \return  void
 */
PSP_Result _DDA_McASP_UnRegisterInt(Uint16 devid, Uint16 evt,Int mode)
{

    Uint32          cookie      = 0;

    devid = devid;
    mode  = mode;

    PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

    /* Disable the event*/
    ECM_disableEvent(evt);

    /* Enabling the interrupts */
    PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

    return PSP_SOK;
}




