/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/** \mainpage    PSP McASP Driver on BIOS
 *
 *  \section    Introduction
 *
 *  \subsection    xxx    Purpose and Scope
 *  The purpose of this document is to explain the McASP driver on the PSP
 *  Framework architecture written in DSP/BIOS for platform.
 *
 *  \subsection    aaa    PSP Architecture
 *  A PSP framework is a software entity providing platform support
 *  for devices that are based on Texas Instruments SoC's. The specific
 *  (Hardware and software) implementation of the PSP framework (the PAL
 *  implementation and the group of device drivers) is referred to as
 *  PSP System. The PSP framework defines a software environment for device
 *  drivers to plug-in by defining standard usage interfaces and
 *  abstractions (OS, CPU/Peripherals). For every platform, the PSP
 *  implements platform specific functionality such as OS adaptation, CPU and
 *  peripherals specific mechanisms, and generic system services such as
 *  debug control,traces, persistent storage, etc.
 *
 *  \subsection    bbb    McASP Controller
 *  The multichannel audio serial port (McASP) functions as a general-purpose
 *  audio serial port optimized for the needs of multichannel audio
 *  applications. The McASP is useful for time-division multiplexed (TDM)
 *  stream, Inter-Integrated Sound (I2S) protocols, and intercomponent digital
 *  audio interface transmission (DIT). The McASP consists of transmit and
 *  receive sections that may operate synchronized, or completely independently
 *  with separate master clocks, bit clocks, and frame syncs, and using different transmit
 *  modes with different bit-stream formats. The McASP module also includes up to 16
 *  serializers that can be individually enabled to either transmit or receive. In
 *  addition, all of the McASP pins can be configured as general-purpose
 *  input/output (GPIO) pins.
 *
 *  \subsection    ccc    DSP/BIOS System
 *  DSP/BIOS is a real time kernel provided by TI to use with its
 *  DSP processors.
 *
 *
 *  \subsection    References
 *    -# McASP user guide, <b>SPRU980_McASP.pdf</b>
 *    -# Design Document for McASP driver,
 *                         <b>BIOS_McASP_Driver_Arch.doc</b>
 *    -# PSP Framework Overview, <b>PSPFramework_Overview.pdf</b>
 *    -# DSP/BIOS driver development <b>SPRU616.pdf</b>
 *
 */


/** \file     ddc_mcasp.h
 *
 *  \brief   DDC defined for a McASP controller
 *
 *  This file defines channel and port objects for the i/o processing.
 *  This file contains definitions for a McASP DDC. This file depends
 *  on some definition from the low-level controller header.
 *  All such definitions are used as opaque objects in the McASP DDC.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author Platform Support Group
 *
 *  \Revision
 *          0.1
 *          0.2  : Dynamic mode start/stop implemented
 */


#ifndef _DDC_MCASP
#define _DDC_MCASP
/*
 *  Mcasp header
 */
#include <std.h>
#include <ti/sdo/pspdrivers/drivers/mcasp/psp_mcasp.h>
#include <ti/sdo/pspdrivers/drivers/mcasp/src/llc_mcasp.h>
#include <ti/sdo/pspdrivers/common/psp_common.h>
#include <ti/sdo/pspdrivers/pal_os/pal_oslist.h>
#include <ti/sdo/pspdrivers/pal_os/pal_ossem.h>
#include <ti/sdo/pspdrivers/pal_os/pal_oscache.h>


#ifdef __cplusplus
extern "C" {
#endif

/** Maximum packet size in bytes */
#define MCASP_MAX_PACKET_SIZE     (32768u)

/** Maximum number of EDMA jobs linked at a time (Must be 2).          */
#define MAXLINKCNT                   (2u)

/** Maximum request that can be queued up at DDC layer                 */
#define PSP_MCASP_MAX_REQ             (30)

/** Reset value of xmt/rcv state m/c and frame sync */
#define PSP_MCASP_SM_RESET    (0u)

/** When xmt is taken out of reset, it needs two words to service it.
 *  This is used to keep tally of this requirement. */
#define PSP_MCASP_TWO_PKTS_QUEUED (2u)

/** Generic invalidate status */
#define PSP_MCASP_INVALID         (0xFFFF)

/** Generic validate status */
#define PSP_MCASP_VALID           (0x1)

/** Used for identifying IO mode       */ 
#define PSP_MCASP_INPUT                     (1u)
#define PSP_MCASP_OUTPUT                    (2u)

/** Transmit / Receive ERROR Flag */
#define PSP_MCASP_XERR                     (0x100u)

/**
 * \brief Error conditions returned by the McASP driver
 *
 *  McASP driver specific error conditions.
 *
 *  I/O request flushed. Queued writes will be completed w/ IOM_COMPLETED.
 *  sQueued read requests return w/ IOM_FLUSHED.
 */

/** I/O flushed. Non-completed read or write requests return w/ IOM_FLUSHED.*/
#define  PSP_MCASP_FLUSHED      (2u)
/** I/O aborted. Non-completed read or write requests return w/ IOM_ABORTED.*/
#define  PSP_MCASP_ABORTED      (3u)
/** Error due to: DMA channel error */
#define PSP_MCASP_DMA_ERROR                (-15)
/**  Error due to: Transmit/Receive Sync Error */
#define PSP_MCASP_SYNC_ERROR               (-16)
/**  Error due to: Transmit Buffer Underrun */
#define PSP_MCASP_XMT_UNDERRUN_ERROR       (-17)
/**  Error due to: Receive Buffer Overrun */
#define PSP_MCASP_RCV_OVERRUN_ERROR        (-18)
/**  Error due to: Rcv Bad Clock */
#define PSP_MCASP_RCV_BAD_CLK_ERROR        (-19)
/**  Error due to: Xmt Bad Clock */
#define PSP_MCASP_XMT_BAD_CLK_ERROR        (-20)
/** Error condition for no free I/O packets in I/O packet pool         */
#define PSP_E_NO_IOPS                      (-50)
/** Error condition for bad I/O operation                              */
#define PSP_E_BAD_IO                       (-51)
/**< Error condition when the pending I/O operation is aborted          */
#define PSP_E_IO_ABORTED                   (-52)

/**
 * \brief Channel setup parameters
 *
 * Application specifies channel requirements and
 * passes information to initialize the DMA channel
 * and interrupt handling for the channel.
 *
 * This is passed as a parameter to mdCreateChan.
 */

/** Interrupt due to: Data Ready */
#define PSP_MCASP_DATA_RDY         (0x20u)
/** Interrupt due to: DMA channel error */
#define PSP_MCASP_DMAERR           (0x80u)

/** Interrupt due to: Transmit Buffer Underrun */
#define PSP_MCASP_XMT_UNDERRUN     (0x1u)
/** Interrupt due to: Receive Buffer Overrun */
#define PSP_MCASP_RCV_OVERRUN      (0x1u)
/** Interrupt due to: Transmit/Receive Sync Error */
#define PSP_MCASP_SYNCERR          (0x2u)

/** Transmit EDMA channel event queue number */
#define TXEVENTQUE                 (0u)

/** Receive EDMA channel event queue number */
#define RXEVENTQUE                 (2u)

/*
toggle the index
*/
#define GetNextIndex(index) (index = ((index+1) & 0x01))

/** Enumeration for serializer status */
typedef enum PSP_McaspSerializerStatus_t
{
   PSP_MCASP_SER_FREE = (0u),
   /**< Serializer not allocated */

   PSP_MCASP_SER_XMT  = (1u),
   /**< Serializer configured to transmit */

   PSP_MCASP_SER_RCV  = (2u)
   /**< Serializer configured to receive  */

} PSP_McaspSerializerStatus;

/* forward declaration */
struct DDC_mcaspDevice_t;


/*Loopjob buffers used in EDMA mode*/
typedef struct localTempbuffer_t
{
    Uint32 scratchBuf[PSP_MCASP_MAX_SERS +  PAL_OS_CACHE_LINE_LEN];

	Uint32 *scratchBuffer;

}localTempBuffer;



typedef struct DDC_mcaspIOP_t
{

    PAL_OsListNodeHeader    iopLink;
    /**< Link to the next & previous in the list                        */

    Uint8                   *addr;
    /**< Application Buffer Pointer                                     */

    Uint32                  size;
    /**< Application Request Size                                       */

    Uint32                  actualSize;
    /**< Actual Data transferred                                        */

    PSP_Result              status;
    /**< Status of the Request. Updated by the Driver                   */

    PSP_mcaspAppCallback    appCbk;
    /**< McASP Application call back registered during Open             */

    Ptr                     appData;
    /**< Application provided Pointer                                   */

    PSP_McaspChStatusRam   *chStat;
    /**< Channel Status RAM info                                   */

    PSP_McaspUserDataRam   *userData;
    /**< User Data RAM info                                        */

    Bool                    writeDitParams;
      /**< Determines whether Channel Status and User Data get
     *  written in case of interrupt mode                         */

    Ptr                     misc;
    /**< misc parameter                                            */


} DDC_mcaspIOP;


/*
 * Channel function definitions
 */

/**
 * \brief Channel Object
 *
 * This structure maintains the current channel state. It also holds
 * information on DMA channel being used and holds the application
 * callback function to be called in case of an interrupt.
 *
 * This structure is initialized by mdCreateChan and a pointer to this
 * is passed down to all other channel related functions. Lifetime of
 * the data structure is from its creation by mdCreateChan till it
 * is invalidated by mdDeleteChan.
 */

typedef struct DDC_mcaspIo_t
{

    PAL_OsListNodeHeader            queueFreeList;
    /**< List containing the Free IO Packets */

    PAL_OsListNodeHeader            queueReqList;
    /**< List containing the Requests received from application. Driver
         checks this list to determine wether TX or RX is active        */

    PAL_OsListNodeHeader            queueFloatingList;
    /**< Queue to manage floating packets in DMA                        */

    PAL_OsSemHandle                 syncSem;
    /**< Semaphore used during Flush Request                            */

    DDC_mcaspIOP                    ioRequests[PSP_MCASP_MAX_REQ];
    /**< Channel transfer complete code                                 */

    PSP_McaspChannelStatus          status;
    /**< Keeps tab of whether channel is already in use
     *  - takes value of UNALLOCATED or ALLOCATED          */

    Uint16                          noOfSerAllocated;
    /**< Serializer(s) allocated                            */

    Uint16                          mode;
    /**< Rx mode or Tx mode                                 */

    Uint16                          inUse;
    /**< Is the channel in use?                             */

    PSP_McaspChanMode               channelOpMode;
    /**< Mode of operation: Transmit(DIT/TDM) or Receive    */

    Bool                            isDmaDriven;
    /**< Determines whether data input is using DMA or not  */

    Uint16                          dataQueuedOnReset;
    /**< McASP needs to service each serializer with two
        words when it awakes from reset                    */

    Uint32                          intStatus;
    /**< Holds id of McASP interrupt cause, if any
        used in interrupt mode                             */

    struct DDC_mcaspDevice_t        *deviceHandle;
    /**< Pointer to McASP device
     *  - Back pointer to device configuration structure   */

    DDC_mcaspIOP                    *flushPacket;
    /**< Used for submit/IOM_FLUSH/flush packet             */

    DDC_mcaspIOP                    *dataPacket;
    /**< Current data packet being serviced.                */

    DDC_mcaspIOP                    *tempPacket;
    /**< Temp IOP holder                 */

    volatile Bool                   isTempPacketValid;
    /**< Valid packet flag in EDMA callback                */

    Uint32                          userDataBufferSize;
    /**<Size of the data buffer per serialiser, hence the buffer
    given by the application should really be
    userDataBufferSize * no. of serialisers allocated for that
    channel*/

    int                             submitCount;
    /**< Number of submit calls pending                     */

    PSP_mcaspAppCallback            cbFxn;
    /**< Callback function                                  */

    Ptr                             cbArg;
    /**< Arguments for callback function                    */

    Uint32                          indexOfSersRequested[PSP_MCASP_MAX_SERS];
    /**< Multi Serializer numbers requested by channel     */

    Ptr                             edmaHandle;
    /**< eDMA  Handler */

    Uint32                          xferChan;
    /**< eDMA  ChannelID */

    Uint32                          tcc ;
    /**< eDMA  tcc */

    Uint32                          pramTbl[MAXLINKCNT];
    /**< Logical channel numbers of EDMA, which are used for linking    */

    Uint32                          pramTblAddr[MAXLINKCNT];
    /**< Physical address of logical channel numbers of EDMA, which
    are used for linking */

    Uint32                          nextLinkParamSetToBeUpdated;
    /**< Used to store the next index of link param to be updated */

    volatile Bool                   loopjobUpdatedinParamset;
    /**<Used to prevent updating second paramset with loopjob for last datapacket 
    edma callback*/

    Uint32                          cpuEventNum;
    /**< This hold the cpu event no. of this channel (TX/Rx) for this instance*/

    volatile Bool                   xferinProgressIntmode;
    /**<use to start int mode transfer from  0 packet status*/

    Ptr                             loopJobBuffer;
    /**< Buffer to be transferred when the loop job is running          */

    Uint16                          loopJobLength;
    /**<Length of userloop job for each serialiser*/

    Uint16                          roundedWordWidth;
    /**< Word length bytes to be transferred for DMA transaction */

    Uint16                          currentDataSize;
    /**< data packet size used for interrupt mode*/

    volatile Bool                   bMuteON;
    /**< True, if playback is muted                                     */

    volatile Bool                   paused;
    /**< TRUE, if pause Request is called                               */

    volatile Bool                   flush;
    /**< TRUE, if Flush Request is called                               */

    EDMA3_RM_TccCallback            edmaCallback;
    /**<stores the edma call back function ptr - functions actual type is
    EDMA3_RM_TccCallback*/

    PSP_mcaspGblCallback            gblErrCbk;
    /**< callback required when the loop job is running
     *  must be callable directly from the IOM layer
    */
    volatile Bool                   nextFlag;

	Uint32							currentPacketErrorStatus;
	/**<
	This member will hold the error status -normally updated from
	cpu interrupt thread and is used in EDMA completion thread for 
	updating the error code in IOP
	*/

}DDC_mcaspIoObj, *DDC_mcaspIoHandle;


/**
 * \brief Device Object
 *
 * This data structure holds the current device state. It also
 * holds the handle to the McASP. The data structure is
 * initialized during mcasp_mdBindDev, which is called during DSP-
 * BIOS initialization, and is persistent till it is
 * invalidated by mcasp_mdUnBindDev.
 */

typedef struct DDC_mcaspDevice_t
{
    Bool                        enablecache;
    /**< Submitted buffers are in cacheable memory                      */

    LLC_McaspObj                hwInfo;
    /**< McASP handle for initial port configuration       */

    DDC_mcaspIoObj               rxIo;
     /**< Holds channels to the McASP.                      */

    DDC_mcaspIoObj               txIo;
    /**< Holds channels to the McASP.                      */

    PSP_McaspSerializerStatus   serStatus[PSP_MCASP_MAX_SERS];
    /**< Holds status information for both the serializers.
     *  - Valid values are FREE, XMT and RCV              */

    Uint16                      InstNum;
    /**< Preserve  instance number in port */

    Bool                        stopSmFsXmt;
    /**< Status of stoping state m/c for XMT */

    Bool                        stopSmFsRcv;
    /**< Status of stoping state m/c for RCV */

    Ptr                         appHandle;
    /**<Pointer to the Port */

}DDC_mcaspDevice,*DDC_mcaspDeviceHandle;


#ifdef __cplusplus
}
#endif /* extern "C" */

#endif /*_DDC_MCASP*/
