/*************************************************************************
**+--------------------------------------------------------------------+**
**|                                   ****                             |**
**|                                   ****                             |**
**|                                   ******o***                       |**
**|                             ********_///_****                      |**
**|                             ***** /_//_/ ****                      |**
**|                             ** ** (__/ ****                        |**
**|                             *********                              |**
**|                             ****                                   |**
**|                             ***                                    |**
**|                                                                    |**
**| Copyright (c) 1998-2006 Texas Instruments Incorporated             |**
**| ALL RIGHTS RESERVED                                                |**
**|                                                                    |**
**| Permission is hereby granted to licensees of Texas Instruments     |**
**| Incorporated (TI) products to use this computer program for sole   |**
**| purpose of implementing a licensee product based on TI products.   |**
**| No other rights to reproduce, use, or disseminate this computer    |**
**| program, whether in part or in whole, are granted.                 |**
**|                                                                    |**
**| TI makes no representation or warranties with respect to the       |**
**| performance of this computer program, and specifically disclaims   |**
**| any responsibility for any damages, special or consequential,      |**
**| connected with the use of this program.                            |**
**|                                                                    |**
**+--------------------------------------------------------------------+**
*************************************************************************/

/**     \file      llc_mcasp.c
 *
 *      \brief      Mini-driver for the McASP.
 *
 *      This file implements the McASP mini-driver to be used with DSP-BIOS.
 *
 *      (C)      Copyright 2006,      Texas Instruments, Inc
 *
 *  \author Platform Support Group
 *
 *  \Revision
 *          0.1  Created
 *          0.2  Created  : DIT mode: DitStatus made true for default mode
 *
 */

#include <ti/sdo/pspdrivers/drivers/mcasp/psp_mcasp.h>
#include <ti/sdo/pspdrivers/drivers/mcasp/src/ddc_mcasp.h>



/**
 *  Status Query Functions of McASP
 */

/**
 * \brief   LLC_mcaspGetCurrentXSlot
 *
 *  This function returns the current transmit time slot count
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *      Current transmit time slot count
 */

static Uint16 LLC_mcaspGetCurrentXSlot (LLC_McaspHandle    hMcasp)
{
    Uint16 val = 0;

    if (NULL != hMcasp)
    {
        val = (Uint16)(CSL_FEXT(hMcasp->regs->XSLOT, MCASP_XSLOT_XSLOTCNT));
    }

    return val;
}

/**
 * \brief   LLC_mcaspGetCurrentRSlot
 *
 *  This function returns the current receive time slot count
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *      Current receive time slot count
 */

static Uint16 LLC_mcaspGetCurrentRSlot (LLC_McaspHandle    hMcasp)
{
    Uint16 val = 0;

    if (NULL != hMcasp)
    {
        val = (Uint16)(CSL_FEXT(hMcasp->regs->RSLOT, MCASP_RSLOT_RSLOTCNT));
    }

    return val;
}


/**
 * \brief   LLC_mcaspGetXmtErr
 *
 *  This function checks whether transmitter error interrupt has occurred or not
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *			TRUE  - Transmitter error interrupt has occurred
 *          FALSE - Transmitter error interrupt has not occurred
 */

static Bool LLC_mcaspGetXmtErr (LLC_McaspHandle    hMcasp)
{
    Bool val = FALSE;

    if (NULL != hMcasp)
    {
        val = (Bool)(CSL_FEXT(hMcasp->regs->XSTAT, MCASP_XSTAT_XERR));
    }

    return val;
}

/**
 * \brief   LLC_mcaspGetXmtClkFail
 *
 *  This function checks whether transmit clock failure flag is set or not
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *			TRUE  - Transmit clock failure flag is set
 *          FALSE - Transmit clock failure flag is not set
 */

static Bool LLC_mcaspGetXmtClkFail (LLC_McaspHandle    hMcasp)
{
    Bool val = FALSE;

    if (NULL != hMcasp)
    {
        val = (Bool)(CSL_FEXT(hMcasp->regs->XSTAT, MCASP_XSTAT_XCKFAIL));
    }

    return val;
}

/**
 * \brief   LLC_mcaspGetXmtSyncErr
 *
 *  This function checks whether unexpected transmit frame sync flag is 
 *  set or not
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *			TRUE  - Unexpected transmit frame sync error has occurred
 *          FALSE - Unexpected transmit frame sync error has not occurred
 */

static Bool LLC_mcaspGetXmtSyncErr (LLC_McaspHandle    hMcasp)
{
    Bool val = FALSE;

    if (NULL != hMcasp)
    {
        val = (Bool)(CSL_FEXT(hMcasp->regs->XSTAT, MCASP_XSTAT_XSYNCERR));
    }

    return val;
}

/**
 * \brief   LLC_mcaspGetXmtUnderrun
 *
 *  This function checks whether transmitter underrun flag is set or not
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *			TRUE  - Transmitter underrun flag is set
 *          FALSE - Transmitter underrun flag is not set
 */

static Bool LLC_mcaspGetXmtUnderrun (LLC_McaspHandle    hMcasp)
{
    Bool val = FALSE;

    if (NULL != hMcasp)
    {
        val = (Bool)(CSL_FEXT(hMcasp->regs->XSTAT, MCASP_XSTAT_XUNDRN));
    }

    return val;
}

/**
 * \brief   LLC_mcaspGetXmtDataReady
 *
 *  This function checks whether transmit data ready flag is set or not
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *			TRUE  - Transmit data ready flag is set
 *          FALSE - Transmit data ready flag is not set
 */

static Bool LLC_mcaspGetXmtDataReady (LLC_McaspHandle    hMcasp )
{
    Bool val = FALSE;

    if (NULL != hMcasp)
    {
        val = (Bool)(CSL_FEXT(hMcasp->regs->XSTAT, MCASP_XSTAT_XDATA));
    }

    return val;
}

/**
 * \brief   LLC_mcaspGetRcvErr
 *
 *  This function checks whether receiver error interrupt has occurred or not
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *			TRUE  - Receiver error interrupt has occurred
 *          FALSE - Receiver error interrupt has not occurred
 */

static Bool LLC_mcaspGetRcvErr (LLC_McaspHandle    hMcasp)
{
    Bool val = FALSE;

    if (NULL != hMcasp)
    {
        val = (Bool)(CSL_FEXT(hMcasp->regs->RSTAT, MCASP_RSTAT_RERR));
    }

    return val;
}

/**
 * \brief   LLC_mcaspGetRcvClkFail
 *
 *  This function checks whether receiver error interrupt has occurred or not
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *			TRUE  - Receive clock failure flag is set
 *          FALSE - Receive clock failure flag is not set
 */

static Bool LLC_mcaspGetRcvClkFail (LLC_McaspHandle    hMcasp)
{
    Bool val = FALSE;

    if (NULL != hMcasp)
    {
        val = (Bool)(CSL_FEXT(hMcasp->regs->RSTAT, MCASP_RSTAT_RCKFAIL));
    }

    return val;
}

/**
 * \brief   LLC_mcaspGetRcvSyncErr
 *
 *  This function checks whether unexpected receive frame sync flag is 
 *  set or not
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *			TRUE  - Unexpected receive frame sync error has occurred
 *          FALSE - Unexpected receive frame sync error has not occurred
 */

static Bool LLC_mcaspGetRcvSyncErr (LLC_McaspHandle    hMcasp)
{
    Bool val = FALSE;

    if (NULL != hMcasp)
    {
        val = (Bool)(CSL_FEXT(hMcasp->regs->RSTAT, MCASP_RSTAT_RSYNCERR));
    }
    return val;
}


/**
 * \brief   LLC_mcaspGetRcvOverrun
 *
 *  This function checks whether receiver overrun flag is set or not
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *			TRUE  - Receiver overrun flag is set
 *          FALSE - Receiver overrun flag is not set
 */

static Bool LLC_mcaspGetRcvOverrun (LLC_McaspHandle    hMcasp)
{
    Bool val = FALSE;

    if (NULL != hMcasp)
    {
        val = (Bool)(CSL_FEXT(hMcasp->regs->RSTAT, MCASP_RSTAT_ROVRN));
    }

    return val;
}

/**
 * \brief   LLC_mcaspGetRcvDataReady
 *
 *  This function checks whether receive data ready flag is set or not
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *			TRUE  - Receive data ready flag is set
 *          FALSE - Receive data ready flag is not set
 */

static Bool LLC_mcaspGetRcvDataReady (LLC_McaspHandle    hMcasp)
{
    Bool val = FALSE;

    if (NULL != hMcasp)
    {
        val = (Bool)(CSL_FEXT(hMcasp->regs->RSTAT, MCASP_RSTAT_RDATA));
    }
    return val;
}

/**
 * \brief   LLC_mcaspGetSerRcvReady
 *
 *  This function checks whether receive buffer ready bit of serializer
 *  control register is set or not
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \param     serRcvReady [IN]     Serailizer status will be stored here
 * \param     serNum      [IN]     Serailizer register to be checked
 * \return
 *			PSP_SOK  		- Command executed successfully
 *          CSL_ESYS_FAIL 	- If serNum passed is invalid.
 */

static Int16 LLC_mcaspGetSerRcvReady (LLC_McaspHandle hMcasp,
                                      Bool *serRcvReady,
                                      PSP_McaspSerializerNum  serNum)
{
    Int16 st = PSP_SOK;
    if((NULL != serRcvReady) && (NULL != hMcasp))
    {
        switch (serNum)
        {
            case SERIALIZER_0:
                *(Bool *)serRcvReady = ((Bool)CSL_FEXT(hMcasp->regs->SRCTL0,
                                       MCASP_SRCTL0_RRDY));
                break;
            case SERIALIZER_1:
                *(Bool *)serRcvReady = ((Bool)CSL_FEXT(hMcasp->regs->SRCTL1,
                                       MCASP_SRCTL1_RRDY));
                break;
            case SERIALIZER_2:
                *(Bool *)serRcvReady = ((Bool)CSL_FEXT(hMcasp->regs->SRCTL2,
                                       MCASP_SRCTL2_RRDY));
                break;
            case SERIALIZER_3:
                *(Bool *)serRcvReady = ((Bool)CSL_FEXT(hMcasp->regs->SRCTL3,
                                       MCASP_SRCTL3_RRDY));
                break;
#if defined(CHIP_DM648) || defined (CHIP_C6452)
            case SERIALIZER_4:
                *(Bool *)serRcvReady = ((Bool)CSL_FEXT(hMcasp->regs->SRCTL4,
                                       MCASP_SRCTL4_RRDY));
                break;
            case SERIALIZER_5:
                *(Bool *)serRcvReady = ((Bool)CSL_FEXT(hMcasp->regs->SRCTL5,
                                       MCASP_SRCTL5_RRDY));
                break;
            case SERIALIZER_6:
                *(Bool *)serRcvReady = ((Bool)CSL_FEXT(hMcasp->regs->SRCTL6,
                                       MCASP_SRCTL6_RRDY));
                break;
            case SERIALIZER_7:
                *(Bool *)serRcvReady = ((Bool)CSL_FEXT(hMcasp->regs->SRCTL7,
                                       MCASP_SRCTL7_RRDY));
                break;
            case SERIALIZER_8:
                *(Bool *)serRcvReady = ((Bool)CSL_FEXT(hMcasp->regs->SRCTL8,
                                       MCASP_SRCTL8_RRDY));
                break;
            case SERIALIZER_9:
                *(Bool *)serRcvReady = ((Bool)CSL_FEXT(hMcasp->regs->SRCTL9,
                                       MCASP_SRCTL9_RRDY));
                break;
#endif
            default:
                st = PSP_E_DRIVER_INIT;
        }
    }
    return st;
}

/**
 * \brief   LLC_mcaspGetSerXmtReady
 *
 *  This function checks whether transmit buffer ready bit of serializer
 *  control register is set or not
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \param     *serXmtReady[IN]     Status of the serializer will be stored here
 * \param     serNum      [IN]     Serailizer register to be checked
 * \return
 *			PSP_SOK  		- Command executed successfully
 *          CSL_ESYS_FAIL 	- If serNum passed is invalid.
 */

static Int16 LLC_mcaspGetSerXmtReady (LLC_McaspHandle   hMcasp,
                                      Bool  *serXmtReady,
                                      PSP_McaspSerializerNum  serNum)
{
    Int16 st = PSP_SOK;
    if((NULL != serXmtReady) && (NULL != hMcasp))
    {
        switch (serNum)
        {
            case SERIALIZER_0:
                *(Bool *)serXmtReady = ((Bool)CSL_FEXT(hMcasp->regs->SRCTL0,
                                       MCASP_SRCTL0_XRDY));
                break;

            case SERIALIZER_1:
                *(Bool *)serXmtReady = ((Bool)CSL_FEXT(hMcasp->regs->SRCTL1,
                                       MCASP_SRCTL1_XRDY));
                break;

            case SERIALIZER_2:
                *(Bool *)serXmtReady = ((Bool)CSL_FEXT(hMcasp->regs->SRCTL2,
                                       MCASP_SRCTL2_XRDY));
                break;

            case SERIALIZER_3:
                *(Bool *)serXmtReady = ((Bool)CSL_FEXT(hMcasp->regs->SRCTL3,
                                       MCASP_SRCTL3_XRDY));
                break;

#if defined(CHIP_DM648) || defined (CHIP_C6452)
            case SERIALIZER_4:
                *(Bool *)serXmtReady = ((Bool)CSL_FEXT(hMcasp->regs->SRCTL4,
                                       MCASP_SRCTL4_XRDY));
                break;

            case SERIALIZER_5:
                *(Bool *)serXmtReady = ((Bool)CSL_FEXT(hMcasp->regs->SRCTL5,
                                       MCASP_SRCTL5_XRDY));
                break;

            case SERIALIZER_6:
                *(Bool *)serXmtReady = ((Bool)CSL_FEXT(hMcasp->regs->SRCTL6,
                                       MCASP_SRCTL6_XRDY));
                break;

            case SERIALIZER_7:
                *(Bool *)serXmtReady = ((Bool)CSL_FEXT(hMcasp->regs->SRCTL7,
                                       MCASP_SRCTL7_XRDY));
                break;
            case SERIALIZER_8:
                *(Bool *)serXmtReady = ((Bool)CSL_FEXT(hMcasp->regs->SRCTL8,
                                       MCASP_SRCTL8_XRDY));
                break;

            case SERIALIZER_9:
                *(Bool *)serXmtReady = ((Bool)CSL_FEXT(hMcasp->regs->SRCTL9,
                                       MCASP_SRCTL9_XRDY));
                break;
#endif
            default:
                st = PSP_E_DRIVER_INIT;
        }
    }/* ((NULL != serXmtReady) && (NULL != hMcasp)) */
    return st;
}

/**
 * \brief   LLC_mcaspGetSerMode
 *
 *  This function gets the serializer mode
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \param     *serMode 	  [IN]     Serializer mode will be here
 * \param     serNum      [IN]     Serailizer register to be checked
 * \return
 *			PSP_SOK  		- Command executed successfully
 *          CSL_ESYS_FAIL 	- If serNum passed is invalid.
 */

static Int16 LLC_mcaspGetSerMode (LLC_McaspHandle         hMcasp,
                                  LLC_McaspSerMode        *serMode,
                                  PSP_McaspSerializerNum  serNum)
{
    Int16 st = PSP_SOK;
    if((NULL != serMode) && (NULL != hMcasp))
    {
        switch (serNum)
        {
            case SERIALIZER_0:
                *(LLC_McaspSerMode *)serMode = ((LLC_McaspSerMode)
                                               CSL_FEXT (hMcasp->regs->SRCTL0,
                                               MCASP_SRCTL0_SRMOD));
                break;
            case SERIALIZER_1:
                *(LLC_McaspSerMode *)serMode = ((LLC_McaspSerMode)
                                               CSL_FEXT(hMcasp->regs->SRCTL1,
                                               MCASP_SRCTL1_SRMOD));
                break;
            case SERIALIZER_2:
                *(LLC_McaspSerMode *)serMode = ((LLC_McaspSerMode)
                                               CSL_FEXT(hMcasp->regs->SRCTL2,
                                               MCASP_SRCTL2_SRMOD));
                break;
            case SERIALIZER_3:
                *(LLC_McaspSerMode *)serMode = ((LLC_McaspSerMode)
                                               CSL_FEXT(hMcasp->regs->SRCTL3,
                                               MCASP_SRCTL3_SRMOD));
                break;
#if defined(CHIP_DM648) || defined (CHIP_C6452)
            case SERIALIZER_4:
                *(LLC_McaspSerMode *)serMode = ((LLC_McaspSerMode)
                                               CSL_FEXT (hMcasp->regs->SRCTL4,
                                               MCASP_SRCTL4_SRMOD));
                break;
            case SERIALIZER_5:
                *(LLC_McaspSerMode *)serMode = ((LLC_McaspSerMode)
                                               CSL_FEXT(hMcasp->regs->SRCTL5,
                                               MCASP_SRCTL5_SRMOD));
                break;
            case SERIALIZER_6:
                *(LLC_McaspSerMode *)serMode = ((LLC_McaspSerMode)
                                               CSL_FEXT(hMcasp->regs->SRCTL6,
                                               MCASP_SRCTL6_SRMOD));
                break;
            case SERIALIZER_7:
                *(LLC_McaspSerMode *)serMode = ((LLC_McaspSerMode)
                                               CSL_FEXT(hMcasp->regs->SRCTL7,
                                               MCASP_SRCTL7_SRMOD));
                break;
            case SERIALIZER_8:
                *(LLC_McaspSerMode *)serMode = ((LLC_McaspSerMode)
                                               CSL_FEXT (hMcasp->regs->SRCTL8,
                                               MCASP_SRCTL8_SRMOD));
                break;
            case SERIALIZER_9:
                *(LLC_McaspSerMode *)serMode = ((LLC_McaspSerMode)
                                               CSL_FEXT(hMcasp->regs->SRCTL9,
                                               MCASP_SRCTL9_SRMOD));
                break;
#endif
            default:
                st = PSP_E_DRIVER_INIT;
            }
    }
    return st;
}

/**
 * \brief   LLC_mcaspGetXmtStat
 *
 *  This function returns the value of XSTAT register.
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *			Value of XSTAT register.
 */

static Uint16 LLC_mcaspGetXmtStat (LLC_McaspHandle hMcasp)
{
    Uint16 val = 0;

    if (NULL != hMcasp)
    {
        val = (Uint16) hMcasp->regs->XSTAT;
    }

    return val;
}

/**
 * \brief   LLC_mcaspGetRcvStat
 *
 *  This function returns the value of RSTAT register.
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *			Value of RSTAT register.
 */

static Uint16 LLC_mcaspGetRcvStat (LLC_McaspHandle hMcasp)
{
    Uint16 val = 0;

    if (NULL != hMcasp)
    {
        val = (Uint16) hMcasp->regs->RSTAT;
    }

    return val;
}

/**
 * \brief   LLC_mcaspGetSmFsXmt
 *
 *  This function returns the XSMRST and XFRST field values of XGBLCTL register.
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *			0x00 		- Both transmit frame generator sync and transmit state
 *                        machine are reset.
 *			0x1  		- Only transmit state machine is active.
 *			0x10 		- Only transmit frame sync generator is active.
 *			0x11 		- Both transmit frame generator sync and transmit state
 *                        machine are active.
 */

static Uint8 LLC_mcaspGetSmFsXmt (LLC_McaspHandle hMcasp)
{

    Uint8 smFsXmt = 0;
    if(NULL != hMcasp)
    {
        smFsXmt = ((Uint8) CSL_FEXT (hMcasp->regs->XGBLCTL, MCASP_XGBLCTL_XFRST));
        smFsXmt = (Uint8) (smFsXmt << 4);
        smFsXmt |= ((Uint8) CSL_FEXT (hMcasp->regs->XGBLCTL, MCASP_XGBLCTL_XSMRST));
    }/* (NULL != hMcasp) */
    return smFsXmt;
}

/**
 * \brief   LLC_mcaspGetSmFsRcv
 *
 *  This function returns the RSMRST and RFRST field values of RGBLCTL register.
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *			0x00 		- Both receive frame generator sync and receive state
 *                        machine are reset.
 *			0x1  		- Only receive state machine is active.
 *			0x10 		- Only receive frame sync generator is active.
 *			0x11 		- Both receive frame generator sync and receive state
 *                        machine are active.
 */

static Uint8 LLC_mcaspGetSmFsRcv (LLC_McaspHandle hMcasp)
{

    Uint8 smFsRcv = 0;
    if(NULL != hMcasp)
    {
        smFsRcv = ((Uint8) CSL_FEXT (hMcasp->regs->RGBLCTL, MCASP_RGBLCTL_RFRST));
        smFsRcv = (Uint8)(smFsRcv << 4);
        smFsRcv |= ((Uint8) CSL_FEXT (hMcasp->regs->RGBLCTL, MCASP_RGBLCTL_RSMRST));
    }/* (NULL != hMcasp) */
    return smFsRcv;
}

/**
 * \brief   LLC_mcaspGetDitMode
 *
 *  This function returns the status of DITEN bit in DITCTL register.
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *			TRUE 		- DIT Mode is enabled
 *			FALSE  		- DIT Mode is not enabled
 */

static Bool LLC_mcaspGetDitMode (LLC_McaspHandle hMcasp)
{
    Bool val = FALSE;

    if (NULL != hMcasp)
    {
        val = (Bool) (CSL_FEXT(hMcasp->regs->DITCTL, MCASP_DITCTL_DITEN));
    }

    return val;
}

/**
 *  Hardware Control Functions of McASP
 */

/**
 * \brief   LLC_mcaspConfigAudioMute
 *
 *  This function configures the AMUTE register with specified values
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \param     audioMute   [IN]     Value to be loaded to the register
 * \return
 *			None
 */

PAL_Result LLC_mcaspConfigAudioMute (LLC_McaspHandle hMcasp,Uint32 audioMute)
{
    PAL_Result result = PSP_SOK;

    if(NULL != hMcasp)
    {
        /* Configure AMUTE register */
        hMcasp->regs->AMUTE = audioMute;
    }/*(NULL != hMcasp)*/
    else
    {
        result = PSP_E_INVAL_PARAM;
    }
    return (result);
}

/**
 * \brief   LLC_mcaspConfigLoopBack
 *
 *  This function sets the digital loopback mode
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \param     loopBack    [IN]     Value to be loaded into the bit filed
 * \return
 *			None
 */

PAL_Result LLC_mcaspConfigLoopBack (LLC_McaspHandle hMcasp,Bool loopBack)
{

    Bool    loopBackEnable      = 0;
    Bool    orderBit            = 0;
    Int16   serNum              = 0;
    CSL_McaspRegsOvly tempAddr  = NULL;
    PAL_Result        result    = PSP_SOK;

    if(NULL != hMcasp)
    {
        tempAddr = hMcasp->regs;

        if (NULL != tempAddr)
        {
            /* Reset the RSRCLR and XSRCLR registers in GBLCTL */
            CSL_FINST (tempAddr->GBLCTL, MCASP_GBLCTL_RSRCLR, CLEAR);
            CSL_FINST (tempAddr->GBLCTL, MCASP_GBLCTL_XSRCLR, CLEAR);

            /* Reset the RSMRST and XSMRST registers in GBLCTL */
            CSL_FINST (tempAddr->GBLCTL, MCASP_GBLCTL_RSMRST, RESET);
            CSL_FINST (tempAddr->GBLCTL, MCASP_GBLCTL_XSMRST, RESET);

            /* Reset the RFRST and XFRST registers in GBLCTL */
            CSL_FINST (tempAddr->GBLCTL, MCASP_GBLCTL_RFRST, RESET);
            CSL_FINST (tempAddr->GBLCTL, MCASP_GBLCTL_XFRST, RESET);

            /* configure loop back mode */
            CSL_FINS (tempAddr->DLBCTL, MCASP_DLBCTL_DLBEN, loopBack);

            loopBackEnable = (Bool)CSL_FEXT (tempAddr->DLBCTL, MCASP_DLBCTL_DLBEN);

            if (loopBackEnable == TRUE)
            {
                CSL_FINST (tempAddr->DLBCTL, MCASP_DLBCTL_MODE, XMTCLK);
            }

            orderBit = (Bool)CSL_FEXT (tempAddr->DLBCTL, MCASP_DLBCTL_ORD);

            if (orderBit == TRUE)
            {

            while (serNum < hMcasp->numOfSerializers)
                {


                    switch(serNum )
                    {
#if defined(CHIP_DM648) || defined (CHIP_C6452)
                    case 9:
                        if(CSL_MCASP_SRCTL9_SRMOD_INACTIVE \
                            !=CSL_FEXT (tempAddr->SRCTL9, MCASP_SRCTL9_SRMOD))
                        {
                            CSL_FINST (tempAddr->SRCTL9, MCASP_SRCTL9_SRMOD, RCV);
                            CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AXR9, INPUT);
                        }
                    break;
                    case 8:
                        if(CSL_MCASP_SRCTL8_SRMOD_INACTIVE \
                            !=CSL_FEXT (tempAddr->SRCTL8, MCASP_SRCTL8_SRMOD))
                        {
                            CSL_FINST (tempAddr->SRCTL8, MCASP_SRCTL8_SRMOD, XMT);
                            CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AXR8, OUTPUT);
                        }
                        break;
                    case 7:
                        if(CSL_MCASP_SRCTL7_SRMOD_INACTIVE \
                            !=CSL_FEXT (tempAddr->SRCTL7, MCASP_SRCTL7_SRMOD))
                        {                  
                            CSL_FINST (tempAddr->SRCTL7, MCASP_SRCTL7_SRMOD, RCV);        
                            CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AXR7, INPUT);
                        }
                    break;
                    case 6:
                        if(CSL_MCASP_SRCTL6_SRMOD_INACTIVE \
                            !=CSL_FEXT (tempAddr->SRCTL6, MCASP_SRCTL6_SRMOD))
                        {
                            CSL_FINST (tempAddr->SRCTL6, MCASP_SRCTL6_SRMOD, XMT);
                            CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AXR6, OUTPUT);
                        }
                    break;
                    case 5:
                        if(CSL_MCASP_SRCTL5_SRMOD_INACTIVE \
                            !=CSL_FEXT (tempAddr->SRCTL5, MCASP_SRCTL5_SRMOD))
                        {
                            CSL_FINST (tempAddr->SRCTL5, MCASP_SRCTL5_SRMOD, RCV);
                            CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AXR5, INPUT);
                        }
                    break;
                    case 4:
                        if(CSL_MCASP_SRCTL4_SRMOD_INACTIVE \
                            !=CSL_FEXT (tempAddr->SRCTL4, MCASP_SRCTL4_SRMOD))
                        {
                            CSL_FINST (tempAddr->SRCTL4, MCASP_SRCTL4_SRMOD, XMT);
                            CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AXR4, OUTPUT);
                        }
                        break;
#endif
                    case 3:
                        if(CSL_MCASP_SRCTL3_SRMOD_INACTIVE \
                            !=CSL_FEXT (tempAddr->SRCTL3, MCASP_SRCTL3_SRMOD))
                        {                  
                            CSL_FINST (tempAddr->SRCTL3, MCASP_SRCTL3_SRMOD, RCV);
                            CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AXR3, INPUT);
                        }
                    break;
                    case 2:
                        if(CSL_MCASP_SRCTL2_SRMOD_INACTIVE \
                            !=CSL_FEXT (tempAddr->SRCTL2, MCASP_SRCTL2_SRMOD))
                        {
                            CSL_FINST (tempAddr->SRCTL2, MCASP_SRCTL2_SRMOD, XMT);
                            CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AXR2, OUTPUT);
                        }
                    break;
                    case 1:
                        if(CSL_MCASP_SRCTL1_SRMOD_INACTIVE \
                            !=CSL_FEXT (tempAddr->SRCTL1, MCASP_SRCTL1_SRMOD))
                        {
                            CSL_FINST (tempAddr->SRCTL1, MCASP_SRCTL1_SRMOD, RCV);
                            CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AXR1, INPUT);
                        }
                    break;
                    case 0:
                        if(CSL_MCASP_SRCTL0_SRMOD_INACTIVE \
                            !=CSL_FEXT (tempAddr->SRCTL0, MCASP_SRCTL0_SRMOD))
                        {
                            CSL_FINST (tempAddr->SRCTL0, MCASP_SRCTL0_SRMOD, XMT);
                            CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AXR0, OUTPUT);
                        }
                    break;

                    default:

                    break;
                    }
                    serNum++;
                }

            }
            else {

                while (serNum < hMcasp->numOfSerializers)
                {

                    switch(serNum)
                    {
#if defined(CHIP_DM648) || defined (CHIP_C6452)
                    case 9:
                        if(CSL_MCASP_SRCTL9_SRMOD_INACTIVE \
                            !=CSL_FEXT (tempAddr->SRCTL9, MCASP_SRCTL9_SRMOD))
                        {
                            CSL_FINST (tempAddr->SRCTL9, MCASP_SRCTL9_SRMOD, XMT);
                            CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AXR9, OUTPUT);
                        }
                    break;
                    case 8:
                        if(CSL_MCASP_SRCTL8_SRMOD_INACTIVE \
                            !=CSL_FEXT (tempAddr->SRCTL8, MCASP_SRCTL8_SRMOD))
                        {
                            CSL_FINST (tempAddr->SRCTL8, MCASP_SRCTL8_SRMOD, RCV);
                            CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AXR8, INPUT);
                        }
                    break;
                    case 7:
                        if(CSL_MCASP_SRCTL7_SRMOD_INACTIVE \
                            !=CSL_FEXT (tempAddr->SRCTL7, MCASP_SRCTL7_SRMOD))
                        {
                            CSL_FINST (tempAddr->SRCTL7, MCASP_SRCTL7_SRMOD, XMT);
                            CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AXR7, OUTPUT);
                        }
                    break;
                    case 6:
                        if(CSL_MCASP_SRCTL6_SRMOD_INACTIVE \
                            !=CSL_FEXT (tempAddr->SRCTL6, MCASP_SRCTL6_SRMOD))
                        {
                            CSL_FINST (tempAddr->SRCTL6, MCASP_SRCTL6_SRMOD, RCV);
                            CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AXR6, INPUT);
                        }
                    break;
                    case 5:
                        if(CSL_MCASP_SRCTL5_SRMOD_INACTIVE \
                            !=CSL_FEXT (tempAddr->SRCTL5, MCASP_SRCTL5_SRMOD))
                        {
                            CSL_FINST (tempAddr->SRCTL5, MCASP_SRCTL5_SRMOD, XMT);
                            CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AXR5, OUTPUT);
                        }
                    break;
                    case 4:
                        if(CSL_MCASP_SRCTL4_SRMOD_INACTIVE \
                            !=CSL_FEXT (tempAddr->SRCTL4, MCASP_SRCTL4_SRMOD))
                        {
                            CSL_FINST (tempAddr->SRCTL4, MCASP_SRCTL4_SRMOD, RCV);
                            CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AXR4, INPUT);
                        }
                    break;
#endif
                    case 3:
                        if(CSL_MCASP_SRCTL3_SRMOD_INACTIVE \
                            !=CSL_FEXT (tempAddr->SRCTL3, MCASP_SRCTL3_SRMOD))
                        {
                            CSL_FINST (tempAddr->SRCTL3, MCASP_SRCTL3_SRMOD, XMT);
                            CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AXR3, OUTPUT);
                        }
                    break;
                    case 2:
                        if(CSL_MCASP_SRCTL2_SRMOD_INACTIVE \
                            !=CSL_FEXT (tempAddr->SRCTL2, MCASP_SRCTL2_SRMOD))
                        {
                            CSL_FINST (tempAddr->SRCTL2, MCASP_SRCTL2_SRMOD, RCV);
                            CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AXR2, INPUT);
                        }
                    break;
                    case 1:
                        if(CSL_MCASP_SRCTL1_SRMOD_INACTIVE \
                            !=CSL_FEXT (tempAddr->SRCTL1, MCASP_SRCTL1_SRMOD))
                        {
                            CSL_FINST (tempAddr->SRCTL1, MCASP_SRCTL1_SRMOD, XMT);
                            CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AXR1, OUTPUT);
                        }
                    break;
                    case 0:
                        if(CSL_MCASP_SRCTL0_SRMOD_INACTIVE \
                            !=CSL_FEXT (tempAddr->SRCTL0, MCASP_SRCTL0_SRMOD))
                        {
                            CSL_FINST (tempAddr->SRCTL0, MCASP_SRCTL0_SRMOD, RCV);
                            CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AXR0, INPUT);
                        }
                    break;
                    default:

                    break;
                    }
                    serNum++;
                }
            }
        } /* (NULL != tempAddr) */

    }/* (NULL != hMcasp) */
    else
    {
        result = PSP_E_INVAL_PARAM;
    }
    return (result);
}

/**
 * \brief   LLC_mcaspConfigXmtSection
 *
 *  This function configures format, frame sync, and other parameters
 *  related to the xmt section. Also configures the xmt clk section.
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \param     xmtData     [IN]     transmit related parameters
 * \return
 *			None
 */

PAL_Result LLC_mcaspConfigXmtSection (LLC_McaspHandle hMcasp,
                                        PSP_McaspHwSetupData  *const xmtData)
{

    volatile Uint32         bitValue = 0;
    CSL_McaspRegsOvly         tempAddr = NULL;
    PAL_Result                result     = PAL_SOK;

    if(NULL != hMcasp)
    {
        tempAddr = hMcasp->regs;

        if((NULL != xmtData)&&(NULL != tempAddr))
        {
            /* Configure TXMASK register */
            tempAddr->XMASK     = xmtData->mask;

            /* Reset the XSMRST bit in GBLCTL register */
            CSL_FINST (tempAddr->GBLCTL, MCASP_GBLCTL_XSMRST, RESET);

            /* Configure TXFMT register */
            tempAddr->XFMT      = xmtData->fmt;

            /*Reset the XFRST register in GBLCTL */
            CSL_FINST (tempAddr->GBLCTL, MCASP_GBLCTL_XFRST, RESET);

            /* Configure TXFMCTL register */
            tempAddr->AFSXCTL = xmtData->frSyncCtl;

            if (CSL_MCASP_AFSXCTL_FSXM_INTERNAL ==
               (CSL_FEXT (tempAddr->AFSXCTL, MCASP_AFSXCTL_FSXM)))
            {
                CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AFSX, OUTPUT);
            }

            /* Reset XHCLKRST, XCLKRST, XSRCLR  in GBLCTL */
            CSL_FINST (tempAddr->GBLCTL, MCASP_GBLCTL_XHCLKRST, RESET);
            CSL_FINST (tempAddr->GBLCTL, MCASP_GBLCTL_XCLKRST, RESET);
            CSL_FINST (tempAddr->GBLCTL, MCASP_GBLCTL_XSRCLR, CLEAR);


            /* Configure XTDM register */
            tempAddr->XTDM =  xmtData->tdm;

            /* Configure XINTCTL register */
            tempAddr->XINTCTL =  xmtData->intCtl;

            /* Configure XCLKCHK register */
            tempAddr->XCLKCHK   =  xmtData->clk.clkChk;

            /* Configure XEVTCTL register */
            tempAddr->XEVTCTL =  xmtData->evtCtl;

           /* Sequence of start: starting hclk first*/
            CSL_FINST (tempAddr->XGBLCTL, MCASP_XGBLCTL_XHCLKRST, ACTIVE);
            {
                bitValue = 0;
                while (CSL_MCASP_GBLCTL_XHCLKRST_ACTIVE != bitValue)
                {
                    bitValue = CSL_FEXT (tempAddr->GBLCTL,
                               MCASP_GBLCTL_XHCLKRST);
                }
            }

             /* start ACLKX only if internal clock is used*/
            if (CSL_MCASP_ACLKXCTL_CLKXM_INTERNAL ==
               (CSL_FEXT (tempAddr->ACLKXCTL, MCASP_ACLKXCTL_CLKXM)))
               {
                    CSL_FINST (tempAddr->XGBLCTL, MCASP_XGBLCTL_XCLKRST, ACTIVE);
                    {
                        bitValue = 0;
                        while (CSL_MCASP_GBLCTL_XCLKRST_ACTIVE != bitValue)
                        {
                            bitValue = CSL_FEXT (tempAddr->GBLCTL,
                                       MCASP_GBLCTL_XCLKRST);
                        }
                    }
                }

            /* Configure ACLKXCTL register */
            tempAddr->ACLKXCTL  = xmtData->clk.clkSetupClk;

            if (CSL_MCASP_ACLKXCTL_CLKXM_INTERNAL ==
               (CSL_FEXT (tempAddr->ACLKXCTL, MCASP_ACLKXCTL_CLKXM)))
            {
                CSL_FINST (tempAddr->PDIR, MCASP_PDIR_ACLKX, OUTPUT);
            }

            /* Configure AHCLKXCTL register */
            tempAddr->AHCLKXCTL = xmtData->clk.clkSetupHiClk;

            if (CSL_MCASP_AHCLKXCTL_HCLKXM_INTERNAL ==
               (CSL_FEXT (tempAddr->AHCLKXCTL, MCASP_AHCLKXCTL_HCLKXM)))
            {
                CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AHCLKX, OUTPUT);
            }

        }
        else
        {
            result = PSP_E_INVAL_PARAM;
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    return(result);
}

/**
 * \brief   LLC_mcaspConfigRcvSection
 *
 *  Configure the format, frame sync, and other parameters related to the
 *  rcv section. Also configures the rcv clk section.
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \param     rcvData     [IN]     Receive related parameters
 * \return
 *			None
 */

PAL_Result LLC_mcaspConfigRcvSection (LLC_McaspHandle           hMcasp,
                                      PSP_McaspHwSetupData      *const rcvData)
{

    Uint32                     bitValue = 0;
    CSL_McaspRegsOvly         tempAddr = NULL;
    PAL_Result                result     = PSP_SOK;

    if(NULL != hMcasp)
    {
        tempAddr = hMcasp->regs;

        if((NULL != rcvData) && (NULL != tempAddr))
        {
            /* Configure RMASK register */
            tempAddr->RMASK = rcvData->mask;

            /* Reset the RSMRST bit in GBLCTL register */
            CSL_FINST (tempAddr->GBLCTL, MCASP_GBLCTL_RSMRST, RESET);

            /* Configure RFMT register */
            tempAddr->RFMT = rcvData->fmt;

            /*Reset the RFRST register in GBLCTL */
            CSL_FINST (tempAddr->GBLCTL, MCASP_GBLCTL_RFRST, RESET);

            /* Configure RXFMCTL register */
            tempAddr->AFSRCTL =  rcvData->frSyncCtl;

            if (CSL_MCASP_AFSRCTL_FSRM_INTERNAL ==
               (CSL_FEXT (tempAddr->AFSRCTL, MCASP_AFSRCTL_FSRM)))
            {
                CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AFSR, OUTPUT);
            }

            /* Reset RHCLKRST, RCLKRST, RSRCLR in GBLCTL */
            CSL_FINST (tempAddr->GBLCTL, MCASP_GBLCTL_RHCLKRST, RESET);
            CSL_FINST (tempAddr->GBLCTL, MCASP_GBLCTL_RCLKRST, RESET);
            CSL_FINST (tempAddr->GBLCTL, MCASP_GBLCTL_RSRCLR, CLEAR);


            /*Configure it completely before starting the clocks*/
            /* Configure RTDM register */
            tempAddr->RTDM =  rcvData->tdm;

            /* Configure EVTCTLR register */
            tempAddr->RINTCTL =  rcvData->intCtl;

            /* Configure RXCLKCHK register */
            tempAddr->RCLKCHK   =  rcvData->clk.clkChk;

            /* Configure REVTCTL register */
            tempAddr->REVTCTL =  rcvData->evtCtl;


           /* Sequence of start: starting hclk first*/
           /* start AHCLKR */
            CSL_FINST (tempAddr->GBLCTL, MCASP_GBLCTL_RHCLKRST, ACTIVE);
            {
                bitValue = 0;
                while (CSL_MCASP_GBLCTL_RHCLKRST_ACTIVE != bitValue)
                {
                    bitValue = CSL_FEXT (tempAddr->GBLCTL,
                               MCASP_GBLCTL_RHCLKRST);
                }
            }

             /* start ACLKR only if internal clock is used*/
             if (CSL_MCASP_ACLKRCTL_CLKRM_INTERNAL ==
                   (CSL_FEXT (tempAddr->ACLKRCTL, MCASP_ACLKRCTL_CLKRM)))
              {
                CSL_FINST (tempAddr->GBLCTL, MCASP_GBLCTL_RCLKRST, ACTIVE);
                    {
                        bitValue = 0;
                        while (CSL_MCASP_GBLCTL_RCLKRST_ACTIVE != bitValue)
                        {
                            bitValue = CSL_FEXT (tempAddr->GBLCTL, MCASP_GBLCTL_RCLKRST);
                        }
                    }
               }
                        /* Configure ACLKRCTL register */
            tempAddr->ACLKRCTL  = rcvData->clk.clkSetupClk;

            if (CSL_MCASP_ACLKRCTL_CLKRM_INTERNAL ==
               (CSL_FEXT (tempAddr->ACLKRCTL, MCASP_ACLKRCTL_CLKRM)))
            {
                CSL_FINST (tempAddr->PDIR, MCASP_PDIR_ACLKR, OUTPUT);
            }

            /* Configure AHCLKRCTL register */
            tempAddr->AHCLKRCTL = rcvData->clk.clkSetupHiClk;

            if (CSL_MCASP_AHCLKRCTL_HCLKRM_INTERNAL ==
               (CSL_FEXT (tempAddr->AHCLKRCTL, MCASP_AHCLKRCTL_HCLKRM)))
            {
                CSL_FINST (tempAddr->PDIR, MCASP_PDIR_AHCLKR, OUTPUT);
            }

        } /* ((NULL != rcvData) && (NULL != tempAddr)) */
        else
        {
            result = PSP_E_INVAL_PARAM;
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    return(result);
}

/**
 * \brief   LLC_mcaspSetSerXmt
 *
 *  This function sets a particular serializer to act as transmitter
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \param     serNum      [IN]     serializer number
 * \return
 *			None
 */

PAL_Result LLC_mcaspSetSerXmt (LLC_McaspHandle hMcasp,
                                 PSP_McaspSerializerNum  serNum)
{
    PAL_Result    result = PSP_SOK;

    if(NULL != hMcasp)
    {
        switch (serNum)
        {
            case SERIALIZER_0:
                CSL_FINST ( hMcasp->regs->SRCTL0, MCASP_SRCTL0_SRMOD, XMT);
                CSL_FINST ( hMcasp->regs->PDIR, MCASP_PDIR_AXR0, OUTPUT);
                break;

            case SERIALIZER_1:
                CSL_FINST ( hMcasp->regs->SRCTL1, MCASP_SRCTL1_SRMOD, XMT);
                CSL_FINST ( hMcasp->regs->PDIR, MCASP_PDIR_AXR1, OUTPUT);
                break;

            case SERIALIZER_2:
                CSL_FINST ( hMcasp->regs->SRCTL2, MCASP_SRCTL2_SRMOD, XMT);
                CSL_FINST ( hMcasp->regs->PDIR, MCASP_PDIR_AXR2, OUTPUT);
                break;

            case SERIALIZER_3:
                CSL_FINST ( hMcasp->regs->SRCTL3, MCASP_SRCTL3_SRMOD, XMT);
                CSL_FINST ( hMcasp->regs->PDIR, MCASP_PDIR_AXR3, OUTPUT);
                break;
#if defined(CHIP_DM648) || defined (CHIP_C6452)

            case SERIALIZER_4:
                CSL_FINST ( hMcasp->regs->SRCTL4, MCASP_SRCTL4_SRMOD, XMT);
                CSL_FINST ( hMcasp->regs->PDIR, MCASP_PDIR_AXR4, OUTPUT);
                break;

            case SERIALIZER_5:
                CSL_FINST ( hMcasp->regs->SRCTL5, MCASP_SRCTL5_SRMOD, XMT);
                CSL_FINST ( hMcasp->regs->PDIR, MCASP_PDIR_AXR5, OUTPUT);
                break;

            case SERIALIZER_6:
                CSL_FINST ( hMcasp->regs->SRCTL6, MCASP_SRCTL6_SRMOD, XMT);
                CSL_FINST ( hMcasp->regs->PDIR, MCASP_PDIR_AXR6, OUTPUT);
                break;

            case SERIALIZER_7:
                CSL_FINST ( hMcasp->regs->SRCTL7, MCASP_SRCTL7_SRMOD, XMT);
                CSL_FINST ( hMcasp->regs->PDIR, MCASP_PDIR_AXR7, OUTPUT);
                break;
            case SERIALIZER_8:
                CSL_FINST ( hMcasp->regs->SRCTL8, MCASP_SRCTL8_SRMOD, XMT);
                CSL_FINST ( hMcasp->regs->PDIR, MCASP_PDIR_AXR8, OUTPUT);
                break;

            case SERIALIZER_9:
                CSL_FINST ( hMcasp->regs->SRCTL9, MCASP_SRCTL9_SRMOD, XMT);
                CSL_FINST ( hMcasp->regs->PDIR, MCASP_PDIR_AXR9, OUTPUT);
                break;
#endif
            default :
                break;
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    return (result);
}

/**
 * \brief   LLC_mcaspSetSerRcv
 *
 *  This function sets a particular serializer to act as receiver
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \param     serNum      [IN]     serializer number
 * \return
 *			None
 */

PAL_Result LLC_mcaspSetSerRcv (LLC_McaspHandle         hMcasp,
                                 PSP_McaspSerializerNum  serNum)
{
    PAL_Result result = PSP_SOK;

    if(NULL != hMcasp)
    {
        switch (serNum)
        {
            case SERIALIZER_0:
                CSL_FINST ( hMcasp->regs->SRCTL0, MCASP_SRCTL0_SRMOD, RCV);
                CSL_FINST ( hMcasp->regs->PDIR, MCASP_PDIR_AXR0, INPUT);
                break;

            case SERIALIZER_1:
                CSL_FINST ( hMcasp->regs->SRCTL1, MCASP_SRCTL1_SRMOD, RCV);
                CSL_FINST ( hMcasp->regs->PDIR, MCASP_PDIR_AXR1, INPUT);
                break;

            case SERIALIZER_2:
                CSL_FINST ( hMcasp->regs->SRCTL2, MCASP_SRCTL2_SRMOD, RCV);
                CSL_FINST ( hMcasp->regs->PDIR, MCASP_PDIR_AXR2, INPUT);
                break;

            case SERIALIZER_3:
                CSL_FINST ( hMcasp->regs->SRCTL3, MCASP_SRCTL3_SRMOD, RCV);
                CSL_FINST ( hMcasp->regs->PDIR, MCASP_PDIR_AXR3, INPUT);
                break;
#if defined(CHIP_DM648) || defined (CHIP_C6452)
            case SERIALIZER_4:
                CSL_FINST ( hMcasp->regs->SRCTL4, MCASP_SRCTL4_SRMOD, RCV);
                CSL_FINST ( hMcasp->regs->PDIR, MCASP_PDIR_AXR4, INPUT);
                break;

            case SERIALIZER_5:
                CSL_FINST ( hMcasp->regs->SRCTL5, MCASP_SRCTL5_SRMOD, RCV);
                CSL_FINST ( hMcasp->regs->PDIR, MCASP_PDIR_AXR5, INPUT);
                break;

            case SERIALIZER_6:
                CSL_FINST ( hMcasp->regs->SRCTL6, MCASP_SRCTL6_SRMOD, RCV);
                CSL_FINST ( hMcasp->regs->PDIR, MCASP_PDIR_AXR6, INPUT);
                break;

            case SERIALIZER_7:
                CSL_FINST ( hMcasp->regs->SRCTL7, MCASP_SRCTL7_SRMOD, RCV);
                CSL_FINST ( hMcasp->regs->PDIR, MCASP_PDIR_AXR7, INPUT);
                break;
            case SERIALIZER_8:
                CSL_FINST ( hMcasp->regs->SRCTL8, MCASP_SRCTL8_SRMOD, RCV);
                CSL_FINST ( hMcasp->regs->PDIR, MCASP_PDIR_AXR8, INPUT);
                break;

            case SERIALIZER_9:
                CSL_FINST ( hMcasp->regs->SRCTL9, MCASP_SRCTL9_SRMOD, RCV);
                CSL_FINST ( hMcasp->regs->PDIR, MCASP_PDIR_AXR9, INPUT);
                break;
#endif
            default :
                break;
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    return (result);
}

/**
 * \brief   LLC_mcaspWriteChanStatRam
 *
 *  This function writes to the Channel status RAM (DITCSRA/B0-5)
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \param     chanStatRam [IN]     status to be written
 * \return
 *			None
 */

PAL_Result LLC_mcaspWriteChanStatRam (LLC_McaspHandle             hMcasp,
                                PSP_McaspChStatusRam       *const chanStatRam)
{
    CSL_McaspRegsOvly     tempAddr     = NULL;
    PAL_Result            result        = PAL_SOK;

    if((NULL != hMcasp) && (NULL != chanStatRam))
    {
        tempAddr = hMcasp->regs;

        if (NULL != tempAddr)
        {
            if (TRUE == hMcasp->ditStatus)
            {
                /* Configure the DIT left channel status registers */
                tempAddr->DITCSRA0 = chanStatRam->chStatusLeft[DIT_REGISTER_0];

                tempAddr->DITCSRA1 = chanStatRam->chStatusLeft[DIT_REGISTER_1];

                tempAddr->DITCSRA2 = chanStatRam->chStatusLeft[DIT_REGISTER_2];

                tempAddr->DITCSRA3 = chanStatRam->chStatusLeft[DIT_REGISTER_3];

                tempAddr->DITCSRA4 = chanStatRam->chStatusLeft[DIT_REGISTER_4];

                tempAddr->DITCSRA5 = chanStatRam->chStatusLeft[DIT_REGISTER_5];

                /* Configure the DIT right channel status registers */
                tempAddr->DITCSRB0 = chanStatRam->chStatusRight[DIT_REGISTER_0];

                tempAddr->DITCSRB1 = chanStatRam->chStatusRight[DIT_REGISTER_1];

                tempAddr->DITCSRB2 = chanStatRam->chStatusRight[DIT_REGISTER_2];

                tempAddr->DITCSRB3 = chanStatRam->chStatusRight[DIT_REGISTER_3];

                tempAddr->DITCSRB4 = chanStatRam->chStatusRight[DIT_REGISTER_4];

                tempAddr->DITCSRB5 = chanStatRam->chStatusRight[DIT_REGISTER_5];
            }
        }
        else
        {
            result = PSP_E_INVAL_PARAM;
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }
    return (result);

}

/**
 * \brief   LLC_mcaspWriteUserDataRam
 *
 *  This function writes to the User Data RAM (DITUDRA/B0-5)
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \param     userDataRam [IN]     data to be written
 * \return
 *			None
 */

PAL_Result LLC_mcaspWriteUserDataRam (LLC_McaspHandle             hMcasp,
                                       PSP_McaspUserDataRam       *const userDataRam)
{
    CSL_McaspRegsOvly     tempAddr     = NULL;
    PAL_Result            result         = PSP_SOK;

    if((NULL != hMcasp) && (NULL != userDataRam))
    {
        tempAddr = hMcasp->regs;

        if (NULL != tempAddr)
        {
            if (TRUE == hMcasp->ditStatus)
            {
                /* Configure the DIT left user data registers */
                tempAddr->DITUDRA0 = userDataRam->userDataLeft[DIT_REGISTER_0];

                tempAddr->DITUDRA1 = userDataRam->userDataLeft[DIT_REGISTER_1];

                tempAddr->DITUDRA2 = userDataRam->userDataLeft[DIT_REGISTER_2];

                tempAddr->DITUDRA3 = userDataRam->userDataLeft[DIT_REGISTER_3];

                tempAddr->DITUDRA4 = userDataRam->userDataLeft[DIT_REGISTER_4];

                tempAddr->DITUDRA5 = userDataRam->userDataLeft[DIT_REGISTER_5];

                /* Configure the DIT right user data registers */
                tempAddr->DITUDRB0 = userDataRam->userDataRight[DIT_REGISTER_0];

                tempAddr->DITUDRB1 = userDataRam->userDataRight[DIT_REGISTER_1];

                tempAddr->DITUDRB2 = userDataRam->userDataRight[DIT_REGISTER_2];

                tempAddr->DITUDRB3 = userDataRam->userDataRight[DIT_REGISTER_3];

                tempAddr->DITUDRB4 = userDataRam->userDataRight[DIT_REGISTER_4];

                tempAddr->DITUDRB5 = userDataRam->userDataRight[DIT_REGISTER_5];
            }
            else
            {
                result = PSP_E_INVAL_PARAM;
            }
        }
        else
        {
            result = PSP_E_INVAL_PARAM;
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    return (result);

}

/**
 * \brief   LLC_mcaspResetXmt
 *
 *  This function resets the bits related to transmit in XGBLCTL.
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *			None
 */

PAL_Result LLC_mcaspResetXmt (LLC_McaspHandle     hMcasp)
{
    CSL_McaspRegsOvly     tempAddr = NULL;
    PAL_Result            result     = PSP_SOK;

    if(NULL != hMcasp)
    {
        tempAddr = hMcasp->regs;
        if (NULL != tempAddr)
        {
            CSL_FINST (tempAddr->XGBLCTL, MCASP_XGBLCTL_XCLKRST, RESET);
            CSL_FINST (tempAddr->XGBLCTL, MCASP_XGBLCTL_XHCLKRST, RESET);
            CSL_FINST (tempAddr->XGBLCTL, MCASP_XGBLCTL_XSRCLR, CLEAR);
            CSL_FINST (tempAddr->XGBLCTL, MCASP_XGBLCTL_XSMRST, RESET);
            CSL_FINST (tempAddr->XGBLCTL, MCASP_XGBLCTL_XFRST, RESET);
        }
        else
        {
            result = PSP_E_INVAL_PARAM;
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    return (result);
}

/**
 * \brief   LLC_mcaspResetRcv
 *
 *  This function resets the bits related to receive in RGBLCTL.
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *			None
 */

PAL_Result LLC_mcaspResetRcv (LLC_McaspHandle     hMcasp)
{
    CSL_McaspRegsOvly tempAddr     = NULL;
    PAL_Result          result    = PSP_SOK;

    if(NULL != hMcasp)
    {
        tempAddr = hMcasp->regs;
        if (NULL != tempAddr)
        {
            CSL_FINST (tempAddr->RGBLCTL, MCASP_RGBLCTL_RCLKRST, RESET);
            CSL_FINST (tempAddr->RGBLCTL, MCASP_RGBLCTL_RHCLKRST, RESET);
            CSL_FINST (tempAddr->RGBLCTL, MCASP_RGBLCTL_RSRCLR, CLEAR);
            CSL_FINST (tempAddr->RGBLCTL, MCASP_RGBLCTL_RSMRST, RESET);
            CSL_FINST (tempAddr->RGBLCTL, MCASP_RGBLCTL_RFRST, RESET);
        }
        else
        {
            result = PSP_E_INVAL_PARAM;
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    return (result);
}

/**
 * \brief   LLC_mcaspResetSmFsXmt
 *
 *  This function resets the XFRST and XSMRST bits in XGBLCTL.
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *			None
 */

PAL_Result LLC_mcaspResetSmFsXmt (LLC_McaspHandle     hMcasp)
{
    PAL_Result    result = PSP_SOK;

    if(NULL != hMcasp)
    {
        CSL_FINST (hMcasp->regs->XGBLCTL, MCASP_XGBLCTL_XSMRST, RESET);
        CSL_FINST (hMcasp->regs->XGBLCTL, MCASP_XGBLCTL_XFRST, RESET);
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    return (result);
}

/**
 * \brief   LLC_mcaspResetSmFsRcv
 *
 *  This function resets the RFRST and RSMRST bits in RGBLCTL.
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *			None
 */

PAL_Result LLC_mcaspResetSmFsRcv (LLC_McaspHandle     hMcasp)
{
    PAL_Result result = PSP_SOK;

    if(NULL != hMcasp)
    {
        CSL_FINST (hMcasp->regs->RGBLCTL, MCASP_RGBLCTL_RSMRST, RESET);
        CSL_FINST (hMcasp->regs->RGBLCTL, MCASP_RGBLCTL_RFRST, RESET);
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    return (result);
}

/**
 * \brief   LLC_mcaspActivateXmtClkSer
 *
 *  This function sets the bits related to transmit in XGBLCTL.
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *			None
 */

PAL_Result LLC_mcaspActivateXmtClkSer (LLC_McaspHandle     hMcasp)
{

    Uint32 bitValue = 0;
    CSL_McaspRegsOvly tempAddr = NULL;
    PAL_Result        result = PSP_SOK;

    if(NULL != hMcasp)
    {
        tempAddr = hMcasp->regs;

        if (NULL != tempAddr)
        {
            /* Sequence of start: starting hclk first*/
            CSL_FINST (tempAddr->XGBLCTL, MCASP_XGBLCTL_XHCLKRST, ACTIVE);
            {
                bitValue = 0;
                while (CSL_MCASP_GBLCTL_XHCLKRST_ACTIVE != bitValue)
                {
                    bitValue = CSL_FEXT (tempAddr->GBLCTL,
                               MCASP_GBLCTL_XHCLKRST);
                }
            }

             /* start ACLKX only if internal clock is used*/
            if (CSL_MCASP_ACLKXCTL_CLKXM_INTERNAL ==
               (CSL_FEXT (tempAddr->ACLKXCTL, MCASP_ACLKXCTL_CLKXM)))
               {
                    CSL_FINST (tempAddr->XGBLCTL, MCASP_XGBLCTL_XCLKRST, ACTIVE);
                    {
                        bitValue = 0;
                        while (CSL_MCASP_GBLCTL_XCLKRST_ACTIVE != bitValue)
                        {
                            bitValue = CSL_FEXT (tempAddr->GBLCTL,
                                       MCASP_GBLCTL_XCLKRST);
                        }
                    }
                }

            CSL_FINST (tempAddr->XGBLCTL, MCASP_XGBLCTL_XSRCLR, ACTIVE);
            {
                bitValue = 0;
                while (CSL_MCASP_GBLCTL_XSRCLR_ACTIVE != bitValue)
                {
                    bitValue = CSL_FEXT (tempAddr->GBLCTL, MCASP_GBLCTL_XSRCLR);
                }
            }
        }
        else
        {
            result = PSP_E_INVAL_PARAM;
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    return (result);
}

/**
 * \brief   LLC_mcaspActivateRcvClkSer
 *
 *  This function sets the bits related to receive in RGBLCTL.
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *			None
 */

PAL_Result LLC_mcaspActivateRcvClkSer (LLC_McaspHandle     hMcasp)
{

    Uint32 bitValue = 0;
    CSL_McaspRegsOvly tempAddr = NULL;
    PAL_Result        result = PSP_SOK;

    if(NULL != hMcasp)
    {
        tempAddr = hMcasp->regs;

        if (NULL != tempAddr)
        {

           /* Sequence of start: starting hclk first*/
           /* start AHCLKR */
            CSL_FINST (tempAddr->GBLCTL, MCASP_GBLCTL_RHCLKRST, ACTIVE);
            {
                bitValue = 0;
                while (CSL_MCASP_GBLCTL_RHCLKRST_ACTIVE != bitValue)
                {
                    bitValue = CSL_FEXT (tempAddr->GBLCTL,
                               MCASP_GBLCTL_RHCLKRST);
                }
            }

             /* start ACLKR only if internal clock is used*/
             if (CSL_MCASP_ACLKRCTL_CLKRM_INTERNAL ==
                   (CSL_FEXT (tempAddr->ACLKRCTL, MCASP_ACLKRCTL_CLKRM)))
              {
                CSL_FINST (tempAddr->GBLCTL, MCASP_GBLCTL_RCLKRST, ACTIVE);
                    {
                        bitValue = 0;
                        while (CSL_MCASP_GBLCTL_RCLKRST_ACTIVE != bitValue)
                        {
                            bitValue = CSL_FEXT (tempAddr->GBLCTL, MCASP_GBLCTL_RCLKRST);
                        }
                    }
               }

            CSL_FINST (tempAddr->RGBLCTL, MCASP_RGBLCTL_RSRCLR, ACTIVE);
            {
                bitValue = 0;
                while (CSL_MCASP_GBLCTL_RSRCLR_ACTIVE != bitValue)
                {
                    bitValue = CSL_FEXT (tempAddr->GBLCTL, MCASP_GBLCTL_RSRCLR);
                }
            }
        }
        else
        {
            result = PSP_E_INVAL_PARAM;
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    return (result);

}

/**
 * \brief   LLC_mcaspSetDitMode
 *
 *  This function enables/disables the DIT mode.
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \param     ditFlag     [IN]     Value to loaded into the DITCTL register
 * \return
 *			None
 */

PAL_Result LLC_mcaspSetDitMode (LLC_McaspHandle     hMcasp,
                          Bool ditFlag)
{
    PAL_Result    result = PSP_SOK;

    if(NULL != hMcasp)
    {
        if (TRUE == hMcasp->ditStatus)
        {
            /* Reset XSMRST and XFRST in GBLCTL */
            CSL_FINST (hMcasp->regs->GBLCTL, MCASP_GBLCTL_XSMRST, RESET);
            CSL_FINST (hMcasp->regs->GBLCTL, MCASP_GBLCTL_XFRST, RESET);

            /* Configure DITEN bit of DITCTL */
            CSL_FINS (hMcasp->regs->DITCTL, MCASP_DITCTL_DITEN, ditFlag);
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    return (result);
}

/**
 * \brief   LLC_mcaspGetAmute
 *
 *  This function returns the value of AMUTE register.
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *			None
 */

static Uint16 LLC_mcaspGetAmute (LLC_McaspHandle hMcasp)
{
    Uint16 val = 0;

    if (NULL != hMcasp)
    {
        val = (Uint16) hMcasp->regs->AMUTE;
    }

    return val;
}

/**
 * \brief   LLC_mcaspActivateClkRcvXmt
 *
 *  This function activates receive and transmit clock.
 *
 * \param     hMcasp      [IN]     Handle to the McASP instance
 * \return
 *			success or failure
 */

PAL_Result LLC_mcaspActivateClkRcvXmt(LLC_McaspHandle hMcasp)
{

    Uint32 bitValue = 0;
    CSL_McaspRegsOvly tempAddr = NULL;
    PAL_Result        result = PSP_SOK;

    if(NULL != hMcasp)
    {
        tempAddr = hMcasp->regs;

        if (NULL != tempAddr)
        {
           /* Sequence of start: starting hclk first*/
           /* start AHCLKR */
            CSL_FINST (tempAddr->GBLCTL, MCASP_GBLCTL_RHCLKRST, ACTIVE);
            {
                bitValue = 0;
                while (CSL_MCASP_GBLCTL_RHCLKRST_ACTIVE != bitValue)
                {
                    bitValue = CSL_FEXT (tempAddr->GBLCTL,
                               MCASP_GBLCTL_RHCLKRST);
                }
            }

             /* start ACLKR only if internal clock is used*/
             if (CSL_MCASP_ACLKRCTL_CLKRM_INTERNAL ==
                   (CSL_FEXT (tempAddr->ACLKRCTL, MCASP_ACLKRCTL_CLKRM)))
              {
                CSL_FINST (tempAddr->GBLCTL, MCASP_GBLCTL_RCLKRST, ACTIVE);
                    {
                        bitValue = 0;
                        while (CSL_MCASP_GBLCTL_RCLKRST_ACTIVE != bitValue)
                        {
                            bitValue = CSL_FEXT (tempAddr->GBLCTL, MCASP_GBLCTL_RCLKRST);
                        }
                    }
               }


          /* Sequence of start: starting hclk first*/
            CSL_FINST (tempAddr->XGBLCTL, MCASP_XGBLCTL_XHCLKRST, ACTIVE);
            {
                bitValue = 0;
                while (CSL_MCASP_GBLCTL_XHCLKRST_ACTIVE != bitValue)
                {
                    bitValue = CSL_FEXT (tempAddr->GBLCTL,
                               MCASP_GBLCTL_XHCLKRST);
                }
            }

             /* start ACLKX only if internal clock is used*/
            if (CSL_MCASP_ACLKXCTL_CLKXM_INTERNAL ==
               (CSL_FEXT (tempAddr->ACLKXCTL, MCASP_ACLKXCTL_CLKXM)))
               {
                    CSL_FINST (tempAddr->XGBLCTL, MCASP_XGBLCTL_XCLKRST, ACTIVE);
                    {
                        bitValue = 0;
                        while (CSL_MCASP_GBLCTL_XCLKRST_ACTIVE != bitValue)
                        {
                            bitValue = CSL_FEXT (tempAddr->GBLCTL,
                                       MCASP_GBLCTL_XCLKRST);
                        }
                    }
                }
        }
        else
        {
            result = PSP_E_INVAL_PARAM;
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }
    return (result);
}


/**
 * \brief   LLC_mcaspGetChipCtxt
 *
 *  This function is used for getting the base address of the peripheral
 *  instance and to configure the number of serializers for a particular
 *  instance of McASP on a chip. This function will be called inside the
 *  LLC_mcaspOpen() function.This function is open for re-implementing
 *  if the user wants to modify the base address of the peripheral object
 *  to point to a different location and there by allow CSL initiated
 *  write/reads into peripheral MMR's go to an alternate location.
 *
 * \param   mcaspNum        [IN]    McASP instance number
 * \param   *pMcaspParam    [IN]    Module specific parameter for McASP instance
 * \param   *pChipContext   [IN]    base-address information for the peripheral
 									instance
 * \return
 *			success or failure
 */

static Int16 LLC_mcaspGetChipCtxt (
                    Int16                    mcaspNum,
                    LLC_McaspParam          *pMcaspParam,
                    LLC_McaspChipContext    *const pChipContext
                    )
{
    Int16 st = PSP_SOK;

    /*To remove warning, kept for future usage*/
    if(NULL !=  pMcaspParam)
    {
        *pMcaspParam = *pMcaspParam;
    }
    mcaspNum = mcaspNum;

    if(NULL == pChipContext)
    {
        st = PSP_E_INVAL_PARAM;
    }
    else
    {

        switch (mcaspNum)
        {
            /*
            *
            * Get Mcasp Base Address and peripharal inforamtion
            *
            */
            case LLC_MCASP1:
                {
                    pChipContext->regs = (CSL_McaspRegsOvly)MCASP0_CNTRL_ADDR;
                    pChipContext->numOfSerializers = PSP_MCASP0_NUM_SERS;
                    pChipContext->ditStatus = TRUE;
                    pChipContext->dataAddr  = (Uint32*)MCASP0_DATA_ADDR;
                    pChipContext->edmaTxEventNum = MCASP0TXEVT;
                    pChipContext->edmaRxEventNum = MCASP0RXEVT;
                    pChipContext->cpuTxEventNum = CSL_INTC_EVENTID_AXINT0;
                    pChipContext->cpuRxEventNum = CSL_INTC_EVENTID_ARINT0;
                    break;
                }
           default:
                {
                    pChipContext->regs = (CSL_McaspRegsOvly)NULL;
                    pChipContext->numOfSerializers = 0;
                    pChipContext->ditStatus = FALSE;
                    pChipContext->dataAddr  = NULL;
                    st = PSP_E_INVAL_PARAM;
                    break;
                }
        }


    }

    return st;
}

/**
 * \brief   LLC_mcaspWrite
 *
 *   This transmits the data from McASP. 32 bits of data will be transmitted
 *   in the data object (variable); the pointer is passed as the 
 *   a first argument.
 *
 * \param   hMcasp   [IN]    Handle to the McASP instance
 * \param   serNum   [IN]    Serailizer number
 * \param   wordLen  [IN]    word length
 * \param   *data    [IN]    data to be written
 *
 *  \return
 *           PSP_SOK             - Successful completion of read
 *           CSL_ESYS_INVPARAMS  - Invalid parameters
 *           CSL_ESYS_BADHANDLE  - Invalid handle
 */

Int16  LLC_mcaspWrite (
                    LLC_McaspHandle         hMcasp,
                    Uint32                  serNum,
                    Uint16                  wordLen,
                    Uint8                  *const data
                    )

{

    Int16           status  = PSP_SOK;
    volatile Uint32 *baseAddr;

    /*Added to remove warning, kept for future use with CHIP_DM642*/
    serNum = serNum;

    if ( (NULL == hMcasp) || (NULL == data) )
    {
        status = PSP_E_INVAL_PARAM;
    }
    else
    {
        /* Access Xbuf using XBUF register address */
        baseAddr = (volatile  Uint32 *)((&(hMcasp->regs->XBUF0)) + (serNum)) ;
        if(NULL == baseAddr)
        {
            status = PSP_E_INVAL_PARAM;
        }
        else
        {
            switch(wordLen)
            {
                case PSP_MCASP_WORDLEN_8:
                    {
                        *baseAddr = ((*data) & (0xffu));
                        break;
                    }

                case PSP_MCASP_WORDLEN_16:
                    {
                        *baseAddr = ((*((Uint16*)data)) & (0xffffu));
                         break;
                    }


                case PSP_MCASP_WORDLEN_32:
                    {
                        *baseAddr = (*((Uint32*)data));
                        break;
                    }
                default:
                    {
                        status = PSP_E_INVAL_PARAM;
                        break;
                    }
                }
            }
        }

    return(status);
}

/**
 * \brief   LLC_mcaspRead
 *
 *   This reads the data from McASP. 32 bits of data will be read in the
 *   data object (variable); the pointer passed as a first argument.
 *
 * \param   hMcasp   [IN]    Handle to the McASP instance
 * \param   serNum   [IN]    Serailizer number
 * \param   wordLen  [IN]    word length
 * \param   *data    [IN]    data to be written
 *
 *  \return
 *           PSP_SOK             - Successful completion of read
 *           CSL_ESYS_INVPARAMS  - Invalid parameters
 *           CSL_ESYS_BADHANDLE  - Invalid handle
 */

Int16  LLC_mcaspRead (
                LLC_McaspHandle         hMcasp,
                Uint32                  serNum,
                Uint16                  wordLen,
                Uint8                  *data
                )

{
    Int16           status = PSP_SOK;
    volatile Uint32 *baseAddr;

    /*Added to remove warning, kept for future use with CHIP_DM642*/
    serNum = serNum;

    if ( (NULL == hMcasp) || (NULL == data) )
    {
        status = PSP_E_INVAL_PARAM;
    }
    else
    {
        /* Access Rbuf using RBUF register address */
        baseAddr = (volatile  Uint32 *) (&(hMcasp->regs->RBUF0) + (serNum)) ;
        if(NULL == baseAddr)
        {
            status = PSP_E_INVAL_PARAM;
        }
        else
        {
            switch(wordLen)
            {
                case PSP_MCASP_WORDLEN_8:
                    {
                        *data = ((*baseAddr) & 0xffu);
                        break;
                    }

                case PSP_MCASP_WORDLEN_16:
                    {
                        *((Uint16*)data) = ((*baseAddr) & 0xffffu);
                        break;
                    }

                case PSP_MCASP_WORDLEN_32:
                    {
                        *((Uint32*)data) = *baseAddr;
                        break;
                    }
               default:
                    {
                        status = PSP_E_INVAL_PARAM;
                        break;
                    }

            }
        }
    }

    return(status);
}

/**
 * \brief   LLC_mcaspRegReset
 *
 *   This Function fills the registers for the McAsp instance with
 *   default reset values
 *
 * \param   hMcasp   [IN]    Pointer to the McASP instance
 * \return
 *          none
 */

PAL_Result  LLC_mcaspRegReset (LLC_McaspHandle     hMcasp)
{
    Int16               serNum     = 0;
    CSL_McaspRegsOvly   tempAddr= NULL;
    PAL_Result            result    = PSP_SOK;

    if(NULL != hMcasp )
    {
        tempAddr = hMcasp->regs;
    }
    if(NULL != tempAddr)
    {
        tempAddr->GBLCTL        = CSL_MCASP_GBLCTL_RESETVAL;
        tempAddr->RMASK         = CSL_MCASP_RMASK_RESETVAL;
        tempAddr->RFMT          = CSL_MCASP_RFMT_RESETVAL;
        tempAddr->AFSRCTL       = CSL_MCASP_AFSRCTL_RESETVAL;
        tempAddr->ACLKRCTL      = CSL_MCASP_ACLKRCTL_RESETVAL;
        tempAddr->AHCLKRCTL     = CSL_MCASP_AHCLKRCTL_RESETVAL;

        tempAddr->RTDM          = CSL_MCASP_RTDM_RESETVAL;
        tempAddr->RINTCTL       = CSL_MCASP_RINTCTL_RESETVAL;
        tempAddr->RCLKCHK       = CSL_MCASP_RCLKCHK_RESETVAL;

        tempAddr->XMASK         = CSL_MCASP_XMASK_RESETVAL;
        tempAddr->XFMT          = CSL_MCASP_XFMT_RESETVAL;
        tempAddr->AFSXCTL       = CSL_MCASP_AFSXCTL_RESETVAL;
        tempAddr->ACLKXCTL      = CSL_MCASP_ACLKXCTL_RESETVAL;
        tempAddr->AHCLKXCTL     = CSL_MCASP_AHCLKXCTL_RESETVAL;
        tempAddr->XTDM          = CSL_MCASP_XTDM_RESETVAL;

        tempAddr->XINTCTL       = CSL_MCASP_XINTCTL_RESETVAL;
        tempAddr->XCLKCHK       = CSL_MCASP_XCLKCHK_RESETVAL;

        while (serNum < hMcasp->numOfSerializers)
        {

            switch(serNum)
            {
#if defined(CHIP_DM648) || defined (CHIP_C6452)
                    case 9:
                        {
                            tempAddr->SRCTL9    = CSL_MCASP_SRCTL9_RESETVAL;
                            break;
                        }
                    case 8:
                        {
                            tempAddr->SRCTL8    = CSL_MCASP_SRCTL8_RESETVAL;
                            break;
                        }
                    case 7:
                        {
                            tempAddr->SRCTL7    = CSL_MCASP_SRCTL7_RESETVAL;
                            break;
                        }
                    case 6:
                        {
                            tempAddr->SRCTL6    = CSL_MCASP_SRCTL6_RESETVAL;
                            break;
                        }
                    case 5:
                        {
                            tempAddr->SRCTL5    = CSL_MCASP_SRCTL5_RESETVAL;
                            break;
                        }
                    case 4:
                        {
                            tempAddr->SRCTL4    = CSL_MCASP_SRCTL4_RESETVAL;
                            break;
                        }
#endif
                    case 3:
                        {
                            tempAddr->SRCTL3    = CSL_MCASP_SRCTL3_RESETVAL;
                            break;
                        }
                    case 2:
                        {
                            tempAddr->SRCTL2    = CSL_MCASP_SRCTL2_RESETVAL;
                            break;
                        }
                    case 1:
                        {
                            tempAddr->SRCTL1    = CSL_MCASP_SRCTL1_RESETVAL;
                            break;
                        }
                    case 0:
                        {
                            tempAddr->SRCTL0        = CSL_MCASP_SRCTL0_RESETVAL;
                            break;
                        }
                    default:
                        {
                            break;
                        }

                }
                serNum++;
            }

        tempAddr->PFUNC     = CSL_MCASP_PFUNC_RESETVAL;
        tempAddr->PDIR      = CSL_MCASP_PDIR_RESETVAL;
        tempAddr->DITCTL    = CSL_MCASP_DITCTL_RESETVAL;
        tempAddr->DLBCTL    = CSL_MCASP_DLBCTL_RESETVAL;
        tempAddr->AMUTE     = CSL_MCASP_AMUTE_RESETVAL;
        tempAddr->RSTAT     = CSL_MCASP_RSTAT_RESETVAL;
        tempAddr->REVTCTL   = CSL_MCASP_REVTCTL_RESETVAL;
        tempAddr->XSTAT     = CSL_MCASP_XSTAT_RESETVAL;
        tempAddr->XEVTCTL   = CSL_MCASP_XEVTCTL_RESETVAL;
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    return (result);

}

/**
 *  \brief LLC_mcaspOpen
 *
 *   This Function populates the peripheral data object for the instance and
 *   returns handle to it. Updates hMcasp parameter to handler
 *
 * \param   *pMcaspObj      [IN]    Pointer to the data object for McASP instance
 * \param    mcaspNum       [IN]    Specifies the instance of the McASP to be opened.
 *                                  There are 4 instances of the McASP.
 * \param   *pMcaspParam    [IN]    Module specific parameter for McASP instance
 * \param   *pStatus        [OUT]   Status of the function call
 *
 * \return
 *      PSP_SOK                 Open call is successful
 *      CSL_ESYS_FAIL           The McASP instance is invalid
 */

LLC_McaspHandle LLC_mcaspOpen (
                            LLC_McaspObj      *const pMcaspObj,
                            Int16                 mcaspNum,
                            LLC_McaspParam    *pMcaspParam,
                            Int16             *pStatus
                            )
{
    Int16                   st;
    LLC_McaspHandle         hMcasp;
    LLC_McaspChipContext    chipContext;

    st = LLC_mcaspGetChipCtxt(mcaspNum, pMcaspParam, (LLC_McaspChipContext *)&chipContext);

    if (PSP_SOK == st)
    {
        if( NULL != pMcaspObj )
        {
            pMcaspObj->regs = chipContext.regs;
            pMcaspObj->perNo = (Int16)mcaspNum;
            pMcaspObj->numOfSerializers = chipContext.numOfSerializers;
            pMcaspObj->ditStatus = chipContext.ditStatus;
            pMcaspObj->dataAddr = chipContext.dataAddr;
            pMcaspObj->edmaTxEventNum = chipContext.edmaTxEventNum;
            pMcaspObj->edmaRxEventNum = chipContext.edmaRxEventNum;
            pMcaspObj->cpuTxEventNum =  chipContext.cpuTxEventNum;
            pMcaspObj->cpuRxEventNum =  chipContext.cpuRxEventNum;

            hMcasp = (LLC_McaspHandle)pMcaspObj;
        }
    }
    else
    {
        if( NULL != pMcaspObj )
        {
            pMcaspObj->regs = (CSL_McaspRegsOvly)NULL;
            pMcaspObj->perNo = (Int16)-1;
            pMcaspObj->numOfSerializers = 0;
            pMcaspObj->ditStatus = FALSE;
            pMcaspObj->dataAddr = NULL;
            hMcasp = (LLC_McaspHandle)NULL;
        }
    }
    if(NULL != pStatus)
    {
        *pStatus = st;
    }
    return hMcasp;
}

/**
 * \brief   LLC_mcaspInit
 *
 *   This is the initialization function for the multi channel audio serial
 *   port CSL. This function needs to be called before any other multi
 *   channel audio serial port CSL functions are to be called. This function
 *   is idem-potent. Currently, this function does not perform anything.
 *
 * \param   pContext   [IN]    Context information for the instance. 
 *							   Should be NULL
 * \return
 *          PSP_SOK - Always returns
 */

Int16  LLC_mcaspInit(LLC_McaspContext    *pContext)
{

    /*To remove warning, kept for future usage*/
    if(NULL !=  pContext)
    {
        *pContext = *pContext;
    }
    return PSP_SOK;
}

/**
 * \brief   LLC_mcaspHwSetup
 *
 *   It configures the  McASP instance registers as per the values passed
 *   in the hardware setup structure.
 *
 * \param   hMcasp     [IN]    Handle to the McASP instance
 * \param   *myHwSetup [IN]    Pointer to hardware setup structure
 * \return
 *          PSP_SOK             - Hardware setup successful
 *          CSL_ESYS_BADHANDLE  - Invalid handle
 *          CSL_ESYS_INVPARAMS  - Hardware structure is not properly initialized
 */

Int16  LLC_mcaspHwSetup(
                        LLC_McaspHandle      hMcasp,
                        PSP_McaspHwSetup     *const myHwSetup
                        )
{
    Int16               serNum = 0;
    Int16               status = PSP_SOK;
    CSL_McaspRegsOvly   tempAddr;

    if ( (NULL == hMcasp) || (NULL == myHwSetup) )
    {
        status = PSP_E_INVAL_PARAM;
    }
    else
    {
        tempAddr = hMcasp->regs;
        if(NULL == tempAddr)
        {
            status = PSP_E_INVAL_PARAM;
        }
        else
        {

            /* Reset McASP to default values by setting GBLCTL = 0 */
            tempAddr->GBLCTL = 0x0000;

            /* Configure the RMASK register */
            tempAddr->RMASK =  (Uint32)myHwSetup->rx.mask;

            /* Configure RFMT */
            tempAddr->RFMT  =  myHwSetup->rx.fmt;

            /* Configure AFSRCTL */
            tempAddr->AFSRCTL =  myHwSetup->rx.frSyncCtl;

            /* Configure ACLKRCTL */
            tempAddr->ACLKRCTL = myHwSetup->rx.clk.clkSetupClk;

            /* Configure AHCLKRCTL */
            tempAddr->AHCLKRCTL =  myHwSetup->rx.clk.clkSetupHiClk;

            /* Configure RTDM */
            tempAddr->RTDM =  myHwSetup->rx.tdm;

            /* Configure RINTCTL */
            tempAddr->RINTCTL = myHwSetup->rx.intCtl;

            /* Configure RCLKCHK */
            tempAddr->RCLKCHK =  myHwSetup->rx.clk.clkChk;

            /* Configure XMASK */
            tempAddr->XMASK =  myHwSetup->tx.mask;

            /* Configure XFMT */
            tempAddr->XFMT  =  myHwSetup->tx.fmt;

            /* Configure AFSXCTL */
            tempAddr->AFSXCTL = myHwSetup->tx.frSyncCtl;

            /* Configure ACLKXCTL */
            tempAddr->ACLKXCTL = myHwSetup->tx.clk.clkSetupClk;

            /* Configure AHCLKXCTL */
            tempAddr->AHCLKXCTL = myHwSetup->tx.clk.clkSetupHiClk;

            /* Configure XTDM */
            tempAddr->XTDM  = myHwSetup->tx.tdm;

            /* Configure XINTCTL */
            tempAddr->XINTCTL = myHwSetup->tx.intCtl;

            /* Configure XCLKCHK */
            tempAddr->XCLKCHK   = myHwSetup->tx.clk.clkChk;


            while (serNum < hMcasp->numOfSerializers)
            {

                switch(serNum)
                {
#if defined(CHIP_DM648) || defined (CHIP_C6452)
                    case 9:
                        {
                            tempAddr->SRCTL9 =   (Uint32) myHwSetup->glb.serSetup[serNum];
                            break;
                        }
                    case 8:
                        {
                            tempAddr->SRCTL8 =   (Uint32) myHwSetup->glb.serSetup[serNum];
                            break;
                        }
                    case 7:
                        {
                            tempAddr->SRCTL7 =   (Uint32) myHwSetup->glb.serSetup[serNum];
                            break;
                        }
                    case 6:
                        {
                            tempAddr->SRCTL6 =   (Uint32) myHwSetup->glb.serSetup[serNum];
                            break;
                        }
                    case 5:
                        {
                            tempAddr->SRCTL5 =   (Uint32) myHwSetup->glb.serSetup[serNum];
                            break;
                        }
                    case 4:
                        {
                            tempAddr->SRCTL4 =   (Uint32) myHwSetup->glb.serSetup[serNum];
                            break;
                        }
#endif
                    case 3:
                        {
                            tempAddr->SRCTL3 =  (Uint32) myHwSetup->glb.serSetup[serNum];
                            break;
                        }
                    case 2:
                        {
                            tempAddr->SRCTL2 =  (Uint32) myHwSetup->glb.serSetup[serNum];
                            break;
                        }
                    case 1:
                        {
                            tempAddr->SRCTL1 =  (Uint32) myHwSetup->glb.serSetup[serNum];
                            break;
                        }
                    case 0:
                        {
                            tempAddr->SRCTL0 =  (Uint32) myHwSetup->glb.serSetup[serNum];
                            break;
                        }
                    default:
                        {
                            break;
                        }

                }
                serNum++;
            }

            /* Configure pin function register */
            tempAddr->PFUNC = myHwSetup->glb.pfunc;

            /* Configure pin direction register */
            tempAddr->PDIR =  myHwSetup->glb.pdir;

            /* Configure DITCTL */
            if (TRUE == hMcasp->ditStatus)
            {
                tempAddr->DITCTL =  myHwSetup->glb.ditCtl;
            }

            /* Configure DLBCTL */
            tempAddr->DLBCTL =  myHwSetup->glb.dlbMode;

            /* Configure AMUTE */
            tempAddr->AMUTE  =  myHwSetup->glb.amute;

            /* Configure RSTAT and XSTAT */
            tempAddr->RSTAT =  myHwSetup->rx.stat;

            tempAddr->XSTAT =  myHwSetup->tx.stat;

            /* Configure REVTCTL and XEVTCTL */
            tempAddr->REVTCTL =  myHwSetup->rx.evtCtl;
            tempAddr->XEVTCTL =  myHwSetup->tx.evtCtl;

            /* Initialize the global control register */
            tempAddr->GBLCTL =  myHwSetup->glb.ctl;

        }
    }

    return status;
}


/**
 * \brief   LLC_mcaspGetHwStatus
 *
 *   This function is used to get the value of various parameters of the
 *   McASP instance. The value returned depends on the query passed.
 *
 * \param   hMcasp     [IN]    Handle to the McASP instance
 * \param   myQuery    [IN]    Query to be performed
 * \param   *response  [IN]    Pointer to buffer to return the data requested by
 *                             the query passed
 * \return
 *      PSP_SOK            - Successful completion of the query
 *      CSL_ESYS_BADHANDLE - Invalid handle
 *      CSL_ESYS_INVQUERY  - Query command not supported
 *      CSL_ESYS_FAIL      - Generic failure
 */

Int16  LLC_mcaspGetHwStatus(
    LLC_McaspHandle          hMcasp,
    LLC_McaspHwStatusQuery   myQuery,
    void                     *const response)
{
   Int16 status = PSP_SOK;

    if((NULL == hMcasp) || (NULL == response))
    {
        status = PSP_E_INVAL_PARAM;
    }
    else
    {
        switch (myQuery)
        {
            /* Return current transmit slot being transmitted */
            case LLC_MCASP_QUERY_CURRENT_XSLOT:
                {
                    *((Uint16 *)response) = LLC_mcaspGetCurrentXSlot (hMcasp);
                    break;
                }

            /* Return current receive slot being received */
            case LLC_MCASP_QUERY_CURRENT_RSLOT:
                {
                    *((Uint16 *)response) = LLC_mcaspGetCurrentRSlot (hMcasp);
                    break;
                }
            /* Return transmit error status bit */
            case LLC_MCASP_QUERY_XSTAT_XERR:
                {
                    *(Bool *)response = LLC_mcaspGetXmtErr (hMcasp);
                    break;
                }

            /* Return transmit clock failure flag status */
            case LLC_MCASP_QUERY_XSTAT_XCLKFAIL:
                {
                    *(Bool *)response = LLC_mcaspGetXmtClkFail (hMcasp);
                    break;
                }

            /* Return unexpected transmit frame sync flag status */
            case LLC_MCASP_QUERY_XSTAT_XSYNCERR:
                {
                    *(Bool *) response = LLC_mcaspGetXmtSyncErr (hMcasp);
                    break;
                }

            /* Return transmit underrun flag status */
            case LLC_MCASP_QUERY_XSTAT_XUNDRN:
                {
                    *((Bool *)response) = LLC_mcaspGetXmtUnderrun (hMcasp);
                    break;
                }

            /* Return transmit data ready flag status */
            case LLC_MCASP_QUERY_XSTAT_XDATA:
                {
                    (*(Bool *)response) = LLC_mcaspGetXmtDataReady (hMcasp);
                    break;
                }

            /* Return receive error status bit */
            case LLC_MCASP_QUERY_RSTAT_RERR:
                {
                    *((Bool *)response) = LLC_mcaspGetRcvErr (hMcasp);
                    break;
                }

            /* Return receive clk failure flag status */
            case LLC_MCASP_QUERY_RSTAT_RCLKFAIL:
                {
                    *((Bool *)response) = LLC_mcaspGetRcvClkFail (hMcasp);
                    break;
                }

            /* Return unexpected receive frame sync flag status */
            case LLC_MCASP_QUERY_RSTAT_RSYNCERR:
                {
                    *((Bool *)response) = LLC_mcaspGetRcvSyncErr (hMcasp);
                    break;
                }

            /* Return receive overrun flag status */
            case LLC_MCASP_QUERY_RSTAT_ROVRN:
                {
                    *((Bool *)response) = LLC_mcaspGetRcvOverrun (hMcasp);
                    break;
                }

            /* Return receive data ready flag status */
            case LLC_MCASP_QUERY_RSTAT_RDATA:
                {
                    *((Bool *)response) = LLC_mcaspGetRcvDataReady (hMcasp);
                    break;
                }

            /* Return status whether rrdy is set or not */
            case LLC_MCASP_QUERY_SRCTL_RRDY:
                {
                    LLC_McaspSerQuery *serQuery = (LLC_McaspSerQuery *)response;
                    status = LLC_mcaspGetSerRcvReady (hMcasp,
                                                        (Bool *)&(serQuery->serStatus),
                                                        serQuery->serNum);
                    break;
                }

            /* Return status whether xrdy is set or not */
            case LLC_MCASP_QUERY_SRCTL_XRDY:
                {
                    LLC_McaspSerQuery *serQuery = (LLC_McaspSerQuery *)response;
                    status = LLC_mcaspGetSerXmtReady (hMcasp,
                                      (Bool  *)&(serQuery->serStatus),
                                      serQuery->serNum);
                    break;
                }

            /*
            * Return status whether serializer is configured as
            * transmit/receive/inactive
            */
            case LLC_MCASP_QUERY_SRCTL_SRMOD:
                {
                    LLC_McaspSerModeQuery *serQuery = (LLC_McaspSerModeQuery *)response;
                    status = LLC_mcaspGetSerMode (hMcasp, &(serQuery->serMode),
                                      serQuery->serNum);
                    break;
                }

            /* Return the value of XSTAT register */
            case LLC_MCASP_QUERY_XSTAT:
                {
                    *((Uint16 *)response) = LLC_mcaspGetXmtStat (hMcasp);
                    break;
                }

            /* Return the value of RSTAT register */
            case LLC_MCASP_QUERY_RSTAT:
                {
                    *((Uint16 *)response) = LLC_mcaspGetRcvStat (hMcasp);
                    break;
                }

            /* Return the XSMRST and XFRST field values GBLCTL register */
            case LLC_MCASP_QUERY_SM_FS_XMT:
                {
                    *((Uint8 *)response) = LLC_mcaspGetSmFsXmt (hMcasp);
                    break;
                }

            /* Return the RSMRST and RFRST field values GBLCTL register */
            case LLC_MCASP_QUERY_SM_FS_RCV:
                {
                    *((Uint8 *)response) = LLC_mcaspGetSmFsRcv (hMcasp);
                    break;
                }

            /* Return status of DITEN bit in DITCTL register */
            case LLC_MCASP_QUERY_DIT_MODE:
                {
                    *((Bool *)response) = LLC_mcaspGetDitMode (hMcasp);
                    break;
                }

            case LLC_MCASP_QUERY_AMUTE:
                {
                    *((Uint16 *)response) = LLC_mcaspGetAmute (hMcasp);
                    break;
                }

            default:
                {
                    status = PSP_E_INVAL_PARAM;
                }
        }
    }

    return status;
}

/**
 * \brief   LLC_mcaspClose
 *
 *   This function closes the McASP instance
 *
 * \param   hMcasp     [IN]    Handle to the McASP instance
 * \return
 *      PSP_SOK            - Close successful
 *      CSL_ESYS_BADHANDLE - Invalid handle
 */

Int16  LLC_mcaspClose (LLC_McaspHandle    hMcasp)
{
    Int16  st;

    if (NULL != hMcasp)
    {
        hMcasp->regs    = (CSL_McaspRegsOvly)NULL;
        /*making the instance invalid*/
        hMcasp->perNo   = (Int16)-1;
        hMcasp->numOfSerializers = 0;
        hMcasp->ditStatus = FALSE;
        st = PSP_SOK;
    }
    else
    {
        st = PSP_E_INVAL_PARAM;
    }
    return st;
}

/**
 *   \brief LLC_mcaspConfigGpio
 *
 *   This function enables/disables the GPIO mode (configure GPIO mode).
 *
 *  \param        hMcasp  [IN]  Handle to the McASP instance
 *  \param        setVal  [IN]  Value to loaded into the GPIO register
 *  \return      
 *         	PSP_SOK				- Successuful return
 *			PSP_E_INVAL_PARAM  	- Invalide Parameter
 */

PAL_Result LLC_mcaspConfigGpio(LLC_McaspHandle hMcasp, Uint32 setVal)
{
    PAL_Result     result = PSP_SOK;

    if(NULL!=hMcasp)
    {
        /* Configure PDIR Pin direction with the value passed */

        hMcasp->regs->PFUNC = 0xFFFFFFFF;
        hMcasp->regs->PDIR  = setVal;
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    return (result);
}

/**
 *  \brief LLC_mcaspReadXmtConfig
 *
 *  Mcasp Configuration related functions, Updates xmtData pointer
 *
 *  \param    hMcasp        [IN]    port  Handler
 *  \param    xmtData       [IN]    configured data is passed to user
 *  \return  void
 */

PAL_Result LLC_mcaspReadXmtConfig (
                        LLC_McaspHandle         hMcasp,
                        PSP_McaspHwSetupData    *const xmtData
                        )
{
    PAL_Result    result = PSP_SOK;

    if((NULL!=hMcasp)&&(NULL!=xmtData))
    {
        /* Read XMASK register */
        xmtData->mask = (Uint32)(hMcasp->regs->XMASK);

        /* Read XFMT register */
        xmtData->fmt = (Uint32)(hMcasp->regs->XFMT);

        /* Read AFSXCTL register */
        xmtData->frSyncCtl = (Uint32)(hMcasp->regs->AFSXCTL);

        xmtData->clk.clkSetupClk = (Uint32)(hMcasp->regs->ACLKXCTL);

        /* Read AHCLKXCTL register */
        xmtData->clk.clkSetupHiClk = (Uint32)(hMcasp->regs->AHCLKXCTL);

        /* Read XTDM register */
        xmtData->tdm = (Uint32)(hMcasp->regs->XTDM);

        /* Read XINTCTL register */
        xmtData->intCtl = (Uint32)(hMcasp->regs->XINTCTL);

        /* Read XCLKCHK register */
        xmtData->clk.clkChk = (Uint32)(hMcasp->regs->XCLKCHK);

        /* Read XSTAT register */
        xmtData->stat = (Uint32)(hMcasp->regs->XSTAT);

        /* Read XEVTCTL register */
        xmtData->evtCtl = (Uint32)(hMcasp->regs->XEVTCTL);
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    return (result);
}

/**
 *  \brief    LLC_mcaspReadRcvConfig
 *
 *  configures receiver section of McAsp
 *  clock, high clock, mask bit,interrupts mode are configured
 *
 *  \param    hMcasp         [IN]    port  Handler
 *  \param    rcvData        [IN]    Input data tobe configured by user
 *  \return void
 */

PAL_Result LLC_mcaspReadRcvConfig (
                        LLC_McaspHandle         hMcasp,
                        PSP_McaspHwSetupData    *const rcvData
                        )
{
    PAL_Result    result = PSP_SOK;

    if((NULL!=hMcasp)&&(NULL!=rcvData))
    {
        /* Read RMASK register */
        rcvData->mask = (Uint32)(hMcasp->regs->RMASK);

        /* Read RFMT register */
        rcvData->fmt = (Uint32)(hMcasp->regs->RFMT);

        /* Read AFSRCTL register */
        rcvData->frSyncCtl = (Uint32)(hMcasp->regs->AFSRCTL);

        rcvData->clk.clkSetupClk = (Uint32)(hMcasp->regs->ACLKRCTL);

        /* Read AHCLKRCTL register */
        rcvData->clk.clkSetupHiClk = (Uint32)(hMcasp->regs->AHCLKRCTL);

        /* Read RTDM register */
        rcvData->tdm = (Uint32)(hMcasp->regs->RTDM);

        /* Read RINTCTL register */
        rcvData->intCtl = (Uint32)(hMcasp->regs->RINTCTL);

        /* Read RCLKCHK register */
        rcvData->clk.clkChk = (Uint32)(hMcasp->regs->RCLKCHK);

        /* Read RSTAT register */
        rcvData->stat = (Uint32)(hMcasp->regs->RSTAT);

        /* Read REVTCTL register */
        rcvData->evtCtl = (Uint32)(hMcasp->regs->REVTCTL);
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }
    return (result);
}

