/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**    \file      psp_mcaspApi.h
 *
 *    \brief      Wrapper APIs called by DDA to access DDC
 *
 *  This file contains APIs defination of APIs that are wrapper APIs for DDC
 *  layer. These APIs will be called by DDA layer.
 *
 *    (C)    Copyright 2006,    Texas Instruments, Inc
 *
 *  \authors    PSP Team (please mail your comments/ queries to
 *              psparch-feedback@list.ti.com)
 *
 *  \note       Set tabstop to 4 (:se ts=4) while viewing this file in an
 *              editor
 *  \author Platform Support Group
 *
 *    \Revision
 *            0.1  Created
 *            0.2  Created  : Dynamic mode start/stop implemented
 *            0.3  modified : modified to PSP architecture and
 *                            implemented EDMA linking  of next pending packet
 *             0.4     modified : DM648 - post silicon changes and cleanup
 */

#ifdef __cplusplus
extern "C" {
#endif

/**
 * \fn  PSP_Result PSP_mcaspInit(Uint32 instanceId)
 *
 * \brief   Initialization of McAsp configuration
 *
 *  This function initialises the data structures
 *
 * \param     instanceId      [IN]     Device number (in this case, there are
 *                                     two McASP present 0 and 1)
 * \return
 *      if successful PSP_SOK , else PSP error code returned
 */
PSP_Result PSP_mcaspInit(Uint32 instanceId);

/**
 *  \brief  Opens a specified McASP driver (instance)
 *
 *  The instance of McASP specified shall be acquired through the
 *  LLC layer.  It internally calls the McASP IOM driver's
 *  mdBindDev function.
 *  Application has to pass pointer to the PSP_mcaspDevParams structure by
 *  specifying all the necessary fields.
 *
 */
PSP_Result PSP_mcaspCreate(Uint32 instanceId, PSP_mcaspDevParams * const params, Ptr arg);



/**
 *  \brief Close the McASP device instance from operation
 *
 *  Application has to call PSP_mcaspOpen API when it finishes
 *  all the I/O operations.
 *
 *  \param  hMcasp [IN]     handle to the McASP device instance
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_mcaspDelete(Uint32 instanceId,Ptr arg);


/**
 *  \brief Create a logical communication channel for data transfer
 *
 *  A communication channel shall be created by the calling
 *  PSP_mcaspOpenChannel by specifying  the I/O mode(INPUT or OUTPUT)
 *  After successfully creating the I/O channel PSP_mcaspOpenChannel shall
 *  return the handle to the I/O channel
 *
 */
PSP_Handle PSP_mcaspOpen(
                Uint32               instanceId,
                PSP_mcaspMode        mode,
                PSP_mcaspChanParams  * const params,
                PSP_mcaspAppCallback drvCbk,
                PSP_Handle           edmaHandle,
                Ptr                  arg
                );

/**
 *  \brief Close an I/O channel on McASP device
 *
 *   Close an I/O channel from operation. It releases all the resources
 *   held by the I/O channel
 *
 *  \param  hMcASPChan [IN]    Handle to the I/O channel to be closed
 *
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_mcaspClose(PSP_Handle hMcASPChan , Ptr arg);

/**
 *  \brief  Write a buffer of data to the output channel of the driver
 *
 *  The application has to call PSP_mcaspWrite() with handle to the
 *    o/p channel and the pointer to buffer to where the data shall be stored
 *    and the number of bytes to write in the packet.
 *
 *  \param  hMcASPChan [IN]     Handle to the McASP channel
 *  \param  pkt        [IN]     Packet passed from the application for processing
 *  \return PSP_SOK or PSP Error code
 */

PSP_Result PSP_mcaspWrite (PSP_Handle handle, PSP_mcaspReq *ioReq,Ptr arg);
/**
 *  \brief Read a buffer of data from the input channel of the driver
 *
 *  The application has to call PSP_mcaspReadChan() with handle to the
 *    i/p channel and the pointer to buffer to where the data shall be stored
 *    and the number of bytes to read. The has to specify the call back function
 *  the shall be called when the I/O is complete
 *
 */
PSP_Result PSP_mcaspRead (PSP_Handle handle, PSP_mcaspReq *ioReq,Ptr arg);


/**
 *  \brief Flush the data in the McASP channel
 *
 *  Flushes the pending I/O packets in the specified channel
 *  and returns the notifications of the pending I/O packets
 *  to the application
 *
 *  \param  hMcASPChan [IN]       Handle to the McASP channel
 *  \param  pkt        [IN]     Packet passed from the application for processing
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_mcaspFlushRequests(PSP_Handle handle,Ptr arg);


/**
 *  \brief Abort the McASP channel operation
 *
 *   Abort the input or output channel operation of the McASP driver
 *   and flush all pending I/O packets. Driver sends the
 *   notifications to application for all pending I/O packets
 *   The application can start freshly by submitting new I/O requests.
 *
 *  \param  hMcASPChan [IN]     Handle to the McASP channel
 *  \param  pkt        [IN]     Packet passed from the application for processing
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_mcaspAbortRequests(PSP_Handle handle,Ptr arg);


/**
 *  \brief Provides McASP device specific control operations
 *
 *   Application can directly execute specific device control operations
 *   by calling PSP_mcaspControlChan API with the appropriate control command
 *
 *  \param  hMcaspChan [IN]     Handle to the McASP channel
 *  \param  cmd        [IN]     Device specific control command exposed by the
                                driver through PSP_McaspIoctlCmd
 *  \param arg         [IN\OUT] Arguments to be used with the command
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_mcaspIoctl (PSP_Handle hMcASPChan, PSP_McaspIoctlCmd cmd, Ptr arg, Ptr param);
#ifdef __cplusplus
}
#endif
