/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 * \file    psp_mcbsp.h
 *
 * \brief   McBSP driver interface definition
 *
 *  This file contains the interfaces, data types and symbolic definitions
 *  that are needed by the application to utilize the services of the McBSP
 *  device driver.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \authors    PSP Team (please mail your comments/ queries to
 *              psparch-feedback@list.ti.com)
 *
 *  \note       Set tabstop to 4 (:se ts=4) while viewing this file in an
 *              editor
 *
 *  \version   0.1     First Release
 *             0.2 -   Pre silicon release
 *
 *  <b>Compiler Switch Used</b>
 *
 *   <b>_DEBUG</b><br>
 *    This is used as a flag to compiler whether to include the debug statements
 *    inserted in the code into the final image. This flag helps to build DEBUG
 *    image of the program. For RELEASE images this is not passed to the compiler.
 *
 *  <b>CHIP_XXXX</b><br>
 *   The CSL layer is written in a common file for all the variants of a SOC. This
 *   flag differentiates the variant we are compiling for, for e.g. CHIP_DM648,
 *   and the CSL definitions for that variant appropriately gets defined for
 *   register base addresses, num of ports of a peripheral etc.
 *
 *  <b>MCBSP_INSTRUMENTATION_ENABLED</b><br>
 *   This flag is passed to the compiler to include the instrumentation code parts
 *   into the final image/lib of the program. This helps build the iRelease/iDebug
 *   versions of the image/lib with a common code base.
 *
 *  <b>GBL_INT</b><br>
 *   This compiler option is used to Enable/Disable Global error Interrupt Handling
 *   for MCBSP module.
 */

#ifndef _PSP_McBSP_H_
#define _PSP_McBSP_H_

#include <ti/sdo/pspdrivers/common/psp_common.h>

#ifdef CHIP_C6424
#include <ti/sdo/pspdrivers/soc/c6424/dsp/tistdtypes.h>
#endif

#ifdef CHIP_DM6437
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/tistdtypes.h>
#endif


#ifdef __cplusplus
extern "C" {
#endif


#define FRAME_LENGTH                (127u)
/**< Frame length                                                       */
#define PSP_MCBSP_NUM_PORTS         (2u)
/**< Number of instances of McBSP                                       */
#define PSP_MCBSP_NUM_CHANS         (2u)
/**< Number of channels per instance of McBSP                           */

/**< Maximum number of EDMA jobs linked at a time (Must be 2).          */
#define MAXLINKCNT                   (2u)

#define PSP_MCBSP_MAX_REQ             (30)

/**< Used for identifying IO mode       */
#define PSP_MCBSP_RX                     (1u)
#define PSP_MCBSP_TX                     (2u)


#define PSP_SPI_TRANSIVE_BUFF_LEN  		 (256)

/**
 * \brief Error conditions needed by the McBSP driver
 *
 *  McBSP driver specific error conditions.
 *
 */

/*
 * I/O request flushed. Queued writes will be completed w/ IOM_COMPLETED.
 *   Queued read requests return w/ IOM_FLUSHED.
 */
#define PSP_MCBSP_FLUSHED                   (2u)
/** I/O aborted. Non-completed read or write requests return w/ IOM_ABORTED.*/
#define PSP_MCBSP_ABORTED                   (3u)
/** Error due to: DMA channel error */
#define PSP_MCBSP_DMA_ERROR                (-15)


/** Support Two McBSP instance
 This macro needs to be changed when porting to new SOC and if the new
 SOC contains different no. of  mcbsp instances */

/** Mcbsp Word length */
#define PSP_MCBSP_WORDLEN_8      (8u)
/** Mcbsp Word length */
#define PSP_MCBSP_WORDLEN_12     (12u)
/** Mcbsp Word length */
#define PSP_MCBSP_WORDLEN_16     (16u)
/** Mcbsp Word length */
#define PSP_MCBSP_WORDLEN_20     (20u)
/** Mcbsp Word length */
#define PSP_MCBSP_WORDLEN_24     (24u)
/** Mcbsp Word length */
#define PSP_MCBSP_WORDLEN_32     (32u)

/*
 * Global settings
 */

/** Generic invalidate status */
#define PSP_MCBSP_INVALID         (0xFFFF)

/** Generic validate status */
#define PSP_MCBSP_VALID           (0x1)

/* Forward Declaration */
struct PSP_mcbspReq_t;


/**
 *  \brief Global error callback function protype
 *
 *  This is the global error callback function for the McBSP driver.
 *  This function is called directly called from ISR context in
 *  case of error.
 */
typedef void (*PSP_mcbspGblCallback)( Int status );


/**
 *  \brief Application callback function protype
 *
 *  This is the application-level callback function for the McBSP driver .
 */
typedef void (*PSP_mcbspAppCallback)(   Ptr chan,
                                        Ptr arg,
                                        const struct PSP_mcbspReq_t *Pkt,
                                        Int32 mode);


typedef void (* pspMcbspIsr)(Ptr iohandlep);


/**
 *  \brief  PSP McBSP Mode
 *
 *  McBSP mode of creation
 */
typedef enum PSP_mcbspOperatingMode_t
{
    PSP_McBSP_McBSP,
    /** operate in normal McBSP mode */

    PSP_McBSP_McBSP_SPIMASTER,
    /** operate McBSP in SPI master mode */

    PSP_McBSP_McBSP_SPISLAVE
    /** operate McBSP in SPI slave mode */
}PSP_mcbspOperatingMode;

/**
  * \brief McBSP device raw setup
  *
  *  This structure holds raw configuration of McBSP registers.
  */
typedef struct PSP_mcbspHwCfgRaw_t
{
    /**< Serial Port Control Register */
    Uint32 SPCR;
    /**< Receive Control Register */
    Uint32 RCR;
    /**< Transmit Control Register */
    Uint32 XCR;
    /**< Sample Rate Generator Register */
    Uint32 SRGR;
    /**< Pin Control Register */
    Uint32 PCR;
} PSP_mcbspHwCfgRaw;


/**
  * \brief McBSP device setup params
  *
  *  This structure basically holds configuration to be used for the
  *  specified McBSP instance
  */
typedef struct PSP_Mcbsp_DevParams_t {

    Bool                    enablecache;
    /**< Submitted buffers are in cacheable memory                      */

    /**< Set to TRUE if internal clock is to be generated               */
    Bool                    enableSrgr;

    /**< Set to TRUE if internal framesync to be generated              */

    Bool                    enableFsg;
    /**< Configuration to use for the McBSP */

    Ptr                     mcbspRawCfgPtr;
    /**<  Initial setup for the McBSP */

    PSP_mcbspOperatingMode  mode;
    /**< Mode in which mcbsp post needs to be created*/

    PSP_OpMode              opMode;
    /**< Operation mode of driver i.e interrupt or EDMA      */

    Uint32                  MasterFreq;

    Uint32                  ModuleClkFreq;
    /**< Module Clock Frequency      */

} PSP_mcbspDevParams;



typedef struct PSP_mcbspChanParams_t
{

    Uint32                  noOfTdmChans;
    /**< Number of TDM channels used by the McBSP                       */

    Uint32                  intrNum;

    Uint32                   wordWidth;
   /**< The parameter informs the driver what is the width word (not slot)
    and this help driver indirectly to decided no. of bytes to be transfered
    into each serialer for each slot- This is very important parameter - in case of
    invalid value default value driver will assume is 32*/


    Ptr                     userLoopJobBuffer;
    /**< Buffer to be transferred when the loop job is running
      it should be noted that this buffer size should be n*userLoopjobLength
      where n is the no of serialisers configured in the direction of the
      channel we are creating*/

    Uint16                  userLoopJobLength;
    /**< Number of bytes of the userloopjob buffer for each serialiser
    Please note that this is no. of bytes and this should be pre-calcuated properly
    for word width of slot - Please refer the wordWidth of this structure*/

    PSP_Handle              edmaHandle;
    /**< Handle to the EDMA Driver                                      */

    PSP_mcbspGblCallback    gblCbk;
    /**<
     * callback required when global error occurs -
     * must be callable  directly from the ISR context
     */

}PSP_mcbspChanParams;


typedef struct PSP_mcbspTransParam_t
{

    Uint8           *transAddr;
    /**< Transmit data buffer pointer                                      */

    Uint8           *receiveAddr;
    /**< Transmit data buffer pointer                                      */

}PSP_mcbspTransParam;

/**
 * \brief McBSP Device ID.
 *
 * The ID of the device that the application wants to send/receive data on.
 * This would be of less significance, if the system contains only one
 * device of this type.
 */
typedef int PSP_McbspInstanceId;


/**<
 *  \brief Structure containing the application callback function and argument
 *
 *  Application shall pass a pointer to PSP_appCallback structure by
 *  specifying the callback function to be called and the argument to be
 *  passed back for the PSP_mcbspChanRead OR PSP_mcbspChanWrite APIs.
 *  The callback function 'fxn' is called with 'arg' and the other params as
 *  defined by the PSP_mcbspAppCallback typedef.
 */
typedef struct PSP_appMcbspCallback_t
{
    PSP_mcbspAppCallback     fxn;
    /**< Application callabck                                           */
    Ptr                      arg;
    /**< Application callabck argument                                  */
} PSP_appMcbspCallback;


/**
  * \brief McBSP Mode Enumeration
  *
  *  This enum defines the mode of operation of the McBSP Driver
  */
typedef enum
{
    /**< Use McBSP in Transmit Only */
    PSP_MCBSP_INPUT  = 0,
    /**< Use McBSP in Receive Only */
    PSP_MCBSP_OUTPUT

} PSP_mcbspMode;


/**
  * \brief McBSP Request Structure
  *
  *  This structure is required to be filled by application while making
  *  a request to McBSP Driver.
  */
typedef struct PSP_mcbspReq_t
{
    /**< Pointer to the application Buffer                              */
    Uint8                  *addr;

    /**< Size of the application Buffer                                 */
    Uint32                  size;

    /**< Application Buffer Pointer                                     */
    Uint8                   *receiveAddr;

    /**< Acutal bytes transmitted by the driver                         */
    Uint32                  actualSize;

    /**< McBSP Application call back per Request
     *   This call back is called in the Interrupt Context              */
    PSP_mcbspAppCallback    cbFxn;

    /**< Application Data. This data can be filled by application for
         request identification after completion                        */
    Ptr                     appPtr;

    /**< Status of the transfer. This data is filled by the driver.     */
    PSP_Result              status;

    /**< McBSP Application Data                                         */
    Ptr                     misc;

} PSP_mcbspReq;

/**
 * \brief  Enumeration for channel allocation status
 */
typedef enum
{
   PSP_MCBSP_UNALLOCATED = 0,
   /**< Channel not allocated  */
   PSP_MCBSP_ALLOCATED = 1
   /**< Channel already in use */
} PSP_McbspChannelStatus;

/**
 * \brief McBSP data stream contains whether single phase or dual phase
 */
typedef enum PSP_mcbspFrmPhase
{
    PSP_MCBSP_PHASE_SINGLE = 0,
    /**< McBSP data stream is single phase                              */
    PSP_MCBSP_PHASE_DUAL = 1
    /**< McBSP data stream is dual phase                                */
} PSP_mcbspFrmPhase;


/**
 * \brief Receive/Transmit data delay - delay for the first data bit
 *        from the start of the  data frame
 */
typedef enum PSP_McbspDataDelay
{
    PSP_MCBSP_DATADELAY_0_BIT  = 0,
    /**< McBSP data starts with 0 bit delay from the start of the frame   */
    PSP_MCBSP_DATADELAY_1_BIT  = 1,
    /**< McBSP data starts with 1 bit delay from the start of the frame   */
    PSP_MCBSP_DATADELAY_2_BITS = 2
    /**< McBSP data starts with 2 bit delay from the start of the frame   */
} PSP_McbspDataDelay;

/**
 * \brief Configuration for the McBSP data stream
 *
 * PSP_mcbspDataConfig specifies the configuration for the McBSP data stream
 * including the whether it is single phase or dual phase, number of frames,
 * the word length in each phase and data delay
 */
typedef struct PSP_mcbspDataConfig_t
{
    Uint32          phaseNum;
    /**< Phase of the McBSP data stream                                 */
    Uint32          wrdLen1;
    /**< Length of the data word in first phase                         */
    Uint32          wrdLen2;
    /**< Length of the data word in second when dual phase is selected  */
    Uint32          frmLen1;
    /**< Length of the data frame in first phase                        */
    Uint32          frmLen2;
    /**< Length of the data frame in second phase                       */
    Uint32          dataDly;
    /**< data delay */
} PSP_mcbspDataConfig;


/**
 *  \brief McBSP Transmitter or Receiver bit clock mode
 *
 *  Specifies whether the bit clock for the McBSP transmitter or receiver
 *  is generated internally using SRGR or externally fed
 */
typedef  enum
{
    PSP_MCBSP_BCLKCLKMODE_EXTERNAL = 0,
    /**< Receive or transmit bit clock is external for McBSP            */
    PSP_MCBSP_BCLKCLKMODE_INTERNAL = 1
    /**< Receive or transmit bit clock is internally generated by McBSP */
} PSP_McbspBclkMode;

/**
 *  \brief McBSP Frame Synchronization signal mode
 *
 *  Specifies whether the frame sync signal for the McBSP transmitter
 *  or receiver is generated internally using FSGR or externally fed
 */
typedef enum PSP_McbspFrSynchMode
{
    PSP_MCBSP_FRSYNCHMODE_EXTERNAL = 0,
    /**< Receive or transmit frame sync signal is external for McBSP         */
    PSP_MCBSP_FRSYNCHMODE_INTERNAL = 1
    /**< Receive or transmit frame sync is internally generated by McBSP     */
} PSP_McbspFrSynchMode;

/**
 *  \brief McBSP bit clock polarity
 *
 *  Specifies whether the bit clock signal for the McBSP transmitter
 *  or receiver is of rising edge or falling edge polarity
 */
typedef enum PSP_McbspBclkPol
{
    PSP_MCBSP_BCLK_RISING_EDGE  = 0,
    /**< McBSP bit clock is of rising edge polarity                     */
    PSP_MCBSP_BCLK_FALLING_EDGE = 1
    /**< McBSP bit clock is of falling edge polarity                    */
}PSP_McbspBclkPol;

/**
 *  \brief McBSP Frame Synchronization signal polarity
 *
 *  Specifies whether the frame sync signal for the McBSP transmitter
 *  or receiver is of active low or active high polarity
 */
typedef enum
{
    PSP_MCBSP_FSPOL_ACTIVE_HIGH = 0,
    /**< McBSP frame sync signal is of active high polarity             */
    PSP_MCBSP_FSPOL_ACTIVE_LOW  = 1
    /**< McBSP frame sync signal is of active low polarity              */
} PSP_McbspFsPol;



/**
 *  \brief Polarity of the CLKS pin
 *
 *  Set the polarity of the CLKS pin used to generate Frame Sync
 *  and bit clock
 */
typedef enum PSP_McbspSrgClkPol
{
    PSP_MCBSP_SRGCLKPOL_RISING_EDGE   = 0,
    /**< SRG clock is of rising edge polarity                           */
    PSP_MCBSP_SRGCLKPOL_FALLING_EDGE  = 1
    /**< SRG clock is of falling edge polarity                          */
} PSP_McbspSrgClkPol;

/**
 *  \brief Source for McBSP SRGR to generate the CLKX/R and FSX/R
 *
 *  Source for McBSP Sample Rate Generator to generate the
 *  bit clock and frame sync signals in master mode
 */
typedef enum
{
    PSP_MCBSP_SRGCLK_CLKS    =   0,
    /**< Use CLKS pin                                                   */
    PSP_MCBSP_SRGCLK_CLKCPU  =   1,
    /**< Use internal CPU clock                                         */
    PSP_MCBSP_SRGCLK_CLKR    =   2,
    /**< Use Rx bit clock                                               */
    PSP_MCBSP_SRGCLK_CLKX    =   3
    /**< Use Tx bit clock                                               */
} PSP_McbspSrgClk;

/**
 *  \brief McBSP digital loopback mode selection
 *
 *  Whether the McBSP should be enabled in digital loopback mode or not
 */
typedef  enum PSP_mcbspLoopback
{
    PSP_MCBSP_LOOPBACK_OFF = 0,
    /**< loopback mode on                                               */
    PSP_MCBSP_LOOPBACK_ON  = 1
    /**< loopback mode off                                              */
} PSP_mcbspLoopback;

/**
 *  \brief McBSP Sample rate generator transmit sync mode selection
 *
 *  Whether Transmit frame sync signal (FSX) due to DXR(1/2)-to-XSR(1/2)
 *  copy or Transmit frame sync signal driven by the sample rate
 *  generator frame sync signal, FSG.
 */
typedef  enum PSP_McbspSrgFsgm
{
    PSP_MCBSP_FSX_DXR_COPY = 0,
    /**< FSX due to DXR(1/2)-to-XSR(1/2) copy                           */
    PSP_MCBSP_FSX_FSG  = 1
    /**< FSX driven by the sample rate generator frame sync signal, FSG */
} PSP_McbspSrgFsgm;

/**
 *  \brief McBSP sample rate generator configuration structure
 *
 *  Configure the Sample rate  generator to generate the BCLK and
 *  Frame Sync signals at the specified rate in McBSP master mode
 */
typedef struct PSP_srgConfig_t
{
    Uint32  srgInputClkMode;
    /**< Source for the Sample rate generator                           */
    PSP_McbspSrgFsgm srgFsgmMode;
    /**< Sample rate generator transmit sync mode selection             */
    Uint32  bclkRate;
    /**< Set Bit clock rate - Divide down value of SRGR input clock     */
    Uint32  srgFrmPulseWidth;
    /**< Set the Frame Sync Pulse width in terms of clock ticks         */
    Uint32  srgFrmPeriod;
    /**< Set the frame sync signal period in terms of clock ticks       */
    Uint32  srgClkPolarity;
    /**< Set Frame sync and Bit clock polarity                          */
} PSP_srgConfig;


/**
 *  \brief  PSP McBSP Ioctl commands
 *
 *  McBSP Ioctl commands
 */
typedef enum PSP_McbspIoctlCmd
{
    PSP_CTRL_McBSP_START = 0,
    /**< Starts the data transfer                                       */
    PSP_CTRL_McBSP_STOP,
    /**< Stops the data transfer                                        */
    PSP_CTRL_McBSP_MUTE_ON,
    /**< Mute ON the transfer                                           */
    PSP_CTRL_McBSP_MUTE_OFF,
    /**< Take out of Muted state                                        */
    PSP_CTRL_McBSP_PAUSE,
    /**< Pause the playback operation                                   */
    PSP_CTRL_McBSP_RESUME,
    /**< Resume the playback operation                                  */
    PSP_CTRL_McBSP_CHAN_RESET,
    /**< Reset an I/O channel                                           */
    PSP_CTRL_McBSP_DEVICE_RESET,
    /**< Reset both input and output channel                            */
    PSP_CTRL_McBSP_CHG_ADDR,
    /**< Change the EDMA data or output address                         */
    PSP_CTRL_McBSP_CONFIG_DATA,
    /**< Configure the McBSP data stream                                */
    PSP_CTRL_McBSP_SRGR_START,
    /**< Start the McBSP SRG                                            */
    PSP_CTRL_McBSP_SRGR_STOP,
    /**< Stop the McBSP SRG                                             */
    PSP_CTRL_McBSP_FSGR_START,
    /**< Start the McBSP FSG                                            */
    PSP_CTRL_McBSP_FSGR_STOP,
    /**< Stop the McBSP FSG                                             */
    PSP_CTRL_McBSP_SET_CLKMODE,
    /**< Set Bit clock mode for the McBSP                               */
    PSP_CTRL_McBSP_SET_FRMSYNCMODE,
    /**< Set Frame Sync mode for the McBSP                              */
    PSP_CTRL_McBSP_CONFIG_SRGR,
    /**< configure Sample Rate Generator                                */
    PSP_CTRL_McBSP_SET_BCLK_POL,
    /**< Set the CLKR or CLKX clock polarity                            */
    PSP_CTRL_McBSP_SET_FRMSYNC_POL,
    /**< Set the FSR and FSX polarity                                   */
    PSP_CTRL_McBSP_MODIFY_LOOPJOB,
    /**< Enable/disable the loopjob                                     */
    PSP_CTRL_McBSP_RECEIVE_SYNCERR_INT_ENABLE,
    /**< Enable/disable the loopjob                                     */
    PSP_CTRL_McBSP_XMIT_SYNCERR_INT_ENABLE,
    /**< Enable/disable the loopjob                                     */
    PSP_CTRL_McBSP_LOOPBACK,
    /**< Reset the SPI channel                                */
	PSP_CTRL_McBSP_SPI_CHAN_RESET,
    /**< Reset the SPI device                                */
	PSP_CTRL_McBSP_SPI_DEVICE_RESET,
	PSP_CTRL_McBSP_SPI_SET_CS_POL,
	PSP_CTRL_McBSP_SPI_SET_CLKX_POL

} PSP_McbspIoctlCmd;


#ifdef __cplusplus
}
#endif /* extern "C" */

#endif /* _PSP_McBSP_H_ */

