/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file    dda_mcbsp_bios.c
 *
 *  \brief   Generic DM6437 McBSP Driver for the ARM and
 *           DSP/BIOS Platform.It uses the EDMA.
 *
 *      This file implements the McBSP mini-driver to be used with DSP-BIOS.
 *
 *      (C)      Copyright 2006,      Texas Instruments, Inc
 *
 *  \author Platform Support Group
 *
 *  \Revision
 *          0.1  Created
 *          0.2  Created  : Dynamic mode start/stop implemented
            0.3  Created  : psp architecture implementation
            0.4 Ported : vichu/Amit : ported for mcbsp from mcbsp
 */
/*
 * DSP/BIOS headers
 */
#include <std.h>
#include <log.h>
#include <ecm.h>
#include <c64.h>
#include <string.h>

#include <ti/sdo/pspdrivers/drivers/mcbsp/psp_mcbsp.h>
#include <ti/sdo/pspdrivers/drivers/mcbsp/src/psp_mcbspApi.h>
#include <ti/sdo/pspdrivers/drivers/mcbsp/src/dda_mcbsp_bios.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osProtect.h>
#include <ti/sdo/pspdrivers/pal_os/pal_oscache.h>

#ifdef CHIP_C6424
#include <ti/sdo/pspdrivers/soc/c6424/dsp/tistdtypes.h>
#include <ti/sdo/pspdrivers/soc/c6424/dsp/soc.h>
#endif

#ifdef CHIP_DM6437
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/tistdtypes.h>
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/soc.h>
#endif

#ifdef MCBSP_INSTRUMENTATION_ENABLED
#include <ti/sdo/pspdrivers/pal_os/bios/psp_log.h>
#endif /* MCBSP_INSTRUMENTATION_ENABLED */




/* Initialize port object for Two instances */
static McBSPPortObj mcbspPort[PSP_MCBSP_NUM_PORTS]={PORTOBJINIT,PORTOBJINIT};

/*
 * Prototypes of static functions
 */
static Int mcbsp_mdBindDev(Ptr *devp, Int devid , Ptr devParams );
static Int mcbsp_mdControlChan(Ptr chanp, Uint32 cmd, Ptr arg);
static Int mcbsp_mdDeleteChan(Ptr chanp);
static Int mcbsp_mdSubmitChan(Ptr chanp, IOM_Packet *packet);
static Int mcbsp_mdUnBindDev(Ptr devp);
static Int getIomErrorCode(PSP_Result pspCode);
static Int mcbsp_mdCreateChan(
                    Ptr                 *chanp,
                    Ptr                 devp,
                    String              name,
                    Int                 mode,
                    Ptr                 chanParams,
                    IOM_TiomCallback    cbFxn,
                    Ptr                 cbArg
                    );
static void mdLayerCallback(Ptr Port,
                            Ptr arg,
                            const PSP_mcbspReq *ioRequest,
                            Int mode);


/**
 * \brief Public IOM interface table
 *
 * This table holds the interface functions exposed by the mini-driver to
 * the GIO layer.
 */

IOM_Fxns McBSP_IOM_FXNS =
{
    &mcbsp_mdBindDev,
    &mcbsp_mdUnBindDev,
    &mcbsp_mdControlChan,
    &mcbsp_mdCreateChan,
    &mcbsp_mdDeleteChan,
    &mcbsp_mdSubmitChan
};

/**
 *
 *  \fn     static Int mcbsp_mdBindDev(Ptr *devp, Int devid , Ptr devParams )
 *  \brief  Allocates and configures the McBSP port specified by devid by
            calling PSP_mcbspCreate() function.
 *
 *  This function calls PSP_mcbspCreate () function which acquires
 *  the Handle of the McBSP.
 *
 * \param     devp         [OUT]    Pointer to hold allocated port object
 * \param     devid        [IN]     Device number (in this case, there are
 *                                  Four McBSP present from 0 to 3)
 * \param     devParams    [IN]     Device parameter structure that supplies
 *                                  data to initialize the McBSP
 *
 * \return
 *      if successful
 *                 IOM_COMPLETED
 *  if not successful
 *            IOM_EBADARGS
 *            IOM_EALLOC
 */
static Int mcbsp_mdBindDev(Ptr *devp, Int devid , Ptr devParams )
{

                    /* Handle to the currently using McBSP port(devID) */
    McBSPPortHandle         mcbspport  = NULL;
    PSP_mcbspDevParams      *devparams = NULL ;
    PSP_Result                  result = PSP_SOK;
    Int                      IOMStatus = IOM_COMPLETED;

    /* The driver must recieve a valid device parameters  */
    if(   (( NULL != devParams ) && ( PSP_MCBSP_NUM_PORTS > devid ) ) &&
            (NULL !=devp))
    {

        devparams  = (PSP_mcbspDevParams *)devParams;

        /* Get the device parameters of the specified port */
        mcbspport = &mcbspPort[devid];
        mcbspport->instNum = (Uint16)devid;

        if(TRUE == mcbspport->inUse)
        {
            IOMStatus = IOM_ENOTIMPL;
        }
        else
        {
            /* Mark the port as in use */
            mcbspport->inUse = TRUE;

            /*Inturn call the PSP function to do all init work*/
            result = (PSP_Result)PSP_mcbspCreate(
                                    devid,
                                    (PSP_mcbspDevParams *)devparams,
                                    (Ptr)mcbspport
                                    );

            if( result != PSP_SOK )
            {
                IOMStatus = IOM_EALLOC;
                *devp = NULL;
            }
            else
            {
                *devp = mcbspport;
            }
        }
    }
    else
    {
        IOMStatus = IOM_EBADARGS;
    }

    return IOMStatus;
}



/**
 *  \fn static Int mcbsp_mdCreateChan(Ptr *chanp,Ptr devp,String name,Int mode,
 *                    Ptr chanParams,IOM_TiomCallback cbFxn,Ptr cbArg)
 *
 *
 *
 *  \brief  Creates a communication channel in specified mode to communicate
 *          data between the application and the McBSP device instance. This
 *            function inturn calls DDC layer function PSP_mcbspOpen ().
 *       -  Returns configured channel handle to application.

 *  This function is called in response to a GIO_create call.
 *
 *      Pre-requisites:
 *       -  Valid chanParams structure
                This takes much information pertaining to
                mcbsp channel configuration such as
                * how many serialisers are used for this channel
                * what are they
                *communication parameters , clock setting etc

 *       -  Valid device pointer
 *
 * \note      Please refer to Appendix A : IOM interface of spru616
 *            DSP/BIOS Driver Developer's Guide for details on
 *            mcbsp_mdCreateChan parameters.
 *
 * \param     chanp        [IN]     Channel Handler
 * \param     devp         [IN]     Device pointer
 * \param     name         [IN]     McAsp Instance name like MCBSP0
 * \param     mode         [IN]     channel  mode -> input or output
 * \param     chanParams   [IN]     channel parameters from user
 * \param     cbFxn        [IN]     callback function pointer
 * \param     cbArg        [IN]     callback function Arguments
 *
 * \return    IOM_COMPLETED     if successful
 *            IOM_EBADIO        if not successful
 *            IOM_EALLOC            "
 *            IOM_EBADARGS      if passed invalid chanParams structure
 *
 */
/* ARGSUSED */
static Int mcbsp_mdCreateChan(
                Ptr                 *chanp,
                Ptr                 devp,
                String              name,
                Int                 mode,
                Ptr                 chanParams,
                IOM_TiomCallback    cbFxn,
                Ptr                 cbArg
                )
{

    McBSPPortHandle port            = NULL;
    Uint32          cookie          = 0;
    Int32           IOMStatus       = IOM_EBADARGS;
    PAL_Result      result          = PSP_E_INVAL_PARAM;
    McBSPChannelHandle  chan        = NULL;
    PSP_mcbspChanParams *chanparam  = NULL;

    /*validate parameters*/
    if ( (((NULL == chanParams) || (NULL == cbFxn)) || (NULL == cbArg)) ||
         (((IOM_INPUT != mode) && (IOM_OUTPUT != mode)) || (NULL == devp)))
    {
         IOMStatus = IOM_EBADARGS;
    }
    else
    {
        port = (McBSPPortHandle)devp;
        chanparam = (PSP_mcbspChanParams *) chanParams;

        if((NULL == port) || (NULL == chanparam))
        {
            IOMStatus = IOM_EBADARGS;
        }
        else
        {
            /*set channel mode and assign the respective channel to the port*/
            if (IOM_INPUT == mode)
            {
                chan = (McBSPChannelHandle)&port->RcvObj;
            }
            else
            {
                chan = (McBSPChannelHandle)&port->XmtObj;
            }

            if(NULL == chan)
            {
                IOMStatus = IOM_EBADARGS;
            }
            else
            {
                if (IOM_INPUT == mode)
                {
                    chan->mode = IOM_INPUT;
                }
                else
                {
                    chan->mode = IOM_OUTPUT;
                }

                chan->portHandle = port;


                if (TRUE == chan->inUse)
                {
                     IOMStatus = IOM_EALLOC;
                }
                else

                {
                    /* enter critical section */
                    PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

                    chan->inUse = TRUE;
                    /* Mark channel status as allocated.*/
                    chan->status = PSP_MCBSP_ALLOCATED;


                    /* Leave critical section */
                    PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

                    /*Assign the respective callback function*/
                    chan->cbFxn = cbFxn;
                    chan->cbArg = cbArg;


                    chan->ioHandle = PSP_mcbspOpen(
                                         port->instNum,
                                         (PSP_mcbspMode)(chan->mode),
                                         chanParams,
                                         (PSP_mcbspAppCallback)&mdLayerCallback,
                                         chanparam->edmaHandle,
                                         NULL
                                         );



                    if (NULL == chan->ioHandle)
                    {
                        result =  PSP_E_DRIVER_INIT;
                    }
                    else
                    {
                        result = PSP_SOK;
                    }

                    /*Retrive the error code returned*/
                    IOMStatus = getIomErrorCode (result);

                    if( IOM_COMPLETED == IOMStatus )
                    {
                        *chanp = (Ptr) chan;
                    }
                }
            }
        }
    }

    return IOMStatus;
}



/**
 *  \fn static Int mcbsp_mdSubmitChan(Ptr chanp, IOM_Packet * const packet)
 *
 *  \brief Submit a I/O packet to a channel for processing
 *
 *  The GIO layer calls this function to cause the mini-driver
 *  to process the IOM_Packet for read/write/flush/abort operations.
 *  This function calls DDC layer funtion PSP_mcbspRead / PSP_mcbspwrite /
 *  PSP_mcbspFlushRequests / PSP_mcbspAbortRequests calls  to execute the
 *  request.
 *  Submits and I/O packet to channel specified
 *
 *  \param   chanp         [IN]    Pointer to channel
 *  \param   packet        [IN]    Pointer to packet to be submitted
 *
 *  \return  IOM_COMPLETED, if packet is fully processed
 *           IOM_ABORTED,   if packet is aborted
 *           IOM_PENDING,   if packet is not fully processed
 */

static Int mcbsp_mdSubmitChan(Ptr chanp, IOM_Packet * const packet)
{

    McBSPChannelHandle        chan          = NULL;
    PAL_Result                result        = PSP_E_INVAL_PARAM;
    Int32                     IOMStatus     = IOM_EBADARGS;
    PSP_mcbspReq              ioRequest     =   {
                                                    NULL,   /* addr */
                                                    0,      /* size */
                                                    0,      /* actualSize */
                                                    NULL,   /* cbFxn */
                                                    NULL,   /* appPtr */
                                                    0,       /* status */
                                                    NULL    /* misc */
                                                 };

        /* Abort if channel is not created /allocated*/

    if((NULL == chanp) || (NULL == packet))
    {
        IOMStatus = IOM_EBADARGS;
    }
    else
    {
        /* Channel Handle */
        chan = chanp;

#ifdef MCBSP_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT(
                &DVTEvent_Log,
                "McBSP",
                PSP_DVT_DESC(
                            PSP_DVT_eFUNC_START,
                            PSP_DVT_dCHANNEL_ID_INST,
                            PSP_DVT_dPACKET_ID,
                            PSP_DVT_dSIZE_BYTES
                            ),
                CHANNELID_INSTANCEID(
                    packet->cmd,
                    chan->portHandle->instNum),
                packet,
                packet->size);
#endif /* MCBSP_INSTRUMENTATION_ENABLED */

        if( FALSE == chan->inUse )
        {
            IOMStatus = IOM_EBADARGS;
        }
        else
        {
            if( NULL == packet->addr)
            {
                if((IOM_READ == packet->cmd)  && (IOM_WRITE == packet->cmd))
                 {
                    IOMStatus = IOM_EBADARGS;
                 }
            }
            else
            {
                ioRequest.addr          = (Uint8 *)packet->addr;
                ioRequest.size          = packet->size;
                ioRequest.cbFxn         = NULL;
                ioRequest.appPtr        = (Ptr)packet;
                ioRequest.actualSize    = 0;
                ioRequest.status        = PSP_SOK;

            }
            switch (packet->cmd)
            {
                case IOM_READ:
                        if ( NULL != ioRequest.addr)
                        {
                            result =  PSP_mcbspRead ((PSP_Handle)chan->ioHandle,
                                                     &ioRequest,NULL);
                        }
                        break;

                case IOM_WRITE:
                        if ( NULL != ioRequest.addr)
                        {
                            result = PSP_mcbspWrite ((PSP_Handle)chan->ioHandle,
                                                     &ioRequest,NULL);
                        }
                        break;

                case IOM_ABORT:
                            result =   PSP_mcbspAbortRequests (
                                                    (PSP_Handle)chan->ioHandle,
                                                    NULL);
                        break;

                case IOM_FLUSH:
                        result =   PSP_mcbspFlushRequests (
                                                    (PSP_Handle)chan->ioHandle,
                                                    NULL);
                        break;

                default:
                        result = PSP_E_INVAL_PARAM;
                        break;

            }
            IOMStatus = getIomErrorCode (result);
        }
    }

#ifdef MCBSP_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(&DVTEvent_Log,
                "McBSP",
                PSP_DVT_DESC(
                            PSP_DVT_eFUNC_START,
                            PSP_DVT_dCHANNEL_ID_INST,
                            PSP_DVT_dPACKET_ID,
                            PSP_DVT_dSIZE_BYTES
                            ),
                CHANNELID_INSTANCEID(
                    packet->cmd,
                    chan->portHandle->instNum),
                packet,
                0);
#endif /* MCBSP_INSTRUMENTATION_ENABLED */

    return IOMStatus;

}


/**
 *  \fn static Int mcbsp_mdUnBindDev(Ptr devp)
 *
 *  \brief Closes the McBSP device from use.
 *
 *  mdUnBindDev() unwinds the actions performed in mdbinddev.
 *  This is accomplished by calling PSP_mcbspDelete
 *
 * \param     devp         [IN]    Pointer to port object to be freed
 *
 * \return    IOM_COMPLETED     if successful
 *            IOM_ABORTED       if not successful
 */
static Int mcbsp_mdUnBindDev(Ptr devp)
{
    McBSPPortHandle     mcbspport   = NULL;
    PAL_Result          result      = PSP_E_INVAL_PARAM;
    Int32               IOMStatus   = IOM_EBADARGS;

    /*
    * Check if this is a valid handle
    */
    if (NULL == devp)
    {
        IOMStatus = IOM_EBADARGS;
    }
    else
    {
        mcbspport = (McBSPPortHandle) devp;

        if ((NULL == mcbspport)|| (TRUE != mcbspport->inUse))
        {
            IOMStatus = IOM_EBADARGS;
        }
        else
        {
            /*
            * Free allocated port
            */
            mcbspport->inUse = FALSE;

            /*
            * Close and free mcbsp port handle
            */
            result = PSP_mcbspDelete(mcbspport->instNum,NULL);

            IOMStatus = getIomErrorCode(result);
        }
    }

    return IOMStatus;
}


/**
 *  \fn     static Int mcbsp_mdDeleteChan(Ptr chanp)
 *
 *  \brief  frees a channel and all it's associated resources.
 *
 *          mdDeleteChan() acts as a wrapper function to DDC layer funtion
 *
 *  \param  chanp  [IN]    Pointer to channel
 *
 * \return  IOM_COMPLETED     if successful
 *          IOM_ABORTED        if not successful
 */
static Int mcbsp_mdDeleteChan(Ptr chanp)
{
    McBSPChannelHandle  chan = NULL;
    PAL_Result          result      = PSP_E_INVAL_PARAM;
    Int             IOMStatus   = IOM_EBADARGS;

    if(NULL == chanp)
    {
        IOMStatus = IOM_EBADARGS;
    }
    else
    {
        chan        = (McBSPChannelHandle) chanp;
        /* Clear the channel in use parater to channel create */
        if ((NULL == chan ) || (TRUE != chan->inUse) )
        {
            IOMStatus = IOM_EBADARGS;
        }
        else
        {
            chan->inUse = FALSE;
            result = PSP_mcbspClose((PSP_Handle)chan->ioHandle,NULL);

            IOMStatus = getIomErrorCode(result);
        }
    }
    return IOMStatus;
}


/**
 *  \fn static Int mcbsp_mdControlChan(Ptr chanp, Uint32 cmd, Ptr arg)
 *
 *  \brief Implements the IOCTLS for McBSP IOM mini driver by calling
 *         DDC layer function ddc_mcbspControlChan.
 *
 *  mdControlChan() implements recieves IOCTL commands from the
 *  GIO layer and then forwards the commands to the DDC by calling
 *  PSP_mcbspIoctl.
 *
 *  \param  chanp  [IN]    Pointer to channel
 *  \param  cmd    [IN]    specific IOCTL command
 *  \param  arg    [IN]    arguments required for specific commands
 *
 *  \return  IOM_COMPLETED, if command is executed correctly
 *            IOM_ABORTED,   if command returns error during execution
 *            IOM_ENOTIMPL,  if command is not supported
 *            IOM_EBADARGS   if args are not correct
 */

static Int mcbsp_mdControlChan(Ptr chanp, Uint32 cmd, Ptr arg)
{
    McBSPChannelHandle  chan        = NULL;
    PAL_Result          result      = PSP_E_INVAL_PARAM;
    Int32               IOMStatus   = IOM_EBADARGS;

    if(NULL == chanp)
    {
        IOMStatus = IOM_EBADARGS;
    }
    else
    {
        chan = (McBSPChannelHandle ) chanp;
        if (TRUE != chan->inUse)
        {
            IOMStatus = IOM_EBADARGS;
        }
        else
        {
            result = PSP_mcbspIoctl ( (PSP_Handle)chan->ioHandle,
                                      (PSP_McbspIoctlCmd)cmd,arg,NULL);

            IOMStatus = getIomErrorCode(result);
        }
    }
    return (IOMStatus);
}

/*!
 *  \fn void  MCBSP_IOM_init()
 *
 *  \brief Initializes McBSP IOM driver's data structures
 *
 *  This function initializes the McBSP IOM driver's data structures including
 *  PortObject instance and channel object instance. Because the BIOS is not
 *  initialsing the globals and these leads the problematic situations.
 *
 *  \param  [IN]  none
 *
 *  \return Pointer to the McBSP IOM driver function table
 */
void  MCBSP_IOM_init()
{
    McBSPPortHandle     port   = NULL;
    McBSPChannelHandle  chan        = NULL;
    Uint32              numPorts    = 0;

    /* Zero initialize the memory */
    memset(mcbspPort, 0, (sizeof(McBSPPortObj) * PSP_MCBSP_NUM_PORTS));

    for (numPorts = 0; numPorts < PSP_MCBSP_NUM_PORTS; numPorts++)
    {
        port = (McBSPPortHandle) &mcbspPort[numPorts];
        port->inUse = FALSE;

        chan = (McBSPChannelHandle)(&port->RcvObj);

        if(NULL != chan)
        {
            chan->inUse = FALSE;
        }

        chan = (McBSPChannelHandle)(& port->XmtObj);

        if(NULL != chan)
        {
            chan->inUse = FALSE;
        }

        PSP_mcbspInit(numPorts);
    }
    return ;
}

/*!
 *  \fn static void mdLayerCallback(Ptr Port, Ptr arg,
 *                                  const PSP_mcbspReq *ioRequest,
 *                                  Int mode)
 *
 *  \brief DDC layer callback
 *
 *  This callback is made from the DDC layer. This function restores appropriate
 *  data into the IOP and  calls GIO callback
 *
 *  \param  [IN]  Port      Port handle
 *  \param  [IN]  arg       optional device-defined data structure
 *  \param  [IN]  ioRequest pointer to io request
 *  \param  [IN]  mode      channel  mode -> input or output
 *
 *  \return  Void
 */

static void mdLayerCallback(Ptr Port, Ptr arg,
                            const PSP_mcbspReq *ioRequest,
                            Int mode)
{
    IOM_Packet          *packet     = NULL;
    McBSPPortHandle     port        = NULL;
    McBSPChannelHandle  chan        = NULL;

    if (( NULL != Port ) && ( NULL != ioRequest ) )
    {
        /* To remove compiler warning */
        if(NULL != arg)
        {
            arg = arg;
        }
        port = Port;
        packet    = (IOM_Packet *)ioRequest->appPtr;

        if ( NULL != packet)
        {
            packet->size    = ioRequest->size;
            packet->status  = ioRequest->status;

            if (PSP_MCBSP_INPUT == mode)
            {
                chan = (McBSPChannelHandle)&port->RcvObj;
            }
            else
            {
                chan = (McBSPChannelHandle)&port->XmtObj;
            }

            packet->status = getIomErrorCode(packet->status);

            if (NULL != chan )
            {
                if (NULL != chan->cbFxn)
                {
                    /* Application callback */
                    (*chan->cbFxn)(chan->cbArg,packet);
                }
            }
        }
    }

}

/*!
 *  \fn static Int getIomErrorCode(PSP_Result pspCode)
 *
 *  \brief get IOM Error code
 *
 *  fuction will get the IOM Error Code
 *
 *  \param  [IN]  pspCode      PSP error code
 *
 *  \return  IOM Error Code
 */

static Int getIomErrorCode(PSP_Result pspCode)
{
    /*-------------------------------------------------------------------------
                            IOM Error code look up table
    -------------------------------------------------------------------------*/
    /**
     * The following mapping shows the mapping done from PSP_Result to IOM error
     * code. This will be used in scenario where IOMini driver interfaces with
     * the DDC (PSP component)
     *
     * PSP_SOK                  (0)   IOM_COMPLETED            0
     * PSP_SINPROGRESS          (1)   IOM_PENDING              1
     * PSP_E_DRIVER_INIT        (-1)  IOM_ETIMEOUTUNREC      -11
     * PSP_E_NO_MEMORY          (-2)  IOM_EALLOC              -5
     * PSP_E_RESOURCES          (-3)  IOM_EALLOC              -5
     * PSP_E_INVAL_STATE        (-4)  IOM_EBADMODE            -7
     * PSP_E_INVAL_PARAM        (-5)  IOM_EBADARGS           -10
     * PSP_E_NOT_SUPPORTED      (-6)  IOM_ENOTIMPL            -9
     * PSP_E_IO_CANCEL_FAIL     (-7)  IOM_ETIMEOUT            -2
     * PSP_E_FIFO_NOT_ENABLED   (-8)  IOM_EBADMODE            -7
     * PSP_E_INVALID_MODE       (-9)  IOM_EBADMODE            -7
     * PSP_E_INVALID_CMD        (-10) IOM_EBADARGS           -10
     *
     */
    static Int iomErrorCodeList[14] ={
                                        IOM_EBADARGS,
                                        IOM_EBADMODE,
                                        IOM_EBADMODE,
                                        IOM_ETIMEOUT,
                                        IOM_ENOTIMPL,
                                        IOM_EBADARGS,
                                        IOM_EBADMODE,
                                        IOM_EALLOC,
                                        IOM_EALLOC,
                                        IOM_ETIMEOUTUNREC,
                                        IOM_COMPLETED,
                                        IOM_PENDING,
                                        IOM_FLUSHED,
                                        IOM_ABORTED
                                     };
    Int iomCode = pspCode+10;

    if((iomCode < 14) && (iomCode >= 0))
    {
        iomCode = iomErrorCodeList[iomCode];
    }
    else
    {
        iomCode = IOM_EBADIO;
    }

    return(iomCode);
}

/**
 *  \fn     PSP_Result _DDA_McBSP_RegisterInt(Ptr chanp,
 *                                            Uint32 intNum,
 *                                            Uint16 evt,
 *                                            pspMcbspIsr intIsr)
 *
 *  \brief   register interrupt handler for channel mode
 *
 *  \param    chanp        [IN]    Pointer to channel
 *  \param    intNum       [IN]    interrupt number
 *  \param    evt          [IN]    event number
 *  \param    intIsr       [IN]    function pointer
 *
 *  \return   If success return PSP_SOK
 */
PSP_Result _DDA_McBSP_RegisterInt(Ptr chanp,
                                  Uint32 intNum,
                                  Uint16 evt,
                                  pspMcbspIsr intIsr)
{

    McBSPChannelHandle  chan        = NULL;
    ECM_Attrs           ecmattrs    = ECM_ATTRS;
    static Uns          cookie      = 0;
    PSP_Result          result      = PSP_SOK;

    intNum = intNum;

    if(( NULL == chanp) || ( NULL == intIsr) )
    {
        result = PSP_E_INVAL_PARAM;
    }
    else
    {

        chan = (McBSPChannelHandle)chanp;

        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);


        /*Enable the interrupt later*/
        ecmattrs.unmask = 0u;

        /*setup arg such that isr will get 'chan' as arg*/
        ecmattrs.arg    = (Arg)chan;

        /* Plug the ISR with interrupt (event) number */
        ECM_dispatchPlug(evt, (ECM_Fxn)intIsr, &ecmattrs);

        /*
        * Enabling the HWI_ID where corresponding ECM group is configured
        */
        C64_enableIER(HWI_EVT02_CMB_IRQ);/*TBD to change to proper HWI*/
        ECM_enableEvent(evt);

        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
    }

    return result;
}

/**
 *  \fn  PSP_Result _DDA_McBSP_UnRegisterInt(Uint16 devid, Uint16 evt,Int mode)
 *
 *  \brief   unregister interrupt handler for channel mode
 *
 *  \param    devid        [IN]    instance number
 *  \param    evt          [IN]    interrupt number
 *  \param    mode         [IN]    input or output
 *
 *  \return  PSP_SOK
 */
PSP_Result _DDA_McBSP_UnRegisterInt(Uint16 devid, Uint16 evt,Int mode)
{

    Uint32          cookie      = 0;

    devid = devid;
    mode  = mode;

    PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

    /* Disable the event*/
    ECM_disableEvent(evt);

    /* Enabling the interrupts */
    PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

    return PSP_SOK;
}
