/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file    dda_mcbsp_bios.h
 *
 *  \brief    Header file for McBSP mini-driver
 *
 *  This file has the globally visible data structures and definitions
 *  for the McBSP mini-driver.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author Platform Support Group
 *
 *  \Revision
 *          0.1  Created  Khanapuri / vichu
 *          0.2  Created  Khanapuri / vichu : Dynamic mode start/stop implemented

            0.3  modified   Vichu/Amit : Ported for mcbsp
 */

#ifndef _DDA_MCBSP_BIOS_H
#define _DDA_MCBSP_BIOS_H

#ifdef __cplusplus
extern "C" {
#endif

#include <std.h>
#include <iom.h>
#include <ti/sdo/pspdrivers/drivers/mcbsp/psp_mcbsp.h>


struct McBSPPortObj_t;
/*
 * Channel function definitions
 */

/**
 * \brief Channel Object
 *
 * This structure maintains the current channel state. It also holds
 * information on DMA channel being used and holds the application
 * callback function to be called in case of an interrupt.
 *
 * This structure is initialized by mdCreateChan and a pointer to this
 * is passed down to all other channel related functions. Lifetime of
 * the data structure is from its creation by mdCreateChan till it
 * is invalidated by mdDeleteChan.
 */

typedef struct McBSPChannelObj_t
{
    PSP_McbspChannelStatus      status;
    /**< Keeps tab of whether channel is already in use
     *  - takes value of UNALLOCATED or ALLOCATED          */

    Uint16                      mode;
    /**< mode for channel                                 */

    Uint16                      inUse;
    /**< mode for cache                                     */

    struct McBSPPortObj_t       *portHandle;
    /**< Pointer to McBSP device
     *  - Back pointer to device configuration structure   */

    PSP_Handle                  ioHandle;
    /**< ddc channel Handle */

    IOM_TiomCallback            cbFxn;
    /**<  Notify client when I/O complete    */

    Ptr                         cbArg;
    /**< Callback Function                                              */

} McBSPChannelObj, *McBSPChannelHandle;



/**
  * Default Initialization of channel object
  */

#define CHANOBJINIT { \
    /* .status              = */    PSP_MCBSP_UNALLOCATED,        \
    /* .mode                = */    NULL,                         \
    /* .inUse               = */    FALSE,                         \
    /* ._PortObj            = */    NULL,                         \
    /* .DDC_mcbspChanHandle = */    NULL,\
    /* .cbFxn               = */    NULL,\
    /* .cbArg               = */    NULL\
}

/**
 * \brief Port Object
 *
 * This data structure holds the current device state. It also
 * holds the handle to the McBSP. The data structure is
 * initialized during mcbsp_mdBindDev, which is called during DSP-
 * BIOS initialization, and is persistent till it is
 * invalidated by mcbsp_mdUnBindDev.
 */
typedef struct McBSPPortObj_t
{
    Bool                        inUse;
    /**< Marks if port is currently in use                 */

    Uint16                      instNum;
    /**< Preserve  instance number in port                 */


    McBSPChannelObj             XmtObj;
    /**< Holds transmit channel to the McBSP.              */

    McBSPChannelObj             RcvObj;
    /**< Holds receive channel to the McBSP.               */

}McBSPPortObj , *McBSPPortHandle;

/**
  * Default Initialization of port object
  */
#define PORTOBJINIT { \
    /* .inUse     = */          FALSE,\
    /* .InstNum = */            0,\
    /* .RcvObj,XmtObj= */      CHANOBJINIT,\
                               CHANOBJINIT\
}

/**
 *  \fn     PSP_Result _DDA_McBSP_RegisterInt(Ptr chanp, Uint32 intNum,
 *                                            Uint16 evt,
 *                                            pspMcbspIsr intIsr)
 *
 *  \brief   register interrupt handler for channel mode
 *
 *  \param    chanp        [IN]    Pointer to channel
 *  \param    intNum       [IN]    interrupt number
 *  \param    evt          [IN]    event number
 *  \param    intIsr       [IN]    function pointer
 *
 *  \return   If success return PSP_SOK
 */
PSP_Result _DDA_McBSP_RegisterInt(Ptr chanp,
                                  Uint32 intNum,
                                  Uint16 evt,
                                  pspMcbspIsr intIsr);



/**
 *  \fn      PSP_Result _DDA_McBSP_UnRegisterInt(Uint16 devid, Uint16 evt,Int mode)
 *
 *  \brief   unregister interrupt handler for channel mode
 *
 *  \param    devid        [IN]    instance number
 *  \param    evt          [IN]    interrupt number
 *  \param    mode         [IN]    input or output
 *
 *  \return  PSP_SOK
 */

PSP_Result _DDA_McBSP_UnRegisterInt (Uint16 devid, Uint16 evt,Int mode);



#ifdef __cplusplus
}
#endif /* extern "C" */

#endif /* _DDA_MCBSP_BIOS_H*/
