/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file    ddc_mcbsp.c
 *
 *  \brief   McBSP driver core processing implementation
 *
 *      This file implements the driver core for McBSP peripheral
 *      This file needs appropriate support files/function (PALOS,PALSYS , some
 *      DDA (register isr etc.)functions to work properly
 *      The file (driver) has code for configure the mcbsp , to perform IO in
 *      in EDMA mode. Interrupt mode code is the to perform I/O but due to
 *      latency and timing constraints underrun/error may occur.
 *      This driver supports multiple serialisers (for buffer handling check
 *      documentation of the payload structure) and all modes supported by mcbsp
 *      peripheral. Please do note that this driver may require expert
 *      configuration during initialisation.
 *
 *      (C)      Copyright 2006,      Texas Instruments, Inc
 *
 *  \author Platform Support Group
 *
 *  \Revision
 *      0.1  Created  :created for DRA44X -presilicon
 *      0.2  modified :Dynamic mode start/stop implemented
 *      0.3  modified :Modified for EDMA - DM64LC Presilicon
 *      0.4  modified :cleanup / DM64Lc post silicon changes
 *                    :implemented AB sync mode of EDMA for multiserialise
 *                     support
        0.5 modified : ported for mcbsp driver from mcasp driver
 */

#include <string.h>
#include <ti/sdo/edma3/drv/edma3_drv.h>
#include <ti/sdo/pspdrivers/pal_os/bios/psputils.h>
#include <ti/sdo/pspdrivers/drivers/mcbsp/src/ddc_mcbsp.h>
#include <ti/sdo/pspdrivers/drivers/mcbsp/src/psp_mcbspApi.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osProtect.h>
#include <ti/sdo/pspdrivers/pal_os/pal_oscache.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osmem.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osList.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osWait.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osInt.h>

#ifdef CHIP_C6424
#include <ti/sdo/pspdrivers/soc/c6424/dsp/csl_error.h>
#include <ti/sdo/pspdrivers/soc/c6424/dsp/soc.h>
#endif

#ifdef CHIP_DM6437
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/csl_error.h>
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/soc.h>
#endif

#ifdef MCBSP_INSTRUMENTATION_ENABLED
#include <ti/sdo/pspdrivers/pal_os/bios/psp_log.h>
#endif /* MCBSP_INSTRUMENTATION_ENABLED */


/* Local function declaration */
static PAL_Result ddc_isValidPacket (DDC_mcbspIoHandle ioHandle);
static PAL_Result ddc_edmaProcessPkt(Ptr iohandlep, DDC_mcbspIOP *ioPacket);
static PAL_Result ddc_mcbspSubmitReq(DDC_mcbspIoHandle ioHandle,
                                     PSP_mcbspReq *pspRequestp);
static PAL_Result ddc_setupEDMADuringInit (DDC_mcbspIoHandle ioHandle);
static PAL_Result ddc_updateDataPacketTolinkParams(
                                            DDC_mcbspIoHandle ioHandle,
                                            DDC_mcbspIOP * const ioPacket
                                            );
static PAL_Result ddc_edmaChannelPaRAMsetup(DDC_mcbspIoHandle ioHandle);
static void ddc_completeCurrentIO (DDC_mcbspIoHandle ioHandle);
static void ddc_cancelAndAbortAllIo (DDC_mcbspIoHandle ioHandle);
static void ddc_edmaMcbspCallback(Uint tcc,
                                  EDMA3_RM_TccStatus status,
                                  Ptr data);
static void ddc_abortReset(DDC_mcbspIoHandle ioHandle);
static int DDC_McBspSpiTransfer(DDC_mcbspIoHandle ioHandle,
                                DDC_mcbspIOP *ioPacket,
                                Ptr param );
static void ddc_copyReqtoIOP (PSP_mcbspReq * const pspRequest,
                              DDC_mcbspIOP *const ioPacket);
static void ddc_copyIOPtoReq (DDC_mcbspIOP *const ioPacket,
                              PSP_mcbspReq *const pspRequest);
static Uint8  DDC_SpiMstrClockCalculate(Uint32 ModuleClkFreq, Uint32 McbspSpiMstrFreq);
#ifdef GBL_INT
void ddc_mcbspISR (Ptr ddcIoObj);
#endif
extern PSP_Result _DDA_McBSP_RegisterInt(Ptr chanp,
                                         Uint32 intNum,
                                         Uint16 evt,
                                         pspMcbspIsr intIsr);
extern PSP_Result _DDA_McBSP_UnRegisterInt (Uint16 devid, Uint16 evt,Int mode);

/* ISR prototypes */
void ddc_mcbspRxISR(Ptr ddcIoObj);
void ddc_mcbspTxISR(Ptr ddcIoObj);
static PAL_Result DDC_McBspSpiSetupEdmaInit( DDC_mcbspIoHandle ioHandle );
static PAL_Result DDC_SPIEdmaSetParamas(DDC_mcbspIoHandle ioHandle );
static void ddc_edmaMcbspSPITXCallback(Uint tcc,
                                       EDMA3_RM_TccStatus status,
                                       Ptr data);
static void ddc_edmaMcbspSPIRXCallback(Uint tcc,
                                       EDMA3_RM_TccStatus status,
                                       Ptr data);



/*
These buffers will be used in case if application has not given any data buffer
and not any app specific loop job buffer and not yet stopped the Mcbsp
*/
static localTempBuffer loopDstBuf[PSP_MCBSP_NUM_PORTS];
static localTempBuffer loopSrcBuf[PSP_MCBSP_NUM_PORTS];

/* Spi Trancive buffer */
#pragma DATA_ALIGN(SpiTransiveBuff, 32);
static Uint8 SpiTransiveBuff[PSP_SPI_TRANSIVE_BUFF_LEN];
/*
Buffer used when mute is ON
*/
static localTempBuffer muteBuf[PSP_MCBSP_NUM_PORTS];

/*
Data structure holding the device information
*/
static  DDC_mcbspDevice ddcMcbspPort[PSP_MCBSP_NUM_PORTS];

/*
Default McBSP Configuration for receive in case not provided by Application
*/
static LLC_RcvDataSetup rcvDataCfg =
{

    LLC_MCBSP_PHASE_SINGLE,
    (LLC_McbspWordLen )LLC_MCBSP_WORDLEN_32,
    1,
    (LLC_McbspWordLen )0,
    1,
    LLC_MCBSP_FRMSYNC_DETECT,
    LLC_MCBSP_COMPAND_OFF_MSB_FIRST,
    LLC_MCBSP_DATADELAY_1_BIT,
    LLC_MCBSP_RJUSTDXENA_RJUST_RZF,
    LLC_MCBSP_INTMODE_ON_READY,
    LLC_MCBSP_32BIT_REVERS_DISABLE
};

/*
Default McBSP Configuration for transmit in case not provided by Application
*/
static LLC_XmitDataSetup xmtDataCfg =
{

    LLC_MCBSP_PHASE_SINGLE,
    (LLC_McbspWordLen )LLC_MCBSP_WORDLEN_32,
    1u,
    (LLC_McbspWordLen )0,
    1u,
    LLC_MCBSP_FRMSYNC_DETECT,
    LLC_MCBSP_COMPAND_OFF_MSB_FIRST,
    LLC_MCBSP_DATADELAY_1_BIT,
    LLC_MCBSP_RJUSTDXENA_RJUST_RZF,
    LLC_MCBSP_INTMODE_ON_READY,
    LLC_MCBSP_32BIT_REVERS_DISABLE
};

/*
Default McBSP Configuration for Clock in case not provided by Application
*/
static LLC_mcbspClkSetup clkCfg =
{
    LLC_MCBSP_FSCLKMODE_EXTERNAL,      /* XMT Frame-sync                    */
    LLC_MCBSP_FSCLKMODE_EXTERNAL,      /* RCV Frame-sync                    */
    LLC_MCBSP_TXRXCLKMODE_EXTERNAL,    /* XMT clock                         */
    LLC_MCBSP_TXRXCLKMODE_EXTERNAL,    /* RCV clock                         */
    LLC_MCBSP_FSPOL_ACTIVE_HIGH,       /* XMT Frame-sync Active High        */
    LLC_MCBSP_FSPOL_ACTIVE_HIGH,       /* RCV Frame-sync Active High        */
    LLC_MCBSP_CLKPOL_TX_RISING_EDGE,   /* XMT clock Falling Edge            */
    LLC_MCBSP_CLKPOL_RX_FALLING_EDGE,  /* RCV clock Rising Edge             */
    1u,                                 /* Frame-sync pulse width = 1 bit   */
    32u,                                /* Frame-sync pulse period = 32 bit */
    255u,                               /* CLKGDIV = 1 means divide by input
                                           clock by 2(Default value)        */
    (LLC_McbspSrgClk)LLC_MCBSP_SRGCLK_CLKCPU, /* SCLKME = 0, CLKSM = 1      */
    (LLC_McbspTxFsMode)LLC_MCBSP_TXFSMODE_SRG,/* If FSXM = 1,XMT Frame-sync */
};

/*
Default McBSP Configuration for hardware setup in case not provided by Application
*/
static LLC_McbspHwSetup myHwSetup =
{
    LLC_MCBSP_DLBMODE_OFF,
    &rcvDataCfg,
    &xmtDataCfg,
    &clkCfg,
    LLC_MCBSP_EMU_FREERUN,
    LLC_MCBSP_DX_HIGHZ,
    NULL
};

/**
 * \fn  PSP_Result PSP_mcbspInit(Uint32 instanceId)
 *
 * \brief   Initialization of McBsp configuration
 *
 *  This function initialises the data structures
 *
 * \param     instanceId      [IN]     Device number (in this case, there are
 *                                     two McBSP present 0 and 1)
 * \return
 *      if successful PSP_SOK , else PSP error code returned
 */
PSP_Result PSP_mcbspInit(Uint32 instanceId)
{
    PSP_Result      result = PSP_SOK;

    /* Zero Initilialize the data structure */
    if(PSP_MCBSP_NUM_PORTS < instanceId)
    {
        result = PSP_E_INVAL_PARAM;
    }
    else
    {
        memset(&ddcMcbspPort[instanceId], 0, sizeof(DDC_mcbspDevice));
    }

    return result;
}

/**
 * \fn  PSP_Result PSP_mcbspCreate( Uint32 instanceId,
 *                                  const PSP_mcbspDevParams * params,Ptr arg)
 *
 * \brief   Create a mcBsp Channel
 *
 *  This function initialises the data structures and completes the
 *  ground work needed for PSP_mcbspOpen
 *
 * \param     instanceId      [IN]     Device number (in this case, there are
 *                                     two McBSP present 0 and 1)
 * \param     params          [IN]     Device parameter structure that supplies
 *                                     data to initialize the McBSP
 * \param     arg             [IN]     application reference pointer
 * \return
 *      if successful PSP_SOK , else PSP error code returned
 */

PSP_Result PSP_mcbspCreate( Uint32 instanceId,
                            const PSP_mcbspDevParams * params,
                            Ptr arg)
{

    Int16                   status          = PSP_SOK;
    Uint32                  ldevid          = instanceId;
    Bool                    falsewhile      = FALSE;
    DDC_mcbspDeviceHandle   deviceHandle    = NULL;
    PSP_Result              ddcResult       = PSP_SOK;
    Uint32                  ctrlMask        = 0;


    do
    {
        falsewhile = FALSE;
        /* assign ddaPort handle to DDC port structure */
        /*Validate parameters*/
        if (((NULL == params)|| ( NULL == arg)))
        {
            ddcResult = PSP_E_INVAL_PARAM;
            break;
        }

        deviceHandle = &ddcMcbspPort[instanceId];

        PAL_osMemSet( (Ptr)&ddcMcbspPort[instanceId],
                      0,
                      sizeof(DDC_mcbspDeviceHandle));
        PAL_osMemSet( (Ptr)&loopDstBuf[instanceId],
                      0,
                      sizeof(localTempBuffer));
        PAL_osMemSet( (Ptr)&loopSrcBuf[instanceId],
                      0,
                      sizeof(localTempBuffer));
        PAL_osMemSet( (Ptr)&muteBuf[instanceId],
                      0,
                      sizeof(localTempBuffer));

        ddcMcbspPort[instanceId].InstNum = (Uint16)instanceId;

        deviceHandle->appHandle   = arg;
        deviceHandle->enablecache = params->enablecache;
        deviceHandle->currentMode = params->mode;
        deviceHandle->opMode      = params->opMode;
        deviceHandle->McbspSpiMstrFreq = params->MasterFreq;
        deviceHandle->ModuleClkFreq = params->ModuleClkFreq;
        /* Initialize the Enable Mask */
        deviceHandle->enableMask = (0u);

        /* Enable the McBSP Sample Rate generator  */
        if (0 != params->enableSrgr)
        {
            deviceHandle->enableMask |= LLC_MCBSP_CTRL_SRG_ENABLE;
        }

        /* Enable the McBSP Frame Sync generator  */
        if (0 != params->enableFsg)
        {
            deviceHandle->enableMask |= LLC_MCBSP_CTRL_FSYNC_ENABLE;
        }

        ddcResult = (PSP_Result) LLC_mcbspInit(NULL);

        if (PSP_SOK != ddcResult)
        {
            deviceHandle    = NULL;
            ddcResult       = PSP_E_INVAL_PARAM;
            break;
        }

        /* Open the McBSP peripheral for use */
        LLC_mcbspOpen(
                (LLC_McbspObj *)&(ddcMcbspPort[instanceId].hwInfo),
                ldevid,
                NULL,
                (CSL_Status *)&status
                );

        /*Check if open is successful or not*/
        if (PSP_SOK != status)
        {
            deviceHandle    = NULL;
            ddcResult       = PSP_E_INVAL_PARAM;
            break;
        }


      /* Reset the McBSP Transmitter, receiver and SRGR before configuration */
        ctrlMask =((LLC_MCBSP_CTRL_TX_DISABLE | LLC_MCBSP_CTRL_RX_DISABLE)|
                   (LLC_MCBSP_CTRL_SRG_DISABLE | LLC_MCBSP_CTRL_FSYNC_DISABLE));


        LLC_mcbspHwControl(
                          (LLC_McbspObj *)&(deviceHandle->hwInfo),
                          LLC_MCBSP_CMD_RESET_CONTROL,
                          &ctrlMask
                          );

        deviceHandle->isUserconfigUsed = FALSE;

        if ( NULL != params->mcbspRawCfgPtr )
        {
            /* Configure the McBSP with user supplied parameters */
            PAL_osMemCopy(
                (Ptr)(&(deviceHandle->currentRawconfiguration)),
                (Ptr)(params->mcbspRawCfgPtr),
                sizeof(LLC_McbspConfig)
                );

            LLC_mcbspHwSetupRaw(
                        (LLC_McbspObj *)&(deviceHandle->hwInfo),
                        (LLC_McbspConfig  *)params->mcbspRawCfgPtr
                        );
            deviceHandle->isUserconfigUsed = TRUE;
        }
        else
        {
            /* Configure the McBSP for  default parameters */
            LLC_mcbspHwSetup(
                        (LLC_McbspObj *)&(deviceHandle->hwInfo),
                        (LLC_McbspHwSetup *)&myHwSetup
                        );
        }


        /* Enable the Sample rate generator if required */
        if ( LLC_MCBSP_CTRL_SRG_ENABLE ==
                     ( deviceHandle->enableMask & LLC_MCBSP_CTRL_SRG_ENABLE ))
        {
            ctrlMask = LLC_MCBSP_CTRL_SRG_ENABLE ;
            LLC_mcbspHwControl(
                        (LLC_McbspObj *)&(deviceHandle->hwInfo),
                        LLC_MCBSP_CMD_RESET_CONTROL,
                        &ctrlMask
                        );
        }

        /* Enable the frame Sync generator if required */
        if (0 != (deviceHandle->enableMask & LLC_MCBSP_CTRL_FSYNC_ENABLE))
        {
            ctrlMask |= LLC_MCBSP_CTRL_FSYNC_ENABLE ;
            LLC_mcbspHwControl(
                        (LLC_McbspObj *)&(deviceHandle->hwInfo),
                        LLC_MCBSP_CMD_RESET_CONTROL,
                        &ctrlMask
                        );
        }

        /*init the inuse flag*/
        ddcMcbspPort[instanceId].rxIo.inUse = FALSE;
        ddcMcbspPort[instanceId].txIo.inUse = FALSE;

    }while(falsewhile);

    return (ddcResult);

}

/**
 * \fn      PSP_mcbspDelete
 * \brief   Delete the McBsp created channel
 *
 * This function is called by DSP-BIOS during system shutdown.
 *
 * \param     instanceId   [IN]    Device instance
 * \param     arg         [IN]    optional channel defined parameter
 *
 * \return    PSP_SOK
 */
PSP_Result PSP_mcbspDelete(Uint32 instanceId,Ptr arg)
{
    DDC_mcbspDeviceHandle deviceHandle      = NULL;
    PSP_Result            ddcResult         = PSP_E_INVAL_PARAM;

    if ( instanceId < PSP_MCBSP_NUM_PORTS )
    {
        /* To avoid compiler warning */
        if(NULL != arg)
        {
            arg = arg;
        }
        deviceHandle    = (DDC_mcbspDeviceHandle) &ddcMcbspPort[(instanceId)];

        if (NULL != deviceHandle)
        {
            /* Close and free mcbsp port handle*/
            ddcResult = (PSP_Result)LLC_mcbspClose(
                                        (LLC_McbspObj *)&(deviceHandle->hwInfo)
                                        );

            if (PSP_SOK != ddcResult)
            {
                deviceHandle    = NULL;
                ddcResult       = PSP_E_INVAL_PARAM;
            }

        }

        /*initialise device object structure - make ready for next init*/
        PAL_osMemSet ( (Ptr)&ddcMcbspPort[instanceId], 0,
                       sizeof(DDC_mcbspDeviceHandle));

    }

    return (ddcResult);
}

/**
 * \fn       Uint8  DDC_SpiMstrClockCalculate(uint32 cMcbspSpiMstrFreqcy)
 *
 * \brief    Calculates the CLKDIV values for Mcbsp SPI master mode operation
 *
 *  This function is called in response to a PSP_mcbspOpen call.
 *
 *
 *      What this does:

            Calculates the CLKDIV values for Mcbsp SPI master mode operation
 *
 * \param     cMcbspSpiMstrFreqcy    [IN]  McBsp spi master frequency provided
 *                                         by user as part of Device paremeters.
 *
 * \return
 *      clddiv value.
 */


static Uint8  DDC_SpiMstrClockCalculate(Uint32 ModuleClkFreq, Uint32 McbspSpiMstrFreq)
{

 Uint8 ClkDiv = 0;


 ClkDiv = ((ModuleClkFreq/McbspSpiMstrFreq) - 1 );

 return ClkDiv;

}


/**
 * \fn       PSP_Handle PSP_mcbspOpen(Uint32 instanceId,
                                      PSP_mcbspMode mode,
                                      const PSP_mcbspChanParams *params,
                                      PSP_mcbspAppCallback drvCbk,
                                      PSP_Handle edmaHandle,
                                      Ptr arg )
 *
 * \brief    Opens the McBsp Channel
 *
 *  This function is called in response to a GIO_create call.
 *
 *      Pre-requisites:
 *       -  Valid chanParams structure
 *          Please note expert configuration is needed for structure and its
 *          members
 *       -  Valid device pointer
 *
 *      What this does:
 *       -  Assigns one of the device channels, if available
 *       -  Assigns requested number of serializers, if available
 *       -  Configures eDMA channel to service the McBSP operation,
 *          configures the loopjobs and enables EDMA transfer and starts mcbsp
 *          if channel operates in eDMA mode
 *       -  Hooks in function to service McBSP interrupts
 *       -  Returns configured channel handle to application.
 *
 * \note      Please refer to Appendix A : IOM interface of spru616
 *            DSP/BIOS Driver Developer's Guide for details on
 *            mcbsp_mdCreateChan parameters.
 *
 * \param     instanceId    [IN]     Instance Id of this peripheral
 * \param     mode          [IN]     channel  mode -> input or output
 * \param     params        [IN]     channel parameters from user or from
 *                                   software above this function
 * \param     drvCbk        [IN]     callback function pointer, driver core will
 *                                   call this function after completion of IO
 * \param     edmaHandle    [IN]     Edma driver handle - if the EDMA mode is
 *                                   enabled,the parameter must not be NULL
 * \param     arg           [IN]     callback function Arguments
 *
 * \return
 *      if successful return PSP_Handle else NULL
 */
PSP_Handle PSP_mcbspOpen(
                Uint32               instanceId,
                PSP_mcbspMode        mode,
                const PSP_mcbspChanParams  * params,
                PSP_mcbspAppCallback drvCbk,
                PSP_Handle           edmaHandle,
                Ptr                  arg
                )

{

    DDC_mcbspDeviceHandle    deviceHandle   = NULL;
    DDC_mcbspIoObj          *ioHandle       = NULL;
    Uint32                    i              = 0;
    PAL_Result              retVal          = PSP_SOK;
    Bool                    falsewhile      = FALSE;
    Uint32                  cookie          = 0;
    Uint32                  mask            = 0;
    EDMA3_DRV_TrigMode      trigMode        = EDMA3_DRV_TRIG_MODE_EVENT;
    PAL_OsSemAttrs          semAttrs        = PAL_OSSEM_ATTRS;
    Uint32                  ctrlMask        = 0;
    Uint8                   ClkDiv          = 0;

    do
    {
        falsewhile      = FALSE;

        /*
        validate parameters
        */
        if (((NULL == params) || (NULL == drvCbk)))
        {
            ioHandle = NULL;
            break;
        }
        if(NULL == edmaHandle)
        {
            ioHandle = NULL;
            break;
        }
        deviceHandle    = (DDC_mcbspDeviceHandle )&ddcMcbspPort[instanceId];


        if ((NULL == deviceHandle)|| (NULL == params))
        {
            ioHandle = NULL;
            break;
        }

        if(PSP_MCBSP_RX == mode )
        {
            ioHandle = (DDC_mcbspIoObj *)&deviceHandle->rxIo;
            ioHandle->mode = PSP_MCBSP_INPUT;
        }
        else
        {
            ioHandle = (DDC_mcbspIoObj *)&deviceHandle->txIo;
            ioHandle->mode = PSP_MCBSP_OUTPUT;
        }

        if (NULL == ioHandle)
        {
            /*
            Control should not come here because the IO
            objects are instantiated within ddcMcbspPort structure
            */
            break;
        }

        if ( PSP_MCBSP_ALLOCATED == ioHandle->status)
        {
            ioHandle = NULL;
            break;
        }


        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

        if (TRUE == ioHandle->inUse)
        {
            PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
            ioHandle = NULL;
            break;
        }

        ioHandle->inUse = TRUE;

        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);


         /*
         * set back pointer of channel - for references
         * set callback function and argument - this function will be called
         * back after completion of IO
         */
        ioHandle->deviceHandle  = deviceHandle;
        ioHandle->cbFxn         = drvCbk;

        if(NULL != arg)
        {
            ioHandle->cbArg = arg;
        }

        /*
        This will be used in EDMA mode to populate the paramsets in PINGPONG
        */
        ioHandle->nextLinkParamSetToBeUpdated = 0;

        /*
        Assume for writing 32bit with data value = 0, localloopjob data ,
        on the serialiser - ofcourse actual bytes written for each sync event
        is decided by param->ordWidth
        */
        ioHandle->loopJobLength = 4u;

        ioHandle->dataPacket    = NULL;
        ioHandle->flushPacket   = NULL;
        ioHandle->flush         = FALSE;

        /*Initialize Mute parameter*/
        ioHandle->bMuteON = FALSE;


        ioHandle->nextFlag = FALSE;
        ioHandle->isTempPacketValid = FALSE;
        ioHandle->noOfTdmChans = params->noOfTdmChans;

        /*
        Counter that counts outstanding requests of this channel
        */
        ioHandle->submitCount   = 0;
        /*
        * Global error callback registered to channel
        */
        ioHandle->gblErrCbk = params->gblCbk;
        /*  Initialize the SPI buffer */
        for(i=0; i< PSP_SPI_TRANSIVE_BUFF_LEN; i++)
          SpiTransiveBuff[i] = 0x00;

        if (PSP_MCBSP_WORDLEN_32 < params->wordWidth)
        {
            ioHandle = NULL;
            break;
        }
        else
        {
            if (PSP_MCBSP_INPUT == ioHandle->mode)
            {
                LLC_mcbspRcvConfigWordLength(
                            (LLC_McbspHandle)&(ioHandle->deviceHandle->hwInfo),
                            params->wordWidth
                            );
            }
            else
            {
                LLC_mcbspXmtConfigWordLength(
                            (LLC_McbspHandle)&(ioHandle->deviceHandle->hwInfo),
                            params->wordWidth
                            );
            }
        }
        /*
        We need to have infomation on what application intend to do.
        for e.g. if application is sending 12bit data and formats it in
        such a way the it in stream it occupies 2 bytes , this members value
        should be 16 (PSP_MCBSP_WORDLEN_16). By defaut we assume that app has
        formated the data byte stream for 4bytes for each serialser for each
        sync event/int from mcbsp.
        Please note that for burst mode we have 1 sync event/int from mcbsp for
        both L and R (in general all slots) where as we have each sync
        event /Int from mcbsp for L and R (in gerneral all slots) seperately
        */
        ioHandle->roundedWordWidth = PSP_MCBSP_WORDLEN_32;

        /*
        we have 3 options for ioHandle->roundedWordWidth variable 32,16 and 8
        so we start from 32 (bit 5) and check if bit5 , 4, or 3 is set
        if  param->wordWidth is invalid - we assume the default of 32bit width
        The following logic translates the intermediate bit widths to what
        this driver wants. It assumes that always the data streams is alligned to
        8 ,16, 32 bits ( rounded to ceil)
        */

        if(PSP_MCBSP_WORDLEN_32 > params->wordWidth)
        {
            mask = 0x20u;

            for (i=0; i<3u;i++)
            {
                if( mask == (mask &  params->wordWidth))
                {
                    ioHandle->roundedWordWidth = (Uint16) mask;

                    if ( params->wordWidth > ioHandle->roundedWordWidth  )
                    {
                        ioHandle->roundedWordWidth= \
                                (Uint16)(ioHandle->roundedWordWidth << 1);
                    }
                    break;
                }
                mask= (Uint32)(mask >> 1);
            }
        }

        /*the driver has been configured for DSP (BURST mode)*/
        ioHandle->roundedWordWidth = (Uint16)(ioHandle->roundedWordWidth >> 3);

        /* store the EDMA3 module handle*/
        ioHandle->edmaHandle   = edmaHandle ;

        if (PSP_MCBSP_INPUT == ioHandle->mode)
        {
            /* Receive mode */

            /*Assign the Channel ID and TCC*/
            ioHandle->xferChan     = deviceHandle->hwInfo.edmaRxEventNum;
            ioHandle->tcc          = deviceHandle->hwInfo.edmaRxEventNum;
            ioHandle->edmaCallback = (EDMA3_RM_TccCallback)&ddc_edmaMcbspCallback;
            ioHandle->cpuEventNum  = deviceHandle->hwInfo.cpuRxEventNum;

        }
        else
        {
            /* Transmit mode */

            /*Assign the Channel ID and TCC*/
            ioHandle->xferChan     = deviceHandle->hwInfo.edmaTxEventNum;
            ioHandle->tcc          = deviceHandle->hwInfo.edmaTxEventNum;
            ioHandle->edmaCallback = (EDMA3_RM_TccCallback)&ddc_edmaMcbspCallback;
            ioHandle->cpuEventNum  = deviceHandle->hwInfo.cpuTxEventNum;

        }
        /*
        *Configure the loop job for the user specified buffer if given
        */
        if (NULL == params->userLoopJobBuffer)
        {
            if (PSP_MCBSP_INPUT == ioHandle->mode)
            {
                ioHandle->loopJobBuffer = &loopDstBuf[(deviceHandle->InstNum)];
            }
            else
            {
                ioHandle->loopJobBuffer = &loopSrcBuf[(deviceHandle->InstNum)];
            }

            /*Length of Uint32 per serialiser - this is what we have allocated ;-)*/
            ioHandle->loopJobLength = 4u;
        }
        else
        {
            /*
            Application has preference on the loopjob buffer and
            lets use that
            */
            ioHandle->loopJobBuffer = params->userLoopJobBuffer;
            ioHandle->loopJobLength = params->userLoopJobLength;

            if ( ioHandle->roundedWordWidth > params->userLoopJobLength )
            {
                /*
                not enough loopjob buffer has been provided
                we should have aleast loopbuffer for 1 sync event
                */
                ioHandle = NULL;
                break;
            }
        }
        /* make node */
        PAL_OSLIST_MKNODE(&ioHandle->queueFreeList);
        PAL_OSLIST_MKNODE(&ioHandle->queueReqList);
        PAL_OSLIST_MKNODE(&ioHandle->queueFloatingList);
        i = 0;
        /*
        link in all the allocatated IOPs into the FreeQueue list
        */
        while (i < PSP_MCBSP_MAX_REQ)
        {
            PAL_osListEnqueue(
                            &ioHandle->queueFreeList,
                            (Ptr)&ioHandle->ioRequests[i]
                            );
            i++;
        }
        /*
         * This semaphore will be used to pend to all TX requests are
         * done - during a flush request
         */
        if ( PAL_SOK != PAL_osSemCreate(
                                    NULL,
                                    0,
                                    &semAttrs,
                                    (PAL_OsSemHandle)&ioHandle->syncSem)
                                    )
        {
            retVal = PSP_E_RESOURCES;
            PSP_mcbspClose((PSP_Handle)ioHandle,NULL);
            break;
        }
        if(PSP_McBSP_McBSP_SPISLAVE == deviceHandle->currentMode)
        {

           /*Implement start up sequence of SPI mode*/

           /*1. Place the transmitter and receiver in reset:
           a. To reset the transmitter, clear the transmitter reset (XRST) bit
                 in SPCR to 0.
           b. To reset the receiver, clear the receiver reset (RRST) bit
                 in SPCR to 0.
           2. To reset the sample rate generator, clear the sample rate
                generator reset (GRST) bit in SPCR to 0
           This is already done by Hwcfg in Devparams */

           /*3. Program the appropriate McBSP registers to configure the
                    McBSP for proper operation as an SPI master or an SPI slave:

           3a)(SPCR) CLKSTP = 2h or 3h The clock stop mode
                    (without or with a clock delay) is    selected.*/
           deviceHandle->hwInfo.regs->SPCR |=2<<11;


           /*3b)Pin control register (PCR) FSXM = 0 The FSX pin is an input pin,
                 so that it can be driven by the    SPI master.*/
           deviceHandle->hwInfo.regs->PCR &= ~(1<<11);

           /*3c CLKXM = 0 The CLKX pin is an input pin,
                so that it can be driven by the SPI master.
                Because CLKSTP is equal to 2h or 3h, CLKR is
                driven internally by CLKX.*/
           deviceHandle->hwInfo.regs->PCR &= ~(1<<9);

           /*3d SCLKME=0 SCLKME bit is used in conjunction with the CLKSM bit in
           SRGR to determine the source for the input clock. The clock
           generated by the sample rate generator (CLKG) is derived
           from the McBSP internal input clock. (The sample rate
           generator is used to synchronize the McBSP logic with the
           externally-generated master clock.)*/
           deviceHandle->hwInfo.regs->PCR &= ~(1<<7);

           /*3e FSXP = 1 The FSX pin is active low.*/
           deviceHandle->hwInfo.regs->PCR |= (1<<3);

           /*3f CLKXP = 0 or 1 The polarity of CLKX as seen on the CLKX pin is
            positive    (CLKXP = 0) or negative (CLKXP = 1).*/
//           deviceHandle->hwInfo.regs->PCR &= ~(1<<1);
//           deviceHandle->hwInfo.regs->PCR |= (1<<1);

           /*3g CLKRP = 0 or 1 The polarity of CLKR as seen on the CLKR pin is
            positive (CLKRP = 0) or negative (CLKRP = 1).*/
//           deviceHandle->hwInfo.regs->PCR &= ~1;


           /*3h Sample rate generator register (SRGR) CLKSM = 1 CLKSM bit is
            used in conjunction with the SCLKME bit in PCR to determine the
            source for the input clock. The clock generated by the sample rate
            generator (CLKG) is derived from the McBSP internal input clock.
            (The sample rate generator is used to synchronize the McBSP logic
            with the externally-generated master clock.)*/
           deviceHandle->hwInfo.regs->SRGR |= 1<<29;

           /*3i CLKGDV = 1 The sample rate generator divides the McBSP internal
             input  clock by 2 before generating CLKG.*/
           deviceHandle->hwInfo.regs->SRGR &= ~0xFF;
           deviceHandle->hwInfo.regs->SRGR |= 1;

           /*3j Transmit control register (XCR) XDATDLY = 0 This bit must be 0
                for SPI slave operation.Receive control register (RCR)
                RDATDLY = 0 This bit must be 0 for SPI slave operation.*/
           deviceHandle->hwInfo.regs->XCR &= ~(3<<16);
           deviceHandle->hwInfo.regs->RCR &= ~(3<<16);

            /*set to generate interupts on XRDY and RRDY*/
           deviceHandle->hwInfo.regs->SPCR &=~((3<<20)|(3<<4));


           if(deviceHandle->opMode == PSP_OPMODE_INTERRUPT)
           {
                   /* Start the McBSP */
                if ( PSP_MCBSP_OUTPUT == ioHandle->mode )

                {
                   _DDA_McBSP_RegisterInt(ioHandle,
                                          ioHandle->cpuEventNum,
                                          deviceHandle->hwInfo.cpuRxEventNum,
                                          &ddc_mcbspRxISR);
                   _DDA_McBSP_RegisterInt(ioHandle,
                                          ioHandle->cpuEventNum,
                                          deviceHandle->hwInfo.cpuTxEventNum,
                                          &ddc_mcbspTxISR);
                }

           }
           else if(deviceHandle->opMode == PSP_OPMODE_DMAINTERRUPT)
           {
               /* Start the McBSP */
                if ( PSP_MCBSP_OUTPUT == ioHandle->mode )

                {
                    DDC_McBspSpiSetupEdmaInit(ioHandle);
                }
            }

        }
        else if(PSP_McBSP_McBSP_SPIMASTER == deviceHandle->currentMode)
        {



               /*Implement start up sequence of SPI mode*/

               /*1. Place the transmitter and receiver in reset:
               a. To reset the transmitter, clear the transmitter reset (XRST)
                    bit in SPCR to 0.
               b. To reset the receiver, clear the receiver reset (RRST)
                    bit in SPCR to 0.
               2. To reset the sample rate generator, clear the sample rate
                    generator reset (GRST) bit in SPCR to 0
                    This is already done by Hwcfg in Devparams*/

               /*3. Program the appropriate McBSP registers to configure the
                    McBSP for proper operation as an SPI master or an SPI slave:

               3a)(SPCR) CLKSTP = 2h or 3h The clock stop mode
                    (without or with a clock delay) is selected.*/
               deviceHandle->hwInfo.regs->SPCR |=2<<11;


               /*3b)Pin control register (PCR) FSXM = 0
                    The FSX pin is an input pin, so that it can be driven by the
                    SPI master.*/
               deviceHandle->hwInfo.regs->PCR |= (1<<11);
               /*3c CLKXM = 0 The CLKX pin is an Output pin, so that it can be
                    driven by the SPI master. Because CLKSTP is equal to
                    2h or 3h, CLKR is  driven internally by CLKX.*/
               deviceHandle->hwInfo.regs->PCR |= (1<<9);
               /*3d SCLKME = 0 SCLKME bit is used in conjunction with the CLKSM
                bit in SRGR to determine the source for the input clock. The
                clock generated by the sample rate generator (CLKG) is derived
                from the McBSP internal input clock. (The sample rate
                generator is used to synchronize the McBSP logic with the
                externally-generated master clock.)*/
               deviceHandle->hwInfo.regs->PCR &= ~(1<<7);

               /*3e FSXP = 1 The FSX pin is active low.*/
               deviceHandle->hwInfo.regs->PCR |= (1<<3);

               /*3f CLKXP = 0 or 1 The polarity of CLKX as seen on the CLKX pin
                is positive (CLKXP = 0) or negative (CLKXP = 1).*/
//               deviceHandle->hwInfo.regs->PCR &= ~(1<<1);

               /*3g CLKRP = 0 or 1 The polarity of CLKR as seen on the CLKR pin
                 is positive   (CLKRP = 0) or negative (CLKRP = 1).*/
//               deviceHandle->hwInfo.regs->PCR &= ~1;


              /* Before Programming the SRG Put the SRG in reset mode */

                ctrlMask = LLC_MCBSP_CTRL_SRG_DISABLE ;
                LLC_mcbspHwControl(
                            (LLC_McbspObj *)&(deviceHandle->hwInfo),
                            LLC_MCBSP_CMD_RESET_CONTROL,
                            &ctrlMask
                            );

               /*3h Sample rate generator register (SRGR) CLKSM = 1 CLKSM bit is used in conjunction with the SCLKME bit in
               PCR to determine the source for the input clock. The clock
               generated by the sample rate generator (CLKG) is derived
               from the McBSP internal input clock. (The sample rate
               generator is used to synchronize the McBSP logic with the
               externally-generated master clock.)*/
               deviceHandle->hwInfo.regs->SRGR |= 1<<29;

               /*3i The transmitter drives a frame-sync pulse on the FSX pin
            every time data is transferred from DXR to XSR. */

               deviceHandle->hwInfo.regs->SRGR &=~(1<<28);


               /*3j CLKGDV = 0x00 to 0xFF1 The sample rate generator divides
                    the McBSP internal input clock by x values which is
                    calculated using SPI master clk speed specifed by the user
                    before generating CLKG.*/

               deviceHandle->hwInfo.regs->SRGR &= ~(0xFF);
               ClkDiv = (Uint8)DDC_SpiMstrClockCalculate(deviceHandle->ModuleClkFreq,
	               						deviceHandle->McbspSpiMstrFreq);
               deviceHandle->hwInfo.regs->SRGR |= ClkDiv;

               /* After Prograimming, take out the SRG from reset */

               ctrlMask = LLC_MCBSP_CTRL_SRG_ENABLE ;
               LLC_mcbspHwControl(
                        (LLC_McbspObj *)&(deviceHandle->hwInfo),
                        LLC_MCBSP_CMD_RESET_CONTROL,
                        &ctrlMask
                        );

               /*3k Transmit control register (XCR) XDATDLY = 1 This bit must
                    be 1 for SPI Master operation.Receive control register (RCR)
                    RDATDLY = 1 This bit must be 1 for SPI master operation.*/
               deviceHandle->hwInfo.regs->XCR &= ~(3<<16);
               deviceHandle->hwInfo.regs->XCR |= (1<<16);
               deviceHandle->hwInfo.regs->RCR &= ~(3<<16);
               deviceHandle->hwInfo.regs->RCR |= (1<<16);



                /*set to generate interupts on XRDY and RRDY*/
               deviceHandle->hwInfo.regs->SPCR &=~((3<<20)|(3<<4));


               /*
               4. To release the sample rate generator from reset, set the
                  sample rate generator reset (GRST) bit in SPCR to 1. Make
                  sure that during the write to SPCR, you only modify the
                  GRST bit; otherwise, you will modify the McBSP configuration
                  you selected in the previous step.*/
              // deviceHandle->hwInfo.regs->SPCR |= 1<<21;

               /*
               5. After the sample rate generator is released from reset,
               wait two sample rate generator clock periods for
               the McBSP logic to stabilize.*/
               //PAL_osWaitMsecs(2);

               /* TBD
               6. Enable the transmitter and receiver:
               a. If the CPU services the McBSP transmit and receive buffers,
               then you can immediately enable the transmitter
               (XRST = 1 in SPCR) and enable the receiver (RRST = 1 in SPCR).*/
               //deviceHandle->hwInfo.regs->SPCR |= 0x101;

               /* b. If the DMA controller services the McBSP transmit and
               receive buffers, then you must first configure the DMA
               controller (this includes enabling the channels that service the
               McBSP buffers).When the DMA controller is ready, set XRST = 1
               and RRST = 1 - NOT Applicable.*/

               /* c. In either case, make sure you only change the XRST and
               RRST bits when you write to SPCR; otherwise, you will modify the
               bit settings you selected earlier in this procedure.*/

               /*
               7. After the transmitter and receiver are released from reset,
               wait two sample rate generator clock periods
               for the McBSP logic to stabilize.*/
               //PAL_osWaitMsecs(2);

               /*
               8. If necessary, enable the frame-sync logic of the sample rate
               generator. After the required data acquisition setup is done
               (DXR is loaded with data), set the FRST bit in SPCR to 1 if an
               internally generated frame-sync pulse is required (that is, if
               the McBSP is the SPI master).- NOT applicable*/


           if(deviceHandle->opMode == PSP_OPMODE_INTERRUPT)
           {
                   /* Start the McBSP */
                if ( PSP_MCBSP_OUTPUT == ioHandle->mode )

                {
                   _DDA_McBSP_RegisterInt(ioHandle,
                                          ioHandle->cpuEventNum,
                                          deviceHandle->hwInfo.cpuRxEventNum,
                                          &ddc_mcbspRxISR);
                   _DDA_McBSP_RegisterInt(ioHandle,
                                          ioHandle->cpuEventNum,
                                          deviceHandle->hwInfo.cpuTxEventNum,
                                          &ddc_mcbspTxISR);
                }

           }

           else if(deviceHandle->opMode == PSP_OPMODE_DMAINTERRUPT)
           {
                   /* Start the McBSP */
                 if ( PSP_MCBSP_OUTPUT == ioHandle->mode )
                 {
                    DDC_McBspSpiSetupEdmaInit(ioHandle);
                 }

            }

        }
        else
        {
            /*
            configure EDMA for this driver
            */
            retVal = ddc_setupEDMADuringInit(ioHandle);

            if ( PSP_SOK != retVal)
            {
                PSP_mcbspClose((PSP_Handle)ioHandle,NULL);
                ioHandle = NULL;
                break;
            }

            ioHandle->loopjobUpdatedinParamset = TRUE;


#ifdef GBL_INT
            _DDA_McBSP_RegisterInt(ioHandle,
                                   ioHandle->cpuEventNum,
                                   0,
                                   &ddc_mcbspISR);
#endif


            /* Start the McBSP */
            if ( PSP_MCBSP_INPUT == ioHandle->mode )

            {
                ctrlMask = ioHandle->deviceHandle->enableMask |
                           LLC_MCBSP_CTRL_RX_ENABLE ;
                LLC_mcbspHwControl(
                            (LLC_McbspObj *)&(ioHandle->deviceHandle->hwInfo),
                            LLC_MCBSP_CMD_RESET_CONTROL,
                            (void *)&ctrlMask
                            );
            }
            else
            {
                ctrlMask =ioHandle->deviceHandle->enableMask |
                          LLC_MCBSP_CTRL_TX_ENABLE ;
                LLC_mcbspHwControl(
                            (LLC_McbspObj *)&(ioHandle->deviceHandle->hwInfo),
                            LLC_MCBSP_CMD_RESET_CONTROL,
                            (void *)&ctrlMask
                            );
            }

			/* Enable the DMA to be ready to handle the Mcbsp requests */
            EDMA3_DRV_enableTransfer(ioHandle->edmaHandle,
                                     ioHandle->xferChan,
                                     trigMode);

            /* enter critical section */
            PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

            /* Mark channel status as allocated.*/
            ioHandle->status = PSP_MCBSP_ALLOCATED;

            /* Leave critical section */
            PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
        }

        /*
        * State m/c stop status
        */
        deviceHandle->stopSmFsXmt = FALSE;
        deviceHandle->stopSmFsRcv = FALSE;
        ioHandle->inUse = TRUE;


    }while(falsewhile);

    return ((PSP_Handle)ioHandle);
}

/*!
 *  \fn PSP_Result PSP_mcbspClose(PSP_Handle hMcBSPChan,Ptr arg)
 *  \brief frees a channel and all it's associated resources.
 *
 *  PSP_mcbspClose() frees the EDMA resources including EDMA
 *  master channel and link PaRAM entries held by the channel
 *
 *  \param  hMcBSPChan  [IN]    PSP_Handle
 *  \param  arg         [IN]    Misc Arg
 *
 *  \return PSP_SOK
 */
PSP_Result PSP_mcbspClose(PSP_Handle hMcBSPChan,Ptr arg)
{
    PSP_Result            retVal            = PSP_SOK;
    DDC_mcbspIoHandle     ioHandle          = NULL;
    DDC_mcbspDeviceHandle deviceHandle      = NULL;
    Bool                  falsewhile        = FALSE;
    Uint8                 linkCnt           = 0;

    /* To avoid compiler warning */
    if(NULL == arg)
    {
        arg = arg;
    }
    do
    {
        if(NULL == hMcBSPChan)
        {
            retVal = PSP_E_RESOURCES;
            break;
        }
        ioHandle = (DDC_mcbspIoHandle)hMcBSPChan;
        if(NULL == ioHandle)
        {
            retVal = PSP_E_RESOURCES;
            break;
        }
        if(NULL == ioHandle->deviceHandle)
        {
            retVal = PSP_E_RESOURCES;
            break;
        }
        deviceHandle = ioHandle->deviceHandle;
        if(NULL == deviceHandle)
        {
            retVal = PSP_E_RESOURCES;
            break;
        }

        falsewhile      = FALSE;
        /* Delete the semaphore created at driver open time. */
        if (PSP_SOK != PAL_osSemDelete(ioHandle->syncSem))
        {
            retVal = PSP_E_RESOURCES;
            break;
        }

        /*
        * restore global serializer state.
        */
        ddc_abortReset(ioHandle);

        /*
        * Diable the current transfer to make sure that there is no
        * running EDMA transfer taking place
        */
        if ( PSP_SOK != (EDMA3_DRV_disableTransfer (
                                                ioHandle->edmaHandle,
                                                ioHandle->xferChan,
                                                EDMA3_DRV_TRIG_MODE_EVENT))
                                                )
        {
            retVal = PSP_E_INVAL_STATE;
            break;
        }

        /* Disable current EDMA transfer */
        EDMA3_DRV_disableTransfer (
                        ioHandle->edmaHandle,
                        ioHandle->xferChan,
                        EDMA3_DRV_TRIG_MODE_EVENT
                        );

        if (PSP_MCBSP_OUTPUT == ioHandle->mode)
        {
            /* Poll for TXEMPTY */
            while (CSL_FEXT(ioHandle->deviceHandle->hwInfo.regs->SPCR,
                            MCBSP_SPCR_XEMPTY))
            {
            }
        }

        /* Reset McBSP */
        PSP_mcbspIoctl(ioHandle,PSP_CTRL_McBSP_STOP, NULL, NULL);

        /* Free Edma channels*/
        if ( PSP_SOK != (EDMA3_DRV_freeChannel(
                                    ioHandle->edmaHandle,
                                    ioHandle->xferChan))
                                    )
        {
            retVal = PSP_E_INVAL_STATE;
            break;
        }

        /* free the EDMA PaRAM entries used for Ping pong buffering */
        for (linkCnt = 0; linkCnt < MAXLINKCNT; linkCnt++)
        {
            EDMA3_DRV_freeChannel(
                                ioHandle->edmaHandle,
                                ioHandle->pramTbl[linkCnt]
                                );
        }

        /*
        * Mark channel as ready to be allocated again.
        */
        ioHandle->status              = PSP_MCBSP_UNALLOCATED;
        /* Set the inUse Parameter as FALSE */
        ioHandle->inUse     = FALSE;
        /* Set the Muted state to FALSE */
        ioHandle->bMuteON   = FALSE;
        /* Set the Paused state to FALSE */
        ioHandle->paused    = FALSE;

#ifdef GBL_INT
        /*Unregister the interrupts registered for the errors/data ready*/
        _DDA_McBSP_UnRegisterInt(
                        deviceHandle->InstNum,
                        (Uint16)ioHandle->cpuEventNum,
                        (Int)ioHandle->mode
                        );
#endif
    }while(falsewhile);

    return retVal;
}

/**
 *  \fn PSP_Result PSP_mcbspWrite(PSP_Handle handle,PSP_mcbspReq *ioReq,Ptr arg)
 *
 *  \brief  Write a packet of data to the output channel of the driver
 *
 *  The application has to call PSP_mcbspWrite() with handle to the
 *  o/p channel and the pointer to ioRequest structure which takes a set of
 *  information need for the IO. The I/O req structure takes a (optional)
 *  callback function pointer which will be called after the completion of IO
 *
 *  \param  handle        [IN]   Handle to the McBSP channel
 *  \param  ioReq         [IN]   io request information structure
 *  \Param  arg           [IN]   optional - for expanasion
 *
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_mcbspWrite (PSP_Handle handle, PSP_mcbspReq *ioReq,Ptr arg)
{

    DDC_mcbspIoHandle   ioHandle    = NULL;
    PSP_Result          result      = PSP_SOK;

    /* To avoid compiler warning */
    if(NULL != arg)
    {
        arg = arg;
    }

    if (NULL == handle)
    {
        result = PSP_E_INVAL_PARAM;
    }
    else
    {
        ioHandle    = (DDC_mcbspIoHandle)handle;
    }

    if (NULL == ioReq)
    {
        result = PSP_E_INVAL_PARAM;
    }

    if(PSP_MCBSP_OUTPUT != ioHandle->mode)
    {
        result = PSP_E_INVAL_PARAM;
    }

    if (PSP_SOK == result)
    {
        result = ddc_mcbspSubmitReq(ioHandle,ioReq);
    }

    return (result);
}


/** \fn PSP_Result PSP_mcbspRead(PSP_Handle handle, PSP_mcbspReq *ioReq,Ptr arg)
 *  \brief Read a buffer of data from the input channel of the driver
 *
 *  The application has to call PSP_mcbspRead() with handle to the
 *  i/p channel and the pointer to buffer to where the data shall be stored
 *  and the number of bytes to read. The has to specify the call back function
 *  the shall be called when the I/O is complete
 *
 *
 *  \param  handle  [IN]     Handle to the McBSP channel
 *  \param  ioReq   [IN]     request structure
 *  \param  arg     [IN]     Miscellenous Argument
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_mcbspRead (PSP_Handle handle, PSP_mcbspReq *ioReq,Ptr arg)
{
    DDC_mcbspIoHandle   ioHandle    = NULL;
    PSP_Result          result      = PSP_SOK;

    /* To avoid compiler warning*/
    if(NULL != arg)
    {
        arg = arg;
    }

    if ((NULL == handle)||(NULL==ioReq))
    {
        result = PSP_E_INVAL_PARAM;
    }
    else
    {
        ioHandle    = (DDC_mcbspIoHandle)handle;
    }

    if(PSP_MCBSP_INPUT != ioHandle->mode)
    {
        result = PSP_E_INVAL_PARAM;
    }

    if (PSP_SOK == result)
    {
        result = ddc_mcbspSubmitReq(ioHandle,ioReq);
    }
    return (result);
}


/** \fn PSP_Result PSP_mcbspFlushRequests(PSP_Handle handle, Ptr arg)
 *
 *  \brief Flush the data in the McBSP channel
 *
 *  Flushes the pending I/O packets in the specified channel
 *  and returns the notifications of the pending I/O packets
 *  to the application
 *
 *  \param  handle  [IN]       Handle to the McBSP Driver
 *  \param  arg     [IN]       Miscellenous Argument
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_mcbspFlushRequests(PSP_Handle handle, Ptr arg)
{

    DDC_mcbspIoHandle      ioHandle         = NULL;
    PAL_Result              result          = PSP_SOK;
    DDC_mcbspDeviceHandle   deviceHandle    = NULL;

    /* To avoid compiler warning */
    if(NULL != arg)
    {
        arg = arg;
    }
    if(NULL!=handle)
    {
        ioHandle    = (DDC_mcbspIoHandle )handle;
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    if (NULL == ioHandle)
    {
        result = PSP_E_INVAL_PARAM;
    }
    else if (NULL==ioHandle->deviceHandle)
    {
        result = PSP_E_INVAL_PARAM;
    }
    else
    {
        deviceHandle = ioHandle->deviceHandle;

        if(NULL == deviceHandle)
        {
            result = PSP_E_INVAL_PARAM;
        }
        else
        {
            if (PSP_SOK == result)
            {
                if (PSP_MCBSP_INPUT == ioHandle->mode)
                {
                    /*
                     * Flush is called for INPUT channel,
                     * that is why we have to enable flush flag.
                     */
                    ioHandle->flush = TRUE;

                    PSP_mcbspAbortRequests(ioHandle, NULL);

                    /* Flush flag is disable */
                    ioHandle->flush = FALSE;

                }
                else
                {
                    if (0 == ioHandle->submitCount)
                    {
                        result = PSP_SOK;
                    }
                    else
                    {
                        ioHandle->flush = TRUE;
                        PAL_osSemTake(ioHandle->syncSem, PAL_OSSEM_NO_TIMEOUT);
                    }
                }
            }
        }
    }
    return (result);
}


/** \fn  PSP_Result PSP_mcbspAbortRequests(PSP_Handle handle, Ptr arg)
 *
 *  \brief Abort the McBSP channel operation
 *
 *   Abort the input or output channel operation of the McBSP driver
 *   and flush all pending I/O packets. Driver sends the
 *   notifications to application for all pending I/O packets
 *   The application can start freshly by submitting new I/O requests.
 *
 *  \param  handle  [IN]     Handle to the McBSP channel
 *  \param  arg     [IN]     Miscellenous Argument
 *  \return PSP_SOK or PSP Error code
 */

PSP_Result PSP_mcbspAbortRequests(PSP_Handle handle, Ptr arg)
{

    DDC_mcbspIoHandle           ioHandle        = NULL;
    PAL_Result                  result          = PSP_SOK ;
    DDC_mcbspDeviceHandle       deviceHandle    = NULL;

    /* To avoid compiler warning */
    if(NULL != arg)
    {
        arg = arg;
    }
    if (NULL == handle)
    {
        result = PSP_E_INVAL_PARAM;
    }
    else
    {
        ioHandle    = (DDC_mcbspIoHandle )handle;
        if(NULL == ioHandle)
        {
            result = PSP_E_INVAL_PARAM;
        }
        else if(NULL== ioHandle->deviceHandle)
        {
            result = PSP_E_INVAL_PARAM;
        }
        else
        {
            deviceHandle    = ioHandle->deviceHandle;
        }
        if(NULL == deviceHandle)
        {
            result = PSP_E_INVAL_PARAM;
        }
        else
        {
            if (PSP_SOK == result)
            {
                if (TRUE == ioHandle->inUse)
                {
                    /* Abort both iohandlenels*/
                    if (PSP_MCBSP_INPUT == ioHandle->mode)
                    {
                        ddc_abortReset((DDC_mcbspIoHandle)&deviceHandle->rxIo);
                    }
                    else
                    {
                        ddc_abortReset((DDC_mcbspIoHandle)&deviceHandle->txIo);
                    }
                }

            }
        }
    }
    return (result);
}


static int DDC_McBspSpiTransfer(DDC_mcbspIoHandle ioHandle,
                                DDC_mcbspIOP *ioPacket,
                                Ptr param )
{

    Int  retVal  = PAL_SOK;


    if( (NULL == ioHandle) || (NULL == ioPacket) )
    {
        retVal = (PSP_E_INVAL_PARAM);
    }
    else
    {

        param = param;   /* To remove the warning */
        ioHandle->pspRequest.actualSize  = ioPacket->actualSize;
        ioHandle->pspRequest.addr        = ioPacket->addr;
        ioHandle->pspRequest.appPtr      = ioPacket->appData;
        ioHandle->pspRequest.size        = ioPacket->size;
        ioHandle->pspRequest.status      = ioPacket->status;
        ioHandle->pspRequest.cbFxn       = ioPacket->appCbk;

        ioHandle->rxBytesIndex = 0;
        ioHandle->txBytesIndex = 0;


        if(ioHandle->deviceHandle->opMode == PSP_OPMODE_DMAINTERRUPT)
        {
            /* Enable the EDMA here */

            if ( PSP_MCBSP_OUTPUT == ioHandle->mode )
            {
               PAL_osEvtDisable (ioHandle->deviceHandle->hwInfo.cpuTxEventNum);
               PAL_osEvtDisable (ioHandle->deviceHandle->hwInfo.cpuRxEventNum);

               retVal = DDC_SPIEdmaSetParamas(ioHandle);
               if(retVal == PSP_SOK)
               {
                EDMA3_DRV_enableTransfer(ioHandle->edmaHandle,
                                         ioHandle->deviceHandle->hwInfo.edmaTxEventNum,
                                         EDMA3_DRV_TRIG_MODE_EVENT);
                EDMA3_DRV_enableTransfer(ioHandle->edmaHandle,
                                         ioHandle->deviceHandle->hwInfo.edmaRxEventNum,
                                         EDMA3_DRV_TRIG_MODE_EVENT);
               }
            }

         }
            /*
              4. To release the sample rate generator from reset, set the sample
                 rate generator reset (GRST) bit in SPCR to 1.
                 Make sure that during the write to SPCR, you only modify the
                 GRST bit; otherwise, you will modify the McBSP configuration
                 you selected in the previous step.*/
            ioHandle->deviceHandle->hwInfo.regs->SPCR |= 1<<22;

            /*
            5.  After the sample rate generator is released from reset,
                wait two sample rate generator clock periods for
                the McBSP logic to stabilize.*/
            PAL_osWaitMsecs(2);

         /*6.   Enable the transmitter and receiver:
                a. If the CPU services the McBSP transmit and receive buffers,
                then you can immediately enable the transmitter (XRST = 1 in
                SPCR) and enable the receiver (RRST = 1 in SPCR).*/
          ioHandle->deviceHandle->hwInfo.regs->SPCR |= 0x10001;

          /* b. If the DMA controller services the McBSP transmit and receive
                buffers, then you must first configure the DMA controller
                (this includes enabling the channels that service the McBSP
                buffers).When the DMA controller is ready, set XRST = 1 and
                RRST = 1 - NOT Applicable.*/

          /* c. In either case, make sure you only change the XRST and RRST bits
                whn you write to SPCR; otherwise, you will modify the bit settings you
                selected earlier in this procedure.*/

          /*
          7.    After the transmitter and receiver are released from reset, wait
                two sample rate generator clock periods for the McBSP logic
                to stabilize.*/
          PAL_osWaitMsecs(2);
          retVal = PSP_SINPROGRESS;
    }

    return retVal;

}


/*!
 *  \fn static PAL_Result ddc_mcbspSubmitReq( DDC_mcbspIoHandle ioHandle,
                                              PSP_mcbspReq *pspRequestp)
 *
 *  \brief Submit a I/O packet to a channel for processing
 *
 *  The upper layer calls this function to cause the mini-driver
 *  to process the McBSP_DDC_Packet.If the cmd code is IOM_READ or IOM_WRITE,
 *  this function should queue the packet on the pending list. If the
 *  cmd code is IOM_ABORT, this function should abort both read and
 *  write packets. If the cmd code is IOM_FLUSH, this function should
 *  complete queued writes, but abort queued reads.
 *
 *
 *  \param  ioHandle    [IN] handle to the chanel which processes the IOM_packet
 *  \param  pspRequestp [IN] pointer to McBSP_DDC_Packet.
 *
 *  \return  PAS_SOK or PSP Error code
 */
static PAL_Result ddc_mcbspSubmitReq(
                                DDC_mcbspIoHandle ioHandle,
                                PSP_mcbspReq *pspRequestp
                                )
{

    PAL_Result                  retVal          = PSP_SOK;
    Uint32                      cookie          = 0;
    Bool                        falsewhile      = FALSE;
    DDC_mcbspIOP                *ioPacket       = NULL;



    do
    {
        falsewhile      = FALSE;

        if(((NULL == ioHandle) || (NULL==pspRequestp)) ||
            (NULL == ioHandle->deviceHandle))
        {
            retVal = PSP_E_INVAL_PARAM;
            break;
        }

        /* check for the valid packet size */
        if (( 0 == pspRequestp->size)||(MCBSP_MAX_PACKET_SIZE<pspRequestp->size))
        {
            retVal =  PSP_E_INVAL_PARAM;
            break;
        }


        /* If currently requests are flushing no request can be served */
        if (TRUE == ioHandle->flush)
        {
            retVal =  PSP_E_INVAL_PARAM;
            break;
        }


        /* maintain cache coherency */
        if (PSP_MCBSP_INPUT == ioHandle->mode)
        {

            /* Cleans and invalidate the D Cache in MVA way */
            if (TRUE == ioHandle->deviceHandle->enablecache)
            {
                PAL_osCacheInvalidate(
                            PAL_osCache_NOT_DEFINED,
                            (Uint32)pspRequestp->addr,
                            pspRequestp->size
                            );
            }

        }
        else
        {

            /* clean the D Cache in MVA way */
            if (TRUE == ioHandle->deviceHandle->enablecache)
            {

                PAL_osCacheFlush(
                            PAL_osCache_NOT_DEFINED,
                            (Uint32)pspRequestp->addr,
                            pspRequestp->size
                            );
            }
        }


        if( PSP_McBSP_McBSP != ioHandle->deviceHandle->currentMode )
        {

            if(TRUE == PAL_osListIsEmpty (&ioHandle->queueFreeList) )
            {
                /*
                all packets already exhausted
                */
                retVal = PSP_E_INVAL_PARAM;
                break;
            }
            else
            {

                    ioPacket = (DDC_mcbspIOP *)PAL_osListDequeue(
                                                    &ioHandle->queueFreeList);

                    if(NULL != ioPacket)
                    {

                        ddc_copyReqtoIOP( pspRequestp, ioPacket);

                        if(NULL == ioHandle->dataPacket)
                        {
                            /* no transfer is hapening in any of the channels in
                             * this instance of the driver.So for sure we can
                             * initiate the transfer here itself */
                             ioHandle->dataPacket        = ioPacket;
                             /* call i2c transfer function                  */
                             retVal =  DDC_McBspSpiTransfer(
                                                    ioHandle,
                                                    ioPacket,
                                                    NULL);

                        }
                        else
                        {
                            /* Enter critical sec to protect the submit count */
                            PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT,
                                               &cookie);

                            /*Increment the submit count*/
                            ioHandle->submitCount++;

                            PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
                            PAL_osListEnqueue (&ioHandle->queueReqList,
                                               (Ptr)ioPacket);

                        }
                    }
             }

        }
        else
        {
            if(TRUE == PAL_osListIsEmpty (&ioHandle->queueFreeList) )
            {

                /*
                all packets already exhausted
                */
                retVal = PSP_E_INVAL_PARAM;
                break;
            }
            else
            {

                ioPacket = (DDC_mcbspIOP *)PAL_osListDequeue(
                                                &ioHandle->queueFreeList);

                if(NULL!=ioPacket)
                {
                    /*
                    We have a ioPacket object to take up the request
                    populate the IOP from request structure
                    */
                    ddc_copyReqtoIOP( pspRequestp, ioPacket);

                }

            }

            /*  Enter critical section to protect the submit count */
            PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);


            /*
            if there are enough floating request packets then
            enqueue this request.
            */
            if ((MAXLINKCNT <= ioHandle->submitCount )
                 || (TRUE == ioHandle->paused))
            {
                /*
                if we dont have enough floating request packets then
                enqueue this request into the pending request queue or
                or paused is issued the we will allow to queue the packets.
                If pause remains for the period more than the size of
                request queue submitreq will return error mentioning
                not enough memory available.
                */

                /*Increment the submit count*/
                ioHandle->submitCount++;

                PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

                PAL_osListEnqueue (&ioHandle->queueReqList, (Ptr)ioPacket);


                /*
                * Processing of request is not complete; so return pending
                * status.request will be marked as complete when it is processed
                * by the edma callback function. If the application makes a sync
                * call to theGIO layer, it will pend with the current result
                * till request status is marked as complete  */
            }
            else
            {
                /*Increment the submit count*/
                ioHandle->submitCount++;

                PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

                /* floating queue not full . Process this request.*/

                PAL_osListEnqueue(&(ioHandle->queueFloatingList),ioPacket);

                ioHandle->currentDataSize = (Uint16)ioPacket->size;
                ioHandle->userDataBufferSize = (Uint32)ioPacket->size;

                /*
                for DMA mode we will not use ioHandle->dataPacket
                to hold current packet - we use floatingqueue
                */
                ioHandle->dataPacket = NULL;


                 /*  Enter critical section to protect the submit count */
                PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

                if( MAXLINKCNT == ioHandle->submitCount)
                {
                    /*
                    Though we have to post to param set directly from here,
                    there will be differene between first such packet and
                    second packet. As we have control here we are second packet
                    and first packet has not yet returned (or corresponding
                    edma callback has not been called.For second packet, we
                    will be updating the second param set, which is currently
                    hosting loopjob parameter. Hence increment the index to
                    point second paramset and since we are moving out loopjob
                    from both param sets, the loopjobUpdatedinParamset is reset
                    */
                    ioHandle->loopjobUpdatedinParamset =FALSE;
                    GetNextIndex(ioHandle->nextLinkParamSetToBeUpdated );
                }
                /*

                else --
                this is a first packet - meaning one paramset is not active
                (to which currently index is pointing and
                the other hosting active self linked loopjob
                */

                if (PSP_SOK != ddc_updateDataPacketTolinkParams(ioHandle, ioPacket))
                {
                    retVal = PSP_MCBSP_DMA_ERROR;
                    /*  Enter critical section to protect the submit count */
                    PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
                    break;
                }


                if ( 1u == ioHandle->submitCount )
                {
                    /*
                    if at all this is the very first packet, then one param set
                    has loop job loaded , self linked and active with the main
                    xfer channel param. other param set is ready loaded (just now
                    and has link paramater set as the one having loopjob (this is
                    to ensure that if at all we are not getting any more packets
                    loopjob be will taken over). Now we have to link the
                    floating / newly loaded param set to xfer channel.
                    */

                    retVal = EDMA3_DRV_linkChannel(
                                    ioHandle->edmaHandle,
                                    ioHandle->xferChan,
                                    ioHandle->pramTbl[ioHandle->nextLinkParamSetToBeUpdated]
                                    );

                    if (PSP_SOK != retVal)
                    {
                        retVal = PSP_MCBSP_DMA_ERROR;
                        /*  Enter critical section to protect the submit count */
                        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
                        break;
                    }
                }

                /*  Enter critical section to protect the submit count */
                PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

                } /* Pending queue empty or not*/

          }

        retVal = PSP_SINPROGRESS;


    }while (falsewhile);

    return (retVal);
}


void DDC_ConfigureMcBSPInSPI(struct DDC_mcbspDevice_t  *deviceHandle)
{

    Uint32                  ctrlMask        = 0;
    Uint8                   ClkDiv          = 0;


        if(PSP_McBSP_McBSP_SPISLAVE == deviceHandle->currentMode)
        {

           /*Implement start up sequence of SPI mode*/

           /*1. Place the transmitter and receiver in reset:
           a. To reset the transmitter, clear the transmitter reset (XRST) bit
                 in SPCR to 0.
           b. To reset the receiver, clear the receiver reset (RRST) bit
                 in SPCR to 0.
           2. To reset the sample rate generator, clear the sample rate
                generator reset (GRST) bit in SPCR to 0
           This is already done by Hwcfg in Devparams */

           /*3. Program the appropriate McBSP registers to configure the
                    McBSP for proper operation as an SPI master or an SPI slave:

           3a)(SPCR) CLKSTP = 2h or 3h The clock stop mode
                    (without or with a clock delay) is    selected.*/
           deviceHandle->hwInfo.regs->SPCR |=2<<11;


           /*3b)Pin control register (PCR) FSXM = 0 The FSX pin is an input pin,
                 so that it can be driven by the    SPI master.*/
           deviceHandle->hwInfo.regs->PCR &= ~(1<<11);

           /*3c CLKXM = 0 The CLKX pin is an input pin,
                so that it can be driven by the SPI master.
                Because CLKSTP is equal to 2h or 3h, CLKR is
                driven internally by CLKX.*/
           deviceHandle->hwInfo.regs->PCR &= ~(1<<9);

           /*3d SCLKME=0 SCLKME bit is used in conjunction with the CLKSM bit in
           SRGR to determine the source for the input clock. The clock
           generated by the sample rate generator (CLKG) is derived
           from the McBSP internal input clock. (The sample rate
           generator is used to synchronize the McBSP logic with the
           externally-generated master clock.)*/
           deviceHandle->hwInfo.regs->PCR &= ~(1<<7);

           /*3e FSXP = 1 The FSX pin is active low.*/
           deviceHandle->hwInfo.regs->PCR |= (1<<3);

           /*3f CLKXP = 0 or 1 The polarity of CLKX as seen on the CLKX pin is
            positive    (CLKXP = 0) or negative (CLKXP = 1).*/
//           deviceHandle->hwInfo.regs->PCR &= ~(1<<1);
//           deviceHandle->hwInfo.regs->PCR |= (1<<1);

           /*3g CLKRP = 0 or 1 The polarity of CLKR as seen on the CLKR pin is
            positive (CLKRP = 0) or negative (CLKRP = 1).*/
//           deviceHandle->hwInfo.regs->PCR &= ~1;


           /*3h Sample rate generator register (SRGR) CLKSM = 1 CLKSM bit is
            used in conjunction with the SCLKME bit in PCR to determine the
            source for the input clock. The clock generated by the sample rate
            generator (CLKG) is derived from the McBSP internal input clock.
            (The sample rate generator is used to synchronize the McBSP logic
            with the externally-generated master clock.)*/
           deviceHandle->hwInfo.regs->SRGR |= 1<<29;

           /*3i CLKGDV = 1 The sample rate generator divides the McBSP internal
             input  clock by 2 before generating CLKG.*/
           deviceHandle->hwInfo.regs->SRGR &= ~0xFF;
           deviceHandle->hwInfo.regs->SRGR |= 1;

           /*3j Transmit control register (XCR) XDATDLY = 0 This bit must be 0
                for SPI slave operation.Receive control register (RCR)
                RDATDLY = 0 This bit must be 0 for SPI slave operation.*/
           deviceHandle->hwInfo.regs->XCR &= ~(3<<16);
           deviceHandle->hwInfo.regs->RCR &= ~(3<<16);

            /*set to generate interupts on XRDY and RRDY*/
           deviceHandle->hwInfo.regs->SPCR &=~((3<<20)|(3<<4));


        }
        else if(PSP_McBSP_McBSP_SPIMASTER == deviceHandle->currentMode)
        {



               /*Implement start up sequence of SPI mode*/

               /*1. Place the transmitter and receiver in reset:
               a. To reset the transmitter, clear the transmitter reset (XRST)
                    bit in SPCR to 0.
               b. To reset the receiver, clear the receiver reset (RRST)
                    bit in SPCR to 0.
               2. To reset the sample rate generator, clear the sample rate
                    generator reset (GRST) bit in SPCR to 0
                    This is already done by Hwcfg in Devparams*/

               /*3. Program the appropriate McBSP registers to configure the
                    McBSP for proper operation as an SPI master or an SPI slave:

               3a)(SPCR) CLKSTP = 2h or 3h The clock stop mode
                    (without or with a clock delay) is selected.*/
               deviceHandle->hwInfo.regs->SPCR |=2<<11;


               /*3b)Pin control register (PCR) FSXM = 0
                    The FSX pin is an input pin, so that it can be driven by the
                    SPI master.*/
               deviceHandle->hwInfo.regs->PCR |= (1<<11);
               /*3c CLKXM = 0 The CLKX pin is an Output pin, so that it can be
                    driven by the SPI master. Because CLKSTP is equal to
                    2h or 3h, CLKR is  driven internally by CLKX.*/
               deviceHandle->hwInfo.regs->PCR |= (1<<9);
               /*3d SCLKME = 0 SCLKME bit is used in conjunction with the CLKSM
                bit in SRGR to determine the source for the input clock. The
                clock generated by the sample rate generator (CLKG) is derived
                from the McBSP internal input clock. (The sample rate
                generator is used to synchronize the McBSP logic with the
                externally-generated master clock.)*/
               deviceHandle->hwInfo.regs->PCR &= ~(1<<7);

               /*3e FSXP = 1 The FSX pin is active low.*/
               deviceHandle->hwInfo.regs->PCR |= (1<<3);

               /*3f CLKXP = 0 or 1 The polarity of CLKX as seen on the CLKX pin
                is positive (CLKXP = 0) or negative (CLKXP = 1).*/
//               deviceHandle->hwInfo.regs->PCR &= ~(1<<1);

               /*3g CLKRP = 0 or 1 The polarity of CLKR as seen on the CLKR pin
                 is positive   (CLKRP = 0) or negative (CLKRP = 1).*/
//               deviceHandle->hwInfo.regs->PCR &= ~1;


              /* Before Programming the SRG Put the SRG in reset mode */

                ctrlMask = LLC_MCBSP_CTRL_SRG_DISABLE ;
                LLC_mcbspHwControl(
                            (LLC_McbspObj *)&(deviceHandle->hwInfo),
                            LLC_MCBSP_CMD_RESET_CONTROL,
                            &ctrlMask
                            );

               /*3h Sample rate generator register (SRGR) CLKSM = 1 CLKSM bit
                is used in conjunction with the SCLKME bit in
                PCR to determine the source for the input clock. The clock
                generated by the sample rate generator (CLKG) is derived
                from the McBSP internal input clock. (The sample rate
                generator is used to synchronize the McBSP logic with the
                externally-generated master clock.)*/
               deviceHandle->hwInfo.regs->SRGR |= 1<<29;

               /*3i The transmitter drives a frame-sync pulse on the FSX pin
                every time data is transferred from DXR to XSR. */

               deviceHandle->hwInfo.regs->SRGR &=~(1<<28);


               /*3j CLKGDV = 0x00 to 0xFF1 The sample rate generator divides
                    the McBSP internal input clock by x values which is
                    calculated using SPI master clk speed specifed by the user
                    before generating CLKG.*/

               deviceHandle->hwInfo.regs->SRGR &= ~(0xFF);
               ClkDiv = (Uint8)DDC_SpiMstrClockCalculate(deviceHandle->ModuleClkFreq,
	               						deviceHandle->McbspSpiMstrFreq);
               deviceHandle->hwInfo.regs->SRGR |= ClkDiv;

               /* After Prograimming, take out the SRG from reset */

               ctrlMask = LLC_MCBSP_CTRL_SRG_ENABLE ;
               LLC_mcbspHwControl(
                        (LLC_McbspObj *)&(deviceHandle->hwInfo),
                        LLC_MCBSP_CMD_RESET_CONTROL,
                        &ctrlMask
                        );

               /*3k Transmit control register (XCR) XDATDLY = 1 This bit must
                    be 1 for SPI Master operation.Receive control register (RCR)
                    RDATDLY = 1 This bit must be 1 for SPI master operation.*/
               deviceHandle->hwInfo.regs->XCR &= ~(3<<16);
               deviceHandle->hwInfo.regs->XCR |= (1<<16);
               deviceHandle->hwInfo.regs->RCR &= ~(3<<16);
               deviceHandle->hwInfo.regs->RCR |= (1<<16);

                /*set to generate interupts on XRDY and RRDY*/
               deviceHandle->hwInfo.regs->SPCR &=~((3<<20)|(3<<4));

               /*
               4. To release the sample rate generator from reset, set the
                  sample rate generator reset (GRST) bit in SPCR to 1. Make
                  sure that during the write to SPCR, you only modify the
                  GRST bit; otherwise, you will modify the McBSP configuration
                  you selected in the previous step.*/
              // deviceHandle->hwInfo.regs->SPCR |= 1<<21;

               /*
               5. After the sample rate generator is released from reset,
               wait two sample rate generator clock periods for
               the McBSP logic to stabilize.*/
               //PAL_osWaitMsecs(2);

               /* TBD
               6. Enable the transmitter and receiver:
               a. If the CPU services the McBSP transmit and receive buffers,
               then you can immediately enable the transmitter
               (XRST = 1 in SPCR) and enable the receiver (RRST = 1 in SPCR).*/
               //deviceHandle->hwInfo.regs->SPCR |= 0x101;

               /* b. If the DMA controller services the McBSP transmit and
               receive buffers, then you must first configure the DMA
               controller (this includes enabling the channels that service the
               McBSP buffers).When the DMA controller is ready, set XRST = 1
               and RRST = 1 - NOT Applicable.*/

               /* c. In either case, make sure you only change the XRST and
               RRST bits when you write to SPCR; otherwise, you will modify the
               bit settings you selected earlier in this procedure.*/

               /*
               7. After the transmitter and receiver are released from reset,
               wait two sample rate generator clock periods
               for the McBSP logic to stabilize.*/
               //PAL_osWaitMsecs(2);

               /*
               8. If necessary, enable the frame-sync logic of the sample rate
               generator. After the required data acquisition setup is done
               (DXR is loaded with data), set the FRST bit in SPCR to 1 if an
               internally generated frame-sync pulse is required (that is, if
               the McBSP is the SPI master).- NOT applicable*/


        }
 }


/*!
 *  \fn  PSP_Result PSP_mcbspIoctl(PSP_Handle hMcBSPChan,
 *                                 PSP_McbspIoctlCmd cmd,
 *                                 Ptr arg,
 *                                *Ptr param)
 *
 *  \brief Implements the IOCTLS for McBSP IOM mini driver.
 *
 *  ddc_mcbspControlChan() implements McBSP device specific control operations
 *  including Start/Stop, Pause/Resume, Channel and Device Reset,
 *  Mute-ON/OFF, McBSP sample rate generator configuration
 *
 *  \param  [IN]  hMcBSPChan    Channel handle
 *  \param  [IN]  cmd           control functionality to perform
 *  \param  [IN]  arg           optional device-defined data structure
 *  \param  [IN]  param         optional parameter
 *  \return  success(PSP_SOK) or failure code
 */
/* ARGSUSED */
PSP_Result PSP_mcbspIoctl   (
                            PSP_Handle hMcBSPChan,
                            PSP_McbspIoctlCmd cmd,
                            Ptr arg,
                            Ptr param
                            )
{
    DDC_mcbspIoHandle       ddcIoObj    = NULL;
    Uint32                  ctrlMask    = 0;
    PSP_mcbspChanParams     *params     = NULL;
    PSP_Result              result      = PSP_SOK;
    Uint32                  falseWhile  = FALSE;
    DDC_mcbspIOP            *ioPacket    = NULL;
    Int16                   linkCnt     = 0;
    Uint32                  mcbspWrdLen = 0;
    EDMA3_DRV_PaRAMRegs     pramPtr     = {0};
    Uint32                  i           = 0;
    Uint32                  mask        = 0;

    if(NULL != param)
    {
        param   =   param;
    }
    if(NULL != hMcBSPChan){
        ddcIoObj = (DDC_mcbspIoHandle) hMcBSPChan;
    }
    switch (cmd)
    {
        /* Starts the data transfer */
        case  PSP_CTRL_McBSP_START:
            {
                if (PSP_MCBSP_INPUT == ddcIoObj->mode)
                {
                    ctrlMask = LLC_MCBSP_CTRL_RX_ENABLE;
                    LLC_mcbspHwControl  (
                                        (LLC_McbspObj *)&(ddcIoObj->deviceHandle->hwInfo),
                                        LLC_MCBSP_CMD_RESET_CONTROL,
                                        &ctrlMask
                                        );
                }
                else
                {
                    ctrlMask = LLC_MCBSP_CTRL_TX_ENABLE;
                    LLC_mcbspHwControl  (
                                        (LLC_McbspObj *)&(ddcIoObj->deviceHandle->hwInfo),
                                        LLC_MCBSP_CMD_RESET_CONTROL,
                                        &ctrlMask
                                        );
                }

                break;
            }

         /* Stops the data transfer */
        case  PSP_CTRL_McBSP_STOP:
            {
                if (PSP_MCBSP_INPUT  == ddcIoObj->mode)
                {
                   ctrlMask =   LLC_MCBSP_CTRL_RX_DISABLE;
                   LLC_mcbspHwControl   (
                                        (LLC_McbspObj *)&(ddcIoObj->deviceHandle->hwInfo),
                                        LLC_MCBSP_CMD_RESET_CONTROL,
                                        &ctrlMask
                                        );

                }
                else
                {
                    ctrlMask = LLC_MCBSP_CTRL_TX_DISABLE;
                    LLC_mcbspHwControl  (
                                        (LLC_McbspObj *)&(ddcIoObj->deviceHandle->hwInfo),
                                        LLC_MCBSP_CMD_RESET_CONTROL,
                                        &ctrlMask
                                        );
                }
                break;
            }

        /* Enable the McBSP in digital loopback mode if the arg param is TRUE */
        case PSP_CTRL_McBSP_LOOPBACK:
            {
                if(NULL != arg)
                {
                    if(PSP_MCBSP_LOOPBACK_ON == (PSP_mcbspLoopback)
                                                (*(Uint32 *)arg))
                    {
                        ddcIoObj->deviceHandle->hwInfo.regs->SPCR |=
                                                   LLC_MCBSP_SPCR_DLB_MASK;
                    }
                    else
                    {
                        ddcIoObj->deviceHandle->hwInfo.regs->SPCR &=
                                                   ( ~LLC_MCBSP_SPCR_DLB_MASK );
                    }
                }
                else
                {
                    result = PSP_E_INVAL_PARAM;
                }
                break;
            }
        /*
         * Pause the playback operation....
         * Regarding with Pause operation, actual operation is similar with
         * stop but current data buffer and pointer should be kept for
         * resume command. Continuously transfer the zero byte
         */
        case PSP_CTRL_McBSP_PAUSE:
            {
                if (FALSE == ddcIoObj->paused)
                {
            ddcIoObj->paused = TRUE;
                }
                else
                {
                    result = PSP_E_INVAL_PARAM;
                }
            break;
            }

        /*
         * This command will Resume the driver to normal data transfer
         * state.
         */
        case  PSP_CTRL_McBSP_RESUME:
            {

                if (FALSE == ddcIoObj->paused )
                {
                    /* Pause is not issued so can not perform resume */
                    result = PSP_E_INVALID_CMD;
                    break;
                }

                if (TRUE == PAL_osList_NP_IsEmpty (&ddcIoObj->queueFloatingList))
                {
                    for (linkCnt = 0;linkCnt < MAXLINKCNT;linkCnt++)
                    {
                        if( FALSE == PAL_osList_NP_IsEmpty (
                                                    &ddcIoObj->queueReqList) )
                        {
                            ioPacket = (DDC_mcbspIOP *)PAL_osList_NP_Dequeue(
                                                      &ddcIoObj->queueReqList);

                            /* floating queue not full . Process this request.*/
                            PAL_osList_NP_Enqueue(&(ddcIoObj->queueFloatingList),
                                                  ioPacket);

                            ddcIoObj->currentDataSize    = (Uint16)ioPacket->size;
                            ddcIoObj->userDataBufferSize = (Uint32)ioPacket->size;

                            /*
                            for DMA mode we will not use ioHandle->dataPacket
                            to hold current packet - we use floatingqueue
                            */
                            ddcIoObj->dataPacket         = NULL;

                            if( (MAXLINKCNT-1u) == linkCnt)
                            {
                                /*
                                Though we have to post to param set directly
                                from here,there will be differene between first
                                such packet and second packet. As we have
                                control here we are second packet and first
                                packet has not yet returned (or corresponding
                                edma callback has not been called.For second
                                packet, we will be updating the second param
                                set, which is currently hosting loopjob
                                parameter. Hence increment the index to point
                                second paramset and since we are moving out
                                loopjob from both param sets, the
                                loopjobUpdatedinParamset is reset   */
                                ddcIoObj->loopjobUpdatedinParamset =FALSE;
                                GetNextIndex(ddcIoObj->nextLinkParamSetToBeUpdated );
                            }
                            /*
                            else --
                            this is a first packet - meaning one paramset is not
                             active (to which currently index is pointing and
                            the other hosting active self linked loopjob
                            */

                            if (PSP_SOK != ddc_updateDataPacketTolinkParams(
                                                                    ddcIoObj,
                                                                    ioPacket))
                            {
                                result = PSP_MCBSP_DMA_ERROR;
                                /*  Enter critical section to protect the
                                    submit count */
                                /* PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie); */
                                break;
                            }


                            if ( 0 == linkCnt )
                            {
                                /*
                                if at all this is the very first packet, then
                                one param set has loop job loaded , self linked
                                and active with the main xfer channel param.
                                other param set is ready loaded (just now and
                                has link paramater set as the one having loopjob
                                (this is to ensure that if at all we are not
                                getting any more packets loopjob be will taken
                                over). Now we have to link the floating / newly
                                loaded param set to xfer channel.
                                */

                                result = EDMA3_DRV_linkChannel(
                                                ddcIoObj->edmaHandle,
                                                ddcIoObj->xferChan,
                                                ddcIoObj->pramTbl[ddcIoObj->nextLinkParamSetToBeUpdated]
                                                );

                                if (PSP_SOK != result)
                                {
                                    result = PSP_MCBSP_DMA_ERROR;
                                    /*  Enter critical section to protect the
                                    submit count */
                                    /* PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT,
                                                         cookie); */
                                    break;
                                }

                                EDMA3_DRV_enableTransfer(
                                            ddcIoObj->edmaHandle,
                                            ddcIoObj->xferChan,
                                            EDMA3_DRV_TRIG_MODE_EVENT
                                            );
                            }
                        }
                    }
                } /* (FALSE == PAL_osList_NP_IsEmpty (
                                               &ddcIoObj->queueFloatingList)) */

                    ddcIoObj->paused = FALSE;
                }
            break;

        /* Changes data or output address of DMA */
        case  PSP_CTRL_McBSP_CHG_ADDR:
            {
                if (PSP_MCBSP_INPUT == ddcIoObj->mode)
                {
                    EDMA3_DRV_setPaRAMEntry (
                                            ddcIoObj->edmaHandle,
                                            ddcIoObj->xferChan,
                                            EDMA3_DRV_PARAM_ENTRY_DST,
                                            (Uint32)arg
                                            );
                }
                else
                {
                    EDMA3_DRV_setPaRAMEntry (
                                            ddcIoObj->edmaHandle,
                                            ddcIoObj->xferChan,
                                            EDMA3_DRV_PARAM_ENTRY_SRC,
                                            (Uint32)arg
                                            );
                }
                break;
            }

         /* Set the CLKR or CLKX clock polarity */
        case PSP_CTRL_McBSP_SPI_SET_CLKX_POL:
            {
                  if (PSP_MCBSP_BCLK_FALLING_EDGE == (PSP_McbspBclkPol)
                                                     (*(Uint32 *)arg))
                  {
                      ddcIoObj->deviceHandle->hwInfo.regs->PCR |=
                                                  LLC_MCBSP_PCR_CLKXP_MASK ;
                  }
                  else
                  {
                      ddcIoObj->deviceHandle->hwInfo.regs->PCR &=
                                                  ( ~LLC_MCBSP_PCR_CLKXP_MASK ) ;
                  }
             }
                break;

         /* Set the FSX polarity */
        case PSP_CTRL_McBSP_SPI_SET_CS_POL:
            {

                 if (PSP_McBSP_McBSP_SPIMASTER != ddcIoObj->deviceHandle->currentMode)
                 {
                    /* FSX can only change in master mode only */
                        result = PSP_E_INVALID_CMD;
                        break;
                  }

                  if(PSP_MCBSP_FSPOL_ACTIVE_LOW  == (PSP_McbspFsPol )
                                                    (*(Uint32 *)arg))
                  {
                      ddcIoObj->deviceHandle->hwInfo.regs->PCR |=
                                                LLC_MCBSP_PCR_FSXP_MASK;
                  }
                  else
                  {
                      ddcIoObj->deviceHandle->hwInfo.regs->PCR &=
                                                (~LLC_MCBSP_PCR_FSXP_MASK);
                  }
                break;
            }

        /*
         * Stop all driver operation and Reset DMA and McBSP and
         * initialize them again with previous initialized
         * configuration. The McBSP module is reset and then
         * re-configured. The DMA channels are stopped and
         * re-programmed. McBSP channel reset and setup the McBSP
         * and EDMA with default parameters
         */
        case  PSP_CTRL_McBSP_SPI_CHAN_RESET:
           {
                /* for input channel */
                if (PSP_MCBSP_INPUT == ddcIoObj->mode)
                {

                    /* Abort all the requests, give application callback
                    */
                    PSP_mcbspAbortRequests(ddcIoObj, NULL);

                }
                else
                {
                   /* Abort all the requests, give application callback
                    */
                   PSP_mcbspAbortRequests(ddcIoObj, NULL);
                }
                break;
            }

        /*
         * Stop all driver operation and Reset DMA and McBSP and
         * initialize them again with previous initialized
         * configuration. The McBSP module is reset and then
         * re-configured. The DMA channels are stopped and
         * re-programmed. McBSP channel reset and setup the McBSP
         * and EDMA with default parameters
         */
        case  PSP_CTRL_McBSP_CHAN_RESET:
           {
                /* for input channel */
                if (PSP_MCBSP_INPUT == ddcIoObj->mode)
                {

                    /* Place the McBSP receiver in Reset state */
                    ctrlMask = LLC_MCBSP_CTRL_RX_DISABLE;
                    LLC_mcbspHwControl(
                                (LLC_McbspObj *)&(ddcIoObj->deviceHandle->hwInfo),
                                LLC_MCBSP_CMD_RESET_CONTROL,
                                &ctrlMask
                                );


                    /* Abort all the request, give application callback and
                       configure EDMA in the LOOPJOB for both input and output
                       channels
                    */
                    PSP_mcbspAbortRequests(ddcIoObj, NULL);

                    ddcIoObj->deviceHandle->hwInfo.regs->RCR = (
                            ( LLC_MCBSP_WORDLEN_16 << LLC_MCBSP_DATA_WDLEN1_SHIFT ) |
                            ( 0 << LLC_MCBSP_DATA_WDLEN2_SHIFT ) |
                            ( 1 << LLC_MCBSP_DATA_FRLEN1_SHIFT ) |
                            ( 0 << LLC_MCBSP_DATA_FRLEN2_SHIFT ) |
                            ( LLC_MCBSP_DATADELAY_1_BIT << LLC_MCBSP_DATA_DELAY_SHIFT )
                        );
                    ddcIoObj->deviceHandle->hwInfo.regs->RCR &=
                                                (~LLC_MCBSP_RCR_RCOMPAND_MASK);
                    ddcIoObj->deviceHandle->hwInfo.regs->RCR &=
                                                (~LLC_MCBSP_RCR_RPHASE_MASK);

                    /*
                     *  As we are resetting the channel, we are configuring with
                     *  default values
                     */

                    ddcIoObj->deviceHandle->hwInfo.regs->SPCR &=
                                                (~LLC_MCBSP_SPCR_RINTM_MASK );
                    ddcIoObj->deviceHandle->hwInfo.regs->SPCR &=
                                                (~LLC_MCBSP_SPCR_RJUST_MASK );

                }
                else
                {
                    /* for output channel */

                    ctrlMask = LLC_MCBSP_CTRL_TX_DISABLE;
                    LLC_mcbspHwControl(
                                (LLC_McbspObj *)&(ddcIoObj->deviceHandle->hwInfo),
                                LLC_MCBSP_CMD_RESET_CONTROL,
                                &ctrlMask
                                );

                    /* Abort all the request, give application callback and
                       configure EDMA in the LOOPJOB for both input and output
                       channels
                    */
                    PSP_mcbspAbortRequests(ddcIoObj, NULL);

                    ddcIoObj->deviceHandle->hwInfo.regs->XCR = (
                            ( LLC_MCBSP_WORDLEN_16 << LLC_MCBSP_DATA_WDLEN1_SHIFT ) |
                            ( 0 << LLC_MCBSP_DATA_WDLEN2_SHIFT ) |
                            ( 1 << LLC_MCBSP_DATA_FRLEN1_SHIFT ) |
                            ( 0 << LLC_MCBSP_DATA_FRLEN2_SHIFT ) |
                            ( LLC_MCBSP_DATADELAY_1_BIT << LLC_MCBSP_DATA_DELAY_SHIFT )
                        );
                    ddcIoObj->deviceHandle->hwInfo.regs->XCR &=
                                                (~LLC_MCBSP_XCR_XCOMPAND_MASK);
                    ddcIoObj->deviceHandle->hwInfo.regs->XCR &=
                                                (~LLC_MCBSP_XCR_XPHASE_MASK);

                    ddcIoObj->deviceHandle->hwInfo.regs->SPCR &=
                                                (~LLC_MCBSP_SPCR_XINTM_MASK );

                }
                break;
            }

        /* Perform the device reset operation(Reset both the channels)  */

        case  PSP_CTRL_McBSP_SPI_DEVICE_RESET:
            {
                ctrlMask = LLC_MCBSP_CTRL_RX_DISABLE | LLC_MCBSP_CTRL_TX_DISABLE |
                           LLC_MCBSP_CTRL_FSYNC_DISABLE | LLC_MCBSP_CTRL_SRG_DISABLE;

                LLC_mcbspHwControl(
                            (LLC_McbspObj *)&(ddcIoObj->deviceHandle->hwInfo),
                            LLC_MCBSP_CMD_RESET_CONTROL,
                            &ctrlMask
                            );

                /* Abort all the request, give application callback and
                   configure EDMA in the LOOPJOB for both input and output
                   channels
                */
                PSP_mcbspAbortRequests((PSP_Handle)&(ddcIoObj->deviceHandle->rxIo),
                                        NULL);
                PSP_mcbspAbortRequests((PSP_Handle)&(ddcIoObj->deviceHandle->txIo),
                                        NULL);

                /* Configure the McBSP in SPI mode */
                DDC_ConfigureMcBSPInSPI(ddcIoObj->deviceHandle);
                break;
            }

        /* Perform the device reset operation(Reset both the channels)  */
        case  PSP_CTRL_McBSP_DEVICE_RESET:
            {
                ctrlMask = LLC_MCBSP_CTRL_RX_DISABLE | LLC_MCBSP_CTRL_TX_DISABLE |
                           LLC_MCBSP_CTRL_FSYNC_DISABLE | LLC_MCBSP_CTRL_SRG_DISABLE;

                LLC_mcbspHwControl(
                            (LLC_McbspObj *)&(ddcIoObj->deviceHandle->hwInfo),
                            LLC_MCBSP_CMD_RESET_CONTROL,
                            &ctrlMask
                            );

                /* Abort all the request, give application callback and
                   configure EDMA in the LOOPJOB for both input and output
                   channels
                */
                PSP_mcbspAbortRequests((PSP_Handle)&(ddcIoObj->deviceHandle->rxIo),
                                        NULL);
                PSP_mcbspAbortRequests((PSP_Handle)&(ddcIoObj->deviceHandle->txIo),
                                        NULL);

                /* Configure the McBSP by default in DSP mode */
                LLC_mcbspHwSetup (
                        (LLC_McbspObj *)&(ddcIoObj->deviceHandle->hwInfo),
                        (LLC_McbspHwSetup *)&myHwSetup
                        );

                break;
            }

        /* Configure the data stream of the McBSP */
        case PSP_CTRL_McBSP_CONFIG_DATA:
            {
                do
                {
                    falseWhile = FALSE;
                     /* invalid parameters for frame length1 */
                    if(FRAME_LENGTH < (((PSP_mcbspDataConfig *)arg)->frmLen1))
                    {
                        result = PSP_E_INVAL_PARAM;
                        break;
                    }

                    /* invalid parameters for frame length2 */
                    if(((PSP_mcbspDataConfig *)arg)->phaseNum == PSP_MCBSP_PHASE_DUAL)
                    {
                        if(FRAME_LENGTH < (((PSP_mcbspDataConfig *)arg)->frmLen2 ))
                        {
                            result = PSP_E_INVAL_PARAM;
                            break;
                        }
                    }

                    /* To remove the warning of comparing 0 with  unsinged value */
                    /* uncomment this if default value is greater then 0*/
                    if  ( ( PSP_MCBSP_WORDLEN_32 < ( ( PSP_mcbspDataConfig * )arg )->wrdLen1 ) )
                    {
                        result = PSP_E_INVAL_PARAM;
                        break;
                    }

                    /* invalid parameters for word length 2 */
                    if(PSP_MCBSP_PHASE_DUAL == ((PSP_mcbspDataConfig *)arg)->phaseNum)
                    {
                        /* To remove the warning of comparing 0 with  unsinged value */
                        /* uncomment this if default value is greater then 0*/
                        if( ( PSP_MCBSP_WORDLEN_32 < ((PSP_mcbspDataConfig *)arg)->wrdLen2 ))
                        {
                            result = PSP_E_INVAL_PARAM;
                            break;
                        }
                    }

                    /* invalid parameters for data delay */
                    if(PSP_MCBSP_DATADELAY_2_BITS < (((PSP_mcbspDataConfig *)arg)->dataDly ))
                    {
                        result = PSP_E_INVAL_PARAM;
                        break;
                    }

                    if (PSP_MCBSP_INPUT == ddcIoObj->mode)
                    {

                        ddcIoObj->deviceHandle->hwInfo.regs->RCR =
                            ddcIoObj->deviceHandle->hwInfo.regs->RCR &= (~LLC_MCBSP_RCR_RPHASE_MASK);
                            ddcIoObj->deviceHandle->hwInfo.regs->RCR |= \
                                ( ( ( ( PSP_mcbspDataConfig *)arg)->phaseNum ) << LLC_MCBSP_DATA_PHASE_SHIFT );

                            ddcIoObj->deviceHandle->hwInfo.regs->RCR &= (~LLC_MCBSP_RCR_RWDLEN1_MASK);
                            ddcIoObj->deviceHandle->hwInfo.regs->RCR |= \
                                ( ( ( ( PSP_mcbspDataConfig *)arg)->wrdLen1 ) << LLC_MCBSP_DATA_WDLEN1_SHIFT );

                            ddcIoObj->deviceHandle->hwInfo.regs->RCR &= (~LLC_MCBSP_RCR_RFRLEN1_MASK);
                            ddcIoObj->deviceHandle->hwInfo.regs->RCR |= \
                                ( ( ( ( PSP_mcbspDataConfig *)arg)->frmLen1 ) << LLC_MCBSP_DATA_FRLEN1_SHIFT );

                            ddcIoObj->deviceHandle->hwInfo.regs->RCR &= (~LLC_MCBSP_RCR_RDATDLY_MASK);
                            ddcIoObj->deviceHandle->hwInfo.regs->RCR |= \
                                ( ( ( ( PSP_mcbspDataConfig *)arg)->dataDly ) << LLC_MCBSP_DATA_DELAY_SHIFT );

                        /* Check for the dual phase */
                        if (PSP_MCBSP_PHASE_DUAL == ((PSP_mcbspDataConfig *)arg)->phaseNum)
                        {

                            ddcIoObj->deviceHandle->hwInfo.regs->RCR &= (~LLC_MCBSP_RCR_RWDLEN2_MASK);
                            ddcIoObj->deviceHandle->hwInfo.regs->RCR |= \
                                ( ( ( ( PSP_mcbspDataConfig *)arg)->wrdLen2 ) << LLC_MCBSP_DATA_WDLEN2_SHIFT );

                            ddcIoObj->deviceHandle->hwInfo.regs->RCR &= (~LLC_MCBSP_RCR_RFRLEN2_MASK);
                            ddcIoObj->deviceHandle->hwInfo.regs->RCR |= \
                                ( ( ( ( PSP_mcbspDataConfig *)arg)->frmLen2 ) << LLC_MCBSP_DATA_FRLEN2_SHIFT );

                        }

                        /*
                         * Configure the EDMA array size depending on the
                         * McBSP wordlength
                         */
                        ddcIoObj->roundedWordWidth = PSP_MCBSP_WORDLEN_32;

                        mcbspWrdLen = (Uint32)(((PSP_mcbspDataConfig *)arg)->wrdLen1);

                        if(PSP_MCBSP_WORDLEN_32 > mcbspWrdLen)
                        {
                            mask = 0x20u;

                            for (i=0; i<3u;i++)
                            {
                                if( mask == (mask &  mcbspWrdLen))
                                {
                                    ddcIoObj->roundedWordWidth = (Uint16)mask;

                                    if ( mcbspWrdLen > ddcIoObj->roundedWordWidth  )
                                    {
                                        ddcIoObj->roundedWordWidth = \
                                            (Uint16)(ddcIoObj->roundedWordWidth << 1);
                                    }
                                    break;
                                }
                                mask = (Uint32)(mask >> 1);
                            }
                        }

                        ddcIoObj->roundedWordWidth =
                            (Uint16)(ddcIoObj->roundedWordWidth >> 3);

                        /* EDMA array size will become 1byte */
                        EDMA3_DRV_getPaRAM(
                                    ddcIoObj->edmaHandle,
                                    ddcIoObj->xferChan,
                                    &pramPtr
                                    );
                        pramPtr.aCnt        = ddcIoObj->roundedWordWidth ;
                        pramPtr.srcBIdx     = 0;
                        pramPtr.destCIdx    = (Int16)ddcIoObj->roundedWordWidth;

                        EDMA3_DRV_setPaRAM(
                                    ddcIoObj->edmaHandle,
                                    ddcIoObj->xferChan,
                                    &pramPtr
                                    );

                        for (linkCnt=0; linkCnt < MAXLINKCNT; linkCnt++)
                        {
                            EDMA3_DRV_getPaRAM  (
                                                ddcIoObj->edmaHandle,
                                                ddcIoObj->pramTbl[linkCnt],
                                                &pramPtr
                                                );
                            pramPtr.aCnt        = ddcIoObj->roundedWordWidth ;
                            pramPtr.srcBIdx     = 0;
                            pramPtr.destCIdx    = (Int16)ddcIoObj->roundedWordWidth;
                            EDMA3_DRV_setPaRAM  (
                                                ddcIoObj->edmaHandle,
                                                ddcIoObj->pramTbl[linkCnt],
                                                &pramPtr
                                                );
                        }

                    }
                    else
                    {
                        ddcIoObj->deviceHandle->hwInfo.regs->XCR &= (~LLC_MCBSP_XCR_XPHASE_MASK);
                        ddcIoObj->deviceHandle->hwInfo.regs->XCR |=
                            ( ( ( ( PSP_mcbspDataConfig *)arg)->phaseNum)<< LLC_MCBSP_DATA_PHASE_SHIFT );

                        ddcIoObj->deviceHandle->hwInfo.regs->XCR &= (~LLC_MCBSP_XCR_XWDLEN1_MASK);
                        ddcIoObj->deviceHandle->hwInfo.regs->XCR |=
                            ( ( ( ( PSP_mcbspDataConfig *)arg)->wrdLen1)<< LLC_MCBSP_DATA_WDLEN1_SHIFT);

                        ddcIoObj->deviceHandle->hwInfo.regs->XCR &= (~LLC_MCBSP_XCR_XFRLEN1_MASK);
                        ddcIoObj->deviceHandle->hwInfo.regs->XCR |=
                            ( ( ( ( PSP_mcbspDataConfig *)arg)->frmLen1)<< LLC_MCBSP_DATA_FRLEN1_SHIFT );

                        ddcIoObj->deviceHandle->hwInfo.regs->XCR &= (~LLC_MCBSP_XCR_XDATDLY_MASK);
                        ddcIoObj->deviceHandle->hwInfo.regs->XCR |=
                            ( ( ( ( PSP_mcbspDataConfig *)arg)->dataDly)<< LLC_MCBSP_DATA_DELAY_SHIFT );

                        /* Check for the dual phase */
                        if (PSP_MCBSP_PHASE_DUAL == ((PSP_mcbspDataConfig *)arg)->phaseNum)
                        {

                            ddcIoObj->deviceHandle->hwInfo.regs->XCR &= (~LLC_MCBSP_XCR_XWDLEN2_MASK);
                            ddcIoObj->deviceHandle->hwInfo.regs->XCR |=
                                ( ( ( ( PSP_mcbspDataConfig *)arg)->wrdLen2 )<< LLC_MCBSP_DATA_WDLEN2_SHIFT );

                            ddcIoObj->deviceHandle->hwInfo.regs->XCR &= (~LLC_MCBSP_XCR_XFRLEN2_MASK);
                            ddcIoObj->deviceHandle->hwInfo.regs->XCR |=
                                ( ( ( ( PSP_mcbspDataConfig *)arg)->frmLen2)<< LLC_MCBSP_DATA_FRLEN2_SHIFT );
                        }
                        /*
                         * Configure the EDMA array size depending on the
                         * McBSP wordlength
                         */
                        ddcIoObj->roundedWordWidth = PSP_MCBSP_WORDLEN_32;

                        mcbspWrdLen = (Uint32)(((PSP_mcbspDataConfig *)arg)->wrdLen1);

                        if(PSP_MCBSP_WORDLEN_32 > mcbspWrdLen)
                        {
                            mask = 0x20u;

                            for (i=0; i<3u;i++)
                            {
                                if( mask == (mask &  mcbspWrdLen))
                                {
                                    ddcIoObj->roundedWordWidth = (Uint16)mask;

                                    if ( mcbspWrdLen > ddcIoObj->roundedWordWidth  )
                                    {
                                        ddcIoObj->roundedWordWidth = \
                                            (Uint16)(ddcIoObj->roundedWordWidth << 1);
                                    }
                                    break;
                                }
                                mask = (Uint32)(mask >> 1);
                            }
                        }

                        /*the driver has been configured for DSP (BURST mode)*/
                        ddcIoObj->roundedWordWidth =
                            (Uint16)(ddcIoObj->roundedWordWidth >> 3);

                        /* EDMA array size will become 1byte */
                        EDMA3_DRV_getPaRAM(
                                    ddcIoObj->edmaHandle,
                                    ddcIoObj->xferChan,
                                    &pramPtr
                                    );

                        pramPtr.aCnt        = ddcIoObj->roundedWordWidth ;
                        pramPtr.destBIdx   = 0;
                        pramPtr.srcCIdx    = (Int16)ddcIoObj->roundedWordWidth;

                        EDMA3_DRV_setPaRAM(
                                    ddcIoObj->edmaHandle,
                                    ddcIoObj->xferChan,
                                    &pramPtr
                                    );

                        for (linkCnt=0; linkCnt < MAXLINKCNT; linkCnt++)
                        {
                            EDMA3_DRV_getPaRAM  (
                                                ddcIoObj->edmaHandle,
                                                ddcIoObj->pramTbl[linkCnt],
                                                &pramPtr
                                                );
                            pramPtr.aCnt       = ddcIoObj->roundedWordWidth ;
                            pramPtr.destBIdx   = 0;
                            pramPtr.srcCIdx    = (Int16)ddcIoObj->roundedWordWidth;
                            EDMA3_DRV_setPaRAM  (
                                                ddcIoObj->edmaHandle,
                                                ddcIoObj->pramTbl[linkCnt],
                                                &pramPtr
                                                );
                        }

                    }
                }while (falseWhile);

                break;
            }

        /* Start the McBSP sample rate generator */
        case PSP_CTRL_McBSP_SRGR_START:
            {
                ctrlMask = LLC_MCBSP_CTRL_SRG_ENABLE ;
                LLC_mcbspHwControl(
                            (LLC_McbspObj *)&(ddcIoObj->deviceHandle->hwInfo),
                            LLC_MCBSP_CMD_RESET_CONTROL,
                            &ctrlMask
                            );
                break;
            }
        /* Stop the McBSP sample rate generator */
        case PSP_CTRL_McBSP_SRGR_STOP:
            {
                ctrlMask = LLC_MCBSP_CTRL_SRG_DISABLE ;
                LLC_mcbspHwControl(
                            (LLC_McbspObj *)&(ddcIoObj->deviceHandle->hwInfo),
                            LLC_MCBSP_CMD_RESET_CONTROL,
                            &ctrlMask
                            );
                break;
            }
        /* Start the McBSP Frame Synch generator */
        case PSP_CTRL_McBSP_FSGR_START:
            {
                ctrlMask = LLC_MCBSP_CTRL_FSYNC_ENABLE ;
                LLC_mcbspHwControl(
                            (LLC_McbspObj *)&(ddcIoObj->deviceHandle->hwInfo),
                            LLC_MCBSP_CMD_RESET_CONTROL,
                            &ctrlMask
                            );
                break;
            }
        /* Stop the McBSP Frame Synch generator */
        case PSP_CTRL_McBSP_FSGR_STOP:
            {
                ctrlMask = LLC_MCBSP_CTRL_FSYNC_DISABLE ;
                LLC_mcbspHwControl(
                            (LLC_McbspObj *)&(ddcIoObj->deviceHandle->hwInfo),
                            LLC_MCBSP_CMD_RESET_CONTROL,
                            &ctrlMask
                            );
                break;
            }
        /*
         * Set the Bit clock mode for the McBSP transmitter
         * or Receiver section
         */
        case PSP_CTRL_McBSP_SET_CLKMODE:
            {
                if (PSP_MCBSP_INPUT == ddcIoObj->mode)
                {
                    /* Change the Reciever Bit Clock mode */
                    if(PSP_MCBSP_BCLKCLKMODE_INTERNAL == (PSP_McbspBclkMode)(* (Uint8 *)arg))
                    {
                        ddcIoObj->deviceHandle->hwInfo.regs->PCR |=
                                                    LLC_MCBSP_PCR_CLKRM_MASK;
                    }
                    else
                    {
                        ddcIoObj->deviceHandle->hwInfo.regs->PCR &=
                                                    (~LLC_MCBSP_PCR_CLKRM_MASK);
                    }
                }
                else
                {
                    /* Change the Transmitter Bit Clock mode */
                    if(PSP_MCBSP_BCLKCLKMODE_INTERNAL == ( PSP_McbspBclkMode )(*(Uint8 *)arg))
                    {
                        ddcIoObj->deviceHandle->hwInfo.regs->PCR |=
                                                    LLC_MCBSP_PCR_CLKXM_MASK;
                    }
                    else
                    {
                        ddcIoObj->deviceHandle->hwInfo.regs->PCR &=
                                                    (~LLC_MCBSP_PCR_CLKXM_MASK);
                    }
                }
                break;
            }
        /* Set the Frame Sync mode for the McBSP module */
        case PSP_CTRL_McBSP_SET_FRMSYNCMODE:
            {
                /* Reciever Bit Clock mode */
                if (ddcIoObj->mode == PSP_MCBSP_INPUT)
                {
                    if(PSP_MCBSP_FRSYNCHMODE_INTERNAL == (PSP_McbspFrSynchMode )(*(Uint8 *)arg))
                    {
                        ddcIoObj->deviceHandle->hwInfo.regs->PCR |=
                                                    LLC_MCBSP_PCR_FSRM_MASK;
                    }
                    else
                    {
                        ddcIoObj->deviceHandle->hwInfo.regs->PCR &=
                                                    (~LLC_MCBSP_PCR_FSRM_MASK);
                    }
                }
                else
                {
                /* Transmitter nit clock mode */
                    if(PSP_MCBSP_FRSYNCHMODE_INTERNAL == (PSP_McbspFrSynchMode )(*(Uint8 *)arg))
                    {
                        ddcIoObj->deviceHandle->hwInfo.regs->PCR |=
                                                    LLC_MCBSP_PCR_FSXM_MASK;
                    }
                    else
                    {
                        ddcIoObj->deviceHandle->hwInfo.regs->PCR &=
                                                    (~LLC_MCBSP_PCR_FSXM_MASK);
                    }
                }
                break;
            }
        /*
         * Configure Sample Rate Generator(SRGR) to generate Frame sync
         * and bit clk when
         * McBSP is configured to generate CLKR,
         * CLKX,FSX and FSR internally (in Master Mode)
         */
        case PSP_CTRL_McBSP_CONFIG_SRGR:
            {
                do
                {
                    falseWhile = FALSE;
                    /*
                     * If the source for the SRGR is
                     * Recieve bit clock or Transmit bit clock
                     */
                    if ((((PSP_srgConfig *)arg)->srgInputClkMode ==
                              PSP_MCBSP_SRGCLK_CLKR) ||
                        (((PSP_srgConfig *)arg)->srgInputClkMode ==
                                    PSP_MCBSP_SRGCLK_CLKX))
                    {
                        ddcIoObj->deviceHandle->hwInfo.regs->PCR  |=
                                                    LLC_MCBSP_PCR_SCLKME_MASK ;
                        if(PSP_MCBSP_SRGCLK_CLKR == ((PSP_srgConfig *) arg)->srgInputClkMode)
                        {
                            ddcIoObj->deviceHandle->hwInfo.regs->SRGR &=
                                                   (~LLC_MCBSP_SRGR_CLKSM_MASK);
                        }
                        else
                        {
                            ddcIoObj->deviceHandle->hwInfo.regs->SRGR |=
                                                    (LLC_MCBSP_SRGR_CLKSM_MASK);
                        }

                    }
                    else
                    {
                        if ((((PSP_srgConfig *)arg)->srgInputClkMode ==
                                                      PSP_MCBSP_SRGCLK_CLKS) ||
                                   (((PSP_srgConfig *)arg) -> srgInputClkMode ==
                                                      PSP_MCBSP_SRGCLK_CLKCPU))
                        {
                            ddcIoObj->deviceHandle->hwInfo.regs->PCR &=
                                                    (~LLC_MCBSP_PCR_SCLKME_MASK);

                            if (PSP_MCBSP_SRGCLK_CLKCPU == ((PSP_srgConfig *)arg)->srgInputClkMode)
                            {
                                ddcIoObj->deviceHandle->hwInfo.regs->SRGR |=
                                                    LLC_MCBSP_SRGR_CLKSM_MASK;
                            }
                            else
                            {
                                ddcIoObj->deviceHandle->hwInfo.regs->SRGR &=
                                                    (~LLC_MCBSP_SRGR_CLKSM_MASK);
                            }
                        }
                        else
                        {
                            result =  PSP_E_INVAL_PARAM;
                            break;
                        }
                    }

                    if (PSP_MCBSP_FSX_DXR_COPY == (PSP_McbspSrgFsgm)((PSP_srgConfig *)arg)->srgFsgmMode)
                    {
                        ddcIoObj->deviceHandle->hwInfo.regs->SRGR &=
                                                    (~LLC_MCBSP_SRGR_FSGM_MASK);
                    }
                    else if (PSP_MCBSP_FSX_FSG == (PSP_McbspSrgFsgm)((PSP_srgConfig *)arg)->srgFsgmMode)
                    {
                        ddcIoObj->deviceHandle->hwInfo.regs->SRGR |=
                                                    LLC_MCBSP_SRGR_FSGM_MASK;
                    }
                    else
                    {
                        result =  PSP_E_INVAL_PARAM;
                        break;
                    }
                    /* Check for invalid clock range */
                    if (((Int32)((PSP_srgConfig *)arg)->bclkRate < 0) ||
                         (((PSP_srgConfig *)arg)->bclkRate > (255u)))
                    {
                        result =  PSP_E_INVAL_PARAM;
                        break;
                    }

                    /* Check for invalid frame synch pulse width range */
                    if (((Int32)(((PSP_srgConfig *)arg)->srgFrmPulseWidth) < 0) ||
                        ((((PSP_srgConfig *)arg)->srgFrmPulseWidth) > (255u)))
                    {
                        result =  PSP_E_INVAL_PARAM;
                        break;
                    }

                    /* Check for invalid frame synch period range */
                    if (((Int32)(((PSP_srgConfig *)arg)->srgFrmPeriod) < 0) ||
                        ((((PSP_srgConfig *)arg)->srgFrmPeriod) > (4095u)))
                    {
                        result =  PSP_E_INVAL_PARAM;
                        break;
                    }
                    ddcIoObj->deviceHandle->hwInfo.regs->SRGR &= (~LLC_MCBSP_SRGR_CLKGDV_MASK );
                    ddcIoObj->deviceHandle->hwInfo.regs->SRGR |=
                        ( ( ( ( PSP_srgConfig * ) arg )->bclkRate ) << LLC_MCBSP_SRGR_CLKGDV_SHIFT  );

                    ddcIoObj->deviceHandle->hwInfo.regs->SRGR &=
                                                (~ LLC_MCBSP_SRGR_FWID_MASK );
                    ddcIoObj->deviceHandle->hwInfo.regs->SRGR |=
                        ( ( ( ( PSP_srgConfig * ) arg )-> srgFrmPulseWidth ) << LLC_MCBSP_SRGR_FWID_SHIFT );

                    ddcIoObj->deviceHandle->hwInfo.regs->SRGR &=
                                                (~ LLC_MCBSP_SRGR_FPER_MASK );
                    ddcIoObj->deviceHandle->hwInfo.regs->SRGR |=
                        ( ( ( ( PSP_srgConfig * )arg )->srgFrmPeriod) << LLC_MCBSP_SRGR_FPER_SHIFT );

                }while (falseWhile);
                break;
            }
         /* Set the CLKR or CLKX clock polarity */
        case PSP_CTRL_McBSP_SET_BCLK_POL:
            {
                if (PSP_MCBSP_INPUT == ddcIoObj->mode)
                {
                    if(PSP_MCBSP_BCLK_FALLING_EDGE == (PSP_McbspBclkPol )(*(Uint32 *)arg))
                    {
                        ddcIoObj->deviceHandle->hwInfo.regs->PCR &=
                                                ( ~LLC_MCBSP_PCR_CLKRP_MASK );
                    }
                    else
                    {
                        ddcIoObj->deviceHandle->hwInfo.regs->PCR |=
                                                LLC_MCBSP_PCR_CLKRP_MASK ;
                    }
                }
                else
                {
                    if (PSP_MCBSP_BCLK_FALLING_EDGE == (PSP_McbspBclkPol) (*(Uint32 *)arg))
                    {
                        ddcIoObj->deviceHandle->hwInfo.regs->PCR |=
                                                LLC_MCBSP_PCR_CLKXP_MASK ;
                    }
                    else
                    {
                        ddcIoObj->deviceHandle->hwInfo.regs->PCR &=
                                                ( ~LLC_MCBSP_PCR_CLKXP_MASK ) ;
                    }
                }
                break;
            }
         /* Set the FSR and FSX polarity */
        case PSP_CTRL_McBSP_SET_FRMSYNC_POL:
            {
                if (PSP_MCBSP_INPUT == ddcIoObj->mode)
                {
                    if(PSP_MCBSP_FSPOL_ACTIVE_LOW == (PSP_McbspFsPol )(*(Uint32 *)arg))
                    {
                        ddcIoObj->deviceHandle->hwInfo.regs->PCR |=
                                                LLC_MCBSP_PCR_FSRP_MASK;
                    }
                    else
                    {
                        ddcIoObj->deviceHandle->hwInfo.regs->PCR &=
                                                (~LLC_MCBSP_PCR_FSRP_MASK);
                    }
                }
                else
                {
                    if(PSP_MCBSP_FSPOL_ACTIVE_LOW  == (PSP_McbspFsPol ) (*(Uint32 *)arg))
                    {
                        ddcIoObj->deviceHandle->hwInfo.regs->PCR |=
                                                LLC_MCBSP_PCR_FSXP_MASK;
                    }
                    else
                    {
                        ddcIoObj->deviceHandle->hwInfo.regs->PCR &=
                                                (~LLC_MCBSP_PCR_FSXP_MASK);
                    }
                }
                break;
            }
        /*
         * MUTE-ON command implementation
         * Typical implementation for Mute is writing zero data
         * in buffer instead of writing real data. Then we can keep
         * everything same with normal operation and it is important
         * because McBSP DMA interrupt count is essential source of
         * AV synchronization
         */
        case  PSP_CTRL_McBSP_MUTE_ON:
            {
                if (PSP_MCBSP_OUTPUT == ddcIoObj->mode)
                {
                    if(TRUE == ddcIoObj->bMuteON)
                    {
                        result = PSP_E_INVAL_PARAM;
                        break;
                    }

                    ddcIoObj->bMuteON = (Bool)TRUE;
                }
                else
                {
                    result = PSP_E_NOT_SUPPORTED;
                }

                break;
            }
        /* Take out from the the Muted state */
        case PSP_CTRL_McBSP_MUTE_OFF:
            {
                if (PSP_MCBSP_OUTPUT == ddcIoObj->mode)
                {
                    /* Check if mute is not ON */
                    if (FALSE == ddcIoObj->bMuteON )
                    {
                        result = PSP_E_INVAL_PARAM;
                        break;
                    }

                    ddcIoObj->bMuteON =(Bool)FALSE;

                }/*if only dma driven transaction*/
                else
                {
                    result = PSP_E_NOT_SUPPORTED;
                }
                break;
            }
        /*
         *  Enable / disable loopjob.
         *  If the buffer pointer is,
         *  NULL then disable loopjob
         *  not NULL then enable the loop job and asign buffer poiter to prdbuf
         */
        case PSP_CTRL_McBSP_MODIFY_LOOPJOB:
            {
                params = (PSP_mcbspChanParams *)arg;

                if(NULL == params)
                {
                    result =  PSP_E_NOT_SUPPORTED;
                    break;
                }


                if(NULL == params->userLoopJobBuffer)
                {
                    if(PSP_MCBSP_OUTPUT == ddcIoObj->mode)
                    {
                        ddcIoObj->loopJobBuffer = &loopSrcBuf[(ddcIoObj->deviceHandle->InstNum)];
                    }
                    else
                    {
                        ddcIoObj->loopJobBuffer = &loopDstBuf[(ddcIoObj->deviceHandle->InstNum)];
                    }
                }
                else
                {
                    ddcIoObj->loopJobBuffer = params->userLoopJobBuffer;
                }

                if(ddcIoObj->roundedWordWidth > params->userLoopJobLength)
                {
                    ddcIoObj->loopJobLength = ddcIoObj->roundedWordWidth;
                }
                else
                {
                    ddcIoObj->loopJobLength = params->userLoopJobLength;
                }



                break;
            }

        case PSP_CTRL_McBSP_RECEIVE_SYNCERR_INT_ENABLE:
            {
                ddcIoObj->deviceHandle->hwInfo.regs->SPCR |=
                                             LLC_MCBSP_SPCR_RINTM_MASK;
                break;
            }
        case PSP_CTRL_McBSP_XMIT_SYNCERR_INT_ENABLE:
            {
                ddcIoObj->deviceHandle->hwInfo.regs->SPCR |=
                                             LLC_MCBSP_SPCR_XINTM_MASK;
                break;
            }
        default:
            result = PSP_E_NOT_SUPPORTED;
    }
    return (result);
}


/**
 *  \fn static PAL_Result ddc_edmaProcessPkt(Ptr iohandlep, DDC_mcbspIOP *ioPacket)
 *
 *  \brief Process the packate
 *
 *       For the transmit operation, the eDMA channel's destination
 *       port is tied to the MCBSP DAT port. In case of receive, the eDMA
 *       channel's source port is tied to the MCBSP DAT port. The source
 *       address for transmit eDMA channel and the destination address for
 *       the receive eDMA channel are set here.
 *
 *  \param    iohandlep   [IN]    Pointer to channel
 *  \param    ioPacket    [IN]    Pointer to request to be processed
 *
 *  \return  IOM_COMPLETED, if the address is set correctly
 *            PSP_MCBSP_ABORTED IOM_EBADIO otherwise
 */
static PAL_Result ddc_edmaProcessPkt(Ptr iohandlep, DDC_mcbspIOP *ioPacket)
{

    DDC_mcbspIoHandle       ioHandle        = NULL;
    PSP_Result              result          = PSP_SOK;
    Bool                    falsewhile      = FALSE;


    do
    {

        falsewhile      = FALSE;

        /*
        Validate pointers and parameters
        */
        if((NULL == iohandlep)||(NULL==ioPacket))
        {
            result = PSP_E_INVAL_PARAM;
            break;
        }

        ioHandle = (DDC_mcbspIoHandle)iohandlep;


        if ((NULL == ioHandle) || (NULL == ioHandle->deviceHandle))
        {
            result = PSP_E_INVAL_PARAM;
            break;
        }


        /*
        we are going to update the paramset from EDMA callback context.
        That means the one of the param sets are having audio data
        just completed data request and one having currently loaded (active)
        data request Current index is pointing only latest paramset (that is
        active)  so move the index
        */
        GetNextIndex(ioHandle->nextLinkParamSetToBeUpdated );

        /*Link the pakcet*/
        result = ddc_updateDataPacketTolinkParams(ioHandle, ioPacket);



    }while(falsewhile);

    return (result); /* return */
}


/*!
 *  \fn static PAL_Result ddc_updateDataPacketTolinkParams  (
                                                  DDC_mcbspIoHandle ioHandle,
                                                  DDC_mcbspIOP * const ioPacket)
 *
 *  \brief
 *      This function updates the link param set accordingly with data or
 *      loopjobbuffer This uses properly maintained index to idendtify to what
 *      param set the the info has to be updated.
 *
 *
 *  \param  ioHandle [IN]  Channel handle
 *  \param  ioPacket [IN]  Pointer I/O request
 *
 *  \return  IOM_COMPLETED, if the address is set correctly
 *           IOM_EBADIO otherwise
 */
static PAL_Result ddc_updateDataPacketTolinkParams(DDC_mcbspIoHandle ioHandle,
                                                   DDC_mcbspIOP * const ioPacket)
{

    EDMA3_DRV_PaRAMRegs         pramPtr         = {0};
    DDC_mcbspDeviceHandle       deviceHandle    = NULL;
    Bool                        falsewhile      = FALSE;
    PAL_Result                  retVal          = PSP_SOK;

#ifdef MCBSP_INSTRUMENTATION_ENABLED
    if( PSP_MCBSP_OUTPUT == ioHandle->mode){
        PSP_LOG_EVENT(
            &DVTEvent_Log,
            "McBSP",
            PSP_DVT_DESC(
                    PSP_DVT_eDATA_SND_START,
                    PSP_DVT_dCHANNEL_ID_INST,
                    PSP_DVT_dPACKET_ID,
                    PSP_DVT_dNONE
                    ),
            CHANNELID_INSTANCEID(
                ioHandle->mode,
                ioHandle->deviceHandle->InstNum),
            ioPacket->appData
            );
    }
    else if( PSP_MCBSP_INPUT == ioHandle->mode){
        PSP_LOG_EVENT(
            &DVTEvent_Log,
            "McBSP",
            PSP_DVT_DESC(
                    PSP_DVT_eRCV_START,
                    PSP_DVT_dCHANNEL_ID_INST,
                    PSP_DVT_dPACKET_ID,
                    PSP_DVT_dNONE
                    ),
            CHANNELID_INSTANCEID(
                ioHandle->mode,
                ioHandle->deviceHandle->InstNum),
            ioPacket->appData
            );
    }
#endif /* MCBSP_INSTRUMENTATION_ENABLED */

    do
    {
        falsewhile      = FALSE;
        if (NULL == ioHandle)
        {
            retVal = PSP_E_INVAL_PARAM;
            break;
        }

        deviceHandle = ioHandle->deviceHandle;

        if ((NULL == deviceHandle))
        {

            retVal = PSP_E_INVAL_PARAM;
            break;
        }

        /*
        Get current info from the identified param
        we would like to change only certain parameters
        */
        if (PSP_SOK !=  EDMA3_DRV_getPaRAM(
                                ioHandle->edmaHandle,
                                ioHandle->pramTbl[ioHandle->nextLinkParamSetToBeUpdated],
                                &pramPtr
                                ))
        {
            retVal = PSP_E_INVAL_PARAM;
            break;
        }

        /*
        following the explanation behind the EDMA configuration

        1) We will have n arrays of data (now it is data buffer each of length
                databuffersize bytes)
        2) for each sync event we have to fetch the data from each array and
                write to each serilaiser
        3) and the no. bytes transferred in this case from each array will be
                roundedwordwidth/8
        4) as the loopjob buffer is contigous, we configure the counts as follows
            a) consider there are arrays of length roundedwordwidth/8 (aCnt)
            b) these arrays are located apart databuffersize size  (bindex)
            c) there are 'no.of.serialiser' no. of such arrays   (bCnt)
            d)  (a) and (b) constitute one frame of EDMA transfer and it happens
                for one sync event in AB sync mode
            e) such frames have to be sent till we have buffer - ie
               databuffersize / (roundedwordwidth/8 ) and this constitutes  cCnt
            f)  address of buffer (rather offset from start of previous frame)
                is roundedwordwidth/8  and this forms cIndex
        Please take some time to visualise the setup mentioned above to check
            with EDMa configuration

        Also we change only specific parameters because others will not have
        change from what we programed  intially
        */

         /* Load the buffer pointer into the EDMA */

        /*
        control will reach this function is two contexts
            1) From submitreq function for first time packets
            2)from EDMA callback for further handling of packets in Queue
                -we are from EDMA callback becuase we have completed some user
                data packet just now
        */

        if ( NULL != ioPacket)
        {

            /*
            1) when control comes to this function from submitrteq control will
                surely comeinto the block
            2) When control comes to this function from EDMA callback, control
                will some only
              if there is fuirther more packets to handle
            */

            if (PSP_MCBSP_INPUT == ioHandle->mode)
            {
                pramPtr.destAddr = (Uint32)ioPacket->addr;
                pramPtr.destBIdx = (Int16)ioHandle->userDataBufferSize;
                pramPtr.destCIdx = (Int16)ioHandle->roundedWordWidth;
                pramPtr.cCnt     = (Uint16)(ioHandle->userDataBufferSize/
                                   (ioHandle->roundedWordWidth));
            }
            else
            {

                if (TRUE == ioHandle->bMuteON)
                {

                    /*data buffer is changed to local mutebuffer*/
                    pramPtr.srcAddr  = (Uint32)&muteBuf[(deviceHandle->InstNum)];
                    pramPtr.srcBIdx  = (Int16)ioHandle->userDataBufferSize;
                    pramPtr.cCnt     = (Uint16)(ioHandle->userDataBufferSize/
                                       (ioHandle->roundedWordWidth));

                    /*Start again with same mute buffer*/
                    pramPtr.srcCIdx = 0;
                }
                else
                {
                    pramPtr.srcAddr = (Uint32)ioPacket->addr;
                    pramPtr.srcBIdx = (Int16)ioHandle->userDataBufferSize;
                    pramPtr.srcCIdx = (Int16)ioHandle->roundedWordWidth;
                    pramPtr.cCnt    = (Uint16)(ioHandle->userDataBufferSize/
                                      (ioHandle->roundedWordWidth));
                }/*for mute on/off*/
            }

            /*
            * Here we are enabling the interrupt for this param set information
            * we do it here in this way because using int enable API after
            * calling EDMA3_DRV_setPaRAM, may fail our intention. This is because
            * immediately after the set param the paramset might be copied to
            * linked param and will make the consecutive int enable (set opt field)
            * api useless for our purpose
            */
            pramPtr.opt |= (0x01u<<20u);

            /*
            we are loading data request in the param set. As per our design
            when we load the data,  we should link this param set with next one
            most probably
                a)That may have loopjob loaded
                b)it may be the currently loaded set, which will be updated in
                  next edma callback hence in both cases to be run next to this
                  data request
            */
            pramPtr.linkAddr = (Uint16)((pramPtr.linkAddr & 0xF000u) +
                    (ioHandle->pramTbl[((ioHandle->nextLinkParamSetToBeUpdated+1)&0x01u)]* 32u));



        }
        else
        {
            /*
            we have reached here becuase we dont have any more packett to submit
            for Xfer assign loop job to params - no change in linking
            IMP: The following assignment will be skipped if already both params
            are having  loopjob buffer
            */

            if (PSP_MCBSP_INPUT == ioHandle->mode)
            {
                pramPtr.destAddr = (Uint32)ioHandle->loopJobBuffer;
                pramPtr.destBIdx = (Int16)ioHandle->loopJobLength;
                pramPtr.destCIdx = (Int16)ioHandle->roundedWordWidth;
                pramPtr.cCnt     = (Uint16)(ioHandle->loopJobLength/(ioHandle->roundedWordWidth));
            }
            else
            {
                pramPtr.srcAddr = (Uint32)ioHandle->loopJobBuffer;
                pramPtr.srcBIdx = (Int16)ioHandle->loopJobLength;
                pramPtr.srcCIdx = (Int16)ioHandle->roundedWordWidth;
                pramPtr.cCnt    = (Uint16)(ioHandle->loopJobLength/(ioHandle->roundedWordWidth));
            }

            /*
             we are loading loopjob into this param.  That means we have reached
             the end of packets inpending and floating queues. So this loopjob
             had to be  linked to itself. Also we need to point the index to
             other param set in order to load data request
             directly from submit call
            */

            pramPtr.linkAddr =  (Uint16)((pramPtr.linkAddr & (0xF000u))  +
                            (ioHandle->pramTbl[ioHandle->nextLinkParamSetToBeUpdated]* (32u)));

            pramPtr.opt &= (~(0x01<<20));
        }

        /*
        Update the changed param set info into the appropriate
        paramset
        */
        if (PSP_SOK !=  EDMA3_DRV_setPaRAM(
                                ioHandle->edmaHandle,
                                ioHandle->pramTbl[ioHandle->nextLinkParamSetToBeUpdated],
                                &pramPtr
                                ))
        {
            retVal = PSP_E_INVAL_PARAM;
            break;
        }

        if ( NULL == ioPacket)
        {

            /*
            we are loading loopjob into this param.
            we need to point the index to other param set in order to load data
            request directly from submit call
            */
            GetNextIndex(ioHandle->nextLinkParamSetToBeUpdated );
        }

        falsewhile = FALSE;
    }while(falsewhile);

    return retVal;
}





/**
 *  \fn static void ddc_edmaMcbspSPITXCallback(Uint tcc,
 *                                             EDMA3_RM_TccStatus status,
 *                                             Ptr data)
 *  \brief Application callback
 *
 *       This calls the registered application callback for the current
 *       receive request and processes the next request in queue.
 *
 *  \param      tcc     [IN]    tcc number of eDMA
 *  \param      status  [IN]    status of eDMA
 *  \param      data    [IN]    paramaters send to eDMA ISR (chan handler)
 *
 *  \return     void
 */

static void ddc_edmaMcbspSPITXCallback(Uint tcc,
                                       EDMA3_RM_TccStatus status,
                                       Ptr data)
{


    /*
    To remove warning, kept for future usage
    */
    tcc = tcc;
    status = status;
    if(NULL != data)
    {
        data = data;
    }

}

/**
 *  \fn static void ddc_edmaMcbspSPIRXCallback(Uint tcc,
                                               EDMA3_RM_TccStatus status,
                                               Ptr data)
 *  \brief Application callback
 *
 *       This calls the registered application callback for the current
 *       receive request and processes the next request in queue.
 *
 *  \param      tcc     [IN]    tcc number of eDMA
 *  \param      status  [IN]    status of eDMA
 *  \param      data    [IN]    paramaters send to eDMA ISR (chan handler)
 *
 *  \return     void
 */

static void ddc_edmaMcbspSPIRXCallback(Uint tcc,
                                       EDMA3_RM_TccStatus status,
                                       Ptr data)
{



    DDC_mcbspIoHandle       ioHandle        = NULL;
    Bool                    falsewhile      = FALSE;
    /*
    To remove warning, kept for future usage
    */
    tcc = tcc;
    status = status;
    if(NULL != data)
    {
        data = data;
    }
    do
    {
        ioHandle = (DDC_mcbspIoHandle)data;
        if(NULL == ioHandle)
        {
            break;
        }
        ioHandle->deviceHandle->hwInfo.regs->SPCR &= ~(0x10001 | (1<<22));
        /* Disable current EDMA transfer */
        EDMA3_DRV_disableTransfer (
                        ioHandle->edmaHandle,
                        ioHandle->deviceHandle->hwInfo.edmaRxEventNum,
                        EDMA3_DRV_TRIG_MODE_EVENT
                        );
        /* Disable current EDMA transfer */
        EDMA3_DRV_disableTransfer (
                        ioHandle->edmaHandle,
                        ioHandle->deviceHandle->hwInfo.edmaTxEventNum,
                        EDMA3_DRV_TRIG_MODE_EVENT
                        );
        if ( EDMA3_RM_XFER_COMPLETE != status)
        {
             /* Clear the EDMA Error bits */
             ioHandle->pspRequest.status  = PSP_MCBSP_DMA_ERROR;
             /*
              There is a data transfer error in EDMA transfer. Return actualSize
              equal to 0 represent that there is an error in the packet transfer
             */
             ioHandle->pspRequest.actualSize = 0;
             EDMA3_DRV_clearErrorBits(ioHandle->edmaHandle, ioHandle->deviceHandle->hwInfo.edmaRxEventNum);
             EDMA3_DRV_clearErrorBits(ioHandle->edmaHandle, ioHandle->deviceHandle->hwInfo.edmaTxEventNum);
        }
        else
        {
            ioHandle->pspRequest.status = PSP_SOK;
            ioHandle->pspRequest.actualSize = ioHandle->pspRequest.size;
        }
        if (NULL !=  ioHandle->cbFxn)
        {
            (*ioHandle->cbFxn)(
                    (Ptr)ioHandle->deviceHandle->appHandle,
                    (Ptr)ioHandle->cbArg,
                    (&(ioHandle->pspRequest)),
                    ioHandle->mode
                    );
         }
         ioHandle->submitCount--;
         PAL_osListEnqueue(&ioHandle->queueFreeList,(Ptr)(ioHandle->dataPacket));
         if(TRUE == PAL_osList_NP_IsEmpty(&(ioHandle->queueReqList)))
         {
             ioHandle->dataPacket  = NULL;
             ioHandle->submitCount = 0;
             ioHandle->rxBytesIndex = 0;
             ioHandle->txBytesIndex = 0;
             if (PSP_MCBSP_OUTPUT == ioHandle->mode)
             {
                 if (TRUE == ioHandle->flush)
                 {
                     PAL_osSemGive(ioHandle->syncSem);
                     ioHandle->flush = FALSE;
                 }
             }
         }
         else
         {
             ioHandle->dataPacket = PAL_osList_NP_Dequeue(&(ioHandle->queueReqList));
             if(NULL != ioHandle->dataPacket)
             {
	                ddc_copyIOPtoReq(ioHandle->dataPacket,(&(ioHandle->pspRequest)));
                    ioHandle->rxBytesIndex = 0;
                    ioHandle->txBytesIndex = 0;
                    /*   Set Edma params */
                    DDC_SPIEdmaSetParamas(ioHandle);

                    /* Start edma */
                    EDMA3_DRV_enableTransfer(ioHandle->edmaHandle,ioHandle->deviceHandle->hwInfo.edmaTxEventNum,
                                             EDMA3_DRV_TRIG_MODE_EVENT);
                    EDMA3_DRV_enableTransfer(ioHandle->edmaHandle,ioHandle->deviceHandle->hwInfo.edmaRxEventNum,
                                             EDMA3_DRV_TRIG_MODE_EVENT);

             }
         }
    }while (falsewhile);
}




/**
 *  \fn static void ddc_edmaMcbspCallback(Uint tcc,
                                          EDMA3_RM_TccStatus status,
                                          Ptr data)
 *  \brief Application callback
 *
 *       This calls the registered application callback for the current
 *       receive request and processes the next request in queue.
 *
 *  \param      tcc     [IN]    tcc number of eDMA
 *  \param      status  [IN]    status of eDMA
 *  \param      data    [IN]    paramaters send to eDMA ISR (chan handler)
 *
 *  \return     void
 */

static void ddc_edmaMcbspCallback(Uint tcc, EDMA3_RM_TccStatus status, Ptr data)
{

    DDC_mcbspIoHandle       ioHandle        = NULL;
    Bool                    falsewhile      = FALSE;
    PSP_Result              result          = PSP_SOK;
    /*
    To remove warning, kept for future usage
    */
    tcc = tcc;
    status = status;
    if(NULL != data)
    {
        data = data;
    }
    do
    {
        falsewhile      = FALSE;

        ioHandle = (DDC_mcbspIoHandle)data;

        if(NULL == ioHandle)
        {
            break;
        }

        if(TRUE==PAL_osList_NP_IsEmpty(&(ioHandle->queueFloatingList)))
        {

            /*This cannot happen, if it happens then it is a sprurios one*/
            break;
        }

        /*
        for DMA transactions all the floating packets are managed in queueFloatingList
        Now pop out the packet from the queue and complete it
        */
        ioHandle->tempPacket = PAL_osList_NP_Dequeue(&(ioHandle->queueFloatingList));

#ifdef MCBSP_INSTRUMENTATION_ENABLED
        if( PSP_MCBSP_OUTPUT == ioHandle->mode){
            PSP_LOG_EVENT(
                &DVTEvent_Log,
                "McBSP",
                PSP_DVT_DESC(
                        PSP_DVT_eDATA_SND_END,
                        PSP_DVT_dCHANNEL_ID_INST,
                        PSP_DVT_dPACKET_ID,
                        PSP_DVT_dNONE
                        ),
                CHANNELID_INSTANCEID(
                    ioHandle->mode,
                    ioHandle->deviceHandle->InstNum),
                ioHandle->tempPacket->appData
                );
        }
        else if( PSP_MCBSP_INPUT == ioHandle->mode){
            PSP_LOG_EVENT(
                &DVTEvent_Log,
                "McBSP",
                PSP_DVT_DESC(
                        PSP_DVT_eRCV_END,
                        PSP_DVT_dCHANNEL_ID_INST,
                        PSP_DVT_dPACKET_ID,
                        PSP_DVT_dNONE
                        ),
                CHANNELID_INSTANCEID(
                    ioHandle->mode,
                    ioHandle->deviceHandle->InstNum),
                ioHandle->tempPacket->appData
                );
        }
#endif /* MCBSP_INSTRUMENTATION_ENABLED */

        if ( EDMA3_RM_XFER_COMPLETE != status)
        {
            /* Clear the EDMA Error bits */
            ioHandle->tempPacket->status  =  PSP_MCBSP_DMA_ERROR;
            /*
             There is a data transfer error in EDMA transfer. Return actualSize
             equal to 0 represent that there is an error in the packet transfer
            */
            ioHandle->tempPacket->actualSize = 0;
            EDMA3_DRV_clearErrorBits(ioHandle->edmaHandle, ioHandle->xferChan);
        }
        else
        {
            ioHandle->tempPacket->status  =  PSP_SOK;
            ioHandle->tempPacket->actualSize = \
                (Uint32) (ioHandle->tempPacket->size - ioHandle->currentDataSize);
        }

        ioHandle->submitCount--;

        if (FALSE == ioHandle->paused)
        {
            do
            {
                falsewhile = FALSE;
                /*
                Now that we have done with the last data packet - we check if
                next packet is available for transmission. Even if we are not
                having a valid packet - previous programming of
                linkchannels (designed in such a way) to send out null buffer
                Also note that ddc_isValidPacket function will set the
                ioHandle->dataPacket with next (current now) data packet from
                queue     */
                if ((PSP_MCBSP_VALID == (ddc_isValidPacket(ioHandle))))
                {
                    /*
                    * This is a valid data request. Call function to process it.
                    * If DMA cannot be programmed properly with request contents,
                    *  abort all queued
                    * up requests and put state-machine into reset.
                    */
                    result = ddc_edmaProcessPkt(ioHandle, ioHandle->dataPacket);

                    if (PSP_MCBSP_ABORTED == result)
                    {
                        ddc_cancelAndAbortAllIo(ioHandle);
                        break;
                    }
                }
                else
                {
                    /*
                    the following function will assign loopjob buffer to free
                    param set Also the control will come here two times for last
                    packet and last but one packet (obviously for more than one
                    packet xfer) For last but packet callback itself we will
                    come to know that there is no more packet pending and hence
                    we load loopjob info into next param set and for the last
                    packet callback will not do it again (for only one packet
                    scenario the following flag will take care not to update the
                    loopjob second time)
                    */

                    if( FALSE  == ioHandle->loopjobUpdatedinParamset)
                    {
                        ioHandle->loopjobUpdatedinParamset = TRUE;

                        GetNextIndex(ioHandle->nextLinkParamSetToBeUpdated );

                        ddc_updateDataPacketTolinkParams(ioHandle, NULL);
                    }

                }

            }while(falsewhile);

        }
        else
        {
            /*
            Control will come over here when either of receive or transmit state
            machine stop command is issued for receive or transmit channel.
            If the nextFlag is already set indicates that its time to
            reset the state machines and disable the edma transfer
            */
            if (TRUE == ioHandle->nextFlag)
            {
                ioHandle->nextFlag = FALSE;
            }
            else
            {
                /*
                For the first time when the stop port command is issued we will
                have one more packet linked with the transfer channel. So we
                will not stop at this time. We link the packet with NULL loopjob
                buffer and set a nextFlag to TRUE. We will wait for another
                callback(indicating the io request is complete) to disable EDMA
                trasnfer and reset state  machines
                */
                ioHandle->nextFlag = TRUE;

                GetNextIndex(ioHandle->nextLinkParamSetToBeUpdated );

                ddc_updateDataPacketTolinkParams(ioHandle, NULL);
            }
        }

        ioHandle->isTempPacketValid = TRUE;

        ddc_completeCurrentIO(ioHandle);

        if (PSP_MCBSP_ABORTED == result)
        {
            ddc_cancelAndAbortAllIo(ioHandle);
        }

        /*
        If there are no further request pending to serve and flush command is
        issued already give/post the semaphore take at the flush time.
        */
        if (0 == ioHandle->submitCount)
        {
            if (PSP_MCBSP_OUTPUT == ioHandle->mode)
            {
                if (TRUE == ioHandle->flush)
                {
                    PAL_osSemGive(ioHandle->syncSem);
                    ioHandle->flush = FALSE;
                }
            }
        }
    }while (falsewhile);

    return;

}


/**
 * \fn      static PAL_Result ddc_isValidPacket(DDC_mcbspIoHandle ioHandle)
 * \brief   Check for Valid packet
 *
 *    This checks is the next request in queue is data request. If this is an
 *    abort or flush request, it calls the appropriate function to deal with
 *    it.
 *
 * \param    ioHandle   [IN]     Channel handle
 *
 * \return   IOM_COMPLETED,   if request is Recieved/Transmitted completed
 *           PSP_MCBSP_VALID, if request is not completed

 */
static PAL_Result ddc_isValidPacket(DDC_mcbspIoHandle ioHandle)
{
    DDC_mcbspDeviceHandle   deviceHandle        = NULL;
    PAL_Result              result              = 0;
    Bool                    falsewhile          = FALSE;


    do
    {
        falsewhile      = FALSE;
        if (NULL == ioHandle)
        {
            result = PSP_E_INVAL_PARAM;
            break;
        }
        deviceHandle = ioHandle->deviceHandle;
        if (NULL == deviceHandle)
        {
            result = PSP_E_INVAL_PARAM;
            break;

        }


        /*
        * Look for new request to be processed
        */

        if(TRUE==PAL_osList_NP_IsEmpty(&(ioHandle->queueReqList)))
        {

            /*
            * we dont have any packet in queue
            * stop xmt/rct sm before giving error
            */
            ioHandle->dataPacket = NULL;

            /*
            for DMA mode only status is enought to program the loopjob buffer
            */
            result = PSP_MCBSP_INVALID;
        }
        else  /*if(TRUE==PAL_osList_NP_IsEmpty(&(ioHandle->queueReqList)))*/
        {

            result = PSP_MCBSP_VALID;


            ioHandle->dataPacket = PAL_osList_NP_Dequeue(&(ioHandle->queueReqList));


            if(NULL != ioHandle->dataPacket)
            {
                /*
                we have a valid packet to process next
                */

                /*
                in int mode - current packet will be only one hence  we can use
                ioHandle->dataPacket for floating / current packet
                */

                ioHandle->currentDataSize    = (Uint16)
                                               (ioHandle->dataPacket->size);
                ioHandle->userDataBufferSize = (Uint32)
                                               (ioHandle->dataPacket->size);
                result = PSP_MCBSP_VALID;


                /*
                Additional to assigning the important parameters as above
                for DMA mode , we will have max 2 packets floating and hence
                we use a activequue to manage this
                */
                PAL_osList_NP_Enqueue(&(ioHandle->queueFloatingList),
                                      ioHandle->dataPacket);


            }

        } /*if(TRUE==PAL_osList_NP_IsEmpty(&(ioHandle->queueReqList)))*/


    }while(falsewhile);

    return result;
}


/**
 *  \fn static void ddc_abortReset(DDC_mcbspIoHandle ioHandle)
 *
 *  \brief Abort Requests
 *
 *   Aborts queued up i/o request requests.
 *
 *  \param    ioHandle   [IN]   Channel handle
 *
 *  \return   void
 */
static void ddc_abortReset(DDC_mcbspIoHandle ioHandle)
{
    DDC_mcbspIOP      *tmpPacket    = NULL;
    Uint32            imask         = 0;
    PSP_mcbspReq      pspRequest    = {0};
    Bool              falsewhile    = FALSE;
    Uint32            localcCnt     = 0;
    Bool              firstReq      = FALSE;

    do
    {
        falsewhile = FALSE;
        if (NULL == ioHandle)
        {
            break;
        }

        if(PSP_McBSP_McBSP == ioHandle->deviceHandle->currentMode)
        {


            PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &imask);

            /*
            * This function services both transmit and receive channels. Define
            * critical section to avoid corruption of local variables.
            * CRITICAL SECTION: begins!
            */

            /* Disable current EDMA transfer */
            EDMA3_DRV_disableTransfer (
                            ioHandle->edmaHandle,
                            ioHandle->xferChan,
                            EDMA3_DRV_TRIG_MODE_EVENT
                            );

            if (PSP_MCBSP_OUTPUT == ioHandle->mode)
            {
                /* Poll for TXEMPTY */
                while (CSL_FEXT(ioHandle->deviceHandle->hwInfo.regs->SPCR,
                                MCBSP_SPCR_XEMPTY))
                {
                }
            }

            /* Reset McBSP */
            PSP_mcbspIoctl(ioHandle,PSP_CTRL_McBSP_STOP, NULL, NULL);
            /* Get the current value of cCnt for the actual data transfered
             */
            EDMA3_DRV_getPaRAMField (
                            ioHandle->edmaHandle,
                            ioHandle->xferChan,
                            EDMA3_DRV_PARAM_FIELD_CCNT,
                            &localcCnt
                            );

            /* Empty the floating queue. Aborted request is currently enqueued
             *   in the floating queue. Dequeue the floating request in EDMA
             *   param table and return the actual transfer element count
             */

            firstReq      = TRUE;

            while (TRUE !=  PAL_osList_NP_IsEmpty(&ioHandle->queueFloatingList))
            {
                tmpPacket = (DDC_mcbspIOP *)PAL_osList_NP_Dequeue(
                                                &ioHandle->queueFloatingList);

                PAL_osList_NP_Enqueue(&(ioHandle->queueFreeList),tmpPacket);

                /*
                 * if FLUSH cmd called for INPUT channel then
                 * status is equal to PSP_MCBSP_FLUSHED otherwise
                 * status is equal to PSP_MCBSP_ABORTED
                 */
                if((TRUE == ioHandle->flush) \
                        && (PSP_MCBSP_INPUT == ioHandle->mode))
                {
                    tmpPacket->status = PSP_MCBSP_FLUSHED;
                }
                else
                {
                    tmpPacket->status = PSP_MCBSP_ABORTED;
                }

                tmpPacket->actualSize = 0;

                /* Actual data transfer will be equal to size provided by
                 * application i.e.userDataBufferSize reduced by current
                 * value of cCnt * aCnt. bCnt is 1
                 */
                if (TRUE == firstReq )
                {
                    tmpPacket->actualSize = (Uint32)(tmpPacket->size - \
                                        (localcCnt * ioHandle->roundedWordWidth));
                    firstReq = FALSE;
                }

                ddc_copyIOPtoReq(tmpPacket, (PSP_mcbspReq *)&pspRequest);

                if(NULL != pspRequest.cbFxn)
                {
                    (*pspRequest.cbFxn)(
                            (Ptr)ioHandle->deviceHandle->appHandle,
                            (Ptr)ioHandle->cbArg,
                            &pspRequest,
                            ioHandle->mode
                            );
                }
                else
                {
                    if (NULL !=  ioHandle->cbFxn)
                    {
                        (*ioHandle->cbFxn)(
                                (Ptr)ioHandle->deviceHandle->appHandle,
                                (Ptr)ioHandle->cbArg,
                                &pspRequest,
                                ioHandle->mode
                                );
                    }
                    /*else
                    {
                        * callback function is null. Somewhere the function
                        * is corrupted. Break from this location
                    }*/
                }

                /* Since current request is aborted and respective callback is
                 * called decrease the submitcount by 1
                 */
                ioHandle->submitCount--;
            }


            /*Empty the pendList queue. Common for DMA/Interrupt Transactions*/
            while (TRUE != PAL_osList_NP_IsEmpty(&ioHandle->queueReqList))
            {

                tmpPacket = (DDC_mcbspIOP *)PAL_osList_NP_Dequeue(
                                                &ioHandle->queueReqList);

                PAL_osList_NP_Enqueue(&(ioHandle->queueFreeList),tmpPacket);

                /*
                 * if FLUSH cmd called for INPUT channel then
                 * status is equal to PSP_MCBSP_FLUSHED otherwise
                 * status is equal to PSP_MCBSP_ABORTED
                 */
                if((TRUE == ioHandle->flush) \
                        && (PSP_MCBSP_INPUT == ioHandle->mode))
                {
                    tmpPacket->status = PSP_MCBSP_FLUSHED;
                }
                else
                {
                    tmpPacket->status = PSP_MCBSP_ABORTED;
                }

                /* Since packet is aborted actual data size of transferred is 0*/
                tmpPacket->actualSize = 0;

                ddc_copyIOPtoReq(tmpPacket, (PSP_mcbspReq *)&pspRequest);

                if(NULL != pspRequest.cbFxn)
                {
                    (*pspRequest.cbFxn)(
                            (Ptr)ioHandle->deviceHandle->appHandle,
                            (Ptr)ioHandle->cbArg,
                            &pspRequest,
                            ioHandle->mode
                            );
                }
                else
                {
                    if (NULL !=  ioHandle->cbFxn)
                    {
                        (*ioHandle->cbFxn)(
                                (Ptr)ioHandle->deviceHandle->appHandle,
                                (Ptr)ioHandle->cbArg,
                                &pspRequest,
                                ioHandle->mode
                                );
                    }
                /*else
                    {
                     callback function is null. Somewhere the function
                     is corrupted. Break from this location
                }*/

                }

                /* Since current request is aborted and respective callback is
                 * called decrease the submitcount by 1
                 */
                ioHandle->submitCount--;
            }

            /* As EDMA is disabled it might have thrown an error and set error
             *   bit Clear the error bit to enable the transfer again
             */

            EDMA3_DRV_clearErrorBits(
                            ioHandle->edmaHandle,
                            ioHandle->xferChan
                            );


            /* Configure the EDMA channel and EDMA param tables with
             * intialization configuration as they are configured at the create
             time.
             */
            ddc_edmaChannelPaRAMsetup(ioHandle);

            /* Disable current EDMA transfer */
            EDMA3_DRV_enableTransfer (
                            ioHandle->edmaHandle,
                            ioHandle->xferChan,
                            EDMA3_DRV_TRIG_MODE_EVENT
                            );


            ioHandle->nextLinkParamSetToBeUpdated = 0;
            ioHandle->loopjobUpdatedinParamset = TRUE;

            /* Enable McBSP*/
            PSP_mcbspIoctl(ioHandle,PSP_CTRL_McBSP_START, NULL, NULL);
            /*
            * CRITICAL SECTION: ends!
            */
            PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, imask);
      }
      else          /* spi mode */
      {
            PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &imask);


            if(PSP_OPMODE_DMAINTERRUPT == ioHandle->deviceHandle->opMode)
            {

                /* Disable current EDMA transfer */
                EDMA3_DRV_disableTransfer (
                                ioHandle->edmaHandle,
                                ioHandle->deviceHandle->hwInfo.edmaTxEventNum,
                                EDMA3_DRV_TRIG_MODE_EVENT
                                );

                /* Disable current EDMA transfer */
                EDMA3_DRV_disableTransfer (
                                ioHandle->edmaHandle,
                                ioHandle->deviceHandle->hwInfo.edmaRxEventNum,
                                EDMA3_DRV_TRIG_MODE_EVENT
                                );

                while (CSL_FEXT(ioHandle->deviceHandle->hwInfo.regs->SPCR,
                                MCBSP_SPCR_XEMPTY))
                {
                }

                /* Reset SPI */

               ioHandle->deviceHandle->hwInfo.regs->SPCR &=~(0x10001);

                /* Get the current value of cCnt for the actual data transfered
                 */

                /* Get the current value of cCnt for the actual data transfered
                 */

                EDMA3_DRV_getPaRAMField (
                                ioHandle->edmaHandle,
                                ioHandle->deviceHandle->hwInfo.edmaRxEventNum,
                                EDMA3_DRV_PARAM_FIELD_CCNT,
                                &localcCnt
                                );

                ioHandle->pspRequest.actualSize = (Uint32)(ioHandle->pspRequest.size - \
                                    (localcCnt * ioHandle->roundedWordWidth));
                /* Clear error bits if any ..*/


                EDMA3_DRV_clearErrorBits(
                                ioHandle->edmaHandle,
                                ioHandle->deviceHandle->hwInfo.edmaRxEventNum
                                );
                EDMA3_DRV_clearErrorBits(
                                ioHandle->edmaHandle,
                                ioHandle->deviceHandle->hwInfo.edmaTxEventNum
                                );

            }
            else if(PSP_OPMODE_INTERRUPT == ioHandle->deviceHandle->opMode)
            {
               PAL_osEvtDisable (ioHandle->deviceHandle->hwInfo.cpuTxEventNum);
               PAL_osEvtDisable (ioHandle->deviceHandle->hwInfo.cpuTxEventNum);

                while (CSL_FEXT(ioHandle->deviceHandle->hwInfo.regs->SPCR,
                                MCBSP_SPCR_XEMPTY))
                {
                }

                /* Reset SPI */

               ioHandle->deviceHandle->hwInfo.regs->SPCR &=~(0x10001);

                if(SpiTransiveBuff == ((PSP_mcbspTransParam*)
                                      (ioHandle->pspRequest.addr))->receiveAddr)
                {
                    /* Tx operation */
                   ioHandle->pspRequest.actualSize = ioHandle->txBytesIndex;
                }
                else
                {
                   ioHandle->pspRequest.actualSize = ioHandle->rxBytesIndex;

                }

            }

            if((TRUE == ioHandle->flush) \
                      && (PSP_MCBSP_INPUT == ioHandle->mode))
            {
                    ioHandle->pspRequest.status = PSP_MCBSP_FLUSHED;
            }
            else
            {
                    ioHandle->pspRequest.status = PSP_MCBSP_ABORTED;
            }

            if(NULL != ioHandle->pspRequest.cbFxn)
            {
                (*ioHandle->pspRequest.cbFxn)(
                        (Ptr)ioHandle->deviceHandle->appHandle,
                        (Ptr)ioHandle->cbArg,
                        (&(ioHandle->pspRequest)),
                        ioHandle->mode
                        );
            }
            else
            {
                if (NULL !=  ioHandle->cbFxn)
                {
                    (*ioHandle->cbFxn)(
                            (Ptr)ioHandle->deviceHandle->appHandle,
                            (Ptr)ioHandle->cbArg,
                            (&(ioHandle->pspRequest)),
                            ioHandle->mode
                            );
                }

            }
            ioHandle->dataPacket = NULL;
            ioHandle->submitCount--;


            /*Empty the pendList queue. Common for DMA/Interrupt Transactions*/
            while (TRUE != PAL_osList_NP_IsEmpty(&ioHandle->queueReqList))
            {

                tmpPacket = (DDC_mcbspIOP *)PAL_osList_NP_Dequeue(
                                                    &ioHandle->queueReqList);

                PAL_osList_NP_Enqueue(&(ioHandle->queueFreeList),tmpPacket);

                /*
                 * if FLUSH cmd called for INPUT channel then
                 * status is equal to PSP_MCBSP_FLUSHED otherwise
                 * status is equal to PSP_MCBSP_ABORTED
                 */
                if((TRUE == ioHandle->flush) \
                        && (PSP_MCBSP_INPUT == ioHandle->mode))
                {
                    tmpPacket->status = PSP_MCBSP_FLUSHED;
                }
                else
                {
                    tmpPacket->status = PSP_MCBSP_ABORTED;
                }

                /* Since packet is aborted actual data size of transferred is 0*/
                tmpPacket->actualSize = 0;

                ddc_copyIOPtoReq(tmpPacket, (PSP_mcbspReq *)&pspRequest);

                if(NULL != pspRequest.cbFxn)
                {
                    (*pspRequest.cbFxn)(
                            (Ptr)ioHandle->deviceHandle->appHandle,
                            (Ptr)ioHandle->cbArg,
                            (&(ioHandle->pspRequest)),
                            ioHandle->mode
                            );
                }
                else
                {
                    if (NULL !=  ioHandle->cbFxn)
                    {
                        (*ioHandle->cbFxn)(
                                (Ptr)ioHandle->deviceHandle->appHandle,
                                (Ptr)ioHandle->cbArg,
                                (&(ioHandle->pspRequest)),
                                ioHandle->mode
                                );
                    }
                /*else
                    {
                     callback function is null. Somewhere the function
                     is corrupted. Break from this location
                }*/

                }
                ioHandle->submitCount--;
             }
         ioHandle->deviceHandle->hwInfo.regs->SPCR |= (0x10001);
         PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, imask);
       }

    }while(falsewhile);
}


static PAL_Result DDC_SPIEdmaSetParamas(DDC_mcbspIoHandle ioHandle )
{

    PAL_Result      retVal           = PSP_SOK;
    Bool            falsewhile       = FALSE;
    Uint8           *TrnsAddr        = (((PSP_mcbspTransParam*)
                                       ioHandle->pspRequest.addr))->transAddr;
    Uint8           *RcvAddr         = (((PSP_mcbspTransParam*)
                                       ioHandle->pspRequest.addr))->receiveAddr;
    EDMA3_DRV_AddrMode  TrnsAddrMode = EDMA3_DRV_ADDR_MODE_INCR;
    EDMA3_DRV_AddrMode  RcvAddrMode  = EDMA3_DRV_ADDR_MODE_INCR;


    do
    {
            falsewhile  = FALSE;
            if(NULL == ioHandle)
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }

            if(NULL == RcvAddr)
            {
                RcvAddr     = &SpiTransiveBuff[0];
                RcvAddrMode = EDMA3_DRV_ADDR_MODE_FIFO;
            }
            else if( NULL == TrnsAddr )
            {
                TrnsAddr     = &SpiTransiveBuff[0];
                TrnsAddrMode = EDMA3_DRV_ADDR_MODE_FIFO;
            }
            if( ((Uint32)TrnsAddr %32 != NULL) || ((Uint32)RcvAddr % 32 != NULL) )
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }
            /* Move the data to physical memory and invalidate the cache line */

           PAL_osCacheInvalidate(
                        (PAL_osCacheMemAddrSpace)NULL,
                        (Uint32)(((PSP_mcbspTransParam*)(ioHandle->pspRequest.addr))->receiveAddr),
                        (Uint32)(ioHandle->pspRequest.size)
                        );


            PAL_osCacheFlush(
                        (PAL_osCacheMemAddrSpace)NULL,
                        (Uint32)(((PSP_mcbspTransParam*)(ioHandle->pspRequest.addr))->transAddr),
                        (Uint32)(ioHandle->pspRequest.size)
                        );


            if (PSP_SOK != EDMA3_DRV_setSrcParams(
                                        ioHandle->edmaHandle,
                                        (ioHandle->deviceHandle->hwInfo.edmaRxEventNum),
                                        (Uint32)(&(ioHandle->deviceHandle->hwInfo.regs->DRR)),
                                        EDMA3_DRV_ADDR_MODE_FIFO,
                                        EDMA3_DRV_W8BIT
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }


            if (PSP_SOK != EDMA3_DRV_setDestParams(
                                        ioHandle->edmaHandle,
                                        ioHandle->deviceHandle->hwInfo.edmaRxEventNum,
                                        (Uint32)((PSP_mcbspTransParam*)(ioHandle->pspRequest.addr))->receiveAddr,
                                        RcvAddrMode,
                                        EDMA3_DRV_W8BIT
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }

            if (PSP_SOK != EDMA3_DRV_setSrcIndex(
                                    ioHandle->edmaHandle,
                                    ioHandle->deviceHandle->hwInfo.edmaRxEventNum,
                                    0,
                                    0
                                    ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }


            if (PSP_SOK != EDMA3_DRV_setDestIndex(
                                        ioHandle->edmaHandle,
                                        ioHandle->deviceHandle->hwInfo.edmaRxEventNum,
                                        1,
                                        1
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }


/* For TX ...*/

            if (PSP_SOK != EDMA3_DRV_setSrcParams(
                                        ioHandle->edmaHandle,
                                        (ioHandle->deviceHandle->hwInfo.edmaTxEventNum),
                                        (Uint32)(((PSP_mcbspTransParam*)
                                        (ioHandle->pspRequest.addr))->transAddr),
                                        TrnsAddrMode,
                                        EDMA3_DRV_W8BIT
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }



            if (PSP_SOK != EDMA3_DRV_setDestParams(
                                        ioHandle->edmaHandle,
                                        (ioHandle->deviceHandle->hwInfo.edmaTxEventNum),
                                        (Uint32)(&(ioHandle->deviceHandle->hwInfo.regs->DXR)),
                                        EDMA3_DRV_ADDR_MODE_INCR,
                                        EDMA3_DRV_W8BIT
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }

            if (PSP_SOK != EDMA3_DRV_setSrcIndex(
                                    ioHandle->edmaHandle,
                                    ioHandle->deviceHandle->hwInfo.edmaTxEventNum,
                                    1,
                                    1
                                    ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }


            if (PSP_SOK != EDMA3_DRV_setDestIndex(
                                        ioHandle->edmaHandle,
                                        ioHandle->deviceHandle->hwInfo.edmaTxEventNum,
                                        0,
                                        0
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }
            if(EDMA3_DRV_setTransferParams (ioHandle->edmaHandle,
                                            ioHandle->deviceHandle->hwInfo.edmaTxEventNum,
                                            1u,
                                            (ioHandle->pspRequest.size),
                                            1u,
                                            1,
                                            EDMA3_DRV_SYNC_A) !=  PAL_SOK )
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }

            if(EDMA3_DRV_setTransferParams (ioHandle->edmaHandle,
                                            ioHandle->deviceHandle->hwInfo.edmaRxEventNum,
                                            1,
                                            (ioHandle->pspRequest.size),
                                            1u,
                                            1,
                                            EDMA3_DRV_SYNC_A) !=  PAL_SOK )
            {
                retVal = PSP_E_INVAL_PARAM;
                break;

            }

            if(EDMA3_DRV_setOptField (ioHandle->edmaHandle,
                           ioHandle->deviceHandle->hwInfo.edmaTxEventNum,
                           EDMA3_DRV_OPT_FIELD_TCINTEN,
                           1u) !=  PAL_SOK )
            {

                retVal = PSP_E_INVAL_PARAM;
                break;

            }
            if (PSP_SOK != EDMA3_DRV_setOptField(
                                        ioHandle->edmaHandle,
                                        ioHandle->deviceHandle->hwInfo.edmaTxEventNum,
                                        EDMA3_DRV_OPT_FIELD_TCC,
                                        ioHandle->deviceHandle->hwInfo.edmaTxEventNum
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }

            if(EDMA3_DRV_setOptField (ioHandle->edmaHandle,
                           ioHandle->deviceHandle->hwInfo.edmaRxEventNum,
                           EDMA3_DRV_OPT_FIELD_TCINTEN,
                           1u) !=  PAL_SOK )
            {

                retVal = PSP_E_INVAL_PARAM;
                break;

            }
            if (PSP_SOK != EDMA3_DRV_setOptField(
                                        ioHandle->edmaHandle,
                                        ioHandle->deviceHandle->hwInfo.edmaRxEventNum,
                                        EDMA3_DRV_OPT_FIELD_TCC,
                                        ioHandle->deviceHandle->hwInfo.edmaRxEventNum
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }






   }while(falsewhile);

    return retVal;


}



static PAL_Result DDC_McBspSpiSetupEdmaInit( DDC_mcbspIoHandle ioHandle )
{


    PAL_Result      retVal      = PSP_SOK;
    Bool            falsewhile  = FALSE;

    do
    {
        falsewhile  = FALSE;
        if(NULL == ioHandle)
        {
            retVal = PSP_E_INVAL_PARAM;
            break;
        }

            if (PSP_SOK != EDMA3_DRV_requestChannel (
                                                ioHandle->edmaHandle,
                                                &(ioHandle->deviceHandle->hwInfo.edmaTxEventNum),
                                                &(ioHandle->deviceHandle->hwInfo.edmaTxEventNum),
                                                (EDMA3_RM_EventQueue)0,
                                                &ddc_edmaMcbspSPITXCallback,
                                                ioHandle))
            {
                retVal = PSP_E_INVAL_PARAM;
                 break;
            }
            else
            {

               if (PSP_SOK != EDMA3_DRV_requestChannel (
                                                   ioHandle->edmaHandle,
                                                   &(ioHandle->deviceHandle->hwInfo.edmaRxEventNum),
                                                   &(ioHandle->deviceHandle->hwInfo.edmaRxEventNum),
                                                   (EDMA3_RM_EventQueue)0,
                                                   &ddc_edmaMcbspSPIRXCallback,
                                                   ioHandle))
               {

                    EDMA3_DRV_freeChannel(ioHandle->edmaHandle,ioHandle->deviceHandle->hwInfo.edmaTxEventNum);
                    retVal = PSP_E_INVAL_PARAM;
                    break;
                }

            }



   }while(falsewhile);

    return retVal;

}








/*
* This function is called durinn psp_mcbspopen (during init)
* This function does the following
*       1)Request the Xfer (in/out depends on mode) channel to EDMA driver for
*        this peripheral;
*       2)requests the link channels for  linking (exclusive for each direction
*        (tx / rx)
*       3)set up the param set of main xfer channel (for loopjob transfers)
*       4)set up the param sets of all link channel (for loopjob transfers)
*       5)link the last link channel to itself (this channel will be linked to
*         main xfer channel later
*         and to supply the loopjob xfer information infinely this is done so;
*         other link channels are setup
*         here just for completion sake
*
*/
static PAL_Result ddc_setupEDMADuringInit( DDC_mcbspIoHandle ioHandle )
{

    Uint32          linkCnt     = 0;
    PAL_Result      retVal      = PSP_SOK;
    Bool            falsewhile  = FALSE;
    Uint32          i               = 0;
    Uint32          edmaChanNum     = 0;
    Uint32          reqTcc          = EDMA3_DRV_TCC_ANY;



    do
    {
        falsewhile  = FALSE;
        if(NULL == ioHandle)
        {
            retVal = PSP_E_INVAL_PARAM;
            break;
        }

        if (PSP_SOK != EDMA3_DRV_requestChannel (
                                            ioHandle->edmaHandle,
                                            &ioHandle->xferChan,
                                            &ioHandle->tcc,
                                            (EDMA3_RM_EventQueue)0,
                                            ioHandle->edmaCallback,
                                            ioHandle))
        {
            retVal = PSP_E_INVAL_PARAM;
            break;
        }

        /*
        *Acquire the  PaRAM entries used for EDMA transfers linking
        */
        for (linkCnt = 0; linkCnt < MAXLINKCNT; linkCnt++)
        {
            /*For requesting for a PaRam set*/
            edmaChanNum = EDMA3_DRV_LINK_CHANNEL;
            reqTcc = EDMA3_DRV_TCC_ANY;
            retVal = EDMA3_DRV_requestChannel (
                                        ioHandle->edmaHandle,
                                        &edmaChanNum,
                                        &reqTcc,
                                        (EDMA3_RM_EventQueue)0,
                                        ioHandle->edmaCallback,
                                        NULL
                                        );
            if (PSP_SOK == retVal )
            {
                ioHandle->pramTbl[linkCnt] = edmaChanNum;
            }
            else
            {
                /* free the Synchonized EDMA channel */
                EDMA3_DRV_freeChannel (ioHandle->edmaHandle, ioHandle->xferChan);

                /* free the already allocated PaRAM entries */
                for (i = 0; i <linkCnt; i++)
                {
                      EDMA3_DRV_freeChannel (ioHandle->edmaHandle, ioHandle->pramTbl[i]);
                }
                retVal = PSP_E_RESOURCES;
                break;
            }
        }

        /*
        * Configure the McBSP synchonized EDMA channel PaRAM entries
        * Configure source and destination address / modes
        * first - main xfer channel
        */

        ddc_edmaChannelPaRAMsetup(ioHandle);

    }while(falsewhile);

    return retVal;

}

/**
 *  \fn     static PAL_Result ddc_edmaChannelPaRAMsetup(DDC_mcbspIoHandle ioHandle)
 *
 *  \brief  setup edma chnnel's parameter structure.
 *          Configure the McBSP synchonized EDMA channel PaRAM entries
 *          Configure source and destination address / modes
 *
 *  \param    ioHandle   [IN]   Channel handle
 *
 *  \return   PSP_SOK               if setup successful
 *            PSP_E_INVAL_PARAM     if configuration has any error
 */

static PAL_Result ddc_edmaChannelPaRAMsetup(DDC_mcbspIoHandle ioHandle)
{

    Uint32          linkCnt         = 0;
    PAL_Result      retVal          = PSP_SOK;
    Bool            falsewhile      = FALSE;

    do
    {
        falsewhile  = FALSE;
        if(NULL == ioHandle)
        {
            retVal = PSP_E_INVAL_PARAM;
            break;
        }

        /*
        * Configure the McBSP synchonized EDMA channel PaRAM entries
        * Configure source and destination address / modes
        * first - main xfer channel
        */

        /*
        following the explanation behind the EDMA configuration

        1) We will have n arrays of data (now it is loopjob buffer each of
                length x bytes)
        2) for each sync event we have to fetch the data from each array and
                write to each serilaiser
        3) and the no. bytes transferred in this case from each array will be
                roundedwordwidth/8
        4) as the loopjob buffer is contigous, we configure the counts as follows
            a) consider there are arrays of length roundedwordwidth/8 (aCnt)
            b) these arrays are located apart lengthofloopjob size  (bindex)
            c) there are 'no.of.serialiser' no. of such arrays   (bCnt)
            d)  (a) and (b) constitute one frame of EDMA transfer and it happens
                for one sync event in AB sync mode
            e) such frames have to be sent till we have buffer - ie
                    lengthofloopjob / (roundedwordwidth/8 )
                    and this constitutes  cCnt
            f)  address of buffer (rather offset from start of previous frame)
                 is roundedwordwidth/8      and this forms cIndex
        Please take some time to visualise the setup mentioned above to check
                with EDMa configuration

        The same kind of setup will be used for data transfers also and the
            change will be buffer pointer  and length of data buffer.
        */

        if (PSP_MCBSP_INPUT == ioHandle->mode )
        {

            if (PSP_SOK != EDMA3_DRV_setSrcParams(
                                        ioHandle->edmaHandle,
                                        ioHandle->xferChan,
                                        (Uint32)(&(ioHandle->deviceHandle->hwInfo.regs->DRR)),
                                        EDMA3_DRV_ADDR_MODE_INCR,
                                        EDMA3_DRV_W8BIT
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }


            if (PSP_SOK != EDMA3_DRV_setDestParams(
                                        ioHandle->edmaHandle,
                                        ioHandle->xferChan,
                                        (Uint32)ioHandle->loopJobBuffer,
                                        EDMA3_DRV_ADDR_MODE_INCR,
                                        EDMA3_DRV_W8BIT
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }

            if (PSP_SOK != EDMA3_DRV_setSrcIndex(
                                    ioHandle->edmaHandle,
                                    ioHandle->xferChan,
                                    0,
                                    0
                                    ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }


            if (PSP_SOK != EDMA3_DRV_setDestIndex(
                                        ioHandle->edmaHandle,
                                        ioHandle->xferChan,
                                        ioHandle->loopJobLength,
                                        ioHandle->roundedWordWidth
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }
        }
        else
        {

            if (PSP_SOK != EDMA3_DRV_setSrcParams(
                                        ioHandle->edmaHandle,
                                        ioHandle->xferChan,
                                        (Uint32)ioHandle->loopJobBuffer,
                                        EDMA3_DRV_ADDR_MODE_INCR,
                                        EDMA3_DRV_W8BIT
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }

            if (PSP_SOK != EDMA3_DRV_setDestParams(
                                        ioHandle->edmaHandle,
                                        ioHandle->xferChan,
                                        (Uint32)(&(ioHandle->deviceHandle->hwInfo.regs->DXR)),
                                        EDMA3_DRV_ADDR_MODE_INCR,
                                        EDMA3_DRV_W8BIT
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }

            if (PSP_SOK != EDMA3_DRV_setSrcIndex(
                                    ioHandle->edmaHandle,
                                    ioHandle->xferChan,
                                    ioHandle->loopJobLength,
                                    ioHandle->roundedWordWidth
                                    ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }


            if (PSP_SOK != EDMA3_DRV_setDestIndex(
                                        ioHandle->edmaHandle,
                                        ioHandle->xferChan,
                                        0,
                                        0
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }
        }

        /*
        for acnt, bcnt, ccnt, bidx , cidx details please refer comment above
        */
        if (PSP_SOK != EDMA3_DRV_setTransferParams(
                                    ioHandle->edmaHandle,
                                    ioHandle->xferChan,
                                    ioHandle->roundedWordWidth,
                                    1,/* ioHandle->noOfSerAllocated,*/
                                    ioHandle->loopJobLength/(ioHandle->roundedWordWidth),
                                    1,/* ioHandle->noOfSerAllocated,*/
                                    EDMA3_DRV_SYNC_AB
                                    ))
        {
            retVal = PSP_E_INVAL_PARAM;
            break;
        }

        if (PSP_SOK != EDMA3_DRV_setOptField(
                                    ioHandle->edmaHandle,
                                    ioHandle->xferChan,
                                    EDMA3_DRV_OPT_FIELD_TCINTEN,
                                    FALSE
                                    ))
        {
            retVal = PSP_E_INVAL_PARAM;
            break;
        }

        if (PSP_SOK != EDMA3_DRV_setOptField(
                                    ioHandle->edmaHandle,
                                    ioHandle->xferChan,
                                    EDMA3_DRV_OPT_FIELD_TCC,
                                    ioHandle->tcc
                                    ))
        {
            retVal = PSP_E_INVAL_PARAM;
            break;
        }


        /*
        * Configure the link PaRAMs with the appropriate parameters
        * Though we configure all the link paramsets, we will be using
        * only one of them to link with main xfer channel
        */
        for (linkCnt = 0; linkCnt < MAXLINKCNT; linkCnt++)
        {

            if (PSP_MCBSP_INPUT == ioHandle->mode )
            {

                if (PSP_SOK != EDMA3_DRV_setSrcParams(
                                            ioHandle->edmaHandle,
                                            ioHandle->pramTbl[linkCnt],
                                            (Uint32)(&(ioHandle->deviceHandle->hwInfo.regs->DRR)),
                                            EDMA3_DRV_ADDR_MODE_INCR,
                                            EDMA3_DRV_W8BIT
                                            ))
                {
                    retVal = PSP_E_INVAL_PARAM;
                    break;
                }

                if (PSP_SOK != EDMA3_DRV_setDestParams(
                                            ioHandle->edmaHandle,
                                            ioHandle->pramTbl[linkCnt],
                                            (Uint32)ioHandle->loopJobBuffer,
                                            EDMA3_DRV_ADDR_MODE_INCR,
                                            EDMA3_DRV_W8BIT
                                            ))
                {
                    retVal = PSP_E_INVAL_PARAM;
                    break;
                }

                if (PSP_SOK != EDMA3_DRV_setSrcIndex(
                                        ioHandle->edmaHandle,
                                        ioHandle->pramTbl[linkCnt],
                                        0,
                                        0
                                        ))
                {
                    retVal = PSP_E_INVAL_PARAM;
                    break;
                }


                if (PSP_SOK != EDMA3_DRV_setDestIndex(
                                            ioHandle->edmaHandle,
                                            ioHandle->pramTbl[linkCnt],
                                            ioHandle->loopJobLength,
                                            ioHandle->roundedWordWidth
                                            ))
                {
                    retVal = PSP_E_INVAL_PARAM;
                    break;
                }
            }
            else
            {

                if (PSP_SOK != EDMA3_DRV_setSrcParams(
                                            ioHandle->edmaHandle,
                                            ioHandle->pramTbl[linkCnt],
                                            (Uint32)ioHandle->loopJobBuffer,
                                            EDMA3_DRV_ADDR_MODE_INCR,
                                            EDMA3_DRV_W8BIT
                                            ))
                {
                    retVal = PSP_E_INVAL_PARAM;
                    break;
                }

                if (PSP_SOK != EDMA3_DRV_setDestParams(
                                            ioHandle->edmaHandle,
                                            ioHandle->pramTbl[linkCnt],
                                            (Uint32)(&(ioHandle->deviceHandle->hwInfo.regs->DXR)),
                                            EDMA3_DRV_ADDR_MODE_INCR,
                                            EDMA3_DRV_W8BIT
                                            ))
                {
                    retVal = PSP_E_INVAL_PARAM;
                    break;
                }

                if (PSP_SOK != EDMA3_DRV_setSrcIndex(
                                        ioHandle->edmaHandle,
                                        ioHandle->pramTbl[linkCnt],
                                        ioHandle->loopJobLength,
                                        ioHandle->roundedWordWidth
                                        ))
                {
                    retVal = PSP_E_INVAL_PARAM;
                    break;
                }


                if (PSP_SOK != EDMA3_DRV_setDestIndex(
                                            ioHandle->edmaHandle,
                                            ioHandle->pramTbl[linkCnt],
                                            0,
                                            0
                                            ))
                {
                    retVal = PSP_E_INVAL_PARAM;
                    break;
                }
            }




            /*
            * set a count , b count and c count
            * for each sync even we need to transfer
            *  no. of serialisers * bit width (wordlen) / 8 bits
            */
            if (PSP_SOK != EDMA3_DRV_setTransferParams(
                                        ioHandle->edmaHandle,
                                        ioHandle->pramTbl[linkCnt],
                                        ioHandle->roundedWordWidth,
                                        1,/* ioHandle->noOfSerAllocated,*/
                                        ioHandle->loopJobLength/(ioHandle->roundedWordWidth),
                                        1,/* ioHandle->noOfSerAllocated,*/
                                        EDMA3_DRV_SYNC_AB
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }


            if (PSP_SOK != EDMA3_DRV_setOptField(
                                        ioHandle->edmaHandle,
                                        ioHandle->pramTbl[linkCnt],
                                        EDMA3_DRV_OPT_FIELD_TCINTEN,
                                        FALSE
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }



            if (PSP_SOK != EDMA3_DRV_setOptField(
                                    ioHandle->edmaHandle,
                                    ioHandle->pramTbl[linkCnt],
                                    EDMA3_DRV_OPT_FIELD_TCC,
                                    ioHandle->tcc
                                    ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }

            if(MAXLINKCNT-1 > linkCnt )
            {
                /*
                *link with next paramset
                *both param sets are  linked to each other to enable continous
                * xfer - either user audio data or loopjob buffer
                * MAXLINKCNT must be 2
                */

                retVal= EDMA3_DRV_linkChannel(
                                ioHandle->edmaHandle,
                                ioHandle->xferChan,
                                ioHandle->pramTbl[linkCnt+1]
                                );
                if (PSP_SOK == retVal)
                {
                    retVal= EDMA3_DRV_linkChannel(
                                    ioHandle->edmaHandle,
                                    ioHandle->pramTbl[linkCnt] ,
                                    ioHandle->pramTbl[linkCnt+1]
                                    );
                }

                if (PSP_SOK == retVal)
                {
                    retVal= EDMA3_DRV_linkChannel(
                                    ioHandle->edmaHandle,
                                    ioHandle->pramTbl[linkCnt+1] ,
                                    ioHandle->pramTbl[linkCnt+1]
                                    );
                }

                ioHandle->nextLinkParamSetToBeUpdated = 0;
                ioHandle->loopjobUpdatedinParamset  = TRUE;

            }

        }/*for loop for linkcount*/

    }while(falsewhile);

    return (retVal);
}

/**
 *  \fn     static void ddc_completeCurrentIO (DDC_mcbspIoHandle ioHandle )
 *
 *  \brief  This completes the current receive request start processing the
 *          next request in queue.
 *
 *  \param  ioHandle   [IN]   Channel handle
 *
 *  \return  void
 */
static void ddc_completeCurrentIO (DDC_mcbspIoHandle ioHandle )
{

    PSP_mcbspReq        pspRequest  ={0};
    DDC_mcbspIOP        *packet  = NULL;

    if(NULL != ioHandle)
    {

        if ( TRUE == ioHandle->isTempPacketValid )
        {
            packet = ioHandle->tempPacket;
            ioHandle->isTempPacketValid = FALSE;
        }
        else
        {
            packet = ioHandle->dataPacket;
        }

        /*
         * if FLUSH cmd called for OUTPUT channel then
         * set packet status is equal to PSP_MCBSP_FLUSHED
         */
        if((TRUE == ioHandle->flush) \
                && (PSP_MCBSP_OUTPUT == ioHandle->mode))
        {
            packet->status = PSP_MCBSP_FLUSHED;
        }

        /*
        Copy the ddc IOP packet information to ddc IO structure known to app layer
        */
        ddc_copyIOPtoReq( packet, (PSP_mcbspReq *)&pspRequest);

        /*
        sometimes the packet can have specific callback function
        */
        if(NULL != pspRequest.cbFxn)
        {
            (*pspRequest.cbFxn)(
                            (Ptr)ioHandle->deviceHandle->appHandle,
                            (Ptr)ioHandle->cbArg,
                            &pspRequest,
                            ioHandle->mode
                            );
        }
        else
        {
            /*
            otherwise call the application completion callback function
            registered with us during psp_mcbspopen
            */
            if (NULL != ioHandle->cbFxn)
            {
                (*ioHandle->cbFxn)(
                            (Ptr)ioHandle->deviceHandle->appHandle,
                            (Ptr)ioHandle->cbArg,
                            &pspRequest,
                            ioHandle->mode
                            );
            }

        }

        PAL_osList_NP_Enqueue(&(ioHandle->queueFreeList), packet);
        ioHandle->dataPacket = NULL;
        ioHandle->userDataBufferSize = 0;
    }/* if(NULL != ioHandle) */
}

/**
 *  \fn     static void ddc_cancelAndAbortAllIo ( DDC_mcbspIoHandle ioHandle )
 *
 *  \brief  This is going to complete the current request and abort
 *          all other reqest.
 *
 *  \param  ioHandle   [IN]   Channel handle
 *
 *  \return  void
 */
static void ddc_cancelAndAbortAllIo ( DDC_mcbspIoHandle ioHandle )
{

    if(NULL != ioHandle)
    {
        ioHandle->submitCount--;
        ioHandle->dataPacket->status = PSP_MCBSP_ABORTED;
        ddc_completeCurrentIO(ioHandle);

        ddc_abortReset(ioHandle);

    }/* if(NULL != ioHandle) */
}



/*ISR to handle receive interrupt in SPI mode*/
void ddc_mcbspRxISR(Ptr ddcIoObj)
{

    DDC_mcbspIoHandle ioHandle;
    Uint32  tempBuffer = 0;
    Uint8   *appRxBuffer;
    Uint8   *appTxBuffer;

    ddcIoObj = ddcIoObj; /* To remove the warning  */
    ioHandle = &(ddcMcbspPort->txIo);

    appRxBuffer = ((PSP_mcbspTransParam*)(ioHandle->pspRequest.addr))->receiveAddr;
    appTxBuffer = ((PSP_mcbspTransParam*)(ioHandle->pspRequest.addr))->transAddr;

    /*check if receive buffe is null, if so read the data to local buffer
    else write to application recieve buffer from mcBSP(SPI) rx buffer*/
    if(NULL == appRxBuffer)
    {
        tempBuffer = ioHandle->deviceHandle->hwInfo.regs->DRR;
        tempBuffer = tempBuffer;
		ioHandle->rxBytesIndex++;
    }
    else
    {
       *(appRxBuffer + ioHandle->rxBytesIndex++)
          = ioHandle->deviceHandle->hwInfo.regs->DRR;
    }


    if ( ioHandle->txBytesIndex < ioHandle->pspRequest.size)
    {
        while( 0x20000 != (0x20000&ioHandle->deviceHandle->hwInfo.regs->SPCR));

        if(NULL == appTxBuffer)
        {
            ioHandle->deviceHandle->hwInfo.regs->DXR = (Uint8)tempBuffer;
			ioHandle->txBytesIndex++;
        }
        else
        {
           ioHandle->deviceHandle->hwInfo.regs->DXR = (Uint8)
               *(appTxBuffer + ioHandle->txBytesIndex++);

        }
     }


    /* check if all bytes have been processed. If so call the GIO callback
      otherwise do nothing waiting for next interrupt */


     if ( ioHandle->rxBytesIndex == ioHandle->pspRequest.size)
     {

        /*Done with IO disable SPI engine*/
        ioHandle->deviceHandle->hwInfo.regs->SPCR &= ~(0x10001 | (1<<22));
        PAL_osEvtEnable (ioHandle->deviceHandle->hwInfo.cpuTxEventNum);

        ioHandle->pspRequest.status = PSP_SOK;
        ioHandle->pspRequest.actualSize = ioHandle->pspRequest.size;

        if (NULL !=  ioHandle->cbFxn)
        {
            (*ioHandle->cbFxn)(
                    (Ptr)ioHandle->deviceHandle->appHandle,
                    (Ptr)ioHandle->cbArg,
                    &(ioHandle->pspRequest),
                    ioHandle->mode
                    );
        }
        PAL_osListEnqueue(&ioHandle->queueFreeList,(Ptr)(ioHandle->dataPacket));
        ioHandle->submitCount--;
        if(TRUE == PAL_osList_NP_IsEmpty(&(ioHandle->queueReqList)))
        {
            ioHandle->dataPacket = NULL;
            ioHandle->submitCount = 0;
            if (PSP_MCBSP_OUTPUT == ioHandle->mode)
            {
                if (TRUE == ioHandle->flush)
                {
                    PAL_osSemGive(ioHandle->syncSem);
                    ioHandle->flush = FALSE;
                }
            }
        }
        else
        {
            ioHandle->dataPacket = PAL_osList_NP_Dequeue(
                                            &(ioHandle->queueReqList));
            if(NULL != ioHandle->dataPacket)
            {
                ddc_copyIOPtoReq(ioHandle->dataPacket,
                                 (PSP_mcbspReq *)&(ioHandle->pspRequest));
                ioHandle->rxBytesIndex = 0;
                ioHandle->txBytesIndex = 0;
            }
        }
     }

}

/*ISR to handle transmit interrupt in SPU mode*/
void ddc_mcbspTxISR(Ptr ddcIoObj)
{
    DDC_mcbspIoHandle ioHandle;
    Uint32  tempBuffer = 0xFF;
    Uint8   *appBuffer;

    ddcIoObj = ddcIoObj; /* To remove the warning  */
    ioHandle = &(ddcMcbspPort->txIo);
    appBuffer = ((PSP_mcbspTransParam*)(ioHandle->pspRequest.addr))->transAddr;

   /*check if the buffer is free*/

   /*update the size of bytes*/
    if ( ioHandle->txBytesIndex < ioHandle->pspRequest.size)
    {
        if(NULL == appBuffer)
        {
            ioHandle->deviceHandle->hwInfo.regs->DXR = (Uint8)tempBuffer;
			ioHandle->txBytesIndex++;
        }
        else
        {
           ioHandle->deviceHandle->hwInfo.regs->DXR = (Uint8)
               *(appBuffer + ioHandle->txBytesIndex);
            ioHandle->txBytesIndex++;
        }

       if(ioHandle->txBytesIndex == 2)
       {

	       /*Disable Tx interrupt from now on and will fill up the tx
	       buffer in rx ISR itself*/
	       PAL_osEvtDisable (ioHandle->deviceHandle->hwInfo.cpuTxEventNum);
       }

    }
}







#ifdef GBL_INT

/**
 *  \fn     void ddc_mcbspISR (Ptr ddcIoObj)
 *
 *  \brief  The local routine checks various interrupts generated and serve the
 *          particular service routine.
 *
 *  \param  ddcIoObj   [IN]   DDC layer IO structure
 *
 *  \return  void
 */
void ddc_mcbspISR (Ptr ddcIoObj)
{
    Int32       status = 0;
    DDC_mcbspIoObj  *mcbspObj = NULL;
    if(NULL != ddcIoObj)
    {
        mcbspObj = (DDC_mcbspIoObj *)ddcIoObj;
    }
    if(NULL != mcbspObj)
    {
        if(NULL != mcbspObj->gblErrCbk)
        {
            (*mcbspObj->gblErrCbk)(status);
        }
    }
    return;
}

#endif
/**
 *  \fn     static void ddc_copyReqtoIOP (PSP_mcbspReq * const pspRequest,
 *                                        DDC_mcbspIOP *const ioPacket)
 *  \brief  DDA to DDC layer copy of sturctures
 *
 *  Copies data from request structure provided by DDA layer to
 *  meaningful IO structure at  DDC layer.
 *
 *  \param    pspRequest       [IN]    Porinter to DDA layer reuqest structuer
 *  \param    ioPacket         [IN]    Porinter to DDC layer IO structuer
 *  \return   void
 */
static void ddc_copyReqtoIOP (PSP_mcbspReq * const pspRequest,
                              DDC_mcbspIOP *const ioPacket)
{

    if ((NULL != pspRequest ) && (NULL != ioPacket))
    {
        ioPacket->addr          = pspRequest->addr;
        ioPacket->size          = pspRequest->size;
        ioPacket->actualSize    = pspRequest->actualSize;
        ioPacket->status        = pspRequest->status;
        ioPacket->appCbk        = pspRequest->cbFxn;
        ioPacket->appData       = pspRequest->appPtr;
    }
}

/**
 *  \fn     static void ddc_copyIOPtoReq (DDC_mcbspIOP *const ioPacket,
 *                                        PSP_mcbspReq *const pspRequest)
 *
 *  \brief  Restore data from DDC
 *
 *          Restores the data from DDC layer request structure to
 *          DDA understandable psp request structure.
 *
 *  \param    ioPacket         [IN]    Porinter to DDC layer IO structuer
 *  \param    pspRequest       [IN]    Porinter to DDA layer reuqest structuer
 *
 *  \return   void
 */
static void ddc_copyIOPtoReq (DDC_mcbspIOP *const ioPacket,
                              PSP_mcbspReq *const pspRequest)
{
    if ((NULL != pspRequest ) && (NULL != ioPacket))
    {
        pspRequest->addr        = ioPacket->addr;
        pspRequest->size        = ioPacket->size;
        pspRequest->actualSize  = ioPacket->actualSize;
        pspRequest->status      = ioPacket->status;
        pspRequest->cbFxn       = ioPacket->appCbk;
        pspRequest->appPtr      = ioPacket->appData;
    }

}
