/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/
/**
 *  \file    ddc_mcbsp.h
 *
 *  \brief   DDC defined for a McBSP controller
 *
 *  This file defines channel and port objects for the i/o processing.
 *  This file contains definitions for a McBSP DDC. This file depends
 *  on some definition from the low-level controller header, llc_mcbspif.h.
 *  All such definitions are used as opaque objects in the McBSP DDC.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author Platform Support Group
 *
 *  \Revision
 *          0.1
 *          0.2  : Dynamic mode start/stop implemented
 *          0.3  Ported to mcbsp driver
 */
#ifndef _DDC_MCBSP_H
#define _DDC_MCBSP_H

#include <std.h>
#include <ti/sdo/edma3/drv/edma3_drv.h>
#include <ti/sdo/pspdrivers/common/psp_common.h>
#include <ti/sdo/pspdrivers/drivers/mcbsp/psp_mcbsp.h>
#include <ti/sdo/pspdrivers/drivers/mcbsp/src/llc_mcbspTypes.h>
#include <ti/sdo/pspdrivers/drivers/mcbsp/src/llc_mcbspIf.h>
#include <ti/sdo/pspdrivers/pal_os/pal_oslist.h>
#include <ti/sdo/pspdrivers/pal_os/pal_ossem.h>

#ifdef CHIP_C6424
#include <ti/sdo/pspdrivers/soc/c6424/dsp/soc.h>
#endif

#ifdef CHIP_DM6437
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/soc.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

/*
toggle the index
*/
#define GetNextIndex(index) (index = ((index+1u) & 0x01u))



/* Enable/Disable Global error Interrupt Handling */
#define GBL_INT 1


#define DDC_MCBSP_MAX_SERS  (1u)
#define MCBSP_MAX_PACKET_SIZE     (32768u)

/*
* In Gel File we are setting clk to 594 Mhz.
*
* therefor McBsp_internal_clk = SysClk3 = (SysClk/6);
*/


/*Loopjob buffers used in EDMA mode*/
typedef struct localTempbuffer_t
{
    Uint32 scratchBuffer[DDC_MCBSP_MAX_SERS];

}localTempBuffer;


/* forward declaration */
struct DDC_mcbspDevice_t;


/*
 *  \brief  McBSP DDC layer request structure
 *  DDC layer request strcture containing request information
 *
 */
typedef struct DDC_mcbspIOP_t
{

    PAL_OsListNodeHeader    iopLink;
    /**< Link to the next & previous in the list                        */

    Uint8                   *addr;
    /**< Application Buffer Pointer                                     */

    Uint32                  size;
    /**< Application Request Size                                       */

    Uint32                  actualSize;
    /**< Actual Data transferred                                        */

    PSP_Result              status;
    /**< Status of the Request. Updated by the Driver                   */

    PSP_mcbspAppCallback    appCbk;
    /**< McBSP Application call back registered during Open             */

    Ptr                     appData;
    /**< Application provided Pointer                                   */

    Ptr                     misc;
    /**< misc parameter                                                 */

} DDC_mcbspIOP;


/*
 * Channel function definitions
 */

/**
 * \brief Channel Object
 *
 * This structure maintains the current channel state. It also holds
 * information on DMA channel being used and holds the application
 * callback function to be called in case of an interrupt.
 *
 * This structure is initialized by mdCreateChan and a pointer to this
 * is passed down to all other channel related functions. Lifetime of
 * the data structure is from its creation by mdCreateChan till it
 * is invalidated by mdDeleteChan.
 */
typedef struct DDC_mcbspIoobj_t
{

    PAL_OsListNodeHeader        queueFreeList;
    /**< List containing the Free IO Packets                            */

    PAL_OsListNodeHeader        queueReqList;
    /**< List containing the Requests received from application. Driver
         checks this list to determine wether TX or RX is active        */

    PAL_OsListNodeHeader        queueFloatingList;
    /**< Queue to manage floating packets in DMA                        */

    PAL_OsSemHandle             syncSem;
    /**< Semaphore used during Flush Request                            */

    DDC_mcbspIOP                    ioRequests[PSP_MCBSP_MAX_REQ];
    /**< Channel transfer complete code                                 */

    PSP_McbspChannelStatus          status;
    /**< Keeps tab of whether channel is already in use
     *  - takes value of UNALLOCATED or ALLOCATED          */


    PSP_mcbspMode                    mode;
    /**< Rx mode or Tx mode                                 */

    Uint16                          inUse;
    /**< Is the channel in use?                             */

    Uint32                          intStatus;
    /**< Holds id of McBSP interrupt cause, if any
        used in interrupt mode                             */

    struct DDC_mcbspDevice_t        *deviceHandle;
    /**< Pointer to McBSP device
     *  - Back pointer to device configuration structure   */

    DDC_mcbspIOP                    *flushPacket;
    /**< Used for submit/IOM_FLUSH/flush packet             */

    DDC_mcbspIOP                    *dataPacket;
    /**< Current data packet being serviced.                */


    DDC_mcbspIOP                    *tempPacket;
    /**< Temp IOP holder                 */


    volatile Bool                   isTempPacketValid;


    Uint32                          userDataBufferSize;
    /**<Size of the data buffer per serialiser, hence the buffer
    given by the application should really be
    userDataBufferSize * no. of serialisers allocated for that
    channel*/

    Uint32                          submitCount;
    /**< Number of submit calls pending                     */

    PSP_mcbspAppCallback            cbFxn;
    /**< Callback function                                  */

    Ptr                             cbArg;
    /**< Arguments for callback function                    */


    Ptr                             edmaHandle;
    /**< eDMA  Handler */

    Uint32                          xferChan;
    /**< eDMA  ChannelID */

    Uint32                          tcc ;
    /**< eDMA  tcc */
    Uint32                          pramTbl[MAXLINKCNT];
    /**< Logical channel numbers of EDMA, which are used for linking    */


    Uint32                          nextLinkParamSetToBeUpdated;
    /** Used to store the next index of link param to be updated        */

    volatile Bool                   loopjobUpdatedinParamset;
    /**Used to prevent updating second paramset with loopjob for
         last datapacket edma callback                                  */

    Uint32                          cpuEventNum;
    /**< This hold the cpu event no. of this channel (TX/Rx)
        for this instance                                               */

    volatile Bool                   xferinProgressIntmode;
    /**<use to start int mode transfer from  0 packet status            */

    Ptr                             loopJobBuffer;
    /**< Buffer to be transferred when the loop job is running          */

    Uint16                          loopJobLength;
    /**<Length of userloop job for each serialiser                      */

    Uint16                          roundedWordWidth;
    /**< Word length bytes to be transferred for DMA transaction        */

    Uint16                          currentDataSize;
    /**< data packet size used for interrupt mode                       */

    volatile Bool                   bMuteON;
    /**< True, if playback is muted                                     */

    volatile Bool                   paused;
    /**< Audio already paused                                           */

    volatile Bool                   flush;
    /**< TRUE, if Flush Request is called                               */

    EDMA3_RM_TccCallback            edmaCallback;
    /* stores the edma call back function ptr - functions actual type
        is EDMA3_RM_TccCallback                                         */

    PSP_mcbspGblCallback            gblErrCbk;
    /**< callback required when the loop job is running
     *  must be callable directly from the IOM layer                    */

    volatile Bool                   nextFlag;

    Uint32                          noOfTdmChans;
    /**< Number of TDM channels used by the McBSP                       */

    PSP_mcbspReq                    pspRequest;
    /*current pending SPI request*/

    Uint32                          rxBytesIndex;


    Uint32                          txBytesIndex;
    /**< Number of bytes to tranceive                                  */


}DDC_mcbspIoObj, *DDC_mcbspIoHandle;



/**
 *  \brief  McBSP device object structure
 *
 *  McBSP device object structure containing the currently running
 *  McBSP instance specific variables
 */
typedef struct DDC_mcbspDevice_t
{

    Bool                        enablecache;
    /**< Submitted buffers are in cacheable memory                      */


    Uint32                      enableMask;
    /**< Holds enable Srgr and Fsgr if applicable                       */

    LLC_McbspObj                hwInfo;
    /**< McBSP handle for initial port configuration                    */

    DDC_mcbspIoObj              rxIo;
     /**< Holds channels to the McBSP.                                  */

    DDC_mcbspIoObj              txIo;
    /**< Holds channels to the McBSP.                                   */

    Uint16                      InstNum;
    /**< Preserve  instance number in port                              */

    Bool                        stopSmFsXmt;
    /**< Status of stoping state m/c for XMT                            */

    Bool                        stopSmFsRcv;
    /**< Status of stoping state m/c for RCV                            */

    /**< Transmit Channel object structure of McBSP                     */
    Ptr                         appHandle;

    PSP_mcbspHwCfgRaw           currentRawconfiguration;

    Bool                        isUserconfigUsed;

    /**< Operating mode of the device                                   */
    PSP_mcbspOperatingMode      currentMode;

    /**< Mcbsp spi master o/p freq                                      */
    Uint32                       McbspSpiMstrFreq;
    /**< Module Clock Frequency      */
	Uint32				  ModuleClkFreq;
    /**< Operation mode of driver                                       */
    PSP_OpMode              opMode;

} DDC_mcbspDevice, *DDC_mcbspDeviceHandle;

#ifdef __cplusplus
}
#endif /* extern "C" */


#endif /* _DDC_MCBSP_H */
