/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file   llc_mcbsp.c
 *
 *  \brief  LLC layer file of McBSP
 *
 *  This file contains lower level implementation of McBSP
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \version 0.1    -   First release
 *            0.2 - Pre silicon release
 */

#include <ti/sdo/pspdrivers/common/psp_common.h>
#include <ti/sdo/pspdrivers/drivers/mcbsp/src/llc_mcbspIf.h>

#ifdef CHIP_C6424
#include <ti/sdo/pspdrivers/soc/c6424/dsp/soc.h>
#include <ti/sdo/pspdrivers/soc/c6424/dsp/cslr_mcbsp.h>
#include <ti/sdo/pspdrivers/soc/c6424/dsp/csl_types.h>
#include <ti/sdo/pspdrivers/soc/c6424/dsp/csl_error.h>
#endif

#ifdef CHIP_DM6437
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/soc.h>
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/cslr_mcbsp.h>
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/csl_types.h>
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/csl_error.h>
#endif

#define MCBSP_INVALID_INSTANCEID    0xFFFFFFFF

/**
 * \fn static void  _LLC_mcbspClearFrmSyncErr(LLC_McbspHandle hMcbsp, LLC_McbspChType chanType)
 *
 *  \brief McBSP IO mode Control function
 *
 *  This Function Clears Transmit & Receive Frame Sync Errors
 *
 *  \param hMcbsp           McBSP Handle
 *  \param chanType         Type of the McBsp channel
 *  \return none
 */
static void  _LLC_mcbspClearFrmSyncErr(
                        LLC_McbspHandle       hMcbsp,
                        LLC_McbspChType       chanType
                        )
{
    if(NULL != hMcbsp)
    {
        /* Clear Receive Frame Sync Error */
        if(LLC_MCBSP_CHTYPE_RX == chanType)
        {
            hMcbsp->regs->SPCR &= (~LLC_MCBSP_SPCR_RSYNCERR_MASK) ;
        }

        /* Clear Transmit Frame Sync Error */
        if(LLC_MCBSP_CHTYPE_TX == chanType)
        {
            hMcbsp->regs->SPCR &= (~LLC_MCBSP_SPCR_XSYNCERR_MASK) ;
        }

        /* Clear Transmit & Receive Frame Sync Errors */
        if(LLC_MCBSP_CHTYPE_TXRX == chanType)
        {
            hMcbsp->regs->SPCR &= (~LLC_MCBSP_SPCR_RSYNCERR_MASK);
            hMcbsp->regs->SPCR &= (~LLC_MCBSP_SPCR_XSYNCERR_MASK);
        }
    }
}

/**
 * \fn static void  _LLC_mcbspIoModeCtrl( LLC_McbspHandle  hMcbsp, Uint32 outputSel,Uint32 inputSel )
 *
 *  \brief McBSP IO mode Control function
 *
 *  This Function is used to control io mode of McBSP
 *
 *  \param hMcbsp           McBSP Handle
 *  \param outputSel        Mask of outputselctor
 *  \param inputSel         Mask of inputselctor
 *  \return none
 */
static void  _LLC_mcbspIoModeCtrl(
                    LLC_McbspHandle hMcbsp,
                    Uint32              outputSel,
                    Uint32              inputSel
                    )
{
    if(NULL != hMcbsp)
    {
        /* Configuring PCR in Output mode */
        if(LLC_MCBSP_IO_CLKX  == (outputSel & LLC_MCBSP_IO_CLKX))
        {
           hMcbsp->regs->PCR |= LLC_MCBSP_PCR_CLKXM_MASK;
        }
        if( LLC_MCBSP_IO_FSX == (outputSel & LLC_MCBSP_IO_FSX))
        {
           hMcbsp->regs->PCR |= LLC_MCBSP_PCR_FSXM_MASK;
        }

        if( LLC_MCBSP_IO_CLKR == (outputSel & LLC_MCBSP_IO_CLKR))
        {
            hMcbsp->regs->PCR |= LLC_MCBSP_PCR_CLKRM_MASK;
        }

        if( LLC_MCBSP_IO_FSR == (outputSel & LLC_MCBSP_IO_FSR))
        {
            hMcbsp->regs->PCR |= LLC_MCBSP_PCR_FSRM_MASK;
        }
        /* Configuring PCR in Input mode */
        if( LLC_MCBSP_IO_CLKX == (inputSel & LLC_MCBSP_IO_CLKX))
        {
            hMcbsp->regs->PCR &= (~LLC_MCBSP_PCR_CLKXM_MASK);
        }

        if( LLC_MCBSP_IO_FSX == (inputSel & LLC_MCBSP_IO_FSX))
        {
            hMcbsp->regs->PCR &= (~LLC_MCBSP_PCR_FSXM_MASK);
        }

        if( LLC_MCBSP_IO_CLKR == (inputSel & LLC_MCBSP_IO_CLKR))
        {
            hMcbsp->regs->PCR &= (~LLC_MCBSP_PCR_CLKRM_MASK);
        }

        if( LLC_MCBSP_IO_FSR == (inputSel & LLC_MCBSP_IO_FSR))
        {
            hMcbsp->regs->PCR &= (~LLC_MCBSP_PCR_FSRM_MASK);
        }
    }
}


/**
 *  \fn static void  _LLC_mcbspRegReset( LLC_McbspHandle   hMcbsp)

 *  \brief McBSP reset function
 *
 *  This Function will initialize McBSP with default values.
 *
 *  \param hMcbsp           McBSP Handle
 *  \return none
 *
 */
/* ARGSUSED */
static void  _LLC_mcbspRegReset( LLC_McbspHandle   hMcbsp)
{
    CSL_McbspRegsOvly mcbspRegs = NULL;
    if(NULL != hMcbsp)
    {
        mcbspRegs = hMcbsp->regs;
        if(NULL != mcbspRegs)
        {

            mcbspRegs->DXR =  CSL_MCBSP_DXR_RESETVAL;

            mcbspRegs->SPCR = CSL_MCBSP_SPCR_RESETVAL;

            mcbspRegs->RCR = CSL_MCBSP_RCR_RESETVAL;

            mcbspRegs->XCR = CSL_MCBSP_XCR_RESETVAL;

            mcbspRegs->SRGR = CSL_MCBSP_SRGR_RESETVAL;

            mcbspRegs->PCR = CSL_MCBSP_PCR_RESETVAL;
        }
    }
}

/**
 * \fn static void  _LLC_mcbspResetCtrl( LLC_McbspHandle  hMcbsp, int selectMask )
 *
 *  \brief McBSP SPCR configuration function
 *
 *  This Function is used to set/reset specific bit of SPCR as specified in mask
 *
 *  \param hMcbsp           McBSP Handle
 *  \param selectMask       mask to set/reset specific SPCR bit
 *  \return none
 */
static void  _LLC_mcbspResetCtrl( LLC_McbspHandle  hMcbsp, Uint32 selectMask )
{
    CSL_McbspRegsOvly mcbspRegs;

    if ( NULL != hMcbsp)
    {
        mcbspRegs = hMcbsp->regs;
    }
    if(NULL != mcbspRegs)
    {
        /* Configuring SPCR for sample rate generator enable/disable */
        if(0 != (selectMask & LLC_MCBSP_CTRL_SRG_ENABLE))
        {
            mcbspRegs->SPCR |=  LLC_MCBSP_SPCR_GRST_MASK;
        }
        if(0 != (selectMask & LLC_MCBSP_CTRL_SRG_DISABLE))
        {
            mcbspRegs->SPCR &=  (~LLC_MCBSP_SPCR_GRST_MASK);
        }

        /* Configuring SPCR for Frame sync generator enable/disable */
        if(0 != (selectMask & LLC_MCBSP_CTRL_FSYNC_ENABLE))
        {
            mcbspRegs->SPCR |=  LLC_MCBSP_SPCR_FRST_MASK;
        }
        if(0 != (selectMask & LLC_MCBSP_CTRL_FSYNC_DISABLE))
        {
            mcbspRegs->SPCR &= ( ~LLC_MCBSP_SPCR_FRST_MASK );
        }

        /* Configuring SPCR for Receive enable/disable */
        if(0 != (selectMask & LLC_MCBSP_CTRL_RX_ENABLE))
        {
            mcbspRegs->SPCR |=   LLC_MCBSP_SPCR_RRST_MASK ;
        }
        if(0 != (selectMask & LLC_MCBSP_CTRL_RX_DISABLE))
        {
            mcbspRegs->SPCR &= ( ~LLC_MCBSP_SPCR_RRST_MASK );
        }

        /* Configuring SPCR for Transmit enable/disable */
        if(0 != (selectMask & LLC_MCBSP_CTRL_TX_ENABLE))
        {
            mcbspRegs->SPCR |=   LLC_MCBSP_SPCR_XRST_MASK;
        }
        if(0 != (selectMask & LLC_MCBSP_CTRL_TX_DISABLE))
        {
            mcbspRegs->SPCR &= ( ~LLC_MCBSP_SPCR_XRST_MASK );
        }
    }
}

/*! \fn CSL_Status  LLC_mcbspInit( const Int32 * pContext)
 *
 *  \brief Init McBSP port.
 *
 *  This Function is used to initialize LLC module.
 *  Currently this function is not implmented
 *
 *  \param *pContext pointer to mask (Not implemented )
 *  \return CSL_Status
 */
/* ARGSUSED */
CSL_Status  LLC_mcbspInit( const Int32 * pContext)
{
    return PSP_SOK;
}

/*! \fn CSL_Status LLC_mcbspGetBaseAddress(Uint32 InstanceId, Ptr param, LLC_McbspBaseAddress *const baseAddr)
 *
 *  \brief Get base address of McBSP port .
 *
 *  This Function is used to assign base address of McBSP port.
 *
 *  \param InstanceId           McBSP instance ID
 *  \param param                pointer to user structure
 *  \param baseAddr             pointer to baseaddress
 *  \return CSL_Status
 */
/* ARGSUSED */
CSL_Status LLC_mcbspGetBaseAddress( Uint32 InstanceId,
                                    Ptr param,
                                    LLC_McbspBaseAddress *const baseAddr)
{
    CSL_Status status = PSP_SOK;

    if(NULL == baseAddr)
    {
        status = CSL_ESYS_INVPARAMS;
    }

    if(PSP_SOK == status)
    {
        switch (InstanceId)
        {

            case 0:
                baseAddr->regs = (CSL_McbspRegsOvly)CSL_MCBSP_0_REGS;
                break;

            case 1:
                baseAddr->regs = (CSL_McbspRegsOvly)CSL_MCBSP_1_REGS;
                break;

            default:
                baseAddr->regs = (CSL_McbspRegsOvly)NULL;
                status = CSL_ESYS_FAIL;
                break;
        }
    }
    return status;
}

/*! \fn Ptr LLC_McbspHandle LLC_mcbspOpen(  LLC_McbspObj * const pMcbspObj,Uint32 InstanceId,Int32   *pMcbspParam, CSL_Status  *pStatus)
 *
 *  \brief Open McBSP port
 *
 *  This Function is used to assign base address of McBSP port and will initialize
 *  mcbsp structure with the same.
 *
 *  \param *pMcbspObj           pointer to LLC_McbspObj
 *  \param InstanceId           McBSP instance ID
 *  \param *pMcbspParam         User parameters
 *  \param *pStatus             CSL_Status
 *  \return CSL_Status
 */
/*ARGSUSED*/
LLC_McbspHandle LLC_mcbspOpen(  LLC_McbspObj * const pMcbspObj,
                                Uint32 InstanceId,
                                Int32   *pMcbspParam,
                                CSL_Status  *pStatus)
{
    CSL_Status            status = PSP_SOK;
    LLC_McbspHandle       hMcbsp = NULL;
    status = PSP_SOK;
    if (((NULL == pMcbspObj ) || (NULL == pStatus) )|| ( InstanceId > 1u ) )
    {
       hMcbsp = NULL;
       status = CSL_ESYS_INVPARAMS;
    }

    if(PSP_SOK == status)
    {
        switch (InstanceId)
        {
        /*Assigning base address for instance 0*/
            case 0:

                /* McBSP 0 dedicated edma channels */
                pMcbspObj->edmaRxEventNum = CSL_EDMA3_CHA_REVT0;
                pMcbspObj->edmaTxEventNum = CSL_EDMA3_CHA_XEVT0;

                /* McBSP 0 interrupts */
                pMcbspObj->cpuRxEventNum = CSL_INTC_EVENTID_MBRINT0;
                pMcbspObj->cpuTxEventNum= CSL_INTC_EVENTID_MBXINT0;

                pMcbspObj->regs = (CSL_McbspRegsOvly)CSL_MCBSP_0_REGS;
                hMcbsp = (LLC_McbspHandle)pMcbspObj;
                break;

            case 1:

                /* McBSP 0 dedicated edma channels */
                pMcbspObj->edmaRxEventNum = CSL_EDMA3_CHA_REVT1;
                pMcbspObj->edmaTxEventNum = CSL_EDMA3_CHA_XEVT1;

                /* McBSP 0 interrupts */
                pMcbspObj->cpuRxEventNum = CSL_INTC_EVENTID_MBRINT1;
                pMcbspObj->cpuTxEventNum = CSL_INTC_EVENTID_MBXINT1;

                pMcbspObj->regs = (CSL_McbspRegsOvly)CSL_MCBSP_1_REGS;
                hMcbsp = (LLC_McbspHandle)pMcbspObj;
                break;

            default:
                pMcbspObj->regs = (CSL_McbspRegsOvly)NULL;
                hMcbsp = (LLC_McbspHandle)NULL;
                status = CSL_ESYS_FAIL;
                break;
        }
    }

    return hMcbsp;
}

/*! \fn CSL_Status  LLC_mcbspClose(LLC_McbspHandle    hMcbsp)
 *
 *  \brief Close McBSP port
 *
 *  This Function will close McBSP port.
 *
 *  \param hMcbsp               User supplied McBSP handle
 *  \return CSL_Status
 */
CSL_Status  LLC_mcbspClose(LLC_McbspHandle    hMcbsp)
{
    CSL_Status  status = PSP_SOK;

    if (hMcbsp != NULL)
    {
        hMcbsp->regs    = (CSL_McbspRegsOvly)NULL;
        hMcbsp->perNum  = MCBSP_INVALID_INSTANCEID;
        status = PSP_SOK;
    }
    else
    {
        status = CSL_ESYS_BADHANDLE;
    }

    return status;
}

/*! \fn CSL_Status  LLC_mcbspHwControl( LLC_McbspHandle  hMcbsp, LLC_McbspControlCmd  cmd,const void *arg)
 *
 *  \brief Control the Mcbsp hardware functionality.
 *
 *  This Function is control function and works as command dispatcher in
 *  LLC layer. this function will configure McBSP as per supplied command and values.
 *
 *  \param hMcbsp           McBSP Handle
 *  \param cmd              Command to control hardware
 *  \param arg              Arguments required for specific commands
 *  \return CSL_Status
 */
CSL_Status  LLC_mcbspHwControl( LLC_McbspHandle  hMcbsp, LLC_McbspControlCmd  cmd,const void *arg)
{
    CSL_Status           status  = PSP_SOK;
    LLC_McbspChType      *chan   = NULL;
    LLC_McbspIoControl   *io     = NULL;
    Uint32               *ctrl   = NULL;

    if( (NULL == hMcbsp) || (NULL == arg))
    {
        status=CSL_ESYS_BADHANDLE;
    }

    if (PSP_SOK == status)
    {
        switch(cmd)
        {
            /* Clear sync error  in (R/X)CR */
            case LLC_MCBSP_CMD_CLEAR_FRAME_SYNC:
                chan = (LLC_McbspChType *)arg;
                _LLC_mcbspClearFrmSyncErr(hMcbsp,*chan);
                break;
            /* Control I/O mode in SPCR */
            case LLC_MCBSP_CMD_IO_MODE_CONTROL:
                io = (LLC_McbspIoControl *)arg;
                _LLC_mcbspIoModeCtrl(hMcbsp, io->outputsel, io->inputsel);
                break;
            /* Reset registers */
            case LLC_MCBSP_CMD_REG_RESET:
                _LLC_mcbspRegReset(hMcbsp);
                break;
            /* reset registers and configure them as per mask */
            case LLC_MCBSP_CMD_RESET_CONTROL:
                ctrl = (Uint32 *)arg;
                _LLC_mcbspResetCtrl(hMcbsp, *ctrl);
                break;

            default:
                status = CSL_ESYS_INVCMD;
                break;
        }
    }
    return (status);
}

/*! \fn CSL_Status  LLC_mcbspHwSetupRaw (LLC_McbspHandle     hMcbsp,const LLC_McbspConfig     *config)
 *
 *  \brief Disable all the registers and restore the backup of mcbsp registers
 *
 *  This Function initializes McBSP with User supplied values.
 *
 *  \param hMcbsp               User supplied McBSP handle
 *  \param *config              pointer to user parameter structure
 *  \return CSL_Status
 */
CSL_Status  LLC_mcbspHwSetupRaw (
    LLC_McbspHandle     hMcbsp,
    const LLC_McbspConfig     *config
)
{
    CSL_Status Status = PSP_SOK;
    if (NULL == hMcbsp)
    {
        Status = CSL_ESYS_BADHANDLE;
    }

    if (NULL == config)
    {
        Status = CSL_ESYS_INVPARAMS;
    }

    if (PSP_SOK == Status)
    {
        /* Disable all registers */
        hMcbsp->regs->SPCR     = 0x0u;

        hMcbsp->regs->RCR      = config->RCR ;
        hMcbsp->regs->XCR      = config->XCR ;

        hMcbsp->regs->SRGR     = config->SRGR;
        hMcbsp->regs->PCR      = config->PCR ;

        hMcbsp->regs->SPCR     = config->SPCR;
    }

    return Status;
}

/*! \fn CSL_Status CSL_Status LLC_mcbspHwSetup( LLC_McbspHandle hMcbsp,const LLC_McbspHwSetup *setup)
 *
 *  \brief Setup the mcbsp hardware
 *
 *  This Function initializes McBSP with User supplied values.
 *
 *  \param hMcbsp               User supplied McBSP handle
 *  \param *setup               pointer to setup parameter structure
 *  \return CSL_Status
 */
CSL_Status LLC_mcbspHwSetup(    LLC_McbspHandle hMcbsp,
                                const LLC_McbspHwSetup *setup)
{

    static Uint32 spcr = 0;
    static Uint32 pcr  = 0;
    CSL_McbspRegsOvly mcbspRegs;
    CSL_Status Status = PSP_SOK;


    if(NULL == setup)
    {
        Status = CSL_ESYS_INVPARAMS;
    }

    if (NULL == hMcbsp)
    {
        Status = CSL_ESYS_BADHANDLE;
    }

    if(PSP_SOK == Status)
    {
        mcbspRegs = hMcbsp->regs;
        if(NULL != mcbspRegs)
        {
            /* Set Emulation Mode Configuration */
            if(LLC_MCBSP_EMU_FREERUN != setup->emumode)
            {
                spcr |= LLC_MCBSP_SPCR_SOFT_MASK;
            }
            else
            {
                spcr |= LLC_MCBSP_SPCR_FREE_MASK;
            }

            mcbspRegs->SPCR = spcr;

            /*Clock and Frame Sync Configuration*/
            if(NULL != setup->clkset)
            {

                Uint32 srgr = 0;
                LLC_mcbspClkSetup clkCfg_Hwsetup = *(setup->clkset);

                /* Configuring PCR  (FSRM)  */
                if(LLC_MCBSP_FSCLKMODE_INTERNAL == clkCfg_Hwsetup.frmSyncRxMode)
                {
                    pcr |= LLC_MCBSP_PCR_FSRM_MASK;
                }
                else
                {
                    pcr &= (~LLC_MCBSP_PCR_FSRM_MASK);
                }

                /* Configuring PCR  (FSXM)  */
                if(LLC_MCBSP_FSCLKMODE_INTERNAL == clkCfg_Hwsetup.frmSyncTxMode)
                {
                    pcr |= LLC_MCBSP_PCR_FSXM_MASK;
                }
                else
                {
                    pcr &= (~LLC_MCBSP_PCR_FSXM_MASK);
                }

                /* Configuring PCR  (FSRP)  */
                if(LLC_MCBSP_FSPOL_ACTIVE_LOW == clkCfg_Hwsetup.frmSyncRxPolarity)
                {
                    pcr |= LLC_MCBSP_PCR_FSRP_MASK;
                }
                else
                {
                    pcr &= (~LLC_MCBSP_PCR_FSRP_MASK);
                }

                /* Configuring PCR  (FSXP)  */
                if(LLC_MCBSP_FSPOL_ACTIVE_LOW == clkCfg_Hwsetup.frmSyncTxPolarity)
                {
                    pcr |= LLC_MCBSP_PCR_FSXP_MASK;
                }
                else
                {
                    pcr &= (~LLC_MCBSP_PCR_FSXP_MASK);
                }
                /* Configuring PCR  (CLKRM)  */
                if(LLC_MCBSP_TXRXCLKMODE_INTERNAL == clkCfg_Hwsetup.clkRxMode)
                {
                    pcr |= LLC_MCBSP_PCR_CLKRM_MASK;
                }
                else
                {
                    pcr &= (~LLC_MCBSP_PCR_CLKRM_MASK);
                }
                /* Configuring PCR  (CLKXM)  */
                if(LLC_MCBSP_TXRXCLKMODE_INTERNAL == clkCfg_Hwsetup.clkTxMode)
                {
                    pcr |= LLC_MCBSP_PCR_CLKXM_MASK;
                }
                else
                {
                    pcr &= (~LLC_MCBSP_PCR_CLKXM_MASK);
                }
                /* Configuring PCR  (CLKRP)  */
                if( (LLC_MCBSP_CLKPOL_RX_RISING_EDGE == clkCfg_Hwsetup.clkRxPolarity) || (LLC_MCBSP_CLKPOL_SRG_FALLING_EDGE == clkCfg_Hwsetup.clkRxPolarity))
                {
                    pcr |= LLC_MCBSP_PCR_CLKRP_MASK;
                }
                else
                {
                    pcr &= (~LLC_MCBSP_PCR_CLKRP_MASK);
                }

                /* Configuring PCR  (CLKXP)  */
                if(  (LLC_MCBSP_CLKPOL_TX_FALLING_EDGE == clkCfg_Hwsetup.clkTxPolarity)  ||  (LLC_MCBSP_CLKPOL_SRG_FALLING_EDGE == clkCfg_Hwsetup.clkTxPolarity) )
                {
                    pcr |= LLC_MCBSP_PCR_CLKXP_MASK;
                }
                else
                {
                    pcr &= (~LLC_MCBSP_PCR_CLKXP_MASK);
                }

                /* Configuring PCR  (SCLKME)  */
                if( (LLC_MCBSP_SRGCLK_CLKX == clkCfg_Hwsetup.srgInputClkMode) || (LLC_MCBSP_SRGCLK_CLKR == clkCfg_Hwsetup.srgInputClkMode) )
                {
                    pcr |= LLC_MCBSP_PCR_SCLKME_MASK;
                }
                else
                {
                    pcr &= (~LLC_MCBSP_PCR_SCLKME_MASK);
                }

                mcbspRegs->PCR = pcr;

                /* Configuring SRGR(CLKSM)  */
                if( (LLC_MCBSP_SRGCLK_CLKCPU == clkCfg_Hwsetup.srgInputClkMode) || (LLC_MCBSP_SRGCLK_CLKX == clkCfg_Hwsetup.srgInputClkMode) )
                {
                    srgr |= LLC_MCBSP_SRGR_CLKSM_MASK;
                }
                else
                {
                    srgr &= (~LLC_MCBSP_SRGR_CLKSM_MASK);
                }

                /* Configuring SRGR  for (FSMOD) */
                if(LLC_MCBSP_TXFSMODE_SRG == clkCfg_Hwsetup.srgTxFrmSyncMode)
                {
                    srgr |= LLC_MCBSP_SRGR_FSGM_MASK;
                }
                else
                {
                    srgr &= (~LLC_MCBSP_SRGR_FSGM_MASK);
                }

                /* Configuring SRGR  */
                srgr = (Uint32) srgr | (
                            ( clkCfg_Hwsetup.srgFrmPulseWidth << LLC_MCBSP_SRGR_FWID_SHIFT)|
                            ( clkCfg_Hwsetup.srgClkDivide << LLC_MCBSP_SRGR_CLKGDV_SHIFT )|
                            ( clkCfg_Hwsetup.srgFrmPeriod << LLC_MCBSP_SRGR_FPER_SHIFT)
                         ) ;

               mcbspRegs->SRGR = srgr;
            }

            /* McBSP Loopcak configuration */
            if(LLC_MCBSP_DLBMODE_ON == setup->dlbMode)
            {
                mcbspRegs->SPCR |= (LLC_MCBSP_SPCR_DLB_MASK);
            }
            else
            {
                mcbspRegs->SPCR &= (~LLC_MCBSP_SPCR_DLB_MASK);
            }

            /* Transmitter Configuration*/
            if(NULL != setup->txdataset)
            {

                static Uint32 xcr = 0 ;
                LLC_XmitDataSetup transmitterCfg = *(setup->txdataset);

                /* Disable Transmitter */
                mcbspRegs->SPCR &= (~LLC_MCBSP_SPCR_XRST_MASK);


                if(LLC_MCBSP_32BIT_REVERS_ENABLE == transmitterCfg.wordReverse)
                {
                    xcr |= LLC_MCBSP_XCR_XWDREVRS_MASK;
                }
                else
                {
                    xcr &= (~LLC_MCBSP_XCR_XWDREVRS_MASK);
                }

                if(LLC_MCBSP_PHASE_DUAL == transmitterCfg.phaseNum)
                {
                    xcr |= LLC_MCBSP_XCR_XPHASE_MASK;
                }
                else
                {
                    xcr &= (~LLC_MCBSP_XCR_XPHASE_MASK);
                }

                if(LLC_MCBSP_FRMSYNC_IGNORE == transmitterCfg.frmSyncIgn)
                {
                    xcr |= LLC_MCBSP_XCR_XFIG_MASK;
                }
                else
                {
                    xcr &= (~LLC_MCBSP_XCR_XFIG_MASK);
                }

                if((0 != transmitterCfg.frmLength2) || ( 0 != transmitterCfg.frmLength1))
                {
                /* Configuring XCR datalength, wordlength and fremlengths for phases*/
                    xcr = (Uint32) xcr | (
                             ( transmitterCfg.wordLength1 << LLC_MCBSP_DATA_WDLEN1_SHIFT )|
                             ( transmitterCfg.wordLength2 << LLC_MCBSP_DATA_WDLEN2_SHIFT)|
                             ( transmitterCfg.compand << LLC_MCBSP_DATA_COMPAND_SHIFT )|
                             ( ((Uint32)(transmitterCfg.frmLength2 - 1u))<< LLC_MCBSP_DATA_FRLEN2_SHIFT )|
                             ( transmitterCfg.dataDelay << LLC_MCBSP_DATA_DELAY_SHIFT ) |
                             ( ((Uint32)(transmitterCfg.frmLength1 - 1u)) << LLC_MCBSP_DATA_FRLEN1_SHIFT )

                            );
                }

                mcbspRegs->XCR = xcr;
                    /* Configuring spcr for interrupt mode */
                mcbspRegs->SPCR &= (~LLC_MCBSP_SPCR_XINTM_MASK);
                mcbspRegs->SPCR |= (Uint32)( transmitterCfg.IntEvent  << LLC_MCBSP_SPCR_XINTM_SHIFT );
            }

            /*Receiver Configuration*/
            if(NULL != setup->rxdataset)
            {

                static Uint32 rcr = 0 ;
                LLC_RcvDataSetup receiverCfg = *(setup->rxdataset);

                /* Disable Receiver */
                mcbspRegs->SPCR &= (~LLC_MCBSP_SPCR_RRST_MASK);

                /* Configuring RCR with user supplied parameters */
                if(LLC_MCBSP_32BIT_REVERS_ENABLE == receiverCfg.wordReverse)
                {
                    rcr |= LLC_MCBSP_RCR_RWDREVRS_MASK;
                }
                else
                {
                    rcr &= (~LLC_MCBSP_RCR_RWDREVRS_MASK);
                }

                if(LLC_MCBSP_PHASE_DUAL == receiverCfg.phaseNum)
                {
                    rcr |= LLC_MCBSP_RCR_RPHASE_MASK;
                }
                else
                {
                    rcr &= (~LLC_MCBSP_RCR_RPHASE_MASK);
                }

                if(LLC_MCBSP_FRMSYNC_IGNORE == receiverCfg.frmSyncIgn)
                {
                    rcr |= LLC_MCBSP_RCR_RFIG_MASK;
                }
                else
                {
                    rcr &= (~LLC_MCBSP_RCR_RFIG_MASK);
                }

                if((0 != receiverCfg.frmLength2) || ( 0 != receiverCfg.frmLength1))
                {
                    /* Configuring RCR datalength, wordlength and fremlengths for phases*/
                    rcr =(Uint32) rcr | (
                             ( receiverCfg.wordLength1 << LLC_MCBSP_DATA_WDLEN1_SHIFT ) |
                             ( receiverCfg.wordLength2 << LLC_MCBSP_DATA_WDLEN2_SHIFT )|
                             ( ((Uint32)(receiverCfg.frmLength2 - 1u))<< LLC_MCBSP_DATA_FRLEN2_SHIFT )|
                             ( receiverCfg.compand << LLC_MCBSP_DATA_COMPAND_SHIFT)|
                             ( receiverCfg.dataDelay << LLC_MCBSP_DATA_DELAY_SHIFT)|
                             ( ((Uint32)(receiverCfg.frmLength1 - 1u))<< LLC_MCBSP_DATA_FRLEN1_SHIFT )
                            );
                }
                 mcbspRegs->RCR = rcr;

                /* Configuring spcr for interrupt mode and data alignment */
                 mcbspRegs->SPCR &= (~LLC_MCBSP_SPCR_RINTM_MASK);
                 mcbspRegs->SPCR |= (Uint32)( receiverCfg.IntEvent  << LLC_MCBSP_SPCR_RINTM_SHIFT );

                 mcbspRegs->SPCR &= (~LLC_MCBSP_SPCR_RJUST_MASK);
                 mcbspRegs->SPCR |=  ((Uint32)receiverCfg.rjust_dxenable << LLC_MCBSP_SPCR_RJUST_SHIFT );
            }
        }
        else
        {
            Status = CSL_ESYS_BADHANDLE;
        }
    }
    return(Status);
}

/*! \fn CSL_Status LLC_mcbspRcvConfigWordLength( LLC_McbspHandle hMcbsp, Uint32 wordLen)
 *
 *  \brief set word length for receive section
 *
 *  This Function configures wordlength1 for receive section with User 
 *  supplied value.
 *
 *  \param hMcbsp               User supplied McBSP handle
 *  \param wordLen              wordlength1 to be configured
 *  \return CSL_Status
 */
CSL_Status LLC_mcbspRcvConfigWordLength( LLC_McbspHandle hMcbsp, Uint32 wordLen)
{
    CSL_Status status = PSP_SOK;

    if (NULL == hMcbsp)
    {
        status = CSL_ESYS_BADHANDLE;
    }
    else
    {
        switch(wordLen)
        {
            case 8:
                CSL_FINST(hMcbsp->regs->RCR,MCBSP_RCR_RWDLEN1,8BIT);
                break;
            case 12:
                CSL_FINST(hMcbsp->regs->RCR,MCBSP_RCR_RWDLEN1,12BIT);
                break;
            case 16:
                CSL_FINST(hMcbsp->regs->RCR,MCBSP_RCR_RWDLEN1,16BIT);
                break;
            case 20:
                CSL_FINST(hMcbsp->regs->RCR,MCBSP_RCR_RWDLEN1,20BIT);
                break;
            case 24:
                CSL_FINST(hMcbsp->regs->RCR,MCBSP_RCR_RWDLEN1,24BIT);
                break;
            case 32:
                CSL_FINST(hMcbsp->regs->RCR,MCBSP_RCR_RWDLEN1,32BIT);
                break;
            default:
                status = CSL_ESYS_INVPARAMS;
                break;
        }
    }

    return (status);
}

/*! \fn CSL_Status CSL_Status LLC_mcbspXmtConfigWordLength( LLC_McbspHandle hMcbsp, Uint32 wordLen)
 *
 *  \brief  set word length for transmit section
 *
 *  This Function configures wordlength1 for transmit section with user 
 *  supplied value.
 *
 *  \param hMcbsp               User supplied McBSP handle
 *  \param wordLen              wordlength1 to be configured
 *  \return CSL_Status
 */
CSL_Status LLC_mcbspXmtConfigWordLength( LLC_McbspHandle hMcbsp, Uint32 wordLen)
{
    CSL_Status status = PSP_SOK;

    if (NULL == hMcbsp)
    {
        status = CSL_ESYS_BADHANDLE;
    }
    else
    {
        switch(wordLen)
        {
            case 8:
                CSL_FINST(hMcbsp->regs->XCR,MCBSP_XCR_XWDLEN1,8BIT);
                break;
            case 12:
                CSL_FINST(hMcbsp->regs->XCR,MCBSP_XCR_XWDLEN1,12BIT);
                break;
            case 16:
                CSL_FINST(hMcbsp->regs->XCR,MCBSP_XCR_XWDLEN1,16BIT);
                break;
            case 20:
                CSL_FINST(hMcbsp->regs->XCR,MCBSP_XCR_XWDLEN1,20BIT);
                break;
            case 24:
                CSL_FINST(hMcbsp->regs->XCR,MCBSP_XCR_XWDLEN1,24BIT);
                break;
            case 32:
                CSL_FINST(hMcbsp->regs->XCR,MCBSP_XCR_XWDLEN1,32BIT);
                break;
            default:
                status = CSL_ESYS_INVPARAMS;
                break;
        }
    }

    return (status);
}
