/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 * \file    psp_mcbspApi.h
 *
 * \brief   Wrapper APIs called by DDA to access DDC
 *
 *  This file contains APIs defination of APIs that are wrapper APIs for DDC
 *  layer. These APIs will be called by DDA layer.
 *
 *    (C)    Copyright 2006,    Texas Instruments, Inc
 *
 *  \note       Set tabstop to 4 (:se ts=4) while viewing this file in an
 *              editor
 *  \author Platform Support Group
 *
 *  \Revision
 *            0.1  Created
 *            0.2  Created  : Dynamic mode start/stop implemented
 *            0.3  modified : modified to PSP architecture and
 *                            implemented EDMA linking  of next pending packet
 *            0.4  modified : DM6437 - post silicon changes and cleanup
 *            0.5  ported for mcbsp
 *
 */
#ifndef _PSP_MCBSP_API_
#define _PSP_MCBSP_API_


#ifdef __cplusplus
extern "C" {
#endif

/*!
 *  \fn PSP_Result PSP_mcbspCreate(Uint32 instanceId,
 *                              PSP_mcbspDevInitParams *devParams,
 *                              Ptr arg)
 *  \brief Allocates and configures the McBSP port specified by devid.
 *
 *  Acquire the Handle of the McBSP and configure the McBSP by
 *  default for the following things.
 *      o   Data output for audio playback
 *      o   Data input for audio recording
 *      o   Configure the McBSP in DSP data format mode of the audio
 *          codec.
 *      o   Configure the McBSP to receive the Frame Sync and bit clock
 *          externally for both receiver and transmitter.
 *      o   McBSP can also be configured to generate Frame Sync and
 *          bit clock internally by enabling sample rate generator and
 *          frame sync generator blocks respectively depending on the
 *          fields set in the device parameter structure which shall
 *          be passed to mdBindDev() function as one of the parameter.
 *
 *  \param  instNum [IN]        McBSP instance number
 *  \param  initCfg [IN]        Pointer to PSP_mcbspDevParams structure
 *  \param  Arg     [IN]        Miscellenous Argument
 *
 *  \return PSP_SOK or PSP Error code
 */

PSP_Result PSP_mcbspCreate( Uint32 instanceId,
                            const PSP_mcbspDevParams *initCfg,
                            Ptr Arg);

/*!
 *  \fn PSP_Result PSP_mcbspDelete(Uint32 instanceId,Ptr arg)
 *  \brief Closes the McBSP device from use.
 *
 *  mdUnBindDev() shall call the PSP_mcbspDelete() API when the
 *  application no longer the needs the mcBSP port.
 *
 *  \param  [IN]  port   Pointer to the McBSP port object structure
 *  \param  [IN]  arg       Misc Argument
 *  \return PSP_SOK
 */
PSP_Result PSP_mcbspDelete(Uint32 instanceId,Ptr Arg);


/**
 *  \brief Create a logical communication channel for data transfer
 *
 *  A communication channel shall be created by the calling
 *  PSP_mcbspOpenChannel by specifying  the I/O mode(INPUT or OUTPUT)
 *  After successfully creating the I/O channel PSP_mcbspOpenChannel shall
 *  return the handle to the I/O channel
 *
 *  \param  instanceId      [IN]    Handle to the McBSP device instance created
 *                                  through PSP_mcbspOpen() call
 *  \param  mode            [IN]    Mode of the  channel INPUT or OUTPUT
 *  \param  params          [IN]    Pointer to the PSP_mcbspChanParams structure
 *  \param  drvCbk          [IN]    Pointer to callback function for McBSP
 *  \param  edmaHandle      [IN]    EDMA handle that is to be used by driver
 *  \param  arg             [IN]    Argument to be passed to open call
 *  \return PSP_Handle              Channel handle if successful or NULL if failes
 *                                  to create the I/O channel)
 */
PSP_Handle PSP_mcbspOpen(
                            Uint32               instanceId,
                            PSP_mcbspMode        mode,
                            const PSP_mcbspChanParams  * params,
                            PSP_mcbspAppCallback drvCbk,
                            PSP_Handle           edmaHandle,
                            Ptr                  arg
                            );

/*!
 *  \fn PSP_Result PSP_mcbspClose(PSP_Handle hMcBSPChan,Ptr arg)
 *  \brief frees a channel and all it's associated resources.
 *
 *  PSP_mcbspClose() frees the EDMA resources including EDMA
 *  master channel and link PaRAM entries held by the channel
 *
 *  \param  hMcBSPChan  [IN]    PSP_Handle
 *  \param  arg         [IN]    Misc Arg
 *
 *  \return PSP_SOK
 */
PSP_Result PSP_mcbspClose(PSP_Handle hMcBSPChan,Ptr Arg);

/** \fn PSP_Result PSP_mcbspWrite (PSP_Handle hMcBSPChan, PSP_mcbspReq *IoReq, Ptr Arg)
 *  \brief  Write a buffer of data to the output channel of the driver
 *
 *  The application has to call PSP_mcbspWrite() with handle to the
 *  o/p channel and the pointer to buffer to where the data shall be stored
 *  and the number of bytes to read. The has to specify the call back function
 *  the shall be called when the I/O is complete
 *
 *  \param  hMcBSPChan [IN]     Handle to the McBSP channel
 *  \param  ioReq [IN]           request structure
 *  \param  Arg     [IN]        Miscellenous Argument
 *  \return PSP_SOK or PSP Error code
 */

PSP_Result PSP_mcbspWrite (PSP_Handle handle, PSP_mcbspReq *ioReq,Ptr Arg);

/** \fn PSP_Result PSP_mcbspRead(PSP_Handle hMcBSPChan, PSP_mcbspReq *IoReq,Ptr Arg)
 *  \brief Read a buffer of data from the input channel of the driver
 *
 *  The application has to call PSP_mcbspRead() with handle to the
 *  i/p channel and the pointer to buffer to where the data shall be stored
 *  and the number of bytes to read. The has to specify the call back function
 *  the shall be called when the I/O is complete
 *
 *
 *  \param  hMcBSPChan [IN]     Handle to the McBSP channel
 *  \param  ioReq [IN]           request structure
 *  \param  Arg     [IN]        Miscellenous Argument
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_mcbspRead (PSP_Handle handle, PSP_mcbspReq *ioReq,Ptr Arg);

/** \fn PSP_Result PSP_mcbspFlushRequests (PSP_Handle hMcBSPChan,Ptr arg)
 *
 *  \brief Flush the data in the McBSP channel
 *
 *  Flushes the pending I/O packets in the specified channel
 *  and returns the notifications of the pending I/O packets
 *  to the application
 *
 *  \param  handle      [IN]       Handle to the McBSP Driver
 *  \param  Arg     [IN]        Miscellenous Argument
 *  \return PSP_SOK or PSP Error code
 */

PSP_Result PSP_mcbspFlushRequests(PSP_Handle handle,Ptr Arg);


/** \fn PSP_Result PSP_mcbspAbortRequests(PSP_Handle hMcBSPChan,Ptr arg)
 *
 *  \brief Abort the McBSP channel operation
 *
 *   Abort the input or output channel operation of the McBSP driver
 *   and flush all pending I/O packets. Driver sends the
 *   notifications to application for all pending I/O packets
 *   The application can start freshly by submitting new I/O requests.
 *
 *  \param  hMcBSPChan  [IN]     Handle to the McBSP channel
 *  \param  Arg         [IN]     Miscellenous Argument
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_mcbspAbortRequests(PSP_Handle handle,Ptr Arg);

/*!
 *  \fn PSP_Result PSP_mcbspIoctl (PSP_Handle hMcBSPChan, PSP_McbspIoctlCmd cmd, Ptr arg, Ptr param);
 *  \brief Implements the IOCTLS for McBSP IOM mini driver.
 *
 *  ddc_mcbspControlChan() implements McBSP device specific control operations
 *  including Start/Stop, Pause/Resume, Channel and Device Reset,
 *  Mute-ON/OFF, McBSP sample rate generator configuration
 *
 *  \param  [IN]  hMcBSPChan    Channel handle
 *  \param  [IN]  cmd           control functionality to perform
 *  \param  [IN]  arg           optional device-defined data structure
 *  \param  [IN]  param         misc param argument
 *
 *  \return  success(PSP_SOK) or failure code
 */
PSP_Result PSP_mcbspIoctl ( PSP_Handle hMcBSPChan,
                            PSP_McbspIoctlCmd cmd,
                            Ptr arg,
                            Ptr param);

/*!
 *  \fn PSP_Result PSP_mcbspInit(Uint32 instanceId)
 *  \brief Initializes device structure
 *
 *  \param  [IN]  instanceId    Instance ID
 *
 *  \return  success(PSP_SOK) or failure code
 */
PSP_Result PSP_mcbspInit(Uint32 instanceId);

#ifdef __cplusplus
}
#endif

#endif /*_PSP_MCBSP_API_*/
