/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file   psp_nand.h
 *
 *  \brief  NAND interface definition
 *
 *  This file contains the definitions, that are needed by the application to
 *   utilize the services of the NAND device driver.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \authors    PSP Team
 *
 *  \note       Set tabstop to 4 (:se ts=4) while viewing this file in an
 *              editor
 *
 *  \version    1.0     Initial Version
 *
 *  <b>Compiler Switch Used</b>
 *
 *  <b>_DEBUG</b><br>
 *   This is used as a flag to compiler whether to include the debug statements
 *   inserted in the code into the final image. This flag helps to build DEBUG
 *   image of the program. For RELEASE images this is not passed to the compiler.
 *
 *  <b>CHIP_DM6437</b><br>
 *   This macro is used for select DM6437 specific header files.
 *
 *  <b>NAND_INSTRUMENTATION_ENABLED</b><br>
 *   This macro is used to enable instrumentation related code.
 *
 *  <b>PSP_NAND_DEBUG</b><br>
 *   This macro is used for debugging purposes. This macro enables display of
 *   general debug messages in the NAND driver code.
 *
 *  <b>NAND_DBG_LEVEL_1</b><br>
 *   This macro is used for debugging purposes. This macro enables display of
 *   debug messages for logical page read and writes operations.
 *
 *  <b>DDC_NAND_POWER_SAFE</b><br>
 *   This macro enables the software features that safeguard the NAND
 *   driver from power failures.
 *
 *  <b>DDC_NAND_BBM</b><br>
 *   This macro is used to enable the support for bad block management.
 *   If this macro is not enabled, bad block management code is not
 *   included in the compilation.
 *
 *  <b>NAND_DBG_LEVEL_3</b><br>
 *   This macro is used for debugging purposes only. It is used to display
 *   the physical page number mapped to the logical page number in FTL page
 *   read and write functions.
 *
 *  <b>DDC_NAND_8_BIT</b><br>
 *   This macro statically (compilation time) selects 8-bit data bus support.
 *   If this macro is defined, only 8-bit NAND devices are supported. If this
 *   macro is not defined, only 16-bit NAND devices are supported.
 *
 *  <b>NAND_DBG_LEVEL_2</b><br>
 *   This macro is used for debugging purposes only. It is used to display
 *   a message on the terminal when a read fails.
 *
 *  <b>DDC_NAND_ENABLE_ECC</b><br>
 *   This macro is used when software based ECC is required. In DM6437,
 *   this macro is disabled since the ECC is always calculated by hardware.
 *
 */

#ifndef _PSP_NAND_H_
#define _PSP_NAND_H_

#include <ti/sdo/pspdrivers/common/psp_common.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Dimensioning Configurable Elements */
#define PSP_NAND_NUM_INSTANCES              (1u)
/**< Number of NAND instances on the hardware.                              */
#define PSP_NAND_NUM_CHANNELS               (1u)
/**< Number of channels                                                     */
#define PSP_NAND_NUM_DEVICES_PER_CONTROLLER (1u)
/**< Max no. of slaves that can be connected on each NAND device instance.  */
#define PSP_NAND_TPOLL_MSECS                (100u)
/**< Poll interval                                                          */


typedef int PSP_NandInstanceId;
/**< Instance Id of NAND on the SoC.                                        */

/**
 * \brief Configuration structure (operational mode) of NAND.
 *
 * Configuration information on
 *  - the current opMode,
 *  - the input clock frequency of NAND and
 *  - whether to erase NAND devices at initialization \n
 * can be configured by the user before the start of the driver.
 */
typedef struct {
    PSP_OpMode  opMode;
    /**< Mode of operation of NAND controller                               */
    Uint32      inputClkFreq;
    /**< Input clock frequency                                              */
    Uint8       eraseAtInit;
    /**< TRUE or 1 - Erase all NAND devices on board during initialization.\n
     * FALSE or 0 - No erase at startup or init.\n
     * - No selective erase supported. \n
     * <b> NOTE: TO BE USED WITH EXTREME CAUTION! </b>                      */
} PSP_NandConfig;

/**
 * \brief Device Configuration structure of NAND.
 *
 * The following parameters are used, if the device ID
 * read from the device is not preset in the 'known' list
 * of device IDs.
 */
typedef struct {
    Int8            *name;
    /**< NAND Flash Name                        */
    Uint32          id;
    /**< NAND Flash Id                          */
    Uint32          bytesPerPage;
    /**< Bytes per page                         */
    Uint32          chipSize;
    /**< NAND Flash Total size                  */
    Uint32          eraseSize;
    /**< NAND Flash Erase size                  */
    Int             nand8Bit;
    /**< 8-bit or 16 bit                        */
    Uint32          nandOptions;
    /**< Extra cmds for Nand-Block              */
} PSP_NandFlashDev;

/**
 * \brief Type of NAND detected.
 */
typedef enum
{
    PSP_NAND_NONE = 0,                      /**< No NAND detected           */
    PSP_NAND_BIG_BLOCK,                     /**< Big Block NAND detected    */
    PSP_NAND_SMALL_BLOCK,                   /**< Small Block NAND detected  */
    PSP_NAND_INVALID                        /**< Unknown/Invalid NAND       */
} PSP_NandType;

/**
 *  \brief  PSP NAND Ioctl commands
 *
 *  NAND Ioctl commands
 */
typedef enum
{
    PSP_NAND_IOCTL_GET_OPMODE,
    /**< Get the current opmode                 */
    PSP_NAND_IOCTL_GET_NAND_SIZE,
    /**< Get the size of NAND                   */
    PSP_NAND_IOCTL_GET_SECTOR_SIZE,
    /**< Get sector size of NAND                */
    PSP_NAND_IOCTL_SET_OPMODE,
    /**< Set the current opmode                 */
    PSP_NAND_IOCTL_CHECK_NAND,
    /**< NAND device(s) present?                */
    PSP_NAND_IOCTL_SET_CFG_PARAMS,
    /**< Set NAND configuration params          */
    PSP_NAND_IOCTL_MAX_IOCTL
    /**< Book-keep - Max ioctl's                */
} PSP_NandIoctlCmd;


/**
 *  \brief Reads data from NAND instance.
 *
 *  This function reads 'xferRequest' bytes from the NAND device to 'buf',
 *  depending on the mode configured by the application. 'xferActual' holds
 *  the actual number of bytes read.
 *
 *  \param  hNand       [IN]    NAND Driver Instance Handle
 *  \param  logSector   [IN]    Sector to be read
 *  \param  pReadBuff   [OUT]   Character Buffer pointer - where read data is
 *                              to be stored
 *  \param  size        [IN]    Number of sector to be read
 *  \param  xferActual  [OUT]   Number of bytes actually transacted.
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result  PSP_nandReadSync(
                        PSP_Handle          hNand,
                        Uint32              logSector,
                        Uint16              *pReadBuff,
                        int                 size,
                        int                 *xferActual);

/**
 *  \brief Writes data to the NAND instance.
 *
 *  This function writes 'xferRequest' bytes to the NAND device from 'buf',
 *  depending on the mode configured by the application. 'xferActual' holds
 *  the actual number of bytes written.
 *
 *  \param  hNand       [IN]    NAND Driver Instance Handle
 *  \param  logSector   [IN]    Sector to be read
 *  \param  pWriteBuff  [OUT]   Character Buffer pointer - where read data is
 *                              to be stored
 *  \param  size        [IN]    Number of sector to be read
 *  \param  xferActual  [OUT]   Number of bytes actually transacted.
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result  PSP_nandWriteSync(
                        PSP_Handle          hNand,
                        Uint32              logSector,
                        Uint16              *pWriteBuff,
                        int                 size,
                        int                 *xferActual);


/**
 *  \brief Erases data on the NAND device.
 *
 *  API to be used with caution! This API erases all the blocks in the NAND
 *  and the action cannot be un-done. From a Filesystem perspective, this
 *  function formats the NAND device by calling the corresponding DDC layer
 *  API. This is an interface to the application.
 *
 *  \param  hNand       [IN]    NAND driver instance handle
 *
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result  PSP_nandErase(
                        PSP_Handle          hNand);

/**
 *  \brief Set the configuration parameters for new NAND device.
 *
 *  This API sets the following configuration parameters:
 *   Bytes per page,
 *   Pages per block,
 *   Size of NAND
 *   Bus Width: 8/16 bit.
 *  These parameters shall be used when the device ID read
 *  from the device doesn't match the one in the known 'list'.
 *
 *  <b>NOTE</b>: Use caution while setting these parameters, as they are
 *        used as-is while initializing the device.
 *
 *  \param  cfgParams   [IN]    Configuration parameter structure
 *
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result  PSP_nandAddDevice(
                        const PSP_NandFlashDev    *cfgParams);

/**
 *  \brief Initialize the device and the driver.
 *
 *  Called by the application, this API initializes the NAND
 *  device and the driver. It prepares the background for the
 *  driver to launch.
 *
 *  \param  instanceId  [IN]     NAND instance number
 *  \param  config      [IN]     Driver config information
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_nandInit(
                        PSP_NandInstanceId  instanceId,
                        const PSP_NandConfig      *config);

/**
 *  \brief Opens the NAND device instance.
 *
 *  Opens the device for transaction.
 *
 *  \param  instanceId  [IN]     NAND instance number
 *  \return PSP_Handle          Nand Driver Instance Handle
 */
PSP_Handle  PSP_nandOpen(
                        PSP_NandInstanceId  instanceId);

/**
 *  \brief Closes the NAND driver.
 *
 *  This API closes the driver that was opened by the Open call.
 *  Any specific objects that were created/used by the open call
 *  are freed here.
 *
 *  \param  hNand       [IN]    Nand Driver Instance Handle
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_nandClose(
                        PSP_Handle          hNand);
/**
 *  \brief Terminates a driver.
 *
 *  This function de-initializes and deletes the driver for a particular
 *  instance.
 *
 *  \param  instanceId  [IN]     NAND instance number.
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_nandTerminate(
                        PSP_NandInstanceId  instanceId);

/**
 *  \brief IO Control for NAND controller.
 *
 *  This function supports various IOCTLs for the NAND controller.
 *
 *  \param  hNand       [IN]    NAND Driver Instance Handle
 *  \param  cmd         [IN]    IOCTL command
 *  \param  cmdArg      [IN]    Arguments, if any, for the command
 *  \param  param       [IN]    IOCTL specific parameter (driver specific).
 *  \return PSP_SOK or PSP Error code.
 */
PSP_Result  PSP_nandIoctl(
                        PSP_Handle          hNand,
                        PSP_NandIoctlCmd    cmd,
                        void*               cmdArg,
                        void*               param);

#ifdef __cplusplus
}
#endif

#endif  /* _PSP_NAND_H_ */

