/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 * \file    dda_nand.c
 *
 * \brief  DDA Initialization for NAND on DM6437
 *
 * This file contains the driver adaptations implemented for the NAND controller
 * on DM6437.
 *
 * (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \note       Set tabstop to 4 (:se ts=4) while viewing this file in an
 *              editor
 *
 *  \author     AK Mistral, Himamsu Mistral,Kiran Sutariya
 *
 *  \version    1.0 AK Mistral      Created.
 *  \version    1.1 Himamsu Mistral Code cleanup and release for BFT5
 *  \version    1.2 Himamsu Mistral Release for BFT6
 *  \version    1.3 Himamsu Mistral Code cleanup and release for BFT7
 *  \version    1.4 Himamsu Mistral Added Multiple NAND support,
 *                                  - Release for BFT8
 *  \version    1.5 Kiran Sutariya  Code cleanup and Modified for DM6437
 *
 */

#include <ti/sdo/pspdrivers/drivers/nand/src/ddc_nand.h>
#include <ti/sdo/pspdrivers/drivers/nand/src/dda_nand.h>

#include <ti/sdo/pspdrivers/pal_os/pal_osmem.h>
#include <ecm.h>
#include <c64.h>
#ifdef NAND_INSTRUMENTATION_ENABLED
#include <ti/sdo/pspdrivers/pal_os/bios/psp_log.h>
#endif /*NAND_INSTRUMENTATION_ENABLED*/

/*
 * GLOBAL Data Declarations
 */


#pragma DATA_ALIGN(gNAND_writeBuf, CACHE_LINE_LENGTH);
Uint16      gNAND_writeBuf[DDC_NAND_WRT_BUF_SIZE + (CACHE_LINE_LENGTH/2)];
/**< Write buffer                                                           */

extern Uint16   gNAND_phyLogBank[DDC_NAND_MAX_BANKS][DDC_NAND_DEF_PHY_BUF_SIZE];
extern Uint16   gNAND_bbmBitBufBank[DDC_NAND_MAX_BANKS][DDC_NAND_DEF_BBM_BUF_SIZE];
extern Uint16   gNAND_wlaBitBufBank[DDC_NAND_MAX_BANKS][DDC_NAND_DEF_WLA_BUF_SIZE];
extern DDC_NandCacheBlkRedInfo gBlockRedInfoBank[DDC_NAND_MAX_BANKS][DDC_NAND_NO_BLKS_CACHED];

extern DDC_NandBankSectorInfo  gNandBankSectorInfo [DDC_NAND_MAX_BANKS];

DDA_NandObj gDDA_NandObj[PSP_NAND_NUM_INSTANCES];
/**< Instance of NAND object */

extern DDC_NandFlashDev     gNandCfgParams;
/**< NAND Configuration parameters  */

/*
 * Common structure definition for PrFILE and NAND
 */
DDC_NandState  gDDC_NandState [DDC_NAND_MAX_BANKS];
/**< NAND State structure   */

DDC_NandAsyncBank gDDCNandAsyncBank [DDC_NAND_MAX_BANKS];
/**< NAND Asynchronous bank configuration info for banks */

/*
 * Static Data Declarations
 */

static Uint32   initCalled = 0;

/*
 * Static Function Prototypes
 */
static PSP_Result _init(
                PSP_NandInstanceId   instanceId,
                const PSP_NandConfig       *config);

static PSP_Result PSP_nandStateInit (
                DDA_NandObj    *DDA);

/*
 * Exported Functions
 */

/**
 *  \brief Initialize the device and the driver.
 *
 *  Called by the application, this API initializes the NAND
 *  device and the driver. It prepares the background for the
 *  driver to launch.
 *
 *  \param  instanceId  [IN]    NAND instance number
 *  \param  config      [IN]    Driver config information
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_nandInit(
            PSP_NandInstanceId  instanceId,
            const PSP_NandConfig      *config)
{
    PSP_Result      result = PSP_SOK;
    DDA_NandObj     *DDA = &gDDA_NandObj[instanceId];

    if(0 == initCalled)
    {
        /* Initialize the value, so that we are being called again */
        initCalled = 1;

        PAL_osMemSet((void*)&gDDA_NandObj[instanceId],0x0,sizeof(DDA_NandObj));
        PAL_osMemSet((void*)&gDDCNandAsyncBank[instanceId], 0x0, sizeof(DDC_NandAsyncBank));

        if ((instanceId >= PSP_NAND_NUM_INSTANCES) || (instanceId < 0)) {
            result = PSP_E_INVAL_PARAM;
        }

        else if( (NULL != DDA) && (NULL != config)) {
            if (DDA->ddaState != NAND_DELETED) {
                result = PSP_E_INVAL_STATE;
            }

            else
            {
                result = _init(instanceId, config);

                if (PSP_SOK == result) {
                    DDA->ddaState = NAND_CREATED;
                }

            }
        }
        else
        {
            result = PSP_E_INVAL_PARAM;
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }
    return (result);
}

/**
 *  \brief Initializes the NAND device instance.
 *
 *  Initializes the NAND device for transaction, by creating the instance
 *  of the driver.
 *
 *  \param  instanceId  [IN]    NAND instance number
 *  \param  config      [IN]    configuration information.
 *  \return PSP_Result          PSP_SOK or PSP Error code
 */
static PSP_Result _init(
            PSP_NandInstanceId  instanceId,
            const PSP_NandConfig      *config)
{
    PSP_Result              retval = PSP_SOK;
    PAL_Result              result;
    DDC_Handle              ddcHandle;
    DDC_NandInitConfig      initConfig;
    DDA_NandObj             *DDA = &gDDA_NandObj[instanceId];

    if(((NULL != DDA) && (NULL != config)) && (config->inputClkFreq > 0) )
    {
        DDA->instanceId = instanceId;

        ddcHandle = (Ptr)&DDA->ddcObj;

        /* Create Nand Instance */
        result = nandCreateInstance(
                        DDA->instanceId,
                        DDA,
                        &ddcHandle,
                        NULL);
        if (PAL_SOK != result) {
            retval = PSP_E_DRIVER_INIT;
        }
        else {
            initConfig.instId = DDA->instanceId;
            initConfig.opMode = (PSP_OpMode)config->opMode;
            initConfig.msecPoll = PSP_NAND_TPOLL_MSECS;

            DDA->llc.opMode = (PSP_OpMode)(config->opMode);

            initConfig.llc = (LLC_NandObj *)&DDA->llc;
            initConfig.llc->clkFreq = config->inputClkFreq;
            initConfig.llc->eraseAtInit = (Uint8)config->eraseAtInit;


            result = PSP_nandStateInit(DDA);
            if (PAL_SOK != result) {
                retval = PSP_E_DRIVER_INIT;
            }
            else {
                result = nandInit(&DDA->ddcObj, &initConfig);

                if (PAL_SOK != result) {
                    nandDelInst(&DDA->ddcObj, NULL);
                    retval = PSP_E_DRIVER_INIT;
                }
            }
        }
    }
    else {
        retval = PSP_E_INVAL_PARAM;
    }

    return (retval);
}

/**
 *  \brief NAND State Initialize.
 *
 *  Initialize the NAND State structure.
 *
 *  \param  DDA         [IN]    DDA Handle
 *
 *  \return PSP_SOK or PSP Error code
 */
static PSP_Result PSP_nandStateInit (
            DDA_NandObj    *DDA)
{
    PSP_Result result=PSP_SOK;
    Uint32  bankNo;

    if((NULL != DDA) && (NULL != &gDDC_NandState)) {
        /* Ensure NAND State structure is initialized */
        for (bankNo = 0; bankNo < DDC_NAND_MAX_BANKS; bankNo++) {
            PAL_osMemSet((void*)&gDDC_NandState[bankNo], 0x0, sizeof(gDDC_NandState));

            gDDC_NandState[bankNo].entryBufferSize      = DDC_NAND_ENT_BUF_SIZE;
            gDDC_NandState[bankNo].deviceID             = DDC_NAND_DEVICE_ID;
            gDDC_NandState[bankNo].nandOptions          = 0;
            gDDC_NandState[bankNo].oldPhyBlock          =
                                                DDC_NAND_DEF_OLD_PHY_BLOCK;
            gDDC_NandState[bankNo].currentLogBlock      =
                                                DDC_NAND_DEF_LOGICAL_BLOCK;
            gDDC_NandState[bankNo].prevWriteZone        =
                                                DDC_NAND_DEF_PREV_WRITTEN_ZONE;
            gDDC_NandState[bankNo].WLAflag              = DDC_NAND_WLA_FLAG;
            gDDC_NandState[bankNo].currentSector        = DDC_NAND_DEF_SECTOR;

        /* Align the buffer */
        gDDC_NandState[bankNo].pWriteBuffer         =
            (Uint16 *)(((Uint32) gNAND_writeBuf + (CACHE_LINE_LENGTH - 1))
                        & ~(CACHE_LINE_LENGTH - 1));

            gDDC_NandState[bankNo].noOfBadBlocks        = 0;
            gDDC_NandState[bankNo].noOfBadPages         = 0;
            gDDC_NandState[bankNo].IntrDisable          = NULL;
            gDDC_NandState[bankNo].IntrEnable           = NULL;
            gDDC_NandState[bankNo].nandDevFlags         = 0;
        }

        PAL_osMemSet (gNAND_writeBuf,
                      (char) DDC_NAND_DEF_INIT_VALUE,
                      sizeof(gNAND_writeBuf));

        for (bankNo = 0; bankNo < DDC_NAND_MAX_BANKS; bankNo++) {

        PAL_osMemSet (gNAND_bbmBitBufBank[bankNo],
                (char) DDC_NAND_DEF_INIT_VALUE,
                (DDC_NAND_DEF_BBM_BUF_SIZE * 2));
        PAL_osMemSet (gNAND_wlaBitBufBank[bankNo],
                (char) DDC_NAND_DEF_INIT_VALUE,
                (DDC_NAND_DEF_WLA_BUF_SIZE * 2));

            gDDC_NandState[bankNo].pBlockRedInfo  = gBlockRedInfoBank[bankNo];
            gDDC_NandState[bankNo].pPhyLogEntries = gNAND_phyLogBank[bankNo];
            gDDC_NandState[bankNo].pBBMbitbuffer  = gNAND_bbmBitBufBank[bankNo];
            gDDC_NandState[bankNo].pWLAbitbuffer  = gNAND_wlaBitBufBank[bankNo];
            DDA->ddcObj.nandState [bankNo] = gDDC_NandState [bankNo];
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    return (result);
}

static Int  nandDmaintrNo = 0;

/**
 *  \brief NAND DMA intrrupt service routine
 *
 *  Intrrupt service routine of Nand in DMA mode of operation.This will be taken
 *  care by DMA and Nand driver will not do anything.
 *
 *  \param  DDA         [IN]    DDA Handle
 *
 *  \return PSP_SOK or PSP Error code
 */
/* ARGSUSED */
static void DDA_nandDmaIsrHandler(Uint32 arg)
{
    DDA_NandObj         *DDA = (DDA_NandObj *)&gDDA_NandObj[0];

    if(nandDmaintrNo != 0xFFFF)
    {
        ECM_disableEvent(CSL_INTC_EVENTID_EMIFAINT);

        LLC_nandHandleDmaISR (&DDA->llc);

        ECM_enableEvent(nandDmaintrNo);
    }
}

/**
 *  \brief Register NAND DMA mode intrrupt service routine
 *
 *  Intrrupt service routine of Nand in DMA mode of operation.This will be taken
 *  care by DMA and Nand driver will not do anything.
 *
 *  \param  DDA         [IN]    DDA Handle
 *
 *  \return PSP_Result  [OUT]   PSP_SOK or PSP Error code
 */
static PAL_Result setDmaModeIsr(
                DDA_NandObj*        DDA)
{
    ECM_Attrs                   ecmattrs = ECM_ATTRS;
    PAL_OsSemAttrs              SemAttr;
    PSP_Result result = PSP_SOK;

    if (NULL != DDA) {
        SemAttr . type = PAL_OSSEM_TYPE_FIFO;
        SemAttr . memSegId = 0;

        nandDmaintrNo = LLC_nandSetDmaMode (&DDA->llc);

        if(nandDmaintrNo != 0xFFFFFFFF)
        {
            /* Driver event semaphore */
            if(DDA->llc.syncSem == NULL)
            {
                if(PAL_SOK == PAL_osSemCreate ("NAND", 1, &SemAttr,
                                                &DDA->llc.syncSem))
                {
                    if(PAL_SOK != PAL_osSemTake (DDA->llc.syncSem,
                                                 PAL_OSSEM_NO_TIMEOUT))
                    {
                        NAND_Debug ("\r\nNAND: Taking syncsem failed after create  \r\n");
                    }
                }
                else
                {
                    NAND_Debug ("\r\nNAND: Creating syncsem failed \r\n");
                }
                if (PSP_OPMODE_DMAINTERRUPT == DDA->llc.opMode)
                {
                    /* DMA INT is handled by DMA driver.. not by us */
                    if(nandDmaintrNo != 0xFFFF)
                    {
                        ecmattrs.unmask = 1u;
                        ECM_dispatchPlug(nandDmaintrNo,
                                        (ECM_Fxn)&DDA_nandDmaIsrHandler,
                                        &ecmattrs);

                        /* Enabling the dma event */
                        ECM_enableEvent(nandDmaintrNo);
                        /* Enabling the HWI_ID */
                        C64_enableIER(HWI_EVT02_CMB_IRQ);
                    }
                }
            }
        }
    }
    else {
        result = PSP_E_INVAL_PARAM;
    }
    return (result);
}

/**
 *  \brief Clear NAND DMA mode routine
 *
 *  Clear routine of Nand in DMA mode of operation.This will be taken
 *  care by DMA and Nand driver will not do anything.
 *
 *  \param  DDA         [IN]    DDA Handle
 *
 *  \return PSP_Result  [OUT]   PSP_SOK or PSP Error code
 */
static PAL_Result clearDmaMode(
            DDA_NandObj*        DDA)
{
    PSP_Result result = PSP_SOK;

    result = PAL_osSemDelete(DDA->llc.syncSem);
    /* Free the semaphore */
    if(PAL_SOK != result)
    {
        NAND_Debug ("\r\nNAND: Deleting syncsem failed \r\n");
        return (PSP_E_INVAL_PARAM);
    }

    DDA->llc.syncSem = NULL;

    /* Close DMA */
    result = LLC_nandCloseDmaMode(&DDA->llc);
    if(PAL_SOK != result)
    {
        NAND_Debug ("\r\nNAND: Deleting EDMA channel failed \r\n");
        return (PSP_E_INVAL_PARAM);
    }

    return (PAL_SOK);
}

/**
 *  \brief Opens the NAND device instance.
 *
 *  Opens the device for transaction.
 *
 *  \param  instanceId  [IN]    NAND instance number
 *
 *  \return PSP_Handle Nand Driver Instance Handle
 */
PSP_Handle  PSP_nandOpen(
            PSP_NandInstanceId    instanceId)
{
    PSP_Result          result;
    DDC_NandOpenParams  openParams;
    DDA_NandObj         *DDA = (DDA_NandObj *)&gDDA_NandObj[instanceId];
    PSP_Handle          handle;
    PAL_OsSemAttrs      semAttr;

    if((NULL != DDA) && ((instanceId < PSP_NAND_NUM_INSTANCES) &&
                    ((NAND_CREATED == DDA->ddaState) ))) {

        semAttr.type        = PAL_OSSEM_TYPE_FIFO;
        semAttr.memSegId    = 0;
        result = PAL_osSemCreate("NAND", 1, &semAttr, &DDA->devBusySem);
        if (PAL_SOK != result) {
            handle = NULL;
        }
        else
        {
            handle = (PSP_Handle)DDA;
            /* Register DMA ISR */
            if(PSP_OPMODE_DMAINTERRUPT == DDA->llc.opMode)
            {
                result = setDmaModeIsr(DDA);
                if (PAL_SOK != result) {
                    handle = NULL;
                }
            }
            if (handle != NULL) {
                openParams.hwConfig = (LLC_NandHwConfig*)&gDDA_NandHwConfig;

                /* Open NAND - setup HW */
                result = nandOpen(&DDA->ddcObj, &openParams);

                if(PSP_SOK == result) {
                    DDA->ddaState = NAND_OPENED;
                    }
            }
            else {
                handle = NULL;
            }
        }
    }
    return handle;
}
/**
 *  \brief Closes the NAND driver.
 *
 *  This API closes the driver that was opened by the Open call.
 *  Any specific objects that were created/used by the open call
 *  are freed here.
 *
 *  \param  hNand   [IN]    Nand Driver Instance Handle
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_nandClose(
            PSP_Handle      hNand)
{
    DDA_NandObj         *DDA;
    PSP_Result  result = PSP_SOK;

    if(NULL != hNand) {
        DDA = (DDA_NandObj*)hNand;

        if(NAND_OPENED == DDA->ddaState) {
            if(DDA->llc.opMode == PSP_OPMODE_DMAINTERRUPT)
            {
                result = clearDmaMode(DDA);
                if (result != PSP_SOK)
                {
                    result = (PSP_E_INVAL_PARAM);
                }
            }

            result = PAL_osSemDelete(DDA->devBusySem);

            if( PAL_SOK == result) {
                result = nandClose(&DDA->ddcObj, NULL);
                if( PAL_SOK == result) {
                    DDA->ddaState = NAND_CLOSED;
                }
            }
        }
        else {
            result = PSP_E_INVAL_STATE;
        }
    }
    else {
        result = PSP_E_INVAL_PARAM;
    }
    return result;
}

/**
 *  \brief Terminates a driver.
 *
 *  This function de-initializes and deletes the driver for a particular
 *  instance.
 *
 *  \param  instanceId  [IN]    NAND instance number.
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_nandTerminate(
            PSP_NandInstanceId   instanceId)
{
    PSP_Result result = PSP_SOK;
    DDA_NandObj  *DDA = &gDDA_NandObj[instanceId];

    if ((instanceId < PSP_NAND_NUM_INSTANCES) && (NULL != DDA)) {
        if(!((DDA->ddaState != NAND_CREATED) &&
                                (DDA->ddaState != NAND_CLOSED))) {
            result =  nandDeinit(&DDA->ddcObj, NULL);
            if (PSP_SOK == result) {
                result = nandDelInst(&DDA->ddcObj, NULL);
            }
            if (PSP_SOK == result) {
                DDA->ddaState = NAND_DELETED;
                initCalled = 0;
            }
        }
        else {
            result = PSP_E_INVAL_PARAM;
        }
    }
    return result;
}

/**
 *  \brief Reads data from NAND instance.
 *
 *  This function reads 'xferRequest' bytes from the NAND device to 'buf',
 *  depending on the mode configured by the application. 'xferActual' holds
 *  the actual number of bytes read.
 *
 *  \param  hNand       [IN]    NAND Driver Instance Handle
 *  \param logSector    [IN]    Sector to be read
 *  \param  pReadBuff   [OUT]   Character Buffer pointer - where read data is
 *                              to be stored
 *  \param  size        [IN]    Number of sector to be read
 *  \param  xferActual  [OUT]   Number of bytes actually transacted.
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_nandReadSync(
            PSP_Handle          hNand,
            Uint32              logSector,
            Uint16              *pReadBuff,
            Int32               size,
            Int32               *xferActual)
{
    DDA_NandObj             *DDA;
    PSP_Result              result = PSP_E_INVAL_PARAM;
    Uint32                  curLogSector = 0;

#ifdef NAND_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT(  &DVTEvent_Log,
                    "NAND",
                    PSP_DVT_DESC(PSP_DVT_eFUNC_START,
                                 PSP_DVT_dCHANNEL_ID_INST,
                                 PSP_DVT_dPACKET_ID,
                                 PSP_DVT_dSIZE_BYTES),
                    CHANNELID_INSTANCEID(
                        0,
                        0),
                    pReadBuff,
                    (size*512) );
#endif /* NAND_INSTRUMENTATION_ENABLED */

    if((NULL != hNand) && ((NULL != pReadBuff) && (NULL != xferActual))) {

        DDA = (DDA_NandObj *)hNand;

        /* Take the semaphore */
        result = PAL_osSemTake(DDA->devBusySem, PAL_OSSEM_NO_TIMEOUT);

        if ((PSP_SOK == result) && (DDA->ddaState == NAND_OPENED)) {
            if (logSector <= DDA->ddcObj.maxNandSectors) {
                /* Get the bank into which the logSector falls and adjust
                 * the logSector number also accordingly. */
                result = (PSP_Result)ddcNandBankConfigure(&DDA->ddcObj,
                                logSector,&curLogSector);

                if(PSP_SOK == result) {
                    logSector = curLogSector;
                    result = nandReadSync(
                                    &DDA->ddcObj,
                                    pReadBuff,
                                    size,
                                    DDC_STREAM_WAIT_FOREVER,
                                    (Ptr)&logSector,
                                    xferActual);
                }

            }
            else {
                result = PSP_E_INVAL_PARAM;
            }
        }
        else {
            result = PSP_E_INVAL_STATE;
        }

        /* Release the semaphore */
        PAL_osSemGive(DDA->devBusySem);

    }

#ifdef NAND_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT( &DVTEvent_Log,
                   "NAND",
                   PSP_DVT_DESC(PSP_DVT_eFUNC_END,
                                PSP_DVT_dCHANNEL_ID_INST,
                                PSP_DVT_dPACKET_ID,
                                PSP_DVT_dSIZE_BYTES),
                   CHANNELID_INSTANCEID(
                       0,
                       0),
                   pReadBuff,
                   0);
#endif /* NAND_INSTRUMENTATION_ENABLED */

    return (result);
}

/**
 *  \brief Writes data to the NAND instance.
 *
 *  This function writes 'size' bytes to the NAND device from 'pWriteBuff',
 *  depending on the mode configured by the application. 'xferActual' holds
 *  the actual number of bytes written.
 *
 *  \param  hNand       [IN]    NAND Driver Instance Handle
 *  \param  logSector   [IN]    Sector to be read
 *  \param  pWriteBuff  [OUT]   Character Buffer pointer - where read data is
 *                              to be stored
 *  \param  size        [IN]    Number of sector to be written
 *  \param  xferActual  [OUT]   Number of bytes actually transacted.
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_nandWriteSync(
            PSP_Handle          hNand,
            Uint32              logSector,
            Uint16              *pWriteBuff,
            Int32               size,
            Int32               *xferActual)
{
    DDA_NandObj             *DDA;
    PSP_Result              result = PSP_E_INVAL_PARAM;
    Uint32                  curLogSector = 0;

#ifdef NAND_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT(      &DVTEvent_Log,
                        "NAND",
                        PSP_DVT_DESC(   PSP_DVT_eFUNC_START,
                                        PSP_DVT_dCHANNEL_ID_INST,
                                        PSP_DVT_dPACKET_ID,
                                        PSP_DVT_dSIZE_BYTES),
                        CHANNELID_INSTANCEID(
                            1,
                            0),
                        pWriteBuff,
                        (size*512));
#endif /* NAND_INSTRUMENTATION_ENABLED */

    if((NULL != hNand) && ((NULL != pWriteBuff) && (NULL != xferActual))) {
        DDA = (DDA_NandObj *)hNand;

        /* Take the semaphore */
        result = PAL_osSemTake(DDA->devBusySem, PAL_OSSEM_NO_TIMEOUT);

        if ((PSP_SOK == result) && (DDA->ddaState == NAND_OPENED)) {
            if (logSector <= DDA->ddcObj.maxNandSectors) {
                /* Get the bank into which the logSector falls and adjust
                 * the logSector number also accordingly. */
                result = (PSP_Result)ddcNandBankConfigure(&DDA->ddcObj,
                                 logSector, &curLogSector);

                if(PSP_SOK == result) {
                    logSector = curLogSector;
                    result = nandWriteSync(
                                    &DDA->ddcObj,
                                    (Uint16*)pWriteBuff,
                                    size,
                                    DDC_STREAM_WAIT_FOREVER,
                                    (Ptr)&logSector,
                                    xferActual);
                }

            }
            else {
                result = PSP_E_INVAL_PARAM;
            }
        }
        else {
            result = PSP_E_INVAL_STATE;
        }

        /* Release the semaphore */
        PAL_osSemGive(DDA->devBusySem);
    }

#ifdef NAND_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT(  &DVTEvent_Log,
                    "NAND",
                    PSP_DVT_DESC(PSP_DVT_eFUNC_END,
                                 PSP_DVT_dCHANNEL_ID_INST,
                                 PSP_DVT_dPACKET_ID,
                                 PSP_DVT_dSIZE_BYTES),
                    CHANNELID_INSTANCEID(
                        1,
                        0),
                    pWriteBuff,
                    0);
#endif /* NAND_INSTRUMENTATION_ENABLED */

    return (result);
}

/**
 *  \brief Erases data on the NAND device.
 *
 *  API to be used with caution! This API erases all the blocks in the NAND
 *  and the action cannot be un-done. From a Filesystem perspective, this
 *  function formats the NAND device by calling the corresponding DDC layer
 *  API. This is an interface to the application.
 *
 *  \param  hNand   [IN]    NAND driver instance handle
 *
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result  PSP_nandErase(
            PSP_Handle          hNand)
{
    PSP_Result              result = PSP_E_INVAL_PARAM;
    DDA_NandObj             *DDA;

    if(NULL != hNand) {
        DDA = (DDA_NandObj *)hNand;

        result = nandControl(
                        &DDA->ddcObj,
                        DDC_NAND_IOCTL_ERASE,
                        NULL,
                        NULL);

    }
    return (result);
}

/**
 *  \brief Set the configuration parameters for new NAND device.
 *
 *  This API sets the following configuration parameters:
 *   Bytes per page,
 *   Pages per block,
 *   Size of NAND
 *   Bus Width: 8/16 bit.
 *  These parameters shall be used when the device ID read
 *  from the device doesn't match the one in the known 'list'.
 *
 *  <b>NOTE</b>: Use caution while setting these parameters, as they are
 *        used as-is while initializing the device.
 *
 *  \param  cfgParams   [IN]    Configuration parameter strucure
 *
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result  PSP_nandAddDevice(
            const PSP_NandFlashDev     *cfgParams)
{
    PSP_Result result = PSP_E_INVAL_PARAM;

    if(NULL != cfgParams) {

        if((cfgParams->bytesPerPage >= 256u) &&
                ((LLC_NAND_16_BIT == cfgParams->nand8Bit) ||
                 (LLC_NAND_8_BIT == cfgParams->nand8Bit))) {
            /* Update the global variable. */
            gNandCfgParams.name         = cfgParams->name;
            gNandCfgParams.id           = cfgParams->id;
            gNandCfgParams.bytesPerPage = cfgParams->bytesPerPage;
            gNandCfgParams.chipSize     = cfgParams->chipSize;
            gNandCfgParams.eraseSize    = cfgParams->eraseSize;
            gNandCfgParams.nand8Bit     = (LLC_NandWidth) cfgParams->nand8Bit;
            gNandCfgParams.nandOptions  = cfgParams->nandOptions;

            result = PSP_SOK; /* Success: Update return value */

        }
    }

    return (result);
}

/**
 *  \brief IO Control for NAND controller.
 *
 *  This function supports various IOCTLs for the NAND controller.
 *
 *  \param  hNand   [IN]    NAND Driver Instance Handle
 *  \param  cmd     [IN]    IOCTL command
 *  \param  cmdArg  [IN]    Arguments, if any, for the command
 *  \param  param   [IN]    IOCTL specific parameter (driver specific).
 *  \return PSP_SOK or PSP Error code.
 */
PSP_Result PSP_nandIoctl(
            PSP_Handle          hNand,
            PSP_NandIoctlCmd    cmd,
            void*               cmdArg,
            void*               param)
{
    DDA_NandObj             *DDA;
    PSP_Result              result = PSP_SOK;
    Uint32                  bankNo;

    /* Note param arg4 is not used intentionally. For future use */
    /* To remove remark while compiling using -pdr diagnostics*/
    param = param;


    if( (NULL != hNand) && (NULL != cmdArg) ) {

        DDA = (DDA_NandObj*)hNand;

        if((NAND_OPENED == DDA->ddaState)
                        && (cmd < PSP_NAND_IOCTL_MAX_IOCTL)) {

            switch (cmd) {
                /* GET IOCTL */
                case PSP_NAND_IOCTL_GET_OPMODE:
                    *(Int32*)cmdArg = (Int32)DDA->llc.opMode;
                    break;


                case PSP_NAND_IOCTL_CHECK_NAND:
                    *(Int32 *) cmdArg = 0; /* No NAND media by default */
                    for (bankNo = 0; bankNo < DDC_NAND_MAX_BANKS; bankNo++) {
                        if (0 != DDA->ddcObj.nandState[bankNo].nandType) {
                            *(Int32 *) cmdArg =
                                    (Int32)DDA->ddcObj.nandState[bankNo].nandType;
                            break; /* atleast 1 NAND device detected */
                        }
                    }
                    break;

                case PSP_NAND_IOCTL_GET_NAND_SIZE:
                    *(Uint32 *) cmdArg = (Uint32)(DDA->ddcObj.maxNandSectors);
                    break;

                case PSP_NAND_IOCTL_GET_SECTOR_SIZE:
                    *(Uint32 *)cmdArg = (Uint32)DDA->ddcObj.maxNandSectorSize;
                    break;

                default:
                    result = PSP_E_INVAL_PARAM;
                    break;
            }
        }
        else {
            result = PSP_E_INVAL_PARAM;
        }
    }
    return (result);
}

