/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/** \mainpage PSP NAND Driver on DSP/BIOS
 *
 *  \section Introduction
 *
 *  \subsection xxx Purpose and Scope
 *  The purpose of this document is to explain the NAND driver on the PSP
 *  Framework architecture.
 *
 *  \subsection aaa PSP Architecture
 * A PSP framework is a software entity providing platform support
 *  for devices that are based on Texas Instruments SoC's. The specific
 *  (Hardware and software) implementation of the PSP framework (the PAL
 *  implementation and the group of device drivers) is referred to as
 *  PSP System. The PSP framework defines a software environment for device
 *  drivers to plug-in by defining standard usage interfaces and
 *  abstractions (OS, CPU/Peripherals). For every platform, the PSP
 *  implements platform specific functionality such as OS adaptation, CPU and
 *  peripherals specific mechanisms, and generic system services such as
 *  debug control,traces, persistent storage, etc.
 *
 *  \subsection bbb NAND Flash Controller
 *  NAND flash is a memory device based on Electrically Erasable-Programmable
 *  Read only memory (EEPROM) technology. The NAND device driver was
 *  implemented for two types of NAND: Big Block and Small block. Though the
 *  lowest communication layer is a bit different, the API structure and
 *  algorithms needed for Error Checking and Correction (ECC), Bad Block
 *  Marking (BBM) and Wear-Leveling Algorithm (WLA) are almost the same. For
 *  reusability, modularization, and to reduce the duplication of efforts,
 *  NAND flash driver is designed in the following manner.
 *      - <b>Flash Translation Layer (FTL)</b>
 *      - <b>Physical Interface Layer</b>
 *
 *  \subsubsection ccc Flash Translation Layer
 *  This layer is the one that interfaces with the application software (for
 *  ex a File System). This layer is responsible for all the software
 *  processing needed before interfacing with the application. This layer
 *  also provides APIs for the application software that uses this library.
 *  The jobs done by this layer typically are:
 *      - Logical to Physical Mapping
 *      - Bad Block Marking (BBM)
 *      - Error Check and Correction (ECC)
 *      - Wear Leveling Algorithm (WLA)
 *
 *  \subsubsection ddd Physical Interface Layer
 *  This layer communicates with the hardware and provides APIs for Flash
 *  Translation layer to perform the rest of the process. This layer is not
 *  standalone. There has to be an abstraction layer above this layer which
 *  follows the protocol specified by the NAND specifications to assure
 *  proper read/write operations from/to the NAND.
 *
 *
 *
 *  \subsection fff Terms and Abbreviations
 *   -# API:  Application Programmer Interface
 *   -# PSP:  Platform Support Package
 *
 *
 */
/**
 * \file dda_nand.h
 *
 * \brief DDA defined for NAND device
 *
 * This file contains the DDA definitions for NAND. This is the main place for
 * memory allocation across the driver. This file defines the PSP Framework
 * for NAND.
 *
 * (C) Copyright 2006, Texas Instruments, Inc
 *
 * \author  Himamsu Mistral
 *
 *  \version    1.0 Himamsu Mistral Created.
 *  \version    1.1 Himamsu Mistral Release for BFT5
 *  \version    1.2 Himamsu Mistral Release for BFT6
 *  \version    1.3 Himamsu Mistral Release for BFT7
 *  \version    1.4 Himamsu Mistral Added Multiple NAND support,
 *                                  - Release for BFT8
 *  \version    1.5 Kiran Sutariya  Code cleanup and Modified for DM6437
 */

#ifndef _DDA_NAND_H_
#define _DDA_NAND_H_

#include <ti/sdo/pspdrivers/drivers/nand/psp_nand.h>
#include <ti/sdo/pspdrivers/drivers/nand/src/ddc_nand.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osSem.h>

#ifdef __cplusplus
extern "C" {
#endif


/**
 * \brief DDA State variable
 */
typedef enum
{
    NAND_DELETED = 0,                       /**< Instance Deleted           */
    NAND_CREATED,                           /**< Instance Created           */
    NAND_OPENED,                            /**< Instance Opened            */
    NAND_CLOSED                             /**< Instance Closed            */
} DDA_NandState;

/**
 * \brief Type of NAND detected.
 */
typedef enum
{
    DDA_NAND_NONE = 0,                      /**< No NAND detected           */
    DDA_NAND_BIG_BLOCK,                     /**< Big Block NAND detected    */
    DDA_NAND_SMALL_BLOCK,                   /**< Small Block NAND detected  */
    DDA_NAND_INVALID                        /**< Unknown/Invalid NAND       */
} DDA_NandType;

/**
 * \brief HW events for NAND media
 */
typedef enum
{
    DDA_NAND_HWEVENT_NO_MEDIA = 0,          /**< No Media HW Event          */
    DDA_NAND_HWEVENT_NAND_PRESENT,          /**< Media inserted             */
    DDA_NAND_HWEVENT_MAX                    /**< Book-keep                  */
} DDA_NandHwEvent;

typedef PSP_Handle  PSP_NandChannelHandle;
/**< Handle to the NAND channel                                             */
/**
 * \brief NAND DDA object
 */
typedef struct _DDA_NandObj {
    Int                     ddcIntrNum;     /**< Interrupt no of the device */

    DDA_NandState           ddaState;       /**< DDA State                  */
    Int32                   instanceId;     /**< Instance Id of NAND on SoC */
    DDC_NandObj             ddcObj;         /**< NAND Controller object     */
                                            /**< NAND Device object         */
    LLC_NandObj             llc;            /**< Low level controller info  */
    PAL_OsSemHandle         devBusySem;     /**< Device Busy Semaphore      */
} DDA_NandObj;

/**
 * \brief NAND Driver Device BIOS Parameter Block Information structure
 *
 */
typedef struct
{
  Uint8    Dummy        [11];   /**< Unused                                 */
  Uint8    BytesperSect [2];    /**< Bytes per Sector                       */
  Uint8    SecperClust;         /**< Sector per Cluster                     */
  Uint8    RsrvSect     [2];    /**< Reserved Sector Count                  */
  Uint8    NumFats;             /**< Number of Fat copies                   */
  Uint8    NumDirEnt    [2];    /**< Number of Root Directory Entries       */
  Uint8    TotSect16    [2];    /**< Tot sector count within 16 bits        */
  Uint8    MediaDisc;           /**< Media Descriptor                       */
  Uint8    TotFat16     [2];    /**< Tot FAT size in sectors within 16 bits */
  Uint8    SectperTrack [2];    /**< Sectors Per Track                      */
  Uint8    Head         [2];    /**< Number of Heads                        */
  Uint8    Dummy1       [4];    /**< Unused                                 */
  Uint8    TotSect32    [4];    /**< Tot sector count within 32 bits        */
} NAND_BootRec_t;

/**
 * \brief NAND Driver Device BPB Volume Information structure
 *
 */
typedef struct
{
  Uint8        DrvNum;          /**< Drive Number                           */
  Uint8        Dummy;           /**< Unused                                 */
  Uint8        BootSig;         /**< Boot Signature                         */
  Uint8        VolId    [4];    /**< Volume ID                              */
  Uint8        VolLbl   [11];   /**< Volume Label                           */
  Uint8        FsStr    [8];    /**< File System String                     */
} NAND_VolInfo_t;


/**
 * \brief NAND Driver Device BPB32 Information structure
 *
 */
typedef struct
{
  NAND_BootRec_t    Bpb;            /**< BIOS Parameter Block               */
  Uint8       TotFat32      [4];    /**< Tot FAT size in sectors in 32 bits */
  Uint8       FatMir        [2];    /**< FAT copy status info               */
  Uint8       Dummy         [2];    /**< Unused                             */
  Uint8       StartRtClust  [4];    /**< Start Cluster number of Root Dir   */
  Uint8       FsInfo        [2];    /**< File System Information            */
  Uint8       BootBkp       [2];    /**< Boot Backup Sector number          */
  Uint8       Dummy1        [12];   /**< Reserved                           */
  NAND_VolInfo_t Vol;               /**< Volume information                 */
} NAND_BootFat32_t;

/**
 * \brief NAND File System Information structure
 *
 */
typedef struct
{
  Uint8   LeadSig       [4];        /**< Lead Signature                     */
  Uint8   Dummy         [480];      /**< Reserved Info                      */
  Uint8   FsSig         [4];        /**< File system Signature              */
  Uint8   FreeCnt       [4];        /**< Last Known free cluster            */
  Uint8   NxtFree       [4];        /**< Next Free Cluster                  */
  Uint8   Dummy1        [12];       /**< Unused                             */
  Uint8   TrialSig      [4];        /**< Trial Signature                    */
} NAND_FileSysInfo_t;

extern LLC_NandHwConfig     gDDA_NandHwConfig;

extern DDA_NandObj          gDDA_NandObj[PSP_NAND_NUM_INSTANCES];

#ifdef __cplusplus
}
#endif

#endif  /* _DDA_NAND_H_ */

