/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/** \file   dda_nandCfg.c
 *
 *  \brief  NAND DDA Configuration file
 *
 *  This file contains the device specific hardware initialization settings.
 *  These settings are passed onto the DDC.
 *
 *  Any default hardware settings should go in these configuration file.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author     AK Mistral, Himamsu Mistral
 *
 *  \version    1.0 AK Mistral      Created.
 *  \version    1.1 Himamsu Mistral Code cleanup and release for BFT5
 *  \version    1.2 Himamsu Mistral Release for BFT6
 *  \version    1.3 Himamsu Mistral Adapted the Linux MTD device ID table
 *                                  for supporting known NAND devices.
 *                                  - Added DMA support.
 *                                  - Release for BFT7
 *  \version    1.4 Himamsu Mistral Added Multiple NAND support,
 *                                  BB extra commands in ddcNandId array
 *                                  - Release for BFT8
 *  \version    1.5 Amit Chatterjee Changed Chip select organization
 *  \version    1.6 Kiran Sutariya  Code cleanup and Modified for DM6437
 */

#include <ti/sdo/pspdrivers/drivers/nand/src/dda_nand.h>


LLC_NandHwConfig        gDDA_NandHwConfig =
{
    {
        LLC_NAND_8_BIT,         /**< 8-bit NAND               */
        LLC_NAND_PAGESZ_2048,   /**< 2048 byte page           */
        LLC_NAND_BANK_2         /**< NAND bank number         */
    }
};

/**
 * \brief Default NAND configuration.
 *
 *  The parameters are explained below:
 *   - Name of the NAND - typically size and organization
 *   - Device ID        - from the datasheets of NAND
 *   - Bytes per page   - bytes in a page excluding spare area.
 *   - Size             - size of NAND in MBytes; 2GB is 2048MB.
 *   - Block/Erase Size - size of a block in bytes;
 *                        Erase is block-wide!
 *   - Organization     - 8 or 16 bit
 *   - Extra commands   - Has extra commands like copy-back,
 *                        cache and random program input/output
 *
 *  The following example is that of 8-bit Big Block NAND:
 *  \verbatim
 *  {
 *      "NAND 128MB  8-Bit",        /// Name it!
 *      0xFE,                       /// Device ID
 *      2048,                       /// Bytes per page
 *      128,                        /// Size (in MB); 2GB is 2048
 *      16384,                      /// Block size or Erase size
 *      LLC_NAND_8_BIT,             /// 8 or 16 bit
 *      LLC_NAND_BB_OPTIONS         /// has extra commands?
 *  };
 *  \endverbatim
 *  <b>NOTE: Use caution while specifying parameters for
 *           your NAND device, as the parameters are taken
 *           as-is!</b>
 */
DDC_NandFlashDev       gNandCfgParams;

/* The following declarations work only here!   */
Uint16      gNAND_phyLogBank[DDC_NAND_MAX_BANKS][DDC_NAND_DEF_PHY_BUF_SIZE];
/**< Physical log map                                                       */
Uint16      gNAND_bbmBitBufBank[DDC_NAND_MAX_BANKS][DDC_NAND_DEF_BBM_BUF_SIZE];
/**< Bad Block management Bit buffer                                        */
Uint16      gNAND_wlaBitBufBank[DDC_NAND_MAX_BANKS][DDC_NAND_DEF_WLA_BUF_SIZE];
/**< Wear level algorithm buffer                                            */
DDC_NandCacheBlkRedInfo
gBlockRedInfoBank[DDC_NAND_MAX_BANKS][DDC_NAND_NO_BLKS_CACHED];
/**< Redundant information                                                  */

DDC_NandBankSectorInfo  gNandBankSectorInfo [DDC_NAND_MAX_BANKS];
/**< Object that holds max and min sector numbers for a bank                */

/**
 * \brief Manufacturer ID list
 *
 * Known manufacturers of NAND devices.
 *
 * NOTE: The driver may not support all devices from all manufacturers.
 */
DDC_NandMfg ddcNandMfg [DDC_NAND_MAX_MFGR] = {
    /* Manufacturer ID            Name     */
    {DDC_NAND_TOSHIBA_MFG_CODE,  "Toshiba "},
    {DDC_NAND_SAMSUNG_MFG_CODE,  "Samsung "},
    {DDC_NAND_FUJITSU_MFG_CODE,  "Fujitsu "},
    {DDC_NAND_NATIONAL_MFG_CODE, "National"},
    {DDC_NAND_RENESAS_MFG_CODE,  "Renesas "},
    {DDC_NAND_STMICRO_MFG_CODE,  "ST Micro"},
    {DDC_NAND_HYNIX_MFG_CODE,    "Hynix   "},
    {0xFFu,                       "Unknown "},
    {0x0,                        "Unknown "}
};

/**
 * \brief Known NAND ID list
 *
 *  List of known NAND IDs. This list may not be exhaustive, but would
 *  cover most of the available NAND devices. An "ID" of a device (read using
 *  0x90 command) consists of a minimum of four bytes:
 *    - Manufacturer's ID/code,
 *    - Device ID,
 *    - Extended ID and
 *    - Plane ID.
 *
 *  For latest chips, information about a chip can be decoded from the "ID" -
 *  Extended ID and Plane ID. However, older chips do not have that feature.
 *
 *  The list is divided into old chips and new chips (based on page size). For
 *  older chips -page size not more than 512 bytes- information about the page
 *  size, block size, organization etc cannot be decoded from the extended ID
 *  (4th ID data). Hence when a new entry is to be made, all the fields of the
 *  list needs to be filled using the datasheets.
 *
 *  However, for the newer/latest chips -page sizes from 1024 bytes and above-
 *  the above info can be decoded from the extended ID (4th ID data). For all
 *  such chips, bytes per page (page size) should be '0'.
 *
 *  New entries can be added to the list, provided the following parameters
 *  are known:
 *    - Device ID (2nd ID data)
 *    - Bytes per page (excluding the spare area)
 *    - Block size or erase size (in bytes; excluding the spare area)
 *    - Organization (8/16 bit)
 *
 *  If a new entry needs to be added, determine if it falls into the newer or
 *  older category and fill in the required details.
 *
 *  Following are the fields:
 *    - Name             - typically <b><tt>"NAND <size>MB <organization>-Bit"
 *                          </tt></b>
 *    - ID               - Device ID (2nd ID data) from NAND datasheets
 *    - Bytes per page   - bytes in a page excluding spare area.
 *    - ChipSize         - size of NAND in MBytes; 2GB is 2048MB.
 *    - Block/Erase Size - size of a block in bytes, excluding spare area.
 *                        Note that erase is block-wide!
 *    - Organization     - 8 or 16 bit
 *
 *  BytesPerPage can be one of 0, 256 or 512:
 *     -   0 - Get from the extended device ID
 *     - 256 - 256 Bytes per page
 *     - 512 - 512 Bytes per page
 *
 *  <b>NOTE: Ensure that the values being added are correct, as
 *        the data would be loaded as-is!</b>
 */
DDC_NandFlashDev ddcNandId[DDC_NAND_MAX_FLASH_SPEC] = {
    /* These are old devices with small page sizes. */
    /**<   Name,     ID,  Bytes,ChipSz, EraseSz, 8/16 bit, options
     *                    per
     *                    page
     */
    {"1MB 8-Bit",    0x6Eu, 256u,   1u, 0x1000u, LLC_NAND_8_BIT , 0 },
    {"2MB 8-Bit",    0x64u, 256u,   2u, 0x1000u, LLC_NAND_8_BIT , 0 },
    {"4MB 8-Bit",    0x6Bu, 512u,   4u, 0x2000u, LLC_NAND_8_BIT , 0 },
    {"1MB 8-Bit",    0xE8u, 256u,   1u, 0x1000u, LLC_NAND_8_BIT , 0 },
    {"1MB 8-Bit",    0xECu, 256u,   1u, 0x1000u, LLC_NAND_8_BIT , 0 },
    {"2MB 8-Bit",    0xEAu, 256u,   2u, 0x1000u, LLC_NAND_8_BIT , 0 },
    {"4MB 8-Bit",    0xD5u, 512u,   4u, 0x2000u, LLC_NAND_8_BIT , 0 },
    {"4MB 8-Bit",    0xE3u, 512u,   4u, 0x2000u, LLC_NAND_8_BIT , 0 },
    {"4MB 8-Bit",    0xE5u, 512u,   4u, 0x2000u, LLC_NAND_8_BIT , 0 },
    {"8MB 8-Bit",    0xD6u, 512u,   8u, 0x2000u, LLC_NAND_8_BIT , 0 },

    {"8MB 8-Bit",    0x39u, 512u,   8u, 0x2000u, LLC_NAND_8_BIT , 0 },
    {"8MB 8-Bit",    0xE6u, 512u,   8u, 0x2000u, LLC_NAND_8_BIT , 0 },
    {"8MB 16-Bit",   0x49u, 512u,   8u, 0x2000u, LLC_NAND_16_BIT, 0 },
    {"8MB 16-Bit",   0x59u, 512u,   8u, 0x2000u, LLC_NAND_16_BIT, 0 },

    {"16MB 8-Bit",   0x33u, 512u,  16u, 0x4000u, LLC_NAND_8_BIT , 0 },
    {"16MB 8-Bit",   0x73u, 512u,  16u, 0x4000u, LLC_NAND_8_BIT , 0 },
    {"16MB 16-Bit",  0x43u, 512u,  16u, 0x4000u, LLC_NAND_16_BIT, 0 },
    {"16MB 16-Bit",  0x53u, 512u,  16u, 0x4000u, LLC_NAND_16_BIT, 0 },

    {"32MB 8-Bit",   0x35u, 512u,  32u, 0x4000u, LLC_NAND_8_BIT , LLC_NAND_BB_OPTIONS },
    {"32MB 8-Bit",   0x75u, 512u,  32u, 0x4000u, LLC_NAND_8_BIT , LLC_NAND_BB_OPTIONS },
    {"32MB 16-Bit",  0x45u, 512u,  32u, 0x4000u, LLC_NAND_16_BIT, 0 },
    {"32MB 16-Bit",  0x55u, 512u,  32u, 0x4000u, LLC_NAND_16_BIT, 0 },

    {"64MB 8-Bit",   0x36u, 512u,  64u, 0x4000u, LLC_NAND_8_BIT ,
        LLC_NAND_BB_OPTIONS },
    {"64MB 8-Bit",   0x76u, 512u,  64u, 0x4000u, LLC_NAND_8_BIT ,
        LLC_NAND_BB_OPTIONS },
    {"64MB 16-Bit",  0x46u, 512u,  64u, 0x4000u, LLC_NAND_16_BIT, 0 },
    {"64MB 16-Bit",  0x56u, 512u,  64u, 0x4000u, LLC_NAND_16_BIT, 0 },

    {"128MB 8-Bit",  0x78u, 512u, 128u, 0x4000u, LLC_NAND_8_BIT , LLC_NAND_BB_OPTIONS },
    {"128MB 8-Bit",  0x39u, 512u, 128u, 0x4000u, LLC_NAND_8_BIT , LLC_NAND_BB_OPTIONS },
    {"128MB 8-Bit",  0x79u, 512u, 128u, 0x4000u, LLC_NAND_8_BIT , LLC_NAND_BB_OPTIONS },
    {"128MB 16-Bit", 0x72u, 512u, 128u, 0x4000u, LLC_NAND_16_BIT, 0 },
    {"128MB 16-Bit", 0x49u, 512u, 128u, 0x4000u, LLC_NAND_16_BIT, 0 },
    {"128MB 16-Bit", 0x74u, 512u, 128u, 0x4000u, LLC_NAND_16_BIT, 0 },
    {"128MB 16-Bit", 0x59u, 512u, 128u, 0x4000u, LLC_NAND_16_BIT, 0 },

    {"256MB 8-Bit",  0x71u, 512u, 256u, 0x4000u, LLC_NAND_8_BIT,  0 },

    /**
     * These are the new chips with large page size. The pagesize
     * and the erasesize is determined from the extended id bytes.
     * For all such devices, the pageSize is '0'.
     */

    /**<   Name,      ID, Bytes,ChipSz, EraseSz, 8/16 bit, options
     *                    per
     *                    page
     */
    /* 512 Megabit */
    {"64MB 8-Bit",   0xA2u, 0,   64u, 0, LLC_NAND_8_BIT , 0 },
    {"64MB 8-Bit",   0xF2u, 0,   64u, 0, LLC_NAND_8_BIT , 0 },
    {"64MB 16-Bit",  0xB2u, 0,   64u, 0, LLC_NAND_16_BIT, 0 },
    {"64MB 16-Bit",  0xC2u, 0,   64u, 0, LLC_NAND_16_BIT, 0 },

    /* 1 Gigabit */
    {"128MB 8-Bit",  0xA1u, 0,  128u, 0, LLC_NAND_8_BIT , LLC_NAND_BB_OPTIONS },
    {"128MB 8-Bit",  0xF1u, 0,  128u, 0, LLC_NAND_8_BIT , LLC_NAND_BB_OPTIONS },
    {"128MB 16-Bit", 0xB1u, 0,  128u, 0, LLC_NAND_16_BIT, LLC_NAND_BB_OPTIONS },
    {"128MB 16-Bit", 0xC1u, 0,  128u, 0, LLC_NAND_16_BIT, LLC_NAND_BB_OPTIONS },

    /* 2 Gigabit */
    {"256MB 8-Bit",  0xAAu, 0,  256u, 0, LLC_NAND_8_BIT , LLC_NAND_BB_OPTIONS },
    {"256MB 8-Bit",  0xDAu, 0,  256u, 0, LLC_NAND_8_BIT , LLC_NAND_BB_OPTIONS },
    {"256MB 16-Bit", 0xBAu, 0,  256u, 0, LLC_NAND_16_BIT, LLC_NAND_BB_OPTIONS },
    {"256MB 16-Bit", 0xCAu, 0,  256u, 0, LLC_NAND_16_BIT, LLC_NAND_BB_OPTIONS },

    /* 4 Gigabit */
    {"512MB 8-Bit",  0xACu, 0,  512u, 0, LLC_NAND_8_BIT , LLC_NAND_BB_OPTIONS },
    {"512MB 8-Bit",  0xDCu, 0,  512u, 0, LLC_NAND_8_BIT , LLC_NAND_BB_OPTIONS },
    {"512MB 16-Bit", 0xBCu, 0,  512u, 0, LLC_NAND_16_BIT, LLC_NAND_BB_OPTIONS },
    {"512MB 16-Bit", 0xCCu, 0,  512u, 0, LLC_NAND_16_BIT, LLC_NAND_BB_OPTIONS },

    /* 8 Gigabit */
    {"1GB 8-Bit",    0xA3u, 0, 1024u, 0, LLC_NAND_8_BIT , LLC_NAND_BB_OPTIONS },
    {"1GB 8-Bit",    0xD3u, 0, 1024u, 0, LLC_NAND_8_BIT , LLC_NAND_BB_OPTIONS },
    {"1GB 16-Bit",   0xB3u, 0, 1024u, 0, LLC_NAND_16_BIT, LLC_NAND_BB_OPTIONS },
    {"1GB 16-Bit",   0xC3u, 0, 1024u, 0, LLC_NAND_16_BIT, LLC_NAND_BB_OPTIONS },

    /* 16 Gigabit */
    {"2GB 8-Bit",    0xA5u, 0, 2048u, 0, LLC_NAND_8_BIT , LLC_NAND_BB_OPTIONS },
    {"2GB 8-Bit",    0xD5u, 0, 2048u, 0, LLC_NAND_8_BIT , LLC_NAND_BB_OPTIONS },
    {"2GB 16-Bit",   0xB5u, 0, 2048u, 0, LLC_NAND_16_BIT, LLC_NAND_BB_OPTIONS },
    {"2GB 16-Bit",   0xC5u, 0, 2048u, 0, LLC_NAND_16_BIT, LLC_NAND_BB_OPTIONS },

    {NULL,}
};

