/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 * \file    ddc_nand.c
 *
 * \brief  DDC Initialization for NAND on DM6437
 *
 * This file contains the NAND DDC implemented for the NAND controller
 * on DM6437.
 *
 * (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \note       Set tabstop to 4 (:se ts=4) while viewing this file in an
 *              editor
 *
 *  \author     AK Mistral, Himamsu Mistral
 *
 *  \version    1.0 AK Mistral      Created.
 *  \version    1.1 Himamsu Mistral Code cleanup and release for BFT5
 *  \version    1.2 Himamsu Mistral Release for BFT6
 *  \version    1.3 Himamsu Mistral Adapted the Linux MTD device ID table.
 *                                  Added code to update/load device
 *                                  information in nandState with extended
 *                                  ID info.
 *                                  - Release for BFT7
 *  \version    1.4 Himamsu Mistral Added Multiple NAND support, enabled
 *                                  interrupt support
 *                                  - Release for BFT8
 */

#include <ti/sdo/pspdrivers/drivers/nand/src/ddc_nand.h>

/*
 * Prototypes of Static Functions
 */

/**
 *  \brief Configure the NAND bank for Read and write.
 *
 *  This API identifies into which bank the logSector falls and configures
 *  the NAND bank accordingly. The logical sector is adjusted for the bank it
 *  falls into.
 *
 *  \param  DDC             [IN]    DDC Structure.
 *  \param  logSector       [IN]    logical sector for read/write.
 *  \param  curLogSector    [OUT]   Adjusted logical sector.
 *
 *  \return PAL_SOK or DDC_NAND_E_INVALID_LSN
 */

static PAL_Result nandStateUpdate (
            DDC_NandObj                     *DDC);

static Uint32 nandReadId (
            DDC_NandObj                     *DDC);

static void ddcNandBankInit (
            LLC_NandHandle                  hNand);




extern DDC_NandFlashDev     ddcNandId[DDC_NAND_MAX_FLASH_SPEC];
/**< Array of known NAND device IDs                             */
extern DDC_NandMfg          ddcNandMfg[DDC_NAND_MAX_MFGR];
/**< Array of knowm NAND Manufacturers                          */
extern DDC_NandFlashDev     gNandCfgParams;
/**< NAND Configuration parameters                              */
extern DDC_NandAsyncBank    gDDCNandAsyncBank[DDC_NAND_MAX_BANKS];
/**< NAND Asynchronous bank configuration                       */
extern DDC_NandBankSectorInfo gNandBankSectorInfo[DDC_NAND_MAX_BANKS];
/**< Object that holds max and min sector numbers for a bank    */



/*
 * Definitions of Static Functions
 */

/**
 *  \brief Creates instance of the NAND driver
 *
 *  Memory for device DDC (instance) object is allocated, and it provides a
 *  handle to the DDC in "hDDC". This will be used in every DDC function
 *  called by the upper layer. It also stores DDA Handle in DDC internal
 *  structure and passes it in every DDA callback. This API exchanges
 *  function pointers with DDA layer.
 *
 *  \note CreateInstance() does not touch the hardware of the device.
 *
 *  \param  instId      Instance Id
 *  \param  hDDA        DDA Handle provided by DDA layer
 *  \param  hDDACbIf    DDA Callback Function Table handle
 *  \param  hDDC        Placeholder for DDC Handle
 *  \param  hDDCIf      Placeholder for DDC Function Table Handle
 *  \param  param       Optional argument if any
 *  \return PAL_SOK or PAL Error code
 */
PAL_Result nandCreateInstance(
            Int                     instId,
            DDA_Handle              hDDA,
            DDC_Handle              *hDDC,
            Ptr                     param)
{
    PAL_Result  result = PAL_SOK;
    DDC_NandObj         *DDC;

    /* To remove remark while compiling using -pdr diagnostics*/
    param = param;

    if((NULL != hDDC) && (NULL != hDDA)) {
        DDC   = (DDC_NandObj *)(*hDDC);
        DDC->hDDA = hDDA;
        DDC->instId = (Uint32)instId;
        DDC->state = PSP_DRIVER_STATE_CREATED;
        DDC->versionId = 0xFFFFFFFF;
    }
    else {
        result = PSP_E_INVAL_PARAM;
    }

    return (result);
}


/**
 *  \brief Deletes the instance of the driver.
 *
 *  This API deletes the NAND driver instance in the system. When a
 *  driver is deleted, no more transaction can be done on that driver
 *  instance.
 *
 *  \param  hDDC    [IN]    Handle of the lower layer
 *  \param  param   [IN]    Extra parameter (implementation specific)
 *  \return PAL_Result or PAL Error code.
 */
PAL_Result nandDelInst(
            DDC_Handle  hDDC,
            Ptr         param)
{
    PAL_Result  result = PAL_SOK;
    DDC_NandObj *DDC;

    /* To remove remark while compiling using -pdr diagnostics*/
    param = param;

    if(NULL != hDDC) {
        DDC   = (DDC_NandObj *)hDDC;
        DDC->state = PSP_DRIVER_STATE_DELETED;
    }
    else {
        result = PSP_E_INVAL_PARAM;
    }

    return (result);
}

/**
 *  \brief Initialize instance of the device
 *
 *  After the instance is "created", DDA calls the initialization function
 *  on the DDC to initialize the instance of the device. Typically, software
 *  bookkeeping functions are performed in this call. Memory for device
 *  instance specific data structures may be allocated and initialized.
 *  Configuration information may also be passed in the call and
 *  initialization based upon this information is done. On successful
 *  completion of this call, the device is ready to be "opened".
 *
 *  \param  hDDC    [IN]        DDC Handle
 *  \param  param   [IN/OUT]    Device specific argument
 *  \return PAL_SUCCESS or PAL Error code
 */
PAL_Result nandInit(
            DDC_Handle  hDDC,
            Ptr         param)
{
    PAL_Result          result = PAL_SOK;
    DDC_NandObj         *DDC;
    DDC_NandInitConfig  *initConfig;

    if( (NULL != hDDC) && (NULL != param)) {
        DDC   = (DDC_NandObj *)hDDC;
        initConfig = (DDC_NandInitConfig *)param;

        DDC->opMode = initConfig->opMode;
        LLC_nandInit(initConfig->llc, DDC->instId);
        DDC->llc = initConfig->llc;

        DDC->state = PSP_DRIVER_STATE_INITIALIZED;
    }
    else {
        result = PSP_E_INVAL_PARAM;
    }

    return (result);
}


/**
 *  \brief De-initializes the instance of the driver.
 *
 *  This API de-initializes the NAND driver instance in the system.
 *
 *  \param  hDDC    [IN]    Handle of the lower layer
 *  \param  param   [IN]    Extra parameter (implementation specific)
 *
 *  \return PAL_Result or PAL Error code.
 */
PAL_Result nandDeinit(
            DDC_Handle  hDDC,
            Ptr         param)
{

    PAL_Result  result = PAL_SOK;
    DDC_NandObj *DDC;

    /* To remove remark while compiling using -pdr diagnostics*/
    param = param;



    if( NULL != hDDC) {
        DDC   = (DDC_NandObj *)hDDC;
        DDC->state = PSP_DRIVER_STATE_DEINITIALIZED;
    }
    else {
        result = PSP_E_INVAL_PARAM;
    }
    return (result);
}


/**
 *  \brief Open Instance of the device
 *
 *  This function prepares the NAND hardware for data transfers and
 *  usage by the upper layer driver software.
 *  On successful completion of open, NAND is ready for data IO.
 *
 *  \param  hDDC    [IN]        DDC Handle
 *  \param  param   [IN/OUT]    Device specific argument
 *
 *  \return PAL_SOK or PAL Error code
 */
PAL_Result nandOpen(
            DDC_Handle  hDDC,
            Ptr         param)
{
    PAL_Result          result  = PAL_SOK;
    DDC_NandObj         *DDC;
    Uint32              bank;
    DDC_NandOpenParams  *openParams;
    DDC_NandState       *pNandState;
    Uint32              maxNandSectors = 0;
    Uint32              maxNandSectorSize = 0;



    if((NULL != hDDC) && (NULL != param)) {
        DDC     = (DDC_NandObj *)hDDC;
        openParams  = (DDC_NandOpenParams *)param;
        /* Initialize the banks registers */
        ddcNandBankInit(DDC->llc);

        for (bank = 0; bank < DDC_NAND_MAX_BANKS; bank++) {
            openParams->hwConfig->cfgBasic.bankNo = (LLC_NandBankNo) bank;
            DDC->bankNo = bank;
            pNandState = (DDC_NandState *) &DDC->nandState[DDC->bankNo];

            LLC_nandHwSetup(DDC->llc, openParams->hwConfig);

            /* Intialize protected blocks */
            pNandState->protectedBlks = 0;
            /* Issue a Reset to the NAND device */
            LLC_nandSendCommand(DDC->llc, DDC_NAND_CMD_RESET);

            /*  Wait for the device to get ready */
            LLC_checkCommandStatus(DDC->llc);

            /* Read the device id of NAND */
            pNandState->deviceID = nandReadId(DDC);

            /* Check if valid device ID is got or the command
            is still on the bus*/
            if (((NULL == pNandState->deviceID) ||
                (DDC_NAND_DEVICE_ID == pNandState->deviceID)) ||
                (DDC_NAND_CMD_DEVID == pNandState->deviceID)) {
#ifdef  PSP_NAND_DEBUG
                    NAND_Debug2("\r\nNAND MEDIA NOT PRESENT/RECOGNISED IN BANK 0x%x.\
                            [Device ID: 0x%x]\n",
                            openParams->hwConfig->cfgBasic.bankNo,
                            pNandState->deviceID);
#endif
            }
            else {
                /* Atleast one NAND found. Open DDC. */
                if (DDC->state != PSP_DRIVER_STATE_OPENED) {
                    DDC->state = PSP_DRIVER_STATE_OPENED;

                    /* Reserve the protected area only for the first device */
                    pNandState->protectedBlks = DDC_NAND_PROTECTED_AREA;
                }
                result = nandStateUpdate (DDC);
                if (PAL_SOK == result) {
                    /* Caution: Low level erase of NAND devices.
                     Cannot be un-done.*/
                    if (FALSE != DDC->llc->eraseAtInit) {
#ifdef PSP_NAND_DEBUG
                        NAND_Debug ("\r\nNAND: Erasing NAND medium...\n\r");
#endif
                        result = (PAL_Result)nandFtlErase (DDC);

#ifdef PSP_NAND_DEBUG
                        if (PAL_SOK == result) {
                            NAND_Debug ("\r\nNAND: Erased successfully.\n\r");
                        }
                        else {
                            NAND_Debug1 ("\r\nNAND: 0x%x blocks failed \
                                    during erase.\n\r", (Uint32) result);
                        }
#endif
                    }

                    /* NAND FTL Layer Initialization */
#ifdef PSP_NAND_DEBUG
                    NAND_Debug ("\r\nInitializing FTL (may take few time)...\n\r");
#endif

                    result = nandFtlInit(DDC);
                    if (PAL_SOK == result) {
                        LLC_nandSendCommand(DDC->llc, DDC_NAND_CMD_RESET);

                        /*  Wait for the device to get ready */
                        LLC_checkCommandStatus(DDC->llc);
                    }
#ifdef PSP_NAND_DEBUG
                    else {
                        NAND_Debug1("\r\nNAND: FTL initialization failed. \
                                [Result = 0x%x].\n\r", result);
                    }
#endif
                }
            }
        }
        /* Updating max nand sectors and size parameters of DDC object */
        for (bank = 0; bank < DDC_NAND_MAX_BANKS; bank++) {

            maxNandSectors+=(DDC->nandState[bank].totalPages);
            /* Adjust the protected partition */
            maxNandSectors -= (DDC->nandState[bank].protectedBlks)
                                * DDC->nandState[bank].pagesPerBlock;
            maxNandSectorSize = (Uint32)MAX(maxNandSectorSize,
                    DDC->nandState[bank].bytesPerPage);

        }

        DDC->maxNandSectors = maxNandSectors;
        DDC->maxNandSectorSize=maxNandSectorSize;
        /* End updating max nand sector and size parms */


    }
    else {
        result = PSP_E_INVAL_PARAM;
    }

    return (result);
}

/**
 *  \brief Close Instance of NAND
 *
 *  Closes the NAND for data transfers and usage by the upper layer driver
 *  software.
 *  After the successful completion of this call, the device cannot perform
 *  any data IO.
 *
 *  \param  hDDC    [IN]        DDC Handle
 *  \param  param   [IN/OUT]    Device specific argument
 *
 *  \return PAL_SOK or PAL Error code
 */
PAL_Result nandClose(
            DDC_Handle  hDDC,
            Ptr         param)
{
    PAL_Result  result = PAL_SOK;
    DDC_NandObj *DDC;

    /* To remove remark while compiling using -pdr diagnostics*/
    param = param;

    if( NULL != hDDC) {
        DDC   = (DDC_NandObj *)hDDC;
        DDC->state = PSP_DRIVER_STATE_CLOSED;
    }
    return (result);
}


/**
 *  \brief Control operations (ioctl)
 *
 * This API provides the capability for control operations to be performed on
 * NAND driver core. This is used by the upper layer driver software to
 * configure a parameter in the device driver core or change a parameter
 * during run time.
 *
 *  \param  hDDC    [IN]        DDC Handle
 *  \param  cmd     [IN]        Operation to be performed, typically an enum
 *                              gets passed
 *  \param  cmdArg  [IN/OUT]    Provides additonal information related to
 *                              the operation
 *  \param  param   [IN/OUT]    Device/Cmd specific argument
 *  \return PAL_SUCCESS or PAL Error code
 */
PAL_Result nandControl(
            DDC_Handle  hDDC,
            Int         cmd,
            Ptr         cmdArg,
            Ptr         param)
{

    PAL_Result  result = PAL_SOK;
    DDC_NandObj *DDC;
    if( NULL != hDDC) {
        DDC   = (DDC_NandObj *)hDDC;

    /* To remove remark while compiling using -pdr diagnostics*/
    param = param;
    cmdArg = cmdArg;

        if (DDC->state != PSP_DRIVER_STATE_OPENED) {
            result = PSP_E_INVAL_STATE;
        } else {
            switch (cmd) {
                case DDC_NAND_IOCTL_ERASE:
                    result  = (PAL_Result)nandFtlErase(DDC);
                    break;
                default:
                    result = PSP_E_NOT_SUPPORTED;
                    break;
            }
        }
    }
    else {
        result = PSP_E_INVAL_PARAM;
    }
    return (result);
}



/**
 *  \brief  This function reads the NAND Device ID
 *
 *  \param  DDC [IN]     DDC Object
 *
 *  \return Device Id
 */
static Uint32 nandReadId (
            DDC_NandObj     *DDC)
{
    Uint8   tmp[5];
    Uint32  devId = 0;
    volatile Uint32     index;

    if( NULL != DDC) {
        /* Send device ID command. */
        LLC_nandSendCommand(DDC->llc, DDC_NAND_CMD_DEVID);

        /* Set address to read from. */
        LLC_nandSetAddr(DDC->llc, DDC_NAND_CMD_DEVADD);

        /* Wait till the device is busy */
        for(index = 0x5; index > 0; index --)
        {
            asm("\tNOP");
        }

        for(index=0; index < DDC_NAND_ADDR_CYCLES; index++)
        {
            tmp[index] = LLC_nandReadByte(DDC->llc);
        }
        devId = (Uint32)((((Uint32)tmp[0]) << 24) | (((Uint32)tmp[1]) << 16)
                            | (((Uint32)tmp[2]) << 8) | ((Uint32)tmp[3]));
    }
    return (devId);
}

/**
 *  \brief Update the NAND State structure with the device parameters.
 *
 *  This function decodes the device ID and hence extended ID - newer NANDs
 *  have this info packed in extended ID - to update various parameters of
 *  NAND structure. In case of old NAND devices, these parameters are loaded
 *  from a known 'list' of NAND devices. If the device ID is not one from
 *  this list too, the user supplied parameters are loaded.
 *
 *  This information is dependent on the  manufacturer and may vary.
 *
 *  \param  DDC         [IN]    DDC Structure.
 *
 *  \return PAL_SOK or DDC_NAND_E_INVALID_MEDIA.
 */
static PAL_Result nandStateUpdate (
            DDC_NandObj           *DDC)
{
    PAL_Result      result = PAL_SOK;
    Uint32          blockSize;
    Uint32          index;
    Uint8   makerCode  = 0;
    Uint8   deviceCode = 0;
    Uint8   thirdByte  = 0;
    Uint8   fourthByte = 0;
    DDC_NandState   *pNandState;

    if((NULL != DDC)) {
        pNandState     = (DDC_NandState *) &DDC->nandState[DDC->bankNo];
        if(NULL != pNandState) {
            makerCode  = (Uint8) (pNandState->deviceID >> 24);
            deviceCode = (Uint8) (pNandState->deviceID >> 16);
            thirdByte  = (Uint8) (pNandState->deviceID >> 8);
            thirdByte = thirdByte;
            fourthByte = (Uint8) (pNandState->deviceID);
        }
        for (index = 0; ddcNandId[index].name != NULL; index++) {
            if (deviceCode == ddcNandId[index].id) {
                if (pNandState->name == NULL) {
                    pNandState->name = ddcNandId[index].name;

                    /* Load the options specified. */
                    pNandState->nandOptions = ddcNandId[index].nandOptions;
                }

                /* Get all the info from Extended ID. */
                if (NULL == ddcNandId[index].bytesPerPage) {
                    /* Bytes per page (Page size) -in multiples of 1024 */
                    pNandState->bytesPerPage =
                    (Uint16)((1 << (fourthByte & LLC_NAND_PAGE_SIZE_MASK))
                             *1024u);

                    /* Spare Area size(Redundant area size; nbytes/512 bytes)*/
                    pNandState->spareAreaSize =
                        ((Uint32)(8u << ((Uint8)((Uint8)(fourthByte &
                                 LLC_NAND_RED_AREA_SIZE_MASK))>>2)))
                        * (Uint16)(pNandState->bytesPerPage >> 9); /* 2^9 = 512 */

                    /* Pages per block (Block Size) -in multiples of 64 KBs */
                    blockSize = (64 << ((fourthByte &
                                    LLC_NAND_BLOCK_SIZE_MASK)>>4) )*1024u;
                    pNandState->pagesPerBlock = (Uint16)(blockSize /
                            pNandState->bytesPerPage);
                    pNandState->noOfBlocks   = (ddcNandId[index].chipSize*
                            1024u*1024u)/blockSize;

                    /* NAND width (Organization) */
                    /* Get this from the fourthByte, rather than loading
                     * from ID array */
                    pNandState->nand8Bit = DDC->llc->nandWidth
                        = (LLC_NandWidth) (fourthByte &
                                LLC_NAND_ORGANIZATION_MASK) >> 6;

                }
                /* Old kind of NANDs */
                else {

                    /* Bytes per page */
                    pNandState->bytesPerPage  =
                        (Uint16)(ddcNandId[index].bytesPerPage);

                    /* Spare Area size */
                    if (ddcNandId[index].bytesPerPage >= 512u) {
                        pNandState->spareAreaSize =
                            (ddcNandId[index].bytesPerPage/512u)*16u;
                    }
                    else {
                        pNandState->spareAreaSize = 16u;
                    }

                    pNandState->noOfBlocks   =
                        (ddcNandId[index].chipSize*1024u*1024u)/
                        ddcNandId[index].eraseSize;
                    /* Pages per block */
                    /* Since, erase is block wide, erase size is nothing
                     * but block size.*/
                    pNandState->pagesPerBlock =
                        (Uint16)(ddcNandId[index].eraseSize/
                                 pNandState->bytesPerPage);
                    /* NAND Width */
                    pNandState->nand8Bit     = DDC->llc->nandWidth   =
                        ddcNandId[index].nand8Bit;
                }
                break;
            } /* End devicode if check */
        } /* End of for loop */

        /* If we didn't find a valid device, load user supplied values. */
        if (ddcNandId[index].name == NULL) {
            /* Check, if a user supplied configuration exists. */
            if (deviceCode == gNandCfgParams.id) {
#ifdef PSP_NAND_DEBUG
                NAND_Debug2 ("\r\nNAND: Unknown NAND Device - 0x%x 0x%x.\n\r",
                                    (Uint16) (pNandState->deviceID >> 16),
                                    (Uint16) (pNandState->deviceID));
#endif
                pNandState->name = gNandCfgParams.name;

                /* Bytes per page */
                /* NAND chips typically start from 256 bytes per page */
                if (gNandCfgParams.bytesPerPage < 256u) {
                    return (DDC_NAND_E_INVALID_MEDIA);
                }
                else {
                    pNandState->bytesPerPage  =
                        (Uint16)(gNandCfgParams.bytesPerPage);
                }

                /* Spare Area size */
                if (gNandCfgParams.bytesPerPage >= 512u) {
                    pNandState->spareAreaSize =
                        (gNandCfgParams.bytesPerPage/512u)*16u;
                }
                else {
                    pNandState->spareAreaSize = 16u;
                }

                pNandState->noOfBlocks   =
                    (gNandCfgParams.chipSize*1024u*1024u)/
                    gNandCfgParams.eraseSize;

                /* Pages per block */
                /* Since, erase is block wide, erase size is nothing
                 * but block size.*/
                pNandState->pagesPerBlock = (Uint16)(gNandCfgParams.eraseSize/
                        pNandState->bytesPerPage);

                /* NAND Width */
                pNandState->nand8Bit      = DDC->llc->nandWidth =
                    gNandCfgParams.nand8Bit;
#ifdef PSP_NAND_DEBUG
                NAND_Debug ("\r\nNAND: Loading user supplied values... done.\n\r");
#endif
            }
            else {
#ifdef PSP_NAND_DEBUG
                NAND_Debug2 ("\r\nNAND: Unknown NAND Device - 0x%x 0x%x.\n\r",
                                    (Uint16) (pNandState->deviceID >> 16),
                                    (Uint16) (pNandState->deviceID));
                NAND_Debug ("\r\nNAND: No user supplied parameters found. \
                Initialization halted!\n\r");
#endif
                return (DDC_NAND_E_INVALID_MEDIA);
            }
        }

        /* Total pages in the device */
        pNandState->totalPages    =
            (Uint32)(pNandState->pagesPerBlock * pNandState->noOfBlocks);

        /* Number of Address cycle */
        pNandState->addrCycle    = 0;
        index = pNandState->totalPages;
        while(index > 1u){
            index = index >> 8u;
            pNandState->addrCycle++;
        }
        /* Number of Zones in the device */
        pNandState->numZones     = (Uint16)(pNandState->noOfBlocks/
                                   DDC_NAND_BLOCKS_PER_ZONE);
    /*  Changed for Bad block management bug  */    
    /* Set total number of usable blocks in zone */
    for(index = 0; index < pNandState->numZones ; index++)
    {
        pNandState->usableBlockInZone[index] = DDC_NAND_USABLE_BLOCKS_PER_ZONE;
    }
    /*  Changed for Bad block management bug  - Up to here  */

        /* Adjust the total pages.
         * There shall be only DDC_NAND_USABLE_BLOCKS_PER_ZONE (1000)
         * out of actual
         * DDC_NAND_BLOCKS_PER_ZONE (1024).
         */
        pNandState->totalPages   -= (Uint32)(pNandState->numZones *
                                    (DDC_NAND_BLOCKS_PER_ZONE -
                                     DDC_NAND_USABLE_BLOCKS_PER_ZONE) *
                                    pNandState->pagesPerBlock);
        /* Number of bad blocks */
        pNandState->noOfBadBlocks = 0;

        /* Number of bad pages */
        pNandState->noOfBadPages  = 0;

        /* Adjust it to blocks */
        pNandState->protectedBlks = pNandState->protectedBlks/pNandState->pagesPerBlock;

        /* NAND type (Big/Small block) */
        if (pNandState->bytesPerPage > 512u) {
            pNandState->nandType = LLC_NAND_BIG_BLOCK;  /* Big Block    */
        }
        else {
            pNandState->nandType = LLC_NAND_SMALL_BLOCK; /* Small Block */
        }
#ifdef PSP_NAND_DEBUG
        NAND_Debug ("\r\n");
#endif
        for (index = 0; ddcNandMfg[index].id != NULL; index++) {
            if (ddcNandMfg[index].id == makerCode) {
                break;
            }
        }
#ifdef PSP_NAND_DEBUG /* Print for debug build only */
    NAND_Debug3("\r\nNAND  :'%s %s' NAND Device Found at bank %d",
                                    ddcNandMfg[index].name,
                                    pNandState->name,
                                    DDC->bankNo);


    NAND_Debug ("\r\n--  Specifications ---------------\n");
    NAND_Debug ("\r\n\n");
    NAND_Debug1 ("\r\nPage  Size          : %d Bytes.\n", pNandState->bytesPerPage);
    NAND_Debug1 ("\r\nBlock Size          : %d KB.\n\n",  (pNandState->pagesPerBlock*
                         pNandState->bytesPerPage)>>10);
    NAND_Debug1 ("\r\nRedundant Area Size : %d Bytes.\n", pNandState->spareAreaSize);
    NAND_Debug1 ("\r\nOrganization        : %d bit.\n",   (8 << pNandState->nand8Bit));
    NAND_Debug1 ("\r\nSize                : %d MB.\n\n",  ((pNandState->totalPages *
                                                    pNandState->bytesPerPage)/1024)/1024);
    NAND_Debug ("\r\n----------------------------------\n");

    NAND_Debug3 ("\r\nNAND  :Device Id     : %d - %d (%s Block NAND)",
                (Uint16) (pNandState->deviceID >> 16),
                (Uint16) (pNandState->deviceID),
                (pNandState->nandType==LLC_NAND_SMALL_BLOCK)?"Small":"Big");
#endif /* PSP_NAND_DEBUG */
    }
    else {
        result = PSP_E_INVAL_PARAM;
    }
    return result;
}

/**
 *  \brief Synchronous Read
 *
 * This API reads data from the NAND device in a synchronous mode.
 *
 *  \param  hChannel    [IN]    Stream Channel Handle
 *  \param  buf         [IN]    Data buffer address
 *  \param  xferRequest [IN]    Number of sectors to be read
 *  \param  timeout     [IN]    Timeout value
 *  \param  param       [IN]    Logsector number
 *  \param  xferActual  [IN]    Result of the function.
 *  \return PAL_SOK or PAL Error code
 */
PAL_Result nandReadSync(
            DDC_NandObj             *hDDC,
            Uint16                  *buf,
            const Int32             xferRequest,
            Uint32                  timeout,
            Ptr                     param,
            Int32             *xferActual)
{
    DDC_NandObj             *DDC;
    DDC_NandState           *pNandState;
    Uint32                  logSector;
    Uint16                  *pReadBuff;

    Uint16                  noOfSectors;
    PAL_Result              result = PSP_E_INVAL_PARAM;

    Uint32                  curLogSector    = 0;
    Uint32                  newLogSector    = 0;

    Uint32                  pagesTillEndOfBank = 0;

    /* To remove remark while compiling using -pdr diagnostics*/
    timeout = timeout;

    if(((NULL != hDDC) && (NULL != xferActual))
                    && ((NULL != buf) && (NULL != param))) {
        DDC             = hDDC;
        pReadBuff       = (Uint16 *)buf;
        pNandState     = (DDC_NandState *) &DDC->nandState[DDC->bankNo];
        logSector       = *((Uint32 *)param);
        *xferActual = 0;

        /* If the number of pages to read goes beyond this NAND bank, first read
         * to the end of this bank and then adjust the logical sector to read
         * from the start of the new bank. */

        /* Advance logical sector by number of protected sectors*/
        logSector += pNandState->protectedBlks * pNandState->pagesPerBlock;

        /* Advance pages */
        if ((logSector + xferRequest) > pNandState->totalPages) {
            pagesTillEndOfBank = (pNandState->totalPages-logSector);
            for (noOfSectors = 0; noOfSectors < pagesTillEndOfBank;
                            noOfSectors++) {
                result = nandFtlReadPage((logSector + noOfSectors),
                                DDC,
                                &pReadBuff[(pNandState->bytesPerPage / 2) *
                                noOfSectors]);

                /* Ensure that write has happened. */
                if (PAL_SOK == result) {
                     *xferActual += (Int32)pNandState->bytesPerPage;
                }
            }

            newLogSector = logSector + pagesTillEndOfBank;
            result = (PAL_Result)ddcNandBankConfigure(DDC, newLogSector, &curLogSector);

            if (PAL_SOK == result) {
                newLogSector = curLogSector;
                pNandState     = (DDC_NandState *) &DDC->nandState[DDC->bankNo];
                for (noOfSectors = 0; noOfSectors <
                        (xferRequest - pagesTillEndOfBank); noOfSectors++) {
                    result = nandFtlReadPage((newLogSector + noOfSectors),
                                    DDC,
                                    &pReadBuff[
                                    (pNandState->bytesPerPage/2)*noOfSectors]);

                    if (PAL_SOK == result) {
                        *xferActual += (Int32)pNandState->bytesPerPage;
                    }
                }
            }
        }
        else {
            for (noOfSectors = 0; noOfSectors < xferRequest; noOfSectors++) {
                result = nandFtlReadPage((logSector + noOfSectors),
                                DDC,
                                &pReadBuff[
                                (pNandState->bytesPerPage / 2) * noOfSectors]);
                if (PAL_SOK == result) {
                    *xferActual += (Int32)pNandState->bytesPerPage;
                }
            }

#ifdef NAND_DBG_LEVEL_1
            NAND_Debug ("\r\n\t-  -->Sector %d read  %7s. [Result = 0x%x xfered =\
                    0x%x].\n",logSector,
                    (result == PAL_SOK) ? "Success":"Failed",
                    result,*xferActual);
#endif /* NAND_DBG_LEVEL_1 */
        }
    }
    return (result);
}

/**
 *  \brief Synchronous Write
 *
 * This API writes data into the NAND device in a synchronous mode.
 *
 *  \param  hChannel    [IN] Stream Channel Handle
 *  \param  buf         [IN] Data buffer address
 *  \param  xferRequest [IN] Number of sectors to be written
 *  \param  timeout     [IN] Timeout value
 *  \param  param       [IN] Logsector number
 *  \param  xferActual  [IN] Result of the function.
 *  \return PAL_SOK or PAL Error code
 */
PAL_Result nandWriteSync(
            DDC_NandObj             *hDDC,
            Uint16                  *buf,
            const Int32             xferRequest,
            Uint32                  timeout,
            Ptr                     param,
            Int32                    *xferActual)
{
    DDC_NandObj             *DDC;
    DDC_NandState           *pNandState;
    Uint32                  logSector;
    Uint16                  *pWriteBuff;
    Uint8                   byteSwap        = 0;

    PAL_Result              result = PSP_E_INVAL_PARAM;
    Uint16                  noOfSectors     = 0;
    Uint32                  curLogSector    = 0;
    Uint32                  newLogSector    = 0;

    Uint32                  pagesTillEndOfBank = 0;

    /* To remove remark while compiling using -pdr diagnostics*/
    timeout = timeout;

    if(((NULL != hDDC) && (NULL != xferActual))
                    && ((NULL != buf) && (NULL != param))) {
        DDC         = (DDC_NandObj *)hDDC;
        pWriteBuff  = (Uint16 *)buf;
        pNandState  = (DDC_NandState *) &DDC->nandState[DDC->bankNo];
        logSector       = *((Uint32 *)param);
        *xferActual = 0;
        /* If the number of pages to write goes beyond this NAND bank, first
         * write to the end of this bank and then adjust the logical sector
         * to write from the new bank.
         */

        /* Advance logical sector by number of protected sectors*/
        logSector += pNandState->protectedBlks * pNandState->pagesPerBlock;

        /* Advance pages */
        if ((logSector + xferRequest) > pNandState->totalPages) {
            pagesTillEndOfBank = (pNandState->totalPages-logSector);
            for (noOfSectors = 0; noOfSectors < pagesTillEndOfBank;
                            noOfSectors++) {
                result = nandFtlWritePage((logSector + noOfSectors),
                                DDC,
                                &pWriteBuff[
                                (pNandState->bytesPerPage / 2) * noOfSectors],
                                byteSwap);

                /* Ensure that write has happened. */
                if (PAL_SOK == result) {
                    *xferActual += (Int32)pNandState->bytesPerPage;
                }
            }

            /* Flush after a complete write. */
            if (PAL_SOK == result) {
                result = nandUtlWriteSectorFlush(DDC);
            }

            newLogSector = logSector + pagesTillEndOfBank;
            result = (PAL_Result)ddcNandBankConfigure(DDC, newLogSector, &curLogSector);

            if(PAL_SOK == result) {
                newLogSector = curLogSector;
                pNandState     = (DDC_NandState *) &DDC->nandState[DDC->bankNo];
                for (noOfSectors = 0; noOfSectors <
                                (xferRequest - pagesTillEndOfBank);
                                noOfSectors++) {
                        result = nandFtlWritePage((newLogSector + noOfSectors),
                                        DDC,
                                        &pWriteBuff[
                                        (pNandState->bytesPerPage / 2) *
                                        noOfSectors],
                                        byteSwap);

                        /* Ensure that write has happened. */
                        if (PAL_SOK == result) {
                            *xferActual += (Int32)pNandState->bytesPerPage;
                        }
                }

                /* Flush after a complete write. */
                if (PAL_SOK == result) {
                    result = nandUtlWriteSectorFlush(DDC);
                }
            }
        }
        else {
            for (noOfSectors = 0; noOfSectors < xferRequest; noOfSectors++) {
                result = nandFtlWritePage((logSector + noOfSectors),
                                DDC,
                                &pWriteBuff[
                                (pNandState->bytesPerPage / 2) * noOfSectors],
                                byteSwap);

                /* Ensure that write has happened. */
                if (PAL_SOK == result) {
                    *xferActual += (Int32)pNandState->bytesPerPage;
                }
            }
#if 0 /* Amit - Is this required ?? */
            /* Flush after a complete write. */
            if (PAL_SOK == result) {
                result = nandUtlWriteSectorFlush(DDC);
            }
#endif
        }

#ifdef NAND_DBG_LEVEL_1
    NAND_Debug ("\r\n\t-  -->Sector %d write %7s. [Result = %x xfered = %x].\n",
                                                logSector,
                                                (result == PAL_SOK) ? "Success":"Failed",
                                                result,
                                                *xferActual);
#endif /* NAND_DBG_LEVEL_1 */
    }
    return (result);
}

/**
 *  \brief Configure the NAND bank for Read and write.
 *
 *  This API identifies into which bank the logSector falls and configures
 *  the NAND bank accordingly. The logical sector is adjusted for the bank it
 *  falls into.
 *
 *  \param  DDC             [IN]    DDC Structure.
 *  \param  logSector       [IN]    logical sector for read/write.
 *  \param  curLogSector    [OUT]   Adjusted logical sector.
 *
 *  \return PAL_SOK or DDC_NAND_E_INVALID_LSN
 */
Uint32 ddcNandBankConfigure (
        DDC_NandObj             *DDC,
        Uint32                  logSector,
        Uint32                  *curLogSector)
{
    Uint32      bankNo;
    Uint32  result = DDC_NAND_E_INVALID_LSN;

    if((NULL != DDC) && (NULL != curLogSector)) {
        /* The logSector needs to be identified into which bank it falls.*/
        *curLogSector = logSector;
        for (bankNo = 0;
                        (bankNo < DDC_NAND_MAX_BANKS) && (result != PAL_SOK);
                        bankNo++) {
            /* If the bank is empty, skip the bank.*/
            if (0 != gNandBankSectorInfo[bankNo].maxSectorNo) {
                if (0 == logSector) {
                    DDC->bankNo = bankNo;
                    /* Adjust the bank addresses also accordingly. */
                    DDC->llc->bank.CExDATA = gDDCNandAsyncBank [bankNo].CExDATA;
                    DDC->llc->bank.CExALE  = gDDCNandAsyncBank [bankNo].CExALE;
                    DDC->llc->bank.CExCLE  = gDDCNandAsyncBank [bankNo].CExCLE;
                    DDC->llc->bank.NANDFxECC =
                            gDDCNandAsyncBank [bankNo].NANDFxECC;
                    result = PAL_SOK;
                }
                else {
                 if ((logSector >= gNandBankSectorInfo[bankNo].minSectorNo)
                            &&
                        (logSector <= gNandBankSectorInfo[bankNo].maxSectorNo))
                 {
                    DDC->bankNo = bankNo;
                    /* Adjust the bank addresses and the logical sector number
                     *  also accordingly. */
                    DDC->llc->bank.CExDATA = gDDCNandAsyncBank [bankNo].CExDATA;
                    DDC->llc->bank.CExALE = gDDCNandAsyncBank [bankNo].CExALE;
                    DDC->llc->bank.CExCLE = gDDCNandAsyncBank [bankNo].CExCLE;
                    DDC->llc->bank.NANDFxECC =
                            gDDCNandAsyncBank [bankNo].NANDFxECC;
                    (*curLogSector) -= (gNandBankSectorInfo[bankNo].minSectorNo);
                    result = PAL_SOK;
                 }
                }
            }
        }
    }
    return (result);
}

/**
 *  \brief Populates the Asynchronous NAND bank register array.
 *
 *  This API populates the global Asynchronous NAND bank array with the
 *  NAND register addresses.
 *
 *  \param  hNand       [IN]    NAND Driver Instance Handle
 *  \return None
 */
static void ddcNandBankInit (
            LLC_NandHandle      hNand)
{
    Int32               index;

    if( NULL != hNand) {
        for(index = 0; index < DDC_NAND_MAX_BANKS; index++)
        {
            LLC_nandGetBankInfo (hNand,
                            index,
                            (LLC_NandAsyncBank *)&gDDCNandAsyncBank[index]);
        }
    }
}

