/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+-------------------------------------------------------------------------+***
*******************************************************************************/

/**
 *  \file ddc_nand.h
 *
 *  \brief DDC defined for a typical NAND controller
 *
 *  This file extends the definition of class Stream and thus incorporates
 *  all features defined by a generic stream. This file contains definitions
 *  for a NAND DDC. This file depends on some definition from the low-level
 *  controller header, llc_nand.h. All such definitions are used as opaque
 *  objects in the NAND DDC.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author AK Mistral, Himamsu Mistral
 *
 *  \version    0.1 AK Mistral      Created.
 *  \version    1.0 Himamsu Mistral Code cleanup and release for BFT5
 *  \version    1.1 Himamsu Mistral Release for BFT6
 *  \version    1.2 Himamsu Mistral Adapted the Linux MTD device ID table
 *                                  for supporting known NAND devices.
 *                                  - Added support to identify newer NAND
 *                                    devices at run-time.
 *                                  - Release for BFT7
 *  \version    1.3 Himamsu Mistral Added Multiple NAND support, WP error
 *                                  and BB extra commands
 *                                  - Release for BFT8
 *  \version    1.4 Kiran Sutariya  Code cleanup and Modified for DM6437
 *
 */

#ifndef _DDC_NAND_H_
#define _DDC_NAND_H_

#include <ti/sdo/pspdrivers/common/psp_common.h>

#include <ti/sdo/pspdrivers/drivers/nand/src/llc_nandIf.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 *  \defgroup DDCNandMain DDC NAND Class Device Interface
 *
 *  All DDC NAND Devices use the following interfaces (along with the base
 *  DDC definitions and Stream Class definitions)
 */
/*@{*/
/**
 *  \defgroup DDCNANDIoctl NAND Ioctl Codes
 *
 *  Refer to DDC Ioctl codes and DDC Stream Class Ioctl codes for the base
 *  definitions. NAND specific extensions can be seen here.
 */
/*@{*/


#define DDC_NAND_IOCTL_ERASE            (0x100u)
    /**< IOCTL for Format                                                   */


/*@} DDCNANDIoctl */

/**
 *  \defgroup DDCNANDErrorCodes NAND Error Codes
 *
 *  Refer to DDC and Stream Class Error codes description. The following error
 *  codes are specific extensions for NAND device.
 */
/*@{*/

#define PAL_ERROR_NANDRIVER(nanderr) \
    PAL_ERROR(PAL_CRITICAL_ERROR, PAL_ERROR_SRC_DRV, 0, nanderr)
/**< NAND error code definition                                             */
#define DDC_NAND_SUCCESS                 PAL_SOK
/**< Return success                                                         */
#define DDC_NAND_E_TIMEOUT               PAL_ERROR_NANDRIVER(1u)
/**< Read/Write/Erase operation timed out                                   */
#define DDC_NAND_E_INVALID_LSN           PAL_ERROR_NANDRIVER(2u)
/**< Invalid logical sector address                                         */
#define DDC_NAND_E_PAGE_READ_FAIL        PAL_ERROR_NANDRIVER(3u)
/**< Read operation failed                                                  */
#define DDC_NAND_E_PAGE_PROGRAM_FAIL     PAL_ERROR_NANDRIVER(4u)
/**< Write operation failed                                                 */
#define DDC_NAND_E_MEDIA_WRITE_PROTECTED PAL_ERROR_NANDRIVER(5u)
/**< Write/erase operation attempted on a write protected media             */
#define DDC_NAND_E_BLOCK_ERASE_FAIL      PAL_ERROR_NANDRIVER(6u)
/**< Erase operation failed                                                 */
#define DDC_NAND_E_ECC_ERROR             PAL_ERROR_NANDRIVER(7u)
/**< ECC verification failed                                                */
#define DDC_NAND_E_MEDIA_NOT_PRESENT     PAL_ERROR_NANDRIVER(8u)
/**< NAND flash media absent                                                */
#define DDC_NAND_E_INVALID_MEDIA         PAL_ERROR_NANDRIVER(9u)
/**< Invalid/unknownNAND flash media                                        */
#define DDC_NAND_E_OUT_OF_FREE_BLOCKS    PAL_ERROR_NANDRIVER(10u)
/**< NO free blocks available in NAND                                       */
#define DDC_NAND_E_SSFDC_ERROR           PAL_ERROR_NANDRIVER(11u)
/**< NAND driver is not SSFDC compliance                                    */
#define DDC_NAND_E_STATUS_READ_FAIL      PAL_ERROR_NANDRIVER(12u)
/**< Status read failed                                                     */
#define DDC_NAND_E_BAD_BLOCK             PAL_ERROR_NANDRIVER(13u)
/**< Block is BAD                                                           */
#define DDC_NAND_E_DMA_FAIL              PAL_ERROR_NANDRIVER(14u)
/**< DMA failed                                                             */
#define DDC_NAND_E_INVALID_PBA           PAL_ERROR_NANDRIVER(15u)
/**< Not a valid physical block address                                     */
#define DDC_NAND_E_NO_PAGE_EXIST         PAL_ERROR_NANDRIVER(16u)
/**< Page doesn't exist in the block                                        */

#define DDC_NAND_E_INVALID_DEVICE_ID    PAL_ERROR_NANDRIVER(17u)
/**< Not a valid device ID                                                  */

/*@} DDCNANDErrorCodes */

#if !(defined DDC_NAND_8_BIT) && !(defined DDC_NAND_16_BIT)
#error "Define NAND to be 8-bit/16-bit (DDC_NAND_8_BIT) in build options."
#endif

#if (defined DDC_NAND_8_BIT) && (defined DDC_NAND_16_BIT)
#error "Define one of 8-bit/16-bit (DDC_NAND_8_BIT) in build options."
#endif

#define MAX(x, y) (((x) < (y)) ? (y) : (x))
/**< Max of two quantities.                                                 */

/**
 *  \defgroup DDCNandDeviceConfiguration NAND Device Configuration
 *
 *  NAND configurations that need to be checked before initializing the
 *  device.
 *
 * @{
 */
#define DDC_NAND_PROTECTED_AREA                         (0u)
/**< Enable Protected Area support. Note - Keep this number in multiple
 *   of minimum erase unit                                                  */

/**
 *  \defgroup DDCNandDriverFeatures NAND Device Options
 *
 *  Options/features that need to be enabled/disabled for the NAND driver.
 *
 * @{
 */
#define DDC_NAND_BBM
/**< Enable Bad Block Management support                                    */
/* #define DDC_NAND_ENABLE_ECC */
/**< Define ECC for NAND - In DM6437, ECC is calculated by the Hardware.    */
#define DDC_NAND_POWER_SAFE
/**< Enable or Disable Power Safe                                           */
#define DDC_NAND_READ_DMA
/**< Enable DMA for read operation                                          */
#define DDC_NAND_WRITE_DMA
/**< Enable DMA for write operation                                         */
#define DDC_NAND_2048_WRITE_READ
/**< Write or read in chunks of 2048 or 256 bytes at the physial layer
 * for ECC calculation/generation.                                          */
#define DDC_NAND_MAX_BANKS                          (1u)
/**< Maximum number of banks on <b> DM6437</b> NAND card. One 8-bit bank    */
#define DDC_NAND_MAX_MFGR                           (9u)
/**< Max. number of Different Flash Manufacturer device support */
#define DDC_NAND_MAX_FLASH_SPEC                         (59u)
/**< Max. number of Different Flash specifications */

/**
 *  \defgroup DDCNandStateDefaults NAND State Structure default configuration
 *
 *  NAND default configuration for NAND state.  However, at run-time, when the
 *  NAND is detected, the no of bytes per page, no of pages per block, no of
 *  blocks per zone are derived from the NAND device detected.
 *
 * @{
 */
#define DDC_NAND_SB_NO_BLOCKS                       (8192u)
/**< Number of blocks in a small block NAND                                 */
#define DDC_NAND_SB_PAGES_PER_BLOCK                 (32u)
/**< Number of pages per block in a small block NAND                        */
#define DDC_NAND_SB_BYTES_PER_PAGE                  (512u)
/**< Number of Bytes per page                                               */
#define DDC_NAND_SB_SPARE_AREA_SIZE                 (16u)
/**< Redundant area size                                                    */

#define DDC_NAND_BB_NO_BLOCKS                       (2048u)
/**< Number of blocks in NAND                                               */
#define DDC_NAND_BB_PAGES_PER_BLOCK                 (64u)
/**< Number of pages per block                                              */
#define DDC_NAND_BB_BYTES_PER_PAGE                  (2048u)
/**< Number of Bytes per page                                               */
#define DDC_NAND_BB_SPARE_AREA_SIZE                 (64u)
/**< Redundant area size                                                    */

#define DDC_NAND_BLOCKS_PER_ZONE                    (1024u)
/**< Number of blocks that constitute a zone                                */
#define DDC_NAND_USABLE_BLOCKS_PER_ZONE             (1000u)
/**< Number of blocks that are usable                                       */

#define DDC_NAND_NUM_ZONES                          (DDC_NAND_NO_BLOCKS/ \
                                                     DDC_NAND_BLOCKS_PER_ZONE)
/**< Number of zones in a device                                            */
#define DDC_NAND_PHY_BLOCK_ENTRY_LEN                10u
/**< number of bits required to hold a physical block entry                 */
#define DDC_NAND_BBM_BUF_SIZE                       ((DDC_NAND_BLOCKS_PER_ZONE/16u) * \
                                                      DDC_NAND_NUM_ZONES)
/**< Size of the BBM Buffer:
 * One bit is enough to check, if a block is bad! So, blocks_per_zone/16
 * (i.e, 1024/16 = 64) words are enough for a zone and hence
 * (64) * Numzones is the size of the BBM buffer
 */
#define DDC_NAND_WLA_BUF_SIZE                       ((DDC_NAND_BLOCKS_PER_ZONE/16u) * \
                                                      DDC_NAND_NUM_ZONES)
/**< Size of the WLA Buffer:
 * One bit is enough to check, for WLA! So, blocks_per_zone/16
 * (i.e, 1024/16 = 64) words are enough for a zone and hence (64) * Numzones
 * is the size of the WLA buffer.
 */
#define DDC_NAND_PHY_BUF_SIZE                       ((DDC_NAND_PHY_BLOCK_ENTRY_LEN  * \
                                                     DDC_NAND_USABLE_BLOCKS_PER_ZONE/16u)* \
                                                     DDC_NAND_NUM_ZONES)
/**< Size of the Phy->log Buffer:
 * Each phy block entry consumes 10 bits and 1000 (Usable) blocks shall be
 * contained in 10*1000 = 10000 bits or 625 words. Hence total size required
 * would be 625 * Numzones in the nand device.
 */

#define DDC_NAND_DEF_SPARE_AREA_SIZE                DDC_NAND_BB_SPARE_AREA_SIZE
/**< Default spare area size; for array allocation                          */

/*  Changed for Bad block management bug  */
/**< Default spare area size; for array allocation                          */
#define DDC_NAND_DEF_NUM_ZONES                      (8u)
/**< Default number of zone size; for array allocation                      */
#define DDC_NAND_DEF_PHY_BUF_SIZE                   (625u*DDC_NAND_DEF_NUM_ZONES)
/**< Default Physical buffer size; for array allocation                     */
#define DDC_NAND_DEF_BBM_BUF_SIZE                   (64u*DDC_NAND_DEF_NUM_ZONES)
/**< Default BBM array size; for array allocation                           */
#define DDC_NAND_DEF_WLA_BUF_SIZE                   (64u*DDC_NAND_DEF_NUM_ZONES)
/**< Default WLA buffer size; for array allocation                          */
/*  Changed for Bad block management bug  - Up to here  */


#define DDC_NAND_ENT_BUF_SIZE                       (DDC_NAND_DEF_PHY_BUF_SIZE)
/**< Size of the Entry buffer size:
 * Each phy block entry consumes 10 bits and 1000 (Usable) blocks shall be
 * contained in 10*1000 = 10000 bits or 625 words. Hence total size required
 * would be 625 * Numzones in the nand device. Numzones is max '8' for small
 * block NAND and '2' for big block NAND.
 *
 * Note: The #define is to say that the **values** of DDC_NAND_ENT_BUF_SIZE
 * and DDC_NAND_DEF_PHY_BUF_SIZE are same.
 */
#define DDC_NAND_INVALID_PHYBLOCK                   0xFFFFu
/**< Invalid physical block number                                          */

#define DDC_NAND_NO_BLKS_CACHED                     8u
/**< Number of redundant area blocks to cache                               */
#define DDC_NAND_NO_PAGES_CACHED                    0u
/**< Number of redundant area pages to cache                                */
#define DDC_NAND_PAGE_CACHE_ENTRY_SIZE             DDC_NAND_BB_PAGES_PER_BLOCK
/**< Page cache entry size - number of pages in a block;
 * 64 for Big Block and 32 for Small Block.                                 */

#define DDC_NAND_ADDR_CYCLES                        4u
/**< Number of Addr cycles                                                  */
#define DDC_NAND_WLA_FLAG                           1u
/**< Enable/Disable WLA Flag                                                */
#define DDC_NAND_DEVICE_ID                          0xFFFFu
/**< Nand Device Id                                                         */
#define DDC_NAND_MAX_BBM_DEPTH                      90u
/**< Allowable BBM depth                                                    */
#define DDC_NAND_DEF_LOGICAL_BLOCK                  0xDEADu
/**< Address of the current logical block                                   */
#define DDC_NAND_DEF_SECTOR                         0xDEADu
/**< Current sector                                                         */
#define  DDC_NAND_DEF_OLD_PHY_BLOCK                 0xDEADu
/**< Address of the previous Physical block                                 */
#define DDC_NAND_DEF_PREV_WRITTEN_ZONE              0xDEADu
/**< Address of the zone previously being accessed                          */
#define DDC_NAND_WRT_BUF_SIZE                       1024u
/**< Size of the writebuffer in the nand structure                          */

#define DDC_NAND_DEF_CAHCE_INIT_VALUE               0xFFu
/**< Default intializer values for cache(d) block(s) and page(s)            */

#define DDC_NAND_DEF_INIT_VALUE                     0xFFFFu
/**< Default intializer values for various buffers and status values        */
#define DDC_NAND_BLOCK_STATUS_FREE                  0xFFFFu
/**< Free block status                                                      */
#define DDC_NAND_BLOCK_ADDR_FREE                    0xFFFFu
/**< Free block address                                                     */
#define DDC_NAND_DATA_STATUS_FREE                   0xFFFFu
/**< Free data status                                                       */
#define DDC_NAND_LOG_PAGE_FREE                      0xFFFFu
/**< Free logical page                                                      */

#define DDC_NAND_READ_STATUS_TIMEOUT                100000u
/**< Timeout when reading read status of an operation                       */
#define DDC_NAND_ECC_SIZE                           17u
/**< ECC Size                                                               */
/* @} DDCNandStateDefaults */

/**
 *  \defgroup DDCNandCmds NAND Device commands
 *
 *  NAND Device commands. These commands should be, ideally, the same
 *  across NAND devices. Some commands are exclusively for a selected
 *  Big Block NAND devices only.
 *
 * @{
 */

/* NAND device standard commands */
#define DDC_NAND_CMD_LO_PAGE                        0x00u
/**< Command to access low page                                             */
#define DDC_NAND_CMD_HI_PAGE                        0x01u
/**< Command to access high page                                            */
#define DDC_NAND_CMD_SPARE_AREA                     0x50u
/**< Command to access spare area - not found in Big Block NANDs            */
#define DDC_NAND_CMD_DEVID                          0x90u
/**< Command to access device Id                                            */
#define DDC_NAND_CMD_DEVADD                         0x00u
/**< Command to access device address                                       */
#define DDC_NAND_CMD_RESET                          0xFFu
/**< Command to perform a reset                                             */
#define DDC_NAND_CMD_SET                            0x00u
/**< Command to perform a set                                               */
#define DDC_NAND_CMD_PGRM_START                     0x80u
/**< Command to start programming NAND                                      */
#define DDC_NAND_CMD_PGRM_END                       0x10u
/**< Command to stop programming NAND                                       */
#define DDC_NAND_CMD_BLK_ERASE_CMD1                 0x60u
/**< Command to erase block                                                 */
#define DDC_NAND_CMD_BLK_ERASE_CMD2                 0xD0u
/**< Command to erase block                                                 */
#define DDC_NAND_CMD_READY                          0x40u
/**< Command to put NAND in ready state                                     */
#define DDC_NAND_CMD_STATUS                         0x70u
/**< Command to get the status of NAND                                      */
#define DDC_NAND_CMD_READ_START                     0x00u
/**< Command to read the start of NAND                                      */
#define DDC_NAND_CMD_READ_END                       0x30u
/**< Command to read the end of NAND                                        */

/* Big block NAND extra commands */
#define DDC_NAND_CMD_RANDOM_PGRM_START              0x85u
/**< Command to perform a random data input <b>(Big block only command)</b> */
#define DDC_NAND_CMD_RANDOM_READ_START              0x05u
/**< Command to start a random data output  <b>(Big block only command)</b> */
#define DDC_NAND_CMD_RANDOM_READ_END                0xE0u
/**< Command to end a random data output    <b>(Big block only command)</b> */
#define DDC_NAND_CMD_COPY_BACK_READ                 0x35u
/**< Command to perform a Copy-back read    <b>(Big block only command)</b> */

/*@} DDCNandCmds */

#define DDC_NAND_BB_CMD_STATUS_SUCCESS            LLC_NAND_BB_CMD_STATUS_READY
/**< BIG block NAND status  - Other bits need not be checked!               */
#define DDC_NAND_SB_CMD_STATUS_SUCCESS            LLC_NAND_SB_CMD_STATUS_READY
/**< SMALL block NAND status - Other bits need not be checked!              */

/**
 *  \defgroup DDCNandMfgCodes NAND Device Manufacturers.
 *
 *  Various manufacturers of NAND devices.
 *  NOTE: <b>All of these NAND devices may not be tested/supported!</b>
 *
 * @{
 */
#define DDC_NAND_TOSHIBA_MFG_CODE                   0x98u
/**< Manufacturer code for SAMSUNG NAND flash                               */
#define DDC_NAND_SAMSUNG_MFG_CODE                   0xECu
/**< Manufacturer code for TOSHIBA NAND flash                               */
#define DDC_NAND_FUJITSU_MFG_CODE                   0x04u
/**< Manufacturer code for FUJITSU NAND flash                               */
#define DDC_NAND_NATIONAL_MFG_CODE                  0x8Fu
/**< Manufacturer code for NATIONAL NAND flash                              */
#define DDC_NAND_RENESAS_MFG_CODE                   0x07u
/**< Manufacturer code for RENESAS NAND flash                               */
#define DDC_NAND_STMICRO_MFG_CODE                   0x20u
/**< Manufacturer code for ST MICRO NAND flash                              */
#define DDC_NAND_HYNIX_MFG_CODE                     0xADu
/**< Manufacturer code for HYNIX NAND flash                                 */
/* @} DDCNandMfgCodes */

#define DDC_nandIsWriteProtected(nandDevFlags)                              \
    ((nandDevFlags) & LLC_NAND_MEDIA_WRITE_PROTECTED)
/**< Check if NAND is write protected
 *  \param  nandDevFlags  [IN]    {Uint32} NAND device options flag         */


#define DDC_STREAM_WAIT_NONE    0u
/**< Do not wait for the IO operation to complete, return immediately. Valid
 * value for DDC_StreamIoPacket::timeout                                    */
#define DDC_STREAM_WAIT_FOREVER 0xFFFFFFFFu
/**< Wait forever until the requested IO operation completes, with or without
 * an error condition. Valid value for DDC_StreamIoPacket::timeout          */
/*@} DDCNandDeviceConfiguration */

typedef Ptr DDC_Handle;
typedef Ptr DDA_Handle;

/**
 *  \brief Type of NAND detected.
 */
typedef enum
{
    DDC_NAND_NONE = 0,
    /**< No NAND Present                                                    */
    DDC_NAND_BIG_BLOCK,
    /**< Big Block NAND detected                                            */
    DDC_NAND_SMALL_BLOCK,
    /**< Small Block NAND detected                                          */
    DDC_NAND_INVALID
    /**< Unknown/Invalid NAND detected                                      */
} DDC_NandType;

/**
 *  \brief Nand Block Status
 *
 */
typedef enum {
    DDC_NAND_BAD_BLOCK,
    /**< Block is BAD                                                       */
    DDC_NAND_GOOD_BLOCK
    /**< Block is Good                                                      */
} DDC_NandBlockStatus;

/**
 *  \brief Nand block State
 *
 */
typedef enum {
    DDC_NAND_USED_BLOCK,
    /**< Block is USED                                                      */
    DDC_NAND_FREE_BLOCK,
    /**< Block is FREE                                                      */
    DDC_NAND_UNKNOWN_BLOCK
    /**< Block state is unknown                                             */
} DDC_NandBlockState;

/**
 *  \brief Status of the logical block
 *
 */
typedef enum {
    DDC_NAND_ENTRY_OUT,
    /**< Remove the logical block entry from the mapping table              */
    DDC_NAND_ENTRY_IN
    /**< Make the Logical Block entry into the mapping table                */
} DDC_NANDLogBlockStatus;

/*
 * A DDC NAND driver will implement these functions and pass the pointers to
 * DDA layer
 */

/**
 *  \defgroup DDCNandFunctions NAND DDC Functions
 *
 *  NAND extends the Stream Class Driver Interfaces
 *
 * @{
 */

/**
 *  \brief NAND DDC Initialization and Configuration Object
 *
 *  Structure of the Initialization and Configuration object for NAND. This
 *  object is per controller. No assumption is made within the driver that
 *  this object shall persist beyond the device initialization phase.
 *  Therefore, contents of this structure are used for device and data
 *  structure initialization. A direct reference to this structure is not
 *  tracked any beyond the initialization phase.
 */
typedef struct {

    Int32              instId;     /**< Instance id/number */

    PSP_OpMode          opMode;

    LLC_NandObj             *llc;
    /**< Low level controller object                                        */
    Int                     msecPoll;
    /**< The interval between two poll requests to the underlying hardware
     * to perform IO                                                        */
} DDC_NandInitConfig;

/**
 *  \brief NAND Open params configuration structure
 *
 */
typedef struct {
    LLC_NandHwConfig        *hwConfig;
    /**< NAND hardware configuration that must be applied to the controller */
} DDC_NandOpenParams;


/**
 *  \brief   Block cache information
 *
 *  Cache information of a block is stored in this object to speed up
 *  searching at run-time, reducing the number of access to the device.
 */
typedef struct {
    Uint8   cachedLogPageNo [DDC_NAND_PAGE_CACHE_ENTRY_SIZE];
    /**< Physical sector nos of pages written sequentially in a block       */
    Uint16  phyBlockNo;
    /**< Physical block number                                              */
    Uint8   cacheIndex;
    /**< Indicates if the Block information can be discarded or not         */
} DDC_NandCacheBlkRedInfo;

/**
 *  \brief   Cached page in a cached block
 *
 *  Information about an already cached block is put in here which again helps
 *  to speed up reads and writes on the device.
 */
typedef struct {
    Uint8 cacheIndex;
    /**< Index of the cached page                                           */
    Uint8 pageFound;
    /**< Is the page found?                                                 */
} DDC_NandCachePageRedInfo;

/**
 *  \brief NAND device state structure.
 *
 *  The interface definition for the NAND device configuration information is
 *  stored in this object. However, at run-time, when the NAND device is
 *  detected, the no of bytes per page, no of pages per block, no of blocks
 *  per zone are loaded from the NAND device detected.
 */
typedef struct {
    LLC_NandType nandType;              /**< Big Block/Small block NAND     */
    LLC_NandWidth nand8Bit;             /**< 8/16 - bit NAND                */
    Uint16 addrCycle;                   /**< Number of Address Cycles       */
    Uint16 bytesPerPage;                /**< Num of bytes per page          */
    Uint16 pagesPerBlock;               /**< Num of pages per block         */
    Uint32 noOfBlocks;                  /**< Num of blocks in NAND          */
    Uint32 spareAreaSize;               /**< Spare area size                */
    Uint16 numZones;                    /**< Num of zones                   */
    Uint32 totalPages;                  /**< Total no of pages              */
    Uint32 deviceID;                    /**< Nand Device Id                 */
    Uint32 noOfBadBlocks;               /**< No of bad blocks detected      */
    Uint32 noOfBadPages;                /**< No of bad pages/sectors        */
    Int8   *name;                       /**< NAND device Name               */
    Uint32 nandOptions;                 /**< Extra cmds for Big-Block       */
    Uint32 nandDevFlags;                /**< Bit-Flags for future use       */

    Uint16 entryBufferSize;             /**< Phy-Log Block mapping Tbl size */
    Uint16 CISBlock;                    /**< CIS block Flag                 */

    Uint16 writeFlush;                  /**< Write Flush for cache          */
    Uint16 writeFlushPending;           /**< Flush pending for prev block?  */
    Uint16 newWriteFlushPending;        /**< Flush pending for current block*/

    Uint16 skipWrite;                   /**< Flag to indicate to skip-write */

    Uint32 currentWritePage;            /**< Current write page             */
    Uint16 currentLogBlock;             /**< Current logical block addr    */
    Uint32 currentSector;               /**< Current sector                 */
    Uint32 currentPhyWritePage;         /**< Current physical write page    */
    Uint16 oldPhyBlock;                 /**< Previous Physical block addr   */
    Uint16 writeZone;                   /**< Current Zone address           */
    Uint16 prevWriteZone;               /**< Previous Zone address          */

    Uint16 *pPhyLogEntries;             /**< Phy-log mapping table address  */
    Uint16 *pMappingbitbuffer;          /**< Phy-log bit mapping buf addr   */
    Uint16 *pBBMbitbuffer;              /**< BBM table address              */
    Uint16 *pWLAbitbuffer;              /**< Wear levelling table address   */
    Uint16 *pWriteBuffer;               /**< Write buffer address           */

    Int  (*IntrDisable)();              /**< Function ptr to Intr disable   */
    Int  (*IntrEnable)();               /**< Function ptr to Intr enable    */

    Uint16 BBMdepth;                    /**< Num of bad blocks              */
    Uint16 WLAflag;                     /**< Wear levelling algorithm flag  */

    Uint16 logicalSecAddr;              /**< Logical Sector Address         */
    Uint16 logicalColAddr;              /**< Logical Column Address         */

    DDC_NandCacheBlkRedInfo *pBlockRedInfo; /**< Cached Block Redundant info*/

    Uint32 protectedBlks;               /**< Number of protected Blocks     */

    /*  Changed for Bad block management bug  */
    Uint16 usableBlockInZone[DDC_NAND_DEF_NUM_ZONES];
    /**< Usable blocks per zone         */
    /*  Changed for Bad block management bug  - Up to here  */

} DDC_NandState;

/**
 *  \brief NAND static device structure
 *
 *  This object contains information of known NAND devices.
 */
typedef struct {
    Int8            *name;              /**< NAND Flash Name                */
    Uint32          id;                 /**< NAND Flash Id                  */
    Uint32          bytesPerPage;       /**< Bytes per page                 */
    Uint32          chipSize;           /**< NAND Flash Total size          */
    Uint32          eraseSize;          /**< NAND Flash Erase size          */
    LLC_NandWidth   nand8Bit;           /**< 8-bit or 16 bit                */
    Uint32          nandOptions;        /**< Extra cmds for Block           */
} DDC_NandFlashDev;

/**
 * \brief NAND Flash Manufacturer's ID Structure
 *
 * Object contains the known manufacturer's Name and ID.
 */
typedef struct {
    Int             id;                 /**< Manufacturer ID                */
    Int8            *name;              /**< Manufacturer name              */
} DDC_NandMfg;

/**
 *  \brief NAND DDC persistent context object
 *
 *  A singleton of this type per NAND controller instance in the system
 */
typedef struct {
    DDA_Handle          hDDA;       /**< DDA Handle */
    Uint32                  flags;


    Uint32              versionId;  /**< Version Id */
    Uint32              instId;     /**< Instance id/number */
    PSP_DriverState     state;      /**< DDC State */

    PSP_OpMode          opMode;     /**< Operation mode of driver       */
    DDC_NandState        nandState[DDC_NAND_MAX_BANKS];
    /**< Nand Device State Structure                                        */
    LLC_NandHandle       llc;
    /**< Handle on "this" instance of the NAND CSL                          */
    Uint32               bankNo;
    /**< Current bank no to access (Multiple NAND banks)                    */
    Uint32               maxNandSectors;
    /**< Maximum number of sectors from all banks.                          */
    Uint32               maxNandSectorSize;
    /**< Maximum sector size from all banks, accepted by NAND driver.       */
} DDC_NandObj;

/**
 * \brief Current Asynchronous NAND Bank configuration.
 */
typedef struct {
    volatile Uint32     CExDATA;
    /**< Current NAND Data register                                         */
    volatile Uint32     CExALE;
    /**< Current NAND Addres latch register                                 */
    volatile Uint32     CExCLE;
    /**< Current NAND Command latch register                                */
    volatile Uint32     NANDFxECC;
    /**< Current NAND ECC register                                          */
} DDC_NandAsyncBank;

 /**
 *  \brief NAND device redundant structure
 *
 *  The redundant information of NAND is stored in this object.
 */
typedef struct {
    Uint16 logPageNo;
    /**< Logical page number                                                */
    Uint16 dataStatus;
    /**< Page data status                                                   */
    Uint16 blockStatus;
    /**< Block status                                                       */
    Uint16 blockAddress;
    /**< Logical Block Address                                              */
} DDC_NandRedundant;

/**
 *  \brief NAND device Redundant area information - Small block
 *
 *  The Samsung format of redundant information for NAND (Small Block).
 */
typedef struct {
    Uint8   LSN0;           /**< Logical Sector number                      */
    Uint8   LSN1;           /**< Logical Sector number                      */
    Uint8   LSN2;           /**< Logical Sector number                      */

    Uint8   WC0;            /**< Status against power failure when write.   */
    Uint8   WC1;            /**< Status against power failure when write.   */

    Uint8   badBlockInfo;   /**< Bad Block Info                             */

    Uint8   ECC0;           /**< ECC code for main data area                */
    Uint8   ECC1;           /**< ECC code for main data area                */
    Uint8   ECC2;           /**< ECC code for main data area                */

    Uint8   S_ECC0;         /**< ECC code for LSN data area                 */
    Uint8   S_ECC1;         /**< ECC code for LSN data area                 */

    Uint8   resv[5];        /**< Reserved                                   */
} DDC_NandSBSpareArea;

/**
 *  \brief NAND device Redundant area information - Small block
 *
 *  The SSFDC format of redundant information for NAND (Small Block).
 */
typedef struct {
    Uint8   resv[4];        /**< Reserved                                   */
    Uint8   dataStatus;     /**< Data status                                */
    Uint8   blockStatus;    /**< Block Status                               */
    Uint8   blockAddr1LSB;  /**< Block Address field 1 LSB                  */
    Uint8   blockAddr1MSB;  /**< Block Address field 1 MSB                  */
    Uint8   ECC2[3];        /**< ECC2 field                                 */
    Uint8   blockAddr2LSB;  /**< Block Address field 2 LSB                  */
    Uint8   blockAddr2MSB;  /**< Block Address field 2 MSB                  */
    Uint8   ECC1[3];        /**< ECC1 field                                 */
} DDC_NandSSFDCSBSpareArea;


/**
 *  \brief NAND device Redundant area information - Big block
 *
 *  The Samsung format of redundant information for NAND (Big Block).
 */
typedef struct {
    Uint8   badBlock;       /**< Bad Block Info                             */

    Uint8   resv0[1];       /**< Reserved                                   */

    Uint8   LSN0;           /**< Logical Sector number                      */
    Uint8   LSN1;           /**< Logical Sector number                      */
    Uint8   LSN2;           /**< Logical Sector number                      */

    Uint8   resv1[1];       /**< Reserved                                   */

    Uint8   WC0;            /**< Status against power failure when write.   */
    Uint8   WC1;            /**< Status against power failure when write.   */

    Uint8   ECC0;           /**< ECC code for main data area                */
    Uint8   ECC1;           /**< ECC code for main data area                */
    Uint8   ECC2;           /**< ECC code for main data area                */

    Uint8   S_ECC0;         /**< ECC code for LSN data area                 */
    Uint8   S_ECC1;         /**< ECC code for LSN data area                 */

    Uint8   resv2[3];       /**< Reserved                                   */
} DDC_NandBBSpareArea;

/**
 *  \brief NAND Bit fail code structure
 *
 */
typedef struct {
    Uint8   parity;
    /**< Parity                                                             */
    Uint8   parity_prime;
    /**< Parity Prime                                                       */
    Uint8   bitAddress;
    /**< Bit Address                                                        */
    Uint8   bitAdressPrime;
    /**< Bit Address prime                                                  */
} DDC_NandBitFailCodeStruct;

/**
 *  \brief NAND sector information structure for a bank.
 *
 */
typedef struct {
    Uint32   minSectorNo;
    /**< Minimum sector number                                              */
    Uint32   maxSectorNo;
    /**< Maximum sector number                                              */
} DDC_NandBankSectorInfo;


/* Driver Create APIs */
PAL_Result nandCreateInstance(
            Int                             instId,
            DDA_Handle                      hDDA,
            DDC_Handle                      *hDDC,
            Ptr                             param);
/* Device Service APIs */
PAL_Result nandDelInst(
            DDC_Handle                      hDDC,
            Ptr                             param);
PAL_Result nandInit(
            DDC_Handle                      hDDC,
            Ptr                             param);
PAL_Result nandDeinit(
            DDC_Handle                      hDDC,
            Ptr                             param);
PAL_Result nandOpen(
            DDC_Handle                      hDDC,
            Ptr                             param);
PAL_Result nandClose(
            DDC_Handle                      hDDC,
            Ptr                             param);
PAL_Result nandControl(
            DDC_Handle                      hDDC,
            Int                             cmd,
            Ptr                             cmdArg,
            Ptr                             param);

PAL_Result nandReadSync(
            DDC_NandObj                     *hDDC,
            Uint16                          *buf,
            const Int32                     xferRequest,
            Uint32                          timeout,
            Ptr                             param,
            Int32                             *xferActual);

PAL_Result nandWriteSync(
            DDC_NandObj                     *hDDC,
            Uint16                          *buf,
            const Int32                     xferRequest,
            Uint32                          timeout,
            Ptr                             param,
            Int32                           *xferActual);
/**
 *  \brief Configure the NAND bank for Read and write.
 *
 *  This API identifies into which bank the logSector falls and configures
 *  the NAND bank accordingly. The logical sector is adjusted for the bank it
 *  falls into.
 *
 *  \param  DDC             [IN]    DDC Structure.
 *  \param  logSector       [IN]    logical sector for read/write.
 *  \param  curLogSector    [OUT]   Adjusted logical sector.
 *
 *  \return PAL_SOK or DDC_NAND_E_INVALID_LSN
 */
Uint32 ddcNandBankConfigure (
                        DDC_NandObj         *DDC,
                        Uint32              logSector,
                        Uint32              *curLogSector);
/**
 *  \defgroup DDCNandFTLFunctions NAND DDC Flash Translation Functions
 *
 *  NAND Flash Translation Layer Interfaces
 *
 * @{
 */

/**
 *  \brief Initializes the FTL of NAND.
 *
 *   This function initializes the NAND device and creates BBM,WLA and
 *   physical to logical mapping table.
 *
 *  \param  DDC [IN]    DDC Object
 *  \return PAL_SOK or PAL Error code
 */
PAL_Result nandFtlInit (
                        DDC_NandObj         *DDC);



/**
 *  \brief Reads a page of NAND.
 *
 *  This function reads one page of data from NAND device to the application
 *  supplied buffer. For the given logical sector the logical block, then the
 *  physical block, and hence the physical sector is identified and proceeds
 *  to read from that physical sector.
 *
 *  \param  logSector   [IN]    Logical sector from where data has to be read
 *  \param  DDC         [IN]    DDC Object
 *  \param  pReadBuff   [IN]    Buffer to which the data has to be copied
 *
 *  \return DDC_NAND_E_TIMEOUT              - Operation timed out        \n
 *          DDC_NAND_E_INVALID_LSN          - Invalid Logical Block Addr \n
 *          DDC_NAND_E_INVALID_PBA          - Invalid Physical block     \n
 *          DDC_NAND_E_STATUS_READ_FAIL     - Read Failed                \n
 *          DDC_NAND_E_OUT_OF_FREE_BLOCKS   - No free blocks available   \n
 *          PAL_SOK                         - Success
 */
PAL_Result nandFtlReadPage(
                        Uint32              logSector,
                        DDC_NandObj         *DDC,
                        Uint16              *pReadBuff);

/**
 *  \brief Writes a page of NAND.
 *
 *   This function maps the logical address of buffer provided by the
 *   application to the physical address, performs Bad Block Management and
 *   wear leveling and then writes the sector data into the NAND device.
 *
 *  \param  logSector   [IN]    Logical sector from where data has to be read
 *  \param  DDC         [IN]    DDC Object
 *  \param  pWriteBuff  [IN]    Buffer containing data to be written
 *  \param  byteSwap    [IN]    If set, byte will be swapped and written else
 *                              without swapping, byte will be written.
 *
 *  \return DDC_NAND_E_TIMEOUT              - Operation timed out        \n
 *          DDC_NAND_E_INVALID_LSN          - Invalid Logical Block Addr \n
 *          DDC_NAND_E_OUT_OF_FREE_BLOCKS   - No free blocks available   \n
 *          PAL_SOK             - Success
 */
PAL_Result nandFtlWritePage (
                        Uint32              logSector,
                        DDC_NandObj         *DDC,
                        Uint16              *pWriteBuff,
                        Uint8               byteSwap);

/**
 *  \brief Erases NAND device completely. API to be used with caution!
 *
 *  This function erases data on the NAND device. From a filesystem view,
 *  ideally, the device is formatted.
 *
 *  \param  DDC         [IN]    DDC Object
 *
 *  \return Success (0) or number of blocks failed.
 */
Uint32 nandFtlErase (
                        DDC_NandObj         *DDC);

/*@} DDCNandFTLFunctions */

/**
 *  \defgroup DDCNandUTLFunctions NAND DDC Utility Functions
 *
 *  NAND Driver Utility Interfaces
 *
 * @{
 */

/**
 *  \brief Flushes the currently written block if any
 *
 *  \param  DDC [IN]    DDC Structure
 *
 *   \return DDC_NAND_E_TIMEOUT             - Operation Timed out        \n
 *           DDC_NAND_E_PAGE_PROGRAM_FAIL   - Page program error         \n
 *           DDC_NAND_E_OUT_OF_FREE_BLOCKS  - No free blocks available   \n
 *           PAL_SOK                        - Return Success
 */
PAL_Result nandUtlWriteSectorFlush (
                        DDC_NandObj        *DDC);

/**
 *   \brief Copy back a page of NAND
 *
 *  The copy-back program is configured to quickly and efficiently rewrite
 *  data stored in one page without utilizing an external (software) memory.
 *
 *  <b>NOTE: This is a Big block NAND feature and may not available on all
 *  NAND devices.</b>
 *
 *   \param DDC             [IN]    DDC Structure
 *   \param curPage         [IN]    current page in current physical block
 *   \param currentPhyBlock [IN]    current physical block
 *
 *   \return DDC_NAND_E_PAGE_READ_FAIL      - Read error                 \n
 *           DDC_NAND_E_PAGE_PROGRAM_FAIL   - Page program error         \n
 *           PAL_SOK                        - Return Success             \n
 */
PAL_Result nandUtlCopyBackPage (
                        DDC_NandObj         *DDC,
                        Uint16              curPage,
                        Uint16              currentPhyBlock);

/**
 *   \brief Checks if the page/sector is free or not
 *
 *  \param  DDC [IN]    DDC Structure
 *   \param phySector   [IN]    physical sector number
 *
 *   \return DDC_NandBlockState:                                \n
 *           - DDC_NAND_FREE_BLOCK       - Free block           \n
 *           - DDC_NAND_USED_BLOCK       - Used block           \n
 *           - DDC_NAND_UNKNOWN_BLOCK    - Block State unknown
 */
DDC_NandBlockState  nandUtlIsFreePage (
                        DDC_NandObj        *DDC,
                        Uint32 phySector);

/**
 *   \brief Find next free block
 *
 *  Finds the next available free block corresponding to the physical block
 *  supplied, by consulting the wear-leveling alogorithm. If the NAND has no
 *  more free blocks, a 'out-of-free' blocks is returned.
 *
 *  \param  DDC [IN]    DDC Structure
 *   \param phyBlock    [IN]    Physical block number
 *
 *   \return DDC_NAND_E_OUT_OF_FREE_BLOCKS  - No free blocks available   \n
 *           PAL_SOK                        - Return Success
 */
PAL_Result nandUtlFndNxtFreeBlk (
                        DDC_NandObj         *DDC,
                        Uint16              *phyBlock);

/*  Changed for Bad block management bug  */
/**
 *   \brief Update the BBM table
 *
 *  Updates Bad Block Managemet table for the physical block supplied. It also
 *  decrements the number of available pages on NAND, if a bad block is found.
 *
 *  \param DDC          DDC Structure
 *  \param phyBlock     Physical block number,
 *  \param  status      Status of the block (good or bad)
 *  \param  nandRed     Spare area info of first page of phyBlock or NULL
 *
 *   \return PAL_SOK   - Return Success
 */
PAL_Result nandUtlUpdateBBMTable (
                        DDC_NandObj             *DDC,
                        Uint16              phyBlock,
                         DDC_NandBlockStatus status,
                         DDC_NandRedundant   *nandRed);

/**
 *   \brief Update physical to logical mapping buffer.
 *
 *  Updates the phyical to logical bit mapping buffer with entry specified in
 *  phyical block and logical block. The API also updates the WLA buffer
 *  accordingly.
 *
 *  \param  DDC [IN]    DDC Structure
 *   \param logBlock    [IN]    logical block number
 *   \param phyBlock    [IN]    physical block number
 *
 *   \return DDC_NAND_INVALID_PHYBLOCK  - Invalid Physical block         \n
 *           PAL_Error                  - in case of error               \n
 *            PAL_SOK     - Return Success
 */
PAL_Result nandUtlUpdateEntryTable (
                        DDC_NandObj         *DDC,
                        Uint16              logBlock,
                        Uint16              phyBlock);


/**
 *   \brief Find physical block from Mapping table.
 *
 *  This function gets the Physical block number corresponding to the logical
 *  block
 *  from the Bit Mapping Table
 *
 *   \param pNandState  [IN]    Nand media state variable
 *   \param logBlock    [IN]    logical block number
 *
 *   \return DDC_NAND_INVALID_PHYBLOCK  - Invalid physical block         \n
 *           Physical Block Number      - on success
 */
Uint16 nandUtlFindPhyBlock (
                        DDC_NandState       *pNandState,
                        Uint16              logBlock);

/**
 *   \brief Insert entry into the mapping buffer.
 *
 *  This function Inserts one entry to the scalable buffer and updates
 *  the bit mapping table
 *
 *   \param pNandState  [IN]    Nand media state variable
 *   \param logBlock    [IN]    logical block number
 *   \param phyBlock    [IN]    physical block number
 *
 *   \return DDC_NAND_E_OUT_OF_FREE_BLOCKS  - No free blocks available   \n
 *           PAL_SOK                        - Return Success
 */
PAL_Result nandUtlInsertEntry (
                        DDC_NandState       *pNandState,
                        Uint16              logBlock,
                        Uint16              phyBlock);

/**
 *   \brief Find the physical block for the logical block
 *
 *  Finds the Physical Block for the corresponding logical block from the
 *  bit mapping buffer. If the entry is not found in the buffer, the physical
 *  block is found by reading the redundant area of the NAND. If no physical
 *  block is found, an invalid block is returned
 *
 *   \param DDC         [IN]    DDC Structure
 *   \param logBlock    [IN]    logical block number
 *
 *   \return DDC_NAND_INVALID_PHYBLOCK  - incase of error.               \n
 *           Physical block address     - on success                     \n
 */
Uint16 nandUtlFindPhysicalBlock (
                        DDC_NandObj         *DDC,
                        Uint32              logBlock);

/**
 *   \brief Updates the Mapping bit buffer
 *
 *  This function updates the bit mapping buffer when a particular entry is
 *  either mapped out or mapped in.
 *
 *   \param pNandState  [IN]    Nand media state variable
 *   \param logBlock    [IN]    logical block number,
 *   \param entry       [IN]    in/out
 *
 *   \return PAL_SOK         - Return Success
 */
PAL_Result nandUtlUpdateMappingBitBuffer (
                        DDC_NandState       *pNandState,
                        Uint16              logBlock,
                        DDC_NANDLogBlockStatus entry);

/**
 *   \brief Mark the physical block as bad.
 *
 *  This API marks a physical block as bad. It starts by updating the entry
 *  table corresponding to the logical block. It procees to find a free
 *  physical block and copies all the contents from the old block to the new
 *  block, monitoring if the new physical block for a failure. In such an
 *  extreme case, it loops till all the data is succesfully written to a new
 *  good block. On a succesful operation the old block is erased and mapped
 *  out. The BBM table and bit mapping tabes are updated as required.
 *
 *   \param DDC         [IN]    DDC Structure
 *   \param logBlock    [IN]    logical block number,
 *   \param phyBlock    [IN]    physical block number,
 *   \param result      [IN]    nand error status
 *
 *   \return PAL_Error        - in case of error                         \n
 *           PAL_SOK          - Return Success                           \n
 */
PAL_Result nandUtlMarkBadBlock (
                        DDC_NandObj         *DDC,
                        Uint16              logBlock,
                        Uint16              phyBlock,
                        PAL_Result          result);

/**
 *   \brief Packs and Unpacks the entries in the entry table
 *
 *   \param pNandState  [IN]    Nand media state variable
 *   \param logBlock    [IN]    logical block number
 *   \param phyBlock    [IN]    physical block number
 *
 *   \return Entry
 */
Uint16 nandUtlPackUnpackEntries (
                        DDC_NandState       *pNandState,
                        Uint16              logBlock,
                        Uint16              phyBlock);

/**
 *   \brief Updates the Free/Used block WLA table
 *
 *   \param pNandState  [IN]    Nand media state variable
 *   \param phyBlock    [IN]    physical block number,
 *   \param bState      [IN]    state of the block (free / used)
 *
 *   \return PAL_SOK      - Return Success
 */
PAL_Result nandUtlUpdateWLATable (
                        DDC_NandState       *pNandState,
                        Uint16              phyBlock,
                        DDC_NandBlockState  bState);

/**
 *   \brief Calculate block address
 *
 *  Converts the 'block' to the format specified by SSFDC standards.
 *
 *   \param block       [IN]    non-SSFDC format block number
 *
 *   \return SSFDC format block number
 */
Uint16 nandUtlCalcBlkAdrs (
                        Uint16              block);
/*@} DDCNandUTLFunctions */

/**
 * \defgroup DDCNandCacheFunctions NAND DDC Cache Functions
 *
 * NAND Cache utility Interfaces. Cache utility just provides an
 * extra buffer(s) to buffer the NAND data/redundant information.
 * This should not be confused with the 'default' cache fundas.
 *
 * @{
 */

/**
 *   \brief Initialize the NAND Cache (implemented through software)
 *
 *  This function initializes the data structures needed for implementing
 *  the cache utilities for NAND driver. This is completely implemented in
 *  software and should not be confused (has no relationship) with the
 *  hardware cache support.
 *
 *   \param DDC         [IN]    DDC Structure
 *
 *  \return PAL_SOK
 */
PAL_Result nandCacheInit (
                        const DDC_NandObj         *DDC);

/**
 *   \brief Cache a NAND block.
 *
 *  This function adds a block's redundant info (physical sector info) to the
 *  cache, by reading from the redundant area of the NAND device.
 *
 *  \param  DDC [IN]    DDC structure
 *   \param phyBlock    [IN]    Address of the physical block on NAND device.
 *   \param cacheIndex  [OUT]   Index where the block is cached.
 *
 *   \return DDC_NAND_E_STATUS_READ_FAIL   - No replaceable cached block \n
 *           PAL_SOK                       - Return Success
 */
PAL_Result nandCacheCacheBlk    (
                        DDC_NandObj         *DDC,
                        Uint16              phyBlock,
                        Uint8               *cacheIndex);

/**
 *   \brief Get page status
 *
 *  This function gets the status of a page. If the page is not cached, it
 *  would be cahched and the index is returned. If the page is not written,
 *  a false is returned for the 'pageFound' status in the cachePageRedInfo.
 *
 *  \param  DDC [IN]    DDC structure
 *   \param phySector           [IN]    Address of physical sector on NAND.
 *   \param cachedPageRedInfo   [OUT]   Cached page redundant info
 *
 *   \return PAL_Error      - on error                                   \n
 *           PAL_SOK        - Return Success
 */
PAL_Result nandCacheGetPageStatus (
                        DDC_NandObj         *DDC,
                        Uint32              phySector,
                        DDC_NandCachePageRedInfo*  cachedPageRedInfo);

/**
 *   \brief Get the blocks redundant info.
 *
 *  This function reads the cached block's redundant information to an array
 *  as pointed by the input pointer (API Not used for now, may be buggy!)
 *
 *  \param  DDC [IN]    DDC structure
 *   \param phyBlock    [IN]    Address of the physical block on NAND device.
 *   \param pRedBuffer  [IN]    pointer to the Redundant page info
 *
 *   \return PAL_Error  - On error                                       \n
 *             PAL_SOK               - Return Sucess
 */
PAL_Result nandCacheGetBlkRed   (
                        DDC_NandObj         *DDC,
                        Uint16              phyBlock,
                        Uint32*             pRedBuffer);

/**
 *   \brief Update Cache Page entry
 *
 *  This function updates the redundant info maintained in the cache- say.
 *  after a page write.
 *
 *   \param DDC         [IN]    DDC Structure
 *   \param phySector   [IN]    Address of the physical sector on NAND device
 *
 *   \return PAL_Error  - On error                                       \n
 *             PAL_SOK               - Return Sucess
 */
PAL_Result nandCacheUpdatePageEntry (
                        DDC_NandObj         *DDC,
                        Uint32              phySector);

/**
 *   \brief Erase a block from Cache.
 *
 *  This function resets the redundant info in the cache for a block.
 *
 *   \param DDC         [IN]    DDC Structure
 *   \param phyBlock    [IN]    Address of the physical block on NAND device.
 *
 *   \return PAL_Error  - On error                                       \n
 *             PAL_SOK               - Return Sucess
 */
PAL_Result nandCacheBlkErase    (
                        const DDC_NandObj         *DDC,
                        Uint16              phyBlock);
/*@} DDCNandCacheFunctions */

/**
 * \defgroup DDCNandPHYFunctions NAND DDC Physical layer Functions
 *
 * NAND Physical layer Interfaces. These interfaces directly talk
 * to the NAND hardware.
 *
 * @{
 */


/**
 *   \brief Reads 'N' bytes from NAND using DMA.
 *
 *  This function reads 'readRequest' number of bytes to the buffer supplied
 *  using DMA.
 *
 *  \param nandObj      [IN]    llc structure
 *  \param readRequest  [IN]    number of bytes to be read
 *   \param pReadBuffer [IN]    address of the buffer, where the read data
 *                              has to  be stored
 *  \param pack         [IN]    if byte packing is to be done or not
 *
 *  \return None
 */
void nandPhyDMAReadNBytes (
                        LLC_NandObj         *nandObj,
                        Uint16              readRequest,
                        Uint16              *pReadBuffer,
                        Uint16              pack);

/**
 *   \brief Writes 'N' bytes to NAND using DMA.
 *
 *  This function writes 'writeRequest' number of bytes to NAND from the
 *  buffer specified using DMA.
 *
 *  \param nandObj      [IN]    LLC structure
 *   \param writeRequest    [IN]    number of bytes to be written to NAND device
 *   \param pWriteBuff      [IN]    Addr of buffer containing data to be written
 *   \param val             [IN]    If a value, (instead of from a buffer) needs
 *                                  to be written
 *
 *  \return None
 */
void nandPhyDMAWriteNBytes (
                        LLC_NandObj         *nandObj,
                        Uint16              writeRequest,
                        Uint16              *pWriteBuff,
                        Uint16              val);


/**
 *   \brief Erase a block of NAND
 *
 *  This function erases one block of data on NAND device and updates the
 *  cache buffer.
 *
 *   \param DDC            [IN]     DDC structure
 *   \param phyBlock       [IN]     Address of the physical block on NAND
 *                                  device to be erased.
 *
 *   \return DDC_NAND_E_MEDIA_WRITE_PROTECTED   - Write Protect error    \n
 *             DDC_NAND_E_BLOCK_ERASE_FAIL      - Page Erase error       \n
 *                  PAL_SOK        - Return Sucess
 */
PAL_Result nandPhyEraseBlock (
                        DDC_NandObj         *DDC,
                        Uint32              phyBlock);

/**
 *   \brief This function calculates the ECC Error
 *
 *   \param ecc0        [IN]    ECC 1
 *   \param ecc1        [IN]    ECC 2
 *
 *   \return Ones Complement
 */
Uint16 nandPhyEccErrorTest (
                        Uint8               *ecc0,
                        Uint8               *ecc1);

/**
 *  \brief This function is used to calculate the failed bits in ECC
 *
 *  \param ecc          [IN]    ECC Code
 *
 *  \return DDC_NandBitFailCodeStruct
 */
DDC_NandBitFailCodeStruct nandPhyGetBitFailCodes (
                        Uint8               *ecc);


/**
 *   \brief Write to a sector on NAND
 *
 *  This function ensures that data is written sequentially to the specified
 *  sector on NAND. Before writing to the specified sector, it checks the
 *  cached area to find if the page exists.
 *
 *  \param DDC          [IN]  Handle to DDC
 *   \param phySector    [IN]    Address of the Sector at which the data has
 *                               to be written
 *   \param pWriteBuff   [IN]    Address of the buffer to write data from
 *   \param pNandRed     [IN]    Pointer to the structure where the NAND
 *                               redundant information is stored
 *   \param byteSwap     [IN]    Swap bytes?
 *
 *   \return PAL_SOK                        - Return Sucess              \n
 *           DDC_NAND_E_PAGE_PROGRAM_FAIL   - Page program error         \n
 */
PAL_Result nandPhyWriteSector (
                        DDC_NandObj         *DDC,
                        Uint32              phySector,
                        Uint16              *pWriteBuffer,
                        DDC_NandRedundant   *pNandRed,
                        Uint8               byteSwap);

/**
 *   \brief Read a sector from NAND
 *
 *  This function ensures reading a sector sequentially. Before reading from
 *  the specified sector, it checks the cached area to find if the page
 *  exists, there by saving some cycle time.
 *
 *   \param DDC         [IN]    Nand media state variable
 *   \param phySector   [IN]    Address of the Sector to be read
 *   \param pReadBuffer [OUT]   Address of the buffer
 *   \param pNandRed    [OUT]   Pointer to the structure where the NAND
 *                              redundant information is to be stored
 *
 *   \return PAL_Error  - On error                                       \n
 *           PAL_SOK    - Return Success
 */
PAL_Result nandPhyReadSector (
                        DDC_NandObj         *DDC,
                        Uint32              phySector,
                        Uint16              *pReadBuffer,
                        DDC_NandRedundant   *pNandRed);

/**
 *   \brief Writes 'N' bytes to NAND.
 *
 *  This function writes 'writeRequest' number of bytes to NAND from the
 *  buffer specified.
 *
 *   \param nandObj     [IN]    LLC structure
 *   \param writeRequest    [IN]    number of bytes to be written to NAND device
 *   \param pWriteBuff      [IN]    Addr of buffer containing data to be written
 *   \param val             [IN]    If a value, (instead of from a buffer) needs
 *                                  to be written.
 *
 *   \return None
 */
void nandPhyWriteNBytes (
            LLC_NandObj    *nandObj,
            Uint16         writeRequest,
            volatile Uint16         *pWriteBuff,
            Uint16         val,
            Bool           spare);

/**
 *   \brief Read status of NAND operation
 *
 *  This function reads the status of the preceding operation on NAND Device.
 *
 *   \param DDC            [IN]     DDC structure
 *
 *   \return DDC_NAND_E_STATUS_READ_FAIL        - Read operation failed  \n
 *           DDC_NAND_E_MEDIA_WRITE_PROTECTED   - Write protect error    \n
 *           PAL_SOK              - Return success
 */
PAL_Result nandPhyReadStatus (
            DDC_NandObj *DDC);

/**
 *   \brief Issue a command to NAND
 *
 *  This function sets the command to read/write from NAND device.
 *
 *   \param DDC        [IN]     Handle to DDC
 *   \param cmd        [IN]     command to be set
 *   \param startAddr  [IN]     row address to be written
 *
 *   \return None
 */
void nandPhyIssueCommand (  DDC_NandObj *DDC,
                            Uint16 cmd,
                            Uint32 startAddr);
/**
 *   \brief Read redundant info from NAND
 *
 *  This function reads the redundant area of a physical sector. Before
 *  reading the redundant area, the cached area is searched to find if the
 *  redundant info for this physical sector is already cached and if so,
 *  reads from the cache. Else, it reads from the device.
 *
 *   \param DDC         [IN]    DDC object
 *   \param phySector   [IN]    Physical Sector number of NAND
 *   \param pNandRed    [OUT]   Address of the buffer
 *
 *   \return PAL_Result
 *                  DDC_NAND_E_READ_FAIL    - Read failed                \n
 *                  PAL_SOK                 - Return Sucess
 */
PAL_Result nandPhyReadRed (
                        DDC_NandObj         *DDC,
                        Uint32              phySector,
                        DDC_NandRedundant   *pNandRed);

/**
 *   \brief Read the redundant info from NAND.
 *
 *  This function reads redundant area of the requested physical sector
 *  physically from the NAND device and stores into the provided buffer.
 *  This is the physical interface to the NAND device.
 *
 *   \param DDC         [IN]  Handle to DDC
 *   \param phySector   [IN]  Physical Sector number of NAND
 *   \param pRedBuffer  [OUT] Address of the buffer
 *
 *   \return PAL_Result
 *                  DDC_NAND_E_PAGE_READ_FAIL   - Page read failed      \n
 *                  PAL_SOK                     - Return Sucess
 */
PAL_Result nandPhyReadRedPage (
                        DDC_NandObj         *DDC,
                        Uint32              phySector,
                        DDC_NandRedundant   *pBuff);

/*@} DDCNandPHYFunctions */
/*@} DDCNandFunctions */
/*@} DDCNandMain */
#ifdef __cplusplus
}
#endif

#endif   /* _DDC_NAND_H_ */
