/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 * \file    ddc_nandFtl.c
 *
 * \brief  Flash Translation Layer interface for NAND on DM6437
 *
 * This file is the Flash Translation Layer for the Nand flash driver. This
 * file contains the interfaces for
 *          - initializing the NAND,
 *          - reading a page,
 *          - writing to a page,
 *          - flushing a block,
 *          - formatting and
 *          - writing a sector of information.
 *
 * (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \note       Set tabstop to 4 (:se ts=4) while viewing this file in an
 *              editor
 *
 *  \author     AK Mistral, Himamsu Mistral
 *
 *  \version    1.0 AK Mistral      Created.
 *  \version    1.1 Himamsu Mistral Code cleanup and release for BFT5
 *  \version    1.2 Himamsu Mistral Release for BFT6
 *  \version    1.3 Himamsu Mistral Code cleanup and release for BFT7
 *  \version    1.4 Himamsu Mistral Added Multiple NAND support,
 *                                  write protect support
 *                                  - Release for BFT8
 */

#include <ti/sdo/pspdrivers/drivers/nand/src/ddc_nand.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osmem.h>

extern DDC_NandBankSectorInfo gNandBankSectorInfo [DDC_NAND_MAX_BANKS];
/**< Object that holds max and min sector numbers for a bank    */

/*
 * Prototypes of Static Functions
 */

/**
 *  \brief Initializes the buffers of NAND.
 *
 *  This function initializes the BBM Bit buffer, Mapping bit buffer
 *  and the physical to logical addressing mapping buffers of NAND.
 *
 *  \param  DDC [IN]    DDC Object
 *
 *  \return None
 */
static void nandFtlNandBufInit (
                        DDC_NandObj         *DDC);

/**
 *  \brief Writes a page of NAND.
 *
 *   This function maps the logical address of buffer provided by the
 *   application to the physical address, performs Bad Block Management and
 *   wear leveling and then writes the sector data into the NAND device.
 *
 *  \param  logSector   [IN]    Logical sector from where data has to be read
 *  \param  DDC         [IN]    DDC Object
 *  \param  pWriteBuff  [IN]    Buffer containing data to be written
 *  \param  byteSwap    [IN]    If set, byte will be swapped and written else
 *                              without swapping, byte will be written.
 *
 *  \return DDC_NAND_E_TIMEOUT              - Operation timed out        \n
 *          DDC_NAND_E_INVALID_LSN          - Invalid Logical Block Addr \n
 *          DDC_NAND_E_OUT_OF_FREE_BLOCKS   - No free blocks available   \n
 *          PAL_SOK             - Success
 */
PAL_Result nandFtlWritePage (
                        Uint32              logSector,
                        DDC_NandObj         *DDC,
                        Uint16              *pWriteBuff,
                        Uint8               byteSwap);

/**
 *  \brief Updates NAND tables.
 *
 *  This function initializes the BBM Table, entry table and WLA Table of
 *  NAND. It also identifies any bad blocks and updates the BBM table
 *  acordingly.
 *
 *  \param  DDC [IN]    DDC Object
 *
 *  \return PAL_SOK or PAL Error code.
 */

static PAL_Result nandFtlUpdateNandTables (
                        DDC_NandObj         *DDC);


/**
 *  \brief Initializes the FTL of NAND.
 *
 *   This function initializes the NAND device and creates BBM,WLA and
 *   physical to logical mapping table.
 *
 *  \param  DDC [IN]    DDC Object
 *  \return PAL_SOK or PAL Error code
 */
PAL_Result nandFtlInit (
            DDC_NandObj        *DDC)
{
    DDC_NandState   *pNandState;
    Uint32          count           = 0;

#ifdef DDC_NAND_POWER_SAFE
    Uint16      tempVar = 0;    /* Temporary variable */
#endif /* DDC_NAND_POWER_SAFE */

    PAL_Result      result  = PAL_SOK;

    if( NULL != DDC) {
        pNandState     = (DDC_NandState *) &DDC->nandState[DDC->bankNo];
        if(NULL != pNandState) {
            /* If wear level algorithm is used... */
            if (TRUE == pNandState->WLAflag) {
                if (pNandState->entryBufferSize <= 2u) {
                    /* one of the Phy2Log mapping scheme */
                    pNandState->pPhyLogEntries = NULL;
                }
                /* other of the Phy2Log mapping scheme */
                if ((pNandState->entryBufferSize >=
                            (128u * pNandState->numZones )) &&
                        (pNandState->entryBufferSize <
                         (625u * pNandState->numZones))) {
                    pNandState->pMappingbitbuffer = pNandState->pPhyLogEntries;
                    pNandState->pPhyLogEntries += (64u * pNandState->numZones);
                } /* one more of the Phy2Log mapping scheme */
                else {
                    if (pNandState->entryBufferSize <
                            (128u * pNandState->numZones)) {
                        pNandState->entryBufferSize  &= (Uint16)~0x1;
                        pNandState->pMappingbitbuffer = (Uint16*)NULL;
                    }
                }

                /* Assume all blocks are free first */
                PAL_osMemSet (pNandState->pWLAbitbuffer, (char)DDC_NAND_DEF_INIT_VALUE,
                        DDC_NAND_DEF_WLA_BUF_SIZE);
                /* Based on the number of blocks protected, adjust the WLA Table */
                if(pNandState->protectedBlks != 0)
                {
                    for(count = 0; count < (pNandState->protectedBlks/16); count ++)
                    {
                        pNandState->pWLAbitbuffer[count] = 0;
                        pNandState->pBBMbitbuffer[count] = 0;
                    }
                    pNandState->pWLAbitbuffer[count] &= (DDC_NAND_DEF_INIT_VALUE >> (pNandState->protectedBlks%16));
                    pNandState->pBBMbitbuffer[count] &= (DDC_NAND_DEF_INIT_VALUE >> (pNandState->protectedBlks%16)) ;
                }

                /* Initialize the current write page */
                pNandState->currentPhyWritePage = pNandState->protectedBlks * pNandState->pagesPerBlock;

                /* Initializing the buffers... */
                nandCacheInit(DDC);
                nandFtlNandBufInit(DDC);
                result = nandFtlUpdateNandTables(DDC);
            }

#ifdef DDC_NAND_POWER_SAFE
            pNandState->skipWrite = 0;
            pNandState->pBBMbitbuffer[tempVar] &= (Uint16)0x7FFFu;
#endif /* DDC_NAND_POWER_SAFE */
        }
    }
    else {
        result = PSP_E_INVAL_PARAM;
    }

    return result;
}

/**
 *  \brief Initializes the buffers of NAND.
 *
 *   This function initializes the BBM Bit buffer, Mapping bit buffer
 *   and the physical to logical addressing mapping buffers of NAND.
 *
 *  \param  DDC [IN] DDC Object
 *
 *  \return None
 */
static void nandFtlNandBufInit (
            DDC_NandObj *DDC)
{
    DDC_NandState   *pNandState;

    if( NULL != DDC) {
        pNandState     = (DDC_NandState *) &DDC->nandState[DDC->bankNo];
        if(NULL != pNandState) {
#ifdef DDC_NAND_BBM
        /* Assume all blocks are good first */
            PAL_osMemSet (pNandState->pBBMbitbuffer, (char)DDC_NAND_DEF_INIT_VALUE,
                    (64u * pNandState->numZones));
#endif /* DDC_NAND_BBM */
            pNandState->noOfBadBlocks = 0;
            pNandState->noOfBadPages  = 0;

            /* Set zero entries first */
            if (NULL != pNandState->pMappingbitbuffer) {
                PAL_osMemSet (pNandState->pMappingbitbuffer, 0,
                        (64u * pNandState->numZones));
            }

            /* If phy2log is defined... */
            if (NULL != pNandState->pPhyLogEntries) {
                if (NULL != pNandState->pMappingbitbuffer) {
                    PAL_osMemSet (pNandState->pPhyLogEntries,
                            (char)DDC_NAND_DEF_INIT_VALUE,
                            (pNandState->entryBufferSize-
                             (64u * pNandState->numZones)));
                }
                else {
                    /* Clear the physical to logical mapping */
                    PAL_osMemSet (pNandState->pPhyLogEntries,
                                  (char)DDC_NAND_DEF_INIT_VALUE,
                                  pNandState->entryBufferSize);
                }
            }
        }
    }
}
/**
 *  \brief Updates NAND tables.
 *
 *  This function initializes the BBM Table, entry table and WLA Table of
 *  NAND. It also identifies any bad blocks and updates the BBM table
 *  acordingly.
 *
 *  \param  DDC [IN] DDC Object
 *
 *  \return PAL_SOK or PAL Error code.
 */
static PAL_Result nandFtlUpdateNandTables (
            DDC_NandObj *DDC)
{
    Uint32          numBlocks       = 0;
    Uint32          nandCurBlock    = 0;
    Uint16          logBlock        = 0;
    Uint16          phyBlock        = 0;
    Uint32          newLogBlock     = 0;
    Uint16          countVar        = 0;    /* Counter variable */
    Uint16          entryBuffCount  = 0;    /* Counter variable for entry
                                               buf size */
    Uint16          priority        = 7u;   /* Priority variable*/

    DDC_NandState   *pNandState;
    Uint16          parityVar;  /* Parity */
    Uint32          phySector;
    PAL_Result      result  = PAL_SOK;
    DDC_NandRedundant   nandRed;

    if( NULL != DDC) {
        pNandState     = (DDC_NandState *) &DDC->nandState[DDC->bankNo];
        if( NULL != pNandState) {
            /* Update Physical to logical entry table and WLA table */
            numBlocks = (Uint32)(DDC_NAND_BLOCKS_PER_ZONE * pNandState->numZones);
#ifdef DDC_NAND_POWER_SAFE
            for (nandCurBlock = pNandState->protectedBlks;
             nandCurBlock < numBlocks;
             nandCurBlock++) {
#else
        for (nandCurBlock = numBlocks;
             nandCurBlock > pNandState->protectedBlks;) {
                    nandCurBlock--;
#endif /* DDC_NAND_POWER_SAFE */
                    /* Initalize the redundant area structure. */
                    PAL_osMemSet ((void*)&nandRed, (char) DDC_NAND_DEF_INIT_VALUE,
                            sizeof(DDC_NandRedundant));

                    /* Starting from page 0, read the redundant info. */
                    phySector = ((Uint32)nandCurBlock *
                            (Uint32)(pNandState->pagesPerBlock));
                    result = nandPhyReadRedPage(DDC, phySector, &nandRed);
                    if (PAL_SOK != result) {
                        return result;
                    }
                    /* Update the BBM Table accordingly. */
#ifdef DDC_NAND_BBM
                    if ((DDC_NAND_SMALL_BLOCK == pNandState->nandType) &&
                            ((nandRed.blockStatus !=
                              DDC_NAND_BLOCK_STATUS_FREE))) {
                        /* Small block */
                        /*  Changed for Bad block management bug  */ 
                        result = nandUtlUpdateBBMTable (DDC,
                                                        nandCurBlock,
                                                        DDC_NAND_BAD_BLOCK,
                                                        &nandRed);
                        /*  Changed for Bad block management bug  - Up to here  */

                        if (PAL_SOK != result) {
                            return result;
                        }
#ifdef PSP_NAND_DEBUG
                        NAND_Debug1 ("\r\nBlock %d is bad!\n", nandCurBlock);
#endif /* PSP_NAND_DEBUG */
                    }

                    else if ((DDC_NAND_BIG_BLOCK == pNandState->nandType) &&
                            ((Uint16)(nandRed.logPageNo >> 8) != 0xFF)) {
                        /* Big block Nand */
                        /*  Changed for Bad block management bug  */ 
                        result = nandUtlUpdateBBMTable (DDC,
                                                        nandCurBlock,
                                                        DDC_NAND_BAD_BLOCK,
                                                        &nandRed);
                        /*  Changed for Bad block management bug  - Up to here  */

                        if (PAL_SOK != result) {
                            return result;
                        }
#ifdef PSP_NAND_DEBUG
                        NAND_Debug1 ("\r\nBlock %d is bad!\n", nandCurBlock);
#endif /* PSP_NAND_DEBUG */
                    }
                    else {
#endif /* DDC_NAND_BBM */

                    /* Used block */
                    if (nandRed.blockAddress != DDC_NAND_BLOCK_ADDR_FREE) {
                        parityVar = 0;

                        for (countVar = 0; countVar < 16u; countVar++) {
                            parityVar ^= ((nandRed.blockAddress &
                                        (1u << countVar)) >> countVar);
                        }

#ifdef DDC_NAND_BBM
                        if (((nandRed.blockStatus & 0xFFu) != 0xFFu) ||
                                (((nandRed.blockAddress & 0xF800u) != 0x1000u)  ||
                                 (0 != parityVar))) {
                            /* Bad Block found.. update the BBM table. */
                            /*  Changed for Bad block management bug  */ 
                            result = nandUtlUpdateBBMTable (DDC,
                                                            nandCurBlock,
                                                            DDC_NAND_BAD_BLOCK,
                                                            &nandRed);
                            /*  Changed for Bad block management bug  - Up to here  */

                            if (PAL_SOK != result) {
                                return result;
                            }
                        }
                        else {
#endif /* DDC_NAND_BBM */

                        /* Extract Logical Block address.
                         * As of now we don't check the parity bit
                         */
                        logBlock =
                            (Uint16)((nandRed.blockAddress >> 1) & 0x3FF);
                        if (logBlock < DDC_NAND_USABLE_BLOCKS_PER_ZONE) {
                            /* Valid Logical Block */
                            if (NULL != pNandState->pPhyLogEntries) {
                                if (pNandState->entryBufferSize >=
                                        (625u * pNandState->numZones)) {
                                    /* Adjust the block number, since there are
                                     * 1000 usable blocks out of 1024. */
                                    newLogBlock = logBlock +
                                        ((nandCurBlock /
                                          DDC_NAND_BLOCKS_PER_ZONE) *
                                         DDC_NAND_USABLE_BLOCKS_PER_ZONE);

                                    result = nandUtlUpdateEntryTable (
                                            DDC, (Uint16)newLogBlock,
                                            (Uint16)(nandCurBlock & 0x3FFu));
                                    if (PAL_SOK != result) {
                                        return result;
                                    }
                                }
                                else if ((pNandState->entryBufferSize <
                                            (128u * pNandState->numZones))
                                        &&
                                        (pNandState->entryBufferSize >= 2u)) {
                                    if (entryBuffCount <
                                            pNandState->entryBufferSize) {
                                        /* Adjust the block number,
                                         * since there are
                                         * 1000 usable blocks out of 1024. */
                                        newLogBlock = logBlock +
                                            ((nandCurBlock /
                                              DDC_NAND_BLOCKS_PER_ZONE) *
                                             DDC_NAND_USABLE_BLOCKS_PER_ZONE);

                                        pNandState->pPhyLogEntries
                                            [entryBuffCount++] =
                                            (Uint16)newLogBlock;

                                        pNandState->pPhyLogEntries[
                                            entryBuffCount++] =
                                            (Uint16)nandCurBlock;
                                    }
                                }

                                else {
                                    if (pNandState->entryBufferSize >=
                                            (128u * pNandState->numZones)) {
                                        phyBlock =
                                            (Uint16)(nandCurBlock |
                                                     (priority<<13u));
                                        /* Adjust the block number,
                                         * since there are
                                         * 1000 usable blocks out of 1024. */
                                        newLogBlock = logBlock +
                                            ((nandCurBlock /
                                              DDC_NAND_BLOCKS_PER_ZONE ) *
                                             DDC_NAND_USABLE_BLOCKS_PER_ZONE);
                                        result = nandUtlInsertEntry(
                                                pNandState,
                                                (Uint16)newLogBlock,
                                                phyBlock);
                                        if (PAL_SOK != result) {
                                            return result;
                                        }
                                    }
                                }
                            }
                            result = nandUtlUpdateWLATable (
                                    pNandState,
                                    (Uint16)nandCurBlock,
                                    DDC_NAND_USED_BLOCK);
                            if (PAL_SOK != result) {
                                return result;
                            }
                        }
#ifdef DDC_NAND_BBM
                        }
#endif
                    }
#ifdef DDC_NAND_BBM
                    }
#endif
                }
                if (pNandState->noOfBadBlocks > 0) {
#ifdef PSP_NAND_DEBUG
                    NAND_Debug4("\r\nNAND: %d out of %d(%d %)block(s) are bad! %s\n\r",
                            pNandState->noOfBadBlocks,pNandState->noOfBlocks,
                            ((pNandState->noOfBadBlocks*100u)
                             /pNandState->noOfBlocks),
                            (((pNandState->noOfBadBlocks*100u)
                              /pNandState->noOfBlocks) > 50u) ?
                            "\n\r#  ##### NAND Media un-usable. Please replace.#####":"\n\r#  ##### Consider changing the NAND media.#####");
#endif
                }

                /* Populate the global sector info structure. */
                if (DDC->bankNo == 0) {
                    gNandBankSectorInfo [DDC->bankNo].minSectorNo = 0;
                    gNandBankSectorInfo [DDC->bankNo].maxSectorNo =
                        pNandState->totalPages - 1u;
                }
                else {
                    gNandBankSectorInfo [DDC->bankNo].minSectorNo =
                        gNandBankSectorInfo [DDC->bankNo-1].maxSectorNo + 1u;
                    /* Adjust, if there are no NAND devices in previous banks.*/
                    if (gNandBankSectorInfo [DDC->bankNo].minSectorNo == 1u) {
                        gNandBankSectorInfo [DDC->bankNo].minSectorNo = 0;
                    }
                    gNandBankSectorInfo [DDC->bankNo].maxSectorNo =
                        (gNandBankSectorInfo [DDC->bankNo].minSectorNo +
                         pNandState->totalPages) - 1u;
                }
            }
        }
        else {
            result = PSP_E_INVAL_PARAM;
        }

        return result;
}

/**
 *  \brief Reads a page of NAND.
 *
 *  This function reads one page of data from NAND device to the application
 *  supplied buffer. For the given logical sector the logical block, then the
 *  physical block, and hence the physical sector is identified and proceeds
 *  to read from that physical sector.
 *
 *  \param  logSector   [IN]    Logical sector from where data has to be read
 *  \param  DDC         [IN]    DDC Object
 *  \param  pReadBuff   [IN]    Buffer to which the data has to be copied
 *
 *  \return DDC_NAND_E_TIMEOUT              - Operation timed out        \n
 *          DDC_NAND_E_INVALID_LSN          - Invalid Logical Block Addr \n
 *          DDC_NAND_E_INVALID_PBA          - Invalid Physical block     \n
 *          DDC_NAND_E_STATUS_READ_FAIL     - Read Failed                \n
 *          DDC_NAND_E_OUT_OF_FREE_BLOCKS   - No free blocks available   \n
 *          PAL_SOK                         - Success
 */
PAL_Result nandFtlReadPage(
            Uint32      logSector,
            DDC_NandObj *DDC,
            Uint16      *pReadBuff)
{
    DDC_NandState       *pNandState;
    PAL_Result          result     = PAL_SOK;
    Uint32              mask;
    DDC_NandBlockState  blockStatus;
    Uint16              logBlock;
    Uint16              phyBlock;
    Uint16              sector;
    Uint32              currentSector;
    Uint32              logPage;
    Uint32              phySector;

    DDC_NandRedundant   nandRed;
    /*  Changed for Bad block management bug  */ 
    Uint8   count;
    Uint32  usableblocks;
    /* Changed for Bad block management bug  - Up to here */

    if( (NULL != DDC) && (NULL != pReadBuff)) {
        pNandState= (DDC_NandState *) &DDC->nandState[DDC->bankNo];
        if(NULL != pNandState) {
            mask       = (Uint32)((Uint32)pNandState->pagesPerBlock- 1u);
            /*  Changed for Bad block management bug  */ 
            logBlock = (logSector / pNandState->pagesPerBlock);
            logPage = logSector;
            sector   = logPage & mask;
            
            if (logPage > (pNandState->totalPages)) {
                   return (DDC_NAND_E_INVALID_LSN);
            }
            
            /* Check for usable number of blocks in zone and move to next zone if block is not available */ 
             count = 0;
            usableblocks = pNandState->usableBlockInZone[count];
            while((logBlock >= usableblocks) && (count < pNandState->numZones))
            {
                logBlock += (DDC_NAND_USABLE_BLOCKS_PER_ZONE - pNandState->usableBlockInZone[count]);
                count++;
                usableblocks += pNandState->usableBlockInZone[count];           
                logSector = (logBlock * pNandState->pagesPerBlock) + sector;
                logPage = logSector;    
            }   
            if (TRUE == pNandState->WLAflag) {
                if (logPage > (pNandState->totalPages)) {
                    return (DDC_NAND_E_INVALID_LSN);
                }

                /*
                 * Find the logical block and hence the physical block for
                 * 'logSector'.
                 */
                /*  Changed for Bad block management bug  - Up to here  */

                phyBlock = nandUtlFindPhysicalBlock (DDC, logBlock);
                if (DDC_NAND_INVALID_PHYBLOCK == phyBlock) {
                    return DDC_NAND_E_INVALID_PBA;
                }

                if (logBlock == pNandState->currentLogBlock) {
                    /*
                     * We want to read the same logical block which is being
                     * written. Flush it
                     */
                    if (TRUE == pNandState->newWriteFlushPending) {
                        currentSector = (Uint32)
                            (phyBlock * pNandState->pagesPerBlock) + sector;
                        /* Check if current page is free */
                        blockStatus = nandUtlIsFreePage (DDC, currentSector);
                        if (DDC_NAND_UNKNOWN_BLOCK == blockStatus) {
                            return DDC_NAND_E_STATUS_READ_FAIL;
                        }
                        if (DDC_NAND_FREE_BLOCK == blockStatus) {
                            /*
                             * This sector has not yet written to! So we have
                             * valid data in the sector of old block
                             */
                            phyBlock = pNandState->oldPhyBlock;
                        }
                    }
                }

                if (phyBlock > (pNandState->noOfBlocks - 1u)) {
                    /* We didn't get a valid free block --
                     * Assume phyblock = logblock */
                    phyBlock = logBlock;
                }
            }
            else {
                return DDC_NAND_E_STATUS_READ_FAIL;
            }

            pNandState->logicalSecAddr  = (Uint16)logPage;
            pNandState->logicalColAddr  = (Uint16)logSector;

            phySector = (sector + ((Uint32)phyBlock *
                        (Uint32)pNandState->pagesPerBlock));
            result = nandPhyReadSector (DDC, phySector, pReadBuff, &nandRed);

#ifdef DDC_NAND_BBM
            if (DDC_NAND_E_ECC_ERROR == result) {
                /* In case of failure, mark the block as bad! */
                result = nandUtlMarkBadBlock (DDC, logBlock, phyBlock, result);
                if (PAL_SOK == result) {
                    result = DDC_NAND_E_ECC_ERROR;
                }
            }
#endif /* DDC_NAND_BBM */

#ifdef NAND_DBG_LEVEL_3
            NAND_Debug ("\r\nLogical Sector: %d, Physical Sector: %d\n\r",
                    logSector, phySector);
            NAND_Debug ("\r\nLogical Block: %d, Physical Block: %d\n\r",
                    logBlock, phyBlock);
#endif
        }
    }
    else {
        result = PSP_E_INVAL_PARAM;
    }
    return (result);
}

/**
 *  \brief Writes a page of NAND.
 *
 *   This function maps the logical address of buffer provided by the
 *   application to the physical address, performs Bad Block Management and
 *   wear leveling and then writes the sector data into the NAND device.
 *
 *  \param  logSector   [IN]    Logical sector from where data has to be read
 *  \param  DDC         [IN]    DDC Object
 *  \param  pWriteBuff  [IN]    Buffer containing data to be written
 *  \param  byteSwap    [IN]    If set, byte will be swapped and written else
 *                              without swapping, byte will be written.
 *  \return DDC_NAND_E_TIMEOUT              - Operation timed out        \n
 *          DDC_NAND_E_INVALID_LSN          - Invalid Logical Block Addr \n
 *          DDC_NAND_E_OUT_OF_FREE_BLOCKS   - No free blocks available   \n
 *           PAL_SOK                - Success
 */
PAL_Result nandFtlWritePage (
            Uint32      logSector,
            DDC_NandObj *DDC,
            Uint16      *pWriteBuff,
            Uint8       byteSwap)
{
    DDC_NandState       *pNandState;
    DDC_NandBlockState  blockStatus = DDC_NAND_FREE_BLOCK;
    PAL_Result          result      = PAL_SOK;
    DDC_NandRedundant   nandRed;

    Uint32  tempVar;
    Uint16  logBlock;
    Uint16  oldPhyBlock = DDC_NAND_DEF_OLD_PHY_BLOCK;
    Uint16  phyBlock;
    Uint16  sector;
    Uint32  mask;
    Uint32  phySector;

    /*  Changed for Bad block management bug  */
    Uint8   count;
    Uint32  usableblocks;
    /*  Changed for Bad block management bug  - Up to here  */ 

    if((NULL != DDC) && (NULL != pWriteBuff)) {
        pNandState = (DDC_NandState *) &DDC->nandState[DDC->bankNo];
        if( NULL != pNandState) {

            if (TRUE == pNandState->skipWrite) {
                return (PAL_SOK);
            }

            if (logSector > (pNandState->totalPages)) {
                return (DDC_NAND_E_INVALID_LSN);
            }

            if (TRUE == DDC_nandIsWriteProtected(pNandState->nandDevFlags)) {
                return (DDC_NAND_E_MEDIA_WRITE_PROTECTED);
            }

            mask = (Uint32)((Uint32)(pNandState->pagesPerBlock) - 1u);
            /*  Changed for Bad block management bug  */ 
            /* Compute the logical block. */
            logBlock = (logSector / pNandState->pagesPerBlock);

            /* Compute the sector in block. */
            sector   = (logSector & mask);
    
            /* Check for usable number of blocks in zone and move to next zone if block is not available */ 
            count = 0;
            usableblocks = pNandState->usableBlockInZone[count];
            while((logBlock >= usableblocks) && (count < pNandState->numZones))
            {
                    logBlock += (DDC_NAND_USABLE_BLOCKS_PER_ZONE - pNandState->usableBlockInZone[count]);
                    count++;
                    usableblocks += pNandState->usableBlockInZone[count];           
            }   
            logSector = (logBlock * pNandState->pagesPerBlock) + sector;    
            /*  Changed for Bad block management bug  - Up to here  */ 
            /* Save the logical sector info later to be used */
            pNandState->logicalSecAddr = (Uint16)logSector;
            pNandState->writeZone = (Uint16)(logSector / (pNandState->pagesPerBlock *
                                              DDC_NAND_USABLE_BLOCKS_PER_ZONE));
            pNandState->writeZone++;

            /* If the zones are different, flush the data to the
             * device first. */
            if (pNandState->writeZone != pNandState->prevWriteZone) {
                result = nandUtlWriteSectorFlush (DDC);
                if (PAL_SOK != result) {
                    return (result);
                }

                pNandState->currentPhyWritePage = (Uint32)((pNandState->writeZone-1) *
                    DDC_NAND_BLOCKS_PER_ZONE * pNandState->pagesPerBlock);

                pNandState->prevWriteZone = pNandState->writeZone;
            }

            /* Compute the logical block. */
            logBlock = (Uint16)(logSector / pNandState->pagesPerBlock);
            sector   = (Uint16)(logSector & mask);

            /* If the blocks are different, flush the data to the
             * device first.*/
            if (logBlock != pNandState->currentLogBlock) {

                if (0 != pNandState->newWriteFlushPending) {
                    result = nandUtlWriteSectorFlush (DDC);
                    if (PAL_SOK != result) {
                        return (result);
                    }
                }

                /* Find the physical block corresponding to
                 * the logical block. */
                phyBlock = nandUtlFindPhysicalBlock (DDC, logBlock);
                if (DDC_NAND_INVALID_PHYBLOCK == phyBlock) {
                    return DDC_NAND_E_INVALID_PBA;
                }
            }
            else {
                phyBlock = (Uint16)(pNandState->currentPhyWritePage/
                        pNandState->pagesPerBlock);
            }

            /* Check if the page is free first. */
            phySector = sector +
                ((Uint32)phyBlock * (Uint32)pNandState->pagesPerBlock);
            blockStatus = nandUtlIsFreePage (DDC, phySector);
            if (DDC_NAND_UNKNOWN_BLOCK == blockStatus) {
                return DDC_NAND_E_PAGE_PROGRAM_FAIL;
            }

            if (DDC_NAND_FREE_BLOCK != blockStatus) {
                /* To Keep Track which page is to be Updated */
                oldPhyBlock = phyBlock;

                if (DDC_NAND_DEF_OLD_PHY_BLOCK == pNandState->oldPhyBlock) {
                    pNandState->oldPhyBlock = oldPhyBlock;
                    pNandState->newWriteFlushPending = 1;
                }
                else {
#ifdef DDC_NAND_POWER_SAFE
                    result = nandUtlWriteSectorFlush (DDC);
                    if (PAL_SOK != result) {
                        return (result);
                    }

                    pNandState->oldPhyBlock = oldPhyBlock;
                    pNandState->newWriteFlushPending = 1;
                    pNandState->writeFlushPending    = 0;
#else
                    pNandState->writeFlushPending = 1;
#endif /* DDC_NAND_POWER_SAFE */
                }

                /* Get the next free block */
                if (PAL_SOK != nandUtlFndNxtFreeBlk (DDC, &phyBlock)) {
                    return (DDC_NAND_E_OUT_OF_FREE_BLOCKS);
                }

                /* Erase the new Block to assure correct write operation */
                result = nandPhyEraseBlock (DDC, phyBlock);
                if (PAL_SOK != result) {
#ifdef DDC_NAND_BBM
                    nandUtlUpdateBBMTable (DDC, phyBlock, DDC_NAND_BAD_BLOCK, NULL);
#endif
                        return (result);
                    }
            }

            if ((logBlock != pNandState->currentLogBlock) ||
                    (DDC_NAND_FREE_BLOCK != blockStatus)) {
                /* Update the WLA Bit Table */
                result = nandUtlUpdateWLATable (
                        pNandState,
                        phyBlock,
                        DDC_NAND_USED_BLOCK);
                if (PAL_SOK != result) {
                    return (result);
                }

                /* Update the Logical to Physical Entry Table */
                result = nandUtlUpdateEntryTable (DDC, logBlock, phyBlock);
                if (PAL_SOK != result) {
                    return (result);
                }
            }

            /* Assign the block address (in the redundant area) */
            nandRed.blockAddress  = (Uint16)(logBlock % DDC_NAND_USABLE_BLOCKS_PER_ZONE);
            nandRed.blockStatus = DDC_NAND_BLOCK_STATUS_FREE;
            nandRed.dataStatus = DDC_NAND_DATA_STATUS_FREE;

            /* First check whether first page is free */
            phySector = (Uint32)phyBlock * (Uint32)pNandState->pagesPerBlock;
            blockStatus = nandUtlIsFreePage (DDC, phySector);
            if (DDC_NAND_UNKNOWN_BLOCK == blockStatus) {
                return DDC_NAND_E_PAGE_PROGRAM_FAIL;
            }

            /* Save the logical sector info later to be used */
            pNandState->logicalSecAddr = (Uint16)logSector;

            /* Start writing to the sector. */
            phySector = sector +
                ((Uint32)phyBlock * (Uint32)pNandState->pagesPerBlock);
            result = nandPhyWriteSector (
                    DDC, phySector, pWriteBuff, &nandRed, byteSwap);
            if (DDC_NAND_E_PAGE_PROGRAM_FAIL != result) {
                if (PAL_SOK != result) {
                    return (result);
                }
            }

#ifdef DDC_NAND_BBM
            if (DDC_NAND_E_PAGE_PROGRAM_FAIL == result) {
                /* In case of failure, mark the block as bad! */
                result = nandUtlMarkBadBlock (DDC, logBlock, phyBlock, result);
                if (PAL_SOK != result) {
                    if (DDC_NAND_E_MEDIA_WRITE_PROTECTED == result) {
                        return (DDC_NAND_E_MEDIA_WRITE_PROTECTED);
                    }
                    return result;
                }

                if (pNandState->BBMdepth++ > DDC_NAND_MAX_BBM_DEPTH) {
                    pNandState->BBMdepth = 0;
                    return (DDC_NAND_E_OUT_OF_FREE_BLOCKS);
                }
                /* Recurse, only if a bad block is found! */
                return (nandFtlWritePage(logSector, DDC, pWriteBuff, byteSwap));
            }
#endif /* DDC_NAND_BBM */

#ifdef DDC_NAND_POWER_SAFE
            if (((DDC_NAND_FREE_BLOCK == blockStatus) && (0 != sector)) &&
                    (1 != pNandState->newWriteFlushPending)) {
#else
                if ((DDC_NAND_FREE_BLOCK == blockStatus) && (0 != sector)) {
#endif /* DDC_NAND_POWER_SAFE */
                    /* Its a free block. Write on its redundant area only */
                /* Reuse the input buffer */
                    for (tempVar = 0; tempVar < ((Uint16)(pNandState->bytesPerPage >> 1));
                            tempVar++) {
                        pNandState->pWriteBuffer[tempVar] = (Uint16)0xFFFFu;
                    }

                    nandRed.blockAddress = DDC_NAND_BLOCK_ADDR_FREE;
                    /* logBlock modulo DDC_NAND_USABLE_BLOCKS_PER_ZONE */
                    nandRed.blockStatus  = DDC_NAND_BLOCK_STATUS_FREE;
                    nandRed.dataStatus   = DDC_NAND_DATA_STATUS_FREE;
            }

#ifndef DDC_NAND_POWER_SAFE
            if (pNandState->writeFlushPending) {
                result = nandUtlWriteSectorFlush (DDC);
                if (PAL_SOK != result) {
                    return result;
                }
                pNandState->oldPhyBlock = oldPhyBlock;
                pNandState->newWriteFlushPending = 1;
                pNandState->writeFlushPending = 0;
            }
#endif /* DDC_NAND_POWER_SAFE */

            pNandState->currentLogBlock = logBlock;
            pNandState->currentPhyWritePage = sector + ((Uint32)phyBlock *
                    (Uint32)pNandState->pagesPerBlock);
#ifdef NAND_DBG_LEVEL_3
            NAND_Debug ("\r\nLogical Sector: %d, Physical Sector: %d\n\r",
                    logSector, phySector);
            NAND_Debug ("\r\nLogical Block: %d, Physical Block: %d\n\r",
                        logBlock, phyBlock);
#endif
            }

        }
        return (result);
}
/**
 *  \brief Erases NAND device completely. API to be used with caution!
 *
 *  This function erases data on the NAND device. From a filesystem view,
 *  ideally, the device is formatted.
 *
 *  \param  DDC         [IN]    DDC Object
 *
 *  \return Success (0) or number of blocks failed.
 */
Uint32 nandFtlErase (
            DDC_NandObj *DDC)
{
    Uint32          curBlock    = 0;
    PAL_Result      result      = PSP_E_INVAL_PARAM;
    Uint32          failedCount = 0;
    DDC_NandState   *pNandState;
    /*  Changed for Bad block management bug  */
    DDC_NandRedundant   nandRed;
    /*  Changed for Bad block management bug  - Up to here  */
    if( NULL != DDC) {
        pNandState = (DDC_NandState *) &DDC->nandState[DDC->bankNo];
        if(NULL != pNandState) {
            for (curBlock = pNandState->protectedBlks;
                 curBlock < pNandState->noOfBlocks;
                 curBlock++) {
                /*  Changed for Bad block management bug  */
                if(1 == DDC->llc->eraseAtInit)  
                {
                    result = nandPhyReadRed(DDC, (curBlock * pNandState->pagesPerBlock), &nandRed);
                    if (PAL_SOK != result) {
                        return (DDC_NAND_UNKNOWN_BLOCK);
                    }
                    if(nandRed.blockStatus != DDC_NAND_BLOCK_STATUS_FREE)
                    {            
                        continue;
                    }
                }
                /*  Changed for Bad block management bug  - Up to here  */  

                result = nandPhyEraseBlock  (DDC, curBlock);
                if (PAL_SOK != result) {
                    failedCount++;
                    /*  Changed for Bad block management bug  */ 
                    result = nandUtlUpdateBBMTable (DDC, curBlock, DDC_NAND_BAD_BLOCK, NULL);
                    /*  Changed for Bad block management bug  - Up to here  */
                    if (PAL_SOK != result) {
                        break; /* Error occurred */
                    }
                }
            }
        }
    }

    if( PAL_SOK == result) {
        result = (PAL_Result)failedCount;
    }
    return ((Uint32)result);
}

