/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 * \file    ddc_nandPhy.c
 *
 * \brief  Physical layer interface for NAND on DM6437
 *
 * This file contains functions for the NAND Generic API's to interact
 * with the NAND device interface functions.
 *
 * (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \note       Set tabstop to 4 (:se ts=4) while viewing this file in an
 *              editor
 *
 *  \author     AK Mistral, Himamsu Mistral
 *
 *  \version    1.0 AK Mistral      Created.
 *  \version    1.1 Himamsu Mistral Code cleanup and release for BFT5
 *  \version    1.2 Himamsu Mistral Release for BFT6
 *  \version    1.3 Himamsu Mistral Added DMA support for NAND
 *                                  Code cleanup and release for BFT7
 *  \version    1.4 Himamsu Mistral Added Multiple NAND support, Write
 *                                  protect support
 *                                  - Release for BFT8
 */

#include <ti/sdo/pspdrivers/drivers/nand/src/ddc_nand.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osmem.h>
#ifdef NAND_INSTRUMENTATION_ENABLED
#include <ti/sdo/pspdrivers/pal_os/bios/psp_log.h>
#endif /*NAND_INSTRUMENTATION_ENABLED*/

/* #define __DEBUG__PRINT */
#define NAND_RW_BLK_SIZE        (512)
#define NAND_MIN_DMA_REQ_SIZE   (64)

/*
 * Prototypes of Static Functions
 */
/**
 *   \brief Reads 'N' bytes from NAND.
 *
 *  This function reads 'readRequest' number of bytes to the buffer supplied.
 *
 *  \param nandObj      [IN]    llc structure
 *  \param readRequest  [IN]    number of bytes to be read
 *   \param pReadBuffer [IN]    address of the buffer, where the read data
 *                              has to  be stored
 *  \param pack         [IN]    if byte packing is to be done or not
 *
 *  \return None
 */
static void nandPhyReadNBytes (
                        LLC_NandObj         *nandObj,
                        Uint16              readRequest,
                        volatile Uint16     *pReadBuffer,
                        Uint16              pack,
                        Bool                spare);



/**
 *   \brief Set address on NAND bus
 *
 *  This function writes page addr (column and row addresses) to the NAND bus
 *
 *   \param DDC         [IN]    DDC structure
 *   \param startAddr   [IN]    page address to be written
 *   \param pageOnly    [IN]    Flag to distinguish b/w page operations and
 *                              block operations
 *   \return None
 */
static void nandPhySetAddress (
                        DDC_NandObj         *DDC,
                        Uint32              startAddr,
                        Uint16              pageOnly);

/**
 *   \brief Writes a page of data to the NAND.
 *
 *  This function writes data to the specified physical page in the NAND.
 *  This is the low level write that the driver performs to write to the
 *  physical NAND device. The API also writes the redundant area information
 *  to the spare area.
 *
 *   \param DDC         [IN]    Handle to DDC
 *   \param phySector   [IN]    Address of the Sector at which the data has to
 *                              be written
 *   \param pWriteBuff  [IN]    Address of the buffer
 *   \param pNandRed    [IN]    Pointer to the structure where the NAND
 *                              redundant information is stored
 *   \param byteSwap    [IN]    Swap bytes?
 *
 *  \return PAL_Result
 *             DDC_NAND_E_MEDIA_WRITE_PROTECTED - Write protect error    \n
 *             PAL_SOK                          - Return Sucess          \n
 *             DDC_NAND_E_PAGE_PROGRAM_FAIL     - Page program error     \n
 */
static PAL_Result nandPhyWritePage (
                        DDC_NandObj         *DDC,
                        Uint32              phySector,
                        Uint16              *pWriteBuff,
                        DDC_NandRedundant   *pNandRed,
                        Uint8               byteSwap);
/**
 *   \brief Reads one page of data from NAND.
 *
 *  This function reads data from the specified physical page in the NAND.
 *  This is the low level read that the driver performs to read from the
 *  physical NAND device. The API also reads the redundant information from
 *  the spare area.
 *
 *   \param DDC         [IN]    Handle to DDC
 *   \param phySector   [IN]    Address of the Sector to be read
 *   \param pReadBuffer [OUT]   Address of the buffer to read data to
 *   \param pNandRed    [OUT]   Pointer to the structure where the NAND
 *                              redundant information is to be stored
 *
 *   \return DDC_NAND_E_PAGE_READ_FAIL  - Read failed                    \n
 *           DDC_NAND_E_ECC_ERROR       - ECC error                      \n
 *           PAL_SOK                    - Return Success
 */
static PAL_Result nandPhyReadPage (
                        DDC_NandObj         *DDC,
                        Uint32              phySector,
                        Uint16              *pReadBuffer,
                        DDC_NandRedundant   *pNandRed);



/**
 *   \brief Reads 'N' bytes from NAND.
 *
 *  This function reads 'readRequest' number of bytes to the buffer supplied.
 *
 *   \param nandObj     [IN]    llc structure
 *   \param readRequest [IN]    number of bytes to be read
 *   \param pReadBuffer [IN]    address of the buffer, where the read data
 *                              has to  be stored
 *   \param pack        [IN]    if byte packing is to be done or not
 *
 *   \return None
 */
static void nandPhyReadNBytes (
            LLC_NandObj *nandObj,
            Uint16      readRequest,
            volatile Uint16      *pReadBuffer,
            Uint16      pack,
            Bool        spare)
{
    Uint16          index;
    if ((pReadBuffer != NULL) && (NULL != nandObj) ) {

#ifdef NAND_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(  &DVTEvent_Log,
                    "NAND",
                    PSP_DVT_DESC(   PSP_DVT_eRCV_START,
                                    PSP_DVT_dCHANNEL_ID_INST,
                                    PSP_DVT_dPACKET_ID,
                                    PSP_DVT_dNONE),
                    CHANNELID_INSTANCEID(
                        0,
                        nandObj->instId),
                    pReadBuffer
                    );
#endif /*NAND_INSTRUMENTATION_ENABLED*/

        if (NULL == pack) {
            if(((0 == spare) && (PSP_OPMODE_DMAINTERRUPT == nandObj->opMode)) &&
                (readRequest > NAND_MIN_DMA_REQ_SIZE))
                {

                LLC_nandDMAReadNBytes(nandObj, readRequest, pReadBuffer, pack);
                /* Function returns from here - No other statements
                 * are executed*/
            }
            else {
                for (index = 0; index < readRequest; index++) {
                    pReadBuffer[index] = LLC_nandReadByte(nandObj);
                }
            }
        }
        else
        {
            if(((0 == spare) && (PSP_OPMODE_DMAINTERRUPT == nandObj->opMode)) &&
                (readRequest > NAND_MIN_DMA_REQ_SIZE))
                {
                LLC_nandDMAReadNBytes(nandObj, readRequest, pReadBuffer, pack);
                /* Function returns from here - No other statements
                 * are executed*/
            }
            else {
                for (index = 0; index < (readRequest/2u); index++) {
#ifdef DDC_NAND_8_BIT
                        pReadBuffer[index]  = LLC_nandReadByte(nandObj);
                        pReadBuffer[index] |=
                            ((Uint16)((Uint16)LLC_nandReadByte(nandObj)) << 8);
#else
                        pReadBuffer[index]  = LLC_nandReadByte(nandObj);
#endif /* DDC_NAND_8_BIT */
                }
            }
        }
    }

#ifdef NAND_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT(  &DVTEvent_Log,
                "NAND",
                PSP_DVT_DESC(   PSP_DVT_eRCV_END,
                                PSP_DVT_dCHANNEL_ID_INST,
                                PSP_DVT_dPACKET_ID,
                                PSP_DVT_dNONE),
                CHANNELID_INSTANCEID(
                    0,
                    nandObj->instId),
                pReadBuffer
                );
#endif /*NAND_INSTRUMENTATION_ENABLED*/

}

/**
 *   \brief Writes 'N' bytes to NAND.
 *
 *  This function writes 'writeRequest' number of bytes to NAND from the
 *  buffer specified.
 *
 *   \param nandObj     [IN]    LLC structure
 *   \param writeRequest    [IN]    number of bytes to be written to NAND device
 *   \param pWriteBuff      [IN]    Addr of buffer containing data to be written
 *   \param val             [IN]    If a value, (instead of from a buffer) needs
 *                                  to be written.
 *
 *   \return None
 */
void nandPhyWriteNBytes (
            LLC_NandObj            *nandObj,
            Uint16                 writeRequest,
            volatile Uint16        *pWriteBuff,
            Uint16                 val,
            Bool                   spare)
{
    Uint16 countVar = 0;

    if(NULL != nandObj) {
        /* If the request is for 1 byte...*/
        if (1u == writeRequest) {
        LLC_nandWriteByte (nandObj, *pWriteBuff);
    }
    else if (NULL == pWriteBuff) {

        /* if a value 'val' needs to be directly written...*/
        for (countVar = 0; countVar < writeRequest; countVar++) {
            LLC_nandWriteByte(nandObj, val);
        }
    }
    else {

#ifdef NAND_INSTRUMENTATION_ENABLED

        PSP_LOG_EVENT(  &DVTEvent_Log,
                        "NAND",
                        PSP_DVT_DESC(   PSP_DVT_eDATA_SND_START,
                                        PSP_DVT_dCHANNEL_ID_INST,
                                        PSP_DVT_dPACKET_ID,
                                        PSP_DVT_dNONE),
                        CHANNELID_INSTANCEID(
                            1,
                            nandObj->instId),
                        pWriteBuff
                        );
#endif /*NAND_INSTRUMENTATION_ENABLED*/

        if(((0 == spare) && (PSP_OPMODE_DMAINTERRUPT == nandObj->opMode)) &&
            (writeRequest > NAND_MIN_DMA_REQ_SIZE))
        {
            LLC_nandDMAWriteNBytes (nandObj, writeRequest,(Uint16*) pWriteBuff, 0);
            return;
        }
        else
        {
            /* Check whether number of bytes is multiple of 2 and then write
             * as bytes into the data register.
             */
            if ((writeRequest % 2u) != 0) {
                for (countVar = 0; countVar < (writeRequest/2u); countVar++) {
#ifdef DDC_NAND_8_BIT
                        LLC_nandWriteByte(nandObj,
                                (Uint16)(pWriteBuff[countVar] & 0xFF));
                        LLC_nandWriteByte(nandObj,
                                (Uint16)(pWriteBuff[countVar] >> 8));
#else
                        LLC_nandWriteByte(nandObj, (pWriteBuff[countVar]));
#endif /* DDC_NAND_8_BIT */
                    }
                    LLC_nandWriteByte(nandObj,(pWriteBuff[countVar]));
                }
                else {
                    for (countVar = 0; countVar < (writeRequest/2u); countVar++) {
#ifdef DDC_NAND_8_BIT
                        LLC_nandWriteByte(nandObj,
                                (pWriteBuff[countVar] & 0xFF));
                        LLC_nandWriteByte(nandObj, (Uint16)(pWriteBuff[countVar] >> 8));
#else
                        LLC_nandWriteByte(nandObj, (pWriteBuff[countVar]));
#endif /* DDC_NAND_8_BIT */
                    }
                }
            }
#ifdef NAND_INSTRUMENTATION_ENABLED

            PSP_LOG_EVENT(  &DVTEvent_Log,
                            "NAND",
                            PSP_DVT_DESC(   PSP_DVT_eDATA_SND_END,
                                            PSP_DVT_dCHANNEL_ID_INST,
                                            PSP_DVT_dPACKET_ID,
                                            PSP_DVT_dNONE),
                            CHANNELID_INSTANCEID(
                                1,
                                nandObj->instId),
                            pWriteBuff);
#endif /*NAND_INSTRUMENTATION_ENABLED*/
        }
    }
}

/**
 *   \brief Read status of NAND operation
 *
 *  This function reads the status of the preceding operation on NAND Device.
 *
 *   \param DDC            [IN]     DDC structure
 *
 *   \return DDC_NAND_E_STATUS_READ_FAIL        - Read operation failed  \n
 *           DDC_NAND_E_MEDIA_WRITE_PROTECTED   - Write protect error    \n
 *           PAL_SOK              - Return success
 */
PAL_Result nandPhyReadStatus (
            DDC_NandObj *DDC)
{
    DDC_NandState   *pNandState;
    volatile Uint32 timeOut = DDC_NAND_READ_STATUS_TIMEOUT;
    volatile Uint8  statusByte;
    Uint16          statusSuccess;
    PAL_Result      retVal = PAL_SOK;

    if (DDC != NULL) {
        pNandState = (DDC_NandState *) &DDC->nandState[DDC->bankNo];
        if( NULL != pNandState) {
        if (DDC_NAND_SMALL_BLOCK == pNandState->nandType) { /* Small block */
            statusSuccess = DDC_NAND_SB_CMD_STATUS_SUCCESS;
        }
        else {
            statusSuccess = DDC_NAND_BB_CMD_STATUS_SUCCESS;
        }

        do {
            LLC_nandSendCommand(DDC->llc, DDC_NAND_CMD_STATUS);

            /* Wait for the device to provide response */
            LLC_waitLoop(30u);

            statusByte = LLC_nandReadByte(DDC->llc);
            timeOut--;
        } while (((statusByte & statusSuccess) != statusSuccess) &&
                (timeOut > 0));

        if (timeOut > 0) {
            retVal = PAL_SOK;
        }
        else {
#ifdef NAND_DBG_LEVEL_2
                NAND_Debug ("\r\n%s: %d - timeOut: %d, statusByte: 0x%x,\
                        nandPhyReadStatus: DDC_NAND_E_STATUS_READ_FAIL \
                        (0x%x).\n\r", __FILE__, __LINE__,
                                timeOut, statusByte,
                                DDC_NAND_E_STATUS_READ_FAIL);
#endif /* NAND_DBG_LEVEL_2 */
            if (0 != LLC_nandIsStatusWriteProtected(statusByte)) {
                pNandState->nandDevFlags |= LLC_NAND_MEDIA_WRITE_PROTECTED;
                retVal = DDC_NAND_E_MEDIA_WRITE_PROTECTED;
            } else {
                retVal = DDC_NAND_E_STATUS_READ_FAIL;
            }
        }
        }
    }
    return retVal;
}

/**
 *   \brief Erase a block of NAND
 *
 *  This function erases one block of data on NAND device and updates the
 *  cache buffer.
 *
 *   \param DDC            [IN]     DDC structure
 *   \param phyBlock       [IN]     Address of the physical block on NAND
 *                                  device to be erased.
 *
 *   \return DDC_NAND_E_MEDIA_WRITE_PROTECTED   - Write Protect error    \n
 *             DDC_NAND_E_BLOCK_ERASE_FAIL      - Page Erase error       \n
 *             PAL_SOK               - Return Sucess
 */
PAL_Result nandPhyEraseBlock (
            DDC_NandObj   *DDC,
            Uint32        phyBlock)
{
    PAL_Result      status;
    DDC_NandState   *pNandState;

    if (DDC != NULL) {
        pNandState     = (DDC_NandState *) &DDC->nandState[DDC->bankNo];

        if (phyBlock < (pNandState->protectedBlks/pNandState->pagesPerBlock))
        {
            NAND_Debug ("\r\nERRO in NAND Protected region ..Erasing protected region\n");
        }
        LLC_nandSendCommand(DDC->llc, DDC_NAND_CMD_BLK_ERASE_CMD1);
        nandPhySetAddress (
                DDC,
                (Uint32)phyBlock * (Uint32)pNandState->pagesPerBlock,
                1u);
        LLC_nandSendCommand(DDC->llc, DDC_NAND_CMD_BLK_ERASE_CMD2);

        /* Check Ready/Busy Line */
        LLC_checkCommandStatus(DDC->llc);

        /* Check the status of the block erase operation */
        status = nandPhyReadStatus (DDC);


        if (DDC_NAND_E_STATUS_READ_FAIL == status) {
            if (DDC_NAND_E_MEDIA_WRITE_PROTECTED == status) {
                status = (DDC_NAND_E_MEDIA_WRITE_PROTECTED);
            }
            else
            {
                /* Block Erase ERROR !!! */
                status = (DDC_NAND_E_BLOCK_ERASE_FAIL);
            }
        }
        else
        {
            status = nandCacheBlkErase (DDC, (Uint16)phyBlock);
        }
    } else {
        status = DDC_NAND_E_INVALID_DEVICE_ID;
    }

    return(status);
}

/**
 *   \brief This function calculates the ECC Error
 *
 *   \param ecc0    [IN]    ECC 1
 *   \param ecc1    [IN]    ECC 2
 *
 *   \return Ones Complement
 */
Uint16 nandPhyEccErrorTest (
            Uint8 *ecc0,
            Uint8 *ecc1)
{
    Uint16 onesCnt = 0;
    Uint16 loopIndex;

    Uint32 ecc0Combined  = 0;
    Uint32 ecc1Combined  = 0;
    Uint32 resultXor     = 0;

    if((NULL != ecc0) && (NULL != ecc1)) {
        ecc0Combined = (Uint32)((Uint32)ecc0[1] << 16) |
            (Uint32)((Uint32)ecc0[0] << 8)  |
            (Uint32)ecc0[2];

        ecc1Combined = (Uint32)((Uint32)ecc1[1] << 16) |
            (Uint32)((Uint32)ecc1[0] << 8) |
            (Uint32)ecc1[2];

        resultXor = ecc0Combined ^ ecc1Combined;

        if (resultXor == 0) {
            onesCnt = 0; /* if XOR result 0 then no error */
        }
        else
        {
            for (loopIndex = 0, onesCnt = 0; loopIndex < 24u; loopIndex++) {
                if ((resultXor & (1u << loopIndex)) != 0) {
                    onesCnt++;
                }
            }
        }
    }
    return (onesCnt);
}


/**
 *   \brief This function is used to calculate the failed bits in ECC
 *
 *   \param ecc - ECC Code
 *
 *   \return DDC_NandBitFailCodeStruct
 */
DDC_NandBitFailCodeStruct nandPhyGetBitFailCodes (
            Uint8 *ecc)
{
    DDC_NandBitFailCodeStruct failCode;

    Uint8 loopVar = 0;

    failCode.parity         = 0;
    failCode.parity_prime   = 0;
    failCode.bitAddress     = 0;
    failCode.bitAdressPrime = 0;

    if(NULL != ecc) {

        for (loopVar = 0; loopVar < 4u; loopVar++) {
            failCode.parity = (Uint8)(((ecc[1] << loopVar)
                        & (0x80 >> loopVar))
                    | failCode.parity);

            failCode.parity = (Uint8)(((ecc[0] >> (loopVar+1))
                        & (0x01 << loopVar))
                    | failCode.parity);

            failCode.parity_prime = (Uint8)(((ecc[1] << (loopVar+1))
                        & (0x80 >> loopVar))
                    | failCode.parity_prime);

            failCode.parity_prime = (Uint8)(((ecc[0] >> loopVar)
                        & (0x01 << loopVar))
                    | failCode.parity_prime);
        }

        for (loopVar = 0; loopVar < 3u; loopVar++) {
            failCode.bitAddress = (Uint8)(((ecc[2] >> (loopVar+3))
                        & (0x01 << loopVar))
                    | failCode.bitAddress);

            failCode.bitAdressPrime = (Uint8)(((ecc[2] >> (loopVar+2))
                        & (0x01 << loopVar))
                    | failCode.bitAdressPrime);
        }
    }

    return (failCode);
}

/**
 *   \brief Issue a command to NAND
 *
 *  This function sets the command to read/write from NAND device.
 *
 *   \param DDC        [IN]     Handle to DDC
 *   \param cmd        [IN]     command to be set
 *   \param startAddr  [IN]     row address to be written
 *
 *   \return None
 */
void nandPhyIssueCommand (
            DDC_NandObj *DDC,
            Uint16      cmd,
            Uint32      startAddr)
{
    if (DDC != NULL)
    {
        LLC_nandSendCommand(DDC->llc, cmd);
        asm("\tNOP");
        asm("\tNOP");
        asm("\tNOP");
        asm("\tNOP");
        nandPhySetAddress (DDC, startAddr, 0);
        asm("\tNOP");
        asm("\tNOP");
        asm("\tNOP");
        asm("\tNOP");

    }
}


/**
 *   \brief Set address on NAND bus
 *
 *  This function writes page addr (column and row addresses) to the NAND bus
 *
 *   \param DDC         [IN]    DDC structure
 *   \param startAddr   [IN]    page address to be written
 *   \param pageOnly    [IN]    Flag to distinguish b/w page operations and
 *                              block operations
 *   \return None
 */
static void nandPhySetAddress (
            DDC_NandObj *DDC,
            Uint32      startAddr,
            Uint16      pageOnly)
{
    Uint16          countVar = 0;

    DDC_NandState   *pNandState;

    if ( DDC != NULL )
    {
        pNandState = (DDC_NandState *) &DDC->nandState[DDC->bankNo];

        if (DDC_NAND_SMALL_BLOCK == pNandState->nandType) { /* Small block */
            if (FALSE == pageOnly) {           /* Column Address   */
                LLC_nandSetAddr(DDC->llc, 0x0);
            }
            for (; countVar < pNandState->addrCycle; countVar++,
                    startAddr >>= 8) {          /* Row Address      */
                LLC_nandSetAddr(DDC->llc, startAddr);
            }
        }
        else {
            if (FALSE == pageOnly) {   /* Column Address   */
                LLC_nandSetAddr(DDC->llc, 0x0);
                LLC_nandSetAddr(DDC->llc, 0x0);
            }

            for (; countVar < pNandState->addrCycle; countVar++,
                    startAddr >>= 8) {  /* Row Address      */
                LLC_nandSetAddr(DDC->llc, startAddr);
            }
        }
    }
}

/**
 *   \brief Write to a sector on NAND
 *
 *  This function ensures that data is written sequentially to the specified
 *  sector on NAND. Before writing to the specified sector, it checks the
 *  cached area to find if the page exists.
 *
 *   \param DDC          [IN]    Handle to DDC
 *   \param phySector    [IN]    Address of the Sector at which the data has
 *                               to be written
 *   \param pWriteBuff   [IN]    Address of the buffer to write data from
 *   \param pNandRed     [IN]    Pointer to the structure where the NAND
 *                               redundant information is stored
 *   \param byteSwap     [IN]    Swap bytes?
 *
 *   \return PAL_SOK                        - Return Sucess              \n
 *           DDC_NAND_E_PAGE_PROGRAM_FAIL   - Page program error         \n
 */
PAL_Result nandPhyWriteSector (
            DDC_NandObj          *DDC,
            Uint32               phySector,
            Uint16              *pWriteBuff,
            DDC_NandRedundant   *pNandRed,
            Uint8               byteSwap)
{
    DDC_NandState   *pNandState;
    Uint32              phyBlkNo;
    Uint32              phySecNo;
    PAL_Result          result = PAL_SOK;
    DDC_NandCachePageRedInfo    cachedPageRedInfo;


    if ((NULL != DDC) && ((NULL != pNandRed) && (NULL != pWriteBuff)))
    {
        pNandState = (DDC_NandState *) &DDC->nandState[DDC->bankNo];
        /* Get the physical block */
        phyBlkNo = (Uint32)(phySector / pNandState->pagesPerBlock);

        /* Get the phySector status; cached or not-cached. */
        result = nandCacheGetPageStatus (DDC, phySector, &cachedPageRedInfo);
        if (PAL_SOK == result) {
            /* Get the sector number */
            phySecNo = (Uint32)((phyBlkNo * pNandState->pagesPerBlock) +
                (cachedPageRedInfo.cacheIndex));

            /* Load the logical page no in redundant aera. */
            pNandRed->logPageNo = (Uint16)(phySector %
                    pNandState->pagesPerBlock);

            /* Start write to the physical page */
            result = nandPhyWritePage (
                    DDC,
                    phySecNo,
                    pWriteBuff,
                    pNandRed,
                    byteSwap);

            if (PAL_SOK == result) {
                    result = nandCacheUpdatePageEntry (DDC, phySector);
            }
            else    {
                 result = DDC_NAND_E_PAGE_PROGRAM_FAIL;
            }
        }
    }
    else
    {
        result = DDC_NAND_E_INVALID_DEVICE_ID;
    }
    return result;
}


/**
 *   \brief Writes a page of data to the NAND.
 *
 *  This function writes data to the specified physical page in the NAND.
 *  This is the low level write that the driver performs to write to the
 *  physical NAND device. The API also writes the redundant area information
 *  to the spare area.
 *
 *   \param DDC         [IN]    Handle to DDC
 *   \param phySector   [IN]    Address of the Sector at which the data has to
 *                              be written
 *   \param pWriteBuff  [IN]    Address of the buffer
 *   \param pNandRed    [IN]    Pointer to the structure where the NAND
 *                              redundant information is stored
 *   \param byteSwap    [IN]    Swap bytes?
 *
 *   \return PAL_Result
 *             DDC_NAND_E_MEDIA_WRITE_PROTECTED - Write protect error    \n
 *             PAL_SOK                          - Return Sucess          \n
 *             DDC_NAND_E_PAGE_PROGRAM_FAIL     - Page program error     \n
 */
static PAL_Result nandPhyWritePage (
            DDC_NandObj         *DDC,
            Uint32              phySector,
            Uint16              *pWriteBuff,
            DDC_NandRedundant  *pNandRed,
            Uint8               byteSwap)
{
    Uint16      dataStatus;
    Uint16      blockStatus;
    Uint16      blockAddr;
    PAL_Result  status;
    Uint16      ii;
    Uint32      hw_ecc = 0;
    Uint8       spareArea [64];
    Uint16      tempVar;

    DDC_NandState   *pNandState;

    Uint16 ECC[DDC_NAND_ECC_SIZE];

    if ((NULL != DDC) && ((NULL != pWriteBuff) && (NULL != pNandRed)))
    {
        pNandState = (DDC_NandState *) &DDC->nandState[DDC->bankNo];

    dataStatus  = DDC_NAND_DATA_STATUS_FREE;
    blockStatus = DDC_NAND_BLOCK_STATUS_FREE;
    blockAddr   = DDC_NAND_BLOCK_ADDR_FREE;

    /* Check whether the bytes has to be swapped */
    if (1u == byteSwap) {
        for (ii=0; ii< ((pNandState->bytesPerPage)/2u); ii++) {
             pWriteBuff[ii] = (Uint16)((pWriteBuff[ii] << 8)
                        | ((pWriteBuff[ii] & 0xff00) >> 8));
        }
    }

    /* Check whether Redundant area has to be written */
    if (pNandRed != NULL) {
        dataStatus = pNandRed->dataStatus;
        blockStatus = pNandRed->blockStatus;

        if (pNandRed->blockAddress < DDC_NAND_USABLE_BLOCKS_PER_ZONE) {
            blockAddr = nandUtlCalcBlkAdrs (pNandRed->blockAddress);
                blockAddr = (Uint16)(((blockAddr & 0xFF00) >> 8)
                        | (blockAddr << 8));
        }
    }

    /* Issue CMD0 to set Page Pointer */
    LLC_nandSendCommand(DDC->llc, DDC_NAND_CMD_LO_PAGE);

    /* Issue Write Command */
    nandPhyIssueCommand (DDC, DDC_NAND_CMD_PGRM_START, phySector);

    /* Clear ECC */
    hw_ecc = LLC_nandReadECC(DDC->llc);
    PAL_osMemSet(ECC, 0, DDC_NAND_ECC_SIZE*2u);

    /* Write Data */
    for(ii = 0; ii < (pNandState->bytesPerPage/NAND_RW_BLK_SIZE); ii++)
    {
        /* Start ECC */
        LLC_nandEnableHwECC(DDC->llc, (LLC_NandBankNo)DDC->bankNo);

        nandPhyWriteNBytes (DDC->llc, (Uint16)NAND_RW_BLK_SIZE,
                &pWriteBuff[ii*(NAND_RW_BLK_SIZE/2)], 0, 0);

        /* Read ECC */
        hw_ecc = LLC_nandReadECC(DDC->llc);

        hw_ecc = (((hw_ecc >> 4) & 0x00FFF000) | (hw_ecc & 0xFFF));
        ECC[ii*2] = (Uint16)(hw_ecc & 0xFFFF);
        ECC[(ii*2)+1] = (Uint16)((hw_ecc >> 16) & 0xFF);
    }

    if ((NULL == pNandRed) && (0 ==
                (phySector % pNandState->pagesPerBlock))) {
        nandPhyWriteNBytes (DDC->llc, (Uint16)pNandState->spareAreaSize,
                NULL, 0xFF,1u);
    }
    else {
            if (DDC_NAND_SMALL_BLOCK == pNandState->nandType) {
                /* Small block */
                /* Write redundant Area */
                spareArea[0]    = 0xFF;
                spareArea[1]    = (Uint8)(pNandRed->logPageNo & 0xFF);
                spareArea[2]    = 0xFF;
                spareArea[3]    = 0xFF;
                spareArea[4]    = (Uint8)(blockStatus & 0xFF);
                spareArea[5]    = (Uint8)(dataStatus & 0xFF);
                spareArea[6]    = (Uint8)(blockAddr & 0xFF);
                spareArea[7]    = (Uint8)((blockAddr >> 8u) & 0xFF);
                spareArea[8]    = (Uint8)(ECC[0] & 0xFF);
                spareArea[9]    = (Uint8)((ECC[0] >> 8 )& 0xFF);
                spareArea[10]   = (Uint8)(ECC[1] & 0xFF);
                spareArea[11]   = (Uint8)(blockAddr & 0xFF);
                spareArea[12]   = (Uint8)((blockAddr >> 8) & 0xFF);
                spareArea[13]   = (Uint8)(ECC[2] & 0xFF);
                spareArea[14]   = (Uint8)((ECC[2] >> 8 )& 0xFF);
                spareArea[15]   = (Uint8)(ECC[3] & 0xFF);

                nandPhyWriteNBytes (DDC->llc,(Uint16)pNandState->spareAreaSize,
                        (Uint16 *)spareArea, 0xFF,1u);
        }
        else {
            /* Write redundant Area */
                spareArea[0]    = (Uint8)(blockStatus & 0xFF);
                spareArea[1]    = (Uint8)(pNandRed->logPageNo & 0xFF);
                spareArea[2]    = 0xFF;
                spareArea[3]    = 0xFF;
                spareArea[4]    = 0xFF;
                spareArea[5]    = 0xFF;
                spareArea[6]    = (Uint8)(blockAddr & 0xFF);
                spareArea[7]    = (Uint8)((blockAddr >> 8) & 0xFF);
                spareArea[8]    = (Uint8)(ECC[0] & 0xFF);
                spareArea[9]    = (Uint8)((ECC[0] >> 8 )& 0xFF);
                spareArea[10]   = (Uint8)(ECC[1] & 0xFF);
                spareArea[11]   = (Uint8)(blockAddr & 0xFF);
                spareArea[12]   = (Uint8)((blockAddr >> 8) & 0xFF);
                spareArea[13]   = (Uint8)(ECC[2] & 0xFF);
                spareArea[14]   = (Uint8)((ECC[2] >> 8 )& 0xFF);
                spareArea[15]   = (Uint8)(ECC[3] & 0xFF);
                spareArea[16]   = (Uint8)(ECC[4] & 0xFF);
                spareArea[17]   = (Uint8)((ECC[4] >> 8 )& 0xFF);
                spareArea[18]   = (Uint8)(ECC[5] & 0xFF);
                spareArea[19]   = (Uint8)(ECC[6] & 0xFF);
                spareArea[20]   = (Uint8)((ECC[6] >> 8 )& 0xFF);
                spareArea[21]   = (Uint8)(ECC[7] & 0xFF);
                spareArea[22]   = (Uint8)(ECC[8] & 0xFF);
                spareArea[23]   = (Uint8)((ECC[8] >> 8 )& 0xFF);
                spareArea[24]   = (Uint8)(ECC[9] & 0xFF);
                spareArea[25]   = (Uint8)(ECC[10] & 0xFF);
                spareArea[26]   = (Uint8)((ECC[10] >> 8 )& 0xFF);
                spareArea[27]   = (Uint8)(ECC[11] & 0xFF);
                spareArea[28]   = (Uint8)(ECC[12] & 0xFF);
                spareArea[29]   = (Uint8)((ECC[12] >> 8 )& 0xFF);
                spareArea[30]   = (Uint8)(ECC[13] & 0xFF);
                spareArea[31]   = (Uint8)(ECC[14] & 0xFF);
                spareArea[32]   = (Uint8)((ECC[14] >> 8 )& 0xFF);
                spareArea[33]   = (Uint8)(ECC[15] & 0xFF);

                for (tempVar = 34u; tempVar < pNandState->spareAreaSize;
                        tempVar++) {
                    spareArea[tempVar]=0xFF;
                }

                nandPhyWriteNBytes (DDC->llc, (Uint16)pNandState->spareAreaSize
                        ,(Uint16 *)spareArea, 0xFF,1u);
        }
    }

    /* Issue write commit */
    LLC_nandSendCommand(DDC->llc, DDC_NAND_CMD_PGRM_END);

    /* Check R/Bn line */
    LLC_checkCommandStatus(DDC->llc);

    status = nandPhyReadStatus (DDC);

#ifdef __DEBUG__PRINT
    {
        Int     count = 0;
        Uint32  *tmp_ptr = (Uint32 *)pWriteBuff;

        PSP_DEBUG1("nandPhyWritePage - page = %d\n", phySector);
        for (count = 0; count < pNandState->bytesPerPage/4; count ++)
        {
            PSP_DEBUG1("%x ", tmp_ptr[count]);
        }

        tmp_ptr = (Uint32 *)spareArea;
        for (count = 0; count < pNandState->spareAreaSize/4; count ++)
        {
            PSP_DEBUG1("%x ", tmp_ptr[count]);
        }

        PSP_DEBUG("\n");
    }
#endif

    if (DDC_NAND_E_STATUS_READ_FAIL == status) {
        if (DDC_NAND_E_MEDIA_WRITE_PROTECTED == status) {
                status = (DDC_NAND_E_MEDIA_WRITE_PROTECTED);
            } else {
        /* Page program ERROR !!! */
                status = (DDC_NAND_E_PAGE_PROGRAM_FAIL);
            }
    }
    else {
            status = (PAL_SOK);
        }
    }
    else
    {
        status = DDC_NAND_E_INVALID_DEVICE_ID;
    }

    return status;
}

/**
 *   \brief Read a sector from NAND
 *
 *  This function ensures reading a sector sequentially. Before reading from
 *  the specified sector, it checks the cached area to find if the page
 *  exists, there by saving some cycle time.
 *
 *   \param DDC         [IN]    Nand media state variable
 *   \param phySector   [IN]    Address of the Sector to be read
 *   \param pReadBuffer [OUT]   Address of the buffer
 *   \param pNandRed    [OUT]   Pointer to the structure where the NAND
 *                              redundant information is to be stored
 *
 *   \return PAL_Error  - On error                                       \n
 *                  PAL_SOK     - Return Success
 */
PAL_Result nandPhyReadSector (
            DDC_NandObj           *DDC,
            Uint32                phySector,
            Uint16                *pReadBuffer,
            DDC_NandRedundant     *pNandRed)
{
    DDC_NandState   *pNandState;

    Uint16              count;
    Uint32              phyBlkNo = 0;
    DDC_NandCachePageRedInfo   cachedPageRedInfo;
    PAL_Result          result;

    if ((NULL != DDC) && ((NULL != pReadBuffer) && (NULL != pNandRed)))
    {

        pNandState = (DDC_NandState *) &DDC->nandState[DDC->bankNo];
        /* Get the block number */
        phyBlkNo = (Uint32)(phySector / pNandState->pagesPerBlock);

        /* Get the page status; did we cache 'phySector' already,
         * if not read and cache! */
        result = nandCacheGetPageStatus (DDC, phySector, &cachedPageRedInfo);
        if (PAL_SOK == result) {
            /* Read the page, if it exists. */
            if (TRUE == cachedPageRedInfo.pageFound) {
                phySector = (Uint32)((phyBlkNo * pNandState->pagesPerBlock)
                    + cachedPageRedInfo.cacheIndex);
                result = nandPhyReadPage(DDC,((Uint32)phySector),
                        (Uint16 *)pReadBuffer, pNandRed);
            }
            else {
                /* Fill with 0xFFFF, if the page is not written. */
                for (count = 0; count < (pNandState->bytesPerPage/2u);
                        count++) {
                    *pReadBuffer = 0xFFFFu;
                    pReadBuffer++;
                }

                pNandRed->blockAddress = DDC_NAND_BLOCK_ADDR_FREE;
                pNandRed->blockStatus  = DDC_NAND_BLOCK_STATUS_FREE;
                pNandRed->dataStatus   = DDC_NAND_DATA_STATUS_FREE;
                pNandRed->logPageNo    = DDC_NAND_LOG_PAGE_FREE;
            }
        }
    }
    else
    {
        result = DDC_NAND_E_INVALID_DEVICE_ID;
    }

    return result;
}

/**
 *   \brief Calculate number of bits in byte
 *
 *  This function calculates numbers of biy in byte
 *
 *   \param n           [IN]    Value in which u want to find num of bits
 *
 *   \return count  -       Number of bits                                       \n
 *
 */
static int bitcount (Uint8 n)
{
    Int32 count = 1;

    do {
        n = (Uint8)(n ^ (n-1u));
        if(n > 0) {
            count ++;
        }
    } while (n > 1);

    return count;
}

/**
 *   \brief Check 1 bit ECC error for NAND.
 *
 *  This function compair's software ECC value with hardware ECC value .
 *  If there is 1 bit error than it will correct it otherwisereturn fail
 *  status
 *
 *   \param pNandState  [IN]    Handle to nand state.
 *   \param pBuffer     [IN]    Address of the buffer data was read.
 *   \param pHwEcc      [IN]    Hardware ECC.
 *   \param pStEcc      [IN]    Software ECC.
 *
 *  \return PAL_SOK or -1
 *
 */
static PAL_Result nandPhyCorrectECC(DDC_NandState   *pNandState,
                                     Uint8          *pBuffer,
                                     Uint16         *pHwEcc,
                                     Uint16         *pStEcc)
{
    Int count = 1, index = 0;
    Int bits = 0;
    PAL_Result result = PAL_SOK;
    Uint32 byte_address;
    Uint32 bit_address;
    Uint8   eccXor[3];
    Int32   maxBitErr = 12;

    if(((NULL != pNandState) && (NULL != pBuffer)) &&
            ((NULL != pHwEcc) && (NULL != pStEcc))) {
        if (DDC_NAND_BIG_BLOCK == pNandState->nandType)
        {
            count = 4;
        }

        for (index = 0; index < count; index ++)
        {
            eccXor[0] = (Uint8)((pHwEcc[((index*2) + 0)]
                        ^ pStEcc[((index*2) + 0)]) & 0xFF);
            eccXor[1] = (Uint8)(((pHwEcc[((index*2) + 0)]
                            ^ pStEcc[((index*2) + 0)]) >> 8) & 0xFF);
            eccXor[2] = (Uint8)((pHwEcc[((index*2) + 1)]
                        ^ pStEcc[((index*2) + 1)]) & 0xFF);

        if(((eccXor[0] | eccXor[1]) | eccXor[2]) != 0) {

                /* Count Number of bits */
                bits = bitcount(eccXor[0]);
                bits += bitcount(eccXor[1]);
                bits += bitcount(eccXor[2]);

                if(bits == maxBitErr)
                {
                    bit_address = (eccXor[0] & 0x07);

                byte_address = (((Uint8)(eccXor[0] >> 3) |  (Uint8)(eccXor[1] >> 3)) |
                                (Uint8)(eccXor[2] >> 3)) & 0x1FF;

                pBuffer[byte_address] ^= (0x1 << bit_address);
                }
                else
                {
#ifdef PSP_NAND_DEBUG
                    if (bits == 1)
                    {
                        NAND_Debug("\r\n2 bit ECC Error\n\r");
                    }
                    else
                    {
                        NAND_Debug("\r\nMore than 2 bit ECC Error\n\r");
                    }
#endif

                    result = -1;
                    break;
                }
            }
        }
    }

    result = 0; /* ECC is not working - Remove this once ECC is verified */
    return (result);
}

/**
 *   \brief Reads one page of data from NAND.
 *
 *  This function reads data from the specified physical page in the NAND.
 *  This is the low level read that the driver performs to read from the
 *  physical NAND device. The API also reads the redundant information from
 *  the spare area.
 *
 *   \param DDC         [IN]    Handle to DDC
 *   \param phySector   [IN]    Address of the Sector to be read
 *   \param pReadBuffer [OUT]   Address of the buffer to read data to
 *   \param pNandRed    [OUT]   Pointer to the structure where the NAND
 *                              redundant information is to be stored
 *
 *   \return DDC_NAND_E_PAGE_READ_FAIL  - Read failed                    \n
 *           DDC_NAND_E_ECC_ERROR       - ECC error                      \n
 *                  PAL_SOK          - Return Success
 */

static PAL_Result nandPhyReadPage (
            DDC_NandObj         *DDC,
            Uint32              phySector,
            Uint16              *pReadBuffer,
            DDC_NandRedundant   *pNandRed)
{
    DDC_NandState   *pNandState;
    PAL_Result      status = PAL_SOK;
    Int32           ii;
    Uint16          ecc[DDC_NAND_ECC_SIZE];
    Uint16          ecc_spare[DDC_NAND_ECC_SIZE];
    Uint32          hw_ecc = 0;
    Uint8           nandSpare[64];
    Int             read_retry = 2; /* Set retry count for read */

    if ( DDC != NULL )
    {
        pNandState = (DDC_NandState *) &DDC->nandState[DDC->bankNo];
        do
        {
            if (DDC_NAND_SMALL_BLOCK == pNandState->nandType) {
                /* Small block */
                nandPhyIssueCommand (DDC, DDC_NAND_CMD_LO_PAGE, phySector);
            }
            else {
                nandPhyIssueCommand (DDC, DDC_NAND_CMD_READ_START, phySector);
                LLC_nandSendCommand (DDC->llc, DDC_NAND_CMD_READ_END);
            }

            LLC_checkCommandStatus(DDC->llc);

            /* Clear ECC */
            hw_ecc = LLC_nandReadECC(DDC->llc);

            /* Read Data */
            for(ii = 0; ii < (pNandState->bytesPerPage/NAND_RW_BLK_SIZE); ii++)
            {
                /* Start ECC */
                LLC_nandEnableHwECC(DDC->llc, (LLC_NandBankNo)DDC->bankNo);

                nandPhyReadNBytes (DDC->llc, (Uint16)NAND_RW_BLK_SIZE,
                        &pReadBuffer[ii*(NAND_RW_BLK_SIZE/2L)], 1u, 0);

                /* Read ECC */
                hw_ecc = LLC_nandReadECC(DDC->llc);

                hw_ecc = (((hw_ecc >> 4) & 0x00FFF000) | (hw_ecc & 0xFFF));
                ecc[ii*2] = (Uint16)(hw_ecc & 0xFFFF);
                ecc[(ii*2)+1] = (Uint16)((hw_ecc >> 16) & 0xFF);
            }

            if (pNandRed != NULL) {
                /* Read the redundant Area */
                if (DDC_NAND_SMALL_BLOCK == pNandState->nandType) {
                    /* Small block NAND */
                    nandPhyReadNBytes (DDC->llc,
                        (Uint16)pNandState->spareAreaSize,
                        (Uint16 *)nandSpare, 1u, 1u);

                    pNandRed->logPageNo     = nandSpare[1];
                    pNandRed->blockStatus   = nandSpare[4];
                    pNandRed->dataStatus    = nandSpare[5];
                    pNandRed->blockAddress  =
                    (Uint16)((nandSpare[6] & 0x7) << 7);

                    pNandRed->blockAddress |=
                    (Uint16)((nandSpare[7] & 0xFF) >> 1);
                    ecc_spare[0]            =
                    (Uint16)(((Uint16)(((Uint16)nandSpare[9]) << 8)) |
                                                nandSpare[8]);
                    ecc_spare[1]            = (Uint16)(nandSpare[10] & 0xFF);
                    ecc_spare[2]            =
                    (Uint16)(((Uint16)(((Uint16)nandSpare[14]) << 8)) |
                                                nandSpare[13]);
                    ecc_spare[3]            = (Uint16)(nandSpare[15] & 0xFF);
                }
                else {
                    nandPhyReadNBytes (DDC->llc,
                            (Uint16)pNandState->spareAreaSize,
                            (Uint16*)nandSpare, 1u, 1u);

                    pNandRed->blockStatus   = nandSpare[0];
                    pNandRed->logPageNo     = nandSpare[1];

                    pNandRed->blockAddress  =
                        (Uint16)((nandSpare[6] & 0x7) << 7);

                    pNandRed->blockAddress |=
                        (Uint16)((nandSpare[7] & 0xFF) >> 1);

                    ecc_spare[0]            =
                        (Uint16)(((Uint16)(((Uint16)nandSpare[9]) << 8)) |
                                                    nandSpare[8]);

                    ecc_spare[1]            = (Uint16)(nandSpare[10] & 0xFF);

                    ecc_spare[2]            =
                        (Uint16)(((Uint16)(((Uint16)nandSpare[14]) << 8)) |
                                                    nandSpare[13]);

                    ecc_spare[3]            = (Uint16)(nandSpare[15] & 0xFF);

                    ecc_spare[4]            =
                        (Uint16)(((Uint16)(((Uint16)nandSpare[17]) << 8)) |
                                                    nandSpare[16]);

                    ecc_spare[5]            = (Uint16)(nandSpare[18] & 0xFF);

                    ecc_spare[6]            =
                        (Uint16)(((Uint16)(((Uint16)nandSpare[20]) << 8)) |
                                                    nandSpare[19]);

                    ecc_spare[7]            = (Uint16)(nandSpare[21] & 0xFF);

                    ecc_spare[8]            =
                        (Uint16)(((Uint16)(((Uint16)nandSpare[23]) << 8)) |
                                                    nandSpare[22]);

                    ecc_spare[9]            = (Uint16)(nandSpare[24] & 0xFF);

                    ecc_spare[10]           =
                        (Uint16)(((Uint16)(((Uint16)nandSpare[26]) << 8)) |
                                                    nandSpare[25]);

                    ecc_spare[11]           = (Uint16)(nandSpare[27] & 0xFF);

                    ecc_spare[12]           =
                        (Uint16)(((Uint16)(((Uint16)nandSpare[29]) << 8)) |
                                                    nandSpare[28]);

                    ecc_spare[13]           = (Uint16)(nandSpare[30] & 0xFF);

                    ecc_spare[14]           =
                        (Uint16)(((Uint16)(((Uint16)nandSpare[32]) << 8)) |
                                                    nandSpare[31]);
                    ecc_spare[15]           = (Uint16)(nandSpare[33] & 0xFF);
                }
            }
            else
            {
                /** Flush Spare Area Data */
                            nandPhyReadNBytes (DDC->llc,
                                    (Uint16)pNandState->spareAreaSize, NULL, 0,1u );
            }

#ifdef __DEBUG__PRINT
    {
        Int     count = 0;
        Uint32  *tmp_ptr = (Uint32 *)pReadBuffer;

        PSP_DEBUG1("nandPhyReadPage - page = %d \n", phySector);
        for (count = 0; count < pNandState->bytesPerPage/4; count ++)
        {
            PSP_DEBUG1("%x ", tmp_ptr[count]);
        }

        tmp_ptr = (Uint32 *)nandSpare;
        for (count = 0; count < pNandState->spareAreaSize/4; count ++)
        {
            PSP_DEBUG1("%x ", tmp_ptr[count]);
        }

        PSP_DEBUG("\n");
    }
#endif
            status = nandPhyCorrectECC(pNandState,
                                   (Uint8 *)pReadBuffer,
                                   ecc,
                                   ecc_spare);

            if(PAL_SOK == status)
            {
                break;
            }

        } while(--read_retry);

        if(0 == read_retry) {
            status = DDC_NAND_E_ECC_ERROR;
        }

    } else {
        status = DDC_NAND_E_INVALID_DEVICE_ID;
    }

    return (status);
}

/**
 *   \brief Read redundant info from NAND
 *
 *  This function reads the redundant area of a physical sector. Before
 *  reading the redundant area, the cached area is searched to find if the
 *  redundant info for this physical sector is already cached and if so,
 *  reads from the cache. Else, it reads from the device.
 *
 *   \param DDC         [IN]    DDC object
 *   \param phySector   [IN]    Physical Sector number of NAND
 *   \param pNandRed    [OUT]   Address of the buffer
 *
 *   \return PAL_Result
 *                  DDC_NAND_E_READ_FAIL    - Read failed                \n
 *                  PAL_SOK        - Return Sucess
 */
PAL_Result nandPhyReadRed (
            DDC_NandObj *DDC,
            Uint32      phySector,
            DDC_NandRedundant   *pNandRed)
{
    PAL_Result                  result = PAL_SOK;
    DDC_NandCachePageRedInfo    cachedPageRedInfo;
    Uint32                      blkNo;
    DDC_NandState               *pNandState;

    if ( (NULL != DDC) && (NULL != pNandRed)) {
        pNandState = (DDC_NandState *) &DDC->nandState[DDC->bankNo];
        /* Get the block number */
        blkNo = (Uint32)(phySector / pNandState->pagesPerBlock);

        /* Read the status of the physical sector in the cache */
        result = nandCacheGetPageStatus (DDC, phySector, &cachedPageRedInfo);
        if (PAL_SOK == result) {
            /* Page found in cache */
            if (TRUE == cachedPageRedInfo.pageFound) {
                phySector = (Uint32)((blkNo * pNandState->pagesPerBlock)
                    + cachedPageRedInfo.cacheIndex);
                result = nandPhyReadRedPage(DDC, phySector,
                        (DDC_NandRedundant*)pNandRed);
            }
            else {
                PAL_osMemSet (pNandRed, (char)DDC_NAND_DEF_INIT_VALUE,
                        sizeof(DDC_NandRedundant));
                pNandRed->blockAddress = DDC_NAND_BLOCK_ADDR_FREE;
            }
        }
    } else {
        result = DDC_NAND_E_INVALID_DEVICE_ID;
    }
    return result;
}

/**
 *   \brief Read the redundant info from NAND.
 *
 *  This function reads redundant area of the requested physical sector
 *  physically from the NAND device and stores into the provided buffer.
 *  This is the physical interface to the NAND device.
 *
 *   \param DDC       [IN]  Handle to DDC
 *   \param phySector [IN]  Physical Sector number of NAND
 *   \param pRedBuffer  [OUT] Address of the buffer
 *
 *   \return PAL_Result
 *                  DDC_NAND_E_PAGE_READ_FAIL   - Page read failed      \n
 *                  PAL_SOK        - Return Sucess
 */
PAL_Result nandPhyReadRedPage (
            DDC_NandObj *DDC,
            Uint32      phySector,
            DDC_NandRedundant   *pRedBuffer)
{
    DDC_NandState   *pNandState;
    PAL_Result      status = PAL_SOK;
    Uint8           nandSpare[64];

    if ((NULL != DDC) && (NULL != pRedBuffer))
    {
        pNandState = (DDC_NandState *) &DDC->nandState[DDC->bankNo];


        if (DDC_NAND_SMALL_BLOCK == pNandState->nandType){
            /* Small block NAND */
            nandPhyIssueCommand (DDC, DDC_NAND_CMD_SPARE_AREA, phySector);

            LLC_checkCommandStatus(DDC->llc);
        }
        else {
            /* Read the redundant area */
            if(TRUE == LLC_nandHasRandomRead (pNandState->nandOptions))
            {
                Uint32      countVar = 0;

                LLC_nandSendCommand(DDC->llc, DDC_NAND_CMD_READ_START);

                /* Column Address set to 2048  */
                LLC_nandSetAddr(DDC->llc, 0x00);
                LLC_nandSetAddr(DDC->llc, 0x08);

                /* Row Address      */
                for (; countVar < pNandState->addrCycle; countVar++,
                        phySector >>= 8) {
                    LLC_nandSetAddr(DDC->llc, (Uint16)phySector);
                }

                LLC_nandSendCommand(DDC->llc, DDC_NAND_CMD_READ_END);
                LLC_checkCommandStatus(DDC->llc);
            }
            else
            {
                nandPhyIssueCommand (DDC, DDC_NAND_CMD_READ_START, phySector);
                LLC_nandSendCommand(DDC->llc, DDC_NAND_CMD_READ_END);
                LLC_checkCommandStatus(DDC->llc);
                nandPhyReadNBytes (DDC->llc, pNandState->bytesPerPage, NULL, 0, 1u);
            }
        }

        nandPhyReadNBytes (DDC->llc, (Uint16)pNandState->spareAreaSize,
                (Uint16*) nandSpare, 1u,1u);

        if(DDC_NAND_SMALL_BLOCK == pNandState->nandType)
        {
            pRedBuffer->blockStatus     = (Uint16)(nandSpare[4] | 0xFF00);
            pRedBuffer->logPageNo       = (Uint16)(nandSpare[1] | 0xFF00);
            pRedBuffer->dataStatus      = (Uint16)0xFF;
            pRedBuffer->blockAddress    = (Uint16)(((Uint16)(((Uint16)
                    nandSpare[6]) << 8)) | nandSpare[7]); /* Swap Bytes */
        }
        else
        {
            pRedBuffer->blockStatus     = (Uint16)(nandSpare[0] | 0xFF00);
            pRedBuffer->logPageNo       = (Uint16)(nandSpare[1] | 0xFF00);
            pRedBuffer->dataStatus      = (Uint16)0xFF;
            pRedBuffer->blockAddress    = (Uint16)(((Uint16)(((Uint16)
                    nandSpare[6]) << 8)) | nandSpare[7]); /* Swap Bytes */
        }

    }
    else
    {
        status = DDC_NAND_E_INVALID_DEVICE_ID;
    }

    return (status);
}


