/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 * \file    ddc_nandUtl.c
 *
 * \brief  NAND Flash Utilities (helper functions) for DM6437.
 *
 * This file contains utilities functions for the Generic NAND flash
 * driver APIs on DM6437.
 *
 * (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \note       Set tabstop to 4 (:se ts=4) while viewing this file in an
 *              editor
 *
 *  \author     AK Mistral, Himamsu Mistral
 *
 *  \version    1.0 AK Mistral      Created.
 *  \version    1.1 Himamsu Mistral Code cleanup and release for BFT5
 *  \version    1.2 Himamsu Mistral Cache uitlity added and release for BFT6
 *  \version    1.3 Himamsu Mistral Release for BFT7
 *  \version    1.4 Himamsu Mistral Added Multiple NAND support,
 *                                  - Release for BFT8
 */

#include <ti/sdo/pspdrivers/drivers/nand/src/ddc_nand.h>

/*  Changed for Bad block management bug  */

#pragma DATA_ALIGN(data, CACHE_LINE_LENGTH);
/*  Buffer to write spare area of bad blocks */
static Uint32 data[((DDC_NAND_BB_BYTES_PER_PAGE + DDC_NAND_BB_SPARE_AREA_SIZE)/4)];

/*  Changed for Bad block management bug  - Up to here  */

/**
 *   \brief Flushes the currently written block if any
 *
 *   \param DDC     [IN]    DDC Structure
 *
 *   \return DDC_NAND_E_TIMEOUT             - Operation Timed out        \n
 *           DDC_NAND_E_PAGE_PROGRAM_FAIL   - Page program error         \n
 *           DDC_NAND_E_OUT_OF_FREE_BLOCKS  - No free blocks available   \n
 *             PAL_SOK             - Return Success
 */
PAL_Result nandUtlWriteSectorFlush (
            DDC_NandObj *DDC)
{
    DDC_NandState   *pNandState;
    Uint16              currentPhyBlock;
    PAL_Result          result          = PAL_SOK;
    Uint32              phySector;
    DDC_NandRedundant   oldBuff;
    DDC_NandRedundant   nandRed;
    Uint16              curPage;
    DDC_NandBlockState  blockState;

#ifdef DDC_NAND_BBM
    Uint16          Sector;
#endif /* DDC_NAND_BBM */

    if(NULL != DDC)
    {
       pNandState     = (DDC_NandState *) &DDC->nandState[DDC->bankNo];

       currentPhyBlock = (Uint16)(pNandState->currentPhyWritePage
               / pNandState->pagesPerBlock);

#ifdef DDC_NAND_BBM
       Sector = (Uint16)( pNandState->currentPhyWritePage & \
               (pNandState->pagesPerBlock - 1u));
#endif /* DDC_NAND_BBM */

       if (pNandState->oldPhyBlock != DDC_NAND_DEF_OLD_PHY_BLOCK) {
#ifdef DDC_NAND_POWER_SAFE
           for (curPage = 0; curPage < pNandState->pagesPerBlock; curPage++) {
#else
               /* Copy all the data from old block to new block */
               for (curPage = 0; curPage < pNandState->pagesPerBlock;
                       curPage++) {
#endif /* DDC_NAND_POWER_SAFE */
                   phySector = (curPage + ((Uint32)currentPhyBlock
                               * (Uint32)pNandState->pagesPerBlock));
                   blockState = nandUtlIsFreePage (DDC, phySector);
                   if (DDC_NAND_UNKNOWN_BLOCK == blockState) {
                       return DDC_NAND_E_PAGE_PROGRAM_FAIL;
                   }

                   if (DDC_NAND_FREE_BLOCK == blockState) {
                       /* Read the redundant area of the old physical block */
                       phySector = (curPage + ((Uint32)pNandState->oldPhyBlock
                                   * (Uint32)pNandState->pagesPerBlock));
                       result = nandPhyReadRed(DDC, phySector, &oldBuff);
                       if (PAL_SOK != result)
                       {
                           return (result);
                       }

                       if (DDC_NAND_BLOCK_ADDR_FREE != oldBuff.blockAddress) {
                           /*
                            * Used Block - So we need to read the contents
                            * and copy to the new location */
                           /* Use the same buffer for reading*/
                           phySector = (curPage +
                                   ((Uint32)pNandState->oldPhyBlock *
                                    (Uint32)pNandState->pagesPerBlock));
                           result = nandPhyReadSector(DDC, phySector,
                                   pNandState->pWriteBuffer, &nandRed);
                           if (PAL_SOK != result) {
                               return (result);
                           }

                           phySector = (curPage + ((Uint32)currentPhyBlock *
                                       (Uint32)pNandState->pagesPerBlock));
                           result = nandPhyWriteSector (DDC, phySector,
                                   pNandState->pWriteBuffer, &nandRed, 0);
                           if (DDC_NAND_E_PAGE_PROGRAM_FAIL != result) {
                               if (PAL_SOK != result) {
                                   return (result);
                               }
                           }
#ifdef DDC_NAND_BBM
                           if (DDC_NAND_E_PAGE_PROGRAM_FAIL == result) {
                               do {
                                   /* Mark the old block as bad in the
                                    * BBM Table */
                                    /*  Changed for Bad block management bug  */ 
                                    result = nandUtlUpdateBBMTable (DDC, currentPhyBlock, DDC_NAND_BAD_BLOCK, NULL);
                                    /*  Changed for Bad block management bug  - Up to here  */
                                   if (PAL_SOK != result) {
                                       return result;
                                   }

                                   /* Get the next free block */
                                   if (nandUtlFndNxtFreeBlk (DDC,
                                               (Uint16*)&currentPhyBlock) != PAL_SOK) {
                                       return (DDC_NAND_E_OUT_OF_FREE_BLOCKS);
                                   }

                                   /* Erase the new Block to assure correct
                                    * write operation */
                                   result = nandPhyEraseBlock (DDC,
                                           currentPhyBlock);
                               } while (result == DDC_NAND_E_BLOCK_ERASE_FAIL);

                               pNandState->currentPhyWritePage =
                                   Sector + ((Uint32)currentPhyBlock * (Uint32)pNandState->pagesPerBlock);

                               if ((pNandState->BBMdepth++) >
                                       DDC_NAND_MAX_BBM_DEPTH) {
                                   pNandState->BBMdepth = 0;
                                   return (DDC_NAND_E_OUT_OF_FREE_BLOCKS);
                               }
                               /* Recurse, only if a BAD block is found! */
                               return (nandUtlWriteSectorFlush (DDC));
                           }
#endif /* DDC_NAND_BBM */
                       }
                   }
               }

               /* Erase Old Block and update the table*/
               result = nandPhyEraseBlock (DDC,
                       (Uint32)pNandState->oldPhyBlock);
#ifdef DDC_NAND_BBM
               if (DDC_NAND_E_BLOCK_ERASE_FAIL == result) {
                   /* Mark the old block as bad in the BBM Table */
                   /*   Changed for Bad block management bug  */ 
                   result = nandUtlUpdateBBMTable (DDC, currentPhyBlock, DDC_NAND_BAD_BLOCK, NULL);
                   /*  Changed for Bad block management bug  - Up to here  */
               }
#endif /* DDC_NAND_BBM */

               if (PAL_SOK != result) {
                   return result;
               }

               /* Update the WLA Bit Table */
               result = nandUtlUpdateWLATable (pNandState,
                       pNandState->oldPhyBlock, DDC_NAND_FREE_BLOCK);
               if (PAL_SOK != result) {
                   return result;
               }

               pNandState->oldPhyBlock             = DDC_NAND_DEF_OLD_PHY_BLOCK;
               pNandState->newWriteFlushPending    = 0;
           }
       }

       return (PAL_SOK);
}

/**
 *   \brief Copy back a page of NAND
 *
 *  The copy-back program is configured to quickly and efficiently rewrite
 *  data stored in one page without utilizing an external (software) memory.
 *
 *  <b>NOTE: This is a Big block NAND feature and may not available on all
 *  NAND devices.</b>
 *
 *   \param DDC             [IN]    DDC Structure
 *   \param curPage         [IN]    current page in current physical block
 *   \param currentPhyBlock [IN]    current physical block
 *
 *   \return DDC_NAND_E_PAGE_READ_FAIL      - Read error                 \n
 *           DDC_NAND_E_PAGE_PROGRAM_FAIL   - Page program error         \n
 *           PAL_SOK                        - Return Success             \n
 */
PAL_Result nandUtlCopyBackPage (
            DDC_NandObj         *DDC,
            Uint16              curPage,
            Uint16              currentPhyBlock)
{
    DDC_NandState *pNandState;
    PAL_Result    status = PAL_SOK;
    Bool bool_False = FALSE;

    Uint32 sourceSector = 0;
    Uint32 destSector = 0;

    if(NULL != DDC)
    {
        pNandState = (DDC_NandState *) &DDC->nandState[DDC->bankNo];
        sourceSector = (curPage + ((Uint32)pNandState->oldPhyBlock *
        (Uint32)pNandState->pagesPerBlock));
        nandPhyIssueCommand(DDC, DDC_NAND_CMD_READ_START, sourceSector);
        LLC_nandSendCommand(DDC->llc, DDC_NAND_CMD_COPY_BACK_READ);
        /* Copy back read   */

        do {
            status = nandPhyReadStatus (DDC);
            if (DDC_NAND_E_STATUS_READ_FAIL == status) {
            /* Send 'result' as page program error */
                status = DDC_NAND_E_PAGE_PROGRAM_FAIL;
                break;
            }

            destSector = (curPage + ((Uint32)currentPhyBlock *
                        (Uint32)pNandState->pagesPerBlock));
            nandPhyIssueCommand(DDC, DDC_NAND_CMD_RANDOM_PGRM_START,
                    destSector);
            LLC_nandSendCommand(DDC->llc, DDC_NAND_CMD_PGRM_END);
            /* Program End  */
            status = nandPhyReadStatus (DDC);
            if (DDC_NAND_E_STATUS_READ_FAIL == status) {
            /* Page program ERROR !!! */
                status = DDC_NAND_E_PAGE_PROGRAM_FAIL;
            }
            bool_False = FALSE;
        }while(bool_False);
    }
    return (status);
}

/**
 *   \brief Checks if the page/sector is free or not
 *
 *   \param DDC         [IN]    DDC Structure
 *   \param phySector   [IN]    physical sector number
 *
 *   \return DDC_NandBlockState:                                \n
 *           - DDC_NAND_FREE_BLOCK       - Free block           \n
 *           - DDC_NAND_USED_BLOCK       - Used block           \n
 *           - DDC_NAND_UNKNOWN_BLOCK    - Block State unknown
 */
DDC_NandBlockState  nandUtlIsFreePage (
            DDC_NandObj *DDC,
            Uint32      phySector)
{
    PAL_Result  result;
    DDC_NandBlockState blockState;
    DDC_NandRedundant   nandRed;

    if(NULL != DDC) {
        /* For Pages in a block we need to
         * read the redundant area to check whether
         * the page is free or not
         */

        result = nandPhyReadRed(DDC, phySector, &nandRed);

        if (PAL_SOK != result) {
            blockState = DDC_NAND_UNKNOWN_BLOCK;
        }
        else {
            if (DDC_NAND_BLOCK_ADDR_FREE != nandRed.blockAddress) {
                blockState = DDC_NAND_USED_BLOCK;
            }
            else {
                /* Else return free block */
                blockState = DDC_NAND_FREE_BLOCK;
            }
        }
    }
    return blockState;
}

/**
 *   \brief Find next free block
 *
 *  Finds the next available free block corresponding to the physical block
 *  supplied, by consulting the wear-leveling alogorithm. If the NAND has no
 *  more free blocks, a 'out-of-free' blocks is returned.
 *
 *   \param DDC         [IN]    DDC Structure
 *   \param phyBlock    [IN]    Physical block number
 *
 *   \return DDC_NAND_E_OUT_OF_FREE_BLOCKS  - No free blocks available   \n
 *             PAL_SOK             - Return Success
 */
PAL_Result nandUtlFndNxtFreeBlk (
            DDC_NandObj *DDC,
            Uint16      *phyBlock)
{
    DDC_NandState   *pNandState;
    Uint16          blockWord;
    Uint16          blockBit;
    Uint16          loopCount   = 0;
    Uint16          WLABufferWord;
    Uint16          maxBlockWordPerZone = 64u;
    Uint16          errorHandlerCount   = 0;
    Uint16          Mask;

    if((NULL != DDC) && (NULL != phyBlock))
    {
        blockWord   = (Uint16)((*phyBlock)/16u);
        blockBit    = (Uint16)((*phyBlock)%16u);
        pNandState = (DDC_NandState *) &DDC->nandState[DDC->bankNo];

        if (blockWord == 0) {
            Mask = (Uint16)((1u << (15L-pNandState->CISBlock)) -1u);
    }
    else {
        Mask = 0xFFFF;
    }

#ifdef DDC_NAND_BBM
        WLABufferWord = (Uint16)((pNandState->pWLAbitbuffer[blockWord] &
                         pNandState->pBBMbitbuffer[blockWord] & Mask));
#else
        WLABufferWord = (Uint16)((pNandState->pWLAbitbuffer[blockWord] & Mask));
#endif /* DDC_NAND_BBM */

    /* This code makes sure that wear leveling operations always moves
     * in a forward direction
     */
    WLABufferWord <<= blockBit;
    WLABufferWord >>= blockBit;

    while (WLABufferWord == 0) {
        blockWord++;
        if (blockWord > ((pNandState->writeZone * maxBlockWordPerZone)-1L)) {
            blockWord = (Uint16)((pNandState->writeZone-1L) *
                    maxBlockWordPerZone);
        }

        /* Search for free block in reverse order */
        if (blockWord == 0) {
            Mask = (Uint16)((1u << (15L-pNandState->CISBlock)) -1L);
        }
        else {
            Mask = 0xFFFF;
        }

#ifdef DDC_NAND_BBM
            WLABufferWord = (Uint16)((pNandState->pWLAbitbuffer[blockWord] &
                             pNandState->pBBMbitbuffer[blockWord] & Mask));
#else
            WLABufferWord = (Uint16)((pNandState->pWLAbitbuffer[blockWord]
                        & Mask));
#endif /* DDC_NAND_BBM */
        loopCount++;

        if (errorHandlerCount++ > (maxBlockWordPerZone + 2u)) {
            return (DDC_NAND_E_OUT_OF_FREE_BLOCKS);
        }
    }

    /*  Changed for Bad block management bug  */
    if ((loopCount  < maxBlockWordPerZone) || (WLABufferWord)) {
    /*  Changed for Bad block management bug  - Up to here  */
        /* We Found the word with a free block
         * Reuse this variables LoopCount and blockBit
         */
        loopCount = 0;
        blockBit = (Uint16)((WLABufferWord>>15u) & 0x1);

        /* Now extract the free block bit */
        while (((++loopCount) != 16u) && (0 == blockBit)) {
            blockBit = (Uint16)(( WLABufferWord >> (15L - loopCount) ) & 0x01);
        }

        *phyBlock = (Uint16)((blockWord*16u) + (loopCount - 1u));
        return (PAL_SOK);
    }
    }
    return (DDC_NAND_E_OUT_OF_FREE_BLOCKS);
}

/*  Changed for Bad block management bug  */ 
/**
 *   \brief Update the BBM table
 *
 *  Updates Bad Block Managemet table for the physical block supplied. It also
 *  decrements the number of available pages on NAND, if a bad block is found.
 *
 *   \param DDC            DDC Structure
 *   \param phyBlock       Physical block number,
 *   \param status         Status of the block (good or bad)
 *   \param nandRed     Spare area info of first page of phyBlock or NULL
 *
 *   \return PAL_SOK   - Return Success
 */
PAL_Result nandUtlUpdateBBMTable (
            DDC_NandObj             *DDC,
            Uint16                  phyBlock,
            DDC_NandBlockStatus     status,
            DDC_NandRedundant       *nandRed)
{
    Uint16 blockBit;
    Uint16 bitPos;
    Uint16 mask;
    Uint8 zone;
    DDC_NandState   *pNandState;
    PAL_Result          result;
    Uint16              pageSize;
    DDC_NandRedundant   nandRedData;
    

    blockBit    = (Uint16)(phyBlock >> 4u);
    bitPos      = (Uint16)(phyBlock % 16u);
    mask        = (Uint16)(1u << (15L - bitPos));
    zone        = phyBlock / DDC_NAND_BLOCKS_PER_ZONE;
    if( NULL != DDC) {
        pNandState  = (DDC_NandState *) &DDC->nandState[DDC->bankNo];

        if(NULL != pNandState) {
            if (DDC_NAND_GOOD_BLOCK == status) {
                pNandState->pBBMbitbuffer[blockBit] |= mask;
            }

            if (DDC_NAND_BAD_BLOCK == status) {
                pNandState->pBBMbitbuffer[blockBit] &= (Uint16)(~mask);
                pNandState->noOfBadBlocks++; /* One bad block found! */
                pNandState->noOfBadPages += (Uint32)pNandState->pagesPerBlock;
				pNandState->totalPages -= (Uint32)pNandState->pagesPerBlock;
                if (DDC->state == PSP_DRIVER_STATE_INITIALIZED)
                {
                    if (nandRed == NULL)
                    {
                        result = nandPhyReadRedPage(DDC, (phyBlock * pNandState->pagesPerBlock), &nandRedData);
                        if (PAL_SOK != result) {
                            return (DDC_NAND_UNKNOWN_BLOCK);
                        }
                        nandRed = &nandRedData ;
            }
                    if(DDC->llc->eraseAtInit != 0)
                    {
                        if(pNandState->totalPages < pNandState->pagesPerBlock)
                        {
                            pNandState->totalPages = 0 ;
                        }
                        else
                        {
                            pNandState->totalPages -= pNandState->pagesPerBlock;
                        }
                        if (pNandState->usableBlockInZone[zone])
                        {
                            (pNandState->usableBlockInZone[zone])--;
                        }

                        /* Check if this block goes bad in last run */
                        if(nandRed->blockStatus == 0xFFAA)
                        {
                            /* Write 0x00 data in to first sector of bad block */
                            /* Erase old Block */
                            result = nandPhyEraseBlock (DDC, phyBlock);

                            if (DDC_NAND_SMALL_BLOCK == pNandState->nandType)
                            {
                                /* Issue spare area cmd to set Page Pointer */
                                LLC_nandSendCommand(DDC->llc, DDC_NAND_CMD_SPARE_AREA);
                                /* Write 0xAAAA in the Bad Block Redundant Area */
                                pageSize = pNandState->spareAreaSize;
                            }
                            else
                            {
                                /* Issue CMD0 to set Page Pointer */
                                LLC_nandSendCommand(DDC->llc, DDC_NAND_CMD_LO_PAGE);
                                /* Write 0xAAAA in the Bad Block Data & Redundant Area */
                                pageSize = pNandState->bytesPerPage + pNandState->spareAreaSize;
                            }

                            nandPhyIssueCommand(DDC, DDC_NAND_CMD_PGRM_START, (phyBlock*pNandState->pagesPerBlock));

                            nandPhyWriteNBytes (DDC->llc, pageSize, NULL, NULL, 0);

                            /* Issue write commit */
                            LLC_nandSendCommand(DDC->llc, DDC_NAND_CMD_PGRM_END);
                            /* Check the status of the write operation */
                            result = nandPhyReadStatus (DDC);
                            if (PAL_SOK != result) {
                                return status;
                            }
                        }
                    }
                    else
                    {
                        /* From spare area of bad block, identify bad block is generated after
                           erase and format or before that
                         */
                        if((nandRed->blockStatus != DDC_NAND_BLOCK_STATUS_FREE) && (nandRed->blockStatus != 0xFFAA))
                        {
                if(pNandState->totalPages < pNandState->pagesPerBlock)
                {           
                    pNandState->totalPages = 0 ;
                }
                else
                {
                    pNandState->totalPages -= pNandState->pagesPerBlock;
                }
                if (pNandState->usableBlockInZone[zone])
                {
                    (pNandState->usableBlockInZone[zone])--;
                }
            }
        }
    }
            }
        }
    }
    return (PAL_SOK);
}

/**
 *   \brief Update physical to logical mapping buffer.
 *
 *  Updates the phyical to logical bit mapping buffer with entry specified in
 *  phyical block and logical block. The API also updates the WLA buffer
 *  accordingly.
 *
 *   \param DDC         [IN]    DDC Structure
 *   \param logBlock    [IN]    logical block number
 *   \param phyBlock    [IN]    physical block number
 *
 *   \return DDC_NAND_INVALID_PHYBLOCK  - Invalid Physical block         \n
 *           PAL_Error                  - in case of error               \n
 *            PAL_SOK     - Return Success
 */
PAL_Result nandUtlUpdateEntryTable (
            DDC_NandObj *DDC,
            Uint16      logBlock,
            Uint16      phyBlock)
{

    Uint16          loop = 0;
    Uint16          j;
    Uint16          Priority    = 0xE000;
    Uint16          PBA;
#ifdef DDC_NAND_POWER_SAFE
    PAL_Result      result;
#endif /* DDC_NAND_POWER_SAFE */
    DDC_NandState   *pNandState;

    if(NULL != DDC) {
       pNandState     = (DDC_NandState *) &DDC->nandState[DDC->bankNo];

       if (pNandState->pPhyLogEntries != NULL) {
           if ((pNandState->entryBufferSize >= (128u * pNandState->numZones)) &&
                   (pNandState->entryBufferSize <
                    (625u * pNandState->numZones))) {

               Uint16 offset = (Uint16)(logBlock >> 4u);
               Uint16 bitpos = (Uint16)(logBlock % 16u);
               Uint16 Index = 0;

               /* We found the logical block. Now get the physical
                * block Index */
               for (loop = 0; loop < offset; loop++) {
                   for (j = 16u; j > 0; j--) {
                       if (((Uint16)(pNandState->pMappingbitbuffer[loop]
                                >> (j - 1)) & 1u) == 1) {
                           Index++;
                       }
                   }
               }

               for (j = 16u; j > (16u - bitpos); j--) {
                   if (((Uint16)(pNandState->pMappingbitbuffer[loop]
                                >> (j - 1)) & 1) == 1) {
                       Index++;
                   }
               }

               pNandState->pPhyLogEntries[Index] =
                   (Uint16)(phyBlock | Priority);
           }
           else {
                if ((pNandState->entryBufferSize <
                       (128u * pNandState->numZones))
                   && (pNandState->entryBufferSize >= 2)) {
               for (loop = 0; loop < pNandState->entryBufferSize; loop+= 2) {
                   if (pNandState->pPhyLogEntries[loop] == logBlock) {
                       pNandState->pPhyLogEntries[loop + 1] = phyBlock;
                       return (PAL_SOK);
                   }
               }
           }
           else {
               if (pNandState->entryBufferSize >=
                       (625u * pNandState->numZones)) {
                   PBA = nandUtlPackUnpackEntries (
                           pNandState,
                           logBlock,
                           (Uint16)(phyBlock & 0x3FF));

                   if (PBA == DDC_NAND_INVALID_PHYBLOCK) {
                       return (DDC_NAND_INVALID_PHYBLOCK);
                   }
#ifdef DDC_NAND_POWER_SAFE
                   if (((PBA & 0x3FF) != 0x3FF)
                           && (pNandState->skipWrite == 0xDEAD)) {
                       result = nandPhyEraseBlock (DDC, PBA);
                       if (PAL_SOK != result) {
                           return result;
                       }

                       result = nandUtlUpdateWLATable (
                               pNandState,
                               PBA,
                               DDC_NAND_FREE_BLOCK);
                       if (PAL_SOK != result) {
                           return result;
                       }
                   }
#endif /* DDC_NAND_POWER_SAFE */
               }
                }
           }
       }
    }
    return (PAL_SOK);
}

/**
 *   \brief Find physical block from Mapping table.
 *
 *  This function gets the Physical block number corresponding to the logical
 *  block
 *  from the Bit Mapping Table
 *
 *   \param pNandState  [IN]    Nand media state variable
 *   \param logBlock    [IN]    logical block number
 *
 *   \return DDC_NAND_INVALID_PHYBLOCK  - Invalid physical block         \n
 *           Physical Block Number      - on success
 */
Uint16 nandUtlFindPhyBlock (
            DDC_NandState   *pNandState,
            Uint16          logBlock)
{
    Uint16  Index   = 0;
    Uint16  i;
    Uint16  j;
    Uint16  offset  = (Uint16)(logBlock >> 4);
    Uint16  bitpos  = (Uint16)(logBlock % 16);
    Uint16  Mask    = (Uint16)(1 << (15L - bitpos));
    Uint16  result;

    if(NULL != pNandState) {
        if ((pNandState->pMappingbitbuffer[offset] & Mask) == Mask) {
            /* We found the logical block. Now get the physical block Index */
            for (i = 0; i < offset; i++) {
                for (j = 16; j > 0; j--) {
                    if ((((Uint16)(pNandState->pMappingbitbuffer[i] >>
                                  (j - 1)) & 1) == 1)) {
                        Index++;
                    }
                }
            }

            for (j = 16; j > (16 - bitpos); j--) {
                if (((Uint16)(pNandState->pMappingbitbuffer[i] >>
                              (j - 1)) & 1) == 1) {
                    Index++;
                }
             }

            /* We found the index to be inserted */
            result = (Uint16)((pNandState->pPhyLogEntries[Index]) & 0x1FFF);
        }
        else {
            result = DDC_NAND_INVALID_PHYBLOCK;
        }
    }
    return result;
}

/**
 *   \brief Insert entry into the mapping buffer.
 *
 *  This function Inserts one entry to the scalable buffer and updates
 *  the bit mapping table
 *
 *   \param pNandState  [IN]    Nand media state variable
 *   \param logBlock    [IN]    logical block number
 *   \param phyBlock    [IN]    physical block number
 *
 *   \return DDC_NAND_E_OUT_OF_FREE_BLOCKS  - No free blocks available   \n
 *             PAL_SOK                - Return Success
 */
PAL_Result nandUtlInsertEntry (
            DDC_NandState   *pNandState,
            Uint16          logBlock,
            Uint16          phyBlock)
{
    Uint16  Index       = 0;
    Uint16  priority    = 1 <<13;
    Uint16  prev_entry  = 0xFFFF;   /* we need a signed number for this */
    Uint16  offset      = (Uint16)(logBlock >> 4);
    Uint16  bitpos      = (Uint16)(logBlock % 16);
    Uint16  flag        = 0;

    Uint16  oldLogBlock;
    Uint16  numentries;
    Uint16  entry;
    Uint16  i;
    Uint16  j;
    Uint16  k;
    Uint16  l;

    PAL_Result  result;

    if(NULL != pNandState) {
        /* We found the logical block. Now get the physical block Index */
        for (i = 0; i < offset; i++) {
            for (j = 16; j > 0; j--) {
                if (((((Uint16)(pNandState->pMappingbitbuffer[i] >>
                               (j - 1))) & 1) == 1)) {
                    Index++;
                }
            }
        }

        for (j = 16; j > (16 - bitpos); j--) {
            if (((((Uint16)(pNandState->pMappingbitbuffer[i] >>
                           (j - 1))) & 1) == 1)) {
                Index++;
            }
        }

        /* We found the index to be inserted */

        /* Find the one with lowest entry value*/
        j = 0;

        for (i = 0; i < (pNandState->entryBufferSize -
                    (64u * pNandState->numZones)); i++) {
            entry = pNandState->pPhyLogEntries[i];
            if (entry == 0xFFFFu) {
                prev_entry = 0xDEADu;
                j = i;
                break;
            }

            if (entry < prev_entry) {
                prev_entry = entry;
                j = i;
            }

            /* Reduce one priority level */
            if (entry > priority) {
                entry -= priority;
            }
        }

        if (Index == (pNandState->entryBufferSize
                    - (64u * pNandState->numZones))) {
            Index--;
        }

        if (j >= Index) {
            for (i = j; i > Index; i--) {
                pNandState->pPhyLogEntries[i] =pNandState->pPhyLogEntries[i-1];
            }
        }
        else {
            if (Index != ((pNandState->entryBufferSize
                            - (64 * pNandState->numZones))-1)) {
                Index--;
            }

            for (i = j; i < Index; i++) {
                pNandState->pPhyLogEntries[i] =
                    pNandState->pPhyLogEntries[i + 1];
            }
        }

        if (prev_entry != 0xDEAD) {
            if (pNandState->numZones == 1) {
                numentries = 62;
            }
            else {
                numentries = (Uint16)(125 * (pNandState->numZones >> 1));
            }

            /* We need to find the Logical block number which is replaced */
            oldLogBlock = 0;

            for (k = 0; k < numentries; k++) {
                for (l = 16; l > 0; l--) {
                    if ((((Uint16)(pNandState->pMappingbitbuffer[k] >>
                                   (l - 1))) & 1) == 1) {
                        if (j == 0) {
                            flag = 1;
                            break;
                        }
                        j--;
                    }
                    oldLogBlock++;
                }
                if (flag == 1) {
                    break;
                }
            }

            if ((pNandState->numZones == 1) &&((k == 62) && (l == 0))) {
                for (l = 16; l > 8; l--) {
                    if ((((Uint16)(pNandState->pMappingbitbuffer[k] >>
                                   (l-1))) & 1) == 1) {
                        if ((j--) == 0) {
                            break;
                        }
                    }
                    oldLogBlock++;
                }

                if (l == 8) {
                    /* Fatal Error */
                    return (DDC_NAND_E_OUT_OF_FREE_BLOCKS);
                }
            }

            /* Update the bit position of the old logical block */
            result = nandUtlUpdateMappingBitBuffer (
                    pNandState,
                    oldLogBlock,
                    DDC_NAND_ENTRY_OUT);
            if (PAL_SOK != result) {
                return result;
            }

            for (i = 0; i < (pNandState->entryBufferSize
                        - (64 * pNandState->numZones)); i++) {
                if (pNandState->pPhyLogEntries[i] > priority) {
                    pNandState->pPhyLogEntries[i] -= priority;
                }
            }
        }

        /* Update the bit position of the current logical block*/
        result = nandUtlUpdateMappingBitBuffer (
                pNandState,
                logBlock,
                DDC_NAND_ENTRY_IN);
        if (PAL_SOK != result) {
            return result;
        }

        pNandState->pPhyLogEntries[Index] = phyBlock;
    }

    return (PAL_SOK);
}

/**
 *   \brief Find the physical block for the logical block
 *
 *  Finds the Physical Block for the corresponding logical block from the
 *  bit mapping buffer. If the entry is not found in the buffer, the physical
 *  block is found by reading the redundant area of the NAND. If no physical
 *  block is found, an invalid block is returned
 *
 *   \param DDC         [IN]    DDC Structure
 *   \param logBlock    [IN]    logical block number
 *
 *   \return DDC_NAND_INVALID_PHYBLOCK  - incase of error.               \n
 *           Physical block address     - on success                     \n
 */
Uint16 nandUtlFindPhysicalBlock (
            DDC_NandObj     *DDC,
            Uint32          logBlock)
{
    Uint16          i           = 0;
    Uint16          blockAddr   = 0;
    Uint16          Priority    = 0xE000;
    DDC_NandState   *pNandState;
    DDC_NandRedundant   nandRed;
    PAL_Result      result;
    Uint32          phySector;
    Uint16          phyBlock;

    Uint16          blockBit;
    Uint16          bitPos;
    Uint16          Mask;

#ifdef DDC_NAND_BBM
    Uint16 parity;
    Uint16 j;
#endif /* DDC_NAND_BBM */

    if(NULL != DDC) {
        pNandState = (DDC_NandState *) &DDC->nandState[DDC->bankNo];
        if ((pNandState->pPhyLogEntries != NULL)
                && (pNandState->entryBufferSize >=
                    (625u * pNandState->numZones))) {
            Uint16 Zone = (Uint16)(logBlock / DDC_NAND_USABLE_BLOCKS_PER_ZONE);

            phyBlock = nandUtlPackUnpackEntries (
                    pNandState,
                    (Uint16)logBlock,
                    DDC_NAND_INVALID_PHYBLOCK);

            if (DDC_NAND_INVALID_PHYBLOCK == phyBlock) {
                return (DDC_NAND_INVALID_PHYBLOCK);
            }

            if (1023 == phyBlock) {
                /* we don't know whether '1023' means free block or
                 * last used block in
                 * the zone. If it is the last used block in the zone, the
                 * corresponding bit in the WLABitBuffer will be set
                 */
                /* Check whether this is a used block */
                blockBit = (Uint16)((phyBlock >> 4) + (Zone * 64));
                bitPos   = (Uint16)(phyBlock % 16);
                Mask     = (Uint16)(1 << (15L - bitPos));

                if ((pNandState->pWLAbitbuffer[blockBit] & Mask) == 1) {
                    /* It's not a used block, return the next free block */
                    phyBlock = (Uint16)((pNandState->currentPhyWritePage)
                            / pNandState->pagesPerBlock);
                    result = nandUtlFndNxtFreeBlk (DDC, (Uint16*)&phyBlock);
                    if (PAL_SOK != result) {
                        return (DDC_NAND_INVALID_PHYBLOCK);
                    }
                    else {
                        return (phyBlock);
                    }
                }
                else {
                    result = nandUtlWriteSectorFlush (DDC);
                    if (PAL_SOK != result) {
                        return (DDC_NAND_INVALID_PHYBLOCK);
                    }

                    phySector = (Uint32)(((Zone * DDC_NAND_BLOCKS_PER_ZONE) + phyBlock)
                        * pNandState->pagesPerBlock);
                    result = nandPhyReadRed(DDC, phySector, &nandRed);
                    if (PAL_SOK != result) {
                        return (Uint16)result;
                    }

                    i = nandRed.blockAddress;

                    blockAddr = nandUtlCalcBlkAdrs (
                            (Uint16)(logBlock %
                                     DDC_NAND_USABLE_BLOCKS_PER_ZONE));

                    if (nandRed.blockAddress != blockAddr) {
                        phyBlock = (Uint16)((pNandState->currentPhyWritePage
                                    / pNandState->pagesPerBlock));
                        result = nandUtlFndNxtFreeBlk (DDC, (Uint16*)&phyBlock);
                        if (PAL_SOK != result) {
                            return (DDC_NAND_INVALID_PHYBLOCK);
                        }
                        else {
                            return (phyBlock);
                        }
                    }
                }
            }

            phyBlock += (Zone * DDC_NAND_BLOCKS_PER_ZONE);
            return (phyBlock);
        }

        if (pNandState->pPhyLogEntries != NULL) {
            if ((pNandState->entryBufferSize < (625u * pNandState->numZones))
                    && (pNandState->entryBufferSize >=
                        (128u * pNandState->numZones))) {

                /* check whether logical Block is updated */
                phyBlock = nandUtlFindPhyBlock (
                        pNandState,
                        (Uint16)logBlock);

                if (DDC_NAND_INVALID_PHYBLOCK != phyBlock) {
                    return (phyBlock);
                }
                else {
                    blockAddr = 1;
                }
            }
            else {
                if ((pNandState->entryBufferSize <
                            (128u * pNandState->numZones)) &&
                        (pNandState->entryBufferSize >= 2)) {

                    for (i = 0; i< pNandState->entryBufferSize; i+= 2) {
                        if (pNandState->pPhyLogEntries[i] == logBlock) {
                            phyBlock = pNandState->pPhyLogEntries[ i + 1];
                            break;
                        }
                    }

                    if ((i < pNandState->entryBufferSize) && (i != 0)) {
                        Uint16 temp1;
                        Uint16 temp2;
                        if (i == (pNandState->entryBufferSize - 2)) {
                            return (phyBlock);
                        }

                        temp1 = pNandState->pPhyLogEntries[i];
                        temp2 = pNandState->pPhyLogEntries[ i + 1];

                        pNandState->pPhyLogEntries[i]     =
                            pNandState->pPhyLogEntries[i + 2];

                        pNandState->pPhyLogEntries[i + 1] =
                            pNandState->pPhyLogEntries[i + 3];

                        pNandState->pPhyLogEntries[i + 2] = temp1;
                        pNandState->pPhyLogEntries[i + 3] = temp2;

                        return (phyBlock);
                    }
                    else {
                        for (i = 0; i < (pNandState->entryBufferSize - 2);
                                i+= 2) {
                            pNandState->pPhyLogEntries[i] =
                                pNandState->pPhyLogEntries[i + 2];
                            pNandState->pPhyLogEntries[i + 1] =
                                pNandState->pPhyLogEntries[i + 3];
                            blockAddr = 1u;
                        }
                    }
                }
            }
        }

        if ((NULL == pNandState->pPhyLogEntries) || (blockAddr != 0)) {
            Uint16 zone = (Uint16)(logBlock / DDC_NAND_USABLE_BLOCKS_PER_ZONE);

            /* we need to find the corresponding physical sector
             * from the Nand */
            for (phyBlock = DDC_NAND_BLOCKS_PER_ZONE; phyBlock > 0;
                    phyBlock--) {
                phySector = (((Uint32)phyBlock + ((Uint32)zone *
                                DDC_NAND_BLOCKS_PER_ZONE)) - 1) *
                    (Uint32)pNandState->pagesPerBlock;
                result = nandPhyReadRed(DDC, phySector, &nandRed);
                if (PAL_SOK != result) {
                    return (Uint16)result;
                }

                if (nandRed.blockAddress != DDC_NAND_BLOCK_ADDR_FREE) {
#ifdef DDC_NAND_BBM
                    parity = 0;
                    for (j = 0; j < 16; j++) {
                        parity ^= ((nandRed.blockAddress & (1 << j )) >> j);
                    }

                    if (!
                            (((((nandRed.blockAddress & 0xF800) != 0x1000) ||
                             (((nandRed.blockStatus  & 0xFF)   != 0xFF)
                              && (pNandState->bytesPerPage <=
                                  DDC_NAND_SB_BYTES_PER_PAGE))) ||
                             (((nandRed.logPageNo    & 0xFF00) != 0xFF00)
                              && (pNandState->bytesPerPage >
                                  DDC_NAND_SB_BYTES_PER_PAGE))) ||
                             (parity != 0))) {
#endif /* DDC_NAND_BBM */
                        if (((Uint16)(nandRed.blockAddress >> 1) & 0x3FF) ==
                                (logBlock % DDC_NAND_USABLE_BLOCKS_PER_ZONE)) {
                            if (blockAddr != 0) {
                                if ((pNandState->entryBufferSize <
                                            (128u * pNandState->numZones))
                                        && (pNandState->entryBufferSize >= 2)) {

                                    pNandState->pPhyLogEntries
                                        [pNandState->entryBufferSize - 2] =
                                        (Uint16)logBlock;
                                    pNandState->pPhyLogEntries
                                        [pNandState->entryBufferSize-1] =
                                        (Uint16)((phyBlock +
                                         (zone * DDC_NAND_BLOCKS_PER_ZONE))- 1);
                                }
                                else {
                                    if ((pNandState->entryBufferSize <
                                                (625u * pNandState->numZones))
                                            &&
                                     (pNandState->entryBufferSize >=
                                      (128u * pNandState->numZones))) {

                                        result = nandUtlInsertEntry (
                                                pNandState,
                                                (Uint16)logBlock,
                                                (Uint16)(((phyBlock +
                                     (zone * DDC_NAND_BLOCKS_PER_ZONE)) - 1)
                                                         | Priority));
                                        if (PAL_SOK != result) {
                                            return (Uint16)result;
                                        }
                                    }
                                }
                            }

                            return (Uint16)((phyBlock +
                                        (zone * DDC_NAND_BLOCKS_PER_ZONE)) - 1);
                        }
#ifdef DDC_NAND_BBM
                    }
#endif /* DDC_NAND_BBM */
                }
            }

            /* If we are here then we haven't found the Log Block */
            phyBlock = (Uint16)((pNandState->currentPhyWritePage
                        / pNandState->pagesPerBlock));
            result = nandUtlFndNxtFreeBlk (DDC, (Uint16*)&phyBlock);
            if (PAL_SOK != result) {
                return (DDC_NAND_INVALID_PHYBLOCK);
            }

            if (blockAddr != 0) {
                if ((pNandState->entryBufferSize <
                            (128u * pNandState->numZones)) &&
                        (pNandState->entryBufferSize >= 2)) {

                    pNandState->pPhyLogEntries[pNandState->entryBufferSize - 2]
                        = (Uint16)logBlock;
                    pNandState->pPhyLogEntries[pNandState->entryBufferSize - 1]
                        = phyBlock;
                }
                else {
                    if ((pNandState->entryBufferSize <
                                (625u * pNandState->numZones)) &&
                            (pNandState->entryBufferSize >=
                             (128u * pNandState->numZones))) {

                        result = nandUtlInsertEntry (
                                pNandState,
                                (Uint16)logBlock,
                                (Uint16)(phyBlock | Priority));
                        if (PAL_SOK != result) {
                            return (Uint16)result;
                        }
                    }
                }
            }
            return (phyBlock);
        }
    }
    return (0);
}

/**
 *   \brief Updates the Mapping bit buffer
 *
 *  This function updates the bit mapping buffer when a particular entry is
 *  either mapped out or mapped in.
 *
 *   \param pNandState  [IN]    Nand media state variable
 *   \param logBlock    [IN]    logical block number,
 *   \param entry       [IN]    in/out
 *
 *   \return PAL_SOK         - Return Success
 */
PAL_Result nandUtlUpdateMappingBitBuffer (
            DDC_NandState           *pNandState,
            Uint16                  logBlock,
            DDC_NANDLogBlockStatus  entry)
{
    Uint16 blockBit;
    Uint16 bitPos;
    Uint16 Mask;

    blockBit    = (Uint16)(logBlock >> 4u);
    bitPos      = (Uint16)(logBlock % 16u);
    Mask        = (Uint16)(1 << (15L - bitPos));

    if( NULL != pNandState) {
        if (DDC_NAND_ENTRY_IN == entry) {
            pNandState->pMappingbitbuffer[blockBit] |= Mask;
        }

        if (DDC_NAND_ENTRY_OUT == entry) {
            pNandState->pMappingbitbuffer[blockBit] &= (Uint16)(~Mask);
        }
    }

    return (PAL_SOK);
}

/**
 *   \brief Mark the physical block as bad.
 *
 *  This API marks a physical block as bad. It starts by updating the entry
 *  table corresponding to the logical block. It procees to find a free
 *  physical block and copies all the contents from the old block to the new
 *  block, monitoring if the new physical block for a failure. In such an
 *  extreme case, it loops till all the data is succesfully written to a new
 *  good block. On a succesful operation the old block is erased and mapped
 *  out. The BBM table and bit mapping tabes are updated as required.
 *
 *   \param DDC         [IN]    DDC Structure
 *   \param logBlock    [IN]    logical block number,
 *   \param phyBlock    [IN]    physical block number,
 *   \param result      [IN]    nand error status
 *
 *   \return PAL_Error        - in case of error                         \n
 *           PAL_SOK          - Return Success                           \n
 */
PAL_Result nandUtlMarkBadBlock (
            DDC_NandObj *DDC,
            Uint16      logBlock,
            Uint16      phyBlock,
            PAL_Result  result)
{
    DDC_NandRedundant  nandRed;
    DDC_NandState       *pNandState;
    PAL_Result          status; /* NOTE: 'result', is a function parameter. */
    Uint16              pageSize;

    if(NULL != DDC) {
        pNandState = (DDC_NandState *) &DDC->nandState[DDC->bankNo];

        /* Update the Logical to Physical Entry Table */
        status = nandUtlUpdateEntryTable (DDC, logBlock, DDC_NAND_INVALID_PHYBLOCK);
        if (DDC_NAND_E_PAGE_PROGRAM_FAIL != status) {
            if (PAL_SOK != status) {
                return status;
            /*  Changed for Bad block management bug  - Up to here  */
            }
        }

        if (DDC_NAND_E_PAGE_PROGRAM_FAIL == result) {

            Uint16 newPhyBlock = phyBlock;
            Uint16 nandPage;

            while (DDC_NAND_E_PAGE_PROGRAM_FAIL == result) {
                result = PAL_SOK;
                status = nandUtlFndNxtFreeBlk (DDC, (Uint16*)&newPhyBlock);
                if (PAL_SOK != status) {
                    return status;
                }

            for (nandPage=0; nandPage < pNandState->pagesPerBlock; nandPage++) {
                /* Copy the already written contents to the new Block */
                status = nandPhyReadSector (DDC,
                                   nandPage + ((Uint32)phyBlock *
                                       (Uint32)pNandState->pagesPerBlock),
                                   pNandState->pWriteBuffer,
                                   &nandRed);
                if (PAL_SOK != status) {
                    return status;
                }

                if (nandRed.blockAddress < DDC_NAND_USABLE_BLOCKS_PER_ZONE) {
                    /* Used Block */
                    result = nandPhyWriteSector (DDC,
                            nandPage + ((Uint32)newPhyBlock *
                                (Uint32)pNandState->pagesPerBlock),
                            pNandState->pWriteBuffer,
                            &nandRed,
                            0);

                        /*  Changed for Bad block management bug*/
                        pNandState->currentLogBlock = logBlock;
                        pNandState->currentPhyWritePage = nandPage + (Uint32)newPhyBlock *
                                                            (Uint32)pNandState->pagesPerBlock;
                        /*  Changed for Bad block management bug  - Up to here  */

                    if (DDC_NAND_E_PAGE_PROGRAM_FAIL == result) {
                        /*  Changed for Bad block management bug  */ 
                        status = nandUtlUpdateBBMTable ( DDC, 
                                                         newPhyBlock, 
                                                             DDC_NAND_BAD_BLOCK,
                                                             NULL);
                        /* Changed for Bad block management bug - Up to here  */

                        if (PAL_SOK != status) {
                            return status;
                        }
                        break;/* Break from the for loop */
                    } /* End if */
                        else {
                            /*  Changed for Bad block management bug  */
                           if(PAL_SOK == result){
                                continue;
                            }
                            else{/* If anything other than PAGE PROGRAM FAIL... */
                                return result;
                            }
                            /*  Changed for Bad block management bug - Up to here */
                        }
                } /* End if */
            } /* End for */
            } /* End while */

            /*  Changed for Bad block management bug */
            if (pNandState->newWriteFlushPending) {
                status = nandUtlWriteSectorFlush (DDC);
                if (PAL_SOK != status)
                    return (status);
            }
            /*  Changed for Bad block management bug  - Up to here  */

            /* If we are here then no error occured during Map out
             * Update the entry with new PhyBlock number
             */
            status = nandUtlUpdateEntryTable (DDC, logBlock, newPhyBlock);
            if (PAL_SOK != status) {
                return status;
        }

            /*  Changed for Bad block management bug  */
            /* Update WLA bit buffer as well to mark new block as used block
             */
             status = nandUtlUpdateWLATable (pNandState,newPhyBlock,DDC_NAND_USED_BLOCK);
            /* Changed for Bad block management bug - Up to here */

        /* Mark the old block as bad in the BBM Table */
        /*  Changed for Bad block management bug  */ 
            status = nandUtlUpdateBBMTable (DDC, phyBlock, DDC_NAND_BAD_BLOCK, NULL);
        /*  Changed for Bad block management bug  - Up to here  */
            if (PAL_SOK != status) {
                return status;
            }

            /* Erase old Block */
            status = nandPhyEraseBlock (DDC, phyBlock);
            /*  Changed for Bad block management bug  */
            if (DDC_NAND_SMALL_BLOCK == pNandState->nandType)
            {
                /* Issue spare area cmd to set Page Pointer */
                LLC_nandSendCommand(DDC->llc, DDC_NAND_CMD_SPARE_AREA);
                /* Write 0xAAAA in the Bad Block Redundant Area */
                pageSize = pNandState->spareAreaSize;
            }
            else
            {
            /* Issue CMD0 to set Page Pointer */
            LLC_nandSendCommand(DDC->llc, DDC_NAND_CMD_LO_PAGE);
                /* Write 0xAAAA in the Bad Block Data & Redundant Area */
                pageSize = pNandState->bytesPerPage + pNandState->spareAreaSize;
            }

            nandPhyIssueCommand(DDC,
                    DDC_NAND_CMD_PGRM_START,
                    (phyBlock*pNandState->pagesPerBlock));

            /* Fill pattern to recognize bad blocks */
            PAL_osMemSet (data, 0xAA, pageSize);
            nandPhyWriteNBytes (DDC->llc, pageSize, (Uint16*)data, NULL, 0);
             /*  Changed for Bad block management bug  - Up to here  */

            /* Issue write commit */
            LLC_nandSendCommand(DDC->llc, DDC_NAND_CMD_PGRM_END);

            /* Check R/Bn line */
            LLC_checkCommandStatus(DDC->llc);

            /* Check the status of the write operation */
            status = nandPhyReadStatus (DDC);
            if (PAL_SOK != status) {
                return status;
            }
        }
        else {
            /*  Changed for Bad block management bug  */ 
            status = nandUtlUpdateBBMTable ( DDC, 
                                             phyBlock, 
                                             DDC_NAND_BAD_BLOCK,
                                             NULL);
            /*  Changed for Bad block management bug  - Up to here  */
            if (PAL_SOK != status) {
                return status;
            }
        }
    }
    return (PAL_SOK);
}

/**
 *   \brief Packs and Unpacks the entries in the entry table
 *
 *   \param pNandState  [IN]    Nand media state variable
 *   \param logBlock    [IN]    logical block number
 *   \param phyBlock    [IN]    physical block number
 *
 *   \return Entry
 */
Uint16 nandUtlPackUnpackEntries (
            DDC_NandState   *pNandState,
            Uint16          logBlock,
            Uint16          phyBlock)
{
    Uint16 blockBit;
    Uint16 BitOffset;
    Uint16 OverLapBit;
    Uint16 Entry;
    Uint32 temp;
    Uint16 Mask;

    if( NULL != pNandState) {
        temp     = ((Uint32)logBlock * 10u) / 16u;
        blockBit = (Uint16)temp;

        temp        = ((Uint32)logBlock * 10u) % 16u;
        BitOffset   = (Uint16)temp;
        Entry       = 0x3FF;

        switch (logBlock % 8) {
            case 1:
                OverLapBit = 1;
                break;

            case 3:
                OverLapBit = 1;
            break;

            case 4:
                OverLapBit = 1;
                break;

            case 6:
                OverLapBit = 1;
                break;

            default:
                OverLapBit = 0;
        }

        if (OverLapBit == 1) {
            if (DDC_NAND_INVALID_PHYBLOCK == phyBlock) {
                /* We need to unpack and return the phyBlock */
                /* Reuse the OverlapBit variable */
                OverLapBit = (Uint16)(BitOffset - 6u);

                Entry  =  (Uint16)(pNandState->pPhyLogEntries[blockBit]
                        << OverLapBit);
                Entry |=  (Uint16)(pNandState->pPhyLogEntries[blockBit + 1]
                        >> (16L - OverLapBit));

                return (Uint16)(Entry & 0x3FF);
            }
            else {/* We need to pack the given phyBlock */
                /* Reuse the OverlapBit variable */
                OverLapBit = (Uint16)(BitOffset - 6u);

#ifdef DDC_NAND_POWER_SAFE
                /* Reuse the OverlapBit variable */
                OverLapBit = (Uint16)(BitOffset - 6);

                Entry  =  (Uint16)(pNandState->pPhyLogEntries[blockBit]
                        << OverLapBit);
                Entry |=  (Uint16)(pNandState->pPhyLogEntries[blockBit+1]
                        >> (16 - OverLapBit));
                Entry &= 0x3FFu;
                Entry += (Uint16)((logBlock / DDC_NAND_USABLE_BLOCKS_PER_ZONE)
                        * DDC_NAND_BLOCKS_PER_ZONE);
#endif /* DDC_NAND_POWER_SAFE */

                Mask = (Uint16)(~((1 << (10 - OverLapBit)) - 1));
                pNandState->pPhyLogEntries[blockBit] &= Mask;
                pNandState->pPhyLogEntries[blockBit] |=
                    (Uint16)(phyBlock >> OverLapBit);

                Mask = (Uint16)((1 << (16 - OverLapBit)) - 1);
                pNandState->pPhyLogEntries[blockBit+1] &= Mask;
                pNandState->pPhyLogEntries[blockBit+1] |=
                    (Uint16)(phyBlock << (16 - OverLapBit));
            }
        }
        else {
            if (DDC_NAND_INVALID_PHYBLOCK == phyBlock) {
                OverLapBit = (Uint16)(6 - (BitOffset % 10));
                Entry =  (Uint16)(pNandState->pPhyLogEntries[blockBit]
                        >> OverLapBit);
                return (Uint16)(Entry & 0x3FF);
            }
            else {

                OverLapBit = (Uint16)(16 - (BitOffset % 10));
                Mask = (Uint16)((1 << OverLapBit) - 1);

#ifdef DDC_NAND_POWER_SAFE
                OverLapBit = (Uint16)(6 - (BitOffset % 10));
                Entry =  (Uint16)(pNandState->pPhyLogEntries[blockBit]
                        >> OverLapBit);
                Entry &= 0x3FFu;
                Entry += (Uint16)((logBlock / DDC_NAND_USABLE_BLOCKS_PER_ZONE)
                        * DDC_NAND_BLOCKS_PER_ZONE);
#endif /* DDC_NAND_POWER_SAFE */
                OverLapBit = (Uint16)(6 - (BitOffset % 10));
                Mask &= (Uint16)(~((1 << OverLapBit) - 1));

                pNandState->pPhyLogEntries[blockBit] &= (Uint16)(~Mask);
                pNandState->pPhyLogEntries[blockBit] |=
                    (Uint16)(phyBlock << OverLapBit);
            }
        }
    }

#ifdef DDC_NAND_POWER_SAFE
    return (Entry);
#else
    return (DDC_NAND_INVALID_PHYBLOCK);
#endif /* DDC_NAND_POWER_SAFE */
}


/**
 *   \brief Updates the Free/Used block WLA table
 *
 *   \param pNandState  [IN]    Nand media state variable
 *   \param phyBlock    [IN]    physical block number,
 *   \param bState      [IN]    state of the block (free / used)
 *
 *   \return PAL_SOK      - Return Success
 */
PAL_Result nandUtlUpdateWLATable (
            DDC_NandState       *pNandState,
            Uint16              phyBlock,
            DDC_NandBlockState  bState)
{
    Uint16 blockBit;
    Uint16 bitPos;
    Uint16 mask;

    if( NULL != pNandState) {
        blockBit = (Uint16)(phyBlock >> 4);
        bitPos   = (Uint16)(phyBlock % 16);
        mask     = (Uint16)(1 << (15L - bitPos));

        if (DDC_NAND_FREE_BLOCK == bState) {
            pNandState->pWLAbitbuffer[blockBit] |= mask;
        }

        if (DDC_NAND_USED_BLOCK == bState) {
            pNandState->pWLAbitbuffer[blockBit] &= (Uint16)(~mask);
        }
    }

    return (PAL_SOK);
}

/**
 *   \brief Calculate block address
 *
 *  Converts the 'block' to the format specified by SSFDC standards.
 *
 *   \param block   [IN]    non-SSFDC format block number
 *
 *   \return SSFDC format block number
 */
Uint16 nandUtlCalcBlkAdrs (
            Uint16  block)
{
    Uint16 parity = 0;
    Uint16 blockAddr;
    Uint16 loopIndex;

    blockAddr = (Uint16)(((block & 0x3FF) << 1) | 0x1000);

    for (loopIndex = 0; loopIndex < 16; loopIndex++) {
        parity ^= ((blockAddr & (1 << loopIndex)) >> loopIndex);
    }

    blockAddr |= parity;  /* Attach even parity bit */

    return (blockAddr);
}

#ifdef DDC_NAND_ENABLE_ECC
/**
 *   \brief Generate Hamming code
 *
 * Function gen_hamming generates 3 byte hamming code for 256 bytes of data.
 * On entry pdata_buf points to the beginning of the 256 byte data array to
 * and phamm_code points to where to store the 3 bytes hamming code  for the
 * 256 bytes of data.
 *
 *   \param pdata_buf   [IN]    buffer containing 256 byte data array
 *   \param phamm_code  [OUT]   buffer to store 3 bytes hamming code
 *                              for pdata_buf
 *
 *   \return block number
 */
void gen_hamming (
            Uint8 *pdata_buf,
            Uint8 *phamm_code)
{
    Uint32 i;
    Uint8 j;

    Uint8   paritr [256];

    Uint8   tmp  = 0;
    Uint8   tmp2 = 0;

    Uint8   data_table0[16] = {0,1,1,0,1,0,0,1,1,0,0,1,0,1,1,0};
    Uint8   data_table1[16] = {1,0,0,1,0,1,1,0,0,1,1,0,1,0,0,1};

    Uint8   sum         = 0;

    Uint8   paritc      = 0;
    Uint8   parit0c     = 0;
    Uint8   parit1c     = 0;
    Uint8   parit2c     = 0;
    Uint8   parit3c     = 0;
    Uint8   parit4c     = 0;
    Uint8   parit5c     = 0;
    Uint8   parit6c     = 0;
    Uint8   parit7c     = 0;

    Uint8   parit1_1;
    Uint8   parit1_2;

    Uint8   parit2_1;
    Uint8   parit2_2;

    Uint8   parit4_1;
    Uint8   parit4_2;

    Uint8   parit8_1    = 0;
    Uint8   parit8_2    = 0;

    Uint8   parit16_1   = 0;
    Uint8   parit16_2   = 0;

    Uint8   parit32_1   = 0;
    Uint8   parit32_2   = 0;

    Uint8   parit64_1   = 0;
    Uint8   parit64_2   = 0;

    Uint8   parit128_1  = 0;
    Uint8   parit128_2  = 0;

    Uint8   parit256_1  = 0;
    Uint8   parit256_2  = 0;

    Uint8   parit512_1  = 0;
    Uint8   parit512_2  = 0;

    Uint8   parit1024_1 = 0;
    Uint8   parit1024_2 = 0;

    if( (NULL != pdata_buf) && (NULL != phamm_code)) {

        for (i = 0; i < 256; i++) {
            /* unpack word organized data */
            if ((i & 0x01) == 0) {  /* if i  even then unpack lower byte word */
                paritc = (Uint8)(paritc ^ ((*(pdata_buf+i/2) & 0xff)));
                tmp = (Uint8)((*(pdata_buf+ i/2) & 0xf0) >> 4);
                tmp2 = (Uint8)(*(pdata_buf+ i/2) & 0x0f);
            }
            else { /* if i odd then unpack upper byte of word */
                paritc = (Uint8)((paritc ^ ((*(pdata_buf+i/2)) >> 8)));
                tmp = (Uint8)((*(pdata_buf+ i/2) & 0xf000) >> 12);
                tmp2 =(Uint8)((*(pdata_buf+ i/2) & 0x0f00) >>  8);
            }

            switch (tmp) {
                case 0:
                    paritr[i] = data_table0[tmp2];
                    break;
                case 1:
                    paritr[i] = data_table1[tmp2];
                    break;
                case 2:
                    paritr[i] = data_table1[tmp2];
                    break;
                case 3:
                    paritr[i] = data_table0[tmp2];
                    break;
                case 4:
                    paritr[i] = data_table1[tmp2];
                    break;
                case 5:
                    paritr[i] = data_table0[tmp2];
                    break;
                case 6:
                    paritr[i] = data_table0[tmp2];
                    break;
                case 7:
                    paritr[i] = data_table1[tmp2];
                    break;
                case 8:
                    paritr[i] = data_table1[tmp2];
                    break;
                case 9:
                    paritr[i] = data_table0[tmp2];
                    break;
                case 10:
                    paritr[i] = data_table0[tmp2];
                    break;
                case 11:
                    paritr[i] = data_table1[tmp2];
                    break;
                case 12:
                    paritr[i] = data_table0[tmp2];
                    break;
                case 13:
                    paritr[i] = data_table1[tmp2];
                    break;
                case 14:
                    paritr[i] = data_table1[tmp2];
                    break;
                case 15:
                    paritr[i] = data_table0[tmp2];
                    break;
                default:
                    break;
            }

        }

        parit0c = (Uint8)((paritc & 0x01) ? 1 : 0);
        parit1c = (Uint8)((paritc & 0x02) ? 1 : 0);
        parit2c = (Uint8)((paritc & 0x04) ? 1 : 0);
        parit3c = (Uint8)((paritc & 0x08) ? 1 : 0);
        parit4c = (Uint8)((paritc & 0x10) ? 1 : 0);
        parit5c = (Uint8)((paritc & 0x20) ? 1 : 0);
        parit6c = (Uint8)((paritc & 0x40) ? 1 : 0);
        parit7c = (Uint8)((paritc & 0x80) ? 1 : 0);

        parit1_2 = (Uint8)(parit6c ^ parit4c ^ parit2c ^ parit0c);
        parit1_1 = (Uint8)(parit7c ^ parit5c ^ parit3c ^ parit1c);
        parit2_2 = (Uint8)(parit5c ^ parit4c ^ parit1c ^ parit0c);
        parit2_1 = (Uint8)(parit7c ^ parit6c ^ parit3c ^ parit2c);
        parit4_2 = (Uint8)(parit3c ^ parit2c ^ parit1c ^ parit0c);
        parit4_1 = (Uint8)(parit7c ^ parit6c ^ parit5c ^ parit4c);

        for (i = 0; i < 256; i++){
            sum= (Uint8)(sum ^ (*(paritr+i)));
        }

        for (i = 0; i < 256; i = i+2) {
            parit8_2 = (Uint8)(parit8_2 ^ (*(paritr + i)));
        }

        for (i = 0; i < 256; i = i+4) {
            parit16_2 = (Uint8)(parit16_2 ^ (*(paritr + i)));
            parit16_2 = (Uint8)(parit16_2 ^ (*(paritr + i + 1)));
        }

        for (i = 0; i < 256; i = i+8) {
            for (j = 0; j <= 3; j++) {
                parit32_2 = (Uint8)((parit32_2 ^ (*(paritr + i + j))));
            }
        }

        for (i = 0; i < 256; i = i+16) {
            for (j = 0; j <= 7; j++) {
                parit64_2 = (Uint8)((parit64_2 ^ (*(paritr + i + j))));
            }
        }

        for (i = 0; i < 256; i = i+32) {
            for (j = 0; j <= 15u; j++) {
                parit128_2 = (Uint8)((parit128_2 ^ (*(paritr + i + j))));
            }
        }

        for (i = 0; i < 256; i = i+64) {
            for (j = 0; j <= 31; j++) {
                parit256_2 = (Uint8)((parit256_2 ^ (*(paritr + i + j))));
            }
        }

        for (i = 0; i < 256; i = i+128u) {
            for (j = 0; j <= 63; j++) {
                parit512_2 = (Uint8)((parit512_2 ^ (*(paritr + i + j))));
            }
        }

        for (i = 0; i < 256; i = i+256) {
            for (j = 0; j <= 127; j++) {
                parit1024_2 = (Uint8)((parit1024_2 ^ (*(paritr + i + j))));
            }
        }

        if (sum==0) {
            parit1024_1 = parit1024_2;
            parit512_1  = parit512_2;
            parit256_1  = parit256_2;
            parit128_1  = parit128_2;
            parit64_1   = parit64_2;
            parit32_1   = parit32_2;
            parit16_1   = parit16_2;
            parit8_1    = parit8_2;
        }
        else {
            parit1024_1 = (Uint8)(parit1024_2 ? 0 : 1);
            parit512_1  = (Uint8)(parit512_2  ? 0 : 1);
            parit256_1  = (Uint8)(parit256_2  ? 0 : 1);
            parit128_1  = (Uint8)(parit128_2  ? 0 : 1);
            parit64_1   = (Uint8)(parit64_2   ? 0 : 1);
            parit32_1   = (Uint8)(parit32_2   ? 0 : 1);
            parit16_1   = (Uint8)(parit16_2   ? 0 : 1);
            parit8_1    = (Uint8)(parit8_2    ? 0 : 1);
        }

        parit1_2    <<= 2;
        parit1_1    <<= 3;
        parit2_2    <<= 4;
        parit2_1    <<= 5;
        parit4_2    <<= 6;
        parit4_1    <<= 7;
        parit128_1  <<= 1;
        parit256_2  <<= 2;
        parit256_1  <<= 3;
        parit512_2  <<= 4;
        parit512_1  <<= 5;
        parit1024_2 <<= 6;
        parit1024_1 <<= 7;
        parit8_1    <<= 1;
        parit16_2   <<= 2;

        parit16_1   <<= 3;
        parit32_2   <<= 4;
        parit32_1   <<= 5;
        parit64_2   <<= 6;
        parit64_1   <<= 7;

        *(phamm_code + 0) = (Uint8)((~(parit64_1     | parit64_2     |
                        parit32_1     | parit32_2     |
                        parit16_1     | parit16_2     |
                        parit8_1      | parit8_2))     & 0xff);

        *(phamm_code + 1) = (Uint8)((~(parit1024_1   | parit1024_2   |
                        parit512_1    | parit512_2    |
                        parit256_1    | parit256_2    |
                        parit128_1    | parit128_2))   & 0xff);

        *(phamm_code + 2) = (Uint8)((~(parit4_1      | parit4_2      |
                        parit2_1      | parit2_2      |
                        parit1_1      | parit1_2      |
                        0x00          | 0x00))         & 0xff);
    }
}
#endif /* DDC_NAND_ENABLE_ECC */

/**
 *   \brief Initialize the NAND Cache (implemented through software)
 *
 *  This function initializes the data structures needed for implementing
 *  the cache utilities for NAND driver. This is completely implemented in
 *  software and should not be confused (has no relationship) with the
 *  hardware cache support.
 *
 *   \param DDC     [IN]    DDC Structure
 *
 *   \return PAL_SOK
 */
PAL_Result nandCacheInit (
            const DDC_NandObj          *DDC
)
{
    volatile Uint16     blockCount; /* Block count */
    volatile Uint16     pageCount;  /* Page count */
    DDC_NandState       *pNandState;

    if(NULL != DDC) {
        pNandState     = (DDC_NandState *) &DDC->nandState[DDC->bankNo];
        /* Fill 0xFF (DDC_NAND_DEF_CAHCE_INIT_VALUE) as the redundant info */
        for (blockCount = 0; blockCount < DDC_NAND_NO_BLKS_CACHED;
                blockCount++) {
            for (pageCount = 0; pageCount < DDC_NAND_PAGE_CACHE_ENTRY_SIZE;
                    pageCount++) {
                pNandState->pBlockRedInfo[blockCount].
                    cachedLogPageNo[pageCount] = DDC_NAND_DEF_CAHCE_INIT_VALUE;
            }

            /* Cached block is replaceable */
            pNandState->pBlockRedInfo[blockCount].cacheIndex = 0;
            /* Block number */
            pNandState->pBlockRedInfo[blockCount].phyBlockNo =
                DDC_NAND_DEF_OLD_PHY_BLOCK;
        }
    }
    return PAL_SOK;
}

/**
 *   \brief Cache a NAND block.
 *
 *  This function adds a block's redundant info (physical sector info) to the
 *  cache, by reading from the redundant area of the NAND device.
 *
 *   \param DDC         [IN]    DDC structure
 *   \param phyBlock    [IN]    Address of the physical block on NAND device.
 *   \param cacheIndex  [OUT]   Index where the block is cached.
 *
 *   \return DDC_NAND_E_STATUS_READ_FAIL   - No replaceable cached block \n
 *             PAL_SOK              - Return Success
 */
PAL_Result nandCacheCacheBlk (
            DDC_NandObj          *DDC,
            Uint16               phyBlock,
            Uint8                *cacheIndex)
{
    DDC_NandRedundant nandRed;

    volatile Uint16 blockCount;
    volatile Uint16 pageCount;
    volatile Uint16 pagesRemaining;

    Uint32          phySector;
    PAL_Result      result;
    DDC_NandState   *pNandState;

    if( (NULL != DDC) && (NULL != cacheIndex)) {

        pNandState     = (DDC_NandState *) &DDC->nandState[DDC->bankNo];
        /* Find if the block is cached, by reading the block numbers */
        for (blockCount = 0; blockCount < DDC_NAND_NO_BLKS_CACHED;
                blockCount++) {
            if (pNandState->pBlockRedInfo[blockCount].phyBlockNo == phyBlock) {
                *cacheIndex = (Uint8)blockCount;
                return PAL_SOK;
            }
        }

        /* Get the index where the next block can be cached in the cache area */
        *cacheIndex = (Uint8)(pNandState->pBlockRedInfo[0].cacheIndex + 1);
        if (DDC_NAND_NO_BLKS_CACHED == (*cacheIndex)) {
            *cacheIndex = 0;
        }

        /* Update the index number in the first cache buffer area */
        pNandState->pBlockRedInfo[0].cacheIndex = *cacheIndex;


        /* Read the redundant info of all the pages in the block that
         * has to be cached.
         * Read till 0xFF (DDC_NAND_DEF_CAHCE_INIT_VALUE) is found and
         * fill all the remaining members of the array with
         * 0xFF (DDC_NAND_DEF_CAHCE_INIT_VALUE).
         */
        for (pageCount = 0; pageCount < pNandState->pagesPerBlock;
                pageCount++) {
            phySector = (((Uint32) phyBlock *
                        (Uint32) pNandState->pagesPerBlock)
                    + (Uint32) pageCount);
            result = nandPhyReadRedPage (DDC, phySector, &nandRed);
            if (PAL_SOK != result) {
                return result;
            }

            if ((nandRed.logPageNo & 0xFF) != DDC_NAND_DEF_CAHCE_INIT_VALUE) {
                pNandState->pBlockRedInfo[*cacheIndex].cachedLogPageNo
                    [pageCount] = (Uint8)(nandRed.logPageNo & 0xFF);
            }
            else {
                for (pagesRemaining = pageCount; pagesRemaining <
                        pNandState->pagesPerBlock; pagesRemaining++) {
                    pNandState->pBlockRedInfo[*cacheIndex].cachedLogPageNo
                        [pagesRemaining] = DDC_NAND_DEF_CAHCE_INIT_VALUE;
                }
                break;
            }
        }
        pNandState->pBlockRedInfo[*cacheIndex].phyBlockNo = phyBlock;
    }

    return PAL_SOK;
}

/**
 *   \brief Get page status
 *
 *  This function gets the status of a page. If the page is not cached, it
 *  would be cahched and the index is returned. If the page is not written,
 *  a false is returned for the 'pageFound' status in the cachePageRedInfo.
 *
 *   \param DDC                 [IN]    DDC structure
 *   \param phySector           [IN]    Address of physical sector on NAND.
 *   \param cachedPageRedInfo   [OUT]   Cached page redundant info
 *
 *   \return PAL_Error      - on error                                   \n
 *             PAL_SOK               - Return Success
 */
PAL_Result nandCacheGetPageStatus (
            DDC_NandObj                 *DDC,
            Uint32                      phySector,
            DDC_NandCachePageRedInfo    *cachedPageRedInfo
)
{
    Uint8           isCached;
    Uint16          phyBlock;
    Uint16          phyPageNo;
    volatile Uint16 blockCount;
    volatile Uint16 pageCount;
    Uint8           cacheIndex;

    PAL_Result      result;
    DDC_NandState   *pNandState;

    if((NULL != DDC) && (NULL != cachedPageRedInfo)) {

        pNandState = (DDC_NandState *) &DDC->nandState[DDC->bankNo];
        /* Find the block number of the given physical sector */
        phyBlock = (Uint16)(phySector / (pNandState->pagesPerBlock));

        /* Find the page in the block */
        phyPageNo = (Uint16)(phySector % (pNandState->pagesPerBlock));

        /* Find if the block is cached */
        isCached = FALSE;
        for (blockCount = 0; blockCount < DDC_NAND_NO_BLKS_CACHED;
                blockCount++) {
            if (pNandState->pBlockRedInfo[blockCount].phyBlockNo == phyBlock) {
                isCached = TRUE;
                break;
            }
        }

        /* The block is not cached */
        if (FALSE == isCached) {
            /* Cache the new block and get where it is cached in cacheIndex */
            result = nandCacheCacheBlk (DDC, phyBlock, &cacheIndex);
            if (PAL_SOK != result) {
                return (result);
            }
            blockCount = cacheIndex;
        }

        /* Find and return the array index and the status of the page
         *  for the required physical sector
         */
        for (pageCount = 0; pageCount < pNandState->pagesPerBlock;
                pageCount++) {
            if (pNandState->pBlockRedInfo[blockCount].cachedLogPageNo
                    [pageCount] != DDC_NAND_DEF_CAHCE_INIT_VALUE) {
                if (pNandState->pBlockRedInfo[blockCount].cachedLogPageNo
                        [pageCount] == phyPageNo) {
                    /* Required page is found. Break and return */
                    cachedPageRedInfo->cacheIndex = (Uint8)pageCount;
                    cachedPageRedInfo->pageFound = TRUE;
                    break;
                }
            }
            /* The required page is empty and so the entry of the page is not
             * found in the block's redundant information
             */
            else {
                cachedPageRedInfo->cacheIndex = (Uint8)pageCount;
                cachedPageRedInfo->pageFound = FALSE;
                /* verify this variable is req */
                break;
            }
        }
    }

    return PAL_SOK;
}

/**
 *   \brief Get the blocks redundant info.
 *
 *  This function reads the cached block's redundant information to an array
 *  as pointed by the input pointer (API Not used for now, may be buggy!)
 *
 *   \param DDC         [IN]    DDC structure
 *   \param phyBlock    [IN]    Address of the physical block on NAND device.
 *   \param pRedBuffer  [IN]    pointer to the Redundant page info
 *
 *   \return PAL_Error  - On error                                       \n
 *             PAL_SOK               - Return Sucess
 */
PAL_Result nandCacheGetBlkRed (
            DDC_NandObj         *DDC,
            Uint16              phyBlock,
            Uint32              *pRedBuffer
)
{
    Uint8           isCached; /* Flag to indicate if the block is
                                 cached or not */
    volatile Uint16 blockCount;
    volatile Uint16 pageCount;
    PAL_Result      result;
    Uint8           cacheIndex;
    DDC_NandState   *pNandState;

    if( (NULL != DDC) && (NULL != pRedBuffer)) {
        pNandState     = (DDC_NandState *) &DDC->nandState[DDC->bankNo];
        isCached = FALSE;
        /* Find if the Block is cached */
        for (blockCount = 0; blockCount < DDC_NAND_NO_BLKS_CACHED;
                blockCount++) {
            if (pNandState->pBlockRedInfo[blockCount].phyBlockNo == phyBlock) {
                isCached = TRUE; /* Block is cached */
                break;
            }
        }

        /* Block is not cached */
        if (FALSE == isCached) {
            /* Cache the new block and get where it is cached in cacheIndex */
            result = nandCacheCacheBlk (DDC, phyBlock, &cacheIndex);
            if (PAL_SOK != result) {
                return result;
            }
            blockCount = cacheIndex;
        }

        /* Read the redundant info of the pages - API not used! */
        for (pageCount = 0; pageCount < pNandState->pagesPerBlock;
                pageCount++) {
            *pRedBuffer = (phyBlock * pNandState->pagesPerBlock) +
                (pNandState->pBlockRedInfo[blockCount].cachedLogPageNo
                 [pageCount]);
            pRedBuffer++;
        }
    }

    return PAL_SOK;
}

/**
 *   \brief Update Cache Page entry
 *
 *  This function updates the redundant info maintained in the cache- say.
 *  after a page write.
 *
 *   \param DDC         [IN]    DDC Structure
 *   \param phySector   [IN]    Address of the physical sector on NAND device
 *
 *   \return PAL_Error  - On error                                       \n
 *             PAL_SOK               - Return Sucess
 */
PAL_Result nandCacheUpdatePageEntry (
            DDC_NandObj         *DDC,
            Uint32              phySector
)
{
    Uint16          phyBlock;
    Uint16          phyPage;

    volatile Uint16 blockCount;
    volatile Uint16 pageCount;
    DDC_NandState   *pNandState;

    if( NULL != DDC) {
        pNandState     = (DDC_NandState *) &DDC->nandState[DDC->bankNo];
        phyBlock = (Uint16)(phySector / (pNandState->pagesPerBlock));
        phyPage = (Uint16)(phySector % (phyBlock * pNandState->pagesPerBlock));

        /* Find the index of the block cached */
        for (blockCount = 0; blockCount < DDC_NAND_NO_BLKS_CACHED;
                blockCount++) {
            if (pNandState->pBlockRedInfo[blockCount].phyBlockNo == phyBlock) {
                break;
            }
        }

        /* Locate the index at which the page number can be appended */
        for (pageCount = 0; pageCount < pNandState->pagesPerBlock;
                pageCount++) {
            if (DDC_NAND_DEF_CAHCE_INIT_VALUE ==
                    pNandState->pBlockRedInfo[blockCount].cachedLogPageNo
                    [pageCount]) {
                break;
            }
        }

        pNandState->pBlockRedInfo[blockCount].cachedLogPageNo[pageCount] =
            (Uint8)phyPage;
    }

    return PAL_SOK;
}

/**
 *   \brief Erase a block from Cache.
 *
 *  This function resets the redundant info in the cache for a block.
 *
 *   \param DDC         [IN]    DDC Structure
 *   \param phyBlock    [IN]    Address of the physical block on NAND device.
 *
 *   \return PAL_Error  - On error                                       \n
 *             PAL_SOK               - Return Sucess
 */
PAL_Result nandCacheBlkErase (
            const DDC_NandObj         *DDC,
            Uint16              phyBlock
)
{
    volatile Uint16  blockCount;
    volatile Uint16  isCached;
    volatile Uint16  pageCount;

    DDC_NandState   *pNandState;

    if( NULL != DDC) {

        pNandState     = (DDC_NandState *) &DDC->nandState[DDC->bankNo];
        isCached = FALSE;

        /* Find the index of the block cached */
        for (blockCount = 0; blockCount < DDC_NAND_NO_BLKS_CACHED;
                blockCount++) {
            if (pNandState->pBlockRedInfo[blockCount].phyBlockNo == phyBlock) {
                isCached = TRUE;
                break;
            }
        }

        if (isCached == TRUE) {
            for (pageCount = 0; pageCount < DDC_NAND_PAGE_CACHE_ENTRY_SIZE;
                    pageCount++) {
                pNandState->pBlockRedInfo[blockCount].cachedLogPageNo
                    [pageCount] = DDC_NAND_DEF_CAHCE_INIT_VALUE;
            }

            pNandState->pBlockRedInfo[blockCount].phyBlockNo  =
                DDC_NAND_DEF_OLD_PHY_BLOCK;
        }
    }

    return PAL_SOK;
}

