/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/** \file   llc_nand.c
 *  \brief  NAND LLC definitions.
 *
 *  This file contains the LLC definitions, that are used by the DDC.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author     Amit Chatterjee
 *
 *  \version    1.0 Amit Chatterjee Migrated to DM350
 *  \version    1.1 Kiran Sutariya Migrated to DM6437
 */

#include <ti/sdo/pspdrivers/drivers/nand/src/llc_nandIf.h>
#include <assert.h>

extern  EDMA3_DRV_Handle hEdma;

/*
 * Definitions of Externally Visible Data Structures
 */

void LLC_checkCommandStatus(LLC_NandHandle hNand);

/*
 * Prototypes of Static Functions
 */

/**
 *  \brief Initialize the Low level controller.
 *
 *  The low level NAND controller is initialized, by this API,
 *  that is a part of the Init call.
 *
 *  \param  nandObj [IN]    NAND Object information
 *  \param  instId  [IN]    Instance Id.
 *  \return None
 */
void LLC_nandInit(
            LLC_NandObj         *nandObj,
            Uint32              instId)
{
    if (nandObj != NULL)
    {
        nandObj->instId = instId;

        switch (nandObj->instId) {
            case CSL_EMIFA:
                nandObj->intrNumNand = CSL_INTC_EVENTID_EMIFAINT;
                nandObj->regs = (CSL_Emif2Regs *)CSL_EMIFA_0_REGS;
                break;
            default:
                assert(0);
        }
    }
    else
    {
        assert(0);
    }
}


/**
 *  \brief Perform the Hw Setup of NAND.
 *
 *  The NAND low level controller is setup according to the hardware
 *  configuration requested.
 *
 *  \param  hNand       [IN]    NAND Driver Instance Handle
 *  \param  hwConfig    [IN]    Hardware Configuration information.
 *  \return None
 */
void LLC_nandHwSetup(
            LLC_NandHandle      hNand,
            LLC_NandHwConfig    *hwConfig)
{
    Uint8 nand16Bit = 0; /* Default value */

    if ((hNand != NULL) && (hwConfig != NULL))
    {
        /*
         * NOTE: Settings for NAND, when the interface is through EMIF.
         */
         /* Get the address of CLE, ALE and DATA register for the Bank.
         * CSL's register overlay structure should be assigning these addresses.
         */
        LLC_nandGetBankInfo(hNand,
                            (Int32)hwConfig->cfgBasic.bankNo,
                            (LLC_NandAsyncBank*)&hNand->bank);

        if (hwConfig->cfgBasic.llcNandWidth == (LLC_NandWidth) LLC_NAND_8_BIT) {
                nand16Bit = 0;
        }
        else {
                nand16Bit = 1;
        }

            /* NAND Flash Control Register (NANDFCR) */
        /* Set Bits to 1 if using NAND Flash on CSxNAND; mapped to '/EM_CS[x]'. */
        *(Uint32 *)&hNand->regs->NANDFCR = (Uint32)(CSL_FMK (EMIF2_NANDFCR_CS2NAND, 0x1) |
                                        CSL_FMK (EMIF2_NANDFCR_CS3NAND, 0x1) |
                                        CSL_FMK (EMIF2_NANDFCR_CS4NAND, 0x1) |
                                        CSL_FMK (EMIF2_NANDFCR_CS5NAND, 0x1) |
                                        CSL_FMK (EMIF2_NANDFCR_CS2ECC , 0x0) |
                                        CSL_FMK (EMIF2_NANDFCR_CS3ECC , 0x0) |
                                        CSL_FMK (EMIF2_NANDFCR_CS4ECC , 0x0) |
                                        CSL_FMK (EMIF2_NANDFCR_CS5ECC , 0x0) );

            /* Asynchronous bank4 configuration */
        *(Uint32 *)&hNand->regs->A4CR = (Uint32) (  CSL_FMK (EMIF2_A4CR_SS,       0x0) |
                                        CSL_FMK (EMIF2_A4CR_EW,       0x0) |
                                        CSL_FMK (EMIF2_A4CR_W_SETUP,  0x1) |
                                        CSL_FMK (EMIF2_A4CR_W_STROBE, 0x4) |
                                        CSL_FMK (EMIF2_A4CR_W_HOLD,   0x1) |
                                        CSL_FMK (EMIF2_A4CR_R_SETUP,  0x1) |
                                        CSL_FMK (EMIF2_A4CR_R_STROBE, 0x6) |
                                        CSL_FMK (EMIF2_A4CR_R_HOLD,   0x1) |
                                        CSL_FMK (EMIF2_A4CR_TA,       0x3) |
                                        CSL_FMK (EMIF2_A4CR_ASIZE,    nand16Bit) ); /*0x0442231C;    Buswidth is 8bit */

            /* Asynchronous bank3 configuration */
        *(Uint32 *)&hNand->regs->A3CR = (Uint32)(   CSL_FMK (EMIF2_A3CR_SS,       0x0) |
                                        CSL_FMK (EMIF2_A3CR_EW,       0x0) |
                                        CSL_FMK (EMIF2_A3CR_W_SETUP,  0x1) |
                                        CSL_FMK (EMIF2_A3CR_W_STROBE, 0x4) |
                                        CSL_FMK (EMIF2_A3CR_W_HOLD,   0x1) |
                                        CSL_FMK (EMIF2_A3CR_R_SETUP,  0x1) |
                                        CSL_FMK (EMIF2_A3CR_R_STROBE, 0x6) |
                                        CSL_FMK (EMIF2_A3CR_R_HOLD,   0x1) |
                                        CSL_FMK (EMIF2_A3CR_TA,       0x3) |
                                        CSL_FMK (EMIF2_A3CR_ASIZE,    nand16Bit) ); /*0x0442231C;    Buswidth is 8bit*/

            /* Asynchronous bank2 configuration */
        *(Uint32 *)&hNand->regs->A2CR = (Uint32)(   CSL_FMK (EMIF2_A2CR_SS,       0x0) |
                                        CSL_FMK (EMIF2_A2CR_EW,       0x0) |
                                        CSL_FMK (EMIF2_A2CR_W_SETUP,  0x1) |
                                        CSL_FMK (EMIF2_A2CR_W_STROBE, 0x4) |
                                        CSL_FMK (EMIF2_A2CR_W_HOLD,   0x1) |
                                        CSL_FMK (EMIF2_A2CR_R_SETUP,  0x1) |
                                        CSL_FMK (EMIF2_A2CR_R_STROBE, 0x6) |
                                        CSL_FMK (EMIF2_A2CR_R_HOLD,   0x1) |
                                        CSL_FMK (EMIF2_A2CR_TA,       0x3) |
                                        CSL_FMK (EMIF2_A2CR_ASIZE,    nand16Bit) ); /*0x0442231C;    Buswidth is 8bit*/

            /* Asynchronous bank1 configuration */
        *(Uint32 *)&hNand->regs->A1CR = (Uint32)(   CSL_FMK (EMIF2_A1CR_SS,       0x0) |
                                        CSL_FMK (EMIF2_A1CR_EW,       0x0) |
                                        CSL_FMK (EMIF2_A1CR_W_SETUP,  0x1) |
                                        CSL_FMK (EMIF2_A1CR_W_STROBE, 0x4) |
                                        CSL_FMK (EMIF2_A1CR_W_HOLD,   0x1) |
                                        CSL_FMK (EMIF2_A1CR_R_SETUP,  0x1) |
                                        CSL_FMK (EMIF2_A1CR_R_STROBE, 0x6) |
                                        CSL_FMK (EMIF2_A1CR_R_HOLD,   0x1) |
                                        CSL_FMK (EMIF2_A1CR_TA,       0x3) |
                                        CSL_FMK (EMIF2_A1CR_ASIZE,    nand16Bit) ); /*0x0442231C;    Buswidth is 8bit*/



            /* Asynchronous wait cycle Configuration Register */
            /* [31:28] are '1'.
             *   1: wait if pad_wait_i[3-0] port is high
             */
        *(Uint32 *)&hNand->regs->AWCCR = (Uint32) (CSL_FMK (EMIF2_AWCCR_WP0, 0x1) |
                                          CSL_FMK (EMIF2_AWCCR_MEWC, 0xFF));
        *(Uint32 *)&hNand->regs->AWCCR |= 0x80000000u;

            /* Disable the interrupts */
        *(Uint32 *)&hNand->regs->EIMCR = (Uint32)(  CSL_FMK (EMIF2_EIMCR_ATMCLR, 1) |
                                          CSL_FMK (EMIF2_EIMCR_WRMCLR, 0xF) ); /* 0x3F;*/
    }
    return;
}

/**
 *  \brief Gets the interrupt number of NAND
 *
 *  This function gets the interrupt number of this instance of NAND on
 *  the SoC as defined in the system's SoC.
 *
 *  \param  deviceId [IN]   Device Id of NAND (instance of NAND on SoC.)
 *  \return interrupt number.
 */
int LLC_nandGetIntrNum (
            Int deviceId)
{
    Int     intrNum;

    switch (deviceId) {
    case CSL_EMIFA:
        intrNum = CSL_INTC_EVENTID_EMIFAINT;
        break;
    default:
        assert(0);
    }

    return (intrNum);
}
#ifdef DDC_NAND_8_BIT
void LLC_nandSendCommand(LLC_NandHandle hNand, VUint8 cmd)
{
    LLC_checkCommandStatus(hNand);
    *(VUint8*)hNand->bank.CExCLE = cmd;
}
#endif
/**
 *   \brief Call back function for EDMA transfer complet
 *
 *  This function will be called when EDMA transfer will complet so
 *  this function will release sem which was taken before EDMA start.
 *
 *   \param lch         [IN]    Not used
 *   \param ch_status   [IN]    Not used
 *   \param data    [OUT]   Address of NAND Driver Instance Handle
 *
 *   \return            void                           \n
 *
 */
static void nand_callback(Uint lch, EDMA3_RM_TccStatus ch_status, Ptr data)
{
    LLC_NandHandle  hNand = (LLC_NandHandle) data;

    /* To remove remark while compiling using -pdr diagnostics*/
    lch = lch;

    if (ch_status != EDMA3_RM_XFER_COMPLETE)
    {
        EDMA3_DRV_freeChannel (hEdma, hNand->dmaChannel);
    }
    else
    {
        EDMA3_DRV_disableTransfer(hEdma,
                                   hNand->dmaChannel, EDMA3_DRV_TRIG_MODE_MANUAL);

        EDMA3_DRV_setOptField (hEdma, hNand->dmaChannel, EDMA3_DRV_OPT_FIELD_TCINTEN, 0);

    }



    /* Release the semaphore */
    if(PAL_SOK != PAL_osSemGive (hNand->syncSem))
    {
        NAND_Debug("\r\nsync sem give failed\n");
    }
}

Uint32 LLC_nandSetDmaMode (LLC_NandHandle           hNand)
{

    EDMA3_DRV_Result result = PSP_SOK;

    hNand->nandDmaTcc = EDMA3_DRV_TCC_ANY;
    hNand->dmaChannel = EDMA3_DRV_DMA_CHANNEL_ANY;

    result = EDMA3_DRV_requestChannel (hEdma,
                                      (Uint32 *)&hNand->dmaChannel,
                                      &hNand->nandDmaTcc,
                                      (EDMA3_RM_EventQueue)0,
                                      nand_callback,
                                      (Ptr) hNand);

    if(result!= PAL_SOK)
    {

        NAND_Debug("\r\nNAND EDMA channel Request failed\n");

    }

    return (0xFFFF);
}

Uint32 LLC_nandCloseDmaMode (LLC_NandHandle         hNand)
{
    EDMA3_DRV_freeChannel (hEdma,hNand->dmaChannel);

    hNand->dmaChannel = 0;

    return 0;
}

Int32 LLC_nandHandleDmaISR (LLC_NandHandle          hNand)
{
    /* To remove remark while compiling using -pdr diagnostics*/
    hNand = hNand;

    /* We are not handling any DMA IRQ here */
    return 0;
}

/**
 *   \brief Reads 'N' bytes from NAND using DMA.
 *
 *  This function reads 'readRequest' number of bytes to the buffer supplied
 *  using DMA.
 *
 *   \param nandObj     [IN]    llc structure
 *   \param readRequest [IN]    number of bytes to be read
 *   \param pReadBuffer [IN]    address of the buffer, where the read data
 *                              has to  be stored
 *   \param pack        [IN]    if byte packing is to be done or not
 *
 *   \return None
 */
void LLC_nandDMAReadNBytes (
            LLC_NandObj *nandObj,
            Uint16      readRequest,
            volatile Uint16      *pReadBuffer,
            Uint16      pack)
{
    static Uint32       readOpt = 0;

    /* To remove remark while compiling using -pdr diagnostics*/
    pack = pack;

    /* Invalidate the cache before read */
    PAL_osCacheInvalidate((PAL_osCacheMemAddrSpace)NULL,(Uint32)pReadBuffer, readRequest);

    EDMA3_DRV_setSrcParams( hEdma,
                            nandObj->dmaChannel,
                            (Uint32)(nandObj->bank.CExDATA),
                            EDMA3_DRV_ADDR_MODE_INCR,
                            EDMA3_DRV_W8BIT);

    EDMA3_DRV_setDestParams(hEdma,
                            nandObj->dmaChannel,
                            (Uint32)(pReadBuffer),
                            EDMA3_DRV_ADDR_MODE_INCR,
                            EDMA3_DRV_W8BIT);

    EDMA3_DRV_setSrcIndex(  hEdma,
                            nandObj->dmaChannel,
                            0,
                            0);

    EDMA3_DRV_setDestIndex( hEdma,
                            nandObj->dmaChannel,
                            1,
                            0);

    EDMA3_DRV_setTransferParams(    hEdma,
                                    nandObj->dmaChannel,
                                    1,
                                    readRequest,
                                    1,
                                    0,
                                    EDMA3_DRV_SYNC_AB);

    if(readOpt == 0)
    {
        EDMA3_DRV_setOptField (     hEdma,
                                    nandObj->dmaChannel,
                                    EDMA3_DRV_OPT_FIELD_TCINTEN,
                                    1);

        EDMA3_DRV_getPaRAMEntry(    hEdma,
                                    nandObj->dmaChannel,
                                    EDMA3_DRV_PARAM_ENTRY_OPT,
                                    &readOpt);
    }
    else
    {
        EDMA3_DRV_setPaRAMEntry (   hEdma,
                                   nandObj->dmaChannel,
                                   EDMA3_DRV_PARAM_ENTRY_OPT,
                                   readOpt);
        }

    EDMA3_DRV_setOptField (hEdma,nandObj->dmaChannel, EDMA3_DRV_OPT_FIELD_TCC, nandObj->nandDmaTcc);

    EDMA3_DRV_enableTransfer (hEdma, nandObj->dmaChannel,EDMA3_DRV_TRIG_MODE_MANUAL);

    if(PAL_SOK != PAL_osSemTake (nandObj->syncSem, PAL_OSSEM_NO_TIMEOUT)) /* PAL_OSSEM_NO_TIMEOUT */
    {
        NAND_Debug("\r\nERROR - LLC_nandDMAReadNBytes Sem Take Failed\n");
    }

    return;
}

/**
 *   \brief Writes 'N' bytes to NAND using DMA.
 *
 *  This function writes 'writeRequest' number of bytes to NAND from the
 *  buffer specified using DMA.
 *
 *   \param nandObj     [IN]    LLC structure
 *   \param writeRequest [IN]   number of bytes to be written to NAND device
 *   \param pWriteBuff   [IN]   Addr of buffer containing data to be written
 *   \param val          [IN]   If a value, (instead of from a buffer) needs
 *                              to be written
 *
 *   \return None
 */
void LLC_nandDMAWriteNBytes (
            LLC_NandObj    *nandObj,
            Uint16         writeRequest,
            Uint16         *pWriteBuff,
            Uint16         val)
{
    static Uint32       writeOpt = 0;

    /* To remove remark while compiling using -pdr diagnostics*/
    val = val;

    PAL_osCacheFlush((PAL_osCacheMemAddrSpace)NULL,(Uint32)pWriteBuff, writeRequest);

    EDMA3_DRV_setSrcParams( hEdma,
                            nandObj->dmaChannel,
                            (Uint32)(pWriteBuff),
                            EDMA3_DRV_ADDR_MODE_INCR,
                            EDMA3_DRV_W8BIT);

    EDMA3_DRV_setDestParams( hEdma,
                             nandObj->dmaChannel,
                             (Uint32)(nandObj->bank.CExDATA),
                             EDMA3_DRV_ADDR_MODE_INCR,
                             EDMA3_DRV_W8BIT);

    EDMA3_DRV_setSrcIndex(  hEdma,
                            nandObj->dmaChannel,
                            1,
                            0);

    EDMA3_DRV_setDestIndex( hEdma,
                            nandObj->dmaChannel,
                            0,
                            0);

    EDMA3_DRV_setTransferParams(    hEdma,
                                    nandObj->dmaChannel,
                                    1,
                                    writeRequest,
                                    1,
                                    0,
                                    EDMA3_DRV_SYNC_AB);

    if(writeOpt == 0)
    {
        EDMA3_DRV_setOptField ( hEdma,
                                nandObj->dmaChannel,
                                EDMA3_DRV_OPT_FIELD_TCINTEN,
                                1);

        EDMA3_DRV_getPaRAMEntry(hEdma,
                                nandObj->dmaChannel,
                                EDMA3_DRV_PARAM_ENTRY_OPT,
                                &writeOpt);
    }
    else
            {
        EDMA3_DRV_setPaRAMEntry(hEdma,
                                nandObj->dmaChannel,
                                EDMA3_DRV_PARAM_ENTRY_OPT,
                                writeOpt);
            }
    EDMA3_DRV_setOptField (hEdma,nandObj->dmaChannel, EDMA3_DRV_OPT_FIELD_TCC, nandObj->nandDmaTcc);
    EDMA3_DRV_enableTransfer (hEdma, nandObj->dmaChannel, EDMA3_DRV_TRIG_MODE_MANUAL);

    if(PAL_SOK != PAL_osSemTake (nandObj->syncSem, PAL_OSSEM_NO_TIMEOUT))
    {
        NAND_Debug("\r\nERROR - LLC_nandDMAWriteNBytes Sem Take Failed\n");
    }
    return;
}
/**
 *   \brief Set Data, Command and Address register address.
 *
 *  This function fills the bank address details for DATA,ALE,CLE
 *
 *   \param hNand     [IN]    Handle to LLC Nand Object
 *   \param bankNo    [IN]    Bank # in which NAND is mapped
 *   \param *bank     [IN/OUT]  LLC_NandAsyncBank
 *
 *   \return None
 */
void LLC_nandGetBankInfo (LLC_NandHandle            hNand,
                          Int32                     bankNo,
                          LLC_NandAsyncBank         *bank)
{

    if( (hNand != NULL) && (bank != NULL))
    {
        switch(bankNo) {
            case LLC_NAND_BANK_0:
                bank->CExDATA = (Uint32)0x42000000u;
                bank->CExALE  = (Uint32)0x42000008u;
                bank->CExCLE  = (Uint32)0x42000010u;
                bank->NANDFxECC  = (Uint32)&(hNand->regs->NANDF1ECC);
                break;

            case LLC_NAND_BANK_1:
                bank->CExDATA = (Uint32)0x44000000u;
                bank->CExALE  = (Uint32)0x44000008u;
                bank->CExCLE  = (Uint32)0x44000010u;
                bank->NANDFxECC  = (Uint32)&(hNand->regs->NANDF2ECC);
                break;

            case LLC_NAND_BANK_2:
                bank->CExDATA = (Uint32)0x46000000u;
                bank->CExALE  = (Uint32)0x46000008u;
                bank->CExCLE  = (Uint32)0x46000010u;
                bank->NANDFxECC  = (Uint32)&(hNand->regs->NANDF3ECC);
                break;

            case LLC_NAND_BANK_3:
                bank->CExDATA = (Uint32)0x48000000u;
                bank->CExALE  = (Uint32)0x48000008u;
                bank->CExCLE  = (Uint32)0x48000010u;
                bank->NANDFxECC  = (Uint32)&(hNand->regs->NANDF4ECC);
                break;

            case LLC_NAND_BANK_4:
            default:
                assert(0);
        }
    }
    else
    {
        assert(0);
    }
}

/**
 *   \brief Provides delay
 *
 *   \param loopCount     [IN]    count value
 *
 *   \return 0
 */
Int32 LLC_waitLoop(volatile Uint32  loopCount)
{
    volatile Uint32  cnt;

    for(cnt = 0; cnt < loopCount; cnt ++)
    {
        volatile Uint32 inner_cnt;
        for(inner_cnt = 0; inner_cnt < 0x10; inner_cnt ++)
        {
        }
    }

    return 0;
}


/**
 *   \brief Check R/B pin status
 *
 *  This function wait until R/B pin status is busy
 *
 *   \param hNand     [IN]    Handle to LLC Nand Object
 *
 *   \return None
 */
void LLC_checkCommandStatus(LLC_NandHandle          hNand)
{
    volatile Int    timeout = 0;

    LLC_waitLoop(5);

    if( NULL != hNand) {
        while(((hNand->regs->NANDFSR & 0x1u) == 0) &&
                        (timeout < LLC_NAND_MAX_TIMEOUT)){
            timeout ++;
        }
    }

#ifdef PSP_NAND_DEBUG
    if(timeout >= LLC_NAND_MAX_TIMEOUT)
    {
        NAND_Debug("\r\nLLC_checkCommandStatus: Timed-out\n");
    }
#endif

    /* Wait for tRR (Ready to RE Low) */
    LLC_waitLoop(5);
}

/**
 *   \brief Enable Hardware ECC
 *
 *   This function enables HW ECC mechanism
 *
 *   \param hNand     [IN]    Handle to LLC Nand Object
 *   \param bankNo    [IN]    LLC_NandBankNo
 *
 *   \return NANDFCR register value
 */
Uint32  LLC_nandEnableHwECC(LLC_NandHandle          hNand,
                                LLC_NandBankNo          bankNo)
{
    Uint32 result;
    if(NULL != hNand) {
        hNand->regs->NANDFCR |= (0x100 << bankNo);
        result = hNand->regs->NANDFCR;
    }

    return(result);
}

/**
 *   \brief Disable Hardware ECC
 *
 *   This function disables HW ECC mechanism
 *
 *   \param hNand     [IN]    Handle to LLC Nand Object
 *   \param bankNo    [IN]    LLC_NandBankNo
 *
 *   \return NANDFCR register value
 */
Uint32  LLC_nandDisableHwECC(LLC_NandHandle         hNand,
                                 LLC_NandBankNo         bankNo)
{
    Uint32      flag = (0x100 << bankNo);
    Uint32      result;
    if( NULL != hNand) {
        hNand->regs->NANDFCR &= (~flag);
        result = hNand->regs->NANDFCR;
    }

    return(result);
}

