/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 * \file    llc_nandIf.h
 *
 * \brief   NAND low-level controller interface
 *
 * This file contains the definitions that are used to abstract the low level
 * controller so that the DDC can be agnostic of the underlying controller.
 * The DDC shall assume a union of all functionalities and the specific
 * underlying controller shall implement or stub out specific interfaces. This
 * file contains the NAND LLC interfaces implemented for the NAND controller
 * on DM6437.
 *
 * (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \note       Set tabstop to 4 (:se ts=4) while viewing this file in an
 *              editor
 *
 *  \author     AK Mistral, Himamsu Mistral
 *
 *  \version    1.0 AK Mistral      Created.
 *  \version    1.1 Himamsu Mistral Code cleanup and release for BFT5
 *  \version    1.2 Himamsu Mistral Release for BFT6
 *  \version    1.3 Himamsu Mistral Release for BFT7
 *  \version    1.4 Himamsu Mistral Added ECC and WP related #defines
 *                                  - Release for BFT8
 */

#ifndef _LLC_NANDIF_H_
#define _LLC_NANDIF_H_

#include <ti/sdo/pspdrivers/drivers/nand/src/llc_nandTypes.h>

#ifdef __cplusplus
extern "C" {
#endif

#define CACHE_LINE_LENGTH           PAL_OS_CACHE_LINE_LEN
/**< Cache line length in bytes                                             */

typedef volatile unsigned int   VUint32;
typedef volatile unsigned short VUint16;
typedef volatile unsigned char  VUint8;

/**
 * \defgroup LLCNANDMain LLC NAND Interface Definition
 *
 * Top-level Encapsulation of all documentation for NAND LLC
 *
 * @{
 */

/**
 * \defgroup LLCNandInterfaces LLC NAND Interfaces
 *
 * Functions exported by the LLC, which are typically called by the DDA to
 * initialize the context of operation of the LLC
 *
 * @{
 */
extern void LLC_nandInit(
                        LLC_NandObj             *nandObj,
                        Uint32                  instId);
extern void LLC_nandHwSetup(
                           LLC_NandHandle       hNand,
                           LLC_NandHwConfig     *hwConfig);
extern int LLC_nandGetIntrNum (
                           int                  deviceId);

#define LLC_NAND_MAX_TIMEOUT    (0x7FFFFFFFu)
Int32 LLC_waitLoop(volatile Uint32  loopCount);
void LLC_checkCommandStatus(LLC_NandHandle          hNand);
Uint32  LLC_nandEnableHwECC(LLC_NandHandle          hNand,
                            LLC_NandBankNo          bankNo);
Uint32  LLC_nandDisableHwECC(LLC_NandHandle         hNand,
                             LLC_NandBankNo         bankNo);

void LLC_nandGetBankInfo (LLC_NandHandle            hNand,
                          Int32                     bankNo,
                          LLC_NandAsyncBank         *bank);

Uint32 LLC_nandSetDmaMode (LLC_NandHandle           hNand);
Uint32 LLC_nandCloseDmaMode (LLC_NandHandle         hNand);
Int32 LLC_nandHandleDmaISR (LLC_NandHandle          hNand);
void LLC_nandDMAReadNBytes (
            LLC_NandObj *nandObj,
            Uint16      readRequest,
            volatile Uint16      *pReadBuffer,
            Uint16      pack);
void LLC_nandDMAWriteNBytes (
            LLC_NandObj    *nandObj,
            Uint16         writeRequest,
            Uint16         *pWriteBuff,
            Uint16         val);


/**
 * \defgroup LLCNandConfig LLC NAND Configuration
 *
 * Interfaces for configuring the NAND controller for a specific mode of
 * operation.
 *
 * @{
 */

#ifdef DDC_NAND_8_BIT
/* Modified by TIJ for Amit. Aug. 30, 2006 */
/*  #define LLC_nandSendCommand(hNand, cmd) (*(VUint8*)hNand->bank.CExCLE = cmd)*/
void LLC_nandSendCommand(LLC_NandHandle hNand, VUint8 cmd);
#else
    #define LLC_nandSendCommand(hNand, cmd) (*(VUint16*)hNand->bank.CExCLE = (Uint16)cmd)
#endif
/**< Set the NAND command
 * \param   hNand   [IN]    {LLC_NandObj *}   Handle to the LLC Nand Object
 * \param   cmd     [IN]    {LLC_NandCommand} Command to be set in Nand    */


#ifdef DDC_NAND_8_BIT
    #define LLC_nandSetAddr(hNand, addr) (*(VUint8*)hNand->bank.CExALE = (Uint8)addr)
#else
    #define LLC_nandSetAddr(hNand, addr) (*(VUint16*)hNand->bank.CExALE = (Uint16)addr)
#endif
/**< Set the NAND Address
 * \param   hNand   [IN]    {LLC_NandObj *} Handle to the LLC Nand Object
 * \param   addr    [IN]    {Uint32}        Address from where data should
 *                                          read or to which data has to be
 *                                          written.                       */

/* @} LLCNANDConfig */

/**
 * \defgroup LLCNANDIo LLC NAND IO
 *
 * Interfaces for performing IO operations and for obtaining the status of
 * an IO operation
 *
 * @{
 */
#ifdef DDC_NAND_8_BIT
    #define LLC_nandReadByte(hNand) (*(VUint8*)hNand->bank.CExDATA)
#else
    #define LLC_nandReadByte(hNand) (*(VUint16*)hNand->bank.CExDATA)
#endif
/**< Read data from NAND
 * \param   hNand   [IN]    {Uint16}        Data to be read from NAND      */

#ifdef DDC_NAND_8_BIT
    #define LLC_nandWriteByte(hNand ,data)                                  \
        (*(VUint8*)hNand->bank.CExDATA = (Uint8)data)
#else
    #define LLC_nandWriteByte(hNand, data)                                  \
        (*(VUint16*)hNand->bank.CExDATA = (Uint16)data)
#endif
/**< Write data to NAND
 * \param   hNand   [IN]    {LLC_NandObj *} Handle to the LLC Nand Object
 * \param   data    [IN]    {Uint16}        Data to be written to NAND     */

#define LLC_nandReadECC(hNand)                                              \
    (*(VUint32*)hNand->bank.NANDFxECC)
/**< Read the ECC register
 * \param   hNand   [IN]    {LLC_NandObj *} Handle to the LLC Nand Object   */

#define LLC_nandIsStatusWriteProtected(status)                              \
(!((status & LLC_NAND_BB_CMD_WP_STATUS_OPEN) == LLC_NAND_BB_CMD_WP_STATUS_OPEN))
/**< Check if NAND is write protected
 *  \param  status  [IN]    {Uint8} status byte from status read command    */

#define LLC_nandHasRandomRead(nandOptions)                                \
((nandOptions & LLC_NAND_BB_RANDOM_PAGE_READ) == LLC_NAND_BB_RANDOM_PAGE_READ)
/**< Check if Big block NAND suports random read
 *  \param  nandOptions   [IN]    Options for Big Block NAND              */

#define LLC_nandHasRandomWrite(nandOptions)                               \
((nandOptions & LLC_NAND_BB_RANDOM_PAGE_WRITE) == LLC_NAND_BB_RANDOM_PAGE_WRITE)
/**< Check if NAND supports random write
 *  \param  nandOptions   [IN]    Options for Big Block NAND              */

#define LLC_nandHasCacheProg(nandOptions)                                 \
((nandOptions & LLC_NAND_BB_CACHE_PROG) == LLC_NAND_BB_CACHE_PROG)
/**< Check if Big block NAND supports cache programming
 *  \param  nandOptions   [IN]    Options for Big Block NAND              */

#define LLC_nandHasCopyBack(nandOptions)                                  \
((nandOptions & LLC_NAND_BB_COPY_BACK) == LLC_NAND_BB_COPY_BACK)
/**< Check if Big block NAND supports copy back
 *  \param  nandOptions   [IN]    Options for Big Block NAND              */
/* @} LLCNANDIo */

/**
 * \defgroup LLCNANDControl LLC NAND Control
 *
 * Interfaces for controlling the operation of the NAND controller when in a
 * specific mode of operation
 *
 * @{
 */

#define LLC_nandEnableECC(hNand, ECCBank)                                   \
    (hNand->regs->NANDFCR) |= (ECCBank);
/**< Read the ECC register
 * \param   hNand   [IN]    {LLC_NandObj *} Handle to the LLC Nand Object
 * \param   ECCBank [IN]    Bank on which ECC needs to be enabled           */

/* @} LLCNANDControl */

/**
 * \defgroup LLCNANDIntr LLC NAND Interrupt Control
 *
 * Interfaces for controlling the Interrupt generation and handling and for
 * obtaining the status of an interrupt condition
 *
 * @{
 */

#define LLC_nandIntrEnable(hNand)                                           \
{\
    Bool loopFlag = FALSE;\
    do {                                                                    \
        (CSL_FINS(hNand->regs->EIMSR, EMIF_EIMSR_ATMSET, TRUE));            \
        (CSL_FINS(hNand->regs->EIMSR, EMIF_EIMSR_WRMSET,                    \
                                                 LLC_NAND_INTR_WRM_ENABLE));\
    } while (loopFlag);\
}
/**< Enable NAND interrupts.
 * \param   hNand    [IN]    {LLC_NandObj *}  Handle to the LLC Nand Object */

#define LLC_nandIntrDisable(hNand)                                          \
    do {                                                                    \
        (CSL_FINS(hNand->regs->EIMCR, EMIF_EIMCR_ATMCLR, TRUE));            \
        (CSL_FINS(hNand->regs->EIMCR, EMIF_EIMCR_WRMCLR,                    \
                                                LLC_NAND_INTR_WRM_DISABLE));\
    } while (0)
/**< Disable NAND interrupts.
 * \param   hNand    [IN]    {LLC_NandObj *}  Handle to the LLC Nand Object */

#define LLC_nandIntrReadAT(hNand)                                           \
        (CSL_FINS(hNand->regs->EIRR, EMIF_EIRR_AT))
/**< Read the Asynchronous timeout status of NAND interrupt register.
 * \param   hNand    [IN]    {LLC_NandObj *}  Handle to the LLC Nand Object */

#define LLC_nandIntrReadLT(hNand)                                           \
        (CSL_FINS(hNand->regs->EIRR, EMIF_EIRR_LT))
/**< Read the line trap status of NAND interrupt register.
 * \param   hNand    [IN]    {LLC_NandObj *}  Handle to the LLC Nand Object */

#define LLC_nandIntrReadWR(hNand)                                           \
        (CSL_FEXT(hNand->regs->EIRR, EMIF_EIRR_WR))
/**< Read the Wait rise bits of NAND interrupt register.
 * \param   hNand    [IN]    {LLC_NandObj *}  Handle to the LLC Nand Object */

#define LLC_nandIntrClear(hNand)                                            \
{\
    Bool loopFlag = FALSE;\
    do {                                                                    \
        (CSL_FINS(hNand->regs->EIMR, EMIF_EIMR_ATM, TRUE));                 \
        (CSL_FINS(hNand->regs->EIMR,EMIF_EIMR_WRM,LLC_NAND_INTR_WRM_CLEAR));\
    } while (loopFlag);\
}
/**< Clear the NAND interrupt register.
 * \param   hNand    [IN]    {LLC_NandObj *}  Handle to the LLC Nand Object */

/* @} LLCNANDIntr */

/**
 * \defgroup LLCNANDDma LLC NAND DMA Operation and Control
 *
 * Interfaces for programming the DMA and for obtaining the status of a
 * specific DMA channel
 *
 * @{
 */

/* @} LLCNANDDma */

/* @} LLCNandInterfaces */

/* @} LLCNANDMain */

#ifdef __cplusplus
}
#endif

#endif  /* _LLC_NANDIF_H_ */
