/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 * \file    llc_nandTypes.h
 *
 * \brief   NAND low-level controller interface
 *
 * This file contains the definitions that are used to abstract the low level
 * controller so that the DDC can be agnostic of the underlying controller.
 * The DDC shall assume a union of all functionalities and the specific
 * underlying controller shall implement or stub out specific interfaces. This
 * file contains the NAND LLC interfaces implemented for the NAND controller
 * on DM6437.
 *
 * (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \note       Set tabstop to 4 (:se ts=4) while viewing this file in an
 *              editor
 *
 *  \author     AK Mistral, Himamsu Mistral
 *
 *  \version    1.0 AK Mistral      Created.
 *  \version    1.1 Himamsu Mistral Code cleanup and release for BFT5
 *  \version    1.2 Himamsu Mistral Release for BFT6
 *  \version    1.3 Himamsu Mistral Release for BFT7
 *  \version    1.4 Himamsu Mistral Added status command status-es,
 *                                  ECC options, WP feature and BB extra
 *                                  command options.
 *                                  - Release for BFT8
 */

#ifndef _LLC_NANDTYPES_H_
#define _LLC_NANDTYPES_H_

#ifdef CHIP_DM6437
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/soc.h>
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/cslr_emif2.h>
#endif

#ifdef CHIP_C6424
#include <ti/sdo/pspdrivers/soc/c6424/dsp/soc.h>
#include <ti/sdo/pspdrivers/soc/c6424/dsp/cslr_emif2.h>
#endif

#include <ti/sdo/edma3/drv/edma3_drv.h>
#include <ti/sdo/pspdrivers/pal_os/pal_os.h>
#include <ti/sdo/pspdrivers/pal_os/bios/psputils.h>

#ifdef __cplusplus
extern "C" {
#endif

#define NAND_Debug   PSP_DEBUG
#define NAND_Debug1  PSP_DEBUG1
#define NAND_Debug2  PSP_DEBUG2
#define NAND_Debug3  PSP_DEBUG3
#define NAND_Debug4  PSP_DEBUG4

/**
 * \defgroup LLCNandTypesMain LLC NAND Specific Definition
 *
 * Top-level Encapsulation of all documentation for NAND LLC
 *
 * @{
 */

/**
 * \defgroup LLCNandGenericDefs LLC NAND Generic Definitions Structures
 *
 * NAND definitions that are applicable to any controller and agnostic of a
 * specific IP block
 *
 * @{
 */

/**
 * \brief NAND Status Bit definitions - Big Block
 *
 */
#define LLC_NAND_BB_CMD_PGRM_STATUS_PASS        0x0000
/**< Program/erase operation passed                                         */
#define LLC_NAND_BB_CMD_CACHE_PGRM_PASS         0x0000
/**< NAND cache program command passed                                      */
#define LLC_NAND_BB_CMD_IO2_STATUS_PASS         0x0000
/**< Not used                                                               */
#define LLC_NAND_BB_CMD_IO3_STATUS_PASS         0x0000
/**< Not used                                                               */
#define LLC_NAND_BB_CMD_IO4_STATUS_PASS         0x0000
/**< Not used                                                               */
#define LLC_NAND_BB_CMD_CACHE_PGRM_READY        0x0020u
/**< NAND Ready for command after a cache opearion                          */
#define LLC_NAND_BB_CMD_CMD_STATUS_READY        0x0040u
/**< NAND Ready for commands after an operation                             */
#define LLC_NAND_BB_CMD_WP_STATUS_OPEN          0x0080u
/**< Not write protected                                                    */

#define LLC_NAND_BB_CMD_STATUS_READY    (LLC_NAND_BB_CMD_CMD_STATUS_READY | \
                                         LLC_NAND_BB_CMD_CACHE_PGRM_READY | \
                                         LLC_NAND_BB_CMD_CACHE_PGRM_PASS  | \
                                         LLC_NAND_BB_CMD_PGRM_STATUS_PASS)
/**< Status check for any Big block NAND operation                          */

/**
 * \brief NAND Status Bit definitions - Small Block
 *
 */
#define LLC_NAND_SB_CMD_PGRM_STATUS_PASS        0x0000
/**< Program/erase operation                                                */
#define LLC_NAND_SB_CMD_IO1_STATUS_PASS         0x0000
/**< Not used                                                               */
#define LLC_NAND_SB_CMD_IO2_STATUS_PASS         0x0000
/**< Not used                                                               */
#define LLC_NAND_SB_CMD_IO3_STATUS_PASS         0x0000
/**< Not used                                                               */
#define LLC_NAND_SB_CMD_IO4_STATUS_PASS         0x0000
/**< Not used                                                               */
#define LLC_NAND_SB_CMD_IO5_STATUS_PASS         0x0000
/**< Not used                                                               */
#define LLC_NAND_SB_CMD_CMD_STATUS_READY        0x0040u
/**< NAND Ready for commands after an operation                             */
#define LLC_NAND_SB_CMD_WP_STATUS_OPEN          0x0080u
/**< Not write protected                                                    */

#define LLC_NAND_SB_CMD_STATUS_READY    (LLC_NAND_SB_CMD_CMD_STATUS_READY | \
                                         LLC_NAND_SB_CMD_PGRM_STATUS_PASS)
/**< Status check for any Small block NAND operation                        */

#define LLC_NAND_FLAG_FIRSTBITPOS       0
/**< First bit-position for NAND device flags                               */
#define LLC_NAND_MEDIA_WRITE_PROTECTED (1L << (LLC_NAND_FLAG_FIRSTBITPOS + 0))
/**< Write protected bit-flag                                               */

#define LLC_NAND_FLAG_LASTBITPOS        1u
/**< Last bit-position for NAND device flags                                */

/**
 * \brief Extended ID masks
 *
 */
#define LLC_NAND_PAGE_SIZE_MASK                 0x03u
/**< Page size mask of extended ID                                          */
#define LLC_NAND_BLOCK_SIZE_MASK                0x30u
/**< Block size mask of extended ID                                         */
#define LLC_NAND_RED_AREA_SIZE_MASK             0x04u
/**< Redundant area size mask of extended ID                                */
#define LLC_NAND_ORGANIZATION_MASK              0x40u
/**< Organization (8/16 bit) mask of extended ID                            */

/**
 * \brief ECC options for NAND Banks
 *
 */
#define LLC_NAND_ECC_BANK1_ENABLE                   0x100u
/**< Enable ECC calculation for bank 1 - maps to CS2ECC                     */
#define LLC_NAND_ECC_BANK2_ENABLE                   0x200u
/**< Enable ECC calculation for bank 2 - maps to CS3ECC                     */
#define LLC_NAND_ECC_BANK3_ENABLE                   0x400u
/**< Enable ECC calculation for bank 3 - maps to CS4ECC                     */
#define LLC_NAND_ECC_BANK4_ENABLE                   0x800u
/**< Enable ECC calculation for bank 4 - maps to CS5ECC                     */
#define LLC_NAND_ECC_BANKx_ENABLE                (LLC_NAND_ECC_BANK1_ENABLE |\
                                                  LLC_NAND_ECC_BANK2_ENABLE |\
                                                  LLC_NAND_ECC_BANK3_ENABLE |\
                                                  LLC_NAND_ECC_BANK4_ENABLE)
/**< Enable ECC calculation for all banks                                   */

/**
 * \brief Big block NAND extra commands
 *
 */
#define LLC_NAND_BB_CACHE_PROG                  0x01u
/**< Big-Block NAND has cache programming command/feature                   */
#define LLC_NAND_BB_COPY_BACK                   0x02u
/**< Big-Block NAND has copy-back command/feature                           */
#define LLC_NAND_BB_RANDOM_PAGE_READ            0x04u
/**< Big-Block NAND has random read/random output command/feature           */
#define LLC_NAND_BB_RANDOM_PAGE_WRITE           0x08u
/**< Big-Block NAND has random write/random input command/feature           */

#define LLC_NAND_BB_OPTIONS                   (LLC_NAND_BB_CACHE_PROG       |\
                                               LLC_NAND_BB_COPY_BACK        |\
                                               LLC_NAND_BB_RANDOM_PAGE_READ |\
                                               LLC_NAND_BB_RANDOM_PAGE_WRITE )
/**< Extra commands/options/features only for Big Block NAND devices        */

#define LLC_NAND_INTR_WRM_ENABLE                0x3Fu
/**< Enable Wait rise (mask) bits in NAND interrupt register                */
#define LLC_NAND_INTR_WRM_DISABLE               0x3Fu
/**< Disable Wait rise (mask) bits in NAND interrupt register               */
#define LLC_NAND_INTR_WRM_CLEAR                 0x3Fu
/**< Clear Wait rise (mask) bits in NAND interrupt register                 */

/**
 * \brief NAND Width enumeration
 */
typedef enum {
    LLC_NAND_8_BIT          = 0U,
    /**< External Device Bus Width 8 bit                                    */
    LLC_NAND_16_BIT         = 1U
    /**< External Device Bus Width 16 bit                                   */
} LLC_NandWidth;


/**
 * \brief Type of NAND detected.
 */
typedef enum
{
    LLC_NAND_NONE = 0u,
    /**< No NAND detected                                                   */
    LLC_NAND_BIG_BLOCK,
    /**< Big Block NAND detected                                            */
    LLC_NAND_SMALL_BLOCK,
    /**< Small Block NAND detected                                          */
    LLC_NAND_INVALID
    /**< Unknown/Invalid NAND                                               */
} LLC_NandType;

/**
 * \brief NAND Page size enumeration
 */
typedef enum {
    LLC_NAND_PAGESZ_256         = 0u,
    /**< Page size is 256 + 8bytes                                          */
    LLC_NAND_PAGESZ_512         = 1u,
    /**< Page size is 512 + 16bytes                                         */
    LLC_NAND_PAGESZ_1024        = 2u,
    /**< Page size is 1024 + 32 bytes                                       */
    LLC_NAND_PAGESZ_2048        = 3u
    /**< Page size is 2048 + 64 bytes                                       */
}LLC_NandPageSize;

/**
 * \brief   NAND Asynchronous Bank number
 */
typedef enum {
    LLC_NAND_BANK_0 = 0u,
    /**< NAND Bank 0                                                        */
    LLC_NAND_BANK_1 = 1u,
    /**< NAND Bank 1                                                        */
    LLC_NAND_BANK_2 = 2u,
    /**< NAND Bank 2                                                        */
    LLC_NAND_BANK_3 = 3u,
    /**< NAND Bank 3                                                        */
    LLC_NAND_BANK_4 = 4u,
    /**< NAND Bank 4                                                        */
    LLC_NAND_BANK_MAX
    /**< NAND Bank Max (Book-keep)                                          */
} LLC_NandBankNo;

/**
 * \brief NAND Basic Configuration enumeration
 */
typedef struct {
    LLC_NandWidth       llcNandWidth;
    /**< External Device Width. Keep. Useful for later use!                 */
    LLC_NandPageSize    llcNandPageSize;
    /**< NAND page Size                                                     */
    LLC_NandBankNo      bankNo;
    /**< Asynchronous bank                                                  */
} LLC_NandBasicConfig;

/**
 * \brief NAND hardware initialization structure
 *
 * This structure contains the hardware initialization data.
 */
typedef struct {
    LLC_NandBasicConfig         cfgBasic;
    /**< NAND Hardware control setup                                        */
} LLC_NandHwConfig;

/* @} LLCNandGenericDefs */

/**
 * \defgroup LLCNandControllerSpecificDefs LLC Specific Definitions
 *
 * Definitions specific to this IP of the NAND
 *
 * @{
 */

/**
 * \brief Current Asynchronous NAND Bank configuration.
 */
 typedef struct {
    volatile Uint32     CExDATA;
    /**< Current NAND Data register                                         */
    volatile Uint32     CExALE;
    /**< Current NAND Addres latch register                                 */
    volatile Uint32     CExCLE;
    /**< Current NAND Command latch register                                */
    volatile Uint32     NANDFxECC;
    /**< Current NAND ECC register                                          */
 } LLC_NandAsyncBank;

 /**
  * \brief NAND object
  */
typedef struct {
    Uint32              instId;
    /**< Instance Id of NAND on the SoC.                                    */
    CSL_Emif2Regs        *regs;
    /**< NAND register overlay structure                                    */
    LLC_NandWidth       nandWidth;
    /**< Width of NAND (8 or 16 bit)                                        */
    Int                 intrNumNand;
    /**< Interrupt number of NAND                                           */
    LLC_NandAsyncBank   bank;
    /**< Current NAND Asynchronous bank                                     */
    Uint32              clkFreq;
    /**< ARM clock Freq; for EMIF based NAND.                               */
    Uint8               eraseAtInit;
    /**< Erase all the NAND devices on board during initialization.
     * <b> NOTE: TO BE USED WITH CAUTION! </b>
     */
    PAL_OsSemHandle     syncSem;
    /**< Synchronization Semaphore                                          */
    PSP_OpMode          opMode;
    /**< Current Mode of Operation                                          */
    Uint32              dmaChannel;
    /**< DMA Channel Number                                                 */
    Uint32              nandDmaTcc;
    /**< DMA TCC Number                                                     */
} LLC_NandObj;

typedef LLC_NandObj *LLC_NandHandle;
/**< Handle to the NAND object structure                                    */

/* @} LLCNandControllerSpecificDefs */

/* @} LLCNandTypesMain */
#ifdef __cplusplus
}
#endif

#endif  /* _LLC_NANDTYPES_H_ */
