/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \file   dda_previewer.c
 *
 *  \brief  previewer implementaion of DDA layer
 *
 *  This file contains the implementation of DDA layer of the previewer driver.
 *
 *  (C) Copyright 2010, Texas Instruments, Inc
 *
 *
 *  \author     EI2
 *
 *  \version    0.1             Created
 *              1.0             First Release
 */

/******************************************************************************
  Header File Inclusion
 ******************************************************************************/

#include <std.h>
#include <mem.h>
#include <ecm.h>
#include <c64.h>
#ifdef PREVIEWER_DEBUG
#ifdef DDA_PREVIEWER_DEBUG_PRINTF
#include <ti/sdo/pspdrivers/pal_os/bios/psputils.h>
#endif /* DDA_PREVIEWER_DEBUG_PRINTF */
#endif /* PREVIEWER_DEBUG */

#include <ti/sdo/pspdrivers/common/psp_common.h>

#include <ti/sdo/pspdrivers/drivers/previewer/psp_previewer.h>
#include <ti/sdo/pspdrivers/drivers/previewer/src/dda_previewer.h>
#include <ti/sdo/pspdrivers/drivers/previewer/src/ddc_previewer.h>
#include <ti/sdo/pspdrivers/drivers/previewer/src/llc_previewer.h>
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/tistdtypes.h>
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/soc.h>


/******************************************************************************
  BIOS SPECIFIC FUNCTIONS
 ******************************************************************************/
/**
 * \brief DDA_prevSetInterrupt
 *
 * It registers interrupt handler for previewer.
 *
 *  \param  intNum     [IN]    interrupt number
 *  \param  isrRoutine [IN]    interrupt routine
 *  return void
 */
static void DDA_prevSetInterrupt(Uint8 intNum, ECM_Fxn isrRoutine);

/**
 * \brief DDA_prevUnsetInterrupt
 *
 * It unregisters Interrupt handler for previewer.
 *
 *  \param intNum     [IN]     interrupt number
 *  return void
 */
static void DDA_prevUnsetInterrupt(Uint8 intNum);

/******************************************************************************
  Internal Functions
 ******************************************************************************/
/**
 * \brief DDA_DDCtoDDAErrorCode
 *
 * It maps DDC error codes to PSP error codes.
 *
 *  \param ddcErrorCode     [IN]     DDC error code
 *  return PSP error code, corresponding to DDC error code.
 */
static PSP_Result DDA_DDCtoDDAErrorCode(DDC_prevResult ddcErrorCode);

/******************************************************************************
  Macros
 ******************************************************************************/

/* Debug Macros */
#ifdef PREVIEWER_DEBUG
/* macro to indicate DDA file */
#define DDA_PREVIEWER_DEBUG_FILE   3

#define PREVIEWER_DEBUG_ERROR_TRACE_SIZE    16
extern Uint32 prevDebugErrorTraceIndex;
extern Uint32 prevDebugErrorTraceArray[];

#define PREVIEWER_DEBUG_ERR_TRACE(LINENO) \
{\
    prevDebugErrorTraceArray[prevDebugErrorTraceIndex] = (Uint32)(\
        (DDA_PREVIEWER_DEBUG_FILE << 24) | (LINENO));\
    prevDebugErrorTraceIndex++;\
    prevDebugErrorTraceIndex %= PREVIEWER_DEBUG_ERROR_TRACE_SIZE;\
}

#ifdef DDA_PREVIEWER_DEBUG_ALL_TRACE
#define PREVIEWER_DEBUG_TRACE_SIZE        128
extern Uint32 prevDebugTraceIndex;
extern Uint32 prevDebugTraceArray[];
#define DDA_PREVIEWER_DEBUG_REG_TRACE(LINENO) \
{\
    prevDebugTraceArray[prevDebugTraceIndex] = (Uint32)(\
        (DDA_PREVIEWER_DEBUG_FILE << 24) | (LINENO));\
    prevDebugTraceIndex++;\
    prevDebugTraceIndex %= PREVIEWER_DEBUG_TRACE_SIZE;\
}
#else /* DDA_PREVIEWER_DEBUG_ALL_TRACE */
#define DDA_PREVIEWER_DEBUG_REG_TRACE(LINENO)
#endif /* DDA_PREVIEWER_DEBUG_ALL_TRACE */

#ifdef DDA_PREVIEWER_DEBUG_PRINTF
#define DDA_PREVIEWER_DEBUG_PRINT(STR) PSP_DEBUG1("DDA%s", (STR));
#else /* DDA_PREVIEWER_DEBUG_PRINTF */
#define DDA_PREVIEWER_DEBUG_PRINT(STR)
#endif /* DDA_PREVIEWER_DEBUG_PRINTF */

#else /* PREVIEWER_DEBUG */

#define DDA_PREVIEWER_DEBUG_REG_TRACE(LINENO)
#define PREVIEWER_DEBUG_ERR_TRACE(LINENO)
#define DDA_PREVIEWER_DEBUG_PRINT(STR)
#endif /* PREVIEWER_DEBUG */




/******************************************************************************
  Data-Structures
 ******************************************************************************/
/**
 * \brief previewer object of port structure
 */
DDA_prevPortObject ddaPrevPortObj =
{
    NULL,
    DDA_PREVIEWER_DEVICE_DELETED,
    NULL
};

/**
 * \brief to store interrupt mapping info.
 */
Uint8 ddaPrevInterruptMapInfo = 0xFF;

/**
 * \brief to store interrupt mapping is needed or not
 */
Uint32 ddaPrevInterruptMapRequired = 0xFF;

/**
 * \brief to store segment id where DDA memory is allocated
 */
Uint32 ddaPrevSegId = 0xFF;

/******************************************************************************
  Functions
 ******************************************************************************/



/**
 * \brief  PSP_prevCreate
 * It creates a given previewer device at DDA layer.
 */
PSP_Result PSP_prevCreate(PSP_Handle* devHandle)
{
    PSP_Result prevCreateReturnCode = PSP_E_DRIVER_INIT;
    DDC_prevResult prevCreateDDCRetCode = DDC_PREV_NOT_SUPPORTED;

    DDA_PREVIEWER_DEBUG_REG_TRACE(__LINE__);

    /* if device instance is already created, raise an error */
    if (DDA_PREVIEWER_DEVICE_DELETED != ddaPrevPortObj.state)
    {
        prevCreateReturnCode = PSP_E_INVAL_STATE;
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
    }
    else /* if device instance is not created yet */
    {
        DDA_PREVIEWER_DEBUG_REG_TRACE(__LINE__);
        /* call DDC level function to create device instance
         * at DDC layer */
        prevCreateDDCRetCode = DDC_prevDeviceCreate(
            &(ddaPrevPortObj.ddcDeviceHandle));
        /* if device is successfully created at DDC layer */
        if (DDC_PREV_SOK == prevCreateDDCRetCode)
        {
            DDA_PREVIEWER_DEBUG_REG_TRACE(__LINE__);
            /* as device instance has been created successfully,
             * change state of device */
            ddaPrevPortObj.state = DDA_PREVIEWER_DEVICE_CREATED;
            *devHandle = &ddaPrevPortObj;
            prevCreateReturnCode = PSP_SOK;
        }
        else /* device is not created at DDC layer */
        {
            /* as device instance has been failed to created,
             * fill global instance of port structure with
             * default values */
            ddaPrevPortObj.state = DDA_PREVIEWER_DEVICE_DELETED;
            ddaPrevPortObj.ddcDeviceHandle = NULL;
            ddaPrevPortObj.channelHandle = NULL;
            *devHandle = NULL;
            prevCreateReturnCode
                = DDA_DDCtoDDAErrorCode(prevCreateDDCRetCode);
            PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
        }
    } /* end if DDA_PREVIEWER_DEVICE_DELETED ... ddaPrevPortObj.state  */
    return prevCreateReturnCode;
}



/**
 * \brief  PSP_prevOpen
 * It Open Channel of the previewer device.
 */
PSP_Result PSP_prevOpen(PSP_Handle *chanHandle,
    PSP_Handle devHandle,
    PSP_previewerChannelCreateMode *chanParams)
{
    PSP_Result prevOpenReturnCode = PSP_E_DRIVER_INIT;
    DDC_prevResult prevOpenDDCRetCode = DDC_PREV_NOT_SUPPORTED;
    Ptr prevOpenChannelHandle = NULL;
    ECM_Fxn prevCreateIsrFuncPtr = NULL;

    DDA_prevPortHandle prevOpenPortHandle;

    if ((NULL == devHandle) || (NULL == chanHandle))
    {
        prevOpenReturnCode = PSP_E_INVAL_PARAM;
        DDA_PREVIEWER_DEBUG_PRINT("Invalid Parameter\n");
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
    }
    else
    {
        DDA_PREVIEWER_DEBUG_REG_TRACE(__LINE__);
        prevOpenPortHandle = (DDA_prevPortHandle)devHandle;
        /* if passed port structure is different then global port structure,
         * raise an error */
        if ((&ddaPrevPortObj) != prevOpenPortHandle)
        {
            prevOpenReturnCode = PSP_E_INVAL_PARAM;
            DDA_PREVIEWER_DEBUG_PRINT("Invalid Parameter\n");
            PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
        }
        else /* if port structure is proper one */
        {
            DDA_PREVIEWER_DEBUG_REG_TRACE(__LINE__);
            /* check that if device is deleted or channel is already open then
             * raise an error */
            if ((DDA_PREVIEWER_DEVICE_DELETED == prevOpenPortHandle->state)
                || (NULL != prevOpenPortHandle->channelHandle))
            {
                prevOpenReturnCode = PSP_E_INVAL_STATE;
                DDA_PREVIEWER_DEBUG_PRINT("Invalid State\n");
                PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
            }
            else /* if device instance is created and channel is not
                    opened yet */
            {
                DDA_PREVIEWER_DEBUG_REG_TRACE(__LINE__);
                /* allocate a memory for channel instance at DDA layer,
                 * alignment is not needed so 0 is passed as an alignment
                 * value */
                prevOpenChannelHandle = MEM_alloc(chanParams->segId,
                    sizeof(DDA_prevChannelObject), 0);
                /* if system is failed to allocate memory */
                if (MEM_ILLEGAL == prevOpenChannelHandle)
                {
                    prevOpenReturnCode = PSP_E_NO_MEMORY;
                    DDA_PREVIEWER_DEBUG_PRINT("Memory Error\n");
                    PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
                }
                else /* memory is allocated successfully */
                {
                    DDA_PREVIEWER_DEBUG_REG_TRACE(__LINE__);
                    /* update port structure as well as channel structure at DDA
                     * layer */
                    prevOpenPortHandle->channelHandle
                        = (DDA_prevChannelHandle)prevOpenChannelHandle;
                    prevOpenPortHandle->channelHandle->portHandle
                        = prevOpenPortHandle;
                    /* call DDC_prevOpenHandle to open channel at DDC layer */
                    prevOpenDDCRetCode = DDC_prevOpenHandle(chanParams,
                        &(prevOpenPortHandle->channelHandle->ddcChannelHandle),
                        &ddaPrevInterruptMapRequired,
                        (Uint8*)(&ddaPrevInterruptMapInfo),
                        (ddcISRType*)(&prevCreateIsrFuncPtr));
                    /* if open DDC channel fails, raise an error */
                    if (DDC_PREV_SOK != prevOpenDDCRetCode)
                    {
                        /* free allocated memory as instance creation has been
                         * failed */
                        MEM_free(ddaPrevSegId, prevOpenChannelHandle,
                            sizeof(DDA_prevChannelObject));
                        prevOpenPortHandle->channelHandle = NULL;
                        prevOpenReturnCode
                            = DDA_DDCtoDDAErrorCode(prevOpenDDCRetCode);
                        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
                        DDA_PREVIEWER_DEBUG_PRINT("Wrong ret from");
                        DDA_PREVIEWER_DEBUG_PRINT(" OpenHandle\n");
                    }
                    else /* if DDC channel is open successfully */
                    {
                        DDA_PREVIEWER_DEBUG_REG_TRACE(__LINE__);
                        prevOpenReturnCode = PSP_SOK;
                        if (DDC_PREVIEWER_INTERRUPT_MAPPING_NEEDED
                            == ddaPrevInterruptMapRequired)
                        {
                            DDA_PREVIEWER_DEBUG_REG_TRACE(__LINE__);
                            /* call DDA_prevSetInterrupt, pass
                             * ddaPrevInterruptMapInfo it will map interrupt */
                            DDA_prevSetInterrupt(ddaPrevInterruptMapInfo,
                                (ECM_Fxn)prevCreateIsrFuncPtr);
                        }
                        DDA_PREVIEWER_DEBUG_REG_TRACE(__LINE__);
                        *chanHandle = (Ptr)(prevOpenPortHandle->channelHandle);
                    } /* end if PSP_SOK == ... for DDC_prevOpenHandle */
                } /* end if NULL neq prevOpenChannelHandle */
            } /* end if  DDA_PREVIEWER_DEVICE_DELETED eq ... */
        } /* end if ddaPrevPortObj neq prevOpenPortHandle  */
    }
    return prevOpenReturnCode;
}



/**
 * \brief  PSP_prevIoctl
 * It Control Channel of the previewer device.
 */
PSP_Result PSP_prevIoctl(PSP_Handle chanHandle,
    PSP_previewerControlCmd cmd,
    Ptr cmdArg)
{
    PSP_Result prevIoctlReturnCode = PSP_E_DRIVER_INIT;
    DDC_prevResult prevIoctlDDCRetCode = DDC_PREV_NOT_SUPPORTED;

    if ((NULL == chanHandle)
        || (ddaPrevPortObj.channelHandle != (DDA_prevChannelHandle)chanHandle))
    {
        prevIoctlReturnCode = PSP_E_INVAL_PARAM;
        DDA_PREVIEWER_DEBUG_PRINT("Invalid Parameter\n");
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
    }
    else
    {
        DDA_PREVIEWER_DEBUG_REG_TRACE(__LINE__);
        prevIoctlDDCRetCode = DDC_prevIoctl((DDC_prevChannelHandle)
            (((DDA_prevChannelHandle)chanHandle)->ddcChannelHandle),
            cmd, cmdArg);
        if (DDC_PREV_SOK != prevIoctlDDCRetCode)
        {
            prevIoctlReturnCode
                = DDA_DDCtoDDAErrorCode(prevIoctlDDCRetCode);
            PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
            DDA_PREVIEWER_DEBUG_PRINT("Wrong ret from Ioctl\n");
        }
        else
        {
            DDA_PREVIEWER_DEBUG_REG_TRACE(__LINE__);
            prevIoctlReturnCode = PSP_SOK;
        }
    }
    return prevIoctlReturnCode;
}



/**
 * \brief  PSP_prevClose
 * It is used to close Channel of the previewer device.
 */
PSP_Result PSP_prevClose(PSP_Handle chanHandle)
{
    PSP_Result prevCloseReturnCode = PSP_E_DRIVER_INIT;
    DDC_prevResult prevCloseDDCRetCode = DDC_PREV_NOT_SUPPORTED;

    if ((NULL == chanHandle)
        || ((DDA_prevChannelHandle)chanHandle != ddaPrevPortObj.channelHandle))
    {
        prevCloseReturnCode = PSP_E_INVAL_PARAM;
        DDA_PREVIEWER_DEBUG_PRINT("Invalid Parameter\n");
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
    }
    else
    {
        DDA_PREVIEWER_DEBUG_REG_TRACE(__LINE__);
        /* close channel at DDC layer */
        prevCloseDDCRetCode = DDC_prevCloseHandle(
            (ddaPrevPortObj.channelHandle)->ddcChannelHandle);
        if (DDC_PREV_SOK != prevCloseDDCRetCode)
        {
            prevCloseReturnCode
                = DDA_DDCtoDDAErrorCode(prevCloseDDCRetCode);
            PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
            DDA_PREVIEWER_DEBUG_PRINT("Wrong ret from CloseHandle\n");
        }
        else
        {
            DDA_PREVIEWER_DEBUG_REG_TRACE(__LINE__);
            if (DDC_PREVIEWER_INTERRUPT_MAPPING_NEEDED
                == ddaPrevInterruptMapRequired)
            {
                DDA_PREVIEWER_DEBUG_REG_TRACE(__LINE__);
                /* deactivate interrupt */
                DDA_prevUnsetInterrupt(ddaPrevInterruptMapInfo);
            }
            DDA_PREVIEWER_DEBUG_REG_TRACE(__LINE__);
            /* free allocated memory */
            MEM_free(ddaPrevSegId, ddaPrevPortObj.channelHandle,
                sizeof(DDA_prevChannelObject));
            ddaPrevPortObj.channelHandle = NULL;
            prevCloseReturnCode = PSP_SOK;
        }
    }
    return prevCloseReturnCode;
}



/**
 * \brief  PSP_prevDelete
 * It is used to delete a given previewer device at DDA layer.
 */
PSP_Result PSP_prevDelete(PSP_Handle devHandle)
{
    PSP_Result prevDeleteReturnCode = PSP_E_DRIVER_INIT;
    DDC_prevResult prevDeleteDDCRetCode = DDC_PREV_NOT_SUPPORTED;
    /* if passed port structure is different then global port structure,
     * raise an error */
    if ((NULL == devHandle)
        || ((DDA_prevPortHandle)devHandle  != (&ddaPrevPortObj)))
    {
        prevDeleteReturnCode = PSP_E_INVAL_PARAM;
        DDA_PREVIEWER_DEBUG_PRINT("Invalid Parameter\n");
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
    }
    else
    {
        DDA_PREVIEWER_DEBUG_REG_TRACE(__LINE__);
        /* device instance must be in created state and channel handle should
         * be null */
        if ((DDA_PREVIEWER_DEVICE_CREATED == ddaPrevPortObj.state)
            && (NULL == ddaPrevPortObj.channelHandle))
        {
            DDA_PREVIEWER_DEBUG_REG_TRACE(__LINE__);
            prevDeleteDDCRetCode = DDC_prevDeviceDelete(
                ddaPrevPortObj.ddcDeviceHandle);
            if (DDC_PREV_SOK != prevDeleteDDCRetCode)
            {
                prevDeleteReturnCode
                    = DDA_DDCtoDDAErrorCode(prevDeleteDDCRetCode);
                PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
                DDA_PREVIEWER_DEBUG_PRINT("Wrong ret from DeviceDelete\n");
            }
            else
            {
                DDA_PREVIEWER_DEBUG_REG_TRACE(__LINE__);
                ddaPrevPortObj.ddcDeviceHandle = NULL;
                ddaPrevPortObj.state = DDA_PREVIEWER_DEVICE_DELETED;
                prevDeleteReturnCode = PSP_SOK;
            }
        }
        else
        {
            prevDeleteReturnCode = PSP_E_INVAL_STATE;
            DDA_PREVIEWER_DEBUG_PRINT("Invalid State\n");
            PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
        }

    }
    return prevDeleteReturnCode;
}


/**
 * \brief  PSP_prevGetPSPHandle
 * It returns PSP handle of channel object.
 */
PSP_Result PSP_prevGetPSPHandle(PSP_Handle *chanHandle)
{
    PSP_Result prevGetPSPReturnCode = PSP_E_DRIVER_INIT;

    /* parameter validation */
    if (NULL == chanHandle)
    {
        prevGetPSPReturnCode = PSP_E_INVAL_PARAM;
        DDA_PREVIEWER_DEBUG_PRINT("Invalid Parameter\n");
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
    }
    else
    {
        DDA_PREVIEWER_DEBUG_REG_TRACE(__LINE__);
        *chanHandle = (PSP_Handle)(ddaPrevPortObj.channelHandle);
        prevGetPSPReturnCode = PSP_SOK;
    }
    return prevGetPSPReturnCode;
}


/**
 * \brief  DDA_prevSetInterrupt
 * it is used to enable interrupt and mapped particular interrupt to
 * isrRoutine.
 */
static void DDA_prevSetInterrupt(Uint8 intNum, ECM_Fxn isrRoutine)
{
    ECM_Attrs prevSetIntrEcmAttrs = ECM_ATTRS;
    DDA_PREVIEWER_DEBUG_REG_TRACE(__LINE__);

    /* Disabling the event */
    ECM_disableEvent((Uns)intNum);

    /* Mapping the eventid to the ECM Dispatch of the DSP/BIOS*/
    prevSetIntrEcmAttrs.unmask = 1u;
    ECM_dispatchPlug((Uns)intNum, isrRoutine, &prevSetIntrEcmAttrs);

    /* Enabling the event */
    ECM_enableEvent((Uns)intNum);

    C64_enableIER(HWI_EVT01_CMB_IRQ);

    DDA_PREVIEWER_DEBUG_REG_TRACE(__LINE__);

    return;
}



/**
 * \brief  DDA_prevUnsetInterrupt
 * It is used to disable particular interrupt.
 */
static void DDA_prevUnsetInterrupt(Uint8 intNum)
{
    DDA_PREVIEWER_DEBUG_REG_TRACE(__LINE__);
    ECM_disableEvent((Uns)intNum);
    return;
}



/**
 * \brief DDA_DDCtoDDAErrorCode
 * It is used to map DDC error codes to PSP error codes.
 */
static PSP_Result DDA_DDCtoDDAErrorCode(DDC_prevResult ddcErrorCode)
{
    PSP_Result pspErrorCode = PSP_E_DRIVER_INIT;
    switch(ddcErrorCode)
    {
        case DDC_PREV_INVALID_STATE:
            pspErrorCode = PSP_E_INVAL_STATE;
            break;
        case DDC_PREV_INVALID_PARAM:
            pspErrorCode = PSP_E_INVAL_PARAM;
            break;
        case DDC_PREV_MEMORY_ERROR:
            pspErrorCode = PSP_E_NO_MEMORY;
            break;
        case DDC_PREV_RESOURCE_ERROR:
            pspErrorCode = PSP_E_RESOURCES;
            break;
        case DDC_PREV_OVERFLOW_ERROR:
            pspErrorCode = PSP_E_IO_CANCEL_FAIL;
            break;
        case DDC_PREV_NOT_SUPPORTED:
            pspErrorCode = PSP_E_NOT_SUPPORTED;
            break;
        default:
            pspErrorCode = PSP_E_DRIVER_INIT;
            break;
    }
    return pspErrorCode;
}

