/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \file   ddc_previewer.c
 *
 *  \brief  previewer implementaion of DDC layer
 *
 *  This file contains the implementation of DDC layer of the previewer driver.
 *
 *  (C) Copyright 2010, Texas Instruments, Inc
 *
 *
 *  \author     EI2
 *
 *  \version    0.1             Created
 *              1.0             First Release
 */

/******************************************************************************
  Header File Inclusion
 ******************************************************************************/

#include <std.h>
#ifdef PREVIEWER_DEBUG
#ifdef DDC_PREVIEWER_DEBUG_PRINTF
#include <ti/sdo/pspdrivers/pal_os/bios/psputils.h>
#endif /* DDC_PREVIEWER_DEBUG_PRINTF */
#endif /* PREVIEWER_DEBUG */
#include <ti/sdo/pspdrivers/common/psp_common.h>
#include <ti/sdo/pspdrivers/pal_os/pal_os.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osmem.h>
#include <ti/sdo/pspdrivers/pal_os/pal_ossem.h>
#include <ti/sdo/pspdrivers/drivers/previewer/psp_previewer.h>
#include <ti/sdo/pspdrivers/drivers/previewer/src/ddc_previewer.h>
#include <ti/sdo/pspdrivers/drivers/previewer/src/llc_previewer.h>

#include <ti/sdo/pspdrivers/soc/dm6437/dsp/csl_error.h>
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/soc.h>
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/cslr_prev.h>
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/cslr_vpss.h>
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/tistdtypes.h>

#ifdef PREV_INSTRUMENTATION_ENABLED
#include <ti/sdo/pspdrivers/pal_os/bios/psp_log.h>
#endif /* PREV_INSTRUMENTATION_ENABLED */

/******************************************************************************
  Macros
 ******************************************************************************/
/* Debug Macros */
#ifdef PREVIEWER_DEBUG
/* macro to indicate DDC file */
#define DDC_PREVIEWER_DEBUG_FILE   3

#define PREVIEWER_DEBUG_ERROR_TRACE_SIZE    16
Uint32 prevDebugErrorTraceIndex = 0;
Uint32 prevDebugErrorTraceArray[PREVIEWER_DEBUG_ERROR_TRACE_SIZE];

#define PREVIEWER_DEBUG_ERR_TRACE(LINENO) \
{\
    prevDebugErrorTraceArray[prevDebugErrorTraceIndex] = (Uint32)(\
        (DDC_PREVIEWER_DEBUG_FILE << 24) | (LINENO));\
    prevDebugErrorTraceIndex++;\
    prevDebugErrorTraceIndex %= PREVIEWER_DEBUG_ERROR_TRACE_SIZE;\
}

#ifdef DDC_PREVIEWER_DEBUG_ALL_TRACE
#define PREVIEWER_DEBUG_TRACE_SIZE        128
Uint32 prevDebugTraceIndex = 0;
Uint32 prevDebugTraceArray[PREVIEWER_DEBUG_TRACE_SIZE];
#define DDC_PREVIEWER_DEBUG_REG_TRACE(LINENO) \
{\
    prevDebugTraceArray[prevDebugTraceIndex] = (Uint32)(\
        (DDC_PREVIEWER_DEBUG_FILE << 24) | (LINENO));\
    prevDebugTraceIndex++;\
    prevDebugTraceIndex %= PREVIEWER_DEBUG_TRACE_SIZE;\
}
#else /* DDC_PREVIEWER_DEBUG_ALL_TRACE */
#define DDC_PREVIEWER_DEBUG_REG_TRACE(LINENO)
#endif /* DDC_PREVIEWER_DEBUG_ALL_TRACE */

#ifdef DDC_PREVIEWER_DEBUG_PRINTF
#define DDC_PREVIEWER_DEBUG_PRINT(STR) PSP_DEBUG1("DDC%s", (STR));
#else /* DDC_PREVIEWER_DEBUG_PRINTF */
#define DDC_PREVIEWER_DEBUG_PRINT(STR)
#endif /* DDC_PREVIEWER_DEBUG_PRINTF */

#else /* PREVIEWER_DEBUG */

#define DDC_PREVIEWER_DEBUG_REG_TRACE(LINENO)
#define PREVIEWER_DEBUG_ERR_TRACE(LINENO)
#define DDC_PREVIEWER_DEBUG_PRINT(STR)
#endif /* PREVIEWER_DEBUG */



/******************************************************************************
  Internal Function declaration
 ******************************************************************************/

/**
 *  \brief DDC_prevPerformRegOverlaying
 *
 *  This is previewer register overlaying function.
 *  It will Perform register overlaying.
 *
 *  \return  success    DDC_PREV_SOK
 *           failure    DDC error
 */
static DDC_prevResult DDC_prevPerformRegOverlaying();

/**
 *  \brief DDC_prevSetParameters
 *
 *  It will set parameters in previewer register.
 *
 *  \param  ddcChannelHandle     [IN]  channel handle
 *  \param  DDC_prevParams  [IN]  parameter structure passed by
 *                                     application, according to that
 *                                     values will be set in registers.
 *  \return  success    DDC_PREV_SOK
 *           failure    DDC error
 */
static DDC_prevResult DDC_prevSetParameters(
    DDC_prevChannelHandle ddcChannelHandle,
    PSP_previewerParams *DDC_prevParams
    );

/**
 *  \brief DDC_prevGetParameters
 *
 *  It will get parameters from previewer register.
 *
 *  \param  ddcChannelHandle     [IN]  channel handle
 *  \param  DDC_prevParams  [OUT] parameter structure passed by
 *                                     application, in which values will be
 *                                     stored.
 *  \return  success    DDC_PREV_SOK
 *           failure    DDC error
 */
static DDC_prevResult DDC_prevGetParameters(
    DDC_prevChannelHandle ddcChannelHandle,
    PSP_previewerParams *DDC_prevParams
    );

/**
 *  \brief DDC_prevValidateParameters
 *
 *  It will validate previewer parameters.
 *
 *  \param  ddcChannelHandle     [IN]  channel handle
 *  \param  DDC_prevParams  [IN] parameter structure passed by
 *                                    application, which will be validated.
 *  \return  success    DDC_PREV_SOK
 *           failure    DDC error
 */
static DDC_prevResult DDC_prevValidateParameters(
    DDC_prevChannelHandle ddcChannelHandle,
    PSP_previewerParams *DDC_prevParams
    );

/**
 *  \brief DDC_prevGetStatus
 *
 *  It will get status of previewer by reading status from PCR register.
 *
 *  \param  ddcChannelHandle     [IN]  channel handle
 *  \param  prevStatus           [OUT] parameter structure passed by
 *                                     application, in which previewer status
 *                                     value will be stored.
 *  \return  success    DDC_PREV_SOK
 *           failure    DDC error
 */
static DDC_prevResult DDC_prevGetStatus(
    DDC_prevChannelHandle ddcChannelHandle,
    PSP_previewerStatus *prevStatus
    );

/**
 *  \brief DDC_prevGetDarkFrameStatus
 *
 *  It will get status of dark frame subtract failure status by reading status
 *  from PCR register.
 *
 *  \param  ddcChannelHandle     [IN]  channel handle
 *  \param  prevDarkFrameStatus  [OUT] parameter structure passed by
 *                                              application, in which dark frame
 *                                              status value will be stored.
 *  \return  success    DDC_PREV_SOK
 *           failure    DDC error
 */
static DDC_prevResult DDC_prevGetDarkFrameStatus(
    DDC_prevChannelHandle ddcChannelHandle,
    PSP_previewerDarkFrameStatus *prevDarkFrameStatus
    );

/**
 *  \brief DDC_prevPreview
 *
 *  It will trigger previewing operation by writing source desti address and
 *  then will set enable bit. is used in one shot mode.
 *
 *  \param  ddcChannelHandle [IN]  channel handle
 *
 *  \param  prevPreview      [IN] parameter structure passed by
 *                                application, according to parameter in this
 *                                previewer will be triggered.
 *
 *  \return  success    DDC_PREV_SOK
 *           failure    DDC error
 *
 */
static DDC_prevResult DDC_prevPreview(
    DDC_prevChannelHandle ddcChannelHandle,
    PSP_preview *prevPreview
    );

/**
 *  \brief DDC_prevGetCropSize
 *
 *  It will calculates crop size as per previewer parameters.
 *
 *  \param  ddcChannelHandle [IN]  channel handle
 *  \param  prevCropSize     [OUT] parameter structure passed by
 *                                 application, in which crop size will be
 *                                 filled.
 *  \return  success    DDC_PREV_SOK
 *           failure    DDC error
 */
static DDC_prevResult DDC_prevGetCropSize(
    DDC_prevChannelHandle ddcChannelHandle,
    PSP_previewerCropSize *prevCropSize
    );

/**
 *  \brief DDC_prevSetReadReqExpand
 *
 *  It will set read request expand field.
 *
 *  \param  ddcChannelHandle [IN]  channel handle
 *  \param  prevReadReqExp   [IN] parameter structure passed by
 *                                application, from which read request expand
 *                                field will be taken.
 *  \return  success    DDC_PREV_SOK
 *           failure    DDC error
 */
static DDC_prevResult DDC_prevSetReadReqExpand(
    DDC_prevChannelHandle ddcChannelHandle,
    PSP_previewerReadReqExp *prevReadReqExp
    );

/**
 *  \brief DDC_prevSetDFC
 *
 *  It will set dark frame capture parameter.
 *
 *  \param  ddcChannelHandle [IN]  channel handle
 *  \param  prevSetDFC       [IN]  parameter structure passed by
 *                                 application, according to that dark frame
 *                                 capture feature will be enabled or disabled.
 *  \return  success    DDC_PREV_SOK
 *           failure    DDC error
 */
static DDC_prevResult DDC_prevSetDFC(
    DDC_prevChannelHandle ddcChannelHandle,
    PSP_previewerDarkFrameCapture *prevSetDFC
    );

/**
 *  \brief DDC_prevGetDFC
 *
 *  It will set dark frame capture parameter.
 *
 *  \param  ddcChannelHandle [IN]  channel handle
 *  \param  prevGetDFC       [OUT] parameter structure passed by
 *                                 application, according to that dark frame
 *                                 capture feature will be enabled or disabled.
 *  \return  success    DDC_PREV_SOK
 *           failure    DDC error
 */
static DDC_prevResult DDC_prevGetDFC(
    DDC_prevChannelHandle ddcChannelHandle,
    PSP_previewerDarkFrameCapture *prevGetDFC
    );

/**
 *  \brief DDC_prevValidateInputParams
 *
 *  It will validate input image parameters for previewer.
 *
 *  \param  ddcChannelHandle     [IN]  channel handle
 *  \param  inBufSize           [IN] input buffer size as specified by
 *                                    applicaion
 *  \return  success    DDC_PREV_SOK
 *           failure    DDC error
 */
static DDC_prevResult DDC_prevValidateInputParams(
    DDC_prevChannelHandle ddcChannelHandle,
    Int32 inBufSize
    );

/**
 *  \brief DDC_prevValidateOutputParams
 *
 *  It will validate output image parameters for previewer
 *
 *  \param  ddcChannelHandle     [IN]  channel handle
 *  \param  outBufSize           [IN] output buffer size as specified by
 *                                    applicaion
 *  \param  outBuf               [IN] output buffer address as specified by
 *                                    applicaion
 *  \return  success    DDC_PREV_SOK
 *           failure    DDC error
 */
static DDC_prevResult DDC_prevValidateOutputParams(
    DDC_prevChannelHandle ddcChannelHandle,
    Int32 outBufSize, Ptr outBuf
    );


/**
 *  \brief DDC_prevGetInfoForCCDC
 *
 *  It will get information related to CCDC
 *
 *  \param  ddcChannelHandle  [IN]  channel handle
 *  \param  prevInfo          [OUT] info struct required by CCDC
 *  \return  success    DDC_PREV_SOK
 *           failure    DDC error
 */
static DDC_prevResult DDC_prevGetInfoForCCDC(
    DDC_prevChannelHandle ddcChannelHandle,
    PSP_previewerGetInfoForCCDC *prevInfo
    );



/******************************************************************************
  Data-Structures
 ******************************************************************************/
/**
 * \brief previewer object of device structure at DDC layer
 */
DDC_prevDeviceObject ddcPrevDeviceObj = { DDC_PREV_UNINIT_ISR_NO, NULL, NULL };

/******************************************************************************
  Functions
 ******************************************************************************/



/**
 *  \brief DDC_prevDeviceCreate
 *  It creates a given previewer device at DDC layer.
 */
DDC_prevResult DDC_prevDeviceCreate(PSP_Handle *ddcDeviceHandle)
{
    DDC_prevResult devCreateReturnCode = DDC_PREV_NOT_SUPPORTED;
    PAL_Result devCreatePalReturnCode = PAL_OS_ERROR_NOT_SUPPORTED;

    /* parameter validation */
    if (NULL == ddcDeviceHandle)
    {
        devCreateReturnCode = DDC_PREV_INVALID_PARAM;
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
    }
    else
    {
        /* create a semaphore for ISR */
        devCreatePalReturnCode = PAL_osSemCreate("isrSem",
            DDC_PREV_INIT_ISR_SEM, NULL, &(ddcPrevDeviceObj.semIsr));
        if (PAL_SOK != devCreatePalReturnCode)
        {
            devCreateReturnCode = DDC_PREV_RESOURCE_ERROR;
            PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
        }
        /* assign device object to passed ddcDeviceHandle */
        *(ddcDeviceHandle) = (Ptr)(&ddcPrevDeviceObj);
        devCreateReturnCode = DDC_prevPerformRegOverlaying();
    }
    return devCreateReturnCode;
}



/**
 *  \brief DDC_prevOpenHandle
 *  It opens Channel of the previewer device at DDC layer.
 */
DDC_prevResult DDC_prevOpenHandle(
    PSP_previewerChannelCreateMode *chanParams,
    PSP_Handle *ddcChannelHandle,
    Uint32 *interruptMapRequired,
    Uint8 *interruptMapInfo,
    ddcISRType *isrFunc
    )
{
    DDC_prevResult openHandleReturnCode = DDC_PREV_NOT_SUPPORTED;
    PAL_Result openHandlePalReturnCode = PAL_OS_ERROR_NOT_SUPPORTED;
    Ptr openHandleChannelHandle = NULL;
    CSL_Status openHandleCSLReturnCode = CSL_ESYS_FAIL;

    /* parameter validation */
    if (((NULL == chanParams) || (NULL == ddcChannelHandle))
        || (((NULL == interruptMapRequired) || (NULL == interruptMapInfo))
            || (NULL == isrFunc)))
    {
        openHandleReturnCode = DDC_PREV_INVALID_PARAM;
        DDC_PREVIEWER_DEBUG_PRINT("Invalid Param\n");
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
    }
    else
    {
        /* if source is SDRAM */
        if ((PSP_PREVIEWER_CHANNEL_SDRAM == (chanParams->chanSource).source)
            || (PSP_PREVIEWER_CHANNEL_CCDC == (chanParams->chanSource).source))
        {
            openHandlePalReturnCode = PAL_osMemAlloc(chanParams->segId,
                sizeof(DDC_prevChannelObject), 0, &openHandleChannelHandle);
            if(NULL != openHandleChannelHandle)
            {
                openHandlePalReturnCode = PAL_osMemSet(openHandleChannelHandle,
                    0, sizeof(DDC_prevChannelObject));
            }
            /* if system is failed to allocate memory */
            if ((PAL_SOK != openHandlePalReturnCode)
                || (NULL == openHandleChannelHandle))
            {
                openHandleReturnCode = DDC_PREV_MEMORY_ERROR;
                DDC_PREVIEWER_DEBUG_PRINT("Memory Error\n");
                PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
            }
            else
            {
                /* create a semaphore for channel */
                openHandlePalReturnCode = PAL_osSemCreate("channelSem",
                    DDC_PREV_INIT_CHANNEL_SEM, NULL,
                    &(((DDC_prevChannelHandle)openHandleChannelHandle)
                        ->channelSem));
                if (PAL_SOK != openHandlePalReturnCode)
                {
                    openHandleReturnCode = DDC_PREV_RESOURCE_ERROR;
                    DDC_PREVIEWER_DEBUG_PRINT("Resource Error\n");
                    PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
                    /* free allocated memory as semaphore creation has been
                     * failed */
                    PAL_osMemFree(chanParams->segId, openHandleChannelHandle,
                        sizeof(DDC_prevChannelObject));
                }
                else
                {
                    ((DDC_prevChannelHandle)openHandleChannelHandle)
                        ->configState = DDC_PREVIEWER_CHANNEL_NOT_CONFIGURED;
                    (((DDC_prevChannelHandle)openHandleChannelHandle)
                     ->channelMode).chanSource.source
                        = (chanParams->chanSource).source;
                    (((DDC_prevChannelHandle)openHandleChannelHandle)
                     ->channelMode).segId
                        = chanParams->segId;
                    ((DDC_prevChannelHandle)openHandleChannelHandle)
                        ->semTimeOut = PAL_OSSEM_NO_TIMEOUT;

                    /* assign channel object to passed ddcChannelHandle */
                    *(ddcChannelHandle) = openHandleChannelHandle;
                    ddcPrevDeviceObj.intNum = CSL_INTC_EVENTID_PRVUINT;
                    if (PSP_PREVIEWER_CHANNEL_SDRAM
                        == (chanParams->chanSource).source)
                    {
                        *interruptMapRequired
                            = DDC_PREVIEWER_INTERRUPT_MAPPING_NEEDED;
                        *interruptMapInfo = ddcPrevDeviceObj.intNum;
                        *isrFunc = &DDC_prevISR;
                    }
                    else
                    {
                        *interruptMapRequired
                            = DDC_PREVIEWER_INTERRUPT_MAPPING_NOT_NEEDED;
                    }
                    openHandleCSLReturnCode
                        = LLC_prevResetPreviewer(ddcPrevDeviceObj.regs);
                    if (CSL_SOK != openHandleCSLReturnCode)
                    {
                        openHandleReturnCode = DDC_PREV_E_FAIL;
                    }
                    else
                    {
                        openHandleReturnCode = DDC_PREV_SOK;
                    }
                }
            }
        }
        else
        {
            openHandleReturnCode = DDC_PREV_NOT_SUPPORTED;
            DDC_PREVIEWER_DEBUG_PRINT("Not supported\n");
            PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
        }
    }
    return openHandleReturnCode;
}



/**
 *  \brief DDC_prevIoctl
 *  It controls Channel of the previewer device.
 */
DDC_prevResult DDC_prevIoctl(
    PSP_Handle ddcChannelHandle,
    PSP_previewerControlCmd cmd,
    Ptr cmdArg
    )
{
    DDC_prevResult ioctlReturnCode = DDC_PREV_NOT_SUPPORTED;
    DDC_prevChannelHandle ioctlChannelHandle = NULL;

    /* parameter validation */
    if ((((NULL == ddcChannelHandle) || (PSP_PREVIEWER_IOCTL_MIN >= cmd))
            ||((PSP_PREVIEWER_IOCTL_MAX) <= cmd)))
    {
        ioctlReturnCode = DDC_PREV_INVALID_PARAM;
        DDC_PREVIEWER_DEBUG_PRINT("Invalid Param\n");
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
    }
    else
    {
        ioctlChannelHandle = (DDC_prevChannelHandle)ddcChannelHandle;
        /* call corresponding function as per ioctl type */
        switch (cmd)
        {
            case PSP_PREVIEWER_IOCTL_SET_PARAMS:
                ioctlReturnCode = DDC_prevSetParameters(ioctlChannelHandle,
                    (PSP_previewerParams*)cmdArg);
                break;
            case PSP_PREVIEWER_IOCTL_GET_PARAMS:
                ioctlReturnCode = DDC_prevGetParameters(ioctlChannelHandle,
                    (PSP_previewerParams*)cmdArg);
                break;
            case PSP_PREVIEWER_IOCTL_GET_STATUS:
                ioctlReturnCode = DDC_prevGetStatus(ioctlChannelHandle,
                    (PSP_previewerStatus*)cmdArg);
                break;
            case PSP_PREVIEWER_IOCTL_GET_DARK_FRAME_STATUS:
                ioctlReturnCode = DDC_prevGetDarkFrameStatus(ioctlChannelHandle,
                    (PSP_previewerDarkFrameStatus*)cmdArg);
                break;
            case PSP_PREVIEWER_IOCTL_PREVIEW:
                ioctlReturnCode = DDC_prevPreview(ioctlChannelHandle,
                    (PSP_preview*)cmdArg);
                break;
            case PSP_PREVIEWER_IOCTL_GET_CROPSIZE:
                ioctlReturnCode = DDC_prevGetCropSize(ioctlChannelHandle,
                    (PSP_previewerCropSize*)cmdArg);
                break;
            case PSP_PREVIEWER_IOCTL_SET_EXP:
                ioctlReturnCode = DDC_prevSetReadReqExpand(ioctlChannelHandle,
                    (PSP_previewerReadReqExp*)cmdArg);
                break;
            case PSP_PREVIEWER_GET_INFO_FOR_CCDC:
                ioctlReturnCode = DDC_prevGetInfoForCCDC(ioctlChannelHandle,
                    (PSP_previewerGetInfoForCCDC*)cmdArg);
                break;
            case PSP_PREVIEWER_SET_DARK_FRAME_CAPTURE:
                ioctlReturnCode = DDC_prevSetDFC(ddcChannelHandle,
                    (PSP_previewerDarkFrameCapture*)cmdArg);
                break;
            case PSP_PREVIEWER_GET_DARK_FRAME_CAPTURE:
                ioctlReturnCode = DDC_prevGetDFC(ddcChannelHandle,
                    (PSP_previewerDarkFrameCapture*)cmdArg);
                break;
            case PSP_PREVIEWER_IOCTL_SET_SEM_TIMEOUT:
                {
                    if (NULL != cmdArg)
                    {
                        /* Change the semTimeOut value */
                        ioctlChannelHandle->semTimeOut = *(Int32 *)cmdArg;
                        ioctlReturnCode = DDC_PREV_SOK;
                    }
                    else
                    {
                        ioctlReturnCode = DDC_PREV_INVALID_PARAM;
                    }
                }
                break;
            default:
                ioctlReturnCode = DDC_PREV_NOT_SUPPORTED;
                break;
        }
    }

    return ioctlReturnCode;
}



/**
 *  \brief DDC_prevCloseHandle
 *  It closes Channel of the previewer device at DDC layer.
 */
DDC_prevResult DDC_prevCloseHandle(PSP_Handle ddcChannelHandle)
{
    DDC_prevResult closeHandleReturnCode = DDC_PREV_NOT_SUPPORTED;
    PAL_Result closeHandlePalReturnCode = PAL_OS_ERROR_NOT_SUPPORTED;

    /* parameter validation */
    if (NULL == ddcChannelHandle)
    {
        closeHandleReturnCode = DDC_PREV_INVALID_PARAM;
        DDC_PREVIEWER_DEBUG_PRINT("Invalid Param\n");
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
    }
    else
    {
        /* take channel semaphore so that other ioctl functions can not be
         * called */
        closeHandlePalReturnCode = PAL_osSemTake(((DDC_prevChannelHandle)
                ddcChannelHandle)->channelSem, ((DDC_prevChannelHandle)
                ddcChannelHandle)->semTimeOut);
        if (PAL_SOK != closeHandlePalReturnCode)
        {
            closeHandleReturnCode = DDC_PREV_INVALID_STATE;
            DDC_PREVIEWER_DEBUG_PRINT("Invalid State\n");
            PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
        }
        else
        {
            ddcPrevDeviceObj.intNum = DDC_PREV_UNINIT_ISR_NO;
            /* delete the channel semaphore */
            closeHandlePalReturnCode = PAL_osSemDelete(
                ((DDC_prevChannelHandle)ddcChannelHandle)->channelSem);
            if (PAL_SOK != closeHandlePalReturnCode)
            {
                PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
                closeHandleReturnCode = DDC_PREV_RESOURCE_ERROR;
                DDC_PREVIEWER_DEBUG_PRINT("Resource Error\n");
            }
            else
            {
                /* free allocated memory */
                closeHandlePalReturnCode = PAL_osMemFree(
                    (((DDC_prevChannelHandle)ddcChannelHandle)
                     ->channelMode).segId,
                    ddcChannelHandle, sizeof(DDC_prevChannelObject));
                if (PAL_SOK != closeHandlePalReturnCode)
                {
                    PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
                    closeHandleReturnCode = DDC_PREV_MEMORY_ERROR;
                    DDC_PREVIEWER_DEBUG_PRINT("Memory Error\n");
                }
                else
                {
                    closeHandleReturnCode = DDC_PREV_SOK;
                }
            }
        }
    }
    return closeHandleReturnCode;
}



/**
 *  \brief DDC_prevDeviceDelete
 *  It deletes a given previewer device at DDC layer.
 */
DDC_prevResult DDC_prevDeviceDelete(PSP_Handle ddcDeviceHandle)
{
    DDC_prevResult devDeleteReturnCode = DDC_PREV_NOT_SUPPORTED;
    PAL_Result devDeletePalReturnCode = PAL_OS_ERROR_NOT_SUPPORTED;

    /* parameter validation */
    if (NULL == ddcDeviceHandle)
    {
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
        devDeleteReturnCode = DDC_PREV_INVALID_PARAM;
        DDC_PREVIEWER_DEBUG_PRINT("Invalid Param\n");
    }
    else
    {
        /* delete semaphore of ISR */
        devDeletePalReturnCode = PAL_osSemDelete(ddcPrevDeviceObj.semIsr);
        if (PAL_SOK != devDeletePalReturnCode)
        {
            PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
            devDeleteReturnCode = DDC_PREV_RESOURCE_ERROR;
            DDC_PREVIEWER_DEBUG_PRINT("Resource Error\n");
        }
        else
        {
            devDeleteReturnCode = DDC_PREV_SOK;
        }
        ddcPrevDeviceObj.semIsr = NULL;
        ddcPrevDeviceObj.regs = NULL;
    }
    return devDeleteReturnCode;
}



/**
 *  \brief DDC_prevPerformRegOverlaying
 *  It is previewer register overlaying function.
 */
static DDC_prevResult DDC_prevPerformRegOverlaying()
{
    DDC_prevResult regOverlayReturnCode = DDC_PREV_NOT_SUPPORTED;
    ddcPrevDeviceObj.regs = (Ptr)CSL_VPSS_PREV_0_REGS;
    regOverlayReturnCode = DDC_PREV_SOK;
    return (regOverlayReturnCode);
}



/**
 *  \brief DDC_prevSetParameters
 *  It is used to set previewer parameters.
 */
static DDC_prevResult DDC_prevSetParameters(
    DDC_prevChannelHandle ddcChannelHandle,
    PSP_previewerParams *DDC_prevParams
    )
{
    DDC_prevResult setParamsReturnCode = DDC_PREV_NOT_SUPPORTED;
    PAL_Result setParamsPalReturnCode = PAL_OS_ERROR_NOT_SUPPORTED;
    CSL_Status setParamsCSLReturnCode = CSL_ESYS_FAIL;
    Uint8 setParamsPreviewerStatus = 0;

    /* parameter validation */
    if ((NULL == ddcChannelHandle) || (NULL == DDC_prevParams))
    {
        setParamsReturnCode = DDC_PREV_INVALID_PARAM;
        DDC_PREVIEWER_DEBUG_PRINT("Invalid Param\n");
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
    }
    else
    {
        /* check previewer is in enable state or in disable state
         * - if it is enabled, raise an error */
        setParamsCSLReturnCode = LLC_prevPreviewerStatus(
            (Uint8*)&setParamsPreviewerStatus, ddcPrevDeviceObj.regs);
        if (CSL_SOK != setParamsCSLReturnCode)
        {
            setParamsReturnCode = DDC_PREV_E_FAIL;
        }
        else
        {
            /* if previewer is already enabled */
            if (CSL_PREV_PCR_ENABLE_PREV_ENABLE == setParamsPreviewerStatus)
            {
                setParamsReturnCode = DDC_PREV_INVALID_STATE;
                DDC_PREVIEWER_DEBUG_PRINT("Invalid State\n");
                PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
            }
            else /* if channel status is free */
            {
                setParamsReturnCode = DDC_PREV_SOK;
                /* validate passed previewer parameters by calling
                 * DDC_prevValidateParameters */
                setParamsReturnCode = DDC_prevValidateParameters(
                    ddcChannelHandle, DDC_prevParams);
                /* if parameter strcture is valid... */
                if (DDC_PREV_SOK == setParamsReturnCode)
                {
                    /* take channel semaphore */
                    setParamsPalReturnCode = PAL_osSemTake(
                        ddcChannelHandle->channelSem,
                        ddcChannelHandle->semTimeOut);
                    if (PAL_SOK != setParamsPalReturnCode)
                    {
                        setParamsReturnCode = DDC_PREV_RESOURCE_ERROR;
                        DDC_PREVIEWER_DEBUG_PRINT("Resource Error\n");
                        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
                    }
                    else
                    {
                        /* copy these values into channel structure */
                        setParamsPalReturnCode = PAL_osMemCopy(
                            (Ptr)(&(ddcChannelHandle->DDC_prevParams)),
                            (Ptr)(DDC_prevParams), sizeof(PSP_previewerParams));
                        if (PAL_SOK != setParamsPalReturnCode)
                        {
                            setParamsReturnCode = DDC_PREV_MEMORY_ERROR;
                            DDC_PREVIEWER_DEBUG_PRINT("Memory Error\n");
                            PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
                        }
                        else
                        {
                            setParamsCSLReturnCode
                                = LLC_prevHardwareSetup(
                                    (PSP_previewerChannelCreateMode*)
                                    (&(ddcChannelHandle->channelMode)),
                                    DDC_prevParams,
                                    ddcPrevDeviceObj.regs);
                            if (CSL_SOK != setParamsCSLReturnCode)
                            {
                                setParamsReturnCode = DDC_PREV_E_FAIL;
                            }
                            else
                            {
                                /* fill dark frame capture structure in channel
                                 * object */
                                (ddcChannelHandle->DFCState).darkFrameState
                                    = PSP_PREVIEWER_DARK_FRAME_CAPTURE_DISABLE;
                                (ddcChannelHandle->DFCState).outPitch
                                    = (ddcChannelHandle
                                        ->DDC_prevParams).sizeParam.outPitch;

                                /* update the channel state to configured */
                                ddcChannelHandle->configState
                                    = DDC_PREVIEWER_CHANNEL_CONFIGURED;
                            }
                        }
                        /* give channel semaphore */
                        setParamsPalReturnCode
                            = PAL_osSemGive(ddcChannelHandle->channelSem);
                        if (PAL_SOK != setParamsPalReturnCode)
                        {
                            setParamsReturnCode = DDC_PREV_RESOURCE_ERROR;
                            DDC_PREVIEWER_DEBUG_PRINT("Resource Error\n");
                            PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
                        }
                    }
                }
            }
        }
    }
    return setParamsReturnCode;
}



/**
 *  \brief DDC_prevGetParameters
 *  It is used to get previewer parameters.
 */
static DDC_prevResult DDC_prevGetParameters(
    DDC_prevChannelHandle ddcChannelHandle,
    PSP_previewerParams *DDC_prevParams
    )
{
    DDC_prevResult getParamsReturnCode = DDC_PREV_NOT_SUPPORTED;
    PAL_Result getParamsPalReturnCode = PAL_OS_ERROR_NOT_SUPPORTED;

    /* parameter validation */
    if ((NULL == ddcChannelHandle) || (NULL == DDC_prevParams))
    {
        getParamsReturnCode = DDC_PREV_INVALID_PARAM;
        DDC_PREVIEWER_DEBUG_PRINT("Invalid Param\n");
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
    }
    else
    {
        /* raise an error if previewer parameters are not configured yet */
        if (DDC_PREVIEWER_CHANNEL_CONFIGURED
            != ddcChannelHandle->configState)
        {
            getParamsReturnCode = DDC_PREV_INVALID_STATE;
            DDC_PREVIEWER_DEBUG_PRINT("Invalid State\n");
            PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
        }
        else
        {
            /* copy these values into channel structure */
            getParamsPalReturnCode = PAL_osMemCopy(
                (Ptr)(DDC_prevParams),
                (Ptr)(&(ddcChannelHandle->DDC_prevParams)),
                sizeof(PSP_previewerParams));
            if (PAL_SOK != getParamsPalReturnCode)
            {
                getParamsReturnCode = DDC_PREV_MEMORY_ERROR;
                DDC_PREVIEWER_DEBUG_PRINT("Memory Error\n");
                PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
            }
            else
            {
                getParamsReturnCode = DDC_PREV_SOK;
            }
        }
    }
    return getParamsReturnCode;
}



/**
 *  \brief DDC_prevValidateParameters
 *  It is used to validate previewer parameters.
 */
static DDC_prevResult DDC_prevValidateParameters(
    DDC_prevChannelHandle ddcChannelHandle,
    PSP_previewerParams *DDC_prevParams
    )
{
    DDC_prevResult validateParamsReturnCode = DDC_PREV_NOT_SUPPORTED;
    Uint32 validateParamsErrorFlag = 0;
    Uint32 validateParamsBytesPerPixel = 0;
    Uint32 validateParamsTempVar = 0;
    Uint32 validateParamsXIndex = 0, validateParamsYIndex = 0;
    Uint32 validateParamsHSize = 0;

    /* parameter validation */
    if ((NULL == ddcChannelHandle) || (NULL == DDC_prevParams))
    {
        validateParamsReturnCode = DDC_PREV_INVALID_PARAM;
        DDC_PREVIEWER_DEBUG_PRINT("Invalid Param\n");
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
    }
    else
    {
        validateParamsErrorFlag = DDC_PREVIEWER_ERROR_NOT_FOUND;
        /* check whether down sampling rate is one of the supported */
        if ((((PSP_PREVIEWER_DOWN_SAMPLE_RATE1
                        != DDC_prevParams->downSampleRate)
                    && (PSP_PREVIEWER_DOWN_SAMPLE_RATE2
                        != DDC_prevParams->downSampleRate))
                && ((PSP_PREVIEWER_DOWN_SAMPLE_RATE4
                        != DDC_prevParams->downSampleRate)
                    && (PSP_PREVIEWER_DOWN_SAMPLE_RATE8
                        != DDC_prevParams->downSampleRate))))
        {
            validateParamsErrorFlag = DDC_PREVIEWER_ERROR_FOUND;
            DDC_PREVIEWER_DEBUG_PRINT("Invalid down sample rate\n");
        }

        /* check for valid values of pixel size */
        if ((DDC_PREVIEWER_ERROR_NOT_FOUND == validateParamsErrorFlag)
            && ((PSP_PREVIEWER_INWIDTH_8BIT
                    != (DDC_prevParams->sizeParam).pixelSize)
                && (PSP_PREVIEWER_INWIDTH_10BIT
                    != (DDC_prevParams->sizeParam).pixelSize)))
        {
            validateParamsErrorFlag = DDC_PREVIEWER_ERROR_FOUND;
            DDC_PREVIEWER_DEBUG_PRINT("Invalid pixel size\n");
        }

        /* if inverse A law feature is enable then pixel width should be 8 bit
         * only */
        if ((DDC_PREVIEWER_ERROR_NOT_FOUND == validateParamsErrorFlag)
            && (((Uint32)(0u) != (DDC_prevParams->features
                        & PSP_PREVIEWER_INVERSE_ALAW))
                && (PSP_PREVIEWER_INWIDTH_8BIT
                    != ((DDC_prevParams->sizeParam).pixelSize))))
        {
            validateParamsErrorFlag = DDC_PREVIEWER_ERROR_FOUND;
            DDC_PREVIEWER_DEBUG_PRINT("Invalid pixel size for inverse A law\n");
        }

        /* check horizontal info is proper or not - in case of CCDC both sph
         * and eph should be greater then equal to 2 */
        if ((DDC_PREVIEWER_ERROR_NOT_FOUND == validateParamsErrorFlag)
            && (PSP_PREVIEWER_CHANNEL_CCDC
                == (ddcChannelHandle->channelMode).chanSource.source))
        {
            if (((Uint16)(2u) > (Uint32)(DDC_prevParams->sizeParam).sph)
                || ((Uint16)(2u) > (Uint32)(DDC_prevParams->sizeParam).eph))
            {
                validateParamsErrorFlag = DDC_PREVIEWER_ERROR_FOUND;
                DDC_PREVIEWER_DEBUG_PRINT("Invalid start/end pixel horizontal");
                DDC_PREVIEWER_DEBUG_PRINT(" value for CCDC source\n");
            }
        }
        /* check horizontal size is proper or not - input width should be
         * in multiple of sampling rate */
        if ((DDC_PREVIEWER_ERROR_NOT_FOUND == validateParamsErrorFlag)
            && (((((Uint32)((DDC_prevParams->sizeParam).eph)
                            - (Uint32)((DDC_prevParams->sizeParam).sph))
                        + (Uint32)(1u))
                    % (Uint32)(DDC_prevParams->downSampleRate))
                != (Uint32)(0u)))
        {
            validateParamsErrorFlag = DDC_PREVIEWER_ERROR_FOUND;
            DDC_PREVIEWER_DEBUG_PRINT("Invalid horizontal size with");
            DDC_PREVIEWER_DEBUG_PRINT(" respect to down sample rate\n");
        }

        /* check that output width is even or not */
        validateParamsHSize = ((((Uint32)((DDC_prevParams->sizeParam).eph)
                    - (Uint32)((DDC_prevParams->sizeParam).sph)) + (Uint32)(1u))
            / (Uint32)(DDC_prevParams->downSampleRate));
        if ((DDC_PREVIEWER_ERROR_NOT_FOUND == validateParamsErrorFlag)
            && (0 != (validateParamsHSize % 2)))
        {
            validateParamsErrorFlag = DDC_PREVIEWER_ERROR_FOUND;
            DDC_PREVIEWER_DEBUG_PRINT("Invalid horizontal size with");
            DDC_PREVIEWER_DEBUG_PRINT(" respect to down sample rate\n");
        }

        /* check for valid values output pixel format */
        if ((DDC_PREVIEWER_ERROR_NOT_FOUND == validateParamsErrorFlag)
            && (((PSP_PREVIEWER_PIXELORDER_YCBYCR
                        != DDC_prevParams->outPixelOrderParam)
                    && (PSP_PREVIEWER_PIXELORDER_YCRYCB
                        != DDC_prevParams->outPixelOrderParam))
                && ((PSP_PREVIEWER_PIXELORDER_CBYCRY
                        != DDC_prevParams->outPixelOrderParam)
                    && (PSP_PREVIEWER_PIXELORDER_CRYCBY
                        != DDC_prevParams->outPixelOrderParam))))
        {
            validateParamsErrorFlag = DDC_PREVIEWER_ERROR_FOUND;
            DDC_PREVIEWER_DEBUG_PRINT("Invalid output format\n");
        }

        /* per pixel how many bytes are there in input image */
        if (PSP_PREVIEWER_INWIDTH_8BIT
            == (DDC_prevParams->sizeParam).pixelSize)
        {
            validateParamsBytesPerPixel = 1u;
        }
        else
        {
            validateParamsBytesPerPixel = 2u;
        }

        /* check inPitch value  - it should be 32 byte aligned,
         * it should also be greater then number of pixels in single row
         *  - in case of CCDC, input pitch will not be there. eph value will be
         *  at least one less then height of previewed image, so 1 is added */
        if ((DDC_PREVIEWER_ERROR_NOT_FOUND == validateParamsErrorFlag)
            && (PSP_PREVIEWER_CHANNEL_SDRAM
                == (ddcChannelHandle->channelMode).chanSource.source))
        {
            if (((Uint16)0x0u != (((DDC_prevParams->sizeParam).inPitch)
                        & (Uint16)(DDC_PREV_32_ALIGNED_MASK)))
                || ((validateParamsBytesPerPixel
                        * ((DDC_prevParams->sizeParam).eph + 1))
                    > (DDC_prevParams->sizeParam).inPitch))
            {
                validateParamsErrorFlag = DDC_PREVIEWER_ERROR_FOUND;
                DDC_PREVIEWER_DEBUG_PRINT("Invalid input pitch value\n");
            }
        }

        /* per pixel how many bytes are needed in output image
         * - 2 bytes per pixel */
        validateParamsBytesPerPixel = 2u;

        /* check outPitch value  - it should be 32 byte aligned,
         * it should also be greater then number of pixels in single row */
        if ((DDC_PREVIEWER_ERROR_NOT_FOUND == validateParamsErrorFlag)
            && (((Uint16)0x0u != (((DDC_prevParams->sizeParam).outPitch)
                        & (Uint16)(DDC_PREV_32_ALIGNED_MASK)))
                || ((validateParamsBytesPerPixel * validateParamsHSize)
                    > (DDC_prevParams->sizeParam).outPitch)))
        {
            validateParamsErrorFlag = DDC_PREVIEWER_ERROR_FOUND;
            DDC_PREVIEWER_DEBUG_PRINT("Invalid output pitch value\n");
        }

        /* check dark frame address and offset value
         *  - that should be 32 byte aligned and it will come in picture when
         *   dark frame subtract or lens shading is enabled */
        if ((DDC_PREVIEWER_ERROR_NOT_FOUND == validateParamsErrorFlag)
            && (((Uint32)(0u) != (DDC_prevParams->features
                        & PSP_PREVIEWER_DARK_FRAME_SUBTRACT))
                || ((Uint32)(0u) != (DDC_prevParams->features
                        & PSP_PREVIEWER_LENS_SHADING))))
        {
            if (((Uint16)0x0u != ((DDC_prevParams->darkFrameOffset)
                        & (Uint16)(DDC_PREV_32_ALIGNED_MASK)))
                || ((Uint32)0x0u != ((Uint32)(DDC_prevParams->darkFrameAddr)
                        & (Uint32)(DDC_PREV_32_ALIGNED_MASK))))
            {
                validateParamsErrorFlag = DDC_PREVIEWER_ERROR_FOUND;
                DDC_PREVIEWER_DEBUG_PRINT("Invalid dark frame");
                DDC_PREVIEWER_DEBUG_PRINT(" address/offset\n");
            }
        }

        /* white balance related */
        /* all white balance coefficients selection value should be between 0-3
         * */
        if (DDC_PREVIEWER_ERROR_NOT_FOUND == validateParamsErrorFlag)
        {
            validateParamsTempVar = 0;
            for (validateParamsXIndex = 0;
                validateParamsXIndex < PSP_PREVIEWER_WB_GAIN_MAX;
                validateParamsXIndex++)
            {
                for(validateParamsYIndex = 0;
                    validateParamsYIndex < PSP_PREVIEWER_WB_GAIN_MAX;
                    validateParamsYIndex++)
                {
                    validateParamsTempVar |= (DDC_prevParams->whiteBalanceParam)
                        .position[validateParamsXIndex][validateParamsYIndex];
                }
            }
            if ((Uint32)0 != (validateParamsTempVar
                    & ~(Uint32)(CSL_PREV_WBSEL_0_0_MASK)))
            {
                validateParamsErrorFlag = DDC_PREVIEWER_ERROR_FOUND;
                DDC_PREVIEWER_DEBUG_PRINT("Invalid white balance");
                DDC_PREVIEWER_DEBUG_PRINT(" coefficients\n");
            }
        }
        /* white balance digital gain should be of max 10 bits only */
        if ((DDC_PREVIEWER_ERROR_NOT_FOUND == validateParamsErrorFlag)
            && ((Uint32)0 != ((DDC_prevParams->whiteBalanceParam).wbDgain
                    & ~(Uint32)(CSL_PREV_WB_DGAIN_DGAIN_MASK))))
        {
            validateParamsErrorFlag = DDC_PREVIEWER_ERROR_FOUND;
            DDC_PREVIEWER_DEBUG_PRINT("Invalid white balance digital gain\n");
        }

        /* RGB blending related */
        /* blending values should be of max 12 bits only */
        if (DDC_PREVIEWER_ERROR_NOT_FOUND == validateParamsErrorFlag)
        {
            validateParamsTempVar = 0;
            for (validateParamsXIndex = 0;
                validateParamsXIndex < PSP_PREVIEWER_RGB_MAX;
                validateParamsXIndex++)
            {
                for(validateParamsYIndex = 0;
                    validateParamsYIndex < PSP_PREVIEWER_RGB_MAX;
                    validateParamsYIndex++)
                {
                    validateParamsTempVar |= (DDC_prevParams->rgbBlendingParam)
                        .blending[validateParamsXIndex][validateParamsYIndex];
                }
            }
            if ((Uint32)0 != (validateParamsTempVar
                    & ~(Uint32)(CSL_PREV_RGB_MAT1_MTX_RR_MASK)))
            {
                validateParamsErrorFlag = DDC_PREVIEWER_ERROR_FOUND;
                DDC_PREVIEWER_DEBUG_PRINT("Invalid RGB blending parameter\n");
            }
        }
        /* offset values should be of max 10 bits only */
        if (DDC_PREVIEWER_ERROR_NOT_FOUND == validateParamsErrorFlag)
        {
            validateParamsTempVar = 0;
            for (validateParamsXIndex = 0;
                validateParamsXIndex < PSP_PREVIEWER_RGB_MAX;
                validateParamsXIndex++)
            {
                validateParamsTempVar
                    |= (Uint32)((DDC_prevParams->rgbBlendingParam)
                        .offset[validateParamsXIndex]);
            }
            if ((Uint32)0 != (validateParamsTempVar
                    & ~(Uint32)(CSL_PREV_RGB_OFF1_MTX_OFFG_MASK)))
            {
                validateParamsErrorFlag = DDC_PREVIEWER_ERROR_FOUND;
                DDC_PREVIEWER_DEBUG_PRINT("Invalid RGB blending offset");
                DDC_PREVIEWER_DEBUG_PRINT(" value\n");
            }
        }

        /* RGB to ycbcrCoeffs coefficient related */
        /* coefficient values should be of max 10 bits */
        if (DDC_PREVIEWER_ERROR_NOT_FOUND == validateParamsErrorFlag)
        {
            validateParamsTempVar = 0;
            for (validateParamsXIndex = 0;
                validateParamsXIndex < PSP_PREVIEWER_RGB_MAX;
                validateParamsXIndex++)
            {
                for(validateParamsYIndex = 0;
                    validateParamsYIndex < PSP_PREVIEWER_RGB_MAX;
                    validateParamsYIndex++)
                {
                    validateParamsTempVar |= (DDC_prevParams->rgb2ycbcrParam)
                        .coeff[validateParamsXIndex][validateParamsYIndex];
                }
            }
            if ((Uint32)0 != (validateParamsTempVar
                    & ~(Uint32)(CSL_PREV_CSC0_CSCRY_MASK)))
            {
                validateParamsErrorFlag = DDC_PREVIEWER_ERROR_FOUND;
                DDC_PREVIEWER_DEBUG_PRINT("Invalid RGB to YCbCr");
                DDC_PREVIEWER_DEBUG_PRINT(" coefficients\n");
            }
        }


        if (DDC_PREVIEWER_ERROR_FOUND == validateParamsErrorFlag)
        {
            validateParamsReturnCode = DDC_PREV_INVALID_PARAM;
            DDC_PREVIEWER_DEBUG_PRINT("Invalid Param\n");
            PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
        }
        else
        {
            validateParamsReturnCode = DDC_PREV_SOK;
        }
    }
    return validateParamsReturnCode;
}



/**
 *  \brief DDC_prevGetStatus
 *  It is used to get previewer hardware status.
 */
static DDC_prevResult DDC_prevGetStatus(
    DDC_prevChannelHandle ddcChannelHandle,
    PSP_previewerStatus *prevStatus
    )
{
    DDC_prevResult getStatusReturnCode = DDC_PREV_NOT_SUPPORTED;
    CSL_Status getStatusCSLReturnCode = CSL_ESYS_FAIL;

    /* parameter validation */
    if ((NULL == ddcChannelHandle) || (NULL == prevStatus))
    {
        getStatusReturnCode = DDC_PREV_INVALID_PARAM;
        DDC_PREVIEWER_DEBUG_PRINT("Invalid Param\n");
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
    }
    else
    {
        /* raise an error if previewer parameters are not configured yet */
        if (DDC_PREVIEWER_CHANNEL_CONFIGURED
            != ddcChannelHandle->configState)
        {
            getStatusReturnCode = DDC_PREV_INVALID_STATE;
            DDC_PREVIEWER_DEBUG_PRINT("Invalid State\n");
            PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
        }
        else
        {
            /* get channel status by calling LLC_prevChannelStatus function */
            getStatusCSLReturnCode = LLC_prevChannelStatus(
                (Uint8*)(&(prevStatus->channelStatus)),
                ddcPrevDeviceObj.regs);
            if (CSL_SOK != getStatusCSLReturnCode)
            {
                getStatusReturnCode = DDC_PREV_E_FAIL;
            }
            else
            {
                getStatusReturnCode = DDC_PREV_SOK;
            }
        }
    }
    return getStatusReturnCode;
}


/**
 *  \brief DDC_prevGetDarkFrameStatus
 *  It is used to get dark frame subtract failure status.
 */
static DDC_prevResult DDC_prevGetDarkFrameStatus(
    DDC_prevChannelHandle ddcChannelHandle,
    PSP_previewerDarkFrameStatus *prevDarkFrameStatus
    )
{
    DDC_prevResult getDarkFrameStatusReturnCode = DDC_PREV_NOT_SUPPORTED;
    PAL_Result getDarkFrameStatusPalReturnCode = PAL_OS_ERROR_NOT_SUPPORTED;
    CSL_Status getDarkFrameStatusCSLReturnCode = CSL_ESYS_FAIL;

    /* parameter validation */
    if ((NULL == ddcChannelHandle) || (NULL == prevDarkFrameStatus))
    {
        getDarkFrameStatusReturnCode = DDC_PREV_INVALID_PARAM;
        DDC_PREVIEWER_DEBUG_PRINT("Invalid Param\n");
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
    }
    else
    {
        /* raise an error if previewer parameters are not configured yet or
         * source is SDRAM as in case of CCDC only dark frame subtract feature
         * can be enabled */
        if (DDC_PREVIEWER_CHANNEL_CONFIGURED
            != ddcChannelHandle->configState)
        {
            getDarkFrameStatusReturnCode = DDC_PREV_INVALID_STATE;
            DDC_PREVIEWER_DEBUG_PRINT("Invalid State\n");
            PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
        }
        else
        {
            /* take channel semaphore */
            getDarkFrameStatusPalReturnCode = PAL_osSemTake(
                ddcChannelHandle->channelSem,
                ddcChannelHandle->semTimeOut);
            if (PAL_SOK != getDarkFrameStatusPalReturnCode)
            {
                getDarkFrameStatusReturnCode = DDC_PREV_RESOURCE_ERROR;
                DDC_PREVIEWER_DEBUG_PRINT("Resource Error\n");
                PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
            }
            else
            {
                /* get channel status by calling LLC_prevChannelStatus function,
                 * this function will clear dark frame status also */
                getDarkFrameStatusCSLReturnCode = LLC_prevDarkFrameStatus(
                    (Uint8*)(&(prevDarkFrameStatus->status)),
                    ddcPrevDeviceObj.regs);
                if (CSL_SOK != getDarkFrameStatusCSLReturnCode)
                {
                    getDarkFrameStatusReturnCode = DDC_PREV_E_FAIL;
                    PAL_osSemGive(ddcChannelHandle->channelSem);
                }
                else
                {
                    /* give channel semaphore */
                    getDarkFrameStatusPalReturnCode
                        = PAL_osSemGive(ddcChannelHandle->channelSem);
                    if (PAL_SOK != getDarkFrameStatusPalReturnCode)
                    {
                        getDarkFrameStatusReturnCode = DDC_PREV_RESOURCE_ERROR;
                        DDC_PREVIEWER_DEBUG_PRINT("Resource Error\n");
                        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
                    }
                    else
                    {
                        getDarkFrameStatusReturnCode = DDC_PREV_SOK;
                    }
                }

            }
        }
    }
    return getDarkFrameStatusReturnCode;
}



/**
 *  \brief DDC_prevPreview
 *  It is used to trigger previewing operation.
 */
static DDC_prevResult DDC_prevPreview(
    DDC_prevChannelHandle ddcChannelHandle,
    PSP_preview *prevPreview
    )
{
    DDC_prevResult previewReturnCode = DDC_PREV_NOT_SUPPORTED;
    PAL_Result previewPalReturnCode = PAL_OS_ERROR_NOT_SUPPORTED;
    PSP_previewerStatus previewStatus = {PSP_PREVIEWER_CHANNEL_BUSY};
    Uint8 previewWBLO = 0;
    CSL_Status previewCSLReturnCode = CSL_ESYS_FAIL;

    /* parameter validation */
    if ((NULL == ddcChannelHandle) || (NULL == prevPreview))
    {
        previewReturnCode = DDC_PREV_INVALID_PARAM;
        DDC_PREVIEWER_DEBUG_PRINT("Invalid Param\n");
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
    }
    else
    {

#ifdef PREV_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT( &DVTEvent_Log, "PREVIEWER",
            PSP_DVT_DESC( PSP_DVT_eFUNC_START,
                          PSP_DVT_dCHANNEL_ID_INST,
                          PSP_DVT_dPACKET_ID,
                          PSP_DVT_dSIZE_BYTES),
            CHANNELID_INSTANCEID
                    (PSP_PREVIEWER_IOCTL_PREVIEW,
                     0),
            prevPreview,
            prevPreview->inBufSize * prevPreview->outBufSize
        );
#endif /* PREV_INSTRUMENTATION_ENABLED */

        /* raise an error if previewer parameters are not configured yet */
        if (DDC_PREVIEWER_CHANNEL_CONFIGURED
            != ddcChannelHandle->configState)
        {
            previewReturnCode = DDC_PREV_INVALID_STATE;
            DDC_PREVIEWER_DEBUG_PRINT("Invalid State\n");
            PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
        }
        else
        {
            previewReturnCode = DDC_prevValidateInputParams(ddcChannelHandle,
                prevPreview->inBufSize);
            if (DDC_PREV_SOK == previewReturnCode)
            {
                /* if input buffer address is not 32 byte aligned */
                if ((Uint32)(0x00u) != ((Uint32)(prevPreview->inBuf)
                        & (Uint32)(DDC_PREV_32_ALIGNED_MASK)))
                {
                    previewReturnCode = DDC_PREV_INVALID_PARAM;
                    DDC_PREVIEWER_DEBUG_PRINT("Invalid Param\n");
                    PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
                }
                else
                {
                    /* to validate output image parameters call
                     * DDC_prevValidateOutputParams */
                    previewReturnCode = DDC_prevValidateOutputParams(
                        ddcChannelHandle,
                        prevPreview->outBufSize,
                        prevPreview->outBuf);
                    if (DDC_PREV_SOK == previewReturnCode)
                    {
                        /* check channel status - if it is busy - raise an
                         * error */
                        previewReturnCode = DDC_prevGetStatus(ddcChannelHandle,
                            (PSP_previewerStatus*)(&previewStatus));
                        if (DDC_PREV_SOK == previewReturnCode)
                        {
                            /* if channel status is busy */
                            if (PSP_PREVIEWER_CHANNEL_BUSY
                                == previewStatus.channelStatus)
                            {
                                previewReturnCode = DDC_PREV_INVALID_STATE;
                                DDC_PREVIEWER_DEBUG_PRINT("Invalid State\n");
                                PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
                            }
                            else /* if channel status is free */
                            {
                                /* take channel semaphore */
                                previewPalReturnCode = PAL_osSemTake(
                                    ddcChannelHandle->channelSem,
                                    ddcChannelHandle->semTimeOut);
                                if (PAL_SOK != previewPalReturnCode)
                                {
                                    previewReturnCode = DDC_PREV_RESOURCE_ERROR;
                                    DDC_PREVIEWER_DEBUG_PRINT("Resource");
                                    DDC_PREVIEWER_DEBUG_PRINT(" Error\n");
                                    PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
                                }
                                else
                                {

#ifdef PREV_INSTRUMENTATION_ENABLED
                                    PSP_LOG_EVENT( &DVTEvent_Log, "PREVIEWER",
                                        PSP_DVT_DESC( PSP_DVT_eDATA_SND_START,
                                                      PSP_DVT_dCHANNEL_ID_INST,
                                                      PSP_DVT_dPACKET_ID,
                                                      PSP_DVT_dNONE),
                                        CHANNELID_INSTANCEID
                                                (PSP_PREVIEWER_IOCTL_PREVIEW,
                                                 0),
                                        prevPreview
                                    );
#endif /* PREV_INSTRUMENTATION_ENABLED */

                                    /* trigger previewer */
                                    previewCSLReturnCode
                                        = LLC_prevOneShotPreviewer(
                                            (PSP_previewerChannelCreateMode*)
                                            (&(ddcChannelHandle->channelMode)),
                                            prevPreview,
                                            ddcPrevDeviceObj.regs);
                                    if (CSL_SOK != previewCSLReturnCode)
                                    {
                                        previewReturnCode = DDC_PREV_E_FAIL;
                                        PREVIEWER_DEBUG_ERR_TRACE(
                                            __LINE__);
                                        PAL_osSemGive(
                                            ddcChannelHandle->channelSem);
                                    }
                                    else
                                    {
                                        /* take ISR semaphore */
                                        previewPalReturnCode = PAL_osSemTake(
                                            ddcPrevDeviceObj.semIsr,
                                            ddcChannelHandle->semTimeOut);
                                        /* if it fails to take ISR semaphore */
                                        if (PAL_SOK != previewPalReturnCode)
                                        {
                                            previewReturnCode
                                                = DDC_PREV_RESOURCE_ERROR;
                                            PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
                                            /* release channel semaphore, but
                                             * error code should be return as
                                             * semaphore for ISR has not been
                                             * retrieved. */
                                            PAL_osSemGive(
                                                ddcChannelHandle->channelSem);
                                        }
                                        else /* ISR semaphore taken
                                                successfully */
                                        {
                                            /* get Write buffer memory overflow
                                             * (Preview engine) */
                                            LLC_prevGetWriteBufMemOverflow
                                                ((Uint8*)(&previewWBLO),
                                                 (Ptr)CSL_VPSS_0_REGS
                                                );
                                            if (1u == previewWBLO)
                                            {
                                                previewReturnCode
                                                    = DDC_PREV_OVERFLOW_ERROR;
                                                PREVIEWER_DEBUG_ERR_TRACE(
                                                    __LINE__);
                                            }
                                            /* Give channel semaphore */
                                            previewPalReturnCode = PAL_osSemGive
                                                (ddcChannelHandle->channelSem);
                                            if (PAL_SOK != previewPalReturnCode)
                                            {
                                                previewReturnCode
                                                    = DDC_PREV_RESOURCE_ERROR;
                                                PREVIEWER_DEBUG_ERR_TRACE(
                                                    __LINE__);
                                            }
                                        }
                                    }
#ifdef PREV_INSTRUMENTATION_ENABLED
                                    PSP_LOG_EVENT( &DVTEvent_Log, "PREVIEWER",
                                        PSP_DVT_DESC( PSP_DVT_eDATA_SND_END,
                                                      PSP_DVT_dCHANNEL_ID_INST,
                                                      PSP_DVT_dPACKET_ID,
                                                      PSP_DVT_dNONE),
                                        CHANNELID_INSTANCEID
                                                (PSP_PREVIEWER_IOCTL_PREVIEW,
                                                 0),
                                        prevPreview
                                    );
#endif /* PREV_INSTRUMENTATION_ENABLED */
                                } /* if PAL_SOK neq previewPalReturnCode for
                                     PAL_osSemTake for channel semaphore */
                            } /* if prev status eq PSP_PREVIEWER_... */
                        } /* if previewer status is read successfully */
                    } /* if output parameters are validated successfully */
                } /* if input address is validated successfully */
            } /* if input size is validated successfully */
        }
    }
    return previewReturnCode;
}



/**
 *  \brief DDC_prevGetCropSize
 *  It is used to calculates crop size.
 */
static DDC_prevResult DDC_prevGetCropSize(
    DDC_prevChannelHandle ddcChannelHandle,
    PSP_previewerCropSize* prevCropSize
    )
{
    DDC_prevResult getCropSizeReturnCode = DDC_PREV_NOT_SUPPORTED;

    /* parameter validation */
    if ((NULL == ddcChannelHandle) || (NULL == prevCropSize))
    {
        getCropSizeReturnCode = DDC_PREV_INVALID_PARAM;
        DDC_PREVIEWER_DEBUG_PRINT("Invalid Param\n");
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
    }
    else
    {
        /* raise an error if previewer parameters are not configured yet */
        if (DDC_PREVIEWER_CHANNEL_CONFIGURED
            != ddcChannelHandle->configState)
        {
            getCropSizeReturnCode = DDC_PREV_INVALID_STATE;
            DDC_PREVIEWER_DEBUG_PRINT("Invalid State\n");
            PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
        }
        else
        {
            prevCropSize->hCrop = 0;
            prevCropSize->vCrop = 0;

            /* if Horizontal Median Filter is enabled, horizontal crop will be
             * 4 pixels */
            if ((Uint32)(0x0u)
                != ((ddcChannelHandle->DDC_prevParams).features
                    & PSP_PREVIEWER_HMF))
            {
                prevCropSize->hCrop += DDC_PREV_HMF_HCROP;
            }
            /* if noise filter is enabled, horizontal crop will be 4 pixels and
             * vertical crop will be 4 lines */
            if ((Uint32)(0x0u)
                != ((ddcChannelHandle->DDC_prevParams).features
                    & PSP_PREVIEWER_NOISE_FILTER))
            {
                prevCropSize->hCrop += DDC_PREV_NOISE_FILTER_HCROP;
                prevCropSize->vCrop += DDC_PREV_NOISE_FILTER_VCROP;
            }
            /* if CFA is enabled, horizontal crop will be 4 pixels and
             * vertical crop will be 4 lines */
            if ((Uint32)(0x0u)
                != ((ddcChannelHandle->DDC_prevParams).features
                    & PSP_PREVIEWER_CFA))
            {
                prevCropSize->hCrop += DDC_PREV_CFA_HCROP;
                prevCropSize->vCrop += DDC_PREV_CFA_VCROP;
            }
            /* if luminance enahancement is enabled, horizontal crop will be
             * 2 pixels */
            if ((Uint32)(0x0u)
                != ((ddcChannelHandle->DDC_prevParams).features
                    & PSP_PREVIEWER_LUMA_ENHANCE))
            {
                prevCropSize->hCrop += DDC_PREV_LUMA_HCROP;
            }
            getCropSizeReturnCode = DDC_PREV_SOK;
        }
    }
    return getCropSizeReturnCode;
}

/**
 *  \brief DDC_prevSetReadReqExpand
 *  It is used to set read request expand
 */
static DDC_prevResult DDC_prevSetReadReqExpand(
    DDC_prevChannelHandle ddcChannelHandle,
    PSP_previewerReadReqExp *prevReadReqExp
    )
{
    DDC_prevResult setReadReqExpReturnCode = DDC_PREV_NOT_SUPPORTED;
    CSL_Status setReadReqExpCSLReturnCode = CSL_ESYS_FAIL;

    /* parameter validation */
    if ((NULL == ddcChannelHandle) || (NULL == prevReadReqExp))
    {
        setReadReqExpReturnCode = DDC_PREV_INVALID_PARAM;
        DDC_PREVIEWER_DEBUG_PRINT("Invalid Param\n");
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
    }
    else
    {
        setReadReqExpCSLReturnCode = LLC_prevSetReadReqExpand(
            (Uint16)(prevReadReqExp->prevExp),
            (Ptr)CSL_VPSS_0_REGS);
        if (CSL_SOK != setReadReqExpCSLReturnCode)
        {
            setReadReqExpReturnCode = DDC_PREV_E_FAIL;
        }
        else
        {
            setReadReqExpReturnCode = DDC_PREV_SOK;
        }
    }
    return setReadReqExpReturnCode;
}


/**
 *  \brief DDC_prevValidateInputParams
 *  It is used to validate input image parameters for previewer.
 */
static DDC_prevResult DDC_prevValidateInputParams(
    DDC_prevChannelHandle ddcChannelHandle,
    Int32 inBufSize
    )
{
    DDC_prevResult validateInputReturnCode = DDC_PREV_NOT_SUPPORTED;
    Uint32 validateInputHSize = 0;
    Uint32 validateInputVSize = 0;
    Uint32 validateInputSize = 0;

    /* parameter validation, if outbuffer address is not 32 byte aligned raise
     * an error */
    if ((NULL == ddcChannelHandle)
        || (0 == inBufSize))
    {
        validateInputReturnCode = DDC_PREV_INVALID_PARAM;
        DDC_PREVIEWER_DEBUG_PRINT("Invalid Param\n");
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
    }
    else
    {
        validateInputHSize
            = ((Uint32)((ddcChannelHandle->DDC_prevParams).sizeParam.eph)
                - (Uint32)((ddcChannelHandle->DDC_prevParams).sizeParam.sph))
            + 1u;

        validateInputVSize
            = ((Uint32)((ddcChannelHandle->DDC_prevParams).sizeParam.elv)
                - (Uint32)((ddcChannelHandle->DDC_prevParams).sizeParam.slv))
            + 1u;

        validateInputSize = (ddcChannelHandle->DDC_prevParams).sizeParam.inPitch
            * validateInputVSize;
        /* if input image size is not sufficient or input buffer address is
         * not 32 byte aligned or horizontal line size is not in multiple
         * of average sample down, raise an error */
        if ((validateInputSize > inBufSize)
            || ((validateInputHSize
                    % ((ddcChannelHandle->DDC_prevParams).downSampleRate))
                != 0))
        {
            validateInputReturnCode = DDC_PREV_INVALID_PARAM;
            DDC_PREVIEWER_DEBUG_PRINT("Invalid input size\n");
            PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
        }
        else
        {
            validateInputReturnCode = DDC_PREV_SOK;
        }
    }
    return validateInputReturnCode;
}



/**
 *  \brief DDC_prevValidateOutputParams
 *  It is used to validate output image parameters for previewer.
 */
static DDC_prevResult DDC_prevValidateOutputParams(
    DDC_prevChannelHandle ddcChannelHandle,
    Int32 outBufSize,
    Ptr outBuf
    )
{
    DDC_prevResult validateOutputReturnCode = DDC_PREV_NOT_SUPPORTED;
    PSP_previewerCropSize validateOutputCropSizeObj = {-1, -1};
    Uint32 validateOutputHSize = 0;
    Uint32 validateOutputVSize = 0;
    Uint32 validateOutputSize = 0;

    /* parameter validation, if outbuffer address is not 32 byte aligned raise
     * an error */
    if (((NULL == ddcChannelHandle) || (0 == outBufSize))
        || ((NULL == outBuf) || (0x00 != ((Uint32)(outBuf)
                    & (Uint32)(DDC_PREV_32_ALIGNED_MASK)))))
    {
        validateOutputReturnCode = DDC_PREV_INVALID_PARAM;
        DDC_PREVIEWER_DEBUG_PRINT("Invalid Param\n");
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
    }
    else
    {
        /* calculate crop size */
        validateOutputReturnCode = DDC_prevGetCropSize(ddcChannelHandle,
            (PSP_previewerCropSize*)&validateOutputCropSizeObj);
        if (DDC_PREV_SOK == validateOutputReturnCode)
        {
            validateOutputHSize = ((Uint32)((ddcChannelHandle
                        ->DDC_prevParams).sizeParam.eph)
                - (Uint32)((ddcChannelHandle
                        ->DDC_prevParams).sizeParam.sph)) + 1u;
            validateOutputVSize = ((Uint32)((ddcChannelHandle
                        ->DDC_prevParams).sizeParam.elv)
                - (Uint32)((ddcChannelHandle
                        ->DDC_prevParams).sizeParam.slv)) + 1u;

            /* output size will be decrease by averager downsampling rate */
            validateOutputHSize
                /= (Uint32)(ddcChannelHandle->DDC_prevParams).downSampleRate;
            if (PSP_PREVIEWER_DARK_FRAME_CAPTURE_ENABLE
                != (ddcChannelHandle->DFCState).darkFrameState)
            {
                validateOutputCropSizeObj.hCrop = 0;
                validateOutputCropSizeObj.vCrop = 0;
            }

            /* subtract crop size values from output size */
            validateOutputHSize = validateOutputHSize
                - (Uint32)validateOutputCropSizeObj.hCrop;
            validateOutputVSize = validateOutputVSize
                - (Uint32)validateOutputCropSizeObj.vCrop;

            validateOutputSize
                = (ddcChannelHandle->DDC_prevParams).sizeParam.outPitch
                * validateOutputVSize;
            /* if given out buffer size is less then required size or
             * horizontal or vertical size exceeds maximum size,
             * raise an error */
            if (((validateOutputSize > outBufSize)
                    || (validateOutputHSize > PSP_PREVIEWER_MAX_IMAGE_WIDTH))
                || (validateOutputVSize > PSP_PREVIEWER_MAX_IMAGE_HEIGHT))
            {
                validateOutputReturnCode = DDC_PREV_INVALID_PARAM;
                DDC_PREVIEWER_DEBUG_PRINT("Invalid output size\n");
                PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
            }
        }
    }
    return validateOutputReturnCode;
}



/**
 *  \brief DDC_prevISR
 *  IT is the previewer interrupt service routine will be used when source is
 *   SDRAM
 */
/* ARGSUSED */
void DDC_prevISR(Uint32 arg)
{
#ifdef PREV_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT( &DVTEvent_Log, "PREVIEWER",
        PSP_DVT_DESC( PSP_DVT_eINT_START,
            PSP_DVT_dINST,
            PSP_DVT_dCHANNEL_ID,
            PSP_DVT_dNONE),
        0, PSP_PREVIEWER_IOCTL_PREVIEW);
#endif /* PREV_INSTRUMENTATION_ENABLED */

    /* post an ISR semaphore */
    if (PAL_SOK != PAL_osSemGive(ddcPrevDeviceObj.semIsr))
    {
        DDC_PREVIEWER_DEBUG_PRINT("DDC_prevISR: Error in semgive\n");
    }

#ifdef PREV_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT( &DVTEvent_Log, "PREVIEWER",
        PSP_DVT_DESC( PSP_DVT_eINT_END,
            PSP_DVT_dINST,
            PSP_DVT_dCHANNEL_ID,
            PSP_DVT_dNONE),
        0, PSP_PREVIEWER_IOCTL_PREVIEW);
#endif /* PREV_INSTRUMENTATION_ENABLED */
}



/**
 *  \brief DDC_prevGetInfoForCCDC
 *  It is used to pass info required by CCDC.
 */
static DDC_prevResult DDC_prevGetInfoForCCDC(
    DDC_prevChannelHandle ddcChannelHandle,
    PSP_previewerGetInfoForCCDC *prevInfo
    )
{
    DDC_prevResult getInfoReturnCode = DDC_PREV_NOT_SUPPORTED;

    /* parameter validation */
    if ((NULL == ddcChannelHandle) || (NULL == prevInfo))
    {
        getInfoReturnCode = DDC_PREV_INVALID_PARAM;
        DDC_PREVIEWER_DEBUG_PRINT("Invalid Param\n");
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
    }
    else
    {
        prevInfo->source
            = (ddcChannelHandle->channelMode).chanSource.source;
        prevInfo->intNo = ddcPrevDeviceObj.intNum;

        getInfoReturnCode = DDC_PREV_SOK;
    }
    return getInfoReturnCode;
}



/**
 *  \brief DDC_prevSetDFC
 *  It will set dark frame capture parameter.
 */
static DDC_prevResult DDC_prevSetDFC(
    DDC_prevChannelHandle ddcChannelHandle,
    PSP_previewerDarkFrameCapture *prevSetDFC
    )
{
    DDC_prevResult setDFCReturnCode = DDC_PREV_NOT_SUPPORTED;
    CSL_Status setDFCCSLReturnCode = CSL_ESYS_FAIL;

    /* parameter validation */
    if ((NULL == ddcChannelHandle) || (NULL == prevSetDFC))
    {
        setDFCReturnCode = DDC_PREV_INVALID_PARAM;
        DDC_PREVIEWER_DEBUG_PRINT("Invalid Param\n");
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
    }
    else
    {
        /* raise an error if previewer parameters are not configured yet */
        if (DDC_PREVIEWER_CHANNEL_CONFIGURED
            != ddcChannelHandle->configState)
        {
            setDFCReturnCode = DDC_PREV_INVALID_STATE;
            DDC_PREVIEWER_DEBUG_PRINT("Invalid State\n");
            PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
        }
        else
        {
            /* if dark frame capture needs to be enabled, set new pitch value as
             * passed in PSP_previewerDarkFrameCapture structure */
            if (PSP_PREVIEWER_DARK_FRAME_CAPTURE_ENABLE
                == (prevSetDFC->darkFrameState))
            {
                /* check outPitch value  - it should be 32 byte aligned,
                 * it should also be greater then number of pixels in
                 * single row */
                if (((Uint16)0x0u != ((prevSetDFC->outPitch)
                            & (Uint16)(DDC_PREV_32_ALIGNED_MASK)))
                    || ((((ddcChannelHandle->DDC_prevParams).sizeParam).eph + 1)
                        > prevSetDFC->outPitch))
                {
                    setDFCReturnCode = DDC_PREV_INVALID_PARAM;
                    DDC_PREVIEWER_DEBUG_PRINT("Invalid output pitch value\n");
                }
                else
                {
                    /* enable dark frame capture feature */
                    (ddcChannelHandle->DFCState).darkFrameState
                        = PSP_PREVIEWER_DARK_FRAME_CAPTURE_ENABLE;
                    /* change the pitch value */
                    (ddcChannelHandle->DFCState).outPitch
                        = prevSetDFC->outPitch;
                    setDFCCSLReturnCode = LLC_prevSetDFC(
                        (Uint8)((ddcChannelHandle->DFCState).darkFrameState),
                        (ddcChannelHandle->DFCState).outPitch,
                        ddcPrevDeviceObj.regs);
                    if (CSL_SOK != setDFCCSLReturnCode)
                    {
                        setDFCReturnCode = DDC_PREV_E_FAIL;
                    }
                    else
                    {
                        setDFCReturnCode = DDC_PREV_SOK;
                    }
                }
            }
            /* if dark frame capture needs to be disabled, restore pitch
             *  value as passed in PSP_previewerParams structure */
            else if (PSP_PREVIEWER_DARK_FRAME_CAPTURE_DISABLE
                == (prevSetDFC->darkFrameState))
            {
                /* disable dark frame capture feature */
                (ddcChannelHandle->DFCState).darkFrameState
                    = PSP_PREVIEWER_DARK_FRAME_CAPTURE_DISABLE;
                /* pitch value will be same as passed in PSP_previewerParams
                 * structure */
                (ddcChannelHandle->DFCState).outPitch
                    = (ddcChannelHandle->DDC_prevParams).sizeParam.outPitch;
                setDFCCSLReturnCode = LLC_prevSetDFC(
                    (Uint8)((ddcChannelHandle->DFCState).darkFrameState),
                    (ddcChannelHandle->DFCState).outPitch,
                    ddcPrevDeviceObj.regs);
                if (CSL_SOK != setDFCCSLReturnCode)
                {
                    setDFCReturnCode = DDC_PREV_E_FAIL;
                }
                else
                {
                    setDFCReturnCode = DDC_PREV_SOK;
                }
            }
            else
            {
                setDFCReturnCode = DDC_PREV_INVALID_PARAM;
                DDC_PREVIEWER_DEBUG_PRINT("Invalid Param\n");
                PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
            }
        }
    }
    return setDFCReturnCode;
}



/**
 *  \brief DDC_prevGetDFC
 *  It will get dark frame capture parameter.
 */
static DDC_prevResult DDC_prevGetDFC(
    DDC_prevChannelHandle ddcChannelHandle,
    PSP_previewerDarkFrameCapture *prevGetDFC
    )
{
    DDC_prevResult getDFCReturnCode = DDC_PREV_NOT_SUPPORTED;
    PAL_Result getDFCPalReturnCode = PAL_OS_ERROR_NOT_SUPPORTED;

    /* parameter validation */
    if ((NULL == ddcChannelHandle) || (NULL == prevGetDFC))
    {
        getDFCReturnCode = DDC_PREV_INVALID_PARAM;
        DDC_PREVIEWER_DEBUG_PRINT("Invalid Param\n");
        PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
    }
    else
    {
        /* raise an error if previewer parameters are not configured yet */
        if (DDC_PREVIEWER_CHANNEL_CONFIGURED
            != ddcChannelHandle->configState)
        {
            getDFCReturnCode = DDC_PREV_INVALID_STATE;
            DDC_PREVIEWER_DEBUG_PRINT("Invalid State\n");
            PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
        }
        else
        {
            getDFCPalReturnCode = PAL_osMemCopy((Ptr)(prevGetDFC),
                (Ptr)(&(ddcChannelHandle->DFCState)),
                sizeof(PSP_previewerDarkFrameCapture));
            if (PAL_SOK != getDFCPalReturnCode)
            {
                getDFCReturnCode = DDC_PREV_MEMORY_ERROR;
                DDC_PREVIEWER_DEBUG_PRINT("Memory Error\n");
                PREVIEWER_DEBUG_ERR_TRACE(__LINE__);
            }
            else
            {
                getDFCReturnCode = DDC_PREV_SOK;
            }
        }
    }
    return getDFCReturnCode;
}

