/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file   psp_resizer.h
 *
 *  \brief  resizer interface definition
 *
 *  This file contains the interfaces, data types and symbolic definitions
 *  that are needed by the application to utilize the serivces of the Resizer
 *  device driver.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *  \author     EI3
 *
 *  \version    0.1     EI3                Created
 *
 *  <b>Compiler Switch Used</b>
 *
 *  <b>_DEBUG</b><br>
 *   This is used as a flag to compiler whether to include the debug statements
 *   inserted in the code into the final image. This flag helps to build DEBUG
 *   image of the program. For RELEASE images this is not passed to the compiler.
 *
 *  <b>CHIP_XXXX</b><br>
 *   The CSL layer is written in a common file for all the variants of a SOC.
 *   This flag differentiates the variant we are compiling for, for e.g. CHIP_DM648,
 *   and the CSL definitions for that variant appropriately gets defined for register
 *   base addresses, num of ports of a peripheral etc.
 *
 *  <b>RESIZER_INSTRUMENTATION_ENABLED</b><br>
 *   This flag is passed to the compiler to include the instrumentation code parts
 *   into the final image/lib of the program. This helps build the iRelease/iDebug
 *   versions of the image/lib with a common code base
 *
 */

#ifndef _PSP_RESIZER_H_
#define _PSP_RESIZER_H_
/******************************************************************************
                            Header File Inclusion
******************************************************************************/

/* PSP Common header provides standard driver return codes, operation modes
 * and other generic defines required by all PSP drivers
 */
#include <ti/sdo/pspdrivers/common/psp_common.h>
#include <gio.h>
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/tistdtypes.h>

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
                                  Macros
******************************************************************************/
#define PSP_RESIZER_NUM_CHANNELS       (16u)
/**< Number of Resizer Channels */
#define PSP_RSZ_NUM_INSTANCES            (0x1u)
/**< Number of Resizer Instances */
#define PSP_RSZ_INTYPE_YCBCR422_16BIT      (0x0)

/**< specifies input is 16 bit */
#define PSP_RSZ_INTYPE_PLANAR_8BIT         (0x1)
#define PSP_RSZ_CBILIN_DISABLE             (0x0)
/**< same as luminance processing */
#define PSP_RSZ_CBILIN_ENABLE              (0x1)
/**< enable bi-linear processing        */
#define PSP_RSZ_PIX_FMT_UYVY               (0x0)
/**< cb:y:cr:y */
#define PSP_RSZ_PIX_FMT_YUYV               (0x1)
/**< y:cb:y:cr */
#define PSP_RSZ_PIX_FMT_PLANAR             (0x2)
/**< 8-bit planar input */
#define PSP_RSZ_YENH_DISABLE               (0x0)
/**< specifies luma is enabled or disabled */
#define PSP_RSZ_YENH_3TAP_HPF              (0x1)
/**< specifies luma  algorithm */
#define PSP_RSZ_YENH_5TAP_HPF              (0x2)
/**< specifies luma algorithm */
#define PSP_RSZ_SRC_CCDC_PREVIEWER         (0x0)
/**< specifies input is from CCDC or previewer */
#define PSP_RSZ_SRC_RAM                    (0x1)
/**< specifies input is from Ram */



/**
 * \defgroup PSPResizerMain PSP Resizer Interface Definitions
 *
 * All interface definitions for PSP Resizer are contained in this file.
 *
 * @{
 */

 /******************************************************************************
                               Enumerations
******************************************************************************/
/**
 *  \brief  Enumeration PSP_RSZIoctlCmd
 *   PSP Resizer Ioctl commands
 *
 *  Resizer Ioctl commands
 */
typedef enum
{
    PSP_RSZ_IOCTL_SET_PARAMS = (128),
    /**< Set Resizer config params,
         cmdArg = pointer to struct _PSP_RSZyenhParams                        */
    PSP_RSZ_IOCTL_GET_PARAMS,
    /**< Get Resizer config params,
         cmdArg= pointer to struct _PSP_RSZyenhParams                         */
    PSP_RSZ_IOCTL_RESIZE,
    /**< Start resizing of image,
         cmdArg= pointer to struct _PSP_RSZresize                             */
    PSP_RSZ_IOCTL_SET_PRIORITY,
    /**< Set priority of channel,
         cmdArg = pointer to struct _PSP_RSZpriority                          */
    PSP_RSZ_IOCTL_GET_PRIORITY,
    /**< Get priority of channel,
         cmdArg = pointer to struct _PSP_RSZpriority                          */
    PSP_RSZ_IOCTL_GET_STATUS,
    /**< Get status of channel and hardware,
         cmdArg = pointer to struct _PSP_RSZstatus                            */
    PSP_RSZ_IOCTL_GET_CROPSIZE,
    /**< Get cropsize , cmdArg = pointer to struct _PSP_RSZcropsize           */
    PSP_RSZ_IOCTL_SET_EXP,
    /**< Set Expand , cmdArg = pointer to Expand value<Int>                   */
    PSP_RSZ_IOCTL_SET_SEM_TIMEOUT,
    /**< Change the timeout value used in semaphore pending operation.
         cmdArg:  pointer to Timeout values<Int32>                            */
    PSP_RSZ_IOCTL_SET_ISRSEM_TIMEOUT,
    /**< Change the timeout value used in interrupt semaphore pending operation.
         cmdArg: pointer to Timeout values<Int>                               */
    PSP_RSZ_IOCTL_SET_RESIZE_CALLBACK,
    /**< Register an application callback with the driver, which will be called 
         upon completion of resizing of the given image.
         cmdArg: pointer to callback structure _PSP_RSZcallback               */
    PSP_RSZ_IOCTL_RESIZE_ASYNC
    /**< Start the resize operation, without putting the application to sleep.
         This IOCTL calls the application callback - registered via the IOCTL 
         PSP_RSZ_IOCTL_SET_RESIZE_CALLBACK. The application should take care of 
         waiting for this notification via callback. This IOCTL can be called 
         again, only after the previous call to this IOCTL has completed 
         (application callback has been called).
         cmdArg: pointer to resize structure _PSP_RSZresize                   */   
} PSP_RSZIoctlCmd;
/*-----------------------------------------------------------------------------
                          IOM Function Table
            Driver function table to be used by applications.
-----------------------------------------------------------------------------*/
    /*extern IOM_Fxns RSZMD_FXNS;*/
    extern IOM_Fxns RSZMD_FXNS;
    
/*-----------------------------------------------------------------------------
                          Resize Callback function
            Resizer specific application callback prototype
-----------------------------------------------------------------------------*/    
/**
 *  \brief  Type PSP_RSZappCallback
 *  Application callback function to notify completion of resize operation in
 *  case PSP_RSZ_IOCTL_RESIZE_ASYNC is used. A function of this type and an 
 *  application specific callback argument need to registered via  
 *  PSP_RSZ_IOCTL_SET_RESIZE_CALLBACK
 *
 */
typedef Void  (*PSP_RSZappCallback)(Ptr arg, Int status, Ptr bufp, size_t size);

/******************************************************************************
                              Data-Structures
******************************************************************************/
/**
 *  \brief  Structure _PSP_RSZyenhParams
 *  Luma enhancement element structure
 *
 *  Structure for the specfying luma edge enhancement structure.
 */
typedef struct _PSP_RSZyenhParams
{
    Int type;
     /**< represents luma enable or disable         */
    Uint8 gain;
     /**< Represents gain             */
    Uint8 slop;
     /**< represents slop               */
    Uint8 core;
     /**< Represents core value               */
}PSP_RSZyenhParams;

/**
 *  \brief  Structure _PSP_RSZparams
 *  Resizer Parameters configuration element structure
 *
 *  Conatins all the parameters for resizing . This structure
 *  is used to configure resiser parameters
 */
typedef struct _PSP_RSZparams
{
    Int inHsize;
     /**< represents input frame horizontal size        */
    Int inVsize;
     /**< represents linput frame vertical size         */
    Int inPitch;
     /**< represents offset between two rows of input frame       */
    Int inptyp;
     /**< For determining 16 bit or 8 bit data        */
    Int vertStartingPixel;
     /**< for specifying vertical starting pixel in input         */
    Int horzStartingPixel;
     /**< for specyfing horizontal starting pixel in input        */
    Int cbilin;
     /**< defined, filter with luma or bi-linear Interpolation        */
    Int pixfmt;
    /**< defined, UYVY or YUYV image format         */
    Int outHsize;
    /**< represents output frame horizontal size        */
    Int outVsize;
    /**< represents output frame vertical size        */
    Int outPitch;
    /**< represents offset between two rows of output frame       */
    Int hstph;
    /**< for specifying horizontal starting phase         */
    Int vstph;
    /**< for specifying vertical starting phase         */
    Int16 hfiltCoeffs[32];
    /**< horizontal filter coefficients       */
    Int16 vfiltCoeffs[32];
    /**< vertical filter coefficients         */
    PSP_RSZyenhParams yenhParams;
    /**< represents luma enhacement parameters */
}PSP_RSZparams;

/**
 *  \brief  Structure _PSP_RSZresize
 *   Resizer input output address containning structure
 *
 *  Resize structure passed during the resize IOCTL which hold the input and
 *  output address
 *
 */
typedef struct _PSP_RSZresize
{
    Ptr  inBuf;
    /**< represents input frame starting address        */
    Ptr  outBuf;
    /**< represents output frame starting address         */
    Int inBufSize;
    /**< represents input buffer size                       */
    Int outBufSize;
    /**< represents output buffer size                      */
}PSP_RSZresize;

/**
 *  \brief  Structure _PSP_RSZstatus
 *  Resizer status containning structure
 *
 *  Resize structure which conatins the status of the channel and hardware.
 *
 */
typedef struct _PSP_RSZstatus
{
    Int chanBusy;
    /**< 1: channel is busy, 0: channel is not busy         */
    Int hwBusy;
    /**< 1: hardware is busy, 0: hardware is not busy     */
    Int src;
    /**< # defined, can be either DDR or CCDC/PREVIEWER       */
}PSP_RSZstatus;

/**
 *  \brief  Structure _PSP_RSZpriority
 *  Resizer priority management structure
 *
 *  Resize structure which conatins the priority of every channel.
 */
typedef struct _PSP_RSZpriority
{
    Int priority;
    /**< 0=>5, with 5 the highest priority          */
}PSP_RSZpriority;
/**
 *  \brief  Strcuture _PSP_RSZcropsize
 *  Resizer crop size structure
 *
 *  Resize structure which conatins the croping information about the image.
 */
typedef struct _PSP_RSZcropsize
{
    Int32 hcrop ;
    /**< represents number of pixels per line cropped in output image       */
    Int32 vcrop;
    /**< represents number of lines cropped in output image         */
}PSP_RSZcropsize;
/**
 *  \brief  Strcuture _PSP_RSZcallback
 *  Resizer application call back structure for async resize operation
 *
 *  Structure contains the application callback function and the application 
 *  callback argument
 */
typedef struct _PSP_RSZcallback
{
    PSP_RSZappCallback fxn;
    Ptr                arg;
}PSP_RSZcallback;
/* @} DDCResizerMain */

#ifdef __cplusplus
}
#endif

#endif  /* _PSP_Resizer_H_ */
