/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \file   dda_rszIom.c
 *
 *  \brief  RSZ IOM layer Functions
 *
 *  This file contains the Implementation details of IOM layer functions
 *
 *  (C) Copyright 2010, Texas Instruments, Inc
 *
 *  \author     EI3
 *
 *  \version    0.1     Created
 *              1.0     First release
 */

/******************************************************************************
  Header File Inclusion
 ******************************************************************************/
#include <ti/sdo/pspdrivers/common/psp_common.h>
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/tistdtypes.h>
#include <iom.h>
#include <sem.h>
#include <ti/sdo/pspdrivers/drivers/resizer/psp_resizer.h>
#include <ti/sdo/pspdrivers/drivers/resizer/src/psp_rszApi.h>
#include <ti/sdo/pspdrivers/drivers/resizer/src/dda_rsz.h>
#include <ti/sdo/pspdrivers/drivers/resizer/src/ddc_rsz.h>
#include <ti/sdo/pspdrivers/drivers/resizer/src/dda_rszIom_BIOS.h>

/******************************************************************************
  Mini Driver Function interface table
 ******************************************************************************/
/**
 * \brief Mini Driver Function interface table
 *
 *
 */
IOM_Fxns RSZMD_FXNS =
{
    &RSZ_mdBindDev,
    &RSZ_mdUnBindDev,
    &RSZ_mdControlChan,
    &RSZ_mdCreateChan,
    &RSZ_mdDeleteChan,
    IOM_SUBMITCHANNOTIMPL
};

/******************************************************************************
  IOM Error code look up table
 ******************************************************************************/
/**
 * The following mapping shows the mapping done from PSP_Result to IOM error
 * code.
 *
 * PSP_SOK                     (0)   IOM_COMPLETED            0
 * PSP_SINPROGRESS             (1)   IOM_PENDING              1
 * PSP_E_DRIVER_INIT           (-1)  IOM_ETIMEOUTUNREC      -11
 * PSP_E_NO_MEMORY             (-2)  IOM_EALLOC              -5
 * PSP_E_RESOURCES             (-3)  IOM_EALLOC              -5
 * PSP_E_INVAL_STATE           (-4)  IOM_EBADMODE            -7
 * PSP_E_INVAL_PARAM           (-5)  IOM_EBADARGS           -10
 * PSP_E_NOT_SUPPORTED         (-6)  IOM_ENOTIMPL            -9
 * PSP_E_IO_CANCEL_FAIL       (-7)  IOM_ETIMEOUT            -2
 * PSP_E_FIFO_NOT_ENABLED     (-8)  IOM_EBADMODE            -7
 * PSP_E_INVALID_MODE       (-9)  IOM_EBADMODE            -7
 * PSP_E_INVALID_CMD        (-10) IOM_EBADARGS           -10
 *
 */


/**
 * \brief IOM Port Object
 *
 */
RSZPortObject rsz_ports = {PSP_RESIZER_DELETED};

/******************************************************************************
  Function prototypes
 ******************************************************************************/


Int RSZ_mdBindDev(Ptr  * devp, Int devid, Ptr const devParams);
static Int getIomErrorCode(PSP_Result pspCode);
Int RSZ_mdUnBindDev(Ptr const devp);
Int RSZ_mdCreateChan(Ptr *chanp, Ptr devp, String name, Int mode,
                     Ptr chanParams, IOM_TiomCallback cbFxn, Ptr cbArg);
Int RSZ_mdDeleteChan(Ptr chanp);
Int RSZ_mdControlChan(Ptr chanp, Uns cmd, Ptr arg);

/**
 *    \brief  PSP error code converter
 *
 * This function converts PSP error codes to appropriate IOM error code.
 *
 * \param  pspCode              [IN]    PSP Error code.
 *
 * \return IOM ERROR CODE     [OUT] Equivalent IOM error code. For unrecoganized
 *                                  error code IOM_EBADIO is returned.
 *
 */
/******************************************************************************
  Functions
 ******************************************************************************/

/**
 *\brief getIomErrorCode
 *  fuction will get the IOM Error Code
 *
 */
static Int getIomErrorCode(PSP_Result pspCode)
{
    /* Mapping of psp level error code with the iom layer code. The maximum
       error code return from the below layer is 10 so taking the error code*/
    /* Please refer to the description and mapping in Line-76               */
    static Int iomErrorCodeList[12] =
    {
        IOM_EBADARGS,
        IOM_EBADMODE,
        IOM_EBADMODE,
        IOM_ETIMEOUT,
        IOM_ENOTIMPL,
        IOM_EBADARGS,
        IOM_EBADMODE,
        IOM_EALLOC,
        IOM_EALLOC,
        IOM_ETIMEOUTUNREC,
        IOM_COMPLETED,
        IOM_PENDING
    };

    Int iomCode = pspCode+10;
    /* Mapping the error code with its equivalent IOM layer error code */
    if ((12 > iomCode)  && (iomCode >= 0))
    {
        iomCode = iomErrorCodeList[iomCode];
    }
    else
    {
        iomCode = IOM_EBADIO;
    }

    return(iomCode);
}

/******************************************************************************
  RSZ_mdBindDev() Function
 ******************************************************************************/

/**
 * \brief  Function RSZ_mdBindDev
 * RSZ_mdBindDev() is called by DEV_init() to bind and initialize
 *            the instance
 */
/* ARGSUSED */
Int RSZ_mdBindDev(Ptr  * devp, Int devid, Ptr const devParams)
{
    Int     statusBD         = IOM_COMPLETED;
    Int index;
    DDC_RSZDeviceObject *rszDeviceObject;


    /*Check the parameters not NULL and validate the instance */


    if ((NULL ==devp))
    {
        statusBD =  (IOM_EBADARGS);
    }
    else
    {
        /* Setting initial global values to zero */
        if (PSP_RESIZER_DELETED !=  rsz_ports.state)
        {
             statusBD = IOM_EBADARGS;
        }
        else
        {
            for(index = 0;index < DDA_NUMCHANS; index++)
            {

                rsz_ports.chan[index].port = NULL;
                rsz_ports.chan[index].ddc_Handle = NULL;
            }
            rsz_ports.numOfOpens = 0;
            rsz_ports.rszPortNumber =0;
            /* Initialize Driver*/
            rszDeviceObject
                = (DDC_RSZDeviceObject *)PSP_RSZCreate((Uint32)devid);
            if (NULL== rszDeviceObject)
            {
                statusBD = IOM_EBADARGS;
            }
            else
            {
                /*Update the Port object */

                rsz_ports.dda_PortObj         = rszDeviceObject;
                rsz_ports.rszPortNumber     = (Uint32)devid;
                rsz_ports.state         = PSP_RESIZER_CREATED;
                rsz_ports.PortSemaphore = SEM_create(1, NULL);
                *devp                 = &rsz_ports;
            }
        }
    }

    return(statusBD);
}

/******************************************************************************
  RSZ_mdUnBindDev
 ******************************************************************************/
/**
 * \brief Function RSZ_mdUnBindDev
 * RSZ_mdUnBindDev() is called by DEV_init() to unbind and
 *           de-init the instance
 */
Int RSZ_mdUnBindDev(Ptr const devp)
{
    Int             statusUB     = IOM_COMPLETED;
    RSZPortObject*  rsz_portsUB     = NULL;
    /*Check the parameter not NULL */
    if (NULL == devp)
    {
        statusUB = IOM_EBADARGS;
    }
    else
    {
        rsz_portsUB = (RSZPortObject *)devp;
        /*Check the state of port */
        if ((PSP_RESIZER_CREATED != rsz_portsUB->state)
            && (PSP_RESIZER_CLOSED  != rsz_portsUB->state))
        {
            statusUB =  IOM_EBADMODE;
        }
        else
        {
            /* De-Init Driver.This function closes channel handle */
            statusUB = PSP_RSZDelete(rsz_portsUB->rszPortNumber);
            if (PAL_SOK != statusUB)
            {
                statusUB = getIomErrorCode(statusUB);
            }
            else
            {
                /* Set the state as deleted */
                SEM_delete(rsz_portsUB->PortSemaphore);
                rsz_portsUB->state = PSP_RESIZER_DELETED;
            }
        }


    }
    return(statusUB);
}
/*******************************************************************************
  RSZ_mdCreateChan
  Creates the channel for the data transfer by setting up some params
 ******************************************************************************/
/**
 * \brief Function RSZ_mdCreateChan
 * Creates the channel for the data transfer by setting up some params
 *
 */
/* ARGSUSED */
Int RSZ_mdCreateChan(Ptr *chanp, Ptr devp, String name, Int mode,
    Ptr chanParams, IOM_TiomCallback cbFxn, Ptr cbArg)

{
    Int                     statusCC     = IOM_COMPLETED;
    RSZPortObject*          portCC     = NULL;
    RSZChannelObject*       channelCC = NULL;
    Int segId;
    Int freechan;


    /*Check the parameters not NULL */
    if ((((NULL == chanp) || (NULL == devp)) || (NULL == chanParams))
        || ((NULL == cbFxn)  || (NULL == cbArg)))

    {
        statusCC = IOM_EBADARGS;
    }
    else if ( ((mode != IOM_INPUT) && (mode != IOM_OUTPUT) )
        && (mode != IOM_INOUT) )
    {
        statusCC = IOM_EBADMODE;
    }
    else
    {
        portCC = (RSZPortObject *)devp;
        /*Check the port state */
        if (((PSP_RESIZER_CREATED != portCC->state)
                && (PSP_RESIZER_CLOSED  != portCC->state))
            && (PSP_RESIZER_OPENED  != portCC->state))
        {
            statusCC =  (IOM_EBADMODE);
        }
        else
        {

            SEM_pend(portCC->PortSemaphore, DDA_OSSEM_NO_TIMEOUT);
            if (portCC->numOfOpens >= PSP_RESIZER_NUM_CHANNELS )
            {
                DEBUG_PRINTF("\nmodules usage count is greater than supported");
                statusCC = IOM_EBADARGS;
            }
            else
            {
                for(freechan = 0;freechan < PSP_RESIZER_NUM_CHANNELS;
                    freechan++)
                {
                    if (NULL == portCC->chan[freechan].ddc_Handle)
                    {
                        break;
                    }
                }

                channelCC = &(portCC->chan[freechan]);
            }
            SEM_post(portCC->PortSemaphore);
        }
    }
    if (IOM_COMPLETED == statusCC)
    {
        /* Creates handle for resizer channel */
        segId = *(Int *)chanParams;
        channelCC->ddc_Handle = (PSP_Handle)PSP_RSZOpen(segId);
        if (NULL == channelCC->ddc_Handle)
        {
            statusCC = IOM_EBADARGS;
        }
        else
        {
            /*Updating port and chan object */
            portCC->state = PSP_RESIZER_OPENED;

            /*Increment the number of opens */

            SEM_pend(portCC->PortSemaphore, DDA_OSSEM_NO_TIMEOUT);

            ++portCC->numOfOpens;

            SEM_post(portCC->PortSemaphore);

            statusCC =  (IOM_COMPLETED);

            /* sets for back referencing */
            channelCC->port = (RSZPortObject *)portCC;
            *chanp = channelCC;
        }
    }
    return(statusCC);
}

/******************************************************************************
  RSZ_mdDeleteChan
 *****************************************************************************/
/**
 * \brief  Function RSZ_mdDeleteChan
 * Deletes an Resizer Channel
 *
 */
Int RSZ_mdDeleteChan(Ptr chanp)
{
    Int                 statusDC         = IOM_COMPLETED;
    RSZChannelObject*   chanDC         = NULL;
    RSZPortObject*      portDC=NULL;

    /*Check the parameters not null */

    if (NULL == portDC)
    {
        if (NULL == chanp)
        {
            statusDC = IOM_EBADARGS;
        }
        else
        {
            /* setting up channel and port handle */
            chanDC = (RSZChannelObject *)chanp;
            portDC = (RSZPortObject *)chanDC->port;
            /*Check the Channel is in use and port handle not null */
            if (((NULL == portDC) || (PSP_RESIZER_OPENED != portDC->state)))
            {
                statusDC = IOM_EBADARGS;
            }
        }
        if (IOM_COMPLETED == statusDC)
        {
            /*Closes the Channel Handle */
            statusDC = PSP_RSZClose(chanDC->ddc_Handle);
            if (PAL_SOK != statusDC)
            {
                statusDC = getIomErrorCode(statusDC);
            }
            else
            {

                /* Updates the state and makes the channel port void*/
                chanDC->port = NULL;
                chanDC->ddc_Handle = NULL;

                /* Check if all the channels are closed */
                /* If all channels are closed then update the state to closed */

                SEM_pend(portDC->PortSemaphore, DDA_OSSEM_NO_TIMEOUT);

                if (0 == --portDC->numOfOpens)
                {
                    portDC->state = PSP_RESIZER_CLOSED;
                }
                SEM_post(portDC->PortSemaphore);

                chanp = &chanDC;
            }
        }
    }
    return(statusDC);
}

/******************************************************************************
  RSZ_mdControlChan
  All dynamic configurations functionality of a mini driver can be done.
 ******************************************************************************/
/**
 *\ brief Function RSZ_mdControlChan
 * mdcontrolchan is used to config the ioctl operation
 *
 */
Int RSZ_mdControlChan(Ptr chanp, Uns cmd, Ptr arg)
{
    Int         statusCO             = IOM_COMPLETED;
    RSZChannelObject* chanCO     = NULL;
    RSZPortObject* portCO = NULL;

    if (NULL == portCO)
    {
        if (NULL == chanp)
        {
            statusCO = IOM_EBADARGS;
        }
        else
        {
            /* setting up channel and port handle */
            chanCO = (RSZChannelHandle)chanp;
            portCO = (RSZPortObject* )chanCO->port;
            /*Check the port and state of port */
            if ((NULL == portCO)|| (PSP_RESIZER_OPENED != portCO->state))
            {
                statusCO = IOM_EBADMODE;
            }
            else
            {
                /* Call the IOctl function which execute the dynamic
                 * configuration*/
                if ((NULL != cmd) && (NULL != arg))
                {
                    statusCO = PSP_RSZIoctl(chanCO->ddc_Handle,
                        (PSP_RSZIoctlCmd)cmd, arg, NULL);
                }
                /*Check status if success */
                if (PSP_SOK != statusCO)
                {
                    /*Generate IOM error code equivalent to PSP error code */
                    statusCO = getIomErrorCode(statusCO);
                }
            }
        }
    }
    return(statusCO);
}

