/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \file   dda_rszIom_BIOS.h
 *
 *  \brief  RESIZER IOM and BIOS layer definition
 *
 *  This file contains the interfaces, data types and symbolic definitions
 *  that are needed by the application to utilize the serivces of the Resizer
 *  device driver.
 *
 *  (C) Copyright 2010, Texas Instruments, Inc
 *
 *  \author     EI3
 *
 *  \version    0.1     Created
 *              1.0     First release
 */

#ifndef _DDA_RSZIOM_BIOS_H
#define _DDA_RSZIOM_BIOS_H
/******************************************************************************
  Header files
 ******************************************************************************/
#include <iom.h>
#include <ti/sdo/pspdrivers/drivers/resizer/psp_resizer.h>
#include <ti/sdo/pspdrivers/drivers/resizer/src/ddc_rsz.h>

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
  Macros
 ******************************************************************************/
/**
 * \brief  max number of channel supported by interfaces.
 *
 */
#define DDA_NUMCHANS      PSP_RESIZER_NUM_CHANNELS

#define DDA_RSZ_PORT_INIT                          ((Uint32)0u)



/******************************************************************************
  Data-Structures
 ******************************************************************************/


/**
 * \brief  Structure RSZCHANNELOBJ
 *  Structure defines IOM Channel Channel structure used by the IOM layer
 *
 */
typedef struct RSZCHANNELOBJ
{


    struct RSZPORTOBJ    *port;
    /**< Reverse pointer to Port */
    PSP_Handle                  ddc_Handle;
    /**< Handle Resizer..module */
} RSZChannelObject , *RSZChannelHandle;

/**
 * \brief Structure RSZPORTOBJ
 * Structure defines the IOM Port .Port structure used by the IOM layer
 */
typedef struct RSZPORTOBJ
{
    resizerState                  state;
    /**< state of Resizerdriver */
    Uns                        rszPortNumber;
    /**< Instance number */
    RSZChannelObject          chan[DDA_NUMCHANS];
    /**< channels for the Resizer */
    DDC_RSZDeviceObject       *dda_PortObj;
    /**< Underlying DDA Structure */
    Uint                       numOfOpens;
    /**< number of channel opens for Resizer driver */
    SEM_Handle               PortSemaphore;
    /** <semaphore to protect the global structure */
} RSZPortObject, *RSZPortHandle;

/******************************************************************************
  Functions
 ******************************************************************************/
/**
 *  \brief Initialize a given Resizer driver instance
 *
 *  It performs register map overlaying and software book-keeping.It register
 *  the interrupt Handler.
 *
 *  \param  devp           [IN]    Void pointer passed by the GIO Layer to
 *                                  store the address of the port structure
 *
 *  \param  devid          [IN] device instance number
 *
 *  \param  devParams     [IN]  Initialization configuration structure
 *
 *  \return success             IOM_COMPLETED
 *          failure             IOM Error
 */
Int RSZ_mdBindDev(Ptr *devp, Int devid, Ptr devParams);
/**
 *  \brief Delete a given RSZ driver instance
 *
 *  This function is called last and is used for driver deletion.This function
 *  does the deletion of the entire RSZ hardware.
 *
 *  \param  devp         [IN]   pointer in the GIO layer giving the address of
 *                              port object of that particular instance
 *
 *  \Return                     On success returns IOM_COMPLETED
 *                              On failure returns IOM Error code
 */
Int RSZ_mdUnBindDev(Ptr devp);
/**
 *  \brief Open Channel of the RSZ device
 *
 *  The function prepares channel of the driver instance for data transfers
 *  and returns an handle to the driver instance.
 *
 *  \param chanp         [IN]     pointer to the channel structure
 *
 *  \param devp         [IN/OUT] pointer to the port structure
 *
 *  \param name         [IN]     name of the driver
 *
 *  \param mode         [IN]     mode of the channel i.e. INPUT or OUTPUT
 *
 *  \param chanParams     [IN/OUT] Extra parameter (Not used in this driver)
 *
 *  \param cbFxn        [IN]     callback to the GIO Layer
 *
 *    \param cbArg      [IN]     arguments for the callback
 *
 *  \Returns                      On success returns IOM_COMPLETED
 *                                 On failure returns IOM Error code
 */
Int RSZ_mdCreateChan(Ptr *chanp, Ptr devp, String name, Int mode,
    Ptr chanParams, IOM_TiomCallback cbFxn, Ptr cbArg);
/**
 *  \brief Close Channel of the RSZ device
 *
 *     This function is used to close Channel handle for individual RSZ
 *      Modules.
 *
 *  \param  chanp         [IN]  RSZ Driver Channel Handle
 *
 *  \return                     On success returns IOM_COMPLETED
 *                              On failure returns IOM Error code
 */
Int RSZ_mdDeleteChan(Ptr chanp);

/**
 *  \brief RSZ Ioctl
 *
 *  This function provides ioctl functionality for RSZ
 *
 *  \param  chanp         [IN]     RSZ Driver Channel Handle
 *
 *  \param  cmd         [IN]     Operation to be performed, typically an enum
 *                                 gets passed
 *
 *  \param  arg         [IN/OUT] Provides additonal information related to the
 *                                 operation
 *
 *  \return success              IOM_COMPLETED
 *          failure               IOM Error
 */
Int RSZ_mdControlChan(Ptr chanp, Uns cmd, Ptr arg);

/**
 *  \brief submit IO Request
 *
 *      This function is called to Queue/Dequeue Frame buffer for individual RSZ
 *     modules.
 *
 *  \param  chanp         [IN]  RSZ Driver Channel Handle
 *
 *  \param  packet         [IN] IOM packet passed by the upper layer containing
 *                              information about the size, address, timeout and
 *                              other necessary parameters
 *
 *  \return                     On success, returns positive number - depending
 *                              upon the operation requested
 *
 *                              If failed, returns negative number - IOM Error
 *                              code
 */
Int RSZ_mdSubmitChan(Ptr chanp, IOM_Packet *packet);


/******************************************************************************
  BIOS SPECIFIC FUNCTIONS
 ******************************************************************************/
/**
 * \brief Registers interrupt handler for Resizer driver
 *
 ** \param  IntNum      [IN]         interrupt number
 *  \returns NULL
 *
 */
void DDA_RSZRegisterIntrHandler( Uint32 IntNum);

/**
 * \brief Unregisters Interrupt handler for RSZ
 *
 ** \param intrNum      [IN]           interrupt number
 * \returns NULL
 *
 */
void DDA_RSZUnregisterIntrHandler(Uint32 intrNum);

#ifdef __cplusplus
}
#endif

#endif /*_DDA_RSZIOM_BIOS_H */

