/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 2006 - 2010 Texas Instruments Incorporated        |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/


/**
 *    \file    psp_spi.h
 *
 *    \brief    SPI interface definition
 *
 *    This file contains the interfaces, data types and symbolic definitions
 *    that are needed by the application to utilize the serivces of the SPI
 *    device driver on DM648 board.
 *
 *    (C) Copyright 2006, Texas Instruments, Inc
 *
 *    \author        PSG
 *
 *    \version       0.1
 *
 *  <b>Compiler Switch Used</b>
 *
 *  <b>_DEBUG</b><br>
 *   This is used as a option to compiler whether to include the debug statement
 *   inserted in the code into the final image. This flag helps to build DEBUG
 *   image of the program. For RELEASE images this is not passed to the compiler.
 *
 *  <b>CHIP_XXXX</b><br>
 *   The CSL layer is written in a common file for all the variants of a SOC.
 *   This flag differentiates the variant we are compiling for, for e.g.
 *   CHIP_DM648, and the CSL definitions for that variant appropriately gets
 *   defined for register base addresses, num of ports of a peripheral etc.
 *
 *  <b>SPI_INSTRUMENTATION_ENABLED</b><br>
 *   This option is passed to the compiler to include the instrumentation code
 *   parts into the final image/lib of the program. This helps build the
 *   iRelease/iDebug versions of the image/lib with a common code base.
 *
 *  <b>PSP_SPI_EDMA_SUPPORT</b><br>
 *   This option is passed to the compiler for EDMA mode.
 *
 *  <b>SPI_DEBUG_PRINTF</b><br>
 *   They are the macros used to print debug messages. These expand a valid
 *   print statement.
 *
 */


#ifndef _PSP_SPI_H_
#define _PSP_SPI_H_

#include <ti/sdo/pspdrivers/common/psp_common.h>

#ifdef CHIP_DM648
#include <ti/sdo/pspdrivers/soc/dm648/dsp/tistdtypes.h>
#endif

#ifdef CHIP_C6452
#include <ti/sdo/pspdrivers/soc/c6452/dsp/tistdtypes.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

/**
  * Max SPI driver instances
  */
#define PSP_SPI_NUM_INSTANCES      1u

/**
  * Max SPI driver opens - which means number of s/w channels which can be
  * opened for a single instancse of SPI
  */
#define PSP_SPI_NUM_OPENS          1u


/* Error return coded*/
/**
 * Base Erroe code
 */
#define PSP_SPI_ERROR_BASE             (-11)
/* Continuation from the PSP_Common.h file*/

/**
  * Continuation from the PSP_Common.h file
  * SPI receive overrun error code
  */
#define PSP_SPI_RECEIVE_OVERRUN_ERR    (PSP_SPI_ERROR_BASE-1)
/**
 * SPI Bit transfer error code
 */
#define PSP_SPI_BIT_ERR                (PSP_SPI_ERROR_BASE-2)
/**
 * SPI Slave desync error code
 */
#define PSP_SPI_DESYNC_ERR             (PSP_SPI_ERROR_BASE-3)
/**
 * SPI parity error code
 */
#define PSP_SPI_PARITY_ERR             (PSP_SPI_ERROR_BASE-4)
/**
 * SPI time out error code
 */
#define PSP_SPI_TIMEOUT_ERR            (PSP_SPI_ERROR_BASE-5)
/**
 * SPI Data length  error code
 */
#define PSP_SPI_DATALENGTH_ERR         (PSP_SPI_ERROR_BASE-6)
/**
 * SPI Cancel IO error code
 */
#define PSP_SPI_CANCEL_IO_ERROR        (PSP_SPI_ERROR_BASE-7)


/**
 * To enable/Disable CSHOLD
 */
#define PSP_SPI_CSHOLD                 (0x1u)

/**
 * To CSHOLD will be selected [pulled low]after the trasceive operation and
 * this is valid if PSP_SPI_CSHOLD flag is enabled. Also if Chip select value is
 * changed in the next transceive operation then also this flag becomes invalid
 * for the previous rtansceive operation i.e CS linies comes high.
 */
#define PSP_SPI_CSHOLD_FOR_MULTI_TRANSCEIVE            (0x2u)

/**< NOTE: If no flag is specified in the flags, by default CSHOLD is disabled*/

/**
  * \brief Enumeration to set SPI master to different opmode
  *
  * PSP_SPI_OPMODE_3PIN        -  3-Pin configuration
  * PSP_SPI_OPMODE_SPISCS_4PIN -  4-Pin configuration with Chip Select
  * PSP_SPI_OPMODE_SPIENA_4PIN -  4-Pin configuration with Enable pin
  * PSP_SPI_OPMODE_5PIN        -  5-Pin configuration with handshake mode
  *
  */
typedef enum PSP_spiPinOpMode_t
{
    PSP_SPI_OPMODE_3PIN         =  0,
    /**< SPI master 3 pin mode                 */
    PSP_SPI_OPMODE_SPISCS_4PIN  =  1,
    /**< SPI master 4 pin mode uses SPISCS     */
    PSP_SPI_OPMODE_SPIENA_4PIN  =  2,
    /**< SPI master 4 pin mode uses SPIENA     */
    PSP_SPI_OPMODE_5PIN         =  3
    /**< SPI master 5 pin mode                 */
}PSP_spiPinOpMode;


/**
  * \brief Enumeration to set SPI as master or slave
  *
  */
typedef enum PSP_spiType_t
{
    PSP_SPI_TYPE_MASTER         =  0,
    /**< SPI master mode of operatoin          */
    PSP_SPI_TYPE_SLAVE  =  1
    /**< SPI slave mode of operatoin           */
}PSP_spiType;

/**
  * \brief Enumeration to select the different data transfer format.
  *        All required param of this register is mention below in
  *        Structure PSP_SpiConfigDataFmt.
  */
typedef enum PSP_spiDataFormat_t
{
    PSP_SPI_FMTSEL_0  =  0,
    /**< select format 0     */
    PSP_SPI_FMTSEL_1  =  1,
    /**< select format 1     */
    PSP_SPI_FMTSEL_2  =  2,
    /**< select format 2     */
    PSP_SPI_FMTSEL_3  =  3
    /**< select format 3     */
}PSP_spiDataFormat;


/**
 * \brief enumeration of IOCTLS Supported for
 *        LOOPBACK,CANCEL IO.
 */
typedef enum PSP_SpiIoctlCmd_t
{
    PSP_SPI_IOCTL_CANCEL_PENDING_IO,
    /**< To cancel pending IO                              */
    PSP_SPI_IOCTL_SET_CS_POLARITY
    /**< to set or reset CS Polarity                       */
} PSP_SpiIoctlCmd;

/**
  * \brief      Delay values for configuring the SPI.  All DELAY value is used
  *             in master mode only.
  *
  * c2TDelay -  It defines a setup time for the slave device that delays
  *                the data transmission from the chip select active edge.
  * t2CDelay -  It defines a hold time for the slave device that delays
  *                the chip select deactivation by a multiple of VBUSPCLK
  *                cycles after the last bit is transferred.
  * t2EDelay -  It defines a time-out value as a multiple of SPI clock before
  *                the ENAble signal has to become inactive and after the CS
  *                becomes  inactive.
  * c2EDelay -  C2EDELAY defines the maximum time between the SPI activates the
  *                chip select signal and the addressed slave has to respond by
  *                activating the ENA signal.
  *                It applies only if the addressed slave generates an ENA
  *                signal as a hardware handshake response.
  *
  */
typedef struct PSP_spiDelayParam_t
{
    Uint32  c2TDelay ;
    /**< [IN] Chip-select-active-to-transmit-start-delay             */
    Uint32  t2CDelay ;
    /**< [IN] Transmit-end-to-chip-select-inactive-delay             */
    Uint32  t2EDelay ;
    /**< [IN] Transmit-data-finished-to-ENA-pin-inactive-time-out    */
    Uint32  c2EDelay ;
    /**< [IN] Chip-select-active-to-ENA-signal-active-time-out       */
} PSP_spiDelayParam;

/**
  * \brief Structure to initialize the data transfer format register.
  */
typedef struct PSP_SpiConfigDataFmt_t
{
    Uint32          wDelay;
    /**< Delay in between two transfers */

    Uint32          charLength;
    /**< defines the word length of the data format,
     * Legal values are b/w 2-16 */

    Bool            lsbFirst;
    /**< Sets the Shift of bits, TRUE - LSB First ; FALSE - MSB First */

    Bool            phaseIn;
    /**< True - Phase In,data & clock in phase ; False - Phase out,
      *  data 1/2 cycle before clock                         */

    Bool            oddParity;
    /**< True - Odd Parity ; False - Even Parity, Also this is valid only
    *    if ParityEnable is true                        */

    volatile Bool   parityEnable;
    /**< Parity generation check in the data format, True - Cheks for parity
      *     False - No Check                        */

    volatile Bool   clkHigh;
    /**< Sets the polarity of the clock, True - POLARITY HIGH ,
      *   False - POLARITY LOW                                    */

    volatile Bool   waitEnable;
    /**< True - if in master mode,  wait for ENA signal from slave
      *  False - do not wait                                */
}PSP_SpiConfigDataFmt ;


/**
 * \brief SPI Hardware Configuartion Structure
 */
typedef struct PSP_spiHWConfigData_t
{
    Bool                intrLevel;
    /**< True - maps to Intr1 Line ; false - Intr0 line      */

    PSP_spiType         masterOrSlave;
    /**< True - Slave Mode , False - Master Mode            */

    Bool                clkInternal;
    /**< True - Selects Internal clock source ; False - Selects external
     *   clock source                                     */

    Bool                enableHighZ;
    /**< whether ENA signal should be tristated when inactive - True ;
     *   or it should bear a value - False                */

    PSP_spiPinOpMode    pinOpModes;
    /**< Spi Operation Modes                                */

    PSP_spiDelayParam   delay;
    /**< SPI delay registers value                            */

    Uint32              waitDelay;
    /**< True - enable format delay between 2 consecutive transfers ;
       *  False - No delay b/w 2 transfers                */

    Uint32              csDefault;
    /**< Default chip select pattern*/

    PSP_SpiConfigDataFmt configDatafmt[4] ;
    /**< Data Format Configuration values */
}PSP_spiHWConfigData ;

/**
 *  \brief SPI PSP Configuration Object
 *
 *                      This structure basically holds the driver mode of
 *                      operation and the Clock Frequency ,the Hardware
 *                      Config Structure and Edma handle.
 *
 */
typedef struct PSP_SpiConfig_t
{
    PSP_OpMode          mode;
    /**< [IN] Driver operation mode                                    */

    Uint32              moduleInputClkFreq;
    /**< [IN] SPI Module Clock frequency to initialize the driver      */

    Uint32              outputClkFreq;
    /**< [IN] output clock frequency from SPI Module , transmits bitrate
        in bits per second                                             */

    PSP_spiHWConfigData spiHWCfgData;
    /**< [IN] Hardware configuration                                   */

    Bool                loopBack;
    /**< [IN] To enable/disable loop back                              */

    PSP_Handle          edmaHandle;
    /**< [IN] Handle for Edma                                          */

} PSP_SpiConfig;


/**
  * \brief SPI Transaction structure
  *
  * This structure holds the information needed to carry out a transaction on
  *                            an SPI bus to a slave device.
  */
typedef struct PSP_spiTransaction_t
{
    Uint8               *outBuffer;
    /**< Data buffer - During transceieve operation this buffer data is
                       transmitted                                      */
    Uint8               *inBuffer;
    /**< Data buffer - During transceive operation read data is stored
                       in this buffer                                   */
    Uint32              bufLen;
    /**< Length of the buffer                                           */

    Uint32              chipSelect;
    /**< chipselect ot be selected                                      */

    PSP_spiDataFormat   dataFormat;
    /**< which data format register to select                           */

    Uint32              flags;
    /**< Flags  -     To indicate the Read / Write modes of operation   */

    Ptr                 param;
    /**< Extra paramerter                                               */
}PSP_spiTransaction;


#ifdef BIOS_BUILD

/**
  *  \brief  Data packet element structure passed to submit function
  *
  *  Structure for the spi specific buffer address to be passed to the GIO.
  */
typedef struct PSP_spiDataParam_t
{

    PSP_spiTransaction  spiTrans;
    /**<  parameters needed for transfer of data    */
    Int                 timeout;
    /**<  timeout value passed ny user              */

}PSP_spiDataParam;

#endif
#ifdef __cplusplus
}
#endif

#endif    /* _PSP_SPI_H_ */

