/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file   dda_spiBios.c
 *
 *  \brief  spi Driver Adaptation for BIOS
 *
 *  This file spi_Ports adaptation of spi driver for BIOS
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author     PSG
 *
 *  \version  0.1  Shailesh Kumar Sharma    Created
 *  \version  0.2  Shailesh Kumar Sharma    Modified for DM648
 *  \version  0.3  Nagarjuna Kristam        Modified for New implementaion for
 *                                          chipSelect,dataFormat and cs hold
 *  \version  0.4  Nagarjuna Kristam        Modified for adding INSTRUMENTAION
 *  \version  0.5  Nagarjuna Kristam        Removed memsetting port object
 *                                           and clean up errort
 *  \version  0.6  NAgarjuna Kristam        Updated as per review coments
 *                                              REVW00004866
 */

/* Header files inclusion */
#include <std.h>
#include <ecm.h>
#include <C64.h>
#include <string.h>

#include <ti/sdo/pspdrivers/pal_os/pal_osWait.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osmem.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osprotect.h>
#include <ti/sdo/pspdrivers/drivers/spi/src/dda_spiBios.h>

#ifdef CHIP_DM648
#include <ti/sdo/pspdrivers/soc/dm648/dsp/soc.h>
#endif /* for DM648 */

#ifdef CHIP_C6452
#include <ti/sdo/pspdrivers/soc/c6452/dsp/soc.h>
#endif /* fro C6452 */

#ifdef SPI_INSTRUMENTATION_ENABLED
#include <ti/sdo/pspdrivers/pal_os/bios/psp_log.h>
#endif /*SPI_INSTRUMENTATION_ENABLED*/

/**
  * SPI hardware interrupt number ,that will enable the interrupt function
  */
#define SPI_HW_INTR_NUM     HWI_EVT01_CMB_IRQ


/**
 * This is the globally declared port object which will be containing
 * all the information for the parameters of the driver
 */
static SpiPortObj spi_Ports[PSP_SPI_NUM_INSTANCES] = {0,
                                                      {0,
                                                       NULL,
                                                       NULL,
                                                       NULL,
                                                       NULL
                                                      },
                                                      SPI_DELETED
                                                     };


/*
 *Local function prototypes
 */
static Int spi_mdBindDev(Ptr *devp, Int devid, Ptr devParams);
static Int spi_mdControlChan(Ptr chanp, Uns cmd, Ptr arg);
static Int spi_mdCreateChan(
                            Ptr *chanp,
                            Ptr devp,
                            String name,
                            Int mode,
                            Ptr chanParams,
                            IOM_TiomCallback cbFxn,
                            Ptr cbArg);

static Int spi_mdDeleteChan(Ptr chanp);
static Int spi_mdSubmitChan(Ptr chanp, IOM_Packet *packet);
static Int spi_mdUnBindDev(Ptr devp);
static Int32 spi_getIomErrorCode(PSP_Result pspErrCode);


/**
 * Public Mini Driver interface table of SPI.
 */
IOM_Fxns SPIMD_FXNS =
{
    &spi_mdBindDev,
    &spi_mdUnBindDev,
    &spi_mdControlChan,
    &spi_mdCreateChan,
    &spi_mdDeleteChan,
    &spi_mdSubmitChan,
};


/**
 *  ======== _DDA_spiRegisterIntrHandler ========
 *  \brief  Register interrupt with BIOS and enable it
 *
 *  \param  instNum      Instance number
 *  \param  intNum       Interrupt number
 *  \param  initIsr      Function Pointer to ISR function
 *  \param  spiObj       Handle to be passed to ISR function
 *
 *  \return void
 */
/* ARGSUSED */
void _DDA_spiRegisterIntrHandler(
                                Uint32      instNum,
                                Uint32      intNum,
                                pspSpiIsr   initIsr,
                                PSP_Handle  spiObj
                               )
{

    ECM_Attrs ecmattrs = ECM_ATTRS;

    /**< Depending upon the device id of the spi*/
    switch(instNum)
    {

        case 0:
            /* Mapping the event id to the ECM Dispatch of the DSP/BIOS*/

            ecmattrs.unmask = (Int)1;
            ecmattrs.arg    = (Arg)spiObj;
            ECM_dispatchPlug(
                             intNum,
                             (ECM_Fxn)initIsr,
                             &ecmattrs
                            );

            /* Enabling the event */
            ECM_enableEvent(intNum);
            break;

        default:
            break;

    }

    /* Enabling the HWI_ID */
    C64_enableIER(SPI_HW_INTR_NUM);
}

/**
 *  ======== _DDA_spiUnregisterIntrHandler ========
 *  \brief  Un-Register interrupt with BIOS and disable it
 *
 *  \param  instNum      Instanse number
 *  \param  intNum       Interrupt number
 *
 *  \return void
 */
/* ARGSUSED */
void _DDA_spiUnregisterIntrHandler(Uint32 instNum, Uint32 intNum)
{
    /* Disabling the interrupts */
    ECM_disableEvent(intNum);
}


/**
  * \brief  spi_mdUnBindDev
  *         spi_mdUnBindDev() is called by DEV_deleteDevice() to unbind
  *         and de-initialize the driver Instance.This is done by calling
  *         psp function PSP_spiDelete().
  *
  * \param devp     [IN]     Device handle in the GIO layer giving the address
  *                          of the port object of that particular instance
  *
  * \returns                 IOM_COMPLETED if success<br>
  *                          IOM_EBADARGS  error code<br>
  */
static Int spi_mdUnBindDev(Ptr devp)
{
    Int             status = IOM_COMPLETED;
    SpiPortObj     *port   = NULL;

    /* if the devp passed by the GIO is NULL*/
    if(devp == NULL)
    {
        status = IOM_EBADARGS;
    }
    else
    {
        port = (SpiPortObj *)devp;
        /* check for port state  */
        if ((port->spiState != SPI_CREATED) && (port->spiState != SPI_CLOSED))
        {
            status =  (IOM_EBADMODE);
        }
        else
        {
            /* Calling the DDC Layer for delete call */
            status = PSP_spiDelete(port->spiPortNumber);
            if(status != PAL_SOK)
            {
                status = spi_getIomErrorCode(status);
            }
            else
            {
                /**< Updating the port object */
                port->spiState = SPI_DELETED;
            }
        }
    }

    return status;
}


/**
  * \brief  spi_mdBindDev
  *         spi_mdBindDev() is called by DEV_createDevice() to bind
  *         and Populate the driver structure element with spi
  *         configuration params.This is done by calling PSP call
  *         of spi PSP_spiCreate().and at last after population of
  *         params it will set the driver state to Created.
  *
  * \param devp         [IN]    Handle passed by the GIO Layer to
  *                             store the address of the port structure.
  * \param devid        [IN]    SPI device or instance id.
  * \param devParams    [IN]    SPI configuration params handle
  *                             which are populated to driver structure.
  * \returns                    IOM_COMPLETED if succcess<br>
  *                             IOM_EBADARGS error code
 */
static Int spi_mdBindDev(Ptr *devp, Int devid, Ptr devParams)
{
    Int             status     = IOM_COMPLETED;
    PSP_SpiConfig  *params     = NULL;
    Uint32          ldevid     = (Uint32)devid;

    if(((devParams == NULL)     ||
        (devp == NULL))         ||
        ((Uns)ldevid > (PSP_SPI_NUM_INSTANCES - 1u)))
    {
         status = IOM_EBADARGS;
    }
    else
    {
        params = (PSP_SpiConfig *)devParams;
        /**< Validate the Mode of operation of SPI  */
        if (( PSP_OPMODE_DMAINTERRUPT   != params->mode) &&
            ((PSP_OPMODE_INTERRUPT      != params->mode) &&
            ( PSP_OPMODE_POLLED         != params->mode)))
        {
            status = IOM_EBADMODE;
        }
    }

    if(status == IOM_COMPLETED)
    {
        /* Populate the spi driver object with spi configuration params.
         * The following function will create an state of spi
         */
        status = PSP_spiCreate((Uint32)devid,   params);

        /**<  Validate the error state */
        if(status != PAL_SOK)
        {
            status = spi_getIomErrorCode(status);
        }
        else
        {
            /**< Updating the port object */
            spi_Ports[ldevid].spiPortNumber = ldevid;
            spi_Ports[ldevid].spiState        = SPI_CREATED;

            /**< Updating GIO Pointer */
             *devp = (SpiPortObj *)&spi_Ports[ldevid];
        }
    }

    return status;

}



/**
  * \brief  spi_mdCreateChan Creates the channel for the data transfer
  *         and also configures the SPI hardware params.This will call
  *         the psp function call PSP_spiopen().
  *
  * \param  chanp       [IN]    channel handler.
  * \param  devp        [IN]    Device handle for spi.
  * \param  name        [IN]    Name of the device channel.
  * \param  mode        [IN]    Mode of spi operation.
  * \param  chanParams  [IN]    Channel params to check no. of opens.
  * \param  cbFxn       [IN]    Callback function.
  * \param  cbArg       [IN]    callback argument which supposed to be return
  *                             to user.
  * \returns      IOM_COMPLETED if success<br>
  *               IOM_EBADARGS if failure
  */
/* ARGSUSED */
static Int spi_mdCreateChan(
                            Ptr *chanp,
                            Ptr devp,
                            String name,
                            Int mode,
                            Ptr chanParams,
                            IOM_TiomCallback cbFxn,
                            Ptr cbArg
                            )
{

    Int              status    = IOM_COMPLETED;
    SpiPortObj      *port      = NULL;
    SpiChanObj      *chan      = NULL;
    Uint32           chanCount = 0;

    /*  Checking the Input parameters and  Checking up the
     *  function pointer for the callback  */

    if( ( (devp == NULL) || (cbFxn == NULL) )
        || ( (cbArg == NULL) || (chan != NULL) ) )
    {
        status = IOM_EBADARGS;
    }
    else
    {
        port = (SpiPortHandle)devp;
        /* Check for the state */
        if (((port->spiState != SPI_CREATED) &&
             (port->spiState != SPI_CLOSED)) &&
             (port->spiState != SPI_OPENED))
        {
            status =  (IOM_EBADMODE);
        }
        else
        {
            if (((mode != IOM_INPUT) &&
                 (mode != IOM_OUTPUT)) &&
                 (mode != IOM_INOUT) )
            {
                status = IOM_EBADMODE;
            }
            else
            {
                for(chanCount = 0 ; chanCount < PSP_SPI_NUM_OPENS ; chanCount++)
                {
                    /* Checking the handles of the channels i.e. which
                     * channel is free */
                    if(port->chans[chanCount].port == NULL)
                    {
                        chan = &port->chans[chanCount];
                        break;
                    }
                }

				if(chanCount == PSP_SPI_NUM_OPENS)
				{
					status = IOM_EINUSE;
				}
				else
				{
					if (( ( mode != IOM_INPUT ) && ( mode != IOM_OUTPUT)) && 
					                                        (mode != IOM_INOUT))
					{
						status = IOM_EBADMODE;
					}
					else
					{
						/* Updating the channel mode */
						chan->mode = (Uint32)mode;
					}
				}

            }
        }
    }

    if(status == IOM_COMPLETED)
    {
        /* Initialize DDC
         * The following function will create a channel for an instance
         * of spi & initialize the SPI Hardware
         */
        chan->ddc_Handle = PSP_spiOpen(
                                       port->spiPortNumber,
                                       NULL,
                                       NULL
                                      );
        /* TO DO look into if the value is to be checked
         * against NULL or less than 0 */

        if(chan->ddc_Handle == NULL )
        {
            status = IOM_EBADARGS;
        }
        else
        {
            /* Updating port and chan object */
            port->spiState = SPI_OPENED;

            chan->cbFxn = cbFxn;
            chan->cbArg = cbArg;
            chan->mode  = mode;
            /* sets for back referencing */
            chan->port = (struct SpiPortObj_t *)port;
            *chanp = chan;
        }
    }
    return status;
}


/**
  * \brief spi_mdDeleteChan Deletes an instance of the SPI channel.
  *        All I/O jobs must be completed prior to calling spi_mdDelete().
  *
  * \param chanp        [IN]    SPI Driver instance handler.
  *
  * \returns     IOM Completed if success <br>
  *              IOM error codes if failure<br>
  */
static Int spi_mdDeleteChan(Ptr chanp)
{
    Int              status    = IOM_COMPLETED;
    SpiChanObj      *chan      = NULL;
    SpiPortObj      *port      = NULL;
    Uint32           chanCount = 0;

    /**< Validate the Instance handler  */
    if(chanp == NULL)
    {
        status = IOM_EBADARGS;
    }
    else
    {
         /**< setting up channel and port handler*/
        chan = (SpiChanHandle)chanp;
        if( chan->port == NULL)
        {
            status = (IOM_EBADMODE);
        }
        else
        {
            port = (SpiPortObj*)chan->port;
            /**< Validate the port and driver state */
            if((port != NULL) && (port->spiState != SPI_OPENED))
            {
                status = (IOM_EBADARGS);
            }
        }
    }

    if(status == IOM_COMPLETED)
    {
        /* De-initialise DDC
         * The following function will deletes a channel for an instance
         * of spi.
         */
        status = PSP_spiClose(chan->ddc_Handle);

        /**< Validate Error  */
        if(status != PAL_SOK)
        {
            status = spi_getIomErrorCode(status);
        }
        else
        {
            chan->port = NULL;
            /**< Check if all the channels are closed */
            for(chanCount = 0;chanCount < PSP_SPI_NUM_OPENS ; chanCount++)
            {
                if(port->chans[chanCount].port != NULL)
                {
                    break;
                }
            }
            /**< If all the channels are closed then update the
              *   state as CLOSED */
            if (chanCount == PSP_SPI_NUM_OPENS)
            {
                port->spiState = SPI_CLOSED;
            }
        }
    }
    return status;
}

/**
  * \brief spi_mdControlChan The Mini driver ctrl function.
  *  All for dynamic configurations. Currently it is only
  *  supported for loopback & thats only for testing.
  *
  * \param  chanp        [IN]    SPI Instance Handler.
  * \param  cmd          [IN]    Command to do the driver.
  * \param  arg          [IN]    Command argument.
  *
  * \returns             IOM_COMPLETED if success<br>
  *                      IOM_EBADARGS error code if failure<br>
  */
static Int spi_mdControlChan(Ptr chanp, Uns cmd, Ptr arg)
{
    Int             status = IOM_COMPLETED;
    SpiChanObj     *chan   = NULL;
    SpiPortObj     *port   = NULL;

    /* Input params checked */
    if(chanp == NULL)
    {
        status = IOM_EBADARGS;
    }
    else
    {
        chan = (SpiChanHandle )chanp;
        /* check for port state */
        if( chan->port == NULL ){
            status = IOM_EBADARGS;
        }
        else
        {
            port = (SpiPortObj* )chan->port;
            /* State of the device checked here */
            if( (port->spiState != SPI_OPENED))
            {
                status = IOM_EBADARGS;
            }
            else
            {
                /* Call to DDC
                  * The following function will perform dynamic configurations
                * as specified by user
                 */
                status = PSP_spiIoctl(
                                      chan->ddc_Handle,
                                      (PSP_SpiIoctlCmd)cmd,
                                      arg,
                                      NULL
                                     );

                /* Validate Error  */
                if(status != PAL_SOK)
                {
                    status = spi_getIomErrorCode(status);
                }

            }/* Checking of state */

        }/* If port is NULL */

    }/* If the GIO Pointer is NULL */
    return status;
}




/**
  * \brief spi_mdSubmitChan The main entry point to the mini driver
  *        for read / write operations. This will call the PSP read
  *        Write function( PSP_spiTransfer() & PSP_spiTranscieve()
  *        mdSubmitChan is called by GIO_read/GIO_write or GIO_submit.
  *
  * \param  chanp       [IN]    SPi driver object Handler.
  * \param  packet      [IN]    Data related info. like buffer
  *                             address etc.
  *
  * \returns     IOM_COMPLETED if success<br>
  *              IOM_EBADARGS  if error<br>
  */
static Int spi_mdSubmitChan(Ptr chanp, IOM_Packet *packet)
{
    Int             status          = IOM_COMPLETED;
    SpiChanObj     *chan            = NULL;
    PSP_spiDataParam   *dataparam       = NULL;
    SpiPortObj     *port            = NULL;

    /** Validating Input Parameters & driver handler */
    if((chanp == NULL)||(packet == NULL))
    {
        status = (IOM_EBADARGS);
    }
    else
    {
        chan = (SpiChanHandle)chanp;

        if(chan->port == NULL)
        {
            status = (IOM_EBADARGS);
        }
        /* validate channel and mode */
        else if( (chan->mode & IOM_INOUT) == 0 )
        {
            status = IOM_EBADMODE;
        }
        else
        {
            port = (SpiPortObj* )chan->port;

            /**< Validate the packet send by the user */
            if( (packet->cmd == IOM_READ) || (packet->cmd == IOM_WRITE) )
            {
                if ((packet->addr == NULL) || (packet->size <= 0 ))
                {
                    status =  (IOM_EBADARGS);
                }
                else
                {
                    dataparam = (PSP_spiDataParam *)packet->addr;
                }
            }

            if(status == IOM_COMPLETED)
            {
                /**< Validating State */
                if(port->spiState != SPI_OPENED)
                {
                    status = IOM_EBADMODE;
                }
                else
                {
#ifdef SPI_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT( &DVTEvent_Log,
                       "SPI",
                       PSP_DVT_DESC(   PSP_DVT_eFUNC_START,
                                       PSP_DVT_dCHANNEL_ID_INST,
                                       PSP_DVT_dPACKET_ID,
                                       PSP_DVT_dSIZE_BYTES),
                       CHANNELID_INSTANCEID(
                           1,
                           chan->port->spiPortNumber),
                       dataparam->spiTrans.outBuffer,
                       dataparam->spiTrans.bufLen
                     );
#endif /*SPI_INSTRUMENTATION_ENABLED */

                    switch(packet->cmd)
                    {
                        case IOM_READ:
                            break;

                        case IOM_WRITE:
                            break;

                        case IOM_FLUSH:
                            /* not supported by the driver */
                            status = IOM_ENOTIMPL;
                            break;

                        case IOM_ABORT:
                            /* not supported by the driver */
                            status = IOM_ENOTIMPL;
                            break;

                        default:
                            status = IOM_EBADARGS;
                            break;
                    }

                    /* If the command is not READ or WRITE os some error */
                    if(status == IOM_COMPLETED)
                    {
                       /* Call to DDC
                        * The following function will do Read/Write
                        * Operation to be performed as requested by user
                        */
                        status =  PSP_spiTransceive(
                                      chan->ddc_Handle,
                                      dataparam->spiTrans.outBuffer,
                                      dataparam->spiTrans.inBuffer,
                                      dataparam->spiTrans.bufLen,
                                      dataparam->spiTrans.chipSelect,
                                      dataparam->spiTrans.dataFormat,
                                      dataparam->spiTrans.flags,
                                      dataparam->timeout,
                                      dataparam->spiTrans.param
                                     );

                    }/* If the command is not READ or WRITE or some error */
                     /* If the operation is successful */

                    if(status != PSP_SOK)
                    {
                        status = spi_getIomErrorCode(status);
                    }
#ifdef SPI_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT( &DVTEvent_Log,
                       "SPI",
                       PSP_DVT_DESC(   PSP_DVT_eFUNC_END,
                                       PSP_DVT_dCHANNEL_ID_INST,
                                       PSP_DVT_dPACKET_ID,
                                       PSP_DVT_dSIZE_BYTES),
                       CHANNELID_INSTANCEID(
                           1,
                           chan->port->spiPortNumber),
                       dataparam->spiTrans.outBuffer,
                       0
                     );
#endif /*SPI_INSTRUMENTATION_ENABLED */

                }/* If the state of the SPI is OPENED*/

            }/* If the packet is appropriate */
        }
    }/* if the input parameters are correct */
    return status;
}


/**
 *   \brief    PSP to IOM error code mapping
 *
 *   This function is used to map PSP error codes into appropriate IOM
 *   error codes.
 *
 *   \param    pspErrCode    [IN]    DDC layer error code
 *
 *   \return   IOM Error code
 */
static Int32 spi_getIomErrorCode(PSP_Result pspErrCode)
{
    /*------------------------------------------------------------------
                            IOM Error code look up table
     -------------------------------------------------------------------
     * The following mapping shows the mapping done from PSP_Result to
     * IOM error code. This will be used in scenario where IOMini driver
     * interfaces with the DDC (PSP component)
     *
     * PSP_SOK                     (0)   IOM_COMPLETED            0
     * PSP_SINPROGRESS             (1)   IOM_PENDING              1
     * PSP_E_DRIVER_INIT           (-1)  IOM_ETIMEOUTUNREC      -11
     * PSP_E_NO_MEMORY             (-2)  IOM_EALLOC              -5
     * PSP_E_RESOURCES             (-3)  IOM_EALLOC              -5
     * PSP_E_INVAL_STATE           (-4)  IOM_EBADMODE            -7
     * PSP_E_INVAL_PARAM           (-5)  IOM_EBADARGS           -10
     * PSP_E_NOT_SUPPORTED         (-6)  IOM_ENOTIMPL            -9
     * PSP_E_IO_CANCEL_FAIL        (-7)  IOM_ETIMEOUT            -2
     * PSP_E_FIFO_NOT_ENABLED      (-8)  IOM_EBADMODE            -7
     * PSP_E_INVALID_MODE          (-9)  IOM_EBADMODE            -7
     * PSP_E_INVALID_CMD           (-10) IOM_EBADARGS           -10
     *
     */
    static Int32 iomErrorCodeList[12] =
    {
        IOM_EBADARGS, /* 0 */
        IOM_EBADMODE, /* 1 */
        IOM_EBADMODE, /* 2 */
        IOM_ETIMEOUT, /* 3 */
        IOM_ENOTIMPL, /* 4 */
        IOM_EBADARGS, /* 5 */
        IOM_EBADMODE, /* 6 */
        IOM_EALLOC,   /* 7 */
        IOM_EALLOC,   /* 8 */
        IOM_ETIMEOUTUNREC,/* 9 */
        IOM_COMPLETED,/* 10 */
        IOM_PENDING,  /* 11 */
    };

    Int32 iomCode = pspErrCode + 10;

    if( (iomCode >=0) && (iomCode < 12 ) )
    {
        iomCode = iomErrorCodeList[iomCode];
    }
    else
    {
        iomCode = pspErrCode;
    }

    return(iomCode);
}

