/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 2006-2010     Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/


/**
 *  \file   ddc_spi.c
 *
 *  \brief  SPI implementation
 *
 *  This file contains the implementation of the SPI driver to utilize the
 *  services of the SPI device driver.
 *
 *  (C) Copyright 2005, Texas Instruments, Inc
 *
 *  \author     PSG
 *  \version  1.x - previous implementation            Shailesh Kumar Sharma<br>
 *            2.0 - Modified as per DM648              Shailesh Kumar Sharma<br>
 *            2.1 - Implemented transceive operation;
 *                   removed flags for read and write  Nagarjuna Kristam<br>
 *            2.2 - Arranged datastruture members
 *                    and clean up effort              Nagarjuna Kristam<br>
 *            2.3 - Implemented review comments        Nagarjuna Kristam<br>
 *            2.4 - Fixed MR 8363
 *            2.5- Updated as per review coments       Nagarjuna Kristam<br>
 *                 REVW00004866
 */


#include <ti/sdo/pspdrivers/pal_os/pal_osCache.h>
#include <ti/sdo/pspdrivers/pal_os/pal_ostime.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osProtect.h>
#include <ti/sdo/pspdrivers/pal_os/pal_oswait.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osmem.h>
#include <ti/sdo/pspdrivers/drivers/spi/psp_spi.h>
#include <ti/sdo/pspdrivers/drivers/spi/src/ddc_spi.h>
#include <ti/sdo/pspdrivers/drivers/spi/src/psp_spiApi.h>

#ifdef PSP_SPI_EDMA_SUPPORT
#include <ti/sdo/edma3/drv/edma3_drv.h>
#endif /* for EDMA support */

#ifdef SPI_INSTRUMENTATION_ENABLED
    #include <ti/sdo/pspdrivers/pal_os/bios/psp_log.h>
#endif /*SPI_INSTRUMENTATION_ENABLED*/

/**
  * macro to print error messages
  */
#ifdef SPI_DEBUG_PRINTF
#include <stdio.h>
#define     SPI_DEBUG    printf
#endif /* for printf */
/**
  * It shows spi supports maximum value of data format register.
  */
#define SPI_DATAFORMAT_VALUE        4u

/**
  * Max chip select numbers
  */
#define SPI_MAX_CHIPSELECT          3u

/**
  * Local Buffer data allocation for transcieve operation.
  */
#define SPI_BUFFER_DATA_SIZE        32u
/**
  * SPI Interrupt register mask
  */
#define SPI_INTERRUPT_MASK            0x000001FFu
/**
  * SPI CSHOLD MASK 8 bit 
  */
#define SPI_SPIDAT1_CSHOLD_8BIT_MASK    0x10u


/**
  *  Global variables for spi object and its driver object structure.
  */
static DDC_SpiObject ddcSPIObj[PSP_SPI_NUM_INSTANCES] = DDC_SPI_OBJECT_DEFAULT ;

/**
  * \brief External variables for spi driver object and to access the multiple
  *        instances  with multiple opens.
  */
static DDC_SpiDriverObject ddcSPIDriverObj[PSP_SPI_NUM_INSTANCES][PSP_SPI_NUM_OPENS];


/*------------------------Global extern functions--------------------------*/


/**
  * \brief Global variable used as local buffer for transcieve operation
  *        It will return data to application in transceieve call
  *        and it will dump the data for other call.
  */
static Uint8 spiTransReceive[SPI_BUFFER_DATA_SIZE];


/* Extern global functions */
extern void _DDA_spiRegisterIntrHandler(
                                 Uint32     instNum,
                                 Uint32     intNum,
                                 pspSpiIsr  initIsr,
                                 PSP_Handle spiObj
                                );



extern void _DDA_spiUnregisterIntrHandler(Uint32 instNum, Uint32 intNum);

/*-----------------------Static Functions ----------------------------*/
static Int spiSetupConfig(const DDC_SpiDriverObject *handle);

static void configurespiOpMode(const DDC_SpiDriverObject * Spihandle);

static void PSP_spiWaitForStaleData(void);

static Uint32 spiControlDataConfig(
                                    PSP_Handle        handle,
                                    Uint32            chipSelect,
                                    PSP_spiDataFormat dataFormat,
                                    Uint32            flags
                                  );

static Int DDC_spiTransfer(
                           PSP_Handle handle,
                           Uint8             *outBuffer,
                           Uint8             *inBuffer,
                           Uint32            bufLen,
                           Uint32            chipSelect,
                           PSP_spiDataFormat dataFormat,
                           Uint32            flags,
                           Bool              transflags,
                           Int               timeout,
                           Ptr               param,
                           Ptr               xferActual
                           );


#ifdef PSP_SPI_EDMA_SUPPORT
static Int start_edma_transfer(PSP_Handle handle,
                               Uint32            chipSelect,
                               PSP_spiDataFormat dataFormat,
                               Uint32            flags
                              );


static void callbackReceive(Uint tcc, EDMA3_RM_TccStatus status, Ptr appData);

static void callbackTransmit(Uint tcc, EDMA3_RM_TccStatus status, Ptr appData);

#endif /* for EDMA support */

/**
  * \brief  Validate the all spi configuration params, it populates the
  *         SPI configuration params to Spi object and will update the
  *         state of driver to created for respective instance id.
  *
  * \param  instanceId  [IN]    Instance id for respective spi module.
  * \param  initConfig  [IN]    SPI configuration params which are supposed to
  *                             be populated to spiobj after validation.
  *
  * \returns   PSP_SOK if success<br>
  *            Suitable error code.
  */
PSP_Result  PSP_spiCreate(Uint32 instanceId, const PSP_SpiConfig *initConfig)
{
    DDC_SpiObject           *spiObj    = NULL;
    Int                      i         = 0 ;
    PSP_Result               retVal    = PSP_SOK;
    PAL_OsSemAttrs           semAttrs  = PAL_OSSEM_ATTRS;
    Uint32                   chanCount = 0;


    /**< Validate instance ID     */
    if ((instanceId >= PSP_SPI_NUM_INSTANCES ) || ( NULL == initConfig))
    {
        retVal = PSP_E_INVAL_PARAM;
    }
    else
    {
        /* Handle of the ddc spi obj with reference to instance id     */
        spiObj = &ddcSPIObj[instanceId];
        semAttrs.type = PAL_OSSEM_TYPE_FIFO;

        if ( spiObj == NULL)
        {
            retVal = PSP_E_INVAL_PARAM;
        }
        else
        {
            if (spiObj->state != SPI_DELETED)
            {
                retVal = PSP_E_INVAL_STATE;
            }
            else
            {
                if ( ((initConfig->mode != PSP_OPMODE_POLLED) &&
                      (initConfig->mode != PSP_OPMODE_INTERRUPT))
#ifdef PSP_SPI_EDMA_SUPPORT
                      && (initConfig->mode != PSP_OPMODE_DMAINTERRUPT)
#endif /* For EDMA */
                   )
                {
                        retVal = PSP_E_INVAL_PARAM;
                }
                else
                {
                    if (((initConfig->loopBack != TRUE) &&
                               (initConfig->loopBack != FALSE)))
                    {
                        retVal = PSP_E_INVAL_PARAM;
                    }
                }
            }
        }
    }

    if (retVal == PSP_SOK)
    {
        /** mAKE SPI DDC object handle NULL */
        for (chanCount=0;chanCount<PSP_SPI_NUM_OPENS;chanCount++)
        {
            /* setting Global driver object initial value to zero */
            PAL_osMemSet(&ddcSPIDriverObj[instanceId][chanCount],
                         0,
                         sizeof(DDC_SpiDriverObject));
        }

        /* Populate the init parameters  to spi diver object */
        spiObj->mode                = initConfig->mode;
        spiObj->loopBack            = initConfig->loopBack;
        spiObj->moduleInputClkFreq  = initConfig->moduleInputClkFreq;
        spiObj->spiBusFreq          = initConfig->outputClkFreq;
        spiObj->spiHWconfig         = initConfig->spiHWCfgData;
        spiObj->edmaSpiHandle       = initConfig->edmaHandle;
        spiObj->instanceId          = instanceId;
        spiObj->charLength16Bits    = FALSE;
        spiObj->dmaChaAllocated     = FALSE;
        spiObj->csHighPolarity      = FALSE;

        /**< Validate the char length to min and max. boundary */
        for(i = 0 ; i < SPI_DATAFORMAT_VALUE ; i++)
        {
            if ((spiObj->spiHWconfig.configDatafmt[i].charLength < 2u)     ||
                (spiObj->spiHWconfig.configDatafmt[i].charLength > 16u))
            {
                retVal = PSP_E_INVAL_PARAM;
            }
        }

        if (retVal == PSP_SOK)
        {

            /**< Validate the spi mode of operation  i.e whether it is master
             *   mode or slave mode. kalish is going to work in Master mode
             *   only
             */
            if((spiObj->spiHWconfig.masterOrSlave != PSP_SPI_TYPE_MASTER )     &&
               (spiObj->spiHWconfig.masterOrSlave != PSP_SPI_TYPE_SLAVE))
            {
                retVal = PSP_E_INVAL_PARAM;
            }
            else
            {
               /**< Validating the edma handle & module frequency */

               if((spiObj->moduleInputClkFreq == 0 )||
                   (spiObj->spiBusFreq == 0))
               {
                    retVal = PSP_E_INVAL_PARAM;
               }
            }
        }

        if (retVal == PSP_SOK)
        {
            /* Create internal semaphores */
            if (PAL_osSemCreate("SPI Dev Busy Sem",
                                 1, &semAttrs,
                                 &spiObj->devBusySem) != PAL_SOK)
            {
                retVal = PSP_E_RESOURCES;
#ifdef SPI_DEBUG_PRINTF
                SPI_DEBUG("\n SpiCreate [%l]:",instanceId);
                SPI_DEBUG("Error creating device busy semaphore");
#endif /* for printf */
            }
            else
            {
				if (PAL_osSemCreate("SPI Completion Sem", 0,
										  &semAttrs, &spiObj->completionSem)
										  != PAL_SOK)
				{

					PAL_osSemDelete(spiObj->devBusySem);
					retVal = PSP_E_RESOURCES;
	#ifdef SPI_DEBUG_PRINTF
					SPI_DEBUG("\n SpiCreate [%l]:",instanceId);
					SPI_DEBUG("Error creating device busy semaphore");
	#endif /* for printf */
				}
		    }
        }
    }/* if retVal */

    if (retVal == PSP_SOK)
    {
        /* set state to created  if validation and object population is done */
        spiObj->numOpens = 0;
        spiObj->state    = SPI_CREATED;
        spiObj->instanceId     = (Int)instanceId;
    }
    return retVal;
}


/**
  * \brief  Delete a given SPI Driver (instance)
  *           The instance of SPI is deleted in software.If there are open handles
  *           then they are invalidated.
  *
  * \param  instanceId  [IN]    SPI instance number
  *
  * \return     PSP_SOK <br>
  *             PSP Error code
  */
PSP_Result PSP_spiDelete(Uint32 instanceId)
{
    DDC_SpiObject    *spiObj;
    PSP_Result        retVal = PSP_SOK;

    /* Validate instance ID */
    if (instanceId >= PSP_SPI_NUM_INSTANCES)
    {
        retVal = PSP_E_INVAL_PARAM;
    }
    else
    {
        spiObj = &ddcSPIObj[instanceId];
        if (spiObj == NULL)
        {
            retVal = PSP_E_INVAL_PARAM;
        }
        else
        {
            if ((spiObj->state != SPI_CREATED)
                 && ((spiObj->state != SPI_CLOSED)))
            {
                retVal = PSP_E_INVAL_STATE;
            }

            else
            {
                /* Delete driver semaphores */
                if (PAL_osSemDelete(spiObj->devBusySem) != PAL_SOK)
                {
                    retVal = (PSP_E_RESOURCES);
                }

                if (PAL_osSemDelete(spiObj->completionSem) != PAL_SOK)
                {
                    retVal = (PSP_E_RESOURCES);
                }
            }
        }
    } /* Outer else loop */

    if (retVal == PSP_SOK)
    {
        /* set state to deleted */
        spiObj->numOpens     = 0;
        spiObj->state        = SPI_DELETED;
    }
    return retVal;
}



/**
  *\brief Open Instance of the SPI device
  *
  *       This function prepares the device hardware for data transfers and
  *       usage by the upper layer driver software. The DDA layer is expected
  *       to install the ISR and enable it only after the completion of this
  *       call. The function prepares the driver instance for data transfers
  *       and returns an handle to the driver instance. It also configure the
  *       channel specific SPI hardware to start the transfer.
  *
  * \note   The driver can be opened multiple times
  *
  * \param  instanceId   [IN]    SPI instance number
  * \param  callback     [IN]    Callback function for  events notification
  * \param  appHandle    [IN]    This appHandle will be provided by user and will
  *                             be returned along with call back
  *
  * \return PSP_Handle(valid handle) if successful<br>
  *         NULL       if error<br>
  */
/* ARGSUSED */
PSP_Handle    PSP_spiOpen(
                          Uint32                    instanceId,
                          PSP_SpiAppCallback        callback,
                          Ptr                       appHandle
                         )
{

    DDC_SpiObject           *spiObj;
    DDC_SpiDriverObject     *spiHandle = NULL;
    Uint32                   cnt       = 0;
    Uint32                  cookie     = 0;

    /**<  Validate instance ID & data format */
    if (instanceId  < PSP_SPI_NUM_INSTANCES)
    {
        spiObj = &ddcSPIObj[instanceId];
        if ((spiObj != NULL) && (spiObj->state != SPI_DELETED))
        {
            /**< Create spi driver object */
            for (cnt=0; cnt < PSP_SPI_NUM_OPENS; cnt++)
            {
                if (ddcSPIDriverObj[spiObj->instanceId][cnt].spiObj == NULL)
                {
                    ddcSPIDriverObj[spiObj->instanceId][cnt].spiObj    = spiObj;
                    spiHandle = &ddcSPIDriverObj[spiObj->instanceId][cnt];
                    break;
                }
            } /* For loop */
        } /* NULL and state condition */
    }/* instance id validation */

    /* Update the hard-ware if its first open */
    if ((0 == spiObj->numOpens) && (spiHandle != NULL))
    {
        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

        /**< This driver hardware supports only one instance */
         spiObj->spiRegs = (CSL_SpiRegs * ) CSL_SPI_0_REGS;

         if( spiObj->spiHWconfig.intrLevel == FALSE)
         {
            spiObj->intNum =  CSL_INTC_EVENTID_SPIINT0;
         }
         else
         {
            spiObj->intNum =  CSL_INTC_EVENTID_SPIINT1;
         }
#ifdef PSP_SPI_EDMA_SUPPORT
        spiObj->chanEdmaTx = CSL_EDMA3_CHA_SPIXEVT;
        spiObj->chanEdmaRx = CSL_EDMA3_CHA_SPIREVT;
#endif /* for EDMA support */

        /**< If interrupt mode, register ISR */
        if (spiObj->mode != PSP_OPMODE_POLLED)
        {
            _DDA_spiRegisterIntrHandler(instanceId,
                                       spiObj->intNum,
                                       (pspSpiIsr)&DDC_spiIntrHandler,
                                       spiObj
                                       );
        }
#ifdef PSP_SPI_EDMA_SUPPORT
        /**< channel request for edma Rx and Tx   */
        if (spiObj->mode == PSP_OPMODE_DMAINTERRUPT)
        {
             /**< request for RX DMA channels   */
             spiObj->dmaChaAllocated = TRUE;
             if(EDMA3_DRV_requestChannel ((spiObj->edmaSpiHandle),
                                         &(spiObj->chanEdmaRx),
                                         &(spiObj->chanEdmaRx),
                                         (EDMA3_RM_EventQueue)0,
                                         &callbackReceive,
                                         ((DDC_SpiObject *)spiObj)) != PAL_SOK)
              {
                    spiHandle        = NULL;
                    spiObj->dmaChaAllocated     = FALSE;
#ifdef SPI_DEBUG_PRINTF
                    SPI_DEBUG("\nSPI Test: EDMA Request failed");
#endif /* for printf */
               }
               else
               {
                    /**< request for TX DMA channels   */
                    if(EDMA3_DRV_requestChannel ((spiObj->edmaSpiHandle),
                                                 &(spiObj->chanEdmaTx),
                                                 &(spiObj->chanEdmaTx),
                                                (EDMA3_RM_EventQueue)0,
                                                &callbackTransmit,
                                                ((DDC_SpiObject *)spiObj))
                                                != PAL_SOK)
                    {
                        EDMA3_DRV_freeChannel(spiObj->edmaSpiHandle,
                                              spiObj->chanEdmaRx);
                        spiHandle        = NULL;
                        spiObj->dmaChaAllocated     = FALSE;
#ifdef SPI_DEBUG_PRINTF
                        SPI_DEBUG("\nSPI Test: EDMA Request failed");
#endif /* for printf */
                    }
                }
            }/**< DMA mode channel request */
#endif /* for EDMA support */
          /**<  Initialize hardware */
        if(PAL_SOK != spiSetupConfig(spiHandle) )
        {
            PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
            spiHandle = NULL;
        }
    }
    if (NULL != spiHandle)
    {
        /**< Increment open count and return driver handle */
        ++spiObj->numOpens;
        spiObj->state         = SPI_OPENED;
        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
    }

    return (PSP_Handle)spiHandle;
}



/**
  *\brief Close Instance of the SPI device
  *
  *       This function closes the device for data transfers and usage by the
  *       upper layer driver software. The hardware is programmed to stop/abort
  *       data transfer (depending upon the type of device and its specifics)
  *       and the device ISR is "disabled" by the upper layer driver software
  *       after the completion of this call. After the successful completion of
  *       this call, the device cannot perform any data IO.
  *
  * \param   handle      [IN]    SPI Driver Instance Handle
  *
  * \return  PSP_SOK <br>
  *          PSP Error code
  */
PSP_Result PSP_spiClose(PSP_Handle handle)
{
    Uint32              cookie ;
    DDC_SpiDriverObject *spiHandle;
    DDC_SpiObject       *spiObj     = NULL;
    PSP_Result          retVal      = PSP_SOK;

    if (handle == NULL)
    {
        retVal = PSP_E_INVAL_PARAM;
    }
    else
    {
        spiHandle     = (DDC_SpiDriverObject *) handle ;
        spiObj        =  spiHandle->spiObj;

        /**< Validate the driver and spi object handle     */
        if((NULL == spiObj) || (spiObj->state != SPI_OPENED))
        {
            retVal = PSP_E_INVAL_STATE;
        }
    }


    if (retVal == PSP_SOK)
    {
        /*  Check for any IO is pending, if yes then cancel io */
        if ( spiObj->pendingState == TRUE )
        {
            PSP_spiIoctl(handle,PSP_SPI_IOCTL_CANCEL_PENDING_IO,NULL,NULL);
        }
        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);
        --spiObj->numOpens;
        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

        if (spiObj->numOpens == 0)
        {
            /**< If interrupt mode, un register ISR */
            if (spiObj->mode != PSP_OPMODE_POLLED)
            {
                _DDA_spiUnregisterIntrHandler(
                                              (Uint32)spiObj->instanceId,
                                              spiObj->intNum
                                             );
            }

#ifdef PSP_SPI_EDMA_SUPPORT
            /**< Free the Edma Channels  if mode is DMA */
            if(spiObj->mode == PSP_OPMODE_DMAINTERRUPT)
            {
                /**< This will verify whether channel is allocated or not */
                if(TRUE == spiObj->dmaChaAllocated )
                {
                    EDMA3_DRV_freeChannel(
                                          spiObj->edmaSpiHandle,
                                          spiObj->chanEdmaRx
                                         );

                    EDMA3_DRV_freeChannel(
                                          spiObj->edmaSpiHandle,
                                          spiObj->chanEdmaTx
                                         );
                }
                else
                {
                    /**< Dma channel is not allocated so through error */
                    retVal = PSP_E_INVAL_STATE;
                }
            }
#endif /* for EDMA support */
             /**< Disable the spi enable pin */
            spiObj->spiRegs->SPIGCR1    &= (~(CSL_SPI_SPIGCR1_SPIEN_MASK));
            /**<Put the device in reset mode and quit*/
            spiObj->spiRegs->SPIGCR0    = CSL_SPI_SPIGCR0_RESETVAL ;
            spiObj->state       = SPI_CLOSED;
        }
        spiHandle->spiObj   = NULL;
        spiHandle           = NULL;
    }
    return retVal;
}


/**
  * \brief SPI transceieve operation means Read\Write will happen
  *           parellaly.
  *
  * \param  handle      [IN]     SPI Driver instance handle.
  * \param  outBuffer   [IN]     User buffer for data Source.
  * \param  inBuffer    [IN]     User or local buffer source for transceieve
  *                          operation.( User or local buffer selection
  *                          depends on transflags status ).
  * \param  bufLen      [IN]     Length of data to be transmitted
  * \param  chipSelect  [IN]     Chip select number
  * \param  dataFormat  [IN]     Data format register selection
  * \param  flags       [IN]     Flag for Read/Write of data
  * \param  timeout     [IN]     Timeout value.
  * \param  param       [IN/OUT] Additional param for future use.
  *
  * \return   Success code or suitable error code.
  *
  */

PSP_Result PSP_spiTransceive(
                        PSP_Handle        handle,
                        Uint8             *outBuffer,
                        Uint8             *inBuffer,
                        Uint32            bufLen,
                        Uint32            chipSelect,
                        PSP_spiDataFormat dataFormat,
                        Uint32            flags,
                        Int               timeout,
                        Ptr               param
                     )
{

    DDC_SpiDriverObject     *spiHandle      = NULL;
    DDC_SpiObject           *spiObj         = NULL;
    Uint32                   cnt            = 0;
    PSP_Result               retVal         = PSP_SOK;
    Bool                     transFlags      = FALSE;

    if ((NULL == handle) || (NULL == outBuffer))
    {
        retVal = PSP_E_INVAL_PARAM;
    }
    else
    {
        spiHandle = (DDC_SpiDriverObject *)handle;
        spiObj =  spiHandle->spiObj;

        if ((bufLen == 0) || (NULL == spiObj))
        {
            retVal = PSP_E_INVAL_PARAM;
        }
        else
        {

#ifdef PSP_SPI_EDMA_SUPPORT
            /**< Validate the buffer alignment if mode is DMA  */
            if(spiObj->mode == PSP_OPMODE_DMAINTERRUPT)
            {
               if ((0 != (((Uint32)outBuffer) % 32         ))     ||
                   (0 != (((Uint32)inBuffer) % 32   )))
               {
                   retVal = PSP_E_INVAL_PARAM;
               }
            }
#endif /* for EDMA support */
            /** check for zero '0' timeout 7508*/
            if ((0 == timeout) && (PSP_SOK == retVal))
            {
                retVal = PSP_E_TIMEOUT;
            }

            if (PSP_SOK == retVal)
            {
                retVal = PAL_osSemTake(spiObj->devBusySem,
                        PAL_OsGetTicksPerMSecs((Int)timeout));
            }

        }/* second else loop */
    }    /* first else loop */

    if(PAL_SOK == retVal)
    {
        /**< Update the transceieve handle with source buffer */
        /* Check if passed spitransbuff is NULL */
        if (NULL == inBuffer)
        {
            /**< If NULL update spiObj->transBuffer with static buffer */
            PAL_osMemSet(spiTransReceive,0,SPI_BUFFER_DATA_SIZE);

            if (spiObj->mode == PSP_OPMODE_DMAINTERRUPT)
            {
                spiObj->transBuffer = (Uint8 *)((((Uint32)&spiTransReceive[0])
                                                   + (32u - 1u)) & 0xFFFFFFE0u);
            }
            else
            {
                spiObj->transBuffer = &spiTransReceive[0];
            }

            /*
             * When user does not need transceive operations i.e if spitranbuff
             * is null update transflags as FALSE
             */
            transFlags = FALSE;
        }
        else
        {
            /**< When passed spitransbuff is not NULL */
            transFlags          = TRUE;
            spiObj->transBuffer = inBuffer;
        }

        /**< Call the DDC_spiTransfer data transfer through hardware */
        retVal =  DDC_spiTransfer (handle, outBuffer, spiObj->transBuffer,
                                   bufLen, chipSelect, dataFormat, flags,
                                   transFlags, timeout, param, (Uint32 *) &cnt);

        if (PAL_osSemGive(spiObj->devBusySem) != PAL_SOK)
        {
#ifdef SPI_DEBUG_PRINTF
            /* Print out but dont return error since spi transfer has
               been successful */
            SPI_DEBUG("\n spiTransceive [%l]:",spiObj->instanceId);
            SPI_DEBUG("Error giving device busy semaphore");
#endif /* for printf */
        }
        /**< Mask off interrupts */
        spiObj->spiRegs->SPIINT0 &= (~(SPI_INTERRUPT_MASK));
        spiObj->charLength16Bits = FALSE;
    } /**< if retVal     */
    return retVal;

}


/**
  * \brief  Interrupt handler for SPI Device
  *         It will check the following errors like bit error, desync error
  *         parity error, overrun error. Data transfer either read or write
  *         is done on RX INT flag only.
  * \param  spiObj      [IN]    Spi object hadle.
  */
void DDC_spiIntrHandler(DDC_SpiObject  * spiObj)
{
    volatile Uint32 intStatus;
    Uint32          spiData1 = 0;

    if (NULL == spiObj)
    {
        return;
    }

    spiData1 = spiObj->spiRegs->SPIDAT1 & (0xFFFF0000);

    intStatus = ((spiObj->spiRegs->SPIFLG ) & (0X000001FFu));

    while (intStatus != 0)
    {
#ifdef SPI_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(
                        &DVTEvent_Log,
                        "SPI",
                        PSP_DVT_DESC(
                                    PSP_DVT_eINT_START,
                                    PSP_DVT_dINST,
                                    PSP_DVT_dCHANNEL_ID,
                                    PSP_DVT_dNONE
                                    ),
                        spiObj->instanceId,
                        1
                     );
#endif /*SPI_INSTRUMENTATION_ENABLED */

        /**< check Timeout interrupt */
        if( (intStatus & CSL_SPI_SPIFLG_TIMEOUT_MASK) ==
                  CSL_SPI_SPIFLG_TIMEOUT_MASK )
        {
             spiObj->currError = PSP_SPI_TIMEOUT_ERR;
             spiObj->spiRegs->SPIFLG |= CSL_SPI_SPIFLG_TIMEOUT_MASK;
            if (PAL_osSemGive(spiObj->completionSem) != PAL_SOK)
            {
#ifdef SPI_DEBUG_PRINTF
                SPI_DEBUG("\n spi ISR [%l]:",spiObj->instanceId);
                SPI_DEBUG("Error giving completion semaphore in AL\n");
#endif /* for printf */
            }
        }
        else if( (intStatus &  CSL_SPI_SPIFLG_PARITYERR_MASK)
                   == CSL_SPI_SPIFLG_PARITYERR_MASK )
        {
              /**< Parity error interrupt */
              spiObj->currError = PSP_SPI_PARITY_ERR ;
              spiObj->spiRegs->SPIFLG |= CSL_SPI_SPIFLG_PARITYERR_MASK;
              if (PAL_osSemGive(spiObj->completionSem) != PAL_SOK)
              {
#ifdef SPI_DEBUG_PRINTF
                   SPI_DEBUG("\n spi ISR [%l]:",spiObj->instanceId);
                SPI_DEBUG("Error giving completion semaphore in AL\n");
#endif /* for printf */
              }

        }
        else if( (intStatus &  CSL_SPI_SPIFLG_DESYNC_MASK) ==
                 CSL_SPI_SPIFLG_DESYNC_MASK )
        {
            /**< De-Synchrinization interrupt, holds only in master mode */
            spiObj->currError = PSP_SPI_DESYNC_ERR ;
            spiObj->spiRegs->SPIFLG |= CSL_SPI_SPIFLG_DESYNC_MASK;
            if (PAL_osSemGive(spiObj->completionSem) != PAL_SOK)
            {
#ifdef SPI_DEBUG_PRINTF
                   SPI_DEBUG("\n spi ISR[%l]:",spiObj->instanceId);
                SPI_DEBUG("Error giving completion semaphore in AL\n");
#endif /* for printf */
            }
        }
        else if( (intStatus &  CSL_SPI_SPIFLG_BITERR_MASK) ==
                 CSL_SPI_SPIFLG_BITERR_MASK )
        {
            /**<  Bit error Interrupt */
            spiObj->currError = PSP_SPI_BIT_ERR ;
            spiObj->spiRegs->SPIFLG |= CSL_SPI_SPIFLG_BITERR_MASK;
            if (PAL_osSemGive(spiObj->completionSem) != PAL_SOK)
            {
#ifdef SPI_DEBUG_PRINTF
               SPI_DEBUG("\n spi ISR[%l]:",spiObj->instanceId);
               SPI_DEBUG("Error giving completion semaphore in AL\n");
#endif /* for printf */
            }
        }
        else if( (intStatus & CSL_SPI_SPIFLG_RXINTFLAG_MASK) ==
                  CSL_SPI_SPIFLG_RXINTFLAG_MASK)
        {
                /* RXINT flag is used for data recieve and Transmit becuase
                    SPI device is transceive device */
            if (spiObj->currBufferLen != FALSE)
            {
                if (spiObj->Cancel_Pending_IO == TRUE)
                {
                    /**< Disable Receive interrupt */
                    spiObj->spiRegs->SPIINT0 &= (~(CSL_SPI_SPIINT0_RXINTEN_MASK));
                    spiObj->currError = PSP_SPI_CANCEL_IO_ERROR;
                    spiObj->currBuffer     = NULL;
                    if (PAL_osSemGive(spiObj->completionSem) != PAL_SOK)
                    {
#ifdef SPI_DEBUG_PRINTF
                        SPI_DEBUG("\n spi ISR[%l]:",spiObj->instanceId);
                        SPI_DEBUG("Error giving completion semaphore in AL\n");
#endif /* for printf */
                    }
                    spiObj->charLength16Bits = FALSE;
                }
                else
                {
                    if(TRUE == (spiObj->transcieveFlags))
                    {
                        if(TRUE == spiObj->charLength16Bits)
                        {
                            /* Consume the received data */
                            (*((Uint16*)spiObj->transBuffer)) = (Uint16)(spiObj->spiRegs->SPIBUF);
                            spiObj->transBuffer += 2;
                        }
                        else
                        {
                            /* Consume the received data and increment the buffer
                             * because it is transcieve operation */
                            *spiObj->transBuffer =(Uint8)(spiObj->spiRegs->SPIBUF);
	                        spiObj->transBuffer++;
                        }
                    }
                    else
                    {
                        if(TRUE == spiObj->charLength16Bits)
                        {
                            /* Consume the received data */
                            (*((Uint16*)spiObj->transBuffer)) = (Uint16)(spiObj->spiRegs->SPIBUF);
                        }
                        else
                        {
                            /* Consume the received data & no need to increment
                             * because it is not transcieve operation */
                            *spiObj->transBuffer = (Uint8)(spiObj->spiRegs->SPIBUF);
                        }
                    }

                    --spiObj->currBufferLen;
                    if (spiObj->currBufferLen == 0)
                    {
                        /* Disable Receive interrupt */
                        spiObj->spiRegs->SPIINT0 &= (CSL_SPI_SPIINT0_RXINTEN_DISABLED
                                                 <<  CSL_SPI_SPIINT0_RXINTEN_SHIFT);
                        spiObj->currError = 0 ;
                        if (PAL_osSemGive(spiObj->completionSem) != PAL_SOK)
                        {
#ifdef SPI_DEBUG_PRINTF
                            SPI_DEBUG("\n spi ISR[%l]:",spiObj->instanceId);
                            SPI_DEBUG("Error giving completion semaphore in AL\n");
#endif /* for printf */
                        }
                        spiObj->charLength16Bits = FALSE;
                    }
                    else
                    {
                        /* Transmit user supplied data */
                        if(TRUE == spiObj->charLength16Bits)
                        {
                            spiObj->spiRegs->SPIDAT1 = spiData1 | (*((Uint16*)spiObj->currBuffer));
                            spiObj->currBuffer += 2;
                        }
                        else
                        {

                            /**< 4 -pin CS and 5 pin configuration */
                            spiObj->spiRegs->SPIDAT1 = spiData1 | (*spiObj->currBuffer);
                            spiObj->currBuffer ++;
                        }
                    }
                }
            }
            else
            {
                /* Disable Receive interrupt */
                spiObj->spiRegs->SPIINT0 &= (CSL_SPI_SPIINT0_RXINTEN_DISABLED
                                         <<  CSL_SPI_SPIINT0_RXINTEN_SHIFT);
                spiObj->currError = 0 ;
                if (PAL_osSemGive(spiObj->completionSem) != PAL_SOK)
                {
#ifdef SPI_DEBUG_PRINTF
                    SPI_DEBUG("\n spi ISR[%l]:",spiObj->instanceId);
                    SPI_DEBUG("Error giving completion semaphore in AL\n");
#endif /* for printf */
                }
                spiObj->charLength16Bits = FALSE;
                return;
            }

        }
        else
        {
            /**< Receive Over run interrupt   */
            if((intStatus & CSL_SPI_SPIFLG_RCVROVRN_MASK) ==
                         CSL_SPI_SPIFLG_RCVROVRN_MASK)
            {
                spiObj->currError = PSP_SPI_RECEIVE_OVERRUN_ERR ;
                spiObj->spiRegs->SPIFLG |= CSL_SPI_SPIFLG_RCVROVRN_MASK ;

                if (PAL_osSemGive(spiObj->completionSem) != PAL_SOK)
                {
#ifdef SPI_DEBUG_PRINTF
                    SPI_DEBUG("\n spi ISR[%l]:",spiObj->instanceId);
                    SPI_DEBUG("Error giving completion semaphore in AL\n");
#endif /* for printf */
                }
            }
        }
#ifdef SPI_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(
                        &DVTEvent_Log,
                        "SPI",
                        PSP_DVT_DESC(
                                    PSP_DVT_eINT_END,
                                    PSP_DVT_dINST,
                                    PSP_DVT_dCHANNEL_ID,
                                    PSP_DVT_dNONE
                                    ),
                        spiObj->instanceId,
                        1
                     );
#endif /*SPI_INSTRUMENTATION_ENABLED */

        intStatus = ((spiObj->spiRegs->SPIFLG ) & (0X000001FFu));
    }
}


/**
  *    \brief  Read/write multilpe transaction using spi peripheral.
  *
  *         This function read/writes multiple transactions data using the spi
  *         peripheral.Though nothing prevents the user from using multiple
  *         invokations of the single transaction transfer function, this
  *         function helps in making sure that all the transactions are
  *         completed in one go before another application gets hold on the spi
  *         bus (from the application perspective).
  *
  *    \param  handle      [IN]    SPI Driver instance handle
  *    \param  xfer        [IN]    SPI Transaction array - first element pointer
  *    \param  numXfer     [IN]    Number of transactions to be done
  *    \param  DataFormat  [IN]    DataFormat to be used
  *    \param  timeout     [IN]    Timeout for Spi transaction to complete
  *    \param  param       [IN]    Extra parameter for future use
  *
  *    \return   Number of transaction successfully completed or the Error code.
  */
/* ARGSUSED */
PSP_Result  PSP_spiTransferMultiple(
                                    PSP_Handle         handle,
                                    PSP_spiTransaction *xfer,
                                    Uint32             numXfer,
                                    PSP_spiDataFormat  DataFormat,
                                    Int                timeout,
                                    Ptr                param
                                   )
{

    DDC_SpiDriverObject     *spiHandle;
    Uint32                   loop ;
    Uint32                   cnt,i ;
    DDC_SpiObject           *spiObj  = NULL;
    PSP_Result               retVal  = PSP_SOK;


    if ((handle == NULL) || (NULL == xfer))
    {
        retVal = PSP_E_INVAL_PARAM;
    }
    else
    {
        spiHandle = (DDC_SpiDriverObject *) handle ;
        spiObj =  spiHandle->spiObj;
    }

    if ((retVal == PSP_SOK) && (NULL != spiObj))
    {
        if( numXfer == 0)
        {
            retVal = PSP_E_INVAL_PARAM;
        }
        else
        {
            /**< If dma mode then validate the buffer allingment */
            if(spiObj->mode == PSP_OPMODE_DMAINTERRUPT)
            {
                if (0 != (((Uint32)xfer->outBuffer) % 32u))
                  {
                       retVal = PSP_E_INVAL_PARAM;
                  }
            }

                retVal = PAL_osSemTake(spiObj->devBusySem,
                         PAL_OsGetTicksPerMSecs((Int) timeout));

        }
    }

    if (retVal == PSP_SOK)
    {
       /**< Initialize the data buffer foro therthan transceieve operation */

        for(i = 0; i< SPI_BUFFER_DATA_SIZE; i++)
        {
            spiTransReceive[i] = 0;
        }

        /**< Aling the buffer for transcieve operation if mode is dma */
        if (spiObj->mode == PSP_OPMODE_DMAINTERRUPT)
        {

           spiObj->transBuffer = (Uint8 *)((((Uint32)&spiTransReceive[0])
                                 + (32u - 1u)) & 0xFFFFFFE0u);
        }
        else
        {
            spiObj->transBuffer = ((Uint8 *)&spiTransReceive[0]);
        }


        for(loop = 0 ; loop < numXfer ; loop++)
        {
            retVal = DDC_spiTransfer(
                                     handle,
                                     xfer->outBuffer,
                                     xfer->inBuffer,
                                     xfer->bufLen,
                                     xfer->chipSelect,
                                     xfer->dataFormat,
                                     xfer->flags,
                                     FALSE,
                                     timeout,
                                     xfer->param,
                                     (Uint32 *) &cnt
                                    );

            /**< Returns number of failed transactions */
            if(retVal!= PSP_SOK)
            {
                retVal = (PSP_Result)numXfer - (PSP_Result)loop;
                break;
            }
            xfer = xfer + 1u;
        }

        if (PAL_osSemGive(spiObj->devBusySem) != PAL_SOK)
        {
            /* Print out but dont return error since spi transfer has been successful */
#ifdef SPI_DEBUG_PRINTF
            SPI_DEBUG("\n spi Transfer[%l]:",spiObj->instanceId);
            SPI_DEBUG("Error giving device busy semaphore");
#endif /* for printf */
        }
        /* Mask off interrupts */
        spiObj->spiRegs->SPIINT0 &= (~(SPI_INTERRUPT_MASK));
    }
    return (retVal);
}



/**
  * \brief  IO Control for SPI device. Curreently it is used for loopback only
  *
  * \param  handle      [IN]    Driver object handle.
  * \param  cmd         [IN]    Command to do operation.
  * \param  cmdArg      [IN]    Params for which operation to be done
  * \param  param       [IN]    For future reference.
  *
  * \return      PAL_SOK if success<br>
  *              Suitable error code
  */
/* ARGSUSED */
PSP_Result  PSP_spiIoctl(
                         PSP_Handle         handle,
                         PSP_SpiIoctlCmd    cmd,
                         Ptr                cmdArg,
                         Ptr                param
                        )
{
    DDC_SpiDriverObject    *spiHandle   = NULL;
    Uint32                  cookie      = 0;
    DDC_SpiObject          *spiObj      = NULL;
    PSP_Result              retVal      = PSP_SOK;

    if (handle == NULL)
    {
        retVal = PSP_E_INVAL_PARAM;
    }
    else
    {
        spiHandle = (DDC_SpiDriverObject *) handle ;
        /**< Validate the driver object handle  */
        if(NULL == spiHandle->spiObj)
        {
            retVal = PSP_E_INVAL_PARAM;
        }
        spiObj =  spiHandle->spiObj;
    }

    if (retVal == PSP_SOK)
    {
        switch (cmd)
        {
            case PSP_SPI_IOCTL_CANCEL_PENDING_IO:
                /* Start of Critical Section */
                PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);
                /* Calling this when no io is in transmission */
                if(spiObj->pendingState == TRUE)
                {
                    /* to cancel io in ddc*/
                    spiObj->Cancel_Pending_IO = TRUE;
                }
                else
                {
                    retVal = PSP_E_INVAL_STATE;
                }
                /* End of Critical Section */
                PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
                break;

            case PSP_SPI_IOCTL_SET_CS_POLARITY:
                if (NULL != cmdArg)
                {
                    spiObj->csHighPolarity = *(Bool *)cmdArg;
                }
                else
                {
                    retVal = PSP_E_INVAL_PARAM;
                }
                break;
            default:
                retVal = PSP_E_INVAL_PARAM;
                break;
        }
    }
    return (retVal);
}


/**
  * \brief It will configure the SPI hardware as per user configuration params.
  *        Before setting the Hardware it will put the SPI device in reset and
  *        again out of reset too before setting hardware.
  *        It will set the enable bit of the SPI device.
  *
  * \param  handle      [IN]    SPI driver object for respective instance.
  *
  * \return    PAL_SOK if success<br>
  *            Suitable error code
  */
static Int spiSetupConfig(const DDC_SpiDriverObject *handle)
{
    DDC_SpiObject     *obj;
    Uint16            sPIPC0   = 0;
    Uint32            prescale = 0;
    PSP_Result        retVal   = PSP_SOK;
    Int               i        = 0 ;


    if (handle == NULL)
    {
        retVal = PSP_E_INVAL_PARAM;
    }
    else
    {
        obj =  handle->spiObj;
        if (obj == NULL)
        {
            retVal = PSP_E_INVAL_PARAM;
        }
    }

    if (retVal == PSP_SOK)
    {
        /* clock is enable in master mode only */
        if(obj->spiHWconfig.masterOrSlave == PSP_SPI_TYPE_MASTER)
        {
           /* Prescale value calculation */
            prescale = ( obj->moduleInputClkFreq / obj->spiBusFreq ) - 1u ;

            /* prescalar is a 8 bit register and can hold maximum 255 */
            if ( prescale > 255u )
            {
                retVal = PSP_E_INVAL_PARAM ;
            }
        }

        if (retVal == PSP_SOK)
        {
            /*Put the module in reset mode */
            obj->spiRegs->SPIGCR0 = CSL_SPI_SPIGCR0_RESET_IN_RESET ;

            /* Bring module out of reset */
            obj->spiRegs->SPIGCR0 |= CSL_SPI_SPIGCR0_RESET_OUT_OF_RESET;

            if(obj->loopBack == TRUE)
            {
                /** Enable loopback            */
                obj->spiRegs->SPIGCR1 |= CSL_SPI_SPIGCR1_LOOPBACK_MASK;
            }
            else
            {
               /** disable loopback            */
               obj->spiRegs->SPIGCR1 &= (~(CSL_SPI_SPIGCR1_LOOPBACK_MASK));
            }
            /*Program the format registers */
            for(i = 0 ; i < SPI_DATAFORMAT_VALUE ; i++)
            {
               /* This value should be loaded only in master mode */
                if( obj->spiHWconfig.masterOrSlave == PSP_SPI_TYPE_MASTER)
                {
                     /* Program the prescalar */
                    /* clear the old value*/
                    obj->spiRegs->SPIFMT[i] &=  (~(CSL_SPI_SPIFMT_PRESCALE_MASK));
                     /*load the new value */
                    obj->spiRegs->SPIFMT[i] |=
                                  (prescale << CSL_SPI_SPIFMT_PRESCALE_SHIFT) ;
                }
                /* Insert Delay value between two consecutive words    */
                /* clear the old value */
                obj->spiRegs->SPIFMT[i] &= (~(CSL_SPI_SPIFMT_WDELAY_MASK));
                /*load the new value*/
                obj->spiRegs->SPIFMT[i] |=
                (obj->spiHWconfig.configDatafmt[i].wDelay <<
                                              CSL_SPI_SPIFMT_WDELAY_SHIFT );

                /* Defines the word length of data format    */
                /*clear the old value*/
                obj->spiRegs->SPIFMT[i] &= (~(CSL_SPI_SPIFMT_CHARLEN_MASK));
                /*load the new value*/
                obj->spiRegs->SPIFMT[i] |=
                            obj->spiHWconfig.configDatafmt[i].charLength;

                    /* configure the phase In */
                if( obj->spiHWconfig.configDatafmt[i].phaseIn != FALSE)
                {

                    obj->spiRegs->SPIFMT[i] |=  (CSL_SPI_SPIFMT_PHASE_DELAY <<
                                                 CSL_SPI_SPIFMT_PHASE_SHIFT);
                }
                else
                {
                    obj->spiRegs->SPIFMT[i] &=  (~(CSL_SPI_SPIFMT_PHASE_MASK));
                }
                    /* configure the polarity */
                if( obj->spiHWconfig.configDatafmt[i].clkHigh  != FALSE)
                {
                    obj->spiRegs->SPIFMT[i] |=
                            (CSL_SPI_SPIFMT_POLARITY_HIGH <<
                                            CSL_SPI_SPIFMT_POLARITY_SHIFT);
                }
                else
                {
                    obj->spiRegs->SPIFMT[i] &=  (~(CSL_SPI_SPIFMT_POLARITY_MASK));
                }
                  /* configure the data direction */
                if( obj->spiHWconfig.configDatafmt[i].lsbFirst != FALSE )
                {
                    obj->spiRegs->SPIFMT[i] |=
                    (CSL_SPI_SPIFMT_SHIFTDIR_LSB <<
                                        CSL_SPI_SPIFMT_SHIFTDIR_SHIFT);
                }
                else
                {
                    obj->spiRegs->SPIFMT[i] &=
                                  (~(CSL_SPI_SPIFMT_SHIFTDIR_MASK));
                }
                    /* Enable the parity */
                if( obj->spiHWconfig.configDatafmt[i].parityEnable != FALSE)
                {
                    obj->spiRegs->SPIFMT[i] |=
                          (CSL_SPI_SPIFMT_PARITY_PARITY <<
                                         CSL_SPI_SPIFMT_PARITY_SHIFT);
                     /* configure the odd parity */
                    if(obj->spiHWconfig.configDatafmt[i].oddParity != FALSE)
                    {
                        obj->spiRegs->SPIFMT[i] |=
                               (CSL_SPI_SPIFMT_PARPOL_ODD <<
                                        CSL_SPI_SPIFMT_PARPOL_SHIFT);
                    }
                    else
                    {
                     /* configure the even parity */
                        obj->spiRegs->SPIFMT[i] &=
                                       (~(CSL_SPI_SPIFMT_PARPOL_MASK));
                    }
                }
                else
                {
                    obj->spiRegs->SPIFMT[i] &= (~(CSL_SPI_SPIFMT_PARITY_MASK));
                }

                    /* configure the wait enable */
                if(((obj->spiHWconfig.pinOpModes) != PSP_SPI_OPMODE_3PIN) &&
                    (( obj->spiHWconfig.masterOrSlave) == PSP_SPI_TYPE_MASTER))
                {

                    if( obj->spiHWconfig.configDatafmt[i].waitEnable != FALSE)
                    {
                        obj->spiRegs->SPIFMT[i] |=
                           (CSL_SPI_SPIFMT_WAITENA_WAIT <<
                                                 CSL_SPI_SPIFMT_WAITENA_SHIFT);
                    }
                    else
                    {
                        obj->spiRegs->SPIFMT[i] &=
                                             (~(CSL_SPI_SPIFMT_WAITENA_MASK));
                    }
                }
            }

            /* Enable SPI for further transaction */
            obj->spiRegs->SPIGCR1 |= (CSL_SPI_SPIGCR1_SPIEN_ENABLED
                                       << CSL_SPI_SPIGCR1_SPIEN_SHIFT);

                /* Clock Internal */
            if( obj->spiHWconfig.clkInternal != FALSE)
            {
                obj->spiRegs->SPIGCR1 |=  CSL_SPI_SPIGCR1_CLKMOD_MASK;
            }
            else
            {
                 /**< configure Clock Externally  */
                obj->spiRegs->SPIGCR1 &=  (~(CSL_SPI_SPIGCR1_CLKMOD_MASK));
            }
            /* Slave mode of operation - this driver will be working on
             * Master mode only. */
            if( obj->spiHWconfig.masterOrSlave != PSP_SPI_TYPE_MASTER)
            {
                obj->spiRegs->SPIGCR1 &= (~(CSL_SPI_SPIGCR1_MASTER_MASK));
            }
            else
            {
                /* Device in master mode */
                obj->spiRegs->SPIGCR1 |=  CSL_SPI_SPIGCR1_MASTER_MASK ;
            }

            /**< Default chip select pattern used for moment when none of
              *  device is active
              */
            obj->spiRegs->SPIDEF = obj->spiHWconfig.csDefault ;

            /**< Interrupt mapped to level INT1 */
            if( obj->spiHWconfig.intrLevel != FALSE)
            {
                obj->spiRegs->SPILVL  = 0x000001FFu;
            }
            else
            {
                /* Interrupt mapped to level INT0 */
                obj->spiRegs->SPILVL  = 0x00000000u;
            }

            if(obj->spiHWconfig.enableHighZ != FALSE)
            {
               obj->spiRegs->SPIINT0 |= (CSL_SPI_SPIINT0_ENABLEHIGHZ_SPIENA_HIZ <<
                                            CSL_SPI_SPIINT0_ENABLEHIGHZ_SHIFT);
            }
            else
            {
               obj->spiRegs->SPIINT0 &= (~(CSL_SPI_SPIINT0_ENABLEHIGHZ_MASK));
            }

            /**<    Modes of operation    */
            switch(obj->spiHWconfig.pinOpModes)
            {
                /**< 3-pin configuration (DI, DO, CLK pins are used) */
                case PSP_SPI_OPMODE_3PIN:
                    sPIPC0 |=
                    ((CSL_SPI_SPIPC0_SOMIFUN_SPI  << CSL_SPI_SPIPC0_SOMIFUN_SHIFT)
                    |(CSL_SPI_SPIPC0_SIMOFUN_SPI  << CSL_SPI_SPIPC0_SIMOFUN_SHIFT)
                    |(CSL_SPI_SPIPC0_CLKFUN_SPI   << CSL_SPI_SPIPC0_CLKFUN_SHIFT ));

                    obj->spiRegs->SPIPC0 = sPIPC0 ;
                    configurespiOpMode(handle);
                    break ;
                case PSP_SPI_OPMODE_SPISCS_4PIN:
                /**< 4-pin CS configuration (DI, DO, CLK & CS pins are used) */
                    sPIPC0 |=
                    ((CSL_SPI_SPIPC0_SOMIFUN_SPI  << CSL_SPI_SPIPC0_SOMIFUN_SHIFT)
                    |(CSL_SPI_SPIPC0_SIMOFUN_SPI  << CSL_SPI_SPIPC0_SIMOFUN_SHIFT)
                    |(CSL_SPI_SPIPC0_CLKFUN_SPI   << CSL_SPI_SPIPC0_CLKFUN_SHIFT ));

                    obj->spiRegs->SPIPC0 = sPIPC0 ;
                    configurespiOpMode(handle);
                    break;
                case PSP_SPI_OPMODE_SPIENA_4PIN:
                /**< 4-pin CS configuration (DI, DO, CLK & ENA pins are used) */
                    sPIPC0 |=
                    ((CSL_SPI_SPIPC0_SOMIFUN_SPI  << CSL_SPI_SPIPC0_SOMIFUN_SHIFT)
                    |(CSL_SPI_SPIPC0_SIMOFUN_SPI  << CSL_SPI_SPIPC0_SIMOFUN_SHIFT)
                    |(CSL_SPI_SPIPC0_CLKFUN_SPI   << CSL_SPI_SPIPC0_CLKFUN_SHIFT )
                    |(CSL_SPI_SPIPC0_ENAFUN_SPI   << CSL_SPI_SPIPC0_ENAFUN_SHIFT ));

                    obj->spiRegs->SPIPC0 = sPIPC0 ;
                    configurespiOpMode(handle);
                    break ;
               case PSP_SPI_OPMODE_5PIN:
                /**< 5-pin configuration(DI, DO, CLK, CS & ENA pins are used) */
                    sPIPC0 |=
                    ((CSL_SPI_SPIPC0_SOMIFUN_SPI  << CSL_SPI_SPIPC0_SOMIFUN_SHIFT)
                    |(CSL_SPI_SPIPC0_SIMOFUN_SPI  << CSL_SPI_SPIPC0_SIMOFUN_SHIFT)
                    |(CSL_SPI_SPIPC0_CLKFUN_SPI   << CSL_SPI_SPIPC0_CLKFUN_SHIFT )
                    |(CSL_SPI_SPIPC0_ENAFUN_SPI   << CSL_SPI_SPIPC0_ENAFUN_SHIFT ));

                    obj->spiRegs->SPIPC0 = sPIPC0 ;
                    configurespiOpMode(handle);
                    break;
                default:
                    retVal = PSP_E_INVAL_PARAM ;
            }
        } /**< if retVal inside  if retVal */

    } /**< if retVal */

    return (retVal) ; /**< success  or error code */
}



/**
  * \brief  Configure the chipselect pin so it will act as per spi protocol
  *         to select the one of slaves.It will also configure the delay
  *         register of following type of delay.
  *
  *         The Chip-select-active-to-ENA-signal-active-time-out
  *         The Transmit-data-finished-to-ENA-pin-inactive-time-out
  *         The Transmit-end-to-chip-select-inactive-delay
  *         The Chip-select-active-to-transmit-start-delay
  *
  * \param  Spihandle   [IN]    SPI driver object for respective instance.
  *
  */

static void configurespiOpMode(const DDC_SpiDriverObject * Spihandle)
{
    DDC_SpiObject *spiObj;
    Uint32         dly = 0 ;

    spiObj = Spihandle->spiObj;


       /**< Delay register is configured only in master mode, No
         meaning to configure in Slave mode  */
    if(PSP_SPI_TYPE_MASTER == spiObj->spiHWconfig.masterOrSlave )
    {
       /**< Insert the Chip-select-active-to-ENA-signal-active-time-out     */
       /**< Insert the Transmit-data-finished-to-ENA-pin-inactive-time-out  */
       /**< Insert the Transmit-end-to-chip-select-inactive-delay           */
       /**< Insert the Chip-select-active-to-transmit-start-delay           */

        dly |=  (spiObj->spiHWconfig.delay.c2EDelay)
               |(spiObj->spiHWconfig.delay.t2EDelay << CSL_SPI_SPIDELAY_T2EDELAY_SHIFT)
               |(spiObj->spiHWconfig.delay.t2CDelay << CSL_SPI_SPIDELAY_T2CDELAY_SHIFT)
               |(spiObj->spiHWconfig.delay.c2TDelay << CSL_SPI_SPIDELAY_C2TDELAY_SHIFT);

        spiObj->spiRegs->SPIDELAY |= dly ;

    }
}


/**
  * \brief  This to configure the delay for stale sdata
  *
  *
  * \return    void
  */

static void PSP_spiWaitForStaleData(void)
{
    volatile Uint32 count = 0x2FFFu;
    while(count != 0)
    {
        count--;
    }
}

/**
  * \brief  This to configure the data control bit of SPI data register SPIDAT1.
  *         All four params (CSHold, waitdelay, chip-select, Data format) will
  *         be uploaded to SPIDAT1 before transfer starts.
  *
  * \param  handle      [IN]    SPI driver object for respective instance.
  * \param  chipSelect  [IN]     Chip select number
  * \param  dataFormat  [IN]     Data format register selection
  * \param  flags       [IN]     Flag for Read/Write of data
   *
  * \return    Param uploaded value if success<br>
  *            Error code
  */
static  Uint32 spiControlDataConfig(
                                     PSP_Handle        handle,
                                     Uint32            chipSelect,
                                     PSP_spiDataFormat dataFormat,
                                     Uint32            flags
                                   )
{
    Uint32          spiData1   = 0;
    Uint32          csHold = FALSE;
    DDC_SpiObject   *spiObj = NULL;

    spiObj = (DDC_SpiObject *)handle;

    if ( (flags & PSP_SPI_CSHOLD) == PSP_SPI_CSHOLD )
    {
        csHold = TRUE;
    }

    if (SPI_MAX_CHIPSELECT >= chipSelect)
    {
        /* Enable the respective Enable pins in SPIPCO */
        switch (chipSelect)
        {
            case 0:
                /* select both EN 1 and EN 0  only */
                spiObj->spiRegs->SPIPC0 |= (CSL_SPI_SPIPC0_SCSFUN0_MASK |
                                            CSL_SPI_SPIPC0_SCSFUN1_MASK   );
                break;
            case 1:
                /* Enable EN0 */
                spiObj->spiRegs->SPIPC0 |= CSL_SPI_SPIPC0_SCSFUN0_MASK;
                break;
            case 2:
                /* Enable EN1 */
                spiObj->spiRegs->SPIPC0 |= CSL_SPI_SPIPC0_SCSFUN1_MASK;
                break;
            case 3:
                /* deselect both */
                spiObj->spiRegs->SPIPC0 &= ~(CSL_SPI_SPIPC0_SCSFUN0_MASK |
                                            CSL_SPI_SPIPC0_SCSFUN1_MASK   );
                break;
            default :
                break;
        }
    }

	if((spiObj->spiHWconfig.pinOpModes == PSP_SPI_OPMODE_SPISCS_4PIN)
	   ||   (spiObj->spiHWconfig.pinOpModes == PSP_SPI_OPMODE_5PIN))
	{
        /* Enable CS Hold only for 4 pin cs and 5 pin mode */
	    spiData1 |= (csHold         << CSL_SPI_SPIDAT1_CSHOLD_SHIFT);

		if(spiObj->csHighPolarity == FALSE)
		{
            /* Set the un-selected CS lines as HIGH */
            spiData1 |= DDC_SPI_SPIDAT1_CSNR_MASK;

            /* Set the selected CS line as active Low */
            spiData1 &= (~(chipSelect << CSL_SPI_SPIDAT1_CSNR_SHIFT));
        }
        else
        {
            /* Set the un-selected CS lines as LOW */
            spiData1 &= (~CSL_SPI_SPIDAT1_CSNR_MASK);
            /* Set the selected CS line as active HIGH */
            spiData1 |= ((0x1 << chipSelect) <<
                                          CSL_SPI_SPIDAT1_CSNR_SHIFT);
        }
    }
    else
    {
        /* Enable CS Hold only for 4 pin cs and 5 pin mode */
        spiData1 &= ~(CSL_SPI_SPIDAT1_CSHOLD_MASK);
    }

    spiData1 |= (spiObj->spiHWconfig.waitDelay << CSL_SPI_SPIDAT1_WDEL_SHIFT);
    spiData1 |= (dataFormat     << CSL_SPI_SPIDAT1_DFSEL_SHIFT);

    return spiData1;
}



/**
  * \brief DDC call for spi data transfer. It transfer data as transcieve
  *        operation.If transflags is true i.e. user can access read & write
  *        data. when transflags is FALSE and flags is write - Read data is
  *        dumped to local buffer (no access to user). When transflags is
  *        FALSE and flags is Read - Write data is dumped to local buffer.
  *
  * \param  handle      [IN]    SPI Driver handle
  * \param  outBuffer   [IN]    User buffer for data Source.
  * \param  inBuffer    [IN]    User or local buffer source for transceieve
  *                          operation.( User or local buffer selection
  *                          depends on transflags status ).
  * \param  bufLen      [IN]    Data Length wich is to be transmitted
  * \param  chipSelect  [IN]     Chip select number
  * \param  dataFormat  [IN]     Data format register selection
  * \param  flags       [IN]    Flags for Write /Read operation.
  * \param  transflags  [IN]    Flag for transceieve True - Used user buffer
  *                          for Transceieve, False - Local buffer for
  *                          Transceieve.
  * \param  timeout     [IN]    Timeout Value.
  * \param  param       [IN/OUT]Additional param for future use.
  * \param  xferActual  [OUT]   Transfer byte count
  *
  * \return            PSP_SOK if success<br>
  *                    Suitable error code
  *
  */
/* ARGSUSED */
static Int DDC_spiTransfer(
                           PSP_Handle handle,
                           Uint8             *outBuffer,
                           Uint8             *inBuffer,
                           Uint32            bufLen,
                           Uint32            chipSelect,
                           PSP_spiDataFormat dataFormat,
                           Uint32            flags,
                           Bool              transflags,
                           Int               timeout,
                           Ptr               param,
                           Ptr               xferActual
                           )

{
     DDC_SpiDriverObject     *spiHandle   = NULL;
     Uint32                   status      = 0;
     Uint32                   timeCnt     = 0;
     Uint32                   cookie      = 0;
     Uint32                   spidat1     = 0;
     Uint32                   cnt         = 0;
     DDC_SpiObject           *spiObj      = NULL;
     PSP_Result               retVal      = PSP_SOK;
     Uint8                   *spiDat1     = NULL;


    /**<Validate the spi driver object handle and Validate the spi data buffer */
    if ( (( (NULL == handle) || (NULL == outBuffer) ) ||
            ( (NULL == inBuffer) || (0 == bufLen) )) ||
                ((dataFormat >= SPI_DATAFORMAT_VALUE) || (chipSelect > SPI_MAX_CHIPSELECT)) )
    {
        retVal = PSP_E_INVAL_PARAM;
    }
    else
    {
        /* Extract the spi driver object handle      */
        spiHandle   = (DDC_SpiDriverObject *) handle;
        spiObj      =  spiHandle->spiObj;
    }

    if (retVal == PSP_SOK)
    {
#ifdef SPI_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT(  &DVTEvent_Log,
                    "SPI",
                    PSP_DVT_DESC(   PSP_DVT_eRCV_START,
                                    PSP_DVT_dCHANNEL_ID_INST,
                                    PSP_DVT_dPACKET_ID,
                                    PSP_DVT_dNONE),
                    CHANNELID_INSTANCEID(
                        1,
                        spiObj->instanceId),
                    outBuffer
                 );
#endif /* SPI_INSTRUMENTATION_ENABLED */
        /* To make transaction is in progress */
        spiObj->pendingState = TRUE;

        spidat1 = spiControlDataConfig(spiObj,chipSelect,dataFormat,flags);

       /** If polled mode, wait for data transfer & completion, else wait
        * (on a semaphore) for transaction to complete in interrupt mode
        * In interrupt mode, for write case, it has to poll on RX_INT bit,
        * as there is no transmit interrupt bit to set
        */
        if( spiObj->mode == PSP_OPMODE_DMAINTERRUPT )
        {
           /** DMAINTERRUPT Mode of data transfer      */
#ifdef PSP_SPI_EDMA_SUPPORT
           /** Mask all interrupts,& populate data to spi object structure
            *  so driver can extact all information at dma configuartion
            *  and dma Rx & Tx callback
            */
             /**< Transmit data  */
            while ((spiObj->spiRegs->SPIBUF & CSL_SPI_SPIBUF_RXEMPTY_MASK )
                                                   != CSL_SPI_SPIBUF_RXEMPTY_MASK)
            {
                /**< Read before Write to remove any stale data  */
                *spiObj->transBuffer = (Uint8)spiObj->spiRegs->SPIBUF;
            }

           spiObj->spiRegs->SPIINT0    = spiObj->spiRegs->SPIINT0 &
                                          (~(SPI_INTERRUPT_MASK));
            spiObj->currError           = 0;
            spiObj->currFlags           = flags;
            spiObj->transcieveFlags     = transflags;
            spiObj->currBuffer          = outBuffer;
            spiObj->currBufferLen       = bufLen;

            /**< Call this to complete edma configuration and transfer */
            retVal = (int)start_edma_transfer(handle,chipSelect,dataFormat,flags);

            if (retVal == PAL_SOK)
            {
              /** Take completion semaphore  and wait till transfer being
                * completed.  */
                retVal = PAL_osSemTake(
                                       spiObj->completionSem,
                                      PAL_OsGetTicksPerMSecs((Int) timeout)
                                      );
                if (retVal != PAL_SOK)
                {
                    /**< Transfer failed, populate error status */
                    PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);
                    spiObj->currError = PSP_E_TIMEOUT ;
                    spiObj->currBufferLen = 0;
                    spiObj->currBuffer = NULL;
                    PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
                }

                /** Transfer success, populate error status,no.of byte
                  * transfer  */
                retVal = spiObj->currError;
                cnt = spiObj->currBufferLen;
                *(Uint32 *)xferActual = cnt;
            }

#endif /* for EDMA support */
        }
        else if(spiObj->mode == PSP_OPMODE_POLLED)
        {
            /* Polled mode of data transfer Mask all interrupts   */
            spiObj->spiRegs->SPIINT0 &= (~(SPI_INTERRUPT_MASK));

            /* To get current ticks to find out the data transfer timeout */
            timeCnt = PAL_osGetCurrentTick();

            while (cnt < bufLen)
            {
                if (( PAL_osCheckTimeOut(timeCnt,timeout) == FALSE) &&
                    ( PAL_SOK == retVal ))
                {
                    /**< Transmit data  */
                    while ((spiObj->spiRegs->SPIBUF & CSL_SPI_SPIBUF_RXEMPTY_MASK )
                                                           != CSL_SPI_SPIBUF_RXEMPTY_MASK)
                    {
                        /**< Read before Write to remove any stale data  */
                        *spiObj->transBuffer = (Uint8)spiObj->spiRegs->SPIBUF;
                    }


                   /*  Transmit data pend on TX_FULL flag i.e
                    *  TX_FULL == TRUE  i.e  data is already in transmit
                    *  buffer.
                    *  TX_FULL == FALSE i.e. No data in transmit buffer,
                    *  write next data.    */

                    if (( spiObj->spiRegs->SPIBUF & CSL_SPI_SPIBUF_TXFULL_MASK)
                                                      == FALSE )
                    {
                        /**< Write data in 4-pin CS & 5-pin modes */
                        if(spiObj->spiHWconfig.configDatafmt[dataFormat].charLength
                           <= 8u)
                        {
                            spiObj->spiRegs->SPIDAT1 = spidat1 | (* outBuffer);
                        }
                        else
                        {
                            spiObj->spiRegs->SPIDAT1  = spidat1 | (*((Uint16*)outBuffer));
                        }
                        PSP_spiWaitForStaleData();
                        /* Make sure data transfer is completed before
                         reading. Read after Write to implement
                         transcieve operation */

                        /* read transceived data */
                        while((spiObj->spiRegs->SPIBUF & CSL_SPI_SPIBUF_RXEMPTY_MASK )
                                                          == CSL_SPI_SPIBUF_RXEMPTY_MASK)
                        {
							if(PAL_osCheckTimeOut(timeCnt,timeout) == TRUE)
							{
							   retVal = PSP_E_TIMEOUT;
							   break;
							}
                        }
                        if(spiObj->spiHWconfig.configDatafmt[dataFormat].charLength
                            <= 8u)
                        {
                            *spiObj->transBuffer = (Uint8)spiObj->spiRegs->SPIBUF;
                            outBuffer++;
		                    /* check this is transcieve call or not if
		                    yes then increment the buffer else not &
		                    dumped the data  to local buffer. */
		                    if(FALSE != transflags)
		                    {
		                        spiObj->transBuffer++;
		                    }
                        }
                        else
                        {
                            (*((Uint16*)spiObj->transBuffer)) = (Uint16)spiObj->spiRegs->SPIBUF;
                            outBuffer += 2;
	                        /* check this is transcieve call or not if
	                        yes then increment the buffer else not &
	                        dumped the data  to local buffer. */
	                        if(FALSE != transflags)
	                        {
	                            spiObj->transBuffer+=2;
	                        }
                        }

                        cnt++;
                    }

                    /* Check for bit error, desync error,parity error,timeout
                       error and receive overflow errors */
                    status = spiObj->spiRegs->SPIFLG;

                    if ((status & CSL_SPI_SPIFLG_BITERR_MASK) ==
                           CSL_SPI_SPIFLG_BITERR_MASK)
                    {
                        spiObj->spiRegs->SPIFLG |= CSL_SPI_SPIFLG_BITERR_MASK;
                        retVal = PSP_SPI_BIT_ERR;
                    }
                    else
                    {
                       if ((status & CSL_SPI_SPIFLG_RCVROVRN_MASK) ==
                            CSL_SPI_SPIFLG_RCVROVRN_MASK)
                       {
                           spiObj->spiRegs->SPIFLG |= CSL_SPI_SPIFLG_RCVROVRN_MASK;
                           retVal = PSP_SPI_RECEIVE_OVERRUN_ERR;
                       }
                    }

                    if((status &  CSL_SPI_SPIFLG_PARITYERR_MASK) ==
                                  CSL_SPI_SPIFLG_PARITYERR_MASK)
                    {
                        spiObj->spiRegs->SPIFLG |= CSL_SPI_SPIFLG_PARITYERR_MASK;
                        retVal = PSP_SPI_PARITY_ERR ;
                    }

                    if ((status & CSL_SPI_SPIFLG_TIMEOUT_MASK) ==
                         CSL_SPI_SPIFLG_TIMEOUT_MASK)
                    {
                        spiObj->spiRegs->SPIFLG |= CSL_SPI_SPIFLG_TIMEOUT_MASK;
                        retVal = PSP_SPI_TIMEOUT_ERR;
                    }

                    if((status &  CSL_SPI_SPIFLG_DESYNC_MASK) ==
                        CSL_SPI_SPIFLG_DESYNC_MASK)
                    {
                        spiObj->spiRegs->SPIFLG |= CSL_SPI_SPIFLG_DESYNC_MASK;
                        retVal = PSP_SPI_DESYNC_ERR ;
                    }
                }
                else
                {
                    if (retVal == PSP_SOK)
                    {
                        retVal = PSP_E_TIMEOUT;
                    }
                    break;
                }

            }/* While */

            /* While number of byte to be transmitted/received loop */
            *(Uint32 *)xferActual = cnt ;
        }
        else
        {
                /**< SPI Interrupt mode operation */
            if(spiObj->mode == PSP_OPMODE_INTERRUPT)
            {
                /** Populate the user send params to spi object structure
                  * So device can use all the information when running at
                  * ISR.
                  */
                spiObj->currError         = 0;
                spiObj->currFlags         = flags;
                spiObj->transcieveFlags   = transflags;
                spiObj->currBuffer        = (outBuffer + cnt);
                spiObj->currBufferLen     = (bufLen- cnt);

                /*Consume any stale data */
                while((spiObj->spiRegs->SPIBUF & CSL_SPI_SPIBUF_RXEMPTY_MASK )
                                                 != CSL_SPI_SPIBUF_RXEMPTY_MASK)
                {
                     *spiObj->transBuffer = (Uint8)spiObj->spiRegs->SPIBUF;
                }

                if(spiObj->spiHWconfig.configDatafmt[dataFormat].charLength > 8u)
                {
                    spiObj->charLength16Bits = TRUE;
                }

                if ((spiObj->currBufferLen) > 0)
                {
                    if(FALSE == spiObj->charLength16Bits)
                    {
                        spiObj->spiRegs->SPIDAT1  =
                            ( spidat1 | * spiObj->currBuffer);
                    }
                    else
                    {
                            spiObj->spiRegs->SPIDAT1  =
                        (spidat1 | (*((Uint16*)spiObj->currBuffer)));
                    }
                    if(FALSE == spiObj->charLength16Bits)
                    {
                           spiObj->currBuffer++;
                    }
                    else
                    {
                       spiObj->currBuffer += 2;
                    }

                    /* Enabling receive and error interrupts */
                     spiObj->spiRegs->SPIINT0 |=
                                 ( CSL_SPI_SPIINT0_BITERRENA_MASK  |
                                   CSL_SPI_SPIINT0_TIMEOUTENA_MASK |
                                   CSL_SPI_SPIINT0_OVRNINTEN_MASK |
                                   CSL_SPI_SPIINT0_DESYNCENA_MASK  |
                                   CSL_SPI_SPIINT0_PARERRENA_MASK  |
                                   CSL_SPI_SPIINT0_RXINTEN_MASK
                                 );

                    /* Take completion semaphore */
                    retVal = PAL_osSemTake(spiObj->completionSem,
                              PAL_OsGetTicksPerMSecs((Int)timeout) );

                    if(retVal != PAL_SOK)
                    {
                       /* Transfer in inetrrupt mode failed,disable
                          The RX interrupt bit and populate error
                          status to notify the dda layer */
                        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

                        /**< Disable Receive interrupt */
                        spiObj->spiRegs->SPIINT0 &=
                                (~(CSL_SPI_SPIINT0_RXINTEN_MASK));

                        spiObj->currError      = PSP_E_TIMEOUT ;
                        spiObj->currBufferLen  = 0 ;
                        spiObj->currBuffer     = NULL;
                        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
                        retVal = spiObj->currError;
                    }
                    else
                    {
                        retVal                  = spiObj->currError;
                        cnt                     = bufLen - spiObj->currBufferLen;
                        *(Uint32 *)xferActual   = cnt;
                    }

                }/**< if currBuffLen */
            } /* if interrupt */
        }/* else */
        /* To trigger completion of transaction */
        spiObj->pendingState = FALSE;

#ifdef SPI_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT(  &DVTEvent_Log,
                    "SPI",
                    PSP_DVT_DESC(   PSP_DVT_eRCV_END,
                                    PSP_DVT_dCHANNEL_ID_INST,
                                    PSP_DVT_dPACKET_ID,
                                    PSP_DVT_dNONE),
                    CHANNELID_INSTANCEID(
                        1,
                        spiObj->instanceId),
                    outBuffer
                 );
#endif /* SPI_INSTRUMENTATION_ENABLED */

    	spiObj->pendingState = FALSE;

        /** check if CSHOLD needs to be enable after the transmission also.
          * If No or if there is any eror disable CSHOLD value, else do nothing
          */
        if(((flags & PSP_SPI_CSHOLD_FOR_MULTI_TRANSCEIVE)!=PSP_SPI_CSHOLD_FOR_MULTI_TRANSCEIVE) || (retVal != 0) )
        {
                /* Pointing to the MSByte of SPIDAT1 to reset CSHOLD bit*/
                spiDat1 = ((Uint8*)&(spiObj->spiRegs->SPIDAT1))+3u;
                *spiDat1 &= ~(SPI_SPIDAT1_CSHOLD_8BIT_MASK);

        }
    } /* if retVal */
    return retVal;
}

#ifdef PSP_SPI_EDMA_SUPPORT
/**
 *  \brief This function initiates Edma mode of operation.SPI Edma Rx & Tx
 *         channels are configured and spi Edma interrupt in enable here
 *           to get Rx and Tx events.
 *
 *
 *  \param  handle   [IN]    Handle of the SPI DDC Object
 *
 *  \return          PAL_SOK if successful<br>
 *                   Suitable error code if failure
 *
 */
static Int start_edma_transfer(PSP_Handle handle,
                               Uint32            chipSelect,
                               PSP_spiDataFormat dataFormat,
                               Uint32            flags
                              )
{

    DDC_SpiDriverObject    *spiHandle   = NULL;
    Uint8                  *temp1Ptr    = NULL;
    Uint8                  *temp2Ptr    = NULL;
    volatile Uint32       *regPtr      = NULL;
    Int                    dstIndex     = 0;
    Uint32                 spidat1      = 0;
    DDC_SpiObject          *spiObj      = NULL;
    PSP_Result             retCode      = PAL_SOK;
    Uint32                 acnt         = 1u;

    /**< Validate the driver object handle */
    if ((handle == NULL) && (spiObj == NULL))
    {
        retCode = PSP_E_INVAL_PARAM;
    }
    else
    {
        spiHandle = (DDC_SpiDriverObject *) handle ;
        spiObj =  spiHandle->spiObj;
        /** Extract data control param values to upload to SPIDAT1
          * before read/write transfer starts   */
        spidat1 = spiControlDataConfig(spiObj,chipSelect,dataFormat,flags);

        /** Buffer assingment to local pointer is done to
          * perform transcieve operation   */
        temp1Ptr = (Uint8 *)spiObj->transBuffer;
        temp2Ptr = (Uint8 *)spiObj->currBuffer;
        *temp1Ptr = spiObj->spiRegs->SPIBUF;
        spiObj->spiRegs->SPIDAT1 = (spidat1 | (* temp2Ptr));

        if(spiObj->spiHWconfig.configDatafmt[dataFormat].charLength > 8u )
        {
            acnt = 2u;
        }

        regPtr = &(spiObj->spiRegs->SPIDAT1 );
    }

    if(retCode == PAL_SOK)
    {
        /* transcieveFlags == True i.e transcieve operetion so increment
           the data buffer else use buffer as local and dumped the data
           onto same location(dont increment buffer). */
        if(TRUE == (spiObj->transcieveFlags))
        {
            dstIndex = 1;
        }

        /**< Read intialization for DMA mode */
        PAL_osCacheFlushAndInvalidate((PAL_osCacheMemAddrSpace)NULL,
                                      (Uint32)(temp1Ptr),spiObj->currBufferLen);

        if(EDMA3_DRV_setSrcParams  ((spiObj->edmaSpiHandle                  ),
                                     spiObj->chanEdmaRx,
                                    (Uint32)&(spiObj->spiRegs->SPIBUF         ),
                                     EDMA3_DRV_ADDR_MODE_INCR,
                                     EDMA3_DRV_W8BIT)           !=  PAL_SOK )
        {
            retCode = PSP_E_INVAL_PARAM;
#ifdef SPI_DEBUG_PRINTF
            SPI_DEBUG("\nSPI Test: EDMA Source Param Setting failed" );
#endif /* for printf */
        }
        else if(EDMA3_DRV_setDestParams ((  spiObj->edmaSpiHandle           ),
                                            spiObj->chanEdmaRx,
                                         (  Uint32)temp1Ptr,
                                              EDMA3_DRV_ADDR_MODE_INCR,
                                              EDMA3_DRV_W8BIT)    !=  PAL_SOK )
        {
            retCode = PSP_E_INVAL_PARAM;
#ifdef SPI_DEBUG_PRINTF
            SPI_DEBUG("\nSPI Test: EDMA Destination Param Setting failed" );
#endif /* for printf */
        }
        else if(EDMA3_DRV_setSrcIndex ((spiObj->edmaSpiHandle               ),
                                        spiObj->chanEdmaRx,0,0           )
                                                                !=  PAL_SOK )
        {
            retCode = PSP_E_INVAL_PARAM;
#ifdef SPI_DEBUG_PRINTF
            SPI_DEBUG("\nSPI Test: EDMA Source Index Param Setting failed" );
#endif /* for printf */
        }
        else if(EDMA3_DRV_setDestIndex ((spiObj->edmaSpiHandle              ),
                                         spiObj->chanEdmaRx,dstIndex,0   )
                                                                !=  PAL_SOK )
        {
            retCode = PSP_E_INVAL_PARAM;
#ifdef SPI_DEBUG_PRINTF
            SPI_DEBUG("\nSPI Test: EDMA Destination Index Param Setting failed " );
#endif /* for printf */
        }
        else if(EDMA3_DRV_setTransferParams ((spiObj->edmaSpiHandle         ),
                                              spiObj->chanEdmaRx,acnt,
                                              spiObj->currBufferLen,1u,0,
                                              EDMA3_DRV_SYNC_A) !=  PAL_SOK )
        {
            retCode = PSP_E_INVAL_PARAM;
#ifdef SPI_DEBUG_PRINTF
            SPI_DEBUG("\nSPI Test: EDMA Transfer Param Setting failed");
#endif /* for printf */
        }
        else if(EDMA3_DRV_setOptField ((spiObj->edmaSpiHandle               ),
                                        spiObj->chanEdmaRx,
                                        EDMA3_DRV_OPT_FIELD_TCINTEN,
                                        1u)                     !=  PAL_SOK )
        {
            retCode = PSP_E_INVAL_PARAM;
#ifdef SPI_DEBUG_PRINTF
            SPI_DEBUG("\nSPI Test: EDMA Param Setting failed" );
#endif /* for printf */
        }
        else
        {
             if(EDMA3_DRV_enableTransfer ((spiObj->edmaSpiHandle),
                                            spiObj->chanEdmaRx,
                                            EDMA3_DRV_TRIG_MODE_EVENT        )
                                                                 !=  PAL_SOK )
             {

                retCode = PSP_E_INVAL_PARAM;
#ifdef SPI_DEBUG_PRINTF
                SPI_DEBUG("\nSPI Test: EDMA Transfer Enable failed" );
#endif /* for printf */
             }
        }

        /**< Write intialization for DMA mode */
         PAL_osCacheFlushAndInvalidate(  (PAL_osCacheMemAddrSpace)NULL,
                                         (Uint32)(temp2Ptr),
                                         spiObj->currBufferLen               );

        if(EDMA3_DRV_setSrcParams (     (spiObj->edmaSpiHandle              ),
                                        spiObj->chanEdmaTx,
                                        (Uint)(temp2Ptr + 1u),
                                        EDMA3_DRV_ADDR_MODE_INCR,
                                        EDMA3_DRV_W8BIT)        !=  PAL_SOK )
        {
            retCode = PSP_E_INVAL_PARAM;
#ifdef SPI_DEBUG_PRINTF
            SPI_DEBUG("\nSPI Test: EDMA Source Param Setting failed" );
#endif /* for printf */
        }
        else if(EDMA3_DRV_setDestParams((spiObj->edmaSpiHandle              ),
                                         spiObj->chanEdmaTx,
                                        (Uint)regPtr,
                                        EDMA3_DRV_ADDR_MODE_INCR,
                                        EDMA3_DRV_W8BIT)        !=  PAL_SOK )
        {
            retCode = PSP_E_INVAL_PARAM;
#ifdef SPI_DEBUG_PRINTF
            SPI_DEBUG("\nSPI Test: EDMA Destination Param Setting failed" );
#endif /* for printf */
        }
        else if(EDMA3_DRV_setSrcIndex ((spiObj->edmaSpiHandle               ),
                                        spiObj->chanEdmaTx,
                                        dstIndex,0)             !=  PAL_SOK )
        {
            retCode = PSP_E_INVAL_PARAM;
#ifdef SPI_DEBUG_PRINTF
            SPI_DEBUG("\nSPI Test: EDMA Source Index Param Setting failed" );
#endif /* for printf */
        }
        else if(EDMA3_DRV_setDestIndex ((spiObj->edmaSpiHandle              ),
                                        spiObj->chanEdmaTx,0,0           )
                                                                !=  PAL_SOK )
        {
            retCode = PSP_E_INVAL_PARAM;
#ifdef SPI_DEBUG_PRINTF
            SPI_DEBUG("\nSPI Test:EDMA Destination Index Param Setting failed ");
#endif /* for printf */
        }
        else if(EDMA3_DRV_setTransferParams ((spiObj->edmaSpiHandle),
                                              spiObj->chanEdmaTx,acnt,
                                             (spiObj->currBufferLen),1u, 0,
                                              EDMA3_DRV_SYNC_A)
                                                                !=  PAL_SOK )
        {
            retCode = PSP_E_INVAL_PARAM;
#ifdef SPI_DEBUG_PRINTF
            SPI_DEBUG("\nSPI Test: EDMA Transfer Param Setting failed");
#endif /* for printf */
        }
        else if(EDMA3_DRV_setOptField ((spiObj->edmaSpiHandle               ),
                                        spiObj->chanEdmaTx,
                                        EDMA3_DRV_OPT_FIELD_TCINTEN,
                                        1u)                     !=  PAL_SOK )
        {
            retCode = PSP_E_INVAL_PARAM;
#ifdef SPI_DEBUG_PRINTF
            SPI_DEBUG("\nSPI Test: EDMA Param Setting failed" );
#endif /* for printf */
        }
        else
        {
              if(EDMA3_DRV_enableTransfer ((spiObj->edmaSpiHandle           ),
                                              spiObj->chanEdmaTx,
                                              EDMA3_DRV_TRIG_MODE_EVENT       )
                                                                  !=  PAL_SOK )
              {
                    retCode = PSP_E_INVAL_PARAM;
#ifdef SPI_DEBUG_PRINTF
                    SPI_DEBUG("\nSPI Test: EDMA Transfer Enable failed" );
#endif /* for printf */
              }

        }
          spiObj->spiRegs->SPIINT0 |= (CSL_SPI_SPIINT0_DMAREQEN_ENABLED <<
                                        CSL_SPI_SPIINT0_DMAREQEN_SHIFT ) ;
    }
     /* if retVal */
          return (retCode) ;
}
#endif /* for EDMA support */


/**< callback function incase DMA receive */
#ifdef PSP_SPI_EDMA_SUPPORT
/**
 *  \brief This function is called in response of the receive is complete
 *         Callback for the receive. Rx channel transfer for Edma is disable
 *         and semaphore is released here.
 *
 *
 *  \param  tcc         [IN]    Receive interrupt event channel
 *  \param  status      [IN]    Transfer status whether transfer is completed
 *                              or not.
 *  \param  appData     [IN]    Data passed Edma during initiation
 *
 */
static void callbackReceive(Uint tcc, EDMA3_RM_TccStatus status, Ptr appData)
{

 DDC_SpiObject         *spiObj;
 Uint                   paRAMField  = 0;
 PAL_Result             result      = PAL_SOK;
 Uint                   chan_Rx     = tcc;

 if (appData != NULL)
 {
     spiObj =  (DDC_SpiObject *) appData;
     if (spiObj != NULL)
     {
         /**< Disable the dma interrupt */
         spiObj->spiRegs->SPIINT0 &= (CSL_SPI_SPIINT0_DMAREQEN_DISABLED <<
                                            CSL_SPI_SPIINT0_DMAREQEN_SHIFT) ;
         spiObj->currError = status;

         EDMA3_DRV_disableTransfer ((spiObj->edmaSpiHandle),
                                        chan_Rx,EDMA3_DRV_TRIG_MODE_EVENT);

         if(PAL_SOK != EDMA3_DRV_setOptField ((  spiObj->edmaSpiHandle),
                                                 chan_Rx,
                                                 EDMA3_DRV_OPT_FIELD_TCC,
                                                 chan_Rx))
         {
             result = PSP_E_INVAL_PARAM;
         }
         else
         {

              if(PAL_SOK != EDMA3_DRV_getPaRAMField ((spiObj->edmaSpiHandle),
                                         chan_Rx,
                                         (EDMA3_DRV_PaRAMField)EDMA3_DRV_PARAM_ENTRY_LINK_BCNTRLD,
                                         &paRAMField))
              {
                  result = PSP_E_INVAL_PARAM;

              }
              else
              {
                   paRAMField |= 0xFFFFu;
                   if(PAL_SOK != (EDMA3_DRV_setPaRAMField (
                                         (spiObj->edmaSpiHandle),
                                          chan_Rx,
                                         (EDMA3_DRV_PaRAMField)EDMA3_DRV_PARAM_ENTRY_LINK_BCNTRLD,
                                         paRAMField )))
                   {
                        result = PSP_E_INVAL_PARAM;
                   }
              }
              spiObj->currError = result;
         }
         /**< Release semaphore when transfer is completed */
         if (PAL_osSemGive(spiObj->completionSem) != PAL_SOK)
         {
#ifdef SPI_DEBUG_PRINTF
              SPI_DEBUG("\nPSP SPI[%l]:ISR: Error giving completion semaphore in RDR",
                            spiObj->instanceId);
#endif /* for printf */
         }
     }
     else
     {
         spiObj->currError = PSP_E_INVAL_PARAM;
     }
  }
  else
  {
      return ;
  }
}
#endif /* for EDMA support */

/**< callback function incase DMA Transmit */
#ifdef PSP_SPI_EDMA_SUPPORT
/**
 *  \brief  This function is called in response of the transmit is completed
 *          Callback for the transmit.Tx channel transfer for Edma is disable
 *          here.
 *
 *  \param  tcc         [IN]    Transmit interrupt event channel
 *  \param  status      [IN]    Transfer status whether transfer is completed
 *                                or not.
 *  \param  appData     [IN]    Data passed Edma during initiation
 *
 */
static void callbackTransmit(Uint tcc, EDMA3_RM_TccStatus status, Ptr appData)
{

    DDC_SpiObject         *spiObj;
    Uint                   paRAMField    = 0;
    PAL_Result             result        = PAL_SOK;
    Uint                   ch_Tx         = tcc;

    if (appData != NULL)
    {
        spiObj =  (DDC_SpiObject *) appData;

        if (spiObj != NULL)
        {

            spiObj->currError = status;
            EDMA3_DRV_disableTransfer (
                                       (spiObj->edmaSpiHandle),
                                        ch_Tx,
                                        EDMA3_DRV_TRIG_MODE_EVENT
                                       );

        if(PAL_SOK != (EDMA3_DRV_setOptField ((spiObj->edmaSpiHandle),
                                               ch_Tx,
                                               EDMA3_DRV_OPT_FIELD_TCC,
                                               ch_Tx)))
        {
             result = PSP_E_INVAL_PARAM;
        }
        else
        {
            if(PAL_SOK != (EDMA3_DRV_getPaRAMField ((spiObj->edmaSpiHandle),
                                                   ch_Tx,
                                                   (EDMA3_DRV_PaRAMField)EDMA3_DRV_PARAM_ENTRY_LINK_BCNTRLD,
                                                   &paRAMField)))
             {
                  result = PSP_E_INVAL_PARAM;
             }
             else
             {
                 paRAMField |= 0xFFFFu;
                 if(PAL_SOK != (EDMA3_DRV_setPaRAMField ((spiObj->edmaSpiHandle),
                                                        ch_Tx,
                                                        (EDMA3_DRV_PaRAMField)EDMA3_DRV_PARAM_ENTRY_LINK_BCNTRLD,
                                                        paRAMField )))
                {
                    result = PSP_E_INVAL_PARAM;
                }
            }
        }
     }
     else
     {
        spiObj->currError = result;
     }
   }
   else
   {
        return;
   }
}
#endif /* for EDMA support */



