/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/


/**
 *  \mainpage    PSP SPI Driver on BIOS
 *
 *  \section    Introduction
 *
 *  \subsection    xxx    Purpose and Scope
 *  The purpose of this document is to explain the SPI driver on the PSP
 *  Framework architecture written in DSP/BIOS for DM648/C6452 platform.
 *
 *  \subsection    aaa    PSP Architecture
 *  A PSP framework is a software entity providing platform support
 *  for devices that are based on Texas Instruments SoC's. The specific
 *  (Hardware and software) implementation of the PSP framework (the PAL
 *  implementation and the group of device drivers) is referred to as
 *  PSP System. The PSP framework defines a software environment for device
 *  drivers to plug-in by defining standard usage interfaces and
 *  abstractions (OS, CPU/Peripherals). For every platform, the PSP
 *  implements platform specific functionality such as OS adaptation, CPU and
 *  peripherals specific mechanisms, and generic system services such as
 *  debug control,traces, persistent storage, etc.
 *
 *  \subsection    bbb    SPI Controller
 *  SPI is a high-speed synchronous serial input/output port that allows a
 *  serial bit stream of programmed length (one to 16 bits) to be shifted into
 *  and out of the device at a programmed bit-transfer rate. SPI is normally
 *  used for communication between the microcontroller and external peripherals
 *  or another microcontroller. Typical applications include interface to
 *  external I/O or peripheral expansion via devices such as shift
 *  registers, display drivers, and analog-to-digital converters.
 *
 *  \subsection    ccc    DSP/BIOS System
 *  DSP/BIOS is a real time kernel provided by TI to use with its
 *  DSP processors.
 *
 *
 *  \subsection    References
 *    -# SPI functional specification, <b>sprue32_SPI.pdf</b>
 *    -# Design Document for SPI driver,
 *                         <b>BIOS_SPI_Driver_Arch.doc</b>
 *    -# PSP Framework Overview, <b>PSPFramework_Overview.pdf</b>
 *    -# DSP/BIOS driver development <b>SPRU616.pdf</b>
 *
 *  \file   ddc_spi.h
 *
 *  \brief  SPI Core header file
 *
 *  This file contains the interfaces, data types and symbolic definitions
 *  that are needed by the application to utilize the services of the SPI
 *  device driver.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author        PSG
 *
 *  \version    1.x - Created                           Shailesh Kumar Sharm
 *              2.0 - Modified for DR6437               Shailesh Kumar Sharma
 *              2.1 - Modified for DM648                Nagarjuna Kristam
 *              2.2 - Made appropriate changes for
 *                    SPIDAT1 register bit performance  Nagarjuna Kristam
 *              2.3 - Arranged datastruture members
 *                      and clean up effort             Nagarjuna Kristam
 *              2.4 - Updated as per review coments     Nagarjuna Kristam
 *                    REVW00004866
 *
 */

#ifndef _DDC_SPI_H_
#define _DDC_SPI_H_

#ifdef CHIP_DM648
#include <ti/sdo/pspdrivers/soc/dm648/dsp/soc.h>
#include <ti/sdo/pspdrivers/soc/dm648/dsp/cslr_spi.h>
#endif

#ifdef CHIP_C6452
#include <ti/sdo/pspdrivers/soc/c6452/dsp/soc.h>
#include <ti/sdo/pspdrivers/soc/c6452/dsp/cslr_spi.h>
#endif

#include <ti/sdo/pspdrivers/pal_os/pal_osSem.h>
#include <ti/sdo/pspdrivers/common/psp_common.h>
#include <ti/sdo/pspdrivers/drivers/spi/src/psp_spiApi.h>

#ifdef __cplusplus
extern "C" {
#endif

/** Chip select number MASK Macro for SPIDAT1 */
#define DDC_SPI_SPIDAT1_CSNR_MASK		0x00030000u

/* Internal enums and data structures */
/**
 * State of SPI Driver
 */
typedef enum DDC_SPIState_t
{
    SPI_DELETED = 0,
    /**< Instance Deleted */
    SPI_CREATED,
    /**< Instance Created */
    SPI_OPENED,
    /**< Instance Open */
    SPI_CLOSED
    /**< Instance Closed */
} DDC_SPIState;

/**
 *  \brief DDC SPI Object
 *
 *  This structure basically holds the SPI driver instance parameters
 */
typedef struct DDC_SpiObject_t
{
    /* SOC Related Details      */
    CSL_SpiRegs     *spiRegs;           /**< SPI peripheral register map      */
    Uint32          intNum;             /**< Interrupt number                 */

    /* SPI Peripheral details  */
    PSP_OpMode      mode;               /**< Driver operating mode - polled,
                                                            interrupt, dma    */
    Uint32          moduleInputClkFreq; /**< Module input clock freq          */
    Uint32          spiBusFreq;         /**< SPI Bus Frequency                */
    PSP_spiHWConfigData spiHWconfig;    /**< SPI Hardware configuration Data  */
    Bool            loopBack;           /**< to enable/disable loop back mode */

    /* Driver internal members */
    Int             instanceId;         /**< Instance number                  */
    DDC_SPIState    state;              /**< State of driver                  */
    Uint32          numOpens;           /**< Number of channels opened        */
    Bool            pendingState;       /**< Shows whether io is in pending
                                             state or not                     */
    Bool            Cancel_Pending_IO;  /**< Shows whether IO has to
                                             cancel or not                    */
	Bool            charLength16Bits;	/**< Flag to indicate if char length
	                                         greater than 8 bits              */
    Bool            csHighPolarity;     /**< Chip Select Polarity. Either active
                                             high/Low. By default it is set to
                                             Active Low. TRUE = Active High   */

    /* Driver internal semaphore */
    PAL_OsSemHandle devBusySem;         /**< Device busy semaphore handle     */
    PAL_OsSemHandle completionSem;      /**< Device busy semaphore handle     */

    /* Driver internal Transmission details */
    Int             currError;          /**< Error code                       */
    Uint32          currFlags;          /**< Current Flags for read/write     */
    Bool            transcieveFlags;    /**< flag for transcieve operation    */
    Uint8           *currBuffer;        /**< User buffer for read/write       */
    Uint8           *transBuffer;       /**< Buffer used for transieve
                                             operation of spi                 */
    Uint32          currBufferLen;      /**< User buffer length               */

    /* Driver EDMA members */
    Ptr             edmaSpiHandle;      /**< Handle used for Edma             */
	Bool            dmaChaAllocated;	/**< Flag to inidicate EDMA channels
	                                         allocation status                */
    Uint            chanEdmaTx;         /**< Edma transmit channel            */
    Uint            chanEdmaRx;         /**< Edma receive channel             */
} DDC_SpiObject;

/**
 * \brief    Default DDC SPI OBJECT values
 */
#define DDC_SPI_OBJECT_DEFAULT                  \
        {                                       \
        /*Regs;*/               NULL,           \
        /*intNum;*/             0,              \
         /*mode*/               PSP_OPMODE_POLLED,      \
        /* moduleInputClkFreq;*/0,              \
        /* spiBusFreq;*/        0,              \
        /* ata spiHWconfig;*/   {               \
          0,PSP_SPI_TYPE_MASTER,0,0,PSP_SPI_OPMODE_3PIN,{0,0,0,0},0,0,{  \
                                                    {0,0,0,0,0,0,0,0},     \
                                                    {0,0,0,0,0,0,0,0},     \
                                                    {0,0,0,0,0,0,0,0},     \
                                                    {0,0,0,0,0,0,0,0}      \
                                                        }  \
                                },              \
        /*loopBack;*/           0,              \
        /*instanceId;*/         0,              \
        /*state;*/              SPI_DELETED,    \
        /*numOpens;*/           0,              \
        /*pendingState;*/       0,              \
        /*charLength16Bits;*/   0,              \
        /*Cancel_Pending_IO;*/  0,              \
        /*csPolarity*/          0,              \
        /*devBusySem;*/         NULL,           \
        /*completionSem;*/      NULL,           \
        /*currError;*/          0,              \
        /*currFlags;*/          0,              \
        /*  transcieveFlags;*/  0,              \
        /**currBuffer;*/        NULL,           \
        /*  *transBuffer;*/     NULL,           \
        /*currBufferLen;*/      0,              \
        /*edmaSpiHandle;*/      NULL,           \
        /*dmaChaAllocated;*/    0,              \
        /*chanEdmaTx;*/         NULL,           \
        /*chanEdmaRx;*/         NULL            \
        }

/**
 *  \brief  SPI Driver Object
 *
 *  This structure basically holds the SPI driver instance parameters
 *  callback,dataformat params.
 */
typedef struct DDC_SpiDriverObject_t
{
    DDC_SpiObject *spiObj ;
    /**< SPI Insatnce object handle                              */
    PSP_SpiAppCallback callBack;
    /**< Callback from applications                              */
    Ptr appHandle;
    /**< application handle which will be returned with callback */

}DDC_SpiDriverObject ;

/**
 * \brief    Default SSC SPI OBJECT values
 */
#define DDC_SPI_DRIVER_OBJECT_DEFAULT           \
        {                                       \
         /*spiObj*/             NULL,           \
        /* callback*/           NULL,           \
        /* app handle*/         NULL,           \
        }

/************< Functions implemented by DDC ************/
/**
  * \brief  Interrupt handler for SPI Device
  *         It will check the following errors like bit error, desync error
  *         parity error, overrun error. Data transfer either read or write
  *         is done on RX INT flag only.
  * \param  spiObj      [IN]    Spi object hadle.
  */
void DDC_spiIntrHandler(DDC_SpiObject *spiObj);

#ifdef __cplusplus
}
#endif

#endif  /*_DDC_SPI_H_ */
