/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/*!
 *  \file   psp_spiApi.h
 *
 *  \brief  Wrapper APIs called by DDA to access DDC
 *
 *  This file contains APIs definition of APIs that are wrapper APIs for DDC
 *  layer. These APIs will be called by DDA layer.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author     PSP Team
 *
 *  \version    1.0 - Created               Shailesh Kumar Sharma
 */

#ifndef _PSP_SPIAPI_H_
#define _PSP_SPIAPI_H_

#ifdef __cplusplus
extern "C" {
#endif


/**
  * \brief  Function pointer passed while registering interrupt handler
  */
typedef void (* pspSpiIsr)(Ptr iohandle);

/**
  *\brief CallBack function for SPI. Please note that this is not supported
  *       currently as driver does not support async mode of operation.
  *
  *    This function will be called when any transaction
  *    has been completed when driver is operating in async mode.
  *
  * \param  appHandle   [OUT]   Registered by application and will be returned
  *                             along with call back.
  * \param  event       [OUT]   Event associated for which call back has been
  *                             called
  * \param  Param       [OUT]   Any parameter returned from driver
  */
typedef void (*PSP_SpiAppCallback)(Ptr appHandle,Int event, Ptr param);


/**
  *\brief Initialize a given SPI driver (instance)
  *
  *       Typically, software bookkeeping functions are performed in this call.
  *       Memory for device instance specific data structures may be allocated
  *       and initialized. Configuration information may be passed in the call
  *       and initialization based upon this information is done.
  *
  * \param  instanceId  [IN]    SPI instance number
  * \param  initConfig  [IN/OUT]Initialization configuration
  *
  * \return    PSP_SOK or PSP Error code
  */
PSP_Result    PSP_spiCreate(Uint32 instanceId ,const PSP_SpiConfig *initConfig);


/**
  *\brief Delete a given SPI Driver (instance)
  *       The instance of SPI is deleted in software.If there are open handles
  *       then they are invalidated.
  *
  * \param  instanceId  [IN]    SPI instance number
  *
  * \return     PSP_SOK or PSP Error code
  */
PSP_Result PSP_spiDelete(Uint32   instanceId);


/**
  *\brief Open Instance of the SPI device
  *
  *       This function prepares the device hardware for data transfers and
  *       usage by the upper layer driver software. The DDA layer is expected
  *       to install the ISR and enable it only after the completion of this
  *       call. The function prepares the driver instance for data transfers
  *       and returns an handle to the driver instance.
  *
  * \note   The driver can be opened multiple times
  *
  * \param  instanceId   [IN]    SPI instance number.
  * \param  callback     [IN]    Callback function for  events notification
  * \param  appHandle    [IN]    This appHandle will be provided by user and will
  *                             be returned along with call back
  *
  * \return Valid PSP_Handle  if success<br>
  *         NULL if any error<br>
  */
PSP_Handle  PSP_spiOpen(
                        Uint32                 	instanceId ,
                        PSP_SpiAppCallback     	callback ,
                        Ptr                    	appHandle
                       );



/**
  *\brief Close Instance of the SPI device
  *
  *      This function closes the device for data transfers and usage by the
  *      upper layer driver software. The hardware is programmed to stop/abort data
  *      transfer (depending upon the type of device and its specifics) and the
  *      device ISR is "disabled" by the upper layer driver software after the
  *      completion of this call. After the successful completion of this call, the
  *      device cannot perform any data IO.
  *
  * \param  handle      [IN]    SPI Driver Instance Handle
  *
  * \return    PSP_SOK or PSP Error code
  */
PSP_Result    PSP_spiClose(PSP_Handle    handle);


/**
  *    \brief Read/write (single transaction) using spi
  *    This function read/writes single transaction data using the spi peripheral.
  *
  *    \param  handle      [IN]    SPI Driver instance handle
  *    \param  buffer      [IN]    Buffer (to read into or write from
  *    \param  bufLen      [IN]    Buffer length - number of bytes to transact
  *    \param  flags       [IN]    Flags to specify mode of operation
  *    \param  DataFormat  [IN]    DataFormat to be used
  *    \param  timeout     [IN]    Timeout for Spi transaction to complete
  *    \param  param       [IN]    Extra parameter for further use
  *
  *    \return    PAL_SOK or  Error code.
  */
Int PSP_spiTransfer (
                     PSP_Handle         handle,
                     Uint8             *buffer,
                     Uint32             bufLen,
                     Uint32             flags,
                     PSP_spiDataFormat  DataFormat,
                     Int                timeout,
                     Ptr                param
                    );



/**
  * \brief SPI transceive operation means Read/Write will happen
  *        parallaly.
  *
  * \param  handle      [IN]     SPI Driver instance handle.
  * \param  outBuffer   [IN]     Buffer (to read into or write from
  * \param  inBuffer    [IN]     Source buffer for tranceieve operation.
  * \param  bufLen      [IN]     Length of data to be transmitted
  * \param  chipSelect  [IN]     Chip select number
  * \param  dataFormat  [IN]     Data format register selection
  * \param  flags       [IN]     Flag for Read/Write of data
  * \param  timeout     [IN]     Timeout value.
  * \param  param       [IN/OUT] Additional param for future use.
  *
  * \return   Success code or suitable error code.
  *
  */
PSP_Result PSP_spiTransceive(
                      PSP_Handle        handle,
                      Uint8             *outBuffer,
                      Uint8             *inBuffer,
                      Uint32            bufLen,
                      Uint32            chipSelect,
                      PSP_spiDataFormat dataFormat,
                      Uint32            flags,
                      Int               timeout,
                      Ptr               param
					 );



/**
  *    \brief Read/write multiple transaction using spi peripheral.

  *      This function read/writes multiple transactions data using the spi
  *      pheripheral.Though nothing prevents the user from using multiple
  *      invocations of the single transaction transfer function, this function
  *      helps in making sure that all the transactions are completed in one go
  *      before another application gets hold on the spi bus
  *      (from the application perspective).
  *
  *    \param  handle      [IN]    SPI Driver instance handle
  *    \param  xfer        [IN]    SPI Transaction array - first element pointer
  *    \param  numXfer     [IN]    Number of transactions to be done
  *    \param  DataFormat  [IN]    DataFormat to be used
  *    \param  timeout     [IN]    Timeout for Spi transaction to complete
  *    \param  param       [IN]    Extra parameter for future use
  *
  *    \return   Number of transaction successfully completed<br>
  *              Error code.
  */

PSP_Result PSP_spiTransferMultiple(
                                   PSP_Handle          handle,
                                   PSP_spiTransaction *xfer,
                                   Uint32              numXfer,
                                   PSP_spiDataFormat   DataFormat,
                                   Int                 timeout,
                                   Ptr                 param
                                  );


/**
  *  \brief IO Ioctl for SPI Controller
  *
  *    This function provides ioctl functionality for SPI Controller
  *
  * \param  handle      [IN]    Spi Driver Instance Handle
  * \param  cmd         [IN]    IOCTL Command
  * \param  cmdArg      [IN]    Arguments (if any) in this case
  *                             PSP_SpiIoctlCommandArg can be passed
  * \param  param       [IN]    IOCTL specific parameter (driver specific).
  *
  * \return    PSP_SOK <br>
  *            PSP Error code
  */
PSP_Result  PSP_spiIoctl(
                         PSP_Handle       handle,
                         PSP_SpiIoctlCmd  cmd,
                         Ptr              cmdArg,
                         Ptr              param
                        );

#ifdef __cplusplus
}
#endif

#endif /*_PSP_SPI_API_*/
