/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/** \file    psp_tsip.h
 *
 *  \brief   TSIP interface definition header file
 *
 *  This file contains the definitions that are needed by the application to
 *  utilize the services of the TSIP device driver.
 *
 *  (C)    Copyright 2006,    Texas Instruments, Inc
 *
 *  \author    Platform Support Group
 *
 *  \version    0.1
 *
 *  <b>Compiler Switch Used</b>
 *
 *  <b>_DEBUG</b><br>
 *   This is used as a flag to compiler whether to include the debug statements
 *   inserted in the code into the final image. This flag helps to build DEBUG
 *   image of the program. For RELEASE images this is not passed to the compiler.
 *
 *  <b>CHIP_XXXX</b><br>
 *   The CSL layer is written in a common file for all the variants of a SOC.
 *   This flag differentiates the variant we are compiling for, for e.g.
 *   CHIP_DM648, and the CSL definitions for that variant appropriately gets
 *   defined for register base addresses, num of ports of a peripheral etc.
 *
 *  <b>TSIP_INSTRUMENTATION_ENABLED</b><br>
 *   This option is passed to the compiler to include the instrumentation code
 *   parts into the final image/lib of the program. This helps build the
 *   iRelease/iDebug versions of the image/lib with a common code base.
 *
 *  <b>PSP_TSIP_USE_FRAME_ISR</b><br>
 *   If this macro is enabled, the Frame interrupt will be enabled and ISR gets
 *   control at the end of each frame completion. Generally this is not
 *   recommended because of this ISR overhead the driver may miss to program the
 *   DMA properly.
 *
 *  <b>PSP_TSIP_DEBUG</b><br>
 *   They are the macros used to print debug messages. These expand a valid print
 *   statement.
 *
 */

#ifndef _PSP_TSIP_H_
#define _PSP_TSIP_H_

/* Specific Includes */
#include <ti/sdo/pspdrivers/common/psp_common.h>
#include <ti/sdo/pspdrivers/soc/c6452/dsp/tistdtypes.h>

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * \brief    Number of TSIP instances in this SoC
 */
#define PSP_TSIP_NUM_PORTS                 (2u)

/**
 * \brief    Number of serial links per instance in this SoC
 *
 *  Maximum number of links supported per direction(tx/rx). Assumption
 *  num of tx links = num of rx links as per TSIP SoC design.
 */
#define PSP_TSIP_NUM_LINKS                 (2u)

/**
 * \brief    Total number of timeslot support
 *
 * Maximum number of timeslots supported by a TSIP instance
 * per direction. Each link supports 128 time slots.
 */
#define PSP_TSIP_NUM_TIMESLOTS             (PSP_TSIP_NUM_LINKS * 128u)

/**
 * \brief    Maximum simulatenous IOP requests supported by the TSIP driver
 */
#define PSP_TSIP_MAX_IOP_REQ               (10u)

/**
 * \brief    Maximum number of superframes supported per IOP request for both
 *           A and B context.
 */
#define PSP_TSIP_MAX_IOP_SFRAME            (10u)


/**
 * \brief    TSIP Channel IOCTL commands <br>
 *
 * This enumeration contains TSIP IOCTL commands.
 *
 */
typedef enum PSP_TsipIoctlCmd_t
{

    PSP_TSIP_IOCTL_GET_PID,
    /**< Gets the Peripheral Version ID of the TSIP.
         Command Argument: PID structure<PSP_tsipPID>                     */

    PSP_TSIP_IOCTL_START_CHANNEL,
    /**< Starts the TDMU-DMATCU channel operation                         */

    PSP_TSIP_IOCTL_STOP_CHANNEL,
    /**< Stops the TDMU-DMATCU  channel operation immediately.
         However the TSIP HW will disable the channel only after
         completion of the current frame.                                 */

    PSP_TSIP_IOCTL_STOP_CHANNEL_EXT,
    /**< Stops the TDMU-DMATCU operation only after completion
         of the current super frame.                                      */

    PSP_TSIP_IOCTL_MODIFY_BITMAP_A,
    /**< Modifies Bitmap configuration-A to select different set of
         timeslots for different PCM formats: 8-bit linear, A and U law.
         Note: Modifying bitmap-A when bitmap-A is active will return
         error (Invalid state). So the application can first check which 
         bitmap is active by using GET_ACTIVE_BITMAP command before 
         invoking this command.
         Command Argument: Bitmap Configuration<PSP_tsipBitmap>           */

    PSP_TSIP_IOCTL_MODIFY_BITMAP_B,
    /**< Modifies Bitmap configuration-B to select different set of
         timeslots for different PCM formats: 8-bit linear, A and U law.
         Note: Modifying bitmap-B when bitmap-B is active will return
         error (Invalid state). So the application can first check which 
         bitmap is active by using GET_ACTIVE_BITMAP command before 
         invoking this command.
         Command Argument: Bitmap Configuration<PSP_tsipBitmap>           */

    PSP_TSIP_IOCTL_GET_ERROR_STATS,
    /**< Gets the driver maintained error statistics counters.
         Command Argument: Error Statistics structure<PSP_tsipErrStats>   */

    PSP_TSIP_IOCTL_CLEAR_ERROR_STATS,
    /**< Clears the driver maintained error statistics counters.          */

    PSP_TSIP_IOCTL_GET_ACTIVE_CONTEXT,
    /**< Returns the current active context (A or B or None).
         Command Argument: Active Context enum<PSP_TsipActiveContext>     */

    PSP_TSIP_IOCTL_GET_ACTIVE_BITMAP,
    /**< Returns the current active bitmap (A or B or None).
         Command Argument: Active Bitmap enum<PSP_TsipActiveBitmap>       */

    PSP_TSIP_IOCTL_SET_ACTIVE_BITMAP,
    /**< Set the active Bitmap(channel config) to A or B
         Command Argument: CID structure for AorB<PSP_tsipCID>            */

    PSP_TSIP_IOCTL_SETUP_SIU,
    /**< Setup/Configure SIU module
        Command Argument:  SIU HW setup structure<PSP_tsipSiuConf>        */

    PSP_TSIP_IOCTL_SETUP_TDMU,
    /**< Setup/Configure TDMU-DMATCU module
        Command Argument:  TDMU HW setup structure<PSP_tsipTdmuConf>      */

    PSP_TSIP_IOCTL_GET_HW_SETUP,
    /**< Returns the TSIP entire hardware setup.
         Command Argument: TSIP HW setup structure<PSP_tsipHwSetup>       */

    PSP_TSIP_IOCTL_RESET_SIU,
    /**< Resets(Soft) SIU module. After reset SIU module needs to be
         reconfigured by the application using SETUP SIU ioctl            */

    PSP_TSIP_IOCTL_RESET_TDMU,
    /**< Resets(Soft) TDMU and DMATCU modules. After reset TDMU-DMATCU
         modules need to be reconfigured by the application using SETUP
         TDMU ioctl                                                       */

    PSP_TSIP_IOCTL_SET_SEM_TIMEOUT
    /**< Change the timeout value used in semaphore pending for flush
         operation. Command Argument:  Timeout values<Int32>              */

} PSP_TsipIoctlCmd;

/**
 * \brief    TSIP Channel Error Codes <br>
 *
 * This enumeration contains various TSIP Error Codes.
 *
 */
typedef enum PSP_TsipErrorCode_t
{
    PSP_TSIP_ERROR_RX_ORUN_BEGIN,
    /**< Receive overrun at beginning of the frame                        */

    PSP_TSIP_ERROR_RX_ORUN,
    /**< Receive overrun during the frame                                 */

    PSP_TSIP_ERROR_RX_BITMAP,
    /**< Receive overrun - Enabled bitmap greater than size               */

    PSP_TSIP_ERROR_RX_INVALID_SIZE,
    /**< Receive size less than 8                                         */

    PSP_TSIP_ERROR_TX_CONFIG_MISMATCH,
    /**< Transmit configuration ID(CID) mismatch                          */

    PSP_TSIP_ERROR_TX_URUN_BEGIN,
    /**< Transmit under run at beginning of the frame                     */

    PSP_TSIP_ERROR_TX_URUN,
    /**< Transmit under run during the frame                              */

    PSP_TSIP_ERROR_TX_BITMAP,
    /**< Transmit under run - Enabled bitmap greater than size            */

    PSP_TSIP_ERROR_TX_INVALID_SIZE,
    /**< Transmit size less than 8                                        */

    PSP_TSIP_ERROR_CBA_READ,
    /**< CBA read status error                                            */

    PSP_TSIP_ERROR_CBA_WRITE,
    /**< CBA write status error                                           */

    PSP_TSIP_ERROR_UNKNOWN
    /**< Unknown error (E.g. TX error in the context of RX channel,etc)   */

} PSP_TsipErrorCode;



/**
 * \brief    TSIP Channel current active context <br>
 *
 *  This is used as an argument for PSP_TSIP_IOCTL_GET_ACTIVE_CONTEXT ioctl 
 *  command.
 */
typedef enum PSP_TsipActiveContext_t
{
    PSP_TSIP_ACTIVE_CONTEXT_NONE = 0x0,
    /**< Both A and B contexts are inactive                               */

    PSP_TSIP_ACTIVE_CONTEXT_A    = 0x1u,
    /**< A context is active                                              */

    PSP_TSIP_ACTIVE_CONTEXT_B    = 0x2u
    /**< B context is active                                              */

} PSP_TsipActiveContext;


/**
 * \brief    TSIP Channel current active bitmap <br>
 *
 * This is used as a status argument for PSP_TSIP_IOCTL_GET_ACTIVE_BITMAP  
 * ioctl command and used as a member of PSP_TsipCID which is used as 
 * a control argument for PSP_TSIP_IOCTL_SET_ACTIVE_BITMAP ioctl command. 
 * 
 */
typedef enum PSP_TsipActiveBitmap_t
{
    PSP_TSIP_ACTIVE_BITMAP_NONE  = 0x0,
    /**< Both A and B bitmaps are inactive                                */

    PSP_TSIP_ACTIVE_BITMAP_A     = 0x1u,
    /**< A bitmap is active                                               */

    PSP_TSIP_ACTIVE_BITMAP_B     = 0x2u
    /**< B bitmap is active                                               */

} PSP_TsipActiveBitmap;


/**
 * \brief    TSIP clock selection <br>
 *
 */
typedef enum PSP_TsipClock_t
{
    PSP_TSIP_CLK_REDUNDANT  = 0x0,
    /**< Common clock for TX and RX                                       */

    PSP_TSIP_CLK_DUAL       = 0x1u
    /**< Seperate clock for TX and RX                                     */

} PSP_TsipClock;

/**
 * \brief    TSIP clock(CLK) source and frame sync(FS) source selection <br>
 *
 */
typedef enum PSP_TsipClkFsSrc_t
{
    PSP_TSIP_CLK_FS_SRC_A    = 0x0,
    /**< TCLK_A and TFS_A are used                                        */

    PSP_TSIP_CLK_FS_SRC_B    = 0x1u
    /**< TCLK_B and TFS_B are used                                        */

} PSP_TsipClkFsSrc;

/**
 * \brief    TSIP transmit output disable state
 *
 */
typedef enum PSP_TsipTxDisState_t
{
    PSP_TSIP_TXDIS_STATE_HIGHZ = 0x0,
    /**< Transmit disable output state is High impedance                 */

    PSP_TSIP_TXDIS_STATE_LOW   = 0x2u,
    /**< Transmit disable output state is driven low                     */

    PSP_TSIP_TXDIS_STATE_HIGH  = 0x3u
    /**< Transmit disable output state is driven high                    */

}PSP_TsipTxDisState;

/**
 * \brief    TSIP transmit output enable delay
 *
 */
typedef enum PSP_TsipTxEnaDly_t
{
    PSP_TSIP_TXENA_DLY_ZERO    = 0x0,
    /**< No added delay                                                  */

    PSP_TSIP_TXENA_DLY_ONEHALF = 0x1u
    /**< Output enable is delayed by one half of serial clk period       */

}PSP_TsipTxEnaDly;

/**
 * \brief    TSIP signal polarity (clock or fsync) selection <br>
 *
 * \note    Active low and Rising Edge AND Active high and Falling edge
 *          having similar config values understood by TSIP.
 *
 */
typedef enum PSP_TsipPolarity_t
{
    PSP_TSIP_POLARITY_ACTIVE_LOW  = 0x0,
    /**< Active low signal                                                */

    PSP_TSIP_POLARITY_RISING_EDGE = 0x0,
    /**< Rising Edge                                                      */

    PSP_TSIP_POLARTIY_ACTIVE_HIGH = 0x1u,
    /**< Active high signal                                               */

    PSP_TSIP_POLARTIY_FALLING_EDGE= 0x1u
    /**< Falling Edge                                                     */

} PSP_TsipPolarity;


/**
 * \brief    TSIP data rate selection <br>
 *
 */
typedef enum PSP_TsipDataRate_t
{
    PSP_TSIP_DATA_RATE_8_192    = 0x0,
    /**< 8.192 Mbps data rate                                             */

    PSP_TSIP_DATA_RATE_16_384   = 0x1u
    /**< 16.384 Mbps data rate                                            */
} PSP_TsipDataRate;

/**
 * \brief    TSIP clock mode timing selection <br>
 *
 */
typedef enum PSP_TsipClkMode_t
{
    PSP_TSIP_CLK_MODE_2X    = 0x0,
    /**< Double data rate timing                                          */

    PSP_TSIP_CLK_MODE_1X    = 0x1u
    /**< Single data rate timing                                          */

} PSP_TsipClkMode;


/**
 * \brief    TSIP endianness selection for data packed/unpacked
 * in channel buffer(CB)
 *
 */
typedef enum PSP_TsipEndian_t
{
    PSP_TSIP_LITTLE_ENDIAN  = 0x0,
    /**< Data packed/unpacked in CB is in little endian format            */

    PSP_TSIP_BIG_ENDIAN     = 0x1u
    /**< Data packed/unpacked in CB is in big endian format               */

} PSP_TsipEndian;

/**
 * \brief    TSIP interrupt assertion selection for frame/sframe <br>
 *
 */
typedef enum PSP_TsipIntSelect_t
{
    PSP_TSIP_INT_XFER_ACK_ONLY    = 0x0,
    /**< Interrupt asserted on transfer acknowledge only                  */

    PSP_TSIP_INT_DELAY_CNT_ONLY   = 0x1u,
    /**< Interrupt asserted on delay count only                           */

    PSP_TSIP_INT_EITHER           = 0x2u,
    /**< Interrupt asserted on either transfer acknowledge or delay count */

    PSP_TSIP_INT_BOTH             = 0x3u
    /**< Interrupt asserted on both transfer acknowledge and delay count  */

} PSP_TsipIntSelect;


/**
 * \brief    TSIP PCM data type <br>
 *
 * \note     Values of this enum elements should not be changed, since they
 *           represent the actual register configuration values understood
 *           by TSIP.
 *
 */
typedef enum PSP_TsipPcmType_t
{
    PSP_TSIP_PCM_DISABLED     = 0x0,
    /**< Timeslot Disabled                                                */

    PSP_TSIP_PCM_8_BIT_LINEAR = 0x1u,
    /**< Timeslot is 8-bit linear PCM encoded                             */

    PSP_TSIP_PCM_U_LAW        = 0x2u,
    /**< Timeslot is U-law PCM encoded                                    */

    PSP_TSIP_PCM_A_LAW        = 0x3u
    /**< Timeslot is A-law PCM encoded                                    */

}PSP_TsipPcmType;


/**
 * \brief    TSIP Loop back mode <br>
 *
 */
typedef enum PSP_TsipLoopBack_t
{
    PSP_TSIP_LOOP_BACK_NONE  = 0x0,
    /**< No loopback mode                                                 */

    PSP_TSIP_LOOP_BACK_DLB   = 0x1u,
    /**< Data loop back                                                   */

    PSP_TSIP_LOOP_BACK_LLB   = 0x3u
    /**< Link loop back                                                   */

} PSP_TsipLoopBack;

/**
 * \brief    TSIP Test clock selection(Only applicable in Loop back mode)
 *
 */
typedef enum PSP_TsipTestClock_t
{
    PSP_TSIP_TEST_CLK_EXTERNAL  = 0x0
    /**< Clock and FrameSync are given from external source               */
} PSP_TsipTestClock;

/**
 * \brief    TSIP driver returned error details structure <br>
 *
 * This structure is used as the command argument for the global
 * error call back function (PSP_tsipErrCallback). In case of error
 * the driver passes the error details through this structure from
 * the driver to the application from error interrupt ISR context.<br>
 */
typedef struct PSP_tsipError_t
{
    PSP_TsipErrorCode    errCode;
    /**< TSIP driver error codes                                          */
    Uint32               errInfo;
    /**< Additional error information associated with the errCode. Refer
         TSIP specs for details about this.                               */
} PSP_tsipError;

/**
 * \brief    Global Error call back function pointer (per tx/rx channel)
 */
typedef void (*PSP_tsipErrCallback)(PSP_tsipError err, Ptr arg);


/**
 * \brief    Frame/Supeframe call back function pointer(per tx/rx channel)
 *
 */
 typedef void (*PSP_tsipFrmCallback)(Ptr arg);

/**
 * \brief    TSIP peripheral ID <br>
 *
 * This structure is used as the command argument for
 * PSP_TSIP_IOCTL_GET_PID command.<br>
 *
 */
typedef struct PSP_tsipPID_t
{
    Uint16     moduleID;
    /**< Module ID of the TSIP                                            */

    Uint8      majorVer;
    /**< Major version ID                                                 */

    Uint8      minorVer;
    /**< Minor version ID                                                 */

} PSP_tsipPID;


/**
 * \brief    TSIP Channel Configuration Identifier (CID)<br>
 *
 * This structure is used as the command argument for
 * PSP_TSIP_IOCTL_SET_ACTIVE_BITMAP command.<br>
 *
 */
typedef struct PSP_tsipCID_t
{
    PSP_TsipActiveBitmap   bitmap;
    /**< Active bitmap selection (A or B)                                         */

    Uint8                  CID;
    /**< New channel configuration identifier value. Least significant
         bit should be zero if A is going to be selected and one if B
         is going to be selected.                                          */

} PSP_tsipCID;


/**
 * \brief    TSIP HW setup: TX SIU configuration
 *
 * This structure contains TX SIU configurations  clock related control,
 * data rate config, polarity config and frame config  parameters.
 *
 */
typedef struct PSP_tsipTxSiuConf_t
{
    PSP_TsipClkFsSrc           clkSrc;
    /**< Selection of clock source and frame sync source for TX           */

    Uint16                     dDly;
    /**< Data delay(14-bits: 0-16383) configuration for TX                */

    PSP_TsipTxDisState         disState;
    /**< Transmit output disable state                                    */

    PSP_TsipTxEnaDly           enaDly;
    /**< Transmit output enable delay. Valid only if disState is High
         impedence                                                        */

    PSP_TsipPolarity           fSyncP;
    /**< Selection of TX frame sync polarity                              */

    PSP_TsipPolarity           fClkP;
    /**< Selection of TX frame clock polarity                             */

    PSP_TsipPolarity           dClkP;
    /**< Selection of TX data clock polarity                              */

    PSP_TsipDataRate           dRate;
    /**< Selection of Data rate for TX                                    */

    PSP_TsipClkMode            clkMode;
    /**< Selection of clock mode: 1X/2X timing for TX                     */

    Uint8                      fCnt;
    /**< Frame count: Number of frames(0-255)+1 in a super-frame for TX   */

    Uint8                      fSize;
    /**< Frame size in number of logical timeslots(0-127)+1 for TX        */

} PSP_tsipTxSiuConf;


/**
 * \brief    TSIP HW setup: RX SIU configuration
 *
 * This structure contains RX SIU configurations  clock related control,
 * data rate config, polarity config and frame config  parameters.
 *
 */
typedef struct PSP_tsipRxSiuConf_t
{
    PSP_TsipClkFsSrc           clkSrc;
    /**< Selection of clock source and frame sync source for RX           */

    Uint16                     dDly;
    /**< Data delay(14-bits: 0-16383) configuration for RX                */

    PSP_TsipPolarity           fSyncP;
    /**< Selection of RX frame sync polarity                              */

    PSP_TsipPolarity           fClkP;
    /**< Selection of RX frame clock polarity                             */

    PSP_TsipPolarity           dClkP;
    /**< Selection of RX data clock polarity                              */

    PSP_TsipDataRate           dRate;
    /**< Selection of Data rate for RX                                    */

    PSP_TsipClkMode            clkMode;
    /**< Selection of clock mode: 1X/2X timing for RX                     */

    Uint8                      fCnt;
    /**< Frame count: Number of frames(0-255)+1 in a super-frame for RX   */

    Uint8                      fSize;
    /**< Frame size in number of logical timeslots(0-127)+1 for RX        */

} PSP_tsipRxSiuConf;


/**
 * \brief    TSIP HW setup: SIU configuration
 *
 * This structure contains the complete SIU configurations: Global config
 * TX SIU config, RX SIU config  and loopback parameters<br>
 *
 * This structure is also used as the command argument for
 * PSP_TSIP_IOCTL_SETUP_SIU command.<br>
 *
 */
typedef struct PSP_tsipSiuConf_t
{
    PSP_TsipClock              clkSel;
    /**< Selection of clock mode: Redundant or dual                       */

    PSP_tsipTxSiuConf          tx;
    /**< Tranmit SIU section configuration                                */

    PSP_tsipRxSiuConf          rx;
    /**< Receive SIU section configuration                                */

    PSP_TsipLoopBack           loopBack;
    /**< Selection of Loop Back mode                                      */

    PSP_TsipTestClock          testClk;
    /**< Selection of Test Clock source for LoopBack mode testing         */

} PSP_tsipSiuConf;

/**
 * \brief    TSIP HW setup: TDMU-DMATCU configuration
 *
 * This structure contains TDMU and DMATCU control and configuration<br>
 *
 * This structure is also used as the command argument for
 * PSP_TSIP_IOCTL_SETUP_TDMU command.<br>
 *
 */
typedef struct PSP_tsipTdmuConf_t
{
    PSP_TsipEndian             endian;
    /**< TDMU: Endianness for data packed/unpacked in the chan buffer     */

    Uint8                      maxPrio;
    /**< DMATCU: Max transfer priority[0(highest)-7(lowest)]              */

    Uint8                      prio;
    /**< DMATCU: Transfer priority[0(highest)-7(lowest)]                  */

    PSP_TsipIntSelect          txFint;
    /**< DMATCU: TX frame interrupt selection                             */

    PSP_TsipIntSelect          rxFint;
    /**< DMATCU: RX frame interrupt selection                             */

    PSP_TsipIntSelect          txSint;
    /**< DMATCU: TX super frame interrupt selection                       */

    PSP_TsipIntSelect          rxSint;
    /**< DMATCU: RX super frame interrupt selection                       */

    Uint8                      txFintDly;
    /**< DMATCU: Tx transfer frame interrupt delay                        */

    Uint8                      rxFintDly;
    /**< DMATCU: Rx transfer frame interrupt delay                        */

} PSP_tsipTdmuConf;


/**
 * \brief    TSIP HW setup configuration
 *
 * This structure is the master TSIP HW setup<br>
 *
 * This structure is also used as the command argument for
 * PSP_TSIP_IOCTL_GET_HW_SETUP command.<br>
 *
 */
typedef struct PSP_tsipHwSetup_t
{
    PSP_tsipSiuConf            siu;
    /**< SIU configurations                                               */

    PSP_tsipTdmuConf           tdmu;
    /**< TDMU-DMATCU configurations                                       */

} PSP_tsipHwSetup;

/**
 * \brief    Device parameters to initialize device(port) object
 *
 * This structure passes a handle to setup the TSIP
 * device registers. The application provides the
 * TSIP HW Setup structure to initialize the device.
 *
 */
typedef struct PSP_tsipHwSetup_t PSP_tsipDevParams;

/**
 * \brief    TSIP bitmap configuration
 *
 * This structure contains TSIP Bitmap configuration for a context either A
 * or B for all timeslots. This is used during channel creation as well
 * as changing the bitmap dynamically using modify bitmap IOCTL command.
 *
 */
typedef struct PSP_tsipBitmap_t
{
    PSP_TsipPcmType  pcmType[PSP_TSIP_NUM_TIMESLOTS];
    /**< Bitmap configuration for all timeslots for a context(A or B)     */

} PSP_tsipBitmap;

/**
 * \brief    TSIP DMA channel frame buffer configuartion
 *
 * This structure contains TSIP DMA channel frame buffer configurations for
 * a context either A or B. This consists of configuration elements
 * base address, frame allocation, frame size and frame count. This is used
 * during channel creation as well as dynamic configuration using IOCTL
 *
 */
typedef struct PSP_tsipDmaConfig_t
{
    Uint32        *baseAddr;
    /**< Pointer to base address of the framebuffer. Note: TSIP DMA
         performs transfer in terms 32-bit elements to the memory buffer.
         So it is recommendedthat this address to be aligned on 32-bit(4 byte)
         boundary i.e two least significant bits to be zero.                           */

    Uint32        frameAlloc;
    /**< Frame buffer allocation. Should be having the maximum frame
         size. frameAlloc should be greater than frameSize. If application
         specifies frameAlloc < frameSize driver shall return the error.
         See TSIP SPECS for more details about how the memory buffers are
         organised                                                        */

    Uint32        frameSize;
    /**< Size of each frame. Frame size in RX side, doesnt have any
         meaning. But it has been given to maintain coherency with TX     */

    Uint32        frameCnt;
    /**< Number of frames in the buffer                                   */

} PSP_tsipDmaConfig;



/**
 * \brief    Channel parameters to initialize TX/RX channel object
 *
 * This structure consist of channel configuration related parameters.
 * Used during I/O channel creation.
 *
 */
typedef struct PSP_tsipChanParams_t
{
    PSP_tsipBitmap       bmA;
    /**< Bitmap configuration for A context.                              */

    PSP_tsipBitmap       bmB;
    /**< Bitmap configuration for B context.                              */

    Uint8                aCID;
    /**< Channel Identfier (CID) for "A" bitmap/context. lsb should be 0  */

    Uint8                bCID;
    /**< Channel Identfier (CID) for "B" bitmap/context. lsb should be 1  */

    PSP_tsipErrCallback  errCb;
    /**< Pointer to error callback function                               */

    Ptr                  eCbArg;
    /**< Error callback  argument                                         */

    PSP_tsipFrmCallback  frmCb;
    /**< Frame interrupt callback function pointer. If registered, it will
         be called every frame interval (from frame interrupt). It can be
         used for house-keeping purpose by the application. The assertion
         type of frame interrupt depends on the txFint/rxFint
         (PSP_TsipIntSelect) param.                                       */

    PSP_tsipFrmCallback  sfrmCb;
    /**< Super frame interrupt callback function pointer. If registered,
         it will be called every super frame interval. It can be used for
         house-keeping purpose by the application. The assertion type of
         super frame interrupt depends on the txSint/rxSint
         (PSP_TsipIntSelect) param.                                       */

    Ptr                  fCbArg;
    /**< Frame interrupt callback argument                                */

    Ptr                  sCbArg;
    /**< Super Frame interrupt callback argument                          */

} PSP_tsipChanParams;


/**
 * \brief    IO submit requests parameters
 *
 * The IO parameters structure is used to pass the TSIP DMA configuration
 * for both A and B context in a single IOP request from the application.
 *
 * \note TSIP driver dynamically configures the DMA channel parameters
 *  with the given A and B context buffer configurations specified in the
 *  IO submit request packet submitted by the application. The number of
 *  super-frames in an IO submit request should be an even number. Because
 *  it is required to give the super-frame buffer configurations for both
 *  A and B contexts in a given IOP. For e.g if user wants to have 4 super
 *  frame in an IO submit request array of PSP_tsipIoParams of size two can
 *  be passed to the GIO submit call. See user guide for more details.
 */
typedef struct PSP_tsipIoParams_t
{
    PSP_tsipDmaConfig    dmaA;
    /**< DMA channel configuration for A context                          */

    PSP_tsipDmaConfig    dmaB;
    /**< DMA channel configuration for B context                          */

}PSP_tsipIoParams;


/**
 * \brief    Error Statistics information structure
 *
 * This structure maintains error counters to provide error statistics
 * information for TX or RX channel.
 *
 */
typedef union PSP_tsipErrStats_t
{
    /**
     * \brief Receive channel error statistics counters
     */
    struct rxStats_t
    {
        Uint32    ovRunCnt;
        /**< Number of times receive overrun error occurred               */

        Uint32    invFrameCnt;
        /**< Number of rx invalid frames error received(size<8)           */

        Uint32    cbaReadCnt;
        /**< CBA read status error counter                                */

        Uint32    cbaWriteCnt;
        /**< CBA write status error counter                               */

        Uint32    dummy;
        /**< Dummy counter to match with tx struct                        */

        Uint32    unKnown;
        /**< Unknown error for this chan.(E.g. TX err is notifd. in RX )  */

    }rx;

    /**
     * \brief Transmit channel error statistics counters
     */
    struct txStats_t
    {
        Uint32    unRunCnt;
        /**< Number of times tranmit underrun error occurred              */

        Uint32    invFrameCnt;
        /**< Number of tx invalid frames error encountered(size<8)        */

        Uint32    cbaReadCnt;
        /**< CBA read status error counter                                */

        Uint32    cbaWriteCnt;
        /**< CBA write status error counter                               */

        Uint32    invConfCnt;
        /**< Number of times invalid config. ID encountered               */

        Uint32    unKnown;
        /**< Unknown error for this chan.(E.g. RX err is notifd. in TX )  */

    }tx;

} PSP_tsipErrStats;


#ifdef __cplusplus
}
#endif

#endif /* _PSP_TSIP_H_ */

/*!
*! Revision History
*! ================
*! 04-Nov-2008    Imt: Removing the references to the internal clock.
*! 12-Jun-2007    JP : Removed Pre-silicon related info and code segments
*! 31-May-2007    JP : Re-organisation of structures(Name changes, addition
                       of configuration parameters, IOCTLs(RESET,CONFIG)
*! 20-Apr-2007    JP : Applied few changes after tested in Tomahawk
*! 15-Mar-2007    JP : Added Modify Bitmap, Set active bitmap IOCTLs
*! 12-Jan-2007    JP : Added LoopBack mode config param in the SIU config.
*! 13-Dec-2006    JP : Removed PSP API prototypes from this file
*! 21-Nov-2006    JP : Combined psp read/write into psp submit
*! 19-Nov-2006    JP : Added Error codes
*! 13-Nov-2006    JP : Added PSP IO params structure;
*!                     Modified struct/enum declaration standard ( braces at
*!                     the next line, name for all enum/struct with _t suffix);
*!                     Added PSP fns prototype;
*! 02-Nov-2006    JP : Added TSIP PSP structures
*! 31-Oct-2006    JP : Initial Creation (0.1)
*! */

