/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/** \file    dda_tsipBios.c
 *
 *  \brief   Mini-driver(DDA Layer)for the TSIP.
 *
 *  This file implements the TSIP mini-driver to be used with DSP-BIOS.
 *
 *  (C)    Copyright 2006,    Texas Instruments, Inc
 *
 *  \author    Platform Support Group
 *
 *  \version    0.1
 *
 */

/*
 * Header includes
 */
#include <ecm.h>
#include <c64.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osProtect.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osMem.h>
#include <ti/sdo/pspdrivers/drivers/tsip/src/dda_tsipBios.h>
#include <ti/sdo/pspdrivers/soc/c6452/dsp/soc.h>

#ifdef TSIP_INSTRUMENTATION_ENABLED
#include <ti/sdo/pspdrivers/pal_os/bios/psp_log.h>
#endif

/*********** local(static) function prototypes ************************/
static Int tsip_mdBindDev(Ptr *devp, Int devId, Ptr devParams);
static Int tsip_mdUnBindDev(Ptr devp);
static Int tsip_mdCreateChan(
                    Ptr                 *chanp,
                    Ptr                 devp,
                    String              name,
                    Int                 mode,
                    Ptr                 chanParams,
                    IOM_TiomCallback    cbFxn,
                    Ptr                 cbArg
                    );
static Int tsip_mdDeleteChan(Ptr chanp);
static Int tsip_mdControlChan(Ptr chanp, Uns cmd, Ptr arg);
static Int tsip_mdSubmitChan(Ptr chanp, IOM_Packet *packet);

static void  tsip_mdCallBack(Ptr chanp, PSP_tsipIoRequest const *ioReq);
static Int32 tsip_getIomErrCode(PSP_Result pspErrCode);


/**
 * \brief    Public TSIP IOM interface table
 *
 * This table holds the interface functions exposed by the mini-driver to
 * the GIO layer.
 */

/* Public DDA interface table */
IOM_Fxns tsipMiniDrvFxns =
{
    &tsip_mdBindDev,
    &tsip_mdUnBindDev,
    &tsip_mdControlChan,
    &tsip_mdCreateChan,
    &tsip_mdDeleteChan,
    &tsip_mdSubmitChan,
};


/**********************************************************
 * Public Functions Definitions
 **********************************************************/

/**
 *  \brief    Registering interrupt handler
 *
 *   This function is invoked from the DDC layer in order for the
 *   DDC to register TSIP interrupts with the requested interrupt
 *   handler with the OS (BIOS) and enables the interrupt.
 *
 *  \param    chanp        [IN]    Pointer to channel
 *  \param    intNum       [IN]    HWI interrupt controller ID
 *  \param    evt          [IN]    event number
 *  \param    isrFunc      [IN]    ISR function pointer
 *
 *  \return
 *            PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result DDA_tsipRegisterInterrupt(
                                 Ptr            chanp,
                                 Uint32         intNum,
                                 Uint16         evt,
                                 PSP_tsipIsr    isrFunc
                                 )
{
    PSP_Result      result   = PSP_SOK;
    Uint32          cookie   = 0;
    ECM_Attrs       ecmattrs = ECM_ATTRS;

    if( (NULL == chanp) || (NULL == isrFunc) )
    {
        result = PSP_E_INVAL_PARAM;
    }
    else
    {
        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

        /*Enable the interrupt later*/
        ecmattrs.unmask = 0u;

        /*setup arg such that isr will get 'chan' as arg*/
        ecmattrs.arg    = (Arg)chanp;

        /* Plug the ISR with interrupt (event) number */
        ECM_dispatchPlug(evt, (ECM_Fxn)isrFunc, &ecmattrs);

        /* Enabling the event */
        ECM_enableEvent(evt);

        /*
         * Enabling the HWI_ID where corresponding ECM group is configured
         */
        C64_enableIER(intNum); 

        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

    }

    return result;
}

/**
 * \brief    Unregistering interrupt handler
 *
 * This function is invoked from DDC to unregister and disable interrupt
 * handler and interrupt cause respectively.
 *
 *  \param    intNum        [IN]    interrupt number
 *  \param    evt           [IN]    event number
 *
 *  \return   PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
/* ARGSUSED */
PSP_Result DDA_tsipUnRegisterInterrupt(Uint32 intNum, Uint16 evt)
{
    PSP_Result      result = PSP_SOK;
    Uint32          cookie = 0;

    PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

    /* Disable the event*/
    ECM_disableEvent(evt);

    PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

    return result;
}

/******************************************************
 * Private Functions Definitions
 ******************************************************/

/**
 *  \brief    Allocates and configures the TSIP port specified by devId
 *
 *  This function gets the default configuration for the port from the
 *  statically allocated default object, and intialize the TSIP hardware
 *  for this port specified by devId by calling PSP_TsipCreate()
 *
 * \param    devp         [OUT]    Pointer to hold allocated port object
 * \param    devId        [IN]     Device number (in this SoC, there are
 *                                  two TSIP present from 0 to 1)
 * \param    devParams    [IN]     Device parameter structure that supplies
 *                                  data to initialize the TSIP
 *
 * \return   IOM_COMPLETED if success<br>
 *           IOM_EBADARGS  if invalid arguments<br>
 *           IOM_EALLOC    if error in DDC layer<br>
 */
static Int tsip_mdBindDev(Ptr *devp, Int devId, Ptr devParams)
{
    Int32                 result     = IOM_COMPLETED;
    TSIP_portHandle       pPort      = NULL;
    PSP_tsipDevParams     *pDevParam = NULL;
    PSP_Result            status     = PSP_SOK;

    /* TSIP port object allocation for all instances */
    static TSIP_portObj TsipPort[PSP_TSIP_NUM_PORTS] = {TSIP_DDA_PORT_DEFAULT,
                                                        TSIP_DDA_PORT_DEFAULT};

    if( (NULL != devParams) && (devId < PSP_TSIP_NUM_PORTS) )
    {
        pDevParam = (PSP_tsipDevParams *)devParams;

        /* Get the default Port configuration for this instance */
        pPort = &TsipPort[devId];

        /* if the port object is not in use */
        if( FALSE == pPort->inUse )
        {
            /* Store devId into PortObject */
            pPort->portNum = devId;

            /* Store TSIP hw setup(device params) */
            PAL_osMemCopy(
                      (Ptr)&pPort->hwSetup,
                      (Ptr)pDevParam,
                      sizeof(PSP_tsipDevParams)
                      );                     

            /* Invoke DDC layer fn to initilize this port */
            status = PSP_tsipCreate(devId, pDevParam, pPort);

            /* Convert DDC to IOM error code */
            result = tsip_getIomErrCode(status);

            if( IOM_COMPLETED == result )
            {
                /* Place the allotted port handle to devp placeholder */
                *devp = pPort;

                /* Make this Resource as Used/Allotted */
                pPort->inUse = TRUE; 
            }
            else /* Some error from DDC layer */
            {
                *devp = NULL;
#ifdef _DEBUG
                TSIP_DEBUG("tsip_mdBindDev <Failed>");
#endif
            }
        }
        else /* Error: Port already allocated */
        {
            result = IOM_EALLOC;
        }
    }
    else
    {
        result = IOM_EBADARGS;
#ifdef _DEBUG
        TSIP_DEBUG("tsip_mdBindDev <Invalid Params>");
#endif

    }

    return result;
}

/**
 *  \brief    Closes the TSIP device from use.
 *
 *  This function unwinds the actions performed by mdbinddev.
 *  This is accomplished by calling PSP_tsipDelete()
 *
 * \param    devp         [IN]    Pointer to port object to be freed
 *
 * \return
 *           IOM_COMPLETED if success<br>
 *           IOM_ABORTED   if any error<br>
 */
static Int tsip_mdUnBindDev(Ptr devp)
{
    Int32             result = IOM_COMPLETED;
    PSP_Result        status = PSP_SOK;
    TSIP_portHandle   pPort  = NULL;

    if( NULL != devp )
    {
        pPort = (TSIP_portHandle)devp;

        pPort->inUse = FALSE; /* Free the port object, i.e it'll be avaialble
                                   for the next port bind request */

        /* Invoke DDC layer fn to un-initialize the port */
        status = PSP_tsipDelete(pPort->portNum);

        /* Clear the Port object resource */
        PAL_osMemSet(pPort,0,sizeof(TSIP_portObj));

        /* Assign instance id to invalid */
        pPort->portNum = (0xDEADu);

        /* Convert DDC to IOM error code */
        result = tsip_getIomErrCode(status);

    }
    else
    {
        result = IOM_EBADARGS;
#ifdef _DEBUG
        TSIP_DEBUG("tsip_mdUnBindDev <Invalid Params>");
#endif
    }

    return result;
}

/**
 *  \brief    Creates a read/write communication channel.
 *
 *  This function is used to create a tx/rx channel to communicate
 *  data between the application and the TSIP device instance. This
 *  function inturn calls DDC layer function PSP_tsipOpen().
 *  Returns configured channel handle to application.
 *
 *  This function is called in response to a GIO_create call.
 *
 *      Pre-requisites:
 *       -  Valid chanParams structure
 *       -  Valid device pointer
 *
 * \note    Please refer to Appendix A : IOM interface of spru616
 *          DSP/BIOS Driver Developer's Guide for details on
 *          tsip_mdCreateChan parameters.
 *
 * \param    chanp        [OUT]    Pointer to hold allotted channel
 * \param    devp         [IN]     Device pointer
 * \param    name         [IN]     TSIP Instance name like TSIP0
 * \param    mode         [IN]     channel  mode -> input or output
 * \param    chanParams   [IN]     channel parameters from user
 * \param    cbFxn        [IN]     callback function pointer
 * \param    cbArg        [IN]     callback function Arguments
 *
 * \return
 *           IOM_COMPLETED     if success<br>
 *           IOM_EALLOC        if no free channel available<br>
 *           IOM_EBADARGS      if invalid input parameters<br>
 *
 */

/* ARGSUSED */
static Int tsip_mdCreateChan(
                    Ptr                 *chanp,
                    Ptr                 devp,
                    String              name,
                    Int                 mode,
                    Ptr                 chanParams,
                    IOM_TiomCallback    cbFxn,
                    Ptr                 cbArg
                    )
{
    Int32                result   = IOM_COMPLETED;
    PSP_Result           status   = PSP_SOK;
    TSIP_chanHandle      pChan    = NULL;
    TSIP_portHandle      pPort    = NULL;
    PSP_TsipMode         tsipMode = PSP_TSIP_MODE_NONE;

    /* Validate input parameters */
    if( (((NULL == devp) || (NULL == chanParams))
          || ((NULL == cbFxn) || (NULL == cbArg)))
          || ((IOM_OUTPUT != mode) && (IOM_INPUT != mode)) )
    {

        result = IOM_EBADARGS;
#ifdef _DEBUG
        TSIP_DEBUG("tsip_mdCreateChan <Invalid Params>");
#endif

    }
    else
    {
        pPort = (TSIP_portHandle)devp;

        /* Assign respective tx/rx base channel pointer from port */
        if( IOM_INPUT == mode )
        {
            pChan = &pPort->rxChan;
        }
        else
        {
            pChan = &pPort->txChan;
        }

        if( NULL != pChan )
        {
            /* Check channel is FREE */
            if( FALSE == pChan->inUse )
            {
                /* Update other channel fields */
                pChan->mode   = mode;
                pChan->port   = pPort;
                pChan->cbFxn  = cbFxn;
                pChan->cbArg  = cbArg;

                /* Assign respective DDC tx/rx using i/o mode */
                if( IOM_INPUT == mode )
                {
                    tsipMode = PSP_TSIP_MODE_RX;
                }
                else
                {
                    tsipMode = PSP_TSIP_MODE_TX;
                }

                /* Invoke DDC layer fn to open the channel */
                pChan->pChanDDC = PSP_tsipOpen(
                                        pPort->portNum,
                                        tsipMode,
                                        chanParams,
                                        &status,
                                        &tsip_mdCallBack,
                                        pChan
                                        );
                /* Convert DDC to IOM error code */
                result = tsip_getIomErrCode(status);
            }
            else
            {
                result = IOM_EALLOC; /* Channel is in Use already */
            }
        }
        else
        {
            result = IOM_EBADARGS;
#ifdef _DEBUG
            TSIP_DEBUG("tsip_mdCreateChan <Invalid Params>");
#endif

        }
    }

    if( IOM_COMPLETED == result )
    {
        /* Place alloted channel into chanp */
        *chanp = (Ptr)pChan;

        /* Make this channel resource as USED */
        pChan->inUse = TRUE; 
    }
    else /* Error in opening a channel */
    {
        *chanp = NULL;
#ifdef _DEBUG
        TSIP_DEBUG("tsip_mdCreateChan <Failed>");
#endif

    }

    return result;
}

/**
 *  \brief    Frees a channel and all it's associated resources.
 *
 *  This function inturn calls PSP_tsipClose() of DDC layer funtion
 *
 *  \param    chanp  [IN]    Pointer to channel
 *
 * \return    IOM_COMPLETED  if success<br>
 *            IOM_EALLOC     if the channel is already deleted<br>
 *            IOM_EBADARGS   if invalid input parameters<br>
 */
static Int tsip_mdDeleteChan(Ptr chanp)
{
    Int32             result = IOM_COMPLETED;
    PSP_Result        status = PSP_SOK;
    TSIP_chanHandle   pChan  = NULL;

    if( NULL != chanp )
    {
        pChan = (TSIP_chanHandle)chanp;

        if( TRUE == pChan->inUse )
        {
            status = PSP_tsipClose(pChan->pChanDDC);

            if( PSP_SOK == status )
            {
                pChan->inUse = FALSE; /* Make this channel object FREE */
            }

            /* Convert DDC to IOM error code */
            result = tsip_getIomErrCode(status);
        }
        else
        {
            result = IOM_EALLOC;
        }
    }
    else
    {
        result = IOM_EBADARGS;
#ifdef _DEBUG
        TSIP_DEBUG("tsip_mdDeleteChan <Invalid Params>");
#endif
    }

    return result;
}

/**
 *  \brief    Implements the IOCTLS for TSIP IOM mini driver
 *
 *  tsip_mdControlChan() receives IOCTL commands from the
 *  GIO layer and then forwards the commands to the DDC by calling
 *  PSP_tsipIoctl().
 *
 *  \param    chanp       [IN]    Pointer to channel
 *  \param    cmd         [IN]    IOCTL command
 *  \param    arg         [IN]    Command argument
 *
 *  \return   IOM_COMPLETED if command is executed correctly<br>
 *            IOM_ABORTED   if command returns error during execution<br>
 *            IOM_ENOTIMPL  if command is not supported<br>
 *            IOM_EBADARGS  if invalid input parameters<br>
 */
static Int tsip_mdControlChan(Ptr chanp, Uns cmd, Ptr arg)
{
    Int32             result = IOM_COMPLETED;
    PSP_Result        status = PSP_SOK;
    TSIP_chanHandle   pChan  = NULL;

    /* Note 3rd argument Ptr arg should not be validated here.
       Since value of NULL or Non-NULL both are valid values depends
       on the command.*/
    if( NULL != chanp )
    {
        pChan = (TSIP_chanHandle)chanp;

        /* Pass the IOCTL to DDC layer */
        status = PSP_tsipIoctl(
                             pChan->pChanDDC,
                             (PSP_TsipIoctlCmd)cmd,
                             arg
                             );

        /* Convert DDC to IOM error code */
        result = tsip_getIomErrCode(status);

    }
    else
    {
        result = IOM_EBADARGS;
#ifdef _DEBUG
        TSIP_DEBUG("tsip_mdControlChan <Invalid Params>");
#endif

    }

    return result;
}

/**
 *  \brief    Submit an I/O packet to a channel for processing
 *
 *  The GIO layer calls this function to cause the mini-driver
 *  to process the IOM_Packet for read/write/flush/abort operations.
 *  This function calls DDC layer funtion PSP_tsipSubmit/
 *  PSP_tsipAbort/PSP_tsipFlush calls  to execute the request.
 *
 *  \param    chanp         [IN]    Pointer to channel
 *  \param    packet        [IN]    Pointer to packet to be submitted
 *
 *  \return
 *            IOM_COMPLETED if packet is fully processed<br>
 *            IOM_ABORTED   if packet is aborted<br>
 *            IOM_PENDING   if packet is not fully processed<br>
 */
static Int tsip_mdSubmitChan(Ptr chanp, IOM_Packet *packet)
{
    Int32              result = IOM_COMPLETED;
    PSP_Result         status = PSP_SOK;
    TSIP_chanHandle    pChan  = NULL;

    if( (NULL != packet) && (NULL != chanp) )
    {
        pChan   = (TSIP_chanHandle)chanp;

        if( FALSE == pChan->inUse ) /* Channel is not in use!!  */
        {
            result = IOM_EBADARGS;
#ifdef _DEBUG
            TSIP_DEBUG("tsip_mdSubmitChan <Error:Channel not in use>");
#endif

        }
        else
        {
            switch(packet->cmd)
            {
             case IOM_READ:
             case IOM_WRITE:
                 {
                     PSP_tsipIoRequest  ioReq;
#ifdef TSIP_INSTRUMENTATION_ENABLED
                     Uint32             numBytes = 0;
#endif

                     if( (NULL != packet->addr) 
                         && (packet->size <= PSP_TSIP_MAX_IOP_SFRAME) )
                     {

#ifdef TSIP_INSTRUMENTATION_ENABLED
                        /* Caculate number of bytes data to be txfd/recvd
                           in this IOP request */
                        if( IOM_READ == packet->cmd )
                        {
                            /* RX Size (bytes) = (#Sf * #frm/Sf * #bytes/frm )*/
                            numBytes = ( packet->size * 
                                        (pChan->port->hwSetup.siu.rx.fCnt *
                                         pChan->port->hwSetup.siu.rx.fSize) );
                
                        }
                        else
                        {
                            /* TX Size(bytes) = (#Sf * #frm/Sf * #bytes/frm )*/
                            numBytes = ( packet->size * 
                                        (pChan->port->hwSetup.siu.tx.fCnt *
                                         pChan->port->hwSetup.siu.tx.fSize) );

                        }


                        PSP_LOG_EVENT(
                                      &DVTEvent_Log,
                                      "TSIP",
                                      PSP_DVT_DESC(
                                                PSP_DVT_eFUNC_START,
                                                PSP_DVT_dCHANNEL_ID_INST,
                                                PSP_DVT_dPACKET_ID,
                                                PSP_DVT_dSIZE_BYTES
                                                ),
                                      CHANNELID_INSTANCEID(
                                                packet->cmd,
                                                pChan->port->portNum
                                                ),
                                      packet->addr,
                                      numBytes
                                    );
#endif /*TSIP_INSTRUMENTATION_ENABLED */


                         /* Fill the IO req. struct with GIO passed details */
                         ioReq.pIoParams = (PSP_tsipIoParams *)packet->addr; 
                         ioReq.numSframe = (Uint16)packet->size; 
                         ioReq.appPtr    = (Ptr)packet;  /* IOM reference */

                         /* Submit I/O packet to DDC */
                         status = PSP_tsipSubmit(pChan->pChanDDC, &ioReq);


                     }
                     else
                     {
                         status = PSP_E_INVAL_PARAM;
                     }
                 }
                 break;

             case IOM_ABORT:
                 /* Invoke DDC Abort functionality */  
                 status = PSP_tsipAbort(pChan->pChanDDC);
                 break;
             
             case IOM_FLUSH:
                 /* Invoke DDC Flush functionality */  
                 status = PSP_tsipFlush(pChan->pChanDDC);
                 break;

             default:
                 status = PSP_E_INVAL_PARAM;
                 break;
            }


            /* Convert DDC to IOM error code */       
            result = tsip_getIomErrCode(status);

        }
    }
    else
    {
        result = IOM_EBADARGS;
#ifdef _DEBUG
        TSIP_DEBUG("tsip_mdSubmitChan <Invalid Params>");
#endif

    }
    return result;
}

/**
 *  \brief    MiniDriver Callback function
 *
 *   This function is called from the DDC layer channel interrupt
 *   context(super-frame completion/error). This shall call the
 *   corresponding registered application callback function for this
 *   IOM channel
 *
 *  \param    chanp     [IN]     Channel Handle
 *  \param    ioReq     [IN]     pointer to iop request
 *
 *  \return   NONE
 */

static void tsip_mdCallBack(Ptr chanp, PSP_tsipIoRequest const *ioReq)
{
    TSIP_chanHandle      pChan    = NULL;
    IOM_Packet          *pIomPkt  = NULL;

    if( (NULL != chanp) && (NULL != ioReq) )
    {
        pChan   = (TSIP_chanHandle)chanp;
        pIomPkt = (IOM_Packet *)ioReq->appPtr;

#ifdef TSIP_INSTRUMENTATION_ENABLED
        if(IOM_READ == pIomPkt->cmd)
        {
            PSP_LOG_EVENT(
                          &DVTEvent_Log,
                          "TSIP",
                          PSP_DVT_DESC(
                                  PSP_DVT_eRCV_END,
                                  PSP_DVT_dCHANNEL_ID_INST,
                                  PSP_DVT_dPACKET_ID,
                                  PSP_DVT_dNONE
                                  ),
                          CHANNELID_INSTANCEID(
                                  IOM_READ,
                                  pChan->port->portNum
                                  ),
                          pIomPkt->addr
                         );
        }
        else
        {
            PSP_LOG_EVENT(
                          &DVTEvent_Log,
                          "TSIP",
                          PSP_DVT_DESC(
                                  PSP_DVT_eDATA_SND_END,
                                  PSP_DVT_dCHANNEL_ID_INST,
                                  PSP_DVT_dPACKET_ID,
                                  PSP_DVT_dNONE
                                  ),
                          CHANNELID_INSTANCEID(
                                  IOM_WRITE,
                                  pChan->port->portNum
                                  ),
                          pIomPkt->addr
                         );
        }

        PSP_LOG_EVENT(
                      &DVTEvent_Log,
                      "TSIP",
                      PSP_DVT_DESC(
                                PSP_DVT_eFUNC_END,
                                PSP_DVT_dCHANNEL_ID_INST,
                                PSP_DVT_dPACKET_ID,
                                PSP_DVT_dSIZE_BYTES),
                      CHANNELID_INSTANCEID(
                                pIomPkt->cmd,
                                pChan->port->portNum),
                      pIomPkt->addr,
                      0
                    );

#endif/* TSIP_INSTRUMENTATION_ENABLED */

        if(NULL != pIomPkt)
        {
            /* Update status of I/O operation into Iom status */
            pIomPkt->status = tsip_getIomErrCode(ioReq->status);
        }

        if( NULL != pChan->cbFxn )
        {
            /* Invoke Application callback for this channel */
            (*pChan->cbFxn)(pChan->cbArg,pIomPkt);
        }
    }
}


/**
 *   \brief    PSP to IOM error code mapping
 *
 *   This function is used to map PSP error codes into appropriate IOM
 *   error codes.
 *
 *   \param    pspErrCode    [IN]    DDC layer error code
 *
 *   \return   IOM Error code
 */
static Int32 tsip_getIomErrCode(PSP_Result pspErrCode)
{
    /*******************************************************************
     *                      IOM Error code look up table               *
     *******************************************************************
     * The following mapping shows the mapping done from PSP_Result to
     * IOM error code. This will be used in scenario where IOMini driver
     * interfaces with the DDC (PSP component)
     *
     * PSP_SINPROGRESS         (1)   IOM_PENDING              1
     * PSP_SOK                 (0)   IOM_COMPLETED            0
     * PSP_E_DRIVER_INIT       (-1)  IOM_ETIMEOUTUNREC      -11
     * PSP_E_NO_MEMORY         (-2)  IOM_EALLOC              -5
     * PSP_E_RESOURCES         (-3)  IOM_EALLOC              -5
     * PSP_E_INVAL_STATE       (-4)  IOM_EBADMODE            -7
     * PSP_E_INVAL_PARAM       (-5)  IOM_EBADARGS           -10
     * PSP_E_NOT_SUPPORTE      (-6)  IOM_ENOTIMPL            -9
     * PSP_E_IO_CANCEL_FAIL    (-7)  IOM_ETIMEOUT            -2
     * PSP_E_FIFO_NOT_ENABLED  (-8)  IOM_EBADMODE            -7
     * PSP_E_INVALID_MODE      (-9)  IOM_EBADMODE            -7
     * PSP_E_INVALID_CMD       (-10) IOM_EBADARGS           -10
     * PSP_E_TIMEOUT           (-11) IOM_ETIMEOUT            -2
     * PSP_TSIP_IO_FLUSHED     (-12) IOM_FLUSHED              2
     * PSP_TSIP_IO_ABORTED     (-13) IOM_ABORTED              3
     */
    static Int32 iomErrorCodeList[15] =
    {
        IOM_ABORTED,      /* 0  */
        IOM_FLUSHED,      /* 1  */
        IOM_ETIMEOUT,     /* 2  */
        IOM_EBADARGS,     /* 3  */
        IOM_EBADMODE,     /* 4  */
        IOM_EBADMODE,     /* 5  */
        IOM_ETIMEOUT,     /* 6  */
        IOM_ENOTIMPL,     /* 7  */
        IOM_EBADARGS,     /* 8  */
        IOM_EBADMODE,     /* 9  */
        IOM_EALLOC,       /* 10 */
        IOM_EALLOC,       /* 11 */
        IOM_ETIMEOUTUNREC,/* 12 */
        IOM_COMPLETED,    /* 13 */
        IOM_PENDING       /* 14 */
    };

    Int32 iomCode = pspErrCode + 13u;

    if( (iomCode >=0) && (iomCode < 15u ) )
    {
        iomCode = iomErrorCodeList[iomCode];
    }
    else
    {
        iomCode = IOM_EBADIO;
    }

    return(iomCode);
}

/*!
*! Revision History
*! ================
*! 12-Jun-2007    JP : Removed Pre-silicon related info and code segments
*! 05-Dec-2006    JP : Added TSIP_DEBUG() traces
*! 13-Nov-2006    JP : Modified braces to come in next line for
*!                     if,else,for,switch loops; Added mdSubmitChan
*!                     getIomErrCode functions.
*! 07-Nov-2006    JP : Added calls to PSP DDC functions.
*! 31-Oct-2006    JP : Initial Creation (0.1)
*! */
