/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/** \file    dda_tsipBios.h
 *
 *  \brief    Header file for TSIP Device Driver Adaptation-BIOS
 *
 *  This file has the globally visible data structures and definitions
 *  for the TSIP mini-driver.
 *
 *  (C)    Copyright 2006,    Texas Instruments, Inc
 *
 *  \author    Platform Support Group
 *
 *  \version    0.1
 */

#ifndef _DDA_TSIP_BIOS_H_
#define _DDA_TSIP_BIOS_H_

/* Standard Includes */
#include <std.h>
#include <iom.h>
/* Specific Includes */
#include <ti/sdo/pspdrivers/drivers/tsip/psp_tsip.h>
#include <ti/sdo/pspdrivers/drivers/tsip/src/psp_tsipApi.h>


#ifdef __cplusplus
extern "C"
{
#endif

/**
 * \brief    TSIP Channel Object
 *
 * This structure maintains the current channel state. It also holds
 * the handle to TSIP port and application callback function to be called
 * in case of an interrupt.
 *
 * This structure is initialized by tsip_mdCreateChan and a pointer to this
 * is passed down to all other channel related functions. Lifetime of
 * the data structure is from its creation by tsip_mdCreateChan till it
 * is invalidated by tsip_mdDeleteChan.
 */
typedef struct TSIP_chanObj_t
{
    Bool                    inUse;
    /**< Keeps track of whether channel is already in use */

    Int32                   mode;
    /**< Mode of operation: INPUT or OUTPUT               */

    struct TSIP_portObj_t   *port;
    /**< Port back pointer to device config. structure    */

    IOM_TiomCallback        cbFxn;
    /**< Callback function                                */

    Ptr                     cbArg;
    /**< Arguments for callback function                  */

    PSP_Handle              pChanDDC;
    /**< Handle to TSIP DDC channel                       */
} TSIP_chanObj, *TSIP_chanHandle;

/**
 * \brief    Default TSIP channel object values
 */
#define TSIP_DDA_CHAN_DEFAULT                            \
        {                                                \
        /* inUse    */     FALSE,                        \
        /* mode     */     (0xDEADu),                    \
        /* port     */     NULL,                         \
        /* cbFxn    */     NULL,                         \
        /* cbArg    */     NULL,                         \
        /* pChanDDC */     NULL                          \
        }
/**
 * \brief    TSIP Port Object
 *
 * This data structure holds the handle to device init params, the current
 * port number and TSIP channel objects. The data structure is initialized
 * during tsip_mdBindDev, which is called during DSP-BIOS initialization,
 * and is persistent till it is invalidated by tsip_mdUnBindDev.
 */
typedef struct  TSIP_portObj_t
{
    Bool                 inUse;
    /**< Keeps track of whether port is already in use or not*/

    Int32                portNum;
    /**< Preserve instance number in port                    */

    TSIP_chanObj         txChan;
    /**< Holds transmit communication channel for this port  */

    TSIP_chanObj         rxChan;
    /**< Holds receive communication channel for this port   */

    PSP_tsipHwSetup      hwSetup;
    /**< Holds the TSIP hardware setup(device params) passed
         by the application                                  */

} TSIP_portObj, *TSIP_portHandle;


/**
 * \brief    Default TSIP HW setup data structure values(all zeros)
 */
#define TSIP_DDA_HW_SETUP_DEFAULT                              \
        {                                                      \
        /* SIU           */ {                                  \
               /* clock   */  PSP_TSIP_CLK_REDUNDANT,          \
               /* Tx.SIU  */  {                                \
                               PSP_TSIP_CLK_FS_SRC_A,          \
                               0,                              \
                               PSP_TSIP_TXDIS_STATE_HIGHZ,     \
                               PSP_TSIP_TXENA_DLY_ZERO,        \
                               PSP_TSIP_POLARITY_ACTIVE_LOW,   \
                               PSP_TSIP_POLARITY_RISING_EDGE,  \
                               PSP_TSIP_POLARITY_RISING_EDGE,  \
                               PSP_TSIP_DATA_RATE_8_192,       \
                               PSP_TSIP_CLK_MODE_2X,           \
                               0,                              \
                               0                               \
                              },                               \
               /* Rx.SIU  */  {                                \
                               PSP_TSIP_CLK_FS_SRC_A,          \
                               0,                              \
                               PSP_TSIP_POLARITY_ACTIVE_LOW,   \
                               PSP_TSIP_POLARITY_RISING_EDGE,  \
                               PSP_TSIP_POLARITY_RISING_EDGE,  \
                               PSP_TSIP_DATA_RATE_8_192,       \
                               PSP_TSIP_CLK_MODE_2X,           \
                               0,                              \
                               0                               \
                              },                               \
               /* LoopBack*/  PSP_TSIP_LOOP_BACK_NONE          \
                            },                                 \
        /* TDMU          */ {                                  \
                              PSP_TSIP_LITTLE_ENDIAN,          \
                              0,                               \
                              0,                               \
                              PSP_TSIP_INT_XFER_ACK_ONLY,      \
                              PSP_TSIP_INT_XFER_ACK_ONLY,      \
                              PSP_TSIP_INT_XFER_ACK_ONLY,      \
                              PSP_TSIP_INT_XFER_ACK_ONLY,      \
                              0,                               \
                              0                                \
                            }                                  \
        }

/**
 * \brief    Default TSIP port object values
 */
#define TSIP_DDA_PORT_DEFAULT                            \
        {                                                \
        /* inUse    */     FALSE,                        \
        /* portNum  */     (0xDEADu),                    \
        /* txchan   */     TSIP_DDA_CHAN_DEFAULT,        \
        /* rxchan   */     TSIP_DDA_CHAN_DEFAULT,        \
        /* hwSetup  */     TSIP_DDA_HW_SETUP_DEFAULT     \
        }


/***********************************
 * Function prototypes
 ***********************************/

/**
 *  \brief    Registering interrupt handler
 *
 *   This function is invoked from the DDC layer in order for the
 *   DDC to register TSIP interrupts with the requested interrupt
 *   handler with the OS (BIOS) and enables the interrupt.
 *
 *  \param    chanp        [IN]    Pointer to channel
 *  \param    intNum       [IN]    Interrupt Controller ID in which evt is
 *                                 mapped
 *  \param    evt          [IN]    event number
 *  \param    isrFunc      [IN]    ISR function pointer
 *
 *  \return
 *            PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result DDA_tsipRegisterInterrupt(
                                 Ptr chanp,
                                 Uint32 intNum,
                                 Uint16 evt,
                                 PSP_tsipIsr isrFunc
                                 );
/**
 * \brief    Unregistering interrupt handler
 *
 * This function is invoked from DDC to unregister and disable interrupt
 * handler and interrupt cause respectively.
 *
 *  \param    intNum        [IN]    interrupt number
 *  \param    evt           [IN]    event number
 *
 *  \return   PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */

PSP_Result DDA_tsipUnRegisterInterrupt(Uint32 intNum, Uint16 evt);


#ifdef __cplusplus
}
#endif

#endif /* _DDA_TSIP_BIOS_H_ */

/*!
*! Revision History
*! ================
*! 20-Apr-2007    JP : Applied changes after testing in pre-silicon board
*!
*! 17-Jan-2007    JP : Added default init value macro for dev and chan objects
*!
*! 13-Nov-2006    JP : Modified struct/enum declaration standard ( braces at
*!                     the next line, name for all enum/struct with _t suffix)
*! 31-Oct-2006    JP : Initial Creation(0.1)
*! */
