/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/** \file    ddc_tsip.c
 *
 *  \brief   Device Driver Core(DDC) of TSIP.
 *
 *  This file implements core driver functionalities like FSMs, ISR handling
 *  of the TSIP driver. This implementation shall not contain any OS specific
 *  informations.
 *
 *  (C)    Copyright 2006,    Texas Instruments, Inc
 *
 *  \author    Platform Support Group
 *
 *  \version    0.1
 *
 */

/*
 * Header includes
 */
#include <ti/sdo/pspdrivers/pal_os/pal_osProtect.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osMem.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osCache.h>
#include <ti/sdo/pspdrivers/drivers/tsip/src/ddc_tsip.h>
#include <ti/sdo/pspdrivers/drivers/tsip/src/ddc_tsipHal.h>
#include <ti/sdo/pspdrivers/soc/c6452/dsp/soc.h>

#ifdef TSIP_INSTRUMENTATION_ENABLED
#include <ti/sdo/pspdrivers/pal_os/bios/psp_log.h>
#endif

/*
 * External References
 */

/********** function prototypes **************************************/
extern PSP_Result DDA_tsipRegisterInterrupt(
                                        Ptr           chanp,
                                        Uint32        intNum,
                                        Uint16        evt,
                                        PSP_tsipIsr   isrFunc
                                        );
extern PSP_Result DDA_tsipUnRegisterInterrupt(Uint32 intNum, Uint16 evt);

/*
 * Public Declarations
 */

/************ data declarations ***************************************/

#ifdef _DEBUG
/* Dignostic interrupt counters */
Uint32  tsipRxSuperFrameCnt = 0;
Uint32  tsipTxSuperFrameCnt = 0;
Uint32  tsipRxFrameCnt      = 0;
Uint32  tsipTxFrameCnt      = 0;
Uint32  tsipErrorCnt        = 0;
#endif

/*
 * Private Declarations
 */

/************ data declarations ***************************************/

/**
 * \brief TSIP DDC device objects allocation
 */
static DDC_tsipDevObj tsipDevObj[PSP_TSIP_NUM_PORTS] = {TSIP_DDC_PORT_DEFAULT,
                                                        TSIP_DDC_PORT_DEFAULT};

/********** function prototypes ***************************************/

static PSP_Result ddc_tsipSubmitIop(
                            DDC_tsipIoHandle          pIoHandle,
                            PSP_tsipIoRequest  const *pIoReq
                            );
static PSP_Result ddc_tsipAbort(DDC_tsipIoHandle ioHandle,Int32 cmdType);

/* ISR functions */
#ifdef PSP_TSIP_USE_FRAME_ISR

static void ddc_tsipTxFrameISR(PSP_Handle pIoHandle);
static void ddc_tsipRxFrameISR(PSP_Handle pIoHandle);

#endif/* PSP_TSIP_USE_FRAME_ISR */

static void ddc_tsipTxSuperFrameISR(PSP_Handle pIoHandle);
static void ddc_tsipRxSuperFrameISR(PSP_Handle pIoHandle);
static void ddc_tsipTxRxErrorISR(PSP_Handle pIoHandle);

static PSP_Result ddc_tsipGetEvtNum(
                                DDC_tsipIoHandle     pIoHandle,
                                DDC_TsipChanIntType  intType,
                                Uint16              *pEvtId
                                );
static void ddc_tsipCopyIop2Req(
                            PSP_tsipIoRequest *pIoReq,
                            const DDC_tsipIop *pIop
                            );

static void ddc_tsipCopyReq2Iop(
                            DDC_tsipIop *pIop,
                            const PSP_tsipIoRequest *pIoReq);



/***********************************************************
 * Public Functions Definitions
 ***********************************************************/


/**
 * \brief    Sets up the TSIP device instance
 *
 *  This function is used to setup the TSIP hardware specified
 *  by the device ID.
 *
 * \param    devId         [IN]     Device instance number
 * \param    initCfg       [IN]     Hardware initialization parameters
 * \param    arg           [IN]     application handle as arg.
 *
 * \return
 *           PSP_SOK         if success<br>
 *           PSP ERROR Codes if failure<br>
 */
PSP_Result PSP_tsipCreate(
                      Int32                     devId,
                      PSP_tsipDevParams const  *initCfg,
                      Ptr                       arg
                      )
{
    PSP_Result              result      = PSP_SOK;
    DDC_tsipDevHandle       devHandle   = NULL;

    if( devId < PSP_TSIP_NUM_PORTS ) /* Is valid device or instance number */
    {
        devHandle = &tsipDevObj[devId];
    }

    if( (NULL == initCfg) || ((NULL == arg) || (NULL == devHandle)) )
    {
        result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
        TSIP_DEBUG("PSP_tsipCreate <Invalid Params>");
#endif

    }
    else
    {
        if( FALSE == devHandle->inUse )
        {
            /* Store the device ID and the arg */
            devHandle->devId     = devId;
            devHandle->appHandle = arg;

            /* Ensure default values for port elements */
            devHandle->isErrRegd = FALSE;

            /* Make TSIP HW engine flags to not running */
            devHandle->isRxSiuON   = FALSE;
            devHandle->isTxSiuON   = FALSE;
            devHandle->isTdmuON    = FALSE;


            /* Store TSIP HW setup    */
            PAL_osMemCopy(
                      (Ptr)&devHandle->hwSetup,
                      (Ptr)initCfg,
                      sizeof(PSP_tsipHwSetup)
                      );


            /* Get TSIP memory-map (Register) base addr for this instance */
            result = HAL_tsipGetBaseAddr(devHandle);

            if( PSP_SOK == result )
            {
                /* Reset the TSIP to make sure TSIP is in proper initial state */
                result  = HAL_tsipSiuReset(devHandle);
                result |= HAL_tsipTdmuReset(devHandle);

                /* SIU configurations */
                if( PSP_SOK == result )
                {
                    result = HAL_tsipSiuSetup(devHandle, &initCfg->siu);
                }

                /* TDMU-DMATCU configurations */
                if( PSP_SOK == result )
                {
                    result = HAL_tsipTdmuSetup(devHandle, &initCfg->tdmu);
                }
            }
        }
        else
        {
            result = PSP_E_RESOURCES;
        }
    }

    if( PSP_SOK == result )
    {
        /* Make this dev resource as allocated */
        devHandle->inUse = TRUE;
    }

    return result;
}

/**
 * \brief    Un-initializes the TSIP device instance
 *
 * This function is used to do the reverse operation of
 * PSP_tsipCreate. It uninitializes the TSIP HW for the given
 * devId.
 *
 * \param    devId         [IN]    DevId of TSIP to be deleted.
 *
 * \return
 *           PSP_SOK            if success<br>
 *           PSP_E_INVAL_PARAM  if invalid arguments<br>
 *
 */
PSP_Result PSP_tsipDelete(Int32 devId)
{
    PSP_Result              result      = PSP_SOK;
    DDC_tsipDevHandle       devHandle   = NULL;

    if( devId < PSP_TSIP_NUM_PORTS ) /* Is valid device ID */
    {
        devHandle = &tsipDevObj[devId];
    }

    if( NULL != devHandle )
    {

       /* Validating the device Object status. Should perform deletion
          only if the device is created i.e inUse is TRUE */
       if( TRUE == devHandle->inUse )
       {

           /* Un-initialize this TSIP for this devId instance */

           /* Disable TSIP engines */
           HAL_TsipDisableDmatcu(devHandle);
           HAL_TsipDisableTxSiu(devHandle);
           HAL_TsipDisableRxSiu(devHandle);

           /* Make this dev resource as FREE */
           devHandle->inUse = FALSE;

           /* Clear port object parameters */
           PAL_osMemSet(devHandle,0,sizeof(DDC_tsipDevObj));
           devHandle->devId = (0xDEADL);
       }
       else
       {
           result = PSP_E_INVAL_STATE;
       }

    }
    else
    {
        result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
        TSIP_DEBUG("PSP_tsipDelete <Invalid Params>");
#endif
    }

    return result;
}

/**
 * \brief    Allocates and configures the DMATCU TX/RX channel
 *
 * \note    Here the number of channel objects(tx+rx) is equal to number of
 *          DMATCU channels supported by TSIP. Becz each DMATCU channel is a
 *          seperate communication path and hence 1 to 1 mapping.
 *
 * This function is used to do the following configurations<br>
 *    - Allocates a free channel(tx/rx) from the availble channel objects
 *    - Configure the bitmap(A&B) for the alloted DMATCU tx/rx channel
 *    - Register tx/rx channel superframe interrupt handler
 *    - Register tx/rx channel error interrupt handler
 *
 *
 * \param    devId         [IN]     TSIP devId(instance)
 * \param    mode          [IN]     TX or RX channel
 * \param    chanParams    [IN]     channel parameters from user
 * \param    pStatus       [OUT]    Status of PSP_tsipOpen
 * \param    drvCb         [IN]     driver callback function
 * \param    cbArg         [IN]     callback function Arguments
 *
 * \return
 *           DDC channel handle   if success<br>
 *           NULL                 if failure<br>
 */
PSP_Handle PSP_tsipOpen(
                    Int32                        devId,
                    PSP_TsipMode                 mode,
                    PSP_tsipChanParams const    *chanParams,
                    PSP_Result                  *pStatus,
                    PSP_tsipAppCallback          drvCb,
                    Ptr                          cbArg
                    )
{
    PSP_Handle             pHandle     = NULL;
    register PSP_Result    status      = PSP_SOK;
    DDC_tsipDevHandle      devHandle   = NULL;
    DDC_tsipIoHandle       ioHandle    = NULL;

    Uint32                 index   = 0;    /* Loop variable */
    Uint16                 evtId   = 0;    /* Event Number */
    DDC_TsipChanIntType    intType;       /* Interrupt Type(frm/SF/Error)*/
    Uint32                 cookie  = 0;

    /* Validating devId and mode parameters */
    if( (devId < PSP_TSIP_NUM_PORTS)
        && ((PSP_TSIP_MODE_TX == mode) || (PSP_TSIP_MODE_RX == mode)) )
    {

        devHandle = &tsipDevObj[devId];

        if( NULL != devHandle )
        {
            /* Assign the respective channel base ptr for the given mode */
            if( PSP_TSIP_MODE_TX == mode )
            {
                ioHandle = &devHandle->txChan;
            }
            else
            {
                ioHandle = &devHandle->rxChan;
            }
            ioHandle->mode   = mode; /* Store the channel mode */
        }
    }
    /* Validating input pointers, chan pointer */
    if( ((NULL == chanParams) || (NULL == ioHandle))
        || ((NULL == drvCb)|| (NULL == cbArg)) )
    {
        status = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
        TSIP_DEBUG("PSP_tsipOpen <Invalid Params>");
#endif

    }
    else
    {
        /* Protect channel status from interrupt: Shared variable */
        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

        /* Check channel is FREE */
        if( DDC_TSIP_CHAN_FREE == ioHandle->status )
        {
            ioHandle->status = DDC_TSIP_CHAN_ALLOTTED;/* Make it allotted*/

            /*Assign the respective drv callback function*/
            ioHandle->drvCb = drvCb;
            ioHandle->cbArg = cbArg;

            /* Store error callback and its argument  */
            ioHandle->errCb  = chanParams->errCb;
            ioHandle->eCbArg = chanParams->eCbArg;

            /* Storing frame and superframe callbacks and args */
            ioHandle->frmCb  = chanParams->frmCb;
            ioHandle->fCbArg = chanParams->fCbArg;
            ioHandle->sfrmCb = chanParams->sfrmCb;
            ioHandle->sCbArg = chanParams->sCbArg;

            /* Store the device handle */
            ioHandle->devHandle = devHandle;

            /* Reset the flags */
            ioHandle->isChEnabled     = FALSE;
            ioHandle->isStopExtCmd    = FALSE;

            /* Clear error stats counters */
            PAL_osMemSet(&ioHandle->errStats,0,sizeof(PSP_tsipErrStats));

            status = PSP_SOK; /* Success. channel allotted */
        }
        else
        {
            status = PSP_E_RESOURCES; /* Resource not available*/
        }

        /* Un protect channel status */
        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
    }

    if( PSP_SOK == status )
    {
        /* Validate CID for "A" and "B" selection.
           lsb(aCID) should be 0 and lsb(bCID) should be 1 */
        if( (0x00 == (chanParams->aCID & 0x01))
         && (0x01 == (chanParams->bCID & 0x01)) )
        {
            /* Configure CID values for the channel */
            ioHandle->aCID = chanParams->aCID;
            ioHandle->bCID = chanParams->bCID;
        }
        else /* Invalid A CID or B CID */
        {
            status = PSP_E_INVAL_PARAM;
        }
    }

    if( PSP_SOK == status )
    {
        /* Create flush Semapore for TX channel (Used in Flush operation) */
        if( PSP_TSIP_MODE_TX == mode )
        {
            PAL_OsSemAttrs    semAttrs = PAL_OSSEM_ATTRS;

            if( PAL_SOK != PAL_osSemCreate(
                                        NULL,
                                        0,
                                        &semAttrs,
                                        (PAL_OsSemHandle)&ioHandle->flushSem
                                        ) )
            {
                status = PSP_E_RESOURCES;
            }
         }
    }

    if( PSP_SOK == status )
    {
        /* Make nodes for free and pending IOP lists */
        PAL_OSLIST_MKNODE( &ioHandle->queueFreeList );
        PAL_OSLIST_MKNODE( &ioHandle->queueReqList );

        /* Enqueue free list upto maximum supported statically allocated IOPs */
        for( index = 0; index < PSP_TSIP_MAX_IOP_REQ; index++ )
        {
            PAL_osListEnqueue(
                         (Ptr)&ioHandle->queueFreeList,
                         (Ptr)&ioHandle->iop[index]
                         );
        }

        /* Get the TSIP channel register base for this channel */
        status = HAL_tsipGetChanBaseAddr(ioHandle);

        /**** Configure Channel Parameters ****/

        if( PSP_SOK == status )
        {
            /* Bitmap A config */
            status = HAL_tsipSetBitmap(
                                     ioHandle,
                                     &chanParams->bmA,
                                     HAL_TSIP_A
                                     );
        }

        if( PSP_SOK == status )
        {
            /* Bitmap B config */
            status = HAL_tsipSetBitmap(
                                     ioHandle,
                                     &chanParams->bmB,
                                     HAL_TSIP_B
                                     );
        }

        /**** Super Frame Interrupt Registration ******/

        if( PSP_SOK == status )
        {
            /* Get Event Num for Super Frame Interrupt */
            intType = DDC_TSIP_CHAN_SFRAME_INT;
            status  = ddc_tsipGetEvtNum(ioHandle,intType,&evtId);
        }

        if( PSP_SOK == status )
        {
            if( PSP_TSIP_MODE_TX == ioHandle->mode )
            {
                status = DDA_tsipRegisterInterrupt(
                                      ioHandle,
                                      HWI_EVT03_CMB_IRQ,
                                      evtId,
                                      &ddc_tsipTxSuperFrameISR
                                      );
            }
            else
            {
                status = DDA_tsipRegisterInterrupt(
                                      ioHandle,
                                      HWI_EVT03_CMB_IRQ,
                                      evtId,
                                      &ddc_tsipRxSuperFrameISR
                                      );
            }

        }

#ifdef PSP_TSIP_USE_FRAME_ISR
        /**** Frame Interrupt Registration ******/

        if( PSP_SOK == status )
        {
            /* Get EVT number for Frame Interrupt for this chan */
            intType = DDC_TSIP_CHAN_FRAME_INT;
            status  = ddc_tsipGetEvtNum(ioHandle,intType,&evtId);
        }

        if( PSP_SOK == status )
        {
            if( PSP_TSIP_MODE_TX == ioHandle->mode )
            {
                status = DDA_tsipRegisterInterrupt(
                                      ioHandle,
                                      HWI_EVT03_CMB_IRQ,
                                      evtId,
                                      &ddc_tsipTxFrameISR
                                      );
            }
            else
            {
                status = DDA_tsipRegisterInterrupt(
                                      ioHandle,
                                      HWI_EVT03_CMB_IRQ,
                                      evtId,
                                      &ddc_tsipRxFrameISR
                                      );
            }

        }
#endif /* PSP_TSIP_USE_FRAME_ISR */

    }

    /**** Error Interrupt Registration ******/

    if( PSP_SOK == status )
    {
        /* Get EVT number for Error interrupt for this channel */

        intType = DDC_TSIP_CHAN_ERROR_INT;

        status  = ddc_tsipGetEvtNum(ioHandle,intType,&evtId);
     }

     if( PSP_SOK == status )
     {
         /* If not already registered */
         if( FALSE == ioHandle->devHandle->isErrRegd )
         {
             /* Register tx/rx channel error interrupt handler */
             status = DDA_tsipRegisterInterrupt(
                                      ioHandle,
                                      HWI_EVT03_CMB_IRQ,
                                      evtId,
                                      &ddc_tsipTxRxErrorISR
                                      );
             if( PSP_SOK == status )
             {
                 /* ErrInt registered */
                 ioHandle->devHandle->isErrRegd = TRUE;
             }
          }

     }


    if( PSP_SOK == status ) /* Yeah! All Success */
    {
        /* Assign configured DDC channel into pHandle*/
        pHandle = (PSP_Handle)ioHandle;
    }
    else /* No! Some issues */
    {
#ifdef _DEBUG
        TSIP_DEBUG("Error: Channel Creation <Failed>");
#endif
        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

        /* Error in opening. Make this channel free to avoid
           resource freezing i.e to allow this channel resource
           available for  another request */
        ioHandle->status = DDC_TSIP_CHAN_FREE;

        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
    }


    if(NULL != pStatus)
    {
        *pStatus = status; /* Update status/error code to the caller */
    }

    return pHandle;
}

/**
 * \brief    Un-allocates the DMATCU TX/RX channel
 *
 *  This function is used to perform the reverse operation
 *  of PSP_tsipOpen().<br>
 *
 *  It does the following actions
 *      - Disables tx/rx channel interrupts
 *      - Unregister tx/rx channel superframe interrupt handler
 *      - Unregister tx/rx channel error interrupt handler if
 *        both tx/rx channels are not in use.
 *      - Make the channel resource as FREE for further channel
 *        requests
 *
 * \param    pIoHandle     [IN]     TSIP DDC channel pointer
 *
 * \return
 *           PSP_SOK             if success<br>
 *           PSP ERROR Codes     if failure<br>
 */
PSP_Result PSP_tsipClose(PSP_Handle pIoHandle)
{
    register PSP_Result   result     = PSP_SOK;
    DDC_tsipIoHandle      ioHandle   = NULL;
    DDC_TsipChanStatus    chTxStat;
    DDC_TsipChanStatus    chRxStat;

    Uint16                evtId  = 0;
    Uint32                cookie = 0;

    if( NULL != pIoHandle )
    {
        ioHandle = (DDC_tsipIoHandle)pIoHandle;
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
        TSIP_DEBUG("PSP_tsipClose <Invalid Params>");
#endif
    }

    if(PSP_SOK == result)
    {
        if( DDC_TSIP_CHAN_ALLOTTED == ioHandle->status )
        {
            /* Disable the DMATCU channel */
            HAL_TsipDisableDmaChan(ioHandle);

            /* Protect shared data: chan status, enable */
            PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

            ioHandle->isChEnabled = FALSE; /* Turn Channel to STOPPED */

            PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

            if( PSP_TSIP_MODE_TX == ioHandle->mode )
            {
                /* Delete the semaphore created at driver open time. */
                if (PAL_SOK != PAL_osSemDelete(ioHandle->flushSem))
                {
                    result = PSP_E_RESOURCES;
                }
            }

#ifdef PSP_TSIP_USE_FRAME_ISR
            if( PSP_SOK == result )
            {
                /* Get EVT number for frame interrupt */
                result = ddc_tsipGetEvtNum(
                                       ioHandle,
                                       DDC_TSIP_CHAN_FRAME_INT,
                                       &evtId
                                       );
            }

            if( PSP_SOK == result )
            {
                /* Unregister frame interrupt handler for this channel */

                result = DDA_tsipUnRegisterInterrupt(HWI_EVT03_CMB_IRQ, evtId);

            }
#endif
            /* Get EVT number for Sframe interrupt */
            result = ddc_tsipGetEvtNum(
                                   ioHandle,
                                   DDC_TSIP_CHAN_SFRAME_INT,
                                   &evtId
                                   );

            if( PSP_SOK == result )
            {
                /* Unregister super-frame interrupt handler for this channel */

                result = DDA_tsipUnRegisterInterrupt(HWI_EVT03_CMB_IRQ, evtId);

            }

            if( PSP_SOK == result )
            {

                /* Unregister error interrupt for if both tx/rx channels are
                 closed. Becz error interrupt is shared by both TX and RX
                 channel.
                 */

                chTxStat = ioHandle->devHandle->txChan.status;
                chRxStat = ioHandle->devHandle->rxChan.status;

                if(  (DDC_TSIP_CHAN_FREE == chTxStat)
                   && (DDC_TSIP_CHAN_FREE == chRxStat) )
                {
                    /* Get EVT number for Error interrupt */
                    result = ddc_tsipGetEvtNum(
                                           ioHandle,
                                           DDC_TSIP_CHAN_ERROR_INT,
                                           &evtId
                                           );
                    if( PSP_SOK == result )
                    {
                        /* Unregister tx-rx error interrupt */
                        result = DDA_tsipUnRegisterInterrupt(
                                                         HWI_EVT03_CMB_IRQ,
                                                         evtId
                                                         );
                    }
                }
            }

            if( PSP_SOK == result )
            {
                /* Protect shared data: chan status, enable */
                PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

                ioHandle->status      = DDC_TSIP_CHAN_FREE; /* Make it FREE */
                ioHandle->isFlushCmd  = FALSE; /* Clear Flush command flag */

                PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
            }
        }
        else
        {
            result = PSP_E_INVAL_STATE;
#ifdef _DEBUG
            TSIP_DEBUG("PSP_tsipClose <Invalid State>");
#endif
        }
    }
    return result;
}

/**
 * \brief    Performs an IOCTL
 *
 * This function is used to perform IOCTLs on the Channel.
 *
 * \param    pIoHandle     [IN]        TSIP DDC channel pointer
 * \param    cmd           [IN]        IOCTL command
 * \param    arg           [IN/OUT]    Place holder for cmd argument
 *
 * \return
 *           PSP_SOK              if success<br>
 *           PSP ERROR codes      if failure<br>
 */
PSP_Result PSP_tsipIoctl(
                     PSP_Handle          pIoHandle,
                     PSP_TsipIoctlCmd    cmd,
                     Ptr                 arg
                     )
{
    register PSP_Result   result    = PSP_SOK;
    DDC_tsipIoHandle      ioHandle  = NULL;
    Uint32                cookie    = 0;

    if( NULL != pIoHandle )
    {
        ioHandle = (DDC_tsipIoHandle)pIoHandle;
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    if( PSP_SOK == result )
    {
        if( DDC_TSIP_CHAN_ALLOTTED == ioHandle->status )
        {
            switch(cmd)
            {

             /***  1. GET peripheral ID ***/
             case PSP_TSIP_IOCTL_GET_PID:

                 /* Get the TSIP peripheral ID from the HW PID register */
                 if( NULL != arg)
                 {
                     PSP_tsipPID *tiPid = (PSP_tsipPID *)arg;

                     tiPid->moduleID = (Uint16)CSL_FEXT(
                                                ioHandle->devHandle->pReg->PID,
                                                TSIP_PID_MODID
                                                );
                     tiPid->majorVer = (Uint8)CSL_FEXT(
                                                ioHandle->devHandle->pReg->PID,
                                                TSIP_PID_MAJOR
                                                );
                     tiPid->minorVer = (Uint8)CSL_FEXT(
                                                ioHandle->devHandle->pReg->PID,
                                                TSIP_PID_MINOR
                                                );
                 }
                 else
                 {
                     result = PSP_E_INVAL_PARAM;
                 }
                 break;


             /***  2. Enable/Start DMA channel ***/
             case PSP_TSIP_IOCTL_START_CHANNEL:

                 /* Enable the DMATCU channel operation */

                 PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

                 HAL_TsipEnableDmaChan(ioHandle);
                 ioHandle->isChEnabled = TRUE; /* Turn ON channel enable flag */

                 PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

                 break;

             /***  3. Disable/Stop DMA channel ***/
             case PSP_TSIP_IOCTL_STOP_CHANNEL:

                 /* Disable the DMATCU channel operation */

                 PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

                 HAL_TsipDisableDmaChan(ioHandle);
                 ioHandle->isChEnabled = FALSE; /* Turn OFF chan enable flag */

                 PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

                 break;

             /***  4. Disable/Stop DMA channel after current superframe ***/
             case PSP_TSIP_IOCTL_STOP_CHANNEL_EXT:

                 PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);
                 /* Turn on Disable Extended IOCTL flag telling the DDC
                   that "Disable the channel only after the completion of
                   the current superframe". This flag isStopExtCmd will be
                   checked in superframe ISR function and if set, the channel
                   will be disabled in ISR itself. */
                 ioHandle->isStopExtCmd = TRUE;

                 PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

                 break;

             /***  5. Modify Bitmap A ***/
             case PSP_TSIP_IOCTL_MODIFY_BITMAP_A:
                 {
                     HAL_TsipChanBmapCtxt   activeBitmap;

                     /* Get the active bitmap */
                     result = HAL_tsipGetActiveBitmap(ioHandle,&activeBitmap);

                     if( PSP_SOK == result )
                     {
                         /* Change bitmap A, if the current active
                            bitmap is not A*/
                         if( HAL_TSIP_A != activeBitmap)
                         {
                             /* Bitmap A config */
                             result = HAL_tsipSetBitmap(
                                                 ioHandle,
                                                 (PSP_tsipBitmap const *)arg,
                                                 HAL_TSIP_A
                                                 );
                         }
                         else
                         {
                             /* Can't modify Bitmap A since it is in use by TSIP
                                now. return INVALID STATE to the caller */
                             result = PSP_E_INVAL_STATE;
                         }
                      }
                 }
                 break;

             /***  6. Modify Bitmap B ***/
             case PSP_TSIP_IOCTL_MODIFY_BITMAP_B:
                 {
                     HAL_TsipChanBmapCtxt   activeBitmap;

                     /* Get the active bitmap */
                     result = HAL_tsipGetActiveBitmap(ioHandle,&activeBitmap);

                     if( PSP_SOK == result )
                     {
                         /* Change bitmap B, if the current active
                            bitmap is not B */
                         if( HAL_TSIP_B != activeBitmap)
                         {
                             /* Bitmap A config */
                             result = HAL_tsipSetBitmap(
                                                 ioHandle,
                                                 (PSP_tsipBitmap const *)arg,
                                                 HAL_TSIP_B
                                                 );
                         }
                         else
                         {
                             /* Can't modify Bitmap B since it is in use by TSIP
                                now.return INVALID STATE to the caller */
                             result = PSP_E_INVAL_STATE;
                         }
                      }
                 }
                 break;

             /***  7. Get driver maintained error statistics counters ***/
             case PSP_TSIP_IOCTL_GET_ERROR_STATS:

                 if( NULL != arg)
                 {
                     PSP_tsipErrStats *pErrStat = (PSP_tsipErrStats *)arg;

                     /* Copy the error statistics counter values.
                      :NOTE: PSP_tsipErrStats is a union for tx/rx
                      stats info.*/
                     PAL_osMemCopy(
                                (Ptr)pErrStat,
                                (Ptr)&ioHandle->errStats,
                                sizeof(PSP_tsipErrStats)
                                );
                 }
                 else
                 {
                     result = PSP_E_INVAL_PARAM;
                 }
                 break;

             /***  8. Clear driver maintained error statistics counters ***/
             case PSP_TSIP_IOCTL_CLEAR_ERROR_STATS:
                 /* Clear the error statistics union */
                 PAL_osMemSet(&ioHandle->errStats,0,sizeof(PSP_tsipErrStats));

                 break;

             /***  9. Get the current active context ***/
             case PSP_TSIP_IOCTL_GET_ACTIVE_CONTEXT:

                 if( NULL != arg)
                 {
                     HAL_TsipChanBmapCtxt   context;
                     PSP_TsipActiveContext *pContext = (PSP_TsipActiveContext *)arg;

                     HAL_tsipGetActiveContext(ioHandle, &context);

                     switch(context)
                     {
                      case HAL_TSIP_A:
                          *pContext = PSP_TSIP_ACTIVE_CONTEXT_A;
                          break;
                      case HAL_TSIP_B:
                          *pContext = PSP_TSIP_ACTIVE_CONTEXT_B;
                          break;
                      case HAL_TSIP_NONE:
                          *pContext = PSP_TSIP_ACTIVE_CONTEXT_NONE;
                          break;
                      default:
                           result = PSP_E_INVALID_MODE;
                           break;
                     }
                 }
                 else
                 {
                     result = PSP_E_INVAL_PARAM;
                 }
                 break;

             /***  10. Get the current active bitmap ***/
             case PSP_TSIP_IOCTL_GET_ACTIVE_BITMAP:

                 if( NULL != arg)
                 {
                     HAL_TsipChanBmapCtxt   context;
                     PSP_TsipActiveBitmap  *pAB = (PSP_TsipActiveBitmap *)arg;

                     HAL_tsipGetActiveBitmap(ioHandle, &context);

                     switch(context)
                     {
                      case HAL_TSIP_A:
                          *pAB = PSP_TSIP_ACTIVE_BITMAP_A;
                          break;
                      case HAL_TSIP_B:
                          *pAB = PSP_TSIP_ACTIVE_BITMAP_B;
                          break;
                      case HAL_TSIP_NONE:
                          *pAB = PSP_TSIP_ACTIVE_BITMAP_NONE;
                          break;
                      default:
                           result = PSP_E_INVALID_MODE;
                           break;
                     }
                 }
                 else
                 {
                     result = PSP_E_INVAL_PARAM;
                 }
                 break;

             /***  11. Change/Choose the current active bitmap ***/
             case PSP_TSIP_IOCTL_SET_ACTIVE_BITMAP:
                 {
                     PSP_tsipCID  *pCID;

                     if(NULL != arg)
                     {
                         pCID = (PSP_tsipCID *)arg;

                         switch(pCID->bitmap)
                         {

                          case PSP_TSIP_ACTIVE_BITMAP_A:
                             if(0 == (pCID->CID & 0x1))
                             {
                                /* Update aCID of ioObject */
                                ioHandle->aCID = pCID->CID;

                                /* Initialize CID with aCID */
                                CSL_FINS(
                                     ioHandle->chenRegs->CHEN,
                                     TSIP_CHEN_CID,
                                     ioHandle->aCID
                                     );

                             }
                             else /* Error:  lsb of CID is not zero for A */
                             {
                                  result = PSP_E_INVAL_PARAM;
                             }
                             break;

                          case PSP_TSIP_ACTIVE_BITMAP_B:
                             if(1 == (pCID->CID & 0x1))
                             {
                                /* Update bCID of ioObject */
                                ioHandle->bCID = pCID->CID;

                                /* Initialize CID with bCID */
                                CSL_FINS(
                                     ioHandle->chenRegs->CHEN,
                                     TSIP_CHEN_CID,
                                     ioHandle->bCID
                                     );

                             }
                             else /* Error: lsb of CID is not one for B */
                             {
                                  result = PSP_E_INVAL_PARAM;
                             }
                             break;

                          case PSP_TSIP_ACTIVE_BITMAP_NONE:
                          default:
                              result = PSP_E_INVAL_PARAM;
                              break;
                         }
                     }
                     else
                     {
                         result = PSP_E_INVAL_PARAM;
                     }
                 }

                 break;

             /***  12. Configure/Setup SIU module ***/
             case PSP_TSIP_IOCTL_SETUP_SIU:
                 {
                     PSP_tsipSiuConf  const *pSiu;


                     if(NULL != arg)
                     {
                         pSiu = (PSP_tsipSiuConf const *)arg;

                         PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);
                         /* If SIU is in running state, SIU configuration
                            should not be done */
                         if( (TRUE == ioHandle->devHandle->isRxSiuON)
                             || (TRUE == ioHandle->devHandle->isTxSiuON) )
                         {
                             result = PSP_E_INVAL_STATE;
                         }
                         else
                         {
                             result = HAL_tsipSiuSetup(
                                                   ioHandle->devHandle,
                                                   pSiu
                                                   );
                         }

                         PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

                         if( PSP_SOK == result )
                         {
                             /* Store new SIU setup values in TSIP HW setup */
                             PAL_osMemCopy(
                                       (Ptr)&ioHandle->devHandle->hwSetup.siu,
                                       (Ptr)pSiu,
                                       sizeof(PSP_tsipSiuConf)
                                       );
                         }

                     }
                     else
                     {
                         result = PSP_E_INVAL_PARAM;
                     }
                 }
                 break;

             /***  13. Configure/Setup TDMU-DMATCU modules ***/
             case PSP_TSIP_IOCTL_SETUP_TDMU:
                 {
                     PSP_tsipTdmuConf  const *pTdmu;

                     if(NULL != arg)
                     {
                         pTdmu = (PSP_tsipTdmuConf const *)arg;

                         PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);
                         /* If TDMU_DMATCU is in running state, TDMU configuration
                            should not be done */
                         if( TRUE == ioHandle->devHandle->isTdmuON )
                         {
                             result = PSP_E_INVAL_STATE;
                         }
                         else
                         {
                             result = HAL_tsipTdmuSetup(
                                                    ioHandle->devHandle,
                                                    pTdmu
                                                    );
                         }
                         PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

                         if( PSP_SOK == result )
                         {
                             /* Store new TDMU setup values in TSIP HW setup */
                             PAL_osMemCopy(
                                       (Ptr)&ioHandle->devHandle->hwSetup.tdmu,
                                       (Ptr)pTdmu,
                                       sizeof(PSP_tsipTdmuConf)
                                       );
                         }

                     }
                     else
                     {
                         result = PSP_E_INVAL_PARAM;
                     }
                 }
                 break;

             /***  14. Get the current TSIP entire HW setup ***/
             case PSP_TSIP_IOCTL_GET_HW_SETUP:
                 {
                     PSP_tsipHwSetup  *pHw;

                     if(NULL != arg)
                     {
                         pHw = (PSP_tsipHwSetup *)arg;

                         /* Get SIU configurations */
                         result = HAL_tsipGetSiuSetup(
                                                  ioHandle->devHandle,
                                                  &pHw->siu
                                                  );

                         /* Get TDMU-DMATCU configurations */
                         result |= HAL_tsipGetTdmuSetup(
                                                    ioHandle->devHandle,
                                                    &pHw->tdmu
                                                    );
                     }
                     else
                     {
                         result = PSP_E_INVAL_PARAM;
                     }
                 }
                 break;

             /***  15. Software Reset SIU ***/
             case PSP_TSIP_IOCTL_RESET_SIU:

                 result = HAL_tsipSiuReset(ioHandle->devHandle);

                 if( PSP_SOK == result )
                 {
                     /* Turn Off isTxSiuON flag */
                     ioHandle->devHandle->isTxSiuON = FALSE;
                     /* Turn Off isTxSiuON flag */
                     ioHandle->devHandle->isRxSiuON = FALSE;
                 }

                 break;

             /***  16. Software Reset TDMU-DMATCU ***/
             case PSP_TSIP_IOCTL_RESET_TDMU:

                 result = HAL_tsipTdmuReset(ioHandle->devHandle);

                 if( PSP_SOK == result )
                 {
                     /* Turn Off isTdmuON flag */
                     ioHandle->devHandle->isTdmuON = FALSE;
                     /* Turn Off Dma Channel enable flage */
                     ioHandle->isChEnabled = FALSE;
                 }
                 break;

             /***  17. Changing flush semaphore pend timeout value ***/
             case PSP_TSIP_IOCTL_SET_SEM_TIMEOUT:

                 if(NULL != arg)
                 {
                     /* Change the semTimeOut value */
                     ioHandle->semTimeOut = *(Int32 *)arg;

                 }
                 else
                 {
                     result = PSP_E_INVAL_PARAM;
                 }

                 break;

             /*** XX. INVALID COMMAND ***/
             default:

                 result = PSP_E_INVALID_CMD; /* hey dude! Invalid IOCTL command */
#ifdef _DEBUG
                 TSIP_DEBUG("PSP_tsipIoctl <Invalid IOCTL Command>");
#endif
                 break;

            }
        }
        else
        {
            result = PSP_E_INVAL_STATE;
        }
    }

    return result;
}


/**
 *  \brief    Submits IOP requests to perform input/output
 *
 *  This function submits the IOP to ddc_tsipSubmitIop() which inturn
 *  do the appropriate queuing of IOP, configures the TX/RX DMATCU channel
 *  and enables the channel.
 *
 *  \param    pIoHandle  [IN]     Handle to the TSIP channel
 *  \param    pIoReq     [IN]     Pointer to IO request
 *
 *  \return
 *            PSP_SOK           if success<br>
 *            PSP Error Codes   if failure<br>
 */
PSP_Result PSP_tsipSubmit(
                      PSP_Handle                 pIoHandle,
                      PSP_tsipIoRequest  const  *pIoReq
                      )
{
    register PSP_Result   result    = PSP_SOK;
    DDC_tsipIoHandle      ioHandle  = NULL;
    Int32                 index     = 0;

    if((NULL != pIoHandle) && (NULL != pIoReq))
    {
        ioHandle = (DDC_tsipIoHandle)pIoHandle;

        if( DDC_TSIP_CHAN_ALLOTTED == ioHandle->status )
        {

            /* Validating IO request parameters */

            if( ( 0x1u == (pIoReq->numSframe & 0x1u) )
                 || (PSP_TSIP_MAX_IOP_SFRAME < pIoReq->numSframe) )
            {
                /* number of superframe in this IOP is not an even number OR
                   greater than the maximum supported size */
                result = PSP_E_INVAL_PARAM;
            }


            /* Validating frameAlloc parameters of A and B configurations */
            if( PSP_SOK == result )
            {
                for(index = 0; index < (pIoReq->numSframe/2) ; index++)
                {

                    /* If frame allocation is less than frame size
                       for A context */
                    if( pIoReq->pIoParams[index].dmaA.frameAlloc <
                           pIoReq->pIoParams[index].dmaA.frameSize )
                    {
                        result = PSP_E_INVAL_PARAM;
                        break;
                    }
                    /* If frame allocation is less than frame size
                       for B context */
                    if( pIoReq->pIoParams[index].dmaB.frameAlloc <
                           pIoReq->pIoParams[index].dmaB.frameSize )
                    {
                        result = PSP_E_INVAL_PARAM;
                        break;
                    }
                }
            }

            /* If io params input validations are OK */
            if( PSP_SOK == result )
            {
                /* Submit IOP request */
                result = ddc_tsipSubmitIop(ioHandle, pIoReq);
            }
        }
        else
        {
            result = PSP_E_INVAL_STATE;
#ifdef _DEBUG
            TSIP_DEBUG("PSP_tsipSubmit <Invalid State>");
#endif

        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
        TSIP_DEBUG("PSP_tsipSubmit <Invalid Params>");
#endif

    }

    return result;
}

/**
 *  \brief    Aborts the TSIP channel IOP operations
 *
 *   Aborts the input or output channel operation of the TSIP driver
 *   and flush all pending I/O packets. Driver sends the
 *   Driver sends the notifications to the application for all pending I/O
 *   packets after abort(Disabling the channel). The application can submit
 *   fresh I/O requests after abort.
 *
 *  \param    pIoHandle   [IN]     Handle to the TSIP channel
 *
 *  \return
 *            PSP_SOK           if success<br>
 *            PSP Error Codes   if failure<br>
 */
PSP_Result PSP_tsipAbort(PSP_Handle pIoHandle)
{
    PSP_Result        result   = PSP_SOK;
    DDC_tsipIoHandle  ioHandle = NULL;

    if( NULL != pIoHandle)
    {
        ioHandle = (DDC_tsipIoHandle)pIoHandle;

        if( DDC_TSIP_CHAN_ALLOTTED == ioHandle->status )
        {
            result = ddc_tsipAbort(ioHandle, PSP_TSIP_IO_ABORTED);
        }
        else
        {
            result = PSP_E_INVAL_STATE;
#ifdef _DEBUG
            TSIP_DEBUG("PSP_tsipAbort <Invalid State>");
#endif
        }

    }
    else
    {
        result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
        TSIP_DEBUG("PSP_tsipAbort <Invalid Params>");
#endif
    }

    return result;
}

/**
 *  \brief    Flush the TSIP channel IOP operations
 *
 *   Flush the input or output channel operation of the TSIP driver.
 *   If the channel is RX, this function aborts(same as PSP_tsipAbort) all
 *   pending RX IOPs and if the channel is TX, this function waits(pending
 *   on the semaphore) for all pending TX IOPs to be completed by the driver.
 *
 *  \note FLUSH and ABORT functionalities are the same for RX channel.
 *
 *  \param    pIoHandle   [IN]     Handle to the TSIP channel
 *
 *  \return
 *            PSP_SOK           if success<br>
 *            PSP Error Codes   if failure<br>
 */
PSP_Result PSP_tsipFlush(PSP_Handle pIoHandle)
{
    PSP_Result          result   = PSP_SOK;
    DDC_tsipIoHandle    ioHandle = NULL;
    Uint32              cookie   = 0;

    if( NULL != pIoHandle )
    {
        ioHandle = (DDC_tsipIoHandle)pIoHandle;

        if( DDC_TSIP_CHAN_ALLOTTED == ioHandle->status )
        {
            /* If it is an RX channel then simply abort all requests */
            if( PSP_TSIP_MODE_RX == ioHandle->mode )
            {
                result = ddc_tsipAbort(ioHandle, PSP_TSIP_IO_FLUSHED);
            }
            else
            {
               if( PSP_TSIP_MODE_TX == ioHandle->mode )
               {
                   /* Wait for all TX iops to complete normally. Pend on
                      flush semaphore.*/

                   if( 0 != ioHandle->iopSubmitCnt )
                   {
                       PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

                         /* Activate Flush Command Flag */
                       ioHandle->isFlushCmd = TRUE;

                       PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

                        /* Pend on flush semaphore until all TX IOPs are
                           completed. This semaphore will be released in
                           TX Superframe ISR */
                       PAL_osSemTake(ioHandle->flushSem,ioHandle->semTimeOut);
                   }
                   else
                   {
                       /* NO IOPs are in the TX pending queue. No need to flush.
                          Simply return SUCCESS */
                       result = PSP_SOK;
                   }

               }
               else
               {
                   result = PSP_E_INVALID_MODE;
               }
            }
        }
        else
        {
            result = PSP_E_INVAL_STATE;
#ifdef _DEBUG
            TSIP_DEBUG("PSP_tsipFlush <Invalid State>");
#endif
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
        TSIP_DEBUG("PSP_tsipFlush <Invalid Params>");
#endif
    }

    return result;
}


/***************************************************************
 * Private Functions Definitions
 ***************************************************************/


/**
 *  \brief    ddc_tsipSubmitIop
 *
 *  This function does the following things.
 *      - If the iop request is the very first request, it configures the
 *        TX/RX DMATCU channel with the first IOP buffer details and enables
 *        the channel.
 *      - If the iop request is not the first one, i.e already some IOPs are
 *        pending, it enqueues this IOP request.
 *      - Return IOP in PROGRESS status for both cases.
 *
 *  \param    pIoHandle  [IN]     Handle to the TSIP channel
 *  \param    pIoReq     [IN]     Packet passed from the application for
 *                                processing
 *
 *  \return
 *            PSP_SOK           if success<br>
 *            PSP_SINPROGRESS   if iop is pending<br>
 *            PSP_E_RESOURCES   if iop resoure is not available<br>
 *            PSP_E_INVAL_PARAM if invalid input parameters<br>
 *
 */
static PSP_Result ddc_tsipSubmitIop(
                            DDC_tsipIoHandle          pIoHandle,
                            PSP_tsipIoRequest const  *pIoReq
                            )
{
    PSP_Result          result  = PSP_SOK;
    Uint32              cookie  = 0;
    DDC_tsipIop        *pIop    = NULL;

#ifdef PSP_TSIP_CACHEABLE_FRAMEBUFFER
    Uint32              i = 0; /* For loop variable */
#endif

    if( (NULL == pIoHandle) || (NULL == pIoReq) )
    {
        result = PSP_E_INVAL_PARAM;
    }
    else
    {

        /* Check whether IOP resource is free to use */
        if( TRUE == PAL_osListIsEmpty(&pIoHandle->queueFreeList) )
        {
            result = PSP_E_RESOURCES; /* All IOPs are in use */
#ifdef _DEBUG
            TSIP_DEBUG("All IOPs are in use <NO RESOURCE>");
#endif
        }
        else
        {
            /* Extract a iop node from freelist queue */
            pIop = (DDC_tsipIop *)PAL_osListDequeue(
                                              &pIoHandle->queueFreeList
                                              );

            if( NULL != pIop )
            {

#ifdef TSIP_INSTRUMENTATION_ENABLED
                if(PSP_TSIP_MODE_RX == pIoHandle->mode)
                {
                    PSP_LOG_EVENT(
                                  &DVTEvent_Log,
                                  "TSIP",
                                  PSP_DVT_DESC(
                                          PSP_DVT_eRCV_START,
                                          PSP_DVT_dCHANNEL_ID_INST,
                                          PSP_DVT_dPACKET_ID,
                                          PSP_DVT_dNONE
                                          ),
                                  CHANNELID_INSTANCEID(
                                          PSP_TSIP_MODE_RX,
                                          pIoHandle->devHandle->devId
                                          ),
                                  pIoReq->pIoParams
                                 );
                }
                else
                {
                    PSP_LOG_EVENT(
                                  &DVTEvent_Log,
                                  "TSIP",
                                  PSP_DVT_DESC(
                                          PSP_DVT_eDATA_SND_START,
                                          PSP_DVT_dCHANNEL_ID_INST,
                                          PSP_DVT_dPACKET_ID,
                                          PSP_DVT_dNONE
                                          ),
                                  CHANNELID_INSTANCEID(
                                          PSP_TSIP_MODE_TX,
                                          pIoHandle->devHandle->devId
                                          ),
                                  pIoReq->pIoParams
                                 );
                }
#endif/* TSIP_INSTRUMENTATION_ENABLED */

                /*copy the request into this node */
                ddc_tsipCopyReq2Iop(pIop,pIoReq);

#ifdef PSP_TSIP_CACHEABLE_FRAMEBUFFER
               /* Maintain cache coherency */
                if (PSP_TSIP_MODE_RX == pIoHandle->mode)
                {
                    /* Invalidate the D cache corresponds to all
                       superframe buffers in the IOP in MVA way */
                    for(i = 0; i < (pIoReq->numSframe / 2u); i++)
                    {
                        /* A context */
                        PAL_osCacheInvalidate(
                                   (PAL_osCacheMemAddrSpace)NULL,
                                   (Uint32)pIoReq->pIoParams[i].dmaA.baseAddr,
                                   (pIoReq->pIoParams[i].dmaA.frameAlloc) *
                                   (pIoReq->pIoParams[i].dmaA.frameCnt)
                                   );
                        /* B context */
                        PAL_osCacheInvalidate(
                                   (PAL_osCacheMemAddrSpace)NULL,
                                   (Uint32)pIoReq->pIoParams[i].dmaB.baseAddr,
                                   (pIoReq->pIoParams[i].dmaB.frameAlloc) *
                                   (pIoReq->pIoParams[i].dmaB.frameCnt)
                                   );
                    }
                }
                else
                {
                    /* Flush and Invalidate the D cache corresponds to all
                       superframe buffers in the IOP in MVA way */
                    for(i = 0; i < (pIoReq->numSframe/ 2u); i++)
                    {
                        /* A context */
                        PAL_osCacheFlushAndInvalidate(
                                (PAL_osCacheMemAddrSpace)NULL,
                                (Uint32)pIoReq->pIoParams[i].dmaA.baseAddr,
                                (pIoReq->pIoParams[i].dmaA.frameAlloc) *
                                (pIoReq->pIoParams[i].dmaA.frameCnt)
                                );

                        /* B context */
                        PAL_osCacheFlushAndInvalidate(
                                (PAL_osCacheMemAddrSpace)NULL,
                                (Uint32)pIoReq->pIoParams[i].dmaB.baseAddr,
                                (pIoReq->pIoParams[i].dmaB.frameAlloc) *
                                (pIoReq->pIoParams[i].dmaB.frameCnt)
                                );
                    }
                }
#endif

                /*  Protect iopSubmitCnt */
                PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

                if( 0 == pIoHandle->iopSubmitCnt ) /* Is First IOP requst */
                {
                    /* Point currIop to this request */
                    pIoHandle->currIop = (DDC_tsipIop *)pIop;
#ifdef _DEBUG
                    if( PSP_TSIP_MODE_TX == pIoHandle->mode )
                    {
                        TSIP_DEBUG("First TX IOP request");
                    }
                    else
                    {
                        TSIP_DEBUG("First RX IOP request");
                    }
#endif

                    /* Configure DMATCU contexts with the current IOP */
                    HAL_tsipDmaConfig(
                                  pIoHandle,
                                  &pIoReq->pIoParams[0].dmaA,
                                  HAL_TSIP_A
                                 );

                    HAL_tsipDmaConfig(
                                  pIoHandle,
                                  &pIoReq->pIoParams[0].dmaB,
                                  HAL_TSIP_B
                                 );

                    /* At this point the channel must be in disbled state.*/
                    if( FALSE == pIoHandle->isChEnabled )
                    {
#ifdef _DEBUG
                        if( PSP_TSIP_MODE_TX == pIoHandle->mode )
                        {
                            TSIP_DEBUG("Enabling TX DMATCU channel");
                        }
                        else
                        {
                            TSIP_DEBUG("Enabling RX DMATCU channel");
                        }
#endif
                        /* Enable DMATCU channel */
                        HAL_TsipEnableDmaChan(pIoHandle);

                        /* Set A a active context. Initialize CID with aCID */
                        CSL_FINS(
                             pIoHandle->chenRegs->CHEN,
                             TSIP_CHEN_CID,
                             pIoHandle->aCID
                             );

                        pIoHandle->isChEnabled = TRUE; /* Ch Enabled */
                    }

                    /* Enabling TSIP Engines */

                    /* Is TDMU-DMATCU is not already enabled */
                    if( FALSE == pIoHandle->devHandle->isTdmuON )
                    {
#ifdef _DEBUG
                        TSIP_DEBUG("Enabling TDMU-DMATCU Engine");
#endif

                        /* Enable TDMU-DMATCU    */
                        HAL_TsipEnableDmatcu(pIoHandle->devHandle);
                        /* Turn ON isTdmuON flag */
                        pIoHandle->devHandle->isTdmuON = TRUE;
                    }

                    /* If TxSIU is not already enabled */
                    if( FALSE == pIoHandle->devHandle->isTxSiuON )
                    {
#ifdef _DEBUG
                        TSIP_DEBUG("Enabling TX SIU Engine");
#endif

                        /* Enable SIU Tx section */
                        HAL_TsipEnableTxSiu(pIoHandle->devHandle);
                        /* Turn ON isTxSiuON flag */
                        pIoHandle->devHandle->isTxSiuON = TRUE;
                    }

                    /* If RxSIU is not already enabled */
                    if( FALSE == pIoHandle->devHandle->isRxSiuON )
                    {
#ifdef _DEBUG
                        TSIP_DEBUG("Enabling RX SIU Engine");
#endif

                        /* Enable SIU Rx section */
                        HAL_TsipEnableRxSiu(pIoHandle->devHandle);
                        /* Turn ON isRxSiuON flag */
                        pIoHandle->devHandle->isRxSiuON = TRUE;

                    }
                    /* Only IO is initiated. DMA is performing
                    frame transfer now. So better return INPROGRESS status,
                    even though this is the first IOP request */
                    result = PSP_SINPROGRESS;
                }
                else /* some iop request(s) are already pending */
                {
#ifdef _DEBUG
                    if( PSP_TSIP_MODE_TX == pIoHandle->mode )
                    {
                        TSIP_DEBUG("Non-first TX IOP request<Queuing>");
                    }
                    else
                    {
                        TSIP_DEBUG("Non-first RX IOP request<Queuing>");
                    }
#endif

                    /* Put this IOP into Pending queue */
                    PAL_osList_NP_Enqueue(&pIoHandle->queueReqList, pIop);

                    /* Processing of iop request is not completed. So return
                    pending status. Request will be marked as complete when
                    it is processed by the superframe ISR context.*/

                    result = PSP_SINPROGRESS;

                }

                /*Increment the iop submit count*/
                pIoHandle->iopSubmitCnt++;

                /*  UnProtect iopSubmitCnt */
                PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);


            }
            else
            {
                result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
                TSIP_DEBUG("ddc_tsipSubmitIop <Invalid Params>");
#endif

            }
        }
    }

    return result;
}


/**
 *  \brief    Aborts the TSIP channel IOP requests
 *
 *   Disables the DMA channel and sends the callback notification to application
 *   for all IOPs
 *
 *  \param    ioHandle   [IN]     Handle to the TSIP channel
 *  \param    cmdType    [IN]     Invoked for Flush or Abort
 *
 *  \return
 *            PSP_SOK           if success<br>
 *            PSP Error Codes   if failure<br>
 */
static PSP_Result ddc_tsipAbort(DDC_tsipIoHandle ioHandle, Int32 cmdType)
{
    PSP_Result          result    = PSP_SOK;
    Ptr                 pIop      = NULL;
    Uint32              cookie    = 0;
    PSP_tsipIoRequest   ioReq;

    if( NULL != ioHandle)
    {
        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

        /* Stop(Disable) the  Channel */
        HAL_TsipDisableDmaChan(ioHandle);

        ioHandle->isChEnabled = FALSE; /* Turn OFF channel enable flag */

        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);


        /* Check for outstanding iop submit count */

        if( 0 != ioHandle->iopSubmitCnt )
        {
            /* Some iop(s) are pending */

            /* Abort the current IOP that is in progress */
            if( NULL != ioHandle->currIop )
            {
                /* Set IOP request status to ABORTED/FLUSHED */
                ioHandle->currIop->status = cmdType;

                /* Copy IOP status to Req to be passed to upper layer */
                ddc_tsipCopyIop2Req(&ioReq,ioHandle->currIop);

                /* Invoke driver callback for current IOP */
                (*ioHandle->drvCb)(ioHandle->cbArg, &ioReq);

                ioHandle->iopSubmitCnt--; /* Decrement iop pending count*/

                /* Enque the current IOP back to free queue */
                PAL_osList_NP_Enqueue(&ioHandle->queueFreeList, ioHandle->currIop);

                /* Set NULL to current IOP pointer */
                ioHandle->currIop = NULL;
            }


           /* Empty pending iop(s) in the pending queue and
            * assert call backs for all pending iops */
            while(FALSE == PAL_osListIsEmpty(&ioHandle->queueReqList))
            {
                /* Extract iop node from pending queue */
                pIop = PAL_osListDequeue(&ioHandle->queueReqList);

                if( NULL != pIop)
                {
                    /* Set IOP request status to ABORTED/FLUSHED */
                    ((DDC_tsipIop *)pIop)->status = cmdType;

                    /* Copy IOP status to Req to be passed to upper layer */
                    ddc_tsipCopyIop2Req(&ioReq, pIop);
                }


                /* Invoke driver callback */
                (*ioHandle->drvCb)(ioHandle->cbArg, &ioReq);

                ioHandle->iopSubmitCnt--; /* Decrement outstanding iop
                                            pending count*/
                /* Enque the packet back to free queue */
                PAL_osList_NP_Enqueue(&ioHandle->queueFreeList, pIop);

            }
            /* Iop submit count should be zero at this point. But for
               the sake of assurance, clear it explicitly here */
            ioHandle->iopSubmitCnt = 0;
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
        TSIP_DEBUG("ddc_tsipAbort <Invalid Params>");
#endif
    }

    return result;
}


/*****************************
 * Interrupt Handlers (ISRs)
 *****************************/

#ifdef PSP_TSIP_USE_FRAME_ISR
/**
 * \brief    Tx Frame Interrupt Handler
 *
 *  Called on every TX frame sync interrupt assertion
 *
 * \param    pIoHandle    [IN]   DDC channel handle
 */
/* ARGSUSED */
static void ddc_tsipTxFrameISR(PSP_Handle pIoHandle)
{
    DDC_tsipIoHandle        ioHandle = NULL;

#ifdef PSP_TSIP_DEBUG
    HAL_TsipChanBmapCtxt    tsCtxt;         /* To hold current active context */
#endif

    if( NULL != pIoHandle )
    {
        ioHandle = pIoHandle;

#ifdef TSIP_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(
                      &DVTEvent_Log,
                      "TSIP",
                      PSP_DVT_DESC(
                              PSP_DVT_eINT_START,
                              PSP_DVT_dINST,
                              PSP_DVT_dCHANNEL_ID,
                              PSP_DVT_dNONE
                              ),
                      ioHandle->devHandle->devId,
                      ioHandle->mode
                     );
#endif /* TSIP_INSTRUMENTATION_ENABLED */

#ifdef PSP_TSIP_DEBUG
        TSIP_DEBUG1("Tx FRFC[%d]",ioHandle->devHandle->pReg->XFRFC);
        HAL_tsipGetActiveContext(ioHandle, &tsCtxt);
        TSIP_DEBUG1("Tx ActiveContext[%d]",tsCtxt);
#endif
        /* If frame interrupt Callback is registered, invoke the callback */
        if( NULL != ioHandle->frmCb )
        {
            (*ioHandle->frmCb)(ioHandle->fCbArg);
        }

#ifdef TSIP_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(
                      &DVTEvent_Log,
                      "TSIP",
                      PSP_DVT_DESC(
                              PSP_DVT_eINT_END,
                              PSP_DVT_dINST,
                              PSP_DVT_dCHANNEL_ID,
                              PSP_DVT_dNONE
                             ),
                      ioHandle->devHandle->devId,
                      ioHandle->mode
                     );
#endif /* TSIP_INSTRUMENTATION_ENABLED */
    }

#ifdef _DEBUG
    tsipTxFrameCnt++; /* Increment tx frame interrupt diagnostic counter */
#endif

    return;
}

/**
 * \brief    Rx Frame Interrupt Handler
 *
 *  Called on every RX frame sync interrupt assertion
 *
 * \param    pIoHandle    [IN]   DDC channel handle
 */
/* ARGSUSED */
static void ddc_tsipRxFrameISR(PSP_Handle pIoHandle)
{
    DDC_tsipIoHandle        ioHandle = NULL;

#ifdef PSP_TSIP_DEBUG
    HAL_TsipChanBmapCtxt    rsCtxt;         /* To hold current active context */
#endif

    if( NULL != pIoHandle )
    {
        ioHandle = pIoHandle;

#ifdef TSIP_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(
                      &DVTEvent_Log,
                      "TSIP",
                      PSP_DVT_DESC(
                              PSP_DVT_eINT_START,
                              PSP_DVT_dINST,
                              PSP_DVT_dCHANNEL_ID,
                              PSP_DVT_dNONE
                             ),
                      ioHandle->devHandle->devId,
                      ioHandle->mode
                     );
#endif /* TSIP_INSTRUMENTATION_ENABLED */

#ifdef PSP_TSIP_DEBUG
        TSIP_DEBUG1("Rx FRFC[%d]",ioHandle->devHandle->pReg->RFRFC);
        HAL_tsipGetActiveContext(ioHandle, &rsCtxt);
        TSIP_DEBUG1("Rx ActiveContext[%d]",rsCtxt);
#endif
        /* If frame interrupt Callback is registered, invoke the callback */
        if( NULL != ioHandle->frmCb )
        {
            (*ioHandle->frmCb)(ioHandle->fCbArg);
        }

#ifdef TSIP_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(
                      &DVTEvent_Log,
                      "TSIP",
                      PSP_DVT_DESC(
                              PSP_DVT_eINT_END,
                              PSP_DVT_dINST,
                              PSP_DVT_dCHANNEL_ID,
                              PSP_DVT_dNONE
                              ),
                      ioHandle->devHandle->devId,
                      ioHandle->mode
                     );
#endif /* TSIP_INSTRUMENTATION_ENABLED */
    }

#ifdef _DEBUG
    tsipRxFrameCnt++; /* Increment rx frame interrupt diagnostic counter */
#endif

    return;

}
#endif /* PSP_TSIP_USE_FRAME_ISR */



/**
 * \brief    Rx Super Frame Interrupt Handler
 *
 * This routine is called upon reception of every RX super frame (Rx Super
 * frame interrupt assertion)
 *
 * \param    pIoHandle    [IN]   DDC channel handle
 *
 */
static void ddc_tsipRxSuperFrameISR(PSP_Handle pIoHandle)
{
    DDC_tsipIoHandle        ioHandle = NULL;
    DDC_tsipIop            *pIop     = NULL;
    PSP_tsipIoParams       *pIoArg   = NULL;/* To hold current IOP's IO params*/
    Uint32                  rsXfer   = 0;   /* To hold actual xferd S-Frm cnt */
    HAL_TsipChanBmapCtxt    rsCtxt;         /* To hold current active context */
    PSP_tsipIoRequest       ioReq;
#ifdef PSP_TSIP_DEBUG
    HAL_TsipChanBmapCtxt    rsBitMap;     /* To hold current active bitmap   */
#endif

#ifdef _DEBUG
    tsipRxSuperFrameCnt++; /* Increment rx super frame interrupt counter */
#endif

    /**************************************************************************
     * Steps followed in Rx superframe ISR are as follows:
     *
     * 1) As per the TSIP driver design each IOP should have even number of
     *    super-frame buffers and the minimum sframeCnt is thus 2. One for A
     *    context and the other for the B context. This ISR is triggered at the
     *    completion of the every superframe.
     *
     * 2) At first, this routine increments the actual number of superframes
     *    transferred(xferActual) counter.
     *
     * 3) Gets the current Active context.
     *
     * 4) Configuration of the next S-frame DMATCU buffer from the IOP is
     *    done as follows,
     *      a) If the next superframe configuration for AorB context is within
     *         the current IOP, it adjusts the pIoArg pointer to point to the
     *         next AorB configuration buffer within the IOP.
     *      b) If it processes the last super-frame in the IOP, it will check
     *         the IOP pending queue to know whether any IOP is pending.
     *          a1) If IOP is available, it will extract an IOP from pending
     *              queue and assigns pIoArg pointer with the extracted IOP's
     *              IO argument.
     *          a2) If IOP pending queue is empty, it will disable the DMA
     *              channel. And sets active context flag to NONE to make
     *              no next DMA context configuration for AorB.
     *          a3) Completes the current IOP by invoking callback. After call
     *              back is invoked it puts the completed IOP in the free list.
     *              And it updates the currIop with the next IOP(if available).
     *       c) Configures the CID of the channel to make the next AorB context.
     *          If the current active context is A, it will enable B and vice
     *          versa using the pIoArg pointer which is pointing to IOP param.
     *
     * 5) Checks whether the application has invoked channel
     *    STOP EXTENDED IOCTL command, if so, it STOPs the DMA channel
     *
     * 6) At the end, it checks whether the application has registered sframe
     *    Callback function, if yes, it will invoke the callback before
     *    returning from the ISR.
     **************************************************************************/
    if( NULL != pIoHandle )
    {

#ifdef TSIP_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(
                      &DVTEvent_Log,
                      "TSIP",
                      PSP_DVT_DESC(
                              PSP_DVT_eINT_START,
                              PSP_DVT_dINST,
                              PSP_DVT_dCHANNEL_ID,
                              PSP_DVT_dNONE
                              ),
                      ioHandle->devHandle->devId,
                      ioHandle->mode
                      );
#endif /*TSIP_INSTRUMENTATION_ENABLED */

        ioHandle = (DDC_tsipIoHandle)pIoHandle;

        /* Increment actual transfer sframe counter */
        rsXfer = ++ioHandle->currIop->xferActual;

#ifdef _DEBUG
        TSIP_DEBUG1("Completed Rx IOP superframe[%d]",rsXfer);
#endif

        /* Get the current active context */
        HAL_tsipGetActiveContext(ioHandle, &rsCtxt);

#ifdef PSP_TSIP_DEBUG
        TSIP_DEBUG1("Rx Active Context[%d]",rsCtxt);

        HAL_tsipGetActiveBitmap(ioHandle, &rsBitMap);
        TSIP_DEBUG1("Rx Active Bitmap[%d]",rsBitMap);
#endif

        /* Is next superframe configuration within the current IOP? */
        if( rsXfer < ioHandle->currIop->numSframe )
        {
            /* Get the next IO params(which has dmaA+dmaB) address
               into pIoArg */
            pIoArg = &ioHandle->currIop->ioParams[(rsXfer/2)];

        }
        else /* Next sframe config is in next IOP */
        {
            /* Is this last superframe in the IOP? */
            if( rsXfer == ioHandle->currIop->numSframe )
            {
#ifdef PSP_TSIP_DEBUG
                TSIP_DEBUG("Rx-SISR <Last sFrame of IOP>");
#endif
                /* Is IOP pending queue not empty */
                if( FALSE == PAL_osList_NP_IsEmpty(&ioHandle->queueReqList) )
                {
                    /* Extract next iop from pending queue */
                    pIop = (DDC_tsipIop *)PAL_osList_NP_Dequeue(
                                                      &ioHandle->queueReqList
                                                      );

                    if( NULL != pIop ) /* pIop must not be NULL here. */
                    {
                        /* Assign pIoArg with new IOP's IO params */
                        pIoArg = &pIop->ioParams[0];
                    }

                }
                else /* No IOPs are pending */
                {
#ifdef _DEBUG
                    TSIP_DEBUG("No pending RX IOPs");
#endif
                    /* Stop the Channel. Note: Once DMA channel is disabled,
                    the active context will be set to NONE by TSIP on the next
                    frame. */
                    HAL_TsipDisableDmaChan(ioHandle);

                    /* Change the channel status to stopped */
                    ioHandle->isChEnabled   = FALSE;

                    /* Clear CID */
                    CSL_FINS(ioHandle->chenRegs->CHEN, TSIP_CHEN_CID, 0x00);

                    /* Set rsCtxt to NONE, becz no need to configure
                       DMA for next A or B contexts */
                    rsCtxt = HAL_TSIP_NONE;
                }

                /******* Completion of  the current IOP ********/

                /* Update status of this IOP */
                ioHandle->currIop->status = PSP_SOK;

                ddc_tsipCopyIop2Req(&ioReq,ioHandle->currIop);

                /* Invoke Callback */
                (*ioHandle->drvCb)(ioHandle->cbArg,&ioReq);

                /* Decrement iop pending count */
                ioHandle->iopSubmitCnt--;

                /* Put the completed IOP into Free list */
                PAL_osList_NP_Enqueue(
                                 &ioHandle->queueFreeList,
                                 (Ptr)ioHandle->currIop
                                 );

                if( TRUE == ioHandle->isChEnabled )
                {
                    /* Point current IOP to next IOP */
                    ioHandle->currIop = pIop;
                }
                else
                {
                    ioHandle->currIop = NULL; /* No next IOP */
                }

            }

        }

        switch(rsCtxt)
        {
         case HAL_TSIP_A:

             /* A context is active now. So configure the
              next B context */
             if( NULL != pIoArg )
             {
                 HAL_tsipDmaConfig(
                               ioHandle,
                               (const PSP_tsipDmaConfig *)&pIoArg->dmaB,
                               HAL_TSIP_B
                               );
                 /* Configure CID to select B context channel and bitmap */
                 CSL_FINS(
                      ioHandle->chenRegs->CHEN,
                      TSIP_CHEN_CID,
                      ioHandle->bCID
                      );
             }

             break;

         case HAL_TSIP_B:

             /* B context is active now. So configure the
              next A context */
             if( NULL != pIoArg )
             {
                 HAL_tsipDmaConfig(
                               ioHandle,
                               (const PSP_tsipDmaConfig *)&pIoArg->dmaA,
                               HAL_TSIP_A
                               );
                 /* Configure CID to select A context channel and bitmap */
                 CSL_FINS(
                      ioHandle->chenRegs->CHEN,
                      TSIP_CHEN_CID,
                      ioHandle->aCID
                      );

             }
             break;

         case HAL_TSIP_NONE:
         default:
#ifdef _DEBUG
             TSIP_DEBUG("Rx Context[NONE]");
#endif
             break;
        }


        /* Check whether stop channel extended command is passed */
        if( TRUE == ioHandle->isStopExtCmd )
        {
            /* Disable the channel */
            HAL_TsipDisableDmaChan(ioHandle);
            /* Acknowledge stop channel ext ioctl command */
            ioHandle->isStopExtCmd = FALSE;
        }

        /* If sframe interrupt Callback is registered, invoke the callback */
        if( NULL != ioHandle->sfrmCb )
        {
            (*ioHandle->sfrmCb)(ioHandle->sCbArg);
        }

#ifdef TSIP_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(
                      &DVTEvent_Log,
                      "TSIP",
                      PSP_DVT_DESC(
                              PSP_DVT_eINT_END,
                              PSP_DVT_dINST,
                              PSP_DVT_dCHANNEL_ID,
                              PSP_DVT_dNONE
                              ),
                      ioHandle->devHandle->devId,
                      ioHandle->mode
                     );
#endif /*TSIP_INSTRUMENTATION_ENABLED */


    } /* pointer validation if loop */
    else
    {
        /* Hey! Invalid channel handle in ISR */
#ifdef _DEBUG
        TSIP_DEBUG("Rx S-ISR <Invalid Params>");
#endif

    }

    return;
}

/**
 * \brief    Tx Super Frame Interrupt Handler
 *
 * This routine is called upon completion of every TX super frame (TX Super
 * frame interrupt assertion)
 *
 * \param    pIoHandle    [IN]   DDC channel handle
 *
 */
static void ddc_tsipTxSuperFrameISR(PSP_Handle pIoHandle)
{
    DDC_tsipIoHandle        ioHandle = NULL;
    DDC_tsipIop            *pIop     = NULL;
    PSP_tsipIoParams       *pIoArg   = NULL;/* To hold current IOP's IO params*/
    Uint32                  tsXfer   = 0;   /* To hold actual xferd S-Frm cnt */
    HAL_TsipChanBmapCtxt    tsCtxt;         /* To hold current active context */
    PSP_tsipIoRequest       ioReq;

#ifdef PSP_TSIP_DEBUG
    HAL_TsipChanBmapCtxt    tsBitMap;     /* To hold current active bitmap   */
#endif

#ifdef _DEBUG
    tsipTxSuperFrameCnt++; /* Increment tx super frame interrupt counter */
#endif
    /**************************************************************************
     * Steps followed in Tx superframe ISR are as follows:
     *
     * 1) As per the TSIP driver design each IOP should have even number of
     *    super-frame buffers and the minimum sframeCnt is thus 2. One for A
     *    context and the other for the B context. This ISR is triggered at the
     *    completion of the every superframe.
     *
     * 2) At first, this routine increments the actual number of superframes
     *    transferred(xferActual) counter.
     *
     * 3) Gets the current Active context.
     *
     * 4) Configuration of the next S-frame DMATCU buffer from the IOP is
     *    done as follows,
     *      a) If the next superframe configuration for AorB context is within
     *         the current IOP, it adjusts the pIoArg pointer to point to the
     *         next AorB configuration buffer within the IOP.
     *      b) If it processes the last super-frame in the IOP, it will check
     *         the IOP pending queue to know whether any IOP is pending.
     *          a1) If IOP is available, it will extract an IOP from pending
     *              queue and assigns pIoArg pointer with the extracted IOP's
     *              IO argument.
     *          a2) If IOP pending queue is empty, it will disable the DMA
     *              channel. And sets active context flag to NONE to make
     *              no next DMA context configuration for AorB.
     *          a3) Completes the current IOP by invoking callback. After call
     *              back is invoked it puts the completed IOP in the free list.
     *              And it updates the currIop with the next IOP(if available).
     *       c) Configures the CID of the channel to make the next AorB context.
     *          If the current active context is A, it will enable B and vice
     *          versa using the pIoArg pointer which is pointing to IOP param.
     *
     * 5) Checks whether the application has invoked channel
     *    STOP EXTENDED IOCTL command, if so, it STOPs the DMA channel
     *
     * 6) Checks whether the application is pending on FLUSH operation, if so
     *    it releases the flush semaphore if no IOPs are pending.
     *
     * 7) At the end, it checks whether the application has registered sframe
     *    Callback function, if yes, it will invoke the callback before
     *    returning from the ISR.
     **************************************************************************/

    if( NULL != pIoHandle )
    {

#ifdef TSIP_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(
                      &DVTEvent_Log,
                      "TSIP",
                      PSP_DVT_DESC(
                              PSP_DVT_eINT_START,
                              PSP_DVT_dINST,
                              PSP_DVT_dCHANNEL_ID,
                              PSP_DVT_dNONE
                             ),
                      ioHandle->devHandle->devId,
                      ioHandle->mode
                     );
#endif /*TSIP_INSTRUMENTATION_ENABLED */

        ioHandle = (DDC_tsipIoHandle)pIoHandle;

        /* Increment actual transfer sframe counter */
        tsXfer = ++ioHandle->currIop->xferActual;

#ifdef _DEBUG
        TSIP_DEBUG1("Completed Tx IOP superframe[%d]",tsXfer);
#endif

        /* Get the current active context */
        HAL_tsipGetActiveContext(ioHandle, &tsCtxt);

#ifdef PSP_TSIP_DEBUG
        TSIP_DEBUG1("Tx Active Context[%d]",tsCtxt);

        HAL_tsipGetActiveBitmap(ioHandle, &tsBitMap);
        TSIP_DEBUG1("Tx Active Bitmap[%d]",tsBitMap);
#endif

        /* Is next superframe configuration within the current IOP? */
        if( tsXfer < ioHandle->currIop->numSframe )
        {
            /* Get the next IO params(which has dmaA+dmaB) address
               into pIoArg */
            pIoArg = &ioHandle->currIop->ioParams[(tsXfer/2)];

        }
        else /* Next sframe config is in next IOP */
        {
            /* Is this last superframe in the IOP? */
            if( tsXfer == ioHandle->currIop->numSframe )
            {
#ifdef PSP_TSIP_DEBUG
                TSIP_DEBUG("Tx-SISR <Last sFrame of IOP>");
#endif
                /* Is IOP pending queue not empty */
                if( FALSE == PAL_osList_NP_IsEmpty(&ioHandle->queueReqList) )
                {
                    /* Extract next iop from pending queue */
                    pIop = (DDC_tsipIop *)PAL_osList_NP_Dequeue(
                                                      &ioHandle->queueReqList
                                                      );

                    if( NULL != pIop ) /* pIop must not be NULL here. */
                    {
                        /* Assign pIoArg with new IOP IO params */
                        pIoArg = &pIop->ioParams[0];
                    }

                }
                else /* No IOPs are pending */
                {
#ifdef _DEBUG
                    TSIP_DEBUG("No pending TX IOPs");
#endif

                    /* Stop the Channel. Note: Once DMA channel is disabled,
                    the active context will be set to NONE by TSIP on the next
                    frame. */
                    HAL_TsipDisableDmaChan(ioHandle);

                    /* Change the channel status to stopped */
                    ioHandle->isChEnabled   = FALSE;

                    /* Clear CID */
                    CSL_FINS(ioHandle->chenRegs->CHEN, TSIP_CHEN_CID, 0x00);

                    /* Set tsCtxt to NONE, becz no need to configure
                       DMA for next A or B contexts */
                    tsCtxt = HAL_TSIP_NONE;


                }

                /******* Completion of  the current IOP ********/

                /* Update status of this IOP */
                ioHandle->currIop->status = PSP_SOK;

                ddc_tsipCopyIop2Req(&ioReq,ioHandle->currIop);

                /* Invoke Callback */
                (*ioHandle->drvCb)(ioHandle->cbArg,&ioReq);

                /* Decrement iop pending count */
                ioHandle->iopSubmitCnt--;

                /* Put the completed IOP into Free list */
                PAL_osList_NP_Enqueue(
                                 &ioHandle->queueFreeList,
                                 (Ptr)ioHandle->currIop
                                 );

                if( TRUE == ioHandle->isChEnabled )
                {
                    /* Point current IOP to next IOP */
                    ioHandle->currIop = pIop;
                }
                else
                {
                    ioHandle->currIop = NULL; /* No next IOP */
                }

            }

        }

        switch(tsCtxt)
        {
         case HAL_TSIP_A:

             /* A context is active now. So configure the
              next B context */
             if( NULL != pIoArg )
             {
                 HAL_tsipDmaConfig(
                               ioHandle,
                               (const PSP_tsipDmaConfig *)&pIoArg->dmaB,
                               HAL_TSIP_B
                               );
                 /* Configure CID to select B context channel and bitmap */
                 CSL_FINS(
                      ioHandle->chenRegs->CHEN,
                      TSIP_CHEN_CID,
                      ioHandle->bCID
                      );
             }

             break;

         case HAL_TSIP_B:

             /* B context is active now. So configure the
              next A context */
             if( NULL != pIoArg )
             {
                 HAL_tsipDmaConfig(
                               ioHandle,
                               (const PSP_tsipDmaConfig *)&pIoArg->dmaA,
                               HAL_TSIP_A
                               );
                 /* Configure CID to select A context channel and bitmap */
                 CSL_FINS(
                      ioHandle->chenRegs->CHEN,
                      TSIP_CHEN_CID,
                      ioHandle->aCID
                      );

             }
             break;

         case HAL_TSIP_NONE:
         default:
#ifdef _DEBUG
             TSIP_DEBUG("Tx Context[NONE]");
#endif
             break;
        }


        /* Check whether stop channel extended command is passed */
        if( TRUE == ioHandle->isStopExtCmd )
        {
            /* Disable the channel */
            HAL_TsipDisableDmaChan(ioHandle);
            /* Acknowledge stop channel ext ioctl command */
            ioHandle->isStopExtCmd = FALSE;
        }

        if(0 == ioHandle->iopSubmitCnt)
        {
            /* If FLUSH operation is invoked, Release the
               flush Semaphore */
            if( TRUE == ioHandle->isFlushCmd )
            {
                PAL_osSemGive(ioHandle->flushSem);
                ioHandle->isFlushCmd  = FALSE;
            }
        }


        /* If sframe interrupt Callback is registered, invoke the callback */
        if( NULL != ioHandle->sfrmCb )
        {
            (*ioHandle->sfrmCb)(ioHandle->sCbArg);
        }

#ifdef TSIP_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(
                      &DVTEvent_Log,
                      "TSIP",
                      PSP_DVT_DESC(
                              PSP_DVT_eINT_END,
                              PSP_DVT_dINST,
                              PSP_DVT_dCHANNEL_ID,
                              PSP_DVT_dNONE
                             ),
                      ioHandle->devHandle->devId,
                      ioHandle->mode
                     );
#endif /*TSIP_INSTRUMENTATION_ENABLED */

    } /* pointer validation if loop */
    else
    {
        /* Hey! Invalid channel handle in ISR */
#ifdef _DEBUG
        TSIP_DEBUG("Tx S-ISR <Invalid Params>");
#endif

    }

    return;
}

/**
 * \brief    Tx-Rx Channel Error Interrupt Handler
 *
 * \param    pIoHandle    [IN]   DDC channel handle
 *
 * This routine is called upon Tx-Rx error interrupt assertion.
 * This is shared between all Tx-Rx channel of all TSIP intances.
 */
static void ddc_tsipTxRxErrorISR(PSP_Handle pIoHandle)
{
    DDC_tsipIoHandle      ioHandle = NULL;
    PSP_tsipError         err;
    Uint32                errCnt   = 0;
    Uint32                error    = 0;
    Uint32                errCode  = 0;

#ifdef _DEBUG
    tsipErrorCnt++; /* Increment error interrupt diagnostic counter */
#endif

    if(NULL != pIoHandle)
    {
        ioHandle = (DDC_tsipIoHandle)pIoHandle;

        /* Check ErrQueue is overflown */
        if( 1u == HAL_TsipIsErrQueueOvf(ioHandle) )
        {
            /* Clear Overflow condition */
            HAL_TsipClearErrQueueOvf(ioHandle);

        }

        /* Read the error count */
        errCnt = HAL_TsipReadErrCnt(ioHandle);

        while(0 != errCnt)
        {
            /* Read the top error entry from the error queue */
            error = HAL_TsipReadError(ioHandle);

            /* Extract the error code information */
            errCode = HAL_TsipExtErrCode(error);

            /* We have read this one from queue. So POP this read
             entry out of ErrQ i.e next new entry will be at the top */
            HAL_TsipPopErrQueue(ioHandle);


            if( PSP_TSIP_MODE_RX == ioHandle->mode )
            {
                switch( errCode )
                {
                 case 0x01: /* RX Overrun @ frame beginning */
                     ioHandle->errStats.rx.ovRunCnt++;
                     err.errCode = PSP_TSIP_ERROR_RX_ORUN_BEGIN;
                     break;
                 case 0x02: /* RX Overrun during frame beginning */
                     ioHandle->errStats.rx.ovRunCnt++;
                     err.errCode = PSP_TSIP_ERROR_RX_ORUN;
                     break;
                 case 0x03: /* RX Overrun BM gr8 than Size */
                     ioHandle->errStats.rx.ovRunCnt++;
                     err.errCode = PSP_TSIP_ERROR_RX_BITMAP;
                     break;
                 case 0x05: /* RX size less than 8 */
                     ioHandle->errStats.rx.invFrameCnt++;
                     err.errCode = PSP_TSIP_ERROR_RX_INVALID_SIZE;
                     break;
                 default:
                     if( (errCode >= 0xF1u) && (errCode <= 0xF7u) )
                     {
                         /* CBA read status error */
                         ioHandle->errStats.rx.cbaReadCnt++;
                         err.errCode = PSP_TSIP_ERROR_CBA_READ;
                     }
                     else
                     {
                         if( (errCode >= 0xF9u) && (errCode <= 0xFFu) )
                         {
                            /* CBA write status error */
                             ioHandle->errStats.rx.cbaWriteCnt++;
                             err.errCode = PSP_TSIP_ERROR_CBA_WRITE;
                         }
                         else /* Unknown error */
                         {
                             ioHandle->errStats.rx.unKnown++;
                             err.errCode = PSP_TSIP_ERROR_UNKNOWN;
                         }
                     }
                     break;
                }
            }
            else /* TX channel */
            {
                switch( errCode )
                {
                 case 0x10: /* TX config. ID mismatch */
                      ioHandle->errStats.tx.invConfCnt++;
                      err.errCode = PSP_TSIP_ERROR_TX_CONFIG_MISMATCH;
                      break;
                 case 0x11: /* TX Underrun @ frame beginning */
                      ioHandle->errStats.tx.unRunCnt++;
                      err.errCode = PSP_TSIP_ERROR_TX_URUN_BEGIN;
                      break;
                 case 0x12: /* TX Underrun during frame beginning */
                      ioHandle->errStats.tx.unRunCnt++;
                      err.errCode = PSP_TSIP_ERROR_TX_URUN;
                      break;

                 case 0x13: /* TX Underrun BM gr8 than size */
                      ioHandle->errStats.tx.unRunCnt++;
                      err.errCode = PSP_TSIP_ERROR_TX_BITMAP;
                      break;

                 case 0x15: /* TX size less than 8 */
                     ioHandle->errStats.tx.invFrameCnt++;
                     err.errCode = PSP_TSIP_ERROR_TX_INVALID_SIZE;
                     break;

                 default:
                     if( (errCode >= 0xF1u) && (errCode <= 0xF7u) )
                     {
                         /* CBA read status error */
                         ioHandle->errStats.tx.cbaReadCnt++;
                         err.errCode = PSP_TSIP_ERROR_CBA_READ;
                     }
                     else
                     {
                         if( (errCode >= 0xF9u) && (errCode <= 0xFFu) )
                         {
                            /* CBA write status error */
                             ioHandle->errStats.tx.cbaWriteCnt++;
                             err.errCode = PSP_TSIP_ERROR_CBA_WRITE;
                         }
                         else /* Unknown error */
                         {
                             ioHandle->errStats.tx.unKnown++;
                             err.errCode = PSP_TSIP_ERROR_UNKNOWN;
                         }
                     }
                     break;
                }
            }

            /* Invoke global error callback */
           if( NULL != ioHandle->errCb)
           {
               /* Extract the additional error info */
               err.errInfo = HAL_TsipExtErrInfo(error);
               /* Passing error details to error callback along with cb arg */
               (*ioHandle->errCb)(err, ioHandle->eCbArg);
           }

            errCnt--;
        }

    }
    else
    {
        /* Hey! Invalid handle in ISR */
#ifdef _DEBUG
        TSIP_DEBUG("Tx-Rx Err-ISR <Invalid Params>");
#endif

    }
    return;
}

/**
 * \brief    Get interrupt event ID for the TSIP instance
 *
 *  This function returns the corresponding HWI number, Event ID
 *  for tx/rx frame/sframe interrupt OR error interrupt.
 *
 *  \param    pIoHandle    [IN]    Handle to the TSIP channel
 *  \param    intType      [IN]    which one: frame/superframe/error?
 *  \param    pEvtId       [OUT]   Place holder for Event ID
 *
 *  \return   PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
static PSP_Result ddc_tsipGetEvtNum(
                                DDC_tsipIoHandle       pIoHandle,
                                DDC_TsipChanIntType    intType,
                                Uint16                *pEvtId
                                )
{
    PSP_Result     result   = PSP_SOK;
    Int32          port     = 0;

    /****************************************************************
     *  EVENT ID  Look-up table
     ****************************************************************/

    /* Rx frame event number Look-up table */
    static  const Uint16 fRxEvt[PSP_TSIP_NUM_PORTS] =
    {
      CSL_INTC_EVENTID_RFSINT0,  /* port[0] */
      CSL_INTC_EVENTID_RFSINT1   /* port[1] */
    };

    /* Rx super frame event number Look-up table */
    static  const Uint16 sRxEvt[PSP_TSIP_NUM_PORTS]=
    {
      CSL_INTC_EVENTID_RSFINT0, /* port[0] */
      CSL_INTC_EVENTID_RSFINT1  /* port[1] */
    };

    /* Tx frame event number Look-up table */
    static  const Uint16 fTxEvt[PSP_TSIP_NUM_PORTS] =
    {
      CSL_INTC_EVENTID_XFSINT0,  /* port[0] */
      CSL_INTC_EVENTID_XFSINT1   /* port[1] */
    };

    /* Tx super frame event number Look-up table */
    static  const Uint16 sTxEvt[PSP_TSIP_NUM_PORTS] =
    {
      CSL_INTC_EVENTID_XSFINT0, /* port[0] */
      CSL_INTC_EVENTID_XSFINT1  /* port[1] */
    };

    /* Error Interrupt Number Look-up table */
    static  const Uint16 eEvt[PSP_TSIP_NUM_PORTS] =
    {
      CSL_INTC_EVENTID_ERRINT0, /* port[0] */
      CSL_INTC_EVENTID_ERRINT1  /* port[1] */
    };

    if( (NULL == pIoHandle) || (NULL == pEvtId) )
    {
        result = PSP_E_INVAL_PARAM;
    }
    else
    {
        port = pIoHandle->devHandle->devId;

        switch(intType)
        {

         case DDC_TSIP_CHAN_FRAME_INT:
             /* Get Frame Interrupt Event ID */
             if( PSP_TSIP_MODE_RX == pIoHandle->mode )
             {
                 *pEvtId  = fRxEvt[port];
             }
             else
             {
                 *pEvtId  = fTxEvt[port];
             }
             break;

         case DDC_TSIP_CHAN_SFRAME_INT:
             /* Get Super Frame Interrupt Event ID */
             if( PSP_TSIP_MODE_RX == pIoHandle->mode )
             {
                 *pEvtId  = sRxEvt[port];
             }
             else
             {
                 *pEvtId  = sTxEvt[port];
             }
             break;

         case DDC_TSIP_CHAN_ERROR_INT:
             /* Get Error Interrupt Event ID */
             *pEvtId  = eEvt[port];
             break;

         default:
             result = PSP_E_INVALID_MODE;
             break;
        }
    }
    return result;
}

/*
 * Local copy function : Request to IOP
 */
static void ddc_tsipCopyReq2Iop(
                            DDC_tsipIop *pIop,
                            const PSP_tsipIoRequest *pIoReq
                            )
{
    if( (NULL != pIop) && (NULL != pIoReq) )
    {
        /* Copy Number of superframe's configuration into IOP */
        PAL_osMemCopy(
                  (Ptr)&pIop->ioParams[0],
                  (Ptr)pIoReq->pIoParams,
                  (pIoReq->numSframe/2) * sizeof(PSP_tsipIoParams)
                  );

        pIop->numSframe  = pIoReq->numSframe;
        pIop->appPtr     = pIoReq->appPtr;

        pIop->xferActual = 0; /* Clear actual transfer counter */
    }
}


/*
 * Local copy function : IOP to Request
 */
static void ddc_tsipCopyIop2Req(
                            PSP_tsipIoRequest *pIoReq,
                            const DDC_tsipIop *pIop
                            )
{
    if( (NULL != pIoReq) && (NULL != pIop) )
    {
        pIoReq->numSframe  = pIop->numSframe;
        pIoReq->xferActual = pIop->xferActual;
        pIoReq->appPtr     = pIop->appPtr;
        pIoReq->status     = pIop->status;
    }
}

/*!
*! Revision History
*! ================
*! 27-Jun-2007    JP : FLUSH functionality implementation
*! 12-Jun-2007    JP : Removed Pre-silicon related info and code segments
*! 20-Apr-2007    JP : Applied changes after pre-silicon board testing
*! 15-Mar-2007    JP : Added Modify Bitmap, Set active bitmap IOCTLs
*!                     refined other IOCTLS.
*!
*! 13-Dec-2006    JP : Changed Log_printf to TSIP_Debug
*!                     Changed local variable names start
*!                     with 'g' to non-g; More comments.
*! 06-Dec-2006    JP : Added Log_printf() traces
*! 23-Nov-2006    JP : Added IOCTLs definitions.
*! 21-Nov-2006    JP : Combined psp write/read fns into psp submit
*! 16-Nov-2006    JP : Added tx/rx superframe ISR definition
*! 15-Nov-2006    JP : Added getIntAndEvtNum function def.
*! 14-Nov-2006    JP : Added PSP write/read/abort & staic submitIop functions;
*!                     Defined ISR handlers for frame/sframe/error
*! 13-Nov-2006    JP : Modified braces to come in next line for
*!                     if,else,for,switch loops.
*! 07-Nov-2006    JP : Added PSP functions definition skeleton.
*! 31-Oct-2006    JP : Initial Creation (0.1)
*! */

