/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/
/** \mainpage    PSP TSIP Driver on BIOS
 *
 *  \section    Introduction
 *
 *  \subsection    xxx    Purpose and Scope
 *  The purpose of this document is to explain the TSIP driver on the PSP
 *  Framework architecture written in DSP/BIOS for this platform.
 *
 *  \subsection    aaa    PSP Architecture
 *  A PSP framework is a software entity providing platform support
 *  for devices that are based on Texas Instruments SoC's. The specific
 *  (Hardware and software) implementation of the PSP framework (the PAL
 *  implementation and the group of device drivers) is referred to as
 *  PSP System. The PSP framework defines a software environment for device
 *  drivers to plug-in by defining standard usage interfaces and
 *  abstractions (OS, CPU/Peripherals). For every platform, the PSP
 *  implements platform specific functionality such as OS adaptation, CPU and
 *  peripherals specific mechanisms, and generic system services such as
 *  debug control,traces, persistent storage, etc.
 *
 *  \subsection    bbb    TSIP Controller
 *  The TSIP is a telecom serial interface peripheral with timeslot data
 *  management and an integrated DMA capability. The primary purpose of this
 *  peripheral is to provide glueless interface to common telecom serial data
 *  streams and efficient internal routing of the data to designated memories
 *  in a multi-CPU device.<br><br>
 *  The TSIP in this SoC is a multi-link serial interface consisting of a
 *  maximum of two transmit data signals (or links), two receive data signals,
 *  two frame sync input signals and two serial clock inputs. The module offers
 *  supports for a maximum of 256 timeslots for transmit and 256 timeslots
 *  for receive.<br><br>
 *  The standard serial data rate for each TSIP transmit and receive data
 *  signal is 8.192 Mbps. The standard frame sync is a one or more bit wide
 *  pulse that occurs once every 125 s or a minimum of one serial clock
 *  period every 1024 serial clocks. At the standard rate and default
 *  configuration there are 2-transmit and 2-receive links that are active.
 *  Each serial interface link supports up to 128 8-bit timeslots. This
 *  corresponds to an H-MVIP or H.110 serial data rate interface. The serial
 *  interface clock frequency may be either 16.384 MHz (default) or 8.192 MHz.
 *  Typical timeslot occupation is 96 timeslots (DS2) for each serial
 *  interface link.Directly tied to the SIU of the TSIP is a TDMU that
 *  selectively packs and unpacks timeslot data for receive and transmit
 *  based on a channel timeslot definition. There are 1-transmit and 1-receive
 *  TDMU channels in this SoC implementation. Each channel is capable of
 *  selecting and transferring all of the possible 256 timeslots in the
 *  respective direction.<br><br>
 *    The TSIP also includes a DMATCU. DMATCU has one to one correspondence with
 *  TDMU channels and shares the transmit and receive channels of TDMU. There
 *  are 1-transmit and 1-receive DMATCU channels in this SoC.
 *
 *  \subsection    ccc    DSP/BIOS System
 *  DSP/BIOS is a real time kernel provided by TI to use with its
 *  DSP processors.
 *
 *
 *  \subsection    References
 *    -# TSIP functional specification, <b>sprueu8_TSIP.pdf</b>
 *    -# Design Document for TSIP driver,
 *                         <b>BIOS_TSIP_Driver_Arch.doc</b>
 *    -# PSP Framework Overview, <b>PSPFramework_Overview.pdf</b>
 *    -# DSP/BIOS driver development <b>SPRU616.pdf</b>
 *
 */

/** \file    ddc_tsip.h
 *
 *  \brief    Header file for TSIP Device Driver Core(DDC)
 *
 *
 *  This file contains definitions for a TSIP DDC. This file defines port,
 *  I/O and etc. This file depends on some definitions from PSP Header files.
 *
 *  (C)    Copyright 2006,    Texas Instruments, Inc
 *
 *  \author    Platform Support Group
 *
 *  \version    0.1
 */

#ifndef _DDC_TSIP_H_
#define _DDC_TSIP_H_
/* Includes */
#include <std.h>

#include <ti/sdo/pspdrivers/drivers/tsip/psp_tsip.h>
#include <ti/sdo/pspdrivers/drivers/tsip/src/psp_tsipApi.h>

#include <ti/sdo/pspdrivers/pal_os/pal_osList.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osSem.h>

#include <ti/sdo/pspdrivers/soc/c6452/dsp/tistdtypes.h>
#include <ti/sdo/pspdrivers/soc/c6452/dsp/cslr_tsip.h>

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * \brief    TSIP DDC channel status
 *
 * This enumeration represents whether the TSIP DDC channel is free
 * or allotted.
 */
typedef enum DDC_TsipChanStatus_t
{
    DDC_TSIP_CHAN_FREE,
    DDC_TSIP_CHAN_ALLOTTED
} DDC_TsipChanStatus;

/**
 * \brief    TSIP DDC channel interrupt type
 *
 * This enumeration contains TSIP channel interrupt sources
 */
typedef enum DDC_TsipChanIntType_t
{
    DDC_TSIP_CHAN_FRAME_INT,
    DDC_TSIP_CHAN_SFRAME_INT,
    DDC_TSIP_CHAN_ERROR_INT
} DDC_TsipChanIntType;

/**
 * \brief    TSIP DDC IOP strcture
 *
 * The IO structure is used to hold the IOP request details
 * per request for the channel.
 *
 */
typedef struct DDC_tsipIop_t
{
    PAL_OsListNodeHeader    iopLink;
    /**< Link to the next & previous in the list                          */
    PSP_tsipIoParams       ioParams[PSP_TSIP_MAX_IOP_SFRAME/2];
    /**< Storage of application passed IO params                          */
    Uint16                 numSframe;
    /**< Number of superframes in the IOP request                         */
    Uint16                 xferActual;
    /**< Actual superframes transferred. Filled by the driver             */
    Ptr                    appPtr;
    /**< Pointer to store application upper layer IOP handle              */
    PSP_Result             status;
    /**< Status of this request. Filled by driver                         */

} DDC_tsipIop;


/**
 * \brief    TSIP I/O(Channel) Object
 *
 *  This object has informations about the channel parameters for each
 *  channels of TSIP.
 */
typedef struct DDC_tsipIoObj_t
{
    PSP_tsipErrStats             errStats;
    /**< Error statistics counters                         */
    
    DDC_tsipIop                  iop[PSP_TSIP_MAX_IOP_REQ];
    /**< IOP array of maximum supported requests           */

    volatile DDC_TsipChanStatus  status;
    /**< Channel status whether free or allocated          */


    PSP_TsipMode                 mode;
    /**< Channel mode TX/RX                                */

    PSP_tsipAppCallback          drvCb;
    /**< Driver callback function for this channel         */

    Ptr                          cbArg;
    /**< Driver callback argument                          */

    PSP_tsipErrCallback          errCb;
    /**< Error callback function for this channel          */

    Ptr                          eCbArg;
    /**< Error callback argument                           */

    PSP_tsipFrmCallback          frmCb;
    /**< Frame interrupt callback function pointer         */

    Ptr                          fCbArg;
    /**< Frame interrupt callback argument                 */

    PSP_tsipFrmCallback          sfrmCb;
    /**< Super frame interrupt callback function pointer.  */

    Ptr                          sCbArg;
    /**< Super Frame interrupt callback argument           */

    struct DDC_tsipDevObj_t      *devHandle;
    /**< Pointer to Device Obj                             */

    PAL_OsListNodeHeader         queueFreeList;
    /**< List containing the Free IO Packets               */

    PAL_OsListNodeHeader         queueReqList;
    /**< List containing the IOP pending requests, received
         from the application                              */

    DDC_tsipIop                  *currIop;
    /**< Current IOP packet being serviced                 */

    PAL_OsSemHandle              flushSem;
    /**< Semaphore used during TX channel Flush operation  */

    volatile Int32               iopSubmitCnt;
    /**< Number of submitted pending IOPs                  */

    volatile Bool                isStopExtCmd;
    /**< Flag to tell whether Stop Channel EXT ioctl cmd
         is invoked from the application                   */

    volatile Bool                isChEnabled;
    /**< Flag to tell whether channel is enabled/stopped   */

    volatile Bool                isFlushCmd;
    /**< Flag to tell whether Flush operation is invoked
         on TX channel                                     */
    
    Uint8                        aCID;
    /**< Channel identifier (CID) for "A" bitmap/context.
        least significant bit(lsb) should be zero          */

    Uint8                        bCID;
    /**< Channel identifier (CID) for "B" bitmap/context.
        least significant bit(lsb) should be one           */

    Int32                        semTimeOut;
    /**< Pending on semaphore timeout for flush operation  */

    volatile CSL_TsipErrRegs    *errRegs;
    /**< TDMU channel error log registers overlay          */

    volatile CSL_TsipChenRegs   *chenRegs;
    /**< TDMU channel  registers overlay                   */

    volatile CSL_TsipDchRegs    *dchRegs;
    /**< DMATCU channel  registers overlay                 */

    volatile CSL_TsipBmRegs     *bmRegs;
    /**< TDMU channel  bitmaps overlay                     */

    volatile CSL_TsipCbRegs     *cbRegs;
    /**< TDMU channel  buffers overlay                     */

} DDC_tsipIoObj, *DDC_tsipIoHandle;

/**
 * \brief    Default IOP values
 */
#define TSIP_IOP_DEFAULT                                 \
        {                                                \
        /* iopLink      */     {NULL,NULL},              \
        /* ioParams[]   */     {TSIP_IOP_PARAMS,         \
                                TSIP_IOP_PARAMS,         \
                                TSIP_IOP_PARAMS,         \
                                TSIP_IOP_PARAMS,         \
                                TSIP_IOP_PARAMS          \
                                },                       \
        /* numSframe    */     (2u),                     \
        /* xferActual   */     0,                        \
        /* appPtr       */     NULL,                     \
        /* status       */     0                         \
        }


/**
 * \brief    Default IOP params
 */
#define TSIP_IOP_PARAMS                                  \
        {                                                \
        /* DMA-A         */ {0,0,0,0},                   \
        /* DMA-B         */ {0,0,0,0}                    \
        }

/**
 * \brief    Default Error stats values
 */
#define TSIP_CLR_ERR_STATS                               \
        {                                                \
        /* un run cnt   */     0,                        \
        /* inv frame cnt*/     0,                        \
        /* cba read cnt */     0,                        \
        /* cba writecnt */     0,                        \
        /* inv conf cnt */     0,                        \
        /* un known cnt */     0                         \
        }

/**
 * \brief    Default TSIP DDC channel object values
 */
#define TSIP_DDC_IO_DEFAULT                              \
        {                                                \
        /* errstats     */     TSIP_CLR_ERR_STATS,       \
                              {                          \
        /* iop-0        */     TSIP_IOP_DEFAULT,         \
        /* iop-1        */     TSIP_IOP_DEFAULT,         \
        /* iop-2        */     TSIP_IOP_DEFAULT,         \
        /* iop-3        */     TSIP_IOP_DEFAULT,         \
        /* iop-4        */     TSIP_IOP_DEFAULT,         \
        /* iop-5        */     TSIP_IOP_DEFAULT,         \
        /* iop-6        */     TSIP_IOP_DEFAULT,         \
        /* iop-7        */     TSIP_IOP_DEFAULT,         \
        /* iop-8        */     TSIP_IOP_DEFAULT,         \
        /* iop-9        */     TSIP_IOP_DEFAULT          \
                              },                         \
        /* status       */     DDC_TSIP_CHAN_FREE,       \
        /* mode         */     PSP_TSIP_MODE_NONE,       \
        /* drvCb        */     NULL,                     \
        /* cbArg        */     NULL,                     \
        /* errCb        */     NULL,                     \
        /* eCbArg       */     NULL,                     \
        /* frmCb        */     NULL,                     \
        /* fCbArg       */     NULL,                     \
        /* sfrmCb       */     NULL,                     \
        /* sCbArg       */     NULL,                     \
        /* port         */     NULL,                     \
        /* queueFreelist*/     {NULL,NULL},              \
        /* queueReqlist */     {NULL,NULL},              \
        /* current IOP  */     NULL,                     \
        /* flush Sem    */     NULL,                     \
        /* IOP sub cnt  */     0,                        \
        /* is DisExIoctl*/     FALSE,                    \
        /* is chEnabled */     FALSE,                    \
        /* is flushCmd  */     FALSE,                    \
        /* aCID         */     0,                        \
        /* bCID         */     1u,                       \
        /* semTimeout   */     PAL_OSSEM_NO_TIMEOUT,     \
        /* ch-reg ptr   */     NULL,                     \
        /* ch-reg ptr   */     NULL,                     \
        /* ch-reg ptr   */     NULL,                     \
        /* ch-reg ptr   */     NULL,                     \
        /* ch-reg ptr   */     NULL                      \
        }



/**
 * \brief    TSIP Device Object
 *
 *  This object has informations about the hardware setup for each
 *  device instance(or Port) of TSIP.
 */
typedef struct DDC_tsipDevObj_t
{

    Int32              devId;
    /**< Device or instance ID of TSIP                             */
    
    Bool               inUse;
    /**< Resource is in use or free                                */

    DDC_tsipIoObj      txChan;
    /**< Transmit channel object                                   */

    DDC_tsipIoObj      rxChan;
    /**< Receive channel object                                    */

    Bool               isErrRegd;
    /**< Flag to say whether tx-rx errInt is registered or not     */
    
    
    volatile Bool       isTxSiuON;
    /**< Flag to say whether Tx SIU TSIP engine is running(enabled)
         or not. This should not be disabled if any one of TX DMA 
         channels is enabled.                                      */
    volatile Bool       isRxSiuON;
    /**< Flag to say whether Rx SIU TSIP engine is running(enabled)
         or not. This should not be enabled if any one of RX DMA
         channels is enabled.                                      */

    volatile Bool       isTdmuON;
    /**< Flag to say whether DMATCU TSIP engine is running(enabled)
         or not. This should not be enabled if any one of TX/RX DMA
         channels is enabled.                                      */

    CSL_TsipRegsOvly    pReg;
    /**< Register overlay pointer for this instance                */

    Ptr                 appHandle;
    /**< Application Handle                                        */

    PSP_tsipHwSetup     hwSetup;
    /**< Holds the TSIP hardware setup parameters passed by the 
         application.                                              */

} DDC_tsipDevObj, *DDC_tsipDevHandle;

/**
 * \brief    Default TSIP HW setup data structure values(all zeros)
 */
#define TSIP_DDC_HW_SETUP_DEFAULT                              \
        {                                                      \
        /* SIU           */ {                                  \
               /* clock   */  PSP_TSIP_CLK_REDUNDANT,          \
               /* Tx.SIU  */  {                                \
                               PSP_TSIP_CLK_FS_SRC_A,          \
                               0,                              \
                               PSP_TSIP_TXDIS_STATE_HIGHZ,     \
                               PSP_TSIP_TXENA_DLY_ZERO,        \
                               PSP_TSIP_POLARITY_ACTIVE_LOW,   \
                               PSP_TSIP_POLARITY_RISING_EDGE,  \
                               PSP_TSIP_POLARITY_RISING_EDGE,  \
                               PSP_TSIP_DATA_RATE_8_192,       \
                               PSP_TSIP_CLK_MODE_2X,           \
                               0,                              \
                               0                               \
                              },                               \
               /* Rx.SIU  */  {                                \
                               PSP_TSIP_CLK_FS_SRC_A,          \
                               0,                              \
                               PSP_TSIP_POLARITY_ACTIVE_LOW,   \
                               PSP_TSIP_POLARITY_RISING_EDGE,  \
                               PSP_TSIP_POLARITY_RISING_EDGE,  \
                               PSP_TSIP_DATA_RATE_8_192,       \
                               PSP_TSIP_CLK_MODE_2X,           \
                               0,                              \
                               0                               \
                              },                               \
               /* LoopBack*/  PSP_TSIP_LOOP_BACK_NONE,         \
                              PSP_TSIP_TEST_CLK_EXTERNAL       \
                            },                                 \
        /* TDMU          */ {                                  \
                              PSP_TSIP_LITTLE_ENDIAN,          \
                              0,                               \
                              0,                               \
                              PSP_TSIP_INT_XFER_ACK_ONLY,      \
                              PSP_TSIP_INT_XFER_ACK_ONLY,      \
                              PSP_TSIP_INT_XFER_ACK_ONLY,      \
                              PSP_TSIP_INT_XFER_ACK_ONLY,      \
                              0,                               \
                              0                                \
                            }                                  \
        }


/**
 * \brief    Default TSIP device object values
 */
#define TSIP_DDC_PORT_DEFAULT                                  \
        {                                                      \
        /* dev id           */     (0xDEADL),                  \
        /* in Use           */     FALSE,                      \
        /* txchan           */     TSIP_DDC_IO_DEFAULT,        \
        /* rxchan           */     TSIP_DDC_IO_DEFAULT,        \
        /* is errReg        */     FALSE,                      \
        /* is Tx SIU ON     */     FALSE,                      \
        /* is Rx SIU ON     */     FALSE,                      \
        /* is DMATCU ON     */     FALSE,                      \
        /* regovly          */     NULL,                       \
        /* app ptr          */     NULL,                       \
        /* app ptr          */     TSIP_DDC_HW_SETUP_DEFAULT   \
        }

        
#ifdef __cplusplus
}
#endif

#endif /* _DDC_TSIP_H_ */
/*!
*! Revision History
*! ================
*! 27-Jun-2007    JP : flushSem and isFlushCmd fields are added into IO object
*!                     to implement FLUSH functionality in DDC.
*! 12-Jun-2007    JP : Removed Pre-silicon related info and code segments
*!
*! 17-Jan-2007    JP : Added default init value macro for port and channel obj.
*!
*! 13-Nov-2006    JP : Modified struct/enum declaration standard ( braces at
*!                     the next line, name for all enum/struct with _t suffix)
*! 31-Oct-2006    JP : Initial Creation (0.1)
*! */
