/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/** \file    ddc_tsipHal.c
 *
 *  \brief   TSIP DDC Hardware Abstraction Layer (HAL)
 *
 *  This file implements the low level hardware access APIs
 *
 *  (C)    Copyright 2006,    Texas Instruments, Inc
 *
 *  \author    Platform Support Group
 *
 *  \version    0.1
 *
 */

/*
 * Header includes
 */

#include <ti/sdo/pspdrivers/drivers/tsip/src/ddc_tsipHal.h>
#include <ti/sdo/pspdrivers/soc/c6452/dsp/soc.h>

/***********************************************************
 * Public Functions Definitions
 ***********************************************************/

/**
 * \brief    Get the memory map(register) base Address for TSIP
 *
 * \param    devHandle   [IN]    DDC port handle
 *
 * \note    On success, this function will update pRegs pointer
 *  of port object with valid register base address.
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipGetBaseAddr(DDC_tsipDevHandle devHandle)
{
    PSP_Result    result = PSP_SOK;

    if( NULL != devHandle)
    {
        switch(devHandle->devId)
        {
         case CSL_TSIP_0:
             /* Assign TSIP instance 0 Base Address */
             devHandle->pReg = (CSL_TsipRegsOvly)CSL_TSIP_0_REGS;
             break;
         case CSL_TSIP_1:
             /* Assign TSIP instance 1 Base Address */
             devHandle->pReg = (CSL_TsipRegsOvly)CSL_TSIP_1_REGS;
             break;

         default:
             result = PSP_E_INVAL_PARAM;
             break;
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
        TSIP_DEBUG("HAL_tsipGetRegBaseAddr <Invalid Params>");
#endif

    }
    return result;
}

/**
 * \brief    Get the DMA channel memory map base address for the tx/rx channel
 *
 * \param    ioHandle     [IN]    DDC channel handle
 *
 * \note    On success, this function will update chan regs pointers
 *  of the channel object with valid register base addresses
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipGetChanBaseAddr(DDC_tsipIoHandle ioHandle)
{
    PSP_Result          result = PSP_SOK;
    CSL_TsipRegsOvly    reg    = NULL;

    if( NULL != ioHandle)
    {
        /* Get register overlay address */
        reg = ioHandle->devHandle->pReg;

        if( NULL != reg )
        {
            if( PSP_TSIP_MODE_TX == ioHandle->mode ) /* TX chan*/
            {
                 /* TDMU Channel Error Log register base */
                 ioHandle->errRegs   = &reg->ERR0;
                   
                 /* TDMU Channel Register base */ 
                 ioHandle->chenRegs  = &reg->XCHEN0;

                 /* DMATCU Channel Register base */
                 ioHandle->dchRegs   = &reg->DXCH0;
                   
                 /* TDMU Channel Bitmap */
                 ioHandle->bmRegs    = &reg->XBM0;
                   
                 /* TDMU Channel Buffer */  
                 ioHandle->cbRegs    = &reg->XCB0;
            }
            else /* RX channel */
            {
                /* Assigning RX Channel Specific Register Overlays address
                using the corresponding channel memory map for each channel */

                 /* TDMU Channel Error Log register base */
                ioHandle->errRegs    = &reg->ERR0;
                   
                /* TDMU Channel Register base */ 
                ioHandle->chenRegs   = &reg->RCHEN0;
                   
                /* DMATCU Channel Register base */
                ioHandle->dchRegs    = &reg->DRCH0;
                   
                /* TDMU Channel Bitmap */
                ioHandle->bmRegs     = &reg->RBM0;
                   
                /* TDMU Channel Buffer */  
                ioHandle->cbRegs     = &reg->RCB0;  
            }
        }
        else
        {
            result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG            
            TSIP_DEBUG("HAL_tsipGetChanRegBaseAddr<Invalid reg Param>");
#endif            
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
        TSIP_DEBUG("HAL_tsipGetChanRegBaseAddr<Invalid Params>");
#endif

    }

    return result;
}

/**
 * \brief    Setup SIU HW
 *
 * \param    devHandle     [IN]    DDC port handle
 * \param    siu           [IN]    SIU config params
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipSiuSetup(
                        DDC_tsipDevHandle       devHandle,
                        PSP_tsipSiuConf const  *siu
                        )
{
    PSP_Result   result = PSP_SOK;

    if( (NULL != siu) && (NULL != devHandle) )
    {
        /* 1.Clock selection */

        switch(siu->clkSel)
        {
         case PSP_TSIP_CLK_REDUNDANT:

             CSL_FINST(devHandle->pReg->SIUCTL, TSIP_SIUCTL_CLKD, REDUN);
             break;

         case PSP_TSIP_CLK_DUAL:

             CSL_FINST(devHandle->pReg->SIUCTL, TSIP_SIUCTL_CLKD, DUAL);
             break;

         default:
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipSIUsetup <Invalid ClkSel>");
#endif

             break;
        }


        /* 2. TX SIU configurations */

        /* a1. TX Clk/FS source selection  */
        switch(siu->tx.clkSrc)
        {
         case PSP_TSIP_CLK_FS_SRC_A:

             CSL_FINST(devHandle->pReg->XCLK, TSIP_XCLK_SRC, ACLK);
             break;

         case PSP_TSIP_CLK_FS_SRC_B:

             CSL_FINST(devHandle->pReg->XCLK, TSIP_XCLK_SRC, BCLK);
             break;

         default:
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipSIUsetup <Invalid TX clkSrc>");
#endif

             break;
        }

        /* a2. Data Delay  TX */
        if( siu->tx.dDly <= (0x3FFFu) ) /* Max Value: 16383 */
        {
            CSL_FINS(
                 devHandle->pReg->XCTL,
                 TSIP_XCTL_DATD,
                 (Uint32)siu->tx.dDly
                 );
        }
        else
        {
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipSIUsetup <Invalid RX dDly>");
#endif

        }

        /* a3. Transmit Output disable state */
        switch(siu->tx.disState)
        {
         case PSP_TSIP_TXDIS_STATE_HIGHZ:

             CSL_FINST(devHandle->pReg->XCTL, TSIP_XCTL_DIS, HIMPED);
             break;

         case PSP_TSIP_TXDIS_STATE_LOW:

             CSL_FINST(devHandle->pReg->XCTL, TSIP_XCTL_DIS, LOW);
             break;

         case PSP_TSIP_TXDIS_STATE_HIGH:

             CSL_FINST(devHandle->pReg->XCTL, TSIP_XCTL_DIS, HIGH);
             break;

         default:
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipSIUsetup <Invalid TX disState>");
#endif
             break;
        }

        /* a4. Transmit Output Enable delay */
        switch(siu->tx.enaDly)
        {
         case PSP_TSIP_TXENA_DLY_ZERO:

             CSL_FINS(devHandle->pReg->XCTL, TSIP_XCTL_DLY, 0);
             break;

         case PSP_TSIP_TXENA_DLY_ONEHALF:

             CSL_FINS(devHandle->pReg->XCTL, TSIP_XCTL_DLY, 1);
             break;

         default:
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipSIUsetup <Invalid TX enaDly>");
#endif
             break;
        }

        /* a5. TX Frame Sync polarity  */
        switch(siu->tx.fSyncP)
        {
         case PSP_TSIP_POLARITY_ACTIVE_LOW:

             CSL_FINST(devHandle->pReg->XCTL, TSIP_XCTL_FSYNCP, ALOW);
             break;

         case PSP_TSIP_POLARTIY_ACTIVE_HIGH:

             CSL_FINST(devHandle->pReg->XCTL, TSIP_XCTL_FSYNCP, AHIGH);
             break;

         default:
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipSIUsetup <Invalid TX fSynP>");
#endif

             break;
        }

        /* a6. TX frame CLK polarity  */
        switch(siu->tx.fClkP)
        {
         case PSP_TSIP_POLARITY_RISING_EDGE:

             CSL_FINST(devHandle->pReg->XCTL, TSIP_XCTL_FCLKP, RISING);
             break;

         case PSP_TSIP_POLARTIY_FALLING_EDGE:

             CSL_FINST(devHandle->pReg->XCTL, TSIP_XCTL_FCLKP, FALLING);
             break;

         default:
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipSIUsetup <Invalid TX fClkP>");
#endif
             break;
        }

        /* a7. TX Data CLK polarity  */
        switch(siu->tx.dClkP)
        {
         case PSP_TSIP_POLARITY_RISING_EDGE:

             CSL_FINST(devHandle->pReg->XCTL, TSIP_XCTL_DCLKP, RISING);
             break;

         case PSP_TSIP_POLARTIY_FALLING_EDGE:

             CSL_FINST(devHandle->pReg->XCTL, TSIP_XCTL_DCLKP, FALLING);
             break;

         default:
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipSIUsetup <Invalid TX dClkP>");
#endif
             break;
        }

        /* a8. TX Data Rate  */
        switch(siu->tx.dRate)
        {
         case PSP_TSIP_DATA_RATE_8_192:
             CSL_FINST(devHandle->pReg->XCTL, TSIP_XCTL_DRATE, 8M);
             break;

         case PSP_TSIP_DATA_RATE_16_384:

             CSL_FINST(devHandle->pReg->XCTL, TSIP_XCTL_DRATE, 16M);
             break;

         default:
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipSIUsetup <Invalid TX dRate>");
#endif
             break;
        }

        /* a9. TX Clock mode  */
        switch(siu->tx.clkMode)
        {
         case PSP_TSIP_CLK_MODE_2X:

             CSL_FINST(devHandle->pReg->XCTL, TSIP_XCTL_CLKM, DBL);
             break;

         case PSP_TSIP_CLK_MODE_1X:

             CSL_FINST(devHandle->pReg->XCTL, TSIP_XCTL_CLKM, SGL);
             break;

         default:
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipSIUsetup <Invalid TX clkMode>");
#endif
             break;
        }

        /* a10. TX Frame Count  */
        if( siu->tx.fCnt >= (0x3u) )
        {
            CSL_FINS(devHandle->pReg->XSIZE,TSIP_XSIZE_FCNT,(Uint32)siu->tx.fCnt);
        }
        else
        {
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipSIUsetup <Invalid TX fCnt>");
#endif

        }


        /* a11. TX Frame size   */
        if( siu->tx.fSize <= (0x7Fu) )
        {
            CSL_FINS(devHandle->pReg->XSIZE,TSIP_XSIZE_FSIZ,(Uint32)siu->tx.fSize);
        }
        else
        {
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipSIUsetup <Invalid TX fSize>");
#endif

        }


        /* 2. RX SIU configurations */

        /* b1. RX Clk/FS source selection  */
        switch(siu->rx.clkSrc)
        {
         case PSP_TSIP_CLK_FS_SRC_A:

             CSL_FINST(devHandle->pReg->RCLK, TSIP_RCLK_SRC, ACLK);
             break;

         case PSP_TSIP_CLK_FS_SRC_B:

             CSL_FINST(devHandle->pReg->RCLK, TSIP_RCLK_SRC, BCLK);
             break;

         default:
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipSIUsetup <Invalid RX clkSrc>");
#endif

             break;
        }

        /* b2. RX Data Delay */
        if( siu->rx.dDly <= (0x3FFFu) ) /* Max Value: 16383 */
        {
            CSL_FINS(
                 devHandle->pReg->RCTL,
                 TSIP_RCTL_DATD,
                 (Uint32)siu->rx.dDly
                 );
        }
        else
        {
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipSIUsetup <Invalid RX dDly>");
#endif
        }

        /* b3. RX Frame Sync polarity  */
        switch(siu->rx.fSyncP)
        {
         case PSP_TSIP_POLARITY_ACTIVE_LOW:

             CSL_FINST(devHandle->pReg->RCTL, TSIP_RCTL_FSYNCP, ALOW);
             break;

         case PSP_TSIP_POLARTIY_ACTIVE_HIGH:

             CSL_FINST(devHandle->pReg->RCTL, TSIP_RCTL_FSYNCP, AHIGH);
             break;

         default:
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipSIUsetup <Invalid RX fSynP>");
#endif

             break;
        }

        /* b4. RX frame CLK polarity  */
        switch(siu->rx.fClkP)
        {
         case PSP_TSIP_POLARITY_RISING_EDGE:

             CSL_FINST(devHandle->pReg->RCTL, TSIP_RCTL_FCLKP, RISING);
             break;

         case PSP_TSIP_POLARTIY_FALLING_EDGE:

             CSL_FINST(devHandle->pReg->RCTL, TSIP_RCTL_FCLKP, FALLING);
             break;

         default:
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipSIUsetup <Invalid RX fClkP>");
#endif
             break;
        }

        /* b5. RX Data CLK polarity  */
        switch(siu->rx.dClkP)
        {
         case PSP_TSIP_POLARITY_RISING_EDGE:

             CSL_FINST(devHandle->pReg->RCTL, TSIP_RCTL_DCLKP, RISING);
             break;

         case PSP_TSIP_POLARTIY_FALLING_EDGE:

             CSL_FINST(devHandle->pReg->RCTL, TSIP_RCTL_DCLKP, FALLING);
             break;

         default:
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipSIUsetup <Invalid RX dClkP>");
#endif
             break;
        }

        /* b6. RX Data Rate  */
        switch(siu->rx.dRate)
        {
         case PSP_TSIP_DATA_RATE_8_192:
             CSL_FINST(devHandle->pReg->RCTL, TSIP_RCTL_DRATE, 8M);
             break;

         case PSP_TSIP_DATA_RATE_16_384:

             CSL_FINST(devHandle->pReg->RCTL, TSIP_RCTL_DRATE, 16M);
             break;

         default:
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipSIUsetup <Invalid RX dRate>");
#endif
             break;
        }

        /* b7. RX Clock mode  */
        switch(siu->rx.clkMode)
        {
         case PSP_TSIP_CLK_MODE_2X:

             CSL_FINST(devHandle->pReg->RCTL, TSIP_RCTL_CLKM, DBL);
             break;

         case PSP_TSIP_CLK_MODE_1X:

             CSL_FINST(devHandle->pReg->RCTL, TSIP_RCTL_CLKM, SGL);
             break;

         default:
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipSIUsetup <Invalid RX clkMode>");
#endif
             break;
        }

        /* b8. RX Frame Count  */
        if( siu->rx.fCnt >= (0x3u) )
        {
            CSL_FINS(devHandle->pReg->RSIZE,TSIP_RSIZE_FCNT,(Uint32)siu->rx.fCnt);
        }
        else
        {
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipSIUsetup <Invalid RX fCnt>");
#endif

        }


        /* b9. RX Frame size   */
        if( siu->rx.fSize <= (0x7Fu) )
        {
            CSL_FINS(devHandle->pReg->RSIZE,TSIP_RSIZE_FSIZ,(Uint32)siu->rx.fSize);
        }
        else
        {
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipSIUsetup <Invalid RX fSize>");
#endif

        }

       /* 4. SIU test mode(Loop back mode) setting */
        switch(siu->loopBack)
        {

         Uint32  lbTemp;

         case PSP_TSIP_LOOP_BACK_NONE: /* NO Loop Back */

             /* Clear SIUTST to ignore loop back mode */
             CSL_FINS(devHandle->pReg->EMUTST, TSIP_EMUTST_SIUTST, 0);
             break;

         case PSP_TSIP_LOOP_BACK_DLB: /* Data Loop Back */

             lbTemp = devHandle->pReg->EMUTST;
             /* Enable testmode: loopback */
             CSL_FINS(lbTemp, TSIP_EMUTST_SIUTST, 1);
             CSL_FINS(lbTemp,TSIP_EMUTST_LBS,  0); /* DLB   */

             /* Write to EMUTST register */
             devHandle->pReg->EMUTST = lbTemp;
             break;

         case PSP_TSIP_LOOP_BACK_LLB: /* Link Loop Back */

             lbTemp = devHandle->pReg->EMUTST;
             /* Enable testmode: loopback */
             CSL_FINS(lbTemp,TSIP_EMUTST_SIUTST,1);
             CSL_FINS(lbTemp,TSIP_EMUTST_LBS,1); /* LLB   */

             /* Write to EMUTST register */
             devHandle->pReg->EMUTST = lbTemp;
             break;

         default:
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipSIUsetup <Invalid loopBack >");
#endif
             break;
        }

        /* 5. LoopBack mode Test clock selection */
        switch(siu->testClk)
        {
         Uint32  temp;

         case PSP_TSIP_TEST_CLK_EXTERNAL:

             temp = devHandle->pReg->EMUTST;

             /* Set TSTCLK = 0 */
             CSL_FINS(temp,TSIP_EMUTST_TSTCLK,0);
             /* Write TSTKEY(0xD1) */
             CSL_FINS(temp,TSIP_EMUTST_TSTKEY,0xD1);

             /* Write to EMUTST register */
             devHandle->pReg->EMUTST = temp;

             break;

         default:
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipSIUsetup <Invalid testClk >");
#endif
             break;

        } 
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }
    return result;
}

/**
 * \brief    Get SIU HW setup
 *
 * \param    devHandle     [IN]    DDC port handle
 * \param    siu          [OUT]   Place holder for SIC config
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipGetSiuSetup(
                           DDC_tsipDevHandle  devHandle,
                           PSP_tsipSiuConf   *siu
                           )
{
    PSP_Result   result = PSP_SOK;

    if( (NULL != siu) && (NULL != devHandle) )
    {
        /* 1. Get Clock selection */

        siu->clkSel = (PSP_TsipClock)CSL_FEXT(
                                             devHandle->pReg->SIUCTL,
                                             TSIP_SIUCTL_CLKD
                                             );

        /* 2. Get TX SIU configurations */

        /* a1. Get TX Clk/FS source selection */
        siu->tx.clkSrc = (PSP_TsipClkFsSrc)CSL_FEXT(
                                             devHandle->pReg->XCLK,
                                             TSIP_XCLK_SRC
                                             );

        /* a2. Get TX data delay */
        siu->tx.dDly = (PSP_TsipDataRate)CSL_FEXT(
                                             devHandle->pReg->XCTL,
                                             TSIP_XCTL_DATD
                                             );

        /* a3. Get TX disable state */
        siu->tx.disState  = (PSP_TsipTxDisState)CSL_FEXT(
                                             devHandle->pReg->XCTL,
                                             TSIP_XCTL_DIS
                                             );

        /* a4. Get TX enable delay */
        siu->tx.enaDly   = (PSP_TsipTxEnaDly)CSL_FEXT(
                                             devHandle->pReg->XCTL,
                                             TSIP_XCTL_DLY
                                             );

        /* a5. Get TX Frame Sync polarity  */
        siu->tx.fSyncP = (PSP_TsipPolarity)CSL_FEXT(
                                             devHandle->pReg->XCTL,
                                             TSIP_XCTL_FSYNCP
                                             );
        /* a6. Get TX frame CLK polarity */
        siu->tx.fClkP = (PSP_TsipPolarity)CSL_FEXT(
                                             devHandle->pReg->XCTL,
                                             TSIP_XCTL_FCLKP
                                             );
        /* a7. Get TX Data CLK polarity */
        siu->tx.dClkP = (PSP_TsipPolarity)CSL_FEXT(
                                             devHandle->pReg->XCTL,
                                             TSIP_XCTL_DCLKP
                                             );

        /* a8. Get TX Data Rate */
        siu->tx.dRate = (PSP_TsipDataRate)CSL_FEXT(
                                             devHandle->pReg->XCTL,
                                             TSIP_XCTL_DRATE
                                             );

        /* a9. Get TX Clock mode  */
        siu->tx.clkMode = (PSP_TsipClkMode)CSL_FEXT(
                                              devHandle->pReg->XCTL,
                                              TSIP_XCTL_CLKM
                                              );
        /* a10. Get TX Frame Count */
        siu->tx.fCnt = (Uint8)CSL_FEXT(devHandle->pReg->XSIZE,TSIP_XSIZE_FCNT);


        /* a11. Get TX Frame size */
        siu->tx.fSize = (Uint8)CSL_FEXT(devHandle->pReg->XSIZE,TSIP_XSIZE_FSIZ);


        /* 3. Get RX SIU configurations */

        /* b1. Get RX Clk/FS source selection */
        siu->rx.clkSrc = (PSP_TsipClkFsSrc)CSL_FEXT(
                                             devHandle->pReg->RCLK,
                                             TSIP_RCLK_SRC
                                             );

        /* b2. Get RX data delay */
        siu->rx.dDly = (PSP_TsipDataRate)CSL_FEXT(
                                             devHandle->pReg->RCTL,
                                             TSIP_RCTL_DATD
                                             );

        /* b3. Get RX Frame Sync polarity  */
        siu->rx.fSyncP = (PSP_TsipPolarity)CSL_FEXT(
                                             devHandle->pReg->RCTL,
                                             TSIP_RCTL_FSYNCP
                                             );
        /* b4. Get RX frame CLK polarity */
        siu->rx.fClkP = (PSP_TsipPolarity)CSL_FEXT(
                                             devHandle->pReg->RCTL,
                                             TSIP_RCTL_FCLKP
                                             );
        /* b5. Get RX Data CLK polarity */
        siu->rx.dClkP = (PSP_TsipPolarity)CSL_FEXT(
                                             devHandle->pReg->RCTL,
                                             TSIP_RCTL_DCLKP
                                             );

        /* b6. Get RX Data Rate */
        siu->rx.dRate = (PSP_TsipDataRate)CSL_FEXT(
                                             devHandle->pReg->RCTL,
                                             TSIP_RCTL_DRATE
                                             );

        /* b7. Get RX Clock mode  */
        siu->rx.clkMode = (PSP_TsipClkMode)CSL_FEXT(
                                              devHandle->pReg->RCTL,
                                              TSIP_RCTL_CLKM
                                              );
        /* b8. Get RX Frame Count */
        siu->rx.fCnt = (Uint8)CSL_FEXT(devHandle->pReg->RSIZE,TSIP_RSIZE_FCNT);


        /* b9. Get RX Frame size */
        siu->rx.fSize = (Uint8)CSL_FEXT(devHandle->pReg->RSIZE,TSIP_RSIZE_FSIZ);

        /* 4. Get Loop Back mode */
        {
            Uint32 etrReg;
            etrReg = devHandle->pReg->EMUTST;
            etrReg &= (0x00070000u); /* Extract IOLB:LBS:SIUTST[18:17:16] bits*/
            etrReg >>= (16u);
            if(0 == (etrReg & 0x1u)) /* If SIUTST is 0 */
            {
                siu->loopBack = PSP_TSIP_LOOP_BACK_NONE;
            }
            else
            {
                siu->loopBack = (PSP_TsipLoopBack)etrReg;
            }
        }

        /* 5. Get Test clock source */
        siu->testClk = (PSP_TsipTestClock)CSL_FEXT(
                                               devHandle->pReg->EMUTST,
                                               TSIP_EMUTST_TSTCLK
                                               );
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
        TSIP_DEBUG("HAL_tsipGetSIUsetup <Invalid Params>");
#endif

    }
    return result;
}


/**
 * \brief    Set TDMU hardware setup
 *
 * \param    devHandle [IN]    DDC port handle
 * \param    tdmu      [IN]    TDMU config params
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipTdmuSetup(
                         DDC_tsipDevHandle        devHandle,
                         PSP_tsipTdmuConf const  *tdmu
                         )
{
    PSP_Result result = PSP_SOK;

    if( (NULL != devHandle) && (NULL != tdmu) )
    {

        /* 1. TDMU: Endian selection for CB */
        switch(tdmu->endian)
        {
         case PSP_TSIP_LITTLE_ENDIAN:

             CSL_FINST(devHandle->pReg->TDMUCFG,TSIP_TDMUCFG_BEND,LITTLE);
             break;

         case PSP_TSIP_BIG_ENDIAN:

             CSL_FINST(devHandle->pReg->TDMUCFG,TSIP_TDMUCFG_BEND,BIG);
             break;

         default:
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipTDMUsetup <Invalid endConf>");
#endif

             break;
        }

        /* 2. DMATCU : Max transfer priority */
        if( tdmu->maxPrio <= 7u )
        {
            CSL_FINS(devHandle->pReg->DMACTL,TSIP_DMACTL_MAXPRI,tdmu->maxPrio);
        }
        else
        {
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipTDMUsetup <Invalid maxPrio>");
#endif
        }

        /* 3. DMATCU : Transfer priority */
        if( tdmu->prio <= 7u )
        {
            CSL_FINS(devHandle->pReg->DMACTL,TSIP_DMACTL_PRI,tdmu->prio);
        }
        else
        {
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipTDMUsetup <Invalid prio>");
#endif
        }

        /* 4. DMATCU: TX Frame Interrupt Selection */
        switch(tdmu->txFint)
        {
         case PSP_TSIP_INT_XFER_ACK_ONLY:
             CSL_FINST(devHandle->pReg->XDLY,TSIP_XDLY_FRINT,ACK);
             break;
         case PSP_TSIP_INT_DELAY_CNT_ONLY:
             CSL_FINST(devHandle->pReg->XDLY,TSIP_XDLY_FRINT,DLY);
             break;
         case PSP_TSIP_INT_EITHER:
             CSL_FINST(devHandle->pReg->XDLY,TSIP_XDLY_FRINT,ACKORDLY);
             break;
         case PSP_TSIP_INT_BOTH:
             CSL_FINST(devHandle->pReg->XDLY,TSIP_XDLY_FRINT,ACKNDLY);
             break;
         default:
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipTDMUsetup <Invalid  txFint>");
#endif
             break;
        }

        /* 5. DMATCU: RX Frame Interrupt Selection */
        switch(tdmu->rxFint)
        {
         case PSP_TSIP_INT_XFER_ACK_ONLY:
             CSL_FINST(devHandle->pReg->RDLY,TSIP_RDLY_FRINT,ACK);
             break;
         case PSP_TSIP_INT_DELAY_CNT_ONLY:
             CSL_FINST(devHandle->pReg->RDLY,TSIP_RDLY_FRINT,DLY);
             break;
         case PSP_TSIP_INT_EITHER:
             CSL_FINST(devHandle->pReg->RDLY,TSIP_RDLY_FRINT,ACKORDLY);
             break;
         case PSP_TSIP_INT_BOTH:
             CSL_FINST(devHandle->pReg->RDLY,TSIP_RDLY_FRINT,ACKNDLY);
             break;
         default:
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipTDMUsetup <Invalid rxFint>");
#endif
             break;
        }

        /* 6. DMATCU: TX Super Frame Interrupt Selection */
        switch(tdmu->txSint)
        {
         case PSP_TSIP_INT_XFER_ACK_ONLY:
             CSL_FINST(devHandle->pReg->XDLY,TSIP_XDLY_SFINT,ACK);
             break;
         case PSP_TSIP_INT_DELAY_CNT_ONLY:
             CSL_FINST(devHandle->pReg->XDLY,TSIP_XDLY_SFINT,DLY);
             break;
         case PSP_TSIP_INT_EITHER:
             CSL_FINST(devHandle->pReg->XDLY,TSIP_XDLY_SFINT,ACKORDLY);
             break;
         case PSP_TSIP_INT_BOTH:
             CSL_FINST(devHandle->pReg->XDLY,TSIP_XDLY_SFINT,ACKNDLY);
             break;
         default:
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipTDMUsetup <Invalid txSint>");
#endif
             break;
        }

        /* 7. DMATCU: RX Super Frame Interrupt Selection */
        switch(tdmu->rxSint)
        {
         case PSP_TSIP_INT_XFER_ACK_ONLY:
             CSL_FINST(devHandle->pReg->RDLY,TSIP_RDLY_SFINT,ACK);
             break;
         case PSP_TSIP_INT_DELAY_CNT_ONLY:
             CSL_FINST(devHandle->pReg->RDLY,TSIP_RDLY_SFINT,DLY);
             break;
         case PSP_TSIP_INT_EITHER:
             CSL_FINST(devHandle->pReg->RDLY,TSIP_RDLY_SFINT,ACKORDLY);
             break;
         case PSP_TSIP_INT_BOTH:
             CSL_FINST(devHandle->pReg->RDLY,TSIP_RDLY_SFINT,ACKNDLY);
             break;
         default:
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipTDMUsetup <Invalid rxSint>");
#endif
             break;
         }

        /* 8. DMATCU : TX Frame Interrupt Delay */
        if( tdmu->txFintDly <= (0x7Fu) )
        {
            CSL_FINS(devHandle->pReg->XDLY,TSIP_XDLY_FDLY,tdmu->txFintDly);
        }
        else
        {
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipTDMUsetup <Invalid txFintDly>");
#endif
        }

        /* 9. DMATCU : RX Frame Interrupt Delay */
        if( tdmu->rxFintDly <= (0x7Fu) )
        {
            CSL_FINS(devHandle->pReg->RDLY,TSIP_RDLY_FDLY,tdmu->rxFintDly);
        }
        else
        {
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
             TSIP_DEBUG("HAL_tsipTDMUsetup <Invalid rxFintDly>");
#endif
        }

    }
    else
    {
        result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
        TSIP_DEBUG("HAL_tsipTDMUsetup <Invalid Params>");
#endif

    }
    return result;
}

/**
 * \brief    Get TDMU setup
 *
 * \param    devHandle [IN]    DDC port handle
 * \param    tdmu      [OUT]   Place holder for TDMU config
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipGetTdmuSetup(
                            DDC_tsipDevHandle  devHandle,
                            PSP_tsipTdmuConf  *tdmu
                            )
{
    PSP_Result result = PSP_SOK;

    if( (NULL != devHandle) && (NULL != tdmu) )
    {

        /* 1. TDMU: Get Endian selection for CB */
        tdmu->endian = (PSP_TsipEndian)CSL_FEXT(
                                                     devHandle->pReg->TDMUCFG,
                                                     TSIP_TDMUCFG_BEND
                                                     );


        /* 2. DMATCU : Get Max transfer priority */
        tdmu->maxPrio = CSL_FEXT(devHandle->pReg->DMACTL,TSIP_DMACTL_MAXPRI);

        /* 3. DMATCU : Get Transfer priority */
        tdmu->prio    = CSL_FEXT(devHandle->pReg->DMACTL,TSIP_DMACTL_PRI);

        /* 4. DMATCU: Get TX Frame Interrupt Selection */
        tdmu->txFint  = (PSP_TsipIntSelect)CSL_FEXT(
                                                devHandle->pReg->XDLY,
                                                TSIP_XDLY_FRINT
                                                );

        /* 5. DMATCU: Get RX Frame Interrupt Selection  */
        tdmu->rxFint  = (PSP_TsipIntSelect)CSL_FEXT(
                                                devHandle->pReg->RDLY,
                                                TSIP_RDLY_FRINT
                                                );

        /* 6. DMATCU: Get TX Super Frame Interrupt Selection */
        tdmu->txSint  = (PSP_TsipIntSelect)CSL_FEXT(
                                                devHandle->pReg->XDLY,
                                                TSIP_XDLY_SFINT
                                                );

        /* 7. DMATCU: Get RX Superframe Interrupt Selection */
        tdmu->rxSint = (PSP_TsipIntSelect)CSL_FEXT(
                                               devHandle->pReg->RDLY,
                                               TSIP_RDLY_SFINT
                                               );

        /* 8. DMATCU : Get TX Frame Interrupt Delay */
        tdmu->txFintDly = CSL_FEXT(devHandle->pReg->XDLY,TSIP_XDLY_FDLY);

        /* 9. DMATCU : Get RX Frame Interrupt Delay */
        tdmu->rxFintDly = CSL_FEXT(devHandle->pReg->RDLY,TSIP_RDLY_FDLY);

    }
    else
    {
        result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
        TSIP_DEBUG("HAL_tsipGetTDMUsetup <Invalid Params>");
#endif

    }
    return result;
}

/**
 * \brief    Resets(Software Reset) the SIU module
 *
 * \param    devHandle     [IN]    DDC port handle
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipSiuReset(DDC_tsipDevHandle  devHandle)
{
    PSP_Result  result = PSP_SOK;
    Bool        bRstStat;
    Uint32      rstTimeOut = HAL_TSIP_MAX_RESET_TIMEOUT;

    if(NULL != devHandle)
    {
        /* Issue SIU reset command */
        HAL_TsipResetSiu(devHandle);

        rstTimeOut = (0xFFFFu);

        /* Wait for the SIU reset to complete. i.e Reset bit
        to be cleared(bool status FALSE). In case of HW
        stall, the control will come out this loop once the
        timeout value becomes zero, i.e Driver shouldnt be stalled*/
        do
        {

         bRstStat = (Bool)HAL_TsipIsSiuResetDone(devHandle);
         rstTimeOut--;
        }while( (TRUE == bRstStat) && (0 != rstTimeOut) );

        if(0 == rstTimeOut)
        {
            result = PSP_E_TIMEOUT; /* HW not functioning. Timeout occurred */
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }
    return result;
}

/**
 * \brief    Resets(Software Reset) the TDMU-DMATCU modules
 *
 * \param    devHandle     [IN]    DDC port handle
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipTdmuReset(DDC_tsipDevHandle  devHandle)
{
    PSP_Result  result = PSP_SOK;
    Bool        bRstStat;
    Uint32      rstTimeOut = HAL_TSIP_MAX_RESET_TIMEOUT;

    if(NULL != devHandle)
    {
        /* Issue TDMU-DMATCU reset command */
        HAL_TsipResetTdmu(devHandle);
        do
        {
         /* Wait for reset to complete. In case of HW stall, the
          control will come out this loop once the timeout value
          becomes zero, i.e Driver shouldnt be stalled. */

         bRstStat = (Bool)HAL_TsipTdmuResetStat(devHandle);
         rstTimeOut--; /* Decrement the timeout */
        }while( (TRUE == bRstStat) && (0 != rstTimeOut) );

        if(0 == rstTimeOut)
        {
            result = PSP_E_TIMEOUT; /* HW not functioning. Timeout occurred */
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }
    return result;
}


/**
 * \brief    Set the bitmap for the specified TSIP channel context
 *
 * \param    ioHandle      [IN]    DDC channel handle
 * \param    pBitmap       [IN]    Given bitmap configuration
 * \param    ctxt          [IN]    Context or Bitmap (A or B)
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipSetBitmap(
                         DDC_tsipIoHandle       ioHandle,
                         PSP_tsipBitmap const  *pBitmap,
                         HAL_TsipChanBmapCtxt   ctxt
                         )
{
    PSP_Result        result  = PSP_SOK;
    Uint32            index   = 0;
    Uint32            tsIndex = 0;
    Int32             shIndex = 0;
    Uint32            temp    = 0;
    Uint32            tmpBm   = 0;

    volatile Uint32   *pTmpBm   = NULL; /* Temp Bitmap pointer */

    const    Uint32    num32bit = (PSP_TSIP_NUM_TIMESLOTS >> 4u);
    /*
     Calculation of number of 32-bit words needed to configure
     bitmaps for this TSIP SoC design.
     num of 32-bit words = (number of timeslots * 2bits/timeslot )/ 32 
     */
    if( (NULL == ioHandle) || (NULL == pBitmap))
    {
        result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
        TSIP_DEBUG("HAL_tsipSetBitmap<Invalid Params>");
#endif

    }
    else
    {
        if( HAL_TSIP_A == ctxt )
        {
            pTmpBm = &ioHandle->bmRegs->BMA[0];
        }
        else
        {
            if( HAL_TSIP_B == ctxt )
            {
                pTmpBm = &ioHandle->bmRegs->BMB[0];
            }
            else
            {
                result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG            
                TSIP_DEBUG("HAL_tsipSetBitmap<Invalid Context>");
#endif            
            }
        }

        /**** Configuration of A or B bitmap ****/

        if( NULL != pTmpBm )
        {

            tsIndex = 0; /* Timeslot counter */

            /* Set bitmap with given configuration PSP_TsipPcmType */
            for(index=0; index < num32bit; index++)
            {
                /* Clear (Disable) the bitmap register first for sake */
                pTmpBm[index] = 0;

                /* Calculating the new Bitmap value from the input configuration 
                  value */
                temp = 0;
                for(shIndex=30u; shIndex >=0; shIndex-= 2u ) 
                {
                   /* Setting individual 2-bits in 32-bit word corresponding
                     to the particular timeslot */
                   tmpBm = (Uint32)(pBitmap->pcmType[tsIndex++]);
                   temp |= (Uint32)(tmpBm << shIndex);
                }

                /* Write the new bitmap value into bitmap register */
                pTmpBm[index] = temp;
            }
        }
        else
        {
            result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG            
            TSIP_DEBUG("HAL_tsipSetBitmap<Invalid Bitmap address>");
#endif            
        }

    }
    return result;
}

/**
 * \brief    Get the current bitmap configuration for the specified
 *           TSIP channel context
 *
 * \param    ioHandle      [IN]    DDC channel handle
 * \param    pBitmap       [OUT]   Place holder for bitmap
 * \param    ctxt          [IN]    Context or Bitmap (A or B)
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipGetBitmap(
                         DDC_tsipIoHandle      ioHandle,
                         PSP_tsipBitmap       *pBitmap,
                         HAL_TsipChanBmapCtxt  ctxt
                         )
{
    PSP_Result        result  = PSP_SOK;
    Uint32            index   = 0;
    Uint32            tsIndex = 0;
    Int32             shIndex = 0;
    Uint32            temp    = 0;

    volatile Uint32   *pTmpBm  = NULL; /* Temp Bitmap pointer */
    const Uint32      num32bit = (PSP_TSIP_NUM_TIMESLOTS >> 4u);
    /*
     Calculation of number of 32-bit words needed to configure
     bitmaps for this TSIP SoC design.
     num of 32-bit words = (number of timeslots * 2bits/timeslot )/ 32
     */

    if( (NULL == ioHandle) || (NULL == pBitmap))
    {
        result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
        TSIP_DEBUG("HAL_tsipGetBitmap<Invalid Params>");
#endif

    }
    else
    {
        /* Clear place holder */
        for(index=0; index < num32bit; index++)
        {
            pBitmap->pcmType[index] = PSP_TSIP_PCM_DISABLED;
        }

        if( HAL_TSIP_A == ctxt )
        {
            pTmpBm = &ioHandle->bmRegs->BMA[0];
        }
        else
        {
            if( HAL_TSIP_B == ctxt )
            {
                pTmpBm = &ioHandle->bmRegs->BMB[0];
            }
            else
            {
                result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG            
                TSIP_DEBUG("HAL_tsipGetBitmap<Invalid Context>");
#endif            
            }
        }

        /***** Get the Bitmap configurations *****/

        if( NULL != pTmpBm )
        {
            tsIndex = 0; /* Timeslot counter */
            /* Get bitmap  */
            for(index=0; index < num32bit; index++)
            {
               temp = pTmpBm[index];

               /* Extracting individual 2-bits */
               for(shIndex=30u; shIndex >=0; shIndex-= 2u )
               {
                   pBitmap->pcmType[tsIndex++] =
                    (PSP_TsipPcmType)((temp>>shIndex) & 0x3u );
               }
            }
        }
        else
        {
            result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG            
            TSIP_DEBUG("HAL_tsipGetBitmap<Invalid Bitmap Address>");
#endif            
        }

    }
    return result;
}


/**
 * \brief    Configures the DMA channel framebuffer configuration for the
 *           specified TSIP channel context.
 *
 * \param    ioHandle      [IN]    DDC channel handle
 * \param    dConf         [IN]    Given DMATCU configuration
 * \param    ctxt          [IN]    Context (A or B)
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipDmaConfig(
                         DDC_tsipIoHandle            ioHandle,
                         PSP_tsipDmaConfig const    *dConf,
                         HAL_TsipChanBmapCtxt        ctxt
                         )
{
    PSP_Result    result = PSP_SOK;

    if( (NULL == ioHandle) || (NULL == dConf))
    {
        result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
        TSIP_DEBUG("HAL_tsipDmaConfig<Invalid Params>");
#endif
    }
    else
    {
        switch(ctxt)
        {
         case HAL_TSIP_A:
             /* Set A context DMATCU config with the given configurations.*/
             ioHandle->dchRegs->ABASE   = (Uint32)dConf->baseAddr;
             ioHandle->dchRegs->AFALLOC = dConf->frameAlloc;
             ioHandle->dchRegs->AFSIZE  = dConf->frameSize;
             ioHandle->dchRegs->AFCNT   = dConf->frameCnt;

             break;

         case HAL_TSIP_B:

             ioHandle->dchRegs->BBASE   = (Uint32)dConf->baseAddr;
             ioHandle->dchRegs->BFALLOC = dConf->frameAlloc;
             ioHandle->dchRegs->BFSIZE  = dConf->frameSize;
             ioHandle->dchRegs->BFCNT   = dConf->frameCnt;
             break;

         case HAL_TSIP_NONE:
         default:
             result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
        TSIP_DEBUG("HAL_tsipDmaConfig<Invalid Context>");
#endif

        }
    }
    return result;
}

/**
 * \brief    Get the current DMATCU frame buffer configurations for
 *           both TSIP channel contexts(A and B)
 *
 * \param    ioHandle      [IN]    DDC channel handle
 * \param    pIO           [OUT]   Place holder for DMATCU conf(A&B)
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipGetDmaConfig(
                         DDC_tsipIoHandle   ioHandle,
                         PSP_tsipIoParams   *pIO
                         )
{
    PSP_Result    result = PSP_SOK;

    if( (NULL == ioHandle) || (NULL == pIO))
    {
        result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
        TSIP_DEBUG("HAL_tsipGetDmaConfig<Invalid Params>");
#endif

    }
    else
    {
        pIO->dmaA.baseAddr   = (Uint32 *)ioHandle->dchRegs->ABASE;
        pIO->dmaA.frameAlloc = ioHandle->dchRegs->AFALLOC;
        pIO->dmaA.frameCnt   = ioHandle->dchRegs->AFCNT;
        pIO->dmaA.frameSize  = ioHandle->dchRegs->AFSIZE;

        pIO->dmaB.baseAddr   = (Uint32 *)ioHandle->dchRegs->BBASE;
        pIO->dmaB.frameAlloc = ioHandle->dchRegs->BFALLOC;
        pIO->dmaB.frameCnt   = ioHandle->dchRegs->BFCNT;
        pIO->dmaB.frameSize  = ioHandle->dchRegs->BFSIZE;
    }

    return result;
}

/**
 * \brief    Get the active context of the specified TX/RX channel
 *
 * \param    ioHandle      [IN]    DDC channel handle
 * \param    pCtxt         [OUT]   Place holder for the context
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipGetActiveContext(
                                DDC_tsipIoHandle    ioHandle,
                                HAL_TsipChanBmapCtxt *pCtxt
                                )
{
    PSP_Result       result = PSP_SOK;
    CSL_TsipRegsOvly reg    = NULL;

    if( (NULL == ioHandle) || (NULL == pCtxt) )
    {
        result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
        TSIP_DEBUG("HAL_tsipGetActiveContext<Invalid Params>");
#endif

    }
    else
    {
        /* Get register overlay address */
        reg = ioHandle->devHandle->pReg;

        if( NULL != reg )
        {
            if(PSP_TSIP_MODE_TX == ioHandle->mode) /* TX chan*/
            {
                /* Get the Context information for TX channel 0*/
                *pCtxt = (HAL_TsipChanBmapCtxt)CSL_FEXT(
                                                    reg->XCHST,
                                                    TSIP_XCHST_CH0CS
                                                    );
            }
            else /* RX channel */
            {
                /* Get the context information for Rx Channel 0 */
                *pCtxt = (HAL_TsipChanBmapCtxt)CSL_FEXT(
                                                    reg->RCHST,
                                                    TSIP_RCHST_CH0CS
                                                    );
            }
        }
        else
        {
            result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
            TSIP_DEBUG("HAL_tsipGetActiveContext<Invalid register Params>");
#endif
        }
    }

    return result;
}

/**
 * \brief    Get the active bitmap of the specified TX/RX channel
 *
 * \param    ioHandle      [IN]    DDC channel handle
 * \param    pCtxt         [OUT]   Place holder for the bitmap
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipGetActiveBitmap(
                               DDC_tsipIoHandle    ioHandle,
                               HAL_TsipChanBmapCtxt *pCtxt
                               )
{
    PSP_Result        result = PSP_SOK;
    CSL_TsipRegsOvly  reg    = NULL;

    if( (NULL == ioHandle) || (NULL == pCtxt) )
    {
        result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
        TSIP_DEBUG("HAL_tsipGetActiveBitmap<Invalid Params>");
#endif

    }
    else
    {
        /* Get register overlay address */
        reg = ioHandle->devHandle->pReg;

        if( NULL != reg )
        {
            if(PSP_TSIP_MODE_TX == ioHandle->mode) /* TX chan*/
            {

                /* Get the current Bitmap for TX channel 0 */
                *pCtxt = (HAL_TsipChanBmapCtxt)CSL_FEXT(
                                                    reg->XBMST,
                                                    TSIP_XBMST_CH0BMS
                                                    );
            }
            else /* RX channel */
            {

                /* Get the active bitmap for RX channel 0 */
                *pCtxt = (HAL_TsipChanBmapCtxt)CSL_FEXT(
                                                    reg->RBMST,
                                                    TSIP_RBMST_CH0BMS
                                                    );
            }
        }
        else
        {
            result = PSP_E_INVAL_PARAM;
#ifdef _DEBUG
            TSIP_DEBUG("HAL_tsipGetActiveBitmap<Invalid register Params>");
#endif
        }
    }

    return result;
}



/*!
*! Revision History
*! ================
*! 12-Jun-2007    JP : Removed Pre-silicon related info and code segments
*! 15-Mar-2007    JP : Used SOC defined Macros for base address, devNum
*! 17-Jan-2007    JP : Added Loopback mode configuration
*!
*! 13-Dec-2006    JP : Changed Log_printf to TSIP_DEBUG
*!                     Changed local variable names start
*!                     with 'g' to non-g; More comments.
*! 06-Dec-2006    JP : Added Log_printf() traces
*! 23-Nov-2006    JP : Added HAL APIs definitions.
*! 21-Nov-2006    JP : Initial Creation (0.1)
*! */
