/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/** \file    ddc_tsipHal.h
 *
 *  \brief    Header file for DDC TSIP Hardware Abstraction Layer (HAL)
 *
 *  (C)    Copyright 2006,    Texas Instruments, Inc
 *
 *  \author    Platform Support Group
 *
 *  \version    0.1
 */

#ifndef _DDC_TSIP_HAL_H_
#define _DDC_TSIP_HAL_H_

/* Includes */
#include <ti/sdo/pspdrivers/drivers/tsip/src/ddc_tsip.h>
#include <ti/sdo/pspdrivers/soc/c6452/dsp/cslr.h>

#ifdef __cplusplus
extern "C"
{
#endif

/*
 * Macro HW functionalities definitions
 */


/**
 * \brief    SIU or TDMU Reset timeout value
 *      After issuing SIU or DMATCU reset, this macro decides the maximum time 
 *      to wait for reset to complete
 */
#define HAL_TSIP_MAX_RESET_TIMEOUT            (0xFFFFu)

/**
 * \brief    Enable particular TX or RX DMATCU channel
 * \param    ioHandle   [IN]  DDC channel handle
 */
#define HAL_TsipEnableDmaChan(ioHandle)                                      \
        (CSL_FINST(ioHandle->chenRegs->CHEN, TSIP_CHEN_CHENB, ENB))

/**
 * \brief    Disable particular TX or RX DMATCU channel
 * \param    ioHandle   [IN]  DDC channel handle
 */
#define HAL_TsipDisableDmaChan(ioHandle)                                     \
        (CSL_FINST(ioHandle->chenRegs->CHEN, TSIP_CHEN_CHENB, DIS))

/**
 * \brief    Enable DMATCU-TDMU engines
 * \param    devHandle   [IN]  DDC device handle
 */
#define HAL_TsipEnableDmatcu(devHandle)                                      \
        (CSL_FINST(devHandle->pReg->TDMUCTL, TSIP_TDMUCTL_DMAENB, ENB))

/**
 * \brief    Disable DMATCU-TDMU engines
 * \param    devHandle   [IN]  DDC device handle
 */
#define HAL_TsipDisableDmatcu(devHandle)                                     \
        ( CSL_FINST(devHandle->pReg->TDMUCTL, TSIP_TDMUCTL_DMAENB, DIS) )

/**
 * \brief    Enable SIU Transmit engine
 * \param    devHandle   [IN]  DDC device handle
 */
#define HAL_TsipEnableTxSiu(devHandle)                                       \
        ( CSL_FINST(devHandle->pReg->SIUCTL, TSIP_SIUCTL_XENB, ENB) )

/**
 * \brief    Disable SIU Transmit engine
 * \param    devHandle   [IN]  DDC device handle
 */
#define HAL_TsipDisableTxSiu(devHandle)                                      \
        ( CSL_FINST(devHandle->pReg->SIUCTL, TSIP_SIUCTL_XENB, DIS) )

/**
 * \brief    Enable SIU Receive engine
 * \param    devHandle   [IN]  DDC device handle
 */
#define HAL_TsipEnableRxSiu(devHandle)                                       \
        ( CSL_FINST(devHandle->pReg->SIUCTL, TSIP_SIUCTL_RENB, ENB) )

/**
 * \brief    Disable SIU Receive engine
 * \param    devHandle   [IN]  DDC device handle
 */
#define HAL_TsipDisableRxSiu(devHandle)                                      \
        ( CSL_FINST(devHandle->pReg->SIUCTL, TSIP_SIUCTL_RENB, DIS) )


/**
 * \brief    Resets the DMATCU-TDMU engine and wait for the
 *           reset to complete
 * \param    devHandle   [IN]  DDC device handle
 */
#define HAL_TsipResetTdmu(devHandle)                                       \
        ( CSL_FINS(devHandle->pReg->RST, TSIP_RST_DMA, 1) )


/**
 * \brief    Check DMATCU-TDMU reset status
 * \param    devHandle   [IN]  DDC device handle
 */
#define HAL_TsipTdmuResetStat(devHandle)                                 \
        ( CSL_FEXT(devHandle->pReg->RST, TSIP_RST_DMA) )


/**
 * \brief    Resets the SIU engine and wait for the reset to complete
 * \param    devHandle   [IN]  DDC device handle
 */
#define HAL_TsipResetSiu(devHandle)                                          \
        ( CSL_FINS(devHandle->pReg->RST, TSIP_RST_SIU, 1) );



/**
 * \brief    Check SIU reset status
 * \param    devHandle   [IN]  DDC device handle
 */
#define HAL_TsipIsSiuResetDone(devHandle)                                    \
        ( CSL_FEXT(devHandle->pReg->RST, TSIP_RST_SIU) )


/**
 * \brief    Check Error Queue Overflow
 * \param    ioHandle   [IN]  DDC channel handle
 */
#define HAL_TsipIsErrQueueOvf(ioHandle)                                      \
        ( CSL_FEXT(ioHandle->errRegs->ERRCNT, TSIP_ERRCNT_QOV) )

/**
 * \brief    Clear Error Queue Overflow condition
 * \param    ioHandle   [IN]  DDC channel handle
 */
#define HAL_TsipClearErrQueueOvf(ioHandle)                                   \
        ( CSL_FINS(ioHandle->errRegs->ERRCTL, TSIP_ERRCTL_CLRQOV, 1) )

/**
 * \brief    POP the top of the error entry out off Error Queue
 * \param    ioHandle   [IN]  DDC channel handle
 */
#define HAL_TsipPopErrQueue(ioHandle)                                        \
        ( CSL_FINS(ioHandle->errRegs->ERRCTL, TSIP_ERRCTL_POPQ, 1) )


/**
 * \brief    Read Error Count
 * \param    ioHandle   [IN]  DDC channel handle
 */
#define HAL_TsipReadErrCnt(ioHandle)                                         \
        ( CSL_FEXT(ioHandle->errRegs->ERRCNT, TSIP_ERRCNT_CNT) )

/**
 * \brief    Read Error (code + info)
 * \param    ioHandle   [IN]  DDC channel handle
 */
#define HAL_TsipReadError(ioHandle)                                          \
        ( ioHandle->errRegs->ERRQ )

/**
 * \brief    Read Error Count
 * \param    errInfo   [IN]  complete error message
 */
#define HAL_TsipExtErrCode(errInfo)                                         \
        ( CSL_FEXT(errInfo, TSIP_ERRQ_ERRCODE) )


/**
 * \brief    Read Error Info
 * \param    errInfo   [IN]  complete error message
 */
#define HAL_TsipExtErrInfo(errInfo)                                         \
        ( CSL_FEXT(errInfo, TSIP_ERRQ_INFO) )




/**
 * \brief    TSIP channel bitmaps/contexts <br>
 *
 * \note     The values of each enum is in accordance with register values
 *  of the same. Should not change these values.
 */
typedef enum HAL_TsipBitmapContext_t
{
    HAL_TSIP_NONE = 0x0,
    /**< None of A and B                                                   */
    HAL_TSIP_A    = 0x1u,
    /**< Bitmap A or Context A                                             */
    HAL_TSIP_B    = 0x2u
    /**< Bitmap B or Context B                                             */
} HAL_TsipChanBmapCtxt;



/*********************************************
 * Function Prototypes
 *********************************************/


/**
 * \brief    Get the memory map(register) base Address for TSIP
 *
 * \param    devHandle   [IN]    DDC port handle
 *
 * \note    On success, this function will update pRegs pointer
 *  of port object with valid register base address.
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipGetBaseAddr(DDC_tsipDevHandle devHandle);

/**
 * \brief    Get the DMA channel memory map base address for the tx/rx channel
 *
 * \param    ioHandle     [IN]    DDC channel handle
 *
 * \note    On success, this function will update chan regs pointers
 *  of the channel object with valid register base addresses
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipGetChanBaseAddr(DDC_tsipIoHandle ioHandle);

/**
 * \brief    Setup SIU HW
 *
 * \param    devHandle     [IN]    DDC port handle
 * \param    siu           [IN]    SIU config params
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipSiuSetup(
                        DDC_tsipDevHandle       devHandle,
                        PSP_tsipSiuConf const  *siu
                        );
/**
 * \brief    Get SIU HW setup
 *
 * \param    devHandle     [IN]    DDC port handle
 * \param    siu          [OUT]   Place holder for SIC config
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipGetSiuSetup(
                           DDC_tsipDevHandle  devHandle,
                           PSP_tsipSiuConf   *siu
                           );


/**
 * \brief    Set TDMU hardware setup
 *
 * \param    devHandle [IN]    DDC port handle
 * \param    tdmu      [IN]    TDMU config params
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipTdmuSetup(
                         DDC_tsipDevHandle        devHandle,
                         PSP_tsipTdmuConf const  *tdmu
                         );

/**
 * \brief    Get TDMU setup
 *
 * \param    devHandle [IN]    DDC port handle
 * \param    tdmu      [OUT]   Place holder for TDMU config
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipGetTdmuSetup(
                            DDC_tsipDevHandle  devHandle,
                            PSP_tsipTdmuConf  *tdmu
                            );


/**
 * \brief    Resets(Software Reset) the SIU module
 *
 * \param    devHandle     [IN]    DDC port handle
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipSiuReset(DDC_tsipDevHandle  devHandle);

/**
 * \brief    Resets(Software Reset) the TDMU-DMATCU modules
 *
 * \param    devHandle     [IN]    DDC port handle
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipTdmuReset(DDC_tsipDevHandle  devHandle);

/**
 * \brief    Set the bitmap for the specified TSIP channel context
 *
 * \param    ioHandle      [IN]    DDC channel handle
 * \param    pBitmap       [IN]    Given bitmap configuration
 * \param    ctxt          [IN]    Context or Bitmap (A or B)
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipSetBitmap(
                         DDC_tsipIoHandle       ioHandle,
                         PSP_tsipBitmap const  *pBitmap,
                         HAL_TsipChanBmapCtxt   ctxt
                         );

/**
 * \brief    Get the current bitmap configuration for the specified
 *           TSIP channel context
 *
 * \param    ioHandle      [IN]    DDC channel handle
 * \param    pBitmap       [OUT]   Place holder for bitmap
 * \param    ctxt          [IN]    Context or Bitmap (A or B)
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipGetBitmap(
                         DDC_tsipIoHandle      ioHandle,
                         PSP_tsipBitmap       *pBitmap,
                         HAL_TsipChanBmapCtxt  ctxt
                         );

/**
 * \brief    Configures the DMA channel framebuffer configuration for the
 *           specified TSIP channel context.
 *
 * \param    ioHandle      [IN]    DDC channel handle
 * \param    dConf         [IN]    Given DMATCU configuration
 * \param    ctxt          [IN]    Context (A or B)
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipDmaConfig(
                         DDC_tsipIoHandle            ioHandle,
                         PSP_tsipDmaConfig const    *dConf,
                         HAL_TsipChanBmapCtxt        ctxt
                         );

/**
 * \brief    Get the current DMATCU frame buffer configurations for
 *           both TSIP channel contexts(A and B)
 *
 * \param    ioHandle      [IN]    DDC channel handle
 * \param    pIO           [OUT]   Place holder for DMATCU conf(A&B)
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipGetDmaConfig(
                         DDC_tsipIoHandle   ioHandle,
                         PSP_tsipIoParams     *pIO
                         );

/**
 * \brief    Get the active context of the specified TX/RX channel
 *
 * \param    ioHandle      [IN]    DDC channel handle
 * \param    pCtxt         [OUT]   Place holder for the context
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipGetActiveContext(
                                DDC_tsipIoHandle    ioHandle,
                                HAL_TsipChanBmapCtxt *pCtxt
                                );

/**
 * \brief    Get the active bitmap of the specified TX/RX channel
 *
 * \param    ioHandle      [IN]    DDC channel handle
 * \param    pCtxt         [OUT]   Place holder for the bitmap
 *
 * \return    PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
PSP_Result HAL_tsipGetActiveBitmap(
                               DDC_tsipIoHandle    ioHandle,
                               HAL_TsipChanBmapCtxt *pCtxt
                               );



#ifdef __cplusplus
}
#endif

#endif /* _DDC_TSIP_HAL_H_ */

/*!
*! Revision History
*! ================
*! 12-Jun-2007    JP : Removed Pre-silicon related info and code segments
*! 15-Mar-2006    JP : Removed Unused Macros
*! 17-Jan-2006    JP : Added TSTKEY value macro
*! 21-Nov-2006    JP : Initial Creation(0.1)
*! */
