/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/** \file    psp_tsipApi.h
 *
 *  \brief   TSIP interface definition header file
 *
 *  This file contains APIs defination of APIs that are wrapper APIs for DDC
 *  layer. These APIs will be called by DDA layer.
 *
 *  (C)    Copyright 2006,    Texas Instruments, Inc
 *
 *  \author    Platform Support Group
 *
 *  \version    0.1
 */

#ifndef _PSP_TSIP_API_H_
#define _PSP_TSIP_API_H_

/* Standard Includes */
#include <ti/sdo/pspdrivers/soc/c6452/dsp/tistdtypes.h>

/* Specific Includes */
#include <ti/sdo/pspdrivers/common/psp_common.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osList.h>

#include <log.h>  /* for BIOS LOG_printf utility */

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * \brief    Log trace object for BIOS real time logging
 */
extern far LOG_Obj trace;

/**
 *  \defgroup  TSIP Debug print macros
 *       Currently define to print on BIOS log trace.
 *       User can choose his own print utility by
 *       changing these macros definitions.
 *  @{
 */
#define TSIP_DEBUG(a)         LOG_printf(&trace,a)
/**< Print the string                            */
#define TSIP_DEBUG1(a,b)      LOG_printf(&trace,a,b)
/**< Print the string + arg1                     */
#define TSIP_DEBUG2(a,b,c)    LOG_printf(&trace,a,b,c)
/**< Print the string + arg1 + arg2              */
#define TSIP_DEBUG3(a,b,c,d)  LOG_printf(&trace,a,b,c,d)
/**< Print the string + arg1 + arg2 + arg3       */
/* @}  */


/**
 * \brief    Return IOP status for Flush requests(-12)
 */
#define PSP_TSIP_IO_FLUSHED            (PSP_E_TIMEOUT - 1)

/**
 * \brief    Return IOP status for ABORT requests(-13)
 */
#define PSP_TSIP_IO_ABORTED            (PSP_E_TIMEOUT - 2)


/**
 * \brief    TSIP Channel operating modes <br>
 *
 */
typedef enum PSP_TsipMode_t
{
    PSP_TSIP_MODE_RX = 0,
    /**< TSIP DMATCU receive channel                                      */
    PSP_TSIP_MODE_TX = 1,
    /**< TSIP DMATCU transmit channel                                     */
    PSP_TSIP_MODE_NONE
    /**< TSIP DMATCU channel NONE                                         */
} PSP_TsipMode;

/**
 * \brief    IO request structure
 *
 * The IO structure is used to hold the IO request details
 * for the channel.
 *
 * \note    GIO model is using its proprietary IOP structure
 * called IOM_Packet. So the application using GIO model no need to
 * pass this structure. The conversion of this struct to IOM_packet
 * should be done in DDA. For the application it is only required to
 * pass PSP_tsipIoParams and number of superframes in the iop.
 * This structure is maintained in DDC level.
 *
 */
typedef struct PSP_tsipIoRequest_t
{
    PSP_tsipIoParams       *pIoParams;
    /**< Pointer to application passed IO params s                        */
    Uint16                 numSframe;
    /**< Number of superframes in the IOP request                         */
    Uint16                 xferActual;
    /**< Actual superframes transferred. Filled by the driver             */
    Ptr                    appPtr;
    /**< Pointer to store application upper layer IOP handle              */
    PSP_Result             status;
    /**< Status of this request. Filled by driver                         */
} PSP_tsipIoRequest;

/**
 * \brief    TSIP Application callback function pointer
 */
typedef void (*PSP_tsipAppCallback)(
                                    Ptr chanp,
                                    PSP_tsipIoRequest const *ioReq
                                   );

/**
 * \brief   TSIP DMATCU channel ISR function pointer
 */
typedef void (*PSP_tsipIsr)(Ptr chanp);

/*
 * Function Prototypes
 */

/**
 * \brief    Sets up the TSIP device instance
 *
 *  This function is used to setup the TSIP hardware specified
 *  by the device ID.
 *
 * \param    devId         [IN]     Device instance number
 * \param    initCfg       [IN]     Hardware initialization parameters
 * \param    arg           [IN]     application handle as arg.
 *
 * \return
 *           PSP_SOK         if success<br>
 *           PSP ERROR Codes if failure<br>
 */
PSP_Result PSP_tsipCreate(
                      Int32                    devId,
                      PSP_tsipDevParams const *initCfg,
                      Ptr                      arg
                      );
/**
 * \brief    Un-initializes the TSIP device instance
 *
 * This function is used to do the reverse operation of
 * PSP_tsipCreate. It uninitializes the TSIP HW for the given
 * devId.
 *
 * \param    devId         [IN]    DevId of TSIP to be deleted.
 *
 * \return
 *           PSP_SOK            if success<br>
 *           PSP_E_INVAL_PARAM  if invalid arguments<br>
 *
 */
PSP_Result PSP_tsipDelete(Int32 devId);

/**
 * \brief    Allocates and configures the DMATCU TX/RX channel
 *
 * This function is used to do the following configurations<br>
 *    - Configure DMATCU tx/rx channel with framebuffer details
 *    - Register tx/rx channel superframe interrupt handler
 *    - Register tx/rx channel error interrupt handler
 *
 *
 * \param    devId         [IN]     TSIP devId(instance)
 * \param    mode          [IN]     TX or RX channel
 * \param    chanParams    [IN]     channel parameters from user
 * \param    pStatus       [OUT]    Status of PSP_tsipOpen
 * \param    drvCb         [IN]     driver callback function
 * \param    cbArg         [IN]     callback function Arguments
 *
 * \return
 *           DDC channel handle   if success<br>
 *           NULL                 if failure<br>
 */
PSP_Handle PSP_tsipOpen(
                    Int32                     devId,
                    PSP_TsipMode              mode,
                    PSP_tsipChanParams const *chanParams,
                    PSP_Result               *pStatus,
                    PSP_tsipAppCallback       drvCb,
                    Ptr                       cbArg
                    );

/**
 * \brief    Un-allocates a DMATCU TX/RX channel
 *
 *  This function is used to perform the reverse operation
 *  of PSP_tsipOpen().<br>
 *
 *  It does the following actions
 *      - Disables tx/rx channel interrupts
 *      - Unregister tx/rx channel superframe interrupt handler
 *      - Unregister tx/rx channel error interrupt handler if
 *        both tx/rx channels are not in use.
 *      - Make the channel resource as FREE for further channel
 *        requests
 *
 * \param    pIoHandle     [IN]     TSIP DDC channel pointer
 *
 * \return
 *           PSP_SOK             if success<br>
 *           PSP ERROR Codes     if failure<br>
 */
PSP_Result PSP_tsipClose(PSP_Handle pIoHandle);

/**
 * \brief    Performs an IOCTL
 *
 * This function is used to perform IOCTLs on the Channel.
 *
 * \param    pIoHandle     [IN]        TSIP DDC channel pointer
 * \param    cmd           [IN]        IOCTL command
 * \param    arg           [IN/OUT]    Place holder for cmd I/O
 *
 * \return
 *           PSP_SOK              if success<br>
 *           PSP ERROR codes      if failure<br>
 */
PSP_Result PSP_tsipIoctl(
                     PSP_Handle         pIoHandle,
                     PSP_TsipIoctlCmd   cmd,
                     Ptr                arg
                     );

/**
 *  \brief    Submits IOP requests to perform input/output
 *
 *  This function submits the IOP to ddc_tsipSubmitIop() which inturn
 *  do the appropriate queuing of IOP, configures the TX/RX DMATCU channel
 *  and enables the channel.
 *
 *  \param    pIoHandle  [IN]     Handle to the TSIP channel
 *  \param    pIoReq     [IN]     Pointer to IO request
 *
 *  \return
 *            PSP_SOK           if success<br>
 *            PSP Error Codes   if failure<br>
 */
PSP_Result PSP_tsipSubmit(
                      PSP_Handle                 pIoHandle,
                      PSP_tsipIoRequest  const  *pIoReq
                      );    
/**
 *  \brief    Aborts the TSIP channel IOP operations
 *
 *   Aborts the input or output channel operation of the TSIP driver
 *   Driver sends the notifications to the application for all pending I/O
 *   packets after abort(Disabling the channel). The application can submit
 *   fresh I/O requests after abort.
 *
 *  \param    pIoHandle   [IN]     Handle to the TSIP channel
 *
 *  \return
 *            PSP_SOK           if success<br>
 *            PSP Error Codes   if failure<br>
 */
PSP_Result PSP_tsipAbort(PSP_Handle pIoHandle);

/**
 *  \brief    Flush the TSIP channel IOP operations
 *
 *   Flush the input or output channel operation of the TSIP driver.
 *   If the channel is RX, this function aborts(same as PSP_tsipAbort) all
 *   pending RX IOPs and if the channel is TX, this function waits(pending
 *   on the semaphore) for all pending TX IOPs to be completed by the driver.
 *
 *  \note FLUSH and ABORT functionalities are the same for RX channel.
 *
 *  \param    pIoHandle   [IN]     Handle to the TSIP channel
 *
 *  \return
 *            PSP_SOK           if success<br>
 *            PSP Error Codes   if failure<br>
 */
PSP_Result PSP_tsipFlush(PSP_Handle pIoHandle);

#ifdef __cplusplus
}
#endif

#endif /* _PSP_TSIP_API_H_ */

/*!
*! Revision History
*! ================
*! 27-Jun-2007    JP : PSP_tsipFlush function added
*! 13-Dec-2006    JP : Initial Creation (0.1)
*! */

