/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file   psp_uart.h
 *
 *  \brief  UART interface definition
 *
 *  This file contains the interfaces, data types and symbolic definitions
 *  that are needed by the application to utilize the serivces of the UART
 *  device driver.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *
 *  \version       1.0
 *
 *  <b>Compiler Switch Used</b>
 *
 *  <b>_DEBUG</b><br>
 *   This is used as a flag to compiler whether to include the debug statement
 *   inserted in the code into the final image. This flag helps to build DEBUG
 *   image of the program. For RELEASE images this is not passed to the compiler.
 *
 *  <b>CHIP_XXXX</b><br>
 *   The CSL layer is written in a common file for all the variants of a SOC. This
 *   flag differentiates the variant we are compiling for, for e.g. CHIP_DM648,
 *   and the CSL definitions for that variant appropriately gets defined for
 *   registers base addresses, num of ports of a peripheral etc.
 *
 *  <b>UART_INSTRUMENTATION_ENABLED</b><br>
 *   This flag is passed to the compiler to include the instrumentation code parts
 *   into the final image/lib of the program. This helps build the iRelease/iDebug
 *   versions of the image/lib with a common code base.
 *
 *  <b>PSP_UART_ASYNC_MODE_SUPPORT</b><br>
 *   This option is used to support asynchronous mode of operation.
 *
 *  <b>PSP_UART_EDMA_SUPPORT</b><br>
 *   This option is passed to the compiler for EDMA mode.
 *
 *  <b>PSP_UART_BUFFERING_ENABLED</b><br>
 *   This option is passed to the compiler when Buffering mode is enabled.
 *
 *  <b>UART_DEBUG_PRINTF</b><br>
 *   They are the macros used to print debug messages. These expand a valid print statement.
 *
 */

#ifndef _PSP_UART_H_
#define _PSP_UART_H_

#include <ti/sdo/pspdrivers/common/psp_common.h>

#ifdef CHIP_DM648
#include <ti/sdo/pspdrivers/pal_sys/dm648/pal_sys_soc.h>
#endif

#ifdef CHIP_C6452
#include <ti/sdo/pspdrivers/pal_sys/c6452/pal_sys_soc.h>
#endif

#ifdef CHIP_DM6437
#include <ti/sdo/pspdrivers/pal_sys/dm6437/pal_sys_soc.h>
#endif

#ifdef CHIP_C6424
#include <ti/sdo/pspdrivers/pal_sys/c6424/pal_sys_soc.h>
#endif

#ifdef __cplusplus
extern "C"
{
#endif

/** \brief  Input freq to UART module                                       */

#if defined(CHIP_DM648) || defined(CHIP_C6452)
#define PSP_UART_MODULE_CLOCK               (PAL_SYS_CPU_CLOCK_FREQUENCY/6u)
#endif

#if defined(CHIP_DM6437) || defined(CHIP_C6424)
#define PSP_UART_MODULE_CLOCK                (27000000u)     
#endif


/** \brief  Number of UART instances in DM64G                               */
#define PSP_UART_NUM_INSTANCES                  (1u)

/**
 * \brief UART driver TX internal buffer size
 */
#define PSP_UART_MAX_BUFFER_SIZE                (1000u)

/** \brief  Maximum number of IOP support                                   */
#define PSP_UART_NUM_IOP                        (20u)

/** \brief Read Command                                                     */
#define PSP_UART_IOCMD_READ                     (0x1u)

/** \brief Write Command                                                    */
#define PSP_UART_IOCMD_WRITE                    (0x2u)

/** \brief Flush Command                                                    */
#define PSP_UART_IOCMD_FLUSH                    (0x3u)

/** \brief Abort  Command                                                   */
#define PSP_UART_IOCMD_ABORT                    (0x4u)

/** Default time out for sem take. This is used where timeout is not passed 
 *  from application
 */
#define PSP_UART_DEFAULT_SEM_TIMEOUT (1000u)


/**
 * \defgroup PSPUartErrorCodes Error Codes
 *
 * UART specific error codes. All error codes of the Stream hierarchy are
 * also applicable.
 *
 * @{
 */
/**  Error Base for the Uart specific error                                   */
#define PSP_UART_ERROR_BASE                     (-11)
/**  Error for the Uart DMA Operation Failure                                 */
#define PSP_UART_E_DMA_TRANSFER_FAIL            (PSP_UART_ERROR_BASE - 1)
/**  Error for the Uart invalid id given by application                       */
#define PSP_UART_E_INVALID_DEVICE_ID            (PSP_UART_ERROR_BASE - 2)
/**  Error for the Uart request timeout     e                                 */
#define PSP_UART_E_REQ_TIMEOUT                  (PSP_UART_ERROR_BASE - 3)
/**  Error for the Uart mode which are not supported                          */
#define PSP_UART_E_NOT_SUPPORTED                (PSP_UART_ERROR_BASE - 4)
/**  Error for the Uart IO Packet is cancelled                                */
#define PSP_UART_E_IOP_CANCELED                 (PSP_UART_ERROR_BASE - 5)
/**  Error for the Uart IO Packet is Aborted                                  */
#define PSP_UART_E_IOP_ABORTED                  (PSP_UART_ERROR_BASE - 6)
/**  Error for the Uart IO Packet is Flushed                                  */
#define PSP_UART_E_IOP_FLUSHED                  (PSP_UART_ERROR_BASE - 7)
/**  Error for the Uart IO Packet has encountered any error                   */
#define PSP_UART_E_IOP_ERROR                    (PSP_UART_ERROR_BASE - 8)
/**  Error for the Uart IO Operation has encountered error                    */
#define PSP_UART_E_IO_ERROR                     (PSP_UART_ERROR_BASE - 9)


/* @} PSPUartErrorCodes */

/* Forward declaration */
struct PSP_uartRequest_t;


/**
 *  \brief  PSP Uart Baud Rate
 *
 *  Uart Baud rate
 */
typedef enum PSP_UartBaudRate_t
{
    PSP_UART_BAUD_RATE_2_4K     = 2400,     /**< Baudrate 2400    bps    */
    PSP_UART_BAUD_RATE_4_8K     = 4800,     /**< Baudrate 4800    bps    */
    PSP_UART_BAUD_RATE_9_6K     = 9600,     /**< Baudrate 9600    bps    */
    PSP_UART_BAUD_RATE_19_2K    = 19200,    /**< Baudrate 19200   bps    */
    PSP_UART_BAUD_RATE_38_4K    = 38400,    /**< Baudrate 38400   bps    */
    PSP_UART_BAUD_RATE_57_6K    = 57600,    /**< Baudrate 57600   bps    */
    PSP_UART_BAUD_RATE_115_2K   = 115200    /**< Baudrate 115200  bps    */
} PSP_UartBaudRate;

/**
 *  \brief  PSP Uart Character Length
 *
 *  Uart Character length
 *
 *  \note The enum values should not be changed since it represents the
 *   actual register configuration values used to configure the UART in
 *   this SoC by the UART driver
 */
typedef enum PSP_UartCharLen_t
{
    PSP_UART_CHARLEN_5  = 0x0,        /**< Character Length 5  bits    */
    PSP_UART_CHARLEN_6  = 0x1,        /**< Character Length 6  bits    */
    PSP_UART_CHARLEN_7  = 0x2,        /**< Character Length 7  bits    */
    PSP_UART_CHARLEN_8  = 0x3         /**< Character Length 8  bits    */
} PSP_UartCharLen;

/**
 *  \brief  PSP Uart Parity
 *
 *  Uart Parity
 *
 *  \note The enum values should not be changed since it represents the
 *   actual register configuration values used to configure the UART in
 *   this SoC by the UART driver

 */
typedef enum PSP_UartParity_t
{
    PSP_UART_PARITY_NONE    = 0x0,    /**< Parity Bits NONE   bits    */
    PSP_UART_PARITY_ODD     = 0x8,    /**< Parity Bits ODD    bits    */
    PSP_UART_PARITY_EVEN    = 0x18    /**< Parity Bits EVEN   bits    */
} PSP_UartParity;

/**
 *  \brief  PSP Uart Stop Bits
 *
 *  Uart Stop Bits
 *
 *  \note The enum values should not be changed since it represents the
 *   actual register configuration values used to configure the UART in
 *   this SoC by the UART driver
 */
typedef enum PSP_UartNumStopBits_t
{
    PSP_UART_NUM_STOP_BITS_1    = 0,    /**< Stop Bits 0    bits    */
    PSP_UART_NUM_STOP_BITS_1_5  = 0x2,  /**< Stop Bits 1.5  bits    */
    PSP_UART_NUM_STOP_BITS_2    = 0x4   /**< Stop Bits 2    bits    */
} PSP_UartNumStopBits;

/**
 *  \brief  PSP Uart Flow Control
 *
 *   Uart Flow Control
 *
 *  \note The enum values should not be changed since it represents the
 *   actual register configuration values used to configure the UART in
 *   this SoC by the UART driver

 */
typedef enum PSP_UartFcType_t
{
    PSP_UART_FC_TYPE_NONE   = 0,    /**< No Flow Control       */
    PSP_UART_FC_TYPE_SW     = 1,    /**< Software Flow Control */
    PSP_UART_FC_TYPE_HW     = 2     /**< Hardware Flow Control */
} PSP_UartFcType;

/**
 *  \brief  PSP Uart Flow Control Parameters
 *          Uses to select the mode of Uart Software and Hardware flow Control 
 *  \note   SW flow control is not supported in this driver implementation
 *
 */
typedef enum PSP_UartFcParam_t
{
    PSP_UART_FC_NONE          = 0,    /**< NONE                      */
    PSP_UART_FC_XONXOFF_1     = 1,    /**< SW FC - XONXOFF 1         */
    PSP_UART_FC_XONXOFF_2     = 2,    /**< SW FC - XONXOFF 2         */
    PSP_UART_FC_XONXOFF_12    = 3,    /**< SW FC - XONXOFF 12        */
    PSP_UART_FC_AUTO_RTS_CTS  = 4,    /**< HW FC - Auto RTS CTS      */
    PSP_UART_FC_AUTO_CTS_ONLY = 5     /**< HW FC - Auto CTS only     */
} PSP_UartFcParam;

/**
 * \brief  PSP Uart Rx Trigger Level Param
 *
 *  Uart Receive Trigger Level Param
 *
 *  \note The enum values should not be changed since it represents the
 *   actual register configuration values used to configure the UART in
 *   this SoC by the UART driver

 */
typedef enum PSP_UartRxTrigLvl_t
{
    PSP_UART_RX_TRIG1   = 1,    /**< Trigger Level 1    */
    PSP_UART_RX_TRIG4   = 4,    /**< Trigger Level 4    */
    PSP_UART_RX_TRIG8   = 8,    /**< Trigger Level 8    */
    PSP_UART_RX_TRIG14  = 14    /**< Trigger Level 14   */
}PSP_UartRxTrigLvl;



/**
 *  \brief  PSP Uart Ioctl commands
 *
 *  Uart Ioctl commands
 */
typedef enum PSP_UartIoctlCmd_t
{
    PSP_UART_IOCTL_SET_BAUD = 1,
    /**< Set baud rate, cmdArg= struct* UartBaudRate                   */
    PSP_UART_IOCTL_SET_STOPBITS,
    /**< Set number of stop bits, cmdArg = sturct* UartNumStopBits     */
    PSP_UART_IOCTL_SET_DATABITS,
    /**< Set number of Data bits, cmdArg = sturct* UartCharLen         */
    PSP_UART_IOCTL_SET_PARITY,
    /**< Set parity type, cmdArg = sturct* UartParity                  */
    PSP_UART_IOCTL_SET_FLOWCONTROL,
    /**< Set flowcontrol, cmdArg = sturct* UartFlowControl             */
    PSP_UART_IOCTL_SET_TRIGGER_LEVEL,
    /**< Changing Trigger level                                        */
    PSP_UART_IOCTL_RESET_RX_FIFO,
    /**< Resets the UART HW RX  FIFO                                   */
    PSP_UART_IOCTL_RESET_TX_FIFO,
    /**< Resets the UART HW TX FIFO                                    */
    PSP_UART_IOCTL_CANCEL_CURRENT_IO,
    /**< Cancel the current IO in TX or RX channel                     */
    PSP_UART_IOCTL_GET_STATS,
    /**< Getting the Uart stats for DDC                                */
    PSP_UART_IOCTL_CLEAR_STATS,
    /**< Clearing the Stats of DDC                                     */
    PSP_UART_IOCTL_MAX_IOCTL
    /**< Book-keep - Max ioctl's                                       */
} PSP_UartIoctlCmd;

/**
 *  \brief  PSP IO Mode
 *
 *  Uart Channel mode - INPUT or OUTPUT
 */
typedef enum PSP_UartIoMode_t
{

    PSP_UART_INPUT = 0,
    PSP_UART_OUTPUT

}PSP_UartIoMode;


/**
 *  \brief  PSP Uart Flow Control Configuration
 *
 *  Uart Software Flow Control Configuration
 */
typedef struct PSP_UartFlowControl_t
{
    PSP_UartFcType  fcType;         /**< Flow Control type     */
    PSP_UartFcParam fcParam;        /**< Flow Control param    */
} PSP_uartFlowControl;


/**
 *  \brief  PSP Device configuration parameters(at Init/Create time)
 *
 *  UART Configuration structure pointer passed to PSP_uartCreate function
 */
typedef struct PSP_uartDevParams_t
{
    Uint32              inputFreq;       /**< Input Frequency in MHZ     */
    Bool                fifoEnable;      /**< Fifo mode of Operation     */
    PSP_OpMode          opMode;          /**< Driver operational mode    */
    Bool                loopbackEnabled; /**< LoopBack Mode flag         */
    PSP_UartBaudRate    baudRate;        /**< Baudrate of Operation      */
    PSP_UartNumStopBits stopBits;        /**< Stopbits of Operation      */
    PSP_UartCharLen     charLen;         /**< Charecter Length           */
    PSP_UartParity      parity;          /**< Parity of Operatio         */
    PSP_UartRxTrigLvl   rxThreshold;     /**< Rx FIFO trigger level      */
    PSP_uartFlowControl fc;              /**< Flow Control               */
} PSP_uartDevParams;

/**
 *  \brief  PSP Uart channel config parameters
 *
 *  UART Configuration structure pointer passed to PSP_uartOpen function
 */
typedef struct PSP_uartIoParams_t
{
    PSP_Handle              hEdma;
    /**< EDMA handle. Valid only in DMA opmode. We dont support this mode  */
}PSP_uartIoParams;




/**
 * \brief UART DDC Statistics Collection Object
 *
 * Statistics are collected on a per-controller basis for UART. Hence, an
 * object of this type is found embedded in the DDC_UartObj structure.
 */
typedef struct PSP_uartStats_t
{
    Uint32  rxBytes;
    /**< Number bytes received                                              */
    Uint32  txBytes;
    /**< Number bytes transmitted                                           */
    Uint32  overrun;
    /**< Number of overrun errors                                           */
    Uint32  rxTimeout;
    /**< Number of Rx timeouts                                              */
    Uint32  rxFramingError;
    /**< Number of Rx Framing errors                                        */
    Uint32  rxBreakError;
    /**< Number of Rx Break Errors                                          */
    Uint32  rxParityError;
    /**< Number of Rx Parity Errors                                         */
} PSP_uartStats;


/**
 *  \brief  Input/Output data parameters
 *
 *  Structure for the uart specific buffer address to be passed to the GIO.
 */
typedef struct PSP_uartDataParam_t
{
    char      *addr;            /**<  Buffer Address       */
    Uint32    timeout;          /**<  Timeout Value        */
} PSP_uartDataParam;


#ifdef __cplusplus
}
#endif

#endif  /* _PSP_UART_H_ */

/*!
*! Revision History
*
*! ================
*! 12-Jun-2007    JP            : Updated UART review comments
*!
*! 15-Jan-2007    JP            : Refined PSP_uartStats according to RDD
*!                                IOCTL commands modificaton wrt RDD.
*!                                Tabs replacement by 4 spaces- Alignment
*!
*! 14-Jan-2007    JP            : Removed unwanted IOCTL commands and refined
*!                                some ioctl command names to suitable name
*!
*! 08-Jan-2007    Kapil         : Removed some elements not required and
*!                                worked on Doxygen
*!
*! 07-Jan-2007    JP            : Removed unused struct elements. Review
*!                                comments updation started.
*! 20-Dec-2006    Kapil/Vichu   : Initial Creation (0.1)
*! */

