/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file   dda_uartBios.c
 *
 *  \brief   Mini-driver(DDA Layer)for the UART.
 *
 *  This file implements the UART mini-driver to be used with DSP-BIOS.
 *
 *  (C)    Copyright 2006,    Texas Instruments, Inc
 *
 *
 *  \version    1.0
 *
 */

 /* Header files includes */

#include <std.h>
#include <ecm.h>
#include <c64.h>

#include <ti/sdo/pspdrivers/drivers/uart/src/ddc_uart.h>
#include <ti/sdo/pspdrivers/drivers/uart/src/dda_uartBios.h>
#include <ti/sdo/pspdrivers/drivers/uart/src/psp_uartApi.h>
#include <ti/sdo/pspdrivers/drivers/uart/psp_uart.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osProtect.h>
#include <ti/sdo/pspdrivers/pal_os/pal_ostime.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osMem.h>

#ifdef CHIP_DM6437
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/soc.h>
#endif

#ifdef CHIP_C6424
#include <ti/sdo/pspdrivers/soc/c6424/dsp/soc.h>
#endif

#ifdef CHIP_DM648
#include <ti/sdo/pspdrivers/soc/dm648/dsp/soc.h>
#endif

#ifdef CHIP_C6452
#include <ti/sdo/pspdrivers/soc/c6452/dsp/soc.h>
#endif


#ifdef UART_INSTRUMENTATION_ENABLED
#include <ti/sdo/pspdrivers/pal_os/bios/psp_log.h>
#endif


/*-------- local(static) function prototypes ------------------------*/
static Int uart_mdBindDev(Ptr *devp, Int devId, Ptr devParams);
static Int uart_mdUnBindDev(Ptr devp);
static Int uart_mdCreateChan(
                    Ptr                 *chanp,
                    Ptr                 devp,
                    String              name,
                    Int                 mode,
                    Ptr                 chanParams,
                    IOM_TiomCallback    cbFxn,
                    Ptr                 cbArg
                    );
static Int uart_mdDeleteChan(Ptr chanp);
static Int uart_mdSubmitChan(Ptr chanp, IOM_Packet *packet);
static Int uart_mdControlChan(Ptr chanp, Uns cmd, Ptr arg);

static void mdLayerCallback(
                        Ptr Port,
                        Ptr arg,
                        const PSP_uartRequest *ioRequest,
                        Int mode
                        );

static Int uart_getIomErrorCode(PSP_Result pspErrCode);


/**
 * \brief Public IOM interface table
 *
 * This table holds the interface functions exposed by the mini-driver to
 * the GIO layer.
 */
IOM_Fxns UARTMD_FXNS =
{
    &uart_mdBindDev,
    &uart_mdUnBindDev,
    &uart_mdControlChan,
    &uart_mdCreateChan,
    &uart_mdDeleteChan,
    &uart_mdSubmitChan,
};


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Public Functions Definitions
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/**
 *  \brief    Registering interrupt handler
 *
 *   This function is invoked from the DDC layer in order for the
 *   DDC to register UART interrupts with the requested interrupt
 *   handler with the OS (BIOS) and enables the interrupt.
 *
 *  \param    devp         [IN]    Pointer to the device
 *  \param    intNum       [IN]    interrupt number
 *  \param    evt          [IN]    event number
 *  \param    intIsr       [IN]    ISR function pointer
 *
 *  \return
 *            PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
/* ARGSUSED */
PSP_Result DDA_uartRegisterInterrupt(
                                 Ptr devp,
                                 Uint32 intNum,
                                 Uint16 evt,
                                 pspUartIsr intIsr
                                 )

{

    ECM_Attrs           ecmattrs    = ECM_ATTRS;
    static Uns          cookie      = 0;
    PSP_Result          result      = PSP_SOK;


    if(( NULL == devp) || ( NULL == intIsr) )
    {
        result = PSP_E_INVAL_PARAM;
    }
    else
    {

        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

        /*Enable the interrupt later*/
        ecmattrs.unmask = 0u;

        /*setup arg such that isr will get 'chan' as arg*/
        ecmattrs.arg    = (Arg)devp;

        /* Plug the ISR with interrupt (event) number */
        ECM_dispatchPlug(evt, (ECM_Fxn)intIsr, &ecmattrs);


        /* Enabling the event */
        ECM_enableEvent(evt);

        /*
        * Enabling the HWI_ID where corresponding ECM group is configured
        */
#if defined(CHIP_DM648) || defined(CHIP_C6452)
        C64_enableIER(HWI_EVT02_CMB_IRQ);
#endif

#if defined(CHIP_DM6437) || defined(CHIP_C6424)
        C64_enableIER(HWI_EVT03_CMB_IRQ);
#endif


        /* Enabling the interrupts */
        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
    }

    return result;
}


/**
 * \brief    Unregistering interrupt handler
 *
 * This function is invoked from DDC to unregister and disable interrupt
 * handler and interrupt cause respectively.
 *
 *  \param    intNum        [IN]    interrupt number
 *  \param    evt           [IN]    event number
 *
 *  \return   PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
/* ARGSUSED */
PSP_Result DDA_uartUnRegisterInterrupt(Uint32 intNum, Uint16 evt)
{

    Uint32    cookie      = 0;


    PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

    /* Disable the event*/
    ECM_disableEvent(evt);

    /* Enabling the interrupts */
    PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

    return PSP_SOK;
}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Private Functions Definitions
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


/**
 *  \brief    Allocates and configures the UART port specified by devId
 *
 *  This function gets the default configuration for the port from the
 *  statically allocated default object, and intialize the UART hardware
 *  for this port specified by devId by calling PSP_uartCreate()
 *
 * \param    devp         [OUT]    Pointer to hold allocated port object
 * \param    devId        [IN]     Device number
 * \param    devParams    [IN]     Device parameter structure that supplies
 *                                  data to initialize the UART
 *
 * \return   IOM_COMPLETED if success<br>
 *           IOM_EBADARGS  if invalid arguments<br>
 *           IOM_EALLOC    if error in DDC layer<br>
 */
static Int uart_mdBindDev(Ptr *devp, Int devId, Ptr devParams)
{

    UartPortObj        *uartport   = NULL;
    PSP_Result          result     = PSP_SOK;
    Int32               IOMstatus  = IOM_COMPLETED;
    PSP_uartDevParams*  params     = NULL;

    static UartPortObj  uart_Ports[IOM_UART_NUMPORTS]= {DDA_PORTOBJ_INIT};

    /* The driver must receive a valid device parameters  */
    if( ((IOM_UART_NUMPORTS > devId) && (devParams != NULL))  && (NULL != devp))
    {
        /* Get the device parameters of the specified port */
        uartport = &uart_Ports[devId];
        uartport->instNum = (Uint16)devId;


        /* Check if the port was already bound to the DSP/BIOS */
        if ( TRUE != uartport->inUse )
        {
            /* Mark the port as in use */
            uartport->inUse = TRUE;

            params = (PSP_uartDevParams*)devParams;

            uartport->opMode = params->opMode;

            /*In turn call the PSP function to do all init work*/
            result = (PSP_Result)PSP_uartCreate(
                                        devId,
                                        params,
                                        (Ptr)uartport
                                        );

            IOMstatus = uart_getIomErrorCode(result);

            if(IOM_COMPLETED != IOMstatus)
            {
                *devp = NULL;
            }
            else
            {
                *devp = uartport;
            }
        }
        else
        {
            *devp = NULL;
            IOMstatus = IOM_EBADMODE;
        }
    }
    else
    {
        *devp = NULL;
        IOMstatus = IOM_EBADARGS;
    }

    return IOMstatus;
}

/**
 *  \brief    Closes the UART device from use.
 *
 *  This function unwinds the actions performed by mdbinddev.
 *  This is accomplished by calling PSP_uartDelete()
 *
 * \param    devp         [IN]    Pointer to port object to be freed
 *
 * \return
 *           IOM_COMPLETED if success<br>
 *           IOM_ABORTED   if any error<br>
 */
static Int uart_mdUnBindDev(Ptr devp)
{
    UartPortHandle      port        = NULL;
    PAL_Result          result      = PSP_E_INVAL_PARAM;
    Int32               IOMstatus   = IOM_EBADARGS;

    /*
     * Check if this is a valid handle
     */
    if (NULL == devp)
    {
        IOMstatus = IOM_EBADARGS;
    }
    else
    {
        port = (UartPortHandle)devp;

       /*
        * Free the allocated port resource
        */
        port->inUse = FALSE;

        /*
        * Invoke DDC layer function to delete the device
        */
        result = PSP_uartDelete(port->instNum, NULL);

        /* Clear the device's channel objects  */
        PAL_osMemSet(port->chans,0x0,IOM_UART_NUMCHANS * sizeof(UartChanObj));

        /* Convert PSP code to IOM code */
        IOMstatus = uart_getIomErrorCode(result);

    }/* if devp is NULL */

    return IOMstatus;
}

/**
 *  \brief    Creates a read/write communication channel.
 *
 *  This function is used to create a tx/rx channel to communicate
 *  data between the application and the UART device instance. This
 *  function inturn calls DDC layer function PSP_uartOpen().
 *  Returns configured channel handle to application.
 *
 *  This function is called in response to a GIO_create call.
 *
 *      Pre-requisites:
 *       -  Valid chanParams structure
 *       -  Valid device pointer
 *
 * \note    Please refer to Appendix A : IOM interface of spru616
 *          DSP/BIOS Driver Developer's Guide for details on
 *          uart_mdCreateChan parameters.
 *
 * \param    chanp        [OUT]    Pointer to hold allotted channel
 * \param    devp         [IN]     Device pointer
 * \param    name         [IN]     UART Instance name like /UART0
 * \param    mode         [IN]     channel  mode -> input or output
 * \param    chanParams   [IN]     channel parameters from user
 * \param    cbFxn        [IN]     callback function pointer
 * \param    cbArg        [IN]     callback function Arguments
 *
 * \return
 *           IOM_COMPLETED     if success<br>
 *           IOM_EALLOC        if no free channel available<br>
 *           IOM_EBADARGS      if invalid input parameters<br>
 *
 */
/* ARGSUSED */
static Int uart_mdCreateChan (
                        Ptr              *chanp,
                        Ptr              devp,
                        String           name,
                        Int              mode,
                        Ptr              chanParams,
                        IOM_TiomCallback cbFxn,
                        Ptr              cbArg
                        )
{


    UartPortHandle  port       = NULL;
    Uint32          cookie     = 0;
    Int32           IOMstatus  = IOM_EBADARGS;
    UartChanHandle  chan       = NULL;

#ifndef PSP_UART_ASYNC_MODE_SUPPORT
    PAL_OsSemAttrs semAttrs    = PAL_OSSEM_ATTRS;
#endif

    /*validate parameters*/
    if ( ((NULL == cbFxn)|| (NULL == cbArg)) ||
         (((IOM_INPUT != mode) && (IOM_OUTPUT != mode)) ||
                ((NULL == chanParams) || (NULL == devp))))
    {
         IOMstatus = IOM_EBADARGS;
    }
    else
    {
        port = (UartPortHandle)devp;

        chan = &port->chans[mode-1];

#ifndef PSP_UART_ASYNC_MODE_SUPPORT
	    semAttrs.type = PAL_OSSEM_TYPE_FIFO;
#endif

        if(NULL == chan)
        {
            IOMstatus = IOM_EBADARGS;
        }
        else
        {
            chan->mode = mode;
            chan->port = port;

            /* enter critical section */
            PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

            /* Make sure the channel wasn't already created */
            if (TRUE == chan->inUse)
            {
                IOMstatus = IOM_EBADMODE;
                /* Leave critical section */
                PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
            }
            else
            {
                /* Mark channel status as allocated.*/
                chan->inUse = TRUE;

                /* Leave critical section */
                PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

                /*Assign the respective callback function*/
                chan->cbFxn = cbFxn;
                chan->cbArg = cbArg;

                /* Call DDC layer Function */
                chan->ioHandle = PSP_uartOpen(
                                         port->instNum,
                                         (PSP_UartIoMode)(chan->mode-1),
                                         (PSP_uartIoParams *)chanParams,
                                         (PSP_uartAppCallback)&mdLayerCallback,
                                         NULL
                                         );
#ifndef PSP_UART_ASYNC_MODE_SUPPORT

                if (NULL != chan->ioHandle)
                {
                    IOMstatus = PAL_osSemCreate("UART_SYNC_SEM", 0,
                                            &semAttrs, &chan->hSyncSem);
                    if (IOMstatus != IOM_COMPLETED)
                    {
                        chan->ioHandle = NULL;
                    }
                }
#endif


                if (NULL == chan->ioHandle)
                {
                    IOMstatus =  IOM_ETIMEOUTUNREC;
                }
                else
                {
                    IOMstatus = IOM_COMPLETED;

                    if( NULL != chanp )
                    {
                        *chanp = (Ptr) chan;
                    }
                }
            }
        }
    }

    return IOMstatus;
}

/**
 *  \brief    Submit an I/O packet to a channel for processing
 *
 *  The GIO layer calls this function to cause the mini-driver
 *  to process the IOM_Packet for read/write/flush/abort operations.
 *  This function calls DDC layer funtion PSP_uartRead/PSP_uartWrite
 *  PSP_uartAbort calls to execute the
 *  request.
 *
 *  \param    chanp         [IN]    Pointer to channel
 *  \param    packet        [IN]    Pointer to packet to be submitted
 *
 *  \return
 *            IOM_COMPLETED if packet is fully processed<br>
 *            IOM_ABORTED   if packet is aborted<br>
 *            IOM_PENDING   if packet is not fully processed<br>
 *            IOM_EBADARGS  if invalid inputs<br>
 */
static Int uart_mdSubmitChan(Ptr chanp, IOM_Packet *packet)
{
    UartChanHandle            chan           = NULL;
    PAL_Result                result         = PSP_E_INVAL_PARAM;
    Int32                     IOMstatus      = IOM_EBADARGS;
    PSP_uartDataParam         *dataParam     = NULL;


    PSP_uartRequest           ioRequest      =   {
                                                    0,      /* cmd       */
                                                    NULL,   /* addr      */
                                                    0,      /* size      */
                                                    0,      /* actualSize*/
                                                    NULL,   /* cbFxn     */
                                                    NULL,   /* appPtr    */
                                                    0,      /* status    */
                                                    0       /* timeout   */
                                                };


    if((NULL == chanp) || (NULL == packet))
    {
        IOMstatus = IOM_EBADARGS;
    }
    else
    {

        /* Channel Handle */
        chan = (UartChanHandle)chanp;

        if( FALSE == chan->inUse )
        {
            IOMstatus = IOM_EBADARGS;
        }
        else
        {
            if( NULL != packet->addr)
            {
                dataParam = (PSP_uartDataParam*)packet->addr;

                /*
                 * This ioRequest is the intermediate structure used to convey
                 * the driver core about the request from GIO, to the underlying
                 * PSP_xxxx calls.The GIO IOP pointer is stored as a member,
                 * because we will need that during IO completion
                 */
                ioRequest.addr          = (Uint8 *)dataParam->addr;
                ioRequest.size          = packet->size;
                ioRequest.cbFxn         = NULL;
                ioRequest.appPtr        = (Ptr)packet;
                ioRequest.actualSize    = 0;
                ioRequest.status        = PSP_SOK;
                ioRequest.timeout       = dataParam->timeout;
            }

#ifndef PSP_UART_ASYNC_MODE_SUPPORT
            chan->isSyncMode            = TRUE;
#endif

#ifdef UART_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(
                      &DVTEvent_Log,
                      "UART",
                      PSP_DVT_DESC(
                                PSP_DVT_eFUNC_START,
                                PSP_DVT_dCHANNEL_ID_INST,
                                PSP_DVT_dPACKET_ID,
                                PSP_DVT_dSIZE_BYTES),
                      CHANNELID_INSTANCEID(
                           (packet->cmd),
                           (chan->port->instNum)),
                      ioRequest.addr,
                      ioRequest.size
                     );
#endif /* UART_INSTRUMENTATION_ENABLED */

            switch (packet->cmd)
            {
                case IOM_READ:
                case IOM_WRITE:
                        if ( (NULL != ioRequest.addr)
                                  && (0 != ioRequest.size))
                        {
                            ioRequest.cmd = (Uint8)(packet->cmd + 1);
                            result =   PSP_uartTransfer(
                                                (PSP_Handle)chan->ioHandle,
                                                &ioRequest,
                                                NULL
                                                );
                        }
                        break;

                case IOM_FLUSH:
                case IOM_ABORT:
                        result =   PSP_uartAbort((PSP_Handle)chan->ioHandle);
                        break;

                default:
                        result = PSP_E_INVAL_PARAM;
                        break;

            }

            IOMstatus = uart_getIomErrorCode (result);
        }
    }

#ifndef PSP_UART_ASYNC_MODE_SUPPORT

    if (IOMstatus == IOM_PENDING)
    {
        /* Sync Mode Operation: So wait on a semaphore for the timeout or
        completion or error */
        Uint32  cookie;

#ifdef PSP_UART_EDMA_SUPPORT
        Uint32  currentTick    = 0;
        Bool    timeoutElapsed = FALSE;

        if(chan->port->opMode == PSP_OPMODE_DMAINTERRUPT)
        {
            /* Update current tick value to perform timeout operation */
            currentTick = PAL_osGetCurrentTick();

            IOMstatus  = PAL_osSemTake(chan->hSyncSem, dataParam->timeout);

            /* Check whether timeout happened or not */
            timeoutElapsed = (Bool)PAL_osCheckTimeOut(
                                                  currentTick,
                                                  dataParam->timeout
                                                  );
            if( (TRUE == timeoutElapsed) )
            {
                IOMstatus = IOM_ETIMEOUT;
            }
        }
        else
#endif
        {
            IOMstatus = PAL_osSemTake(chan->hSyncSem, dataParam->timeout);

#ifdef UART_INSTRUMENTATION_ENABLED
            if (packet->cmd == PSP_UART_OUTPUT)
            {
                PSP_LOG_EVENT(
                              &DVTEvent_Log,
                              "UART",
                              PSP_DVT_DESC(   PSP_DVT_eDATA_SND_END,
                                              PSP_DVT_dCHANNEL_ID_INST,
                                              PSP_DVT_dPACKET_ID,
                                              PSP_DVT_dNONE),
                              CHANNELID_INSTANCEID(
                                   (PSP_UART_OUTPUT),
                                   (chan->port->instNum)),
                              ioRequest.addr);
            }
            else
            {
                PSP_LOG_EVENT(
                              &DVTEvent_Log,
                              "UART",
                              PSP_DVT_DESC(   PSP_DVT_eRCV_END,
                                              PSP_DVT_dCHANNEL_ID_INST,
                                              PSP_DVT_dPACKET_ID,
                                              PSP_DVT_dNONE),
                              CHANNELID_INSTANCEID(
                                   (PSP_UART_INPUT),
                                   (chan->port->instNum)),
                              ioRequest.addr);
            }
#endif /* UART_INSTRUMENTATION_ENABLED */

            if (IOM_COMPLETED != IOMstatus)
            {
                chan->isSyncMode  = FALSE;
                /* Either IOP timedout or error occurred and the notifycallback
                   released the semaphore so cancel the IOP */
                PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

                /* Cancel the current IO */
                result = PSP_uartIoctl(
                                  chan->ioHandle,
                                  PSP_UART_IOCTL_CANCEL_CURRENT_IO,
                                  NULL,
                                  NULL
                                  );


                PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

                IOMstatus = uart_getIomErrorCode (result);

                if(IOM_COMPLETED == IOMstatus)/* To check IOCTL success */
                {
                    IOMstatus = IOM_ETIMEOUT; /* Turn status to TimeOut */
                }

            }
        }

    }
#endif /* Not Defined PSP_UART_ASYNC_MODE_SUPPORT */

#ifdef UART_INSTRUMENTATION_ENABLED
            PSP_LOG_EVENT(
                          &DVTEvent_Log,
                          "UART",
                          PSP_DVT_DESC(
                                    PSP_DVT_eFUNC_END,
                                    PSP_DVT_dCHANNEL_ID_INST,
                                    PSP_DVT_dPACKET_ID,
                                    PSP_DVT_dSIZE_BYTES),
                          CHANNELID_INSTANCEID(
                               (packet->cmd),
                               (chan->port->instNum)),
                          ioRequest.addr,
                          0
                         );
#endif /* UART_INSTRUMENTATION_ENABLED */


    return IOMstatus;
}


/**
 *  \brief    Frees a channel and all it's associated resources.
 *
 *  This function inturn calls PSP_uartClose() of DDC layer funtion
 *
 *  \param    chanp  [IN]    Pointer to channel
 *
 * \return    IOM_COMPLETED  if success<br>
 *            IOM_EALLOC     if the channel is already deleted<br>
 *            IOM_EBADARGS   if invalid input parameters<br>
 */
static Int uart_mdDeleteChan(Ptr chanp)
{
    UartChanHandle      chan        = NULL;
    PAL_Result          result      = PSP_E_INVAL_PARAM;
    Int32               IOMstatus   = IOM_EBADARGS;

    if(NULL == chanp)
    {
        IOMstatus = IOM_EBADARGS;
    }
    else
    {
        chan        = (UartChanHandle) chanp;

        if (TRUE != chan->inUse)
        {
            IOMstatus = IOM_EBADMODE;
        }
        else
        {
            result = PSP_uartClose(chan->ioHandle, &chan->mode);

            if (PSP_SOK  == result)
            {
                /* Updates the channel port void*/
                chan->inUse = FALSE;
                chan->port = NULL;

#ifndef PSP_UART_ASYNC_MODE_SUPPORT
                /* Delete Sempahore for sync */
                PAL_osSemDelete(chan->hSyncSem);
#endif
            }

            IOMstatus = uart_getIomErrorCode(result);
        }
    }
    return IOMstatus;
}


/**
 *  \brief    Implements the IOCTLS for UART IOM mini driver
 *
 *  uart_mdControlChan() receives IOCTL commands from the
 *  GIO layer and then forwards the commands to the DDC by calling
 *  PSP_uartIoctl().
 *
 *  \param    chanp       [IN]    Pointer to channel
 *  \param    cmd         [IN]    IOCTL command
 *  \param    arg         [IN]    Command argument
 *
 *  \return   IOM_COMPLETED if command is executed correctly<br>
 *            IOM_ABORTED   if command returns error during execution<br>
 *            IOM_ENOTIMPL  if command is not supported<br>
 *            IOM_EBADARGS  if invalid input parameters<br>
 */
static Int uart_mdControlChan(Ptr chanp, Uns cmd, Ptr arg)
{
    UartChanHandle      chan        = NULL;
    PAL_Result          result      = PSP_E_INVAL_PARAM;
    Int32               IOMstatus   = IOM_EBADARGS;

    /* Checking if the handler is NULL*/
    if(NULL == chanp)
    {
        IOMstatus = IOM_EBADARGS;
    }
    else
    {
        /* setting up channel and port handler*/
        chan = (UartChanHandle)chanp;

        /* Check for the channel state */
        if (TRUE != chan->inUse)
        {
            IOMstatus = IOM_EBADMODE;
        }
        else
        {
            result = PSP_uartIoctl(
                              chan->ioHandle,
                              (PSP_UartIoctlCmd)cmd,
                              arg,
                              NULL
                              );

            IOMstatus = uart_getIomErrorCode(result);

        }
    }
    return (IOMstatus);
}

/**
 *  \brief DDC layer callback
 *
 *  This callback is made from the DDC layer. This function restores appropriate
 *  data into the IOP and  calls GIO callback
 *
 *  \param  Port      [IN]    Port handle
 *  \param  arg         [IN]    optional device-defined data structure
 *  \param  ioRequest [IN]    pointer to io request
 *  \param  mode      [IN]    channel  mode -> input or output
 *
 *  \return  Void
 */
/* ARGSUSED */
static void mdLayerCallback(
                        Ptr Port,
                        Ptr arg,
                        const PSP_uartRequest *ioRequest,
                        Int mode
                            )
{
    IOM_Packet      *packet     = NULL;
    UartPortObj     *port       = NULL;
    UartChanObj     *chan       = NULL;

    if (( NULL != Port ) && ( NULL != ioRequest ) )
    {
        port      = Port;
        packet    = (IOM_Packet *)ioRequest->appPtr;

        if ( NULL != packet)
        {
            packet->size    = ioRequest->size;
            packet->status  = ioRequest->status;
            packet->status  = uart_getIomErrorCode(packet->status);

            /* :NOTE: IOM mode & Driver Mode are using different values */
            chan = (UartChanObj*)&port->chans[mode];

            if (NULL != chan )
            {
#ifndef PSP_UART_ASYNC_MODE_SUPPORT
                if ( (chan->port->opMode != PSP_OPMODE_POLLED)
                      && (chan->isSyncMode == TRUE) )
                {
                    PAL_osSemGive(chan->hSyncSem);
                }
#else /* ASYNC mode of operation */
                
                if( NULL != chan->cbFxn )
                {
                    if( PSP_OPMODE_POLLED != chan->port->opMode )
                    {
                        /* Invoke Application callback for this channel */
                        (*chan->cbFxn)(chan->cbArg,packet);
                    }
                }

#ifdef UART_INSTRUMENTATION_ENABLED
                if (packet->cmd == PSP_UART_OUTPUT)
                {
                    PSP_LOG_EVENT(
                              &DVTEvent_Log,
                              "UART",
                              PSP_DVT_DESC(   PSP_DVT_eDATA_SND_END,
                                              PSP_DVT_dCHANNEL_ID_INST,
                                              PSP_DVT_dPACKET_ID,
                                              PSP_DVT_dNONE),
                              CHANNELID_INSTANCEID(
                                   (PSP_UART_OUTPUT),
                                   (chan->port->instNum)),
                              ioRequest.addr);
                }
                else
                {
                    PSP_LOG_EVENT(
                              &DVTEvent_Log,
                              "UART",
                              PSP_DVT_DESC(   PSP_DVT_eRCV_END,
                                              PSP_DVT_dCHANNEL_ID_INST,
                                              PSP_DVT_dPACKET_ID,
                                              PSP_DVT_dNONE),
                              CHANNELID_INSTANCEID(
                                   (PSP_UART_INPUT),
                                   (chan->port->instNum)),
                              ioRequest.addr);
                }
#endif /* UART_INSTRUMENTATION_ENABLED */

#endif
            }
        }
    }

}



/**
 *   \brief    PSP to IOM error code mapping
 *
 *   This function is used to map PSP error codes into appropriate IOM
 *   error codes.
 *
 *   \param    pspErrCode    [IN]    DDC layer error code
 *
 *   \return   IOM Error code
 */
static Int32 uart_getIomErrorCode(PSP_Result pspErrCode)
{
    /*------------------------------------------------------------------
                            IOM Error code look up table
     -------------------------------------------------------------------
     * The following mapping shows the mapping done from PSP_Result to
     * IOM error code. This will be used in scenario where IOMini driver
     * interfaces with the DDC (PSP component)
     *
     * PSP_SOK                 (0)   IOM_COMPLETED            0
     * PSP_SINPROGRESS         (1)   IOM_PENDING              1
     * PSP_E_DRIVER_INIT       (-1)  IOM_ETIMEOUTUNREC      -11
     * PSP_E_NO_MEMORY         (-2)  IOM_EALLOC              -5
     * PSP_E_RESOURCES         (-3)  IOM_EALLOC              -5
     * PSP_E_INVAL_STATE       (-4)  IOM_EBADMODE            -7
     * PSP_E_INVAL_PARAM       (-5)  IOM_EBADARGS           -10
     * PSP_E_NOT_SUPPORTE      (-6)  IOM_ENOTIMPL            -9
     * PSP_E_IO_CANCEL_FAIL    (-7)  IOM_ETIMEOUT            -2
     * PSP_E_FIFO_NOT_ENABLED  (-8)  IOM_EBADMODE            -7
     * PSP_E_INVALID_MODE      (-9)  IOM_EBADMODE            -7
     * PSP_E_INVALID_CMD       (-10) IOM_EBADARGS           -10
     *
     */
    static Int32 iomErrorCodeList[13] =
    {
        IOM_ETIMEOUT,
        IOM_EBADARGS, /* 0 */
        IOM_EBADMODE, /* 1 */
        IOM_EBADMODE, /* 2 */
        IOM_ETIMEOUT, /* 3 */
        IOM_ENOTIMPL, /* 4 */
        IOM_EBADARGS, /* 5 */
        IOM_EBADMODE, /* 6 */
        IOM_EALLOC,   /* 7 */
        IOM_EALLOC,   /* 8 */
        IOM_ETIMEOUTUNREC,/* 9 */
        IOM_COMPLETED,/* 10 */
        IOM_PENDING,  /* 11 */
    };

    Int32 iomCode = pspErrCode + 11;

    if( (iomCode >=0) && (iomCode < 13 ) )
    {
        iomCode = iomErrorCodeList[iomCode];
    }
    else
    {
        iomCode = IOM_EBADIO;
    }

    return(iomCode);
}

/*!
*! Revision History
*
*! ================
*! 14-Jan-2007    JP            : Review comments updation.
*!                                -> Valid validation of chan->inuse
*!                                -> Other review comments.
*! 08-Jan-2007    Kapil            : Removed some elements not required and
*!                                  worked on Doxygen
*!
*! 07-Jan-2007    JP            : Removed unused struct elements. Review
*!                                comments updation started.
*! 20-Dec-2006    Kapil/Vichu   : Initial Creation (0.1)
*! */




