/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file   dda_uartBios.h
 *
 *  \brief  Header file for UART Device Driver Adaptation-BIOS
 *
 *  This file has the globally visible data structures and definitions
 *  for the UART mini-driver.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *
 *  \version    1.0
 *
 */

#ifndef _DDA_UART_BIOS_H_
#define _DDA_UART_BIOS_H_

/* header includes */
#include <iom.h>
#include <ti/sdo/pspdrivers/drivers/uart/src/psp_uartApi.h>
#include <ti/sdo/pspdrivers/drivers/uart/psp_uart.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osSem.h>

#ifdef __cplusplus
extern "C" {
#endif

/**  \brief  Number of UART instances in the SoC */
#define IOM_UART_NUMPORTS       PSP_UART_NUM_INSTANCES 
/**  \brief  Number of UART channels supported   */
#define IOM_UART_NUMCHANS        (2u) 

/**
 *  \brief  channel structure
 *
 * There is one UartChanObj per direction.  This mini-driver must be
 * opened for input and output separately (does not support IOM_INOUT).
 *
 */
typedef struct UartChanObj_t
{
    Bool                    inUse;
    /**<  Flag to tell channel object resource is used or free */
    Int32                   mode;
    /**<  Type of the channel INPUT or OUTPUT                  */
    struct UartPortObj_t    *port;
    /**<  Handle to the device or port object                  */
    IOM_TiomCallback        cbFxn;          
    /**<  IOM call back function to notify I/O complete        */
    Ptr                     cbArg;
    /**<  IOM call back argument                               */
    PAL_OsSemHandle         chanProtectLock;
    /**<  Semaphore to protect channel                         */
    Bool                    isSyncMode;
    /**<  Boolean to find out whether to give the semaphore    */
    PAL_OsSemHandle         hSyncSem;
    /**< Semaphore used when the sync mode                     */
    PSP_Handle              ioHandle;
    /**<  iohandle from DDC                                    */
} UartChanObj , *UartChanHandle;

/**
 *  \brief  Default(init) Channel object values.
 */
#define DDA_CHANOBJ_INIT     \
{ \
    /* .inUse               = */    FALSE,        \
    /* .mode                = */    0,            \
    /* .port                = */    NULL,         \
    /* .cbFxn               = */    NULL,         \
    /* .cbArg               = */    NULL,         \
    /* .chanProtectLock     = */    NULL,         \
    /* .isSyncMode          = */    FALSE,        \
    /* .hSyncSem            = */    NULL,         \
    /* .ioHandle            = */    NULL          \
}


/**
 *  \brief  port structure
 *
 * There is one UartPortObj per UART.
 * This mini-driver supports 'IOM_UART_NUMPORTS' uart devices
 *
 */
typedef struct UartPortObj_t
{
    Uint32             instNum; 
    /**<  Instance number or device ID of the UART          */
    
    Bool               inUse;
    /**< Flag to tell the object resource is used or free   */

    UartChanObj        chans[IOM_UART_NUMCHANS];
    /**<  In/Out Channel config for each Port/Peripheral    */

    PSP_OpMode         opMode;
    /**< Mode of operation                                  */
} UartPortObj, *UartPortHandle;

/**
 *  \brief  Default(init) Port object values.
 */
#define DDA_PORTOBJ_INIT     \
{ \
    /* .instNum               = */    0xDEADu,             \
    /* .inUse                 = */    FALSE,               \
    /* .chans[0]              = */   {DDA_CHANOBJ_INIT,    \
    /* .chans[1]              = */    DDA_CHANOBJ_INIT},   \
    /* .opMode                = */    PSP_OPMODE_POLLED    \
}


/**
 *  \brief    Registering interrupt handler
 *
 *   This function is invoked from the DDC layer in order for the 
 *   DDC to register UART interrupts with the requested interrupt
 *   handler with the OS (BIOS) and enables the interrupt.
 *   
 *  \param    devp         [IN]    Pointer to the device
 *  \param    intNum       [IN]    interrupt number
 *  \param    evt          [IN]    event number
 *  \param    intIsr       [IN]    ISR function pointer
 *
 *  \return   
 *            PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
/* ARGSUSED */ 
PSP_Result DDA_uartRegisterInterrupt(
                                     Ptr devp,
                                     Uint32 intNum,
                                     Uint16 evt,
                                     pspUartIsr intIsr
                                    );
/*!
 * \brief    Unregistering interrupt handler
 *
 * This function is invoked from DDC to unregister and disable interrupt
 * handler and interrupt cause respectively.
 *
 *  \param    intNum        [IN]    interrupt number
 *  \param    evt           [IN]    event number
 *
 *  \return   PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
/* ARGSUSED */
PSP_Result DDA_uartUnRegisterInterrupt(Uint32 intNum, Uint16 evt);
                                     

#ifdef __cplusplus
}
#endif /* extern "C" */


#endif /* _DDA_UART_BIOS_H_ */


/*!
*! Revision History
*
*! ================
*! 14-Jan-2007    JP            : Created default init value macro for Chan 
*!                                and Device object
*! 08-Jan-2007    Kapil            : Removed some elements not required and 
*!                                  worked on Doxygen
*!
*! 07-Jan-2007    JP            : Removed unused struct elements. Review 
*!                                comments updation started.
*! 20-Dec-2006    Kapil/Vichu   : Initial Creation (0.1)
*! */

