/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file   ddc_uart.c
 *
 *  \brief  UART Driver Adaptation Common code (does not contain OS specifics)
 *
 *  This file contains adaptation of UART driver
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *
 *  \version       0.3
 */

#include <ti/sdo/pspdrivers/common/psp_common.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osProtect.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osSem.h>
#include <ti/sdo/pspdrivers/pal_os/pal_ostime.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osWait.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osMem.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osInt.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osTaskLet.h>
#include <ti/sdo/pspdrivers/drivers/uart/src/ddc_uart.h>
#include <ti/sdo/pspdrivers/drivers/uart/src/psp_uartApi.h>

#ifdef UART_INSTRUMENTATION_ENABLED
#include <ti/sdo/pspdrivers/pal_os/bios/psp_log.h>
#endif

#ifdef PSP_UART_EDMA_SUPPORT
#include <ti/sdo/edma3/drv/edma3_drv.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osCache.h>
#endif





/*
 * External References
 */
/*++++++++ extern data +++++++++++++++++*/

/*++++++++ extern functions +++++++++++++++++*/
extern PSP_Result DDA_uartRegisterInterrupt(
                                         Ptr        devp,
                                         Uint32     intNum,
                                         Uint16     evt,
                                         pspUartIsr intIsr
                                        );

extern PSP_Result DDA_uartUnRegisterInterrupt(
                                           Uint32 intNum,
                                           Uint16 evt
                                          );

/*+++++++++++++++++ global data ++++++++++++++++++++++*/

/**< Global DDC Device structure     */
DDC_uartDevObj    gDDC_uartDevObj[PSP_UART_NUM_INSTANCES];


/**> Default hardware configuration */
PSP_uartDevParams   guartDefaultConfig =
{
    PSP_UART_MODULE_CLOCK,      /* clock for the Uart        */
    TRUE,                       /* Enable/Disable Fifo       */
    PSP_OPMODE_INTERRUPT,          /* mode of the operation     */
    FALSE,                      /* Enable Loopback           */
    PSP_UART_BAUD_RATE_115_2K,  /* Baud rate                 */
    PSP_UART_NUM_STOP_BITS_1,   /* Number of stop bits       */
    PSP_UART_CHARLEN_8,         /* Character Length          */
    PSP_UART_PARITY_NONE,       /* Parity                    */
    PSP_UART_RX_TRIG14,         /* RX FIFO Threshold         */
    {                           /* ddc_UartFlowControlConfig */
      PSP_UART_FC_TYPE_NONE,    /* Flow control type         */
      PSP_UART_FC_NONE
    }
};

/*+++++++++++++++++ private data ++++++++++++++++++++++*/


/*
 * Private Declarations
 */

static PSP_Result ddc_ValidateParams(
                                 DDC_uartDevObj          *devHandle ,
                                 const PSP_uartDevParams *openParams
                                 );


static PAL_Result ddc_uartConfigure(
                                DDC_uartDevHandle         devHandle,
                                const PSP_uartDevParams  *openParams
                                );

static PAL_Result ddc_uartSubmitIoReq(
                                  DDC_uartIoObj      *ioHandle,
                                  PSP_uartRequest    *ioRequest
                                  );


static void  ddc_uartCancelAllIo(DDC_uartIoHandle ioHandle);

static void ddc_uartWriteByte(DDC_uartDevHandle devHandle, Char ch);

static Uint8 ddc_uartReadByte(DDC_uartDevHandle devHandle);

static Uint32 ddc_uartStatusIsDataReady(DDC_uartIoHandle ioHandle);

static void ddc_uartEnableHwFlowControl(
                               DDC_uartDevHandle           devHandle,
                               const PSP_uartFlowControl  *pFlowCtrl
                               );

static void ddc_uartIntrEnable(
                           DDC_uartDevHandle    devHandle,
                           DDC_UartRegister     bitmask
                           );

static void ddc_uartIntrDisable(
                            DDC_uartDevHandle   devHandle,
                            DDC_UartRegister    bitmask
                            );

static void ddc_uartSetBaudRate(DDC_uartDevHandle devHandle, Uint32 baudRate);

static void ddc_uartIsr(DDC_uartDevObj  *devHandle);

static PSP_Result ddc_uartGetHWInfo(DDC_uartDevHandle    devHandle);

static void ddc_uartHwSetup(
                        DDC_uartDevHandle          phwInfo,
                        const PSP_uartDevParams   *currentParams
                        );


static Uint32 ddc_uartFIFORead(
                           DDC_uartIoHandle       ioHandle,
                           Uint8                 *buffer,
                           Uint32                 bufSize
                           );

static Uint32 ddc_uartFIFOWrite(
                         DDC_uartIoHandle   ioHandle,
                         const Uint8       *buffer,
                         Uint32             nwrite
                         );

static void ddc_uartIntrHandler(DDC_uartIoHandle  ioHandle);

#ifdef PSP_UART_BUFFERING_ENABLED
static void ddc_uartBufferHandler(DDC_uartIoHandle  ioHandle);
static void ddc_uartBufferringTaskLet(DDC_uartIoHandle  ioHandle);
#endif

static void ddc_completeCurrentIO(DDC_uartIoHandle ioHandle);

static void ddc_copyIOPtoReq(
                         DDC_UartIoPacket const  *ioPacket,
                         PSP_uartRequest         *pspRequest
                         );

static void ddc_copyReqtoIOP(
                         PSP_uartRequest  const *pspRequest,
                         DDC_UartIoPacket       *ioPacket
                         );

static Uint32 ddc_uartXfer(
                       DDC_uartIoHandle       ioHandle,
                       Uint8                 *buffer,
                       Uint32                 numBytes
                       );


static PAL_Result ddc_uartCancelCurrentIo(
                               DDC_uartIoHandle    ioHandle,
                               DDC_UartIoPacket      *ioPacket
                               );

static PAL_Result ddc_uartHandleRxError(DDC_uartIoHandle  ioHandle);



#ifdef PSP_UART_EDMA_SUPPORT

static PAL_Result start_edma_transfer(DDC_uartIoHandle ioHandle, Uint32 cmd);
static void       isredma(Uint tcc, EDMA3_RM_TccStatus status, Ptr appData);

#endif



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Public Functions Definitions
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


/**
 * \brief    Creates/Sets up the UART device instance
 *
 *  This function is used to setup the UART hardware specified
 *  by the device ID.
 *
 * \param    instanceId    [IN]     Device instance number
 * \param    params        [IN]     Hardware initialization parameters
 * \param    arg           [IN]     application handle as arg.
 *
 * \return
 *           PSP_SOK         if success<br>
 *           PSP ERROR Codes if failure<br>
 */
PSP_Result PSP_uartCreate(
                      Uint32                    instanceId,
                      PSP_uartDevParams  const *params,
                      Ptr                       arg
                      )
{
    PAL_Result                 result           = PSP_E_INVAL_PARAM;
    DDC_uartDevObj            *devHandle        = NULL;
    PSP_uartDevParams   const *currentConfig    = NULL;

    PAL_osMemSet(&gDDC_uartDevObj[instanceId], 0x0, sizeof(DDC_uartDevObj));

    if (instanceId < PSP_UART_NUM_INSTANCES)
    {
        devHandle = &gDDC_uartDevObj[instanceId];

        if (( NULL  != devHandle) &&
                (PSP_DRIVER_STATE_DELETED == devHandle->devState))
        {
            if(NULL == params)
            {
                currentConfig = &guartDefaultConfig;
            }
            else
            {
                currentConfig = params;
            }

            devHandle->instanceId = instanceId;
            devHandle->inputFreq  = currentConfig->inputFreq;
            devHandle->fifoEnable = currentConfig->fifoEnable;
            /* Set Here for default operating mode: INTERRUPT */
            devHandle->opMode     = PSP_OPMODE_INTERRUPT;

            /* Validate device parameters */
            result = ddc_ValidateParams(devHandle , currentConfig);


            if(PSP_SOK ==  result)
            {
                /* Get UART HW memory map address and interrupt num for this
                 instance specified by instance ID */
                result = ddc_uartGetHWInfo(devHandle);
            }

            if(PSP_SOK ==  result)
            {
                /* Configure UART HW */
                result = ddc_uartConfigure(devHandle,currentConfig);
            }

        }/* uartDevObj verfication */
    }

    /* Checks for success*/
    if (PSP_SOK ==  result)
    {
        if( PSP_OPMODE_INTERRUPT == devHandle->opMode )
        {
            /* Register interrupts */
            result = DDA_uartRegisterInterrupt(
                                (Ptr)devHandle,
                                0,
                                (Uint16)devHandle->intrNum,
                                (pspUartIsr)&ddc_uartIsr
                                );
            /* Creating TaskLet(SWI)to schedule TX empty interrupt cause. Becz
               TX empty handler requires a delay wait for the transmitter to
               become empty in order to write 16 bytes into FIFO. This delay in
               UART HWI handler context shall cause interrupt latencies for
               other HWIs which are having less priority than UART HWI. Thus
               scheduling this TX processing in TaskLet(SWI) context for better
               system performance. */
            if( PSP_SOK == result)
            {
                Ptr *pTskLet = &devHandle->tskLetHandle;

#ifdef PSP_UART_BUFFERING_ENABLED
                *pTskLet = PAL_osCreateTaskLet(
                                DDC_UART_TASKLET_PRIORITY, /* TaskLet priority */
                                (PAL_osTaskLetFxn)&ddc_uartBufferringTaskLet,
                                (Ptr)&devHandle->txIO /* Tx IO Handle */
                                );
#else
                *pTskLet = PAL_osCreateTaskLet(
                                DDC_UART_TASKLET_PRIORITY,
                                (PAL_osTaskLetFxn)&ddc_uartIntrHandler,
                                (Ptr)&devHandle->txIO
                                );
#endif

            }

        }

        devHandle->appHandle    = arg;
        devHandle->devState     = PSP_DRIVER_STATE_CREATED;
    }

    return result;
}


/**
 * \brief    Delete/Un-initializes the UART device instance
 *
 * This function is used to do the reverse operation of
 * PSP_uartCreate. It uninitializes the UART HW for the given
 * devId.
 *
 * \param    instNum         [IN]    DevId of UART to be deleted.
 * \param    param           [IN]    additional param(Not used)
 *
 * \return
 *           PSP_SOK            if success<br>
 *           PSP_E_INVAL_PARAM  if invalid arguments<br>
 *
 */

/*  ARGSUSED */
PSP_Result  PSP_uartDelete(Uint32 instNum, Ptr param)
{

    DDC_uartDevObj *devHandle = NULL;
    PSP_Result      result    = PSP_E_INVAL_PARAM;
    Uint32          waitCnt   = 0x1000u;

    /* Error checking on params */
    if (instNum < PSP_UART_NUM_INSTANCES)
    {
        devHandle = &gDDC_uartDevObj[instNum];

        if( NULL  != devHandle)
        {
            /* Validate State */
            if ((devHandle->devState != PSP_DRIVER_STATE_CREATED) &&
                (devHandle->devState != PSP_DRIVER_STATE_CLOSED))
            {
                  result = PSP_E_INVAL_STATE;
            }
            else
            {
                /* Unregister interrupts */
                DDA_uartUnRegisterInterrupt(
                                         devHandle->instanceId,
                                         devHandle->intrNum
                                         );
                /* Deleting TaskLet handler */
                PAL_osDeleteTaskLet(devHandle->tskLetHandle);

                result = devHandle->regs->LSR;

                while( (0 == (result & DDC_UART_TRANSMITTER_EMPTY))
                       && (waitCnt != 0))
                {
                    result = DDC_uartGetLineStatus(devHandle);
                    waitCnt--;
                }

                result = PSP_SOK;
                devHandle->devState = PSP_DRIVER_STATE_DELETED;
            }
        }
    }

    return result;
}

/**
 *  \brief Open a read or write communication channel in UART device
 *
 *  This function prepares the device hardware for data transfers and
 *  usage by the upper layer driver software. The DDA layer is expected
 *  to install the ISR and enable it only after the completion of this
 *  call. The function prepares the driver instance for data transfers
 *  and returns an handle to the driver instance.
 *
 *  \param  instanceId    [IN]        Uart instance number
 *  \param  params        [IN/OUT]    Extra parameter (implementation specific)
 *  \param  mode          [IN]        Driver IO mode
 *  \param  drvCbk        [IN]        Driver callback
 *  \param  arg           [IN]        Argument for future use
 *
 *  \return PSP_Handle  Uart Driver Instance Handle
 */
 /*ARGSUSED*/
PSP_Handle PSP_uartOpen(
                    Uint32               instanceId,
                    PSP_UartIoMode       mode,
                    PSP_uartIoParams    *const params,
                    PSP_uartAppCallback  drvCbk,
                    Ptr                  arg
                   )
{
    PAL_Result          result        = PSP_SOK;
    DDC_uartDevObj     *devHandle     = NULL;
    DDC_uartIoObj      *ioHandle      = NULL;
    Uint32              i             = 0;
    PAL_OsSemAttrs      semAttrs      = PAL_OSSEM_ATTRS;

    if ((NULL != params) && (NULL != drvCbk))
    {
        /* Check instance parameter */
        if( PSP_UART_NUM_INSTANCES > instanceId )
        {
            devHandle = &gDDC_uartDevObj[instanceId];

            if( PSP_UART_OUTPUT == mode )
            {
                ioHandle = &devHandle->txIO;
            }
            else
            {
                ioHandle = &devHandle->rxIO;
            }

            if ( ((devHandle->devState != PSP_DRIVER_STATE_CREATED)
                  &&(devHandle->devState != PSP_DRIVER_STATE_CLOSED))
                 &&(devHandle->devState != PSP_DRIVER_STATE_OPENED) )
            {
                devHandle = (NULL);
                result = PSP_E_INVAL_STATE;
            }
            else
            {
#ifdef PSP_UART_EDMA_SUPPORT
                if (PSP_OPMODE_DMAINTERRUPT == devHandle->opMode )
                {
                    if( NULL != params->hEdma)
                    {
                        ioHandle->hEdma = params->hEdma;

                        /* Depending upon the instance update the channel number */
                        switch(devHandle->instanceId)
                        {
                         case 0:
                             if( PSP_UART_OUTPUT == mode) /* TX EDMA event config */
                             {
                                 ioHandle->Tcc  = CSL_EDMA3_CHA_UTXEVT0;
                                 ioHandle->ChId = CSL_EDMA3_CHA_UTXEVT0;
                             }
                             else /* RX EDMA event config */
                             {
                                 ioHandle->Tcc  = CSL_EDMA3_CHA_URXEVT0;
                                 ioHandle->ChId = CSL_EDMA3_CHA_URXEVT0;
                             }
                             break;


                         default:
                             result = PSP_E_INVAL_PARAM;
                             break;
                         }

                         result = EDMA3_DRV_requestChannel(
                                             (EDMA3_DRV_Handle)ioHandle->hEdma,
                                             &ioHandle->ChId,
                                             &ioHandle->Tcc,
                                             (EDMA3_RM_EventQueue)0,
                                             &isredma,
                                             (Ptr)ioHandle
                                             );

                         if(PAL_SOK != result)
                         {
                             EDMA3_DRV_freeChannel(
                                              devHandle->rxIO.hEdma,
                                              devHandle->rxIO.ChId
                                              );
                             result = PSP_E_INVAL_PARAM;
                         }
                     }
                     else /* if EDMA handle is NULL */
                     {
                        result = PSP_E_INVAL_PARAM;
                     }
                 }/* End of if opmode dma loop */
#endif /* PSP_UART_EDMA_SUPPORT */

             } /* End of state validation else loop */
        } /* End of if valid instance id check loop */
        else
        {
            result = PSP_E_INVAL_PARAM;
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }


    if( PSP_SOK == result)
    {
        if( TRUE == ioHandle->inUse )
        {
            ioHandle = (NULL);
            result = PSP_E_INVAL_STATE;
        }

        if(ioHandle != NULL)
        {
             /*
             * This semaphore will be used to pend to all TX requests are
             * done - during a flush request
             */
            if ( PAL_SOK != PAL_osSemCreate(
                                        NULL,
                                        0,
                                        &semAttrs,
                                        (PAL_OsSemHandle)&ioHandle->flushSyncSem
                                        ))
            {
                result = PSP_E_RESOURCES;

            }
            else
            {
                PAL_OSLIST_MKNODE(&ioHandle->queueFreeList);
                PAL_OSLIST_MKNODE(&ioHandle->queuePendingList);

                for (i = 0; i < PSP_UART_NUM_IOP; i++)
                {
                    PAL_osListEnqueue(
                                  &ioHandle->queueFreeList,
                                  (Ptr)(&ioHandle->iopPool[i])
                                 );
                }

                devHandle->devState    = PSP_DRIVER_STATE_OPENED;
                /* Update the Handle with the parameters passed */
                ioHandle->cbFxn        = drvCbk;
                ioHandle->cbArg        = arg;
                ioHandle->mode         = mode ;
                ioHandle->inUse        = TRUE;
                ioHandle->activeBuf    = NULL;
                ioHandle->activeIOP    = NULL;

                ioHandle->bytesRemaining = 0;

#ifdef PSP_UART_BUFFERING_ENABLED
                ioHandle->iBuffer.xferActual    = 0;
                ioHandle->iBuffer.bufferIndex   = 0;
                ioHandle->iBuffer.bufFillPtr    = ioHandle->iBuffer.buffer;
                ioHandle->iBuffer.bufReadPtr    = ioHandle->iBuffer.buffer;
#endif

                ioHandle->chunkSize      = DDC_UART_FIFO_SIZE;
                ioHandle->devHandle      = devHandle;
                ioHandle->fifoEnabled    = TRUE;
            }


        if( (PSP_OPMODE_POLLED != ioHandle->devHandle->opMode)
               && (PSP_UART_INPUT == ioHandle->mode ) )
        {
             /* Enabling Line Status Interrupt if receive channel
                is created, in order to process Line status error
                interrupts. */

             ddc_uartIntrEnable(
                            ioHandle->devHandle,
                            DDC_UART_INTR_RLS
                            );
            }
        }



    }
    return ((PSP_Handle)ioHandle);
}

/**
 *  \brief Close a channel of the UART device
 *
 *  This function closes the device for data transfers and usage by the
 *  upper layer driver software. The hardware is programmed to stop/abort data
 *  transfer (depending upon the type of device and its specifics) and the
 *  device ISR is "disabled" by the upper layer driver software after the
 *  completion of this call. After the successful completion of this call, the
 *  device cannot perform any data IO.
 *
 *  \param  handle   [IN]          UART channel handle
 *  \param  arg      [IN/OUT]      Extra parameter (implementation specific)
 *
 *  \return PSP_SOK or PSP Error code
 */
/*ARGSUSED*/
PSP_Result  PSP_uartClose(PSP_Handle handle,Ptr arg)
{
    DDC_uartIoObj  *ioHandle     = NULL;
    PSP_Result      retCode      = PSP_E_INVAL_PARAM;
    Uint32          cookie       = 0;

    /* Validate parameters */
    if (NULL != handle)
    {
        ioHandle = (DDC_uartIoObj *)handle;
        if(TRUE == PAL_osList_NP_IsEmpty(&ioHandle->queuePendingList))
		{
#ifdef PSP_UART_EDMA_SUPPORT
            /* Close EDMA Channel */
            if((PSP_OPMODE_DMAINTERRUPT == ioHandle->devHandle->opMode ))
            {
                EDMA3_DRV_freeChannel(
                              ioHandle->hEdma,
                              ioHandle->ChId
                              );
            }
#endif

            PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);
            /* Free this channel resource */
            ioHandle->inUse = FALSE;

            /* Disable UART interrupt causes */
            if( PSP_UART_INPUT == ioHandle->mode )
            {

                ddc_uartIntrDisable(
                            ioHandle->devHandle,
                            (DDC_UART_INTR_RLS | DDC_UART_INTR_RHR)
                            );
            }
            else
            {
                ddc_uartIntrDisable(
                            ioHandle->devHandle,
                            DDC_UART_INTR_THR
                            );

            }


            PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
            retCode = PSP_SOK;

            if( (ioHandle->devHandle->rxIO.inUse == FALSE)
                && (ioHandle->devHandle->txIO.inUse == FALSE) )
            {
                ioHandle->devHandle->devState = PSP_DRIVER_STATE_CLOSED;
            }
		}
		else /* Some IOPs are pending */
		{
		   retCode = PSP_E_INVAL_STATE;
		}
    }
    return (retCode);
}


/**
 *  \brief Read/Write data from UART channel
 *
 *  This function reads/writes data from the UART Instance.
 *
 *  \param  handle     [IN]     UART channel handle
 *  \param  ioRequest  [IN]     IO Packet Pointer
 *  \param  arg        [IN/OUT] Extra parameter (implementation specific)
 *
 *  \return
 *            PSP_SOK           if success<br>
 *            PAL_SINPROGRESS   if request is in progress<br>
 *            PSP Error Codes   if failure<br>
 */
/*  ARGSUSED */
PSP_Result PSP_uartTransfer(
                       PSP_Handle handle,
                       PSP_uartRequest *ioRequest,
                       Ptr arg
                       )
{
    DDC_uartIoHandle   ioHandle = NULL;
    PSP_Result         result   = PSP_SOK;

    /* Validate pointers */
    if ((NULL == handle)||(NULL== ioRequest))
    {
        result = PSP_E_INVAL_PARAM;
    }
    else
    {
        ioHandle = (DDC_uartIoHandle)handle;
    }

    /* validate mode */
    if((NULL == ioHandle->devHandle  ) || (ioRequest->timeout == 0) )
    {
        result = PSP_E_INVAL_PARAM;
    }
    else
    {
        if (result == PSP_SOK)
        {
            if (PSP_DRIVER_STATE_OPENED != ioHandle->devHandle->devState)
            {
                result = PSP_E_INVAL_STATE;
            }
            else
            {

#ifdef PSP_UART_EDMA_SUPPORT
               if(PSP_OPMODE_DMAINTERRUPT == ioHandle->devHandle->opMode)
               {
                   /* Check the buffer is aligned on Cache line length */
                   if(((Uint32)ioRequest->addr & (PAL_OS_CACHE_LINE_LEN-1)) != 0)
                   {

                       result = (PSP_E_INVAL_PARAM);

                   }
               }
#endif
               if( (PSP_UART_INPUT == ioHandle->mode) &&
                        (ioRequest->cmd == PSP_UART_IOCMD_READ) )
               {
                   /* Submit IOP to DDC */
                   result = ddc_uartSubmitIoReq(ioHandle, ioRequest);
               }
               else
               {
                    if((PSP_UART_OUTPUT == ioHandle->mode) &&
                        (ioRequest->cmd == PSP_UART_IOCMD_WRITE))
                    {
                         /* Submit IOP to DDC */
                         result = ddc_uartSubmitIoReq(ioHandle, ioRequest);
                    }
                    else
                    {
                        result = PSP_E_INVAL_PARAM;
                    }
               }
            }
        }
    }

    return (result);
}
/**
 *  \brief Abort pending requests
 *
 *    This API will abort or cancel all the pending requests in the UART
 *    TX or RX channel.
 *
 *  \param  handle   [IN]   Handle to the UART channel
 *
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_uartAbort(PSP_Handle handle)
{
    DDC_uartIoHandle      ioHandle  = NULL;
    PAL_Result            result    = PSP_SOK ;

    if( NULL != handle)
    {
        ioHandle    = (DDC_uartIoHandle )handle;

        if (TRUE == ioHandle->inUse)
        {
            ddc_uartCancelAllIo(ioHandle);
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    return (result);
}


/**
 *  \brief UART Ioctl
 *
 *  This function provides ioctl functionality for UART
 *
 *  \param  handle   [IN]       UART channel handle
 *  \param  cmd      [IN]       IOCTL command to be performed
 *  \param  cmdArg   [IN/OUT]   IOCTL command argument (if any)
 *  \param  param    [IN/OUT]   Device/Cmd specific argument
 *
 *  \return PSP_SOK or PSP Error code
 */
/*  ARGSUSED */
PSP_Result  PSP_uartIoctl(
                      PSP_Handle          handle,
                      PSP_UartIoctlCmd    cmd,
                      Ptr                 cmdArg,
                      Ptr                 param
                      )
{
    DDC_uartIoHandle      ioHandle  = NULL;
    DDC_uartDevHandle     devHandle = NULL;
    DDC_UartRegister      tempReg   = 0;
    PAL_Result            result    = PSP_SOK;
    Uint32                lineStatus= 0;

    /* Error checking of pointers */
    if ( NULL  == handle)
    {
        result = (PSP_E_INVAL_STATE);
    }
    else
    {
        ioHandle   = (DDC_uartIoHandle)handle;
        if (NULL != ioHandle->devHandle)
        {
            devHandle  = ioHandle->devHandle;

            /* validating state */
            if (PSP_DRIVER_STATE_OPENED != devHandle->devState)
            {
                  result = (PSP_E_INVAL_STATE);
            }
        }
        else
        {
            result = PSP_E_INVAL_PARAM;
        }
    }

    if(PAL_SOK == result )
    {

        /* Get the Line status of UART. UART ioctl commands to change baud rate,
           stop bit, data bit, parity, flow control and other clock related
           paramters should not be allowed when IO is on progress ie. TX FIFO
           is not empty */
        lineStatus = DDC_uartGetLineStatus(ioHandle->devHandle);

        switch(cmd)
        {
            case PSP_UART_IOCTL_SET_BAUD:

                /* If TX FIFO is empty, then process this command */
                if( (lineStatus & DDC_UART_TX_EMPTY) == DDC_UART_TX_EMPTY )
                {
                    PSP_UartBaudRate    baudRate = PSP_UART_BAUD_RATE_115_2K;

                    if (NULL != cmdArg)
                    {
                        /* Perform error checking on baud rate */
                        baudRate = *((PSP_UartBaudRate *)cmdArg);
                        switch(baudRate)
                        {
                            case PSP_UART_BAUD_RATE_2_4K:
                            case PSP_UART_BAUD_RATE_4_8K:
                            case PSP_UART_BAUD_RATE_9_6K:
                            case PSP_UART_BAUD_RATE_19_2K:
                            case PSP_UART_BAUD_RATE_38_4K:
                            case PSP_UART_BAUD_RATE_57_6K:
                            case PSP_UART_BAUD_RATE_115_2K:
                            /* Release Note: Due to the inaccuracies introduced
                               due to clock divide for baud rate selection,
                               higher baud rates may not work - we will set
                               this working threshold after testing on silicon
                             */
                                result = PSP_SOK;
                                break;
                            default:
                                result = PSP_E_INVAL_PARAM;
                                break;
                        }

                        if (PSP_SOK == result)
                        {
                            /* Setting the baud rate  */
                            ddc_uartSetBaudRate(
                                            devHandle,
                                            *(Uint32 *)cmdArg
                                            );

                        }
                    }
                    else
                    {
                        result = PSP_E_INVAL_PARAM;
                    }
                }
                else
                {
                    /* IO operation is going. Baud rate change should not be
                    entertained. */
                    result = PSP_E_INVAL_STATE;
                }
                break;


            case PSP_UART_IOCTL_SET_STOPBITS:

                /* If TX FIFO is empty, then process this command */
                if( (lineStatus & DDC_UART_TX_EMPTY) == DDC_UART_TX_EMPTY )
                {
                    PSP_UartNumStopBits    stopBits = PSP_UART_NUM_STOP_BITS_1;

                    if (cmdArg != NULL)
                    {
                        stopBits = *((PSP_UartNumStopBits *)cmdArg);
                        /* Perform error checking on stop bits */
                        switch (stopBits)
                        {
                            case PSP_UART_NUM_STOP_BITS_1:
                            case PSP_UART_NUM_STOP_BITS_1_5:
                            case PSP_UART_NUM_STOP_BITS_2:
                                break;
                            default:
                                result = (PSP_E_INVAL_PARAM);
                        }

                        if(PSP_SOK == result)
                        {
                            /* Setting the stop bits */
                            tempReg = devHandle->regs->LCR;
                            tempReg &= (~DDC_UART_STOP_BITS_MASK);
                            tempReg |= (Uint32) *(Int *)&stopBits;
                            devHandle->regs->LCR = tempReg;

                        }
                    }
                    else
                    {
                        result = PSP_E_INVAL_PARAM;
                    }
                }
                else
                {
                    /* IO operation is going. Stop bit change should not be
                    entertained. */
                    result = PSP_E_INVAL_STATE;
                }
                break;

            case PSP_UART_IOCTL_SET_DATABITS:

                /* If TX FIFO is empty, then process this command */
                if( (lineStatus & DDC_UART_TX_EMPTY) == DDC_UART_TX_EMPTY )
                {
                    PSP_UartCharLen    charLen  = PSP_UART_CHARLEN_8;

                    if (cmdArg != NULL)
                    {
                        charLen = *((PSP_UartCharLen *)cmdArg);
                        /* Perform error checking on character bits */
                        switch (charLen)
                        {
                            case PSP_UART_CHARLEN_5:
                            case PSP_UART_CHARLEN_6:
                            case PSP_UART_CHARLEN_7:
                            case PSP_UART_CHARLEN_8:
                                break;
                            default:
                                result = (PSP_E_INVAL_PARAM);
                        }

                        if(PSP_SOK == result)
                        {
                            /* Setting the character length */
                            tempReg = devHandle->regs->LCR;
                            tempReg &= (~DDC_UART_NUM_DATA_BITS_MASK);
                            tempReg |= (Uint32) *(Int *)&charLen;
                            devHandle->regs->LCR = tempReg;
                        }
                    }
                    else
                    {
                        result = PSP_E_INVAL_PARAM;
                    }

                }
                else
                {
                    /* IO operation is going. Data bit change should not be
                    entertained. */
                    result = PSP_E_INVAL_STATE;
                }
                break;

            case PSP_UART_IOCTL_SET_PARITY:

                /* If TX FIFO is empty, then process this command */
                if( (lineStatus & DDC_UART_TX_EMPTY) == DDC_UART_TX_EMPTY )
                {
                    PSP_UartParity          parity = PSP_UART_PARITY_NONE;

                    if (cmdArg != NULL)
                    {
                        parity = *((PSP_UartParity *)cmdArg);
                        /* Perform error checking on parity bits */
                        switch (parity)
                        {
                            case PSP_UART_PARITY_ODD:
                            case PSP_UART_PARITY_EVEN:
                            case PSP_UART_PARITY_NONE:
                                break;
                            default:
                                result = (PSP_E_INVAL_PARAM);
                                break;
                        }

                        if(PSP_SOK == result)
                        {
                            /* Setting the parity */
                            tempReg = devHandle->regs->LCR;
                            tempReg &= (~DDC_UART_PARITY_MASK);
                            tempReg |= (Uint32) *(Int *)&parity;
                            devHandle->regs->LCR = tempReg;
                        }
                    }
                    else
                    {
                        result = PSP_E_INVAL_PARAM;
                    }

                }
                else
                {
                    /* IO operation is going. Parity change should not be
                    entertained. */
                    result = PSP_E_INVAL_STATE;
                }
                break;

            case PSP_UART_IOCTL_SET_FLOWCONTROL:

                /* If TX FIFO is empty, then process this command */
                if( (lineStatus & DDC_UART_TX_EMPTY) == DDC_UART_TX_EMPTY )
                {
                    PSP_uartFlowControl     *fc = NULL;
                    fc = (PSP_uartFlowControl *)cmdArg;
                    if( NULL  != fc)
                    {
                       switch(fc->fcType)
                       {
                        case PSP_UART_FC_TYPE_HW:

                            /* Enable HW flow control */
                            ddc_uartEnableHwFlowControl(devHandle,fc);
                            break;

                        case PSP_UART_FC_TYPE_NONE:

                            /* Disable AFE bit */
                            CSL_FINST(devHandle->regs->MCR,UART_MCR_AFE,DISABLE);
                            break;

                        case PSP_UART_FC_TYPE_SW:
                        default:
                              result = PSP_E_NOT_SUPPORTED;
                        break;
                       }
                    }
                    else
                    {
                        result = PSP_E_INVAL_PARAM;
                    }

                }
                else
                {
                    /* IO operation is going. Flow control change should not be
                    entertained. */
                    result = PSP_E_INVAL_STATE;
                }
                break;

            case PSP_UART_IOCTL_SET_TRIGGER_LEVEL:

                /* If TX FIFO is empty, then process this command */
                if( (lineStatus & DDC_UART_TX_EMPTY) == DDC_UART_TX_EMPTY )
                {
                    Uint32            value    = 0;
                    PSP_UartRxTrigLvl trigLevel;

                    /* Validate params and FIFO Enable */
                    if((cmdArg != NULL)
                        && (TRUE == devHandle->fifoEnable ))
                    {
                        trigLevel = *((PSP_UartRxTrigLvl *)cmdArg);
                        switch (trigLevel)
                        {
                             case PSP_UART_RX_TRIG1:
                                 value |= 0x0;/* Rx FIFO trigger level 1 byte */
                                  break;
                             case PSP_UART_RX_TRIG4:
                                 value |= 0x40u;/* trigger level 4 byte */
                                 break;
                             case PSP_UART_RX_TRIG8:
                                 value |= 0x80u;/* trigger level 8 byte */
                                 break;
                             case PSP_UART_RX_TRIG14:
                                 value |= 0xC0u;/* trigger level 14 byte */
                                 break;
                             default:
                                 result = PSP_E_INVAL_PARAM;
                                 break;
                        }
                        if( 0 != value)/* If valid trigger level passed */
                        {

                          if( PSP_OPMODE_DMAINTERRUPT == devHandle->opMode )
                          {
                             /*  Set DMAMODE bit */
                             value |= CSL_UART_FCR_DMAMODE1_MASK;

                          }

                          /* Set FIFO enable bit */
                          value |= CSL_UART_FCR_FIFOEN_MASK;

                          /* Write into FCR */
                          devHandle->regs->FCR = value;

                        }
                    }
                    else
                    {
                        result = PSP_E_FIFO_NOT_ENABLED;
                    }
                }
                else
                {
                    /* IO operation is going. Trigger level change should not be
                    entertained. */
                    result = PSP_E_INVAL_STATE;
                }
                break;

            case PSP_UART_IOCTL_RESET_TX_FIFO:

                /* check if FIFO is supported and then Call to DDC layer */
                if( TRUE == devHandle->fifoEnable)
                {
                    Uint32   value    = 0;

                    if(PSP_UART_OUTPUT == ioHandle->mode)
                    {
                        /* Set TXFIFO Clear Bit */
                        value |= DDC_UART_TX_FIFO_RESET;

                        if( PSP_OPMODE_DMAINTERRUPT == devHandle->opMode )
                        {
                           /*  Set DMAMODE bit */
                           value |= CSL_UART_FCR_DMAMODE1_MASK;

                        }

                        /* Set FIFO enable bit */
                        value |= CSL_UART_FCR_FIFOEN_MASK;

                        /* Write into FCR */
                        devHandle->regs->FCR = value;
                    }
                    else
                    {
                        /* channel underprivileged to do this operation */
                        result = PSP_E_INVALID_MODE;
                    }
                }
                else
                {
                    result = PSP_E_FIFO_NOT_ENABLED;
                }
                break;

            case PSP_UART_IOCTL_RESET_RX_FIFO:

                /* check if FIFO is supported and then Call to DDC layer */
                if( TRUE == devHandle->fifoEnable)
                {
                    Uint32   value = 0;

                    if(PSP_UART_INPUT == ioHandle->mode)
                    {
                        /* Set RXFIFO Clear Bit */
                        value |= DDC_UART_RX_FIFO_RESET;

                        if( PSP_OPMODE_DMAINTERRUPT == devHandle->opMode )
                        {
                           /*  Set DMAMODE bit */
                           value |= CSL_UART_FCR_DMAMODE1_MASK;

                        }

                        /* Set FIFO enable bit */
                        value |= CSL_UART_FCR_FIFOEN_MASK;

                        /* Write into FCR */
                        devHandle->regs->FCR = value;


                    }
                    else
                    {
                        /* channel underprivileged to do this operation */
                        result = PSP_E_INVALID_MODE;
                    }
                }
                else
                {
                    result = PSP_E_FIFO_NOT_ENABLED;
                }
                break;

            /* Cancel the current IO on the channel */
            case PSP_UART_IOCTL_CANCEL_CURRENT_IO:
                ddc_uartCancelCurrentIo(ioHandle, NULL);
                break;

            case PSP_UART_IOCTL_GET_STATS:
                *(PSP_uartStats *)cmdArg = devHandle->stats;
                break;

            case PSP_UART_IOCTL_CLEAR_STATS:
                PAL_osMemSet(&devHandle->stats,0,sizeof(PSP_uartStats));
                break;

            default:
                /* Hey dude! you passed the invalid IOCTL cmd */
                result = PSP_E_INVALID_CMD;
                break;
        }
    }

    return (result);
}





/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Local  Functions Definitions
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



/**
 *  \brief    Validating Parameters passed to the Uart
 *
 *   This function is invoked from the DDC layer in order for the
 *   DDC to validate the various parameters passed to the Uart statically
 *   like baud Rate,Stop Bits,etc.
 *
 *  \param    devHandle    [IN]    Pointer to the device
 *  \param    openParams   [IN]    Parameters passed to the device
 *
 *  \return
 *            PSP_SOK         if success<br>
 *            PSP ERROR Codes if failure<br>
 */
/* ARGSUSED */
static PSP_Result ddc_ValidateParams(
                                 DDC_uartDevObj          *devHandle,
                                 const PSP_uartDevParams *openParams
                                 )
{

    PSP_Result    result = PSP_SOK;

    if((NULL != devHandle) && (NULL != openParams))
    {
        switch(openParams->opMode)
        {
         case PSP_OPMODE_POLLED:
         case PSP_OPMODE_INTERRUPT:
#ifdef PSP_UART_EDMA_SUPPORT
         case PSP_OPMODE_DMAINTERRUPT:
#endif
             devHandle->opMode = openParams->opMode;
             break;

         default:
#ifdef UART_DEBUG_PRINTF
             UART_DBG("\n\r<Error> UART[%d] Invalid Driver opMode",instNum);
#endif
             result =  (PSP_E_INVAL_PARAM);
             break;
        }

         /* Get the stop bits from user configuration */
        switch (openParams->stopBits)
        {
         case PSP_UART_NUM_STOP_BITS_1:
         case PSP_UART_NUM_STOP_BITS_1_5:
         case PSP_UART_NUM_STOP_BITS_2:
              break;

         default:

#ifdef UART_DEBUG_PRINTF
             UART_DBG("\n\r<Error> UART[%d] Invalid stop bit parameter",instNum);
#endif
             result =  (PSP_E_INVAL_PARAM);
             break;
        }

        /* Get the char len from user configuration */
        switch (openParams->charLen)
        {
         case PSP_UART_CHARLEN_5:
         case PSP_UART_CHARLEN_6:
         case PSP_UART_CHARLEN_7:
         case PSP_UART_CHARLEN_8:
              break;

         default:

#ifdef UART_DEBUG_PRINTF
             UART_DBG("\n\r<Error> UART[%d] Invalid Char Len parameter",instNum);
#endif
             result =  (PSP_E_INVAL_PARAM);
             break;
        }

        /* Get the parity from user configuration */
        switch (openParams->parity)
        {
         case PSP_UART_PARITY_ODD:
         case PSP_UART_PARITY_EVEN:
         case PSP_UART_PARITY_NONE:
              break;

         default:
#ifdef UART_DEBUG_PRINTF
              UART_DBG("\n\r<Error> UART[%d] Invalid Parity parameter",instNum);
#endif
              result =  (PSP_E_INVAL_PARAM);
              break;
        }

        /* Update baud rate */
        switch (openParams->baudRate)
        {
         case PSP_UART_BAUD_RATE_2_4K:
         case PSP_UART_BAUD_RATE_4_8K:
         case PSP_UART_BAUD_RATE_9_6K:
         case PSP_UART_BAUD_RATE_19_2K:
         case PSP_UART_BAUD_RATE_38_4K:
         case PSP_UART_BAUD_RATE_57_6K:
         case PSP_UART_BAUD_RATE_115_2K:
              break;

         default:
#ifdef UART_DEBUG_PRINTF
            UART_DBG("\n\r<Error> UART[%d] Invalid BaudRate parameter",instNum);
#endif
            result =  (PSP_E_INVAL_PARAM);
            break;
        }

        /* Update receive threshold */
        switch (openParams->rxThreshold)
        {
         case PSP_UART_RX_TRIG1 :
         case PSP_UART_RX_TRIG4 :
         case PSP_UART_RX_TRIG8 :
         case PSP_UART_RX_TRIG14:
              devHandle->rxThreshold = openParams->rxThreshold;
              break;

         default:
#ifdef UART_DEBUG_PRINTF
             UART_DBG("\n\r<Error>UART[%d] Invalid Trigger Level param",instNum);
#endif
             result =  (PSP_E_INVAL_PARAM);
             break;
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    return result;
}

/**
 *  \brief Read from RX FIFO
 *
 *  This function Reads from the RX FIFO into the iop buffer
 *
 *  \param  ioHandle       [IN]             Handle of the DDC Channel Object
 *  \param  buffer      [IN]             Buffer for the operation
 *  \param  bufSize     [IN]             No. of bytes left to be read
 *
 *  \return No. of bytes read
 *
 */
static Uint32 ddc_uartFIFORead(
                          DDC_uartIoHandle  ioHandle,
                          Uint8            *buffer,
                          Uint32            bufSize
                          )
{
    Uint32 size      = bufSize;
    Bool   isRxReady = FALSE;

    if( ( NULL  != ioHandle) && ( NULL  != buffer) )
    {
        isRxReady = ddc_uartStatusIsDataReady(ioHandle);

        while ((TRUE == isRxReady)&& ( 0 != bufSize))
        {
            /* writing to specified location by the user once the H/w is ready*/

           /* reading from the H/w*/
            *buffer = (Uint8)ddc_uartReadByte(ioHandle->devHandle);

            buffer++;
            --bufSize;

            isRxReady = ddc_uartStatusIsDataReady(ioHandle);
        }
    }
    return (size - bufSize);
}


/**
 *  \brief Write into TX FIFO
 *
 *  This function writes into TX FIFO from the iop buffer
 *
 *  \param  ioHandle      [IN]             Handle of the DDC Channel Object
 *  \param  buffer        [IN]             Buffer for the operation
 *  \param  nwrite        [IN]             No. of bytes left to be written
 *
 *  \return No. of bytes written
 *
 */
static Uint32 ddc_uartFIFOWrite(
                         DDC_uartIoHandle   ioHandle,
                         const Uint8       *buffer,
                         Uint32             nwrite
                         )
{
    Uint32    size          = nwrite;
    Uint32    lineStatus    = 0;
    Uint32    tempChunksize = 0;


    volatile  Int32 maxTrialCount = DDC_UART_TRANSMITEMPTY_TRIALCOUNT;

    if(( NULL != ioHandle) && (NULL != buffer))
    {
        /* Load the chunk size(fifo) */
        tempChunksize = ioHandle->chunkSize;

        /*
        Before we could write 'chunksize' no. of bytes, we should have
        'chunk size' no. of free buffers, (in this case max 16 bytes)
        hence we check for shiftregister empty (ensure the whole FIFO
        is empty) to write 16bytes
        */

        do
        {
            lineStatus = ioHandle->devHandle->regs->LSR;
            maxTrialCount--;

        }while ((DDC_UART_TX_EMPTY != (DDC_UART_TX_EMPTY & lineStatus))
                 && (0 < maxTrialCount));

        if( (0 != maxTrialCount)
                 && (DDC_UART_TX_EMPTY == (DDC_UART_TX_EMPTY & lineStatus)) )
        {
            while ( ((tempChunksize != 0)&&(nwrite != 0)) != 0)
            {
                /* Writing to the H/w */
                ddc_uartWriteByte(ioHandle->devHandle, *buffer);
                buffer++;
                nwrite--;
                tempChunksize--;
            }
        }
    }

    /* Returns the size actually written */
    return (size - nwrite);
}



/**
 *  \brief Cancels all iops in the tx/rx channel
 *
 *  This function cancels on the Channel.
 *
 *  \param  ioHandle     [IN]       Handle of the DDC Channel Object
 *
 *  \return If successful, returns PAL_SOK else PAL Error
 *
 */
static Void ddc_uartCancelAllIo(DDC_uartIoHandle ioHandle)
{
    Uint32              cookie     = 0;
    DDC_UartIoPacket   *tmpPacket  = NULL;
    PSP_uartRequest     pspRequest;

    PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

    /* validate params */
    if(NULL != ioHandle)
    {
        tmpPacket = ioHandle->activeIOP;
    }

    if (NULL != tmpPacket)
    {
        tmpPacket->status     = PSP_UART_E_IOP_ABORTED;
        tmpPacket->xferActual = tmpPacket->xferRequest - ioHandle->bytesRemaining;

        ioHandle->bytesRemaining = 0;

        if( PSP_UART_INPUT == ioHandle->mode )
        {
            ioHandle->devHandle->stats.rxBytes  += tmpPacket->xferActual;
        }
        else
        {
            ioHandle->devHandle->stats.txBytes  += tmpPacket->xferActual;
        }

        /* Copy iop data into Requested data */
        ddc_copyIOPtoReq(tmpPacket ,&pspRequest);

        /* Check for call back functions and call as needed */
        if(NULL != pspRequest.cbFxn)
        {
            (*pspRequest.cbFxn)(
                            (Ptr)ioHandle->devHandle->appHandle,
                            (Ptr)ioHandle->cbArg,
                            &pspRequest,
                            ioHandle->mode
                            );
        }
        else
        {
            /* Check for call back functions and call as needed */
            if (NULL !=  ioHandle->cbFxn)
            {
                (*ioHandle->cbFxn)(
                            (Ptr)ioHandle->devHandle->appHandle,
                            (Ptr)ioHandle->cbArg,
                            &pspRequest,
                            ioHandle->mode
                            );
            }
        }

         /*Empty the pendList queue. Common for DMA/Interrupt Transactions*/
        while (TRUE != PAL_osList_NP_IsEmpty(&ioHandle->queuePendingList))
        {

            tmpPacket = (DDC_UartIoPacket *)PAL_osList_NP_Dequeue(
                                                  &ioHandle->queuePendingList);

            tmpPacket->status = PSP_UART_E_IOP_ABORTED;

            /* Copy iop data into Requested data */
            ddc_copyIOPtoReq(tmpPacket ,&pspRequest);

            /* Check for call back functions and call as needed */
            if(NULL != pspRequest.cbFxn)
            {
                (*pspRequest.cbFxn)(
                                (Ptr)ioHandle->devHandle->appHandle,
                                (Ptr)ioHandle->cbArg,
                                &pspRequest,
                                ioHandle->mode
                                );
            }
            else
            {
                /* Check for call back functions and call as needed */
                if (NULL !=  ioHandle->cbFxn)
                {
                    (*ioHandle->cbFxn)(
                                (Ptr)ioHandle->devHandle->appHandle,
                                (Ptr)ioHandle->cbArg,
                                &pspRequest,
                                ioHandle->mode
                                );
                }

            }

        }

    }
    PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
}



/**
 *  \brief Configure UART device HW
 *
 *  This function is used to set up the hardware for the current settings
 * "param" points to the hardware config and other open params.
 *
 *  \param  devHandle     [IN]         Handle of the DDC  Object
 *  \param  openParams    [IN]         Parameters passed by the user
 *
 *  \return If successful, returns PAL_SOK else PAL Error
 *
 */
PAL_Result ddc_uartConfigure(
                         DDC_uartDevHandle           devHandle,
                         const PSP_uartDevParams    *openParams
                         )
{
    PAL_Result          result      = PAL_SOK;
    volatile Uint32     cnt         = 10000u;


    if(( NULL  != devHandle) && ( NULL  != openParams))
    {
            if(PSP_SOK == result )
            {
                /* Reset UART and setup hardware params */
                devHandle->regs->PWREMU_MGMT = 0;

                while (--cnt > 0)
                {
                    ;
                }

                /* Putting Transmitter/Receiver in unreset state.
                   Emulation FREE mode is set */
                devHandle->regs->PWREMU_MGMT = (0xE003u);

                ddc_uartHwSetup(devHandle, openParams);

                /* Disable all interrupts */
                ddc_uartIntrDisable(devHandle, 0xFFu);

                /* If the set parameters set the H/w Flowcontrol */
                if (PSP_UART_FC_TYPE_HW == openParams->fc.fcType )
                {
                   /* Enable HW flow control */
                   ddc_uartEnableHwFlowControl(devHandle, &openParams->fc);
                }

                devHandle->fifoEnable  = openParams->fifoEnable;

                /* Enable Loopback */
                if((Bool)TRUE == openParams->loopbackEnabled )
                {
                    DDC_UartRegister  tempReg;

                    tempReg = devHandle->regs->MCR;
                    tempReg &= (~DDC_UART_LOOPBACK_MASK);
                    tempReg |= DDC_UART_LOOPBACK_MASK;

                    devHandle->regs->MCR = tempReg;
                }
            }

        } /* Check for DDC Object */
        else
        {
            result = PSP_E_INVAL_PARAM;
        }


    return (result);
}


/**
 *  \brief Read/Write operation are performed
 *
 *  This function read/write function as requested by the application
 *  corresponding to iop packets as requeseted in all the modes i.e.
 *  POLLED/INTERRUPT.
 *
 *  \param  ioHandle  [IN]       Handle of the Channel
 *  \param  ioRequest [IN]       Packet passed by the upper layer
 *
 *  \return If successful, returns PAL_SOK else PAL Error
 *
 */
static PAL_Result ddc_uartSubmitIoReq(
                                  DDC_uartIoObj    *ioHandle,
                                  PSP_uartRequest    *ioRequest
                                  )
{
    Uint32              cookie         = 0;
    PAL_Result          result         = PAL_SOK;
    DDC_UartIoPacket   *ioPacket       = NULL;
    Uint32              currentTick    = 0;
    Bool                timeoutElapsed = FALSE;

    /* Validating inputs */
    if( (NULL != ioHandle) && (NULL != ioRequest) )
    {
        if( (PSP_UART_IOCMD_READ != ioRequest->cmd)
                && (PSP_UART_IOCMD_WRITE != ioRequest->cmd) )
        {
            result = PSP_E_INVAL_PARAM;
        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }


    if(PAL_SOK == result )
    {
        if(TRUE == PAL_osListIsEmpty (&ioHandle->queueFreeList))
        {
            /*
            all packets already exhausted
            */
            result = PSP_E_RESOURCES;
        }
        else
        {
#ifdef UART_INSTRUMENTATION_ENABLED
    if (ioHandle->mode == PSP_UART_OUTPUT)
    {
        PSP_LOG_EVENT(
                      &DVTEvent_Log,
                      "UART",
                      PSP_DVT_DESC(   PSP_DVT_eDATA_SND_START,
                                      PSP_DVT_dCHANNEL_ID_INST,
                                      PSP_DVT_dPACKET_ID,
                                      PSP_DVT_dNONE),
                      CHANNELID_INSTANCEID(
                           (PSP_UART_OUTPUT),
                           (ioHandle->devHandle->instanceId)),
                      ioRequest->addr);
    }
    else
    {
        PSP_LOG_EVENT(
                      &DVTEvent_Log,
                      "UART",
                      PSP_DVT_DESC(   PSP_DVT_eRCV_START,
                                      PSP_DVT_dCHANNEL_ID_INST,
                                      PSP_DVT_dPACKET_ID,
                                      PSP_DVT_dNONE),
                      CHANNELID_INSTANCEID(
                           (PSP_UART_INPUT),
                           (ioHandle->devHandle->instanceId)),
                      ioRequest->addr);
    }
#endif /* UART_INSTRUMENTATION_ENABLED */

            ioPacket = (DDC_UartIoPacket *)PAL_osListDequeue(
                                                     &ioHandle->queueFreeList);

            ddc_copyReqtoIOP(ioRequest ,ioPacket);

            /* If polled mode set the active iop as the received iop and call
             * the function "uartRxIntrHandler" where the control will block
             * till it gets the requested number of bytes. */
            if (PSP_OPMODE_POLLED == ioHandle->devHandle->opMode)
            {
                /*
                This submit does not allow a second request (from different
                task ) when we are in polled mode
                */
                if ( NULL  != ioHandle->activeIOP)
                {
                    ioPacket->status = PSP_UART_E_IOP_ERROR;
                    result           = PSP_UART_E_IOP_ERROR;
                }

                if(PAL_SOK == result)
                {

                    ioHandle->errors         = 0 ;
                    ioHandle->activeIOP      = ioPacket;
                    ioHandle->activeBuf      = ioPacket->buf;
                    ioHandle->bytesRemaining = ioPacket->xferRequest;

                    result = PSP_UART_E_REQ_TIMEOUT;

                    /* Update current tick value to perform timeout operation */
                    currentTick = PAL_osGetCurrentTick();

                    while((FALSE == timeoutElapsed) &&
                            (0 != ioHandle->bytesRemaining))
                    {
                        ddc_uartIntrHandler(ioHandle);

                        /* Check whether timeout happened or not */
                        timeoutElapsed = (Bool)PAL_osCheckTimeOut(
                                                              currentTick,
                                                              ioPacket->timeout
                                                              );

                        PAL_osWaitMsecs (DDC_UART_TPOLL_MSECS);
                    }

                    if((0 == ioHandle->bytesRemaining) && (0 == ioHandle->errors))
                    {
                        result = PSP_SOK;
                    }
                }
                if( (TRUE == timeoutElapsed) )
                {
                    ioPacket->status = result;

                    /* Perform the operation to complete the IO */
                    ddc_completeCurrentIO (ioHandle );
                }
            }
            else
            {
                PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

#ifdef PSP_UART_BUFFERING_ENABLED
                /*
                 * Implementation of buffering for Tx in Interrrupt mode
                 */
                if( (PSP_OPMODE_INTERRUPT == ioHandle->devHandle->opMode )
                   && (PSP_UART_OUTPUT == ioHandle->mode))
                {
                    /* If Pending list is not empty OR current transaction is
                       happening(activeIOP) OR request is more than the MAX
                       INTERNAL BUFFER SIZE, then queue it up in Pending list */
                    if(((TRUE != PAL_osListIsEmpty (&ioHandle->queuePendingList) )
                       ||(( NULL  != ioHandle->activeIOP)
                       || ( PSP_UART_MAX_BUFFER_SIZE < (ioHandle->iBuffer.bufferIndex + ioPacket->xferRequest))))
                    )
                    {
                        /* Suppose activeIOP is NULL and Internal Buffer is
                           empty, then update activeIOP with the current
                           ioPacket */
                        if( (NULL == ioHandle->activeIOP) &&
                            (0 == ioHandle->iBuffer.bufferIndex) )
                        {
                            ioHandle->activeIOP      = ioPacket;
                            ioHandle->activeBuf      = ioPacket->buf;
                            ioHandle->bytesRemaining = ioPacket->xferRequest;

                        }
                        else
                        {
                            /* Queue it up */
                            PAL_osListEnqueue(&ioHandle->queuePendingList, ioPacket);
                        }
                        /* Update the result */
                        ioPacket->status = PAL_SINPROGRESS;
                        result = PAL_SINPROGRESS;
                    }
                    else
                    {
                        /*
                        We are here because, this is the first packet
                        or buffer can accomodate further request
                        */
                        /* copy to internal buffer */
                        PAL_osMemCopy(
                                 ioHandle->iBuffer.bufFillPtr,
                                 ioPacket->buf,
                                 ioPacket->xferRequest
                                 );

                        ioHandle->iBuffer.bufferIndex += ioPacket->xferRequest;
                        ioHandle->iBuffer.bufFillPtr += ioPacket->xferRequest;

                        PAL_osListEnqueue(&ioHandle->queueFreeList, ioPacket);
                        /* iop status to OK(completed) */
                        result = PAL_SOK;
                    }

                    /* Enable Tx Holding Register Empty interrupt */
                    ddc_uartIntrEnable(
                                   ioHandle->devHandle,
                                   DDC_UART_INTR_THR
                                   );
                 }
                 else
#endif
                 {
                    /*
                     If not in polled mode, then it may be in INT mode or DMA mode.
                     Disable the interrupts.check if any IOP is active, if yes
                     check if the new IOP has a timeout value. If not then post an
                     error. else enque the iop into the list and return back.
                    */
                    if ( NULL  != ioHandle->activeIOP)
                    {
                        /* Queue it and update the result */
                        PAL_osListEnqueue(&ioHandle->queuePendingList, ioPacket);
                        ioPacket->status = PAL_SINPROGRESS;
                        result = PSP_SINPROGRESS;
                    }
                    else
                    {
                        /*
                        * No other IOP is active, We can process this IOP, set it as
                        * active IOP */
                        ioHandle->activeIOP      = ioPacket;
                        ioHandle->activeBuf      = ioPacket->buf;
                        ioHandle->bytesRemaining = ioPacket->xferRequest;
                        ioHandle->errors         = 0 ;

#ifdef PSP_UART_EDMA_SUPPORT
                        if(PSP_OPMODE_DMAINTERRUPT == ioHandle->devHandle->opMode )
                        {
                            /*
                            If DMA mode then start the DMA, ALSO ENSURE THAT
                            BUFFERING IS NOT DONE IN DMA MODE
                             */
                            PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

                            result = start_edma_transfer(ioHandle,(Uint32)ioPacket->cmd);

                            if  (PAL_SOK == result)
                            {
                                result = PSP_SINPROGRESS;
                            }
                        }
                        else
#endif
                        {

                            result  = PSP_SINPROGRESS;
                            /*
                            enable interrupt
                            */
                                if( PSP_UART_INPUT == ioHandle->mode )
                            {
                                ddc_uartIntrEnable(
                                               ioHandle->devHandle,
                                               DDC_UART_INTR_RHR
                                              );
                            }
                            else
                            {
                                ddc_uartIntrEnable(
                                               ioHandle->devHandle,
                                               DDC_UART_INTR_THR
                                              );
                            }
                        }
                    }/*end of buffer mode check*/
                }/* End of else loop */
                PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
            }
        }
    }

    return result;
}

/**
 *  \brief Perform read/write transfer
 *
 *  This function is called from ISR and Task context to do transfer
 *
 *  \param  ioHandle     [IN]     Handle of the DDC Channel Object
 *
 *  \return If successful, returns PSP_SOK else PSP Error
 *
 */
static void ddc_uartIntrHandler(
                DDC_uartIoHandle  ioHandle
                )
{
    Uint32                   xfer        = 0;
    DDC_UartIoPacket        *ioPacket   = NULL;


    if(NULL  != ioHandle)
    {
#ifdef UART_INSTRUMENTATION_ENABLED
        if (ioHandle->devHandle->opMode != PSP_OPMODE_POLLED)
        {
            PSP_LOG_EVENT(
                          &DVTEvent_Log,
                          "UART",
                          PSP_DVT_DESC(
                            PSP_DVT_eINT_START,
                            PSP_DVT_dINST,
                            PSP_DVT_dCHANNEL_ID,
                            PSP_DVT_dNONE),
                          ioHandle->devHandle->instanceId,
                          ioHandle->mode
                         );
        }
#endif /* UART_INSTRUMENTATION_ENABLED */

        ioPacket = ioHandle->activeIOP;

        /* If this is called from SWI(TaskLet) context */
        if( (PSP_UART_OUTPUT == ioHandle->mode)
            && (PSP_OPMODE_INTERRUPT == ioHandle->devHandle->opMode) )
        {
            PAL_osEvtDisable(ioHandle->devHandle->intrNum);
        }

        if ( NULL  != ioPacket)
        {
            xfer = ddc_uartXfer(
                            ioHandle,
                            ioHandle->activeBuf,
                            ioHandle->bytesRemaining
                           );

            ioHandle->bytesRemaining    -= xfer;
            ioHandle->activeBuf         += xfer;

            if( PSP_UART_INPUT == ioHandle->mode )
            {
                ioHandle->devHandle->stats.rxBytes  += xfer;
            }
            else
            {
                ioHandle->devHandle->stats.txBytes  += xfer;
            }

            if (0 == ioHandle->bytesRemaining)
            {
                /* Get next IOP from RX pending list - do this before
                * notifyCompletion is called as it will manipulate the
                * next and prev pointers of iop disturbing the pending list
                */


                ioPacket->xferActual   = ioPacket->xferRequest;

                if (0 != ioHandle->errors)
                {
                    ioPacket->status = PSP_UART_E_IO_ERROR;
                }
                else
                {
                    ioPacket->status = PAL_SOK;
                }

                /* Perform th operation to complete the IO */
                ddc_completeCurrentIO (ioHandle );

                if (PSP_OPMODE_INTERRUPT == ioHandle->devHandle->opMode )
                {
                    /*
                    Now that we have just completed the current IOP
                    we proceed to check if there are still packets pending in
                    pending queue
                    */
                    if( TRUE == PAL_osListIsEmpty(&ioHandle->queuePendingList))
                    {
                        /*
                        No more packets in queue
                        */
                        ioHandle->activeIOP = NULL;
                        if( PSP_UART_INPUT == ioHandle->mode )
                        {
                            ddc_uartIntrDisable(
                                           ioHandle->devHandle,
                                           DDC_UART_INTR_RHR
                                           );
                        }
                        else
                        {
                            ddc_uartIntrDisable(
                                           ioHandle->devHandle,
                                           DDC_UART_INTR_THR
                                           );
                        }
                    }
                    else
                    {
                        /*
                        we have atleast one packet
                        */
                        ioPacket = (DDC_UartIoPacket *)PAL_osListDequeue(
                                                     &ioHandle->queuePendingList);

                        /* validae and update the iop */
                        if( NULL  != ioPacket)
                        {
                            ioHandle->activeIOP      = ioPacket;
                            ioHandle->activeBuf      = ioPacket->buf;
                            ioHandle->bytesRemaining = ioPacket->xferRequest;
                        }
                    }
                }
            }/* for bytesRemaining ewuals to Zero */
        }/* iop NULL condition */
        else
        {
            if( PSP_UART_INPUT == ioHandle->mode )
            {
                ddc_uartIntrDisable(
                               ioHandle->devHandle,
                               DDC_UART_INTR_RHR
                              );
            }
            else
            {
                ddc_uartIntrDisable(
                               ioHandle->devHandle,
                               DDC_UART_INTR_THR
                              );
            }
        }

        /* If this is called from SWI(TaskLet) context */
        if( (PSP_UART_OUTPUT == ioHandle->mode)
            && (PSP_OPMODE_INTERRUPT == ioHandle->devHandle->opMode) )
        {
            PAL_osEvtEnable(ioHandle->devHandle->intrNum);
        }

     } /* IF loop ioHandle != NULL */

#ifdef UART_INSTRUMENTATION_ENABLED
    if (ioHandle->devHandle->opMode != PSP_OPMODE_POLLED)
    {
        PSP_LOG_EVENT(
                      &DVTEvent_Log,
                      "UART",
                      PSP_DVT_DESC(
                        PSP_DVT_eINT_END,
                        PSP_DVT_dINST,
                        PSP_DVT_dCHANNEL_ID,
                        PSP_DVT_dNONE),
                      ioHandle->devHandle->instanceId,
                      ioHandle->mode
                     );
    }
#endif /* UART_INSTRUMENTATION_ENABLED */

}

#ifdef PSP_UART_BUFFERING_ENABLED
/**
 *  \brief Perform transfer from UART TX internal buffer
 *         to UART HW FIFO

 *  \param  ioHandle  [IN]  Handle of the DDC Channel Object
 *
 *  \return If successful, returns PSP_SOK else PSP Error
 *
 */
static void ddc_uartBufferHandler(
                DDC_uartIoHandle  ioHandle
                )
{
    Uint32                   xfer        = 0;

    if(NULL  != ioHandle)
    {
        xfer = ddc_uartXfer(
                ioHandle,
                ioHandle->iBuffer.bufReadPtr,
                (ioHandle->iBuffer.bufferIndex-ioHandle->iBuffer.xferActual)
                );

         ioHandle->iBuffer.xferActual   += xfer;
         ioHandle->iBuffer.bufReadPtr += xfer;

         if( PSP_UART_INPUT == ioHandle->mode )
         {
             ioHandle->devHandle->stats.rxBytes  += xfer;
         }
         else
         {
             ioHandle->devHandle->stats.txBytes  += xfer;
         }
    }
}
#endif

/**
 *  \brief Cancels the current iop on the channel
 *
 *  Cancel IO function - cancels the current request only
 *    Used for cancel the request incase of Timeout
 *
 *  \param  ioHandle  [IN]    Handle of the Channel
 *  \param  ioPacket  [IN]    StreamIO Packet to be completed
 *
 *  \return If successful, returns PAL_SOK else PAL Error
 *
 */
PAL_Result ddc_uartCancelCurrentIo(
                               DDC_uartIoHandle       ioHandle,
                               DDC_UartIoPacket      *ioPacket
                               )
{

    Uint32     cookie = 0;
    PSP_Result result = PSP_SOK;

    if(NULL != ioHandle)
    {
        /* Protect from UART interrupt and task switching so that
         * the process is cancellation is fast */
        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

        /* Check the io packet */
        if( NULL == ioPacket)
        {
            ioPacket = ioHandle->activeIOP;
        }

        /* if iop passed is null the update iop with active iop which is to
         * be cancelled */
        if(ioHandle->activeIOP == ioPacket)
        {
            /* Current IOP */
            if( NULL != ioHandle->activeIOP)
            {

                ioPacket->xferActual =
                       (ioPacket->xferRequest - ioHandle->bytesRemaining);
                ioHandle->bytesRemaining = 0;

                if( PSP_UART_INPUT == ioHandle->mode )
                {
                    ioHandle->devHandle->stats.rxBytes += ioPacket->xferActual;
                }
                else
                {
                    ioHandle->devHandle->stats.txBytes += ioPacket->xferActual;
                }

                ioPacket->status = PSP_E_TIMEOUT;

                ddc_completeCurrentIO(ioHandle);


                if(FALSE == PAL_osListIsEmpty(&ioHandle->queuePendingList))
                {
                    /*
                    * The last Active IOP got completed, so need
                    * to modify this pointer
                    */
                    ioPacket = (DDC_UartIoPacket *)PAL_osListDequeue(
                                                  &ioHandle->queuePendingList);

                    ioHandle->activeIOP      = ioPacket;
                    ioHandle->activeBuf      = ioPacket->buf;
                    ioHandle->bytesRemaining = (Int)ioPacket->xferRequest;
                }
                else
                {
                    ioHandle->activeIOP  = NULL;
                    if( PSP_UART_INPUT == ioHandle->mode )
                    {
                        ddc_uartIntrDisable(
                                       ioHandle->devHandle,
                                       DDC_UART_INTR_RHR
                                       );
                    }
                    else
                    {
                        ddc_uartIntrDisable(
                                       ioHandle->devHandle,
                                       DDC_UART_INTR_THR
                                       );
                    }
                }
            }
        }
        else
        {
            if( NULL != ioHandle->activeIOP)
            {
                PAL_osList_NP_Remove((Ptr)ioPacket);
            }
        }

        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    return result;
}


/**
 *  \brief UART ISR
 *
 *  This function is the ISR for the entire UART device. This routine further
 *  process the exact cause of interrupt by reading IIR
 *
 *  \param  devHandle     [IN]       Handle of the DDC Object
 *
 *  \return If successful, returns PAL_SOK else PAL Error
 *
 */
static void ddc_uartIsr(DDC_uartDevObj  *devHandle)
{

    volatile Uint32  status     = 0x01u;
    Uint32           intrCnt    = 0;
    DDC_uartIoHandle ioHandle   = NULL;


    if( NULL != devHandle )
    {
        status = (Uint32)DDC_uartIntrGetStatus(devHandle);

        while( (FALSE == (status & 0x1u)) &&
               ((intrCnt) < (DDC_UART_MAX_ISR_LOOP)) )
        {
            intrCnt++;

            /*  check for any line error while transfer */
            if (TRUE == (Bool)DDC_uartIntrIsReceiverLineError(status) )
            {
                /* call this function when ther is any line error  */
                ddc_uartHandleRxError (&(devHandle->rxIO));
            }
            else
            {
                /* check whether any character timeout happened */
                if(( 0x1u == DDC_uartIntrIsDataReceived(status))
                   ||(0x1u == DDC_uartIntrIsCharacterTimeout(status)))
                {

                    if(TRUE == (Bool)DDC_uartIntrIsCharacterTimeout(status))
                    {
                        /* if timeout error then update stats */
                        devHandle->stats.rxTimeout++;
                    }
                    ioHandle = &devHandle->rxIO;

                    ddc_uartIntrHandler(ioHandle);
                }
                else
                {

                    /* check whether Tx register is empty or not */
                     if(TRUE == (Bool)DDC_uartIntrIsXmitterShiftRegEmpty(status))
                     {
                        ioHandle = &devHandle->txIO;
                        PAL_osPostTaskLet(devHandle->tskLetHandle);
                     }
                     else
                     {
                        /*check modem status*/
                        if(TRUE == (Bool)DDC_uartIntrIsModemStatus(status))
                        {
                            /* Read the MSR to clear Modem Status Int cause */
                            devHandle->regs->SCR = devHandle->regs->MSR;
                        }
                     }
                }
            }

            status = DDC_uartIntrGetStatus(devHandle);
        }
    }
}

#ifdef PSP_UART_BUFFERING_ENABLED
static void ddc_uartBufferringTaskLet(DDC_uartIoHandle  ioHandle)
{

    DDC_UartIoPacket *ioPacket  = NULL;

    if(NULL != ioHandle)
    {
        /* Now we are in SWI context. Disable UART HWI inorder to avoid
        mutiple SWI posting from UART ISR. Since UART TX empty will be
        triggered if atleast 1 byte is free in TX FIFO.*/

        PAL_osEvtDisable(ioHandle->devHandle->intrNum);

        if( ioHandle->iBuffer.bufferIndex > ioHandle->iBuffer.xferActual )
        {
            ddc_uartBufferHandler(ioHandle);

            /* if bytes to be transferred from buffer is 0 */
            if (ioHandle->iBuffer.bufferIndex == ioHandle->iBuffer.xferActual)
            {
                /* Reset buffer pointers */
                ioHandle->iBuffer.bufFillPtr = ioHandle->iBuffer.buffer;
                ioHandle->iBuffer.bufReadPtr = ioHandle->iBuffer.buffer;

                ioHandle->iBuffer.bufferIndex = 0;
                ioHandle->iBuffer.xferActual  = 0;


                /* Is pending list not empty */
                if(FALSE == PAL_osListIsEmpty(&ioHandle->queuePendingList))
                {
                    ioPacket = (DDC_UartIoPacket *)PAL_osListDequeue(
                                                 &ioHandle->queuePendingList);

                    /* validate and update the iop */
                    if( NULL  != ioPacket)
                    {
                        ioHandle->activeIOP      = ioPacket;
                        ioHandle->activeBuf      = ioPacket->buf;
                        ioHandle->bytesRemaining = ioPacket->xferRequest;
                    }
                 }
                 else /* Pending list empty. No more data to transfer */
                 {
                     /* Disable THR interupt */
                     ddc_uartIntrDisable(
                              ioHandle->devHandle,
                              DDC_UART_INTR_THR
                              );

                 }
            }
        }
        else
        {
            ddc_uartIntrHandler(ioHandle);
        }

        /* Enable UART HWI event */
        PAL_osEvtEnable(ioHandle->devHandle->intrNum);

    }

}
#endif

/**
 *  \brief Receiver Errors
 *
 *  This function finds errors on the Receiver buffer.
 *
 *  \param  ioHandle     [IN]   Handle of the DDC Channel Object
 *
 *  \return If successful, returns PAL_SOK else PAL Error
 *
 */
static PAL_Result ddc_uartHandleRxError (DDC_uartIoHandle  ioHandle)
{

    Uint32        status    = 0;
    Uint32        intStatus = 0;
    Uint32        iteration = 0;
    Uint8         tempByte  = 0;
    PAL_Result    result    = PAL_SOK;

    if(NULL != ioHandle)
    {
       /* check for the errors and if there is any error update
         the corresponding stats */
        status = DDC_uartGetLineStatus(ioHandle->devHandle);

        if( (Bool)(status & CSL_UART_LSR_OE_MASK) != FALSE)
        {
            ioHandle->devHandle->stats.overrun++;
            ioHandle->errors++;

#ifdef UART_DEBUG_PRINTF
            UART_DBG("\nError: Receive Line Error - FIFO Overflow");
#endif
        }

        if( (Bool)(status & CSL_UART_LSR_RXFIFOE_MASK  ) != FALSE )
        {
            if( (status & CSL_UART_LSR_FE_MASK) != FALSE )
            {
                ioHandle->devHandle->stats.rxFramingError++ ;
                ioHandle->errors++;
#ifdef UART_DEBUG_PRINTF
                UART_DBG("\nError: Receive Line Error - Framing Error");
#endif
            }

            if( (status & CSL_UART_LSR_PE_MASK) != FALSE )
            {
                ioHandle->devHandle->stats.rxParityError++ ;
                ioHandle->errors++;
#ifdef UART_DEBUG_PRINTF
                UART_DBG("\nError: Receive Line Error - Parity Error");
#endif
            }

            if( (status & CSL_UART_LSR_BI_MASK) != FALSE )
            {
                ioHandle->devHandle->stats.rxBreakError++ ;
                ioHandle->errors++;
#ifdef UART_DEBUG_PRINTF
                UART_DBG("\nError: Receive Line Error - Break Indicator");
#endif
            }

            /* Clearing Receive Errors(FE,BI,PE) by reading the
               erroneous data from RX FIFO */
            intStatus = (Uint32)DDC_uartIntrGetStatus(ioHandle->devHandle);

            iteration = 16; /* Iteration count: Worst case = FIFO size(16)*/
            while((intStatus == 0x6u) && (iteration != 0) )
            {
                /* Read and throw error byte */
                /* Till Line status int is pending */
                tempByte = (Uint8)ioHandle->devHandle->regs->RBR;

                tempByte = tempByte;

                iteration--;

                intStatus = (Uint32)DDC_uartIntrGetStatus(ioHandle->devHandle);
            }
        }

        result = (PAL_Result)PAL_ERROR(PAL_CRITICAL_ERROR, PAL_ERROR_SRC_DRV, 0, 0);
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    return result;
}



#ifdef PSP_UART_EDMA_SUPPORT

/**
 *  \brief Edma transfer
 *
 *  This function initiates Edma mode of operation
 *
 *  \note  As this function is also executed in interrupt context
 *        validation of the result is not done. Once this is tested
 *        there  should be no issues, unless DMA is changed
 *
 *
 *  \param  ioHandle  [IN]     Handle of the DDC Object
 *  \param  cmd       [IN]     Read\Write cmd
 *
 *  \return If successful, returns PAL_SOK else PAL Error
 *
 */
static PAL_Result start_edma_transfer(DDC_uartIoHandle ioHandle, Uint32 cmd)
{
    PAL_Result          result = PAL_SOK;

    EDMA3_DRV_Handle    hEdma  = NULL;

    if(NULL != ioHandle)
    {
       hEdma = (EDMA3_DRV_Handle )ioHandle->hEdma;
    }

    if(NULL != hEdma)
    {
        if(cmd == PSP_UART_IOCMD_READ)
        {
            /* Move the data to physical memory and invalidate the cache line */
            PAL_osCacheFlushAndInvalidate(
                                      PAL_osCache_NOT_DEFINED,
                                      (Uint32)(ioHandle->activeBuf),
                                      (Uint32)ioHandle->bytesRemaining
                                      );

            if( PAL_SOK != EDMA3_DRV_setSrcParams(
                                    (EDMA3_DRV_Handle)hEdma,
                                    ioHandle->ChId,
                                    (Uint32)(&(ioHandle->devHandle->regs->RBR)),
                                    EDMA3_DRV_ADDR_MODE_FIFO,
                                    EDMA3_DRV_W8BIT
                                    ) )
            {
                result = PSP_E_INVAL_PARAM;
            }
            else if(PAL_SOK != EDMA3_DRV_setDestParams((EDMA3_DRV_Handle)hEdma,
                                           (Uint)ioHandle->ChId,
                                           (Uint)(ioHandle->activeBuf),
                                           EDMA3_DRV_ADDR_MODE_INCR,
                                           EDMA3_DRV_W8BIT) )
            {
                result = PSP_E_INVAL_PARAM;
            }

            else if( PAL_SOK != EDMA3_DRV_setSrcIndex((EDMA3_DRV_Handle)hEdma,
                                ioHandle->ChId,
                                0,
                                0) )
            {
                result = PSP_E_INVAL_PARAM;
            }

            else if( PAL_SOK != EDMA3_DRV_setDestIndex((EDMA3_DRV_Handle)hEdma,
                                                        ioHandle->ChId,
                                                        1,
                                                        0) )
            {
                result = PSP_E_INVAL_PARAM;
            }

            else if( PAL_SOK != EDMA3_DRV_setTransferParams(
                                               (EDMA3_DRV_Handle)hEdma,
                                               ioHandle->ChId,
                                               1u,
                                               ioHandle->bytesRemaining,
                                               1u,
                                               0,
                                               EDMA3_DRV_SYNC_A) )
            {
                result = PSP_E_INVAL_PARAM;
            }

            else
            {
                if( PAL_SOK != EDMA3_DRV_setOptField(hEdma,
                                                    ioHandle->ChId,
                                                    EDMA3_DRV_OPT_FIELD_TCINTEN,
                                                    1u) )
                {
                    result = PSP_E_INVAL_PARAM;
                }
            }

            if(result == PAL_SOK)
            {
                Uint32   value = 0;

                switch (ioHandle->devHandle->rxThreshold)
                {
                 case PSP_UART_RX_TRIG1:
                    value |= 0x0;   /* Receive FIFO trigger level 1 byte */
                    break;
                 case PSP_UART_RX_TRIG4:
                    value |= 0x40u;  /* Receive FIFO trigger level 4 byte */
                    break;
                 case PSP_UART_RX_TRIG8:
                    value |= 0x80u;  /* Receive FIFO trigger level 8 byte */
                    break;
                 case PSP_UART_RX_TRIG14:
                    value |= 0xC0u;  /* Receive FIFO trigger level 14 byte */
                    break;
                 default: /* Cant be default. Becz already this param is validated */
                    break;
               }
                /*Enable DMA Request*/
                value |= CSL_UART_FCR_DMAMODE1_MASK;
                value |= CSL_UART_FCR_FIFOEN_MASK;

                /* Write into FCR */
                ioHandle->devHandle->regs->FCR = value;

                if( PAL_SOK != EDMA3_DRV_enableTransfer (
                                                    (EDMA3_DRV_Handle)hEdma,
                                                    ioHandle->ChId,
                                                    EDMA3_DRV_TRIG_MODE_EVENT) )
                {
                    result = PSP_E_INVAL_PARAM;
                }
            }
        }
        else
        {
            if(cmd == PSP_UART_IOCMD_WRITE)
            {
                PAL_osCacheFlush(PAL_osCache_NOT_DEFINED,
                                 (Uint32)(ioHandle->activeBuf),
                                 ioHandle->bytesRemaining);

                if( PAL_SOK != EDMA3_DRV_setSrcParams((EDMA3_DRV_Handle)hEdma,
                                ioHandle->ChId,
                                (Uint32)(ioHandle->activeBuf),
                                EDMA3_DRV_ADDR_MODE_INCR,
                                EDMA3_DRV_W8BIT) )
                {
                    result = PSP_E_INVAL_PARAM;
                }

                else if( PAL_SOK != EDMA3_DRV_setDestParams(
                                       (EDMA3_DRV_Handle)hEdma,
                                       ioHandle->ChId,
                                       (Uint32)(&(ioHandle->devHandle->regs->THR)),
                                       EDMA3_DRV_ADDR_MODE_FIFO,
                                       EDMA3_DRV_W8BIT) )
                {
                    result = PSP_E_INVAL_PARAM;
                }

                else if( PAL_SOK != EDMA3_DRV_setSrcIndex(
                                                (EDMA3_DRV_Handle)hEdma,
                                                ioHandle->ChId,
                                                1,
                                                0) )
                {
                    result = PSP_E_INVAL_PARAM;
                }

                else if( PAL_SOK != EDMA3_DRV_setDestIndex(
                                                (EDMA3_DRV_Handle)hEdma,
                                                ioHandle->ChId,
                                                0,
                                                0) )
                {
                    result = PSP_E_INVAL_PARAM;
                }

                else if( PAL_SOK != EDMA3_DRV_setTransferParams(
                                                  (EDMA3_DRV_Handle)hEdma,
                                                   ioHandle->ChId,
                                                   1u,
                                                   ioHandle->bytesRemaining,
                                                   1u,
                                                   0,
                                                   EDMA3_DRV_SYNC_A) )
                {
                    result = PSP_E_INVAL_PARAM;
                }

                else if( PAL_SOK != EDMA3_DRV_setOptField (hEdma,
                                                           ioHandle->ChId,
                                            EDMA3_DRV_OPT_FIELD_TCINTEN, 1u) )
                {
                    result = PSP_E_INVAL_PARAM;
                }
                else
                {
                    Uint32   value = 0;

                    switch (ioHandle->devHandle->rxThreshold)
                    {
                     case PSP_UART_RX_TRIG1:
                        value |= 0x0;   /* Receive FIFO trigger level 1 byte */
                        break;
                     case PSP_UART_RX_TRIG4:
                        value |= 0x40u;  /* Receive FIFO trigger level 4 byte */
                        break;
                     case PSP_UART_RX_TRIG8:
                        value |= 0x80u;  /* Receive FIFO trigger level 8 byte */
                        break;
                     case PSP_UART_RX_TRIG14:
                        value |= 0xC0u;  /* Receive FIFO trigger level 14 byte */
                        break;
                     default: /* Cant be default. Becz already this param is validated */
                        break;
                   }
                    /*Enable DMA Request*/
                    value |= CSL_UART_FCR_DMAMODE1_MASK;
                    value |= CSL_UART_FCR_FIFOEN_MASK;

                    /* Write into FCR */
                    ioHandle->devHandle->regs->FCR = value;


                    if( PAL_SOK != EDMA3_DRV_enableTransfer (
                                                    (EDMA3_DRV_Handle)hEdma,
                                                    ioHandle->ChId,
                                                    EDMA3_DRV_TRIG_MODE_EVENT) )
                    {
                        result = PSP_E_INVAL_PARAM;
                    }
                }
            }/* End of write */
            else
            {
                result = PSP_E_INVALID_CMD;
            }
        }/* End of else */
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    return result;
}


/**
 *  \brief Callback for the receive
 *
 *  This function is the callback for the operation is complete
 *
 *  \param  tcc     [IN]            Handle of the Edma Object
 *  \param  status  [IN]            Error code returned by the Edma
 *  \param  appData [IN]            Data passed Edma during initiation
 *
 *  \return If successful, returns PAL_SOK else PAL Error
 *
 */
/* ARGSUSED */
static void isredma(Uint tcc, EDMA3_RM_TccStatus status, Ptr appData)
{
    PAL_Result         result     = PAL_SOK;
    DDC_uartIoHandle   ioHandle   = NULL;
    Uint               paRAMField = 0;
    Uint32             value = 0;
 

    if( NULL  != appData)
    {
        ioHandle = (DDC_uartIoHandle) appData;

#ifdef UART_INSTRUMENTATION_ENABLED
    if (ioHandle->devHandle->opMode != PSP_OPMODE_POLLED)
    {
        PSP_LOG_EVENT(
                      &DVTEvent_Log,
                      "UART",
                      PSP_DVT_DESC(
                        PSP_DVT_eINT_START,
                        PSP_DVT_dINST,
                        PSP_DVT_dCHANNEL_ID,
                        PSP_DVT_dNONE),
                      ioHandle->devHandle->instanceId,
                      ioHandle->mode
                     );
    }
#endif /* UART_INSTRUMENTATION_ENABLED */


        if(ioHandle->devHandle->fifoEnable == TRUE)
        {
            value |= CSL_UART_FCR_FIFOEN_MASK;
        }
     

        switch (ioHandle->devHandle->rxThreshold)
        {
         case PSP_UART_RX_TRIG1:
            value |= 0x0;   /* Receive FIFO trigger level 1 byte */
            break;
         case PSP_UART_RX_TRIG4:
            value |= 0x40u;  /* Receive FIFO trigger level 4 byte */
            break;
         case PSP_UART_RX_TRIG8:
            value |= 0x80u;  /* Receive FIFO trigger level 8 byte */
            break;
         case PSP_UART_RX_TRIG14:
            value |= 0xC0u;  /* Receive FIFO trigger level 14 byte */
            break;
         default: /* Cant be default. Becz already this param is validated */
            break;
       }

        /* Write into FCR (Diable DMAMODE. DMAMODE1 bit should be ZERO */
        ioHandle->devHandle->regs->FCR = value;

        EDMA3_DRV_disableTransfer (
                        ioHandle->hEdma,
                        ioHandle->ChId,
                        EDMA3_DRV_TRIG_MODE_EVENT
                        );
        if( PAL_SOK != EDMA3_DRV_setOptField (
                                        ioHandle->hEdma,
                                        ioHandle->ChId,
                                        EDMA3_DRV_OPT_FIELD_TCC,
                                        ioHandle->Tcc
                                        )
          )
        {
            result = PSP_E_INVAL_PARAM;
        }
        else
        {
            if ( PAL_SOK != EDMA3_DRV_getPaRAMEntry (
                                            ioHandle->hEdma,
                                            ioHandle->ChId,
                                            EDMA3_DRV_PARAM_ENTRY_LINK_BCNTRLD,
                                            &paRAMField
                                            )
               )
            {
                result = PSP_E_INVAL_PARAM;
            }
            else
            {
                paRAMField |= 0xFFFFu;

                if( PAL_SOK != EDMA3_DRV_setPaRAMEntry (
                                    ioHandle->hEdma,
                                    ioHandle->ChId,
                                    EDMA3_DRV_PARAM_ENTRY_LINK_BCNTRLD,
                                    paRAMField
                                    )
                  )
                {
                    result = PSP_E_INVAL_PARAM;
                }
            }
        }

        if(EDMA3_RM_XFER_COMPLETE != status)
        {
            result = (PAL_Result)PSP_UART_E_DMA_TRANSFER_FAIL;
            ioHandle->activeIOP->status = result;
            ddc_completeCurrentIO (ioHandle );
        }
        else
        {

            if( PSP_UART_INPUT == ioHandle->mode )
            {
                ioHandle->devHandle->stats.rxBytes += ioHandle->bytesRemaining;
            }
            else
            {
                ioHandle->devHandle->stats.txBytes += ioHandle->bytesRemaining;
            }

            result = PAL_SOK;
            ioHandle->activeIOP->status = result;
            ddc_completeCurrentIO (ioHandle );
        }

        if( (Bool)TRUE == (PAL_osListIsEmpty(&ioHandle->queuePendingList))   )
        {
            ioHandle->activeIOP = NULL;
        }
        else
        {
            DDC_UartIoPacket *iop = NULL;
            iop = (DDC_UartIoPacket *)PAL_osListDequeue(
                                                &ioHandle->queuePendingList
                                                );
            if( NULL  != iop)
            {
                ioHandle->activeIOP = iop;
                ioHandle->activeBuf = iop->buf;
                ioHandle->bytesRemaining = (Int)iop->xferRequest;
                start_edma_transfer(ioHandle,iop->cmd);
            }
        }
    }
#ifdef UART_INSTRUMENTATION_ENABLED
    if (ioHandle->devHandle->opMode != PSP_OPMODE_POLLED)
    {
        PSP_LOG_EVENT(
                      &DVTEvent_Log,
                      "UART",
                      PSP_DVT_DESC(
                        PSP_DVT_eINT_END,
                        PSP_DVT_dINST,
                        PSP_DVT_dCHANNEL_ID,
                        PSP_DVT_dNONE),
                      ioHandle->devHandle->instanceId,
                      ioHandle->mode
                     );
    }
#endif /* UART_INSTRUMENTATION_ENABLED */
}

#endif

/**
 * \brief  To get UART device Register base address and interrupt number.
 *
 * \param   devHandle   [IN]     Handle to the DDC Object
 *
 * \return Void
 */
static PSP_Result ddc_uartGetHWInfo(DDC_uartDevHandle    devHandle)
{

    PSP_Result  result = PSP_SOK;   

   if ( NULL != devHandle )
   {
        /* Set the base address and interrupt number */
        switch (devHandle->instanceId)
        {
            case 0:
                devHandle->intrNum = CSL_INTC_EVENTID_UARTINT0;
                devHandle->regs    = (CSL_UartRegsOvly)CSL_UART_0_REGS;
                break;

#if defined(CHIP_DM6437) || defined(CHIP_C6424)
            case 1:
                devHandle->intrNum = CSL_INTC_EVENTID_UARTINT1;
                devHandle->regs    = (CSL_UartRegsOvly)CSL_UART_1_REGS;
                break;
#endif

            default:
                devHandle->intrNum = 0xFFFFu;
                devHandle->regs    = NULL;
                /* Large number outside interrut number range */

        }
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    return result;
}


/**
 * \brief  Initialize the UART device
 *
 * \param   phwInfo         [IN]     Handle to the DDC Hardware Object
 * \param   currentParams   [IN]     Pointer for the Hardware configuration
 *
 * \return Void
 */
static void ddc_uartHwSetup(
                        DDC_uartDevHandle            phwInfo,
                        const PSP_uartDevParams     *currentParams
                        )
{
    Uint32  value = 0;

    if( (NULL != phwInfo) && (NULL != currentParams) )
    {
        /* Set the divisor */
        value = currentParams->inputFreq / \
            (DDC_UART_BAUD_MULTIPLIER * currentParams->baudRate);

        phwInfo->regs->DLL = (Uint8)(value & 0xFF);
        phwInfo->regs->DLH = (Uint8) ((value & 0xFF00) >> 8);

        /* Set the Fifo control - FIFO Disable not supported */
        phwInfo->regs->FCR = 0; /* Clear FIFO counters */

        value = 0;
        if((Bool)TRUE == currentParams->fifoEnable  )
        {

            value = (CSL_UART_FCR_FIFOEN_ENABLE);

            switch (currentParams->rxThreshold)
            {
             case PSP_UART_RX_TRIG1:
                value |= 0x0;   /* Receive FIFO trigger level 1 byte */
                break;
             case PSP_UART_RX_TRIG4:
                value |= 0x40u;  /* Receive FIFO trigger level 4 byte */
                break;
             case PSP_UART_RX_TRIG8:
                value |= 0x80u;  /* Receive FIFO trigger level 8 byte */
                break;
             case PSP_UART_RX_TRIG14:
                value |= 0xC0u;  /* Receive FIFO trigger level 14 byte */
                break;
             default: /* Cant be default. Becz already this param is validated */
                break;
           }

        }

#ifdef PSP_UART_EDMA_SUPPORT
        if(PSP_OPMODE_DMAINTERRUPT == currentParams->opMode)
        {
            value |= 0x0008u;
        }
#endif

        phwInfo->regs->FCR = value;

        /* Setother parameters based upon configuration */
        /* modem control register not programmed */
        phwInfo->regs->MCR = 0;

        value = ((Uint32) currentParams->charLen |
                 (Uint32) currentParams->stopBits |
                 (Uint32) currentParams->parity);

        phwInfo->regs->LCR = value;

        phwInfo->regs->PWREMU_MGMT |= 0x01u;
    }
    return;
}


/**
 * \defgroup ddcUartConfig DDC UART Configuration
 *
 * Interfaces for configuring the UART controller for a specific mode of
 * operation
 *
 * @{
 */

void ddc_uartSetBaudRate(DDC_uartDevHandle devHandle, Uint32 baudRate)
{
    Uint32    value = 0;

    if( NULL != devHandle )
    {
        value = (Uint32)((devHandle->inputFreq) /
                (Uint32)((Uint32)DDC_UART_BAUD_MULTIPLIER * (Uint32)baudRate));
        devHandle->regs->DLL = (Uint8)(value & 0xFF);
        devHandle->regs->DLH = (Uint8)((value & 0xFF00) >> 8);
    }
}
/**< Set the baud rate at which the UART must operate in.
 * \param   devHandle [IN]    {ddc_UartObj *} Handle to the ddc Uart Object
 * \param   baudRate     [IN]    {Int}          Baud rate provided by upper layer
 */

static void ddc_uartEnableHwFlowControl(
                               DDC_uartDevHandle           devHandle,
                               const PSP_uartFlowControl  *pFlowCtrl
                               )
{
    DDC_UartRegister    mcr;

    if( (NULL != devHandle) && (NULL != pFlowCtrl) )
    {
        mcr = devHandle->regs->MCR;

        /* Enable Auto Flow control */
        CSL_FINST(mcr,UART_MCR_AFE,ENABLE);

        /* If only Auto CTS flow control */
        if( PSP_UART_FC_AUTO_CTS_ONLY == pFlowCtrl->fcParam )
        {
            CSL_FINST(mcr,UART_MCR_RTS,DISABLE);
        }
        else /* Auto RTS and auto CTS (Default) */
        {
            CSL_FINST(mcr,UART_MCR_RTS,ENABLE);
        }

        /* Write into MCR register */
        devHandle->regs->MCR = mcr;
    }
}
/**< This enables the hardware flow control for the UART - the hardware supports
 *   auto rts/cts functionality
 * \param   devHandle   [IN]    {DDC_UartObj *} Handle to the DDC Uart Object   */



/**
 * \param   devHandle   [IN]     Handle to the DDC Hardware Object
 *
 * \return Nothing
 */
static Uint8 ddc_uartReadByte(DDC_uartDevHandle devHandle)
{
    Uint32  rbr;

    if( NULL != devHandle )
    {
        /* Read and throw Erroneous bytes from RxFIFO */
        while ((DDC_uartGetLineStatus(devHandle)) & DDC_UART_RX_LINE_ERR_MASK)
        {
            rbr = devHandle->regs->RBR;
        }

        /* Read non-erroneous byte from RxFIFO */
        rbr = devHandle->regs->RBR;
    }

    return (Uint8)rbr;
}

/**
 * \param   ioHandle   [IN]     Handle to the DDC Channel Object
 *
 * \return Nothing
 */
static Uint32 ddc_uartStatusIsDataReady(DDC_uartIoHandle ioHandle)
{
    Uint32    status = 0;

    if( NULL != ioHandle )
    {
        status = DDC_uartGetLineStatus(ioHandle->devHandle);

        /* Added for error checks */
        if ( (status & CSL_UART_LSR_RXFIFOE_MASK ) != FALSE )
        {
            ddc_uartHandleRxError(ioHandle);
        }
    }

    return (Uint32) ((status & CSL_UART_LSR_DR_MASK ) ? 1 : 0);
}

/**
 * \param   devHandle   [IN]     Handle to the DDC Hardware Object
 * \param   ch          [IN]     Character to be written
 *
 * \return Nothing
 */
static void ddc_uartWriteByte(DDC_uartDevHandle devHandle, Char ch)
{
    if(NULL != devHandle)
    {
        devHandle->regs->THR = (Uint32)ch;
    }
}

/**
 * \param   devHandle  [IN]     Handle to the DDC Hardware Object
 * \param   bitmask    [IN]     Bitmask for the register to enable interrupts
 *
 * \return Nothing
 */
void ddc_uartIntrEnable(DDC_uartDevHandle devHandle, DDC_UartRegister bitmask)
{
    DDC_UartRegister temp;

    if(NULL != devHandle)
    {
        temp = devHandle->regs->IER;
        temp |= (bitmask & 0xFFu);
        devHandle->regs->IER = temp;
    }
}


/**
 * \param   devHandle  [IN]     Handle to the DDC Hardware Object
 * \param   bitmask    [IN]     Bitmask for the register to disable interrupts
 *
 * \return Nothing
 */
void ddc_uartIntrDisable(DDC_uartDevHandle devHandle, DDC_UartRegister bitmask)
{
    DDC_UartRegister    temp;

    if(NULL != devHandle)
    {
        temp = devHandle->regs->IER;
        temp &= (~bitmask);
        devHandle->regs->IER = temp & 0xFFu;
    }
}

/**
 * \param   ioHandle   [IN]     Handle to the DDC Channel Object
 *
 * \return Nothing
 */
static void ddc_completeCurrentIO (DDC_uartIoHandle ioHandle )
{

    PSP_uartRequest        pspRequest  ={0};

    if(NULL != ioHandle)
    {
        /* Queue the packet */

        /*
        Copy the ddc IOP packet information to ddc IO structure
        known to app layer
        */
        ddc_copyIOPtoReq(ioHandle->activeIOP, (PSP_uartRequest *)&pspRequest);

        /*
        sometimes the packet can have specific callback function
        */
        if(NULL != pspRequest.cbFxn)
        {
            (*pspRequest.cbFxn)(
                            (Ptr)ioHandle->devHandle->appHandle,
                            (Ptr)ioHandle->cbArg,
                            &pspRequest,
                            ioHandle->mode
                            );
        }
        else
        {
            /*
            otherwise call the application completion callback function
            registered with us during psp_mcaspopen
            */
            if (NULL != ioHandle->cbFxn)
            {
                (*ioHandle->cbFxn)(
                            (Ptr)ioHandle->devHandle->appHandle,
                            (Ptr)ioHandle->cbArg,
                            &pspRequest,
                            ioHandle->mode
                            );
            }

        }

        PAL_osList_NP_Enqueue(&(ioHandle->queueFreeList), ioHandle->activeIOP);
        ioHandle->activeIOP = NULL;
    }/* NULL condition for ioHandle */
}



/**
 *  \brief ddc_copyReqtoIOP
 *
 *  Copies data from request structure provided by DDA layer to
 *  meaningful IO structure at  DDC layer.
 *
 *  \param    pspRequest       [IN]    Porinter to DDA layer reuqest structuer
 *  \param    ioPacket         [IN]    Porinter to DDC layer IO structuer
 *  \return   void
 */
static void ddc_copyReqtoIOP(
                         PSP_uartRequest  const *pspRequest,
                         DDC_UartIoPacket       *ioPacket
                         )
{

    if ((NULL != pspRequest ) && (NULL != ioPacket))
    {
        ioPacket->buf           = pspRequest->addr;
        ioPacket->xferRequest   = pspRequest->size;
        ioPacket->xferActual    = pspRequest->actualSize;
        ioPacket->status        = pspRequest->status;
        ioPacket->appCbk        = pspRequest->cbFxn;
        ioPacket->appData       = pspRequest->appPtr;
        ioPacket->timeout       = pspRequest->timeout;
        ioPacket->cmd           = pspRequest->cmd;
    }
}

/**
 *  \brief ddc_copyIOPtoReq
 *
 *  Restores the data from DDC layer request structure to
 *  DDA understandable psp request structure.
 *
 *  \param    ioPacket         [IN]    Porinter to DDC layer IO structuer
 *  \param    pspRequest       [IN]    Porinter to DDA layer reuqest structuer
 *
 *  \return   void
 */
static void ddc_copyIOPtoReq(
                         DDC_UartIoPacket const *ioPacket,
                         PSP_uartRequest        *pspRequest
                         )
{
    if ((NULL != pspRequest ) && (NULL != ioPacket))
    {
        pspRequest->addr        = ioPacket->buf;
        pspRequest->size        = ioPacket->xferRequest;
        pspRequest->actualSize  = ioPacket->xferActual;
        pspRequest->status      = ioPacket->status;
        pspRequest->cbFxn       = ioPacket->appCbk;
        pspRequest->appPtr      = ioPacket->appData;
        pspRequest->timeout     = 0;
    }

}

/**
 *  \brief Transferring the bytes
 *
 *  \param    ioHandle          [IN]    Handle to the DDC Channel Object
 *  \param    buffer            [IN]    Buffer for the transfer
 *  \param    numBytes          [IN]    No. of bytes to be transferred
 *
 *  \return   void
 */
static Uint32 ddc_uartXfer(
                       DDC_uartIoHandle       ioHandle,
                       Uint8                 *buffer,
                       Uint32                 numBytes
                      )
{
    Uint32 xferedBytes = 0;

    if( NULL != ioHandle )
    {
        /* Depending on the mode call its respective functions */
        if ( PSP_UART_INPUT == ioHandle->mode )
        {
            xferedBytes = ddc_uartFIFORead( ioHandle, buffer, numBytes);
        }
        else
        {
            xferedBytes = ddc_uartFIFOWrite(ioHandle, buffer, numBytes);
        }
    }

    return xferedBytes;

}
/* @} */

/*!
*! Revision History
*
*! ================
*! 31-Jan-2007    JP            : Adding code to enable Line status interrupt
*!                                when rx channel is created.Added code to
*!                                disable UART interrupt causes in PSP_uartClose.
*!
*! 15-Jan-2007    JP            : Review comments updation
*!                                -> unnecessary do while(FALSE) removed.
*!                                -> Struct elements optimization.
*!                                -> Alignment correction.
*!
*! 14-Jan-2007    JP            : Review comments updation
*!                                -> Registering interrupt only for Int mode
*!                                -> IOCTL refinement
*!                                -> ddc_uartClearstats replaced by PAL memset
*!                                -> Correcting the wrong enable of interrupts.
*!                                -> Character timeout will inc only timeout cnt
*! 08-Jan-2007    Kapil            : Removed some elements not required and
*!                                  worked on Doxygen
*!
*! 07-Jan-2007    JP            : Removed unused struct elements. Review
*!                                comments updation started.
*! 20-Dec-2006    Kapil/Vichu   : Initial Creation (0.1)
*! */



