/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \mainpage    PSP UART Driver on BIOS
 *
 *  \section    Introduction
 *
 *  \subsection    xxx    Purpose and Scope
 *  The purpose of this document is to explain the UART driver on the PSP
 *  Framework architecture written in DSP/BIOS for DM648/DM6437/C6424/C6452 platform.
 *
 *  \subsection    aaa    PSP Architecture
 *  A PSP framework is a software entity providing platform support
 *  for devices that are based on Texas Instruments SoC's. The specific
 *  (Hardware and software) implementation of the PSP framework (the PAL
 *  implementation and the group of device drivers) is referred to as
 *  PSP System. The PSP framework defines a software environment for device
 *  drivers to plug-in by defining standard usage interfaces and
 *  abstractions (OS, CPU/Peripherals). For every platform, the PSP
 *  implements platform specific functionality such as OS adaptation, CPU and
 *  peripherals specific mechanisms, and generic system services such as
 *  debug control,traces, persistent storage, etc.
 *
 *  \subsection    bbb    UART Controller
 *  This module is a Universal Asynchronous Receiver/Transmitter (UART) with a VBUS slave
 *  interface. This module performs serial-to-parallel conversion on data received from a
 *  peripheral device or modem, and parallel-to-serial conversion on data received from
 *  the VBUS central resource. The UART module includes complete modem control capability
 *  and an interrupt system that can be tailored to minimize software management of the
 *  communication link. This module also includes a programmable baud rate generator
 *  capable of dividing a reference clock by divisors from 1 to 65,535 to produce a 16 x
 *  clock driving the internal logic.
 *
 *  \subsection    ccc    DSP/BIOS System
 *  DSP/BIOS is a real time kernel provided by TI to use with its
 *  DSP processors.
 *
 *
 *  \subsection    References
 *    -# UART functional specification, <b>uart_req_1_0_0.pdf</b>
 *    -# Design Document for UART driver,
 *                         <b>BIOS_UART_Driver_Arch.doc</b>
 *    -# PSP Framework Overview, <b>PSPFramework_Overview.pdf</b>
 *    -# DSP/BIOS driver development <b>SPRU616.pdf</b>
 *
 *  \file ddc_uart.h
 *
 *  \brief    Header file for UART Device Driver Core(DDC)
 *
 *  This file defines channel and port objects.
 *  This file contains definitions for a UART DDC. This file depends
 *  on some definition from the low-level controller headers.
 *
 * (C) Copyright 2006, Texas Instruments, Inc
 *
 *
 *  \version   1.0
 *
 */

#ifndef _DDC_UART_H
#define _DDC_UART_H

/* header includes */
#include <ti/sdo/pspdrivers/drivers/uart/psp_uart.h>
#include <ti/sdo/pspdrivers/drivers/uart/src/psp_uartApi.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osSem.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osList.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osSem.h>

#ifdef CHIP_DM648
#include <ti/sdo/pspdrivers/soc/dm648/dsp/cslr_uart.h>
#include <ti/sdo/pspdrivers/soc/dm648/dsp/soc.h>
#endif

#ifdef CHIP_C6452
#include <ti/sdo/pspdrivers/soc/c6452/dsp/cslr_uart.h>
#include <ti/sdo/pspdrivers/soc/c6452/dsp/soc.h>
#endif

#ifdef CHIP_DM6437
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/cslr_uart.h>
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/soc.h>
#endif

#ifdef CHIP_C6424
#include <ti/sdo/pspdrivers/soc/c6424/dsp/cslr_uart.h>
#include <ti/sdo/pspdrivers/soc/c6424/dsp/soc.h>
#endif

#ifdef __cplusplus
extern "C"
{
#endif

/** \brief Poll time in milliSeconds                                         */
#define DDC_UART_TPOLL_MSECS                     (1u)
/**
 * \brief Maximum number of iterations allowed within ISR
 *        to check the RX/TX status.                                         */
#define DDC_UART_MAX_ISR_LOOP                    (5u)

/**
 * \brief TaskLet priority for TX interrupt handling in SWI context of DSP.
 */
#define DDC_UART_TASKLET_PRIORITY                    (1u)

/**
 * \defgroup Hardware specific Definitions
 *
 * Definitions specific to this IP of the UART
 *
 * @{
 */
#define DDC_UART_FIFO_SIZE                        (16u)
/**<  FIFO size                                                             */

#define DDC_UART_TX_FIFO_SIZE                     (16u)
/**< TX FIFO size                                                           */

#define DDC_UART_RX_FIFO_SIZE                     (16u)
/**< RX FIFO size                                                           */

#define DDC_UART_BAUD_MULTIPLIER                  (16u)
/**< Baud Multiplier                                                        */

#define DDC_UART_TRANSMITEMPTY_TRIALCOUNT          (12000u)
/**< To make interrupt mode working                                         */

#define DDC_UART_LOOPBACK_MASK                    (0x10u)
/**< Bit postion to enable Loopback                                         */

#define DDC_UART_TRANSMITTER_EMPTY                (0x40u)
/**< Bit postion to find if the transmitter is empty                        */

#define DDC_UART_PARITY_MASK                      (0x18u)
/**< Bit position to enable even parity                                     */

#define DDC_UART_STOP_BITS_MASK                   (0x4u)
/**< Bit postion to set the number of Stop Bits                             */

#define DDC_UART_NUM_DATA_BITS_MASK               (0x3u)
/**< Bit postion to set the number of the DataBits                          */

#define DDC_UART_TX_FIFO_RESET                    (0x04u)
/**< Bit postion to set Transmitter FIFO                                    */

#define DDC_UART_RX_FIFO_RESET                    (0x02u)
/**< Bit postion to set Receiver FIFO                                       */

#define DDC_UART_RESET                            (0x8000u)
/**< Bit postion to unreset the Uart                                        */


#define DDC_UART_HW_FLOW_CONTROL_ENABLE           (0x22u)
/**< Bit postion to enable Flow Control                                     */

#define DDC_UART_HW_FLOW_CONTROL_MASK             (0x20u)
/**< Bit postion to set the auto Flow Control                               */

#define DDC_UART_RX_LINE_STATUS_INT               (0x6u)
/**< Bit Position to ensure the receiver line interrupt                     */

#define DDC_UART_RX_DATA_AVAILABLE_INT            (0x4u)
/**< Bit Position to ensure the received data is available                  */

#define DDC_UART_RX_CHAR_TIMEOUT_INT              (0xCu)
/**< Bit Position to find out the interrupt for timeout                     */

#define DDC_UART_TX_HOLDING_REG_EMPTY_INT         (0x2u)
/**< Bit Position to ensure the Tx Holding Buffer is empty                  */

#define DDC_UART_MODEMSTAT_INT                    (0x0)
/**< Bit Position to set the modem register                                 */

#define DDC_UART_RX_LINE_ERR_MASK   (CSL_UART_LSR_PE_MASK |   \
                                     CSL_UART_LSR_FE_MASK |   \
                                     CSL_UART_LSR_BI_MASK)
/**< Bit position to make sure that error has occured on receiver           */

#define DDC_UART_TX_EMPTY  (CSL_UART_LSR_TEMT_MASK | CSL_UART_LSR_THRE_MASK)
/**< Transmiter empty : THR and TSR both empty                              */

#define DDC_UART_DMA_MODE  \
                       (CSL_UART_FCR_DMAMODE1_MASK | CSL_UART_FCR_FIFOEN_MASK)
/**< DMA mode select macro: Enables DMA mode and enables FIFO in FCR        */

#define DDC_uartGetLineStatus(hUart)    ((hUart)->regs->LSR)
/**< Getting the line status for errors or data ready                       */

#define DDC_uartIntrGetStatus(hUart)   ((hUart)->regs->IIR & 0xFu)
/**< Get interrupt status                                                   */


#define DDC_uartIntrIsReceiverLineError(status)  \
          (((status) == (Uint32)DDC_UART_RX_LINE_STATUS_INT) ? 1 : 0)
/**< Return 1 if receiver line error                                        */


#define DDC_uartIntrIsDataReceived(status)       \
        (((status) == DDC_UART_RX_DATA_AVAILABLE_INT) ? 1 : 0)
/**< Return 1 if data received                                              */


#define DDC_uartIntrIsCharacterTimeout(status)   \
        (((status) == DDC_UART_RX_CHAR_TIMEOUT_INT) ? 1 : 0)
/**< Return 1 if character timeout occured                                  */

#define DDC_uartIntrIsModemStatus(status)        \
        (((status) == DDC_UART_MODEMSTAT_INT) ? 1 : 0)
/**< Return 1 if modem status event occured                                 */

#define DDC_uartIntrIsXmitterShiftRegEmpty(status)   \
       (((status) == DDC_UART_TX_HOLDING_REG_EMPTY_INT) ? 1 : 0)
/**< Return 1 if the transmitter shift register is empty                    */

typedef Uint32   DDC_UartRegister;

/**< uartDevObj UART Object forward declaration                             */
struct DDC_uartDevObj_t;


/**
 * \brief    Uart Packet Structure
 *
 *  This structure has informations about the Uart Packet
 */
typedef struct DDC_UartIoPacket_t
{
    PAL_OsListNodeHeader           iopLink;
    /**< Link for the next IO Packet in the Queue                            */

    Uint32                         cmd;
    /**< IO commnad read/write                                               */

    Ptr                            buf;
    /**< Application supplied IO buffer address                              */

    Uint32                         xferRequest;
    /**< Number of bytes being requested in this transaction                 */

    Int32                          timeout;
    /**< Amount of time given to this iop to complete                        */

    Ptr                            appContext;
    /**< Application supplied application context                            */

    PAL_Result                     status;
    /**< Status of IO transaction                                            */

    Uint32                         xferActual;
    /**< Number of bytes actually transferred                                */

    PSP_uartAppCallback            appCbk;
    /**< Call back function for this request                                 */

    Ptr                            appData;
    /**< Call back function argument if any                                  */
} DDC_UartIoPacket;

#ifdef PSP_UART_BUFFERING_ENABLED
/**
 * \brief    Uart Internal Buffer
 *
 *  This object has uart internal buffer and associated params.
 */
typedef struct DDC_uartInternalBuffer_t
{

    Uint8   buffer[PSP_UART_MAX_BUFFER_SIZE];
    /**<The actual internal buffer that holds the data to be
    tranferred.The size is configured through PSP_UART_MAX_BUFFER_SIZE;
    Also note that this buffer implementation is not circular one */

    Uint8   *bufFillPtr;
    /**< This pointer is used to fill up the temp internal
    buffer from task context from all IOPs; Please note that
    the internal buffer will be filled till it is able to
    accomodate the data. Once it is full or the current application
    request makes to exceed the buffer limit, that request will be queued
    */

    Uint8   *bufReadPtr;
    /**<This pointer is used to read and pump the data out to the
    UART transmitter in ISR context*/

    Uint32  bufferIndex;
    /**<This index maintains the count of bytes filled into the
    internal buffer from application request from Task context*/

    Uint32  xferActual;
    /**This maintains the actual no. of bytes transferred from
    internal buffer. When this equals the bufferIndex, it means that
    we are done with current buffer and reset the members to
    init values*/

}DDC_uartInternalBuffer;
#endif

/**
 * \brief    Uart Channel Object
 *
 *  This object has informations about the Channel of Uart.
 */
typedef struct DDC_uartIoObj_t
{
    PAL_OsListNodeHeader    queuePendingList;
    /**< pending Iop List head                                              */

    PAL_OsListNodeHeader    queueFreeList;
    /**< Free Iop List head                                                 */

    DDC_UartIoPacket        *activeIOP;
    /**< Current Active Packet under progress in the channel                */

    Uint8                   *activeBuf;
    /**< Buffer address of the current packet                               */

    Uint32                  bytesRemaining;
    /**< Bytes remaining for the completion of the current packet           */

    Bool                    fifoEnabled;
    /**< set if the FIFO is enabled                                         */

    Uint32                  chunkSize;
    /**< No. of bytes to be transferred per operation (i.e FIFO Size or 1)  */

    struct DDC_uartDevObj_t  *devHandle;
    /**< Handle to the Device                                               */

    PSP_uartAppCallback     cbFxn;
    /**< Callback to the application                                        */

    Ptr                     cbArg;
    /**< Argument to be passed in the callback                              */

    PSP_UartIoMode              mode;
    /**< mode of the channel i.e INPUT or OUTPUT mode                       */

    DDC_UartIoPacket        iopPool[PSP_UART_NUM_IOP];
    /**< No. of packets per channel to be extracted from the IO Packet pools*/

    Uint32                  errors;
    /**< Counts how many errors encountered per IOP request                 */

    Bool                    inUse;
    /**< set if the channel is in use                                       */

    PAL_OsSemHandle         flushSyncSem;
    /**< Semaphore used when the command is to flush                        */

#ifdef PSP_UART_EDMA_SUPPORT
    Ptr                     hEdma;
    /**< Handle to DMA object                                               */

    Uint32                  Tcc;
    /**< Transfer complete code number                                      */

    Uint32                  ChId;
    /**< DMA Channel                                                        */
#endif

#ifdef PSP_UART_BUFFERING_ENABLED
    DDC_uartInternalBuffer  iBuffer;
    /**< Internal driver buffer                                             */
#endif

} DDC_uartIoObj, *DDC_uartIoHandle;




/**
 * \brief    Uart Device Object
 *
 *  This object has informations about the elements for each
 *  device instance(or Port) of Uart.
 */
typedef struct DDC_uartDevObj_t
{
    Uint32                  instanceId;  /**< instance of Uart               */
    PSP_OpMode              opMode;      /**< Operation mode of driver       */
    PSP_DriverState         devState;    /**< uartDevObj State               */
    DDC_uartIoObj           txIO;        /**< transmiiter channel            */
    DDC_uartIoObj           rxIO;        /**< receiver channel               */
    CSL_UartRegsOvly        regs;        /**< UART HW Register overlay       */
    Uint32                  intrNum;     /**< Interrupt number               */
    PSP_uartStats           stats;       /**< UART specific stats            */
    Bool                    fifoEnable;  /**< HW FIFO enabled or not (FLAG)  */
    Uint32                  inputFreq;   /**< UART Input Frequency in MHZ    */
    Ptr                     appHandle;   /**< Handle of  Upper layer         */
    Ptr                     tskLetHandle;/**< Tasklet(Software int) handle   */
    PSP_UartRxTrigLvl       rxThreshold; /**< RXFIFO threshold value         */
} DDC_uartDevObj ,*DDC_uartDevHandle;


/**
 * \brief    Uart interrupt Macros
 *
 *  This object has informations about the macros for the interrupts
 */
typedef enum DDC_UartIntr_t
{
    DDC_UART_INTR_RHR   = (1L << 0),
    /**< Data ready in receive register                                     */
    DDC_UART_INTR_THR   = (1L << 1),
    /**< Transmitter empty interrupt                                        */
    DDC_UART_INTR_RLS   = (1L << 2),
    /**< A line status changed or an error condition was detected           */
    DDC_UART_INTR_MS    = (1L << 3),
    /**< Modem status interrupt                                             */
    DDC_UART_INTR_ALL   = (0xF)
    /**< All interrupts */
} DDC_UartIntr;

/* @} */

#ifdef __cplusplus
}
#endif


#endif /* _DDC_UART_H_ */

/*!
*! Revision History
*
*! ================
*! 15-Jan-2007    JP            : Added TXEMPTY and DMAMODE macro definitions
*!                                Usage of PSP_uartStats, Removed DDC_uartStats
*!                                Removed unused element ddcIntrNum in Dev obj.
*!                                Optimization of struct elements. Alignment.
*!
*! 08-Jan-2007    Kapil         : Removed some elements not required and
*!                                worked on Doxygen
*!
*! 07-Jan-2007    JP            : Removed unused struct elements. Review
*!                                comments updation started.
*! 20-Dec-2006    Kapil/Vichu   : Initial Creation (0.1)
*! */

