/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file   psp_uartApi.h
 *
 *  \brief  Wrapper APIs called by application (or DDA) to access DDC
 *
 *  This file contains APIs defination of APIs that are wrapper APIs for DDC
 *  layer. These APIs will be called by DDA layer.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *
 *  \version    1.0
 */


#ifndef _PSP_UARTAPI_H_
#define _PSP_UARTAPI_H_

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * \brief UART ISR function pointer
 */
typedef void (* pspUartIsr)(Arg arg);

/**
 *  \brief Application callback function protype
 *
 *  This is the application-level callback function for the UART driver .
 */
typedef void (*PSP_uartAppCallback)(
                                    Ptr chan,
                                    Ptr arg,
                                    const struct PSP_uartRequest_t *Pkt,
                                    Int32 mode
                                    );

/**
  * \brief UART Request Structure
  *
  *  This structure is required to be filled by application while making
  *  a request to UART Driver.
  */
typedef struct  PSP_uartRequest_t
{

    Uint8                cmd;
    /**< IO request command. Read/Write                              */

    Uint8               *addr;
    /**< Pointer to the application Buffer                           */

    Uint32               size;
    /**< Size of the application Buffer                              */

    Uint32               actualSize;
    /**< Acutal bytes transmitted by the driver                      */

    PSP_uartAppCallback  cbFxn;
    /**< UART Application call back per request                      */

    Ptr                  appPtr;
    /**< Pointer to application specified data                       */

    PSP_Result           status;
    /**< Status of the transfer. This data is filled by the driver.  */

    Int32                timeout;
    /**< timeout in millisecs                                        */

} PSP_uartRequest;


/**
 * \brief    Creates/Sets up the UART device instance
 *
 *  This function is used to setup the UART hardware specified
 *  by the device ID.
 *
 * \param    instanceId    [IN]     Device instance number
 * \param    params        [IN]     Hardware initialization parameters
 * \param    arg           [IN]     application handle as arg.
 *
 * \return
 *           PSP_SOK         if success<br>
 *           PSP ERROR Codes if failure<br>
 */
PSP_Result PSP_uartCreate(
                      Uint32                   instanceId,
                      PSP_uartDevParams const *params,
                      Ptr                      arg
                     );


/**
 * \brief    Delete/Un-initializes the UART device instance
 *
 * This function is used to do the reverse operation of
 * PSP_tsipCreate. It uninitializes the TSIP HW for the given
 * devId.
 *
 * \param    instNum         [IN]    DevId of TSIP to be deleted.
 * \param    param           [IN]    additional param(Not used)
 *
 * \return
 *           PSP_SOK            if success<br>
 *           PSP_E_INVAL_PARAM  if invalid arguments<br>
 *
 */
PSP_Result  PSP_uartDelete(Uint32 instNum, Ptr param);

/**
 *  \brief Open a read or write communication channel in UART device
 *
 *  This function prepares the device hardware for data transfers and
 *  usage by the upper layer driver software. The DDA layer is expected
 *  to install the ISR and enable it only after the completion of this
 *  call. The function prepares the driver instance for data transfers
 *  and returns an handle to the driver instance.
 *
 *  \param  instanceId    [IN]        Uart instance number
 *  \param  params        [IN/OUT]    Extra parameter (implementation specific)
 *  \param  mode          [IN]        Driver IO mode
 *  \param  drvCbk        [IN]        Driver callback
 *  \param  arg           [IN]        Argument for future use
 *
 *  \return PSP_Handle  Uart Driver Instance Handle
 */
PSP_Handle PSP_uartOpen(
                    Uint32               instanceId,
                    PSP_UartIoMode       mode,
                    PSP_uartIoParams     *const params,
                    PSP_uartAppCallback  drvCbk,
                    Ptr                  arg
                   );

/**
 *  \brief Close a channel of the UART device
 *
 *  This function closes the device for data transfers and usage by the
 *  upper layer driver software. The hardware is programmed to stop/abort data
 *  transfer (depending upon the type of device and its specifics) and the
 *  device ISR is "disabled" by the upper layer driver software after the
 *  completion of this call. After the successful completion of this call, the
 *  device cannot perform any data IO.
 *
 *  \param  hChan   [IN]          UART channel handle
 *  \param  arg     [IN/OUT]      Extra parameter (implementation specific)
 *
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result  PSP_uartClose(PSP_Handle handle,Ptr arg);

/**
 *  \brief Perform transfer (Read/write) on UART channel
 *
 *  This function submits read/write IO request to the UART channel.
 *
 *  \param  hChan     [IN]     UART read channel handle
 *  \param  ioRequest [IN]     Character Buffer pointer - data to be written
 *  \param  arg       [IN/OUT] Extra parameter (implementation specific)
 *
 *  \return
 *            PSP_SOK           if success<br>
 *            PAL_SINPROGRESS   if request is in progress<br>
 *            PSP Error Codes   if failure<br>
 */
PSP_Result PSP_uartTransfer(
                        PSP_Handle       hChan,
                        PSP_uartRequest *ioRequest,
                        Ptr arg
                        );

/**
 *  \brief Abort pending requests
 *
 *    This API will abort or cancel all the pending requests in the UART
 *    TX or RX channel.
 *
 *  \param  handle   [IN]   Handle to the UART channel
 *
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_uartAbort(PSP_Handle handle);


/**
 *  \brief UART Ioctl
 *
 *  This function provides ioctl functionality for UART
 *
 *  \param  hChan    [IN]       UART channel handle
 *  \param  cmd      [IN]       IOCTL command to be performed
 *  \param  cmdArg   [IN/OUT]   IOCTL command argument (if any)
 *  \param  param    [IN/OUT]   Device/Cmd specific argument
 *
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result  PSP_uartIoctl(
                      PSP_Handle hChan,
                      PSP_UartIoctlCmd cmd,
                      Ptr cmdArg,
                      Ptr param
                     );
#ifdef __cplusplus
}
#endif

#endif  /* _PSP_UART_API_H_ */

/*!
*! Revision History
*
*! ================
*! 14-Jan-2007    JP            : Dead codes removal and API PSP_uartAbort()
*!                                introduced.
*! 08-Jan-2007    Kapil         : Removed some elements not required and
*!                                worked on Doxygen
*!
*! 07-Jan-2007    JP            : Removed unused struct elements. Review
*!                                comments updation started.
*! 20-Dec-2006    Kapil/Vichu   : Initial Creation (0.1)
*! */


