/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file   dda_vpbeIom.c
 *
 *  \brief  VPBE IOM layer Functions
 *
 *  This file contains the Implementation details of IOM layer functions
 *
 *  (C) Copyright 2010, Texas Instruments, Inc
 *
 *  \note       Set tabstop to 4 (:se ts=4) while viewing this file in an
 *              editor
 *
 *  \author     Maulik Desai
 *
 *  \version    0.1     Created
 */

/******************************************************************************
                            Header File Inclusion
******************************************************************************/
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/tistdtypes.h>
#include <std.h>
#include <string.h>
#include <iom.h>
#include <clk.h>

#include <ti/sdo/pspdrivers/drivers/vpbe/psp_vpbe.h>
#include <ti/sdo/pspdrivers/drivers/vpbe/src/psp_vpbeApi.h>
#include <ti/sdo/pspdrivers/drivers/vpbe/src/dda_vpbeIom.h>

#ifdef VPBE_INSTRUMENTATION_ENABLED
    #include <ti/sdo/pspdrivers/pal_os/bios/psp_log.h>
#endif /*VPBE_INSTRUMENTATION_ENABLED */

/*-----------------------------------------------------------------------------
                        Mini Driver Function interface table
-----------------------------------------------------------------------------*/
/**
  * \brief Mini Driver Function interface table
  *
  *
  */
IOM_Fxns VPBEMD_FXNS =
{
    &VPBE_mdBindDev,
    &VPBE_mdUnBindDev,
    &VPBE_mdControlChan,
    &VPBE_mdCreateChan,
    &VPBE_mdDeleteChan,
    &VPBE_mdSubmitChan,
};

/*-----------------------------------------------------------------------------
                        IOM Error code look up table
-----------------------------------------------------------------------------*/
/**
 * The following mapping shows the mapping done from PSP_Result to IOM error
 * code.
 *
 * PSP_SOK                  (0)   IOM_COMPLETED            0
 * PSP_SINPROGRESS          (1)   IOM_PENDING              1
 * PSP_E_DRIVER_INIT        (-1)  IOM_ETIMEOUTUNREC      -11
 * PSP_E_NO_MEMORY          (-2)  IOM_EALLOC              -5
 * PSP_E_RESOURCES          (-3)  IOM_EALLOC              -5
 * PSP_E_INVAL_STATE        (-4)  IOM_EBADMODE            -7
 * PSP_E_INVAL_PARAM        (-5)  IOM_EBADARGS           -10
 * PSP_E_NOT_SUPPORTED      (-6)  IOM_ENOTIMPL            -9
 * PSP_E_IO_CANCEL_FAIL     (-7)  IOM_ETIMEOUT            -2
 * PSP_E_FIFO_NOT_ENABLED   (-8)  IOM_EBADMODE            -7
 * PSP_E_INVALID_MODE       (-9)  IOM_EBADMODE            -7
 * PSP_E_INVALID_CMD        (-10) IOM_EBADARGS           -10
 *
 */
static Int iomErrorCodeList[12] =
{
    IOM_EBADARGS,
    IOM_EBADMODE,
    IOM_EBADMODE,
    IOM_ETIMEOUT,
    IOM_ENOTIMPL,
    IOM_EBADARGS,
    IOM_EBADMODE,
    IOM_EALLOC,
    IOM_EALLOC,
    IOM_ETIMEOUTUNREC,
    IOM_COMPLETED,
    IOM_PENDING
};

/**
  * \brief IOM Port Object
  *
  */
static VPBEPortObject vpbe_ports;

/**
 *  \brief  PSP error code converter
 *
 * This function converts PSP error codes to appropriate IOM error code.
 *
 * \param  pspCode          [IN]    PSP Error code.
 *
 * \return IOM ERROR CODE   [OUT]   Equivalent IOM error code. For unrecoganized
 *                                  error code IOM_EBADIO is returned.
 *
 */
static Int getIomErrorCode(PSP_Result pspCode);

/******************************************************************************
                                Functions
******************************************************************************/

/**
  * \brief fuction will get the IOM Error Code
  *
  */
static Int getIomErrorCode(PSP_Result pspCode)
{
    Int iomCode = pspCode+10;

    if(iomCode < 12)
    {
        iomCode = iomErrorCodeList[iomCode];
    }
    else
    {
        iomCode = IOM_EBADIO;
    }

    return(iomCode);
}

/*-----------------------------------------------------------------------------
                            VPBE_mdBindDev() Function
-----------------------------------------------------------------------------*/

/**
  * \brief  VPBE_mdBindDev() is called by DEV_init() to bind and initialize
  *         the instance
  */
/*ARGSUSED */
Int VPBE_mdBindDev(Ptr *devp, Int devid, Ptr devParams)
{
    Int     statusBnd   = IOM_COMPLETED;
    Uint32  ldevid      = (Uint32)devid;
    DDC_VPBEDeviceObject * vpbeDeviceHandleCreate =
                            (DDC_VPBEDeviceObject * )DDC_VPBEGetDeviceHandle();

    /*Check the parameters not NULL and validate the instance */
    if((NULL == devp) || ((Uns)ldevid >= PSP_VPBE_NUM_INSTANCES))
    {
        statusBnd =  (IOM_EBADARGS);
    }
    else
    {
        if(NULL != vpbeDeviceHandleCreate)
        {
        /* Setting initial global values to zero */
            memset(&vpbe_ports,0,sizeof(VPBEPortObject));
            memset(vpbeDeviceHandleCreate,0,sizeof(DDC_VPBEDeviceObject));

            /* Initialize Driver */
            statusBnd = PSP_VPBECreate((Uint32)devid);
            if(PAL_SOK != statusBnd )
            {
                statusBnd = getIomErrorCode(statusBnd);
            }
            else
            {
                /*Update the Port object */
                vpbe_ports.dda_PortObj      = vpbeDeviceHandleCreate;
                vpbe_ports.vpssPortNumber   = ldevid;
                /*Set the state as created */
                vpbe_ports.state            = VPSS_CREATED;
                *devp                       = &vpbe_ports;

            }
        }
    }
    return(statusBnd);
}

/*-----------------------------------------------------------------------------
                                VPBE_mdUnBindDev
-----------------------------------------------------------------------------*/
/**
  * \brief VPBE_mdUnBindDev() is called by DEV_init() to unbind and
  *        de-init the instance
  */
Int VPBE_mdUnBindDev(Ptr devp)
{
    Int             statusUnBnd     = IOM_COMPLETED;
    VPBEPortObject* portUnBnd       = NULL;
    /*Check the parameter not NULL */
    if(NULL == devp)
    {
        statusUnBnd = IOM_EBADARGS;
    }
    else
    {
        portUnBnd = (VPBEPortObject *)devp;
        /*Check the state of port */
        if ((VPSS_CREATED != portUnBnd->state) &&
            (VPSS_CLOSED  != portUnBnd->state))
        {
            statusUnBnd =  IOM_EBADMODE;
        }
        /*Validate the Instance number */
        else if (portUnBnd->vpssPortNumber >= PSP_VPBE_NUM_INSTANCES )
        {
            statusUnBnd = IOM_EBADARGS;
        }
        else
        {
            /* De-Init Driver.This function closes channel handle  */
            statusUnBnd = PSP_VPBEDelete(portUnBnd->vpssPortNumber);
            if(PAL_SOK != statusUnBnd)
            {
                statusUnBnd = getIomErrorCode(statusUnBnd);
            }
            else
            {
                /* Set the state as deleted */
                portUnBnd->state = VPSS_DELETED;
            }
        }
    }

    return(statusUnBnd);
}

/*-----------------------------------------------------------------------------
                                VPBE_mdCreateChan
    Creates the channel for the data transfer by setting up some params
-----------------------------------------------------------------------------*/
/**
  * \brief Creates the channel for the data transfer by setting up some params
  *
  */
/* ARGSUSED */
Int VPBE_mdCreateChan(Ptr *chanp, Ptr devp, String name, Int mode,
                      Ptr chanParams, IOM_TiomCallback cbFxn, Ptr cbArg)
{
    Int                     statusCR    = IOM_COMPLETED;
    VPBEPortObject*         portCR      = NULL;
    VPBEChannelObject*      channel     = NULL;
    PSP_VPBEChannelParams*  param       = NULL;
    /*Check the parameters not NULL */
    if((((NULL == chanp) || (NULL == devp)) || (NULL == chanParams)) ||
       ((NULL == cbFxn)  || (NULL == cbArg)))
    {
        statusCR = IOM_EBADARGS;
    }
    else
    {
        portCR = (VPBEPortObject *)devp;
        /*Check the port state */
        if (((VPSS_CREATED != portCR->state)    &&
             (VPSS_CLOSED  != portCR->state))   &&
             (VPSS_OPENED  != portCR->state))
        {
            statusCR =  (IOM_EBADMODE);
        }
        else
        {
            param = (PSP_VPBEChannelParams *)chanParams;
            /*Check whether channel is in use */
            if(FALSE == portCR->chan[param->id].inUse)
            {
                /*Assign channel when not in use */
                channel = (VPBEChannelObject *)(&(portCR->chan[param->id]));
            }
            else
            {
                statusCR = IOM_EINUSE;
            }
            if (((mode != IOM_INPUT) && (mode != IOM_OUTPUT) )
                             && (mode != IOM_INOUT) )
            {
                    statusCR = IOM_EBADMODE;
            }
            else
            {
                    channel->mode = mode;
            }

        }
    }

    if(IOM_COMPLETED == statusCR)
    {
        /* Creates handle for VENC/OSD Planes channel */
        channel->ddc_Handle = (PSP_Handle)PSP_VPBEOpen(portCR->vpssPortNumber,
                                                       chanParams);
        if(NULL == channel->ddc_Handle)
        {
            statusCR = IOM_EBADARGS;
        }
        else
        {
            portCR->state = VPSS_OPENED;
            channel->cbFxn = cbFxn;
            channel->cbArg = cbArg;
            channel->inUse = TRUE;
            /*Increment the number of opens */
            ++portCR->numOfOpens;
            /* sets for back referencing */
            channel->port = (VPBEPortObject *)portCR;
            *chanp = channel;
        }
    }

    return(statusCR);
}

/*-----------------------------------------------------------------------------
                                VPBE_mdDeleteChan
-----------------------------------------------------------------------------*/

/**
  * \brief  Deletes an OSD Planes/VENC Channel
  *
  */
Int VPBE_mdDeleteChan(Ptr chanp)
{
    Int                 statusDL    = IOM_COMPLETED;
    VPBEChannelObject*  chanDL      = NULL;
    VPBEPortObject*     portDL      = NULL;
    /*Check the parameters not null */
    if(NULL == chanp)
    {
        statusDL = IOM_EBADARGS;
    }
    else
    {
        chanDL = (VPBEChannelObject *)chanp;
        portDL = (VPBEPortObject *)chanDL->port;
        /*Check the Channel is in use and port handle not null */
        if((FALSE == chanDL->inUse) ||
           ((NULL == portDL) || (VPSS_OPENED != portDL->state)))
        {
            statusDL = IOM_EBADARGS;
        }
    }
    if(IOM_COMPLETED == statusDL)
    {
        /*Closes the Channel Handle */
        statusDL = PSP_VPBEClose(chanDL->ddc_Handle);
        if(PAL_SOK != statusDL)
        {
            statusDL = getIomErrorCode(statusDL);
        }
        else
        {
            chanDL->inUse = FALSE;
            /* Updates the state and makes the channel port void*/
            chanDL->port = NULL;
            /* Check if all the channels are closed */
            /* If all channels are closed then update the state to closed */
            if (0 == --portDL->numOfOpens)
            {
                portDL->state = VPSS_CLOSED;
            }
            chanp = &chanDL;
        }
    }
    return(statusDL);
}

/*-----------------------------------------------------------------------------
                                VPBE_mdControlChan
    All dynamic configurations functionality of a mini driver can be done.
-----------------------------------------------------------------------------*/
/**
  *\ brief  mdcontrolchan is used to config the ioctl operation
  *
  */
Int VPBE_mdControlChan(Ptr chanp, Uns cmd, Ptr arg)
{
    Int                 statusCNTRL     = IOM_COMPLETED;
    VPBEChannelObject * chanCNTRL       = NULL;
    VPBEPortObject    * portCNTRL       = NULL;

    if((NULL == chanp)&&(NULL == arg))
    {
        statusCNTRL = IOM_EBADARGS;
    }
    else
    {
        chanCNTRL = (VPBEChannelHandle)chanp;
        portCNTRL = (VPBEPortObject* )chanCNTRL->port;
        /*Check the port and state of port */
        if((NULL == portCNTRL) || (VPSS_OPENED != portCNTRL->state))
        {
            statusCNTRL = IOM_EBADMODE;
        }
        else
        {   /*Call the IOctl function which execute the dynamic configuration */
            statusCNTRL = PSP_VPBEIoctl(chanCNTRL->ddc_Handle,(PSP_VPBEIoctlCommand)cmd,arg,NULL);
            /*Check status if success */
            if (PSP_SOK != statusCNTRL)
            {
                /*Generate IOM error code equivalent to PSP error code */
                statusCNTRL = getIomErrorCode(statusCNTRL);
            }
        }
    }

    return(statusCNTRL);
}

/*-----------------------------------------------------------------------------
                                VPFE_mdSubmitChan
-----------------------------------------------------------------------------*/
/**
  * \brief VPFE_mdSubmitChan is called by Bios to submit the data packet
  *
  */
Int VPBE_mdSubmitChan(Ptr chanp, IOM_Packet *packet)
{
    Int                 statusSBMT  = IOM_COMPLETED;
    VPBEChannelObject * chanSBMT    = NULL;
    VPBEPortObject    * portSBMT    = NULL;

    /* Check the parameters if null */
    if( (chanp == NULL)||(packet == NULL) )
    {
        statusSBMT = IOM_EBADARGS;
    }
    else
    {
        chanSBMT = (VPBEChannelObject*)chanp;

        if ((NULL == packet->addr) || (NULL == chanSBMT->port))
        {
            statusSBMT = IOM_EBADARGS;
        }
        else
        {

#ifdef VPBE_INSTRUMENTATION_ENABLED
            PSP_LOG_EVENT(  &DVTEvent_Log,
                            "VPBE",
                            PSP_DVT_DESC(
                                PSP_DVT_eFUNC_START,
                                PSP_DVT_dCHANNEL_ID_INST,
                                PSP_DVT_dPACKET_ID,
                                PSP_DVT_dSIZE_BYTES),
                            CHANNELID_INSTANCEID(
                                1,
                                0),
                            packet->addr,
                            packet->size);
#endif /*VPBE_INSTRUMENTATION_ENABLED */

            portSBMT = (VPBEPortObject *)chanSBMT->port;
            /* Check whether channel is inuse */
            if(((NULL== portSBMT)||(FALSE == chanSBMT->inUse)) || (NULL!=chanSBMT->dataPacket))
            {
                statusSBMT = IOM_EBADMODE;
            }
            else
            {
                /*Check state is opened */
                if(portSBMT->state != VPSS_OPENED){
                    statusSBMT = IOM_EBADMODE;
                }
                else
                {
                    chanSBMT->dataPacket = packet;
                    if((FVID_QUEUE == packet->cmd))
                    {
                        /* Call the SubmitRequest(queue/dequeue) */
                        statusSBMT = PSP_VPBESubmitRequest(chanSBMT->ddc_Handle,FVID_QUEUE,packet->addr);
                        /*Check the status if success */
                        if(statusSBMT != PSP_SOK)
                        {
                            /*Generate the IOM error code equivalent to PSP error code */
                            statusSBMT = getIomErrorCode(statusSBMT);
                        }
                    }
                    if((FVID_DEQUEUE == packet->cmd))
                    {
                        /* Call the SubmitRequest(queue/dequeue) */
                        statusSBMT = PSP_VPBESubmitRequest(chanSBMT->ddc_Handle,FVID_DEQUEUE,packet->addr);
                        /*Check the status if success */
                        if(statusSBMT != PSP_SOK){
                            /*Generate the IOM error code equivalent to PSP error code */
                            statusSBMT = getIomErrorCode(statusSBMT);
                        }
                    }
                    if(FVID_EXCHANGE == packet->cmd)
                    {
                        /* Call the SubmitRequest(queue/dequeue) */
                        /* Value of Void pointer cannot be typecasted */
                        statusSBMT = PSP_VPBESubmitRequest(chanSBMT->ddc_Handle,FVID_QUEUE,(Ptr)(*(Uint32 *)(packet->addr)));
                        /*Check the status if success */
                        if(statusSBMT != PSP_SOK){
                            /*Generate the IOM error code equivalent to PSP error code */
                            statusSBMT = getIomErrorCode(statusSBMT);
                        }
                        if(PSP_SOK == statusSBMT)
                        {
                            /* Call the SubmitRequest(queue/dequeue) */
                            statusSBMT = PSP_VPBESubmitRequest(chanSBMT->ddc_Handle,FVID_DEQUEUE,packet->addr);
                            /*Check the status if success */
                            if(statusSBMT != PSP_SOK){
                                /*Generate the IOM error code equivalent to PSP error code */
                                statusSBMT = getIomErrorCode(statusSBMT);
                            }
                        }
                    }

                    /* Call the SubmitRequest(queue/dequeue) */
                    chanSBMT->dataPacket = NULL;
                }
            }

#ifdef VPBE_INSTRUMENTATION_ENABLED
            PSP_LOG_EVENT(  &DVTEvent_Log,
                            "VPBE",
                            PSP_DVT_DESC(
                                PSP_DVT_eFUNC_END,
                                PSP_DVT_dCHANNEL_ID_INST,
                                PSP_DVT_dPACKET_ID,
                                PSP_DVT_dSIZE_BYTES),
                            CHANNELID_INSTANCEID(
                                1,
                                0),
                            packet->addr,
                            0);
#endif /*VPBE_INSTRUMENTATION_ENABLED */

        }
    }/*else */

    return statusSBMT;
}

 /**
   * \brief VPFE Time Stamp Function
   */
extern void _DDA_VPBEGetTimeStamp(Ptr timeStamp)
{
    Uint32 *time = NULL;

    if(NULL != timeStamp)
    {
        time = (Uint32 *)timeStamp;
        if(NULL != time)
        {
            *time = CLK_getltime();
        }
    }
}
