/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/
/**
 *  \file   ddc_vpbe.h
 *
 *  \brief  DDC header file for VPBE
 *
 *  This file contains the dependencies across the VPBE driver and does
 *  not have dependencies specific to a plane or VENC however it
 *  contains elements that may be used in one or all of the above.
 *
 *  (C) Copyright 2010, Texas Instruments, Inc
 *
 *  \note       Set tabstop to 4 (:se ts=4) while viewing this file in an
 *              editor
 *
 *  \author     Maulik Desai
 *
 *  \version    0.1     Created
 */
#ifndef _DDC_VPBE_H
#define _DDC_VPBE_H

#include <ti/sdo/pspdrivers/system/dm6437/bios/evmDM6437/video/psp_vpss_extvidEncoder.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osSem.h>

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
                                 Macros
******************************************************************************/

/** Max buffers associated within driver        */
#define DDC_VPBE_MAX_FRAME_BUFFERS       (20u)

/** Number of OSD plane supported by h/w        */
#define DDC_VPBE_NUM_OF_OSD_PLANES       (4u)

/*-----------------------------------------------------------------------------
                              DDC Channel State
-----------------------------------------------------------------------------*/
/**
 * \brief       This enum contains description about the state of the
 *              plane(video/OSD),VENC/CURSOR. Individually these are
 *              termed as channels.
 */
typedef enum _DDC_VPBEState
{
    /**< VPBE channel closed state for planes(video/OSD),VENC/CURSOR    */
    DDC_VPBE_CLOSED,
    /**< VPBE channel open state for planes(video/OSD),VENC/CURSOR      */
    DDC_VPBE_OPENED
} DDC_VPBEState;

/*-----------------------------------------------------------------------------
                              VPSS Driver State
-----------------------------------------------------------------------------*/
/**
 * This enum describes the various states which the VPBE driver goes through
 * the entire cycle.
 *
 */
typedef enum VPSS_STATE
{
    VPSS_DELETED,
    VPSS_CREATED,
    VPSS_OPENED,
    VPSS_CLOSED
} vpssState;

/*-----------------------------------------------------------------------------
                              VPBE Plane Index
-----------------------------------------------------------------------------*/
/**
 * \brief       This enum holds the indexes of the planes (Video/OSD).
 */
typedef enum _DDC_VPBEPlaneIndex
{
    /**< VPBE plane Zero for VIDEO 0        */
    DDC_VPBE_PLANE_VIDEO0 = 0,
    /**< VPBE plane One for VIDEO 1         */
    DDC_VPBE_PLANE_VIDEO1,
    /**< VPBE plane two for OSD 0           */
    DDC_VPBE_PLANE_OSD0 ,
    /**< VPBE plane three for OSD 1         */
    DDC_VPBE_PLANE_OSD1
} DDC_VPBEPlaneIndex;

/*-----------------------------------------------------------------------------
                              Queue Types
-----------------------------------------------------------------------------*/
/**
 * \brief       This enum contains the indexes for active or free queue. The
 *              VPBE driver uses the active queue and the free queue for
 *              implementing the flipping of the frame buffers
 */
typedef enum _DDC_VPBEQueueType
{
    /**< VPBE Free Queue Type       */
    DDC_VPBE_FREE_QUEUE = 0,
    /**< VPBE Active Queue Type     */
    DDC_VPBE_ACTIVE_QUEUE
} DDC_VPBEQueueType;

/*-----------------------------------------------------------------------------
                            Error Codes
-----------------------------------------------------------------------------*/
/**
 * \brief       This enum contains DDC related error codes that will be passed
 *              to the DDA layer.
 */
typedef enum _DDC_VPBEStatus
{
    /**< Operation failed error code for DDC related functionality           */
    DDC_VPBE_E_FAIL = 0,
    /**< Operation passed code for DDC related functionality                 */
    DDC_VPBE_SOK,
    /**< Invalid state error code for DDC related functionality              */
    DDC_VPBE_INVALID_STATE,
    /**< Invalid param error code for DDC related functionality              */
    DDC_VPBE_INVALID_PARAM,
    /**< Resources not available error code for DDC related functionality    */
    DDC_VPBE_E_RESOURCES,
    /**< Invalid module error code for DDC related functionality             */
    DDC_VPBE_NO_MEMORY,
    /**< No Memory Error  for DDC related functionality                      */

    DDC_VPBE_NOT_SUPPORTED
} DDC_VPBEStatus;

/*-----------------------------------------------------------------------------
                            Field Id
-----------------------------------------------------------------------------*/
/**
 * \brief       This enum is used to get the current field information.
 */
typedef enum _DDC_VPBEFieldId
{
    DDC_VPBE_1ST_FIELD  = 0,
      /**< Odd field information              */
    DDC_VPBE_2ND_FIELD
    /**< Even field information             */
} DDC_VPBEFieldId;

/*-----------------------------------------------------------------------------
                          Frame Buffer Object
-----------------------------------------------------------------------------*/
/**
 * \brief       This structure defines the frame buffer object which contains
 *              the relevant elements responsible for the flipping of multiple
 *              frame buffers and produce a seamless display.
 */
typedef struct _DDC_VPBE_FrameBufferObject
{
    /**< Variable to write index active queue                                */
    Uint                      writeIndexActiveQue;
    /**<  Variable to read index active queue                                */
    Uint                      readIndexActiveQue;
    /**< Variable to write index free queue                                  */
    Uint                      writeIndexFreeQue;
    /**<  Variable to read index free queue                                  */
    Uint                      readIndexFreeQue;
    /**<  Array to access the various VpssSurfaceParams                      */
    FVID_Frame                *activeQueue[DDC_VPBE_MAX_FRAME_BUFFERS];
    /**<  Array to access the various VpssSurfaceParams                      */
    FVID_Frame                *freeQueue[DDC_VPBE_MAX_FRAME_BUFFERS];
    /**<  Temporary address to store framebuffer address                     */
    FVID_Frame                *tempfbAddFree;

} DDC_VPBEFrameBufferObject;


/* ------------------------------------------------------------------------- */
/* -----------------------------| OSD Object |------------------------------ */
/* ------------------------------------------------------------------------- */
/**
  * \brief      Structure to define the OSD object which includes relevant
  *             element for OSD functionality
  */
typedef struct _DDC_VPBEOsdObject
{
    PSP_VPBE_Id                   OsdChanId;
    /**< Channel id for OSD plane                                        */
    DDC_VPBEState                 state;
    /**< Vpss state either closed or open                                */
    Ptr                           OsdRegs;
    /**< Handle for vpbe register                                        */
    PSP_VPBEOsdConfigParams       OsdParams;
    /**< VPBE osd config params                                          */
    struct _DDC_VPBEDeviceObject *vpssDeviceHandle;
    /**< Handle for vpbe device object                                   */
    Bool                          PlaneActive;
    /**< VIDEO/OSD Plane is Active or Not                                */
    Ptr                           fbPtr;
    /**< Frame Buffer Pointer                                            */
    DDC_VPBEFrameBufferObject     fb;
    /**< Variable for buffer object                                      */
    PAL_OsSemHandle                vpbedequeueSem;
    /**< DequeueSemaphore handle                                         */
    Bool                           vpbedequeueWaitFlag;
    /**< DequeueSemaphore Wait Flag                                      */

} DDC_VPBEOsdObject;

/* ------------------------------------------------------------------------- */
/* -----------------------| Cursor Object (Cursor) |------------------------ */
/* ------------------------------------------------------------------------- */
/**
  * \brief      Structure to define the cursor object which includes relevant
  *             element for cursor functionality
  *
  */
typedef struct _DDC_VPBECursorObject
{
    Int                           CursorChanId;
    /**< Channel id for cursor object                                   */
    DDC_VPBEState                 state;
    /**< Vpss state either closed or open                               */
    Ptr                           CursorRegs;
    /**< Handle for vpbe register                                       */
    PSP_VPBECursorConfigParams    CursorParams;
    /**< Handle for the cursor config params                            */
    struct _DDC_VPBEDeviceObject  *vpssDeviceHandle;
    /**< Handle for vpbe device object                                  */
} DDC_VPBECursorObject;


/* ------------------------------------------------------------------------- */
/* ---------------------| VENC Object (Video Encoder) |--------------------- */
/* ------------------------------------------------------------------------- */
/**
  * \brief      Structure to define the VENC object which includes relevant
  *             element for VENC functionality
  */
typedef struct _DDC_VPBEVencObject
{
    Int                          VencChanId;
    /**< Channel id for Venc object                                      */
    DDC_VPBEState                state;
    /**< Vpss state either closed or open                                */
    Ptr                          VencRegs;
    /**< Handle for vpbe register                                        */
    PSP_VPBEVencConfigParams     VencParams;
    /**< Handle for Venc config params                                   */
    Bool                           fieldId;
    /**<    Field Id for Interlaced Mode                                 */
    struct _DDC_VPBEDeviceObject *vpssDeviceHandle;
    /**< Handle for vpbe device object                                   */
    PSP_VPSS_ExtVideoEncoderFxns  *extVidEncFxn;
    /**< External Video Encoder Function pointer                         */
    Ptr                            extVidEncHandle;
    /**< External Video Encoder Handle                                   */
} DDC_VPBEVencObject;

/* ------------------------------------------------------------------------- */
/* ---------------------|   Device Object  |-------------------------------- */
/* ------------------------------------------------------------------------- */
/**
  * \brief      Stucture for the vpbe device object as a whole
  */
typedef struct _DDC_VPBEDeviceObject
{
    Uint                        vpbeInstanceId;
    /**< Vpbe Instance id                                                */
    vpssState                   state;
    /**< Vpss channel state                                              */
    Uint                        IntNum;
    /**< Back End Interrupt Number                                       */
    DDC_VPBEOsdObject           osd[DDC_VPBE_NUM_OF_OSD_PLANES];
    /**< Handle for OSD object                                           */
    DDC_VPBECursorObject        cursor;
    /**< Handle for Cursor object                                        */
    DDC_VPBEVencObject          venc;
    /**< Handle for Venc object                                          */
} DDC_VPBEDeviceObject;

/******************************************************************************
                         INTERFACE FUNCTIONS
                 This Functions are called from the DDA Layer
******************************************************************************/
/**
  * \brief      This function validate the basic parameters for various channels
  *             It configure the hardware and return handle of the specific
  *             VPBE (Video/OSD/VENC/CURSOR) channel
  *
  * \param      id          [IN]        Id of the channel for which
  *                                     the handle is to be opened.
  *
  * \param      params      [IN]        Parameters with which the
  *                                     channel has to be created.
  *
  * \returns    Handle to the specific channel
  */
Ptr DDC_VPBEOpenHandle(PSP_VPBE_Id id, Ptr params);

/**
  * \brief      This function closes the handle of the corresponding channel
  *
  * \param      handle      [IN]        Handle of the channel for which
  *                                     the handle to be closed.
  *
  * \returns    Count of the number of channels currently in open state
  */
DDC_VPBEStatus DDC_VPBECloseHandle(Ptr handle);

/**
  * \brief      This function is used to check if the handle is in closed state
  *
  * \param      handleICH   [IN]     Handle of PLANES(VID/OSD)
  *
  * \returns    DDC_VPSS_E_FAIL - if channel handle is Open
  *             DDC_VPSS_SOK    - if channel handle is Close
  */
DDC_VPBEStatus DDC_VPBEIsChannelHandleClosed(Ptr handleICH);

/**
  * \brief      This Functions returns vpbe device handle for the entire VPBE
  *             device object. Through this VPBE device object can be access
  *             by any of the VPBE (Video/OSD/VENC/CURSOR) channel.
  *
  * \returns     It will return the handle to VPBE device object
  */
Ptr DDC_VPBEGetDeviceHandle(void);

/**
 *  \brief      This function is used to add frame buffers to either the
 *              ACTIVE queue or the FREE queue. This is a high layer function
 *              which will be invoking the DDC_VpssAddqueue function.
 *
 *              On exit this fucnition returns the status of the operation that
 *              was performed on the queue.
 *
 *  \param      handleQF        [IN]        Handle of PLANES for
 *                                          which the queuing of the
 *                                          frame buffer has to be
 *                                          performed.
 *
 *  \param      fbPtrQF         [IN]        Pointer to the frame buffer
 *                                          that will be updated in the
 *                                          queue.
 *
 *  \returns    Status of the operation performed
 */

DDC_VPBEStatus DDC_VPBEQueueFrameBuffer(Ptr                     handleQF,
                                        FVID_Frame * fbPtrQF);
/**
 *  \brief      This function is used to remove frame buffers from either the
 *              ACTIVE queue or the FREE queue. This is a high layer function
 *              which will be invoking the DDC_VpssDelqueue function.
 *
 *              On exit this fucnition returns the status of the operation that
 *              was performed on the queue.Also it returns the pointer to the
 *              frame buffer that has been removed or dequeued.
 *
 *  \param      handleDQ                [IN]        Handle of PLANES for
 *                                                  which the queuing of the
 *                                                  frame buffer has to be
 *                                                  performed.
 *
 *  \param      deQueuedFrameBufferDQ   [OUT]       Pointer to the frame buffer
 *                                                  that has been removed from
 *                                                  the driver.
 *
 *  \returns    Status of the operation performed
 */
DDC_VPBEStatus DDC_VPBEDeQueueFrameBuffer(Ptr                      handleDQ,
                                          FVID_Frame ** deQueuedFrameBufferDQ);
/**
 *  \brief      Validate the frame buffers allocated by the application
 *
 *              This function is used to validate the frame buffers allocated
 *              by the application. Function returns a SUCCESS if the frame
 *              buffer in the surface params is valid.
 *
 *              The surface parameters that this function validates are all
 *              except the Pixelparams because this parameter may change.
 *              If not then the function returns a failure.
 *
 *  \param      surfVF              [IN]    Details of the frame buffer
 *                                          to be validated.
 *
 *  \param      handleVF            [IN]    Handle for OSD Planes frame
 *
 *  \returns    SUCCESS/FAILURE of status
 */
DDC_VPBEStatus DDC_VPBECheckFrameBuffer(Ptr                     handleVF,
                                           FVID_Frame * surfVF);

/**
  * \brief      This funcion performs register overlaying
  *
  */
void DDC_VPBERegisterOverlaying(void);

/**
  * \brief      This function initialize VPBE Module and related  VPBE
  *             hardware parameters
  */
void DDC_VPBEInitializeModule(void);
/**
  * \brief      This function set VPBE ISR number
  *
  */
void DDC_VPBESetInterruptNumber();


/**
  * \brief      Configure Window boundary Parameter
  *
  * \param      handleCW            [IN]    Handle of the channel of
  *                                         which application desires
  *                                         to configure the window
  *                                         parameters.
  *
  * \param      paramsCW            [IN]    Parameters with which the
  *                                         channel window parameters
  *                                         are configured.
  *
  * \Returns    Success or failure status
  */
DDC_VPBEStatus DDC_VPBEConfigWindowBoundary(Ptr handleCW, Ptr paramsCW);

/**
 *  \brief      This Function is called from IOCTL Function.It configures
 *              the zoom parameters of OSD Planes(VIDEO and OSD).Function
 *              support the Horizontal and Vertical spatial expansion
 *              along with Zooming.
 *
 *
 *  \param      handleCZ            [IN]        OSD Planes(VIDEO/OSD)
 *                                              Channel Handle
 *
 *
 *  \param      paramsCZ            [IN]        Zoom Params.
 *
 *  \returns    SUCCESS/FAILURE
 */
DDC_VPBEStatus DDC_VPBEConfigZoomParameters(Ptr handleCZ, Ptr paramsCZ);

DDC_VPBEStatus DDC_VPBEValidateFrameBuffer(PSP_VPBEValidateParams *surfVF);

/************************************************************************
                          LOCAL FUNCTIONS
               This Functions are called by DDC layer functions
*************************************************************************/
/**
 *  \brief      This function returns the ID of the corresponding Channel handle
 *              passed as parameter to this function
 *
 *
 *  \param      handle              [IN]     Handle of VPBE channel
 *                                           (Video/OSD/VENC/CURSOR)
 *                                           for which the ID is required
 *
 *  \returns     ID of the Channel handle
 */
PSP_VPBE_Id DDC_VPBEGetChannelId(Ptr handle);

/**
 *  \brief      This function returns Channel handle of the corresponding ID
 *              passed as parameter to this function.
 *
 *
 *  \param      Id                  [IN]    Enumerated ID for which the
 *                                          handle is required
 *
 *  \returns    Handle of VPBE Channel for which the ID is passed.
 */
Ptr DDC_VPBEGetChannelHandle(PSP_VPBE_Id id);

/**
  * \brief      This function will return the OSD plane status
  *             whether active/not active
  *
  * \param      idGV            [IN]    VPBE Channel id
  *
  * \returns    0 - If OSD plane is not opened/is not active
  *             1 - If OSD plane is already opened and active
  */

Bool DDC_VPBEIsOsdPlaneActive(PSP_VPBE_Id idGV);

/**
  * \brief      This function will reset the frame buffer object
  *
  * \param      fbRF                [IN]        Pointer to the frame buffer
  *                                             object to be reset.
  */
void DDC_VPBEResetFrameBufferObject(DDC_VPBEFrameBufferObject * fbRF);

/**
 *  \brief      This function is used to add the frame buffers either in the
 *              ACTIVE queue or the FREE queue.
 *
 *              On exit this funcition returns the status of the operation that
 *              was performed on the queue.
 *
 *  \param      qTypeAQ             [IN]    Index of the Queue where the
 *                                          frame buffer has to be added
 *
 *                                          ACTIVE and FREE queues are
 *                                          the 2 types of queues used.
 *
 *  \param      handle              [IN]     Handle of VPBE channel
 *
 *  \param      fbAddressAQ         [IN]    Pointer to the frame buffer
 *                                          to be queued.
 *
 *  \returns    Pass and fail status
 */
DDC_VPBEStatus DDC_VPBEAddQueue(DDC_VPBEQueueType       qTypeAQ,
                                Ptr                     handle,
                                FVID_Frame * fbAddressAQ);

/**
 *  \brief      This function is used to remove frame buffers from either the
 *              ACTIVE queue or the FREE queue.
 *
 *              On exit this fucnition returns the status of the operation that
 *              was performed on the queue.
 *
 *  \param      qTypeDQ             [IN]    Index of the Queue where the
 *                                          frame buffer has to be removed.
 *
 *                                          ACTIVE and FREE queues are
 *                                          the 2 types of queues used.
 *
 *  \param      handle              [IN]     Handle of VPBE channel
 *
 *  \param      fbAddressDQ         [IN]    Pointer to the frame buffer
 *                                          that will be updated in this
 *                                          address.
 *
 *  \returns    Pass or Fail status
 */
DDC_VPBEStatus DDC_VPBEDelQueue(DDC_VPBEQueueType        qTypeDQ,
                                Ptr                      handle,
                                FVID_Frame ** fbAddressDQ);
/**
 *  \brief      This function is used get the next frame buffer address to be
 *              used for Source address for display .
 *
 *              On exit this funcition returns pointer to the frame buffer that
 *              can be updated in the DMA address
 *
 *  \param      fbGN            [IN]        Frame buffer object which
 *                                          contains the queue for a
 *                                          particular handle.
 *
 *  \returns    Address of the next valid frame buffer
 *
 */
FVID_Frame * DDC_VPBEGetNxtFBAddress(const DDC_VPBEFrameBufferObject
                                                          *fbGN);

/**
 *  \brief      Back end ISR.This will be invoked at every VSYNC interrupt.
 *
 *              If the VENC is in INTERLACED mode then next valid frame buffer
 *              pointer will be updated when an EVEN field has been displayed.
 *
 *              If the VENC is in PROGRESSIVE mode then the next valid frame
 *              buffer address will be updated in every VSYNC interrupt
 *
 *              At each VSYNC interrupt the callback function will be called.
 *
 *  \param      arg            [IN]         Future Use.
 *
 *  \returns    NONE
 */
void DDC_VPBEIsr(Uint32 arg);

/**
  * \brief      This function is used to call the application Callback function
  *             from within the driver's context. This function returns the
  *             state of the current field information to the application.
  *
  * \param      idCACF                  [IN]        Vpss id
  *
  * \param      currentFieldCACF        [IN]        Vpss current field
  *
  */
void DDC_VPBE_AppCallbackFunction(PSP_VPBE_Id       idCACF,
                                  DDC_VPBEFieldId   currentFieldCACF);

/**
  * \brief      It will validate the window boundary by cheking leftMargin,
  *             topMargin,Width and Height.This function is used to check
  *             if the target plane boundary parameters are within the source
  *             plane boundary parameters.
  *
  * \param      srcleftMargin           [IN]       LeftMargin of Video 0 Window
  *
  * \param      srctopMargin            [IN]       TopMargin of Video 0 Window
  *
  * \param      srcwidth                [IN]       Width of Video 0 Window
  *
  * \param      srcheight               [IN]       Height of Video 0 Window
  *
  * \param      trgleftMargin           [IN]       LeftMargin of Other windows
  *
  * \param      trgtopMargin            [IN]       TopMargin of Other windows
  *
  * \param      trgwidth                [IN]       Width of Other windows
  *
  * \param      trgheight               [IN]       Height of Other windows
  *
  * \param      trgPlaneHandle          [IN]       Handle for trigger boundary
  *                                                params
  *
  * \returns    Success or failure status
  */
Bool DDC_VPBEIsWithinWindowBoundary(Uint32 srcleftMargin,Uint32 srctopMargin,Uint32 srcwidth,Uint srcheight,
                                    Uint32 trgleftMargin,Uint32 trgtopMargin,Uint32 trgwidth,Uint trgheight);

DDC_VPBEStatus DDC_VPBEAllocFrameBuffer(Ptr handle,
                                        FVID_Frame ** allocFrameBuffer);

DDC_VPBEStatus DDC_VPBEFreeFrameBuffer(Ptr handle,
                                       FVID_Frame ** allocFrameBuffer);

void DDC_VPBEUpdateFbAdress(PSP_VPBE_Id idUN);
void DDC_VPBEUpdateFreeQueue(PSP_VPBE_Id idUN);

DDC_VPBEStatus DDC_VPBEStopEngine(Ptr handle);

DDC_VPBEStatus DDC_VPBEStartEngine(Ptr handle);
DDC_VPBEStatus DDC_ConfigextVIDENCParams(Ptr handle, Uint32 cmd ,Ptr cmdArg);

#ifdef __cplusplus
}
#endif

#endif /* _DDC_VPBE_H */
