/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file   dda_vpfe.c
 *
 *  \brief  VPFE implementation at DDA Layer
 *
 *  This file contains the implementation of the vpfe driver to utilize the
 *  serivces of the vpfe device driver.
 *
 *  (C) Copyright 2010, Texas Instruments, Inc
 *
 *  \note       Set tabstop to 4 (:se ts=4) while viewing this file in an
 *              editor
 *
 *  \author     Maulik Desai
 *
 *  \version    1.0     Created
 */

/******************************************************************************
                            Header File Inclusion
******************************************************************************/

#include <string.h>

#include <ti/sdo/pspdrivers/drivers/vpfe/psp_vpfe.h>
#include <ti/sdo/pspdrivers/drivers/vpfe/src/ddc_vpfe.h>
#include <ti/sdo/pspdrivers/drivers/vpfe/src/ddc_vpfeCcdc.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osSem.h>

/******************************************************************************
                                Functions
******************************************************************************/
/**
  * \brief      Create the given vpfe driver (instance)
  */
PSP_Result  PSP_VPFECreate(Uint32 instanceId)
{
    PSP_Result resultCR = PSP_E_INVAL_PARAM;
    DDC_VPFEDeviceObject * vpfeDeviceHandleCreate =
                        (DDC_VPFEDeviceObject * )DDC_VPFEGetDeviceHandle();

    /* Validate instance ID */
    if((NULL == vpfeDeviceHandleCreate)||(PSP_VPFE_NUM_INSTANCES <= instanceId))
    {
        resultCR = PSP_E_INVAL_PARAM;
    }
    /* Check state of driver */
    else if (VPSS_DELETED != vpfeDeviceHandleCreate->state)
    {
        resultCR = PSP_E_INVAL_PARAM;
    }
    else
    {
        /*Set the Interrupt Number */
        DDC_VPFESetInterruptNumber();
        /*Set the Base Address of Register */
        DDC_VPFERegisterOverlaying();
        /*Update the Device state as Created */
        vpfeDeviceHandleCreate->state           = VPSS_CREATED;
        vpfeDeviceHandleCreate->ccdc.state      = DDC_VPFE_CLOSED;
        resultCR = PSP_SOK;
    }
    return (resultCR);
}

/**
  * \brief  Delete vpfe driver instance
  */
PSP_Result PSP_VPFEDelete(Uint32 instanceId)
{
    PSP_Result resultDL = PSP_E_INVAL_PARAM;
    /* Get the Channel Handle */
    DDC_VPFEDeviceObject * vpfeDeviceHandleDelete =
                        (DDC_VPFEDeviceObject * )DDC_VPFEGetDeviceHandle();
    /* Validate instance ID */
    if((NULL == vpfeDeviceHandleDelete)||(PSP_VPFE_NUM_INSTANCES <= instanceId))
    {
        resultDL = PSP_E_INVAL_PARAM;
    }
    /* Check state of driver */
    else if((VPSS_CREATED != vpfeDeviceHandleDelete->state) &&
            (VPSS_CLOSED  != vpfeDeviceHandleDelete->state))
    {
        resultDL = PSP_E_INVAL_PARAM;
    }
    else
    {
        /* set state to deleted */
        vpfeDeviceHandleDelete->state = VPSS_DELETED;
        memset(vpfeDeviceHandleDelete,0,sizeof(DDC_VPFEDeviceObject));
        resultDL = PSP_SOK;
    }
    return (resultDL);
}
/**
  * \brief      Open Instance of the vpss device
  */
PSP_Handle  PSP_VPFEOpen(Uint32 instanceId,PSP_VPFEChannelParams * chanParams)
{
    PSP_Handle handleToReturn   = NULL;
    Ptr Params                  = NULL;
    PSP_VPFE_Id ChannelId;
    /*Get the Channel Handle */
    DDC_VPFEDeviceObject * vpfeDeviceHandleOpen =
                            (DDC_VPFEDeviceObject * )DDC_VPFEGetDeviceHandle();

    if(((NULL== chanParams)||(NULL== chanParams->params))||
                               (NULL== vpfeDeviceHandleOpen))
    {
      handleToReturn  = NULL;
    }
    /*Check parameters not NULL */
    else
    {
        ChannelId = chanParams->id;
        Params    = chanParams->params;
        /* Validate instance ID */
        if (((PSP_VPFE_NUM_INSTANCES <= instanceId)||(NULL ==Params))||
            ((PSP_VPFE_CCDC > ChannelId)&&
             (PSP_VPFE_MAX_INTERFACES_SUPPORTED < ChannelId)))
        {
            handleToReturn = NULL;
        }
        /* Check state of driver */
        else if (((VPSS_CREATED != vpfeDeviceHandleOpen->state) &&
                 (VPSS_CLOSED   != vpfeDeviceHandleOpen->state))&&
                 (VPSS_OPENED   != vpfeDeviceHandleOpen->state))
        {
            handleToReturn = NULL;
        }
        else
        {   /*Configure the Channel and returns Channel Handle */
            handleToReturn = (PSP_Handle)DDC_VPFEOpenHandle(ChannelId,Params);
            if(NULL != handleToReturn)
            {
                /*Update the State as Opened */
                vpfeDeviceHandleOpen->state            = VPSS_OPENED;
                vpfeDeviceHandleOpen->vpssInstanceId = instanceId;
            }
        }
    }
    return (handleToReturn);
}

/**
  * \brief      Close the VPSS channel with the specified handle
  */
PSP_Result PSP_VPFEClose(PSP_Handle handle)
{
    PSP_Result resultCL   = PSP_E_INVAL_PARAM;
    DDC_VPFEStatus result = DDC_VPFE_E_FAIL;
    DDC_VPFEDeviceObject* vpfeDeviceHandle =
                          (DDC_VPFEDeviceObject * )DDC_VPFEGetDeviceHandle();
        /* Check the input parameters */
    if((NULL == handle)||(NULL == vpfeDeviceHandle))
    {
        resultCL = PSP_E_INVAL_PARAM;
    }
    else
    {
        result = DDC_VPFECloseHandle(handle);
        if(DDC_VPFE_SOK == result)
        {
            vpfeDeviceHandle->state = VPSS_CLOSED;
            resultCL = PSP_SOK;
        }
        else if (DDC_VPFE_E_RESOURCES == result)
        {
            resultCL = PSP_E_RESOURCES;
        }
        else
        {
            resultCL = PSP_E_INVAL_PARAM;
        }
    }
    return (resultCL);
}

PSP_Result PSP_VPFESubmitRequest(PSP_Handle                 handle,
                                 Uint32      cmd,
                                 Ptr                        cmdArg)
{
    FVID_Frame * dequeuedFrameBuffer     = NULL;
    FVID_Frame * surfParamPtr            = NULL;

    PSP_Result resultSR = PSP_E_INVAL_PARAM;
    DDC_VPFEStatus result = DDC_VPFE_E_FAIL;

    /*Check the Parameters and check Channel handle is closed */
    if(((NULL == handle) || (NULL == cmdArg ))||
       (DDC_VPFE_SOK == DDC_VPFEIsChannelHandleClosed(handle)))
    {
        resultSR = PSP_E_INVAL_PARAM;
    }
    else
    {
        switch (cmd)
        {
            case FVID_DEQUEUE:
            {
                dequeuedFrameBuffer = (FVID_Frame *)cmdArg;
                /*Dequeue the Frame Buffer */
                if(DDC_VPFE_SOK == DDC_VPFEDeQueueFrameBuffer(handle,
                                                          &dequeuedFrameBuffer))
                {
                    if(NULL != dequeuedFrameBuffer)
                    {
                      *(FVID_Frame **)cmdArg = dequeuedFrameBuffer;
                      resultSR = PSP_SOK;
                    }
                }
            }
            break;
            case FVID_QUEUE:
            {
                surfParamPtr = (FVID_Frame *)cmdArg;
                result = DDC_VPFEQueueFrameBuffer(handle,surfParamPtr);
                /* Add FrameBuffer in Queue */
                if(DDC_VPFE_SOK == result)
                {
                    resultSR = PSP_SOK;
                }
                else if(DDC_VPFE_E_RESOURCES == result)
                {
                    resultSR = PSP_E_RESOURCES;
                }
                else
                {
                    resultSR = PSP_E_INVAL_PARAM;
                }
            }
            break;
            default:
            {
                resultSR = PSP_E_INVAL_PARAM;
            }
            break;
        }
    }
    return resultSR;
}
/**
  * \brief  I/O Control of the VPFE device
  */
/*ARGSUSED */
PSP_Result  PSP_VPFEIoctl( PSP_Handle               handle,
                           Uint32                   cmd,
                           Ptr                      cmdArg,
                           Ptr                      params)
{
    PSP_Result resultIOTL                   = PSP_SOK;
    PSP_VPFECCDC_BlackAdj *BlackAdj         = NULL;
    PSP_VPFE_CCDC_Fpc    *FpcSettings       = NULL;
    FVID_Frame * allocFramebuffer           = NULL;
    DDC_VPFEStatus  result                  = DDC_VPFE_E_FAIL;

    /* Check the Parameters               */
    /* Check the Channel Handle is Closed */
    if(((NULL == handle))||
       (DDC_VPFE_SOK == DDC_VPFEIsChannelHandleClosed(handle)))
    {
        resultIOTL = PSP_E_INVAL_PARAM;
    }
    else
    {
        switch (cmd)
        {
            case PSP_VPFE_IOCTL_CCDC_VALIDATE_BUFFER:
            {
                /*Validate the Frame Buffer */
                if(DDC_VPFE_SOK != DDC_VPFEValidateFrameBuffer(cmdArg))
                {
                    resultIOTL = PSP_E_INVAL_PARAM;
                }
                else
                {
                    resultIOTL = PSP_SOK;
                }
             }
             break;
             case PSP_VPFE_IOCTL_CCDC_BLACK_ADJUST:
             {
                BlackAdj = (PSP_VPFECCDC_BlackAdj *)cmdArg;
                if(DDC_VPFE_SOK != DDC_VPFEBlackAdjust(handle,BlackAdj))
                {
                    resultIOTL = PSP_E_INVAL_PARAM;
                }
                else
                {
                    resultIOTL = PSP_SOK;
                }
             }
             break;
             case PSP_VPFE_IOCTL_CCDC_FPC:
             {
                FpcSettings = (PSP_VPFE_CCDC_Fpc *)cmdArg;
                if (DDC_VPFE_SOK != DDC_VPFEFpc(handle,FpcSettings))
                {
                    resultIOTL = PSP_E_INVAL_PARAM;
                }
                else
                {
                    resultIOTL = PSP_SOK;
                }
             }
             break;
             case  PSP_VPFE_IOCTL_START_CCDC:
             {
                if(DDC_VPFE_SOK!= DDC_VPFEStartEngine(handle))
                {
                    resultIOTL = PSP_E_INVAL_PARAM;
                }
                else
                {
                    resultIOTL = PSP_SOK;
                }
             }
             break;
             case PSP_VPFE_IOCTL_STOP_CCDC:
             {
                if(DDC_VPFE_SOK != DDC_VPFEStopEngine(handle))
                {
                    resultIOTL = PSP_E_INVAL_PARAM;
                }
                else
                {
                    resultIOTL = PSP_SOK;
                }
             }
             break;
            case PSP_VPFE_IOCTL_FIELD_SEPARATE:
            {
                if(DDC_VPFE_SOK != DDC_VPFEFieldSeparate(handle))
                {
                    resultIOTL = PSP_E_INVAL_PARAM;
                }
                else
                {
                    resultIOTL = PSP_SOK;
                }
            }
            break; 
            case PSP_VPFE_IOCTL_FIELD_INTERLACED:
            {
                if(DDC_VPFE_SOK != DDC_VPFEFieldInterlaced(handle))
                {
                    resultIOTL = PSP_E_INVAL_PARAM;
                }
                else
                {
                    resultIOTL = PSP_SOK;
                }
            }
            break; 
            case PSP_VPFE_IOCTL_CONFIG_VDINT:
            { 
                if(DDC_VPFE_SOK != DDC_VPFEConfigVDINT(handle,cmdArg))
                {
                    resultIOTL = PSP_E_INVAL_PARAM;
                }
                else
                {
                    resultIOTL = PSP_SOK;
                }               
             }
            break;   
            case FVID_ALLOC_BUFFER :
            {
                result = DDC_VPFEAllocFrameBuffer(handle,&allocFramebuffer);
                if(DDC_VPFE_SOK == result)
                {
                  *(FVID_Frame **)cmdArg = allocFramebuffer;
                  resultIOTL = PSP_SOK;
                }
                else if(DDC_VPFE_NO_MEMORY == result)
                {
                    resultIOTL = PSP_E_NO_MEMORY;
                }
                else
                {
                    resultIOTL = PSP_E_INVAL_PARAM;
                }
            }
            break;
            case FVID_FREE_BUFFER :
            {
                if(DDC_VPFE_SOK == DDC_VPFEFreeFrameBuffer(handle,
                                                           (FVID_Frame **)cmdArg))
                {
                    resultIOTL = PSP_SOK;
                }
            }
            break;
            default :
            {
                if(DDC_VPFE_SOK!= DDC_ConfigextVDParams(handle,cmd,cmdArg))
                {
                    resultIOTL = PSP_E_INVAL_PARAM;
                }
                else
                {
                    resultIOTL = PSP_SOK;
                }
            }
            break;
        }
    }
    return (resultIOTL);
}
