/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file   dda_vpfeIom.c
 *
 *  \brief  VPFE IOM layer Functions
 *
 *  This file contains the Implementation details of IOM layer functions
 *
 *  (C) Copyright 2010, Texas Instruments, Inc
 *
 *  \note       Set tabstop to 4 (:se ts=4) while viewing this file in an
 *              editor
 *
 *  \author     Maulik Desai
 *
 *  \version    0.1     created
 */

/******************************************************************************
                            Header File Inclusion
******************************************************************************/
#include <string.h>
#include <std.h>
#include <iom.h>
#include <clk.h>

#include <ti/sdo/pspdrivers/system/dm6437/bios/evmDM6437/video/fvid.h>
#include <ti/sdo/pspdrivers/drivers/vpfe/psp_vpfe.h>
#include <ti/sdo/pspdrivers/drivers/vpfe/src/psp_vpfeApi.h>
#include <ti/sdo/pspdrivers/drivers/vpfe/src/dda_vpfeIom.h>


#ifdef VPFE_INSTRUMENTATION_ENABLED
    #include <ti/sdo/pspdrivers/pal_os/bios/psp_log.h>
#endif /* VPFE_INSTRUMENTATION_ENABLED */

/*-----------------------------------------------------------------------------
                        Mini Driver Function interface table
-----------------------------------------------------------------------------*/
/**
  * \brief Mini Driver Function interface table called by BIOS Itself
  *
  */

IOM_Fxns VPFEMD_FXNS =
{
    &VPFE_mdBindDev,
    &VPFE_mdUnBindDev,
    &VPFE_mdControlChan,
    &VPFE_mdCreateChan,
    &VPFE_mdDeleteChan,
    &VPFE_mdSubmitChan,
};

/*-----------------------------------------------------------------------------
                        IOM Error code look up table
-----------------------------------------------------------------------------*/
/**
 * The following mapping shows the mapping done from PSP_Result to IOM error
 * code.
 *
 * PSP_SOK                  (0)   IOM_COMPLETED            0
 * PSP_SINPROGRESS          (1)   IOM_PENDING              1
 * PSP_E_DRIVER_INIT        (-1)  IOM_ETIMEOUTUNREC      -11
 * PSP_E_NO_MEMORY          (-2)  IOM_EALLOC              -5
 * PSP_E_RESOURCES          (-3)  IOM_EALLOC              -5
 * PSP_E_INVAL_STATE        (-4)  IOM_EBADMODE            -7
 * PSP_E_INVAL_PARAM        (-5)  IOM_EBADARGS           -10
 * PSP_E_NOT_SUPPORTED      (-6)  IOM_ENOTIMPL            -9
 * PSP_E_IO_CANCEL_FAIL     (-7)  IOM_ETIMEOUT            -2
 * PSP_E_FIFO_NOT_ENABLED   (-8)  IOM_EBADMODE            -7
 * PSP_E_INVALID_MODE       (-9)  IOM_EBADMODE            -7
 * PSP_E_INVALID_CMD        (-10) IOM_EBADARGS           -10
 *
 */
static Int iomErrorCodeList[12] =
{
    IOM_EBADARGS,
    IOM_EBADMODE,
    IOM_EBADMODE,
    IOM_ETIMEOUT,
    IOM_ENOTIMPL,
    IOM_EBADARGS,
    IOM_EBADMODE,
    IOM_EALLOC,
    IOM_EALLOC,
    IOM_ETIMEOUTUNREC,
    IOM_COMPLETED,
    IOM_PENDING
};

/*-----------------------------------------------------------------------------
                                IOM Port Object
-----------------------------------------------------------------------------*/
/**
  * \brief IOM Port Object
  *
  */
static VPFEPortObject vpfe_ports;
/**
 *  \brief  PSP error code converter
 *
 * This function converts PSP error codes to appropriate IOM error code.
 *
 * \param  pspCode          [IN]    PSP Error code.
 *
 * \return IOM ERROR CODE   [OUT]   Equivalent IOM error code. For unrecoganized
 *                                  error code IOM_EBADIO is returned.
 *
 */
static Int getIomErrorCode(PSP_Result pspCode);

/******************************************************************************
                                Functions
******************************************************************************/
/**
  *\ fuction will get the IOM Error Code
  *
  */
static Int getIomErrorCode(PSP_Result pspCode)
{
    Int iomCode = pspCode + 10;

    if(iomCode < 12)
    {
        iomCode = iomErrorCodeList[iomCode];
    }
    else
    {
        iomCode = IOM_EBADIO;
    }

    return(iomCode);
}

/*-----------------------------------------------------------------------------
                            VPFE_mdBindDev()

-----------------------------------------------------------------------------*/

/**
  * \brief  VPFE_mdBindDev() is called by DEV_init() to bind and initialize
  *         the instance
  */
/*ARGSUSED */
Int VPFE_mdBindDev(Ptr *devp, Int devid, Ptr devParams)
{
    Int     statusBnd   = IOM_COMPLETED;
    Uint32  ldevid      = (Uint32)devid;

    DDC_VPFEDeviceObject * vpfeDeviceHandleCreate =
                        (DDC_VPFEDeviceObject * )DDC_VPFEGetDeviceHandle();

    /*Check the Parameters not NULL */
    if((NULL == devp) || ((Uns)ldevid >= PSP_VPFE_NUM_INSTANCES))
    {
        statusBnd =  (IOM_EBADARGS);
    }
    else
    {
       /* Setting initial global values to zero */
        memset(&vpfe_ports,0,sizeof(VPFEPortObject));
        memset(vpfeDeviceHandleCreate,0,sizeof(DDC_VPFEDeviceObject));

        /*This function will initialize and create an instance of vpfe */
        statusBnd = PSP_VPFECreate((Uint32)devid);

        if(PAL_SOK != statusBnd)
        {
            /*Generate the IOM Error code based on PSP error */
            statusBnd = getIomErrorCode(statusBnd);
        }
        else
        {

           if(NULL != vpfeDeviceHandleCreate)
           {
               /* Update VPFE port object */
                vpfe_ports.dda_PortObj      = vpfeDeviceHandleCreate;
                vpfe_ports.vpssPortNumber   = ldevid;
                vpfe_ports.state            = VPSS_CREATED;
                *devp                       = &vpfe_ports;
           }
        }
    }
    return(statusBnd);
}

/*-----------------------------------------------------------------------------
                                VPFE_mdUnBindDev
-----------------------------------------------------------------------------*/

/**
  * \brief VPFE_mdUnBindDev() is called by DEV_init() to unbind and de-init
  *        the instance
  */
Int VPFE_mdUnBindDev(Ptr devp)
{
    Int             statusUnBnd     = IOM_COMPLETED;
    VPFEPortObject *portUnBnd       = NULL;
    /*Check the parameter whether its NULL */
    if(NULL == devp)
    {
        statusUnBnd = IOM_EBADARGS;
    }
    else
    {
        portUnBnd = (VPFEPortObject *)devp;
        /* Check the State of VPSS Port */
        if ((VPSS_CREATED != portUnBnd->state) &&
            (VPSS_CLOSED  != portUnBnd->state))
        {
            statusUnBnd =  IOM_EBADMODE;
        }
        /* Check the instance number with maximum limits */
        else if (portUnBnd->vpssPortNumber >= PSP_VPFE_NUM_INSTANCES )
        {
            statusUnBnd = IOM_EBADARGS;
        }
        else
        {
            /*This function de-initialize and closes handle of a channel */
            statusUnBnd = PSP_VPFEDelete(portUnBnd->vpssPortNumber);
            /* Check status for success */
            if(PAL_SOK != statusUnBnd)
            {
                /*Generate the IOM Error code based on PSP error */
                statusUnBnd = getIomErrorCode(statusUnBnd);
            }
            else
            {
                portUnBnd->state = VPSS_DELETED;
            }
        }
    }
    return(statusUnBnd);
}

/*-----------------------------------------------------------------------------
                                VPFE_mdCreateChan
    Creates the channel for the data transfer by setting up some params
-----------------------------------------------------------------------------*/

/**
  * \brief Creates the channel for the data transfer by setting up some params
  *
  */
/*ARGSUSED */
Int VPFE_mdCreateChan(Ptr *chanp, Ptr devp, String name, Int mode,
                      Ptr chanParams, IOM_TiomCallback cbFxn, Ptr cbArg)

{
    Int                         statusCR    = IOM_COMPLETED;
    VPFEPortObject*             portCR      = NULL;
    VPFEChannelObject*          channel     = NULL;
    PSP_VPFEChannelParams*      param       = NULL;
    /* Check the Parameters whether its NULL */
    if((((NULL == chanp) || (NULL == devp)) || (NULL == chanParams)) ||
       ((NULL == cbFxn) || (NULL == cbArg)))
    {
        statusCR = IOM_EBADARGS;
    }
    else
    {
        portCR = (VPFEPortObject *)devp;
        /* Check the State of the Port */
        if (((VPSS_CREATED  != portCR->state)  &&
             (VPSS_CLOSED   != portCR->state)) &&
             (VPSS_OPENED   != portCR->state))
        {
            statusCR =  (IOM_EBADMODE);
        }
        else
        {
            param = (PSP_VPFEChannelParams *)chanParams;
            /* Check Channel is inuse or not */
            if(FALSE == portCR->chan[param->id].inUse)
            {
                /*Channel is not inuse then assign the channel */
                channel = (VPFEChannelObject *)&(portCR->chan[param->id]);
            }
            else
            {
                statusCR = IOM_EINUSE;
            }
            if ( ((IOM_INPUT != mode) && (IOM_OUTPUT != mode) )
                && (IOM_INOUT != mode) )
            {
                statusCR = IOM_EBADMODE;
            }
            else
            {
                channel->mode = mode;
            }
        }
    }
    if(IOM_COMPLETED == statusCR)
    {
        /* Initialize the channel and Create the channel for CCDC module */
        channel->ddc_Handle = (PSP_Handle)PSP_VPFEOpen(portCR->vpssPortNumber,chanParams);
        /* Check whether the Channel Handle is NULL */
        if(NULL == channel->ddc_Handle)
        {
            statusCR = IOM_EBADARGS;
        }
        else
        {
            /*Update the State as opened if Channel Handle not NULL */
            portCR->state = VPSS_OPENED;
            channel->cbFxn = cbFxn;
            channel->cbArg = cbArg;
            /*Sets Channel is inuse */
            channel->inUse = TRUE;
            /*Increment the Number of Opens */
            ++portCR->numOfOpens;
            /* sets for back referencing */
            channel->port = (VPFEPortObject *)portCR;
            *chanp = channel;
        }
    }

    return(statusCR);
}

/*-----------------------------------------------------------------------------
                                vpss_mdDeleteChan
-----------------------------------------------------------------------------*/
/**
  * \brief  Deletes an instance of the VPFE channel.
  *
  */
Int VPFE_mdDeleteChan(Ptr chanp)
{
    Int                 statusDL    = IOM_COMPLETED;
    VPFEChannelObject*  chanDL      = NULL;
    VPFEPortObject*     portDL      = NULL;

    /*Check Channel whether its NULL */
    if(NULL == chanp)
    {
        statusDL = IOM_EBADARGS;
    }
    else
    {
        chanDL = (VPFEChannelObject *)chanp;
        portDL = (VPFEPortObject *)chanDL->port;
        /*Check the Channel State,port handle and Channel inuse */
        if((FALSE == chanDL->inUse) ||
           ((NULL == portDL) || (VPSS_OPENED != portDL->state)))
        {
            statusDL = IOM_EBADARGS;
        }
    }/*else*/

    if(IOM_COMPLETED == statusDL)
    {
        /*Function closes the Front End Module Channel */
        statusDL = PSP_VPFEClose(chanDL->ddc_Handle);
        /*Check the Status if success */
        if(PAL_SOK != statusDL)
        {
            /*Generate the IOM error code equivalent to PSP error */
            statusDL = getIomErrorCode(statusDL);
        }
        else
        {
            /*Mark Channel is not inuse */
            chanDL->inUse = FALSE;
            /* Updates the state and makes the channel port void*/
            chanDL->port = NULL;
            /* If all channels are closed then update the state to closed */
            if (0 == --portDL->numOfOpens)
            {
                portDL->state = VPSS_CLOSED;
            }
            chanp = &chanDL;
        }
    }/*if status ==IOM_COMPLETED */

    return(statusDL);
}

/*-----------------------------------------------------------------------------
                                VPFE_mdControlChan
        All dynamic configurations of a mini driver is done by this function
-----------------------------------------------------------------------------*/
/**
  *\ brief  mdcontrolchan is used to config the ioctl operation
  *
  */
Int VPFE_mdControlChan(Ptr chanp, Uns cmd, Ptr arg)
{
    Int                  statusCNTRL = IOM_COMPLETED;
    VPFEChannelObject  * chanCNTRL  = NULL;
    VPFEPortObject     * portCNTRL  = NULL;
    /*Check Channel Handle is NULL */
    if((NULL == chanp))
    {
        statusCNTRL = IOM_EBADARGS;
    }
    else
    {
        chanCNTRL = (VPFEChannelHandle)chanp;
        portCNTRL = (VPFEPortObject* )chanCNTRL->port;
        /*Check the port and port State */
        if((NULL == portCNTRL) || (VPSS_OPENED != portCNTRL->state))
        {
            statusCNTRL = IOM_EBADMODE;
        }
        else
        {

            /*DDA Layer function for Dynamic Configuration of  Channels */
            statusCNTRL = PSP_VPFEIoctl(chanCNTRL->ddc_Handle,cmd,arg,NULL);
            /*Check the status if success */
            if (PSP_SOK != statusCNTRL)
            {
                /* IOM error code equivalent to PSP error code */
                statusCNTRL = getIomErrorCode(statusCNTRL);
            }

        }/* else part of port Check */
    }/*else part of channel Handle check */

    return(statusCNTRL);
}

/*-----------------------------------------------------------------------------
                                VPFE_mdSubmitChan
-----------------------------------------------------------------------------*/
/**
  * \brief VPFE_mdSubmitChan is used to submit the IO request.
  *
  */
Int VPFE_mdSubmitChan(Ptr chanp, IOM_Packet *packet)
{
    Int                   statusSBMT = IOM_COMPLETED;
    VPFEChannelObject   * chanSBMT   = NULL;
    VPFEPortObject      * portSBMT   = NULL;
    /*Check the formal arguments whether NULL */
    if( (NULL == chanp )||(NULL == packet) )
    {
        statusSBMT = IOM_EBADARGS;
    }
    else
    {
        chanSBMT = (VPFEChannelHandle)chanp;

        /*Check the port and address if NULL */
        if ((NULL == packet->addr) || (NULL == chanSBMT->port))
        {
            statusSBMT = IOM_EBADARGS;
        }
        else
        {

#ifdef VPFE_INSTRUMENTATION_ENABLED /* Receive */
            PSP_LOG_EVENT(  &DVTEvent_Log,
                            "VPFE",
                            PSP_DVT_DESC(
                                PSP_DVT_eFUNC_START,
                                PSP_DVT_dCHANNEL_ID_INST,
                                PSP_DVT_dPACKET_ID,
                                PSP_DVT_dSIZE_BYTES),
                            CHANNELID_INSTANCEID(
                                0,
                                0),
                            packet->addr,
                            packet->size
                            );
#endif /*VPFE_INSTRUMENTATION_ENABLED */

            portSBMT = (VPFEPortObject *)chanSBMT->port;
            /*Check whether channel is not inuse and datapacket not NULL */
            if(((NULL == portSBMT)||(FALSE == chanSBMT->inUse))||
                  (NULL != chanSBMT->dataPacket))
            {
                statusSBMT = IOM_EBADMODE;
            }
            else
            {
                /*Check the State of port */
                if(VPSS_OPENED != portSBMT->state)
                {
                    statusSBMT = IOM_EBADMODE;
                }
                else
                {
                    chanSBMT->dataPacket = packet;
                    if(FVID_QUEUE == packet->cmd )
                    {
                        /*DDA Layer function is called to submit the data request */
                        statusSBMT = PSP_VPFESubmitRequest(chanSBMT->ddc_Handle,
                                                           FVID_QUEUE,
                                                                                           packet->addr);
                        /*Check the status if succes */
                        if(PSP_SOK != statusSBMT){
                            /* IOM error code equivalent to PSP error code */
                            statusSBMT = getIomErrorCode(statusSBMT);
                        }
                    }
                    if(FVID_DEQUEUE == packet->cmd)
                    {
                       /*DDA Layer function to submit the data request */
                        statusSBMT = PSP_VPFESubmitRequest(chanSBMT->ddc_Handle,
                                                           FVID_DEQUEUE,
                                                           packet->addr);
                        /*Check the status if succes */
                        if(PSP_SOK != statusSBMT)
                        {
                            /* IOM error code equivalent to PSP error code */
                            statusSBMT = getIomErrorCode(statusSBMT);
                        }
                    }
                    if(FVID_EXCHANGE == packet->cmd)
                    {
                        statusSBMT = PSP_VPFESubmitRequest(chanSBMT->ddc_Handle,
                                                           FVID_QUEUE,
                                                           (Ptr)(*(Uint32 *)(packet->addr)));
                        /*Check the status if succes */
                        if(PSP_SOK != statusSBMT)
                        {
                            /* IOM error code equivalent to PSP error code */
                            statusSBMT = getIomErrorCode(statusSBMT);
                        }
                        if(PSP_SOK == statusSBMT)
                        {
                            statusSBMT = PSP_VPFESubmitRequest(chanSBMT->ddc_Handle,
                                                               FVID_DEQUEUE,
                                                               packet->addr);
                            /*Check the status if succes */
                            if(PSP_SOK != statusSBMT)
                            {
                                /* IOM error code equivalent to PSP error code */
                                statusSBMT = getIomErrorCode(statusSBMT);
                            }
                        }
                    }
                }
                /*Make dataPacket to be NULL */
                chanSBMT->dataPacket = NULL;
            }/*Check state of the port */

#ifdef VPFE_INSTRUMENTATION_ENABLED /* Receive */
            PSP_LOG_EVENT(  &DVTEvent_Log,
                            "VPFE",
                            PSP_DVT_DESC(
                                PSP_DVT_eFUNC_END,
                                PSP_DVT_dCHANNEL_ID_INST,
                                PSP_DVT_dPACKET_ID,
                                PSP_DVT_dSIZE_BYTES),
                            CHANNELID_INSTANCEID(
                                0,
                                0),
                            packet->addr,
                            0
                            );
#endif /*VPFE_INSTRUMENTATION_ENABLED */

        }/*check of Channel and data packet */
    }/*else */

    return statusSBMT;
}

 /**
   * \brief VPFE Time Stamp Function
   */
extern void _DDA_VPFEGetTimeStamp(Ptr timeStamp)
{
    Uint32 * time = NULL ;

    if(NULL != timeStamp)
    {
        time = (Uint32*)timeStamp;
        if(NULL != time)
        {
            *time = CLK_getltime();
        }
    }
}
