/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file   dda_vpfeIom.h
 *
 *  \brief  VPFE IOM  layer definition
 *
 *  This file contains the interfaces, data types and symbolic definitions
 *  that are needed by the application to utilize the serivces of the VPFE
 *  device driver.
 *
 *  (C) Copyright 2010, Texas Instruments, Inc
 *
 *  \note       Set tabstop to 4 (:se ts=4) while viewing this file in an
 *              editor
 *
 *  \author     Maulik Desai
 *
 *  \version    0.1     Created
 */

#ifndef _DDA_VPFEIOM_H
#define _DDA_VPFEIOM_H

#include <ti/sdo/pspdrivers/drivers/vpfe/src/ddc_vpfe.h>

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
                               Macros
******************************************************************************/
/**
  * \brief  max number of channel supported by interfaces.
  *
  */
#define NUMCHANS        PSP_VPFE_MAX_INTERFACES_SUPPORTED

/******************************************************************************
                           Data-Structures
******************************************************************************/
/*-----------------------------------------------------------------------------
                          IOM Function Table
            Driver function table to be used by applications.
-----------------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C" {
#endif
    extern IOM_Fxns VPFEMD_FXNS;
#ifdef __cplusplus
}
#endif /* extern "C" */

/**
  * \brief  structure defines IOM Channel
  *         Channel structure used by the IOM layer
  */
typedef struct VPFE_IOM_CHANNEL_OBJ
{
    Bool                        inUse;
    /**< channel in-use                     */
    Int                         mode;
    /**< INPUT or OUTPUT/INOUT              */
    struct VPFE_IOM_PORT_OBJ    *port;
    /**< Reverse pointer to Port            */
    IOM_Packet                  *dataPacket;
    /**< current active IOP                 */
    IOM_TiomCallback            cbFxn;
    /**< To be called from ISR              */
    Ptr                         cbArg;
    /**< argument for cbFxn()               */
    PSP_Handle                  ddc_Handle;
    /**< Handle CCDC module                         */
} VPFEChannelObject , *VPFEChannelHandle;

/**
  * \brief structure defines the IOM Port
  *        Port structure used by the IOM layer
  */
typedef struct VPFE_IOM_PORT_OBJ
{
    Uns                        vpssPortNumber;
    /**< Instance number                          */
    VPFEChannelObject          chan[NUMCHANS];
    /**< channels for the VPFE                    */
    DDC_VPFEDeviceObject       *dda_PortObj;
    /**< Underlying DDA Structure                 */
    vpssState                  state;
    /**< state of VPSS driver                     */
    Uint                       numOfOpens;
    /**< number of channel opens for VPFE driver  */
} VPFEPortObject, *VPFEPortHandle;

/******************************************************************************
                                Functions
******************************************************************************/
/**
 *  \brief Initialize a given VPFE driver (instance)
 *
 *  It performs register map overlaying and software book-keeping.It register
 *  the interrupt Handler.
 *
 *  \param  devp        [IN]    Void pointer passed by the GIO Layer to
 *                              store the address of the port structure
 *
 *  \param  devid       [IN]    VPSS instance number
 *
 *  \param  devParams   [IN]    Initialization configuration structure
 *
 *  \return success             IOM_COMPLETED
 *          failure             IOM Error
 */
Int VPFE_mdBindDev(Ptr *devp, Int devid, Ptr devParams);
/**
 *  \brief Delete a given VPFE driver (instance)
 *
 *  This function is called last and is used for driver deletion.This function
 *  does the deletion of the entire VPFE hardware.
 *
 *  \param  devp        [IN]    pointer in the GIO layer giving the address of
 *                              port object of that particular instance
 *
 *  \Return                     On success returns IOM_COMPLETED
 *                              On failure returns IOM Error code
 */
Int VPFE_mdUnBindDev(Ptr devp);
/**
 *  \brief Open Channel of the VPFE device
 *
 *  The function prepares channel of the driver instance for data transfers
 *  and returns an handle to the driver instance.
 *
 *  \param chanp        [IN]     pointer to the channel structure
 *
 *  \param devp         [IN/OUT] pointer to the port structure
 *
 *  \param name         [IN]     name of the driver
 *
 *  \param mode         [IN]     mode of the channel i.e. INPUT or OUTPUT
 *
 *  \param chanParams   [IN/OUT] Extra parameter (Not used in this driver)
 *
 *  \param cbFxn        [IN]     callback to the GIO Layer
 *
 *  \param cbArg        [IN]     arguments for the callback
 *
 *  \Returns                     On success returns IOM_COMPLETED
 *                               On failure returns IOM Error code
 */
Int VPFE_mdCreateChan(Ptr *chanp, Ptr devp, String name, Int mode,
                      Ptr chanParams, IOM_TiomCallback cbFxn, Ptr cbArg);
/**
 *  \brief Close Channel of the VPFE device
 *
 *   This function is used to close Channel handle for individual VPFE
 *   Modules.
 *
 *  \param  chanp       [IN]    VPFE Driver Channel Handle
 *
 *  \return                     On success returns IOM_COMPLETED
 *                              On failure returns IOM Error code
 */
Int VPFE_mdDeleteChan(Ptr chanp);

/**
 *  \brief VPFE Ioctl
 *
 *  This function provides ioctl functionality for VPFE
 *
 *  \param  chanp       [IN]     VPFE Driver Channel Handle
 *
 *  \param  cmd         [IN]     Operation to be performed, typically an enum
 *                               gets passed
 *
 *  \param  arg         [IN/OUT] Provides additonal information related to the
 *                               operation
 *
 *  \return success              IOM_COMPLETED
 *          failure              IOM Error
 */
Int VPFE_mdControlChan(Ptr chanp, Uns cmd, Ptr arg);

/**
 *  \brief submit IO Request
 *
 *   This function is called to Queue/Dequeue Frame buffer for individual VPFE
 *  (CCDC and other) modules.
 *
 *  \param  chanp       [IN]    VPFE Driver Channel Handle
 *
 *  \param  packet      [IN]    IOM packet passed by the upper layer containing
 *                              information about the size,address,timeout and
 *                              other necessary parameters
 *
 *  \return                     On success, returns positive number - depending
 *                              upon the operation requested
 *
 *                              If failed, returns negative number - IOM Error
 *                              code
 */
Int VPFE_mdSubmitChan(Ptr chanp, IOM_Packet *packet);

#ifdef __cplusplus
}
#endif

#endif /*_DDA_VPFEIOM_H */
