/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file   ddc_vpfe.c
 *
 *  \brief  vpfe implementation
 *
 *  This file contains the implementation of the vpfe driver to utilize the
 *  serivces of the vpfe device driver.
 *
 *  (C) Copyright 2010, Texas Instruments, Inc
 *
 *  \note       Set tabstop to 4 (:se ts=4) while viewing this file in an
 *              editor
 *
 *  \author     Maulik Desai
 *
 *  \version    0.1     created
 */

/******************************************************************************
                            Header File Inclusion
******************************************************************************/
#include <std.h>
#include <clk.h>
#include <math.h>

#include <ti/sdo/pspdrivers/common/psp_common.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osSem.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osmem.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osProtect.h>

#include <ti/sdo/pspdrivers/drivers/vpfe/psp_vpfe.h>
#include <ti/sdo/pspdrivers/drivers/vpfe/src/ddc_vpfe.h>
#include <ti/sdo/pspdrivers/drivers/vpfe/src/ddc_vpfeCcdc.h>
#include <ti/sdo/pspdrivers/drivers/vpfe/src/llc_vpfeCcdc.h>
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/soc.h>

#ifdef VPFE_INSTRUMENTATION_ENABLED
    #include <ti/sdo/pspdrivers/pal_os/bios/psp_log.h>
#endif /*VPFE_INSTRUMENTATION_ENABLED */

#include <ti/sdo/pspdrivers/soc/dm6437/dsp/cslr_prev.h>

static CSL_PrevRegsOvly vpfePrevRegs = (CSL_PrevRegsOvly)(CSL_VPSS_PREV_0_REGS);

/******************************************************************************
                                Data-Structures
******************************************************************************/
/**
  * \brief vpfe device object handle
  */
static DDC_VPFEDeviceObject        vpfeDeviceObject;

extern void _DDA_VPFEGetTimeStamp(Ptr timeStamp);

extern void _DDA_VPFERegisterIntrHandler(Uint32 instNum, Uint32 IntNum);

extern void _DDA_VPFEUnregisterIntrHandler(Uint32 instNum, Uint32 intrNum);

/******************************************************************************
                                    Functions
******************************************************************************/
/**
  * \brief      This function validate the parameters and configure the hardware
  *             and return the Channel Handle
  */
Ptr DDC_VPFEOpenHandle(PSP_VPFE_Id id, Ptr params)
{
    DDC_VPFEDeviceObject * handleDeviceOH   = NULL;
    DDC_VPFECcdcObject   * handleCcdcOH     = NULL;
    Ptr handleToReturnOH                    = NULL;
    PAL_OsSemAttrs semAttrs = PAL_OSSEM_ATTRS;

    semAttrs.type = PAL_OSSEM_TYPE_FIFO;
    /*Get the Device Handle */
    handleDeviceOH      = (DDC_VPFEDeviceObject* )DDC_VPFEGetDeviceHandle();
    /*Get the Channel Handle based on Channel id */
    handleToReturnOH    = DDC_VPFEGetChannelHandle(id);
    /*Check params and Channel Handle not NULL */
    if ((NULL != params) && (NULL != handleToReturnOH))
    {
        switch(id)
        {
            case PSP_VPFE_CCDC:
            {
                handleCcdcOH = (DDC_VPFECcdcObject* )handleToReturnOH;
                /*Check the state of CCDC Channel */
                /*Return NULL if Channel is already in Open State */
                if((DDC_VPFE_OPENED == handleCcdcOH->state) ||
                   (NULL == handleDeviceOH ))
                {
                    handleToReturnOH = NULL;
                }
                else
                {
                    /*Configure the Ccdc Parameters Successfully     */
                    /*Returns Null if Configuring CCDC Channel fails */
                    if(DDC_VPFE_SOK == DDC_VPFEConfigCcdcParams(handleCcdcOH,
                                                                   params))
                    {
                        if (PAL_osSemCreate("VPFEDequeueSEM",
                                                                 0,
                                                                 &semAttrs,
                                                                 &handleCcdcOH->dequeueSem ) != PAL_SOK)
                        {
                            handleToReturnOH = NULL;
                        }
                        else
                        {
                            /*Update the CCDC Handle ID */
                            handleCcdcOH->channelId         = id;
                            /*Update the State as Open */
                            handleCcdcOH->state             = DDC_VPFE_OPENED;
                            /*Sets back referencing of Device Handle */
                            handleCcdcOH->vpssDeviceHandle  = handleDeviceOH;
                         }
                    }
                    else
                    {
                            handleToReturnOH = NULL;
                    }
                }
            }/*Case*/
            break;
            default:
            {
                handleToReturnOH = NULL;
            }
            break;
        }
    }
    return(handleToReturnOH);
}

/**
  * \brief      This function close the VPFE Channel handle(CCDC and other FE)
  *             once created.
  */
DDC_VPFEStatus DDC_VPFECloseHandle(Ptr handle)
{
    PSP_VPFE_Id idCH                        = PSP_VPFE_INVALID_ID;
    DDC_VPFEStatus resultCH                 = DDC_VPFE_INVALID_PARAM;
    DDC_VPFEDeviceObject * handleDeviceCH   = NULL;
    DDC_VPFECcdcObject   * handleCcdcCH     = NULL;

    /*Get the Device Handle */
    handleDeviceCH = (DDC_VPFEDeviceObject* )DDC_VPFEGetDeviceHandle();
    /*Check Device handle not NULL */
    if((NULL != handle)&&(NULL != handleDeviceCH))
    {
        /*Get the Channel Id from Channel Handle */
        idCH = DDC_VPFEGetChannelId(handle);
        switch(idCH)
        {
            case PSP_VPFE_CCDC:
            {
                handleCcdcCH            = (DDC_VPFECcdcObject *)handle;
                /*Check the CCDC Channel not NULL and
                  CCDC Channel not Closed */
                if((DDC_VPFE_CLOSED != handleCcdcCH->state) ||
                   (NULL != handleCcdcCH))
                {
                    resultCH = DDC_VPFE_SOK;
                    /*Turn OFF the Capture */
                    LLC_setCcdcCaptureOff(handleCcdcCH->regs);
                    /* Reset the Frame Buffer Object */
                    DDC_VPFEResetFrameBufferObject(&(handleCcdcCH->fb));
                    /*Load the SDRAM address with 0x00 */
                    LLC_setCcdcFBStartAddress(handleCcdcCH->regs,0x00000000);
                    /*Unregister the CCDC VD0 Interrupt */
                    _DDA_VPFEUnregisterIntrHandler(handleDeviceCH->vpssInstanceId,
                                                    handleDeviceCH->IntNum);
                    if(PSP_VPFE_SENSOR_CCDC_PREVIEWER == handleCcdcCH->dataflow )
                    {
                       CSL_FINS(vpfePrevRegs->PCR, PREV_PCR_ENABLE,
                           (Uint32)CSL_PREV_PCR_ENABLE_PREV_DISABLE);

                    }
                    if(NULL != handleCcdcCH->extVidDecFxn->Close)
                    {
                        handleCcdcCH->extVidDecFxn->Close(handleCcdcCH->extVidDecHandle);
                    }

                    /* Delete the Semaphore */
                    if( NULL != handleCcdcCH->dequeueSem)
                    {
                        if (PAL_osSemDelete(handleCcdcCH->dequeueSem)!= PAL_SOK){
                            resultCH = (DDC_VPFE_E_RESOURCES);
                        }
                    }
                    /* Update the CCDC channel state as Closed */
                    handleCcdcCH->CcdcActive = FALSE;
                    handleCcdcCH->state = DDC_VPFE_CLOSED;
                }
            }
            break;
            default:
            {
                resultCH = DDC_VPFE_E_FAIL;
            }
            break;
        }
    }
    return resultCH;
}

/**
  * \brief  Check if the handle is in closed state
  *
  */
DDC_VPFEStatus DDC_VPFEIsChannelHandleClosed(Ptr handleICH)
{
    PSP_VPFE_Id idICH                       = PSP_VPFE_INVALID_ID;
    DDC_VPFEStatus resultICH                = DDC_VPFE_INVALID_PARAM;
    DDC_VPFECcdcObject   * handleCcdcICH    = NULL;
    /*Check Channel handle not NULL */
    if(NULL != handleICH)
    {
        /*Get the Channel Id based on Channel Handle */
        idICH = DDC_VPFEGetChannelId(handleICH);
        switch(idICH)
        {
            case PSP_VPFE_CCDC:
            {
                handleCcdcICH           = (DDC_VPFECcdcObject *)handleICH;
                /*Check the State of CCDC Channel if Closed */
                if(DDC_VPFE_CLOSED == handleCcdcICH->state)
                {
                    resultICH = DDC_VPFE_SOK;
                }
            }
            break;
            default:
            {
                    resultICH = DDC_VPFE_SOK;
            }
            break;
        }
    }
    return(resultICH);
}

/**
  * \brief  To Get VPFE device handle
  */
Ptr DDC_VPFEGetDeviceHandle(void)
{
    return(&vpfeDeviceObject);
}

/**
  * \brief  To Get VPFE Channel(CCDC) Id
  */
PSP_VPFE_Id DDC_VPFEGetChannelId(Ptr handle)
{
    PSP_VPFE_Id idGI = PSP_VPFE_INVALID_ID;

    /* Get the Device Handle */
    DDC_VPFEDeviceObject *vpfeDeviceHandleGI  =
                        (DDC_VPFEDeviceObject * )DDC_VPFEGetDeviceHandle();
    /*Check the Handle parameter not NULL */
    if ((NULL != handle)&& (NULL!=vpfeDeviceHandleGI))
    {
        /*Compare the CCDC Channel Handle with Handle parameter */
        if( &(vpfeDeviceHandleGI->ccdc) == handle)
        {
            /* Returns the Channel ID */
            idGI = PSP_VPFE_CCDC;
        }
        else
        {
            idGI = PSP_VPFE_INVALID_ID;
        }
    }
    return(idGI);
}

/**
  * \brief      This function will returns the different CCDC FE Plane
  *             handle
  */
Ptr DDC_VPFEGetChannelHandle(PSP_VPFE_Id id)
{
    Ptr handleToReturnGH                      = NULL;
    /*Get the Device Handle */
    DDC_VPFEDeviceObject * vpfeDeviceHandleGH =
                        (DDC_VPFEDeviceObject * )DDC_VPFEGetDeviceHandle();
    if(NULL!=vpfeDeviceHandleGH)
    {
        switch(id)
        {
            case PSP_VPFE_CCDC:
            {
                /*Returns CCDC Channel Handle */
                handleToReturnGH =(DDC_VPFECcdcObject*)&(vpfeDeviceHandleGH->ccdc);
            }
            break;
            default:
            {
                handleToReturnGH = NULL;
            }
            break;
        }
    }
    return(handleToReturnGH);
}

/**
  * \brief  This will reset the frame buffer object
  */
void DDC_VPFEResetFrameBufferObject(DDC_VPFEFrameBufferObject * fbRF)
{
    Uint8 fbIndex = 0x00;
    /*Check the FrameBuffer Object not NULL */
    if(NULL != fbRF)
    {
        fbRF->writeIndexFreeQue         = 0x00000000;
        fbRF->readIndexFreeQue          = 0x00000000;

        fbRF->writeIndexActiveQue       = 0x00000000;
        fbRF->readIndexActiveQue        = 0x00000000;

        for(fbIndex = 0x00; fbIndex < DDC_VPFE_MAX_FRAME_BUFFERS; fbIndex++)
        {
            fbRF->freeQueue[fbIndex]    = NULL;
            fbRF->activeQueue[fbIndex]  = NULL;
        }
    }
}

/**
  * \brief      This function will add the framebuffers in the queue
  */
DDC_VPFEStatus DDC_VPFEAddQueue(DDC_VPFEQueueType       qTypeAQ,
                                Ptr                     handle,
                                FVID_Frame * fbAddressAQ)
{
    Uint32 cookieAQ         = 0x00000000;
    DDC_VPFEStatus resultAQ = DDC_VPFE_E_FAIL;
    DDC_VPFECcdcObject *handleAQ = NULL;
    DDC_VPFEFrameBufferObject * fbAQ = NULL;
    if(NULL!=handle)
    {
        handleAQ = (DDC_VPFECcdcObject *)handle;
        fbAQ     = (DDC_VPFEFrameBufferObject *)&(handleAQ->fb);
    }
    if(((NULL != fbAddressAQ) && (NULL != handleAQ))&&(NULL != fbAQ))
    {
        switch(qTypeAQ)
        {
            case DDC_VPFE_FREE_QUEUE:
            {
                /*Check whether FreeQueue is NULL */
                if(NULL == fbAQ->freeQueue[fbAQ->writeIndexFreeQue])
                {
                    /*Add the FrameBuffer in Free Queue */
                    fbAQ->freeQueue[fbAQ->writeIndexFreeQue] = fbAddressAQ;
                    /*Increment the WriteIndex */
                    fbAQ->writeIndexFreeQue = (fbAQ->writeIndexFreeQue+1u)% DDC_VPFE_MAX_FRAME_BUFFERS;
                    if(TRUE == handleAQ->dequeueWaitFlag)
                    {
                        if (PAL_osSemGive(handleAQ->dequeueSem) != PAL_SOK) {

                        }
                        handleAQ->dequeueWaitFlag = FALSE;
                    }
                    resultAQ = DDC_VPFE_SOK;
                }
            }
            break;
            case DDC_VPFE_ACTIVE_QUEUE:
            {
                PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT,&cookieAQ);
                /*Check whether ActiveQueue is NULL */
                if(NULL == fbAQ->activeQueue[fbAQ->writeIndexActiveQue])
                {
                    /*Add the FrameBuffer in Active Queue */
                    fbAQ->activeQueue[fbAQ->writeIndexActiveQue] = fbAddressAQ;
                    /*Increment the WriteIndex */
                    fbAQ->writeIndexActiveQue = (fbAQ->writeIndexActiveQue+1u)% DDC_VPFE_MAX_FRAME_BUFFERS;
                    resultAQ                  = DDC_VPFE_SOK;
                }
                else
                {
                    resultAQ = DDC_VPFE_E_RESOURCES;
                }
                PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT,cookieAQ);
            }
            break;
            default:
            {
                resultAQ = DDC_VPFE_E_FAIL;
            }
            break;
        }
    }
    return(resultAQ);
}


/**
  * \brief      This will delete a framebuffer from the Queue
  */
DDC_VPFEStatus DDC_VPFEDelQueue(DDC_VPFEQueueType        qTypeDQ,
                                Ptr                      handle,
                                FVID_Frame **  fbAddressDQ)
{
    Uint indexDQ    = 0x00000000;
    Uint32 cookieDQ = 0x00000000;
    DDC_VPFEStatus result = DDC_VPFE_E_FAIL;
    DDC_VPFECcdcObject *handleDQ = NULL;
    DDC_VPFEFrameBufferObject *   fbDQ = NULL;
    if(NULL!= handle)
    {
        handleDQ = (DDC_VPFECcdcObject *)handle;
        fbDQ     = (DDC_VPFEFrameBufferObject *)&(handleDQ->fb);
    }
    /*Check the Framebuffer not Null */
    if (((NULL!=fbAddressDQ) && (NULL!=handleDQ)) && (NULL!=fbDQ))
    {
        switch(qTypeDQ)
        {
            case DDC_VPFE_FREE_QUEUE:
            {
                /*Disable Interrupts to protect Critical Section */
                PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT,&cookieDQ);
                /* Check Free Queue is NULL */
                if(NULL == fbDQ->freeQueue[fbDQ->readIndexFreeQue])
                {
                    handleDQ->dequeueWaitFlag = TRUE;
                    PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT,cookieDQ);
                    if (PAL_SOK != PAL_osSemTake(handleDQ->dequeueSem, 2000)){
                        result = DDC_VPFE_E_FAIL;
                    }
                }
                else
                {
                    PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT,cookieDQ);
                }
                if(NULL != fbDQ->freeQueue[fbDQ->readIndexFreeQue])
                {
                    /* Disable Interrupts to protect Critical Section */
                    PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT,&cookieDQ);
                    *fbAddressDQ            = fbDQ->freeQueue[fbDQ->readIndexFreeQue];
                    fbDQ->freeQueue[fbDQ->readIndexFreeQue] = NULL;
                    fbDQ->readIndexFreeQue  = (fbDQ->readIndexFreeQue+1u)% DDC_VPFE_MAX_FRAME_BUFFERS;
                    PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT,cookieDQ);
                    result = DDC_VPFE_SOK;
                }
            }
            break;
            case DDC_VPFE_ACTIVE_QUEUE:
            {
                /*Check next Framebuffer in Queue is not NULL */
                /*If there is framebuffer in Queue make the current index
                  in queue to be Null */
                 indexDQ = (fbDQ->readIndexActiveQue+1u)% DDC_VPFE_MAX_FRAME_BUFFERS;
                if(NULL != fbDQ->activeQueue[indexDQ])
                {
                    *fbAddressDQ             = fbDQ->activeQueue[fbDQ->readIndexActiveQue];
                    fbDQ->activeQueue[fbDQ->readIndexActiveQue] = NULL;
                    fbDQ->readIndexActiveQue = indexDQ;
                    result                   = DDC_VPFE_SOK;
                }
            }
            break;
            default:
            {
                result = DDC_VPFE_E_FAIL;
            }
            break;
        }

    }

    return(result);
}

/**
  * \brief      Get next frame buffer address
  */
FVID_Frame * DDC_VPFEGetNxtFBAddress(const DDC_VPFEFrameBufferObject * fbGN)
{
    FVID_Frame * retHandle  = NULL;
    /*Check the FrameBuffer not NULL */
    if (NULL != fbGN)
    {
        retHandle = (fbGN->activeQueue[fbGN->readIndexActiveQue]);
    }
    return retHandle;
}

/**
  * \brief      It will add the frame buffer to the Queue. It is valid for both
  *             plane of Video and OSD
  */
DDC_VPFEStatus DDC_VPFEQueueFrameBuffer(Ptr                     handleQF,
                                        FVID_Frame * fbPtrQF)
{
    PSP_VPFE_Id idQF                        = PSP_VPFE_INVALID_ID;
    DDC_VPFEStatus resultQF                 = DDC_VPFE_E_FAIL;
    DDC_VPFECcdcObject  * vpfeCcdcHandleQF  = NULL;
    DDC_VPFEDeviceObject * vpfeDeviceHandleCC = NULL;
    if((NULL == handleQF)||(NULL == fbPtrQF))
    {
        resultQF = DDC_VPFE_INVALID_PARAM;
    }
    else
    {
        /*Get the Device Handle */
        vpfeDeviceHandleCC = (DDC_VPFEDeviceObject * )DDC_VPFEGetDeviceHandle();
        /*Check the Channel Id */
        idQF = DDC_VPFEGetChannelId(handleQF);
        switch(idQF)
        {
            case PSP_VPFE_CCDC:
            {
                vpfeCcdcHandleQF = (DDC_VPFECcdcObject *)handleQF;
                if(DDC_VPFE_SOK != DDC_VPFECheckFrameBuffer(vpfeCcdcHandleQF,fbPtrQF))
                {
                  resultQF = DDC_VPFE_INVALID_PARAM;
                }
                else
                {   /*Add FrameBuffer in Active Queue */
                    resultQF = DDC_VPFEAddQueue(DDC_VPFE_ACTIVE_QUEUE,
                            vpfeCcdcHandleQF, fbPtrQF);
                        if ((FALSE == vpfeCcdcHandleQF->CcdcActive)&&
                              (vpfeCcdcHandleQF->dataflow
                                == PSP_VPFE_SENSOR_CCDC_PREVIEWER))
                        {
                            LLC_setCcdcOutputToSDRAMDisable(
                                vpfeCcdcHandleQF->regs);
                            CSL_FINS(vpfePrevRegs->WSDR_ADDR,
                                PREV_WSDR_ADDR_ADDR,
                                (Uint32)fbPtrQF->frame.frameBufferPtr);
                            CSL_FINS(vpfePrevRegs->PCR, PREV_PCR_ENABLE,
                                (Uint32)CSL_PREV_PCR_ENABLE_PREV_ENABLE);
                        }
                        if ((FALSE == vpfeCcdcHandleQF->CcdcActive)
                            && (DDC_VPFE_SOK == resultQF))
                        {

#ifdef VPFE_INSTRUMENTATION_ENABLED
                            PSP_LOG_EVENT(  &DVTEvent_Log,
                                            "VPFE",
                                            PSP_DVT_DESC(   PSP_DVT_eRCV_START,
                                                            PSP_DVT_dCHANNEL_ID_INST,
                                                            PSP_DVT_dPACKET_ID,
                                                            PSP_DVT_dNONE),
                                            CHANNELID_INSTANCEID(
                                                0,
                                                0),
                                            fbPtrQF
                                         );
#endif /* VPFE_INSTRUMENTATION_ENABLED */

                            LLC_setCcdcFBStartAddress(vpfeCcdcHandleQF->regs,
                                fbPtrQF->frame.frameBufferPtr);
                            LLC_setCcdcOffset(vpfeCcdcHandleQF->regs,
                                fbPtrQF->pitch);
                            LLC_setCcdcCaptureOn(vpfeCcdcHandleQF->regs);
                               vpfeCcdcHandleQF->CcdcActive = TRUE;
                            /*Register the Interrupt */
                            if(NULL != vpfeDeviceHandleCC)
                            {
                                _DDA_VPFERegisterIntrHandler(0,
                                            vpfeDeviceHandleCC->IntNum);
                            }
                            resultQF = DDC_VPFE_SOK;
                        }

                }
            }
            break;
            default:
            {
                resultQF = DDC_VPFE_E_FAIL;
            }
            break;
        }
    }
    return(resultQF);
}

/**
  * \brief      It will De-Queue the frame buffer from the Queue. It is valid
  *             for both plane of Video and OSD.
  */
DDC_VPFEStatus DDC_VPFEDeQueueFrameBuffer(Ptr              handleDQ,
                                  FVID_Frame ** deQueuedFrameBufferDQ)
{
    PSP_VPFE_Id idDQ                            = PSP_VPFE_INVALID_ID;
    DDC_VPFEStatus resultDQ                     = DDC_VPFE_E_FAIL;
    FVID_Frame * fbAddrDQ            = NULL;
    DDC_VPFECcdcObject    * vpfeCcdcHandleDQ    = NULL;

    if(NULL == handleDQ)
    {
        resultDQ    = DDC_VPFE_INVALID_PARAM;
    }
    else
    {
        /*Get the Channel Id  */
        idDQ = DDC_VPFEGetChannelId(handleDQ);
        switch(idDQ)
        {
            case PSP_VPFE_CCDC:
            {
                vpfeCcdcHandleDQ = (DDC_VPFECcdcObject *)handleDQ;
                /* Delete the FrameBuffer from the Active Queue */
                if(DDC_VPFE_SOK == DDC_VPFEDelQueue(DDC_VPFE_FREE_QUEUE,
                                                    vpfeCcdcHandleDQ,
                                                    &fbAddrDQ))
                {
                    if(NULL != fbAddrDQ)
                    {
                        *deQueuedFrameBufferDQ  = fbAddrDQ;
                        resultDQ                = DDC_VPFE_SOK;
                    }
                    else
                    {
                        resultDQ = DDC_VPFE_E_FAIL;
                    }
                }
            }
            break;
            default:
            {
                    resultDQ = DDC_VPFE_E_FAIL;
            }
            break;
        }
    }
    return(resultDQ);
}


/**
  * \brief      Interrupt service routine for Front - End
  */
/* ARGSUSED */
void DDC_VPFEIsr(Uint32 arg)
{
    DDC_VPFECcdcObject* vpfeCcdcHandleVFI   = NULL;
    DDC_VPFEFieldId currentFieldVFI;
    FVID_Frame * fbAddActiveUN    = NULL;
#ifdef VPFE_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT(  &DVTEvent_Log,
                    "VPFE",
                    PSP_DVT_DESC(   PSP_DVT_eINT_START,
                                    PSP_DVT_dNONE,
                                    PSP_DVT_dNONE,
                                    PSP_DVT_dNONE));
#endif /* VPFE_INSTRUMENTATION_ENABLED */

    /*Get the CCDC Channel Handle  */
    vpfeCcdcHandleVFI = (DDC_VPFECcdcObject*)DDC_VPFEGetChannelHandle(PSP_VPFE_CCDC);
    if(NULL!=vpfeCcdcHandleVFI)
    {
        currentFieldVFI = (DDC_VPFEFieldId)LLC_getCcdcField(vpfeCcdcHandleVFI->regs);
        vpfeCcdcHandleVFI->fieldId ^= 1;

        if (FVID_FRAME_MODE == vpfeCcdcHandleVFI->CcdcParams.ffMode)
        {
            if(currentFieldVFI == vpfeCcdcHandleVFI->fieldId)
            {
                /* Field separated Mode */
                if(DDC_VPFE_FIELD_SEPARATE == vpfeCcdcHandleVFI->FieldType)
                {
                    if(DDC_VPFE_1ST_FIELD == currentFieldVFI)
                    {
                        fbAddActiveUN = DDC_VPFEGetFBAddress(&(vpfeCcdcHandleVFI->fb));
                        if(NULL!=fbAddActiveUN)
                        {
                            /* Update the Frame Buffer Address on Hardware Register */
                            LLC_setCcdcFBStartAddress(vpfeCcdcHandleVFI->regs,
                                                     (( Uint8 *)(fbAddActiveUN->frame.frameBufferPtr)+(fbAddActiveUN->pitch * ((fbAddActiveUN->lines>>1)-1))));
                        }
                        DDC_VPFEUpdateFreeQueue(PSP_VPFE_CCDC);
                    }
                    if(DDC_VPFE_2ND_FIELD == currentFieldVFI)
                    {
                        DDC_VPFEUpdateFbAdress(PSP_VPFE_CCDC);
                    }
                }
                else
                {
                    if(DDC_VPFE_1ST_FIELD == currentFieldVFI)
                    {
                        DDC_VPFEUpdateFreeQueue(PSP_VPFE_CCDC);
                    }
                    else if(DDC_VPFE_2ND_FIELD == currentFieldVFI)
                    {
                        DDC_VPFEUpdateFbAdress(PSP_VPFE_CCDC);
                    }
                }
                DDC_VPFE_AppCallbackFunction(PSP_VPFE_CCDC,currentFieldVFI);
            }
            else if (DDC_VPFE_1ST_FIELD == currentFieldVFI)
            {
                vpfeCcdcHandleVFI->fieldId = currentFieldVFI;
            }
        }
        else
        {
            DDC_VPFEUpdateFreeQueue(PSP_VPFE_CCDC);
            DDC_VPFEUpdateFbAdress(PSP_VPFE_CCDC);
            DDC_VPFE_AppCallbackFunction(PSP_VPFE_CCDC,currentFieldVFI);
        }
    }

#ifdef VPFE_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT(  &DVTEvent_Log,
                    "VPFE",
                    PSP_DVT_DESC(   PSP_DVT_eINT_END,
                                    PSP_DVT_dNONE,
                                    PSP_DVT_dNONE,
                                    PSP_DVT_dNONE));
#endif /* VPFE_INSTRUMENTATION_ENABLED */
}


/**
  * \brief      This function invokes the callback that the application has
  *             registered with the driver.
  */
void DDC_VPFE_AppCallbackFunction(PSP_VPFE_Id      idCACF,
                                  DDC_VPFEFieldId   currentFieldCACF)
{
    DDC_VPFECcdcObject    * vpfeCcdcHandleCACF  = NULL;
    DDC_VPFEFrameBufferObject * fbDQ = NULL;
    FVID_Frame * fbAddActiveUN = NULL;
    PSP_VPFECallbackParams callbackParams = {NULL,0,NULL};
    Uint32 value = 0;
    switch(idCACF)
    {
        case PSP_VPFE_CCDC:
        {
            /*Get the  CCDC Channel Handle */
            vpfeCcdcHandleCACF = (DDC_VPFECcdcObject*)DDC_VPFEGetChannelHandle(idCACF);
            /*Check the CCDC Channel handle not NULL */
            if((NULL != vpfeCcdcHandleCACF )                    &&
               ((DDC_VPFE_OPENED == vpfeCcdcHandleCACF->state)  &&
               (NULL != vpfeCcdcHandleCACF->CcdcParams.appCallback)))
            {
                vpfeCcdcHandleCACF->fbPtr = NULL;
                fbDQ  = (DDC_VPFEFrameBufferObject *)&(vpfeCcdcHandleCACF->fb);
                value =  LLC_getVDINT(vpfeCcdcHandleCACF->regs);
                if(DDC_VPFE_FIELD_SEPARATE == vpfeCcdcHandleCACF->FieldType)
                {
                    if((DDC_VPFE_2ND_FIELD == currentFieldCACF)&&
                       (NULL != vpfeCcdcHandleCACF->fb.tempfbAddFree))
                    {
                        fbAddActiveUN = vpfeCcdcHandleCACF->fb.tempfbAddFree;
                        callbackParams.currentFrame = fbAddActiveUN->frame.frameBufferPtr;
                    }
                    if((DDC_VPFE_1ST_FIELD == currentFieldCACF)&&
                      (NULL != fbDQ->freeQueue[fbDQ->readIndexFreeQue]))
                    {
                        fbAddActiveUN = fbDQ->freeQueue[fbDQ->readIndexFreeQue];
                        callbackParams.currentFrame = (( Uint8 *)(fbAddActiveUN->frame.frameBufferPtr)+(fbAddActiveUN->pitch * ((fbAddActiveUN->lines>>1)-1)));

                    }
                }
                else    /* DDC_VPFE_FIELD_INTERLACED */
                {
                     /* EvenField */
                    if((DDC_VPFE_2ND_FIELD == currentFieldCACF)&&
                       (NULL != vpfeCcdcHandleCACF->fb.tempfbAddFree))
                    {
                        fbAddActiveUN = vpfeCcdcHandleCACF->fb.tempfbAddFree;
                        callbackParams.currentFrame = fbAddActiveUN->frame.frameBufferPtr;
                    }
                    /* Odd Field */
                    if((DDC_VPFE_1ST_FIELD == currentFieldCACF)&&
                      (NULL != fbDQ->freeQueue[fbDQ->readIndexFreeQue]))
                    {
                        fbAddActiveUN = fbDQ->freeQueue[fbDQ->readIndexFreeQue];
                        callbackParams.currentFrame   = fbAddActiveUN->frame.frameBufferPtr;
                    }
                }
                callbackParams.NumLines_VDINT = value;
                vpfeCcdcHandleCACF->CcdcParams.appCallback(&(vpfeCcdcHandleCACF->fbPtr),
                                                            &callbackParams,
                                                            currentFieldCACF);
                if(NULL != vpfeCcdcHandleCACF->fbPtr)
                {
                    LLC_setCcdcFBStartAddress(vpfeCcdcHandleCACF->regs,
                                                   vpfeCcdcHandleCACF->fbPtr);
                }
            }
        }
        break;
        default:
        {
            /* Not Supported Currently */
        }
        break;
    }
}
/**
  * \brief      It will Perform register overlaying
  */
void DDC_VPFERegisterOverlaying(void)
{
    DDC_VPFECcdcObject* vpfeCcdcHandlePR        = NULL;
    /*Get the CCDC Channel Handle */
    vpfeCcdcHandlePR = (DDC_VPFECcdcObject* )DDC_VPFEGetChannelHandle(PSP_VPFE_CCDC);
    /*Check the Handle not NULL */
    if(NULL != vpfeCcdcHandlePR)
    {
        vpfeCcdcHandlePR->regs = (DDC_VPFECcdcObject* )(CSL_VPSS_CCDC_0_REGS);
        if(NULL!=vpfeCcdcHandlePR->regs)
        {
            LLC_ResetCcdcReg(vpfeCcdcHandlePR->regs);
        }
    }
}
/**
  * \brief  Set device ISR number
  */
void DDC_VPFESetInterruptNumber()
{
    DDC_VPFEDeviceObject * vpfeDeviceHandleSIN = NULL;
    /*Get the Device Handle */
    vpfeDeviceHandleSIN = (DDC_VPFEDeviceObject *)DDC_VPFEGetDeviceHandle();
    if(NULL!=vpfeDeviceHandleSIN)
    {
        /*Set the Interrupt Number */
        vpfeDeviceHandleSIN->IntNum = CSL_INTC_EVENTID_VDINT0;
    }

}

DDC_VPFEStatus DDC_VPFEValidateFrameBuffer(PSP_VPFEValidateParams *surfVF)
{

    DDC_VPFEStatus  result                      = DDC_VPFE_SOK;
    typedef float float_32;
    float_32                rawPitchGV      = 0.0;
    Uint32                  roundedPitchGV  = 0x00000000;

    /*Check the Parameters not NULL */
    if((NULL == surfVF))
    {
        result = DDC_VPFE_INVALID_PARAM;
    }
    else
    {
        if((0== surfVF->InParams.pitch)||(0!=surfVF->InParams.pitch%32u))
        {
            if((0!=surfVF->InParams.pitch)&&((surfVF->InParams.pitch % 32u)!=0))
            {
                rawPitchGV              = (float)(((float)surfVF->InParams.pitch/32u));
                roundedPitchGV          = (Uint32)ceil(rawPitchGV);
                surfVF->OutParams.pitch = (Uint)(roundedPitchGV * (32u));
            }
            result = DDC_VPFE_INVALID_PARAM;
        }
    }
    return(result);
}

/**
  * \brief      Validate Frame Buffers For VPSS Interface Object
  */
DDC_VPFEStatus DDC_VPFECheckFrameBuffer(Ptr                     handleVF,
                                        FVID_Frame * const surfVF)
{
    DDC_VPFEStatus result                       = DDC_VPFE_SOK;
    Int fbAddressVF                             = NULL;
    /*Check the Parameters  not NULL */
    if((NULL == surfVF) || (NULL == handleVF))
    {
        result = DDC_VPFE_INVALID_PARAM;
    }
    else
        {
            fbAddressVF = (Int) surfVF->frame.frameBufferPtr;
        }
    if(((result != DDC_VPFE_SOK)    ||
       ((fbAddressVF % (32u)) != 0))    ||
       ((surfVF->pitch == 0)        ||
       ((surfVF->pitch % (32u)) != 0)))
    {
        result = DDC_VPFE_INVALID_PARAM;
    }
    return(result);
}

/**
  * \brief  To allocate the Memory to FrameBuffers
  */
DDC_VPFEStatus DDC_VPFEAllocFrameBuffer(Ptr handle ,
                                        FVID_Frame ** allocFB)
{
    Ptr                     bufferalloc      = NULL;
    FVID_Frame              * surfaceFB        = NULL;
    DDC_VPFECcdcObject    * vpfeCcdcHandle   = NULL;
    DDC_VPFEStatus          result  = DDC_VPFE_E_FAIL;
    PAL_Result              status  = PSP_E_NO_MEMORY;
    Uint32 fb_size = 0;

    if(NULL != handle)
    {
        result  = DDC_VPFE_NO_MEMORY;
        vpfeCcdcHandle = (DDC_VPFECcdcObject * )handle;
        status = PAL_osMemAlloc(vpfeCcdcHandle->CcdcParams.segId,sizeof(FVID_Frame),32u,(Ptr *)&bufferalloc);
        if(PAL_SOK == status)
        {
            surfaceFB   =   (FVID_Frame *)bufferalloc;
            if(NULL!=surfaceFB)
            {
                fb_size = (vpfeCcdcHandle->CcdcParams.height)*(vpfeCcdcHandle->CcdcParams.pitch);
                status = PAL_osMemAlloc(vpfeCcdcHandle->CcdcParams.segId,fb_size,128u,&(surfaceFB->frame.frameBufferPtr));
                if(PAL_SOK == status)
                {
                    if(NULL!=surfaceFB->frame.frameBufferPtr)
                    {
                        surfaceFB->lines    = vpfeCcdcHandle->CcdcParams.height;
                        surfaceFB->pitch    = vpfeCcdcHandle->CcdcParams.pitch;
                        surfaceFB->bpp      = (FVID_bitsPerPixel)
                                              (((vpfeCcdcHandle->CcdcParams.pitch)/vpfeCcdcHandle->CcdcParams.width)*8);
                        *allocFB = surfaceFB;
                        result = DDC_VPFE_SOK;
                    }
                }
                else
                {
                    status = PAL_osMemFree(vpfeCcdcHandle->CcdcParams.segId,surfaceFB,sizeof(FVID_Frame));
                    if(PAL_SOK== status)
                    {
                        result  = DDC_VPFE_NO_MEMORY;
                    }
                }
            }
        }
    }
    return result;
}
/**
  * \brief  To Free the Allocated Frame Buffer Memory
  */
DDC_VPFEStatus DDC_VPFEFreeFrameBuffer(Ptr handle,
                                       FVID_Frame ** allocFrameBuffer)
{
    DDC_VPFECcdcObject    * vpfeCcdcHandle   = NULL;
    DDC_VPFEStatus result = DDC_VPFE_E_FAIL;
    Uint32 fb_size = 0;
    FVID_Frame *ptrFrame;

    if((NULL == handle)||(NULL == allocFrameBuffer))
    {
        result = DDC_VPFE_E_FAIL;
    }
    else
    {
        ptrFrame = *allocFrameBuffer;
        vpfeCcdcHandle = (DDC_VPFECcdcObject *)handle;
        fb_size = (vpfeCcdcHandle->CcdcParams.height)*(vpfeCcdcHandle->CcdcParams.pitch);
        if(PAL_SOK == PAL_osMemFree(vpfeCcdcHandle->CcdcParams.segId,ptrFrame->frame.frameBufferPtr,fb_size))
        {
            if(PAL_SOK ==PAL_osMemFree(vpfeCcdcHandle->CcdcParams.segId,ptrFrame,sizeof(FVID_Frame)))
            {
                result = DDC_VPFE_SOK;
            }
        }
    }
    return result;
}

/**
  * \brief  To configure the TVP5146 Chip
  */
DDC_VPFEStatus DDC_ConfigextVDParams(Ptr handle, Uint32 cmd ,Ptr cmdArg)
{
    DDC_VPFECcdcObject * vpfeCcdcHandle   = NULL;
    DDC_VPFEStatus  result = DDC_VPFE_E_FAIL;
    if((NULL== handle)||(NULL == cmdArg))
    {
        result = DDC_VPFE_E_FAIL;
    }
    else
    {
        vpfeCcdcHandle = (DDC_VPFECcdcObject *)handle;
        if(NULL != vpfeCcdcHandle->extVidDecFxn)
        {
            if(NULL != vpfeCcdcHandle->extVidDecFxn->Control)
            {
            result = (DDC_VPFEStatus)vpfeCcdcHandle->extVidDecFxn->Control(vpfeCcdcHandle->extVidDecHandle,
                                                                      cmd-VPFE_ExtVD_BASE ,cmdArg);
            }
        }
    }
    return result;
}
/**
  * \brief  To Update the Frame buffer address
  */
void DDC_VPFEUpdateFbAdress(PSP_VPFE_Id idUN)
{
    DDC_VPFECcdcObject    * vpfeCcdcHandleUN = NULL;
    FVID_Frame * fbAddActiveUN    = NULL;
    switch(idUN)
    {
        case PSP_VPFE_CCDC:
        {
            /*Get the Channel handle */
            vpfeCcdcHandleUN = (DDC_VPFECcdcObject*)DDC_VPFEGetChannelHandle(idUN);
            if((NULL != vpfeCcdcHandleUN ) &&
               (DDC_VPFE_OPENED == vpfeCcdcHandleUN->state))
            {
                if(NULL == vpfeCcdcHandleUN->fb.tempfbAddFree)
                {

#ifdef VPFE_INSTRUMENTATION_ENABLED
                    PSP_LOG_EVENT(  &DVTEvent_Log,
                                    "VPFE",
                                    PSP_DVT_DESC(   PSP_DVT_eRCV_END,
                                                    PSP_DVT_dCHANNEL_ID_INST,
                                                    PSP_DVT_dPACKET_ID,
                                                    PSP_DVT_dNONE),
                                    CHANNELID_INSTANCEID(
                                        0,
                                        0),
                                    vpfeCcdcHandleUN->fb.tempfbAddFree
                                 );
#endif /* VPFE_INSTRUMENTATION_ENABLED */

                    /* Delete the FrameBuffer from the Active Queue */
                    if(DDC_VPFE_SOK == DDC_VPFEDelQueue(DDC_VPFE_ACTIVE_QUEUE,
                                                        vpfeCcdcHandleUN,
                                                        &(vpfeCcdcHandleUN->fb.tempfbAddFree)))
                    {
                        /*Get Next Frame Buffer Address */
                        fbAddActiveUN = DDC_VPFEGetNxtFBAddress(&(vpfeCcdcHandleUN->fb));
                        if(NULL!=fbAddActiveUN)
                        {

#ifdef VPFE_INSTRUMENTATION_ENABLED
                            PSP_LOG_EVENT(  &DVTEvent_Log,
                                            "VPFE",
                                            PSP_DVT_DESC(   PSP_DVT_eRCV_START,
                                                            PSP_DVT_dCHANNEL_ID_INST,
                                                            PSP_DVT_dPACKET_ID,
                                                            PSP_DVT_dNONE),
                                            CHANNELID_INSTANCEID(
                                                0,
                                                0),
                                            fbAddActiveUN
                                         );
#endif /* VPFE_INSTRUMENTATION_ENABLED */

                            if (vpfeCcdcHandleUN->dataflow == PSP_VPFE_SENSOR_CCDC_PREVIEWER)
                            {
                                CSL_FINS(vpfePrevRegs->WSDR_ADDR,
                                         PREV_WSDR_ADDR_ADDR,
                                             (Uint32)fbAddActiveUN->frame.frameBufferPtr);
                            }
                            else
                            {
                                /*Update the Frame Buffer Address on Hardware Register*/
                                LLC_setCcdcFBStartAddress(vpfeCcdcHandleUN->regs,
                                                              fbAddActiveUN->frame.frameBufferPtr);
                            }
                        }
                    }
                }

            }
        }
        break;
        default:
        {

        }
        break;
    }
}
/**
  * \brief  To Update the Frame Buffer to Free Queue
  */
void DDC_VPFEUpdateFreeQueue(PSP_VPFE_Id idUN)
{
    DDC_VPFECcdcObject    * vpfeCcdcHandleUN = NULL;
    switch(idUN)
    {
        case PSP_VPFE_CCDC:
        {
            /*Get the Channel handle */
            vpfeCcdcHandleUN = (DDC_VPFECcdcObject*)DDC_VPFEGetChannelHandle(idUN);
            if((NULL != vpfeCcdcHandleUN ) &&
               (DDC_VPFE_OPENED == vpfeCcdcHandleUN->state))
            {
                if(NULL!= vpfeCcdcHandleUN->fb.tempfbAddFree)
                {
                   _DDA_VPFEGetTimeStamp(&(vpfeCcdcHandleUN->fb.tempfbAddFree)->timeStamp);
                   DDC_VPFEAddQueue(DDC_VPFE_FREE_QUEUE,
                                     vpfeCcdcHandleUN,vpfeCcdcHandleUN->fb.tempfbAddFree);
                    vpfeCcdcHandleUN->fb.tempfbAddFree = NULL;
                }

            }
        }
        break;
        default:
        {

        }
        break;
    }
}

DDC_VPFEStatus DDC_VPFEStopEngine(Ptr handle)
{

    DDC_VPFEFrameBufferObject * fbStop       = NULL;
    DDC_VPFECcdcObject        * vpfeCcdcHandle = NULL;
    DDC_VPFEStatus result = DDC_VPFE_E_FAIL;
    DDC_VPFEDeviceObject * handleDeviceCH   = NULL;
    Uint32 cookieAQ = 0;

    /*Get the Device Handle */
    handleDeviceCH = (DDC_VPFEDeviceObject* )DDC_VPFEGetDeviceHandle();

    if(NULL!= handle)
    {
        vpfeCcdcHandle = (DDC_VPFECcdcObject *)handle;
        fbStop         = (DDC_VPFEFrameBufferObject *)&(vpfeCcdcHandle->fb);
        /*Turn OFF the Capture */
        LLC_setCcdcCaptureOff(vpfeCcdcHandle->regs);
        /*Load the SDRAM address with 0x00 */
        LLC_setCcdcFBStartAddress(vpfeCcdcHandle->regs,0x00000000);
        /*Unregister the CCDC VD0 Interrupt */
        _DDA_VPFEUnregisterIntrHandler(handleDeviceCH->vpssInstanceId,
                                        handleDeviceCH->IntNum);

        if(PSP_VPFE_SENSOR_CCDC_PREVIEWER == vpfeCcdcHandle->dataflow)
        {
           CSL_FINS(vpfePrevRegs->PCR, PREV_PCR_ENABLE,
               (Uint32)CSL_PREV_PCR_ENABLE_PREV_DISABLE);
        }

        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT,&cookieAQ);
        if((fbStop->writeIndexActiveQue - fbStop->readIndexActiveQue)!= 0)
        {
            while(fbStop->readIndexActiveQue != fbStop->writeIndexActiveQue)
            {
                if( (NULL != fbStop->activeQueue[fbStop->readIndexActiveQue]) &&
                    (NULL == fbStop->freeQueue[fbStop->writeIndexFreeQue]))
                {
                    fbStop->freeQueue[fbStop->writeIndexFreeQue] =
                                fbStop->activeQueue[fbStop->readIndexActiveQue];
                    fbStop->activeQueue[fbStop->readIndexActiveQue] = NULL;
                    fbStop->writeIndexFreeQue  = (fbStop->writeIndexFreeQue +1u);
                    /* Circular buffer upto DDC_VPFE_MAX_FRAME_BUFFERS */
                    fbStop->writeIndexFreeQue  %= DDC_VPFE_MAX_FRAME_BUFFERS;
                    fbStop->readIndexActiveQue = (fbStop->readIndexActiveQue +1u);
                    /* Circular buffer upto DDC_VPFE_MAX_FRAME_BUFFERS */
                    fbStop->readIndexActiveQue %= DDC_VPFE_MAX_FRAME_BUFFERS;
                }
            }
        }
        if(NULL!= vpfeCcdcHandle->fb.tempfbAddFree)
        {
            fbStop->freeQueue[fbStop->writeIndexFreeQue] = vpfeCcdcHandle->fb.tempfbAddFree;
            fbStop->writeIndexFreeQue  = (fbStop->writeIndexFreeQue +1u);
            fbStop->writeIndexFreeQue  %= DDC_VPFE_MAX_FRAME_BUFFERS;
            vpfeCcdcHandle->fb.tempfbAddFree = NULL;
        }
        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT,cookieAQ);

        vpfeCcdcHandle->CcdcActive = FALSE;
        result = DDC_VPFE_SOK;
    }
    return result;
}

DDC_VPFEStatus DDC_VPFEStartEngine(Ptr handle)
{
    DDC_VPFEStatus result = DDC_VPFE_E_FAIL;
    DDC_VPFEFrameBufferObject * fbStart        = NULL;
    DDC_VPFECcdcObject        * vpfeCcdcHandle = NULL;
    DDC_VPFEDeviceObject * vpfeDeviceHandleCC = NULL;
    FVID_Frame * fbPtrQF = NULL;
    Bool   CcdcStat;
    Uint32 cookieAQ =0;

    vpfeCcdcHandle = (DDC_VPFECcdcObject *)handle;
    fbStart        = (DDC_VPFEFrameBufferObject *)&(vpfeCcdcHandle->fb);

    if((NULL!= vpfeCcdcHandle)&&(NULL != fbStart))
    {
        CcdcStat = LLC_getCcdcEngineStat(vpfeCcdcHandle->regs);

        /*Get the Device Handle */
        vpfeDeviceHandleCC = (DDC_VPFEDeviceObject * )DDC_VPFEGetDeviceHandle();

        /*Ccdc Engine OFF */
        if(0 == CcdcStat)
        {
            /* Check Free Queue is Empty */
            /*If Free Queue is not Empty Copy the Free Queue in Active Queue */
            PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT,&cookieAQ);
            if((fbStart->writeIndexFreeQue - fbStart->readIndexFreeQue)!= 0)
            {
                while(fbStart->readIndexFreeQue !=fbStart->writeIndexFreeQue)
                {
                    if((NULL != fbStart->freeQueue[fbStart->readIndexFreeQue]) &&
                       (NULL == fbStart->activeQueue[fbStart->writeIndexActiveQue]))
                    {
                        fbStart->activeQueue[fbStart->writeIndexActiveQue] =
                                  fbStart->freeQueue[fbStart->readIndexFreeQue];
                        fbStart->freeQueue[fbStart->readIndexFreeQue] = NULL;
                        fbStart->writeIndexActiveQue = (fbStart->writeIndexActiveQue +1u);
                        /* Circular buffer upto DDC_VPFE_MAX_FRAME_BUFFERS */
                        fbStart->writeIndexActiveQue %= DDC_VPFE_MAX_FRAME_BUFFERS;
                        fbStart->readIndexFreeQue   = (fbStart->readIndexFreeQue +1u);
                        /* Circular buffer upto DDC_VPFE_MAX_FRAME_BUFFERS */
                        fbStart->readIndexFreeQue   %= DDC_VPFE_MAX_FRAME_BUFFERS;
                    }
                }
            }
            PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT,cookieAQ);
            /*Read the Framebuffer from Active Queue */
            if(NULL != fbStart->activeQueue[fbStart->readIndexActiveQue])
            {
                fbPtrQF = fbStart->activeQueue[fbStart->readIndexActiveQue];
                 /*If ActiveQueue is not NULL */
                if(NULL != fbPtrQF)
                {
                    if ((FALSE == vpfeCcdcHandle->CcdcActive)&&
                        (vpfeCcdcHandle->dataflow== PSP_VPFE_SENSOR_CCDC_PREVIEWER))
                    {
                        LLC_setCcdcOutputToSDRAMDisable(vpfeCcdcHandle->regs);
                        CSL_FINS(vpfePrevRegs->WSDR_ADDR,PREV_WSDR_ADDR_ADDR,
                                 (Uint32)fbPtrQF->frame.frameBufferPtr);
                        CSL_FINS(vpfePrevRegs->PCR, PREV_PCR_ENABLE,
                            (Uint32)CSL_PREV_PCR_ENABLE_PREV_ENABLE);
                    }
                    if ((FALSE == vpfeCcdcHandle->CcdcActive))
                    {
                        LLC_setCcdcFBStartAddress(vpfeCcdcHandle->regs,
                                                  fbPtrQF->frame.frameBufferPtr);
                        LLC_setCcdcOffset(vpfeCcdcHandle->regs,fbPtrQF->pitch);
                        LLC_setCcdcCaptureOn(vpfeCcdcHandle->regs);
                        /* Register the Interrupt */
                        if(NULL!= vpfeDeviceHandleCC)
                        {
                            _DDA_VPFERegisterIntrHandler(0,
                                            vpfeDeviceHandleCC->IntNum);
                        }
                        vpfeCcdcHandle->CcdcActive = TRUE;
                        result = DDC_VPFE_SOK;
                    }
                }
            }
            else
            {
                result = DDC_VPFE_E_FAIL;
            }
        }
        else
        {
            result = DDC_VPFE_SOK;
        }
    }

    return result;
}
/**
  * \brief      Function for Field Separate
  */
DDC_VPFEStatus DDC_VPFEFieldSeparate(Ptr handle)
{
    DDC_VPFEStatus result = DDC_VPFE_E_FAIL;
    DDC_VPFECcdcObject * vpfeCcdcHandle = NULL;
    if (NULL!= handle)
    {
        vpfeCcdcHandle = (DDC_VPFECcdcObject *)handle;
        LLC_setLnOfstEvnLnEvnFld_1Line(vpfeCcdcHandle->regs);
        LLC_setLnOfstOddLnEvnFld_1Line(vpfeCcdcHandle->regs);
        LLC_setLnOfstEvnLnOddFld_1Line(vpfeCcdcHandle->regs);
        LLC_setLnOfstOddLnOddFld_1Line(vpfeCcdcHandle->regs);
        vpfeCcdcHandle->FieldType = DDC_VPFE_FIELD_SEPARATE;
        result = DDC_VPFE_SOK;
    }
    return result;
}
/**
  * \brief      Function for Field Interlaced
  */
DDC_VPFEStatus DDC_VPFEFieldInterlaced(Ptr handle)
{
    DDC_VPFEStatus result = DDC_VPFE_E_FAIL;
    DDC_VPFECcdcObject * vpfeCcdcHandle = NULL;
    if (NULL!= handle)
    {
        vpfeCcdcHandle = (DDC_VPFECcdcObject *)handle;
        LLC_setLnOfstEvnLnEvnFld_2Lines(vpfeCcdcHandle->regs);
        LLC_setLnOfstOddLnEvnFld_2Lines(vpfeCcdcHandle->regs);
        LLC_setLnOfstEvnLnOddFld_2Lines(vpfeCcdcHandle->regs);
        LLC_setLnOfstOddLnOddFld_2Lines(vpfeCcdcHandle->regs);
        vpfeCcdcHandle->FieldType = DDC_VPFE_FIELD_INTERLACED;
        result = DDC_VPFE_SOK;
    }
    return result;
}

/**
  * \brief      Get next frame buffer address
  */
FVID_Frame * DDC_VPFEGetFBAddress(const DDC_VPFEFrameBufferObject * fbGN)
{
    FVID_Frame * retHandle  = NULL;
    Uint32 index = 0x0;
    /*Check the FrameBuffer not NULL */
    if (NULL != fbGN)
    {
        index = (fbGN->readIndexActiveQue)% DDC_VPFE_MAX_FRAME_BUFFERS;
        if(NULL!= fbGN->activeQueue[index])
        {
            retHandle = fbGN->activeQueue[index];
        }
    }
    return retHandle;
}
/**
  * \brief      Config the VDINT equal to Number of lines
  */
DDC_VPFEStatus DDC_VPFEConfigVDINT(Ptr handle,Ptr cmdArg)
{
    DDC_VPFECcdcObject * vpfeCcdcHandle  = NULL;
    DDC_VPFEStatus  result = DDC_VPFE_E_FAIL;
	Uint32 vdIntCount = 0;
    if((NULL== handle)||(NULL == cmdArg))
    {
        result = DDC_VPFE_E_FAIL;
    }
    else
    {
        vpfeCcdcHandle = (DDC_VPFECcdcObject *)handle;
		vdIntCount = *(Uint32 *)cmdArg;
		/* Check for the line boundary before configuring VDINT */
		if( (vdIntCount << vpfeCcdcHandle->CcdcParams.ffMode) >
											vpfeCcdcHandle->CcdcParams.height)
	    {
	        result = DDC_VPFE_INVALID_PARAM;
	    }
		else
		{
	        LLC_configVDINT(vpfeCcdcHandle->regs,vdIntCount);
	        result = DDC_VPFE_SOK;
		}
    }
    return result;
}
