/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/
/**
 *  \file   ddc_vpfe.h
 *
 *  \brief  DDC header file for VPFE
 *
 *  This file contains the dependencies across the VPFE driver and does
 *  not have dependencies specific to a plane or VENC or FE however it
 *  contains elements that may be used in one or all of the above.
 *
 *  (C) Copyright 2010, Texas Instruments, Inc
 *
 *  \note       Set tabstop to 4 (:se ts=4) while viewing this file in an
 *              editor
 *
 *  \author     Maulik Desai
 *
 *  \version    0.1     Created
 */
#ifndef _DDC_VPFE_H
#define _DDC_VPFE_H

#include <ti/sdo/pspdrivers/system/dm6437/bios/evmDM6437/video/psp_vpss_extvidDecoder.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osSem.h>

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
                                Macros
******************************************************************************/
/**< Max buffers associated within driver           */
#define DDC_VPFE_MAX_FRAME_BUFFERS  (20u)

/*-----------------------------------------------------------------------------
                              DDC Channel State
-----------------------------------------------------------------------------*/
/**
 * \brief       This enum contains description about the state of the
 *              plane(CCDC). Individually these are termed as channels.
 */
typedef enum _DDC_VPFEState
{
    /**< VPFE channel closed state for Ccdc                                  */
    DDC_VPFE_CLOSED,
    /**< VPFE channel open state for Ccdc                                    */
    DDC_VPFE_OPENED
} DDC_VPFEState;

/******************************************************************************
                                 Enumerations
******************************************************************************/
/**
 * This enum describes the various states of VPFE driver goes through
 * the entire cycle.
 *
 */
typedef enum VPSS_STATE
{
    VPSS_DELETED,
    VPSS_CREATED,
    VPSS_OPENED,
    VPSS_CLOSED
} vpssState;

/*-----------------------------------------------------------------------------
                                Queue Types
-----------------------------------------------------------------------------*/
/**
 * \brief       This enum contains the indexes for active or free queue. The
 *              VPSS driver uses the active queue and the free queue for
 *              implementing the flipping of the frame buffers
 */
typedef enum _DDC_VPFEQueueType
{
    /**< VPFE Free Queue Type                                                */
    DDC_VPFE_FREE_QUEUE = 0,
    /**< VPFE Active Queue Type                                              */
    DDC_VPFE_ACTIVE_QUEUE
} DDC_VPFEQueueType;
/**
 * \brief       This enum contains Field type.This enum can be used as cmd
 *              for IOCTL.
 */
typedef enum _DDC_VPFEFieldType
{
    /**< Field Type - INTERLACED                                             */
    DDC_VPFE_FIELD_INTERLACED = 0,
    /**< Field Type - INTERLACED                                             */
    DDC_VPFE_FIELD_SEPARATE
}DDC_VPFEFieldType;

/*-----------------------------------------------------------------------------
                             Error Codes
-----------------------------------------------------------------------------*/
/**
 * \brief       This enum contains DDC related error codes that will be passed
 *              to the DDA layer.
 */
typedef enum _DDC_VPFEStatus
{
    /**< Operation failed code for DDC related functionality                 */
    DDC_VPFE_E_FAIL = 0,
    /**< Operation passed code for DDC related functionality                 */
    DDC_VPFE_SOK,
    /**< Invalid state error code for DDC related functionality              */
    DDC_VPFE_INVALID_STATE,
    /**< Invalid param error code for DDC related functionality              */
    DDC_VPFE_INVALID_PARAM,
    /**< Invalid module error code for DDC related functionality             */
    DDC_VPFE_E_RESOURCES,
    /**< Invalid module error code for DDC related functionality             */
    DDC_VPFE_NO_MEMORY,
    /**< No memory error code for DDC related functionality                  */
    DDC_VPFE_NOT_SUPPORTED
} DDC_VPFEStatus;

/*-----------------------------------------------------------------------------
                             Field Id
-----------------------------------------------------------------------------*/
/**
 * \brief       This enum is used to get the current field information.
 */
typedef enum _DDC_VPFEFieldId
{
    /**< ODD field information                                               */
    DDC_VPFE_1ST_FIELD = 0,
    /**< Even field information                                              */
    DDC_VPFE_2ND_FIELD
} DDC_VPFEFieldId;

/*-----------------------------------------------------------------------------
                           Frame Buffer Object
-----------------------------------------------------------------------------*/
/**
 * \brief       This structure defines the frame buffer object which contains
 *              the relevant elements responsible for the flipping of multiple
 *              frame buffers and produce a seamless display or capture.
 */
typedef struct _DDC_VPFE_FrameBufferObject
{
    Uint                         writeIndexActiveQue;
    /**< Variable to write index active queue                              */
    Uint                         readIndexActiveQue;
    /**<  Variable to read index active queue                              */
    Uint                         writeIndexFreeQue;
    /**< Variable to write index free queue                                */
    Uint                         readIndexFreeQue;
    /**<  Variable to read index free queue                                */
    FVID_Frame        *activeQueue[DDC_VPFE_MAX_FRAME_BUFFERS];
    /**<  Array to access the various VpssSurfaceParams                    */
    FVID_Frame        *freeQueue[DDC_VPFE_MAX_FRAME_BUFFERS];
    /**<  Array to access the various VpssSurfaceParams                    */
    FVID_Frame     *tempfbAddFree;
    /**<  Temporary address to store framebuffer address                   */
} DDC_VPFEFrameBufferObject;

/* ------------------------------------------------------------------------*/
/* -----------------------------| CCDC Object |----------------------------*/
/* ------------------------------------------------------------------------*/
/**
  * \brief      Structure to define the CCDC object which includes relevant
  *             element for CCDC functionality
  */
typedef struct _DDC_VPFECcdcObject
{
    PSP_VPFE_Id                     channelId;
    /**<    Channel id for Ccdc object                        */
    DDC_VPFEState                   state;
    /**<    Vpss state either closed or open for Ccdc         */
    Ptr                             regs;
    /**<    Handle for vpss register                          */
    Bool                            CcdcActive;
    /**<    Ccdc Active or not                                */
    PSP_VPFECcdcConfigParams        CcdcParams;
    /**<    Handle for access the Ccdc config params          */
    struct _DDC_VPFEDeviceObject    *vpssDeviceHandle;
    /**<    Handle for accessing vpss device object           */
    PAL_OsSemHandle                dequeueSem;
    /**<    Semaphore Handle                                  */
    Bool                           dequeueWaitFlag;
    /**<    Semaphore Wait Flag                               */
    Bool                           fieldId;
    /**<    Field Id for Interlaced Mode                      */
    Ptr                           fbPtr;
    /**< Frame Buffer Pointer                                 */
    DDC_VPFEFrameBufferObject       fb;
    /**< Handle for accessing the frame buffer object         */
    PSP_VPSS_ExtVideoDecoderFxns      *extVidDecFxn;
    /**< External Video Decoder Function pointer              */
    Ptr                               extVidDecHandle;
    /**< External Video Decoder Handle                        */
    PSP_VPFE_CCDC_DataFlow dataflow;
    /**< DataFlow Path for CCDC specifies the Input to
         Output path                                          */
    DDC_VPFEFieldType              FieldType;
    /**< Field Type : Separate field or Interlaced Field      */
} DDC_VPFECcdcObject;

/* ------------------------------------------------------------------------- */
/* ---------------------| Device Object     |------------------------------- */
/* ------------------------------------------------------------------------- */
/**
  * \brief      Stucture for the vpfe device object as a whole
  */
typedef struct _DDC_VPFEDeviceObject
{
    Uint                            vpssInstanceId;
    /**<  Vpss device id                                    */
    vpssState                       state;
    /**<  Vpss channel state                                */
    Uint                            IntNum;
    /**<  Value for Front-end device object                 */
    DDC_VPFECcdcObject              ccdc;
    /**< Handle for  vpss Ccdc object                       */
} DDC_VPFEDeviceObject;

/******************************************************************************
                               INTERFACE FUNCTIONS
                       This Functions are called from the DDA Layer
******************************************************************************/
/**
  * \brief      This function validate the basic parameters for various the
  *             channels, configure the hardware and return the handle to the
  *             specific VPFE channel CCDC
  *
  * \param      id         [IN]        Id of the channel for which
  *                                    the handle is to be opened.
  *
  * \param      params     [IN]        Parameters with which the
  *                                    channel has to be created.
  *
  * \returns    Handle to the specific channel
  */
Ptr DDC_VPFEOpenHandle(PSP_VPFE_Id id, Ptr params);

/**
  * \brief      This function closes the handle of the corresponding channel
  *
  * \param      handle    [IN]        Handle of the channel for
  *                                   which the handle to be
  *                                   closed.
  *
  * \returns    status    [OUT]       Status of the operation.
  *
  * \returns    Count of the number of channels currently in open state
  */
DDC_VPFEStatus DDC_VPFECloseHandle(Ptr handle);

/**
  * \brief      This function is used to check if the handle is in closed state
  *
  * \returns    DDC_VPSS_E_FAIL - if channel handle is Open
  *             DDC_VPSS_SOK    - if channel handle is Close
  */
DDC_VPFEStatus DDC_VPFEIsChannelHandleClosed(Ptr handleICH);

/**
 *  \brief      This function returns the handle of the corresponding ID that
 *              has been passed.
 *
 *  \param      Id       [IN]        Enumerated ID for which the
 *                                   handle is required
 *
 *  \returns    Handle of CCDC for which the ID is passed.
 */
Ptr DDC_VPFEGetDeviceHandle(void);

/**
 *  \brief      This function is used to add frame buffers to either the
 *              ACTIVE queue or the FREE queue. This is a high layer function
 *              which will be invoking the DDC_VpssAddqueue function.
 *
 *              On exit this fucnition returns the status of the operation that
 *              was performed on the queue.
 *
 *  \param      handleQF  [IN]      Handle of CCDC/PLANES for
 *                                  which the queuing of the
 *                                  frame buffer has to be
 *                                  performed.
 *
 *  \param      fbPtrQF   [IN]      Pointer to the frame buffer
 *                                  that will be updated in the
 *                                  queue.
 *
 *  \returns    Status of the operation performed
 */
DDC_VPFEStatus DDC_VPFEQueueFrameBuffer(Ptr                     handleQF,
                                        FVID_Frame * fbPtrQF);

/**
 *  \brief      This function is used to remove frame buffers from either the
 *              ACTIVE queue or the FREE queue. This is a high layer function
 *              which will be invoking the DDC_VpssDelqueue function.
 *
 *              On exit this fucnition returns the status of the operation that
 *              was performed on the queue.Also it returns the pointer to the
 *              frame buffer that has been removed or dequeued.
 *
 *  \param      handleDQ                [IN]        Handle of CCDC/PLANES for
 *                                                  which the queuing of the
 *                                                  frame buffer has to be
 *                                                  performed.
 *
 *  \param      deQueuedFrameBufferDQ   [OUT]       Pointer to the frame buffer
 *                                                  that has been removed from
 *                                                  the driver.
 *
 *  \returns    Status of the operation performed
 */
DDC_VPFEStatus DDC_VPFEDeQueueFrameBuffer(Ptr                   handleDQ,
                         FVID_Frame ** deQueuedFrameBufferDQ);
/**
 *  \brief      Validate the frame buffers that are allocated by the application
 *
 *              This function is used to validate the frame buffers that are
 *              allocated by the application. Function returns a SUCCESS if the
 *              frame buffer in the surface params is valid.
 *
 *              The surface parameters that this function validates are all
 *              except the Pixelparams because this parameter may change.
 *              If not then the function returns a failure.
 *
 *  \param      surfVF       IN]        Details of the frame buffer
 *                                      to be validated.
 *
 *  \param      handleVF     IN]        Handle for video frame
 *
 *  \returns    SUCCESS/FAILURE of status
 */
DDC_VPFEStatus DDC_VPFECheckFrameBuffer(Ptr                   handleVF,
                                        FVID_Frame *const surfVF);

/**
 *  \brief      This is the front end ISR. This will be invoked at every VSYNC
 *              interrupt.
 *
 *              If the CCDC is in INTERLACED mode then next valid frame buffer
 *              pointer will be updated when an EVEN field has been captured.
 *
 *              If the CCDC is in PROGRESSIVE mode then the next valid frame
 *              buffer address will be updated in every VSYNC interrupt
 *
 *              At each VSYNC interrupt the callback function that was
 *              registered during the OPEN call will be called.
 *
 *  \param      arg          [IN]       Future Use
 *
 *  \returns    None
 */

void DDC_VPFEIsr(Uint32 arg);

/**
  * \brief      It will Perform register overlaying
  *
  */
void DDC_VPFERegisterOverlaying(void);
/**
  * \brief      Initialize VPSS Module and related VPSS hardware parameters
  *
  */
void DDC_VPFEInitializeVPSSModule(void);
/**
  * \brief      Set device ISR number
  *
  */
void DDC_VPFESetInterruptNumber();
/**
  * \brief      Application callback function
  *
  * \param      idCACF                  [IN]      Module Id - CCDC
  *
  * \param      currentFieldCACF        [IN]      Field Id (Even/Odd)
  *
  */
void DDC_VPFE_AppCallbackFunction(PSP_VPFE_Id      idCACF,
                                  DDC_VPFEFieldId   currentFieldCACF);
/**
  * \brief      Validate the frame buffer
  *
  * \param      surfVF                  [IN]      ValidateParams Pointer to
  *                                               validate Pitch
  *
  */
DDC_VPFEStatus DDC_VPFEValidateFrameBuffer(PSP_VPFEValidateParams *surfVF);

/******************************************************************************
                                      LOCAL FUNCTIONS
                       This Functions are called at DDC Layer functions
******************************************************************************/

/**
 *  \brief      This function is used to add the frame buffers to either the
 *              ACTIVE queue or the FREE queue.
 *
 *              On exit this fucnition returns the status of the operation that
 *              was performed on the queue.
 *
 *  \param      qTypeAQ           [IN]        Index of the Queue where the
 *                                            frame buffer has to be added
 *
 *                                            ACTIVE and FREE queues are
 *                                            the 2 types of queues used.
 *
 *  \param      handle            [IN]        Handle of CCDC
 *
 *  \param      fbAddressAQ       [IN]        Pointer to the frame buffer
 *                                            to be queued.
 *
 *  \returns    Pass and fail status
 */

DDC_VPFEStatus DDC_VPFEAddQueue(DDC_VPFEQueueType         qTypeAQ,
                                Ptr                       handle,
                                FVID_Frame     *fbAddressAQ);

/**
 *  \brief      This function is used to remove frame buffers from either the
 *              ACTIVE queue or the FREE queue.
 *
 *              On exit this fucnition returns the status of the operation that
 *              was performed on the queue.
 *
 *  \param      qTypeDQ          [IN]        Index of the Queue where the
 *                                           frame buffer has to be
 *                                           removed.
 *
 *                                           ACTIVE and FREE queues are
 *                                           the 2 types of queues used.
 *
 *  \param      handle           [IN]        Handle of CCDC
 *
 *  \param      fbAddressDQ      [IN]        Pointer to the frame buffer
 *                                           that will be updated in this
 *                                           address.
 *
 *  \returns    It will return the pass and fail status
 */
DDC_VPFEStatus DDC_VPFEDelQueue(DDC_VPFEQueueType         qTypeDQ,
                                Ptr handle,
                                FVID_Frame      **fbAddressDQ);

/**
 *  \brief      This function returns the handle of the corresponding ID that
 *              has been passed.
 *
 *  \param      Id              [IN]        Enumerated ID for which the
 *                                          handle is required
 *
 *  \returns    Handle of CCDC for which the ID is passed.
 */
Ptr DDC_VPFEGetChannelHandle(PSP_VPFE_Id id);


/**
 *  \brief      This function returns the ID of the corresponding handle that
 *              has been passed.
 *
 *  \param      handle         [IN]        Handle of CCDC for which
 *                                         ID is required.
 *
 *  \returns    Enumerated ID for the handle passed.
 */
PSP_VPFE_Id DDC_VPFEGetChannelId(Ptr handle);

/**
 *  \brief      This function is used get the next frame buffer address to be
 *              used in the DMA address for either display or capture.
 *
 *              On exit this fucnition returns pointer to the frame buffer that
 *              can be updated in the DMA address
 *
 *  \param      fbGN          [IN]        Frame buffer object which
 *                                        contains the queue for a
 *                                        particular handle.
 *
 *  \returns    Address of the next valid frame buffer that can be used in the
 *              DMA address.
 */

FVID_Frame * DDC_VPFEGetNxtFBAddress(const DDC_VPFEFrameBufferObject * fbGN);
/**
  * \brief      This function will program the default values and hence reset
  *             the frame buffer object
  *
  * \param      fbRF         [IN]        Pointer to the frame buffer
  *                                      object to be reset.
  */
void DDC_VPFEResetFrameBufferObject(DDC_VPFEFrameBufferObject * fbRF);

/**
  * \brief      This function initialize VPBE Module and related  VPBE
  *             hardware parameters
  */
void DDC_VPFEInitializeModule(void);
/**
 *  \brief      This function is used to allocate the memory to the Frame Buffer
 *
 *  \param      handle                  [IN]        Handle of CCDC
 *  \param      allocFrameBuffer        [IN}        Pointer to allocate the memory
 *
 *  \returns    Status                              Successs or Failure
 */
DDC_VPFEStatus DDC_VPFEAllocFrameBuffer(Ptr handle ,
                                        FVID_Frame ** allocFrameBuffer);
/**
 *  \brief      This function is used to allocate the memory to the Frame Buffer
 *
 *  \param      handle                  [IN]        Handle of CCDC
 *  \param      allocFrameBuffer        [IN}        Pointer to Free the Allocated
 *                                                  Frame Buffer memory
 *
 *  \returns    Status                              Successs or Failure
 */
DDC_VPFEStatus DDC_VPFEFreeFrameBuffer(Ptr handle,
                                       FVID_Frame ** allocFrameBuffer);
/**
 *  \brief      This function is used to allocate the memory to the Frame Buffer
 *
 *  \param      handle          [IN]       Handle of CCDC
 *  \param      Cmd             [IN]        TVP5146 video Decoder command
 *  \params     cmdarg          [IN]        cmd argument where config,control or
 *                                          status params of TVP5146 VD can passed
 *
 *  \returns    Status                     Successs or Failure
 */
DDC_VPFEStatus DDC_ConfigextVDParams(Ptr handle, Uint32 cmd ,Ptr cmdArg);
/**
 *  \brief      This function is used to update the hardware address to next frame
 *              buffer
 *
 *  \param      idUN                  [IN]       Front end module id
 */
void DDC_VPFEUpdateFbAdress(PSP_VPFE_Id idUN);
/**
 *  \brief      This function is used to update the Frame buffer in the free queue
 *              buffer
 *
 *  \param      idUN                  [IN]       Front end module id
 */
void DDC_VPFEUpdateFreeQueue(PSP_VPFE_Id idUN);
/**
 *  \brief      This function is used to Start the CCDC Engine
 *
 *  \param      handle                  [IN]       CCDC Handle
 */

DDC_VPFEStatus DDC_VPFEStopEngine(Ptr handle);
/**
 *  \brief      This function is used to Stop the CCDC Engine
 *
 *  \param      handle                  [IN]       CCDC Handle
 */

DDC_VPFEStatus DDC_VPFEStartEngine(Ptr handle);
/**
 *  \brief      This function is used to configure in Field Separate mode
 *
 *  \param      handle                  [IN]       CCDC Handle
 */
DDC_VPFEStatus DDC_VPFEFieldSeparate(Ptr handle);
/**
 *  \brief      This function is used to configure in Field Interlaced mode
 *
 *  \param      handle                  [IN]       CCDC Handle
 */
DDC_VPFEStatus DDC_VPFEFieldInterlaced(Ptr handle);
/**
 *  \brief      This function is used to Stop the CCDC Engine
 *
 *  \param      fbGN                  [IN]       Framebuffer object pointer
 */
FVID_Frame * DDC_VPFEGetFBAddress(const DDC_VPFEFrameBufferObject * fbGN);

DDC_VPFEStatus DDC_VPFEConfigVDINT(Ptr handle,Ptr cmdArg);
#ifdef __cplusplus
}
#endif

#endif /*_DDC_VPFE_H */
