/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2007 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \mainpage    PSP VPORT Driver on BIOS
 *
 *  \section    Introduction
 *
 *  \subsection    xxx    Purpose and Scope
 *  The purpose of this document is to explain the VPORT driver written in
 *  DSP/BIOS for DM648 platform.
 *
 *  \subsection    aaa    VPORT Controller
 *  The DM648 device has five video port peripherals. The video port peripheral
 *  can operate as a video capture port or video display port. The port consists
 *  of two channels: A and B. A 5120-byte capture/display buffer is splittable
 *  between the two channels. The entire port (both channels) is always
 *  configured for either video capture or display only.
 *  For video capture operation, the video port may operate as two 8-bit
 *  channels of BT.656 or raw video capture; or as a single channel of 8-bit
 *  BT.656, 8-bit raw video, 16-bit Y/C video or 16-bit raw video.
 *  For video display operation, the video port may operate as a single channel
 *  of 8-bit BT.656; or as a single channel of 8-bit BT.656, 8-bit raw video,
 *  16-bit Y/C video, or 16-bit raw video. It may also operate in a two channel
 *  8-bit raw mode in which the two channels are locked to the same timing.
 *  Channel B is not used during single channel operation.
 *
 *  \subsection    bbb    DSP/BIOS System
 *  DSP/BIOS is a real time kernel provided by TI to use with its DSP
 *  processors.
 *
 *
 *  \subsection    References
 *    -# VPORT functional specification, <b>spruem1.pdf</b>
 *    -# Design Document for VPORT driver,
 *                         <b>BIOS_VPORT_Driver_Arch.pdf</b>
 *    -# DSP/BIOS driver development <b>SPRU616.pdf</b>
 *
 *  \file  _vport.h file
 *  This header file defines common channel object and channel status code
 *  for both capture and display
 *
 */

#ifndef __VPORT_H
#define __VPORT_H

#ifdef __cplusplus
extern "C" {
#endif

#include <std.h>
#include <iom.h>

#include <ti/sdo/edma3/drv/edma3_drv.h>
#include <ti/sdo/pspdrivers/drivers/vport/vport.h>
#include <ti/sdo/pspdrivers/drivers/vport/vportcap.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/fvid.h>
#include <ti/sdo/pspdrivers/soc/dm648/dsp/cslr_vport.h>


#ifdef _DEBUG
/* \brief VPORT debug macro. Disable this by default */
/* #define VPORT_DEBUG */
#endif  /* #ifdef _DEBUG */

/** \brief Remaining number of lines (safety room) to transfer while determining
           not too late check in ISR. Since global interrupts are not disabled
           in ISR context, this is kept a bit high compared to submit room */
#define NOT_TOO_LATE_LINE_ROOM_ISR      (10)
/** \brief Remaining number of lines (safety room) to transfer while determining
           not too late check in ISR. Since global interrupts are disabled in
           submit call, this is kept a bit low compared to ISR room */
#define NOT_TOO_LATE_LINE_ROOM_SUBMIT   (5)

/* channel status */
/** \brief channel operation started */
#define _VPORT_STARTED                  (0x10)
/** \brief buffers allocated */
#define _VPORT_BUF_ALLOCATED            (0x08)
/** \brief channel EDMA configured */
#define _VPORT_READY                    (0x04)
/** \brief channel registers configured */
#define _VPORT_CFGED                    (0x02)
/** \brief VPORT channel opened */
#define _VPORT_OPENED                   (0x01)

/** \brief VPORT minimum buffers required by driver */
#define _VPORT_MIN_BUF                  (2)

/** \brief Flag for legacy (DM642) mode */
#define LEGACY_MODE                     (1)
/** \brief Flag for normal operation mode */
#define NORMAL_MODE                     (0)

/** \brief Mask for 10 bit data transfer mode */
#define _VPORT_MASK_10BIT               (1)
/** \brief Mask for RAW mode */
#define _VPORT_MASK_RAW                 (2)

/** \brief Number of EDMA channels */
#define _VPORT_NUM_EDMA_CHANS           (3)
/** \brief Number of VPORT supported channels */
#define _VPORT_CHAN_CNT                 (2)

/** \brief Negative value function pointer */
#define INVCB                           ((Fxn)(-1))

/* Shift values for OPT field */
/** \brief EDMA DAM shift value */
#define EDMA_DAM_SHIFT                  (1)
/** \brief EDMA synchronous DIM shift value */
#define EDMA_SYNCDIM_SHIFT              (2)
/** \brief EDMA FIFO Width shift value */
#define EDMA_FWID_SHIFT                 (8)
/** \brief EDMA TCC shift value */
#define EDMA_TCC_SHIFT                  (12)
/** \brief EDMA TCINTEN shift value */
#define EDMA_TCINTEN_SHIFT              (20)
/** \brief EDMA TCC mask value */
#define EDMA_TCC_MASK                   (0xFFFC0FFF)

/* EDMA 3 TC number to be used for each of the VP channel */
/** \brief EDMA event queue for VPORT 0 channel A */
#define EDMA_CHA_VP0EVTQA               (2)
/** \brief EDMA event queue for VPORT 0 channel B */
#define EDMA_CHA_VP0EVTQB               (2)

/** \brief EDMA event queue for VPORT 1 channel A */
#define EDMA_CHA_VP1EVTQA               (3)
/** \brief EDMA event queue for VPORT 1 channel B */
#define EDMA_CHA_VP1EVTQB               (3)

/** \brief EDMA event queue for VPORT 2 channel A */
#define EDMA_CHA_VP2EVTQA               (2)
/** \brief EDMA event queue for VPORT 2 channel B */
#define EDMA_CHA_VP2EVTQB               (2)

/** \brief EDMA event queue for VPORT 3 channel A */
#define EDMA_CHA_VP3EVTQA               (2)
/** \brief EDMA event queue for VPORT 3 channel B */
#define EDMA_CHA_VP3EVTQB               (2)

/** \brief EDMA event queue for VPORT 4 channel A */
#define EDMA_CHA_VP4EVTQA               (2)
/** \brief EDMA event queue for VPORT 4 channel B */
#define EDMA_CHA_VP4EVTQB               (2)

/** \brief  command to enable 3:2 asynchronous driver call-back */
#define VPORTDIS_CMD_ASYNC_MODE_ENABLE         (VPORT_CMD_USER + 1)
/** \brief  command to disable 3:2 asynchronous driver call-back */
#define VPORTDIS_CMD_ASYNC_MODE_DISABLE        (VPORT_CMD_USER + 2)
/** \brief  command to reset count in 3:2 asynchronous driver call-back mode */
#define VPORTDIS_CMD_ASYNC_MODE_RESET_FRAMECT  (VPORT_CMD_USER + 3)

/** \brief  disable 3:2 pull-down */
#define VPORTDIS_PULLDOWN_DISABLE       (0)
/** \brief  enable 3:2 pull-down for interlaced mode */
#define VPORTDIS_PULLDOWN_24P_30I       (1)
/** \brief  disable 3:2 pull-down for progressive mode */
#define VPORTDIS_PULLDOWN_24P_60P       (2)

/**
 *  \brief Application call-back function for 3:2 pull-down
 *  for 3:2 pull-down from 24p to 30i, the frame count varies from 0-4.
 */
typedef void (*VPORTDIS_AsyncModeCallBack)
                                (int id, int frameCt, FVID_Frame **frame);

/** \brief Async mode 3:2 pull-down parameters Structure*/
typedef struct {
    VPORTDIS_AsyncModeCallBack callBackFxn;
    /**< async mode callback function pointer */
    Int arg;
    /**< argument for async mode callback */
    Int mode;
    /**< type of pull down mode as defined above */
    Int irqId;
    /**< irq Id for async mode Isr */
} VPORTDIS_AsyncModeParam;


/**
 *  \brief  VPORT channel object Structure
 *  video port channel object structure to maintain driver's internal
 *  parameters and state.
 */
typedef struct {
    Uns                 status;
    /**< Channel status opened, configured, allocated, etc...*/
    Uns                 portNum;
    /**< Port number for which channel is created */
    Uns                 chanNum;
    /**< Channel number of port */
    CSL_VportRegsOvly   vportRegs;
    /**< Pointer to VPORT register structure defined in cslr_vport.h
     * This will point to base address of Video Port
     */
    Uns                 edmaChanNum[_VPORT_NUM_EDMA_CHANS];
    /**< Array to store EDMA channel number */
    Uns                 edmaAddr[_VPORT_NUM_EDMA_CHANS];
    /**< Array to store VPORT FIFO address for EDMA transfer */
    Uns                 vpFifoAddr;
    /**< VPORT channel FIFO base address */
    EDMA3_RM_EventQueue evtQueue;
    /**< EDMA 3 Transfer Controller to use for the channel */
    Uns                 hEdmaChan[_VPORT_NUM_EDMA_CHANS];
    /**< Array to store EDMA channel handle */
    Uns                 edmaLinkChanNum[_VPORT_NUM_EDMA_CHANS * 4];
    /**< Array to store EDMA link channel number */
    EDMA3_DRV_PaRAMRegs *hEdmaChanPhyAddr[_VPORT_NUM_EDMA_CHANS];
    /**< Physical address of EDMA channels for direct accessing */
    EDMA3_DRV_PaRAMRegs *hEdmaLinkChanPhyAddr[_VPORT_NUM_EDMA_CHANS * 4];
    /**< Physical address of EDMA link channels for direct accessing */
    Uns                 vIntMask;
    /**< VPORT interrupt mask value */
    FVID_Frame          viops[VPORT_MAX_NUM_FRMBUFS];
    /**< To store frame buffers in Legacy mode */
    QUE_Elem            qIn;
    /**< Queue element to maintain buffer queue */
    QUE_Elem            qOut;
    /**< Queue element to maintain buffer queue */
    QUE_Elem            qPullDown;
    /**< Queue element */
    EDMA3_DRV_Handle    hEdma;
    /**< To store EDMA driver handle */
    Uns                 tcc[3];
    /**< EDMA Transfer Complete Code */
    Bool                mergeFlds;
    /**< Flag to identify fields are merged or separated */
    Bool                interlaced;
    /**< Flag to identify fields are interlaced or progressive */
    IOM_TiomCallback    cbFxn;
    /**< IOM callback function */
    Arg                 cbArg;
    /**< IOM callback function argument */
    VPORT_IntCallBack   vIntFxn;
    /**< VPORT interrupt callback function */
    volatile Bool       queEmpty;
    /**< Flag to maintain queue status */
    Bool                scale;
    /**< Flag for scaling enabled or disabled */
    Bool                resmpl;
    /**< Flag for resampling enabled or disabled */
    Uns                 yPitch;
    /**< Y pitch value */
    Uns                 cPitch;
    /**< Cb/Cr pitch value */
    Uns                 numLines;
    /**< Number of lines on one frame */
    Uns                 numPixels;
    /**< Number of pixels in one line */
    Uns                 numLinesFld1;
    /**< Number of lines in one field */
    Uns                 numFrms;
    /**< Number of frame buffers used by driver */
    Uns                 numEvents;
    /**< Number of events to transfer one frame */
    Uns                 numEventsFld1;
    /**< Number of events to transfer one field */
    Uns                 yThrld;
    /**< Y fifo threshold value */
    Uns                 cThrld;
    /**< Cb/Cb fifo threshold value */
    FVID_Frame          *curViop;
    /**< Current video IO packet */
    FVID_Frame          *nextViop;
    /**< Next video IO packet */

    FVID_Frame          *mrViop;
    /**< Most recent video IO packet */
    Int                 mode;
    /**< Driver mode */
    EDC_Fxns           *edcFxns;
    /**< Pointer to function table of external device */
    Ptr                 edcHandle;
    /**< Handle of external device */
    IOM_Packet         *packetIOM;
    /**< Pointer to IOM packet */
    Int                 vIntCbArg;
    /**< Vport interrupt callback argument */
    Int                 segId;
    /**< Memory segment id to be used to allocate frame buffer memory */
    Int                 bufSz;
    /**< Buffer size in bytes */
    Int                 lastLineNum;
    /**< Last line number that has been transferred */
    Int                 nextEDMARlds;
    /**< Next EDMA reload entry */
    Int                 numTotalLinesFld1;
    /**< Number of total lines in field 1 */
    Bool                autoSyncEnable;
    /**< Flag for auto sync enable or disabled */

    /* for async mode and 3:2 pull-down support */
    Bool                asyncModeEnable;
    /**< Flag to enable asynchronous transfer mode */
    Int                 pullDownMode;
    /**< 3:2 pull down mode */
    Int                 asyncFrameCt;
    /**< Asynchronous transfer frame count */
    Fxn                 asyncCallBackFxn;
    /**< Asynchronous transfer callback function */
    Int                 frmSz;
    /**< Frame size */
    Int                 asyncCbArg;
    /**< Asynchronous callback argument */
    Int                 startFlag;
    /**< Start flag for asynchronous transfer mode */
    Int                 driverMode;
    /**< Driver operation mode  0 = normal mode, 1 = legacy mode */
    Int                 numQueBuf;
    /**< Number of queued buffer in normal mode */
    Int                 alignment;
    /**< Frame buffer alignment */
    Int                 fldOp;
    /**< Flag for field operation or frame operation */
    volatile CSL_VportCaptureRegs   *vportChanRegs;
    /**< Pointer to VPORT capture channel register structure defined in
     * cslr_vport.h. This will point to respective channel structure
     * (CSL_VportCaptureRegs) of CSL_VportRegs overlay structure
     * This is only used in capture driver
     */
    VPORTCAP_LineIntParam  lineIntInfo;
    /**< Structure variable containing Line interrupt parameters, This is only
      * used in capture driver
      */

    /* Debug counters */
#ifdef VPORT_DEBUG
    Uint32                  submitCount;
    /**< Total Submit count for the channel */
    Uint32                  errCount;
    /**< Display underrun or capture overrun error count */
    Uint32                  frameCount;
    /**< EDMA ISR frame count */
    Uint32                  shortFldDetectCount;
    /**< Short field detect count for capture driver */
    Uint32                  autoSyncCount;
    /**< Auto Sync count. This is used only in capture driver */
    Uint32                  isrSwapCount;
    /**< Current and Next viop swap count in EDMA ISR */
    Uint32                  isrSwapErrCount;
    /**< Invalid state error count while swapping current & next VIOP in ISR */
    Uint32                  reconfigErrCount;
    /**< Invalid state error count while reconfiguring EDMA channel */
    Uint32                  tooLateInISRCount[5];
    /**< EDMA reload and EDMA ISR race condition error count */
#endif  /* #ifdef VPORT_DEBUG */
} _VPORT_ChanObj;

/**
 *  \brief  VPORT external device object Structure
 *  video external device object structure to access external devices.
 */
typedef struct _VPORT_EDCObj {
    Int8* name;
    /**< Name of external device */
    EDC_Fxns* fxns;
    /**< Pointer to function table of external device */
} _VPORT_EDCObj;

#ifdef __cplusplus
}
#endif

#endif /* __VPORT_H */

/******************************************************************************/
/* end of file                                                                */
/******************************************************************************/
