/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2007 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \file vportcap.c file
 *
 *  This source file contains code for VPORT capture driver
 */

/*
 * ========= DSP/BIOS standard include files =========
 */
#include <std.h>

/*
 * ========= Chip-support library include files ===========
 */
#include <ti/sdo/pspdrivers/soc/dm648/dsp/soc.h>
#include <ti/sdo/pspdrivers/soc/dm648/dsp/cslr_vport.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osprotect.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osCache.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osmem.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osInt.h>
#include <ecm.h>

/*
 * ======== video driver specific include files ===========
 */
#include <ti/sdo/pspdrivers/drivers/vport/vport.h>
#include <ti/sdo/pspdrivers/drivers/vport/vportcap.h>
#include <ti/sdo/edma3/drv/edma3_drv.h>
#include <ti/sdo/pspdrivers/drivers/vport/src/_vportrmk.h>
#include <ti/sdo/pspdrivers/drivers/vport/src/_vport.h>

/*
 * ========== debug include files ==============
 */
/* to minimize code size and cycle count overhead of the driver
 * error checking is only performed at debug time
 */
#include <assert.h>

#ifdef VPORT_INSTRUMENTATION_ENABLED
#include <ti/sdo/pspdrivers/pal_os/bios/psp_log.h>
#endif /* VPORT_INSTRUMENTATION_ENABLED */

/*
 * ============ type defines and data structures ============
 */
/** \brief data structure for video port object */
typedef struct PortObj {
    Int                 status;
    /**< port status register, contains information on whether
     * port is opened, configured, etc.
     */

    CSL_VportRegsOvly   vportBase;
    /**< Pointer to VPORT register structure defined in cslr_vport.h
     * This will point to base address of Video Port
     */

    _VPORT_ChanObj      chanObj[2];
    /**< two channel objects for channel A & B.
     * This is only for capture operation
     */
} PortObj;

/*
 * ============ mini-driver API functions ============
 */
static Int mdBindDev(Ptr *devp, Int devid, Ptr devParams);
static Int mdControlChan(Ptr chanp, Uns cmd, Ptr args);
static Int mdCreateChan(Ptr *chanp,
                        Ptr devp,
                        String name,
                        Int mode,
                        Ptr chanParams,
                        IOM_TiomCallback cbFxn,
                        Ptr cbArg);
static Int mdDeleteChan(Ptr chanp);
static Int mdSubmitChan(Ptr chanp, IOM_Packet *packet);

/*
 * ============== local functions ===============
 */
static void captureEdmaISR(unsigned int tcc,
                           EDMA3_RM_TccStatus status,
                           void *appData);
static void captureISR(Int portNum);
static Int _configRegs(Ptr chanp, VPORTCAP_Params *params);
static Int _configChan(Ptr chanp, Ptr args);
static Int _configPort(Ptr chanp, Ptr args);
static Int _configEDMA(Ptr chanp);
static Int _covrRecover(Ptr chanp);
static Int _setVIntCb(Ptr chanp, Ptr args);
static Int _startCapture(Ptr chanp);
static Int _stopCapture(Ptr chanp);
static void _autoSync(_VPORT_ChanObj *chan);
static Int _getNumPendingIORqsts(Ptr chanp, Ptr args);
static Int _allocBuffer(_VPORT_ChanObj *chanp, FVID_Frame **newFrame);
static Int _freeBuffer(_VPORT_ChanObj *chanp, FVID_Frame **freeFrame);
static Int _setCovrIsr(Ptr, Int);
static Int _allocAssignBuffer(_VPORT_ChanObj *, FVID_Frame *);
static Int _cmdDequeue(_VPORT_ChanObj *, IOM_Packet *, Bool);
static Int _cmdQueue(_VPORT_ChanObj *, IOM_Packet *, Bool);
static Int _getChanParams(Ptr, VPORT_CurrParams *);
static Int _cmdGetBufs(_VPORT_ChanObj *, IOM_Packet *);
static Int _reconfigEDMAchan(Ptr);
static Int _setVportLineIntCB(Ptr, Ptr);
static Int _getLinesCapturedEDMA(Ptr, Ptr);
static Bool _isItTooLateInISR(_VPORT_ChanObj *chan);
static Int _updateEDMAParamAddress(const _VPORT_ChanObj *chan, Int offset,
                                   const FVID_Frame *viop);

/* ============ global and static variables ============= */
/** \brief VPORT capture function pointer table */
IOM_Fxns VPORTCAP_Fxns =
{
    mdBindDev,
    (IOM_TmdUnBindDev)IOM_mdNotImpl,
    mdControlChan,
    mdCreateChan,
    mdDeleteChan,
    mdSubmitChan
};

/** \brief Static allocation and initialization of port objects */
#ifndef VPORT_DEBUG
static
#endif  /* #ifndef VPORT_DEBUG */
PortObj portObjs[CSL_VIDEOPORT_PER_CNT] =
{
    /* video port 0 */
    {0, (CSL_VportRegs*)CSL_VIDEOPORT_0_REGS,
    /* channel A */
        {{0, 0, 0, (CSL_VportRegs*)CSL_VIDEOPORT_0_REGS, CSL_EDMA3_CHA_VP0EVTYA,
        CSL_EDMA3_CHA_VP0EVTUA, CSL_EDMA3_CHA_VP0EVTVA,
        0, 0, 0, CSL_VIDEOPORT0_CHANNELA_DATA,
        EDMA_CHA_VP0EVTQA},
    /* channel B */
        {0, 0, 1, (CSL_VportRegs*)CSL_VIDEOPORT_0_REGS, CSL_EDMA3_CHA_VP0EVTYB,
        CSL_EDMA3_CHA_VP0EVTUB, CSL_EDMA3_CHA_VP0EVTVB,
        0, 0, 0, CSL_VIDEOPORT0_CHANNELB_DATA,
        EDMA_CHA_VP0EVTQB}}},

    /* video port 1 */
    {0, (CSL_VportRegs*)CSL_VIDEOPORT_1_REGS,
    /* channel A */
        {{0, 1, 0, (CSL_VportRegs*)CSL_VIDEOPORT_1_REGS, CSL_EDMA3_CHA_VP1EVTYA,
        CSL_EDMA3_CHA_VP1EVTUA, CSL_EDMA3_CHA_VP1EVTVA,
        0, 0, 0, CSL_VIDEOPORT1_CHANNELA_DATA,
        EDMA_CHA_VP1EVTQA},
    /* channel B */
        {0, 1, 1, (CSL_VportRegs*)CSL_VIDEOPORT_1_REGS, CSL_EDMA3_CHA_VP1EVTYB,
        CSL_EDMA3_CHA_VP1EVTUB, CSL_EDMA3_CHA_VP1EVTVB,
        0, 0, 0, CSL_VIDEOPORT1_CHANNELB_DATA,
        EDMA_CHA_VP1EVTQB}}},

    /* video port 2 */
    {0, (CSL_VportRegs*)CSL_VIDEOPORT_2_REGS,
    /* channel A */
        {{0, 2, 0, (CSL_VportRegs*)CSL_VIDEOPORT_2_REGS, CSL_EDMA3_CHA_VP2EVTYA,
        CSL_EDMA3_CHA_VP2EVTUA, CSL_EDMA3_CHA_VP2EVTVA,
        0, 0, 0, CSL_VIDEOPORT2_CHANNELA_DATA,
        EDMA_CHA_VP2EVTQA},
    /* channel B */
        {0, 2, 1, (CSL_VportRegs*)CSL_VIDEOPORT_2_REGS, CSL_EDMA3_CHA_VP2EVTYB,
        CSL_EDMA3_CHA_VP2EVTUB, CSL_EDMA3_CHA_VP2EVTVB,
        0, 0, 0, CSL_VIDEOPORT2_CHANNELB_DATA,
        EDMA_CHA_VP2EVTQB}}},

    /* video port 3 */
    {0, (CSL_VportRegs*)CSL_VIDEOPORT_3_REGS,
    /* channel A */
        {{0, 3, 0, (CSL_VportRegs*)CSL_VIDEOPORT_3_REGS, CSL_EDMA3_CHA_VP3EVTYA,
        CSL_EDMA3_CHA_VP3EVTUA, CSL_EDMA3_CHA_VP3EVTVA,
        0, 0, 0, CSL_VIDEOPORT3_CHANNELA_DATA,
        EDMA_CHA_VP3EVTQA},
    /* channel B */
        {0, 3, 1, (CSL_VportRegs*)CSL_VIDEOPORT_3_REGS, CSL_EDMA3_CHA_VP3EVTYB,
        CSL_EDMA3_CHA_VP3EVTUB, CSL_EDMA3_CHA_VP3EVTVB,
        0, 0, 0, CSL_VIDEOPORT3_CHANNELB_DATA,
        EDMA_CHA_VP3EVTQB}}},

    /* video port 4 */
    {0, (CSL_VportRegs*)CSL_VIDEOPORT_4_REGS,
    /* channel A */
        {{0, 4, 0, (CSL_VportRegs*)CSL_VIDEOPORT_4_REGS, CSL_EDMA3_CHA_VP4EVTYA,
        CSL_EDMA3_CHA_VP4EVTUA, CSL_EDMA3_CHA_VP4EVTVA,
        0, 0, 0, CSL_VIDEOPORT4_CHANNELA_DATA,
        EDMA_CHA_VP4EVTQA},
    /* channel B */
        {0, 4, 1, (CSL_VportRegs*)CSL_VIDEOPORT_4_REGS, CSL_EDMA3_CHA_VP4EVTYB,
        CSL_EDMA3_CHA_VP4EVTUB, CSL_EDMA3_CHA_VP4EVTVB,
        0, 0, 0, CSL_VIDEOPORT4_CHANNELB_DATA,
        EDMA_CHA_VP4EVTQB}}}
};


/**
 *  \brief Initialize a given VPORT driver (instance)
 *
 *  Register all external devices to video port capture driver.
 *
 *  \param  devp        [IN]    Void pointer passed by the GIO Layer to
 *                              store the address of the port structure
 *  \param  devid       [IN]    VPORT instance number
 *  \param  devParams   [IN]    Initialization configuration structure
 *
 *  \return success             IOM_COMPLETED
 *          failure             IOM Error
 */
static Int mdBindDev(Ptr *devp, Int devid, Ptr devParams)
{
    Int portNum = devid;
    Int retVal = IOM_COMPLETED;
    CSL_VportChannelARegs *vportFifoARegs;
    CSL_VportChannelBRegs *vportFifoBRegs;

    assert(portNum < CSL_VIDEOPORT_PER_CNT);

    /* Assign port object pointer to devp */
    *devp = &portObjs[portNum];

    /* assign capture channel base address for channel A and B */
    portObjs[portNum].chanObj[0].vportChanRegs
            = &(portObjs[portNum].chanObj[0].vportRegs->CSL_VportCapARegs);
    portObjs[portNum].chanObj[1].vportChanRegs
            = &(portObjs[portNum].chanObj[1].vportRegs->CSL_VportCapBRegs);

    /* assign EDMA FIFO addresses for channel A */
    vportFifoARegs = (CSL_VportChannelARegs *)
                                (portObjs[portNum].chanObj[0].vpFifoAddr);
    portObjs[portNum].chanObj[0].edmaAddr[0] = (Uns)&(vportFifoARegs->YSRCA);
    portObjs[portNum].chanObj[0].edmaAddr[1] = (Uns)&(vportFifoARegs->CBSRCA);
    portObjs[portNum].chanObj[0].edmaAddr[2] = (Uns)&(vportFifoARegs->CRSRCA);

    /* assign EDMA FIFO addresses for channel B */
    vportFifoBRegs = (CSL_VportChannelBRegs *)
                                (portObjs[portNum].chanObj[1].vpFifoAddr);
    portObjs[portNum].chanObj[1].edmaAddr[0] = (Uns)&(vportFifoBRegs->YSRCB);
    portObjs[portNum].chanObj[1].edmaAddr[1] = (Uns)&(vportFifoBRegs->CBSRCB);
    portObjs[portNum].chanObj[1].edmaAddr[2] = (Uns)&(vportFifoBRegs->CRSRCB);

    if (NULL == devParams)
    {
        retVal = IOM_EBADARGS;
    }
    else
    {
        /* Call _configPort() to configure video port */
        retVal = _configPort(&portObjs[portNum], devParams);
    }

    return retVal;
}



/**
 *  \brief VPORT Ioctls
 *
 *  This function provides ioctl functionality for VPORT
 *
 *  \param  chanp       [IN]     VPORT Driver Channel Handle
 *  \param  cmd         [IN]     Operation to be performed, typically an enum
 *                               gets passed
 *  \param  args        [IN/OUT] Provides additional information related to the
 *                               operation
 *
 *  \return success              IOM_COMPLETED
 *          failure              Suitable error code
 */
static Int mdControlChan(Ptr chanp, Uns cmd, Ptr args)
{
    Int             retVal = IOM_EBADMODE;
    _VPORT_ChanObj  *chan = (_VPORT_ChanObj *)chanp;

    if (NULL == chan)
    {
        retVal = IOM_EBADARGS;
    }
    else
    {
        switch (cmd)
        {
            case VPORT_CMD_START:
                /* For NORMAL MODE, If EDMA is not configured then return
                 * ERROR
                 */
                if ((!(_VPORT_READY & chan->status))
                    || (chan->status & _VPORT_STARTED))
                {
                    retVal = IOM_EBADIO;
                }
                else
                {
                    retVal = _startCapture(chanp);
                }
                break;

            case VPORT_CMD_STOP:
                if (!(chan->status & _VPORT_STARTED))
                {
                    retVal = IOM_EBADIO;
                }
                else
                {
                    retVal = _stopCapture(chanp);
                }
                break;

            case VPORT_CMD_SET_VINTCB:
                if (NULL == args)
                {
                    retVal = IOM_EBADARGS;
                }
                else
                {
                    retVal = _setVIntCb(chanp, args);
                }
                break;

            case VPORT_CMD_COVR_RECOVER:
                if (!(_VPORT_READY & chan->status))
                {
                    retVal = IOM_EBADIO;
                }
                else
                {
                    retVal = _covrRecover(chanp);
                }
                break;

            case VPORT_CMD_CONFIG_CHAN:
                if (NULL == args)
                {
                    retVal = IOM_EBADARGS;
                }
                else
                {
                    retVal = _configChan(chanp, args);
                }
                break;

            case VPORT_CMD_GET_NUM_IORQST_PENDING:
                if (NULL == args)
                {
                    retVal = IOM_EBADARGS;
                }
                else if (!(_VPORT_READY & chan->status))
                {
                    retVal = IOM_EBADIO;
                }
                else
                {
                    retVal = _getNumPendingIORqsts(chanp, args);
                }
                break;

            case VPORT_CMD_GET_PARAMS:
                if (NULL == args)
                {
                    retVal = IOM_EBADARGS;
                }
                else
                {
                    retVal = _getChanParams(chanp, (VPORT_CurrParams *)args);
                }
                break;

            case FVID_ALLOC_BUFFER:
                /* This IOCTL is not supported in LEGACY MODE */
                if (LEGACY_MODE == chan->driverMode)
                {
                    retVal = IOM_ENOTIMPL;
                }
                else if (NULL == args)
                {
                    retVal = IOM_EBADARGS;
                }
                else
                {
                    /* function call for allocating buffer in normal mode */
                    retVal = _allocBuffer(chanp, args);
                }
                break;

            case FVID_FREE_BUFFER:
                /* This IOCTL is not supported in LEGACY MODE */
                if (LEGACY_MODE == chan->driverMode)
                {
                    retVal = IOM_ENOTIMPL;
                }
                else if (NULL == args)
                {
                    retVal = IOM_EBADARGS;
                }
                else
                {
                    /* function call to free the allocated buffer */
                    retVal = _freeBuffer(chanp, args);
                }
                break;

            case VPORTCAP_CMD_SET_LINE_INT:
                /* set vertical line interrupt callback function */
                if (NULL == args)
                {
                    retVal = IOM_EBADARGS;
                }
                else
                {
                    retVal = _setVportLineIntCB(chanp, args);
                }
                break;

            case VPORTCAP_CMD_GET_NUMLINES_CAPTURED:
                /* Get number of lines currently transferred by EDMA */
                if (NULL == args)
                {
                    retVal = IOM_EBADARGS;
                }
                else if (!(chan->status & _VPORT_STARTED))
                {
                    retVal = IOM_EBADIO;
                }
                else
                {
                    retVal = _getLinesCapturedEDMA(chanp, args);
                }
                break;

            default:
                if (chan->edcFxns != INV)
                {
                    /* Call EDC control function for EDC configuration */
                    retVal = chan->edcFxns->ctrl(chan->edcHandle,
                                                 cmd - VPORT_CMD_EDC_BASE,
                                                 (Arg)args);
                }
                else
                {
                    retVal = IOM_ENOTIMPL;
                }
                break;
        }
    }

    return retVal;
}



/**
 *  \brief Open Channel of the VPORT device
 *
 *  The function prepares channel of the driver instance for data transfers
 *  and returns an handle to the driver instance.
 *
 *  \param chanp        [IN]     pointer to the channel structure
 *  \param devp         [IN/OUT] pointer to the port structure
 *  \param name         [IN]     name of the driver
 *  \param mode         [IN]     mode of the channel i.e. INPUT or OUTPUT
 *  \param chanParams   [IN/OUT] Extra parameter (Not used in this driver)
 *  \param cbFxn        [IN]     callback to the GIO Layer
 *  \param cbArg        [IN]     arguments for the callback
 *
 *  \return                     On success returns IOM_COMPLETED
 *                              On failure returns suitable error code
 */
static Int mdCreateChan(Ptr *chanp,
                        Ptr devp,
                        String name,
                        Int mode,
                        Ptr chanParams,
                        IOM_TiomCallback cbFxn,
                        Ptr cbArg)
{
    Int                 chanNum;
    PortObj             *port;
    Int                 retVal = IOM_COMPLETED;
    unsigned int        linkTccId = 0;
    Int                 j;
    _VPORT_ChanObj      *chan;
    Int32               cmode;
    Int32               numFrmBufs;
    EDMA3_DRV_Result    result = EDMA3_DRV_SOK;

    /* For capture, mode must be IOM_INPUT */
    if (mode != IOM_INPUT)
    {
        return IOM_EBADARGS;
    }
    if (*name++ != '/')
    {
        return IOM_EBADARGS;
    }
    chanNum = *name++ - 'A';

    /* Return error if any pointer parameter is NULL */
    if ((INV == chanParams) || (NULL == chanParams) || (NULL == cbFxn) ||
        (NULL == chanp) || (NULL == devp) || (chanNum >= _VPORT_CHAN_CNT))
    {
        retVal = IOM_EBADARGS;
    }
    else
    {
        cmode = ((VPORTCAP_Params *)chanParams)->cmode;
        if ((cmode & _VPORT_MASK_RAW) &&
            (NULL == ((VPORTCAP_ParamsRaw *)chanParams)->hEdma))
        {
            retVal = IOM_EBADARGS;
        }
        else if (!(cmode & _VPORT_MASK_RAW) &&
                  (NULL == ((VPORTCAP_Params *)chanParams)->hEdma))
        {
            retVal = IOM_EBADARGS;
        }
    }

    if (IOM_COMPLETED == retVal)
    {
        port = (PortObj *)devp;
        if (port->chanObj[chanNum].edcFxns != INV)
        {
            /* open external device */
            port->chanObj[chanNum].edcHandle =
                        port->chanObj[chanNum].edcFxns->open(name, (Arg)INV);
            if (NULL == port->chanObj[chanNum].edcHandle)
            {
                retVal = IOM_EBADIO;
            }
        }

        if ((!(port->status & _VPORT_OPENED)) && (IOM_COMPLETED == retVal))
        {
            port->status |= _VPORT_OPENED;

            /* reset both channels */
            port->vportBase->CSL_VportCapARegs.VC_CTL
                |= CSL_VPORT_VC_CTL_RSTCH_MASK;
            port->vportBase->CSL_VportCapBRegs.VC_CTL
                |= CSL_VPORT_VC_CTL_RSTCH_MASK;

            /* Wait for video channel to get reset */
            while (port->vportBase->CSL_VportCapARegs.VC_CTL
                        & CSL_VPORT_VC_CTL_RSTCH_MASK);
            while (port->vportBase->CSL_VportCapBRegs.VC_CTL
                        & CSL_VPORT_VC_CTL_RSTCH_MASK);
        } /* if (!port->opened)  */

        /* initialize the channel object */
        if ((!(port->chanObj[chanNum].status & _VPORT_OPENED))
            && (IOM_COMPLETED == retVal))
        {
            chan = &port->chanObj[chanNum];

#ifdef VPORT_DEBUG
            chan->submitCount = 0;
            chan->errCount = 0;
            chan->frameCount = 0;
            chan->shortFldDetectCount = 0;
            chan->autoSyncCount = 0;
            chan->isrSwapCount = 0;
            chan->isrSwapErrCount = 0;
            chan->reconfigErrCount = 0;

            chan->tooLateInISRCount[0] = 0;
            chan->tooLateInISRCount[1] = 0;
            chan->tooLateInISRCount[2] = 0;
            chan->tooLateInISRCount[3] = 0;
            chan->tooLateInISRCount[4] = 0;
#endif  /* #ifdef VPORT_DEBUG */

            chan->status |= _VPORT_OPENED;
            chan->cbFxn = cbFxn;
            chan->cbArg = (Arg)cbArg;
            chan->numFrms = 0;
            chan->vIntMask = 0;
            chan->vIntFxn = (VPORT_IntCallBack)INVCB;
            chan->vIntCbArg = (Int)INV;
            chan->lineIntInfo.callBackFxn =
                                    (VPORTCAP_LinesCapturedCB)INVCB;
            chan->lineIntInfo.cbArg = (Int)INV;
            chan->lineIntInfo.errCheckEnable = FALSE;
            chan->mode = ((VPORTCAP_Params *)chanParams)->cmode;

            /* Assign EDMA handle passed to global channel structure variable */
            if (chan->mode & _VPORT_MASK_RAW)
            {
                chan->hEdma = ((VPORTCAP_ParamsRaw *)chanParams)->hEdma;
                numFrmBufs = ((VPORTCAP_ParamsRaw *)chanParams)->numFrmBufs;
            }
            else
            {
                chan->hEdma = ((VPORTCAP_Params *)chanParams)->hEdma;
                numFrmBufs = ((VPORTCAP_Params *)chanParams)->numFrmBufs;
            }

            /* Request EDMA channels and param tables for Y, Cb, Cr transfers */
            for (j = 0;
                    (j < _VPORT_NUM_EDMA_CHANS) && (IOM_COMPLETED == retVal);
                    j++)
            {
                chan->edmaLinkChanNum[4 * j] = EDMA3_DRV_LINK_CHANNEL;
                chan->edmaLinkChanNum[4 * j + 1] = EDMA3_DRV_LINK_CHANNEL;
                chan->edmaLinkChanNum[4 * j + 2] = EDMA3_DRV_LINK_CHANNEL;
                chan->edmaLinkChanNum[4 * j + 3] = EDMA3_DRV_LINK_CHANNEL;

                chan->tcc[j] = chan->hEdmaChan[j] = chan->edmaChanNum[j];
                linkTccId = EDMA3_DRV_TCC_ANY;

                if ((EDMA3_DRV_requestChannel(chan->hEdma,
                                              &chan->hEdmaChan[j],
                                              &chan->tcc[j],
                                              chan->evtQueue,
                                              (EDMA3_RM_TccCallback)&
                                              captureEdmaISR,
                                              NULL)) != EDMA3_DRV_SOK ||
                    (EDMA3_DRV_requestChannel(chan->hEdma,
                                              &chan->edmaLinkChanNum[4 * j],
                                              &linkTccId,
                                              chan->evtQueue,
                                              (EDMA3_RM_TccCallback)&
                                              captureEdmaISR,
                                              NULL)) != EDMA3_DRV_SOK ||
                    (EDMA3_DRV_requestChannel(chan->hEdma,
                                              &chan->edmaLinkChanNum[4 * j + 1],
                                              &linkTccId,
                                              chan->evtQueue,
                                              (EDMA3_RM_TccCallback)&
                                              captureEdmaISR,
                                              NULL)) != EDMA3_DRV_SOK ||
                    (EDMA3_DRV_requestChannel(chan->hEdma,
                                              &chan->edmaLinkChanNum[4 * j + 2],
                                              &linkTccId,
                                              chan->evtQueue,
                                              (EDMA3_RM_TccCallback)&
                                              captureEdmaISR,
                                              NULL)) != EDMA3_DRV_SOK ||
                    (EDMA3_DRV_requestChannel(chan->hEdma,
                                              &chan->edmaLinkChanNum[4 * j + 3],
                                              &linkTccId,
                                              chan->evtQueue,
                                              (EDMA3_RM_TccCallback)&
                                              captureEdmaISR,
                                              NULL)) != EDMA3_DRV_SOK)
                {
                    retVal = IOM_EALLOC;
                } /* if ((port->... */

                /* Get the physical address of all the EDMA channels */
                result |= EDMA3_DRV_getPaRAMPhyAddr(
                        chan->hEdma,
                        chan->hEdmaChan[j],
                        (unsigned int *)&chan->hEdmaChanPhyAddr[j]);
                result |= EDMA3_DRV_getPaRAMPhyAddr(
                        chan->hEdma,
                        chan->edmaLinkChanNum[4 * j],
                        (unsigned int *)&chan->hEdmaLinkChanPhyAddr[4 * j]);
                result |= EDMA3_DRV_getPaRAMPhyAddr(
                        chan->hEdma,
                        chan->edmaLinkChanNum[4 * j + 1],
                        (unsigned int *)&chan->hEdmaLinkChanPhyAddr[4 * j + 1]);
                result |= EDMA3_DRV_getPaRAMPhyAddr(
                        chan->hEdma,
                        chan->edmaLinkChanNum[4 * j + 2],
                        (unsigned int *)&chan->hEdmaLinkChanPhyAddr[4 * j + 2]);
                result |= EDMA3_DRV_getPaRAMPhyAddr(
                        chan->hEdma,
                        chan->edmaLinkChanNum[4 * j + 3],
                        (unsigned int *)&chan->hEdmaLinkChanPhyAddr[4 * j + 3]);

                if (result != EDMA3_DRV_SOK)
                {
                    retVal = IOM_EALLOC;
                }
            } /* for (j = 0; j < NUM_EDMA_CHANS; j++) ... */

            if (IOM_COMPLETED == retVal)
            {
                /* Set driver mode as per numFrmBufs passed from application */
                if (0 == numFrmBufs)
                {
                    chan->driverMode = NORMAL_MODE;  /* normal mode */
                }
                else
                {
                    chan->driverMode = LEGACY_MODE;  /* legacy(DM642) mode */
                }

                retVal = mdControlChan(&port->chanObj[chanNum],
                                       VPORT_CMD_CONFIG_CHAN,
                                       chanParams);
            }

            if (IOM_COMPLETED == retVal)
            {
                /* enabling capture over run error handling */
                _setCovrIsr((Ptr)&port->chanObj[chanNum], _IRQ_ID);
            }

            if (IOM_COMPLETED == retVal)
            {
                /* configure the channel */
                *chanp = &port->chanObj[chanNum];
            }
            else
            {
                /* failed, free all resources and return error */
                mdDeleteChan(&port->chanObj);
                *chanp = INV;
            }
        } /* if (!chan->opened) */
    } /* if (IOM_COMPLETED == retVal) */

    return retVal;
}



/**
 *  \brief Close Channel of the VPORT device
 *
 *  This function is used to close Channel handle for individual VPORT
 *  Modules.
 *
 *  \param  chanp       [IN]    VPORT Driver Channel Handle
 *
 *  \return                     On success returns IOM_COMPLETED
 *                              On failure returns suitable error code
 */
static Int mdDeleteChan(Ptr chanp)
{
    _VPORT_ChanObj      *chan = (_VPORT_ChanObj *)chanp;
    PortObj             *port = &portObjs[chan->portNum];
    Int                 j;
    Int                 numEdmaChans = _VPORT_NUM_EDMA_CHANS;
    Int                 retVal = IOM_COMPLETED;
    EDMA3_DRV_Result    result = EDMA3_DRV_SOK;

    if (NULL == chan)
    {
        retVal = IOM_EBADARGS;
    }
    else
    {
        if (chan->status & _VPORT_OPENED)
        {
            if (chan->status & _VPORT_STARTED)
            {
                /* Stop capture operation first */
                retVal = mdControlChan(chanp, VPORT_CMD_STOP, NULL);
            }

            if (IOM_COMPLETED == retVal)
            {
                for (j = 0; j < numEdmaChans; j++)
                {
                    /* Disable Transfer */
                    result = EDMA3_DRV_disableTransfer(chan->hEdma,
                                                   chan->hEdmaChan[j],
                                                   EDMA3_DRV_TRIG_MODE_EVENT);
                    /* Free Channels and PaRam Tables */
                    result |= EDMA3_DRV_freeChannel(chan->hEdma,
                                            chan->hEdmaChan[j]);
                    result |= EDMA3_DRV_freeChannel(chan->hEdma,
                                            chan->edmaLinkChanNum[4 * j]);
                    result |= EDMA3_DRV_freeChannel(chan->hEdma,
                                            chan->edmaLinkChanNum[4 * j + 1]);
                    result |= EDMA3_DRV_freeChannel(chan->hEdma,
                                            chan->edmaLinkChanNum[4 * j + 2]);
                    result |= EDMA3_DRV_freeChannel(chan->hEdma,
                                            chan->edmaLinkChanNum[4 * j + 3]);


                    /* Reset the physical addresses of all EDMA channels */
                    chan->hEdmaChanPhyAddr[j] = NULL;
                    chan->hEdmaLinkChanPhyAddr[4 * j] = NULL;
                    chan->hEdmaLinkChanPhyAddr[4 * j + 1] = NULL;
                    chan->hEdmaLinkChanPhyAddr[4 * j + 2] = NULL;
                    chan->hEdmaLinkChanPhyAddr[4 * j + 3] = NULL;
                }
                if (EDMA3_DRV_SOK != result)
                {
                    retVal = IOM_EFREE;
                }

                if (_VPORT_BUF_ALLOCATED & chan->status)
                {
                    /* If buffers are allocated then free them, This condition
                     * will be TRUE only in LEGACY MODE
                     */
                    for (j = 0; j < chan->numFrms; j++)
                    {
                        if (PAL_SOK != PAL_osMemFree(chan->segId,
                                                 chan->viops[j].frame.iFrm.y1,
                                                 chan->bufSz))
                        {
                            retVal = IOM_EFREE;
                        }
                    }
                }
                chan->status = 0;

                /* close external device */
                if (chan->edcFxns != INV)
                {
                    if (EDC_FAILED == chan->edcFxns->close(chan->edcHandle))
                    {
                        retVal = IOM_EBADIO;
                    }
                }
            } /* if (IOM_COMPLETED == retVal) */
        } /* if (chan->status & _VPORT_OPENED) */

        /* If both the channels of port are closed then make port
         * status as closed
         */
        if (!(port->chanObj[0].status & _VPORT_OPENED) &&
            !(port->chanObj[1].status & _VPORT_OPENED))
        {
            port->status = 0;
        }
    }

    return retVal;
}



/**
 *  \brief submit IO Request
 *
 *  This function is called to Queue/Dequeue Frame buffer.
 *
 *  \param  chanp      [IN]    VPORT Driver Channel Handle
 *  \param  packet     [IN/OUT] IOM packet passed by the upper layer containing
 *                              information about the size,address,timeout and
 *                              other necessary parameters
 *
 *  \return                     On success, returns positive number - depending
 *                              upon the operation requested
 *
 *                              If failed, returns negative number - suitable
 *                              error code
 */
static Int mdSubmitChan(Ptr chanp, IOM_Packet *packet)
{
    _VPORT_ChanObj  *chan = (_VPORT_ChanObj *)chanp;
    Uint32          gie;
    Int             retVal = IOM_PENDING;
    Bool            notTooLate = TRUE;
    Int             capStatReg;
    Int             vpis;
    short           lineNum;

    PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &gie);

    /* Get the VPORT register status after interrupts are disabled!! */
    capStatReg = chan->vportChanRegs->VC_STAT;
    vpis = chan->vportRegs->VP_IS >> (chan->chanNum * 16);

#ifdef VPORT_DEBUG
    chan->submitCount++;
#endif  /* #ifdef VPORT_DEBUG */

    if ((NULL == chan) || (NULL == packet))
    {
        retVal = IOM_EBADARGS;
    }
    else
    {
#ifdef VPORT_INSTRUMENTATION_ENABLED
        FVID_Frame      *tempViop = NULL;

        if (NULL != packet->addr)
        {
            tempViop = (*(void**)packet->addr);
        }

        PSP_LOG_EVENT(&DVTEvent_Log,
                      "VPORT_CAPTURE",
                      PSP_DVT_DESC(PSP_DVT_eFUNC_START,
                                   PSP_DVT_dCHANNEL_ID_INST,
                                   PSP_DVT_dPACKET_ID,
                                   PSP_DVT_dSIZE_BYTES),
                      CHANNELID_INSTANCEID(
                                   chan->chanNum,
                                   chan->portNum),
                      tempViop,
                      chan->bufSz
                      );
#endif /* VPORT_INSTRUMENTATION_ENABLED */

        /* checking for notTooLate only if capture is started */
        if (chan->status & _VPORT_STARTED)
        {
            if (chan->mode & _VPORT_MASK_RAW)
            {
                /* Note: We can't determine notTooLate for RAW capture. Hence
                   this is kept as FALSE always for RAW capture */
                notTooLate = FALSE;
            }
            else
            {
                /* get the current line number being captured */
                lineNum = (capStatReg & CSL_VPORT_VC_STAT_VC_YPOS_MASK)
                            >> CSL_VPORT_VC_STAT_VC_YPOS_SHIFT;

                /* account for field/frame operation modes */
                lineNum += (chan->numLinesFld1 *
                           ((capStatReg & CSL_VPORT_VC_STAT_VCFLD_MASK)
                           >> CSL_VPORT_VC_STAT_VCFLD_SHIFT));

                /* make sure we are not too close to the end of a frame */
                notTooLate =
                (lineNum < (chan->lastLineNum - NOT_TOO_LATE_LINE_ROOM_SUBMIT));
            }
        }
        else
        {
            /* if capture is not started then set notTooLate as false */
            notTooLate = FALSE;
        }

        switch (packet->cmd)
        {
            case FVID_FREE:
                /* This command is not supported in NORMAL MODE */
                if (LEGACY_MODE != chan->driverMode)
                {
                    retVal = IOM_ENOTIMPL;
                }
                else if (!(chan->status & _VPORT_READY))
                {
                    retVal = IOM_EABORT;
                }
                else
                {
                    /* function call to queue the buffer to driver */
                    retVal = _cmdQueue(chan, packet, notTooLate);
                }
                break;

            case FVID_QUEUE:
                /* This command is not supported in LEGACY MODE */
                if (LEGACY_MODE == chan->driverMode)
                {
                    retVal = IOM_ENOTIMPL;
                }
                else
                {
                    /* function call to queue the buffer to driver */
                    retVal = _cmdQueue(chan, packet, notTooLate);
                }
                break;

            case FVID_ALLOC:
                /* This command is not supported in NORMAL MODE */
                if (LEGACY_MODE != chan->driverMode)
                {
                    retVal = IOM_ENOTIMPL;
                }
                else if (!(chan->status & _VPORT_READY))
                {
                    retVal = IOM_EABORT;
                }
                else if (chan->mrViop != INV)
                {
                    /* only when there is no outstanding pending request */
                    if (INV == chan->packetIOM)
                    {
                        /* function call to get the buffer from driver */
                        retVal = _cmdDequeue(chan, packet, notTooLate);
                    }
                    else
                    {
                        retVal = IOM_EINUSE;
                    } /* if (INV == chan->packetIOM) */
                }
                else
                {
                    /* if mrviop is INV then retutn IOM_PENDING. This call will
                     * wait at GIO layer, when there will be new buffer
                     * available with driver, GIO layer callback will be called
                     */
                    chan->packetIOM = packet;
                    retVal = packet->status = IOM_PENDING;
                }
                break;

            case FVID_DEQUEUE:
                /* This command is not supported in LEGACY MODE */
                if (LEGACY_MODE == chan->driverMode)
                {
                    retVal = IOM_ENOTIMPL;
                }
                else if (!(chan->status & _VPORT_STARTED))
                {
                    /* If capture is stopped then return all bufs one-by-one */
                    retVal = _cmdGetBufs(chan, packet);
                }
                else if (chan->mrViop != INV)
                {
                    /* only when there is no outstanding pending request */
                    if (INV == chan->packetIOM)
                    {
                        /* If capture is started then return captured buffer to
                         * application. If buffer is not available, return
                         * IOM_PENDING
                         */
                        retVal = _cmdDequeue(chan, packet, notTooLate);
                    }
                    else
                    {
                        retVal = IOM_EINUSE;
                    } /* if (INV == chan->packetIOM) */
                }
                else
                {
                    /* if mrviop is INV then retutn IOM_PENDING. This call will
                     * wait at GIO layer, when there will be new buffer
                     * available with driver, GIO layer callback will be called
                     */
                    chan->packetIOM = packet;
                    retVal = packet->status = IOM_PENDING;
                }
                break;

            case FVID_EXCHANGE:
                if (!(chan->status & _VPORT_READY))
                {
                    retVal = IOM_EABORT;
                }
                else if (chan->mrViop != INV)
                {
                    /* only when there is no outstanding pending request */
                    if (INV == chan->packetIOM)
                    {
                        retVal = _cmdQueue(chan, packet, notTooLate);
                        if (IOM_COMPLETED == retVal)
                        {
                            retVal = _cmdDequeue(chan, packet, notTooLate);
                        }
                    }
                    else
                    {
                        retVal = IOM_EINUSE;
                    } /* if (INV == chan->packetIOM) */
                }
                else
                {
                    /* if mrviop is INV then queue the buffer provided by
                     * application.
                     */
                    retVal = _cmdQueue(chan, packet, notTooLate);
                    if (IOM_COMPLETED == retVal)
                    {
                        /* If queue is successful then return IOM_PENDING.
                         * Returning IOM_PENDING will wait at GIO layer. When
                         * there is new buffer available with driver,GIO layer
                         * callback will be called
                         */
                        chan->packetIOM = packet;
                        retVal = packet->status = IOM_PENDING;
                    }
                }
                break;

            default:
                retVal = IOM_ENOTIMPL;
                break;
        }

        if ((chan->status & _VPORT_STARTED) &&
            (vpis & CSL_VPORT_VP_IS_SFDA_MASK))
        {
#ifdef VPORT_DEBUG
            chan->shortFldDetectCount++;
#endif  /* #ifdef VPORT_DEBUG */

            /* short field detected, re-sync */
            _covrRecover(chanp);
        }

#ifdef VPORT_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(&DVTEvent_Log,
                      "VPORT_CAPTURE",
                      PSP_DVT_DESC(PSP_DVT_eFUNC_END,
                                   PSP_DVT_dCHANNEL_ID_INST,
                                   PSP_DVT_dPACKET_ID,
                                   PSP_DVT_dSIZE_BYTES),
                      CHANNELID_INSTANCEID(
                                   chan->chanNum,
                                   chan->portNum),
                      tempViop,
                      0
                      );
#endif /* VPORT_INSTRUMENTATION_ENABLED */
    }

    PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, gie);

    return retVal;
}



/**
 *  \brief submit the empty buffer to driver
 *
 *  \param  chan      [In] VPORT channel object pointer
 *  \param  packet    [In] video IO packet
 *  \param  notTooLate [In] flag to check that frame capture is not at the end
 *                         of buffer
 *
 *  \return  IOM_COMPLETED on success
 *           IOM errors on failure
 */
static Int _cmdQueue(_VPORT_ChanObj *chan, IOM_Packet *packet, Bool notTooLate)
{
    FVID_Frame          *viop;
    Bool                nextViopChanged = FALSE;
    Int                 retVal = IOM_COMPLETED;
    Int                 offset = chan->nextEDMARlds << 1;

    if (NULL == *(void**)packet->addr)
    {
        retVal = IOM_EBADARGS;
    }
    else
    {
        viop = *(void**)packet->addr; /* pointer of a video I/O packet */

        if (!(chan->status & _VPORT_READY))
        {
            /* maintain the count for calling _configEDMA after minimum
             * number of required queued buffers
             */
            chan->numQueBuf++;
            QUE_enqueue(&chan->qIn, (QUE_Handle)viop);

            chan->queEmpty = FALSE;

            /* _configEDMA can only be called after minimum required
             * number of buffers are queued into the buffer queue
             */
            if (_VPORT_MIN_BUF == chan->numQueBuf)
            {
                retVal = _configEDMA(chan);
            }

            packet->status = IOM_COMPLETED;
        }
        else
        {
            if (chan->queEmpty && notTooLate)
            {
                /* don't put it into queue, update the rld register
                 * directly
                 */
                chan->nextViop = viop;
                /* notTooLate will not be true before configTransfer
                 * completed so it won't allow nextViopChanged to be TRUE
                 */
                nextViopChanged = TRUE;
            }
            else
            {
                /* notTooLate will not be TRUE for _VPORT_MIN_BUF
                 * queue request
                 */
                QUE_enqueue(&chan->qIn, (QUE_Handle)viop);
            }

            chan->queEmpty = FALSE;
            packet->status = IOM_COMPLETED;

            if (NORMAL_MODE == chan->driverMode)
            {
                /* increment number of buffers available with driver */
                chan->numQueBuf++;
            }
        } /* if (!(chan->status & _VPORT_READY)) */

        if (nextViopChanged)
        {
            /* Modify EDMA rld entries */
            retVal = _updateEDMAParamAddress(chan, offset, chan->nextViop);
        }
    }

    return retVal;
}



/**
 *  \brief provide the captured buffer to application application
 *
 *  \param  chan       [In] VPORT channel object pointer
 *  \param  packet     [OUT] video IO packet
 *  \param  notTooLate [In] flag to check that frame capture is not at the end
 *                         of buffer
 *
 *  \return  IOM_COMPLETED on success
 *           IOM_PENDING if buffer is not available or suitable error code
 *                       on failure
 */
static Int _cmdDequeue(_VPORT_ChanObj *chan, IOM_Packet *packet,
                       Bool notTooLate)
{
    Int                 retVal = IOM_PENDING;
    Int                 offset = chan->nextEDMARlds << 1;

    if (chan->nextViop != chan->mrViop)
    {
        *(void**)packet->addr = (void*)chan->mrViop;
        chan->mrViop = INV;
        packet->size = sizeof(FVID_Frame);
        retVal = packet->status = IOM_COMPLETED;
    }
    else
    {
        if (notTooLate)
        {
            *(void**)packet->addr = (void*)chan->mrViop;
            chan->mrViop = INV;
            packet->size = sizeof(FVID_Frame);
            retVal = packet->status = IOM_COMPLETED;

            /* If queue is already empty, it means the driver
             * currently only owns one frame, which is the
             * current frame. So make next frame same as the
             * current one
             */
            chan->nextViop = chan->curViop;

            /* Modify EDMA rld entries */
            retVal = _updateEDMAParamAddress(chan, offset, chan->nextViop);
        }
        else
        {
            /* too late, just recycle mrViop */
            chan->mrViop = INV;
            chan->packetIOM = packet;
            retVal = packet->status = IOM_PENDING;
        } /* if (notTooLate) */
    } /* if (chan->nextViop != chan->mrViop) */

    if ((NORMAL_MODE == chan->driverMode) && (IOM_COMPLETED == retVal))
    {
        /* decrement number of buffers available with driver */
        chan->numQueBuf--;
    }

    return retVal;
}



/**
 *  \brief Return all buffers to application
 *
 *  This function returns all buffers to application held by driver in
 *  normal mode. The buffers may not contain proper data
 *
 *  \param  chan    [In] VPORT channel object pointer
 *  \param  packet  [Out] video IO packet
 *
 *  \return  IOM_COMPLETED on success
 *           IOM_ENOPACKETS if buffer is not available
 */
static Int _cmdGetBufs(_VPORT_ChanObj *chan, IOM_Packet *packet)
{
    Int         retVal = IOM_COMPLETED;
    FVID_Frame  *viop = NULL;

    if (0 == chan->numQueBuf)
    {
        chan->queEmpty = TRUE;
        retVal = IOM_ENOPACKETS;
    }
    else
    {
        /* Take buffer from queue if queue is not empty */
        viop = (FVID_Frame *)QUE_dequeue(&chan->qIn);

        if (viop == (FVID_Frame *)&chan->qIn)
        {
            /* If queue is empty then return buffers from viop pointers
             * one-by-one
             */
            chan->queEmpty = TRUE;

            if (chan->mrViop != INV)
            {
                viop = chan->mrViop;

                /* if nextViop or curViop is same as mrViop then make it INV */
                if (chan->nextViop == chan->mrViop)
                {
                    chan->nextViop = INV;
                }
                if (chan->curViop == chan->mrViop)
                {
                    chan->curViop = INV;
                }
                chan->mrViop = INV;

            } /* if (chan->mrViop != INV) */
            else if (chan->nextViop != INV)
            {
                viop = chan->nextViop;

                /* if curViop is same as nextViop then make it INV */
                if (chan->curViop == chan->nextViop)
                {
                    chan->curViop = INV;
                }
                chan->nextViop = INV;

            } /* else if (chan->nextViop != INV) */
        } /* if (viop == (FVID_Frame *)&chan->qIn) */

        /* Fill buffer address and other information in packet */
        *(void**)packet->addr = (void*)viop;
        packet->size = sizeof(FVID_Frame);
        retVal = packet->status = IOM_COMPLETED;

        /* Decrement number of buffers with driver */
        chan->numQueBuf--;

        if (chan->numQueBuf < _VPORT_MIN_BUF)
        {
            /* If chan->numQueBuf is less than _VPORT_MIN_BUF then, put curViop,
             * nextViop and mrViop in queue.
             * Take care that every buffer is being queued only once.
             */
            if (chan->mrViop != INV)
            {
                QUE_enqueue(&chan->qIn, (QUE_Handle)chan->mrViop);
                chan->queEmpty = FALSE;
            }
            if ((chan->nextViop != INV) && (chan->nextViop != chan->mrViop))
            {
                QUE_enqueue(&chan->qIn, (QUE_Handle)chan->nextViop);
                chan->queEmpty = FALSE;
            }
            if ((chan->curViop != INV) && (chan->curViop != chan->mrViop) &&
                (chan->curViop != chan->nextViop))
            {
                QUE_enqueue(&chan->qIn, (QUE_Handle)chan->curViop);
                chan->queEmpty = FALSE;
            }

            /* make all viop pointers INV */
            chan->mrViop = INV;
            chan->nextViop = INV;
            chan->curViop = INV;

            /* set channel status as EDMA not configured */
            chan->status &= ~_VPORT_READY;
        } /* if (chan->numQueBuf < _VPORT_MIN_BUF) */
    } /* if (0 == chan->numQueBuf) */

    return retVal;
}



/**
 *  \brief Checks whether EDMA parameters can be safely updated in the EDMA ISR
 *
 *  This functions checks if it is safe to update the EDMA parameters (voip
 *  buffer address in param tables) in EDMA completion ISR. This error detection
 *  is required to prevent any mismatch (loss of synchronization) between
 *  EDMA current/next buffer and software maintained current/next buffer.
 *
 *  This loss of synchronization can happen when EDMA ISR could not have a
 *  chance to run or is delayed more than one frame period. This delay or ISR
 *  miss can happen when the CPU load is very high (near 100%) or when break
 *  points are kept or when emulation pause happens during memory/variable
 *  updates using CCS.
 *
 *  Detection is done in two steps:
 *      1. Current working EDMA param set and param set for which the EDMA ISR
 *         has occurred are compared. If both are equal, then the previous param
 *         set ISR is missed!! Software current viop and EDMA working viop are
 *         same. This could lead to buffer corruption if the buffer is given
 *         to the application!!
 *         For comparison, link param field is used as it is the only field
 *         which is unique for each param set and which is not modified for each
 *         viop.
 *
 *      2. Check whether EDMA is capturing at the end of current param set. If
 *         it very near to completion of the current param set, EDMA reload of
 *         next viop param could soon happen before we could successfully update
 *         the next param table. This could also lead to buffer corruption if
 *         the buffer is given to the application!!
 *
 *  \param  chan    [In] VPORT channel object pointer
 *
 *  \return  TRUE if it is too late to update EDMA parameters in EDMA ISR
 *           FALSE if EDMA parameters can be safely updated in EDMA ISR
 */
static Bool _isItTooLateInISR(_VPORT_ChanObj *chan)
{
    Bool                tooLate = FALSE;
    Uint32              linesTransferred;
    Int                 offset, nextOffset;
    volatile Uint16     paramLinkAddr1, paramLinkAddr2;
    volatile Uint16     hwLinkAddr, hw_cCnt, hwLinkAddr1;

    offset = chan->nextEDMARlds << 1;
    nextOffset = (!chan->nextEDMARlds) << 1;

    /*
     * Check 1: Check if EDMA reload for the current viop has already happened
     * before the ISR could get a chance to run
     */
    /* Get the hardware param set */
    hwLinkAddr = chan->hEdmaChanPhyAddr[0]->linkAddr;
    hw_cCnt = chan->hEdmaChanPhyAddr[0]->cCnt;

    /* Read the link address again to make sure reload has not happened!! */
    hwLinkAddr1 = chan->hEdmaChanPhyAddr[0]->linkAddr;
    if (hwLinkAddr != hwLinkAddr1)
    {
        /* EDMA reload has happened - Read the parameters again hoping that EDMA
           reload cannot happen again so soon!!  */
        hwLinkAddr = chan->hEdmaChanPhyAddr[0]->linkAddr;
        hw_cCnt = chan->hEdmaChanPhyAddr[0]->cCnt;

#ifdef VPORT_DEBUG
        chan->tooLateInISRCount[0]++;
#endif  /* #ifdef VPORT_DEBUG */
    }

    /* Get the link address for current param set for which we have got ISR */
    paramLinkAddr1 = chan->hEdmaLinkChanPhyAddr[offset]->linkAddr;
    paramLinkAddr2 = chan->hEdmaLinkChanPhyAddr[offset + 1]->linkAddr;

    /* Check whether link addresses are matching */
    if (chan->interlaced && ((hwLinkAddr == paramLinkAddr1)
                            || (hwLinkAddr == paramLinkAddr2)))
    {
        /* One of the link addresses is matching for interlaced mode */
        tooLate = TRUE;

#ifdef VPORT_DEBUG
        chan->tooLateInISRCount[1]++;
#endif  /* #ifdef VPORT_DEBUG */
    }
    else if (!chan->interlaced && (hwLinkAddr == paramLinkAddr1))
    {
        /* Link addresses are matching for non-interlaced mode. 2nd param is not
           used in non-interlaced mode, hence not used in comparison */
        tooLate = TRUE;

#ifdef VPORT_DEBUG
        chan->tooLateInISRCount[2]++;
#endif  /* #ifdef VPORT_DEBUG */
    }
    else
    {
        /*
         * Check whether EDMA is going to get complete for the current param set
         */
        /* Note: We can't determine notTooLate for RAW capture. Hence
           this is kept as FALSE for RAW capture */
        if (!(chan->mode & _VPORT_MASK_RAW))
        {
            linesTransferred = chan->numLines - hw_cCnt;

            /* Get the link address for next param set */
            paramLinkAddr1 = chan->hEdmaLinkChanPhyAddr[nextOffset]->linkAddr;
            paramLinkAddr2
                        = chan->hEdmaLinkChanPhyAddr[nextOffset + 1]->linkAddr;

            /* Check whether param set 0 or 1 is getting transferred */
            if (chan->interlaced)
            {
                if (hwLinkAddr == paramLinkAddr1)
                {
                    /* Param set 0 is getting transferred. So substract field 2
                       line added earlier!! */
                    linesTransferred -= (chan->numLines - chan->numLinesFld1);
                }
#ifdef VPORT_DEBUG
                else if (hwLinkAddr != paramLinkAddr2)
                {
                    chan->tooLateInISRCount[3]++;
                }
#endif  /* #ifdef VPORT_DEBUG */
            }

            /* Check if EDMA is capturing at the last few lines */
            if (chan->numLines > NOT_TOO_LATE_LINE_ROOM_ISR)
            {                               /* Prevent negative value */
                if (linesTransferred >=
                        (chan->numLines - NOT_TOO_LATE_LINE_ROOM_ISR))
                {
                    tooLate = TRUE;     /* EDMA is going to complete soon */

#ifdef VPORT_DEBUG
                    chan->tooLateInISRCount[4]++;
#endif  /* #ifdef VPORT_DEBUG */
                }
            }
        }
    }

    return tooLate;
}



/**
 *  \brief Updates the destination buffer address of the given param table
 *         depending on various modes
 *
 *  \param  chan    [In] VPORT channel object pointer
 *  \param  offset  [In] EDMA param table to update
 *  \param  viop    [In] Destination buffer address frame pointer
 *
 *  \return  IOM_COMPLETED on success
 *           IOM_EBADIO if EDMA error occurs while updating address
 */
static Int _updateEDMAParamAddress(const _VPORT_ChanObj *chan, Int offset,
                                   const FVID_Frame *viop)
{
    Int retVal = IOM_COMPLETED;

    if (chan->interlaced)
    {
        /* Modify EDMA rld entries for Y transfer */
        chan->hEdmaLinkChanPhyAddr[offset]->destAddr
                                    = (unsigned int)viop->frame.iFrm.y1;
        chan->hEdmaLinkChanPhyAddr[offset + 1]->destAddr
                                    = (unsigned int)viop->frame.iFrm.y2;

        if (!(chan->mode & _VPORT_MASK_RAW))
        {
            /* Modify EDMA rld entries for Cb transfer */
            chan->hEdmaLinkChanPhyAddr[offset + 4]->destAddr
                                    = (unsigned int)viop->frame.iFrm.cb1;
            chan->hEdmaLinkChanPhyAddr[offset + 5]->destAddr
                                    = (unsigned int)viop->frame.iFrm.cb2;

            /* Modify EDMA rld entries for Cr transfer */
            chan->hEdmaLinkChanPhyAddr[offset + 8]->destAddr
                                    = (unsigned int)viop->frame.iFrm.cr1;
            chan->hEdmaLinkChanPhyAddr[offset + 9]->destAddr
                                    = (unsigned int)viop->frame.iFrm.cr2;
        }
    }
    else    /* Non-interlaced mode only one param table is used in each set */
    {
        /* Modify EDMA rld entries for Y transfer */
        chan->hEdmaLinkChanPhyAddr[offset]->destAddr
                                    = (unsigned int)viop->frame.iFrm.y1;

        if (!(chan->mode & _VPORT_MASK_RAW))
        {
            /* Modify EDMA rld entries for Cb transfer */
            chan->hEdmaLinkChanPhyAddr[offset + 4]->destAddr
                                    = (unsigned int)viop->frame.iFrm.cb1;

            /* Modify EDMA rld entries for Cr transfer */
            chan->hEdmaLinkChanPhyAddr[offset + 8]->destAddr
                                    = (unsigned int)viop->frame.iFrm.cr1;
        }
    } /* if (chan->interlaced) */

    return retVal;
}



/**
 *  \brief capture EDMA ISR
 *
 *  \param tcc     [In] transfer complete code
 *  \param status  [In] status of EDMA transfer
 *  \param appData []   not used
 */
static void captureEdmaISR(unsigned int tcc,
                           EDMA3_RM_TccStatus status,
                           void *appData)
{
    Int                 i, j;
    Int                 offset;
    FVID_Frame          *viop, *curViop;
    _VPORT_ChanObj      *chan;
    unsigned int        curParamAddr;
    Int                 vpis;
    Bool                callBackFlag = FALSE;

    /* find out the source of the EDMA interrupt */
    for (j = 0; j < CSL_VIDEOPORT_PER_CNT; j++)
    {
        for (i = 0; i < _VPORT_CHAN_CNT; i++)
        {
            /* loop through two channels */
            chan = &portObjs[j].chanObj[i];

            if ((chan->status & _VPORT_READY) &&
                (tcc == chan->tcc[0] || tcc == chan->tcc[1]))
            {
#ifdef VPORT_INSTRUMENTATION_ENABLED
                PSP_LOG_EVENT(&DVTEvent_Log,
                              "VPORT_CAPTURE",
                              PSP_DVT_DESC(PSP_DVT_eRCV_END,
                                           PSP_DVT_dCHANNEL_ID_INST,
                                           PSP_DVT_dPACKET_ID,
                                           PSP_DVT_dNONE),
                              CHANNELID_INSTANCEID(
                                           chan->chanNum,
                                           chan->portNum),
                              chan->curViop
                              );

                PSP_LOG_EVENT(&DVTEvent_Log,
                              "VPORT_CAPTURE",
                              PSP_DVT_DESC(PSP_DVT_eRCV_START,
                                           PSP_DVT_dCHANNEL_ID_INST,
                                           PSP_DVT_dPACKET_ID,
                                           PSP_DVT_dNONE),
                              CHANNELID_INSTANCEID(
                                           chan->chanNum,
                                           chan->portNum),
                              chan->nextViop
                              );
#endif /* VPORT_INSTRUMENTATION_ENABLED */

                if (EDMA3_RM_XFER_COMPLETE != status)
                {
                    _covrRecover(chan);
                }
                else
                {
#ifdef VPORT_DEBUG
                    chan->frameCount++;
#endif  /* #ifdef VPORT_DEBUG */

                    if (chan->lineIntInfo.errCheckEnable)
                    {
                        vpis = chan->vportRegs->VP_IS >> (chan->chanNum * 16);
                        if (vpis & CSL_VPORT_VP_IS_SFDA_MASK)
                        {
#ifdef VPORT_DEBUG
                            chan->shortFldDetectCount++;
#endif  /* #ifdef VPORT_DEBUG */

                            /* short field detected, re-sync */
                            _covrRecover(chan);
                        }
                    }

                    /* re-sync tcc with activeEDMARlds
                     * they may be out of sync after interrupt over-run
                     * e.g. execution is halted at break-point
                     */
                    chan->nextEDMARlds = (tcc == chan->tcc[0]) ? 0 : 1;
                    offset = chan->nextEDMARlds << 1;

                    /* Check if is it too late to update the next param table */
                    if (_isItTooLateInISR(chan))
                    {
                        /* Too late. Reconfigure channel and start fresh!! */
                        _covrRecover(chan);
                    }
                    else
                    {

                        /* Get current address of Y data EDMA transfer */
                        curParamAddr =
                                chan->hEdmaLinkChanPhyAddr[offset]->destAddr;

                        /* If software maintained current and next addresses are
                         * not in sync with hardware EDMA parameters then
                         * re-sync them
                         */
                        if ((char *)curParamAddr
                                != chan->curViop->frame.iFrm.y1)
                        {
                            /* Before swapping next and current VIOP, just check
                               whether current param address is atleast same as
                               next VIOP */
                            if ((char *)curParamAddr
                                    == chan->nextViop->frame.iFrm.y1)
                            {
#ifdef VPORT_DEBUG
                                chan->isrSwapCount++;
#endif  /* #ifdef VPORT_DEBUG */

                                /* Safe to swap current and next VIOP */
                                curViop = chan->curViop;
                                chan->curViop = chan->nextViop;
                                chan->nextViop = curViop;
                            }
#ifdef VPORT_DEBUG
                            else
                            {
                                chan->isrSwapErrCount++;
                            }
#endif  /* #ifdef VPORT_DEBUG */
                        }

                        /* Store current time in timeStamp field */
                        (chan->curViop->timeStamp) = CLK_getltime();

                        if (chan->autoSyncEnable)
                        {
                            _autoSync(chan);
                        }

                        /* update the current and next viop pointers */
                        curViop = chan->curViop;
                        chan->curViop = chan->nextViop;

                        /* update the most recent viop */
                        if ((curViop != chan->mrViop) && (chan->mrViop != INV)
                             && (chan->nextViop != chan->mrViop))
                        {
                            QUE_enqueue(&chan->qIn, chan->mrViop);
                        }
                        chan->mrViop = curViop;

                        if ((viop = (FVID_Frame *)QUE_dequeue(&chan->qIn)) !=
                            (FVID_Frame *)&chan->qIn)
                        {
                            /* queue IS not empty */
                            chan->nextViop = viop;
                        }
                        else
                        {
                            if (INV == chan->packetIOM)
                            {
                                /* in queue is empty, but no request is pending
                                 * recycle the current VIOP back but still set
                                 * it as the most recent VIOP
                                 */
                                chan->nextViop = curViop;
                            }
                            chan->queEmpty = TRUE;
                        }

                        /* call the channel's callback function */
                        if (chan->mrViop != chan->curViop)
                        {
                            if (chan->packetIOM != INV)
                            {
                                /* Call the callback after updating the EDMA
                                   parameters */
                                callBackFlag = TRUE;
                            }
                            else if (chan->queEmpty)
                            {
                                chan->nextViop = chan->mrViop;
                            }
                        }
                        else
                        {
                            chan->mrViop = INV;
                        }

                        /* Update the EDMA reload entry */
                        _updateEDMAParamAddress(chan, offset, chan->nextViop);

                        if (callBackFlag)
                        {
                            /* call the channel's callback function */
                            *(void**)chan->packetIOM->addr = curViop;
                            chan->packetIOM->size = sizeof(FVID_Frame);
                            chan->packetIOM->status = IOM_COMPLETED;
                            chan->mrViop = INV;
                            if (NORMAL_MODE == chan->driverMode)
                            {
                                /* decrement number of buffers available */
                                chan->numQueBuf--;
                            }

                            chan->cbFxn((Ptr)chan->cbArg, chan->packetIOM);
                            chan->packetIOM = INV;
                        }
                    }
                }
            }
        }
    }
}



/**
 *  \brief configure video port channel settings
 *
 *  \param   chanp  [Out]  VPORT channel object pointer
 *  \param   params [In]   channel configuration parameters
 *
 *  \return  IOM_COMPLETED on success
 *           assertion failed on invalid parameters
 */
static Int _configRegs(Ptr chanp, VPORTCAP_Params *params)
{
    _VPORT_ChanObj  *chan = (_VPORT_ChanObj *)chanp;
    volatile Int    vpCtl, vcCtl, fld1Strt, fld2Strt, fld1Stop, fld2Stop;
    Int             retVal = IOM_COMPLETED;
    Int             numPixels, numLines, numCPixels;

    if (chan->status & _VPORT_OPENED)
    {
        QUE_new(&chan->qIn);
        chan->queEmpty = FALSE;
        chan->mrViop = INV;
        chan->packetIOM = INV;
        chan->bufSz = 0;
        chan->mode = params->cmode;
        if (chan->mode & _VPORT_MASK_RAW)
        {
            chan->autoSyncEnable = FALSE; /* no auto-sync in raw mode */
        }
        else
        {
            chan->autoSyncEnable = params->autoSyncEnable;
        }

        if (!(chan->mode & _VPORT_MASK_RAW))
        {
            /* configure channel A capture settings */
            vcCtl = VP_VCCTL_RMK(0, /* Channel reset bit, No effect */
                                 1, /* Block EDMA events and flush FIFOs */
                                 0, /* Field identification enable, Not used */
                                 params->fldInv,
                                 params->extCtl,
                                 params->fldDect,
                                 params->vCtRst,
                                 params->hCtRst,
                                 0, /* Disable video capture */
                                 params->bpk10Bit, /* Reserved */
                                 1, /* Enable long field detection */
                                 1, /* Enable short field detection */
                                 params->resmpl,
                                 params->scale,
                                 1, /* Enable Continuous capture */
                                 ((params->fldOp & 4) >> 2), /* Cap frame bit */
                                 ((params->fldOp & 2) >> 1), /* Cap fld 2 bit */
                                 (params->fldOp & 1), /* Cap fld 1 bit */
                                 params->cmode);

            fld1Strt = params->fldXStrt1 + (params->fldYStrt1 << 16);
            fld1Stop = params->fldXStop1 + (params->fldYStop1 << 16);
            fld2Strt = params->fldXStrt2 + (params->fldYStrt2 << 16);
            fld2Stop = params->fldXStop2 + (params->fldYStop2 << 16);
            if (VPORT_FLDOP_FRAME == params->fldOp)
            {
                assert(params->fldXStop1 == params->fldXStop2);
                assert(params->fldXStrt1 == params->fldXStrt2);
            }

            /* reset channel */
            chan->vportChanRegs->VC_CTL |= CSL_VPORT_VC_CTL_RSTCH_MASK;
            /* Wait for video channel to get reset */
            while (chan->vportChanRegs->VC_CTL & CSL_VPORT_VC_CTL_RSTCH_MASK);

            chan->vportChanRegs->VC_CTL = vcCtl;
            chan->vportChanRegs->VC_STRT1 = fld1Strt;
            chan->vportChanRegs->VC_STOP1 = fld1Stop;
            chan->vportChanRegs->VC_STRT2 = fld2Strt;
            chan->vportChanRegs->VC_STOP2 = fld2Stop;
            /* line size */
            numPixels = params->fldXStop1 - params->fldXStrt1 + 1;
            numLines = 0;

            if (params->fldOp != VPORT_FLDOP_FLD2)
            {
                numLines += params->fldYStop1 - params->fldYStrt1 + 1;
            }
            chan->numLinesFld1 = numLines;
            chan->numTotalLinesFld1 = params->fldYStop1;

            if (VPORT_FLDOP_FLD2 == params->fldOp ||
                VPORT_FLDOP_FRAME == params->fldOp)
            {
                numLines += params->fldYStop2 - params->fldYStrt2 + 1;
            }

            chan->resmpl = params->resmpl;
            chan->scale = params->scale;
            chan->numLines = numLines;
            numPixels >>= params->scale;
            numCPixels = numPixels >> 1;
            /* set both field1 and field2 threshold to the line size */
            chan->numPixels = numPixels;
            chan->lastLineNum = chan->numLines;

            if (params->cmode & _VPORT_MASK_10BIT)
            {
                /* 10-bit BT.656 or 20-bit Y/C mode */
                if (VPORTCAP_BPK_10BIT_ZERO_EXTENDED == params->bpk10Bit ||
                    VPORTCAP_BPK_10BIT_SIGN_EXTENDED == params->bpk10Bit)
                {
                    chan->yPitch = (numPixels * 2 + 7) & (~7);
                    chan->cPitch = (numCPixels * 2 + 7) & (~7);
                }
                else
                {
                    chan->yPitch = (numPixels * 4 / 3 + 7) & (~7);
                    chan->cPitch = (numCPixels * 4 / 3 + 7) & (~7);
                }
            }
            else
            {
                /* 8-bit BT.656 or 16-bit Y/C mode */
                chan->yPitch = (numPixels + 7) & (~7);
                chan->cPitch = (numCPixels + 7) & (~7);
            }

            chan->yThrld = params->thrld;
            chan->interlaced = TRUE;

            if (0 == chan->yThrld)
            {
                /* if threshold is zero then set it same as line size */
                chan->yThrld = chan->yPitch >> 3;
            }
            else if (((chan->yThrld << 3) > chan->yPitch) ||
                     ((chan->yPitch % (chan->yThrld << 3)) != 0))
            {
                /* if threshold is greater than yPitch or is not multiple of
                 * yPitch then set threshold same as line size
                 */
                chan->yThrld = chan->yPitch >> 3;
            }

            chan->cThrld = (chan->yThrld + 1) >> 1;

            if (((chan->cThrld << 3) > chan->cPitch) ||
                ((chan->cPitch % (chan->cThrld << 3)) != 0))
            {
                /* if threshold is greater than cPitch or is not multiple of
                 * cPitch then set threshold same as line size
                 */
                chan->yThrld = chan->yPitch >> 3;
                chan->cThrld = (chan->yThrld + 1) >> 1;
            }

            assert(((chan->yPitch*chan->numLinesFld1) / (chan->yThrld << 3))
                *(chan->yThrld <<3) == (chan->yPitch * chan->numLinesFld1));
            assert(((chan->yPitch * chan->numLines) / (chan->yThrld << 3))
                *(chan->yThrld << 3) == (chan->yPitch * chan->numLines));

            chan->numEventsFld1 = chan->yPitch * chan->numLinesFld1 /
                                  (chan->yThrld << 3);
            chan->numEvents = chan->yPitch * chan->numLines /
                              (chan->yThrld << 3);

            chan->mergeFlds = params->mergeFlds;
            if (params->fldOp != VPORT_FLDOP_FRAME)
            {
                chan->interlaced = FALSE;
                chan->mergeFlds = FALSE;
            }

            chan->vportChanRegs->VC_EVTCT = VP_VCEVTCT_RMK(
                                        (chan->numEvents - chan->numEventsFld1),
                                        chan->numEventsFld1);
            chan->vportChanRegs->VC_THRLD = VP_VCTHRLD_RMK(chan->yThrld,
                                                           chan->yThrld);
        }
        else
        {
            /* raw capture */
            VPORTCAP_ParamsRaw *paramsRaw = (VPORTCAP_ParamsRaw *)params;
            Int         totalNumPixels = paramsRaw->lineSz *paramsRaw->numLines;
            Int         totalBytesPerFrame;
            Int         bytesPerLine;

            /* make sure the number is in the correct range */
            assert(totalNumPixels < 2 ^ 24);

            /* configure channel A capture settings */
            vcCtl = VP_VCCTL_RMK(0, /* Channel reset bit, No effect */
                                 1, /* Block EDMA events and flush FIFOs */
                                 0, /* Field identification is disabled */
                                 0, /* field invert bit, Not used */
                                 0, /* External control select bit, Not used */
                                 0, /* Field detect method bit, Not used */
                                 0, /* VCOUNT reset method bit, Not used */
                                 0, /* HCOUNT reset method bit, Not used */
                                 0, /* Video capture is disabled */
                                 paramsRaw->bpk10Bit, /* Reserved */
                                 0, /* Long field detect enable, Not used */
                                 0, /* Short field detect enable, Not used */
                                 0, /* Chroma re-sampling enable, Not used */
                                 0, /* Scaling select bit, Not used */
                                 1, /* Continuous capture is enabled */
                                 0, /* Capture frame (data) bit */
                                 0, /* Do not capture field 2 */
                                 0, /* Do not capture field 1 */
                                 paramsRaw->cmode);

            /* reset channel */
            chan->vportChanRegs->VC_CTL |= CSL_VPORT_VC_CTL_RSTCH_MASK;
            /* Wait for video channel to get reset */
            while (chan->vportChanRegs->VC_CTL & CSL_VPORT_VC_CTL_RSTCH_MASK);

            chan->vportChanRegs->VC_CTL = vcCtl;
            chan->vportChanRegs->VC_STRT1
                = VP_VCSTRT1_RMK(0, paramsRaw->startupSyncEnable,
                                 paramsRaw->blankperiod);

            chan->vportChanRegs->VC_STOP1
             = VP_VCSTOP1_RMK((totalNumPixels >> 12),(totalNumPixels & 0xFFF));

            chan->resmpl = FALSE;
            chan->scale = FALSE;

            if (VPORT_MODE_RAW_16BIT == paramsRaw->cmode)
            {
                bytesPerLine = paramsRaw->lineSz * 2;
                totalBytesPerFrame = paramsRaw->lineSz * 2 *
                                     paramsRaw->numLines;
            }
            else if (VPORT_MODE_RAW_20BIT == paramsRaw->cmode)
            {
                bytesPerLine = paramsRaw->lineSz * 4;
                totalBytesPerFrame = paramsRaw->lineSz * 4 *
                                     paramsRaw->numLines;
            }
            else if (VPORT_MODE_RAW_10BIT == paramsRaw->cmode)
            {
                if (VPORTCAP_BPK_10BIT_ZERO_EXTENDED == paramsRaw->bpk10Bit ||
                    VPORTCAP_BPK_10BIT_SIGN_EXTENDED == paramsRaw->bpk10Bit)
                {
                    bytesPerLine = paramsRaw->lineSz * 2;
                    /* each 10-bit pixel takes 2 bytes */
                    totalBytesPerFrame = paramsRaw->lineSz * 2 *
                                         paramsRaw->numLines;
                }
                else
                {
                    /* dense packing, 3 10-bit pixel takes 4 bytes */
                    /* make sure this is dividable */
                    assert((paramsRaw->lineSz * paramsRaw->numLines / 3) * 3 ==
                           paramsRaw->lineSz * paramsRaw->numLines);
                    totalBytesPerFrame = paramsRaw->lineSz * 4 *
                                         paramsRaw->numLines / 3;
                    bytesPerLine = totalBytesPerFrame / paramsRaw->numLines;
                }
            }
            else
            {
                /* 8-bit raw mode */
                bytesPerLine = paramsRaw->lineSz;
                totalBytesPerFrame = paramsRaw->lineSz * paramsRaw->numLines;
            }

            chan->yPitch = bytesPerLine;
            chan->yThrld = paramsRaw->thrld;
            if (0 == chan->yThrld)
            {
                /* if threshold is zero then set it same as line size */
                chan->yThrld = chan->yPitch >> 3;
            }
            else if (((chan->yThrld << 3) > chan->yPitch) ||
                     ((chan->yPitch % (chan->yThrld << 3)) != 0))
            {
                /* if threshold is greater than yPitch or is not multiple of
                 * yPitch then set threshold same as line size
                 */
                chan->yThrld = chan->yPitch >> 3;
            }

            chan->cPitch = 0;
            chan->cThrld = 0;
            chan->interlaced = FALSE;
            chan->mergeFlds = FALSE;

            assert(totalBytesPerFrame / chan->yPitch * chan->yPitch ==
                   totalBytesPerFrame);
            /* Check whether line pitch is multiple of 8 bytes */
            assert((chan->yPitch & ~7) == chan->yPitch);

            chan->numLines = totalBytesPerFrame / chan->yPitch;
            chan->numLinesFld1 = chan->numLines;
            chan->numEvents = chan->numLines;
            chan->lastLineNum = chan->numLines;
            chan->numPixels = paramsRaw->lineSz;

            chan->vportChanRegs->VC_THRLD = VP_VCTHRLD_RMK(chan->yThrld,
                                                           chan->yThrld);
        } /* if (!(chan->mode & _VPORT_MASK_RAW)) */

        chan->status |= _VPORT_CFGED;
        retVal = IOM_COMPLETED;
    }

    return retVal;
}



/**
 *  \brief configure video port channel and EDMA settings
 *
 *  \param   chanp  [Out]  VPORT channel object pointer
 *  \param   args   [In]   capture channel configuration parameters
 *
 *  \return  IOM_COMPLETED on success
 *           suitable error code on failure
 */
static Int _configChan(Ptr chanp, Ptr args)
{
    VPORTCAP_Params     *params = (VPORTCAP_Params *)args;
    VPORTCAP_ParamsRaw  *paramsRaw = (VPORTCAP_ParamsRaw *)args;
    _VPORT_ChanObj      *chan = (_VPORT_ChanObj *)chanp;
    Int                 retVal = IOM_COMPLETED;
    Uint8               bufCnt = 0;
    Uint32              prevStatus = chan->status;
    Int32               numFrmBufs;
    Int32               segId;

    if (!(params->cmode & _VPORT_MASK_RAW))
    {
        numFrmBufs = params->numFrmBufs;
        segId = params->segId;
    }
    else
    {
        numFrmBufs = paramsRaw->numFrmBufs;
        segId = paramsRaw->segId;
    }

    /* Validate parameters
     * Driver mode cannot be changed at runtime
     */
    if (LEGACY_MODE == chan->driverMode)
    {
        if ((numFrmBufs < _VPORT_MIN_BUF) ||
            (numFrmBufs > VPORT_MAX_NUM_FRMBUFS))
        {
            retVal = IOM_EBADARGS;
        }
    }
    else if ((NORMAL_MODE == chan->driverMode) && (0 != numFrmBufs))
    {
        retVal = IOM_EBADARGS;
    }

    if ((INV == (void*)segId))
    {
        retVal = IOM_EBADARGS;
    }

    if (IOM_COMPLETED == retVal)
    {
        /* If capture operation is started then first stop the operation */
        if (_VPORT_STARTED & prevStatus)
        {
            retVal = mdControlChan(chan, VPORT_CMD_STOP, NULL);
        }

        /* Free memory of previously allocated buffers
         * (Applicable in LEGACY MODE)
         */
        if ((_VPORT_BUF_ALLOCATED & prevStatus) && (IOM_COMPLETED == retVal))
        {
            for (bufCnt = 0; bufCnt < chan->numFrms; bufCnt++)
            {
                if (PAL_SOK != PAL_osMemFree(chan->segId,
                                             chan->viops[bufCnt].frame.iFrm.y1,
                                             chan->bufSz))
                {
                    retVal = IOM_EFREE;
                }
            } /* for (bufCnt = 0;... */
        } /* if (_VPORT_BUF_ALLOCATED & prevStatus) */
    }

    if (IOM_COMPLETED == retVal)
    {
        /* Initialize channel status as Opened */
        chan->status = _VPORT_OPENED;

        /* configure video port channel A/B control register */
        _configRegs(chanp, params);

        /* assign VPORTCAP_Params passed from application to _VPORT_ChanObj
         * global member
         */
        chan->numFrms = numFrmBufs;
        if (!(params->cmode & _VPORT_MASK_RAW))
        {
            chan->segId = params->segId;
            chan->alignment = params->alignment;
            chan->fldOp = params->fldOp;
        }
        else
        {
            chan->segId = paramsRaw->segId;
            chan->alignment = paramsRaw->alignment;
            chan->fldOp = 0;
        }

        chan->numQueBuf = 0;
        chan->mrViop = INV;
        chan->packetIOM = INV;
        chan->bufSz = (chan->yPitch * chan->numLines)
                         + (chan->cPitch * chan->numLines * 2);

        if (LEGACY_MODE == chan->driverMode)
        {
            /* allocate frame buffers and configure EDMA */
            retVal = _configEDMA(chanp);

            /* If channel's previous status was started then,start capture
             * after reconfiguration(Applicable only in LEGACY MODE.
             * In NORMAL MODE, application has to queue buffers before starting
             * capture again)
             */
            if ((IOM_COMPLETED == retVal) && (_VPORT_STARTED & prevStatus))
            {
                retVal = mdControlChan(chan, VPORT_CMD_START, NULL);
            }
        } /* if (LEGACY_MODE == chan->driverMode) */
    } /* if (IOM_COMPLETED == retVal) */

    return retVal;
}



/**
 *  \brief configure port level registers
 *
 *  \param   chanp  [In]  VPORT channel object pointer
 *  \param   args   [In]  capture channel configuration parameters
 *
 *  \return  IOM_COMPLETED
 */
static Int _configPort(Ptr chanp, Ptr args)
{
    PortObj             *port = (PortObj *)chanp;
    VPORT_PortParams    *portParams = (VPORT_PortParams *)args;

    /* enable video port */
    port->vportBase->PCR |= CSL_VPORT_PCR_PEREN_PORT_FUNC
                            << CSL_VPORT_PCR_PEREN_SHIFT;

    /* reset video port */
    port->vportBase->VP_CTL |= CSL_VPORT_VP_CTL_VPRST_MASK;

    /* Wait for video port to get reset */
    while (port->vportBase->VP_CTL & CSL_VPORT_VP_CTL_VPRST_MASK);

    port->vportBase->VP_CTL = VP_VPCTL_RMK(0, /* VP reset enable, No effect */
                                           0, /* VP halt bit, No effect */
                                           0, /* VCLK2 pin polarity, Noeffect */
                                           portParams->vc3Polarity,
                                           portParams->vc2Polarity,
                                           portParams->vc1Polarity,
                                           0, /* Disable TCI capture */
                                           0, /* Enable Capture mode */
                                           portParams->dualChanEnable);

    /* enable video port */
    port->vportBase->VP_CTL |= (CSL_VPORT_VP_CTL_VPHLT_RESETVAL
                               << CSL_VPORT_VP_CTL_VPHLT_SHIFT);

    /* Assign EDC function table pointer to global channel object */
    port->chanObj[0].edcFxns = portParams->edcTbl[0];
    port->chanObj[1].edcFxns = portParams->edcTbl[1];

    return IOM_COMPLETED;
}



/**
 *  \brief configure channel EDMA settings
 *
 *  \param   chanp  [In/Out]  VPORT channel object pointer
 *
 *  \return  IOM_COMPLETED on success
 *           IOM_EALLOC on failure
 */
static Int _configEDMA(Ptr chanp)
{
    _VPORT_ChanObj      *chan = (_VPORT_ChanObj *)chanp;
    Int                 i, j;
    Int                 thrld, pitch;
    Int                 numEdmaChans;
    Int                 retVal = IOM_COMPLETED;
    EDMA3_DRV_PaRAMRegs paramPtr = { 0 };
    EDMA3_DRV_Result    result = EDMA3_DRV_SOK;
    unsigned short      tccStatus;

    if (chan->status & _VPORT_CFGED)
    {
        if (LEGACY_MODE == chan->driverMode)
        {
            chan->queEmpty = FALSE;
        }

        /* Clear any pending interrupts */
        result |= EDMA3_DRV_checkAndClearTcc(chan->hEdma,
                                             chan->tcc[0],
                                             &tccStatus);
        result |= EDMA3_DRV_checkAndClearTcc(chan->hEdma,
                                             chan->tcc[1],
                                             &tccStatus);

        if ((!(_VPORT_BUF_ALLOCATED & chan->status)) &&
            (LEGACY_MODE == chan->driverMode))
        {
            /* allocate frame buffer */
            for (i = 0; i < chan->numFrms; i++)
            {
                /* allocate frame buffer and assign to FVID_Frame structure */
                if (IOM_COMPLETED != _allocAssignBuffer(chan,
                                                        &(chan->viops[i])))
                {
                    retVal = IOM_EALLOC;

                    for (j = 0; j < i; j++)
                    {
                        PAL_osMemFree(chan->segId,
                                      chan->viops[j].frame.iFrm.y1,
                                      chan->bufSz);
                    }
                    break;
                }
            }

            if (IOM_COMPLETED == retVal)
            {
                chan->status |= _VPORT_BUF_ALLOCATED;

                for (i = 2; i < chan->numFrms; i++)
                {
                    /* don't put the first 2 viop into the queue */
                    QUE_enqueue(&chan->qIn, (QUE_Handle)&chan->viops[i]);
                }
            }
        } /* if ((!(_VPORT_BUF_ALLOCATED & chan->status)) */

        if (IOM_COMPLETED == retVal)
        {
            /* set current and next viop, which will be used for EDMA
             * configurations
             */
            if (LEGACY_MODE == chan->driverMode)
            {
                /* first 2 buffers are not queued, take buffers from chan obj */
                chan->curViop = &chan->viops[0];
                chan->nextViop = &chan->viops[1];
            }
            else
            {
                /* get buffers from queue */
                chan->curViop = (FVID_Frame *)(QUE_dequeue(&chan->qIn));
                chan->nextViop = (FVID_Frame *)(QUE_dequeue(&chan->qIn));

                /* Here, we have dequeued 2 buffers from qIn, check if there
                 * are any buffers in qIn or not. If there is no buffer in qIn
                 * then set queEmpty as TRUE
                 */
                if (0 == (chan->numQueBuf - 2))
                {
                    chan->queEmpty = TRUE;
                }
                else
                {
                    chan->queEmpty = FALSE;
                }
            }

            if (chan->mode & _VPORT_MASK_RAW)
            {
                numEdmaChans = 1;
            }
            else
            {
                numEdmaChans = _VPORT_NUM_EDMA_CHANS;
            }

            for (i = 0; i < numEdmaChans; i++)
            {
                thrld = (i == 0) ? chan->yThrld : chan->cThrld;
                pitch = (i == 0) ? chan->yPitch : chan->cPitch;

                paramPtr.opt = 0;
                /* set source address mode */
                paramPtr.opt |= EDMA3_DRV_ADDR_MODE_FIFO;
                /* set destination address mode */
                paramPtr.opt |= (EDMA3_DRV_ADDR_MODE_INCR << EDMA_DAM_SHIFT);
                /* set sync mode */
                paramPtr.opt |= (EDMA3_DRV_SYNC_A << EDMA_SYNCDIM_SHIFT);
                paramPtr.opt |= (EDMA3_DRV_W64BIT << EDMA_FWID_SHIFT);

                /* set source address as FIFO address */
                paramPtr.srcAddr = (unsigned int)(chan->edmaAddr[i]);

                if (chan->interlaced)
                {
                    /* to merge the two fields together, EDMA is configured
                     * to transfer only field 1 initially.
                     * line pitch is twice the line size,
                     * this requires that threshold is the same as line size
                     */

                    /* set aCnt, bCnt, cCnt values */
                    paramPtr.aCnt = (thrld << 3);
                    paramPtr.bCnt = (pitch / paramPtr.aCnt);
                    paramPtr.cCnt = chan->numLinesFld1;
                    paramPtr.bCntReload = paramPtr.bCnt;

                    /* Set index values for source and destination */
                    paramPtr.srcBIdx = 0;
                    paramPtr.destBIdx = paramPtr.aCnt;
                    paramPtr.srcCIdx = 0;
                    if (chan->mergeFlds)
                    {
                        paramPtr.destCIdx = paramPtr.aCnt + pitch;
                    }
                    else
                    {
                        paramPtr.destCIdx = paramPtr.aCnt;
                    }

                    /* hard code first 2 frames as current and reload buffers
                     * First Field
                     */
                    paramPtr.opt |= (EDMA3_DRV_TCINTEN_DIS
                                  << EDMA_TCINTEN_SHIFT);
                    paramPtr.destAddr
                        = (*((Int *)(&(chan->curViop->frame.iFrm.y1)) + i));

                    paramPtr.linkAddr = (unsigned short)
                       ((Uint32)chan->hEdmaLinkChanPhyAddr[4 * i + 1] & 0xFFFF);

                    result |= EDMA3_DRV_setPaRAM(chan->hEdma,
                                                 chan->hEdmaChan[i],
                                                 &paramPtr);
                    result |= EDMA3_DRV_setPaRAM(chan->hEdma,
                                                 chan->edmaLinkChanNum[4 * i],
                                                 &paramPtr);

                    paramPtr.destAddr =
                             (*((Int *)(&(chan->nextViop->frame.iFrm.y1)) + i));

                    paramPtr.linkAddr = (unsigned short)
                       ((Uint32)chan->hEdmaLinkChanPhyAddr[4 * i + 3] & 0xFFFF);

                    result |= EDMA3_DRV_setPaRAM(chan->hEdma,
                                            chan->edmaLinkChanNum[4 * i + 2],
                                            &paramPtr);

                    /* Second Field */
                    paramPtr.opt |=
                               ((i == 0)
                               ?(EDMA3_DRV_TCINTEN_EN << EDMA_TCINTEN_SHIFT)
                               :(EDMA3_DRV_TCINTEN_DIS << EDMA_TCINTEN_SHIFT));

                    paramPtr.opt &= EDMA_TCC_MASK;
                    paramPtr.opt |= ((i == 0)
                                     ? (chan->tcc[0] << EDMA_TCC_SHIFT)
                                     : (0 << EDMA_TCC_SHIFT));

                    paramPtr.cCnt = (chan->numLines - chan->numLinesFld1);
                    paramPtr.destAddr =
                            (*((Int *)(&(chan->curViop->frame.iFrm.y2)) + i));

                    paramPtr.linkAddr = (unsigned short)
                       ((Uint32)chan->hEdmaLinkChanPhyAddr[4 * i + 2] & 0xFFFF);

                    result |= EDMA3_DRV_setPaRAM(chan->hEdma,
                                            chan->edmaLinkChanNum[4 * i + 1],
                                            &paramPtr);

                    paramPtr.opt &= EDMA_TCC_MASK;
                    paramPtr.opt |= ((i == 0)
                                     ? (chan->tcc[1] << EDMA_TCC_SHIFT)
                                     : (0 << EDMA_TCC_SHIFT));

                    paramPtr.destAddr =
                            (*((Int *)(&(chan->nextViop->frame.iFrm.y2)) + i));

                    paramPtr.linkAddr = (unsigned short)
                       ((Uint32)chan->hEdmaLinkChanPhyAddr[4 * i] & 0xFFFF);

                    result |= EDMA3_DRV_setPaRAM(chan->hEdma,
                                            chan->edmaLinkChanNum[4 * i + 3],
                                            &paramPtr);
                }
                else
                {
                    /* if not interlaced capture, configure EDMA to transfer
                     * the whole frame. The line pitch is just the line size
                     * set aCnt, bCnt, cCnt values
                     */
                    paramPtr.aCnt = (thrld << 3);
                    paramPtr.bCnt = (pitch / paramPtr.aCnt);
                    paramPtr.cCnt = chan->numLines;
                    paramPtr.bCntReload = paramPtr.bCnt;

                    /* Set index values for source and destination */
                    paramPtr.srcBIdx = 0;
                    paramPtr.destBIdx = paramPtr.aCnt;
                    paramPtr.srcCIdx = 0;
                    paramPtr.destCIdx = paramPtr.aCnt;

                    paramPtr.opt |=
                                ((i == 0)
                            ?(EDMA3_DRV_TCINTEN_EN << EDMA_TCINTEN_SHIFT)
                            :(EDMA3_DRV_TCINTEN_DIS << EDMA_TCINTEN_SHIFT));

                    paramPtr.opt &= EDMA_TCC_MASK;
                    paramPtr.opt |=
                            ((i == 0)
                            ?(chan->tcc[0] << EDMA_TCC_SHIFT)
                            :(0 << EDMA_TCC_SHIFT));

                    /* hard code the first and second frame buffer as current
                     * and reload buffers
                     */
                    paramPtr.destAddr =
                            (*((Int *)(&(chan->curViop->frame.iFrm.y1)) + i));

                    paramPtr.linkAddr = (unsigned short)
                       ((Uint32)chan->hEdmaLinkChanPhyAddr[4 * i + 2] & 0xFFFF);

                    result |= EDMA3_DRV_setPaRAM(chan->hEdma,
                                                 chan->hEdmaChan[i],
                                                 &paramPtr);
                    result |= EDMA3_DRV_setPaRAM(chan->hEdma,
                                                 chan->edmaLinkChanNum[4 * i],
                                                 &paramPtr);

                    paramPtr.opt &= EDMA_TCC_MASK;
                    paramPtr.opt |=
                            ((i == 0)
                            ?(chan->tcc[1] << EDMA_TCC_SHIFT)
                            :(0 << EDMA_TCC_SHIFT));

                    paramPtr.destAddr =
                            (*((Int *)(&(chan->nextViop->frame.iFrm.y1)) + i));

                    paramPtr.linkAddr = (unsigned short)
                       ((Uint32)chan->hEdmaLinkChanPhyAddr[4 * i] & 0xFFFF);

                    result |= EDMA3_DRV_setPaRAM(chan->hEdma,
                                             chan->edmaLinkChanNum[4 * i + 2],
                                             &paramPtr);

                } /* if (chan->interlaced) */
            } /* for (i = 0; i < numEdmaChans; i++) */

            chan->nextEDMARlds = 1;
            /* enable EDMA channel */
            /*
             * The EDMA interrupt dispatcher will be called by the
             * BIOS HWI interrupt dispatcher.
             */
            for (i = 0; i < numEdmaChans; i++)
            {
                result |= EDMA3_DRV_enableTransfer(chan->hEdma,
                                                   chan->hEdmaChan[i],
                                                   EDMA3_DRV_TRIG_MODE_EVENT);
            }

            chan->status |= _VPORT_READY;
        } /* if (IOM_COMPLETED == retVal) */
    } /* if (chan->status & _VPORT_CFGED) */

    if ((EDMA3_DRV_SOK != result) && (IOM_COMPLETED == retVal))
    {
        retVal = IOM_EALLOC;
    }

    return retVal;
}



/**
 *  \brief   Reconfigure EDMA hardware channels with current param set
 *
 *  \param   chanp  [In/Out]  VPORT channel object pointer
 *
 *  \return  IOM_COMPLETED on success
 *           IOM_EALLOC on failure
 */
static Int _reconfigEDMAchan(Ptr chanp)
{
    _VPORT_ChanObj      *chan = (_VPORT_ChanObj *)chanp;
    Int                 offset;
    EDMA3_DRV_PaRAMRegs paramPtr = { 0 };
    Int                 retVal = IOM_COMPLETED;
    EDMA3_DRV_Result    result = EDMA3_DRV_SOK;
    Int                 numEdmaChans;
    volatile Int        i;
    unsigned short      tccStatus;
    unsigned int        link1DestAddr, link2DestAddr;

    /* Disable the EDMA before setting them up */
    if (chan->mode & _VPORT_MASK_RAW)
    {
        numEdmaChans = 1;
    }
    else
    {
        numEdmaChans = _VPORT_NUM_EDMA_CHANS;
    }

    /* Disable EDMA transfers */
    for (i = 0; i < numEdmaChans; i++)
    {
        result |= EDMA3_DRV_disableTransfer(chan->hEdma,
                                            chan->hEdmaChan[i],
                                            EDMA3_DRV_TRIG_MODE_EVENT);

        /* Clear any pending errors and events */
        result |= EDMA3_DRV_clearErrorBits(chan->hEdma, chan->hEdmaChan[i]);
    }

    /* Clear any pending interrupts - for both the TCC (current & next VIOP) */
    result |= EDMA3_DRV_checkAndClearTcc(chan->hEdma, chan->tcc[0], &tccStatus);
    result |= EDMA3_DRV_checkAndClearTcc(chan->hEdma, chan->tcc[1], &tccStatus);

    /************** Reconfigure H/W EDMA parameters again **************/

    /* Find out which param set matches with the software current viop */
    link1DestAddr = chan->hEdmaLinkChanPhyAddr[0]->destAddr;
    link2DestAddr = chan->hEdmaLinkChanPhyAddr[2]->destAddr;
    if (link1DestAddr == (Uint32)chan->curViop->frame.iFrm.y1)
    {
        /* 1st param set matches with current viop in software. Reload param set
           0 into hardware channel for restart */
        offset = 0;
        /* 1st param set will be reloaded to hardware channel. Hence next EDMA
           reload will be param set 1 */
        chan->nextEDMARlds = 1;
    }
    else if (link2DestAddr == (Uint32)chan->curViop->frame.iFrm.y1)
    {
        /* 2nd param set matches with current viop in software. Reload param set
           1 into hardware channel for restart */
        offset = 2;
        /* 2nd Param will be reloaded to hardware channel. Hence next EDMA
           reload will be param set 0 */
        chan->nextEDMARlds = 0;
    }
    else
    {
#ifdef VPORT_DEBUG
        chan->reconfigErrCount++;
#endif  /* #ifdef VPORT_DEBUG */

        /*
         * Some error has happened!! Reuse software maintained param offset
         */
        /* Based on current EDMA parameters loaded in EDMA channels, get
         * particular param set and configure EDMA channels again using this
         * param set. chan->nextEDMARlds indicates next EDMA paramset,
         * complement of it will give current EDMA parameters (i.e if
         * chan->nextEDMARlds is 0 then current EDMA paramset will be 1 and
         * vice-versa).
         */
        offset = ((!chan->nextEDMARlds) << 1);
    }

    for (i = 0; i < numEdmaChans; i++)
    {
        /* Reconfigure Y, Cb and Cr Parameters */
        result |= EDMA3_DRV_getPaRAM(chan->hEdma,
                                     chan->edmaLinkChanNum[(i * 4) + offset],
                                     &paramPtr);
        result |= EDMA3_DRV_setPaRAM(chan->hEdma,
                                     chan->hEdmaChan[i],
                                     &paramPtr);
    }

    /* enable the EDMA events again */
    for (i = 0; i < numEdmaChans; i++)
    {
        result |= EDMA3_DRV_enableTransfer(chan->hEdma,
                                           chan->hEdmaChan[i],
                                           EDMA3_DRV_TRIG_MODE_EVENT);
    }

    if (EDMA3_DRV_SOK == result)
    {
        retVal = IOM_COMPLETED;
    }
    else
    {
        retVal = IOM_EALLOC;
    }

    return retVal;
}



/**
 *  \brief   allocating buffer as requested by application
 *
 *  \param   chan     [In]   VPORT channel object pointer
 *  \param   newFrame [Out]  Pointer to FVID frame pointer
 *
 *  \return  IOM_COMPLETED on success
 *           IOM_EALLOC on failure
 */
static Int _allocBuffer(_VPORT_ChanObj *chan, FVID_Frame **newFrame)
{
    Int retVal = IOM_COMPLETED;

    /* allocate frame buffer structure */
    if (PAL_SOK != PAL_osMemAlloc(chan->segId,
                                  sizeof(FVID_Frame),
                                  NULL,
                                  (Ptr *)newFrame))
    {
        retVal = IOM_EALLOC;
    }

    if (IOM_EALLOC != retVal)
    {
        /* allocate frame buffer and assign to FVID_Frame structure */
        if (IOM_COMPLETED != _allocAssignBuffer(chan, *newFrame))
        {
            retVal = IOM_EALLOC;
            PAL_osMemFree(chan->segId, *newFrame, sizeof(FVID_Frame));
        }
    }

    return retVal;
}



/**
 *  \brief   allocating buffer and assigning it to FVID_Frame structure
 *
 *  \param   chan     [In]  VPORT channel object pointer
 *  \param   newFrame [Out]  FVID frame pointer
 *
 *  \return  IOM_COMPLETED on success
 *           IOM_EALLOC on failure
 */
static Int _allocAssignBuffer(_VPORT_ChanObj *chan, FVID_Frame *newFrame)
{
    Int     retVal = IOM_COMPLETED;
    Int8    *curAddr = NULL;

    /* allocate frame buffer */
    if (PAL_SOK == PAL_osMemAlloc(chan->segId,
                                  chan->bufSz,
                                  chan->alignment,
                                  (Ptr *)&curAddr))
    {
        /* field 1 */
        newFrame->frame.iFrm.y1 = curAddr;

        PAL_osCacheFlushAndInvalidate(PAL_osCache_NOT_DEFINED,
                                      (Uint32)newFrame->frame.iFrm.y1,
                                      chan->bufSz);

        if (!(chan->mode & _VPORT_MASK_RAW))
        {
            curAddr += chan->numLines * chan->yPitch;
            newFrame->frame.iFrm.cb1 = curAddr;
            curAddr += chan->numLines * chan->cPitch;
            newFrame->frame.iFrm.cr1 = curAddr;
            curAddr += chan->numLines * chan->cPitch;
            /* field 2 */
            if (VPORT_FLDOP_FLD2 == chan->fldOp)
            {
                newFrame->frame.iFrm.y2 = newFrame->frame.iFrm.y1;
                newFrame->frame.iFrm.cb2 = newFrame->frame.iFrm.cb1;
                newFrame->frame.iFrm.cr2 = newFrame->frame.iFrm.cr1;
            }
            else if (!chan->mergeFlds)
            {
                newFrame->frame.iFrm.y2 = newFrame->frame.iFrm.y1 +
                                          chan->numLinesFld1 * chan->yPitch;
                newFrame->frame.iFrm.cb2 = newFrame->frame.iFrm.cb1 +
                                           (chan->numLinesFld1 * chan->cPitch);
                newFrame->frame.iFrm.cr2 = newFrame->frame.iFrm.cr1 +
                                           (chan->numLinesFld1 * chan->cPitch);
            }
            else
            {
                newFrame->frame.iFrm.y2 = newFrame->frame.iFrm.y1 +
                                          chan->yPitch;
                newFrame->frame.iFrm.cb2 = newFrame->frame.iFrm.cb1 +
                                           chan->cPitch;
                newFrame->frame.iFrm.cr2 = newFrame->frame.iFrm.cr1 +
                                           chan->cPitch;
            }

            newFrame->frameFormat = FVID_YCbCr422_PLANAR;
            newFrame->bpp = FVID_BPP_BITS16;
        }
        else
        {
            newFrame->frameFormat = FVID_RAW_FORMAT;

            if (VPORT_MODE_RAW_8BIT == chan->mode)
            {
                newFrame->bpp = FVID_BPP_BITS8;
            }
            else
            {
                newFrame->bpp = FVID_BPP_BITS16;
            }
        } /* if (!(chan->mode & _VPORT_MASK_RAW)) */

        /* Set FVID_Frame fields */
        newFrame->timeStamp = 0;
        newFrame->pitch = chan->yPitch;
        newFrame->lines = chan->numLines;
    }
    else
    {
        retVal = IOM_EALLOC;
    }

    return retVal;
}



/**
 *  \brief   free the given frame buffer and its structure memory
 *
 *  \param   chan     [In]  VPORT channel object pointer
 *  \param   freeFrame [In]  FVID frame pointer
 *
 *  \return  IOM_COMPLETED on success
 *           IOM_EFREE on failure
 */
static Int _freeBuffer(_VPORT_ChanObj *chan, FVID_Frame **freeFrame)
{
    Int retVal = IOM_COMPLETED;
    FVID_Frame *ptrFrame = *freeFrame;

    if ((NULL == ptrFrame) || (NULL == ptrFrame->frame.iFrm.y1))
    {
        retVal = IOM_EBADARGS;
    }
    else
    {
        /* free frame buffer */
        if (PAL_SOK == PAL_osMemFree(chan->segId,
                                     ptrFrame->frame.iFrm.y1,
                                     chan->bufSz))
        {
            if (PAL_SOK != PAL_osMemFree(chan->segId, ptrFrame,
                                         sizeof(FVID_Frame)))
            {
                retVal = IOM_EFREE;
            }
        }
        else
        {
            retVal = IOM_EFREE;
        }
    }

    return retVal;
}



/**
 *  \brief   Force recover from FIFO over-run and short field detect error
 *
 *  \param   chanp  [In]  VPORT channel object pointer
 *
 *  \return  IOM_COMPLETED on success
 *           IOM_EALLOC on failure
 */
static Int _covrRecover(Ptr chanp)
{
    _VPORT_ChanObj      *chan = (_VPORT_ChanObj *)chanp;
    Int                 retVal = IOM_COMPLETED;
    volatile Uint32     prevIE_Reg = chan->vportRegs->VP_IE;

    /* disable over-run and short field detect interrupts */
    chan->vportRegs->VP_IE &= ~((CSL_VPORT_VP_IE_COVRA_MASK
                                    | CSL_VPORT_VP_IE_SFDA_MASK)
                                    << (chan->chanNum * 16));
    /* block capture events */
    chan->vportChanRegs->VC_CTL |= CSL_VPORT_VC_CTL_BLKCAP_MASK;

    /* Configure EDMA channel again */
    retVal = _reconfigEDMAchan(chan);

    /* clear any pending over-run and short field detect interrupt */
    if (0 == chan->chanNum)
    {
        chan->vportRegs->VP_IS = (CSL_VPORT_VP_IS_COVRA_MASK
                                | CSL_VPORT_VP_IS_SFDA_MASK);
    }
    else
    {
        chan->vportRegs->VP_IS = (CSL_VPORT_VP_IS_COVRB_MASK
                                | CSL_VPORT_VP_IS_SFDB_MASK);
    }

    /* enable event generation */
    chan->vportChanRegs->VC_CTL &= ~(CSL_VPORT_VC_CTL_BLKCAP_MASK);

    /* enable over-run interrupt */
    chan->vportRegs->VP_IE |= CSL_VPORT_VP_IE_COVRA_MASK
                              << (chan->chanNum * 16);

    /* Enable short field detect interrupt if already enabled */
    if (prevIE_Reg & (CSL_VPORT_VP_IE_SFDA_MASK << (chan->chanNum * 16)))
    {
        chan->vportRegs->VP_IE |= CSL_VPORT_VP_IE_SFDA_MASK
                              << (chan->chanNum * 16);
    }

    return retVal;
}



/**
 *  \brief   get the number of pending requests
 *
 *  \param   chanp     [In]  VPORT channel object pointer
 *  \param   args      [Out] number of pending requests
 *
 *  \return  IOM_COMPLETED
 */
static Int _getNumPendingIORqsts(Ptr chanp, Ptr args)
{
    _VPORT_ChanObj  *chan = (_VPORT_ChanObj *)chanp;
    QUE_Handle      nextElem = &chan->qIn;
    Int             i = 0;

    while ((nextElem = QUE_next(nextElem)) != &chan->qIn)
    {
        i++;
    }

    if (chan->curViop != chan->nextViop)
    {
        i++;
    }

    *(int*)args = i;

    return IOM_COMPLETED;
}



/**
 *  \brief   get current channel parameters
 *
 *  \param   chanp      [In]  VPORT channel object pointer
 *  \param   currParams [Out] pointer to current channel parameters structure
 *
 *  \return  IOM_COMPLETED
 */
static Int _getChanParams(Ptr chanp, VPORT_CurrParams *currParams)
{
    _VPORT_ChanObj *chan = (_VPORT_ChanObj *)chanp;

    /* Return current parameters from global channel object */
    currParams->mode = chan->mode;
    currParams->fldOp = chan->fldOp;
    currParams->scale = chan->scale;
    currParams->resmpl = chan->resmpl;
    currParams->yPitch = chan->yPitch;
    currParams->cPitch = chan->cPitch;
    currParams->numLines = chan->numLines;
    currParams->numPixels = chan->numPixels;
    currParams->bufSize = chan->bufSz;
    currParams->mergeFlds = chan->mergeFlds;

    return IOM_COMPLETED;
}



/**
 *  \brief   setup video port interrupt call-back
 *
 *  \param   chanp   [In]  VPORT channel object pointer
 *  \param   args    [In]  VPORT interrupt callback parameters
 *
 *  \return  IOM_COMPLETED
 */
static Int _setVIntCb(Ptr chanp, Ptr args)
{
    _VPORT_ChanObj      *chan = (_VPORT_ChanObj *)chanp;
    VPORT_VIntCbParams  *vIntCbParams = (void *)args;
    Int                 mask = vIntCbParams->vIntMask;
    Uns                 vif2 = 0, vInt2 = 0, vif1 = 0, vInt1 = 0, fscl2 = 0;
    Uns                 vcVint = 0;
    ECM_Attrs           ecmattrs = ECM_ATTRS;
    Uns                 cookie = 0;

    /* check to see if vertical interrupt is enabled */
    if (mask & (VPORT_INT_VINT1 | VPORT_INT_VINT2))
    {
        /* Read current value of VC_VINT register */
        vcVint = chan->vportChanRegs->VC_VINT;

        vif2 = (vcVint & CSL_VPORT_VC_VINT_VIF2_MASK)
                >> CSL_VPORT_VC_VINT_VIF2_SHIFT;
        vInt2 = (vcVint & CSL_VPORT_VC_VINT_VINT2_MASK)
                >> CSL_VPORT_VC_VINT_VINT2_SHIFT;
        vif1 = (vcVint & CSL_VPORT_VC_VINT_VIF1_MASK)
                >> CSL_VPORT_VC_VINT_VIF1_SHIFT;
        vInt1 = (vcVint & CSL_VPORT_VC_VINT_VINT1_MASK)
                >> CSL_VPORT_VC_VINT_VINT1_SHIFT;

        if (mask & VPORT_INT_VINT1)
        {
            vif1 = 1;
            vInt1 = vIntCbParams->vIntLine;
            /* modify line interrupt value, this value is used if vertical
             * interrupt is set using VPORTCAP_CMD_SET_LINE_INT ioctl
             */
            chan->lineIntInfo.vIntFld1 = vInt1;
        }
        if (mask & VPORT_INT_VINT2)
        {
            vif2 = 1;
            vInt2 = vIntCbParams->vIntLine;
            /* modify line interrupt value, this value is used if vertical
             * interrupt is set using VPORTCAP_CMD_SET_LINE_INT ioctl
             */
            chan->lineIntInfo.vIntFld2 = vInt2;
        }
        fscl2 = vif2 & (~vif1);

        /* setup vertical interrupt */
        chan->vportChanRegs->VC_VINT = VP_VCVINT_RMK(vif2,
                                                     fscl2,
                                                     vInt2,
                                                     vif1,
                                                     vInt1);
    } /* if (mask & (VPORT_INT_VINT1 | VPORT_INT_VINT2)) */

    if (1 == chan->chanNum)
    {
        mask <<= 16;    /* channel B */
    }
    chan->vIntMask = mask;
    if (mask)
    {
        mask |= CSL_VPORT_VP_IE_VIE_MASK; /* turn on video port interrupt */

        PAL_osEvtDisable((Uint32)(CSL_INTC_EVENTID_VP0_INT + chan->portNum));
        /* Enable the interrupt */
        ecmattrs.unmask = 1u;

        /* setup arg such that isr will get 'chan' as arg */
        ecmattrs.arg = (Arg)chan->portNum;

        /* Disabling the interrupts */
        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

        /* Plug the ISR with interrupt (event) number */
        ECM_dispatchPlug((Uns)(CSL_INTC_EVENTID_VP0_INT + chan->portNum),
                         (ECM_Fxn)captureISR,
                         &ecmattrs);
        /* Enabling the HWI_ID where corresponding ECM group is configured */
        C64_enableIER(1 << _IRQ_ID);

        /* Enabling the interrupts */
        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
    }

    chan->vportRegs->VP_IE |= mask;
    chan->vIntFxn = vIntCbParams->vIntCbFxn;
    chan->vIntCbArg = vIntCbParams->cbArg;

    return IOM_COMPLETED;
}



/**
 *  \brief   start video port capture operation
 *
 *  \param   chanp   [In]  VPORT channel object pointer
 *
 *  \return  IOM_COMPLETED
 */
static Int _startCapture(Ptr chanp)
{
    _VPORT_ChanObj  *chan = (_VPORT_ChanObj *)chanp;

    /* enable channel */
    chan->vportChanRegs->VC_CTL |= CSL_VPORT_VC_CTL_VCEN_MASK;

    /* clear the block capture event mask bit to enable generating capture
     * events
     */
    chan->vportChanRegs->VC_CTL &= ~(CSL_VPORT_VC_CTL_BLKCAP_MASK);

    /* enable interrupt generation in video port level */
    chan->vportRegs->VP_IE |= CSL_VPORT_VP_IE_VIE_MASK;
    chan->vportRegs->VP_IS = 0XFFFFFFFF;

    /* Enabling the event */
    PAL_osEvtEnable((Uint32)(CSL_INTC_EVENTID_VP0_INT + chan->portNum));

    /* set status as start opration done successfully */
    chan->status |= _VPORT_STARTED;

#ifdef VPORT_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT(&DVTEvent_Log,
                  "VPORT_CAPTURE",
                  PSP_DVT_DESC(PSP_DVT_eRCV_START,
                               PSP_DVT_dCHANNEL_ID_INST,
                               PSP_DVT_dPACKET_ID,
                               PSP_DVT_dNONE),
                  CHANNELID_INSTANCEID(
                               chan->chanNum,
                               chan->portNum),
                  chan->curViop
                  );
#endif /* VPORT_INSTRUMENTATION_ENABLED */

    return IOM_COMPLETED;
}



/**
 *  \brief   stop video port capture operation
 *
 *  \param   chanp   [In]  VPORT channel object pointer
 *
 *  \return  IOM_COMPLETED
 */
static Int _stopCapture(Ptr chanp)
{
    _VPORT_ChanObj  *chan = (_VPORT_ChanObj *)chanp;
    Int             retVal = IOM_COMPLETED;

    /* block events generation */
    chan->vportChanRegs->VC_CTL |= (CSL_VPORT_VC_CTL_BLKCAP_MASK);

    /* disable channel */
    chan->vportChanRegs->VC_CTL &= ~(CSL_VPORT_VC_CTL_VCEN_MASK);

    /* disable interrupt generation in video port level */
    chan->vportRegs->VP_IE &= ~(CSL_VPORT_VP_IE_VIE_MASK);
    chan->vportRegs->VP_IS = 0XFFFFFFFF;

    PAL_osEvtDisable((Uint32)(CSL_INTC_EVENTID_VP0_INT + chan->portNum));

    /* Configure EDMA channel again */
    retVal = _reconfigEDMAchan(chan);

    /* set status as channel stopped */
    chan->status &= ~_VPORT_STARTED;

    return retVal;
}



/**
 *  \brief   enables interrupt for capture over run error handling
 *
 *  \param   chanp  [In]  VPORT channel object pointer
 *  \param   irqId  [In]  irq ID at which interrupt occurs
 *
 *  \return  IOM_COMPLETED
 */
static Int _setCovrIsr(Ptr chanp, Int irqId)
{
    VPORT_VIntCbParams setVIntParams;

    setVIntParams.vIntCbFxn = (VPORT_IntCallBack)INVCB;

    /* capture over run interrupt enabled */
    setVIntParams.vIntMask = VPORT_INT_COVR;

    setVIntParams.irqId = irqId;

    return (_setVIntCb((Ptr)chanp, (Ptr)&setVIntParams));
}



/**
 *  \brief   ISR to handle VPORT interrupts
 *
 *  \param   portNum  [In]  to identify VPORT at which interrupt occurs
 */
static void captureISR(Int portNum)
{
    _VPORT_ChanObj  *chanObjs = portObjs[portNum].chanObj;
    Int             vpis = chanObjs->vportRegs->VP_IS;
    Int             mask = vpis;
    Int             vIntMask = 0;
    VPORTCAP_LinesCapturedInfo lineInfo;

#ifdef VPORT_INSTRUMENTATION_ENABLED
    Int             chanNum = 0;

    if ((chanObjs[0].status & _VPORT_OPENED) && (vpis & chanObjs[0].vIntMask))
    {
        chanNum = 0;
    }
    else if ((chanObjs[1].status & _VPORT_OPENED) &&
             (vpis & chanObjs[1].vIntMask))
    {
        chanNum = 1;
    }

    PSP_LOG_EVENT(&DVTEvent_Log,
                  "VPORT_CAPTURE",
                  PSP_DVT_DESC(PSP_DVT_eINT_START,
                               PSP_DVT_dCHANNEL_ID_INST,
                               PSP_DVT_dNONE,
                               PSP_DVT_dNONE),
                  CHANNELID_INSTANCEID(
                               chanNum,
                               portNum)
                  );
#endif /* VPORT_INSTRUMENTATION_ENABLED */

    if ((chanObjs[0].status & _VPORT_OPENED) &&
        (vpis & chanObjs[0].vIntMask && chanObjs[0].vIntFxn != INV))
    {
        chanObjs[0].vIntFxn(chanObjs[0].vIntCbArg, vpis);
        mask &= chanObjs[0]. vIntMask;

        /* clear interrupts that has been handled */
        chanObjs->vportRegs->VP_IS = mask;
    }
    else if ((chanObjs[1].status & _VPORT_OPENED) &&
             (vpis & chanObjs[1].vIntMask && chanObjs[1].vIntFxn != INV))
    {
        chanObjs[1].vIntFxn(chanObjs[1].vIntCbArg, (vpis >> 16));
        mask &= chanObjs[1].vIntMask;

        /* clear interrupts that has been handled */
        chanObjs->vportRegs->VP_IS = mask;
    }

    /* If Vertical interrupt has triggered then check if VportLineIntCbFxn
     * is set
     */

    if ((chanObjs[0].status & _VPORT_OPENED) &&
        ((vpis & (VPORT_INT_VINT1 | VPORT_INT_VINT2))
        && (chanObjs[0].lineIntInfo.callBackFxn != INV)))
    {
        /* Here, vIntMask will contain whether vertical interrupt is for field 1
         * or field 2
         */
        vIntMask = (vpis & (VPORT_INT_VINT1 | VPORT_INT_VINT2));

        /* Assign current frame pointer and lines captured by VPORT for current
         * field
         */
        lineInfo.frmPtr = chanObjs[0].curViop;
        lineInfo.numLines = (vIntMask & VPORT_INT_VINT1)
                             ?chanObjs[0].lineIntInfo.vIntFld1
                             :chanObjs[0].lineIntInfo.vIntFld2;

        chanObjs[0].lineIntInfo.callBackFxn(chanObjs[0].lineIntInfo.cbArg,
                                            &lineInfo);

        /* clear interrupts that has been handled */
        chanObjs->vportRegs->VP_IS = vIntMask;
    }
    else if ((chanObjs[1].status & _VPORT_OPENED) &&
             ((vpis & ((VPORT_INT_VINT1 | VPORT_INT_VINT2) << 16))
             && (chanObjs[1].lineIntInfo.callBackFxn != INV)))
    {
        /* Here, vIntMask will contain whether vertical interrupt is for field 1
         * or field 2
         */
        vIntMask = (vpis & ((VPORT_INT_VINT1 | VPORT_INT_VINT2) << 16));

        /* Assign current frame pointer and lines captured by VPORT for current
         * field
         */
        lineInfo.frmPtr = chanObjs[1].curViop;
        lineInfo.numLines = (vIntMask & (VPORT_INT_VINT1 << 16))
                             ?chanObjs[1].lineIntInfo.vIntFld1
                             :chanObjs[1].lineIntInfo.vIntFld2;

        chanObjs[1].lineIntInfo.callBackFxn(chanObjs[1].lineIntInfo.cbArg,
                                            &lineInfo);

        /* clear interrupts that has been handled */
        chanObjs->vportRegs->VP_IS = vIntMask;
    }

    if (vpis & VPORT_INT_COVR)
    {
#ifdef VPORT_DEBUG
        chanObjs[0].errCount++;
#endif  /* #ifdef VPORT_DEBUG */

        /* channel 0 capture overrun error handling */
        _covrRecover((Ptr)&chanObjs[0]);
        /* Make vpis variable to contain only Overrun interrupt bit set */
        vpis &= VPORT_INT_COVR;

        /* clear capture channel 0 overrun interrupt */
        chanObjs->vportRegs->VP_IS = vpis;
    }
    else if (vpis & (VPORT_INT_COVR << 16))
    {
#ifdef VPORT_DEBUG
        chanObjs[1].errCount++;
#endif  /* #ifdef VPORT_DEBUG */

        /* channel 1 capture overrun error handling */
        _covrRecover((Ptr)&chanObjs[1]);
        /* Make vpis variable to contain only Overrun interrupt bit set */
        vpis &= (VPORT_INT_COVR << 16);

        /* clear capture channel 1 overrun interrupt */
        chanObjs->vportRegs->VP_IS = vpis;
    }

#ifdef VPORT_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT(&DVTEvent_Log,
                  "VPORT_CAPTURE",
                  PSP_DVT_DESC(PSP_DVT_eINT_END,
                               PSP_DVT_dCHANNEL_ID_INST,
                               PSP_DVT_dNONE,
                               PSP_DVT_dNONE),
                  CHANNELID_INSTANCEID(
                               chanNum,
                               portNum)
                  );
#endif /* VPORT_INSTRUMENTATION_ENABLED */

}



/**
 *  \brief   Function to automatically synchronize capture operation
 *
 *  \param   chan  [In]  VPORT channel object pointer
 */
static void _autoSync(_VPORT_ChanObj *chan)
{
    volatile Int    fid_stat;
    volatile Int    ypos;
    volatile Uint32 capStatReg;
    Int             offset;

    /* Take the inverse of EDMA reload as _autoSync() function is called after
       update of nextEDMARlds in EDMA ISR */
    offset = (!chan->nextEDMARlds) << 1;

    capStatReg = chan->vportChanRegs->VC_STAT;
    fid_stat = (capStatReg & CSL_VPORT_VC_STAT_VCFLD_MASK)
                    >> CSL_VPORT_VC_STAT_VCFLD_SHIFT;
    ypos = (capStatReg & CSL_VPORT_VC_STAT_VC_YPOS_MASK)
                    >> CSL_VPORT_VC_STAT_VC_YPOS_SHIFT;

    if (chan->interlaced
        && (fid_stat == 0 && ypos >= chan->numTotalLinesFld1
        || fid_stat == 1 && ypos <= chan->numLinesFld1))
    {
#ifdef VPORT_DEBUG
        chan->autoSyncCount++;
#endif  /* #ifdef VPORT_DEBUG */

        /* Reconfig the EDMA to link to field 0 of the current transfer */
        chan->hEdmaChanPhyAddr[0]->linkAddr = (Uint16)
                    ((Uint32)chan->hEdmaLinkChanPhyAddr[offset] & 0xFFFF);
        chan->hEdmaChanPhyAddr[1]->linkAddr = (Uint16)
                    ((Uint32)chan->hEdmaLinkChanPhyAddr[offset + 4] & 0xFFFF);
        chan->hEdmaChanPhyAddr[2]->linkAddr = (Uint16)
                    ((Uint32)chan->hEdmaLinkChanPhyAddr[offset + 8] & 0xFFFF);
    }
}



/**
 *  \brief   Function to set VPORT line interrupt callback function
 *
 *  \param   chanp  [In]  VPORT channel object pointer
 *  \param   args   [In]  line interrupt parameter structure pointer
 *
 *  \return  IOM_COMPLETED on success
 *           IOM_EBADARGS on failure
 */
static Int _setVportLineIntCB(Ptr chanp, Ptr args)
{
    _VPORT_ChanObj      *chan = (_VPORT_ChanObj *)chanp;
    VPORTCAP_LineIntParam *lineIntCbParams =
                            (VPORTCAP_LineIntParam *)args;
    ECM_Attrs           ecmattrs = ECM_ATTRS;
    Uns                 cookie = 0;
    Int                 retVal = IOM_COMPLETED;

    /* set mask to enable video port interrupt for vInt1 and vInt2 */
    Int mask             = (VPORT_INT_VINT1 | VPORT_INT_VINT2
                          | CSL_VPORT_VP_IE_VIE_MASK);

    if ((INV == lineIntCbParams->callBackFxn)
        || (NULL == lineIntCbParams->callBackFxn)
        || (lineIntCbParams->vIntFld1 < 0)
        || (lineIntCbParams->vIntFld2 < 0))
    {
        retVal = IOM_EBADARGS;
    }
    else
    {
        /* setup vertical interrupt for both fields */
        chan->vportChanRegs->VC_VINT = VP_VCVINT_RMK(1, /* Enable field2 VINT */
                                                     0, /* Clear FSYNC bit */
                                                     lineIntCbParams->vIntFld2,
                                                     1, /* Enable field1 VINT */
                                                     lineIntCbParams->vIntFld1);

        if (1 == chan->chanNum)
        {
            mask <<= 16;    /* channel B */
        }

        PAL_osEvtDisable((Uint32)(CSL_INTC_EVENTID_VP0_INT + chan->portNum));
        /* Enable the interrupt */
        ecmattrs.unmask = 1u;

        /* setup arg such that isr will get 'chan' as arg */
        ecmattrs.arg = (Arg)chan->portNum;

        /* Disabling the interrupts */
        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

        /* Plug the ISR with interrupt (event) number */
        ECM_dispatchPlug((Uns)(CSL_INTC_EVENTID_VP0_INT + chan->portNum),
                         (ECM_Fxn)captureISR,
                         &ecmattrs);
        /* Enabling the HWI_ID where corresponding ECM group is configured */
        C64_enableIER(1 << _IRQ_ID);

        /* Enabling the interrupts */
        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

        chan->vportRegs->VP_IE |= mask;
        /* Store Line Interrupt information in chanObj structure */
        chan->lineIntInfo.callBackFxn       = lineIntCbParams->callBackFxn;
        chan->lineIntInfo.cbArg             = lineIntCbParams->cbArg;
        chan->lineIntInfo.vIntFld1          = lineIntCbParams->vIntFld1;
        chan->lineIntInfo.vIntFld2          = lineIntCbParams->vIntFld2;
        chan->lineIntInfo.errCheckEnable    = lineIntCbParams->errCheckEnable;
    }

    return retVal;
}



/**
 *  \brief   Function to get number of lines transferred by EDMA
 *
 *  \param   chanp  [In]  VPORT channel object pointer
 *  \param   args   [In]  VPORTCAP_LinesCapturedInfo structure pointer
 *
 *  \return  IOM_COMPLETED on success
 *           IOM_EABORT on failure
 */
static Int _getLinesCapturedEDMA(Ptr chanp, Ptr args)
{
    _VPORT_ChanObj      *chan = (_VPORT_ChanObj *)chanp;
    VPORTCAP_LinesCapturedInfo *lineInfoPtr = (VPORTCAP_LinesCapturedInfo*)args;
    Uint32              optField = 0;
    Uint16              lineCnt = 0;
    Int                 retVal = IOM_COMPLETED;
    Uns                 cookie = 0;

    /* Disable interrupts */
    PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

    lineInfoPtr->frmPtr = chan->curViop;

    /* Enable interrupts */
    PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

    /* Read Ccnt to get number of lines transferred */
    lineCnt = chan->hEdmaChanPhyAddr[0]->cCnt;

    /* Read whether tcc interrupt is enabled or not */
    optField = chan->hEdmaChanPhyAddr[0]->opt;

    lineInfoPtr->numLines = chan->numLines - lineCnt;

    /* If tccIntEn is disabled then, it indicates that EDMA transfer for
     * first field is going on. So, subtract number of lines of field2 from
     * lineInfoPtr->numLines
     */
    if (!(optField & (1 << EDMA_TCINTEN_SHIFT)))
    {
        lineInfoPtr->numLines = (lineInfoPtr->numLines -
                                 (chan->numLines - chan->numLinesFld1));
    }

    return retVal;
}


/******************************************************************************/
/* End of file                                                                */
/******************************************************************************/
