/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2007 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \file vportdis.c file
 *
 *  This source file contains code for VPORT display driver
 */

/*
 * ========= DSP/BIOS standard include files =========
 */
#include <std.h>

/*
 * ========= Chip-support library include files ===========
 */
#include <ti/sdo/pspdrivers/soc/dm648/dsp/soc.h>
#include <ti/sdo/pspdrivers/soc/dm648/dsp/cslr_vport.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osprotect.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osCache.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osmem.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osInt.h>
#include <ecm.h>

/*
 * ======== video driver specific include files ===========
 */
#include <ti/sdo/pspdrivers/drivers/vport/vport.h>
#include <ti/sdo/pspdrivers/drivers/vport/vportdis.h>
#include <ti/sdo/edma3/drv/edma3_drv.h>
#include <ti/sdo/pspdrivers/drivers/vport/src/_vportrmk.h>
#include <ti/sdo/pspdrivers/drivers/vport/src/_vport.h>

/*
 * =========== debug include files ==========
 */
/* to minimize code size and cycle count overhead of the driver
 * error checking is only performed at debug time
 */
#include <assert.h>

#ifdef VPORT_INSTRUMENTATION_ENABLED
#include <ti/sdo/pspdrivers/pal_os/bios/psp_log.h>
#endif /* VPORT_INSTRUMENTATION_ENABLED */

/*
 * ============== mini-driver API functions ===============
 */
static Int mdBindDev(Ptr *devp, Int devid, Ptr devParams);
static Int mdControlChan(Ptr chanp, Uns cmd, Ptr args);
static Int mdCreateChan(Ptr *chanp,
                        Ptr devp,
                        String name,
                        Int mode,
                        Ptr chanParams,
                        IOM_TiomCallback cbFxn,
                        Ptr cbArg);
static Int mdDeleteChan(Ptr chanp);
static Int mdSubmitChan(Ptr chanp, IOM_Packet *packet);

/*
 * =============== local functions =============
 */
static Int _configAsyncMode(Ptr chanp, Uns cmd, Ptr args);
static Int _configChan(Ptr chanp, Ptr args);
static Int _configEDMA(Ptr chanp);
static Int _configPort(Ptr chanp, Ptr args);
static Int _configRegs(Ptr chanp, VPORTDIS_Params *params);
static void displayISR(Int portNum);
static Int _startDisplay(Ptr chanp);
static Int _stopDisplay(Ptr chanp);
static Int _dundRecover(Ptr chanp);
static Int _frmAlmostCompleteIsr(Int portNum);
static Int _getlastActiveLine(VPORTDIS_Params *params);
static Int _getNumPendingIORqsts(Ptr chanp, Ptr args);
static Int _setVIntCb(Ptr chanp, Ptr args);
static Int _allocBuffer(_VPORT_ChanObj *chanp, FVID_Frame **newFrame);
static Int _freeBuffer(_VPORT_ChanObj *chanp, FVID_Frame **freeFrame);
static Int _setDundIsr(Ptr chanp, Int irqId);
static void _resetDisp(Ptr chanp);
static Int _allocAssignBuffer(_VPORT_ChanObj *, FVID_Frame *);
static Int _cmdQueue(_VPORT_ChanObj *, IOM_Packet *);
static Int _cmdDequeue(_VPORT_ChanObj *, IOM_Packet *);
static Int _getChanParams(Ptr, VPORT_CurrParams *);
static Int _cmdGetBufs(_VPORT_ChanObj *, IOM_Packet *);
static Int _reconfigEDMAchan(Ptr);
static void displayEdmaISR(unsigned int tcc,
                           EDMA3_RM_TccStatus status,
                           void *appData);
static Bool _isItTooLateInISR(_VPORT_ChanObj *chan);
static Int _updateEDMAParamAddress(const _VPORT_ChanObj *chan, Int offset,
                                   const FVID_Frame *viop);

/*
 * ============= global and static variables ===========
 */
/** \brief default value to be filled in display buffers */
static const Int fillValue[3] =
{
    0x10, 0x80, 0x80
};

/** \brief VPORT display function pointer table */
IOM_Fxns VPORTDIS_Fxns =
{
    mdBindDev,
    (IOM_TmdUnBindDev)IOM_mdNotImpl,
    mdControlChan,
    mdCreateChan,
    mdDeleteChan,
    mdSubmitChan
};

/**
 *  \brief Static allocation and initialization of port objects as display
 *  supports only 1 channel, the same channel object for capture is used here
 *  for the whole port
 */
#ifndef VPORT_DEBUG
static
#endif  /* #ifndef VPORT_DEBUG */
_VPORT_ChanObj chanObjs[CSL_VIDEOPORT_PER_CNT] =
{
    {0, 0, 0, (CSL_VportRegs*)CSL_VIDEOPORT_0_REGS, CSL_EDMA3_CHA_VP0EVTYA,
        CSL_EDMA3_CHA_VP0EVTUA, CSL_EDMA3_CHA_VP0EVTVA,
        0, 0, 0, CSL_VIDEOPORT0_CHANNELA_DATA,
        EDMA_CHA_VP0EVTQA},
    {0, 1, 0, (CSL_VportRegs*)CSL_VIDEOPORT_1_REGS, CSL_EDMA3_CHA_VP1EVTYA,
        CSL_EDMA3_CHA_VP1EVTUA, CSL_EDMA3_CHA_VP1EVTVA,
        0, 0, 0, CSL_VIDEOPORT1_CHANNELA_DATA,
        EDMA_CHA_VP1EVTQA},
    {0, 2, 0, (CSL_VportRegs*)CSL_VIDEOPORT_2_REGS, CSL_EDMA3_CHA_VP2EVTYA,
        CSL_EDMA3_CHA_VP2EVTUA, CSL_EDMA3_CHA_VP2EVTVA,
        0, 0, 0, CSL_VIDEOPORT2_CHANNELA_DATA,
        EDMA_CHA_VP2EVTQA},
    {0, 3, 0, (CSL_VportRegs*)CSL_VIDEOPORT_3_REGS, CSL_EDMA3_CHA_VP3EVTYA,
        CSL_EDMA3_CHA_VP3EVTUA, CSL_EDMA3_CHA_VP3EVTVA,
        0, 0, 0, CSL_VIDEOPORT3_CHANNELA_DATA,
        EDMA_CHA_VP3EVTQA},
    {0, 4, 0, (CSL_VportRegs*)CSL_VIDEOPORT_4_REGS, CSL_EDMA3_CHA_VP4EVTYA,
        CSL_EDMA3_CHA_VP4EVTUA, CSL_EDMA3_CHA_VP4EVTVA,
        0, 0, 0, CSL_VIDEOPORT4_CHANNELA_DATA,
        EDMA_CHA_VP4EVTQA},
};


/**
 *  \brief Initialize a given VPORT driver (instance)
 *
 *  Register all external devices to video port display driver.
 *
 *  \param  devp        [IN]    Void pointer passed by the GIO Layer to
 *                              store the address of the port structure
 *
 *  \param  devid       [IN]    VPSS instance number
 *
 *  \param  devParams   [IN]    Initialization configuration structure
 *
 *  \return success             IOM_COMPLETED
 *          failure             IOM Error
 */
static Int mdBindDev(Ptr *devp, Int devid, Ptr devParams)
{
    Int portNum = devid;
    Int retVal = IOM_COMPLETED;

    assert(portNum < CSL_VIDEOPORT_PER_CNT);

    /* Assign port object pointer to devp */
    *devp = &chanObjs[portNum];

    if (NULL == devParams)
    {
        retVal = IOM_EBADARGS;
    }
    else
    {
        /* Call _configPort() to configure video port */
        _configPort(&chanObjs[portNum], devParams);
    }

    return (retVal);
}



/**
 *  \brief VPORT Ioctls
 *
 *  This function provides ioctl functionality for VPORT
 *
 *  \param  chanp       [IN]     VPORT Driver Channel Handle
 *  \param  cmd         [IN]     Operation to be performed, typically an enum
 *                               gets passed
 *  \param  args        [IN/OUT] Provides additional information related to the
 *                               operation
 *
 *  \return success              IOM_COMPLETED
 *          failure              Suitable error code
 */
static Int mdControlChan(Ptr chanp, Uns cmd, Ptr args)
{
    Int             retVal = IOM_COMPLETED;
    _VPORT_ChanObj  *chan = (_VPORT_ChanObj *)chanp;

    if (NULL == chanp)
    {
        retVal = IOM_EBADARGS;
    }
    else
    {
        /* command dispatcher */
        switch (cmd)
        {
            case VPORT_CMD_START:
                /* For NORMAL MODE, If EDMA is not configured then return ERROR
                 */
                if ((!(_VPORT_READY & chan->status))
                    || (chan->status & _VPORT_STARTED))
                {
                    retVal = IOM_EBADIO;
                }
                else
                {
                    retVal = _startDisplay(chanp);
                }
                break;

            case VPORT_CMD_STOP:
                if (!(chan->status & _VPORT_STARTED))
                {
                    retVal = IOM_EBADIO;
                }
                else
                {
                    retVal = _stopDisplay(chanp);
                }
                break;

            case VPORT_CMD_CONFIG_CHAN:
                if (NULL == args)
                {
                    retVal = IOM_EBADARGS;
                }
                else
                {
                    retVal = _configChan(chanp, args);
                }
                break;

            case VPORT_CMD_SET_VINTCB:
                if (NULL == args)
                {
                    retVal = IOM_EBADARGS;
                }
                else
                {
                    retVal = _setVIntCb(chanp, args);
                }
                break;

            case VPORT_CMD_DUND_RECOVER:
                if (!(_VPORT_READY & chan->status))
                {
                    retVal = IOM_EBADIO;
                }
                else
                {
                    retVal = _dundRecover(chanp);
                }
                break;

            case VPORTDIS_CMD_ASYNC_MODE_ENABLE:
                if (NULL == args)
                {
                    retVal = IOM_EBADARGS;
                    break;
                }
            case VPORTDIS_CMD_ASYNC_MODE_DISABLE:
            case VPORTDIS_CMD_ASYNC_MODE_RESET_FRAMECT:
                if (!(_VPORT_READY & chan->status))
                {
                    retVal = IOM_EBADIO;
                }
                else
                {
                    retVal = _configAsyncMode(chanp, cmd, args);
                }
                break;

            case VPORT_CMD_GET_NUM_IORQST_PENDING:
                if (NULL == args)
                {
                    retVal = IOM_EBADARGS;
                }
                else if (!(_VPORT_READY & chan->status))
                {
                    retVal = IOM_EBADIO;
                }
                else
                {
                    retVal = _getNumPendingIORqsts(chanp, args);
                }
                break;

            case VPORT_CMD_GET_PARAMS:
                if (NULL == args)
                {
                    retVal = IOM_EBADARGS;
                }
                else
                {
                    retVal = _getChanParams(chanp, (VPORT_CurrParams *)args);
                }
                break;

            case FVID_ALLOC_BUFFER:
                /* This IOCTL is not supported in LEGACY MODE */
                if (LEGACY_MODE == chan->driverMode)
                {
                    retVal = IOM_ENOTIMPL;
                }
                else if (NULL == args)
                {
                    retVal = IOM_EBADARGS;
                }
                else
                {
                    /* function call for allocating buffer in normal mode */
                    retVal = _allocBuffer(chanp, args);
                }
                break;

            case FVID_FREE_BUFFER:
                /* This IOCTL is not supported in LEGACY MODE */
                if (LEGACY_MODE == chan->driverMode)
                {
                    retVal = IOM_ENOTIMPL;
                }
                else if (NULL == args)
                {
                    retVal = IOM_EBADARGS;
                }
                else
                {
                    /* function call to free the allocated buffer */
                    retVal = _freeBuffer(chanp, args);
                }
                break;

            default:
                if (chan->edcFxns != INV)
                {
                    /* Call EDC control function for EDC configuration */
                    retVal = chan->edcFxns->ctrl(chan->edcHandle,
                                                 cmd - VPORT_CMD_EDC_BASE,
                                                 (Arg)args);
                }
                else
                {
                    retVal = IOM_ENOTIMPL;
                }
                break;
        }
    }

    return retVal;
}



/**
 *  \brief Open Channel of the VPORT device
 *
 *  The function prepares channel of the driver instance for data transfers
 *  and returns an handle to the driver instance.
 *
 *  \param chanp        [IN]     pointer to the channel structure
 *  \param devp         [IN/OUT] pointer to the port structure
 *  \param name         [IN]     name of the driver
 *  \param mode         [IN]     mode of the channel i.e. INPUT or OUTPUT
 *  \param chanParams   [IN/OUT] Extra parameter (Not used in this driver)
 *  \param cbFxn        [IN]     callback to the GIO Layer
 *  \param cbArg        [IN]     arguments for the callback
 *
 *  \return                     On success returns IOM_COMPLETED
 *                              On failure returns suitable error code
 */
static Int mdCreateChan(Ptr *chanp,
                        Ptr devp,
                        String name,
                        Int mode,
                        Ptr chanParams,
                        IOM_TiomCallback cbFxn,
                        Ptr cbArg)
{
    _VPORT_ChanObj          *chan;
    Int                     retVal = IOM_COMPLETED;
    Int                     j;
    CSL_VportChannelARegs   *vportDstRegs;
    unsigned int            linkTccId = 0;
    EDMA3_DRV_Result        result = EDMA3_DRV_SOK;

    /* For display, mode must be IOM_OUTPUT */
    if (mode != IOM_OUTPUT)
    {
        return IOM_EBADARGS;
    }

    /* Return error if any pointer parameter is NULL */
    if ((INV == chanParams) || (NULL == chanParams) || (NULL == cbFxn) ||
        (NULL == chanp) || (NULL == devp) ||
        (NULL == ((VPORTDIS_Params *)chanParams)->hEdma))
    {
        retVal = IOM_EBADARGS;
    }

    if (IOM_COMPLETED == retVal)
    {
        chan = (_VPORT_ChanObj *)devp;
        if (chan->edcFxns != INV)
        {
            /* open external device */
            chan->edcHandle = chan->edcFxns->open(name, (Arg)INV);
            if (NULL == chan->edcHandle)
            {
                retVal = IOM_EBADIO;
            }
        }

        if ((!(chan->status & _VPORT_OPENED)) && (IOM_COMPLETED == retVal))
        {
#ifdef VPORT_DEBUG
            chan->submitCount = 0;
            chan->errCount = 0;
            chan->frameCount = 0;
            chan->shortFldDetectCount = 0;
            chan->autoSyncCount = 0;
            chan->isrSwapCount = 0;
            chan->isrSwapErrCount = 0;
            chan->reconfigErrCount = 0;

            chan->tooLateInISRCount[0] = 0;
            chan->tooLateInISRCount[1] = 0;
            chan->tooLateInISRCount[2] = 0;
            chan->tooLateInISRCount[3] = 0;
            chan->tooLateInISRCount[4] = 0;
#endif  /* #ifdef VPORT_DEBUG */

            chan->status |= _VPORT_OPENED;
            chan->vIntMask = 0;
            chan->cbFxn = cbFxn;
            chan->vIntFxn = (VPORT_IntCallBack)INVCB;
            chan->queEmpty = FALSE;
            chan->cbArg = (Arg)cbArg;
            chan->packetIOM = INV;
            chan->vIntCbArg = (Int)INV;
            chan->bufSz = 0;
            chan->asyncModeEnable = 0;   /* disable async mode(call-back mode)
                                          * be default
                                          */
            /* disable 3:2 pull-down */
            chan->pullDownMode = VPORTDIS_PULLDOWN_DISABLE;
            chan->asyncCallBackFxn = (Fxn)INVCB;
            chan->startFlag = 0;

            /* Assign EDMA handle passed to global channel structure variable */
            chan->hEdma = ((VPORTDIS_Params *)chanParams)->hEdma;

            /* assigning VPORT FIFO destination base address to channel A
             * register structure that can be used to calculate Y, Cb and Cr
             * FIFO addresses
             */
            vportDstRegs = (CSL_VportChannelARegs *)(chan->vpFifoAddr);
            chan->edmaAddr[0] = (Uns)&(vportDstRegs->YDSTA);
            chan->edmaAddr[1] = (Uns)&(vportDstRegs->CBDST);
            chan->edmaAddr[2] = (Uns)&(vportDstRegs->CRDST);

            /* Request EDMA channels and param tables for Y, Cb, Cr transfers */
            for (j = 0;
                    (j < _VPORT_NUM_EDMA_CHANS) && (IOM_COMPLETED == retVal);
                    j++)
            {
                chan->edmaLinkChanNum[4 * j] = EDMA3_DRV_LINK_CHANNEL;
                chan->edmaLinkChanNum[4 * j + 1] = EDMA3_DRV_LINK_CHANNEL;
                chan->edmaLinkChanNum[4 * j + 2] = EDMA3_DRV_LINK_CHANNEL;
                chan->edmaLinkChanNum[4 * j + 3] = EDMA3_DRV_LINK_CHANNEL;

                chan->tcc[j] = chan->hEdmaChan[j] = chan->edmaChanNum[j];
                linkTccId = EDMA3_DRV_TCC_ANY;

                if ((EDMA3_DRV_requestChannel(chan->hEdma,
                                              &chan->hEdmaChan[j],
                                              &chan->tcc[j],
                                              chan->evtQueue,
                                              (EDMA3_RM_TccCallback)&
                                              displayEdmaISR,
                                              NULL)) != EDMA3_DRV_SOK ||
                    (EDMA3_DRV_requestChannel(chan->hEdma,
                                              &chan->edmaLinkChanNum[4 * j],
                                              &linkTccId,
                                              chan->evtQueue,
                                              (EDMA3_RM_TccCallback)&
                                              displayEdmaISR,
                                              NULL)) != EDMA3_DRV_SOK ||
                    (EDMA3_DRV_requestChannel(chan->hEdma,
                                              &chan->edmaLinkChanNum[4 * j + 1],
                                              &linkTccId,
                                              chan->evtQueue,
                                              (EDMA3_RM_TccCallback)&
                                              displayEdmaISR,
                                              NULL)) != EDMA3_DRV_SOK ||
                    (EDMA3_DRV_requestChannel(chan->hEdma,
                                              &chan->edmaLinkChanNum[4 * j + 2],
                                              &linkTccId,
                                              chan->evtQueue,
                                              (EDMA3_RM_TccCallback)&
                                              displayEdmaISR,
                                              NULL)) != EDMA3_DRV_SOK ||
                    (EDMA3_DRV_requestChannel(chan->hEdma,
                                              &chan->edmaLinkChanNum[4 * j + 3],
                                              &linkTccId,
                                              chan->evtQueue,
                                              (EDMA3_RM_TccCallback)&
                                              displayEdmaISR,
                                              NULL)) != EDMA3_DRV_SOK)
                {
                    retVal = IOM_EALLOC;
                } /* if ((port->... */

                /* Get the physical address of all the EDMA channels */
                result |= EDMA3_DRV_getPaRAMPhyAddr(
                        chan->hEdma,
                        chan->hEdmaChan[j],
                        (unsigned int *)&chan->hEdmaChanPhyAddr[j]);
                result |= EDMA3_DRV_getPaRAMPhyAddr(
                        chan->hEdma,
                        chan->edmaLinkChanNum[4 * j],
                        (unsigned int *)&chan->hEdmaLinkChanPhyAddr[4 * j]);
                result |= EDMA3_DRV_getPaRAMPhyAddr(
                        chan->hEdma,
                        chan->edmaLinkChanNum[4 * j + 1],
                        (unsigned int *)&chan->hEdmaLinkChanPhyAddr[4 * j + 1]);
                result |= EDMA3_DRV_getPaRAMPhyAddr(
                        chan->hEdma,
                        chan->edmaLinkChanNum[4 * j + 2],
                        (unsigned int *)&chan->hEdmaLinkChanPhyAddr[4 * j + 2]);
                result |= EDMA3_DRV_getPaRAMPhyAddr(
                        chan->hEdma,
                        chan->edmaLinkChanNum[4 * j + 3],
                        (unsigned int *)&chan->hEdmaLinkChanPhyAddr[4 * j + 3]);

                if (result != EDMA3_DRV_SOK)
                {
                    retVal = IOM_EALLOC;
                }
            } /* for (j = 0; j < NUM_EDMA_CHANS; j++) ... */

            /* if EDMA resources are allocated successfully, go ahead
             * and configure the channel
             */
            if (IOM_COMPLETED == retVal)
            {
                /* decide driver operation mode from number of buffers passed
                 * in disparams
                 */
                if (0 == ((VPORTDIS_Params *)chanParams)->numFrmBufs)
                {
                    chan->driverMode = NORMAL_MODE;  /* normal mode */
                }
                else
                {
                    chan->driverMode = LEGACY_MODE; /* legacy mode(DM642 mode)*/
                }

                retVal = mdControlChan(chan, VPORT_CMD_CONFIG_CHAN, chanParams);
            }
            if (IOM_COMPLETED == retVal)
            {
                /* enabling display under run error handling */
                retVal = _setDundIsr((Ptr)chan, _IRQ_ID);
            }

            if (IOM_COMPLETED == retVal)
            {
                /* configured the channel successfully, return handle */
                *chanp = chan;
            }
            else
            {
                /* failed, free all resources and return error */
                mdDeleteChan(chan);
                *chanp = INV;
            }
        } /* if (!chan->opened) */
    } /* if (IOM_COMPLETED == retVal) */

    return retVal;
}



/**
 *  \brief Close Channel of the VPORT device
 *
 *  This function is used to close Channel handle for individual VPORT
 *  Modules.
 *
 *  \param  chanp       [IN]    VPORT Driver Channel Handle
 *
 *  \return                     On success returns IOM_COMPLETED
 *                              On failure returns suitable error code
 */
static Int mdDeleteChan(Ptr chanp)
{
    _VPORT_ChanObj      *chan = (_VPORT_ChanObj *)chanp;
    Int                 j;
    Int                 retVal = IOM_COMPLETED;
    EDMA3_DRV_Result    result = EDMA3_DRV_SOK;

    if (NULL == chan)
    {
        retVal = IOM_EBADARGS;
    }
    else
    {
        if (chan->status & _VPORT_OPENED)
        {
            if (chan->status & _VPORT_STARTED)
            {
                /* Stop display operation first */
                retVal = mdControlChan(chanp, VPORT_CMD_STOP, NULL);
            }

            if (IOM_COMPLETED == retVal)
            {
                for (j = 0; j < _VPORT_NUM_EDMA_CHANS; j++)
                {
                    /* Disable Transfer */
                    result = EDMA3_DRV_disableTransfer(chan->hEdma,
                                                   chan->hEdmaChan[j],
                                                   EDMA3_DRV_TRIG_MODE_EVENT);
                    /* Free Channels and PaRam Tables */
                    result |= EDMA3_DRV_freeChannel(chan->hEdma,
                                            chan->hEdmaChan[j]);
                    result |= EDMA3_DRV_freeChannel(chan->hEdma,
                                            chan->edmaLinkChanNum[4 * j]);
                    result |= EDMA3_DRV_freeChannel(chan->hEdma,
                                            chan->edmaLinkChanNum[4 * j + 1]);
                    result |= EDMA3_DRV_freeChannel(chan->hEdma,
                                            chan->edmaLinkChanNum[4 * j + 2]);
                    result |= EDMA3_DRV_freeChannel(chan->hEdma,
                                            chan->edmaLinkChanNum[4 * j + 3]);

                    /* Reset the physical addresses of all EDMA channels */
                    chan->hEdmaChanPhyAddr[j] = NULL;
                    chan->hEdmaLinkChanPhyAddr[4 * j] = NULL;
                    chan->hEdmaLinkChanPhyAddr[4 * j + 1] = NULL;
                    chan->hEdmaLinkChanPhyAddr[4 * j + 2] = NULL;
                    chan->hEdmaLinkChanPhyAddr[4 * j + 3] = NULL;
                }
                if (EDMA3_DRV_SOK != result)
                {
                    retVal = IOM_EFREE;
                }

                if (_VPORT_BUF_ALLOCATED & chan->status)
                {
                    /* If buffers are allocated then free them, This condition
                     * will be TRUE only in LEGACY MODE
                     */
                    for (j = 0; j < chan->numFrms; j++)
                    {
                        if (PAL_SOK != PAL_osMemFree(chan->segId,
                                                 chan->viops[j].frame.iFrm.y1,
                                                 chan->bufSz))
                        {
                            retVal = IOM_EFREE;
                        }
                    }
                }

                /* reset display */
                _resetDisp(chan);

                chan->status = 0;

                /* close external device */
                if (chan->edcFxns != INV)
                {
                    if (EDC_FAILED == chan->edcFxns->close(chan->edcHandle))
                    {
                        retVal = IOM_EBADIO;
                    }
                }
            } /* if (IOM_COMPLETED == retVal) */
        } /* if (chan->status & _VPORT_OPENED) */
    }

    return retVal;
}



/**
 *  \brief submit IO Request
 *
 *  This function is called to Queue/Dequeue Frame buffer.
 *
 *  \param  chanp      [IN]    VPORT Driver Channel Handle
 *  \param  packet     [IN/OUT] IOM packet passed by the upper layer containing
 *                              information about the size,address,timeout and
 *                              other necessary parameters
 *
 *  \return  On success, returns positive number - depending upon the operation
 *           requested
 *           If failed, returns negative number - suitable error code
 */
static Int mdSubmitChan(Ptr chanp, IOM_Packet *packet)
{
    _VPORT_ChanObj  *chan = (_VPORT_ChanObj *)chanp;
    Int             retVal = IOM_PENDING;
    Uint32          gie;

    PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &gie);

#ifdef VPORT_DEBUG
    chan->submitCount++;
#endif  /* #ifdef VPORT_DEBUG */

    if ((NULL == chan) || (NULL == packet))
    {
        retVal = IOM_EBADARGS;
    }
    else
    {
#ifdef VPORT_INSTRUMENTATION_ENABLED
        FVID_Frame      *tempViop = NULL;

        if (NULL != packet->addr)
        {
            tempViop = (*(void**)packet->addr);
        }

        PSP_LOG_EVENT(&DVTEvent_Log,
                      "VPORT_DISPLAY",
                      PSP_DVT_DESC(PSP_DVT_eFUNC_START,
                                   PSP_DVT_dCHANNEL_ID_INST,
                                   PSP_DVT_dPACKET_ID,
                                   PSP_DVT_dSIZE_BYTES),
                      CHANNELID_INSTANCEID(
                                   chan->chanNum,
                                   chan->portNum),
                      tempViop,
                      chan->bufSz
                      );
#endif /* VPORT_INSTRUMENTATION_ENABLED */

        switch (packet->cmd)
        {
            case IOM_ABORT:
                /* clear all pending I/O requests */
                while (!QUE_empty(&chan->qIn))
                {
                    QUE_enqueue(&chan->qOut, QUE_dequeue(&chan->qIn));
                }
                retVal = IOM_ABORTED;
                break;

            case FVID_FREE:
                /* This command is not supported in NORMAL MODE */
                if (LEGACY_MODE != chan->driverMode)
                {
                    retVal = IOM_ENOTIMPL;
                }
                else
                {
                    /* function call to queue the buffer to driver */
                    retVal = _cmdQueue(chan, packet);
                }
                break;

            case FVID_QUEUE:
                /* This command is not supported in LEGACY MODE */
                if (LEGACY_MODE == chan->driverMode)
                {
                    retVal = IOM_ENOTIMPL;
                }
                else
                {
                    /* function call to queue the buffer to driver */
                    retVal = _cmdQueue(chan, packet);
                }
                break;

            case FVID_ALLOC:
                /* This command is not supported in NORMAL MODE */
                if (LEGACY_MODE != chan->driverMode)
                {
                    retVal = IOM_ENOTIMPL;
                }
                else if (chan->packetIOM == INV)
                {
                    /* function call to get the buffer from driver, only when
                     * there is no outstanding pending request
                     */
                    retVal = _cmdDequeue(chan, packet);
                }
                else
                {
                    /* retutn IOM_EINUSE if there is any outstanding pending
                     * request
                     */
                    retVal = IOM_EINUSE;
                }
                break;

            case FVID_DEQUEUE:
                /* This command is not supported in LEGACY MODE */
                if (LEGACY_MODE == chan->driverMode)
                {
                    retVal = IOM_ENOTIMPL;
                }
                else if (!(chan->status & _VPORT_STARTED))
                {
                    /* If display is stopped then return all bufs one-by-one */
                    retVal = _cmdGetBufs(chan, packet);
                }
                else if (chan->packetIOM == INV)
                {
                    /* If display is started then return empty buffer to
                     * application. If buffer is not available, then return
                     * IOM_PENDING
                     */
                    retVal = _cmdDequeue(chan, packet);
                }
                else
                {
                    /* retutn IOM_EINUSE if there is any outstanding pending
                     * request
                     */
                    retVal = IOM_EINUSE;
                }
                break;

            case FVID_EXCHANGE:
                if (chan->packetIOM == INV)
                {
                    /* only when there is no outstanding pending request */
                    retVal = _cmdQueue(chan, packet);
                    if (IOM_COMPLETED == retVal)
                    {
                        retVal = _cmdDequeue(chan, packet);
                    }
                }
                else
                {
                    /* return IOM_EINUSE if there is any outstanding pending
                     * request
                     */
                    retVal = IOM_EINUSE;
                }
                break;

            default:
                retVal = IOM_ENOTIMPL;
                break;
        }
#ifdef VPORT_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(&DVTEvent_Log,
                      "VPORT_DISPLAY",
                      PSP_DVT_DESC(PSP_DVT_eFUNC_END,
                                   PSP_DVT_dCHANNEL_ID_INST,
                                   PSP_DVT_dPACKET_ID,
                                   PSP_DVT_dSIZE_BYTES),
                      CHANNELID_INSTANCEID(
                                   chan->chanNum,
                                   chan->portNum),
                      tempViop,
                      0
                      );
#endif /* VPORT_INSTRUMENTATION_ENABLED */
    }

    PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, gie);

    return retVal;
}



/**
 *  \brief submit the filled buffer to driver
 *
 *  \param  chan      [In] VPORT channel object pointer
 *  \param  packet    [In] video IO packet
 *
 *  \return  IOM_COMPLETED on success
 *           IOM_EALLOC on failure
 */
static Int _cmdQueue(_VPORT_ChanObj *chan, IOM_Packet *packet)
{
    FVID_Frame          *viop = NULL;
    Int                 retVal = IOM_COMPLETED;
    Int                 offset;

    if (NULL == *(void**)packet->addr)
    {
        retVal = IOM_EBADARGS;
    }
    else
    {
        /* FVID_FREE or FVID_EXCHANGE or FVID_QUEUE */
        if (chan->asyncModeEnable &&
                ((LEGACY_MODE == chan->driverMode) ||
                    ((NORMAL_MODE == chan->driverMode) &&
                        (_VPORT_STARTED & chan->status))))
        {
            /* For async mode in LEGACY Mode, FVID_free and FVID_exchange is
             * not supported. In NORMAL Mode, these commands are not supported
             * after display is started. Till driver starts we have to allow
             * this to queue buffers in normal mode.
             */
            retVal = IOM_EINUSE;
        }
        else
        {
            /* pointer of a video I/O packet */
            viop = *(void**)packet->addr;

            if (NORMAL_MODE == chan->driverMode)
            {
                /* increment number of buffers available with driver */
                chan->numQueBuf++;
            }

            if (!(_VPORT_READY & chan->status))
            {
                /* maintain the count for calling _configEDMA after
                 * minimum number of required queued buffers
                 */
                QUE_enqueue(&chan->qIn, (QUE_Handle)viop);

                chan->queEmpty = FALSE;

                /* _configEDMA can only be called after minimum required
                 * number of buffers are queued into the buffer queue
                 */
                if (_VPORT_MIN_BUF == chan->numQueBuf)
                {
                    retVal = _configEDMA(chan);
                }
                retVal = packet->status = IOM_COMPLETED;
            }
            else
            {
                if (chan->queEmpty)
                {
                    /* check to make sure it is not to at the end of a
                     * frame to avoid any race condition
                     */
                    short lineNum = (chan->vportRegs->VD_STAT
                                     & CSL_VPORT_VD_STAT_VD_YPOS_MASK)
                                     >> CSL_VPORT_VD_STAT_VD_YPOS_SHIFT;
                    /* make sure we are at least five lines before
                     * completion to prevent EDMA reload happening while
                     * it is being modified
                     */
                    if ((lineNum <=
                        (chan->lastLineNum - NOT_TOO_LATE_LINE_ROOM_SUBMIT)) &&
                            (chan->status & _VPORT_STARTED))
                    {
                        offset = chan->nextEDMARlds << 1;
                        chan->nextViop = viop;

                        /* Update the EDMA reload entry */
                        retVal = _updateEDMAParamAddress(chan, offset, viop);
                    }
                    else
                    {
                        QUE_enqueue(&chan->qIn, (QUE_Handle)viop);
                    }
                    chan->queEmpty = FALSE;
                }
                else
                {
                    QUE_enqueue(&chan->qIn, (QUE_Handle)viop);
                }
                retVal = packet->status = IOM_COMPLETED;
            } /* else _VPORT_READY */
        }
    }

    return retVal;
}



/**
 *  \brief provide the empty buffer to application application
 *
 *  \param  chan      [In] VPORT channel object pointer
 *  \param  packet    [OUT] video IO packet
 *
 *  \return  IOM_COMPLETED on success
 *           IOM_PENDING if buffer is not available
 *           suitable error code on failure
 */
static Int _cmdDequeue(_VPORT_ChanObj *chan, IOM_Packet *packet)
{
    Int retVal = IOM_COMPLETED;
    FVID_Frame *viop = NULL;

    if ((viop = (FVID_Frame *)QUE_dequeue(&chan->qOut)) != (FVID_Frame *)&
        chan->qOut)
    {
        /* only when there is no outstanding pending request */
        *(void**)packet->addr = (void*)viop;
        packet->size = sizeof(FVID_Frame);
        retVal = packet->status = IOM_COMPLETED;

        if (NORMAL_MODE == chan->driverMode)
        {
            /* decrement number of buffers available with driver */
            chan->numQueBuf--;
        }
    }
    else
    {
        chan->packetIOM = packet;
        retVal = packet->status = IOM_PENDING;
    }

    return retVal;
}



/**
 *  \brief Return all buffers to application
 *
 *  This function returns all buffers to application held by driver in
 *  normal mode. The buffers may not contain proper data
 *
 *  \param  chan    [In] VPORT channel object pointer
 *  \param  packet  [Out] video IO packet
 *
 *  \return  IOM_COMPLETED on success
 *           IOM_ENOPACKETS if buffer is not available
 */
static Int _cmdGetBufs(_VPORT_ChanObj *chan, IOM_Packet *packet)
{
    Int retVal = IOM_COMPLETED;
    FVID_Frame *viop = NULL;

    if (0 == chan->numQueBuf)
    {
        chan->queEmpty = TRUE;
        retVal = IOM_ENOPACKETS;
    }
    else
    {
        /* Take buffer from qOut if it is not empty */
        viop = (FVID_Frame *)QUE_dequeue(&chan->qOut);

        if (viop == (FVID_Frame *)&chan->qOut)
        {
            /* Take buffer from qIn if qOut is empty */
            viop = (FVID_Frame *)QUE_dequeue(&chan->qIn);
        }
        if ((viop == (FVID_Frame *)&chan->qIn) && (chan->nextViop != INV))
        {
            /* If qIn is empty then return buffers from viop pointers */
            chan->queEmpty = TRUE;

            viop = chan->nextViop;

            /* if curViop is same as nextViop then make it INV */
            if (chan->curViop == chan->nextViop)
            {
                chan->curViop = INV;
            }

            chan->nextViop = INV;

        } /* if (viop == (FVID_Frame *)&chan->qOut) */

        /* Fill buffer address and other information in packet */
        *(void**)packet->addr = (void*)viop;
        packet->size = sizeof(FVID_Frame);
        retVal = packet->status = IOM_COMPLETED;

        /* Decrement number of buffers with driver */
        chan->numQueBuf--;

        if (chan->numQueBuf < _VPORT_MIN_BUF)
        {
            /* If chan->numQueBuf is less than _VPORT_MIN_BUF then, put curViop
             * and nextViop in queue.
             * Take care that every buffer is being queued only once.
             */
            if (chan->nextViop != INV)
            {
                QUE_enqueue(&chan->qIn, (QUE_Handle)chan->nextViop);
                chan->queEmpty = FALSE;
            }
            if ((chan->curViop != INV) && (chan->curViop != chan->nextViop))
            {
                QUE_enqueue(&chan->qIn, (QUE_Handle)chan->curViop);
                chan->queEmpty = FALSE;
            }

            /* make all viop pointers INV */
            chan->nextViop = INV;
            chan->curViop = INV;

            /* set channel status as EDMA not configured */
            chan->status &= ~_VPORT_READY;
        } /* if (chan->numQueBuf < _VPORT_MIN_BUF) */
    } /* if (0 == chan->numQueBuf) */

    return retVal;
}



/**
 *  \brief configure async mode for display
 *
 *  \param  chanp    [In] VPORT channel object pointer
 *  \param  cmd      [In] async mode commands
 *  \param  args     [In] async mode configuration parameters
 *
 *  \return  IOM_COMPLETED on success
 *           suitable error code if failure
 */
static Int _configAsyncMode(Ptr chanp, Uns cmd, Ptr args)
{
    _VPORT_ChanObj          *chan = (_VPORT_ChanObj *)chanp;
    VPORTDIS_AsyncModeParam *params = (VPORTDIS_AsyncModeParam *)args;
    Uint32                  gie;
    Int                     vif1 = 0, vif2 = 0;
    Int                     vInt1 = 0, vInt2 = 0;
    Int                     i;
    Int                     retVal = IOM_COMPLETED;
    ECM_Attrs               ecmattrs = ECM_ATTRS;
    Uns                     cookie = 0;
    Int                     mask = 0;
    Ptr                     elem;
    Uint32                  numBuffers = 0;
    FVID_Frame              *viop;

    PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &gie);

    chan->asyncFrameCt = 0;
    /* set the interrupt to happen 30 lines before frame complete to give enough
     * time for the application call-back function to execute successfully
     */
    if (chan->interlaced)
    {
        vif2 = 1;
        vInt2 = chan->frmSz - 30;
        mask = VPORT_INT_VINT2;
    }
    else
    {
        vif1 = 1;
        vInt1 = chan->frmSz - 30;
        mask = VPORT_INT_VINT1;
    }

    if (cmd == VPORTDIS_CMD_ASYNC_MODE_ENABLE)
    {
        numBuffers = (LEGACY_MODE == chan->driverMode) ? chan->numFrms :
                     chan->numQueBuf;
        if (params->mode == VPORTDIS_PULLDOWN_24P_30I)
        {
            if ((numBuffers < 4) || (!(chan->mergeFlds)))
            {
                /* make sure the mergeFlds flag is enabled */
                retVal = IOM_EBADARGS;
            }
        }
        else if (numBuffers < 2)
        {
            retVal = IOM_EBADARGS;
        }

        if (IOM_COMPLETED == retVal)
        {
            chan->asyncModeEnable = TRUE;
            chan->asyncCbArg = params->arg;
            chan->asyncCallBackFxn = (Fxn)params->callBackFxn;
            chan->queEmpty = TRUE; /* always TRUE */

            chan->vportRegs->VD_VINT = VP_VDVINT_RMK(vif2, vInt2, vif1, vInt1);

            chan->pullDownMode = params->mode;

            PAL_osEvtDisable((Uint32)(CSL_INTC_EVENTID_VP0_INT
                                            + chan->portNum));
            /* Enable the interrupt */
            ecmattrs.unmask = 1u;

            /* setup arg such that isr will get 'chan' as arg */
            ecmattrs.arg = (Arg)chan->portNum;

            /* Disabling the interrupts */
            PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

            /* Plug the ISR with interrupt (event) number */
            ECM_dispatchPlug((Uns)(CSL_INTC_EVENTID_VP0_INT + chan->portNum),
                             (ECM_Fxn)displayISR,
                             &ecmattrs);
            /* Enabling the HWI_ID where corresponding ECM group is
             * configured
             */
            C64_enableIER(1 << _IRQ_ID);

            /* Enabling the interrupts */
            PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

            /* clear vertical interrupts */
            chan->vportRegs->VP_IE &= ~mask;

            /* enable the correct one */
            chan->vportRegs->VP_IE |= mask;
            chan->vIntMask |= mask;

            /* Enabling the event */
            PAL_osEvtEnable((Uint32)(CSL_INTC_EVENTID_VP0_INT + chan->portNum));

            while ((elem = QUE_dequeue(&chan->qIn)) != &chan->qIn)
            {
                QUE_enqueue(&chan->qOut, elem);
            }
        }
    }
    else if (cmd == VPORTDIS_CMD_ASYNC_MODE_DISABLE)
    {
        if (chan->vIntFxn == INV)
        {
            PAL_osEvtDisable((Uint32)(CSL_INTC_EVENTID_VP0_INT
                                                + chan->portNum));
        }

        /* clear vertical interrupts */
        chan->vportRegs->VP_IE &= (~mask);
        chan->asyncModeEnable = FALSE;
        chan->asyncCallBackFxn = (Fxn)INVCB;

        if (LEGACY_MODE == chan->driverMode)
        {
            QUE_new(&chan->qIn);
            QUE_new(&chan->qOut);
            for (i = 0; i < chan->numFrms; i++)
            {
                if (&chan->viops[i] != chan->curViop &&
                    &chan->viops[i] != chan->nextViop)
                {
                    QUE_enqueue(&chan->qOut, &chan->viops[i]);
                }
            }
        }
        else
        {
            while ((viop = (FVID_Frame *)QUE_dequeue(&chan->qIn)) !=
                   (FVID_Frame *)&chan->qIn)
            {
                if (viop != chan->curViop && viop != chan->nextViop)
                {
                    QUE_enqueue(&chan->qOut, viop);
                }
            }
        }
    }
    else if (cmd == VPORTDIS_CMD_ASYNC_MODE_RESET_FRAMECT)
    {
        while (!QUE_empty(&chan->qIn))
        {
            QUE_enqueue(&chan->qOut, QUE_dequeue(&chan->qIn));
        }
    }
    else
    {
        retVal = IOM_EBADARGS;
    }

    PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, gie);

    return retVal;
}



/**
 *  \brief configure video port channel and EDMA settings
 *
 *  \param   chanp  [Out]  VPORT channel object pointer
 *  \param   args   [In]   display channel configuration parameters
 *
 *  \return  IOM_COMPLETED on success
 *           suitable error code on failure
 */
static Int _configChan(Ptr chanp, Ptr args)
{
    _VPORT_ChanObj  *chan = (_VPORT_ChanObj *)chanp;
    VPORTDIS_Params *params = (VPORTDIS_Params *)args;
    Int             retVal = IOM_COMPLETED;
    Uint8           bufCnt = 0;
    Uint32          prevStatus = chan->status;

    /* Validate channel parameters */
    /* Driver mode cannot be changed at runtime */
    if (LEGACY_MODE == chan->driverMode)
    {
        if ((params->numFrmBufs < _VPORT_MIN_BUF) ||
            (params->numFrmBufs > VPORT_MAX_NUM_FRMBUFS))
        {
            retVal = IOM_EBADARGS;
        }
    }
    else if ((NORMAL_MODE == chan->driverMode) && (0 != params->numFrmBufs))
    {
        retVal = IOM_EBADARGS;
    }

    if ((INV == (void*)params->segId))
    {
        retVal = IOM_EBADARGS;
    }

    if (IOM_COMPLETED == retVal)
    {
        /* If display operation is started then first stop the operation */
        if (_VPORT_STARTED & prevStatus)
        {
            retVal = mdControlChan(chan, VPORT_CMD_STOP, NULL);
        }

        /* Free memory of previously allocated buffers
         * (Applicable in LEGACY MODE only)
         */
        if ((_VPORT_BUF_ALLOCATED & prevStatus) && (IOM_COMPLETED == retVal))
        {
            for (bufCnt = 0; bufCnt < chan->numFrms; bufCnt++)
            {
                if (PAL_SOK != PAL_osMemFree(chan->segId,
                                             chan->viops[bufCnt].frame.iFrm.y1,
                                             chan->bufSz))
                {
                    retVal = IOM_EFREE;
                }
            } /* for (bufCnt = 0;... */
        } /* if (_VPORT_BUF_ALLOCATED & prevStatus) */

        if (IOM_COMPLETED == retVal)
        {
            /* Initialize channel status as Opened */
            chan->status = _VPORT_OPENED;
            chan->lastLineNum = _getlastActiveLine(params);

            /* configure video port channel A/B control register */
            _configRegs(chanp, params);

            /* assign required VPORTDIS_Params to _VPORT_ChanObj */
            chan->numFrms = params->numFrmBufs;
            chan->segId = params->segId;
            chan->alignment = params->alignment;
            chan->fldOp = params->fldOp;
            chan->numQueBuf = 0;
            chan->bufSz = (chan->yPitch * chan->numLines)
                            + (chan->cPitch * chan->numLines * 2);

            if (LEGACY_MODE == chan->driverMode)
            {
                /* configure EDMA and frame buffer only in legacy mode */
                retVal = _configEDMA(chanp);

                /* If channel's previous status was started then,start display
                 * after reconfiguration(Applicable only in LEGACY MODE.
                 * In NORMAL MODE, application has to queue buffers before
                 * starting display again)
                 */
                if ((IOM_COMPLETED == retVal) && (_VPORT_STARTED & prevStatus))
                {
                    retVal = mdControlChan(chan, VPORT_CMD_START, NULL);
                }
            } /* if (LEGACY_MODE == chan->driverMode) */
        } /* if (IOM_COMPLETED == retVal) */
    } /* if (IOM_COMPLETED == retVal) */

    return retVal;
}



/**
 *  \brief configure channel EDMA settings
 *
 *  \param   chanp  [In/Out]  VPORT channel object pointer
 *
 *  \return  IOM_COMPLETED on success
 *           IOM_EALLOC on failure
 */
static Int _configEDMA(Ptr chanp)
{
    _VPORT_ChanObj      *chan = (_VPORT_ChanObj *)chanp;
    Int                 i;
    Int                 j;
    Int                 thrld, pitch;
    Int                 edmaChans;
    Int                 retVal = IOM_COMPLETED;
    EDMA3_DRV_PaRAMRegs paramPtr = { 0 };
    EDMA3_DRV_Result    result = EDMA3_DRV_SOK;

    if (chan->status & _VPORT_CFGED)
    {
        if (LEGACY_MODE == chan->driverMode)
        {
            /* allocate frame buffers */
            for (i = 0; i < chan->numFrms; i++)
            {
                if (IOM_COMPLETED != _allocAssignBuffer(chan,
                                                        &(chan->viops[i])))
                {
                    for (j = 0; j < i; j++)
                    {
                        PAL_osMemFree(chan->segId,
                                      chan->viops[j].frame.iFrm.y1,
                                      chan->bufSz);
                    }
                    /* memory allocation fails */
                    retVal = IOM_EALLOC;
                    break;
                }
            } /* for (i = 0; i < chan->numFrms; i++) */

            if (IOM_COMPLETED == retVal)
            {
                chan->status |= _VPORT_BUF_ALLOCATED;

                for (i = 1; i < chan->numFrms; i++)
                {
                    /* don't put the first 1 viop into the queue */
                    QUE_enqueue(&chan->qOut, (QUE_Handle)&chan->viops[i]);
                }
            }
        } /* if (LEGACY_MODE == chan->driverMode) */

        if (IOM_COMPLETED == retVal)
        {
            if (LEGACY_MODE == chan->driverMode)
            {
                chan->curViop = &chan->viops[0];
                chan->nextViop = &chan->viops[0];
                chan->queEmpty = TRUE;
            }
            else
            {
                chan->curViop = (FVID_Frame *)(QUE_dequeue(&chan->qIn));
                chan->nextViop = chan->curViop;
                chan->queEmpty = FALSE ;
            }

            if (chan->mode & _VPORT_MASK_RAW)
            {
                edmaChans = 1;
            }
            else
            {
                edmaChans = 3;
            }

            for (i = 0; i < edmaChans; i++)
            {
                thrld = (i == 0) ? chan->yThrld : chan->cThrld;
                pitch = (i == 0) ? chan->yPitch : chan->cPitch;

                paramPtr.opt = 0;
                /* set source address mode */
                paramPtr.opt |= EDMA3_DRV_ADDR_MODE_INCR;
                /* set destination address mode */
                paramPtr.opt |= (EDMA3_DRV_ADDR_MODE_FIFO << EDMA_DAM_SHIFT);
                /* set sync mode */
                paramPtr.opt |= (EDMA3_DRV_SYNC_A << EDMA_SYNCDIM_SHIFT);
                paramPtr.opt |= (EDMA3_DRV_W64BIT << EDMA_FWID_SHIFT);

                /* set source address as FIFO address */
                paramPtr.destAddr = (unsigned int)(chan->edmaAddr[i]);

                if (chan->mergeFlds)
                {
                    /*
                     * To merge the two fields together
                     * EDMA is configured to transfer only field 1 initially
                     * line pitch is twice the line size
                     * this requires that the threshold is the same as line size
                     */

                    /* set aCnt, bCnt, cCnt values */
                    paramPtr.aCnt = (thrld << 3);
                    paramPtr.bCnt = (pitch / paramPtr.aCnt);
                    paramPtr.cCnt = chan->numLinesFld1;
                    paramPtr.bCntReload = paramPtr.bCnt;

                    /* Set index values for source and destination */
                    paramPtr.srcBIdx = paramPtr.aCnt;
                    paramPtr.destBIdx = 0;
                    paramPtr.srcCIdx = paramPtr.aCnt + pitch;
                    paramPtr.destCIdx = 0;

                    /* hard code first two frames as current and reload bufs */
                    /* First Field */
                    paramPtr.opt |=
                            (EDMA3_DRV_TCINTEN_DIS << EDMA_TCINTEN_SHIFT);
                    paramPtr.srcAddr =
                            (*((Int *)(&(chan->curViop->frame.iFrm.y1)) + i));

                    paramPtr.linkAddr = (unsigned short)
                       ((Uint32)chan->hEdmaLinkChanPhyAddr[4 * i + 1] & 0xFFFF);

                    result |= EDMA3_DRV_setPaRAM(chan->hEdma,
                                                 chan->hEdmaChan[i],
                                                 &paramPtr);
                    result |= EDMA3_DRV_setPaRAM(chan->hEdma,
                                                 chan->edmaLinkChanNum[4 * i],
                                                 &paramPtr);

                    paramPtr.linkAddr = (unsigned short)
                       ((Uint32)chan->hEdmaLinkChanPhyAddr[4 * i + 3] & 0xFFFF);

                    result |= EDMA3_DRV_setPaRAM(chan->hEdma,
                                             chan->edmaLinkChanNum[4 * i + 2],
                                             &paramPtr);

                    /* Second Field */
                    paramPtr.opt |=
                                ((i == 0)
                                ?(EDMA3_DRV_TCINTEN_EN << EDMA_TCINTEN_SHIFT)
                                :(EDMA3_DRV_TCINTEN_DIS << EDMA_TCINTEN_SHIFT));

                    paramPtr.opt &= EDMA_TCC_MASK;
                    paramPtr.opt |=
                                ((i == 0)
                                ?(chan->tcc[0] << EDMA_TCC_SHIFT)
                                :(0 << EDMA_TCC_SHIFT));

                    paramPtr.cCnt = (chan->numLines - chan->numLinesFld1);

                    if (!(chan->mode & _VPORT_MASK_RAW))
                    {
                        paramPtr.srcAddr =
                           (*((Int *)(&(chan->curViop->frame.iFrm.y2)) + i));
                    }
                    else
                    {
                        paramPtr.srcAddr =
                           (*((Int *)(&(chan->curViop->frame.riFrm.buf2)) + i));
                    }

                    paramPtr.linkAddr = (unsigned short)
                       ((Uint32)chan->hEdmaLinkChanPhyAddr[4 * i + 2] & 0xFFFF);

                    result |= EDMA3_DRV_setPaRAM(chan->hEdma,
                                             chan->edmaLinkChanNum[4 * i + 1],
                                             &paramPtr);

                    paramPtr.opt &= EDMA_TCC_MASK;
                    paramPtr.opt |=
                            ((i == 0)
                            ?(chan->tcc[1] << EDMA_TCC_SHIFT)
                            :(0 << EDMA_TCC_SHIFT));

                    paramPtr.linkAddr = (unsigned short)
                       ((Uint32)chan->hEdmaLinkChanPhyAddr[4 * i] & 0xFFFF);

                    result |= EDMA3_DRV_setPaRAM(chan->hEdma,
                                            chan->edmaLinkChanNum[4 * i + 3],
                                            &paramPtr);
                }
                else
                {
                    /* if not interlaced display, configure EDMA to transfer
                     * the whole frame. The line pitch is just the line size
                     */

                    /* set aCnt, bCnt, cCnt values */
                    paramPtr.aCnt = (thrld << 3);
                    paramPtr.bCnt = (pitch / paramPtr.aCnt);
                    paramPtr.cCnt = chan->numLines;
                    paramPtr.bCntReload = paramPtr.bCnt;

                    /* Set index values for source and destination */
                    paramPtr.srcBIdx = paramPtr.aCnt;
                    paramPtr.destBIdx = 0;
                    paramPtr.srcCIdx = paramPtr.aCnt;
                    paramPtr.destCIdx = 0;

                    paramPtr.opt |=
                            ((i == 0)
                            ?(EDMA3_DRV_TCINTEN_EN << EDMA_TCINTEN_SHIFT)
                            :(EDMA3_DRV_TCINTEN_DIS << EDMA_TCINTEN_SHIFT));

                    paramPtr.opt &= EDMA_TCC_MASK;
                    paramPtr.opt |=
                            ((i == 0)
                            ?(chan->tcc[0] << EDMA_TCC_SHIFT)
                            :(0 << EDMA_TCC_SHIFT));

                    /* hard code the first and second frame buffer as current
                     * and reload buffers
                     */
                    paramPtr.srcAddr =
                            (*((Int *)(&(chan->curViop->frame.iFrm.y1)) + i));

                    paramPtr.linkAddr = (unsigned short)
                       ((Uint32)chan->hEdmaLinkChanPhyAddr[4 * i + 2] & 0xFFFF);

                    result |= EDMA3_DRV_setPaRAM(chan->hEdma,
                                                 chan->hEdmaChan[i],
                                                 &paramPtr);
                    result |= EDMA3_DRV_setPaRAM(chan->hEdma,
                                                 chan->edmaLinkChanNum[4 * i],
                                                 &paramPtr);

                    paramPtr.opt &= EDMA_TCC_MASK;
                    paramPtr.opt |=
                            ((i == 0)
                            ?(chan->tcc[1] << EDMA_TCC_SHIFT)
                            :(0 << EDMA_TCC_SHIFT));

                    paramPtr.linkAddr = (unsigned short)
                       ((Uint32)chan->hEdmaLinkChanPhyAddr[4 * i] & 0xFFFF);

                    result |= EDMA3_DRV_setPaRAM(chan->hEdma,
                                             chan->edmaLinkChanNum[4 * i + 2],
                                             &paramPtr);
                } /* if (chan->mergeFlds) */
            } /* for (i = 0; i < numEdmaChans; i++) */

            chan->nextEDMARlds = 1;

            for (i = 0; i < edmaChans; i++)
            {
                result |= EDMA3_DRV_enableTransfer(chan->hEdma,
                                                   chan->hEdmaChan[i],
                                                   EDMA3_DRV_TRIG_MODE_EVENT);
            }

            chan->status |= _VPORT_READY;
        } /* if (IOM_COMPLETED == retVal) */
    } /* if (chan->status & _VPORT_CFGED) */

    if (EDMA3_DRV_SOK != result)
    {
        retVal = IOM_EALLOC;
    }

    return retVal;
}



/**
 *  \brief   Reconfigure EDMA hardware channels with current param set
 *
 *  \param   chanp  [In/Out]  VPORT channel object pointer
 *
 *  \return  IOM_COMPLETED on success
 *           IOM_EALLOC on failure
 */
static Int _reconfigEDMAchan(Ptr chanp)
{
    _VPORT_ChanObj      *chan = (_VPORT_ChanObj *)chanp;
    Int                 offset;
    EDMA3_DRV_PaRAMRegs paramPtr = { 0 };
    Int                 retVal = IOM_COMPLETED;
    EDMA3_DRV_Result    result = EDMA3_DRV_SOK;
    Int                 numEdmaChans;
    volatile Int        i;
    unsigned short      tccStatus;
    unsigned int        link1SrcAddr, link2SrcAddr;

    /* Disable the EDMA before setting them up */
    if (chan->mode & _VPORT_MASK_RAW)
    {
        numEdmaChans = 1;
    }
    else
    {
        numEdmaChans = _VPORT_NUM_EDMA_CHANS;
    }

    /* Disable EDMA transfers */
    for (i = 0; i < numEdmaChans; i++)
    {
        result |= EDMA3_DRV_disableTransfer(chan->hEdma,
                                            chan->hEdmaChan[i],
                                            EDMA3_DRV_TRIG_MODE_EVENT);

        /* Clear any pending errors and events */
        result |= EDMA3_DRV_clearErrorBits(chan->hEdma, chan->hEdmaChan[i]);
    }

    /* Clear any pending interrupts - for both the TCC (current & next VIOP) */
    result |= EDMA3_DRV_checkAndClearTcc(chan->hEdma, chan->tcc[0], &tccStatus);
    result |= EDMA3_DRV_checkAndClearTcc(chan->hEdma, chan->tcc[1], &tccStatus);

    /************** Reconfigure H/W EDMA parameters again **************/

    /* Find out which param set matches with the software current viop */
    link1SrcAddr = chan->hEdmaLinkChanPhyAddr[0]->srcAddr;
    link2SrcAddr = chan->hEdmaLinkChanPhyAddr[2]->srcAddr;
    if (link1SrcAddr == (Uint32)chan->curViop->frame.iFrm.y1)
    {
        /* 1st param set matches with current viop in software. Reload param set
           0 into hardware channel for restart */
        offset = 0;
        /* 1st param set will be reloaded to hardware channel. Hence next EDMA
           reload will be param set 1 */
        chan->nextEDMARlds = 1;
    }
    else if (link2SrcAddr == (Uint32)chan->curViop->frame.iFrm.y1)
    {
        /* 2nd param set matches with current viop in software. Reload param set
           1 into hardware channel for restart */
        offset = 2;
        /* 2nd Param will be reloaded to hardware channel. Hence next EDMA
           reload will be param set 0 */
        chan->nextEDMARlds = 0;
    }
    else
    {
#ifdef VPORT_DEBUG
        chan->reconfigErrCount++;
#endif  /* #ifdef VPORT_DEBUG */

        /*
         * Some error has happened!! Reuse software maintained param offset
         */
        /* Based on current EDMA parameters loaded in EDMA channels, get
         * particular param set and configure EDMA channels again using this
         * param set. chan->nextEDMARlds indicates next EDMA paramset,
         * complement of it will give current EDMA parameters (i.e if
         * chan->nextEDMARlds is 0 then current EDMA paramset will be 1 and
         * vice-versa).
         */
        offset = ((!chan->nextEDMARlds) << 1);
    }

    for (i = 0; i < numEdmaChans; i++)
    {
        /* Reconfigure Y, Cb and Cr Parameters */
        result |= EDMA3_DRV_getPaRAM(chan->hEdma,
                                     chan->edmaLinkChanNum[(i * 4) + offset],
                                     &paramPtr);
        result |= EDMA3_DRV_setPaRAM(chan->hEdma,
                                     chan->hEdmaChan[i],
                                     &paramPtr);
    }

    /* enable the EDMA events again */
    for (i = 0; i < numEdmaChans; i++)
    {
        result |= EDMA3_DRV_enableTransfer(chan->hEdma,
                                           chan->hEdmaChan[i],
                                           EDMA3_DRV_TRIG_MODE_EVENT);
    }

    if (EDMA3_DRV_SOK == result)
    {
        retVal = IOM_COMPLETED;
    }
    else
    {
        retVal = IOM_EALLOC;
    }

    return retVal;
}



/**
 *  \brief   allocating buffer as requested by application
 *
 *  \param   chan     [In]   VPORT channel object pointer
 *  \param   newFrame [Out]  Pointer to FVID frame pointer
 *
 *  \return  IOM_COMPLETED on success
 *           IOM_EALLOC on failure
 */
static Int _allocBuffer(_VPORT_ChanObj *chan, FVID_Frame **newFrame)
{
    Int retVal = IOM_COMPLETED;

    /* allocate frame buffer structure */
    if (PAL_SOK != PAL_osMemAlloc(chan->segId,
                                  sizeof(FVID_Frame),
                                  NULL,
                                  (Ptr *)newFrame))
    {
        retVal = IOM_EALLOC;
    }

    if (IOM_EALLOC != retVal)
    {
        if (IOM_COMPLETED != _allocAssignBuffer(chan, *newFrame))
        {
            /* memory allocation fails */
            PAL_osMemFree(chan->segId, *newFrame, sizeof(FVID_Frame));

            retVal = IOM_EALLOC;
        }
    }

    return retVal;
}



/**
 *  \brief   allocating buffer and assigning it to FVID_Frame structure
 *
 *  \param   chan     [In]  VPORT channel object pointer
 *  \param   newFrame [Out]  FVID frame pointer
 *
 *  \return  IOM_COMPLETED on success
 *           IOM_EALLOC on failure
 */
static Int _allocAssignBuffer(_VPORT_ChanObj *chan, FVID_Frame *newFrame)
{
    Int     retVal = IOM_COMPLETED;
    Int8    *curAddr = NULL;
    Int     j;

    /* allocate frame buffer */
    if (PAL_SOK == PAL_osMemAlloc(chan->segId,
                                  chan->bufSz,
                                  chan->alignment,
                                  (Ptr *)&curAddr))
    {
        if (!(chan->mode & _VPORT_MASK_RAW))
        {
            /* field 1 */
            newFrame->frame.iFrm.y1 = curAddr;

            curAddr += chan->numLines * chan->yPitch;
            newFrame->frame.iFrm.cb1 = curAddr;
            curAddr += chan->numLines * chan->cPitch;
            newFrame->frame.iFrm.cr1 = curAddr;
            curAddr += chan->numLines * chan->cPitch;
            /* field 2 */
            if (chan->fldOp == VPORT_FLDOP_FLD2)
            {
                newFrame->frame.iFrm.y2 = newFrame->frame.iFrm.y1;
                newFrame->frame.iFrm.cb2 = newFrame->frame.iFrm.cb1;
                newFrame->frame.iFrm.cr2 = newFrame->frame.iFrm.cr1;
            }
            else if (!chan->mergeFlds)
            {
                newFrame->frame.iFrm.y2 = newFrame->frame.iFrm.y1 +
                                          chan->numLinesFld1 * chan->yPitch;
                newFrame->frame.iFrm.cb2 = newFrame->frame.iFrm.cb1 +
                                           (chan->numLinesFld1 * chan->cPitch);
                newFrame->frame.iFrm.cr2 = newFrame->frame.iFrm.cr1 +
                                           (chan->numLinesFld1 * chan->cPitch);
            }
            else
            {
                newFrame->frame.iFrm.y2 = newFrame->frame.iFrm.y1 +
                                          chan->yPitch;
                newFrame->frame.iFrm.cb2 = newFrame->frame.iFrm.cb1 +
                                           chan->cPitch;
                newFrame->frame.iFrm.cr2 = newFrame->frame.iFrm.cr1 +
                                           chan->cPitch;
            }

            for (j = 0; j < chan->numLines; j++)
            {
                PAL_osMemSet(newFrame->frame.iFrm.y1 + j * chan->yPitch,
                             fillValue[0],
                             chan->yPitch);
                PAL_osMemSet(newFrame->frame.iFrm.cb1 + j * chan->cPitch,
                             fillValue[1],
                             chan->cPitch);
                PAL_osMemSet(newFrame->frame.iFrm.cr1 + j * chan->cPitch,
                             fillValue[2],
                             chan->cPitch);
            }

            PAL_osCacheFlushAndInvalidate(PAL_osCache_NOT_DEFINED,
                                          (Uint32)newFrame->frame.iFrm.y1,
                                          chan->bufSz);

            newFrame->frameFormat = FVID_YCbCr422_PLANAR;
            newFrame->bpp = FVID_BPP_BITS16;
        }
        else
        {
            /* field 1 */
            newFrame->frame.riFrm.buf1 = curAddr;

            /* field 2 */
            if (chan->fldOp == VPORT_FLDOP_FLD2)
            {
                newFrame->frame.riFrm.buf2 = newFrame->frame.riFrm.buf1;
            }
            else if (!chan->mergeFlds)
            {
                newFrame->frame.riFrm.buf2 = newFrame->frame.riFrm.buf1 +
                                          chan->numLinesFld1 * chan->yPitch;
            }
            else
            {
                newFrame->frame.riFrm.buf2 = newFrame->frame.riFrm.buf1 +
                                          chan->yPitch;
            }

            /* Memset Raw buffer to Zero */
            PAL_osMemSet(newFrame->frame.riFrm.buf1, 0, chan->bufSz);
            PAL_osCacheFlushAndInvalidate(PAL_osCache_NOT_DEFINED,
                                          (Uint32)newFrame->frame.riFrm.buf1,
                                          chan->bufSz);

            newFrame->frameFormat = FVID_RAW_FORMAT;

            if (VPORT_MODE_RAW_8BIT == chan->mode)
            {
                newFrame->bpp = FVID_BPP_BITS8;
            }
            else
            {
                newFrame->bpp = FVID_BPP_BITS16;
            }
        } /* if (!(chan->mode & _VPORT_MASK_RAW)) */

        /* Set FVID_Frame fields */
        newFrame->timeStamp = 0;
        newFrame->pitch = chan->yPitch;
        newFrame->lines = chan->numLines;
    }
    else
    {
        retVal = IOM_EALLOC;
    }

    return retVal;
}



/**
 *  \brief   free the given frame buffer and its structure memory
 *
 *  \param   chan         [In]  VPORT channel object pointer
 *  \param   freeFrame     [In]  FVID frame pointer
 *
 *  \return  IOM_COMPLETED on success
 *           IOM_EFREE on failure
 */
static Int _freeBuffer(_VPORT_ChanObj *chan, FVID_Frame **freeFrame)
{
    Int retVal = IOM_COMPLETED;
    FVID_Frame *ptrFrame = *freeFrame;

    if ((NULL == ptrFrame) || (NULL == ptrFrame->frame.iFrm.y1))
    {
        retVal = IOM_EBADARGS;
    }
    else
    {
        /* free frame buffer */
        if (PAL_SOK == PAL_osMemFree(chan->segId,
                                     ptrFrame->frame.iFrm.y1,
                                     chan->bufSz))
        {
            if (PAL_SOK != PAL_osMemFree(chan->segId, ptrFrame,
                                         sizeof(FVID_Frame)))
            {
                retVal = IOM_EFREE;
            }
        }
        else
        {
            retVal = IOM_EFREE;
        }
    }

    return retVal;
}



/**
 *  \brief   reset display operation
 *
 *  \param   chanp     [In]  VPORT channel object pointer
 */
static void _resetDisp(Ptr chanp)
{
    _VPORT_ChanObj *chan = (_VPORT_ChanObj *)chanp;
    volatile Uint32 vpDisCtrl;

    /* reset display */
    chan->vportRegs->VD_CTL = CSL_VPORT_VD_CTL_RSTCH_MASK;
    while (1)
    {
        vpDisCtrl = chan->vportRegs->VD_CTL;
        if (!(vpDisCtrl & CSL_VPORT_VD_CTL_RSTCH_MASK))
        {
            break;
        }
    }
}



/**
 *  \brief   configure port level registers
 *
 *  \param   chanp  [In]  VPORT channel object pointer
 *  \param   args   [In]  display channel configuration parameters
 *
 *  \return  IOM_COMPLETED
 */
static Int _configPort(Ptr chanp, Ptr args)
{
    _VPORT_ChanObj      *chan = (_VPORT_ChanObj *)chanp;
    VPORT_PortParams    *portParams = (VPORT_PortParams *)args;
    volatile Uint32     vpCtrl, vpDisCtrl;

    /* reset video port */
    chan->vportRegs->VP_CTL = CSL_VPORT_VP_CTL_VPRST_MASK;

    /* enable video port */
    chan->vportRegs->PCR |= CSL_VPORT_PCR_PEREN_MASK;

    while (1)
    {
        vpCtrl = chan->vportRegs->VP_CTL;
        if (!(vpCtrl & CSL_VPORT_VP_CTL_VPRST_MASK))
        {
            break;
        }
    }

    /* reset display */
    _resetDisp(chan);

    /* configure video port control register */
    chan->vportRegs->VP_CTL = VP_VPCTL_RMK(0, /* VP reset enable, No effect */
                                           0, /* VP halt bit, No effect */
                                           1, /* Invert VCLK2 clock polarity */
                                           portParams->vc3Polarity,
                                           portParams->vc2Polarity,
                                           portParams->vc1Polarity,
                                           0, /* Disable TCI capture mode */
                                           1, /* Enable display mode */
                                           0 /* Disable Dual channel */);

    /* enable video port */
    chan->vportRegs->VP_CTL |= CSL_VPORT_VP_CTL_VPHLT_MASK;

    /* Assign EDC function table pointer to global channel object */
    chan->edcFxns = portParams->edcTbl[0];

    return IOM_COMPLETED;
}



/**
 *  \brief configure video port registers for display
 *
 *  \param   chanp  [In]  VPORT channel object pointer
 *  \param   params [In]  display channel configuration parameters
 *
 *  \return  IOM_COMPLETED
 */
static  Int _configRegs(Ptr chanp, VPORTDIS_Params *params)
{
    _VPORT_ChanObj  *chan = (_VPORT_ChanObj *)chanp;
    Int             numPixels, numLines, numCPixels;
    Int             vdCtl;
    Int             nh1 = 0, nh2 = 0, nv1 = 0, nv2 = 0;

    if (chan->status & _VPORT_OPENED)
    {
        QUE_new(&chan->qIn);
        QUE_new(&chan->qOut);
        chan->queEmpty = FALSE;
        chan->packetIOM = INV;

        /* configure display settings */
        vdCtl = VP_VDCTL_RMK(0, /* channel reset bit, No effect */
                             1, /* Block EDMA events and flush display FIFOs */
                             0, /* Disable Previous VPORT sync */
                             ((params->extCtl & 4) >> 2), /* Field control */
                             ((params->extCtl & 2) >> 1), /* Vertical control */
                             params->extCtl & 1, /* Horizontal control */
                             params->vctl3Config,
                             params->vctl2Config,
                             params->vctl1Config,
                             0, /* Disable video display */
                             params->bpk10Bit, /* Reserved */
                             params->rgbX,
                             0, /* synchronized raw channel enable, Not used */
                             params->defValEn,
                             params->resmpl,
                             params->scale,
                             1, /* Enable continuous display */
                             ((params->fldOp & 4) >> 2), /* Display frame bit */
                             ((params->fldOp & 2) >> 1), /* Display fld 2 bit */
                             (params->fldOp & 1), /* Display fld 1 bit */
                             params->dmode);

        numLines = 0;
        numPixels = 0;
        if (params->fldOp != VPORT_FLDOP_FLD2)
        {
            /* progressive scan, or field 1 or frame */
            numPixels = params->imgHSizeFld1;
            numLines = params->imgVSizeFld1;
        }
        chan->numLinesFld1 = numLines;

        if (params->fldOp == VPORT_FLDOP_FLD2 ||
            params->fldOp == VPORT_FLDOP_FRAME)
        {
            if (!numPixels)
            {
                numPixels = params->imgHSizeFld2;
            }
            numLines += params->imgVSizeFld2;
            chan->interlaced = TRUE;
        }
        chan->numLines = numLines;
        chan->frmSz = params->frmVSize;

        chan->vportRegs->VD_CTL = vdCtl;

        chan->vportRegs->VD_FRMSZ = VP_VDFRMSZ_RMK(params->frmVSize,
                                                   params->frmHSize);

        chan->vportRegs->VD_HBLNK = VP_VDHBLNK_RMK(params->hBlnkStop,
                                    !(params->dmode & _VPORT_MASK_RAW),
                                    params->hBlnkStart);
        chan->vportRegs->VD_VBLKS1 = VP_VDVBLKS1_RMK(params->vBlnkYStartFld1,
                                                     params->vBlnkXStartFld1);
        chan->vportRegs->VD_VBLKE1 = VP_VDVBLKE1_RMK(params->vBlnkYStopFld1,
                                                     params->vBlnkXStopFld1);
        chan->vportRegs->VD_VBLKS2 = VP_VDVBLKS2_RMK(params->vBlnkYStartFld2,
                                                     params->vBlnkXStartFld2);
        chan->vportRegs->VD_VBLKE2 = VP_VDVBLKE2_RMK(params->vBlnkYStopFld2,
                                                     params->vBlnkXStopFld2);

        chan->vportRegs->VD_VBIT1 = VP_VDVBIT1_RMK(params->vBlnkYStopFld1,
                                                   params->vBlnkYStartFld1);
        chan->vportRegs->VD_VBIT2 = VP_VDVBIT1_RMK(params->vBlnkYStopFld2,
                                                   params->vBlnkYStartFld2);

        if (params->imgHOffsetFld1 < 0)
        {
            nh1 = 1;
            params->imgHOffsetFld1 = -params->imgHOffsetFld1;
        }
        if (params->imgHOffsetFld2 < 0)
        {
            nh2 = 1;
            params->imgHOffsetFld2 = -params->imgHOffsetFld2;
        }
        if (params->imgVOffsetFld1 < 0)
        {
            nv1 = 1;
            params->imgVOffsetFld1 = -params->imgVOffsetFld1;
        }
        if (params->imgHOffsetFld2 < 0)
        {
            nv2 = 1;
            params->imgVOffsetFld2 = -params->imgVOffsetFld2;
        }

        chan->vportRegs->VD_IMGOFF1 = VP_VDIMGOFF1_RMK(nv1,
                                                       params->imgVOffsetFld1,
                                                       nh1,
                                                       params->imgHOffsetFld1);
        chan->vportRegs->VD_IMGSZ1 = VP_VDIMGSZ1_RMK(params->imgVSizeFld1,
                                                     params->imgHSizeFld1);
        chan->vportRegs->VD_IMGOFF2 = VP_VDIMGOFF2_RMK(nv2,
                                                       params->imgVOffsetFld2,
                                                       nh2,
                                                       params->imgHOffsetFld2);
        chan->vportRegs->VD_IMGSZ2 = VP_VDIMGSZ2_RMK(params->imgVSizeFld2,
                                                     params->imgHSizeFld2);

        chan->vportRegs->VD_FLDT1 = VP_VDFLDT1_RMK(params->yStartFld1,
                                                   params->xStartFld1);
        chan->vportRegs->VD_FLDT2 = VP_VDFLDT2_RMK(params->yStartFld2,
                                                   params->xStartFld2);
        chan->vportRegs->VD_FBIT = VP_VDFBIT_RMK(params->yStartFld2,
                                                 params->yStartFld1);

        chan->vportRegs->VD_HSYNC = VP_VDHSYNC_RMK(params->hSyncStop,
                                                   params->hSyncStart);
        chan->vportRegs->VD_VSYNS1 = VP_VDVSYNS1_RMK(params->vSyncYStartFld1,
                                                     params->vSyncXStartFld1);
        chan->vportRegs->VD_VSYNE1 = VP_VDVSYNE1_RMK(params->vSyncYStopFld1,
                                                     params->vSyncXStopFld1);
        chan->vportRegs->VD_VSYNS2 = VP_VDVSYNS2_RMK(params->vSyncYStartFld2,
                                                     params->vSyncXStartFld2);
        chan->vportRegs->VD_VSYNE2 = VP_VDVSYNE2_RMK(params->vSyncYStopFld2,
                                                     params->vSyncXStopFld2);

        chan->vportRegs->VD_CLIP = VP_VDCLIP_RMK(params->cClipHigh,
                                                 params->cClipLow,
                                                 params->yClipHigh,
                                                 params->yClipLow);
        if (params->dmode & _VPORT_MASK_RAW)
        {
            chan->vportRegs->VD_DEFVAL = params->yDefVal |
                                        (params->cbDefVal << 8) |
                                        ((params->crDefVal & 0x0F) << 16);
        }
        else
        {
            chan->vportRegs->VD_DEFVAL = params->yDefVal |
                                        (params->cbDefVal << 16) |
                                        (params->crDefVal << 24);
        }

        chan->resmpl = params->resmpl;
        chan->scale = params->scale;
        numPixels >>= params->scale;
        chan->numPixels = numPixels;
        numCPixels = (params->dmode & _VPORT_MASK_RAW) ? 0 : numPixels >> 1;

        if (params->dmode & _VPORT_MASK_RAW)
        {
            /* raw mode */
            chan->cPitch = 0;
            if (params->dmode == VPORT_MODE_RAW_8BIT)
            {
                chan->yPitch = (numPixels + 7) & (~7);
                chan->cPitch = (numCPixels + 7) & (~7);
            }
            else if (params->dmode == VPORT_MODE_RAW_10BIT)
            {
                if (params->bpk10Bit == VPORTDIS_BPK_10BIT_DENSE)
                {
                    chan->yPitch = (numPixels * 4 / 3 + 7) & (~7);
                }
                else
                {
                    chan->yPitch = (numPixels * 2 + 7) & (~7);
                }
            }
            else if (params->dmode == VPORT_MODE_RAW_16BIT)
            {
                chan->yPitch = (numPixels * 2 + 7) & (~7);
            }
            else
            {
                chan->yPitch = (numPixels * 4 + 7) & (~7);
            }
        }
        else
        {
            if (params->dmode & _VPORT_MASK_10BIT)
            {
                if (params->bpk10Bit == VPORTDIS_BPK_10BIT_DENSE)
                {
                    chan->yPitch = (numPixels * 4 / 3 + 7) & (~7);
                    chan->cPitch = (numCPixels * 4 / 3 + 7) & (~7);
                }
                else
                {
                    chan->yPitch = (numPixels * 2 + 7) & (~7);
                    chan->cPitch = (numCPixels * 2 + 7) & (~7);
                }
            }
            else
            {
                /* 8 bit mode */
                chan->yPitch = (numPixels + 7) & (~7);
                chan->cPitch = (numCPixels + 7) & (~7);
            }
        } /* if (params->dmode & _VPORT_MASK_RAW) */

        chan->yThrld = params->thrld;

        if (0 == chan->yThrld)
        {
            /* if threshold is zero then set it same as line size */
            chan->yThrld = chan->yPitch >> 3;
        }
        else if (((chan->yThrld << 3) > chan->yPitch) ||
                 ((chan->yPitch % (chan->yThrld << 3)) != 0))
        {
            /* if threshold is greater than yPitch or is not multiple of
             * yPitch then set threshold same as line size
             */
            chan->yThrld = chan->yPitch >> 3;
        }

        if (params->dmode & _VPORT_MASK_RAW)
        {
            chan->cThrld = 0;
        }
        else
        {
            chan->cThrld = (chan->yThrld + 1) >> 1;

            if (((chan->cThrld << 3) > chan->cPitch) ||
                ((chan->cPitch % (chan->cThrld << 3)) != 0))
            {
                /* if threshold is greater than cPitch or is not multiple of
                 * cPitch then set threshold same as line size
                 */
                chan->yThrld = chan->yPitch >> 3;
                chan->cThrld = (chan->yThrld + 1) >> 1;
            }
        }

        if (VPORT_FLDOP_FRAME == params->fldOp)
        {
            chan->mergeFlds = params->mergeFlds;
        }
        else
        {
            chan->mergeFlds = FALSE;
        }

        /* these two asserts make sure that total transfer sizes of both the
         * whole frame and the first field are multiples of the threshold
         */
        assert(((chan->yPitch * chan->numLinesFld1) / (chan->yThrld << 3)) *
               (chan->yThrld << 3) == (chan->yPitch * chan->numLinesFld1));
        assert(((chan->yPitch * chan->numLines) / (chan->yThrld << 3)) *
               (chan->yThrld << 3) == (chan->yPitch * chan->numLines));

        chan->numEventsFld1 = chan->yPitch * chan->numLinesFld1 /
                              (chan->yThrld << 3);
        chan->numEvents = chan->yPitch * chan->numLines /
                          (chan->yThrld << 3);


        chan->vportRegs->VD_THRLD = VP_VDTHRLD_RMK(chan->yThrld,
                                                   params->incPix,
                                                   chan->yThrld);
        chan->vportRegs->VD_DISPEVT = VP_VDDISPEVT_RMK((chan->numEvents -
                                                       chan->numEventsFld1),
                                                       chan->numEventsFld1);

        chan->mode = params->dmode;
        chan->status |= _VPORT_CFGED;
    } /* if (chan->status & _VPORT_OPENED) */

    return IOM_COMPLETED;
}



/**
 *  \brief Checks whether EDMA parameters can be safely updated in the EDMA ISR
 *
 *  This functions checks if it is safe to update the EDMA parameters (voip
 *  buffer address in param tables) in EDMA completion ISR. This error detection
 *  is required to prevent any mismatch (loss of synchronization) between
 *  EDMA current/next buffer and software maintained current/next buffer.
 *
 *  This loss of synchronization can happen when EDMA ISR could not have a
 *  chance to run or is delayed more than one frame period. This delay or ISR
 *  miss can happen when the CPU load is very high (near 100%) or when break
 *  points are kept or when emulation pause happens during memory/variable
 *  updates using CCS.
 *
 *  Detection is done in two steps:
 *      1. Current working EDMA param set and param set for which the EDMA ISR
 *         has occurred are compared. If both are equal, then the previous param
 *         set ISR is missed!! Software current viop and EDMA working viop are
 *         same. This could lead to buffer corruption if the buffer is given
 *         to the application!!
 *         For comparison, link param field is used as it is the only field
 *         which is unique for each param set and which is not modified for each
 *         viop.
 *
 *      2. Check whether EDMA is capturing at the end of current param set. If
 *         it very near to completion of the current param set, EDMA reload of
 *         next viop param could soon happen before we could successfully update
 *         the next param table. This could also lead to buffer corruption if
 *         the buffer is given to the application!!
 *
 *  \param  chan    [In] VPORT channel object pointer
 *
 *  \return  TRUE if it is too late to update EDMA parameters in EDMA ISR
 *           FALSE if EDMA parameters can be safely updated in EDMA ISR
 */
static Bool _isItTooLateInISR(_VPORT_ChanObj *chan)
{
    Bool                tooLate = FALSE;
    Uint32              linesTransferred;
    Int                 offset, nextOffset;
    volatile Uint16     paramLinkAddr1, paramLinkAddr2;
    volatile Uint16     hwLinkAddr, hw_cCnt, hwLinkAddr1;

    offset = chan->nextEDMARlds << 1;
    nextOffset = (!chan->nextEDMARlds) << 1;

    /*
     * Check 1: Check if EDMA reload for the current viop has already happened
     * before the ISR could get a chance to run
     */
    /* Get the hardware param set */
    hwLinkAddr = chan->hEdmaChanPhyAddr[0]->linkAddr;
    hw_cCnt = chan->hEdmaChanPhyAddr[0]->cCnt;

    /* Read the link address again to make sure reload has not happened!! */
    hwLinkAddr1 = chan->hEdmaChanPhyAddr[0]->linkAddr;
    if (hwLinkAddr != hwLinkAddr1)
    {
        /* EDMA reload has happened - Read the parameters again hoping that EDMA
           reload cannot happen again so soon!!  */
        hwLinkAddr = chan->hEdmaChanPhyAddr[0]->linkAddr;
        hw_cCnt = chan->hEdmaChanPhyAddr[0]->cCnt;

#ifdef VPORT_DEBUG
        chan->tooLateInISRCount[0]++;
#endif  /* #ifdef VPORT_DEBUG */
    }

    /* Get the link address for current param set for which we have got ISR */
    paramLinkAddr1 = chan->hEdmaLinkChanPhyAddr[offset]->linkAddr;
    paramLinkAddr2 = chan->hEdmaLinkChanPhyAddr[offset + 1]->linkAddr;

    /* Check whether link addresses are matching */
    if (chan->mergeFlds && ((hwLinkAddr == paramLinkAddr1)
                            || (hwLinkAddr == paramLinkAddr2)))
    {
        /* One of the link addresses is matching for interlaced mode */
        tooLate = TRUE;

#ifdef VPORT_DEBUG
        chan->tooLateInISRCount[1]++;
#endif  /* #ifdef VPORT_DEBUG */
    }
    else if (!chan->mergeFlds && (hwLinkAddr == paramLinkAddr1))
    {
        /* Link addresses are matching for non-interlaced mode. 2nd param is not
           used in non-interlaced mode, hence not used in comparison */
        tooLate = TRUE;

#ifdef VPORT_DEBUG
        chan->tooLateInISRCount[2]++;
#endif  /* #ifdef VPORT_DEBUG */
    }
    else
    {
        /*
         * Check whether EDMA is going to get complete for the current param set
         */
        /* Note: We can't determine notTooLate for RAW display. Hence
           this is kept as FALSE for RAW display */
        if (!(chan->mode & _VPORT_MASK_RAW))
        {
            linesTransferred = chan->numLines - hw_cCnt;

            /* Get the link address for next param set */
            paramLinkAddr1 = chan->hEdmaLinkChanPhyAddr[nextOffset]->linkAddr;
            paramLinkAddr2
                        = chan->hEdmaLinkChanPhyAddr[nextOffset + 1]->linkAddr;

            /* Check whether param set 0 or 1 is getting transferred */
            if (chan->mergeFlds)
            {
                if (hwLinkAddr == paramLinkAddr1)
                {
                    /* Param set 0 is getting transferred. So substract field 2
                       line added earlier!! */
                    linesTransferred -= (chan->numLines - chan->numLinesFld1);
                }
#ifdef VPORT_DEBUG
                else if (hwLinkAddr != paramLinkAddr2)
                {
                    chan->tooLateInISRCount[3]++;
                }
#endif  /* #ifdef VPORT_DEBUG */
            }

            /* Check if EDMA is capturing at the last few lines */
            if (chan->numLines > NOT_TOO_LATE_LINE_ROOM_ISR)
            {                               /* Prevent negative value */
                if (linesTransferred
                        >= (chan->numLines - NOT_TOO_LATE_LINE_ROOM_ISR))
                {
                    tooLate = TRUE;     /* EDMA is going to complete soon */

#ifdef VPORT_DEBUG
                    chan->tooLateInISRCount[4]++;
#endif  /* #ifdef VPORT_DEBUG */
                }
            }
        }
    }

    return tooLate;
}



/**
 *  \brief Updates the source buffer address of the given param table
 *         depending on various modes
 *
 *  \param  chan    [In] VPORT channel object pointer
 *  \param  offset  [In] EDMA param table to update
 *  \param  viop    [In] Source buffer address frame pointer
 *
 *  \return  IOM_COMPLETED on success
 *           IOM_EBADIO if EDMA error occurs while updating address
 */
static Int _updateEDMAParamAddress(const _VPORT_ChanObj *chan, Int offset,
                                   const FVID_Frame *viop)
{
    Int retVal = IOM_COMPLETED;

    if (chan->mergeFlds)
    {
        if (!(chan->mode & _VPORT_MASK_RAW))
        {
            /* Modify EDMA rld entries for Y transfer */
            chan->hEdmaLinkChanPhyAddr[offset]->srcAddr
                                    = (unsigned int)viop->frame.iFrm.y1;
            chan->hEdmaLinkChanPhyAddr[offset + 1]->srcAddr
                                    = (unsigned int)viop->frame.iFrm.y2;

            /* Modify EDMA rld entries for Cb transfer */
            chan->hEdmaLinkChanPhyAddr[offset + 4]->srcAddr
                                    = (unsigned int)viop->frame.iFrm.cb1;
            chan->hEdmaLinkChanPhyAddr[offset + 5]->srcAddr
                                    = (unsigned int)viop->frame.iFrm.cb2;

            /* Modify EDMA rld entries for Cr transfer */
            chan->hEdmaLinkChanPhyAddr[offset + 8]->srcAddr
                                    = (unsigned int)viop->frame.iFrm.cr1;
            chan->hEdmaLinkChanPhyAddr[offset + 9]->srcAddr
                                    = (unsigned int)viop->frame.iFrm.cr2;
        }
        else
        {
            /* Modify EDMA rld entries for RAW transfer */
            chan->hEdmaLinkChanPhyAddr[offset]->srcAddr
                                    = (unsigned int)viop->frame.riFrm.buf1;
            chan->hEdmaLinkChanPhyAddr[offset + 1]->srcAddr
                                    = (unsigned int)viop->frame.riFrm.buf2;
        }
    }
    else    /* Non-interlaced mode only one param table is used in each set */
    {
        if (!(chan->mode & _VPORT_MASK_RAW))
        {
            /* Modify EDMA rld entries for Y transfer */
            chan->hEdmaLinkChanPhyAddr[offset]->srcAddr
                                    = (unsigned int)viop->frame.iFrm.y1;

            /* Modify EDMA rld entries for Cb transfer */
            chan->hEdmaLinkChanPhyAddr[offset + 4]->srcAddr
                                    = (unsigned int)viop->frame.iFrm.cb1;

            /* Modify EDMA rld entries for Cr transfer */
            chan->hEdmaLinkChanPhyAddr[offset + 8]->srcAddr
                                    = (unsigned int)viop->frame.iFrm.cr1;
        }
        else
        {
            /* Modify EDMA rld entries for RAW transfer */
            chan->hEdmaLinkChanPhyAddr[offset]->srcAddr
                                    = (unsigned int)viop->frame.riFrm.buf1;
        }
    } /* if (chan->mergeFlds) */

    return retVal;
}



/**
 *  \brief display EDMA ISR
 *
 *  \param tcc     [In] transfer complete code
 *  \param status  [In] status of EDMA transfer
 *  \param appData []   not used
 */
static void displayEdmaISR(unsigned int tcc,
                           EDMA3_RM_TccStatus status,
                           void *appData)
{
    Int                 i;
    FVID_Frame          *viop, *curViop;
    Int                 offset;
    _VPORT_ChanObj      *chan;
    unsigned int        curParamAddr;

    /* find out the source of the EDMA interrupt */
    for (i = 0; i < CSL_VIDEOPORT_PER_CNT; i++)
    {
        chan = &chanObjs[i];
        if ((chan->status & _VPORT_READY) &&
            (tcc == chan->tcc[0] || tcc == chan->tcc[1]))
        {
#ifdef VPORT_INSTRUMENTATION_ENABLED
            PSP_LOG_EVENT(&DVTEvent_Log,
                          "VPORT_DISPLAY",
                          PSP_DVT_DESC(PSP_DVT_eDATA_SND_END,
                                       PSP_DVT_dCHANNEL_ID_INST,
                                       PSP_DVT_dPACKET_ID,
                                       PSP_DVT_dNONE),
                          CHANNELID_INSTANCEID(
                                       chan->chanNum,
                                       chan->portNum),
                          chan->curViop
                          );

            PSP_LOG_EVENT(&DVTEvent_Log,
                          "VPORT_DISPLAY",
                          PSP_DVT_DESC(PSP_DVT_eDATA_SND_START,
                                       PSP_DVT_dCHANNEL_ID_INST,
                                       PSP_DVT_dPACKET_ID,
                                       PSP_DVT_dNONE),
                          CHANNELID_INSTANCEID(
                                       chan->chanNum,
                                       chan->portNum),
                          chan->nextViop
                          );
#endif /* VPORT_INSTRUMENTATION_ENABLED */

            if (EDMA3_RM_XFER_COMPLETE != status)
            {
                _dundRecover(chan);
            }
            else
            {
#ifdef VPORT_DEBUG
                chan->frameCount++;
#endif  /* #ifdef VPORT_DEBUG */

                /* re-sync tcc with activeEDMARlds, they may be out of sync
                 * after interrupt over-run
                 * e.g. execution is halted at break-point
                 */
                chan->nextEDMARlds = (tcc == chan->tcc[0]) ? 0 : 1;
                offset = chan->nextEDMARlds << 1;

                /* Check if is it too late to update the next param table */
                if (_isItTooLateInISR(chan))
                {
                    /* Too late. Reconfigure channel and start fresh!! */
                    _dundRecover(chan);
                }
                else
                {
                    /* Get current address of Y data EDMA transfer */
                    curParamAddr =
                            chan->hEdmaLinkChanPhyAddr[offset]->srcAddr;

                    /* If software maintained current and next addresses are not
                     * in sync with hardware EDMA parameters then re-sync them
                     */
                    if ((char *)curParamAddr
                            != chan->curViop->frame.iFrm.y1)
                    {
                        /* Before swapping next and current VIOP, just check
                           whether current param address is atleast same as
                           next VIOP */
                        if ((char *)curParamAddr
                                == chan->nextViop->frame.iFrm.y1)
                        {
#ifdef VPORT_DEBUG
                            chan->isrSwapCount++;
#endif  /* #ifdef VPORT_DEBUG */

                            /* Safe to swap current and next VIOP */
                            curViop = chan->curViop;
                            chan->curViop = chan->nextViop;
                            chan->nextViop = curViop;
                        }
#ifdef VPORT_DEBUG
                        else
                        {
                            chan->isrSwapErrCount++;
                        }
#endif  /* #ifdef VPORT_DEBUG */
                    }

                    /* Store current time in timeStamp field */
                    (chan->curViop->timeStamp) = CLK_getltime();

                    /* update the current and next viop pointers */
                    curViop = chan->curViop;
                    chan->curViop = chan->nextViop;
                    chan->startFlag = 1;

                    if (!chan->asyncModeEnable)
                    {
                        if ((viop = (FVID_Frame *)QUE_dequeue(&chan->qIn)) !=
                            (FVID_Frame *)&chan->qIn)
                        {
                            /* queue IS not empty */
                            chan->nextViop = viop;
                        }
                        else
                        {
                            chan->queEmpty = TRUE;
                        }
                    }

                    /* Update the EDMA reload entry */
                    _updateEDMAParamAddress(chan, offset, chan->nextViop);

                    if (curViop != chan->curViop)
                    {
                        if (chan->packetIOM != INV)
                        {
                            /* call the channel's callback function */
                            *(void**)chan->packetIOM->addr = curViop;
                            chan->packetIOM->size = sizeof(FVID_Frame);
                            chan->packetIOM->status = IOM_COMPLETED;
                            if (NORMAL_MODE == chan->driverMode)
                            {
                                /* decrement number of buffers available */
                                chan->numQueBuf--;
                            }

                            chan->cbFxn((Ptr)chan->cbArg, chan->packetIOM);
                            chan->packetIOM = INV;
                        }
                        else
                        {
                            QUE_enqueue(&chan->qOut, curViop);
                        }
                    } /* if (curViop != chan->curViop) */
                }
            }
        } /* if ((chan->status & READY) && tcc == chan->tcc) */
    } /* for (i = 0; i < CSL_VIDEOPORT_PER_CNT; i++) */
}



/**
 *  \brief   enables interrupt for display under run error handling
 *
 *  \param   chanp  [In]  VPORT channel object pointer
 *  \param   irqId  [In]  irq ID at which interrupt occurs
 *
 *  \return  IOM_COMPLETED
 */
static Int _setDundIsr(Ptr chanp, Int irqId)
{
    VPORT_VIntCbParams setVIntParams;

    setVIntParams.vIntCbFxn = (VPORT_IntCallBack)INVCB;

    /* display under run interrupt enabled */
    setVIntParams.vIntMask = VPORT_INT_DUND;

    setVIntParams.irqId = irqId;

    return (_setVIntCb((Ptr)chanp, (Ptr)&setVIntParams));
}



/**
 *  \brief   ISR to handle VPORT interrupts
 *
 *  \param   portNum  [In]  to identify VPORT at which interrupt occurs
 */
static void displayISR(Int portNum)
{
    Int             vpis = chanObjs[portNum].vportRegs->VP_IS;
    Int             mask = vpis & chanObjs[portNum].vIntMask;

#ifdef VPORT_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT(&DVTEvent_Log,
                  "VPORT_DISPLAY",
                  PSP_DVT_DESC(PSP_DVT_eINT_START,
                               PSP_DVT_dCHANNEL_ID_INST,
                               PSP_DVT_dNONE,
                               PSP_DVT_dNONE),
                  CHANNELID_INSTANCEID(
                               0,
                               portNum)
                  );
#endif /* VPORT_INSTRUMENTATION_ENABLED */

    if (chanObjs[portNum].asyncModeEnable)
    {
        if (chanObjs[portNum].interlaced && (mask & VPORT_INT_VINT2) ||
            !chanObjs[portNum].interlaced && (mask & VPORT_INT_VINT1))
        {
            _frmAlmostCompleteIsr(portNum);
        }
    }
    else if (mask && chanObjs[portNum].vIntFxn != INV)
    {
        chanObjs[portNum].vIntFxn(chanObjs[portNum].vIntCbArg, mask);
    }

    /* clear interrupts that has been handled */
    chanObjs[portNum].vportRegs->VP_IS = mask;

    if (vpis & VPORT_INT_DUND)
    {
#ifdef VPORT_DEBUG
        chanObjs[portNum].errCount++;
#endif  /* #ifdef VPORT_DEBUG */

        /* display underrun error handling */
        _dundRecover((Ptr)&chanObjs[portNum]);

        /* Make vpis variable to contain only Underrun interrupt bit set */
        vpis &= VPORT_INT_DUND;

        /* clear display underrun interrupt */
        chanObjs[portNum].vportRegs->VP_IS = vpis;
    }

#ifdef VPORT_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT(&DVTEvent_Log,
                  "VPORT_DISPLAY",
                  PSP_DVT_DESC(PSP_DVT_eINT_END,
                               PSP_DVT_dCHANNEL_ID_INST,
                               PSP_DVT_dNONE,
                               PSP_DVT_dNONE),
                  CHANNELID_INSTANCEID(
                               0,
                               portNum)
                  );
#endif /* VPORT_INSTRUMENTATION_ENABLED */
}



/**
 *  \brief   start video port display operation
 *
 *  \param   chanp   [In]  VPORT channel object pointer
 *
 *  \return  IOM_COMPLETED
 */
static Int _startDisplay(Ptr chanp)
{
    _VPORT_ChanObj  *chan = (_VPORT_ChanObj *)chanp;
    volatile Int    stat;
    volatile Int    i = 0;

    /* enable port and display */
    chan->vportRegs->VP_CTL |= CSL_VPORT_VP_CTL_VPHLT_MASK;
    chan->vportRegs->VD_CTL |= CSL_VPORT_VD_CTL_VDEN_MASK;

    /* wait for display counters and control signals to sync up */
    do
    {
        stat = chan->vportRegs->VD_STAT;
        i++;
    }
    while (!(stat & 0x70000000) && i < 1000000);

    chan->vportRegs->VD_STAT = stat;
    do
    {
        stat = chan->vportRegs->VD_STAT;
        i++;
    }
    while (!(stat & 0x70000000) && i < 1000000);

    chan->vportRegs->VD_STAT = stat;
    chan->vportRegs->VD_CTL &= ~(CSL_VPORT_VD_CTL_BLKDIS_MASK);

    /* enable interrupt */
    chan->vportRegs->VP_IE |= CSL_VPORT_VP_IE_VIE_MASK;
    chan->vportRegs->VP_IS = 0XFFFFFFFF;

    /* Enabling the event */
    PAL_osEvtEnable((Uint32)(CSL_INTC_EVENTID_VP0_INT + chan->portNum));

    chan->status |= _VPORT_STARTED;

#ifdef VPORT_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT(&DVTEvent_Log,
                  "VPORT_DISPLAY",
                  PSP_DVT_DESC(PSP_DVT_eDATA_SND_START,
                               PSP_DVT_dCHANNEL_ID_INST,
                               PSP_DVT_dPACKET_ID,
                               PSP_DVT_dNONE),
                  CHANNELID_INSTANCEID(
                               chan->chanNum,
                               chan->portNum),
                  chan->curViop
                  );
#endif /* VPORT_INSTRUMENTATION_ENABLED */

    return IOM_COMPLETED;
}



/**
 *  \brief   stop video port display operation
 *
 *  \param   chanp   [In]  VPORT channel object pointer
 *
 *  \return  IOM_COMPLETED
 */
static Int _stopDisplay(Ptr chanp)
{
    _VPORT_ChanObj  *chan = (_VPORT_ChanObj *)chanp;
    Int             retVal = IOM_COMPLETED;

    /* block events generation */
    chan->vportRegs->VD_CTL |= (CSL_VPORT_VD_CTL_BLKDIS_MASK);

    /* disable channel */
    chan->vportRegs->VD_CTL &= ~(CSL_VPORT_VD_CTL_VDEN_MASK);

    /* disable interrupt generation in video port level */
    chan->vportRegs->VP_IE &= ~(CSL_VPORT_VP_IE_VIE_MASK);
    chan->vportRegs->VP_IS = 0XFFFFFFFF;

    PAL_osEvtDisable((Uint32)(CSL_INTC_EVENTID_VP0_INT + chan->portNum));

    /* Configure EDMA channel again */
    retVal = _reconfigEDMAchan(chan);

    /* set status as port stopped */
    chan->status &= ~_VPORT_STARTED;

    return retVal;
}



/**
 *  \brief   force recover from FIFO under-run
 *
 *  \param   chanp  [In]  VPORT channel object pointer
 *
 *  \return  IOM_COMPLETED on success
 *           IOM_EALLOC on failure
 */
static Int _dundRecover(Ptr chanp)
{
    _VPORT_ChanObj      *chan = (_VPORT_ChanObj *)chanp;
    Int                 i;
    Int                 stat;
    Int                 retVal = IOM_COMPLETED;

    /* disable under run interrupt */
    chan->vportRegs->VP_IE &= ~(CSL_VPORT_VP_IE_DUND_MASK);

    /* block display events */
    chan->vportRegs->VD_CTL |= CSL_VPORT_VD_CTL_BLKDIS_MASK;

    /* clear any pending under-run interrupt */
    chan->vportRegs->VP_IS = (CSL_VPORT_VP_IS_DUND_MASK);

    /* Configure EDMA channel again */
    retVal = _reconfigEDMAchan(chan);

    chan->vportRegs->VD_CTL |= CSL_VPORT_VD_CTL_VDEN_MASK;
    /* wait for display counters and control signals to sync up */
    do
    {
        stat = chan->vportRegs->VD_STAT;
        i++;
    }
    while (!(stat & 0x70000000) && i < 1000000);
    chan->vportRegs->VD_STAT = stat;

    do
    {
        stat = chan->vportRegs->VD_STAT;
        i++;
    }
    while (!(stat & 0x70000000) && i < 1000000);

    chan->vportRegs->VD_CTL &= ~(CSL_VPORT_VD_CTL_BLKDIS_MASK);
    /* re-enable under run interrupt */
    chan->vportRegs->VP_IE |= CSL_VPORT_VP_IE_DUND_MASK;

    return retVal;
}



/**
 *  \brief   get last active line
 *
 *  \param   params  [In]  VPORT display parameters
 *
 *  \return  last active line that has been displayed
 */
static Int _getlastActiveLine(VPORTDIS_Params *params)
{
    int lastActiveLine = -1;

    switch (params->fldOp)
    {
        case VPORT_FLDOP_FRAME:
        case VPORT_FLDOP_FLD2:
            lastActiveLine = params->vBlnkYStopFld2 + params->imgVOffsetFld2 +
                             params->imgVSizeFld2;
            break;

        case VPORT_FLDOP_FLD1:
        case VPORT_FLDOP_PROGRESSIVE:
            lastActiveLine = params->vBlnkYStopFld1 + params->imgVOffsetFld1 +
                             params->imgVSizeFld1;
        break;
    }

    return lastActiveLine;
}



/**
 *  \brief   get the number of pending requests in queue In
 *
 *  \param   chanp     [In]  VPORT channel object pointer
 *  \param   args      [Out] number of pending requests
 *
 *  \return  IOM_COMPLETED
 */
static Int _getNumPendingIORqsts(Ptr chanp, Ptr args)
{
    _VPORT_ChanObj  *chan = (_VPORT_ChanObj *)chanp;
    QUE_Handle      nextElem = &chan->qIn;
    int             i = 0;

    while ((nextElem = QUE_next(nextElem)) != &chan->qIn)
    {
        i++;
    }

    if (chan->curViop != chan->nextViop)
    {
        i++;
    }

    *(int*)args = i;

    return IOM_COMPLETED;
}



/**
 *  \brief   get current channel parameters
 *
 *  \param   chanp      [In]  VPORT channel object pointer
 *  \param   currParams [Out] pointer to current channel parameters structure
 *
 *  \return  IOM_COMPLETED
 */
static Int _getChanParams(Ptr chanp, VPORT_CurrParams *currParams)
{
    _VPORT_ChanObj *chan = (_VPORT_ChanObj *)chanp;

    /* Return current parameters from global channel object */
    currParams->mode = chan->mode;
    currParams->fldOp = chan->fldOp;
    currParams->scale = chan->scale;
    currParams->resmpl = chan->resmpl;
    currParams->yPitch = chan->yPitch;
    currParams->cPitch = chan->cPitch;
    currParams->numLines = chan->numLines;
    currParams->numPixels = chan->numPixels;
    currParams->bufSize = chan->bufSz;
    currParams->mergeFlds = chan->mergeFlds;

    return IOM_COMPLETED;
}



/**
 *  \brief   setup video port interrupt call-back
 *
 *  \param   chanp   [In]  VPORT channel object pointer
 *  \param   args    [In]  VPORT interrupt callback parameters
 *
 *  \return  IOM_COMPLETED
 */
static Int _setVIntCb(Ptr chanp, Ptr args)
{
    _VPORT_ChanObj      *chan = (_VPORT_ChanObj *)chanp;
    VPORT_VIntCbParams  *vIntCbParams = (void *)args;
    Int                 mask = vIntCbParams->vIntMask;
    Uns                 vif2 = 0, vInt2 = 0, vif1 = 0, vInt1 = 0;
    ECM_Attrs           ecmattrs = ECM_ATTRS;
    Uns                 cookie = 0;

    chan->vIntMask = mask;
    /* check to see if vertical interrupt is enabled */
    if (!chan->asyncModeEnable)
    {
        if (mask & VPORT_INT_VINT1)
        {
            vif1 = 1;
            vInt1 = vIntCbParams->vIntLine;
        }
        if (mask & VPORT_INT_VINT2)
        {
            vif2 = 1;
            vInt2 = vIntCbParams->vIntLine;
        }
    }
    else
    {
        return IOM_EINUSE;
    }

    chan->vportRegs->VD_VINT = VP_VDVINT_RMK(vif2, vInt2, vif1, vInt1);

    if (mask)
    {
        mask |= CSL_VPORT_VP_IE_VIE_MASK; /* turn on video port interrupt */

        PAL_osEvtDisable((Uint32)(CSL_INTC_EVENTID_VP0_INT + chan->portNum));
        /* Enable the interrupt */
        ecmattrs.unmask = 1u;

        /* setup arg such that isr will get 'chan' as arg */
        ecmattrs.arg = (Arg)chan->portNum;

        /* Disabling the interrupts */
        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

        /* Plug the ISR with interrupt (event) number */
        ECM_dispatchPlug((Uns)(CSL_INTC_EVENTID_VP0_INT + chan->portNum),
                         (ECM_Fxn)displayISR,
                         &ecmattrs);

        /* Enabling the HWI_ID where corresponding ECM group is configured */
        C64_enableIER(1 << _IRQ_ID);

        /* Enabling the interrupts */
        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
    }

    chan->vportRegs->VP_IE |= mask;  /* register write */
    chan->vIntFxn = vIntCbParams->vIntCbFxn;
    chan->vIntCbArg = vIntCbParams->cbArg;

    return IOM_COMPLETED;
}



/**
 *  \brief   ISR triggred in Async mode
 *
 *  \param   portNum  [In]  to identify VPORT at which interrupt occurs
 *
 *  \return  IOM_COMPLETED
 */
static Int _frmAlmostCompleteIsr(Int portNum)
{
    _VPORT_ChanObj      *chan = &chanObjs[portNum];
    FVID_Frame          *viop, *viopOut;
    Uint32              gie;
    Int                 offset = chan->nextEDMARlds << 1;
    Int                 retVal = IOM_COMPLETED;

    PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &gie);

    if ((!chan->startFlag) || (chan->asyncCallBackFxn == INV))
    {
        (retVal = IOM_EBADIO);
    }
    else
    {
        if ((viop = (FVID_Frame *)QUE_dequeue(&chan->qOut)) == (FVID_Frame *)
            &chan->qOut)
        {
            /* queue enpty */
            viop = INV;
        }

        viopOut = viop;
        if (chan->pullDownMode == VPORTDIS_PULLDOWN_24P_30I)
        {
            if (chan->asyncFrameCt < 2)
            {
                /* 0, 1 */
                chan->asyncCallBackFxn(chan->asyncCbArg,
                                       chan->asyncFrameCt,
                                       &viopOut);
                if (viop == viopOut && viopOut != INV)
                {
                    QUE_enqueue(&chan->qOut, viopOut);
                }
                else if (viopOut != INV)
                {
                    chan->nextViop = viopOut;

                    /* now modify the EDMA rld entries */
                    if (!(chan->mode & _VPORT_MASK_RAW))
                    {
                        chan->hEdmaLinkChanPhyAddr[offset]->srcAddr
                                    = (unsigned int)viop->frame.iFrm.y1;
                        chan->hEdmaLinkChanPhyAddr[offset + 1]->srcAddr
                                    = (unsigned int)viop->frame.iFrm.y2;

                        chan->hEdmaLinkChanPhyAddr[offset + 4]->srcAddr
                                    = (unsigned int)viop->frame.iFrm.cb1;
                        chan->hEdmaLinkChanPhyAddr[offset + 5]->srcAddr
                                    = (unsigned int)viop->frame.iFrm.cb2;

                        chan->hEdmaLinkChanPhyAddr[offset + 8]->srcAddr
                                    = (unsigned int)viop->frame.iFrm.cr1;
                        chan->hEdmaLinkChanPhyAddr[offset + 9]->srcAddr
                                    = (unsigned int)viop->frame.iFrm.cr2;
                    }
                    else
                    {
                        chan->hEdmaLinkChanPhyAddr[offset]->srcAddr
                                    = (unsigned int)viop->frame.riFrm.buf1;
                        chan->hEdmaLinkChanPhyAddr[offset + 1]->srcAddr
                                    = (unsigned int)viop->frame.riFrm.buf2;
                    }

                    chan->asyncFrameCt++;
                }
                else
                {
                    chan->asyncFrameCt = 0;
                }
            }
            else if (chan->asyncFrameCt < 4)
            {
                /* 2, 3 */
                chan->asyncCallBackFxn(chan->asyncCbArg,
                                       chan->asyncFrameCt,
                                       &viopOut);
                if (viop == viopOut && viopOut != INV)
                {
                    QUE_enqueue(&chan->qOut, viopOut);
                }
                else if (viopOut != INV)
                {
                    chan->nextViop = viopOut;
                    if (!(chan->mode & _VPORT_MASK_RAW))
                    {
                        chan->hEdmaLinkChanPhyAddr[offset + 1]->srcAddr
                                    = (unsigned int)viop->frame.iFrm.y2;
                        chan->hEdmaLinkChanPhyAddr[offset + 5]->srcAddr
                                    = (unsigned int)viop->frame.iFrm.cb2;
                        chan->hEdmaLinkChanPhyAddr[offset + 9]->srcAddr
                                    = (unsigned int)viop->frame.iFrm.cr2;
                    }
                    else
                    {
                        chan->hEdmaLinkChanPhyAddr[offset + 1]->srcAddr
                                    = (unsigned int)viop->frame.riFrm.buf2;
                    }

                    chan->asyncFrameCt++;
                }
                else
                {
                    chan->asyncFrameCt = 0;
                }
            }
            else
            {
                /* 4 */
                chan->asyncFrameCt = 0;
                QUE_enqueue(&chan->qOut, viop);
            }
        }
        else if (chan->pullDownMode == VPORTDIS_PULLDOWN_24P_60P)
        {
            if (chan->asyncFrameCt == 0 || chan->asyncFrameCt == 2)
            {
                chan->asyncCallBackFxn(chan->asyncCbArg,
                                       chan->asyncFrameCt,
                                       &viopOut);
                if (viop == viopOut && viopOut != INV)
                {
                    QUE_enqueue(&chan->qOut, viopOut);
                }
                else if (viopOut != INV)
                {
                    chan->nextViop = viopOut;

                    if (!(chan->mode & _VPORT_MASK_RAW))
                    {
                        chan->hEdmaLinkChanPhyAddr[offset]->srcAddr
                                    = (unsigned int)viop->frame.iFrm.y1;
                        chan->hEdmaLinkChanPhyAddr[offset + 4]->srcAddr
                                    = (unsigned int)viop->frame.iFrm.cb1;
                        chan->hEdmaLinkChanPhyAddr[offset + 8]->srcAddr
                                    = (unsigned int)viop->frame.iFrm.cr1;
                    }
                    else
                    {
                        chan->hEdmaLinkChanPhyAddr[offset]->srcAddr
                                    = (unsigned int)viop->frame.riFrm.buf1;
                    }
                }
                else
                {
                    chan->asyncFrameCt = 0;
                }
                chan->asyncFrameCt++;
            }
            else if (chan->asyncFrameCt < 5)
            {
                QUE_enqueue(&chan->qOut, viop);
                chan->asyncFrameCt++;

                if (chan->asyncFrameCt == 5)
                {
                    chan->asyncFrameCt = 0;
                }
            }
            else
            {
                QUE_enqueue(&chan->qOut, viop);
                chan->asyncFrameCt = 0;
            }
        }
        else
        {
            /* not in pull-down modes */
            chan->asyncCallBackFxn(chan->asyncCbArg,
                                   chan->asyncFrameCt,
                                   &viopOut);
            if (viop == viopOut && viopOut != INV)
            {
                QUE_enqueue(&chan->qOut, viopOut);
            }
            else if (viopOut != INV)
            {
                chan->nextViop = viopOut;

                /* Update the EDMA reload entry */
                retVal = _updateEDMAParamAddress(chan, offset, viopOut);
            }
        }
    }

    PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, gie);

    if (retVal != IOM_COMPLETED)
    {
        retVal = IOM_EALLOC;
    }

    return retVal;
}


/******************************************************************************/
/* End of file                                                                */
/******************************************************************************/
