/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2007 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file   vport.h file
 *
 *  This file contains defines and structures common for capture and display
 *
 *  <b>Compiler Switch Used</b>
 *
 *  <b>_DEBUG</b><br>
 *   This is used as a flag to compiler whether to include the debug statmements
 *   inserted in the code into the final image. This flag helps tp build DEBUG
 *   image of the program. For RELEASE images this is not passed to the compiler.
 *   Reference : Project file for vport.vpfe etc
 *
 *  <b>CHIP_XXXX</b><br>
 *   The CSL layer is written in a common file for all the variants of a SOC.
 *   This flag differentiates the variant we are compiling for, for eg CHIP_DM648,
 *   and the CSL definitions for that variant appropriately gets defined for regs
 *   base addresses, num of ports of a peripheral etc.
 *   Reference : soc\DM648\dsp\inc\csl_vport_vphal.h
 *
 *  <b>VPORT_INSTRUMENTATION_ENABLED</b><br>
 *   This flag is passed to the compiler to include the instrumentation code parts
 *   into the final image/lib of the program. This helps build the iRelease/iDebug
 *   versions of the image/lib with a common code base
 *
 *  <b>VPORT_DEBUG</b><br>
 *   (un)definition of this macro in vport.h disables/enables selectively the
 *   debug statements for vport to be included into the final image/lib
 *
 */

#ifndef _VPORT_H
#define _VPORT_H

#ifdef __cplusplus
extern "C" {
#endif

#include <std.h>
#include <ti/sdo/pspdrivers/drivers/vport/edc.h>

#ifndef INV
/** \brief Define for invalid parameter */
  #define INV    ((void*)(-1))
#endif /* INV */

/** \brief IRQ id for capture/display ISRs, driver will register capture overrun
 * or display underrun interrupts with below defined Interrupt ID.
 * The value of this depends on which ECM block the VPORT ISR fall.
 * Currently it is assumed that in the application, ECM are mapped from IRQ 7.
 * And since VPORT interrupts fall in the second block, 8 is used as IRQ ID
 */
#define _IRQ_ID                         (8)

/**
 *  \brief maximum number of video frame buffers, this define is used only in
 *  LEGACY Mode
 */
#define VPORT_MAX_NUM_FRMBUFS           (10)

/** \brief fldOp: indicates to capture/display only field 1 data */
#define VPORT_FLDOP_FLD1                (0x01)
/** \brief fldOp: indicates to capture/display only field 2 data */
#define VPORT_FLDOP_FLD2                (0x02)
/** \brief fldOp: indicates to capture/display complete frame */
#define VPORT_FLDOP_FRAME               (0x04)
/** \brief fldOp: progressive capture/display */
#define VPORT_FLDOP_PROGRESSIVE         (0x05)

/** \brief VPORT operation mode: BT656_8BIT */
#define VPORT_MODE_BT656_8BIT           (0x00)
/** \brief VPORT operation mode: BT656_10BIT */
#define VPORT_MODE_BT656_10BIT          (0x01)
/** \brief VPORT operation mode: RAW_8BIT */
#define VPORT_MODE_RAW_8BIT             (0x02)
/** \brief VPORT operation mode: RAW_10BIT */
#define VPORT_MODE_RAW_10BIT            (0x03)
/** \brief VPORT operation mode: YCBCR_8BIT */
#define VPORT_MODE_YCBCR_8BIT           (0x04)
/** \brief VPORT operation mode: YCBCR_10BIT */
#define VPORT_MODE_YCBCR_10BIT          (0x05)
/** \brief VPORT operation mode: RAW_16BIT */
#define VPORT_MODE_RAW_16BIT            (0x06)
/** \brief VPORT operation mode: RAW_20BIT */
#define VPORT_MODE_RAW_20BIT            (0x07)

/**
 *  \brief VPORT external PIN POLARITY HIGH, indicates polarity of external
 *  control signal
 */
#define VPORT_POLARITY_ACTIVE_HIGH      (0x00)
/**
 *  \brief VPORT external PIN POLARITY LOW, indicates polarity of external
 *  control signal
 */
#define VPORT_POLARITY_ACTIVE_LOW       (0x01)

/** \brief video port HORIZONTAL scaling mode disable */
#define VPORT_SCALING_DISABLE           (0x00)
/**
 *  \brief video port HORIZONTAL scaling mode enable
 *  capture: 1/2x scale        display: 2x scale */
#define VPORT_SCALING_ENABLE            (0x01)

/** \brief video port chroma resampling mode disable */
#define VPORT_RESMPL_DISABLE            (0x00)
/**
 *  \brief video port chroma resampling mode enable
 *  capture : 4:2:2 => 4:2:0    display: 4:2:0 => 4:2:2
 */
#define VPORT_RESMPL_ENABLE             (0x01)

/**
 *  \brief  video port field operation: field 1 and field 2 will be stored
 *  separately
 */
#define VPORT_FLDS_SEPARATED            (0)
/**
 *  \brief  video port field operation: field 1 and field 2 will be stored in
 *  merged pattern i.e. one line of field 1, one line of field 2
 */
#define VPORT_FLDS_MERGED               (1)

/* Video mini-driver control command
 * The following commands are common for both capture and display operation
 */
/** \brief video port reset command, Not Implemented */
#define VPORT_CMD_RESET                 (0x10000001)
/** \brief configure video port, Not Implemented */
#define VPORT_CMD_CONFIG_PORT           (0x10000002)
/**
 *  \brief configure video channel
 *  This command is used to change channel parameters at runtime
 */
#define VPORT_CMD_CONFIG_CHAN           (0x10000003)
/** \brief start video port operation   */
#define VPORT_CMD_START                 (0x10000004)
/** \brief stop video port operation    */
#define VPORT_CMD_STOP                  (0x10000005)
/** \brief set video channel error call-back function */
#define VPORT_CMD_SET_VINTCB            (0x10000006)
/** \brief force the recovery of display under-run */
#define VPORT_CMD_DUND_RECOVER          (0x10000007)
/** \brief force the recovery of capture over-run */
#define VPORT_CMD_COVR_RECOVER          (0x10000008)
/** \brief get number of pending I/O requests in the driver queue */
#define VPORT_CMD_GET_NUM_IORQST_PENDING    (0x10000009)
/** \brief get the current configuration parameters of driver */
#define VPORT_CMD_GET_PARAMS            (0x1000000A)
/** \brief User defined command */
#define VPORT_CMD_USER                  (0x20000000)

/**
 *  \brief base of control commands for external device that is stack under
 *  video driver
 */
#define VPORT_CMD_EDC_BASE              (0x80000000)

/**
 *  \brief Error call-back function prototype
 *  Only one video port global interrupt call-back function can be defined
 *  for each channel/device pair. Multiple calls to setup the call-back
 *  function overwrite the previous setup.
 */
typedef void (*VPORT_IntCallBack)(Arg id, Arg mask);

/** \brief capture fifo over-run interrupt mask */
#define VPORT_INT_COVR                  (0x0002)
/** \brief capture complete interrupt mask */
#define VPORT_INT_CCMP                  (0x0004)
/** \brief synchronization error interrupt mask */
#define VPORT_INT_SERR                  (0x0008)
/** \brief field 1 vertical interrupt mask */
#define VPORT_INT_VINT1                 (0x0010)
/** \brief field 2 vertical interrupt mask */
#define VPORT_INT_VINT2                 (0x0020)
/** \brief short field detected interrupt mask */
#define VPORT_INT_SFD                   (0x0040)
/** \brief long field detected interrupt mask */
#define VPORT_INT_LFD                   (0x0080)
/** \brief system time clock interrupt mask */
#define VPORT_INT_STC                   (0x0400)
/** \brief clock tick interrupt mask */
#define VPORT_INT_TICK                  (0x0800)
/** \brief display fifo under-run mask */
#define VPORT_INT_DUND                  (0x1000)
/** \brief display complete mask */
#define VPORT_INT_DCMP                  (0x2000)
/** \brief display complete not acknowledged mask */
#define VPORT_INT_DCNA                  (0x4000)

/**
 *  \brief  VPORT interrupt callback Parameters Structure
 *  parameter structure to setup video port call-back functions.
 */
typedef struct VPORT_VIntCbParams {
    Int                 cbArg;
    /**< Call-back argument is for the application to identify which channel or
     * device causes the interrupt
     */
    VPORT_IntCallBack   vIntCbFxn;
    /**< Pointer to the video port interrupt call-back function */
    Uint16              vIntMask;
    /**< Interrupt mask, All masks OR together to indicate errors that are
     * handled by the call-back
     */
    Uint16              vIntLine;
    /**< Line number where vertical interrupt occurs, don't care otherwise. */
    Int                 irqId;
    /**< Irq id at which interrupt occurs */
} VPORT_VIntCbParams;

/**
 *  \brief  VPORT Init Parameters Structure
 *  A statically defined EDC function table is passed to the _mdBindDev()
 *  function via the "devParam" argument. This argument must point to a
 *  type of data structure that is defined below.
 */
typedef struct VPORT_PortParams {
    Bool             dualChanEnable;
    /**< Dual channel mode to select both channels of VPORT or single channel
     * selection
     */
    Uns              vc1Polarity;
    /**< Vport control pin 1 polarity. Available values for this field are
     * defined above with VPORT external PIN POLARITY title.
     */
    Uns              vc2Polarity;
    /**< Vport control pin 2 polarity. Available values for this field are
     * defined above with VPORT external PIN POLARITY title
     */
    Uns              vc3Polarity;
    /**< Vport control pin 3 polarity. Available values for this field are
     * defined above with VPORT external PIN POLARITY title
     */
    EDC_Fxns*        edcTbl[2];
    /**< Function tables of edc modules for both channels. For single channel
     * operation assign other pointer with INV(-1)
     */
} VPORT_PortParams;

/**
 *  \brief  VPORT Current status Parameters Structure
 *  This structure contains current port parameters when application asks
 *  for current driver status
 */
typedef struct {
    Int32               mode;
    /**< VPORT operation mode. Available values for this field are defined
     * above with VPORT operation mode title
     */
    Int32               fldOp;
    /**< Field or frame operation. Available values for this field are defined
     * above with fldop title.
     */
    Bool                scale;
    /**< Scaling enabled or disabled. Available values for this field are
     * defined above with video port HORIZONTAL scaling mode title.
     */
    Bool                resmpl;
    /**< Resampling enabled or disabled. Available values for this field are
     * defined above with video port HORIZONTAL chroma resampling mode title.
     */
    Uint32              yPitch;
    /**< Luminance pitch value */
    Uint32              cPitch;
    /**< Chrominance pitch value */
    Uint32              numLines;
    /**< Total number of lines per frame */
    Uint32              numPixels;
    /**< Number of pixel in one line */
    Uint32              bufSize;
    /**< Size of one FVID frame buffer */
    Bool                mergeFlds;
    /**< Field operation, Flag to indicate whether fields are merged or
     * separated.  Available values for this field are defined above with
     * video port field operation title
     */
} VPORT_CurrParams;

#ifdef __cplusplus
}
#endif /* extern "C" */

#endif /* _VPORT_H */

/******************************************************************************/
/* End of file                                                                */
/******************************************************************************/
