/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2007 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file   vportcap.h file
 *
 *  This file contains channel object structure for capture driver
 */

#ifndef _VPORTCAP_H
#define _VPORTCAP_H

#ifdef __cplusplus
extern "C" {
#endif

#include <iom.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/fvid.h>
#include <ti/sdo/edma3/drv/edma3_drv.h>

/********************************************************/
/* video port capture field inverse enable              */
/********************************************************/
/**
 *  \brief  video port capture field inverse disable
 *  select whether detected field needs to be inverted. this is only used when
 *  exc is enabled and line counter is reset at the start of vertical blanking
 *  period and when FID signal is toggled during the vertical blanking period,
 *  e.g., in NTSC
 */
#define VPORTCAP_FINV_DISABLE           (0x00)

/** \brief  video port capture field inverse enable */
#define VPORTCAP_FINV_ENABLE            (0x01)

/***********************************************/
/* video port capture line counter reset mode  */
/**********************************************/
/** \brief   video port capture line counter reset on vblank */
#define VPORTCAP_VRST_START_VBLK        (0x00)
/** \brief   video port capture line counter reset on vsync */
#define VPORTCAP_VRST_START_VSYNC       (0x00)
/**
 *  \brief   video port capture line counter reset at inactive edge, used with
 *  embedded sync mode
 */
#define VPORTCAP_VRST_EAV_V1            (0x00)

/** \brief  line counter reset at the end of vertical blank */
#define VPORTCAP_VRST_END_VBLK          (0x01)
/** \brief  video port capture line counter reset on vsync */
#define VPORTCAP_VRST_END_VSYNC         (0x01)
/**
 *  \brief  video port capture line counter reset at inactive edge, used with
 *  embedded sync mode
 */
#define VPORTCAP_VRST_EAV_V0            (0x01)

/***********************************************/
/* video port capture pixel counter reset mode */
/***********************************************/
/** \brief  pixel counter reset at the EAV code, used with embedded sync mode */
#define VPORTCAP_HRST_EAV               (0x00)
/** \brief  pixel counter reset at the start of horizontal blank period */
#define VPORTCAP_HRST_START_HBLK        (0x00)
/** \brief  pixel counter reset at the start of hsync signal */
#define VPORTCAP_HRST_START_HSYNC       (0x00)

/** \brief  pixel counter reset at the SAV code, used with embedded sync mode */
#define VPORTCAP_HRST_SAV               (0x01)
/** \brief  pixel counter reset at the end of horizontal blank period */
#define VPORTCAP_HRST_END_HBLK          (0x01)
/** \brief  pixel counter reset at the end of hsync signal */
#define VPORTCAP_HRST_END_HSYNC         (0x01)

/***********************************************/
/* video port capture field detect method      */
/***********************************************/
/* used only when exc is enabled */
/** \brief  field detect logic off */
#define VPORTCAP_FLDD_DISABLE           (0x00)
/** \brief  field detect logic on using EAV or FID input for field detection */
#define VPORTCAP_FLDD_ENABLE            (0x01)

/**
 *  \brief  video port capture external control enable
 *  In capture operation, embedded EAV/SAV code is used when exc disabled.
 */
#define VPORTCAP_EXC_DISABLE            (0x00)
/**
 *  \brief  video port external control disable
 *  HCTL/VCTL/FID is used when it is enabled. This mode is for some legacy video
 *  decoders that don't insert embedded EAV/SAV code
 */
#define VPORTCAP_EXC_ENABLE             (0x01)

/** \brief  VPORT capture 10-Bit mode zero extended Packing format */
#define VPORTCAP_BPK_10BIT_ZERO_EXTENDED (0x00)
/** \brief  VPORT capture 10-Bit mode sign extended Packing format */
#define VPORTCAP_BPK_10BIT_SIGN_EXTENDED (0x01)
/** \brief  VPORT capture 10-Bit mode dense Packing format */
#define VPORTCAP_BPK_10BIT_DENSE         (0x02)

/** \brief An optional setup synchronization enable bit enabled */
#define VPORTCAP_SSE_ENABLE             (0x01)
/** \brief An optional setup synchronization enable bit disabled */
#define VPORTCAP_SSE_DISABLE            (0x00)

/** \brief  command to enable call-back after specified number of lines */
#define VPORTCAP_CMD_SET_LINE_INT           (VPORT_CMD_USER + 1)
/** \brief  command to get number of lines transferred by EDMA */
#define VPORTCAP_CMD_GET_NUMLINES_CAPTURED  (VPORT_CMD_USER + 2)

/**
 *  \brief VPORT Capture Params Structure
 *  The parameter structure below is for BT656 and Y/C capture mode only.
 */
typedef struct {
    Int                 cmode;
    /**< Capture mode settings. Available values for this field are defined in
     * "vport.h" file with VPORT operation mode title.
     */

    Int                 fldOp;
    /**< Field & frame operation. Available values for this field are defined in
     * "vport.h" file with fldOp title
     */

    /* bit 8-15 */
    Int                 scale;
    /**< 1/2 scaling, available values for this field are defined in "vport.h"
     * file with video port HORIZONTAL scaling mode title.
     */

    Int                 resmpl;
    /**< Chroma resampling, available values for this field are defined in
     * "vport.h" file with Video port chroma resampling mode title
     */

    Int                 bpk10Bit;
    /**< VPORT capture 10-Bit mode. Available values for this field are defined
     * above.
     */

    /* bit 16-23 */
    Int                 hCtRst;
    /**< Video port capture pixel counter reset mode. Available values for this
     * field are defined above.
     */

    Int                 vCtRst;
    /**< Video port capture line counter reset mode. Available values for this
     * field are defined above
     */

    Int                 fldDect;
    /**< Enable whether to use FID input or field detection logic based on the
     * timing relation of hsync and vsync. Available values for this field are
     * defined in "vport.h" file.
     */

    Int                 extCtl;
    /**< Enable external timing control, available values for this field are
     * defined above
     */

    Int                 fldInv;
    /**< Enable inversion of the detected field. Available values for this field
     * are defined defined above.
     */

    Uint16              fldXStrt1;
    /**< Starting pixel number of field 1, must be greater than or equal to
     * zero.
     */

    Uint16              fldYStrt1;
    /**< Starting line number of field 1, must be greater than or equal to
     * one.
     */

    Uint16              fldXStrt2;
    /**< Starting pixel number of field 2, must be greater than or equal to
     * zero.
     */

    Uint16              fldYStrt2;
    /**< Starting line number of field 2, must be greater than or equal to
     * one.
     */

    Uint16              fldXStop1;
    /**< The last captured pixel of field 1, must be greater than or equal
     * to fldXStrt1.
     */

    Uint16              fldYStop1;
    /**< The last captured line of field 1, must be greater than or equal to
     * fldYStrt1.
     */

    Uint16              fldXStop2;
    /**< The last captured pixel of field 2, must be greater than or equal to
     * fldXStrt2.
     */

    Uint16              fldYStop2;
    /**< The last captured line of field 2, must be greater than or equal to
     * fldYStrt2.
     */

    Uint16              thrld;
    /**< Video FIFO threshold value, indicates number of double words to
     * generate DMA events
     */

    /* Frame buffer settings */
    Int                 numFrmBufs;
    /**< Number of frame buffers that the driver allocates. For normal mode
     * operation numFrmBufs should be 0 while for legacy mode operation
     * numFrmBufs value should be between minimum and maximum number buffers.
     */

    Int                 alignment;
    /**< Frame buffer alignment used by driver while allocating memory for video
     * frame buffer
     */

    Int                 mergeFlds;
    /**< Indicates whether to interleave data of the two fields in memory or
     * just store them separated. Available values for this field are defined in
     * "vport.h" file.
     */

    Int                 segId;
    /**< Memory segment ID, used by driver to allocate video frame buffer */

    Bool                autoSyncEnable;
    /**< Re-sync tcc with activeEDMARlds they may be out of sync after interrupt
     * over-run e.g. execution is halted at break-point
     */

    EDMA3_DRV_Handle    hEdma;
    /**< EDMA driver handle */
} VPORTCAP_Params;

/**
 *  \brief RAW capture mode Structure
 *  The parameter structure below is for raw capture mode only. This structure
 *  should be used at the time of channel creation for RAW mode capture.
 */
typedef struct {
    Int                 cmode;
    /**< Capture mode settings. Available values for this field are defined in
     * "vport.h" file with VPORT operation mode title.
     */

    Int                 bpk10Bit;
    /**< VPORT capture 10-Bit mode, available values for this field are defined
     * above
     */

    Bool                startupSyncEnable;
    /**< Startup Sync Enable (SSE). If the SSE bit is set, when the capture
     * first begins, the capture does not start until two vertical blanking
     * intervals are counted. This allows the video port to synchronize its
     * capture to the top of a frame when first started.
     */

    Uint16              blankperiod;
    /**< Minimum time for CAPEN signal to stay inactive before interpret it as
     * vertical blanking period.
     */

    Uint16              lineSz;
    /**< Number of pixels per line */

    Uint16              numLines;
    /**< Number of lines per frame */

    Uint16              thrld;
    /**< Video FIFO threshold value, indicates number of double words to
     * generate DMA events
     */

    /* Frame buffer settings */
    Int                 numFrmBufs;
    /**< Number of frame buffers that the driver allocates. For normal mode
     * operation numFrmBufs should be 0 while for legacy mode operation
     * numFrmBufs value should be between minimum and maximum number of buffers.
     */

    Int                 alignment;
    /**< Frame buffer alignment used by driver while allocating memory for video
     * frame buffer
     */

    Int                 segId;
    /**< Memory segment ID, used by driver to allocate video frame buffer */

    EDMA3_DRV_Handle    hEdma;
    /**< EDMA driver handle */
} VPORTCAP_ParamsRaw;

/**
 *  \brief Structure for number of lines captured by VPORT and its frame
 *  pointer
 */
typedef struct {
    FVID_Frame      *frmPtr;
    /**< FVID_Frame pointer which is currently used by EDMA */
    Uint32          numLines;
    /**< number of lines captured by Video Port (per field) */
} VPORTCAP_LinesCapturedInfo;

/**
 *  \brief Application call-back function for line interrupt enabled using
 *  VPORTCAP_CMD_SET_LINE_INT ioctl
 */
typedef void (*VPORTCAP_LinesCapturedCB)(Int id,
                                    VPORTCAP_LinesCapturedInfo *capLineInfo);

/**
 *  \brief parameter structure to enable line interrupt, this is similar to
 *  vertical interrupt enable, but its callback will also provide frame buffer
 *  pointer to application
 */
typedef struct {
    VPORTCAP_LinesCapturedCB callBackFxn;
    /**< callback function pointer to be called after vIntFld1 or vIntFld2 lines
      * are captured
      */
    Int         vIntFld1;
    /**< line number at which interrupt occurs in field1 */
    Int         vIntFld2;
    /**< line number at which interrupt occurs in field2 */
    Int         cbArg;
    /**< Call-back argument is for the application to identify which channel or
     * device causes the interrupt
     */
    Bool        errCheckEnable;
    /**< Enable short field detect error checking in EDMA ISR. This should be
     * set to TRUE to handle short field detect error. Generally SFD error
     * occurs when cable is plugged out/in or when driver is configured for PAL
     * and a NTSC input is connected.
     */
} VPORTCAP_LineIntParam;


/** \brief VPORT capture function pointer table */
extern IOM_Fxns VPORTCAP_Fxns;

#ifdef __cplusplus
}
#endif /* extern "C" */

#endif /* _VPORTCAP_H */

/******************************************************************************/
/* End of file                                                                */
/******************************************************************************/
