/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 * \file   pal_osCache.c
 *
 * \brief  Cache Services Source File for BIOS
 *
 *  This file implements Cache functionalities.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author     PSP group
 *  \version    0.3 - Modified for DM64G
 */

#include <std.h>
#include <bcache.h>

#include <ti/sdo/pspdrivers/pal_os/pal_oscache.h>


/**
 * \brief Cache Wait Flag
 */
#define CACHE_WAIT      (TRUE)
/**
 * \brief Compilation Flag to enable Cache Check
 */
#define ENABLE_CHECK_CACHE

/**
 *  \brief Check the cache line
 *
 *  This function checks the address is aligned on cache line boundary
 *
 *  \param  mem_start_ptr   [IN]  Starting adress of memory
 *  \return  0 if aligned properly<br>
 *           1 if not aligned on cache line length
 */
static __inline Int checkCacheLine (Uint32 mem_start_ptr)
{
    Int value = 0;

    if((mem_start_ptr & (PAL_OS_CACHE_LINE_LEN - 1))   !=  0)
    {
        value = 1;
    }

    return value;
}

/**
 *  \brief Clean the Cache
 *
 *  This function clean the D Cache in MVA way.
 *
 *  \param  cache           [IN]  Cache address space
 *  \param  mem_start_ptr   [IN]  Starting adress of memory. Please note
 *                                that this should be 32 bytes alinged.
 *  \param  num_bytes       [IN]   Length of buffer
 */
void PAL_osCacheFlush(PAL_osCacheMemAddrSpace   cache,
                        Uint32 mem_start_ptr ,
                        Uint32 num_bytes)
{
    /* To avoid compiler warning */
    cache = cache;
#ifdef CACHE_ENABLED

#ifdef ENABLE_CHECK_CACHE
    /* Verify whether the start address is 32-bit aligned or not */
    checkCacheLine(mem_start_ptr);
#endif /* ENABLE_CHECK_CACHE */

    BCACHE_wb ((Ptr)mem_start_ptr,
                        num_bytes,
                        CACHE_WAIT);
#endif /* #ifdef CACHE_ENABLED */
}


/**
 * \brief   PAL OS Cache Invalidate
 *
 * This function invalidates the D cache.
 *
 * \note   cacheAddrSpace is not cared in this API
 *
 * \param  cache            [IN]        Cache address space
 * \param  mem_start_ptr    [IN]        Starting adress of memory. Please note
 *                                      this should be 32 bytes alinged.
 * \param  num_bytes        [IN]        Length of buffer
 *
 * \return  None
 */
/* ARGSUSED */
void PAL_osCacheInvalidate(PAL_osCacheMemAddrSpace  cache,
                           Uint32                   mem_start_ptr,
                           Uint32                   num_bytes)
{
#ifdef CACHE_ENABLED

#ifdef ENABLE_CHECK_CACHE
    /* Verify whether the start address is 32-bit aligned or not */
    checkCacheLine(mem_start_ptr);
#endif /* ENABLE_CHECK_CACHE */

    BCACHE_inv((Ptr)mem_start_ptr,
                num_bytes,
                CACHE_WAIT);
#endif /* #ifdef CACHE_ENABLED */
}
 /**
 *  \brief invalidate the Cache
 *
 *  This function invalidate the D Cache in MVA way
 *
 *  \param    cache           [IN]        Cache address space
 *  \param    mem_start_ptr   [IN]        Starting adress of memory.Please note
 *                                        that this should be 32 bytes alinged.
 *  \param    num_bytes       [IN]        Length of buffer
 *
 *  \return    None
 */
/* ARGSUSED */
void PAL_osCacheFlushAndInvalidate(PAL_osCacheMemAddrSpace  cache,
                                    Uint32 mem_start_ptr,
                                    Uint32 num_bytes)
{
#ifdef CACHE_ENABLED

#ifdef ENABLE_CHECK_CACHE
    /* Verify whether the start address is 32-bit aligned or not */
    checkCacheLine(mem_start_ptr);
#endif /* ENABLE_CHECK_CACHE */

    BCACHE_wbInv ((Ptr)mem_start_ptr,
                        num_bytes,
                        CACHE_WAIT);
#endif /* #ifdef CACHE_ENABLED */
}

/**
 * \brief   PAL OS Set Cache line Length
 *
 *  Function does nothing. Implemented for consistance.
 *
 * \note   cacheAddrSpace is not cared in this API
 *
 * \param  cache         [IN]      Cache address space
 * \param  length        [IN]      Cache length
 *
 * \return  None
 */
/* ARGSUSED */
void PAL_osCacheSetCacheLineLength(PAL_osCacheMemAddrSpace  cache,
                                   Uint8                    length)
{

}

/**
 * \brief   PAL OS Get Cache line Length
 *
 *  Function does nothing. Implemented for consistance.
 *
 *  \note   cacheAddrSpace is not cared in this API
 *  \param  cache     [IN]      Cache address space
 *
 *  \return  Cache Length
 */
Uint8 PAL_osCacheGetCacheLineLength(PAL_osCacheMemAddrSpace cache)
{
    /* Does Nothing */
    /* Added to remove warnings */
    cache = cache;
    return 0;
}


/**
 *  \brief Drain the write buffer
 *
 *  Function does nothing. Implemented for consistance.
 *
 *  \note   cacheAddrSpace is not cared in this API
 *  \param  cache      [IN]      Cache address space
 *  \param  waitForInt [IN]      if TRUE -- Drains the writebuffer and put
 *                                          processor in low power mode till
 *                                          next interrupts  occurs.
 *                               if FALSE -- Drains the writebuffer and stops
 *                               excuting further till buffer is drained.
 */
void PSP_cacheDrainWriteBuffer(PAL_osCacheMemAddrSpace  cache,
                               Bool                     waitForInt)
{
    /* Does Nothing */
    /* Added to remove warnings */
    cache = cache;
    waitForInt = waitForInt;

}
