/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2004 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 * \file   pal_osMem.c
 * \brief  OsMEM Services Source File for PrOS
 *
 *  This file implements OS abstraction services for variable length heap
 *  memory management operations. The PAL module implementing these APIs
 *  is called OsMEM.
 *
 *  Implementation Notes for PrOS
 *
 *  [1] Default Memory Pool
 *  PrOS does not create a "default" memory pool like many other RTOS's. All
 *  allocations are typically "static" in nature. In order to support dynamic
 *  memory allocation for device drivers, it is assumed that for PrOS, the
 *  system configurator declares(creates) a default memory pool using static
 *  API's provided by BIOS. By default when segId=PAL_OSMEM_DEFAULT_SEGID which
 *  has a value of "0" is passed for memory allocation/free, this default memory
 *  pool is used to allocate memory.
 *
 *  The size of this default memory pool should be defined by the system
 *  configuration. By default the id of this default memory pool is selected
 *  to be "1" for PrOS. If this is changed, the macro PAL_OSMEM_DEFAULT_SEGID
 *  should be updated.
 *
 *  [2] Memory Pool Create/Delete
 *  In PrOS memory allocations are typically static and a default pool should
 *  be created during system configuration. To support dynamic pool creation
 *  and deletion, PAL_osMemSegDefine() and PAL_osMemSegUndefine() are supported.
 *
 *  (C) Copyright 2004, Texas Instruments, Inc
 *
 *  \author     PSP Group
 *  \version    0.1 - Created
 */

#include <std.h>
#include <mem.h>
#include <ti/sdo/pspdrivers/pal_os/pal_os.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osmem.h>

#ifndef MEM_HEADERSIZE
/** \brief Memory Header size */
#define MEM_HEADERSIZE (sizeof(MEM_Header))
#endif

/**
 * \brief Init PAL OSMEM module
 * \param param [IN]  Memory buffer seg/pool
 */
/* ARGSUSED */
PAL_Result PAL_osMemInit(Ptr param)
{
    /* No bookkeep required */
    return (PAL_SOK);
}

/**
 * \brief Create/Define PAL OSMEM Segment
 *
 * \param  name      [IN]  Memory Segment name
 * \param  startAddr [IN]  Starting address
 * \param  numBytes  [IN]  Length
 * \param  attrs     [IN]  Memory Segment attributes
 * \param  segId     [OUT] Allotted Segment ID
 *
 * \return PAL error codes
 */
/* ARGSUSED */
PAL_Result PAL_osMemSegDefine(const Uint8* name,
                              Uint32 startAddr,
                              Uint32 numBytes,
                              PAL_OsMemAttrs *attrs,
                              Uint32 *segId)
{
    MEM_Attrs   mem_attrs;
    PAL_Result  status = PAL_OS_ERROR_INVALID_PARAM;


    /* Check if the address is correct */
    if( (NULL != startAddr) && (NULL != segId) )
    {
        /* Check if the address & the length is aligned */
        if((( 0 == (startAddr % MEM_HEADERSIZE)))
                && (0 == (numBytes % MEM_HEADERSIZE)))
        {
            *segId = MEM_define((Ptr) startAddr, numBytes, &mem_attrs);
            status  =   PAL_SOK;
        }

    }

    return (status);
}


/**
 * \brief Delete/Undefine PAL OSMEM Segment
 *
 * \note  If segId=0, error is returned as default pool cannot be deleted
 *
 * \param  segId     [IN] Segment ID to be deleted
 *
 * \return PAL error codes
 */

PAL_Result PAL_osMemSegUndefine(Uint32 segId)
{
    MEM_undefine(segId);

    return (PAL_SOK);
}


/**
 * \brief Allocate memory from a PAL OSMEM pool
 *
 * \note "Alignment" parameter is not explicitely honoured - it is assumed
 *  that BIOS variable block allocator aligns memory by default to "int"
 *
 * \param  segId     [IN]  Segment ID
 * \param  numBytes  [IN]  Length
 * \param  alignment [IN]  Alignment
 * \param  memAddr   [OUT] Pointer to allocated memory pool
 *
 * \return PAL error codes
 */
PAL_Result PAL_osMemAlloc(Uint32 segId,
                          Uint32 numBytes,
                          Uint16 alignment,
                          Ptr *memAddr)
{
    PAL_Result status = PAL_SOK;

    if( NULL != memAddr )
    {
        *memAddr = MEM_calloc(segId, numBytes, alignment);

        if( NULL == (*memAddr) )
        {
            status = PAL_OS_ERROR_INVALID_PARAM;
        }
    }
    else
    {
        status = PAL_OS_ERROR_INVALID_PARAM;
    }

    return status;
}

/**
 * \brief Free memory allocated from PAL OSMEM pool
 *
 * \param  segId     [IN]  Segment ID
 * \param  memAddr   [IN]  Starting address
 * \param  numBytes  [IN]  Length
 *
 * \return PAL error codes
 */
PAL_Result PAL_osMemFree(Uint32 segId, Ptr memAddr, Uint32 numBytes)
{
    PAL_Result     status = PAL_SOK;
    Bool       mem_status = FALSE;

    mem_status = MEM_free(segId, memAddr, numBytes);

    if(TRUE != mem_status)
    {
        status = PAL_OS_ERROR_INVALID_PARAM;
    }

    return status;
}



/**
 * \brief PAL OS Memory Copy
 *
 *
 * \param  dest      [OUT]  Pointer to destination
 * \param  src       [IN]   Pointer to source
 * \param  numBytes  [IN]   Number of bytes to copy
 *
 * \return PAL_SOK
 */
PAL_Result PAL_osMemCopy(Ptr dest, const Ptr src, Uint32 numBytes)
{
    Uint8 *tmp = (Uint8 *) dest, *s = (Uint8 *) src;

    if ((0 == numBytes) ||  ((NULL == dest) || (NULL ==src)) )
    {
        return (PAL_OS_ERROR_INVALID_PARAM);
    }

    while (numBytes--)
    {
        *tmp = *s;
        tmp++;
        s++;
    }

    return (PAL_SOK);
}


/**
 * \brief PAL OS Memory Set
 *
 *
 * \param  memAddr   [IN]   Pointer to destination
 * \param  fillVal   [IN]   Value to be set
 * \param  numBytes  [IN]   Number of bytes to be filled
 *
 * \return PAL_SOK if success<br>
 *         PAL_OS_ERROR_INVALID_PARAM
 */
PAL_Result PAL_osMemSet(Ptr memAddr, Char fillVal, Uint32 numBytes)
{
    Uint8 *xs = (Uint8 *) memAddr;

    if ((numBytes == 0) || (NULL == memAddr))
    {
        return (PAL_OS_ERROR_INVALID_PARAM);
    }

    while (numBytes--)
    {
        *xs++ = fillVal;
    }

    return (PAL_SOK);
}

/**
 * \brief PAL OS Memory Lock
 *
 * \note  Not implemented
 *
 * \param  memAddr   [IN]   Memory address
 * \param  byteLen   [IN]   Length
 * \param  cookie    [IN]   Dummy
 *
 * \return PAL_SOK
 */
/* ARGSUSED */
PAL_Result PAL_osMemLock(Ptr memAddr, Uint32 byteLen, Uint32 *cookie)
{
    return (PAL_OS_ERROR_NOT_SUPPORTED);
}

/**
 * \brief PAL OS Memory Un-Lock
 *
 * \note  Not implemented
 *
 * \param  memAddr   [IN]   Memory address
 * \param  byteLen   [IN]   Length
 * \param  cookie    [IN]   Dummy
 *
 * \return PAL_SOK
 */
/* ARGSUSED */
PAL_Result PAL_osMemUnlock(Ptr memAddr, Uint32 byteLen, Uint32 *cookie)
{

    return (PAL_OS_ERROR_NOT_SUPPORTED);
}


/**
 * \brief PAL OS Memory Virtual To Physical
 *
 * \note  Not implemented
 *
 * \param  virtAddress   [IN]   Virtual Address
 *
 * \return Physical address(32-bit)
 */
/* ARGSUSED */
Uint32 PAL_osMemVirt2Phy(Ptr virtAddress)
{
    return ((Uint32)virtAddress);
}


/**
 * \brief PAL OS Memory Physical To Virtual
 *
 * \note  Not implemented
 *
 * \param  phyAddress   [IN]   Physical Address
 *
 * \return Virtual address
 */
Ptr PAL_osMemPhy2Virt(Uint32 phyAddress)
{
    return ((Ptr)phyAddress);
}


/**
 * \brief PAL OS Memory Report
 *
 * \note  Not implemented
 *
 * \param  segId     [IN]   Segment ID
 * \param  report    [OUT]  Report
 * \param  buf       [IN]   Memory address to get report
 *
 * \return PAL_OS_ERROR_NOT_SUPPORTED
 */
/* ARGSUSED */
PAL_Result PAL_osMemReport(Uint32 segId, PAL_OsMemReport *report, Char* buf)
{
    return (PAL_OS_ERROR_NOT_SUPPORTED);
}
