/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/** \file   pal_osSem.c
 *   \brief  OsSem Services Source File for BIOS
 *
 *  This file implements OS abstraction services for counting semaphores.
 *  All services run in the context of the calling thread or program.
 *  OsSEM does not spawn a thread of its own to implement the APIs
 *  declared here.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author     PSP Group
 *  \version    0.1 - Created
 *
 */
#include <std.h>
#include <sem.h>
#include <tsk.h>
#include <ti/sdo/pspdrivers/common/psp_common.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osSem.h>

#ifdef PAL_OSSEM_ENABLE_STATIC_MEM
#include <ti/sdo/pspdrivers/pal_os/pal_osProtect.h>

/** maximum string length that can be defined for semaphore name */
#define PAL_OSSEM_MAX_STR_LEN       (64)

/**
 * \brief   pal os semaphore object declaration
 *          This structure is used for static allocation of PAL os sem object
 */typedef struct
{
    SEM_Obj     biosSemObj;
    Uint32      state;
    Uint8       name[PAL_OSSEM_MAX_STR_LEN];
} PAL_OsSemObj;

static PAL_OsSemObj palOsSemObj[PAL_OSSEM_MAX_STATIC_ALLOC];

/* To initialize the statically allocated semaphore */
/* ARGSUSED */
PAL_Result PAL_osSemInit(Ptr param)
{
    Uint32  cnt;

    for(cnt = 0; cnt < PAL_OSSEM_MAX_STATIC_ALLOC; cnt++)
    {
        palOsSemObj[cnt].state = 0;
    }

    return PAL_SOK;
}
#endif /* PAL_OSSEM_ENABLE_STATIC_MEM */


/* To create a semaphore */
/* ARGSUSED */
PAL_Result PAL_osSemCreate(const char * name,
                           Int32 initVal,
                           const PAL_OsSemAttrs *attrs,
                           PAL_OsSemHandle * hSem)
{
    PAL_Result      semCreateResult = PAL_SOK;

    static Bool PAL_osSemInit_check = FALSE;

#ifdef PAL_OSSEM_ENABLE_STATIC_MEM
    Uint32      cookie  = 0;
    Uint32      cnt     = 0;
    Uint32      len     = 0;
#else
    SEM_Attrs   temp; /* used for passing the 'name' parameter to SEM_create */
#endif

    if(NULL == hSem)
    {
        semCreateResult   = PSP_E_RESOURCES;
    }
    else
    {
#ifndef PAL_OSSEM_ENABLE_STATIC_MEM
        temp.name = (String) name;
        *hSem = (PAL_OsSemHandle)SEM_create(initVal, &temp);
#else
        if (FALSE == PAL_osSemInit_check)
        {
            PAL_osSemInit(NULL);
            PAL_osSemInit_check = TRUE;
        }
        *hSem = NULL;
        /* Zero initialize the variable */

        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);
        /* Search for a free semaphore object */
        for(cnt = 0; cnt < PAL_OSSEM_MAX_STATIC_ALLOC; cnt++)
        {
            if(0 == palOsSemObj[cnt].state)
            {
                /* Set the variable state */
                palOsSemObj[cnt].state   = 1;

                /* Initialize the semaphore structure */
                SEM_new(&(palOsSemObj[cnt].biosSemObj),initVal);

                /* Initialize the temporary variable */
                len = 0;

                /* Copy the name field */
                while(('\0' != name[len]) && (len < (PAL_OSSEM_MAX_STR_LEN - 1)))
                {
                    palOsSemObj[cnt].name[len] = name[len];
                    len++;
                }

                /* Ensure that the last element is '\0' */
                palOsSemObj[cnt].name[len] = '\0';

                /* Now point to the name filed */
                palOsSemObj[cnt].biosSemObj.name = (String)(palOsSemObj[cnt].name);
#ifdef PRIORITY
                /* Copy the type filed */
                palOsSemObj[cnt].biosSemObj.type = attrs->type;
#endif /* PRIORITY */
                /* Copy the handle & return */
                *hSem = (PAL_OsSemHandle) &palOsSemObj[cnt];
                break;
            }
        }
        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
#endif /* PAL_OSSEM_ENABLE_STATIC_MEM */
        if ( (*hSem) == NULL )
        {
            semCreateResult = PSP_E_RESOURCES;
        }
    }
    return semCreateResult;
}

/* To Delete a semaphore */
PAL_Result PAL_osSemDelete(PAL_OsSemHandle hSem)
{
    PAL_Result  semDeleteResult = PAL_SOK;

    if( NULL != hSem )
    {
#ifdef PAL_OSSEM_ENABLE_STATIC_MEM
        Uint32      cookie;

        /* Set the value of the semaphore to free */
        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);
        ((PAL_OsSemObj *) hSem)->state = 0;
        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
#else
        SEM_delete(hSem);
#endif /* PAL_OSSEM_ENABLE_STATIC_MEM*/
    }
    else
    {
        semDeleteResult = PSP_E_INVAL_PARAM;
    }
    return semDeleteResult;
}

/* To take a semaphore */
PAL_Result PAL_osSemTake(PAL_OsSemHandle hSem, Int32 mSecTimeout)
{
    PAL_Result  semTakeResult = PAL_SOK;
    Bool        semPendResult;

    if( NULL != hSem )
    {

        /* Only allow semTake, if it is not called from main */
        if (TSK_self() != (TSK_Handle)&KNL_dummy)
        {

#ifdef PAL_OSSEM_ENABLE_STATIC_MEM
            semPendResult = SEM_pend(&((PAL_OsSemObj *) hSem)->biosSemObj, mSecTimeout);
#else
            semPendResult = SEM_pend(hSem, mSecTimeout);
#endif
            if ( semPendResult == FALSE )
            {
                semTakeResult = PSP_E_RESOURCES;
            }
            else
            {
                semTakeResult = PAL_SOK;
            }
        }

    }
    else
    {
        semTakeResult = PSP_E_INVAL_PARAM;
    }

    return semTakeResult;
}

/* To give a semaphore */
PAL_Result PAL_osSemGive(PAL_OsSemHandle hSem)
{
    PAL_Result semGiveResult = PAL_SOK;

    if( NULL != hSem )
    {
        /* Only allow semGive, if it is not called from main */
        if (TSK_self() != (TSK_Handle)&KNL_dummy)
        {

#ifdef PAL_OSSEM_ENABLE_STATIC_MEM
            SEM_post(&((PAL_OsSemObj *) hSem)->biosSemObj);
#else
            SEM_post(hSem);
#endif

        }
    }
    else
    {
        semGiveResult = PSP_E_INVAL_PARAM;
    }
    return semGiveResult;
}
