/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \file   pal_osTaskLet.c
 *
 *  \brief  BIOS OS TaskLet(Uses SWI of DSP) Source File
 *
 *  This file implements additional OS abstraction services for BIOS.
 *  The following BIOS services are provided:
 *
 *   1. Create a tasklet
 *   2. Post a tasklet
 *   3. Delete a tasklet
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author     PSP Group
 *  \version    0.1
 */

#include <std.h>
#include <swi.h> /* BIOS swi.h file */

#include <ti/sdo/pspdrivers/pal_os/pal_osTaskLet.h>


/**
 *  \brief    PAL OS create a TaskLet handler
 *
 *  \param    priority     [IN]   TaskLet priority
 *  \param    tsklFunc     [IN]   TaskLet handler function
 *  \param    arg          [IN]   Argument to be passed to TaskLet handler
 *
 *  \return
 *            Valid TaskLet Handle  if success<br>
 *            NULL              if any errors<br>
 */
Ptr  PAL_osCreateTaskLet(Int32 priority, PAL_osTaskLetFxn tsklFunc, Ptr arg)
{
    SWI_Attrs   attrs  = SWI_ATTRS;
    SWI_Handle  hSwi   = NULL;

    /* priority of zero should not be used. It is reserved by DSP/BIOS
       for kernel scheduling */
    if( (0 != priority) && (NULL != tsklFunc) )
    {
        attrs.priority = priority;
        attrs.fxn      = (SWI_Fxn)tsklFunc;
        attrs.arg0     = (Arg)arg;

        hSwi = SWI_create(&attrs);

    }
    return (Ptr)hSwi;
}

/**
 * \brief    PAL OS post or trigger TaskLet
 *
 * \param    handle     [IN]    TaskLet handle
 *
 * \return   PAL_SOK         if success<br>
 *           PAL ERROR Codes if failure<br>
 */
PAL_Result PAL_osPostTaskLet(Ptr handle)
{
    PAL_Result  result = PAL_SOK;

    if(NULL != handle)
    {
        SWI_post((SWI_Handle)handle);
    }
    else
    {
        result = PAL_OS_ERROR_INVALID_PARAM;
    }
    return result;

}

/**
 * \brief    PAL OS delete a TaskLet handler
 *
 * \param    handle     [IN]    TaskLet handle to be deleted
 *
 *  \return   PAL_SOK         if success<br>
 *            PAL ERROR Codes if failure<br>
 */
PAL_Result PAL_osDeleteTaskLet(Ptr handle)
{
    PAL_Result  result = PAL_SOK;

    if(NULL != handle)
    {
        SWI_delete((SWI_Handle)handle);
    }
    else
    {
        result = PAL_OS_ERROR_INVALID_PARAM;
    }
    return result;

}

