/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/** \file   pal_osWait.c
    \brief  OsWAIT Services Source File

    This file implements OS abstraction services for programmed interval-waits.

    (C) Copyright 2006, Texas Instruments, Inc

    \author     PSP Group
    \version    0.1 Initial version<br>
                0.2 Added BIOS_OsGetOsTicksPerMSecs function
 */

#include <std.h>
#include <tsk.h>
#include <clk.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osWait.h>

/**
 * \brief   PAL_osWaitMsecs()
 *
 *      This function leverages OS implemented "wait" to delay
 *      further execution of current thread for specified milliseconds
 *      period of time.
 * \note    The currently running thread might be preempted and placed
 *      on scheduler's wait queue for the specified duration of time.
 *      If a more effecient (w/o context switch overhead) wait is required
 *      at finer time granularity (order of microseconds), please use
 *      SysWAIT services defined in pal_sys.h file
 * \note    The function will do any milliseconds-to-ticks conversion
 *      as appropriate for implementing the wait using underlying
 *      OS supported APIs.
 * \param   mSecs [IN] is the duration in milliseconds to wait for
 * \return  PAL_SOK if succesful else a suitable error code
 */

PAL_Result PAL_osWaitMsecs(Uint32 mSecs)
{
    Uint32 ticksForSleeping = ((CLK_countspms () / CLK_getprd ()) * mSecs) ;
    TSK_sleep(ticksForSleeping);
    return (PAL_SOK);
}

/**
 * \brief Get OS ticks
 *
 * Function exposed by IOM, to DDC, to get the OS ticks
 *
 * \param   mSecs   [IN]    Number of milliSecs
 *
 * \return  number of ticks corresponding to MilliSecs
 *
 */
Int  PAL_OsGetTicksPerMSecs(Int mSecs)
{
    Int retVal = -1;
    /* if the valuse passed is negative */
    if ( -1 != mSecs )
    {
        retVal = (Int)((CLK_countspms () / CLK_getprd ()) * mSecs);
    }
    return retVal;
}

