/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/** \file   pal_os.h
    \brief  OS Abstraction Header File

    This file provides visibility to OS abstraction APIs by including
    only the configured service modules interface files.

    (C) Copyright 2006, Texas Instruments, Inc

    \author     PSP Architecture Team
    \version    1.0 - Original PSPF header + added generic error codes
                1.1 - Added Timeout error code

    <b>Compiler Switch Used</b>

    <b>_DEBUG</b><br>
     This is used as a flag to compiler whether to include the debug statements
     inserted in the code into the final image. This flag helps to build DEBUG
     image of the program. For RELEASE images this is not passed to the compiler

    <b>CACHE_ENABLED</b><br>
     This flag is passed to the compiler when cache is enabled.

    <b>PAL_OSSEM_ENABLE_STATIC_MEM</b><br>
     When this macro is enables, each semcreate function call from driver will
     not allocate semaphore object, but fetches a pre-allocated (statically)
     sem object and return. This is done especially for the application that
     dont want driver to allocate memory.

    <b>PRIORITY</b><br>
     This option is used in BIOS to define two different semaphore object

 */

#ifndef __PAL_OS_H__
#define __PAL_OS_H__

#include <ti/sdo/pspdrivers/pal_os/bios/src/pal_osCfg.h>

#ifdef PAL_INCLUDE_OSMEM
#include <ti/sdo/pspdrivers/pal_os/pal_osMem.h>          /* OsMEM Services */
#endif /* PAL_INCLUDE_OSMEM */

#ifdef PAL_INCLUDE_OSBUF
#include <ti/sdo/pspdrivers/pal_os/pal_osBuf.h>          /* OsBUF Services */
#endif /* PAL_INCLUDE_OSBUF */

#ifdef PAL_INCLUDE_OSSEM
#include <ti/sdo/pspdrivers/pal_os/pal_osSem.h>          /* OsSEM Services */
#endif /* PAL_INCLUDE_OSSEM */

#ifdef PAL_INCLUDE_OSMUTEX
#include <ti/sdo/pspdrivers/pal_os/pal_osMutex.h>        /* OsMUTEX Services */
#endif /* PAL_INCLUDE_OSMUTEX */

#ifdef PAL_INCLUDE_OSWAIT
#include <ti/sdo/pspdrivers/pal_os/pal_osWait.h>         /* OsWAIT Services */
#endif /* PAL_INCLUDE_OSWAIT */

#ifdef PAL_INCLUDE_OSLIST
#include <ti/sdo/pspdrivers/pal_os/pal_osList.h>         /* OsLIST Services */
#endif /* PAL_INCLUDE_OSLIST */

#ifdef PAL_INCLUDE_OSPROTECT
#include <ti/sdo/pspdrivers/pal_os/pal_osProtect.h>      /* OsPROTECT Services */
#endif /* PAL_INCLUDE_OSPROTECT */

#ifdef PAL_INCLUDE_OSCACHE
#include <ti/sdo/pspdrivers/pal_os/pal_osCache.h>      /* OsCACHE Services */
#endif /* PAL_INCLUDE_OSCACHE */

#ifdef __cplusplus
extern "C" {
#endif

/*
 * PAL OS module type to be placed in section D of the final "ERROR CODE"
 * as described in pal_defs.h
 */

/**
 * \brief    PALOS common error code
 */
#define PAL_OS_COMMON_ERR   (0)

/**
 * \brief    PALOS memory error
 */
#define PAL_OSMEM_ERR       (1)

/**
 * \brief    PALOS buffer error
 */
#define PAL_OSBUF_ERR       (2)

/**
 * \brief    PALOS semaphore error
 */
#define PAL_OSSEM_ERR       (3)

/**
 * \brief    PALOS mutex error
 */
#define PAL_OSMUTEX_ERR     (4)

/**
 * \brief    PALOS wait error
 */
#define PAL_OSWAIT_ERR      (5)

/**
 * \brief    PALOS list error
 */
#define PAL_OSLIST_ERR      (6)

/**
 * \brief    PALOS protect error
 */
#define PAL_OSPROTECT_ERR   (7)

/*
 * Common error codes for ALL PAL OS modules
 */

/**
 * \brief    PALOS Common create error
 */
#define PAL_OS_COMMON_ERROR_CREATE(x)   \
             (PAL_ERROR(PAL_CRITICAL_ERROR, PAL_OS_COMMON_ERR, 0, (x)))
/**
 * \brief    PALOS Invalid parameter passed to the function error
 */
#define PAL_OS_ERROR_INVALID_PARAM      (PAL_OS_COMMON_ERROR_CREATE(1))

/**
 * \brief    PALOS Feature not supported error
 */
#define PAL_OS_ERROR_NOT_SUPPORTED      (PAL_OS_COMMON_ERROR_CREATE(2))

/**
 * \brief    PALOS No resources available error
 */
#define PAL_OS_ERROR_NO_RESOURCES       (PAL_OS_COMMON_ERROR_CREATE(3))

/**
 * \brief    PALOS Timeout from the OS call error
 */
#define PAL_OS_ERROR_TIMEOUT            (PAL_OS_COMMON_ERROR_CREATE(4))

/**
 * \brief    PALOS OS specific error - not classified, not passed on to
 *           the caller
 */
#define PAL_OS_ERROR_OS_SPECFIC         (PAL_OS_COMMON_ERROR_CREATE(5))


/* Default (memory) segment Id - Many of the modules (like OSSEM, OSBUF),
 * depend upon a segment id to be passed in the API's. The macro below
 * defines a default segment Id that can be used in these API's
 */


/* Default Pool ID - Used when segId=0 is passed for memory alloc/free
 * This macro should be updated if the default pool id is changed
 */

/**
 * \brief    PALOS MEMORY default segment ID
 */
#define PAL_OSMEM_DEFAULT_SEGID    1

/**
 * \brief    PALOS MEMORY no cache segment ID
 */
#define PAL_OSMEM_NOCACHE_SEGID    2

#ifdef __cplusplus
}
#endif

#endif /* _PAL_OS_H_ */
