/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \file   pal_osSem.h
 *
 *  \brief  PALOS Semaphore Services Header File
 *
 *  This file declares OS abstraction services for counting semaphores.
 *  All services run in the context of the calling thread or program.
 *  OsSEM does not spawn a thread of its own to implement the APIs
 *  declared here.
 *
 *   (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author     PSP Architecture Team
 *  \version    1.0
 */

#ifndef __PAL_OSSEM_H__
#define __PAL_OSSEM_H__

#include <ti/sdo/pspdrivers/common/psp_common.h>

#ifndef BIOS_BUILD
    #include <ti/sdo/pspdrivers/pal_os/pal_osMem.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

/**
 * \defgroup PalOSSem PAL OS Semaphore Interface
 *
 * PAL OS Semaphore Interface
 * @{
 */

/** \name PAL OS Semaphore Interface
 *  PAL OS Semaphore Interface
 * @{
 */

 /** PAL OS Semaphore default attributes*/
#define PAL_OSSEM_ATTRS {PAL_OSSEM_TYPE_PRIORITY,NULL}

/** Define to Enable/Disable Static Memory allocation */
#define PAL_OSSEM_ENABLE_STATIC_MEM

/** Define to Max Statically allocated Semaphore Objects */
#define PAL_OSSEM_MAX_STATIC_ALLOC          (64)

/**
 * \brief Blocking call without timeout
 */
#define PAL_OSSEM_NO_TIMEOUT    (-1)

/**
 * \brief PAL OS Semaphore Handle
 */
typedef Ptr PAL_OsSemHandle;

/**
 * \brief   PAL OS Semaphore Init
 *
 *      This is an idempotent function that must be called ahead of
 *      calling any other OsSEM services. It initializes OsSEM internal
 *      data structures and does any book-keep necessary to implement
 *      the published services of OsSEM.
 * \param   param [IN] is an arbitrary void* data type used to pass platform
 *      specific initialization information for OsSEM. This can be used
 *      to extend OsSEM configurability to decisions made at run-time.
 *      This is added for future extensions only.
 * \return  PAL_SOK if successful else a suitable error code.
 */
PAL_Result PAL_osSemInit(Ptr param);

/**
 * \brief    PALOS Semaphore type
 */
typedef enum PAL_OsSemType_t
{
  PAL_OSSEM_TYPE_FIFO = 0,
  /**< FIFO based                */
  PAL_OSSEM_TYPE_PRIORITY = 1
  /**< Priority based            */
} PAL_OsSemType;

/**
 * \brief    PALOS Semaphore attributes
 */
typedef struct
{
  PAL_OsSemType  type;     /**< Semaphore type             */
  Uint32         memSegId; /**< Memory segment ID          */
} PAL_OsSemAttrs;

/**
 * \brief   PAL OS Semaphore Create
 *
 *      This function creates a counting semaphore with specified
 *      attributes and initial value.
 * \param   name    [IN]  String name of the semaphore
 * \param   initVal [IN]  Initial value for semaphore
 * \param   attrs   [IN]  The semaphore attributes ex: Fifo type
 * \param   hSem    [OUT] Location to recieve the handle to just created
 *      semaphore
 * \return  PAL_SOK if succesful<br>
 *          PALOS error codes
 */
PAL_Result PAL_osSemCreate(const char * name,
                           Int32 initVal,
                           const PAL_OsSemAttrs *attrs,
                           PAL_OsSemHandle * hSem);

/**
 * \brief   PAL OS Semaphore Delete
 *
 *      This function deletes or removes the specified semaphore
 *      from the system. Associated dynamically allocated memory
 *      if any is also freed up.
 * \warning OsSEM services run in client context and not in a thread
 *      of their own. If there exist threads pended on a semaphore
 *      that is being deleted, results are undefined.
 * \param   hSem [IN] handle to the semaphore to be deleted
 * \return  PAL_SOK if succesful else a suitable error code
 */
PAL_Result PAL_osSemDelete(PAL_OsSemHandle hSem);

/**
 * \brief   PAL OS Semaphore Take
 *
 *      This function takes a semaphore token if available.
 *      If a semaphore is unavailable, it blocks currently
 *      running thread in wait (for specified duration) for
 *      a free semaphore.
 * \param   hSem [IN] is the handle of the specified semaphore
 * \param   mSecTimeout [IN] is wait time in milliseconds
 * \return  PAL_SOK if successful else a suitable error code
 */
PAL_Result PAL_osSemTake(PAL_OsSemHandle hSem, Int32 mSecTimeout);

/**
 * \brief   PAL OS Semaphore Give
 *
 *      This function gives or relinquishes an already
 *      acquired semaphore token
 * \param   hSem [IN] is the handle of the specified semaphore
 * \return  PAL_SOK if successful else a suitable error code
 */
PAL_Result PAL_osSemGive(PAL_OsSemHandle hSem);

/**
 * \brief Semaphore Report Data Structure
 *
 * Data structure for Semaphore Report service
 */
typedef struct
{
    char     *name;       /**< Name of the semaphore     */
    Int32    initVal;     /**< Init value                */
    Int32 currentVal;     /**< Current value             */
    PAL_OsSemAttrs attrs; /**< Attributes                */
} PAL_OsSemReport;

/**
 * \brief   PAL OS Semaphore Report
 *
 *      This function reports assorted usage statistics information
 *      regarding the specified semaphore
 * \param   hSem [IN] handle of semaphore to be reported on.
 * \param   report [IN/OUT] location where information must be reported
 *      If NULL, structure is not filled.
 * \param   buf [IN/OUT] string buffer where a text formatted report will
 *      be printed. If NULL, no text style reporting is done.
 * \return  PAL_SOK if successful, else a suitable error code.
 */
PAL_Result PAL_osSemReport(PAL_OsSemHandle hSem, PAL_OsSemReport *report, Char* buf);

/*@}*/
/*@}*/
#ifdef __cplusplus
}
#endif

#endif /* _PAL_OSSEM_H_ */
