/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/** \file   pal_sysPci.h
 *  \brief  APIs of PCI driver
 *
 *  This file contains the APIs that will be the interface between PCI driver
 *  and application.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \version    1.0        Rinkal Shah   Created
 *
 *  <b>Compiler Switch Used</b>
 *
 *  <b>_DEBUG</b><br>
 *   This is used as a flag to compiler whether to include the debug statements
 *   inserted in the code into the final image. This flag helps to build DEBUG
 *   image of the program. For RELEASE images this is not passed to the compiler.
 *
 *  <b>CHIP_XXXX</b><br>
 *   The CSL layer is written in a common file for all the variants of a SOC.
 *   This flag differentiates the variant we are compiling for, for e.g.
 *   CHIP_DM648, and the CSL definitions for that variant appropriately gets
 *   defined for register base addresses, num of ports of a peripheral etc.
 *
 *  <b>PCI_ECM_HOOK_ENABLE</b><br>
 *   This option is used when Ecm Hook is enabled.
 *
 *  <b>PCI_INSTRUMENTATION_ENABLED</b><br>
 *   This option is passed to the compiler to include the instrumentation code
 *   parts into the final image/lib of the program. This helps build the
 *   iRelease/iDebug versions of the image/lib with a common code base
 *
 *
 */

#ifndef _PAL_SYSPCI_H_
#define _PAL_SYSPCI_H_

#include <ti/sdo/pspdrivers/pal_os/pal_osSem.h>

#ifdef __cplusplus
extern "C" {
#endif

/** Maximum PCI driver instance supported */
#define PAL_SYSPCI_MAX_PCI_INSTANCE    (1u)
/** Maximum number of slaves supported by PCI */
#define PAL_SYSPCI_MAX_NUM_SLAVES    (4u)
/** Maximum base address registers supported by PCI controller */
#define PAL_SYSPCI_MAX_BASE_ADDR    (6u)
/** Interrupt mask to enable/disable Target abort interrupt */
#define PAL_SYSPCI_TARGET_ABORT_INT_MASK        (0x00000002u)
/** Interrupt mask to enable/disable Master abort interrupt */
#define PAL_SYSPCI_MASTER_ABORT_INT_MASK        (0x00000004u)
/** Interrupt mask to enable/disable system error interrupt */
#define PAL_SYSPCI_SYS_ERR_INT_MASK             (0x00000020u)
/** Interrupt mask to enable/disable parity error interrupt */
#define PAL_SYSPCI_PARITY_ERR_INT_MASK          (0x00000040u)


/**
 * \defgroup PalSysPciErrCode Error Codes
 *
 * Error Codes returned by the PAL SYS PCI driver
 *
 * @{
 */
/** PCI driver error base define */
#define PAL_SYS_PCI_ERR_BASE                    (-40)
/**
 * PCI Driver Object Not Deleted yet.
 * So the object cannot be created.
 */
#define PAL_SYS_PCI_OBJ_NOT_DELETED         (PAL_SYS_PCI_ERR_BASE)
/**
 * PCI Driver Object Not Created yet.
 * So the object cannot be deleted.
 */
#define PAL_SYS_PCI_OBJ_NOT_CREATED         (PAL_SYS_PCI_ERR_BASE-1)
/**
 * PCI Driver Object Not Closed yet.
 * So the object cannot be deleted.
 */
#define PAL_SYS_PCI_OBJ_NOT_CLOSED          (PAL_SYS_PCI_ERR_BASE-2)
/**
 * PCI Driver Not Opened yet
 * So the object cannot be closed.
 */
#define PAL_SYS_PCI_OBJ_NOT_OPENED          (PAL_SYS_PCI_ERR_BASE-3)
/** Invalid Parameter passed to API */

#define PAL_SYS_PCI_INVALID_PARAM           (PAL_SYS_PCI_ERR_BASE-4)
/** Error encountered in PCI driver while semaphore operation */

#define PAL_SYS_PCI_SEM_ERR                     (PAL_SYS_PCI_ERR_BASE-5)
/* @} PalSysPciErrCode */

/**
 * \defgroup PalSysPciIntCode
 *
 * Codes to enable/disable PCI interrupts
 *
 * @{
 */

/**
 * Code to enable/disable Target abort interrupt of Host
 */
#define PAL_SYS_PCI_HOST_INT_TGT_ABORT                     (0x01u)
/**
 * Code to enable/disable Master abort interrupt of Host
 */
#define PAL_SYS_PCI_HOST_INT_MS_ABORT                      (0x02u)
/**
 * Code to enable/disable system error detect interrupt of Host
 */
#define PAL_SYS_PCI_HOST_INT_SYS_ERR_DETECT                (0x03u)
/**
 * Code to enable/disable parity error detect interrupt of Host
 */
#define PAL_SYS_PCI_HOST_INT_PARITY_ERR_DETECT             (0x04u)
/**
 * Code to enable/disable Software Interrupt 0 of Host
 */
#define PAL_SYS_PCI_HOST_INT_SOFTINT0                      (0x05u)
/**
 * Code to enable/disable Software interrupt 1 of Host
 */
#define PAL_SYS_PCI_HOST_INT_SOFTINT1                      (0x06u)
/**
 * Code to enable/disable Software interrupt 2 of Host
 */
#define PAL_SYS_PCI_HOST_INT_SOFTINT2                      (0x07u)
/**
 * Code to enable/disable Software interrupt 3 of Host
 */
#define PAL_SYS_PCI_HOST_INT_SOFTINT3                      (0x08u)

/**
 * Code to enable/disable target abort error interrupt of DSP applicaiton
 */
#define PAL_SYS_PCI_DSP_INT_TGT_ABORT                      (0x10u)
/**
 * Code to enable/disable master abort error interrupt of DSP applicaiton
 */
#define PAL_SYS_PCI_DSP_INT_MS_ABORT                       (0x20u)
/**
 * Code to enable/disable system error detect interrupt of DSP applicaiton
 */
#define PAL_SYS_PCI_DSP_INT_SYS_ERR_DETECT                 (0x30u)
/**
 * Code to enable/disable parity error detect interrupt of DSP applicaiton
 */
#define PAL_SYS_PCI_DSP_INT_PARITY_ERR_DETECT              (0x40u)
/**
 * Code to enable/disable Software Interrupt 0 of DSP applicaiton
 */
#define PAL_SYS_PCI_DSP_INT_SOFTINT0                       (0x50u)
/**
 * Code to enable/disable Software interrupt 1 of DSP applicaiton
 */
#define PAL_SYS_PCI_DSP_INT_SOFTINT1                       (0x60u)
/**
 * Code to enable/disable Software interrupt 2 of DSP applicaiton
 */
#define PAL_SYS_PCI_DSP_INT_SOFTINT2                       (0x70u)
/**
 * Code to enable/disable Software interrupt 3 of DSP applicaiton
 */
#define PAL_SYS_PCI_DSP_INT_SOFTINT3                       (0x80u)


/* @} PalSysPciIntCode */

/**
 * \brief     Application callback function. This function will be called on
 *            occurance any PCI interrupt. This callback function will be called
 *            on occurance of any PCI interrupt. "interruptMask" will contains
 *            interrupt status register of PCI controller
 *
 * \param   interruptMask   [IN]    PCI driver will pass the interrupt mask
 *                                  value at the time of occurance of interrupt.
 *                                  Application have to identify the interrupt
 *                                  that has occurred by reading this mask.
 * \param    appData        [IN]    Pointer passed by application while
 *                                  PAL_sysPCIOpen as appData.
 *
 */
typedef void (* PAL_sysPciAppCallback)(Uint32 interruptMask,Ptr appData);

/**
 * \brief PCI driver configuration structure
 *
 * Configuration structure to be passed as argument while open call for PCI
 * driver
 */
typedef struct _PAL_sysPciInitConfig
{
    /** Instance wide callback function to catch errors*/
    PAL_sysPciAppCallback appCb;
    /** Application data to be passed back to the app callback */
    Ptr appData;
} PAL_sysPciInitConfig;



/*
 * Enum containing various sizes of read/write transections
 */
typedef enum
{
    /** Read/write one byte */
    PAL_SYS_PCI_READ_WRITE_SIZE_BYTE        = 0u,
    /** Read/write two byte */
    PAL_SYS_PCI_READ_WRITE_SIZE_HALFWORD    = 1u,
    /** Read/write four bytes */
    PAL_SYS_PCI_READ_WRITE_SIZE_WORD        = 2u
}PAL_sysPciConfigFieldSize;


/*
 * Enum containing offsets of all the back end registers
 */
typedef enum
{
    /** status set register  */
    PAL_SYSPCI_STATUS_SET                   = 0x010u,
    /** status clear register  */
    PAL_SYSPCI_STATUS_CLEAR                 = 0x014u,
    /** Host interrupt enable set register  */
    PAL_SYSPCI_HOST_INT_ENABLE_SET          = 0x020u,
    /** Host interrupt enable clear register  */
    PAL_SYSPCI_HOST_INT_ENABLE_CLEAR        = 0x024u,
    /** DSP interrupt enable set register  */
    PAL_SYSPCI_DSP_INT_ENABLE_SET           = 0x030u,
    /** DSP interrupt enable clear register  */
    PAL_SYSPCI_DSP_INT_ENABLE_CLEAR         = 0x034u,

    /** Vendor ID mirror register  */
    PAL_SYSPCI_MIRROR_VENDOR_ID             = 0x100u,
    /** Device ID mirror register  */
    PAL_SYSPCI_MIRROR_DEVICE_ID             = 0x102u,
    /** Command mirror register  */
    PAL_SYSPCI_MIRROR_CMD_STATUS_REG        = 0x104u,
    /** Revision ID mirror register  */
    PAL_SYSPCI_MIRROR_REVISION_ID           = 0x108u,
    /** Class code mirror register  */
    PAL_SYSPCI_MIRROR_CLASS_CODE            = 0x109u,
    /** Cache size mirror register  */
    PAL_SYSPCI_MIRROR_CACHE_SIZE            = 0x10Cu,
    /** Latency timer mirror register  */
    PAL_SYSPCI_MIRROR_LATENCY_TIMER         = 0x10Du,
    /** Header type mirror register  */
    PAL_SYSPCI_MIRROR_HEADER_TYPE           = 0x10Eu,
    /** BIST mirror register  */
    PAL_SYSPCI_MIRROR_BIST_REG              = 0x10Fu,
    /** BAR 0 mask register  */
    PAL_SYSPCI_MASK_BAR_0                   = 0x110u,
    /** BAR 1 mask register  */
    PAL_SYSPCI_MASK_BAR_1                   = 0x114u,
    /** BAR 2 mask register  */
    PAL_SYSPCI_MASK_BAR_2                   = 0x118u,
    /** BAR 3 mask register  */
    PAL_SYSPCI_MASK_BAR_3                   = 0x11Cu,
    /** BAR 4 mask register  */
    PAL_SYSPCI_MASK_BAR_4                   = 0x120u,
    /** BAR 5 mask register  */
    PAL_SYSPCI_MASK_BAR_5                   = 0x124u,
    /** Subsytem vendor ID mirror register  */
    PAL_SYSPCI_MIRROR_SUBSYS_VENDOR_ID      = 0x12Cu,
    /** Subsytem ID mirror register  */
    PAL_SYSPCI_MIRROR_SUBSYS_ID             = 0x12Eu,

    /** Capabilities pointer mirror register  */
    PAL_SYSPCI_MIRROR_CAP_PTR_ID            = 0x134u,
    /** Interrupt line mirror register  */
    PAL_SYSPCI_MIRROR_INT_LINE              = 0x13Cu,
    /** Interrupt Pin mirror register  */
    PAL_SYSPCI_MIRROR_INT_PIN               = 0x13Du,
    /** Minimum grant bits mirror register  */
    PAL_SYSPCI_MIRROR_MIN_GRANT_BITS        = 0x13Eu,
    /** Maximum latency bits mirror register  */
    PAL_SYSPCI_MIRROR_MAX_LATENCY_BITS      = 0x13Fu,

    /** Slave control register  */
    PAL_SYSPCI_SLAVE_CNTL_REG                = 0x180u,

    /** Slave base address 0 translation register  */
    PAL_SYSPCI_SLAVE_BASE_ADDR_TRANS_REG_0   = 0x1C0u,
    /** Slave base address 1 translation register  */
    PAL_SYSPCI_SLAVE_BASE_ADDR_TRANS_REG_1   = 0x1C4u,
    /** Slave base address 2 translation register  */
    PAL_SYSPCI_SLAVE_BASE_ADDR_TRANS_REG_2   = 0x1C8u,
    /** Slave base address 3 translation register  */
    PAL_SYSPCI_SLAVE_BASE_ADDR_TRANS_REG_3   = 0x1CCu,
    /** Slave base address 4 translation register  */
    PAL_SYSPCI_SLAVE_BASE_ADDR_TRANS_REG_4   = 0x1D0u,
    /** Slave base address 5 translation register  */
    PAL_SYSPCI_SLAVE_BASE_ADDR_TRANS_REG_5   = 0x1D4u,

    /** Base address 0 mirror register  */
    PAL_SYSPCI_MIRROR_BAR_0                  = 0x1E0u,
    /** Base address 1 mirror register  */
    PAL_SYSPCI_MIRROR_BAR_1                  = 0x1E4u,
    /** Base address 2 mirror register  */
    PAL_SYSPCI_MIRROR_BAR_2                  = 0x1E8u,
    /** Base address 3 mirror register  */
    PAL_SYSPCI_MIRROR_BAR_3                  = 0x1ECu,
    /** Base address 4 mirror register  */
    PAL_SYSPCI_MIRROR_BAR_4                  = 0x1F0u,
    /** Base address 5 mirror register  */
    PAL_SYSPCI_MIRROR_BAR_5                  = 0x1F4u,

    /** Master configuration/IO access data register  */
    PAL_SYSPCI_MASTER_CONFIG_IO_DATA_REG     = 0x300u,
    /** Master configuration/IO access address register  */
    PAL_SYSPCI_MASTER_CONFIG_IO_ADDR_REG     = 0x304u,
    /** Master configuration/IO access command register  */
    PAL_SYSPCI_MASTER_CONFIG_IO_CMD_REG      = 0x308u,

    /** Master configuration register  */
    PAL_SYSPCI_MASTER_CONFIG_REG             = 0x310u,

    /** PCI Address Substitution register 0  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG0        = 0x314u,
    /** PCI Address Substitution register 1  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG1        = 0x318u,
    /** PCI Address Substitution register 2  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG2        = 0x31Cu,
    /** PCI Address Substitution register 3  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG3        = 0x320u,
    /** PCI Address Substitution register 4  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG4        = 0x324u,
    /** PCI Address Substitution register 5  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG5        = 0x328u,
    /** PCI Address Substitution register 6  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG6        = 0x32Cu,
    /** PCI Address Substitution register 7  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG7        = 0x330u,
    /** PCI Address Substitution register 8  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG8        = 0x334u,
    /** PCI Address Substitution register 9  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG9        = 0x338u,
    /** PCI Address Substitution register 10  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG10    = 0x33Cu,
    /** PCI Address Substitution register 11  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG11    = 0x340u,
    /** PCI Address Substitution register 12  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG12    = 0x344u,
    /** PCI Address Substitution register 13  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG13    = 0x348u,
    /** PCI Address Substitution register 14  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG14    = 0x34Cu,
    /** PCI Address Substitution register 15  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG15    = 0x350u,
    /** PCI Address Substitution register 16  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG16    = 0x354u,
    /** PCI Address Substitution register 17  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG17    = 0x358u,
    /** PCI Address Substitution register 18  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG18    = 0x35Cu,
    /** PCI Address Substitution register 19  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG19    = 0x360u,
    /** PCI Address Substitution register 20  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG20    = 0x364u,
    /** PCI Address Substitution register 21  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG21    = 0x368u,
    /** PCI Address Substitution register 22  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG22    = 0x36Cu,
    /** PCI Address Substitution register 23  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG23    = 0x370u,
    /** PCI Address Substitution register 24  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG24    = 0x374u,
    /** PCI Address Substitution register 25  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG25    = 0x378u,
    /** PCI Address Substitution register 26  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG26    = 0x37Cu,
    /** PCI Address Substitution register 27  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG27    = 0x380u,
    /** PCI Address Substitution register 28  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG28    = 0x384u,
    /** PCI Address Substitution register 29  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG29    = 0x388u,
    /** PCI Address Substitution register 30  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG30    = 0x38Cu,
    /** PCI Address Substitution register 31  */
    PAL_SYSPCI_ADDR_SUBSTITUTION_REG31    = 0x390u

}PAL_sysPciMemMapField;


/*
 * Enum containing offsets of hook configuraiton register
 */
typedef enum
{
    /** Vendor ID program register  */
    PAL_SYSPCI_HOOK_REG_PROGRAM_VENDOR_ID                       = 0x394u,
    /** Device ID program register */
    PAL_SYSPCI_HOOK_REG_PROGRAM_DEVICE_ID                       = 0x396u,

    /** Class code and revision ID program register  */
    PAL_SYSPCI_HOOK_REG_PROGRAM_CLASS_CODE_REVISION_ID          = 0x39Cu,
    /** Regsiter to program vendor ID  */
    PAL_SYSPCI_HOOK_REG_PROGRAM_SUB_SYS_VENDOR_ID_SUB_SYS_ID    = 0x3A0u,
    /** Max latency and min grant program register  */
    PAL_SYSPCI_HOOK_REG_PROGRAM_MAX_LATENCY_MIN_GRANT           = 0x3A4u,

    /** Configuration done registers  */
    PAL_SYSPCI_HOOK_REG_CONFIG_DONE_REG                         = 0x3ACu

}PAL_sysPciHookRegister;

/*
 * Enum containing the value of Cache line size register
 */
typedef enum
{
    /** Cache line size - Disabled  */
    PAL_SYSPCI_CACHELINE_SIZE_DISABLED     = 0x00u,
    /** Cache line size - 16 bytes  */
    PAL_SYSPCI_CACHELINE_SIZE_16BYTES     = 0x04u,
    /** Cache line size - 32 bytes  */
    PAL_SYSPCI_CACHELINE_SIZE_32BYTES     = 0x08u,
    /** Cache line size - 64 bytes  */
    PAL_SYSPCI_CACHELINE_SIZE_64BYTES     = 0x10u,
    /** Cache line size - 128 bytes  */
    PAL_SYSPCI_CACHELINE_SIZE_128BYTES     = 0x20u
}PAL_sysPciCacheLineSize;

/*
 * \brief    The macro will provide the will convert the PCI address to memory
 *            map address.
 *
 * \param   pciAdd         [IN]        Address on PCI bus
 */

#if defined (CHIP_DM6437) || defined (CHIP_C6424)
#define     PAL_SYS_PCI_TO_DSP_MEM_MAP_ADDR(pciAddr)    \
                (0x30000000u | ((Uint32)pciAddr & ~(0xFF800000u)))
#endif

#if defined (CHIP_DM648) || defined (CHIP_C6452)
#define     PAL_SYS_PCI_TO_DSP_MEM_MAP_ADDR(pciAddr)    \
                (0x40000000u | ((Uint32)pciAddr & ~(0xFF800000u)))
#endif

/**
 * PAL_sysPCICreate
 * \brief   Create the instance of PCI driver. Check for the maximum instance
 *          number. If instance number is proper, change the state of driver.
 *          is just to bring consistency in PAL SYS drivers.
 *
 *
 * \param   instId         [IN]     Instance Id of PCI controller
 * \param   param          [IN]     Pointer parameters required for creating PCI
 *                                  driver handle
 *
 * \return    PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCICreate(Uint32 instId, Ptr param);

/**
 * PAL_sysPCIDelete
 * \brief    Delete the instance of PCI driver. Check for the maximum instance
 *           number. If instance number is proper, change the state of driver.
 *           This function is just to bring consistancy in PAL SYS drivers.
 *
 * \param   instId         [IN]        Instance Id of PCI controller
 *
 * \return    PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCIDelete(Uint32 instId);

/**
 * PAL_sysPCIOpen
 * \brief   Open the driver. Check for the maximum instance number. If the
 *          the handle is proper Initialize the address of configuration space
 *          registers, backend registers, address translation register and hook
 *          configuration register base addresses. Register the interrupt
 *          callback for PCI interrupt. This callback will receive the interrupt
 *          status register as one argument and applicaiton data pointer as
 *          other argument on occurance of interrupt.
 *
 * \param   instId         [IN]     Instance Id of PCI controller
 * \param   param          [IN]     Pointer parameters required for opening PCI
 *                                  driver handle
 *
 * \return    PSP_Handle if successful else NULL if unsuccessful
 */
PSP_Handle PAL_sysPCIOpen(Uint32 instId, const PAL_sysPciInitConfig *param);


/**
 * PAL_sysPCIClose
 * \brief   Close the PCI driver. Check the instance number and state of driver.
 *          If state and instance number are proper uninitialize the interrupt
 *          routine. Uninitialize the base addresses of configuration space
 *          register, back end registers, hook configuration registers and
 *          slave address translation register by NULL. Change the state of
 *          driver instance to CLOSED.
 *
 * \param   hPci           [IN]     Handle of PCI controller driver
 * \param   param          [IN]     Pointer parameters required for closing PCI
 *                                  driver handle
 *
 * \return    PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCIClose(PSP_Handle hPci, Ptr param);

/**
 * PAL_sysPCIEnableInterrupt
 * \brief   Enables  PCI interrupt/s. Depending upon the mask passed interrupts
 *          are enabled. If a particular bit is set, corresponding interrupt
 *          is enabled.
 *
 * \param   hPci           [IN]     Handle of PCI controller driver
 * \param   intEnableCode  [IN]     Interrupt enable mask. 5 LSBs of
 *                                  intEnableMask will correspond to 5
 *                                  interrupts of PCI. If a particular bit is
 *                                  set, corresponding interrupt would be
 *                                  enabled.
 *
 * \return    PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCIEnableInterrupt(PSP_Handle hPci, Uint32 intEnableCode);

/**
 * PAL_sysPCIDisableInterrupt
 * \brief   Disables  PCI interrupt/s. Depending upon the mask passed interrupts
 *          are disabled. If a particular bit is set, corresponding interrupt
 *          is disabled.
 *
 * \param   hPci           [IN]     Handle of PCI controller driver
 * \param   intDisableCode [IN]     Interrupt disable mask. 5 LSBs of
 *                                  intDisableMask will correspond to 5
 *                                  interrupts of PCI. If a particular bit is
 *                                  set, corresponding interrupt would be
 *                                  disabled.
 *
 * \return    PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCIDisableInterrupt(PSP_Handle hPci, Uint32 intDisableCode);

/**
 * PAL_sysPCISetMemMapReg
 * \brief   Set memory mapped register. Check for the handle. If handle is
 *          proper set the specified 32-bit register by the specified value.
 *
 * \param   hPci            [IN]    Handle of PCI controller driver
 * \param   memMapField     [IN]    Field of memory mapped register that has
 *                                  to be set
 * \param   writeSize       [IN]    Size of data to be set (byte/word/dword)
 * \param   newFieldVal     [IN]    New value of field that is to be set
 *
 * \return  PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCISetMemMapReg( PSP_Handle hPci,
                                   PAL_sysPciMemMapField memMapField,
                                    PAL_sysPciConfigFieldSize    writeSize,
                                    Uint32 newFieldVal);

/**
 * PAL_sysPCIGetMemMapReg
 * \brief   Get memory mapped register. Check for the handle. If handle is
 *          proper get the specified 32-bit register in specified variable.
 *
 * \param   hPci            [IN]    Handle of PCI controller driver
 * \param   memMapField     [IN]    Field of memory mapped register that has
 *                                  to be read
 * \param   readSize        [IN]    Size of data to be read (byte/word/dword)
 * \param   *newFieldVal    [OUT]   Pointer to new value of field that is
 *                                  to be read from hardware
 *
 * \return  PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCIGetMemMapReg(  PSP_Handle hPci,
                                    PAL_sysPciMemMapField memMapField,
                                    PAL_sysPciConfigFieldSize    readSize,
                                    Uint32* newFieldVal);

/**
 * PAL_sysPCISetHookReg
 * \brief   Set Hook register. Check for the handle. If handle is proper
 *          set the specified 32-bit register by the specified value.
 *
 * \param   hPci            [IN]        Handle of PCI controller driver
 * \param   hookRegField    [IN]        Field of hook register that has
 *                                      to be set
 * \param   writeSize       [IN]        Size of data to be set (byte/word/dword)
 * \param   newFieldVal     [IN]        New value of field that is to be set
 *
 * \return  PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCISetHookReg(    PSP_Handle hPci,
                                    PAL_sysPciHookRegister hookRegField,
                                    PAL_sysPciConfigFieldSize    writeSize,
                                    Uint32 newFieldVal);

/**
 * PAL_sysPCIGetHookReg
 * \brief   Get hook register. Check for the handle. If handle is proper
 *          get the specified 32-bit register in specified variable.
 *
 * \param   hPci            [IN]    Handle of PCI controller driver
 * \param   hookRegField    [IN]    Field of hook register that has
 *                                  to be read
 * \param   readSize        [IN]    Size of data to be read (byte/word/dword)
 * \param   *newFieldVal    [OUT]   Pointer to new value of field that is
 *                                  to be read from hardware
 *
 * \return    PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCIGetHookReg(    PSP_Handle hPci,
                                    PAL_sysPciHookRegister hookRegField,
                                    PAL_sysPciConfigFieldSize    readSize,
                                    Uint32* newFieldVal);

/**
 * PAL_sysPCIEnableBASE
 * \brief   API to Enable BasePrefetch. API to be used while device in slave
 *          mode.
 *
 * \param	hPci				[IN]		Handle of PCI controller driver
 * \param	baseId				[IN]		ID of BASE to be enabled
 *
 * \return	PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCIEnableBasePrefetch (
                        PSP_Handle hPci,
	                    Uint32 baseId);
/**
 * PAL_sysPCIDisableBASE
 * \brief	API to Disable BASE. API to be used while device in slave mode.
 *
 * \param	hPci				[IN]		Handle of PCI controller driver
 * \param	baseId				[IN]		ID of BASE to be enabled
 *
 * \return	PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCIDisableBasePrefetch (
                        PSP_Handle hPci,
	                    Uint32 baseId);

/**
 * PAL_sysPCIProgramCacheLineSize
 * \brief   API to set Cache line size. API to be used while device in slave
 *          mode.
 *
 * \param	hPci				[IN]		Handle of PCI controller driver
 * \param	valCacheLineSize	[IN]		Size of cache line to be set
 *
 * \return	PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCIProgramCacheLineSize (
                        PSP_Handle hPci,
	                    PAL_sysPciCacheLineSize valCacheLineSize);
/**
 * PAL_sysPCIProgramLatencyTimer
 * \brief   API to set Cache line size. API to be used while device in slave
 *          mode.
 *
 * \param	hPci				[IN]		Handle of PCI controller driver
 * \param	valLatencyTimer		[IN]		Value of latency timer
 *
 * \return	PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCIProgramLatencyTimer (	PSP_Handle hPci,
	                                		 Uint32 valLatencyTimer);
#ifdef __cplusplus
}
#endif

#endif /*_PAL_SYSPCI_H_*/
