/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/** \file   pal_sysPci.c
 *  \brief  APIs of PCI driver
 *
 *  This file contains the implementation of APIs that will be the interface
 *  between PCI driver and application.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \version    1.0        Rinkal Shah   Created
 *
 */
#include <std.h>
#include <tsk.h>
#include <ecm.h>
#include <c64.h>

#include <ti/sdo/pspdrivers/pal_os/pal_os.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osSem.h>
#include <ti/sdo/pspdrivers/pal_sys/pci/src/pal_sysPciInt.h>
#include <ti/sdo/pspdrivers/pal_sys/pci/pal_sysPci.h>

#ifdef CHIP_DM648
#include <ti/sdo/pspdrivers/soc/dm648/dsp/cslr_pci.h>
#endif

#ifdef CHIP_C6452
#include <ti/sdo/pspdrivers/soc/c6452/dsp/cslr_pci.h>
#endif

#ifdef CHIP_DM6437
#include <ti/sdo/pspdrivers/soc/dm6437/dsp/cslr_pci.h>
#endif

#ifdef CHIP_C6424
#include <ti/sdo/pspdrivers/soc/c6424/dsp/cslr_pci.h>
#endif

#ifdef PCI_ECM_HOOK_ENABLE
#include <ecm.h>
#else
#include <hwi.h>
#endif /* PCI_ECM_HOOK_ENABLE */

#ifdef PCI_INSTRUMENTATION_ENABLED
#include <ti/sdo/pspdrivers/pal_os/bios/psp_log.h>
#endif /*I2C_INSTRUMENTATION_ENABLED*/

/* Interrupt source id for PCI evemt */
#define INT_SOURCE_FOR_PCI (4u)

pciInstanceObject    pciInstObj[CSL_PCI_PER_CNT] =
                                {
                                    {
                                        /* State */
                                        PCI_DELETED,
                                        {
                                            /* appCb */
                                            NULL,
                                            /* appData */
                                            NULL
                                        },
                                        /* pciSpaceAccessLock */
                                        NULL,
                                        /* pciBaseAddr */
                                        NULL
                                    }
                                };
/*
 * PCI ISR. This ISR will be called on occurance of PCI interrupt. If
 * application callback is registered. This ISR will call the callback function.
 */
static void pciIsr(Uint32 instId);
/**
 * PAL_sysPCICreate
 * \brief    Create the instance of PCI driver. Check for the maximum instance
 *            number. If instance number is proper, change the state of driver.
 *            This function is just to bring consistancy in PAL SYS drivers.
 *
 *
 * \param   instId         [IN]        Instance Id of PCI controller
 * \param    param          [IN]        Pointer parameters required for creating
 *                                    PCI driver handle
 *
 * \return    PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCICreate(Uint32 instId, Ptr param)
{
    PAL_Result status = PAL_SYS_PCI_INVALID_PARAM;
    Bool  boolFalse    =    FALSE;

    /* Added to remove remarks while compilation */
    if(NULL != param)
    {
        param    =    param;
    }

    /* Check whether the instance number is proper or not */
    if(CSL_PCI_PER_CNT > instId)
    {
        do
        {
            /* Check whether the driver is in deleted state or not */
            if(PCI_DELETED != pciInstObj[instId].state )
            {
                status = PAL_SYS_PCI_OBJ_NOT_DELETED;
                break;
            }
            /*
             * Create semaphores to protect multiple access of memory mapped
             * config space
             */
            status = PAL_osSemCreate("pciSpaceAccessLock",
                                    1,
                                    NULL,
                                    &pciInstObj[instId].pciSpaceAccessLock);
            if(PAL_SOK    != status)
            {
                break;
            }

            pciInstObj[instId].state = PCI_CREATED;
            status = PAL_SOK;
            boolFalse    =    FALSE;
        }while(boolFalse);
    }
    return status;
}
/**
 * PAL_sysPCIDelete
 * \brief    Delete the instance of PCI driver. Check for the maximum instance
 *           number. If instance number is proper, change the state of driver.
 *           This function is just to bring consistancy in PAL SYS drivers.
 *
 * \param   instId         [IN]        Instance Id of PCI controller
 *
 * \return    PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCIDelete(Uint32 instId)
{
    PAL_Result status = PAL_SYS_PCI_INVALID_PARAM;
    Bool    boolFalse    =    FALSE;
    if(CSL_PCI_PER_CNT > instId)
    {
        if( (PCI_CLOSED == pciInstObj[instId].state) ||
            (PCI_CREATED == pciInstObj[instId].state))
        {
            do
            {
                status = PAL_osSemDelete(pciInstObj[instId].pciSpaceAccessLock);
                if(PAL_SOK    != status)
                {
                    break;
                }
                status = PAL_SOK;
                pciInstObj[instId].state = PCI_DELETED;

                boolFalse    =    FALSE;
            }while(boolFalse);
        }
        else
        {
            status = PAL_SYS_PCI_OBJ_NOT_CLOSED;
        }
    }
    return status;
}
/**
 * PAL_sysPCIOpen
 * \brief     Open the driver. Check for the maximum instance number. If the
 *            the handle is proper Initialize the address of configuration space
 *            registers, memory mapped registers, address translation register
 *            and hook configuration register base addresses. Register the
 *            interrupt callback for PCI interrupt. This callback will receive
 *            the interrupt status register as one argument and application data
 *            pointer as other argument on occurance of interrupt.
 *
 * \param   instId         [IN]     Instance Id of PCI controller
 * \param   param          [IN]     Pointer parameters required for opening
 *                                  PCI driver handle
 *
 * \return    PSP_Handle if successful else NULL if unsuccessful
 */
PSP_Handle PAL_sysPCIOpen(Uint32 instId, const PAL_sysPciInitConfig *param)
{
    pciInstanceObject *hPci = NULL;
    Uint32 disAbleIntMask = (0u);
#ifdef PCI_ECM_HOOK_ENABLE
    ECM_Attrs ecmattrs = ECM_ATTRS;
#else
    HWI_Attrs hwiattrs = HWI_ATTRS;
#endif /* PCI_ECM_HOOK_ENABLE */

        if(    (CSL_PCI_PER_CNT > instId) &&
        (NULL    !=    param))
    {
        if( (PCI_CREATED == pciInstObj[instId].state)||
            (PCI_CLOSED == pciInstObj[instId].state))
        {
            /* Initialize the base address of various PCI register spaces */
            pciInstObj[instId].pciBaseAddr  =
                                            (volatile Uint32 *)CSL_PCI_0_REGS;

            /* Assign the configuration parameters to proper structure */
            pciInstObj[instId].cfgParams.appCb  =
                                            (PAL_sysPciAppCallback)param->appCb;
            pciInstObj[instId].cfgParams.appData =
                                            param->appData;
#ifdef PCI_ECM_HOOK_ENABLE
            /* Disabling the interrupts */
            PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &disAbleIntMask);

            /* Register PCI ISR with ECM and enable the event */
            ecmattrs.arg = (Arg)instId;
            ecmattrs.unmask = 1u;
            ECM_dispatchPlug(   CSL_INTC_EVENTID_PCIINT,
                                (ECM_Fxn)&pciIsr,
                                &ecmattrs);
            ECM_enableEvent(CSL_INTC_EVENTID_PCIINT);

               /* Enabling the hardware interrupt */
            C64_enableIER(CSL_PCI_INT_ENABLE_MASK);
            /* Enabling the interrupts */
            PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, disAbleIntMask);
#else
            /* Disabling the interrupts */
            PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &disAbleIntMask);

            /* Register PCI ISR with ECM and enable the event */
            hwiattrs.arg = (Arg)instId;
            /*Hardware interrupt source mapping with PCI event */
            HWI_eventMap( INT_SOURCE_FOR_PCI, CSL_INTC_EVENTID_PCIINT );
            HWI_dispatchPlug(   INT_SOURCE_FOR_PCI,
                                (Fxn)&pciIsr,
                                -1,
                                &hwiattrs );
            /* Enabling the hardware interrupt */
            C64_enableIER(HWI_PCI_IRQ);
            /* Enabling the interrupts */
            PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, disAbleIntMask);
#endif /* PCI_ECM_HOOK_ENABLE */

            pciInstObj[instId].state = PCI_OPENED;
            hPci = &pciInstObj[instId];
        }
    }

    return ((PSP_Handle)(hPci));
}
/**
 * PAL_sysPCIClose
 * \brief     Close the PCI driver. Check the instance number and state of
 *            driver. If state and instance number are proper uninitialize the
 *            interrupt routine. Uninitialize the base addresses of
 *            configuration space register, memory mapped registers, hook
 *            configuration registers and slave address translation register by
 *            NULL. Change the state of driver instance to CLOSED.
 *
 * \param   hPci           [IN]        Handle of PCI controller driver
 * \param   param          [IN]        Pointer parameters required for closing
 *                                     PCI driverhandle
 *
 * \return    PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCIClose(PSP_Handle hPci, Ptr param)
{
    pciInstanceHandle    pciHandle    =    (pciInstanceHandle)hPci;
    PAL_Result status = PAL_SYS_PCI_INVALID_PARAM;

    Uint32 disAbleIntMask    =    (0u);
    /* Added to remove remarks */
    if(NULL    !=    param)
    {
        param    =    param;
    }
    if(NULL != pciHandle)
    {
        if(PCI_OPENED == pciHandle->state)
        {
            /* Initialize the base address of various PCI register spaces */
            pciHandle->pciBaseAddr            =    NULL;

            /* Assign the configuration parameters to proper structure */
            pciHandle->cfgParams.appCb                    =    NULL;
            pciHandle->cfgParams.appData                    =    NULL;
            /* Disabling the interrupts */
            PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &disAbleIntMask);
#ifdef PCI_ECM_HOOK_ENABLE
            /* Register PCI ISR with ECM and enable the event */
            ECM_disableEvent(CSL_INTC_EVENTID_PCIINT);
#endif
            /* Enabling the interrupts */
            PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, disAbleIntMask);

            pciHandle->state = PCI_CLOSED;

            status    =    PAL_SOK;
        }
        else
        {
            status = PAL_SYS_PCI_OBJ_NOT_OPENED;
        }
    }
    return status;
}
/**
 * PAL_sysPCIEnableInterrupt
 * \brief    Enables  PCI interrupt/s. Depending upon the mask passed interrupts
 *            are enabled. If a particular bit is set, corresponding interrupt
 *            is enabled.
 *
 * \param   hPci           [IN]     Handle of PCI controller driver
 * \param   intEnableCode  [IN]     Interrupt enable mask. 5 LSBs of
 *                                  intEnableCode will correspond to 5
 *                                  interrupts of PCI. If a particular bit is
 *                                  set, corresponding interrupt would be
 *                                  enabled.
 *
 * \return    PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCIEnableInterrupt(PSP_Handle hPci, Uint32 intEnableCode)
{
    PAL_Result status = PAL_SYS_PCI_INVALID_PARAM;
    pciInstanceHandle    pciHandle    =    (pciInstanceHandle)hPci;
    PAL_sysPciMemMapField    hostIntEnableRegOffset =
                                        PAL_SYSPCI_HOST_INT_ENABLE_SET;
    PAL_sysPciMemMapField    memMapAppIntEnableRegOffset =
                                        PAL_SYSPCI_DSP_INT_ENABLE_SET;
    volatile Uint32*    ptrHostIntEnableReg = NULL;
    volatile Uint32*    ptrBackEndAppIntEnableReg = NULL;
    Uint32    tempHostIntEnableReg    =    0;
    Uint32    tempBackEndAppIntEnableReg    =    0;
    Uint32    disAbleIntMask    =    0;

    if(NULL    != pciHandle)
    {
        if(PCI_OPENED != pciHandle->state)
        {
            return PAL_SYS_PCI_OBJ_NOT_OPENED;
        }
        /* Get the lock */
        if(NULL != pciHandle->pciSpaceAccessLock)
        {
            if(    PAL_SOK !=
                PAL_osSemTake(pciHandle->pciSpaceAccessLock,(Int32)SYS_FOREVER))
            {
                return PAL_SYS_PCI_SEM_ERR;
            }
        }
        else
        {
            return PAL_SYS_PCI_OBJ_NOT_OPENED;
        }
        /*
         * Get addresses of Host interrupt enable register and DSP
         * interrupt enable register
         */
        ptrHostIntEnableReg     =   (volatile Uint32 *)(
                                    (Uint8 *)pciHandle->pciBaseAddr +
                                    (Uint32)hostIntEnableRegOffset);
        ptrBackEndAppIntEnableReg  =(volatile Uint32 *)(
                                    (Uint8 *)pciHandle->pciBaseAddr +
                                    (Uint32)memMapAppIntEnableRegOffset);
        tempHostIntEnableReg    =    *ptrHostIntEnableReg;
        tempBackEndAppIntEnableReg    =    *ptrBackEndAppIntEnableReg;

        /* Enable appropriate interrupt bit */
        if( PAL_SYS_PCI_HOST_INT_TGT_ABORT    ==
            (intEnableCode & PAL_SYS_PCI_HOST_INT_TGT_ABORT))
        {
            tempHostIntEnableReg    |=    PCI_HOST_INT_SET_CLEAR_MASK_TGT_ABORT;
            status = PAL_SOK;
        }
        if( PAL_SYS_PCI_HOST_INT_MS_ABORT    ==
            (intEnableCode & PAL_SYS_PCI_HOST_INT_MS_ABORT))
        {
            tempHostIntEnableReg    |=    PCI_HOST_INT_SET_CLEAR_MASK_MS_ABORT;
            status = PAL_SOK;
        }
        if( PAL_SYS_PCI_HOST_INT_SYS_ERR_DETECT    ==
            (intEnableCode & PAL_SYS_PCI_HOST_INT_SYS_ERR_DETECT))
        {
            tempHostIntEnableReg    |=
                                    PCI_HOST_INT_SET_CLEAR_MASK_SYS_ERR_DETECT;
            status = PAL_SOK;
        }
        if( PAL_SYS_PCI_HOST_INT_PARITY_ERR_DETECT    ==
            (intEnableCode & PAL_SYS_PCI_HOST_INT_PARITY_ERR_DETECT))
        {
            tempHostIntEnableReg    |=
                                PCI_HOST_INT_SET_CLEAR_MASK_PARITY_ERR_DETECT;
            status = PAL_SOK;
        }
        if( PAL_SYS_PCI_HOST_INT_SOFTINT0    ==
            (intEnableCode & PAL_SYS_PCI_HOST_INT_SOFTINT0))
        {
            tempHostIntEnableReg    |=
                                PCI_HOST_INT_SET_CLEAR_MASK_SOFTINT0;
            status = PAL_SOK;
        }
        if( PAL_SYS_PCI_HOST_INT_SOFTINT1    ==
            (intEnableCode & PAL_SYS_PCI_HOST_INT_SOFTINT1))
        {
            tempHostIntEnableReg    |=
                                PCI_HOST_INT_SET_CLEAR_MASK_SOFTINT1;
            status = PAL_SOK;
        }
        if( PAL_SYS_PCI_HOST_INT_SOFTINT2    ==
            (intEnableCode & PAL_SYS_PCI_HOST_INT_SOFTINT2))
        {
            tempHostIntEnableReg    |=
                                PCI_HOST_INT_SET_CLEAR_MASK_SOFTINT2;
            status = PAL_SOK;
        }
        if( PAL_SYS_PCI_HOST_INT_SOFTINT3    ==
            (intEnableCode & PAL_SYS_PCI_HOST_INT_SOFTINT3))
        {
            tempHostIntEnableReg    |=
                                PCI_HOST_INT_SET_CLEAR_MASK_SOFTINT3;
            status = PAL_SOK;
        }


        if( PAL_SYS_PCI_DSP_INT_TGT_ABORT    ==
            (intEnableCode & PAL_SYS_PCI_DSP_INT_TGT_ABORT))
        {
            tempBackEndAppIntEnableReg    |=
                                PCI_DSP_INT_SET_CLEAR_MASK_TGT_ABORT;
            status = PAL_SOK;
        }
        if( PAL_SYS_PCI_DSP_INT_MS_ABORT            ==
            (intEnableCode & PAL_SYS_PCI_DSP_INT_MS_ABORT))
        {
            tempBackEndAppIntEnableReg    |=
                                PCI_DSP_INT_SET_CLEAR_MASK_MS_ABORT;
            status = PAL_SOK;
        }
        if( PAL_SYS_PCI_DSP_INT_SYS_ERR_DETECT    ==
            (intEnableCode & PAL_SYS_PCI_DSP_INT_SYS_ERR_DETECT))
        {
            tempBackEndAppIntEnableReg    |=
                            PCI_DSP_INT_SET_CLEAR_MASK_SYS_ERR_DETECT;
            status = PAL_SOK;
        }
        if( PAL_SYS_PCI_DSP_INT_PARITY_ERR_DETECT    ==
            (intEnableCode & PAL_SYS_PCI_DSP_INT_PARITY_ERR_DETECT))
        {
            tempBackEndAppIntEnableReg    |=
                        PCI_DSP_INT_SET_CLEAR_MASK_PARITY_ERR_DETECT;
            status = PAL_SOK;
        }
        if( PAL_SYS_PCI_DSP_INT_SOFTINT0    ==
            (intEnableCode & PAL_SYS_PCI_DSP_INT_SOFTINT0))
        {
            tempBackEndAppIntEnableReg    |=
                        PCI_DSP_INT_SET_CLEAR_MASK_SOFTINT0;
            status = PAL_SOK;
        }
        if( PAL_SYS_PCI_DSP_INT_SOFTINT1    ==
            (intEnableCode & PAL_SYS_PCI_DSP_INT_SOFTINT1))
        {
            tempBackEndAppIntEnableReg    |=
                        PCI_DSP_INT_SET_CLEAR_MASK_SOFTINT1;
            status = PAL_SOK;
        }
        if( PAL_SYS_PCI_DSP_INT_SOFTINT2    ==
            (intEnableCode & PAL_SYS_PCI_DSP_INT_SOFTINT2))
        {
            tempBackEndAppIntEnableReg    |=
                        PCI_DSP_INT_SET_CLEAR_MASK_SOFTINT2;
            status = PAL_SOK;
        }
        if( PAL_SYS_PCI_DSP_INT_SOFTINT3    ==
            (intEnableCode & PAL_SYS_PCI_DSP_INT_SOFTINT3))
        {
            tempBackEndAppIntEnableReg    |=
                        PCI_DSP_INT_SET_CLEAR_MASK_SOFTINT3;
            status = PAL_SOK;
        }

        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &disAbleIntMask);
        *ptrHostIntEnableReg    =    tempHostIntEnableReg;
        *ptrBackEndAppIntEnableReg    =    tempBackEndAppIntEnableReg;
        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, disAbleIntMask);

        /* Release the lock */
        if(NULL != pciHandle->pciSpaceAccessLock)
        {
            if(PAL_SOK != PAL_osSemGive(pciHandle->pciSpaceAccessLock))
            {
                return PAL_SYS_PCI_SEM_ERR;
            }
        }
        else
        {
            return PAL_SYS_PCI_OBJ_NOT_OPENED;
        }
    }
    return status;
}


/**
 * PAL_sysPCIDisableInterrupt
 * \brief   Disables  PCI interrupt/s. Depending upon the mask passed interrupts
 *          are disabled. If a particular bit is set, corresponding interrupt
 *          is disabled.
 *
 * \param    hPci           [IN]    Handle of PCI controller driver
 * \param    intDisableCode [IN]    Interrupt disable mask. 5 LSBs of
 *                                  intDisableCode will correspond to 5
 *                                  interrupts of PCI. If a particular bit is
 *                                  set, corresponding interrupt would be
 *                                  disabled.
 *
 * \return    PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCIDisableInterrupt(PSP_Handle hPci, Uint32 intDisableCode)
{
    pciInstanceHandle    pciHandle    =    (pciInstanceHandle)hPci;
    PAL_Result status = PAL_SYS_PCI_INVALID_PARAM;
    PAL_sysPciMemMapField    hostIntDisableRegOffset   =
                                    PAL_SYSPCI_HOST_INT_ENABLE_CLEAR;
    PAL_sysPciMemMapField    memMapAppIntDisableRegOffset =
                                        PAL_SYSPCI_DSP_INT_ENABLE_CLEAR;
    volatile Uint32*    ptrHostIntDisableReg = NULL;
    volatile Uint32*    ptrBackEndAppIntDisableReg = NULL;
    Uint32    tempHostIntDisableReg    =    0;
    Uint32    tempBackEndAppIntDisableReg    =    0;
    Uint32    disAbleIntMask    =    0;

    if(NULL    != pciHandle)
    {
        if(PCI_OPENED != pciHandle->state)
        {
            return PAL_SYS_PCI_OBJ_NOT_OPENED;
        }

        /* Get the lock */
        if(NULL != pciHandle->pciSpaceAccessLock)
        {
            if(    PAL_SOK !=
                PAL_osSemTake(pciHandle->pciSpaceAccessLock,(Int32)SYS_FOREVER))
            {
                return PAL_SYS_PCI_SEM_ERR;
            }
        }
        else
        {
            return PAL_SYS_PCI_OBJ_NOT_OPENED;
        }
        /*
         * Get address of host interrupt disable register and DSP Interrupt
         * disable register
         */
        ptrHostIntDisableReg    =   (volatile Uint32 *)(
                                    (Uint8 *)pciHandle->pciBaseAddr +
                                    (Uint32)hostIntDisableRegOffset);
        ptrBackEndAppIntDisableReg =(volatile Uint32 *)(
                                    (Uint8 *)pciHandle->pciBaseAddr +
                                    (Uint32)memMapAppIntDisableRegOffset);
        tempHostIntDisableReg    =    *ptrHostIntDisableReg;
        tempBackEndAppIntDisableReg    =    *ptrBackEndAppIntDisableReg;

        /* Set appropriate interrupt bit to clear interrupt */
        if( PAL_SYS_PCI_HOST_INT_TGT_ABORT    ==
            (intDisableCode & PAL_SYS_PCI_HOST_INT_TGT_ABORT))
        {
            tempHostIntDisableReg    |=   PCI_HOST_INT_SET_CLEAR_MASK_TGT_ABORT;
            status = PAL_SOK;
        }
        if( PAL_SYS_PCI_HOST_INT_MS_ABORT    ==
            (intDisableCode & PAL_SYS_PCI_HOST_INT_MS_ABORT))
        {
            tempHostIntDisableReg    |=    PCI_HOST_INT_SET_CLEAR_MASK_MS_ABORT;
            status = PAL_SOK;
        }
        if(PAL_SYS_PCI_HOST_INT_SYS_ERR_DETECT    ==
            (intDisableCode & PAL_SYS_PCI_HOST_INT_SYS_ERR_DETECT))
        {
            tempHostIntDisableReg   |=
                                    PCI_HOST_INT_SET_CLEAR_MASK_SYS_ERR_DETECT;
            status = PAL_SOK;
        }
        if(PAL_SYS_PCI_HOST_INT_PARITY_ERR_DETECT    ==
            (intDisableCode & PAL_SYS_PCI_HOST_INT_PARITY_ERR_DETECT))
        {
            tempHostIntDisableReg   |=
                                PCI_HOST_INT_SET_CLEAR_MASK_PARITY_ERR_DETECT;
            status = PAL_SOK;
        }
        if(PAL_SYS_PCI_HOST_INT_SOFTINT0    ==
            (intDisableCode & PAL_SYS_PCI_HOST_INT_SOFTINT0))
        {
            tempHostIntDisableReg   |=
                                PCI_HOST_INT_SET_CLEAR_MASK_SOFTINT0;
            status = PAL_SOK;
        }
        if(PAL_SYS_PCI_HOST_INT_SOFTINT1    ==
            (intDisableCode & PAL_SYS_PCI_HOST_INT_SOFTINT1))
        {
            tempHostIntDisableReg   |=
                                PCI_HOST_INT_SET_CLEAR_MASK_SOFTINT1;
            status = PAL_SOK;
        }
        if(PAL_SYS_PCI_HOST_INT_SOFTINT2    ==
            (intDisableCode & PAL_SYS_PCI_HOST_INT_SOFTINT2))
        {
            tempHostIntDisableReg   |=
                                PCI_HOST_INT_SET_CLEAR_MASK_SOFTINT2;
            status = PAL_SOK;
        }
        if(PAL_SYS_PCI_HOST_INT_SOFTINT3    ==
            (intDisableCode & PAL_SYS_PCI_HOST_INT_SOFTINT3))
        {
            tempHostIntDisableReg   |=
                                PCI_HOST_INT_SET_CLEAR_MASK_SOFTINT3;
            status = PAL_SOK;
        }

        if(PAL_SYS_PCI_DSP_INT_TGT_ABORT    ==
            (intDisableCode & PAL_SYS_PCI_DSP_INT_TGT_ABORT))
        {
            tempBackEndAppIntDisableReg |=
                                PCI_DSP_INT_SET_CLEAR_MASK_TGT_ABORT;
            status = PAL_SOK;
        }
        if(PAL_SYS_PCI_DSP_INT_MS_ABORT    ==
            (intDisableCode & PAL_SYS_PCI_DSP_INT_MS_ABORT))
        {
            tempBackEndAppIntDisableReg |=
                                    PCI_DSP_INT_SET_CLEAR_MASK_MS_ABORT;
            status = PAL_SOK;
        }
        if(PAL_SYS_PCI_DSP_INT_SYS_ERR_DETECT    ==
            (intDisableCode & PAL_SYS_PCI_DSP_INT_SYS_ERR_DETECT))
        {
            tempBackEndAppIntDisableReg |=
                            PCI_DSP_INT_SET_CLEAR_MASK_SYS_ERR_DETECT;
            status = PAL_SOK;
        }
        if(PAL_SYS_PCI_DSP_INT_PARITY_ERR_DETECT    ==
            (intDisableCode & PAL_SYS_PCI_DSP_INT_PARITY_ERR_DETECT))
        {
            tempBackEndAppIntDisableReg |=
                        PCI_DSP_INT_SET_CLEAR_MASK_PARITY_ERR_DETECT;
            status = PAL_SOK;
        }
        if(PAL_SYS_PCI_DSP_INT_SOFTINT0    ==
            (intDisableCode & PAL_SYS_PCI_DSP_INT_SOFTINT0))
        {
            tempBackEndAppIntDisableReg |=
                        PCI_DSP_INT_SET_CLEAR_MASK_SOFTINT0;
            status = PAL_SOK;
        }
        if(PAL_SYS_PCI_DSP_INT_SOFTINT1    ==
            (intDisableCode & PAL_SYS_PCI_DSP_INT_SOFTINT1))
        {
            tempBackEndAppIntDisableReg |=
                        PCI_DSP_INT_SET_CLEAR_MASK_SOFTINT1;
            status = PAL_SOK;
        }
        if(PAL_SYS_PCI_DSP_INT_SOFTINT2    ==
            (intDisableCode & PAL_SYS_PCI_DSP_INT_SOFTINT2))
        {
            tempBackEndAppIntDisableReg |=
                        PCI_DSP_INT_SET_CLEAR_MASK_SOFTINT2;
            status = PAL_SOK;
        }
        if(PAL_SYS_PCI_DSP_INT_SOFTINT3    ==
            (intDisableCode & PAL_SYS_PCI_DSP_INT_SOFTINT3))
        {
            tempBackEndAppIntDisableReg |=
                        PCI_DSP_INT_SET_CLEAR_MASK_SOFTINT3;
            status = PAL_SOK;
        }


        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &disAbleIntMask);
        *ptrHostIntDisableReg            =    tempHostIntDisableReg;
        *ptrBackEndAppIntDisableReg        =    tempBackEndAppIntDisableReg;
        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, disAbleIntMask);

        /* Release the lock */
        if(NULL != pciHandle->pciSpaceAccessLock)
        {

            if(PAL_SOK != PAL_osSemGive(pciHandle->pciSpaceAccessLock))
            {
                return PAL_SYS_PCI_SEM_ERR;
            }
        }
        else
        {
            return PAL_SYS_PCI_OBJ_NOT_OPENED;
        }

    }
    return status;
}



/**
 * PAL_sysPCISetMemMapReg
 * \brief    Set memory mapped register. Check for the handle. If handle is
 *           proper set the specified 32-bit register by the specified value.
 *
 * \param    hPci           [IN]        Handle of PCI controller driver
 * \param    memMapField    [IN]        Field of memory mapped register that has
 *                                       to be set
 * \param    writeSize      [IN]        Size of data to be set (byte/word/dword)
 * \param    newFieldVal    [IN]        New value of field that is to be set
 *
 * \return    PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCISetMemMapReg(  PSP_Handle hPci,
                                    PAL_sysPciMemMapField memMapField,
                                    PAL_sysPciConfigFieldSize    writeSize,
                                    Uint32 newFieldVal)
{
    pciInstanceHandle    pciHandle    =    (pciInstanceHandle)hPci;
    PAL_Result status = PAL_SYS_PCI_INVALID_PARAM;
    Uint32    offsetFrom32BitAlignedAddr = 0u;
    volatile Uint32* ptrActualLocationToSet = NULL;
    Uint32 tempVal = 0u;
    volatile Uint32*    ptrPciBaseAddrRegs    =    NULL;
    if(NULL != pciHandle)
    {
        if(PCI_OPENED != pciHandle->state)
        {
            return PAL_SYS_PCI_OBJ_NOT_OPENED;
        }
        if(NULL != pciHandle->pciSpaceAccessLock)
        {
            /* Take the lock to avoid multiple access */
            if(    PAL_SOK !=
                PAL_osSemTake(pciHandle->pciSpaceAccessLock,(Int32)SYS_FOREVER))
            {
                return PAL_SYS_PCI_SEM_ERR;
            }
        }
        else
        {
            return PAL_SYS_PCI_OBJ_NOT_OPENED;
        }
        /*
         * Get the base address of memory mapped configuration register in temporary
         * pointer
         */
        ptrPciBaseAddrRegs    =    pciHandle->pciBaseAddr;
        /* If the offset is 32-bit aligned set the value directly */
        if( (PAL_SYS_PCI_READ_WRITE_SIZE_WORD == writeSize) &&
            ( (0u    ==    ( (Uint32)memMapField & 0x3u ))     &&
              (0u    ==    ( (Uint32)ptrPciBaseAddrRegs & 0x3u )) ) )
        {
            ptrActualLocationToSet =    (volatile Uint32 *)(
                                        (Uint8 *)ptrPciBaseAddrRegs +
                                        (Uint32)memMapField);
            *ptrActualLocationToSet    =    newFieldVal;
            status = PAL_SOK;
        }
        else if((PAL_SYS_PCI_READ_WRITE_SIZE_HALFWORD == writeSize) &&
                ((0u)    ==    (((Uint32)ptrPciBaseAddrRegs) & (0x1u))))
        {
            /*
             * Get the offset of the location to be set from the nearest
             * 32-bit aligned address
             */
            offsetFrom32BitAlignedAddr = (Uint32)memMapField & (0x3u);
            if(    (0u    == offsetFrom32BitAlignedAddr )||
                (2u    == offsetFrom32BitAlignedAddr ))
            {
                ptrActualLocationToSet    = (volatile Uint32 *)(
                                            ((Uint8 *)ptrPciBaseAddrRegs) +
                                            ((Uint32)memMapField));
                /* Mask off the bits to be set */
                tempVal =   (*ptrActualLocationToSet) &
                            ((0xffff0000u)>>((8u)*offsetFrom32BitAlignedAddr));
                /* Set the requried bits */
                tempVal |=  (newFieldVal &
                            ((0xffffu)<<((8u)*offsetFrom32BitAlignedAddr)));
                (*ptrActualLocationToSet)    =    tempVal;
                status = PAL_SOK;
            }
            else
            {
                status = PAL_SYS_PCI_INVALID_PARAM;
            }
        }
        else if(PAL_SYS_PCI_READ_WRITE_SIZE_BYTE == writeSize)
        {
            offsetFrom32BitAlignedAddr = (Uint32)memMapField & (0x3u);

            if((    (0u    == offsetFrom32BitAlignedAddr ) ||
                    (1u    == offsetFrom32BitAlignedAddr ))||
                (   (2u    == offsetFrom32BitAlignedAddr ) ||
                    (3u    == offsetFrom32BitAlignedAddr )))
            {
                ptrActualLocationToSet  =   (volatile Uint32 *)(
                                            (Uint8 *)ptrPciBaseAddrRegs +
                                            (Uint32)memMapField);
                /* Mask off the bits to be set */
                tempVal =   ((*ptrActualLocationToSet) &
                            (~((0xffu)<<((8u)*offsetFrom32BitAlignedAddr))));
                /* Set the requried bits */
                tempVal |=  (newFieldVal &
                            ((0xffu)<<((8u)*offsetFrom32BitAlignedAddr)));
                (*ptrActualLocationToSet)    =    tempVal;
                status = PAL_SOK;
            }
            else
            {
                status = PAL_SYS_PCI_INVALID_PARAM;
            }

        }
        else
        {
            status = PAL_SYS_PCI_INVALID_PARAM;
        }
        if(NULL != pciHandle->pciSpaceAccessLock)
        {
            /* Release the lock */
            if(PAL_SOK != PAL_osSemGive(pciHandle->pciSpaceAccessLock))
            {
                return PAL_SYS_PCI_SEM_ERR;
            }
        }
        else
        {
            return PAL_SYS_PCI_OBJ_NOT_OPENED;
        }
    }
    return status;
}


/**
 * PAL_sysPCIGetMemMapReg
 * \brief    Get memory mapped register. Check for the handle. If handle is
 *           proper get the specified 32-bit register in specified variable.
 *
 * \param    hPci           [IN]    Handle of PCI controller driver
 * \param    memMapField    [IN]    Field of memory mapped register that has
 *                                  to be read
 * \param    readSize       [IN]    Size of data to be read (byte/word/dword)
 * \param    *newFieldVal   [OUT]   Pointer to new value of field that is
 *                                  to be read from hardware
 *
 * \return    PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCIGetMemMapReg( PSP_Handle hPci,
                                    PAL_sysPciMemMapField memMapField,
                                    PAL_sysPciConfigFieldSize    readSize,
                                    Uint32* newFieldVal)
{
    pciInstanceHandle    pciHandle    =    (pciInstanceHandle)hPci;
    PAL_Result status = PAL_SYS_PCI_INVALID_PARAM;
    Uint32    offsetFrom32BitAlignedAddr = 0u;
    volatile Uint32* ptrActualLocationToGet = NULL;
    Uint32 tempVal = 0u;
    volatile Uint32*    ptrPciBaseAddrRegs    =    NULL;

    if(    (NULL != pciHandle)    &&
        (NULL    !=    newFieldVal))
    {
        if(PCI_OPENED != pciHandle->state)
        {
            return PAL_SYS_PCI_OBJ_NOT_OPENED;
        }
        ptrPciBaseAddrRegs    =    pciHandle->pciBaseAddr;

        /* If the offset is 32-bit aligned read the appropriate value */
        if(    (PAL_SYS_PCI_READ_WRITE_SIZE_WORD == readSize)     &&
            ( ( 0u    ==    ((Uint32)memMapField        & 0x3u) )    &&
              ( 0u    ==    ((Uint32)ptrPciBaseAddrRegs & 0x3u) ) ) )
        {
            ptrActualLocationToGet  =   (volatile Uint32 *)(
                                        ((Uint8 *)ptrPciBaseAddrRegs) +
                                        ((Uint32)memMapField));
            (*newFieldVal) = *ptrActualLocationToGet;

            status = PAL_SOK;
        }
        else if((PAL_SYS_PCI_READ_WRITE_SIZE_HALFWORD == readSize)    &&
                ((0u)    ==    (((Uint32)ptrPciBaseAddrRegs) & (0x1u))))
        {
            /*
             * Get the offset of the location to be set from the nearest
             * 32-bit aligned address
             */
            offsetFrom32BitAlignedAddr = (Uint32)memMapField & (0x3u);

            if( (0u    == offsetFrom32BitAlignedAddr )||
                (2u    == offsetFrom32BitAlignedAddr ))
            {
                ptrActualLocationToGet  =   (volatile Uint32 *)(
                                            (((Uint8 *)ptrPciBaseAddrRegs) +
                                            ((Uint32)memMapField))    -
                                            offsetFrom32BitAlignedAddr);

                tempVal =   (*ptrActualLocationToGet);
                tempVal =   tempVal &
                            ((0xffffu) << ((8u)*offsetFrom32BitAlignedAddr));
                (*newFieldVal)  = tempVal >> ((8u)*offsetFrom32BitAlignedAddr);

                status = PAL_SOK;
            }
            else
            {
                status = PAL_SYS_PCI_INVALID_PARAM;
            }
        }
        else if(PAL_SYS_PCI_READ_WRITE_SIZE_BYTE == readSize)
        {
            offsetFrom32BitAlignedAddr  = (Uint32)memMapField & (0x3u);
            ptrActualLocationToGet      = (volatile Uint32 *)(
                                          (((Uint8 *)ptrPciBaseAddrRegs) +
                                          ((Uint32)memMapField)) -
                                          offsetFrom32BitAlignedAddr);

            tempVal = (*ptrActualLocationToGet);
            tempVal = tempVal & ((0xffu) << ((8u)*offsetFrom32BitAlignedAddr));
            *newFieldVal    =    tempVal >> ((8u)*offsetFrom32BitAlignedAddr);

            status = PAL_SOK;
        }
        else
        {
            status = PAL_SYS_PCI_INVALID_PARAM;
        }

    }
    return status;
}

/**
 * PAL_sysPCISetHookReg
 * \brief    Set Hook register. Check for the handle. If handle is proper
 *            set the specified 32-bit register by the specified value.
 *
 * \param    hPci           [IN]        Handle of PCI controller driver
 * \param    hookRegField   [IN]        Field of hook register that has
 *                                       to be set
 * \param    writeSize      [IN]        Size of data to be set (byte/word/dword)
 * \param    newFieldVal    [IN]        New value of field that is to be set
 *
 * \return    PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCISetHookReg(    PSP_Handle hPci,
                                    PAL_sysPciHookRegister hookRegField,
                                    PAL_sysPciConfigFieldSize    writeSize,
                                    Uint32 newFieldVal)
{
    pciInstanceHandle    pciHandle    =    (pciInstanceHandle)hPci;
    PAL_Result status = PAL_SYS_PCI_INVALID_PARAM;
    Uint32    offsetFrom32BitAlignedAddr = 0u;
    volatile Uint32* ptrActualLocationToSet = NULL;
    Uint32 tempVal = 0u;
    volatile Uint32*    ptrPciBaseAddrRegs    =    NULL;

    if(NULL != pciHandle)
    {
        if(PCI_OPENED != pciHandle->state)
        {
            return PAL_SYS_PCI_OBJ_NOT_OPENED;
        }
        if(NULL != pciHandle->pciSpaceAccessLock)
        {
            /* Take the lock to avoid multiple access */
            if(    PAL_SOK !=
                PAL_osSemTake(pciHandle->pciSpaceAccessLock,(Int32)SYS_FOREVER))
            {
                return PAL_SYS_PCI_SEM_ERR;
            }
        }
        else
        {
            return PAL_SYS_PCI_OBJ_NOT_OPENED;
        }
        /*
         * Get the base address of hook register in temporary pointer
         */
        ptrPciBaseAddrRegs    =    pciHandle->pciBaseAddr;
        /* If the offset is 32-bit aligned set the value directly */
        if( (PAL_SYS_PCI_READ_WRITE_SIZE_WORD == writeSize) &&
            ( (0u    ==    ((Uint32)hookRegField       & 0x3u))    &&
              (0u    ==    ((Uint32)ptrPciBaseAddrRegs & 0x3u)) ) )
        {
            ptrActualLocationToSet =    (volatile Uint32 *)(
                                        (Uint8 *)ptrPciBaseAddrRegs +
                                        (Uint32)hookRegField);
            *ptrActualLocationToSet    =    newFieldVal;
            status = PAL_SOK;
        }
        else if((PAL_SYS_PCI_READ_WRITE_SIZE_HALFWORD == writeSize)&&
                ((0u)    ==    (((Uint32)ptrPciBaseAddrRegs) & (0x1u))))
        {
            /*
             * Get the offset of the location to be set from the nearest
             * 32-bit aligned address
             */
            offsetFrom32BitAlignedAddr = (Uint32)hookRegField & (0x3u);
            if( (0u    == offsetFrom32BitAlignedAddr )||
                (2u    == offsetFrom32BitAlignedAddr ))
            {
                ptrActualLocationToSet    = (volatile Uint32 *)(
                                            ((Uint8 *)ptrPciBaseAddrRegs) +
                                            ((Uint32)hookRegField));
                /* Mask off the bits to be set */
                tempVal =   (*ptrActualLocationToSet) &
                            ((0xffff0000u)>>((8u)*offsetFrom32BitAlignedAddr));
                /* Set the requried bits */
                tempVal |=  (newFieldVal &
                            ((0xffffu)<<((8u)*offsetFrom32BitAlignedAddr)));
                (*ptrActualLocationToSet)    =    tempVal;
                status = PAL_SOK;
            }
            else
            {
                status = PAL_SYS_PCI_INVALID_PARAM;
            }
        }
        else if(PAL_SYS_PCI_READ_WRITE_SIZE_BYTE == writeSize)
        {
            offsetFrom32BitAlignedAddr = (Uint32)hookRegField & (0x3u);

            if((    (0u    == offsetFrom32BitAlignedAddr )  ||
                    (1u    == offsetFrom32BitAlignedAddr )) ||
                (   (2u    == offsetFrom32BitAlignedAddr )  ||
                    (3u    == offsetFrom32BitAlignedAddr )))
            {
                ptrActualLocationToSet  = (volatile Uint32 *)(
                                          (Uint8 *)ptrPciBaseAddrRegs +
                                          (Uint32)hookRegField);
                /* Mask off the bits to be set */
                tempVal =  ((*ptrActualLocationToSet) &
                           (~((0xffu)<<((8u)*offsetFrom32BitAlignedAddr))));
                /* Set the requried bits */
                tempVal |=  (newFieldVal &
                            ((0xffu)<<((8u)*offsetFrom32BitAlignedAddr)));
                (*ptrActualLocationToSet)    =    tempVal;
                status = PAL_SOK;
            }
            else
            {
                status = PAL_SYS_PCI_INVALID_PARAM;
            }

        }
        else
        {
            status = PAL_SYS_PCI_INVALID_PARAM;
        }
        if(NULL != pciHandle->pciSpaceAccessLock)
        {
            /* Release the lock */
            if(PAL_SOK != PAL_osSemGive(pciHandle->pciSpaceAccessLock))
            {
                return PAL_SYS_PCI_SEM_ERR;
            }
        }
        else
        {
            return PAL_SYS_PCI_OBJ_NOT_OPENED;
        }
    }
    return status;
}

/**
 * PAL_sysPCIGetHookReg
 * \brief    Get hook register. Check for the handle. If handle is proper
 *            get the specified 32-bit register in specified variable.
 *
 * \param    hPci           [IN]    Handle of PCI controller driver
 * \param    hookRegField   [IN]    Field of hook register that has
 *                                  to be read
 * \param    readSize       [IN]    Size of data to be read (byte/word/dword)
 * \param    *newFieldVal   [OUT]   Pointer to new value of field that is
 *                                  to be read from hardware
 *
 * \return    PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCIGetHookReg(    PSP_Handle hPci,
                                    PAL_sysPciHookRegister hookRegField,
                                    PAL_sysPciConfigFieldSize    readSize,
                                    Uint32* newFieldVal)
{
    pciInstanceHandle    pciHandle    =    (pciInstanceHandle)hPci;
    PAL_Result status = PAL_SYS_PCI_INVALID_PARAM;
    Uint32    offsetFrom32BitAlignedAddr = 0u;
    volatile Uint32* ptrActualLocationToGet = NULL;
    Uint32 tempVal = 0u;
    volatile Uint32*    ptrPciBaseAddrRegs    =    NULL;

    if(    (NULL != pciHandle)    &&
        (NULL    !=    newFieldVal))
    {
        if(PCI_OPENED != pciHandle->state)
        {
            return PAL_SYS_PCI_OBJ_NOT_OPENED;
        }
        if(NULL != pciHandle->pciSpaceAccessLock)
        {
            /* Take the lock to avoid multiple access */
            if(    PAL_SOK !=
                PAL_osSemTake(pciHandle->pciSpaceAccessLock,(Int32)SYS_FOREVER))
            {
                return PAL_SYS_PCI_SEM_ERR;
            }
        }
        else
        {
            return PAL_SYS_PCI_OBJ_NOT_OPENED;
        }
        ptrPciBaseAddrRegs    =    pciHandle->pciBaseAddr;

        /* If the offset is 32-bit aligned read the appropriate value */
        if( (PAL_SYS_PCI_READ_WRITE_SIZE_WORD == readSize)     &&
            ( (0u   ==    ((Uint32)hookRegField       & 0x3u))    &&
              (0u    ==   ((Uint32)ptrPciBaseAddrRegs & 0x3u)) ) )
        {
            ptrActualLocationToGet    = (volatile Uint32 *)(
                                        ((Uint8 *)ptrPciBaseAddrRegs) +
                                        ((Uint32)hookRegField));
            (*newFieldVal) = (*ptrActualLocationToGet);
            status = PAL_SOK;
        }
        else if((PAL_SYS_PCI_READ_WRITE_SIZE_HALFWORD == readSize)    &&
                ((0u)    ==    (((Uint32)ptrPciBaseAddrRegs) & (0x1u))))
        {
            /*
             * Get the offset of the location to be set from the nearest
             * 32-bit aligned address
             */
            offsetFrom32BitAlignedAddr = (Uint32)hookRegField & (0x3u);

            if( (0u    == offsetFrom32BitAlignedAddr )||
                (2u    == offsetFrom32BitAlignedAddr ))
            {
                ptrActualLocationToGet  =   (volatile Uint32 *)(
                                            (((Uint8 *)ptrPciBaseAddrRegs) +
                                            ((Uint32)hookRegField))    -
                                            offsetFrom32BitAlignedAddr);

                tempVal =   (*ptrActualLocationToGet);
                tempVal =   tempVal &
                            ((0xffffu) << ((8u)*offsetFrom32BitAlignedAddr));
                (*newFieldVal)  =
                            tempVal >> ((8u)*offsetFrom32BitAlignedAddr);

                status = PAL_SOK;
            }
            else
            {
                status = PAL_SYS_PCI_INVALID_PARAM;
            }
        }
        else if(PAL_SYS_PCI_READ_WRITE_SIZE_BYTE == readSize)
        {
            offsetFrom32BitAlignedAddr = (Uint32)hookRegField & (0x3u);
            ptrActualLocationToGet  =   (volatile Uint32 *)(
                                        (((Uint8 *)ptrPciBaseAddrRegs) +
                                        ((Uint32)hookRegField)) -
                                        offsetFrom32BitAlignedAddr);

            tempVal         =    (*ptrActualLocationToGet);
            tempVal         =    tempVal &
                                ((0xffu) << ((8u)*offsetFrom32BitAlignedAddr));
            *newFieldVal    =    tempVal >> ((8u)*offsetFrom32BitAlignedAddr);

            status = PAL_SOK;
        }
        else
        {
            status = PAL_SYS_PCI_INVALID_PARAM;
        }
        if(NULL != pciHandle->pciSpaceAccessLock)
        {
            /* Release the lock */
            if(PAL_SOK != PAL_osSemGive(pciHandle->pciSpaceAccessLock))
            {
                return PAL_SYS_PCI_SEM_ERR;
            }
        }
        else
        {
            return PAL_SYS_PCI_OBJ_NOT_OPENED;
        }

    }
    return status;
}


/**
 * PAL_sysPCIEnableBasePrefetch
 * \brief   API to Enable BasePrefetch.API to be used while device in slave
 *          mode.
 *
 * \param    hPci                [IN]        Handle of PCI controller driver
 * \param    baseId              [IN]        ID of BASE to be enabled
 *
 * \return    PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCIEnableBasePrefetch (    PSP_Handle hPci,
                                    Uint32 baseId)
{
    pciInstanceHandle    pciHandle    =    (pciInstanceHandle)hPci;
    PAL_Result status = PAL_SYS_PCI_INVALID_PARAM;
    volatile Uint32* ptrSlaveControlReg = NULL;
    volatile Uint32* ptrBackEndConfigReg = NULL;

    if(    (NULL != pciHandle)    &&
        (PAL_SYSPCI_MAX_BASE_ADDR > baseId))
    {
        if(PCI_OPENED != pciHandle->state)
        {
            return PAL_SYS_PCI_OBJ_NOT_OPENED;
        }
        if(NULL != pciHandle->pciSpaceAccessLock)
        {
            /* Take the lock to avoid multiple access */
            if(    PAL_SOK !=
                PAL_osSemTake(pciHandle->pciSpaceAccessLock,(Int32)SYS_FOREVER))
            {
                return PAL_SYS_PCI_SEM_ERR;
            }
        }
        else
        {
            return PAL_SYS_PCI_OBJ_NOT_OPENED;
        }

        ptrBackEndConfigReg    =    pciHandle->pciBaseAddr;
        if(NULL != ptrBackEndConfigReg)
        {
            /* Get the address of slave base address mask register */
            ptrSlaveControlReg    = (volatile Uint32 *)(
                                    (Uint8 *)ptrBackEndConfigReg  +
                                    PAL_SYSPCI_MASK_BAR_0 +
                                    (baseId * (4u)));
            /* Set the bit 3 to make BAR prefetchable */
            *ptrSlaveControlReg |=  PCI_BASE_ENABLE_MASK;
            status = PAL_SOK;
        }
        if(NULL != pciHandle->pciSpaceAccessLock)
        {
            /* Release the lock*/
            if(PAL_SOK != PAL_osSemGive(pciHandle->pciSpaceAccessLock))
            {
                return PAL_SYS_PCI_SEM_ERR;
            }
        }
        else
        {
            return PAL_SYS_PCI_OBJ_NOT_OPENED;
        }


    }
    return status;
}
/**
 * PAL_sysPCIDisableBasePrefetch
 * \brief   API to Disable BasePrefetch. API to be used while device in slave
 *          mode.
 *
 * \param   hPci                [IN]        Handle of PCI controller driver
 * \param   baseId              [IN]        ID of BASE to be enabled
 *
 * \return    PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCIDisableBasePrefetch (    PSP_Handle hPci,
                                            Uint32 baseId)
{
    pciInstanceHandle    pciHandle    =    (pciInstanceHandle)hPci;
    PAL_Result status = PAL_SYS_PCI_INVALID_PARAM;
    volatile Uint32* ptrSlaveControlReg = NULL;
    volatile Uint32* ptrBackEndConfigReg = NULL;

    if(    (NULL != pciHandle)    &&
        (PAL_SYSPCI_MAX_BASE_ADDR > baseId))
    {
        if(PCI_OPENED != pciHandle->state)
        {
            return PAL_SYS_PCI_OBJ_NOT_OPENED;
        }
        if(NULL != pciHandle->pciSpaceAccessLock)
        {
            /* Take the lock to avoid multiple access of memory mapped config
             * space
             */
            if( PAL_SOK !=
                PAL_osSemTake(pciHandle->pciSpaceAccessLock,(Int32)SYS_FOREVER))
            {
                return PAL_SYS_PCI_SEM_ERR;
            }
        }
        else
        {
            return PAL_SYS_PCI_OBJ_NOT_OPENED;
        }

        ptrBackEndConfigReg    =    pciHandle->pciBaseAddr;
        if(NULL != ptrBackEndConfigReg)
        {
            /* Get the address of slave base address mask register */
            ptrSlaveControlReg    = (volatile Uint32 *)(
                                    (Uint8 *)ptrBackEndConfigReg +
                                    PAL_SYSPCI_MASK_BAR_0+baseId*(4u));
            /* Reset the bit 3 to make BAR non-prefetchable */
            *ptrSlaveControlReg &=  (~PCI_BASE_ENABLE_MASK);
            status = PAL_SOK;
        }
        if(NULL != pciHandle->pciSpaceAccessLock)
        {
            /* Release the lock */
            if(PAL_SOK != PAL_osSemGive(pciHandle->pciSpaceAccessLock))
            {
                return PAL_SYS_PCI_SEM_ERR;
            }
        }
        else
        {
            return PAL_SYS_PCI_OBJ_NOT_OPENED;
        }

    }
    return status;
}
/**
 * PAL_sysPCIProgramCacheLineSize
 * \brief   API to set Cache line size. API to be used while device in slave
 *          mode.
 *
 * \param   hPci                [IN]        Handle of PCI controller driver
 * \param   valCacheLineSize    [IN]        Size of cache line to be set
 *
 * \return    PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCIProgramCacheLineSize (
                                PSP_Handle hPci,
                                PAL_sysPciCacheLineSize valCacheLineSize)
{
    PAL_Result status = PAL_SYS_PCI_INVALID_PARAM;
    if(NULL != hPci)
    {
        status = PAL_sysPCISetMemMapReg(
                                        hPci,
                                        PAL_SYSPCI_MIRROR_CACHE_SIZE,
                                        PAL_SYS_PCI_READ_WRITE_SIZE_BYTE,
                                        valCacheLineSize
                                        );
    }
    return status;
}

/**
 * PAL_sysPCIProgramLatencyTimer
 * \brief   API to set Cache line size. API to be used while device in slave
 *          mode.
 *
 * \param   hPci                [IN]        Handle of PCI controller driver
 * \param   valLatencyTimer     [IN]        Value of latency timer
 *
 * \return    PAL_SOK or PAL Error Code
 */
PAL_Result PAL_sysPCIProgramLatencyTimer (    PSP_Handle hPci,
                                             Uint32 valLatencyTimer)
{
    PAL_Result status = PAL_SYS_PCI_INVALID_PARAM;
    if(NULL != hPci)
    {
        status = PAL_sysPCISetMemMapReg(
                                        hPci,
                                        PAL_SYSPCI_MIRROR_LATENCY_TIMER,
                                        PAL_SYS_PCI_READ_WRITE_SIZE_BYTE,
                                        valLatencyTimer
                                        );
    }
    return status;
}
/*
 * PCI ISR. This ISR will be called on occurance of PCI interrupt. If
 * application callback is registered. This ISR will call the callback function.
 */
static void pciIsr(Uint32 instId)
{
    Uint32  pciHandleNum = instId;
    Uint32 interruptStatus = (0u);
    Uint32 cbInterruptMask = (0u);
    volatile Uint32* pciBaseAddress = pciInstObj[pciHandleNum].pciBaseAddr;
    volatile Uint32* ptrToPciReg;

#ifdef PCI_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT(  &DVTEvent_Log,
                    "PCI",
                     PSP_DVT_DESC(PSP_DVT_eINT_START,
                                  PSP_DVT_dNONE,
                                  PSP_DVT_dNONE,
                                  PSP_DVT_dNONE));
#endif
    if(NULL != pciInstObj[pciHandleNum].cfgParams.appCb)
    {
        /* Read interrupt status register */
        ptrToPciReg =  (volatile Uint32* )(( Uint8* )pciBaseAddress + PAL_SYSPCI_STATUS_SET );
        interruptStatus = *ptrToPciReg;
        /* Clearing the interrupt status in the PCI register*/
        ptrToPciReg =  (volatile Uint32* )(( Uint8* )pciBaseAddress + PAL_SYSPCI_STATUS_CLEAR );
        *ptrToPciReg = *ptrToPciReg;

        if( PCI_DSP_INT_SET_CLEAR_MASK_TGT_ABORT    ==
            (   interruptStatus    &
                PCI_DSP_INT_SET_CLEAR_MASK_TGT_ABORT))
        {

            cbInterruptMask |=
                            PAL_SYS_PCI_DSP_INT_TGT_ABORT;
        }
        if( PCI_DSP_INT_SET_CLEAR_MASK_MS_ABORT    ==
            (   interruptStatus    &
                PCI_DSP_INT_SET_CLEAR_MASK_MS_ABORT))
        {

            cbInterruptMask |=    PAL_SYS_PCI_DSP_INT_MS_ABORT;
        }
        if( PCI_DSP_INT_SET_CLEAR_MASK_SYS_ERR_DETECT    ==
            (   interruptStatus    &
                PCI_DSP_INT_SET_CLEAR_MASK_SYS_ERR_DETECT))
        {

            cbInterruptMask    |=    PAL_SYS_PCI_DSP_INT_SYS_ERR_DETECT;
        }
        if( PCI_DSP_INT_SET_CLEAR_MASK_PARITY_ERR_DETECT    ==
            (   interruptStatus    &
                PCI_DSP_INT_SET_CLEAR_MASK_PARITY_ERR_DETECT))
        {

            cbInterruptMask    |=
                    PAL_SYS_PCI_DSP_INT_PARITY_ERR_DETECT;
        }

        if( PCI_DSP_INT_SET_CLEAR_MASK_SOFTINT0    ==
            (   interruptStatus    &
                PCI_DSP_INT_SET_CLEAR_MASK_SOFTINT0))
        {

            cbInterruptMask    |=
                    PAL_SYS_PCI_DSP_INT_SOFTINT0;
        }
        if( PCI_DSP_INT_SET_CLEAR_MASK_SOFTINT1    ==
            (   interruptStatus    &
                PCI_DSP_INT_SET_CLEAR_MASK_SOFTINT1))
        {

            cbInterruptMask    |=
                    PAL_SYS_PCI_DSP_INT_SOFTINT1;
        }
        if( PCI_DSP_INT_SET_CLEAR_MASK_SOFTINT2    ==
            (   interruptStatus    &
                PCI_DSP_INT_SET_CLEAR_MASK_SOFTINT2))
        {

            cbInterruptMask    |=
                    PAL_SYS_PCI_DSP_INT_SOFTINT2;
        }
        if( PCI_DSP_INT_SET_CLEAR_MASK_SOFTINT3    ==
            (   interruptStatus    &
                PCI_DSP_INT_SET_CLEAR_MASK_SOFTINT3))
        {

            cbInterruptMask    |=
                    PAL_SYS_PCI_DSP_INT_SOFTINT3;
        }


        pciInstObj[pciHandleNum].cfgParams.appCb(cbInterruptMask,
                pciInstObj[pciHandleNum].cfgParams.appData);
    }
#ifdef PCI_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT(  &DVTEvent_Log,
                    "PCI",
                     PSP_DVT_DESC(PSP_DVT_eINT_END,
                                  PSP_DVT_dNONE,
                                  PSP_DVT_dNONE,
                                  PSP_DVT_dNONE));
#endif

}


