/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/
/** \mainpage    PALSYS VLYNQ component on DM648
 *
 *  \section    Introduction
 *
 *  \subsection    xxx    Purpose and Scope
 *  The purpose of this document is to explain the VLYNQ PALSYS component
 *  on the PSP Framework architecture written in DSP/BIOS for DM648 platform.
 *
 *  \subsection    aaa    PSP Architecture
 *  A PSP framework is a software entity providing platform support
 *  for devices that are based on Texas Instruments SoC's. The specific
 *  (Hardware and software) implementation of the PSP framework (the PAL
 *  implementation and the group of device drivers) is referred to as
 *  PSP System. The PSP framework defines a software environment for device
 *  drivers to plug-in by defining standard usage interfaces and
 *  abstractions (OS, CPU/Peripherals). For every platform, the PSP
 *  implements platform specific functionality such as OS adaptation, CPU and
 *  peripherals specific mechanisms, and generic system services such as
 *  debug control,traces, persistent storage, etc.
 *
 *  \subsection    bbb    VLYNQ Controller
 *       The VLYNQ communications interface port is a serial interface with a
 *   low pin count, high-speed point-to-point serial interface in the
 *   TMS320DM647 Digital Media System-on-Chip (DMSoC) for connecting to host
 *   processors and other VLYNQ compatible devices. The VLYNQ port is a
 *   full-duplex serial bus where transmit and receive operations occur
 *   separately and simultaneously without interference.<br><br>
 *       VLYNQ enables the extension of an internal bus segment to one or
 *   more external physical devices. The external devices are mapped to local
 *   physical address space and appear as if they are on the internal bus
 *   of the DM647 DMSoC. The external devices must also have a VLYNQ
 *   interface.<br><br>
 *       VLYNQ uses a simple block code (8b/10b) packet format and supports
 *   in-band flow control so that no extra terminals are needed to indicate
 *   that overflow conditions might occur. The VLYNQ module on the DM647
 *   DMSoC serializes a write transaction to the remote/external device and
 *   transfers the write via the VLYNQ port (TX pins). The remote VLYNQ module
 *   deserializes the transaction on the other side.<br><br>
 *       The read transactions to the remote/external device follow the same
 *   process, but the remote device's VLYNQ module serializes the read return
 *   data and transfers it to the VLYNQ port (RX pins). The read return data
 *   is finally deserialized and released to the device internal bus.<br><br>
 *   The external device can also initiate read and write transactions.
 *
 *  \subsection    ccc    DSP/BIOS System
 *  DSP/BIOS is a real time kernel provided by TI to use with its
 *  DSP processors.
 *
 *
 *  \subsection    References
 *    -# VLYNQ functional specification, <b>spruel9_vlynq.pdf</b>
 *    -# Design Document for VLYNQ driver,
 *                         <b>BIOS_VLYNQ_Driver_Arch.doc.doc</b>
 *    -# PSP Framework Overview, <b>PSPFramework_Overview.pdf</b>
 *    -# DSP/BIOS driver development <b>SPRU616.pdf</b>
 *
 */

/**
 *  \file    pal_sysvlynq.h
 *
 *  \brief   This file contains the header for PAL SYS driver API's for the VLYNQ.
 *
 *    (C)    Copyright 2006,    Texas Instruments, Inc
 *
 *  \author     PSG
 *
 *  \version    0.1     Suraj      Created
 *  \version    0.2     PSP Team - Modified for DM6437
 *
 *  <b>Compiler Switch Used</b>
 *
 *  <b>_DEBUG</b><br>
 *   This is used as a flag to compiler whether to include the debug statements
 *   inserted in the code into the final image. This flag helps to build DEBUG
 *   image of the program. For RELEASE images this is not passed to the compiler
 *
 *  <b>CHIP_XXXX</b><br>
 *   The CSL layer is written in a common file for all the variants of a SOC.
 *   This flag differentiates the variant we are compiling for, for eg 
 *   CHIP_DM648, and the CSL definitions for that variant appropriately gets
 *   defined for regs base addresses, num of ports of a peripheral etc
 *
 *  <b>VLYNQ_INSTRUMENTATION_ENABLED</b><br>
 *   This flag is passed to the compiler to include the instrumentation code
 *   parts into the final image/lib of the program. This helps build the
 *   iRelease/iDebug versions of the image/lib with a common code base
 *
 *  <b>VLYNQ_LOOPBACK_MODE</b><br>
 *   This flag is passed to the compiler when vlynq loopback mode is used.
 *
 *
 */

#ifndef _PAL_SYSVLYNQ_H
#define _PAL_SYSVLYNQ_H

#include <string.h>
#include <stdio.h>

#include <ti/sdo/pspdrivers/pal_sys/vlynq/src/pal_sysvlynqOs.h>
#include <ti/sdo/pspdrivers/common/psp_common.h>

#ifdef __cplusplus
extern "C" {
#endif

/** Defined local error code */
#define PAL_VLYNQ_EVENT_LOCAL_ERROR 0
/** Defined peer error code */
#define PAL_VLYNQ_EVENT_PEER_ERROR  1u

/** Device ioctl command to add IRQ */
#define PAL_VLYNQ_DEV_ADD_IRQ       0
/** Device ioctl command to remove IRQ */
#define PAL_VLYNQ_DEV_REMOVE_IRQ    1u

/*VLYNQ Revision numbers*/
#define VLYNQ_REV_10100    0x00010100u
#define VLYNQ_REV_101ff    0x000101ffu
#define VLYNQ_REV_10200    0x00010200u
#define VLYNQ_REV_10204    0x00010204u
#define VLYNQ_REV_10205    0x00010205u
#define LATEST_VLYNQ_REV   0x00010206u

/* Defines for Endianess */
#define LITTLE_ENDIAN      0x2f2f2f2f
#define BIG_ENDIAN         0xf2f2f2f2
/*
 * The parameters:
 *
 * #define MAX_ROOT_VLYNQ     The number of vlynq ports on the SoC.
 * #define MAX_VLYNQ_COUNT    The maximum number of vlynq interfaces with in the system.
 *
 * are system specific and should be defined outside this module.
 *
 */
/**<   Maximum vlynq memory region              */
#define MAX_VLYNQ_REGION             4u

/**<  Maximum device per vlynq interface         */
#define MAX_DEV_PER_VLYNQ            MAX_VLYNQ_REGION

/**<  Maximum device count per vlynq interface     */
/* in DM6437 SOC one VLYNQ port generates clock and operates as master
        and another VLYNQ port sinks that clock and operates as slave
        So 2 devices generated. one as Local and other as Peer
*/
#define MAX_DEV_COUNT                2u
/**< maximum device ISR request                  */
#define TYPICAL_NUM_ISR_PER_IRQ      1u
/**< Max IRQ in chaning                          */
#define MAX_IRQ_PER_CHAIN            32u
/**< Max IRQ per vlynq module                   */
#define MAX_IRQ_PER_VLYNQ            8u


/**<     Operation Success code                 */
#define PAL_VLYNQ_OK                    0
/**<   Invalid vlynq param error code           */
#define PAL_VLYNQ_INVALID_PARAM        -1
/**<   Internal vlynq error code                */
#define PAL_VLYNQ_INTERNAL_ERROR       -2
/**<  No link between Local and Peer error code */
#define PAL_VLYNQ_NO_LINK              -3

#ifndef MAX_ROOT_VLYNQ
#error "Please define MAX_ROOT_VLYNQ - The number of vlynq ports on the SoC."
#endif

/**< The maximum number of vlynq interfaces with in the system */
#ifndef MAX_VLYNQ_COUNT
#error  "Please define MAX_VLYNQ_COUNT - The maximum number of vlynq interfaces with in the system."
#endif

/**< Forward declaration of data structure pal_vlynq_irq_map_t.*/
typedef struct pal_vlynq_irq_map_t  PAL_VLYNQ_IRQ_MAP_T;
/**< Forward declaration of data structure pal_vlynq_irq_info_t.*/
typedef struct pal_vlynq_irq_info_t PAL_VLYNQ_IRQ_INFO_T;
/**< Forward declaration of data structure pal_vlynq_isr_info_t.*/
typedef struct pal_vlynq_isr_info_t PAL_VLYNQ_ISR_INFO_T;


/**<-----------------------------------------------------------------------------
 * The PAL VLYNQ Information. Priavate structures and API(s).
 *----------------------------------------------------------------------------*/
/**
 * \brief Holds the vlynq device region information.
 */
typedef struct pal_vlynq_region_info_t
{
    Int8               owner_dev_index;
    /**< Element is for owner device index */
    Bool               owner_dev_locale;
    /**< Element is for whether device is local/peer */

} PAL_VLYNQ_REGION_INFO_T;

/**
 * \brief  Holds the information of vlynq object through the driver life.
 */
typedef struct pal_vlynq_t
{
    Uint32                  base;
    /**<  Base information for vlynq */
    Bool                    soc;
    /**<  vlynq is on soc or off chip */
    Uint32                  vlynq_version;
    /**<  Vlynq version information   */
    Uint32                  timeout_ms;
    /**<  Vlynq timeout value  */
    /**<  functionality not implemented  */
    struct pal_vlynq_t      *next;
    /**< next vlynq module in chain */
    struct pal_vlynq_t      *prev;
    /**< previos vlynq module in chain */
    PAL_VLYNQ_DEV_HND       *local_dev[MAX_DEV_PER_VLYNQ];
    /**< Handle for Local device */
    PAL_VLYNQ_DEV_HND       *peer_dev[MAX_DEV_PER_VLYNQ];
    /**< Handle for Peer device */
    PAL_VLYNQ_REGION_INFO_T local_region_info[MAX_VLYNQ_REGION];
    /**< Array for local region information */
    PAL_VLYNQ_REGION_INFO_T remote_region_info[MAX_VLYNQ_REGION];
    /**< Array for Remote region information */
    PAL_VLYNQ_IRQ_MAP_T     *root_irq_map;
    /**< Handle for interrupt request Mapping */
    PAL_VLYNQ_IRQ_INFO_T    *root_irq_info;
    /**< Handle for interrupt request information */
    PAL_VLYNQ_ISR_INFO_T    *local_isr_info;
    /**< Handle for local ISR information */
    PAL_VLYNQ_ISR_INFO_T    *peer_isr_info;
    /**< Handle for Remote ISR information */
    Uint32                  backup_local_cntl_word;
    /**< Variable for local control word backup */
    Uint32                  backup_local_intr_ptr;
    /**< Variable for local interrupt pointer backup */
    Uint32                  backup_local_tx_map;
    /**< Variable for local TX map backup */
    Uint32                  backup_local_endian;
    /**< Variable for local endian info */
    Uint32                  backup_peer_cntl_word;
    /**< Variable for peer control word backup */
    Uint32                  backup_peer_intr_ptr;
    /**< Variable for peer interrupt pointer backup */
    Uint32                  backup_peer_tx_map;
    /**< Variable for peer interrupt pointer backup */
    Uint32                  backup_peer_endian;
    /**< Variable for peer endian info */
    Int8                    local_irq;
    /**< Local irq number */
    Int8                    peer_irq;
    /**< Peer irq number  */
    Bool                    local_swap;
    /**< Swap info for local device */
    Bool                    peer_swap;
    /**< Swap info for remote device */

} PAL_VLYNQ_T;

/**
 * \brief Register overlay for RX.
 */
typedef struct vlynq_rx_reg_t
{
    volatile Uint32 size;
   /**< Size of the rx region*/
    volatile Uint32 offset;
    /**< Offset of the rx region*/

} VLYNQ_RX_REG_T;

/**
 * \brief  Overlay of Vlynq 2.6 register set
 */
typedef struct vlynq_reg_t
{
    volatile Uint32         revision;
    /**<  Hold the revision of the soc, offset : 0x00 */
    volatile Uint32         control;
    /**< Control register of the VLYNQ      0x04 */
    volatile Uint32         status;
    /**< Status register for vlynq device 0x08 */
    volatile Uint32         intPriority;
    /**< Interrupt priority register 0x0c    */
    volatile Uint32         intStatus;
    /**< Interrupt status register.0x10          */
    volatile Uint32         intPending;
    /**<   Interrupt pending register.0x14     */
    volatile Uint32         intPtr;
    /**<   Interrupt pointer register.0x18     */
    volatile Uint32         txMap;
    /**<  Tx address for the remote        0x1C */
    VLYNQ_RX_REG_T          rx_set[4];
    /**< rx_set is of type VLYNQ_RX_REG_T      */
    volatile Uint32         chipVersion;
    /**< Chip version for vlynq         0x40 */
    volatile Uint32         autoNegn;
    /**< auto negotation register    0x44     */
    volatile Uint32         manualNegn;
    /**<  Manual negotation register    0x48     */
    volatile Uint32         negnStatus;
#ifdef INCLUDE_ENDIAN_REGISTER
    /**<  Negotation status register     */
    volatile Uint32         pad2[3];
    /**<     0x4C                          */
    volatile Uint32         endian;
#else
    /**<  The Endian Register is removed for DM6437 LC
    The functionality is still there to support endianess for
    other SoC. Define INCLUDE_ENDIAN_REGISTER to support it */
    /**<  Negotation status register     */
    volatile Uint32         pad2[4];
#endif
    /**<  Endiness of the device 0x5C      */
    volatile Uint32         ivr30;
    /**<  Interrupt vector register 0 - 3 0x60     */
    volatile Uint32         ivr74;
       /**< Interrupt vector register 4 - 7     */
    volatile Uint32         pad3[7];
    /**<      0x64                              */
} VLYNQ_REG_T;


/**
  * \brief To decide the clock direction
  */
typedef enum pal_vlynq_clock_dir_enum
{
    pal_vlynq_clk_in = 0,
    /**< Sink the clock.         */
    pal_vlynq_clk_out
    /**< Source the clock         */
} PAL_VLYNQ_CLOCK_DIR_ENUM_T;


/**
  * \brief To decide the endianess type
  */
typedef enum pal_vlynq_endian_enum
{
    pal_vlynq_ignore_en = 0,
    /**< Ignore the endianess    */
    pal_vlynq_little_en = (Int32)LITTLE_ENDIAN,
       /**< For little endianess    */
    pal_vlynq_big_en    = (Int32)BIG_ENDIAN
     /**< For big endianess        */
} PAL_VLYNQ_ENDIAN_ENUM_T;


/**
  * \brief  This is for RTM configuration
  */
typedef enum pal_vlynq_rtm_cfg_enum
{
    no_rtm_cfg = 0,
    /**< No RTM configuration.*/
    rtm_auto_select_sample_val,
    /**< Auto select sample value. */
    rtm_force_sample_val
    /**< Force RTM sample value. */
} PAL_VLYNQ_RTM_CFG_ENUM_T;


/**
  * \brief This is for raising the error and mapped caused
  */
typedef enum
{
    pal_vlynq_init_success             = 0,
    /**< No error */
    pal_vlynq_init_no_link_on_reset,
   /**< Error if there is no link on reset */
    pal_vlynq_init_no_mem,
     /**< Error if enough memory is not allocated */
    pal_vlynq_init_clk_cfg,
    /**< Error in configuration of clock */
    pal_vlynq_init_no_link_on_clk_cfg,
    /**< Error if no link on clk configuration */
    pal_vlynq_init_internal_problem,
    /**< Error if its internal problem */
    pal_vlynq_init_invalid_param,
    /**< Error if there are invalid param. */
    pal_vlynq_init_local_high_rev,
    /**< Error if local has high revision. */
    pal_vlynq_init_peer_high_rev,
    /**< Error if peer has high revision.*/
    pal_vlynq_init_rtm_cfg,
    /**< Error if RTM configuration */
    pal_vlynq_init_no_link_on_rtm_cfg
    /**< Error if no link on RTM configuration */
} PAL_VLYNQ_INIT_ERR_ENUM_T;

/**
 * \brief    Structure is for vlynq config elements.
 */
typedef struct
{

    Uint8                  on_soc;
    /**< 1 => the VLYNQ module is on the SOC running the code, 0 => otherwise.    */
    PAL_VLYNQ_INIT_ERR_ENUM_T error_status;
    /**<   status of error     */
    char                   error_msg[50];
    /**<  holds error message string      */
    Uint32                 base_addr;
    /**< Virtual Base Address of the module     */
    Uint32                 init_timeout_in_ms;
    /**< The number of millsecs that the software should allow for initialization
      *  to complete     */
    /**<  functionality not implemented  */
    Uint8                  local_clock_div;
    /**< The clock divisor for the local VLYNQ module      */
    PAL_VLYNQ_CLOCK_DIR_ENUM_T local_clock_dir;
     /**< The clock direction; sink or source for the local VLYNQ module     */
    Uint8                  local_intr_local;
    /**< 1 => interrupts are being handled locally or 0 => sent interrupt packate over the interface     */
    Uint8                  local_intr_vector;
    /**<    The IRQ vector# to be used on the local VLYNQ module. Valid values are
      *    0 to 31. Should be unique      */
    Uint8                  local_intr_enable;
    /**<     1 => enable the local irq vector or => disable      */
    Uint8                  local_int2cfg;
    /**<   Valid only if intr_local is set.
      *            1 => write to the local VLYNQ interrupt pending register;
      *            0 => write to the location refered by local address pointer       */
    Uint32                 local_intr_pointer;
#ifdef INCLUDE_ENDIAN_REGISTER
    /**<     Address to which the irq should be written to; valid only if int2cfg is not set */
    PAL_VLYNQ_ENDIAN_ENUM_T  local_endianness;
#endif
    /**<    Endianess of the local VLYNQ module      */
    Uint32                 local_tx_addr;
    /**< The physical portal address of the local VLYNQ      */
    PAL_VLYNQ_RTM_CFG_ENUM_T local_rtm_cfg_type;
     /**< The RTM configuration for the local VLYNQ      */
    Uint8                  local_rtm_sample_value;
    /**< The RTM sample value for the local VLYNQ, valid only if forced rtm cfg
      *  type is selected     */
    Bool                   local_tx_fast_path;
    /**<    TX Fast path 0 => slow path, 1 => fast path.     */
    Uint8                  peer_clock_div;
    /**<    The clock divisor for the peer VLYNQ module     */
    PAL_VLYNQ_CLOCK_DIR_ENUM_T peer_clock_dir;
    /**<    The clock direction; sink or source for the peer VLYNQ module    */
    Uint8                  peer_intr_local;
    /**< 1 => interrupts are being handled by peer VLYNQ or 0 => sent over the bus      */
    Uint8                  peer_intr_enable;
    /**< 1 => enable the peer irq vector or => disable      */
    Uint8                  peer_intr_vector;
    /**<  The IRQ vector# to be used on the peer VLYNQ module. Valid values are 0 to 31.
      *   Should be unique                                     */
    Uint8                  peer_int2cfg;
    /**<         Valid only if intr_local is set.
     *             1 => write to the local VLYNQ interrupt pending register;
     *           0 => write to the location refered by local address pointer   */
    Uint32                 peer_intr_pointer;
#ifdef INCLUDE_ENDIAN_REGISTER
    /**< Address to which the irq should be written to; valid only if int2cfg is not set     */
    PAL_VLYNQ_ENDIAN_ENUM_T  peer_endianness;
#endif
    /**<    Endianess of the local VLYNQ module      */
    Uint32                 peer_tx_addr;
    /**<  The physical portal address of the peer VLYNQ      */
    PAL_VLYNQ_RTM_CFG_ENUM_T peer_rtm_cfg_type;
    /**<  The RTM configuration for the peer VLYNQ      */
    Uint8                  peer_rtm_sample_value;
    /**< The RTM sample value for the local VLYNQ, valid only if forced rtm cfg type is selected   */
    Bool                   peer_tx_fast_path;
    /**<    TX Fast path 0 => slow path, 1 => fast path     */
    Bool                   init_swap_flag;
    /**<  Is the Vlynq module in Endian swapped state to begin with      */
} PAL_VLYNQ_CONFIG_T;

/**
  * \brief IRQ active level.
  */
typedef enum
{

   pal_vlynq_high_irq_pol = 0,
   /**< IRQ polarity high. */
   pal_vlynq_low_irq_pol
    /**< IRQ polarity low. */
}PAL_VLYNQ_IRQ_POL_ENUM_T;

/**
  * \brief IRQ level or edge triggered.
  */
typedef enum
{

   pal_vlynq_level_irq_type = 0,
   /**< IRQ level triggered. */
   pal_vlynq_edge_irq_type
    /**< IRQ edge triggered.  */
}PAL_VLYNQ_IRQ_TYPE_ENUM_T;

/**
 * To keep the ISR dispatch simple we align the signature of the ISR
 * to that stipulated by the OS.
 */

/**
 * \fn      PAL_Result PAL_sysVlynqInit(void);
 * \brief   Initialize the VLYNQ control module.
 *
 * \param   none
 *
 * \return
 */
PAL_Result PAL_sysVlynqInit(void);

/**
 * \fn      PAL_Result PAL_sysVlynqAddDevice(PAL_VLYNQ_HND *vlynq, PAL_VLYNQ_DEV_HND *vlynq_dev,
                       Bool peer)
 *
 * \brief    Add the device reference into VLYNQ
 *
 *
 * \param   vlynq        [IN]    Handle to the opened Resource Manager
 *
 * \param    vlynq_dev   [IN]    The device instance, which has to be added into the VLYNQ.
 *                                This value would have been returned by PAL_sysVlynqDevCreate().
 * \param    peer          [IN]    The flag whether the device is associated with the local
 *                                VLYNQ or the peer.
 *
 * \return                         0 on success , -1 on failure.
 */

PAL_Result PAL_sysVlynqAddDevice(PAL_VLYNQ_HND *vlynq, PAL_VLYNQ_DEV_HND *vlynq_dev,
           Bool peer);

/**
 * \fn      PAL_Result PAL_sysVlynqRemoveDevice( PAL_VLYNQ_HND *vlynq, const PAL_VLYNQ_DEV_HND *vlynq_dev)
 *
 * \brief    Removes the device reference from VLYNQ.
 *
 *
 * \param   vlynq       [IN]    The VLYNQ module instance, from
 *                                 which the device instance has to be removed.
 * \param    vlynq_dev   [IN]    The device instance, which is to be removed from
 *                                the VLYNQ.
 *
 * \return                        0 on success.,-1 on failure.
 */
PAL_Result PAL_sysVlynqRemoveDevice( PAL_VLYNQ_HND *vlynq, const PAL_VLYNQ_DEV_HND *vlynq_dev);

/**
 * \fn      PAL_Result PAL_sysVlynqMapRegion(PAL_VLYNQ_HND *vlynq, Bool remote, Uint32 region_id,
 *                     Uint32 region_offset, Uint32 region_size,
 *                     PAL_VLYNQ_DEV_HND *vlynq_dev);
 *
 * \brief   Map the memory regions of the device
 *
 *
 * \param   vlynq       [IN]    The VLYNQ instance, which on which regions have to be mapped.
 *                              This value would have been returned by PAL_sysVlynqInit( )
 *
 * \param   remote      [IN]    Identifies whether the region to be mapped is on
 *                                remote/peer or local VLYNQ.
 * \param   region_id   [IN]    Identifier of the RX region; valid values are
 *                                from 0 to 3.
 * \param  region_offset [IN]    The offset of the region in the physical memory
 *                                map of the local or remote SOC.
 * \param   region_size [IN]    The size of the region to be mapped.
 * \param   vlynq_dev   [IN]    The device instance for which certain regions have
 *                                to be mapped.This value would have been returned
 *                                by PAL_sysVlynqDevCreate( ).
 * \return                      0 on success.,-1 on failure.
 */
PAL_Result PAL_sysVlynqMapRegion(PAL_VLYNQ_HND *vlynq, Bool remote, Uint32 region_id,
                                Uint32 region_offset, Uint32 region_size,
                                PAL_VLYNQ_DEV_HND *vlynq_dev);

/**
 * \fn      PAL_Result PAL_sysVlynqUnMapRegion(PAL_VLYNQ_HND *vlynq, Bool remote, Uint32 region_id,
 *                     PAL_VLYNQ_DEV_HND *vlynq_dev);
 *
 * \brief    UnMap the memory regions of the device.
 *
 *
 * \param   vlynq       [IN]    The VLYNQ instance, on which previously
 *                                mapped regions have to be un-mapped.
 * \param   remote      [IN]    Identifies whether the region to be
 *                                mapped is on remote/peer or local VLYNQ.
 * \param   region_id   [IN]    Identifier of the RX region; valid values
 *                                are from 0 to 3.
 * \param    vlynq_dev   [IN]    The device instance for which certain
 *                                regions have been previously mapped;
 *                                and now are being un-mapped.
 *
 * \return                        0 on success.,-1 on failure.
 */

PAL_Result PAL_sysVlynqUnMapRegion(PAL_VLYNQ_HND *vlynq, Bool remote, Uint32 region_id,
           PAL_VLYNQ_DEV_HND *vlynq_dev);

/**
 * \fn      PAL_Result PAL_sysVlynqGetDevBase(PAL_VLYNQ_HND     *vlynq,
 *          Uint32              offset,
 *          Uint32              *base_addr,
 *          PAL_VLYNQ_DEV_HND   *vlynq_dev);
 *
 * \brief   Get the physical  base address of the device.
 *
 *
 * \param   vlynq       [IN]    The instance on which the physical base address
 *                                is being sought.
 * \param  offset       [IN]    The offset (in bytes) into the remote device
 *                                memory map.
 * \param  base_addr    [IN]    The placeholder for the evaluated physical base
 *                                address of the remote device in the context of
 *                                the root SOC. The caller manages the memory for this.
 * \param   vlynq_dev   [IN]    The handle to remote device instance; would have been
 *                                returned by PAL_sysVlynqDevFind( ).
 *                                The offset is being sought for this device.
 *
 * \return                      0 on success.,-1 on failure.
 */

PAL_Result PAL_sysVlynqGetDevBase(PAL_VLYNQ_HND     *vlynq,
           Uint32               offset,
           Uint32               *base_addr,
           PAL_VLYNQ_DEV_HND    *vlynq_dev);

/**
 * \fn      Bool PAL_sysVlynqGetLinkStatus( PAL_VLYNQ_HND *vlynq);
 * \brief    Get the link status of the VLYNQ module.
 *
 *
 * \param   vlynq        [IN]    The VLYNQ module instance whose link status with
 *                                the peer is being requested.
 * \return                        1 on link, 0 on link failure.
 */

Bool PAL_sysVlynqGetLinkStatus( PAL_VLYNQ_HND *vlynq);

/**
 * \fn      Int32 PAL_sysVlynqGetNumRoot(void);
 *
 * \brief    Get the number of the root VLYNQ(s).
 *
 *
 * \return                        The number of root VLYNQ modules on the SOC.
 *                                0 means no VLYNQ,1 means an root index of 0,2
 *                                 mean indices0 and 1.
 */
Int32 PAL_sysVlynqGetNumRoot(void);
/**
 * \fn      PAL_VLYNQ_HND* PAL_sysVlynqGetRoot(Int32 index);
 * \brief    Get the handle to the specified root VLYNQ
 *
 *
 * \param   index       [IN]    0 based. Starts at 0 and extends to
 *                                (inclusive of) one less number of
 *                                roots returned by PAL_sysVlynqGetNumRoot ( )
 *
 * \return                        NULL, on failure otherwise a valid handle
 *                                to access the root VLYNQ module.
 */

PAL_VLYNQ_HND* PAL_sysVlynqGetRoot(Int32 index);
/**
 * \brief    Get the next VLYNQ module in the chain
 *
 *
 * \param   this        [IN]    The VLYNQ module instance whose next
 *                                chained entity (away from the root) is
 *                                being sought.
 *
 * \return                        NULL, on failure otherwise a valid handle
 *                                to access the neighbor module.
 */

PAL_VLYNQ_HND* PAL_sysVlynqGetNext( PAL_VLYNQ_HND *this);
/**
 * \brief    Is this VLYNQ module the last one in the chain
 *
 *
 * \param   this        [IN]    The VLYNQ module instance
 *                                to be ascertained whether last in the
 *                                chain (away from the root).
 * \return                        0 on false.,1 on true.
 */

Int32 PAL_sysVlynqIsLast( PAL_VLYNQ_HND *this);

/**
 * \brief    Get the length of the VLYNQ chain
 *
 *
 * \param   this        [IN]    The starting VLYNQ module instance inclusive
 *                                of which, the number of VLYNQ modules existing
 *                                in the chain (away from root) is being sought.
 *                                If this happens to be the root VLYNQ module, then the
 *                                length of the entire chain is ascertained.
 *
 * \return                        The number of VLYNQ modules in the chain
 *                                (away from the root).
 */

Int32 PAL_sysVlynqGetChainLength(PAL_VLYNQ_HND *this);
/**
 * \brief    Returns the base address of the VLYNQ
 *
 *
 * \param   vlynq        [IN]    The Vlynq module whose base address is being sought.
 * \param    base          [IN]    The placeholder for the virtual base address of the
 *                                VLYNQ module on the SOC.
 *
 * \return                        0  on success, -1 on failure.
 */

PAL_Result PAL_sysVlynqGetBaseAddr(PAL_VLYNQ_HND *vlynq, Uint32 *base);
/**
 * \brief    Get the root VLYNQ  at the  base address.
 *
 *
 * \param   base_addr    [IN]    The virtual base address of the VLYNQ module on the SOC.
 *
 * \return                        NULL, on failure otherwise a valid handle to access
 *                                the root VLYNQ module.
 */

PAL_VLYNQ_HND *PAL_sysVlynqGetRootAtBase(Uint32 base_addr);
/**
 * \brief    Get root for the given VLYNQ.
 *
 *
 * \param   vlynq        [IN]    Handle to the VLYNQ whose root is being sought.
 *
 * \return                        NULL, on failure otherwise a valid handle,
 *                                to access the root VLYNQ module.
 */
PAL_VLYNQ_HND *PAL_sysVlynqGetRootVlynq(PAL_VLYNQ_HND *vlynq);

/**
 * \brief    Append to  the VLYNQ chain
 *
 *
 * \param   this        [IN]    The VLYNQ module instance, which has to be added
 *                                to the chain (away from the root).
 *                                This value would have been returned by PAL_sysVlynqInit( ).
 * \param    to            [IN]    The VLYNQ module instance, to which this has to be chained
 *                                (away from the root).This value would have been returned
 *                                by PAL_sysVlynqInit( )..
 *
 * \return                        0  on success, -1 on failure.
 */
PAL_Result PAL_sysVlynqChainAppend( PAL_VLYNQ_HND *this,  PAL_VLYNQ_HND *to);

/**
 * \brief    Remove (the tail) from the VLYNQ chain
 *
 *
 * \param   this        [IN]    The VLYNQ module instance, which has to be removed
 *                                from the chain (away from the root).
 * \param    from        [IN]    The VLYNQ module instance, from which
 *                                this has to be removed (away from the root).
 *
 * \return                        0  on success, -1 on failure.
 */
PAL_Result PAL_sysVlynqChainUnAppend( PAL_VLYNQ_HND *this,  PAL_VLYNQ_HND *from);

/**
 * \brief    Maps the IRQ hardware line onto the VLYNQ
 *
 *
 * \param   vlynq        [IN]    The vlynq instance on which the IRQ mapping is to be carried
 *                                 out.This value would have been returned by PAL_sysVlynqInit( ).
 * \param    hw_intr_line [IN]    Identifies the hardware vector line (in the perspective of
 *                                the VLYNQ module), which runs from the device to the VLYNQ
 *                                module. The valid values are 0 to 7 (both inclusive).
 * \param    irq            [IN]    Identifies the IRQ number to which the interrupt hardware line
 *                                shall be mapped.The valid values are 0 to 31 (both inclusive).
 *                                It is the responsibility of the user to ensure that there is a
 *                                unique irq_hw_line to irq mapping within a VLYNQ chain.
 * \param    vlynq_dev   [IN]    The device instance for which certain hardware interrupts
 *                                have to be mapped on the VLYNQ;this value would have been
 *                                returned by PAL_sysVlynqDevCreate().
 *
 * \return                        0  on success, -1 on failure.
 */
PAL_Result PAL_sysVlynqMapIrq(PAL_VLYNQ_HND *vlynq, Uint32 hw_intr_line,
                   Uint32 irq, PAL_VLYNQ_DEV_HND *vlynq_dev);
/**
 * \brief    UnMaps the IRQ hardware line.
 *
 *
 * \param   vlynq        [IN]    The VLYNQ instance on which the un-mapping has to be carried out.
 * \param    irq         [IN]    Identifies the IRQ number for which the interrupt hardware
 *                                line shall be un-mapped. The valid values are 0 to 31 (both inclusive).
 * \param   vlynq_dev    [IN]    The device instance for which certain hardware interrupts are
 *                                required to be un-mapped on VLYNQ; this value would have been
 *                                returned by PAL_sysVlynqDevCreate().
 * \return                        0 on success.-1 on failure.
 */
PAL_Result PAL_sysVlynqUnMapIrq(PAL_VLYNQ_HND *vlynq, Uint32 irq,
                             PAL_VLYNQ_DEV_HND *vlynq_dev);
/**
 * \brief    Install the ISR for the device.
 *
 *
 * \param   vlynq        [IN]    The VLYNQ module instance for whose associated remote device,
 *                                 an isr is being installed.
 * \param    irq         [IN]    One of the identifier of the IRQ as returned by PAL_sysVlynqGetDevIrq( )
 *                                or VLYNQ module specific interrupt.
 * \param   dev_isr        [IN]    The OS specific function pointer (ISR handler); the type of the function
 *                                PAL_VLYNQ_DEV_ISR_FN shall be defined for each of the OS by the user.
 *                                Driver for the remote device shall load appropriate function instance.
 * \param    isr_param   [IN]    It is a collection of parameters for each instance of the remote device
 *                                driver. The user shall define the type PAL_VLYNQ_DEV_ISR_PARAM_GRP_T for
 *                                the collection of the parameters for each OS.  The only caveat here is
 *                                that the name of the members of the PAL_VLYNQ_DEV_ISR_PARAM_GRP_T should
 *                                start (i.e. first member) with arg0 (inclusive) and should continue in
 *                                sequence of arg1, arg2 up to arg9 (inclusive); it is assumed that no OS
 *                                implementation shall require more than 10 callback ISR parameters.
 *                                User shall define and specify the number of ISR parameter,
 *                                PAL_VLYNQ_DEV_ISR_PARM_NUM as required for a specific OS implementation;
 *                                this should not exceed 10. For a value, less than 10, the member names
 *                                shall start as arg0, arg1 up to arg< PAL_VLYNQ_DEV_ISR_PARM_NUM 1>.
 *
 * \return                        0 on success., -1 on error
 */
PAL_Result PAL_sysVlynqAddIsr(PAL_VLYNQ_HND *vlynq, Uint32 irq,
                        PAL_VLYNQ_DEV_ISR_FN  dev_isr,
                        PAL_VLYNQ_DEV_ISR_PARAM_GRP_T* isr_param);


/**
 * \brief    Uinstall the previously installed ISR
 *
 *
 * \param   vlynq        [IN]    The VLYNQ module instance for whose
 *                                associated remote device, an isr is being removed
 *                              object.
 * \param    irq         [IN]    The identifier of the IRQ as
 *                                provided in the PAL_sysVlynqAddIsr( ).
 * \param   isr_param   [IN]    Same as that provided in the PAL_sysVlynqAddIsr( )
 *
 * \return                        0 on success., -1 on error
 */
PAL_Result PAL_sysVlynqRemoveIsr( PAL_VLYNQ_HND *vlynq, Uint32 irq,
                      const PAL_VLYNQ_DEV_ISR_PARAM_GRP_T *isr_param);
/**
 * \brief    The Root ISR; register it with the system
 *
 *
 * \return                        0 on success., -1 on error
 */
PAL_Result PAL_sysVlynqRootIsr(PAL_VLYNQ_T *p_vlynq);
/**
 * \brief    Get the number of times this IRQ occurred
 *
 *
 * \param   vlynq        [IN]    The VLYNQ module instance for whose associated
 *                                irq has to be queried for number of dispatches so far.
 * \param    irq         [IN]    The interrupt number representing the mapped interrupt
 *                                hardware line on the VLYNQ module. This should be unique
 *                                for a given VLYNQ chain.
 * \param    count       [IN]    Placeholder to store the number of times the interrupt irq
 *                                has been raised.The value in the placeholder is valid only if
 *                                the function returns success.
 *
 * \return                        0 on success., -1 on error
 */
PAL_Result PAL_sysVlynqGetIrqCount( PAL_VLYNQ_HND *vlynq, Uint32 irq, Uint32 *count);
/**
 * \brief    Get the VLYNQ for the IRQ.
 *
 *
 * \param   root        [IN]    The VLYNQ chain identifier (the root VLYNQ);
 *                                the interrupt numbers are unique with in a chain.
 *                              object.
 * \param    irq         [IN]    The interrupt number whose association with a VLYNQ module
 *                                is being sought. The valid values are 0 to 31 (both inclusive).
 *
 * \return                        0 on success., -1 on error
 */
PAL_VLYNQ_HND* PAL_sysVlynqGetForIrq(PAL_VLYNQ_HND *root, Uint32 irq);
/**
 * \brief    Set the polarity of the hardware IRQ line
 *
 *
 * \param   vlynq        [IN]    The VLYNQ instance whose hardware
 *                                interrupt line is to be set for polarity
 * \param    irq         [IN]    The interrupt number representing the mapped interrupt
 *                                hardwareline on the VLYNQ module.
 *                                This should be unique for a given VLYNQ chain.
 * \param    pol            [IN]    The polarity of the line. Refer to the data structure in
 *                                Appendix A.
 *
 * \return                        0 on success., -1 on error
 */
PAL_Result PAL_sysVlynqSetIrqPol( PAL_VLYNQ_HND *vlynq, Uint32 irq,
                      PAL_VLYNQ_IRQ_POL_ENUM_T pol);
/**
 * \brief    Get the polarity of the hardware IRQ line
 *
 *
 * \param   vlynq        [IN]    The VLYNQ instance whose hardware interrupt
 *                                line is to being looked up for polarity setting.
 * \param    irq         [IN]    The interrupt number representing the mapped interrupt
 *                                hardware line on the VLYNQ module. This should be
 *                                unique for a given VLYNQ chain
 * \param    pol            [IN]    The polarity of the line. The caller manages memory
 *
 * \return                        0 on success., -1 on error
 */
PAL_Result PAL_sysVlynqGetIrqPol( PAL_VLYNQ_HND *vlynq, Uint32 irq,
                      PAL_VLYNQ_IRQ_POL_ENUM_T *pol);
/**
 * \brief    Set the type of the hardware IRQ  line
 *
 *
 * \param   vlynq        [IN]    The VLYNQ instance whose hardware
 *                                interrupt line is to be set for trigger type.
 * \param    irq            [IN]    The interrupt number representing the mapped interrupt
 *                                hardware line on the VLYNQ module. This should be
 *                                unique for a given VLYNQ chain.
 * \param    type        [IN]    The type of the interrupt line
 *
 * \return                        0 on success., -1 on error
 */
PAL_Result PAL_sysVlynqSetIrqType(PAL_VLYNQ_HND *vlynq, Uint32 irq,
                       PAL_VLYNQ_IRQ_TYPE_ENUM_T type);
/**
 * \brief    Get the type of the hardware IRQ type
 *
 *
 * \param   vlynq        [IN]    The VLYNQ instance whose hardware interrupt line
 *                                is being looked up for trigger type.
 * \param    irq         [IN]    The interrupt number representing the mapped interrupt
 *                                hardwareline on the VLYNQ module. This should be unique
 *                                for a given VLYNQ chain.
 * \param   type        [IN]    The type of the interrupt line. The caller manages memory
 *
 * \return                        0 on success., -1 on error
 */
PAL_Result PAL_sysVlynqGetIrqType( PAL_VLYNQ_HND *vlynq, Uint32 irq,
                       PAL_VLYNQ_IRQ_TYPE_ENUM_T *type);
/**
 * \brief    Disable the IRQ
 *
 *
 * \param   vlynq        [IN]    The VLYNQ module instance for whose associated
 *                                irq hardware line is to be disabled for generating interrupts.
 * \param    irq         [IN]    The interrupt number representing the mapped interrupt hardware line on the VLYNQ module.
 *                                This should be unique for a given VLYNQ chain. .
 *
 * \return                        0 on success., -1 on error
 */
PAL_Result PAL_sysVlynqDisableIrq( PAL_VLYNQ_HND *vlynq, Uint32 irq);
/**
 * \brief    Enable  the IRQ
 *
 *
 * \param   vlynq        [IN]    The VLYNQ module instance for whose associated
 *                                "irq" hardware line is to be enabled for generating interrupts
 * \param    irq         [IN]    The interrupt number representing the mapped interrupt hardware line
 *                                on the VLYNQ module. This should be unique for a given
 *                                VLYNQ chain.
 *
 * \return                        0 on success., -1 on error
 */
PAL_Result PAL_sysVlynqEnableIrq( PAL_VLYNQ_HND *vlynq, Uint32 irq);
/**
 * \brief    Configures the Clock for the VLYNQ bridge
 *
 *
 * \param   p_vlynq_clocksys [IN] The VLYNQ module instance, whose register has
 *                                to be controlled; this value would have been returned
 *                                by PAL_sysVlynqInit().
 * \param  local_clock_dir [IN]    The clock direction for the local VLYNQ.
 * \param  peer_clock_dir  [IN]    The clock direction for the peer VLYNQ
 * \param  local_clock_div [IN]    The divisor for the local clock. Valid values are 1 to 255.
 * \param  peer_clock_div [IN]    The divisor for the peer clock. Valid values are 1 to 255.
 *
 * \return                        0 on success., -1 on error
 */
PAL_Result PAL_sysVlynqConfigClock(PAL_VLYNQ_HND                *p_vlynq_clocksys,
                          PAL_VLYNQ_CLOCK_DIR_ENUM_T   local_clock_dir,
                        PAL_VLYNQ_CLOCK_DIR_ENUM_T   peer_clock_dir,
                Uint8                        local_clock_div,
                Uint8                        peer_clock_div);
/**
 * \brief    Initialize the VLYNQ control module
 *
 *
 * \param   pal_vlynq_config [IN] The collection of configuration parameters;
 *                                 to initialize     the VLYNQ interface.
 *
 * \return                        On failure, it returns a NULL, otherwise, it
 *                                returns a valid handle to the instance so initialized.
 */
PAL_VLYNQ_HND *PAL_sysVlynqInitSoc(PAL_VLYNQ_CONFIG_T *pal_vlynq_config);
/**
 * \brief    Un-Initialize the VLYNQ control module.
 *
 *
 * \param   vlynq        [IN]    Handle to the VLYNQ module instance; would have been
 *                                returned by PAL_sysVlynqInit( ).
 *                                Consequence of using this handle after a call to this
 *                                function is undefined.
 *
 *
 * \return                        0 on success., -1 on error
 */
PAL_Result PAL_sysVlynqCleanUp(PAL_VLYNQ_HND *vlynq);
/**
 * \brief    Dumps vital VLYNQ information into the buffer.
 *
 *
 * \param   vlynq        [IN]    The vlynq instance for which the dump has to be provided.
 * \param    dump_type   [IN]    Identifies the information being sought.Some of the commands
 *                                can be raw byte dump of the hardware, complete chain dump,
 *                                raw register values or specific register value with enumeration
 *                                such as status register or control register
 * \param    buf        [IN]    The placeholder for the buffer. After the function, the buffer
 *                                can be printed out for reading and information.
 * \param    limit        [IN]    The size of the buffer. It is strongly recommended to provide
 *                                a buffer of at least 4096 bytes.
 * \param    eof        [IN]    Whether the buffer was insufficient.
 *
 * \return                        The number of bytes that have been formatted and placed in
 *                                the buffer,-1 on error.
 */
Int32 PAL_sysVlynqDump(PAL_VLYNQ_HND *vlynq, Uint32 dump_type, char *buf, Int32 limit, Int32 *eof);
/**
 * \brief    Read/Write  register of the VLYNQ module
 *
 *
 * \param   vlynq_hnd    [IN]    The VLYNQ module instance, whose register has to be controlled;
 *                                this value would have been returned by PAL_sysVlynqInit().
 * \param   cmd            [IN]    Various read and write commands to be carried out.
 *                                Refer below for the 32 bit break up of the command.
 * \param    cmd_val     [IN]    Pointer variable for write command(s) this provides the value
 *                                to be written and for read operations it provides the placeholder
 *                                for the value to be read.
 *
 * \return                        0 on success., -1 on error
 */
PAL_Result PAL_sysVlynqIoctl(PAL_VLYNQ_HND *vlynq_hnd, Uint32 cmd, Uint32 *cmd_val);


/**
 * \brief   Return the Mapped Region configuration for Local/Peer
 *
 *
 * \param   p_vlynq_mapped [IN]    Handle to the opened Resource Manager
 * \param  address      [OUT]   Return values of TxMap,RxMap.size[4] &
 *                                RxMap.offset[4].
 * \param  peer         [IN]    The flag whether the device is associated with the
 *                                local VLYNQ or the peer.
 */
void PAL_sysVlynqMappedRegion(const PAL_VLYNQ_T *p_vlynq_mapped, Uint32 *address, Uint32 peer);

#ifdef __cplusplus
}
#endif

#endif /* __PAL_VLYNQ_H__ */

