/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/** \file   pal_sysvlynqDev.c
 *
 *  \brief  This file contains the device API's to be connected with VLYNQ.
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author     Suraj
 *  \version    0.3
 *  \author     PSP Team
 *  \version    0.4
 *
 */
#include <ti/sdo/pspdrivers/common/psp_common.h>
#include <ti/sdo/pspdrivers/pal_sys/vlynq/pal_sysvlynqDev.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osProtect.h>

/**
  * \brief  Number of callbacks per vlynq deveice
  */
#define  CB_COUNT_PER_DEV 2u

/**
 * \brief Structure to be used to hold information of callback function..
 */
typedef struct pal_vlynq_dev_cb_t
{

    PAL_VLYNQ_DEV_CB_FN       cb_fn;
    /**< Is of the type of cd_fn */
    void                      *cb_param;
    /**< Holds the pointer to parameters to callback function*/
    struct pal_vlynq_dev_cb_t *next;
    /**< Holds the pointer to next callback function */

} PAL_VLYNQ_DEV_CB_T;

/**
  * \brief Structure handle for list
  */
static PAL_VLYNQ_DEV_CB_T *gp_free_vlynq_dev_cb = NULL;

/*
 *  This function initializes the list for callback functions
 *
 *  Input:
 *          void
 *  Return:
 *          If successful, returns PAL_VLYNQ_OK
 *          If failed, returns negative number
 */
static PAL_Result vlynq_dev_cb_init(void)
{
    static PAL_VLYNQ_DEV_CB_T vlynq_dev_cb[MAX_DEV_COUNT*CB_COUNT_PER_DEV];
    PAL_VLYNQ_DEV_CB_T *p_vlynq_dev_cb = gp_free_vlynq_dev_cb = &vlynq_dev_cb[0];
    Uint32 i = 0;
    PAL_Result retCode = PAL_VLYNQ_OK;
    if(p_vlynq_dev_cb == NULL)
    {
        retCode = PAL_VLYNQ_INVALID_PARAM ;
    }else{
    for(i = 0; i < (CB_COUNT_PER_DEV * MAX_DEV_COUNT); i++)
    {
        p_vlynq_dev_cb->cb_param = NULL;
        p_vlynq_dev_cb->cb_fn    = NULL;
        p_vlynq_dev_cb->next = (i == (CB_COUNT_PER_DEV * MAX_DEV_COUNT) - 1u) ? NULL: &vlynq_dev_cb[i+1];
        p_vlynq_dev_cb = p_vlynq_dev_cb->next;
    }
    }
    return (retCode);
}
/*
 *  This function gets the location to the free callback function
 *
 *  Input:
 *         void
 *  Return:
 *          Returns the next free callback function from the list
 */
static PAL_VLYNQ_DEV_CB_T *vlynq_dev_get_free_dev_cb(void)
{
    PAL_VLYNQ_DEV_CB_T *p_vlynq_dev_cb = NULL;

    p_vlynq_dev_cb = gp_free_vlynq_dev_cb;
    if(p_vlynq_dev_cb != NULL) {
        gp_free_vlynq_dev_cb = gp_free_vlynq_dev_cb->next;
    }
    return (p_vlynq_dev_cb);
}
/*
 *  This function frees the callback function passed to it and adds it to the list
 *
 *  Input:
 *         p_vlynq_dev_cb [IN] The callback function to free
 *
 *  Return:
 *         void
 */
static void vlynq_dev_free_dev_cb(PAL_VLYNQ_DEV_CB_T *p_vlynq_dev_cb)
{
    if(NULL != p_vlynq_dev_cb)
{
        p_vlynq_dev_cb->cb_param = NULL;
        p_vlynq_dev_cb->cb_fn    = NULL;
        p_vlynq_dev_cb->next     = gp_free_vlynq_dev_cb;
        gp_free_vlynq_dev_cb     = p_vlynq_dev_cb;
    }
    return;
}

/**
 * \brief Used for device object information.
 */
typedef struct pal_vlynq_dev_t
{

    Char                   name[32];
    /**<  Name of the device */
    PAL_VLYNQ_HND          *vlynq;
    /**< Handle to the VLYNQ object     */
    struct pal_vlynq_dev_t * next;
    /**< Address pointer for pal_vlynq_dev_t */
    Uint8                  reset_bit;
    /**< Reset bit for the device  */
    Uint8                  irq_count;
    /**< No of irq's per device      */
    Uint8                  instance;
    /**< Instance no of the device  */
    Bool                   peer;
    /**< Element peer suggest peer or local */
    Int8                   irq[8];
    /**< The number of irq's per device   */
    PAL_VLYNQ_DEV_CB_T     *dev_cb;
    /**< Callback function for the device IRQ  */
} PAL_VLYNQ_DEV_T;


PAL_VLYNQ_DEV_T vlynq_dev[MAX_DEV_COUNT];
/**< Array of maximum device count of type PAL_VLYNQ_DEV_T */
static PAL_VLYNQ_DEV_T *gp_free_vlynq_dev_list = NULL;
/**< Handle for free device list */
static PAL_VLYNQ_DEV_T *gp_used_vlynq_dev_list = NULL;
/**< Handle for used device list */

/*
 *  This function gets free device from the device list
 *
 *  Input:
 *         void
 *
 *  Return:
 *         Returns a valid handle to the device
 */
static PAL_VLYNQ_DEV_T *vlynq_dev_get_free_dev(void)
{
    PAL_VLYNQ_DEV_T *p_vlynq_dev = NULL;
    /*Assign the device from the free list*/
    p_vlynq_dev = gp_free_vlynq_dev_list;
    /*On success assign update the free and used device list*/
    if(NULL != p_vlynq_dev)
    {
        gp_free_vlynq_dev_list = gp_free_vlynq_dev_list->next;
        p_vlynq_dev->next      = gp_used_vlynq_dev_list;
        gp_used_vlynq_dev_list = p_vlynq_dev;
    }

    return (p_vlynq_dev);
}

/*
 *  This function frees device and adds it to device list
 *
 *  Input:
 *        p_vlynq_dev  [IN] handle to the device to free
 *
 *  Return:
 *         void
 */
static void vlynq_dev_free_dev( PAL_VLYNQ_DEV_T *p_vlynq_dev)
{
    PAL_VLYNQ_DEV_T *p_tmp_vlynq_dev = NULL;
    PAL_VLYNQ_DEV_T *p_last_vlynq_dev = NULL;

    if(NULL != p_vlynq_dev )
    {

        for(p_tmp_vlynq_dev  = gp_used_vlynq_dev_list;
                p_tmp_vlynq_dev != NULL;
           p_tmp_vlynq_dev = p_tmp_vlynq_dev->next)
        {
            /*Loop till the match is found*/
            if(p_tmp_vlynq_dev == p_vlynq_dev)
            {
                if(NULL == p_last_vlynq_dev) {
                    gp_used_vlynq_dev_list = gp_used_vlynq_dev_list->next;
                }
                /*assigning the next pointer of the previous element to that of current element */
                else{
                    p_last_vlynq_dev->next = p_tmp_vlynq_dev->next;
                }
                break;
            }
            p_last_vlynq_dev = p_tmp_vlynq_dev;
        }

        if(NULL != p_tmp_vlynq_dev)
        {
            p_vlynq_dev->next      = gp_free_vlynq_dev_list;
            /*Add the device to the free list to make it the current pointer*/
            gp_free_vlynq_dev_list = p_vlynq_dev;

        }
    }
    return;
}

/**
 *  \brief This function Register for the callbacks.
 *
 *
 *  \param  vlynq_dev_cb    [IN]    The handle to remote device instance
 *  \param  cb_fn           [IN]    The callback function that is be called by
 *                                  implementation for local error, or remote
 *                                  error update conditions.
 *  \param  this_driver     [IN]    The driver instance, which is registering
 *                                  for the callbacks
 *
 *  \return If successful, returns 0
 *          If failed, returns negative number
 */
PAL_Result PAL_sysVlynqDevCbRegister( PAL_VLYNQ_DEV_HND *vlynq_dev_cb,
        PAL_VLYNQ_DEV_CB_FN cb_fn,
        void *this_driver)
{
    PAL_VLYNQ_DEV_T    *p_vlynq_dev = NULL;
    PAL_VLYNQ_DEV_CB_T *p_dev_cb = NULL;
    Uint32             cookie = 0;
    PAL_Result ret_val = PAL_VLYNQ_OK;

    if((NULL == vlynq_dev_cb) || (NULL == this_driver))
    {
        ret_val = PAL_VLYNQ_INVALID_PARAM;
    }else
      {
        p_vlynq_dev = (PAL_VLYNQ_DEV_T*)vlynq_dev_cb;
        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);
        /*Get free device callback from the list*/
        p_dev_cb = vlynq_dev_get_free_dev_cb();
        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

        if( (NULL == p_dev_cb) || (NULL == cb_fn) )
        {
            ret_val = PAL_VLYNQ_INTERNAL_ERROR;
        }else
          {
            p_dev_cb->cb_fn    = cb_fn;
            p_dev_cb->cb_param = this_driver;
            p_dev_cb->next      = p_vlynq_dev->dev_cb;
            p_vlynq_dev->dev_cb = p_dev_cb;
        }
    }
    return (ret_val);
}


/**
 *  \brief This function Unregister the callbacks.
 *
 *
 *  \param  vlynq_dev_uncb  [IN]    The handle to remote device instance
 *  \param  this_driver     [IN]    The driver instance, which is registering
 *                                  for the callbacks
 *
 *  \return If successful, returns 0
 *          If failed, returns negative number
 */
PAL_Result PAL_sysVlynqDevCbUnregister( PAL_VLYNQ_DEV_HND *vlynq_dev_uncb,
        void  *this_driver)
{
    PAL_Result         ret_val = PAL_VLYNQ_OK;
    PAL_VLYNQ_DEV_T    *p_vlynq_dev = NULL;
    PAL_VLYNQ_DEV_CB_T *p_dev_cb = NULL, *p_last_dev_cb = NULL;

    if(NULL == vlynq_dev_uncb)
    {
        ret_val = PAL_VLYNQ_INVALID_PARAM;
    }
    else
    {
        p_vlynq_dev = (PAL_VLYNQ_DEV_T * )vlynq_dev_uncb;

        for(p_dev_cb  = p_vlynq_dev->dev_cb;
            p_dev_cb != NULL;
            p_dev_cb = p_dev_cb->next )
        {
            if(p_dev_cb->cb_param == this_driver)
            {
                Uint32 cookie;

                if(NULL == p_last_dev_cb) {
                    p_vlynq_dev->dev_cb = p_dev_cb->next;
                }
                else {
                    p_last_dev_cb->next = p_dev_cb->next;
                }

                PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);
                vlynq_dev_free_dev_cb(p_dev_cb);
                PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
                break;
            }
            p_last_dev_cb = p_dev_cb;
        }
    }
    return (ret_val);
}


/**
 *  \brief  Get the handle for the device
 *
 *  \param  name         [IN]    The name of the device whose information is
 *                               being sought. This should same as that used
 *                               in the PAL_sysVlynqDevCreate( ).
 *  \param  instance     [IN]    The instance of the remote device.This should
 *                               same as that used in the PAL_sysVlynqDevCreate( ).
 *
 *  \return NULL, if no device could be found or a valid handle.
 */
PAL_VLYNQ_DEV_HND *PAL_sysVlynqDevFind(const Char *name, Uint8 instance)
{
    PAL_VLYNQ_DEV_T * p_vlynq_dev = NULL;

    if(NULL != name )
    {
        for(p_vlynq_dev = gp_used_vlynq_dev_list;
                p_vlynq_dev != NULL;
                p_vlynq_dev = p_vlynq_dev->next)
{
            if((strcmp(p_vlynq_dev->name, name) == 0) &&
                    (p_vlynq_dev->instance == instance)) {
                break;
            }
        }
    }
    return (PAL_VLYNQ_DEV_HND * )(p_vlynq_dev);
}

/**
 *  \brief This function Get the VLYNQ for this device.
 *
 *
 *  \param  vlynq_dev_get   [IN]    The device whose associated VLYNQ control module
 *                                  is being sought
 *
 *  \return NULL on error or a valid VLYNQ handle
 */
PAL_VLYNQ_HND *PAL_sysVlynqDevGetVlynq( PAL_VLYNQ_DEV_HND *vlynq_dev_get)
{
    PAL_VLYNQ_DEV_T * p_vlynq_dev = NULL;
    PAL_VLYNQ_HND *p_vlynq_hnd = NULL;

    if(NULL != vlynq_dev_get)
    {
        p_vlynq_dev = (PAL_VLYNQ_DEV_T* ) vlynq_dev_get;
        p_vlynq_hnd = p_vlynq_dev->vlynq ;
    }
    return (p_vlynq_hnd);

}

/**
 *  \brief This IOCTL is for adding and removing IRQ information from device
 *
 *  \param  vlynq_dev_ioctl [IN]  The VLYNQ device instance which would have been returned
 *  \param  cmd             [IN]  The command which is to be processed
 *  \param  cmd_val         [IN]  The value of the command to be executed
 *  \return If successful, returns 0
 *          If failed, returns error code
 */
PAL_Result pal_vlynq_dev_ioctl( PAL_VLYNQ_DEV_HND *vlynq_dev_ioctl, Uint32 cmd,
        Uint32 cmd_val)
{
    Uint32 i;
    PAL_VLYNQ_DEV_T * p_vlynq_dev = NULL;
    PAL_Result retCode = PAL_VLYNQ_OK;

    if(NULL == vlynq_dev_ioctl)
    {
        retCode = PAL_VLYNQ_INVALID_PARAM ;
    }else{
        p_vlynq_dev = (PAL_VLYNQ_DEV_T * )vlynq_dev_ioctl;

        switch(cmd)
        {
            case PAL_VLYNQ_DEV_ADD_IRQ:
                {
                   if(p_vlynq_dev->irq_count == MAX_IRQ_PER_VLYNQ) {
                        retCode = PAL_VLYNQ_INTERNAL_ERROR ;
                    }else{

                    for(i = 0; i < MAX_IRQ_PER_VLYNQ; i++)
                    {
                            if(p_vlynq_dev->irq[i] == (Int8)-1)
                        {
                                p_vlynq_dev->irq[i] = (Int8)cmd_val;
                                p_vlynq_dev->irq_count++;
                                break;
                        }
                    }
                }
                }
                break;

            case PAL_VLYNQ_DEV_REMOVE_IRQ:
                {
                   if(p_vlynq_dev->irq_count == 0)
                   {
                        retCode = (-2);
                   }else{

            for(i = 0; i < MAX_IRQ_PER_VLYNQ; i++)
            {
                            if(p_vlynq_dev->irq[i] == cmd_val)
                {
                                p_vlynq_dev->irq[i] = (Int8)PAL_VLYNQ_INVALID_PARAM;
                                p_vlynq_dev->irq_count--;
                    break;
                }
            }

            if(i== MAX_IRQ_PER_VLYNQ) {
                            retCode = PAL_VLYNQ_INTERNAL_ERROR;
                        }
            }
                }
                break;

            default:
                break;
        }
    }
    return(retCode);
}

/**
 *  \brief This function Get the mapped interrupts of the device.
 *
 *  \param  vlynq_dev_irq   [IN]  The vlynq handle
 *  \param  irq             [IN]  The placeholder for the identifiers of the IRQ(s)
 *                                vectors in the perspective of the chain
 *  \param  num_irq         [IN]  The number of interrupts to be read
 *
 *  \return If successful, returns PAL_VLYNQ_OK
 *          If failed, returns negative number
 */
PAL_Result PAL_sysVlynqDevFindIrq( PAL_VLYNQ_DEV_HND *vlynq_dev_irq,
            Uint8 *irq,
            Uint32 num_irq)
{
    PAL_Result retCode = PAL_VLYNQ_OK;
    PAL_VLYNQ_DEV_T * p_vlynq_dev = NULL;
    Uint32           i = 0;
    Int8            *p_dev_irq = NULL;

    if((NULL == vlynq_dev_irq ) || (NULL == irq))
    {
        retCode =  PAL_VLYNQ_INVALID_PARAM;
    }else{
         p_vlynq_dev = (PAL_VLYNQ_DEV_T * )vlynq_dev_irq;
        /*Check if the number of irq's requested is same as the irq count of the device*/
         if(num_irq <= p_vlynq_dev->irq_count)
         {
            p_dev_irq = (Int8*)p_vlynq_dev->irq;
            if(NULL != p_dev_irq )
            {
                for(i = 0; i < num_irq; i++)
                {
                    if(p_dev_irq[i] != (Int8)-1) {
                        irq[i] = (Uint8 )p_dev_irq[i];
                    }
                }

            }
        }else{
            retCode =  PAL_VLYNQ_INVALID_PARAM;
        }
    }
    return (retCode);
}


/**
 *  \brief This function Creates a device reference.
 *
 *
 *  \param  vlynq       [IN]    The VLYNQ module instance to which a device has
 *                              to be added
 *  \param  name        [IN]    Name of the device; shall not exceed 30 characters
 *                              and shall be NULL terminated
 *  \param  instance    [IN]    The instance of the device; it is 0 based.
 *  \param  reset_bit   [IN]    The reset bit for the VLYNQ device in the perspective
 *                              of the system
 *  \param  peer        [IN]    Identifies whether the device is connected to the local
 *                              or peer VLYNQ module.
 *
 *  \return On failure, returns NULL otherwise,
 *          returns a valid handle to the device instance.
 */
PAL_VLYNQ_DEV_HND *PAL_sysVlynqDevCreate(PAL_VLYNQ_HND *vlynq, const char *name,
                    Uint32 instance, Int32 reset_bit,
                    Bool peer)
{
    PAL_VLYNQ_DEV_T * p_vlynq_dev = NULL;

    if((NULL != vlynq) && (NULL != name ))
    {
        p_vlynq_dev = vlynq_dev_get_free_dev();

        if( NULL != p_vlynq_dev)
        {
            strcpy(p_vlynq_dev->name, name);
            p_vlynq_dev->instance  = (Uint8)instance;
            p_vlynq_dev->reset_bit = (Uint8)reset_bit;
            p_vlynq_dev->vlynq     = vlynq;
            p_vlynq_dev->peer      = peer;
    }
    }
    return (PAL_VLYNQ_DEV_HND * )(p_vlynq_dev);
}


/**
 *  \brief This function Destroys the device reference
 *
 *
 *  \param  vlynq_dev_destroy   [IN]    The VLYNQ device instance which would have been
 *                                      returned
 *
 *  \return If successful, returns 0
 *          If failed, returns negative number
 */
PAL_Result PAL_sysVlynqDevDestroy( PAL_VLYNQ_DEV_HND *vlynq_dev_destroy)
{
    PAL_Result retCode = PAL_VLYNQ_OK;
    PAL_VLYNQ_DEV_T * p_vlynq_dev = NULL;
    Uint32 j = 0;
    Uint32 cookie = 0;

    PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);
    if(NULL == vlynq_dev_destroy)
    {
        retCode = PAL_VLYNQ_INVALID_PARAM;
    }else{
        p_vlynq_dev = (PAL_VLYNQ_DEV_T * )vlynq_dev_destroy;

        if(NULL != p_vlynq_dev->dev_cb)
        {
            retCode = PAL_VLYNQ_INTERNAL_ERROR;
        }else{

            p_vlynq_dev->vlynq      = NULL;
            p_vlynq_dev->irq_count  = 0;

            for(j = 0; j < MAX_IRQ_PER_VLYNQ; j++)
            {
                p_vlynq_dev->irq[j] = PAL_VLYNQ_INVALID_PARAM;
            }

            vlynq_dev_free_dev(p_vlynq_dev);
        }
    }
    PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
    return (retCode);
}
/**
 *  \brief Get the reset bit of the device.
 *
 *
 *  \param  vlynq_dev_bit   [IN]    The handle to remote device instance; would have
 *                                  been returned by PAL_sysVlynqDevFind( ).
 *
 *  \param  reset_bit       [IN]    The placeholder for the identifiers of reset bit.
 *                                  Only for on board remote devices for which hardware
 *                                  reset provisions have been made.
 *
 *  \return If successful, returns 0
 *          If failed, returns negative number
 */
PAL_Result PAL_sysVlynqDevGetResetBit( PAL_VLYNQ_DEV_HND *vlynq_dev_bit,
            Uint32 *reset_bit)
{
    PAL_VLYNQ_DEV_T * p_vlynq_dev = NULL;
    PAL_Result ret_val = PAL_VLYNQ_INVALID_PARAM;

    if((NULL != vlynq_dev_bit) && (NULL != reset_bit ) )
    {
        p_vlynq_dev = (PAL_VLYNQ_DEV_T * )vlynq_dev_bit;
        * reset_bit = (Uint32)p_vlynq_dev->reset_bit;
        ret_val = PAL_VLYNQ_OK;
    }
    return (ret_val);
}

/**
  * \brief Get Vlynq initialize
  */
PAL_Result pal_vlynq_dev_init(void)
{
    PAL_VLYNQ_DEV_T *p_vlynq_dev = gp_free_vlynq_dev_list = &vlynq_dev[0];
    Uint32 i = 0;
    Uint32 j = 0;
    PAL_Result ret_val = PAL_VLYNQ_INVALID_PARAM;

    if(NULL != p_vlynq_dev)
    {
        for(i = 0; i < MAX_DEV_COUNT; i++)
        {
            p_vlynq_dev->vlynq     = NULL;
            p_vlynq_dev->irq_count = 0;
            p_vlynq_dev->dev_cb    = NULL;

            for(j = 0; j < MAX_IRQ_PER_VLYNQ; j++)
            {
                p_vlynq_dev->irq[j] = (Int8)PAL_VLYNQ_INVALID_PARAM;
            }

            p_vlynq_dev->next = (i== MAX_DEV_COUNT - 1u) ? NULL: &vlynq_dev[i+1];
            p_vlynq_dev       = p_vlynq_dev->next;
        }

        vlynq_dev_cb_init();
        ret_val = PAL_VLYNQ_OK;
    }
    return(ret_val);
}
/*
 *  This will be called by the Local and Peer error ISR.
 *  This will call the application callback
 *  (Protected API to be used only by pal_sysvlynq.c)
 *  Input:
 *    vlynq_dev_event   [IN]    The handle device instance; would have
 *                              been returned by PAL_sysVlynqDevFind( ).
 *    cmd               [IN]    Input command parameter to the callback function
 *    val               [IN]    Input value parameter to the callback function
 *   Return:
 *   If successful, returns PAL_VLYNQ_OK
 *   If failed, returns error code
 */
PAL_Result pal_vlynq_dev_handle_event( PAL_VLYNQ_DEV_HND *vlynq_dev_event,
        Uint32 cmd, Uint32 val)
{
    PAL_VLYNQ_DEV_T * p_vlynq_dev = NULL;
    PAL_Result      ret_val      = PAL_VLYNQ_INTERNAL_ERROR;

    if(vlynq_dev_event != NULL)
    {
        PAL_VLYNQ_DEV_CB_T * p_dev_cb;
        p_vlynq_dev = (PAL_VLYNQ_DEV_T * )vlynq_dev_event;

        for(p_dev_cb  = p_vlynq_dev->dev_cb;
                p_dev_cb != NULL;
                p_dev_cb  = p_dev_cb->next)
        {
            p_dev_cb->cb_fn(p_dev_cb->cb_param, cmd, val);
        }
        ret_val = PAL_VLYNQ_OK;
    }
    return (ret_val);
}

/**
 *  \brief Dump the device information
 *
 *
 *  \param  vlynq_dev_dump  [IN]    The handle to remote device instance; would have
 *                                  been returned by PAL_sysVlynqDevFind( ).
 *
 *  \param  buf             [OUT]   The placeholder for the device information
 *  \param  limit           [IN]    limit of buffer
 *  \param  eof             [OUT]   end of buffer indicator
 *
 *  \return If successful, returns 0
 *          If failed, returns error code.
 */
Int32 vlynq_dev_dump_dev( PAL_VLYNQ_DEV_HND * vlynq_dev_dump, Char * buf, Int32 limit,
        const Int32 *eof)
{
    Int32 irq_count = 0, len = 0;

    PAL_VLYNQ_DEV_T  * p_vlynq_dev = NULL;

    if((vlynq_dev_dump == NULL) || ((eof == NULL) || (buf == NULL) ) )
    {
        return(len);
    }
     p_vlynq_dev = (PAL_VLYNQ_DEV_T  * )vlynq_dev_dump;

    if(len < limit)
    {
        len += sprintf(buf + len, "    %s%d\n", p_vlynq_dev->name,
                p_vlynq_dev->instance);
        }

        if(len < limit) {
            len += sprintf(buf + len, "      IRQ-id(s): ");
        }

    for (irq_count = 0; irq_count < MAX_IRQ_PER_VLYNQ; irq_count++)
        {
            Int32 val;
        if(p_vlynq_dev->irq[irq_count] != (Int8)-1)
        {
            val = (Int32)p_vlynq_dev->irq[irq_count];
            if(len < limit)
            {
                    len += sprintf(buf + len, "%d, ", val);
                }
            }
        }

        if(len < limit) {
            len += sprintf(buf + len, "\n");
    }
    return(len);
}





