/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006, 2007
 *
 *   Use of this software is controlled by the terms and conditions found
 *   in the license agreement under which this software has been supplied.
 *  ============================================================================
 */
/** ============================================================================
 *
 *   @file  Ddr2_example.c
 *
 *   @path  $(CSL_EXAMPLE_PATH)\ddr2\src
 *
 *   @desc  example for DDR2 Register layer usage.
 *
 * =============================================================================
 *   @n Target Platform: EVM
 *  ============================================================================
 *   @n <b> Example Description </b>
 *   @verbatim
 *    This Example, 
 *     1. Configure DDR2 exactly the same as "setup_ddr2( freq )" in 
 *        evmdm6437.gel does. During the configuration, 8 messages are printed 
 *	      out in the CCS stdout window showing how to use the 8 CSL macros.
 *     2. Write data to DDR2 memory and read it back to verify.
 *
 * =============================================================================
 *      
 *   <b> Procedure to run the example </b>
 *   @verbatim
 *     1. Disable the line setup_ddr2() in your gel file and then boot the 
 *        board using CCS
 *     2. Open memory window, set address to 0x80000000, and try to write 
 *        anything to DDR2. You will find that the correct value is not 
 *        displayed in the memory window, which means DDR2 is not ready.
 *     3. Load and run the program.
 *     4. After the program finishes, write to the DDR2 memory window. You 
 *        should find that the correct value is displayed in the window.
 *          
  * How to test 
 *     1. Disable the line setup_ddr2() in your gel file and then
 *        boot the board using CCS
 *     2. Open memory window, set address to 0x80000000, and try
 *        to write anything to DDR2. You will find that the correct
 *        value is not displayed in the memory window, which means
 *        DDR2 is not ready.
 *     3. Load and run the program. The program basically configures
 *        DDR2 as being done in the setup_ddr2() function in the 
 *        gel file. Some messages are printed out to show how to
 *        use CSL macros.
 *     4. After the program finishes, write to the DDR2 memory 
 *        window. You should find that the correct value is 
 *        displayed in the window.
 *
 *   @endverbatim
 *
 */
/* =============================================================================
 *  Revision History
 *  ===============
 *
 *  22-March-2007 File Created.
 *
 * =============================================================================
 */
#include <tistdtypes.h>
#include <soc.h>
#include <cslr_sys.h>
#include <cslr_psc.h>
#include <cslr_ddr2.h>
#include <stdio.h>


/*
 * =============================================================================
 *   @func   sw_wait
 *
 *   @desc
 *     This is the delay routine.
 *
 * =============================================================================
 */
void sw_wait (int delay) 
{
    volatile int i;
    for( i = 0 ; i < delay ; i++ ) {
    }
}


/*
 * =============================================================================
 *   @func   setup_ddr2
 *
 *   @desc
 *       example code showing how to use CSL macros, setting up DDR2 
 *
 * =============================================================================
 */
void setup_ddr2 ( 
    int    freq 
) 
{
    CSL_Ddr2RegsOvly ddr2Regs = (CSL_Ddr2RegsOvly)CSL_DDR2_0_REGS; 
    CSL_PscRegsOvly pscRegs = (CSL_PscRegsOvly)CSL_PSC_0_REGS;
    CSL_SysRegsOvly devRegs = (CSL_SysRegsOvly)CSL_SYS_0_REGS;
    int ddrvtprVal;
    
    printf( "DDR2 is being setup ( DDR2 @ %d MHz ) w/ 32-bit bus... \n", freq );
    
    /* 1. Enable DDR2 PHY 
    ** clear state */
    printf("\n*******************************************************\n");
    printf("CSL_FINST(reg, PER_REG_FIELD, TOKEN) example\n");
    printf("reg: pscRegs->MDCTL[13] which is used to config DDR2; \n");
    printf("PER_REG_FIELD: PSC_MDCTL_NEXT corresponding to bit [4:0]; \n" );
    printf("TOKEN: SWRSTDISABLE = 0; \n" );
    printf("This macro sets the PER_REG_FIELD in reg to TOKEN value\n");
    printf("That is, bits[4:0] in pscRegs->MDCTL[13] is set to 00000b\n\n");
    CSL_FINST( pscRegs->MDCTL[CSL_PSC_DDR2], PSC_MDCTL_NEXT, SWRSTDISABLE );                   
    
    /* set to enable */
    printf("\n*******************************************************\n");
    printf("CSL_FINS(reg, PER_REG_FIELD, val) example\n");
    printf("reg: pscRegs->MDCTL[13] which is used to config DDR2; \n");
    printf("PER_REG_FIELD: PSC_MDCTL_NEXT corresponding to bit [4:0]; \n" );
    printf("val = 3; \n" );
    printf("This macro sets the PER_REG_FIELD in reg to val\n");
    printf("That is, bits[4:0] in pscRegs->MDCTL[13] is set to 00011b\n\n");
    CSL_FINS( pscRegs->MDCTL[CSL_PSC_DDR2], PSC_MDCTL_NEXT, 3 );                    
    /* transition power (always on)*/
    CSL_FINST( pscRegs->PTCMD, PSC_PTCMD_GO0, SET ); 
    
    /* Wait for transtion to finish*/
    printf("\n*******************************************************\n");
    printf("CSL_FEXT(reg, PER_REG_FIELD) example\n");
    printf("reg: pscRegs->PSTAT; \n");
    printf("PER_REG_FIELD: PSC_PTSTAT_GOSTAT corresponding to bit [31:3]; \n" );
    printf("This macro reads the PER_REG_FIELD in reg and right \n");
    printf("justifies it;\n");
    printf("That is, bits[31:3] in pscRegs->PSTAT is read and stored to \n");
    printf("bit[28:0] of the output; \n\n" );
    while ( CSL_FEXT( pscRegs->PTSTAT, PSC_PTSTAT_GOSTAT0 )  
          & CSL_PSC_PTSTAT_GOSTAT0_IN_PROGRESS );
    
    /* Wait for module state enable  */
    printf("\n*******************************************************\n");
    printf("CSL_FEXTR(reg, msb, lsb) example\n");
    printf("reg: pscRegs->MDSTAT[13] which contains DDR2 status; \n");
    printf("msb: 6; \n" );
    printf("lsb: 0; \n");
    printf("This macro reads the bits[msb:lsb] in reg and right \n" );
    printf("justifies it;\n");
    printf("That is, bit[6:0] in pscRegs->MDSTAT[13] is read and \n");
    printf("stored to bit[6:0] of the output; \n\n" );
    while ( CSL_FEXTR( pscRegs->MDSTAT[CSL_PSC_DDR2], 6, 0 )
          != CSL_PSC_MDSTAT_STATE_ENABLE );
    
    
    /* 2. DDR2 Initialization
    * DLL powered, ReadLatency=5
    * [31:6] = 0x1400190h; always write this value to it (reset value)
    * [5]: DLLRESET = 0;   DLL is out of reset
    * [4]: DLLPWRDN = 0;   DLL is powered up
    * [2:0]: READLAT = 5;  read latency is 5 */
    CSL_FINST( ddr2Regs->DDRPHYCR, DDR2_DDRPHYCR_DLLRESET, OUTOFRESET );
    CSL_FINST( ddr2Regs->DDRPHYCR, DDR2_DDRPHYCR_DLLPWRDN, DISABLE ); 
    printf("\n*******************************************************\n");
    printf("CSL_FINSR(reg, msb, lsb, val) example\n");
    printf("reg: DDR_DDRPHYCR; \n");
    printf("msb: 2 \n" );
    printf("lsb: 0 \n" );
    printf("val = 5; \n" );
    printf("This macro sets the bit[msb:lsb] in reg to val;\n");
    printf("That is, bits[2:0] in DDR_DDRPHYCR is set to 101b\n\n");
    CSL_FINSR( ddr2Regs->DDRPHYCR, 2, 0, 5 );
    
    /* DDR Bank: 32-bit bus, CAS=4,
    * [31:24]: rsvd
    * [23]: BOOTUNLOCK = 0; // DDRDRIVE bit may not be changed
    * [22:19]: rsvd; // always = 02h
    * [18]: DDRDRIVE = 0; // normal drive strength
    * [17:16]: rsvd; // always = 03h
    * [15]: TIMUNLOCK = 1; // SDTIMR and SDTIMR2 can be changed
    * [14]: NM = 0; // 32 bit width
    * [13:12]: rsvd
    * [11:9]: CL = 4h; // cas latency is 4
    * [8:7]: rsvd
    * [6:4]: IBANK = 2; // 4 banks
    * [3]: rsvd
    * [2:0]: PAGESIZE = 2; // 1024-word per page  */
    CSL_FINS( ddr2Regs->SDBCR, DDR2_SDBCR_BOOTUNLOCK, 0 );
    CSL_FINST( ddr2Regs->SDBCR, DDR2_SDBCR_DDRDRIVE, NORMAL );
    
    printf("\n*******************************************************\n");
    printf("CSL_FMKT(PER_REG_FIELD, TOKEN) example\n");
    printf("PER_REG_FIELD: DDR2_CONFIG_TIMING_UNLOCK, which is bit[15]\n");
    printf("in register ddr2Regs->CONFIG\n");
    printf("TOKEN: UNLOCK = 1b\n" );
    printf("The output of this macro is a temporary variable in which \n");
    printf("bit(s) corresponding to PER_REG_FIELD is(are) set TOKEN\n" );
    printf("value and other bits are cleared to zero\n");
    printf("That is, bit[15] is set to 1b and other bits are 0s, which\n");
    printf("makes the output 0x00008000\n");
    printf("This macro is typically used to set certain bit(s) in a\n" );
    printf("register, i.e. reg |= CSL_FMKT(PER_REG_FIELD, TOKEN)\n\n" );
    ddr2Regs->SDBCR |= CSL_FMKT( DDR2_SDBCR_TIMUNLOCK, SET );
    
    printf("\n*******************************************************\n");
    printf("CSL_FMK(PER_REG_FIELD, val) example\n");
    printf("PER_REG_FIELD: DDR2_CONFIG_NARROW_MODE, which is bit[14]\n");
    printf("in register ddr2Regs->CONFIG\n");
    printf("val = 1 which makes the bus width 32bits\n" );
    printf("The output of this macro is a temporary variable in which\n");
    printf("bit(s) corresponding to PER_REG_FIELD is(are) set val and\n" );
    printf("other bits are cleared to zero\n");
    printf("That is, bit[14] is set to 1b and other bits are 0s, which\n");
    printf("makes the output 0x00004000\n");
    printf("This macro is typically used to set certain bit(s) in a\n" );
    printf("register, i.e. reg |= CSL_FMK(PER_REG_FIELD, val) or clear\n");
    printf("certain bit(s), i.e. reg &= ~(CSL_FMK(PER_REG_FIELD, val));\n\n");
    ddr2Regs->SDBCR &= ~( CSL_FMK( DDR2_SDBCR_NM, 1 ) );
    
    CSL_FINST( ddr2Regs->SDBCR, DDR2_SDBCR_CL, FOUR );
    CSL_FINST( ddr2Regs->SDBCR, DDR2_SDBCR_IBANK, FOUR );
    CSL_FINS( ddr2Regs->SDBCR, DDR2_SDBCR_PAGESIZE, 2 );                
    
    /* DDR Timing ( set for 162 MHz )
    * [31:25]: T_RFC = 11; 
    * [24:22]: T_RP = 1;  
    * [21:19]: T_RCD = 1; 
    * [18:16]: T_WR = 1; 
    * [15:11]: T_RAS = 4; 
    * [10:6]: T_RC = 5; 
    * [5:3]: T_RRD = 1;
    * [2]: rsvd
    * [1:0]: T_WTR = 0; */
    CSL_FINS( ddr2Regs->SDTIMR, DDR2_SDTIMR_T_RFC, 11 );
    CSL_FINS( ddr2Regs->SDTIMR, DDR2_SDTIMR_T_RP, 1 );
    CSL_FINS( ddr2Regs->SDTIMR, DDR2_SDTIMR_T_RCD, 1 );
    CSL_FINS( ddr2Regs->SDTIMR, DDR2_SDTIMR_T_WR, 1 );
    CSL_FINS( ddr2Regs->SDTIMR, DDR2_SDTIMR_T_RAS, 4 );
    CSL_FINS( ddr2Regs->SDTIMR, DDR2_SDTIMR_T_RC, 5 );
    CSL_FINS( ddr2Regs->SDTIMR, DDR2_SDTIMR_T_RRD, 1 );
    CSL_FINS( ddr2Regs->SDTIMR, DDR2_SDTIMR_T_WTR, 0 );    
                
    /* DDR Timing ( set for 162 MHz )
    * [22:16] T_XSNR = 0xC; 
    * [15:8] T_XSRD = 0xC7;  
    * [7:5] T_RTP = 0; 
    * [4:0] T_CKE = 2;  */
    CSL_FINS( ddr2Regs->SDTIMR2, DDR2_SDTIMR2_T_XSNR, 0xC );
    CSL_FINS( ddr2Regs->SDTIMR2, DDR2_SDTIMR2_T_XSRD, 0xC7 );
    CSL_FINS( ddr2Regs->SDTIMR2, DDR2_SDTIMR2_T_RTP, 0 );
    CSL_FINS( ddr2Regs->SDTIMR2, DDR2_SDTIMR2_T_CKE, 2 );
    
    /* DDR Bank: cannot modify */
    CSL_FINST( ddr2Regs->SDBCR, DDR2_SDBCR_TIMUNLOCK, CLEAR );
                        
    /* Refresh Rate - XXX MHz * 7.8us */
    freq *= 7.8;
    CSL_FINS( ddr2Regs->SDRCR, DDR2_SDRCR_RR, freq );      
    
    /* 3. Reset DDR2 PHY
    * clear state */
    CSL_FINST( pscRegs->MDCTL[CSL_PSC_DDR2], PSC_MDCTL_NEXT, SWRSTDISABLE );                 
    
    /* Set to SyncReset*/
    CSL_FINST( pscRegs->MDCTL[CSL_PSC_DDR2], PSC_MDCTL_NEXT, SYNCRST );  
    
    /* transition power (always on)*/
    CSL_FINST( pscRegs->PTCMD, PSC_PTCMD_GO0, SET );                  
    
    /* Wait for transtion to finish*/
    while ( CSL_FEXT( pscRegs->PTSTAT, PSC_PTSTAT_GOSTAT0 )  
          & CSL_PSC_PTSTAT_GOSTAT0_IN_PROGRESS );
    
    /* Wait for module state syncreset*/
    while ( CSL_FEXT( pscRegs->MDSTAT[CSL_PSC_DDR2], PSC_MDSTAT_STATE )
          != CSL_PSC_MDSTAT_STATE_SYNCRST );
    
    /* 4. Enable DDR2 PHY*/
    CSL_FINST( pscRegs->MDCTL[CSL_PSC_DDR2], PSC_MDCTL_NEXT, SWRSTDISABLE );     
    
    /* set to enable*/
    CSL_FINST( pscRegs->MDCTL[CSL_PSC_DDR2], PSC_MDCTL_NEXT, ENABLE );  
    
    /* transition power (always on)*/
    CSL_FINST( pscRegs->PTCMD, PSC_PTCMD_GO0, SET );  
    
    /* Wait for transtion to finish*/
    while ( CSL_FEXT( pscRegs->PTSTAT, PSC_PTSTAT_GOSTAT0 )  
          & CSL_PSC_PTSTAT_GOSTAT0_IN_PROGRESS );
    
    /* Wait for module state enable*/
    while ( CSL_FEXT( pscRegs->MDSTAT[CSL_PSC_DDR2], PSC_MDSTAT_STATE )
          != CSL_PSC_MDSTAT_STATE_ENABLE );
    
    
    /* 5. Enable VTP calibration
    * [31:16]: rsvd
    * [15] RECAL = 0; // normal operation
    * [14] rsvd;  
    * [13] EN = 1; // calibration enabled
    * [12:10]: rsvd
    * [9:5] PCH = 0; // P channel value
    * [4:0] NCH = 0x1F; // N channel value */
    CSL_FINS( ddr2Regs->VTPIOCR, DDR2_VTPIOCR_RECAL, 0 );                       
    
    printf("\n*******************************************************\n");
    printf("CSL_FMKR(msb, lsb, val) example\n");
    printf("msb: 13\n");
    printf("lsb: 13\n");
    printf("val: 1b\n" );
    printf("The output of this macro is a temporary variable in which \n");
    printf("bit(s)[msb:lsb] is(are) set val and other bits are cleared\n");
    printf("to zero\n");
    printf("That is, bit[13] is set to 1b and other bits are 0s, which\n");
    printf("makes the output 0x00002000\n");
    printf("This macro is typically used to set certain bit(s) in a\n" );
    printf("register, i.e. reg |= CSL_FMKR(msb, lsb, val)\n\n" );
    ddr2Regs->VTPIOCR |= CSL_FMKR(13, 13, 1);
    
    CSL_FINS( ddr2Regs->VTPIOCR, DDR2_VTPIOCR_PCH, 0 );
    CSL_FINS( ddr2Regs->VTPIOCR, DDR2_VTPIOCR_NCH, 0x1F );
           
    CSL_FINST( ddr2Regs->VTPIOCR, DDR2_VTPIOCR_RECAL, VTPIO );
    
    sw_wait( 1500 ); // wait for 33 VTP clock cycles
    
    /* Enable access to DDR VTP reg */
    CSL_FINS( devRegs->DDRVTPER, SYS_DDRVTPER_EN, 1 );                          
    
    /* Set VTP fields PCH & NCH (bits [9:0]) */
    ddrvtprVal = CSL_FEXTR( devRegs->DDRVTPR, 9, 0 );
    CSL_FINSR( ddr2Regs->VTPIOCR, 9, 0, ddrvtprVal );
    
    /* Wait for few more clock cycles*/
    sw_wait( 1500 );  
    
    /* Disable VTP calibration */
    CSL_FINST( ddr2Regs->VTPIOCR, DDR2_VTPIOCR_EN, DISABLED );
              
    /* Disable access to DDR VTP reg */
    CSL_FINS( devRegs->DDRVTPER, SYS_DDRVTPER_EN, 0 );   
    
    printf( "DDR2 setup is done\n\n" );
}

/*
 * =============================================================================
 *   @func   main
 *
 *   @desc
 *     This is the main routine for the file.
 *
 * =============================================================================
 */
void main (
    void
) 
{
    volatile unsigned int *ddr2Addr = (unsigned int*)0x80000000;
    int i;
    setup_ddr2( 162 );
    
    printf("clearing DDR2\n\n");
    
    /* 128M memory */
    for( i = 0; i < 0x2000000; i += 2 ) {
        *ddr2Addr++ = 0;
        *ddr2Addr++ = 0;
    }
    
    printf("writing to DDR2\n\n");
    ddr2Addr = (unsigned int*)0x80000000;
    for( i = 0; i < 0x2000000; i += 2 ) {
        *ddr2Addr++ = 0xA5A5A5A5;
        *ddr2Addr++ = 0x5A5A5A5A;
    }
    
    printf("reading DDR2 and verifying\n\n");
    printf("Please wait...\n");
    ddr2Addr = (unsigned int*)0x80000000;
    for( i = 0; i < 0x2000000; i += 2 ) {
        if ( *ddr2Addr++ != 0xA5A5A5A5) {
            printf("Error!\n");
            break;
        }
        if( *ddr2Addr++ != 0x5A5A5A5A ) {
            printf("Error!\n");
            break;
        }
    }
    printf("Example Done\n");
    return;
}

