/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006, 2007
 *
 *   Use of this software is controlled by the terms and conditions found
 *   in the license agreement under which this software has been supplied.
 *  ============================================================================
 */

/** ============================================================================
 *  @brief  A simple example to demonstrate CSL 3.x I2C register layer usage.  
 *
 *  @file  I2c_example.c
 *
 *  <b> Example description </b> 
 *  @verbatim
    The I2C example sets up the i2c interface in loopback mode and will transmit 
    and receive 8 characters and verify that they are correct.  A pass\fail 
    status will be returned from main.
    @endverbatim
 *   
 *   
 *  @n <b> Procedure to run the example </b>
 *  @verbatim
    1. Configure the CCS setup to work with the emulator being used.
    2. Please refer CCS manual for setup configuration and loading 
       proper GEL file.
    3. Launch CCS.
    4. Open project I2c_example.pjt.
    5. Build the project and load the .out file for the project.
    @endverbatim
 *
 *  ============================================================================
 **/

#include <csl_types.h>
#include <soc.h>
#include <cslr_i2c.h>
#include <cslr_psc.h>
#include <stdio.h>

#define NUM_CHARS       8

static void device_init(void); 
static void init_i2c(void);   
static int test_i2c(void);   
 
CSL_I2cRegsOvly i2cRegs = (CSL_I2cRegsOvly)CSL_I2C_0_REGS;


int main (void) 
{
  //enable i2c in power and sleep controller
  device_init(); 
  
  //setup i2c registers and start i2c running 
  init_i2c();
  
  //test loopback, returns 0 for pass and 1 for fail 
  return(test_i2c());

}

void init_i2c(void)
{
  //i2c mode register 
  i2cRegs->ICMDR = CSL_FMKT(I2C_ICMDR_MST,MASTER) //master mode 
                            | CSL_FMKT(I2C_ICMDR_TRX,TX_MODE) //transmitter mode
                            | CSL_FMKT(I2C_ICMDR_RM,ENABLE) //repeat mode
                            | CSL_FMKT(I2C_ICMDR_DLB,ENABLE) //digital loopback
                            | CSL_FMKT(I2C_ICMDR_STT,SET); //enable start 
  
  //set i2c clock low time divide down value, 0-FFFFh
  i2cRegs->ICCLKL = CSL_FMK(I2C_ICCLKL_ICCL,0x10); 
                   
  //set i2c clock high time divide down value, 0-FFFFh
  i2cRegs->ICCLKH = CSL_FMK(I2C_ICCLKH_ICCH,0x10);
         
  //set i2c prescalar divide down value, 0-FFh
  i2cRegs->ICPSC = CSL_FMK(I2C_ICPSC_IPSC,0x2);

  //bring i2c module out of reset
  CSL_FINST(i2cRegs->ICMDR,I2C_ICMDR_IRS,ENABLE);

}

int test_i2c(void)
{
 int i;
 char trans_val[NUM_CHARS]={'l','o','o','p','b','a','c','k'};
 char rec_val[NUM_CHARS];

 printf("\nTesting I2C loopback.\n\n");

 for(i=0; i<(sizeof(trans_val)/sizeof(trans_val[0])); i++)
 {
   //poll transmitter ready
   while(CSL_FEXT(i2cRegs->ICSTR,I2C_ICSTR_ICXRDY)!= 1);
 
   //write to transmit register
   i2cRegs->ICDXR = trans_val[i];

   //poll receiver is ready
   while(CSL_FEXT(i2cRegs->ICSTR,I2C_ICSTR_ICRRDY)!= 1);
        
   //read receive register
   rec_val[i] = i2cRegs->ICDRR;
 }
   
 for(i=0; i<(sizeof(trans_val)/sizeof(trans_val[0])); i++)
 {  
   if(trans_val[i]==rec_val[i])
     printf("transmit: %c  receive: %c\n",trans_val[i],rec_val[i]);
   else
     {
       printf(" error\n");
       printf("\nI2C Loopback Test:  FAILED");
       return(1);
     } 
 }
 printf("\nI2C Loopback Test:  PASSED\n"); 
 printf("\nLoopback test complete.\n");
 return(0);
}

void device_init(void) 
{
  
  CSL_PscRegsOvly pscRegs = (CSL_PscRegsOvly)CSL_PSC_0_REGS;

  // deassert I2C local PSC reset and set NEXT state to ENABLE
  pscRegs->MDCTL[CSL_PSC_I2C] = CSL_FMKT( PSC_MDCTL_NEXT, ENABLE ) 
                               | CSL_FMKT( PSC_MDCTL_LRST, DEASSERT );
  //move I2C PSC to Next state
  pscRegs->PTCMD = CSL_FMKT(  PSC_PTCMD_GO0, SET );
 
  //wait for transition
  while ( CSL_FEXT( pscRegs->MDSTAT[CSL_PSC_I2C], PSC_MDSTAT_STATE )
          != CSL_PSC_MDSTAT_STATE_ENABLE );

}

 
