/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006, 2007
 *
 *   Use of this software is controlled by the terms and conditions found
 *   in the license agreement under which this software has been supplied.
 *  ============================================================================
 */

/** ============================================================================
 *  @brief  A simple example to demonstrate CSL 3.x IDMA register layer usage.  
 *
 *  @file  idma_example.c
 *
 *  <b> Example description </b> 
 *  @verbatim
    The IDMA example sets up a simple internal memory to internal memory 
    transfer using the IDMA.  The IDMA will transfer 20 32-bit words from the 
    source buffer to the destination buffer.  The contents of the buffers are 
    compared and a pass\fail status will be returned from main.
    @endverbatim
 *   
 *   
 *  @n <b> Procedure to run the example </b>
 *  @verbatim
    1. Configure the CCS setup to work with the emulator being used.
    2. Please refer CCS manual for setup configuration and loading 
       proper GEL file.
    3. Launch CCS.
    4. Open project Idma_example.pjt.
    5. Build the project and load the .out file for the project.
    @endverbatim
 *
 *  ============================================================================
 **/
   
#include <stdio.h>
#include <cslr_idma.h>
#include <soc.h>

#define DATA_COUNT_BYTES   80
#define DATA_CNT    20                              // Data count
                        
const unsigned int src[DATA_CNT] =                  // Source buffer 
 {
    0xDEADBEEF, 0xFADEBABE, 0x5AA51C3A, 0xD4536BA3,
    0x5E69BA23, 0x4884A01F, 0x9265ACDA, 0xFFFF0123,
    0xBEADDABE, 0x234A76B2, 0x9675ABCD, 0xABCDEF12,
    0xEEEECDEA, 0x01234567, 0x00000000, 0xFEEDFADE,
    0x0A1B2C3D, 0x4E5F6B7C, 0x5AA5ECCE, 0xFABEFACE
 };

const unsigned int dst[DATA_CNT];                   // Destination buffer
static void init_idma(void);
static int test_idma(void);
                                                          
CSL_IdmaRegsOvly idmaRegs = (CSL_IdmaRegsOvly)CSL_IDMA_0_REGS;

int main(void)
{
  // Initialize IDMA and perform transfer
  init_idma();
    
  // Verify destination buffer matches source buffer
  return(test_idma());	
}

void init_idma(void)    
{
  unsigned int restore_value;
   
  restore_value = _disable_interrupts();      // Save interrupt state                             
    
  idmaRegs->IDMA1_SOURCE = (Uint32)src;       // Set source
  idmaRegs->IDMA1_DEST = (Uint32)dst;         // Set destination
     
  // Set the priority to low and turn on cpu interrupt enable
  // Set the transfer count
  idmaRegs->IDMA1_COUNT = CSL_FMKT(IDMA_IDMA1_COUNT_PRI,PRI7)
                        | CSL_FMKT(IDMA_IDMA1_COUNT_INT, INT)  
                        | CSL_FMK(IDMA_IDMA1_COUNT_COUNT,DATA_COUNT_BYTES);
      
  // Wait while the transfer is active
  while(CSL_FEXT(idmaRegs->IDMA1_STAT,IDMA_IDMA1_STAT_ACTV) == 1);
    
  _restore_interrupts(restore_value);        // Restore interrupt state
}

int test_idma(void)
{
  int index;

  printf("\nTesting IDMA transfer\n");
	
  // Compare source and destination buffers
  for(index=0;index<DATA_CNT;index++) 
     {
      if (src[index] != dst[index]) 
         {
           printf("\nIDMA data copy test:  FAILED\n");
           return(1);                     
         }
     }
	
  printf("\nIDMA data copy test:  PASSED\n");
  
  return(0);
}

   
