/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006, 2007
 *
 *   Use of this software is controlled by the terms and conditions found
 *   in the license agreement under which this software has been supplied.
 *  ============================================================================
 */

/** ============================================================================
 *   @brief A simple example to demonstrate CSL 3.x TIMER register layer usage 
     and interrupts in a BIOS application.
 *
 *   @file  Timer_example.c
 *
 *   <b> Example Description </b>
 *   @verbatim
     This example demonstrates using timer register layer csl to configure the 
     timers in a BIOS application.  The example sets up TIMER0 and TIMER1 in 32 
     bit unchained mode.  The TIMER0 1:2 side is used by BIOS and the TIMER0 3:4 
     side is used by the application.  For TIMER1, both sides are used by the 
     application.  All the timer period values have been set to the default BIOS 
     timer period value.  The example uses a task to call the test function 
     which enables the interrupts and start all 3 timers running.  Once the 
     timers have been started, the task will sleep for 3 BIOS timer periods to 
     ensure that all 3 timers used by the application have had time to run.  We 
     use 3 for the argument to the TSK_sleep() because the 3:4 sides of the 
     timers run through a pre-scaler which is slowing the increment by a divide 
     by 2.  When the task wakes,  it calls SYS_exit() which calls the exit 
     handler.  The exit handler will check that all 3 interrupts where generated 
     by the timers used by the application and indicate a pass/fail status.     
     @endverbatim
 *      
 *
 *   <b> Procedure to run the example </b>
 *   @verbatim
     1. Configure the CCS setup to work with the emulator being used
     2. Please refer CCS manual for setup configuration and loading 
        proper GEL file
     3. Launch CCS window
     4. Open project Timer_example.pjt
     5. Build the project and load the .out file of the project.
     6. Open the bios message log.
     @endverbatim
 *
 * ============================================================================
 **/

#include <std.h>
#include <tsk.h>
#include <c64.h>
#include <stdlib.h>
#include <csl_types.h>
#include <soc.h>
#include <cslr_tmr.h>
#include <cslr_psc.h>
#include <cslr_sys.h>
#include "timer_interruptcfg.h"

const unsigned int TIMER_PERIOD = 0x00006978;  // Default BIOS timer period 
                                               // setting for dm648

static void device_init(void);
static void setup_TIMER0(void);
static void setup_TIMER1(void);
void myExit(int);
void test_TIMERS(void);
void TIMER0_TINT34_isr();
void TIMER1_TINT12_isr();
void TIMER1_TINT34_isr();

static int TIMER0_TINT34_status = 1;          // Pass/Fail flag
static int TIMER1_TINT12_status = 1;          // Pass/Fail flag
static int TIMER1_TINT34_status = 1;          // Pass/Fail flag
static int test_status = 1;                   // Test status

CSL_TmrRegsOvly tmr0Regs = (CSL_TmrRegsOvly)CSL_TMR_0_REGS;
CSL_TmrRegsOvly tmr1Regs = (CSL_TmrRegsOvly)CSL_TMR_1_REGS;

extern const unsigned int TIMER_PERIOD;

void main (void) 
{
  // Enable TIMER0 and TIMER1 in mux and psc
  device_init();                  
  
  // Configure TIMER0, BIOS using 1:2 side 
  // Upper half to be used by application
  setup_TIMER0();   
  
  // Configure TIMER1 as dual 32 bit unchained timers for use by the application
  setup_TIMER1();                   

}  
 

void setup_TIMER0 (void) 
{
  // Clear TIM12 register                 
  CSL_FINST(tmr0Regs->TIM12,TMR_TIM12_TIM12,RESETVAL);
  
  // Clear TIM34 register                 
  CSL_FINST(tmr0Regs->TIM34,TMR_TIM34_TIM34,RESETVAL);
  
  // Select 32 bit unchained mode 
  // Take the timer out of reset and set the pre-scalar count for 3:4
  tmr0Regs->TGCR = CSL_FMKT(TMR_TGCR_TIMMODE,32BIT_UNCHAIN)
                 | CSL_FMKT(TMR_TGCR_TIM12RS,NO_RESET)
                 | CSL_FMKT(TMR_TGCR_TIM34RS,NO_RESET)
                 | CSL_FMK(TMR_TGCR_PSC34,0x1);
                      
  // Set timer0 PRD3:4
  CSL_FINS(tmr0Regs->PRD34,TMR_PRD34_PRD34,TIMER_PERIOD);

}


void setup_TIMER1 (void) 
{
  // Clear TIM12 register                 
  CSL_FINST(tmr1Regs->TIM12,TMR_TIM12_TIM12,RESETVAL);
  
   // Clear TIM34 register                 
  CSL_FINST(tmr1Regs->TIM34,TMR_TIM34_TIM34,RESETVAL);

  // Select 32 bit unchained mode 
  // Take the timer out of reset and set the pre-scalar count for 3:4
  tmr1Regs->TGCR = CSL_FMKT(TMR_TGCR_TIMMODE,32BIT_UNCHAIN)
                 | CSL_FMKT(TMR_TGCR_TIM12RS,NO_RESET)
                 | CSL_FMKT(TMR_TGCR_TIM34RS,NO_RESET)
                 | CSL_FMK(TMR_TGCR_PSC34,0x1);
                      
  // Set timer1 PRD1:2                 
  CSL_FINS(tmr1Regs->PRD12,TMR_PRD12_PRD12,TIMER_PERIOD);
  
  // Set timer1 PRD3:4
  CSL_FINS(tmr1Regs->PRD34,TMR_PRD34_PRD34,TIMER_PERIOD);

}

void test_TIMERS (void)                      
{
  
  LOG_printf(&trace, "Testing TIMER0 and TIMER1 in unchained 32 bit mode.");
  LOG_printf(&trace, "BIOS is using TIMER0 1:2 side.");
  LOG_printf(&trace, "Starting TIMER0 3:4 side and TIMER1 1:2 and 3:4 sides.");
  
  // Enable CPU interrupts 5,6, and 7
  C64_enableIER(C64_EINT5);   
  C64_enableIER(C64_EINT6);
  C64_enableIER(C64_EINT7);
  
   // Enable TIMER0 3:4 side, continuous mode
  CSL_FINST(tmr0Regs->TCR,TMR_TCR_ENAMODE34,EN_CONT); 
    
  // Enable TIMER1 1:2 side, continuous mode
  CSL_FINST(tmr1Regs->TCR,TMR_TCR_ENAMODE12,EN_CONT);  
  
  // Enable TIMER1 3:4 side, continuous mode
  CSL_FINST(tmr1Regs->TCR,TMR_TCR_ENAMODE34,EN_CONT);  
  
  // Sleep for 3 system clock PRD's to ensure other timers have ran.
  // Since 3:4 sides are running at a divide by 2 PRD, we need to
  // sleep at least 2 PRDs.  We add one since the BIOS clock as been
  // running. 
  TSK_sleep(3);
 
  // Check if interrupts occurred
  test_status=TIMER0_TINT34_status|TIMER1_TINT12_status|TIMER1_TINT34_status;
  
  // Call exit function 
  SYS_exit(test_status); 
}

void TIMER0_TINT34_isr()
{
  // Disable TIMER0 3:4 side
  CSL_FINST(tmr0Regs->TCR,TMR_TCR_ENAMODE34,DISABLE); 

  LOG_printf(&trace, "Timer0 TINT34 side interrupt occurred.");
  
  // Set flag to 0 indicating ISR occurred
  TIMER0_TINT34_status=0;
}


void TIMER1_TINT12_isr()
{
  // Disable TIMER1 1:2 side
  CSL_FINST(tmr1Regs->TCR,TMR_TCR_ENAMODE12,DISABLE); 

  LOG_printf(&trace, "Timer1 TINT12 side interrupt occurred.");
  
  // Set flag to 0 indicating ISR occurred
  TIMER1_TINT12_status=0; 
}

void TIMER1_TINT34_isr()
{
  // Disable TIMER1 3:4 side
  CSL_FINST(tmr1Regs->TCR,TMR_TCR_ENAMODE34,DISABLE); 

  LOG_printf(&trace, "Timer1 TINT34 side interrupt occurred.");
  
  // Set flag to 0 indicating ISR occurred
  TIMER1_TINT34_status=0; 
}

void myExit(int test_result)
{
  // Disable TIMER0 1:2 side
  CSL_FINST(tmr0Regs->TCR,TMR_TCR_ENAMODE12,DISABLE); 

  // Disable TIMER0 3:4 side
  CSL_FINST(tmr0Regs->TCR,TMR_TCR_ENAMODE34,DISABLE); 
    
  // Disable TIMER1 1:2 side
  CSL_FINST(tmr1Regs->TCR,TMR_TCR_ENAMODE12,DISABLE);  
  
  // Disable TIMER1 3:4 side
  CSL_FINST(tmr1Regs->TCR,TMR_TCR_ENAMODE34,DISABLE);  
  
  // Turn off all interrupts
  HWI_disable();                         
  
  // Display test status
  if(test_result==0)
     LOG_printf(&trace, "Timer0 and Timer1 interrupt test: PASSED.");
  else
     LOG_printf(&trace, "Timer0 and Timer1 interrupt test: FAILED.");
  
  exit(0);
}


void device_init(void) 
{
  
  CSL_SysRegsOvly sysRegs = (CSL_SysRegsOvly)CSL_SYS_0_REGS;
  CSL_PscRegsOvly pscRegs = (CSL_PscRegsOvly)CSL_PSC_0_REGS;

  // enable TIMER0 and TIMER1 in the PINMUX1 register
  sysRegs->PINMUX1 |= CSL_FMK(SYS_PINMUX1_TIM0BK,1)  // TIM0BK- Timer0 enabled
                      | CSL_FMK(SYS_PINMUX1_TIM1BK,1); // TIM1BK- Timer1 enabled 
  
  // ensure TIMER0 and TIMER1 are powered
  sysRegs->VDD3P3V_PWDN |= CSL_FMKT(SYS_VDD3P3V_PWDN_TIMER0,POWERUP)
                          |CSL_FMKT(SYS_VDD3P3V_PWDN_TIMER1,POWERUP);

  // deassert TIMER0 local PSC reset and set NEXT state to ENABLE
  pscRegs->MDCTL[CSL_PSC_TIMER0] = CSL_FMKT(PSC_MDCTL_NEXT, ENABLE) | 
                                   CSL_FMKT(PSC_MDCTL_LRST, DEASSERT);
    
  pscRegs->PTCMD = CSL_FMKT(PSC_PTCMD_GO0, SET);
    
  while (CSL_FEXT(pscRegs->MDSTAT[CSL_PSC_TIMER0], PSC_MDSTAT_STATE)
        != CSL_PSC_MDSTAT_STATE_ENABLE);   

  // deassert TIMER1 local PSC reset and set NEXT state to ENABLE
  pscRegs->MDCTL[CSL_PSC_TIMER1] = CSL_FMKT(PSC_MDCTL_NEXT, ENABLE) | 
                                   CSL_FMKT(PSC_MDCTL_LRST, DEASSERT);
    
  pscRegs->PTCMD = CSL_FMKT(PSC_PTCMD_GO0, SET);
    
  while (CSL_FEXT(pscRegs->MDSTAT[CSL_PSC_TIMER1], PSC_MDSTAT_STATE)
        != CSL_PSC_MDSTAT_STATE_ENABLE);   
}
 
