/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006, 2007
 *
 *   Use of this software is controlled by the terms and conditions found
 *   in the license agreement under which this software has been supplied.
 *  ============================================================================
 */

/** ============================================================================
 *
 *   @file  Ddr2_example.c
 *
 *   @path $(CSL_EXAMPLE_PATH)\ddr2\src
 *
 *   @desc  Read write example of DDR2 in Normal Mode
 *
 *  ============================================================================
 *   @n Target Platform: EVM
 *  ============================================================================
 *   @n <b> Example Description </b>
 *   @n This example, 
 *        1. Enables the DDR2 module
 *        2 Sets up the hardware to default values and Normal Mode.
 *        3 Writes the Invalid values into DDR2 SDRAM area to over write the 
 *           previous values.
 *        4 Writes valid data
 *        5 Does the data comparision to ensure the written data is proper or 
 *           not and 
 *        6 Displays the messages based on step 5
 *
 * =============================================================================
 *      
 *   <b> Procedure to run the example </b>
 *   @verbatim
 *      1. Configure the CCS setup to work with the emulator being used
 *      2. Please refer CCS manual for setup configuration and loading 
 *         proper GEL file
 *      3. Launch CCS window
 *      4. Open project Ddr2_example.pjt
 *      5. Build the project and load the .out file of the project.
 *          
 *   @endverbatim
 * ============================================================================
 */

#include <cslr_ddr2.h>
#include <cslr_sys.h>
#include <stdio.h>
#include <soc.h> 
#include <cslr_psc.h>

/** Result - Passed */
#define DATA_MATCH_SUCCESS    1
/** Result - Failed */
#define DATA_MATCH_FAIL       0

/** Data count(number write/readbacks) */
#define DATA_CNT    10

/* Forwards declarations */
static void device_init(void);
static void init_ddr2(void);
static int test_ddr2(void);

/* Pointer to register overlay structure for DDR2 */    
CSL_Ddr2RegsOvly ddr2Regs = (CSL_Ddr2RegsOvly)CSL_DDR2_0_REGS;         


/*
 * =============================================================================
 *   @func   main
 *
 *   @desc
 *     This is the main routine for the file.  
 *    
 * =============================================================================
 */
int main(void)
{   
    /* Enable the ddr2 in power sleep controller */
    device_init();

    /* Set up read_write functionality of DDR2 */
    init_ddr2();   
	
	/* Test DDR2 read write functionality, returns 0 for pass and 1 for fail */
    return(test_ddr2()); 
}

/*
 * =============================================================================
 *   @func   init_ddr2
 *
 *   @desc
 *       This function code showing how to use CSL macros, setting up DDR2 
 *
 * =============================================================================
 */
void init_ddr2(void)
{ 
    Uint32              mask;

    /* Refresh rate*/
    CSL_FINS(ddr2Regs->SDRFC, DDR2_SDRFC_REFRESH_RATE, CSL_DDR2_SDRFC_REFRESH_RATE_RESETVAL);

    /* Writing the ddr2 sdram Settings in SDRAM Config register */
    mask = ~( 
              (CSL_DDR2_SDCFG_TIMUNLOCK_MASK) |
              (CSL_DDR2_SDCFG_CL_MASK) |
              (CSL_DDR2_SDCFG_IBANK_MASK) |
              (CSL_DDR2_SDCFG_PAGESIZE_MASK)|
              (CSL_DDR2_SDCFG_NM_MASK));
               
    /* Set the TIMUNLOCK bit : A write to this bit will cause the DDR2 Memory 
                               Controller to start the SDRAM initialization sequence. */
    /* Set the CAS latency of 5 */
    /* Set the Internal SDRAM bank to 4 */
    /* Set the DDR2 data bus width to 32 bit */
    /* Set the page size to 256-word page */
        
    ddr2Regs->SDCFG = (ddr2Regs->SDCFG & mask ) |
                          (CSL_FMK(DDR2_SDCFG_TIMUNLOCK,
                                   CSL_DDR2_SDCFG_TIMUNLOCK_SET)) |
                          (CSL_FMKT(DDR2_SDCFG_CL, FIVE)) |
                          (CSL_FMKT(DDR2_SDCFG_IBANK, FOUR)) |
                          (CSL_FMKT(DDR2_SDCFG_NM, 32BIT )) |
                          (CSL_FMKT(DDR2_SDCFG_PAGESIZE, 256W_PAGE ));
                          
    /* Unlock the BOOT_UNLOCK bit */
    CSL_FINS(ddr2Regs->SDCFG, DDR2_SDCFG_BOOT_UNLOCK, \
                                      CSL_DDR2_SDCFG_BOOT_UNLOCK_UNLOCKED);
    /* Set the DDR2 SDRAM drive strength to Normal */
    CSL_FINS(ddr2Regs->SDCFG, DDR2_SDCFG_DDR_DRIVE, \
                                      CSL_DDR2_SDCFG_DDR_DRIVE_NORMAL);
    /* Lock the BOOT_UNLOCK bit */                                  
    CSL_FINS(ddr2Regs->SDCFG, DDR2_SDCFG_BOOT_UNLOCK, \
                                      CSL_DDR2_SDCFG_BOOT_UNLOCK_LOCKED); 
                                            
   /* Locking the timing_unlock to prevent further changes */
    CSL_FINS(ddr2Regs->SDCFG, DDR2_SDCFG_TIMUNLOCK, 
                                      CSL_DDR2_SDCFG_TIMUNLOCK_CLEAR);
}

/*
 * =============================================================================
 *   @func   test_ddr2
 *
 *   @desc
 *       example code showing DDR2 read write functionality test
 *
 * =============================================================================
 */
int test_ddr2(void)
{
    volatile Uint32     result, index ;
    Uint32              tempData; 

	/* Pointer that points to SDRAM start area */
    Uint32 *pDdr2Data = (Uint32 *)CSL_DDR2_SDRAM_ADDR ;

    printf("\n Testing DDR2 read write functionality\n");

	/* Write 'invalid' values into DDR2 SDRAM area. This is to overwrite the 
    * previous valid values
    */
    tempData = 0xdeadbeef;
    for (index = 0; index < DATA_CNT; index++) {
        pDdr2Data[index] =  tempData;
    }

    /* Write **valid** values into SDRAM area. */
    tempData = 0x56780000;
    for (index = 0; index < DATA_CNT; index++) {
        pDdr2Data[index] = tempData + index ;
    }

    /* Verify that the data was indeed written */
    result = DATA_MATCH_SUCCESS;
    for (index = 0; index < DATA_CNT; index++) {
        if (pDdr2Data[index] != (tempData + index)) {
            result = DATA_MATCH_FAIL;                     
            break ;
        }
    }

   /* Print the appropriate message based on result */
    if (result == DATA_MATCH_SUCCESS) {     
        printf("\nWrite to and Read from DDR2 SDRAM is Successful\n"); 
		printf("\n DDR2 read write example passed\n");
		return(0);
    }
    else {      
        printf("\nWrite to and Read from DDR2 SDRAM is NOT Successful\n"); 
		printf("\n DDR2 read write example failed\n");
		return(1);
    }
}

/*
 * ============================================================================
 *   @func   device_init
 *
 *   @desc
 *     This function enables the DDR2 in the power and sleep controller.
 *
 *  @arg  
 *      None
 *
 *  @return
 *      None
 * ============================================================================
*/
void device_init(void) 
{
  
  CSL_PscRegsOvly pscRegs = (CSL_PscRegsOvly)CSL_PSC_0_REGS;

  // deassert DDR2 local PSC reset and set NEXT state to ENABLE
  pscRegs->MDCTL[CSL_PSC_DDR2] = CSL_FMKT(PSC_MDCTL_NEXT, ENABLE) |
                                   CSL_FMKT(PSC_MDCTL_LRST, DEASSERT);
  //move DDR2 PSC to Next state
  pscRegs->PTCMD = CSL_FMKT(PSC_PTCMD_GO0, SET);
 
  //wait for transition
  while (CSL_FEXT( pscRegs->PTSTAT, PSC_PTSTAT_GOSTAT0)
          == CSL_PSC_PTSTAT_GOSTAT0_IN_PROGRESS);
}

