/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006, 2007
 *
 *   Use of this software is controlled by the terms and conditions found
 *   in the license agreement under which this software has been supplied.
 *  ============================================================================
 */

/** ============================================================================
 *   @brief A simple example to demonstrate CSL 3.x GPIO register layer usage 
     and interrupts in a BIOS application.
 *
 *   @file  Gpio_example.c
 *
 *   <b> Example Description </b>
 *   @verbatim
     This example sets up GPIO pin as an output and a GPIO pin as an input using 
     the register layer csl.  The example is bios based and will toggle GPIO 
     output pin to cause a rising edge transition to occur on the GPIO input 
     pin.  The rising edge transition detected by GPIO input pin will cause an 
     interrupt to occur.  The ISR will set a pass/fail flag which is passed by 
     the SYS_exit api to the exit function.  Open the bios message log to view 
     the test results.
     NOTE:
     The GPIO output pin and GPIO input pin numbers are set using pre-defined 
     symbols in the pjt settings.  The example requires a jumper to connect the 
     output GPIO pin to the input GPIO pin.  See the readme.txt file in the 
     project directory for the jumper requirement for the specific target.
     @endverbatim
 *      
 *
 *   <b> Procedure to run the example </b>
 *   @verbatim
     1. Before powering up the evm, place a jumper between GPIO output pin and
        GPIO input pin.
     2. Configure the CCS setup to work with the emulator being used
     3. Please refer CCS manual for setup configuration and loading 
        proper GEL file
     4. Launch CCS window
     5. Open project Gpio_example.pjt
     6. Build the project and load the .out file of the project.
     7. Open the bios message log.
     @endverbatim
 *
 * ============================================================================
 **/

#include <log.h>
#include <tsk.h>
#include <c64.h>
#include <stdlib.h>
#include <soc.h>
#include <cslr_gpio.h>
#include <cslr_psc.h>
#include <cslr_sys.h>
#include <hwi.h>
#include "gpio_interruptcfg.h"

#define LOW 0x0
#define HIGH 0x1

static void device_init(void);
static void setup_GPIO();
void myExit(int);
void test_GPIO(void);
void GPIO_input_isr();

static int status=1;                         // Pass/Fail flag

CSL_GpioRegsOvly gpioRegs = (CSL_GpioRegsOvly)CSL_GPIO_0_REGS;

void main (void) 
{
  // Enable GPIO in the power and sleep controller
  device_init();                  
  
  // Configure output and input GPIO, enable rising edge detection
  setup_GPIO();                 
  
}

void setup_GPIO() 
{
  // The setup code below uses the raw register layer macros to allow the pin 
  // numbers to be passed in via pre-defined symbols in the pjt settings.  If 
  // variable pin settings are not required, the token macros should be used.
  
  // Configure GPIO(OUTPUT_PIN) as an output and GPIO(INPUT_PIN) as an input
  gpioRegs->DIR01 = CSL_FMKR(OUTPUT_PIN,OUTPUT_PIN,LOW)
                   | CSL_FMKR(INPUT_PIN,INPUT_PIN,HIGH);
  
  // Set Data low in SET_DATA register for GPIO(OUTPUT_PIN)
  gpioRegs->CLR_DATA01 = CSL_FMKR(OUTPUT_PIN,OUTPUT_PIN,HIGH); 
  
  // Enable GPIO Bank 0 interrupts
  CSL_FINST(gpioRegs->BINTEN,GPIO_BINTEN_EN0,ENABLE);
 
  // Configure GPIO(INPUT_PIN) to generate interrupt on rising edge
  CSL_FINSR(gpioRegs->SET_RIS_TRIG01,INPUT_PIN,INPUT_PIN,HIGH);
 
}

void test_GPIO(void)                      
{
  // Enable cpu interrupt 4 in IER
  C64_enableIER(C64_EINT4);   
  
  LOG_printf(&trace, "GPIO Interrupt Test.");
  
  LOG_printf(&trace, "Set output GPIO high.");
  
  // Set GPIO(OUTPUT_PIN) high
  gpioRegs->SET_DATA01 = CSL_FMKR(OUTPUT_PIN,OUTPUT_PIN,HIGH);  
  
  // Wait for GPIO(OUTPUT_PIN) to go high
  while(CSL_FEXTR(gpioRegs->OUT_DATA01,OUTPUT_PIN,OUTPUT_PIN)!=1);
 
  // Call exit function
  SYS_exit(status); 
}

void GPIO_input_isr()
{
  LOG_printf(&trace, "GPIO interrupt occured.");
  
  // Set flag to 0 indicating ISR occurred
  status=0; 
}

void myExit(int test_result)
{
  // Turn off interrupts
  HWI_disable();                         
    
  if(test_result==0)
    LOG_printf(&trace, "GPIO interrupt test: PASSED.");
  else
    LOG_printf(&trace, "GPIO interrupt test: FAILED.  Check jumper.");
  
  exit(0);
}

void device_init(void) 
{
  CSL_SysRegsOvly sysRegs = (CSL_SysRegsOvly)CSL_SYS_0_REGS;
  CSL_PscRegsOvly pscRegs = (CSL_PscRegsOvly)CSL_PSC_0_REGS;
 
  // mux between Timer 0/1 and GPIO[8:11], enable GPIO[8:11]
  sysRegs->PINMUX = CSL_FMKT(SYS_PINMUX_TIMER_EN,TIM0_TIM1_DISABLE);
   
  // deassert GPIO local PSC reset and set NEXT state to ENABLE
  pscRegs->MDCTL[CSL_PSC_GPIO] = CSL_FMKT( PSC_MDCTL_NEXT, ENABLE ) 
                               | CSL_FMKT( PSC_MDCTL_LRST, DEASSERT );
  // move GPIO PSC to Next state
  pscRegs->PTCMD = CSL_FMKT(  PSC_PTCMD_GO0, SET );

  // wait for transition
  while ( CSL_FEXT( pscRegs->MDSTAT[CSL_PSC_GPIO], PSC_MDSTAT_STATE )
          != CSL_PSC_MDSTAT_STATE_ENABLE );
}

 
