/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006, 2007
 *
 *   Use of this software is controlled by the terms and conditions found
 *   in the license agreement under which this software has been supplied.
 *  ============================================================================
 */

/** ============================================================================
 *  @brief  A simple example to demonstrate CSL 3.x register layer 
 *   usage and interrupts.  
 *
 *  @file  interrupt_example.c
 *
 *  <b> Example description </b> 
 *  @verbatim
    The interrupt example is a non-bios example that uses TIMER0 to generate cpu 
    INT4.  TIMER0 is configured in 32bit unchained once mode and its event is 
    mapped CPU interrupt 4.  The test enables TIMER0 and verifies that the 
    interrupt service routine is executed when the period value is reached.  
    A pass/fail status is returned to main.
    @endverbatim
 *   
 *   
 *  @n <b> Procedure to run the example </b>
 *  @verbatim
    1. Configure the CCS setup to work with the emulator being used.
    2. Please refer CCS manual for setup configuration and loading 
       proper GEL file.
    3. Launch CCS.
    4. Open project Intc_example.pjt.
    5. Build the project and load the .out file for the project.
    @endverbatim
 *
 *  ============================================================================
 **/

#include <stdio.h>
#include <soc.h>
#include <c6x.h>
#include <cslr_intc.h>
#include <cslr_tmr.h>
#include <cslr_psc.h>
#include <cslr_sys.h>


static void setup(void);   
static int test_interrupt(void); 
extern void intcVectorTable(void);  
static void device_init(void);

const unsigned int TIMER_PERIOD = 0x00006978;  // timer period setting 
int intFlag=1;                                 // interrupt flag 

CSL_TmrRegsOvly tmr0Regs = (CSL_TmrRegsOvly)CSL_TMR_0_REGS;
CSL_IntcRegsOvly intcRegs = (CSL_IntcRegsOvly)CSL_INTC_0_REGS;

int main (void) 
{
  // enable TIMER0 in MUX and PSC
  device_init();     
  
  // setup TIMER0 and interrupt  
  setup();
  
  // test interrupt, returns 0 for pass and 1 for fail 
  return(test_interrupt());
}

void setup(void)
{
  // clear TIM12 register                 
  CSL_FINST(tmr0Regs->TIM12,TMR_TIM12_TIM12,RESETVAL);
  
  // select 32 bit unchained mode and take the timer out of reset
  tmr0Regs->TGCR = CSL_FMKT(TMR_TGCR_TIMMODE,32BIT_UNCHAIN)
                 | CSL_FMKT(TMR_TGCR_TIM12RS,NO_RESET);
                               
  // set TIMER0 PRD1:2
  CSL_FINS(tmr0Regs->PRD12,TMR_PRD12_PRD12,TIMER_PERIOD);
  
  // map TIMER0 event to cpu int4
  CSL_FINS(intcRegs->INTMUX1, INTC_INTMUX1_INTSEL4,TIMER0_EVENT); 
  
  // set ISTP to point to the vector table address
  ISTP = (unsigned int)intcVectorTable;
  
  // clear all interrupts, bits 4 thru 15
  ICR = 0xFFF0; 

  // enable the bits for non maskable interrupt and CPUINT4 */
  IER |= 0x12; 
   
  // enable interrupts, set GIE bit 
  _enable_interrupts();
}

int test_interrupt(void)
{
  printf("\nTesting CPU Interrupt 4 using TIMER0\n");

  // enable TIMER0 1:2 side, once mode
  CSL_FINST(tmr0Regs->TCR,TMR_TCR_ENAMODE12,EN_ONCE);
    
  printf("\nStarting TIMER0\n");
  
  // wait for TIMER0 to run to the period value and reset count
  while(CSL_FEXT(tmr0Regs->TIM12,TMR_TIM12_TIM12)!=0x0);
  
  if(intFlag!=0)
  {
    printf("Error\n");
    printf("\nInterrupt test:  FAILED\n");
  } 
  else
    printf("\nInterrupt test:  PASSED\n"); 
 
  return(intFlag);
}

interrupt void interrupt4(void)
{
  // disable TIMER0 1:2 side
  CSL_FINST(tmr0Regs->TCR,TMR_TCR_ENAMODE12,DISABLE); 
    
  // set the interrupt flag  
  intFlag=0;
}


void device_init(void) 
{
  CSL_SysRegsOvly sysRegs = (CSL_SysRegsOvly)CSL_SYS_0_REGS;
  CSL_PscRegsOvly pscRegs = (CSL_PscRegsOvly)CSL_PSC_0_REGS;

  // mux between Timer 0/1 and GPIO[8:11], enable Timer0/1
  sysRegs->PINMUX = CSL_FMKT(SYS_PINMUX_TIMER_EN,TIM0_TIM1_ENABLE);      

  // deassert TIMER0 local PSC reset and set NEXT state to ENABLE
  pscRegs->MDCTL[CSL_PSC_TIMER0] = CSL_FMKT(PSC_MDCTL_NEXT, ENABLE) | 
                                   CSL_FMKT(PSC_MDCTL_LRST, DEASSERT);
    
  pscRegs->PTCMD = CSL_FMKT(PSC_PTCMD_GO0, SET);
    
  while (CSL_FEXT(pscRegs->MDSTAT[CSL_PSC_TIMER0], PSC_MDSTAT_STATE)
        != CSL_PSC_MDSTAT_STATE_ENABLE);   
}
 
