/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006, 2007
 *
 *   Use of this software is controlled by the terms and conditions found
 *   in the license agreement under which this software has been supplied.
 *  ============================================================================
 */

/** ============================================================================
 *  @brief  A simple example to demonstrate CSL 3.x Cache register layer usage.  
 *
 *  @file  cache_config_example.c
 *
 *  <b> Example description </b> 
 *  @verbatim
    The cache configuration example configures L1P, L1D, and L2 cache using the 
    register layer csl.  The configuration is then verified using the BCACHE 
    module api's.  A pass\fail status will be returned from main.
    
    To use BCACHE: 
    - BIOS application : simply #include the bcache.h header file. BIOS 
    automatically selects the appropriate library based on the chip 
    configuration.
    - non-BIOS application : simply #include the bcache.h header file. Link in 
    the appropriate bios library and explicitly place the .bios section in your
    linker command file. Unfortunately there is no single bcache.a64P library
    hence the easiest way to determine which library to link in is either (a)
    look at the library these examples link in (b) see which library the
    BIOS examples link in. Note that BCACHE does *not* link in any other BIOS 
    modules so you do *not* need a BIOS TCF configuration. Think of BCACHE as 
    a standalone module that just happens to be distributed with DSP/BIOS.
    Please reference the BIOS API Reference Guide for a complete list of 
    available APIs.
        
    The code below gives an example of using the BCACHE module read APIs
    in a non-BIOS application
    @endverbatim
 *   
 *   
 *  @n <b> Procedure to run the example </b>
 *  @verbatim
    1. Configure the CCS setup to work with the emulator being used.
    2. Please refer CCS manual for setup configuration and loading 
       proper GEL file.
    3. Launch CCS.
    4. Open project Cache_example.pjt.
    5. Build the project and load the .out file for the project.
    @endverbatim
 *
 *  ============================================================================
 **/

#include <stdio.h>
#include <std.h>
#include <bcache.h>
#include <cslr_cache.h>
#include <soc.h>

void config_cache(void);
int test_config(void);
int testResult=0;                              // Test status
  
int main(void)
{
  // Configure the cache using register layer      
  config_cache();
 
  // Verify configuration using the BCACHE module api's
  return(test_config());
}  

void config_cache(void)
{
  CSL_CacheRegsOvly cacheRegs = (CSL_CacheRegsOvly)CSL_CACHE_0_REGS;
  volatile unsigned int stall;
    
  // The below writes to the CFG registers are followed by a dummy read.  Mode
  // switches require that a read is performed immediately after the write.  The 
  // read stalls the cpu to ensure the mode change completes.
  
  // Set L1P size to 32K 
  CSL_FINST(cacheRegs->L1PCFG,CACHE_L1PCFG_MODE,32K);
  stall = cacheRegs->L1PCFG;
  
  // Set L1D size to 32K 
  CSL_FINST(cacheRegs->L1DCFG,CACHE_L1DCFG_MODE,32K);
  stall = cacheRegs->L1DCFG;
  
  // Set L2 size to 64k and normal opperation
  cacheRegs->L2CFG = CSL_FMKT(CACHE_L2CFG_MODE,64K)
                   | CSL_FMKT(CACHE_L2CFG_L2CC,NORMAL);
  stall = cacheRegs->L2CFG;
  
  // The MAR registers set the cachability of memory spaces external to the 
  // megamodule.  Below is an example of turning on cachability for two ranges.
  // Reference spru187 for a complete list the MAR ranges.
  
  // Set MAR[16] range 0x10000000-0x10FFFFFF as cacheable
  CSL_FINST(cacheRegs->MAR[16],CACHE_MAR_PC,CACHEABLE);

  // Set MAR[17] range 0x11000000-0x11FFFFFF as cacheable
  CSL_FINST(cacheRegs->MAR[17],CACHE_MAR_PC,CACHEABLE);

}

int test_config(void)
{
  BCACHE_Size checkSize;
  BCACHE_Mode checkMode;
  BCACHE_Mar checkMar16;
  BCACHE_Mar checkMar17;
  Ptr baseMar16;
  Ptr baseMar17;

  baseMar16=(Ptr)0x10000000;                // MAR16 base address
  baseMar17=(Ptr)0x11000000;                // MAR17 base address
 
  // The BCACHE module read API's are used to verify the previous configuration 
  // that was performed using the register layer.
    
  // Read L1P, L1D, and L2 cache size settings
  BCACHE_getSize(&checkSize);
  
  // Read L2 mode setting
  checkMode = BCACHE_getMode(BCACHE_L2);

  // Read MAR16 and MAR17 setting
  checkMar16 = BCACHE_getMar(baseMar16);
  checkMar17 = BCACHE_getMar(baseMar17);
  
  
  printf("\nTest cache configuraton\n");

  // Verify cache size settings for L1P, L1D, and L2
  if((checkSize.l1psize==BCACHE_L1_32K)&&(checkSize.l1dsize==BCACHE_L1_32K)&&
     (checkSize.l2size==BCACHE_L2_64K))
     {
       printf("\nL1P size = 32K\n");
       printf("L1D size = 32K\n");
       printf("L2 size = 64K\n");
     }
  else
     {
      printf("\nError setting cache sizes\n");
      testResult=1;
     }
  
  // Verify mode setting for L2
  if(checkMode==BCACHE_NORMAL)
     printf("\nL2 mode set to NORMAL\n");
  else
   	 {
      printf("\nError setting mode\n");
      testResult=1;
     }
  
  // Verify MAR16 and MAR17 settings
  if((checkMar16==BCACHE_MAR_ENABLE)&&(checkMar17==BCACHE_MAR_ENABLE))
     printf("\nMAR16 and MAR17 range set to cacheable\n");
  else
     {
      printf("\nError setting MAR range\n");
      testResult=1;
     }

  // Display test status 
  if(testResult==0)
     printf("\nCache configuration test:  PASSED\n");
  else
     printf("\nCache configuration test:  FAILED\n");
  
  return(testResult);
}

