/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006, 2007
 *
 *   Use of this software is controlled by the terms and conditions found
 *   in the license agreement under which this software has been supplied.
 *  ============================================================================
 */

/** ============================================================================
 *  @brief  A simple example to demonstrate CSL 3.x HECC register layer usage.
 *
 *  @file  Hecc_example.c
 *
 *  <b> Example description </b>
 *  @verbatim
    The Hecc example sets up the hecc interface  will transmit  and receive 4
    characters and verify that they are correct in loopback mode and between two
    boards. A pass\fail status will be returned from main.
    @endverbatim
 *
 *
 *  @n <b> Procedure to run the example </b>
 *  @verbatim

LOOPBACKMODE:

    1. Configure the CCS setup to work with the emulator being used.
    2. Please refer CCS manual for setup configuration and loading
       proper GEL file.
    3. Launch CCS.
    4. Open project Hecc_example.pjt.
    5. Uncomment the "#define LOOPBACK" statement in the Hecc_example.c to
       enable the loopback mode.
    6. Build the project and load the .out file for the project.

NONLOOPBACKMODE (BOARD TO BOARD tx/rx test) :


HW CONFIGURATION:

    To test the data transfer between two boards using CAN, CAN connectors(ports)
    should be connested as --



   CAN PORT OF BOARD1                             CAN PORT OF BOARD2

     ===========                                 ===========
    #          #                                #          #
    #   B_CANH # <<==========================>> # B_CANH   #
    #          #                                #          #
    #   B_CANL # <<==========================>> # B_CANL   #
    #          #                                #          #
    #     GND  # <<==========================>> # GND      #
    #          #                                #          #
    #          #                                #          #
     ==========                                  ==========


SWCONFIGURATION:

  BOARD1:

    1. Configure the CCS setup to work with the emulator being used.
    2. Please refer CCS manual for setup configuration and loading
       proper GEL file.
    3. Launch CCS.
    4. Open project Hecc_example.pjt.
    5. Uncomment the "#define MASTER" statement in the Hecc_example.c to
       enable the loopback mode.
    6. Build the project and load the .out file for the project.

  BOARD2:

    1. Configure the CCS setup to work with the emulator being used.
    2. Please refer CCS manual for setup configuration and loading
       proper GEL file.
    3. Launch CCS.
    4. Open project Hecc_example.pjt.
    5. Uncomment the "#define MASTER" statement in the Hecc_example.c to
       enable the loopback mode.
    6. Build the project and load the .out file for the project.

    @endverbatim
 *
 *  ============================================================================
 **/

#include <csl_types.h>
#include <soc.h>
#include <cslr_hecc.h>
#include <cslr_psc.h>
#include <cslr_sys.h>
#include <stdio.h>


//#define LOOPBACK
#define LOOPCOUNT                              10
#define NUMOFBYTESPERTRANSFER                   4
#define MASTER



#define CSL_HECC_SYNC_FALL                      (0)
#define CSL_HECC_ONE_POINT_SAMPLE               (0)

#define CSL_HECC_CANMD_MD_TX                    (0x00000000u)
#define CSL_HECC_CANMD_MD_RX                    (0x00000001u)


#define CSL_HECC_CANTA_TA_SENT                  (0x00000001u)


#define CSL_HECC_MBX_1                          (0x1)
#define CSL_HECC_MBX_2                          (0x2)
#define CSL_MID_FOR_MBX1                        (0x15AC0000)
#define CSL_MID_FOR_MBX2                        (0x15AC0000)


#define CSL_LOOPBACK_ENABLE                     (1)
#define TESTBYTES_HIGH                          (0x12345678)
#define TESTBYTES_LOW                           (0x87654321)


#define CSL_ENABLE_CAN                          (0x00300000)

static void DeviceInit(void);
static void HECCInit(void);
static int TestHecc(void);
void HeccTransfer(Uint32 Data);
void HeccReceive(Uint32 *Data);


CSL_HeccRegsOvly HeccRegs = (CSL_HeccRegsOvly)CSL_HECC_0_REGS;


int main (void)
{
  DeviceInit();
  HECCInit();
  return(TestHecc());

}


/*
 * =============================================================================
 *   @func  HECCInit
 *
 *   @arg
 *      NONE
 *
 *   @desc
 *      This function initializes the CAN controller for Tx and Rx operation.
 *
 *
 *   @return
 *      NONE
 *
 * =============================================================================
 */

void HECCInit(void)
{

/*  To Transfer, we have to configure the mail box first and then
               Transfer the data

    1. Clear the appropriate TRS[n] bit in the transmission request set register
       (CANTRS) to 0. Writing a 0 to the TRS[n] bit has no effect; instead, set
       the TRR[n] bit in the transmission request reset register(CANTRR) to 1
       and wait until the TRS[n] bit is cleared.
           Clear TRS[1] = 0 (set the TRR[1] bit in CANTRR to 1 and wait
                    until the TRS[1] bit is cleared.) */

        CSL_FINSR(HeccRegs->CANTRR,CSL_HECC_MBX_1,CSL_HECC_MBX_1,TRUE);
        while((CSL_FEXTR(HeccRegs->CANTRS, CSL_HECC_MBX_1,CSL_HECC_MBX_1) !=
                                                CSL_HECC_CANTRS_TRS1_NOP))
           ;

/*  2. Disable the object by clearing the corresponding ME[n] bit in the mailbox
                   enable register (CANME) to 0.
             Clear CANME[1] = 0   */
        CSL_FINSR(HeccRegs->CANME,CSL_HECC_MBX_1,CSL_HECC_MBX_1,FALSE);

/* 3. Load the message identifier register n (MIDn) of the object. Clear the
        AME bit and the AAM bit to 0 for a normal send mailbox. This register is
        usually not modified during operation and can only be modified when the
        mailbox is disabled.
             For example: write MID1 = 15AC 0000h
*/
        HeccRegs->MBX[CSL_HECC_MBX_1].MID = CSL_MID_FOR_MBX1;
/*
    4. Write the data length into the DLC bits in the message control field
       register n (MCFn). The RTR bit is usually cleared to 0. This register is
       usually not modified during operation and can only be modifiedwhen the
       object is disabled.
            For example: MCF1 = 2 */
        HeccRegs->MBX[CSL_HECC_MBX_1].MCF = NUMOFBYTESPERTRANSFER;

/*  5. Configure the mailbox direction as a transmit mailbox by clearing the
        corresponding MD[n] bit in the  mailbox direction register (CANMD) to 0.
             Clear MD[1] = 0
*/
        CSL_FINSR(HeccRegs->CANMD,CSL_HECC_MBX_1,CSL_HECC_MBX_1,
                                                    CSL_HECC_CANMD_MD_TX);

/*  6. Configure the mailbox enable by setting the corresponding ME[n] bit in
             CANME to 1.
             Set ME[1] = 1
*/
        CSL_FINSR(HeccRegs->CANME,CSL_HECC_MBX_1,CSL_HECC_MBX_1,TRUE);
/*
    To configure a mailbox to receive messages, the following steps must be
     performed (in this example, mailbox 2):
    1. Disable mailbox n by clearing the corresponding ME[n] bit in the mailbox
     enable register (CANME) to     0.
     Clear ME[2] = 0
*/
        CSL_FINSR(HeccRegs->CANME,CSL_HECC_MBX_2,CSL_HECC_MBX_2,FALSE);
/*
    2. Write the selected identifier into the corresponding message identifier
        register n (MIDn). The identifier extension (IDE) bit must be configured
        to fit the expected identifier. If the acceptance mask is used, the
        AME bit must be set to 1.
             For example: write MID2 = 4F78 0000h
*/
        HeccRegs->MBX[CSL_HECC_MBX_2].MID = CSL_MID_FOR_MBX2;
/*
    3. For the HECC: If the AME bit is set to 1, the corresponding acceptance
       mask must be programmed.For the SCC: The acceptance masks must be
       programmed during the initialization phase.
*/

/*
    4. Configure the mailbox direction as a receive mailbox by setting the
        corresponding MD[n] bit in the mailbox direction register (CANMD) to
         1. Make sure no other bits in this register are affected by this
                operation.
                     Set MD[3] = 1
*/
        CSL_FINSR(HeccRegs->CANMD,CSL_HECC_MBX_2,CSL_HECC_MBX_2,
                                                        CSL_HECC_CANMD_MD_RX);
/*

    5. If data in the mailbox is to be protected, the overwrite protection
        control register (CANOPC) should be programmed. This protection is
        useful if no message must be lost. If OPC[n] is set, the software has
        to make sure that an additional mailbox (buffer mailbox) is configured
        to store overflow messages.
            Otherwise, messages can be lost without notification.
             Write OPC[3] = 1
*/

//  HeccRegs->CANOPC |= 4;          /* Not required for this example */

/*
    6. Enable mailbox n by setting the corresponding ME[n] bit in CANME. This
    should be done by reading CANME and writing back (CANME |= 0008h) to make
     sure no other flag has changed accidentally.
*/
    CSL_FINSR(HeccRegs->CANME,CSL_HECC_MBX_2,CSL_HECC_MBX_2,TRUE);

#ifdef LOOPBACK
        /* Enable the loopback mode */
    CSL_FINS(HeccRegs->CANMC,HECC_CANMC_STM,CSL_LOOPBACK_ENABLE);
#endif
}


/*
 * =============================================================================
 *   @func  TestHecc
 *
 *   @arg
 *      NONE
 *
 *   @desc
 *      This function Transmits/receives  the data from Tx/Rx mail box and tests
 *      the validity of the data and displays the result of the test.
 *
 *   @return
 *      0 if test fails
 *      1 if test passed
 * =============================================================================
 */

int TestHecc(void)
{


#ifdef LOOPBACK

/*
        To start a transmission (in this example, for object 1):

        1. Write the message data into the mailbox data field. Since the DBO bit
            in the master control register  (CANMC) is cleared to 0, in the
            configuration section, and the message control field register 1
            (MCF1)is set to 2, the data are stored in the 2 MSBytes of the
            message data low register 1 (MDL1).
                 Write MDL1 = xxxx 0000h
*/
        HeccRegs->MBX[CSL_HECC_MBX_1].MDL = TESTBYTES_LOW;

/*      2. Set the corresponding TRS[n] bit in the transmission request set
            register (CANTRS) to 1, to start the transmission of the message.
            The CAN module now handles the complete transmission of the CAN
            message.
*/
        CSL_FINSR(HeccRegs->CANTRS,CSL_HECC_MBX_1,CSL_HECC_MBX_1,TRUE);
/*
    3. Wait until the transmit acknowledge (TA[n]) bit in the transmission
        acknowledge register (CANTA) of the corresponding mailbox is set to 1.
         After a successful transmission, this flag is set by the CAN module.
                 TA[1] = 1

*/
         while((CSL_FEXT(HeccRegs->CANTA, HECC_CANTA_TA1) !=
                                                    CSL_HECC_CANTA_TA_SENT))
           ;

    /* Wait for the messge to arrive */

    while(CSL_FEXT(HeccRegs->CANRMP,HECC_CANRMP_RMP2) !=1)
        ;

    printf("\nReception Successful !\n");

    /* Message is arrived, Read the message and store/display */
    /*
      The CPU may then read the message from the mailbox RAM. Before the CPU
      reads the message from the mailbox,it should first clear the RMP bit
       (RMP[3] = 1).
    */
    if ((HeccRegs->MBX[CSL_HECC_MBX_1].MDL !=  HeccRegs->MBX[CSL_HECC_MBX_2].MDL))
    {
        printf("\n HECC LOOPBACK TEST FAILED ...!!!\n");

        return 0;
    }
    else
    {
        printf("\n HECC LOOPBACK TEST PASSED ...!!!\n");
        return 1;
    }

#else

    Uint32 data,j,i;

 #ifdef MASTER

      i = j = 0;
      while(j < LOOPCOUNT)
      {
       data = 0;
       for(i=0;i<NUMOFBYTESPERTRANSFER;i++)
       {

/* *****************************************************************************
 *                                                                             *
 *   Below line of code generates the 4 bytes data of below mentioned pattern  *
 *                                                                             *
 *       00 01 02 03                                                           *
 *       01 02 03 04                                                           *
 *       02 03 04 05                                                           *
 *       .                                                                     *
 *       .                                                                     *
 *                                                                             *
 *        (i+j)                                Gets the byte value i.e         *
 *                                              00 01 02 03 04 05 06 ....      *
 *                                                                             *
 *        (((NUMOFBYTESPERTRANSFER-(i+1))*8))  Will generates the shift count  *
 *                                              24,16,8,0....per transfer.     *
 *                                                                             *
 *                                                                             *
 *   For Example:                                                              *
 *           when j = 0,and                                                    *
 *              when i = 0,1,2,3   with shift count of 24,16,8,0               *
 *               Generates the pattern 00 01 02 03                             *
 *                                                                             *
 *           when j = 1,and                                                    *
 *              when i = 0,1,2,3  with shift count of 24,16,8,0                *
 *               Generates the pattern 01 02 03 04                             *
 *           when j = 2,and                                                    *
 *              when i = 0,1,2,3 shift count of 24,16,8,0                      *
 *               Generates the pattern 02 03 04 05                             *
 ******************************************************************************/

        data |=   ( ((Uint8)(i + j)) << (((NUMOFBYTESPERTRANSFER-(i+1))*8)) );
      }
      HeccTransfer(data);
      j++;
    }
    if(j == LOOPCOUNT )
    {
      printf("\nData Written  Succussfully for %d times...!!!",j);
      return 1;
    }
    else
    {
      printf("\nData Write  Failed for %d time...!!!",j);
      return 0;
    }
 #else

    Uint8 bytedata,flag = 0;

    i = j =  0;
    while(j < LOOPCOUNT)
    {
      data = bytedata = 0;
      HeccReceive(&data);
      for(i=0; i<NUMOFBYTESPERTRANSFER; i++)
       {
          bytedata =  (Uint8)( data >> (((NUMOFBYTESPERTRANSFER-(i+1))*8)) );
          if( bytedata != (i+j) )
          {
            printf("\n\n Data Mismatch ....Communication Failed");
            flag = 1;
            break;
          }
        }
        if(flag)
          break;
        j++;
    }
    if(j == LOOPCOUNT )
    {
        printf("\n\nCommunication Succussful ...!!!");
        return 1;
    }
    else
    {
        printf("\n\nCommunication Test Failed ...!!!");
        return 0;
    }
 #endif

#endif

}

/*
 * =============================================================================
 *   @func  DeviceInit
 *
 *   @arg
 *      NONE
 *
 *   @desc
 *      Initializes the device for CAN operation.
 *
 *
 *   @return
 *      NONE
 *
 * =============================================================================
 */


void DeviceInit(void)
{

      CSL_PscRegsOvly pscRegs = (CSL_PscRegsOvly)CSL_PSC_0_REGS;

      CSL_SysRegsOvly SysRegs = (CSL_SysRegsOvly)CSL_SYS_0_REGS;


      pscRegs->MDCTL[CSL_PSC_HECC] = CSL_FMKT( PSC_MDCTL_NEXT, ENABLE )
                                   | CSL_FMKT( PSC_MDCTL_LRST, DEASSERT );
      pscRegs->PTCMD = CSL_FMKT(  PSC_PTCMD_GO0, SET );

  /*  wait for transition  */
      while ( CSL_FEXT( pscRegs->MDSTAT[CSL_PSC_HECC], PSC_MDSTAT_STATE )
              != CSL_PSC_MDSTAT_STATE_ENABLE );

/*
 *  The CAN module must be initialized before the utilization.
        To initialize the CAN module:*/

/* 1. Perform the necessary device pin multiplexing setup
        (see the device-specific data manual).*/

      SysRegs->PINMUX1 |= CSL_ENABLE_CAN;

/* 2. Program the VDD3P3V_PWDN register to power up the IO pins for the
         CAN module */

/* 3. Enable the CANTX pin by setting the TXFUNC bit in the transmit I/O control
        register (CANTIOC) to 1.*/

      CSL_FINST(HeccRegs->CANTIOC,HECC_CANTIOC_TXFUNC,CANTX);

/* 4. Enable the CANRX pin by setting the RXFUNC bit in the receive I/O control
        register (CANRIOC) to 1. */

      CSL_FINST(HeccRegs->CANRIOC,HECC_CANRIOC_RXFUNC,CANRX);

/*
   5. After a reset, the CCR bit in the master control register (CANMC) and the
        CCE bit in the error and  status register (CANES) are set to 1. This
        allows you to configure the bit-timing configuration register (CANBTC).

        If the CCE bit is set to 1, proceed to the next step; otherwise, set the
        CCR bit in CANMC to 1 and wait  until the CCE bit in CANES is set to 1.
*/
    CSL_FINST(HeccRegs->CANMC,HECC_CANMC_CCR,WRT_ACCESS);
    while (CSL_FEXT(HeccRegs->CANES,HECC_CANES_CCE) != CSL_HECC_CANES_CCE_WRITE);

/*
  6. Program CANBTC with the appropriate timing values. Make sure that the
      TSEG1 and TSEG2 bit values are not 0. If they are 0, the module does not
      leave the initialization mode. For example: write CANBTC = 1021Ah.
*/
    HeccRegs->CANBTC =  CSL_FMK(HECC_CANBTC_BRP,0x01)      | /* prescalar */
                        CSL_FMK(HECC_CANBTC_ERM,CSL_HECC_SYNC_FALL) |
                        CSL_FMK(HECC_CANBTC_SJW, 0x2u)|     /* sync jump width */
                        CSL_FMK(HECC_CANBTC_SAM,CSL_HECC_ONE_POINT_SAMPLE)  |
                        CSL_FMK(HECC_CANBTC_TSEG1, 0x03)    |
                        CSL_FMK(HECC_CANBTC_TSEG2, 0x02) ;

/*
  7. For the SCC, program the acceptance mask registers.
        For example: write SCCLAM3 = 3C 0000h.*/


/*
  5. Program the master control register (CANMC):
         Clear the CCR bit to 0
         Clear the PDR bit to 0
         Clear the DBO bit to 0
         Clear the WUBA bit to 0
         Clear the CDR bit to 0
         Clear the ABO bit to 0
         Clear the STM bit to 0
         Clear the SRES bit to 0
         Clear the MBNR bits to 0  */

    CSL_FINST(HeccRegs->CANMC,HECC_CANMC_CCR,NORMAL);

/*
  6. Verify the CCE bit in CANES is cleared to 0, indicating that the CAN module
        has been configured.*/

    while (CSL_FEXT(HeccRegs->CANES,HECC_CANES_CCE) != CSL_HECC_CANES_CCE_NOWRITE);

    printf("\nCAN module has been configured...\n");

}

/*
 * =============================================================================
 *   @func  HeccTransfer
 *
 *   @arg
 *      Data   Data to be transmit from CAN module
 *
 *   @desc
 *      Transmit the data from the Transmit mail box.
 *
 *
 *   @return
 *      NONE
 *
 * =============================================================================
 */

void HeccTransfer(Uint32 data)
{

/*

    To start a transmission (in this example, for object 1):
     1. Write the message data into the mailbox data field. Since the DBO bit in
        the master control register (CANMC) is cleared to 0, in the configuration
        section, and the message control field register 1 (MCF1)
        is set to 2, the data are stored in the 2 MSBytes of the message data
        low register 1 (MDL1).
             Write MDL1 = xxxx 0000h
*/
        HeccRegs->MBX[CSL_HECC_MBX_1].MDL = data;

/*
    2. Set the corresponding TRS[n] bit in the transmission request set register
       (CANTRS) to 1, to start the  transmission of the message. The CAN module
       now handles the complete transmission of the CAN message.
*/

        CSL_FINSR(HeccRegs->CANTRS,CSL_HECC_MBX_1,CSL_HECC_MBX_1,TRUE);

/*
    3. Wait until the transmit acknowledge (TA[n]) bit in the transmission
       acknowledge register (CANTA) of the corresponding mailbox is set to 1.
       After a successful transmission, this flag is set by the CAN module.
         TA[1] = 1
*/

         while((CSL_FEXT(HeccRegs->CANTA, HECC_CANTA_TA1) != CSL_HECC_CANTA_TA_SENT))
           ;

/*
    4. The TRS[n] bit in CANTRS is reset to 0 by the module after a successful
        or aborted transmission.
         TRS[1] = 0
*/

        while((CSL_FEXTR(HeccRegs->CANTRS, CSL_HECC_MBX_1,CSL_HECC_MBX_1) !=
                                                    CSL_HECC_CANTRS_TRS1_NOP))
           ;
/*
    5. The transmit acknowledge must be cleared for the next transmission.
        Set the TA[n] bit in CANTA to 1.Wait until read, TA[n] bit is 0.
             Set TA[1] = 1. Wait until read, TA[1] = 0.
*/
        CSL_FINSR(HeccRegs->CANTA,CSL_HECC_MBX_1,CSL_HECC_MBX_1,TRUE);

        while((CSL_FEXT(HeccRegs->CANTA, HECC_CANTA_TA1) == CSL_HECC_CANTA_TA_SENT))
           ;

    printf("\nCAN module Written the data Successfully...\n");

}

/*
 * =============================================================================
 *   @func  HeccReceive
 *
 *   @arg
 *      *Data   Pointer to the memory where received data should be placed
 *
 *   @desc
 *      Receives the data from received mail box.
 *
 *
 *   @return
 *      NONE
 *
 * =============================================================================
 */

void HeccReceive(Uint32 *data)
{


    /* Wait for the messge to arrive */

    while(CSL_FEXT(HeccRegs->CANRMP,HECC_CANRMP_RMP2) !=CSL_HECC_CANRMP_RMP2_MESG)
        ;

    /* Message is arrived, Read the message and store/display */
    /*
      The CPU may then read the message from the mailbox RAM. Before the CPU
      reads the message from the mailbox,it should first clear the RMP bit
      (RMP[2] = 1).
    */

    CSL_FINS(HeccRegs->CANRMP,HECC_CANRMP_RMP2,TRUE);

    *data = HeccRegs->MBX[CSL_HECC_MBX_2].MDL;

    printf("\nCAN module Read the data Successfully...\n");
    printf("\n\n 0x%x...\n",*data);

}
