/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006, 2007
 *
 *   Use of this software is controlled by the terms and conditions found
 *   in the license agreement under which this software has been supplied.
 *  ============================================================================
 */

/** ============================================================================
 *   @brief A simple example to demonstrate CSL 3.x MCBSP register layer usage.
 *
 *   @file  mcbsp_example.c
 *
 *   <b> Example Description </b>
 *   @verbatim
     This example configures MCBSP0 in 32 bit loopback mode and uses software 
     polling to transmit and receive 32bit words. Four 32 bit words are 
     transmitted and received and the results are verified.  A pass\fail 
     test status is returned from main.     
     @endverbatim
 *
 *      
 *   <b> Procedure to run the example </b>
 *   @verbatim
     1. Configure the CCS setup to work with the emulator being used
     2. Please refer CCS manual for setup configuration and loading 
        proper GEL file
     3. Launch CCS window
     4. Open project Mcbsp_example.pjt
     5. Build the project and load the .out file of the project.
     @endverbatim
 *
 * =============================================================================
 **/

#include <csl_types.h>
#include <soc.h>
#include <cslr_sys.h>
#include <cslr_mcbsp.h>
#include <stdio.h>
#include <cslr_psc.h>
#define NUM_WORDS   4

static void device_init(void); 
static void init_mcbsp(void);   
static int test_mcbsp(void);   
 

CSL_McbspRegsOvly mcbspRegs = (CSL_McbspRegsOvly)CSL_MCBSP_0_REGS;

int main (void) 
{
  //enable mcbsp0 in power and sleep controller
  device_init(); 
  
  //setup mcbsp registers and start mcbsp running 
  init_mcbsp();
  
  //test loopback, returns 0 for pass and 1 for fail 
  return(test_mcbsp());
}

void init_mcbsp(void)
{
  //serial port control register SPCR
  mcbspRegs->SPCR = CSL_FMKT(MCBSP_SPCR_DLB,ENABLE);    //enable loopback mode     
                 
  // receive control register                
  mcbspRegs->RCR = CSL_FMKT(MCBSP_RCR_RWDLEN1,32BIT);   //receive word 32bit 
  
  //transmit control register
  mcbspRegs->XCR = CSL_FMKT(MCBSP_XCR_XWDLEN1,32BIT);   //trans word 32bit
  
  //sample rate generator SRGR
  mcbspRegs->SRGR = CSL_FMKT(MCBSP_SRGR_CLKSM,INTERNAL) //internal clock
                  | CSL_FMK(MCBSP_SRGR_FPER,63)         //frame period
                  | CSL_FMK(MCBSP_SRGR_FWID,31)         //frame width
                  | CSL_FMK(MCBSP_SRGR_CLKGDV,15);      //clock divider value

  //pin control register
  mcbspRegs->PCR = CSL_FMKT(MCBSP_PCR_FSXM,INTERNAL)    //internal frame sync
                 | CSL_FMKT(MCBSP_PCR_FSRM,INTERNAL)    //internal fram sync
                 | CSL_FMKT(MCBSP_PCR_CLKXM,OUTPUT);    //trans clock mode
        
                 
  //start the mcbsp running
  CSL_FINST(mcbspRegs->SPCR,MCBSP_SPCR_RRST,ENABLE);    //receiver enable
  CSL_FINST(mcbspRegs->SPCR,MCBSP_SPCR_XRST,ENABLE);    //transmitter enable
  CSL_FINST(mcbspRegs->SPCR,MCBSP_SPCR_GRST,CLKG);      //SRGR out of reset
  CSL_FINST(mcbspRegs->SPCR,MCBSP_SPCR_FRST,FSG);       //enable frame sync
}

int test_mcbsp(void)
{
    int i;
    Uint32 trans_val[NUM_WORDS] = {0x12345678,0x87654321,0x11223344,0x55667788};
    Uint32 rec_val[NUM_WORDS];

    printf("\nTesting mcbsp loopback\n\n");

    for(i=0;i<(sizeof(trans_val)/sizeof(trans_val[0]));i++) 
    {
        //poll transmitter ready
        while(CSL_FEXT(mcbspRegs->SPCR,MCBSP_SPCR_XRDY)
             != CSL_MCBSP_SPCR_XRDY_YES);

        //write to transmit register
        mcbspRegs->DXR = trans_val[i];

        //poll receiver is ready
        while(CSL_FEXT(mcbspRegs->SPCR,MCBSP_SPCR_RRDY)
             != CSL_MCBSP_SPCR_RRDY_YES);

        //read receive register
        rec_val[i] = mcbspRegs->DRR;
    }

    //check results
    for(i=0;i<(sizeof(trans_val)/sizeof(trans_val[0]));i++) 
    {
        if(trans_val[i]==rec_val[i])
            printf("transmit: 0x%x   receive: 0x%x\n",trans_val[i],rec_val[i]);
        else
           {
             printf(" error\n");
             printf("\nMCBSP Loopback Test:  FAILED\n");
             return(1);
           }
    }

    printf("\nMCBSP Loopback Test:  PASSED\n");
    
    return(0);
}

void device_init(void) 
{
  
  CSL_PscRegsOvly pscRegs = (CSL_PscRegsOvly)CSL_PSC_0_REGS;

  // deassert MCBSP0 local PSC reset and set NEXT state to ENABLE
  pscRegs->MDCTL[CSL_PSC_MCBSP0] = CSL_FMKT( PSC_MDCTL_NEXT, ENABLE )
                                 | CSL_FMKT( PSC_MDCTL_LRST, DEASSERT );
 
  //move MCBSP0 PSC to Next state
  pscRegs->PTCMD = CSL_FMKT(  PSC_PTCMD_GO0, SET );
 
  //wait for transition
  while ( CSL_FEXT( pscRegs->MDSTAT[CSL_PSC_MCBSP0], PSC_MDSTAT_STATE )
          != CSL_PSC_MDSTAT_STATE_ENABLE );
 
}
 
