/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/*
 * \file   aic23Codec.c
 *
 * \brief  AIC23 audio codec configuration file
 *
 * This file contains the API used for configuring the audio codec
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \note       Set tabstop to 4 (:se ts=4) while viewing this file in an
 *              editor
 *
 *  \author
 *
 *  \version    0.1
 *
 */

#include <std.h>
#include <stdio.h>
#include <log.h>
#include <ti/sdo/pspdrivers/common/psp_common.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osWait.h>

#include <ti/sdo/pspdrivers/pal_os/pal_osTask.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osSem.h>

#include <ti/sdo/pspdrivers/pal_os/bios/psputils.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osmem.h>
#include <ti/sdo/pspdrivers/drivers/i2c/psp_i2c.h>
#include <ti/sdo/pspdrivers/drivers/i2c/src/psp_i2cApi.h>

#ifdef CHIP_DM6437
#include <ti/sdo/pspdrivers/system/dm6437/bios/evmDM6437/audio/src/psp_aic33Codec.h>
#endif

#ifdef CHIP_C6424
#include <ti/sdo/pspdrivers/system/c6424/bios/evm6424/audio/src/psp_aic33Codec.h>
#endif


#define AIC33_DEBUG                 LOG_printf
#define I2C_INSTANCE_TO_AIC33       (0)
#define MCASP_MCBSP_IO_EXPANDER     (0x3B)
#define MCBSP_SELECT_MASK           (0xFF)
#define MCASP_SELECT_MASK           (0xFC)


/* PLL IO expander macro definitions  */
#define PLL_IO_EXPANDER_ADDRESS     (0x3A)
/* FS bit positions */
#define IO_EXP_PLL1705_FS1_PORT     (6)
#define IO_EXP_PLL1705_FS2_PORT     (5)
#define IO_EXP_PLL1705_CSEL_PORT    (7)
#define IO_EXP_PLL1705_SR_PORT      (4)

#define INPUT_GAIN_MIN            LIG_MIN
#define INPUT_GAIN_MAX            LIG_MAX
#define INPUT_GAIN_RANGE          (INPUT_GAIN_MAX - INPUT_GAIN_MIN)

#define OUTPUT_GAIN_MIN           LOG_MIN
#define OUTPUT_GAIN_MAX           LOG_MAX
#define OUTPUT_GAIN_RANGE         (INPUT_GAIN_MAX - INPUT_GAIN_MIN)

extern far LOG_Obj trace;

static Int gCount;
static aic33_CodecHandle aic33handle = {0};

/** Read/Write the AIC33 registers */
static Int aic33_rset      ( aic33_CodecHandle aic33handle, Uint16 regnum, Uint16 regval );

/** Write the data to the I2C controller's DXR  */
static Int i2c_write       (Uint16 slaveaddr, Uint8* data, Uint16 len );
/** Read the data to the I2C controller's DXR  */
static Int i2c_read(Uint16 slaveaddr, Uint8* data, Uint16 len);
/** Read the data from the I2C controller's DRR */
static void    i2c_close       (void);
static Int     i2c_setup       (void);
static Void aic31CallBack (Ptr semHandle,Ptr arg, Ptr ioRequest);


static PSP_Handle i2cHandle = NULL;

/* *
 * AIC33_rset( )
 *     Set codec register regnum to value regval
 *
 **/
Int aic33_rset( aic33_CodecHandle aic33handle, Uint16 regnum, Uint16 regval )
{
    Int retStat;
    Uint8 cmd[2];

    /* To avoid compiler warning */
    aic33handle = aic33handle;

    cmd[0] = (Uint8) regnum;            /* 7-bit Device Address */
    cmd[1] = (Uint8) regval;            /* 8-bit Register Data */

    retStat = i2c_write( I2C_AIC33_SLAVE_ADDR, cmd, 2 );
    gCount++;

    return retStat;
}

/**
 *  aic33_sampleRateControl( PSP_audio_cfg *audioCfg )
 *  Configure the sampling rate of the Audio Codec
 **/
Int aic33_sampleRateControl ( PSP_audio_cfg *audioCfg )
{
    Bool   closeRequired    = FALSE;
    Int    retCode          = 0;
	Uint16 ADC_DualRate     = ADC_DUALRATE_DISABLE;
	Uint16 DAC_DualRate     = DAC_DUALRATE_DISABLE;
    Uint8  fsRef            = FSREF_44100;
#ifdef PLL_INPUT_CLK
    Uint16 PLLQ_Value       = 0;
    Uint16 sampleRateDiv    = 0u;
#else
    Uint16 sampleRateDiv    = 2u;
    Uint16 PLLB_Value       = 0;
    Uint16 PLLC_Value       = 0;
    Uint16 PLLD_Value       = 0;
#endif

    /* Open only when this function is called seperately */
    if (NULL == i2cHandle)
    {
        i2c_setup ( );
        closeRequired = TRUE;
    }

#ifndef PLL_INPUT_CLK
    /** Configure the codec to generate 44.1KHz */
    /**  Fs == Fsref
     *   AIC33_MCLK = 22.5792 MHz
     *   Fsref = ( AIC33_MCLK * R * K ) / ( 2048 * P )
     *   441000Hz = (22.5792 MHz * R * K) /(2048 * P)
     *   for P = 2 => K = J . D = 8.02816 ; J = 8, D = 281
     *   Reset value of R = 1
     **/

    switch(audioCfg->sampleFreq)
    {
        case 7350:  /* FS = Fref/6   */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x00;
            PLLD_Value      = 0x00;
            fsRef           = FSREF_44100;
            sampleRateDiv   = 0x0A;
            break;

        case 8018:  /* FS = Fref/5.5 */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x00;
            PLLD_Value      = 0x00;
            fsRef           = FSREF_44100;
            sampleRateDiv   = 0x09;
            break;

        case 8820:  /* FS = Fref/5   */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x00;
            PLLD_Value      = 0x00;
            fsRef           = FSREF_44100;
            sampleRateDiv   = 0x08;
            break;

        case 9800:  /* FS = Fref/4.5 */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x00;
            PLLD_Value      = 0x00;
            fsRef           = FSREF_44100;
            sampleRateDiv   = 0x07;
            break;

        case 11025: /* FS = Fref/4   */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x00;
            PLLD_Value      = 0x00;
            fsRef           = FSREF_44100;
            sampleRateDiv   = 0x06;
            break;

        case 12600: /* FS = Fref/3.5 */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x00;
            PLLD_Value      = 0x00;
            fsRef           = FSREF_44100;
            sampleRateDiv   = 0x05;
            break;

        case 14700: /* FS = Fref/3   */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x00;
            PLLD_Value      = 0x00;
            fsRef           = FSREF_44100;
            sampleRateDiv   = 0x04;
            break;

        case 17640: /* FS = Fref/2.5 */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x00;
            PLLD_Value      = 0x00;
            fsRef           = FSREF_44100;
            sampleRateDiv   = 0x03;
            break;

        case 22050: /* FS = Fref/2   */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x00;
            PLLD_Value      = 0x00;
            fsRef           = FSREF_44100;
            sampleRateDiv   = 0x02;
            break;

        case 29400: /* FS = Fref/1.5 */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x00;
            PLLD_Value      = 0x00;
            fsRef           = FSREF_44100;
            sampleRateDiv   = 0x01;
            break;

        case 44100: /* FS = Fref/1   */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x00;
            PLLD_Value      = 0x00;
            fsRef           = FSREF_44100;
            sampleRateDiv   = 0x00;
            break;

        case 8000:  /* FS = Fref/6   */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x6E;
            PLLD_Value      = 0x23;
            fsRef           = FSREF_48000;
            sampleRateDiv   = 0x0A;
            break;

        case 8727:  /* FS = Fref/5.5 */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x6E;
            PLLD_Value      = 0x23;
            fsRef           = FSREF_48000;
            sampleRateDiv   = 0x09;
            break;

        case 9600:  /* FS = Fref/5   */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x6E;
            PLLD_Value      = 0x23;
            fsRef           = FSREF_48000;
            sampleRateDiv   = 0x08;
            break;

        case 10667: /* FS = Fref/4.5 */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x6E;
            PLLD_Value      = 0x23;
            fsRef           = FSREF_48000;
            sampleRateDiv   = 0x07;
            break;

        case 12000: /* FS = Fref/4   */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x6E;
            PLLD_Value      = 0x23;
            fsRef           = FSREF_48000;
            sampleRateDiv   = 0x06;
            break;

        case 13714: /* FS = Fref/3.5 */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x6E;
            PLLD_Value      = 0x23;
            fsRef           = FSREF_48000;
            sampleRateDiv   = 0x05;
            break;

        case 16000: /* FS = Fref/3   */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x6E;
            PLLD_Value      = 0x23;
            fsRef           = FSREF_48000;
            sampleRateDiv   = 0x04;
            break;

        case 19200: /* FS = Fref/2.5 */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x6E;
            PLLD_Value      = 0x23;
            fsRef           = FSREF_48000;
            sampleRateDiv   = 0x03;
            break;

        case 24000: /* FS = Fref/2   */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x6E;
            PLLD_Value      = 0x23;
            fsRef           = FSREF_48000;
            sampleRateDiv   = 0x02;
            break;

        case 32000: /* FS = Fref/1.5 */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x6E;
            PLLD_Value      = 0x23;
            fsRef           = FSREF_48000;
            sampleRateDiv   = 0x01;
            break;

        case 48000: /* FS = Fref/1   */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x6E;
            PLLD_Value      = 0x23;
            fsRef           = FSREF_48000;
            sampleRateDiv   = 0x00;
            break;

        case 88200: /* FS = Fref/1   */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x00;
            PLLD_Value      = 0x00;
            fsRef           = FSREF_44100;
            sampleRateDiv   = 0x00;
            ADC_DualRate    = ADC_DUALRATE_ENABLE;
            DAC_DualRate    = DAC_DUALRATE_ENABLE;
            break;

        case 96000: /* FS = Fref/1   */
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x6E;
            PLLD_Value      = 0x23;
            fsRef           = FSREF_48000;
            sampleRateDiv   = 0x00;
            ADC_DualRate    = ADC_DUALRATE_ENABLE;
            DAC_DualRate    = DAC_DUALRATE_ENABLE;
            break;

        default:   /* Default to 44.1 KHz */
            retCode         = -1;
            break;
    }

    if (-1 != retCode)
    {
        /* This is a hack to take care of the noise problem when AIC33 is
           slave mode. AIC33 is configured in 8000KHz sample rate when in
           slave mode.
         */
        if (PSP_AUDIO_AIC_SLAVE == audioCfg->mode)
        {
            PLLB_Value      = 0x20;
            PLLC_Value      = 0x6E;
            PLLD_Value      = 0x23;
            fsRef           = FSREF_48000;
            sampleRateDiv   = 0x0A;
        }


        /* Copy sample rate div value for Input & Output */
        sampleRateDiv = (Uint16)(((Uint32)sampleRateDiv << 4) | (Uint32)sampleRateDiv);

        /** Select ADC Fs == Fsref and DAC Fs == Fsref */
        aic33_rset(aic33handle, AIC33_CODEC_SAMPLE_RATE_ADDR, sampleRateDiv);
        aic33_rset(aic33handle, AIC33_PLLA_PROG_ADDR,0x0092); /**  [PLL=ON][P=2][Q=2] */

        aic33_rset(aic33handle, AIC33_PLLB_PROG_ADDR,PLLB_Value); /** [J] */
        aic33_rset(aic33handle, AIC33_PLLC_PROG_ADDR,PLLC_Value); /** [D MSB] */
        aic33_rset(aic33handle, AIC33_PLLD_PROG_ADDR,PLLD_Value); /** [D LSB] */
        aic33_rset(aic33handle, AIC33_CODEC_DATAPATH_SETUP_ADDR, (Uint16)(fsRef |
                                              ADC_DualRate |
                                              DAC_DualRate |
                                              ((Uint16)LEFT_DAC_PATH_PLAYS_LEFTDATA<<3) |
                                              ((Uint16)RIGHT_DAC_PATH_PLAYS_RIGHTDATA <<1)));

    }

#else
        switch (audioCfg->sampleFreq)
        {
            case 11025:
                PLLQ_Value = 2;
                fsRef = FSREF_44100;
                sampleRateDiv = 0x06;
                break;
            case 22050:
                PLLQ_Value = 2;
                fsRef = FSREF_44100;
                sampleRateDiv = 0x02;
                break;
            case 44100:
                PLLQ_Value = 2;
                fsRef = FSREF_44100;
                sampleRateDiv = 0x00;
                break;
            /* sampling rate of 8KHz,16KHz,32KHz,48KHz       */
            case 8000:
                PLLQ_Value = 2;
                fsRef = FSREF_48000;
                sampleRateDiv = 0xA;
                break;
            case 12000:
                PLLQ_Value = 2;
                fsRef = FSREF_48000;
                sampleRateDiv = 0x06;
                break;
            case 16000:
                PLLQ_Value = 2;
                fsRef = FSREF_48000;
                sampleRateDiv = 0x04;
                break;
            case 24000:
                PLLQ_Value = 2;
                fsRef = FSREF_48000;
                sampleRateDiv = 0x02;
                break;
            case 32000:
                PLLQ_Value = 2;
                fsRef = FSREF_48000;
                sampleRateDiv = 0x01;
                break;
            case 48000:
                PLLQ_Value = 2;
                fsRef = FSREF_48000;
                sampleRateDiv = 0x00;
                break;
            /* sampling rate of 96KHz                                         */
            case 96000:
                PLLQ_Value = 2;
                fsRef = FSREF_48000;
                ADC_DualRate = ADC_DUALRATE_ENABLE;
                DAC_DualRate = DAC_DUALRATE_ENABLE;
                sampleRateDiv = 0x00;
                break;
            default:
                retCode = -1;
                break;
        }

        /* select the "CLKDIV_OUT" to be output                               */
        aic33_rset(aic33handle, AIC33_ADDITIONAL_GPIOCTRLB_ADDR, 0x01);

        /* select the Mclk as the input for the "CLKDIV_IN" source selection  */
        aic33_rset(aic33handle, AIC33_CLKGEN_CTRL_ADDR, 0x02);

        sampleRateDiv = (Uint16)(((Uint32)sampleRateDiv << 4)
                        | (Uint32)sampleRateDiv);

        /* Select ADC Fs == Fsref and DAC Fs == Fsref                         */
        aic33_rset(aic33handle, AIC33_CODEC_SAMPLE_RATE_ADDR, sampleRateDiv);

        /* Disable the PLL and set Q value  to output the MCLK                */
        aic33_rset(aic33handle, AIC33_PLLA_PROG_ADDR, (PLLQ_Value << 3));

        aic33_rset(
            aic33handle,
            AIC33_CODEC_DATAPATH_SETUP_ADDR,
            (Uint16)((fsRef)
                | ADC_DualRate
                | DAC_DualRate
                | ((Uint16)LEFT_DAC_PATH_PLAYS_LEFTDATA << 3)
                | ((Uint16)RIGHT_DAC_PATH_PLAYS_RIGHTDATA <<1)));

#endif

    /* Close only when this function did a I2C Open */
    if (TRUE == closeRequired)
    {
        i2c_close();
    }

    return (retCode);
}

/**
 *  aic33Codec_config( Uint32 id, PSP_audio_cfg *audioCfg )
 *     Configure the codec
 **/
Int aic33Codec_config( Uint32 id, PSP_audio_cfg *audioCfg )
{

    Int    retCode        = 0;
    Uint16 gain           = 0;
    Uint16 micBiasVoltage = 0;

    /* To avoid compiler warning */
    id = id;

    AIC33_DEBUG(&trace, "\r\nAIC33  :Trying to Configure Audio Codec\n");

    retCode = i2c_setup ( );
    if (!retCode)
    {
        return -1;
    }

    /* checking for master/slave mode configuration */
    if((PSP_AUDIO_AIC_MASTER != audioCfg->mode)
       && (PSP_AUDIO_AIC_SLAVE != audioCfg->mode))
    {
        return -1;
    }

    /* checking for input/record mode */
    if((PSP_AUDIO_LINE_IN != audioCfg->inputMode)
       && (PSP_AUDIO_MIC_IN != audioCfg->inputMode)
       && (PSP_AUDIO_MIC_IN_2_0_V != audioCfg->inputMode)
       && (PSP_AUDIO_MIC_IN_2_5_V != audioCfg->inputMode)
       && (PSP_AUDIO_MIC_IN_AVDD  != audioCfg->inputMode))
    {
        return -1;
    }

    /* Checking for input gain range */
    if ((0 == audioCfg->inputGain)||(audioCfg->inputGain >= 100u))
    {
        return -1;
    }

    /* Checking for output gain range */
    if ((0 == audioCfg->outputGain)||(audioCfg->outputGain >= 100))
    {
        return -1;
    }

    aic33_rset( aic33handle, AIC33_PAGE_SELECT_ADDR, 0x00 );  /* Select Page 0 */
    aic33_rset( aic33handle, AIC33_RESET_ADDR, 0x80 );        /* Reset the AIC33 */

    /* Change the sampling rate */
    retCode = aic33_sampleRateControl ( audioCfg );
    if (0 != retCode)
    {
        return -1;
    }

    if(PSP_AUDIO_AIC_MASTER == audioCfg->mode)
    {
        AIC33_DEBUG(&trace, "\r\nAIC33  :Configuring the AIC33 in Master mode\n");

        /** Both bit clock and word clock are driven by codec */
        aic33_rset(aic33handle,AIC33_SERIAL_INTERFACEA_ADDR, 0x00C0);
    }
    else
    {
        AIC33_DEBUG(&trace, "\r\nAIC33  :Configuring the AIC33 in Slave mode\n");

        /** Both bit clock and word clock are driven by the slave only (McBSP) */
        aic33_rset(aic33handle, AIC33_SERIAL_INTERFACEA_ADDR,
                      AIC33_BCLK_SLAVE | AIC33_WCLK_SLAVE);
    }

    /**  DSP-mode , 16-bit of audio word length */
    aic33_rset(
            aic33handle,
            AIC33_SERIAL_INTERFACEB_ADDR,
            (AIC33_DSP << 6)| (AIC33_WORDLEN_16 << 4)|AIC33_BCLK_CTRL_CONTINUOS
            );

    /** Data offset is zero */
    aic33_rset(aic33handle,AIC33_SERIAL_INTERFACEC_ADDR,0x0000);

    /** Configure the reset value only */
    aic33_rset(aic33handle,AIC33_CODEC_DIGIFILTER_CTRL_ADDR,0x0000);

    /** Headset Detection  is OFF */
    aic33_rset(aic33handle,AIC33_CODEC_HEADSET_DETA_ADDR,0x0000);
    aic33_rset(aic33handle,AIC33_CODEC_HEADSET_DETB_ADDR,0x0000);

    /* Mute OFF the left ADC PGA and set the gain to 1db */
    aic33_rset(aic33handle,AIC33_LEFT_ADCPGA_GAIN_CTRL_ADDR,0x0004);

    /* Mute OFF the right ADC PGA and set the gain to 1db */
    aic33_rset(aic33handle,AIC33_RIGHT_ADCPGA_GAIN_CTRL_ADDR,0x0004);


    if (PSP_AUDIO_LINE_IN != audioCfg->inputMode)
    {
        /** Configure the MICBIAS control */
        switch(audioCfg->inputMode)
        {
            case PSP_AUDIO_MIC_IN:
                    micBiasVoltage = 0x0000;
                    break;
            case PSP_AUDIO_MIC_IN_2_0_V:
                    micBiasVoltage = 0x0040;
                    break;
            case PSP_AUDIO_MIC_IN_2_5_V:
                    micBiasVoltage = 0x0080;
                    break;
            case PSP_AUDIO_MIC_IN_AVDD:
                    micBiasVoltage = 0x00C0;
                    break;
            default:
                    retCode = -1;
                break;
        }

        if (-1 == retCode)
        {
            return -1;
        }
        /* enable the mic input*/
        aic33_rset(aic33handle,AIC33_MIC3LR_TO_LEFTADC_CTRL_ADDR, 0x0F);
        aic33_rset(aic33handle,AIC33_MIC3LR_TO_RIGHTADC_CTRL_ADDR, 0xF0);

        /* enable ADC's and disable the line input*/
        aic33_rset(aic33handle,AIC33_LINE1L_TO_LEFT_ADC_CTRL_ADDR, 0x7C);
        aic33_rset(aic33handle,AIC33_LINE1R_TO_RIGHT_ADC_CTL_ADDR, 0x7C);

        aic33_rset(aic33handle,AIC33_MICBIAS_CTRL_ADDR,micBiasVoltage);
    }
    else
    {
        /* enable ADC's, enable line iput */
        aic33_rset(aic33handle,AIC33_LINE1L_TO_LEFT_ADC_CTRL_ADDR, 0x04);
        aic33_rset(aic33handle,AIC33_LINE1R_TO_RIGHT_ADC_CTL_ADDR, 0x04);

        /* disable the mic input */
        aic33_rset(aic33handle,AIC33_MIC3LR_TO_LEFTADC_CTRL_ADDR, 0xFF);
        aic33_rset(aic33handle,AIC33_MIC3LR_TO_RIGHTADC_CTRL_ADDR, 0xFF);

        /** Power OFF the MICBIAS control */
        aic33_rset(aic33handle,AIC33_MICBIAS_CTRL_ADDR,0x0000);
    }

    /**
    ***** ROUTING THE LINE1 SIGNAL TO ADC PGA OF THE AIC33 CODEC *****
    * Connect the LINE1L to left ADC PGA and LINE1R to Right ADC PGA
    * Set the both ADC gain to 0dB
    */

    /** don't connect LINE2L to the Right ADC PGA */
    aic33_rset(aic33handle,AIC33_LINE2L_TO_LEFT_ADC_CTRL_ADDR,0x0078);

    /** don't connect LINE1R to the Left ADC PGA */
    aic33_rset(aic33handle,AIC33_LINE1R_TO_LEFT_ADC_CTRL_ADDR,0x0078);

    /** don't connect LINE2R to the Right ADC PGA */
    aic33_rset(aic33handle,AIC33_LINE2R_TO_RIGTH_ADC_CTRL_ADDR,0x0078);

    /** don't connects LINE1L to the Right ADC PGA */
    aic33_rset(aic33handle,AIC33_LINE1L_TO_RIGTH_ADC_CTRL_ADDR,0x0078);


    /** Configure for input gain */
    gain = (Uint16)(((audioCfg->inputGain * INPUT_GAIN_RANGE) / 100u) + INPUT_GAIN_MIN);

    /* Left AGC control */
    aic33_rset(aic33handle,AIC33_LEFT_AGC_CTRLA_ADDR, 0x80);
    aic33_rset(aic33handle,AIC33_LEFT_AGC_CTRLB_ADDR, (Uint16)((Uint32)gain << 1));
    aic33_rset(aic33handle,AIC33_LEFT_AGC_CTRLC_ADDR, 0x00);

    /* Right AGC control */
    aic33_rset(aic33handle,AIC33_RIGHT_AGC_CTRLA_ADDR, 0x80);
    aic33_rset(aic33handle,AIC33_RIGHT_AGC_CTRLB_ADDR, (Uint16)((Uint32)gain << 1));
    aic33_rset(aic33handle,AIC33_RIGHT_AGC_CTRLC_ADDR, 0x00);

    aic33_rset(aic33handle,AIC33_LEFT_AGC_NOISE_GATE_DEBOUNCE_ADDR,0x0000);
    aic33_rset(aic33handle,AIC33_RIGHT_AGC_NOISE_GATE_DEBOUNCE_ADDR,0x0000);

    /** Turn ON the LeftDACPower and RightDACPower and configure the HPLCOM as SingleEnded */
     aic33_rset(aic33handle,AIC33_DAC_POWER_OUTPUT_DRIVER_CTRL_ADDR,0x00E0);

    /**configure the HPRCOM as SingleEnded  and Short Circuit protection control to OFF */
    aic33_rset(aic33handle,AIC33_HP_OUTPUT_DRIVER_CTRL_ADDR,0x0010);

    aic33_rset(aic33handle,AIC33_HP_OUTPUT_STAGE_CTRL_ADDR,0x0000);

    /** Left DAC output selects DAC_L1 path and Right DAC output selects DAC_R1 path and
        Right DAC volume follows the left channel control register */
    aic33_rset(aic33handle,AIC33_DAC_OUTPUT_SWITCH_CTRL_ADDR,0x0002);

    /** Output driver pop reduction */
    aic33_rset(aic33handle,AIC33_OUTPUT_DRV_POP_REDUCTION_ADDR,0x009C);

    /** Configure output gain */
    gain = (Uint16)(((audioCfg->outputGain * OUTPUT_GAIN_RANGE) / 100) + OUTPUT_GAIN_MIN);
    gain = (Uint16)(OUTPUT_GAIN_MAX - gain);

    /* Left/Right DAC digital volume gain */
    aic33_rset(aic33handle,AIC33_LEFT_DAC_VOL_CTRL_ADDR, gain);
    aic33_rset(aic33handle,AIC33_RIGHT_DAC_VOL_CTRL_ADDR, gain);

    aic33_rset(aic33handle,AIC33_LINE2L_TO_HPLOUT_VOL_CTRL_ADDR,0x0000);

    aic33_rset(aic33handle,AIC33_PGA_L_TO_HPLOUT_VOL_CTRL_ADDR,0x0000);

    /** Route the DAC L1 to HPLOUT */
    aic33_rset(aic33handle,AIC33_DAC_L1_TO_HPLOUT_VOL_CTRL_ADDR,0x0080);

    aic33_rset(aic33handle,AIC33_LINE2R_TO_HPLOUT_VOL_CTRL_ADDR,0x0000);

    aic33_rset(aic33handle,AIC33_PGA_R_TO_HPLOUT_VOL_CTRL_ADDR,0x0000);

    aic33_rset(aic33handle,AIC33_DAC_R1_TO_HPLOUT_VOL_CTRL_ADDR,0x0000);

    /** Power ON the HPLOUT */
    aic33_rset(aic33handle,AIC33_HPLOUT_OUTPUT_LEVEL_CTRL_ADDR,0x0001);
    /** Mute OFF the HPLOUT */
    aic33_rset(aic33handle,AIC33_HPLOUT_OUTPUT_LEVEL_CTRL_ADDR,0x0009);


    aic33_rset(aic33handle,AIC33_LIN2L_TO_HPLCOM_VOL_CTRL_ADDR,0x0000);
    aic33_rset(aic33handle,AIC33_PGA_L_TO_HPLCOM_VOL_CTRL_ADDR,0x0000);
    aic33_rset(aic33handle,AIC33_DAC_L1_TO_HPLCOM_VOL_CTRL_ADDR,0x0000);
    aic33_rset(aic33handle,AIC33_LINE2R_TO_HPLCOM_VOL_CTRL_ADDR,0x0000);
    aic33_rset(aic33handle,AIC33_PGA_R_TO_HPLCOM_VOL_CTRL_ADDR,0x0000);
    aic33_rset(aic33handle,AIC33_DAC_R1_TO_HPLCOM_VOL_CTRL_ADDR,0x0000);
    aic33_rset(aic33handle,AIC33_HPLCOM_OUTPUT_LEVEL_CTRL_ADDR,0x0000);


    aic33_rset(aic33handle,AIC33_LINE2L_TO_HPROUT_VOL_CTRL_ADDR,0x0000);

    aic33_rset(aic33handle,AIC33_PGA_L_TO_HPROUT_VOL_CTRL_ADDR,0x0000);

    aic33_rset(aic33handle,AIC33_DAC_L1_TO_HPROUT_VOL_CTRL_ADDR,0x0000);

    aic33_rset(aic33handle,AIC33_LINE2R_TO_HPROUT_VOL_CTRL_ADDR,0x0000);

    aic33_rset(aic33handle,AIC33_PGA_R_TO_HPROUT_VOL_CTRL_ADDR,0x0000);

    /** Route the DAC R1 to HPROUT */
    aic33_rset(aic33handle,AIC33_DAC_R1_TO_HPROUT_VOL_CTRL_ADDR,0x0080);


    /** Power ON the HPROUT */
    aic33_rset(aic33handle,AIC33_HPROUT_OUTPUT_LEVEL_CTRL_ADDR,0x0001);
    /** Mute OFF the HPROUT */
    aic33_rset(aic33handle,AIC33_HPROUT_OUTPUT_LEVEL_CTRL_ADDR,0x0009);


    aic33_rset(aic33handle,AIC33_LIN2L_TO_HPRCOM_VOL_CTRL_ADDR,0x0000);
    aic33_rset(aic33handle,AIC33_PGA_L_TO_HPRCOM_VOL_CTRL_ADDR,0x0000);
    aic33_rset(aic33handle,AIC33_DAC_L1_TO_HPRCOM_VOL_CTRL_ADDR,0x0000);
    aic33_rset(aic33handle,AIC33_LINE2R_TO_HPRCOM_VOL_CTRL_ADDR,0x0000);
    aic33_rset(aic33handle,AIC33_LINE2R_TO_HPRCOM_VOL_CTRL_ADDR,0x0000);
    aic33_rset(aic33handle,AIC33_DAC_R1_TO_HPRCOM_VOL_CTRL_ADDR,0x0000);
    aic33_rset(aic33handle,AIC33_HPRCOM_OUTPUT_LEVEL_CTRL_ADDR,0x0000);

    aic33_rset(aic33handle,AIC33_LIN2L_TO_MONO_LOPM_VOL_CTRL_ADDR,0x0000);
    aic33_rset(aic33handle,AIC33_PGA_L_TO_MONO_LOPM_VOL_CTRL_ADDR,0x0000);
    aic33_rset(aic33handle,AIC33_DAC_L1_TO_MONO_LOPM_VOL_CTRL_ADDR,0x0000);
    aic33_rset(aic33handle,AIC33_LINE2R_TO_MONO_LOPM_VOL_CTRL_ADDR,0x0000);
    aic33_rset(aic33handle,AIC33_PGA_R_TO_MONO_LOPM_VOL_CTRL_ADDR,0x0000);
    aic33_rset(aic33handle,AIC33_DAC_R1_TO_MONO_LOPM_VOL_CTRL_ADDR,0x0000);
    aic33_rset(aic33handle,AIC33_MONO_LOPM_OUTPUT_LEVEL_CTRL_ADDR,0x0000);


    aic33_rset(aic33handle,AIC33_LIN2L_TO_LEFT_LOPM_VOL_CTRL_ADDR,0x0000);

    aic33_rset(aic33handle,AIC33_PGA_L_TO_LEFT_LOPM_VOL_CTRL_ADDR,0x0000);

    /** Route the DAC_L1 to LEFT_LOP/M */
    aic33_rset(aic33handle,AIC33_DAC_L1_TO_LEFT_LOPM_VOL_CTRL_ADDR,0x0080);

    aic33_rset(aic33handle,AIC33_LINE2R_TO_LEFT_LOPM_VOL_CTRL_ADDR,0x0000);

    aic33_rset(aic33handle,AIC33_PGA_R_TO_LEFT_LOPM_VOL_CTRL_ADDR,0x0000);

    aic33_rset(aic33handle,AIC33_DAC_R1_TO_LEFT_LOPM_VOL_CTRL_ADDR,0x0000);

    /** Mute OFF and power ON the LEFT_LOP/M */
    aic33_rset(aic33handle,AIC33_LEFT_LOPM_OUTPUT_LEVEL_CTRL_ADDR,0x0009);



    aic33_rset(aic33handle,AIC33_LIN2L_TO_RIGHT_LOPM_VOL_CTRL_ADDR,0x0000);

    aic33_rset(aic33handle,AIC33_PGA_L_TO_RIGHT_LOPM_VOL_CTRL_ADDR,0x0000);

    aic33_rset(aic33handle,AIC33_DAC_L1_TO_RIGHT_LOPM_VOL_CTRL_ADDR,0x0000);

    aic33_rset(aic33handle,AIC33_LINE2R_TO_RIGHT_LOPM_VOL_CTRL_ADDR,0x0000);

    aic33_rset(aic33handle,AIC33_PGA_R_TO_RIGHT_LOPM_VOL_CTRL_ADDR,0x0000);

    /** Route the DAC_R1 to RIGHT_LOP/M */
    aic33_rset(aic33handle,AIC33_DAC_R1_TO_RIGHT_LOPM_VOL_CTRL_ADDR,0x0080);

    /** Mute OFF and power ON the RIGHT_LOP/M */
    aic33_rset(aic33handle,AIC33_RIGHT_LOPM_OUTPUT_LEVEL_CTRL_ADDR,0x0009);

    aic33_rset(aic33handle,AIC33_GPIO1_CTRL_ADDR,0x0000);
    aic33_rset(aic33handle,AIC33_GPIO2_CTRL_ADDR,0x0000);
    aic33_rset(aic33handle,AIC33_ADDITIONAL_GPIOCTRLA_ADDR,0x0000);

#ifndef PLL_INPUT_CLK
    aic33_rset(aic33handle,AIC33_ADDITIONAL_GPIOCTRLB_ADDR,0x0000);
#endif

    i2c_close();

    return (retCode);
}

/*
 *  aic33_inGainControl(Uint8 newGain, PSP_audio_cfg *audioCfg)
 *     Change the gain setting of input channel with a passed value
 *
 */
Int aic33_inGainControl(Uint8 newGain, PSP_audio_cfg *audioCfg)
{
    Uint16 gain   = 0;
    Int    status = -1;

    if (!i2c_setup ( ))
    {
        return -1;
    }

    if ((newGain > 0u) && (newGain < 100u))
    {
        gain = (Uint16)(((newGain * INPUT_GAIN_RANGE) / 100u) + INPUT_GAIN_MIN);

        /* Left AGC control */
        aic33_rset(aic33handle,AIC33_LEFT_AGC_CTRLA_ADDR, 0x80);
        aic33_rset(aic33handle,AIC33_LEFT_AGC_CTRLB_ADDR, (Uint16)((Uint32)gain << 1));
        aic33_rset(aic33handle,AIC33_LEFT_AGC_CTRLC_ADDR, 0x00);

        /* Right AGC control */
        aic33_rset(aic33handle,AIC33_RIGHT_AGC_CTRLA_ADDR, 0x80);
        aic33_rset(aic33handle,AIC33_RIGHT_AGC_CTRLB_ADDR, (Uint16)((Uint32)gain << 1));
        aic33_rset(aic33handle,AIC33_RIGHT_AGC_CTRLC_ADDR, 0x00);

        /* Update the configuration */
        audioCfg->inputGain = newGain;

        status = 0u;
    }

    i2c_close();

    return (status);
}

/*
 *  aic33_outGainControl(Uint8 newGain, PSP_audio_cfg *audioCfg)
 *     Change the gain setting of output channel with a passed value
 *
 */
Int aic33_outGainControl(Uint8 newGain, PSP_audio_cfg *audioCfg)
{
    Uint16 gain = 0;
    Int    status = -1;

    if (!i2c_setup ( ))
    {
        return -1;
    }

    if ((newGain > 0) && (newGain < 100))
    {
        gain = (Uint16)(((newGain * OUTPUT_GAIN_RANGE) / 100) + OUTPUT_GAIN_MIN);
        gain = (Uint16)(OUTPUT_GAIN_MAX - gain);

        /* Left/Right DAC digital volume gain */
        aic33_rset(aic33handle,AIC33_LEFT_DAC_VOL_CTRL_ADDR, gain);
        aic33_rset(aic33handle,AIC33_RIGHT_DAC_VOL_CTRL_ADDR, gain);

        /* Update the configuration */
        audioCfg->outputGain = newGain;

        status = 0u;
    }

    i2c_close();

    return (status);
}


/*
 *  aic33_closeCodec( )
 *     Close the codec
 *
 */
void aic33_closeCodec( aic33_CodecHandle aic33handle )
{
    if (!i2c_setup ( ))
    {
        return;
    }

    aic33_rset( aic33handle, 0, 0x00 );              /* Select Page 0 */
    aic33_rset( aic33handle, AIC33_RESET, 0x80 );    /* Reset the aic33 */

    i2c_close();
    return;
}

/*
 *  writeToIOExpander( )
 *    This function selects peripheral either McASP or
 *    McBSP should be active.
 *
 */
Int writeToIOExpander(Uint32 deviceSelect)
{
    Uint8    val1    = 0;
    Uint8    val2    = 0;
    Int      retCode = 1;

    if (!i2c_setup ( ))
    {
        return 0;
    }

    if (0 == deviceSelect)
    {
        /* McBSP select */
        val1 = MCBSP_SELECT_MASK;
        retCode  = i2c_write(MCASP_MCBSP_IO_EXPANDER, (Uint8*)&val1, 1);

        i2c_read(MCASP_MCBSP_IO_EXPANDER, (Uint8*)&val2, 1);
        if (val1 != val2)
        {
            AIC33_DEBUG(&trace,"McBSP peripheral not selected");
            retCode = 0;
        }
    }
    else
    {
        /* McASP select */

        val1 = MCASP_SELECT_MASK;
        retCode  = i2c_write(MCASP_MCBSP_IO_EXPANDER, (Uint8*)&val1, 1);

        i2c_read(MCASP_MCBSP_IO_EXPANDER, (Uint8*)&val2, 1);

        if (val1 != val2)
        {
            AIC33_DEBUG(&trace,"McASP peripheral not selected");
            retCode = 0;
        }

    }

    i2c_close();

    return (retCode);
}
/**
  *  Set up the I2C controller in order to communicate with the aic33 codec slave
  *
 **/
static Int i2c_setup(void)
{
    Int32            status    = 1;
    PAL_OsSemAttrs   semAttrs  = PAL_OSSEM_ATTRS;

    if (NULL == i2cHandle)
    {
        /* Create Completion internal Semaphore                               */
		if (PAL_osSemCreate(
		        "I2C Completion Sem",
		        0,
		        &semAttrs,
		        &(aic33handle.completionSem)) != PAL_SOK)
		{
		    status = 0;
		}

		/* we will create the I2c instance if we can create the semaphore     */
		if (0 != status)
        {
		    i2cHandle = PSP_i2cOpen(
		                    I2C_INSTANCE_TO_AIC33,
		                    &aic31CallBack,
		                    &(aic33handle.completionSem));

			if (NULL == i2cHandle)
			{
			    AIC33_DEBUG(&trace,"\r\nAIC33:aic33codec i2c Create failed\r\n");
			    status = 0;
            }
        }
    }
    return (status);
}

/*
 *  Write the specified number of bytes to the spacified slave device.
 *   - Shall pass the address of the slave, pointer to the buffer and number of bytes to write
 *
**/
static Int i2c_write(Uint16 slaveaddr, Uint8* data, Uint16 len)
{
    PSP_i2cDataParam     buf;
    PSP_I2cRequest      IoRequest;
    Int                 retCode = -1;

    buf.i2cTrans.buffer    = data;
    buf.i2cTrans.bufLen    = len;
    buf.i2cTrans.flags     = PSP_I2C_DEFAULT_WRITE | PSP_I2C_RESTART;
    buf.i2cTrans.param     = NULL;
    buf.i2cTrans.slaveAddr = slaveaddr;
    buf.timeout     = -1;


     /* Update dataparams */
    IoRequest.dataparam  = &buf;
    IoRequest.actualSize = 0;
    IoRequest.appCbk     = NULL;
    IoRequest.status     = PSP_SOK;

    if (NULL != i2cHandle)
    {
        retCode = PSP_i2cTransferIOP(i2cHandle,&IoRequest,(Ptr)NULL);
    }

    if (PSP_SINPROGRESS == retCode)
    {
        if( (PAL_SOK != PAL_osSemTake(aic33handle.completionSem, -1)) )
        {
            AIC33_DEBUG(&trace, "\r\nAIC33  : Error in  PAL_osSemTake");
			retCode = -1;
        }
		else
		{
 		    retCode = 0;
 		}
    }

    if ((retCode < 0) )
    {
        AIC33_DEBUG(&trace, "\r\nAIC33  : Error writing the codec register : %x", data[0]);
    }

    return (retCode);
}

/*
 *  Read the specified number of bytes to the spacified slave device.
 *   - Shall pass the address of the slave, pointer to the buffer and number of bytes to write
 *
**/
static Int i2c_read(Uint16 slaveaddr, Uint8* data, Uint16 len)
{
    Int retCode = -1;

    PSP_i2cDataParam     buf;
    PSP_I2cRequest      IoRequest;

    buf.i2cTrans.buffer    = data;
    buf.i2cTrans.bufLen    = len;
    buf.i2cTrans.flags     = PSP_I2C_DEFAULT_READ;
    buf.i2cTrans.param     = NULL;
    buf.i2cTrans.slaveAddr = slaveaddr;
    buf.timeout     = -1;


     /* Update dataparams */
    IoRequest.dataparam  = &buf;
    IoRequest.actualSize = 0;
    IoRequest.appCbk     = NULL;
    IoRequest.status     = PSP_SOK;

    if (NULL != i2cHandle)
    {
    retCode = PSP_i2cTransferIOP( i2cHandle,&IoRequest,(Ptr)NULL);
    }

    if (PSP_SINPROGRESS == retCode)
    {
        if ((PAL_SOK != PAL_osSemTake(aic33handle.completionSem, -1)))
        {
            AIC33_DEBUG(&trace, "\r\nAIC33  : Error in  PAL_osSemTake");
		    retCode = -1;
        }
		else
		{
 		    retCode = 0;
 		}
    }

    if ((retCode < 0))
    {
        AIC33_DEBUG(&trace, "\r\nAIC33  : Error reading the codec register : %x", data[0]);
    }

    return (retCode);
}


/*
 *  Stop the I2C controller from use.
 *
**/

static void i2c_close()
{
    if (NULL != i2cHandle)
    {
	    /* Delete Completion Semaphore  */
        PSP_i2cClose(i2cHandle);
		PAL_osSemDelete(aic33handle.completionSem);
        i2cHandle = NULL;
		aic33handle.completionSem = NULL;
    }
}
/**
 *  \fn      static Void aic31CallBack ()
 *
 *  \brief   This function is the codec call back function invoked from the I2c
 *           driver on transfer completion.
 *
 *  \param   semHandle [IN] pointer to the Semaphore Handle
 *
 *  \return  none
 */
static Void aic31CallBack (Ptr Handle,Ptr arg, Ptr ioRequest)
{
    arg = arg;
    ioRequest = ioRequest;
	Handle    = Handle;
    if( (PAL_SOK != PAL_osSemGive(aic33handle.completionSem)) )
    {
        AIC33_DEBUG(&trace, "\r\nAIC33  : Error in PAL_osSemGive");
    }
    return;
}


#ifdef PLL_INPUT_CLK
/*
 *  \brief  This function configures the IO expander so that the PLL can be
 *          configured to output the required sample frequencies.
 *
 *          The values written to the IO expander configure the PLL so that it
 *          can output different frequencies.
 *
 *  \param  samplingRate   - sampling rate to be output by the codec.
 *
 *  \return   0 - in case that the PLL is configured sucessfully else
 *           -1 - in case that the PLL configuration has failed.
 */

Int32 audio_PllConfigure(Uint32 samplingRate)
{
    Uint32  pllVal  = 0;
    Uint32  tempVal = 0;
    Bool    Fs1     = 0;   /* Fs1 pin of PLL */
    Bool    Fs2     = 0;   /* Fs2 pin of PLL */
    Bool    Sr      = 0;   /* Sr  pin of PLL */
    Int32   retCode = -1;

    /* set up the I2c bus,return in case the setup has failed                 */
    if (!i2c_setup ())
    {
        return -1;
    }

    /* read the current settings of the IO expander                           */
    i2c_read(PLL_IO_EXPANDER_ADDRESS, (Uint8*)&tempVal, 1);

    /* clear the values of the  pins to be configured                         */
    pllVal = tempVal;
    pllVal &= ~(1 << IO_EXP_PLL1705_SR_PORT);
    pllVal &= ~(1 << IO_EXP_PLL1705_FS1_PORT);
    pllVal &= ~(1 << IO_EXP_PLL1705_FS2_PORT);

    /* confgiure the required sampling rate                                   */
    switch (samplingRate)
    {
        /* we will use the 44.1,44,and 96 as reference frequencies and then   *
         * divide internally in the Codec to generate the required sample rate*/
        /* sampling rate of 11.025KHz,22.050KHz,44.1KHz  */
        case 11025:
        case 22050:
        case 44100:
            Fs1 = 1;
            Fs2 = 0;
            Sr  = 0;
            break;
        /* sampling rate of 8KHz,16KHz,32KHz,48KHz       */
        case 8000:
        case 12000:
        case 16000:
        case 24000:
        case 32000:
        case 48000:
        case 96000:
            Fs1 = 0;
            Fs2 = 0;
            Sr  = 0;
            break;
        default:
            retCode = -1;
            break;
    }

    pllVal |= ((Fs1 << IO_EXP_PLL1705_FS1_PORT)
              | (Fs2 << IO_EXP_PLL1705_FS2_PORT)
              | (Sr << IO_EXP_PLL1705_SR_PORT));

    /* write the value to the PLL IO expander                                 */
    retCode  = i2c_write(PLL_IO_EXPANDER_ADDRESS, (Uint8*)&pllVal, 1);

    /* read the value from the IO expander and check if the value is written  *
     * properly                                                               */
    i2c_read(PLL_IO_EXPANDER_ADDRESS, (Uint8*)&tempVal, 1);

    /* close the I2c Bus                                                      */
    i2c_close();

    if (pllVal != tempVal)
    {
        retCode = -1;
    }

    return retCode;
}
#endif


/*
 *  \brief  This function provides the interface to write and read from the
 *          codec registers.
 *
 *  \param  cmd      [IN]     Read or the write command
 *  \param  regData  [IN/OUT] register data to be written read
 *
 *  \return   0 - in case that the command is successfully else
 *           -1 - in case that the command has failed.
 */
Int32 aic33ReadWriteIoctl(Uint32 cmd,aic33_RegData *regData)
{
    PSP_i2cDataParam    transBuf;
    PSP_I2cRequest      ioRequest;
    Int32               retCode       = -1;
    Uint8               tempBuffer[2] = {0};

    /* validate the input parameters and also configure the i2c bus           */
    if ((NULL != regData) && (0 != i2c_setup()))
    {
        /* check whether the command is to read the register or to write to   *
         * the codec register                                                 */
        if (AIC33_READ_CMD == cmd)
        {
            /* The following the procedure to read data from Aic33 codec

               Step A: Write the address of the register to be read.
                       Note: The I2C stop should not be issued.
               Step B: Issue I2C restart.
               Step C: Read the value of the register.                        */
            transBuf.i2cTrans.slaveAddr = I2C_AIC33_SLAVE_ADDR;
            transBuf.i2cTrans.buffer = &regData->regIndex;
            transBuf.i2cTrans.bufLen = 1;
            transBuf.i2cTrans.flags = PSP_I2C_WRITE | PSP_I2C_MASTER
                                      | PSP_I2C_START;

            ioRequest.dataparam  = &transBuf;
            ioRequest.actualSize = 0;
            ioRequest.appCbk     = NULL;
            ioRequest.status     = PSP_SOK;

            retCode = PSP_i2cTransferIOP(i2cHandle,&ioRequest,(Ptr)NULL);

            if (PSP_SINPROGRESS == retCode)
            {
                if (PSP_SOK == PAL_osSemTake(aic33handle.completionSem, -1))
                {
                    retCode = 0;
                }
                else
                {
                    retCode = -1;
                }
            }

            if (0 == retCode)
            {

                transBuf.i2cTrans.flags = PSP_I2C_READ | PSP_I2C_MASTER
                                          | PSP_I2C_STOP | PSP_I2C_RESTART;

                transBuf.i2cTrans.buffer = &regData->data;
                ioRequest.actualSize = 0;

                retCode = PSP_i2cTransferIOP(i2cHandle,&ioRequest,(Ptr)NULL);

                if (PSP_SINPROGRESS == retCode)
                {
                    if (PAL_SOK == PAL_osSemTake(aic33handle.completionSem,-1))
                    {
                        retCode = 0;
                    }
                    else
                    {
                        retCode = -1;
                    }
                }
            }
        }
        else if (AIC33_WRITE_CMD == cmd)
        {
            /* command issued is write command   */
            /* we have only one code and the transfer length is always 2      */
            tempBuffer[0] = regData->regIndex;   /* Device Address     */
            tempBuffer[1] = regData->data;       /* Register Data      */
            retCode = i2c_write(I2C_AIC33_SLAVE_ADDR,&tempBuffer[0],2);
        }
        else
        {
            retCode = -1;
        }

        /* close the opened i2c driver instance                               */
        i2c_close();
    }

    /* return the status of the command complaetion                           */
    return retCode;
}


/*============================================================================*/
/*                              END OF FILE                                   */
/*============================================================================*/
