/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/** \file   psp_bios_i2c_sample_led.c
 *
 *   \brief  This file contains sample code to demonstrate the
 *          I2C driver implemented for DSP/BIOS
 *
 *   This file performs tests on the following:
 *
 *      1. LED Blinking Test
 *
 * (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author     Maulik Desai
 *  \version    0.1
 */

/*
 *  HEADER FILE INCLUSION
 */
#include <std.h>
#include <gio.h>
#include <tsk.h>
#include <ti/sdo/pspdrivers/drivers/i2c/psp_i2c.h>
#include <ti/sdo/pspdrivers/pal_os/bios/psputils.h>

/*
 * This application demonstrates the usage of GIO based I2C driver
 * of DM6437/C6424.This application opens the driver instance, demonstrates
 * usage of I2C driver by writing proper values to the LED to blink the LEDs
 * and closes the driver instance.
 */

/*
 *    MACROS DEFINED
 */
#define  I2C_DEBUG               PSP_DEBUG
#define  I2C_LED_SLAVE_ADDR      (0x39u)
#define  PSP_I2C_ASYNC_MODE_SUPPORT 1
/*
 * I2C Driver handle
 */
static GIO_Handle      i2cHandle = NULL;

/* Function prototype */

static void blinkLED();
static void Blinkdelay();
#ifdef PSP_I2C_ASYNC_MODE_SUPPORT
void BlinkLedCalback(Ptr arg,Int status, Ptr bufp, size_t size);
#endif
/**
 *  ======== I2C_INIT ========
 *  This function is used to Initialize the I2C Bus Driver
 *
 */
void I2C_INIT()
{
    return;
}

/*! \fn start_i2c_sample
 * \brief Function to illustrate usage of I2C driver.
 *
 *  This function will illustrate the usage of I2C driver by blinking LEDs.
 *
 *  \param None
 */
void start_i2c_sample()
{
    GIO_Attrs gioAttrs = GIO_ATTRS;

    I2C_DEBUG("\r\nI2C  :Start of I2C sample application\r\n");

    /*
     * Create I2C Channel
     */
    i2cHandle = GIO_create("/I2C0",IOM_INOUT,NULL,NULL,&gioAttrs);
    if(NULL == i2cHandle)
    {
       I2C_DEBUG("I2C : I2C Create Failed.......! \r\n");
    }
    else
    {
        /*
         * Call function to blink LEDs
         */
         blinkLED();
    }

    /*
     * Delete the I2C Handle
     */
    GIO_delete(i2cHandle);
    i2cHandle   =   NULL;
    I2C_DEBUG("I2C  :End of I2C sample application\r\n");
}


/*! \fn blinkLED
 *  \brief Function will blink LEDs on EVMDM6437/EVM6424.
 *
 *   This function will blink the LEDs on EVMDM6437/EVM6424 for some time.
 *
 *  \param None
 */
static void blinkLED()
{
   PSP_i2cDataParam     buf;
   size_t             size;
   Uint32 count    =  0;
   Int    retCode  =  0;
   Int buffer;

#ifdef PSP_I2C_ASYNC_MODE_SUPPORT
   GIO_AppCallback BlinkledCbk;

#endif

   /* Delay for other I2C Transfers to complete during INIT */
   TSK_sleep(2000u);

   I2C_DEBUG("I2C : Start of LED Blink demo \r\n");
   I2C_DEBUG("I2C : LED blinks for 5 seconds \r\n");

   for(count =0; count <20; count++)
   {
       /*
        * Value to be written to toggle the LEDs
        */
        buffer                 = 0x05;
        buf.i2cTrans.buffer    = (Uint8 *)&buffer;
        buf.i2cTrans.bufLen    = 1u;
        buf.i2cTrans.flags     = PSP_I2C_DEFAULT_WRITE | PSP_I2C_RESTART;
        buf.i2cTrans.param     = NULL;
        buf.i2cTrans.slaveAddr = I2C_LED_SLAVE_ADDR;
        buf.timeout            = -1;
        size                   = (size_t)buf.i2cTrans.bufLen;

#ifdef PSP_I2C_ASYNC_MODE_SUPPORT

        /* Call the GIO_SUBMIT with the call back ..i.e async mode */

        BlinkledCbk.fxn = BlinkLedCalback;
        BlinkledCbk.arg = NULL;
        retCode = GIO_submit(i2cHandle,IOM_WRITE,&buf,&size,&BlinkledCbk);
#else

        /* Call the GIO_SUBMIT with NULL in the call back argument .
            .i.e sync mode */
        retCode = GIO_submit(i2cHandle,IOM_WRITE,&buf,&size,NULL);
#endif
        if(retCode < 0)
        {
            I2C_DEBUG("LED Operation Failed\n");
        }

        Blinkdelay();

        buffer                 = 0x0A;
        buf.i2cTrans.buffer    = (Uint8 *)&buffer;
        buf.i2cTrans.bufLen    = 1u;
        buf.i2cTrans.flags     = PSP_I2C_DEFAULT_WRITE | PSP_I2C_RESTART;
        buf.i2cTrans.param     = NULL;
        buf.i2cTrans.slaveAddr = I2C_LED_SLAVE_ADDR;
        buf.timeout            = -1;
        size                   = (size_t)buf.i2cTrans.bufLen;

#ifdef PSP_I2C_ASYNC_MODE_SUPPORT

        /* Call the GIO_SUBMIT with the call back ..i.e async mode */

        BlinkledCbk.fxn = BlinkLedCalback;
        BlinkledCbk.arg = NULL;
        retCode = GIO_submit(i2cHandle,IOM_WRITE,&buf,&size,&BlinkledCbk);
#else

        /* Call the GIO_SUBMIT with NULL in the call back argument .
            .i.e sync mode */
        retCode = GIO_submit(i2cHandle,IOM_WRITE,&buf,&size,NULL);
#endif
        if(retCode < 0)
        {
            I2C_DEBUG("LED Operation Failed\n");
        }

        Blinkdelay();

    }

    I2C_DEBUG("I2C : End of LED Blink demo \r\n");
}


/*! \fn BlinkLedCalback
 *  \brief Call Back function Used in async mode.
 *
 *
 *  \param None
 */

void BlinkLedCalback(Ptr arg,Int status, Ptr bufp, size_t size)
{
    /*To avoid compiler warnings*/
    arg = arg;
    bufp = bufp;
    size = size;
    status = status;

    return;

}

/*! \fn Blinkdelay
 *  \brief Function used for delay.
 *
 *
 *  \param None
 */


static void Blinkdelay()
{
  volatile Uint32 i;

  for(i=0; i<0x2FFFFF ;i++)
    ;
  for(i=0; i<0x2FFFFF ;i++)
    ;
}

