/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/** \file   psp_bios_nand_sample.c
 *
 *  \brief  This file contains the demo application to demonstrate the
 *          NAND driver implemented using the PSP Framework Architecture
 *
 *  This file tests the PSP open, * init, close , read sync and write
 *  sync functionalities.
 *
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author     PSP Team
 *  \version    1.0         Created
 *
 */

#include <std.h>
#include <stdlib.h>
#include <string.h>
#include <ti/sdo/pspdrivers/drivers/nand/psp_nand.h>
#include <log.h>

/*
 * Log Handle
 */
extern far LOG_Obj trace;

/**< Size of the buffer                                                     */
#define APP_NAND_BUF_SIZE           (512u)

#define READ_WRITE_STARTING_SECTOR  (100u)

static PSP_Handle          hNand;
/**< Handle to the NAND device                                              */

/* Function Prototypes */
void    initNand(Uint32);
void    nandTestCode( Uint32 nandClkFrequency );

static Bool sectorReadWriteTest(Uint32 sectorNo, Uint32 sectorSize);

/* Write Buffer and Read Buffer addresses are made 128 byte aligned
 * to support CACHE Operations in DMA mode of operation
 */
#pragma DATA_ALIGN(RData, 128);
static Uint16   RData[(APP_NAND_BUF_SIZE/(2u))]; /* Buffer is Uint16 NOT Uint8 */

#pragma DATA_ALIGN(WData, 128);
static Uint16   WData[(APP_NAND_BUF_SIZE/(2u))]; /* Buffer is Uint16 NOT Uint8 */

static PSP_NandType        nandType = PSP_NAND_NONE;
/**< Type of NAND device                                                */

/**
 *  \brief Initialize NAND device.
 *
 *  This API does the NAND driver initialization and NAND device initialization.
 *
 *  \return None
 */
void initNand(Uint32 nandClkFreq)
{
    PSP_Result   result = PSP_SOK;
    PSP_NandConfig   config;

    /* Note: The behaviour of the driver is same when configured in
             PSP_OPMODE_DMAINTERRUPT or PSP_OPMODE_DMA
     */
    config.opMode       = PSP_OPMODE_DMAINTERRUPT;

    config.inputClkFreq = nandClkFreq;
    config.eraseAtInit  = FALSE; /* <CAUTION>: TRUE-erases all NAND media before use!*/

    /* Initialize the driver */
    result = PSP_nandInit(0, &config);

    if (PAL_SOK != result)
    {
        LOG_printf(&trace,"\nNAND: Driver Intialization... failed. [Result = %x]\n", result);
    }

    /* Open the driver */
    hNand = PSP_nandOpen(0);
    if (NULL == hNand)
    {
        LOG_printf(&trace,"\nNAND: Open failed. [Handle = %x]\n", hNand);
    }

    /* Check if the NAND medium is present */
    result = PSP_nandIoctl(hNand,
                           PSP_NAND_IOCTL_CHECK_NAND,
                           (void*)&nandType,
                           0);

    if (PSP_SOK != result) {
        LOG_printf(&trace,"\nNAND  :Device Detection failed. [Result = 0x%x]\n", result);
    }
    else
    {
        LOG_printf(&trace,"NAND  :Device Detection successfull\n");
    }

    return;
}


/**
 *  \brief Starts NAND Tests.
 *
 *  This API does the actual testing of NAND. It initialises the nand driver and
 *  tests are done at sector level.
 *
 *  The 'Sector Level Test' tests the raw sector level read and write capabilities.
 *
 *  \r\neturn None
 */
/* ARGSUSED */
void nandTestCode( Uint32 nandClkFreq )
{
    PSP_Result  status      = PSP_SOK;
    Uint32      sectorNo    = 0;
    Uint32      sectorSize  = 0;
    Bool        result      = FALSE;

    if (NULL == hNand)
    {
        return;
    }
    sectorNo = READ_WRITE_STARTING_SECTOR; /* starting sector number on which
                                               read write operation is carried out */

    /* Get the sector size */
    result = PSP_nandIoctl(hNand,
                           PSP_NAND_IOCTL_GET_SECTOR_SIZE,
                           (void*)&sectorSize,
                           0);
    if(PSP_SOK == status)
    {
        /* ReadWriteTest called for sectorNo  */
        result = sectorReadWriteTest(sectorNo, sectorSize);
        if (TRUE == result)
        {
            LOG_printf(&trace,"NAND  :Sector Read/Write Passed.\n");
        }
        else
        {
            LOG_printf(&trace,"\nNAND  :Sector Read/Write Failed.\n");
        }
        /* Nand device is closed after read write operation completes */
        status = PSP_nandClose(hNand);
        if (PSP_SOK != status )
        {
            LOG_printf(&trace,"\nNAND  :PSP_nandClose Failed.\n");
        }

        LOG_printf(&trace,"NAND  :Sector Read/Write Test Complete \n");
    }
    else
    {
        LOG_printf(&trace,"\nNAND  :Error Unable to get sector size\n");
    }

    return;
}

/**
 *  \brief Sector test for NAND.
 *
 *  This API writes and reads a specified sector of NAND.
 *  This test exercises the low level write of NAND.
 *
 *  \r\neturn TRUE if success, FALSE otherwise.
 */
Bool sectorReadWriteTest(Uint32 sectorNo, Uint32 sectorSize)
{
    PSP_Result  result;
    Int32       xferActual = 0;
    Uint32      Sectorcount;
    Uint32      bufIndex;
    Bool Erase = (1u);  /* If set to 1, then acts as a command to erase the nand device */

    if (1 == Erase)
    {
        result =  PSP_nandErase(hNand); /* Erases all the sectors except protected blocks */

        if (PSP_SOK != result)
        {
            LOG_printf(&trace,"\nNAND  :Driver Error Status = %d\n", result);
            return FALSE;
        }
        else
        {
            LOG_printf(&trace,"NAND  :NAND Flash erase Success.\n");
        }
    }

    for (Sectorcount = 0; Sectorcount < (10u); Sectorcount++)
    {
        /* WData and RData data initialized */
        for (bufIndex = 0; bufIndex < (APP_NAND_BUF_SIZE/(2u)); bufIndex++)
        {
            WData[bufIndex] = ((0xAAu) << (8u)) | bufIndex;
            RData [bufIndex] = 0x0;
        }

        /* Data written to the logical sector (sectorNo + Sectorcount) */
        result = PSP_nandWriteSync( hNand,
                                    sectorNo + Sectorcount,
                                    WData,
                                    (APP_NAND_BUF_SIZE/sectorSize),
                                    &xferActual);
        if (PSP_SOK != result)
        {
            LOG_printf(&trace,"\nNAND  :Driver Error Status = %d\n", result);
            break;
        }
        /* Data read from the logical sector (sectorNo + Sectorcount) */
        result = PSP_nandReadSync(  hNand,
                                    sectorNo + Sectorcount,
                                    RData,
                                    (APP_NAND_BUF_SIZE/sectorSize),
                                    &xferActual);
        if (PSP_SOK != result)
        {
            LOG_printf(&trace,"\nNAND  :Driver Error Status = %d\n", result);
            break;
        }

       /* Compare whether the read data match with the writen data */
        if ((memcmp (WData, RData, sectorSize)) == 0)
        {
            LOG_printf(&trace,"NAND  :Sector %d Write/Read: Buffer compare Success.\n", sectorNo+Sectorcount);
        }
        else
        {
            LOG_printf(&trace,"\nNAND  :Sector %d Write/Read: Write buffer and read buffer don't match!\n",sectorNo+Sectorcount);
            break;
        }
    }

    if(PSP_SOK != result)
    {
        return (Bool) FALSE;
    }
    else
    {
        return (Bool) TRUE;
    }
}
