/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/*
 * \file   aic23Codec.c
 *
 * \brief  AIC23 audio codec configuration file
 *
 * This file contains the API used for configuring the audio codec
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \note       Set tabstop to 4 (:se ts=4) while viewing this file in an
 *              editor
 *
 *  \author
 *
 *  \version    0.1`
 *
 */

#include <std.h>
#include <log.h>

#include <ti/sdo/pspdrivers/common/psp_common.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osWait.h>
#include <ti/sdo/pspdrivers/pal_os/bios/psputils.h>

#ifdef CHIP_DM648
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/audio/src/psp_aic33Codec.h>
#endif

#ifdef CHIP_C6452
#include <ti/sdo/pspdrivers/system/c6452/bios/evm6452/audio/src/psp_aic33Codec.h>
#endif

#include <ti/sdo/pspdrivers/drivers/i2c/psp_i2c.h>
#include <ti/sdo/pspdrivers/drivers/i2c/src/psp_i2cApi.h>

#define AIC33_DEBUG                 LOG_printf
#define I2C_INSTANCE_TO_AIC33       (0)


#define INPUT_GAIN_MIN            LIG_MIN
#define INPUT_GAIN_MAX            LIG_MAX
#define INPUT_GAIN_RANGE          (INPUT_GAIN_MAX - INPUT_GAIN_MIN)

#define OUTPUT_GAIN_MIN           LOG_MIN
#define OUTPUT_GAIN_MAX           LOG_MAX
#define OUTPUT_GAIN_RANGE         (INPUT_GAIN_MAX - INPUT_GAIN_MIN)

extern far LOG_Obj trace;

Uint16 aic33SlaveAddr;
static Int gCount;
static aic33_CodecHandle aic33handle = {0};

/** Read/Write the AIC33 registers */
static Int aic33_rset      ( aic33_CodecHandle aic33handle1, Uint16 regnum, Uint16 regval );

/** Write the data to the I2C controller's DXR  */
static Int     i2c_write       (Uint16 slaveaddr, Uint8* data, Uint16 len );


/** Read the data from the I2C controller's DRR */
static void    i2c_close       (void);
static Int     i2c_setup       (void);
static Void aic31CallBack (Ptr semHandle,Ptr arg, Ptr ioRequest);

static PSP_Handle i2cHandle = NULL;

/* *
 * AIC33_rset( )
 *     Set codec register regnum to value regval
 *
 **/
/* ARGSUSED */
Int aic33_rset( aic33_CodecHandle aic33handle, Uint16 regnum, Uint16 regval )
{
    Int retStat;
    Uint8 cmd[2];

    /* To avoid compiler warning */
    aic33handle = aic33handle;

    cmd[0] = (Uint8) regnum;            /* 7-bit Device Address */
    cmd[1] = (Uint8) regval;            /* 8-bit Register Data  */

    retStat = i2c_write( aic33SlaveAddr, cmd, 2 );
    gCount++;

    return retStat;
}

/**
 *  aic33_sampleRateControl( PSP_audio_cfg *audioCfg )
 *  Configure the sampling rate of the Audio Codec
 **/
Int aic33_sampleRateControl ( PSP_audio_cfg *audioCfg )
{
    Uint16 PLLB_Value       = 0;
    Uint16 PLLC_Value       = 0;
    Uint16 PLLD_Value       = 0;
    Uint16 ADC_DualRate     = ADC_DUALRATE_DISABLE;
    Uint16 DAC_DualRate     = DAC_DUALRATE_DISABLE;
    Bool   closeRequired    = FALSE;
    Uint16  sampleRateDiv    = 2u;
    Uint8  fsRef            = FSREF_44100;
    Int    retCode          = 0;

    if(NULL != audioCfg)
    {
        /* Open only when this function is called seperately */
        if(NULL == i2cHandle)
        {
            i2c_setup ( );
            closeRequired = TRUE;
        }

        /** Configure the codec to generate 44.1KHz */
        /**  Fs == Fsref
         *   AIC33_MCLK = 11.66 MHz
         *   Fsref = ( AIC33_MCLK * R * K ) / ( 2048 * P )
         *   441000Hz = (11.66 MHz * R * K) /(2048 * P)
         *   for P = 2 => K = J . D = 15.4917 ; J = 15, D = 4917
         *   Reset value of R = 1
         **/

        switch(audioCfg->sampleFreq)
        {
            case 7350:  /* FS = Fref/6   */
                PLLB_Value      = 0x1C;
                PLLC_Value      = 0x74;
                PLLD_Value      = 0x8C;
                fsRef           = FSREF_44100;
                sampleRateDiv   = 0x0A;
                break;

            case 8018:  /* FS = Fref/5.5 */
                PLLB_Value      = 0x1C;
                PLLC_Value      = 0x74;
                PLLD_Value      = 0x8C;
                fsRef           = FSREF_44100;
                sampleRateDiv   = 0x09;
                break;

            case 8820:  /* FS = Fref/5   */
                PLLB_Value      = 0x1C;
                PLLC_Value      = 0x74;
                PLLD_Value      = 0x8C;
                fsRef           = FSREF_44100;
                sampleRateDiv   = 0x08;
                break;

            case 9800:  /* FS = Fref/4.5 */
                PLLB_Value      = 0x1C;
                PLLC_Value      = 0x74;
                PLLD_Value      = 0x8C;
                fsRef           = FSREF_44100;
                sampleRateDiv   = 0x07;
                break;

            case 11025: /* FS = Fref/4   */
                PLLB_Value      = 0x1C;
                PLLC_Value      = 0x74;
                PLLD_Value      = 0x8C;
                fsRef           = FSREF_44100;
                sampleRateDiv   = 0x06;
                break;

            case 12600: /* FS = Fref/3.5 */
                PLLB_Value      = 0x1C;
                PLLC_Value      = 0x74;
                PLLD_Value      = 0x8C;
                fsRef           = FSREF_44100;
                sampleRateDiv   = 0x05;
                break;

            case 14700: /* FS = Fref/3   */
                PLLB_Value      = 0x1C;
                PLLC_Value      = 0x74;
                PLLD_Value      = 0x8C;
                fsRef           = FSREF_44100;
                sampleRateDiv   = 0x04;
                break;

            case 17640: /* FS = Fref/2.5 */
                PLLB_Value      = 0x1C;
                PLLC_Value      = 0x74;
                PLLD_Value      = 0x8C;
                fsRef           = FSREF_44100;
                sampleRateDiv   = 0x03;
                break;

            case 22050: /* FS = Fref/2   */
                PLLB_Value      = 0x1C;
                PLLC_Value      = 0x74;
                PLLD_Value      = 0x8C;
                fsRef           = FSREF_44100;
                sampleRateDiv   = 0x02;
                break;

            case 29400: /* FS = Fref/1.5 */
                PLLB_Value      = 0x1C;
                PLLC_Value      = 0x74;
                PLLD_Value      = 0x8C;
                fsRef           = FSREF_44100;
                sampleRateDiv   = 0x01;
                break;

            case 44100: /* FS = Fref/1   */
                PLLB_Value      = 0x1C;
                PLLC_Value      = 0x74;
                PLLD_Value      = 0x8C;
                fsRef           = FSREF_44100;
                sampleRateDiv   = 0x00;
                break;

            case 8000:  /* FS = Fref/6   */
                PLLB_Value      = 0x20;
                PLLC_Value      = 0x43;
                PLLD_Value      = 0x54;
                fsRef           = FSREF_48000;
                sampleRateDiv   = 0x0A;
                break;

            case 8727:  /* FS = Fref/5.5 */
                PLLB_Value      = 0x20;
                PLLC_Value      = 0x43;
                PLLD_Value      = 0x54;
                fsRef           = FSREF_48000;
                sampleRateDiv   = 0x09;
                break;

            case 9600:  /* FS = Fref/5   */
                PLLB_Value      = 0x20;
                PLLC_Value      = 0x43;
                PLLD_Value      = 0x54;
                fsRef           = FSREF_48000;
                sampleRateDiv   = 0x08;
                break;

            case 10667: /* FS = Fref/4.5 */
                PLLB_Value      = 0x20;
                PLLC_Value      = 0x43;
                PLLD_Value      = 0x54;
                fsRef           = FSREF_48000;
                sampleRateDiv   = 0x07;
                break;

            case 12000: /* FS = Fref/4   */
                PLLB_Value      = 0x20;
                PLLC_Value      = 0x43;
                PLLD_Value      = 0x54;
                fsRef           = FSREF_48000;
                sampleRateDiv   = 0x06;
                break;

            case 13714: /* FS = Fref/3.5 */
                PLLB_Value      = 0x20;
                PLLC_Value      = 0x43;
                PLLD_Value      = 0x54;
                fsRef           = FSREF_48000;
                sampleRateDiv   = 0x05;
                break;

            case 16000: /* FS = Fref/3   */
                PLLB_Value      = 0x20;
                PLLC_Value      = 0x43;
                PLLD_Value      = 0x54;
                fsRef           = FSREF_48000;
                sampleRateDiv   = 0x04;
                break;

            case 19200: /* FS = Fref/2.5 */
                PLLB_Value      = 0x20;
                PLLC_Value      = 0x43;
                PLLD_Value      = 0x54;
                fsRef           = FSREF_48000;
                sampleRateDiv   = 0x03;
                break;

            case 24000: /* FS = Fref/2   */
                PLLB_Value      = 0x20;
                PLLC_Value      = 0x43;
                PLLD_Value      = 0x54;
                fsRef           = FSREF_48000;
                sampleRateDiv   = 0x02;
                break;

            case 32000: /* FS = Fref/1.5 */
                PLLB_Value      = 0x20;
                PLLC_Value      = 0x43;
                PLLD_Value      = 0x54;
                fsRef           = FSREF_48000;
                sampleRateDiv   = 0x01;
                break;

            case 48000: /* FS = Fref/1   */
                PLLB_Value      = 0x20;
                PLLC_Value      = 0x43;
                PLLD_Value      = 0x54;
                fsRef           = FSREF_48000;
                sampleRateDiv   = 0x00;
                break;

            case 88200: /* FS = Fref/1   */
                PLLB_Value      = 0x1C;
                PLLC_Value      = 0x74;
                PLLD_Value      = 0x8C;
                fsRef           = FSREF_44100;
                sampleRateDiv   = 0x00;
                ADC_DualRate    = ADC_DUALRATE_ENABLE;
                DAC_DualRate    = DAC_DUALRATE_ENABLE;
                break;

            case 96000: /* FS = Fref/1   */
                PLLB_Value      = 0x20;
                PLLC_Value      = 0x43;
                PLLD_Value      = 0x54;
                fsRef           = FSREF_48000;
                sampleRateDiv   = 0x00;
                ADC_DualRate    = ADC_DUALRATE_ENABLE;
                DAC_DualRate    = DAC_DUALRATE_ENABLE;
                break;

            default:   /* Default to 44.1 KHz */
                retCode         = -1;
                break;
        }

        if (-1 != retCode)
        {
            /* Copy sample rate div value for Input & Output */
            sampleRateDiv = (Uint16)(((Uint32)sampleRateDiv << 4) | (Uint32)sampleRateDiv);

            /** Select ADC Fs == Fsref and DAC Fs == Fsref */
            aic33_rset(aic33handle, AIC33_CODEC_SAMPLE_RATE_ADDR, sampleRateDiv);
            aic33_rset(aic33handle, AIC33_PLLA_PROG_ADDR,0x0091); /**  [PLL=ON][P=2][Q=2] */

            aic33_rset(aic33handle, AIC33_PLLB_PROG_ADDR,PLLB_Value); /** [J] */
            aic33_rset(aic33handle, AIC33_PLLC_PROG_ADDR,PLLC_Value); /** [D MSB] */
            aic33_rset(aic33handle, AIC33_PLLD_PROG_ADDR,PLLD_Value); /** [D LSB] */
            aic33_rset(aic33handle, AIC33_CODEC_DATAPATH_SETUP_ADDR, (Uint16)(fsRef |
                                                  ADC_DualRate |
                                                  DAC_DualRate |
                                                  ((Uint16)LEFT_DAC_PATH_PLAYS_LEFTDATA<<3) |
                                                  ((Uint16)RIGHT_DAC_PATH_PLAYS_RIGHTDATA <<1)));

        }

        /* Close only when this function did a I2C Open */
        if(TRUE == closeRequired)
        {
            i2c_close();
        }
    }
    else
    {
        retCode = PSP_E_INVAL_PARAM;
    }

    return (retCode);
}

/**
 *  aic33Codec_config( Uint32 id, PSP_audio_cfg *audioCfg )
 *     Configure the codec
 **/
/* ARGSUSED */
Int aic33Codec_config( Uint32 id, PSP_audio_cfg *audioCfg )
{

    Int    retCode        = 0;
    Uint16 gain           = 0;


    if(NULL != audioCfg)
    {
        retCode = i2c_setup ( );
        if (0 == retCode)
        {
            return -1;
        }

        /* checking for master/slave mode configuration */
        if((PSP_AUDIO_AIC_MASTER != audioCfg->mode)
           && (PSP_AUDIO_AIC_SLAVE != audioCfg->mode))
        {
            return -1;
        }

        /* checking for input/record mode */
        if(PSP_AUDIO_LINE_IN != audioCfg->inputMode)
        {
            return -1;
        }

        /* Checking for input gain range */
        if ((0 == audioCfg->inputGain)||(audioCfg->inputGain >= 100u))
        {
            return -1;
        }

        /* Checking for output gain range */
        if ((0 == audioCfg->outputGain)||(audioCfg->outputGain >= 100))
        {
            return -1;
        }

        aic33_rset( aic33handle, AIC33_PAGE_SELECT_ADDR, 0x00 ); /* Select Page 0 */
        aic33_rset( aic33handle, AIC33_RESET_ADDR, 0x80 );       /* Reset the AIC33 */


        /* Change the sampling rate */
        retCode = aic33_sampleRateControl ( audioCfg );
        if (0 != retCode)
        {
            return -1;
        }

        if(PSP_AUDIO_AIC_MASTER == audioCfg->mode)
        {

            /** Both bit clock and word clock are driven by codec */
            aic33_rset(aic33handle,AIC33_SERIAL_INTERFACEA_ADDR, 0x00C0);
        }
        else
        {

            /** Both bit clock and word clock are driven by the slave only (McBSP) */
            aic33_rset(aic33handle, AIC33_SERIAL_INTERFACEA_ADDR,
                          AIC33_BCLK_SLAVE | AIC33_WCLK_SLAVE);
        }

        /**  DSP-mode , 16-bit of audio word length */
        aic33_rset(
                aic33handle,
                AIC33_SERIAL_INTERFACEB_ADDR,
                (AIC33_DSP << 6)| (AIC33_WORDLEN_16 << 4)|AIC33_BCLK_CTRL_CONTINUOS
                );

        /** Data offset is zero */
        aic33_rset(aic33handle,AIC33_SERIAL_INTERFACEC_ADDR,0x0000);

        /** Configure the reset value only */
        aic33_rset(aic33handle,AIC33_CODEC_DIGIFILTER_CTRL_ADDR,0x0000);

        /** Headset Detection  is OFF */
        aic33_rset(aic33handle,AIC33_CODEC_HEADSET_DETA_ADDR,0x0000);
        aic33_rset(aic33handle,AIC33_CODEC_HEADSET_DETB_ADDR,0x0000);

        /* Mute OFF the left ADC PGA and set the gain to 1db */
        aic33_rset(aic33handle,AIC33_LEFT_ADCPGA_GAIN_CTRL_ADDR,0x0004);

        /* Mute OFF the right ADC PGA and set the gain to 1db */
        aic33_rset(aic33handle,AIC33_RIGHT_ADCPGA_GAIN_CTRL_ADDR,0x0004);


        if (PSP_AUDIO_LINE_IN == audioCfg->inputMode)
        {
            /* enable ADC's, enable line iput */
            aic33_rset(aic33handle,AIC33_LINE1L_TO_LEFT_ADC_CTRL_ADDR, 0x04);
            aic33_rset(aic33handle,AIC33_LINE1R_TO_RIGHT_ADC_CTL_ADDR, 0x04);

            /* disable the mic input */
            aic33_rset(aic33handle,AIC33_MIC3LR_TO_LEFTADC_CTRL_ADDR, 0xFF);
            aic33_rset(aic33handle,AIC33_MIC3LR_TO_RIGHTADC_CTRL_ADDR, 0xFF);

            /** Power OFF the MICBIAS control */
            aic33_rset(aic33handle,AIC33_MICBIAS_CTRL_ADDR,0x0000);
        }

        /**
        ***** ROUTING THE LINE1 SIGNAL TO ADC PGA OF THE AIC33 CODEC *****
        * Connect the LINE1L to left ADC PGA and LINE1R to Right ADC PGA
        * Set the both ADC gain to 0dB
        */

        /** don't connect LINE2L to the Right ADC PGA */
        aic33_rset(aic33handle,AIC33_LINE2L_TO_LEFT_ADC_CTRL_ADDR,0x0078);

        /** don't connect LINE1R to the Left ADC PGA */
        aic33_rset(aic33handle,AIC33_LINE1R_TO_LEFT_ADC_CTRL_ADDR,0x0078);

        /** don't connect LINE2R to the Right ADC PGA */
        aic33_rset(aic33handle,AIC33_LINE2R_TO_RIGTH_ADC_CTRL_ADDR,0x0078);

        /** don't connects LINE1L to the Right ADC PGA */
        aic33_rset(aic33handle,AIC33_LINE1L_TO_RIGTH_ADC_CTRL_ADDR,0x0078);


        /** Configure for input gain */
        gain = (Uint16)(((audioCfg->inputGain * INPUT_GAIN_RANGE) / 100u) + INPUT_GAIN_MIN);

        /* Left AGC control */
        aic33_rset(aic33handle,AIC33_LEFT_AGC_CTRLA_ADDR, 0x80);
        aic33_rset(aic33handle,AIC33_LEFT_AGC_CTRLB_ADDR, (Uint16)((Uint32)gain << 1));
        aic33_rset(aic33handle,AIC33_LEFT_AGC_CTRLC_ADDR, 0x00);

        /* Right AGC control */
        aic33_rset(aic33handle,AIC33_RIGHT_AGC_CTRLA_ADDR, 0x80);
        aic33_rset(aic33handle,AIC33_RIGHT_AGC_CTRLB_ADDR, (Uint16)((Uint32)gain << 1));
        aic33_rset(aic33handle,AIC33_RIGHT_AGC_CTRLC_ADDR, 0x00);

        aic33_rset(aic33handle,AIC33_LEFT_AGC_NOISE_GATE_DEBOUNCE_ADDR,0x0000);
        aic33_rset(aic33handle,AIC33_RIGHT_AGC_NOISE_GATE_DEBOUNCE_ADDR,0x0000);

        /** Turn ON the LeftDACPower and RightDACPower and configure the HPLCOM as SingleEnded */
         aic33_rset(aic33handle,AIC33_DAC_POWER_OUTPUT_DRIVER_CTRL_ADDR,0x00E0);

        /**configure the HPRCOM as SingleEnded  and Short Circuit protection control to OFF */
        aic33_rset(aic33handle,AIC33_HP_OUTPUT_DRIVER_CTRL_ADDR,0x0010);

        aic33_rset(aic33handle,AIC33_HP_OUTPUT_STAGE_CTRL_ADDR,0x0000);

        /** Left DAC output selects DAC_L1 path and Right DAC output selects DAC_R1 path and
            Right DAC volume follows the left channel control register */
        aic33_rset(aic33handle,AIC33_DAC_OUTPUT_SWITCH_CTRL_ADDR,0x0002);

        /** Output driver pop reduction */
        aic33_rset(aic33handle,AIC33_OUTPUT_DRV_POP_REDUCTION_ADDR,0x006C);

        /** Configure output gain */
        gain = (Uint16)(((audioCfg->outputGain * OUTPUT_GAIN_RANGE) / 100) + OUTPUT_GAIN_MIN);
        gain = (Uint16)(OUTPUT_GAIN_MAX - gain);

        /* Left/Right DAC digital volume gain */
        aic33_rset(aic33handle,AIC33_LEFT_DAC_VOL_CTRL_ADDR, gain);
        aic33_rset(aic33handle,AIC33_RIGHT_DAC_VOL_CTRL_ADDR, gain);

        aic33_rset(aic33handle,AIC33_LINE2L_TO_HPLOUT_VOL_CTRL_ADDR,0x0000);

        aic33_rset(aic33handle,AIC33_PGA_L_TO_HPLOUT_VOL_CTRL_ADDR,0x0000);

        /** Route the DAC L1 to HPLOUT */
        aic33_rset(aic33handle,AIC33_DAC_L1_TO_HPLOUT_VOL_CTRL_ADDR,0x0080);

        aic33_rset(aic33handle,AIC33_LINE2R_TO_HPLOUT_VOL_CTRL_ADDR,0x0000);

        aic33_rset(aic33handle,AIC33_PGA_R_TO_HPLOUT_VOL_CTRL_ADDR,0x0000);

        aic33_rset(aic33handle,AIC33_DAC_R1_TO_HPLOUT_VOL_CTRL_ADDR,0x0000);

        /** Power ON the HPLOUT */
        aic33_rset(aic33handle,AIC33_HPLOUT_OUTPUT_LEVEL_CTRL_ADDR,0x0001);
        /** Mute OFF the HPLOUT */
        aic33_rset(aic33handle,AIC33_HPLOUT_OUTPUT_LEVEL_CTRL_ADDR,0x0009);


        aic33_rset(aic33handle,AIC33_LIN2L_TO_HPLCOM_VOL_CTRL_ADDR,0x0000);
        aic33_rset(aic33handle,AIC33_PGA_L_TO_HPLCOM_VOL_CTRL_ADDR,0x0000);
        aic33_rset(aic33handle,AIC33_DAC_L1_TO_HPLCOM_VOL_CTRL_ADDR,0x0000);
        aic33_rset(aic33handle,AIC33_LINE2R_TO_HPLCOM_VOL_CTRL_ADDR,0x0000);
        aic33_rset(aic33handle,AIC33_PGA_R_TO_HPLCOM_VOL_CTRL_ADDR,0x0000);
        aic33_rset(aic33handle,AIC33_DAC_R1_TO_HPLCOM_VOL_CTRL_ADDR,0x0000);
        aic33_rset(aic33handle,AIC33_HPLCOM_OUTPUT_LEVEL_CTRL_ADDR,0x0000);


        aic33_rset(aic33handle,AIC33_LINE2L_TO_HPROUT_VOL_CTRL_ADDR,0x0000);

        aic33_rset(aic33handle,AIC33_PGA_L_TO_HPROUT_VOL_CTRL_ADDR,0x0000);

        aic33_rset(aic33handle,AIC33_DAC_L1_TO_HPROUT_VOL_CTRL_ADDR,0x0000);

        aic33_rset(aic33handle,AIC33_LINE2R_TO_HPROUT_VOL_CTRL_ADDR,0x0000);

        aic33_rset(aic33handle,AIC33_PGA_R_TO_HPROUT_VOL_CTRL_ADDR,0x0000);

        /** Route the DAC R1 to HPROUT */
        aic33_rset(aic33handle,AIC33_DAC_R1_TO_HPROUT_VOL_CTRL_ADDR,0x0080);


        /** Power ON the HPROUT */
        aic33_rset(aic33handle,AIC33_HPROUT_OUTPUT_LEVEL_CTRL_ADDR,0x0001);
        /** Mute OFF the HPROUT */
        aic33_rset(aic33handle,AIC33_HPROUT_OUTPUT_LEVEL_CTRL_ADDR,0x0009);


        aic33_rset(aic33handle,AIC33_LIN2L_TO_HPRCOM_VOL_CTRL_ADDR,0x0000);
        aic33_rset(aic33handle,AIC33_PGA_L_TO_HPRCOM_VOL_CTRL_ADDR,0x0000);
        aic33_rset(aic33handle,AIC33_DAC_L1_TO_HPRCOM_VOL_CTRL_ADDR,0x0000);
        aic33_rset(aic33handle,AIC33_LINE2R_TO_HPRCOM_VOL_CTRL_ADDR,0x0000);
        aic33_rset(aic33handle,AIC33_LINE2R_TO_HPRCOM_VOL_CTRL_ADDR,0x0000);
        aic33_rset(aic33handle,AIC33_DAC_R1_TO_HPRCOM_VOL_CTRL_ADDR,0x0000);
        aic33_rset(aic33handle,AIC33_HPRCOM_OUTPUT_LEVEL_CTRL_ADDR,0x0000);

        aic33_rset(aic33handle,AIC33_LIN2L_TO_MONO_LOPM_VOL_CTRL_ADDR,0x0000);
        aic33_rset(aic33handle,AIC33_PGA_L_TO_MONO_LOPM_VOL_CTRL_ADDR,0x0000);
        aic33_rset(aic33handle,AIC33_DAC_L1_TO_MONO_LOPM_VOL_CTRL_ADDR,0x0000);
        aic33_rset(aic33handle,AIC33_LINE2R_TO_MONO_LOPM_VOL_CTRL_ADDR,0x0000);
        aic33_rset(aic33handle,AIC33_PGA_R_TO_MONO_LOPM_VOL_CTRL_ADDR,0x0000);
        aic33_rset(aic33handle,AIC33_DAC_R1_TO_MONO_LOPM_VOL_CTRL_ADDR,0x0000);
        aic33_rset(aic33handle,AIC33_MONO_LOPM_OUTPUT_LEVEL_CTRL_ADDR,0x0000);


        aic33_rset(aic33handle,AIC33_LIN2L_TO_LEFT_LOPM_VOL_CTRL_ADDR,0x0000);

        aic33_rset(aic33handle,AIC33_PGA_L_TO_LEFT_LOPM_VOL_CTRL_ADDR,0x0000);

        /** Route the DAC_L1 to LEFT_LOP/M */
        aic33_rset(aic33handle,AIC33_DAC_L1_TO_LEFT_LOPM_VOL_CTRL_ADDR,0x0080);

        aic33_rset(aic33handle,AIC33_LINE2R_TO_LEFT_LOPM_VOL_CTRL_ADDR,0x0000);

        aic33_rset(aic33handle,AIC33_PGA_R_TO_LEFT_LOPM_VOL_CTRL_ADDR,0x0000);

        aic33_rset(aic33handle,AIC33_DAC_R1_TO_LEFT_LOPM_VOL_CTRL_ADDR,0x0000);

        /** Mute OFF and power ON the LEFT_LOP/M */
        aic33_rset(aic33handle,AIC33_LEFT_LOPM_OUTPUT_LEVEL_CTRL_ADDR,0x0009);



        aic33_rset(aic33handle,AIC33_LIN2L_TO_RIGHT_LOPM_VOL_CTRL_ADDR,0x0000);

        aic33_rset(aic33handle,AIC33_PGA_L_TO_RIGHT_LOPM_VOL_CTRL_ADDR,0x0000);

        aic33_rset(aic33handle,AIC33_DAC_L1_TO_RIGHT_LOPM_VOL_CTRL_ADDR,0x0000);

        aic33_rset(aic33handle,AIC33_LINE2R_TO_RIGHT_LOPM_VOL_CTRL_ADDR,0x0000);

        aic33_rset(aic33handle,AIC33_PGA_R_TO_RIGHT_LOPM_VOL_CTRL_ADDR,0x0000);

        /** Route the DAC_R1 to RIGHT_LOP/M */
        aic33_rset(aic33handle,AIC33_DAC_R1_TO_RIGHT_LOPM_VOL_CTRL_ADDR,0x0080);

        /** Mute OFF and power ON the RIGHT_LOP/M */
        aic33_rset(aic33handle,AIC33_RIGHT_LOPM_OUTPUT_LEVEL_CTRL_ADDR,0x0009);

        aic33_rset(aic33handle,AIC33_GPIO1_CTRL_ADDR,0x0000);
        aic33_rset(aic33handle,AIC33_GPIO2_CTRL_ADDR,0x0000);
        aic33_rset(aic33handle,AIC33_ADDITIONAL_GPIOCTRLA_ADDR,0x0000);
        aic33_rset(aic33handle,AIC33_ADDITIONAL_GPIOCTRLB_ADDR,0x0000);

        i2c_close();
    }
    else
    {
        retCode = PSP_E_INVAL_PARAM;
    }

    return (retCode);
}

/*
 *  aic33_inGainControl(Uint8 newGain, PSP_audio_cfg *audioCfg)
 *     Change the gain setting of input channel with a passed value
 *
 */
Int aic33_inGainControl(Uint8 newGain, PSP_audio_cfg *audioCfg)
{
    Uint16 gain   = 0;
    Int    status = -1;

    if( NULL != audioCfg )
    {
        if (0 == i2c_setup ( ))
        {
            return -1;
        }

        if ((newGain > 0u) && (newGain < 100u))
        {
            gain = (Uint16)(((newGain * INPUT_GAIN_RANGE) / 100u) + INPUT_GAIN_MIN);

            /* Left AGC control */
            aic33_rset(aic33handle,AIC33_LEFT_AGC_CTRLA_ADDR, 0x80);
            aic33_rset(aic33handle,AIC33_LEFT_AGC_CTRLB_ADDR, (Uint16)((Uint32)gain << 1));
            aic33_rset(aic33handle,AIC33_LEFT_AGC_CTRLC_ADDR, 0x00);

            /* Right AGC control */
            aic33_rset(aic33handle,AIC33_RIGHT_AGC_CTRLA_ADDR, 0x80);
            aic33_rset(aic33handle,AIC33_RIGHT_AGC_CTRLB_ADDR, (Uint16)((Uint32)gain << 1));
            aic33_rset(aic33handle,AIC33_RIGHT_AGC_CTRLC_ADDR, 0x00);

            /* Update the configuration */
            audioCfg->inputGain = newGain;

            status = 0u;
        }

        i2c_close();
    }
    else
    {
        status = PSP_E_INVAL_PARAM;
    }

    return (status);
}

/*
 *  aic33_outGainControl(Uint8 newGain, PSP_audio_cfg *audioCfg)
 *     Change the gain setting of output channel with a passed value
 *
 */
Int aic33_outGainControl(Uint8 newGain, PSP_audio_cfg *audioCfg)
{
    Uint16 gain = 0;
    Int    status = -1;

    if( NULL != audioCfg )
    {
        if (0 == i2c_setup ( ))
        {
            return -1;
        }

        if ((newGain > 0) && (newGain < 100))
        {
            gain = (Uint16)(((newGain * OUTPUT_GAIN_RANGE) / 100) + OUTPUT_GAIN_MIN);
            gain = (Uint16)(OUTPUT_GAIN_MAX - gain);

            /* Left/Right DAC digital volume gain */
            aic33_rset(aic33handle,AIC33_LEFT_DAC_VOL_CTRL_ADDR, gain);
            aic33_rset(aic33handle,AIC33_RIGHT_DAC_VOL_CTRL_ADDR, gain);

            /* Update the configuration */
            audioCfg->outputGain = newGain;

            status = 0u;
        }

        i2c_close();
    }
    else
    {
        status = PSP_E_INVAL_PARAM;
    }

    return (status);
}


/*
 *  aic33_closeCodec( )
 *     Close the codec
 *
 */
void aic33_closeCodec( aic33_CodecHandle aic33handle1 )
{
    if (0 == i2c_setup ( ))
    {
        return;
    }

    aic33_rset( aic33handle1, 0, 0x00 );              /* Select Page 0 */
    aic33_rset( aic33handle1, AIC33_RESET, 0x80 );    /* Reset the aic33 */

    i2c_close();
    return;
}

/**
  *  Set up the I2C controller in order to communicate with the aic33 codec slave
  *
 **/
static Int i2c_setup(void)
{
    Int32            status   = 1;
    PAL_OsSemAttrs   semAttrs = PAL_OSSEM_ATTRS;
    
    if (NULL == i2cHandle)
    {
         /* Create Completion internal Semaphore */
        if (PAL_osSemCreate(
                            "I2C Completion Sem",
                            0,
                            &semAttrs,
                            &(aic33handle.completionSem)
                           ) != PAL_SOK)
        {
            status = 0;
        }

        i2cHandle = PSP_i2cOpen(I2C_INSTANCE_TO_AIC33,&aic31CallBack,
                                &(aic33handle.completionSem));

        if (NULL == i2cHandle) 
        {
            AIC33_DEBUG(&trace,"\r\nAIC33: aic33codec i2c Create failed\r\n");
            status = 0;
        }
    }

    return (status);
}

/*
 *  Write the specified number of bytes to the spacified slave device.
 *   - Shall pass the address of the slave, pointer to the buffer and number of 
 *  bytes to write
 *
**/

static Int i2c_write(Uint16 slaveaddr, Uint8* data, Uint16 len)
{
    Int                retCode = -1;
    PSP_i2cDataParam   buf;
    PSP_I2cRequest     IoRequest;

    buf.i2cTrans.buffer    = data;
    buf.i2cTrans.bufLen    = len;
    buf.i2cTrans.flags     = PSP_I2C_DEFAULT_WRITE | PSP_I2C_RESTART;
    buf.i2cTrans.param     = NULL;
    buf.i2cTrans.slaveAddr = slaveaddr;
    buf.timeout     = -1;


     /* Update dataparams */
    IoRequest.dataparam  = &buf;
    IoRequest.actualSize = 0;
    IoRequest.appCbk     = NULL;
    IoRequest.status     = PSP_SOK;



    retCode = PSP_i2cTransferIOP( i2cHandle,&IoRequest,(Ptr)NULL);

    if (PSP_SINPROGRESS == retCode)
    {
        if( (PAL_SOK != PAL_osSemTake(aic33handle.completionSem, -1)) )
        {
            AIC33_DEBUG(&trace, "\r\nAIC33  : Error in  PAL_osSemTake");
        }
        else
        {
            retCode = 0;
        }
    }

    if (retCode < 0)
    {
        AIC33_DEBUG(&trace,"\r\nAIC33:Error writing the codec register: %x", 
                    data[0]);
    }

    return (retCode);
}


/*
 *  Deinitilaise the I2c controller which is used for communication with the
 *  Aic31 audio codec.
 *
 */
static void i2c_close()
{
    if (NULL != i2cHandle)
    {
        /* Delete Completion Semaphore  */
        PSP_i2cClose(i2cHandle);
        PAL_osSemDelete(aic33handle.completionSem);

        i2cHandle    = NULL;
        aic33handle.completionSem = NULL;
    }
}

/**
 *
 *  \brief   This function is the codec call back function invoked from the I2c
 *           driver on transfer completion. This function is registered during
 *           the opening of the I2C driver for transactions.
 *
 *  \param   Handle    [IN]  Handle to the I2c channel
 *  \param   arg       [IN]  Argument to the function (Semaphore handle)
 *  \param   ioRequest [IN]  io packet which has completed and invoked the
 *                           callback function
 *
 *  \return  none
 */
static Void aic31CallBack (Ptr Handle,Ptr arg, Ptr ioRequest)
{
    arg = arg;
    ioRequest = ioRequest;
    Handle    = Handle;

    if (PAL_SOK != PAL_osSemGive(aic33handle.completionSem))
    {
        AIC33_DEBUG(&trace, "\r\nAIC33  : Error in PAL_osSemGive");
    }
}

/*============================================================================*/
/*                            END OF FILE                                     */
/*============================================================================*/
