/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/** \file   psp_bios_tsip_sample.c
 *
 *  \brief  This file contains the test / demo code to demonstrate the
 *          TSIP driver implemented for DSP/BIOS in this SoC platform
 *
 *  (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author     PSP Team
 *  \version    0.1
 */

#include <std.h>
#include <gio.h>
#include <string.h>
#include <ti/sdo/pspdrivers/pal_sys/c6452/pal_sys.h>
#include <ti/sdo/pspdrivers/pal_sys/c6452/pal_sys_soc.h>
#include <ti/sdo/pspdrivers/drivers/tsip/psp_tsip.h>
#include <ti/sdo/pspdrivers/drivers/tsip/src/psp_tsipApi.h> 



/*
 * Macros/Constants definitions used in Sample app
 */







#define TSIP_DEVICE        "/TSIP0"

#define TSIP_TEST_ITERATION (1u)


#define TX_DATARATE    ((PSP_TSIP_DATA_RATE_8_192+1)*8192u ) /* units in KHz */
#define RX_DATARATE    ((PSP_TSIP_DATA_RATE_8_192+1)*8192u )

#define TX_CLKMODE     (PSP_TSIP_CLK_MODE_2X)
#define RX_CLKMODE     (PSP_TSIP_CLK_MODE_2X)


/*
 * TSIP Channel Configuration Identifier for A and B context
 */
#define CID_A           (0xAAu)
#define CID_B           (0xBBu)

/* Max frame size for 8-bit linear PCM timeslots enabled frame
   Max = PSP_TSIP_NUM_TIMESLOTS */

#define FRAME_SIZE      (4u)


#define NUM_FRAMES      (4u)
#define FRAME_OVERHEAD  (8u) /* CID+FRFC at the beginning and end of the frame */

/***********************************************************************
 *                    External References
 ***********************************************************************/



/***********************************************************************
 *                    Global References
 ***********************************************************************/

/* TSIP0 Device Params. To be filled by tsip0_dev_init functions */
PSP_tsipDevParams   tsip0DevParams;
/* TSIP1 Device Params. To be filled by tsip1_dev_init functions */
PSP_tsipDevParams   tsip1DevParams;

/* Callback counter to count number of GIO_submit callbacks */
volatile Int32 gGioSubmitCnt = 0;
/* Global callback counter: To count number of times appCall back is called */
volatile Int32 gCbCntr = 0;
/* Global Error Call back counter: Counts no. of times err CallBack is called */
volatile Int32 gErrCntr = 0;

/* Channel handles for TSIP */
static GIO_Handle  xmtHandle;
static GIO_Handle  rcvHandle;

/* GIO application call back for TX and RX */
GIO_AppCallback     txAppCb;
GIO_AppCallback     rxAppCb;

/* TSIP channel params. To be filled as per need */
PSP_tsipChanParams      chanParams;


/* TX IOP params for each GIO_Submit(max 3) */
PSP_tsipIoParams    txIop1[2];
PSP_tsipIoParams    txIop2;

/* RX IOP params for each GIO_Submit(max 3) */
PSP_tsipIoParams    rxIop1[2];
PSP_tsipIoParams    rxIop2;
/*
 * TX IOP Buffers
 */
/* Tx-IOP1 Superframe-1(A context) Buffer */
#pragma DATA_ALIGN(aTxBuff1_s1, 128u);
Uint32   aTxBuff1_s1[(NUM_FRAMES * (FRAME_SIZE + FRAME_OVERHEAD))/4u];
/* Tx-IOP1 Superframe-2(B context) Buffer */
#pragma DATA_ALIGN(bTxBuff1_s2, 128u);
Uint32   bTxBuff1_s2[(NUM_FRAMES * (FRAME_SIZE + FRAME_OVERHEAD))/4u];
/* Tx-IOP1 Superframe-3(A context) Buffer */
#pragma DATA_ALIGN(aTxBuff1_s3, 128u);
Uint32   aTxBuff1_s3[(NUM_FRAMES * (FRAME_SIZE + FRAME_OVERHEAD))/4u];
/* Tx-IOP1 Superframe-4(B context) Buffer */
#pragma DATA_ALIGN(bTxBuff1_s4, 128u);
Uint32   bTxBuff1_s4[(NUM_FRAMES * (FRAME_SIZE + FRAME_OVERHEAD))/4u];

/* Tx-IOP2 Superframe-1(A context) Buffer */
#pragma DATA_ALIGN(aTxBuff2_s1, 128u);
Uint32   aTxBuff2_s1[(NUM_FRAMES * (FRAME_SIZE + FRAME_OVERHEAD))/4u];
/* Tx-IOP2 Superframe-2(B context) Buffer */
#pragma DATA_ALIGN(bTxBuff2_s2, 128u);
Uint32   bTxBuff2_s2[(NUM_FRAMES * (FRAME_SIZE + FRAME_OVERHEAD))/4u];



/*
 * RX IOP Buffers
 */
/* Rx-IOP1 Superframe-1(A context) Buffer */
#pragma DATA_ALIGN(aRxBuff1_s1, 128u);
Uint32   aRxBuff1_s1[(NUM_FRAMES * (FRAME_SIZE + FRAME_OVERHEAD))/4u];
/* Rx-IOP1 Superframe-2(B context) Buffer */
#pragma DATA_ALIGN(bRxBuff1_s2, 128u);
Uint32   bRxBuff1_s2[(NUM_FRAMES * (FRAME_SIZE + FRAME_OVERHEAD))/4u];
/* Rx-IOP1 Superframe-3(A context) Buffer */
#pragma DATA_ALIGN(aRxBuff1_s3, 128u);
Uint32   aRxBuff1_s3[(NUM_FRAMES * (FRAME_SIZE + FRAME_OVERHEAD))/4u];
/* Rx-IOP1 Superframe-4(B context) Buffer */
#pragma DATA_ALIGN(bRxBuff1_s4, 128u);
Uint32   bRxBuff1_s4[(NUM_FRAMES * (FRAME_SIZE + FRAME_OVERHEAD))/4u];

/* Rx-IOP2 Superframe-1(A context) Buffer */
#pragma DATA_ALIGN(aRxBuff2_s1, 128u);
Uint32   aRxBuff2_s1[(NUM_FRAMES * (FRAME_SIZE + FRAME_OVERHEAD))/4u];
/* Rx-IOP2 Superframe-2(B context) Buffer */
#pragma DATA_ALIGN(bRxBuff2_s2, 128u);
Uint32   bRxBuff2_s2[(NUM_FRAMES * (FRAME_SIZE + FRAME_OVERHEAD))/4u];


/* global function prototypes */
void tsip0_dev_init(void);
void tsip1_dev_init(void);
void start_tsip_sample(void);


/****************************************************************************
 *                           Local References
 ****************************************************************************/



static void enable_tsip_LPSC(int devId);

static void open_tsip_channel(char *device);
static void close_tsip_channel(char *device);

static void submit_iop(void);
static void prepare_bitmaps_and_buffers(void);

static void tsip_TxAppCallback(Ptr arg,Int status, Ptr bufp, size_t size);
static void tsip_RxAppCallback(Ptr arg,Int status, Ptr bufp, size_t size);
static void tsipErrCallback(PSP_tsipError err, Ptr arg);



/***********************************************************************
 *                   Function definitions
 ***********************************************************************/

#define PINMUX *(Uint32 volatile *)0x02049000u
/*
 * TSIP-0 init function called when creating driver.
 */
void tsip0_dev_init()
{
    /* Enable TSIP0 in LPSC */
    enable_tsip_LPSC(0);

    /* Enable TSIP(0-1) pinmuxes.(If board is in DVR mode) */
    PINMUX |= (0x0300);

    /*++ SIU ++*/

    tsip0DevParams.siu.clkSel      = PSP_TSIP_CLK_REDUNDANT;

    /* TX-SIU */
    tsip0DevParams.siu.tx.clkSrc   = PSP_TSIP_CLK_FS_SRC_A;

    /* Tx:  Select delay by 1 frame
       To provide zero    dly: Data delay(in # of clocks) = 1023 (2047 in 2X)
       To provide 1 frame dly: Data delay(in # of clocks) = 2047 (4095 in 2X)
       To provide N frame dly: dDly = (N+1) * (DataRate/8) * ClkMode
     */
       
    if( TX_CLKMODE == PSP_TSIP_CLK_MODE_2X) /* 2X clock */
    {
        tsip0DevParams.siu.tx.dDly     = (2u *((TX_DATARATE>>3u)* 2u))-1u;
    }
    else /* 1X clock */
    {
        tsip0DevParams.siu.tx.dDly     = (2u *((TX_DATARATE>>3u)* 1u))-1u;
    }

    tsip0DevParams.siu.tx.disState = PSP_TSIP_TXDIS_STATE_HIGHZ;
    tsip0DevParams.siu.tx.enaDly   = PSP_TSIP_TXENA_DLY_ZERO;
    tsip0DevParams.siu.tx.fSyncP   = PSP_TSIP_POLARITY_ACTIVE_LOW;
    tsip0DevParams.siu.tx.fClkP    = PSP_TSIP_POLARITY_RISING_EDGE;
    tsip0DevParams.siu.tx.dClkP    = PSP_TSIP_POLARITY_RISING_EDGE;
    tsip0DevParams.siu.tx.dRate    = PSP_TSIP_DATA_RATE_8_192;
    tsip0DevParams.siu.tx.clkMode  = TX_CLKMODE;
    tsip0DevParams.siu.tx.fCnt     = 3u;
    tsip0DevParams.siu.tx.fSize    = 127u;

    /* RX-SIU */
    tsip0DevParams.siu.rx.clkSrc   = PSP_TSIP_CLK_FS_SRC_A;

    /* Rx:  Select delay by 1 frame */
    if( RX_CLKMODE == PSP_TSIP_CLK_MODE_2X) /* 2X clock */
    {
        tsip0DevParams.siu.rx.dDly     = (2u *((RX_DATARATE>>3u)* 2u))-1u;
    }
    else /* 1X clock */
    {
        tsip0DevParams.siu.rx.dDly     = (2u *((RX_DATARATE>>3u)* 1u))-1u;
    }

    tsip0DevParams.siu.rx.fSyncP   = PSP_TSIP_POLARITY_ACTIVE_LOW;
    tsip0DevParams.siu.rx.fClkP    = PSP_TSIP_POLARITY_RISING_EDGE;
    tsip0DevParams.siu.rx.dClkP    = PSP_TSIP_POLARITY_RISING_EDGE;
    tsip0DevParams.siu.rx.dRate    = PSP_TSIP_DATA_RATE_8_192;
    tsip0DevParams.siu.rx.clkMode  = RX_CLKMODE;
    tsip0DevParams.siu.rx.fCnt     = 3u;
    tsip0DevParams.siu.rx.fSize    = 127u;

    /* Normal mode or loopback */
    tsip0DevParams.siu.loopBack    = PSP_TSIP_LOOP_BACK_DLB;
    tsip0DevParams.siu.testClk     = PSP_TSIP_TEST_CLK_EXTERNAL;


    /*++ TDMU-DMATCU ++*/

    /* TDMU */
    tsip0DevParams.tdmu.endian  = PSP_TSIP_LITTLE_ENDIAN;

    /* DMATCU */
    tsip0DevParams.tdmu.maxPrio = 7u;
    tsip0DevParams.tdmu.prio    = 7u;
    tsip0DevParams.tdmu.txFint  = PSP_TSIP_INT_XFER_ACK_ONLY;
    tsip0DevParams.tdmu.txSint  = PSP_TSIP_INT_XFER_ACK_ONLY;
    tsip0DevParams.tdmu.rxFint  = PSP_TSIP_INT_XFER_ACK_ONLY;
    tsip0DevParams.tdmu.rxSint  = PSP_TSIP_INT_XFER_ACK_ONLY;

    tsip0DevParams.tdmu.txFintDly  = 0x7Fu;
    tsip0DevParams.tdmu.rxFintDly  = 0x7Fu;


}


/*
 *TSIP-1 init function called when creating driver.
 */
void tsip1_dev_init()
{
    /* Enable TSIP1 in LPSC */
    enable_tsip_LPSC(1);

    /* Enable TSIP(0-1) pinmuxes.(If board is in DVR mode) */
    PINMUX |= (0x0300);

    /*++ SIU ++*/

    tsip1DevParams.siu.clkSel         = PSP_TSIP_CLK_REDUNDANT;

    /* TX-SIU */
    tsip1DevParams.siu.tx.clkSrc   = PSP_TSIP_CLK_FS_SRC_A;

    /* Tx:  Select delay by 1 frame
       To provide zero    dly: Data delay(in # of clocks) = 1023 (2047 in 2X)
       To provide 1 frame dly: Data delay(in # of clocks) = 2047 (4095 in 2X)
       To provide N frame dly: dDly = (N+1) * (DataRate/8) * ClkMode
     */
    if( TX_CLKMODE == PSP_TSIP_CLK_MODE_2X) /* 2X clock */
    {
        tsip1DevParams.siu.tx.dDly     = (2u *((TX_DATARATE>>3u)* 2u))-1u;
    }
    else /* 1X clock */
    {
        tsip1DevParams.siu.tx.dDly     = (2u *((TX_DATARATE>>3u)* 1u))-1u;
    }

    tsip1DevParams.siu.tx.disState = PSP_TSIP_TXDIS_STATE_HIGHZ;
    tsip1DevParams.siu.tx.enaDly   = PSP_TSIP_TXENA_DLY_ZERO;
    tsip1DevParams.siu.tx.fSyncP   = PSP_TSIP_POLARITY_ACTIVE_LOW;
    tsip1DevParams.siu.tx.fClkP    = PSP_TSIP_POLARITY_RISING_EDGE;
    tsip1DevParams.siu.tx.dClkP    = PSP_TSIP_POLARITY_RISING_EDGE;
    tsip1DevParams.siu.tx.dRate    = PSP_TSIP_DATA_RATE_8_192;
    tsip1DevParams.siu.tx.clkMode  = TX_CLKMODE;
    tsip1DevParams.siu.tx.fCnt     = 3u;
    tsip1DevParams.siu.tx.fSize    = 127u;

    /* RX-SIU */
    tsip1DevParams.siu.rx.clkSrc   = PSP_TSIP_CLK_FS_SRC_A;
    /* Rx: Select delay by 1 frame. Since we are trying with loop back, To
    sync Tx-Rx path, Ignoring 1st frame(Dummy event Refer spec) and start
    receiving from 2nd frame. Rx Buffer from 2nd frame
    will be having TxBuffer's first frame data and so on. This is just to
    demonstrate DLB. In real time Tx and Rx will be independent */
    if( RX_CLKMODE == PSP_TSIP_CLK_MODE_2X) /* 2X clock */
    {
        tsip1DevParams.siu.rx.dDly     = (2u *((RX_DATARATE>>3u)* 2u))-1u;
    }
    else /* 1X clock */
    {
        tsip1DevParams.siu.rx.dDly     = (2u *((RX_DATARATE>>3u)* 1u))-1u;
    }

    tsip1DevParams.siu.rx.fSyncP   = PSP_TSIP_POLARITY_ACTIVE_LOW;
    tsip1DevParams.siu.rx.fClkP    = PSP_TSIP_POLARITY_RISING_EDGE;
    tsip1DevParams.siu.rx.dClkP    = PSP_TSIP_POLARITY_RISING_EDGE;
    tsip1DevParams.siu.rx.dRate    = PSP_TSIP_DATA_RATE_8_192;
    tsip1DevParams.siu.rx.clkMode  = RX_CLKMODE;
    tsip1DevParams.siu.rx.fCnt     = 3u;
    tsip1DevParams.siu.rx.fSize    = 127u;

    /* Normal mode or loopback */
    tsip1DevParams.siu.loopBack    = PSP_TSIP_LOOP_BACK_DLB;
    tsip1DevParams.siu.testClk     = PSP_TSIP_TEST_CLK_EXTERNAL;



    /*++ TDMU-DMATCU ++*/

    /* TDMU */
    tsip1DevParams.tdmu.endian  = PSP_TSIP_LITTLE_ENDIAN;

    /* DMATCU */
    tsip1DevParams.tdmu.maxPrio = 7u;
    tsip1DevParams.tdmu.prio    = 7u;
    tsip1DevParams.tdmu.txFint  = PSP_TSIP_INT_XFER_ACK_ONLY;
    tsip1DevParams.tdmu.txSint  = PSP_TSIP_INT_XFER_ACK_ONLY;
    tsip1DevParams.tdmu.rxFint  = PSP_TSIP_INT_XFER_ACK_ONLY;
    tsip1DevParams.tdmu.rxSint  = PSP_TSIP_INT_XFER_ACK_ONLY;

    tsip1DevParams.tdmu.txFintDly  = 0x7Fu;
    tsip1DevParams.tdmu.rxFintDly  = 0x7Fu;

}



/********************************************************************
 * This function performs Sample/Demo application for TSIP. This
 * can be called from any task context of BIOS
 ********************************************************************/
void start_tsip_sample()
{
    int     i=0;


    TSIP_DEBUG("Starting TSIP Sample Application");

    for(i=1; i<= TSIP_TEST_ITERATION; i++)
    {

        open_tsip_channel(TSIP_DEVICE);


        prepare_bitmaps_and_buffers();

        /* Clear global counters */
        gGioSubmitCnt = 0;
        gCbCntr = 0;
        gErrCntr = 0;
        submit_iop();

        /* Wait until all submit requests completed */
        while(gCbCntr < gGioSubmitCnt)
        {;}

        close_tsip_channel(TSIP_DEVICE);
    }

    TSIP_DEBUG("TSIP Sample Application <Completed>");
}


/*
 * Call back notification of IOP completion(Called from ISR)
 */
/* ARGSUSED */
static void tsip_TxAppCallback(Ptr arg,Int status, Ptr bufp, size_t size)
{
    gCbCntr++;
    TSIP_DEBUG2("Tx Call back: Status[%d] size[%d]",status,size);

}

/* ARGSUSED */
static void tsip_RxAppCallback(Ptr arg,Int status, Ptr bufp, size_t size)
{
    gCbCntr++;
    TSIP_DEBUG2("Rx Call back: Status[%d] size[%d]",status,size);
}


/*
 * Error notification callback function(Called from ISR)
 */
/* ARGSUSED */
static void tsipErrCallback(PSP_tsipError err, Ptr arg)
{
    gErrCntr++;
    TSIP_DEBUG2("ErrCode[%d]:ErrInfo[%x]",err.errCode,err.errInfo);
}

/*
 * Opens I/O channels(GIO_create) for TSIP0 and TSIP1
 */
static void open_tsip_channel(char *device)
{
    Int                     stat = IOM_COMPLETED;
    GIO_Attrs               gioAttrs;
    Int                     tsCnt = 0;

    if( NULL != device )
    {
        /*
         * Initialize channel attributes.
         */
        gioAttrs.nPackets       = PSP_TSIP_MAX_IOP_REQ;

        for(tsCnt=0; tsCnt < PSP_TSIP_NUM_TIMESLOTS; tsCnt++)
        {
           chanParams.bmA.pcmType[tsCnt] = PSP_TSIP_PCM_DISABLED;
           chanParams.bmB.pcmType[tsCnt] = PSP_TSIP_PCM_DISABLED;
        }
        chanParams.errCb = &tsipErrCallback;
        chanParams.eCbArg= NULL;
        chanParams.aCID  = CID_A;
        chanParams.bCID  = CID_B;

        /* If user wants some function to be called from every frame/sframe period
         from frameISR, he/she has to register some valid function address*/
        chanParams.frmCb  = NULL;
        chanParams.fCbArg = NULL;
        chanParams.sfrmCb = NULL;
        chanParams.sCbArg = NULL;

        /* Enable all A/B bitmaps to linear */
        for(tsCnt=0; tsCnt < FRAME_SIZE; tsCnt++)
        {
           chanParams.bmA.pcmType[tsCnt] = PSP_TSIP_PCM_8_BIT_LINEAR;
           chanParams.bmB.pcmType[tsCnt] = PSP_TSIP_PCM_8_BIT_LINEAR;
        }

        /* Create TX TSIP channel */
        xmtHandle = GIO_create(
                                device,
                                IOM_OUTPUT,
                                &stat,
                                &chanParams,
                                &gioAttrs
                                );

        if (NULL == xmtHandle)
        {
            TSIP_DEBUG("TSIP-TX channel creation <Failed>");

        }
        else
        {
            TSIP_DEBUG("TSIP-TX channel creation <Success>");
        }

        /* Create RX TSIP channel */
        rcvHandle  = GIO_create(
                                device,
                                IOM_INPUT,
                                &stat,
                                &chanParams,
                                &gioAttrs
                                );

        if (NULL == rcvHandle)
        {
            TSIP_DEBUG("TSIP-RX channel creation <Failed>");

        }
        else
        {
            TSIP_DEBUG("TSIP-RX channel creation <Success>");
        }
    }

}


/* ARGSUSED */
static void close_tsip_channel(char *device)
{

    Int    uCCstat = IOM_COMPLETED;

    /********* TSIP0 ************/
    uCCstat=GIO_delete(xmtHandle);

    if(uCCstat!=IOM_COMPLETED)
    {
        TSIP_DEBUG("TSIP-TX channel deletion <Failed>");
    }
    else
    {
        TSIP_DEBUG("TSIP-TX channel deletion <Success>");
    }

    uCCstat=GIO_delete(rcvHandle);

    if(uCCstat!=IOM_COMPLETED)
    {
        TSIP_DEBUG("TSIP-RX channel deletion <Failed>");
    }
    else
    {
        TSIP_DEBUG("TSIP-RX channel deletion <Success>");
    }
}


static void submit_iop()
{
    Int                 retCode;
    size_t              sfrmSize;

    /* Application Callback */
    rxAppCb.fxn = &tsip_RxAppCallback;
    rxAppCb.arg = &rcvHandle;

    txAppCb.fxn = &tsip_TxAppCallback;
    txAppCb.arg = &xmtHandle;

    /* First Rx IOP submit */
    sfrmSize = 4;
    retCode = GIO_submit(rcvHandle,IOM_READ,&rxIop1[0],&sfrmSize,&rxAppCb);
    TSIP_DEBUG1("GIO_submit:RxIOP1 retcode[%d]",retCode);
    gGioSubmitCnt++;

    /* First Tx IOP submit */
    sfrmSize = 4u;
    retCode = GIO_submit(xmtHandle,IOM_WRITE,&txIop1[0],&sfrmSize,&txAppCb);
    TSIP_DEBUG1("GIO_submit:TxIOP1 retcode[%d]",retCode);
    gGioSubmitCnt++;

    /* Second Rx IOP submit */
    sfrmSize = 2u;
    retCode = GIO_submit(rcvHandle,IOM_READ,&rxIop2,&sfrmSize,&rxAppCb);
    TSIP_DEBUG1("GIO_submit:RxIOP2 retcode[%d]",retCode);
    gGioSubmitCnt++;


    /* Second Tx IOP submit */
    sfrmSize = 2u;
    retCode = GIO_submit(xmtHandle,IOM_WRITE,&txIop2,&sfrmSize,&txAppCb);
    TSIP_DEBUG1("GIO_submit:TxIOP2 retcode[%d]",retCode);
    gGioSubmitCnt++;

}


/*
 * Prepare IOP submit params, bitmaps, TX-RX buffer.
 */
static void prepare_bitmaps_and_buffers()
{
    Uint32              i;
    Uint32              fCnt;
    Uint32              fSize;

    for(i=0; i < (NUM_FRAMES * (FRAME_SIZE + FRAME_OVERHEAD))/4u; i++)
    {
       /* Clear Tx A-context buffer */
       aTxBuff1_s1[i] = 0;
       aTxBuff1_s3[i] = 0;
       aTxBuff2_s1[i] = 0;

       /* Clear Rx A-context buffer */
       aRxBuff1_s1[i] = 0;
       aRxBuff1_s3[i] = 0;
       aRxBuff2_s1[i] = 0;

       /* Clear Tx B-context buffer */
       bTxBuff1_s2[i] = 0;
       bTxBuff1_s4[i] = 0;
       bTxBuff2_s2[i] = 0;

       /* Clear Rx B-context buffer */
       bRxBuff1_s2[i] = 0;
       bRxBuff1_s4[i] = 0;
       bRxBuff2_s2[i] = 0;
    }

    /*** TX-IOP1 */

    /* Four Superframes in Tx-IOP1 */
    /* 1st Sframe (A) */
    txIop1[0].dmaA.baseAddr   = &aTxBuff1_s1[0];
    txIop1[0].dmaA.frameAlloc = FRAME_SIZE + FRAME_OVERHEAD;
    txIop1[0].dmaA.frameCnt   = NUM_FRAMES;
    txIop1[0].dmaA.frameSize  = FRAME_SIZE + FRAME_OVERHEAD;
    /* 2nd Sframe (B) */
    txIop1[0].dmaB.baseAddr   = &bTxBuff1_s2[0];
    txIop1[0].dmaB.frameAlloc = FRAME_SIZE + FRAME_OVERHEAD;
    txIop1[0].dmaB.frameCnt   = NUM_FRAMES;
    txIop1[0].dmaB.frameSize  = FRAME_SIZE + FRAME_OVERHEAD;
    /* 3rd Sframe (A) */
    txIop1[1].dmaA.baseAddr   = &aTxBuff1_s3[0];
    txIop1[1].dmaA.frameAlloc = FRAME_SIZE + FRAME_OVERHEAD;
    txIop1[1].dmaA.frameCnt   = NUM_FRAMES;
    txIop1[1].dmaA.frameSize  = FRAME_SIZE + FRAME_OVERHEAD;
    /* 4th Sframe (B) */
    txIop1[1].dmaB.baseAddr   = &bTxBuff1_s4[0];
    txIop1[1].dmaB.frameAlloc = FRAME_SIZE + FRAME_OVERHEAD;
    txIop1[1].dmaB.frameCnt   = NUM_FRAMES;
    txIop1[1].dmaB.frameSize  = FRAME_SIZE + FRAME_OVERHEAD;

    /*** TX-IOP2 */

    /* Two Superframes in Tx-IOP2 */
    /* 1st Sframe (A) */
    txIop2.dmaA.baseAddr   = &aTxBuff2_s1[0];
    txIop2.dmaA.frameAlloc = FRAME_SIZE + FRAME_OVERHEAD;
    txIop2.dmaA.frameCnt   = NUM_FRAMES;
    txIop2.dmaA.frameSize  = FRAME_SIZE + FRAME_OVERHEAD;
    /* 2nd Sframe (B) */
    txIop2.dmaB.baseAddr   = &bTxBuff2_s2[0];
    txIop2.dmaB.frameAlloc = FRAME_SIZE + FRAME_OVERHEAD;
    txIop2.dmaB.frameCnt   = NUM_FRAMES;
    txIop2.dmaB.frameSize  = FRAME_SIZE + FRAME_OVERHEAD;


    /* Filling TX IOP buffers with Sample Data */

    /* TX-IOP1 */
    /* 1st Sframe Buffer (A) */
    for(fCnt=0;fCnt< txIop1[0].dmaA.frameCnt;fCnt++)
    {
        aTxBuff1_s1[(fCnt*txIop1[0].dmaA.frameAlloc)/4u] = (CID_A<<24u) + 0xC0FFEEu;

        for(fSize=1u;fSize<= ((txIop1[0].dmaA.frameSize - 8u)/4u);fSize++)
        {
            aTxBuff1_s1[((fCnt*txIop1[0].dmaA.frameAlloc)/4u) + fSize]= 0x11223344u;
        }
        aTxBuff1_s1[((fCnt*txIop1[0].dmaA.frameAlloc)/4u) + fSize] = (CID_A<<24u)+0xC0FFEEu;
    }

    /* 2nd Sframe Buffer (B) */
    for(fCnt=0;fCnt< txIop1[0].dmaB.frameCnt;fCnt++)
    {
        bTxBuff1_s2[(fCnt*txIop1[0].dmaB.frameAlloc)/4u] = (CID_B<<24u) + 0xC0FFEEu;

        for(fSize=1u;fSize<= ((txIop1[0].dmaB.frameSize - 8u)/4u);fSize++)
        {
            bTxBuff1_s2[((fCnt*txIop1[0].dmaB.frameAlloc)/4u) + fSize]= 0x55667788u;
        }
        bTxBuff1_s2[((fCnt*txIop1[0].dmaB.frameAlloc)/4u) + fSize] = (CID_B<<24u)+0xC0FFEEu;
    }
    /* 3rd Sframe Buffer (A) */
    for(fCnt=0;fCnt< txIop1[1].dmaA.frameCnt;fCnt++)
    {
        aTxBuff1_s3[(fCnt*txIop1[1].dmaA.frameAlloc)/4u] = (CID_A<<24u) + 0xC0FFEEu;

        for(fSize=1u;fSize<= (txIop1[1].dmaA.frameSize - 8u)/4u;fSize++)
        {
            aTxBuff1_s3[((fCnt*txIop1[1].dmaA.frameAlloc)/4u) + fSize]= 0x99AABBCCu;
        }
        aTxBuff1_s3[((fCnt*txIop1[1].dmaA.frameAlloc)/4u) + fSize] = (CID_A<<24)+0xC0FFEEu;
    }
    /* 4th Sframe Buffer (B) */
    for(fCnt=0;fCnt< txIop1[1].dmaB.frameCnt;fCnt++)
    {
        bTxBuff1_s4[(fCnt*txIop1[1].dmaB.frameAlloc)/4] = (CID_B<<24u) + 0xC0FFEEu;

        for(fSize=1u;fSize<= ((txIop1[1].dmaB.frameSize - 8u))/4u;fSize++)
        {
            bTxBuff1_s4[((fCnt*txIop1[1].dmaB.frameAlloc)/4u) + fSize]= 0xDEADBEEFu;
        }
        bTxBuff1_s4[((fCnt*txIop1[1].dmaB.frameAlloc)/4u) + fSize] = (CID_B<<24u)+0xC0FFEEu;
    }

    /* TX-IOP2 */
    /* 1st Sframe Buffer (A) */
    for(fCnt=0;fCnt< txIop2.dmaA.frameCnt;fCnt++)
    {
        aTxBuff2_s1[(fCnt*txIop2.dmaA.frameAlloc)/4u] = (CID_A<<24u) + 0xC0FFEEu;

        for(fSize=1u; fSize<= ((txIop2.dmaA.frameSize - 8u)/4u);fSize++)
        {
            aTxBuff2_s1[((fCnt*txIop2.dmaA.frameAlloc)/4u) + fSize]= 0x11223344u;
        }
        aTxBuff2_s1[((fCnt*txIop2.dmaA.frameAlloc)/4u) + fSize] = (CID_A<<24u)+0xC0FFEEu;
    }

    /* 2nd Sframe Buffer (B) */
    for(fCnt=0;fCnt< txIop2.dmaB.frameCnt;fCnt++)
    {
        bTxBuff2_s2[(fCnt*txIop2.dmaB.frameAlloc)/4u] = (CID_B<<24u) + 0xC0FFEEu;

        for(fSize=1u ;fSize<= ((txIop2.dmaB.frameSize - 8u)/4u); fSize++)
        {
            bTxBuff2_s2[((fCnt*txIop2.dmaB.frameAlloc)/4u) + fSize]= 0x55667788u;
        }
        bTxBuff2_s2[((fCnt*txIop2.dmaB.frameAlloc)/4u) + fSize] = (CID_B<<24u)+0xC0FFEEu;
    }



    /*** RX-IOP1 */

    /* Four Superframes in Rx-IOP1 */
    /* 1st Sframe (A) */
    rxIop1[0].dmaA.baseAddr   = &aRxBuff1_s1[0];
    rxIop1[0].dmaA.frameAlloc = FRAME_SIZE + FRAME_OVERHEAD;
    rxIop1[0].dmaA.frameCnt   = NUM_FRAMES;
    rxIop1[0].dmaA.frameSize  = FRAME_SIZE + FRAME_OVERHEAD;
    /* 2nd Sframe (B) */
    rxIop1[0].dmaB.baseAddr   = &bRxBuff1_s2[0];
    rxIop1[0].dmaB.frameAlloc = FRAME_SIZE + FRAME_OVERHEAD;
    rxIop1[0].dmaB.frameCnt   = NUM_FRAMES;
    rxIop1[0].dmaB.frameSize  = FRAME_SIZE + FRAME_OVERHEAD;
    /* 3rd Sframe (A) */
    rxIop1[1].dmaA.baseAddr   = &aRxBuff1_s3[0];
    rxIop1[1].dmaA.frameAlloc = FRAME_SIZE + FRAME_OVERHEAD;
    rxIop1[1].dmaA.frameCnt   = NUM_FRAMES;
    rxIop1[1].dmaA.frameSize  = FRAME_SIZE + FRAME_OVERHEAD;
    /* 4th Sframe (B) */
    rxIop1[1].dmaB.baseAddr   = &bRxBuff1_s4[0];
    rxIop1[1].dmaB.frameAlloc = FRAME_SIZE + FRAME_OVERHEAD;
    rxIop1[1].dmaB.frameCnt   = NUM_FRAMES;
    rxIop1[1].dmaB.frameSize  = FRAME_SIZE + FRAME_OVERHEAD;

    /*** RX-IOP2 */

    /* Two Superframes in Rx-IOP2 */
    /* 1st Sframe (A) */
    rxIop2.dmaA.baseAddr   = &aRxBuff2_s1[0];
    rxIop2.dmaA.frameAlloc = FRAME_SIZE + FRAME_OVERHEAD;
    rxIop2.dmaA.frameCnt   = NUM_FRAMES;
    rxIop2.dmaA.frameSize  = FRAME_SIZE + FRAME_OVERHEAD;
    /* 2nd Sframe (B) */
    rxIop2.dmaB.baseAddr   = &bRxBuff2_s2[0];
    rxIop2.dmaB.frameAlloc = FRAME_SIZE + FRAME_OVERHEAD;
    rxIop2.dmaB.frameCnt   = NUM_FRAMES;
    rxIop2.dmaB.frameSize  = FRAME_SIZE + FRAME_OVERHEAD;

}


/*
 * Enable TSIP in LPSC
 */
static void enable_tsip_LPSC(int devId)
{

    PAL_Result   result = PSP_SOK;

    if(0 == devId)
    {
        /* Power On TSIP0 */
        result = PAL_sysModuleClkCtrl(
                                PAL_SYS_CLK_MODULE_TSIP0,
                                PAL_SYS_MODULE_CLK_ENABLE
                                );

    }

    if(1 == devId)
    {
        /* Power On TSIP1 */
        result = PAL_sysModuleClkCtrl(
                                PAL_SYS_CLK_MODULE_TSIP1,
                                PAL_SYS_MODULE_CLK_ENABLE
                                );

    }


    if(PAL_SOK != result)
    {
        TSIP_DEBUG("<ERROR in TSIP module enable in LPSC>");
    }

}



