/** ============================================================================
 *  Steps to test this driver
 *  -------------------------
 *    1. Copy Makefile and pcidrv.c from "pci\src\linuxapp"
 *       folder to "/home" directory of linux machine.
 *    2. Go to "/home" directory by giving "cd /home" command
 *    3. Give "make" command. This will generate the object file of a PCI host
 *       driver for linux. Driver will be generated by module name "pcidriver.ko"
 *       in same directory.
 *    4.Give "insmod pcidriver.ko" command. This will insert the driver as a
 *       module
 *    5.This will give some print messages on linux machine. Give "dmesg" command
 *       on linux console to see the print messages of linux driver for PCI.
 *    6.There will be a print message saying "Address to write from DSP = 0x434000".
 *       Note this address(0x434000).
 *    7.After entering this address in CCS3 dialogue box, messages will print
 *      in CCS3 stdout window whethe test cases are successfull or not. Also it will
 *      give some print messages saying "Data comparison success" in the linux console.
 *    8.Also you can see the test output by giving "dmesg" command on linux console.
 *
 *  The following functionality are been tested in this driver
 *  ----------------------------------------------------------
 *    1. DSP target generates Software(2) interrupt to Linux host.
 *    2. Linux host generate Software(3) interrupt to DSP target.
 *    3. Data Written from DSP target side is read and compared here.
 *    4. Data is been written to the same memory space which is been
 *       read and compared by DSP target.
 *
 *  ============================================================================
 */
#include <linux/config.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/pci.h>
#include <linux/delay.h>
#include <linux/fs.h>
#include <linux/init.h>
#include <linux/ioport.h>
#include <asm/uaccess.h>
#include <linux/interrupt.h>
#include <asm/irq.h>

/** ============================================================================
 *  @macro  Data types
 *
 *  @desc   Basic data types
 *  ============================================================================
 */
typedef signed char         Int8 ;      /*  8 bit value */
typedef signed short int    Int16 ;     /* 16 bit value */
typedef signed long  int    Int32 ;     /* 32 bit value */

typedef unsigned char       Uint8 ;     /*  8 bit value */
typedef unsigned short int  Uint16 ;    /* 16 bit value */
typedef unsigned long  int  Uint32 ;    /* 32 bit value */

typedef float               Real32 ;    /* 32 bit value */
typedef double              Real64 ;    /* 64 bit value */

typedef short int           Bool ;      /* 16 bit value */

typedef char                Char8 ;     /*  8 bit value */
typedef short               Char16 ;    /* 16 bit value */

typedef unsigned char       Uchar8 ;    /*  8 bit value */
typedef unsigned short      Uchar16 ;   /* 16 bit value */

#define Void                void
typedef void *              Pvoid ;

typedef Char8 *             Pstr ;
typedef Uchar8 *            Pustr ;

#define LPSC_DDR    (13)
/** ============================================================================
 *  @const  PCI_TI_VENDOR
 *
 *  @desc   TI Vendor ID.
 *  ============================================================================
 */
#define PCI_TI_VENDOR   0x104c

/** ============================================================================
 *  @const  PCI_TI_DEVICE
 *
 *  @desc   PCI Device ID.
 *  ============================================================================
 */
#define PCI_TI_DEVICE   0xB001

/** ============================================================================
 *  @const   DM6437PCI_DEVREG_BASE
 *
 *  @desc    Base address of Device config registers.
 *  ============================================================================
 */
#define DM6437PCI_DEVREG_BASE      0x01C40000

/** ============================================================================
 *  @const   DM6437PCI_PLLREG_BASE
 *
 *  @desc    Base address of PLL0 registers.
 *  ============================================================================
 */
#define DM6437PCI_PLL0REG_BASE      0x01C40800

/** ============================================================================
 *  @const   DM6437PCI_PLLREG_BASE
 *
 *  @desc    Base address of PLL1 registers.
 *  ============================================================================
 */
#define DM6437PCI_PLL1REG_BASE      0x01C40C00

/** ============================================================================
 *  @const   DM6437PCI_EDMAREG_BASE
 *
 *  @desc    Base address of EDMA registers.
 *  ============================================================================
 */
#define DM6437PCI_EDMAREG_BASE      0x01C00000

/** ============================================================================
 *  @const   DM6437PCI_DDRREG_BASE
 *
 *  @desc    Base address of DDR PHY registers.
 *  ============================================================================
 */
#define DM6437PCI_DDRREG_BASE       0x20000000

/** ============================================================================
 *  @const   DM6437PCI_PSCREG_BASE
 *
 *  @desc    Base address of PSC registers.
 *  ============================================================================
 */
#define DM6437PCI_PSCREG_BASE       0x01C41000

/** ============================================================================
 *  @const   DM6437PCI_PCIREG_BASE
 *
 *  @desc    Base address of PCI backend registers.
 *  ============================================================================
 */
#define DM6437PCI_PCIREG_BASE       0x01C1A000

/** ============================================================================
 *  @const   DM6437PCI_SOFTINT2_MASK
 *
 *  @desc    Mask for generating soft int2 (DSP->GPP)
 *  ============================================================================
 */
#define DM6437PCI_SOFTINT2_MASK    0x04000000
/** ============================================================================
 *  @const   DM6437PCI_SOFTINT1_MASK
 *
 *  @desc    Mask for generating soft int1 (GPP->DSP)
 *  ============================================================================
 */
#define DM6437PCI_SOFTINT1_MASK       0x02000000

/** ============================================================================
 *  @const   DM6437PCI_LRESET_MASK
 *
 *  @desc    Mask for reseting/releasing GEM.
 *  ============================================================================
 */
#define DM6437PCI_LRESET_MASK         0x00000100

/** ============================================================================
 *  @const   DM6437PCI_INTSTATUS_MASK
 *
 *  @desc    Bitmask for Interrupt status (DSP->GPP)
 *  ============================================================================
 */
#define DM6437PCI_INTSTATUS_MASK      0x00080000

/** ============================================================================
 *  @const   DM6437PCI_PCIADLEN
 *
 *  @desc    Length each segment of addressable PCI Space..
 *  ============================================================================
 */
#define DM6437PCI_PCIADLEN            0x00800000

/** ============================================================================
 *  @const   DM6437PCI_PCIADWRBITMASK
 *
 *  @desc    Mask indicating writeable bits in PCI Address Window registers.
 *  ============================================================================
 */
#define DM6437PCI_PCIADWRBITMASK      0xFF800000

/** ============================================================================
 *  @const   DM6437PCI_PAGEWRBITMASK
 *
 *  @desc    Mask indicating writeable bits in PCI Base Address Mask Register5.
 *  ============================================================================
 */
#define DM6437PCI_PAGEWRBITMASK       0xFF800000

/** ============================================================================
 *  @name   DM6437PCI_pllRegs
 *
 *  @desc   Register Overlay Structure for Device config registers.
 *  ============================================================================
 */
typedef struct DM6437PCI_devRegs_tag {
    volatile Uint32 PINMUX0       ;
    volatile Uint32 PINMUX1       ;
    volatile Uint32 DSPBOOTADDR   ;
    volatile Uint32 BOOTCMPLT     ;
    volatile Uint32 RSVD0         ;
    volatile Uint32 BOOTCFG       ;
    volatile Uint32 VDD1P0V_ADJ   ;
    volatile Uint32 VDD1P2V_ADJ   ;
    volatile Uint32 DDR_SLEW      ;
    volatile Uint32 PERIPHEN      ;
    volatile Uint32 DEVICE_ID     ;
    volatile Uint32 DAC_DEMEN     ;
    volatile Uint32 UHPICTL       ;
    volatile Uint32 RSVD1 [2]     ;
    volatile Uint32 MSTPRI0       ;
    volatile Uint32 MSTPRI1       ;
    volatile Uint32 VPSS_CLK_CTRL ;
    volatile Uint32 VDD3P3V_PWRDN ;
    volatile Uint32 DFT_ENABLE    ;
    volatile Uint32 SEC_SCAN_REG  ;
    volatile Uint32 SEC_TEST_REG  ;
    volatile Uint32 SEC_TAP_CTL   ;
    volatile Uint32 PUBLIC_KEY0   ;
    volatile Uint32 PUBLIC_KEY1   ;
    volatile Uint32 PUBLIC_KEY2   ;
    volatile Uint32 PUBLIC_KEY3   ;
    volatile Uint32 VLYNQ_DELAY   ;
    volatile Uint32 SEC_PSEUDO0   ;
    volatile Uint32 SEC_PSEUDO1   ;
    volatile Uint32 RSVD2 [3]     ;
    volatile Uint32 TIMERCTL      ;
    volatile Uint32 TPTCCCFG      ;
    volatile Uint32 RSVD3         ;
} DM6437PCI_devRegs ;

/** ============================================================================
 *  @name   DM6437PCI_pllRegs
 *
 *  @desc   Register Overlay Structure for PLL.
 *  ============================================================================
 */
typedef struct DM6437PCI_pllRegs_tag {
    volatile Uint32 PID         ;
    volatile Uint8  RSVD0 [220] ;
    volatile Uint32 FUSERR      ;
    volatile Uint32 RSTYPE      ;
    volatile Uint32 RSTDEF      ;
    volatile Uint8  RSVD1 [20]  ;
    volatile Uint32 PLLCTL      ;
    volatile Uint32 OCSEL       ;
    volatile Uint32 SECCTL      ;
    volatile Uint8  RSVD2 [4]   ;
    volatile Uint32 PLLM        ;
    volatile Uint32 PREDIV      ;
    volatile Uint32 PLLDIV1     ;
    volatile Uint32 PLLDIV2     ;
    volatile Uint32 PLLDIV3     ;
    volatile Uint32 OSCDIV1     ;
    volatile Uint32 POSTDIV     ;
    volatile Uint32 BPDIV       ;
    volatile Uint32 WAKEUP      ;
    volatile Uint8  RSVD3 [4]   ;
    volatile Uint32 PLLCMD      ;
    volatile Uint32 PLLSTAT     ;
    volatile Uint32 ALNCTL      ;
    volatile Uint32 DCHANGE     ;
    volatile Uint32 CKEN        ;
    volatile Uint32 CKSTAT      ;
    volatile Uint32 SYSTAT      ;
    volatile Uint8  RSVD4 [12]  ;
    volatile Uint32 PLLDIV4     ;
    volatile Uint32 PLLDIV5     ;
    volatile Uint32 PLLDIV6     ;
    volatile Uint32 PLLDIV7     ;
    volatile Uint32 PLLDIV8     ;
} DM6437PCI_pllRegs ;

/** ============================================================================
 *  @name   DM6437PCI_ddrRegs
 *
 *  @desc   Register Overlay Structure for DDR.
 *  ============================================================================
 */
typedef struct  {
    volatile Uint32 ERCSR       ;
    volatile Uint32 SDRSTAT     ;
    volatile Uint32 SDBCR       ;
    volatile Uint32 SDRCR       ;
    volatile Uint32 SDTIMR      ;
    volatile Uint32 SDTIMR2     ;
    volatile Uint8  RSVD0 [8]   ;
    volatile Uint32 VBPR        ;
    volatile Uint8  RSVD1 [4]   ;
    volatile Uint32 VBCFG1      ;
    volatile Uint32 VBCFG2      ;
    volatile Uint8  RSVD2 [16]  ;
    volatile Uint32 PERFC1R     ;
    volatile Uint32 PERFC2R     ;
    volatile Uint32 PCCR        ;
    volatile Uint32 PCMRSR      ;
    volatile Uint8  RSVD3 [48]  ;
    volatile Uint32 ASYNCCS2CR  ;
    volatile Uint32 ASYNCCS3CR  ;
    volatile Uint32 ASYNCCS4CR  ;
    volatile Uint32 ASYNCCS5CR  ;
    volatile Uint8  RSVD4 [16]  ;
    volatile Uint32 AWCCR       ;
    volatile Uint8  RSVD5 [28]  ;
    volatile Uint32 IRR         ;
    volatile Uint32 IMR         ;
    volatile Uint32 IMSR        ;
    volatile Uint32 IMCR        ;
    volatile Uint8  RSVD6 [16]  ;
    volatile Uint32 DDRPHYREV   ;
    volatile Uint32 DDRPHYCR    ;
    volatile Uint32 DDRPHYSR    ;
    volatile Uint8  RSVD7 [4]   ;
    volatile Uint32 VTPCTRL     ;
    volatile Uint32 VTPSTAT     ;
} DM6437PCI_ddrRegs ;

/** ============================================================================
 *  @name   DM6437PCI_pscRegs
 *
 *  @desc   Register Overlay Structure for PSC.
 *  ============================================================================
 */
typedef struct DM6437PCI_pscRegs_tag {
    volatile Uint32 PID          ;
    volatile Uint8  RSVD0 [12]   ;
    volatile Uint32 GBLCTL       ;
    volatile Uint8  RSVD1 [4]    ;
    volatile Uint32 INTEVAL      ;
    volatile Uint8  RSVD2 [36]   ;
    volatile Uint32 ERRPR0       ;
    volatile Uint32 ERRPR1       ;
    volatile Uint8  RSVD3 [8]    ;
    volatile Uint32 ERRCR0       ;
    volatile Uint32 ERRCR1       ;
    volatile Uint8  RSVD4 [8]    ;
    volatile Uint32 PERRPR       ;
    volatile Uint8  RSVD5 [4]    ;
    volatile Uint32 PERRCR       ;
    volatile Uint8  RSVD6 [4]    ;
    volatile Uint32 EPCPR        ;
    volatile Uint8  RSVD7 [4]    ;
    volatile Uint32 EPCR         ;
    volatile Uint8  RSVD8 [132]  ;
    volatile Uint32 RAILSTAT     ;
    volatile Uint32 RAILCTL      ;
    volatile Uint32 RAILSEL      ;
    volatile Uint8  RSVD9 [20]   ;
    volatile Uint32 PTCMD        ;
    volatile Uint8  RSVD10 [4]   ;
    volatile Uint32 PTSTAT       ;
    volatile Uint8  RSVD11 [212] ;
    volatile Uint32 PDSTAT [41]  ;
    volatile Uint8  RSVD12 [92]  ;
    volatile Uint32 PDCTL [41]   ;
    volatile Uint8  RSVD13 [348] ;
    volatile Uint32 MRSTOUT [2]  ;
    volatile Uint8  RSVD14 [8]   ;
    volatile Uint32 MCKOUT [2]   ;
    volatile Uint8  RSVD15 [232] ;
    volatile Uint32 MDCFG [41]   ;
    volatile Uint8  RSVD16 [348] ;
    volatile Uint32 MDSTAT [41]  ;
    volatile Uint8  RSVD17 [348] ;
    volatile Uint32 MDCTL [41]   ;
} DM6437PCI_pscRegs ;

/** ============================================================================
 *  @name   DM6437PCI_pciRegs
 *
 *  @desc   PCI Back end register overlay structure.
 *  ============================================================================
 */
typedef struct DM6437PCI_pciRegs_tag {
    volatile Uint32 PCIREVID       ;
    volatile Uint8  RSVD0 [12]     ;
    volatile Uint32 PCISTATSET     ;
    volatile Uint32 PCISTATCLR     ;
    volatile Uint8  RSVD1 [8]      ;
    volatile Uint32 PCIHINTSET     ;
    volatile Uint32 PCIHINTCLR     ;
    volatile Uint8  RSVD2 [8]      ;
    volatile Uint32 PCIBINTSET     ;
    volatile Uint32 PCIBINTCLR     ;
    volatile Uint32 PCIBCLKMGT     ;
    volatile Uint8  RSVD3 [196]    ;
    volatile Uint32 PCIVENDEVMIR   ;
    volatile Uint32 PCICSRMIR      ;
    volatile Uint32 PCICLREVMIR    ;
    volatile Uint32 PCICLINEMIR    ;
    volatile Uint32 PCIBAR0MSK     ;
    volatile Uint32 PCIBAR1MSK     ;
    volatile Uint32 PCIBAR2MSK     ;
    volatile Uint32 PCIBAR3MSK     ;
    volatile Uint32 PCIBAR4MSK     ;
    volatile Uint32 PCIBAR5MSK     ;
    volatile Uint8  RSVD4[4]       ;
    volatile Uint32 PCISUBIDMIR    ;
    volatile Uint8  RSVD5 [4]      ;
    volatile Uint32 PCICPBPTRMIR   ;
    volatile Uint8  RSVD6 [4]      ;
    volatile Uint32 PCILGINTMIR    ;
    volatile Uint8  RSVD7 [64]     ;
    volatile Uint32 PCISLVCNTL     ;
    volatile Uint8  RSVD8 [60]     ;
    volatile Uint32 PCIBAR0TRL     ;
    volatile Uint32 PCIBAR1TRL     ;
    volatile Uint32 PCIBAR2TRL     ;
    volatile Uint32 PCIBAR3TRL     ;
    volatile Uint32 PCIBAR4TRL     ;
    volatile Uint32 PCIBAR5TRL     ;
    volatile Uint8  RSVD9 [8]      ;
    volatile Uint32 PCIBARMIR [6]  ;
    volatile Uint8  RSVD10 [264]   ;
    volatile Uint32 PCIMCFGDAT     ;
    volatile Uint32 PCIMCFGADR     ;
    volatile Uint32 PCIMCFGCMD     ;
    volatile Uint8  RSVD11 [4]     ;
    volatile Uint32 PCIMSTCFG      ;
    volatile Uint32 PCIADDSUB [32] ;
    volatile Uint32 PCIVENDEVPRG   ;
    volatile Uint32 PCICMDSTATPRG  ;
    volatile Uint32 PCICLREVPRG    ;
    volatile Uint32 PCISUBIDPRG    ;
    volatile Uint32 PCIMAXLGPRG    ;
    volatile Uint32 PCILRSTREG     ;
    volatile Uint32 PCICFGDONE     ;
    volatile Uint32 PCIBAR0MPRG    ;
    volatile Uint32 PCIBAR1MPRG    ;
    volatile Uint32 PCIBAR2MPRG    ;
    volatile Uint32 PCIBAR3MPRG    ;
    volatile Uint32 PCIBAR4MPRG    ;
    volatile Uint32 PCIBAR5MPRG    ;
    volatile Uint32 PCIBAR0PRG     ;
    volatile Uint32 PCIBAR1PRG     ;
    volatile Uint32 PCIBAR2PRG     ;
    volatile Uint32 PCIBAR3PRG     ;
    volatile Uint32 PCIBAR4PRG     ;
    volatile Uint32 PCIBAR5PRG     ;
    volatile Uint32 PCIBAR0TRLPRG  ;
    volatile Uint32 PCIBAR1TRLPRG  ;
    volatile Uint32 PCIBAR2TRLPRG  ;
    volatile Uint32 PCIBAR3TRLPRG  ;
    volatile Uint32 PCIBAR4TRLPRG  ;
    volatile Uint32 PCIBAR5TRLPRG  ;
    volatile Uint32 PCIBASENPRG    ;
} DM6437PCI_pciRegs ;


struct pci_dev * GEM = NULL ;

Uint32   regBase = 0 ;
Uint32   memBase = 0 ;
Uint32   ddrRegBase = 0 ;

Uint32 * regVirt = NULL ;
Uint32 * memVirt = NULL ;
Uint32 * ddrRegVirt = NULL ;

Uint32   memLen  = 0 ;
Uint32   regLen  = 0 ;
Uint32   ddrRegLen  = 0 ;


/* ============================================================================
 *  @func   PCI_FindPciDevices
 *
 *  @desc   This function locates DM642 PCI cards on system.
 *
 *  @modif  None.
 *  ============================================================================
 */
Void
PCI_FindPciDevices (Void)
{
    struct pci_dev * dev = NULL ;

    while ((dev = pci_find_device (PCI_TI_VENDOR, PCI_TI_DEVICE, dev)) != NULL)
    {
        GEM = dev ;
        break ;
    }
}

/* ============================================================================
 *  @func   PCI_readBAR
 *
 *  @desc   This function reads config.
 *
 *  @modif  None.
 *  ============================================================================
 */
Void PCI_readBAR (Void)
{
    Uint32  barStart [3]  ;
    Uint32  barLen   [3]  ;
    Uint32  barFlags [3]  ;

    barStart [0] = pci_resource_start (GEM, 0);
    barLen   [0] = pci_resource_len   (GEM, 0);
    barFlags [0] = pci_resource_flags (GEM, 0);
    barStart [1] = pci_resource_start (GEM, 1);
    barLen   [1] = pci_resource_len   (GEM, 1);
    barFlags [1] = pci_resource_flags (GEM, 1);
    barStart [2] = pci_resource_start (GEM, 2);
    barLen   [2] = pci_resource_len   (GEM, 2);
    barFlags [2] = pci_resource_flags (GEM, 2);

    /* ---------------------------------------------------------------------
     * Map the L2RAM memory region
     * ---------------------------------------------------------------------
     */
    if (barFlags [0] & IORESOURCE_MEM) {
        memBase = barStart [0] ;
        /* Map the memory region. */
        request_mem_region (memBase,
                            barLen [0],
                            "DSPLINK");
    }
    else {
        /* Map the memory region. */
        request_region (memBase,
                        barLen [0],
                        "DSPLINK");
    }

    if (memBase > 0) {
        memVirt = ioremap (barStart [0],
                           barLen [0]) ;
    }

    /* ---------------------------------------------------------------------
     * Map the DDR REG memory region
     * ---------------------------------------------------------------------
     */
    if (barFlags [1] & IORESOURCE_MEM) {
        ddrRegBase = barStart [1] ;
        /* Map the memory region. */
        request_mem_region (ddrRegBase,
                            barLen [1],
                            "DSPLINK");
    }
    else {
        /* Map the memory region. */
        request_region (ddrRegBase,
                        barLen [1],
                        "DSPLINK");
    }

    if (ddrRegBase > 0) {
        ddrRegVirt = ioremap (barStart [1],
                              barLen [1]) ;
    }

    /* ---------------------------------------------------------------------
     * Map the REG memory region
     * ---------------------------------------------------------------------
     */
    if (barFlags [2] & IORESOURCE_MEM) {
        regBase = barStart [2] ;
        /* Map the memory region. */
        request_mem_region (regBase,
                            barLen [2],
                            "DSPLINK");
    }
    else {
        /* Map the memory region. */
        request_region (regBase,
                        barLen [2],
                        "DSPLINK");
    }

    if (regBase > 0) {
        regVirt = ioremap (barStart [2],
                           barLen [2]) ;
    }

    memLen    = barLen [0] ;
    ddrRegLen = barLen [1] ;
    regLen    = barLen [2] ;
}

/* =============================================================================
 *  @func   PCI_setMaster
 *
 *  @desc   This function makes the given device to be master.
 *
 *  @modif  None.
 *  ============================================================================
 */
Void
PCI_setMaster (void)
{
    Int32   retVal ;
    Uint16  cmdVal ;
    struct pci_dev * dev ;

    dev = GEM ;

    /* set the DMA mask */
    if (pci_set_dma_mask (dev, 0xfffffff0ULL)) {
    }

    /*
     * set the desired PCI dev to be master, this internally sets the latency
     * timer.
     */
    pci_set_master (dev) ;
    pci_write_config_byte(dev, PCI_LATENCY_TIMER, 0x80);

    /* Add support memory write invalidate */
    retVal = pci_set_mwi (dev) ;

    pci_read_config_word (dev, PCI_COMMAND, (u16 *) &cmdVal) ;
    /* and set the master bit in command register. */
    cmdVal |=   PCI_COMMAND_MEMORY
              | PCI_COMMAND_MASTER
              | PCI_COMMAND_SERR  ;
    /* and clear the interrupt disable bit in command register. */
    cmdVal &=   ~PCI_COMMAND_INTX_DISABLE;
    pci_write_config_word (dev, PCI_COMMAND, cmdVal) ;
}

static Uint8 * wData ;
static Uint8 * rData ;

/* ============================================================================
 *  @func   PCI_dma
 *
 *  @desc   Tests the DMA component.
 *
 *  @modif  None.
 *  ============================================================================
 */
Void PCI_dma (void)
{
    Int32 i ;
    volatile Uint32 addressToWrite = 0x0;
    volatile DM6437PCI_pciRegs * pciregs = (DM6437PCI_pciRegs *) ((Uint32) regVirt + (DM6437PCI_PCIREG_BASE - 0x01C00000)) ;

    wData = (Uint8 *) kmalloc (0x4000, GFP_DMA) ;
    rData = (Uint8 *) kmalloc (0x4000, GFP_DMA) ;

    for (i = 0 ; i < 0x4000 ; i++) {
         wData [i] = 0x41 ;
    }
    addressToWrite = (Uint32)virt_to_bus(wData);
    printk("Address to write from DSP = 0x%x\n",(int)addressToWrite);

    pciregs->PCIADDSUB [0] = ((Uint32) virt_to_bus (wData)) & 0xFF800000 ;
    pciregs->PCIADDSUB [0] = ((Uint32) virt_to_bus (rData)) & 0xFF800000 ;
}

/* =============================================================================
 *  @func   HAL_CheckPciInterrupt
 *
 *  @desc   This function check whether interrupt is generated by DM642 or not.
 *
 *  @modif  None.
 *  ============================================================================
 */
Bool
HAL_CheckPciInterrupt (void)
{
    volatile DM6437PCI_pciRegs * pciRegs = (DM6437PCI_pciRegs *) ((Uint32) regVirt + (DM6437PCI_PCIREG_BASE - 0x01C00000)) ;
    Bool                 status = 0 ;
    if (   (pciRegs->PCICSRMIR & DM6437PCI_INTSTATUS_MASK)
        == (DM6437PCI_INTSTATUS_MASK)) {
        status = 1 ;
    }
    return status ;
}


/** ============================================================================
 *  @func   HAL_PciClearDspInterrupt
 *
 *  @desc   Clear pending interrupt from DSP to Host.
 *
 *  @modif  None.
 *  ============================================================================
 */
Void
HAL_PciClearDspInterrupt (Void)
{
    volatile DM6437PCI_pciRegs * pciRegs = (DM6437PCI_pciRegs *) ((Uint32) regVirt + (DM6437PCI_PCIREG_BASE - 0x01C00000)) ;

    pciRegs->PCISTATCLR |= DM6437PCI_SOFTINT2_MASK ;
}


/** ============================================================================
 *  @func   HAL_PciEnableDspInterrupt
 *
 *  @desc   Allow the DSP to generate interrupts to the Host.
 *
 *  @modif  None.
 *  ============================================================================
 */
Void
HAL_PciEnableDspInterrupt (Void)
{
    volatile DM6437PCI_pciRegs * pciRegs = (DM6437PCI_pciRegs *) ((Uint32) regVirt + (DM6437PCI_PCIREG_BASE - 0x01C00000)) ;
    printk("PCI host interrupt register vaue = 0x%X\n",(int)pciRegs->PCIHINTSET);

    pciRegs->PCIHINTSET = 0x0;

    printk("PCI host interrupt register value = 0x%X\n", (int)pciRegs->PCIHINTSET);
    pciRegs->PCIHINTSET |= DM6437PCI_SOFTINT2_MASK ;

    printk("PCI host interrupt register value = 0x%X\n", (int)pciRegs->PCIHINTSET);
}


/** ============================================================================
 *  @func   HAL_PciDisableDspInterrupt
 *
 *  @desc   Disallow the DSP to generate interrupts to the Host.
 *
 *  @modif  None.
 *  ============================================================================
 */
Void
HAL_PciDisableDspInterrupt (Void)
{
    volatile DM6437PCI_pciRegs * pciRegs = (DM6437PCI_pciRegs *) ((Uint32) regVirt + (DM6437PCI_PCIREG_BASE - 0x01C00000)) ;

    pciRegs->PCIHINTCLR |= DM6437PCI_SOFTINT2_MASK ;
}


static int ISR_handler5 (int irq, void * arg, struct pt_regs * flags)
{
    Uint32 i =0;
    Uint32 status = HAL_CheckPciInterrupt ()  ;
    Uint8  comData[256];
    Uint32 flag = 0;

    volatile DM6437PCI_pciRegs * pciRegs = (DM6437PCI_pciRegs *)((Uint32)regVirt+ (DM6437PCI_PCIREG_BASE - 0x01c00000));

    for (i = 0; i<256; i++)
    {
       comData[i] = i;
    }

    if (status == 1)
    {
        HAL_PciClearDspInterrupt () ;
        printk ("<0>Interrupt received from DSP\n") ;

        for (i=0;i<256;i++)
        {
            if (comData[i] != wData[i])
            {
                printk("<0>comData: %d, wData: %d\n",comData[i],wData[i]);
                printk("<0>Data comparison fails....\n");
                flag = 1;
            }
        }
        if (flag == 0)
        {
            printk("<0>Data comparison success\n");
        }


    }
    else
    {
        return IRQ_NONE;
    }
    printk("Exiting from DSP to Host interrupt ISR\n");


    for(i=0;i<256;i++)
    {
        wData[i]= 127 -i;
    }

    printk("PCISTAT = 0x%X\n",(int)pciRegs->PCISTATSET);
    pciRegs->PCISTATSET = 0x0f000000;
    printk("PCISTAT = 0x%X\n",(int)pciRegs->PCISTATSET);

    printk("Host to DSP interrupt triggered\n");
    return IRQ_HANDLED;
}

/*  ----------------------------------------------------------------------------
 *  @func   HAL_changeState
 *
 *  @desc   Change power module state to ( ENABLE, DISABLE, SYNCRESET, RESET ).
 *
 *  @modif  None
 *  ----------------------------------------------------------------------------
 */
Void
HAL_changeState (Uint32 lpscNum, Uint16 state)
{
    volatile DM6437PCI_pscRegs * pscRegs = (DM6437PCI_pscRegs *) ((Uint32) regVirt + (DM6437PCI_PSCREG_BASE - 0x01C00000)) ;
    /*  ------------------------------------------------------------------------
     *  Step 1 - Wait for PTSTAT.GOSTAT to clear
     *  ------------------------------------------------------------------------
     */
    while (pscRegs->PTSTAT & 0x1) ;

    /*  ------------------------------------------------------------------------
     *  Step 2 - Set MDCTLx.NEXT to new state
     *  ------------------------------------------------------------------------
     */
    pscRegs->MDCTL [lpscNum] &= 0xFFE0 ;
    pscRegs->MDCTL [lpscNum] |= state  ;

    /*  ------------------------------------------------------------------------
     *  Step 3 - Start power transition ( set PTCMD.GO to 1 )
     *  ------------------------------------------------------------------------
     */
    pscRegs->PTCMD = 0x0001 ;

    /*  ------------------------------------------------------------------------
     *  Step 4 - Wait for PTSTAT.GOSTAT to clear
     *  ------------------------------------------------------------------------
     */
    while (pscRegs->PTSTAT & 0x1) ;

    /*  ------------------------------------------------------------------------
     *  Step 5 - Verify state changed
     *  ------------------------------------------------------------------------
     */
    while ((pscRegs->MDSTAT [lpscNum] & 0x001F) != state) ;
}


/*  ----------------------------------------------------------------------------
 *  @func   HAL_cfgPLL1
 *
 *  @desc   Configures the PLL1.
 *
 *  @modif  None
 *  ----------------------------------------------------------------------------
 */
Void
HAL_cfgPLL1 (Uint32       pllm,
             Uint32       div1,
             Uint32       div2)
{
    volatile DM6437PCI_pllRegs * pllRegs = (DM6437PCI_pllRegs *) ((Uint32) regVirt + (DM6437PCI_PLL1REG_BASE - 0x01C00000)) ;
    /*  ------------------------------------------------------------------------
     *  Step 1 - Set PLL to BYPASS mode and select clock source.
     *  ------------------------------------------------------------------------
     */
    /*  ------------------------------------------------------------------------
     *  Step 1 - Set PLL to BYPASS mode and select clock source.
     *  ------------------------------------------------------------------------
     */
    pllRegs->PLLCTL &= 0xFFFFFEFF ;
    pllRegs->PLLCTL |= 0x0 ;
    pllRegs->PLLCTL &= 0xFFFFFFDF ;
    pllRegs->PLLCTL &= 0xFFFFFFFE ;

    udelay (10000) ;

    pllRegs->PLLCTL &= 0xFFFFFFF7 ;
    pllRegs->PLLCTL |= 0x00000010 ;
    pllRegs->PLLCTL &= 0xFFFFFFFD ;
    pllRegs->PLLCTL &= 0xFFFFFFEF ;
    pllRegs->PLLCTL &= 0xFFFFFEFF ;
    pllRegs->PLLCTL |= 0x0 ;

    /*  ------------------------------------------------------------------------
     *  Step 2 - Load PLL multiplier.
     *  ------------------------------------------------------------------------
     */
    pllRegs->PLLM = pllm ;

    /*  ------------------------------------------------------------------------
     *  Step 3 - Load PLL2 divider - DDR2 and VPBE are programmable.
     *  ------------------------------------------------------------------------
     */
    pllRegs->PLLDIV1  =  div2 ;
    pllRegs->PLLDIV2  =  div1 ;
    pllRegs->PLLDIV1 &= 0xFFFF7FFF ;
    pllRegs->PLLDIV2 &= 0xFFFF7FFF ;
    pllRegs->PLLDIV1 |= 0x00008000 ;
    pllRegs->PLLDIV2 |= 0x00008000 ;

    /*  ------------------------------------------------------------------------
     *  Step 4 - Set phase alignment and Wait for operation to finish.
     *  ------------------------------------------------------------------------
     */
    pllRegs->PLLCMD |= 0x00000001 ;
    while ((pllRegs->PLLSTAT & 0x1) == 0x1) ;
    /*  ------------------------------------------------------------------------
     *  Step 5 - Wait for PLL to re-lock ( 2000 cycles ).
     *  ------------------------------------------------------------------------
     */
    udelay (10000) ;
    pllRegs->PLLCTL |= 0x00000008 ;

    /*  ------------------------------------------------------------------------
     *  Step 6 - Switch out of BYPASS mode.
     *  ------------------------------------------------------------------------
     */
    udelay (10000) ;
    pllRegs->PLLCTL |= 0x00000001 ;
}


/*  ----------------------------------------------------------------------------
 *  @func   HAL_cfgDDR2
 *
 *  @desc   Configures the DDR2 module.
 *
 *  @modif  None
 *  ----------------------------------------------------------------------------
 */
Void
HAL_cfgDDR2 (Uint32  freq)
{
    volatile DM6437PCI_ddrRegs * ddrRegs = (DM6437PCI_ddrRegs *) ((Uint32) ddrRegVirt) ;
    Uint32 *             vtpiocr = NULL ;
    Uint32 *             vtpr    = NULL ;
    Uint32 *             dftEnb  = NULL ;

    vtpiocr  = (Uint32 *) ((Uint32) ddrRegVirt  + 0xF0) ;
    vtpr     = (Uint32 *) ((Uint32) regVirt + 0x42038) ;
    dftEnb   = (Uint32 *) ((Uint32) regVirt + 0x4004C) ;

    /* Enable the DDR2 PSC Module */
    HAL_changeState (LPSC_DDR, 0x3) ;
    /*  ------------------------------------------------------------------------
     *  Step 1. Enable DDR2 PHY.
     *  ------------------------------------------------------------------------
     */
    ddrRegs->DDRPHYCR =  0x50006405 ;
    ddrRegs->SDBCR    =  0x00138822 ;
    ddrRegs->SDTIMR   =  0x16492148 ;
    ddrRegs->SDTIMR2  =  0x000CC702 ;
    ddrRegs->SDBCR    =  0x00130822 ;

    /*  ------------------------------------------------------------------------
     *  Step 3. Refresh Control [ 7.8 usec * freq in MHz ].
     *  ------------------------------------------------------------------------
     */
    ddrRegs->SDRCR = 0x4EF ;

    /* Reset the DDR2 PSC Module */
    HAL_changeState (LPSC_DDR, 0x1) ;
    /* Enable the DDR2 PSC Module */
    HAL_changeState (LPSC_DDR, 0x3) ;

    /*  ------------------------------------------------------------------------
     *  Step 4. Enable VTP calibration
     *  ------------------------------------------------------------------------
     */
    *vtpiocr  = 0x201F ;
    *vtpiocr  = 0xA01F ;

    /* wait for 33 VTP clock cycles */
    udelay (10000);
    *dftEnb   = 1 ;
    *vtpiocr  = 0xA000 | (*vtpr | 0x3FF) ;

    /* Wait for few more clock cycles */
    udelay (10000);
    *vtpiocr &= 0xFFFFDFFF ;
    *dftEnb   = 0 ;
}

#define DM6437PCI_LRESET_MASK   0x00000100
#define GEM_MDCTL               (0x01C41a00 + (4 * 39))
#define ADDR32(x)                 *((Uint32 *) ((Uint32)memVirt + x))
#define DM6437PCI_DSPBOOTADDR   0x01C4000C

#define OP1                     0x02084068 /* MVKH.S1       0x10800000,A4 */
#define OP2                     0x0090B362 /* BNOP.S2X      A4,5          */
Uint32 dummy ;

int init_module(void)
{
    Uint8 * writeData ;
    Uint8 * readData ;
    Uint32  i     ;

    writeData = (Uint8 *) kmalloc (0x8000, GFP_DMA) ;
    readData = (Uint8 *) kmalloc (0x8000, GFP_DMA) ;

    printk("PCIDRV - Find PCI Devices\n");
    PCI_FindPciDevices () ;

    mdelay(1000);
    printk("PCIDRV - Reading Bar\n");
    PCI_readBAR () ;

    mdelay(1000);
    printk("PCIDRV - Enable Device\n");
    pci_enable_device (GEM) ;

    mdelay(1000);
    printk("PCIDRV - Set DRV Data\n");
    pci_set_drvdata (GEM, memVirt) ;

    mdelay(1000);
    printk("PCIDRV - Set Master\n");
    PCI_setMaster () ;
#if 0
    /*This part of code is not been tested */
    /* Enable the DDR2 PSC Module */
    HAL_changeState (13, 0x3) ;
    /* Programme the PLL1 (DDR2) to run at higher frequency. */
    HAL_cfgPLL1 (23, 11, 1) ;
    /* Programme the DDR PHY register corresponding to PLL1 frequency. */
    HAL_cfgDDR2 (162) ;
    /* Reset the DDR2 PSC Module */
    HAL_changeState (13, 0x1) ;
    /* Enable the DDR2 PSC Module */
    HAL_changeState (13, 0x3) ;
#endif

    PCI_dma () ;
    for (i = 0 ; i < 0x8000 ; i++) {
         writeData [i] = 0x42 ;
    }
    printk("Code ends here\n");
    mdelay(1000);
    printk("PCIDRV: Registering interrupt\n");
    HAL_PciDisableDspInterrupt () ;
    printk("Interrupt number is : %d\n", GEM->irq);
    request_irq (GEM->irq,   ISR_handler5,   SA_SHIRQ, "PCIDRV", &dummy) ;
    HAL_PciEnableDspInterrupt () ;
    mdelay(1000);
    printk("PCIDRV: Interrupts registered\n");

    if (regVirt) {
        /* put GEM in reset */
        *((Uint32 *) ((Uint32) regVirt + (GEM_MDCTL - 0x01C00000))) &= ~DM6437PCI_LRESET_MASK ;

        /* Set the DSP boot address. */
        *((Uint32 *) ((Uint32) regVirt + (DM6437PCI_DSPBOOTADDR - 0x01C00000))) = 0x10800000 ;
    }
    mdelay(1000);
    printk("PCIDRV: GEM is reset and Boot address is set\n");
    if (memVirt) {
        /* Write the self loop code at 0x10800000 */
        ADDR32 (0x0)  = OP1 ;
        ADDR32 (0x4)  = OP2 ;
        ADDR32 (0x8)  = 0x0 ;
        ADDR32 (0xC)  = 0x0 ;
        ADDR32 (0x10) = 0x0 ;
        ADDR32 (0x14) = 0x0 ;
        ADDR32 (0x18) = 0x0 ;
        ADDR32 (0x1C) = 0x0 ;
    }
    mdelay(1000);
    printk("PCIDRV: Opcode written\n");
    if (regVirt) {
        /* release the GEM from reset */
        *((Uint32 *) ((Uint32) regVirt + (GEM_MDCTL - 0x01C00000))) |= DM6437PCI_LRESET_MASK ;
    }
    mdelay(1000);
    printk("PCIDRV: Release GEM reset\n");
    kfree(readData);
    kfree(writeData);
    return 0 ;
}

void cleanup_module(void)
{
    HAL_PciDisableDspInterrupt () ;

    iounmap(memVirt) ;
    if (pci_resource_flags (GEM, 0) & IORESOURCE_MEM) {
        /* Map the memory region. */
        release_mem_region (memBase,
                            memLen);
    }
    else {
        /* Map the memory region. */
        release_region (memBase,
                        memLen);
    }

    /* ---------------------------------------------------------------------
     * Unmap baseRegs region & release the reg region.
     * ---------------------------------------------------------------------
     */
    iounmap(ddrRegVirt) ;
    if (pci_resource_flags (GEM, 1) & IORESOURCE_MEM) {
        /* Map the memory region. */
        release_mem_region (ddrRegBase,
                            ddrRegLen);
    }
    else {
        /* Map the memory region. */
        release_region (ddrRegBase,
                        ddrRegLen);
    }

    /* ---------------------------------------------------------------------
     * Unmap baseRegs region & release the reg region.
     * ---------------------------------------------------------------------
     */
    iounmap(regVirt) ;
    if (pci_resource_flags (GEM, 2) & IORESOURCE_MEM) {
        /* Map the memory region. */
        release_mem_region (regBase,
                            regLen);
    }
    else {
        /* Map the memory region. */
        release_region (regBase,
                        regLen);
    }

    free_irq (GEM->irq, &dummy) ;
    /* Free the allocated memory for read-write */
    kfree((void*)rData);
    kfree((void*)wData);
}

