/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2007 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \file   ths8200.c
 *
 *  \brief  THS8200 Video Encoder related parameters
 *
 *  This file contains the interfaces, data types and symbolic definitions
 *  that are required for the THS8200 video encoder driver.
 *
 *  NOTE: Use file for configuring the external THS8200 video encoder.
 *
 *  (C) Copyright 2010, Texas Instruments, Inc
 *
 *  \author     Maulik Desai
 *
 *  \version    0.1     Created
 */

/*
 * ============ INCLUDE FILES ==========
 */

#include <ti/sdo/pspdrivers/system/dm6437/bios/evmDM6437/video/src/psp_i2c_interface.h>
#include <ti/sdo/pspdrivers/system/dm6437/bios/evmDM6437/video/src/THS8200_extVidEncoder.h>

/*
 * ========== FUNCTION DECLARATIONS =========
 */
static Int32 THS8200_setCSC(Int32 devAddr,
                            THS8200_InputFormat iFmt,
                            THS8200_AnalogFormat aFmt);
static Int32 THS8200_setInputFormat(Int32 devAddr,
                                    THS8200_Mode mode,
                                    THS8200_InputFormat iFmt);
static Int32 THS8200_setVideoMode(Int32 devAddr, THS8200_Mode mode);
static Int32 THS8200_setOpMode(Int32 devAddr,
                               THS8200_Mode mode,
                               Bool enableSlaveMode);
static Int32 THS8200_setEmbeddedSync(Int32 devAddr, Bool enableBT656Sync);
static Int32 THS8200_setAnalogFormat(Int32 devAddr, THS8200_AnalogFormat aFmt);
static Int32 THS8200_config(Int32 devAddr, THS8200_ConfParams *);
static Int32 THS8200_powerdown(Int32 devAddr, Bool enable);
static Int32 THS8200_reset(Int32 devAddr);

/*
 * ========== FUNCTION POINTER TABLE =========
 */
PSP_VPSS_ExtVideoEncoderFxns THS8200_Fxns =
{
    THS8200_open,
    THS8200_close,
    THS8200_ctrl
};

Uint8 THS8200_Regs[256] =
{
    0
};

/**
 *  \brief  I2C write register
 *
 *  This function updates register array with i2c write(only in debugging mode)
 *
 *  \param  slaveAddr  [In] slave address of i2c device
 *  \param  subAddr    [In] sub address of slave on which write
 *                          operation to be performed
 *  \param  buffer     [In] buffer pointer to write values
 *  \param  length     [In] number of bytes to write
 *
 *  \return EVENC_SUCCESS on successful operation
 *          EVENC_FAIL on failure of write operation
 */
inline static Int32 regSetTHS8200(Uint8 slaveAddr,
                                  Uint8 subAddr,
                                  const Uint8 *buffer,
                                  Uint32 length)
{
    Int32 count = 0;
    Int32 status = EVENC_FAIL;
    THS8200_Regs[0] = subAddr;
    for (count = 1; count <= length; count++)
    {
        THS8200_Regs[count] = buffer[count-1];
    }
    status = i2c_writeReg(slaveAddr, THS8200_Regs, length + 1u);

    if(TRUE == status)
    {
       status  = EVENC_SUCCESS;
    }
    return (status);

}

/**
 *  \brief  Set Colour Space Conversion Registers
 *
 *  \param  devAddr [In]  Slave address of THS8200
 *  \param  iFmt    [In]  Digtal input format
 *  \param  aFmt    [In]  Analog output format
 *
 *  \return EVENC_SUCCESS on successful operation
 *          EVENC_FAIL if error occured
 */
static Int32 THS8200_setCSC(Int32 devAddr,
                            THS8200_InputFormat iFmt,
                            THS8200_AnalogFormat aFmt)
{
    Int32 status = EVENC_SUCCESS;
    Uint8 inColourSpace, outColourSpace;
    const _THS8200_CSC_Regs *pCscRegSet = NULL;

    if((THS8200_IFMT_YCBCR422_10_BIT == iFmt)||
       (THS8200_IFMT_YCBCR422_20_BIT == iFmt)||
       (THS8200_IFMT_YCBCR444 == iFmt))
    {
        inColourSpace = COLOUR_SPACE_YCBCR;
    }
    else
    {
        status = EVENC_FAIL;
    }
    switch (aFmt)
    {
        case THS8200_AFMT_RGB_NO_SYNC:
        case THS8200_AFMT_RGB_SYNC_ON_GREEN:
        case THS8200_AFMT_RGB_SYNC_ON_ALL:
            outColourSpace = COLOUR_SPACE_RGB;
        break;

        case THS8200_AFMT_YPBPR_SYNC_ON_Y:
        case THS8200_AFMT_YPBPR_SYNC_ON_ALL:
            outColourSpace = COLOUR_SPACE_YCBCR;
        break;
        default:
            status = EVENC_FAIL;
        break;
    } /* switch (aFmt) */

    if (EVENC_SUCCESS == status)
    {
        if (inColourSpace == outColourSpace)
        {
            /* No conversion required */
           pCscRegSet = &ths8200_CSC_Disable_Regs;
        }
        else if (COLOUR_SPACE_RGB == inColourSpace)
        {
            /* RGB to YCbCr conversion is required */
            pCscRegSet = &ths8200_CSC_RGB_To_YCbCr_Regs;
        }
        else
        {
            /* YCbCr to RGB conversion is required */
            pCscRegSet = &ths8200_CSC_YCbCr_To_RGB_Regs;
        }

        status = regSetTHS8200(devAddr,
                               THS8200_CSC_R11_ADDR,
                               &(pCscRegSet->csc_r11),
                               THS8200_CSC_NUM_REGISTERS);
    }
    return status;
}

/**
 *  \brief  Set input format
 *
 *  \param  devAddr [In]  Slave address of THS8200
 *  \param  mode    [In]  THS8200 video mode
 *  \param  iFmt    [In]  Digtal input format
 *
 *  \return EVENC_SUCCESS on successful operation
 *          EVENC_FAIL if error occured
 */
/*ARGSUSED*/
static Int32 THS8200_setInputFormat(Int32 devAddr,
                                    THS8200_Mode mode,
                                    THS8200_InputFormat iFmt)
{
    Int32 status = EVENC_SUCCESS;
    Bool result = FALSE;
    Uint8 numBytes = 1,dataPathCtrl = 0;
    Uint8 buffer[2] = {0};
    /*Copy the SubAddres to buffer */
    buffer[0] = THS8200_DATA_CNTL_ADDR;
    status = i2c_readReg(devAddr,
                         buffer,
                         numBytes + 1u);

    /*Copy the data to local variable */
    dataPathCtrl = buffer[1];

    dataPathCtrl &= ~THS8200_DATA_DMAN_CTL_MASK; /* Clear DMAN control bits */
    switch (iFmt)
    {
        case THS8200_IFMT_YCBCR422_10_BIT:
            dataPathCtrl |= THS8200_DATA_DMAN_CTL_10BIT_YCBCR_VAL;
            dataPathCtrl &= ~THS8200_DATA_IFIR12_BYPASS_BIT;
        break;

        case THS8200_IFMT_YCBCR422_20_BIT:
            dataPathCtrl |= THS8200_DATA_DMAN_CTL_20BIT_YCBCR_VAL;
            dataPathCtrl &= ~THS8200_DATA_IFIR12_BYPASS_BIT;
        break;

        case THS8200_IFMT_YCBCR444:
        case THS8200_IFMT_RGB444:
            dataPathCtrl |= THS8200_DATA_DMAN_CTL_30BIT_VAL;
            dataPathCtrl |= THS8200_DATA_IFIR12_BYPASS_BIT;
        break;

        case THS8200_IFMT_RGB565:
            dataPathCtrl |= THS8200_DATA_DMAN_CTL_16BIT_RGB_VAL;
            dataPathCtrl |= THS8200_DATA_IFIR12_BYPASS_BIT;
        break;

        case THS8200_IFMT_RGB555:
            dataPathCtrl |= THS8200_DATA_DMAN_CTL_15BIT_RGB_VAL;
            dataPathCtrl |= THS8200_DATA_IFIR12_BYPASS_BIT;
        break;

        default:
            status = EVENC_FAIL;
        break;
    } /* switch (iFmt) */

    if (status != EVENC_FAIL)
    {
        /* Set the other bits in DMAN Control register */
        dataPathCtrl |= THS8200_DATA_FSADJ_BIT;
        dataPathCtrl &= ~THS8200_DATA_IFIR35_BYPASS_BIT;

        /* Configure digital input format */
        buffer[0] = THS8200_DATA_CNTL_ADDR;
        buffer[1] = dataPathCtrl;
        result    = i2c_writeReg(devAddr,buffer,numBytes + 1u);
        if(FALSE == result)
        {
            status = EVENC_FAIL;
        }
        else
        {
            status = EVENC_SUCCESS;
        }
    }
    return status;
}

/**
 *  \brief  Set video mode
 *
 *  \param  devAddr [In]  Slave address of THS8200
 *  \param  mode    [In]  THS8200 video mode
 *
 *  \return EVENC_SUCCESS on successful operation
 *          EVENC_FAIL if error occured
 */
static Int32 THS8200_setVideoMode(Int32 devAddr, THS8200_Mode mode)
{
    Int32 status = EVENC_SUCCESS;
    Uint8 numBytes = 1;
    const _THS8200_Regs *pRegSet = NULL;

    /* Select register set according to video format */
    if(THS8200_MODE_480P_60HZ == mode)
    {
        pRegSet = &ths8200_480p_60Hz_Regs;
    }
    else
    {
        status = EVENC_FAIL;
    }
    /* Configure video mode specific registers */
    if (status != EVENC_FAIL)
    {
        status = regSetTHS8200(devAddr,
                               THS8200_CHIP_CTL_ADDR,
                               &(pRegSet->chip_cntl),
                               numBytes);

        status = regSetTHS8200(devAddr,
                               THS8200_TST_CNTL1_ADDR,
                               &(pRegSet->tst_cntl1),
                               THS8200_TST_CNTL1_ADDR_CNT);

        status |= regSetTHS8200(devAddr,
                                THS8200_DTG1_Y_SYNC1_LSB_ADDR,
                                &(pRegSet->dtg_y_sync1),
                                THS8200_DTG1_Y_SYNC1_LSB_ADDR_CNT);

        status |= regSetTHS8200(devAddr,
                                THS8200_CSM_GY_CNTL_MULT_MSB_ADDR,
                                &(pRegSet->csm_gy_cntl_mult_msb),
                                numBytes);

        status |= regSetTHS8200(devAddr,
                                THS8200_CSM_RCR_BCB_CNTL_ADDR,
                                &(pRegSet->csm_rcr_bcb_cntl),
                                numBytes);

        status |= regSetTHS8200(devAddr,
                                THS8200_DTG2_HLENGTH_LSB_ADDR,
                                &(pRegSet->dtg2_hlength_lsb),
                                THS8200_DTG2_HLENGTH_LSB_ADDR_CNT);

        status |= regSetTHS8200(devAddr,
                                THS8200_DTG2_CNTL_ADDR,
                                &(pRegSet->dtg2_cntl),
                                numBytes);
    }
    return status;
}

/**
 *  \brief  Set operation mode like master mode or slave mode
 *
 *  \param  devAddr         [In]  Slave address of THS8200
 *  \param  mode            [In]  THS8200 video mode
 *  \param  enableSlaveMode [In]  Flag to enable or disable slave mode
 *
 *  \return EVENC_SUCCESS on successful operation
 *          EVENC_FAIL if error occured
 */
/*ARGSUSED*/
static Int32 THS8200_setOpMode(Int32 devAddr,
                               THS8200_Mode mode,
                               Bool enableSlaveMode)
{
    Uint8 chipCtrl = 0,dtg1Mode = 0;
    Int32 status   = EVENC_SUCCESS;
    Uint8 numBytes = 1;
    Uint8 readbuffer[2] ={0};
    /* Read the previous value */
    readbuffer[0] = THS8200_CHIP_CTL_ADDR;
    status = i2c_readReg(devAddr, readbuffer, numBytes + 1u);
    if(TRUE == status)
    {
        chipCtrl = readbuffer[1];
        status = EVENC_SUCCESS;
    }
    readbuffer[0] = THS8200_DTG1_MODE_ADDR;
    status |= i2c_readReg(devAddr, readbuffer,numBytes + 1u);
    if(TRUE == status)
    {
        dtg1Mode = readbuffer[1];
        status = EVENC_SUCCESS;
    }

    /* Configure operation mode */
    if (EVENC_SUCCESS == status)
    {
        if (enableSlaveMode)
        {
            /* Slave mode set bit1 = 0 for slave mode */
            chipCtrl &= ~THS8200_SYSCTRL_CHIP_MS_BIT;
        }
        else
        {
            status = EVENC_FAIL;
        }

        /* configure operation mode specific registers */
        status |= regSetTHS8200(devAddr,
                                THS8200_CHIP_CTL_ADDR,
                                &chipCtrl,
                                numBytes);
        status |= regSetTHS8200(devAddr,
                                THS8200_DTG1_MODE_ADDR,
                                &dtg1Mode,
                                numBytes);
    }

    return status;
}

/**
 *  \brief  Set registers for embedded sync mode
 *
 *  \param  devAddr         [In]  Slave address of THS8200
 *  \param  enableBT656Sync [In]  Flag to enable or disable BT656 sync
 *
 *  \return EVENC_SUCCESS on successful operation
 *          EVENC_FAIL if error occured
 */
static Int32 THS8200_setEmbeddedSync(Int32 devAddr, Bool enableBT656Sync)
{
    Uint8 dtg2Ctrl = 0;
    Bool result;
    Int32 status = EVENC_SUCCESS;
    Uint8 numBytes = 1;
    Uint8 readBuffer[2] = {0};
    /* Read the previous value */
    readBuffer[0] = THS8200_DTG2_CNTL_ADDR;
    result = i2c_readReg(devAddr, readBuffer, numBytes+1u);
    if(TRUE == result)
    {
        dtg2Ctrl = readBuffer[1];
        status   = EVENC_SUCCESS;
    }

    if (enableBT656Sync)
    {
        /* bit 5 = TRUE for embedded sync */
        dtg2Ctrl |= THS8200_DTG2_EMBEDDED_SYNC_BIT;
    }
    else
    {
        /* bit 5 = FALSE for external sync*/
        dtg2Ctrl &= ~THS8200_DTG2_EMBEDDED_SYNC_BIT;
    }

    if (EVENC_SUCCESS == status)
    {
        /* Configure DTG2 control register */
        status |= regSetTHS8200(devAddr,
                                THS8200_DTG2_CNTL_ADDR,
                                &dtg2Ctrl,
                                numBytes);
    }
    return status;
}

/**
 *  \brief  Set analog output mode
 *
 *  \param  devAddr [In]  Slave address of THS8200
 *  \param  afmt    [In]  THS8200 analog output mode
 *
 *  \return EVENC_SUCCESS on successful operation
 *          EVENC_FAIL if error occured
 */
static Int32 THS8200_setAnalogFormat(Int32 devAddr, THS8200_AnalogFormat aFmt)
{
    Int32 status = EVENC_SUCCESS;
    Uint16 ySyncLowVal = 0, ySyncHighVal = 0;
    Uint16 cbcrSyncLowVal = 0, cbcrSyncHighVal = 0;
    Uint8 outColourSpace = 0;
    Uint8 lowByte = 0, highByte = 0;
    Uint8 dtg2Ctrl = 0;
    Uint8 numBytes = 1;
    Bool result ;
    Uint8 readBuffer[2] = {0};
    switch (aFmt)
    {
        case THS8200_AFMT_RGB_NO_SYNC:
            ySyncLowVal = THS8200_NO_SYNC_VAL;
            ySyncHighVal = THS8200_NO_SYNC_VAL;
            cbcrSyncLowVal = THS8200_NO_SYNC_VAL;
            cbcrSyncHighVal = THS8200_NO_SYNC_VAL;

            outColourSpace = COLOUR_SPACE_RGB;
        break;

        case THS8200_AFMT_RGB_SYNC_ON_GREEN:
            ySyncLowVal = THS8200_SYNC_ON_Y_LOW;
            ySyncHighVal = THS8200_SYNC_ON_Y_HIGH;
            cbcrSyncLowVal = THS8200_NO_SYNC_VAL;
            cbcrSyncHighVal = THS8200_NO_SYNC_VAL;

            outColourSpace = COLOUR_SPACE_RGB;
        break;

        case THS8200_AFMT_RGB_SYNC_ON_ALL:
            ySyncLowVal = THS8200_SYNC_ON_Y_LOW;
            ySyncHighVal = THS8200_SYNC_ON_Y_HIGH;
            cbcrSyncLowVal = THS8200_SYNC_ON_CBCR_LOW;
            cbcrSyncHighVal = THS8200_SYNC_ON_CBCR_HIGH;

            outColourSpace = COLOUR_SPACE_RGB;
        break;

        case THS8200_AFMT_YPBPR_SYNC_ON_Y:
            ySyncLowVal = THS8200_SYNC_ON_Y_LOW;
            ySyncHighVal = THS8200_SYNC_ON_Y_HIGH;
            cbcrSyncLowVal = THS8200_NO_SYNC_VAL;
            cbcrSyncHighVal = THS8200_NO_SYNC_VAL;

            outColourSpace = COLOUR_SPACE_YCBCR;
        break;

        case THS8200_AFMT_YPBPR_SYNC_ON_ALL:
            ySyncLowVal = THS8200_SYNC_ON_Y_LOW;
            ySyncHighVal = THS8200_SYNC_ON_Y_HIGH;
            cbcrSyncLowVal = THS8200_SYNC_ON_CBCR_LOW;
            cbcrSyncHighVal = THS8200_SYNC_ON_CBCR_HIGH;

            outColourSpace = COLOUR_SPACE_YCBCR;
        break;

        default:
            status = EVENC_FAIL;
        break;
    } /* switch (aFmt) */

    /* Configure analog output format */
    if (EVENC_SUCCESS == status)
    {
        /* Write Y Sync Low LSB */
        lowByte = (Uint8)(ySyncLowVal & THS8200_AFMT_LOWER_BYTE_MASK);
        status |= regSetTHS8200(devAddr,
                                THS8200_DTG1_Y_SYNC2_LSB_ADDR,
                                &lowByte,
                                numBytes);

        /* Write Y Sync High LSB */
        lowByte = (Uint8)(ySyncHighVal & THS8200_AFMT_LOWER_BYTE_MASK);
        status |= regSetTHS8200(devAddr,
                                THS8200_DTG1_Y_SYNC3_LSB_ADDR,
                                &lowByte,
                                numBytes);

        /* Write CbCr Sync Low LSB */
        lowByte = (Uint8)(cbcrSyncLowVal & THS8200_AFMT_LOWER_BYTE_MASK);
        status |= regSetTHS8200(devAddr,
                                THS8200_DTG1_CBCR_SYNC2_LSB_ADDR,
                                &lowByte,
                                numBytes);

        /* Write CbCr Sync High LSB */
        lowByte = (Uint8)(cbcrSyncHighVal & THS8200_AFMT_LOWER_BYTE_MASK);
        status |= regSetTHS8200(devAddr,
                                THS8200_DTG1_CBCR_SYNC3_LSB_ADDR,
                                &lowByte,
                                numBytes);

        /* Update Y Sync Low and High MSB */
        readBuffer[0] = THS8200_DTG1_Y_SYNC_MSB_ADDR;
        result = i2c_readReg(devAddr,readBuffer,numBytes+1u);
        if(TRUE == result)
        {
            highByte = readBuffer[1];
            status = EVENC_SUCCESS;
        }
        highByte &= THS8200_AFMT_HIGHER_BYTE_READ_MASK;
        /* write (((ySyncLowVal & 0300) >> 6) || ((ySyncHighVal & 0300) >> 8)))
         * to bit 0 to 3 of 0x23
         */
        highByte |=
            (Uint8)(((ySyncLowVal & THS8200_AFMT_HIGHER_BYTE_MASK)
            >> THS8200_AFMT_LOW_VAL_SHIFT)|
            ((ySyncHighVal & THS8200_AFMT_HIGHER_BYTE_MASK)
            >> THS8200_AFMT_HIGH_VAL_SHIFT));

        status |= regSetTHS8200(devAddr,
                                THS8200_DTG1_Y_SYNC_MSB_ADDR,
                                &highByte,
                                numBytes);

        /* Update CbCr Sync Low and High MSB */
        readBuffer[0] = THS8200_DTG1_CBCR_SYNC_MSB_ADDR;
        result = i2c_readReg(devAddr,readBuffer, numBytes+1u);
        if(TRUE == result)
        {
            highByte = readBuffer[1];
            status = EVENC_SUCCESS;
        }

        highByte &= THS8200_AFMT_HIGHER_BYTE_READ_MASK;

        /* Write (((cbcrSyncLowVal & 0300)>>6)|((cbcrSyncHighVal & 0300)>>8)))
         to bit 0 to 3 of 0x24 */
        highByte |=
            (Uint8)(((cbcrSyncLowVal & THS8200_AFMT_HIGHER_BYTE_MASK)
            >> THS8200_AFMT_LOW_VAL_SHIFT)|
            ((cbcrSyncHighVal & THS8200_AFMT_HIGHER_BYTE_MASK)
            >> THS8200_AFMT_HIGH_VAL_SHIFT));

        status |= regSetTHS8200(devAddr,
                                THS8200_DTG1_CBCR_SYNC_MSB_ADDR,
                                &highByte,
                                numBytes);

        /* Read value of dtg2_cntl to set the Analog colour format */
        readBuffer[0] = THS8200_DTG2_CNTL_ADDR;
        result = i2c_readReg(devAddr,readBuffer,numBytes+1u);
        if(TRUE == result)
        {
            dtg2Ctrl  = readBuffer[1];
            status = EVENC_SUCCESS;
        }
        else
        {
            status  = EVENC_FAIL;
        }
        if (COLOUR_SPACE_RGB == outColourSpace) /* RGB mode */
        {
            /* Set RGB output mode */
            dtg2Ctrl |= THS8200_DTG2_RGB_MODE_ON_BIT;
        }
        else
        {
            /* Set YPbPr output mode */
            dtg2Ctrl &= ~THS8200_DTG2_RGB_MODE_ON_BIT;
        }

        /* Configure RGB or YPBPR format in dtg2_cntl */
        status |= regSetTHS8200(devAddr,
                                THS8200_DTG2_CNTL_ADDR,
                                &dtg2Ctrl,
                                numBytes);
    }

    return status;
}
/**
 *  \brief  Configure registers of THS8200
 *
 *  \param  devAddr  [In]  Slave address of THS8200
 *  \param  params   [In]  THS8200 configuration parameter structure pointer
 *
 *  \return EVENC_SUCCESS on successful operation
 *          EVENC_FAIL if error occured
 */
static Int32 THS8200_config(Int32 devAddr, THS8200_ConfParams *params)
{
    Int32 status = EVENC_SUCCESS;

    /* Configure video mode */
    status = THS8200_setVideoMode(devAddr, params->mode);

    if (EVENC_SUCCESS == status)
    {
        /* Config Colour Space Conversion (CSC) */
        status = THS8200_setCSC(devAddr, params->iFmt, params->aFmt);
    }

    if (EVENC_SUCCESS == status)
    {
        /* Configure input format */
        status = THS8200_setInputFormat(devAddr, params->mode, params->iFmt);
    }

    if (EVENC_SUCCESS == status)
    {
        status = THS8200_setAnalogFormat(devAddr, params->aFmt);
    }

    if (EVENC_SUCCESS == status)
    {
        status = THS8200_setOpMode(devAddr, params->mode,
                                   params->enableSlaveMode);
    }

    if (EVENC_SUCCESS == status)
    {
        status = THS8200_setEmbeddedSync(devAddr, params->enableBT656Sync);
    }

    /* Reset chip so that THS8200 synchronize with the new input format */
    status |= THS8200_reset(devAddr);

    return status;
}

/**
 *  \brief  Power Up or Power Down THS8200
 *
 *  \param  devAddr  [In]  Slave address of THS8200
 *  \param  enable   [In]  Flag to power-up or power-down THS8200
 *
 *  \return EVENC_SUCCESS on successful operation
 *          EVENC_FAIL if error occured
 */
static Int32 THS8200_powerdown(Int32 devAddr, Bool enable)
{
    Int32 status = EVENC_SUCCESS;
    Uint8 chipCtrl = 0;
    Uint8 numBytes = 1;
    Uint8 buffpwrdwn[2] = {0};
    /* First Check if the Handle is correct */
    if (devAddr != THS8200_I2CADDR)
    {
        status = EVENC_FAIL;
    }
    else
    {
        /* Read chip control register to set/reset the powerdown bits */
        buffpwrdwn[0] = THS8200_CHIP_CTL_ADDR;
        status   = i2c_readReg(devAddr,buffpwrdwn ,numBytes + 1u);
        chipCtrl = buffpwrdwn[1];

        if (enable)
        {
            /* Put THS8200 in power down mode */
            chipCtrl |= (THS8200_SYSCTRL_DAC_PWDN_BIT |
                         THS8200_SYSCTRL_CHIP_PWDN_BIT);
        }
        else
        {
            /* Put THS8200 in power up mode */
            chipCtrl &= ~(THS8200_SYSCTRL_DAC_PWDN_BIT |
                          THS8200_SYSCTRL_CHIP_PWDN_BIT);
            chipCtrl |= THS8200_SYSCTRL_ARST_FUNC_N_BIT;
        }

        /* Update the register */
        if (status != EVENC_FAIL)
        {
            status = regSetTHS8200(devAddr,
                                   THS8200_CHIP_CTL_ADDR,
                                   &chipCtrl,
                                   numBytes);
        }
    }
  return status;
}

/**
 *  \brief  Reset THS8200
 *
 *  \param  devAddr   [In]  Slave address of THS8200
 *
 *  \return EVENC_SUCCESS on successful operation
 *          EVENC_FAIL if error occured
 */
static Int32 THS8200_reset(Int32 devAddr)
{
    Int32 status = EVENC_SUCCESS;
    Uint8 chipCtrl;
    Uint8 numBytes = 1;
    Uint8 readBuffer[2] = {0};

    /* Read chip control register to set the software reset bit */
    readBuffer[0] = THS8200_CHIP_CTL_ADDR;
    status |= i2c_readReg(devAddr, readBuffer, numBytes+1u);
    chipCtrl = readBuffer[1];

    /* Enable software reset bit (Set 0) */
    chipCtrl &= ~THS8200_SYSCTRL_ARST_FUNC_N_BIT;
    status |= regSetTHS8200(devAddr,
                            THS8200_CHIP_CTL_ADDR,
                            &chipCtrl,
                            numBytes);

    /* Disable software reset bit (Set 1 for normal operation) */
    chipCtrl |= THS8200_SYSCTRL_ARST_FUNC_N_BIT;
    status |= regSetTHS8200(devAddr,
                            THS8200_CHIP_CTL_ADDR,
                            &chipCtrl,
                            numBytes);

    return status;
}

/**
 *  \brief  THS8200 Close
 *
 *  \param  handle [IN]  External device channel handle
 *
 *  \return  Success    EVENC_SUCCESS
 *           Error      EVENC_FAIL
 */
Int32 THS8200_close(EVENC_Handle handle)
{
    Int32 status = EVENC_SUCCESS;

    /* First Check if the Handle is correct */
    if (THS8200_I2CADDR == (Int32)handle)
    {
        /* Put THS8200 in power down mode, all dac and digital logic except
           I2C */
        status |= THS8200_powerdown((Int32)handle, TRUE);
    }
    else
    {
        status = EVENC_FAIL;
    }

    if (EVENC_SUCCESS == status)
    {
        if (i2c_deInit() != EVENC_SUCCESS)
        {
            status = EVENC_FAIL;
        }
    }
    return status;
}

/**
 *  \brief  THS8200 Ioctls
 *
 *  \param  handle [IN]  External device channel handle
 *  \param  cmd    [IN]  IOCTL to be performed (mostly enum)
 *  \param  param  [IN]  Arguement as required by Ioctl
 *
 *  \return  Success    EVENC_SUCCESS
 *           Error      EVENC_FAIL
 */
Int32 THS8200_ctrl(EVENC_Handle handle, Uint32 cmd, Ptr arg1)
{
    Int32 status = EVENC_SUCCESS;
    Int32 devAddr = (Int32)handle;

    /* First Check if the Handle is correct */
    if (THS8200_I2CADDR == (Int32)handle)
    {
        switch (cmd)
        {
            case PSP_VPSS_EXT_VIDEO_ENCODER_CONFIG:
                status = THS8200_config(devAddr, (void*)arg1);
            break;
            case PSP_VPSS_EXT_VIDEO_ENCODER_RESET:
                status = THS8200_reset(devAddr);
            break;
            default:
                status = EVENC_FAIL;
            break;
        }
    }
    else
    {
        status = EVENC_FAIL;
    }

    return status;
}

/**
 *  \brief  THS8200 device open
 *
 *  \param  devName [IN]  External device name
 *  \param  optArg  [IN]  Optional arguement (not used here)
 *
 *  \return handle  EDC channel handle when success
 *                  EVENC_FAIL when unsuccessful
 */
/*ARGSUSED */
EVENC_Handle THS8200_open()
{
    Int32 status = EVENC_SUCCESS;
    Int32 devId = 0;

    /* Open i2c device */
    if (TRUE == i2c_init())
    {
        devId = THS8200_I2CADDR;
        /* Power up the device */
        status = THS8200_powerdown(devId, FALSE);
    }
    else
    {
        status = EVENC_FAIL;
    }

    if (EVENC_FAIL == status)
    {
        devId = EVENC_FAIL;
    }
    return ((EVENC_Handle)(devId));
}

