#include <std.h>
#include <log.h>
#include <sys.h>
#include <mem.h>
#include <sio.h>
#include <iom.h>
#include <sem.h>

#ifdef CHIP_DM648
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/audio/psp_audio.h>
#endif

#ifdef CHIP_C6452
#include <ti/sdo/pspdrivers/system/c6452/bios/evm6452/audio/psp_audio.h>
#endif

extern far LOG_Obj trace;
extern Int edma3init();

/*
 * Buffers placed in external memory are aligned on a 128 bytes boundary.
 * In addition, the buffer should be of a size multiple of 128 bytes for
 * the cache work optimally on the C6x.
 */
#define BUFLEN                  2560         /* number of samples in the frame */
#define BUFALIGN                128          /* alignment of buffer to allow use of L2 cache */


/** Number of serializers configured for record */
#ifdef ONE_TO_ONE_TEST
#define RX_NUM_SERIALIZER          (1u)
#define TX_NUM_SERIALIZER          (1u)
#endif /* ONE_TO_ONE_TEST */

#ifdef FOUR_TO_ONE_TEST
#define RX_NUM_SERIALIZER          (4u)
#define TX_NUM_SERIALIZER          (1u)
#endif /* FOUR_TO_ONE_TEST */


#define BUFSIZE                 (BUFLEN * sizeof(MdUns))

#define NUM_BUFS                4            /*Number of Buffers to be issued and reclaimed*/

#define NEXT_SER_PLAY_SIZE      ((BUFLEN/4)/2)


/* inStream and outStream are SIO handles created in main */
static SIO_Handle inStream, outStream;

/* Function prototype */
static Void createStreams();
static Void prime();


/*
 * ======== createStreams ========
 */
static Void createStreams()
{
    SIO_Attrs attrs;

    /* align the buffer to allow it to be used with L2 cache */
    attrs       = SIO_ATTRS;
    attrs.nbufs = NUM_BUFS;
    attrs.align = BUFALIGN;
    attrs.model = SIO_ISSUERECLAIM;

    /* open the I/O streams */
    outStream = SIO_create("/dioCodec", SIO_OUTPUT, BUFSIZE, &attrs);
    if (outStream == NULL) {
        SYS_abort("Create output stream FAILED.");
    }
    inStream = SIO_create("/dioCodec", SIO_INPUT, BUFSIZE, &attrs);
    if (inStream == NULL) {
        SYS_abort("Create input stream FAILED.");
    }
}

Ptr buf[NUM_BUFS * 2];
/*
 * ======== prime ========
 */
static Void prime()
{

    Int32 count = 0;

    /* Allocate buffers for the SIO buffer exchanges */
    for(count = 0; count < (NUM_BUFS ); count ++)
    {
        buf[count] = (Ptr)MEM_calloc(0, BUFSIZE * RX_NUM_SERIALIZER, BUFALIGN);
        if(NULL == buf[count]) {
            SYS_abort("MEM_calloc failed.");
        }
    }

    /* Allocate buffers for the SIO buffer exchanges */
    for(count = NUM_BUFS; count < (NUM_BUFS * 2); count ++)
    {
        buf[count] = (Ptr)MEM_calloc(0, BUFSIZE * TX_NUM_SERIALIZER, BUFALIGN);
        if(NULL == buf[count]) {
            SYS_abort("MEM_calloc failed.");
        }
    }

    for(count = 0; count < NUM_BUFS; count ++)
    {
        /* Issue the first & second empty buffers to the input stream */
        if (SIO_issue(inStream, buf[count], BUFSIZE * RX_NUM_SERIALIZER, NULL) != SYS_OK) {
            SYS_abort("Error issuing buffer to the input stream");
        }
    }

    for(count = NUM_BUFS; count < (NUM_BUFS * 2); count ++)
    {
        if (SIO_issue(outStream, buf[count], BUFSIZE * TX_NUM_SERIALIZER, NULL) != SYS_OK) {
            SYS_abort("Error issuing buffer to the output stream");
        }
    }
}

/*
 * ======== echo ========
 * This function copies from the input SIO to the output SIO. You could
 * easily replace the copy function with a signal processing algorithm.
 */
Void Audio_echo_Task()
{
    volatile int i,j,n;
    Int nmadus = 0;         /* number of minimal addressable units */
    Int count  = 0;
    MdUns *rcv, *xmt;
#ifdef FOUR_TO_ONE_TEST
    Uns channelId    = 0;
    Uns serializerId = 0;
#endif /* FOUR_TO_ONE_TEST */

    edma3init();

    /* Call createStream function to create I/O streams */
    createStreams();

    /* Call prime function to do priming */
    prime();

    /* Loop forever looping back buffers */
    for (j=0;j>=0;j++) {

	    nmadus = SIO_reclaim(inStream,(Ptr *)&rcv, NULL);
        /* Reclaim full buffer from the input stream */
        if (nmadus < 0) {
            SYS_abort("Error reclaiming full buffer from the input stream");
        }

        /* Reclaim empty buffer from the output stream to be reused */
        if (SIO_reclaim(outStream, (Ptr *)&xmt, NULL) < 0) {
            SYS_abort("Error reclaiming empty buffer from the output stream");
        }

#ifdef ONE_TO_ONE_TEST
        /** nmadus value updated from SIO_reclaim is always positive */
        count = nmadus / sizeof(short);
        for (i = 0 ; i < count; i++)
        {
            xmt[i] = rcv[i];
        }
#endif /* ONE_TO_ONE_TEST */
#ifdef FOUR_TO_ONE_TEST
        /*
         Recorded data will be interleaved for serializers.
         -----------------------------------------------------
         |ser-0(0)|...|ser-0(n-1)|ser-1(0)|...|ser-1(n-1)|...
         -----------------------------------------------------
         Here each sample is assumed to be of 16-bit. Each
         serializer will record left-right data.
         */

        if (NEXT_SER_PLAY_SIZE/10 == j)
        {
            channelId ++;

            channelId = (channelId%8);
            serializerId = channelId /2;


            j = 0;
        }

        /** nmadus value updated from SIO_reclaim is always positive */
        count = nmadus / (sizeof(short)*2*RX_NUM_SERIALIZER);
        for (i = 0; i < count;i++)
        {
            switch(channelId)
            {
                case 0:
                case 2:
                case 4:
                case 6:
                    /* Serializer - 0, left channel data */
                    xmt[2*i] = rcv[(serializerId * BUFLEN)+2*i];
                    xmt[2*i+1] = 0;
                    break;
                case 1:
                case 3:
                case 5:
                case 7:
                    /* Serializer - 0, right channel data */
                    xmt[2*i] = rcv[(serializerId * BUFLEN)+2*i+1];
                    xmt[2*i+1] = 0;
                    break;

                default:
                    /* Serializer - 0, left channel data */
                    xmt[2*i] =rcv[(serializerId * BUFLEN)+2*i];
                    xmt[2*i+1] = 0;
                    break;
            }
        }
#endif /* FOUR_TO_ONE_TEST */


        /* Issue full buffer to the output stream */
        if (SIO_issue(outStream, xmt, nmadus/RX_NUM_SERIALIZER, NULL) != SYS_OK) {
            SYS_abort("Error issuing full buffer to the output stream");
        }

        /* Issue an empty buffer to the input stream */
        if (SIO_issue(inStream, rcv, BUFSIZE * RX_NUM_SERIALIZER, NULL) != SYS_OK) {
            SYS_abort("Error issuing empty buffer to the input stream");
        }
    }
}


/*
 *  ======== prd10secs ========
 *  prd10secs is configured to be called every 10 seconds
 */
Void prd10secs()
{
    static Int seconds = 0;
    static Int minutes = 0;
    static Int hours = 0;

    seconds += 10;

    if (seconds == 60) {
        seconds = 0;
        minutes++;
        if (minutes == 60) {
            minutes = 0;
            hours++;
        }
        LOG_printf(&trace, "%d hours and %d minutes", hours, minutes);
    }
    else {
        LOG_printf(&trace, "%d seconds", seconds);
    }
}

