/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/** \file   psp_bios_i2c_sample_.c
 *
 *   \brief  This file contains sample code to demonstrate the
 *          I2C driver implemented for DSP/BIOS
 *
 *   This file performs tests on the following:
 *
 *      1. Read/Write test on MSP
 *
 * (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author     PSG
 *  \version    0.1
 */

/*
 *  HEADER FILE INCLUSION
 */
#include <std.h>
#include <gio.h>
#include <log.h>
#include <tsk.h>
#include <stdio.h>

#include <ti/sdo/pspdrivers/drivers/i2c/psp_i2c.h>

#ifdef CHIP_DM648
#include <ti/sdo/pspdrivers/pal_sys/dm648/pal_sys_soc.h>
#endif

#ifdef CHIP_C6452
#include <ti/sdo/pspdrivers/pal_sys/c6452/pal_sys_soc.h>
#endif

/*
 * This application demonstrates the usage of GIO based I2C driver
 * of DM648.This application opens the driver instance, demonstrates
 * usage of I2C driver by writing proper values to the LED to blink the LEDs
 * and closes the driver instance.
 * In Asynchronous mode of operation (with the application callback supplied)
 * the application functionality does not change. However, the application gets
 * a return value of PSP_SINPROGRESS from GIO _submit. The actual completion for
 * the request submitted, occurs via the call-back function (BlinkLedCalback)
 * registered with the driver.In this sample application nothing is done in the
 * callback though.
 */

/*
 *    MACROS DEFINED
 */

#define  I2C_LED_SLAVE_ADDR      (0x70u)
#define  I2C_LED_OFFSET          (0x07u)
#define  I2C_TRANSFER_SIZE       (2u)

/*
 * I2C Driver handle
 */
static GIO_Handle      i2cHandle = NULL;

/* Function prototype */
static void blinkLED();
static void Blinkdelay();
#ifdef PSP_I2C_ASYNC_MODE_SUPPORT
static void BlinkLedCalback(Ptr arg,Int status, Ptr bufp, size_t size);
#endif
/**
 *  ======== I2C_INIT ========
 *  This function is used to Initialize the I2C Bus Driver
 *
 */
void I2C_INIT()
{
    return;
}

/*! \fn start_i2c_sample
 * \brief Function to illustrate usage of I2C driver.
 *
 *  This function will illustrate the usage of I2C driver by blinking LEDs.
 *
 *  \param None
 */
void start_i2c_sample()
{
    GIO_Attrs gioAttrs = GIO_ATTRS;

    printf("\r\nI2C  :Start of I2C sample application\r\n");

    /*
     * Create I2C Channel
     */
    i2cHandle = GIO_create("/I2C0",IOM_INOUT,NULL,NULL,&gioAttrs);
    if(NULL == i2cHandle)
    {
       printf("I2C : I2C Create Failed.......! \r\n");
    }
    else
    {
        /*
         * Call function to blink LEDs
         */
         blinkLED();
    }

    /*
     * Delete the I2C Handle
     */
    GIO_delete(i2cHandle);
    i2cHandle   =   NULL;
    printf("I2C  :End of I2C sample application\r\n");
}

/*! \fn blinkLED
 *  \brief Function will blink LEDs on EVMDM648.
 *
 *   This function will blink the LEDs on EVMDM648 for some time.
 *
 *  \param None
 */
static void blinkLED()
{
   PSP_i2cDataParam     buf;
   size_t             size;
   Uint32 count    =  0;
   Int    retCode  =  0;
   Uint8 buffer[I2C_TRANSFER_SIZE];


#ifdef PSP_I2C_ASYNC_MODE_SUPPORT
   GIO_AppCallback BlinkledCbk;
#endif

   /* Delay for other I2C Transfers to complete during INIT */
   TSK_sleep(2000u);

   printf("I2C : Start of LED Blink demo \r\n");
   printf("I2C : LED blinks for 5 seconds \r\n");


   for(count =0; count <20; count++)
   {
       /*
        * Value to be written to toggle the LEDs
        */
       buffer[0] = I2C_LED_OFFSET;
       buffer[1u] = 0x50u;
       buf.i2cTrans.buffer    = &buffer[0];
       buf.i2cTrans.bufLen    = I2C_TRANSFER_SIZE;
       buf.i2cTrans.flags     = PSP_I2C_DEFAULT_WRITE | PSP_I2C_RESTART;
       buf.i2cTrans.param     = NULL;
       buf.i2cTrans.slaveAddr = I2C_LED_SLAVE_ADDR;
       buf.timeout            = -1;
       size                   = (size_t)buf.i2cTrans.bufLen;

#ifdef PSP_I2C_ASYNC_MODE_SUPPORT

        /* Call the GIO_SUBMIT with the call back ..i.e async mode */
        BlinkledCbk.fxn = BlinkLedCalback;
        BlinkledCbk.arg = NULL;
        retCode = GIO_submit(i2cHandle,IOM_WRITE,&buf,&size,&BlinkledCbk);
#else

        /* Call the GIO_SUBMIT with NULL in the call back argument .
            .i.e sync mode */
        retCode = GIO_submit(i2cHandle,IOM_WRITE,&buf,&size,NULL);
#endif
        if(retCode < 0)
        {
            printf("LED Operation Failed\n");
        }

        Blinkdelay();

        buffer[0] = I2C_LED_OFFSET;
        buffer[1u] = 0xA0u;
        buf.i2cTrans.buffer    = &buffer[0];
        buf.i2cTrans.bufLen    = I2C_TRANSFER_SIZE;
        buf.i2cTrans.flags     = PSP_I2C_DEFAULT_WRITE | PSP_I2C_RESTART;
        buf.i2cTrans.param     = NULL;
        buf.i2cTrans.slaveAddr = I2C_LED_SLAVE_ADDR;
        buf.timeout            = -1;
        size                   = (size_t)buf.i2cTrans.bufLen;

#ifdef PSP_I2C_ASYNC_MODE_SUPPORT

        /* Call the GIO_SUBMIT with the call back ..i.e async mode */
        BlinkledCbk.fxn = BlinkLedCalback;
        BlinkledCbk.arg = NULL;
        retCode = GIO_submit(i2cHandle,IOM_WRITE,&buf,&size,&BlinkledCbk);
#else

        /* Call the GIO_SUBMIT with NULL in the call back argument .
            .i.e sync mode */
        retCode = GIO_submit(i2cHandle,IOM_WRITE,&buf,&size,NULL);
#endif
        if(retCode < 0)
        {
            printf("LED Operation Failed\n");
        }

        Blinkdelay();
    }

    printf("I2C : End of LED Blink demo \r\n");
}


/*! \fn BlinkLedCalback
 *  \brief Call Back function Used in async mode.
 *
 *
 *  \param None
 */

void BlinkLedCalback(Ptr arg,Int status, Ptr bufp, size_t size)
{
    /* To avoid compiler warnings*/
    arg = arg;
    bufp = bufp;
    size = size;
    status= status;

    return;

}

/*! \fn Blinkdelay
 *  \brief Function used for delay.
 *
 *
 *  \param None
 */


static void Blinkdelay()
{
  volatile Uint32 i;

  for(i=0; i<0x2FFFFF ;i++)
    ;
  for(i=0; i<0x2FFFFF ;i++)
    ;
}

