/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/
/**
 *  \file   psp_pci_bios_sample_main.c
 *
 *  \brief  Test application to test PCI driver for DM648/C6452 EVM.
 *
 *  This file contains test application for PCI driver.
 *  Steps to run PCI sample application:
 *  1. Fit EVMDM648/EVM6452 card into the PCI slot of a linux machine having linux
 *     kernel 2.6.9(RHEL 4 AS).
 *  2. Connect USB emulator to JTAG port of DM648/C6452 EVM. Connect the USB cable
 *        to other PC having CCS on that.
 *    3. Power on Linux machine. BIOS of linux machine should not boot.
 *    4. Open CCS 3.3 on the PC and try to connect to DM648/C6452 EVM. When the board
 *       will be connected to CCS, BIOS of linux machine will boot up.
 *    5. Press Ctrl+Alt+F1 to go to console in linux machine
 *    6. Enter username and password for "root" user
 *    7. Copy Makefile and pcidrv.c from "pspdrivers\pal_sys\pci\docs\linuxapp"
 *       folder to "/home" directory of linux machine.
 *    8. Go to "/home" directory by giving "cd /home" command
 *    9. Give "make" command. This will generate the object file of a PCI host
 *       driver for linux. Driver will be generated by module name "pcidriver.ko"
 *       in same directory.
 *    10.Give "insmod pcidriver.ko" command. This will insert the driver as a
 *       module
 *    11.This will give some print messages on linux machine. Give "dmesg" command
 *       on linux console to see the print messages of linux driver for PCI.
 *    12.There will be a print message saying "Address to write from DSP = 0x518000".
 *       Note this address(0x518000).
 *    13.Open \pspdrivers\system\c6452\bios\evm6452\pci\build\psp_pci_bios_sample_c6452.pjt for
 *       C6452 project and \pspdrivers\system\dm648\bios\evmdm648\pci\build\psp_pci_bios_sample_dm648.pjt
 *       for DM648 in CCS. Open \pspdrivers\pal_sys\pci\sample\psp_pci_bios_sample_main.c
 *    14.Build CCS project and run the out file.
 *    15.Out file will generate a host interrupt on linux machine. On generation of
 *       host interrupt there will be print messages saying "Interrupt received from
 *       DSP". This indicates that DSP to Host interrupt is generated.
 *    16.Sample application loaded on EVM DM648/C6452 through CCS will also perform read/
 *       write test. A pop up will come to fetch the address for read write. Enter
 *       the address for read write printed on the consol of Linux host machine
 *       noted in step 12.The sample application will perform read write and print
 *       the message of success or failure on STDOUT
 *
 *  \version    0.1     Rinkal Shah   Created
 */


/*
 * Include files
 */
#include <std.h>
#include <stdio.h>
#include <mem.h>
#include <tsk.h>
#include <log.h>
#include <gio.h>
#include <string.h>
#include <bcache.h>

#ifdef CHIP_DM648
#include <ti/sdo/pspdrivers/soc/dm648/dsp/soc.h>
#include <ti/sdo/pspdrivers/soc/dm648/dsp/cslr_pci.h>
#include <ti/sdo/pspdrivers/pal_sys/dm648/pal_sys.h>
#include <ti/sdo/pspdrivers/pal_sys/dm648/pal_sys_soc.h>
#endif

#ifdef CHIP_C6452
#include <ti/sdo/pspdrivers/soc/c6452/dsp/soc.h>
#include <ti/sdo/pspdrivers/soc/c6452/dsp/cslr_pci.h>
#include <ti/sdo/pspdrivers/pal_sys/c6452/pal_sys.h>
#include <ti/sdo/pspdrivers/pal_sys/c6452/pal_sys_soc.h>
#endif

#include <ti/sdo/pspdrivers/pal_sys/pci/pal_syspci.h>

#define PCI_DEBUG   printf
#define PCI_CONTROLLER_INSTANCE        (0u)
#define TRIGGER_SOFT2INT_MASK          (0x04000000u)
#define ENABLE_SOFTINT3_MASK           (0x08000000u)
/*
 * This test application demonstrates usage of PCI driver for DM648/C6452. The test
 * application demonstrates master mode read and write operation. Application
 * will write data to the host space of linux machine and read the data back.
 */
#define SIZE_OF_MEMORY_SPACE_OF_NW_CARD     (256)

#define TEST_WRITE_VALUE                    (127)

extern LOG_Obj trace;

CSL_PciRegsOvly        pciRegs  = (CSL_PciRegsOvly)CSL_PCI_0_REGS;

Ptr initPCI();
void deInitPCI(Ptr hPci);
Int32 readData(Ptr hPci, const Uint8* srcAddr, Uint8* dstAddr, Uint32 readCount);
Int32 writeData(Ptr hPci,Uint8* srcAddr, Uint8* dstAddr, Uint32 writeCount);
void generateSoft2HostInterrupt();


/*
 * Global variables
 */
Ptr         pciHandle;
extern LOG_Obj trace;
extern Int32  extHeap;
extern Int32  intHeap;
Uint8 interruptCalled = 0;
void tskDataExchange();

Uint8 pciReadBuff[SIZE_OF_MEMORY_SPACE_OF_NW_CARD];
Uint8 pciWriteBuff[SIZE_OF_MEMORY_SPACE_OF_NW_CARD];

Ptr pciBusAddressOfNewCard;

void main()
{
    if(PAL_sysModuleClkCtrl(PAL_SYS_CLK_MODULE_PCI,PAL_SYS_MODULE_CLK_ENABLE) != PAL_SOK)
    {
        PCI_DEBUG("ERROR: Unable to power up PCI module\n");
        return;
    }
    /*
     * Initialize the PCI driver and hardware
     */
    pciHandle = initPCI();
}

void tskDataExchange()
{

    Uint32  noOfByteXfered = SIZE_OF_MEMORY_SPACE_OF_NW_CARD;
    Int status = 0;
    Uint32 initCount = 0;
    Uint32 checkLoopCount   =   0;
    Uint32 counterDelay = 10000;

    /*
     * Enable software3 interrupts of PCI driver
     */
    status = PAL_sysPCIEnableInterrupt(pciHandle, PAL_SYS_PCI_DSP_INT_SOFTINT3);

    if (PAL_SOK != status)
    {
        PCI_DEBUG("ERROR: PCI driver unable to set Interrupts\n");
    }

    /*
     * If PCI driver handle is NULL, don't move ahead
     */
    if(NULL == pciHandle)
    {
        PCI_DEBUG("ERROR: PCI driver handle not created\n");
        return;
    }
    memset(pciReadBuff,     0x0, SIZE_OF_MEMORY_SPACE_OF_NW_CARD);
    memset(pciWriteBuff,    0x0, SIZE_OF_MEMORY_SPACE_OF_NW_CARD);

    printf("Enter the pci address of memory space for read write in hex (the address printed out on the linux host machine)\n");
    scanf("%x", &pciBusAddressOfNewCard);

    /*
     * If the transfer count is non-zero that go ahead
     */
    if(0 != noOfByteXfered)
    {
        /*
         * Initialize the write buffer by new values
         */
        for(initCount = 0; initCount < SIZE_OF_MEMORY_SPACE_OF_NW_CARD; initCount++)
        {
            pciWriteBuff[initCount] = initCount;
        }
        /*
         * Write back data to DDR from Cache
         */
        BCACHE_wb(pciWriteBuff, SIZE_OF_MEMORY_SPACE_OF_NW_CARD, TRUE);
        /*
         * Write data onto network card
         */
        status = writeData( pciHandle,
                            pciWriteBuff,
                            pciBusAddressOfNewCard,
                            SIZE_OF_MEMORY_SPACE_OF_NW_CARD);

        if(status < 0)
        {
            PCI_DEBUG( "ERROR: GIO_write unsuccessful status == %d; Size == %u\n",status, noOfByteXfered);
            PCI_DEBUG( "ERROR: PCI test failed in write.\n");
        }

        generateSoft2HostInterrupt();
        PCI_DEBUG("DSP to Host Interrupt generated\n");
        /* Written data to host to be compared at host side */

        /* Data to host is written. Now it's the turn of host to give us some data.
         * So, wait for it, until it is notified via an interrupt generated by the
         * host
         */
        while (0 ==  interruptCalled)
        {
            counterDelay--;
            if (0 == counterDelay)
            {
                PCI_DEBUG( "ERROR: PCI Host interrupt not received.");
                return;
            }
        }

        PCI_DEBUG("Host to DSP interrupt test successfully\n");
        status = readData(pciHandle,
                          pciBusAddressOfNewCard,
                          pciReadBuff,
                          SIZE_OF_MEMORY_SPACE_OF_NW_CARD);
        if(status < 0)
        {
            PCI_DEBUG( "ERROR: GIO_read unsuccessful status == %d; Size == %u\n",status, noOfByteXfered);
            PCI_DEBUG( "ERROR: PCI test failed in read.");
        }

        /* This pattern of data is been written from Host side */
        for (checkLoopCount = 0; checkLoopCount < SIZE_OF_MEMORY_SPACE_OF_NW_CARD; checkLoopCount++)
        {
           pciWriteBuff[checkLoopCount] = TEST_WRITE_VALUE - checkLoopCount;
        }

        for(checkLoopCount = 0; checkLoopCount < SIZE_OF_MEMORY_SPACE_OF_NW_CARD; checkLoopCount++)
        {
            /*
             * Check whether read buffer and above mentioned pattern buffer are same or not
             */
            if(pciReadBuff[checkLoopCount] != pciWriteBuff[checkLoopCount])
            {
                PCI_DEBUG( "ERROR: PCI data comparision test failed.");
                return;
            }
        }

        PCI_DEBUG("PCI data comparision test passed\n");
    }

    /*
     * De-initialize the PCI driver and hardware
     */
    deInitPCI(pciHandle);
    PCI_DEBUG("PCI Test completed successfully\n");
}

void pciCallback(Uint32 interruptMask,Ptr appData)
{
    if(NULL     !=  appData)
    {
        appData =   appData;
    }

    if (PAL_SYS_PCI_DSP_INT_SOFTINT3 ==
               (interruptMask & PAL_SYS_PCI_DSP_INT_SOFTINT3))
    {
        interruptCalled = 1;
    }
}
/**
 *  \brief Initialize the PCI driver and hardware
 *
 *  This function does the initialization of PCI controller and PCI
 *  driver. This function is deviced for DM648/C6452 PCI driver. This function
 *  will change if the PCI controller or PCI device changes.
 *
 *  \return hPci Handle to PCI driver
 */
Ptr initPCI()
{
    PAL_Result status = PAL_SOK;
    Ptr hPci = NULL;
    PAL_sysPciInitConfig    pciParams;

    /* Create PCI driver instance */
    status =    PAL_sysPCICreate(PCI_CONTROLLER_INSTANCE, NULL);
    if(PAL_SOK  !=  status)
    {
        PCI_DEBUG("ERROR: Creation of PCI driver instance failed\r\n");
        return NULL;
    }
    /* Pass application callback as NULL and application data as NULL */
    pciParams.appCb     =   &pciCallback;
    pciParams.appData   =   NULL;
    /* Open driver instance */
    hPci    =   (Ptr)PAL_sysPCIOpen(PCI_CONTROLLER_INSTANCE, &pciParams);
    if(NULL ==  hPci)
    {
        PCI_DEBUG("ERROR: Opening PCI driver failed\r\n");
        return NULL;
    }
    return hPci;
}
/**
 *  \brief De-initialize PCI driver
 *
 *  This function de-initializes the PCI driver. This function is deviced for DM648/C6452
 *  PCI driver. This function will change if the PCI controller or PCI device changes.
 *
 *  \param  hPci [IN]       Handle to PCI driver
 *  \return None
 */
void deInitPCI(Ptr hPci)
{
    PAL_sysPCIClose((PSP_Handle)hPci, NULL);
    PAL_sysPCIDelete(PCI_CONTROLLER_INSTANCE);
}
/**
*  \brief Read data from PCI devices attached to the PCI bus
 *
 *  This function will read data from PCI devices attached to the PCI bus.
 *  Source address, destination address, number of bytes to be read and transfer
 *  mode have to be passed to this API. This function is deviced for DM648/C6452
 *  PCI driver. This function will change if the PCI controller or PCI device changes.
 *
 *  \param  hPci      [IN]      Handle to PCI driver
 *  \param  srcAddr   [IN]      Source adddress
 *  \param  dstAddr   [IN]      Destination adddress
 *  \param  readCount [IN]      Number of bytes to read
 *  \param  xfrMode   [IN]      Options for read operation

 *  \return If successful, 0
 *          If failed, returns negative number - IOM Error code
 */
Int32 readData(Ptr hPci, const Uint8* srcAddr, Uint8* dstAddr, Uint32 readCount)
{
    Uint32  masterConfig    =   0;
    PAL_Result status = PAL_SOK;

    Uint32 tmpSrcAddr = 0;
    Int32 i = 0;

    /* Get the master control */
    status  =    PAL_sysPCIGetMemMapReg(hPci,
                                        PAL_SYSPCI_MIRROR_CMD_STATUS_REG,
                                        PAL_SYS_PCI_READ_WRITE_SIZE_WORD,
                                        &masterConfig);
    if(PAL_SOK  !=  status)
    {
        PCI_DEBUG("ERROR: Unable to read the Memory mapped configuration register\r\n");
        return status;
    }
    /*
     * Enable bus master bit
     */
    masterConfig    |=  0x00000004;
    status  =    PAL_sysPCISetMemMapReg(hPci,
                                        PAL_SYSPCI_MIRROR_CMD_STATUS_REG,
                                        PAL_SYS_PCI_READ_WRITE_SIZE_WORD,
                                        masterConfig);
    if(PAL_SOK  !=  status)
    {
        PCI_DEBUG("ERROR: Unable to write the memory mapped configuration register\r\n");
        return status;
    }
    /*
     * Set the DSP translation register
     */
    status = PAL_sysPCISetMemMapReg(hPci,
                                    PAL_SYSPCI_ADDR_SUBSTITUTION_REG0,
                                    PAL_SYS_PCI_READ_WRITE_SIZE_WORD,
                                    (Uint32)srcAddr);
    if(PAL_SOK  !=  status)
    {
        PCI_DEBUG("ERROR: Unable to write the DSP to PCI translation register\r\n");
        return status;
    }

    /* Update PCI Register for Address Sub   */
    tmpSrcAddr = PAL_SYS_PCI_TO_DSP_MEM_MAP_ADDR(srcAddr );

    BCACHE_inv((Ptr)dstAddr, readCount, 1);
    BCACHE_inv((Ptr)tmpSrcAddr, readCount, 1);

    for(i = 0; i< readCount; i+=4)
    {
        *((Uint32 *)dstAddr) = *((Uint32 *)tmpSrcAddr);
        tmpSrcAddr+=4;
        dstAddr+=4;
    }

    return PAL_SOK;
}
/**
 *  \brief Write data onto PCI devices attached to the PCI bus
 *
 *  This function will write data on PCI devices attached to the PCI bus.
 *  Source address, destination address, number of bytes to be written and transfer
 *  mode have to be passed to this API. This function is deviced for DM648/C6452
 *  PCI driver. This function will change if the PCI controller or PCI device changes.
 *
 *  \param  hPci      [IN]      Handle to PCI driver
 *  \param  srcAddr   [IN]      Source adddress
 *  \param  dstAddr   [IN]      Destination adddress
 *  \param  readCount [IN]      Number of bytes to read
 *  \param  xfrMode   [IN]      Options for read operation

 *  \return If successful, 0
 *          If failed, returns negative number - IOM Error code
 */
Int32 writeData(Ptr hPci,Uint8* srcAddr, Uint8* dstAddr, Uint32 writeCount)
{
    Uint32  masterConfig    =   0;
    PAL_Result status = PAL_SOK;
    Uint32 pageBase = 0;
    Uint32 tmpDstAddr = 0;
    Int32 i = 0;

    /* Get the master control */
    status  =    PAL_sysPCIGetMemMapReg(hPci,
                                        PAL_SYSPCI_MIRROR_CMD_STATUS_REG,
                                        PAL_SYS_PCI_READ_WRITE_SIZE_WORD,
                                        &masterConfig);
    if(PAL_SOK  !=  status)
    {
        PCI_DEBUG("ERROR: Unable to read the memory mapped configuration register\r\n");
        return status;
    }
    /* Enable bus master bit */
    masterConfig    |=  0x00000004;
    status  =    PAL_sysPCISetMemMapReg(hPci,
                                        PAL_SYSPCI_MIRROR_CMD_STATUS_REG,
                                        PAL_SYS_PCI_READ_WRITE_SIZE_WORD,
                                        masterConfig);
    if(PAL_SOK  !=  status)
    {
        PCI_DEBUG("ERROR: Unable to write the memory mapped configuration register\r\n");
        return status;
    }

    /*
     * Program the pagebase address
     */
    pageBase = (Uint32) dstAddr & 0xFF800000;
    status = PAL_sysPCISetMemMapReg(    hPci,
                                        PAL_SYSPCI_ADDR_SUBSTITUTION_REG0,
                                        PAL_SYS_PCI_READ_WRITE_SIZE_WORD,
                                        pageBase);
    if(PAL_SOK  !=  status)
    {
        PCI_DEBUG("ERROR: Unable to write the DSP to PCI translation register\r\n");
        return status;
    }

    /* Update PCI Register for Address Sub */

    tmpDstAddr = PAL_SYS_PCI_TO_DSP_MEM_MAP_ADDR(dstAddr);

    for(i = 0; i< writeCount; i+=4)
    {
        *((Uint32 *)tmpDstAddr) = *((Uint32 *)srcAddr);
        tmpDstAddr+=4;
        srcAddr+=4;
    }

    BCACHE_wb ((Ptr)tmpDstAddr, writeCount, 1);

    return PAL_SOK;
}

void generateSoft2HostInterrupt()
{
    Uint32 interruptStatus = 0;
    PAL_Result    status   = 0;

    if(NULL != pciHandle)
    {
        /*
         * Read the status set register
         */
        status  =    PAL_sysPCIGetMemMapReg(pciHandle,
                                            PAL_SYSPCI_STATUS_SET,
                                            PAL_SYS_PCI_READ_WRITE_SIZE_WORD,
                                            &interruptStatus);
         if(PAL_SOK  !=  status)
         {
             PCI_DEBUG("ERROR: Unable to read the memory mapped configuration register while interrupt genration\n");
             return;
         }

        /*
         * Set SOFTWARE3 bit in memory mapped status set register
         */
        interruptStatus    =    interruptStatus | TRIGGER_SOFT2INT_MASK;
        /*
         * Write the status set register to trigger interrupt
         */
        status  =    PAL_sysPCISetMemMapReg(pciHandle,
                                            PAL_SYSPCI_STATUS_SET,
                                            PAL_SYS_PCI_READ_WRITE_SIZE_WORD,
                                            interruptStatus);
        if(PAL_SOK  !=  status)
        {
            PCI_DEBUG("ERROR: Unable to write the Backend configuration register while interrupt genration\n");
            return;
        }
    }
}
