/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 2006-2010 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**
 *  \file   psp_bios_spi_sample.c
 *
 *  \brief  Sample App code
 *
 *  (C) Copyright 2010, Texas Instruments, Inc
 *
 *  \author     PSG
 *
 *  \version    0.1     Nagarjuna Kristam   Created.
 *  \version    0.2     Nagarjuna Kristam   Added EEPROM test cases
 *
 */

#include <stdio.h>
#include <std.h>
#include <string.h>
#include <gio.h>
#include <tsk.h>
#include <log.h>

#include <ti/sdo/pspdrivers/drivers/spi/psp_spi.h>

#ifdef CHIP_DM648
#include <ti/sdo/pspdrivers/pal_sys/dm648/pal_sys.h>
#include <ti/sdo/pspdrivers/pal_sys/dm648/pal_sys_soc.h>
#endif

#ifdef CHIP_C6452
#include <ti/sdo/pspdrivers/pal_sys/c6452/pal_sys.h>
#include <ti/sdo/pspdrivers/pal_sys/c6452/pal_sys_soc.h>
#endif

/* SPI Driver IOM Function Table and trace*/
extern IOM_Fxns  SPIMD_FXNS;
extern far LOG_Obj trace;

/* Clock for SPI Module */
#define PSP_SPI_MODULE_CLK_FREQ             (PAL_SYS_CPU_CLOCK_FREQUENCY/6u)
/* SPI bus frequency */
#define SPI_BUS_FREQ                        3000000u

/* SPI transceive macro's */
#define MAX_TRANSFER_SIZE                   100
#define SPI_DATA_TEST_CHAR                  '0'

#define SPI_BYTE_WRITE                      60u
#define SPI_BYTE_READ                       10u

/* SPI HW conig macro's */
#define CS_DEFAULT_SPI                      0x000000FFu
#define CHIP_NUMBER                         1u
#define SPI_DATA_CHAR_LENGTH                8u

/** Update this define to
  * PSP_OPMODE_POLLED       for POLLED mode of operation
  * PSP_OPMODE_INTERRUPT    for INTERRUPT mode of operation
  */
#define USER_CHOOSEN_MODE_OF_OPERATION       PSP_OPMODE_INTERRUPT

/* EEPROM macro's */
#define SPI_EEPROM_WRITE_OPCODE             0x0002u
#define SPI_EEPROM_READ_OPCODE              0x0003u
#define SPI_EEPROM_MSB                      0x10u
#define SPI_EEPROM_LSB                      0x00u

static Int spi_Test();
void start_spi_test(PSP_Handle hEdma);
/* Global SPI init config data structure */
PSP_SpiConfig       SPI_devParams ;

/* Global variables */
Uint8  loopWrite[MAX_TRANSFER_SIZE + 32];
Uint8  loopRead[MAX_TRANSFER_SIZE   + 32];
GIO_Handle          spiHandle0,spiHandle1;


void spi_initDevParams(void)
{

    PSP_spiHWConfigData spiHWData ;
    Int i               = 0;

    memset(&SPI_devParams,0,sizeof(PSP_SpiConfig));

    spiHWData.clkInternal           = TRUE;
    spiHWData.delay.c2EDelay        = 0;
    spiHWData.delay.c2TDelay        = 0;
    spiHWData.delay.t2CDelay        = 0 ;
    spiHWData.delay.t2EDelay        = 0;
    spiHWData.enableHighZ           = FALSE;
    spiHWData.intrLevel             = FALSE;
    spiHWData.pinOpModes            = PSP_SPI_OPMODE_SPISCS_4PIN;
    spiHWData.masterOrSlave         = PSP_SPI_TYPE_MASTER;
    spiHWData.waitDelay             = FALSE;
    spiHWData.csDefault             = CS_DEFAULT_SPI;

    for(i = 0 ; i < 4 ; i++)
    {
        spiHWData.configDatafmt[i].charLength   = SPI_DATA_CHAR_LENGTH;
        spiHWData.configDatafmt[i].clkHigh      = TRUE ;
        spiHWData.configDatafmt[i].lsbFirst     = FALSE;
        spiHWData.configDatafmt[i].oddParity    = FALSE;
        spiHWData.configDatafmt[i].parityEnable = FALSE ;
        spiHWData.configDatafmt[i].phaseIn      = FALSE ;
        spiHWData.configDatafmt[i].waitEnable   = FALSE;
        spiHWData.configDatafmt[i].wDelay       = 0;
    }

    spiHWData.configDatafmt[1].charLength   = 16u;

    SPI_devParams.mode               = USER_CHOOSEN_MODE_OF_OPERATION;
    SPI_devParams.moduleInputClkFreq = PSP_SPI_MODULE_CLK_FREQ;
    SPI_devParams.outputClkFreq      = SPI_BUS_FREQ;
    SPI_devParams.loopBack          = FALSE;
    SPI_devParams.edmaHandle        = NULL;
    SPI_devParams.spiHWCfgData      = spiHWData;
}



/* ARGSUSED */
void start_spi_sample(PSP_Handle hEdma)
{

    Int                 devStatus;
    GIO_Attrs           gioAttrs            = GIO_ATTRS;

    /* create SPI channel for transmission */
    spiHandle0 =  GIO_create("/SPI0",IOM_INOUT,NULL,NULL,&gioAttrs);

    if (NULL != spiHandle0)
    {
        devStatus = spi_Test();
        if(IOM_COMPLETED != devStatus)
        {
            printf("\r\nBIOS SPI:Error in SPI sample Test \n");
        }
        else
        {
            printf("\r\nBIOS SPI:SPI sample transceive ended succesfully \n");
        }

        GIO_delete(spiHandle0);
    }
    else
    {
        printf("\r\n SPI Driver Handle creation Failed ");
    }

    return;
}

#define SPI_CHIPSELECT_EERPOM   1u

/**< SPI DATA transfer test for read/write */
static Int spi_Test(void)
{

    Int                 i           = 0;
    Int                 status      = IOM_COMPLETED;
    PSP_spiDataParam    dataparam;
    size_t              size        = 1u;


    /* update transceive buffers */
    for(i = 0;i<100;i++)
    {
        loopWrite[i] = SPI_DATA_TEST_CHAR + i;
        loopRead[i] = 0;
    }

    loopWrite[0]    = 0x06u;/** OPCODE for enabling write in EEPROM */

    dataparam.spiTrans.inBuffer     = NULL;
    dataparam.spiTrans.outBuffer    = (Uint8*)&loopWrite[0];
    dataparam.spiTrans.chipSelect  = SPI_CHIPSELECT_EERPOM;
    dataparam.spiTrans.dataFormat  = PSP_SPI_FMTSEL_0;
    dataparam.spiTrans.flags        = 0;
    dataparam.timeout               = 1000;
    dataparam.spiTrans.bufLen       = 1u;

    /* SPI Data Write OPCODE Transfer */
    status = GIO_write (spiHandle0,&dataparam,&size);

    loopWrite[0] = 0x05u;   /* OPCODE to read status register  */
    dataparam.spiTrans.inBuffer     = loopRead;
    dataparam.spiTrans.outBuffer   =  loopWrite;
    dataparam.spiTrans.bufLen       = 1u;
    dataparam.spiTrans.dataFormat  = PSP_SPI_FMTSEL_0;
    /* Opcode to rea status register */
    status = GIO_write (spiHandle0,&dataparam,&size);
    TSK_sleep(5u);

    loopWrite[0] = SPI_EEPROM_WRITE_OPCODE;
    loopWrite[1] = SPI_EEPROM_MSB;/* MSB of SPI_EEPROM_START_ADDR */
    loopWrite[2] = SPI_EEPROM_LSB;
    dataparam.spiTrans.bufLen       = SPI_BYTE_WRITE+3;
    dataparam.spiTrans.outBuffer    = &loopWrite[0u];
    dataparam.spiTrans.flags       = PSP_SPI_CSHOLD;
    dataparam.spiTrans.dataFormat  = PSP_SPI_FMTSEL_0;
    /* write data to EEPROM */
    status = GIO_write (spiHandle0,&dataparam,&size);

    /** This delay is given as EEPROM takes 5Ms to complete its write cycle
      * which is mentioned in EEPROM data sheets available at
      * http://www.atmel.com/dyn/resources/prod_documents/doc3368.pdf
      * Also depending on the EEPROM data sheet one can modify this sleep time */
    TSK_sleep(5u);

    if(status == IOM_COMPLETED)
    {
        /* Read Transfer */
        loopWrite[0] = SPI_EEPROM_READ_OPCODE;  /*  Opcode for read  */
        loopWrite[1] = SPI_EEPROM_MSB;
        loopWrite[2] = SPI_EEPROM_LSB;  /*  Starting adress to read data */
        dataparam.spiTrans.flags       = PSP_SPI_CSHOLD;
        dataparam.spiTrans.dataFormat  = PSP_SPI_FMTSEL_0;
        dataparam.spiTrans.bufLen       = SPI_BYTE_WRITE+3;
        dataparam.spiTrans.outBuffer   = &loopWrite[0];
        dataparam.spiTrans.inBuffer    = &loopRead[0];

        /* read from EEPROM */
        status = GIO_write (spiHandle0,&dataparam,&size);


        for(i=3;i<(SPI_BYTE_WRITE+3);i++)
        {
            if (loopWrite[i] != loopRead[i])
            {
                break;
            }
        }
        if (i == SPI_BYTE_WRITE+3u)
        {
            printf("\r\n Data comparison success");
        }
        else
        {
            printf("\r\n Data comparision failed");
        }
    }
    else
    {
        printf("\r\nBIOS SPI:Write Test failed in interrupt mode \n");
    }

    return status;
}

