/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/** \file   psp_bios_uart_sample_read_write.c
 *
 *   \brief  This file contains sample code to demonstrate the
 *          UART driver implemented for DSP/BIOS - DM648/C6452
 *
 *  This file contains function that demonstrate 1000 bytes read/write
 *  of UART of DSP/BIOS - DM648.
 *
 * (C) Copyright 2006, Texas Instruments, Inc
 *
 *  \author     PSG
 *  \version    0.1
 */
#include <stdio.h>
#include <std.h>
#include <string.h>
#include <tsk.h>
#include <gio.h>
#include <iom.h>
#include <log.h>

#include <ti/sdo/pspdrivers/drivers/uart/psp_uart.h>

#ifdef CHIP_DM648
#include <ti/sdo/pspdrivers/pal_sys/dm648/pal_sys.h>
#endif

#ifdef CHIP_C6452
#include <ti/sdo/pspdrivers/pal_sys/c6452/pal_sys.h>
#endif

#ifdef PSP_UART_EDMA_SUPPORT
#include <ti/sdo/edma3/drv/edma3_drv.h>
#endif

/*
 * This test application demonstrates the usage of GIO based UART driver of
 * DM648/C6452. This test application creates Input and output UART driver handle
 * for default settings of baud rate, stop bit, character length, parity and
 * fifo trigger level. The application does a 1000 byte read operation followed
 * by 1000 bytes write operation.
 */
#define UART_DEBUG    printf

#define SIZEOF_UART_STRING_START (128u)
#define SIZEOF_UART_STRING_STOP (32u)
#define SIZEOF_UART_STRING (32u)

#ifdef PSP_UART_EDMA_SUPPORT
extern EDMA3_DRV_Handle hEdma;
extern EDMA3_DRV_Result edma3init();
#endif
/*
 * External References
 */

/* UART handle for input channel */
extern GIO_Handle hUart_IN;

/* UART handle for output channel */
extern GIO_Handle hUart_OUT;

extern LOG_Obj    trace;

#ifdef PSP_UART_ASYNC_MODE_SUPPORT
/* GIO application call back for TX and RX */
GIO_AppCallback     txAppCb;
GIO_AppCallback     rxAppCb;

/* local function prototypes */
static void uart_TxAppCallback(Ptr arg,Int status, Ptr bufp, size_t size);
static void uart_RxAppCallback(Ptr arg,Int status, Ptr bufp, size_t size);
#endif

/*
 * Global References
 */

/* UART0 device params. To be filled in uart0_dev_init function which
  is called before driver creation */
PSP_uartDevParams   uart0DevParams;

/* Global function prototypes */
void uart0_dev_init(void);

/*
 * Local References
 */

/* Local function prototypes */
static void genericUartTest();

#if defined(CHIP_DM648)||defined(CHIP_C6452)||defined(CHIP_DM6437)||defined(CHIP_C6424)
/*
 * Starting message printing string
 * Note: Buffer alignement is required only when working in DMA Mode.
 */
#pragma DATA_ALIGN(uartTestStringStart, 128);
Int8 uartTestStringStart[SIZEOF_UART_STRING_START];
/* Tx memory buffer */
#pragma DATA_ALIGN(Uart_TxBuffer, 128);
Int8  Uart_TxBuffer[1024];
#endif


/*
 * UART0 init function called when creating the driver.
 */
void uart0_dev_init()
{
#ifdef PSP_UART_EDMA_SUPPORT
    EDMA3_DRV_Result    edmaResult      = 0;
#endif

    /* Power On UART */
    if(PAL_sysModuleClkCtrl(PAL_SYS_CLK_MODULE_UART,PAL_SYS_MODULE_CLK_ENABLE)
       != PAL_SOK)
    {
        return;
    }

    /* UART driver - Initializing device params */
    uart0DevParams.inputFreq       = PSP_UART_MODULE_CLOCK;
    uart0DevParams.fifoEnable      = TRUE;
    uart0DevParams.opMode          = PSP_OPMODE_INTERRUPT;

#ifdef PSP_UART_EDMA_SUPPORT
    uart0DevParams.opMode          = PSP_OPMODE_DMAINTERRUPT;
#endif
    uart0DevParams.loopbackEnabled = FALSE;

    uart0DevParams.baudRate    = PSP_UART_BAUD_RATE_115_2K;
    uart0DevParams.stopBits    = PSP_UART_NUM_STOP_BITS_1;
    uart0DevParams.charLen     = PSP_UART_CHARLEN_8;
    uart0DevParams.parity      = PSP_UART_PARITY_NONE;
    uart0DevParams.rxThreshold = PSP_UART_RX_TRIG14;
    uart0DevParams.fc.fcType   = PSP_UART_FC_TYPE_NONE;
    uart0DevParams.fc.fcParam  = PSP_UART_FC_NONE;


#ifdef PSP_UART_EDMA_SUPPORT
    if (NULL == hEdma)
    {
        edmaResult = edma3init();
        if (edmaResult != EDMA3_DRV_SOK)
        {
            /* Report EDMA Error */
            UART_DEBUG("\r\nEDMA3 : edma3init() failed\r\n");
        }
        else
        {
            UART_DEBUG("\r\nEDMA3 : edma3init() passed\r\n");
        }
    }
#endif

}


/*! \fn start_uart_test
 *   \brief Function will invoke all the UART tests.
 *
 *     This function will invoke all the tests.
 *
 *  \param uartInst      Uart Instance ID.
 */
void start_uart_sample()
{
    UART_DEBUG(" Starting UART sample application \n\r");

    genericUartTest();

    UART_DEBUG("UART sample application completed \n\r ");
}

#ifdef PSP_UART_ASYNC_MODE_SUPPORT
/* Callback counter to count number of GIO_submit callbacks */
volatile Int32 gGioSubmitCnt = 0;
/* Global callback counter: To count number of times appCall back is called */
volatile Int32 gCbCntr = 0;

/*
 * Call back notification of IOP completion(Called from ISR)
 */
/* ARGSUSED */
static void uart_TxAppCallback(Ptr arg,Int status, Ptr bufp, size_t size)
{
    gCbCntr++;
}

/* ARGSUSED */
static void uart_RxAppCallback(Ptr arg,Int status, Ptr bufp, size_t size)
{
    gCbCntr++;
}
#endif

/*! \fn void genericUartTest()
 * \brief Generic uart test.
 *
 *   This Function is called to test generic UART test. In this function basic
 *   read/write functionality
 *  is tested.
 *
 */
static void genericUartTest()
{
    PSP_uartDataParam buf;

    Int status   =  0;
    size_t len   =  0;
    Int8   *str  =  NULL;

#ifdef PSP_UART_ASYNC_MODE_SUPPORT
    /* Application Callback */
    rxAppCb.fxn = &uart_RxAppCallback;
    rxAppCb.arg = hUart_IN;

    txAppCb.fxn = &uart_TxAppCallback;
    txAppCb.arg = hUart_OUT;


    gCbCntr       = 0;
    gGioSubmitCnt = 0;
#endif

    str = "UART Demo Starts:  INPUT a file of size 1000 bytes";
    /*
     * Copy to start string to Cache aligned buffer
     */
    strcpy(uartTestStringStart,str);

    /*
     * Input buffer
     */
    buf.addr = uartTestStringStart;
    /*
     * Time out value
     */
    buf.timeout = SYS_FOREVER;
    /*
     * Length of data
     */

    len = strlen(uartTestStringStart);

#ifndef PSP_UART_ASYNC_MODE_SUPPORT
    status = GIO_submit(hUart_OUT,IOM_WRITE, &buf, &len, NULL);
#else
    status = GIO_submit(hUart_OUT,IOM_WRITE,&buf,&len,&txAppCb);
    gGioSubmitCnt++;

    if(uart0DevParams.opMode == PSP_OPMODE_POLLED)
    /* In polled mode you will not get callback */
    {
        gCbCntr++;
    }

#endif

    /*
     * Error status detection
     */
    if(!((status == IOM_COMPLETED)||(status == IOM_PENDING))){
        UART_DEBUG("\r\n Error from GIO_write for UART Test string\n");
    }


    /*
     * Input buffer
     */
    buf.addr = Uart_TxBuffer;
    /*
     * Time out value
     */
    buf.timeout = SYS_FOREVER;
    /*
     * Length of data
     */
    len = 1000u;

#ifndef PSP_UART_ASYNC_MODE_SUPPORT
    status = GIO_submit(hUart_IN,IOM_READ,&buf,&len,NULL);
#else
    status = GIO_submit(hUart_IN,IOM_READ,&buf,&len,&rxAppCb);

    gGioSubmitCnt++;

    if(uart0DevParams.opMode == PSP_OPMODE_POLLED)
    /* In polled mode you will not get callback */
    {
        gCbCntr++;
    }

    /* Wait until all submit requests completed */
    while(gCbCntr < gGioSubmitCnt)
    {
        ;
    }
#endif



    if(!((status == IOM_COMPLETED)||(status == IOM_PENDING))){
        UART_DEBUG("\r\n Error from GIO_read for 1000 bytes read\n");
    }

    buf.addr = Uart_TxBuffer;
    /*
     * Time out value
     */
    buf.timeout = SYS_FOREVER;
    /*
     * Length of data
     */
    len = 1000u;
#ifndef PSP_UART_ASYNC_MODE_SUPPORT
    status = GIO_submit(hUart_OUT,IOM_WRITE,&buf,&len,NULL);
#else
    status = GIO_submit(hUart_OUT,IOM_WRITE,&buf,&len,&txAppCb);

    if(uart0DevParams.opMode == PSP_OPMODE_POLLED)
    /* In polled mode you will not get callback */
    {
        gCbCntr++;
    }

    gGioSubmitCnt++;
#endif

    if(!((status == IOM_COMPLETED)||(status == IOM_PENDING)))
    {
        UART_DEBUG("\r\n Error from GIO_write for 1000 bytes write\n");
    }


#ifdef PSP_UART_ASYNC_MODE_SUPPORT
    /* Wait until all submit requests completed */
    while(gCbCntr < gGioSubmitCnt)
    {
        ;
    }
#endif
}
