/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/** \file   uart_kprintf.c
    \brief  Printf() Utility using UART as console(kprintf)

    This file implements printf() functionality that will use UART the console
    
    (C) Copyright 2006, Texas Instruments, Inc

    \author     PSG
 */

#include <stdarg.h>
#include <stdio.h>
#include <std.h>
#include <gio.h>

#ifdef CHIP_DM648
#include <ti/sdo/pspdrivers/soc/dm648/dsp/tistdtypes.h>
#endif

#ifdef CHIP_C6452
#include <ti/sdo/pspdrivers/soc/c6452/dsp/tistdtypes.h>
#endif

#include <ti/sdo/pspdrivers/drivers/uart/psp_uart.h>
#include <ti/sdo/pspdrivers/pal_os/bios/pspUtils.h>

/* UART handle for output channel */
extern GIO_Handle hUart_OUT;


static void lprbyte(Uint8 b);
static void lprhexd(Uint8 d);
static void pspConsoleOut(Int data);


/*--------------------------------------------------------------------------*
 * Module       : kprintf
 * Function     : Kernel print routine.
 * Calling Seq. : void kprintf ( Int8* strPtr, ... )
 * Input        : Int8*     strPtr      : Format string and pointer to arguments
 *                ...               :
 * Output       : None
 * Return val.  : None
 * Note         : Only hex, string and Int8 formats supported.
 *--------------------------------------------------------------------------*/
void kprintf(char *strPtr, ...)
{

  va_list ap; /* Ptr to arguments                 */
  Uint8 *sp; /* Ptr to string                    */
  Int32 pr; /* Argument                         */
  /*
   *  Check format string being 0
   */
  if (strPtr == (char*)0)
    return ;
  va_start(ap, strPtr);
  /*
   *  Loop until format string end
   */
  while (*strPtr)
  {
    if (*strPtr == '%')
    {
      switch (*++strPtr)
      {
        case 's':
          sp = va_arg(ap, Uint8*);
          while (*sp)
            pspConsoleOut(*sp++);
          break;
        case 'c':
          pr = va_arg(ap, Int32);
          pspConsoleOut((Uint8)pr);
          break;
        case 'b':
          pr = va_arg(ap, Uint32);
          lprbyte((Uint8)pr);
          break;
        case 'w':
          pr = va_arg(ap, Uint32);
          lprbyte((Uint8)(pr >> 8));
          lprbyte((Uint8)(pr >> 0));
          break;
        case 'l':
          pr = va_arg(ap, Uint32);
          lprbyte((Uint8)(pr >> 24));
          lprbyte((Uint8)(pr >> 16));
          lprbyte((Uint8)(pr >> 8));
          lprbyte((Uint8)(pr >> 0));
          break;
        case 'd':
          pr = va_arg(ap, Uint32);
          lprbyte((Uint8)(pr >> 24));
          lprbyte((Uint8)(pr >> 16));
          lprbyte((Uint8)(pr >> 8));
          lprbyte((Uint8)(pr >> 0));
          break;
        default:
          pspConsoleOut('%');
          pspConsoleOut(*strPtr);
          break;
      }
      strPtr++;
      continue;
    }
    pspConsoleOut(*strPtr++);
  }
  va_end(ap);

} /* kprintf */

/*--------------------------------------------------------------------------*
 * Module       : lprbyte
 * Function     : Hex byte printing rouitine.
 * Calling Seq. : static void lprbyte ( UB b )
 * Input        : UB        b       :
 * Output       : None
 * Return val.  : None
 * Note         :
 *--------------------------------------------------------------------------*/
static void lprbyte(Uint8 b)
{
  lprhexd(b >> 4);
  lprhexd(b &0xf);
} /* lprbyte */

/*--------------------------------------------------------------------------*
 * Module       : lprhexd
 * Function     : Hex digit printing routine.
 * Calling Seq. : static void lprhexd ( UB d )
 * Input        : UB        d       :
 * Output       : None
 * Return val.  : None
 * Note         :
 *--------------------------------------------------------------------------*/
static void lprhexd(Uint8 d)
{
  if (d > 9)
    d += 7;
  pspConsoleOut(d + '0');
} /* lprhexd */



/**
 * \brief   which prints on to the console
 */
 /* ARGSUSED */
static void pspConsoleOut(Int32 data)
{
    char charData = (char) data;
    size_t len=1;
    PSP_uartDataParam buf;
    Int32   status;

    if(NULL != hUart_OUT)
    {
        buf.addr = &charData;
        buf.timeout = SYS_FOREVER;
        len = 1;
        status = GIO_write(hUart_OUT, &buf, &len);
        if(! ((status != IOM_COMPLETED)||(status != IOM_PENDING)))
        {
            printf("\npspConsoleOut: Error writing data to UART\n\r");
        }
    }
}
