/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2007 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \file    saa7105.h
 *
 *  This file contains configuration structure and enums for SAA7105 encoder
 */

#ifndef _SAA7105_H
#define _SAA7105_H


/*
 * ========== INCLUDE FILES ==============
 */
#include <std.h>
#include <ti/sdo/pspdrivers/drivers/vport/edc.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 * ========== Defines ==============
 */
/*
 * IOCTLS Supported
 */
/**
 *  \brief SAA7105 powerdown command
 *  This doesn't take any argument
 */
#define SAA7105_POWERDOWN               (EDC_USER + 1)

/**
 *  \brief SAA7105 powerup command
 *  This doesn't take any argument
 */
#define SAA7105_POWERUP                 (EDC_USER + 2)

/**
 *  \brief SAA7105 internal colorbar enable or disable command
 *  The argument for this IOCTL is bool
 *  TRUE - Enable color bar
 *  FALSE - Disable color bar
 */
#define SAA7105_ENABLE_COLORBAR         (EDC_USER + 3)

/**
 * Apart from above IOCTLS the following generic EDC IOCTLS are supported
 *
 * EDC_CONFIG   - Argument is a pointer to SAA7105_ConfParams structure
 * EDC_RESET    - Doesn't take any argument
 *
 * The following generic EDC IOCTLS are NOT supported
 *
 * EDC_START
 * EDC_STOP
 * EDC_GET_CONFIG
 * EDC_GET_STATUS
 */

/** \brief type define SAA7105 IOCTL command type */
typedef unsigned int SAA7105_Cmd;

/**
 *  \brief SAA7105 analog output format Enum
 *  This enum gives available analog format selection with SAA7105 encoder
 */
typedef enum SAA7105_AnalogFormat_t
{
    SAA7105_AFMT_SVIDEO = 0,
    /**< SVIDEO analog out */
    SAA7105_AFMT_RGB = 1,
    /**< RGB analog out - Not supported on EVM */
    SAA7105_AFMT_YPBPR = 1,
    /**< YPbPr analog out - Not supported on EVM */
    SAA7105_AFMT_COMPOSITE = 2
    /**< Composite analog out - Not supported on EVM */
} SAA7105_AnalogFormat;     /**< Typedef for enum SAA7105_AnalogFormat_t */

/**
 *  \brief SAA7105 digital input format Enum
 *  This enum gives available input format for SAA7105 encoder
 */
typedef enum SAA7105_InputFormat_t
{
    SAA7105_IFMT_RGB24_YCBCR444,
    /**< 24 bit RGB or YCbCr 444 input - Not supported */
    SAA7105_IFMT_RGB555,
    /**< RGB 555 input - Not supported */
    SAA7105_IFMT_RGB565,
    /**< RGB 565 input - Not supported */
    SAA7105_IFMT_YCBCR422_NONEINTERLACED,
    /**< Noninterlaced YCbCr 422 input - Not supported */
    SAA7105_IFMT_YCBCR422_INTERLACED,
    /**< Interlaced YCbCr 422 input */
    SAA7105_IFMT_COLOR_INDEX
    /**< COLOR index input - Not supported */
} SAA7105_InputFormat;      /**< Typedef for enum SAA7105_InputFormat_t */

/**
 *  \brief SAA7105 operation mode Enum
 *  This enum gives available operation mode for SAA7105 encoder
 */
typedef enum SAA7105_Mode_t
{
    SAA7105_MODE_NTSC720,
    /**< NTSC 720 pixels */
    SAA7105_MODE_PAL720,
    /**< PAL 720 pixels */
    SAA7105_MODE_VGA,
    /**< VGA vesa mode - Not supported */
    SAA7105_MODE_SVGA,
    /**< SVGA vesa mode - Not supported */
    SAA7105_MODE_XGA,
    /**< XGA vesa mode - Not supported */
    SAA7105_MODE_HD480P60F,
    /**< HD 480P 60Hz mode - Not supported */
    SAA7105_MODE_HD720P60F,
    /**< HD 720P 60Hz mode - Not supported */
    SAA7105_MODE_HD1080I30F,
    /**< HD 1080I 30Hz mode - Not supported */
    SAA7105_MODE_HD720P24F
    /**< HD 720P 24Hz mode - Not supported */
} SAA7105_Mode;             /**< Typedef for enum SAA7105_Mode_t */

/**
 *  \brief SAA7105 configuration structure
 *  This structure contains configuration parameters for SAA7105 encoder
 */
typedef struct SAA7105_ConfParams_t
{
    SAA7105_AnalogFormat aFmt;
    /**< analog output format */

    SAA7105_Mode         mode;
    /**< display operation mode */

    SAA7105_InputFormat  iFmt;
    /**< Digital input mode */

    Bool                 enableSlaveMode;
    /**< selection of slave mode */

    Bool                 enableBT656Sync;
    /**< selection of BT656 mode */
} SAA7105_ConfParams;       /**< Typedef for structure SAA7105_ConfParams_t */

#ifdef __cplusplus
}
#endif

#endif  /* #ifndef _SAA7105_H */

/******************************************************************************/
/* End of file                                                                */
/******************************************************************************/
