/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2007 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/* ======================================================================== */
/*  NAME                                                                    */
/*      double_word_byte_swap -- byte swap of a double word                 */
/*S                                                                        S*/
/*S AUTHOR                                                                 S*/
/*S     Pavan Shastry                                                      S*/
/*S                                                                        S*/
/*S REVISION HISTORY                                                       S*/
/*S     25-Apr-2007 Initial version of double_word_byte_swap ... P.Shastry S*/
/*S                                                                        S*/
/*                                                                          */
/*  USAGE                                                                   */
/*      This routine has the following C prototype:                         */
/*                                                                          */
/*      void double_word_byte_swap                                          */
/*      (                                                                   */
/*          unsigned char *restrict   buffer_ptr,                           */
/*          unsigned int              no_of_bytes,                          */
/*      )                                                                   */
/*                                                                          */
/*      The double_word_byte_swap routine accepts a buffer of size          */
/*      no_of_bytes bytes. It does a byte swap of double word.              */
/*      See description below for more details.                             */
/*                                                                          */
/*  DESCRIPTION                                                             */
/*      This code is written to perform a byte swap operation of            */
/*      a double word, in the manner indicated below -                      */
/*                                                                          */
/*      Input  Double word: A7 A6 A5 A4  A3 A2 A1 A0                        */
/*      Output Double word: A0 A1 A2 A3  A4 A5 A6 A7                        */
/*                                                                          */
/*      The Computation is in-place. The input buffer is written with       */
/*      bytes swapped in the above manner.                                  */
/*                                                                          */
/*      The number of double words, for which byte swapping will            */
/*      be performed is equal to (no_of_bytes >> 3).                        */
/*                                                                          */
/*  ASSUMPTIONS                                                             */
/*      1. 'no_of_bytes' is a multiple of 8.                                */
/*      2. Input buffer should be double word aligned.                      */
/*                                                                          */
/*  NOTES                                                                   */
/*      1. If 'no_of_bytes' is not a multiple of 8, then largest            */
/*         multiple of 8, which is lesser than no_of_bytes will be          */
/*         assumed.                                                         */
/*      2. For best performance of this routine, use -o3 and No debug       */
/*         as compile options.                                              */
/*                                                                          */
/*  SOURCE                                                                  */
/*      None.                                                               */
/* ------------------------------------------------------------------------ */
/*            Copyright (c) 2007 Texas Instruments, Incorporated.           */
/*                           All Rights Reserved.                           */
/* ======================================================================== */

void double_word_byte_swap(unsigned char *buffer_ptr, unsigned int no_of_bytes)
{
    double *restrict read_ptr;
    double *restrict write_ptr;
    double input_dword;
    double output_word;
    unsigned int word0;
    unsigned int word1;
    unsigned int dword_cntr;
    unsigned int dword_total_cnt;
    unsigned int odd_flag;

    read_ptr = (double*)buffer_ptr;
    write_ptr = (double*)buffer_ptr;

    /* ------------------------------------------------------------- */
    /* No of double words to be processed                            */
    /* ------------------------------------------------------------- */
    dword_total_cnt = no_of_bytes >> 3;

    /* ------------------------------------------------------------- */
    /* Check if number of double words is odd.                       */
    /* Force number of double words to be even.                      */
    /* ------------------------------------------------------------- */
    odd_flag = dword_total_cnt & 0x01;
    dword_total_cnt = dword_total_cnt & 0xFFFFFFFE;

    /* ------------------------------------------------------------- */
    /* Each iteration handles two double words                       */
    /* ------------------------------------------------------------- */
    for (dword_cntr = 0; dword_cntr < dword_total_cnt; dword_cntr += 2)
    {
        input_dword = read_ptr[dword_cntr];

        word0 = _lo(input_dword);
        word1 = _hi(input_dword);
        word0 = _swap4(word0);
        word1 = _swap4(word1);
        word0 = _rotl(word0, 16);
        word1 = _rotl(word1, 16);
        output_word = _itod(word0, word1);

        write_ptr[dword_cntr] = output_word;

        input_dword = read_ptr[dword_cntr + 1];

        word0 = _lo(input_dword);
        word1 = _hi(input_dword);
        word0 = _swap4(word0);
        word1 = _swap4(word1);
        word0 = _rotl(word0, 16);
        word1 = _rotl(word1, 16);
        output_word = _itod(word0, word1);

        write_ptr[dword_cntr + 1] = output_word;
    }

    /* ------------------------------------------------------------- */
    /* If the number of double words is odd, then the last double    */
    /* word is handled here.                                         */
    /* ------------------------------------------------------------- */
    if (odd_flag)
    {
        input_dword = read_ptr[dword_cntr];

        word0 = _lo(input_dword);
        word1 = _hi(input_dword);
        word0 = _swap4(word0);
        word1 = _swap4(word1);
        word0 = _rotl(word0, 16);
        word1 = _rotl(word1, 16);
        output_word = _itod(word0, word1);

        write_ptr[dword_cntr] = output_word;
    }
}
/* ======================================================================== */
/*  End of file:  double_word_byte_swap_i.c                                 */
/* ------------------------------------------------------------------------ */
/*            Copyright (c) 2007 Texas Instruments, Incorporated.           */
/*                           All Rights Reserved.                           */
/* ======================================================================== */
