/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2007 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/** \file   psp_bios_vport_hd_loopback.c
 *
 *  \brief  This file contains the function which captures 720P/1080I
 *          resolution from VP0 and displays it in VP1
 *
 */

/*
 * ========== INCLUDE FILES ===========
 */

#include <std.h>
#include <log.h>
#include <bcache.h>
#include <string.h>

#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/fvid.h>
#include <ti/sdo/pspdrivers/drivers/vport/edc.h>
#include <ti/sdo/pspdrivers/drivers/vport/vport.h>
#include <ti/sdo/pspdrivers/drivers/vport/vportdis.h>
#include <ti/sdo/pspdrivers/drivers/vport/vportcap.h>

#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/sample/psp_bios_vport_common.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/sample/psp_bios_vport_capParams.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/sample/psp_bios_vport_disParams.h>


/*
 * ======== USER CONFIGURABLE MACROS ========
 */

/*
 * Video Mode
 *
 * Set this macro to MODE_720P_60HZ or MODE_1080I_60HZ for HD
 * loopback
 * MODE_720P_60HZ  - To capture and display in MODE_720P_60HZ format
 * MODE_1080I_60HZ - To capture and display in MODE_1080I_60HZ format
 * MODE_1080I_50HZ - To capture and display in MODE_1080I_50HZ format
 * Others video modes are not supported
 *
 */
#define CFG_VIDEO_MODE                      (MODE_1080I_60HZ)

/*
 * Number of frames for video loop back
 *
 * This sample application will run for 'CFG_NUM_FRAME_COUNT' amount of frames.
 * After which the application will close.
 */
#define CFG_NUM_FRAME_COUNT                 (100000u)

/*
 * Enable or disable double word byte swap function
 *
 * Set this macro as 1/0 to enable or disable double word byte swap software
 * work around for Video Port endian issue
 *
 * 1    - Enable double word byte swap
 * 0    - Disable double word byte swap
 *
 */
#define CFG_ENABLE_DOUBLEWORD_BYTE_SWAP     (1)

/*
 * Define this macro to 1/0 to enable or disable software workaround for 1080i
 * and 720p capture in production EVM.
 * For proper 1080i capture, TVP7000 HSYNC polarity is inverted (active low) and
 * hence a horizontal shift is required in the field1 and field2 HSTART.
 * For 720p capture, DM648 VPORT detects the progressive input from TVP7000 as
 * interlaced video. Hence VPORT driver is configured to capture both fields
 * so that two 720p frames are captured per FVID call.
 *
 * Note: For proper HD capture "CFG_ENABLE_HDCAPTURE_WORKAROUND" in file
 *       "_tvp7000.c" should be set same as this macro value and the EDC driver
 *       should be recompiled. By default both these macros are set as 1. So no
 *       recompilation is required.
 *
 * 1    - Enable software around for 1080i and 720p HD capture
 * 0    - Disable software around for 1080i and 720p HD capture
 *
 */
#define CFG_ENABLE_HDCAPTURE_WORKAROUND     (1)


/*
 * ======== DEFINES =========
 */

#if (CFG_VIDEO_MODE == MODE_720P_60HZ)

#if (CFG_ENABLE_HDCAPTURE_WORKAROUND == 1)
/* Number of Horizontal Pixels to capture */
#define CAP_NUMPIXELS                       (1280)
/* Number of Lines to capture */
#define CAP_NUMLINES                        (1440)
#else
/* Number of Horizontal Pixels to capture */
#define CAP_NUMPIXELS                       (1280)
/* Number of Lines to capture */
#define CAP_NUMLINES                        (720)
#endif  /* #if (CFG_ENABLE_HDCAPTURE_WORKAROUND == 1) */

/* Number of Horizontal Pixels to display */
#define DIS_NUMPIXELS                       (1280)
/* Number of Lines to display */
#define DIS_NUMLINES                        (720)

#endif  /* #if (CFG_VIDEO_MODE == MODE_720P_60HZ) */


#if (CFG_VIDEO_MODE == MODE_1080I_60HZ) || (CFG_VIDEO_MODE == MODE_1080I_50HZ)

/* Number of Horizontal Pixels to capture */
#define CAP_NUMPIXELS                       (1920)
/* Number of Lines to capture */
#define CAP_NUMLINES                        (1080)

/* Number of Horizontal Pixels to display */
#define DIS_NUMPIXELS                       (1920)
/* Number of Lines to display */
#define DIS_NUMLINES                        (1080)

#endif  /* #if (CFG_VIDEO_MODE == MODE_1080I) */


/* Capture frame size in bytes */
#define CAP_FRAME_SIZE                      (CAP_NUMPIXELS * CAP_NUMLINES * 2)

/* Display frame size in bytes */
#define DIS_FRAME_SIZE                      (DIS_NUMPIXELS * DIS_NUMLINES * 2)


/*
 * ======== GLOBALS =========
 */

/* Video Port configuration parameters for HDTV Capture.
   Using default value and disabling dual channel.
   Note: This variable is used in BIOS TCF file. */
VPORT_PortParams vCapParamsPort = CAP_PARAMS_PORT_HDTV_DEFAULT;

/* Video Port configuration parameters for display.
   Using default value.
   Note: This variable is used in BIOS TCF file. */
VPORT_PortParams vDisParamsPort = DIS_PARAMS_PORT_DEFAULT;

/* Global variable to check the current frame count in CCS watch window */
Uint32 AppHdFrameCnt = 0;


#if (CFG_VIDEO_MODE == MODE_720P_60HZ)

#if (CFG_ENABLE_HDCAPTURE_WORKAROUND == 1)
/* Vport Capture driver configuration */
static VPORTCAP_Params vCapParamsChan = CAP_PARAMS_CHAN_720P_60HZ_DEFAULT_FIX;
#else
/* Vport Capture driver configuration */
static VPORTCAP_Params vCapParamsChan = CAP_PARAMS_CHAN_720P_60HZ_DEFAULT;
#endif  /* #if (CFG_ENABLE_HDCAPTURE_WORKAROUND == 1) */

/* TVP7000 EDC driver configuration */
static TVP7000_ConfParams vCapParamsDecoder
    = CAP_PARAMS_TVP7000_720P_60HZ_DEFAULT;

/* Vport Display driver configuration */
static VPORTDIS_Params vDisParamsChan = DIS_PARAMS_CHAN_720P_60HZ_DEFAULT;

/* THS8200 EDC driver configuration */
static THS8200_ConfParams vDisParamsEncoder
    = DIS_PARAMS_THS8200_720P_60HZ_DEFAULT;

#endif  /* #if (CFG_VIDEO_MODE == MODE_720P_60HZ) */


#if (CFG_VIDEO_MODE == MODE_1080I_60HZ)

#if (CFG_ENABLE_HDCAPTURE_WORKAROUND == 1)
/* Vport Capture driver configuration */
static VPORTCAP_Params vCapParamsChan = CAP_PARAMS_CHAN_1080I_60HZ_DEFAULT_FIX;
#else
/* Vport Capture driver configuration */
static VPORTCAP_Params vCapParamsChan = CAP_PARAMS_CHAN_1080I_60HZ_DEFAULT;
#endif  /* #if (CFG_ENABLE_HDCAPTURE_WORKAROUND == 1) */

/* Vport Display driver configuration */
static VPORTDIS_Params vDisParamsChan = DIS_PARAMS_CHAN_1080I_60HZ_DEFAULT;

#endif  /* #if (CFG_VIDEO_MODE == MODE_1080I_60HZ) */


#if (CFG_VIDEO_MODE == MODE_1080I_50HZ)

#if (CFG_ENABLE_HDCAPTURE_WORKAROUND == 1)
/* Vport Capture driver configuration */
static VPORTCAP_Params vCapParamsChan = CAP_PARAMS_CHAN_1080I_50HZ_DEFAULT_FIX;
#else
/* Vport Capture driver configuration */
static VPORTCAP_Params vCapParamsChan = CAP_PARAMS_CHAN_1080I_50HZ_DEFAULT;
#endif  /* #if (CFG_ENABLE_HDCAPTURE_WORKAROUND == 1) */

/* TVP7000 EDC driver configuration */
static TVP7000_ConfParams vCapParamsDecoder
    = CAP_PARAMS_TVP7000_1080I_50HZ_DEFAULT;

/* Vport Display driver configuration */
static VPORTDIS_Params vDisParamsChan = DIS_PARAMS_CHAN_1080I_50HZ_DEFAULT;

/* THS8200 EDC driver configuration */
static THS8200_ConfParams vDisParamsEncoder
    = DIS_PARAMS_THS8200_1080I_50HZ_DEFAULT;

#endif  /* #if (CFG_VIDEO_MODE == MODE_1080I_50HZ) */


/*
 * ============= FUNC DECLARATIONS ==============
 */

#if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1)

/* Function which does the 8-byte swapping - Software fix for big endian mode
   operation of Video Port */
extern void double_word_byte_swap(Uint8 *bufferPtr, Uint32 numOfBytes);

#endif  /* #if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1) */

extern EDMA3_DRV_Result edma3init(void);

static Int32 test_hd_loopback(EDMA3_DRV_Handle hEdma,
                              Uint32 frameCnt,
                              Int32 sedId);


/*
 * ============= EXTERNS ==============
 */

#ifdef _6x_
extern far LOG_Obj trace;
#else
extern LOG_Obj trace;
#endif

/* Heap ID defined in the BIOS configuration file.
   This memory segment is used to allocate video frame buffers. */
extern Int EXTERNALHEAP;

/* EDMA handle */
extern EDMA3_DRV_Handle hEdma;


/*
 * ============= DEFINITIONS ==============
 */


/*
 * ======== tskSdLoopback ========
 * Video application test task.
 * Created as a separate task in BIOS TCF file.
 */
void tskHdLoopback(void)
{
    EDMA3_DRV_Result result = EDMA3_DRV_SOK;

    /* Initialize EDMA3 driver */
    result = edma3init();
    if (EDMA3_DRV_SOK != result)
    {
        LOG_printf(&trace, "Can't initialized EDMA3");
        return;
    }

    /* Run HD loop back application */
    test_hd_loopback(hEdma, CFG_NUM_FRAME_COUNT, EXTERNALHEAP);
}



/*
 * ======== test_hd_loopback ========
 * Video HD loopback function
 * Function which performs the HD loopback
 * hEdma            - Handle to the EDMA3 driver
 * totalFrameCnt    - Number of frames for loop back operation
 * sedId            - Heap ID where the driver FVID buffers will be allocated
 */
static Int32 test_hd_loopback(EDMA3_DRV_Handle hEdma,
                              Uint32 totalFrameCnt,
                              Int32 sedId)
{
    Int         status = IOM_COMPLETED;
    Int         frames;
    /* Array containing display and capture channel information */
    ChannelInfo disChInfo;
    ChannelInfo capChInfo;
    /* Array containing display and capture driver name strings
       Use this strings as they are for proper driver creation */
    Int8        *vPortCapStrings = "/VP0CAPTURE/A/0";
    Int8        *vPortDisStrings = "/VP1DISPLAY/THS8200";

    if (!totalFrameCnt)
    {
        LOG_printf(&trace, "Give non-zero frame count\n");
        return -1;
    }


    /* Allocate both capture and display frame buffers
       in external heap memory */
    vCapParamsChan.segId = sedId;
    vDisParamsChan.segId = sedId;

    /* Assign EDMA3 driver handle to channel params */
    vCapParamsChan.hEdma = hEdma;
    vDisParamsChan.hEdma = hEdma;


    /* Create and configure capture drivers */
    capChInfo.chanHandle = FVID_create(vPortCapStrings,
                                       IOM_INPUT,
                                       &status,
                                       (Ptr)&vCapParamsChan,
                                       NULL);
    if (IOM_COMPLETED != status)
    {
        LOG_printf(&trace, "Failed to create capture channel\n");
    }


    /* Create and configure display driver */
    if (IOM_COMPLETED == status)
    {
        disChInfo.chanHandle = FVID_create(vPortDisStrings,
                                           IOM_OUTPUT,
                                           &status,
                                           (Ptr)&vDisParamsChan,
                                           NULL);
        if (IOM_COMPLETED != status)
        {
            LOG_printf(&trace, "Failed to create display channels\n");
        }
    }


    /* Configure external video encoders (THS8200) & decoders (TVP7000) */
    if (IOM_COMPLETED == status)
    {
        /* Only need to configure if the mode is not 1080I 60Hz. Since 1080I
           60Hz is the default mode of operation for EDC driver in HD */
#if !(CFG_VIDEO_MODE == MODE_1080I_60HZ)

        /* Configure THS8200 */
        status |= FVID_control(disChInfo.chanHandle,
                               VPORT_CMD_EDC_BASE + EDC_CONFIG,
                               (Ptr)&vDisParamsEncoder);

        /* Configure TVP7000 */
        status |= FVID_control(capChInfo.chanHandle,
                               VPORT_CMD_EDC_BASE + EDC_CONFIG,
                               (Ptr)&vCapParamsDecoder);

#endif  /* #if !(CFG_VIDEO_MODE == MODE_1080I_60HZ) */
    }
    else
    {
        LOG_printf(&trace, "Failed to open - Display channel\n");
    }


    /* Start display and capture operations */
    if (IOM_COMPLETED == status)
    {
        status |= FVID_control(disChInfo.chanHandle, VPORT_CMD_START, NULL);
        status |= FVID_control(capChInfo.chanHandle, VPORT_CMD_START, NULL);
    }
    else
    {
        LOG_printf(&trace, "Failed to configure cap/disp channel device\n");
    }


    /* Request a frame buffer from display & capture driver */
    if (IOM_COMPLETED == status)
    {
        /* Display buffer will return a free buffer */
        status |= FVID_alloc(disChInfo.chanHandle, &(disChInfo.frame));

        /* Capture buffer will return the latest captured buffer */
        status |= FVID_alloc(capChInfo.chanHandle, &(capChInfo.frame));
    }
    else
    {
        LOG_printf(&trace, "Failed to start display/capture driver\n");
    }


    if (IOM_COMPLETED == status)
    {
        frames = 0;
        while (frames < totalFrameCnt)
        {
            /* Invalidate the buffer before giving to capture driver */
            BCACHE_inv((Uint8 *)capChInfo.frame->frame.iFrm.y1,
                       (CAP_FRAME_SIZE),
                       TRUE);

            /* Give the old capture frame buffer back to driver and get the
               recently captured frame buffer */
            status = FVID_exchange(capChInfo.chanHandle, &(capChInfo.frame));

#if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1)
            /* Fix for Video Port bigendian mode - This is for capture port */
            double_word_byte_swap((Uint8 *)capChInfo.frame->frame.iFrm.y1,
                                  CAP_FRAME_SIZE);
#endif  /* #if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1) */


#if (CFG_VIDEO_MODE == MODE_720P_60HZ)

#if (CFG_ENABLE_HDCAPTURE_WORKAROUND == 1)

            /*
             * Fpr 720p, capture driver is configured to capture two frames in
             * field 1 and field 2 data - each field contains one frame of 720p
             * data
             */

            /* Copy frame 1 - field 1 data to display buffer */
            memcpy(disChInfo.frame->frame.iFrm.y1,
                   capChInfo.frame->frame.iFrm.y1,
                   DIS_NUMPIXELS * DIS_NUMLINES);
            memcpy(disChInfo.frame->frame.iFrm.cb1,
                   capChInfo.frame->frame.iFrm.cb1,
                   (DIS_NUMPIXELS * DIS_NUMLINES) >> 1);
            memcpy(disChInfo.frame->frame.iFrm.cr1,
                   capChInfo.frame->frame.iFrm.cr1,
                   (DIS_NUMPIXELS * DIS_NUMLINES) >> 1);

#if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1)
            /* Fix for Video Port bigendian mode - This is for display port */
            double_word_byte_swap((Uint8 *)disChInfo.frame->frame.iFrm.y1,
                                  DIS_FRAME_SIZE);
#endif  /* #if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1) */

            /* Flush and invalidate the processed buffer so that the EDMA reads
               the processed data */
            BCACHE_wbInv((Uint8 *)disChInfo.frame->frame.iFrm.y1,
                         (DIS_FRAME_SIZE),
                         TRUE);

            /* Give the captured frame buffer to display driver and get a
               free frame buffer for next capture */
            status |= FVID_exchange(disChInfo.chanHandle, &(disChInfo.frame));

            /* LOG_printf(&trace, "%d\n", frames); */

            if (IOM_COMPLETED != status)
            {
                LOG_printf(&trace, "IOM_COMPLETED != status\n");
                break;
            }

            frames++;
            /* Update global frame count for check purpose */
            AppHdFrameCnt = frames;

            /* Copy frame 2 - field 2 data to display buffer */
            memcpy(disChInfo.frame->frame.iFrm.y1,
                   capChInfo.frame->frame.iFrm.y2,
                   DIS_NUMPIXELS * DIS_NUMLINES);
            memcpy(disChInfo.frame->frame.iFrm.cb1,
                   capChInfo.frame->frame.iFrm.cb2,
                   (DIS_NUMPIXELS * DIS_NUMLINES) >> 1);
            memcpy(disChInfo.frame->frame.iFrm.cr1,
                   capChInfo.frame->frame.iFrm.cr2,
                   (DIS_NUMPIXELS * DIS_NUMLINES) >> 1);

#else   /* #if (CFG_ENABLE_HDCAPTURE_WORKAROUND == 1) */

            /* Copy data to display buffer if no HD captrue workaround */
            memcpy(disChInfo.frame->frame.iFrm.y1,
                   capChInfo.frame->frame.iFrm.y1,
                   DIS_FRAME_SIZE);

#endif  /* #if (CFG_ENABLE_HDCAPTURE_WORKAROUND == 1) */

#else   /* #if (CFG_VIDEO_MODE == MODE_720P_60HZ) */

            /* Copy data to display buffer for 1080i HD capture */
            memcpy(disChInfo.frame->frame.iFrm.y1,
                   capChInfo.frame->frame.iFrm.y1,
                   DIS_FRAME_SIZE);

#endif  /* #if (CFG_VIDEO_MODE == MODE_720P_60HZ) */


#if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1)
            /* Fix for Video Port bigendian mode - This is for display port */
            double_word_byte_swap((Uint8 *)disChInfo.frame->frame.iFrm.y1,
                                  DIS_FRAME_SIZE);
#endif  /* #if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1) */

            /* Flush and invalidate the processed buffer so that the EDMA reads
               the processed data */
            BCACHE_wbInv((Uint8 *)disChInfo.frame->frame.iFrm.y1,
                         (DIS_FRAME_SIZE),
                         TRUE);

            /* Give the captured frame buffer to display driver and get a
               free frame buffer for next capture */
            status |= FVID_exchange(disChInfo.chanHandle, &(disChInfo.frame));

            /* LOG_printf(&trace, "%d\n", frames); */

            if (IOM_COMPLETED != status)
            {
                LOG_printf(&trace, "IOM_COMPLETED != status\n");
                break;
            }

            frames++;
            /* Update global frame count for check purpose */
            AppHdFrameCnt = frames;
        }/* while (frame count) */


        /* Stop capture and display operation */
        if (IOM_COMPLETED == status)
        {
            status |= FVID_control(disChInfo.chanHandle, VPORT_CMD_STOP, NULL);
            status |= FVID_control(capChInfo.chanHandle, VPORT_CMD_STOP, NULL);
        }
        else
        {
            LOG_printf(&trace, "Failed to alloc or exchange buffers\n");
        }


        /* Delete capture and display drivers */
        if (IOM_COMPLETED == status)
        {
            status |= FVID_delete(disChInfo.chanHandle);
            status |= FVID_delete(capChInfo.chanHandle);
            if (IOM_COMPLETED == status)
            {
                LOG_printf(&trace, "Application closed successfully!!\n");
            }
        }
        else
        {
            LOG_printf(&trace, "Failed to stop display/capture channel\n");
        }
    }
    else if (IOM_COMPLETED != status)
    {
        LOG_printf(&trace, "Failed to alloc or exchange buffers\n");
    }

    return status;
}


/******************************************************************************/
/* End of file                                                                */
/******************************************************************************/
