/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2007 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/** \file   psp_bios_vport_raw_capture_loopback.c
 *
 *  \brief  This file contains two tasks which illustrate 8bit/16bit raw capture
 *          of VPORT driver.
 *          Task 1 captures SD video frames from video player or camera through
 *          VP0 channel A and displays the captured frames in RAW mode through
 *          VP3 in 8bit/16bit. With the help of Daughter card VP3 signals are
 *          looped back to VP4.
 *          Task 2 captures the loop backed frame in RAW format through VP4
 *          channel A in 8bit/16bit and displays the same in SD resolution
 *          through VP1 which can be viewed using a TV.
 *
 *          Note:
 *          For 8-bit operation, 27 MHz external clock should be given to
 *          VCLK1 (VCLKIN) of VP3.
 *          For 16-bit operation, 13.5 MHz external clock should be given to
 *          VCLK1 (VCLKIN) of VP3.
 *
 *          This application uses the NEW FVID APIs.
 *
 */

/*
 * ========== INCLUDE FILES ===========
 */

#include <std.h>
#include <log.h>
#include <string.h>
#include <bcache.h>
#include <sem.h>

#include <ti/sdo/pspdrivers/pal_sys/dm648/pal_sys.h>
#include <ti/sdo/pspdrivers/pal_sys/dm648/pal_sys_soc.h>

#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/fvid.h>
#include <ti/sdo/pspdrivers/drivers/vport/edc.h>
#include <ti/sdo/pspdrivers/drivers/vport/vport.h>
#include <ti/sdo/pspdrivers/drivers/vport/vportcap.h>
#include <ti/sdo/pspdrivers/drivers/vport/vportdis.h>

#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/sample/psp_bios_vport_common.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/sample/psp_bios_vport_capParams.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/sample/psp_bios_vport_disParams.h>


/*
 * ======== USER CONFIGURABLE MACROS ========
 */

/*
 * Enables 8-bit or 16-bit RAW display/capture
 *
 * Set this macro to MODE_RAW_8BIT or MODE_RAW_16BIT for 8/16-bit operation
 * MODE_RAW_8BIT  - for 8-Bit RAW loop back mode
 * MODE_RAW_16BIT - for 16-Bit RAW loop back mode
 *
 */
#define CFG_RAW_LOOPBACK_MODE               (MODE_RAW_8BIT)

/*
 * Video Mode
 *
 * Set this macro to MODE_NTSC or MODE_PAL for NTSC/PAL loopback
 * MODE_NTSC - To capture and display in NTSC format
 * MODE_PAL  - To capture and display in PAL format
 * Others video modes are not supported
 *
 */
#define CFG_VIDEO_MODE                      (MODE_NTSC)

/*
 * Number of frames for loop back operation
 *
 * This sample application will run for 'CFG_NUM_FRAME_COUNT' amount of frames.
 * After which the application will close.
 */
#define CFG_NUM_FRAME_COUNT                 (100000u)

/*
 * Enable or disable double word byte swap function
 *
 * Set this macro as 1/0 to enable or disable double word byte swap software
 * work around for Video Port endian issue
 *
 * 1    - Enable double word byte swap
 * 0    - Disable double word byte swap
 *
 */
#define CFG_ENABLE_DOUBLEWORD_BYTE_SWAP     (1)


/*
 * ======== DEFINES =========
 */

/* Number of frame buffers to allocate per driver instance */
#define NUM_FRAME_BUFFERS                   (3)

#if (CFG_VIDEO_MODE == MODE_NTSC)

/* Number of Horizontal Pixels */
#define NUMPIXELS                           (720)
/* NTSC D1 Number of Lines */
#define NUMLINES                            (480)

#endif  /* #if (CFG_VIDEO_MODE == MODE_NTSC) */

#if (CFG_VIDEO_MODE == MODE_PAL)

/* Number of Horizontal Pixels */
#define NUMPIXELS                           (720)
/* PAL D1 Number of Lines */
#define NUMLINES                            (576)

#endif  /* #if (CFG_VIDEO_MODE == MODE_PAL) */

/* Frame size in bytes */
#define FRAME_SIZE                          (NUMPIXELS * NUMLINES * 2)


/*
 * ======== GLOBALS =========
 */

/* Video Port configuration parameters for VP4 RAW Capture.
   No EDC drivers are hooked to the VPORT driver.
   Note: This variable is used in BIOS TCF file. */
VPORT_PortParams vCapParamsPortRaw = CAP_PARAMS_PORT_RAW_DEFAULT(FALSE);

/* Video Port configuration parameters for VP3 RAW display.
   No EDC drivers are hooked to the VPORT driver.
   Note: This variable is used in BIOS TCF file. */
VPORT_PortParams vDisParamsPortRaw = DIS_PARAMS_PORT_RAW_DEFAULT;

/* Video Port configuration parameters for VP0 Capture.
   Note: This variable is used in BIOS TCF file. */
VPORT_PortParams vCapParamsPort = CAP_PARAMS_PORT_DEFAULT(FALSE);

/* Video Port configuration parameters for VP1 display.
   Note: This variable is used in BIOS TCF file. */
VPORT_PortParams vDisParamsPort = DIS_PARAMS_PORT_DEFAULT;

/* Frame count of display task.
   Global variable to check the current frame count in CCS watch window */
Uint32 AppRawDisplayFrameCnt = 0;

/* Frame count of capture task.
   Global variable to check the current frame count in CCS watch window */
Uint32 AppRawCaptureFrameCnt = 0;


#if (CFG_VIDEO_MODE == MODE_NTSC)

#if (CFG_RAW_LOOPBACK_MODE == MODE_RAW_16BIT)

/* Vport Capture driver configuration for VP4 */
static VPORTCAP_ParamsRaw vCapParamsChanRaw = CAP_PARAMS_CHAN_NTSC_RAW_16BIT;

/* Vport Display driver configuration for VP3 */
static VPORTDIS_Params    vDisParamsChanRaw = DIS_PARAMS_CHAN_NTSC_RAW_16BIT;

#endif  /* #if (CFG_RAW_LOOPBACK_MODE == MODE_RAW_16BIT) */

#if (CFG_RAW_LOOPBACK_MODE == MODE_RAW_8BIT)

/* Vport Capture driver configuration for VP4 */
static VPORTCAP_ParamsRaw vCapParamsChanRaw = CAP_PARAMS_CHAN_NTSC_RAW_8BIT;

/* Vport Display driver configuration for VP3 */
static VPORTDIS_Params    vDisParamsChanRaw = DIS_PARAMS_CHAN_NTSC_RAW_8BIT;

#endif  /* #if (CFG_RAW_LOOPBACK_MODE == MODE_RAW_8BIT) */

/* Vport Capture driver configuration.
   Using embedded sync. */
static VPORTCAP_Params vCapParamsChan = CAP_PARAMS_CHAN_BT656_DEFAULT(NTSC);

/* Vport Display driver configuration.
   Using embedded sync */
static VPORTDIS_Params vDisParamsChan = DIS_PARAMS_CHAN_BT656_DEFAULT(NTSC);

#endif  /* #if (CFG_VIDEO_MODE == MODE_NTSC) */


#if (CFG_VIDEO_MODE == MODE_PAL)

#if (CFG_RAW_LOOPBACK_MODE == MODE_RAW_16BIT)

/* Vport Capture driver configuration for VP4 */
static VPORTCAP_ParamsRaw vCapParamsChanRaw = CAP_PARAMS_CHAN_PAL_RAW_16BIT;

/* Vport Display driver configuration for VP3 */
static VPORTDIS_Params    vDisParamsChanRaw = DIS_PARAMS_CHAN_PAL_RAW_16BIT;

#endif  /* #if (CFG_RAW_LOOPBACK_MODE == MODE_RAW_16BIT) */

#if (CFG_RAW_LOOPBACK_MODE == MODE_RAW_8BIT)

/* Vport Capture driver configuration for VP4 */
static VPORTCAP_ParamsRaw vCapParamsChanRaw = CAP_PARAMS_CHAN_PAL_RAW_8BIT;

/* Vport Display driver configuration for VP3 */
static VPORTDIS_Params    vDisParamsChanRaw = DIS_PARAMS_CHAN_PAL_RAW_8BIT;

#endif  /* #if (CFG_RAW_LOOPBACK_MODE == MODE_RAW_8BIT) */

/* Vport Capture driver configuration.
   Using embedded sync */
static VPORTCAP_Params vCapParamsChan = CAP_PARAMS_CHAN_BT656_DEFAULT(PAL);

/* TVP5154 EDC driver configuration.
   Using embedded sync and Composite Analog Channel A for all 8 decoders */
static TVP5154_ConfParams vCapParamsDecoder
    = CAP_PARAMS_TVP5154_EMBEDDED_DEFAULT(PAL, COMPOSITE_A);

/* Vport Display driver configuration.
   Using embedded sync */
static VPORTDIS_Params vDisParamsChan = DIS_PARAMS_CHAN_BT656_DEFAULT(PAL);

/* SAA7105 EDC driver configuration.
   Using Svideo Analog output */
static SAA7105_ConfParams vDisParamsEncoder
    = DIS_PARAMS_SAA7105_SDTV_EMBEDDED_DEFAULT(PAL720, SVIDEO);

#endif  /* #if (CFG_VIDEO_MODE == MODE_PAL) */

/* Semaphore used by display task to signal start of capture operation */
static SEM_Handle hSemStartCapture;
/* Semaphore used by capture task to signal stop of display operation */
static SEM_Handle hSemStopDisplay;


/*
 * ============= FUNC DECLARATIONS ==============
 */

#if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1)

/* Function which does the 8-byte swapping - Software fix for big endian mode
   operation of Video Port */
extern void double_word_byte_swap(Uint8 *bufferPtr, Uint32 numOfBytes);

#endif  /* #if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1) */

extern EDMA3_DRV_Result edma3init(void);

static Int32 test_raw_display(EDMA3_DRV_Handle hEdma,
                              Uint32 frameCnt,
                              Int32 sedId);

static Int32 test_raw_capture(EDMA3_DRV_Handle hEdma,
                              Uint32 frameCnt,
                              Int32 sedId);


/*
 * ============= EXTERNS ==============
 */

#ifdef _6x_
extern far LOG_Obj trace;
#else
extern LOG_Obj trace;
#endif

/* Heap ID defined in the BIOS configuration file.
   This memory segment is used to allocate video frame buffers. */
extern Int EXTERNALHEAP;

/* EDMA handle */
extern EDMA3_DRV_Handle hEdma;


/*
 * ============= DEFINITIONS ==============
 */


/*
 * ======== tskInit ========
 * Video application init task.
 * Created as a separate task in BIOS TCF file. This task is given higher
 * priority than the other two tasks so that it runs first and initialize
 * EDMA driver.
 */
void tskInit(void)
{
    EDMA3_DRV_Result    result = EDMA3_DRV_SOK;

    /* Initialize EDMA3 driver */
    result = edma3init();
    if (EDMA3_DRV_SOK != result)
    {
        LOG_printf(&trace, "Can't initialize EDMA3!!\n");
        return;
    }

    /* Create the semaphores */
    hSemStartCapture = SEM_create(0, NULL);
    hSemStopDisplay = SEM_create(0, NULL);
    if (NULL == hSemStartCapture || NULL == hSemStopDisplay)
    {
        LOG_printf(&trace, "Can't create semaphores!!\n");
        return;
    }
}



/*
 * ======== tskRawDisplay ========
 * VP0 channel A to VP3 raw display task
 * Created as a separate task in BIOS TCF file.
 */
void tskRawDisplay(void)
{
    /* Run VP0 NTSC/PAL capture -> VP3 raw display */
    test_raw_display(hEdma, CFG_NUM_FRAME_COUNT, EXTERNALHEAP);
}



/*
 * ======== tskRawCapture ========
 * VP4 raw capture to VP1 display task
 * Created as a separate task in BIOS TCF file.
 */
void tskRawCapture(void)
{
    /* Run VP4 raw capture -> VP1 display */
    test_raw_capture(hEdma, CFG_NUM_FRAME_COUNT, EXTERNALHEAP);
}



/*
 * ======== test_raw_display ========
 * RAW Display function
 * Function which performs VP0 NTSC/PAL capture and displays the frame buffer
 * in RAW format in VP3.
 *
 * hEdma            - Handle to the EDMA3 driver
 * totalFrameCnt    - Number of frames for capture/display operation
 * sedId            - Heap ID where the driver FVID buffers will be allocated
 */
static Int32 test_raw_display(EDMA3_DRV_Handle hEdma,
                              Uint32 totalFrameCnt,
                              Int32 sedId)
{
    Int         status = IOM_COMPLETED;
    Int         frames, bufCount;
    /* Display and capture channel information */
    ChannelInfo disChInfoRaw;
    ChannelInfo capChInfo;
    /* Display and capture driver name strings
       Use this strings as they are for proper driver creation */
    Int8        *vPortDisStringsRaw = "/VP3DISPLAY";
    Int8        *vPortCapStrings = "/VP0CAPTURE/A/0";

    if (!totalFrameCnt)
    {
        LOG_printf(&trace, "Give non-zero frame count\n");
        return -1;
    }


    /* Allocate both capture and display frame buffers
       in external heap memory */
    vCapParamsChan.segId = sedId;
    vDisParamsChanRaw.segId = sedId;

    /* Assign EDMA3 driver handle to channel params */
    vCapParamsChan.hEdma = hEdma;
    vDisParamsChanRaw.hEdma = hEdma;

    /*
     * Assign number of frame buffers as 0 for using NEW FVID APIs
     */
    vCapParamsChan.numFrmBufs = 0;
    vDisParamsChanRaw.numFrmBufs = 0;

    /* Create and configure capture driver */
    capChInfo.chanHandle = FVID_create(vPortCapStrings,
                                       IOM_INPUT,
                                       &status,
                                       (Ptr)&vCapParamsChan,
                                       NULL);
    if (IOM_COMPLETED != status)
    {
        LOG_printf(&trace, "Failed to create capture channel\n");
    }


    /* Create and configure display driver for VP3 */
    if (IOM_COMPLETED == status)
    {
        disChInfoRaw.chanHandle = FVID_create(vPortDisStringsRaw,
                                              IOM_OUTPUT,
                                              &status,
                                              (Ptr)&vDisParamsChanRaw,
                                              NULL);
        if (IOM_COMPLETED != status)
        {
            LOG_printf(&trace, "Failed to create display channel\n");
        }
    }

    /* Allocate and Queue buffers for display and capture drivers */
    if (IOM_COMPLETED == status)
    {
        for (bufCount = 0; bufCount < NUM_FRAME_BUFFERS; bufCount++)
        {
            /* Allocate Frame buffer for capture driver */
            status = FVID_allocBuffer(capChInfo.chanHandle, &(capChInfo.frame));
            if (IOM_COMPLETED != status)
            {
                LOG_printf(&trace, "Failed to allocate buffer\n");
                break;
            }

            /* Queue the buffers */
            status = FVID_queue(capChInfo.chanHandle, &(capChInfo.frame));
            if (IOM_COMPLETED != status)
            {
                LOG_printf(&trace, "Failed to Queue buffer\n");
                break;
            }

            /* Allocate Frame buffer for display driver - VP3 */
            status = FVID_allocBuffer(disChInfoRaw.chanHandle,
                                      &(disChInfoRaw.frame));
            if (IOM_COMPLETED != status)
            {
                LOG_printf(&trace, "Failed to allocate buffer\n");
                break;
            }

            /* Queue the buffers */
            status = FVID_queue(disChInfoRaw.chanHandle, &(disChInfoRaw.frame));
            if (IOM_COMPLETED != status)
            {
                LOG_printf(&trace, "Failed to Queue buffer\n");
                break;
            }
        }
    }
    else
    {
        LOG_printf(&trace, "Failed to create - Display channel\n");
    }


    /* Configure external video decoder (TVP5154) */
    if (IOM_COMPLETED == status)
    {
        /* Only need to configure if the mode is PAL. Since NTSC is the default
           mode of operation for EDC driver */
#if !(CFG_VIDEO_MODE == MODE_NTSC)

        /* Configure TVP5154 */
        status |= FVID_control(capChInfo.chanHandle,
                               VPORT_CMD_EDC_BASE + EDC_CONFIG,
                               (Ptr)&vCapParamsDecoder);

#endif  /* #if !(CFG_VIDEO_MODE == MODE_NTSC) */
    }
    else
    {
        LOG_printf(&trace, "Failed to initialize capture/display drivers\n");
    }


    /* Start display and capture operations */
    if (IOM_COMPLETED == status)
    {
        status = FVID_control(disChInfoRaw.chanHandle,
                              VPORT_CMD_START,
                              NULL);

        status |= FVID_control(capChInfo.chanHandle,
                               VPORT_CMD_START,
                               NULL);

        /* Signal to start capture task */
        SEM_post(hSemStartCapture);
    }
    else
    {
        LOG_printf(&trace, "Failed to configure cap/disp channel device\n");
    }


    /* Request a frame buffer from driver */
    if (IOM_COMPLETED == status)
    {
        /* Display buffer will return a free buffer */
        status |= FVID_dequeue(disChInfoRaw.chanHandle, &(disChInfoRaw.frame));

        /* Capture buffer will return a filled buffer */
        status |= FVID_dequeue(capChInfo.chanHandle, &(capChInfo.frame));
    }
    else
    {
        LOG_printf(&trace, "Failed to start capture/display driver\n");
    }

    if (IOM_COMPLETED == status)
    {
        frames = 0;
        while (frames < totalFrameCnt)
        {
            /* Invalidate the buffer before giving to capture driver */
            BCACHE_inv((Uint8 *)capChInfo.frame->frame.iFrm.y1,
                       (FRAME_SIZE),
                       TRUE);

            /* Capture buffer will return the latest captured buffer */
            status |= FVID_exchange(capChInfo.chanHandle,
                                    &(capChInfo.frame));

#if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1)

            /* Fix for Video Port bigendian mode - for capture port */
            double_word_byte_swap((Uint8 *)capChInfo.frame->frame.iFrm.y1,
                                  FRAME_SIZE);

#endif  /* #if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1) */

            /* Copy the data from Capture buffer to VP1 display buffer */
            memcpy((Uint8 *)disChInfoRaw.frame->frame.rpFrm.buf,
                   (Uint8 *)capChInfo.frame->frame.iFrm.y1,
                   FRAME_SIZE);

#if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1)

            /* Fix for Video Port bigendian mode - This is for display port */
            double_word_byte_swap((Uint8 *)disChInfoRaw.frame->frame.rpFrm.buf,
                                  FRAME_SIZE);

#endif  /* #if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1) */

            /* Flush and invalidate the processed buffer so that the EDMA reads
               the processed data */
            BCACHE_wbInv((Uint8 *)disChInfoRaw.frame->frame.rpFrm.buf,
                         (FRAME_SIZE),
                         TRUE);

            /* Give the filled frame buffer to display driver and get a
               free frame buffer for next filling */
            status |= FVID_exchange(disChInfoRaw.chanHandle,
                                    &(disChInfoRaw.frame));

            /* LOG_printf(&trace, "%d\n", frames); */

            if (IOM_COMPLETED != status)
            {
                LOG_printf(&trace, "IOM_COMPLETED != status\n");
                break;
            }

            frames++;

            /* Update global frame count for check purpose */
            AppRawDisplayFrameCnt = frames;
        }/* while (frame count) */


        /* Stop display/capture operation */
        if (IOM_COMPLETED == status)
        {
            /* Wait for capture task to stop capturing */
            SEM_pend(hSemStopDisplay, SYS_FOREVER);

            status |= FVID_control(capChInfo.chanHandle,
                                   VPORT_CMD_STOP,
                                   NULL);

            status |= FVID_control(disChInfoRaw.chanHandle,
                                   VPORT_CMD_STOP,
                                   NULL);

            /* Free the buffer owned by application */
            status |= FVID_freeBuffer(capChInfo.chanHandle,
                                      &(capChInfo.frame));

            /* Free the buffer owned by application */
            status |= FVID_freeBuffer(disChInfoRaw.chanHandle,
                                      &(disChInfoRaw.frame));

            /* Dequeue buffers from driver and free them */
            for (bufCount = 0; bufCount < (NUM_FRAME_BUFFERS - 1); bufCount++)
            {
                status |= FVID_dequeue(capChInfo.chanHandle,
                                       &(capChInfo.frame));
                status |= FVID_freeBuffer(capChInfo.chanHandle,
                                          &(capChInfo.frame));

                status |= FVID_dequeue(disChInfoRaw.chanHandle,
                                       &(disChInfoRaw.frame));
                status |= FVID_freeBuffer(disChInfoRaw.chanHandle,
                                          &(disChInfoRaw.frame));
            }
        }
        else
        {
            LOG_printf(&trace, "Failed to alloc or exchange buffers\n");
        }


        /* Delete display and capture drivers */
        if (IOM_COMPLETED == status)
        {
            status |= FVID_delete(capChInfo.chanHandle);
            status |= FVID_delete(disChInfoRaw.chanHandle);
            if (IOM_COMPLETED == status)
            {
                LOG_printf(&trace, "Application closed successfully!!\n");
            }
        }
        else
        {
            LOG_printf(&trace, "Failed to stop display/capture channel\n");
        }
    }
    else if (IOM_COMPLETED != status)
    {
        LOG_printf(&trace, "Failed to dequeue buffers\n");
    }

    return status;
}



/*
 * ======== test_raw_capture ========
 * RAW Capture function
 * Function which performs VP4 Raw capture and displays the frame buffer
 * in SD resolution in VP1.
 *
 * hEdma            - Handle to the EDMA3 driver
 * totalFrameCnt    - Number of frames for capture/display operation
 * sedId            - Heap ID where the driver FVID buffers will be allocated
 */
static Int32 test_raw_capture(EDMA3_DRV_Handle hEdma,
                              Uint32 totalFrameCnt,
                              Int32 sedId)
{
    Int         status = IOM_COMPLETED;
    Int         frames, bufCount;
    /* Display and capture channel information */
    ChannelInfo capChInfoRaw;
    ChannelInfo disChInfo;
    /* Display and capture driver name strings
       Use this strings as they are for proper driver creation */
    Int8        *vPortCapStringsRaw = "/VP4CAPTURE/A";
    Int8        *vPortDisStrings = "/VP1DISPLAY/SAA7105";

    if (!totalFrameCnt)
    {
        LOG_printf(&trace, "Give non-zero frame count\n");
        return -1;
    }


    /* Allocate both capture and display frame buffers
       in external heap memory */
    vCapParamsChanRaw.segId = sedId;
    vDisParamsChan.segId = sedId;

    /* Assign EDMA3 driver handle to channel params */
    vCapParamsChanRaw.hEdma = hEdma;
    vDisParamsChan.hEdma = hEdma;

    /*
     * Assign number of frame buffers as 0 for using NEW FVID APIs
     */
    vCapParamsChanRaw.numFrmBufs = 0;
    vDisParamsChan.numFrmBufs = 0;

    /* Create and configure capture driver */
    capChInfoRaw.chanHandle = FVID_create(vPortCapStringsRaw,
                                          IOM_INPUT,
                                          &status,
                                          (Ptr)&vCapParamsChanRaw,
                                          NULL);
    if (IOM_COMPLETED != status)
    {
        LOG_printf(&trace, "Failed to create capture channel\n");
    }

    /* Create and configure display driver for VP1 */
    if (IOM_COMPLETED == status)
    {
        disChInfo.chanHandle = FVID_create(vPortDisStrings,
                                           IOM_OUTPUT,
                                           &status,
                                           (Ptr)&vDisParamsChan,
                                           NULL);
        if (IOM_COMPLETED != status)
        {
            LOG_printf(&trace, "Failed to create display channel\n");
        }
    }

    /* Allocate and Queue buffers for display and capture drivers */
    if (IOM_COMPLETED == status)
    {
        for (bufCount = 0; bufCount < NUM_FRAME_BUFFERS; bufCount++)
        {
            /* Allocate Frame buffer for capture driver */
            status = FVID_allocBuffer(capChInfoRaw.chanHandle,
                                      &(capChInfoRaw.frame));
            if (IOM_COMPLETED != status)
            {
                LOG_printf(&trace, "Failed to allocate buffer\n");
                break;
            }

            /* Queue the buffers */
            status = FVID_queue(capChInfoRaw.chanHandle, &(capChInfoRaw.frame));
            if (IOM_COMPLETED != status)
            {
                LOG_printf(&trace, "Failed to Queue buffer\n");
                break;
            }

            /* Allocate Frame buffer for display driver - VP1 */
            status = FVID_allocBuffer(disChInfo.chanHandle, &(disChInfo.frame));
            if (IOM_COMPLETED != status)
            {
                LOG_printf(&trace, "Failed to allocate buffer\n");
                break;
            }

            /* Queue the buffers */
            status = FVID_queue(disChInfo.chanHandle, &(disChInfo.frame));
            if (IOM_COMPLETED != status)
            {
                LOG_printf(&trace, "Failed to Queue buffer\n");
                break;
            }
        }
    }
    else
    {
        LOG_printf(&trace, "Failed to create - Display channel\n");
    }


    /* Configure external video encoder (SAA7105) */
    if (IOM_COMPLETED == status)
    {
        /* Only need to configure if the mode is PAL. Since NTSC is the default
           mode of operation for EDC driver */
#if !(CFG_VIDEO_MODE == MODE_NTSC)

        /* Configure SAA7105 */
        status |= FVID_control(disChInfo.chanHandle,
                               VPORT_CMD_EDC_BASE + EDC_CONFIG,
                               (Ptr)&vDisParamsEncoder);

#endif  /* #if !(CFG_VIDEO_MODE == MODE_NTSC) */
    }
    else
    {
        LOG_printf(&trace, "Failed to initialize capture/display drivers\n");
    }

    /* Start display and capture operations */
    if (IOM_COMPLETED == status)
    {
        status = FVID_control(disChInfo.chanHandle,
                              VPORT_CMD_START,
                              NULL);

        /* Wait for display task to start display */
        SEM_pend(hSemStartCapture, SYS_FOREVER);

        status |= FVID_control(capChInfoRaw.chanHandle,
                               VPORT_CMD_START,
                               NULL);
    }
    else
    {
        LOG_printf(&trace, "Failed to configure cap/disp channel device\n");
    }


    /* Request a frame buffer from driver */
    if (IOM_COMPLETED == status)
    {
        /* Capture buffer will return a filled buffer */
        status |= FVID_dequeue(capChInfoRaw.chanHandle, &(capChInfoRaw.frame));

        /* Display buffer will return a free buffer */
        status |= FVID_dequeue(disChInfo.chanHandle, &(disChInfo.frame));
    }
    else
    {
        LOG_printf(&trace, "Failed to start capture/display driver\n");
    }

    if (IOM_COMPLETED == status)
    {
        frames = 0;
        while (frames < totalFrameCnt)
        {
            /* Invalidate the buffer before giving to capture driver */
            BCACHE_inv((Uint8 *)capChInfoRaw.frame->frame.rpFrm.buf,
                       (FRAME_SIZE),
                       TRUE);

            /* Capture buffer will return the latest captured buffer */
            status |= FVID_exchange(capChInfoRaw.chanHandle,
                                    &(capChInfoRaw.frame));

#if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1)

            /* Fix for Video Port bigendian mode - for capture port */
            double_word_byte_swap((Uint8 *)capChInfoRaw.frame->frame.rpFrm.buf,
                                  FRAME_SIZE);

#endif  /* #if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1) */

            /* Copy the data from Capture buffer to VP1 display buffer */
            memcpy((Uint8 *)disChInfo.frame->frame.iFrm.y1,
                   (Uint8 *)capChInfoRaw.frame->frame.rpFrm.buf,
                   FRAME_SIZE);

#if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1)

            /* Fix for Video Port bigendian mode - for display port */
            double_word_byte_swap(
                        (Uint8 *)disChInfo.frame->frame.iFrm.y1,
                        FRAME_SIZE);

#endif  /* #if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1) */

            /* Flush and invalidate the processed buffer so that the EDMA reads
               the processed data */
            BCACHE_wbInv((Uint8 *)disChInfo.frame->frame.iFrm.y1,
                         (FRAME_SIZE),
                         TRUE);

            /* Give the filled frame buffer to display driver and get a
               free frame buffer for next filling */
            status |= FVID_exchange(disChInfo.chanHandle,
                                    &(disChInfo.frame));

            /* LOG_printf(&trace, "%d\n", frames); */

            if (IOM_COMPLETED != status)
            {
                LOG_printf(&trace, "IOM_COMPLETED != status\n");
                break;
            }

            frames++;

            /* Update global frame count for check purpose */
            AppRawCaptureFrameCnt = frames;
        }/* while (frame count) */


        /* Stop display/capture operation */
        if (IOM_COMPLETED == status)
        {
            status |= FVID_control(disChInfo.chanHandle,
                                   VPORT_CMD_STOP,
                                   NULL);

            status |= FVID_control(capChInfoRaw.chanHandle,
                                   VPORT_CMD_STOP,
                                   NULL);

            /* Signal to stop display task */
            SEM_post(hSemStopDisplay);

            /* Free the buffer owned by application */
            status |= FVID_freeBuffer(disChInfo.chanHandle,
                                      &(disChInfo.frame));

            /* Free the buffer owned by application */
            status |= FVID_freeBuffer(capChInfoRaw.chanHandle,
                                      &(capChInfoRaw.frame));

            /* Dequeue buffers from driver and free them */
            for (bufCount = 0; bufCount < (NUM_FRAME_BUFFERS - 1); bufCount++)
            {
                status |= FVID_dequeue(disChInfo.chanHandle,
                                       &(disChInfo.frame));
                status |= FVID_freeBuffer(disChInfo.chanHandle,
                                          &(disChInfo.frame));

                status |= FVID_dequeue(capChInfoRaw.chanHandle,
                                       &(capChInfoRaw.frame));
                status |= FVID_freeBuffer(capChInfoRaw.chanHandle,
                                          &(capChInfoRaw.frame));
            }
        }
        else
        {
            LOG_printf(&trace, "Failed to alloc or exchange buffers\n");
        }


        /* Delete display and capture drivers */
        if (IOM_COMPLETED == status)
        {
            status |= FVID_delete(disChInfo.chanHandle);
            status |= FVID_delete(capChInfoRaw.chanHandle);
            if (IOM_COMPLETED == status)
            {
                LOG_printf(&trace, "Application closed successfully!!\n");
            }
        }
        else
        {
            LOG_printf(&trace, "Failed to stop display/capture channel\n");
        }
    }
    else if (IOM_COMPLETED != status)
    {
        LOG_printf(&trace, "Failed to dequeue buffers\n");
    }

    return status;
}


/******************************************************************************/
/* End of file                                                                */
/******************************************************************************/
