/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/** \file   psp_bios_vport_sd_compositor.c
 *
 *  \brief  This sample application captures 1 to 4 NTSC/PAL video, makes a
 *          single frame and displays it in NTSC/PAL. This application
 *          uses the NEW FVID APIs.
 *
 */

/*
 * ========== INCLUDE FILES ===========
 */
#include <std.h>
#include <bcache.h>
#include <log.h>

#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/fvid.h>
#include <ti/sdo/pspdrivers/drivers/vport/edc.h>
#include <ti/sdo/pspdrivers/drivers/vport/vport.h>
#include <ti/sdo/pspdrivers/drivers/vport/vportdis.h>
#include <ti/sdo/pspdrivers/drivers/vport/vportcap.h>

#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/sample/psp_bios_vport_common.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/sample/psp_bios_vport_capParams.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/sample/psp_bios_vport_disParams.h>


/*
 * ======== USER CONFIGURABLE MACROS ========
 */

/*
 * Video Mode
 *
 * Set this macro to MODE_NTSC or MODE_PAL for NTSC/PAL loopback
 * MODE_NTSC - To capture and display in NTSC format
 * MODE_PAL  - To capture and display in PAL format
 * Others video modes are not supported
 *
 */
#define CFG_VIDEO_MODE                      (MODE_NTSC)

/*
 * Number of channels to capture
 *
 * Set this macro to 1 to 4, to capture the required number of channels.
 *
 */
#define CFG_NUM_CAP_CHANNEL                 (4)

/*
 * Number of frames for video loop back
 *
 * This sample application will run for 'CFG_NUM_FRAME_COUNT' amount of frames.
 * After which the application will close. With the current value of 100000
 * frames, the sample application will run for 55 mins of NTSC video or 66 mins
 * of PAL video.
 */
#define CFG_NUM_FRAME_COUNT                 (100000u)

/*
 * Enable or disable double word byte swap function
 *
 * Set this macro as 1/0 to enable or disable double word byte swap software
 * work around for Video Port endian issue
 *
 * 1    - Enable double word byte swap
 * 0    - Disable double word byte swap
 *
 */
#define CFG_ENABLE_DOUBLEWORD_BYTE_SWAP     (1)


/*
 * ======== DEFINES =========
 */

/* Number of frame buffers to allocate per driver instance */
#define NUM_FRAME_BUFFERS                   (3)

/* Capture driver mode of operation */
#define CAPTURE_MODE                        (CFG_VIDEO_MODE)


#if (CAPTURE_MODE == MODE_NTSC)

/* Number of Horizontal Pixels - Use square pixel so that Y, Cb and Cr buffers
   are aligned on 8 byte boundaries when scaling is enabled */
#define CAP_NUMPIXELS                       (704)
#define CAP_NUMLINES                        (480)
#define DISPLAY_MODE                        (MODE_NTSC)
#define DIS_NUMPIXELS                       (704)
#define DIS_NUMLINES                        (480)

#if (CFG_NUM_CAP_CHANNEL <= 1)

#define ENABLE_SCALING                      (0)

#else   /* #if (CFG_NUM_CAP_CHANNEL <= 1) */

#define ENABLE_SCALING                      (1)

#endif  /* #if (CFG_NUM_CAP_CHANNEL <= 1) */

#endif  /* #if (CAPTURE_MODE == MODE_NTSC) */


#if (CAPTURE_MODE == MODE_PAL)

/* Number of Horizontal Pixels - Use square pixel so that Y, Cb and Cr buffers
   are aligned on 8 byte boundaries when scaling is enabled */
#define CAP_NUMPIXELS                       (704)
#define CAP_NUMLINES                        (576)
#define DISPLAY_MODE                        (MODE_PAL)
#define DIS_NUMPIXELS                       (704)
#define DIS_NUMLINES                        (576)

#if (CFG_NUM_CAP_CHANNEL <= 1)

#define ENABLE_SCALING                      (0)

#else   /* #if (CFG_NUM_CAP_CHANNEL <= 1) */

#define ENABLE_SCALING                      (1)

#endif  /* #if (CFG_NUM_CAP_CHANNEL <= 1) */

#endif  /* #if (CAPTURE_MODE == MODE_PAL) */


#if (ENABLE_SCALING == 1)

#define CAP_FRAME_SIZE              ((CAP_NUMPIXELS / 2) * CAP_NUMLINES * 2)
#define DIS_FRAME_SIZE              (DIS_NUMPIXELS * DIS_NUMLINES * 2)

#else   /* #if (ENABLE_SCALING == 1) */

#define CAP_FRAME_SIZE              (CAP_NUMPIXELS * CAP_NUMLINES * 2)
#define DIS_FRAME_SIZE              (DIS_NUMPIXELS * DIS_NUMLINES * 2)

#endif  /* #if (ENABLE_SCALING == 1) */



/*
 * ======== GLOBALS =========
 */

/* Video Port configuration parameters for SDTV Capture.
   Using default value and enabling dual channel.
   Note: This variable is used in BIOS TCF file. */
VPORT_PortParams vCapParamsPort = CAP_PARAMS_PORT_DEFAULT(TRUE);

/* Video Port configuration parameters for HDTV display.
   Using default value.
   Note: This variable is used in BIOS TCF file. */
VPORT_PortParams vDisParamsPort = DIS_PARAMS_PORT_DEFAULT;

/* Global variable to check the current frame count in CCS watch window */
Uint32 AppSdCompositorFrameCnt = 0;


#if (CAPTURE_MODE == MODE_NTSC)

/* Vport Capture driver configuration.
   Using embedded sync. */
static VPORTCAP_Params vCapParamsChan = CAP_PARAMS_CHAN_BT656_DEFAULT(NTSC);

#endif  /* #if (CAPTURE_MODE == MODE_NTSC) */


#if (CAPTURE_MODE == MODE_PAL)

/* Vport Capture driver configuration.
   Using embedded sync */
static VPORTCAP_Params vCapParamsChan = CAP_PARAMS_CHAN_BT656_DEFAULT(PAL);

/* TVP5154 EDC driver configuration.
   Using embedded sync and Composite Analog Channel A for all 8 decoders */
static TVP5154_ConfParams vCapParamsDecoder
    = CAP_PARAMS_TVP5154_EMBEDDED_DEFAULT(PAL, COMPOSITE_A);

#endif  /* #if (CAPTURE_MODE == MODE_PAL) */


#if (DISPLAY_MODE == MODE_NTSC)

/* Vport Display driver configuration.
   Using embedded sync */
static VPORTDIS_Params vDisParamsChan = DIS_PARAMS_CHAN_BT656_DEFAULT(NTSC);

#endif  /* (DISPLAY_MODE == MODE_NTSC) */

#if (DISPLAY_MODE == MODE_PAL)

/* Vport Display driver configuration.
   Using embedded sync */
static VPORTDIS_Params vDisParamsChan = DIS_PARAMS_CHAN_BT656_DEFAULT(PAL);

/* SAA7105 EDC driver configuration.
   Using Svideo Analog output */
static SAA7105_ConfParams vDisParamsEncoder
    = DIS_PARAMS_SAA7105_SDTV_EMBEDDED_DEFAULT(PAL720, SVIDEO);

#endif  /* (DISPLAY_MODE == MODE_PAL) */


/*
 * ============= FUNC DECLARATIONS ==============
 */

#if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1)

/* Function which does the 8-byte swapping - Software fix for big endian mode
   operation of Video Port */
extern void double_word_byte_swap(Uint8 *bufferPtr, Uint32 numOfBytes);

#endif  /* #if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1) */

extern EDMA3_DRV_Result edma3init(void);

static Uint32 test_video_compositor(EDMA3_DRV_Handle hEdma,
                                    Uint32 numCapChannel,
                                    Uint32 totalFrameCnt,
                                    Int32 sedId);

static Int32 video_compositor(ChannelInfo *outChInfo,
                              ChannelInfo *inChInfo,
                              Uint32 numOfInChannels,
                              Uint32 outNumPixels,
                              Uint32 outNumLines,
                              Uint32 inNumPixels,
                              Uint32 inNumLines);


/*
 * ============= EXTERNS ==============
 */
#ifdef _6x_
extern far LOG_Obj trace;
#else
extern LOG_Obj trace;
#endif

/* Heap ID defined in the BIOS configuration file.
   This memory segment is used to allocate video frame buffers. */
extern Int EXTERNALHEAP;

/* EDMA handle */
extern EDMA3_DRV_Handle hEdma;


/*
 * ============= DEFINITIONS ==============
 */


/*
 * ======== tskSdCompositor ========
 * Video application test task.
 * Created as a separate task in BIOS TCF file.
 */
void tskSdCompositor(void)
{
    EDMA3_DRV_Result result = EDMA3_DRV_SOK;

    /* Initialize EDMA3 driver */
    result = edma3init();
    if (EDMA3_DRV_SOK != result)
    {
        LOG_printf(&trace, "Can't initialized EDMA3");
        return;
    }

    /* Run Video Compositor Test */
    test_video_compositor(hEdma,
                          CFG_NUM_CAP_CHANNEL,
                          CFG_NUM_FRAME_COUNT,
                          EXTERNALHEAP);
}



/*
 * ======== test_video_compositor ========
 * Video compositor function
 * Function which performs the NTSC/PAL to SD compositor function
 * hEdma            - Handle to the EDMA3 driver
 * numCapChannel    - Total number of channel to capture.
 *                    This can be 1 to 4.
 * totalFrameCnt    - Number of frames for loop back operation
 * sedId            - Heap ID where the driver FVID buffers will be allocated
 */
static Uint32 test_video_compositor(EDMA3_DRV_Handle hEdma,
                                    Uint32 numCapChannel,
                                    Uint32 totalFrameCnt,
                                    Int32 sedId)
{
    Int             status = IOM_COMPLETED;
    Int             frames;
    Int             count, bufCount;
    /* Array containing display and capture channel information */
    ChannelInfo     disChInfo;
    ChannelInfo     capChInfo[MAX_CAP_CHAN];
    /* Array containing display and capture driver name strings
       Use this strings as they are for proper driver creation */
    Int8            *vPortCapStrings[MAX_CAP_CHAN] =
    {
        "/VP0CAPTURE/A/0",
        "/VP0CAPTURE/B/1",
        "/VP2CAPTURE/A/2",
        "/VP2CAPTURE/B/3",
        "/VP3CAPTURE/A/4",
        "/VP3CAPTURE/B/5",
        "/VP4CAPTURE/A/6",
        "/VP4CAPTURE/B/7"
    };
    Int8            *vPortDisStrings = "/VP1DISPLAY/SAA7105";
    /* Total number of Capture channel to capture (user configurable)-1 to 8 */
    Int             numCompositorChannel;
    Uint32          capNumPixels;

    if ((!numCapChannel) || (numCapChannel > 4))
    {
        LOG_printf(&trace,
                   "Give number of capture channel between 1 and %d\n",
                   4);
        return 1;
    }

    if (!totalFrameCnt)
    {
        LOG_printf(&trace, "Give non-zero frame count\n");
        return 1;
    }

    numCompositorChannel = numCapChannel;

    /* Allocate both capture and display frame buffers
     * in external heap memory
     */
    vCapParamsChan.segId = sedId;
    vDisParamsChan.segId = sedId;

    /* Assign EDMA3 driver handle to channel params */
    vCapParamsChan.hEdma = hEdma;
    vDisParamsChan.hEdma = hEdma;

    /*
     * Assign number of frame buffers as 0 for using NEW FVID APIs
     */
    vCapParamsChan.numFrmBufs = 0;
    vDisParamsChan.numFrmBufs = 0;

    /* Enable or disable horizontal scaling */
#if (ENABLE_SCALING == 1)

    vCapParamsChan.scale = VPORT_SCALING_ENABLE;
    capNumPixels = CAP_NUMPIXELS >> 1;
    vCapParamsChan.thrld = CAP_NUMPIXELS >> 4;

#else

    vCapParamsChan.scale = VPORT_SCALING_DISABLE;
    capNumPixels = CAP_NUMPIXELS;
    vCapParamsChan.thrld = CAP_NUMPIXELS >> 3;

#endif  /* #if (ENABLE_SCALING == 1) */

    vDisParamsChan.scale = VPORT_SCALING_DISABLE;
    vDisParamsChan.thrld = DIS_NUMPIXELS >> 3;
    vDisParamsChan.imgHOffsetFld1 = (DIS_NTSC_LINE_SZ - DIS_NUMPIXELS) / 2;
    vDisParamsChan.imgHOffsetFld2 = (DIS_NTSC_LINE_SZ - DIS_NUMPIXELS) / 2;
    vDisParamsChan.imgHSizeFld1 = DIS_NUMPIXELS;
    vDisParamsChan.imgHSizeFld2 = DIS_NUMPIXELS;

#if (CAPTURE_MODE == MODE_NTSC)

    vCapParamsChan.fldXStrt1 = (CAP_NTSC_LINE_SZ - CAP_NUMPIXELS) / 2;
    vCapParamsChan.fldXStrt2 = (CAP_NTSC_LINE_SZ - CAP_NUMPIXELS) / 2;
    vCapParamsChan.fldXStop1 = CAP_NTSC_LINE_SZ - 1 -
                               (CAP_NTSC_LINE_SZ - CAP_NUMPIXELS) / 2;
    vCapParamsChan.fldXStop2 = CAP_NTSC_LINE_SZ - 1 -
                               (CAP_NTSC_LINE_SZ - CAP_NUMPIXELS) / 2;

#endif  /* #if (CAPTURE_MODE == MODE_NTSC) */

#if (CAPTURE_MODE == MODE_PAL)

    vCapParamsChan.fldXStrt1 = (CAP_PAL_LINE_SZ - CAP_NUMPIXELS) / 2;
    vCapParamsChan.fldXStrt2 = (CAP_PAL_LINE_SZ - CAP_NUMPIXELS) / 2;
    vCapParamsChan.fldXStop1 = CAP_PAL_LINE_SZ - 1 -
                               (CAP_PAL_LINE_SZ - CAP_NUMPIXELS) / 2;
    vCapParamsChan.fldXStop2 = CAP_PAL_LINE_SZ - 1 -
                               (CAP_PAL_LINE_SZ - CAP_NUMPIXELS) / 2;

#endif  /* #if (CAPTURE_MODE == MODE_PAL) */


    /* Create and configure capture drivers */
    for (count = 0; count < numCapChannel; count++)
    {
        capChInfo[count].chanHandle = FVID_create(vPortCapStrings[count],
                                                  IOM_INPUT,
                                                  &status,
                                                  (Ptr)&vCapParamsChan,
                                                  NULL);
        if (IOM_COMPLETED != status)
        {
            LOG_printf(&trace, "Failed to create capture channel %d\n", count);
            break;
        }

        /* Allocate and Queue buffers for capture driver */
        for (bufCount = 0; bufCount < NUM_FRAME_BUFFERS; bufCount++)
        {
            /* Allocate Frame buffer for capture driver */
            status = FVID_allocBuffer(capChInfo[count].chanHandle,
                                      &(capChInfo[count].frame));
            if (IOM_COMPLETED != status)
            {
                LOG_printf(&trace, "Failed to allocate buffer\n");
                break;
            }

            /* Queue the buffers */
            status = FVID_queue(capChInfo[count].chanHandle,
                                      &(capChInfo[count].frame));
            if (IOM_COMPLETED != status)
            {
                LOG_printf(&trace, "Failed to Queue buffer\n");
                break;
            }
        }

        if (IOM_COMPLETED != status)
        {
            break;
        }
    }


    /* Create and configure display driver */
    if (IOM_COMPLETED == status)
    {
        disChInfo.chanHandle = FVID_create(vPortDisStrings,
                                           IOM_OUTPUT,
                                           &status,
                                           (Ptr)&vDisParamsChan,
                                           NULL);
        if (IOM_COMPLETED != status)
        {
            LOG_printf(&trace, "Failed to create display channels\n");
        }

        /* Allocate and Queue buffers for display driver */
        for (bufCount = 0; bufCount < NUM_FRAME_BUFFERS; bufCount++)
        {
            /* Allocate Frame buffer for display driver */
            status = FVID_allocBuffer(disChInfo.chanHandle, &(disChInfo.frame));
            if (IOM_COMPLETED != status)
            {
                LOG_printf(&trace, "Failed to allocate buffer\n");
                break;
            }

            /* Queue the buffers */
            status = FVID_queue(disChInfo.chanHandle, &(disChInfo.frame));
            if (IOM_COMPLETED != status)
            {
                LOG_printf(&trace, "Failed to Queue buffer\n");
                break;
            }
        }
    }


    /* Configure external video encoders (SAA7105) & decoders (TVP5154) */
    if (IOM_COMPLETED == status)
    {
        /* Only need to configure if the mode is not NTSC. Since NTSC is the
           default mode of operation for EDC driver in HD */
#if !(DISPLAY_MODE == MODE_NTSC)

        /* Configure SAA7105 */
        status |= FVID_control(disChInfo.chanHandle,
                               VPORT_CMD_EDC_BASE + EDC_CONFIG,
                               (Ptr)&vDisParamsEncoder);

#endif  /* #if !(DISPLAY_MODE == MODE_NTSC) */

        /* Only need to configure if the mode is PAL. Since NTSC is the default
           mode of operation for EDC driver */
#if !(CAPTURE_MODE == MODE_NTSC)

        for (count = 0; count < numCapChannel; count++)
        {
            /* Configure TVP5154 */
            status |= FVID_control(capChInfo[count].chanHandle,
                                   VPORT_CMD_EDC_BASE + EDC_CONFIG,
                                   (Ptr)&vCapParamsDecoder);
        }

#endif  /* #if !(CAPTURE_MODE == MODE_NTSC) */
    }
    else
    {
        LOG_printf(&trace, "Failed to open - Display channel\n");
    }


    /* Start display and capture operations */
    if (IOM_COMPLETED == status)
    {
        status |= FVID_control(disChInfo.chanHandle, VPORT_CMD_START, NULL);

        for (count = 0; count < numCapChannel; count++)
        {
            status |= FVID_control(capChInfo[count].chanHandle,
                                   VPORT_CMD_START,
                                   NULL);
        }
    }
    else
    {
        LOG_printf(&trace, "Failed to configure cap/disp channel device\n");
    }


    /* Request a frame buffer from display & capture driver */
    if (IOM_COMPLETED == status)
    {
        /* Display buffer will return a free buffer */
        status |= FVID_dequeue(disChInfo.chanHandle, &(disChInfo.frame));

        for (count = 0; count < numCapChannel; count++)
        {
            /* Capture buffer will return the latest captured buffer */
            status |= FVID_dequeue(capChInfo[count].chanHandle,
                                   &(capChInfo[count].frame));
        }
    }
    else
    {
        LOG_printf(&trace, "Failed to start display/capture driver\n");
    }


    if (IOM_COMPLETED == status)
    {
        frames = 0;
        while (frames < totalFrameCnt)
        {
            /* Give the old capture frame buffer back to driver and get the
               recently captured frame buffer */
            for (count = 0; count < numCapChannel; count++)
            {
                /* Invalidate the buffer before giving to capture driver */
                BCACHE_inv((Uint8 *)capChInfo[count].frame->frame.iFrm.y1,
                           (CAP_FRAME_SIZE),
                           TRUE);

                /* Capture buffer will return the latest captured buffer */
                status |= FVID_exchange(capChInfo[count].chanHandle,
                                       &(capChInfo[count].frame));


#if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1)

                /* Fix for Video Port bigendian mode - for capture port */
                double_word_byte_swap(
                            (Uint8 *)capChInfo[count].frame->frame.iFrm.y1,
                            CAP_FRAME_SIZE);

#endif  /* #if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1) */
            }

            video_compositor(&disChInfo,
                             &capChInfo[0],
                             numCompositorChannel,      /* numOfInChannels */
                             DIS_NUMPIXELS,             /* outNumPixels */
                             DIS_NUMLINES,              /* outNumLines */
                             capNumPixels,              /* inNumPixels */
                             CAP_NUMLINES);             /* inNumLines */

#if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1)

            /* Fix for Video Port bigendian mode - This is for display port */
            double_word_byte_swap((Uint8 *)disChInfo.frame->frame.iFrm.y1,
                                  DIS_FRAME_SIZE);

#endif  /* #if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1) */

            /* Flush and invalidate the processed buffer so that the EDMA reads
               the processed data */
            BCACHE_wbInv((Uint8 *)disChInfo.frame->frame.iFrm.y1,
                         DIS_FRAME_SIZE,
                         TRUE);

            status |= FVID_exchange(disChInfo.chanHandle, &(disChInfo.frame));

            /* LOG_printf(&trace, "%d\n", frames); */

            if (IOM_COMPLETED != status)
            {
                LOG_printf(&trace, "IOM_COMPLETED != status\n");
                break;
            }

            frames++;

            /* Update global frame count for check purpose */
            AppSdCompositorFrameCnt = frames;
        }/* while (frame count) */


        /* Stop capture and display operation */
        if (IOM_COMPLETED == status)
        {
            status |= FVID_control(disChInfo.chanHandle, VPORT_CMD_STOP, NULL);

            /* Free the buffer owned by application */
            status |= FVID_freeBuffer(disChInfo.chanHandle,
                                      &(disChInfo.frame));

            /* Dequeue buffers from driver and free them */
            for (bufCount = 0; bufCount < (NUM_FRAME_BUFFERS - 1); bufCount++)
            {
                status |= FVID_dequeue(disChInfo.chanHandle,
                                       &(disChInfo.frame));
                status |= FVID_freeBuffer(disChInfo.chanHandle,
                                          &(disChInfo.frame));
            }

            for (count = 0; count < numCapChannel; count++)
            {
                status |= FVID_control(capChInfo[count].chanHandle,
                                       VPORT_CMD_STOP,
                                       NULL);

                status |= FVID_freeBuffer(capChInfo[count].chanHandle,
                                          &(capChInfo[count].frame));

                /* Dequeue buffers from driver and free them */
                for (bufCount = 0; bufCount < (NUM_FRAME_BUFFERS - 1); bufCount++)
                {
                    status |= FVID_dequeue(capChInfo[count].chanHandle,
                                           &(capChInfo[count].frame));
                    status |= FVID_freeBuffer(capChInfo[count].chanHandle,
                                              &(capChInfo[count].frame));
                }
            }
        }
        else
        {
            LOG_printf(&trace, "Failed to alloc or exchange buffers\n");
        }


        /* Delete capture and display drivers */
        if (IOM_COMPLETED == status)
        {
            status |= FVID_delete(disChInfo.chanHandle);

            for (count = 0; count < numCapChannel; count++)
            {
                status |= FVID_delete(capChInfo[count].chanHandle);
                if (IOM_COMPLETED != status)
                {
                    LOG_printf(&trace,
                               "Failed to delete capture channel %d\n",
                               count);
                }
            }

            if (IOM_COMPLETED == status)
            {
                LOG_printf(&trace, "Application closed successfully!!\n");
            }
        }
        else
        {
            LOG_printf(&trace, "Failed to stop display/capture channel\n");
        }
    }
    else if (IOM_COMPLETED != status)
    {
        LOG_printf(&trace, "Failed to alloc or exchange buffers\n");
    }

    return status;
}



/*
 * ======== frameBufferCopy ========
 * Copies a FVID frame data to another FVID frame data
 * outFrame  - Destination frame buffer pointer
 * inFrame   - Source frame buffer pointer
 * numPixels - Number of horizontal pixels to copy (in bytes)
 * numLines  - Number of lines to copy
 * outPitch  - Horizontal pitch of output frame buffer
 * inPitch   - Horizontal pitch of input frame buffer
 */
static Int32 frameBufferCopy(Int8 *outFrame,
                             Int8 *inFrame,
                             Uint32 numPixels,
                             Uint32 numLines,
                             Uint32 outPitch,
                             Uint32 inPitch)
{
    Uint32 lines;

    for (lines = 0; lines < numLines; lines++)
    {
        memcpy(outFrame, inFrame, numPixels);
        outFrame += outPitch;
        inFrame += inPitch;
    }

    return 0;
}



/*
 * ======== video_compositor ========
 * Video compositor Function
 * outChInfo       - Display channel information containing FVID buffer
 * inChInfo        - Capture channel information containing FVID buffer
 * numOfInChannels - Number of capture channel
 * outNumPixels    - Display horizontal pixels
 * outNumLines     - Display number of lines
 * inNumPixels     - Capture horizontal pixels
 * inNumLines      - Capture number of lines
 */
static Int32 video_compositor(ChannelInfo *outChInfo,
                              ChannelInfo *inChInfo,
                              Uint32 numOfInChannels,
                              Uint32 outNumPixels,
                              Uint32 outNumLines,
                              Uint32 inNumPixels,
                              Uint32 inNumLines)
{
    Uint32  vOffset, hOffset;
    Uint32  numCol, numRow, col, row;
    Int8    *tempPtr;
    Uint32  inYPitch;

    switch (numOfInChannels)
    {
        case 1:
            numRow = 1;
            numCol = 1;
            break;

        case 2:
            numRow = 1;
            numCol = 2;
            break;

        case 3:
            numRow = 2;
            numCol = 2;
            break;

        case 4:
            numRow = 2;
            numCol = 2;
            break;

        default:
            return -1;
    }

#if (ENABLE_SCALING == 1)

    inNumLines >>= 1;           /* Assuming vertical scaling is not done */
    inYPitch = inNumPixels * 2; /* Assuming vertical scaling is not done */

#else

    inYPitch = inNumPixels;     /* Assuming vertical scaling is not done */

#endif  /* #if (ENABLE_SCALING == 1) */

    /* Check if merging can be done */
    /* Check horizontal pixels are sufficient */
    if ((inNumPixels * numCol) > outNumPixels)
    {
        return -1;
    }

    /* Check vertical lines are sufficient */
    if ((inNumLines * numRow) > outNumLines)
    {
        return -1;
    }

    vOffset = (outNumLines - (inNumLines * numRow)) / 2;
    hOffset = (outNumPixels - (inNumPixels * numCol)) / 2;
    /* Make horizontal offset multiple of 8 to avoid non 8 byte boundary buffer
       copy. Since Cb/Cr offset should also be multiple of 8, make this multiple
       of 16 so that Cb/Cr buffer copy becomes multiple of 8 */
    hOffset &= ~0x0F;

    for (row = 0; row < numRow; row++)
    {
        for (col = 0; col < numCol; col++)
        {
            if (0 == numOfInChannels)
            {
                break;
            }

            numOfInChannels--;

            /* Copy Y frame data */
            tempPtr = (outChInfo->frame->frame.iFrm.y1);
            tempPtr += ((vOffset + (row * inNumLines)) * outNumPixels) +
                       hOffset + (col * inNumPixels);
            frameBufferCopy(tempPtr,
                            inChInfo->frame->frame.iFrm.y1,
                            inNumPixels,
                            inNumLines,
                            outNumPixels,
                            inYPitch);

            /* Copy Cb frame data */
            tempPtr = (outChInfo->frame->frame.iFrm.cb1);
            tempPtr += ((vOffset + (row * inNumLines)) * (outNumPixels >> 1)) +
                       (hOffset >> 1) + (col * (inNumPixels >> 1));
            frameBufferCopy(tempPtr,
                            inChInfo->frame->frame.iFrm.cb1,
                            inNumPixels >> 1,
                            inNumLines,
                            outNumPixels >> 1,
                            inYPitch >> 1);

            /* Copy Cr frame data */
            tempPtr = (outChInfo->frame->frame.iFrm.cr1);
            tempPtr += ((vOffset + (row * inNumLines)) * (outNumPixels >> 1)) +
                       (hOffset >> 1) + (col * (inNumPixels >> 1));
            frameBufferCopy(tempPtr,
                            inChInfo->frame->frame.iFrm.cr1,
                            inNumPixels >> 1,
                            inNumLines,
                            outNumPixels >> 1,
                            inYPitch >> 1);

            /* Move to next capture channel */
            inChInfo++;
        }
    }

    return 0;
}

/******************************************************************************/
/* End of file                                                                */
/******************************************************************************/
