/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2007 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/** \file   psp_bios_vport_sd_loopback.c
 *
 *  \brief  This file contains the function which captures NTSC/PAL D1
 *          resolution from one out of 8 capture channel and displays it in VP1
 *          in SD resolution (NTSC/PAL)
 *
 */

/*
 * ========== INCLUDE FILES ===========
 */

#include <std.h>
#include <log.h>
#include <bcache.h>

#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/fvid.h>
#include <ti/sdo/pspdrivers/drivers/vport/edc.h>
#include <ti/sdo/pspdrivers/drivers/vport/vport.h>
#include <ti/sdo/pspdrivers/drivers/vport/vportdis.h>
#include <ti/sdo/pspdrivers/drivers/vport/vportcap.h>

#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/sample/psp_bios_vport_common.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/sample/psp_bios_vport_capParams.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/sample/psp_bios_vport_disParams.h>


/*
 * ======== USER CONFIGURABLE MACROS ========
 */

/*
 * Video Mode
 *
 * Set this macro to MODE_NTSC or MODE_PAL for NTSC/PAL loopback
 * MODE_NTSC - To capture and display in NTSC format
 * MODE_PAL  - To capture and display in PAL format
 * Others video modes are not supported
 *
 */
#define CFG_VIDEO_MODE                      (MODE_NTSC)

/*
 * Number of channels to capture
 *
 * Set this macro to 1 to 8, to capture the required number of channels.
 * If this macro is set a value greater than 1, then this sample application
 * will capture and display channel 1 for 'CFG_LOOP_TIME' seconds, then switches
 * to channel 2 and so on up to 'CFG_NUM_CAP_CHANNEL'.
 * After that it will start from channel and the cycle continues.
 *
 */
#define CFG_NUM_CAP_CHANNEL                 (1)

/*
 * Number seconds to capture and display a single channel.
 *
 * After 'CFG_LOOP_TIME' seconds and if 'CFG_NUM_CAP_CHANNEL' is greater than
 * 1, loopback operation will switch to the next channel.
 */
#define CFG_LOOP_TIME                       (4)

/*
 * Enable/disable horizontal half scaling
 *
 * Set this macro to 1 to enable horizontal 1/2 scaling
 * Set this macro to 0 to disable horizontal 1/2 scaling
 */
#define CFG_ENABLE_SCALING                  (0)

/*
 * Number of frames for video loop back
 *
 * This sample application will run for 'CFG_NUM_FRAME_COUNT' amount of frames.
 * After which the application will close. With the current value of 100000
 * frames, the sample application will run for 55 mins of NTSC video or 66 mins
 * of PAL video.
 */
#define CFG_NUM_FRAME_COUNT                 (100000u)

/*
 * Enable or disable double word byte swap function
 *
 * Set this macro as 1/0 to enable or disable double word byte swap software
 * work around for Video Port endian issue
 *
 * 1    - Enable double word byte swap
 * 0    - Disable double word byte swap
 *
 */
#define CFG_ENABLE_DOUBLEWORD_BYTE_SWAP     (1)


/*
 * ======== DEFINES =========
 */

#if (CFG_VIDEO_MODE == MODE_NTSC)

/* Number of Horizontal Pixels */
#define NUMPIXELS                           (720)
/* NTSC D1 Number of Lines */
#define NUMLINES                            (480)
#define NUM_FRAMES_LOOP                     (CFG_LOOP_TIME * 30)

#endif  /* #if (CFG_VIDEO_MODE == MODE_NTSC) */

#if (CFG_VIDEO_MODE == MODE_PAL)

/* Number of Horizontal Pixels */
#define NUMPIXELS                           (720)
/* PAL D1 Number of Lines */
#define NUMLINES                            (576)
#define NUM_FRAMES_LOOP                     (CFG_LOOP_TIME * 25)

#endif  /* #if (CFG_VIDEO_MODE == MODE_PAL) */


/* Frame size in bytes */
#if (CFG_ENABLE_SCALING == 1)
/* When horizontal half scaling is enabled, cb and cr line pitch will be 184
   bytes instead of 180 bytes as expected. This is to ensure that cb abd cr
   lines are aligned to 8 byte boundary. Hence (8 * NUMLINES) is added to frame
   size to compensate for this */
#define FRAME_SIZE          (((NUMPIXELS / 2) * NUMLINES * 2) + (8 * NUMLINES))

#else

#define FRAME_SIZE                          (NUMPIXELS * NUMLINES * 2)

#endif  /* #if (CFG_ENABLE_SCALING == 1) */


/*
 * ======== GLOBALS =========
 */

/* Video Port configuration parameters for SDTV Capture.
   Using default value and enabling dual channel.
   Note: This variable is used in BIOS TCF file. */
VPORT_PortParams vCapParamsPort = CAP_PARAMS_PORT_DEFAULT(TRUE);

/* Video Port configuration parameters for display.
   Using default value.
   Note: This variable is used in BIOS TCF file. */
VPORT_PortParams vDisParamsPort = DIS_PARAMS_PORT_DEFAULT;

/* Global variable to check the current frame count in CCS watch window */
Uint32 AppSdFrameCnt = 0;


#if (CFG_VIDEO_MODE == MODE_NTSC)

/* Vport Capture driver configuration.
   Using embedded sync. */
static VPORTCAP_Params vCapParamsChan = CAP_PARAMS_CHAN_BT656_DEFAULT(NTSC);

/* Vport Display driver configuration.
   Using embedded sync */
static VPORTDIS_Params vDisParamsChan = DIS_PARAMS_CHAN_BT656_DEFAULT(NTSC);

#endif  /* #if (CFG_VIDEO_MODE == MODE_NTSC) */


#if (CFG_VIDEO_MODE == MODE_PAL)

/* Vport Capture driver configuration.
   Using embedded sync */
static VPORTCAP_Params vCapParamsChan = CAP_PARAMS_CHAN_BT656_DEFAULT(PAL);

/* TVP5154 EDC driver configuration.
   Using embedded sync and Composite Analog Channel A for all 8 decoders */
static TVP5154_ConfParams vCapParamsDecoder
    = CAP_PARAMS_TVP5154_EMBEDDED_DEFAULT(PAL, COMPOSITE_A);

/* Vport Display driver configuration.
   Using embedded sync */
static VPORTDIS_Params vDisParamsChan = DIS_PARAMS_CHAN_BT656_DEFAULT(PAL);

/* SAA7105 EDC driver configuration.
   Using Svideo Analog output */
static SAA7105_ConfParams vDisParamsEncoder
    = DIS_PARAMS_SAA7105_SDTV_EMBEDDED_DEFAULT(PAL720, SVIDEO);

#endif  /* #if (CFG_VIDEO_MODE == MODE_PAL) */


/*
 * ============= FUNC DECLARATIONS ==============
 */

#if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1)

/* Function which does the 8-byte swapping - Software fix for big endian mode
   operation of Video Port */
extern void double_word_byte_swap(Uint8 *bufferPtr, Uint32 numOfBytes);

#endif  /* #if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1) */

extern EDMA3_DRV_Result edma3init(void);

static Int32 test_sd_loopback(EDMA3_DRV_Handle hEdma,
                              Uint32 numCapChannel,
                              Uint32 frameCnt,
                              Uint32 numFrameLoop,
                              Int32 sedId);


/*
 * ============= EXTERNS ==============
 */

#ifdef _6x_
extern far LOG_Obj trace;
#else
extern LOG_Obj trace;
#endif

/* Heap ID defined in the BIOS configuration file.
   This memory segment is used to allocate video frame buffers. */
extern Int EXTERNALHEAP;

/* EDMA handle */
extern EDMA3_DRV_Handle hEdma;


/*
 * ============= DEFINITIONS ==============
 */


/*
 * ======== tskSdLoopback ========
 * Video application test task.
 * Created as a separate task in BIOS TCF file.
 */
void tskSdLoopback(void)
{
    EDMA3_DRV_Result result = EDMA3_DRV_SOK;

    /* Initialize EDMA3 driver */
    result = edma3init();
    if (EDMA3_DRV_SOK != result)
    {
        LOG_printf(&trace, "Can't initialized EDMA3");
        return;
    }

    /* Run SD loop back application */
    test_sd_loopback(hEdma,
                     CFG_NUM_CAP_CHANNEL,
                     CFG_NUM_FRAME_COUNT,
                     NUM_FRAMES_LOOP,
                     EXTERNALHEAP);
}



/*
 * ======== test_sd_loopback ========
 * Video SD loopback function
 * Function which performs the NTSC/PAL D1 loopback
 * hEdma            - Handle to the EDMA3 driver
 * numCapChannel    - Total number of channel to capture.
 *                    This can vary from 1 to 8.
 * totalFrameCnt    - Number of frames for loop back operation
 * numFrameLoop     - Number of frames to change to next channel
 * sedId            - Heap ID where the driver FVID buffers will be allocated
 */
static Int32 test_sd_loopback(EDMA3_DRV_Handle hEdma,
                              Uint32 numCapChannel,
                              Uint32 totalFrameCnt,
                              Uint32 numFrameLoop,
                              Int32 sedId)
{
    Int         status = IOM_COMPLETED;
    Int         inputChan;
    Int         frames;
    Int         count;
    /* Array containing display and capture channel information */
    ChannelInfo disChInfo;
    ChannelInfo capChInfo[MAX_CAP_CHAN];
    /* Array containing display and capture driver name strings
       Use this strings as they are for proper driver creation */
    Int8        *vPortCapStrings[MAX_CAP_CHAN] =
    {
        "/VP0CAPTURE/A/0",
        "/VP0CAPTURE/B/1",
        "/VP2CAPTURE/A/2",
        "/VP2CAPTURE/B/3",
        "/VP3CAPTURE/A/4",
        "/VP3CAPTURE/B/5",
        "/VP4CAPTURE/A/6",
        "/VP4CAPTURE/B/7"
    };
    Int8        *vPortDisStrings = "/VP1DISPLAY/SAA7105";

    if ((!numCapChannel) || (numCapChannel > MAX_CAP_CHAN))
    {
        LOG_printf(&trace,
                   "Give number of capture channel between 1 and %d\n",
                   MAX_CAP_CHAN);
        return -1;
    }

    if ((!totalFrameCnt) || (!numFrameLoop))
    {
        LOG_printf(&trace, "Give non-zero frame count/loop count\n");
        return -1;
    }

    /* Allocate both capture and display frame buffers
       in external heap memory */
    vCapParamsChan.segId = sedId;
    vDisParamsChan.segId = sedId;

    /* Assign EDMA3 driver handle to channel params */
    vCapParamsChan.hEdma = hEdma;
    vDisParamsChan.hEdma = hEdma;

    /* Enable or disable horizontal scaling */
#if (CFG_ENABLE_SCALING == 1)

    vCapParamsChan.scale = VPORT_SCALING_ENABLE;
    vCapParamsChan.thrld = NUMPIXELS >> 4;
    vDisParamsChan.scale = VPORT_SCALING_ENABLE;
    vDisParamsChan.thrld = NUMPIXELS >> 4;

#else

    vCapParamsChan.scale = VPORT_SCALING_DISABLE;
    vCapParamsChan.thrld = NUMPIXELS >> 3;
    vDisParamsChan.scale = VPORT_SCALING_DISABLE;
    vDisParamsChan.thrld = NUMPIXELS >> 3;

#endif  /* #if (CFG_ENABLE_SCALING == 1) */


    /* Create and configure capture drivers */
    for (count = 0; count < numCapChannel; count++)
    {
        capChInfo[count].chanHandle = FVID_create(vPortCapStrings[count],
                                                   IOM_INPUT,
                                                   &status,
                                                   (Ptr)&vCapParamsChan,
                                                   NULL);
        if (IOM_COMPLETED != status)
        {
            LOG_printf(&trace, "Failed to create capture channel %d\n", count);
            break;
        }
    }


    /* Create and configure display driver */
    if (IOM_COMPLETED == status)
    {
        disChInfo.chanHandle = FVID_create(vPortDisStrings,
                                            IOM_OUTPUT,
                                            &status,
                                            (Ptr)&vDisParamsChan,
                                            NULL);
        if (IOM_COMPLETED != status)
        {
            LOG_printf(&trace, "Failed to create display channels\n");
        }
    }


    /* Configure external video encoders (SAA7105) & decoders (TVP5154) */
    if (IOM_COMPLETED == status)
    {
        /* Only need to configure if the mode is PAL. Since NTSC is the default
           mode of operation for EDC driver */
#if !(CFG_VIDEO_MODE == MODE_NTSC)

        /* Configure SAA7105 */
        status |= FVID_control(disChInfo.chanHandle,
                               VPORT_CMD_EDC_BASE + EDC_CONFIG,
                               (Ptr)&vDisParamsEncoder);

        for (count = 0; count < numCapChannel; count++)
        {
            /* Configure TVP5154 */
            status |= FVID_control(capChInfo[count].chanHandle,
                                   VPORT_CMD_EDC_BASE + EDC_CONFIG,
                                   (Ptr)&vCapParamsDecoder);
        }

#endif  /* #if !(CFG_VIDEO_MODE == MODE_NTSC) */
    }
    else
    {
        LOG_printf(&trace, "Failed to open - Display channel\n");
    }


    /* Start display and capture operations */
    if (IOM_COMPLETED == status)
    {
        status |= FVID_control(disChInfo.chanHandle,
                               VPORT_CMD_START,
                               NULL);

        for (count = 0; count < numCapChannel; count++)
        {
            status |= FVID_control(capChInfo[count].chanHandle,
                                   VPORT_CMD_START,
                                   NULL);
        }
    }
    else
    {
        LOG_printf(&trace, "Failed to configure cap/disp channel device\n");
    }


    /* Request a frame buffer from display & capture driver */
    if (IOM_COMPLETED == status)
    {
        /* Display buffer will return a free buffer */
        status |= FVID_alloc(disChInfo.chanHandle, &(disChInfo.frame));

        for (count = 0; count < numCapChannel; count++)
        {
            /* Capture buffer will return the latest captured buffer */
            status |= FVID_alloc(capChInfo[count].chanHandle,
                                 &(capChInfo[count].frame));
        }
    }
    else
    {
        LOG_printf(&trace, "Failed to start display/capture driver\n");
    }


    if (IOM_COMPLETED == status)
    {
        frames = 0;
        inputChan = 0;          /* Default to VP0 Capture Channel A */
        while (frames < totalFrameCnt)
        {
            /* Invalidate the buffer before giving to capture driver */
            BCACHE_inv((Uint8 *)capChInfo[inputChan].frame->frame.iFrm.y1,
                       (FRAME_SIZE),
                       TRUE);

            /* Give the old capture frame buffer back to driver and get the
               recently captured frame buffer */
            status = FVID_exchange(capChInfo[inputChan].chanHandle,
                                   &(capChInfo[inputChan].frame));


#if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1)

            /* Fix for Video Port bigendian mode - This is for capture port */
            double_word_byte_swap(
                        (Uint8 *)capChInfo[inputChan].frame->frame.iFrm.y1,
                        FRAME_SIZE);

            /* Fix for Video Port bigendian mode - This is for display port */
            double_word_byte_swap(
                        (Uint8 *)capChInfo[inputChan].frame->frame.iFrm.y1,
                        FRAME_SIZE);

#endif  /* #if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1) */

            /* Flush and invalidate the processed buffer so that the EDMA reads
               the processed data */
            BCACHE_wbInv((Uint8 *)capChInfo[inputChan].frame->frame.iFrm.y1,
                         (FRAME_SIZE),
                         TRUE);


            /* Note: For DM648 we are directly exchanging the capture
             * buffer to display channel instead of copying to display buffer.
             * Care has to be taken that the buffers are with their
             * respective driver (where they were allocated) while deleting
             * (GIO delete) the drivers, because these driver will free their
             * buffers at this point.
             * But in real scenario, need for GIO delete will not occur.
             */
            /* Give the captured frame buffer to display driver and get a
               free frame buffer for next capture */
            status |= FVID_exchange(disChInfo.chanHandle,
                                    &(capChInfo[inputChan].frame));

            /* LOG_printf(&trace, "%d\n", frames); */

            if (IOM_COMPLETED != status)
            {
                LOG_printf(&trace, "IOM_COMPLETED != status\n");
                break;
            }

            frames++;

            /* Check if we need to change capture channel */
            if (0 == (frames % numFrameLoop))
            {
                if (inputChan < (numCapChannel - 1))
                {
                    inputChan++;
                }
                else if ((numCapChannel - 1) == inputChan)
                {
                    inputChan = 0;      /* Go back to channel 0 */
                }
            }

            /* Update global frame count for check purpose */
            AppSdFrameCnt = frames;
        }/* while (frame count) */


        /* Stop capture and display operation */
        if (IOM_COMPLETED == status)
        {
            status |= FVID_control(disChInfo.chanHandle,
                                   VPORT_CMD_STOP,
                                   NULL);

            for (count = 0; count < numCapChannel; count++)
            {
                status |= FVID_control(capChInfo[count].chanHandle,
                                       VPORT_CMD_STOP,
                                       NULL);
            }
        }
        else
        {
            LOG_printf(&trace, "Failed to alloc or exchange buffers\n");
        }


        /* Delete capture and display drivers */
        if (IOM_COMPLETED == status)
        {
            status |= FVID_delete(disChInfo.chanHandle);

            for (count = 0; count < numCapChannel; count++)
            {
                status |= FVID_delete(capChInfo[count].chanHandle);

                if (IOM_COMPLETED != status)
                {
                    LOG_printf(&trace,
                               "Failed to delete capture channel %d\n",
                               count);
                }
            }

            if (IOM_COMPLETED == status)
            {
                LOG_printf(&trace, "Application closed successfully!!\n");
            }
        }
        else
        {
            LOG_printf(&trace, "Failed to stop display/capture channel\n");
        }
    }
    else if (IOM_COMPLETED != status)
    {
        LOG_printf(&trace, "Failed to alloc or exchange buffers\n");
    }

    return status;
}


/******************************************************************************/
/* End of file                                                                */
/******************************************************************************/
