/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2007 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/** \file   psp_bios_vport_vesa_colorbar.c
 *
 *  \brief  This file contains the function which displays vertical colorbar
 *          in VGA, SVGA, XGA and SXGA VESA resolutions. This application
 *          uses the NEW FVID APIs.
 *
 */

/*
 * ========== INCLUDE FILES ===========
 */

#include <std.h>
#include <log.h>
#include <string.h>
#include <bcache.h>

#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/fvid.h>
#include <ti/sdo/pspdrivers/drivers/vport/edc.h>
#include <ti/sdo/pspdrivers/drivers/vport/vport.h>
#include <ti/sdo/pspdrivers/drivers/vport/vportdis.h>

#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/sample/psp_bios_vport_common.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/sample/psp_bios_vport_disParams.h>


/*
 * ======== USER CONFIGURABLE MACROS ========
 */

/*
 * Video Mode
 *
 * Set this macro to any one of the following video modes
 *
 * MODE_VGA_60HZ    - To display color bars in VGA 60Hz resolution
 * MODE_VGA_72HZ    - To display color bars in VGA 72Hz resolution
 * MODE_VGA_75HZ    - To display color bars in VGA 75Hz resolution
 * MODE_VGA_85HZ    - To display color bars in VGA 85Hz resolution
 * MODE_SVGA_60HZ   - To display color bars in SVGA 60Hz resolution
 * MODE_SVGA_72HZ   - To display color bars in SVGA 72Hz resolution
 * MODE_SVGA_75HZ   - To display color bars in SVGA 75Hz resolution
 * MODE_SVGA_85HZ   - To display color bars in SVGA 85Hz resolution
 * MODE_XGA_60HZ    - To display color bars in XGA 60Hz resolution
 * MODE_XGA_70HZ    - To display color bars in XGA 70Hz resolution
 * MODE_SXGA_60HZ   - To display color bars in SXGA 60Hz resolution
 *
 */
#define CFG_VIDEO_MODE                      (MODE_VGA_60HZ)

/*
 * Number of frames for color bar display
 *
 * This sample application will run for 'CFG_NUM_FRAME_COUNT' amount of frames.
 * After which the application will close.
 */
#define CFG_NUM_FRAME_COUNT                 (1000000u)

/*
 * Enable or disable double word byte swap function
 *
 * Set this macro as 1/0 to enable or disable double word byte swap software
 * work around for Video Port endian issue
 *
 * 1    - Enable double word byte swap
 * 0    - Disable double word byte swap
 *
 */
#define CFG_ENABLE_DOUBLEWORD_BYTE_SWAP     (1)


/*
 * ======== DEFINES =========
 */

/* Number of frame buffers to allocate per driver instance */
#define NUM_FRAME_BUFFERS                   (3)

#if (CFG_VIDEO_MODE == MODE_VGA_60HZ) || (CFG_VIDEO_MODE == MODE_VGA_72HZ) ||  \
    (CFG_VIDEO_MODE == MODE_VGA_75HZ) || (CFG_VIDEO_MODE == MODE_VGA_85HZ)

/* Number of Horizontal Pixels */
#define NUMPIXELS                           (640)
/* Number of Lines */
#define NUMLINES                            (480)

#endif  /* #if (CFG_VIDEO_MODE == MODE_VGA) */

#if (CFG_VIDEO_MODE == MODE_SVGA_60HZ) || (CFG_VIDEO_MODE == MODE_SVGA_72HZ) ||\
    (CFG_VIDEO_MODE == MODE_SVGA_75HZ) || (CFG_VIDEO_MODE == MODE_SVGA_85HZ)

/* Number of Horizontal Pixels */
#define NUMPIXELS                           (800)
/* Number of Lines */
#define NUMLINES                            (600)

#endif  /* #if (CFG_VIDEO_MODE == MODE_SVGA) */

#if (CFG_VIDEO_MODE == MODE_XGA_60HZ) || (CFG_VIDEO_MODE == MODE_XGA_70HZ)

/* Number of Horizontal Pixels */
#define NUMPIXELS                           (1024)
/* Number of Lines */
#define NUMLINES                            (768)

#endif  /* #if (CFG_VIDEO_MODE == MODE_XGA) */

#if (CFG_VIDEO_MODE == MODE_SXGA_60HZ)

/* Number of Horizontal Pixels */
#define NUMPIXELS                           (1280)
/* Number of Lines */
#define NUMLINES                            (1024)

#endif  /* #if (CFG_VIDEO_MODE == MODE_SXGA) */

/* Frame size in bytes */
#define FRAME_SIZE                          (NUMPIXELS * NUMLINES * 2)

/* 16-bit RGB values used in color bar */
#define WHITE                               (0xBDF7)
#define YELLOW                              (0xC5E0)
#define CYAN                                (0x05F7)
#define GREEN                               (0x05E0)
#define MAGENTA                             (0xB818)
#define RED                                 (0xB800)
#define BLUE                                (0x0017)
#define BLACK                               (0x0000)


/*
 * ======== GLOBALS =========
 */

/* Video Port configuration parameters for display.
   Using default value.
   Note: This variable is used in BIOS TCF file. */
VPORT_PortParams vDisParamsPort = DIS_PARAMS_PORT_DEFAULT;

/* Global variable to check the current frame count in CCS watch window */
Uint32 AppVesaFrameCnt = 0;

/* Buffer to store a line of color bar */
static Uint16 imgLineRGB[NUMPIXELS * 3];


#if (CFG_VIDEO_MODE == MODE_VGA_60HZ)

/* Vport Display driver configuration */
static VPORTDIS_Params vDisParamsChan = DIS_PARAMS_CHAN_VGA_60HZ_DEFAULT;

/* THS8200 EDC driver configuration */
static THS8200_ConfParams vDisParamsEncoder
    = DIS_PARAMS_THS8200_VGA_60HZ_DEFAULT;

#endif  /* #if (CFG_VIDEO_MODE == MODE_VGA_60HZ) */

#if (CFG_VIDEO_MODE == MODE_VGA_72HZ)

/* Vport Display driver configuration */
static VPORTDIS_Params vDisParamsChan = DIS_PARAMS_CHAN_VGA_72HZ_DEFAULT;

/* THS8200 EDC driver configuration */
static THS8200_ConfParams vDisParamsEncoder
    = DIS_PARAMS_THS8200_VGA_72HZ_DEFAULT;

#endif  /* #if (CFG_VIDEO_MODE == MODE_VGA_72HZ) */

#if (CFG_VIDEO_MODE == MODE_VGA_75HZ)

/* Vport Display driver configuration */
static VPORTDIS_Params vDisParamsChan = DIS_PARAMS_CHAN_VGA_75HZ_DEFAULT;

/* THS8200 EDC driver configuration */
static THS8200_ConfParams vDisParamsEncoder
    = DIS_PARAMS_THS8200_VGA_75HZ_DEFAULT;

#endif  /* #if (CFG_VIDEO_MODE == MODE_VGA_75HZ) */

#if (CFG_VIDEO_MODE == MODE_VGA_85HZ)

/* Vport Display driver configuration */
static VPORTDIS_Params vDisParamsChan = DIS_PARAMS_CHAN_VGA_85HZ_DEFAULT;

/* THS8200 EDC driver configuration */
static THS8200_ConfParams vDisParamsEncoder
    = DIS_PARAMS_THS8200_VGA_85HZ_DEFAULT;

#endif  /* #if (CFG_VIDEO_MODE == MODE_VGA_85HZ) */

#if (CFG_VIDEO_MODE == MODE_SVGA_60HZ)

/* Vport Display driver configuration */
static VPORTDIS_Params vDisParamsChan = DIS_PARAMS_CHAN_SVGA_60HZ_DEFAULT;

/* THS8200 EDC driver configuration */
static THS8200_ConfParams vDisParamsEncoder
    = DIS_PARAMS_THS8200_SVGA_60HZ_DEFAULT;

#endif  /* #if (CFG_VIDEO_MODE == MODE_SVGA_60HZ) */

#if (CFG_VIDEO_MODE == MODE_SVGA_72HZ)

/* Vport Display driver configuration */
static VPORTDIS_Params vDisParamsChan = DIS_PARAMS_CHAN_SVGA_72HZ_DEFAULT;

/* THS8200 EDC driver configuration */
static THS8200_ConfParams vDisParamsEncoder
    = DIS_PARAMS_THS8200_SVGA_72HZ_DEFAULT;

#endif  /* #if (CFG_VIDEO_MODE == MODE_SVGA_72HZ) */

#if (CFG_VIDEO_MODE == MODE_SVGA_75HZ)

/* Vport Display driver configuration */
static VPORTDIS_Params vDisParamsChan = DIS_PARAMS_CHAN_SVGA_75HZ_DEFAULT;

/* THS8200 EDC driver configuration */
static THS8200_ConfParams vDisParamsEncoder
    = DIS_PARAMS_THS8200_SVGA_75HZ_DEFAULT;

#endif  /* #if (CFG_VIDEO_MODE == MODE_SVGA_75HZ) */

#if (CFG_VIDEO_MODE == MODE_SVGA_85HZ)

/* Vport Display driver configuration */
static VPORTDIS_Params vDisParamsChan = DIS_PARAMS_CHAN_SVGA_85HZ_DEFAULT;

/* THS8200 EDC driver configuration */
static THS8200_ConfParams vDisParamsEncoder
    = DIS_PARAMS_THS8200_SVGA_85HZ_DEFAULT;

#endif  /* #if (CFG_VIDEO_MODE == MODE_SVGA_85HZ) */

#if (CFG_VIDEO_MODE == MODE_XGA_60HZ)

/* Vport Display driver configuration */
static VPORTDIS_Params vDisParamsChan = DIS_PARAMS_CHAN_XGA_60HZ_DEFAULT;

/* THS8200 EDC driver configuration */
static THS8200_ConfParams vDisParamsEncoder
    = DIS_PARAMS_THS8200_XGA_60HZ_DEFAULT;

#endif  /* #if (CFG_VIDEO_MODE == MODE_XGA_60HZ) */

#if (CFG_VIDEO_MODE == MODE_XGA_70HZ)

/* Vport Display driver configuration */
static VPORTDIS_Params vDisParamsChan = DIS_PARAMS_CHAN_XGA_70HZ_DEFAULT;

/* THS8200 EDC driver configuration */
static THS8200_ConfParams vDisParamsEncoder
    = DIS_PARAMS_THS8200_XGA_70HZ_DEFAULT;

#endif  /* #if (CFG_VIDEO_MODE == MODE_XGA_70HZ) */

#if (CFG_VIDEO_MODE == MODE_SXGA_60HZ)

/* Vport Display driver configuration */
static VPORTDIS_Params vDisParamsChan = DIS_PARAMS_CHAN_SXGA_60HZ_DEFAULT;

/* THS8200 EDC driver configuration */
static THS8200_ConfParams vDisParamsEncoder
    = DIS_PARAMS_THS8200_SXGA_60HZ_DEFAULT;

#endif  /* #if (CFG_VIDEO_MODE == MODE_SXGA_60HZ) */


/*
 * ============= FUNC DECLARATIONS ==============
 */

#if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1)

/* Function which does the 8-byte swapping - Software fix for big endian mode
   operation of Video Port */
extern void double_word_byte_swap(Uint8 *bufferPtr, Uint32 numOfBytes);

#endif  /* #if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1) */

extern EDMA3_DRV_Result edma3init(void);

static Int32 test_vesa_colorbar(EDMA3_DRV_Handle hEdma,
                                Uint32 frameCnt,
                                Int32 sedId);

static void generateColorBarRGB(Uint16* buf, Uint32 lineSize);

static void fill_colorbar(Uint8 *buffer, Uint32 lineSize, Uint32 numLines,
                          Int32 offset);


/*
 * ============= EXTERNS ==============
 */

#ifdef _6x_
extern far LOG_Obj trace;
#else
extern LOG_Obj trace;
#endif

/* Heap ID defined in the BIOS configuration file.
   This memory segment is used to allocate video frame buffers. */
extern Int EXTERNALHEAP;

/* EDMA handle */
extern EDMA3_DRV_Handle hEdma;


/*
 * ============= DEFINITIONS ==============
 */

/*
 * ======== tskVesaColorbar ========
 * Video application test task.
 * Created as a separate task in BIOS TCF file.
 */
void tskVesaColorbar(void)
{
    EDMA3_DRV_Result result = EDMA3_DRV_SOK;

    /* Initialize EDMA3 driver */
    result = edma3init();
    if (EDMA3_DRV_SOK != result)
    {
        LOG_printf(&trace, "Can't initialized EDMA3");
        return;
    }

    /* Run VESA colorbar application */
    test_vesa_colorbar(hEdma, CFG_NUM_FRAME_COUNT, EXTERNALHEAP);
}



/*
 * ======== test_vesa_colorbar ========
 * VESA colorbar function
 * Function which performs VESA vertical colorbar display
 * hEdma            - Handle to the EDMA3 driver
 * totalFrameCnt    - Number of frames for display operation
 * sedId            - Heap ID where the driver FVID buffers will be allocated
 */
static Int32 test_vesa_colorbar(EDMA3_DRV_Handle hEdma,
                                Uint32 totalFrameCnt,
                                Int32 sedId)
{
    Int         status = IOM_COMPLETED;
    Int         frames, bufCount;
    /* Array containing display and capture channel information */
    ChannelInfo disChInfo;
    /* Array containing display driver name strings
       Use this strings as they are for proper driver creation */
    Int8        *vPortDisStrings = "/VP1DISPLAY/THS8200";

    if (!totalFrameCnt)
    {
        LOG_printf(&trace, "Give non-zero frame count\n");
        return -1;
    }


    /* Allocate both capture and display frame buffers
       in external heap memory */
    vDisParamsChan.segId = sedId;

    /* Assign EDMA3 driver handle to channel params */
    vDisParamsChan.hEdma = hEdma;

    /*
     * Assign number of frame buffers as 0 for using NEW FVID APIs
     */
    vDisParamsChan.numFrmBufs = 0;

    /* Create and configure display driver */
    disChInfo.chanHandle = FVID_create(vPortDisStrings,
                                       IOM_OUTPUT,
                                       &status,
                                       (Ptr)&vDisParamsChan,
                                       NULL);
    if (IOM_COMPLETED != status)
    {
        LOG_printf(&trace, "Failed to create display channels\n");
    }

    /* Allocate and Queue buffers for display driver */
    if (IOM_COMPLETED == status)
    {
        for (bufCount = 0; bufCount < NUM_FRAME_BUFFERS; bufCount++)
        {
            /* Allocate Frame buffer for display driver */
            status = FVID_allocBuffer(disChInfo.chanHandle, &(disChInfo.frame));
            if (IOM_COMPLETED != status)
            {
                LOG_printf(&trace, "Failed to allocate buffer\n");
                break;
            }

            /* Queue the buffers */
            status = FVID_queue(disChInfo.chanHandle, &(disChInfo.frame));
            if (IOM_COMPLETED != status)
            {
                LOG_printf(&trace, "Failed to Queue buffer\n");
                break;
            }
        }
    }
    else
    {
        LOG_printf(&trace, "Failed to open - Display channel\n");
    }

    /* Configure external video encoder (THS8200) */
    if (IOM_COMPLETED == status)
    {
        /* Configure THS8200 */
        status = FVID_control(disChInfo.chanHandle,
                              VPORT_CMD_EDC_BASE + EDC_CONFIG,
                              (Ptr)&vDisParamsEncoder);
    }
    else
    {
        LOG_printf(&trace, "Failed to init display channel\n");
    }


    /* Start display operation */
    if (IOM_COMPLETED == status)
    {
        status = FVID_control(disChInfo.chanHandle,
                              VPORT_CMD_START,
                              NULL);
    }
    else
    {
        LOG_printf(&trace, "Failed to configure disp channel device\n");
    }


    /* Request a frame buffer from display driver */
    if (IOM_COMPLETED == status)
    {
        /* Display buffer will return a free buffer */
        status |= FVID_dequeue(disChInfo.chanHandle, &(disChInfo.frame));
    }
    else
    {
        LOG_printf(&trace, "Failed to start display driver\n");
    }

    if (IOM_COMPLETED == status)
    {
        frames = 0;
        while (frames < totalFrameCnt)
        {
            fill_colorbar((Uint8 *)disChInfo.frame->frame.rpFrm.buf,
                          NUMPIXELS, NUMLINES, 0);

#if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1)

            /* Fix for Video Port bigendian mode - This is for display port */
            double_word_byte_swap((Uint8 *)disChInfo.frame->frame.rpFrm.buf,
                                  FRAME_SIZE);

#endif  /* #if (CFG_ENABLE_DOUBLEWORD_BYTE_SWAP == 1) */

            /* Flush and invalidate the processed buffer so that the EDMA reads
               the processed data */
            BCACHE_wbInv((Uint8 *)disChInfo.frame->frame.rpFrm.buf,
                         (FRAME_SIZE),
                         TRUE);

            /* Give the filled frame buffer to display driver and get a
               free frame buffer for next filling */
            status |= FVID_exchange(disChInfo.chanHandle,
                                    &(disChInfo.frame));

            /* LOG_printf(&trace, "%d\n", frames); */

            if (IOM_COMPLETED != status)
            {
                LOG_printf(&trace, "IOM_COMPLETED != status\n");
                break;
            }

            frames++;

            /* Update global frame count for check purpose */
            AppVesaFrameCnt = frames;
        }/* while (frame count) */


        /* Stop display operation */
        if (IOM_COMPLETED == status)
        {
            status |= FVID_control(disChInfo.chanHandle,
                                   VPORT_CMD_STOP,
                                   NULL);

            /* Free the buffer owned by application */
            status |= FVID_freeBuffer(disChInfo.chanHandle,
                                      &(disChInfo.frame));

            /* Dequeue buffers from driver and free them */
            for (bufCount = 0; bufCount < (NUM_FRAME_BUFFERS - 1); bufCount++)
            {
                status |= FVID_dequeue(disChInfo.chanHandle,
                                       &(disChInfo.frame));
                status |= FVID_freeBuffer(disChInfo.chanHandle,
                                          &(disChInfo.frame));
            }
        }
        else
        {
            LOG_printf(&trace, "Failed to alloc or exchange buffers\n");
        }


        /* Delete display drivers */
        if (IOM_COMPLETED == status)
        {
            status |= FVID_delete(disChInfo.chanHandle);
            if (IOM_COMPLETED == status)
            {
                LOG_printf(&trace, "Application closed successfully!!\n");
            }
        }
        else
        {
            LOG_printf(&trace, "Failed to stop display channel\n");
        }
    }
    else if (IOM_COMPLETED != status)
    {
        LOG_printf(&trace, "Failed to alloc or exchange buffers\n");
    }

    return status;
}



/*
 * ======== fill_colorbar ========
 * This function fills the given buffer with one frame of color bar in RGB565
 * format.
 * buffer       - Frame buffer pointer
 * lineSize     - Number of samples (16-bit) per line
 * numLines     - Number of lines per frame
 * offset       - Used for moving color bar
 */
void fill_colorbar(Uint8 *buffer, Uint32 lineSize, Uint32 numLines,
                   Int32 offset)
{
    Uint32          i;
    static Int32    init = 0;

    if (!init)
    {
        generateColorBarRGB(imgLineRGB, lineSize);
        init = 1;
    }

    for (i = 0; i < numLines; i++)
    {
        memcpy(buffer + (lineSize << 1) * i, imgLineRGB + offset * 8,
               (lineSize << 1));
    }

    return;
}



/*
 * ======== generateColorBarRGB ========
 * This function generates a line of color bar with RGB565 format.
 * lineBuf      - Line buffer pointer
 * lineSize     - Number of samples (16-bit) per line
 */
static void generateColorBarRGB(Uint16 *lineBuf, Uint32 lineSize)
{
    Int i;
    Int fillSize = lineSize >> 3;
    Int k;

    for (k = 0; k < 3; k++)
    {
        for (i = 0; i < fillSize; i++)
        {
            lineBuf[fillSize * 0 + i] = WHITE;
            lineBuf[fillSize * 1 + i] = BLACK;
            lineBuf[fillSize * 2 + i] = CYAN;
            lineBuf[fillSize * 3 + i] = GREEN;
            lineBuf[fillSize * 4 + i] = MAGENTA;
            lineBuf[fillSize * 5 + i] = RED;
            lineBuf[fillSize * 6 + i] = BLUE;
            lineBuf[fillSize * 7 + i] = YELLOW;
        }

        lineBuf += lineSize;
    }

    return;
}


/******************************************************************************/
/* End of file                                                                */
/******************************************************************************/
