/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2007 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \file edcEncoder.c
 *
 *  This file contains interface functions for both (THS8200 & SAA7105) encoders
 */

#include <std.h>
#include <string.h>
#include <ti/sdo/pspdrivers/drivers/vport/edc.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/edcEncoder.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/src/_ths8200.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/ths8200.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/src/_saa7105.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/src/edc_debug.h>

/* Flags indicating which encoder is opened */
#define THS8200_FLAG                    (8200)  /**< THS8200 Flag */
#define SAA7105_FLAG                    (7105)  /**< SAA7105 Flag */
#define INVALID_FLAG                    (-1)    /**< Invalid Flag */

/*
 * ========= PRIVATE DEFINITIONS =========
 */
/** Indicates which encoder has been selected */
static Int32 encFlag = INVALID_FLAG;

/** Handle of THS8200, used when SAA7105 is selected */
static EDC_Handle ths8200_handle = NULL;

/** Default configuration of THS8200 */
static THS8200_ConfParams THS8200_default =
{
    THS8200_MODE_1080I_60HZ,            /* Mode              */
    THS8200_IFMT_YCBCR422_20_BIT,       /* InputFormat       */
    THS8200_AFMT_YPBPR_SYNC_ON_ALL,     /* AnalogFormat      */
    TRUE,                               /* enableSlaveMode   */
    TRUE                                /* enableBT656Sync   */
};

/** Default configuration of SAA7105 */
static SAA7105_ConfParams SAA7105_default =
{
    SAA7105_AFMT_SVIDEO,                  /* AnalogFormat      */
    SAA7105_MODE_NTSC720,                 /* Mode              */
    SAA7105_IFMT_YCBCR422_INTERLACED,     /* InputFormat       */
    TRUE,                                 /* enableSlaveMode   */
    TRUE,                                 /* enableBT656Sync   */
};


/*
 * ========== FUNCTION DECLARATIONS =========
 */
static Int32 edcEncoder_ctrl(EDC_Handle, Uns, Arg);
static Int32 edcEncoder_close(EDC_Handle);
static EDC_Handle edcEncoder_open(char *, Arg);


/*
 * ========== FUNCTION POINTER TABLE =========
 */
/** EDC Encoder function table */
EDC_Fxns edcEncoder_Fxns =
{
    edcEncoder_open,
    edcEncoder_close,
    edcEncoder_ctrl
};


/**
 *  \brief  Close Encoder device
 *
 *  \param  handle [IN]  External device channel handle
 *
 *  \return  Success    EDC_SUCCESS
 *           Error      EDC_FAILED
 */
static Int32 edcEncoder_close(EDC_Handle handle)
{
    Int32 status = EDC_SUCCESS;

    EDC_DEBUG("\n\nEntered ---> edcEncoder_close\n");
    EDC_DEBUG1("EDC_Handle = 0x%0.8X\n", handle);

    if (THS8200_FLAG == encFlag)
    {
        /* Close THS8200 */
        status = THS8200_Fxns.close(handle);
    }
    else if (SAA7105_FLAG == encFlag)
    {
        /* Close THS8200 and SAA7105 */
        status = SAA7105_Fxns.close(handle);
        status |= THS8200_Fxns.close(ths8200_handle);
    }
    else
    {
        status = EDC_FAILED;
    }

    encFlag = INVALID_FLAG;

    EDC_DEBUG("\nExited ---> edcEncoder_close\n");

    return status;
}



/**
 *  \brief  Call Encoder's Ioctl function according to encoder selected at
 *          device open
 *
 *  \param  handle [IN]  External device channel handle
 *  \param  cmd    [IN]  IOCTL to be performed (mostly enum)
 *  \param  arg1   [IN]  Argument as required by Ioctl
 *
 *  \return  Success    EDC_SUCCESS
 *           Error      EDC_FAILED
 */
static Int32 edcEncoder_ctrl(EDC_Handle handle, Uns cmd, Arg arg1)
{
    Int32 status = EDC_SUCCESS;

    EDC_DEBUG("\n\nEntered ---> edcEncoder_ctrl\n");
    EDC_DEBUG2("EDC_Handle = 0x%0.8X, cmd = 0x%0.8X, ", handle, cmd);
    EDC_DEBUG1("Arg = 0x%0.8X\n", arg1);

    /* Call control function as per encFlag set */
    if (THS8200_FLAG == encFlag)
    {
        status = THS8200_Fxns.ctrl(handle, cmd, (Arg)arg1);
    }
    else if (SAA7105_FLAG == encFlag)
    {
        status = SAA7105_Fxns.ctrl(handle, cmd, (Arg)arg1);
    }
    else
    {
        status = EDC_FAILED;
    }

    EDC_DEBUG("Exited ---> edcEncoder_ctrl\n\n");

    return status;
}



/**
 *  \brief  Open Encoder as per application's input
 *
 *  \param  devName [IN]  External device name
 *  \param  optArg  [IN]  Optional argument (not used here)
 *
 *  \return  handle  EDC channel handle when success
 *                   EDC_FAILED when unsuccessful
 */
static EDC_Handle edcEncoder_open(char *devName, Arg optArg)
{
    char        *ths8200Str = "THS8200";
    char        *saa7105Str = "SAA7105";
    Int32       status = EDC_FAILED;
    EDC_Handle  handle = NULL;

    EDC_DEBUG("\n\nEntered ---> edcEncoder_open\n");
    EDC_DEBUG2("devName = %s, Arg = 0x%0.8X\n", devName, optArg);

    /* Check if the address is correct and if already opened */
    if (('/' == *devName++) && (INVALID_FLAG == encFlag))
    {
        /* Compare devName with "THS8200" */
        if (!strcmp(devName, ths8200Str))
        {
            /* Open THS8200 */
            ths8200_handle = THS8200_Fxns.open(devName, (Arg)INV);
            if (ths8200_handle != NULL)
            {
                /* Disable THS8200 bypass mode */
                status = THS8200_Fxns.ctrl(ths8200_handle,
                                           THS8200_ENABLE_BYPASS,
                                           (Arg)FALSE);

                /* Configure THS8200 to default mode */
                status |= THS8200_Fxns.ctrl(ths8200_handle,
                                            EDC_CONFIG,
                                            (Arg)&THS8200_default);
                if (EDC_SUCCESS == status)
                {
                    /* Put SAA7105 in power down mode */
                    handle = SAA7105_Fxns.open(devName, (Arg)INV);
                    if (handle != NULL)
                    {
                        status = SAA7105_Fxns.ctrl(handle, SAA7105_POWERDOWN,
                                                   (Arg)NULL);
                        status |= SAA7105_Fxns.close(handle);

                        /* Assign THS8200 handle to be returned */
                        handle = ths8200_handle;
                        encFlag = THS8200_FLAG;
                    }
                }
                else
                {
                    handle = (EDC_Handle)NULL;
                }
            }
        }
        /* Compare devName with "SAA7105" */
        else if (!strcmp(devName, saa7105Str))
        {
            /* Enable THS8200 bypass mode */
            ths8200_handle = THS8200_Fxns.open(devName, (Arg)INV);
            if (ths8200_handle != NULL)
            {
                status = THS8200_Fxns.ctrl(ths8200_handle,
                                           THS8200_ENABLE_BYPASS,
                                           (Arg)TRUE);
            }
            else
            {
                status = EDC_FAILED;
            }

            if (EDC_SUCCESS == status)
            {
                /* Open SAA7105 */
                handle = SAA7105_Fxns.open(devName, (Arg)INV);
                status = SAA7105_Fxns.ctrl(handle,
                                           EDC_CONFIG,
                                           (Arg)&SAA7105_default);
                if (EDC_SUCCESS == status)
                {
                    encFlag = SAA7105_FLAG;
                }
                else
                {
                    handle = (EDC_Handle)NULL;
                }
            }
        }
    }

    EDC_DEBUG("\nExited ---> edcEncoder_open\n");

    return handle;
}


/******************************************************************************/
/* End of file                                                                */
/******************************************************************************/
