/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2007 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \file psp_i2c_interface.c
 *
 *  This file contains interface functions for i2c transfers
 */

/*
 * ======== INCLUDE FILES ========
 */
#include <std.h>
#include <string.h>

#include <ti/sdo/pspdrivers/pal_os/pal_osprotect.h>

#ifndef I2C_DISABLE
#include <ti/sdo/pspdrivers/common/psp_common.h>
#include <ti/sdo/pspdrivers/drivers/i2c/psp_i2c.h>
#include <ti/sdo/pspdrivers/drivers/i2c/src/psp_i2cApi.h>
#endif /* I2C_DISABLE */

#include <ti/sdo/pspdrivers/drivers/vport/edc.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/src/edc_debug.h>

#include <ti/sdo/pspdrivers/pal_os/pal_osTask.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osSem.h>


/*
 * ======== DEFINES ========
 */
/** Buffer size for the write buffer */
#define BUFFER_SIZE                     (260)


/*
 * ======== STATIC VARIABLES ========
 */
#ifndef I2C_DISABLE

static Uint32 i2cInstanceId = 0x00; /**< I2C driver instance ID */
static PSP_Handle i2cHandle = NULL; /**< I2C driver handle */
static Int32 initCount = 0;         /**< Make record of init */
static PAL_OsSemHandle    completionSem; /**< This semaphore is used to make the
                                              PSP interface at EDC level sync,
                                              since the driver layer has been
                                              made async*/
static Uint32 xferActual = 0; /**< Since the actual xfer size can only be known
                                   in the callback context, we need to have a
                                   global variable to check this in the caller
                                   context */

static Void I2cCallBack (Ptr semHandle,Ptr arg, Ptr ioRequest);

#endif /* I2C_DISABLE */


/**
 *  \brief  initialization of i2c object
 *
 *  \return EDC_SUCCESS on success
 *          EDC_FAILED on failure
 */
Int32 i2c_init(void)
{
    Int32 retVal = EDC_SUCCESS;
#ifndef I2C_DISABLE
    PAL_OsSemAttrs        semAttrs        = PAL_OSSEM_ATTRS;
    Uint32 imask;

    PAL_osProtectEntry(PAL_OSPROTECT_SCHEDULER, &imask);

    if ((0 == initCount))
    {
        /* Create Completion internal Semaphore */
        if (PAL_osSemCreate(
                            "I2C Completion Sem",
                            0,
                            &semAttrs,
                            &completionSem
                           ) != PAL_SOK)
        {
            retVal = FALSE;
        }

        /* open an instance */
        i2cHandle = PSP_i2cOpen(i2cInstanceId, &I2cCallBack, &completionSem);
    }

    if (NULL == i2cHandle)
    {
        retVal = EDC_FAILED;
    }
    else
    {
        initCount++;
    }

    PAL_osProtectExit(PAL_OSPROTECT_SCHEDULER, imask);

#endif /* I2C_DISABLE */

    return retVal;
}



/**
 *  \brief  I2C write register
 *
 *  \param  slaveAddr [In] slave address of i2c device
 *  \param  subAddr   [In] sub address of slave on which write
 *                        operation to be performed
 *  \param  buffer    [In] buffer pointer to write values
 *  \param  length    [In] number of bytes to write
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED on failure of write operation
 */
Int32 i2c_writeReg(Uint8 slaveAddr, Uint8 subAddr, Uint8 *buffer, Uint32 length)
{
    Int32 retCode = -1;
    Int32 count = 0;
    Int32 retVal = EDC_FAILED;
    Uint8 writeBuffer[BUFFER_SIZE + 1];
    PSP_i2cDataParam     buf;
    PSP_I2cRequest      IoRequest;

    /* Write to Slave Device */
    if ((buffer != NULL) && (length <= BUFFER_SIZE))
    {
        for (count = 0; count < length; count++)
        {
            /* printf("$%0.2X,W,Y,$0064,%0.2X,%0.2X,\n", (slaveAddr << 1),
                        (subAddr + count), buffer[count]); */
            EDC_DEBUG1("I2C Write: SlaveAddr = 0x%0.2X, ", slaveAddr);
            EDC_DEBUG2("SubAddr = 0x%0.2X, Value = 0x%0.2X\n",
                       subAddr + count,
                       buffer[count]);
        }

        writeBuffer[0] = subAddr;
        memcpy(&writeBuffer[1], buffer, length);

#ifndef I2C_DISABLE
        buf.i2cTrans.buffer    = writeBuffer;
        buf.i2cTrans.bufLen    = length+1;
        buf.i2cTrans.flags     = PSP_I2C_DEFAULT_WRITE;
        buf.i2cTrans.param     = NULL;
        buf.i2cTrans.slaveAddr = slaveAddr;
        buf.timeout     = -1;


        /* Update dataparams */
        IoRequest.dataparam  = &buf;
        IoRequest.actualSize = 0;
        IoRequest.appCbk     = NULL;
        IoRequest.status     = PSP_SOK;
        retCode = PSP_i2cTransferIOP( i2cHandle,&IoRequest,(Ptr)NULL);

        if (PSP_SINPROGRESS == retCode)
        {
            if( (PAL_SOK != PAL_osSemTake(completionSem, -1)))
            {
                retVal = FALSE;
                retCode = -1;
            }
            else
            {
                retVal = TRUE;
                retCode = 0;

            }
        }

        if( (retCode < 0) )
        {
                retVal = FALSE;
        }
#else

        retCode = 0;
        xferActual = length + 1;

#endif /* I2C_DISABLE */

        if ((xferActual != (length + 1)) && (retCode != 0))
        {
            retVal = EDC_FAILED;
        }
        else
        {
            retVal = EDC_SUCCESS;
        }
    } /* if ((buffer != NULL) && (length < BUFFER_SIZE)) */

    return retVal;
}



/**
 *  \brief I2C read register
 *
 *  \param  slaveAddr [In]  slave address of i2c device
 *  \param  subAddr   [In]  sub address of slave on which write
 *                            operation to be performed
 *  \param  buffer    [Out] buffer pointer to store read values
 *  \param  length    [In]  number of bytes to read
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED on failure of read operation
 */
Int32 i2c_readReg(Uint8 slaveAddr, Uint8 subAddr, Uint8 *buffer, Uint32 length)
{
    Int32   retCode = -1;
    Int32   count = 0;
    Int32   retVal = EDC_FAILED;
    PSP_i2cDataParam    buf;
    PSP_I2cRequest      IoRequest;

    /* Write Subaddress of Slave Device */
    if (buffer != NULL)
    {
        for (count = 0; count < length; count++)
        {
#ifndef I2C_DISABLE
        buf.i2cTrans.buffer    = &subAddr;
        buf.i2cTrans.bufLen    = 1;
        buf.i2cTrans.flags     = PSP_I2C_DEFAULT_WRITE;
        buf.i2cTrans.param     = NULL;
        buf.i2cTrans.slaveAddr = slaveAddr;
        buf.timeout     = -1;


         /* Update dataparams */
        IoRequest.dataparam  = &buf;
        IoRequest.actualSize = 0;
        IoRequest.appCbk     = NULL;
        IoRequest.status     = PSP_SOK;

        retCode = PSP_i2cTransferIOP( i2cHandle,&IoRequest,(Ptr)NULL);

        if (PSP_SINPROGRESS == retCode)
        {
              if( (PAL_SOK != PAL_osSemTake(completionSem, -1)) )
              {
                retVal = FALSE;
                retCode = -1;
              }
              else
              {
                retVal =    TRUE;
                retCode = 0;
              }
        }

        if( (retCode < 0) )
        {
                retVal = FALSE;
        }
#else

            retCode = 0;
            xferActual = 1;

#endif /* I2C_DISABLE */

            /* Read from Slave Device */
            if ((1 == xferActual) && (0 == retCode))
            {
#ifndef I2C_DISABLE
        buf.i2cTrans.buffer    = &buffer[count];
        buf.i2cTrans.bufLen    = 1;
        buf.i2cTrans.flags     = PSP_I2C_DEFAULT_READ;
        buf.i2cTrans.param     = NULL;
        buf.i2cTrans.slaveAddr = slaveAddr;
        buf.timeout     = -1;


         /* Update dataparams */
        IoRequest.dataparam  = &buf;
        IoRequest.actualSize = 0;
        IoRequest.appCbk     = NULL;
        IoRequest.status     = PSP_SOK;

        retCode = PSP_i2cTransferIOP( i2cHandle,&IoRequest,(Ptr)NULL);

        if (PSP_SINPROGRESS == retCode)
        {
              if( (PAL_SOK != PAL_osSemTake(completionSem, -1)) )
              {
                retVal = FALSE;
                retCode = -1;
              }
              else
              {
                retCode = 0;
                retVal =    TRUE;
              }
        }

        if( (retCode < 0) )
        {
                retVal = FALSE;
        }
#else

                retCode = 0;
                xferActual = 1;

#endif  /* I2C_DISABLE */

                EDC_DEBUG1("I2C Read: SlaveAddr = 0x%0.2X, ", slaveAddr);
                EDC_DEBUG2("SubAddr = 0x%0.2X, Value = 0x%0.2X\n",
                           subAddr,
                           buffer[count]);
            }

            if ((xferActual != 1) && (retCode != 0))
            {
                retVal = EDC_FAILED;
                break;
            }
            else
            {
                retVal = EDC_SUCCESS;
            }

            subAddr++;
        } /* for(count = 0;...) */
    }

    return retVal;
}



/**
 *  \brief  deinitialize i2c object
 *
 *  \return EDC_SUCCESS on success
 *          EDC_FAILED on failure
 */
Int32 i2c_deInit(void)
{
    Int32 retVal = EDC_SUCCESS;
#ifndef I2C_DISABLE

    Uint32 imask;

    PAL_osProtectEntry(PAL_OSPROTECT_SCHEDULER, &imask);

    if (initCount > 0)
    {
        initCount--;
    }

    /* Close driver if last i2c instance */
    if (0 == initCount)
    {
        /* Delete Completion Semaphore  */
        if (PAL_osSemDelete(completionSem) != PAL_SOK)
        {
            retVal = FALSE;
        }

        /* Close i2c driver and delete instance */
        if (PSP_i2cClose(i2cHandle) != PSP_SOK)
        {
            retVal = EDC_FAILED;
        }
    }

    PAL_osProtectExit(PAL_OSPROTECT_SCHEDULER, imask);

#endif /* I2C_DISABLE */

    return retVal;
}


/**
 *  \fn      static Void aic31CallBack ()
 *
 *  \brief   This function is the codec call back function invoked from the I2c
 *           driver on transfer completion.
 *
 *  \param   semHandle [IN] pointer to the Semaphore Handle
 *
 *  \return  none
 */
static Void I2cCallBack (Ptr Handle,Ptr arg, Ptr ioRequest)
{
    arg = arg;
    Handle = Handle;

    /* Update the actual data transfer that ws done */
    /* Let the submitter of the request decide what needs to be done after
       checking this */
    xferActual = ((PSP_I2cRequest*)ioRequest)->actualSize;

    PAL_osSemGive(completionSem);
    return;
}
/******************************************************************************/
/* End of file                                                                */
/******************************************************************************/
