/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2007 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \file saa7105.c
 *
 *  This file contains interface functions for SAA7105 encoder
 */
#include <std.h>
#include <ti/sdo/pspdrivers/drivers/vport/edc.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/saa7105.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/src/_saa7105.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/src/psp_i2c_interface.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/src/edc_debug.h>

/* local functions declaration */
static Int32 configSAA7105(SAA7105_Mode,
                           Bool,
                           SAA7105_AnalogFormat,
                           SAA7105_InputFormat,
                           Bool);
static Int32 enableColorBars(Bool);
static Int32 enableSlaveMode(Bool);
static Int32 initSAA7105Def(void);
static Int32 powerdownSAA7105(Bool);
static Int32 resetSAA7105(void);
static Int32 loadCursor(SAA7105_Cursor*);
static Int32 setOutputMode(SAA7105_AnalogFormat);
static Int32 setSAA7105SyncMode(Bool);
static Int32 setVideoDisplay(SAA7105_Mode,
                             SAA7105_AnalogFormat,
                             SAA7105_InputFormat,
                             Bool);

static Int32 SAA7105_close(EDC_Handle);
static Int32 SAA7105_ctrl(EDC_Handle, SAA7105_Cmd, Arg);
static EDC_Handle SAA7105_open(String, Arg);

/** SAA7105 EDC function table */
EDC_Fxns SAA7105_Fxns =
{
    SAA7105_open,
    SAA7105_close,
    SAA7105_ctrl
};

#ifdef ENABLE_EDC_DEBUG
/** SAA7105 register copy */
static Uint8 SAA7105_Regs[512] =
{
    0
};
#endif /* _ENABLE_EDC_DEBUG */

/** SAA7105 output mode table */
static const Int32 outputModeTbl[3][5] =
{
    {1, 1, 0, 0, 1},
    {0, 0, 0, 0, 0},
    {0, 1, 1, 1, 0}
};

/** SAA7105 mode table */
static const Int32 modeTbl[2][8] =
{
    /* NTSC */{248, 14, 1715, 720, 242, 0, 0, 0x20},
    /* PAL */{232, 18, 1727, 720, 290, 0, 0, 0x20},
};

/** SAA7105 Cursor data */
static SAA7105_Cursor _SAA7105_cursorData;


/**
 *  \brief  I2C write register
 *
 *  This function updates register array with I2C write(only in debugging mode)
 *
 *  \param  devAddr   [In] slave address of I2C device
 *  \param  subAddr   [In] sub address of slave on which write
 *                            operation to be performed
 *  \param  buffer    [In] buffer pointer to write values
 *  \param  length    [In] number of bytes to write
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED on failure of write operation
 */
inline static Int32 _IIC_write(Uint8 devAddr,
                               Uint8 subAddr,
                               Uint8 *buffer,
                               Int32 length)
{
#ifdef ENABLE_EDC_DEBUG
    Int32 count = 0;

    for (count = 0; count < length; count++)
    {
        SAA7105_Regs[subAddr + count] = *(buffer + count);
    }

    SAA7105_Regs[0] = SAA7105_Regs[0]; /* to remove compiler warning */
#endif /* ENABLE_EDC_DEBUG */

    /* call to intermediate PSP I2C interface function */
    return (i2c_writeReg(devAddr, subAddr, buffer, length));
}



/**
 *  \brief configure SAA7105 registers
 *
 *  \param  videoMode        [In]  SAA7105 Video Mode
 *  \param  slaveModeEnable  [In]  Flag to enable or disable slave mode
 *  \param  outMode          [In]  SAA7105 analog format
 *  \param  inFormat         [In]  SAA7105 input format
 *  \param  enableBT656Sync  [In]  Flag to enable or disable BT656 sync
 *
 *  \return  Success    EDC_SUCCESS
 *           Error      EDC_FAILED
 */
static Int32 configSAA7105(SAA7105_Mode videoMode,
                          Bool slaveModeEnable,
                          SAA7105_AnalogFormat outMode,
                          SAA7105_InputFormat inFormat,
                          Bool enableBT656Sync)
{
    Int32 status = EDC_SUCCESS;
    /***************************************************************/
    /* Enable master/slave mode                                    */
    /***************************************************************/

    EDC_DEBUG("Entered ---> configSAA7105\n");
    EDC_DEBUG2("SAA7105_Mode = %d, slaveModeEnable = %d, ",
               videoMode,
               slaveModeEnable);
    EDC_DEBUG2("SAA7105_AnalogFormat = %d, SAA7105_InputFormat = %d, ",
               outMode,
               inFormat);
    EDC_DEBUG1("enableBT656Sync = %d\n", enableBT656Sync);

    status = initSAA7105Def();

    if (EDC_SUCCESS == status)
    {
        status = setVideoDisplay(videoMode, outMode, inFormat, enableBT656Sync);
    }

    if (EDC_SUCCESS == status)
    {
        status = enableSlaveMode(slaveModeEnable);
    }

    if (EDC_SUCCESS == status)
    {
        status = enableColorBars(FALSE);
    }

    if (EDC_SUCCESS == status)
    {
        status = loadCursor(NULL);
    }

    EDC_DEBUG("Exited ---> configSAA7105\n");

    return status;
}



/**
 *  \brief  Enable/Disable Colorbars
 *
 *  \param enable [In] Flag to enable or disable colorbars
 *
 *  \return  Success    EDC_SUCCESS
 *           Error      EDC_FAILED
 */
static Int32 enableColorBars(Bool enable)
{
    _SAA7105_Regs *regPtr;
    Uint8 *charPtr;
    Int32 status = EDC_SUCCESS;

    EDC_DEBUG("Entered ---> enableColorBars\n");
    EDC_DEBUG1("enable = %d\n", enable);

    regPtr = &_SAA7105_settingsDef;

    if (enable)
    {
        regPtr->colorBarsEnable = TRUE;
    }
    else
    {
        regPtr->colorBarsEnable = FALSE;
    }

    charPtr = (Uint8 *)&_SAA7105_settingsDef;

    status = _IIC_write(_SAA7105_IICADDR,
                        SAA7105_INPUT_PORT_CTRL_1_ADDR,
                        &charPtr[SAA7105_INPUT_PORT_CTRL_1_ADDR],
                        1);

    EDC_DEBUG("Exited ---> enableColorBars\n");

    return status;
}



/**
 *  \brief Load Hardware Cursor Bit Map
 *
 *  \param cursorTable [In] SAA7105_Cursor structure pointer
 *
 *  \return  Success    EDC_SUCCESS
 *           Error      EDC_FAILED
 */
static Int32 loadCursor(SAA7105_Cursor* cursorTable)
{
    Uint32 i;
    Int32 status = EDC_SUCCESS;

    if (NULL == cursorTable)
    {
        /* Load Hardware Cursor Bit Map with transparency */
        cursorTable = &_SAA7105_cursorData;
        cursorTable->address = SAA7105_CURSOR_RAM_ADDR;

        for (i = 0; i < SAA7105_CURSOR_NUMBYTES; i++)
        {
            cursorTable->cursorBlock[i] = SAA7105_CURSOR_TRANS_VAL;
        }
    }

    status = _IIC_write(_SAA7105_IICADDR, SAA7105_CURSOR_BIT_MAP_ADDR,
                        (void *)(cursorTable), (SAA7105_CURSOR_NUMBYTES + 1));

    return status;
}



/**
 *  \brief  Enable/Disable Slave Mode
 *
 *  \param enable [In] Flag to enable or disable slave mode
 *
 *  \return  Success    EDC_SUCCESS
 *           Error      EDC_FAILED
 */
static Int32 enableSlaveMode(Bool enable)
{
    _SAA7105_Regs *regPtr;
    Uint8 *charPtr;
    Int32 status = EDC_SUCCESS;

    EDC_DEBUG("Entered ---> enableSlaveMode\n");
    EDC_DEBUG1("enable = %d\n", enable);

    regPtr = &_SAA7105_settingsDef;
    if (!enable)
    {
        regPtr->masterSlaveMode = FALSE;
        regPtr->vSyncOutput = TRUE;
        regPtr->hSyncOutput = TRUE;
        regPtr->fieldOutput = TRUE;
        regPtr->bypassInputFIFO = FALSE;
    }
    else
    {
        regPtr->masterSlaveMode = TRUE;
        regPtr->vSyncOutput = FALSE;
        regPtr->hSyncOutput = FALSE;
        regPtr->fieldOutput = FALSE;
        regPtr->bypassInputFIFO = TRUE;
    }
    charPtr = (Uint8 *)&_SAA7105_settingsDef;

    status = _IIC_write(_SAA7105_IICADDR,
                        SAA7105_SCALER_CTRL_MCB_YPIX_ADDR,
                        &charPtr[SAA7105_SCALER_CTRL_MCB_YPIX_ADDR],
                        2);
    status |= _IIC_write(_SAA7105_IICADDR,
                         SAA7105_PIXEL_CLK_CTRL_ADDR,
                         &charPtr[SAA7105_PIXEL_CLK_CTRL_ADDR],
                         1);

    EDC_DEBUG("Exited ---> enableSlaveMode\n");

    return status;
}



/**
 *  \brief Initialize the SAA7105 in default setup
 *
 *  \return  Success    EDC_SUCCESS
 *           Error      EDC_FAILED
 */
static Int32 initSAA7105Def(void)
{
    Uint8 *charPtr, subAddress;
    Int32 status = EDC_SUCCESS;

    /* Load registers from ROM to RAM */
    EDC_DEBUG("Entered ---> initSAA7105Def\n");

    charPtr = (Uint8 *)&_SAA7105_settingsDef;
    subAddress = SAA7105_HD_SYNC_LINE_COUNT_ARRAY_ADDR;
    status = _IIC_write(_SAA7105_IICADDR, subAddress, &charPtr[subAddress], 13);
    subAddress = SAA7105_CURSOR_COLOR_1_R_ADDR;
    status |= _IIC_write(_SAA7105_IICADDR, subAddress, &charPtr[subAddress],
                         14);

    EDC_DEBUG("Exited ---> initSAA7105Def\n");

    return status;
}



/**
 *  \brief Powerdown SAA7105
 *
 *  \param powerDownEnable [In] Flag to select power down or power up mode
 *
 *  \return  Success    EDC_SUCCESS
 *           Error      EDC_FAILED
 */
static Int32 powerdownSAA7105(Bool powerDownEnable)
{
    Uint8 *charPtr = (Uint8 *)&_SAA7105_settingsDef;
    Int32 status = EDC_SUCCESS;

    EDC_DEBUG("Entered ---> powerdownSAA7105\n");
    EDC_DEBUG1("powerDownEnable = %d\n", powerDownEnable);

    /* Put _SAA7105 in power down mode */
    if (powerDownEnable)
    {
        _SAA7105_settingsDef.downA = TRUE;
        _SAA7105_settingsDef.downD = TRUE;
    }
    else
    {
        _SAA7105_settingsDef.downA = FALSE;
        _SAA7105_settingsDef.downD = FALSE;
    }

    status = _IIC_write(_SAA7105_IICADDR,
                        SAA7105_STANDARD_CTRL_ADDR,
                        &charPtr[SAA7105_STANDARD_CTRL_ADDR],
                        1);

    EDC_DEBUG("Exited ---> powerdownSAA7105\n");

    return status;
}



/**
 *  \brief Reset SAA7105
 *
 *  \return  Success    EDC_SUCCESS
 *           Error      EDC_FAILED
 */
static Int32 resetSAA7105(void)
{
    Int32 status = EDC_SUCCESS;

    /* power-up SAA7105 */

    EDC_DEBUG("Entered ---> resetSAA7105\n");

    status = powerdownSAA7105(FALSE);

    EDC_DEBUG("Exited ---> resetSAA7105\n");

    return status;
}



/**
 *  \brief Set Output Mode
 *
 *  \param outMode [In]  SAA7105 analog format
 *
 *  \return  Success    EDC_SUCCESS
 *           Error      EDC_FAILED
 */
static Int32 setOutputMode(SAA7105_AnalogFormat outMode)
{
    _SAA7105_Regs *regPtr;
    Uint8 *charPtr;
    Int32 status = EDC_SUCCESS;

    EDC_DEBUG("Entered ---> setOutputMode\n");
    EDC_DEBUG1("SAA7105_AnalogFormat = %d\n", outMode);

    regPtr = &_SAA7105_settingsDef;
    regPtr->vbsEn = outputModeTbl[outMode][0];
    regPtr->cvbsEn0 = outputModeTbl[outMode][1];
    regPtr->cvbsEn1 = outputModeTbl[outMode][2];
    regPtr->cvbsEn2 = outputModeTbl[outMode][3];
    regPtr->cEn = outputModeTbl[outMode][4];
    charPtr = (Uint8 *)&_SAA7105_settingsDef;

    status = _IIC_write(_SAA7105_IICADDR,
                        SAA7105_OUTPUT_PORT_CTRL_ADDR,
                        &charPtr[SAA7105_OUTPUT_PORT_CTRL_ADDR],
                        1);

    EDC_DEBUG("Exited ---> setOutputMode\n");

    return status;
}



/**
 *  \brief  Set SAA7105 Sync Mode
 *
 *  \param  enableBT656Sync [In] Flag to enable or disable BT656 sync
 *
 *  \return  Success    EDC_SUCCESS
 *           Error      EDC_FAILED
 */
static Int32 setSAA7105SyncMode(Bool enableBT656Sync)
{
    Uint8 *charPtr = (Uint8 *)&_SAA7105_settingsDef;
    Int32 status = EDC_SUCCESS;

    EDC_DEBUG("Entered ---> setSAA7105SyncMode\n");
    EDC_DEBUG1("enableBT656Sync = %d\n", enableBT656Sync);

    if (enableBT656Sync)
    {
        _SAA7105_settingsDef.syncVia656 = 1;
    }
    else
    {
        _SAA7105_settingsDef.syncVia656 = 0;
    }
    status = _IIC_write(_SAA7105_IICADDR,
                        SAA7105_INPUT_PORT_CTRL_1_ADDR,
                        &charPtr[SAA7105_INPUT_PORT_CTRL_1_ADDR],
                        1);
    status |= _IIC_write(_SAA7105_IICADDR,
                        SAA7105_IP_DELAY_MSB_LINE_LENGTH_ADDR,
                        &charPtr[SAA7105_IP_DELAY_MSB_LINE_LENGTH_ADDR],
                        1);
    status |= _IIC_write(_SAA7105_IICADDR,
                        SAA7105_HORIZ_OFFSET_ADDR,
                        &charPtr[SAA7105_HORIZ_OFFSET_ADDR],
                        1);
    status |= _IIC_write(_SAA7105_IICADDR,
                        SAA7105_MSBS_ADDR,
                        &charPtr[SAA7105_MSBS_ADDR],
                        1);
    status |= _IIC_write(_SAA7105_IICADDR,
                         SAA7105_TRIGGER_CTRL_ADDR1,
                         &charPtr[SAA7105_TRIGGER_CTRL_ADDR1],
                         1);
    status |= _IIC_write(_SAA7105_IICADDR,
                         SAA7105_TRIGGER_CTRL_ADDR2,
                         &charPtr[SAA7105_TRIGGER_CTRL_ADDR2],
                         1);

    EDC_DEBUG("Exited ---> setSAA7105SyncMode\n");

    return status;
}



/**
 *  \brief  Set Video Display Mode
 *
 *  \param  displayType      [In]  SAA7105 Video Mode
 *  \param  outMode          [In]  SAA7105 analog format
 *  \param  inFormat         [In]  SAA7105 input format
 *  \param  enableBT656Sync  [In]  Flag to enable or disable BT656 sync
 *
 *  \return  Success    EDC_SUCCESS
 *           Error      EDC_FAILED
 */
static Int32 setVideoDisplay(SAA7105_Mode displayType,
                            SAA7105_AnalogFormat outMode,
                            SAA7105_InputFormat inFormat,
                            Bool enableBT656Sync)
{
    Uint8 *charPtr;
    Uns activePixels, activeLines;
    Uns hLen;
    Uns hStart, hEnd, fal, lal;
    volatile _SAA7105_Regs *regPtr = (_SAA7105_Regs *)&_SAA7105_settingsDef;
    Uns yOffset;
    Uns xOffset;
    Uns yInc = 0;
    Uns ySkip = 0;
    Uns yIWGTO = 100;
    Uns yIWGTE = 100;
    Uns xInc = 0;
    volatile Int32 i;
    Int32 status = EDC_SUCCESS;

    EDC_DEBUG("Entered ---> setVideoDisplay\n");
    EDC_DEBUG2("SAA7105_Mode = %d, SAA7105_AnalogFormat = %d, ",
               displayType,
               outMode);
    EDC_DEBUG2("SAA7105_InputFormat = %d, enableBT656Sync = %d\n",
               inFormat,
               enableBT656Sync);

    /* Validate all parameters */
    /* Validate Mode */
    if ((displayType < SAA7105_MODE_NTSC720) ||
        (displayType > SAA7105_MODE_PAL720))
    {
        status = EDC_FAILED;
    }

    /* Validate Analog Format */
    if ((outMode < SAA7105_AFMT_SVIDEO) || (outMode > SAA7105_AFMT_COMPOSITE))
    {
        status = EDC_FAILED;
    }

    /* Validate Input Format */
    if (inFormat != SAA7105_IFMT_YCBCR422_INTERLACED)
    {
        status = EDC_FAILED;
    }

    if (EDC_SUCCESS == status)
    {
        hStart = 272;
        xOffset = modeTbl[displayType][0];
        yOffset = modeTbl[displayType][1];
        hLen = modeTbl[displayType][2];;
        activePixels = modeTbl[displayType][3];
        activeLines = modeTbl[displayType][4];
        regPtr->inputFormat = inFormat;
        regPtr->pixClock0 = modeTbl[displayType][5];
        regPtr->pixClock1 = modeTbl[displayType][6];
        regPtr->pixClock2 = modeTbl[displayType][7];

        regPtr->chromaBinary = 1;
        regPtr->compSyncEnable = 0;
        regPtr->encOff = 1;
        regPtr->idel = 1;
        fal = 18;

        if (SAA7105_IFMT_YCBCR422_INTERLACED == regPtr->inputFormat)
        {
            /* 27MHz input clock (input mode 4) */
            xInc = 2048;
            regPtr->edge = 1;
            regPtr->slot = 0;
            regPtr->encOff = 0;
            regPtr->idel = 3;
            regPtr->compSyncEnable = 0;
        }

        if (SAA7105_MODE_PAL720 == displayType)
        {
            regPtr->palEnable = 1;
            regPtr->fise = 0;
            regPtr->burstStart = 0x21;
            regPtr->burstEnd = 0x1d;
            regPtr->chromaPhase = 0x6b;
            regPtr->burstAmplitude = 0x2d;
            regPtr->subCarrier0 = 0xCB;
            regPtr->subCarrier1 = 0x8A;
            regPtr->subCarrier2 = 0x09;
            regPtr->subCarrier3 = 0x2A;
            regPtr->hTriggerMSBs = 0;
            regPtr->hTrigger = 2;

            if (enableBT656Sync)
            {
                regPtr->vTrigNegative = 0;
                hStart = 280;
                xOffset = 0x04;
                regPtr->syncVia656 = 1;
                fal = 21;
                regPtr->vTrigger = 0;
            }
            else
            {
                hStart = 284;
                regPtr->syncVia656 = 0;
                fal = 21;
                regPtr->vTrigger = 2;
            }

            regPtr->lumaDelay = 0;
            regPtr->cbGain = 0x7b;
            regPtr->crGain = 0xAe;
        }
        else if (SAA7105_MODE_NTSC720 == displayType)
        {
            regPtr->hTriggerMSBs = 0;
            regPtr->hTrigger = 2;
            regPtr->palEnable = 0;
            regPtr->fise = 1;
            regPtr->burstStart = 0x19;
            regPtr->burstEnd = 0x1d;

            if (enableBT656Sync)
            {
                regPtr->vTrigNegative = 1;
                regPtr->vTrigger = 4;
                fal = 13;
                activeLines += 3;
                xOffset = 0x10;
                regPtr->syncVia656 = 1;
            }
            else
            {
                regPtr->vTrigNegative = 0;
                regPtr->vTrigger = 0;
                fal = 17;
                regPtr->syncVia656 = 0;
            }
            regPtr->lumaDelay = 0;
            regPtr->chromaPhase = 0x33;
            regPtr->burstAmplitude = 0x3F;
            regPtr->subCarrier0 = 0x1F;
            regPtr->subCarrier1 = 0x7C;
            regPtr->subCarrier2 = 0xF0;
            regPtr->subCarrier3 = 0x21;
            regPtr->cbGain = 0x7b;
            regPtr->crGain = 0xAe;
        }

        regPtr->hLen = hLen;
        regPtr->hLenMsb = hLen >> 8;
        regPtr->xInc = xInc;
        regPtr->xIncMsb = xInc >> 8;
        regPtr->yInc = yInc;
        regPtr->yIncMsb = yInc >> 8;
        regPtr->yIWGTO = yIWGTO;
        regPtr->yIWGTOMsb = yIWGTO >> 8;
        regPtr->yIWGTE = yIWGTE;
        regPtr->yIWGTEMsb = yIWGTE >> 8;
        regPtr->ySkip = ySkip;
        regPtr->ySkipMsb = ySkip >> 8;

        hEnd = hStart + (2 * activePixels);
        regPtr->hStart = hStart;
        regPtr->hEnd = hEnd;
        regPtr->hStartMSB = hStart >> 8;
        regPtr->hEndMSB = hEnd >> 8;

        regPtr->xPix = activePixels;
        regPtr->xPixMsb = activePixels >> 8;
        regPtr->yPix = activeLines;
        regPtr->yPixMsb = activeLines >> 8;

        lal = fal + activeLines;
        regPtr->fal = fal;
        regPtr->falMsb = fal >> 8;
        regPtr->lal = lal;
        regPtr->lalMsb = lal >> 8;

        regPtr->xOfs = xOffset;
        regPtr->xOfsMsb = xOffset >> 8;
        regPtr->yOfsOdd = yOffset;
        regPtr->yOfsOddMsb = yOffset >> 8;
        regPtr->yOfsEven = yOffset;
        regPtr->yOfsEvenMsb = yOffset >> 8;

        charPtr = (Uint8 *)&_SAA7105_settingsDef;
        status = _IIC_write(_SAA7105_IICADDR, 1, &charPtr[1], 0xA3);
        status |= _IIC_write(_SAA7105_IICADDR,
                             SAA7105_INPUT_PATH_CTRL_ADDR2,
                             &charPtr[SAA7105_INPUT_PATH_CTRL_ADDR2],
                             1);
        if (EDC_SUCCESS == status)
        {
            status = setOutputMode(outMode);
        }
    } /* if (EDC_SUCCESS == status) */

    EDC_DEBUG("Exited ---> setVideoDisplay\n");

    return status;
}



/**
 *  \brief  SAA7105 Close
 *
 *  \param  handle [IN]  External device channel handle
 *
 *  \return  Success    EDC_SUCCESS
 *           Error      EDC_FAILED
 */
static Int32 SAA7105_close(EDC_Handle handle)
{
    Int32 devId = (Int32)handle;
    Int32 status = EDC_SUCCESS;

    EDC_DEBUG("\n\nEntered ---> SAA7105_close\n");
    EDC_DEBUG1("EDC_Handle = 0x%0.8x\n", handle);

    /*First Check if the Handle is correct */
    if (_SAA7105_IICADDR == devId)
    {
        /*Put _SAA7105 in power down mode */
        status = powerdownSAA7105(TRUE);
        if (EDC_SUCCESS == status)
        {
            if (EDC_SUCCESS == i2c_deInit()) /*close I2C channel*/
            {
                status = EDC_SUCCESS;
            }
            else
            {
                status = EDC_FAILED;
            }
        }
    }
    else
    {
        status = EDC_FAILED;
    }

    EDC_DEBUG("Exited ---> SAA7105_close\n\n");

    return status;
}



/**
 *  \brief  SAA7105 Ioctls
 *
 *  \param  handle [IN]  External device channel handle
 *  \param  cmd    [IN]  IOCTL to be performed (mostly enum)
 *  \param  param  [IN]  Argument as required by Ioctl
 *
 *  \return  Success    EDC_SUCCESS
 *           Error      EDC_FAILED
 */
static Int32 SAA7105_ctrl(EDC_Handle handle, SAA7105_Cmd cmd, Arg param)
{
    Int32 devId = (Int32)handle;
    Int32 status = EDC_SUCCESS;

    EDC_DEBUG("\n\nEntered ---> SAA7105_ctrl\n");
    EDC_DEBUG2("EDC_Handle = 0x%0.8X, Arg = 0x%0.8X\n", handle, param);

    /* First Check if the Handle is correct */
    if (_SAA7105_IICADDR != devId)
    {
        status = EDC_FAILED;
    }
    else
    {
        switch (cmd)
        {
            case EDC_CONFIG:
            EDC_DEBUG1("Command: EDC_CONFIG(0x%0.8X)\n", cmd);
            if (NULL != param)
            {
                SAA7105_ConfParams *saa7105Params = (SAA7105_ConfParams *)param;

                status = configSAA7105(saa7105Params->mode,
                                       saa7105Params->enableSlaveMode,
                                       saa7105Params->aFmt,
                                       saa7105Params->iFmt,
                                       saa7105Params->enableBT656Sync);
                if (EDC_SUCCESS == status)
                {
                    status = setSAA7105SyncMode(saa7105Params->enableBT656Sync);
                }
            }
            else
            {
                status = EDC_FAILED;
            }
            break;

            case EDC_RESET:
            EDC_DEBUG1("Command: EDC_RESET(0x%0.8X)\n", cmd);
            status = resetSAA7105();
            break;

            case SAA7105_POWERDOWN:
            EDC_DEBUG1("Command: SAA7105_POWERDOWN(0x%0.8X)\n", cmd);
            status = powerdownSAA7105(TRUE);
            break;

            case SAA7105_POWERUP:
            EDC_DEBUG1("Command: SAA7105_POWERUP(0x%0.8X)\n", cmd);
            status = powerdownSAA7105(FALSE);
            break;

            case SAA7105_ENABLE_COLORBAR:
            EDC_DEBUG1("Command: SAA7105_ENABLE_COLORBAR(0x%0.8X)\n", cmd);
            status = enableColorBars((Bool)param);
            break;

            default:
            EDC_DEBUG1("Command: default(0x%0.8X)\n", cmd);
            status = EDC_FAILED;
            break;
        }
    }

    EDC_DEBUG("Exited ---> SAA7105_ctrl\n\n");

    return status;
}



/**
 *  \brief  SAA7105 device open
 *
 *  \param  devName [IN]  External device name
 *  \param  optArg  [IN]  Optional argument (not used here)
 *
 *  \return  handle  EDC channel handle when success
 *                   EDC_FAILED when unsuccessful
 */
static EDC_Handle SAA7105_open(String devName, Arg optArg)
{
    EDC_Handle handle = (EDC_Handle)NULL;
    Int32 status = EDC_SUCCESS;

    EDC_DEBUG("\n\nEntered ---> SAA7105_open\n");
    EDC_DEBUG2("devName = %s, Arg = 0x%0.8X\n", devName, optArg);

    if (EDC_SUCCESS == i2c_init()) /*open I2C channel*/
    {
        status = resetSAA7105();
        if (EDC_SUCCESS == status)
        {
            handle = (EDC_Handle)_SAA7105_IICADDR;
        }
    }

    EDC_DEBUG("Exited ---> SAA7105_open\n\n");

    return handle;
}


/******************************************************************************/
/* End of file                                                                */
/******************************************************************************/
