/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2007 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \file ths8200.c
 *
 *  This file contains interface functions for THS8200 encoder
 */

/*
 * ============ INCLUDE FILES ==========
 */
#include <std.h>
#include <ti/sdo/pspdrivers/drivers/vport/edc.h>

#ifndef I2C_DISABLE
#include <ti/sdo/pspdrivers/pal_os/pal_osWait.h>
#endif /* I2C_DISABLE */

#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/src/psp_i2c_interface.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/ths8200.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/src/_ths8200.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/src/edc_debug.h>


/*
 * ========== FUNCTION DECLARATIONS =========
 */
static Int32 THS8200_setCSC(Int32 devAddr,
                            THS8200_InputFormat iFmt,
                            THS8200_AnalogFormat aFmt);
static Int32 THS8200_setInputFormat(Int32 devAddr,
                                    THS8200_Mode mode,
                                    THS8200_InputFormat iFmt);
static Int32 THS8200_setVideoMode(Int32 devAddr, THS8200_Mode mode);
static Int32 THS8200_setOpMode(Int32 devAddr,
                               THS8200_Mode mode,
                               Bool enableSlaveMode);
static Int32 THS8200_setEmbeddedSync(Int32 devAddr, Bool enableBT656Sync);
static Int32 THS8200_setAnalogFormat(Int32 devAddr, THS8200_AnalogFormat aFmt);
static Int32 THS8200_colorbar(Int32 devAddr, THS8200_ColorbarParams *);
static Int32 THS8200_config(Int32 devAddr, THS8200_ConfParams *);
static Int32 THS8200_powerdown(Int32 devAddr, Bool enable);
static Int32 THS8200_reset(Int32 devAddr);
static Int32 THS8200_enableBypass(Int32 devAddr, Bool bypassEnable);

static Int32        THS8200_ctrl(EDC_Handle, THS8200_Cmd, Arg);
static Int32        THS8200_close(EDC_Handle);
static EDC_Handle   THS8200_open(Char *, Arg);


/*
 * ========== FUNCTION POINTER TABLE =========
 */
/** THS8200 EDC function table */
EDC_Fxns THS8200_Fxns =
{
    THS8200_open,
    THS8200_close,
    THS8200_ctrl
};

#ifdef ENABLE_EDC_DEBUG
/** THS8200 register copy */
Uint8 THS8200_Regs[256] =
{
    0
};
#endif /* #ifdef ENABLE_EDC_DEBUG */


/**
 *  \brief  I2C write register
 *
 *  This function updates register array with I2C write(only in debugging mode)
 *
 *  \param  slaveAddr  [In] slave address of I2C device
 *  \param  subAddr    [In] sub address of slave on which write
 *                          operation to be performed
 *  \param  buffer     [In] buffer pointer to write values
 *  \param  length     [In] number of bytes to write
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED on failure of write operation
 */
inline static Int32 regSetTHS8200(Uint8 slaveAddr,
                                  Uint8 subAddr,
                                  const Uint8 *buffer,
                                  Uint32 length)
{
#ifdef ENABLE_EDC_DEBUG
    Int32 count = 0;

    for (count = 0; count < length; count++)
    {
        THS8200_Regs[subAddr + count] = buffer[count];
    }

#endif /* ENABLE_EDC_DEBUG */

    return (i2c_writeReg(slaveAddr, subAddr, buffer, length));
}



/**
 *  \brief  Set Colour Space Conversion Registers
 *
 *  \param  devAddr [In]  Slave address of THS8200
 *  \param  iFmt    [In]  Digital input format
 *  \param  aFmt    [In]  Analog output format
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED if error occurred
 */
static Int32 THS8200_setCSC(Int32 devAddr,
                            THS8200_InputFormat iFmt,
                            THS8200_AnalogFormat aFmt)
{
    Int32 status = EDC_SUCCESS;
    Uint8 inColourSpace, outColourSpace;
    const _THS8200_CSC_Regs *pCscRegSet = NULL;

    EDC_DEBUG("\nEntered ---> THS8200_setCSC\n");
    EDC_DEBUG2("devAddr = 0x%0.8x , THS8200_AnalogFormat = 0x%0.8x\n",
               devAddr,
               aFmt);

    switch (iFmt)
    {
        case THS8200_IFMT_RGB444:
        case THS8200_IFMT_RGB565:
        case THS8200_IFMT_RGB555:
            inColourSpace = COLOUR_SPACE_RGB;
        break;

        case THS8200_IFMT_YCBCR422_10_BIT:
        case THS8200_IFMT_YCBCR422_20_BIT:
        case THS8200_IFMT_YCBCR444:
            inColourSpace = COLOUR_SPACE_YCBCR;
        break;

        default:
            status = EDC_FAILED;
        break;
    } /* switch (iFmt) */


    switch (aFmt)
    {
        case THS8200_AFMT_RGB_NO_SYNC:
        case THS8200_AFMT_RGB_SYNC_ON_GREEN:
        case THS8200_AFMT_RGB_SYNC_ON_ALL:
            outColourSpace = COLOUR_SPACE_RGB;
        break;

        case THS8200_AFMT_YPBPR_SYNC_ON_Y:
        case THS8200_AFMT_YPBPR_SYNC_ON_ALL:
            outColourSpace = COLOUR_SPACE_YCBCR;
        break;

        default:
            status = EDC_FAILED;
        break;
    } /* switch (aFmt) */

    if (EDC_SUCCESS == status)
    {
        if (inColourSpace == outColourSpace)
        {
            /* No conversion required */
            pCscRegSet = &ths8200_CSC_Disable_Regs;
        }
        else if (COLOUR_SPACE_RGB == inColourSpace)
        {
            /* RGB to YCbCr conversion is required */
            pCscRegSet = &ths8200_CSC_RGB_To_YCbCr_Regs;
        }
        else
        {
            /* YCbCr to RGB conversion is required */
            pCscRegSet = &ths8200_CSC_YCbCr_To_RGB_Regs;
        }

        status = regSetTHS8200(devAddr,
                               THS8200_CSC_R11_ADDR,
                               &(pCscRegSet->csc_r11),
                               THS8200_CSC_NUM_REGISTERS);
    }

    EDC_DEBUG("\nExited ---> THS8200_setCSC\n");

    return status;
}



/**
 *  \brief  Set input format
 *
 *  \param  devAddr [In]  Slave address of THS8200
 *  \param  mode    [In]  THS8200 video mode
 *  \param  iFmt    [In]  Digital input format
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED if error occurred
 */
static Int32 THS8200_setInputFormat(Int32 devAddr,
                                    THS8200_Mode mode,
                                    THS8200_InputFormat iFmt)
{
    Int32 status = EDC_SUCCESS;
    Uint8 numBytes = 1;
    Uint8 dataPathCtrl = 0;

    EDC_DEBUG("\nEntered ---> THS8200_setInputFormat\n");
    EDC_DEBUG2("devAddr = 0x%0.8x , THS8200_InputFormat = 0x%0.8x\n",
               devAddr,
               iFmt);

    status = i2c_readReg(devAddr,
                         THS8200_DATA_CNTL_ADDR,
                         &dataPathCtrl,
                         numBytes);

    dataPathCtrl &= ~THS8200_DATA_DMAN_CTL_MASK; /* Clear DMAN control bits */
    switch (iFmt)
    {
        case THS8200_IFMT_YCBCR422_10_BIT:
            dataPathCtrl |= THS8200_DATA_DMAN_CTL_10BIT_YCBCR_VAL;
            dataPathCtrl &= ~THS8200_DATA_IFIR12_BYPASS_BIT;
        break;

        case THS8200_IFMT_YCBCR422_20_BIT:
            dataPathCtrl |= THS8200_DATA_DMAN_CTL_20BIT_YCBCR_VAL;
            dataPathCtrl &= ~THS8200_DATA_IFIR12_BYPASS_BIT;
        break;

        case THS8200_IFMT_YCBCR444:
        case THS8200_IFMT_RGB444:
            dataPathCtrl |= THS8200_DATA_DMAN_CTL_30BIT_VAL;
            dataPathCtrl |= THS8200_DATA_IFIR12_BYPASS_BIT;
        break;

        case THS8200_IFMT_RGB565:
            dataPathCtrl |= THS8200_DATA_DMAN_CTL_16BIT_RGB_VAL;
            dataPathCtrl |= THS8200_DATA_IFIR12_BYPASS_BIT;
        break;

        case THS8200_IFMT_RGB555:
            dataPathCtrl |= THS8200_DATA_DMAN_CTL_15BIT_RGB_VAL;
            dataPathCtrl |= THS8200_DATA_IFIR12_BYPASS_BIT;
        break;

        default:
            status = EDC_FAILED;
        break;
    } /* switch (iFmt) */

    if (status != EDC_FAILED)
    {
        /* Set the other bits in DMAN Control register */
        dataPathCtrl |= THS8200_DATA_FSADJ_BIT;
        dataPathCtrl &= ~THS8200_DATA_IFIR35_BYPASS_BIT;

        /* Configure digital input format */
        status = regSetTHS8200(devAddr,
                               THS8200_DATA_CNTL_ADDR,
                               &dataPathCtrl,
                               numBytes);
    }

    EDC_DEBUG("\nExited ---> THS8200_setInputFormat\n");

    return status;
}



/**
 *  \brief  Set video mode
 *
 *  \param  devAddr [In]  Slave address of THS8200
 *  \param  mode    [In]  THS8200 video mode
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED if error occurred
 */
static Int32 THS8200_setVideoMode(Int32 devAddr, THS8200_Mode mode)
{
    Int32 status = EDC_SUCCESS;
    Uint8 numBytes = 1;
    const _THS8200_Regs *pRegSet = NULL;

    EDC_DEBUG("\nEntered ---> THS8200_setVideoMode\n");
    EDC_DEBUG2("devAddr = 0x%0.8x , Mode = 0x%0.8x\n", devAddr, mode);

    /* Select register set according to video format */
    switch (mode)
    {
        case THS8200_MODE_480P_60HZ:
            pRegSet = &ths8200_480p_60Hz_Regs;
        break;

        case THS8200_MODE_720P_60HZ:
            pRegSet = &ths8200_720p_60Hz_Regs;
        break;

        case THS8200_MODE_720P_50HZ:
            pRegSet = &ths8200_720p_50Hz_Regs;
        break;

        case THS8200_MODE_1080I_60HZ:
            pRegSet = &ths8200_1080i_60Hz_Regs;
        break;

        case THS8200_MODE_1080I_50HZ:
            pRegSet = &ths8200_1080i_50Hz_Regs;
        break;

        case THS8200_MODE_VGA_60HZ:
            pRegSet = &ths8200_VGA_60Hz_Regs;
        break;

        case THS8200_MODE_VGA_72HZ:
            pRegSet = &ths8200_VGA_72Hz_Regs;
        break;

        case THS8200_MODE_VGA_75HZ:
            pRegSet = &ths8200_VGA_75Hz_Regs;
        break;

        case THS8200_MODE_VGA_85HZ:
            pRegSet = &ths8200_VGA_85Hz_Regs;
        break;

        case THS8200_MODE_SVGA_60HZ:
            pRegSet = &ths8200_SVGA_60Hz_Regs;
        break;

        case THS8200_MODE_SVGA_72HZ:
            pRegSet = &ths8200_SVGA_72Hz_Regs;
        break;

        case THS8200_MODE_SVGA_75HZ:
            pRegSet = &ths8200_SVGA_75Hz_Regs;
        break;

        case THS8200_MODE_SVGA_85HZ:
            pRegSet = &ths8200_SVGA_85Hz_Regs;
        break;

        case THS8200_MODE_XGA_60HZ:
            pRegSet = &ths8200_XGA_60Hz_Regs;
        break;

        case THS8200_MODE_XGA_70HZ:
            pRegSet = &ths8200_XGA_70Hz_Regs;
        break;

        case THS8200_MODE_SXGA_60HZ:
            pRegSet = &ths8200_SXGA_60Hz_Regs;
        break;

        default:
            status = EDC_FAILED;
        break;
    }

    /* Configure video mode specific registers */
    if (status != EDC_FAILED)
    {
        status = regSetTHS8200(devAddr,
                               THS8200_CHIP_CTL_ADDR,
                               &(pRegSet->chip_cntl),
                               numBytes);

        status |= regSetTHS8200(devAddr,
                                THS8200_TST_CNTL1_ADDR,
                                &(pRegSet->tst_cntl1),
                                THS8200_TST_CNTL1_ADDR_CNT);

        status |= regSetTHS8200(devAddr,
                                THS8200_DTG1_Y_SYNC1_LSB_ADDR,
                                &(pRegSet->dtg_y_sync1),
                                THS8200_DTG1_Y_SYNC1_LSB_ADDR_CNT);

        status |= regSetTHS8200(devAddr,
                                THS8200_CSM_GY_CNTL_MULT_MSB_ADDR,
                                &(pRegSet->csm_gy_cntl_mult_msb),
                                numBytes);

        status |= regSetTHS8200(devAddr,
                                THS8200_CSM_RCR_BCB_CNTL_ADDR,
                                &(pRegSet->csm_rcr_bcb_cntl),
                                numBytes);

        status |= regSetTHS8200(devAddr,
                                THS8200_DTG2_HLENGTH_LSB_ADDR,
                                &(pRegSet->dtg2_hlength_lsb),
                                THS8200_DTG2_HLENGTH_LSB_ADDR_CNT);

        status |= regSetTHS8200(devAddr,
                                THS8200_DTG2_CNTL_ADDR,
                                &(pRegSet->dtg2_cntl),
                                numBytes);
    }

    EDC_DEBUG("\nExited ---> THS8200_setVideoMode\n");

    return status;
}



/**
 *  \brief  Set operation mode like master mode or slave mode
 *
 *  \param  devAddr         [In]  Slave address of THS8200
 *  \param  mode            [In]  THS8200 video mode
 *  \param  enableSlaveMode [In]  Flag to enable or disable slave mode
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED if error occurred
 */
static Int32 THS8200_setOpMode(Int32 devAddr,
                               THS8200_Mode mode,
                               Bool enableSlaveMode)
{
    Uint8 chipCtrl = 0;
    Uint8 dtg1Mode = 0;
    Int32 status = EDC_SUCCESS;
    Uint8 numBytes = 1;

    EDC_DEBUG("\nEntered ---> THS8200_setOpMode\n");
    EDC_DEBUG2("EDC_Handle = 0x%0.8x , enableSlaveMode = 0x%0.8x\n",
               devAddr,
               enableSlaveMode);

    /* Read the previous value */
    status = i2c_readReg(devAddr, THS8200_CHIP_CTL_ADDR, &chipCtrl, numBytes);
    status |= i2c_readReg(devAddr, THS8200_DTG1_MODE_ADDR, &dtg1Mode, numBytes);

    /* Configure operation mode */
    if (EDC_SUCCESS == status)
    {
        if (enableSlaveMode)
        {
            /* Slave mode set bit1 = 0 for slave mode */
            chipCtrl &= ~THS8200_SYSCTRL_CHIP_MS_BIT ;

            if ((mode >= THS8200_MODE_VGA_60HZ) &&
                (mode <= THS8200_MODE_SXGA_85HZ))
            {
                /* Mask lower nibble */
                dtg1Mode &= ~THS8200_DTG1_MODE_MASK;
                /* Assigning the value for slave mode */
                dtg1Mode |= THS8200_DTG1_MODE_VESA_SLAVE_VAL;
            }
        }
        else
        {
            /* Master mode can be enabled only in VESA modes */
            if ((mode >= THS8200_MODE_VGA_60HZ) &&
                (mode <= THS8200_MODE_SXGA_85HZ))
            {
                /* Set bit 1 = TRUE for master mode */
                chipCtrl |= THS8200_SYSCTRL_CHIP_MS_BIT;
                /* Mask lower nibble with 0 */
                dtg1Mode &= ~THS8200_DTG1_MODE_MASK;
                /* Assigning the value for master mode */
                dtg1Mode |= THS8200_DTG1_MODE_VESA_MASTER_VAL;
            }
            else
            {
                status = EDC_FAILED;
            }
        }

        /* configure operation mode specific registers */
        status |= regSetTHS8200(devAddr,
                                THS8200_CHIP_CTL_ADDR,
                                &chipCtrl,
                                numBytes);
        status |= regSetTHS8200(devAddr,
                                THS8200_DTG1_MODE_ADDR,
                                &dtg1Mode,
                                numBytes);
    }

    EDC_DEBUG("\nExited ---> THS8200_setOpMode\n");

    return status;
}



/**
 *  \brief  Set registers for embedded sync mode
 *
 *  \param  devAddr         [In]  Slave address of THS8200
 *  \param  enableBT656Sync [In]  Flag to enable or disable BT656 sync
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED if error occurred
 */
static Int32 THS8200_setEmbeddedSync(Int32 devAddr, Bool enableBT656Sync)
{
    Uint8 dtg2Ctrl = 0;
    Int32 status = EDC_SUCCESS;
    Uint8 numBytes = 1;

    EDC_DEBUG("\nEntered ---> THS8200_setEmbeddedSync\n");
    EDC_DEBUG2("devAddr = 0x%0.8x , enableBT656Sync = 0x%0.8x\n",
               devAddr,
               enableBT656Sync);
    /* Read the previous value */
    status = i2c_readReg(devAddr, THS8200_DTG2_CNTL_ADDR, &dtg2Ctrl, numBytes);
    if (enableBT656Sync)
    {
        /* bit 5 = TRUE for embedded sync */
        dtg2Ctrl |= THS8200_DTG2_EMBEDDED_SYNC_BIT;
    }
    else
    {
        /* bit 5 = FALSE for external sync*/
        dtg2Ctrl &= ~THS8200_DTG2_EMBEDDED_SYNC_BIT;
    }

    if (EDC_SUCCESS == status)
    {
        /* Configure DTG2 control register */
        status |= regSetTHS8200(devAddr,
                                THS8200_DTG2_CNTL_ADDR,
                                &dtg2Ctrl,
                                numBytes);
    }

    EDC_DEBUG("\nExited ---> THS8200_setEmbeddedSync\n");

    return status;
}



/**
 *  \brief  Set analog output mode
 *
 *  \param  devAddr [In]  Slave address of THS8200
 *  \param  aFmt    [In]  THS8200 analog output mode
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED if error occurred
 */
static Int32 THS8200_setAnalogFormat(Int32 devAddr, THS8200_AnalogFormat aFmt)
{
    Int32 status = EDC_SUCCESS;
    Uint16 ySyncLowVal = 0, ySyncHighVal = 0;
    Uint16 cbcrSyncLowVal = 0, cbcrSyncHighVal = 0;
    Uint8 outColourSpace = 0;
    Uint8 lowByte = 0, highByte = 0;
    Uint8 dtg2Ctrl = 0;
    Uint8 numBytes = 1;

    EDC_DEBUG("\nEntered ---> THS8200_setAnalogFormat\n");
    EDC_DEBUG2("devAddr = 0x%0.8x , THS8200_AnalogFormat = 0x%0.8x\n",
               devAddr,
               aFmt);

    switch (aFmt)
    {
        case THS8200_AFMT_RGB_NO_SYNC:
            ySyncLowVal = THS8200_NO_SYNC_VAL;
            ySyncHighVal = THS8200_NO_SYNC_VAL;
            cbcrSyncLowVal = THS8200_NO_SYNC_VAL;
            cbcrSyncHighVal = THS8200_NO_SYNC_VAL;

            outColourSpace = COLOUR_SPACE_RGB;
        break;

        case THS8200_AFMT_RGB_SYNC_ON_GREEN:
            ySyncLowVal = THS8200_SYNC_ON_Y_LOW;
            ySyncHighVal = THS8200_SYNC_ON_Y_HIGH;
            cbcrSyncLowVal = THS8200_NO_SYNC_VAL;
            cbcrSyncHighVal = THS8200_NO_SYNC_VAL;

            outColourSpace = COLOUR_SPACE_RGB;
        break;

        case THS8200_AFMT_RGB_SYNC_ON_ALL:
            ySyncLowVal = THS8200_SYNC_ON_Y_LOW;
            ySyncHighVal = THS8200_SYNC_ON_Y_HIGH;
            cbcrSyncLowVal = THS8200_SYNC_ON_CBCR_LOW;
            cbcrSyncHighVal = THS8200_SYNC_ON_CBCR_HIGH;

            outColourSpace = COLOUR_SPACE_RGB;
        break;

        case THS8200_AFMT_YPBPR_SYNC_ON_Y:
            ySyncLowVal = THS8200_SYNC_ON_Y_LOW;
            ySyncHighVal = THS8200_SYNC_ON_Y_HIGH;
            cbcrSyncLowVal = THS8200_NO_SYNC_VAL;
            cbcrSyncHighVal = THS8200_NO_SYNC_VAL;

            outColourSpace = COLOUR_SPACE_YCBCR;
        break;

        case THS8200_AFMT_YPBPR_SYNC_ON_ALL:
            ySyncLowVal = THS8200_SYNC_ON_Y_LOW;
            ySyncHighVal = THS8200_SYNC_ON_Y_HIGH;
            cbcrSyncLowVal = THS8200_SYNC_ON_CBCR_LOW;
            cbcrSyncHighVal = THS8200_SYNC_ON_CBCR_HIGH;

            outColourSpace = COLOUR_SPACE_YCBCR;
        break;

        default:
            status = EDC_FAILED;
        break;
    } /* switch (aFmt) */

    /* Configure analog output format */
    if (EDC_SUCCESS == status)
    {
        /* Write Y Sync Low LSB */
        lowByte = (Uint8)(ySyncLowVal & THS8200_AFMT_LOWER_BYTE_MASK);
        status |= regSetTHS8200(devAddr,
                                THS8200_DTG1_Y_SYNC2_LSB_ADDR,
                                &lowByte,
                                numBytes);

        /* Write Y Sync High LSB */
        lowByte = (Uint8)(ySyncHighVal & THS8200_AFMT_LOWER_BYTE_MASK);
        status |= regSetTHS8200(devAddr,
                                THS8200_DTG1_Y_SYNC3_LSB_ADDR,
                                &lowByte,
                                numBytes);

        /* Write CbCr Sync Low LSB */
        lowByte = (Uint8)(cbcrSyncLowVal & THS8200_AFMT_LOWER_BYTE_MASK);
        status |= regSetTHS8200(devAddr,
                                THS8200_DTG1_CBCR_SYNC2_LSB_ADDR,
                                &lowByte,
                                numBytes);

        /* Write CbCr Sync High LSB */
        lowByte = (Uint8)(cbcrSyncHighVal & THS8200_AFMT_LOWER_BYTE_MASK);
        status |= regSetTHS8200(devAddr,
                                THS8200_DTG1_CBCR_SYNC3_LSB_ADDR,
                                &lowByte,
                                numBytes);


        /* Update Y Sync Low and High MSB */
        status |= i2c_readReg(devAddr,
                              THS8200_DTG1_Y_SYNC_MSB_ADDR,
                              &highByte,
                              numBytes);

        highByte &= THS8200_AFMT_HIGHER_BYTE_READ_MASK;
        /* write (((ySyncLowVal & 0300) >> 6) || ((ySyncHighVal & 0300) >> 8)))
         * to bit 0 to 3 of 0x23
         */
        highByte |=
            (Uint8)(((ySyncLowVal & THS8200_AFMT_HIGHER_BYTE_MASK)
            >> THS8200_AFMT_LOW_VAL_SHIFT)|
            ((ySyncHighVal & THS8200_AFMT_HIGHER_BYTE_MASK)
            >> THS8200_AFMT_HIGH_VAL_SHIFT));

        status |= regSetTHS8200(devAddr,
                                THS8200_DTG1_Y_SYNC_MSB_ADDR,
                                &highByte,
                                numBytes);


        /* Update CbCr Sync Low and High MSB */
        status |= i2c_readReg(devAddr,
                              THS8200_DTG1_CBCR_SYNC_MSB_ADDR,
                              &highByte,
                              numBytes);

        highByte &= THS8200_AFMT_HIGHER_BYTE_READ_MASK;

        /* Write (((cbcrSyncLowVal & 0300)>>6)|((cbcrSyncHighVal & 0300)>>8)))
         to bit 0 to 3 of 0x24 */
        highByte |=
            (Uint8)(((cbcrSyncLowVal & THS8200_AFMT_HIGHER_BYTE_MASK)
            >> THS8200_AFMT_LOW_VAL_SHIFT)|
            ((cbcrSyncHighVal & THS8200_AFMT_HIGHER_BYTE_MASK)
            >> THS8200_AFMT_HIGH_VAL_SHIFT));

        status |= regSetTHS8200(devAddr,
                                THS8200_DTG1_CBCR_SYNC_MSB_ADDR,
                                &highByte,
                                numBytes);


        /* Read value of dtg2_cntl to set the Analog colour format */
        status |= i2c_readReg(devAddr,
                              THS8200_DTG2_CNTL_ADDR,
                              &dtg2Ctrl,
                              numBytes);

        if (COLOUR_SPACE_RGB == outColourSpace) /* RGB mode */
        {
            /* Set RGB output mode */
            dtg2Ctrl |= THS8200_DTG2_RGB_MODE_ON_BIT;
        }
        else
        {
            /* Set YPbPr output mode */
            dtg2Ctrl &= ~THS8200_DTG2_RGB_MODE_ON_BIT;
        }

        /* Configure RGB or YPBPR format in dtg2_cntl */
        status |= regSetTHS8200(devAddr,
                                THS8200_DTG2_CNTL_ADDR,
                                &dtg2Ctrl,
                                numBytes);
    }

    EDC_DEBUG("\nExited ---> THS8200_setAnalogFormat\n");

    return status;
}



/**
 *  \brief  Enable or disable colorbars
 *
 *  \param  devAddr   [In]  Slave address of THS8200
 *  \param  cbParams  [In]  Colorbar parameter structure pointer
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED if error occurred
 */
static Int32 THS8200_colorbar(Int32 devAddr, THS8200_ColorbarParams *cbParams)
{
    Int32 status = EDC_SUCCESS;
    Uint8 dtg2Ctrl = 0;
    Uint8 barWidth = cbParams->colorbarWidth;
    Uint8 enableColorbar = 0;
    Uint8 numBytes = 1;

    EDC_DEBUG("\nEntered ---> THS8200_colorbar\n");
    EDC_DEBUG2("devAddr = 0x%0.8x , cbParams = 0x%0.8x\n", devAddr, cbParams);

    status = i2c_readReg(devAddr, THS8200_DTG2_CNTL_ADDR, &dtg2Ctrl, numBytes);

    /* bit 6 = 0 for YUV modes and 1 for RGB modes
     * colorbar generation is only available with RGB(VESA) modes
     */
    if ((dtg2Ctrl & THS8200_AFMT_RGB_MODE_VAL) && (EDC_SUCCESS == status))
    {
        if (cbParams->enableColorbar)
        {
            status = i2c_readReg(devAddr,
                                 THS8200_CHIP_CTL_ADDR,
                                 &enableColorbar,
                                 numBytes);

            /* enable colorbar only if THS8200 is configured in Master Mode */
            if (enableColorbar & THS8200_SYSCTRL_CHIP_MS_BIT)
            {
                /* bit 5 = 1 colorbar on */
                enableColorbar |= THS8200_SYSCTRL_VESA_COLORBARS_BIT;

                status |= regSetTHS8200(devAddr,
                                        THS8200_CHIP_CTL_ADDR,
                                        &enableColorbar,
                                        numBytes);

                status |= regSetTHS8200(devAddr,
                                        THS8200_DTG1_VESA_CBAR_SIZE_ADDR,
                                        &barWidth,
                                        numBytes);
            }
            else
            {
                status = EDC_FAILED;
            }
        }
        else /* disable colorbars */
        {
            status = i2c_readReg(devAddr,
                                 THS8200_CHIP_CTL_ADDR,
                                 &enableColorbar,
                                 numBytes);

            /* bit 5 = 0 colorbar off */
            enableColorbar &= ~THS8200_SYSCTRL_VESA_COLORBARS_BIT;
            status |= regSetTHS8200(devAddr,
                                    THS8200_CHIP_CTL_ADDR,
                                    &enableColorbar,
                                    numBytes);
        }
    }
    else
    {
        status = EDC_FAILED;
    }

    EDC_DEBUG("\nExited ---> THS8200_colorbar\n");

    return status;
}



/**
 *  \brief  Configure registers of THS8200
 *
 *  \param  devAddr  [In]  Slave address of THS8200
 *  \param  params   [In]  THS8200 configuration parameter structure pointer
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED if error occurred
 */
static Int32 THS8200_config(Int32 devAddr, THS8200_ConfParams *params)
{
    Int32 status = EDC_SUCCESS;

    EDC_DEBUG("\nEntered ---> THS8200_config\n");
    EDC_DEBUG2("devAddr = 0x%0.8x , THS8200_ConfParams = 0x%0.8x\n",
               devAddr,
               params);

    /* Configure video mode */
    status = THS8200_setVideoMode(devAddr, params->mode);

    if (EDC_SUCCESS == status)
    {
        /* Config Colour Space Conversion (CSC) */
        status = THS8200_setCSC(devAddr, params->iFmt, params->aFmt);
    }

    if (EDC_SUCCESS == status)
    {
        /* Configure input format */
        status = THS8200_setInputFormat(devAddr, params->mode, params->iFmt);
    }

    if (EDC_SUCCESS == status)
    {
        status = THS8200_setAnalogFormat(devAddr, params->aFmt);
    }

    if (EDC_SUCCESS == status)
    {
        status = THS8200_setOpMode(devAddr, params->mode,
                                   params->enableSlaveMode);
    }

    if (EDC_SUCCESS == status)
    {
        status = THS8200_setEmbeddedSync(devAddr, params->enableBT656Sync);
    }

    /* Reset chip so that THS8200 synchronize with the new input format */
    status |= THS8200_reset(devAddr);

    EDC_DEBUG("\nExited ---> THS8200_config\n");

    return status;
}



/**
 *  \brief  Enable or disable bypass mode of THS8200
 *
 *  \param  devAddr       [In]  Slave address of THS8200
 *  \param  bypassEnable  [In]  Flag to enable or disable bypass mode
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED if error occurred
 */
static Int32 THS8200_enableBypass(Int32 devAddr, Bool bypassEnable)
{
    Int32 status = EDC_SUCCESS;
    Uint8 dataPathCtrl = 0;
    Uint8 numBytes = 1;

    EDC_DEBUG("\nEntered ---> THS8200_enableBypass\n");
    EDC_DEBUG2("devAddr = 0x%0.8x , bypassEnable = 0x%0.8x\n",
               devAddr,
               bypassEnable);

    status = i2c_readReg(devAddr,
                         THS8200_DATA_CNTL_ADDR,
                         &dataPathCtrl,
                         numBytes);
    if (EDC_SUCCESS == status)
    {
        if (bypassEnable)
        {
            /* Clear mode */
            dataPathCtrl &= ~(THS8200_DATA_DMAN_CTL_MASK |
                              THS8200_DATA_TRISTATE656_BIT);
            /* Enable bypass and set 10-bit mode */
            dataPathCtrl |= THS8200_DATA_DMAN_CTL_10BIT_YCBCR_VAL;

            /* Enable output clock */
            dataPathCtrl |= THS8200_DATA_CLK656_ON_BIT;
        }
        else
        {
            /* Disable bypass */
            dataPathCtrl |= THS8200_DATA_TRISTATE656_BIT;

            /* Disable output clock */
            dataPathCtrl &= ~THS8200_DATA_CLK656_ON_BIT;
        }

        status |= regSetTHS8200(devAddr,
                                THS8200_DATA_CNTL_ADDR,
                                &dataPathCtrl,
                                numBytes);
    }

    EDC_DEBUG("\nExited ---> THS8200_enableBypass\n");

    return status;
}



/**
 *  \brief  Power Up or Power Down THS8200
 *
 *  \param  devAddr  [In]  Slave address of THS8200
 *  \param  enable   [In]  Flag to power-up or power-down THS8200
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED if error occurred
 */
static Int32 THS8200_powerdown(Int32 devAddr, Bool enable)
{
    Int32 status = EDC_SUCCESS;
    Uint8 chipCtrl = 0;
    Uint8 numBytes = 1;

    EDC_DEBUG("\nEntered ---> THS8200_powerdown\n");
    EDC_DEBUG2("devAddr = 0x%0.8x , enable = 0x%0.1x\n", devAddr, enable);

    /* First Check if the Handle is correct */
    if (devAddr != THS8200_I2CADDR)
    {
        status = EDC_FAILED;
    }
    else
    {
        /* Read chip control register to set/reset the powerdown bits */
        status = i2c_readReg(devAddr,
                             THS8200_CHIP_CTL_ADDR,
                             &chipCtrl,
                             numBytes);

        if (enable)
        {
            /* Put THS8200 in power down mode */
            chipCtrl |= (THS8200_SYSCTRL_DAC_PWDN_BIT |
                         THS8200_SYSCTRL_CHIP_PWDN_BIT);
        }
        else
        {
            /* Put THS8200 in power up mode */
            chipCtrl &= ~(THS8200_SYSCTRL_DAC_PWDN_BIT |
                          THS8200_SYSCTRL_CHIP_PWDN_BIT);
            chipCtrl |= THS8200_SYSCTRL_ARST_FUNC_N_BIT;
        }

        /* Update the register */
        if (status != EDC_FAILED)
        {
            status = regSetTHS8200(devAddr,
                                   THS8200_CHIP_CTL_ADDR,
                                   &chipCtrl,
                                   numBytes);
        }
    }

    EDC_DEBUG("\nExited ---> THS8200_powerdown\n");

    return status;
}



/**
 *  \brief  Reset THS8200
 *
 *  \param  devAddr   [In]  Slave address of THS8200
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED if error occurred
 */
static Int32 THS8200_reset(Int32 devAddr)
{
    Int32 status = EDC_SUCCESS;
    Uint8 chipCtrl;
    Uint8 numBytes = 1;

    EDC_DEBUG("\nEntered ---> THS8200_reset\n");
    EDC_DEBUG1("devAddr = 0x%0.8x \n", devAddr);

    /* Read chip control register to set the software reset bit */
    status |= i2c_readReg(devAddr, THS8200_CHIP_CTL_ADDR, &chipCtrl, numBytes);

    /* Enable software reset bit (Set 0) */
    chipCtrl &= ~THS8200_SYSCTRL_ARST_FUNC_N_BIT;
    status |= regSetTHS8200(devAddr,
                            THS8200_CHIP_CTL_ADDR,
                            &chipCtrl,
                            numBytes);

#ifndef I2C_DISABLE
    /* Wait for 1ms, for soft reset to complete,
     * since that is the min we can give via pal_os
     */
    PAL_osWaitMsecs(1);
#endif /* I2C_DISABLE */

    /* Disable software reset bit (Set 1 for normal operation) */
    chipCtrl |= THS8200_SYSCTRL_ARST_FUNC_N_BIT;
    status |= regSetTHS8200(devAddr,
                            THS8200_CHIP_CTL_ADDR,
                            &chipCtrl,
                            numBytes);

    EDC_DEBUG("\nExited ---> THS8200_reset\n");

    return status;
}



/**
 *  \brief  THS8200 Close
 *
 *  \param  handle [IN]  External device channel handle
 *
 *  \return  Success    EDC_SUCCESS
 *           Error      EDC_FAILED
 */
static Int32 THS8200_close(EDC_Handle handle)
{
    Int32 status = EDC_SUCCESS;

    EDC_DEBUG("\nEntered ---> THS8200_close\n");
    EDC_DEBUG1("EDC_Handle = 0x%0.8x \n", handle);

    /* First Check if the Handle is correct */
    if (THS8200_I2CADDR == (Int32)handle)
    {
        /* Put THS8200 in power down mode, all dac and digital logic except
           I2C */
        status |= THS8200_powerdown((Int32)handle, TRUE);
    }
    else
    {
        status = EDC_FAILED;
    }

    if (EDC_SUCCESS == status)
    {
        if (i2c_deInit() != EDC_SUCCESS)
        {
            status = EDC_FAILED;
        }
    }

    EDC_DEBUG("\nExited ---> THS8200_close\n");

    return status;
}



/**
 *  \brief  THS8200 Ioctls
 *
 *  \param  handle [IN]  External device channel handle
 *  \param  cmd    [IN]  IOCTL to be performed (mostly enum)
 *  \param  arg1   [IN]  Argument as required by Ioctl
 *
 *  \return  Success    EDC_SUCCESS
 *           Error      EDC_FAILED
 */
static Int32 THS8200_ctrl(EDC_Handle handle, THS8200_Cmd cmd, Arg arg1)
{
    Int32 status = EDC_SUCCESS;
    Int32 devAddr = (Int32)handle;

    EDC_DEBUG("\nEntered ---> THS8200_ctrl\n");
    EDC_DEBUG2("EDC_Handle = 0x%0.8x , cmd = 0x%0.8x ,", handle, cmd);
    EDC_DEBUG1("Arg = 0x%0.8x\n", arg1);

    /* First Check if the Handle is correct */
    if (THS8200_I2CADDR == (Int32)handle)
    {
        switch (cmd)
        {
            case EDC_CONFIG:
                if (NULL != arg1)
                {
                    status = THS8200_config(devAddr, (void*)arg1);
                }
                else
                {
                    status = EDC_FAILED;
                }
            break;

            case EDC_RESET:
                status = THS8200_reset(devAddr);
            break;

            case THS8200_POWERDOWN:
                status = THS8200_powerdown(devAddr, TRUE);
            break;

            case THS8200_POWERUP:
                status = THS8200_powerdown(devAddr, FALSE);
            break;

            case THS8200_ENABLE_COLORBAR:
                if (NULL != arg1)
                {
                    status = THS8200_colorbar(devAddr,
                                              (THS8200_ColorbarParams *)arg1);
                }
                else
                {
                    status = EDC_FAILED;
                }
            break;

            case THS8200_ENABLE_BYPASS:
                status = THS8200_enableBypass(devAddr, (Bool)arg1);
            break;

            default:
                status = EDC_FAILED;
            break;
        }
    }
    else
    {
        status = EDC_FAILED;
    }

    EDC_DEBUG("\nExited ---> THS8200_ctrl\n");

    return status;
}



/**
 *  \brief  THS8200 device open
 *
 *  \param  devName [IN]  External device name
 *  \param  optArg  [IN]  Optional argument (not used here)
 *
 *  \return handle  EDC channel handle when success
 *                  EDC_FAILED when unsuccessful
 */
static EDC_Handle THS8200_open(Char *devName, Arg optArg)
{
    Int32 status = EDC_SUCCESS;
    Int32 devId = NULL;

    EDC_DEBUG("\nEntered ---> THS8200_open\n");
    EDC_DEBUG2("devName = %s , Arg = 0x%0.8x\n", devName, optArg);

    /* Open I2C device */
    if (EDC_SUCCESS == i2c_init())
    {
        devId = THS8200_I2CADDR;
        /* Power up the device */
        status = THS8200_powerdown(devId, FALSE);
    }
    else
    {
        status = EDC_FAILED;
    }

    if (EDC_FAILED == status)
    {
        devId = NULL;
    }

    EDC_DEBUG("\nExited ---> THS8200_open\n");

    return ((EDC_Handle)(devId));
}


/******************************************************************************/
/* End of file                                                                */
/******************************************************************************/
