/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2007 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \file tvp5154.c
 *
 *  This file contains interface functions for TVP5154 decoder
 */

/*
 * ========= INCLUDE FILES ==========
 */
#include <std.h>

#include <ti/sdo/pspdrivers/drivers/vport/edc.h>

#ifndef I2C_DISABLE
#include <ti/sdo/pspdrivers/pal_os/pal_osprotect.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osWait.h>
#endif /* I2C_DISABLE */

#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/src/psp_i2c_interface.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/tvp5154.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/src/_tvp5154.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/src/edc_debug.h>

/*
 * ======== VARIABLE DECLARATION ===============
 */
/** Local structure array to contain channel objects, semaphore handles
 * and channel open count
 */
static _TVP5154_decObj decObj[NUM_DECODERS] =
{
{_TVP5154_1_IIC_ADDR, NULL, 0, {{0, 1}, {0, 2}, {0, 4}, {0, 8}}},
{_TVP5154_2_IIC_ADDR, NULL, 0, {{1, 1}, {1, 2}, {1, 4}, {1, 8}}}
};

/**
 *  \brief  TVP5154 default configuration parameters Structure
 *  Structure to contain TVP5154 default configuration setting values
 */
static TVP5154_ConfParams TVP5154_default =
{
    TVP5154_MODE_NTSC,
    /**< NTSC operation defayult */
    TVP5154_AFMT_COMPOSITE_A,
    /**< Composite channel A selection default */
    TRUE
    /**< BT656 sync enabled */
};


/*
 * ============= FUNCTION DECLARATIONS ===============
 */
static Int32 TVP5154_config(_TVP5154_chanObj *, TVP5154_ConfParams *);
static Int32 TVP5154_powerdown(_TVP5154_chanObj *, Bool);
static Int32 TVP5154_reset(_TVP5154_chanObj *);
static Int32 TVP5154_ctrl(EDC_Handle, TVP5154_Cmd, Arg);
static Int32 TVP5154_close(EDC_Handle);
static EDC_Handle TVP5154_open(String, Arg);
static Int32 TVP5154_setAnalogChan(_TVP5154_chanObj *, Int32);

/*
 * ========== FUNCTION TABLE ========================
 */
/** TVP5154 EDC function table */
EDC_Fxns TVP5154_Fxns =
{
    TVP5154_open,
    TVP5154_close,
    TVP5154_ctrl
};


/**
 *  \brief  I2C write register
 *
 *  This function updates register array with I2C write(only in debugging mode)
 *
 *  \param  chan       [In] TVP5154 channel object
 *  \param  slaveAddr  [In] slave address of I2C device
 *  \param  subAddr    [In] sub address of slave on which write
 *                           operation to be performed
 *  \param  buffer     [In] buffer pointer to write values
 *  \param  length     [In] number of bytes to write
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED on failure of write operation
 */
inline static Int32 regSetTVP5154(_TVP5154_chanObj *chan,
                                  Uint8 slaveAddr,
                                  Uint8 subAddr,
                                  Uint8 *buffer,
                                  Uint32 length)
{
#ifdef ENABLE_EDC_DEBUG
    Int32 count = 0;

    for (count = 0; count < length; count++)
    {
        chan->regSpace[subAddr] = *(buffer + count);
    }

#endif /* ENABLE_EDC_DEBUG */
    return (i2c_writeReg(slaveAddr, subAddr, buffer, length));
}



/**
 *  \brief  Configure registers of TVP5154
 *
 *  \param  chan    [In]  TVP5154 channel parameters
 *  \param  params  [In]  TVP5154 configuration parameter structure pointer
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED if error occurred
 */
static Int32 TVP5154_config(_TVP5154_chanObj *chan, TVP5154_ConfParams *params)
{
    Uint8 addrI2C = 0;
    Uint8 numBytes = 1;
    Uint8 decoderChan = 0;
    Uint8 externalSync = TVP5154_EMBEDDED_SYNC; /* Embedded sync default */
    Uint8 hsync_start = TVP5154_HSYNC_START_NTSC;   /* Default to NTSC */
    Uint8 mode = TVP5154_MODE_VAL_AUTOSWITCH;       /* Default to Auto Switch */
    Uint8 analogFmt = TVP5154_AFMT_VAL_COMPOSITE_A;    /* COMPOSITE-A default */
    Uint8 outputEn = TVP5154_EN_YCBCR_CLK;   /* enable output YCbCr and clock */
    Uint8 pedestalCfg = TVP5154_7_5_IRE_PEDESTAL_MASK; /* 7.5 IRE Pedestal */
    Int32 configStatus = EDC_SUCCESS;
    TVP5154_ConfParams *TVP5154Params = params;

    EDC_DEBUG("Entered ---> TVP5154_config\n");
    EDC_DEBUG2("_TVP5154_chanObj = 0x%0.8X , TVP5154_ConfParams = 0x%0.8X\n",
               (Int32)chan,
               params);

    decoderChan = chan->chanMask;
    addrI2C = decObj[chan->decNum].addrI2C;

    if (TVP5154_MODE_PAL == TVP5154Params->mode)
    {
        hsync_start = TVP5154_HSYNC_START_PAL;  /* sync start value for PAL */
    }

    if (FALSE == (TVP5154Params->enableBT656Sync))
    {
        externalSync = TVP5154_EXTERNAL_SYNC; /* external sync enable */
    }

    if (TVP5154_AFMT_SVIDEO == TVP5154Params->aFmt)
    {
        analogFmt = TVP5154_AFMT_VAL_SVIDEO;
    }
    else if (TVP5154_AFMT_COMPOSITE_B == TVP5154Params->aFmt)
    {
        analogFmt = TVP5154_AFMT_VAL_COMPOSITE_B;
    }

    /* enable specified decoder channel from available four channel to write
     * via I2C
     */
    configStatus |= regSetTVP5154(chan,
                                  addrI2C,
                                  TVP5154_DECODER_WRITE_ENABLES_ADDR,
                                  &decoderChan,
                                  numBytes);
    configStatus |= regSetTVP5154(chan,
                                  addrI2C,
                                  TVP5154_INPUT_SRC_SEL_1_ADDR,
                                  &analogFmt,
                                  numBytes);
    configStatus |= regSetTVP5154(chan,
                                  addrI2C,
                                  TVP5154_MISC_CTRL_ADDR,
                                  &outputEn,
                                  numBytes);
    configStatus |= regSetTVP5154(chan,
                                  addrI2C,
                                  TVP5154_OUTPUTS_DATA_RATES_SEL_ADDR,
                                  &externalSync,
                                  numBytes);
    configStatus |= regSetTVP5154(chan,
                                  addrI2C,
                                  TVP5154_HORIZ_SYNC_START_ADDR,
                                  &hsync_start,
                                  numBytes);
    configStatus |= regSetTVP5154(chan,
                                  addrI2C,
                                  TVP5154_LUMA_PROC_CTRL_1_ADDR,
                                  &pedestalCfg,
                                  numBytes);

    /* force to operate in selected or detected mode */
    configStatus |= regSetTVP5154(chan,
                                  addrI2C,
                                  TVP5154_VIDEO_STANDARD_ADDR,
                                  &mode,
                                  numBytes);

    decoderChan = 0;
    /* disable all decoder channel to access via I2C */
    configStatus |= regSetTVP5154(chan,
                                  addrI2C,
                                  TVP5154_DECODER_WRITE_ENABLES_ADDR,
                                  &decoderChan,
                                  numBytes);

    EDC_DEBUG("Exited ---> TVP5154_config\n");

    return configStatus;
}



/**
 *  \brief  Set analog input channel of TVP5154
 *
 *  \param  chan    [In]  TVP5154 channel parameters
 *  \param  chanNo  [In]  TVP5154 channel number
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED if error occurred
 */
static Int32 TVP5154_setAnalogChan(_TVP5154_chanObj *chan, Int32 chanNo)
{
    Uint8 analogFmt = TVP5154_AFMT_VAL_COMPOSITE_A; /* channel A default */
    Uint8 numBytes = 1;
    Uint8 addrI2C, decoderChan;
    Int32 retVal = EDC_SUCCESS;
    Uint8 readVal = 0;

    EDC_DEBUG("Entered ---> TVP5154_setAnalogChan\n");
    EDC_DEBUG2("_TVP5154_chanObj = 0x%0.8X, Int32 = 0x%0.8X\n", chan, chanNo);

    switch (chanNo)
    {
        case TVP5154_AFMT_CHANNEL_A:
            analogFmt = TVP5154_AFMT_VAL_COMPOSITE_A;
        break;

        case TVP5154_AFMT_CHANNEL_B:
            analogFmt = TVP5154_AFMT_VAL_COMPOSITE_B;
        break;

        default:
            retVal = EDC_FAILED;
        break;
    }

    if (EDC_SUCCESS == retVal)
    {
        decoderChan = chan->chanMask;
        addrI2C = decObj[chan->decNum].addrI2C;

        /* enable specified decoder channel from available four channel to read
         * via I2C
         */
        retVal |= regSetTVP5154(chan,
                                addrI2C,
                                TVP5154_DECODER_READ_ENABLES_ADDR,
                                &decoderChan,
                                numBytes);

        retVal |= i2c_readReg(addrI2C,
                              TVP5154_INPUT_SRC_SEL_1_ADDR,
                              &readVal,
                              numBytes);

        if ((TVP5154_AFMT_VAL_SVIDEO & readVal) && (EDC_SUCCESS == retVal))
        {
            retVal = EDC_FAILED;
        }
        else
        {
            /* enable/disable specified decoder channel from available four
             * channel to write via I2C
             */
            retVal |= regSetTVP5154(chan,
                                    addrI2C,
                                    TVP5154_DECODER_WRITE_ENABLES_ADDR,
                                    &decoderChan,
                                    numBytes);
            retVal |= regSetTVP5154(chan,
                                    addrI2C,
                                    TVP5154_INPUT_SRC_SEL_1_ADDR,
                                    &analogFmt,
                                    numBytes);

            decoderChan = 0;
            /* disable all decoder channel to access via I2C */
            retVal |= regSetTVP5154(chan,
                                    addrI2C,
                                    TVP5154_DECODER_WRITE_ENABLES_ADDR,
                                    &decoderChan,
                                    numBytes);
        }
    }

    EDC_DEBUG("Exited ---> TVP5154_setAnalogChan\n");

    return retVal;
}



/**
 *  \brief  Power Up or Power Down TVP5154
 *
 *  \param  chan             [In] TVP5154 channel parameters
 *  \param  powerDownEnable [In] Flag to power-up or power-down TVP5154
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED if error occurred
 */
static Int32 TVP5154_powerdown(_TVP5154_chanObj *chan, Bool powerDownEnable)
{
    Uint8 powerdownSettings = TVP5154_OP_MODE_POWER_OFF_VAL; /* off default */
    Uint8 addrI2C = 0;
    Uint8 numBytes = 1;
    Uint8 enableDecoder = TVP5154_RESERVED_POWER_ON_VAL;
    Int32 retVal = EDC_SUCCESS;
    Uint8 readVal = 0x00;
    Uint8 decoderChan = 0;

    EDC_DEBUG("Entered ---> TVP5154_powerdown\n");
    EDC_DEBUG2("_TVP5154_chanObj = 0x%0.8X, powerDownEnable = %d \n",
               chan,
               powerDownEnable);

    decoderChan = chan->chanMask;
    addrI2C = decObj[chan->decNum].addrI2C;

    /* Check for Power Up */
    if (!powerDownEnable)
    {
        powerdownSettings = TVP5154_OP_MODE_POWER_ON_VAL;

        /* enable specified decoder channel from available four channel to write
         * via I2C
         */
        retVal |= regSetTVP5154(chan,
                                addrI2C,
                                TVP5154_DECODER_WRITE_ENABLES_ADDR,
                                &decoderChan,
                                numBytes);
        retVal |= regSetTVP5154(chan,
                                addrI2C,
                                TVP5154_POWERUP_REG_ADDR,
                                &enableDecoder,
                                numBytes);

#ifndef I2C_DISABLE
        /* wait for 2ms for power up to complete
	     * Please refer the doc: SLES163A,page 19,
	     * section 8. Give an extra 1ms (worst case)
	     */
        PAL_osWaitMsecs(2);
#endif /* I2C_DISABLE */

        /* enable specified decoder channel from available four channel to read
         * via I2C
         */
        retVal |= regSetTVP5154(chan,
                                addrI2C,
                                TVP5154_DECODER_READ_ENABLES_ADDR,
                                &decoderChan,
                                numBytes);

        retVal |= i2c_readReg(addrI2C,
                              TVP5154_LSB_OF_DEVICE_ID_ADDR,
                              &readVal,
                              numBytes);

#ifndef I2C_DISABLE

        /* check weather the channel is on or off. Return error if off */
        if ((readVal != TVP5154_READ_VALUE) && (EDC_SUCCESS == retVal))
        {
            retVal = EDC_FAILED;
        }
#endif /* I2C_DISABLE */
    }

    /* enable/disable specified decoder channel from available four channel to
     * write via I2C
     */
    retVal |= regSetTVP5154(chan,
                            addrI2C,
                            TVP5154_DECODER_WRITE_ENABLES_ADDR,
                            &decoderChan,
                            numBytes);
    retVal |= regSetTVP5154(chan,
                            addrI2C,
                            TVP5154_OPERATION_MODE_CTRL_ADDR,
                            &powerdownSettings,
                            numBytes);

    decoderChan = 0;
    /* disable all decoder channel to access via I2C */
    retVal |= regSetTVP5154(chan,
                            addrI2C,
                            TVP5154_DECODER_WRITE_ENABLES_ADDR,
                            &decoderChan,
                            numBytes);

    EDC_DEBUG("Exited ---> TVP5154_powerdown\n");

    return retVal;
}



/**
 *  \brief  Reset TVP5154
 *
 *  \param  chan  [In]  TVP5154 channel parameters
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED if error occurred
 */
static Int32 TVP5154_reset(_TVP5154_chanObj *chan)
{
    /* Reset 5154 */
    return TVP5154_powerdown(chan, FALSE);
}



/**
 *  \brief  TVP5154 Close
 *
 *  \param  edcHandle [IN]  External device channel handle
 *
 *  \return  Success    EDC_SUCCESS
 *           Error      EDC_FAILED
 */
static Int32 TVP5154_close(EDC_Handle edcHandle)
{
    Int32 retVal = EDC_SUCCESS;
#ifndef I2C_DISABLE
    _TVP5154_chanObj *chan = (_TVP5154_chanObj *)edcHandle;
    Uint32 imask;
#endif

    EDC_DEBUG("\n\nEntered ---> TVP5154_close\n");
    EDC_DEBUG1("EDC_Handle = 0x%0.8X\n", edcHandle);

    /* First Check if the Handle is correct */
    if (NULL == edcHandle)
    {
        retVal = EDC_FAILED;
    }
    else
    {
        /* Put _TVP5154 in power down mode */
        retVal = TVP5154_ctrl(edcHandle, TVP5154_POWERDOWN, NULL);
        retVal |= i2c_deInit();

#ifndef I2C_DISABLE
        /* decrement decoder open count */
        PAL_osProtectEntry(PAL_OSPROTECT_SCHEDULER, &imask);

        if ((decObj[chan->decNum].chanOpenCnt) != 0)
        {
            (decObj[chan->decNum].chanOpenCnt)--;
        }

        /* delete semaphore */
        if (0 == (decObj[chan->decNum].chanOpenCnt))
        {
            PAL_osProtectExit(PAL_OSPROTECT_SCHEDULER, imask);
            retVal |= PAL_osSemDelete(decObj[chan->decNum].decChanSem);
            decObj[chan->decNum].decChanSem = NULL;
        }
        else
        {
            PAL_osProtectExit(PAL_OSPROTECT_SCHEDULER, imask);
        }
#endif /* I2C_DISABLE */
    }

    EDC_DEBUG("Exited ---> TVP5154_close\n\n");

    return retVal;
}



/**
 *  \brief  TVP5154 Ioctls
 *
 *  \param  edcHandle [IN]  External device channel handle
 *  \param  cmd       [IN]  IOCTL to be performed (mostly enum)
 *  \param  arg       [IN]  Argument as required by Ioctl
 *
 *  \return  Success    EDC_SUCCESS
 *           Error      EDC_FAILED
 */
static Int32 TVP5154_ctrl(EDC_Handle edcHandle, TVP5154_Cmd cmd, Arg arg)
{
    Int32 retVal = EDC_SUCCESS;

    EDC_DEBUG("\n\nEntered ---> TVP5154_ctrl\n");
    EDC_DEBUG2("EDC_Handle = 0x%0.8X, Arg = 0x%0.8X\n", edcHandle, arg);

    if (NULL == edcHandle)
    {
        retVal = EDC_FAILED;
    }
    else
    {
#ifndef I2C_DISABLE
        if (PAL_SOK !=
            PAL_osSemTake(
            decObj[((_TVP5154_chanObj *)edcHandle)->decNum].decChanSem,
            PAL_OSSEM_NO_TIMEOUT))
        {
            retVal = EDC_FAILED;
        }
#endif /* I2C_DISABLE */

        switch (cmd)
        {
            case EDC_CONFIG:
                EDC_DEBUG1("Command: EDC_CONFIG(0x%0.8X)\n", cmd);
                if (NULL != arg)
                {
                    retVal = TVP5154_config(edcHandle,
                                            (TVP5154_ConfParams *)arg);
                }
                else
                {
                    retVal = EDC_FAILED;
                }
            break;

            case EDC_RESET:
                EDC_DEBUG1("Command: EDC_RESET(0x%0.8X)\n", cmd);
                retVal = TVP5154_reset(edcHandle);
            break;

            case TVP5154_POWERDOWN:
                EDC_DEBUG1("Command: TVP5154_POWERDOWN(0x%0.8X)\n", cmd);
                retVal = TVP5154_powerdown(edcHandle, TRUE);
            break;

            case TVP5154_POWERUP:
                EDC_DEBUG1("Command: TVP5154_POWERUP(0x%0.8X)\n", cmd);
                retVal = TVP5154_powerdown(edcHandle, FALSE);
            break;

            case TVP5154_SET_ANALOG_CHAN:
                EDC_DEBUG1("Command: TVP5154_SET_ANALOG_CHAN(0x%0.8X)\n", cmd);
                retVal = TVP5154_setAnalogChan(edcHandle, arg);
            break;

            default:
                EDC_DEBUG1("Command: default(0x%0.8X)\n", cmd);
                retVal = EDC_FAILED;
            break;
        }

#ifndef I2C_DISABLE
        /* release the lock */
        if (PAL_SOK !=
            PAL_osSemGive(
           decObj[((_TVP5154_chanObj *)edcHandle)->decNum].decChanSem))
        {
            retVal = EDC_FAILED;
        }
#endif /* I2C_DISABLE */
    }

    EDC_DEBUG("Exited ---> TVP5154_ctrl\n\n");

    return retVal;
}



/**
 *  \brief  TVP5154 device open
 *
 *  \param  devName [IN]  External device name
 *  \param  optArg  [IN]  Optional argument (not used here)
 *
 *  \return  handle  EDC channel handle when success
 *                   EDC_FAILED when unsuccessful
 */
static EDC_Handle TVP5154_open(String devName, Arg optArg)
{
    _TVP5154_chanObj *handle = NULL;
    Int32 retVal = EDC_SUCCESS;
    Int32 chanNum = 0;
#ifndef I2C_DISABLE
    Uint32 imask;
#endif

    EDC_DEBUG("\n\nEntered ---> TVP5154_open\n");
    EDC_DEBUG2("devName = %s, Arg = 0x%0.8X\n", devName, optArg);

    /* Check if the address is correct */
    if (devName[0] != '/')
    {
        handle = (_TVP5154_chanObj *)NULL;
    }
    else
    {
        chanNum = (Int32)(devName[1] - '0');
        /* assigning channel object structure */
        handle = &(decObj[(chanNum /
                           NUM_CHANNELS)].chanObj[chanNum % NUM_CHANNELS]);
    }

    if ((Int32)handle != NULL)
    {
        if (i2c_init() != EDC_SUCCESS) /* open I2C channel */
        {
            handle = (_TVP5154_chanObj *)NULL;
        }

#ifndef I2C_DISABLE
        if (((Int32)handle != NULL) &&
            (NULL == decObj[handle->decNum].decChanSem))
        {
            /* Create the channel Semaphore */
            if (PAL_SOK != PAL_osSemCreate("channelSem",
                                           1,
                                           NULL,
                                           &decObj[handle->decNum].decChanSem))
            {
                i2c_deInit();
                handle = (_TVP5154_chanObj *)NULL;
            }
        }

#endif /* I2C_DISABLE */

        /* power on decoder */
        if ((Int32)handle != NULL)
        {
            if (EDC_SUCCESS != TVP5154_ctrl(handle, TVP5154_POWERUP, NULL))
            {
#ifndef I2C_DISABLE
                i2c_deInit();

                /* delete semaphore */
                if (0 == decObj[handle->decNum].chanOpenCnt)
                {
                    PAL_osSemDelete(decObj[handle->decNum].decChanSem);
                    decObj[handle->decNum].decChanSem = NULL;
                }
#endif /* I2C_DISABLE */
                handle = (_TVP5154_chanObj *)NULL;
            }
            else
            {
#ifndef I2C_DISABLE
                /* increment decoder open count */
                PAL_osProtectEntry(PAL_OSPROTECT_SCHEDULER, &imask);
                decObj[handle->decNum].chanOpenCnt++;
                PAL_osProtectExit(PAL_OSPROTECT_SCHEDULER, imask);
#endif /* I2C_DISABLE */
                if (EDC_SUCCESS != TVP5154_ctrl(handle,
                                                EDC_CONFIG,
                                                (Arg)&TVP5154_default))
                {
                    retVal = EDC_FAILED;
                }
            }
        }
    }

    EDC_DEBUG("Exited ---> TVP5154_open\n\n");
    if (EDC_FAILED == retVal)
    {
        handle = (_TVP5154_chanObj *)NULL;
    }

    return (EDC_Handle)handle;
}


/******************************************************************************/
/* End of file                                                                */
/******************************************************************************/
