/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2007 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \file tvp7000.c
 *
 *  This file contains interface functions for TVP7000 decoder
 */

/*
 * ======== Include Files ==========
 */
#include <std.h>

#include <ti/sdo/pspdrivers/drivers/vport/edc.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/src/psp_i2c_interface.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/tvp7000.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/src/_tvp7000.h>
#include <ti/sdo/pspdrivers/system/dm648/bios/evmDM648/video/src/edc_debug.h>


/*
 * ======== Configurable Macros ========
 */

/** Set this macro to 1 to enable THS7353 configuration. This should be done
 * in EVM prior to production EVM in which THS7353 filter is present.
 * Set this macro to 0 to disable THS7353 configuration. In production EVM,
 * THS7353 filter is removed, hence THS7353 configuration should be disabled.
 */
#define CFG_ENABLE_THS7353              (0)


/*
 * ======== Function Declarations ===========
 */
static Int32 TVP7000_config(TVP7000_ConfParams *);
static Int32 TVP7000_setMode(TVP7000_Mode);
static Int32 TVP7000_setOutFormat(TVP7000_OutputFormat oFmt);
static Int32 TVP7000_powerdown(Bool);
static Int32 TVP7000_reset(void);
static Int32 TVP7000_setDefault(void);
static Int32 TVP7000_setAnalogChan(TVP7000_AnalogChanParams *);

static Int32 TVP7000_ctrl(EDC_Handle, TVP7000_Cmd, Arg);
static Int32 TVP7000_close(EDC_Handle);
static EDC_Handle TVP7000_open(String, Arg);


/*
 * ========== FUNCTION POINTER TABLE ==========
 */
/** TVP7000 EDC function table */
EDC_Fxns TVP7000_Fxns =
{
    TVP7000_open,
    TVP7000_close,
    TVP7000_ctrl
};

#ifdef ENABLE_EDC_DEBUG
/** TVP7000 register copy */
Uint8 TVP7000_Regs[256] =
{
    0
};
#endif /* _ENABLE_EDC_DEBUG */


/**
 *  \brief  TVP7000 default configuration parameters Structure
 *  Structure to contain TVP7000 default configuration setting values
 */
static TVP7000_ConfParams TVP7000_default =
{
    TVP7000_MODE_1080I_60HZ,
    /**< 1080I 60Hz default */
    TVP7000_OFMT_16BIT_YCBCR_422
    /**< 16 bit YCbCr 422 default */
};


/**
 *  \brief  I2C write register
 *
 *  This function updates register array with I2C write(only in debugging mode)
 *
 *  \param  slaveAddr  [In] slave address of I2C device
 *  \param  subAddr    [In] sub address of slave on which write
 *                          operation to be performed
 *  \param  buffer     [In] buffer pointer to write values
 *  \param  length     [In] number of bytes to write
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED on failure of write operation
 */
inline static Int32 TVP7000_regSet(Uint8 slaveAddr,
                                   Uint8 subAddr,
                                   const Uint8 *buffer,
                                   Uint32 length)
{
#ifdef ENABLE_EDC_DEBUG

    Int32 count = 0;

    for (count = 0; count < length; count++)
    {
        TVP7000_Regs[subAddr + count] = buffer[count];
    }

#endif /* ENABLE_EDC_DEBUG */

    return (i2c_writeReg(slaveAddr, subAddr, buffer, length));
}



/**
 *  \brief  Configure registers of TVP7000 for the given mode
 *
 *  \param  TVP7000Params [In] TVP7000 configuration parameter structure pointer
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED if error occurred
 */
static Int32 TVP7000_config(TVP7000_ConfParams *TVP7000Params)
{
    Int32 status = EDC_SUCCESS;

    EDC_DEBUG("\nEntered ---> TVP7000_config\n");
    EDC_DEBUG1("TVP7000Params = 0x%0.8x\n", TVP7000Params);

    /* Configure Mode */
    status = TVP7000_setMode(TVP7000Params->mode);

    /* Configure Output Format */
    status |= TVP7000_setOutFormat(TVP7000Params->oFmt);

    EDC_DEBUG("\nExited ---> TVP7000_config\n");

    return status;
}



/**
 *  \brief  Sets the output digital format of TVP7000
 *
 *  \param  oFmt  [In]  TVP7000 output format
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED if error occurred
 */
static Int32 TVP7000_setOutFormat(TVP7000_OutputFormat oFmt)
{
    Int32 status = EDC_SUCCESS;
    Uint8 numBytes = 1;
    Uint8 addrI2C = _TVP7000_IIC_ADDR;
    Uint8 outFormater = 0;

    EDC_DEBUG("\nEntered ---> TVP7000_setOutFormat\n");
    EDC_DEBUG1("TVP7000_OutputFormat = 0x%0.8x\n", oFmt);

    /* Read output formatter register */
    status = i2c_readReg(addrI2C,
                         TVP7000_OUTPUT_FORMATER_ADDR,
                         &outFormater,
                         numBytes);

    if (TVP7000_OFMT_24BIT_RGB_YCBCR_444 == oFmt)
    {
        /* Set output digital format as 444 */
        outFormater &= ~TVP7000_OUT_FORMATER_444_422_BIT;
    }
    else
    {
        /* Set output digital format as 422 */
        outFormater |= TVP7000_OUT_FORMATER_444_422_BIT;
    }

    /* Update the register */
    if (status != EDC_FAILED)
    {
        status = TVP7000_regSet(addrI2C,
                                TVP7000_OUTPUT_FORMATER_ADDR,
                                &outFormater,
                                numBytes);
    }

    EDC_DEBUG("\nExited ---> TVP7000_setOutFormat\n");

    return status;
}



#if CFG_ENABLE_THS7353

/**
 *  \brief  Configure THS7353 filter
 *
 *  \param  mode  [In] video mode
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED if error occurred
 */
static Int32 TVP7000_configTHS7353(TVP7000_Mode mode)
{
    Uint8 addrI2C = _THS7353_IIC_ADDR, numBytes;
    Int32 status = EDC_SUCCESS;
    Uint8 filterChan1, filterChan2, filterChan3;

    EDC_DEBUG("\nEntered ---> TVP7000_configTHS7353\n");
    EDC_DEBUG1("mode = 0x%0.8x\n", mode);

    /* Select register set according to video format */
    switch (mode)
    {
        case TVP7000_MODE_480P_60HZ:
        case TVP7000_MODE_576P_50HZ:
            filterChan1 = THS7353_EDTV_Y_CHANNEL_DEF;
            filterChan2 = THS7353_EDTV_PB_CHANNEL_DEF;
            filterChan3 = THS7353_EDTV_PR_CHANNEL_DEF;
        break;

        case TVP7000_MODE_720P_60HZ:
        case TVP7000_MODE_720P_50HZ:
        case TVP7000_MODE_1080I_60HZ:
        case TVP7000_MODE_1080I_50HZ:
            filterChan1 = THS7353_HDTV_Y_CHANNEL_DEF;
            filterChan2 = THS7353_HDTV_PB_CHANNEL_DEF;
            filterChan3 = THS7353_HDTV_PR_CHANNEL_DEF;
        break;

        default:
            status = EDC_FAILED;
        break;
    }

    if (EDC_SUCCESS == status)
    {
        /* Configure THS7353 filter */
        numBytes = 1;
        status |= i2c_writeReg(addrI2C, THS7353_CHANNEL1_ADDR,
                               &filterChan1, numBytes);
        status |= i2c_writeReg(addrI2C, THS7353_CHANNEL2_ADDR,
                               &filterChan2, numBytes);
        status |= i2c_writeReg(addrI2C, THS7353_CHANNEL3_ADDR,
                               &filterChan3, numBytes);
    }

    EDC_DEBUG("\nExited ---> TVP7000_configTHS7353\n");

    return status;
}

#endif  /* #if CFG_ENABLE_THS7353 */



/**
 *  \brief  Set video mode
 *
 *  \param  mode  [In]  TVP7000 video mode
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED if error occurred
 */
static Int32 TVP7000_setMode(TVP7000_Mode mode)
{
    Uint8 addrI2C = _TVP7000_IIC_ADDR, numBytes;
    const _TVP7000_Regs *pRegSet = NULL;
    Int32 status = EDC_SUCCESS;

    EDC_DEBUG("\nEntered ---> TVP7000_setMode\n");
    EDC_DEBUG1("mode = 0x%0.8x\n", mode);

    /* Select register set according to video format */
    switch (mode)
    {
        case TVP7000_MODE_480P_60HZ:
            pRegSet = &tvp7000_480p_60Hz_Regs;
        break;

        case TVP7000_MODE_576P_50HZ:
            pRegSet = &tvp7000_576p_50Hz_Regs;
        break;

        case TVP7000_MODE_720P_60HZ:
            pRegSet = &tvp7000_720p_60Hz_Regs;
        break;

        case TVP7000_MODE_720P_50HZ:
            pRegSet = &tvp7000_720p_50Hz_Regs;
        break;

        case TVP7000_MODE_1080I_60HZ:
            pRegSet = &tvp7000_1080i_60Hz_Regs;
        break;

        case TVP7000_MODE_1080I_50HZ:
            pRegSet = &tvp7000_1080i_50Hz_Regs;
        break;

        default:
            status = EDC_FAILED;
        break;
    }

    if (EDC_SUCCESS == status)
    {
        /* Configure mode dependent registers */
        numBytes = PLL_REG_CNT;
        status = TVP7000_regSet(addrI2C,
                                TVP7000_PLL_DIVIDE_MSB_ADDR,
                                (Uint8 *)&(pRegSet->pll_divide_msb),
                                numBytes);

        numBytes = 1;
        status |= TVP7000_regSet(addrI2C,
                                 TVP7000_SYNC_CTRL_1_ADDR,
                                 (Uint8 *)&(pRegSet->sync_ctrl_1),
                                 numBytes);

        numBytes = 1;
        status |= TVP7000_regSet(addrI2C,
                                 TVP7000_SYNC_ON_G_THRLD_ADDR,
                                 (Uint8 *)&(pRegSet->sync_on_g_thrld),
                                 numBytes);

        numBytes = COAST_REG_CNT;
        status |= TVP7000_regSet(addrI2C,
                                 TVP7000_PRE_COAST_ADDR,
                                 (Uint8 *)&(pRegSet->pre_coast),
                                 numBytes);

        numBytes = 1;
        status |= TVP7000_regSet(addrI2C,
                                 TVP7000_HSOUT_OUTPUT_START_ADDR,
                                 (Uint8 *)&pRegSet->hsout_start,
                                 numBytes);

        numBytes = 1;
        status |= TVP7000_regSet(addrI2C,
                                 TVP7000_ALC_PLACEMENT_ADDR,
                                 (Uint8 *)&pRegSet->alc_placement,
                                 numBytes);

#if CFG_ENABLE_THS7353

        /* Configure THS7353 filter */
        status |= TVP7000_configTHS7353(mode);

#endif /* #if CFG_ENABLE_THS7353 */

    }

    EDC_DEBUG("\nExited ---> TVP7000_setMode\n");

    return status;
}



/**
 *  \brief  Set analog input channel of TVP7000
 *
 *  \param  inParams  [In]  TVP7000 analog channel parameters
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED if error occurred
 */
static Int32 TVP7000_setAnalogChan(TVP7000_AnalogChanParams *inParams)
{
    Int32 status = EDC_SUCCESS;
    Uint8 numBytes = 1;
    Uint8 addrI2C = _TVP7000_IIC_ADDR;
    Uint8 inMuxSel1 = 0, inMuxSel2 = TVP7000_IN_MUX_SEL2_DEF;

    EDC_DEBUG("\nEntered ---> TVP7000_setAnalogChan\n");
    EDC_DEBUG1("inParams = 0x%0.8x\n", inParams);

    inMuxSel1 = ((inParams->chanB & TVP7000_IN_MUX_SEL1_MASK)
              << TVP7000_IN_MUX_SEL1_BLUE_SHIFT);
    inMuxSel1 |= ((inParams->chanG & TVP7000_IN_MUX_SEL1_MASK)
               << TVP7000_IN_MUX_SEL1_GREEN_SHIFT);
    inMuxSel1 |= ((inParams->chanR & TVP7000_IN_MUX_SEL1_MASK)
               << TVP7000_IN_MUX_SEL1_RED_SHIFT);
    inMuxSel1 |= ((inParams->chanSOG & TVP7000_IN_MUX_SEL1_MASK)
               << TVP7000_IN_MUX_SEL1_SOG_SHIFT);

    inMuxSel2 |= ((inParams->chanHsync & TVP7000_IN_MUX_SEL2_MASK)
               << TVP7000_IN_MUX_SEL2_HSYNC_SHIFT);
    inMuxSel2 |= ((inParams->chanVsync & TVP7000_IN_MUX_SEL2_MASK)
               << TVP7000_IN_MUX_SEL2_VSYNC_SHIFT);

    /* Configure Input Mux Select 1 and 2 register */
    status = TVP7000_regSet(addrI2C,
                            TVP7000_INPUT_MUX_SEL_1_ADDR,
                            &inMuxSel1,
                            numBytes);

    status |= TVP7000_regSet(addrI2C,
                             TVP7000_INPUT_MUX_SEL_2_ADDR,
                             &inMuxSel2,
                             numBytes);

    EDC_DEBUG("\nExited ---> TVP7000_setAnalogChan\n");

    return status;
}



/**
 *  \brief  Power Up or Power Down TVP7000
 *
 *  \param  powerDownEnable [In]  Flag to power-up or power-down TVP7000
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED if error occurred
 */
static Int32 TVP7000_powerdown(Bool powerDownEnable)
{
    Int32 status = EDC_SUCCESS;
    Uint8 addrI2C = _TVP7000_IIC_ADDR;
    Uint8 powerdownSettings = 0;
    Uint8 numBytes = 1;

    EDC_DEBUG("\nEntered ---> TVP7000_powerdown\n");
    EDC_DEBUG1("powerDownEnable = 0x%0.8x\n", powerDownEnable);

    /* Read pll and clamp control register to set/reset the powerdown bits */
    status = i2c_readReg(addrI2C,
                         TVP7000_PLL_AND_CLAMP_CTRL_ADDR,
                         &powerdownSettings,
                         numBytes);

    if (powerDownEnable)
    {
        /* Put TVP7000 in power down mode */
        powerdownSettings &= ~TVP7000_CHIP_POWER_DOWN_BIT;
    }
    else
    {
        /* Put TVP7000 in power up mode */
        powerdownSettings |= TVP7000_CHIP_POWER_DOWN_BIT;
    }

    /* Update the register */
    if (status != EDC_FAILED)
    {
        status = TVP7000_regSet(addrI2C,
                                TVP7000_PLL_AND_CLAMP_CTRL_ADDR,
                                &powerdownSettings,
                                numBytes);
    }

    EDC_DEBUG("\nExited ---> TVP7000_powerdown\n");

    return status;
}



/**
 *  \brief  Reset TVP7000
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED if error occurred
 */
static Int32 TVP7000_reset(void)
{
    EDC_DEBUG("\nEntered ---> TVP7000_reset\n");
    EDC_DEBUG("\nCalling ---> TVP7000_powerdown\n");

    return TVP7000_powerdown(FALSE);
}



/**
 *  \brief  Sets the registers of TVP7000 to default value
 *
 *  \return EDC_SUCCESS on successful operation
 *          EDC_FAILED if error occurred
 */
static Int32 TVP7000_setDefault(void)
{
    Int32 status = EDC_SUCCESS;
    Uint8 addrI2C = _TVP7000_IIC_ADDR;
    Uint8 numBytes = 1;

    EDC_DEBUG("\nEntered ---> TVP7000_setDefault\n");

    numBytes = 10;
    status |= TVP7000_regSet(addrI2C,
                             TVP7000_B_FINE_GAIN_ADDR,
                             &tvp7000_default[0],
                             numBytes);

    numBytes = 2;
    status |= TVP7000_regSet(addrI2C,
                             TVP7000_OUTPUT_FORMATER_ADDR,
                             &tvp7000_default[10],
                             numBytes);

    numBytes = 10;
    status |= TVP7000_regSet(addrI2C,
                             TVP7000_INPUT_MUX_SEL_1_ADDR,
                             &tvp7000_default[12],
                             numBytes);

    numBytes = 1;
    status |= TVP7000_regSet(addrI2C,
                             TVP7000_AUTO_LVL_CNT_ENABLE_ADDR,
                             &tvp7000_default[22],
                             numBytes);

    numBytes = 1;
    status |= TVP7000_regSet(addrI2C,
                             TVP7000_AUTO_LVL_CNT_FILTER_ADDR,
                             &tvp7000_default[23],
                             numBytes);

    numBytes = 5;
    status |= TVP7000_regSet(addrI2C,
                             TVP7000_FINE_CLAMP_CTRL_ADDR,
                             &tvp7000_default[24],
                             numBytes);

    EDC_DEBUG("\nExited ---> TVP7000_setDefault\n");

    return status;
}



/**
 *  \brief  TVP7000 Close
 *
 *  \param  edcHandle [IN]  External device channel handle
 *
 *  \return  Success    EDC_SUCCESS
 *           Error      EDC_FAILED
 */
static Int32 TVP7000_close(EDC_Handle edcHandle)
{
    Int32 status = EDC_SUCCESS;

    EDC_DEBUG("\nEntered ---> TVP7000_close\n");
    EDC_DEBUG1("EDC_Handle = 0x%0.8x\n", edcHandle);

    /* First Check if the Handle is correct */
    if (_TVP7000_IIC_ADDR != (Int32)edcHandle)
    {
        status = EDC_FAILED;
    }
    else
    {
        /* When TVP7000 is powered down it stops generating clock.
           Hence when the application is reloaded VPORT can't be reset during
           bind time as there is no clock available at VCLK0.
           So TVP7000 is not powered down. */
        /* TVP7000_powerdown(TRUE); */

        if (EDC_SUCCESS == i2c_deInit())
        {
            status = EDC_SUCCESS;
        }
        else
        {
            status = EDC_FAILED;
        }
    }

    EDC_DEBUG("\nExited ---> TVP7000_close\n");

    return status;
}



/**
 *  \brief  TVP7000 Ioctls
 *
 *  \param  edcHandle [IN]  External device channel handle
 *  \param  cmd       [IN]  IOCTL to be performed (mostly enum)
 *  \param  arg       [IN]  Argument as required by each Ioctl
 *
 *  \return  Success    EDC_SUCCESS
 *           Error      EDC_FAILED
 */
static Int32 TVP7000_ctrl(EDC_Handle edcHandle, TVP7000_Cmd cmd, Arg arg)
{
    Int32 status = EDC_FAILED;

    EDC_DEBUG("\nEntered ---> TVP7000_ctrl\n");
    EDC_DEBUG1("EDC_Handle = 0x%0.8x, ", edcHandle);
    EDC_DEBUG2("TVP7000_Cmd = 0x%0.8x, Arg = 0x%0.8x\n", cmd, arg);

    if (_TVP7000_IIC_ADDR == (Int32)edcHandle)
    {
        switch (cmd)
        {
            case EDC_CONFIG:
                if (NULL != arg)
                {
                    status = TVP7000_config((TVP7000_ConfParams *)arg);
                }
                else
                {
                    status = EDC_FAILED;
                }
            break;

            case EDC_RESET:
                status = TVP7000_reset();
            break;

            case TVP7000_POWERDOWN:
                status = TVP7000_powerdown(TRUE);
            break;

            case TVP7000_POWERUP:
                status = TVP7000_powerdown(FALSE);
            break;

            case TVP7000_SET_ANALOG_CHAN:
                if (NULL != arg)
                {
                    status = TVP7000_setAnalogChan(
                                            (TVP7000_AnalogChanParams *)arg);
                }
                else
                {
                    status = EDC_FAILED;
                }
            break;

            default:
                status = EDC_FAILED;
            break;
        }
    }

    EDC_DEBUG("\nExited ---> TVP7000_ctrl\n");

    return status;
}



/**
 *  \brief  TVP7000 device open
 *
 *  \param  devName [IN]  External device name - should be "/0"
 *  \param  optArg  [IN]  Optional argument (not used here)
 *
 *  \return  handle  EDC channel handle when success
 *                   EDC_FAILED when unsuccessful
 */
static EDC_Handle TVP7000_open(String devName, Arg optArg)
{
    Int32 handle = NULL;

    EDC_DEBUG("\nEntered ---> TVP7000_open\n");
    EDC_DEBUG2("devName = %s , Arg = 0x%0.8x\n", devName, optArg);

    /* Check if the address is correct */
    if (devName[0] != '/' && devName[1] != '0')
    {
        handle = (Int32)NULL;
    }
    else
    {
        handle = (Int32)(_TVP7000_IIC_ADDR);
    }

    if (_TVP7000_IIC_ADDR == handle)
    {
        /* Open I2C channel */
        if (EDC_SUCCESS == i2c_init())
        {
            /* Power up decoder chip */
            TVP7000_powerdown(FALSE);

            /* Set registers to default value */
            TVP7000_setDefault();

            /* Configure TVP7000 in default mode */
            if (EDC_SUCCESS != TVP7000_ctrl((EDC_Handle)handle,
                                            EDC_CONFIG,
                                            (Arg)&TVP7000_default))
            {
                handle = NULL;
            }
        }
        else
        {
            handle = NULL;
        }
    }

    EDC_DEBUG("\nExited ---> TVP7000_open\n");

    return (EDC_Handle)handle;
}


/******************************************************************************/
/* End of file                                                                */
/******************************************************************************/
