/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2007 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \file    ths8200.h
 *
 *  This file contains configuration structure and enums for THS8200 encoder
 */

#ifndef _THS8200_H
#define _THS8200_H


/*
 * ========== INCLUDE FILES ==============
 */
#include <std.h>
#include <ti/sdo/pspdrivers/drivers/vport/edc.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 * ========== Defines ==============
 */
/*
 * IOCTLS Supported
 */
/**
 *  \brief THS8200 powerdown command
 *  This doesn't take any argument
 */
#define THS8200_POWERDOWN               (EDC_USER + 1)

/**
 *  \brief THS8200 powerup command
 *  This doesn't take any argument
 */
#define THS8200_POWERUP                 (EDC_USER + 2)

/**
 *  \brief THS8200 internal colorbar enable or disable command
 *  The argument for this IOCTL is a pointer to THS8200_ColorbarParams
 *  This is not supported in current implementation
 */
#define THS8200_ENABLE_COLORBAR         (EDC_USER + 3)

/**
 *  \brief THS8200 BT.656 bypass mode enable or disable command
 *  The argument for this IOCTL is bool
 *  TRUE - Enable BT.656 bypass mode
 *  FALSE - Disable BT.656 bypass mode
 */
#define THS8200_ENABLE_BYPASS           (EDC_USER + 4)

/**
 * Apart from above IOCTLS the following generic EDC IOCTLS are supported
 *
 * EDC_CONFIG   - Argument is a pointer to THS8200_ConfParams structure
 * EDC_RESET    - Doesn't take any argument
 *
 * The following generic EDC IOCTLS are NOT supported
 *
 * EDC_START
 * EDC_STOP
 * EDC_GET_CONFIG
 * EDC_GET_STATUS
 */

/** \brief type define THS8200 IOCTL command type */
typedef unsigned int THS8200_Cmd;


/**
 *  \brief THS8200 analog format Enum
 *  This enum gives available analog format selection with THS8200 encoder
 */
typedef enum THS8200_AnalogFormat_t
{
    THS8200_AFMT_RGB_NO_SYNC,
    /**< RGB out without sync signal - Used for VESA formats */
    THS8200_AFMT_RGB_SYNC_ON_GREEN,
    /**< RGB out with sync signal on Green */
    THS8200_AFMT_RGB_SYNC_ON_ALL,
    /**< RGB out with sync signal on All */
    THS8200_AFMT_YPBPR_SYNC_ON_Y,
    /**< YPbPr out with sync signal on Y - USED for EDTV and HDTV formats */
    THS8200_AFMT_YPBPR_SYNC_ON_ALL
    /**< YPbPr out with sync singnal on All */
} THS8200_AnalogFormat;     /**< Typedef for enum THS8200_AnalogFormat_t */

/**
 *  \brief THS8200 input format Enum
 *  This enum gives available input format for THS8200 encoder
 */
typedef enum THS8200_InputFormat_t
{
    THS8200_IFMT_YCBCR422_10_BIT,
    /**< Not supported */
    THS8200_IFMT_YCBCR422_20_BIT,
    /**< 16/20 bit YUV422 */
    THS8200_IFMT_YCBCR444,
    /**< 24/30 bit YUV444 - Not supported on EVM */
    THS8200_IFMT_RGB444,
    /**< 24/30 bit RGB444 - Not supported on EVM */
    THS8200_IFMT_RGB565,
    /**< 16 bit RGB565 */
    THS8200_IFMT_RGB555
    /**< 15 bit RGB555 */
} THS8200_InputFormat;      /**< Typedef for structure THS8200_InputFormat_t */

/**
 *  \brief THS8200 operation mode Enum
 *  This enum gives available operation mode for THS8200 encoder
 */
typedef enum THS8200_Mode_t
{
    THS8200_MODE_480P_60HZ,
    /**< HD 480P 60Hz */
    THS8200_MODE_576P_50HZ,
    /**< HD 576P 50Hz - Not supported */

    THS8200_MODE_720P_60HZ,
    /**< HD 720P 60Hz */
    THS8200_MODE_720P_50HZ,
    /**< HD 720P 50Hz */

    THS8200_MODE_1080I_60HZ,
    /** HD 1080I 60Hz */
    THS8200_MODE_1080I_50HZ,
    /**< HD 1080I 50Hz */

    THS8200_MODE_VGA_60HZ,
    /**< VESA VGA 60Hz */
    THS8200_MODE_VGA_72HZ,
    /**< VESA VGA 72Hz */
    THS8200_MODE_VGA_75HZ,
    /**< VESA VGA 75Hz */
    THS8200_MODE_VGA_85HZ,
    /**< VESA VGA 85Hz */

    THS8200_MODE_SVGA_60HZ,
    /**< VESA SVGA 60Hz */
    THS8200_MODE_SVGA_72HZ,
    /**< VESA SVGA 72Hz */
    THS8200_MODE_SVGA_75HZ,
    /**< VESA SVGA 75Hz */
    THS8200_MODE_SVGA_85HZ,
    /**< VESA SVGA 85Hz */

    THS8200_MODE_XGA_60HZ,
    /**< VESA XGA 60Hz */
    THS8200_MODE_XGA_70HZ,
    /**< VESA XGA 70Hz */
    THS8200_MODE_XGA_75HZ,
    /**< VESA XGA 75Hz - Not supported */
    THS8200_MODE_XGA_85HZ,
    /**< VESA XGA 85Hz - Not supported */

    THS8200_MODE_SXGA_60HZ,
    /**< VESA SXGA 60Hz */
    THS8200_MODE_SXGA_72HZ,
    /**< VESA SXGA 72Hz - Not supported */
    THS8200_MODE_SXGA_75HZ,
    /**< VESA SXGA 75Hz - Not supported */
    THS8200_MODE_SXGA_85HZ
    /**< VESA SXGA 85Hz - Not supported */
} THS8200_Mode;             /**< Typedef for enum THS8200_Mode_t */

/**
 *  \brief  THS8200 configuration parameters Structure
 *  Structure to contain THS8200 register setting values
 */
typedef struct THS8200_ConfParams_t
{
    THS8200_Mode            mode;
    /**< Indicates resolution for THS8200 */

    THS8200_InputFormat     iFmt;
    /**< Selection for digital input format for THS8200 */

    THS8200_AnalogFormat    aFmt;
    /**< indicates analog output format for THS8200 */

    Bool                    enableSlaveMode;
    /**< Master or Slave mode
     * TRUE  = Slave mode(Timing signals are generated by Video Port)
     * FALSE = Master mode(Timing signals are generated THS8200) - Not supported
     */

    Bool                    enableBT656Sync;
    /**< Boolean to select embedded or external sync for digital input
     * TRUE  = embedded sync (Should be used for EDTV and HDTV modes)
     * FALSE = external sync (Should be used for VESA modes)
     */
} THS8200_ConfParams;       /**< Typedef for structure THS8200_ConfParams_t */

/**
 *  \brief THS8200 Colorbar parameters Structures
 *   This structure contains colorbar parameters fro THS8200 encoder
 */
typedef struct THS8200_ColorbarParams_t
{
    Bool        enableColorbar;
    /**<  Boolean to enable or disable colorbar TRUE  = enable colorbar,
     *    FALSE = disable colorbar
     */

    Uint8       colorbarWidth;
    /**< Width for each color bar */
} THS8200_ColorbarParams; /**< Typedef for structure THS8200_ColorbarParams_t */

#ifdef __cplusplus
}
#endif

#endif  /* #ifndef _THS8200_H */

/******************************************************************************/
/* End of file                                                                */
/******************************************************************************/
