/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2007 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/**
 *  \file    tvp7000.h
 *
 *  This file contains configuration structure and enums for tvp7000 decoder
 */

#ifndef _TVP7000_H
#define _TVP7000_H


/*
 * ========== INCLUDE FILES ==============
 */
#include <std.h>
#include <ti/sdo/pspdrivers/drivers/vport/edc.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 * ========== Defines ==============
 */

/*
 * IOCTLS Supported
 */
/**
 *  \brief TVP7000 powerdown command
 *  This doesn't take any argument
 */
#define TVP7000_POWERDOWN               (EDC_USER + 1)

/**
 *  \brief TVP7000 powerup command
 *  This doesn't take any argument
 */
#define TVP7000_POWERUP                 (EDC_USER + 2)

/**
 *  \brief TVP7000 analog channel selection command
 *  The argument for this IOCTL is a pointer to TVP7000_AnalogChan structure
 */
#define TVP7000_SET_ANALOG_CHAN         (EDC_USER + 3)

/**
 * Apart from above IOCTLS the following generic EDC IOCTLS are supported
 *
 * EDC_CONFIG   - Argument is a pointer to TVP7000_ConfParams structure
 * EDC_RESET    - Doesn't take any argument
 *
 * The following generic EDC IOCTLS are NOT supported
 *
 * EDC_START
 * EDC_STOP
 * EDC_GET_CONFIG
 * EDC_GET_STATUS
 */

/** \brief type define TVP7000 IOCTL command type */
typedef unsigned int TVP7000_Cmd;


/**
 *  \brief TVP7000 output format Enum
 *  This enum gives available output formats with TVP7000 decoder
 */
typedef enum TVP7000_OutputFormat_t
{
    TVP7000_OFMT_24BIT_RGB_YCBCR_444,
    /**< 24bit RGB or YCbCr 444 mode (Not supported on EVM) */
    TVP7000_OFMT_16BIT_YCBCR_422
    /**< 16bit RGB or YCbCr 422 mode */
} TVP7000_OutputFormat;     /**< Typedef for enum TVP7000_OutputFormat_t */

/**
 *  \brief TVP7000 analog channel selection Enum
 *  This enum gives available analog channel selection with TVP7000 decoder
 */
typedef enum TVP7000_AnalogChan_t
{
    TVP7000_AFMT_CHAN_A = 0,
    /**< decoder channel A selection */
    TVP7000_AFMT_CHAN_B,
    /**< decoder channel B selection (Not supported on EVM) */
    TVP7000_AFMT_CHAN_C
    /**< decoder channel C selection (Not supported on EVM) */
} TVP7000_AnalogChan;       /**< Typedef for enum TVP7000_AnalogChan_t */

/**
 *  \brief TVP7000 operation mode selection Enum
 *  This enum gives available video resolution selection with TVP7000 decoder
 */
typedef enum TVP7000_Mode_t
{
    TVP7000_MODE_480I_60HZ,
    /**< 480I 60Hz not supported */
    TVP7000_MODE_576I_50HZ,
    /**< 576I 50Hz not supported */

    TVP7000_MODE_480P_60HZ,
    /**< 480P 60Hz */
    TVP7000_MODE_576P_50HZ,
    /**< 576P 50Hz not supported */

    TVP7000_MODE_720P_60HZ,
    /**< HD 720P 60Hz */
    TVP7000_MODE_720P_50HZ,
    /**< HD 720P 50Hz */

    TVP7000_MODE_1080I_60HZ,
    /**< HD 1080I 60Hz */
    TVP7000_MODE_1080I_50HZ,
    /**< HD 1080I 50Hz */

    TVP7000_MODE_1080P_60HZ,
    /**< HD 1080P 60Hz not supported in EVM */
    TVP7000_MODE_1080P_50HZ,
    /**< HD 1080P 50Hz not supported in EVM */

    TVP7000_MODE_VGA_60HZ,
    /**< VESA VGA 60Hz not supported in EVM */
    TVP7000_MODE_VGA_72HZ,
    /**< VESA VGA 72Hz not supported in EVM */
    TVP7000_MODE_VGA_75HZ,
    /**< VESA VGA 75Hz not supported in EVM */
    TVP7000_MODE_VGA_85HZ,
    /**< VESA VGA 85Hz not supported in EVM */

    TVP7000_MODE_SVGA_60HZ,
    /**< VESA SVGA 60Hz not supported in EVM */
    TVP7000_MODE_SVGA_72HZ,
    /**< VESA SVGA 72Hz not supported in EVM */
    TVP7000_MODE_SVGA_75HZ,
    /**< VESA SVGA 75Hz not supported in EVM */
    TVP7000_MODE_SVGA_85HZ,
    /**< VESA SVGA 85Hz not supported in EVM */

    TVP7000_MODE_XGA_60HZ,
    /**< VESA XGA 60Hz not supported in EVM */
    TVP7000_MODE_XGA_70HZ,
    /**< VESA XGA 70Hz not supported in EVM */
    TVP7000_MODE_XGA_75HZ,
    /**< VESA XGA 75Hz not supported in EVM */
    TVP7000_MODE_XGA_85HZ,
    /**< VESA XGA 85Hz not supported in EVM */

    TVP7000_MODE_SXGA_60HZ,
    /**< VESA SXGA 60Hz not supported in EVM */
    TVP7000_MODE_SXGA_72HZ,
    /**< VESA SXGA 72Hz not supported in EVM */
    TVP7000_MODE_SXGA_75HZ,
    /**< VESA SXGA 75Hz not supported in EVM */
    TVP7000_MODE_SXGA_85HZ
    /**< VESA SXGA 85Hz not supported in EVM */
} TVP7000_Mode;             /**< Typedef for enum TVP7000_Mode_t */

/**
 *  \brief TVP7000 configuration structure
 *  This structure contains configuration parameters for TVP7000 decoder
 *  Used by application to pass configuration parameters needed to configure
 *  TVP7000
 */
typedef struct TVP7000_ConfParams_t
{
    TVP7000_Mode            mode;
    /**< indicates video resolution for TVP7000 */

    TVP7000_OutputFormat    oFmt;
    /**< digital video output format */
} TVP7000_ConfParams;       /**< Typedef for structure TVP7000_ConfParams_t */

/**
 *  \brief TVP7000 analog input channel selection structure
 *  enum for analog input channel selection
 */
typedef struct TVP7000_AnalogChanParams_t
{
    TVP7000_AnalogChan  chanR;
    /**< analog channel selection for R input */
    TVP7000_AnalogChan  chanG;
    /**< analog channel selection for G input */
    TVP7000_AnalogChan  chanB;
    /**< analog channel selection for B input */
    TVP7000_AnalogChan  chanSOG;
    /**< analog channel selection for SOG input */

    TVP7000_AnalogChan  chanHsync;
    /**< analog channel selection for Hsync input
     * TVP7000_AFMT_CHAN_A or TVP7000_AFMT_CHAN_B
     */

    TVP7000_AnalogChan chanVsync;
    /**< analog channel selection for Vsync input TVP7000_AFMT_CHAN_A or
     * TVP7000_AFMT_CHAN_B
     */
} TVP7000_AnalogChanParams;
/**< Typedef for structure TVP7000_AnalogChanParams_t */


/*
 * ========== EXTERNS ===========
 */
/* TVP7000 driver function table */
extern EDC_Fxns TVP7000_Fxns;

#ifdef __cplusplus
}
#endif

#endif  /* #ifndef _TVP7000_H */

/******************************************************************************/
/* End of file                                                                */
/******************************************************************************/
