 /*
 *  Copyright 2006 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) TCP/IP_Network_Developers_Kit 1.92.00.13 11-17-2006 (ndk-b13)
 */
/*
 *  ======== evmc6424init.c ========
 *
 */

#include <stdio.h>
#include <netmain.h>
#include <evm6424.h>
#include <evm6424_psc.h>
#include <evm6424_eeprom.h>
#include <evm6424_gpio.h>
#include <log.h>

extern far LOG_Obj logTrace;

/* following define chooses speed. Set to true for 100Mbps*/
#define RMII_SPEED100 TRUE

/* MDIO phy access functions*/
uint MDIO_phyRegWrite( uint phyIdx, uint phyReg, Uint16 data );

/* This string array corresponds to link state as defined in 64lc_mdio.h*/
static char *LinkStr[] = { "No Link",
                           "10Mb/s Half Duplex",
                           "10Mb/s Full Duplex",
                           "100Mb/s Half Duplex",
                           "100Mb/s Full Duplex"};

static Uint8 bMacAddr[8] = {0};
static Uint32 isRmiiMode = 0, rmiiSpeed100 = 0;

/* we dynamically try to detect the Switch settings for MII/RMII mode.
 * If RMII mode is chosen, MDIO functionality will not be available
 * because of the Pin Muxing. Hence, the driver will not be able
 * to detect link status changes in RMII mode and also the speed must
 * provided as configuration input as there is no way of detecting this
 * from the PHY without MDIO*/
static Int16 emac_rmii_check( );

/*
 *  Main Thread
 *
 *  We changed our TCF file to point call this private init
 * function. Here we initialize our board and read in our
 *  MAC address.
 */
void c6424_init()
{
    Int16 rmii_mode;

    /* Init the board */
    EVM6424_init();

    rmii_mode = emac_rmii_check( );

    if(rmii_mode)
    {

        EVM6424_setPinMux( ( 1 << 3 ), 0 ); // RMII enable
        EVM6424_clrPinMux( 0, ( 3 << 4 ) ); // MII disable
        isRmiiMode = 1 ; /*RMII mode */
        rmiiSpeed100 = RMII_SPEED100 ; /* use 100Mbps */

    }
    else
    {
        /*default BSL Init enables MII/MDIO, just populate
         * correct config callback params */
         isRmiiMode = 0; /* MII Mode */
         rmiiSpeed100 = 0; /* essentially dont care in this mode */

    }


    /*Read the MAC Address from EEPROM*/
    EVM6424_EEPROM_read (0x00007F00, (Uint32) bMacAddr, 8);
}

/*
 * EMAC_64LC_getConfig()
 *
 *  This is a callback from the Ethernet driver. This function
 * is used by the driver to get its 6 byte MAC address, and
 * to determine which DSP interrupt the EMAC should be mapped to.
 */
void EMAC_64LC_getConfig( UINT8 *pMacAddr, uint *pIntVector )
{
    printf("Using MAC Address: %02x-%02x-%02x-%02x-%02x-%02x\n",
            bMacAddr[0], bMacAddr[1], bMacAddr[2],
            bMacAddr[3], bMacAddr[4], bMacAddr[5]);

    /* We fill in the two pointers here. We'll use int 5 for EMAC*/
    mmCopy( pMacAddr, bMacAddr, 6 );
    *pIntVector = 5;
}

/*
 * EMAC_64LC_linkStatus()
 *
 * This is a callback from the Ethernet driver. This function
 * is called whenever there is a change in link state. The
 *  current PHY and current link state are passed as parameters.
 */
void EMAC_64LC_linkStatus( uint phy, uint linkStatus )
{
    printf("Link Status: %s on PHY %d\n",LinkStr[linkStatus],phy);
}

/*
 * EMAC_64LC_getPhyMode()
 *
 * This is a callback from the Ethernet driver. This function
 * is called to obtain config info about phy interface mode
 */
void EMAC_64LC_getPhyMode (Uint32 * paramIsRmiiMode,Uint32 *paramIsSpeed100)
{
    *paramIsRmiiMode = isRmiiMode;
    *paramIsSpeed100 = rmiiSpeed100;

    printf("Using configuration Rmii=%d speed100=%d\n",isRmiiMode,rmiiSpeed100);
}


 /* ------------------------------------------------------------------------ *
 *                                                                          *
 *  Check for 50 MHz clock. We can identify Phy Mode
 * MII/RMII (switch SW3 position) by exmaining for RMII Ref Clock                                                 *
 *                                                                          *
 * ------------------------------------------------------------------------ */
static Int16 emac_rmii_check( )
{
    Int16 count = 0x0100;
    Int32 lookingfor = 0;
    Int32 numfound = 0;
    volatile Uint32* gpio31_in = ( Uint32* )0x01C67020;

    EVM6424_GPIO_setDir( GPIO31, GPIO_IN );
    EVM6424_clrPinMux( ( 1 << 3 ), 0 );

    while ( count-- > 0 )
    {
        if ( lookingfor )
        {
            if ( ( *gpio31_in & 0x80000000 ) != 0 )
            {
                numfound++;
                lookingfor = 0;
            }
        }
        else
        {
            if ( ( *gpio31_in & 0x80000000 ) == 0 )
            {
                numfound++;
                lookingfor = 0x80000000;
            }
        }
    }

    return ( numfound > 2 ) ? 1 : 0;
}


