/*
 *  Copyright 2006 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) TCP/IP_Network_Developers_Kit 1.92.00.13 11-17-2006 (ndk-b13)
 */
/*
 *  ======== 64lc_mdio.c ========
 *
 */

/*
 * NOTE:
 *   When used in an multitasking environment, no MDIO function may be
 *   called while another MDIO function is operating on the same device
 *   handle in another thread. It is the responsibility of the application
 *   to assure adherence to this restriction.
 *
 */

/* Include the MDIO header file */
#include "64lc_mdio.h"

/*
 * Standard defines/assumptions for MDIO interface
 */
#define VBUSCLK     99 

/*
 * PHY_MASK defines the PHY addresses that we are interested in 
 * Ideally should be provided as init param to open call.
 */
#ifndef PHY_MASK
#define PHY_MASK     0x2  /* its a bitmap.PHY on evmdm6424 at address 1 -set bit position 1 */
#endif

/*-----------------------------------------------------------------------
 * PHY Control Registers
 *
 * Used by MDIO to configure a MII compliant PHY
 *-----------------------------------------------------------------------*/
#define PHYREG_CONTROL              0
#define PHYREG_CONTROL_RESET        (1<<15)
#define PHYREG_CONTROL_LOOPBACK     (1<<14)
#define PHYREG_CONTROL_SPEEDLSB     (1<<13)
#define PHYREG_CONTROL_AUTONEGEN    (1<<12)
#define PHYREG_CONTROL_POWERDOWN    (1<<11)
#define PHYREG_CONTROL_ISOLATE      (1<<10)
#define PHYREG_CONTROL_AUTORESTART  (1<<9)
#define PHYREG_CONTROL_DUPLEXFULL   (1<<8)

#define PHYREG_STATUS               1
#define PHYREG_STATUS_FD100         (1<<14)
#define PHYREG_STATUS_HD100         (1<<13)
#define PHYREG_STATUS_FD10          (1<<12)
#define PHYREG_STATUS_HD10          (1<<11)
#define PHYREG_STATUS_NOPREAMBLE    (1<<6)
#define PHYREG_STATUS_AUTOCOMPLETE  (1<<5)
#define PHYREG_STATUS_REMOTEFAULT   (1<<4)
#define PHYREG_STATUS_AUTOCAPABLE   (1<<3)
#define PHYREG_STATUS_LINKSTATUS    (1<<2)
#define PHYREG_STATUS_JABBER        (1<<1)
#define PHYREG_STATUS_EXTENDED      (1<<0)

#define PHYREG_ID1                  2

#define PHYREG_ID2                  3

#define PHYREG_ADVERTISE            4
#define PHYREG_ADVERTISE_NEXTPAGE   (1<<15)
#define PHYREG_ADVERTISE_ACK        (1<<14)
#define PHYREG_ADVERTISE_FAULT      (1<<13)
#define PHYREG_ADVERTISE_PAUSE      (1<<10)
#define PHYREG_ADVERTISE_FD100      (1<<8)
#define PHYREG_ADVERTISE_HD100      (1<<7)
#define PHYREG_ADVERTISE_FD10       (1<<6)
#define PHYREG_ADVERTISE_HD10       (1<<5)
#define PHYREG_ADVERTISE_MSGMASK    (0x1F)
#define PHYREG_ADVERTISE_MSG        (1)

#define PHYREG_PARTNER              5
#define PHYREG_PARTNER_NEXTPAGE     (1<<15)
#define PHYREG_PARTNER_ACK          (1<<14)
#define PHYREG_PARTNER_FAULT        (1<<13)
#define PHYREG_PARTNER_PAUSE        (1<<10)
#define PHYREG_PARTNER_FD100        (1<<8)
#define PHYREG_PARTNER_HD100        (1<<7)
#define PHYREG_PARTNER_FD10         (1<<6)
#define PHYREG_PARTNER_HD10         (1<<5)
#define PHYREG_PARTNER_MSGMASK      (0x1F)


/*-----------------------------------------------------------------------
 * PHY Control Register Macros
 *
 * These MACROS provide an easy way to read/write PHY registers
 *-----------------------------------------------------------------------*/
#define PHYREG_read(regadr, phyadr)                             \
            MDIO_REGS->USERACCESS0 =                            \
                    CSL_FMK(MDIO_USERACCESS0_GO,1u)         |   \
                    CSL_FMK(MDIO_USERACCESS0_REGADR,regadr) |   \
                    CSL_FMK(MDIO_USERACCESS0_PHYADR,phyadr)

#define PHYREG_write(regadr, phyadr, data)                      \
            MDIO_REGS->USERACCESS0 =                            \
                    CSL_FMK(MDIO_USERACCESS0_GO,1u)         |   \
                    CSL_FMK(MDIO_USERACCESS0_WRITE,1u)      |   \
                    CSL_FMK(MDIO_USERACCESS0_REGADR,regadr) |   \
                    CSL_FMK(MDIO_USERACCESS0_PHYADR,phyadr) |   \
                    CSL_FMK(MDIO_USERACCESS0_DATA, data)

#define PHYREG_wait()                                           \
            while( CSL_FEXT(MDIO_REGS->USERACCESS0,MDIO_USERACCESS0_GO)){}

#define PHYREG_waitResults( results )  \
            { \
                while( CSL_FEXT(MDIO_REGS->USERACCESS0,MDIO_USERACCESS0_GO) ){}; \
                results = CSL_FEXT(MDIO_REGS->USERACCESS0,MDIO_USERACCESS0_DATA);\
            }

#define PHYREG_waitResultsAck( results, ack )  \
            { \
                while( CSL_FEXT(MDIO_REGS->USERACCESS0,MDIO_USERACCESS0_GO) ){}; \
                results = CSL_FEXT( MDIO_REGS->USERACCESS0,MDIO_USERACCESS0_DATA );\
                ack = CSL_FEXT( MDIO_REGS->USERACCESS0, MDIO_USERACCESS0_ACK); \
            }


/*-----------------------------------------------------------------------
 * PHY State Machine
 *
 * When using auto-negotiation, the software must keep the MAC in
 * sync with the PHY (for duplex). This module will also attempt to
 * "auto-select" the PHY from a potential list of 32 based on which is
 * first to get a link.
 *
 * On detection of a good link, the link speed and duplex settings will be
 * used to update the EMAC configuration (done external to this module).
 *-----------------------------------------------------------------------*/

/* States in the PHY State Machine */
#define PHYSTATE_MDIOINIT       0u
#define PHYSTATE_RESET          1u
#define PHYSTATE_NWAYSTART      2u
#define PHYSTATE_NWAYWAIT       3u
#define PHYSTATE_LINKWAIT       4u
#define PHYSTATE_LINKED         5u

/*
 * Tick counts for timeout of each state
 * Note that NWAYSTART falls through to NWAYWAIT which falls through
 * to LINKWAIT. The timeout is not reset progressing from one state
 * to the next, so the system has 5 seconds total to find a link.
 */
static uint PhyStateTimeout[] = { 2u,  /* PHYSTATE_MDIOINIT   - min-delay */
                                  6u,  /* PHYSTATE_RESET      - 0.5 sec max */
                                  41u, /* PHYSTATE_NWAYSTART  - 4 seconds */
                                  41u, /* PHYSTATE_NWAYWAIT   - 4 seconds */
                                  51u, /* PHYSTATE_LINKWAIT   - 5 seconds */
                                  0u };/* PHYSTATE_LINKED     - no timeout*/


typedef struct _MDIO_Device {
    uint            ModeFlags;      /* User specified configuration flags */
    uint            phyAddr;        /* Current (or next) PHY addr (0-31) */
    uint            phyState;       /* PHY State                         */
    uint            phyMask;        /* PHY Mask                          */
    uint            phyStateTicks;  /* Ticks elapsed in this PHY state   */
    uint            PendingStatus;  /* Pending Link Status               */
    uint            LinkStatus;     /* Link State PHYREG_STATUS_LINKSTATUS  */
} MDIO_Device;

static void MDIO_initStateMachine( MDIO_Device *pd );
static uint MDIO_initContinue( MDIO_Device *pd );

/*-----------------------------------------------------------------------
 * MDIO_initStateMachine()
 *
 * Internal function to initialize the state machine. It is referred to
 * often in the code as it is called in case of a PHY error
 *-----------------------------------------------------------------------*/
static void MDIO_initStateMachine( MDIO_Device *pd )
{
    /* Setup the state machine defaults */
    pd->phyAddr       = 0;                  /* The next PHY to try  */
    pd->phyState      = PHYSTATE_MDIOINIT;  /* PHY State            */
    pd->phyStateTicks = 0;                  /* Ticks elapsed        */
    pd->LinkStatus    = MDIO_LINKSTATUS_NOLINK;
}

/*-----------------------------------------------------------------------
 * MDIO_open()
 *
 * Opens the MDIO peripheral and start searching for a PHY device.
 *
 * It is assumed that the MDIO module is reset prior to calling this
 * function.
 *-----------------------------------------------------------------------*/
Handle MDIO_open( uint mdioModeFlags )
{
    /*
     * Note: In a multi-instance environment, we'd have to allocate "localDev"
     */
    static MDIO_Device localDev;

    /* Get the mode flags from the user - clear our reserved flag */
    localDev.ModeFlags = mdioModeFlags & ~MDIO_MODEFLG_NWAYACTIVE;
    
    localDev.phyMask = PHY_MASK;

    /* Setup the MDIO state machine */
    MDIO_initStateMachine( &localDev );

    /* Enable MDIO and setup divider */
    MDIO_REGS->CONTROL = CSL_FMKT(MDIO_CONTROL_ENABLE,ENABLE) |
                         CSL_FMK(MDIO_CONTROL_CLKDIV,VBUSCLK) ;

    /* We're done for now - all the rest is done via MDIO_event() */
    return( &localDev );
}


/*-----------------------------------------------------------------------
 * MDIO_close()
 *
 * Close the  MDIO peripheral and disable further operation.
 *-----------------------------------------------------------------------*/
void MDIO_close( Handle hMDIO )
{
    Uint32         ltmp1;
    uint           i;

    (void)hMDIO;

    /*
     * We really don't care what state anything is in at this point,
     * but to be safe, we'll isolate all the PHY devices.
     */
    ltmp1 = MDIO_REGS->ALIVE;
    for( i=0u; ltmp1; i++,ltmp1>>=1 )
    {
        if( 0u != (ltmp1 & 1u) )
        {
            PHYREG_write( PHYREG_CONTROL, i, (PHYREG_CONTROL_ISOLATE |
                                             PHYREG_CONTROL_POWERDOWN) );
            PHYREG_wait();
        }
    }
}


/*-----------------------------------------------------------------------
 * MDIO_getStatus()
 *
 * Called to get the status of the MDIO/PHY
 *-----------------------------------------------------------------------*/
void MDIO_getStatus( Handle hMDIO, uint *pPhy, uint *pLinkStatus )
{
    MDIO_Device *pd = (MDIO_Device *)hMDIO;

    if( (uint*)0 != pPhy )
    {
        *pPhy = pd->phyAddr;
    }
    if( (uint*)0 != pLinkStatus )
    {
        *pLinkStatus = pd->LinkStatus;
    }
}


/*-----------------------------------------------------------------------
 * MDIO_timerTick()
 *
 * Called to signify that approx 100mS have elapsed
 *
 * Returns an MDIO event code (see MDIO Events in dm6446_MDIO.H).
 *-----------------------------------------------------------------------*/
uint MDIO_timerTick( Handle hMDIO )
{
    MDIO_Device *pd = (MDIO_Device *)hMDIO;
    Uint16      tmp1,tmp2,ack;
    Uint32      ltmp1 = 0u;
    Uint32      virtualState=0u, checkTimeout=0u;
    uint        RetVal = MDIO_EVENT_NOCHANGE;

    /*
     * If we are linked, we just check to see if we lost link. Otherwise;
     * we keep treking through our state machine.
     */
    if( pd->phyState == PHYSTATE_LINKED )
    {
        /*
         * Here we check for a "link-change" status indication or a link
         * down indication.
         */
        ltmp1 = MDIO_REGS->LINKINTRAW & 1u;
        MDIO_REGS->LINKINTRAW = ltmp1;
        if( (0u != ltmp1) || (0u == ((MDIO_REGS->LINK)&(1u<<pd->phyAddr)) )  )
        {
            /*
             * There has been a change in link (or it is down)
             * If we do not auto-neg, then we just wait for a new link
             * Otherwise, we enter NWAYSTART or NWAYWAIT
             */
            
            pd->LinkStatus = MDIO_LINKSTATUS_NOLINK;
            RetVal = MDIO_EVENT_LINKDOWN;
            pd->phyStateTicks = 0u;  /* Reset timeout */

            /* If not NWAY, just wait for link */
            if( 0u == (pd->ModeFlags & MDIO_MODEFLG_NWAYACTIVE) )
            {
                pd->phyState = PHYSTATE_LINKWAIT;
            }
            else
            {
                /* Handle NWAY condition */

                /* First see if link is really down */
                PHYREG_read( PHYREG_STATUS, pd->phyAddr );
                PHYREG_wait();
                PHYREG_read( PHYREG_STATUS, pd->phyAddr );
                PHYREG_waitResultsAck( tmp1, ack );
                if( 0u == ack )
                {
                    /* No PHY response, maybe it was unplugged */
                    MDIO_initStateMachine( pd );
                }
                else if( 0u == (tmp1 & PHYREG_STATUS_LINKSTATUS) )
                {
                    /* No Link - restart NWAY */
                    pd->phyState = PHYSTATE_NWAYSTART;

                    PHYREG_write( PHYREG_CONTROL, pd->phyAddr,
                                  PHYREG_CONTROL_AUTONEGEN |
                                  PHYREG_CONTROL_AUTORESTART );
                    PHYREG_wait();
                }
                else
                {
                    /* We have a Link - re-read NWAY params  */
                    pd->phyState = PHYSTATE_NWAYWAIT;
                }
            }
        }
    }

    /*
     * If running in a non-linked state, execute the next
     * state of the state machine.
     */
    if( pd->phyState != PHYSTATE_LINKED )
    {
        /* Bump the time counter */
        pd->phyStateTicks++;

        /* Eliminate gotos from the code. Use virtual state variable for 
         * tracking when we need to check for timeout */
        virtualState = pd->phyState;
        
        do
        {
            checkTimeout = 0u;

            /* Process differently based on state */
            switch( virtualState )
            {
            case PHYSTATE_RESET:
                /* Don't try to read reset status for the first 100 to 200 ms */
                if( pd->phyStateTicks < 2u )
                {
                    break;
                }

                /* See if the PHY has come out of reset */
                PHYREG_read( PHYREG_CONTROL, pd->phyAddr );
                PHYREG_waitResultsAck( tmp1, ack );
                if((0u != ack) && (0u == (tmp1 & PHYREG_CONTROL_RESET)) )
                {
                    /* PHY is not reset. If the PHY init is going well, break out */
                    if( 0u != MDIO_initContinue( pd ) )
                    {
                        break;
                    }
                    /* Else, this PHY is toast. Manually trigger a timeout */
                    pd->phyStateTicks = PhyStateTimeout[pd->phyState];
                }

                /* Fall through to timeout check */

            case PHYSTATE_MDIOINIT:
            
                /* Here we just check timeout and try to find a PHY */
                if( pd->phyStateTicks >= PhyStateTimeout[pd->phyState] )
                {
                    /**
                     * Try the next PHY if anything but a MDIOINIT condition
                     */
                    if( pd->phyState != PHYSTATE_MDIOINIT )
                    {
                        if( ++pd->phyAddr == 32u )
                        {
                            pd->phyAddr = 0u;
                        }
                    }
                
                    /** Run through PHYs present from ALIVE register
		     *  instead of Link register.ltmp1 = MDIO_REGS->LINK;*/
                    ltmp1 = MDIO_REGS->ALIVE;
                    ltmp1 &= pd->phyMask;
                    for( tmp1=0u; tmp1<32u; tmp1++ )
                    {
                        if( 0u != (ltmp1 & (1u <<pd->phyAddr)) )
                        {
                            if( 0u != MDIO_initPHY( pd, pd->phyAddr ) )
                            {
                                break;
                            }
                        }

                        if( ++pd->phyAddr == 32u )
                        {
                            pd->phyAddr = 0u;
                        }
                    }/* end of for tmp1 loop */

                    /**
                     * If we didn't find a PHY, try again
                     */
                    if( tmp1 == 32u )
                    {
                        pd->phyAddr       = 0u;
                        pd->phyState      = PHYSTATE_MDIOINIT;
                        pd->phyStateTicks = 0u;
                        RetVal = MDIO_EVENT_PHYERROR;
                    }
                }
                
                break;

            case PHYSTATE_NWAYSTART:
                /*
                 * Here we started NWAY. We check to see if NWAY is done.
                 * If not done and timeout occured, we find another PHY.
                 */
            
                /* Read the CONTROL reg to verify "restart" is not set */
                PHYREG_read( PHYREG_CONTROL, pd->phyAddr );
                PHYREG_waitResultsAck( tmp1, ack );
                if( 0u == ack )
                {
                    MDIO_initStateMachine( pd );
                    break;
                }
                if( 0u != (tmp1 & PHYREG_CONTROL_AUTORESTART) )
                {
                    virtualState = PHYSTATE_MDIOINIT;
                    checkTimeout = 1u;
                    break;
                }

                /* Flush latched "link status" from the STATUS reg */
                PHYREG_read( PHYREG_STATUS, pd->phyAddr );
                PHYREG_wait();

                pd->phyState = PHYSTATE_NWAYWAIT;

                /* Fallthrough */

            case PHYSTATE_NWAYWAIT:
                /*
                 * Here we are waiting for NWAY to complete.
                 */
            
                /* Read the STATUS reg to check for "complete" */
                PHYREG_read( PHYREG_STATUS, pd->phyAddr );
                PHYREG_waitResultsAck( tmp1, ack );
                if( 0u == ack )
                {
                    MDIO_initStateMachine( pd );
                    break;
                }
                if( 0u == (tmp1 & PHYREG_STATUS_AUTOCOMPLETE) )
                {
                    virtualState = PHYSTATE_MDIOINIT;
                    checkTimeout = 1u;
                    break;                    
                }

                /* We can now check the negotiation results */
                PHYREG_read( PHYREG_ADVERTISE, pd->phyAddr );
                PHYREG_waitResults( tmp1 );
                PHYREG_read( PHYREG_PARTNER, pd->phyAddr );
                PHYREG_waitResults( tmp2 );

                /* Use the "best" results */
                tmp2 &= tmp1;
                if( 0u != (tmp2 & PHYREG_ADVERTISE_FD100) )
                {
                    pd->PendingStatus = MDIO_LINKSTATUS_FD100;
                }
                else if( 0u != (tmp2 & PHYREG_ADVERTISE_HD100) )
                {
                    pd->PendingStatus = MDIO_LINKSTATUS_HD100;
                }
                else if( 0u != (tmp2 & PHYREG_ADVERTISE_FD10) )
                {
                    pd->PendingStatus = MDIO_LINKSTATUS_FD10;
                }
                else if( 0u != (tmp2 & PHYREG_ADVERTISE_HD10) )
                {
                    pd->PendingStatus = MDIO_LINKSTATUS_HD10;
                }
                /*
                 * If we get here the negotiation failed
                 * We just use HD 100 or 10 - the best we think we can do
                 */
                else if( 0u != (tmp1 & PHYREG_ADVERTISE_HD100) )
                {
                    pd->PendingStatus = MDIO_LINKSTATUS_HD100;
                }
                else
                {
                    pd->PendingStatus = MDIO_LINKSTATUS_HD10;
                }
    
                pd->phyState = PHYSTATE_LINKWAIT;

                /* Fallthrough */

            case PHYSTATE_LINKWAIT:
                /*
                 * Here we are waiting for LINK
                 */

                /* Read the STATUS reg to check for "link" */
                PHYREG_read( PHYREG_STATUS, pd->phyAddr );
                PHYREG_waitResultsAck( tmp1, ack );
                if( 0u == ack )
                {
                    MDIO_initStateMachine( pd );
                    break;
                }
                if( 0u == (tmp1 & PHYREG_STATUS_LINKSTATUS) )
                {
                    virtualState = PHYSTATE_MDIOINIT;
                    checkTimeout = 1u;
                    break;
                }

                /* Make sure we're linked in the MDIO module as well */
                ltmp1 = MDIO_REGS->LINK;
                if( 0u == (ltmp1 & (1u<<pd->phyAddr) )  )
                {
                    virtualState = PHYSTATE_MDIOINIT;
                    checkTimeout = 1u;
                    break;
                }

                /* Start monitoring this PHY */
                MDIO_REGS->USERPHYSEL0 = pd->phyAddr;

                /* Clear the link change flag so we can detect a "re-link" later */
                MDIO_REGS->LINKINTRAW = 1u;

                /* Setup our linked state */
                pd->phyState   = PHYSTATE_LINKED;
                pd->LinkStatus = pd->PendingStatus;
                RetVal = MDIO_EVENT_LINKUP;

                break;

            default:
                break;
            }/* end of switch construct */
            
        }while( 0u != checkTimeout);
        
    } /* end of if conditional block */

    return( RetVal );
}


/*-----------------------------------------------------------------------
 * MDIO_initPHY()
 *
 * Force a switch to the specified PHY, and start the negotiation process.
 *
 * Returns 1 if the PHY selection completed OK, else 0
 *-----------------------------------------------------------------------*/
uint MDIO_initPHY( Handle hMDIO, volatile uint phyAddr )
{
    MDIO_Device *pd = (MDIO_Device *)hMDIO;
    Uint32         ltmp1;
    Uint32     ret = 0u;
    uint           i,ack;

    /* Switch the PHY */
    pd->phyAddr = phyAddr;

    /* There will be no link when we're done with this PHY */
    pd->LinkStatus = MDIO_LINKSTATUS_NOLINK;

    /* Shutdown all other PHYs */
    ltmp1 = MDIO_REGS->ALIVE ;
    for( i=0u; ltmp1; i++,ltmp1>>=1u )
    {
        if( (0u != (ltmp1 & 1u)) && (i != phyAddr) )   
        {
            PHYREG_write( PHYREG_CONTROL, i, (PHYREG_CONTROL_ISOLATE |
                                             PHYREG_CONTROL_POWERDOWN) );
            PHYREG_wait();
        }
    }

    /* Reset the PHY we plan to use */
    PHYREG_write( PHYREG_CONTROL, phyAddr, PHYREG_CONTROL_RESET );
    PHYREG_waitResultsAck( i, ack );

    /* If the PHY did not ACK the write, return zero */
    if( 0u == ack )
    {
        ret= 0u;
    }
    else
    {
    /* Setup for our next state */
        pd->phyState = PHYSTATE_RESET;
        pd->phyStateTicks = 0u;  /* Reset timeout */
        ret = 1u;
    }

    return(ret);
}


/*-----------------------------------------------------------------------*\
 * MDIO_initContinue()
 *
 * Continues the initialization process started in MDIO_initPHY()
 *
 * Returns 0 on an error, 1 on success
 *-----------------------------------------------------------------------*/
static uint MDIO_initContinue( MDIO_Device *pd )
{
    Uint16         tmp1,tmp2;
    Uint32         val = 0u;
    Uint32     ret = 1u; 

    /* Read the STATUS reg to check autonegotiation capability */
    PHYREG_read( PHYREG_STATUS, pd->phyAddr );
    PHYREG_waitResults( tmp1 );

    /* See if we auto-neg or not */
    if(  (0u != (pd->ModeFlags & MDIO_MODEFLG_AUTONEG) ) &&
        (0u != (tmp1 & PHYREG_STATUS_AUTOCAPABLE) ) ) 
    {
        /* We will use NWAY */

        /* Shift down the capability bits */
        tmp1 >>= 6u;

        /* Mask with the capabilities */
        tmp1 &= ( PHYREG_ADVERTISE_FD100 | PHYREG_ADVERTISE_HD100 |
                  PHYREG_ADVERTISE_FD10 | PHYREG_ADVERTISE_HD10 );

        /* Set Ethernet message bit */
        tmp1 |= PHYREG_ADVERTISE_MSG;

        /* Write out advertisement */
        val = (Uint32) tmp1;
        PHYREG_write( PHYREG_ADVERTISE, pd->phyAddr, val );
        PHYREG_wait();

        /* Start NWAY */
        PHYREG_write( PHYREG_CONTROL, pd->phyAddr, PHYREG_CONTROL_AUTONEGEN );
        PHYREG_wait();

        PHYREG_write( PHYREG_CONTROL, pd->phyAddr,
                      PHYREG_CONTROL_AUTONEGEN|PHYREG_CONTROL_AUTORESTART );
        PHYREG_wait();

        /* Setup current state */
        pd->ModeFlags |= MDIO_MODEFLG_NWAYACTIVE;
        pd->phyState = PHYSTATE_NWAYSTART;
        pd->phyStateTicks = 0u;  /* Reset timeout */
    }
    else
    {
        /* We will use a fixed configuration */

        /* Shift down the capability bits */
        tmp1 >>= 10u;

        /* Mask with possible modes */
        tmp1 &= ( MDIO_MODEFLG_HD10 | MDIO_MODEFLG_FD10 |
                  MDIO_MODEFLG_HD100 | MDIO_MODEFLG_FD100 );

        /* Mask with what the User wants to allow */
        tmp1 &= pd->ModeFlags;

        /* If nothing if left, move on */
        if( 0u == tmp1 )
        {
            ret = 0u;
        }
        else
        {

            /* Setup Control word and pending status */
            if( 0u != (tmp1 & MDIO_MODEFLG_FD100) )
            {
                tmp2 = PHYREG_CONTROL_SPEEDLSB | PHYREG_CONTROL_DUPLEXFULL;
                pd->PendingStatus = MDIO_LINKSTATUS_FD100;
            }
            else if( 0u != (tmp1 & MDIO_MODEFLG_HD100) )
            {
                tmp2 = PHYREG_CONTROL_SPEEDLSB;
                pd->PendingStatus = MDIO_LINKSTATUS_HD100;
            }
            else if( 0u != (tmp1 & MDIO_MODEFLG_FD10) )
            {
                tmp2 = PHYREG_CONTROL_DUPLEXFULL;
                pd->PendingStatus = MDIO_LINKSTATUS_FD10;
            }
            else
            {
                tmp2 = 0u;
                pd->PendingStatus = MDIO_LINKSTATUS_HD10;
            }

            /* Add in internal phy loopback if user wanted it */
            if( 0u != (pd->ModeFlags & MDIO_MODEFLG_LOOPBACK) )
            {
                tmp2 |= PHYREG_CONTROL_LOOPBACK;
            }

            /* Configure PHY */
            PHYREG_write( PHYREG_CONTROL, pd->phyAddr,
                    tmp2 | PHYREG_CONTROL_AUTORESTART );
            PHYREG_wait();

            /* Setup current state */
            pd->ModeFlags &= ~MDIO_MODEFLG_NWAYACTIVE;
            pd->phyState = PHYSTATE_LINKWAIT;
            pd->phyStateTicks = 0u;  /* Reset timeout */
        }
    }

    return(ret);
}


/*-----------------------------------------------------------------------
 * MDIO_phyRegRead()
 *
 * Raw data read of a PHY register.
 *
 * Returns 1 if the PHY ACK'd the read, else 0
 *-----------------------------------------------------------------------*/
uint MDIO_phyRegRead( volatile uint phyIdx, volatile uint phyReg, Uint16 *pData )
{
    uint data,ack;
    Uint32 ret =0u;

    PHYREG_read( phyReg, phyIdx );
    PHYREG_waitResultsAck( data, ack );
    if( 0u == ack )
    {
        ret = 0u;
    }
    else
    {
        if( (Uint16*)0 != pData )
        {
            *pData = data;
        }
        ret = 1u;
    }
    return ret;
}


/*-----------------------------------------------------------------------
 * MDIO_phyRegWrite()
 *
 * Raw data write of a PHY register.
 *
 * Returns 1 if the PHY ACK'd the write, else 0
 *-----------------------------------------------------------------------*/
uint MDIO_phyRegWrite( volatile uint phyIdx, volatile uint phyReg, Uint16 data )
{
    uint ack;
    Uint32 val;
    Uint32 ret =0u;

    val = data;

    PHYREG_write( phyReg, phyIdx, val );
    PHYREG_waitResultsAck( data, ack );
    if( 0u == ack )
    {
        ret = 0u;
    }
    else
    {
        ret =1u;
    }

    return ret;
}

/******************************************************************************
 * End of dm64lc_mdio.c
 ******************************************************************************/

