/*
 *    Copyright 2006 by Texas Instruments Incorporated.
 *    All rights reserved. Property of Texas Instruments Incorporated.
 *    Restricted rights to use, duplicate or disclose this code are
 *    granted through contract.
 *
 *    @(#) TCP/IP_Network_Developers_Kit 1.92.00.13 11-17-2006 (ndk-b13)
 */
/*****************************************************************************
*            Copyright (C) 1999-2003 Texas Instruments Incorporated.
*                            All Rights Reserved
*------------------------------------------------------------------------------
* FILENAME...... 64lc_mdio.h
* DATE CREATED.. 10/07/2005
* LAST MODIFIED.
*------------------------------------------------------------------------------
* NOTE:
*    When used in an multitasking environment, no MDIO function may be
*    called while another MDIO function is operating on the same device
*    handle in another thread. It is the responsibility of the application
*    to assure adherence to this restriction.
*
******************************************************************************/

#ifndef _64LC_MDIO_H_
#define _64LC_MDIO_H_

#include <64lc_common.h>
#include <usertype.h>

#ifdef __cplusplus
extern "C" {
#endif

/*-----------------------------------------------------------------------
 * MDIO Configuration Mode Flags
 *
 * These flags determine how the MDIO module behaves
 *-----------------------------------------------------------------------*/
#define MDIO_MODEFLG_AUTONEG     0x0001u         /* Use Autonegotiate          */
/* The following bits are used for manual and fallback configuration */
#define MDIO_MODEFLG_HD10         0x0002u         /* Use 10Mb/s Half Duplex      */
#define MDIO_MODEFLG_FD10         0x0004u         /* Use 10Mb/s Full Duplex      */
#define MDIO_MODEFLG_HD100         0x0008u         /* Use 100Mb/s Half Duplex   */
#define MDIO_MODEFLG_FD100         0x0010u         /* Use 100Mb/s Full Duplex   */
#define MDIO_MODEFLG_LOOPBACK     0x0040u         /* Use PHY Loopback          */
/* The following bits are reserved for use by the MDIO module             */
#define MDIO_MODEFLG_NWAYACTIVE  0x0080u         /* NWAY is currently active  */

/*-----------------------------------------------------------------------
 * MDIO Link Status Values
 *
 * These values indicate current PHY link status
 *-----------------------------------------------------------------------*/
#define MDIO_LINKSTATUS_NOLINK     0u
#define MDIO_LINKSTATUS_HD10     1u
#define MDIO_LINKSTATUS_FD10     2u
#define MDIO_LINKSTATUS_HD100     3u
#define MDIO_LINKSTATUS_FD100     4u

/*-----------------------------------------------------------------------
 * MDIO Events
 *
 * These events are returned by MDIO_timerTick() to allow the application
 * (or EMAC) to track MDIO status.
 *-----------------------------------------------------------------------*/
#define MDIO_EVENT_NOCHANGE      0u   /* No change from previous status */
#define MDIO_EVENT_LINKDOWN      1u   /* Link down event                */
#define MDIO_EVENT_LINKUP         2u   /* Link (or re-link) event        */
#define MDIO_EVENT_PHYERROR      3u   /* No PHY connected                */

/*-----------------------------------------------------------------------
 * NEW TYPES
 *-----------------------------------------------------------------------*/
#ifndef _EMAC_TYPES
#define _EMAC_TYPES
typedef void * Handle;
#endif

/*-----------------------------------------------------------------------
 * MDIO_open()
 *
 * Opens the MDIO peripheral and start searching for a PHY device.
 *
 * It is assumed that the MDIO module is reset prior to calling this
 * function.
 *-----------------------------------------------------------------------*/
Handle MDIO_open( uint mdioModeFlags );

/*-----------------------------------------------------------------------
 * MDIO_close()
 *
 * Close the MDIO peripheral and disable further operation.
 *-----------------------------------------------------------------------*/
void MDIO_close( Handle hMDIO );

/*-----------------------------------------------------------------------
 * MDIO_getStatus()
 *
 * Called to get the status of the MDIO/PHY
 *-----------------------------------------------------------------------*/
void MDIO_getStatus( Handle hMDIO, uint *pPhy, uint *pLinkStatus );

/*-----------------------------------------------------------------------
 * MDIO_timerTick()
 *
 * Called to signify that approx 100mS have elapsed
 *
 * Returns an MDIO event code (see MDIO Events in DM6446_MDIO.H).
 *-----------------------------------------------------------------------*/
uint MDIO_timerTick( Handle hMDIO );

/*-----------------------------------------------------------------------
 * MDIO_initPHY()
 *
 * Force a switch to the specified PHY, and start negotiation.
 *
 * This call is only used to override the normal PHY detection process.
 *
 * Returns 1 if the PHY selection completed OK, else 0
 *-----------------------------------------------------------------------*/
uint MDIO_initPHY( Handle hMDIO, uint phyAddr );

/*-----------------------------------------------------------------------
 * MDIO_phyRegRead()
 *
 * Raw data read of a PHY register.
 *
 * Returns 1 if the PHY ACK'd the read, else 0
 *-----------------------------------------------------------------------*/
uint MDIO_phyRegRead( uint phyIdx, uint phyReg, Uint16 *pData );

/*-----------------------------------------------------------------------
 * MDIO_phyRegWrite()
 *
 * Raw data write  of a PHY register.
 *
 * Returns 1 if the PHY ACK'd the write, else 0
 *-----------------------------------------------------------------------*/
uint MDIO_phyRegWrite( uint phyIdx, uint phyReg, Uint16 data );

/******************************************************************************
 * End of dm64lc_mdio.h
 ******************************************************************************/
#ifdef __cplusplus
}
#endif

#endif
