/*
 *  Copyright 2006 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) TCP/IP_Network_Developers_Kit 1.92.00.13 11-17-2006 (ndk-b13)
 */
/*
 *  ======== dm64lc.c ========
 *
 */

/**
 *--------------------------------------------------------------------------
 * IP Stack
 *--------------------------------------------------------------------------
 * dm64lc.c
 *
 * DM64LC EMAC Packet Driver
 *
 * Copyright 2006 by Texas Instruments Inc.
 *-------------------------------------------------------------------------
 */
#include <std.h>
#include <sys.h>
#include <hwi.h>
#include <bcache.h>
#include <c64.h>

#include <stkmain.h>
#include <dm64lc_common.h>
#include "dm64lc_mdio.h"
#include "llpacket.h"
#include "instrumentation.h"


#define EXTMEM          0x80000000u

/**
 *
 * This single global can be checked via CCS if there's trouble
 *
 */
uint  DM64LCEMAC_FatalError = 0;

/**
 *
 * Configuration (Obtained via callback)
 *
 */

/* ideally we want to add one more param to getConfig, indicating mii/rmii phy mode,
 * so that we can dynamically configure as necessary at init time
 * instead rebuilding the library*/
extern void     DM64LCEMAC_getConfig( UINT8 *pMacAddr, uint *pIntVector);

extern void     DM64LCEMAC_linkStatus( uint phy, uint linkStatus );
extern Uint32   macsel;
static UINT8    bMacAddr[6];    /* MAC Address */
static uint     IntVector = 5u;  /* Interrupt vector to use */
static uint     IntFlag;        /* Interrupt flag  */

/**
 * Transmit/Receive Descriptor Channel Structure
 * (One receive and one transmit in this driver)
 */
typedef struct _EMAC_DescCh {
    PBMQ            DescQueue;    /* Packets queued as desc */
    uint            DescMax;      /* Max number of desc (buffs) */
    uint            DescCount;    /* Current number of desc */
    EMAC_Desc       *pDescFirst;  /* First desc location */
    EMAC_Desc       *pDescLast;   /* Last desc location */
    EMAC_Desc       *pDescRead;   /* Location to read next desc */
    EMAC_Desc       *pDescWrite;  /* Location to write nest desc */
} EMAC_DescCh;

static EMAC_DescCh chTx;
static EMAC_DescCh chRx;

static uint FlashActiveLED = 0;

#define EMAC_NUMSTATS   36         /* The number of statistics regs */

/**
 *
 * RX Buffer Depth
 *
 * We set this to 16 by default. This is the max (typical) number
 * of free buffers ready for rx packets.
 *
 * Note that this driver always assumes it can enqueue a new TX
 * packet to the EMAC. It justifies this by assuming that the size of
 * its transmit descriptor chain (512 - EMAC_MAX_RX) is greater than
 * or equal to the size of the free packet buffer pool (defined by
 * PKT_NUM_FRAMEBUF in pbm.c). Thus the value of EMAC_MAX_RX
 * plus the size of PKT_NUM_FRAMEBUF (defaults to 48) must be less
 * than or equal to 512 for the driver to work properly.
 *
 */
#define EMAC_MAX_RX     16      /* 64-him */

/**
 * Local instance information pointer for our single device instance
 */
static PDINFO *pPDI = 0;
static Handle hMDIO = 0;

/**
 * Local Functions
 */
static void HwInt();
static uint emacInit();
static void emacDequeueTx( EMAC_Desc *pDescAck );
static void emacEnqueueRx( uint fRestart );
static void emacDequeueRx( EMAC_Desc *pDescAck );


/**
 *--------------------------------------------------------------------
 * HwPktInit()
 *
 * Initialize Device environment and return instance count
 *--------------------------------------------------------------------
 */
uint HwPktInit()
{
    /**
     *
     * Get our MAC address and interrupt to use
     *
     */
    DM64LCEMAC_getConfig( bMacAddr, &IntVector);
    IntFlag = 1<<IntVector;

    return(1u);
}

/**
 *--------------------------------------------------------------------
 * HwPktShutdown()
 *
 * Shutdown Device Environment
 *--------------------------------------------------------------------
 */
void HwPktShutdown()
{
}

/**
 *--------------------------------------------------------------------
 * HwPktOpen()
 *
 * Open Device Instance
 *--------------------------------------------------------------------
 */
uint HwPktOpen( PDINFO *pi )
{
    uint        ReturnCode = 1u;

    /**
     * We only have one device, so store off the PDINFO pointer as
     * global to this module. Otherwise; we'd pick an unclaimed device
     * and associate it with the PDINFO.
     */

    #ifdef RMII_SELECT
    /* Program fullduplex_out,ifctl_a in MAC_CTL */
    CSL_FINST( EMAC_REGS->MACCONTROL,EMAC_MACCONTROL_FULLDUPLEX, FULLDUPLEX );
    CSL_FINST( EMAC_REGS->MACCONTROL,EMAC_MACCONTROL_IFCTLA, ONE );
    #endif

    /**
     * Disable our general purpose interrupt
     */
    C64_disableIER( IntFlag );

    /**
     * If pPDI is already set, this is a restart.
     */
    if( (PDINFO*)0 == pPDI )
    {
        /**
         *
         * Now initialize the EMAC device
         *
         */
        pPDI = pi;

        HWI_eventMap( IntVector, 43 );

        /**
         * Hook our interrupt
         */
        HWI_dispatchPlug( IntVector, ((Fxn)(&HwInt)), -1, 0 );

        BCACHE_invL1pAll();

    }

    /**
     * Use the new PDINFO
     */
    pPDI = pi;

    /**
     * Copy in the MAC address from the configuration
     */
    mmCopy( pPDI->bMacAddr, bMacAddr, 6u );

    /**
     * Initialize the EMAC
     */
    ReturnCode = emacInit();

    /**
     * Enable our general purpose interrupt
     */
    C64_enableIER( IntFlag );

    /**
     * Say not ready to send (we wait for link)
     */
    pi->TxFree = 0;

    return(ReturnCode);
}

/**
 *--------------------------------------------------------------------
 * HwPktClose()
 *
 * Close Device Instance
 *--------------------------------------------------------------------
 */
void HwPktClose( PDINFO *pi )
{
    PBM_Handle  hPkt;

    /* to avoid warning */
    if((PDINFO*)0 != pi)
    {
        (void)pi;
    };

    /**
     * Disable the DSP interrupt
     */
    C64_disableIER( IntFlag );

    /**
     * Disable EMAC/MDIO interrupts in wrapper
     */
    CSL_FINST( ECTL_REGS->EWCTL, ECTL_EWCTL_INTEN, DISABLE );

    /**
     * Teardown RX
     */
    EMAC_REGS->RXTEARDOWN = 0;

    /**
     * Teardown TX
     */
    EMAC_REGS->TXTEARDOWN = 0;

    /**
     * Disable RX, TX, and Clear MACCONTROL
     */
    CSL_FINST( EMAC_REGS->TXCONTROL, EMAC_TXCONTROL_TXEN, DISABLE );
    CSL_FINST( EMAC_REGS->RXCONTROL, EMAC_RXCONTROL_RXEN, DISABLE );
    EMAC_REGS->MACCONTROL = 0 ;

    /**
     * Free all RX buffers
     */
    hPkt =  PBMQ_deq( &chRx.DescQueue );
    while((PBM_Handle)0 != hPkt )
    {
        PBM_free( hPkt );
        hPkt =  PBMQ_deq( &chRx.DescQueue );
    }

    /**
     * Free all TX buffers
     */
    hPkt =   PBMQ_deq( &chTx.DescQueue );
    while( (PBM_Handle)0 != hPkt )
    {
        PBM_free( hPkt );
        hPkt = PBMQ_deq( &chTx.DescQueue );
    }

    /**
     * Close the MDIO Module
     */
    MDIO_close( hMDIO );
}

/**
 *--------------------------------------------------------------------
 * HwPktSetRx()
 *
 * Update Rx Mode (Rx filter and multicast hash table)
 *--------------------------------------------------------------------
 */
void HwPktSetRx( PDINFO *pi )
{
    Uint32        tmp1,tmp2;
    Uint8       HashVal,tmpval;
    Uint8       *pMCastList;
    Uint32      MacHash1,MacHash2;

    /**
     * Clear the hash bits
     */
    MacHash1 = 0u;
    MacHash2 = 0u;

    /**
     * For each address in the list, hash and set the bit
     */
    pMCastList = (Uint8*)(&(pi->bMCast[0]));
    for( tmp1=0; tmp1<pi->MCastCnt; tmp1++ )
    {
        HashVal=0u;

        for( tmp2=0; tmp2<2u; tmp2++ )
        {
            tmpval = (Uint8)(*pMCastList);
            pMCastList += 1;
            HashVal = (Uint8)((HashVal)  ^ (tmpval >> 2u) ^ (tmpval << 4u));
            tmpval = (Uint8) (*(pMCastList++));
            HashVal = (Uint8)((HashVal) ^ (tmpval >> 4u) ^ (tmpval << 2u));
            tmpval = (Uint8) (*(pMCastList++));
            HashVal = (Uint8)((HashVal) ^ (tmpval >> 6u) ^ (tmpval));
        }

        if( (Uint8)0 != ((HashVal) & ((Uint8)0x20u)) )
        {
            MacHash2 |= (1u << ((HashVal) & ((Uint8)0x1fu)) );
        }
        else
        {
            MacHash1 |= (1u << ( (HashVal) & ((Uint8)0x1fu)) );
        }
    }

    /**
     * The following code relies on the numeric relation of the filter
     * value such that the higher filter values receive more types of
     * packets.
     */

    /**
     * Disable Section
     */
    if ( pi->Filter < ETH_PKTFLT_ALL )
    {
        CSL_FINST( EMAC_REGS->RXMBPENABLE, EMAC_RXMBPENABLE_RXCAFEN, DISABLE );
    }

    if ( pi->Filter < ETH_PKTFLT_ALLMULTICAST )
    {
        EMAC_REGS->MACHASH1 = MacHash1;
        EMAC_REGS->MACHASH2 = MacHash2;
    }

    if ( pi->Filter < ETH_PKTFLT_MULTICAST )
    {
        CSL_FINST( EMAC_REGS->RXMBPENABLE,
            EMAC_RXMBPENABLE_RXMULTEN, DISABLE );
    }

    if ( pi->Filter < ETH_PKTFLT_BROADCAST )
    {
        CSL_FINST( EMAC_REGS->RXMBPENABLE,
            EMAC_RXMBPENABLE_RXBROADEN, DISABLE );
    }

    if ( pi->Filter < ETH_PKTFLT_DIRECT )
    {
        EMAC_REGS->RXUNICASTCLEAR = 1;
    }


    /**
     * Enable Section
     */
    if ( pi->Filter >= ETH_PKTFLT_DIRECT )
    {
        EMAC_REGS->RXUNICASTSET = 1;
    }

    if ( pi->Filter >= ETH_PKTFLT_BROADCAST )
    {
        CSL_FINST( EMAC_REGS->RXMBPENABLE,
                   EMAC_RXMBPENABLE_RXBROADEN, ENABLE );
    }

    if ( pi->Filter >= ETH_PKTFLT_MULTICAST )
    {
        CSL_FINST( EMAC_REGS->RXMBPENABLE,
                   EMAC_RXMBPENABLE_RXMULTEN, ENABLE );
    }

    if ( pi->Filter >= ETH_PKTFLT_ALLMULTICAST )
    {
        EMAC_REGS->MACHASH1 = 0xffffffff;
        EMAC_REGS->MACHASH1 = 0xffffffff;
    }

    if ( pi->Filter == ETH_PKTFLT_ALL )
    {
        CSL_FINST( EMAC_REGS->RXMBPENABLE,
                   EMAC_RXMBPENABLE_RXCAFEN, ENABLE );
    }
}

/**
 *--------------------------------------------------------------------
 * HwPktTxNext()
 *
 * Transmit Next Packet on Tx Queue
 *--------------------------------------------------------------------
 */
void HwPktTxNext( PDINFO *pi )
{
    register EMAC_Desc *pDescThis;
    register UINT8     *buffer;
    register uint      length;
    PBM_Handle         hPkt;
    uint               intmask;


    do
    {
        /**
         *Make sure we have a packet to send
         */
        hPkt = PBMQ_deq(&pi->PBMQ_tx);
        if( (PBM_Handle)0 == hPkt )
        {
            pi->TxFree = 1;
            return;
        }

        buffer = PBM_getDataBuffer(hPkt) + PBM_getDataOffset(hPkt);
        length = PBM_getValidLen(hPkt);

        /**
         * Clean the cache for external addesses
         */
        if( 0 != ( ((Uint32)buffer) & EXTMEM ) )
        {
            OEMCacheClean( (void *)buffer, length );
        }

        /**
         * Disable our device interrupt
         */
        intmask = C64_disableIER( IntFlag );

        /**
         * This can't happen, but check anyway
         */
        if( chTx.DescCount == chTx.DescMax )
        {
            PBM_free( hPkt );
            break;
        }

        /**
         * Assign the pointer to "this" desc
         */
        pDescThis = chTx.pDescWrite;

        /**
         * Move the write pointer and bump count
         */
        if( pDescThis == chTx.pDescLast )
        {
            chTx.pDescWrite = chTx.pDescFirst;
        }
        else
        {
            chTx.pDescWrite++;
        }
        chTx.DescCount++;

        /**
         * Fill out the descriptor
         */
        pDescThis->pNext     = 0;
        pDescThis->pBuffer   = buffer;
        pDescThis->BufOffLen = length;
        pDescThis->PktFlgLen = EMAC_DSC_FLAG_SOP | EMAC_DSC_FLAG_EOP |
                               length | EMAC_DSC_FLAG_OWNER;

        /**
         * Enqueue this packet onto the desc queue
         */
        PBMQ_enq( &chTx.DescQueue, hPkt );

        #ifdef ETH_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
                PSP_DVT_DESC(PSP_DVT_eDATA_SND_START, \
                             PSP_DVT_dCHANNEL_ID_INST, \
                             PSP_DVT_dPACKET_ID, \
                             PSP_DVT_dNONE),CHANNELID_INSTANCEID(0,0), hPkt);
        #endif

        /**
         * Synch the cache
         */
        if( 0u != ( (UINT32)buffer & EXTMEM) )
        {
            OEMCacheCleanSynch();
        }

        /**
         * Start it sending
         */
        if( chTx.DescCount > 1 )
        {
            /**
             * Transmitter is already running.
             * Make the previous buffer point to us
             */
            if( pDescThis == chTx.pDescFirst )
            {
                chTx.pDescLast->pNext = pDescThis;
            }
            else
            {
                (pDescThis-1)->pNext = pDescThis;
            }
        }
        else
        {
            /**
             * Transmitter is not running, start it up
             */
            EMAC_REGS->TX0HDP = (Uint32)pDescThis ;
        }

    }while(0);


    C64_enableIER( intmask );
}

#ifdef RMII_SELECT
static Uint8 rmiiLinkUpNotify = 0;
#endif

/**
 *--------------------------------------------------------------------
 * _HwPktPoll()
 *
 * Poll routine - CALLED OUTSIDE OF KERNEL MODE
 *
 * This function is called at least every 100ms, faster in a
 * polling environment. The fTimerTick flag is set only when
 * called on a 100ms event.
 *--------------------------------------------------------------------
 */
void _HwPktPoll( PDINFO *pi, uint fTimerTick )
{
    uint intmask;
    uint mdioStatus,phy,linkStatus;

    (void)pi;

    if( (0u != fTimerTick) && ((Handle)0 != hMDIO) )
    {
        LED_TOGGLE( USER_LED2 );
        if(0u != FlashActiveLED )
        {
            FlashActiveLED = 0;
            LED_TOGGLE( USER_LED3 );
        }

        llEnter();
        intmask = C64_disableIER( IntFlag );


        #ifdef RMII_SELECT
        /* Notify the APP for the first time around */
        if (0u == rmiiLinkUpNotify)
        {
            phy = 0;
            linkStatus = MDIO_LINKSTATUS_FD100;
            DM64LCEMAC_linkStatus( phy, linkStatus );
            rmiiLinkUpNotify = 1;
        }


        /* with MDIO not available, we assume the link is always up */
        while( 0u == pi->TxFree )
        {
            HwPktTxNext( pi );
        }

        #else
        /**
         * Signal the MDIO
         */
        mdioStatus = MDIO_timerTick( hMDIO );

        /**
         * Track new or lost link
         */
        if( (mdioStatus == MDIO_EVENT_LINKDOWN) ||
            (mdioStatus == MDIO_EVENT_LINKUP) )
        {
            MDIO_getStatus( hMDIO, &phy, &linkStatus );

            /**
             * On a new link, set the EMAC duplex
             */
            if( mdioStatus == MDIO_EVENT_LINKUP )
            {
                if( (linkStatus == MDIO_LINKSTATUS_FD10) ||
                    (linkStatus == MDIO_LINKSTATUS_FD100) )
                {
                    CSL_FINST( EMAC_REGS->MACCONTROL,
                               EMAC_MACCONTROL_FULLDUPLEX, FULLDUPLEX );
                }
                else
                {
                    CSL_FINST( EMAC_REGS->MACCONTROL,
                               EMAC_MACCONTROL_FULLDUPLEX, HALFDUPLEX );
                }

                /**
                 * Now that we have a link, send any queued packets
                 */
                while( 0u == pi->TxFree )
                {
                    HwPktTxNext( pi );
                }

            }

            /**
             * Tell application
             */
            DM64LCEMAC_linkStatus( phy, linkStatus );
        }

    #endif

        /**
         * Re-fill Rx buffer queue if needed
         */
        if( chRx.DescCount != chRx.DescMax )
        {
            emacEnqueueRx( 1 );
        }

        C64_enableIER( intmask );
        llExit();
    }
}

/**
 *--------------------------------------------------------------------
 * HwInt()
 *
 * EMAC Hardware Interrupt
 *--------------------------------------------------------------------
 */
static void HwInt(void)
{
    Uint32      intflags,Desc;
    uint        tmp;

    #ifdef ETH_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
            PSP_DVT_DESC(PSP_DVT_eINT_START, \
                         PSP_DVT_dNONE, \
                         PSP_DVT_dNONE, \
                         PSP_DVT_dNONE));
    #endif

    /**
     * Disable EMAC/MDIO interrupts in wrapper
     */
    CSL_FINST( ECTL_REGS->EWCTL, ECTL_EWCTL_INTEN, DISABLE );

    /**
     * Read the interrupt cause
     */
    intflags = EMAC_REGS->MACINVECTOR ;

    /**
     * Look for fatal errors first
     */
    if( 0u != (intflags & CSL_FMK( EMAC_MACINVECTOR_HOSTPEND, 1 ))  )
    {
        /**
         * A fatal error can only be caused by a software
         * logic error. We'll read the fatal error code
         * and exit with the EMAC interrupt still disabled.
         * This will halt all network functionality. The
         * application programmer can manually check the
         * status of DM64LCEMAC_FatalError if the network
         * stops working.
         */
        DM64LCEMAC_FatalError = EMAC_REGS->MACSTATUS ;
        return;
    }

    /**
     * Look for statistics interrupt
     */
    if( 0u != (intflags & CSL_FMK( EMAC_MACINVECTOR_STATPEND, 1 )) )
    {
        volatile Uint32 *pRegAddr;
        Uint32 statval;

        pRegAddr = &EMAC_REGS->RXGOODFRAMES ;

        /*
         * There are "EMAC_NUMSTATS" statistics registers
         * Note that when MIIEN is set in MACCONTROL, these registers
         * are "write to decrement".
         */
        for( tmp=0; tmp<EMAC_NUMSTATS; tmp++ )
        {
            statval = *pRegAddr;
            *pRegAddr++ = statval;
        }
    }

    /**
     * Look for TX interrupt (channel 0)
     */
    if( 0u != (intflags & CSL_FMK( EMAC_MACINVECTOR_TXPEND, 1<<0 )) )
    {
        Desc = EMAC_REGS->TX0CP;
        EMAC_REGS->TX0CP = Desc ;

        emacDequeueTx( (EMAC_Desc *)Desc );
    }

    /**
     * Look for RX interrupt (channel 0)
     */
    if( 0u != (intflags & CSL_FMK( EMAC_MACINVECTOR_RXPEND, 1<<0 )) )
    {
        Desc = EMAC_REGS->RX0CP ;
        EMAC_REGS->RX0CP = Desc ;

        emacDequeueRx( (EMAC_Desc *)Desc );
    }

    /**
     * Enable EMAC/MDIO interrupts in wrapper
     */
    CSL_FINST( ECTL_REGS->EWCTL, ECTL_EWCTL_INTEN, ENABLE );


    #ifdef ETH_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
            PSP_DVT_DESC(PSP_DVT_eINT_END, \
                         PSP_DVT_dNONE, \
                         PSP_DVT_dNONE, \
                         PSP_DVT_dNONE));
    #endif
}

/*--------------------------------------------------------------------*\
* emacInit()
*
* Initialize EMAC. Return "1" if all is OK
\*--------------------------------------------------------------------*/
static uint emacInit()
{
    Uint32          tmpval;
    Uint32          ret = 1u;
    Int32           i;
    volatile Uint32 *pRegAddr;
    uint            utemp1;
    EMAC_Desc       *pDesc;
    PBM_Handle      hPkt;

    /* Initialize the EMAC and MDIO devices */

    ECTL_REGS->EWCTL = CSL_FMKT(ECTL_EWCTL_INTEN, DISABLE );

    /* Reset EMAC */
    EMAC_REGS->SOFTRESET = 0x1;
    while (EMAC_REGS->SOFTRESET != 0x0);


    /* Program the interrupt pacer */
    ECTL_REGS->EWINTTCNT = 4000;

    /* Start the MII Configuration */
    hMDIO = MDIO_open( MDIO_MODEFLG_AUTONEG );

    /* Setup the EMAC */

    /* Reset MAC Control */
    EMAC_REGS->MACCONTROL = 0 ;

    /* Must manually init HDPs to NULL */
    pRegAddr = &EMAC_REGS->TX0HDP;
    for( i=0; i<8; i++ )
    {
        *pRegAddr++ = 0;
    }
    pRegAddr = &EMAC_REGS->RX0HDP;
    for( i=0; i<8; i++ )
    {
        *pRegAddr++ = 0;
    }

    /*
     * While GMIIEN is clear in MACCONTROL, we can write directly to
     * the statistics registers (there are "EMAC_NUMSTATS" of them).
     */
    pRegAddr = &EMAC_REGS->RXGOODFRAMES;
    for( i=0; i<EMAC_NUMSTATS; i++ )
    {
        *pRegAddr++ = 0;
    }

    /* Setup device MAC address */
    EMAC_REGS->MACINDEX = 0x0;

    tmpval = 0u;
    for( i=3; i>=0; i-- )
    {
        tmpval = (tmpval<<8) | ((Uint32)(*(pPDI->bMacAddr+i)) );
    }
    EMAC_REGS->MACADDRHI = tmpval;

    tmpval = (Uint32)(*(pPDI->bMacAddr+5u));
    EMAC_REGS->MACADDRLO = (tmpval<<8) | ((Uint32)(*(pPDI->bMacAddr+4))) ;

    /* For us buffer offset will always be zero */
    EMAC_REGS->RXBUFFEROFFSET = 0;

    /* Reset RX (M)ulticast (B)roadcast (P)romiscuous Enable register */
    EMAC_REGS->RXMBPENABLE = 0;
    EMAC_REGS->MACHASH1 = 0;
    EMAC_REGS->MACHASH2 = 0;

    /* Clear Unicast RX on channel 0-7 */
    EMAC_REGS->RXUNICASTCLEAR = 0xFF;

    /* Set the pass RX CRC mode */
    CSL_FINST(EMAC_REGS->RXMBPENABLE, EMAC_RXMBPENABLE_RXPASSCRC, INCLUDE );

    /* Set MAC loopback if requested */
    /**
     *  CSL_FINST(EMAC_REGS->MACCONTROL, EMAC_MACCONTROL_LOOPBACK, ENABLE );
     */

    /*
     * Enable TX and RX channel interrupts (set mask bits)
     * Enable Host interrupts
     */
    EMAC_REGS->RXINTMASKCLEAR = 0xFF;
    EMAC_REGS->TXINTMASKCLEAR = 0xFF;
    EMAC_REGS->RXINTMASKSET = 0x1;
    EMAC_REGS->TXINTMASKSET = 0x1;
    EMAC_REGS->MACINTMASKSET =  CSL_FMK( EMAC_MACINTMASKSET_HOSTMASK, 1 ) |
                                CSL_FMK( EMAC_MACINTMASKSET_STATMASK, 1 ) ;

    /*
     * Setup Receive Buffers
     */

    /* Pointer to first descriptor to use on RX */
    pDesc = (EMAC_Desc *)_EMAC_DSC_BASE_ADDR;

    /* Number of descriptors for RX channel */
    utemp1 = EMAC_MAX_RX;

    /* Init Rx channel */
    mmZeroInit( &chRx, sizeof(EMAC_DescCh) );
    chRx.DescMax    = utemp1;
    chRx.pDescFirst = pDesc;
    chRx.pDescLast  = pDesc + (utemp1 - 1);
    chRx.pDescRead  = pDesc;
    chRx.pDescWrite = pDesc;

    /* Fill the descriptor table */
    emacEnqueueRx(0);

    /*
     * If we didn't get the number of descriptor buffers that we
     * wanted to, abort now.
     */
    if( chRx.DescCount < utemp1 )
    {
        /* Free all RX descriptors */
        hPkt = PBMQ_deq( &chRx.DescQueue );
        while( (PBM_Handle)0 != hPkt  )
        {
            PBM_free(hPkt);
            hPkt = PBMQ_deq( &chRx.DescQueue );
        }

        /* Close the MDIO Module */
        MDIO_close( hMDIO );

        /* Return the error condition */
        ret = 0;
    }
    else
    {

       /*
    * Setup Transmit Buffers
    */

    /* Pointer to first descriptor to use on TX */
    pDesc += utemp1;

    /* Number of descriptors (max) for TX channel */
    utemp1 = (_EDMA_DSC_ENTRY_COUNT-utemp1);

    /* Init TX channel */
    mmZeroInit( &chTx, sizeof(EMAC_DescCh) );
    chTx.DescMax    = utemp1;
    chTx.pDescFirst = pDesc;
    chTx.pDescLast  = pDesc + (utemp1 - 1);
    chTx.pDescRead  = pDesc;
    chTx.pDescWrite = pDesc;


       /*
    * Enable RX, TX, and MII
    *
    * Note in full duplex mode we also need to set the FULLDUPLEX
    * bit in MACCRONTROL. However, we don't know what to set until
    * we have a link. Also, we must be able to dynamically change
    * this bit if the cable is unplugged and re-linked with a different
    * duplex.
    */
    CSL_FINST( EMAC_REGS->TXCONTROL, EMAC_TXCONTROL_TXEN, ENABLE );
    CSL_FINST( EMAC_REGS->RXCONTROL, EMAC_RXCONTROL_RXEN, ENABLE );
    EMAC_REGS->MACCONTROL = CSL_FMKT( EMAC_MACCONTROL_RXOWNERSHIP, ONE ) |
                            CSL_FMKT( EMAC_MACCONTROL_RXOFFLENBLOCK, BLOCK ) |
                        CSL_FMKT( EMAC_MACCONTROL_GMIIEN, ENABLE );

    /* Startup RX */
    EMAC_REGS->RX0HDP = (Uint32)chRx.pDescRead ;

    /* Enable global interrupt in wrapper */
    CSL_FINST( ECTL_REGS->EWCTL, ECTL_EWCTL_INTEN, ENABLE );
    }

    return(ret);
}

/*--------------------------------------------------------------------*\
* emacDequeueTx()
*
* Dequeue all completed TX packets and return buffers to application
\*--------------------------------------------------------------------*/
static void emacDequeueTx( EMAC_Desc *pDescAck )
{
    PBM_Handle  hPkt;
    Uint32      PktFlgLen;
    register uint  i,j = (uint)chTx.pDescRead;

    /* Get the status of the ACK descriptor */
    PktFlgLen = pDescAck->PktFlgLen;

    /* Calc the new "Read" descriptor */
    if( pDescAck == chTx.pDescLast )
    {
        chTx.pDescRead = chTx.pDescFirst;
    }
    else
    {
        chTx.pDescRead = pDescAck+1;
    }

    i = (uint)chTx.pDescRead;

    /* Turn i into a descriptor count */
    if( j < i )
    {
        i = (i-j)/sizeof(EMAC_Desc);
    }
    else
    {
        i = chTx.DescMax - ((j-i)/sizeof(EMAC_Desc));
    }

    chTx.DescCount -= i;

    /* Pop & Free Buffers 'till the last Descriptor */
    while( i-- )
    {
        /**
         * Recover the packet and free it
         */
        hPkt = PBMQ_deq( &chTx.DescQueue );
        if( (PBM_Handle)0 != hPkt )
        {
            #ifdef ETH_INSTRUMENTATION_ENABLED
            PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
                    PSP_DVT_DESC(PSP_DVT_eDATA_SND_END, \
                                 PSP_DVT_dCHANNEL_ID_INST, \
                                 PSP_DVT_dPACKET_ID, \
                                 PSP_DVT_dNONE), CHANNELID_INSTANCEID(0,0), hPkt);
            #endif

            PBM_free( hPkt );
        }
    }

    /**
     * If the transmitter stopped and we have more descriptors, then restart
     */
    if( (0u != (PktFlgLen & EMAC_DSC_FLAG_EOQ) ) && (0u != chTx.DescCount) )
    {
        EMAC_REGS->TX0HDP = (Uint32)chTx.pDescRead;
    }
}


/*--------------------------------------------------------------------*\
* emacEnqueueRx()
*
* Fill any empty RX descriptors with new buffers from the application
\*--------------------------------------------------------------------*/
static void emacEnqueueRx( uint fRestart )
{
    PBM_Handle  hPkt;
    EMAC_Desc   *pDesc;
    uint        CountOrg;
    UINT8       *buffer;

    /**
     * Keep the old count around
     */
    CountOrg = chRx.DescCount;

    /**
     * Fill RX Packets Until Full
     */
    while( chRx.DescCount < chRx.DescMax )
    {
        /**
         * Try and get a buffer.
         */
        hPkt = PBM_alloc( 1518 + PKT_PREPAD );

        if( (PBM_Handle) 0 == hPkt )
        {
            break;
        }

        #ifdef ETH_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
                PSP_DVT_DESC(PSP_DVT_eFUNC_START, \
                             PSP_DVT_dCHANNEL_ID_INST, \
                             PSP_DVT_dPACKET_ID, \
                             PSP_DVT_dSIZE_BYTES),CHANNELID_INSTANCEID(1,0),hPkt,PBM_getValidLen(hPkt));
        #endif

        PBM_setDataOffset( hPkt, PKT_PREPAD );
        buffer = PBM_getDataBuffer(hPkt) + PKT_PREPAD;

        /**
         * Clean the cache for external addesses
         */
        if( 0u != ((UINT32)buffer & EXTMEM) )
        {
            OEMCacheClean( (void *)buffer, 1518 );
        }

        /**
         * Fill in the descriptor for this buffer
         */
        pDesc = chRx.pDescWrite;

        /**
         * Move the write pointer and bump count
         */
        if( chRx.pDescWrite == chRx.pDescLast )
        {
            chRx.pDescWrite = chRx.pDescFirst;
        }
        else
        {
            chRx.pDescWrite++;
        }
        chRx.DescCount++;

        /**
         * Fill in the descriptor
         */
        pDesc->pNext     = 0;
        pDesc->pBuffer   = buffer;
        pDesc->BufOffLen = 1518;
        pDesc->PktFlgLen = EMAC_DSC_FLAG_OWNER;

        #ifdef ETH_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
                    PSP_DVT_DESC(PSP_DVT_eRCV_START, \
                                 PSP_DVT_dCHANNEL_ID_INST, \
                                 PSP_DVT_dPACKET_ID, \
                                 PSP_DVT_dNONE),CHANNELID_INSTANCEID(1,0),hPkt);
        #endif

        /**
         * Enqueue this packet onto the desc queue
         */
        PBMQ_enq( &chRx.DescQueue, hPkt );

        /**
         * Synch the cache
         */
        if( 0u != ((UINT32)buffer & EXTMEM) )
        {
            OEMCacheCleanSynch();
        }

        /**
         * Make the previous buffer point to us
         */
        if( pDesc == chRx.pDescFirst )
        {
             chRx.pDescLast->pNext = pDesc;
        }
        else
        {
            (pDesc-1)->pNext = pDesc;
        }

        #ifdef ETH_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
                PSP_DVT_DESC(PSP_DVT_eFUNC_END, \
                             PSP_DVT_dCHANNEL_ID_INST, \
                             PSP_DVT_dPACKET_ID, \
                             PSP_DVT_dSIZE_BYTES),CHANNELID_INSTANCEID(1,0),hPkt,0);
        #endif
    }

    /* Restart RX if we had ran out of descriptors and got some here */
    if( ((0u != fRestart) && (0u == CountOrg)) && (0u != chRx.DescCount) )
    {
        EMAC_REGS->RX0HDP = (Uint32)chRx.pDescRead;
    }
}

Uint32 ethernetRxCount =0;
/*--------------------------------------------------------------------*\
* emacDequeueRx()
*
* Dequeue all completed RX packets and give buffers to application
\*--------------------------------------------------------------------*/
static void emacDequeueRx( EMAC_Desc *pDescAck )
{
    PBM_Handle  hPkt;
    uint        tmp;
    Uint32      PktFlgLen;
    Uint32      NeedService = 0;
    EMAC_Desc   *pDescNewRxFirst,*pDescNewRxLast=0,*pTemp;
    UINT8       *buffer;

    /* Remember the first new descriptor slot */
    pDescNewRxFirst = chRx.pDescWrite;

    /*
     * Pop & Free Buffers 'till the last Descriptor
     * One thing we know for sure is that all the decriptors from
     * the read pointer to pDescAsk are linked to each other via
     * their pNext field.
     */
    tmp = 1u;
    do
    {

        /* Get the status of this descriptor */
        PktFlgLen = chRx.pDescRead->PktFlgLen;

        /**
         * Recover the packet and pass it on
         */
        hPkt = PBMQ_deq( &chRx.DescQueue );
        if( (PBM_Handle)0 != hPkt )
        {
            /**
             * Enque the packet and signal the stack
             */
            PBM_setIFRx( hPkt, pPDI->hEther );
            PBM_setValidLen(hPkt,(PktFlgLen & 0xFFFF));

            #ifdef ETH_INSTRUMENTATION_ENABLED
            PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
                    PSP_DVT_DESC(PSP_DVT_eRCV_END, \
                                 PSP_DVT_dCHANNEL_ID_INST, \
                                 PSP_DVT_dPACKET_ID, \
                                 PSP_DVT_dNONE),CHANNELID_INSTANCEID(1,0),hPkt);
            #endif

            ethernetRxCount++;
            PBMQ_enq( &PBMQ_rx, hPkt );
            NeedService = 1;
        }

        /* See if this was the last buffer */
        if( chRx.pDescRead == pDescAck )
        {
            tmp = 0;
        }

        /* Move the read pointer and decrement count */
        if( chRx.pDescRead == chRx.pDescLast )
        {
            chRx.pDescRead = chRx.pDescFirst;
        }
        else
        {
            chRx.pDescRead++;
        }
        chRx.DescCount--;

        /**
         * Try and get a buffer.
         */
        hPkt = PBM_alloc( 1518 + PKT_PREPAD );

        if( (PBM_Handle)0 != hPkt )
        {
            #ifdef ETH_INSTRUMENTATION_ENABLED
            PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
                                PSP_DVT_DESC(PSP_DVT_eFUNC_START, \
                                             PSP_DVT_dCHANNEL_ID_INST, \
                                             PSP_DVT_dPACKET_ID, \
                                             PSP_DVT_dSIZE_BYTES),CHANNELID_INSTANCEID(1,0),hPkt,PBM_getValidLen(hPkt));
            #endif

            PBM_setDataOffset( hPkt, PKT_PREPAD );
            buffer = PBM_getDataBuffer(hPkt) + PKT_PREPAD;

            /**
             * Clean the cache for external addresses
             */
            if( 0u != ((UINT32)buffer & EXTMEM) )
            {
                OEMCacheClean( (void *)buffer, 1518 );
            }

            /**
             * Fill in the descriptor for this buffer
             */
            pDescNewRxLast = chRx.pDescWrite;

            /**
             * Move the write pointer and bump count
             */
            if( chRx.pDescWrite == chRx.pDescLast )
            {
                chRx.pDescWrite = chRx.pDescFirst;
            }
            else
            {
                chRx.pDescWrite++;
            }
            chRx.DescCount++;

            /**
             * Fill in the descriptor
             */
            pDescNewRxLast->pBuffer   = buffer;

            #ifdef ETH_INSTRUMENTATION_ENABLED
            PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
                    PSP_DVT_DESC(PSP_DVT_eRCV_START, \
                                 PSP_DVT_dCHANNEL_ID_INST, \
                                 PSP_DVT_dPACKET_ID, \
                                 PSP_DVT_dNONE),CHANNELID_INSTANCEID(1,0),hPkt);
            #endif

            /**
             * Enqueue this packet onto the desc queue
             */
            PBMQ_enq( &chRx.DescQueue, hPkt );

            #ifdef ETH_INSTRUMENTATION_ENABLED
            PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
                    PSP_DVT_DESC(PSP_DVT_eFUNC_END, \
                                 PSP_DVT_dCHANNEL_ID_INST, \
                                 PSP_DVT_dPACKET_ID, \
                                 PSP_DVT_dSIZE_BYTES),CHANNELID_INSTANCEID(1,0),hPkt,0);
            #endif
        }

    }while (0u != tmp);

    /*
     * If we added descriptors, make the pNext of the last NULL, and
     * make the previous descriptor point to the new list we added.
     */
    if( (EMAC_Desc*)0 != pDescNewRxLast )
    {
        pDescNewRxLast->pNext = 0;

        /**
         * Synch the cache
         */
        if( 0u != ((UINT32)buffer & EXTMEM) )
        {
            OEMCacheCleanSynch();
        }

        /* Make the previous buffer point to us */
        if( pDescNewRxFirst == chRx.pDescFirst )
        {
            pTemp = chRx.pDescLast;
        }
        else
        {
            pTemp = pDescNewRxFirst-1;
        }

        /*
         * If these pointers wrapped, the RX engine is stopped
         * Otherwise; tack the new list to the old
         */
        if( pTemp != pDescNewRxLast )
        {
            pTemp->pNext = pDescNewRxFirst;
        }
    }

    /* If the receiver stopped and we have more descriptors, then restart */
    if( (0u != (PktFlgLen & EMAC_DSC_FLAG_EOQ) ) && (0u !=chRx.DescCount) )
    {
        EMAC_REGS->RX0HDP = (Uint32)chRx.pDescRead ;
    }

    /**
     * Signal the stack if needed
     */
    if( 0u != NeedService )
    {
        FlashActiveLED = 1;
        STKEVENT_signal( pPDI->hEvent, STKEVENT_ETHERNET, 1 );
    }
}


/**
 * If statistics abstraction is added to the core stack,stats should be exported thru HwPktStats kind of API
 *
 * extern void HwPktGetStats(PDINFO *pi, PDSTATS *stats);
 *
 * extern void HxPktClearStats(PDINFO * pi);
 *
 * For now, provide direct interface, assuming only one MAC instance
 */

/**
 *--------------------------------------------------------------------
 * getEtherStats()
 *
 * Get Device Statistics
 *--------------------------------------------------------------------
 */
void getEtherStats(ETHERSTATS * stats)
{
    Uint32 iteration;
    volatile Uint32 *pRegAddr;
    volatile Uint32 *statCounter;

    if( (ETHERSTATS*)0 != stats )
    {
        /* fill statistics into stats structure */
        pRegAddr = &EMAC_REGS->RXGOODFRAMES ;
        statCounter = &stats->RXGOODFRAMES;

        /*
         * There are "EMAC_NUMSTATS" statistics registers
         */
        for( iteration = 0; iteration < EMAC_NUMSTATS; iteration++ )
        {
            *statCounter = *pRegAddr ;
            pRegAddr++;
            statCounter++;
        }
    }

    return;

}/* end of function getEtherStats() */


/**
 *--------------------------------------------------------------------
 * clearEtherStats()
 *
 * Clear Device Statistics
 *--------------------------------------------------------------------
 */
void clearEtherStats()
{
    Uint32 iteration = 0 ;
    volatile Uint32 *pRegAddr;
    Uint32 statval;

    pRegAddr = &EMAC_REGS->RXGOODFRAMES ;

    /*
     * There are "EMAC_NUMSTATS" statistics registers
     * Note that when MIIEN is set in MACCONTROL, these registers
     * are "write to decrement".
     */
    for( iteration = 0; iteration < EMAC_NUMSTATS; iteration++ )
    {
        statval = *pRegAddr;
        *pRegAddr = statval;
        pRegAddr++;
    }

    return;
}/* end of function clearEtherStats() */
