/*
 *  Copyright 2006 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) TCP/IP_Network_Developers_Kit 1.92.00.13 11-17-2006 (ndk-b13)
 */
/**
 *--------------------------------------------------------------------------
 * IP Stack
 *--------------------------------------------------------------------------
 * llPacket.c
 *
 * Ethernet Packet Driver
 *
 * Author: Michael A. Denio
 * Copyright 1998, 2003 by Texas Instruments Inc.
 *-------------------------------------------------------------------------
 */
#include <stkmain.h>
#include "llpacket.h"
#include "instrumentation.h"

/**
 *--------------------------------------------------------------------
 * PUBLIC FUNCTIONS USED FOR INTIALIZATION AND EVENTS
 *--------------------------------------------------------------------
 */
#define MAX_INSTANCE    2u

static uint     PktDevCount = 0u;        /* Number of Net Devices */
static PDINFO   pdi[MAX_INSTANCE];      /* Private Info about devices */

/**
 * Queue for Received Packets
 */
PBMQ    PBMQ_rx;

/**
 *--------------------------------------------------------------------
 * PUBLIC FUNCTIONS USED BY NETCTRL
 *--------------------------------------------------------------------
 */

/**
 *--------------------------------------------------------------------
 * _llPacketInit()
 *
 * Opens the packet driver environment and enumerates devices
 *--------------------------------------------------------------------
 */
uint _llPacketInit(STKEVENT_Handle hEvent)
{
    Uint32 i;

    /**
     * Initialize RX Queue
     */
    PBMQ_init(&PBMQ_rx);

    /**
     * Initialize the packet driver(s)
     */
    PktDevCount = HwPktInit();

    if( PktDevCount > MAX_INSTANCE )
    {
        PktDevCount = MAX_INSTANCE;
    }

    /**
     * Initialize PDINFO for each driver instance
     */
    for(i=0; i<PktDevCount; i++)
    {
        mmZeroInit( &pdi[i], sizeof( PDINFO ) );

        /**
         * Set physical index
         */
        pdi[i].PhysIdx     = i;

        /**
         * Copy Event Handle
         */
        pdi[i].hEvent      = hEvent;

        /**
         * MCast List
         */
        pdi[i].MCastCnt    = 0u;

        /**
         * Default MAC Address (can be overwritten by HwPktOpen())
         */
        pdi[i].bMacAddr[0] = ((Uint8) 0x08u);
        pdi[i].bMacAddr[1] = ((Uint8) 0x00u);
        pdi[i].bMacAddr[2] = (Uint8) 0x28u;
        pdi[i].bMacAddr[3] = (Uint8) 0xFFu;
        pdi[i].bMacAddr[4] = (Uint8) 0xFFu;
        pdi[i].bMacAddr[5] = (Uint8) i;
    }

    return( PktDevCount );
}

/**
 *--------------------------------------------------------------------
 * _llPacketShutdown()
 *
 * Called to shutdown packet driver environment
 *--------------------------------------------------------------------
 */
void _llPacketShutdown()
{
    PktDevCount = 0u;
    HwPktShutdown();

    /**
     * Flush out our pending queue
     */
    while( 0u != PBMQ_count(&PBMQ_rx) )
    {
        PBM_free( PBMQ_deq(&PBMQ_rx) );
    }
}

/**
 *--------------------------------------------------------------------
 * _llPacketServiceCheck()
 *
 * Called to check for packet activity
 *--------------------------------------------------------------------
 */
void _llPacketServiceCheck( uint fTimerTick )
{
    uint dev;

    for( dev=0; dev<PktDevCount; dev++ )
    {
        _HwPktPoll( &(pdi[dev]), fTimerTick );
    }
}

/**
 *--------------------------------------------------------------------
 * PUBLIC FUNCTIONS USED BY THE STACK
 *--------------------------------------------------------------------
 */

/**
 *--------------------------------------------------------------------
 * llPacketOpen()
 *
 * Opens the packet driver, and request packets of our desired type.
 *--------------------------------------------------------------------
 */
uint llPacketOpen( uint dev, HANDLE hEther )
{
    Uint32 ret =0u;

    /**
     * Our device index is "1" based to the upper layers
     */
    dev--;

    if (dev < PktDevCount )
    {
       /**
        * Init Logical Device
        */
        pdi[dev].hEther = hEther;

       /**
        * Call low-level open function
        */
        ret =  HwPktOpen( &pdi[dev] );
    }

    return ret;
}

/**
 *--------------------------------------------------------------------
 * PacketClose()
 *
 * Called to shutdown packet driver.
 *--------------------------------------------------------------------
 */
void llPacketClose( uint dev )
{
    /**
     * Our device index is "1" based to the upper layers
     */
    dev--;

    /**
     * Call low-level close function
     */
    if (dev < PktDevCount)
    {
        HwPktClose( &pdi[dev] );
    }

    return;
}

/**
 *--------------------------------------------------------------------
 * llPacketSend()
 *
 * Called to send data to a device. The buffer has been completely
 * formatted by the time this call is made, and is ready for
 * transmit. The dev parameter is the physical destination device.
 *--------------------------------------------------------------------
 */
void llPacketSend( uint dev, PBM_Handle hPkt )
{
    /**
     * Our device index is "1" based to the upper layers
     */
    dev--;

    if( PBM_getValidLen(hPkt) < 60u )
    {
        PBM_setValidLen( hPkt, 60u );
    }

    if( (dev < PktDevCount) && (PBM_getValidLen(hPkt) <= 1514u) )
    {
        #ifdef ETH_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
                PSP_DVT_DESC(PSP_DVT_eFUNC_START, \
                             PSP_DVT_dCHANNEL_ID_INST, \
                             PSP_DVT_dPACKET_ID, \
                             PSP_DVT_dSIZE_BYTES),CHANNELID_INSTANCEID(0,0),hPkt,PBM_getValidLen(hPkt));
        #endif

        PBMQ_enq( &(pdi[dev].PBMQ_tx), hPkt );

        if( 0u != (pdi[dev].TxFree) )
        {
            HwPktTxNext( &(pdi[dev]) );
        }

        #ifdef ETH_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
                PSP_DVT_DESC(PSP_DVT_eFUNC_END, \
                             PSP_DVT_dCHANNEL_ID_INST, \
                             PSP_DVT_dPACKET_ID, \
                             PSP_DVT_dSIZE_BYTES),CHANNELID_INSTANCEID(0,0),hPkt,0);
        #endif
    }
    else
    {
        PBM_free( hPkt );
    }
}

/**
 *--------------------------------------------------------------------
 * llPacketGetMacAddr()
 *
 * Called to the MAC address of the specified device.
 *--------------------------------------------------------------------
 */
void llPacketGetMacAddr( uint dev, UINT8 *pbData )
{
    /**
     * Our device index is "1" based to the upper layers
     */
    dev--;

    if( dev < PktDevCount )
    {
        mmCopy( pbData, pdi[dev].bMacAddr, 6u );
    }
}

/**
 *--------------------------------------------------------------------
 * llPacketGetMCastMax()
 *
 * Called to the maxumim multicast addresses that the device can hold.
 *--------------------------------------------------------------------
 */
uint llPacketGetMCastMax( uint dev )
{
    Uint32 ret = 0u;
    /**
     * Our device index is "1" based to the upper layers
     */
    dev--;

    if( dev < PktDevCount )
    {
        ret = PKT_MAX_MCAST;
    }
    else
    {
        ret = 0u;
    }

    return ret;
}

/**
 *--------------------------------------------------------------------
 * llPacketSetRxFilter()
 *
 * Called to set the Rx Filter mode of the device. The legal modes
 * are defined in ETHER.H
 *--------------------------------------------------------------------
 */
void llPacketSetRxFilter( uint dev, uint Filter )
{
    /**
     * Our device index is "1" based to the upper layers
     */
    dev--;

    if( (dev < PktDevCount) && (Filter <= ETH_PKTFLT_ALL) )
    {
        pdi[dev].Filter = Filter;
        HwPktSetRx( &(pdi[dev]) );
    }
}

/**
 *--------------------------------------------------------------------
 * llPacketSetMCast()
 *
 * Called to set the Multicast list for the device.
 *--------------------------------------------------------------------
 */
void llPacketSetMCast( uint dev, uint addrcnt, UINT8 *bAddr )
{
    /**
     * Our device index is "1" based to the upper layers
     */
    dev--;

    /**
     * Copy the multicast list from ETH and update HW driver
     */
    if( (dev < PktDevCount) && (addrcnt <= PKT_MAX_MCAST) )
    {
        pdi[dev].MCastCnt = addrcnt;
        mmCopy( pdi[dev].bMCast, bAddr, 6u * addrcnt );
        HwPktSetRx( &(pdi[dev]) );
    }
}

/**
 *--------------------------------------------------------------------
 * llPacketGetMCast()
 *
 * Called to get the Multicast list for the device.
 *--------------------------------------------------------------------
 */
uint llPacketGetMCast( uint dev, uint max, UINT8 *bAddr )
{
    Uint32 ret = 0u;

    /**
     * Our device index is "1" based to the upper layers
     */
    dev--;

    /**
     * Copy the multicast list to ETH
     */
    if( ((dev < PktDevCount) && (max >= pdi[dev].MCastCnt)) && ((UINT8*)0 != bAddr) )
    {
        mmCopy( bAddr, pdi[dev].bMCast, pdi[dev].MCastCnt * 6u );
        ret = pdi[dev].MCastCnt;
    }
    return(ret);
}

/**
 *--------------------------------------------------------------------
 * llPacketService()
 *
 * Called to service packet activity.
 *--------------------------------------------------------------------
 */
void llPacketService()
{
    /**
     * Give all queued packets to the Ether module
     */
    while( 0u != PBMQ_count(&PBMQ_rx) )
    {
        EtherRxPacket( PBMQ_deq(&PBMQ_rx) );
    }
}
