/***************************************************************************
**  Copyright(c) 2007, Texas Instruments Incorporated. All Rights Reserved.
**
**  FILE: cpsw_miimdio.c
**
**  DESCRIPTION:
**    MDIO Polling State Machine API. Functions will enable mii-Phy
**    negotiation.
**
*****************************************************************************/

#include <std.h>
#include <c64.h>
#include <stkmain.h>
#include "cpsw3g_common.h"
#include "cpsw_miimdio.h"
#include "cpsw_miimdio_regs.h"

/* R A N D O M  N U M B E R  S U P P O R T */
#define COMMON_RANDOM_MAX        0xFFFFu

static Uint32 RandomSeed = 1u;

static Uint32 cpswRandom(void)
{
  RandomSeed = (RandomSeed * (1103515245u)) + (12345u);
  return ((Uint32) (RandomSeed/(65536u)) % (COMMON_RANDOM_MAX + 1u));
}


static Uint32 cpswRandomRange(Uint32 min, Uint32 max)
{
  Uint32 iTmp;

  iTmp =  cpswRandom();
  iTmp %= ((max - min) + 1u);
  iTmp += min;
  return(iTmp);
}

/* Debug Print infrastructure */
#ifdef __CPSW_MDIO_DEBUG

static char *lstate[]={"NULL","INIT","FINDING","FOUND","NWAY_START","NWAY_WAIT","LINK_WAIT","LINKED","LOOPBACK"};

static void _cpsw_MDIO_DumpPhy(PHY_DEVICE *PhyDev, Uint32 p);
static void _cpsw_MDIO_DumpState(PHY_DEVICE *PhyDev);
static void _cpsw_MDIO_DumpPhyDetailed(PHY_DEVICE *PhyDev);
#endif

/* Map this print to appropriate function. Should be of the form
#define mdioPrintf(format, args...)  
*/
#define mdioPrintf printf


/* Local Helper Function Prototypes */
static void   _cpsw_MDIO_WaitForAccessComplete(PHY_DEVICE *PhyDev);
static void   _cpsw_MDIO_UserAccess(PHY_DEVICE *PhyDev, Uint32 method, Uint32 regadr, Uint32 phyadr, Uint32 data);


static void _cpsw_MDIO_DisablePhy(PHY_DEVICE *PhyDev,Uint32 PhyNum);
static void _cpsw_MDIO_PhyTimeOut(PHY_DEVICE *PhyDev);
static void _cpsw_MDIO_ResetPhy(PHY_DEVICE *PhyDev,Uint32 PhyNum);


static void _cpsw_MDIO_DefaultState  (PHY_DEVICE *PhyDev);
static void _cpsw_MDIO_FindingState  (PHY_DEVICE *PhyDev);
static void _cpsw_MDIO_FoundState    (PHY_DEVICE *PhyDev);
static void _cpsw_MDIO_InitState     (PHY_DEVICE *PhyDev);
static void _cpsw_MDIO_LinkedState   (PHY_DEVICE *PhyDev);
static void _cpsw_MDIO_LinkWaitState (PHY_DEVICE *PhyDev);
static void _cpsw_MDIO_LoopbackState (PHY_DEVICE *PhyDev);
static void _cpsw_MDIO_NwayStartState(PHY_DEVICE *PhyDev);
static void _cpsw_MDIO_NwayWaitState (PHY_DEVICE *PhyDev);

#define   _cpswIsGigPhy(PhyDev) 	(TRUE)


#define PHY_NOT_FOUND  0xFFFFu    /*  Used in Phy Detection */

/*CHECK:: PhyState field breakup  */
#define PHY_DEV_OFFSET      (0u)
#define PHY_DEV_SIZE        (5u)
#define PHY_DEV_MASK        (0x1f<<PHY_DEV_OFFSET)

#define PHY_STATE_OFFSET    (PHY_DEV_SIZE+PHY_DEV_OFFSET)
#define PHY_STATE_SIZE      (5u)
#define PHY_STATE_MASK      (0x1fu<<PHY_STATE_OFFSET)
    #define INIT       (1u<<PHY_STATE_OFFSET)
    #define FINDING    (2u<<PHY_STATE_OFFSET)
    #define FOUND      (3u<<PHY_STATE_OFFSET)
    #define NWAY_START (4u<<PHY_STATE_OFFSET)
    #define NWAY_WAIT  (5u<<PHY_STATE_OFFSET)
    #define LINK_WAIT  (6u<<PHY_STATE_OFFSET)
    #define LINKED     (7u<<PHY_STATE_OFFSET)
    #define LOOPBACK   (8u<<PHY_STATE_OFFSET)

#define PHY_SPEED_OFFSET    (PHY_STATE_OFFSET+PHY_STATE_SIZE)
#define PHY_SPEED_SIZE      (1u)
#define PHY_SPEED_MASK      (1u<<PHY_SPEED_OFFSET)

#define PHY_DUPLEX_OFFSET   (PHY_SPEED_OFFSET+PHY_SPEED_SIZE)
#define PHY_DUPLEX_SIZE     (1u)
#define PHY_DUPLEX_MASK     (1u<<PHY_DUPLEX_OFFSET)

#define PHY_TIM_OFFSET      (PHY_DUPLEX_OFFSET+PHY_DUPLEX_SIZE)
#define PHY_TIM_SIZE        (10u)
#define PHY_TIM_MASK        (0x3ffu<<PHY_TIM_OFFSET)

/* we are working with 100ms ticks here */
  #define PHY_FIND_TO (  1u<<PHY_TIM_OFFSET)
    #define PHY_RECK_TO (20u<<PHY_TIM_OFFSET)
    #define PHY_LINK_TO (50u<<PHY_TIM_OFFSET)
    #define PHY_NWST_TO (50u<<PHY_TIM_OFFSET)
    #define PHY_NWDN_TO (80u<<PHY_TIM_OFFSET)
    #define PHY_MDIX_TO (27u<<PHY_TIM_OFFSET) /* 2.74 Seconds <--Spec and empirical */

#define PHY_SMODE_OFFSET    (PHY_TIM_OFFSET+PHY_TIM_SIZE)
#define PHY_SMODE_SIZE      (7u)
#define PHY_SMODE_MASK      (0x7fu<<PHY_SMODE_OFFSET)
    #define SMODE_LPBK   (0x40u<<PHY_SMODE_OFFSET)
    #define SMODE_AUTO   (0x20u<<PHY_SMODE_OFFSET)
    #define SMODE_FD1000 (0x10u<<PHY_SMODE_OFFSET)
    #define SMODE_FD100  (0x08u<<PHY_SMODE_OFFSET)
    #define SMODE_HD100  (0x04u<<PHY_SMODE_OFFSET)
    #define SMODE_FD10   (0x02u<<PHY_SMODE_OFFSET)
    #define SMODE_HD10   (0x01u<<PHY_SMODE_OFFSET)
    #define SMODE_ALL    (0x1fu<<PHY_SMODE_OFFSET)

#define PHY_CHNG_OFFSET    (PHY_SMODE_OFFSET+PHY_SMODE_SIZE)
#define PHY_CHNG_SIZE      (1u)
#define PHY_CHNG_MASK      (1u<<PHY_CHNG_OFFSET)
    #define PHY_CHANGE (1u<<PHY_CHNG_OFFSET)

#define PHY_TIMEDOUT_OFFSET    (PHY_CHNG_OFFSET+PHY_CHNG_SIZE)
#define PHY_TIMEDOUT_SIZE      (1u)     /*  30 Bits used */
#define PHY_TIMEDOUT_MASK   ((Uint32)(1u<<PHY_TIMEDOUT_OFFSET))
#define PHY_MDIX_SWITCH     ((Uint32)(1u<<PHY_TIMEDOUT_OFFSET))

#define PHY_MDIX_OFFSET    (PHY_TIMEDOUT_OFFSET+PHY_TIMEDOUT_SIZE)
#define PHY_MDIX_SIZE      (1u)     /*  31 Bits used */
#define PHY_MDIX_MASK      ((Uint32)(1u<<PHY_MDIX_OFFSET))
#define PHY_MDIX           ((Uint32)(1u<<PHY_MDIX_OFFSET))


/*  Local MDIO Register Macros    */
#define myMDIO_ALIVE           MDIO_ALIVE(PhyDev->miibase)
#define myMDIO_CONTROL         MDIO_CONTROL(PhyDev->miibase)
#define myMDIO_LINK            MDIO_LINK(PhyDev->miibase)
#define myMDIO_LINKINT         MDIO_LINKINT(PhyDev->miibase)
#define myMDIO_USERACCESS      MDIO_USERACCESS(PhyDev->miibase, PhyDev->inst)
#define myMDIO_USERPHYSEL      MDIO_USERPHYSEL(PhyDev->miibase, PhyDev->inst)
#define myMDIO_VER             MDIO_VER(PhyDev->miibase)

#ifndef VOLATILE32
#define VOLATILE32(addr) (*((volatile Uint32 *)(addr)))
#endif

Int32 g_speed_set = 0;
Int32 g_soft_reset_status = 0;

/*User Calls*********************************************************       */

/* Updates book-keeping with info provided, programs ControlReg
 * with clkdiv,enable bits*/
int cpsw_MDIO_Init(PHY_DEVICE *PhyDev, Uint32 miibase, Uint32 inst, Uint32 PhyMask, Uint32 MLinkMask, Uint32 MdixMask, Uint32 ResetReg, Uint32 ResetBit, Uint32 MdioBusFreq, Uint32 MdioClockFreq,int verbose)
  {
  Uint32 HighestChannel;
  Uint32 ControlState;
  Uint32 *PhyState = &PhyDev->PhyState;
  Uint32 clkdiv;
  Int32  ret =0 ;

  PhyDev->miibase   = miibase;
  PhyDev->inst      = inst;
  PhyDev->PhyMask   = PhyMask;
  PhyDev->MLinkMask = MLinkMask;
  PhyDev->MdixMask  = MdixMask;
  
  (void)verbose;/* remove if not required */
  
 
  *PhyState &= ~PHY_MDIX_MASK;   /* Set initial State to MDI */
   
  /* Check that the channel supplied is within range */
  /* GSG 1/18/05 - Applied fix identified by Denis.  The definition of the
     Control register changed between versions of MDIO. */
  HighestChannel = myMDIO_CONTROL >> 8;
  if ((myMDIO_VER & 0x0ffff) > 0x0102)
    {
    HighestChannel = myMDIO_CONTROL >> 24;
    }
  HighestChannel &= 0x1f;
  
  if(inst > HighestChannel)
    {
    ret =(Int32)HighestChannel;
    }
  else
    {

  /*Setup MII MDIO access regs*/

  /* Calculate the correct value for the mclkdiv */
    if (0u != MdioClockFreq)                                              /*MJH+030402*/
	{
    clkdiv = (MdioBusFreq / MdioClockFreq) - 1;                  /*MJH+030402*/
	}
   else                                                            /*MJH+030402*/
	{
    clkdiv = MDIO_CONTROL_CLKDIV;                                  /*MJH+030402*/
	}

  ControlState  = MDIO_CONTROL_ENABLE;
  ControlState |= ((clkdiv) & MDIO_CONTROL_CLKDIV);

  myMDIO_CONTROL = ControlState;  /* Enable MDIO   */
  (void)ResetReg; /* suppress warning */
  (void)ResetBit; /* suppress warning */

  *PhyState=INIT;

  mdioPrintf("cpsw_MDIO_Init\n");
    ret = 0;
    }

  return ret;
  }/* end of function cpsw_MDIO_Init*/

void cpsw_MDIO_SetPhyMode(PHY_DEVICE *PhyDev,Uint32 PhyMode)
  {
  Uint32 *PhyState = &PhyDev->PhyState;
  Uint32 CurrentState;

  PhyDev->PhyMode = PhyMode;   /* used for AUTOMIDX, planned to replace PhyState fields */
  *PhyState &= (~PHY_SMODE_MASK);

  if (0u != (PhyMode & NWAY_LPBK))
    {
    *PhyState |= SMODE_LPBK;
    }
  if (0u != (PhyMode & NWAY_AUTO))
    {
     *PhyState |= SMODE_AUTO;
    }
  if (0u != (PhyMode & NWAY_FD1000) )
    {
    *PhyState |= SMODE_FD1000;
    }
  if (0u!= (PhyMode & NWAY_FD100))
    {
    *PhyState |= SMODE_FD100;
    }
  if (0u != (PhyMode & NWAY_HD100))
    {
    *PhyState |= SMODE_HD100;
    }
  if (0u != (PhyMode & NWAY_FD10))
    {
    *PhyState |= SMODE_FD10;
    }
  if (0u != (PhyMode & NWAY_HD10))
    {
    *PhyState |= SMODE_HD10;
    }

  CurrentState = (*PhyState) & PHY_STATE_MASK;
  if ( ((CurrentState == NWAY_START)|| (CurrentState == NWAY_WAIT)) ||
       ( ((CurrentState == LINK_WAIT) || (CurrentState == LINKED) ) || (CurrentState == LOOPBACK)) )
    {
    *PhyState = ( (*PhyState)&(~PHY_STATE_MASK)) | (FOUND | PHY_CHANGE);
    }

     mdioPrintf("SetPhyMode:%08X Auto:%d, FD10:%d, HD10:%d, FD100:%d, HD100:%d, FD1000:%d LPBK:%d\n", PhyMode,
     (PhyMode & NWAY_AUTO), (PhyMode & MII_NWAY_FD10),(PhyMode & MII_NWAY_HD10),(PhyMode & MII_NWAY_FD100),
     (PhyMode & MII_NWAY_HD100), (PhyMode & NWAY_FD1000),
     (PhyMode & NWAY_LPBK)  );
  }

Uint32 cpsw_MDIO_GetPhyMode(PHY_DEVICE *PhyDev)
{
    Uint32  PhyMode =0;
    Uint32  status_reg = 0,control_reg = 0,advertize_reg;
    Uint32  PhyNum,j;
    Uint32  PhyMask   = PhyDev->PhyMask;
   
    PhyNum = 0; 
    for(j = 1;PhyNum < 32;PhyNum++)
    {
      if( 0u != ( j & PhyMask))
	{
	break;
	}
      j = j<<1;
    }

    if(0u != _cpswIsGigPhy(PhyDev))
    {
       /* read gig status */
       control_reg =_cpsw_MDIO_UserAccessRead(PhyDev, NWAY_1000BT_ADVERTISE_REG, PhyNum);
       /* read gig status */
       status_reg =_cpsw_MDIO_UserAccessRead(PhyDev, NWAY_1000BT_REMADVERTISE_REG, PhyNum);
    }
    /* figure out if gig connected at FD 1000 or not first */
    if ((0u != (control_reg & MII_NWAY_MY_FD1000)) && (0u != (status_reg & MII_NWAY_REM_FD1000)) )
    {
        PhyMode |= NWAY_FD1000;
    }
    
    control_reg = _cpsw_MDIO_UserAccessRead(PhyDev, PHY_CONTROL_REG, PhyNum);
    if(0u != (control_reg & MII_AUTO_NEGOTIATE_EN)  )
    {
       PhyMode |= NWAY_AUTO;
    }
    status_reg = _cpsw_MDIO_UserAccessRead(PhyDev, PHY_STATUS_REG, PhyNum);  
    advertize_reg = _cpsw_MDIO_UserAccessRead(PhyDev, NWAY_ADVERTIZE_REG, PhyNum);

    /* Check for 10/100 capabilities of PHYs */
    if((0u != (control_reg & MII_PHY_100))|| (0u != (PhyMode & NWAY_AUTO)) )
    {
       if(0u != (advertize_reg & MII_NWAY_FD100))
	    {
	    PhyMode |= NWAY_FD100;
	    }
       if(0u != (advertize_reg & MII_NWAY_HD100))
	    {
	    PhyMode |= NWAY_HD100;
	    }
       
    }
   
    if(((control_reg & MII_PHY_100) == 0) || (0u != (PhyMode & NWAY_AUTO)) )
    {
       if(0u != (advertize_reg & MII_NWAY_FD10))
	    {
	    PhyMode |= NWAY_FD10;
	    }
       if(0u != (advertize_reg & MII_NWAY_HD10))
	    {
	    PhyMode |= NWAY_HD10;
	    }
       
    }
    return PhyMode;
}
/* cpsw_MDIO_Tic is called every 10 mili seconds to process Phy states         */

int cpsw_MDIO_Tic(PHY_DEVICE *PhyDev,Uint32* mdioStatus)
{
  Uint32 *PhyState = &PhyDev->PhyState;
  Uint32  CurrentState;
  Int32   ret = 0;

  *mdioStatus = MDIO_EVENT_NOCHANGE;
  
  /*Act on current state of the Phy */
  CurrentState=*PhyState;
  switch(CurrentState&PHY_STATE_MASK)
    {
    case INIT:         _cpsw_MDIO_InitState(PhyDev);      break;
    case FINDING:      _cpsw_MDIO_FindingState(PhyDev);   break;
    case FOUND:        _cpsw_MDIO_FoundState(PhyDev);     break;
    case NWAY_START:   _cpsw_MDIO_NwayStartState(PhyDev); break;
    case NWAY_WAIT:    _cpsw_MDIO_NwayWaitState(PhyDev);  break;
    case LINK_WAIT:    _cpsw_MDIO_LinkWaitState(PhyDev);  break;
    case LINKED:       _cpsw_MDIO_LinkedState(PhyDev);    break;
    case LOOPBACK:     _cpsw_MDIO_LoopbackState(PhyDev);  break;
    default:           _cpsw_MDIO_DefaultState(PhyDev);   break;
    }

  /*Dump state info if a change has been detected                            */
#ifdef __CPSW_MDIO_DEBUG
  if ((CurrentState & (~PHY_TIM_MASK))!=( (*PhyState)&(~PHY_TIM_MASK) ))
    {
    _cpsw_MDIO_DumpState(PhyDev);
    }
#endif

  /*  Check is MDI/MDIX mode switch is needed */
  if(0u != ((*PhyState) & PHY_MDIX_SWITCH) )
    {
    Uint32 Mdix;

    *PhyState &= (~PHY_MDIX_SWITCH);  /* Clear Mdix Flip indicator */

    if(0u != ((*PhyState) & PHY_MDIX))
	{
      Mdix = 1;
	}
    else
	{
      Mdix = 0;
    }
    ret = (Int32)(_MIIMDIO_MDIXFLIP | Mdix);
    }
  else
    {

  /*Return state change to user */
  /** CHECK : report MDIO_LINKEVENTS as MDIO_EVENT_NOCHANGE, MDIO_EVENT_LINKDOWN,
   * MDIO_EVENT_PHYERROR,MDIO_EVENT_LINKUP
   * Currently PHY_CHNG_MASK set for any state transition*/
   
    if (0u != ((*PhyState) & PHY_CHNG_MASK))
   {
   	if(  ( ((*PhyState) & PHY_STATE_MASK) == LINKED) && ((CurrentState & PHY_STATE_MASK) != LINKED) )
   		{
   		/* we have just entered LInked state */
   		*mdioStatus = MDIO_EVENT_LINKUP;
   		}
   	if(  ((CurrentState & PHY_STATE_MASK) == LINKED) && ( ( (*PhyState) & PHY_STATE_MASK) != LINKED))
   		{
   		/* we started in Linked state and we have a state change */
   		*mdioStatus = MDIO_EVENT_LINKDOWN;
   		}
    	*PhyState &= (~PHY_CHNG_MASK);
    	ret = (Int32)(TRUE);
    }
   else
   {
    	ret = (Int32)(FALSE);
	}
    }
   	
  return ret;

}


void cpsw_MDIO_LinkChange(PHY_DEVICE *PhyDev)
  {
  Uint32 *PhyState = &PhyDev->PhyState;
  Uint32  PhyNum,PhyStatus;

  PhyNum = ( (*PhyState) & PHY_DEV_MASK) >> PHY_DEV_OFFSET;

  if (0u != (cpsw_MDIO_GetLinked(PhyDev)) )
    {
    PhyStatus=_cpsw_MDIO_UserAccessRead(PhyDev, PHY_STATUS_REG, PhyNum);

    if ((PhyStatus & MII_PHY_LINKED) == 0u)
      {
      *PhyState &= (~(PHY_TIM_MASK | PHY_STATE_MASK));
      if (0u!=((*PhyState) & SMODE_AUTO))
        {
        _cpsw_MDIO_UserAccessWrite(PhyDev, PHY_CONTROL_REG, PhyNum, MII_AUTO_NEGOTIATE_EN|MII_RENEGOTIATE);
        *PhyState |= ((PHY_CHANGE | PHY_NWST_TO) | NWAY_START);
        }
       else
        {
        *PhyState|= ((PHY_CHANGE | PHY_LINK_TO) | LINK_WAIT);
        }
      }
    }
  }


  /* returns 0 if current Phy has AutoMdix support, otherwise 0 */
static int _cpsw_MDIO_MdixSupported(PHY_DEVICE *PhyDev)
  {
  Uint32 *PhyState = &PhyDev->PhyState;
  Uint32 PhyNum;
  Int32 ret;

  if((PhyDev->PhyMode & NWAY_AUTOMDIX) == 0)
  {
    ret = (0);  /* AutoMdix not turned on */
  }
  else
  {
    PhyNum = ((*PhyState) & PHY_DEV_MASK) >> PHY_DEV_OFFSET;
  if( ((1<<PhyNum) & PhyDev->MdixMask) == 0)
    {
	ret = 0;  /*  Phy does not support AutoMdix*/
    }

    ret = 1;
  }
  return ret;
  }

/* If current Phy has AutoMdix support add Mdix Delay to the Timer State Value */
static void _cpsw_MDIO_MdixDelay(PHY_DEVICE *PhyDev)
  {
  Uint32 Delay;
  Uint32 *PhyState = &PhyDev->PhyState;


  if(_cpsw_MDIO_MdixSupported(PhyDev) != 0)
    {
  /* Get the Delay value in milli-seconds and convert to ten-milli second value */
  Delay = cpswRandomRange(_AUTOMDIX_DELAY_MIN, _AUTOMDIX_DELAY_MAX);
  Delay /= 10;

  /*  Add AutoMidx Random Switch Delay to AutoMdix Link Delay */

  Delay += (PHY_MDIX_TO>>PHY_TIM_OFFSET);

  /* Change Timeout value to AutoMdix standard */
    *PhyState &= (~(PHY_TIM_MASK));  /* Clear current Time out value */
  *PhyState |=  (Delay<<PHY_TIM_OFFSET);     /* Set new value */
    }
  return;

  }

void _cpsw_MDIO_DisablePhy(PHY_DEVICE *PhyDev,Uint32 PhyNum)
  {
  _cpsw_MDIO_UserAccessWrite(PhyDev, PHY_CONTROL_REG, PhyNum, (MII_PHY_ISOLATE | MII_PHY_PDOWN));

  mdioPrintf("cpsw_MDIO_DisablePhy(%d)\n",PhyNum);
  }

void _cpsw_MDIO_InitState(PHY_DEVICE *PhyDev)
  {
  Uint32 *PhyState = &PhyDev->PhyState;
  Uint32 CurrentState;

  CurrentState=*PhyState;
  CurrentState = (CurrentState & (~PHY_TIM_MASK)) | (PHY_FIND_TO);
  CurrentState=(CurrentState & (~PHY_STATE_MASK)) | (FINDING);
  CurrentState=(CurrentState & (~PHY_SPEED_MASK));
  CurrentState=(CurrentState & (~PHY_DUPLEX_MASK));
  CurrentState|=PHY_CHANGE;

  *PhyState=CurrentState;

  }

void _cpsw_MDIO_FindingState(PHY_DEVICE *PhyDev)
  {
  Uint32 *PhyState = &PhyDev->PhyState;
  Uint32  PhyMask  = PhyDev->PhyMask;
  Uint32  PhyNum,i,j,PhyAcks;
 
  PhyNum=PHY_NOT_FOUND;

  if (0u!= ((*PhyState) & PHY_TIM_MASK))
    {
    *PhyState=( (*PhyState) & (~PHY_TIM_MASK) ) | (( (*PhyState) & PHY_TIM_MASK) - (1 << PHY_TIM_OFFSET));
    }
   else
    {
    PhyAcks=myMDIO_ALIVE;
    PhyAcks&=PhyMask;   /* Only interested in 'our' Phys */
    
    j =1;
    for(i=0; i<32; i++)
    {
	if( 0u != (j & PhyAcks))
	{
	    break;
	}
	j = j << 1;
    }
	
    
    if ((0u != PhyAcks) && (i < 32)) 
	{
	PhyNum=i;
	}
    
    if (PhyNum!=PHY_NOT_FOUND)
      {
      /*  Phy Found! */
      *PhyState = ((*PhyState) & (~PHY_DEV_MASK) ) | ((PhyNum & PHY_DEV_MASK) << PHY_DEV_OFFSET);
      *PhyState = ( (*PhyState) & (~PHY_STATE_MASK) ) | (FOUND);
      *PhyState|=PHY_CHANGE;
      mdioPrintf("cpsw_MDIO_FindingState: PhyNum: %d\n",PhyNum);
      }
     else
      {
      mdioPrintf("cpsw_MDIO_FindingState: Timed Out looking for a Phy!\n");
      *PhyState|=PHY_RECK_TO;  /* Set Timer */
      }
    }
  }

void _cpsw_MDIO_FoundState(PHY_DEVICE *PhyDev)
 {
  Uint32 *PhyState  = &PhyDev->PhyState;
  Uint32  PhyMask   = PhyDev->PhyMask;
  Uint32  MLinkMask = PhyDev->MLinkMask;
  Uint32  PhyNum,PhyStatus,NWAYadvertise,NWAY1000advertise=0,m,phynum,i,j,PhyAcks;
  Uint32  PhySel;
  static  Uint32 auto_neg1 = 0xFFFFFFFF,auto_neg2= 0xFFFFFFFF;


  if (( (*PhyState) & PHY_SMODE_MASK) == 0) 
    {
    return;
    }

  PhyNum = ((*PhyState) & PHY_DEV_MASK) >> PHY_DEV_OFFSET;
  PhyAcks=myMDIO_ALIVE;
  PhyAcks&=PhyMask;   /* Only interested in 'our' Phys */
  
  /* Will now isolate all our Phys, except the one we have decided to use */
  j = 1;
  for(phynum = 0;phynum < 32;phynum++)
    {
    if (0u!= (PhyAcks & j))
    {
        if (phynum != PhyNum)
      {
	    /* Do not disabled Found Phy */
          _cpsw_MDIO_DisablePhy(PhyDev,phynum);
      }
  }
    j = j<<1;
    }
  /* If Aries SWR2 reset then don't reset phy*/ 
  /*  Reset the Phy and proceed with auto-negotiation */
  if(( 0u != g_speed_set) || ((0u ==g_soft_reset_status) || ((auto_neg1 & (1 << PhyNum)) == 0) )  )
  {
      _cpsw_MDIO_ResetPhy(PhyDev,PhyNum);
  }

  /* Now setup the MDIOUserPhySel register */
  PhySel=PhyNum;  /* Set the phy address */
  
  /*  Set the way Link will be Monitored */
  /* Check the Link Selection Method */
  if (0u != ((1 << PhyNum) & MLinkMask) )
    {
    PhySel |= MDIO_USERPHYSEL_LINKSEL;
    }

  myMDIO_USERPHYSEL = PhySel;  /* update PHYSEL */

  /* Get the Phy Status */
  PhyStatus = _cpsw_MDIO_UserAccessRead(PhyDev, PHY_STATUS_REG, PhyNum);

  /*  For Phy Internal loopback test, need to wait until Phy
      found, then set Loopback */
  if (0u != ((*PhyState) & SMODE_LPBK))
    {
    /* Set Phy in Loopback */
    _cpsw_MDIO_UserAccessWrite(PhyDev, PHY_CONTROL_REG, PhyNum, MII_PHY_LOOP|MII_PHY_FD);
    /* Do a read to ensure PHY_LOOP has completed */
    _cpsw_MDIO_UserAccessRead(PhyDev, PHY_STATUS_REG, PhyNum);
    *PhyState = ( (*PhyState) & (~PHY_STATE_MASK) ) | (LOOPBACK);
    *PhyState|=PHY_CHANGE;
    return;
    }

  mdioPrintf("Enable Phy to negotiate external connection\n");
  NWAYadvertise=MII_NWAY_SEL;
  if (0u != ((*PhyState) & SMODE_FD100))
  {
    NWAYadvertise |= MII_NWAY_FD100;
    }
  if (0u != ((*PhyState) & SMODE_HD100))
    {
    NWAYadvertise |= MII_NWAY_HD100;
    }
  if (0u != ((*PhyState) & SMODE_FD10))
    {
    NWAYadvertise |= MII_NWAY_FD10;
    }
  if (0u != ((*PhyState) & SMODE_HD10))
    {
    NWAYadvertise |= MII_NWAY_HD10;
    }
  if (0u != ((*PhyState) & SMODE_FD1000))
    {
    NWAY1000advertise |= MII_NWAY_MY_FD1000;
    }

  *PhyState &= ( ~(PHY_TIM_MASK | PHY_STATE_MASK));
  if ((0u != (PhyStatus & MII_NWAY_CAPABLE)) && (0u != ( (*PhyState) & SMODE_AUTO)) )   /*NWAY Phy Detected*/
  {
    if(( 0u != g_speed_set) || ((0u ==g_soft_reset_status) || ((auto_neg1 & (1 << PhyNum)) == 0) )  )
    {
       /*For NWAY compliant Phys                                                */
      _cpsw_MDIO_UserAccessWrite(PhyDev, NWAY_ADVERTIZE_REG, PhyNum, NWAYadvertise);
       /* for gig negotiation */
      if(0u != _cpswIsGigPhy(PhyDev))
      {
          _cpsw_MDIO_UserAccessWrite(PhyDev, NWAY_1000BT_ADVERTISE_REG, PhyNum, NWAY1000advertise);
      }
      
      mdioPrintf("NWAY Advertising: ");
      if (0u != ((*PhyState) & SMODE_FD1000))
	{
	mdioPrintf("FullDuplex-1000 ");
	}
      if (0u != (NWAYadvertise & MII_NWAY_FD100))
	{
	mdioPrintf("FullDuplex-100 ");
	}
      if (0u != (NWAYadvertise & MII_NWAY_HD100))
	{
	mdioPrintf("HalfDuplex-100 ");
	}
      if (0u != (NWAYadvertise & MII_NWAY_FD10))
	{
	mdioPrintf("FullDuplex-10 ");
	}
      if (0u != (NWAYadvertise & MII_NWAY_HD10))
	{
	mdioPrintf("HalfDuplex-10 ");
	}

      mdioPrintf("\n");
      _cpsw_MDIO_UserAccessWrite(PhyDev, PHY_CONTROL_REG, PhyNum, MII_AUTO_NEGOTIATE_EN);
      _cpsw_MDIO_UserAccessWrite(PhyDev, PHY_CONTROL_REG, PhyNum, MII_AUTO_NEGOTIATE_EN|MII_RENEGOTIATE);
     *PhyState|= ( (PHY_CHANGE | PHY_NWST_TO) | NWAY_START);
    }
    else
    {
      *PhyState |= ( (PHY_CHANGE | PHY_NWST_TO) | NWAY_START);
      auto_neg1 = auto_neg1 & (~(1 << PhyNum));
    }  
  }
  else
  {
    *PhyState &= (~SMODE_AUTO);   /*The Phy is not capable of auto negotiation!  */
    m=NWAYadvertise;
    j = 0x8000u;
    
    for(i=0;(i<16);i++)
    {
	if(0u != (j & m))
	{
	    break;
	}
	j = j >> 1 ;

    }
    m=j;
    j=0;

    /* figure out if gig connected at FD 1000 or not first */
    if ((NWAY1000advertise & MII_NWAY_MY_FD1000)== MII_NWAY_MY_FD1000)
    {
         j = MII_PHY_1000;
         j |= MII_PHY_FD;
    }
    else
    {
        if (0u != (m & (MII_NWAY_FD100 | MII_NWAY_HD100)) )
        {
           j=MII_PHY_100;
           m&=(MII_NWAY_FD100|MII_NWAY_HD100);
        }
        if (0u != (m & (MII_NWAY_FD100 | MII_NWAY_FD10)) )
	    {
           j |= MII_PHY_FD;
    }  
    }  
    mdioPrintf("Requested PHY mode %s Duplex %s Mbps\n",(j & MII_PHY_FD) ? "Full":"Half",
			(j & MII_PHY_1000) ? "1000":((j & MII_PHY_100) ? "100":"10"));
    if((0u != g_speed_set) || ( (0u == g_soft_reset_status) || ((auto_neg2 & (1 << PhyNum)) == 0) )) 
    {  
       if(0u != (j & MII_PHY_1000))
       { 
          _cpsw_MDIO_UserAccessWrite(PhyDev,NWAY_1000BT_ADVERTISE_REG, PhyNum, j);
       }
       else
       {
           _cpsw_MDIO_UserAccessWrite(PhyDev, PHY_CONTROL_REG, PhyNum, j);
           _cpsw_MDIO_UserAccessWrite(PhyDev, NWAY_ADVERTIZE_REG, PhyNum, NWAYadvertise);
       }
       g_speed_set = 0;    
    }
    else
    {
       auto_neg2 = auto_neg2 & (~(1 << PhyNum));
    } 
    *PhyState &= (~PHY_SPEED_MASK);
    if(0u != (j & MII_PHY_1000) )
    {
        *PhyState|=(1<<PHY_DUPLEX_OFFSET);      
        PhyDev->SPEED_1000 = 1; 
    }
    else
    {
        PhyDev->SPEED_1000 = 0;
    } 
    if (0u != (j & MII_PHY_100) )
    {
       *PhyState|=(1<<PHY_SPEED_OFFSET);
    }  
    *PhyState &= (~PHY_DUPLEX_MASK);
    if (0u != (j & MII_PHY_FD) )
    {
       *PhyState|=(1<<PHY_DUPLEX_OFFSET);
    }
    *PhyState |= ((PHY_CHANGE | PHY_LINK_TO) | LINK_WAIT);
  }
    _cpsw_MDIO_MdixDelay(PhyDev);  /* If AutoMdix add delay */
  }

void _cpsw_MDIO_ResetPhy(PHY_DEVICE *PhyDev,Uint32 PhyNum)
  {
  Uint16 PhyControlReg;

  _cpsw_MDIO_UserAccessWrite(PhyDev, PHY_CONTROL_REG, PhyNum, MII_PHY_RESET);

  mdioPrintf("cpsw_MDIO_PhYReset(%d)\n",PhyNum);

  /* Read control register until Phy Reset is complete */
  do
   {
    PhyControlReg = _cpsw_MDIO_UserAccessRead(PhyDev, PHY_CONTROL_REG, PhyNum);
   }
   while (0u != (PhyControlReg & MII_PHY_RESET)); /* Wait for Reset to clear */
  }

void _cpsw_MDIO_NwayStartState(PHY_DEVICE *PhyDev)
  {
  Uint32 *PhyState = &PhyDev->PhyState;
  Uint32 PhyNum,PhyMode;

  PhyNum = ( (*PhyState) & PHY_DEV_MASK) >> PHY_DEV_OFFSET;

  /*Wait for Negotiation to start                                            */

  PhyMode=_cpsw_MDIO_UserAccessRead(PhyDev, PHY_CONTROL_REG, PhyNum);

  if((PhyMode&MII_RENEGOTIATE)==0)
    {
    _cpsw_MDIO_UserAccessRead(PhyDev, PHY_STATUS_REG, PhyNum); /*Flush pending latch bits*/
    *PhyState &= (~(PHY_STATE_MASK|PHY_TIM_MASK));
    *PhyState |= ((PHY_CHANGE | NWAY_WAIT) |PHY_NWDN_TO);
    _cpsw_MDIO_MdixDelay(PhyDev);  /* If AutoMdix add delay */
    }
   else
    {
    if (0u != ((*PhyState) & PHY_TIM_MASK) )
	{
	*PhyState=( (*PhyState) & (~PHY_TIM_MASK)) | ( ( (*PhyState) & PHY_TIM_MASK)-(1 << PHY_TIM_OFFSET));
	}
     else
	{
      _cpsw_MDIO_PhyTimeOut(PhyDev);
    }
  }
  }

void _cpsw_MDIO_NwayWaitState(PHY_DEVICE *PhyDev)
  {
  Uint32 *PhyState = &PhyDev->PhyState;
  Uint32  PhyNum,PhyStatus,NWAYadvertise = 0,NWAYREadvertise = 0,NegMode,i,j;
  Uint32 NWAY1000advertise = 0, NWAY1000REMadvertise = 0;

  PhyNum=( (*PhyState) & PHY_DEV_MASK) >> PHY_DEV_OFFSET;

  PhyStatus=_cpsw_MDIO_UserAccessRead(PhyDev, PHY_STATUS_REG, PhyNum);

  if (0u != (PhyStatus & MII_NWAY_COMPLETE))
    {
    *PhyState|=PHY_CHANGE;
    *PhyState &= (~PHY_SPEED_MASK);
    *PhyState &= (~PHY_DUPLEX_MASK);

    NWAYadvertise   =_cpsw_MDIO_UserAccessRead(PhyDev, NWAY_ADVERTIZE_REG, PhyNum);
    NWAYREadvertise =_cpsw_MDIO_UserAccessRead(PhyDev, NWAY_REMADVERTISE_REG, PhyNum);

    /* read gig status */
    if(0u != _cpswIsGigPhy(PhyDev))
    {
       NWAY1000advertise   =_cpsw_MDIO_UserAccessRead(PhyDev, NWAY_1000BT_ADVERTISE_REG, PhyNum);
       NWAY1000REMadvertise =_cpsw_MDIO_UserAccessRead(PhyDev, NWAY_1000BT_REMADVERTISE_REG, PhyNum);
    }

    /* figure out if gig connected at FD 1000 or not first */
    if ((0u !=(NWAY1000advertise & MII_NWAY_MY_FD1000)) && (0u!=(NWAY1000REMadvertise & MII_NWAY_REM_FD1000)) )
	{
      NegMode = MII_NWAY_MY_FD1000;
	}
    else
	{
	NegMode = 0u;
	}

    if (NegMode == 0u)
      {
	/* continue checking for 100 and 10 connection */

	/* Negotiated mode is we and the remote have in common */
	NegMode = NWAYadvertise & NWAYREadvertise;

	mdioPrintf("Phy: %d, ",( (*PhyState) & PHY_DEV_MASK) >> PHY_DEV_OFFSET);
	mdioPrintf("NegMode %04X, NWAYadvertise %04X, NWAYREadvertise %04X\n",
		   NegMode,  NWAYadvertise, NWAYREadvertise);

	/* Limit negotiation to fields below */
	NegMode &= ( (MII_NWAY_FD100|MII_NWAY_HD100) | (MII_NWAY_FD10|MII_NWAY_HD10));
	
	if (NegMode == 0u)
	  {
	    NegMode=(MII_NWAY_HD100|MII_NWAY_HD10)&NWAYadvertise; /*or 10 ?? who knows, Phy is not MII compliant*/
	    
	    mdioPrintf("Mdio:WARNING: Negotiation complete but NO agreement, default is HD\n");
#ifdef __CPSW_MDIO_DEBUG
	    _cpsw_MDIO_DumpPhyDetailed(PhyDev);
#endif
	  }

	j = 0x8000u;
	for(i = 0; i <16 ; i++)
	{
	    if (0u != (j & NegMode))
	    {
		break;
	    }
	    j = j >> 1;
	}

	NegMode=j;

	mdioPrintf("Negotiated connection: ");
	if (0u != (NegMode & MII_NWAY_FD100))
	    {
	    mdioPrintf("FullDuplex 100 Mbs\n");
	    }
	if (0u != (NegMode & MII_NWAY_HD100))
	    {
	    mdioPrintf("HalfDuplex 100 Mbs\n");
	    }
	if (0u != (NegMode & MII_NWAY_FD10))
	    {
	    mdioPrintf("FullDuplex 10 Mbs\n");
	    }
	if (0u != (NegMode & MII_NWAY_HD10))
	    {
	    mdioPrintf("HalfDuplex 10 Mbs\n");
	    }

	PhyDev->SPEED_1000 = 0u;
      }
    else
      {
	/* found 1000 negotiated connection! */
	mdioPrintf("Negotiated connection: ");
	mdioPrintf("FullDuplex 1000 Mbs\n");

	PhyDev->SPEED_1000 = 1u;
      }

    if (NegMode != 0u)
      {
      if (0u != (PhyStatus & MII_PHY_LINKED))
	{
        *PhyState=( (*PhyState) & (~PHY_STATE_MASK)) | LINKED;
	}
       else
	{
        *PhyState=( (*PhyState) & (~PHY_STATE_MASK)) | LINK_WAIT;
	}
      if (0u != (NegMode & (MII_NWAY_FD100 | MII_NWAY_HD100)) )
	{
        *PhyState=( (*PhyState) & (~PHY_SPEED_MASK)) | (1 << PHY_SPEED_OFFSET);
	}
      if (0u != (NegMode & (MII_NWAY_FD100 | MII_NWAY_FD10 | MII_NWAY_MY_FD1000)))
	{
        *PhyState=( (*PhyState) & (~PHY_DUPLEX_MASK)) | (1 << PHY_DUPLEX_OFFSET);
	}
      }
    }
   else
    {
    if ( 0u != ( (*PhyState) & PHY_TIM_MASK) )
	{
	*PhyState=( (*PhyState) & (~PHY_TIM_MASK)) | (( (*PhyState) & PHY_TIM_MASK)-(1 << PHY_TIM_OFFSET));
	}
     else
	{
      _cpsw_MDIO_PhyTimeOut(PhyDev);
    }
  }
  }

void _cpsw_MDIO_LinkWaitState(PHY_DEVICE *PhyDev)
  {
  Uint32 *PhyState = &PhyDev->PhyState;
  Uint32  PhyStatus;
  Uint32  PhyNum;

  PhyNum=( (*PhyState) & PHY_DEV_MASK) >> PHY_DEV_OFFSET;

  PhyStatus=_cpsw_MDIO_UserAccessRead(PhyDev, PHY_STATUS_REG, PhyNum);

  if (0u != (PhyStatus & MII_PHY_LINKED) )
    {
    *PhyState=( (*PhyState) & (~PHY_STATE_MASK) ) | LINKED;
    *PhyState|=PHY_CHANGE;
    }
   else
    {
    if (0u != ( (*PhyState) & PHY_TIM_MASK)  )
	{
      *PhyState=(*PhyState&~PHY_TIM_MASK)|((*PhyState&PHY_TIM_MASK)-(1<<PHY_TIM_OFFSET));
	}
     else
	{
      _cpsw_MDIO_PhyTimeOut(PhyDev);
    }
  }
  }

void _cpsw_MDIO_PhyTimeOut(PHY_DEVICE *PhyDev)
  {
  Uint32 *PhyState;

  if(_cpsw_MDIO_MdixSupported(PhyDev) == 0)
    {
    return;  /* AutoMdix not supported */
    }

  PhyState = &PhyDev->PhyState;

  /*  Indicate MDI/MDIX mode switch is needed */
  *PhyState|=PHY_MDIX_SWITCH;

  /* Toggle the MDIX mode indicatir */
  if(0u != ( (*PhyState) & PHY_MDIX) )
    {
    *PhyState &= (~PHY_MDIX_MASK);       /* Current State is MDIX, set to MDI */
    }
  else
    {
    *PhyState |=  PHY_MDIX_MASK;      /* Current State is MDI, set to MDIX */
    }

  /* Reset state machine to FOUND */
  *PhyState = ( (*PhyState) & (~PHY_STATE_MASK)) | (FOUND);
  }

void _cpsw_MDIO_LoopbackState(PHY_DEVICE *PhyDev)
{
    (void)PhyDev; /* remove if not needed, to avoid warning */
  return;
}

void _cpsw_MDIO_LinkedState(PHY_DEVICE *PhyDev)
  {
  Uint32 *PhyState = &PhyDev->PhyState;
  Uint32  PhyNum   = ( (*PhyState) & PHY_DEV_MASK) >> PHY_DEV_OFFSET;
  
  if (0u != (myMDIO_LINK & (1 << PhyNum)))
    {
     return;  /* if still Linked, exit*/
    }

  /* Not Linked */
  *PhyState &= (~(PHY_STATE_MASK | PHY_TIM_MASK));
  if (0u != ( (*PhyState) & SMODE_AUTO) )
  {
    *PhyState |= ( (PHY_CHANGE | NWAY_WAIT) | PHY_NWDN_TO);
  }
   else
    {
    *PhyState |= ( (PHY_CHANGE | PHY_LINK_TO) | LINK_WAIT);
    }

  _cpsw_MDIO_MdixDelay(PhyDev);  /* If AutoMdix add delay */
  }

void _cpsw_MDIO_DefaultState(PHY_DEVICE *PhyDev)
  {
  Uint32 *PhyState = &PhyDev->PhyState;
  /*Awaiting a cpsw_MDIO_Init call                                             */
  *PhyState|=PHY_CHANGE;
  }

/* Simple Query Functions for reporting speed,duplex */
/* cpsw_MDIO_GetDuplex is called to retrieve the Duplex info */
int cpsw_MDIO_GetDuplex(PHY_DEVICE *PhyDev)
  {
  Uint32 *PhyState = &PhyDev->PhyState;
  return(( (*PhyState) & PHY_DUPLEX_MASK) ? 1:0);  /* return 0 or a 1  */
  }

/* cpsw_MDIO_GetSpeed is called to retreive the Speed info */
int cpsw_MDIO_GetSpeed(PHY_DEVICE *PhyDev)
  {
  Uint32 *PhyState = &PhyDev->PhyState;
  Int32 ret = 0 ;

  if (PhyDev->SPEED_1000 == 1)
    {
    ret = 2; /* if we are linked at 1000, we return a value of 2 */
    }
  else
    {
    ret = ( (*PhyState) & PHY_SPEED_MASK);
    }

  return ret;
  }

/* cpsw_MDIO_GetPhyNum is called to retreive the Phy Device Adr info*/
int cpsw_MDIO_GetPhyNum(PHY_DEVICE *PhyDev)
  {
  Uint32 *PhyState = &PhyDev->PhyState;
  return((Int32)(( (*PhyState) & PHY_DEV_MASK) >> PHY_DEV_OFFSET));
  }

/* cpsw_MDIO_GetLoopback is called to Determine if the LOOPBACK state has been reached*/
int cpsw_MDIO_GetLoopback(PHY_DEVICE *PhyDev)
  {
  Uint32 *PhyState = &PhyDev->PhyState;
  return(( (*PhyState) & PHY_STATE_MASK) == LOOPBACK);
  }

/* cpsw_MDIO_GetLinked is called to Determine if the LINKED state has been reached*/
int cpsw_MDIO_GetLinked(PHY_DEVICE *PhyDev)
  {
    Uint32 *PhyState = &PhyDev->PhyState;
    return (( (*PhyState) & PHY_STATE_MASK) == LINKED);
  }

void cpsw_MDIO_GetVer(Uint32 miibase, Uint32 *ModID,  Uint32 *RevMaj,  Uint32 *RevMin)
{
  Uint32  Ver;

  Ver = MDIO_VER(miibase);

  *ModID  = (Ver & MDIO_VER_MODID) >> 16;
  *RevMaj = (Ver & MDIO_VER_REVMAJ) >> 8;
  *RevMin = (Ver & MDIO_VER_REVMIN);
}

int cpsw_MDIO_GetPhyDevSize(void)
{
  return(sizeof(PHY_DEVICE));
}


/************************************
***
*** Waits for MDIO_USERACCESS to be ready and reads data
*** If 'WaitForData' set, waits for read to complete and returns Data,
*** otherwise returns 0
*** Note: 'data' is 16 bits but we use 32 bits
***        to be consistent with rest of the code.
***
**************************************/
Uint32 _cpsw_MDIO_UserAccessRead(PHY_DEVICE *PhyDev, Uint32 regadr, Uint32 phyadr)
  {

  _cpsw_MDIO_WaitForAccessComplete(PhyDev);  /* Wait until UserAccess ready */
  _cpsw_MDIO_UserAccess(PhyDev, MDIO_USERACCESS_READ, regadr, phyadr, 0);
  _cpsw_MDIO_WaitForAccessComplete(PhyDev);  /* Wait for Read to complete */

  return(myMDIO_USERACCESS & MDIO_USERACCESS_DATA);
  }


/************************************
***
*** Waits for MDIO_USERACCESS to be ready and writes data
***
**************************************/
void _cpsw_MDIO_UserAccessWrite(PHY_DEVICE *PhyDev, Uint32 regadr, Uint32 phyadr, Uint32 data)
  {
  _cpsw_MDIO_WaitForAccessComplete(PhyDev);  /* Wait until UserAccess ready */
  _cpsw_MDIO_UserAccess(PhyDev, MDIO_USERACCESS_WRITE, regadr, phyadr, data);
  }

void _cpsw_MDIO_WaitForAccessComplete(PHY_DEVICE *PhyDev)
  {
  while((myMDIO_USERACCESS & MDIO_USERACCESS_GO)!=0u)
    {
    }
  }

void _cpsw_MDIO_UserAccess(PHY_DEVICE *PhyDev, Uint32 method, Uint32 regadr, Uint32 phyadr, Uint32 data)
  {
  Uint32  control;

  control =  MDIO_USERACCESS_GO |
             (method) |
             (((regadr) << 21u) & MDIO_USERACCESS_REGADR) |
             (((phyadr) << 16u) & MDIO_USERACCESS_PHYADR) |
             ((data) & MDIO_USERACCESS_DATA);

  myMDIO_USERACCESS = control;
  }

#ifdef __CPSW_MDIO_DEBUG
/** Debug print functions , exported for Apps */
void _cpsw_MDIO_DumpState(PHY_DEVICE *PhyDev)
  {
  Uint32 state    = PhyDev->PhyState;

  mdioPrintf("Phy: %d, ",(state&PHY_DEV_MASK)>>PHY_DEV_OFFSET);
  mdioPrintf("State: %d/%s, ",(state & PHY_STATE_MASK) >> PHY_STATE_OFFSET,
		  lstate[(state & PHY_STATE_MASK) >> PHY_STATE_OFFSET]);
  mdioPrintf("Speed: %d, ",(state&PHY_SPEED_MASK)>>PHY_SPEED_OFFSET);
  mdioPrintf("Dup: %d, ",(state&PHY_DUPLEX_MASK)>>PHY_DUPLEX_OFFSET);
  mdioPrintf("Tim: %d, ",(state&PHY_TIM_MASK)>>PHY_TIM_OFFSET);
  mdioPrintf("SMode: %d, ",(state&PHY_SMODE_MASK)>>PHY_SMODE_OFFSET);
  mdioPrintf("Chng: %d",(state&PHY_CHNG_MASK)>>PHY_CHNG_OFFSET);
  mdioPrintf("\n");

  if (((state & PHY_STATE_MASK) != FINDING) && 
	((state & PHY_STATE_MASK) != INIT))
    {
    _cpsw_MDIO_DumpPhy(PhyDev, (state&PHY_DEV_MASK)>>PHY_DEV_OFFSET);
  }
  }

void _cpsw_MDIO_DumpPhy(PHY_DEVICE *PhyDev, Uint32 p)
  {
  Uint32 j,n,PhyAcks;
  Uint32 PhyRegAddr;
  Uint32 phy_num;
  Uint32 PhyMask  = PhyDev->PhyMask;

  PhyAcks=myMDIO_ALIVE;
  PhyAcks&=PhyMask;   /* Only interested in 'our' Phys */

  phy_num = 0;
  for(j=1;phy_num<32;phy_num++)
    {
    if (0u != (PhyAcks & j))
      {
      mdioPrintf("%2d%s:",phy_num,(phy_num==p)?">":" ");
      for(PhyRegAddr=0;PhyRegAddr<6;PhyRegAddr++)
        {
        n = _cpsw_MDIO_UserAccessRead(PhyDev, PhyRegAddr, phy_num);
        mdioPrintf(" %04x",( n & 0x0ffffu));
        }/* end of for loop */
      mdioPrintf("\n");
      }/* end of if conditional loop */
    j <<= 1;
    }/* end of for phy_num loop */
  _cpsw_MDIO_DumpPhyDetailed(PhyDev);
  }

void _cpsw_MDIO_DumpPhyDetailed(PHY_DEVICE *PhyDev)
{
  Uint32 *PhyState = &PhyDev->PhyState;
  Uint32  PhyNum;
  int     RegData, RegData2;

  PhyNum=( (*PhyState) & PHY_DEV_MASK) >> PHY_DEV_OFFSET;

  RegData = _cpsw_MDIO_UserAccessRead(PhyDev, 0, PhyNum);
  mdioPrintf("PhyControl: %04X, Lookback=%s, Speed=%s, Duplex=%s\n",
    RegData,
    (RegData & MII_PHY_LOOP) ? "On":"Off",
    (RegData & MII_PHY_1000) ? "1000" : (RegData & MII_PHY_100 ? "100":"10"),
    (RegData & MII_PHY_FD ) ? "Full":"Half");
  RegData = _cpsw_MDIO_UserAccessRead(PhyDev, 1, PhyNum);
  mdioPrintf("PhyStatus: %04X, AutoNeg=%s, Link=%s\n",
    RegData,
    (RegData & MII_NWAY_COMPLETE) ? "Complete":"NotComplete",
    (RegData & MII_PHY_LINKED) ? "Up":"Down");
  RegData = _cpsw_MDIO_UserAccessRead(PhyDev, 4, PhyNum);
  RegData2 = _cpsw_MDIO_UserAccessRead(PhyDev, 9, PhyNum);
  mdioPrintf("PhyMyCapability: %04X, 1000FD=%s, 100FD=%s, 100HD=%s, 10FD=%s, 10HD=%s\n",
    RegData,
    (RegData2 & MII_NWAY_MY_FD1000) ? "Yes":"No",
    (RegData & MII_NWAY_FD100) ? "Yes":"No",
    (RegData & MII_NWAY_HD100) ? "Yes":"No",
    (RegData & MII_NWAY_FD10) ? "Yes":"No",
    (RegData & MII_NWAY_HD10) ? "Yes":"No");

  RegData = _cpsw_MDIO_UserAccessRead(PhyDev, 5, PhyNum);
  RegData2 = _cpsw_MDIO_UserAccessRead(PhyDev, 10, PhyNum);
  mdioPrintf("PhyPartnerCapability: %04X, 1000FD=%s, 100FD=%s, 100HD=%s, 10FD=%s, 10HD=%s\n",
    RegData,
    (RegData2 & MII_NWAY_REM_FD1000 ) ? "Yes":"No",
    (RegData & MII_NWAY_FD100) ? "Yes":"No",
    (RegData & MII_NWAY_HD100) ? "Yes":"No",
    (RegData & MII_NWAY_FD10) ? "Yes":"No",
    (RegData & MII_NWAY_HD10) ? "Yes":"No");
}

#endif
