/*
 *  Copyright 2007 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) TCP/IP_Network_Developers_Kit 1.92.00.13 
 */
/**
 *--------------------------------------------------------------------------
 * IP Stack
 *--------------------------------------------------------------------------
 * llPacket.c
 *
 * Packet Driver for CPSW
 *
 * Author: Based on llpacket interface by Michael A. Denio
 * Copyright 1998, 2007 by Texas Instruments Inc.
 *-------------------------------------------------------------------------
 */
#include <stkmain.h>
#include "llpacket.h"
#include "instrumentation.h"

/**
 *--------------------------------------------------------------------
 * PUBLIC FUNCTIONS USED FOR INTIALIZATION AND EVENTS
 *--------------------------------------------------------------------
 */
#define MAX_INSTANCE    2u

static Uint32     PktDevCount = 0u;        /* Number of Net Devices */
static PDINFO   pdi[MAX_INSTANCE];      /* Private Info about devices */

/**
 * Queue for Received Packets. Only 1 Rx queue irrespective of number of interfaces.
 */
PBMQ    PBMQ_rx;

/**
 *--------------------------------------------------------------------
 * PUBLIC FUNCTIONS USED BY NETCTRL
 *--------------------------------------------------------------------
 */

/**
 *--------------------------------------------------------------------
 * _llPacketInit()
 *
 * Opens the packet driver environment and enumerates devices
 *--------------------------------------------------------------------
 */
Uint32 _llPacketInit(STKEVENT_Handle hEvent)
{
    Uint32 i;

    /**
     * Initialize RX Queue
     */
    PBMQ_init(&PBMQ_rx);

    /**
     * Initialize the packet driver(s)
     */
    PktDevCount = HwPktInit();

    if( PktDevCount > MAX_INSTANCE )
    {
        PktDevCount = MAX_INSTANCE;
    }

    /**
     * Initialize PDINFO for each driver instance
     */
    for(i=0; i<PktDevCount; i++)
    {
        mmZeroInit( &pdi[i], sizeof( PDINFO ) );

        /**
         * Set physical index
         */
        pdi[i].PhysIdx     = i;

        /**
         * Copy Event Handle
         */
        pdi[i].hEvent      = hEvent;

        /**
         * MCast List
         */
        pdi[i].MCastCnt    = 0u;

        /**
         * Default MAC Address (can be overwritten by HwPktOpen())
         */
        pdi[i].bMacAddr[0] = ((Uint8) 0x08u);
        pdi[i].bMacAddr[1] = ((Uint8) 0x00u);
        pdi[i].bMacAddr[2] = (Uint8) 0x28u;
        pdi[i].bMacAddr[3] = (Uint8) 0xFFu;
        pdi[i].bMacAddr[4] = (Uint8) ((Uint32)0xFFu);
        pdi[i].bMacAddr[5] = (Uint8) i;
    }

    return( PktDevCount );
}

/**
 *--------------------------------------------------------------------
 * _llPacketShutdown()
 *
 * Called to shutdown packet driver environment
 *--------------------------------------------------------------------
 */
void _llPacketShutdown()
{
    PktDevCount = 0u;
    HwPktShutdown();

    /**
     * Flush out our pending queue
     */
    while( 0u != PBMQ_count(&PBMQ_rx) )
    {
        PBM_free( PBMQ_deq(&PBMQ_rx) );
    }
}

/**
 *--------------------------------------------------------------------
 * _llPacketServiceCheck()
 *
 * Called to check for packet activity
 *--------------------------------------------------------------------
 */
void _llPacketServiceCheck( Uint32 fTimerTick )
{
    Uint32 dev;

    for( dev=0; dev<PktDevCount; dev++ )
    {
        _HwPktPoll( &(pdi[dev]), fTimerTick );
    }
}

/**
 *--------------------------------------------------------------------
 * PUBLIC FUNCTIONS USED BY THE STACK
 *--------------------------------------------------------------------
 */

/**
 *--------------------------------------------------------------------
 * llPacketOpen()
 *
 * Opens the packet driver, and request packets of our desired type.
 *--------------------------------------------------------------------
 */
Uint32 llPacketOpen( Uint32 dev, HANDLE hEther )
{
    Uint32 ret =0u; 
    
    /**
     * Our device index is "1" based to the upper layers
     */
    dev--;
    
    if (dev < PktDevCount )
    {
       /**
        * Init Logical Device
        */
        pdi[dev].hEther = hEther;

       /**
        * Call low-level open function
        */
        ret =  HwPktOpen( &pdi[dev] );
    }
    
    return ret;
}

/**
 *--------------------------------------------------------------------
 * PacketClose()
 *
 * Called to shutdown packet driver.
 *--------------------------------------------------------------------
 */
void llPacketClose( Uint32 dev )
{
	PBM_Handle hPkt;
	
    /**
     * Our device index is "1" based to the upper layers
     */
    dev--;
    
    /**
     * Call low-level close function
     */
    if (dev < PktDevCount)
    {
        HwPktClose( &pdi[dev] );

	/**CHECK */
        hPkt = PBMQ_deq(&pdi[dev].PBMQ_tx);
        while( (PBM_Handle)0 != hPkt )
        {
        	PBM_free(hPkt);
        	hPkt = PBMQ_deq(&pdi[dev].PBMQ_tx);	
        }

    }

    return;
}/* end of function llPackeClose */

/**
 *--------------------------------------------------------------------
 * llPacketSend()
 *
 * Called to send data to a device. The buffer has been completely
 * formatted by the time this call is made, and is ready for
 * transmit. The dev parameter is the physical destination device.
 *--------------------------------------------------------------------
 */
void llPacketSend( Uint32 dev, PBM_Handle hPkt )
{
    if((PBM_Handle)0 != hPkt)
    {	    
    /**
     * Our device index is "1" based to the upper layers
     */
    dev--;

    if( PBM_getValidLen(hPkt) < 60u )
    {
        PBM_setValidLen( hPkt, 60u );
    }


    /** CHECK : Impact of VLAN tags on max packet size */
    if( (dev < PktDevCount) && (PBM_getValidLen(hPkt) <= 1514u) )
    {
        #ifdef ETH_INSTRUMENTATION_ENABLED
       PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
                PSP_DVT_DESC(PSP_DVT_eFUNC_START, \
                             PSP_DVT_dINST, \
                             PSP_DVT_dCHANNEL_ID, \
                             PSP_DVT_dSIZE_BYTES),0,0,PBM_getValidLen(hPkt));
        #endif

        PBMQ_enq( &(pdi[dev].PBMQ_tx), hPkt );

	#ifdef ETH_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
                PSP_DVT_DESC(PSP_DVT_eFUNC_END, \
                             PSP_DVT_dINST, \
                             PSP_DVT_dCHANNEL_ID, \
                             PSP_DVT_dSIZE_BYTES),0,0,0);
        #endif

	/* call HwPktTx function if Trasmit is on and free */
        if( (0u != (pdi[dev].TxFree) )  &&  (0u != pdi[dev].TxOn) )
        {
            HwPktTxNext( &(pdi[dev]) );
        }
    }
    else
    {
        PBM_free( hPkt );
    }
}
}

/**
 *--------------------------------------------------------------------
 * llPacketGetMacAddr()
 *
 * Called to the MAC address of the specified device.
 *--------------------------------------------------------------------
 */
void llPacketGetMacAddr( Uint32 dev, UINT8 *pbData )
{
    if((Uint8*)0 != pbData)
	{
    /**
     * Our device index is "1" based to the upper layers
     */
    dev--;

    if( dev < PktDevCount )
    {
        mmCopy( pbData, pdi[dev].bMacAddr, 6u );
    }
}
}

/**
 *--------------------------------------------------------------------
 * llPacketGetMCastMax()
 *
 * Called to the maxumim multicast addresses that the device can hold.
 *--------------------------------------------------------------------
 */
Uint32 llPacketGetMCastMax( Uint32 dev )
{
    Uint32 ret = 0u;    
    /**
     * Our device index is "1" based to the upper layers
     */
    dev--;

    if( dev < PktDevCount )
    {
        ret = PKT_MAX_MCAST;
    }
    else
    {
        ret = 0u;
    }

    return ret;
}

/**
 *--------------------------------------------------------------------
 * llPacketSetRxFilter()
 *
 * Called to set the Rx Filter mode of the device. The legal modes
 * are defined in ETHER.H
 *--------------------------------------------------------------------
 */
void llPacketSetRxFilter( Uint32 dev, Uint32 Filter )
{
    /**
     * Our device index is "1" based to the upper layers
     */
    dev--;

    if( (dev < PktDevCount) && (Filter <= ETH_PKTFLT_ALL) )
    {
        pdi[dev].Filter = Filter;
        HwPktSetRx( &(pdi[dev]) );
    }
}

/**
 *--------------------------------------------------------------------
 * llPacketSetMCast()
 *
 * Called to set the Multicast list for the device.
 *--------------------------------------------------------------------
 */
void llPacketSetMCast( Uint32 dev, Uint32 addrcnt, UINT8 *bAddr )
{
    if((Uint8*)0 != bAddr)
	{
    /**
     * Our device index is "1" based to the upper layers
     */
    dev--;

    /**
     * Copy the multicast list from ETH and update HW driver
     */
    if( (dev < PktDevCount) && (addrcnt <= PKT_MAX_MCAST) )
    {
        pdi[dev].MCastCnt = addrcnt;
        mmCopy( pdi[dev].bMCast, bAddr, 6u * addrcnt );
        HwPktSetRx( &(pdi[dev]) );
    }
}
}

/**
 *--------------------------------------------------------------------
 * llPacketGetMCast()
 *
 * Called to get the Multicast list for the device.
 *--------------------------------------------------------------------
 */
Uint32 llPacketGetMCast( Uint32 dev, Uint32 max, UINT8 *bAddr )
{
    Uint32 ret = 0u;
    
    if((Uint8*)0 == bAddr)
	{
	ret = 1u;
	}
    else
	{
    
    /**
     * Our device index is "1" based to the upper layers
     */
    dev--;

    /**
     * Copy the multicast list to ETH
     */
    if( ((dev < PktDevCount) && (max >= pdi[dev].MCastCnt)) && ((UINT8*)0 != bAddr) )
    {
        mmCopy( bAddr, pdi[dev].bMCast, pdi[dev].MCastCnt * 6u );
        ret = pdi[dev].MCastCnt;
    }
	}
    return(ret);
}

/**
 *--------------------------------------------------------------------
 * llPacketService()
 *
 * Called to service packet activity.
 *--------------------------------------------------------------------
 */
void llPacketService()
{
    /**
     * Give all queued packets to the Ether module
     */
    while( 0u != PBMQ_count(&PBMQ_rx) )
    {
        EtherRxPacket( PBMQ_deq(&PBMQ_rx) );
    }
}


/**
 * ------------------------------------------------------------------
 *  llPacketIoctl()
 *
 *  Generic Ioctl interface for devicec specific commands/configuration
 *
 *  -----------------------------------------------------------------
 */
void llPacketIoctl(Uint32 dev,Uint32 cmd, void *param)
{
    /**
     * Our device index is "1" based to the upper layers
     */
    dev--;

    HwPktIoctl(&pdi[dev], cmd, param);
}/* end of function llPacketIoctl */

