/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/

/**     \file      ddc_mcasp.c
 *
 *      \brief      ddc for the  McASP.
 *
 *      This file implements the driver core for McASP peripheral
 *      This file needs appropriate support files/function (PALOS, PALSYS , some
 *      DDA (register isr etc.)functions to work properly
 *      The file (driver) has code for configure the mcasp , to perform IO in
 *      in EDMA mode. Interrupt mode code is the to perform I/O but due to
 *      latency and timing constraints underrun/error may occur.
 *      This driver supports multiple serialisers (for buffer handling check
 *      documentation of the payload structure ) and all modes supported by mcasp
 *      peripheral. Please do note that this driver may require expert configuration
 *      during initialisation.
 *
 *      (C)      Copyright 2006,      Texas Instruments, Inc
 *
 *  \author Platform Support Group
 *
 *  \Revision
 *      0.1  Created  :created for DRA44X -presilicon
 *      0.2  modified :Dynamic mode start/stop implemented
 *      0.3  modified :Modified for EDMA - DM64LC Presilicon
 *      0.4  modified :cleanup / DM64Lc post silicon changes
 *                    :implemented AB sync mode of EDMA for multiserialise support
 */



#include <ti/sdo/pspdrivers/drivers/mcasp/src/ddc_mcasp.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osProtect.h>
#include <ti/sdo/pspdrivers/pal_os/pal_oscache.h>
#include <ti/sdo/pspdrivers/pal_os/pal_ossem.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osint.h>
#include <ti/sdo/pspdrivers/pal_os/pal_osmem.h>
#include <ti/sdo/pspdrivers/drivers/mcasp/src/psp_mcaspApi.h>
#include <ti/sdo/pspdrivers/drivers/mcasp/src/llc_mcasp.h>
#include <ti/sdo/edma3/drv/edma3_drv.h>

#ifdef MCASP_INSTRUMENTATION_ENABLED
#include  <ti/sdo/pspdrivers/pal_os/bios/psp_log.h>
#endif /* MCASP_INSTRUMENTATION_ENABLED */

/*
 * Local functions for ddc_mcasp.c
 */
static void ddc_copyIOPtoReq (DDC_mcaspIOP * const ioPacket,PSP_mcaspReq * const pspRequest);
static void ddc_copyReqtoIOP (PSP_mcaspReq * const pspRequest,DDC_mcaspIOP * const ioPacket);

static void ddc_isrInput (Ptr iohandlep);
static void ddc_isrOutput (Ptr iohandlep);

static void ddc_xmtMcaspIsr (DDC_mcaspIoHandle ioHandle);
static void ddc_rcvMcaspIsr (DDC_mcaspIoHandle ioHandle);

static PAL_Result ddc_isValidPacket (DDC_mcaspIoHandle ioHandle);
static PAL_Result ddc_edmaProcessPkt(Ptr iohandlep, DDC_mcaspIOP *ioPacket);
static PAL_Result ddc_xmtIsrProcessPkt (Ptr iohandlep, DDC_mcaspIOP * const ioPacket);
static PAL_Result ddc_rcvIsrProcessPkt (Ptr iohandlep, DDC_mcaspIOP * const ioPacket);
static PAL_Result ddc_mcaspSubmitReq(DDC_mcaspIoHandle ioHandle,PSP_mcaspReq *pspRequestp);
static PAL_Result ddc_updateDataPacketTolinkParams(DDC_mcaspIoHandle ioHandle, DDC_mcaspIOP * const ioPacket);
static PAL_Result ddc_setupEDMADuringInit (DDC_mcaspIoHandle ioHandle);
static PAL_Result ddc_edmaChannelPaRAMsetup(DDC_mcaspIoHandle ioHandle);

static PAL_Result ddc_ConfigureRCVchannel( DDC_mcaspDeviceHandle deviceHandle, Ptr rcvParams);
static PAL_Result ddc_ConfigureXMTChannel( DDC_mcaspDeviceHandle deviceHandle, Ptr xmtParams);

static void ddc_activateSmFsForXmt (DDC_mcaspDeviceHandle deviceHandle);
static void ddc_activateSmFsForRcv (DDC_mcaspDeviceHandle deviceHandle);
static void ddc_activateSmFsForBoth (DDC_mcaspDeviceHandle deviceHandle);
static void ddc_deactivateSmFsForXmt (DDC_mcaspDeviceHandle deviceHandle);
static void ddc_deactivateSmFsForRcv (DDC_mcaspDeviceHandle deviceHandle);


static void ddc_multiSerInitializationRCV ( DDC_mcaspDeviceHandle deviceHandle, Ptr iohandlep);
static void ddc_multiSerInitializationXMT ( DDC_mcaspDeviceHandle deviceHandle, Ptr iohandlep);
static void ddc_multiSerGlobalRestore ( Ptr iohandlep );
static Bool ddc_isRequestedSerzsFree ( DDC_mcaspDeviceHandle deviceHandle, Ptr chanParams);

static void ddc_bitSetGblCtl(DDC_mcaspDeviceHandle deviceHandle, Uint32 bitMaskVal);
static void ddc_bitSetGblRCtl(DDC_mcaspDeviceHandle deviceHandle, Uint32 bitMaskVal);
static void ddc_bitSetGblXCtl(DDC_mcaspDeviceHandle deviceHandle, Uint32 bitMaskVal);
static void ddc_bitRemoveGblRCtl(DDC_mcaspDeviceHandle deviceHandle, Uint32 bitMaskVal);
static void ddc_bitRemoveGblXCtl(DDC_mcaspDeviceHandle deviceHandle, Uint32 bitMaskVal);

static void ddc_completeCurrentIO (DDC_mcaspIoHandle ioHandle);
static void ddc_cancelAndAbortAllIo (DDC_mcaspIoHandle ioHandle);

static void ddc_edmaMcaspCallback(Uint tcc, EDMA3_RM_TccStatus status, Ptr data);
static void ddc_abortReset(DDC_mcaspIoHandle ioHandle);
static void ddc_smReset(DDC_mcaspIoHandle ioHandle);
static void ddc_smSet(DDC_mcaspIoHandle ioHandle);

extern PSP_Result _DDA_McASP_RegisterInt(Ptr chanp, Uint32 intNum, Uint16 evt, pspMcaspIsr intIsr);
extern PSP_Result _DDA_McASP_UnRegisterInt (Uint16 devid, Uint16 evt,Int mode);

/*
These buffers will be used in case if application has not given any data buffer
and not any app specific loop job buffer and not yet stopped the Mcasp
*/
static localTempBuffer loopDstBuf[PSP_MCASP_NUM_PORTS];
static localTempBuffer loopSrcBuf[PSP_MCASP_NUM_PORTS];

/* Buffer used when  mute is ON */
static localTempBuffer muteBuf[PSP_MCASP_NUM_PORTS];


/*Data structure holding the device information*/
static  DDC_mcaspDevice ddcPort[PSP_MCASP_NUM_PORTS];

/**
 * \fn  PSP_Result PSP_mcaspInit(Uint32 instanceId)
 *
 * \brief   Initialization of McAsp configuration
 *
 *  This function initialises the data structures
 *
 * \param     instanceId      [IN]     Device number
 * \return
 *      if successful PSP_SOK , else PSP error code returned
 */
PSP_Result PSP_mcaspInit(Uint32 instanceId)
{
    PSP_Result      result = PSP_SOK;

    /* Zero Initilialize the data structure */
    if(instanceId > PSP_MCASP_NUM_PORTS)
    {
        result = PSP_E_INVAL_PARAM;
    }
    else
    {
        PAL_osMemSet( (Ptr)&ddcPort[instanceId], 0, sizeof(DDC_mcaspDevice));
        PAL_osMemSet( (Ptr)&loopDstBuf[instanceId], 0, sizeof(localTempBuffer));
        PAL_osMemSet( (Ptr)&loopSrcBuf[instanceId], 0, sizeof(localTempBuffer));
        PAL_osMemSet( (Ptr)&muteBuf[instanceId], 0, sizeof(localTempBuffer));

		muteBuf[instanceId].scratchBuffer = (Uint32 *)(((Uint32)muteBuf[instanceId].scratchBuf + 0x7F) & ~0x7F);
		loopSrcBuf[instanceId].scratchBuffer = (Uint32 *)(((Uint32)loopSrcBuf[instanceId].scratchBuf + 0x7F) & ~0x7F);
 		loopDstBuf[instanceId].scratchBuffer = (Uint32 *)(((Uint32)loopDstBuf[instanceId].scratchBuf + 0x7F) & ~0x7F);
    }

    return result;
}


/**
 * \brief   PSP_mcaspCreate
 *
 *  This function initialises the data structures and completes the
 *  ground work needed for PSP_mcaspOpen
 *
 * \param     instanceId      [IN]     Device number (in this case, there are
 *                                     two McASP present 0 and 1)
 * \param     params          [IN]     Device parameter structure that supplies
 *                                     data to initialize the McASP
 * \param     arg             [IN]     application reference pointer
 * \return
 *      if successful PSP_SOK , else PSP error code returned
 */
PSP_Result PSP_mcaspCreate(Uint32 instanceId, PSP_mcaspDevParams * const params, Ptr arg)
{
    Int16                   status          = PSP_SOK;
    Uint32                  ldevid          = instanceId;
    Bool                    falsewhile      = FALSE;
    DDC_mcaspDevice         *deviceHandle   = NULL;
    PSP_Result              ddcResult       = PSP_SOK;

    do
    {
        falsewhile   = FALSE;

        /*Validate parameters*/
        if (((NULL == params)|| ( NULL == arg)))
        {
            ddcResult = PSP_E_INVAL_PARAM;
            break;
        }

		if ((TRUE == ddcPort[instanceId].rxIo.inUse) ||
		    (TRUE == ddcPort[instanceId].txIo.inUse))
		{
		    /* Instance is already in use */
            ddcResult = PSP_E_RESOURCES;
            break;
		}

        deviceHandle = &ddcPort[instanceId];

        ddcPort[instanceId].InstNum = (Uint16)instanceId;

        deviceHandle->appHandle = arg;

        deviceHandle->enablecache = params->enablecache;

        /*
        The following functions fetches some SOC dependent parameters from
        lower layer and this will be used here -e.g. EDMA channel id to used,
        start address of MCASP registers etc.
        */
        ddcResult = (PSP_Result)LLC_mcaspInit(NULL);

        if (PSP_SOK != ddcResult)
        {
            deviceHandle    = NULL;
            ddcResult       = PSP_E_INVAL_PARAM;
            break;
        }

        LLC_mcaspOpen(&(ddcPort[instanceId].hwInfo), ldevid, NULL, &status);


        /*Check if open is successful or not*/
        if (PSP_SOK != status)
        {
            deviceHandle    = NULL;
            ddcResult       = PSP_E_INVAL_PARAM;
            break;
        }


        /*
        Mcasp reset values...Configure registers -these values are passed by
        application as init parameters.All global registers get configured here
        xmt and rcv unit registers are reconfigured during channel create.
        */
        if (PSP_SOK != LLC_mcaspHwSetup(&(deviceHandle->hwInfo), params->mcaspHwSetup))
        {
            deviceHandle = NULL;
            ddcResult = PSP_E_INVAL_PARAM;
            break;
        }

        /*init the inuse flag*/
        ddcPort[instanceId].rxIo.inUse = (Uint16) FALSE;
        ddcPort[instanceId].txIo.inUse = (Uint16) FALSE;


    }while(falsewhile);

    return (ddcResult);
}

/**
 * \brief   PSP_mcaspDelete
 *
 * This function is called by OS during system shutdown.
 *
 * \param     hMcasp   [IN]    Pointer to port object to be freed
 * \param     arg      [IN]    optional channel defined parameter
 *
 * \return    PSP_SOK
 */
PSP_Result PSP_mcaspDelete(Uint32 instanceId,Ptr arg)
{
    DDC_mcaspDeviceHandle deviceHandle      = NULL;
    PSP_Result            ddcResult         = PSP_E_INVAL_PARAM;

    if ( instanceId < PSP_MCASP_NUM_PORTS )
    {
        /* To avoid compiler warning */
        if(NULL != arg)
        {
            arg = arg;
        }
        deviceHandle    = (DDC_mcaspDeviceHandle) &ddcPort[(instanceId)];

        if (NULL != deviceHandle)
        {
            /* Close and free mcasp port handle*/
            ddcResult = (PSP_Result)LLC_mcaspClose(&(deviceHandle->hwInfo));

            if (PSP_SOK != ddcResult)
            {
                deviceHandle    = NULL;
                ddcResult       = PSP_E_INVAL_PARAM;
            }

        }

        /*initialise device object structure - make ready for next init*/
        PAL_osMemSet ( (Ptr)&ddcPort[instanceId], 0, sizeof(DDC_mcaspDevice));

    }

    return (ddcResult);
}



/**
 * \brief PSP_mcaspOpen
 *
 *  This function is called in response to a GIO_create call.
 *
 *      Pre-requisites:
 *       -  Valid chanParams structure
 *          Please note expert configuration is needed for structure and its
 *          members
 *       -  Valid device pointer
 *
 *      What this does:
 *       -  Assigns one of the device channels, if available
 *       -  Assigns requested number of serializers, if available
 *       -  Configures eDMA channel to service the McASP operation,
 *          configures the loopjobs and enables EDMA transfer and starts mcasp
 *          if channel operates in eDMA mode
 *       -  Hooks in function to service McASP interrupts
 *       -  Returns configured channel handle to application.
 *
 *
 * \param     instanceId    [IN]     Instance Id of this peripheral
 * \param     mode          [IN]     channel  mode -> input or output
 * \param     chanparams    [IN]     channel parameters from user or from
 *                                   software above this function
 * \param     drvCbk        [IN]     callback function pointer, driver core will
 *                                   call this function after completion of IO
 * \param     edmaHandle    [IN]     Edma driver handle - if the EDMA mode is enabled,
 *                                   the parameter must not be NULL
 * \param     arg           [IN]     callback function Arguments
 *
 * \return
 *      if successful return PSP_Handle else NULL
 */
PSP_Handle PSP_mcaspOpen(
                Uint32               instanceId,
                PSP_mcaspMode        mode,
                PSP_mcaspChanParams  * const params,
                PSP_mcaspAppCallback drvCbk,
                PSP_Handle           edmaHandle,
                Ptr                  arg
                )

{

    DDC_mcaspDevice         *deviceHandle   = NULL;
    DDC_mcaspIoObj          *ioHandle       = NULL;
    PSP_Result              status          = 0;
    Uint8                    i              = 0;
    PAL_Result              retVal          = PSP_SOK;
    Uint32                  dlbMode         = 0;
    Bool                    falsewhile      = FALSE;
    Uint32                  cookie          = 0;
    Uint32                  mask            = 0;
    EDMA3_DRV_TrigMode      trigMode        = EDMA3_DRV_TRIG_MODE_EVENT;
    PAL_OsSemAttrs          semAttrs        = PAL_OSSEM_ATTRS;
    Uint32                  mod             = 0;


    do
    {
        falsewhile      = FALSE;

        /*
        validate parameters
        */
        if (((NULL == params) || (NULL == drvCbk)))
        {
            ioHandle = NULL;
            break;
        }

        deviceHandle    = (DDC_mcaspDevice *)&ddcPort[instanceId];


        if ((NULL == deviceHandle)|| (NULL == params))
        {
            ioHandle = NULL;
            break;
        }


        if(PSP_MCASP_INPUT == mode )
        {
            ioHandle = (DDC_mcaspIoObj *)&deviceHandle->rxIo;
        }
        else
        {
            ioHandle = (DDC_mcaspIoObj *)&deviceHandle->txIo;
        }

        if (NULL == ioHandle)
        {
            /*
            Control should not come here because the IO objects are instantiated within
            ddcPort structure
            */
            break;
        }

        if ( PSP_MCASP_ALLOCATED == ioHandle->status)
        {
            ioHandle = NULL;
            break;
        }

        /*
        check if requested serialzers are more than device supports
        This is a first level check, later we check what are serialiser
        remaining and if they match the requested serialisers
        */
        if( (params->noOfSerRequested) > (deviceHandle->hwInfo.numOfSerializers) )
        {
            ioHandle = NULL;
            break;
        }

		/* In case of DMA mode, EDMA handle has to be non NULL */
        if ((TRUE == params->isDmaDriven) && (NULL == edmaHandle))
        {
            ioHandle = NULL;
            break;
        }

        /*  check if requested mode (transmit or receive mode) is DIT or TDM*/
        if ((PSP_MCASP_TDM_MODE != params->channelMode) &&
          (PSP_MCASP_DIT_MODE != params->channelMode))
        {
            ioHandle = NULL;
            break;
        }

		if ((PSP_MCASP_DIT_MODE == params->channelMode) &&
           ((PSP_MCASP_OUTPUT != mode) || (PSP_MCASP_WORDLEN_32 != params->wordWidth)))
        {
            ioHandle = NULL;
            break;
        }
		if ((PSP_MCASP_DIT_MODE == params->channelMode) &&
           (FALSE == deviceHandle->hwInfo.ditStatus))
        {   /* if this instance is not supporting DIT mode then return error*/
            ioHandle = NULL;
            break;
        }

        if (TRUE == ioHandle->deviceHandle->enablecache)
        {

            PAL_osCacheFlush(
                        (PAL_osCacheMemAddrSpace)NULL,
                        (Uint32)muteBuf[instanceId].scratchBuffer,
                        sizeof(Uint32)
                        );

            PAL_osCacheFlush(
                        (PAL_osCacheMemAddrSpace)NULL,
                        (Uint32)loopDstBuf[instanceId].scratchBuffer,
                        sizeof(Uint32)
                        );

            PAL_osCacheFlush(
                        (PAL_osCacheMemAddrSpace)NULL,
                        (Uint32)loopSrcBuf[instanceId].scratchBuffer,
                        sizeof(Uint32)
                        );

        }


        /*store the mode for the Object , whether input or output*/
        ioHandle->mode = (Uint16)mode;

        /*
         * set back pointer of channel - for references
         * set callback function and argument - this function will be called
         * back after completion of IO
         */
        ioHandle->deviceHandle  = deviceHandle;
        ioHandle->cbFxn         = drvCbk;
        ioHandle->cbArg         = arg;

        /*
        This will be used in EDMA mode to populate the paramsets in PINGPONG
        */
        ioHandle->nextLinkParamSetToBeUpdated = 0;

        /*
        Assume for writing 32bit with data value = 0, localloopjob data ,
        on the serialiser - ofcourse actual bytes written for each sync event
        is decided by param->ordWidth
        */
        ioHandle->loopJobLength = 4u;

        ioHandle->dataPacket    = NULL;
        ioHandle->flushPacket   = NULL;
		ioHandle->currentPacketErrorStatus = PSP_SOK;

        /*Initialize Mute parameter*/
        ioHandle->bMuteON = FALSE;

        /*Initialize Pause parameter*/
        ioHandle->paused = FALSE;

        /*
         * This semaphore will be used to pend to all TX requests are
         * done - during a flush request
         */
        if ( PAL_SOK != PAL_osSemCreate(
                                    NULL,
                                    0,
                                    &semAttrs,
                                    (PAL_OsSemHandle)&ioHandle->syncSem)
                                    )
        {
            ioHandle = NULL;
            break;
        }

        /* Check requested serialzers are free or not*/
        if (FALSE == ddc_isRequestedSerzsFree (deviceHandle, params) )
        {
            /*
            * since serializers cannot be allocated,
            * return NULL handle
            */
            ioHandle->channelOpMode = PSP_MCASP_CHAN_FREE;
            ioHandle = NULL;
            break;
        }

        /*
        Since we have request serialisers as free, we allocate them to this
        channel as requested
        */
        for (i=0; i< params->noOfSerRequested; i++)
        {
            ioHandle->indexOfSersRequested[i] = params->indexOfSersRequested[i];
        }

        /* Requested serializers can be allocated to channel*/
        ioHandle->noOfSerAllocated = params->noOfSerRequested;

        /*
        Counter that counts outstanding requests of this channel
        */
        ioHandle->submitCount   = 0;

        /*
        * Global error callback registered to channel
        */
        ioHandle->gblErrCbk = params->gblCbk;

        /*
        We need to have infomation on what application intend to do.
        for e.g. if application is sending 12bit data and formats it in
        such a way the it in stream it occupies 2 bytes , this members value
        should be 16 (PSP_MCASP_WORDLEN_16). By defaut we assume that app has
        formated the data byte stream fro 4bytes for each serialser for each
        sync event/int from mcasp.
        Please note that for burst mode we have 1 sync event/int from mcasp for
        both L and R (in general all slots) where as we have each sync
        event /Int from mcasp for L and R (in gerneral all slots) seperately
        */
        ioHandle->roundedWordWidth = PSP_MCASP_WORDLEN_32;

        /*
        we have 3 options for ioHandle->roundedWordWidth variable 32,16 and 8
        so we start from 32 (bit 5) and check if bit5 , 4, or 3 is set
        if  param->wordWidth is invalid - we assume the default of 32bit width
        The following logic translates the intermediate bit widths to what
        this driver wants. It assumes that always the data streams is alligned to
        8 ,16, 32 bits ( rounded to ceil)
        */

        if(PSP_MCASP_WORDLEN_32 > params->wordWidth)
        {
            mask = 0x20u;

            for (i=0; i<(3u); i++)
            {
                if( mask == (mask &  params->wordWidth))
                {
                    ioHandle->roundedWordWidth = (Uint16)mask;

                    if ( params->wordWidth > ioHandle->roundedWordWidth  )
                    {
                        ioHandle->roundedWordWidth= \
                                (Uint16)(ioHandle->roundedWordWidth << 1);
                    }
                    break;
                }
                mask= (Uint32)(mask >> 1);
            }
        }


        if ( 0 == ( (0x0000FF80u) & params->mcaspSetup->frSyncCtl))
        {
            /*the driver has been configured for DSP (BURST mode)*/
            if (0 ==  params->noOfChannels)
            {
                ioHandle = NULL;
                break;
            }

            ioHandle->roundedWordWidth = \
                    (Uint16)(params->noOfChannels * ioHandle->roundedWordWidth);

            /* rounded wordwidth can not go beyond 32 bits. In burst mode
               multiplication of no of channels and wordwidth can be max. 32
               bits. Beyond that it is not supported
             */
            if ( PSP_MCASP_WORDLEN_32 < ioHandle->roundedWordWidth)
            {
                ioHandle = NULL;
                break;
            }
        }
        /* Converting word width from bits to bytes */
        ioHandle->roundedWordWidth = (Uint16)(ioHandle->roundedWordWidth >> 3);

        /* Here is where the approaches part ways.For interrupt and DMA modes*/
        if (TRUE == params->isDmaDriven)
        {
            /* Store this info in the channel handle too*/
            ioHandle->isDmaDriven = TRUE;

            /* store the EDMA3 module handle*/
            ioHandle->edmaHandle   = edmaHandle ;

            if (PSP_MCASP_INPUT == ioHandle->mode)
            {
                /* Receive mode */

                /*Assign the Channel ID and TCC*/
                ioHandle->xferChan  = deviceHandle->hwInfo.edmaRxEventNum;
                ioHandle->tcc   = deviceHandle->hwInfo.edmaRxEventNum;
                ioHandle->edmaCallback = (EDMA3_RM_TccCallback)&ddc_edmaMcaspCallback;
                ioHandle->cpuEventNum  = deviceHandle->hwInfo.cpuRxEventNum;

            }
            else
            {
                /* Transmit mode */

                /*Assign the Channel ID and TCC*/
                ioHandle->xferChan  = deviceHandle->hwInfo.edmaTxEventNum;
                ioHandle->tcc   = deviceHandle->hwInfo.edmaTxEventNum;
                ioHandle->edmaCallback = (EDMA3_RM_TccCallback)&ddc_edmaMcaspCallback;
                ioHandle->cpuEventNum  = deviceHandle->hwInfo.cpuTxEventNum;

            }


            /*
            *Configure the loop job for the user specified buffer if given
            */
            if (NULL == params->userLoopJobBuffer)
            {
                if (PSP_MCASP_INPUT == ioHandle->mode)
                {
                    ioHandle->loopJobBuffer = loopDstBuf[(deviceHandle->InstNum)].scratchBuffer;
                }
                else
                {
                    ioHandle->loopJobBuffer = loopSrcBuf[(deviceHandle->InstNum)].scratchBuffer;
                }

                /*Length of Uint32 per serialiser - this is what we have allocated ;-)*/
             	if (PSP_MCASP_DIT_MODE == params->channelMode)
				{
                	ioHandle->loopJobLength = 8u;
				}
				else
				{
                	ioHandle->loopJobLength = 4u;
				}
            }
            else
            {

                /*
                Application has preference on the loopjob buffer and
                lets use that
                */
                ioHandle->loopJobBuffer = params->userLoopJobBuffer;
                ioHandle->loopJobLength = params->userLoopJobLength;

                if ( ioHandle->roundedWordWidth > params->userLoopJobLength )
                {
                    /*
                    not enough loopjob buffer has been provided
                    we should have aleast loopbuffer for 1 sync event
                    */
                    ioHandle = NULL;
                    break;

                }
            }

            /* make node */
            PAL_OSLIST_MKNODE(&ioHandle->queueFreeList);
            PAL_OSLIST_MKNODE(&ioHandle->queueReqList);
            PAL_OSLIST_MKNODE(&ioHandle->queueFloatingList);


            /*
            configure EDMA for this driver
            */
            retVal = ddc_setupEDMADuringInit(ioHandle);

            if ( PSP_SOK != retVal)
            {
                PSP_mcaspClose((PSP_Handle)ioHandle,NULL);
                ioHandle = NULL;
                break;
            }


        }/* if DMA driven*/
        else
        {   /* Interrupt driven operation */

            /* Store this info in the channel handle too.*/
            ioHandle->isDmaDriven = FALSE;

            if (PSP_MCASP_INPUT == ioHandle->mode)
            {   /* Receive mode */
                ioHandle->cpuEventNum  = deviceHandle->hwInfo.cpuRxEventNum;
            }
            else
            {
                ioHandle->cpuEventNum  = deviceHandle->hwInfo.cpuTxEventNum;
            }

            /*
            * On being released from reset, the McASP xmt section will trigger
            * two interrupts to fill it's buffers (this has been handled in
            * submitreq function. Keep count of packets to service this.
            */
            ioHandle->dataQueuedOnReset = 0;
            ioHandle->xferinProgressIntmode = FALSE; 
            
            /* make node */
            PAL_OSLIST_MKNODE(&ioHandle->queueFreeList);
            PAL_OSLIST_MKNODE(&ioHandle->queueReqList);
            PAL_OSLIST_MKNODE(&ioHandle->queueFloatingList);

        }

        i = 0;

        /*
        link in all the allocatated IOPs into the FreeQueue list
        */
        while (i < PSP_MCASP_MAX_REQ)
        {
            PAL_osListEnqueue(
                            (Ptr)&ioHandle->queueFreeList,
                            (Ptr)&ioHandle->ioRequests[i]
                            );
            i++;
        }

        if (PSP_MCASP_INPUT == mode)
        {
            /* receive channel */

            /*Re configure the RCV section*/
            status = ddc_ConfigureRCVchannel(deviceHandle, params);

            if (PSP_SOK != status)
            {
                /* Clear allocated channel state*/
                PSP_mcaspClose((PSP_Handle)ioHandle,NULL);
                ioHandle = NULL;
                break;
            }

            /*
            * Mark REQUESTED serializers as RECEIVE mode
            * This also configures PDIR pins as INPUT pin
            */
            ddc_multiSerInitializationRCV (deviceHandle, params);

            /* Set channel operational mode*/
            ioHandle->channelOpMode= (PSP_McaspChanMode)PSP_MCASP_CHAN_RCV;

        }
        else
        {
            /* transmit channel */

            /*Re-configure the XMT section*/
            status = ddc_ConfigureXMTChannel(deviceHandle, params);

            if (PSP_SOK != status)
            {
                /* Clear allocated channel state*/
                PSP_mcaspClose((PSP_Handle)ioHandle,NULL);
                ioHandle = NULL;
                break;
            }

            /*
            * Mark REQUESTED serializers as RECEIVE mode
            * This also configures PDIR pins as INPUT pin
            */
            ddc_multiSerInitializationXMT (deviceHandle, params);

            /* Set channel mode*/
            if (PSP_MCASP_DIT_MODE == params->channelMode)
            {
                /* Channel is to operate in DIT mode*/
                ioHandle->channelOpMode= (PSP_McaspChanMode)PSP_MCASP_CHAN_XMT_DIT;
            }
            else
            {
                /* Channel is to operate in normal TDM mode*/
                ioHandle->channelOpMode= (PSP_McaspChanMode)PSP_MCASP_CHAN_XMT_TDM;
            }

        }

        /*
        For burtsmode of operation frame sync error has to ignored
        to indentify that we do the following
        */
        if (PSP_MCASP_INPUT == ioHandle->mode)
        {
            mod = (Uint32)(deviceHandle->hwInfo.regs->AFSRCTL & CSL_MCASP_AFSRCTL_RMOD_MASK) ;

            /* Clear the framsync error flag from EVTCTLR*/
            if(0 != mod)
            {
                CSL_FINST(deviceHandle->hwInfo.regs->RINTCTL,MCASP_RINTCTL_RSYNCERR,RESETVAL);
            }

        }
        else
        {
            mod = (Uint32)(deviceHandle->hwInfo.regs->AFSXCTL & CSL_MCASP_AFSXCTL_XMOD_MASK) ;

            /* Clear the framsync error flag from EVTCTLX*/
            if(0 != mod)
            {
                CSL_FINST(deviceHandle->hwInfo.regs->XINTCTL,MCASP_XINTCTL_XSYNCERR,RESETVAL);
            }
        }

        /*
        Register the interrupts for events for both  DMA/Interrupt modes*
        because in DMA mode also we will get interrupts for error
        */
        if (PSP_MCASP_INPUT == ioHandle->mode)
        {
            _DDA_McASP_RegisterInt( ioHandle,
                                    instanceId,
                                    (Uint16)ioHandle->cpuEventNum,
                                    (pspMcaspIsr )&ddc_isrInput
                                    );
        }
        else
        {
            _DDA_McASP_RegisterInt( ioHandle,
                                    instanceId,
                                    (Uint16)ioHandle->cpuEventNum,
                                    (pspMcaspIsr )&ddc_isrOutput
                                    );
        }


        /*Starting DMA after starting clocks and before Serialiser SM FS */
        if(TRUE == ioHandle->isDmaDriven)
        {
            /*
            When we get first submit packet
            let us start from linkparam set 0
            */
            ioHandle->nextLinkParamSetToBeUpdated = 0;

            /* enable the DMA */
            EDMA3_DRV_enableTransfer(
                                ioHandle->edmaHandle,
                                ioHandle->xferChan,
                                trigMode
                                );

            /*
            * if DLB mode is set then Enable both state m/c
            * else rcv or xmt state m/c
            */

            dlbMode = (Uint32) CSL_FEXT(deviceHandle->hwInfo.regs->DLBCTL, MCASP_DLBCTL_DLBEN);

            if (CSL_MCASP_DLBCTL_DLBEN_DISABLE == dlbMode)
            {
                if (PSP_MCASP_INPUT == mode)
                { /* Receive channel */

                    /*
                    * activate recieve state machine and frame sync
                    */
                    PAL_osEvtEnable(ioHandle->cpuEventNum);

                    ddc_activateSmFsForRcv(deviceHandle);

                }
                else
                {
                    /* Transmit channel */

                    /*
                    * activate xmit state machine and frame sync
                    */
                    PAL_osEvtEnable(ioHandle->cpuEventNum);

                    ddc_activateSmFsForXmt(deviceHandle);

                }
            }
			else
			{
                /* DLB Mode */

				/*
				* activate receive and transmit state machine and frame sync
				*/

                PAL_osEvtEnable(deviceHandle->hwInfo.cpuRxEventNum);
                PAL_osEvtEnable(deviceHandle->hwInfo.cpuTxEventNum);

                ddc_activateSmFsForBoth(deviceHandle);
			}

        } /* dma driven*/

        /*
        for interrupt mode actual start of mcasp starts in
        Submit function and only when enough data packets are avaialble
        Also note that we have not implemented loopjobbuffer for this mode
        */

        /* enter critical section */
        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

        /* Mark channel status as allocated.*/
        ioHandle->status = PSP_MCASP_ALLOCATED;

        /* Leave critical section */
        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

        /*
        * State m/c stop status
        */
        deviceHandle->stopSmFsXmt = FALSE;
        deviceHandle->stopSmFsRcv = FALSE;

        ioHandle->inUse = TRUE;
    }while(falsewhile);


    return ((PSP_Handle)ioHandle);
}


/**
 *  \brief   ddc_mcbspDeleteChan
 *
 *   Mark the channel available and disable the appropriate interrupt.
 *
 *  \param    hMcASPChan  [IN]    Pointer to channel
 *  \param    arg         [IN]    optional parameter
 *
 *  \return    PSP_SOK on success, or else failure
 *
 */
PSP_Result PSP_mcaspClose(PSP_Handle hMcASPChan,Ptr arg)
{
    PAL_Result            retVal            = PSP_SOK;
    DDC_mcaspIoHandle     ioHandle          = NULL;
    DDC_mcaspDeviceHandle deviceHandle      = NULL;
    Bool                  falsewhile        = FALSE;
    Uint8                 linkCnt           = 0;


    /* To avoid compiler warning */
    if(NULL == arg)
    {
        arg = arg;
    }

    do
    {
        if(NULL == hMcASPChan)
        {
            retVal = PSP_E_RESOURCES;
            break;
        }

        ioHandle = (DDC_mcaspIoHandle)hMcASPChan;

        if(NULL == ioHandle)
        {
            retVal = PSP_E_RESOURCES;
            break;
        }
        else
        {
            if(NULL == ioHandle->deviceHandle)
            {
                retVal = PSP_E_RESOURCES;
                break;
            }
        }

        deviceHandle = ioHandle->deviceHandle;
        if(NULL == deviceHandle)
        {
            retVal = PSP_E_RESOURCES;
            break;
        }

        falsewhile      = FALSE;

        /* Delete the semaphore created at driver open time. */
        if (PSP_SOK != PAL_osSemDelete(ioHandle->syncSem))
        {
            retVal = PSP_E_RESOURCES;
            break;
        }

        if (TRUE == ioHandle->isDmaDriven)
        {
            /* DMA driven transaction */

            /*
	       * Stop the xmt/rcv state machine.
	       */
	       if (PSP_MCASP_CHAN_RCV == ioHandle->channelOpMode)
	       {
	           /* Reset state m/c and frame sync of receive section*/
	           ddc_deactivateSmFsForRcv (deviceHandle);
	       }
	       else
	       {
	           /* Reset state m/c and frame sync of transmit section*/
	           ddc_deactivateSmFsForXmt (deviceHandle);
	       }

            /*
            * Diable the current transfer to make sure that there is no
            * running EDMA transfer taking place
            */
            if ( PSP_SOK != (EDMA3_DRV_disableTransfer (
                                                    ioHandle->edmaHandle,
                                                    ioHandle->xferChan,
                                                    EDMA3_DRV_TRIG_MODE_EVENT))
                                                    )
            {
                retVal = PSP_E_INVAL_STATE;
                break;
            }

            /* Free Edma channels*/
            if ( PSP_SOK != (EDMA3_DRV_freeChannel(
                                        ioHandle->edmaHandle,
                                        ioHandle->xferChan))
                                        )
            {
                retVal = PSP_E_INVAL_STATE;
                break;
            }

            /* free the EDMA PaRAM entries used for Ping pong buffering */
            for (linkCnt = 0; linkCnt < MAXLINKCNT; linkCnt++)
            {
                EDMA3_DRV_freeChannel(
                                    ioHandle->edmaHandle,
                                    ioHandle->pramTbl[linkCnt]
                                    );
            }

        } /* DMA Driven flag is TRUE if loop */

        /*
        * restore global serializer state.
        */
        ddc_multiSerGlobalRestore (ioHandle);

        if ( (PSP_MCASP_CHAN_XMT_DIT == ioHandle->channelOpMode) ||
           (PSP_MCASP_CHAN_XMT_TDM == ioHandle->channelOpMode))
        {
            deviceHandle->hwInfo.regs->XGBLCTL = 0;
        }
        else
        {
            deviceHandle->hwInfo.regs->RGBLCTL = 0;
        }

        /* Set the Muted state to FALSE */
        ioHandle->bMuteON   = FALSE;

        /* Set the Pause state to FALSE */
        ioHandle->paused   = FALSE;

        /*
        * Mark channel as ready to be allocated again.
        */
        ioHandle->status              = PSP_MCASP_UNALLOCATED;
        ioHandle->noOfSerAllocated    = 0;
        ioHandle->channelOpMode       = PSP_MCASP_CHAN_FREE;

        ioHandle->inUse = FALSE;

        /*Unregister the interrupts registered for the errors/data ready*/
        _DDA_McASP_UnRegisterInt(
                        deviceHandle->InstNum,
                        (Uint16)ioHandle->cpuEventNum,
                        (Int)ioHandle->mode
                        );

    }while(falsewhile);

    return retVal;
}

/**
 *  \brief  Write a packet of data to the output channel of the driver
 *
 *  The application has to call PSP_mcaspWrite() with handle to the
 *  o/p channel and the pointer to ioRequest structure which takes a set of
 *  information need for the IO. The I/O req structure takes a (optional)
 *  callback function pointer which will be called after the completion of IO
 *
 *  \param  handle        [IN]   Handle to the McASP channel
 *  \param  ioReq         [IN]   io request information structure
 *  \Param  arg           [IN]   optional - for expanasion
 *
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_mcaspWrite (PSP_Handle handle, PSP_mcaspReq *ioReq,Ptr arg)
{
    DDC_mcaspIoHandle   ioHandle    = NULL;
    PSP_Result          result      = PSP_SOK;

    /* To avoid compiler warning */
    if(NULL != arg)
    {
        arg = arg;
    }

    if (NULL == handle)
    {
        result = PSP_E_INVAL_PARAM;
    }
    else
    {
        ioHandle    = (DDC_mcaspIoHandle)handle;
    }

    if (NULL == ioReq)
    {
        result = PSP_E_INVAL_PARAM;
    }

    if(PSP_MCASP_OUTPUT != ioHandle->mode)
    {
        result = PSP_E_INVAL_PARAM;
    }

    if (PSP_SOK == result)
    {
        result = ddc_mcaspSubmitReq(ioHandle,ioReq);
    }

    return (result);
}


/**
 *  \brief  read a packet of data to the output channel of the driver
 *
 *  The application has to call PSP_mcaspRead() with handle to the
 *  o/p channel and the pointer to ioRequest structure which takes a set of
 *  information need for the IO. The I/O req structure takes a (optional)
 *  callback function pointer which will be called after the completion of IO
 *
 *  \param  handle       [IN]   Handle to the McASP channel
 *  \param  ioReq        [IN]   io request information structure
 *  \Param  arg          [IN]   optional - for expansion
 *
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_mcaspRead (PSP_Handle handle, PSP_mcaspReq *ioReq,Ptr arg)
{
    DDC_mcaspIoHandle   ioHandle    = NULL;
    PSP_Result          result      = PSP_SOK;

    /* To avoid compiler warning*/
    if(NULL != arg)
    {
        arg = arg;
    }

    if ((NULL == handle)||(NULL==ioReq))
    {
        result = PSP_E_INVAL_PARAM;
    }
    else
    {

        ioHandle    = (DDC_mcaspIoHandle)handle;
    }

    if(PSP_MCASP_INPUT != ioHandle->mode)
    {
        result = PSP_E_INVAL_PARAM;
    }

    if (PSP_SOK == result)
    {
        result = ddc_mcaspSubmitReq(ioHandle,ioReq);
    }
    return (result);
}

/**
 *  \brief Flush the data in the McASP channel
 *         abort all read requests now.
 *         wait for normal completion of write request and return.
 *         coincedentally if the tx packet count is 0, return now itslelf
 *
 *  \param  handle     [IN]       Handle to the McBSP channel
 *  \param  arg        [IN]       Packet sent
 *
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_mcaspFlushRequests(PSP_Handle handle, Ptr arg)
{

    DDC_mcaspIoHandle      ioHandle         = NULL;
    PAL_Result              result          = PSP_SOK;
    DDC_mcaspDeviceHandle   deviceHandle    = NULL;
    Uint32                  dlbMode;

    /* To avoid compiler warning */
    if(NULL != arg)
    {
        arg = arg;
    }
    if(NULL!=handle)
    {
        ioHandle    = (DDC_mcaspIoHandle )handle;
    }
    else
    {
        result = PSP_E_INVAL_PARAM;
    }

    if(NULL == ioHandle)
    {
        result = PSP_E_INVAL_PARAM;
    }
    else if(NULL == ioHandle->deviceHandle)
    {
        result = PSP_E_INVAL_PARAM;
    }
    else
    {
        deviceHandle = ioHandle->deviceHandle;
        if(NULL == deviceHandle)
        {
            result = PSP_E_INVAL_PARAM;
        }
        else
        {
            if (PSP_SOK == result)
            {

                /* read mcasp DLB mode*/
                dlbMode = (Uint32) CSL_FEXT(
                                        deviceHandle->hwInfo.regs->DLBCTL,
                                        MCASP_DLBCTL_DLBEN
                                        );


                if (CSL_MCASP_DLBCTL_DLBEN_ENABLE == dlbMode)
                {
                    if (TRUE == deviceHandle->rxIo.inUse)
                    {
                        ddc_abortReset((DDC_mcaspIoHandle)&deviceHandle->rxIo);
                    }
                    if (TRUE == deviceHandle->txIo.inUse)
                    {
                        ddc_abortReset((DDC_mcaspIoHandle)&deviceHandle->txIo);
                    }
                }
                else
                {
                    if (TRUE == ioHandle->inUse)
                    {
                        if (TRUE == ioHandle->isDmaDriven)
                        {
                            if (PSP_MCASP_INPUT == ioHandle->mode)
                            {
                                if (TRUE == ioHandle->inUse)
                                {
                                    /*
                                     * Flush is called for INPUT channel,
                                     * that is why we have to enable flush flag.
                                     */
                                    ioHandle->flush = TRUE;
                                    /* If DLB mode is set then Abort both channels else Receive channel only*/
                                    ddc_abortReset((DDC_mcaspIoHandle)&deviceHandle->rxIo);
                                    /* Flush flag is disable */
                                    ioHandle->flush = FALSE;
                                }
                            }
                            else
                            {
                                if (0 >= ioHandle->submitCount)
                                {
                                    result = PSP_SOK;
                                }
                                else
                                {
                                    ioHandle->flush = TRUE;
                                    PAL_osSemTake(ioHandle->syncSem,PAL_OSSEM_NO_TIMEOUT);
                                }

                            }
                        }
                        else
                        {

                            if (PSP_MCASP_INPUT == ioHandle->mode)
                            {
                                /* If DLB mode is set then Abort both channels else Receive channel only*/
                                ddc_abortReset((DDC_mcaspIoHandle)&deviceHandle->rxIo);
                            }
                            else
                            {
                                ioHandle->flush = TRUE;
                                PAL_osSemTake(ioHandle->syncSem,PAL_OSSEM_NO_TIMEOUT);

                            }
                        }
                    }
                }
            }
        }
    }
    return (result);
}


/**
 *  \brief Abort the McASP channel operation
 *
 *   Abort all current and pending Read and write requests
 *
 *  \param  handle   [IN]     Handle to the McASP channel
 *  \param  arg      [IN]     Packet sent
 *
 *  \return PSP_SOK or PSP Error code
 */
PSP_Result PSP_mcaspAbortRequests(PSP_Handle handle,Ptr arg)
{

    DDC_mcaspIoHandle           ioHandle        = NULL;
    PAL_Result                  result          = PSP_SOK ;
    DDC_mcaspDeviceHandle       deviceHandle    = NULL;

    /* To avoid compiler warning */
    if(NULL != arg)
    {
        arg = arg;
    }
    if (NULL == handle)
    {
        result = PSP_E_INVAL_PARAM;
    }
    else
    {
        ioHandle    = (DDC_mcaspIoHandle )handle;
        if(NULL == ioHandle)
        {
            result = PSP_E_INVAL_PARAM;
        }
        else if(NULL == ioHandle->deviceHandle)
        {
            result = PSP_E_INVAL_PARAM;
        }
        else
        {
            deviceHandle    = ioHandle->deviceHandle;
        }
        if(NULL == deviceHandle)
        {
            result = PSP_E_INVAL_PARAM;
        }
        else
        {
            if (PSP_SOK == result)
            {
                if (TRUE == ioHandle->inUse)
                {
                    /* Abort both iohandlenels*/
                    if (PSP_MCASP_INPUT == ioHandle->mode)
                    {
                        ddc_abortReset((DDC_mcaspIoHandle)&deviceHandle->rxIo);
                    }
                    else
                    {
                        ddc_abortReset((DDC_mcaspIoHandle)&deviceHandle->txIo);
                    }
                }

            }
        }
    }
    return (result);
}

/**
 *  \brief    ddc_mcbspSubmitReq
 *  Submits and I/O packet to channel specified . This is called by
 *  PSP_mcaspRead and PSP_mcaspWrite to perform actual I/O programming
 *
 *  \param   iohandlep       [IN]    Pointer to channel
 *  \param   pspRequestp     [IN]    Pointer to ioReq structure
 *
 *  \return  PSP_SOK on success or else failure
 */
static PAL_Result ddc_mcaspSubmitReq(
                                DDC_mcaspIoHandle ioHandle,
                                PSP_mcaspReq *pspRequestp
                                )
{


    PAL_Result                  retVal          = PSP_SOK;
    Uint32                      dlbMode         = 0;
    Uint32                      cookie          = 0;
    Bool                        falsewhile      = FALSE;
    DDC_mcaspDeviceHandle       deviceHandle    = NULL;
    DDC_mcaspIOP                *ioPacket       = NULL;


    do
    {
        falsewhile      = FALSE;

        if((NULL == ioHandle) || (NULL==pspRequestp))
        {
            retVal = PSP_E_INVAL_PARAM;
            break;
        }
        else
        {
            if(NULL == ioHandle->deviceHandle)
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }
        }

        /* check for the valid packet size */
        if (( 0 == pspRequestp->size)
            ||(MCASP_MAX_PACKET_SIZE<(pspRequestp->size/ioHandle->noOfSerAllocated)))
        {
            retVal =  PSP_E_INVAL_PARAM;
            break;
        }

        deviceHandle    = ioHandle->deviceHandle;

        /* If currently requests are flushing no request can be served */
        if (TRUE == ioHandle->flush)
        {
            retVal =  PSP_E_INVAL_PARAM;
            break;
        }

        if (TRUE == ioHandle->isDmaDriven)
        {
            /* DMA driven transaction */

            /* maintain cache coherency */
            if (PSP_MCASP_INPUT == ioHandle->mode)
            {

                /* Cleans and invalidate the D Cache in MVA way */
                if (TRUE == ioHandle->deviceHandle->enablecache)
                {
                    PAL_osCacheInvalidate(
                                (PAL_osCacheMemAddrSpace)NULL,
                                (Uint32)pspRequestp->addr,
                                pspRequestp->size
                                );
                }

            }
            else
            {

                /* clean the D Cache in MVA way */
                if (TRUE == ioHandle->deviceHandle->enablecache)
                {

                    PAL_osCacheFlush(
                                (PAL_osCacheMemAddrSpace)NULL,
                                (Uint32)pspRequestp->addr,
                                pspRequestp->size
                                );
                }
            }

            if(TRUE == PAL_osListIsEmpty (&ioHandle->queueFreeList) )
            {

                /*
                all packets already exhausted
                */
                retVal = PSP_E_INVAL_PARAM;
                break;
            }
            else
            {

                ioPacket = (DDC_mcaspIOP *)PAL_osListDequeue(&ioHandle->queueFreeList);

                if(NULL!=ioPacket)
                {
                    /*
                    We have a ioPacket object to take up the request
                    populate the IOP from request structure
                    */
                    ddc_copyReqtoIOP( pspRequestp, ioPacket);

					ioPacket->size = ioPacket->size / ioHandle->noOfSerAllocated;
                }

            }

            /*  Enter critical section to protect the submit count */
            PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);


            /*
            if there are enought floating  request packets then
            enqueue this request.
            */
            if ((MAXLINKCNT <= ioHandle->submitCount )
             || (TRUE == ioHandle->paused))
            {
                /*
                if we dont have enough floating request packets then
                enqueue this request into the pending request queue
                */

                /*Increment the submit count*/
                ioHandle->submitCount++;

                PAL_osList_NP_Enqueue (&ioHandle->queueReqList, (Ptr)ioPacket);

                PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);


                /*
                * Processing of request is not complete; so return pending status.
                * request will be marked as complete when it is processed by the
                * dmaIsr function. If the application makes a sync call to the
                * GIO layer, it will pend here till request status is marked as
                * complete
                */

            }
            else
            {

                /*Increment the submit count*/
                ioHandle->submitCount++;

				/* floating queue not full . Process this request.*/

                PAL_osList_NP_Enqueue(&(ioHandle->queueFloatingList),ioPacket);


                PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);


                /*
                * For a DIT  request,  channel has to be programmed with its
                * contents (DIT RAM registers also if this is an xmt channel in
                * DIT mode).
                */
                if (PSP_MCASP_CHAN_RCV != ioHandle->channelOpMode)
                {
                    if ((PSP_MCASP_CHAN_XMT_DIT == ioHandle->channelOpMode) &&
                            (TRUE == pspRequestp->writeDitParams))
                    {
                        /*
                        * DIT Mode:
                        * Write to the Channel Status RAM and User Data RAM.
                        */
                        retVal = LLC_mcaspWriteChanStatRam (
                                            &(deviceHandle->hwInfo),
                                            (PSP_McaspChStatusRam *)pspRequestp->chStat
                                            );

                        retVal = LLC_mcaspWriteUserDataRam (
                                            &(deviceHandle->hwInfo),
                                            (PSP_McaspUserDataRam *)pspRequestp->userData
                                            );

                        if (PSP_SOK != retVal)
                        {
                            retVal = PSP_MCASP_ABORTED;
                            break;
                        }

                    }
                }


                ioHandle->currentDataSize = (Uint16)ioPacket->size;
                ioHandle->userDataBufferSize = (Uint32)ioPacket->size;

                /*
                for DMA mode we will not use ioHandle->dataPacket
                to hold current packet - we use floatingqueue
                */
                ioHandle->dataPacket = NULL;


                 /*  Enter critical section to protect the submit count */
                PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);

                if( 2 == ioHandle->submitCount)
                {
                    /*
                    Though we have to post to param set directly from here,
                    there will be differene between first such packet and
                    second packet. As we have control here we are second packet
                    and first packet has not yet returned (or corresponding
                    edma callback has not been called.For second packet, we
                    will be updating the second param set, which is currently
                    hosting loopjob parameter. Hence increment the index to
                    point second paramset and since we are moving out loopjob
                    from both param sets, the loopjobUpdatedinParamset is reset
                    */
                    ioHandle->loopjobUpdatedinParamset =FALSE;
                    GetNextIndex(ioHandle->nextLinkParamSetToBeUpdated );
                }
                /*

                else --
                this is a first packet - meaning one paramset is not active (to which currently index is pointing and
                the other hosting active self linked loopjob
                */

                if (PSP_SOK != ddc_updateDataPacketTolinkParams(ioHandle, ioPacket))
                {
                    retVal = PSP_MCASP_DMA_ERROR;
                    /*  Enter critical section to protect the submit count */
                    PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
                    break;
                }


                if ( 1 == ioHandle->submitCount )
                {
                    /*
                    if at all this is the very first packet, then one param set
                    has loop job loaded , self linked and active with the main
                    xfer channel param. other param set is ready loaded (just now
                    and has link paramater set as the one having loopjob (this is
                    to ensure that if at all we are not getting any more packets
                    loopjob be will taken over). Now we have to link the
                    floating / newly loaded param set to xfer channel.
                    */

                    retVal = EDMA3_DRV_linkChannel(
                                    ioHandle->edmaHandle,
                                    ioHandle->xferChan,
                                    ioHandle->pramTbl[ioHandle->nextLinkParamSetToBeUpdated]
                                    );

                    if (PSP_SOK != retVal)
                    {
                        retVal = PSP_MCASP_DMA_ERROR;
                        /*  Enter critical section to protect the submit count */
                        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
                        break;
                    }
                }

                /*  Enter critical section to protect the submit count */
                PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);

            } /* Pending queue empty or not*/

            retVal = PSP_SINPROGRESS;


        } /* DMA driven transactions*/
        else
        {
            /* Interrupt driven transactions */

            /* When xmt is taken out of reset, it needs two words to service it.*/
            /* Pending queue empty. Process this request.*/
            if (( FALSE == ioHandle->xferinProgressIntmode) &&
                ((PSP_MCASP_TWO_PKTS_QUEUED*ioHandle->noOfSerAllocated) > (ioHandle->dataQueuedOnReset)))

            {
                ioHandle->currentDataSize = (Uint16)pspRequestp->size;
                ioHandle->userDataBufferSize = (Uint32)pspRequestp->size;


                if(TRUE == PAL_osListIsEmpty (&(ioHandle->queueFreeList)))
                {
                    retVal = PSP_E_RESOURCES;
                    break;
                }

                ioPacket = (DDC_mcaspIOP *) PAL_osListDequeue (&(ioHandle->queueFreeList));

                if(NULL != ioPacket)
                {
                    /* copy the io request data to psp request data */
                    ddc_copyReqtoIOP(pspRequestp ,ioPacket);

                    PAL_osListEnqueue(&(ioHandle->queueReqList), (Ptr)ioPacket);
                     /*  Enter critical section to protect the submit count */
                    PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);
                    /*Increment the submit count*/
                    ioHandle->submitCount++;

                    PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
                }
                else
                {
                    retVal = PSP_E_RESOURCES;
                    break;
                }

                /*
                * If data in request and queue can serve the pending requests, we
                * can take the McASP section out of reset. Else the request is
                * queued to take action on later.
                */

                ioHandle->dataQueuedOnReset = \
                    (Uint16)((Uint32)ioHandle->dataQueuedOnReset + ioPacket->size);

                if (ioHandle->dataQueuedOnReset >=
                        (PSP_MCASP_TWO_PKTS_QUEUED*ioHandle->noOfSerAllocated) )
                {
                    /*
                    * if DLB mode is set then Enable both state m/c
                    * else rcv or xmt state m/c
                    */
                    dlbMode = (Uint32) CSL_FEXT(deviceHandle->hwInfo.regs->DLBCTL, MCASP_DLBCTL_DLBEN);

                    ioHandle->xferinProgressIntmode = TRUE;

                    if (PSP_MCASP_INPUT == ioHandle->mode )
                    { /* Receive channel */

                        /* Activate receive section state machine and frame sync*/
                        if (CSL_MCASP_DLBCTL_DLBEN_DISABLE == dlbMode)
                        {
                            PAL_osEvtEnable(ioHandle->cpuEventNum);

                            ddc_activateSmFsForRcv(deviceHandle);

                        }
                    }
                    else
                    {
                        /* Transmit channel */

                        /* Activate transmit section state machine and frame sync*/
                        if (CSL_MCASP_DLBCTL_DLBEN_DISABLE == dlbMode)
                        {
                            PAL_osEvtEnable(ioHandle->cpuEventNum);

                            ddc_activateSmFsForXmt(deviceHandle);
                        }
                        else
                        {
                        /* Activate state machine and frame sync for both sections*/

                            PAL_osEvtEnable(deviceHandle->rxIo.cpuEventNum);

                            PAL_osEvtEnable(deviceHandle->txIo.cpuEventNum);


                            ddc_activateSmFsForBoth(deviceHandle);

                        }
                    }

                }
            }
            else
            {
                /* Queue up requests*/

                if(TRUE ==PAL_osListIsEmpty (&(ioHandle->queueFreeList)) )
                {
                    retVal = PSP_E_RESOURCES;
                    break;
                }
                else
                {
                    ioPacket = PAL_osListDequeue (&(ioHandle->queueFreeList));
                    ddc_copyReqtoIOP(pspRequestp ,ioPacket);

                    PAL_osListEnqueue (&(ioHandle->queueReqList), ioPacket);
                     /*  Enter critical section to protect the submit count */
                    PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &cookie);
                    /*Increment the submit count*/
                    ioHandle->submitCount++;

                    PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, cookie);
                }

            }

            retVal = PSP_SINPROGRESS;


        } /* else for interrupt driven */

    }while(falsewhile);

    return (retVal);
}

/**
 *  \brief ddc_mcaspControlChan(
 *         Passes on control command to the mini-driver
 *
 *  \param    hMcASPChan  [IN]  Pointer to channel
 *  \param    cmd         [IN]  Control command number
 *  \param    arg         [IN]  Arguments for control command
 *  \param    param       [IN]  user defined data structure
 *
 *  \return  IOM_COMPLETED,     if command is executed correctly
 *           PSP_MCASP_ABORTED, if command returns error during execution
 *           IOM_ENOTIMPL,      if command is not supported
 *           IOM_EBADARGS       if args are not correct
 */
/*ARGSUSED*/
PSP_Result PSP_mcaspIoctl(
                    PSP_Handle          hMcASPChan,
                    PSP_McaspIoctlCmd   cmd,
                    Ptr                 arg,
                    Ptr                 param
                    )

{
    PAL_Result              retVal          = PSP_SOK;
    Uint32                  queryRes        = 0;
    Bool                    dlbMode         = FALSE;
    Bool                    falsewhile      = 0;
    DDC_mcaspIoHandle       ioHandle        = NULL;
    DDC_mcaspDeviceHandle   deviceHandle    = NULL;
    DDC_mcaspIoHandle       iohandlep       = NULL;
    PSP_mcaspChanParams     *params         = NULL;
    EDMA3_DRV_PaRAMRegs     pramPtr         = {0};
    DDC_mcaspIOP            *ioPacket       = NULL;
    Int16                   linkCnt         = 0;
    Uint32                  tempVal         = 0;
	Int16                   serNum          = 0;
    /*
    * If cmd = PSP_MCASP_SET/GET_CNTRL_FORMAT_CHAN, we need this as argument
    */

    PSP_McaspHwSetupData *chanSetup;

    /*
    * If cmd = PSP_MCASP_CNTRL_SET_GBL_REGS, we need this as argument
    */

    PSP_McaspHwSetup    *mcaspHwSetup;

    if(NULL != param)
    {
        param = param;
    }

    if(NULL == hMcASPChan)
    {
        retVal = PSP_E_INVAL_PARAM;
    }
    else
    {
        ioHandle = (DDC_mcaspIoHandle)hMcASPChan;


        if(NULL == ioHandle->deviceHandle)
        {
            retVal = PSP_E_INVAL_PARAM;
        }
        else
        {
            deviceHandle = ioHandle->deviceHandle;
            /*
            * If cmd = PSP_MCASP_SET_DLB_MODE
            * or cmd = PSP_MCASP_SET_DIT_MODE, we need this as argument
            */

            mcaspHwSetup    = NULL;
            chanSetup       = NULL;

            switch (cmd)
            {

              case PSP_CHAN_TIMEDOUT:
                {
                    /*
                    * If a channel timeouts(in IOM class driver) a calldown is made to
                    * mcasp_mdControlChan w/ cmd = IOM_CHAN_TIMED out. Perform needed channel
                    * cleanup.
                    */

                    ddc_abortReset(ioHandle);
                    break;
                }

              case PSP_CHAN_RESET:
                {
                    /*
                    * Reset the xmt/rcv channel. Call GIO_flush before this.
                    */

                    ddc_abortReset(ioHandle);

                    /*
                    * Start the xmt/rcv state machine.
                    */
                    ddc_smSet(ioHandle);

                    break;
                }

              case PSP_MCASP_CNTRL_SET_FORMAT_CHAN:
                {
                    do
                    {
                        falsewhile      = FALSE;
                        if (NULL == arg)
                        {
                            retVal  = PSP_E_INVAL_STATE;
                            break;
                        }

                        if (NULL == ioHandle)
                        {
                            retVal  = PSP_E_INVAL_STATE;
                            break;
                        }

                        /*
                        * Reprogram xmt/rcv format unit registers.
                        * State m/c associated with channel should have been reset.
                        * A pointer to h/w setup data would have been passed through the
                        * argument field.
                        */
                        chanSetup = (PSP_McaspHwSetupData *)arg;

                        if (NULL == chanSetup)
                        {
                           retVal  = PSP_E_INVAL_STATE;
                           break;
                        }
                        /*
                         * Stop the xmt/rcv state machine.
                         */
                        ddc_smReset(ioHandle);

                        if (PSP_MCASP_CHAN_RCV == ioHandle->channelOpMode)
                        {

                            /*
                            * Check if receive state machine and frame sync are reset
                            */

                            retVal = (PAL_Result)LLC_mcaspGetHwStatus(
                                                    &(deviceHandle->hwInfo),
                                                    LLC_MCASP_QUERY_SM_FS_RCV,
                                                    &queryRes
                                                    );

                        }
                        else
                        {

                            /*
                            * Check if transmit state machine and frame sync are reset
                            */

                            retVal = (PAL_Result)LLC_mcaspGetHwStatus(
                                                        &(deviceHandle->hwInfo),
                                                        LLC_MCASP_QUERY_SM_FS_XMT,
                                                        &queryRes
                                                        );

                        }

                        if (PSP_SOK != retVal)
                        {
                           retVal  = PSP_E_INVAL_STATE;
                           break;
                        }

                        if (PSP_MCASP_SM_RESET != queryRes)
                        {
                            /*
                            * The corresponding state m/c and frame sync have to be reset.
                            * And they are not. So return error.
                            */

                            retVal = PSP_E_INVAL_PARAM;
                            break;
                        }

                        if ((PSP_MCASP_CHAN_XMT_DIT == ioHandle->channelOpMode) ||
                            (PSP_MCASP_CHAN_XMT_TDM == ioHandle->channelOpMode))
                        {
                            /*
                            * Configure transmit section
                            */
                            retVal = LLC_mcaspConfigXmtSection (
                                                    (LLC_McaspHandle)&(deviceHandle->hwInfo),
                                                    (PSP_McaspHwSetupData *)chanSetup
                                                    );
                        }
                        else
                        {
                            /*
                            * Configure receive section
                            */

                            retVal = LLC_mcaspConfigRcvSection (
                                                    &(deviceHandle->hwInfo),
                                                    (PSP_McaspHwSetupData *)chanSetup
                                                    );
                        }

                        if (PSP_SOK != retVal)
                        {
                            /*
                            * Error configuring section. Mark channel status as INVALID
                            */

                            ioHandle->status  = PSP_MCASP_UNALLOCATED;
                            retVal       = PSP_E_INVAL_STATE;
                            break;
                        }

                        /*
                         * Stop the xmt/rcv state machine.
                         */
                        ddc_smSet(ioHandle);

                    }while(falsewhile);
                    break;
                }

              case PSP_MCASP_CNTRL_GET_FORMAT_CHAN:
                {
                    do
                    {
                        falsewhile      = FALSE;
                        if (NULL == arg)
                        {
                            retVal  = PSP_E_INVAL_STATE;
                            break;
                        }

                        if (NULL == deviceHandle)
                        {
                            retVal  = PSP_E_INVAL_STATE;
                            break;
                        }

                        /*
                        * Read xmt/rcv format unit registers.
                        * A pointer to h/w setup data would have been passed through the
                        * argument field.
                        */

                        chanSetup = (PSP_McaspHwSetupData *)arg;

                        if ((PSP_MCASP_CHAN_XMT_DIT == ioHandle->channelOpMode) ||
                            (PSP_MCASP_CHAN_XMT_TDM == ioHandle->channelOpMode))
                        {
                            /*
                            * Read transmit section
                            */
                            retVal = LLC_mcaspReadXmtConfig(&(deviceHandle->hwInfo), chanSetup);

                            if (PSP_SOK != retVal)
                            {
                                break;
                            }

                        }
                        else
                        {
                            /*
                            * Read receive section
                            */
                            retVal = LLC_mcaspReadRcvConfig(&(deviceHandle->hwInfo),chanSetup);
                            if (PSP_SOK != retVal)
                            {
                                break;
                            }
                        }
                    }while(falsewhile);
                    break;
                }

                case PSP_MCASP_CNTRL_SET_GBL_REGS:
                    {
                        do
                        {
                            falsewhile      = FALSE;
                            if (NULL == arg)
                            {
                                retVal  = PSP_E_INVAL_STATE;
                                break;
                            }

                            if (NULL == deviceHandle)
                            {
                                retVal  = PSP_E_INVAL_STATE;
                                break;
                            }

                            /*
                            * This will program the global registers.
                            * Both channels will be reset and global registers programmed.
                            * A pointer to h/w setup would have been passed through the
                            * argument field.
                            */

                            mcaspHwSetup = (PSP_McaspHwSetup *)arg;

                            if ((PSP_MCASP_CHAN_XMT_DIT == ioHandle->channelOpMode) ||
                                (PSP_MCASP_CHAN_XMT_TDM == ioHandle->channelOpMode))
                            {
                                deviceHandle->hwInfo.regs->XGBLCTL = 0;
                            }
                            else
                            {
                                deviceHandle->hwInfo.regs->RGBLCTL = 0;
                            }

                            retVal = (PAL_Result)LLC_mcaspHwSetup(&(deviceHandle->hwInfo), mcaspHwSetup);

                            if (PSP_SOK != retVal)
                            {
                                /* Error in setup!*/
                                retVal = PSP_MCASP_ABORTED;
                                break;
                            }
                        }while(falsewhile);
                        break;
                    }
                case PSP_MCASP_SET_DIT_MODE:
                    {
                        do
                        {
                            falsewhile      = FALSE;
                            if (NULL == arg)
                            {
                                retVal  = PSP_E_INVAL_STATE;
                                break;
                            }

                            if (NULL == deviceHandle)
                            {
                                retVal  = PSP_E_INVAL_STATE;
                                break;
                            }

                            /*
                            * This will set the DIT mode
                            */
                            tempVal = *(Uint32 *)arg;

                            if ((PSP_MCASP_CHAN_XMT_DIT == deviceHandle->txIo.channelOpMode) ||
                                (PSP_MCASP_CHAN_XMT_TDM == deviceHandle->txIo.channelOpMode))
                            {
                                deviceHandle->hwInfo.regs->DITCTL |= (Uint32)tempVal;
                                ioHandle->channelOpMode= (PSP_McaspChanMode)PSP_MCASP_CHAN_XMT_DIT;
                            }
                        }while(falsewhile);
                        break;
                    }
                case PSP_MCASP_SET_DLB_MODE:
                    {
                        do
                        {
                            falsewhile      = FALSE;
                            /*
                            * This will program the digital loopback mode.
                            */
                            if (NULL == arg)
                            {
                                retVal =   PSP_E_INVAL_PARAM;
                                break;
                            }
                            if (NULL == ioHandle)
                            {
                                retVal =   PSP_E_INVAL_PARAM;
                                break;
                            }

                            dlbMode = *(Bool*)arg;

                            iohandlep = (DDC_mcaspIoHandle)&deviceHandle->rxIo;
                            if (NULL == iohandlep)
                            {
                                /*
                                * May lead to unexpected results.
                                * Revert with error status
                                */
                                retVal = PSP_E_INVAL_STATE;
                                break;

                            }
                            if (NULL != iohandlep->dataPacket)
                            {
                                /*
                                * May lead to unexpected results.
                                * Revert with error status
                                */
                                retVal = PSP_E_INVAL_STATE;
                                break;
                            }

                            queryRes = (Uint32)iohandlep->noOfSerAllocated;

                            iohandlep = (DDC_mcaspIoHandle)&deviceHandle->txIo;

                            if (NULL != iohandlep->dataPacket)
                            {
                                /*
                                * May lead to unexpected results.
                                * Revert with error status
                                */
                                retVal = PSP_E_INVAL_STATE;
                                break;
                            }

                            if ( queryRes != iohandlep->noOfSerAllocated)
                            {
                                /*
                                * Need equal number of serializers for DLB mode
                                *
                                */
                                retVal = PSP_E_INVAL_STATE;
                                break;
                            }

                            while (serNum < (deviceHandle->hwInfo.numOfSerializers - (1u)))
							{
                                if(PSP_MCASP_SER_FREE != deviceHandle->serStatus[serNum])
								{
                                    if(PSP_MCASP_SER_FREE != deviceHandle->serStatus[serNum+1])
									{
							           /*
							           * Increment serializers counter
						        	   *
				        			   */
                                       serNum+=2;
									}
									else
									{
									    /*
                                        * Serializers not in sequence
										*
										*/
                                        retVal = PSP_E_INVAL_STATE;
                                        break;
									}
								}
                                else
								{
                                    if(PSP_MCASP_SER_FREE == deviceHandle->serStatus[serNum+1])
									{
							           /*
							           * Increment serializers counter
						        	   *
				        			   */
                                       serNum+=2;
									}
									else
									{
									    /*
                                        * Serializers not in sequence
										*
										*/
                                        retVal = PSP_E_INVAL_STATE;
                                        break;
									}
								}
							}

                            if(PSP_SOK != retVal)
                            {
                                break;
                            }

                            retVal = LLC_mcaspConfigLoopBack (
                                                    &(deviceHandle->hwInfo),
                                                    dlbMode
                                                    );

                            if (PSP_SOK != retVal)
                            {
                                /*
                                * Error in setup!
                                */
                                retVal = PSP_MCASP_ABORTED;
                                break;
                            }

							ddc_activateSmFsForBoth(deviceHandle);

                        }while (falsewhile);
                        break;
                    }

                case  PSP_MCASP_CNTRL_AMUTE:
                    {
                        do
                        {
                            falsewhile      = FALSE;
                            if (NULL == arg)
                            {
                                retVal =   PSP_E_INVAL_PARAM;
                                break;
                            }
                            if (NULL == deviceHandle)
                            {
                                retVal =   PSP_E_INVAL_PARAM;
                                break;
                            }

                            tempVal = *(Uint32 *)arg;

                            /*
                            * Enable or Disable AMUTE register
                            */
                            retVal = LLC_mcaspConfigAudioMute (
                                            &(deviceHandle->hwInfo),
                                            tempVal
                                            );
                            if (PSP_SOK != retVal)
                            {
                                retVal = PSP_E_INVAL_STATE;
                                break;
                            }
                        }while(falsewhile);
                        break;
                    }
                case PSP_MCASP_DEVICE_RESET:
                    {
                        do
                        {
                            falsewhile      = FALSE;

                            if (NULL == deviceHandle)
                            {
                                retVal =   PSP_E_INVAL_PARAM;
                                break;
                            }

                            /* Abort all the request, give application callback and
                               configure EDMA in the LOOPJOB for both input and output
                               channels
                            */
                            PSP_mcaspAbortRequests(
                                        (DDC_mcaspIoHandle)&(deviceHandle->rxIo),
                                        NULL
                                        );
                            PSP_mcaspAbortRequests(
                                        (DDC_mcaspIoHandle)&(deviceHandle->txIo),
                                        NULL
                                        );

                            retVal = LLC_mcaspRegReset (&(deviceHandle->hwInfo));

                            if (PSP_SOK != retVal)
                            {
                               retVal = PSP_E_INVAL_STATE;
                            }


                        }while(falsewhile);
                        break;
                    }
                case  PSP_MCASP_GPIO_CONFIG:
                    {
                        if (NULL != arg)
                        {
                            tempVal = *(Uint32*)arg;

                            /*
                            * Configure GPIO pins
                            */
                            retVal = LLC_mcaspConfigGpio (&(deviceHandle->hwInfo), tempVal);

                            if (PSP_SOK != retVal)
                            {
                                break;
                            }

                        }
                        else
                        {
                            retVal = PSP_E_INVAL_STATE;
                        }
                        break;
                    }
                case PSP_MCASP_START_PORT:
                    {
                        if (PSP_MCASP_INPUT == ioHandle->mode)
                        {
                            if (TRUE == ioHandle->deviceHandle->stopSmFsRcv)
                            {
                                ioHandle->deviceHandle->stopSmFsRcv = FALSE;
                            }
                            else
                            {
                                 /* Start port is issued without issuing stop*/
                                 retVal = PSP_E_INVALID_CMD;
                                 break;
                            }
                        }
                        else
                        {
                            if (TRUE == ioHandle->deviceHandle->stopSmFsXmt)
                            {
                                ioHandle->deviceHandle->stopSmFsXmt = FALSE;
                            }
                            else
                            {
                                 /* Start port is issued without issuing stop*/
                                 retVal = PSP_E_INVALID_CMD;
                                 break;
                            }
                        }

                        if(TRUE == ioHandle->isDmaDriven)
                        {
                            if (TRUE == PAL_osList_NP_IsEmpty (&ioHandle->queueFloatingList))
                            {
                                for (linkCnt = 0;linkCnt < MAXLINKCNT;linkCnt++)
                                {
                                    if( FALSE == PAL_osList_NP_IsEmpty (&ioHandle->queueReqList) )
                                    {
                                        ioPacket = (DDC_mcaspIOP *)PAL_osList_NP_Dequeue(
                                                                            &ioHandle->queueReqList
                                                                            );

                                        if (NULL == ioPacket)
                                        {
                                            retVal =   PSP_E_INVAL_PARAM;
                                            break;
                                        }

                                        /* floating queue not full . Process this request.*/
                                        PAL_osList_NP_Enqueue(&(ioHandle->queueFloatingList),ioPacket);

                                        ioHandle->currentDataSize = (Uint16)ioPacket->size;
                                        ioHandle->userDataBufferSize = (Uint32)ioPacket->size;

                                        /*
                                        for DMA mode we will not use ioHandle->dataPacket
                                        to hold current packet - we use floatingqueue
                                        */
                                        ioHandle->dataPacket = NULL;

                                        if( (MAXLINKCNT-1u) == linkCnt)
                                        {
                                            /*
                                            Though we have to post to param set directly from here,
                                            there will be differene between first such packet and
                                            second packet. As we have control here we are second packet
                                            and first packet has not yet returned (or corresponding
                                            edma callback has not been called.For second packet, we
                                            will be updating the second param set, which is currently
                                            hosting loopjob parameter. Hence increment the index to
                                            point second paramset and since we are moving out loopjob
                                            from both param sets, the loopjobUpdatedinParamset is reset
                                            */
                                            ioHandle->loopjobUpdatedinParamset =FALSE;
                                            GetNextIndex(ioHandle->nextLinkParamSetToBeUpdated );
                                        }
                                        /*
                                        else --
                                        this is a first packet - meaning one paramset is not active (to which currently index is pointing and
                                        the other hosting active self linked loopjob
                                        */

                                        if (PSP_SOK != ddc_updateDataPacketTolinkParams(ioHandle, ioPacket))
                                        {
                                            retVal = PSP_MCASP_DMA_ERROR;
                                            break;
                                        }


                                        if ( 0 == linkCnt )
                                        {
                                            /*
                                            if at all this is the very first packet, then one param set
                                            has loop job loaded , self linked and active with the main
                                            xfer channel param. other param set is ready loaded (just now
                                            and has link paramater set as the one having loopjob (this is
                                            to ensure that if at all we are not getting any more packets
                                            loopjob be will taken over). Now we have to link the
                                            floating / newly loaded param set to xfer channel.
                                            */

                                            retVal = EDMA3_DRV_linkChannel(
                                                            ioHandle->edmaHandle,
                                                            ioHandle->xferChan,
                                                            ioHandle->pramTbl[ioHandle->nextLinkParamSetToBeUpdated]
                                                            );

                                            if (PSP_SOK != retVal)
                                            {
                                                retVal = PSP_MCASP_DMA_ERROR;
                                                break;
                                            }
                                        }
                                    }
                                }
                            } /* Floating ListEmpty is FALSE */


                            EDMA3_DRV_enableTransfer(ioHandle->edmaHandle,ioHandle->xferChan,EDMA3_DRV_TRIG_MODE_EVENT);

                            dlbMode = (Bool) CSL_FEXT(deviceHandle->hwInfo.regs->DLBCTL, MCASP_DLBCTL_DLBEN);

                            if( CSL_MCASP_DLBCTL_DLBEN_ENABLE == dlbMode)
                            { /* start both RCV and XMT state m/cs */

                                /* Activate the transmit and receive clocks
                                */
                                retVal = LLC_mcaspActivateClkRcvXmt(&(deviceHandle->hwInfo));

                                if (PSP_SOK != retVal)
                                {
                                    retVal = PSP_E_INVAL_PARAM;
                                }

                                ddc_activateSmFsForBoth(deviceHandle);
                            }
                            else
                            {
                                if ( PSP_MCASP_INPUT == ioHandle->mode)
                                {
                                    /* start Receive state m/c */
                                    /*
                                     * before touching McASP registers, place receive section in reset
                                     */
                                    retVal = LLC_mcaspActivateRcvClkSer (&(deviceHandle->hwInfo));

                                    if (PSP_SOK != retVal)
                                    {
                                        retVal = PSP_E_INVAL_PARAM;
                                    }

                                    ddc_activateSmFsForRcv(deviceHandle);

                                }
                                else
                                {
                                    /* start Transmit state m/c*/
                                    /*
                                     * before touching McASP registers, place receive
                                     * section in reset
                                     */

                                    retVal = LLC_mcaspActivateXmtClkSer (&(deviceHandle->hwInfo));

                                    if (PSP_SOK != retVal)
                                    {
                                        retVal = PSP_E_INVAL_PARAM;
                                    }

                                    ddc_activateSmFsForXmt(deviceHandle);
                                }

                            }

                        }/* DMA driven is TRUE if loop */
                        break;
                    }

                case PSP_MCASP_STOP_PORT:
                    {
                        if (PSP_MCASP_INPUT == ioHandle->mode)
                        {
                            if (FALSE == ioHandle->deviceHandle->stopSmFsRcv)
                            {
                                ioHandle->deviceHandle->stopSmFsRcv = TRUE;
                            }
                            else
                            {
                                 /* Already stop port command is issued. This
                                  * is a redundant command.
                                  */
                                 retVal = PSP_E_INVALID_CMD;
                                 break;
                            }

                        }
                        else
                        {
                            if (FALSE == ioHandle->deviceHandle->stopSmFsXmt)
                            {
                                ioHandle->deviceHandle->stopSmFsXmt = TRUE;
                            }
                            else
                            {
                                 /* Already stop port command is issued. This
                                  * is a redundant command.
                                  */
                                 retVal = PSP_E_INVALID_CMD;
                                 break;
                            }
                        }

                    }

                    break;

                case PSP_MCASP_PAUSE:
                    {
                        if (FALSE == ioHandle->paused)
                        {
                            ioHandle->paused = TRUE;
                        }
                        else
                        {
                            retVal = PSP_E_INVAL_PARAM;
                        }
                        break;
                    }

                case PSP_MCASP_RESUME:
                    {
                        if (FALSE == ioHandle->paused )
                        {
                            /* Pause is not issued so can not perform resume */
                            retVal = PSP_E_INVALID_CMD;
                            break;
                        }

                        if(TRUE == ioHandle->isDmaDriven)
                        {

                            if (TRUE == PAL_osList_NP_IsEmpty (&ioHandle->queueFloatingList))
                            {
                                for (linkCnt = 0;linkCnt < MAXLINKCNT;linkCnt++)
                                {
                                    if( FALSE == PAL_osList_NP_IsEmpty (&ioHandle->queueReqList) )
                                    {
                                        ioPacket = (DDC_mcaspIOP *)PAL_osList_NP_Dequeue(
                                                                            &ioHandle->queueReqList
                                                                            );

                                        if (NULL == ioPacket)
                                        {
                                            retVal =   PSP_E_INVAL_PARAM;
                                            break;
                                        }

                                        /* floating queue not full . Process this request.*/
                                        PAL_osList_NP_Enqueue(&(ioHandle->queueFloatingList),ioPacket);

                                        ioHandle->currentDataSize = (Uint16)ioPacket->size;
                                        ioHandle->userDataBufferSize = (Uint32)ioPacket->size;

                                        /*
                                        for DMA mode we will not use ioHandle->dataPacket
                                        to hold current packet - we use floatingqueue
                                        */
                                        ioHandle->dataPacket = NULL;

                                        if( (MAXLINKCNT-1u) == linkCnt)
                                        {
                                            /*
                                            Though we have to post to param set directly from here,
                                            there will be differene between first such packet and
                                            second packet. As we have control here we are second packet
                                            and first packet has not yet returned (or corresponding
                                            edma callback has not been called.For second packet, we
                                            will be updating the second param set, which is currently
                                            hosting loopjob parameter. Hence increment the index to
                                            point second paramset and since we are moving out loopjob
                                            from both param sets, the loopjobUpdatedinParamset is reset
                                            */
                                            ioHandle->loopjobUpdatedinParamset =FALSE;
                                            GetNextIndex(ioHandle->nextLinkParamSetToBeUpdated );
                                        }
                                        /*
                                        else --
                                        this is a first packet - meaning one paramset is not active (to which currently index is pointing and
                                        the other hosting active self linked loopjob
                                        */

                                        if (PSP_SOK != ddc_updateDataPacketTolinkParams(ioHandle, ioPacket))
                                        {
                                            retVal = PSP_MCASP_DMA_ERROR;
                                            break;
                                        }


                                        if ( 0 == linkCnt )
                                        {
                                            /*
                                            if at all this is the very first packet, then one param set
                                            has loop job loaded , self linked and active with the main
                                            xfer channel param. other param set is ready loaded (just now
                                            and has link paramater set as the one having loopjob (this is
                                            to ensure that if at all we are not getting any more packets
                                            loopjob be will taken over). Now we have to link the
                                            floating / newly loaded param set to xfer channel.
                                            */

                                            retVal = EDMA3_DRV_linkChannel(
                                                            ioHandle->edmaHandle,
                                                            ioHandle->xferChan,
                                                            ioHandle->pramTbl[ioHandle->nextLinkParamSetToBeUpdated]
                                                            );

                                            if (PSP_SOK != retVal)
                                            {
                                                retVal = PSP_MCASP_DMA_ERROR;
                                                break;
                                            }
                                        }
                                    }
                                }
                            } /* Floating ListEmpty is FALSE loop */
                        }

                        ioHandle->paused = FALSE;
                        /* Implement the Pause algorithm */
                        break;
                    }

                case PSP_MCASP_QUERY_AMUTE:
                    {
                        retVal = (PAL_Result)LLC_mcaspGetHwStatus(&(deviceHandle->hwInfo),
                                      LLC_MCASP_QUERY_AMUTE,
                                      &queryRes );
                        /*Sending back the value of the Amute Register*/
                        *(Uint32 *)arg = queryRes;
                        if (PSP_SOK != retVal)
                        {
                            retVal = PSP_E_INVAL_STATE;
                            break;
                        }

                        break;
                    }
                /*
                 * Enable / disable loopjob.
                 * If the buffer pointer is,
                 * NULL then disable loopjob
                 * not NULL then enable the loop job and asign buffer poiter to prdbuf
                 */
                case PSP_CTRL_McASP_MODIFY_LOOPJOB:
                    {
                        params = (PSP_mcaspChanParams *)arg;

                        if(NULL == params)
                        {
                            retVal =  PSP_E_NOT_SUPPORTED;
                            break;
                        }

                        if(TRUE == ioHandle->isDmaDriven)
                        {
                            if (PSP_SOK !=  EDMA3_DRV_getPaRAM(
                                                ioHandle->edmaHandle,
                                                ioHandle->pramTbl[((ioHandle->nextLinkParamSetToBeUpdated + 1)% 2)],
                                                &pramPtr
                                                ))
                            {
                                retVal = PSP_E_INVAL_PARAM;
                                break;
                            }

                            /*
                            here we need to change only the buffer pointer and the lenght of the array
                            which will go into bcnt
                            */
                            if (PSP_MCASP_OUTPUT == (ioHandle->mode))
                            {
                                ioHandle->loopJobBuffer  = &loopSrcBuf[(deviceHandle->InstNum)];
                                ioHandle->loopJobLength  = 4u;

                                if ( NULL != params->userLoopJobBuffer)
                                {
                                    ioHandle->loopJobBuffer   = params->userLoopJobBuffer;
                                    ioHandle->loopJobLength   = params->userLoopJobLength;
                                }
                                pramPtr.srcAddr = (Uint32)params->userLoopJobBuffer;
                                /* here it was bCnt. We need to configure cCnt with loopjob count*/
                                pramPtr.cCnt     = ioHandle->loopJobLength/(ioHandle->roundedWordWidth);

                            }
                            else
                            {
                                ioHandle->loopJobBuffer  = &loopDstBuf[(deviceHandle->InstNum)];
                                ioHandle->loopJobLength  = 4u;

                                if ( NULL != params->userLoopJobBuffer)
                                {
                                    ioHandle->loopJobBuffer   = params->userLoopJobBuffer;
                                    ioHandle->loopJobLength   = params->userLoopJobLength;
                                }

                                pramPtr.destAddr = (Uint32)params->userLoopJobBuffer;
                                /* here it was bCnt. We need to configure cCnt with loopjob count*/
                                pramPtr.cCnt     = ioHandle->loopJobLength/(ioHandle->roundedWordWidth);

                            }



                            /* set BUS to VBUSP i.e DMA mode */
                            CSL_FINST (deviceHandle->hwInfo.regs->AFSRCTL, MCASP_RFMT_RBUSEL, VBUSP);

                            if (PSP_SOK !=  EDMA3_DRV_setPaRAM(
                                                    ioHandle->edmaHandle,
                                                    ioHandle->pramTbl[((ioHandle->nextLinkParamSetToBeUpdated + 1) % 2)],
                                                    &pramPtr
                                                    ))
                            {
                                retVal = PSP_E_INVAL_PARAM;
                                break;
                            }

                        }/* DMA mode */
                        else
                        {
                            retVal =  PSP_E_NOT_SUPPORTED;
                            break;
                        }
                        break;
                    }


                /*
                 * MUTE-ON command implementation
                 * Typical implementation for Mute is writing zero data
                 * in buffer instead of writing real data.
                 */
                case  PSP_CTRL_McASP_MUTE_ON:
                    {
                        if (PSP_MCASP_OUTPUT == ioHandle->mode)
                        {
                            if(TRUE == ioHandle->bMuteON)
                            {
                                retVal = PSP_E_INVAL_PARAM;
                                break;
                            }

                            ioHandle->bMuteON = (Bool)TRUE;
                        }
                        else
                        {
                            retVal = PSP_E_NOT_SUPPORTED;
                        }

                        break;
                    }

                /* Take out from the the Muted state */
                case PSP_CTRL_McASP_MUTE_OFF:
                    {
                        if (PSP_MCASP_OUTPUT == ioHandle->mode)
                        {
                            /* Check if mute is not ON */
                            if (FALSE == ioHandle->bMuteON )
                            {
                                retVal = PSP_E_INVAL_PARAM;
                                break;
                            }

                            ioHandle->bMuteON =(Bool)FALSE;

                        }/*if only dma driven transaction*/
                        else
                        {
                            retVal = PSP_E_NOT_SUPPORTED;
                        }
                        break;

                    }

                case  PSP_CTRL_RCV_GPIO_INPUT:
                    {
                        /* Configure AFSR in GPIO mode */
                        CSL_FINS(
                            ioHandle->deviceHandle->hwInfo.regs->PFUNC,
                            MCASP_PFUNC_AFSR,
                            0x1u
                            );

                        /* Configure AFSR in GPIO input mode */
                        CSL_FINST(
                            ioHandle->deviceHandle->hwInfo.regs->PDIR,
                            MCASP_PDIR_AFSR,
                            INPUT
                            );

                        /* Configure AFSR in GPIO mode */
                        CSL_FINS(
                            ioHandle->deviceHandle->hwInfo.regs->PFUNC,
                            MCASP_PFUNC_ACLKR,
                            0x1u
                            );

                        /* Configure AFSR in GPIO input mode */
                        CSL_FINST(
                            ioHandle->deviceHandle->hwInfo.regs->PDIR,
                            MCASP_PDIR_ACLKR,
                            INPUT
                            );

                        break;
                    }
                default:
                    /*
                    * No other control command is supported. Return a not-implemented
                    * status for these commands.
                    */

                    retVal = PSP_E_NOT_SUPPORTED;
                    break;
            }
        }
    }
    return (retVal);
}


/**
 *  \brief ddc_edmaProcessPkt
 *
 *       For the transmit operation, the eDMA channel's destination
 *       port is tied to the MCASP DAT port. In case of receive, the eDMA
 *       channel's source port is tied to the MCASP DAT port. The source
 *       address for transmit eDMA channel and the destination address for
 *       the receive eDMA channel are set here.
 *
 *  \param    iohandlep   [IN]    Pointer to channel
 *  \param    ioPacket    [IN]    Pointer to request to be processed
 *
 *  \return  IOM_COMPLETED, if the address is set correctly
 *            PSP_MCASP_ABORTED IOM_EBADIO otherwise
 */
static PAL_Result ddc_edmaProcessPkt(Ptr iohandlep, DDC_mcaspIOP *ioPacket)
{

    DDC_mcaspIoHandle       ioHandle        = NULL;
    DDC_mcaspDeviceHandle   deviceHandle    = NULL;
    PSP_Result              result          = PSP_SOK;
    Bool                    falsewhile      = FALSE;


    do
    {

        falsewhile      = FALSE;

        /*
        Validate pointers and parameters
        */
        if((NULL == iohandlep)||(NULL==ioPacket))
        {
            result = PSP_E_INVAL_PARAM;
            break;
        }

        ioHandle = (DDC_mcaspIoHandle)iohandlep;


        if(NULL == ioHandle)
        {
            result = PSP_E_INVAL_PARAM;
            break;
        }
        else
        {
            if(NULL == ioHandle->deviceHandle)
            {
                result = PSP_E_INVAL_PARAM;
                break;
            }
        }

        deviceHandle = ioHandle->deviceHandle;


        if (PSP_MCASP_CHAN_RCV != ioHandle->channelOpMode)
        {

            if ((PSP_MCASP_CHAN_XMT_DIT == ioHandle->channelOpMode) &&
                            (TRUE == ioPacket->writeDitParams))
            {

                /*
                * DIT Mode:
                *for DIT mode as of now each packet (IOP) carries/should carry 1 super frame
                * = 192 frames = 384 sub frames of data max
                */
                result = LLC_mcaspWriteChanStatRam (
                                        &(deviceHandle->hwInfo),
                                        (PSP_McaspChStatusRam *)ioPacket->chStat
                                        );

                if (PSP_SOK != result)
                {
                    result = PSP_MCASP_ABORTED;
                    break;
                }
                result = LLC_mcaspWriteUserDataRam (
                                        &(deviceHandle->hwInfo),
                                        (PSP_McaspUserDataRam *)ioPacket->userData
                                        );

                if (PSP_SOK != result)
                {
                    result = PSP_MCASP_ABORTED;
                    break;
                }

            }
        }

        /*
        we are going to update the paramset from EDMA callback context.
        That means the one of the param sets are having audio data
        just completed data request and one having currently loaded (active) data request
        Current index is pointing only latest paramset (that is active)
        so move the index
        */
        GetNextIndex(ioHandle->nextLinkParamSetToBeUpdated );

        /*Link the pakcet*/
        result = ddc_updateDataPacketTolinkParams(ioHandle, ioPacket);



    }while(falsewhile);

    return (result); /* return */
}


/*!
 *  \fn static void ddc_updateDataPacketTolinkParams(DDC_mcaspIoHandle ioHandle, DDC_mcaspIOP *request)
 *
 *  \brief
 *      This function updates the link param set accordingly with data or loopjobbuffer
 *      This uses properly maintained index to idendtify to what param set the
 *      the info has to be updated.
 *
 *
 *  \param  ioHandle [IN]  Channel handle
 *  \param  ioPacket [IN]  Pointer I/O request
 *
 *  \return None
 */
static PAL_Result ddc_updateDataPacketTolinkParams(DDC_mcaspIoHandle ioHandle, DDC_mcaspIOP * const ioPacket)
{

    EDMA3_DRV_PaRAMRegs         pramPtr         = {0};
    DDC_mcaspDeviceHandle       deviceHandle    = NULL;
    Bool                        falsewhile      = FALSE;
    PAL_Result                  retVal          = PSP_SOK;

#ifdef MCASP_INSTRUMENTATION_ENABLED
    if (PSP_MCASP_OUTPUT == ioHandle->mode){
        PSP_LOG_EVENT(
            &DVTEvent_Log,
            "McASP",
            PSP_DVT_DESC(
                    PSP_DVT_eDATA_SND_START,
                    PSP_DVT_dCHANNEL_ID_INST,
                    PSP_DVT_dPACKET_ID,
                    PSP_DVT_dNONE
                    ),
            CHANNELID_INSTANCEID(
                ioHandle->mode,
                ioHandle->deviceHandle->InstNum),
            ioPacket->addr
            );
    }
    else if (PSP_MCASP_INPUT == ioHandle->mode){
        PSP_LOG_EVENT(
            &DVTEvent_Log,
            "McASP",
            PSP_DVT_DESC(
                    PSP_DVT_eRCV_START,
                    PSP_DVT_dCHANNEL_ID_INST,
                    PSP_DVT_dPACKET_ID,
                    PSP_DVT_dNONE
                   ),
            CHANNELID_INSTANCEID(
                ioHandle->mode,
                ioHandle->deviceHandle->InstNum),
            ioPacket->addr
            );
    }
#endif /* MCASP_INSTRUMENTATION_ENABLED */

    do
    {
        falsewhile      = FALSE;
        if (NULL == ioHandle)
        {
            retVal = PSP_E_INVAL_PARAM;
            break;
        }

        if (NULL == ioHandle->deviceHandle)
        {

            retVal = PSP_E_INVAL_PARAM;
            break;
        }

        deviceHandle = ioHandle->deviceHandle;

        /*
        Get current info from the identified param
        we would like to change only certain parameters
        */
        if (PSP_SOK !=  EDMA3_DRV_getPaRAM(
                                ioHandle->edmaHandle,
                                ioHandle->pramTbl[ioHandle->nextLinkParamSetToBeUpdated],
                                &pramPtr
                                ))
        {
            retVal = PSP_E_INVAL_PARAM;
            break;
        }

        /*
        following the explanation behind the EDMA configuration

        1) We will have n arrays of data (now it is data buffer each of length databuffersize bytes)
        2) for each sync event we have to fetch the data from each array and write to each serilaiser
        3) and the no. bytes transferred in this case from each array will be roundedwordwidth/8
        4) as the loopjob buffer is contigous, we configure the counts as follows
            a) consider there are arrays of length roundedwordwidth/8 (aCnt)
            b) these arrays are located apart databuffersize size  (bindex)
            c) there are 'no.of.serialiser' no. of such arrays   (bCnt)
            d)  (a) and (b) constitute one frame of EDMA transfer and it happens for
                one sync event in AB sync mode
            e) such frames have to be sent till we have buffer - ie databuffersize / (roundedwordwidth/8 )
                and this constitutes  cCnt
            f)  address of buffer (rather offset from start of previous frame) is roundedwordwidth/8
                 and this forms cIndex
        Please take some time to visualise the setup mentioned above to check with EDMa configuration

        Also we change only specific parameters because others will not have change from what we programed
        intially
        */

         /* Load the buffer pointer into the EDMA */

        /*
        control will reach this function is two contexts
            1) From submitreq function for first time packets
            2)from EDMA callback for further handling of packets in Queue
                -we are from EDMA callback becuase we have completed some user data packet just now
        */

        if ( NULL != ioPacket)
        {

            /*
            1) when control comes to this function from submitrteq control will surely comeinto the block
            2) When control comes to this function from EDMA callback, control will some only
              if there is fuirther more packets to handle
            */

            if (PSP_MCASP_INPUT == ioHandle->mode)
            {
                pramPtr.destAddr = (Uint32)ioPacket->addr;
                pramPtr.destBIdx = (Int16)ioHandle->userDataBufferSize;
                pramPtr.destCIdx = (Int16)ioHandle->roundedWordWidth;
                pramPtr.cCnt     = (Uint16)(ioHandle->userDataBufferSize/(ioHandle->roundedWordWidth));
            }
            else
            {

                if (TRUE == ioHandle->bMuteON)
                {

                    /*data buffer is changed to local mutebuffer*/
                    pramPtr.srcAddr  = (Uint32)&muteBuf[(deviceHandle->InstNum)];
                    pramPtr.srcBIdx  = (Int16)ioHandle->userDataBufferSize;
                    pramPtr.cCnt     = (Uint16)(ioHandle->userDataBufferSize/(ioHandle->roundedWordWidth));

                    /*Start again with same mute buffer*/
                    pramPtr.srcCIdx = 0;
                }
                else
                {
                    pramPtr.srcAddr = (Uint32)ioPacket->addr;
                    pramPtr.srcBIdx = (Int16)ioHandle->userDataBufferSize;
                    pramPtr.srcCIdx = (Int16)ioHandle->roundedWordWidth;
                    pramPtr.cCnt    = (Uint16)(ioHandle->userDataBufferSize/(ioHandle->roundedWordWidth));
                }/*for mute on/off*/
            }

            /*
            * Here we are enabling the interrupt for this param set information
            * we do it here in this way because using int enable API after
            * calling EDMA3_DRV_setPaRAM, may fail our intention. This is because
            * immediately after the set param the paramset might be copied to
            * linked param and will make the consecutive int enable (set opt field)
            * api useless for our purpose
            */
            pramPtr.opt |= (0x01u<<20u);

            /*
            we are loading data request in the param set. As per our design
            when we load the data,  we should link this param set with next one
            most probably
                a)That may have loopjob loaded
                b)it may be the currently loaded set, which will be updated in next edma callback
                  hence in both cases to be run next to this data request
            */
            pramPtr.linkAddr =
                (Uint16)(ioHandle->pramTblAddr[((ioHandle->nextLinkParamSetToBeUpdated+1)&0x01)]
                        &
                        0x0000FFFF
                        );


        }
        else
        {
            /*
            we have reached here becuase we dont have any more packett to submit for Xfer
            assign loop job to params - no change in linking
            IMP: The following assignment will be skipped if already both params are having
            loopjob buffer
            */

            if (PSP_MCASP_INPUT == ioHandle->mode)
            {
                pramPtr.destAddr = (Uint32)ioHandle->loopJobBuffer;
                pramPtr.destBIdx = (Int16)ioHandle->loopJobLength;
                pramPtr.destCIdx = (Int16)ioHandle->roundedWordWidth;
                pramPtr.cCnt     = (Uint16)(ioHandle->loopJobLength/(ioHandle->roundedWordWidth));
            }
            else
            {
                pramPtr.srcAddr = (Uint32)ioHandle->loopJobBuffer;
                pramPtr.srcBIdx = (Int16)ioHandle->loopJobLength;
                pramPtr.srcCIdx = (Int16)ioHandle->roundedWordWidth;
                pramPtr.cCnt    = (Uint16)(ioHandle->loopJobLength/(ioHandle->roundedWordWidth));
            }

            /*
            we are loading loopjob into this param.  That means we have reached the end of packets in
            pending and floating queues. So this loopjob had to be  linked to itself.
            Also we need to point the index to other param set in order to load data request
            directly from submit call
            */
            pramPtr.linkAddr = (Uint16)
                (ioHandle->pramTblAddr[ioHandle->nextLinkParamSetToBeUpdated] & 0x0000FFFF);

            pramPtr.opt &= (~(0x01<<20));
        }

        /*
        Update the changed param set info into the appropriate
        paramset
        */
        if (PSP_SOK !=  EDMA3_DRV_setPaRAM(
                                ioHandle->edmaHandle,
                                ioHandle->pramTbl[ioHandle->nextLinkParamSetToBeUpdated],
                                &pramPtr
                                ))
        {
            retVal = PSP_E_INVAL_PARAM;
            break;
        }

        if ( NULL == ioPacket)
        {

            /*
            we are loading loopjob into this param.
            we need to point the index to other param set in order to load data request
            directly from submit call
            */
            GetNextIndex(ioHandle->nextLinkParamSetToBeUpdated );
        }


    }while(falsewhile);

    return retVal;
}

/**
 *  \brief ddc_edmaMcaspCallback
 *
 *       This calls the registered application callback for the current
 *       receive request and processes the next request in queue.
 *
 *  \param      tcc     [IN]    tcc number of eDMA
 *  \param      status  [IN]    status of eDMA
 *  \param      data    [IN]    paramaters send to eDMA ISR (chan handler)
 *
 *  \return     void
 */

static void ddc_edmaMcaspCallback(Uint tcc, EDMA3_RM_TccStatus status, Ptr data)
{

    DDC_mcaspIoHandle       ioHandle        = NULL;
    Bool                    falsewhile      = FALSE;
    Uint32                  dlbMode         = 0;
    PAL_Result              retVal          = PSP_SOK;
	EDMA3_DRV_PaRAMRegs     pramTbl         = {0};
	DDC_mcaspIOP            *floatingPacket = NULL;
    /*
    To remove warning, kept for future usage
    */
    tcc = tcc;
    status = status;
    if(NULL != data)
    {
        data = data;
    }
    do
    {
        falsewhile      = FALSE;

        ioHandle = (DDC_mcaspIoHandle)data;

        if(NULL == ioHandle)
        {
            break;
        }

        if(TRUE==PAL_osList_NP_IsEmpty(&(ioHandle->queueFloatingList)))
        {

            /*This cannot happen, if it happens then it is a sprurios one*/
            break;
        }

        /*
        for DMA transactions all the floating packets are managed in queueFloatingList
        Now pop out the packet from the queue and complete it
        */
        ioHandle->tempPacket = PAL_osList_NP_Dequeue(&(ioHandle->queueFloatingList));

		/**
							 -------
							| xfer	|
							| chan	|
							 -------
														Floating queue
														--------------
					 -------	 -------	 -------		|
					|		|	|		|	|		|<------
					|   B	|	|   A	|	|   A	|
					 -------	 -------     -------
											|		|
											|   B	|
									 		 -------
		    As shown in the figure above, There are two IO request present in
		    floating queue. Whenever EDMA completetion callback is called,
		    xfer channel will be transferring data of one of the request. As
		    this is a ping-pong implementation, link params are linked to each
		    other. Meaning once data transfer of IO A is completed, data transfer
		    of IO B will be initiated. Ideally, when control comes to this
		    function xfer channel should be servicing the 2nd request of the
		    floating queue. Here, in above illustration when callback is called,
		    xfer channel should be servicing request B while request A is
		    completed.

		    When due to any reason, CPU is halted (it can be due to break-point,
		    memory access view using CCS, accessing variables using watch window
		    etc.) EDMA will continue trasferring data. So assuming when CPU is
		    halted EDMA is servicing request A, data transfer of request A will
		    continue but ISR of transfer complete will not serve as CPU is halted.
		    As request A is complete and param A is linked with param B which
		    is holding request B, EDMA will start servicing request B. Again
		    EDMA will complete the request B service but transfer complete
		    ISR will not come as CPU is halted. But now as this is a ping-pong
		    implementation, EDMA will start servicing request A. This will
		    ping-pong will continue until CPU is taken out of halt. After that
		    first transfer complete ISR is due to which request is not known to
		    driver. It can be either request A or B. The problem will arise when
		    xfer channel is servicing A when callback is called. In that case,
		    request that is in the servicing, will be returned to the application
		    as serviced. This will lead to continuous noise.

            To take care the above problem, when callback is called, check is
            done to verify if the xfer channel is servicing the request A or
            B. If EDMA is servicing request A, requests are queued in a proper
            sequence and no callback is called. When a next callback is called
            it will be of request A, so floating queue is dequeued and from
            there on normal operation will continue.
		*/

		/* get the param table information of transfer channel */
		EDMA3_DRV_getPaRAM(ioHandle->edmaHandle,ioHandle->xferChan,&pramTbl);

        if (ioHandle->mode == PSP_MCASP_INPUT)
		{
		    /*
		        Check if destination address falls into the range of
		        1st request in the floating queue.
		    */
		    if ((pramTbl.destAddr >= (Uint32)ioHandle->tempPacket->addr) &&
			    (pramTbl.destAddr < (Uint32)ioHandle->tempPacket->addr
			                              + ioHandle->tempPacket->size)
               )
			{
			    /* Since we have already dequeue the 1st request, dequeue 2nd
			    io request from floating queue */
	            floatingPacket = PAL_osList_NP_Dequeue(&(ioHandle->queueFloatingList));

				/* Queue the tempPacket (i.e. 1st io request) as this packet should be
				  first in a queue */
                PAL_osList_NP_Enqueue(&(ioHandle->queueFloatingList), ioHandle->tempPacket);

				/* Queue the floatingPacket i.e. 2nd request in floating queue */
                PAL_osList_NP_Enqueue(&(ioHandle->queueFloatingList), floatingPacket);

				break;
			}

		}
		else
		{
		    /*
		        Check if destination address falls into the range of
		        1st request in the floating queue.
		    */
		    if ((pramTbl.srcAddr >= (Uint32)ioHandle->tempPacket->addr)&&
			    (pramTbl.srcAddr < (Uint32)ioHandle->tempPacket->addr
			                          +ioHandle->tempPacket->size)
			   )
			{
			    /* Since we have already dequeue the 1st request, dequeue 2nd
			    io request from floating queue */
	            floatingPacket = PAL_osList_NP_Dequeue(&(ioHandle->queueFloatingList));

				/* Queue the tempPacket (i.e. 1st io request) as this packet should be
				  first in a queue */
                PAL_osList_NP_Enqueue(&(ioHandle->queueFloatingList), ioHandle->tempPacket);

				/* Queue the floatingPacket i.e. 2nd request in floating queue */
                PAL_osList_NP_Enqueue(&(ioHandle->queueFloatingList), floatingPacket);

				break;
			}

		}


#ifdef MCASP_INSTRUMENTATION_ENABLED
        if (PSP_MCASP_OUTPUT == ioHandle->mode){
            PSP_LOG_EVENT(
                &DVTEvent_Log,
                "McASP",
                PSP_DVT_DESC(
                        PSP_DVT_eDATA_SND_END,
                        PSP_DVT_dCHANNEL_ID_INST,
                        PSP_DVT_dPACKET_ID,
                        PSP_DVT_dNONE
                        ),
                CHANNELID_INSTANCEID(
                    ioHandle->mode,
                    ioHandle->deviceHandle->InstNum),
                ioHandle->tempPacket->addr
                );
        }
        else if (PSP_MCASP_INPUT == ioHandle->mode){
            PSP_LOG_EVENT(
                &DVTEvent_Log,
                "McASP",
                PSP_DVT_DESC(
                        PSP_DVT_eRCV_END,
                        PSP_DVT_dCHANNEL_ID_INST,
                        PSP_DVT_dPACKET_ID,
                        PSP_DVT_dNONE
                       ),
                CHANNELID_INSTANCEID(
                    ioHandle->mode,
                    ioHandle->deviceHandle->InstNum),
                ioHandle->tempPacket->addr
                );
        }

#endif /* MCASP_INSTRUMENTATION_ENABLED */

        if ( EDMA3_RM_XFER_COMPLETE != status)
        {
            /* Clear the EDMA Error bits */
            ioHandle->tempPacket->status  =  PSP_MCASP_DMA_ERROR;
            /*
             There is a data transfer error in EDMA transfer. Return actualSize
             equal to 0 represent that there is an error in the packet transfer
            */
            ioHandle->tempPacket->actualSize = 0;
            EDMA3_DRV_clearErrorBits(ioHandle->edmaHandle, ioHandle->xferChan);
        }
        else
        {
            ioHandle->tempPacket->status  =  PSP_SOK;
            ioHandle->tempPacket->actualSize = \
                (Uint32) (ioHandle->tempPacket->size - ioHandle->currentDataSize);

			if(PSP_SOK != ioHandle->currentPacketErrorStatus )
			{
				ioHandle->tempPacket->status = ioHandle->currentPacketErrorStatus;
				ioHandle->currentPacketErrorStatus = PSP_SOK;
	            ioHandle->tempPacket->actualSize = 0;

			}

        }

        ioHandle->submitCount--;

        /* If stopSmFs flag for the channel is false and pause is not issued
         * control will go in the if part. In case when stopSmFS is issued or
         * pause is issued, control will go in else part
         */
        if ((((PSP_MCASP_INPUT == ioHandle->mode)
            &&(FALSE == ioHandle->deviceHandle->stopSmFsRcv))
            ||
            ((PSP_MCASP_OUTPUT == ioHandle->mode)
            &&(FALSE == ioHandle->deviceHandle->stopSmFsXmt)))
            &&
            (FALSE == ioHandle->paused))
        {
            /*
            Now that we have done with the last data packet - we check if next packet is available
            for transmission. Even if we are not having a valid packet - previous programming of
            linkchannels (designed in such a way) to send out null buffer
            Also note that ddc_isValidPacket function will set the ioHandle->dataPacket with
            next (current now) data packet from queue
            */
            if ((PSP_MCASP_VALID == (ddc_isValidPacket(ioHandle))))
            {
                /*
                * This is a valid data request. Call function to process it.
                * If DMA cannot be programmed properly with request contents, abort all queued
                * up requests and put state-machine into reset.
                */
                if (PSP_MCASP_ABORTED == (ddc_edmaProcessPkt(ioHandle, ioHandle->dataPacket)))
                {
                    ddc_cancelAndAbortAllIo(ioHandle);
                }
            }
            else
            {
                /*
                the following function will assign loopjob buffer to free param set
                Also the control will come here two times for last packet and last but one
                packet (obviously for more than one packet xfer) For last but packet callback
                itself we will come to know that there is no more packet pending and hence
                we load loopjob info into next param set and for the last packet callback will not
                do it again (for only one packet scenario the following
                flag will take care not to update the loopjob second time)
                */

                if( FALSE  == ioHandle->loopjobUpdatedinParamset)
                {

                    ioHandle->loopjobUpdatedinParamset = TRUE;

                    GetNextIndex(ioHandle->nextLinkParamSetToBeUpdated );

                    ddc_updateDataPacketTolinkParams(ioHandle, NULL);
                }



            }
        }
        else
        {
            /*
            Control will come over here when either of receive or transmit state
            machine stop command is issued for receive or transmit channel.
            If the nextFlag is already set indicates that its time to
            reset the state machines and disable the edma transfer
            */
            if (TRUE == ioHandle->nextFlag)
            {
                ioHandle->nextFlag = FALSE;
                if (FALSE == ioHandle->paused)
                {
                    dlbMode = (Uint32) CSL_FEXT(ioHandle->deviceHandle->hwInfo.regs->DLBCTL, MCASP_DLBCTL_DLBEN);

                    if( CSL_MCASP_DLBCTL_DLBEN_ENABLE == dlbMode)
                    {
                        /* Stop both RCV and XMT state m/cs */
                        /*
                         * before touching McASP registers, place receive section in reset
                         */
                        retVal = LLC_mcaspResetRcv (&(ioHandle->deviceHandle->hwInfo));

                        if (PSP_SOK != retVal)
                        {
                            break;
                        }
                        /*
                         * before touching McASP registers, place transmit section in reset
                         */
                        retVal = LLC_mcaspResetXmt (&(ioHandle->deviceHandle->hwInfo));

                        if (PSP_SOK != retVal)
                        {
                            break;
                        }

                    }
                    else
                    {
                        if ( PSP_MCASP_INPUT == ioHandle->mode)
                        {
                            /* stop Receive state m/c */

                            /*
                            * before touching McASP registers, place receive section in reset
                            */
                            retVal = LLC_mcaspResetRcv (&(ioHandle->deviceHandle->hwInfo));

                            if (PSP_SOK != retVal)
                            {
                                break;
                            }

                        }
                        else
                        {
                            /* stop Transmit state m/c*/
                            /*
                            * before touching McASP registers, place transmit section in reset
                            */

                            retVal = LLC_mcaspResetXmt (&(ioHandle->deviceHandle->hwInfo));
                            if (PSP_SOK != retVal)
                            {
                                break;
                            }
                        }

                    }

                    EDMA3_DRV_disableTransfer(ioHandle->edmaHandle, ioHandle->xferChan,EDMA3_DRV_TRIG_MODE_EVENT);
                }
            }
            else
            {
                /*
                For the first time when the stop port command is issued we will have one more packet
                linked with the transfer channel. So we will not stop at this time. We link the packet
                with NULL loopjob buffer and set a nextFlag to TRUE. We will wait for another callback
                (indicating the io request is complete) to disable EDMA trasnfer and reset state
                machines
                */
                ioHandle->nextFlag = TRUE;

                GetNextIndex(ioHandle->nextLinkParamSetToBeUpdated );

                ddc_updateDataPacketTolinkParams(ioHandle, NULL);
            }
        }

        ioHandle->isTempPacketValid = TRUE;

        ddc_completeCurrentIO(ioHandle);

        if (PSP_MCASP_ABORTED == retVal)
        {
            ddc_cancelAndAbortAllIo(ioHandle);
        }
        /*
        If there are no further request pending to serve and flush command is
        issued already give/post the semaphore take at the flush time.
        */
        if (0 >= ioHandle->submitCount)
        {
            if (PSP_MCASP_OUTPUT == ioHandle->mode)
            {
                if (TRUE == ioHandle->flush)
                {
                    PAL_osSemGive(ioHandle->syncSem);
                    ioHandle->flush = FALSE;
                }
            }
        }
    }while (falsewhile);

    return;

}


/**
 * \brief  ddc_isValidPacket
 *
 *    This checks is the next request in queue is data request. If this is an
 *    abort or flush request, it calls the appropriate function to deal with
 *    it.
 *
 * \param    ioHandle   [IN]     Channel handle
 *
 * \return   IOM_COMPLETED,   if request is Recieved/Transmitted completed
 *           PSP_MCASP_VALID, if request is not completed

 */
static PAL_Result ddc_isValidPacket(DDC_mcaspIoHandle ioHandle)
{
    PAL_Result              result              = 0;
    Bool                    falsewhile          = FALSE;


    do
    {
        falsewhile      = FALSE;
        if (NULL == ioHandle)
        {
            result = PSP_E_INVAL_PARAM;
            break;
        }

        if (NULL == ioHandle->deviceHandle)
        {
            result = PSP_E_INVAL_PARAM;
            break;

        }

        /*
        * Present request validation for int mode transfer
        */
        if ( ( (FALSE == ioHandle->isDmaDriven) &&
             (NULL != ioHandle->dataPacket) ) && (0u != ioHandle->currentDataSize))
        {

            /*
            in interrupt mode - some transaction is going on */
            result = PSP_MCASP_VALID;
        }
        else
        {
            /*
            * Look for new request to be processed
            */

            if(TRUE==PAL_osList_NP_IsEmpty(&(ioHandle->queueReqList)))
            {

                /*
                * we dont have any packet in queue
                * stop xmt/rct sm before giving error
                */
                ioHandle->dataPacket = NULL;



                if(FALSE == ioHandle->isDmaDriven)
                {

                    /*
                    As interrupt mode there is not loopjob implementation
                    we have to stop the state machine here
                    */
                    ioHandle->dataQueuedOnReset = 0;
                    ioHandle->dataPacket = NULL;
                    ioHandle->currentDataSize = 0;
                    ioHandle->userDataBufferSize = 0;
                    ioHandle->xferinProgressIntmode = FALSE;

                    /*
                    Stopping the statemachines is taken care outside this fucntion
                    */

                }/* DMA driven is FALSE if loop */


                /*
                for DMA mode only status is enought to program the loopjob buffer
                */
                result = PSP_MCASP_INVALID;
            }
            else  
            {

                result = PSP_MCASP_VALID;


                ioHandle->dataPacket = PAL_osList_NP_Dequeue(&(ioHandle->queueReqList));


                if(NULL != ioHandle->dataPacket)
                {
                    /*
                    we have a valid packet to process next
                    */

                    /*
                    in int mode - current packet will be only one hence  we can use
                    ioHandle->dataPacket for floating / current packet
                    */

                    ioHandle->currentDataSize = (Uint16)(ioHandle->dataPacket->size);
                    ioHandle->userDataBufferSize = (Uint32)(ioHandle->dataPacket->size);
                    result = PSP_MCASP_VALID;

                    if(TRUE == ioHandle->isDmaDriven)
                    {
                        /*
                        Additional to assigning the important parameters as above
                        for DMA mode , we will have max 2 packets floating and hence
                        we use a activequue to manage this
                        */
                        PAL_osList_NP_Enqueue(&(ioHandle->queueFloatingList), ioHandle->dataPacket);

                    }
                }

            } /* Request List queue Empty is TRUE if loop */

        }

    }while(falsewhile);

    return result;
}

/**
 *  \brief ddc_smReset
 *
 *       Resets the state m/c of the McASP xmt/rcv unit. In case of interrupt
 *       driven transactions, this also resets data queued for servicing the
 *       MCASP xmt/rcv unit when it is taken out of reset.
 *
 *  \param   ioHandle  [IN]  Channel handle
 *  \return  void
 */

static void ddc_smReset(DDC_mcaspIoHandle ioHandle)
{

    DDC_mcaspDeviceHandle   deviceHandle        = NULL;
    Bool                    falsewhile          = FALSE;

    do
    {
        falsewhile = FALSE;
        if(NULL == ioHandle)
        {
            break;
        }
        else
        {
            if(NULL == ioHandle->deviceHandle)
            {
                break;
            }
        }
        deviceHandle = ioHandle->deviceHandle;
        if (PSP_MCASP_CHAN_RCV == ioHandle->channelOpMode)
        {
            /* Reset state m/c and frame sync of receive section*/
              ddc_deactivateSmFsForRcv (deviceHandle);
        }
        else
        {
            /* Reset state m/c and frame sync of transmit section*/
            ddc_deactivateSmFsForXmt (deviceHandle);
        }

        if (FALSE == ioHandle->isDmaDriven)
        {
            /* Interrupt driven transactions */

            /*
            * When iohandlenel state m/c is reactivated, it will require two
            * requests to service it. Reset count to ensure this.
            */

            ioHandle->dataQueuedOnReset = 0;
            ioHandle->xferinProgressIntmode = FALSE;

        }
        else
        {
            EDMA3_DRV_disableTransfer (ioHandle->edmaHandle, ioHandle->xferChan,EDMA3_DRV_TRIG_MODE_EVENT);
        }
    }while(falsewhile);
}

/**
 *  \brief ddc_smSet
 *
 *       Activate the state m/c of the McASP xmt/rcv unit.
 *       And enable EDMA transfer.
 *
 *  \param   ioHandle  [IN]  Channel handle
 *  \return  void
 */

void ddc_smSet(DDC_mcaspIoHandle ioHandle)
{

    DDC_mcaspDeviceHandle   deviceHandle        = NULL;
    Bool                    falsewhile          = FALSE;

    do
    {
        falsewhile = FALSE;
        if(NULL == ioHandle)
        {
            break;
        }
        else
        {
            if(NULL == ioHandle->deviceHandle)
            {
                break;
            }
        }
        deviceHandle = ioHandle->deviceHandle;

        if (FALSE == ioHandle->isDmaDriven)
        {
            /* Interrupt driven transactions */

            /*
            * When iohandlenel state m/c is reactivated, it will require two
            * requests to service it. Reset count to ensure this.
            */

            ioHandle->dataQueuedOnReset = 0;
            ioHandle->xferinProgressIntmode = FALSE;

        }
        else
        {
            /*
            * Start the DMA iohandle operation.
            */
            EDMA3_DRV_enableTransfer (ioHandle->edmaHandle, ioHandle->xferChan,EDMA3_DRV_TRIG_MODE_EVENT);
        }


        if (PSP_MCASP_CHAN_RCV == ioHandle->channelOpMode)
        {
            /* Reset state m/c and frame sync of receive section*/
            ddc_activateSmFsForRcv (deviceHandle);
        }
        else
        {
            /* Reset state m/c and frame sync of transmit section*/
            ddc_activateSmFsForXmt (deviceHandle);
        }

    }while(falsewhile);
}

/**
 *  \brief ddc_abortReset
 *
 *   Aborts queued up i/o request requests.
 *
 *  \param    ioHandle   [IN]   Channel handle
 *
 *  \return   void
 */
static void ddc_abortReset(DDC_mcaspIoHandle ioHandle)
{
    DDC_mcaspIOP      *tmpPacket                = NULL;
    Uint32            imask                     = 0;
    PSP_mcaspReq      pspRequest                = {0};
    Bool              falsewhile                = FALSE;
    Uint32            localcCnt                 = 0;
    Bool              firstReq                  = FALSE;
    DDC_mcaspDeviceHandle   deviceHandle        = NULL;

    do
    {
        falsewhile = FALSE;

        if(NULL == ioHandle)
        {
            break;
        }
        else
        {
            if(NULL == ioHandle->deviceHandle)
            {
                break;
            }
        }

        deviceHandle = ioHandle->deviceHandle;

        /*
        * This function services both transmit and receive channels. Define
        * critical section to avoid corruption of local variables.
        * CRITICAL SECTION: begins!
        */
        PAL_osProtectEntry(PAL_OSPROTECT_INTERRUPT, &imask);

        /*
        * Stop the xmt/rcv state machine.
        */
        if (PSP_MCASP_CHAN_RCV == ioHandle->channelOpMode)
        {
            /* Reset state m/c and frame sync of receive section*/
            ddc_deactivateSmFsForRcv (deviceHandle);
        }
        else
        {
            /* Reset state m/c and frame sync of transmit section*/
            ddc_deactivateSmFsForXmt (deviceHandle);
        }

        /* Disable EDMA  transfer */
        EDMA3_DRV_disableTransfer (
                    ioHandle->edmaHandle,
                    ioHandle->xferChan,
                    EDMA3_DRV_TRIG_MODE_EVENT
                    );

        /* Clear the error bits of McASP peripheral due to stopping of
           peripheral.
         */
        if (PSP_MCASP_INPUT == ioHandle->mode)
        {

            CSL_FINST(
                    ioHandle->deviceHandle->hwInfo.regs->RSTAT,
                    MCASP_RSTAT_RDMAERR,
                    NO_ERROR
                    );
            CSL_FINST(
                    ioHandle->deviceHandle->hwInfo.regs->RSTAT,
                    MCASP_RSTAT_ROVRN,
                    NO
                    );
            CSL_FINST(
                    ioHandle->deviceHandle->hwInfo.regs->RSTAT,
                    MCASP_RSTAT_RSYNCERR,
                    NO
                    );
        }
        else
        {
            CSL_FINST(
                    ioHandle->deviceHandle->hwInfo.regs->XSTAT,
                    MCASP_XSTAT_XDMAERR,
                    NO_ERROR
                    );
            CSL_FINST(
                    ioHandle->deviceHandle->hwInfo.regs->XSTAT,
                    MCASP_XSTAT_XUNDRN,
                    NO
                    );
            CSL_FINST(
                    ioHandle->deviceHandle->hwInfo.regs->XSTAT,
                    MCASP_XSTAT_XSYNCERR,
                    NO
                    );
        }

        /* Get the current value of cCnt for the actual data transfered
         */
        EDMA3_DRV_getPaRAMField (
                        ioHandle->edmaHandle,
                        ioHandle->xferChan,
                        EDMA3_DRV_PARAM_FIELD_CCNT,
                        &localcCnt
                        );

        firstReq      = TRUE;

        if (TRUE == ioHandle->isDmaDriven)
        {
            /*Empty the floating queue. */
            while (TRUE !=  PAL_osList_NP_IsEmpty(&ioHandle->queueFloatingList))
            {
                tmpPacket = (DDC_mcaspIOP *)PAL_osList_NP_Dequeue(&ioHandle->queueFloatingList);

                if (NULL == tmpPacket)
                {
                    /* Ideally control should not come here */
                    break;
                }
                /* Enque the packet back to free queue */
                PAL_osList_NP_Enqueue(&(ioHandle->queueFreeList), tmpPacket);
                /*
                 * if FLUSH cmd called for INPUT channel then
                 * status is equal to PSP_MCASP_FLUSHED otherwise
                 * status is equal to PSP_MCASP_ABORTED
                 */
                if((TRUE == ioHandle->flush) \
                        && (PSP_MCASP_CHAN_RCV == ioHandle->channelOpMode))
                {
                    tmpPacket->status = PSP_MCASP_FLUSHED;
                }
                else
                {
                tmpPacket->status = PSP_MCASP_ABORTED;
                }

                tmpPacket->actualSize = 0;

                /* Actual data transfer will be equal to size provided by
                 * application i.e.userDataBufferSize reduced by current
                 * value of cCnt * aCnt. bCnt is 1
                 */
                if (TRUE == firstReq )
                {
                    tmpPacket->actualSize = (Uint32)(tmpPacket->size - \
                                                        (localcCnt * ioHandle->roundedWordWidth));

                    firstReq = FALSE;
                }

                tmpPacket->size = tmpPacket->size * ioHandle->noOfSerAllocated;
                ddc_copyIOPtoReq(tmpPacket, (PSP_mcaspReq *)&pspRequest);

                if(NULL != pspRequest.cbFxn)
                {
                    (*pspRequest.cbFxn)(
                            (Ptr)ioHandle->deviceHandle->appHandle,
                            (Ptr)ioHandle->cbArg,
                            &pspRequest,
                            ioHandle->mode
                            );
                }
                else
                {
                    if (NULL !=  ioHandle->cbFxn)
                    {
                        (*ioHandle->cbFxn)(
                                (Ptr)ioHandle->deviceHandle->appHandle,
                                (Ptr)ioHandle->cbArg,
                                &pspRequest,
                                ioHandle->mode
                                );
                    }
                    /*else
                    {
                        * callback function is null. Somewhere the function
                        * is corrupted. Break from this location
                    }*/
                }

                /* decrement the submit count as request is aborted */
                ioHandle->submitCount--;
            }
        }
        else
        {
            tmpPacket = ioHandle->dataPacket;

            if (NULL != tmpPacket)
            {
                /*
                 * if FLUSH cmd called for INPUT channel then
                 * status is equal to PSP_MCASP_FLUSHED otherwise
                 * status is equal to PSP_MCASP_ABORTED
                 */
                if((TRUE == ioHandle->flush) \
                        && (PSP_MCASP_CHAN_RCV == ioHandle->channelOpMode))
                {
                    tmpPacket->status = PSP_MCASP_FLUSHED;
                }
                else
                {
                tmpPacket->status = PSP_MCASP_ABORTED;
                }

                tmpPacket->size = tmpPacket->size * ioHandle->noOfSerAllocated;

                ddc_copyIOPtoReq(tmpPacket,(PSP_mcaspReq *)&pspRequest);

                if(NULL != pspRequest.cbFxn)
                {
                    (*pspRequest.cbFxn)(
                            (Ptr)ioHandle->deviceHandle->appHandle,
                            (Ptr)ioHandle->cbArg,
                            &pspRequest,
                            ioHandle->mode
                            );
            }
                else
                {
                    if (NULL !=  ioHandle->cbFxn)
                    {
                        (*ioHandle->cbFxn)(
                                (Ptr)ioHandle->deviceHandle->appHandle,
                                (Ptr)ioHandle->cbArg,
                                &pspRequest,
                                ioHandle->mode
                                );
                    }
                }
            }
        }

        /*Empty the pendList queue. Common for DMA/Interrupt Transactions*/
        while (TRUE != PAL_osList_NP_IsEmpty(&ioHandle->queueReqList))
        {

            tmpPacket = (DDC_mcaspIOP *)PAL_osList_NP_Dequeue(&ioHandle->queueReqList);

            /* Enque the packet back to free queue */
            PAL_osList_NP_Enqueue(&(ioHandle->queueFreeList), tmpPacket);
            /*
             * if FLUSH cmd called for INPUT channel then
             * status is equal to PSP_MCASP_FLUSHED otherwise
             * status is equal to PSP_MCASP_ABORTED
             */
            if((TRUE == ioHandle->flush) \
                    && (PSP_MCASP_CHAN_RCV == ioHandle->channelOpMode))
            {
                tmpPacket->status = PSP_MCASP_FLUSHED;
            }
            else
            {
            tmpPacket->status = PSP_MCASP_ABORTED;
            }

            tmpPacket->actualSize = 0;

            tmpPacket->size = tmpPacket->size * ioHandle->noOfSerAllocated;

            ddc_copyIOPtoReq(tmpPacket, (PSP_mcaspReq *)&pspRequest);

            if(NULL != pspRequest.cbFxn)
            {
                (*pspRequest.cbFxn)(
                        (Ptr)ioHandle->deviceHandle->appHandle,
                        (Ptr)ioHandle->cbArg,
                        &pspRequest,
                        ioHandle->mode
                        );
            }
            else
            {
                if (NULL !=  ioHandle->cbFxn)
                {
                    (*ioHandle->cbFxn)(
                            (Ptr)ioHandle->deviceHandle->appHandle,
                            (Ptr)ioHandle->cbArg,
                            &pspRequest,
                            ioHandle->mode
                            );
                }
            /*else
                {
                 callback function is null. Somewhere the function
                 is corrupted. Break from this location
            }*/

            }

            /* decrement the submit count as request is aborted */
            ioHandle->submitCount--;
        }

        /* As EDMA is disabled it might have thrown an error and set error bit.
         * Clear the error bit to enable the transfer again
         */

        EDMA3_DRV_clearErrorBits(
                        ioHandle->edmaHandle,
                        ioHandle->xferChan
                        );


        /* Configure the EDMA channel and EDMA param tables with intialization
         * configuration as they are configured at the create time.
         */

        ioHandle->nextLinkParamSetToBeUpdated = 0;
        ioHandle->loopjobUpdatedinParamset = TRUE;

        ddc_edmaChannelPaRAMsetup(ioHandle);

        /* Enable EDMA transfer, state machine and fram sync*/
        EDMA3_DRV_enableTransfer (
                        ioHandle->edmaHandle,
                        ioHandle->xferChan,
                        EDMA3_DRV_TRIG_MODE_EVENT
                        );

        if (PSP_MCASP_CHAN_RCV == ioHandle->channelOpMode)
        {
            /* Reset state m/c and frame sync of receive section*/
            ddc_activateSmFsForRcv (deviceHandle);
        }
        else
        {
            /* Reset state m/c and frame sync of transmit section*/
            ddc_activateSmFsForXmt (deviceHandle);
        }

        /*
        * CRITICAL SECTION: ends!
        */
        PAL_osProtectExit(PAL_OSPROTECT_INTERRUPT, imask);

    }while(falsewhile);
}


/**
 *  \brief    ddc_rcvMcaspIsr
 *
 *  This ISR gets control during Receive interrupt.
 *  Does the following:
 *      Disables RXSTAT interrupt flag
 *      Checks request error status
 *      Receives data from peripharal
 *      call the callback if request reading is completed
 *      and make ready for the next request
 *      In this process it also handles flush/abort request on the que
 *
 *  Returns the status of request
 *      PSP_MCASP_ABORTED,   if request is aborted
 *      IOM_PENDING,   if request is not fully processed
 *
 *  \param    ioHandle         [IN]    channel Handler
 *
 *  \return   void
 */

static void ddc_rcvMcaspIsr(DDC_mcaspIoHandle ioHandle)
{
    DDC_mcaspDeviceHandle  deviceHandle         = NULL;
    Uint32                 dlbMode              = 0;
    Bool                   falsewhile           = FALSE;
    PSP_Result             status               = PSP_SOK;



    do
    {
        falsewhile = FALSE;

        if(NULL == ioHandle)
        {
            break;
        }
        else
        {
            if(NULL == ioHandle->deviceHandle)
            {
                break;
            }
        }

        deviceHandle = ioHandle->deviceHandle;

        ioHandle->intStatus = (Uint32) deviceHandle->hwInfo.regs->RSTAT;
        /*Interrupt status word to be written back to reset the status bits*/
        deviceHandle->hwInfo.regs->RSTAT = ioHandle->intStatus;

        /*validates if the current packet is valid*/
        if (PSP_MCASP_VALID != ddc_isValidPacket(ioHandle))
        {
            /*
            spurious interrupt or xmt statemachine running without any datapacket
            For interrupt mode this driver is not having loopjob buffer implementation
            and hence in this case the statemachine should be stopped
            */
            dlbMode = (Uint32) CSL_FEXT(deviceHandle->hwInfo.regs->DLBCTL, MCASP_DLBCTL_DLBEN);

            /*
            While aborting care should be taken about DLB mode otherwise aborting
            packets in only one direction will make the packet in other direction
            not to return
            */
            if( CSL_MCASP_DLBCTL_DLBEN_ENABLE == dlbMode)
            {

                /*
                rcv packet comes to end before xmt does
                */
                ddc_abortReset((DDC_mcaspIoHandle)&(deviceHandle->txIo));
                ddc_abortReset((DDC_mcaspIoHandle)&(deviceHandle->rxIo));

            }
            else
            {
                ddc_abortReset(ioHandle);
            }

            break;
        }


        /*
        ddc_isValidPacket ensures that ioHandle->dataPacket has a valid packet
        */
        status              = PSP_SOK;

        /*
        * check for errors
        */
        do
        {
            falsewhile = FALSE;

            if (((PSP_MCASP_RCV_OVERRUN) == (PSP_MCASP_RCV_OVERRUN & ioHandle->intStatus)))
            {
                ioHandle->dataPacket->status = PSP_MCASP_RCV_OVERRUN_ERROR;
                break;
            }

            if (((PSP_MCASP_SYNCERR) == (PSP_MCASP_SYNCERR & ioHandle->intStatus)))
            {
                status = PSP_MCASP_SYNC_ERROR;
                break;
            }


        }while(falsewhile);


        if (PSP_SOK == status)
        {
            if (PSP_MCASP_DATA_RDY ==(ioHandle->intStatus & PSP_MCASP_DATA_RDY))
            {
                /*
                * Interrupt based transaction for Receive mode
                * Handle abort, flush and queue empty cases.
                *
                */
                status =  ddc_rcvIsrProcessPkt (ioHandle, ioHandle->dataPacket);


                if (PSP_SOK == status)
                {

                    if (0u != ioHandle->currentDataSize)
                    {
                        /*
                        * all the data bytes had not been xferred
                        * some more interrupts are still needed
                        */
                        break;
                    }

                    /*
                    *  if control comes here - all words are transferrred
                    * If all words are Read from McAsp then call callback function
                    */
                    ioHandle->dataPacket->status        = PSP_SOK;
                    ioHandle->dataPacket->actualSize    = (Uint32) (ioHandle->dataPacket->size - ioHandle->currentDataSize);
                    ioHandle->submitCount--;

                    if ( 0 == ioHandle->submitCount )
                    {
                        ioHandle->xferinProgressIntmode = 0;
                    }
                    ddc_completeCurrentIO(ioHandle);



                } /* Data packet status is PSP_SOK if loop */

            }/* Int status is DATA READY if loop */

        }
        else
        {

            /*
            * If ERROR .. call callback function and abort
            */
            ioHandle->dataPacket->actualSize = (Uint32) (ioHandle->dataPacket->size - ioHandle->currentDataSize);
            ioHandle->dataPacket->status  = status;
            ddc_cancelAndAbortAllIo(ioHandle);

        }



        /*
        * Stop state m/c and frame sync of receive section...
        * if requested by application
        */
        if( (TRUE == deviceHandle->stopSmFsRcv) && (0 == ioHandle->currentDataSize) )
        {
            deviceHandle->hwInfo.regs->RINTCTL = 0;
            deviceHandle->stopSmFsRcv           = FALSE;

            status = LLC_mcaspResetSmFsRcv (&(deviceHandle->hwInfo));

            if (PSP_SOK != status)
            {
                break;
            }
            deviceHandle->stopSmFsXmt = TRUE;
        }

    }while(falsewhile);

    return;
}


/**
 *  \brief ddc_xmtMcaspIsr
 *
 *  This ISR gets control during Transmit interrupt.
 *  This ISR does the following
 *      Disables TXSTAT interrupt flag
 *      Checks packet error status
 *      Transmits data to peripharal
 *      call the callback if request writing is completed
 *      and make ready for the next request
 *      In this process it also handles flush/abort request on the que
 *
 * Callback returns the status of request
 *         PSP_MCASP_ABORTED,   if request is aborted
 *         IOM_PENDING,   if request is not fully processed
 *
 *  \param    ioHandle        [IN]    Pointer to channel
 *
 *  \return   void
 *
 */
static void  ddc_xmtMcaspIsr( DDC_mcaspIoHandle ioHandle)
{

    DDC_mcaspDeviceHandle   deviceHandle    = NULL;
    Uint32                  dlbMode         = 0;
    Bool                    falsewhile      = FALSE;
    PAL_Result                result            = PSP_SOK;

    do
    {
        falsewhile = FALSE;

        if(NULL == ioHandle)
        {
            break;
        }
        else
        {
            if(NULL == ioHandle->deviceHandle)
            {
                break;
            }
        }

        deviceHandle = ioHandle->deviceHandle;

        ioHandle->intStatus = (Uint32) deviceHandle->hwInfo.regs->XSTAT;
        /*Interrupt status word to be written back to reset the status bits*/
        deviceHandle->hwInfo.regs->XSTAT = ioHandle->intStatus;

        /*
        * Stop state m/c and frame sync of xmit section...
        * if requested by application
        */
        if ((TRUE == deviceHandle->stopSmFsXmt) && (0 == ioHandle->currentDataSize) )
        {
            if (((PSP_MCASP_XMT_UNDERRUN) == (PSP_MCASP_XMT_UNDERRUN & ioHandle->intStatus)))
            {

                /*
                we have a underrun check here because some times when we get control at
                this point the bits are still being shifted from shift register into
                serialisers and hence stopping the statemachine will stop the data
                in middle - xfer will be incomplete - so we ensure not to send any more data
                from any new packet
                and wait for underrun and then stop statemachine
                */
                deviceHandle->hwInfo.regs->XINTCTL = 0;
                deviceHandle->stopSmFsXmt = FALSE;

                result = LLC_mcaspResetSmFsXmt (&(deviceHandle->hwInfo));

                if (PSP_SOK != result)
                {
                    break;
                }
            }

            /*do nothing - refer the previous comment*/
            break;
        }


        /*validates if the current packet is valid*/
        if (PSP_MCASP_VALID != ddc_isValidPacket(ioHandle))
        {
            /*
            spurious interrupt or xmt statemachine running without any datapacket
            For interrupt mode this driver is not having loopjob buffer implementation
            and hence in this case the statemachine should be stopped
            */
            dlbMode = (Uint32) CSL_FEXT(deviceHandle->hwInfo.regs->DLBCTL, MCASP_DLBCTL_DLBEN);


            /*
            While aborting care should be taken about DLB mode otherwise aborting
            packets in only one direction will make the packet in other direction
            not to return
            */
            if(CSL_MCASP_DLBCTL_DLBEN_ENABLE == dlbMode)
            {

                deviceHandle->stopSmFsXmt = TRUE;
                /*
                When we are in DLB mode , at the end of xmt if we stop the
                xmt statemachine  and rcv statemachine, some bits are in que
                and due for receive also cease.
                So we set a flag so that this stops the rcv statemachine at the
                end of packet
                */
                deviceHandle->stopSmFsRcv = TRUE;

            }
            else
            {
                ddc_abortReset(ioHandle);
            }
            break;
        }

        /*
        ddc_isValidPacket ensures that ioHandle->dataPacket has a valid packet
        */
        ioHandle->dataPacket->status = PSP_SOK;

        /*
        * check for errors
        */

        do
        {
            falsewhile = FALSE;

            if (((PSP_MCASP_XMT_UNDERRUN) == (PSP_MCASP_XMT_UNDERRUN & ioHandle->intStatus)))
            {
                ioHandle->dataPacket->status = PSP_MCASP_XMT_UNDERRUN_ERROR;
                break;
            }

            if (((PSP_MCASP_SYNCERR) == (PSP_MCASP_SYNCERR & ioHandle->intStatus)))
            {
                ioHandle->dataPacket->status = PSP_MCASP_SYNC_ERROR;
                break;
            }


        }while(falsewhile);


        if(PSP_SOK == ioHandle->dataPacket->status)
        {

            if (PSP_MCASP_DATA_RDY ==(ioHandle->intStatus & PSP_MCASP_DATA_RDY))
            {
                /*
                * Interrupt based transaction for Receive mode
                * Handle abort, flush and queue empty cases.
                *
                */
                ioHandle->dataPacket->status = ddc_xmtIsrProcessPkt (ioHandle, ioHandle->dataPacket);

            }

            /*
            * now that we have completed writing the data out into the
            * serialisers -we have to handle error / completion of IOP
            * i.e. all the databytes requested had been xferred
            */
            if(PSP_SOK == ioHandle->dataPacket->status)
            {

                if (0u != ioHandle->currentDataSize)
                {

                    /*
                    * all the data bytes had not been xferred
                    * some more interrupts are still needed
                    */
                    break;

                }


                /*
                * If all words are written to McAsp then call callback function
                */

                ioHandle->dataPacket->status = PSP_SOK;
                ioHandle->dataPacket->actualSize = (Uint32) (ioHandle->dataPacket->size - ioHandle->currentDataSize);
                ioHandle->submitCount--;
                if ( 0 == ioHandle->submitCount )
                {
                    ioHandle->xferinProgressIntmode = 0;
                }
                ddc_completeCurrentIO (ioHandle);

                /*
                * current packet is done and next packet will be
                * taken up in 'ddc_isValidPacket' function
                * when we get the interrupt again
                */

            }

        }
        else
        {

            /*
            * If ERROR .. call callback function and abort
            */
            ioHandle->dataPacket->actualSize =
                (Uint32) (ioHandle->dataPacket->size - ioHandle->currentDataSize);

            ddc_cancelAndAbortAllIo (ioHandle);

        }

        /*
        * if flush command is sent by application , rx packets will be immediately
        * cleaned up. But tx packets will not be aborted. As the flush is a sync command
        * the control will be released back to the function only after
        *  all tx packets are run to completion
        */
        if(TRUE == PAL_osList_NP_IsEmpty(&(ioHandle->queueReqList)))
        {
            /* if flush command is true */
            if (TRUE == ioHandle->flush)
            {
                /* Give semaphore which was taken at the time of flush */
                PAL_osSemGive(ioHandle->syncSem);
            }
        }

    }while(falsewhile);

    return;
}

/**
 *  \brief ddc_xmtIsrProcessPkt
 *
 *       Does the write requested by the packet. Also writes into
 *       DIT specific registers for DIT mode.
 *
 *  \param    ioPacket    [IN]    Pointer to channel
 *  \param    request      [IN]    Pointer to request to be processed
 *
 *  \return   IOM_COMPLETED, if the address is set correctly
 *            PSP_MCASP_ABORTED,   otherwise
 */

static PAL_Result ddc_xmtIsrProcessPkt (Ptr iohandlep, DDC_mcaspIOP *ioPacket)
{

    DDC_mcaspIoHandle       ioHandle            = NULL;
    DDC_mcaspDeviceHandle   deviceHandle        = NULL;
    Uint32                  i                   = 0;
    Bool                    falsewhile          = FALSE;
    PAL_Result                retVal                = PSP_SOK;

    if( (NULL != ioPacket) &&  (NULL != iohandlep))
    {
        ioHandle = (DDC_mcaspIoHandle)iohandlep;

        if(NULL != ioHandle->deviceHandle)
        {
            deviceHandle = ioHandle->deviceHandle;

            /*
            now that all our parameter/pointer validation is done
            we proceed for actual interrupt servicing
            */


            /*
            First comes handling of soft mute - mute command sent by application
            if mute is on we will just send data from mutebuf (Uint32) for
            for whatever length the xferwas requested
            */
            if( TRUE == ioHandle->bMuteON)
            {
                /*
                * Write this to all selected XRBUFs as transmit mode
                */
                for( i=0; i < ioHandle->noOfSerAllocated ; i++)
                {
                    if (PSP_MCASP_SER_XMT == (deviceHandle->serStatus[ ioHandle->indexOfSersRequested[i]]))
                    {
                        if(0 != ioHandle->currentDataSize)
                        {

                            /*
                            for the following call the buffer address is
                            starting address of mutebuf.scractbuffer[nth serialiser].
                            This address is cast to Uint8* for the api requirement
                            */
                            if (PSP_SOK != LLC_mcaspWrite(
                                                &(deviceHandle->hwInfo),
                                                ioHandle->indexOfSersRequested[i],
                                                ioHandle->roundedWordWidth,
                                                ((Uint8*)&(muteBuf[(deviceHandle->InstNum)].scratchBuffer[i])))
                                                )
                            {
                                ioPacket->status = PSP_MCASP_ABORTED;
                                break;
                            }


                            ioHandle->currentDataSize -= (ioHandle->roundedWordWidth);
                            /*no change in address*/
                        }
                    }

                }

            }/*if mute is ON*/
            else
            {
                /*
                *if mute is not enabled we send actual data sent by application
                */
                do
                {
                    falsewhile = FALSE;

                    if ((PSP_MCASP_CHAN_XMT_DIT == ioHandle->channelOpMode) &&
                          (TRUE == ioPacket->writeDitParams))
                    {

                        /*
                        * DIT Mode:
                        * we have an extra responsiblity of updating/writing User ram
                        * Write to the Channel Status RAM and User Data RAM.
                        */
                        retVal = LLC_mcaspWriteChanStatRam (
                                                    &(deviceHandle->hwInfo),
                                                    (PSP_McaspChStatusRam *)ioPacket->chStat
                                                    );

                        if (retVal != PSP_SOK)
                        {
                            ioPacket->status = PSP_MCASP_ABORTED;
                            break;
                        }

                        retVal = LLC_mcaspWriteUserDataRam (
                                                    &(deviceHandle->hwInfo),
                                                    (PSP_McaspUserDataRam *)ioPacket->userData
                                                    );

                        if (retVal != PSP_SOK)
                        {
                            ioPacket->status = PSP_MCASP_ABORTED;
                            break;
                        }

                        ioPacket->writeDitParams = FALSE;
                    }

                    /*
                    *now the actual payload data to serialisers
                    * Write this to all selected XRBUFs as transmit mode
                    */

                    ioPacket->status = PSP_SOK;

                    if(0 != ioHandle->currentDataSize)
                    {

                        for( i=0; i < ioHandle->noOfSerAllocated ; i++)
                        {
                            /*
                            * Get serializer number requested by user
                            */

                            if ((deviceHandle->serStatus[ ioHandle->indexOfSersRequested[i]]) == PSP_MCASP_SER_XMT)
                            {


                                if (PSP_SOK != LLC_mcaspWrite(
                                                        &(deviceHandle->hwInfo),
                                                        ioHandle->indexOfSersRequested[i],
                                                        (Uint16)(ioHandle->roundedWordWidth<<3),
                                                        (ioPacket->addr + (i*ioHandle->userDataBufferSize))
                                                        ))
                                {
                                    ioPacket->status = PSP_MCASP_ABORTED;
                                    break;
                                }

                            }

                        }

                    }

                    if( PSP_MCASP_ABORTED != ioPacket->status)
                    {
                        /*increment address and decrement bytes yet to be sent*/
                        ioPacket->addr = (Uint8 *)((ioPacket->addr) + (ioHandle->roundedWordWidth));
                        ioHandle->currentDataSize = \
                                (ioHandle->currentDataSize - (ioHandle->roundedWordWidth));
                    }

                }while(falsewhile);

            }/*In mute or no mute*/

        }/* Device Handle is not NULL loop */

    }/* Request and IO handle are non NULL loop */

    return (ioPacket->status);
}


/**
 *  \brief ddc_rcvIsrProcessPkt
 *
 *       Does the read requested by the request. Also writes into
 *       DIT specific registers for DIT mode.
 *
 *  \param    iohandlep        [IN]    Pointer to channel
 *  \param    ioPacket          [IN]    Pointer to request to be processed
 *
 *  \return   IOM_COMPLETED, if the address is set correctly
 *            PSP_MCASP_ABORTED,   otherwise
 *        IOM_PENDING    if need to transfer more data
 */

static PAL_Result ddc_rcvIsrProcessPkt (Ptr iohandlep, DDC_mcaspIOP * const ioPacket)
{

    DDC_mcaspIoHandle       ioHandle        = NULL;
    DDC_mcaspDeviceHandle   deviceHandle    = NULL;
    Uint32                  i               = 0;
    Bool                    falsewhile      = FALSE;

    do
    {
        falsewhile = FALSE;

        if((NULL == iohandlep )||(NULL == ioPacket))
        {
            break;
        }

        ioHandle = (DDC_mcaspIoHandle)iohandlep;

        if (NULL == ioHandle->deviceHandle)
        {
            break;
        }

        deviceHandle = ioHandle->deviceHandle;

        ioPacket->status = PSP_SOK;

        /*
        now that all our parameter/pointer validation is done
        we proceed for actual interrupt servicing
        Note that we will not be having mute implemenation as we had in xmit
        Also we dont have SPDIF - DOR ( sPDIF receive) as hardware does not support
        */

        if( 0 != ioHandle->currentDataSize )
        {

            for( i=0; i < ioHandle->noOfSerAllocated ; i++)
            {

                /*
                * check serializer status
                */
                if (PSP_MCASP_SER_RCV == (deviceHandle->serStatus[ ioHandle->indexOfSersRequested[i]]) )
                {


                    if (PSP_SOK != LLC_mcaspRead(
                                        &(deviceHandle->hwInfo),
                                        ioHandle->indexOfSersRequested[i],
                                        (Uint16)(ioHandle->roundedWordWidth<<(3u)),
                                        (ioPacket->addr + (i*ioHandle->userDataBufferSize))
                                        ))
                    {
                        ioPacket->status = PSP_MCASP_ABORTED;
                        break;
                    }
                }

            }
        } /* for loop */

        if( PSP_MCASP_ABORTED != ioPacket->status)
        {
            /*increment address and decrement bytes yet to be sent*/
            ioPacket->addr = (Uint8 *)((ioPacket->addr) + (ioHandle->roundedWordWidth));
            ioHandle->currentDataSize = \
                    (ioHandle->currentDataSize - (ioHandle->roundedWordWidth));
        }

    }while(falsewhile);


    return (ioPacket->status);
}


/**
 *  \brief ddc_isRequestedSerzsFree
 *
 *  Check requested serialzers are free or not
 *
 * \param     deviceHandle  [IN]     deviceHandle Handler
 * \param     chanParams    [IN]     Device parameter structure that supplies
 *                                  data to initialize the McASP
 *
 *  \return  Bool     TRUE if all serializers are free (PSP_MCASP_SER_FREE)
 *            FALSE if any one requested serializer is not free
 */

static Bool ddc_isRequestedSerzsFree ( DDC_mcaspDeviceHandle deviceHandle, Ptr chanParams)
{

    PSP_mcaspChanParams    *params      = NULL;
    Uint32                  i           = 0 ;
    Bool                    status      = TRUE;
    Bool                    falsewhile  = FALSE;
    do
    {
        falsewhile = FALSE;

        if ((NULL == deviceHandle) || (NULL == chanParams))
        {
            status = FALSE;
            break;
        }
        params = (PSP_mcaspChanParams *)chanParams;

        for( i=0; i < params->noOfSerRequested; i++)
        {
            /*
            * Get serializer number requested by user
            */
            if (PSP_MCASP_SER_FREE != (deviceHandle->serStatus[params->indexOfSersRequested[i]]))
            {
                status = FALSE;
                break;
            }

        }
        if (FALSE == status)
        {
            break;
        }
    }while(falsewhile);
    return( status);
}

/**
 *  \brief ddc_multiSerInitializationRCV
 *
 *  Configure all requested serialzers in receive mode
 *
 * \param     deviceHandle     [IN]     deviceHandle Handler
 * \param     iohandlep        [IN]    Pointer to channel
 * \return    void
 */

static void ddc_multiSerInitializationRCV ( DDC_mcaspDeviceHandle deviceHandle,   Ptr iohandlep)
{

    PSP_mcaspChanParams    *params = NULL;
    PSP_McaspSerializerNum  serializerToConfigure = (PSP_McaspSerializerNum)0;
    Uint32                  i           = 0;
    Uint32                  serNum      = 0;
    Bool                    falsewhile  = FALSE;

    do
    {
        falsewhile = FALSE;

        if ((NULL == deviceHandle) || (NULL == iohandlep))
        {
            break;
        }
        params = (PSP_mcaspChanParams *)iohandlep;

        for( i=0; i < params->noOfSerRequested; i++)
        {

            /*
            * Get serializer number requested by user
            */

            serNum = (Uint32)params->indexOfSersRequested[i] ;

            if ( PSP_MCASP_SER_FREE == deviceHandle->serStatus[serNum])
            {

                /*
                * Configure requested serialzer as receive mode
                */

                deviceHandle->serStatus[serNum]  = PSP_MCASP_SER_RCV;

                serializerToConfigure = (PSP_McaspSerializerNum) serNum;

                LLC_mcaspSetSerRcv (
                                &(deviceHandle->hwInfo),
                                (PSP_McaspSerializerNum )serializerToConfigure
                                );
            }
        }
    }while(falsewhile);
}


/**
 *  \brief ddc_multiSerInitializationXMT
 *
 *  Configure all requested serialzers as transmit mode
 *
 * \param     deviceHandle     [IN]     deviceHandle Handler
 * \param     iohandlep        [IN]    Pointer to channel
 *
 * \return    void
 */

static void ddc_multiSerInitializationXMT (DDC_mcaspDeviceHandle deviceHandle,  Ptr iohandlep)
{

    PSP_mcaspChanParams         *params     = NULL;
    PSP_McaspSerializerNum      serializerToConfigure = (PSP_McaspSerializerNum)0;
    Uint32                      i           = 0;
    Uint32                      serNum      = 0;
    Bool                        falsewhile = FALSE;


    do
    {
        falsewhile = FALSE;

        if((NULL == deviceHandle) || (NULL == iohandlep))
        {
            break;
        }

        params = (PSP_mcaspChanParams *)iohandlep;

        for( i=0; i < params->noOfSerRequested; i++)
        {

            /*
            * Get serializer number requested by user
            */

            serNum = (Uint32)params->indexOfSersRequested[i] ;

            if ( PSP_MCASP_SER_FREE == deviceHandle->serStatus[serNum])
            {

                /*
                * Configure requested serialzer as transmit mode
                */

                deviceHandle->serStatus[serNum]  = PSP_MCASP_SER_XMT;

                serializerToConfigure = (PSP_McaspSerializerNum) serNum;

                LLC_mcaspSetSerXmt (
                                &(deviceHandle->hwInfo),
                                (PSP_McaspSerializerNum) serializerToConfigure
                                );

            }
        }

    }while(falsewhile);
}

/**
 *  \brief multiSerGobalRestore
 *
 *  restore global serializer state as available for further use.
 *
 *  \param     iohandlep        [IN]   channel Handler
 *
 *  \return   void
 */

static void ddc_multiSerGlobalRestore ( Ptr iohandlep )
{

    DDC_mcaspIoHandle       ioHandle        = NULL;
    DDC_mcaspDeviceHandle   deviceHandle    = NULL;
    Uint32                  i               = 0;
    Bool                    falsewhile      = FALSE;

    do
    {
        falsewhile = FALSE;

        if (NULL == iohandlep)
        {
            break;
        }

        ioHandle = (DDC_mcaspIoHandle)iohandlep;

        if(NULL == ioHandle)
        {
            break;
        }
        else
        {
            if(NULL == ioHandle->deviceHandle)
            {
                break;
            }
        }

        deviceHandle = ioHandle->deviceHandle;

        for( i=0; i < ioHandle->noOfSerAllocated; i++)
        {
            deviceHandle->serStatus[ioHandle->indexOfSersRequested[i]]  = PSP_MCASP_SER_FREE;
        }

    }while(falsewhile);

}

/**
 *  \brief ddc_isrInput
 *
 *
 *  This ISR will be called when McAsp0 RCV interrupt is triggered
 *
 *  \param    ioHandle        [IN]    channel Handler
 *
 *  \return  void
 *
 */

void ddc_isrInput (Ptr iohandlep)
{
    DDC_mcaspIoHandle       ioHandle        =  NULL;
    DDC_mcaspDeviceHandle   deviceHandle    = NULL;
    Bool                    falsewhile      = 0;
    PAL_Result              status          = PSP_SOK;
    Uint32                  rmod            = 0;
	EDMA3_DRV_PaRAMRegs     pramTbl         = {0};
	Uint32 					dlbMode			= 0;

    do
    {
        falsewhile = FALSE;


        /*validae parameters and pointers*/
        if (NULL == iohandlep)
        {
            break;
        }

        ioHandle    =   (DDC_mcaspIoHandle)iohandlep;

        if(NULL == ioHandle)
        {
            break;
        }

        if(NULL == ioHandle->deviceHandle)
        {
            break;
        }

        deviceHandle = ioHandle->deviceHandle;


        /*Interrupt handling starts*/

        if(FALSE == ioHandle->isDmaDriven)
        {
            /*The driver is in interrupt mode and all the interrupts have to be handled accordinglY*/
             ddc_rcvMcaspIsr(ioHandle);
        }
        else
        {
            /*The driver is in DMA mode and it is enough if we handle only errors*/

            status = PSP_SOK;

            /* Read interrupt-status register.*/
            ioHandle->intStatus = (Uint32)deviceHandle->hwInfo.regs->RSTAT;

            /*Interrupt status word to be written back to reset the status bits*/
            deviceHandle->hwInfo.regs->RSTAT = ioHandle->intStatus;

            /*
            for bursmode of operation frame sync error has to ignored
            to indentify that we do the following
            */
            rmod = (Uint32)(deviceHandle->hwInfo.regs->AFSRCTL & CSL_MCASP_AFSRCTL_RMOD_MASK) ;

            do
            {
                falsewhile = FALSE;

                if (((PSP_MCASP_DMAERR) == (PSP_MCASP_DMAERR & ioHandle->intStatus)))
                {
                    status = PSP_MCASP_DMA_ERROR;
                    break;
                }

                if (((PSP_MCASP_RCV_OVERRUN) == (PSP_MCASP_RCV_OVERRUN & ioHandle->intStatus)))
                {
                    status = PSP_MCASP_RCV_OVERRUN_ERROR;
                    break;
                }

                /*no sync error check for burst mode*/
                if(0 != rmod)
                {
                    if (((PSP_MCASP_SYNCERR) == (PSP_MCASP_SYNCERR & ioHandle->intStatus)))
                    {
                        status = PSP_MCASP_SYNC_ERROR;
                        break;
                    }
                }

            }while(falsewhile);


            if(PSP_SOK != status)
            {

                /*
                * call the application registered callback if any
                */
                if(NULL != (ioHandle->gblErrCbk))
                {
                    (ioHandle->gblErrCbk)(status);
                }




                /*
                * For sync error we call the application callback
                * and we abort all other pending IOPs also
                */
                if(PSP_MCASP_SYNC_ERROR == status)
                {

                    /*
			        While aborting care should be taken about DLB mode otherwise aborting
			        packets in only one direction will make the packet in other direction
			        not to return
			        */
			        dlbMode = (Uint32) CSL_FEXT(ioHandle->deviceHandle->hwInfo.regs->DLBCTL, MCASP_DLBCTL_DLBEN);

			        if( CSL_MCASP_DLBCTL_DLBEN_ENABLE == dlbMode)
			        {
			            ddc_abortReset((DDC_mcaspIoHandle)&(ioHandle->deviceHandle->rxIo));
			            ddc_abortReset((DDC_mcaspIoHandle)&(ioHandle->deviceHandle->txIo));
			        }
			        else
			        {
			            ddc_abortReset(ioHandle);
			        }

                    break;
                }



                /*
                if the error is a non- framesync error - we just send error status for the current packet
                in the callback function and continue with other packets for xfer
                */

                if(PSP_MCASP_SYNC_ERROR != status)
                {

					/*
	                 * Disable Serializer, State machine and Frame sync of RCV Section.
	                 * Also Disable EDMA Transfer.
	                 */
	                ddc_smReset(ioHandle);


					/*
					Initiate dummy transfer (or reduce the c-count to 1) in EDMA-current transfer channel param
					This would load the next link paramset and give a edma callback where we
					do futher steps needed to close the current packet and load next
					pending packet etc
					*/

					/* get the param table information of transfer channel */
					EDMA3_DRV_getPaRAM(ioHandle->edmaHandle,ioHandle->xferChan,&pramTbl);

					if(0 != pramTbl.cCnt)
					{
			    		pramTbl.cCnt = 0;

						EDMA3_DRV_setPaRAM(ioHandle->edmaHandle,ioHandle->xferChan,&pramTbl);
					}



                    ioHandle->currentPacketErrorStatus = status;

                    /*
                     * Enable Serializer, State machine and Frame sync of RCV Section.
                     * Also Enable EDMA Transfer.
                     */
                    ddc_smSet(ioHandle);

                }/*for errors except sync errors*/

            }/*dma transfer for all errors*/

        } /* dma driven transactions*/

    }while(falsewhile);
}



/**
 *  \brief ddc_isrOutput
 *
 *
 * This ISR will be called when McAsp0 XMT interrupt is triggered
 *
 *  \param    ioHandle        [IN]    channel Handler
 *
 *  \return  void
 *
 */

void ddc_isrOutput(Ptr iohandlep)
{
    DDC_mcaspIoHandle       ioHandle        = NULL;
    DDC_mcaspDeviceHandle   deviceHandle    = NULL;
    Bool                    falsewhile      = FALSE;
    PAL_Result              status          = PSP_SOK;
    Uint32                  xmod            = 0;
	Uint32 					dlbMode			= 0;
	EDMA3_DRV_PaRAMRegs     pramTbl         = {0};


    do
    {
        falsewhile = FALSE;

        /*validae parameters and pointers*/
        if (NULL == iohandlep)
        {
            break;
        }

        ioHandle = (DDC_mcaspIoHandle)iohandlep;
        if(NULL == ioHandle)
        {
            break;
        }

        if(NULL == ioHandle->deviceHandle)
        {
            break;
        }

        deviceHandle = ioHandle->deviceHandle;

        /*Interrupt handling starts*/

        if(FALSE == ioHandle->isDmaDriven )
        {
            /*The driver is in interrupt mode and all the interrupts have to be handled accordinglY*/
            ddc_xmtMcaspIsr(ioHandle);
        }
        else
        {
            /*The driver is in DMA mode and it is enough if we handle only errors*/

            status = PSP_SOK;

            /* Read interrupt-status register.*/
            ioHandle->intStatus = (Uint32)deviceHandle->hwInfo.regs->XSTAT;
            /*Interrupt status word to be written back to reset the status bits*/
            deviceHandle->hwInfo.regs->XSTAT = ioHandle->intStatus;


            /*
            for bursmode of operation frame sync error has to ignored
            to indentify that we do the following
            */
            xmod = (Uint32)(deviceHandle->hwInfo.regs->AFSXCTL & CSL_MCASP_AFSXCTL_XMOD_MASK) ;

            do
            {
                falsewhile = FALSE;

                if (((PSP_MCASP_DMAERR) == (PSP_MCASP_DMAERR & ioHandle->intStatus)))
                {
                    status = PSP_MCASP_DMA_ERROR;
                    break;
                }

                if (((PSP_MCASP_XMT_UNDERRUN) == (PSP_MCASP_XMT_UNDERRUN & ioHandle->intStatus)))
                {
                    status = PSP_MCASP_XMT_UNDERRUN_ERROR;
                    break;
                }

                /*no sync error check for burst mode*/
                if(0 != xmod)
                {
                    if (((PSP_MCASP_SYNCERR) == (PSP_MCASP_SYNCERR & ioHandle->intStatus)))
                    {
                        status = PSP_MCASP_SYNC_ERROR;
                        break;
                    }
                }
            }while(falsewhile);

            if(PSP_SOK != status)
            {

                /*
                * call the application registered callback if any
                */
                if(NULL != (ioHandle->gblErrCbk))
                {
                    (ioHandle->gblErrCbk)(status);
                }

                /*
                * at this point we will decide whether to continue other pending
                *IOPs  or abort - it depends on error
                */


                /*
                * For sync error we call the application callback
                * and we abort all other pending IOPs also
                */
                if(PSP_MCASP_SYNC_ERROR == status)
                {
                    /*
			        While aborting care should be taken about DLB mode otherwise aborting
			        packets in only one direction will make the packet in other direction
			        not to return
			        */
			        dlbMode = (Uint32) CSL_FEXT(ioHandle->deviceHandle->hwInfo.regs->DLBCTL, MCASP_DLBCTL_DLBEN);

			        if( CSL_MCASP_DLBCTL_DLBEN_ENABLE == dlbMode)
			        {
			            ddc_abortReset((DDC_mcaspIoHandle)&(ioHandle->deviceHandle->rxIo));
			            ddc_abortReset((DDC_mcaspIoHandle)&(ioHandle->deviceHandle->txIo));
			        }
			        else
			        {
			            ddc_abortReset(ioHandle);
			        }
                    break;
                }


                /*
                if the error is a non- framesync error - we just send error status for the current packet
                in the callback function and continue with other packets for xfer
                */

                if(PSP_MCASP_SYNC_ERROR != status)
                {

					/*
	                 * Disable Serializer, State machine and Frame sync of RCV Section.
	                 * Also Disable EDMA Transfer.
	                 */
	                ddc_smReset(ioHandle);


					/*
					Initiate dummy transfer (or reduce the c-count to 1) in EDMA-current transfer channel param
					This would load the next link paramset and give a edma callback where we
					do futher steps needed to close the current packet and load next
					pending packet etc
					*/

					/* get the param table information of transfer channel */
					EDMA3_DRV_getPaRAM(ioHandle->edmaHandle,ioHandle->xferChan,&pramTbl);

					if(0 != pramTbl.cCnt)
					{
			    		pramTbl.cCnt = 0;

						EDMA3_DRV_setPaRAM(ioHandle->edmaHandle,ioHandle->xferChan,&pramTbl);
					}



                    ioHandle->currentPacketErrorStatus = status;

                    /*
                     * Enable Serializer, State machine and Frame sync of RCV Section.
                     * Also Enable EDMA Transfer.
                     */
                    ddc_smSet(ioHandle);
                }/*for errors except sync errors*/

            }/*dma transfer for all errors*/

        }/*else dma driven transaction*/

    }while(falsewhile);
}

/**
 *  \brief ConfigureRCViohandlenel
 *
 *  configure Mcasp to receive mode
 *
 *  \param    deviceHandle     [IN]    device  Handler
 *  \param    rcvParams        [IN]    receive channel parameters
 *  \return  status        [OUT]   CSL_OK If configuration is sucess
 *                   0 if failed
 */

static PAL_Result ddc_ConfigureRCVchannel( DDC_mcaspDeviceHandle deviceHandle, Ptr rcvParams)
{

    PSP_mcaspChanParams    *params      = NULL;
    PAL_Result              status      = PSP_SOK ;
    Bool                    falsewhile  = FALSE;
    do
    {
        falsewhile = FALSE;

        if ((NULL == deviceHandle) || (NULL == rcvParams))
        {
            status = PSP_E_INVAL_PARAM;
            break;
        }
        params   = (PSP_mcaspChanParams *)rcvParams;

        /*
        * before touching McASP registers, place receive section in reset
        */
        status = LLC_mcaspResetRcv (&(deviceHandle->hwInfo));

        if (PSP_SOK != status)
        {
            status = PSP_E_INVAL_PARAM;
            break;
        }
        /*
        * Configure the receive section of the device:
        * start Receive high clock and clock
        */
        status = LLC_mcaspConfigRcvSection (
                                     &(deviceHandle->hwInfo),
                                    (PSP_McaspHwSetupData *)params->mcaspSetup
                                    );

        if (PSP_SOK != status)
        {
            status = PSP_E_INVAL_PARAM;
            break;
        }


    }while(falsewhile);

    return (status);

}

/**
 *  \brief ddc_ConfigureXMTChannel
 *
 *  configure Mcasp to transmit mode
 *
 *  \param    deviceHandle  [IN]    device  Handler
 *  \param    xmtParams     [IN]    transmit channel parameters
 *
 *  \return  status        [OUT]   CSL_OK If configuration is sucess
 *                   0 if failed
 */

static PAL_Result ddc_ConfigureXMTChannel( DDC_mcaspDeviceHandle deviceHandle, Ptr xmtParams)
{

    PSP_mcaspChanParams     *params     = NULL;
    PAL_Result              status      = PSP_SOK;
    Bool                    falsewhile  = FALSE;
    do
    {
        falsewhile = FALSE;

        if((NULL == deviceHandle) || (NULL==xmtParams))
        {
            status = PSP_E_INVAL_PARAM;
            break;
        }
        params  = (PSP_mcaspChanParams *)xmtParams;
        /*
        * before touching McASP registers, place transmit section in reset
        */
        status = LLC_mcaspResetXmt (&(deviceHandle->hwInfo));

        if (PSP_SOK != status)
        {
            status = PSP_E_INVAL_PARAM;
            break;
        }

        /*
        * Configure the transmit section of the device:
        * Start transmit high clock and clock
        */
        status = LLC_mcaspConfigXmtSection (
                             &(deviceHandle->hwInfo),
                            (PSP_McaspHwSetupData *)params->mcaspSetup
                            );
        if (PSP_SOK != status)
        {
            status = PSP_E_INVAL_PARAM;
            break;
        }
        /*
        * Set DITEN bit if DIT mode else set to I2S mode.
        */

        if ( PSP_MCASP_DIT_MODE == params->channelMode)
        {
          /* check hardware supports dit mode or not*/

            if(TRUE == deviceHandle->hwInfo.ditStatus )
            {

                status = LLC_mcaspSetDitMode (
                                &(deviceHandle->hwInfo),
                                params->channelMode
                                );
                if (PSP_SOK != status)
                {
                    status = PSP_E_INVAL_PARAM;
                    break;
                }

            }
        }

    }while(falsewhile);

  return(status);
}

/**
 *  \brief ddc_setupEDMADuringInit
 *
 *  This function is called durinn psp_mcaspopen (during init)
 *  This function does the following
 *       1)Request the Xfer (in/out depends on mode) channel to EDMA
 *         driver for this peripheral;
 *       2)requests the link channels for  linking (exclusive for each
 *         direction (tx / rx)
 *
 *  \param    ioHandle     [IN]    channel  Handler
 *
 *  \return  status        [OUT]   PSP_SOK if is sucess
 *                                 PSP error if failed
 */

static PAL_Result ddc_setupEDMADuringInit( DDC_mcaspIoHandle ioHandle )
{

    Uint32          linkCnt     = 0;
    PAL_Result      retVal      = PSP_SOK;
    Bool            falsewhile  = FALSE;
    Uint32          i               = 0;
    Uint32          edmaChanNum     = 0;
    Uint32          reqTcc          = EDMA3_DRV_TCC_ANY;
    EDMA3_RM_EventQueue queueNum     = 0;



    do
    {
        falsewhile  = FALSE;
        if(NULL == ioHandle)
        {
            retVal = PSP_E_INVAL_PARAM;
            break;
        }

        /*
        * For receive use EDMA Queue 1 and for transmit use EDMA queue 0
        */
        if(PSP_MCASP_INPUT == ioHandle->mode)
        {
            queueNum = (EDMA3_RM_EventQueue)RXEVENTQUE;
        }
        else
        {
            queueNum = (EDMA3_RM_EventQueue)TXEVENTQUE;
        }

        if (PSP_SOK != EDMA3_DRV_requestChannel (
                                            ioHandle->edmaHandle,
                                            &ioHandle->xferChan,
                                            &ioHandle->tcc,
                                            queueNum,
                                            ioHandle->edmaCallback,
                                            ioHandle))
        {
            retVal = PSP_E_INVAL_PARAM;
            break;
        }

        /*
        *Acquire the  PaRAM entries used for EDMA transfers linking
        */
        for (linkCnt = 0; linkCnt < MAXLINKCNT; linkCnt++)
        {
            /*For requesting for a PaRam set*/
            edmaChanNum = EDMA3_DRV_LINK_CHANNEL;
            reqTcc = EDMA3_DRV_TCC_ANY;
            retVal = EDMA3_DRV_requestChannel (
                                        ioHandle->edmaHandle,
                                        &edmaChanNum,
                                        &reqTcc,
                                        queueNum,
                                        ioHandle->edmaCallback,
                                        NULL
                                        );
            if (PSP_SOK == retVal )
            {
                ioHandle->pramTbl[linkCnt] = edmaChanNum;

                retVal = EDMA3_DRV_getPaRAMPhyAddr(
                                        ioHandle->edmaHandle,
                                        ioHandle->pramTbl[linkCnt],
                                        &(ioHandle->pramTblAddr[linkCnt])
                                        );
            }
            if (PSP_SOK != retVal )
            {
                /* free the Synchonized EDMA channel */
                EDMA3_DRV_freeChannel (ioHandle->edmaHandle, ioHandle->xferChan);

                /* free the already allocated PaRAM entries */
                for (i = 0; i <linkCnt; i++)
                {
                      EDMA3_DRV_freeChannel (ioHandle->edmaHandle, ioHandle->pramTbl[i]);
                }
                retVal = PSP_E_RESOURCES;
                break;
            }
        }

        /* Configure EDMA channels */
        retVal = ddc_edmaChannelPaRAMsetup(ioHandle);

    }while(falsewhile);

    return retVal;
}

/**
 *  \brief ddc_setupEDMADuringInit
 *
 *   This function does the following
 *       1)set up the param set of main xfer channel (for loopjob transfers)
 *       2)set up the param sets of all link channel (for loopjob transfers)
 *       3)link the last link channel to itself (this channel will be linked
 *         to  main xfer channel later and to supply the loopjob xfer
 *         information infinely this is done so; other link channels are setup
 *         here just for completion sake
 *
 *  \param    ioHandle     [IN]    channel  Handler
 *
 *  \return  status        [OUT]   PSP_SOK if is sucess
 *                                 PSP error if failed
 */

static PAL_Result ddc_edmaChannelPaRAMsetup(DDC_mcaspIoHandle ioHandle)
{
    Uint32          linkCnt     = 0;
    PAL_Result      retVal      = PSP_SOK;
    Bool            falsewhile  = FALSE;

    do
    {
        falsewhile  = FALSE;
        if(NULL == ioHandle)
        {
            retVal = PSP_E_INVAL_PARAM;
            break;
        }

       /*
        * Configure the McASP synchonized EDMA channel PaRAM entries
        * Configure source and destination address / modes
        * first - main xfer channel
        */

        /*
        following the explanation behind the EDMA configuration

        1) We will have n arrays of data (now it is loopjob buffer each of length x bytes)
        2) for each sync event we have to fetch the data from each array and write to each serilaiser
        3) and the no. bytes transferred in this case from each array will be roundedwordwidth/8
        4) as the loopjob buffer is contigous, we configure the counts as follows
            a) consider there are arrays of length roundedwordwidth/8 (aCnt)
            b) these arrays are located apart lengthofloopjob size  (bindex)
            c) there are 'no.of.serialiser' no. of such arrays   (bCnt)
            d)  (a) and (b) constitute one frame of EDMA transfer and it happens for
                one sync event in AB sync mode
            e) such frames have to be sent till we have buffer - ie lengthofloopjob / (roundedwordwidth/8 )
                and this constitutes  cCnt
            f)  address of buffer (rather offset from start of previous frame) is roundedwordwidth/8
                 and this forms cIndex
        Please take some time to visualise the setup mentioned above to check with EDMa configuration

        The same kind of setup will be used for data transfers also and the change will be buffer pointer
        and length of data buffer.
        */



        if (PSP_MCASP_INPUT == ioHandle->mode )
        {

            if (PSP_SOK != EDMA3_DRV_setSrcParams(
                                        ioHandle->edmaHandle,
                                        ioHandle->xferChan,
                                        (Uint32) ioHandle->deviceHandle->hwInfo.dataAddr,
                                        EDMA3_DRV_ADDR_MODE_INCR,
                                        EDMA3_DRV_W8BIT
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }



            if (PSP_SOK != EDMA3_DRV_setDestParams(
                                        ioHandle->edmaHandle,
                                        ioHandle->xferChan,
                                        (Uint32)ioHandle->loopJobBuffer,
                                        EDMA3_DRV_ADDR_MODE_INCR,
                                        EDMA3_DRV_W8BIT
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }

            if (PSP_SOK != EDMA3_DRV_setSrcIndex(
                                    ioHandle->edmaHandle,
                                    ioHandle->xferChan,
                                    0,
                                    0
                                    ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }


            if (PSP_SOK != EDMA3_DRV_setDestIndex(
                                        ioHandle->edmaHandle,
                                        ioHandle->xferChan,
                                        ioHandle->loopJobLength,
                                        ioHandle->roundedWordWidth
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }
        }
        else
        {

            if (PSP_SOK != EDMA3_DRV_setSrcParams(
                                        ioHandle->edmaHandle,
                                        ioHandle->xferChan,
                                        (Uint32)ioHandle->loopJobBuffer,
                                        EDMA3_DRV_ADDR_MODE_INCR,
                                        EDMA3_DRV_W8BIT
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }

            if (PSP_SOK != EDMA3_DRV_setDestParams(
                                        ioHandle->edmaHandle,
                                        ioHandle->xferChan,
                                        (Uint32) ioHandle->deviceHandle->hwInfo.dataAddr,
                                        EDMA3_DRV_ADDR_MODE_INCR,
                                        EDMA3_DRV_W8BIT
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }

            if (PSP_SOK != EDMA3_DRV_setSrcIndex(
                                    ioHandle->edmaHandle,
                                    ioHandle->xferChan,
                                    ioHandle->loopJobLength,
                                    ioHandle->roundedWordWidth
                                    ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }


            if (PSP_SOK != EDMA3_DRV_setDestIndex(
                                        ioHandle->edmaHandle,
                                        ioHandle->xferChan,
                                        0,
                                        0
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }
        }




        /*
        for acnt, bcnt, ccnt, bidx , cidx details please refer comment above
        */
        if (PSP_SOK != EDMA3_DRV_setTransferParams(
                                    ioHandle->edmaHandle,
                                    ioHandle->xferChan,
                                    ioHandle->roundedWordWidth,
                                    ioHandle->noOfSerAllocated,
                                    ioHandle->loopJobLength/(ioHandle->roundedWordWidth),
                                    ioHandle->noOfSerAllocated,
                                    EDMA3_DRV_SYNC_AB
                                    ))
        {
            retVal = PSP_E_INVAL_PARAM;
            break;
        }


        if (PSP_SOK != EDMA3_DRV_setOptField(
                                    ioHandle->edmaHandle,
                                    ioHandle->xferChan,
                                    EDMA3_DRV_OPT_FIELD_TCINTEN,
                                    FALSE
                                    ))
        {
            retVal = PSP_E_INVAL_PARAM;
            break;
        }

        if (PSP_SOK != EDMA3_DRV_setOptField(
                                    ioHandle->edmaHandle,
                                    ioHandle->xferChan,
                                    EDMA3_DRV_OPT_FIELD_TCC,
                                    ioHandle->tcc
                                    ))
        {
            retVal = PSP_E_INVAL_PARAM;
            break;
        }


        /*
        * Configure the link PaRAMs with the appropriate parameters
        * Though we configure all the link paramsets, we will be using
        * only one of them to link with main xfer channel
        */
        for (linkCnt = 0; linkCnt < MAXLINKCNT; linkCnt++)
        {

            if (PSP_MCASP_INPUT == ioHandle->mode )
            {

                if (PSP_SOK != EDMA3_DRV_setSrcParams(
                                            ioHandle->edmaHandle,
                                            ioHandle->pramTbl[linkCnt],
                                            (Uint32) ioHandle->deviceHandle->hwInfo.dataAddr,
                                            EDMA3_DRV_ADDR_MODE_INCR,
                                            EDMA3_DRV_W8BIT
                                            ))
                {
                    retVal = PSP_E_INVAL_PARAM;
                    break;
                }



                if (PSP_SOK != EDMA3_DRV_setDestParams(
                                            ioHandle->edmaHandle,
                                            ioHandle->pramTbl[linkCnt],
                                            (Uint32)ioHandle->loopJobBuffer,
                                            EDMA3_DRV_ADDR_MODE_INCR,
                                            EDMA3_DRV_W8BIT
                                            ))
                {
                    retVal = PSP_E_INVAL_PARAM;
                    break;
                }

                if (PSP_SOK != EDMA3_DRV_setSrcIndex(
                                        ioHandle->edmaHandle,
                                        ioHandle->pramTbl[linkCnt],
                                        0,
                                        0
                                        ))
                {
                    retVal = PSP_E_INVAL_PARAM;
                    break;
                }


                if (PSP_SOK != EDMA3_DRV_setDestIndex(
                                            ioHandle->edmaHandle,
                                            ioHandle->pramTbl[linkCnt],
                                            ioHandle->loopJobLength,
                                            (Uint16)(ioHandle->roundedWordWidth>>3)
                                            ))
                {
                    retVal = PSP_E_INVAL_PARAM;
                    break;
                }
            }
            else
            {

                if (PSP_SOK != EDMA3_DRV_setSrcParams(
                                            ioHandle->edmaHandle,
                                            ioHandle->pramTbl[linkCnt],
                                            (Uint32)ioHandle->loopJobBuffer,
                                            EDMA3_DRV_ADDR_MODE_INCR,
                                            EDMA3_DRV_W8BIT
                                            ))
                {
                    retVal = PSP_E_INVAL_PARAM;
                    break;
                }

                if (PSP_SOK != EDMA3_DRV_setDestParams(
                                            ioHandle->edmaHandle,
                                            ioHandle->pramTbl[linkCnt],
                                            (Uint32) ioHandle->deviceHandle->hwInfo.dataAddr,
                                            EDMA3_DRV_ADDR_MODE_INCR,
                                            EDMA3_DRV_W8BIT
                                            ))
                {
                    retVal = PSP_E_INVAL_PARAM;
                    break;
                }

                if (PSP_SOK != EDMA3_DRV_setSrcIndex(
                                        ioHandle->edmaHandle,
                                        ioHandle->pramTbl[linkCnt],
                                        ioHandle->loopJobLength,
                                        ioHandle->roundedWordWidth
                                        ))
                {
                    retVal = PSP_E_INVAL_PARAM;
                    break;
                }


                if (PSP_SOK != EDMA3_DRV_setDestIndex(
                                            ioHandle->edmaHandle,
                                            ioHandle->pramTbl[linkCnt],
                                            0,
                                            0
                                            ))
                {
                    retVal = PSP_E_INVAL_PARAM;
                    break;
                }
            }




            /*
            * set a count , b count and c count
            * for each sync even we need to transfer
            *  no. of serialisers * bit width (wordlen) / 8 bits
            */
            if (PSP_SOK != EDMA3_DRV_setTransferParams(
                                        ioHandle->edmaHandle,
                                        ioHandle->pramTbl[linkCnt],
                                        ioHandle->roundedWordWidth,
                                        ioHandle->noOfSerAllocated,
                                        ioHandle->loopJobLength/(ioHandle->roundedWordWidth),
                                        ioHandle->noOfSerAllocated,
                                        EDMA3_DRV_SYNC_AB
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }


            if (PSP_SOK != EDMA3_DRV_setOptField(
                                        ioHandle->edmaHandle,
                                        ioHandle->pramTbl[linkCnt],
                                        EDMA3_DRV_OPT_FIELD_TCINTEN,
                                        FALSE
                                        ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }



            if (PSP_SOK != EDMA3_DRV_setOptField(
                                    ioHandle->edmaHandle,
                                    ioHandle->pramTbl[linkCnt],
                                    EDMA3_DRV_OPT_FIELD_TCC,
                                    ioHandle->tcc
                                    ))
            {
                retVal = PSP_E_INVAL_PARAM;
                break;
            }

            if(MAXLINKCNT-1 > linkCnt )
            {
                /*
                *link with next paramset
                *both param sets are  linked to each other to enable continous
                * xfer - either user audio data or loopjob buffer
                * MAXLINKCNT must be 2
                */

                if (PSP_SOK != EDMA3_DRV_linkChannel(
                                ioHandle->edmaHandle,
                                ioHandle->xferChan,
                                ioHandle->pramTbl[linkCnt+1]
                                ))
                {
                    retVal = PSP_E_INVAL_PARAM;
                    break;
                }

                if (PSP_SOK != EDMA3_DRV_linkChannel(
                                ioHandle->edmaHandle,
                                ioHandle->pramTbl[linkCnt] ,
                                ioHandle->pramTbl[linkCnt+1]
                                ))
                {
                    retVal = PSP_E_INVAL_PARAM;
                    break;
                }

                if (PSP_SOK != EDMA3_DRV_linkChannel(
                                ioHandle->edmaHandle,
                                ioHandle->pramTbl[linkCnt+1] ,
                                ioHandle->pramTbl[linkCnt+1]
                                ))
                {
                    retVal = PSP_E_INVAL_PARAM;
                    break;
                }

                ioHandle->nextLinkParamSetToBeUpdated = 0;
                ioHandle->loopjobUpdatedinParamset  = TRUE;

            }

        }/*for loop for linkcount*/
    }while(falsewhile);

    return retVal;
}

/**
 *  \brief ddc_completeCurrentIO
 *
 *  Call the DDA layer callback and enqueue the packet back to free queue
 *
 *  \param    ioHandle             [IN]    channel handler
 *  \return   void
 */
static void ddc_completeCurrentIO (DDC_mcaspIoHandle ioHandle )
{

    PSP_mcaspReq        pspRequest  ={0};
    DDC_mcaspIOP        *packet     = NULL;

    if(NULL != ioHandle)
    {

        if ( TRUE == ioHandle->isTempPacketValid )
        {
            packet = ioHandle->tempPacket;
            ioHandle->isTempPacketValid = FALSE;
        }
        else
        {
            packet = ioHandle->dataPacket;
        }

        /*
         * if FLUSH cmd called for OUTPUT channel then
         * set packet status is equal to PSP_MCASP_FLUSHED
         */
		if((TRUE == ioHandle->flush) \
		        && (PSP_MCASP_OUTPUT == ioHandle->channelOpMode))
		{
		    packet->status = PSP_MCASP_FLUSHED;
		}

        packet->size = packet->size * ioHandle->noOfSerAllocated;
        /*
        Copy the ddc IOP packet information to ddc IO structure known to app layer
        */
        ddc_copyIOPtoReq(packet, (PSP_mcaspReq *)&pspRequest);

        /*
        sometimes the packet can have specific callback function
        */
        if(NULL != pspRequest.cbFxn)
        {
            (*pspRequest.cbFxn)(
                            (Ptr)ioHandle->deviceHandle->appHandle,
                            (Ptr)ioHandle->cbArg,
                            &pspRequest,
                            ioHandle->mode
                            );
        }
        else
        {
            /*
            otherwise call the application completion callback function
            registered with us during psp_mcaspopen
            */
            if (NULL != ioHandle->cbFxn)
            {
                (*ioHandle->cbFxn)(
                            (Ptr)ioHandle->deviceHandle->appHandle,
                            (Ptr)ioHandle->cbArg,
                            &pspRequest,
                            ioHandle->mode
                            );
            }

        }

        if (NULL != packet)
        {
            PAL_osList_NP_Enqueue(&(ioHandle->queueFreeList), packet);
        }
        ioHandle->dataPacket = NULL;
        ioHandle->userDataBufferSize = 0;
    }/* IO handle is not NULL if loop  */
}

/**
 *  \brief ddc_cancelAndAbortAllIo
 *
 *  Abort all the pending requests in both free and request queue
 *
 *  \param    ioHandle             [IN]    channel handler
 *  \return   void
 */
static void ddc_cancelAndAbortAllIo ( DDC_mcaspIoHandle ioHandle )
{

    Uint32              dlbMode     = 0;
    if(NULL != ioHandle)
    {
        ioHandle->submitCount--;
        ioHandle->dataPacket->status = PSP_MCASP_ABORTED;
        ddc_completeCurrentIO(ioHandle);

        /*
        While aborting care should be taken about DLB mode otherwise aborting
        packets in only one direction will make the packet in other direction
        not to return
        */
        dlbMode = (Uint32) CSL_FEXT(ioHandle->deviceHandle->hwInfo.regs->DLBCTL, MCASP_DLBCTL_DLBEN);

        if( CSL_MCASP_DLBCTL_DLBEN_ENABLE == dlbMode)
        {
            ddc_abortReset((DDC_mcaspIoHandle)&(ioHandle->deviceHandle->rxIo));
            ddc_abortReset((DDC_mcaspIoHandle)&(ioHandle->deviceHandle->txIo));
        }
        else
        {
            ddc_abortReset(ioHandle);
        }
    }
}

/**
 * \brief   ddc_activateSmFsForRcv
 *
 *      This function activates the receive section state machine and
 *      the corresponding frame sync.
 *
 *
 * \param     deviceHandle         [IN]     deviceHandle handle
 *
 * \return    void
 */

static void ddc_activateSmFsForRcv (DDC_mcaspDeviceHandle deviceHandle)
{
    if(NULL != deviceHandle)
    {
        deviceHandle->hwInfo.regs->RSTAT = (CSL_MCASP_RSTAT_RERR_MASK
        								    |CSL_MCASP_RSTAT_RDMAERR_MASK
        								    |CSL_MCASP_RSTAT_RSTAFRM_MASK
        								    |CSL_MCASP_RSTAT_RDATA_MASK
        								    |CSL_MCASP_RSTAT_RLAST_MASK
        								    |CSL_MCASP_RSTAT_RTDMSLOT_MASK
        								    |CSL_MCASP_RSTAT_RCKFAIL_MASK
        								    |CSL_MCASP_RSTAT_RSYNCERR_MASK
        								    |CSL_MCASP_RSTAT_ROVRN_MASK);

        ddc_bitSetGblRCtl(deviceHandle, CSL_MCASP_RGBLCTL_RSRCLR_MASK); /*serializer*/
        ddc_bitSetGblRCtl(deviceHandle, CSL_MCASP_RGBLCTL_RSMRST_MASK); /*state m/c*/
        ddc_bitSetGblRCtl(deviceHandle, CSL_MCASP_RGBLCTL_RFRST_MASK);  /*frame sync*/
    }
}


/**
 * \brief   ddc_deactivateSmFsForRcv
 *
 *      This function DEactivates the receive section state machine and
 *      the corresponding frame sync.
 *
 *
 * \param     deviceHandle         [IN]     deviceHandle handle
 *
 * \return    void
 */

static void ddc_deactivateSmFsForRcv (DDC_mcaspDeviceHandle deviceHandle)
{
    if(NULL != deviceHandle)
    {
        deviceHandle->hwInfo.regs->RSTAT = (~(CSL_MCASP_RSTAT_RERR_MASK
        								     |CSL_MCASP_RSTAT_RDMAERR_MASK
        								     |CSL_MCASP_RSTAT_RSTAFRM_MASK
        								     |CSL_MCASP_RSTAT_RDATA_MASK
        								     |CSL_MCASP_RSTAT_RLAST_MASK
        								     |CSL_MCASP_RSTAT_RTDMSLOT_MASK
           							         |CSL_MCASP_RSTAT_RCKFAIL_MASK
        								     |CSL_MCASP_RSTAT_RSYNCERR_MASK
        								     |CSL_MCASP_RSTAT_ROVRN_MASK));

        ddc_bitRemoveGblRCtl(deviceHandle, ~CSL_MCASP_RGBLCTL_RSRCLR_MASK); /*serializer*/
        ddc_bitRemoveGblRCtl(deviceHandle, ~CSL_MCASP_RGBLCTL_RSMRST_MASK); /*state m/c*/
        ddc_bitRemoveGblRCtl(deviceHandle, ~CSL_MCASP_RGBLCTL_RFRST_MASK);  /*frame sync*/
    }
}


/**
 * \brief   ddc_activateSmFsForXmt
 *
 *      This function activates the transmit section state machine and
 *      the corresponding frame sync.
 *
 *
 * \param     deviceHandle         [IN]     deviceHandle handle
 *
 * \return    void
 */

static void ddc_activateSmFsForXmt (DDC_mcaspDeviceHandle deviceHandle)
{
    if(NULL!=deviceHandle)
    {
        deviceHandle->hwInfo.regs->XSTAT = (CSL_MCASP_XSTAT_XERR_MASK
                                           |CSL_MCASP_XSTAT_XDMAERR_MASK
										   |CSL_MCASP_XSTAT_XSTAFRM_MASK
										   |CSL_MCASP_XSTAT_XDATA_MASK
										   |CSL_MCASP_XSTAT_XLAST_MASK
										   |CSL_MCASP_XSTAT_XTDMSLOT_MASK
										   |CSL_MCASP_XSTAT_XCKFAIL_MASK
										   |CSL_MCASP_XSTAT_XSYNCERR_MASK
										   |CSL_MCASP_XSTAT_XUNDRN_MASK);

        /*Start the Serializer, State M/C and Frame Sync Generator*/
        ddc_bitSetGblXCtl(deviceHandle, CSL_MCASP_XGBLCTL_XSRCLR_MASK);
        ddc_bitSetGblXCtl(deviceHandle, CSL_MCASP_XGBLCTL_XSMRST_MASK);
        ddc_bitSetGblXCtl(deviceHandle, CSL_MCASP_XGBLCTL_XFRST_MASK);
    }
}

/**
 * \brief   ddc_deactivateSmFsForXmt
 *
 *      This function deactivates the transmit section state machine and
 *      the corresponding frame sync.
 *
 *
 * \param     deviceHandle         [IN]     deviceHandle handle
 *
 * \return    void
 */

static void ddc_deactivateSmFsForXmt (DDC_mcaspDeviceHandle deviceHandle)
{
    if(NULL!=deviceHandle)
    {
        deviceHandle->hwInfo.regs->XSTAT = (~(CSL_MCASP_XSTAT_XERR_MASK
                                             |CSL_MCASP_XSTAT_XDMAERR_MASK
										     |CSL_MCASP_XSTAT_XSTAFRM_MASK
										     |CSL_MCASP_XSTAT_XDATA_MASK
										     |CSL_MCASP_XSTAT_XLAST_MASK
										     |CSL_MCASP_XSTAT_XTDMSLOT_MASK
										     |CSL_MCASP_XSTAT_XCKFAIL_MASK
										     |CSL_MCASP_XSTAT_XSYNCERR_MASK
										     |CSL_MCASP_XSTAT_XUNDRN_MASK));

        /*Start the Serializer, State M/C and Frame Sync Generator*/
        ddc_bitRemoveGblXCtl(deviceHandle, ~CSL_MCASP_XGBLCTL_XSRCLR_MASK);
        ddc_bitRemoveGblXCtl(deviceHandle, ~CSL_MCASP_XGBLCTL_XSMRST_MASK);
        ddc_bitRemoveGblXCtl(deviceHandle, ~CSL_MCASP_XGBLCTL_XFRST_MASK);
    }
}

/**
 * \brief   ddc_activateSmFsForBoth
 *
 *      This function activates the state machine and the corresponding frame
 *      sync for both receive and transmit sections.
 *
 *
 * \param     deviceHandle         [IN]     deviceHandle handle
 *
 * \return    void
 */

static void ddc_activateSmFsForBoth (DDC_mcaspDeviceHandle deviceHandle)
{
    if(NULL!=deviceHandle)
    {
        /*Clear the interrupts*/

        deviceHandle->hwInfo.regs->XSTAT = (CSL_MCASP_XSTAT_XERR_MASK
                                           |CSL_MCASP_XSTAT_XDMAERR_MASK
										   |CSL_MCASP_XSTAT_XSTAFRM_MASK
										   |CSL_MCASP_XSTAT_XDATA_MASK
										   |CSL_MCASP_XSTAT_XLAST_MASK
										   |CSL_MCASP_XSTAT_XTDMSLOT_MASK
										   |CSL_MCASP_XSTAT_XCKFAIL_MASK
										   |CSL_MCASP_XSTAT_XSYNCERR_MASK
										   |CSL_MCASP_XSTAT_XUNDRN_MASK);

        deviceHandle->hwInfo.regs->RSTAT = (CSL_MCASP_RSTAT_RERR_MASK
        								   |CSL_MCASP_RSTAT_RDMAERR_MASK
        								   |CSL_MCASP_RSTAT_RSTAFRM_MASK
        								   |CSL_MCASP_RSTAT_RDATA_MASK
        								   |CSL_MCASP_RSTAT_RLAST_MASK
        								   |CSL_MCASP_RSTAT_RTDMSLOT_MASK
        								   |CSL_MCASP_RSTAT_RCKFAIL_MASK
        								   |CSL_MCASP_RSTAT_RSYNCERR_MASK
        								   |CSL_MCASP_RSTAT_ROVRN_MASK);

        /*Start the Serializer, State M/C and Frame Sync Generator*/
        ddc_bitSetGblCtl(deviceHandle, (CSL_MCASP_XGBLCTL_XSRCLR_MASK | CSL_MCASP_RGBLCTL_RSRCLR_MASK));
        ddc_bitSetGblCtl(deviceHandle, (CSL_MCASP_XGBLCTL_XSMRST_MASK | CSL_MCASP_RGBLCTL_RSMRST_MASK));
        ddc_bitSetGblCtl(deviceHandle, (CSL_MCASP_XGBLCTL_XFRST_MASK  | CSL_MCASP_RGBLCTL_RFRST_MASK ));

    }
}


/**
 *  \brief ddc_bitSetGblCtl
 *
 *  set the GBLCTL bit and wait and check that bit is set or not
 *
 *  \param    deviceHandle             [IN]    deviceHandle Handler
 *  \param    bitMaskVal         [IN]    mask bit value
 *  \return   void
 */
static void ddc_bitSetGblCtl(DDC_mcaspDeviceHandle deviceHandle, Uint32 bitMaskVal)
{
    if (NULL != deviceHandle)
    {
        /* If already this bit is set then don't set again */
        if ((deviceHandle->hwInfo.regs->GBLCTL & bitMaskVal) != bitMaskVal)
        {
            deviceHandle->hwInfo.regs->GBLCTL = (deviceHandle->hwInfo.regs->GBLCTL | bitMaskVal);
            while ((deviceHandle->hwInfo.regs->GBLCTL & bitMaskVal) != bitMaskVal)
            {
            }

        }
    }
}


/**
 *  \brief ddc_bitSetGblRCtl
 *
 *  set the GBLCTLR bit and wait and check that bit is set or not
 *
 *  \param    deviceHandle             [IN]    deviceHandle Handler
 *  \param    bitMaskVal         [IN]    mask bit value
 *  \return   void
 */
static void ddc_bitSetGblRCtl(DDC_mcaspDeviceHandle deviceHandle, Uint32 bitMaskVal)
{
    if (NULL != deviceHandle)
    {
        /* If already this bit is set then don't set again */
        if ((deviceHandle->hwInfo.regs->GBLCTL & bitMaskVal) != bitMaskVal)
        {
            if ( FALSE == deviceHandle->rxIo.isDmaDriven)
            {
                PAL_osEvtDisable(deviceHandle->rxIo.cpuEventNum);
            }
            deviceHandle->hwInfo.regs->RGBLCTL = (deviceHandle->hwInfo.regs->RGBLCTL | bitMaskVal);
            while ((deviceHandle->hwInfo.regs->GBLCTL & bitMaskVal) != bitMaskVal)
            {
            }
            if ( FALSE == deviceHandle->rxIo.isDmaDriven)
            {
                /* Leave critical section */
                   PAL_osEvtEnable(deviceHandle->rxIo.cpuEventNum);
            }
        }
    }
}

/**
 *  \brief ddc_bitRemoveGblRCtl
 *
 *  Remove the GBLCTLR bit and wait and check that bit is set or not
 *
 *  \param    deviceHandle       [IN]    deviceHandle Handler
 *  \param    bitMaskVal         [IN]    mask bit value
 *  \return   void
 */
static void ddc_bitRemoveGblRCtl(DDC_mcaspDeviceHandle deviceHandle, Uint32 bitMaskVal)
{
    Uint32  temp_bitMaskVal;
    temp_bitMaskVal = (0xFFFFFFFFu & (Uint32)bitMaskVal);
    if (NULL != deviceHandle)
    {
        /* If already this bit is set then don't set again */
        if ((deviceHandle->hwInfo.regs->GBLCTL & ~temp_bitMaskVal) == (0x0000FFFFu & ~temp_bitMaskVal))
        {
            deviceHandle->hwInfo.regs->RGBLCTL = (deviceHandle->hwInfo.regs->RGBLCTL & (0xFFFF0000u | temp_bitMaskVal));
            while ((deviceHandle->hwInfo.regs->GBLCTL & ~temp_bitMaskVal) == (0x0000FFFFu & ~temp_bitMaskVal))
            {
            }

        }
    }
}

/**
 *  \brief ddc_bitSetGblXCtl
 *
 *  set the GBLCTLX bit and wait and check that bit is set or not
 *
 *  \param    deviceHandle       [IN]    deviceHandle Handler
 *  \param    bitMaskVal         [IN]    mask bit value
 *  \return   void
 */
static void ddc_bitSetGblXCtl(DDC_mcaspDeviceHandle deviceHandle, Uint32 bitMaskVal)
{
    if (NULL != deviceHandle)
    {
        /* If already this bit is set then don't set again */
        if ((deviceHandle->hwInfo.regs->GBLCTL & bitMaskVal) != bitMaskVal)
        {
            if ( FALSE == deviceHandle->txIo.isDmaDriven)
            {
                PAL_osEvtDisable(deviceHandle->txIo.cpuEventNum);
            }
            deviceHandle->hwInfo.regs->XGBLCTL = (deviceHandle->hwInfo.regs->XGBLCTL | bitMaskVal);
            while ((deviceHandle->hwInfo.regs->GBLCTL & bitMaskVal) != bitMaskVal)
            {
            }
            if ( FALSE == deviceHandle->txIo.isDmaDriven)
            {
                /* Leave critical section */
                   PAL_osEvtEnable(deviceHandle->txIo.cpuEventNum);
            }
        }
    }
}

/**
 *  \brief ddc_bitRemoveGblXCtl
 *
 *  Remove the GBLCTLX bit and wait and check that bit is set or not
 *
 *  \param    deviceHandle       [IN]    deviceHandle Handler
 *  \param    bitMaskVal         [IN]    mask bit value
 *  \return   void
 */
static void ddc_bitRemoveGblXCtl(DDC_mcaspDeviceHandle deviceHandle, Uint32 bitMaskVal)
{
    volatile Uint32  temp_bitMaskVal;
    temp_bitMaskVal = (0xFFFFFFFFu & (Uint32)bitMaskVal);
    if (NULL != deviceHandle)
    {
        /* If already this bit is set then don't set again */
        if ((deviceHandle->hwInfo.regs->GBLCTL & ~temp_bitMaskVal) == (0x0000FFFFu & ~temp_bitMaskVal))
        {
            deviceHandle->hwInfo.regs->XGBLCTL = (deviceHandle->hwInfo.regs->XGBLCTL & (0xFFFF0000u | temp_bitMaskVal));
            while ((deviceHandle->hwInfo.regs->GBLCTL & ~temp_bitMaskVal) == (0x0000FFFFu & ~temp_bitMaskVal))
            {
            }
        }
    }
}

/**
 *  \brief ddc_copyReqtoIOP
 *
 *  Copies data from request structure provided by DDA layer to
 *  meaningful IO structure at  DDC layer.
 *
 *  \param    pspRequest       [IN]    Porinter to DDA layer reuqest structuer
 *  \param    ioPacket         [IN]    Porinter to DDC layer IO structuer
 *  \return   void
 */
static void ddc_copyReqtoIOP (PSP_mcaspReq * const pspRequest,DDC_mcaspIOP *const ioPacket)
{

    if ((NULL != pspRequest ) && (NULL != ioPacket))
    {
        ioPacket->addr          = pspRequest->addr;
        ioPacket->size          = pspRequest->size;
        ioPacket->actualSize    = pspRequest->actualSize;
        ioPacket->status        = pspRequest->status;
        ioPacket->appCbk        = pspRequest->cbFxn;
        ioPacket->appData       = pspRequest->appPtr;
    }
}

/**
 *  \brief ddc_copyIOPtoReq
 *
 *  Restores the data from DDC layer request structure to
 *  DDA understandable psp request structure.
 *
 *  \param    ioPacket         [IN]    Porinter to DDC layer IO structuer
 *  \param    pspRequest       [IN]    Porinter to DDA layer reuqest structuer
 *
 *  \return   void
 */
static void ddc_copyIOPtoReq (DDC_mcaspIOP *const ioPacket,PSP_mcaspReq *const pspRequest)
{
    if ((NULL != pspRequest ) && (NULL != ioPacket))
    {
        pspRequest->addr        = ioPacket->addr;
        pspRequest->size        = ioPacket->size;
        pspRequest->actualSize  = ioPacket->actualSize;
        pspRequest->status      = ioPacket->status;
        pspRequest->cbFxn       = ioPacket->appCbk;
        pspRequest->appPtr      = ioPacket->appData;
    }

}

