/*
 *  Copyright 2007 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) TCP/IP_Network_Developers_Kit 1.92.00.13 
 */
/*
 *  ======== cpsw3g_core.c ========
 *
 */

/**
 *--------------------------------------------------------------------------
 * IP Stack
 *--------------------------------------------------------------------------
 * cpsw3g_core.c
 *
 * Low level implementation for CPSW packet driver
 *
 * Author : A.G.Sriramakrishnan
 * Copyright 2007 by Texas Instruments Inc.
 *-------------------------------------------------------------------------
 */
#include <std.h>
#include <sys.h>
#include <hwi.h>
#include <bcache.h>
#include <c64.h>
#include<ecm.h>

#include <stkmain.h>
#include "llpacket.h"
#include "instrumentation.h"
#include "cpsw3g_common.h"
#include "cpsw3g_core.h"


#define EXTMEM          0x80000000u
#define CPSW3G_NUM_INSTANCES 0x1u
/**
 *
 * This single global can be checked via CCS if there's trouble
 *
 */
Uint32  CPSW3G_FatalError = 0;

/**
 *
 * Configuration (Obtained via callback)
 *
 */


extern void     CPSW3G_getConfig( Uint8 *pMacAddr, Uint32 *pIntVector);
extern void     CPSW3G_linkStatus( Uint32 phy, Uint32 linkStatus );

extern Uint32   macsel;
static Uint32     IntVector = 5u;  /* Interrupt vector to use */

/* flag that indicates whether we use SMA connectors(no-PHY) mode or
 * interface the SERDES to PHYs. APP must set this using driver
 * callback before NDK/Driver Initialization */
static Uint32     use_sma_port0 = 0u,use_sma_port1 =0u;

/* keep track of number of BDs allocated */
static Uint32 numBdsAllocated = 0;
static Uint32 FlashActiveLED = 0;
Uint32 cpsw3g_statistics = 0;

#define EMAC_NUMSTATS   36         /* The number of statistics regs */

/**
 *
 * RX Buffer Depth
 *
 * We set this to 16 by default. This is the max (typical) number
 * of free buffers ready for rx packets.
 *
 * 
 * Note that this driver always assumes it can enqueue a new TX
 * packet to the EMAC. It justifies this by assuming that the size of
 * its transmit descriptor chain (512 - EMAC_MAX_RX) is greater than
 * or equal to the size of the free packet buffer pool (defined by
 * PKT_NUM_FRAMEBUF in pbm.c). Thus the value of EMAC_MAX_RX
 * plus the size of PKT_NUM_FRAMEBUF (defaults to 48) must be less
 * than or equal to 512 for the driver to work properly.
 *
 * #define EMAC_MAX_RX     64     
 *
 * CHECK: should work even if assumption not met. Number of Rx BDs from
 * Init config in cpsw3g_init_cfg.h
 */


/** CHECK : For GIG interface 16 will not be enough, change from 16 to 64 min 
 * Factors to consider: Number of BDs that can be fit in BD RAM
 * Number of buffers in free packet buffer pool, PKT_NUM_FRAMEBUF = 192 currently */


/**
 * Local instance information pointer for our single device instance
 */
static PDINFO *pPDI = 0;

typedef unsigned long long Uint64;

Uint64 cpswTxPkts = 0,cpswRxPkts = 0;

/* Local Cpsw book-keeping structure */
static Cpsw3gDevice cpsw3gDev = {0};
static Cpsw3gInitConfig cpsw3gCfg = {0};
static Uint32 driver_initialized = 0u;

/* followig 3 are dummy params */
#define CPSW3G_ESWITCH_RESET_BIT		0
#define CPSW3G_MDIO_RESET_BIT 			0
#define CPSW3G_HIGH_CPMAC_INT			0

#define CPSW3G_SWBD_SIZE		(sizeof(CPPI_Desc))
#define CPU_TO_HW(addr)    		(addr)
#define HW_TO_CPU(addr)    		(addr)

#define CPU_TO_HWBD(addr)    		(((Uint32)(addr) - 0x7E000))
#define HW_TO_CPUBD(addr)    		(((Uint32)(addr) + 0x7E000))

#define CACHE_FLUSH_BUFFER(addr,size)			BCACHE_wb(((Ptr)(addr)), (size), CACHE_WAIT)
#define CACHE_INVALIDATE_BUFFER(addr,size)		BCACHE_inv(((Ptr)(addr)), (size), CACHE_WAIT)

/* The BDs are internal, no need for cache ops here- define as NOOP */
#define CACHE_FLUSH_BD(addr,size)			
#define CACHE_INVALIDATE_BD(addr,size)		




/* ALE_DUMP_PRINT macro should be mapped to cpsw3g_Printf ideally */
#define ALE_DUMP_PRINT  printf


/* macro for Delay, if taskdelay works ,just map it to taskDelay of Bios. Default to tight loop
 *  Rough calculation, have a counter that runs 1024 times the input value*/
#define waitMsecs(numMSecs) 		{\
									volatile Uint32 ctr = (numMSecs) << 10;\
									do{\
										ctr --;\
										}while (ctr!=0);\
									}
									


/**
 * Local Functions
 */
static void HwInt();
static Uint32 cpsw3g_open();
static Uint32 cpsw3g_close(Cpsw3gDevice *devPtr);
static void cpsw3g_get_cfg();

static int cpsw3g_port_close(Cpsw3gPort *hPort);
static Int32 cpsw3g_port_open(Cpsw3gPort * hPort);
static void cpgmacMacOpen(Cpsw3gPort * hPort);
static void cpgmacMacClose(Cpsw3gPort * hPort);
static Uint32 cpdmaOpen(Cpsw3gPort * hPort);
static Uint32 cpdmaClose(Cpsw3gPort * hPort);

/* Ioctl low level implementation functions */
static void cpsw3g_SetCpswCfg(void);
static void cpsw3g_SetAleCfg(void);
static void cpsw3g_SetAlePortCfg(Cpsw3gPort * hPort);
static void cpsw3g_SetDmaCfg(Cpsw3gPort * hPort);
static void cpsw3g_SetMacCfg(Cpsw3gPort * hPort);

static Uint32 cpsw3g_AleAddVlan(Uint32 vlanId,Uint32 vlanMemberList,
        Uint32 unregMcastFloodMask, Uint32 regMcastFloodMask, 
        Uint32 forceUntaggedEgress);

static Int32 cpsw3g_AleFindVlan(Uint32 vlanId, Uint32 *vlanMemberList, 
            Uint32 *unregMcastFloodMask, Uint32 *regMcastFloodMask,
            Uint32 *forceUntaggedEgress);

static Uint32 cpsw3g_AleAddUniAddr(Cpsw3gPort *hPort, Uint8 * addr, Uint32 vlanId,
                                    Uint32 blocked, Uint32 secure, Uint32 ageable);

static Int32 cpsw3g_AleFindAddress( Uint8 *addr, Uint32 vlanId);
static Uint32 cpsw3g_AleAddMltAddr(Uint8 *addr, Uint32 vlanId, Uint32 portMask,
                                    Uint32 super, Uint32 fwdState);
static Uint32 cpsw3g_AleAddOuiAddr(Uint8 *ouiAddress);
static Int32 cpsw3g_AleDelAddress( Uint8 *addr, Uint32 vlanId);
static Int32 cpsw3g_AleDelVlan(Uint32 vlanId);

static void cpsw3g_RateLimitTx(Uint32 enable, Uint32 port, 
                                typeOfData type, Uint32 noOfPkts);
static void cpsw3g_RateLimitRx(Uint32 enable, Uint32 port, 
                                typeOfData type, Uint32 noOfPkts);
static void cpsw3g_SetPortVlanCfg(Cpsw3gPort* hPort, Uint32 pri, Uint32 cfi, Uint32 vlanId);
static void cpsw3g_SetPriorityMapping(Cpsw3gPort * hPort, Uint32 rxPriority,
                                        Uint32 txPriority, Uint32 switchPriority);
static Uint32 cpsw3g_SetSwitchFlowControl(Cpsw3gPort *hInPort, Cpsw3gPort *hOutPort,
                                            Uint32 thresh, Uint32 enable);


static Uint32 cpsw3g_AleGetFreeEntry();
static void cpsw3g_AleFreeTableClear(Uint32 entry);
static Uint32 cpsw3g_Control(Uint32 cmd, void *cmdArg, void *param);

static void cpsw3g_SetPhyMode(CpgmacObj *gmacObject,Uint32 instanceNum);
static Uint32 cpsw3g_Update_Phystatus(CpgmacObj * gmacObject,Uint32 instanceNum);
static Uint32 cpsw3g_ControlCb(Cpsw3gDevice *devPtr,Uint32 cmd,Ptr cmdArg, Ptr param);
static Uint32 cpsw3g_EnableChannel(Uint32 channel, Uint32 direction);
static Uint32 cpsw3g_DisableChannel(Uint32 channel, Uint32 direction);

static Uint32 cpsw3g_NetChOpen(Cpsw3gChInfo * chInfo);
static Uint32 cpsw3g_NetChClose(Cpsw3gChInfo * chInfo);
static Uint32 cpsw3g_UnInitTxChannel(Cpsw3gChInfo *chInfo);
static Uint32 cpsw3g_UnInitRxChannel(Cpsw3gChInfo *chInfo);
static Uint32 cpsw3g_InitTxChannel(Cpsw3gChInfo * chInfo);
static Uint32 cpsw3g_InitRxChannel(Cpsw3gChInfo * chInfo);
static void cpsw3g_Enqueue_Rx(Cpsw3gRxCppiCh * rxChan,Uint32 restart);
static void cpsw3g_Dequeue_Rx(Cpsw3gRxCppiCh * rxChan,CPPI_Desc *pDescAck);
static void cpsw3g_Dequeue_Tx(Cpsw3gTxCppiCh * txChan,CPPI_Desc *pDescAck);
static Uint32 cpsw3g_Tick(Cpsw3gDevice * devPtr);
static void  cpsw3g_AleDump(Cpsw3gDevice * devPtr,int(*printFnArg)(const char*,...));

/* The following callback must be invoked by the Application to override
 * the default settings. The application needs to exercise this  before
 * the init for NDK stack, ESS driver
 * This function will return an error if this callback is issued
 * after the driver initialization*/
Uint32 CPSW3G_set_connector(Uint32 use_SMA_on_port0,Uint32 use_SMA_on_port1)
{
    Uint32 ret = 0u;

    if (0u == driver_initialized)
    {
	use_sma_port0 = use_SMA_on_port0;
	use_sma_port1 = use_SMA_on_port1;
	ret =  CPSW3G_SUCCESS;
    }
    else
    {
	ret = 1u;
    }
    
    return ret;
}/*end of function CPSW3G_set_connector */

#define ECM_INTVECMAP_NUM		8 	 
/* We use eventIds 32.RxPulse, 33.RxThresh,34.TxPulse,35.Misc.Pulse*/
static void cpsw3g_disable_interrupts()
{
    C64_disableIER( (1<<ECM_INTVECMAP_NUM ));

    /* disable all 4 ecm events */
    ECM_disableEvent(RX_PULSE_INTNUM);
    ECM_disableEvent(RX_THRESH_PULSE_INTNUM);
    ECM_disableEvent(TX_PULSE_INTNUM);
    ECM_disableEvent(MISC_PULSE_INTNUM);

    C64_enableIER( (1<<ECM_INTVECMAP_NUM ));
}/* end of function cpsw3g_disable_interrupts */

static void cpsw3g_enable_interrupts()
{
    C64_disableIER( (1<<ECM_INTVECMAP_NUM ));

    /* enable all 4 ecm events */
    ECM_enableEvent(RX_PULSE_INTNUM);
    ECM_enableEvent(RX_THRESH_PULSE_INTNUM);
    ECM_enableEvent(TX_PULSE_INTNUM);
    ECM_enableEvent(MISC_PULSE_INTNUM);

    C64_enableIER( (1<<ECM_INTVECMAP_NUM ));

}/* end of function cpsw3g_enable_interrupts */
	


/**
 *--------------------------------------------------------------------
 * HwPktInit()
 *
 * Initialize Device environment and return instance count
 *--------------------------------------------------------------------
 */
Uint32 HwPktInit()
{

    cpsw3g_get_cfg();
    /* Device is not open yet */
    cpsw3gDev.setToClose = 1u;
    cpsw3gDev.regs = (CSL_Cpsw3g_RegsOvly)(cpsw3gCfg.cpswCfg.baseAddress);
    cpsw3gDev.ectlRegs = (CSL_EctlRegsOvly)(cpsw3gCfg.cpswCfg.cpsw3GSSBaseAddress);
    cpsw3gDev.sgmii0Regs = (CSL_SgmiiRegsOvly)(cpsw3gCfg.cpswCfg.sgmii0BaseAddress);
    cpsw3gDev.sgmii1Regs = (CSL_SgmiiRegsOvly)(cpsw3gCfg.cpswCfg.sgmii1BaseAddress);


    /* there are soft resets in indv. modules, and one from LPSC */
    /* order of reset CPDMA->SL0->SL1->3G->SG0->SG1->3Gss */

    /* Soft reset CPDMA */
    CSL_FINS(cpsw3gDev.regs->SOFT_RESET,
                        CPSW3G_SOFT_RESET_SOFT_RESET, CSL_CPSW3G_SOFT_RESET_SOFT_RESET_RESET);
    while(CSL_FEXT(cpsw3gDev.regs->SOFT_RESET, CPSW3G_SOFT_RESET_SOFT_RESET) !=
    		CSL_CPSW3G_SOFT_RESET_SOFT_RESET_NORESET)
    {
    }

    /* soft reset CPGMAC_SL0 */
    CSL_FINS(cpsw3gDev.regs->GMAC[0].GMAC_SOFT_RESET,
                        CPSW3G_GMAC_SOFT_RESET_SOFT_RESET, CSL_CPSW3G_GMAC_SOFT_RESET_SOFT_RESET_RESET);
    while(CSL_FEXT(cpsw3gDev.regs->GMAC[0].GMAC_SOFT_RESET, CPSW3G_GMAC_SOFT_RESET_SOFT_RESET) !=
    		CSL_CPSW3G_GMAC_SOFT_RESET_SOFT_RESET_NORESET)
    {
    }

    /* soft reset CPGMAC_SL1 */
    CSL_FINS(cpsw3gDev.regs->GMAC[1].GMAC_SOFT_RESET,
                        CPSW3G_GMAC_SOFT_RESET_SOFT_RESET, CSL_CPSW3G_GMAC_SOFT_RESET_SOFT_RESET_RESET);
    while(CSL_FEXT(cpsw3gDev.regs->GMAC[1].GMAC_SOFT_RESET, CPSW3G_GMAC_SOFT_RESET_SOFT_RESET) !=
    		CSL_CPSW3G_GMAC_SOFT_RESET_SOFT_RESET_NORESET)
    {
    }

    /*soft reset CPSW_3G */
    CSL_FINS(cpsw3gDev.regs->CPSW_SOFT_RESET,
                        CPSW3G_CPSW_SOFT_RESET_SOFT_RESET, CSL_CPSW3G_CPSW_SOFT_RESET_SOFT_RESET_RESET);
    while(CSL_FEXT(cpsw3gDev.regs->CPSW_SOFT_RESET, CPSW3G_CPSW_SOFT_RESET_SOFT_RESET) !=
    		CSL_CPSW3G_CPSW_SOFT_RESET_SOFT_RESET_NORESET)
    {
    }

    /* soft reset SGMII 0 - immediate no need to wait*/
    CSL_FINS(cpsw3gDev.sgmii0Regs->SOFT_RESET,
                        SGMII_SOFT_RESET_SOFT_RESET, CSL_SGMII_SOFT_RESET_SOFT_RESET_YES);
    

    /* soft reset SGMII 1 - immediate no need to wait*/
    CSL_FINS(cpsw3gDev.sgmii1Regs->SOFT_RESET,
                        SGMII_SOFT_RESET_SOFT_RESET, CSL_SGMII_SOFT_RESET_SOFT_RESET_YES);

    /* Soft reset CPSW_3G_SS */
    CSL_FINS(cpsw3gDev.ectlRegs->SOFT_RESET,
    				ECTL_SOFT_RESET_SOFT_RESET,CSL_ECTL_SOFT_RESET_SOFT_RESET_YES);

	
   /* program Mr_Adv_ability of  SGMII 0 - MAC to PHY Config
     * Make this a config param in the future*/
    if(0u == use_sma_port0)
    	{
    cpsw3gDev.sgmii0Regs->MR_ADV_ABILITY = 0x1;
   /* program Control reg  of  SGMII 0 - autoneg, slave mode
     * Make this a config param in the future*/
    CSL_FINS(cpsw3gDev.sgmii0Regs->CONTROL,
                        SGMII_CONTROL_MASTER, CSL_SGMII_CONTROL_MASTER_SLAVE);
    CSL_FINS(cpsw3gDev.sgmii0Regs->CONTROL,
                        SGMII_CONTROL_MR_AN_ENABLE, CSL_SGMII_CONTROL_MR_AN_ENABLE_YES);
	}
    else
	{
	cpsw3gDev.sgmii0Regs->MR_ADV_ABILITY = 0x9801;
	/* program Control reg  of  SGMII 0 - no autoneg, master mode
     	* Make this a config param in the future*/
	CSL_FINS(cpsw3gDev.sgmii0Regs->CONTROL,
                        SGMII_CONTROL_MASTER, CSL_SGMII_CONTROL_MASTER_MASTER);
    	CSL_FINS(cpsw3gDev.sgmii0Regs->CONTROL,
                        SGMII_CONTROL_MR_AN_ENABLE, CSL_SGMII_CONTROL_MR_AN_ENABLE_NO);
	}

   /* program Mr_Adv_ability of  SGMII 1 - MAC to PHY Config
     * Make this a config param in the future*/
    if(0u == use_sma_port1 )
    	{
	cpsw3gDev.sgmii1Regs->MR_ADV_ABILITY = 0x1;
   /* program Control reg  of  SGMII 1 - autoneg, slave mode
     * Make this a config param in the future*/
    CSL_FINS(cpsw3gDev.sgmii1Regs->CONTROL,
                        SGMII_CONTROL_MASTER, CSL_SGMII_CONTROL_MASTER_SLAVE);
    CSL_FINS(cpsw3gDev.sgmii1Regs->CONTROL,
                        SGMII_CONTROL_MR_AN_ENABLE, CSL_SGMII_CONTROL_MR_AN_ENABLE_YES);

	}
    else
	{
	cpsw3gDev.sgmii1Regs->MR_ADV_ABILITY = 0x9801;
	/* program Control reg  of  SGMII 1 - no autoneg, master mode
     	 * Make this a config param in the future*/
    	CSL_FINS(cpsw3gDev.sgmii1Regs->CONTROL,
                        SGMII_CONTROL_MASTER, CSL_SGMII_CONTROL_MASTER_MASTER);
    	CSL_FINS(cpsw3gDev.sgmii1Regs->CONTROL,
                        SGMII_CONTROL_MR_AN_ENABLE, CSL_SGMII_CONTROL_MR_AN_ENABLE_NO);
	}
    


    #ifdef CPSW3G_USE_INTERRUPT_PACING
    /* enable Interrupt Pacing Logic in the Wrapper */
    CSL_FINS(cpsw3gDev.ectlRegs->RX_IMAX,ECTL_RX_IMAX_RX_IMAX,NUM_RX_INTS_PERMSEC);
    CSL_FINS(cpsw3gDev.ectlRegs->TX_IMAX,ECTL_TX_IMAX_TX_IMAX,NUM_TX_INTS_PERMSEC);

    CSL_FINS(cpsw3gDev.ectlRegs->INT_CONTROL,ECTL_INT_CONTROL_INT_PACE_EN,0x3F);
    CSL_FINS(cpsw3gDev.ectlRegs->INT_CONTROL,
	    ECTL_INT_CONTROL_INT_PRESCALE,(cpsw3gCfg.cpswCfg.cpmacBusFrequency /1000000 * 4));
    #endif




     cpsw3gDev.Clear_EOI = 0;
     cpsw3gDev.timerActive = 0;

     driver_initialized = 1u;
 
    /* return the number of instances, which would be 1 in this case */ 
    return(CPSW3G_NUM_INSTANCES);
}/* end of function HwPktInit() */


/**
 *--------------------------------------------------------------------
 * HwPktShutdown()
 *
 * Shutdown Device Environment
 *--------------------------------------------------------------------
 */
void HwPktShutdown()
{
	/* If possible put the module back in reset*/
	/* Free up any allocated mem resources */
    driver_initialized = 0u;



}/* end of function HwPktShutdown() */

/**
 *--------------------------------------------------------------------
 * HwPktOpen()
 *
 * Open Device Instance
 *--------------------------------------------------------------------
 */
Uint32 HwPktOpen( PDINFO *pi )
{
    Uint32  ReturnCode = 1u;
    ECM_Attrs ecmattrs = ECM_ATTRS;

    /** We only have one device, so store off the PDINFO pointer as
     * global to this module. Otherwise; we'd pick an unclaimed device
     * and associate it with the PDINFO. */

    if( (PDINFO*)0 == pi)
    {
	ReturnCode = 0u;
    }
    else
    {
    /** Disable our general purpose interrupt */
	cpsw3g_disable_interrupts();

    /** If pPDI is already set, this is a restart.*/
    if( (PDINFO*)0 == pPDI )
    {  
        /** Now initialize the EMAC device */
        pPDI = pi;

        /* register the same Isr for events 32-35 */
    	ecmattrs.arg = (Arg) pi;
        /* do not enable the interrupt events yet */
        ecmattrs.unmask = 0;
	    ECM_dispatchPlug(RX_PULSE_INTNUM,(ECM_Fxn)&HwInt,&ecmattrs);
	    ECM_dispatchPlug(RX_THRESH_PULSE_INTNUM,(ECM_Fxn)&HwInt,&ecmattrs);
	    ECM_dispatchPlug(TX_PULSE_INTNUM,(ECM_Fxn)&HwInt,&ecmattrs);
	    ECM_dispatchPlug(MISC_PULSE_INTNUM,(ECM_Fxn)&HwInt,&ecmattrs);
        
        BCACHE_invL1pAll(); 
    }

    /** Use the new PDINFO */
    pPDI = pi;
    pi->hNetDev = (HANDLE) &cpsw3gDev;
    cpsw3gDev.pdinfo = pi;

    /**
     * Copy the MAC address from the configuration information
     */
    mmCopy(pPDI->bMacAddr,cpsw3gDev.macAddr,6u);

    /* Init internal bookkeeping fields */
    pi->OldMCastCnt = 0;

    /** Initialize the cpsw3g device HW */
    cpsw3g_open(&cpsw3gDev);

    
    /** Enable our general purpose interrupt */
    cpsw3g_enable_interrupts();
    /** Say not ready to send (we wait for link) */
    pi->TxOn = 0;
    pi->TxFree = 1;
    }

    return(ReturnCode);
}/* end of function HwPktOpen */

/**
 *--------------------------------------------------------------------
 * HwPktClose()
 *
 * Close Device Instance
 *--------------------------------------------------------------------
 */
void HwPktClose( PDINFO *pi )
{
    Cpsw3gDevice * devPtr = (Cpsw3gDevice*)pi->hNetDev;

    if((PDINFO*)0 != pi)
    {

    /**
     * Disable the DSP interrupt
     */
	cpsw3g_disable_interrupts();

    /* close */
    cpsw3g_close(devPtr);

    /* indicate all Bds are free now
     * May have to move to shutdown call */
    numBdsAllocated = 0;
    pPDI = (PDINFO*) 0;
    pi->hNetDev = (HANDLE) 0;
    }
    
}/* end of function HwPktClose() */

/**
 *--------------------------------------------------------------------
 * HwPktSetRx()
 *
 * Update Rx Mode (Rx filter and multicast hash table)
 *--------------------------------------------------------------------
 */
void HwPktSetRx( PDINFO *pi )
{
    Cpsw3gDevice *devPtr;
    Cpsw3gInitConfig *cfgPtr;

    if ((PDINFO*)0 != pi)
    {
    devPtr = pi->hNetDev;

    if(0==devPtr)
        {
            printf("HwPktSetRx:Dev Struct is NULL.Fatal Error\n");
            return;
        }
    cfgPtr = devPtr->initCfg;
    if(0==cfgPtr)
        {
            printf("HwPktSetRx:CFG Struct is NULL.Fatal Error\n");
            return;
        }

   
    /**
     * The following code relies on the numeric relation of the filter
     * value such that the higher filter values receive more types of
     * packets.
     */

    /**
     * Disable Section
     */
    
    /* remove multicast entries that was added from previous SetRx if there
     * are any*/
    if( 0u != pi->OldMCastCnt)
    {
	Uint32 i;

	for(i = 0u; i < pi->OldMCastCnt; i++)
        {
	    cpsw3g_AleDelAddress((&pi->bOldMCast[6*i]), CPSW3G_DEFAULT_MAC_PORTVID);
	}

	/* re-init OldMcastCnt to zero, will be set to right count(further down)
	 * if multicast entries are being added*/
	pi->OldMCastCnt = 0;    
    }

    if ( pi->Filter < ETH_PKTFLT_ALL )
    {
        /* ENSURE ALE BYPASS IS TURNED OFF */
        if (0 != cfgPtr->aleCfg.aleBypass)
            {
            cfgPtr->aleCfg.aleBypass = 0;
            cfgPtr->aleCfg.clearTable = 0;
            cpsw3g_SetAleCfg();
            }
    }

    if ( pi->Filter < ETH_PKTFLT_ALLMULTICAST ) 
    {
        /* Ensure that unreg_multicast_mask has CPU port cleared */
        cfgPtr->aleCfg.unknownMcastFloodMask = ( (0 << 2) | (1 << 1)  | (1 <<0));
        cfgPtr->aleCfg.aleBypass = 0;
        cfgPtr->aleCfg.clearTable = 0;
        cpsw3g_SetAleCfg();
        /* Also Program VLAN entry settings for Default VID */
        cpsw3g_AleAddVlan(CPSW3G_DEFAULT_MAC_PORTVID,CPSW3G_DEFAULT_UNK_VLANMEMLIST,
                3,7,CPSW3G_DEFAULT_UNK_FORCE_UNTAG_EGR); 
    }

    if ( pi->Filter < ETH_PKTFLT_MULTICAST )
    {
         /* Ensure that reg/unreg_multicast_mask has CPU port cleared */
        cfgPtr->aleCfg.unknownMcastFloodMask = ( (0 << 2) | (1 << 1)  | (1 <<0));
        cfgPtr->aleCfg.unknownRegMcastFloodMask = ( (0 << 2) | (1 << 1)  | (1 <<0));
        cfgPtr->aleCfg.aleBypass = 0;
        cfgPtr->aleCfg.clearTable = 0;
        cpsw3g_SetAleCfg();
        /* Also Program VLAN entry settings for Default VID */
        cpsw3g_AleAddVlan(CPSW3G_DEFAULT_MAC_PORTVID,CPSW3G_DEFAULT_UNK_VLANMEMLIST,
                3,3,CPSW3G_DEFAULT_UNK_FORCE_UNTAG_EGR); 
        
    }

    if ( pi->Filter < ETH_PKTFLT_BROADCAST )
    {       
        /* Broadcast cannot be turned off */
    }

    if ( pi->Filter < ETH_PKTFLT_DIRECT )
    {
        /* Remove the unicast address here*/
    }


    /**
     * Enable Section
     */
    if ( pi->Filter >= ETH_PKTFLT_DIRECT )
    {
       /* Add the unicast address here*/
    }

    if ( pi->Filter >= ETH_PKTFLT_BROADCAST )
    {
        
    }

    if ( pi->Filter == ETH_PKTFLT_MULTICAST )
    {
        Int32 i;

        /* Ensure that reg_multicast_mask has CPU port enabled */
        cfgPtr->aleCfg.unknownMcastFloodMask = ( (0 << 2) | (1 << 1)  | (1 <<0));
        cfgPtr->aleCfg.unknownRegMcastFloodMask = ( (1 << 2) | (1 << 1)  | (1 <<0));
        cpsw3g_SetAleCfg();
        
        /* Also Program VLAN entry settings for Default VID */
        cpsw3g_AleAddVlan(CPSW3G_DEFAULT_MAC_PORTVID,CPSW3G_DEFAULT_UNK_VLANMEMLIST,
                3,7,CPSW3G_DEFAULT_UNK_FORCE_UNTAG_EGR); 

        /* run through the list of multicast entries and add them to ALE */

	/* save count,entries for deletion next time around */
	pi->OldMCastCnt = pi->MCastCnt;
	for(i = 0; i < pi->MCastCnt; i++)
        {
	    mmCopy(&pi->bOldMCast[6*i],&pi->bMCast[6*i],6u);
	}
       
        /* CHECK: Do we delete all existing multicast entries from ALE before
         * adding new entries from the list */
        for(i = 0; i < pi->MCastCnt; i++)
        {
            /* adding with Portmask=0x7 so that pkt is forwarded to remaining 2 ports */
            cpsw3g_AleAddMltAddr((&pi->bMCast[6*i]), CPSW3G_DEFAULT_MAC_PORTVID , 0x7,
                                    0 , 3);
        }

    }
	else 
	{
	    if ( pi->Filter == ETH_PKTFLT_ALLMULTICAST ) 
    {
        /* Ensure that reg/unreg_multicast_mask have CPU port enabled */
        cfgPtr->aleCfg.unknownMcastFloodMask = ( (1 << 2) | (1 << 1)  | (1 <<0));
        cfgPtr->aleCfg.unknownRegMcastFloodMask = ( (1 << 2) | (1 << 1)  | (1 <<0));
        cpsw3g_SetAleCfg();       
        /* Also Program VLAN entry settings for Default VID */
        cpsw3g_AleAddVlan(CPSW3G_DEFAULT_MAC_PORTVID,CPSW3G_DEFAULT_UNK_VLANMEMLIST,
                7,7,CPSW3G_DEFAULT_UNK_FORCE_UNTAG_EGR); 
    }
	    else 
	    {
		if ( pi->Filter == ETH_PKTFLT_ALL )
    {
        if (0 == cfgPtr->aleCfg.aleBypass)
            {
            cfgPtr->aleCfg.aleBypass = 1;
            cfgPtr->aleCfg.clearTable = 0;
            cpsw3g_SetAleCfg();
            } 
     }
	    }
	}
    }
}/* end of function HwPktSetRx() */

Uint32 cpsw3g_outofBD = 0;

/**
 *--------------------------------------------------------------------
 * HwPktTxNext()
 *
 * Transmit Next Packet on Tx Queue
 *--------------------------------------------------------------------
 */
void HwPktTxNext( PDINFO *pi )
{
    register CPPI_Desc *pDescThis;
    register Uint8     *buffer;
    register Uint32      length;
    PBM_Handle         hPkt;
    Uint32             chNum=0;
    Cpsw3gTxCppiCh * txChan;
    CpdmaObj *dmaObj;
    Cpsw3gDevice *devPtr;
    
    devPtr = pi->hNetDev;
    if(0==devPtr)
        {
            printf("HwPktTxNext:Dev Struct is NULL.Fatal Error\n");
            return;
        }
    
    /* We only use one Tx DMA channel  now. If QOS/Priority has to be
      * supported, identify the channel number based on vlan/priority tag or
      * any other field from the header fields of the ethernet frame. */

    chNum = CPSW3G_DEFAULT_TX_CHANNEL;
    dmaObj = &devPtr->cpdma;
    txChan = &dmaObj->txCppi[chNum];

    /**
     * Disable our device interrupt
     */
    cpsw3g_disable_interrupts();

    do
    {

    	/**
         *  If our assumption that number of BDs on the Tx CPPI channel is more than
         *  the number of Buffers from PBM buffer pool, doesnt hold true, we can come
         * across this situation. Only action possible is to drop the packet as there is 
         * no free BD available 
         */
        if( txChan->DescCount == txChan->DescMax )
        {
            if(PBMQ_count(&pi->PBMQ_tx) != 0)
            {
            	printf(" Run out of Tx BD\n");
            	cpsw3g_outofBD ++;
            }
            pi->TxFree = 0;
            break;
        }
        
        /**
         *Make sure we have a packet to send
         */
        hPkt = PBMQ_deq(&pi->PBMQ_tx);
        if( (PBM_Handle)0 == hPkt )
        {
            pi->TxFree = 1;
            break;
        }
        

        buffer = PBM_getDataBuffer(hPkt) + PBM_getDataOffset(hPkt);
        length = PBM_getValidLen(hPkt);

	cpswTxPkts++;

        /**
         * Clean the cache for external addesses
         */
        if( 0 != ( ((Uint32)buffer) & EXTMEM ) )
        {     
            OEMCacheClean( (void *)buffer, length );
        }
        

        /**
         * Assign the pointer to "this" desc
         */
        pDescThis = txChan->pDescWrite;

        /**
         * Move the write pointer and bump count
         */
        if( pDescThis == txChan->pDescLast )
        {
            txChan->pDescWrite = txChan->pDescFirst;
        }
        else
        {
            txChan->pDescWrite = (CPPI_Desc*)(  ((Uint32)txChan->pDescWrite) +
                                                                            CPSW3G_SWBD_SIZE);
         }
        txChan->DescCount++;

        /**
         * Fill out the descriptor
         */
        pDescThis->pNext     = 0;
        pDescThis->pBuffer   = (Uint8*)(CPU_TO_HW((Uint32)buffer));
        pDescThis->BufOffLen = length;
        pDescThis->PktFlgLen = CPPI_DESC_FLAG_SOP | CPPI_DESC_FLAG_EOP |
                               length | CPPI_DESC_FLAG_OWNER;
        CACHE_FLUSH_BD(((Uint32)pDescThis),CPSW3G_SWBD_SIZE);

        /**
         * Enqueue this packet onto the desc queue
         */
        PBMQ_enq( &txChan->DescQueue, hPkt );

	#ifdef ETH_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
                PSP_DVT_DESC(PSP_DVT_eDATA_SND_START, \
                             PSP_DVT_dINST, \
                             PSP_DVT_dCHANNEL_ID, \
                             PSP_DVT_dNONE),0, 0);
        #endif


        /**
         * Synch the cache
         */
        if( 0u != ( (Uint32)buffer & EXTMEM) )
        {
            OEMCacheCleanSynch();
        }

        /**
         * Start it sending
         */
        if( txChan->DescCount > 1 )
        {
            /**
             * Transmitter is already running.
             * Make the previous buffer point to us
             */
            if( pDescThis == txChan->pDescFirst )
            {
                txChan->pDescLast->pNext = (CPPI_Desc*) (CPU_TO_HWBD((Uint32)pDescThis));
                CACHE_FLUSH_BD(((Uint32)txChan->pDescLast),CPSW3G_SWBD_SIZE);
            }
            else
            {
                CPPI_Desc *pPrevDesc;

                pPrevDesc = (CPPI_Desc*)(  ((Uint32)pDescThis) - CPSW3G_SWBD_SIZE);
                pPrevDesc->pNext = (CPPI_Desc*) (CPU_TO_HWBD((Uint32)pDescThis));
                CACHE_FLUSH_BD(((Uint32)pPrevDesc),CPSW3G_SWBD_SIZE);
            }
        }
        else
        {
            /**
             * Transmitter is not running, start it up
             */
            devPtr->regs->TX_HDP[chNum] =
                        CPU_TO_HWBD(((Uint32)pDescThis));
        }
    
    }while(1);

    cpsw3g_enable_interrupts();
}/* end of function HwPktTxNext() */


/**
 *--------------------------------------------------------------------
 * _HwPktPoll()
 *
 * Poll routine - CALLED OUTSIDE OF KERNEL MODE
 *
 * This function is called at least every 100ms, faster in a
 * polling environment. The fTimerTick flag is set only when
 * called on a 100ms event.
 *--------------------------------------------------------------------
 */
void _HwPktPoll( PDINFO *pi, Uint32 fTimerTick )
{
    Cpsw3gDevice * devPtr = (Cpsw3gDevice*)pi->hNetDev;
    Cpsw3gRxCppiCh * rxChan;
    CpdmaObj *dmaObj;
    Uint32 chNum;
    static Uint32 aleTickCount = 0;
    
    /** CHECK : in polling based environment , doesnt this add 100 ms latency */
    if (0u != fTimerTick) 
    {
        LED_TOGGLE( USER_LED2 );
        if(0u != FlashActiveLED )
        {
            FlashActiveLED = 0;
            LED_TOGGLE( USER_LED3 );
        }

	if( (0u != devPtr->aleTimerActive) && (0 != devPtr->aleTicks) )
	{
		/* aleTickCount can become greater if timeout period is changed in between */
		if( (++aleTickCount) >=  devPtr->aleTicks)
		{
			/* age out now */
			devPtr->regs->ALE_CONTROL |=CSL_CPSW3G_ALE_CONTROL_AGE_OUT_NOW_MASK;
			aleTickCount = 0;
		}
	}
        llEnter();
        cpsw3g_disable_interrupts();
        cpsw3g_Tick(devPtr);

        /* We only use one Rx DMA channel  now. If QOS/Priority has to be
         * supported, identify the channel number based on vlan/priority tag or
         * any other field from the header fields of the ethernet frame. */

         chNum = CPSW3G_DEFAULT_RX_CHANNEL;
         dmaObj = &devPtr->cpdma;
         rxChan = &dmaObj->rxCppi[chNum];
         
        /**
         * Re-fill Rx buffer queue if needed
         */
        if( rxChan->DescCount != rxChan->DescMax )
        {
            cpsw3g_Enqueue_Rx(rxChan,1);
        }

        /** Send any queued Tx Packets - packets that could not be enqueued
         * directly with H/w as we did not have BDs to enqueue */
         if( (pi->TxOn !=0) && (pi->TxFree == 0) )
         {
         	HwPktTxNext(pi);
         }

        cpsw3g_enable_interrupts();
        llExit();
    }
}/* end of function _HwPktPoll() */

#define PERFORM_ARGUMENT_RANGE_CHECKS
#ifdef PERFORM_ARGUMENT_RANGE_CHECKS
    #define RANGE_CHECK(value,max)  ((value) < (max))
#else
    #define RANGE_CHECK(value,max)
#endif
/**
 * ------------------------------------------------------------------
 *  HwPktIoctl()
 *
 * Low level driver Ioctl interface. This interface can be used for
 * ALE configuration,control,statistics 
 *-------------------------------------------------------------------
 */
Uint32 HwPktIoctl(PDINFO *pi, Uint32 cmd, void* param)
{
    Cpsw3gDevice *cpsw3gDevPtr;
    Uint32 vlanMemberList = 0,unregMcastFloodMask = 0;
    Uint32 regMcastFloodMask = 0,forceUntaggedEgress = 0,port = 0;
    Int32 ret_val = CPSW3G_SUCCESS;
    int(*printFunctionPtr)(const char*,...) = NULL;
    Uint8 add_802[6]={0x01,0x80,0xc2,0x0,0x0,0x03};


    cpsw3gDevPtr = (Cpsw3gDevice*) pi->hNetDev;
    if( (Cpsw3gDevice*)0 != cpsw3gDevPtr)
    {

        if(CPSW3G_IOCTL_SWITCH_CMD == cmd)
        {
	     SwitchConsCmd *switchConsCommand = (SwitchConsCmd*) param;
	     if( 1 != switchConsCommand->usePrintFn)
    	     {
	    	printFunctionPtr = ALE_DUMP_PRINT ;
    	     }
    	     else
             {
	    	printFunctionPtr = (int (*)(const char*,...))switchConsCommand->consolePrint;
    	     }
	     
            switch(switchConsCommand->opcode)
            {
                case CPSW3G_IOCTL_ALE_ADDMLT:
                    if(0u != cpsw3gCfg.cpswCfg.vlanAware)
                    {
                        ALE_VLANCMD *lpVlanCmd = &switchConsCommand->CmdType.AleVlanCmd;
                        if( (RANGE_CHECK(lpVlanCmd->vid,4095)&& 
                            RANGE_CHECK(lpVlanCmd->mem_port,8)) && 
				            (RANGE_CHECK(lpVlanCmd->flag,2)&& 
				            RANGE_CHECK(lpVlanCmd->untag_port,4)) )
				        {
                            ret_val = cpsw3g_AleAddMltAddr(lpVlanCmd->add,
                                           lpVlanCmd->vid,(Uint32)lpVlanCmd->mem_port,
                                           (Uint32)lpVlanCmd->flag,(Uint32)lpVlanCmd->untag_port);
                        }
				        else
                        {
                            printFunctionPtr("ALE_ADDMLT:Invalid Arguments\n");
                            /* set ret_val to indicate error */
                        } 

                    }
                    else
                    {
                        printFunctionPtr("ALE_ADDMLT: Vlan Unaware mode \n");
                    }
                    break;

                case CPSW3G_IOCTL_ALE_ADDUNI:
                    if(0u != cpsw3gCfg.cpswCfg.vlanAware)
                    {
                        ALE_VLANCMD *lpVlanCmd = &switchConsCommand->CmdType.AleVlanCmd;
                        if( ((RANGE_CHECK(lpVlanCmd->vid,4095)&&
                            RANGE_CHECK(lpVlanCmd->mem_port,3)) &&
                            (RANGE_CHECK(lpVlanCmd->blocked,2)&&
                            RANGE_CHECK(lpVlanCmd->secure,2)))&&
                            RANGE_CHECK(lpVlanCmd->ageable,2))
                        { 
                            cpsw3gDev.cpsw3gPort[lpVlanCmd->mem_port].portNum = 
                                               (int)lpVlanCmd->mem_port;
                            ret_val = cpsw3g_AleAddUniAddr(
                                        &cpsw3gDev.cpsw3gPort[(int)lpVlanCmd->mem_port],
                                        lpVlanCmd->add,lpVlanCmd->vid,
                                        lpVlanCmd->blocked,lpVlanCmd->secure,
                                        lpVlanCmd->ageable);
                        }
                        else
                        {
                            printFunctionPtr("ALE_ADDUNI:Invalid Arguments\n");
                            /* set ret_val to indicate error */
                        }
                    }
                    else
                    {
                        printFunctionPtr("ALE_ADDUNI:Vlan Unaware Mode\n");
                    }
                    break;

                case CPSW3G_IOCTL_ALE_ADDOUI:
                    ret_val = cpsw3g_AleAddOuiAddr(switchConsCommand->CmdType.AleVlanCmd.add);
                    break;

                case CPSW3G_IOCTL_ALE_ADDVLAN:
                    if(0u != cpsw3gCfg.cpswCfg.vlanAware)
                    {
                        ALE_VLANCMD *lpVlanCmd = &switchConsCommand->CmdType.AleVlanCmd;
                        if((0 < lpVlanCmd->vid) &&(lpVlanCmd->vid < 4095))
                        {
                            if( (RANGE_CHECK(lpVlanCmd->mem_port,8)&&
                                RANGE_CHECK(lpVlanCmd->unreg_port,8))  &&
                                (RANGE_CHECK(lpVlanCmd->regag_port,8)&&
                                RANGE_CHECK(lpVlanCmd->untag_port,8)) )
                            {
                                ret_val = cpsw3g_AleAddVlan(lpVlanCmd->vid,
                                            (Uint32)lpVlanCmd->mem_port,
                                            (Uint32)lpVlanCmd->unreg_port,
                                            (Uint32)lpVlanCmd->regag_port,
                                            (Uint32)lpVlanCmd->untag_port);
                            }
                            else
                            {
                                printFunctionPtr("ALE_ADDVLAN:Invalid Arguments\n");
                            }
                        }
                        else
                        {
                            printFunctionPtr("ALE_ADDVLAN:Invalid VID\n");
                        }

                    }
                    else
                    {
                        printFunctionPtr("ALE_ADDVLAN:Vlan Unaware Mode\n");
                    }  
                    break;

                case CPSW3G_IOCTL_ALE_FINDVLAN:
                    if(0u != cpsw3gCfg.cpswCfg.vlanAware)
                    {
                        ALE_VLANCMD *lpVlanCmd = &switchConsCommand->CmdType.AleVlanCmd;

                        if((0 < lpVlanCmd->vid) &&(lpVlanCmd->vid < 4095))  
                        {
                            ret_val =  cpsw3g_AleFindVlan(lpVlanCmd->vid,
                                            &vlanMemberList,&unregMcastFloodMask,
                                            &regMcastFloodMask,&forceUntaggedEgress);
                        }
                        else
                        {
                            printFunctionPtr("ALE_FINDVLAN:Invalid Arguments\n");
                        }
                    }
                    else
                    {
                        printFunctionPtr("ALE_FINDVLAN:Vlan Unaware Mode\n");
                    }

                    if(ret_val != -1)
                    {
                        printFunctionPtr("VLAN entry :%d found at Index :%d\n",
                                switchConsCommand->CmdType.AleVlanCmd.vid,ret_val);
                        printFunctionPtr("vlanMembers :%d unregMcastFloodMask :%d ",vlanMemberList,unregMcastFloodMask);
                        printFunctionPtr("regMcastFloodMask :%d forceUntaggedEgress :%d\n",regMcastFloodMask,forceUntaggedEgress); 
                    }
                    else
                    {
                        printFunctionPtr("ALE_FINDVLAN:VLAN entry with VID : %d not Found\n",
                                switchConsCommand->CmdType.AleVlanCmd.vid);
                    }
                    break;

                case CPSW3G_IOCTL_ALE_DELADDR:
                    if(0u != cpsw3gCfg.cpswCfg.vlanAware)
                    {
                        ALE_VLANCMD *lpVlanCmd = &switchConsCommand->CmdType.AleVlanCmd;
                        if(RANGE_CHECK(lpVlanCmd->vid,4095))
                        { 
                            ret_val = cpsw3g_AleDelAddress(lpVlanCmd->add,
                                             (Uint32)lpVlanCmd->vid);
                        }
                        else
                        {
                            printFunctionPtr("ALE_DELADDR:Invalid VID\n");
                        }

                    }
                    else
                    {
                        printFunctionPtr("ALE_DELADDR: Vlan Unaware Mode \n");
                    }
                    if(ret_val != -1)
                    {
                        printFunctionPtr("ALE_DELADDR: Entry deleted at index %d\n",ret_val);
                    }
                    else
                    {
                        Int32 i;

                        printFunctionPtr("Address:");
                        for(i=0;i<5;i++)
                        {
                            printFunctionPtr("%02x:",switchConsCommand->CmdType.AleVlanCmd.add[i]);
                        }
                        printFunctionPtr("%02x Not found\n",switchConsCommand->CmdType.AleVlanCmd.add[i]);
                    }

                    break;

                case CPSW3G_IOCTL_ALE_DELVLAN:
                    if(0u != cpsw3gCfg.cpswCfg.vlanAware)
                    {
                        ALE_VLANCMD *lpVlanCmd = &switchConsCommand->CmdType.AleVlanCmd;
                        
                        if((0 < lpVlanCmd->vid) &&(lpVlanCmd->vid < 4095)) 
                        {
                            ret_val = cpsw3g_AleDelVlan(lpVlanCmd->vid);
                        }
                        else
                        {
                            printFunctionPtr("ALE_DELVLAN:Invalid VID\n");
                        }
                    }
                    else
                    {
                        printFunctionPtr("ALE_DELVLAN:Vlan Unaware Mode\n");
                    }
                    
                    break;

                case CPSW3G_IOCTL_SET_PORT_VLAN_CONFIG:
                    if(0u != cpsw3gCfg.cpswCfg.vlanAware)
                    {
                        ALE_VLANCMD *lpVlanCmd = &switchConsCommand->CmdType.AleVlanCmd;
                        
                        if((0 < lpVlanCmd->vid) && (lpVlanCmd->vid < 4095))
                        {
                            if( (RANGE_CHECK(lpVlanCmd->mem_port,3)&&
                               RANGE_CHECK(lpVlanCmd->CFI_port,2)) &&
                               RANGE_CHECK(lpVlanCmd->prio_port,8))
                            {
                            cpsw3gDev.cpsw3gPort[(int)lpVlanCmd->mem_port].portNum =
                                                        (int)lpVlanCmd->mem_port;
                            cpsw3g_SetPortVlanCfg(
                                      &cpsw3gDev.cpsw3gPort[lpVlanCmd->mem_port],
                                      (Uint32)lpVlanCmd->prio_port,
                                      (Uint32)lpVlanCmd->CFI_port,
                                      (Uint32)lpVlanCmd->vid );
                            }
                            else
                            {
                                printFunctionPtr("SET_PORT_VLAN_CONFIG:Invalid Arguments\n");
                            }
                        }
                        else
                        {
                            printFunctionPtr("SET_PORT_VLAN_CONFIG:Invalid VID\n");
                        }
   
                    }
                    else
                    {
                        printFunctionPtr("SET_PORT_VLAN_CONFIG:Vlan Unaware Mode\n");
                    }
                    break;

                case CPSW3G_IOCTL_ALE_TIMEOUT:
                	/* aleTimeout input param expressed in Millisecs */
		    	if(0u == switchConsCommand->CmdType.AleVlanCmd.aleTimeout)
			{
				cpsw3gDevPtr->aleTicks = 0u;
			}
			else
			{
                		cpsw3gDevPtr->aleTicks = (switchConsCommand->CmdType.AleVlanCmd.aleTimeout / 100u) + 1u;
			}
                    break;
                case CPSW3G_IOCTL_ALE_DUMP:
		    	if(switchConsCommand->usePrintFn == 1)
			{
                	    cpsw3g_AleDump(cpsw3gDevPtr,switchConsCommand->consolePrint);
			}
			else
			{
			    cpsw3g_AleDump(cpsw3gDevPtr,NULL);
			}
                    break;
                case CPSW3G_IOCTL_ALE_FINDADDR:
                    if(0u != cpsw3gCfg.cpswCfg.vlanAware)
                    {
                        ALE_VLANCMD *lpVlanCmd = &switchConsCommand->CmdType.AleVlanCmd;

                        if(RANGE_CHECK(lpVlanCmd->vid,4095))
                        {     
                            ret_val = cpsw3g_AleFindAddress(lpVlanCmd->add,
                                        lpVlanCmd->vid);
                        }
                        else
                        {
                            printFunctionPtr("ALE_FINDADDR:Invalid VID\n");
                        }
                    }
                    else
                    {
                        printFunctionPtr("ALE_FINDADDR:Vlan Unaware Mode\n");
                    }
                    if(ret_val != -1)
                    {
                        printFunctionPtr("ALE_FINDADDR:Entry found at Index = %d\n",ret_val);
                    }
                    else
                    {
                        ALE_VLANCMD *lpVlanCmd = &switchConsCommand->CmdType.AleVlanCmd;
                        Int32 i;
                        
                        printFunctionPtr("ALE_FINDADDR:Address : ");
                        for(i=0;i<5;i++)
                        {
                            printFunctionPtr("%02x:",lpVlanCmd->add[i]);
                        }
                        printFunctionPtr("%02x not Found\n",lpVlanCmd->add[i]);
                    }

                    break;
                case CPSW3G_IOCTL_SET_SWITCH_FLOW_CONTROL:
                    {
                        PORT_CMD *lpPortCmd = &switchConsCommand->CmdType.PortCmd;
                        if( (RANGE_CHECK(lpPortCmd->port,3)&&
                            RANGE_CHECK(lpPortCmd->value,3))  &&
                            (RANGE_CHECK(lpPortCmd->threshold,17)&&  
                            RANGE_CHECK(lpPortCmd->EnDis,2))  )
                        {
                            cpsw3gDev.cpsw3gPort[lpPortCmd->port].portNum =
                                                  (int)lpPortCmd->port;
                            cpsw3gDev.cpsw3gPort[lpPortCmd->value].portNum =
                                                  lpPortCmd->value;
                            ret_val = cpsw3g_SetSwitchFlowControl(
                                        &cpsw3gDev.cpsw3gPort[lpPortCmd->port],
                                        &cpsw3gDev.cpsw3gPort[lpPortCmd->value],
                                        lpPortCmd->threshold,
                                        lpPortCmd->EnDis);
                        }
                        else
                        {
                            printFunctionPtr("SET_SWITCH_FLOW_CONTROL:Invalid Arguments\n");
                        }
                    }
                    break;

                case CPSW3G_IOCTL_SET_PRIORITY_MAPPING:
                    {
                        PRIO_CMD *lpPrioCmd = &switchConsCommand->CmdType.PrioCmd;
                        if( (RANGE_CHECK(lpPrioCmd->port,3)&&
                            RANGE_CHECK(lpPrioCmd->prio_rx,8))  &&
                            (RANGE_CHECK(lpPrioCmd->prio_tx,8)&&
                            RANGE_CHECK(lpPrioCmd->prio_switch,4))  )
                        { 
                            cpsw3gDev.cpsw3gPort[lpPrioCmd->port].portNum =
                                                    lpPrioCmd->port;
                            cpsw3g_SetPriorityMapping(
                                        &cpsw3gDev.cpsw3gPort[lpPrioCmd->port],
                                        (Uint32)lpPrioCmd->prio_rx,
                                        (Uint32)lpPrioCmd->prio_tx,
                                        (Uint32)lpPrioCmd->prio_switch);
                        }
                        else
                        {
                            printFunctionPtr("SET_PRIORITY_MAPPING:Invalid Arguments\n");
                        }
                    }
                    break;

                case CPSW3G_IOCTL_RATELIMIT:
                    {
                        PORT_CMD *lpPortCmd = &switchConsCommand->CmdType.PortCmd;
                        
                        if(RANGE_CHECK(lpPortCmd->EnDis,2)) 
                        {
                            if(0u != lpPortCmd->EnDis)
                            {
                                if( (RANGE_CHECK(lpPortCmd->port,3)&&
                                    RANGE_CHECK(lpPortCmd->ml,2))   &&
                                    (RANGE_CHECK(lpPortCmd->value,2)&&
                                    RANGE_CHECK(lpPortCmd->bl,256))  )
                                {
			                        /*For host port transmit/receive terminlogy is inverse of cpgmac port*/	
                                    if(lpPortCmd->port == 0x2)
                                    {
                                        if(0u != lpPortCmd->value)
                                        {
                                            cpsw3g_RateLimitRx(lpPortCmd->EnDis,
                                                        lpPortCmd->port,((typeOfData)lpPortCmd->ml),
                                                        lpPortCmd->bl);
                                        }
                                        else
                                        {
                                            cpsw3g_RateLimitTx(lpPortCmd->EnDis,
                                                               lpPortCmd->port,
                                                               ((typeOfData)lpPortCmd->ml),
                                                               lpPortCmd->bl);
                                         }
                                    }
                                    else
                                    {
                                        if(0u != lpPortCmd->value)
                                        {
                                            cpsw3g_RateLimitTx(lpPortCmd->EnDis,
                                                               lpPortCmd->port,
                                                               ((typeOfData)lpPortCmd->ml),
                                                               lpPortCmd->bl);
                                        }
                                        else
                                        {
                                            cpsw3g_RateLimitRx(lpPortCmd->EnDis,
                                                               lpPortCmd->port,
                                                               ((typeOfData)lpPortCmd->ml),
                                                               lpPortCmd->bl);
                                        }
                                    }
                                }
                                else
                                {
                                    printFunctionPtr("RATELIMIT:Invalid Arguments\n");
                                }
   
                            } 
                            else
                            { 
                                cpsw3g_RateLimitRx(lpPortCmd->EnDis,0,((typeOfData)0),0);  
                            }
                        }
                        else
                        {
                            printFunctionPtr("RATELIMIT:Invalid Arguments\n");
                        }
                    }
                    break;
                    
                case CPSW3G_IOCTL_VIDINGRESSCHECK:
                    {
                        PORT_CMD *lpPortCmd = &switchConsCommand->CmdType.PortCmd;

                        if( (RANGE_CHECK(lpPortCmd->bl,2)&& RANGE_CHECK(lpPortCmd->ml,2))
                                && RANGE_CHECK(lpPortCmd->port,3))
                        {
                            cpsw3gCfg.alePortCfg[lpPortCmd->port].dropUntagged
                                            = lpPortCmd->bl;
                            cpsw3gCfg.alePortCfg[lpPortCmd->port].vidIngressCheck
                                            = lpPortCmd->ml;
                            cpsw3g_SetAlePortCfg(&cpsw3gDev.cpsw3gPort[lpPortCmd->port]);
                                            
                        }
                        else
                        {
                            printFunctionPtr("VIDINGRESSCHECK:Invalid Arguments\n");
                        }

                    }
                    break;

                case CPSW3G_IOCTL_PORT_STATISTICS_ENABLE:
                    {
                        PORT_CMD *lpPortCmd = &switchConsCommand->CmdType.PortCmd;
                        if( (RANGE_CHECK(lpPortCmd->value,2)&&
                            RANGE_CHECK(lpPortCmd->ml,2))   &&
                            RANGE_CHECK(lpPortCmd->bl,2))
                        {  
                            cpsw3gCfg.cpswCfg.p0StatEn = lpPortCmd->value;
                            cpsw3gCfg.cpswCfg.p1StatEn = lpPortCmd->ml;
                            cpsw3gCfg.cpswCfg.p2StatEn = lpPortCmd->bl;
                            cpsw3g_statistics = (cpsw3gCfg.cpswCfg.p2StatEn << 2)|(cpsw3gCfg.cpswCfg.p1StatEn << 1)
                                                |cpsw3gCfg.cpswCfg.p0StatEn;
                            
                            if(cpsw3g_Control(CPSW3G_IOCTL_PORT_STATISTICS_ENABLE,(Ptr)cpsw3g_statistics,NULL)!=CPSW3G_SUCCESS)
                            {
                                printFunctionPtr("Error enabling statistics for port %d\n",lpPortCmd->value);
                            }
                        }
                        else
                        {
                            printFunctionPtr("PORT_STATISTICS_ENABLE:Invalid Arguments\n");
                        }
                    }

                    break;
                case CPSW3G_IOCTL_ADD_UNKNOWN_VLAN_INFO:
                    {
                        PORT_CMD *lpPortCmd = &switchConsCommand->CmdType.PortCmd;
                        if( (RANGE_CHECK(lpPortCmd->value,8)&&
                             RANGE_CHECK(lpPortCmd->port,8))  &&
                             (RANGE_CHECK(lpPortCmd->bl,8)&&
                             RANGE_CHECK(lpPortCmd->ml,8)) )
                        { 
                            cpsw3gCfg.aleCfg.unknownForceUntaggedEgress = lpPortCmd->value;
                            cpsw3gCfg.aleCfg.unknownRegMcastFloodMask = lpPortCmd->port;
                            cpsw3gCfg.aleCfg.unknownMcastFloodMask = lpPortCmd->bl;
                            cpsw3gCfg.aleCfg.unknownVlanMemberList = lpPortCmd->ml;
                            cpsw3gCfg.aleCfg.clearTable = 0;
                            cpsw3gCfg.aleCfg.ageOutNow = 0;
                            cpsw3g_SetAleCfg();
                        }
                        else
                        {
                            printFunctionPtr("ADD_UNKNOWN_VLAN_INFO:Invalid Arguments\n");
                        }

                    }
                    break;

                case CPSW3G_IOCTL_DUMP_CONFIG:
                    break;
                case CPSW3G_IOCTL_802_1:
                    {
                        PORT_CMD *lpPortCmd = &switchConsCommand->CmdType.PortCmd;
                        if(RANGE_CHECK(lpPortCmd->EnDis,2))
                        { 
                            if(0u != lpPortCmd->EnDis)
                            {
					            cpsw3gCfg.aleCfg.ageOutNow = 1;
                                cpsw3gCfg.aleCfg.clearTable = 0;
                                cpsw3g_SetAleCfg();
                                cpsw3g_AleAddMltAddr(add_802,0,0x7,0,0x3);
                            }
                            else 
                            {
					            cpsw3gCfg.aleCfg.ageOutNow = 0;
                                cpsw3gCfg.aleCfg.clearTable = 0;
                                cpsw3g_SetAleCfg();
                                cpsw3g_AleAddMltAddr(add_802,0,0x3,0,0x3);
                            }
                        }
                        else
                        {
                            printFunctionPtr("Invalid Arguments\n");
                        }
                    }

                    break;
                case CPSW3G_IOCTL_RESET:
                    break;
                case CPSW3G_IOCTL_SET_PORT_CONFIG:
                    {
                        PORT_CMD *lpPortCmd = &switchConsCommand->CmdType.PortCmd;
                        Uint32 fault = 0u;

                        if( (RANGE_CHECK(lpPortCmd->port,2)&&
                            RANGE_CHECK(lpPortCmd->value,1001))  && 
                            RANGE_CHECK(lpPortCmd->EnDis,2))
                        {
                            switch(lpPortCmd->value)
                            {
                                case 0  :  
                                    if(cpsw3gDev.cpgmacMac[lpPortCmd->port].intPhy == 0)
                                    {
                                        cpsw3gCfg.macInitCfg[lpPortCmd->port].phyMode =
                                                        SNWAY_AUTOALL;                                                                                                
                                    }
                                    else
                                    {
                                        cpsw3gCfg.macInitCfg[lpPortCmd->port].phyMode =
                                            SNWAY_FD100|SNWAY_FD10|SNWAY_HD100|SNWAY_HD10|SNWAY_AUTO;
                                    }
                                    break; 

                                case 10 : 
                                    if(lpPortCmd->EnDis ==1)
                                    {
                                        cpsw3gCfg.macInitCfg[lpPortCmd->port].phyMode =
                                              SNWAY_FD10;
                                    }
                                    else
                                    {
                                        if(lpPortCmd->EnDis ==0)   
                                        {
                                            cpsw3gCfg.macInitCfg[lpPortCmd->port].phyMode =
                                                SNWAY_HD10;
                                        }
                                        else
                                        {
                                            cpsw3gCfg.macInitCfg[lpPortCmd->port].phyMode = 
                                                SNWAY_HD10|SNWAY_FD10;
                                        }
                                    }
                                    break;

                                case 100 : 
                                    if(lpPortCmd->EnDis ==1)
                                    {
                                        cpsw3gCfg.macInitCfg[lpPortCmd->port].phyMode =
                                                SNWAY_FD100;
                                    }
                                    else
                                    {
                                        if(lpPortCmd->EnDis ==0)
                                        {
                                            cpsw3gCfg.macInitCfg[lpPortCmd->port].phyMode =
                                                SNWAY_HD100;
                                        }
                                        else
                                        {
                                            cpsw3gCfg.macInitCfg[lpPortCmd->port].phyMode =
                                                SNWAY_HD100|SNWAY_FD100;
                                        }
                                    }
                                    break;

                                case 1000: 
                                    if(0u == cpsw3gDev.cpgmacMac[lpPortCmd->port].intPhy)                
                                    {
                                        cpsw3gCfg.macInitCfg[lpPortCmd->port].phyMode = 
                                              SNWAY_FD1000;
                                        if(lpPortCmd->EnDis ==0)      
                                        {
                                            printFunctionPtr("Half duplex is not supported with 1000Mbps\n");
                                            fault = 1u;
                                        }
                                    }
                                    else
                                    {
                                        printFunctionPtr("1000Mbps is not supported with internal phy\n");
                                        fault = 1u;
                                    }
                                    
                                    break;
				default:
				    break;
                            }/* end of switch construct  */
                            
                            if(0u == fault)
                            {
                                cpsw3g_SetPhyMode(&cpsw3gDev.cpgmacMac[lpPortCmd->port],lpPortCmd->port); 
                            }
                        }
                        else
                        {
                            printFunctionPtr("SET_PORT_CONFIG:Invalid Arguments\n");
                        } 

                    }
                    break;
                case CPSW3G_IOCTL_GET_PORT_CONFIG:
                    {
                        PORT_CMD *lpPortCmd = &switchConsCommand->CmdType.PortCmd;

                        if(RANGE_CHECK(lpPortCmd->port,2))
                        {
                            port  = lpPortCmd->port;
                            lpPortCmd->ml    = port;      
                            lpPortCmd->port  = cpsw3gDev.cpgmacMac[port].PhyLinked; 
                            lpPortCmd->value = cpsw3gDev.linkSpeed[port];
                            lpPortCmd->EnDis = cpsw3gDev.linkMode[port];
                        }
                        else
                        {
                            printFunctionPtr("GET_PORT_CONFIG:Invalid Arguments\n");
                        }
                    }
                    break;

                case CPSW3G_IOCTL_STP_CONFIG:
                    {
                        PORT_CMD *lpPortCmd = &switchConsCommand->CmdType.PortCmd;
                        if(RANGE_CHECK(lpPortCmd->port,3)&&
                            RANGE_CHECK(lpPortCmd->value,4))
                        {
                        cpsw3gCfg.alePortCfg[lpPortCmd->port].portState =
                                        lpPortCmd->value;
                        cpsw3g_SetAlePortCfg(&cpsw3gDev.cpsw3gPort[lpPortCmd->port]);
                        }
                        else
                        {
                            printFunctionPtr("STP_CONFIG:Invalid Arguments\n");
                        }
                    }

                    break;

                default:
                    break;

            }/* end of switch construct */

            switchConsCommand->ret_type = ret_val;

        }
	else 
	    {
	    if (CPSW3G_IOCTL_STAT_CMD ==   cmd)
	{
		/* handle STATs related IOCTL here */
		StatsCmd *statsCommand = (StatsCmd*) param;
		volatile Uint32  * statRegs;
		Uint32 i;
		volatile Uint32 statClearVal,portEnable;

		statRegs = (Uint32*) &cpsw3gDevPtr->regs->RXGOODFRAMES;
		portEnable = cpsw3gDevPtr->regs->CPSW_STAT_PORT_EN;
		
		switch(statsCommand->opcode)
		{
			case CPSW3G_IOCTL_STAT_CLEAR:
				/* if atleast one port is enabled, all values are write to clear hence 
			  	* we write all FFs to clear, else write zero to set value */
				if (portEnable == 0) 
				{
					statClearVal = 0 ;
				}
				else
				{
					statClearVal = CPSW3G_STAT_CLEAR;
				}

    				for(i = 0; i < CPSW3G_NUM_STAT_REGS; i++)
            			{
              			*statRegs = statClearVal; 
				statRegs++;
            			}
    		
    				/* update our s/w stat counters also to zero */
    				mmZeroInit(&cpsw3gDevPtr->statsObj,sizeof(cpsw3gDevPtr->statsObj));		
				break;

				
			case CPSW3G_IOCTL_STAT_GET:
				{
					STATCTR * statFieldPtr = (STATCTR*) (&cpsw3gDevPtr->statsObj.ifInGoodFrames);
					volatile Uint32 temp;
					STATCTR * retFieldPtr = (STATCTR*)(&statsCommand->statsObj.ifInGoodFrames);
					
					for(i=0;i < CPSW3G_NUM_STAT_REGS;i++)
					{
						/* read 32-bit H/w stat counter */
						temp = (*statRegs);
						/* update our book keeping counter */
						(*statFieldPtr) = (*statFieldPtr) + temp ;
						
						/* here we dont check whether port enable bit is set
						atleast one port must be enabled, else count would be
						zero anyway as clear on every port enable state change */
						*statRegs  = temp;

						/* Copy the counter to user output param */
						(*retFieldPtr) = (*statFieldPtr) ;

						/* Advance to next element of stats struct */
						statFieldPtr++;
						retFieldPtr++;
						
						statRegs++;
						
					}
				
				}
				break;
			default:
				break;
		    }/*end of switch construct */	
		}/* end of if command is CPSW3G_IOCTL_STAT_CMD  */
	}
        

    }

    return 0;
}/* end of function HwPktIoctl */



/**
 *--------------------------------------------------------------------
 * HwInt()
 *
 * EMAC Hardware Interrupt
 *--------------------------------------------------------------------
 */
static void HwInt(Arg param)
{
    PDINFO * pdPtr = (PDINFO*)param;
    Cpsw3gDevice * devPtr = (Cpsw3gDevice*) pdPtr->hNetDev;
    CpdmaNetStatus status;
    Uint32 vector,channel=0;

    #ifdef ETH_INSTRUMENTATION_ENABLED
    PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
            PSP_DVT_DESC(PSP_DVT_eINT_START, \
                         PSP_DVT_dNONE, \
                         PSP_DVT_dNONE, \
                         PSP_DVT_dNONE));
    #endif

    vector = devPtr->regs->CPDMA_IN_VECTOR;

   
   /* disable peripheral interrupt and re-enable at the end */

   /** CHECK: HAndle MDIO User/Link,Stats,Rx Thresh interrupt also here */

    /* Check host interrupts */
    status.errPending = CSL_FEXT(vector,CPSW_MACINVECTOR_HOSTPEND);
    if (0u != status.errPending)
    {
        /**
          * A fatal error can only be caused by a software
          * logic error. We'll read the fatal error code
          * and exit with the EMAC interrupt still disabled.
          * This will halt all network functionality. The
          * application programmer can manually check the
          * status of CPSW3G_FatalError if the network
           * stops working.
           */
            CPSW3G_FatalError = devPtr->regs->DMASTATUS;
	     /*ack by writing to EOI */
  	    devPtr->regs->CPDMA_EOI_VECTOR = 0x3u;  	    
            return;
  	}

    /* Check Statistics Pending interrupt */
	status.statPending = CSL_FEXT(vector,CPSW_MACINVECTOR_STATPEND );
    	if(0u != status.statPending)
    	{
    		STATCTR * statFieldPtr = (STATCTR*) (&devPtr->statsObj.ifInGoodFrames);
		volatile Uint32 temp;
		Uint32 i;
		volatile Uint32  * statRegs = (Uint32*) &devPtr->regs->RXGOODFRAMES;
		

    		/* Update S/w stats counter here, write to decrement HW counter */
		for(i=0;i < CPSW3G_NUM_STAT_REGS;i++)
		{
			/* read 32-bit H/w stat counter */
			temp = (*statRegs);
			/* update our book keeping counter */
			(*statFieldPtr) = (*statFieldPtr) + temp ;
						
			/* here we dont check whether port enable bit is set
			atleast one port must be enabled, else count would be
			zero anyway as clear on every port enable state change */
			*statRegs  = temp;

			/* Advance to next element of stats struct */
			statFieldPtr++;

			statRegs++;
		}

		/*ack by writing to EOI */
  	    	devPtr->regs->CPDMA_EOI_VECTOR = 0x3u;  	    
    	}
    

  	/*Check Tx Pending */
  	status.txChannel = CSL_FEXT(vector,CPSW_MACINVECTOR_TXPEND);
  	channel = 0;
  	while(0u != status.txChannel)
  	    {
  	        if(0u != (status.txChannel & 0x1) )
  	        {
  	            Cpsw3gTxCppiCh *txChan;
  	            CPPI_Desc *pDesc;
  	       		
    		      txChan = &devPtr->cpdma.txCppi[channel];
    		    pDesc = (CPPI_Desc*)devPtr->regs->TX_CP[channel] ;
		    devPtr->regs->TX_CP[channel] = (Uint32)pDesc ;
    		      cpsw3g_Dequeue_Tx(txChan , pDesc);
  	        }
  	        channel++;
  	        status.txChannel >>= 1;
  	    }

  	if(channel !=0 )
  	{
  	    /* we  processed atleast one channel, ack by writing to EOI */
  	    devPtr->regs->CPDMA_EOI_VECTOR = 0x2u;  	    
  	}

	/* Check Rx Pending */
	status.rxChannel = CSL_FEXT(vector,CPSW_MACINVECTOR_RXPEND);
  	channel = 0;
  	while(0u != status.rxChannel)
  	    {
  	        if(0u != (status.rxChannel & 0x1) )
  	        {
  	            Cpsw3gRxCppiCh *rxChan;
  	            CPPI_Desc *pDesc;
  	       		
    		      rxChan = &devPtr->cpdma.rxCppi[channel];
    		    pDesc = (CPPI_Desc*)devPtr->regs->RX_CP[channel] ;
		    devPtr->regs->RX_CP[channel] = (Uint32)pDesc ;
    		      cpsw3g_Dequeue_Rx(rxChan , pDesc);
  	        }
  	        channel++;
  	        status.rxChannel >>= 1;
  	    }

  	if(channel !=0u )
  	{
  	    /* we  processed atleast one channel, ack by writing to EOI */
  	    devPtr->regs->CPDMA_EOI_VECTOR = 0x1u;  	    
  	}

	#ifdef ETH_INSTRUMENTATION_ENABLED
    	PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
            PSP_DVT_DESC(PSP_DVT_eINT_END, \
                         PSP_DVT_dNONE, \
                         PSP_DVT_dNONE, \
                         PSP_DVT_dNONE));
    	#endif
    
}/* end of function HwInt() */


/*-------------------------------------------------------------------
 * cpsw3g_open()
 *
 * Program the cpsw3g H/w.
 *
 *------------------------------------------------------------------*/
static Uint32 cpsw3g_open(Cpsw3gDevice *devPtr)
{
    Int32 i;

    /* clear set to close flag*/
    devPtr->setToClose = 0;
    devPtr->timerActive = FALSE;

    /* Open all ports */
    for(i=0;i<CPSW3G_NUM_PORTS; i++)
    {
	    devPtr->cpsw3gPort[i].portNum = i;
	    devPtr->cpsw3gPort[i].chNum = i;
	    cpsw3g_port_open(&devPtr->cpsw3gPort[i]);
    }/* end of for loop over PORTS */

    /* Add vlan entry into Ale for default PORT VID*/
    if(0u != cpsw3gCfg.cpswCfg.vlanAware)
    {
        /* Modified Unreg_multicast_mask from 0x7 to 0x3 to exclude CPU port*/
        cpsw3g_AleAddVlan(CPSW3G_DEFAULT_MAC_PORTVID,CPSW3G_DEFAULT_UNK_VLANMEMLIST,
                3,7,CPSW3G_DEFAULT_UNK_FORCE_UNTAG_EGR); 
    }

    /* Kick start transfer queues here */
    /* Transmission should be started if we have link on port 0/1 */

    /* Add ALE entry for CPU port.i.e.Port 2 */
    cpsw3g_AleAddUniAddr(&devPtr->cpsw3gPort[2],devPtr->macAddr,0,0,1,0);

    /* Activate the ALE ageout timer here */
    devPtr->aleTimerActive = 1;

    return 0;

}/* end of function cpsw3g_open() */

/*-------------------------------------------------------------------
 * cpsw3g_close()
 *
 * Program the cpsw3g H/w.
 *
 *------------------------------------------------------------------*/
static Uint32 cpsw3g_close(Cpsw3gDevice *devPtr)
{
    Int32 i;

    /* clear set to close flag*/
    devPtr->setToClose = 1;
    devPtr->timerActive = FALSE;


    /* set flag indicating Transmission queue ie being shut off */

    /* Close all ports */
    for(i=0;i<CPSW3G_NUM_PORTS; i++)
    {
	    cpsw3g_port_close(&devPtr->cpsw3gPort[i]);
    }/* end of for loop over PORTS */

    /* De-Activate the ALE ageout timer here */
    devPtr->aleTimerActive = 0;
    
    return 0;

}/* end of function cpsw3g_open() */

/**
 *--------------------------------------------------------------------
 * cpsw3g_get_cfg()
 *
 * Populate the initialization/configuration structures with Init data
 *--------------------------------------------------------------------
 */
static void cpsw3g_get_cfg()
{
    Cpsw3gInitConfig *iCfg = 0;
    Int32 speed0=0,duplex0=0,speed1=0,duplex1=0;
    Uint32 macAddrHi[2],macAddrLo[2];
    Int32 i;

    /** Allocate Switch internal structure here - using static for now */
    mmZeroInit(&cpsw3gDev,sizeof(Cpsw3gDevice));
    mmZeroInit(&cpsw3gCfg,sizeof(Cpsw3gInitConfig));

    /* Initialize the cfg ptr to the static cpsw3gCfg struct */
    cpsw3gDev.initCfg = &cpsw3gCfg;

    /** Populate the default configuration params*/
    CPSW3G_getConfig( cpsw3gDev.macAddr, &IntVector);

    /* populate macaddr_lo,macaddr_hi.Same Addr for both Ports */
    macAddrHi[0] =(cpsw3gDev.macAddr[3]<< 24) |(cpsw3gDev.macAddr[2]<< 16) 
		    |(cpsw3gDev.macAddr[1]<< 8) |cpsw3gDev.macAddr[0];
    macAddrLo[0] =(cpsw3gDev.macAddr[5]<< 8)  | cpsw3gDev.macAddr[4];

    macAddrHi[1] =(cpsw3gDev.macAddr[3]<< 24) |(cpsw3gDev.macAddr[2]<< 16) 
		    |(cpsw3gDev.macAddr[1]<< 8) |cpsw3gDev.macAddr[0];
    macAddrLo[1] =(cpsw3gDev.macAddr[5]<< 8)  | cpsw3gDev.macAddr[4];

    cpsw3gDev.cpgmacMac[0].intPhy = 0;
    cpsw3gDev.cpgmacMac[1].intPhy = 0;

    cpsw3gDev.cpgmacMac[0].broadcastEnable = CPSW3G_INITPARAM_BROADEN0 ;
    cpsw3gDev.cpgmacMac[0].multicastEnable = CPSW3G_INITPARAM_MULTIEN0;

    cpsw3gDev.cpgmacMac[1].broadcastEnable = CPSW3G_INITPARAM_BROADEN1 ;
    cpsw3gDev.cpgmacMac[1].multicastEnable = CPSW3G_INITPARAM_MULTIEN1;
    
    iCfg = cpsw3gDev.initCfg;
    iCfg->dmaInitCfg.numChannels[NET_CH_DIR_TX] = CPSW3G_INITPARAM_TXNUMCHAN;
    iCfg->dmaInitCfg.numChannels[NET_CH_DIR_RX] = CPSW3G_INITPARAM_RXNUMCHAN;

    cpsw3gDev.cpgmacMac[0].promiscousEnable = CPSW3G_INITPARAM_PROMEN;
    iCfg->cpswCfg.vlanAware = CPSW3G_INITPARAM_VLANAWARE;

    for(i=0; i<iCfg->dmaInitCfg.numChannels[NET_CH_DIR_TX]; i++)
    {
        Cpsw3gChInfo *txChCfg = &cpsw3gDev.chInfo[NET_CH_DIR_TX][i];
        
         txChCfg->chNum = i;
         txChCfg->chDir = NET_CH_DIR_TX;
         txChCfg->numBD =CPSW3G_INITPARAM_TXNUMBDS;
    }
    
     for(i=0; i<iCfg->dmaInitCfg.numChannels[NET_CH_DIR_RX]; i++)
     {
        Cpsw3gChInfo *rxChCfg = &cpsw3gDev.chInfo[NET_CH_DIR_RX][i];

        rxChCfg->chNum = i;
        rxChCfg->chDir = NET_CH_DIR_RX;
        rxChCfg->bufSize = 1522;/**CHECK**/
        rxChCfg->numBD = CPSW3G_INITPARAM_RXNUMBDS;
        
      }

    /* parse the speed/duplex params*/
    if(0u == use_sma_port0)
    {
	/* use auto-neg */
    speed0 = CPSW3G_INITPARAM_SPEED0;
    }
    else
    {
	/* Always linked state*/
	speed0 = CONFIG_CPGMAC_NOPHY;
    }
    if(0u == use_sma_port1)
    {
	/* use auto-neg */
	speed1 = CPSW3G_INITPARAM_SPEED0;
    }
    else
    {
	/* Always linked state */
	speed1 = CONFIG_CPGMAC_NOPHY;
    }

    duplex0 = CPSW3G_INITPARAM_DUPLEX0;
    duplex1 = CPSW3G_INITPARAM_DUPLEX1;

    if (speed0 == CONFIG_CPGMAC_NOPHY)
      {
          iCfg->macInitCfg[0].phyMode = SNWAY_NOPHY;
      }
      else
      {
          if ((speed0 == 0) && (duplex0 == 0))
          {
		/** CHECK: */
                if(cpsw3gDev.cpgmacMac[0].intPhy == 0)
		    {
                   iCfg->macInitCfg[0].phyMode = SNWAY_AUTOALL;  /* Auto detection */
		    }
                else 
		    {
                   iCfg->macInitCfg[0].phyMode = SNWAY_FD100|SNWAY_FD10|SNWAY_HD100|SNWAY_HD10|SNWAY_AUTO;
          }
          }
          else if (speed0 == 10)
          {
              if (duplex0 == 1) { iCfg->macInitCfg[0].phyMode = SNWAY_HD10; }
              else if (duplex0 == 2) {iCfg->macInitCfg[0].phyMode = SNWAY_FD10; }
              else {iCfg->macInitCfg[0].phyMode = SNWAY_HD10 | SNWAY_FD10; }
          }
          else if (speed0 == 100)
          {
              if (duplex0 == 1) {iCfg->macInitCfg[0].phyMode = SNWAY_HD100; }
              else if (duplex0 == 2) {iCfg->macInitCfg[0].phyMode = SNWAY_FD100; }
              else {iCfg->macInitCfg[0].phyMode = SNWAY_HD100 | SNWAY_FD100; }
          }
          else if(speed0 == 1000)
          {   
              iCfg->macInitCfg[0].phyMode = SNWAY_FD1000; 
          }
          else
          {
              if (duplex0 == 1) {iCfg->macInitCfg[0].phyMode = SNWAY_HD10 | SNWAY_HD100 ; }
              else {iCfg->macInitCfg[0].phyMode = SNWAY_FD10 | SNWAY_FD100; }
          }
      }

     if(speed1 == CONFIG_CPGMAC_NOPHY)
      {
          iCfg->macInitCfg[1].phyMode = SNWAY_NOPHY;
      }
      else
      {
          if ((speed1 == 0) && (duplex1 == 0))
          {
		if(cpsw3gDev.cpgmacMac[1].intPhy == 0)
		    {
                 iCfg->macInitCfg[1].phyMode = SNWAY_AUTOALL;  /* Auto detection */
		    }
              else
		    {
                 iCfg->macInitCfg[1].phyMode = SNWAY_FD100|SNWAY_FD10|SNWAY_HD100|SNWAY_HD10|SNWAY_AUTO;
		    }

          }
          else if (speed1 == 10)
          {
              if (duplex1 == 1) { iCfg->macInitCfg[1].phyMode = SNWAY_HD10; }
              else if (duplex1 == 2) {iCfg->macInitCfg[1].phyMode = SNWAY_FD10; }
              else {iCfg->macInitCfg[1].phyMode = SNWAY_HD10 | SNWAY_FD10; }
          }
          else if (speed1 == 100)
          {
              if (duplex1 == 1) {iCfg->macInitCfg[1].phyMode = SNWAY_HD100; }
              else if (duplex1 == 2) {iCfg->macInitCfg[1].phyMode = SNWAY_FD100; }
              else {iCfg->macInitCfg[1].phyMode = SNWAY_HD100 | SNWAY_FD100; }
          }
          else if(speed1 ==1000)
          {
             iCfg->macInitCfg[1].phyMode = SNWAY_FD1000; 
          }
          else
          {
             if (duplex1 == 1) {iCfg->macInitCfg[1].phyMode = SNWAY_HD10 | SNWAY_HD100 ; }
             else {iCfg->macInitCfg[1].phyMode = SNWAY_FD10 | SNWAY_FD100; }
          }
       }


      /* Program default MAC SL config */
      for (i=0; i < CPSW3G_NUM_MAC_PORTS; i++)
      {
	      if(0 == i)
	      {
		/* program phy_mask for port 0 */
		iCfg->macInitCfg[i].PhyMask = 0x1;/*corresponding to Addr=0*/
		iCfg->macInitCfg[i].MLinkMask = 0x0;
	      }
	      else
	      {
		/* program phy_mask for port 1 */
		iCfg->macInitCfg[i].PhyMask = 0x2;/*corresponding to address 1 */
		iCfg->macInitCfg[i].MLinkMask = 0x0;
	      }/* end of else block covering settings for port 1 */


	  iCfg->macInitCfg[i].sourceAddressHi = macAddrHi[i];
         iCfg->macInitCfg[i].sourceAddressLo = macAddrLo[i];
          iCfg->macInitCfg[i].portPri = CPSW3G_DEFAULT_MAC_PORTPRI;
          iCfg->macInitCfg[i].portCfi = CPSW3G_DEFAULT_MAC_PORTCFI;
          iCfg->macInitCfg[i].portVID = CPSW3G_DEFAULT_MAC_PORTVID;

          if(0u != iCfg->cpswCfg.vlanAware)
	    {
             iCfg->macInitCfg[i].rxMaxlen = CPSW3G_DEFAULT_RXMAXLEN + 4u;
	    }
          else
	    {
             iCfg->macInitCfg[i].rxMaxlen = CPSW3G_DEFAULT_RXMAXLEN;
	    }

          iCfg->macInitCfg[i].passCrc = CPSW3G_DEFAULT_PASSCRC;
          iCfg->macInitCfg[i].rxCmfEn = CPSW3G_DEFAULT_RXCMFEN;
          iCfg->macInitCfg[i].rxCsfEn = CPSW3G_DEFAULT_RXCSFEN;
          iCfg->macInitCfg[i].rxCefEn = CPSW3G_DEFAULT_RXCEFEN;
	  
	  iCfg->macInitCfg[i].extEn = CPSW3G_DEFAULT_EXTEN;
	  iCfg->macInitCfg[i].gigForce =CPSW3G_DEFAULT_GIGFORCE;
          iCfg->macInitCfg[i].ifCtlB = CPSW3G_DEFAULT_IFCTLB;
          iCfg->macInitCfg[i].ifCtlA = CPSW3G_DEFAULT_IFCTLA;
	  iCfg->macInitCfg[i].gigaBitEnable = CPSW3G_DEFAULT_GIGABITEN;
	  iCfg->macInitCfg[i].gmiiEnable= CPSW3G_DEFAULT_GMIIEN;
	  iCfg->macInitCfg[i].fullduplex = CPSW3G_DEFAULT_FD;

          iCfg->macInitCfg[i].cmdIdle = CPSW3G_DEFAULT_MAC_CMDIDLE;
          iCfg->macInitCfg[i].txShortGapEnable = CPSW3G_DEFAULT_TXSHORTGAPEN;
          iCfg->macInitCfg[i].txPacingEnable = CPSW3G_DEFAULT_TXPACINGEN;
          iCfg->macInitCfg[i].txFlowEnable = CPSW3G_DEFAULT_TXFLOWEN;
          iCfg->macInitCfg[i].rxFlowEnable = CPSW3G_DEFAULT_RXFLOWEN;
          iCfg->macInitCfg[i].loopbackEnable = CPSW3G_DEFAULT_LOOPBKEN;
          iCfg->macInitCfg[i].internalPhy = cpsw3gDev.cpgmacMac[i].intPhy;

      }/* end of for loop over CPSW3G_NUM_MAC_PORTS */



      /* Program Default Dma COnfig */
       iCfg->dmaInitCfg.portPri = CPSW3G_DEFAULT_DMA_PORTPRI;
       iCfg->dmaInitCfg.portCfi = CPSW3G_DEFAULT_DMA_PORTCFI;
       iCfg->dmaInitCfg.portVID = CPSW3G_DEFAULT_DMA_PORTVID;
       iCfg->dmaInitCfg.rxBufferOffset = CPSW3G_DEFAULT_RXBUFOFF;
       iCfg->dmaInitCfg.rxCef = CPSW3G_DEFAULT_RXCEF;
       iCfg->dmaInitCfg.cmdIdle = CPSW3G_DEFAULT_CMDIDLE;
       iCfg->dmaInitCfg.rxOfflenBlock = CPSW3G_DEFAULT_RXOFFLEN;
       iCfg->dmaInitCfg.rxOwnership = CPSW3G_DEFAULT_RXOWNERSHIP;
       iCfg->dmaInitCfg.txPtype = CPSW3G_DEFAULT_TXPTYPE;
       iCfg->dmaInitCfg.hostRxCh_gmac0Pri0 = CPSW3G_DEFAULT_SL0_PRI0;
       iCfg->dmaInitCfg.hostRxCh_gmac0Pri1 = CPSW3G_DEFAULT_SL0_PRI1;
       iCfg->dmaInitCfg.hostRxCh_gmac0Pri2 = CPSW3G_DEFAULT_SL0_PRI2;
       iCfg->dmaInitCfg.hostRxCh_gmac0Pri3 = CPSW3G_DEFAULT_SL0_PRI3;
       iCfg->dmaInitCfg.hostRxCh_gmac1Pri0 = CPSW3G_DEFAULT_SL1_PRI0;
       iCfg->dmaInitCfg.hostRxCh_gmac1Pri1 = CPSW3G_DEFAULT_SL1_PRI1;
       iCfg->dmaInitCfg.hostRxCh_gmac1Pri2 = CPSW3G_DEFAULT_SL1_PRI2;
       iCfg->dmaInitCfg.hostRxCh_gmac1Pri3 = CPSW3G_DEFAULT_SL1_PRI3;

       /* Program Default ALE Configuration */
       iCfg->aleCfg.enableAle = CPSW3G_DEFAULT_ENABLEALE;
       iCfg->aleCfg.clearTable = CPSW3G_DEFAULT_CLEARTABLE;
       iCfg->aleCfg.ageOutNow = CPSW3G_DEFAULT_AGEOUTNOW;
       iCfg->aleCfg.learnNoVid = CPSW3G_DEFAULT_LEARNNOVID;
       iCfg->aleCfg.enVid0Mode = CPSW3G_DEFAULT_ENVID0MODE;
       iCfg->aleCfg.enableOuiDeny = CPSW3G_DEFAULT_ENABLEOUIDENY;
       iCfg->aleCfg.rateLimitTx = CPSW3G_DEFAULT_RATELIMITTX;
       iCfg->aleCfg.aleVlanAware = CPSW3G_DEFAULT_ALEVLANAWARE;
       iCfg->aleCfg.enableAuthMode = CPSW3G_DEFAULT_ENAUTHMODE;
       iCfg->aleCfg.enableRateLimit = CPSW3G_DEFAULT_EnRATELIMIT;
       iCfg->aleCfg.unknownForceUntaggedEgress =  CPSW3G_DEFAULT_UNK_FORCE_UNTAG_EGR;
       iCfg->aleCfg.unknownRegMcastFloodMask = CPSW3G_DEFAULT_UNK_REGMCAST_FLOODMASK;
       iCfg->aleCfg.unknownMcastFloodMask = CPSW3G_DEFAULT_UNK_MCAST_FLOODMASK;
       iCfg->aleCfg.unknownVlanMemberList = CPSW3G_DEFAULT_UNK_VLANMEMLIST;

       /** CHECK: Set this to correct value.Set Ratelimit counter for 1msec*/
       iCfg->aleCfg.alePrescale =  CPSW3G_DEFAULT_ALEPRESCALE;


       /* Ale Port Config */
       for(i=0; i < CPSW3G_NUM_PORTS; i++)
       {
          iCfg->alePortCfg[i].bcastLimit = CPSW3G_DEFAULT_BCASTLIMIT;
          iCfg->alePortCfg[i].mcastLimit = CPSW3G_DEFAULT_MCASTLIMIT;
          iCfg->alePortCfg[i].noLearn =  CPSW3G_DEFAULT_LEARN;
          iCfg->alePortCfg[i].portState = CPSW3G_DEFAULT_PORTSTATE;
          iCfg->alePortCfg[i].vidIngressCheck = CPSW3G_DEFAULT_VIDINGRESSCHECK;
          iCfg->alePortCfg[i].dropUntagged = CPSW3G_DEFAULT_DROPUNTAGGED;
       }
       /*CPSW3G default config*/
       iCfg->cpswCfg.p2PassPriTagged = CPSW3G_DEFAULT_P2_PASSPRIOTAG;
       iCfg->cpswCfg.p1PassPriTagged = CPSW3G_DEFAULT_P1_PASSPRIOTAG;
       iCfg->cpswCfg.p0PassPriTagged = CPSW3G_DEFAULT_P0_PASSPRIOTAG;
       iCfg->cpswCfg.rxVlanEncap = 0;
       iCfg->cpswCfg.p2P2RxFlowEn = CPSW3G_DEFAULT_P2_P2RXFLOWEN;
       iCfg->cpswCfg.p2P1TxFlowEn = CPSW3G_DEFAULT_P2_P1TXFLOWEN;
       iCfg->cpswCfg.p2P0TxFlowEn = CPSW3G_DEFAULT_P2_P0TXFLOWEN;
       iCfg->cpswCfg.p1P2TxFlowEn = CPSW3G_DEFAULT_P1_P2TXFLOWEN;
       iCfg->cpswCfg.p1P0TxFlowEn = CPSW3G_DEFAULT_P1_P0TXFLOWEN;
       iCfg->cpswCfg.p0P2TxFlowEn = CPSW3G_DEFAULT_P0_P2TXFLOWEN;
       iCfg->cpswCfg.p0P1TxFlowEn = CPSW3G_DEFAULT_P0_P1TXFLOWEN;
       iCfg->cpswCfg.fifoLoopback = CPSW3G_DEFAULT_FIFO_LOOPBACK;

       iCfg->cpswCfg.p2PtypeEsc = CPSW3G_DEFAULT_P2PTYPE_ESC;
       iCfg->cpswCfg.p1PtypeEsc = CPSW3G_DEFAULT_P1PTYPE_ESC;
       iCfg->cpswCfg.p0PtypeEsc = CPSW3G_DEFAULT_P0PTYPE_ESC;
       iCfg->cpswCfg.escPriLdVal = CPSW3G_DEFAULT_ESCPRI_LD_VAL;

       iCfg->cpswCfg.p2StatEn = CPSW3G_DEFAULT_P2STATEN;
       iCfg->cpswCfg.p1StatEn = CPSW3G_DEFAULT_P1STATEN;
       iCfg->cpswCfg.p0StatEn = CPSW3G_DEFAULT_P0STATEN;

       iCfg->cpswCfg.resetLine = CPSW3G_ESWITCH_RESET_BIT;
       iCfg->cpswCfg.intrLine = CPSW3G_HIGH_CPMAC_INT;
       iCfg->cpswCfg.mdioResetLine = CPSW3G_MDIO_RESET_BIT;

       iCfg->cpswCfg.mdioTickMSec = CPSW3G_DEFAULT_MDIOTICK;
       iCfg->cpswCfg.baseAddress = CPSW3G_BASEADDR;
       iCfg->cpswCfg.mdioBaseAddress = MDIO_BASEADDR;
       
       iCfg->cpswCfg.mdioBusFrequency = CPSW3G_DEFAULT_MDIOBUSFREQ;
       iCfg->cpswCfg.mdioClockFrequency = CPSW3G_DEFAULT_MDIOCLOCKFREQ;
       iCfg->cpswCfg.cpmacBusFrequency = CPSW3G_DEFAULT_CPMACBUSFREQ ;
       iCfg->cpswCfg.sgmii0BaseAddress = CPSW3G_SGMII0_BASEADDR;
       iCfg->cpswCfg.sgmii1BaseAddress = CPSW3G_SGMII1_BASEADDR;
       iCfg->cpswCfg.cpsw3GSSBaseAddress = CPSW_3GSS_S_BASEADDR;

       iCfg->Mib64CntMsec = CPSW3G_MIB_TIMER_TIMEOUT;
       /*set the receive buffer size */
       cpsw3gDev.rxBufSize=1522;
       cpsw3gDev.rxBufOffset=0x02;

	/* We count in 100ms Ticks */
	cpsw3gDev.aleTicks = (ALE_AGE_OUT_TIME / 100); 
	cpsw3gDev.aleTimerActive = 0;

}/* end of function cpsw3g_get_cfg() */


static int cpsw3g_port_close(Cpsw3gPort *hPort)
{

	if(hPort->portNum == 2)
		{
		/* handle DMA port cleanup here */
		cpsw3g_disable_interrupts();
		cpdmaClose(hPort);
		}
	else
		{
		cpgmacMacClose(hPort);
		}

    return (CPSW3G_SUCCESS);
}/* end of function cpsw3g_port_close */

static int cpsw3g_port_open(Cpsw3gPort *hPort)
{

    /* Open DDC */
    if(hPort->portNum < 2)
    {
	    /* Ports 0 & 1 are CPGMAC_SL */
	    cpgmacMacOpen(hPort);
    }
    else
    {
	    cpdmaOpen(hPort);
    }

    /**CHECK  sequence here */
    /*Program the CPSW_Control register */
    cpsw3g_SetCpswCfg();
    /* Program the ALE Control register */
    cpsw3g_SetAleCfg();
    /* Configure ALE Port Control Register */
    cpsw3g_SetAlePortCfg(hPort);

    /**CHECK: kick start any timers here */



    /**Port num 2: Program DMA channels here */
    if (hPort->portNum == 2)
    {
        Uint32 dir,ch,retCode;
        for (dir = NET_CH_DIR_TX; dir <= NET_CH_DIR_RX; dir ++)
          {
            for (ch = 0; ch < cpsw3gCfg.dmaInitCfg.numChannels[dir]; ch++)
             {
              /*Create a channel */
              retCode = cpsw3g_NetChOpen( &cpsw3gDev.chInfo[dir][ch]);
              if (retCode != CPSW3G_SUCCESS)
                 {
                    return (-1);
                 }
             }/* end of for loop over ch */
           }/*end of for loop over dir*/
      
  }

    return (CPSW3G_SUCCESS);
}/* end of function cpsw3g_port_open() */
	
static void cpgmacMacOpen(Cpsw3gPort *hPort)
{
    CSL_Cpsw3g_RegsOvly regs = cpsw3gDev.regs;
    Cpsw3gConfig *cpswCfg = &cpsw3gCfg.cpswCfg;
    Uint32 instId = hPort->portNum;
    CpgmacMacConfig *macInitCfg = &cpsw3gCfg.macInitCfg[instId];
    CpgmacObj *gmacObject = &cpsw3gDev.cpgmacMac[instId];


    /* Do a soft reset of the module */
    regs->GMAC[instId].GMAC_SOFT_RESET = CPSW3G_SOFT_RESET_BIT;
    /* wait for reset complete */
    while (regs->GMAC[instId].GMAC_SOFT_RESET != CPSW3G_SOFT_RESET_COMPLETE)
    {
        /* Taskdelay if necessary */
    }


    /* Set MacControl Register */
    cpsw3g_SetMacCfg(hPort);
    /* Configure Mac Address */
    regs->GMAC_PORT[instId].GMAC_SA_LO = macInitCfg->sourceAddressLo;
    regs->GMAC_PORT[instId].GMAC_SA_HI = macInitCfg->sourceAddressHi;
    /* Configure port VLAN information */
    cpsw3g_SetPortVlanCfg(hPort, macInitCfg->portPri,
                        macInitCfg->portCfi, macInitCfg->portVID);
    /* write rx maxlen register */
    regs->GMAC[instId].GMAC_RX_MAXLEN = macInitCfg->rxMaxlen;


    /*Kick Start MDIO , auto-negn */
    mmZeroInit(&gmacObject->PhyDev,sizeof(PHY_DEVICE));

    cpsw_MDIO_Init(&gmacObject->PhyDev, cpswCfg->mdioBaseAddress, instId,macInitCfg->PhyMask,
                    macInitCfg->MLinkMask, 0,0, cpswCfg->mdioResetLine, 
                    cpswCfg->mdioBusFrequency,cpswCfg->mdioClockFrequency,0);

    /* set the Phy to given mode based on Config Params */
    cpsw3g_SetPhyMode(gmacObject,hPort->portNum);

   
    /* Finally Set the Mac Control register. Enable MII */
    gmacObject->macControl = regs->GMAC[instId].GMAC_MACCONTROL;
    gmacObject->macControl |= (1 << CSL_CPSW3G_GMAC_MACCONTROL_GMII_EN_SHIFT);
    regs->GMAC[instId].GMAC_MACCONTROL = gmacObject->macControl;


}/* end of function cpgmacMacOpen */


static void cpgmacMacClose(Cpsw3gPort *hPort)
{
    CSL_Cpsw3g_RegsOvly regs = cpsw3gDev.regs;

    /* soft reset */
    regs->GMAC[hPort->portNum].GMAC_SOFT_RESET = CPSW3G_SOFT_RESET_BIT;
    /* wait for reset complete */
    while (regs->GMAC[hPort->portNum].GMAC_SOFT_RESET != CPSW3G_SOFT_RESET_COMPLETE)
    {
        /* CHECK :: Task Delay if necessary */
    }

    return;


}/* end of function cpgmacMacclose */

static Uint32 cpdmaOpen(Cpsw3gPort *hPort)
{
    CSL_Cpsw3g_RegsOvly regs = cpsw3gDev.regs;
    CSL_EctlRegsOvly ectlRegs =  cpsw3gDev.ectlRegs;
    Uint32 retVal;
    CpdmaConfig *dmaInitCfg = &cpsw3gCfg.dmaInitCfg;
    CpdmaObj *dmaObj = &cpsw3gDev.cpdma;
    Uint32 channel, dir;

    /* soft reset */
    regs->SOFT_RESET = CPSW3G_SOFT_RESET_BIT;

    /* wait for reset complete */
    while (regs->SOFT_RESET != CPSW3G_SOFT_RESET_COMPLETE)
    {
        /**CHECK : task delay if necessary */
    }

    for (channel = 0; channel < CPDMA_MAX_CHANNELS; channel++)
    {
        /* program tx/rx headDescriptorPointes to 0 */
        regs->TX_HDP[channel] = 0;
        regs->RX_HDP[channel] = 0;
        /* program tx/rx completionPointers to 0 */
        regs->TX_CP[channel] = 0;
        regs->RX_CP[channel] = 0;
    }

    /* Enable Opened TX Channels */
    for (dir = NET_CH_DIR_TX; dir <= NET_CH_DIR_RX; dir ++)
    {
        for (channel = 0; channel < CPDMA_MAX_CHANNELS; channel++)
	    {
	        /**CHECK: Normal scenario channel wont be init at this stage */
	        if (dmaObj->chIsInit[dir][channel] == TRUE)
	        {
	            retVal = cpsw3g_EnableChannel(channel, dir);
	            if (retVal != CPSW3G_SUCCESS)
		        {
		            printf("\nERROR:cpdmaOpen: Error enabling channel %d", channel);
		            return (retVal);
		        }
	        }
	    }/* end of for channel loop */
    }/* end of for loop over dir */
    
    /* Set the DMACONTROL register */
    cpsw3g_SetDmaCfg(hPort);

    cpsw3g_SetPortVlanCfg(hPort, dmaInitCfg->portPri,
			            dmaInitCfg->portCfi, dmaInitCfg->portVID);

    /* Set the rx buffer offset */
    regs->RX_BUFFER_OFFSET = dmaInitCfg->rxBufferOffset;

    /* Enable TX/RX DMA */
    regs->TX_CONTROL |= CPSW3G_TX_CONTROL_TX_ENABLE_VAL;
    regs->RX_CONTROL |= CPSW3G_RX_CONTROL_RX_ENABLE_VAL;

    /* Enable Adapter check interrupts - enable stats interupt */
    regs->DMA_INTMASK_SET = CPSW3G_HOST_ERR_INTMASK_VAL |
    							CPSW3G_STAT_PEND_INTMASK_VAL;
    /* enable host,stats interrupt in cpsw_3gss_s wrapper */
    ectlRegs->MISC_EN =  CPSW3G_ECTL_HOSTERR_INTMASK |
    						CPSW3G_ECTL_STATPEND_INTMASK;
    return (CPSW3G_SUCCESS);


}/* end of function cpdmaOpen */


static Uint32 cpdmaClose(Cpsw3gPort *hPort)
{
    CSL_Cpsw3g_RegsOvly regs = cpsw3gDev.regs;
    CSL_EctlRegsOvly ectlRegs =  cpsw3gDev.ectlRegs;
    Uint32 retVal=CPSW3G_SUCCESS;
    Uint32 channel;

    (void)hPort;/*to suppress warning */

    /* Close TX Channels */
    for (channel = 0; channel < CPDMA_MAX_CHANNELS; channel++)
    	{
      	  retVal = cpsw3g_NetChClose(&cpsw3gDev.chInfo[NET_CH_DIR_TX][channel]);
	  
    	}

    /* Close RX Channels */
    for (channel = 0; channel < CPDMA_MAX_CHANNELS; channel++)
    {
     	retVal = cpsw3g_NetChClose(&cpsw3gDev.chInfo[NET_CH_DIR_RX][channel]);

	}
    
    /* Disable TX/RX DMA */
    regs->TX_CONTROL = 0;
    regs->RX_CONTROL = 0;
 
    /* Disable Adapter check interrupts - Disable stats interupt */
    regs->DMA_INTMASK_CLEAR = CPSW3G_HOST_ERR_INTMASK_VAL |
    							CPSW3G_STAT_PEND_INTMASK_VAL; 
    /* Disable host,stats interrupt in cpsw_3gss_s wrapper */
    ectlRegs->MISC_EN &= ( ~( CPSW3G_ECTL_HOSTERR_INTMASK |
    						CPSW3G_ECTL_STATPEND_INTMASK));
  /* soft reset */
  regs->SOFT_RESET = CPSW3G_SOFT_RESET_BIT;
  /* wait for reset complete */
  while (regs->SOFT_RESET != CPSW3G_SOFT_RESET_COMPLETE)
	{
  waitMsecs(1);
	}

    return (retVal);

}/* end of function cpdmaClose */


/** Low level functions for programming the 
 * Switch control registers, configuring entries
 * in the ALE 
 */

static void cpsw3g_SetCpswCfg(void)
{
    Uint32 cpswControlVal;
    /* Local Pointer to Switch Configuration */
    Cpsw3gConfig  *lpCpswCfg;

    /*Get the switch config info*/
    lpCpswCfg = &cpsw3gCfg.cpswCfg; 

    cpswControlVal =
    CSL_FMK(CPSW3G_CPSW_CONTROL_P2_PASS_PRI_TAGGED, lpCpswCfg->p2PassPriTagged) |
    CSL_FMK(CPSW3G_CPSW_CONTROL_P1_PASS_PRI_TAGGED, lpCpswCfg->p1PassPriTagged) |
    CSL_FMK(CPSW3G_CPSW_CONTROL_P0_PASS_PRI_TAGGED, lpCpswCfg->p0PassPriTagged) |
    CSL_FMK(CPSW3G_CPSW_CONTROL_VLAN_AWARE, lpCpswCfg->vlanAware) |
    CSL_FMK(CPSW3G_CPSW_CONTROL_RX_VLAN_ENCAP, lpCpswCfg->rxVlanEncap) |
    CSL_FMK(CPSW3G_CPSW_CONTROL_P2_P2RX_FLOW_EN, lpCpswCfg->p2P2RxFlowEn) |
    CSL_FMK(CPSW3G_CPSW_CONTROL_P2_P1TX_FLOW_EN, lpCpswCfg->p2P1TxFlowEn) |
    CSL_FMK(CPSW3G_CPSW_CONTROL_P2_P0TX_FLOW_EN, lpCpswCfg->p2P0TxFlowEn) |
    CSL_FMK(CPSW3G_CPSW_CONTROL_P1_P2TX_FLOW_EN, lpCpswCfg->p1P2TxFlowEn) |
    CSL_FMK(CPSW3G_CPSW_CONTROL_P1_P0TX_FLOW_EN, lpCpswCfg->p1P0TxFlowEn) |
    CSL_FMK(CPSW3G_CPSW_CONTROL_P0_P2TX_FLOW_EN, lpCpswCfg->p0P2TxFlowEn) |
    CSL_FMK(CPSW3G_CPSW_CONTROL_P0_P1TX_FLOW_EN, lpCpswCfg->p0P1TxFlowEn) |
    CSL_FMK(CPSW3G_CPSW_CONTROL_FIFO_LOOPBACK, lpCpswCfg->fifoLoopback);

    {
      cpsw3gDev.regs->CPSW_CONTROL = cpswControlVal;
    }

    cpsw3gDev.regs->CPSW_PTYPE =
    CSL_FMK(CPSW3G_CPSW_PTYPE_P2_PTYPE_ESC, lpCpswCfg->p2PtypeEsc) |
    CSL_FMK(CPSW3G_CPSW_PTYPE_P1_PTYPE_ESC, lpCpswCfg->p1PtypeEsc) |
    CSL_FMK(CPSW3G_CPSW_PTYPE_P0_PTYPE_ESC, lpCpswCfg->p0PtypeEsc) |
    CSL_FMK(CPSW3G_CPSW_PTYPE_ESC_PRI_LD_VAL, lpCpswCfg->escPriLdVal);

  /* Configure use of statistics */
    cpsw3gDev.regs->CPSW_STAT_PORT_EN =
    CSL_FMK(CPSW3G_CPSW_STAT_PORT_EN_P2_STAT_EN, lpCpswCfg->p2StatEn) |
    CSL_FMK(CPSW3G_CPSW_STAT_PORT_EN_P1_STAT_EN, lpCpswCfg->p1StatEn) |
    CSL_FMK(CPSW3G_CPSW_STAT_PORT_EN_P0_STAT_EN, lpCpswCfg->p0StatEn);

}/* end of function cpsw3g_SetCpswCfg() */


static void cpsw3g_SetAleCfg(void)
{
    Uint32 aleControlVal = 0;
    /* Local Pointer to Ale Configuration */
    AleConfig * lpAleConfig ;

    lpAleConfig =  &cpsw3gCfg.aleCfg;

    aleControlVal =
    CSL_FMK(CPSW3G_ALE_CONTROL_ENABLE_ALE, lpAleConfig->enableAle) |
    CSL_FMK(CPSW3G_ALE_CONTROL_CLEAR_TABLE, lpAleConfig->clearTable) |
    CSL_FMK(CPSW3G_ALE_CONTROL_AGE_OUT_NOW, lpAleConfig->ageOutNow) |
    CSL_FMK(CPSW3G_ALE_CONTROL_LEARN_NO_VID, lpAleConfig->learnNoVid) |
    CSL_FMK(CPSW3G_ALE_CONTROL_EN_VID0_MODE, lpAleConfig->enVid0Mode) |
    CSL_FMK(CPSW3G_ALE_CONTROL_ENABLE_OUI_DENY, lpAleConfig->enableOuiDeny) |
    CSL_FMK(CPSW3G_ALE_CONTROL_ALE_BYPASS, lpAleConfig->aleBypass) |
    CSL_FMK(CPSW3G_ALE_CONTROL_RATE_LIMIT_TX, lpAleConfig->rateLimitTx) |
    CSL_FMK(CPSW3G_ALE_CONTROL_ALE_VLAN_AWARE, lpAleConfig->aleVlanAware) |
    CSL_FMK(CPSW3G_ALE_CONTROL_ENABLE_AUTH_MODE, lpAleConfig->enableAuthMode) |
    CSL_FMK(CPSW3G_ALE_CONTROL_ENABLE_RATE_LIMIT, lpAleConfig->enableRateLimit);

    {
      cpsw3gDev.regs->ALE_CONTROL = aleControlVal;
    }

    /* Clear out toggle bits */
    lpAleConfig->clearTable = 0;
    lpAleConfig->ageOutNow = 0;

    /* Configure ALE Prescale register */
    cpsw3gDev.regs->ALE_PRESCALE = lpAleConfig->alePrescale;

    /* Configure Unknown VLAN items */
    cpsw3gDev.regs->ALE_UNKNOWN_VLAN =
    CSL_FMK(CPSW3G_ALE_UNKNOWN_VLAN_UNKNOWN_FORCE_UNTAGGED_EGRESS,
        lpAleConfig->unknownForceUntaggedEgress) |
    CSL_FMK(CPSW3G_ALE_UNKNOWN_VLAN_UNKNOWN_REG_MCAST_FLOOD_MASK,
        lpAleConfig->unknownRegMcastFloodMask) |
    CSL_FMK(CPSW3G_ALE_UNKNOWN_VLAN_UNKNOWN_MCAST_FLOOD_MASK,
        lpAleConfig->unknownMcastFloodMask)|
    CSL_FMK(CPSW3G_ALE_UNKNOWN_VLAN_UNKNOWN_VLAN_MEMBER_LIST,
        lpAleConfig->unknownVlanMemberList);

}/* end of function cpsw3g_SetAleCfg */

static void cpsw3g_SetAlePortCfg(Cpsw3gPort* hPort)
{
    Uint32 alePortControlVal=0, portNo = 0;
    /* Local Pointer to AlePort Configuration */
    AlePortConfig *lpAlePortConfign = 0;

    portNo = hPort->portNum;
    lpAlePortConfign = &cpsw3gCfg.alePortCfg[portNo];

    alePortControlVal =
    CSL_FMK(CPSW3G_ALE_PORTCTL_BCAST_LIMIT, lpAlePortConfign->bcastLimit) |
    CSL_FMK(CPSW3G_ALE_PORTCTL_MCAST_LIMIT, lpAlePortConfign->mcastLimit) |
    CSL_FMK(CPSW3G_ALE_PORTCTL_NO_LEARN, lpAlePortConfign->noLearn) |
    CSL_FMK(CPSW3G_ALE_PORTCTL_VID_INGRESS_CHECK, lpAlePortConfign->vidIngressCheck) |
    CSL_FMK(CPSW3G_ALE_PORTCTL_DROP_UNTAGGED, lpAlePortConfign->dropUntagged) |
    CSL_FMK(CPSW3G_ALE_PORTCTL_PORT_STATE, lpAlePortConfign->portState);

    if (portNo < 2)
    {
	CpgmacObj * gmacObject;
	gmacObject = &cpsw3gDev.cpgmacMac[portNo];

	gmacObject->portState = CSL_FEXT(alePortControlVal, CPSW3G_ALE_PORTCTL_PORT_STATE);
	cpsw3gDev.regs->ALE_PORTCTL[portNo] = alePortControlVal;
    }
    else
    {
	CpdmaObj * dmaObj;
	dmaObj = &cpsw3gDev.cpdma;

	dmaObj->portState = CSL_FEXT(alePortControlVal, CPSW3G_ALE_PORTCTL_PORT_STATE);
	cpsw3gDev.regs->ALE_PORTCTL[portNo] = alePortControlVal;
    }


}/* end of function cpsw3g_SetAlePortcfg */

static void cpsw3g_SetDmaCfg(Cpsw3gPort* hPort)
{
    Uint32 dmaControlVal = 0;
    CpdmaConfig *lpDmaConfign = 0;

    lpDmaConfign = &cpsw3gCfg.dmaInitCfg;
    (void)hPort;/*to suppress warning */

    dmaControlVal =
        CSL_FMK(CPSW3G_DMACONTROL_RX_CEF, lpDmaConfign->rxCef) |
        CSL_FMK(CPSW3G_DMACONTROL_CMD_IDLE, lpDmaConfign->cmdIdle) |
        CSL_FMK(CPSW3G_DMACONTROL_RX_OFFLEN_BLOCK, lpDmaConfign->rxOfflenBlock) |
        CSL_FMK(CPSW3G_DMACONTROL_RX_OWNERSHIP, lpDmaConfign->rxOwnership) |
        CSL_FMK(CPSW3G_DMACONTROL_TX_PTYPE, lpDmaConfign->txPtype);

    cpsw3gDev.regs->DMACONTROL = dmaControlVal;

    cpsw3gDev.regs->CPDMA_RX_CH_MAP =
        CSL_FMK(CPSW3G_CPDMA_RX_CH_MAP_SU_CH0, lpDmaConfign->hostRxCh_gmac1Pri0) |
        CSL_FMK(CPSW3G_CPDMA_RX_CH_MAP_SU_CH1, lpDmaConfign->hostRxCh_gmac1Pri1) |
        CSL_FMK(CPSW3G_CPDMA_RX_CH_MAP_SU_CH2, lpDmaConfign->hostRxCh_gmac1Pri2) |
        CSL_FMK(CPSW3G_CPDMA_RX_CH_MAP_SU_CH3, lpDmaConfign->hostRxCh_gmac1Pri3) |
        CSL_FMK(CPSW3G_CPDMA_RX_CH_MAP_CH0, lpDmaConfign->hostRxCh_gmac0Pri0) |
        CSL_FMK(CPSW3G_CPDMA_RX_CH_MAP_CH1, lpDmaConfign->hostRxCh_gmac0Pri1) |
        CSL_FMK(CPSW3G_CPDMA_RX_CH_MAP_CH2, lpDmaConfign->hostRxCh_gmac0Pri2) |
        CSL_FMK(CPSW3G_CPDMA_RX_CH_MAP_CH3, lpDmaConfign->hostRxCh_gmac0Pri3);

}/* end of function cpsw3g_SetDmaCfg */

static void cpsw3g_SetMacCfg(Cpsw3gPort * hPort)
{
    Uint32 macControlVal = 0,portNo = 0;
    CpgmacMacConfig * lpMacConfign = 0;

    portNo = hPort->portNum;

    if(portNo > 1)
    {
        /* error condition */
        printf("Error in SetMacCfg\n");
        /* return without doing anything, you
         * cant dereference past instances 0 & 1
         * of CPGMAC_SL config structures */
    }
    else
    {
        lpMacConfign = &cpsw3gCfg.macInitCfg[portNo];

        macControlVal =
            CSL_FMK(CPSW3G_GMAC_MACCONTROL_RX_CMF_EN, lpMacConfign->rxCmfEn) |
            CSL_FMK(CPSW3G_GMAC_MACCONTROL_RX_CSF_EN, lpMacConfign->rxCsfEn) |
            CSL_FMK(CPSW3G_GMAC_MACCONTROL_RX_CEF_EN, lpMacConfign->rxCefEn) |
            CSL_FMK(CPSW3G_GMAC_MACCONTROL_EXT_EN, lpMacConfign->extEn) |
            CSL_FMK(CPSW3G_GMAC_MACCONTROL_GIG_FORCE, lpMacConfign->gigForce) |
            CSL_FMK(CPSW3G_GMAC_MACCONTROL_IFCTL_B, lpMacConfign->ifCtlB) |
            CSL_FMK(CPSW3G_GMAC_MACCONTROL_IFCTL_A, lpMacConfign->ifCtlA) |
            CSL_FMK(CPSW3G_GMAC_MACCONTROL_CMD_IDLE, lpMacConfign->cmdIdle) |
            CSL_FMK(CPSW3G_GMAC_MACCONTROL_TX_SHORT_GAP_EN, lpMacConfign->txShortGapEnable) |
            CSL_FMK(CPSW3G_GMAC_MACCONTROL_GIG, lpMacConfign->gigaBitEnable) |
            CSL_FMK(CPSW3G_GMAC_MACCONTROL_TX_PACE,lpMacConfign->txPacingEnable) |
            CSL_FMK(CPSW3G_GMAC_MACCONTROL_GMII_EN, lpMacConfign->gmiiEnable)|
            CSL_FMK(CPSW3G_GMAC_MACCONTROL_TX_FLOW_EN, lpMacConfign->txFlowEnable) |
            CSL_FMK(CPSW3G_GMAC_MACCONTROL_RX_FLOW_EN, lpMacConfign->rxFlowEnable) |
            CSL_FMK(CPSW3G_GMAC_MACCONTROL_LOOPBACK, lpMacConfign->loopbackEnable) |
            CSL_FMK(CPSW3G_GMAC_MACCONTROL_FULLDUPLEX, lpMacConfign->fullduplex);

        cpsw3gDev.regs->GMAC[portNo].GMAC_MACCONTROL = macControlVal;

    }

    return;

}/* end of function cpsw3g_SetMacCfg */

/* Ioctl low level implementation functions */

static Uint32 cpsw3g_AleAddVlan(Uint32 vlanId,Uint32 vlanMemberList,
        Uint32 unregMcastFloodMask, Uint32 regMcastFloodMask, 
        Uint32 forceUntaggedEgress)
{
    Uint32 vlanAware =0,freeEntry=0,ret = CPSW3G_SUCCESS;
    Int32 vlanEntry = 0;

    /* Check the current setting for vlanAware */
    vlanAware = CSL_FEXT(cpsw3gDev.regs->CPSW_CONTROL, CPSW3G_CPSW_CONTROL_VLAN_AWARE);

    if (vlanAware == 0) 
    {
	ret = (CPSW3G_VLAN_UNAWARE_MODE);
    }
    else
    {

    vlanId &= 0x0fff;
    vlanMemberList &= 0x7;
    unregMcastFloodMask &= 0x7;
    regMcastFloodMask &= 0x7;
    forceUntaggedEgress &= 0x7;

    vlanEntry = cpsw3g_AleFindVlan(vlanId,NULL,NULL,NULL,NULL);
    if((vlanEntry != -1) && (vlanEntry < CPSW3G_ALE_TABLE_DEPTH))
    {
        /* update existing entry */
        freeEntry = (Uint32)vlanEntry ;
    }
    else
    {
        /* Update VLAN Info */
        freeEntry = cpsw3g_AleGetFreeEntry();
    }

    if(freeEntry == CPSW3G_ALE_TABLE_FULL)
    {
        ret = freeEntry;
    }
    else
    {
	    cpsw3gDev.regs->ALE_TBLW2 = 0;
	    cpsw3gDev.regs->ALE_TBLW1 = (2<<28)|(vlanId<<16);
	    cpsw3gDev.regs->ALE_TBLW0 = (forceUntaggedEgress<<24)|
            (regMcastFloodMask<<16)|(unregMcastFloodMask<<8)|(vlanMemberList<<0);
	    cpsw3gDev.regs->ALE_TBLCTL = freeEntry | ((Uint32)CPSW3G_ALE_TABLE_WRITE);
	}
    }

    return(ret);

}/* end of function cpsw3g_AleAddVlan */

/*
 *  This function finds the specified VLAN entry in the ALE table.  The
 *  information associated with the entry is returned through the parameters.
 */
static Int32 cpsw3g_AleFindVlan(Uint32 vlanId, Uint32 *vlanMemberList, 
            Uint32 *unregMcastFloodMask, Uint32 *regMcastFloodMask,
            Uint32 *forceUntaggedEgress)
{
    Uint32 i=0;
    Uint32 info0=0,info1=0, VID=0;
    Int32 ret = -1;
  
    for(i=0;i<CPSW3G_ALE_TABLE_DEPTH;i++)
    {
        cpsw3gDev.regs->ALE_TBLCTL = i;
        info0 = cpsw3gDev.regs->ALE_TBLW0;
        info1 = cpsw3gDev.regs->ALE_TBLW1;
        VID = info1 >> 16;
        if(((info1 >>28) & 0x3) == 0x2)
        {
            if((VID & 0x0FFF) == vlanId)
            {
                info0 = cpsw3gDev.regs->ALE_TBLW0;
                if(i<CPSW3G_ALE_TABLE_DEPTH)
                {
                    if(0u != vlanMemberList) 
                    {
                        *vlanMemberList = ( info0 & 0x00000007);
                    }
                    if(0u != unregMcastFloodMask)
                    {
                        *unregMcastFloodMask = ((info0 & 0x00000700) >> 8);
                    }
                    if(0u != regMcastFloodMask) 
                    {
                        *regMcastFloodMask   = ((info0 & 0x00070000) >> 16);
                    }
                    if(0u != forceUntaggedEgress)
                    {
                        *forceUntaggedEgress = ((info0 & 0x07000000) >> 24);
                    }
                    
                    ret = i;
                    break;
                }
            }/* end of id VID == vlanId block */
        }
    }/* end of for i loop */
 
    return ret;
}/* end of function cpsw3g_AleFindVlan */

static Uint32 cpsw3g_AleGetFreeEntry()
{
    Uint32 i=0,reg1=0,ret=0;

    for(i=0; i<CPSW3G_ALE_TABLE_DEPTH; i++)
    {
        cpsw3gDev.regs->ALE_TBLCTL = i ;
        reg1 = cpsw3gDev.regs->ALE_TBLW1;

        if((reg1 & 0x30000000) == 0 )
	{
             break;
	}
    }/* end of for i loop */
    if(i==CPSW3G_ALE_TABLE_DEPTH)
    {
        ret = CPSW3G_ALE_TABLE_FULL;
    }
    else
    {
        ret =i;
    }

    return ret;
}/* end of function cpsw3g_AleGetFreeEntry */

static Uint32 cpsw3g_AleAddUniAddr(Cpsw3gPort *hPort, Uint8 * addr, Uint32 vlanId,
                                    Uint32 blocked, Uint32 secure, Uint32 ageable)
{
    Uint32 freeEntry, vlanAware, info0, info1, info2,ret;
    Int32 addrEntry = 0;

    /* Check the current setting for vlanAware */
    vlanAware = CSL_FEXT(cpsw3gDev.regs->CPSW_CONTROL, CPSW3G_CPSW_CONTROL_VLAN_AWARE);

    /* ignore input vlanId if not in VLAN aware mode */
    if (vlanAware == 0) 
    {
	vlanId = 0;
    }
    /* Check for space in the ALE table */
    addrEntry = cpsw3g_AleFindAddress(addr, vlanId);
    if(addrEntry == -1)
    {
        freeEntry = cpsw3g_AleGetFreeEntry();
    }
    else
    {
        /* update the existing entry itself */
        freeEntry = (Uint32) addrEntry;
    }

    if (freeEntry == CPSW3G_ALE_TABLE_FULL)
    {
        ret = freeEntry;
    }
    else
    {
        /* Add new entry to ALE table */
        info2=(ageable<<2)|(hPort->portNum&3);
        info1=((blocked<<31)|(secure<<30)|(1<<28)|
            ((vlanId&0x0fff)<<16)|((addr[0]&0x0ff)<<8)|
            ((addr[1]&0x0ff)<<0));
        
        if(vlanId != 0)
        {
            info1 |= (1<<29);
        }

        info0=(((addr[2]&0x0ff)<<24)|((addr[3]&0x0ff)<<16)|
            ((addr[4]&0x0ff)<<8)|((addr[5]&0x0ff)<<0));

        cpsw3gDev.regs->ALE_TBLW2 = info2;
        cpsw3gDev.regs->ALE_TBLW1 = info1;
        cpsw3gDev.regs->ALE_TBLW0 = info0;
        cpsw3gDev.regs->ALE_TBLCTL = freeEntry | CPSW3G_ALE_TABLE_WRITE;
        ret = CPSW3G_SUCCESS;
    }

    return(ret);

}/* end of function cpsw3g_AleAddUniAddr() */


/*
 *  This function will attempt to find the address in the ALE table, if
 *  successful, it will return CPSW3G_SUCCESS.
 */
static Int32 cpsw3g_AleFindAddress( Uint8 *addr, Uint32 vlanId)
{
    Uint32 i=0,VID=0,mask1=0, info1=0, info0=0,reg0=0,reg1=0;
    Int32 ret = -1;

    /* We only need to check the entry type and address fields */
    mask1=0x0000ffff;

    /* figure out the expected values to match with */
    info1=((addr[0]&0x0ff)<<8)|((addr[1]&0x0ff)<<0);
    info1=info1 & 0x0000FFFF;
    info0=((addr[2]&0x0ff)<<24)|((addr[3]&0x0ff)<<16)|((addr[4]&0x0ff)<<8)|
        ((addr[5]&0x0ff)<<0);

    for(i=0;i<CPSW3G_ALE_TABLE_DEPTH;i++)
    {
        /* read from the ALE table */
        cpsw3gDev.regs->ALE_TBLCTL =i;
        reg0=cpsw3gDev.regs->ALE_TBLW0;
        reg1=cpsw3gDev.regs->ALE_TBLW1;
        if((((reg1 >>28) & 0x3) == 0x1)|| (((reg1 >>28) & 0x3)== 0x3))
        {
            if(((reg1&mask1) == info1) && ((reg0 & 0xFFFFFFFF) == info0))
            {
                if(vlanId < 4096)
                {
                    VID = reg1 >> 16;
                    if((VID & 0x0FFF) == vlanId)
                    {
                        break;
                    }
                }
                else
                {
                    break;
                }
            }
        }
    }/* end of for i loop */

    if(i != CPSW3G_ALE_TABLE_DEPTH)
    {
        ret = i;
    }
    
    return ret;


}/* end of function cpsw3g_AleFindAddress */

static Uint32 cpsw3g_AleAddMltAddr(Uint8 *addr, Uint32 vlanId, Uint32 portMask,
                                    Uint32 super, Uint32 fwdState)
{
    Uint32 freeEntry, vlanAware, info0, info1, info2,ret = CPSW3G_SUCCESS;
    Int32 addrEntry = 0;

    /* Check the current setting for vlanAware */
    vlanAware = CSL_FEXT(cpsw3gDev.regs->CPSW_CONTROL, CPSW3G_CPSW_CONTROL_VLAN_AWARE);

    /* ignore input vlanId if not in VLAN aware mode */
    if (vlanAware == 0) 
    {
	vlanId = 0;
    }

    /* Check for space in the ALE table */
    /** CHECK: call cpsw3g_AleFindMcastAddress?? */
    addrEntry = cpsw3g_AleFindAddress(addr, vlanId);

    if(addrEntry == -1)
    {
        freeEntry = cpsw3g_AleGetFreeEntry();
    }
    else
    {
        /* update the existing entry */
        freeEntry = (Uint32) addrEntry;
    }
    
    if (freeEntry == CPSW3G_ALE_TABLE_FULL)
    {
        ret = CPSW3G_ALE_TABLE_FULL;
    }
    else
    {
        /* Add new entry to the ALE table */
        fwdState&=3;
        info2=(super<<3)|(portMask&7);
	if(0u != vlanId)
	{
            info1=((fwdState<<30)|(vlanAware<<29)|(1<<28)|((vlanId&0x0fff)<<16)|
            		((addr[0]&0x0ff)<<8)|((addr[1]&0x0ff)<<0));
	}
	else
	{
	    info1=((fwdState<<30)|(0<<29)|(1<<28)|((vlanId&0x0fff)<<16)|
            		((addr[0]&0x0ff)<<8)|((addr[1]&0x0ff)<<0));
	}
        info0=(((addr[2]&0x0ff)<<24)|((addr[3]&0x0ff)<<16)|
            ((addr[4]&0x0ff)<<8)|((addr[5]&0x0ff)<<0));
        cpsw3gDev.regs->ALE_TBLW2 = info2;
        cpsw3gDev.regs->ALE_TBLW1 = info1;
        cpsw3gDev.regs->ALE_TBLW0 = info0;
        cpsw3gDev.regs->ALE_TBLCTL = freeEntry|CPSW3G_ALE_TABLE_WRITE;
    }

    return(ret);

}/* end of funciton cpsw3g_AleAddMltAddr */

static Uint32 cpsw3g_AleAddOuiAddr(Uint8 *ouiAddress)
{
    Uint32 freeEntry, info0, info1, info2,ret=CPSW3G_SUCCESS;
    Int32 addrEntry = 0;

    /* Check for space in the ALE table */
    addrEntry = cpsw3g_AleFindAddress(ouiAddress,4096);
    if(addrEntry == -1)
    {
        freeEntry = cpsw3g_AleGetFreeEntry();
    }
    else
    {
        freeEntry = (Uint32) addrEntry;
    }

    if (freeEntry == CPSW3G_ALE_TABLE_FULL)
    {
        ret = CPSW3G_ALE_TABLE_FULL;
    }
    else
    {
        info2 = (1 << 3);
        info1 = ((1 << 28)|((ouiAddress[0] & 0x0ff) << 8)| 
                ((ouiAddress[1] & 0x0ff) << 0));
        info0 = ((ouiAddress[2] & 0x0ff) << 24);
        cpsw3gDev.regs->ALE_TBLW2 = info2;
        cpsw3gDev.regs->ALE_TBLW1 = info1;
        cpsw3gDev.regs->ALE_TBLW0 = info0;
        cpsw3gDev.regs->ALE_TBLCTL = freeEntry|CPSW3G_ALE_TABLE_WRITE;
    }

    return(ret);

}/* end of function cpsw3g_AleAddOuiAddr */

static Int32 cpsw3g_AleDelAddress( Uint8 *addr, Uint32 vlanId)
{
    Uint32  VID, mask1, info1, info0,reg0,reg1;
    Int32 ret = -1,i;

    /* We only need to check the entry type and address fields */
    mask1 = 0x0000ffff;

    /* figure out the expected values to match with */
    info1 = ((addr[0] & 0x0ff) << 8)|((addr[1] & 0x0ff) << 0);
    info1 = info1 & 0x0000FFFF;
    info0 = ((addr[2] & 0x0ff) << 24)|((addr[3] & 0x0ff) << 16)
                |((addr[4] & 0x0ff) << 8)|((addr[5] & 0x0ff) << 0);

    for(i=0;i<CPSW3G_ALE_TABLE_DEPTH;i++)
    {
        /* read from the ALE table */
        cpsw3gDev.regs->ALE_TBLCTL =i;
        reg0=cpsw3gDev.regs->ALE_TBLW0;
        reg1=cpsw3gDev.regs->ALE_TBLW1;
        if((((reg1 >>28) & 0x3) == 0x1)|| (((reg1 >>28) & 0x3)== 0x3)) 
        {
            if(((reg1 & mask1) == info1) && ((reg0 & 0xFFFFFFFF) == info0))
            {
                if(vlanId < 4096)
                {
                    VID = reg1 >> 16;
                    if((VID & 0x0FFF) == vlanId)
                    {
                        cpsw3g_AleFreeTableClear(i);
                        break;
                    }
                }/* end of if block covering vlanId < 4096 */
                else
                {
                    cpsw3g_AleFreeTableClear(i);
                    break;
                }/* end of else block covering vlanId >= 4096 */
            }/* end of if address match block*/
       }
    }/* end of for i loop over ALE entries */

    if(i != CPSW3G_ALE_TABLE_DEPTH)
    {
        ret = i;
    }
    
    return ret;
}/* end of function cpsw3g_AleDelAddress */


static Int32 cpsw3g_AleDelVlan(Uint32 vlanId)
{
    Int32 delCount=0;
    Uint32 vlanMemberList, unregMcastFloodMask, regMcastFloodMask, forceUntaggedEgress;
    Int32 ret = -1;

    delCount = cpsw3g_AleFindVlan(vlanId,
                        &vlanMemberList,
                        &unregMcastFloodMask,
                        &regMcastFloodMask,
                        &forceUntaggedEgress);
    if((delCount != -1) && (delCount < CPSW3G_ALE_TABLE_DEPTH))
    {
        cpsw3g_AleFreeTableClear((Uint32)delCount);
        ret = delCount;
    }

    return ret;

}/* end of function cpsw3g_AleDelVlan */



static void cpsw3g_AleFreeTableClear(Uint32 entry)
{

    cpsw3gDev.regs->ALE_TBLW2 = 0;
    cpsw3gDev.regs->ALE_TBLW1 = 0;
    cpsw3gDev.regs->ALE_TBLW0 = 0;
    cpsw3gDev.regs->ALE_TBLCTL = (CPSW3G_ALE_TABLE_WRITE | entry) ;

    return;

}/* end of function cpsw3g_AleFreeTableClear */


static void cpsw3g_RateLimitTx(Uint32 enable, Uint32 port, 
                                typeOfData type, Uint32 noOfPkts)
{

    if(0u != enable)
    {
        /**CHECK : assumption is ALE_PRESCALE set for 1ms */
        if( BROADCAST == type)
        {
            cpsw3gDev.regs->ALE_PORTCTL[port] &= ~CSL_CPSW3G_ALE_PORTCTL_BCAST_LIMIT_MASK;
            cpsw3gDev.regs->ALE_PORTCTL[port] |= (  (noOfPkts & 0xFF) 
                                << CSL_CPSW3G_ALE_PORTCTL_BCAST_LIMIT_SHIFT);
	    cpsw3gDev.regs->ALE_CONTROL |= 
                    ((1 << CSL_CPSW3G_ALE_CONTROL_RATE_LIMIT_TX_SHIFT) |
                     (1 << CSL_CPSW3G_ALE_CONTROL_ENABLE_RATE_LIMIT_SHIFT));
        }
        else 
	{
	    if(MULTICAST == type)
	    {
            cpsw3gDev.regs->ALE_PORTCTL[port] &= ~CSL_CPSW3G_ALE_PORTCTL_MCAST_LIMIT_MASK;
            cpsw3gDev.regs->ALE_PORTCTL[port] |= ((noOfPkts & 0xFF) 
                                << CSL_CPSW3G_ALE_PORTCTL_MCAST_LIMIT_SHIFT);
	    cpsw3gDev.regs->ALE_CONTROL |= 
                    ((1 << CSL_CPSW3G_ALE_CONTROL_RATE_LIMIT_TX_SHIFT) |
                     (1 << CSL_CPSW3G_ALE_CONTROL_ENABLE_RATE_LIMIT_SHIFT));
	    }
        }
        
        
    }/* end of if enable conditional block */
    else
    {
        cpsw3gDev.regs->ALE_CONTROL &= 
                ~(1 << CSL_CPSW3G_ALE_CONTROL_ENABLE_RATE_LIMIT_SHIFT);
        /* Clear off limit registers for all ports */
        cpsw3gDev.regs->ALE_PORTCTL[0] &= 
                ~(0xFF << CSL_CPSW3G_ALE_PORTCTL_BCAST_LIMIT_SHIFT);
        cpsw3gDev.regs->ALE_PORTCTL[1] &= 
                ~(0xFF << CSL_CPSW3G_ALE_PORTCTL_BCAST_LIMIT_SHIFT);
        cpsw3gDev.regs->ALE_PORTCTL[2] &= 
                ~(0xFF << CSL_CPSW3G_ALE_PORTCTL_BCAST_LIMIT_SHIFT);

        cpsw3gDev.regs->ALE_PORTCTL[0] &= 
                ~(0xFF << CSL_CPSW3G_ALE_PORTCTL_MCAST_LIMIT_SHIFT);
        cpsw3gDev.regs->ALE_PORTCTL[1] &= 
                ~(0xFF << CSL_CPSW3G_ALE_PORTCTL_MCAST_LIMIT_SHIFT);
        cpsw3gDev.regs->ALE_PORTCTL[2] &= 
                ~(0xFF << CSL_CPSW3G_ALE_PORTCTL_MCAST_LIMIT_SHIFT);
    }/* end of else conditional block */

    return;
}/* end of function cpsw3g_RateLimitTx */

static void cpsw3g_RateLimitRx(Uint32 enable, Uint32 port, 
                                typeOfData type, Uint32 noOfPkts)
{
    if(0u != enable)
    {
        /** CHECK:Assumption is that ALE prescale is set for 1 ms time */
        if(type == BROADCAST)
        {
            cpsw3gDev.regs->ALE_PORTCTL[port] &= ~CSL_CPSW3G_ALE_PORTCTL_BCAST_LIMIT_MASK;
            cpsw3gDev.regs->ALE_PORTCTL[port] |= ((noOfPkts & 0xFF) 
                                    << CSL_CPSW3G_ALE_PORTCTL_BCAST_LIMIT_SHIFT);
	    /* Enable the bit for rate limiting transmission */
            cpsw3gDev.regs->ALE_CONTROL &= ~(1 << CSL_CPSW3G_ALE_CONTROL_RATE_LIMIT_TX_SHIFT);
            cpsw3gDev.regs->ALE_CONTROL |= (1 << CSL_CPSW3G_ALE_CONTROL_ENABLE_RATE_LIMIT_SHIFT);
        }
        else 
	{
	    if (type == MULTICAST)
        {
		cpsw3gDev.regs->ALE_PORTCTL[port] &= ~CSL_CPSW3G_ALE_PORTCTL_MCAST_LIMIT_MASK;
		cpsw3gDev.regs->ALE_PORTCTL[port] |= ((noOfPkts & 0xFF) 
                                    << CSL_CPSW3G_ALE_PORTCTL_MCAST_LIMIT_SHIFT);
	    /* Enable the bit for rate limiting transmission */
		cpsw3gDev.regs->ALE_CONTROL &= ~(1 << CSL_CPSW3G_ALE_CONTROL_RATE_LIMIT_TX_SHIFT);
		cpsw3gDev.regs->ALE_CONTROL |= (1 << CSL_CPSW3G_ALE_CONTROL_ENABLE_RATE_LIMIT_SHIFT);
	    }
        }
     }     
    else
    {
        cpsw3gDev.regs->ALE_CONTROL &= ~(1 << CSL_CPSW3G_ALE_CONTROL_ENABLE_RATE_LIMIT_SHIFT);
        /* Clear off limit registers for all ports */
        cpsw3gDev.regs->ALE_PORTCTL[0] &= ~(0xFF << CSL_CPSW3G_ALE_PORTCTL_BCAST_LIMIT_SHIFT);
        cpsw3gDev.regs->ALE_PORTCTL[1] &= ~(0xFF << CSL_CPSW3G_ALE_PORTCTL_BCAST_LIMIT_SHIFT);
        cpsw3gDev.regs->ALE_PORTCTL[2] &= ~(0xFF << CSL_CPSW3G_ALE_PORTCTL_BCAST_LIMIT_SHIFT);

        cpsw3gDev.regs->ALE_PORTCTL[0] &= ~(0xFF << CSL_CPSW3G_ALE_PORTCTL_MCAST_LIMIT_SHIFT);
        cpsw3gDev.regs->ALE_PORTCTL[1] &= ~(0xFF << CSL_CPSW3G_ALE_PORTCTL_MCAST_LIMIT_SHIFT);
        cpsw3gDev.regs->ALE_PORTCTL[2] &= ~(0xFF << CSL_CPSW3G_ALE_PORTCTL_MCAST_LIMIT_SHIFT);
    }

}/* end of function cpsw3g_RateLimitRx */


/*
 * This function Defines the default VLAN ID of a port
 */
static void cpsw3g_SetPortVlanCfg(Cpsw3gPort* hPort, Uint32 pri, Uint32 cfi, Uint32 vlanId)
{
    if(hPort->portNum < 2)
    {
        cpsw3gDev.regs->GMAC_PORT[hPort->portNum].P_PORT_VLAN =
                            CSL_FMK(CPSW3G_P_PORT_VLAN_PORT_PRI, pri) |
                            CSL_FMK(CPSW3G_P_PORT_VLAN_PORT_CFI, cfi) |
                            CSL_FMK(CPSW3G_P_PORT_VLAN_PORT_VID, vlanId);
    }
    else
    {
        cpsw3gDev.regs->P2_PORT_VLAN =
                            CSL_FMK(CPSW3G_P_PORT_VLAN_PORT_PRI, pri) |
                            CSL_FMK(CPSW3G_P_PORT_VLAN_PORT_CFI, cfi) |
                            CSL_FMK(CPSW3G_P_PORT_VLAN_PORT_VID, vlanId);

    }
    return;
}/* end of function cpsw3g_SetPortVlanCfg */


/**
 *    This function configures the packet priority handling on each port.
 */
static void cpsw3g_SetPriorityMapping(Cpsw3gPort * hPort, Uint32 rxPriority,
                                        Uint32 txPriority, Uint32 switchPriority)
{
    if(hPort->portNum == 2)
    {
        switch(rxPriority)
	{
	    case 0: 
		CSL_FINS(cpsw3gDev.regs->CPDMA_TX_PRI_MAP, CPSW3G_CPDMA_TX_PRI_MAP_PRI0, txPriority);
		break;
	    case 1: 
		CSL_FINS(cpsw3gDev.regs->CPDMA_TX_PRI_MAP, CPSW3G_CPDMA_TX_PRI_MAP_PRI1, txPriority);
		break;
	    case 2: 
		CSL_FINS(cpsw3gDev.regs->CPDMA_TX_PRI_MAP, CPSW3G_CPDMA_TX_PRI_MAP_PRI2, txPriority);
		break;
	    case 3: 
		CSL_FINS(cpsw3gDev.regs->CPDMA_TX_PRI_MAP, CPSW3G_CPDMA_TX_PRI_MAP_PRI3, txPriority);
		break;
	    case 4: 
		CSL_FINS(cpsw3gDev.regs->CPDMA_TX_PRI_MAP, CPSW3G_CPDMA_TX_PRI_MAP_PRI4, txPriority);
		break;
	    case 5: 
		CSL_FINS(cpsw3gDev.regs->CPDMA_TX_PRI_MAP, CPSW3G_CPDMA_TX_PRI_MAP_PRI5, txPriority);
		break;
	    case 6: 
		CSL_FINS(cpsw3gDev.regs->CPDMA_TX_PRI_MAP, CPSW3G_CPDMA_TX_PRI_MAP_PRI6, txPriority);
		break;
	    case 7: 
		CSL_FINS(cpsw3gDev.regs->CPDMA_TX_PRI_MAP, CPSW3G_CPDMA_TX_PRI_MAP_PRI7, txPriority);
		break;
	    default : break;

	}/* end of switch rxPriority block */

	if (switchPriority != CPSW3G_USE_DEFAULT)
    	{
	    switch(txPriority)
	    {
		case 0:
		    CSL_FINS(cpsw3gDev.regs->P2_TX_PRI_MAP, CPSW3G_P2_TX_PRI_MAP_PRI0, switchPriority);
		    break;
		case 1:
		    CSL_FINS(cpsw3gDev.regs->P2_TX_PRI_MAP, CPSW3G_P2_TX_PRI_MAP_PRI1, switchPriority); 
		    break;
		case 2:
		    CSL_FINS(cpsw3gDev.regs->P2_TX_PRI_MAP, CPSW3G_P2_TX_PRI_MAP_PRI2, switchPriority);
		    break;
		case 3:
		    CSL_FINS(cpsw3gDev.regs->P2_TX_PRI_MAP, CPSW3G_P2_TX_PRI_MAP_PRI3, switchPriority); 
		    break;
		case 4:
		    CSL_FINS(cpsw3gDev.regs->P2_TX_PRI_MAP, CPSW3G_P2_TX_PRI_MAP_PRI4, switchPriority); 
		    break;
		case 5:
		    CSL_FINS(cpsw3gDev.regs->P2_TX_PRI_MAP, CPSW3G_P2_TX_PRI_MAP_PRI5, switchPriority); 
		    break;
		case 6:
		    CSL_FINS(cpsw3gDev.regs->P2_TX_PRI_MAP, CPSW3G_P2_TX_PRI_MAP_PRI6, switchPriority); 
		    break;
		case 7:
		    CSL_FINS(cpsw3gDev.regs->P2_TX_PRI_MAP, CPSW3G_P2_TX_PRI_MAP_PRI7, switchPriority); 
		    break;
		default:
		    break;
	    }/* end of switch txPriority block */
	}/* end of if switchPriority != default conditional block */
    }/* end of if hPort->Portnum ==2 conditional block */
    else
    {
	/* Configure MAC priority mapping registers */
	switch(rxPriority)
	{
	    case 0:
		CSL_FINS(cpsw3gDev.regs->GMAC[hPort->portNum].GMAC_RX_PRI_MAP, CPSW3G_GMAC_RX_PRI_MAP_PRI0, txPriority); 
		break;
	    case 1:
		CSL_FINS(cpsw3gDev.regs->GMAC[hPort->portNum].GMAC_RX_PRI_MAP, CPSW3G_GMAC_RX_PRI_MAP_PRI1, txPriority); 
		break;
	    case 2:
		CSL_FINS(cpsw3gDev.regs->GMAC[hPort->portNum].GMAC_RX_PRI_MAP, CPSW3G_GMAC_RX_PRI_MAP_PRI2, txPriority); 
		break;
	    case 3:
		CSL_FINS(cpsw3gDev.regs->GMAC[hPort->portNum].GMAC_RX_PRI_MAP, CPSW3G_GMAC_RX_PRI_MAP_PRI3, txPriority); 
		break;
	    case 4:
		CSL_FINS(cpsw3gDev.regs->GMAC[hPort->portNum].GMAC_RX_PRI_MAP, CPSW3G_GMAC_RX_PRI_MAP_PRI4, txPriority); 
		break;
	    case 5:
		CSL_FINS(cpsw3gDev.regs->GMAC[hPort->portNum].GMAC_RX_PRI_MAP, CPSW3G_GMAC_RX_PRI_MAP_PRI5, txPriority); 
		break;
	    case 6:
		CSL_FINS(cpsw3gDev.regs->GMAC[hPort->portNum].GMAC_RX_PRI_MAP, CPSW3G_GMAC_RX_PRI_MAP_PRI6, txPriority); 
		break;
	    case 7:
		CSL_FINS(cpsw3gDev.regs->GMAC[hPort->portNum].GMAC_RX_PRI_MAP, CPSW3G_GMAC_RX_PRI_MAP_PRI7, txPriority); 
		break;
	    default:
		break;
	}/* end of switch rxPriority block */

	if (switchPriority != CPSW3G_USE_DEFAULT)
	{
	    switch(txPriority)
	    {
		case 0:
		    CSL_FINS(cpsw3gDev.regs->GMAC_PORT[hPort->portNum].P_TX_PRI_MAP, CPSW3G_P_TX_PRI_MAP_PRI0, switchPriority);
		    break;
		case 1:
		    CSL_FINS(cpsw3gDev.regs->GMAC_PORT[hPort->portNum].P_TX_PRI_MAP, CPSW3G_P_TX_PRI_MAP_PRI1, switchPriority); 
		    break;
		case 2:
		    CSL_FINS(cpsw3gDev.regs->GMAC_PORT[hPort->portNum].P_TX_PRI_MAP, CPSW3G_P_TX_PRI_MAP_PRI2, switchPriority);
		    break;
		case 3:
		    CSL_FINS(cpsw3gDev.regs->GMAC_PORT[hPort->portNum].P_TX_PRI_MAP, CPSW3G_P_TX_PRI_MAP_PRI3, switchPriority); 
		    break;
		case 4:
		    CSL_FINS(cpsw3gDev.regs->GMAC_PORT[hPort->portNum].P_TX_PRI_MAP, CPSW3G_P_TX_PRI_MAP_PRI4, switchPriority);
		    break;
		case 5:
		    CSL_FINS(cpsw3gDev.regs->GMAC_PORT[hPort->portNum].P_TX_PRI_MAP, CPSW3G_P_TX_PRI_MAP_PRI5, switchPriority);
		    break;
		case 6:
		    CSL_FINS(cpsw3gDev.regs->GMAC_PORT[hPort->portNum].P_TX_PRI_MAP, CPSW3G_P_TX_PRI_MAP_PRI6, switchPriority); 
		    break;
		case 7:
		    CSL_FINS(cpsw3gDev.regs->GMAC_PORT[hPort->portNum].P_TX_PRI_MAP, CPSW3G_P_TX_PRI_MAP_PRI7, switchPriority); 
		    break;
		default:
                    break;
	    }/* end of if switch txPriority block*/
	}/* end of if switchPriority != default conditional block */
    }/* end of else conditional block */


}/* end of function cpsw3g_SetPriorityMapping */

/*
 *  This function configures internal flow control support for the ethernet
 *  switch.  It is necessary because the architecture of the switch contains
 *  several internal FIFO blocks that are subject to being overrun without the
 *  use of flow control.
 */
static Uint32 cpsw3g_SetSwitchFlowControl(Cpsw3gPort *hInPort, Cpsw3gPort *hOutPort,
                                            Uint32 thresh, Uint32 enable)
{
    Uint32 retVal = CPSW3G_SUCCESS;

    switch (hInPort->portNum)
    {
        case 0:
            if (hOutPort->portNum == 1)
            {
                CSL_FINS(cpsw3gDev.regs->GMAC_PORT[hInPort->portNum].P_FLOW_THRESH,
                        CPSW3G_P_FLOW_THRESH_P_P1TX_THRESH, thresh);
                CSL_FINS(cpsw3gDev.regs->CPSW_CONTROL, CPSW3G_CPSW_CONTROL_P0_P1TX_FLOW_EN,
                        enable);
            }
            else
            {
                if (hOutPort->portNum == 2)
                {
                    CSL_FINS(cpsw3gDev.regs->GMAC_PORT[hInPort->portNum].P_FLOW_THRESH,
                        CPSW3G_P_FLOW_THRESH_P_P2TX_THRESH, thresh);
                    CSL_FINS(cpsw3gDev.regs->CPSW_CONTROL, CPSW3G_CPSW_CONTROL_P0_P2TX_FLOW_EN,
                        enable);
                }
            }
            break;

        case 1:
            if (hOutPort->portNum == 0)
            {
                CSL_FINS(cpsw3gDev.regs->GMAC_PORT[hInPort->portNum].P_FLOW_THRESH,
                    CPSW3G_P_FLOW_THRESH_P_P0TX_THRESH, thresh);
                CSL_FINS(cpsw3gDev.regs->CPSW_CONTROL, CPSW3G_CPSW_CONTROL_P1_P0TX_FLOW_EN,
                    enable);
            }
            else
            {   
                if (hOutPort->portNum == 2)
                {
                    CSL_FINS(cpsw3gDev.regs->GMAC_PORT[hInPort->portNum].P_FLOW_THRESH,
                        CPSW3G_P_FLOW_THRESH_P_P2TX_THRESH, thresh);
                    CSL_FINS(cpsw3gDev.regs->CPSW_CONTROL, CPSW3G_CPSW_CONTROL_P1_P2TX_FLOW_EN,
                        enable);
                }
            }
            break;

        case 2:
            if (hOutPort->portNum == 0)
            {
                CSL_FINS(cpsw3gDev.regs->P2_FLOW_THRESH,
                    CPSW3G_P2_FLOW_THRESH_P2_P0TX_THRESH, thresh);
                CSL_FINS(cpsw3gDev.regs->CPSW_CONTROL, CPSW3G_CPSW_CONTROL_P2_P0TX_FLOW_EN,
                    enable);
            }
            else 
            {
                if (hOutPort->portNum == 1)
                {
                    CSL_FINS(cpsw3gDev.regs->P2_FLOW_THRESH,
                        CPSW3G_P2_FLOW_THRESH_P2_P1TX_THRESH, thresh);
                    CSL_FINS(cpsw3gDev.regs->CPSW_CONTROL, CPSW3G_CPSW_CONTROL_P2_P1TX_FLOW_EN,
                        enable);
                }
                else
                {
                    if (hOutPort->portNum == 2)
                    {
                        CSL_FINS(cpsw3gDev.regs->P2_FLOW_THRESH,
                            CPSW3G_P2_FLOW_THRESH_P2_P2RX_THRESH, thresh);
                        CSL_FINS(cpsw3gDev.regs->CPSW_CONTROL, CPSW3G_CPSW_CONTROL_P2_P2RX_FLOW_EN,
                            enable);
                    }
                }
            }
            break;

        default:
            printf("\nWARN: DDC_cpsw3gSetSwitchFlowControl: Invalid in port %d",
                        hInPort->portNum);
            retVal = (CPSW3G_INVALID_PORT);
            break;
    }
  return (retVal);

}/* end of function cpsw3g_SetSwitchFlowControl */


static void cpsw3g_SetPhyMode(CpgmacObj * gmacObject,Uint32 instanceNum)
{
	Uint32 setPhyMode,PhyMode;
	CpgmacMacConfig * macInitCfg =  &cpsw3gCfg.macInitCfg[instanceNum];
	CpgmacMacStatus macStat;

	setPhyMode = macInitCfg->phyMode;
	PhyMode = 0;

	if(0u != (setPhyMode & SNWAY_AUTO))
	    {
	    PhyMode |= NWAY_AUTO;
	    }
  	if(0u!= (setPhyMode & SNWAY_FD10))  
	    {
	    PhyMode |= NWAY_FD10;
	    }
  	if(0u != (setPhyMode & SNWAY_FD100)) 
	    {
	    PhyMode |= NWAY_FD100;
	    }
  	if(0u != (setPhyMode & SNWAY_FD1000))
	    {
	    PhyMode |= NWAY_FD1000;
	    }
  	if(0u != (setPhyMode & SNWAY_HD10 )) 
	    {
	    PhyMode |= NWAY_HD10;
	    }
  	if(0u != (setPhyMode & SNWAY_HD100))
	    {
	    PhyMode |= NWAY_HD100;
	    }
  	if(0u != (setPhyMode & SNWAY_HD1000))
	    {
	    PhyMode |= NWAY_HD1000;
	    }
  	if(0u !=(setPhyMode & SNWAY_LPBK ))
	    {
	    PhyMode |= NWAY_LPBK;
	    }
  	if(0u != (setPhyMode & SNWAY_AUTOMDIX ))
	    {
	    PhyMode |= NWAY_AUTOMDIX;
	    }


	cpsw_MDIO_SetPhyMode(&gmacObject->PhyDev, PhyMode);

	cpsw3g_Update_Phystatus(gmacObject,instanceNum);
	
  	/**CHECK: notify status change thru callback here */
  	macStat.PhyDuplex = gmacObject->PhyDuplex;
  	macStat.PhyLinked =  gmacObject->PhyLinked;
  	macStat.PhyNum = gmacObject->PhyNum;
  	macStat.PhySpeed = gmacObject->PhySpeed;
  	
  	cpsw3g_ControlCb(&cpsw3gDev, CPSW3G_PHY_STATUS_UPDATE, &macStat, &cpsw3gDev.cpsw3gPort[instanceNum]);
  	

}

static Uint32 cpsw3g_Update_Phystatus(CpgmacObj *gmacObject , Uint32 instanceNum )
{
	Uint32 setPhyMode;
	PHY_DEVICE * PhyDev =  &gmacObject->PhyDev;
	CSL_Cpsw3g_RegsOvly regs = cpsw3gDev.regs;

	setPhyMode = cpsw3gCfg.macInitCfg[instanceNum].phyMode;

  	gmacObject->macControl = regs->GMAC[instanceNum].GMAC_MACCONTROL;

  	/* No Phy Condition */
  	if(0u != (setPhyMode & SNWAY_NOPHY))
   	{

      		/* No Phy condition, always linked */
      		gmacObject->PhyLinked = 1;
      		gmacObject->PhySpeed = 1;
      		gmacObject->PhyDuplex = 1;
      		gmacObject->PhyNum = 0xFFFFFFFF;  /* No Phy */
      		gmacObject->macControl = regs->GMAC[instanceNum].GMAC_MACCONTROL;
      		gmacObject->macControl |= (1 << CSL_CPSW3G_GMAC_MACCONTROL_FULLDUPLEX_SHIFT);
      		/* Write mac control register from stored value */
      		regs->GMAC[instanceNum].GMAC_MACCONTROL = gmacObject->macControl;
      		gmacObject->macControl= regs->GMAC[instanceNum].GMAC_MACCONTROL;
    	}
	else
	{
  	/* If loopback set in hardware, set link to ON */
	    if(0u != (gmacObject->macControl & CSL_CPSW3G_GMAC_MACCONTROL_LOOPBACK_MASK))
    	{

       		gmacObject->PhyLinked = 1;
    	}
	    else
	    {


		if(0u != (setPhyMode & SNWAY_LPBK))
    	{
		    gmacObject->PhyLinked = cpsw_MDIO_GetLoopback(PhyDev);
	}

  	else
    	{
		    gmacObject->PhyLinked = cpsw_MDIO_GetLinked(PhyDev);
    	}

  	
		if (0u != (gmacObject->PhyLinked) )
    	{
      		/*  Retreive Duplex and Speed and the Phy Number  */
		    if (0u != (setPhyMode & SNWAY_LPBK))
		{
			gmacObject->PhyDuplex = 1; 
		}
      		else
		{ 
			gmacObject->PhyDuplex = cpsw_MDIO_GetDuplex(PhyDev);
        	}

		    gmacObject->PhySpeed  = cpsw_MDIO_GetSpeed(PhyDev);

		    if (gmacObject->PhySpeed != 2)
      		{
      			gmacObject->PhySpeed = (gmacObject->PhySpeed >> 10);
      		}

		    gmacObject->PhyNum =  cpsw_MDIO_GetPhyNum(PhyDev);

      		/* Set the duplex bit in maccontrol */
		    if (0u != (gmacObject->PhyDuplex))
		{ 
			gmacObject->macControl |= (1 << CSL_CPSW3G_GMAC_MACCONTROL_FULLDUPLEX_SHIFT); 
		}
      		else
		{
			gmacObject->macControl &= ~(1 << CSL_CPSW3G_GMAC_MACCONTROL_FULLDUPLEX_SHIFT); 
		}
    	}
  	

  	/* Write mac control register from stored value */
		regs->GMAC[instanceNum].GMAC_MACCONTROL = gmacObject->macControl;

		gmacObject->macControl= regs->GMAC[instanceNum].GMAC_MACCONTROL;
	    }
	}


	return (CPSW3G_SUCCESS);

}/* end of function cpsw3g_update_phystatus */

static Uint32 cpsw3g_ControlCb(Cpsw3gDevice *devPtr,Uint32 cmd,Ptr cmdArg, Ptr param)
{
	static Uint32 linkspeed[2]= {0};
	PDINFO *pdev = devPtr->pdinfo;
	Cpsw3gPort *hPort = (Cpsw3gPort*)param;
	CpgmacMacStatus *macStat;
	

	switch (cmd)
	{
		case CPSW3G_PHY_STATUS_UPDATE:
			macStat = (CpgmacMacStatus*)cmdArg;
						
			devPtr->linkStatus[hPort->portNum] = macStat->PhyLinked;
			if(0u != (macStat->PhyLinked) )
			{
				devPtr->linkSpeed[hPort->portNum] = (macStat->PhySpeed == 2)? 1000000000 :((macStat->PhySpeed)?100000000 : 10000000);
				devPtr->linkMode[hPort->portNum]  = ((macStat->PhyDuplex) ? 2 : 1);
				if(linkspeed[hPort->portNum]!= devPtr->linkSpeed[hPort->portNum])
                     	{
                      		if(devPtr->linkSpeed[hPort->portNum]== 1000000000)
                      		{
                            		devPtr->initCfg->macInitCfg[hPort->portNum].gigaBitEnable = 1;
                      		}
                            	else
                            	{
                            		devPtr->initCfg->macInitCfg[hPort->portNum].gigaBitEnable = 0;
                            	}

                            	devPtr->initCfg->macInitCfg[hPort->portNum].fullduplex = macStat->PhyDuplex;
                            	cpsw3g_SetMacCfg(hPort);
                     	}
			
                     	linkspeed[hPort->portNum]= devPtr->linkSpeed[hPort->portNum];
                     	/*Reactivate the transmit queue if it is stopped */
                     	pdev->TxOn = 1;
                     	
			}
			else
			{
				if(  (0 == devPtr->linkStatus[0] ) && ( 0 == devPtr->linkStatus[1]) )
				{
					devPtr->linkSpeed[hPort->portNum] = 100000000;
                                	devPtr->linkMode[hPort->portNum]  = 1;
					/* stop xmit , set flag so that HwPktTxNext is not invoked*/
                                	pdev->TxOn = 0;
				}
			}
                     	
			break;
		default:
			break;
	}
	
    return (CPSW3G_SUCCESS);
}/* end of function cpsw3g_ControlCb */


static Uint32 cpsw3g_DisableChannel(Uint32 channel, Uint32 direction)
{
    CpdmaObj     * dmaObj;

    dmaObj     =  &cpsw3gDev.cpdma;

    dmaObj->tdPending[direction][channel] = TRUE;
    if(direction == NET_CH_DIR_TX)
        {
        /* disable the requested Tx DMA channel here */

        /* command teardown  */
        cpsw3gDev.regs->TX_TEARDOWN = channel;
        /* wait for ack ,examine CP for 0xfffffffc */
       while(cpsw3gDev.regs->TX_CP[channel] != 0xFFFFFFFC)
       {
       }

        /*disable Tx Interrupt on this channel */
        cpsw3gDev.regs->TX_INTMASK_CLEAR =  (1<< channel);
        /* also disable in the wrapper */
        cpsw3gDev.ectlRegs->TX_EN &= (~(1<<channel));
    	}
    else
    	{
    	/* disable the requested Rx Dma channel here */

    	/* command teardown  */
        cpsw3gDev.regs->RX_TEARDOWN = channel;
        /* wait for ack ,examine CP for 0xfffffffc */
	while(cpsw3gDev.regs->RX_CP[channel] != 0xFFFFFFFC)
	{
	}
        /*disable Tx Interrupt on this channel */
        cpsw3gDev.regs->RX_INTMASK_CLEAR =  (1<< channel);
        /* also disable in the wrapper */
        cpsw3gDev.ectlRegs->RX_EN &= (~(1<<channel));
    	}

     dmaObj->tdPending[direction][channel] = FALSE;
     dmaObj->chIsOpen[direction][channel] = FALSE;

     return (CPSW3G_SUCCESS);

    
}/* end of function cpsw3g_disableChannel */

static Uint32 cpsw3g_EnableChannel(Uint32 channel, Uint32 direction)
{
    CpdmaObj     * dmaObj;

    dmaObj     =  &cpsw3gDev.cpdma;

    if(direction == NET_CH_DIR_TX)
        {
        /* enable a TX Dma channel for transfers here */

        /* Init Head Pointer */
        cpsw3gDev.regs->TX_HDP[channel] = 0;

        /* enable the corresponding channel interrupt */
        cpsw3gDev.regs->TX_INTMASK_SET = (1 << channel);
        /* enable the channel interrupt in CPSW_3G_SS  wrapper*/
        cpsw3gDev.ectlRegs->TX_EN |= (1<<channel);

       /* mark the channel as open */
        dmaObj->chIsOpen[NET_CH_DIR_TX][channel] = TRUE;
        }
    else
        {
        /* enable RX Dma channel for transfer here */
        Cpsw3gRxCppiCh  *rxChan;

        rxChan = &cpsw3gDev.cpdma.rxCppi[channel];
        /* enable channel interrupt*/
        cpsw3gDev.regs->RX_INTMASK_SET = (1 << channel) ;
        /* enable the channel interrupt in CPSW_3G_SS  wrapper*/
        cpsw3gDev.ectlRegs->RX_EN |= (1<<channel);
        
        /* mark the channel as open */
        dmaObj->chIsOpen[NET_CH_DIR_RX][channel] =TRUE;
        /* Init Head Pointer, created the chain in InitRxChannel */
        cpsw3gDev.regs->RX_HDP[channel] = CPU_TO_HWBD(((Uint32)rxChan->pDescRead));
              
        
        }
    
    return (CPSW3G_SUCCESS);
               
}/* end of function cpsw3g_EnableChannel */

static Uint32 cpsw3g_Control(Uint32 cmd, void *cmdArg, void *param)
{

    switch(cmd)
    {
    	case CPSW3G_IOCTL_PORT_STATISTICS_ENABLE:
    		cpsw3gDev.regs->CPSW_STAT_PORT_EN  = (Uint32) cmdArg;
    		{    /* clear old all old stats */
    			volatile Uint32  * statRegs = (Uint32*) &cpsw3gDev.regs->RXGOODFRAMES;
			Int32 i;
			Uint32 statClearVal;

			/* if atleast one port is enabled, all values are write to clear hence 
			  * we write all FFs to clear, else write zero to set value */
			if (cmdArg == 0) 
				{
				statClearVal = 0 ;
				}
			else
				{
				statClearVal = CPSW3G_STAT_CLEAR;
				}

    			for(i = 0; i < CPSW3G_NUM_STAT_REGS; i++)
            		{
              		*statRegs = statClearVal; 
			statRegs++;
            		}
    		}
    		/* update our s/w stat counters also to zero */
    		mmZeroInit(&cpsw3gDev.statsObj,sizeof(cpsw3gDev.statsObj));
    		break;
	default:
		break;
    }

    /** second argument param added for future extensibility, since it is not being
     * used, add dummy to suppress warning*/
    (void)param;

    return CPSW3G_SUCCESS;
}/* end of function cpsw3g_Control */


/** This function opens a data channel on the CPPI DMA engine
  */
static Uint32 cpsw3g_NetChOpen(Cpsw3gChInfo *chInfo)
{
	Uint32 retVal;

	/* Perform sanity checks on input params */
	if( chInfo->chNum >= cpsw3gCfg.dmaInitCfg.numChannels[chInfo->chDir])
	{
    		printf("NetChOpen: Channel number invalid \n");
    		return (CPSW3G_ERR_CH_INVALID);
	}
	if(cpsw3gDev.cpdma.chIsInit[chInfo->chDir][chInfo->chNum] == TRUE)
	{
    		printf("NetChOpen: %s Channel %d already initialized\n",
	     		((chInfo->chDir == NET_CH_DIR_TX) ? "TX" : "RX"), chInfo->chNum);
   	 return (CPSW3G_ERR_CH_ALREADY_INIT);
	}

	/* Perform book keeping for indv channel */
	if (chInfo->chDir == NET_CH_DIR_TX)
	{
    		retVal = cpsw3g_InitTxChannel(chInfo);
	}
	else
	{
    		retVal = cpsw3g_InitRxChannel( chInfo);
	}

	if (retVal != CPSW3G_SUCCESS)
	{
      		printf("NetChOpen: Error in initializing %s channel %d",
	     	((chInfo->chDir == NET_CH_DIR_TX) ? "TX" : "RX"), chInfo->chNum);
      	return (retVal);
	}

	/**CHECK : if device already opened then enable ?? */
	/* Enable this channel for use */
	retVal = cpsw3g_EnableChannel(chInfo->chNum, chInfo->chDir);
	if (retVal != CPSW3G_SUCCESS)
	{
    		printf("NetChOpen: Error enabling channel %d in %d direction\n",
                	 chInfo->chNum, chInfo->chDir);
    		return (retVal);
	}

 	return (CPSW3G_SUCCESS);
}/* end of function cpsw3g_NetChOpen */

static Uint32 cpsw3g_NetChClose(Cpsw3gChInfo *chInfo)
{
	CpdmaObj *dmaObj;

	dmaObj = &cpsw3gDev.cpdma;

	if(dmaObj->chIsOpen[chInfo->chDir][chInfo->chNum]  == TRUE)
		{
		cpsw3g_DisableChannel(chInfo->chNum, chInfo->chDir);
		}
	if(dmaObj->chIsInit[chInfo->chDir][chInfo->chNum] == TRUE)
		{
		if(chInfo->chDir == NET_CH_DIR_TX)
			{
			cpsw3g_UnInitTxChannel(chInfo);
			}
		else
			{
			cpsw3g_UnInitRxChannel(chInfo);
			}
		}

	return (CPSW3G_SUCCESS);
}/* end of function cpsw3g_NetChClose */

static Uint32 cpsw3g_UnInitTxChannel(Cpsw3gChInfo *chInfo)
{
	CpdmaObj *dmaObj;
	Cpsw3gTxCppiCh * txChan;
	PBM_Handle hPkt;

	dmaObj = &cpsw3gDev.cpdma;
	txChan = &dmaObj->txCppi[chInfo->chNum];

	hPkt =   PBMQ_deq( &txChan->DescQueue );
    	while( (PBM_Handle)0 != hPkt )
    	{
        	PBM_free( hPkt );
        	hPkt = PBMQ_deq( &txChan->DescQueue );
    	}


	dmaObj->chIsInit[NET_CH_DIR_TX][chInfo->chNum] = FALSE;
    return (CPSW3G_SUCCESS);
	

}/* end of fuction cpsw3g_UnInitTxChannel */

static Uint32 cpsw3g_UnInitRxChannel(Cpsw3gChInfo *chInfo)
{
	CpdmaObj *dmaObj;
	Cpsw3gRxCppiCh * rxChan;
	PBM_Handle hPkt;

	dmaObj = &cpsw3gDev.cpdma;
	rxChan = &dmaObj->rxCppi[chInfo->chNum];

	hPkt =   PBMQ_deq( &rxChan->DescQueue );
    	while( (PBM_Handle)0 != hPkt )
    	{
        	PBM_free( hPkt );
        	hPkt = PBMQ_deq( &rxChan->DescQueue );
    	}


	dmaObj->chIsInit[NET_CH_DIR_RX][chInfo->chNum] = FALSE;
    return (CPSW3G_SUCCESS);
	
}/* end of function cpsw3g_UnInitRxChannel */

static Uint32 cpsw3g_InitTxChannel(Cpsw3gChInfo *chInfo)
{
CpdmaObj *dmaObj;
Cpsw3gTxCppiCh * txChan;
CPPI_Desc   *pDesc,*pLastDesc;

dmaObj = &cpsw3gDev.cpdma;
txChan = &dmaObj->txCppi[chInfo->chNum];

/* zero init the book keeping structure */
mmZeroInit(txChan,sizeof(Cpsw3gTxCppiCh));

/* Init the Desc Queue Structure */
PBMQ_init(&txChan->DescQueue);

/* store pointer to channel info structure */
txChan->chInfo = chInfo;
pDesc = (CPPI_Desc *)_CPPI_DSC_BASE_ADDR;
pDesc += numBdsAllocated; /* advance to next free BD */

/* if we have room, lastDesc would point to this */
pLastDesc = pDesc + (chInfo->numBD -1);

if( (numBdsAllocated + chInfo->numBD) > (_CPPI_DSC_ENTRY_COUNT))
{
    /* not enough room for the requested number of BDs, fail request */
    printf("InitTx Channel : Unable to allocate %d BDs for channel %d.%d BDs already in use\n",
            chInfo->numBD,chInfo->chNum,numBdsAllocated);
    return CPSW3G_BD_ALLOC_FAIL;
}

/* clear the teardown pending flag */
dmaObj->tdPending[NET_CH_DIR_TX][chInfo->chNum] = FALSE;
/* update the Bd allocation count */
numBdsAllocated += chInfo->numBD;

/* we assume that the allocated BDs are sequential
  * for aries this includes padding space in each BD for cache alignment */
txChan->DescMax = chInfo->numBD;
txChan->pDescFirst = pDesc;
txChan->pDescRead = pDesc;
txChan->pDescWrite = pDesc;
txChan->pDescLast = pLastDesc;

dmaObj->chIsInit[NET_CH_DIR_TX][chInfo->chNum] = TRUE;
return (CPSW3G_SUCCESS);


}/* end of function cpsw3g_InitTxChannel() */

static Uint32 cpsw3g_InitRxChannel(Cpsw3gChInfo *chInfo)
{
CpdmaObj *dmaObj;
Cpsw3gRxCppiCh * rxChan;
CPPI_Desc   *pDesc,*pLastDesc;

dmaObj = &cpsw3gDev.cpdma;
rxChan = &dmaObj->rxCppi[chInfo->chNum];

/* zero init the book keeping structure */
mmZeroInit(rxChan,sizeof(Cpsw3gRxCppiCh));

/* Init the Desc Queue Structure */
PBMQ_init(&rxChan->DescQueue);

/* store pointer to channel info structure */
rxChan->chInfo = chInfo;
pDesc = (CPPI_Desc *)_CPPI_DSC_BASE_ADDR;
pDesc += numBdsAllocated; /* advance to next free BD */

/* if we have room, lastDesc would point to this */
pLastDesc = pDesc + (chInfo->numBD -1);

if( (numBdsAllocated + chInfo->numBD) > (_CPPI_DSC_ENTRY_COUNT))
{
    /* not enough room for the requested number of BDs, fail request */
    printf("InitRx Channel : Unable to allocate %d BDs for channel %d.%d BDs already in use\n",
            chInfo->numBD,chInfo->chNum,numBdsAllocated);
    return CPSW3G_BD_ALLOC_FAIL;
}

/* clear the teardown pending flag */
dmaObj->tdPending[NET_CH_DIR_RX][chInfo->chNum] = FALSE;
/* update the Bd allocation count */
numBdsAllocated += chInfo->numBD;

/* we assume that the allocated BDs are sequential
  * for aries this includes padding space in each BD for cache alignment */
rxChan->DescMax = chInfo->numBD;
rxChan->pDescFirst = pDesc;
rxChan->pDescRead = pDesc;
rxChan->pDescWrite = pDesc;
rxChan->pDescLast = pLastDesc;

/* construct the list of BDs for this channel */
cpsw3g_Enqueue_Rx(rxChan,0);

if( rxChan->DescCount < rxChan->DescMax )
    {
    PBM_Handle hPkt;
    
    /* Free all RX descriptors */
    hPkt = PBMQ_deq( &rxChan->DescQueue );
    while( (PBM_Handle)0 != hPkt  )
        {
         PBM_free(hPkt);
         hPkt = PBMQ_deq( &rxChan->DescQueue );
        }
    printf("InitRxChannel : Unable to allocate %d Buffers on channel %d\n",
                rxChan->DescMax,chInfo->chNum);
    return CPSW3G_RXBUFF_ALLOC_FAIL;
        
    }

dmaObj->chIsInit[NET_CH_DIR_RX][chInfo->chNum] = TRUE;
return(CPSW3G_SUCCESS);

}/* end of function cpsw3g_InitRxChannel() */

static void cpsw3g_Enqueue_Rx(Cpsw3gRxCppiCh * rxChan,Uint32 restart)
{
    PBM_Handle      hPkt;
    CPPI_Desc       *pDesc;
    Uint32              CountOrg;
    UINT8               *buffer;

    /**
     * Keep the old count around
     */
    CountOrg = rxChan->DescCount;

    #if 1
    if(0 == rxChan->chInfo)
    {
     	printf("ChanInfo pointer for rx channel  is NULL.Fatal error \n");
     }
     #endif

    /**
     * Fill RX Packets Until Full
     */
    while( rxChan->DescCount < rxChan->DescMax )
    {
        /**
         * Try and get a buffer.
         */
        
        hPkt = PBM_alloc( rxChan->chInfo->bufSize + PKT_PREPAD );

        if( (PBM_Handle) 0 == hPkt )
        {
            break;
        }

	#ifdef ETH_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
                PSP_DVT_DESC(PSP_DVT_eFUNC_START, \
                             PSP_DVT_dINST, \
                             PSP_DVT_dCHANNEL_ID, \
                             PSP_DVT_dSIZE_BYTES),0,1,PBM_getValidLen(hPkt));
        #endif

        PBM_setDataOffset( hPkt, PKT_PREPAD );
        buffer = PBM_getDataBuffer(hPkt) + PKT_PREPAD;

        /**
         * Clean the cache for external addesses
         */
        if( 0u != ((UINT32)buffer & EXTMEM) )
        {
            OEMCacheClean( (void *)buffer, rxChan->chInfo->bufSize );
        }

        /**
         * Fill in the descriptor for this buffer
         */
        pDesc = rxChan->pDescWrite;

        /**
         * Move the write pointer and bump count
         */
        if( rxChan->pDescWrite == rxChan->pDescLast )
        {
            rxChan->pDescWrite = rxChan->pDescFirst;
        }
        else
        {
            rxChan->pDescWrite = (CPPI_Desc*)(  ((Uint32)rxChan->pDescWrite) +
                                                                            CPSW3G_SWBD_SIZE);
         }

        rxChan->DescCount++;

        /**
         * Fill in the descriptor
         */
        pDesc->pNext     = 0;
        pDesc->pBuffer   = (Uint8*)(CPU_TO_HW((Uint32)buffer));
        pDesc->BufOffLen = rxChan->chInfo->bufSize;
        pDesc->PktFlgLen = CPPI_DESC_FLAG_OWNER;
        CACHE_FLUSH_BD(((Uint32)pDesc),CPSW3G_SWBD_SIZE);

	#ifdef ETH_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
                    PSP_DVT_DESC(PSP_DVT_eRCV_START, \
                                 PSP_DVT_dINST, \
                                 PSP_DVT_dCHANNEL_ID, \
                                 PSP_DVT_dNONE),0,1);
        #endif

        /**
         * Enqueue this packet onto the desc queue
         */
        PBMQ_enq( &rxChan->DescQueue, hPkt );

        
        /**
         * Synch the cache
         */
        if( 0u != ((UINT32)buffer & EXTMEM) )
        {
            OEMCacheCleanSynch();
        }

        /**
         * Make the previous buffer point to us
         */
        if( pDesc == rxChan->pDescFirst )
        {
             rxChan->pDescLast->pNext = (CPPI_Desc*) (CPU_TO_HWBD((Uint32)pDesc));
             CACHE_FLUSH_BD(((Uint32)rxChan->pDescLast),CPSW3G_SWBD_SIZE);
        }
        else
        {
            CPPI_Desc * pPrevDesc;

            pPrevDesc = (CPPI_Desc*)(  ((Uint32)pDesc) - CPSW3G_SWBD_SIZE);
            pPrevDesc->pNext = (CPPI_Desc*) (CPU_TO_HWBD((Uint32)pDesc));
            CACHE_FLUSH_BD(((Uint32)pPrevDesc),CPSW3G_SWBD_SIZE);
        }

 	#ifdef ETH_INSTRUMENTATION_ENABLED
        PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
                PSP_DVT_DESC(PSP_DVT_eFUNC_END, \
                             PSP_DVT_dINST, \
                             PSP_DVT_dCHANNEL_ID, \
                             PSP_DVT_dSIZE_BYTES),0,1,0);
        #endif
    }    
    /* Restart RX if we had ran out of descriptors and got some here */
    if( ((0u != restart) && (0u == CountOrg)) && (0u != rxChan->DescCount) )
    {
        cpsw3gDev.regs->RX_HDP[rxChan->chInfo->chNum] =
                        CPU_TO_HWBD(((Uint32)rxChan->pDescRead));
    }

    return;
}/* end of function cpsw3g_Enqueue_rx */


static void cpsw3g_Dequeue_Rx(Cpsw3gRxCppiCh * rxChan,CPPI_Desc *pDescAck)
{
 	PBM_Handle  hPkt;
    	Uint32        tmp;
   	Uint32      PktFlgLen;
    	Uint32      NeedService = 0;
    	CPPI_Desc   *pDescNewRxFirst,*pDescNewRxLast=0,*pTemp;
    	Uint8       *buffer = 0;

    	/* address obbtained from Completion Pointer is HW address,convert to CPU */
    	pDescAck = (CPPI_Desc*)  ( HW_TO_CPUBD((Uint32)pDescAck)  );

    	/* Remember the first new descriptor slot */
    	pDescNewRxFirst = rxChan->pDescWrite;

    	/*
     	 * Pop & Free Buffers 'till the last Descriptor
     	 * One thing we know for sure is that all the decriptors from
        * the read pointer to pDescAsk are linked to each other via
        * their pNext field.
        */
    	tmp = 1u;
    	do
    	{
		cpswRxPkts++;
		CACHE_INVALIDATE_BD(((Uint32)rxChan->pDescRead),CPSW3G_SWBD_SIZE);
        	/* Get the status of this descriptor */
        	PktFlgLen = rxChan->pDescRead->PktFlgLen;

        	/**
         	 * Recover the packet and pass it on
         	 */
        	hPkt = PBMQ_deq( &rxChan->DescQueue );
        	if( (PBM_Handle)0 != hPkt )
        	{
        		Uint32 protocol,i;
        		Uint8 *src=0,*dst=0;
        		Uint32 pktLen,dataOffset;
        		
            		/**
             		 * Enque the packet and signal the stack
             		 */
             		 /**CHECK: If VLAN Aware mode, we need to remove Vlan Tag before passing it to
             		  * the upper layer.This is needed unless the upper layer can handle Vlan tags as part
             		  * of ethernet frame. */
            		PBM_setIFRx( hPkt, pPDI->hEther );
            		PBM_setValidLen(hPkt,(PktFlgLen & 0xFFFF));

			/* handle frames with 802.1Q VLAN Extended header */
			dataOffset = PBM_getDataOffset(hPkt);
			buffer = PBM_getDataBuffer(hPkt) + dataOffset;
			
			protocol = ( buffer[12] << 8) | buffer[13] ;
			if(0x8100u == (protocol & 0xFFFFu))
			{
				/* HW doesnt support forced untag on CPU port, hence if layers
				  * above the driver cannot handle VLAN tagged packets, we need
				  * to remove VLAN header here */
				src = buffer + 11;
				dst = buffer + 15;
				/* move the first 12 bytes of header */
				for(i=12;i>0;i--)
				{
					*dst = *src;
					dst--;
					src--;
				}

				/* update the buffer len, data pointer , now that the
				 * VLAN tag has been removed/overwritten */
				pktLen = PBM_getValidLen(hPkt) - 4;
				PBM_setValidLen(hPkt,pktLen);
				dataOffset += 4;
				PBM_setDataOffset(hPkt,dataOffset);
			}

			#ifdef ETH_INSTRUMENTATION_ENABLED
            		PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
                    		PSP_DVT_DESC(PSP_DVT_eRCV_END, \
                                PSP_DVT_dINST, \
                                PSP_DVT_dCHANNEL_ID, \
                                PSP_DVT_dNONE),0,1);
            		#endif
            		
		       PBMQ_enq( &PBMQ_rx, hPkt );
            		NeedService = 1;

        	}

        	/* See if this was the last buffer */
        	if( rxChan->pDescRead == pDescAck )
        	{
            		tmp = 0;
        	}

        	/* Move the read pointer and decrement count */
        	if( rxChan->pDescRead == rxChan->pDescLast )
        	{
            		rxChan->pDescRead = rxChan->pDescFirst;
        	}
        	else
        	{
        		 rxChan->pDescRead = (CPPI_Desc*)(  ((Uint32)rxChan->pDescRead) +
                                                                            CPSW3G_SWBD_SIZE);
            		
        	}
        	rxChan->DescCount--;

        	/**
         	 * Try and get a buffer.
         	 */
        	hPkt = PBM_alloc( rxChan->chInfo->bufSize + PKT_PREPAD );

        	if( (PBM_Handle)0 != hPkt )
        	{
			#ifdef ETH_INSTRUMENTATION_ENABLED
            		PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
                        	PSP_DVT_DESC(PSP_DVT_eFUNC_START, \
                                PSP_DVT_dINST, \
                                PSP_DVT_dCHANNEL_ID, \
                                PSP_DVT_dSIZE_BYTES),0,1,PBM_getValidLen(hPkt));
            		#endif

            		PBM_setDataOffset( hPkt, PKT_PREPAD );
            		buffer = PBM_getDataBuffer(hPkt) + PKT_PREPAD;

           		 /**
             		   * Clean the cache for external addresses
             		   */
            		if( 0u != ((UINT32)buffer & EXTMEM) )
            		{
                		OEMCacheClean( (void *)buffer, rxChan->chInfo->bufSize );
            		}

            		/**
             		  * Fill in the descriptor for this buffer
             		  */
            		pDescNewRxLast = rxChan->pDescWrite;

            		/**
             		  * Move the write pointer and bump count
             		  */
            		if( rxChan->pDescWrite == rxChan->pDescLast )
            		{
                		rxChan->pDescWrite = rxChan->pDescFirst;
            		}
            		else
            		{
            			rxChan->pDescWrite = (CPPI_Desc*)(  ((Uint32)rxChan->pDescWrite) +
                                                                            CPSW3G_SWBD_SIZE);
            		}

            		rxChan->DescCount++;

            		/**
             		  * Fill in the descriptor
             		  */
             		pDescNewRxLast->pBuffer   = (Uint8*)(CPU_TO_HW((Uint32)buffer));
        		pDescNewRxLast->BufOffLen = rxChan->chInfo->bufSize;
       	 		pDescNewRxLast->PktFlgLen = CPPI_DESC_FLAG_OWNER;
       	 		CACHE_FLUSH_BD(((Uint32)pDescNewRxLast),CPSW3G_SWBD_SIZE);

			#ifdef ETH_INSTRUMENTATION_ENABLED
		        PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
                    		PSP_DVT_DESC(PSP_DVT_eRCV_START, \
                                PSP_DVT_dINST, \
                                PSP_DVT_dCHANNEL_ID, \
                                PSP_DVT_dNONE),0,1);
            		#endif
            		
            		/**
             		  * Enqueue this packet onto the desc queue
             		  */
            		PBMQ_enq( &rxChan->DescQueue, hPkt );

			#ifdef ETH_INSTRUMENTATION_ENABLED
	            	PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
                    		PSP_DVT_DESC(PSP_DVT_eFUNC_END, \
                                PSP_DVT_dINST, \
                                PSP_DVT_dCHANNEL_ID, \
                                PSP_DVT_dSIZE_BYTES),0,1,0);
            		#endif
           
        	}
    }while (0u != tmp);

	/*
     	  * If we added descriptors, make the pNext of the last NULL, and
         * make the previous descriptor point to the new list we added.
         */
    	if( (CPPI_Desc*)0 != pDescNewRxLast )
    	{
        	pDescNewRxLast->pNext = 0;
        	CACHE_FLUSH_BD(((Uint32)pDescNewRxLast),CPSW3G_SWBD_SIZE);

        	/* Synch the cache */
        	if( 0u != ((UINT32)buffer & EXTMEM) )
        	{
            		OEMCacheCleanSynch();
        	}

        	/* Make the previous buffer point to us */
        	if( pDescNewRxFirst == rxChan->pDescFirst )
        	{
            		pTemp = rxChan->pDescLast;
        	}
        	else
        	{
            		pTemp = (CPPI_Desc*)(  ((Uint32)pDescNewRxFirst) -
                                                                            CPSW3G_SWBD_SIZE);
            		
        	}

        	/*
         	  * If these pointers wrapped, the RX engine is stopped
         	  * Otherwise; tack the new list to the old
         	  */
        	if( pTemp != pDescNewRxLast )
        	{
            		pTemp->pNext = (CPPI_Desc*) (CPU_TO_HWBD((Uint32)pDescNewRxFirst));
            		CACHE_FLUSH_BD(((Uint32)pTemp),CPSW3G_SWBD_SIZE);
        	}
   	 }

    	/* If the receiver stopped and we have more descriptors, then restart */
    	if( (0u != (PktFlgLen & CPPI_DESC_FLAG_EOQ) ) && (0u !=rxChan->DescCount) )
    	{
       	 cpsw3gDev.regs->RX_HDP[rxChan->chInfo->chNum] =
                        (  (Uint32) (CPU_TO_HWBD(rxChan->pDescRead)) );
    	}

    	/**
     	* Signal the stack if needed
     	*/
    	if( 0u != NeedService )
    	{
       	FlashActiveLED = 1;
        	STKEVENT_signal( pPDI->hEvent, STKEVENT_ETHERNET, 1 );
    	}
}/* end of function cpsw3g_Dequeue_Rx() */


static void cpsw3g_Dequeue_Tx(Cpsw3gTxCppiCh * txChan, CPPI_Desc *pDescAck)
{
 	PBM_Handle  hPkt;
    	Uint32      PktFlgLen;
    	register Uint32  i,j = (Uint32)txChan->pDescRead;


        /* address obbtained from Completion Pointer is HW address,convert to CPU */
    	pDescAck = (CPPI_Desc*)  (  HW_TO_CPUBD(((Uint32)pDescAck))  );

    	/* Get the status of the ACK descriptor */
    	CACHE_INVALIDATE_BD(((Uint32)pDescAck),CPSW3G_SWBD_SIZE);
    	PktFlgLen = pDescAck->PktFlgLen;

   	 /* Calc the new "Read" descriptor */
    	if( pDescAck == txChan->pDescLast )
    	{
       	 txChan->pDescRead = txChan->pDescFirst;
    	}
    	else
    	{
        	txChan->pDescRead = (CPPI_Desc*)(  ((Uint32)pDescAck) +
                                                                            CPSW3G_SWBD_SIZE);
    	}

   	 i = (Uint32)txChan->pDescRead;

   	 /* Turn i into a descriptor count */
   	 if( j < i )
    	{
       	 i = (i-j)/(CPSW3G_SWBD_SIZE);
    	}
    	else
    	{
        	i = txChan->DescMax - ((j-i)/(CPSW3G_SWBD_SIZE));
    	}

    	txChan->DescCount -= i;

    	/* Pop & Free Buffers 'till the last Descriptor */
    	while( i-- )
    	{
       	 /**
         	   * Recover the packet and free it
         	   */
        	hPkt = PBMQ_deq( &txChan->DescQueue );
        	if( (PBM_Handle)0 != hPkt )
        	{
		    #ifdef ETH_INSTRUMENTATION_ENABLED
            	    PSP_LOG_EVENT(&DVTEvent_Log, "ETHERNET", \
                    	PSP_DVT_DESC(PSP_DVT_eDATA_SND_END, \
                                 PSP_DVT_dINST, \
                                 PSP_DVT_dCHANNEL_ID, \
                                 PSP_DVT_dNONE), 0, 0);
            	    #endif
               	    PBM_free( hPkt );
        	}
    	}

    	/**
     	  * If the transmitter stopped and we have more descriptors, then restart
     	  */
    	if( (0u != (PktFlgLen & CPPI_DESC_FLAG_EOQ) ) && (0u != txChan->DescCount) )
    	{
        	cpsw3gDev.regs->TX_HDP[txChan->chInfo->chNum] =
                        ((Uint32)(CPU_TO_HWBD(txChan->pDescRead)));
        	
    	}
}/* end of function cpsw3g_Dequeue_Tx() */


static Uint32 cpsw3g_Tick(Cpsw3gDevice * devPtr)
{
	Uint32 i,mdioStatus,linkStatus;

    if((Cpsw3gDevice*)0 != devPtr)
    {
	for(i=0;i<CPSW3G_NUM_MAC_PORTS;i++)
		{
		/* check for port in forward state, not NO_PHY mode */
		if (  (0u !=(devPtr->regs->ALE_PORTCTL[i] & 0x3))  &&
			(0u == (devPtr->initCfg->macInitCfg[i].phyMode & SNWAY_NOPHY)) )
			{
			CpgmacObj *gmacObject  = &devPtr->cpgmacMac[i];
			Int32 tickChange;
			CpgmacMacStatus macStat;

			tickChange = cpsw_MDIO_Tic(&gmacObject->PhyDev,&mdioStatus);

			if(tickChange == 1)
				{
				
				cpsw3g_Update_Phystatus(gmacObject, i);
				macStat.PhyDuplex = gmacObject->PhyDuplex;
  				macStat.PhyLinked =  gmacObject->PhyLinked;
  				macStat.PhyNum = gmacObject->PhyNum;
  				macStat.PhySpeed = gmacObject->PhySpeed;
				cpsw3g_ControlCb(devPtr, CPSW3G_PHY_STATUS_UPDATE, &macStat, &devPtr->cpsw3gPort[i]);
				}
			if ( (mdioStatus == MDIO_EVENT_LINKDOWN)  ||
				(mdioStatus ==  MDIO_EVENT_LINKUP) )
				{
				 if(mdioStatus == MDIO_EVENT_LINKDOWN)
				 	{
						linkStatus = MDIO_LINKSTATUS_NOLINK; 
				 	}
				 	else
				 	{
				 linkStatus = ((gmacObject->PhySpeed + 1) << 1) + gmacObject->PhyDuplex ;
					}
				CPSW3G_linkStatus( macStat.PhyNum, linkStatus );
				}
			if (mdioStatus == MDIO_EVENT_LINKUP)
				{
				/* now that we have atleast link up on one  of the ports ,see whether
				 * we can start transmitting */
				 HwPktTxNext(devPtr->pdinfo);
				}
			}
		}
    }

	return CPSW3G_SUCCESS;
}/* end of function cpsw3g_tick() */

static void  cpsw3g_AleDump(Cpsw3gDevice * devPtr, int(*printFnArg)(const char*,...))
{
    Uint32 i, free=0;
    int(*printFunctionPtr)(const char*,...) = NULL;

    if( NULL == printFnArg)
    {
	    printFunctionPtr = ALE_DUMP_PRINT ;
    }
    else
    {
	    printFunctionPtr = (int (*)(const char*,...))printFnArg;
    }

    printFunctionPtr("\n");

    for (i=0; i<CPSW3G_ALE_TABLE_DEPTH; i++)
    {
      Uint32 info0, info1, info2;

      devPtr->regs->ALE_TBLCTL = i;
      info0 = devPtr->regs->ALE_TBLW0;
      info1 = devPtr->regs->ALE_TBLW1;
      info2 = devPtr->regs->ALE_TBLW2;

      if ((info0==0xfffffffe)&&((info1&0x0ffff)==0x0ffff))
	{
	}
      else
	{

     switch((info1>>28)&3)
     {
        case 0: 
            free++;
        break;
        case 1: /*MacAddress*/
		if (0u != (info1&(1<<8)) )
            {
                printFunctionPtr("%4d: Address: %02x%02x%02x%02x%02x%02x, Member:%03x Su=%x STP=%d",
                    i,(info1>>8)&0x0ff,(info1>>0)&0x0ff,(info0>>24)&0x0ff,
                    (info0>>16)&0x0ff,(info0>>8)&0x0ff,(info0>>0)&0x0ff,
                    (info2>>0)&0x7,((info2>>3)&0x1),(info1>>30)&0x3);
            }
            else
            {
                printFunctionPtr("%4d: Address: %02x%02x%02x%02x%02x%02x, Port: %03x Se=%x Bl=%d Type=%d",
                   i,(info1>>8)&0x0ff,(info1>>0)&0x0ff, (info0>>24)&0x0ff,
                   (info0>>16)&0x0ff,(info0>>8)&0x0ff,(info0>>0)&0x0ff,
                   (info2>>0)&0x3, ((info1>>30)&0x1), (info1>>31)&0x1, (info2>>2)&0x3);
            }
        break;
      
        case 2: /*VLAN*/
            printFunctionPtr("%4d: Vlanid: %04d, UTagged: %x, Mult: %x, UMult: %x, Member: %x",
                 i,((info1>>16)&0x0fff),
                 ((info0>>24)&0x07),((info0>>16)&0x07),((info0>>8)&0x07),((info0>>0)&0x07));
        break;
        
        case 3: /*VLAN-MacAddress*/

		if (0u != (info1&(1<<8)))
            {
                printFunctionPtr("%4d: Address: %02x%02x%02x%02x%02x%02x, VlanId: %04d, Member: %03x",
                   i,(info1>>8)&0x0ff,(info1>>0)&0x0ff,
                   (info0>>24)&0x0ff,(info0>>16)&0x0ff,(info0>>8)&0x0ff,(info0>>0)&0x0ff,
                   (info1>>16)&0x0fff,
                   (info2>>0)&0x7);
            }
            else
            {
                printFunctionPtr("%4d: Address: %02x%02x%02x%02x%02x%02x, VlanId: %04d, Port: %03x",
                   i,(info1>>8)&0x0ff,(info1>>0)&0x0ff,
                   (info0>>24)&0x0ff,(info0>>16)&0x0ff,(info0>>8)&0x0ff,(info0>>0)&0x0ff,
                   (info1>>16)&0x0fff,
                   (info2>>0)&0x3);
            }
        break;
        
        default:
        break;
        }/* end of switch construct */


	/*Raw Dump,followed by parsed output*/
	if(0 != ((info1 >> 28) & 0x3u) )
	{
	/* print Info for entried that are not free */
	printFunctionPtr(" RAW:[%01x %02x%02x%02x",(info2 & 0xF),
	      (info1 >> 24)&0xFF,(info1 >> 16)&0xFF, (info1 >> 8)&0xFF);
	printFunctionPtr("%02x %02x%02x%02x%02x] \n",(info1 & 0xF),
	      (info0 >> 24)&0xFF,(info0 >> 16)&0xFF, 
	      (info0 >> 8)&0xFF),(info0 & 0xFF);
	}
	}/* end of else conditional block */
    }/* end of for loop over ALE entries */
    printFunctionPtr("\n\r"); 
    printFunctionPtr("%d Free Entries \r\n",free);
}/* end of function cpsw3g_AleDump */
